// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/*eslint-disable block-scoped-var, id-length, no-control-regex, no-magic-numbers, no-prototype-builtins, no-redeclare, no-shadow, no-var, sort-vars*/
(function(global, factory) { /* global define, require, module */

    /* AMD */ if (typeof define === 'function' && define.amd)
        define(["protobufjs/minimal"], factory);

    /* CommonJS */ else if (typeof require === 'function' && typeof module === 'object' && module && module.exports)
        module.exports = factory(require("google-gax").protobufMinimal);

})(this, function($protobuf) {
    "use strict";

    // Common aliases
    var $Reader = $protobuf.Reader, $Writer = $protobuf.Writer, $util = $protobuf.util;
    
    // Exported root namespace
    var $root = $protobuf.roots.google_ads_protos || ($protobuf.roots.google_ads_protos = {});
    
    $root.google = (function() {
    
        /**
         * Namespace google.
         * @exports google
         * @namespace
         */
        var google = {};
    
        google.ads = (function() {
    
            /**
             * Namespace ads.
             * @memberof google
             * @namespace
             */
            var ads = {};
    
            ads.googleads = (function() {
    
                /**
                 * Namespace googleads.
                 * @memberof google.ads
                 * @namespace
                 */
                var googleads = {};
    
                googleads.v11 = (function() {
    
                    /**
                     * Namespace v11.
                     * @memberof google.ads.googleads
                     * @namespace
                     */
                    var v11 = {};
    
                    v11.common = (function() {
    
                        /**
                         * Namespace common.
                         * @memberof google.ads.googleads.v11
                         * @namespace
                         */
                        var common = {};
    
                        common.AdTextAsset = (function() {
    
                            /**
                             * Properties of an AdTextAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAdTextAsset
                             * @property {string|null} [text] AdTextAsset text
                             * @property {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.ServedAssetFieldType|null} [pinned_field] AdTextAsset pinned_field
                             * @property {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel|null} [asset_performance_label] AdTextAsset asset_performance_label
                             * @property {google.ads.googleads.v11.common.IAdAssetPolicySummary|null} [policy_summary_info] AdTextAsset policy_summary_info
                             */
    
                            /**
                             * Constructs a new AdTextAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AdTextAsset.
                             * @implements IAdTextAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAdTextAsset=} [p] Properties to set
                             */
                            function AdTextAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdTextAsset text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @instance
                             */
                            AdTextAsset.prototype.text = null;
    
                            /**
                             * AdTextAsset pinned_field.
                             * @member {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.ServedAssetFieldType} pinned_field
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @instance
                             */
                            AdTextAsset.prototype.pinned_field = 0;
    
                            /**
                             * AdTextAsset asset_performance_label.
                             * @member {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel} asset_performance_label
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @instance
                             */
                            AdTextAsset.prototype.asset_performance_label = 0;
    
                            /**
                             * AdTextAsset policy_summary_info.
                             * @member {google.ads.googleads.v11.common.IAdAssetPolicySummary|null|undefined} policy_summary_info
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @instance
                             */
                            AdTextAsset.prototype.policy_summary_info = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdTextAsset _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @instance
                             */
                            Object.defineProperty(AdTextAsset.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdTextAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdTextAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AdTextAsset} AdTextAsset instance
                             */
                            AdTextAsset.create = function create(properties) {
                                return new AdTextAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AdTextAsset message. Does not implicitly {@link google.ads.googleads.v11.common.AdTextAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdTextAsset} m AdTextAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdTextAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.pinned_field != null && Object.hasOwnProperty.call(m, "pinned_field"))
                                    w.uint32(16).int32(m.pinned_field);
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(34).string(m.text);
                                if (m.asset_performance_label != null && Object.hasOwnProperty.call(m, "asset_performance_label"))
                                    w.uint32(40).int32(m.asset_performance_label);
                                if (m.policy_summary_info != null && Object.hasOwnProperty.call(m, "policy_summary_info"))
                                    $root.google.ads.googleads.v11.common.AdAssetPolicySummary.encode(m.policy_summary_info, w.uint32(50).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdTextAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AdTextAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdTextAsset} message AdTextAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdTextAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdTextAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AdTextAsset} AdTextAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdTextAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AdTextAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.text = r.string();
                                        break;
                                    case 2:
                                        m.pinned_field = r.int32();
                                        break;
                                    case 5:
                                        m.asset_performance_label = r.int32();
                                        break;
                                    case 6:
                                        m.policy_summary_info = $root.google.ads.googleads.v11.common.AdAssetPolicySummary.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdTextAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AdTextAsset} AdTextAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdTextAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdTextAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdTextAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.pinned_field != null && m.hasOwnProperty("pinned_field")) {
                                    switch (m.pinned_field) {
                                    default:
                                        return "pinned_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.asset_performance_label != null && m.hasOwnProperty("asset_performance_label")) {
                                    switch (m.asset_performance_label) {
                                    default:
                                        return "asset_performance_label: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.policy_summary_info != null && m.hasOwnProperty("policy_summary_info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdAssetPolicySummary.verify(m.policy_summary_info);
                                        if (e)
                                            return "policy_summary_info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdTextAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AdTextAsset} AdTextAsset
                             */
                            AdTextAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AdTextAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AdTextAsset();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                switch (d.pinned_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.pinned_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.pinned_field = 1;
                                    break;
                                case "HEADLINE_1":
                                case 2:
                                    m.pinned_field = 2;
                                    break;
                                case "HEADLINE_2":
                                case 3:
                                    m.pinned_field = 3;
                                    break;
                                case "HEADLINE_3":
                                case 4:
                                    m.pinned_field = 4;
                                    break;
                                case "DESCRIPTION_1":
                                case 5:
                                    m.pinned_field = 5;
                                    break;
                                case "DESCRIPTION_2":
                                case 6:
                                    m.pinned_field = 6;
                                    break;
                                }
                                switch (d.asset_performance_label) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_performance_label = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_performance_label = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.asset_performance_label = 2;
                                    break;
                                case "LEARNING":
                                case 3:
                                    m.asset_performance_label = 3;
                                    break;
                                case "LOW":
                                case 4:
                                    m.asset_performance_label = 4;
                                    break;
                                case "GOOD":
                                case 5:
                                    m.asset_performance_label = 5;
                                    break;
                                case "BEST":
                                case 6:
                                    m.asset_performance_label = 6;
                                    break;
                                }
                                if (d.policy_summary_info != null) {
                                    if (typeof d.policy_summary_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AdTextAsset.policy_summary_info: object expected");
                                    m.policy_summary_info = $root.google.ads.googleads.v11.common.AdAssetPolicySummary.fromObject(d.policy_summary_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdTextAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.AdTextAsset} m AdTextAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdTextAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.pinned_field = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.asset_performance_label = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.policy_summary_info = null;
                                }
                                if (m.pinned_field != null && m.hasOwnProperty("pinned_field")) {
                                    d.pinned_field = o.enums === String ? $root.google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.ServedAssetFieldType[m.pinned_field] : m.pinned_field;
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.asset_performance_label != null && m.hasOwnProperty("asset_performance_label")) {
                                    d.asset_performance_label = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel[m.asset_performance_label] : m.asset_performance_label;
                                }
                                if (m.policy_summary_info != null && m.hasOwnProperty("policy_summary_info")) {
                                    d.policy_summary_info = $root.google.ads.googleads.v11.common.AdAssetPolicySummary.toObject(m.policy_summary_info, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdTextAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AdTextAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdTextAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdTextAsset;
                        })();
    
                        common.AdImageAsset = (function() {
    
                            /**
                             * Properties of an AdImageAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAdImageAsset
                             * @property {string|null} [asset] AdImageAsset asset
                             */
    
                            /**
                             * Constructs a new AdImageAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AdImageAsset.
                             * @implements IAdImageAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAdImageAsset=} [p] Properties to set
                             */
                            function AdImageAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdImageAsset asset.
                             * @member {string|null|undefined} asset
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @instance
                             */
                            AdImageAsset.prototype.asset = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdImageAsset _asset.
                             * @member {"asset"|undefined} _asset
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @instance
                             */
                            Object.defineProperty(AdImageAsset.prototype, "_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdImageAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdImageAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AdImageAsset} AdImageAsset instance
                             */
                            AdImageAsset.create = function create(properties) {
                                return new AdImageAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AdImageAsset message. Does not implicitly {@link google.ads.googleads.v11.common.AdImageAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdImageAsset} m AdImageAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdImageAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(18).string(m.asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdImageAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AdImageAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdImageAsset} message AdImageAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdImageAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdImageAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AdImageAsset} AdImageAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdImageAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AdImageAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.asset = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdImageAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AdImageAsset} AdImageAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdImageAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdImageAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdImageAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    p._asset = 1;
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdImageAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AdImageAsset} AdImageAsset
                             */
                            AdImageAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AdImageAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AdImageAsset();
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdImageAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.AdImageAsset} m AdImageAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdImageAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                    if (o.oneofs)
                                        d._asset = "asset";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdImageAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AdImageAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdImageAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdImageAsset;
                        })();
    
                        common.AdVideoAsset = (function() {
    
                            /**
                             * Properties of an AdVideoAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAdVideoAsset
                             * @property {string|null} [asset] AdVideoAsset asset
                             */
    
                            /**
                             * Constructs a new AdVideoAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AdVideoAsset.
                             * @implements IAdVideoAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAdVideoAsset=} [p] Properties to set
                             */
                            function AdVideoAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdVideoAsset asset.
                             * @member {string|null|undefined} asset
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @instance
                             */
                            AdVideoAsset.prototype.asset = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdVideoAsset _asset.
                             * @member {"asset"|undefined} _asset
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @instance
                             */
                            Object.defineProperty(AdVideoAsset.prototype, "_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdVideoAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdVideoAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AdVideoAsset} AdVideoAsset instance
                             */
                            AdVideoAsset.create = function create(properties) {
                                return new AdVideoAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AdVideoAsset message. Does not implicitly {@link google.ads.googleads.v11.common.AdVideoAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdVideoAsset} m AdVideoAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdVideoAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(18).string(m.asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdVideoAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AdVideoAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdVideoAsset} message AdVideoAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdVideoAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdVideoAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AdVideoAsset} AdVideoAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdVideoAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AdVideoAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.asset = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdVideoAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AdVideoAsset} AdVideoAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdVideoAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdVideoAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdVideoAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    p._asset = 1;
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdVideoAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AdVideoAsset} AdVideoAsset
                             */
                            AdVideoAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AdVideoAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AdVideoAsset();
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdVideoAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.AdVideoAsset} m AdVideoAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdVideoAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                    if (o.oneofs)
                                        d._asset = "asset";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdVideoAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AdVideoAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdVideoAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdVideoAsset;
                        })();
    
                        common.AdMediaBundleAsset = (function() {
    
                            /**
                             * Properties of an AdMediaBundleAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAdMediaBundleAsset
                             * @property {string|null} [asset] AdMediaBundleAsset asset
                             */
    
                            /**
                             * Constructs a new AdMediaBundleAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AdMediaBundleAsset.
                             * @implements IAdMediaBundleAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAdMediaBundleAsset=} [p] Properties to set
                             */
                            function AdMediaBundleAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdMediaBundleAsset asset.
                             * @member {string|null|undefined} asset
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @instance
                             */
                            AdMediaBundleAsset.prototype.asset = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdMediaBundleAsset _asset.
                             * @member {"asset"|undefined} _asset
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @instance
                             */
                            Object.defineProperty(AdMediaBundleAsset.prototype, "_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdMediaBundleAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdMediaBundleAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AdMediaBundleAsset} AdMediaBundleAsset instance
                             */
                            AdMediaBundleAsset.create = function create(properties) {
                                return new AdMediaBundleAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AdMediaBundleAsset message. Does not implicitly {@link google.ads.googleads.v11.common.AdMediaBundleAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdMediaBundleAsset} m AdMediaBundleAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdMediaBundleAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(18).string(m.asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdMediaBundleAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AdMediaBundleAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdMediaBundleAsset} message AdMediaBundleAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdMediaBundleAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdMediaBundleAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AdMediaBundleAsset} AdMediaBundleAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdMediaBundleAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AdMediaBundleAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.asset = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdMediaBundleAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AdMediaBundleAsset} AdMediaBundleAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdMediaBundleAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdMediaBundleAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdMediaBundleAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    p._asset = 1;
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdMediaBundleAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AdMediaBundleAsset} AdMediaBundleAsset
                             */
                            AdMediaBundleAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AdMediaBundleAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AdMediaBundleAsset();
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdMediaBundleAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.AdMediaBundleAsset} m AdMediaBundleAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdMediaBundleAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                    if (o.oneofs)
                                        d._asset = "asset";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdMediaBundleAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AdMediaBundleAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdMediaBundleAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdMediaBundleAsset;
                        })();
    
                        common.AdDiscoveryCarouselCardAsset = (function() {
    
                            /**
                             * Properties of an AdDiscoveryCarouselCardAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAdDiscoveryCarouselCardAsset
                             * @property {string|null} [asset] AdDiscoveryCarouselCardAsset asset
                             */
    
                            /**
                             * Constructs a new AdDiscoveryCarouselCardAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AdDiscoveryCarouselCardAsset.
                             * @implements IAdDiscoveryCarouselCardAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAdDiscoveryCarouselCardAsset=} [p] Properties to set
                             */
                            function AdDiscoveryCarouselCardAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdDiscoveryCarouselCardAsset asset.
                             * @member {string|null|undefined} asset
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @instance
                             */
                            AdDiscoveryCarouselCardAsset.prototype.asset = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdDiscoveryCarouselCardAsset _asset.
                             * @member {"asset"|undefined} _asset
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @instance
                             */
                            Object.defineProperty(AdDiscoveryCarouselCardAsset.prototype, "_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdDiscoveryCarouselCardAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdDiscoveryCarouselCardAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset} AdDiscoveryCarouselCardAsset instance
                             */
                            AdDiscoveryCarouselCardAsset.create = function create(properties) {
                                return new AdDiscoveryCarouselCardAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AdDiscoveryCarouselCardAsset message. Does not implicitly {@link google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdDiscoveryCarouselCardAsset} m AdDiscoveryCarouselCardAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdDiscoveryCarouselCardAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(10).string(m.asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdDiscoveryCarouselCardAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdDiscoveryCarouselCardAsset} message AdDiscoveryCarouselCardAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdDiscoveryCarouselCardAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdDiscoveryCarouselCardAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset} AdDiscoveryCarouselCardAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdDiscoveryCarouselCardAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.asset = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdDiscoveryCarouselCardAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset} AdDiscoveryCarouselCardAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdDiscoveryCarouselCardAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdDiscoveryCarouselCardAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdDiscoveryCarouselCardAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    p._asset = 1;
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdDiscoveryCarouselCardAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset} AdDiscoveryCarouselCardAsset
                             */
                            AdDiscoveryCarouselCardAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset();
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdDiscoveryCarouselCardAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset} m AdDiscoveryCarouselCardAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdDiscoveryCarouselCardAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                    if (o.oneofs)
                                        d._asset = "asset";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdDiscoveryCarouselCardAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdDiscoveryCarouselCardAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdDiscoveryCarouselCardAsset;
                        })();
    
                        common.AdAssetPolicySummary = (function() {
    
                            /**
                             * Properties of an AdAssetPolicySummary.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAdAssetPolicySummary
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>|null} [policy_topic_entries] AdAssetPolicySummary policy_topic_entries
                             * @property {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus|null} [review_status] AdAssetPolicySummary review_status
                             * @property {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus|null} [approval_status] AdAssetPolicySummary approval_status
                             */
    
                            /**
                             * Constructs a new AdAssetPolicySummary.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AdAssetPolicySummary.
                             * @implements IAdAssetPolicySummary
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAdAssetPolicySummary=} [p] Properties to set
                             */
                            function AdAssetPolicySummary(p) {
                                this.policy_topic_entries = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdAssetPolicySummary policy_topic_entries.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>} policy_topic_entries
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @instance
                             */
                            AdAssetPolicySummary.prototype.policy_topic_entries = $util.emptyArray;
    
                            /**
                             * AdAssetPolicySummary review_status.
                             * @member {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus} review_status
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @instance
                             */
                            AdAssetPolicySummary.prototype.review_status = 0;
    
                            /**
                             * AdAssetPolicySummary approval_status.
                             * @member {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus} approval_status
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @instance
                             */
                            AdAssetPolicySummary.prototype.approval_status = 0;
    
                            /**
                             * Creates a new AdAssetPolicySummary instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdAssetPolicySummary=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AdAssetPolicySummary} AdAssetPolicySummary instance
                             */
                            AdAssetPolicySummary.create = function create(properties) {
                                return new AdAssetPolicySummary(properties);
                            };
    
                            /**
                             * Encodes the specified AdAssetPolicySummary message. Does not implicitly {@link google.ads.googleads.v11.common.AdAssetPolicySummary.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdAssetPolicySummary} m AdAssetPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdAssetPolicySummary.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.policy_topic_entries != null && m.policy_topic_entries.length) {
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEntry.encode(m.policy_topic_entries[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.review_status != null && Object.hasOwnProperty.call(m, "review_status"))
                                    w.uint32(16).int32(m.review_status);
                                if (m.approval_status != null && Object.hasOwnProperty.call(m, "approval_status"))
                                    w.uint32(24).int32(m.approval_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdAssetPolicySummary message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AdAssetPolicySummary.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdAssetPolicySummary} message AdAssetPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdAssetPolicySummary.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdAssetPolicySummary message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AdAssetPolicySummary} AdAssetPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdAssetPolicySummary.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AdAssetPolicySummary();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.policy_topic_entries && m.policy_topic_entries.length))
                                            m.policy_topic_entries = [];
                                        m.policy_topic_entries.push($root.google.ads.googleads.v11.common.PolicyTopicEntry.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.review_status = r.int32();
                                        break;
                                    case 3:
                                        m.approval_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdAssetPolicySummary message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AdAssetPolicySummary} AdAssetPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdAssetPolicySummary.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdAssetPolicySummary message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdAssetPolicySummary.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.policy_topic_entries != null && m.hasOwnProperty("policy_topic_entries")) {
                                    if (!Array.isArray(m.policy_topic_entries))
                                        return "policy_topic_entries: array expected";
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEntry.verify(m.policy_topic_entries[i]);
                                            if (e)
                                                return "policy_topic_entries." + e;
                                        }
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    switch (m.review_status) {
                                    default:
                                        return "review_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    switch (m.approval_status) {
                                    default:
                                        return "approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdAssetPolicySummary message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AdAssetPolicySummary} AdAssetPolicySummary
                             */
                            AdAssetPolicySummary.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AdAssetPolicySummary)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AdAssetPolicySummary();
                                if (d.policy_topic_entries) {
                                    if (!Array.isArray(d.policy_topic_entries))
                                        throw TypeError(".google.ads.googleads.v11.common.AdAssetPolicySummary.policy_topic_entries: array expected");
                                    m.policy_topic_entries = [];
                                    for (var i = 0; i < d.policy_topic_entries.length; ++i) {
                                        if (typeof d.policy_topic_entries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AdAssetPolicySummary.policy_topic_entries: object expected");
                                        m.policy_topic_entries[i] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.fromObject(d.policy_topic_entries[i]);
                                    }
                                }
                                switch (d.review_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.review_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.review_status = 1;
                                    break;
                                case "REVIEW_IN_PROGRESS":
                                case 2:
                                    m.review_status = 2;
                                    break;
                                case "REVIEWED":
                                case 3:
                                    m.review_status = 3;
                                    break;
                                case "UNDER_APPEAL":
                                case 4:
                                    m.review_status = 4;
                                    break;
                                case "ELIGIBLE_MAY_SERVE":
                                case 5:
                                    m.review_status = 5;
                                    break;
                                }
                                switch (d.approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approval_status = 1;
                                    break;
                                case "DISAPPROVED":
                                case 2:
                                    m.approval_status = 2;
                                    break;
                                case "APPROVED_LIMITED":
                                case 3:
                                    m.approval_status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.approval_status = 4;
                                    break;
                                case "AREA_OF_INTEREST_ONLY":
                                case 5:
                                    m.approval_status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdAssetPolicySummary message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.AdAssetPolicySummary} m AdAssetPolicySummary
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdAssetPolicySummary.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.policy_topic_entries = [];
                                }
                                if (o.defaults) {
                                    d.review_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.policy_topic_entries && m.policy_topic_entries.length) {
                                    d.policy_topic_entries = [];
                                    for (var j = 0; j < m.policy_topic_entries.length; ++j) {
                                        d.policy_topic_entries[j] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.toObject(m.policy_topic_entries[j], o);
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    d.review_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus[m.review_status] : m.review_status;
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    d.approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus[m.approval_status] : m.approval_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdAssetPolicySummary to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AdAssetPolicySummary
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdAssetPolicySummary.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdAssetPolicySummary;
                        })();
    
                        common.PolicyViolationKey = (function() {
    
                            /**
                             * Properties of a PolicyViolationKey.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPolicyViolationKey
                             * @property {string|null} [policy_name] PolicyViolationKey policy_name
                             * @property {string|null} [violating_text] PolicyViolationKey violating_text
                             */
    
                            /**
                             * Constructs a new PolicyViolationKey.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PolicyViolationKey.
                             * @implements IPolicyViolationKey
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPolicyViolationKey=} [p] Properties to set
                             */
                            function PolicyViolationKey(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicyViolationKey policy_name.
                             * @member {string|null|undefined} policy_name
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @instance
                             */
                            PolicyViolationKey.prototype.policy_name = null;
    
                            /**
                             * PolicyViolationKey violating_text.
                             * @member {string|null|undefined} violating_text
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @instance
                             */
                            PolicyViolationKey.prototype.violating_text = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PolicyViolationKey _policy_name.
                             * @member {"policy_name"|undefined} _policy_name
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @instance
                             */
                            Object.defineProperty(PolicyViolationKey.prototype, "_policy_name", {
                                get: $util.oneOfGetter($oneOfFields = ["policy_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PolicyViolationKey _violating_text.
                             * @member {"violating_text"|undefined} _violating_text
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @instance
                             */
                            Object.defineProperty(PolicyViolationKey.prototype, "_violating_text", {
                                get: $util.oneOfGetter($oneOfFields = ["violating_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PolicyViolationKey instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyViolationKey=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PolicyViolationKey} PolicyViolationKey instance
                             */
                            PolicyViolationKey.create = function create(properties) {
                                return new PolicyViolationKey(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyViolationKey message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyViolationKey.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyViolationKey} m PolicyViolationKey message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyViolationKey.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.policy_name != null && Object.hasOwnProperty.call(m, "policy_name"))
                                    w.uint32(26).string(m.policy_name);
                                if (m.violating_text != null && Object.hasOwnProperty.call(m, "violating_text"))
                                    w.uint32(34).string(m.violating_text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyViolationKey message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyViolationKey.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyViolationKey} message PolicyViolationKey message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyViolationKey.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyViolationKey message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PolicyViolationKey} PolicyViolationKey
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyViolationKey.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyViolationKey();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.policy_name = r.string();
                                        break;
                                    case 4:
                                        m.violating_text = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyViolationKey message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PolicyViolationKey} PolicyViolationKey
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyViolationKey.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyViolationKey message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyViolationKey.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.policy_name != null && m.hasOwnProperty("policy_name")) {
                                    p._policy_name = 1;
                                    if (!$util.isString(m.policy_name))
                                        return "policy_name: string expected";
                                }
                                if (m.violating_text != null && m.hasOwnProperty("violating_text")) {
                                    p._violating_text = 1;
                                    if (!$util.isString(m.violating_text))
                                        return "violating_text: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicyViolationKey message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PolicyViolationKey} PolicyViolationKey
                             */
                            PolicyViolationKey.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PolicyViolationKey)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PolicyViolationKey();
                                if (d.policy_name != null) {
                                    m.policy_name = String(d.policy_name);
                                }
                                if (d.violating_text != null) {
                                    m.violating_text = String(d.violating_text);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicyViolationKey message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @static
                             * @param {google.ads.googleads.v11.common.PolicyViolationKey} m PolicyViolationKey
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyViolationKey.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.policy_name != null && m.hasOwnProperty("policy_name")) {
                                    d.policy_name = m.policy_name;
                                    if (o.oneofs)
                                        d._policy_name = "policy_name";
                                }
                                if (m.violating_text != null && m.hasOwnProperty("violating_text")) {
                                    d.violating_text = m.violating_text;
                                    if (o.oneofs)
                                        d._violating_text = "violating_text";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicyViolationKey to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PolicyViolationKey
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyViolationKey.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PolicyViolationKey;
                        })();
    
                        common.PolicyValidationParameter = (function() {
    
                            /**
                             * Properties of a PolicyValidationParameter.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPolicyValidationParameter
                             * @property {Array.<string>|null} [ignorable_policy_topics] PolicyValidationParameter ignorable_policy_topics
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyViolationKey>|null} [exempt_policy_violation_keys] PolicyValidationParameter exempt_policy_violation_keys
                             */
    
                            /**
                             * Constructs a new PolicyValidationParameter.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PolicyValidationParameter.
                             * @implements IPolicyValidationParameter
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPolicyValidationParameter=} [p] Properties to set
                             */
                            function PolicyValidationParameter(p) {
                                this.ignorable_policy_topics = [];
                                this.exempt_policy_violation_keys = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicyValidationParameter ignorable_policy_topics.
                             * @member {Array.<string>} ignorable_policy_topics
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @instance
                             */
                            PolicyValidationParameter.prototype.ignorable_policy_topics = $util.emptyArray;
    
                            /**
                             * PolicyValidationParameter exempt_policy_violation_keys.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyViolationKey>} exempt_policy_violation_keys
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @instance
                             */
                            PolicyValidationParameter.prototype.exempt_policy_violation_keys = $util.emptyArray;
    
                            /**
                             * Creates a new PolicyValidationParameter instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyValidationParameter=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PolicyValidationParameter} PolicyValidationParameter instance
                             */
                            PolicyValidationParameter.create = function create(properties) {
                                return new PolicyValidationParameter(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyValidationParameter message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyValidationParameter.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyValidationParameter} m PolicyValidationParameter message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyValidationParameter.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.exempt_policy_violation_keys != null && m.exempt_policy_violation_keys.length) {
                                    for (var i = 0; i < m.exempt_policy_violation_keys.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyViolationKey.encode(m.exempt_policy_violation_keys[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.ignorable_policy_topics != null && m.ignorable_policy_topics.length) {
                                    for (var i = 0; i < m.ignorable_policy_topics.length; ++i)
                                        w.uint32(26).string(m.ignorable_policy_topics[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyValidationParameter message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyValidationParameter.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyValidationParameter} message PolicyValidationParameter message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyValidationParameter.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyValidationParameter message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PolicyValidationParameter} PolicyValidationParameter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyValidationParameter.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyValidationParameter();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        if (!(m.ignorable_policy_topics && m.ignorable_policy_topics.length))
                                            m.ignorable_policy_topics = [];
                                        m.ignorable_policy_topics.push(r.string());
                                        break;
                                    case 2:
                                        if (!(m.exempt_policy_violation_keys && m.exempt_policy_violation_keys.length))
                                            m.exempt_policy_violation_keys = [];
                                        m.exempt_policy_violation_keys.push($root.google.ads.googleads.v11.common.PolicyViolationKey.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyValidationParameter message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PolicyValidationParameter} PolicyValidationParameter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyValidationParameter.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyValidationParameter message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyValidationParameter.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.ignorable_policy_topics != null && m.hasOwnProperty("ignorable_policy_topics")) {
                                    if (!Array.isArray(m.ignorable_policy_topics))
                                        return "ignorable_policy_topics: array expected";
                                    for (var i = 0; i < m.ignorable_policy_topics.length; ++i) {
                                        if (!$util.isString(m.ignorable_policy_topics[i]))
                                            return "ignorable_policy_topics: string[] expected";
                                    }
                                }
                                if (m.exempt_policy_violation_keys != null && m.hasOwnProperty("exempt_policy_violation_keys")) {
                                    if (!Array.isArray(m.exempt_policy_violation_keys))
                                        return "exempt_policy_violation_keys: array expected";
                                    for (var i = 0; i < m.exempt_policy_violation_keys.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyViolationKey.verify(m.exempt_policy_violation_keys[i]);
                                            if (e)
                                                return "exempt_policy_violation_keys." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicyValidationParameter message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PolicyValidationParameter} PolicyValidationParameter
                             */
                            PolicyValidationParameter.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PolicyValidationParameter)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PolicyValidationParameter();
                                if (d.ignorable_policy_topics) {
                                    if (!Array.isArray(d.ignorable_policy_topics))
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyValidationParameter.ignorable_policy_topics: array expected");
                                    m.ignorable_policy_topics = [];
                                    for (var i = 0; i < d.ignorable_policy_topics.length; ++i) {
                                        m.ignorable_policy_topics[i] = String(d.ignorable_policy_topics[i]);
                                    }
                                }
                                if (d.exempt_policy_violation_keys) {
                                    if (!Array.isArray(d.exempt_policy_violation_keys))
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyValidationParameter.exempt_policy_violation_keys: array expected");
                                    m.exempt_policy_violation_keys = [];
                                    for (var i = 0; i < d.exempt_policy_violation_keys.length; ++i) {
                                        if (typeof d.exempt_policy_violation_keys[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyValidationParameter.exempt_policy_violation_keys: object expected");
                                        m.exempt_policy_violation_keys[i] = $root.google.ads.googleads.v11.common.PolicyViolationKey.fromObject(d.exempt_policy_violation_keys[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicyValidationParameter message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.PolicyValidationParameter} m PolicyValidationParameter
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyValidationParameter.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.exempt_policy_violation_keys = [];
                                    d.ignorable_policy_topics = [];
                                }
                                if (m.exempt_policy_violation_keys && m.exempt_policy_violation_keys.length) {
                                    d.exempt_policy_violation_keys = [];
                                    for (var j = 0; j < m.exempt_policy_violation_keys.length; ++j) {
                                        d.exempt_policy_violation_keys[j] = $root.google.ads.googleads.v11.common.PolicyViolationKey.toObject(m.exempt_policy_violation_keys[j], o);
                                    }
                                }
                                if (m.ignorable_policy_topics && m.ignorable_policy_topics.length) {
                                    d.ignorable_policy_topics = [];
                                    for (var j = 0; j < m.ignorable_policy_topics.length; ++j) {
                                        d.ignorable_policy_topics[j] = m.ignorable_policy_topics[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicyValidationParameter to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PolicyValidationParameter
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyValidationParameter.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PolicyValidationParameter;
                        })();
    
                        common.PolicyTopicEntry = (function() {
    
                            /**
                             * Properties of a PolicyTopicEntry.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPolicyTopicEntry
                             * @property {string|null} [topic] PolicyTopicEntry topic
                             * @property {google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum.PolicyTopicEntryType|null} [type] PolicyTopicEntry type
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEvidence>|null} [evidences] PolicyTopicEntry evidences
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicConstraint>|null} [constraints] PolicyTopicEntry constraints
                             */
    
                            /**
                             * Constructs a new PolicyTopicEntry.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PolicyTopicEntry.
                             * @implements IPolicyTopicEntry
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEntry=} [p] Properties to set
                             */
                            function PolicyTopicEntry(p) {
                                this.evidences = [];
                                this.constraints = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicyTopicEntry topic.
                             * @member {string|null|undefined} topic
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @instance
                             */
                            PolicyTopicEntry.prototype.topic = null;
    
                            /**
                             * PolicyTopicEntry type.
                             * @member {google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum.PolicyTopicEntryType} type
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @instance
                             */
                            PolicyTopicEntry.prototype.type = 0;
    
                            /**
                             * PolicyTopicEntry evidences.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEvidence>} evidences
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @instance
                             */
                            PolicyTopicEntry.prototype.evidences = $util.emptyArray;
    
                            /**
                             * PolicyTopicEntry constraints.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicConstraint>} constraints
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @instance
                             */
                            PolicyTopicEntry.prototype.constraints = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PolicyTopicEntry _topic.
                             * @member {"topic"|undefined} _topic
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @instance
                             */
                            Object.defineProperty(PolicyTopicEntry.prototype, "_topic", {
                                get: $util.oneOfGetter($oneOfFields = ["topic"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PolicyTopicEntry instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEntry=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEntry} PolicyTopicEntry instance
                             */
                            PolicyTopicEntry.create = function create(properties) {
                                return new PolicyTopicEntry(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEntry message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEntry.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEntry} m PolicyTopicEntry message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEntry.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(16).int32(m.type);
                                if (m.evidences != null && m.evidences.length) {
                                    for (var i = 0; i < m.evidences.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEvidence.encode(m.evidences[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.constraints != null && m.constraints.length) {
                                    for (var i = 0; i < m.constraints.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicConstraint.encode(m.constraints[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.topic != null && Object.hasOwnProperty.call(m, "topic"))
                                    w.uint32(42).string(m.topic);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEntry message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEntry.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEntry} message PolicyTopicEntry message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEntry.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyTopicEntry message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEntry} PolicyTopicEntry
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEntry.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicEntry();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.topic = r.string();
                                        break;
                                    case 2:
                                        m.type = r.int32();
                                        break;
                                    case 3:
                                        if (!(m.evidences && m.evidences.length))
                                            m.evidences = [];
                                        m.evidences.push($root.google.ads.googleads.v11.common.PolicyTopicEvidence.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.constraints && m.constraints.length))
                                            m.constraints = [];
                                        m.constraints.push($root.google.ads.googleads.v11.common.PolicyTopicConstraint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyTopicEntry message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEntry} PolicyTopicEntry
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEntry.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyTopicEntry message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyTopicEntry.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.topic != null && m.hasOwnProperty("topic")) {
                                    p._topic = 1;
                                    if (!$util.isString(m.topic))
                                        return "topic: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                    case 8:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.evidences != null && m.hasOwnProperty("evidences")) {
                                    if (!Array.isArray(m.evidences))
                                        return "evidences: array expected";
                                    for (var i = 0; i < m.evidences.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.verify(m.evidences[i]);
                                            if (e)
                                                return "evidences." + e;
                                        }
                                    }
                                }
                                if (m.constraints != null && m.hasOwnProperty("constraints")) {
                                    if (!Array.isArray(m.constraints))
                                        return "constraints: array expected";
                                    for (var i = 0; i < m.constraints.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.verify(m.constraints[i]);
                                            if (e)
                                                return "constraints." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicyTopicEntry message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEntry} PolicyTopicEntry
                             */
                            PolicyTopicEntry.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicEntry)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PolicyTopicEntry();
                                if (d.topic != null) {
                                    m.topic = String(d.topic);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "PROHIBITED":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "LIMITED":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "FULLY_LIMITED":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "DESCRIPTIVE":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "BROADENING":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "AREA_OF_INTEREST_ONLY":
                                case 7:
                                    m.type = 7;
                                    break;
                                }
                                if (d.evidences) {
                                    if (!Array.isArray(d.evidences))
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEntry.evidences: array expected");
                                    m.evidences = [];
                                    for (var i = 0; i < d.evidences.length; ++i) {
                                        if (typeof d.evidences[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEntry.evidences: object expected");
                                        m.evidences[i] = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.fromObject(d.evidences[i]);
                                    }
                                }
                                if (d.constraints) {
                                    if (!Array.isArray(d.constraints))
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEntry.constraints: array expected");
                                    m.constraints = [];
                                    for (var i = 0; i < d.constraints.length; ++i) {
                                        if (typeof d.constraints[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEntry.constraints: object expected");
                                        m.constraints[i] = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.fromObject(d.constraints[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicyTopicEntry message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.PolicyTopicEntry} m PolicyTopicEntry
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyTopicEntry.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.evidences = [];
                                    d.constraints = [];
                                }
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum.PolicyTopicEntryType[m.type] : m.type;
                                }
                                if (m.evidences && m.evidences.length) {
                                    d.evidences = [];
                                    for (var j = 0; j < m.evidences.length; ++j) {
                                        d.evidences[j] = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.toObject(m.evidences[j], o);
                                    }
                                }
                                if (m.constraints && m.constraints.length) {
                                    d.constraints = [];
                                    for (var j = 0; j < m.constraints.length; ++j) {
                                        d.constraints[j] = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.toObject(m.constraints[j], o);
                                    }
                                }
                                if (m.topic != null && m.hasOwnProperty("topic")) {
                                    d.topic = m.topic;
                                    if (o.oneofs)
                                        d._topic = "topic";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicyTopicEntry to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEntry
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyTopicEntry.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PolicyTopicEntry;
                        })();
    
                        common.PolicyTopicEvidence = (function() {
    
                            /**
                             * Properties of a PolicyTopicEvidence.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPolicyTopicEvidence
                             * @property {google.ads.googleads.v11.common.PolicyTopicEvidence.IWebsiteList|null} [website_list] PolicyTopicEvidence website_list
                             * @property {google.ads.googleads.v11.common.PolicyTopicEvidence.ITextList|null} [text_list] PolicyTopicEvidence text_list
                             * @property {string|null} [language_code] PolicyTopicEvidence language_code
                             * @property {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationTextList|null} [destination_text_list] PolicyTopicEvidence destination_text_list
                             * @property {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationMismatch|null} [destination_mismatch] PolicyTopicEvidence destination_mismatch
                             * @property {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationNotWorking|null} [destination_not_working] PolicyTopicEvidence destination_not_working
                             */
    
                            /**
                             * Constructs a new PolicyTopicEvidence.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PolicyTopicEvidence.
                             * @implements IPolicyTopicEvidence
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEvidence=} [p] Properties to set
                             */
                            function PolicyTopicEvidence(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicyTopicEvidence website_list.
                             * @member {google.ads.googleads.v11.common.PolicyTopicEvidence.IWebsiteList|null|undefined} website_list
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             */
                            PolicyTopicEvidence.prototype.website_list = null;
    
                            /**
                             * PolicyTopicEvidence text_list.
                             * @member {google.ads.googleads.v11.common.PolicyTopicEvidence.ITextList|null|undefined} text_list
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             */
                            PolicyTopicEvidence.prototype.text_list = null;
    
                            /**
                             * PolicyTopicEvidence language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             */
                            PolicyTopicEvidence.prototype.language_code = null;
    
                            /**
                             * PolicyTopicEvidence destination_text_list.
                             * @member {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationTextList|null|undefined} destination_text_list
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             */
                            PolicyTopicEvidence.prototype.destination_text_list = null;
    
                            /**
                             * PolicyTopicEvidence destination_mismatch.
                             * @member {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationMismatch|null|undefined} destination_mismatch
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             */
                            PolicyTopicEvidence.prototype.destination_mismatch = null;
    
                            /**
                             * PolicyTopicEvidence destination_not_working.
                             * @member {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationNotWorking|null|undefined} destination_not_working
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             */
                            PolicyTopicEvidence.prototype.destination_not_working = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PolicyTopicEvidence value.
                             * @member {"website_list"|"text_list"|"language_code"|"destination_text_list"|"destination_mismatch"|"destination_not_working"|undefined} value
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             */
                            Object.defineProperty(PolicyTopicEvidence.prototype, "value", {
                                get: $util.oneOfGetter($oneOfFields = ["website_list", "text_list", "language_code", "destination_text_list", "destination_mismatch", "destination_not_working"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PolicyTopicEvidence instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEvidence=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence} PolicyTopicEvidence instance
                             */
                            PolicyTopicEvidence.create = function create(properties) {
                                return new PolicyTopicEvidence(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidence message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEvidence} m PolicyTopicEvidence message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidence.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.website_list != null && Object.hasOwnProperty.call(m, "website_list"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.encode(m.website_list, w.uint32(26).fork()).ldelim();
                                if (m.text_list != null && Object.hasOwnProperty.call(m, "text_list"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.encode(m.text_list, w.uint32(34).fork()).ldelim();
                                if (m.destination_text_list != null && Object.hasOwnProperty.call(m, "destination_text_list"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.encode(m.destination_text_list, w.uint32(50).fork()).ldelim();
                                if (m.destination_mismatch != null && Object.hasOwnProperty.call(m, "destination_mismatch"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.encode(m.destination_mismatch, w.uint32(58).fork()).ldelim();
                                if (m.destination_not_working != null && Object.hasOwnProperty.call(m, "destination_not_working"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking.encode(m.destination_not_working, w.uint32(66).fork()).ldelim();
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(74).string(m.language_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidence message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicEvidence} message PolicyTopicEvidence message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidence.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidence message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence} PolicyTopicEvidence
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidence.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.website_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.text_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.language_code = r.string();
                                        break;
                                    case 6:
                                        m.destination_text_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.destination_mismatch = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.destination_not_working = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidence message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence} PolicyTopicEvidence
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidence.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyTopicEvidence message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyTopicEvidence.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.website_list != null && m.hasOwnProperty("website_list")) {
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.verify(m.website_list);
                                        if (e)
                                            return "website_list." + e;
                                    }
                                }
                                if (m.text_list != null && m.hasOwnProperty("text_list")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.verify(m.text_list);
                                        if (e)
                                            return "text_list." + e;
                                    }
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.destination_text_list != null && m.hasOwnProperty("destination_text_list")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.verify(m.destination_text_list);
                                        if (e)
                                            return "destination_text_list." + e;
                                    }
                                }
                                if (m.destination_mismatch != null && m.hasOwnProperty("destination_mismatch")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.verify(m.destination_mismatch);
                                        if (e)
                                            return "destination_mismatch." + e;
                                    }
                                }
                                if (m.destination_not_working != null && m.hasOwnProperty("destination_not_working")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking.verify(m.destination_not_working);
                                        if (e)
                                            return "destination_not_working." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicyTopicEvidence message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence} PolicyTopicEvidence
                             */
                            PolicyTopicEvidence.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicEvidence)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence();
                                if (d.website_list != null) {
                                    if (typeof d.website_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.website_list: object expected");
                                    m.website_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.fromObject(d.website_list);
                                }
                                if (d.text_list != null) {
                                    if (typeof d.text_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.text_list: object expected");
                                    m.text_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.fromObject(d.text_list);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.destination_text_list != null) {
                                    if (typeof d.destination_text_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.destination_text_list: object expected");
                                    m.destination_text_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.fromObject(d.destination_text_list);
                                }
                                if (d.destination_mismatch != null) {
                                    if (typeof d.destination_mismatch !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.destination_mismatch: object expected");
                                    m.destination_mismatch = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.fromObject(d.destination_mismatch);
                                }
                                if (d.destination_not_working != null) {
                                    if (typeof d.destination_not_working !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.destination_not_working: object expected");
                                    m.destination_not_working = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking.fromObject(d.destination_not_working);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicyTopicEvidence message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @static
                             * @param {google.ads.googleads.v11.common.PolicyTopicEvidence} m PolicyTopicEvidence
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyTopicEvidence.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.website_list != null && m.hasOwnProperty("website_list")) {
                                    d.website_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.toObject(m.website_list, o);
                                    if (o.oneofs)
                                        d.value = "website_list";
                                }
                                if (m.text_list != null && m.hasOwnProperty("text_list")) {
                                    d.text_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.toObject(m.text_list, o);
                                    if (o.oneofs)
                                        d.value = "text_list";
                                }
                                if (m.destination_text_list != null && m.hasOwnProperty("destination_text_list")) {
                                    d.destination_text_list = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.toObject(m.destination_text_list, o);
                                    if (o.oneofs)
                                        d.value = "destination_text_list";
                                }
                                if (m.destination_mismatch != null && m.hasOwnProperty("destination_mismatch")) {
                                    d.destination_mismatch = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.toObject(m.destination_mismatch, o);
                                    if (o.oneofs)
                                        d.value = "destination_mismatch";
                                }
                                if (m.destination_not_working != null && m.hasOwnProperty("destination_not_working")) {
                                    d.destination_not_working = $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking.toObject(m.destination_not_working, o);
                                    if (o.oneofs)
                                        d.value = "destination_not_working";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d.value = "language_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicyTopicEvidence to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyTopicEvidence.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            PolicyTopicEvidence.TextList = (function() {
    
                                /**
                                 * Properties of a TextList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @interface ITextList
                                 * @property {Array.<string>|null} [texts] TextList texts
                                 */
    
                                /**
                                 * Constructs a new TextList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @classdesc Represents a TextList.
                                 * @implements ITextList
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.ITextList=} [p] Properties to set
                                 */
                                function TextList(p) {
                                    this.texts = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TextList texts.
                                 * @member {Array.<string>} texts
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @instance
                                 */
                                TextList.prototype.texts = $util.emptyArray;
    
                                /**
                                 * Creates a new TextList instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.ITextList=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.TextList} TextList instance
                                 */
                                TextList.create = function create(properties) {
                                    return new TextList(properties);
                                };
    
                                /**
                                 * Encodes the specified TextList message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.ITextList} m TextList message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TextList.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.texts != null && m.texts.length) {
                                        for (var i = 0; i < m.texts.length; ++i)
                                            w.uint32(18).string(m.texts[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TextList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.ITextList} message TextList message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TextList.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TextList message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.TextList} TextList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TextList.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            if (!(m.texts && m.texts.length))
                                                m.texts = [];
                                            m.texts.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TextList message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.TextList} TextList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TextList.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TextList message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TextList.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.texts != null && m.hasOwnProperty("texts")) {
                                        if (!Array.isArray(m.texts))
                                            return "texts: array expected";
                                        for (var i = 0; i < m.texts.length; ++i) {
                                            if (!$util.isString(m.texts[i]))
                                                return "texts: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TextList message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.TextList} TextList
                                 */
                                TextList.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.TextList();
                                    if (d.texts) {
                                        if (!Array.isArray(d.texts))
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.TextList.texts: array expected");
                                        m.texts = [];
                                        for (var i = 0; i < d.texts.length; ++i) {
                                            m.texts[i] = String(d.texts[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TextList message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.TextList} m TextList
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TextList.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.texts = [];
                                    }
                                    if (m.texts && m.texts.length) {
                                        d.texts = [];
                                        for (var j = 0; j < m.texts.length; ++j) {
                                            d.texts[j] = m.texts[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TextList to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.TextList
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TextList.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TextList;
                            })();
    
                            PolicyTopicEvidence.WebsiteList = (function() {
    
                                /**
                                 * Properties of a WebsiteList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @interface IWebsiteList
                                 * @property {Array.<string>|null} [websites] WebsiteList websites
                                 */
    
                                /**
                                 * Constructs a new WebsiteList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @classdesc Represents a WebsiteList.
                                 * @implements IWebsiteList
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IWebsiteList=} [p] Properties to set
                                 */
                                function WebsiteList(p) {
                                    this.websites = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * WebsiteList websites.
                                 * @member {Array.<string>} websites
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @instance
                                 */
                                WebsiteList.prototype.websites = $util.emptyArray;
    
                                /**
                                 * Creates a new WebsiteList instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IWebsiteList=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList} WebsiteList instance
                                 */
                                WebsiteList.create = function create(properties) {
                                    return new WebsiteList(properties);
                                };
    
                                /**
                                 * Encodes the specified WebsiteList message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IWebsiteList} m WebsiteList message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                WebsiteList.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.websites != null && m.websites.length) {
                                        for (var i = 0; i < m.websites.length; ++i)
                                            w.uint32(18).string(m.websites[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified WebsiteList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IWebsiteList} message WebsiteList message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                WebsiteList.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a WebsiteList message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList} WebsiteList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                WebsiteList.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            if (!(m.websites && m.websites.length))
                                                m.websites = [];
                                            m.websites.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a WebsiteList message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList} WebsiteList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                WebsiteList.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a WebsiteList message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                WebsiteList.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.websites != null && m.hasOwnProperty("websites")) {
                                        if (!Array.isArray(m.websites))
                                            return "websites: array expected";
                                        for (var i = 0; i < m.websites.length; ++i) {
                                            if (!$util.isString(m.websites[i]))
                                                return "websites: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a WebsiteList message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList} WebsiteList
                                 */
                                WebsiteList.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList();
                                    if (d.websites) {
                                        if (!Array.isArray(d.websites))
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList.websites: array expected");
                                        m.websites = [];
                                        for (var i = 0; i < d.websites.length; ++i) {
                                            m.websites[i] = String(d.websites[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a WebsiteList message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList} m WebsiteList
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                WebsiteList.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.websites = [];
                                    }
                                    if (m.websites && m.websites.length) {
                                        d.websites = [];
                                        for (var j = 0; j < m.websites.length; ++j) {
                                            d.websites[j] = m.websites[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this WebsiteList to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.WebsiteList
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                WebsiteList.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return WebsiteList;
                            })();
    
                            PolicyTopicEvidence.DestinationTextList = (function() {
    
                                /**
                                 * Properties of a DestinationTextList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @interface IDestinationTextList
                                 * @property {Array.<string>|null} [destination_texts] DestinationTextList destination_texts
                                 */
    
                                /**
                                 * Constructs a new DestinationTextList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @classdesc Represents a DestinationTextList.
                                 * @implements IDestinationTextList
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationTextList=} [p] Properties to set
                                 */
                                function DestinationTextList(p) {
                                    this.destination_texts = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * DestinationTextList destination_texts.
                                 * @member {Array.<string>} destination_texts
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @instance
                                 */
                                DestinationTextList.prototype.destination_texts = $util.emptyArray;
    
                                /**
                                 * Creates a new DestinationTextList instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationTextList=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList} DestinationTextList instance
                                 */
                                DestinationTextList.create = function create(properties) {
                                    return new DestinationTextList(properties);
                                };
    
                                /**
                                 * Encodes the specified DestinationTextList message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationTextList} m DestinationTextList message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DestinationTextList.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.destination_texts != null && m.destination_texts.length) {
                                        for (var i = 0; i < m.destination_texts.length; ++i)
                                            w.uint32(18).string(m.destination_texts[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified DestinationTextList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationTextList} message DestinationTextList message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DestinationTextList.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a DestinationTextList message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList} DestinationTextList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DestinationTextList.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            if (!(m.destination_texts && m.destination_texts.length))
                                                m.destination_texts = [];
                                            m.destination_texts.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a DestinationTextList message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList} DestinationTextList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DestinationTextList.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a DestinationTextList message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                DestinationTextList.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.destination_texts != null && m.hasOwnProperty("destination_texts")) {
                                        if (!Array.isArray(m.destination_texts))
                                            return "destination_texts: array expected";
                                        for (var i = 0; i < m.destination_texts.length; ++i) {
                                            if (!$util.isString(m.destination_texts[i]))
                                                return "destination_texts: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a DestinationTextList message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList} DestinationTextList
                                 */
                                DestinationTextList.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList();
                                    if (d.destination_texts) {
                                        if (!Array.isArray(d.destination_texts))
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList.destination_texts: array expected");
                                        m.destination_texts = [];
                                        for (var i = 0; i < d.destination_texts.length; ++i) {
                                            m.destination_texts[i] = String(d.destination_texts[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a DestinationTextList message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList} m DestinationTextList
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                DestinationTextList.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.destination_texts = [];
                                    }
                                    if (m.destination_texts && m.destination_texts.length) {
                                        d.destination_texts = [];
                                        for (var j = 0; j < m.destination_texts.length; ++j) {
                                            d.destination_texts[j] = m.destination_texts[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this DestinationTextList to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationTextList
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                DestinationTextList.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return DestinationTextList;
                            })();
    
                            PolicyTopicEvidence.DestinationMismatch = (function() {
    
                                /**
                                 * Properties of a DestinationMismatch.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @interface IDestinationMismatch
                                 * @property {Array.<google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType>|null} [url_types] DestinationMismatch url_types
                                 */
    
                                /**
                                 * Constructs a new DestinationMismatch.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @classdesc Represents a DestinationMismatch.
                                 * @implements IDestinationMismatch
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationMismatch=} [p] Properties to set
                                 */
                                function DestinationMismatch(p) {
                                    this.url_types = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * DestinationMismatch url_types.
                                 * @member {Array.<google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType>} url_types
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @instance
                                 */
                                DestinationMismatch.prototype.url_types = $util.emptyArray;
    
                                /**
                                 * Creates a new DestinationMismatch instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationMismatch=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch} DestinationMismatch instance
                                 */
                                DestinationMismatch.create = function create(properties) {
                                    return new DestinationMismatch(properties);
                                };
    
                                /**
                                 * Encodes the specified DestinationMismatch message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationMismatch} m DestinationMismatch message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DestinationMismatch.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.url_types != null && m.url_types.length) {
                                        w.uint32(10).fork();
                                        for (var i = 0; i < m.url_types.length; ++i)
                                            w.int32(m.url_types[i]);
                                        w.ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified DestinationMismatch message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationMismatch} message DestinationMismatch message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DestinationMismatch.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a DestinationMismatch message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch} DestinationMismatch
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DestinationMismatch.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.url_types && m.url_types.length))
                                                m.url_types = [];
                                            if ((t & 7) === 2) {
                                                var c2 = r.uint32() + r.pos;
                                                while (r.pos < c2)
                                                    m.url_types.push(r.int32());
                                            } else
                                                m.url_types.push(r.int32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a DestinationMismatch message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch} DestinationMismatch
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DestinationMismatch.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a DestinationMismatch message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                DestinationMismatch.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.url_types != null && m.hasOwnProperty("url_types")) {
                                        if (!Array.isArray(m.url_types))
                                            return "url_types: array expected";
                                        for (var i = 0; i < m.url_types.length; ++i) {
                                            switch (m.url_types[i]) {
                                            default:
                                                return "url_types: enum value[] expected";
                                            case 0:
                                            case 1:
                                            case 2:
                                            case 3:
                                            case 4:
                                            case 5:
                                            case 6:
                                                break;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a DestinationMismatch message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch} DestinationMismatch
                                 */
                                DestinationMismatch.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch();
                                    if (d.url_types) {
                                        if (!Array.isArray(d.url_types))
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch.url_types: array expected");
                                        m.url_types = [];
                                        for (var i = 0; i < d.url_types.length; ++i) {
                                            switch (d.url_types[i]) {
                                            default:
                                            case "UNSPECIFIED":
                                            case 0:
                                                m.url_types[i] = 0;
                                                break;
                                            case "UNKNOWN":
                                            case 1:
                                                m.url_types[i] = 1;
                                                break;
                                            case "DISPLAY_URL":
                                            case 2:
                                                m.url_types[i] = 2;
                                                break;
                                            case "FINAL_URL":
                                            case 3:
                                                m.url_types[i] = 3;
                                                break;
                                            case "FINAL_MOBILE_URL":
                                            case 4:
                                                m.url_types[i] = 4;
                                                break;
                                            case "TRACKING_URL":
                                            case 5:
                                                m.url_types[i] = 5;
                                                break;
                                            case "MOBILE_TRACKING_URL":
                                            case 6:
                                                m.url_types[i] = 6;
                                                break;
                                            }
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a DestinationMismatch message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch} m DestinationMismatch
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                DestinationMismatch.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.url_types = [];
                                    }
                                    if (m.url_types && m.url_types.length) {
                                        d.url_types = [];
                                        for (var j = 0; j < m.url_types.length; ++j) {
                                            d.url_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType[m.url_types[j]] : m.url_types[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this DestinationMismatch to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationMismatch
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                DestinationMismatch.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return DestinationMismatch;
                            })();
    
                            PolicyTopicEvidence.DestinationNotWorking = (function() {
    
                                /**
                                 * Properties of a DestinationNotWorking.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @interface IDestinationNotWorking
                                 * @property {string|null} [expanded_url] DestinationNotWorking expanded_url
                                 * @property {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice|null} [device] DestinationNotWorking device
                                 * @property {string|null} [last_checked_date_time] DestinationNotWorking last_checked_date_time
                                 * @property {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType|null} [dns_error_type] DestinationNotWorking dns_error_type
                                 * @property {number|null} [http_error_code] DestinationNotWorking http_error_code
                                 */
    
                                /**
                                 * Constructs a new DestinationNotWorking.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence
                                 * @classdesc Represents a DestinationNotWorking.
                                 * @implements IDestinationNotWorking
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationNotWorking=} [p] Properties to set
                                 */
                                function DestinationNotWorking(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * DestinationNotWorking expanded_url.
                                 * @member {string|null|undefined} expanded_url
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                DestinationNotWorking.prototype.expanded_url = null;
    
                                /**
                                 * DestinationNotWorking device.
                                 * @member {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice} device
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                DestinationNotWorking.prototype.device = 0;
    
                                /**
                                 * DestinationNotWorking last_checked_date_time.
                                 * @member {string|null|undefined} last_checked_date_time
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                DestinationNotWorking.prototype.last_checked_date_time = null;
    
                                /**
                                 * DestinationNotWorking dns_error_type.
                                 * @member {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType|null|undefined} dns_error_type
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                DestinationNotWorking.prototype.dns_error_type = null;
    
                                /**
                                 * DestinationNotWorking http_error_code.
                                 * @member {number|null|undefined} http_error_code
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                DestinationNotWorking.prototype.http_error_code = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * DestinationNotWorking _expanded_url.
                                 * @member {"expanded_url"|undefined} _expanded_url
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                Object.defineProperty(DestinationNotWorking.prototype, "_expanded_url", {
                                    get: $util.oneOfGetter($oneOfFields = ["expanded_url"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * DestinationNotWorking _last_checked_date_time.
                                 * @member {"last_checked_date_time"|undefined} _last_checked_date_time
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                Object.defineProperty(DestinationNotWorking.prototype, "_last_checked_date_time", {
                                    get: $util.oneOfGetter($oneOfFields = ["last_checked_date_time"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * DestinationNotWorking reason.
                                 * @member {"dns_error_type"|"http_error_code"|undefined} reason
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 */
                                Object.defineProperty(DestinationNotWorking.prototype, "reason", {
                                    get: $util.oneOfGetter($oneOfFields = ["dns_error_type", "http_error_code"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new DestinationNotWorking instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationNotWorking=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking} DestinationNotWorking instance
                                 */
                                DestinationNotWorking.create = function create(properties) {
                                    return new DestinationNotWorking(properties);
                                };
    
                                /**
                                 * Encodes the specified DestinationNotWorking message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationNotWorking} m DestinationNotWorking message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DestinationNotWorking.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.dns_error_type != null && Object.hasOwnProperty.call(m, "dns_error_type"))
                                        w.uint32(8).int32(m.dns_error_type);
                                    if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                        w.uint32(32).int32(m.device);
                                    if (m.http_error_code != null && Object.hasOwnProperty.call(m, "http_error_code"))
                                        w.uint32(48).int64(m.http_error_code);
                                    if (m.expanded_url != null && Object.hasOwnProperty.call(m, "expanded_url"))
                                        w.uint32(58).string(m.expanded_url);
                                    if (m.last_checked_date_time != null && Object.hasOwnProperty.call(m, "last_checked_date_time"))
                                        w.uint32(66).string(m.last_checked_date_time);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified DestinationNotWorking message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.IDestinationNotWorking} message DestinationNotWorking message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DestinationNotWorking.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a DestinationNotWorking message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking} DestinationNotWorking
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DestinationNotWorking.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 7:
                                            m.expanded_url = r.string();
                                            break;
                                        case 4:
                                            m.device = r.int32();
                                            break;
                                        case 8:
                                            m.last_checked_date_time = r.string();
                                            break;
                                        case 1:
                                            m.dns_error_type = r.int32();
                                            break;
                                        case 6:
                                            m.http_error_code = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a DestinationNotWorking message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking} DestinationNotWorking
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DestinationNotWorking.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a DestinationNotWorking message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                DestinationNotWorking.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.expanded_url != null && m.hasOwnProperty("expanded_url")) {
                                        p._expanded_url = 1;
                                        if (!$util.isString(m.expanded_url))
                                            return "expanded_url: string expected";
                                    }
                                    if (m.device != null && m.hasOwnProperty("device")) {
                                        switch (m.device) {
                                        default:
                                            return "device: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    if (m.last_checked_date_time != null && m.hasOwnProperty("last_checked_date_time")) {
                                        p._last_checked_date_time = 1;
                                        if (!$util.isString(m.last_checked_date_time))
                                            return "last_checked_date_time: string expected";
                                    }
                                    if (m.dns_error_type != null && m.hasOwnProperty("dns_error_type")) {
                                        p.reason = 1;
                                        switch (m.dns_error_type) {
                                        default:
                                            return "dns_error_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    if (m.http_error_code != null && m.hasOwnProperty("http_error_code")) {
                                        if (p.reason === 1)
                                            return "reason: multiple values";
                                        p.reason = 1;
                                        if (!$util.isInteger(m.http_error_code) && !(m.http_error_code && $util.isInteger(m.http_error_code.low) && $util.isInteger(m.http_error_code.high)))
                                            return "http_error_code: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a DestinationNotWorking message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking} DestinationNotWorking
                                 */
                                DestinationNotWorking.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking();
                                    if (d.expanded_url != null) {
                                        m.expanded_url = String(d.expanded_url);
                                    }
                                    switch (d.device) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.device = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.device = 1;
                                        break;
                                    case "DESKTOP":
                                    case 2:
                                        m.device = 2;
                                        break;
                                    case "ANDROID":
                                    case 3:
                                        m.device = 3;
                                        break;
                                    case "IOS":
                                    case 4:
                                        m.device = 4;
                                        break;
                                    }
                                    if (d.last_checked_date_time != null) {
                                        m.last_checked_date_time = String(d.last_checked_date_time);
                                    }
                                    switch (d.dns_error_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.dns_error_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.dns_error_type = 1;
                                        break;
                                    case "HOSTNAME_NOT_FOUND":
                                    case 2:
                                        m.dns_error_type = 2;
                                        break;
                                    case "GOOGLE_CRAWLER_DNS_ISSUE":
                                    case 3:
                                        m.dns_error_type = 3;
                                        break;
                                    }
                                    if (d.http_error_code != null) {
                                        if ($util.Long)
                                            (m.http_error_code = $util.Long.fromValue(d.http_error_code)).unsigned = false;
                                        else if (typeof d.http_error_code === "string")
                                            m.http_error_code = parseInt(d.http_error_code, 10);
                                        else if (typeof d.http_error_code === "number")
                                            m.http_error_code = d.http_error_code;
                                        else if (typeof d.http_error_code === "object")
                                            m.http_error_code = new $util.LongBits(d.http_error_code.low >>> 0, d.http_error_code.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a DestinationNotWorking message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking} m DestinationNotWorking
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                DestinationNotWorking.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.device = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.dns_error_type != null && m.hasOwnProperty("dns_error_type")) {
                                        d.dns_error_type = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType[m.dns_error_type] : m.dns_error_type;
                                        if (o.oneofs)
                                            d.reason = "dns_error_type";
                                    }
                                    if (m.device != null && m.hasOwnProperty("device")) {
                                        d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice[m.device] : m.device;
                                    }
                                    if (m.http_error_code != null && m.hasOwnProperty("http_error_code")) {
                                        if (typeof m.http_error_code === "number")
                                            d.http_error_code = o.longs === String ? String(m.http_error_code) : m.http_error_code;
                                        else
                                            d.http_error_code = o.longs === String ? $util.Long.prototype.toString.call(m.http_error_code) : o.longs === Number ? new $util.LongBits(m.http_error_code.low >>> 0, m.http_error_code.high >>> 0).toNumber() : m.http_error_code;
                                        if (o.oneofs)
                                            d.reason = "http_error_code";
                                    }
                                    if (m.expanded_url != null && m.hasOwnProperty("expanded_url")) {
                                        d.expanded_url = m.expanded_url;
                                        if (o.oneofs)
                                            d._expanded_url = "expanded_url";
                                    }
                                    if (m.last_checked_date_time != null && m.hasOwnProperty("last_checked_date_time")) {
                                        d.last_checked_date_time = m.last_checked_date_time;
                                        if (o.oneofs)
                                            d._last_checked_date_time = "last_checked_date_time";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this DestinationNotWorking to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicEvidence.DestinationNotWorking
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                DestinationNotWorking.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return DestinationNotWorking;
                            })();
    
                            return PolicyTopicEvidence;
                        })();
    
                        common.PolicyTopicConstraint = (function() {
    
                            /**
                             * Properties of a PolicyTopicConstraint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPolicyTopicConstraint
                             * @property {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList|null} [country_constraint_list] PolicyTopicConstraint country_constraint_list
                             * @property {google.ads.googleads.v11.common.PolicyTopicConstraint.IResellerConstraint|null} [reseller_constraint] PolicyTopicConstraint reseller_constraint
                             * @property {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList|null} [certificate_missing_in_country_list] PolicyTopicConstraint certificate_missing_in_country_list
                             * @property {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList|null} [certificate_domain_mismatch_in_country_list] PolicyTopicConstraint certificate_domain_mismatch_in_country_list
                             */
    
                            /**
                             * Constructs a new PolicyTopicConstraint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PolicyTopicConstraint.
                             * @implements IPolicyTopicConstraint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPolicyTopicConstraint=} [p] Properties to set
                             */
                            function PolicyTopicConstraint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicyTopicConstraint country_constraint_list.
                             * @member {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList|null|undefined} country_constraint_list
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @instance
                             */
                            PolicyTopicConstraint.prototype.country_constraint_list = null;
    
                            /**
                             * PolicyTopicConstraint reseller_constraint.
                             * @member {google.ads.googleads.v11.common.PolicyTopicConstraint.IResellerConstraint|null|undefined} reseller_constraint
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @instance
                             */
                            PolicyTopicConstraint.prototype.reseller_constraint = null;
    
                            /**
                             * PolicyTopicConstraint certificate_missing_in_country_list.
                             * @member {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList|null|undefined} certificate_missing_in_country_list
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @instance
                             */
                            PolicyTopicConstraint.prototype.certificate_missing_in_country_list = null;
    
                            /**
                             * PolicyTopicConstraint certificate_domain_mismatch_in_country_list.
                             * @member {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList|null|undefined} certificate_domain_mismatch_in_country_list
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @instance
                             */
                            PolicyTopicConstraint.prototype.certificate_domain_mismatch_in_country_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PolicyTopicConstraint value.
                             * @member {"country_constraint_list"|"reseller_constraint"|"certificate_missing_in_country_list"|"certificate_domain_mismatch_in_country_list"|undefined} value
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @instance
                             */
                            Object.defineProperty(PolicyTopicConstraint.prototype, "value", {
                                get: $util.oneOfGetter($oneOfFields = ["country_constraint_list", "reseller_constraint", "certificate_missing_in_country_list", "certificate_domain_mismatch_in_country_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PolicyTopicConstraint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicConstraint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint} PolicyTopicConstraint instance
                             */
                            PolicyTopicConstraint.create = function create(properties) {
                                return new PolicyTopicConstraint(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyTopicConstraint message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicConstraint} m PolicyTopicConstraint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicConstraint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.country_constraint_list != null && Object.hasOwnProperty.call(m, "country_constraint_list"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.encode(m.country_constraint_list, w.uint32(10).fork()).ldelim();
                                if (m.reseller_constraint != null && Object.hasOwnProperty.call(m, "reseller_constraint"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint.encode(m.reseller_constraint, w.uint32(18).fork()).ldelim();
                                if (m.certificate_missing_in_country_list != null && Object.hasOwnProperty.call(m, "certificate_missing_in_country_list"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.encode(m.certificate_missing_in_country_list, w.uint32(26).fork()).ldelim();
                                if (m.certificate_domain_mismatch_in_country_list != null && Object.hasOwnProperty.call(m, "certificate_domain_mismatch_in_country_list"))
                                    $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.encode(m.certificate_domain_mismatch_in_country_list, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyTopicConstraint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicyTopicConstraint} message PolicyTopicConstraint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicConstraint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyTopicConstraint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint} PolicyTopicConstraint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicConstraint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicConstraint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.country_constraint_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.reseller_constraint = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.certificate_missing_in_country_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.certificate_domain_mismatch_in_country_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyTopicConstraint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint} PolicyTopicConstraint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicConstraint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyTopicConstraint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyTopicConstraint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.country_constraint_list != null && m.hasOwnProperty("country_constraint_list")) {
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.verify(m.country_constraint_list);
                                        if (e)
                                            return "country_constraint_list." + e;
                                    }
                                }
                                if (m.reseller_constraint != null && m.hasOwnProperty("reseller_constraint")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint.verify(m.reseller_constraint);
                                        if (e)
                                            return "reseller_constraint." + e;
                                    }
                                }
                                if (m.certificate_missing_in_country_list != null && m.hasOwnProperty("certificate_missing_in_country_list")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.verify(m.certificate_missing_in_country_list);
                                        if (e)
                                            return "certificate_missing_in_country_list." + e;
                                    }
                                }
                                if (m.certificate_domain_mismatch_in_country_list != null && m.hasOwnProperty("certificate_domain_mismatch_in_country_list")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.verify(m.certificate_domain_mismatch_in_country_list);
                                        if (e)
                                            return "certificate_domain_mismatch_in_country_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicyTopicConstraint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint} PolicyTopicConstraint
                             */
                            PolicyTopicConstraint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicConstraint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PolicyTopicConstraint();
                                if (d.country_constraint_list != null) {
                                    if (typeof d.country_constraint_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicConstraint.country_constraint_list: object expected");
                                    m.country_constraint_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.fromObject(d.country_constraint_list);
                                }
                                if (d.reseller_constraint != null) {
                                    if (typeof d.reseller_constraint !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicConstraint.reseller_constraint: object expected");
                                    m.reseller_constraint = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint.fromObject(d.reseller_constraint);
                                }
                                if (d.certificate_missing_in_country_list != null) {
                                    if (typeof d.certificate_missing_in_country_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicConstraint.certificate_missing_in_country_list: object expected");
                                    m.certificate_missing_in_country_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.fromObject(d.certificate_missing_in_country_list);
                                }
                                if (d.certificate_domain_mismatch_in_country_list != null) {
                                    if (typeof d.certificate_domain_mismatch_in_country_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PolicyTopicConstraint.certificate_domain_mismatch_in_country_list: object expected");
                                    m.certificate_domain_mismatch_in_country_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.fromObject(d.certificate_domain_mismatch_in_country_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicyTopicConstraint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @static
                             * @param {google.ads.googleads.v11.common.PolicyTopicConstraint} m PolicyTopicConstraint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyTopicConstraint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.country_constraint_list != null && m.hasOwnProperty("country_constraint_list")) {
                                    d.country_constraint_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.toObject(m.country_constraint_list, o);
                                    if (o.oneofs)
                                        d.value = "country_constraint_list";
                                }
                                if (m.reseller_constraint != null && m.hasOwnProperty("reseller_constraint")) {
                                    d.reseller_constraint = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint.toObject(m.reseller_constraint, o);
                                    if (o.oneofs)
                                        d.value = "reseller_constraint";
                                }
                                if (m.certificate_missing_in_country_list != null && m.hasOwnProperty("certificate_missing_in_country_list")) {
                                    d.certificate_missing_in_country_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.toObject(m.certificate_missing_in_country_list, o);
                                    if (o.oneofs)
                                        d.value = "certificate_missing_in_country_list";
                                }
                                if (m.certificate_domain_mismatch_in_country_list != null && m.hasOwnProperty("certificate_domain_mismatch_in_country_list")) {
                                    d.certificate_domain_mismatch_in_country_list = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.toObject(m.certificate_domain_mismatch_in_country_list, o);
                                    if (o.oneofs)
                                        d.value = "certificate_domain_mismatch_in_country_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicyTopicConstraint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyTopicConstraint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            PolicyTopicConstraint.CountryConstraintList = (function() {
    
                                /**
                                 * Properties of a CountryConstraintList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                                 * @interface ICountryConstraintList
                                 * @property {number|null} [total_targeted_countries] CountryConstraintList total_targeted_countries
                                 * @property {Array.<google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraint>|null} [countries] CountryConstraintList countries
                                 */
    
                                /**
                                 * Constructs a new CountryConstraintList.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                                 * @classdesc Represents a CountryConstraintList.
                                 * @implements ICountryConstraintList
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList=} [p] Properties to set
                                 */
                                function CountryConstraintList(p) {
                                    this.countries = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CountryConstraintList total_targeted_countries.
                                 * @member {number|null|undefined} total_targeted_countries
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @instance
                                 */
                                CountryConstraintList.prototype.total_targeted_countries = null;
    
                                /**
                                 * CountryConstraintList countries.
                                 * @member {Array.<google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraint>} countries
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @instance
                                 */
                                CountryConstraintList.prototype.countries = $util.emptyArray;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * CountryConstraintList _total_targeted_countries.
                                 * @member {"total_targeted_countries"|undefined} _total_targeted_countries
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @instance
                                 */
                                Object.defineProperty(CountryConstraintList.prototype, "_total_targeted_countries", {
                                    get: $util.oneOfGetter($oneOfFields = ["total_targeted_countries"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new CountryConstraintList instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList} CountryConstraintList instance
                                 */
                                CountryConstraintList.create = function create(properties) {
                                    return new CountryConstraintList(properties);
                                };
    
                                /**
                                 * Encodes the specified CountryConstraintList message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList} m CountryConstraintList message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CountryConstraintList.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.countries != null && m.countries.length) {
                                        for (var i = 0; i < m.countries.length; ++i)
                                            $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint.encode(m.countries[i], w.uint32(18).fork()).ldelim();
                                    }
                                    if (m.total_targeted_countries != null && Object.hasOwnProperty.call(m, "total_targeted_countries"))
                                        w.uint32(24).int32(m.total_targeted_countries);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CountryConstraintList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraintList} message CountryConstraintList message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CountryConstraintList.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CountryConstraintList message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList} CountryConstraintList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CountryConstraintList.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 3:
                                            m.total_targeted_countries = r.int32();
                                            break;
                                        case 2:
                                            if (!(m.countries && m.countries.length))
                                                m.countries = [];
                                            m.countries.push($root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CountryConstraintList message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList} CountryConstraintList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CountryConstraintList.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CountryConstraintList message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CountryConstraintList.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.total_targeted_countries != null && m.hasOwnProperty("total_targeted_countries")) {
                                        p._total_targeted_countries = 1;
                                        if (!$util.isInteger(m.total_targeted_countries))
                                            return "total_targeted_countries: integer expected";
                                    }
                                    if (m.countries != null && m.hasOwnProperty("countries")) {
                                        if (!Array.isArray(m.countries))
                                            return "countries: array expected";
                                        for (var i = 0; i < m.countries.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint.verify(m.countries[i]);
                                                if (e)
                                                    return "countries." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CountryConstraintList message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList} CountryConstraintList
                                 */
                                CountryConstraintList.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList();
                                    if (d.total_targeted_countries != null) {
                                        m.total_targeted_countries = d.total_targeted_countries | 0;
                                    }
                                    if (d.countries) {
                                        if (!Array.isArray(d.countries))
                                            throw TypeError(".google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.countries: array expected");
                                        m.countries = [];
                                        for (var i = 0; i < d.countries.length; ++i) {
                                            if (typeof d.countries[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList.countries: object expected");
                                            m.countries[i] = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint.fromObject(d.countries[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CountryConstraintList message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList} m CountryConstraintList
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CountryConstraintList.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.countries = [];
                                    }
                                    if (m.countries && m.countries.length) {
                                        d.countries = [];
                                        for (var j = 0; j < m.countries.length; ++j) {
                                            d.countries[j] = $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint.toObject(m.countries[j], o);
                                        }
                                    }
                                    if (m.total_targeted_countries != null && m.hasOwnProperty("total_targeted_countries")) {
                                        d.total_targeted_countries = m.total_targeted_countries;
                                        if (o.oneofs)
                                            d._total_targeted_countries = "total_targeted_countries";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CountryConstraintList to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraintList
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CountryConstraintList.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CountryConstraintList;
                            })();
    
                            PolicyTopicConstraint.ResellerConstraint = (function() {
    
                                /**
                                 * Properties of a ResellerConstraint.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                                 * @interface IResellerConstraint
                                 */
    
                                /**
                                 * Constructs a new ResellerConstraint.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                                 * @classdesc Represents a ResellerConstraint.
                                 * @implements IResellerConstraint
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.IResellerConstraint=} [p] Properties to set
                                 */
                                function ResellerConstraint(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * Creates a new ResellerConstraint instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.IResellerConstraint=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint} ResellerConstraint instance
                                 */
                                ResellerConstraint.create = function create(properties) {
                                    return new ResellerConstraint(properties);
                                };
    
                                /**
                                 * Encodes the specified ResellerConstraint message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.IResellerConstraint} m ResellerConstraint message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResellerConstraint.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ResellerConstraint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.IResellerConstraint} message ResellerConstraint message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResellerConstraint.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ResellerConstraint message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint} ResellerConstraint
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResellerConstraint.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ResellerConstraint message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint} ResellerConstraint
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResellerConstraint.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ResellerConstraint message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ResellerConstraint.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    return null;
                                };
    
                                /**
                                 * Creates a ResellerConstraint message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint} ResellerConstraint
                                 */
                                ResellerConstraint.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint)
                                        return d;
                                    return new $root.google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint();
                                };
    
                                /**
                                 * Creates a plain object from a ResellerConstraint message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint} m ResellerConstraint
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ResellerConstraint.toObject = function toObject() {
                                    return {};
                                };
    
                                /**
                                 * Converts this ResellerConstraint to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.ResellerConstraint
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ResellerConstraint.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ResellerConstraint;
                            })();
    
                            PolicyTopicConstraint.CountryConstraint = (function() {
    
                                /**
                                 * Properties of a CountryConstraint.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                                 * @interface ICountryConstraint
                                 * @property {string|null} [country_criterion] CountryConstraint country_criterion
                                 */
    
                                /**
                                 * Constructs a new CountryConstraint.
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint
                                 * @classdesc Represents a CountryConstraint.
                                 * @implements ICountryConstraint
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraint=} [p] Properties to set
                                 */
                                function CountryConstraint(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CountryConstraint country_criterion.
                                 * @member {string|null|undefined} country_criterion
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @instance
                                 */
                                CountryConstraint.prototype.country_criterion = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * CountryConstraint _country_criterion.
                                 * @member {"country_criterion"|undefined} _country_criterion
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @instance
                                 */
                                Object.defineProperty(CountryConstraint.prototype, "_country_criterion", {
                                    get: $util.oneOfGetter($oneOfFields = ["country_criterion"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new CountryConstraint instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraint=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint} CountryConstraint instance
                                 */
                                CountryConstraint.create = function create(properties) {
                                    return new CountryConstraint(properties);
                                };
    
                                /**
                                 * Encodes the specified CountryConstraint message. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraint} m CountryConstraint message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CountryConstraint.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.country_criterion != null && Object.hasOwnProperty.call(m, "country_criterion"))
                                        w.uint32(18).string(m.country_criterion);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CountryConstraint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.ICountryConstraint} message CountryConstraint message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CountryConstraint.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CountryConstraint message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint} CountryConstraint
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CountryConstraint.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.country_criterion = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CountryConstraint message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint} CountryConstraint
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CountryConstraint.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CountryConstraint message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CountryConstraint.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.country_criterion != null && m.hasOwnProperty("country_criterion")) {
                                        p._country_criterion = 1;
                                        if (!$util.isString(m.country_criterion))
                                            return "country_criterion: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CountryConstraint message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint} CountryConstraint
                                 */
                                CountryConstraint.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint();
                                    if (d.country_criterion != null) {
                                        m.country_criterion = String(d.country_criterion);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CountryConstraint message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @static
                                 * @param {google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint} m CountryConstraint
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CountryConstraint.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.country_criterion != null && m.hasOwnProperty("country_criterion")) {
                                        d.country_criterion = m.country_criterion;
                                        if (o.oneofs)
                                            d._country_criterion = "country_criterion";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CountryConstraint to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.PolicyTopicConstraint.CountryConstraint
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CountryConstraint.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CountryConstraint;
                            })();
    
                            return PolicyTopicConstraint;
                        })();
    
                        common.TextAdInfo = (function() {
    
                            /**
                             * Properties of a TextAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITextAdInfo
                             * @property {string|null} [headline] TextAdInfo headline
                             * @property {string|null} [description1] TextAdInfo description1
                             * @property {string|null} [description2] TextAdInfo description2
                             */
    
                            /**
                             * Constructs a new TextAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TextAdInfo.
                             * @implements ITextAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITextAdInfo=} [p] Properties to set
                             */
                            function TextAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TextAdInfo headline.
                             * @member {string|null|undefined} headline
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @instance
                             */
                            TextAdInfo.prototype.headline = null;
    
                            /**
                             * TextAdInfo description1.
                             * @member {string|null|undefined} description1
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @instance
                             */
                            TextAdInfo.prototype.description1 = null;
    
                            /**
                             * TextAdInfo description2.
                             * @member {string|null|undefined} description2
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @instance
                             */
                            TextAdInfo.prototype.description2 = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TextAdInfo _headline.
                             * @member {"headline"|undefined} _headline
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @instance
                             */
                            Object.defineProperty(TextAdInfo.prototype, "_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TextAdInfo _description1.
                             * @member {"description1"|undefined} _description1
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @instance
                             */
                            Object.defineProperty(TextAdInfo.prototype, "_description1", {
                                get: $util.oneOfGetter($oneOfFields = ["description1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TextAdInfo _description2.
                             * @member {"description2"|undefined} _description2
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @instance
                             */
                            Object.defineProperty(TextAdInfo.prototype, "_description2", {
                                get: $util.oneOfGetter($oneOfFields = ["description2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TextAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TextAdInfo} TextAdInfo instance
                             */
                            TextAdInfo.create = function create(properties) {
                                return new TextAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified TextAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.TextAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextAdInfo} m TextAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(34).string(m.headline);
                                if (m.description1 != null && Object.hasOwnProperty.call(m, "description1"))
                                    w.uint32(42).string(m.description1);
                                if (m.description2 != null && Object.hasOwnProperty.call(m, "description2"))
                                    w.uint32(50).string(m.description2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TextAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TextAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextAdInfo} message TextAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TextAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TextAdInfo} TextAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TextAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.headline = r.string();
                                        break;
                                    case 5:
                                        m.description1 = r.string();
                                        break;
                                    case 6:
                                        m.description2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TextAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TextAdInfo} TextAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TextAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TextAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    p._headline = 1;
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    p._description1 = 1;
                                    if (!$util.isString(m.description1))
                                        return "description1: string expected";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    p._description2 = 1;
                                    if (!$util.isString(m.description2))
                                        return "description2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TextAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TextAdInfo} TextAdInfo
                             */
                            TextAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TextAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TextAdInfo();
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.description1 != null) {
                                    m.description1 = String(d.description1);
                                }
                                if (d.description2 != null) {
                                    m.description2 = String(d.description2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TextAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.TextAdInfo} m TextAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TextAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                    if (o.oneofs)
                                        d._headline = "headline";
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    d.description1 = m.description1;
                                    if (o.oneofs)
                                        d._description1 = "description1";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    d.description2 = m.description2;
                                    if (o.oneofs)
                                        d._description2 = "description2";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TextAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TextAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TextAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TextAdInfo;
                        })();
    
                        common.ExpandedTextAdInfo = (function() {
    
                            /**
                             * Properties of an ExpandedTextAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IExpandedTextAdInfo
                             * @property {string|null} [headline_part1] ExpandedTextAdInfo headline_part1
                             * @property {string|null} [headline_part2] ExpandedTextAdInfo headline_part2
                             * @property {string|null} [headline_part3] ExpandedTextAdInfo headline_part3
                             * @property {string|null} [description] ExpandedTextAdInfo description
                             * @property {string|null} [description2] ExpandedTextAdInfo description2
                             * @property {string|null} [path1] ExpandedTextAdInfo path1
                             * @property {string|null} [path2] ExpandedTextAdInfo path2
                             */
    
                            /**
                             * Constructs a new ExpandedTextAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ExpandedTextAdInfo.
                             * @implements IExpandedTextAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IExpandedTextAdInfo=} [p] Properties to set
                             */
                            function ExpandedTextAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExpandedTextAdInfo headline_part1.
                             * @member {string|null|undefined} headline_part1
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            ExpandedTextAdInfo.prototype.headline_part1 = null;
    
                            /**
                             * ExpandedTextAdInfo headline_part2.
                             * @member {string|null|undefined} headline_part2
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            ExpandedTextAdInfo.prototype.headline_part2 = null;
    
                            /**
                             * ExpandedTextAdInfo headline_part3.
                             * @member {string|null|undefined} headline_part3
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            ExpandedTextAdInfo.prototype.headline_part3 = null;
    
                            /**
                             * ExpandedTextAdInfo description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            ExpandedTextAdInfo.prototype.description = null;
    
                            /**
                             * ExpandedTextAdInfo description2.
                             * @member {string|null|undefined} description2
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            ExpandedTextAdInfo.prototype.description2 = null;
    
                            /**
                             * ExpandedTextAdInfo path1.
                             * @member {string|null|undefined} path1
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            ExpandedTextAdInfo.prototype.path1 = null;
    
                            /**
                             * ExpandedTextAdInfo path2.
                             * @member {string|null|undefined} path2
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            ExpandedTextAdInfo.prototype.path2 = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExpandedTextAdInfo _headline_part1.
                             * @member {"headline_part1"|undefined} _headline_part1
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedTextAdInfo.prototype, "_headline_part1", {
                                get: $util.oneOfGetter($oneOfFields = ["headline_part1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExpandedTextAdInfo _headline_part2.
                             * @member {"headline_part2"|undefined} _headline_part2
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedTextAdInfo.prototype, "_headline_part2", {
                                get: $util.oneOfGetter($oneOfFields = ["headline_part2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExpandedTextAdInfo _headline_part3.
                             * @member {"headline_part3"|undefined} _headline_part3
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedTextAdInfo.prototype, "_headline_part3", {
                                get: $util.oneOfGetter($oneOfFields = ["headline_part3"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExpandedTextAdInfo _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedTextAdInfo.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExpandedTextAdInfo _description2.
                             * @member {"description2"|undefined} _description2
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedTextAdInfo.prototype, "_description2", {
                                get: $util.oneOfGetter($oneOfFields = ["description2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExpandedTextAdInfo _path1.
                             * @member {"path1"|undefined} _path1
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedTextAdInfo.prototype, "_path1", {
                                get: $util.oneOfGetter($oneOfFields = ["path1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExpandedTextAdInfo _path2.
                             * @member {"path2"|undefined} _path2
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedTextAdInfo.prototype, "_path2", {
                                get: $util.oneOfGetter($oneOfFields = ["path2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExpandedTextAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpandedTextAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ExpandedTextAdInfo} ExpandedTextAdInfo instance
                             */
                            ExpandedTextAdInfo.create = function create(properties) {
                                return new ExpandedTextAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ExpandedTextAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ExpandedTextAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpandedTextAdInfo} m ExpandedTextAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpandedTextAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headline_part1 != null && Object.hasOwnProperty.call(m, "headline_part1"))
                                    w.uint32(66).string(m.headline_part1);
                                if (m.headline_part2 != null && Object.hasOwnProperty.call(m, "headline_part2"))
                                    w.uint32(74).string(m.headline_part2);
                                if (m.headline_part3 != null && Object.hasOwnProperty.call(m, "headline_part3"))
                                    w.uint32(82).string(m.headline_part3);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(90).string(m.description);
                                if (m.description2 != null && Object.hasOwnProperty.call(m, "description2"))
                                    w.uint32(98).string(m.description2);
                                if (m.path1 != null && Object.hasOwnProperty.call(m, "path1"))
                                    w.uint32(106).string(m.path1);
                                if (m.path2 != null && Object.hasOwnProperty.call(m, "path2"))
                                    w.uint32(114).string(m.path2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExpandedTextAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ExpandedTextAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpandedTextAdInfo} message ExpandedTextAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpandedTextAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExpandedTextAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ExpandedTextAdInfo} ExpandedTextAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpandedTextAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ExpandedTextAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 8:
                                        m.headline_part1 = r.string();
                                        break;
                                    case 9:
                                        m.headline_part2 = r.string();
                                        break;
                                    case 10:
                                        m.headline_part3 = r.string();
                                        break;
                                    case 11:
                                        m.description = r.string();
                                        break;
                                    case 12:
                                        m.description2 = r.string();
                                        break;
                                    case 13:
                                        m.path1 = r.string();
                                        break;
                                    case 14:
                                        m.path2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExpandedTextAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ExpandedTextAdInfo} ExpandedTextAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpandedTextAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExpandedTextAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExpandedTextAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.headline_part1 != null && m.hasOwnProperty("headline_part1")) {
                                    p._headline_part1 = 1;
                                    if (!$util.isString(m.headline_part1))
                                        return "headline_part1: string expected";
                                }
                                if (m.headline_part2 != null && m.hasOwnProperty("headline_part2")) {
                                    p._headline_part2 = 1;
                                    if (!$util.isString(m.headline_part2))
                                        return "headline_part2: string expected";
                                }
                                if (m.headline_part3 != null && m.hasOwnProperty("headline_part3")) {
                                    p._headline_part3 = 1;
                                    if (!$util.isString(m.headline_part3))
                                        return "headline_part3: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    p._description2 = 1;
                                    if (!$util.isString(m.description2))
                                        return "description2: string expected";
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    p._path1 = 1;
                                    if (!$util.isString(m.path1))
                                        return "path1: string expected";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    p._path2 = 1;
                                    if (!$util.isString(m.path2))
                                        return "path2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExpandedTextAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ExpandedTextAdInfo} ExpandedTextAdInfo
                             */
                            ExpandedTextAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ExpandedTextAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ExpandedTextAdInfo();
                                if (d.headline_part1 != null) {
                                    m.headline_part1 = String(d.headline_part1);
                                }
                                if (d.headline_part2 != null) {
                                    m.headline_part2 = String(d.headline_part2);
                                }
                                if (d.headline_part3 != null) {
                                    m.headline_part3 = String(d.headline_part3);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.description2 != null) {
                                    m.description2 = String(d.description2);
                                }
                                if (d.path1 != null) {
                                    m.path1 = String(d.path1);
                                }
                                if (d.path2 != null) {
                                    m.path2 = String(d.path2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExpandedTextAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ExpandedTextAdInfo} m ExpandedTextAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExpandedTextAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.headline_part1 != null && m.hasOwnProperty("headline_part1")) {
                                    d.headline_part1 = m.headline_part1;
                                    if (o.oneofs)
                                        d._headline_part1 = "headline_part1";
                                }
                                if (m.headline_part2 != null && m.hasOwnProperty("headline_part2")) {
                                    d.headline_part2 = m.headline_part2;
                                    if (o.oneofs)
                                        d._headline_part2 = "headline_part2";
                                }
                                if (m.headline_part3 != null && m.hasOwnProperty("headline_part3")) {
                                    d.headline_part3 = m.headline_part3;
                                    if (o.oneofs)
                                        d._headline_part3 = "headline_part3";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    d.description2 = m.description2;
                                    if (o.oneofs)
                                        d._description2 = "description2";
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    d.path1 = m.path1;
                                    if (o.oneofs)
                                        d._path1 = "path1";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    d.path2 = m.path2;
                                    if (o.oneofs)
                                        d._path2 = "path2";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExpandedTextAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ExpandedTextAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExpandedTextAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExpandedTextAdInfo;
                        })();
    
                        common.ExpandedDynamicSearchAdInfo = (function() {
    
                            /**
                             * Properties of an ExpandedDynamicSearchAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IExpandedDynamicSearchAdInfo
                             * @property {string|null} [description] ExpandedDynamicSearchAdInfo description
                             * @property {string|null} [description2] ExpandedDynamicSearchAdInfo description2
                             */
    
                            /**
                             * Constructs a new ExpandedDynamicSearchAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ExpandedDynamicSearchAdInfo.
                             * @implements IExpandedDynamicSearchAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IExpandedDynamicSearchAdInfo=} [p] Properties to set
                             */
                            function ExpandedDynamicSearchAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExpandedDynamicSearchAdInfo description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @instance
                             */
                            ExpandedDynamicSearchAdInfo.prototype.description = null;
    
                            /**
                             * ExpandedDynamicSearchAdInfo description2.
                             * @member {string|null|undefined} description2
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @instance
                             */
                            ExpandedDynamicSearchAdInfo.prototype.description2 = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExpandedDynamicSearchAdInfo _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedDynamicSearchAdInfo.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExpandedDynamicSearchAdInfo _description2.
                             * @member {"description2"|undefined} _description2
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @instance
                             */
                            Object.defineProperty(ExpandedDynamicSearchAdInfo.prototype, "_description2", {
                                get: $util.oneOfGetter($oneOfFields = ["description2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExpandedDynamicSearchAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpandedDynamicSearchAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo} ExpandedDynamicSearchAdInfo instance
                             */
                            ExpandedDynamicSearchAdInfo.create = function create(properties) {
                                return new ExpandedDynamicSearchAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ExpandedDynamicSearchAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpandedDynamicSearchAdInfo} m ExpandedDynamicSearchAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpandedDynamicSearchAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(26).string(m.description);
                                if (m.description2 != null && Object.hasOwnProperty.call(m, "description2"))
                                    w.uint32(34).string(m.description2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExpandedDynamicSearchAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpandedDynamicSearchAdInfo} message ExpandedDynamicSearchAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpandedDynamicSearchAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExpandedDynamicSearchAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo} ExpandedDynamicSearchAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpandedDynamicSearchAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.description = r.string();
                                        break;
                                    case 4:
                                        m.description2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExpandedDynamicSearchAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo} ExpandedDynamicSearchAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpandedDynamicSearchAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExpandedDynamicSearchAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExpandedDynamicSearchAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    p._description2 = 1;
                                    if (!$util.isString(m.description2))
                                        return "description2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExpandedDynamicSearchAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo} ExpandedDynamicSearchAdInfo
                             */
                            ExpandedDynamicSearchAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo();
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.description2 != null) {
                                    m.description2 = String(d.description2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExpandedDynamicSearchAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo} m ExpandedDynamicSearchAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExpandedDynamicSearchAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    d.description2 = m.description2;
                                    if (o.oneofs)
                                        d._description2 = "description2";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExpandedDynamicSearchAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExpandedDynamicSearchAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExpandedDynamicSearchAdInfo;
                        })();
    
                        common.HotelAdInfo = (function() {
    
                            /**
                             * Properties of a HotelAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelAdInfo
                             */
    
                            /**
                             * Constructs a new HotelAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelAdInfo.
                             * @implements IHotelAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelAdInfo=} [p] Properties to set
                             */
                            function HotelAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new HotelAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelAdInfo} HotelAdInfo instance
                             */
                            HotelAdInfo.create = function create(properties) {
                                return new HotelAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelAdInfo} m HotelAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelAdInfo} message HotelAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelAdInfo} HotelAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelAdInfo} HotelAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a HotelAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelAdInfo} HotelAdInfo
                             */
                            HotelAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelAdInfo)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.HotelAdInfo();
                            };
    
                            /**
                             * Creates a plain object from a HotelAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelAdInfo} m HotelAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelAdInfo.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this HotelAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelAdInfo;
                        })();
    
                        common.ShoppingSmartAdInfo = (function() {
    
                            /**
                             * Properties of a ShoppingSmartAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IShoppingSmartAdInfo
                             */
    
                            /**
                             * Constructs a new ShoppingSmartAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ShoppingSmartAdInfo.
                             * @implements IShoppingSmartAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IShoppingSmartAdInfo=} [p] Properties to set
                             */
                            function ShoppingSmartAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ShoppingSmartAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingSmartAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ShoppingSmartAdInfo} ShoppingSmartAdInfo instance
                             */
                            ShoppingSmartAdInfo.create = function create(properties) {
                                return new ShoppingSmartAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ShoppingSmartAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingSmartAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingSmartAdInfo} m ShoppingSmartAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingSmartAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ShoppingSmartAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingSmartAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingSmartAdInfo} message ShoppingSmartAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingSmartAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ShoppingSmartAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ShoppingSmartAdInfo} ShoppingSmartAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingSmartAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ShoppingSmartAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ShoppingSmartAdInfo} ShoppingSmartAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingSmartAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ShoppingSmartAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ShoppingSmartAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ShoppingSmartAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ShoppingSmartAdInfo} ShoppingSmartAdInfo
                             */
                            ShoppingSmartAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo();
                            };
    
                            /**
                             * Creates a plain object from a ShoppingSmartAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ShoppingSmartAdInfo} m ShoppingSmartAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ShoppingSmartAdInfo.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ShoppingSmartAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ShoppingSmartAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ShoppingSmartAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ShoppingSmartAdInfo;
                        })();
    
                        common.ShoppingProductAdInfo = (function() {
    
                            /**
                             * Properties of a ShoppingProductAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IShoppingProductAdInfo
                             */
    
                            /**
                             * Constructs a new ShoppingProductAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ShoppingProductAdInfo.
                             * @implements IShoppingProductAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IShoppingProductAdInfo=} [p] Properties to set
                             */
                            function ShoppingProductAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ShoppingProductAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingProductAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ShoppingProductAdInfo} ShoppingProductAdInfo instance
                             */
                            ShoppingProductAdInfo.create = function create(properties) {
                                return new ShoppingProductAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ShoppingProductAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingProductAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingProductAdInfo} m ShoppingProductAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingProductAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ShoppingProductAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingProductAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingProductAdInfo} message ShoppingProductAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingProductAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ShoppingProductAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ShoppingProductAdInfo} ShoppingProductAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingProductAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ShoppingProductAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ShoppingProductAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ShoppingProductAdInfo} ShoppingProductAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingProductAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ShoppingProductAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ShoppingProductAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ShoppingProductAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ShoppingProductAdInfo} ShoppingProductAdInfo
                             */
                            ShoppingProductAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ShoppingProductAdInfo)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.ShoppingProductAdInfo();
                            };
    
                            /**
                             * Creates a plain object from a ShoppingProductAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ShoppingProductAdInfo} m ShoppingProductAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ShoppingProductAdInfo.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ShoppingProductAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ShoppingProductAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ShoppingProductAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ShoppingProductAdInfo;
                        })();
    
                        common.ShoppingComparisonListingAdInfo = (function() {
    
                            /**
                             * Properties of a ShoppingComparisonListingAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IShoppingComparisonListingAdInfo
                             * @property {string|null} [headline] ShoppingComparisonListingAdInfo headline
                             */
    
                            /**
                             * Constructs a new ShoppingComparisonListingAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ShoppingComparisonListingAdInfo.
                             * @implements IShoppingComparisonListingAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IShoppingComparisonListingAdInfo=} [p] Properties to set
                             */
                            function ShoppingComparisonListingAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ShoppingComparisonListingAdInfo headline.
                             * @member {string|null|undefined} headline
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @instance
                             */
                            ShoppingComparisonListingAdInfo.prototype.headline = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ShoppingComparisonListingAdInfo _headline.
                             * @member {"headline"|undefined} _headline
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @instance
                             */
                            Object.defineProperty(ShoppingComparisonListingAdInfo.prototype, "_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ShoppingComparisonListingAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingComparisonListingAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo} ShoppingComparisonListingAdInfo instance
                             */
                            ShoppingComparisonListingAdInfo.create = function create(properties) {
                                return new ShoppingComparisonListingAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ShoppingComparisonListingAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingComparisonListingAdInfo} m ShoppingComparisonListingAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingComparisonListingAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(18).string(m.headline);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ShoppingComparisonListingAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingComparisonListingAdInfo} message ShoppingComparisonListingAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingComparisonListingAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ShoppingComparisonListingAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo} ShoppingComparisonListingAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingComparisonListingAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.headline = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ShoppingComparisonListingAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo} ShoppingComparisonListingAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingComparisonListingAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ShoppingComparisonListingAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ShoppingComparisonListingAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    p._headline = 1;
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ShoppingComparisonListingAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo} ShoppingComparisonListingAdInfo
                             */
                            ShoppingComparisonListingAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo();
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ShoppingComparisonListingAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo} m ShoppingComparisonListingAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ShoppingComparisonListingAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                    if (o.oneofs)
                                        d._headline = "headline";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ShoppingComparisonListingAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ShoppingComparisonListingAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ShoppingComparisonListingAdInfo;
                        })();
    
                        common.GmailAdInfo = (function() {
    
                            /**
                             * Properties of a GmailAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IGmailAdInfo
                             * @property {google.ads.googleads.v11.common.IGmailTeaser|null} [teaser] GmailAdInfo teaser
                             * @property {string|null} [header_image] GmailAdInfo header_image
                             * @property {string|null} [marketing_image] GmailAdInfo marketing_image
                             * @property {string|null} [marketing_image_headline] GmailAdInfo marketing_image_headline
                             * @property {string|null} [marketing_image_description] GmailAdInfo marketing_image_description
                             * @property {google.ads.googleads.v11.common.IDisplayCallToAction|null} [marketing_image_display_call_to_action] GmailAdInfo marketing_image_display_call_to_action
                             * @property {Array.<google.ads.googleads.v11.common.IProductImage>|null} [product_images] GmailAdInfo product_images
                             * @property {Array.<google.ads.googleads.v11.common.IProductVideo>|null} [product_videos] GmailAdInfo product_videos
                             */
    
                            /**
                             * Constructs a new GmailAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a GmailAdInfo.
                             * @implements IGmailAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IGmailAdInfo=} [p] Properties to set
                             */
                            function GmailAdInfo(p) {
                                this.product_images = [];
                                this.product_videos = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GmailAdInfo teaser.
                             * @member {google.ads.googleads.v11.common.IGmailTeaser|null|undefined} teaser
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.teaser = null;
    
                            /**
                             * GmailAdInfo header_image.
                             * @member {string|null|undefined} header_image
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.header_image = null;
    
                            /**
                             * GmailAdInfo marketing_image.
                             * @member {string|null|undefined} marketing_image
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.marketing_image = null;
    
                            /**
                             * GmailAdInfo marketing_image_headline.
                             * @member {string|null|undefined} marketing_image_headline
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.marketing_image_headline = null;
    
                            /**
                             * GmailAdInfo marketing_image_description.
                             * @member {string|null|undefined} marketing_image_description
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.marketing_image_description = null;
    
                            /**
                             * GmailAdInfo marketing_image_display_call_to_action.
                             * @member {google.ads.googleads.v11.common.IDisplayCallToAction|null|undefined} marketing_image_display_call_to_action
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.marketing_image_display_call_to_action = null;
    
                            /**
                             * GmailAdInfo product_images.
                             * @member {Array.<google.ads.googleads.v11.common.IProductImage>} product_images
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.product_images = $util.emptyArray;
    
                            /**
                             * GmailAdInfo product_videos.
                             * @member {Array.<google.ads.googleads.v11.common.IProductVideo>} product_videos
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            GmailAdInfo.prototype.product_videos = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GmailAdInfo _header_image.
                             * @member {"header_image"|undefined} _header_image
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            Object.defineProperty(GmailAdInfo.prototype, "_header_image", {
                                get: $util.oneOfGetter($oneOfFields = ["header_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GmailAdInfo _marketing_image.
                             * @member {"marketing_image"|undefined} _marketing_image
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            Object.defineProperty(GmailAdInfo.prototype, "_marketing_image", {
                                get: $util.oneOfGetter($oneOfFields = ["marketing_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GmailAdInfo _marketing_image_headline.
                             * @member {"marketing_image_headline"|undefined} _marketing_image_headline
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            Object.defineProperty(GmailAdInfo.prototype, "_marketing_image_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["marketing_image_headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GmailAdInfo _marketing_image_description.
                             * @member {"marketing_image_description"|undefined} _marketing_image_description
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             */
                            Object.defineProperty(GmailAdInfo.prototype, "_marketing_image_description", {
                                get: $util.oneOfGetter($oneOfFields = ["marketing_image_description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GmailAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGmailAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.GmailAdInfo} GmailAdInfo instance
                             */
                            GmailAdInfo.create = function create(properties) {
                                return new GmailAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified GmailAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.GmailAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGmailAdInfo} m GmailAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GmailAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.teaser != null && Object.hasOwnProperty.call(m, "teaser"))
                                    $root.google.ads.googleads.v11.common.GmailTeaser.encode(m.teaser, w.uint32(10).fork()).ldelim();
                                if (m.marketing_image_display_call_to_action != null && Object.hasOwnProperty.call(m, "marketing_image_display_call_to_action"))
                                    $root.google.ads.googleads.v11.common.DisplayCallToAction.encode(m.marketing_image_display_call_to_action, w.uint32(50).fork()).ldelim();
                                if (m.product_images != null && m.product_images.length) {
                                    for (var i = 0; i < m.product_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.ProductImage.encode(m.product_images[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.product_videos != null && m.product_videos.length) {
                                    for (var i = 0; i < m.product_videos.length; ++i)
                                        $root.google.ads.googleads.v11.common.ProductVideo.encode(m.product_videos[i], w.uint32(66).fork()).ldelim();
                                }
                                if (m.header_image != null && Object.hasOwnProperty.call(m, "header_image"))
                                    w.uint32(82).string(m.header_image);
                                if (m.marketing_image != null && Object.hasOwnProperty.call(m, "marketing_image"))
                                    w.uint32(90).string(m.marketing_image);
                                if (m.marketing_image_headline != null && Object.hasOwnProperty.call(m, "marketing_image_headline"))
                                    w.uint32(98).string(m.marketing_image_headline);
                                if (m.marketing_image_description != null && Object.hasOwnProperty.call(m, "marketing_image_description"))
                                    w.uint32(106).string(m.marketing_image_description);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GmailAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.GmailAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGmailAdInfo} message GmailAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GmailAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GmailAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.GmailAdInfo} GmailAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GmailAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.GmailAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.teaser = $root.google.ads.googleads.v11.common.GmailTeaser.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.header_image = r.string();
                                        break;
                                    case 11:
                                        m.marketing_image = r.string();
                                        break;
                                    case 12:
                                        m.marketing_image_headline = r.string();
                                        break;
                                    case 13:
                                        m.marketing_image_description = r.string();
                                        break;
                                    case 6:
                                        m.marketing_image_display_call_to_action = $root.google.ads.googleads.v11.common.DisplayCallToAction.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        if (!(m.product_images && m.product_images.length))
                                            m.product_images = [];
                                        m.product_images.push($root.google.ads.googleads.v11.common.ProductImage.decode(r, r.uint32()));
                                        break;
                                    case 8:
                                        if (!(m.product_videos && m.product_videos.length))
                                            m.product_videos = [];
                                        m.product_videos.push($root.google.ads.googleads.v11.common.ProductVideo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GmailAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.GmailAdInfo} GmailAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GmailAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GmailAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GmailAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.teaser != null && m.hasOwnProperty("teaser")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GmailTeaser.verify(m.teaser);
                                        if (e)
                                            return "teaser." + e;
                                    }
                                }
                                if (m.header_image != null && m.hasOwnProperty("header_image")) {
                                    p._header_image = 1;
                                    if (!$util.isString(m.header_image))
                                        return "header_image: string expected";
                                }
                                if (m.marketing_image != null && m.hasOwnProperty("marketing_image")) {
                                    p._marketing_image = 1;
                                    if (!$util.isString(m.marketing_image))
                                        return "marketing_image: string expected";
                                }
                                if (m.marketing_image_headline != null && m.hasOwnProperty("marketing_image_headline")) {
                                    p._marketing_image_headline = 1;
                                    if (!$util.isString(m.marketing_image_headline))
                                        return "marketing_image_headline: string expected";
                                }
                                if (m.marketing_image_description != null && m.hasOwnProperty("marketing_image_description")) {
                                    p._marketing_image_description = 1;
                                    if (!$util.isString(m.marketing_image_description))
                                        return "marketing_image_description: string expected";
                                }
                                if (m.marketing_image_display_call_to_action != null && m.hasOwnProperty("marketing_image_display_call_to_action")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DisplayCallToAction.verify(m.marketing_image_display_call_to_action);
                                        if (e)
                                            return "marketing_image_display_call_to_action." + e;
                                    }
                                }
                                if (m.product_images != null && m.hasOwnProperty("product_images")) {
                                    if (!Array.isArray(m.product_images))
                                        return "product_images: array expected";
                                    for (var i = 0; i < m.product_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.ProductImage.verify(m.product_images[i]);
                                            if (e)
                                                return "product_images." + e;
                                        }
                                    }
                                }
                                if (m.product_videos != null && m.hasOwnProperty("product_videos")) {
                                    if (!Array.isArray(m.product_videos))
                                        return "product_videos: array expected";
                                    for (var i = 0; i < m.product_videos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.ProductVideo.verify(m.product_videos[i]);
                                            if (e)
                                                return "product_videos." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GmailAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.GmailAdInfo} GmailAdInfo
                             */
                            GmailAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.GmailAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.GmailAdInfo();
                                if (d.teaser != null) {
                                    if (typeof d.teaser !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.GmailAdInfo.teaser: object expected");
                                    m.teaser = $root.google.ads.googleads.v11.common.GmailTeaser.fromObject(d.teaser);
                                }
                                if (d.header_image != null) {
                                    m.header_image = String(d.header_image);
                                }
                                if (d.marketing_image != null) {
                                    m.marketing_image = String(d.marketing_image);
                                }
                                if (d.marketing_image_headline != null) {
                                    m.marketing_image_headline = String(d.marketing_image_headline);
                                }
                                if (d.marketing_image_description != null) {
                                    m.marketing_image_description = String(d.marketing_image_description);
                                }
                                if (d.marketing_image_display_call_to_action != null) {
                                    if (typeof d.marketing_image_display_call_to_action !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.GmailAdInfo.marketing_image_display_call_to_action: object expected");
                                    m.marketing_image_display_call_to_action = $root.google.ads.googleads.v11.common.DisplayCallToAction.fromObject(d.marketing_image_display_call_to_action);
                                }
                                if (d.product_images) {
                                    if (!Array.isArray(d.product_images))
                                        throw TypeError(".google.ads.googleads.v11.common.GmailAdInfo.product_images: array expected");
                                    m.product_images = [];
                                    for (var i = 0; i < d.product_images.length; ++i) {
                                        if (typeof d.product_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.GmailAdInfo.product_images: object expected");
                                        m.product_images[i] = $root.google.ads.googleads.v11.common.ProductImage.fromObject(d.product_images[i]);
                                    }
                                }
                                if (d.product_videos) {
                                    if (!Array.isArray(d.product_videos))
                                        throw TypeError(".google.ads.googleads.v11.common.GmailAdInfo.product_videos: array expected");
                                    m.product_videos = [];
                                    for (var i = 0; i < d.product_videos.length; ++i) {
                                        if (typeof d.product_videos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.GmailAdInfo.product_videos: object expected");
                                        m.product_videos[i] = $root.google.ads.googleads.v11.common.ProductVideo.fromObject(d.product_videos[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GmailAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.GmailAdInfo} m GmailAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GmailAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.product_images = [];
                                    d.product_videos = [];
                                }
                                if (o.defaults) {
                                    d.teaser = null;
                                    d.marketing_image_display_call_to_action = null;
                                }
                                if (m.teaser != null && m.hasOwnProperty("teaser")) {
                                    d.teaser = $root.google.ads.googleads.v11.common.GmailTeaser.toObject(m.teaser, o);
                                }
                                if (m.marketing_image_display_call_to_action != null && m.hasOwnProperty("marketing_image_display_call_to_action")) {
                                    d.marketing_image_display_call_to_action = $root.google.ads.googleads.v11.common.DisplayCallToAction.toObject(m.marketing_image_display_call_to_action, o);
                                }
                                if (m.product_images && m.product_images.length) {
                                    d.product_images = [];
                                    for (var j = 0; j < m.product_images.length; ++j) {
                                        d.product_images[j] = $root.google.ads.googleads.v11.common.ProductImage.toObject(m.product_images[j], o);
                                    }
                                }
                                if (m.product_videos && m.product_videos.length) {
                                    d.product_videos = [];
                                    for (var j = 0; j < m.product_videos.length; ++j) {
                                        d.product_videos[j] = $root.google.ads.googleads.v11.common.ProductVideo.toObject(m.product_videos[j], o);
                                    }
                                }
                                if (m.header_image != null && m.hasOwnProperty("header_image")) {
                                    d.header_image = m.header_image;
                                    if (o.oneofs)
                                        d._header_image = "header_image";
                                }
                                if (m.marketing_image != null && m.hasOwnProperty("marketing_image")) {
                                    d.marketing_image = m.marketing_image;
                                    if (o.oneofs)
                                        d._marketing_image = "marketing_image";
                                }
                                if (m.marketing_image_headline != null && m.hasOwnProperty("marketing_image_headline")) {
                                    d.marketing_image_headline = m.marketing_image_headline;
                                    if (o.oneofs)
                                        d._marketing_image_headline = "marketing_image_headline";
                                }
                                if (m.marketing_image_description != null && m.hasOwnProperty("marketing_image_description")) {
                                    d.marketing_image_description = m.marketing_image_description;
                                    if (o.oneofs)
                                        d._marketing_image_description = "marketing_image_description";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GmailAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.GmailAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GmailAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GmailAdInfo;
                        })();
    
                        common.GmailTeaser = (function() {
    
                            /**
                             * Properties of a GmailTeaser.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IGmailTeaser
                             * @property {string|null} [headline] GmailTeaser headline
                             * @property {string|null} [description] GmailTeaser description
                             * @property {string|null} [business_name] GmailTeaser business_name
                             * @property {string|null} [logo_image] GmailTeaser logo_image
                             */
    
                            /**
                             * Constructs a new GmailTeaser.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a GmailTeaser.
                             * @implements IGmailTeaser
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IGmailTeaser=} [p] Properties to set
                             */
                            function GmailTeaser(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GmailTeaser headline.
                             * @member {string|null|undefined} headline
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            GmailTeaser.prototype.headline = null;
    
                            /**
                             * GmailTeaser description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            GmailTeaser.prototype.description = null;
    
                            /**
                             * GmailTeaser business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            GmailTeaser.prototype.business_name = null;
    
                            /**
                             * GmailTeaser logo_image.
                             * @member {string|null|undefined} logo_image
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            GmailTeaser.prototype.logo_image = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GmailTeaser _headline.
                             * @member {"headline"|undefined} _headline
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            Object.defineProperty(GmailTeaser.prototype, "_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GmailTeaser _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            Object.defineProperty(GmailTeaser.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GmailTeaser _business_name.
                             * @member {"business_name"|undefined} _business_name
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            Object.defineProperty(GmailTeaser.prototype, "_business_name", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GmailTeaser _logo_image.
                             * @member {"logo_image"|undefined} _logo_image
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             */
                            Object.defineProperty(GmailTeaser.prototype, "_logo_image", {
                                get: $util.oneOfGetter($oneOfFields = ["logo_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GmailTeaser instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {google.ads.googleads.v11.common.IGmailTeaser=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.GmailTeaser} GmailTeaser instance
                             */
                            GmailTeaser.create = function create(properties) {
                                return new GmailTeaser(properties);
                            };
    
                            /**
                             * Encodes the specified GmailTeaser message. Does not implicitly {@link google.ads.googleads.v11.common.GmailTeaser.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {google.ads.googleads.v11.common.IGmailTeaser} m GmailTeaser message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GmailTeaser.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(42).string(m.headline);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(50).string(m.description);
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(58).string(m.business_name);
                                if (m.logo_image != null && Object.hasOwnProperty.call(m, "logo_image"))
                                    w.uint32(66).string(m.logo_image);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GmailTeaser message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.GmailTeaser.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {google.ads.googleads.v11.common.IGmailTeaser} message GmailTeaser message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GmailTeaser.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GmailTeaser message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.GmailTeaser} GmailTeaser
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GmailTeaser.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.GmailTeaser();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.headline = r.string();
                                        break;
                                    case 6:
                                        m.description = r.string();
                                        break;
                                    case 7:
                                        m.business_name = r.string();
                                        break;
                                    case 8:
                                        m.logo_image = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GmailTeaser message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.GmailTeaser} GmailTeaser
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GmailTeaser.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GmailTeaser message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GmailTeaser.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    p._headline = 1;
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p._business_name = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.logo_image != null && m.hasOwnProperty("logo_image")) {
                                    p._logo_image = 1;
                                    if (!$util.isString(m.logo_image))
                                        return "logo_image: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GmailTeaser message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.GmailTeaser} GmailTeaser
                             */
                            GmailTeaser.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.GmailTeaser)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.GmailTeaser();
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.logo_image != null) {
                                    m.logo_image = String(d.logo_image);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GmailTeaser message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @static
                             * @param {google.ads.googleads.v11.common.GmailTeaser} m GmailTeaser
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GmailTeaser.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                    if (o.oneofs)
                                        d._headline = "headline";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d._business_name = "business_name";
                                }
                                if (m.logo_image != null && m.hasOwnProperty("logo_image")) {
                                    d.logo_image = m.logo_image;
                                    if (o.oneofs)
                                        d._logo_image = "logo_image";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GmailTeaser to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.GmailTeaser
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GmailTeaser.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GmailTeaser;
                        })();
    
                        common.DisplayCallToAction = (function() {
    
                            /**
                             * Properties of a DisplayCallToAction.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDisplayCallToAction
                             * @property {string|null} [text] DisplayCallToAction text
                             * @property {string|null} [text_color] DisplayCallToAction text_color
                             * @property {string|null} [url_collection_id] DisplayCallToAction url_collection_id
                             */
    
                            /**
                             * Constructs a new DisplayCallToAction.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DisplayCallToAction.
                             * @implements IDisplayCallToAction
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDisplayCallToAction=} [p] Properties to set
                             */
                            function DisplayCallToAction(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DisplayCallToAction text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @instance
                             */
                            DisplayCallToAction.prototype.text = null;
    
                            /**
                             * DisplayCallToAction text_color.
                             * @member {string|null|undefined} text_color
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @instance
                             */
                            DisplayCallToAction.prototype.text_color = null;
    
                            /**
                             * DisplayCallToAction url_collection_id.
                             * @member {string|null|undefined} url_collection_id
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @instance
                             */
                            DisplayCallToAction.prototype.url_collection_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DisplayCallToAction _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @instance
                             */
                            Object.defineProperty(DisplayCallToAction.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DisplayCallToAction _text_color.
                             * @member {"text_color"|undefined} _text_color
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @instance
                             */
                            Object.defineProperty(DisplayCallToAction.prototype, "_text_color", {
                                get: $util.oneOfGetter($oneOfFields = ["text_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DisplayCallToAction _url_collection_id.
                             * @member {"url_collection_id"|undefined} _url_collection_id
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @instance
                             */
                            Object.defineProperty(DisplayCallToAction.prototype, "_url_collection_id", {
                                get: $util.oneOfGetter($oneOfFields = ["url_collection_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DisplayCallToAction instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {google.ads.googleads.v11.common.IDisplayCallToAction=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DisplayCallToAction} DisplayCallToAction instance
                             */
                            DisplayCallToAction.create = function create(properties) {
                                return new DisplayCallToAction(properties);
                            };
    
                            /**
                             * Encodes the specified DisplayCallToAction message. Does not implicitly {@link google.ads.googleads.v11.common.DisplayCallToAction.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {google.ads.googleads.v11.common.IDisplayCallToAction} m DisplayCallToAction message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayCallToAction.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(42).string(m.text);
                                if (m.text_color != null && Object.hasOwnProperty.call(m, "text_color"))
                                    w.uint32(50).string(m.text_color);
                                if (m.url_collection_id != null && Object.hasOwnProperty.call(m, "url_collection_id"))
                                    w.uint32(58).string(m.url_collection_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DisplayCallToAction message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DisplayCallToAction.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {google.ads.googleads.v11.common.IDisplayCallToAction} message DisplayCallToAction message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayCallToAction.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DisplayCallToAction message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DisplayCallToAction} DisplayCallToAction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayCallToAction.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DisplayCallToAction();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.text = r.string();
                                        break;
                                    case 6:
                                        m.text_color = r.string();
                                        break;
                                    case 7:
                                        m.url_collection_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DisplayCallToAction message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DisplayCallToAction} DisplayCallToAction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayCallToAction.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DisplayCallToAction message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DisplayCallToAction.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.text_color != null && m.hasOwnProperty("text_color")) {
                                    p._text_color = 1;
                                    if (!$util.isString(m.text_color))
                                        return "text_color: string expected";
                                }
                                if (m.url_collection_id != null && m.hasOwnProperty("url_collection_id")) {
                                    p._url_collection_id = 1;
                                    if (!$util.isString(m.url_collection_id))
                                        return "url_collection_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DisplayCallToAction message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DisplayCallToAction} DisplayCallToAction
                             */
                            DisplayCallToAction.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DisplayCallToAction)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DisplayCallToAction();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                if (d.text_color != null) {
                                    m.text_color = String(d.text_color);
                                }
                                if (d.url_collection_id != null) {
                                    m.url_collection_id = String(d.url_collection_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DisplayCallToAction message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @static
                             * @param {google.ads.googleads.v11.common.DisplayCallToAction} m DisplayCallToAction
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DisplayCallToAction.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.text_color != null && m.hasOwnProperty("text_color")) {
                                    d.text_color = m.text_color;
                                    if (o.oneofs)
                                        d._text_color = "text_color";
                                }
                                if (m.url_collection_id != null && m.hasOwnProperty("url_collection_id")) {
                                    d.url_collection_id = m.url_collection_id;
                                    if (o.oneofs)
                                        d._url_collection_id = "url_collection_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DisplayCallToAction to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DisplayCallToAction
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DisplayCallToAction.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DisplayCallToAction;
                        })();
    
                        common.ProductImage = (function() {
    
                            /**
                             * Properties of a ProductImage.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductImage
                             * @property {string|null} [product_image] ProductImage product_image
                             * @property {string|null} [description] ProductImage description
                             * @property {google.ads.googleads.v11.common.IDisplayCallToAction|null} [display_call_to_action] ProductImage display_call_to_action
                             */
    
                            /**
                             * Constructs a new ProductImage.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductImage.
                             * @implements IProductImage
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductImage=} [p] Properties to set
                             */
                            function ProductImage(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductImage product_image.
                             * @member {string|null|undefined} product_image
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @instance
                             */
                            ProductImage.prototype.product_image = null;
    
                            /**
                             * ProductImage description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @instance
                             */
                            ProductImage.prototype.description = null;
    
                            /**
                             * ProductImage display_call_to_action.
                             * @member {google.ads.googleads.v11.common.IDisplayCallToAction|null|undefined} display_call_to_action
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @instance
                             */
                            ProductImage.prototype.display_call_to_action = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductImage _product_image.
                             * @member {"product_image"|undefined} _product_image
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @instance
                             */
                            Object.defineProperty(ProductImage.prototype, "_product_image", {
                                get: $util.oneOfGetter($oneOfFields = ["product_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ProductImage _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @instance
                             */
                            Object.defineProperty(ProductImage.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductImage instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductImage=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductImage} ProductImage instance
                             */
                            ProductImage.create = function create(properties) {
                                return new ProductImage(properties);
                            };
    
                            /**
                             * Encodes the specified ProductImage message. Does not implicitly {@link google.ads.googleads.v11.common.ProductImage.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductImage} m ProductImage message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductImage.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.display_call_to_action != null && Object.hasOwnProperty.call(m, "display_call_to_action"))
                                    $root.google.ads.googleads.v11.common.DisplayCallToAction.encode(m.display_call_to_action, w.uint32(26).fork()).ldelim();
                                if (m.product_image != null && Object.hasOwnProperty.call(m, "product_image"))
                                    w.uint32(34).string(m.product_image);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(42).string(m.description);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductImage message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductImage.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductImage} message ProductImage message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductImage.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductImage message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductImage} ProductImage
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductImage.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductImage();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.product_image = r.string();
                                        break;
                                    case 5:
                                        m.description = r.string();
                                        break;
                                    case 3:
                                        m.display_call_to_action = $root.google.ads.googleads.v11.common.DisplayCallToAction.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductImage message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductImage} ProductImage
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductImage.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductImage message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductImage.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.product_image != null && m.hasOwnProperty("product_image")) {
                                    p._product_image = 1;
                                    if (!$util.isString(m.product_image))
                                        return "product_image: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.display_call_to_action != null && m.hasOwnProperty("display_call_to_action")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DisplayCallToAction.verify(m.display_call_to_action);
                                        if (e)
                                            return "display_call_to_action." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductImage message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductImage} ProductImage
                             */
                            ProductImage.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductImage)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductImage();
                                if (d.product_image != null) {
                                    m.product_image = String(d.product_image);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.display_call_to_action != null) {
                                    if (typeof d.display_call_to_action !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ProductImage.display_call_to_action: object expected");
                                    m.display_call_to_action = $root.google.ads.googleads.v11.common.DisplayCallToAction.fromObject(d.display_call_to_action);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductImage message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductImage} m ProductImage
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductImage.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.display_call_to_action = null;
                                }
                                if (m.display_call_to_action != null && m.hasOwnProperty("display_call_to_action")) {
                                    d.display_call_to_action = $root.google.ads.googleads.v11.common.DisplayCallToAction.toObject(m.display_call_to_action, o);
                                }
                                if (m.product_image != null && m.hasOwnProperty("product_image")) {
                                    d.product_image = m.product_image;
                                    if (o.oneofs)
                                        d._product_image = "product_image";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductImage to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductImage
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductImage.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductImage;
                        })();
    
                        common.ProductVideo = (function() {
    
                            /**
                             * Properties of a ProductVideo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductVideo
                             * @property {string|null} [product_video] ProductVideo product_video
                             */
    
                            /**
                             * Constructs a new ProductVideo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductVideo.
                             * @implements IProductVideo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductVideo=} [p] Properties to set
                             */
                            function ProductVideo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductVideo product_video.
                             * @member {string|null|undefined} product_video
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @instance
                             */
                            ProductVideo.prototype.product_video = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductVideo _product_video.
                             * @member {"product_video"|undefined} _product_video
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @instance
                             */
                            Object.defineProperty(ProductVideo.prototype, "_product_video", {
                                get: $util.oneOfGetter($oneOfFields = ["product_video"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductVideo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductVideo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductVideo} ProductVideo instance
                             */
                            ProductVideo.create = function create(properties) {
                                return new ProductVideo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductVideo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductVideo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductVideo} m ProductVideo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductVideo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.product_video != null && Object.hasOwnProperty.call(m, "product_video"))
                                    w.uint32(18).string(m.product_video);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductVideo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductVideo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductVideo} message ProductVideo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductVideo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductVideo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductVideo} ProductVideo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductVideo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductVideo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.product_video = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductVideo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductVideo} ProductVideo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductVideo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductVideo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductVideo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.product_video != null && m.hasOwnProperty("product_video")) {
                                    p._product_video = 1;
                                    if (!$util.isString(m.product_video))
                                        return "product_video: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductVideo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductVideo} ProductVideo
                             */
                            ProductVideo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductVideo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductVideo();
                                if (d.product_video != null) {
                                    m.product_video = String(d.product_video);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductVideo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductVideo} m ProductVideo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductVideo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.product_video != null && m.hasOwnProperty("product_video")) {
                                    d.product_video = m.product_video;
                                    if (o.oneofs)
                                        d._product_video = "product_video";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductVideo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductVideo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductVideo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductVideo;
                        })();
    
                        common.ImageAdInfo = (function() {
    
                            /**
                             * Properties of an ImageAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IImageAdInfo
                             * @property {number|null} [pixel_width] ImageAdInfo pixel_width
                             * @property {number|null} [pixel_height] ImageAdInfo pixel_height
                             * @property {string|null} [image_url] ImageAdInfo image_url
                             * @property {number|null} [preview_pixel_width] ImageAdInfo preview_pixel_width
                             * @property {number|null} [preview_pixel_height] ImageAdInfo preview_pixel_height
                             * @property {string|null} [preview_image_url] ImageAdInfo preview_image_url
                             * @property {google.ads.googleads.v11.enums.MimeTypeEnum.MimeType|null} [mime_type] ImageAdInfo mime_type
                             * @property {string|null} [name] ImageAdInfo name
                             * @property {string|null} [media_file] ImageAdInfo media_file
                             * @property {Uint8Array|null} [data] ImageAdInfo data
                             * @property {number|null} [ad_id_to_copy_image_from] ImageAdInfo ad_id_to_copy_image_from
                             */
    
                            /**
                             * Constructs a new ImageAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ImageAdInfo.
                             * @implements IImageAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IImageAdInfo=} [p] Properties to set
                             */
                            function ImageAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ImageAdInfo pixel_width.
                             * @member {number|null|undefined} pixel_width
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.pixel_width = null;
    
                            /**
                             * ImageAdInfo pixel_height.
                             * @member {number|null|undefined} pixel_height
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.pixel_height = null;
    
                            /**
                             * ImageAdInfo image_url.
                             * @member {string|null|undefined} image_url
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.image_url = null;
    
                            /**
                             * ImageAdInfo preview_pixel_width.
                             * @member {number|null|undefined} preview_pixel_width
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.preview_pixel_width = null;
    
                            /**
                             * ImageAdInfo preview_pixel_height.
                             * @member {number|null|undefined} preview_pixel_height
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.preview_pixel_height = null;
    
                            /**
                             * ImageAdInfo preview_image_url.
                             * @member {string|null|undefined} preview_image_url
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.preview_image_url = null;
    
                            /**
                             * ImageAdInfo mime_type.
                             * @member {google.ads.googleads.v11.enums.MimeTypeEnum.MimeType} mime_type
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.mime_type = 0;
    
                            /**
                             * ImageAdInfo name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.name = null;
    
                            /**
                             * ImageAdInfo media_file.
                             * @member {string|null|undefined} media_file
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.media_file = null;
    
                            /**
                             * ImageAdInfo data.
                             * @member {Uint8Array|null|undefined} data
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.data = null;
    
                            /**
                             * ImageAdInfo ad_id_to_copy_image_from.
                             * @member {number|null|undefined} ad_id_to_copy_image_from
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            ImageAdInfo.prototype.ad_id_to_copy_image_from = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ImageAdInfo _pixel_width.
                             * @member {"pixel_width"|undefined} _pixel_width
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "_pixel_width", {
                                get: $util.oneOfGetter($oneOfFields = ["pixel_width"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAdInfo _pixel_height.
                             * @member {"pixel_height"|undefined} _pixel_height
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "_pixel_height", {
                                get: $util.oneOfGetter($oneOfFields = ["pixel_height"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAdInfo _image_url.
                             * @member {"image_url"|undefined} _image_url
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "_image_url", {
                                get: $util.oneOfGetter($oneOfFields = ["image_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAdInfo _preview_pixel_width.
                             * @member {"preview_pixel_width"|undefined} _preview_pixel_width
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "_preview_pixel_width", {
                                get: $util.oneOfGetter($oneOfFields = ["preview_pixel_width"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAdInfo _preview_pixel_height.
                             * @member {"preview_pixel_height"|undefined} _preview_pixel_height
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "_preview_pixel_height", {
                                get: $util.oneOfGetter($oneOfFields = ["preview_pixel_height"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAdInfo _preview_image_url.
                             * @member {"preview_image_url"|undefined} _preview_image_url
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "_preview_image_url", {
                                get: $util.oneOfGetter($oneOfFields = ["preview_image_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAdInfo _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAdInfo image.
                             * @member {"media_file"|"data"|"ad_id_to_copy_image_from"|undefined} image
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             */
                            Object.defineProperty(ImageAdInfo.prototype, "image", {
                                get: $util.oneOfGetter($oneOfFields = ["media_file", "data", "ad_id_to_copy_image_from"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ImageAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ImageAdInfo} ImageAdInfo instance
                             */
                            ImageAdInfo.create = function create(properties) {
                                return new ImageAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ImageAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ImageAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageAdInfo} m ImageAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.mime_type != null && Object.hasOwnProperty.call(m, "mime_type"))
                                    w.uint32(80).int32(m.mime_type);
                                if (m.media_file != null && Object.hasOwnProperty.call(m, "media_file"))
                                    w.uint32(98).string(m.media_file);
                                if (m.data != null && Object.hasOwnProperty.call(m, "data"))
                                    w.uint32(106).bytes(m.data);
                                if (m.ad_id_to_copy_image_from != null && Object.hasOwnProperty.call(m, "ad_id_to_copy_image_from"))
                                    w.uint32(112).int64(m.ad_id_to_copy_image_from);
                                if (m.pixel_width != null && Object.hasOwnProperty.call(m, "pixel_width"))
                                    w.uint32(120).int64(m.pixel_width);
                                if (m.pixel_height != null && Object.hasOwnProperty.call(m, "pixel_height"))
                                    w.uint32(128).int64(m.pixel_height);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(138).string(m.image_url);
                                if (m.preview_pixel_width != null && Object.hasOwnProperty.call(m, "preview_pixel_width"))
                                    w.uint32(144).int64(m.preview_pixel_width);
                                if (m.preview_pixel_height != null && Object.hasOwnProperty.call(m, "preview_pixel_height"))
                                    w.uint32(152).int64(m.preview_pixel_height);
                                if (m.preview_image_url != null && Object.hasOwnProperty.call(m, "preview_image_url"))
                                    w.uint32(162).string(m.preview_image_url);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(170).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ImageAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ImageAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageAdInfo} message ImageAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ImageAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ImageAdInfo} ImageAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ImageAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 15:
                                        m.pixel_width = r.int64();
                                        break;
                                    case 16:
                                        m.pixel_height = r.int64();
                                        break;
                                    case 17:
                                        m.image_url = r.string();
                                        break;
                                    case 18:
                                        m.preview_pixel_width = r.int64();
                                        break;
                                    case 19:
                                        m.preview_pixel_height = r.int64();
                                        break;
                                    case 20:
                                        m.preview_image_url = r.string();
                                        break;
                                    case 10:
                                        m.mime_type = r.int32();
                                        break;
                                    case 21:
                                        m.name = r.string();
                                        break;
                                    case 12:
                                        m.media_file = r.string();
                                        break;
                                    case 13:
                                        m.data = r.bytes();
                                        break;
                                    case 14:
                                        m.ad_id_to_copy_image_from = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ImageAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ImageAdInfo} ImageAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ImageAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ImageAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.pixel_width != null && m.hasOwnProperty("pixel_width")) {
                                    p._pixel_width = 1;
                                    if (!$util.isInteger(m.pixel_width) && !(m.pixel_width && $util.isInteger(m.pixel_width.low) && $util.isInteger(m.pixel_width.high)))
                                        return "pixel_width: integer|Long expected";
                                }
                                if (m.pixel_height != null && m.hasOwnProperty("pixel_height")) {
                                    p._pixel_height = 1;
                                    if (!$util.isInteger(m.pixel_height) && !(m.pixel_height && $util.isInteger(m.pixel_height.low) && $util.isInteger(m.pixel_height.high)))
                                        return "pixel_height: integer|Long expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    p._image_url = 1;
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.preview_pixel_width != null && m.hasOwnProperty("preview_pixel_width")) {
                                    p._preview_pixel_width = 1;
                                    if (!$util.isInteger(m.preview_pixel_width) && !(m.preview_pixel_width && $util.isInteger(m.preview_pixel_width.low) && $util.isInteger(m.preview_pixel_width.high)))
                                        return "preview_pixel_width: integer|Long expected";
                                }
                                if (m.preview_pixel_height != null && m.hasOwnProperty("preview_pixel_height")) {
                                    p._preview_pixel_height = 1;
                                    if (!$util.isInteger(m.preview_pixel_height) && !(m.preview_pixel_height && $util.isInteger(m.preview_pixel_height.low) && $util.isInteger(m.preview_pixel_height.high)))
                                        return "preview_pixel_height: integer|Long expected";
                                }
                                if (m.preview_image_url != null && m.hasOwnProperty("preview_image_url")) {
                                    p._preview_image_url = 1;
                                    if (!$util.isString(m.preview_image_url))
                                        return "preview_image_url: string expected";
                                }
                                if (m.mime_type != null && m.hasOwnProperty("mime_type")) {
                                    switch (m.mime_type) {
                                    default:
                                        return "mime_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.media_file != null && m.hasOwnProperty("media_file")) {
                                    p.image = 1;
                                    if (!$util.isString(m.media_file))
                                        return "media_file: string expected";
                                }
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    if (p.image === 1)
                                        return "image: multiple values";
                                    p.image = 1;
                                    if (!(m.data && typeof m.data.length === "number" || $util.isString(m.data)))
                                        return "data: buffer expected";
                                }
                                if (m.ad_id_to_copy_image_from != null && m.hasOwnProperty("ad_id_to_copy_image_from")) {
                                    if (p.image === 1)
                                        return "image: multiple values";
                                    p.image = 1;
                                    if (!$util.isInteger(m.ad_id_to_copy_image_from) && !(m.ad_id_to_copy_image_from && $util.isInteger(m.ad_id_to_copy_image_from.low) && $util.isInteger(m.ad_id_to_copy_image_from.high)))
                                        return "ad_id_to_copy_image_from: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ImageAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ImageAdInfo} ImageAdInfo
                             */
                            ImageAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ImageAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ImageAdInfo();
                                if (d.pixel_width != null) {
                                    if ($util.Long)
                                        (m.pixel_width = $util.Long.fromValue(d.pixel_width)).unsigned = false;
                                    else if (typeof d.pixel_width === "string")
                                        m.pixel_width = parseInt(d.pixel_width, 10);
                                    else if (typeof d.pixel_width === "number")
                                        m.pixel_width = d.pixel_width;
                                    else if (typeof d.pixel_width === "object")
                                        m.pixel_width = new $util.LongBits(d.pixel_width.low >>> 0, d.pixel_width.high >>> 0).toNumber();
                                }
                                if (d.pixel_height != null) {
                                    if ($util.Long)
                                        (m.pixel_height = $util.Long.fromValue(d.pixel_height)).unsigned = false;
                                    else if (typeof d.pixel_height === "string")
                                        m.pixel_height = parseInt(d.pixel_height, 10);
                                    else if (typeof d.pixel_height === "number")
                                        m.pixel_height = d.pixel_height;
                                    else if (typeof d.pixel_height === "object")
                                        m.pixel_height = new $util.LongBits(d.pixel_height.low >>> 0, d.pixel_height.high >>> 0).toNumber();
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.preview_pixel_width != null) {
                                    if ($util.Long)
                                        (m.preview_pixel_width = $util.Long.fromValue(d.preview_pixel_width)).unsigned = false;
                                    else if (typeof d.preview_pixel_width === "string")
                                        m.preview_pixel_width = parseInt(d.preview_pixel_width, 10);
                                    else if (typeof d.preview_pixel_width === "number")
                                        m.preview_pixel_width = d.preview_pixel_width;
                                    else if (typeof d.preview_pixel_width === "object")
                                        m.preview_pixel_width = new $util.LongBits(d.preview_pixel_width.low >>> 0, d.preview_pixel_width.high >>> 0).toNumber();
                                }
                                if (d.preview_pixel_height != null) {
                                    if ($util.Long)
                                        (m.preview_pixel_height = $util.Long.fromValue(d.preview_pixel_height)).unsigned = false;
                                    else if (typeof d.preview_pixel_height === "string")
                                        m.preview_pixel_height = parseInt(d.preview_pixel_height, 10);
                                    else if (typeof d.preview_pixel_height === "number")
                                        m.preview_pixel_height = d.preview_pixel_height;
                                    else if (typeof d.preview_pixel_height === "object")
                                        m.preview_pixel_height = new $util.LongBits(d.preview_pixel_height.low >>> 0, d.preview_pixel_height.high >>> 0).toNumber();
                                }
                                if (d.preview_image_url != null) {
                                    m.preview_image_url = String(d.preview_image_url);
                                }
                                switch (d.mime_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.mime_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.mime_type = 1;
                                    break;
                                case "IMAGE_JPEG":
                                case 2:
                                    m.mime_type = 2;
                                    break;
                                case "IMAGE_GIF":
                                case 3:
                                    m.mime_type = 3;
                                    break;
                                case "IMAGE_PNG":
                                case 4:
                                    m.mime_type = 4;
                                    break;
                                case "FLASH":
                                case 5:
                                    m.mime_type = 5;
                                    break;
                                case "TEXT_HTML":
                                case 6:
                                    m.mime_type = 6;
                                    break;
                                case "PDF":
                                case 7:
                                    m.mime_type = 7;
                                    break;
                                case "MSWORD":
                                case 8:
                                    m.mime_type = 8;
                                    break;
                                case "MSEXCEL":
                                case 9:
                                    m.mime_type = 9;
                                    break;
                                case "RTF":
                                case 10:
                                    m.mime_type = 10;
                                    break;
                                case "AUDIO_WAV":
                                case 11:
                                    m.mime_type = 11;
                                    break;
                                case "AUDIO_MP3":
                                case 12:
                                    m.mime_type = 12;
                                    break;
                                case "HTML5_AD_ZIP":
                                case 13:
                                    m.mime_type = 13;
                                    break;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.media_file != null) {
                                    m.media_file = String(d.media_file);
                                }
                                if (d.data != null) {
                                    if (typeof d.data === "string")
                                        $util.base64.decode(d.data, m.data = $util.newBuffer($util.base64.length(d.data)), 0);
                                    else if (d.data.length)
                                        m.data = d.data;
                                }
                                if (d.ad_id_to_copy_image_from != null) {
                                    if ($util.Long)
                                        (m.ad_id_to_copy_image_from = $util.Long.fromValue(d.ad_id_to_copy_image_from)).unsigned = false;
                                    else if (typeof d.ad_id_to_copy_image_from === "string")
                                        m.ad_id_to_copy_image_from = parseInt(d.ad_id_to_copy_image_from, 10);
                                    else if (typeof d.ad_id_to_copy_image_from === "number")
                                        m.ad_id_to_copy_image_from = d.ad_id_to_copy_image_from;
                                    else if (typeof d.ad_id_to_copy_image_from === "object")
                                        m.ad_id_to_copy_image_from = new $util.LongBits(d.ad_id_to_copy_image_from.low >>> 0, d.ad_id_to_copy_image_from.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ImageAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ImageAdInfo} m ImageAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ImageAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.mime_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.mime_type != null && m.hasOwnProperty("mime_type")) {
                                    d.mime_type = o.enums === String ? $root.google.ads.googleads.v11.enums.MimeTypeEnum.MimeType[m.mime_type] : m.mime_type;
                                }
                                if (m.media_file != null && m.hasOwnProperty("media_file")) {
                                    d.media_file = m.media_file;
                                    if (o.oneofs)
                                        d.image = "media_file";
                                }
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    d.data = o.bytes === String ? $util.base64.encode(m.data, 0, m.data.length) : o.bytes === Array ? Array.prototype.slice.call(m.data) : m.data;
                                    if (o.oneofs)
                                        d.image = "data";
                                }
                                if (m.ad_id_to_copy_image_from != null && m.hasOwnProperty("ad_id_to_copy_image_from")) {
                                    if (typeof m.ad_id_to_copy_image_from === "number")
                                        d.ad_id_to_copy_image_from = o.longs === String ? String(m.ad_id_to_copy_image_from) : m.ad_id_to_copy_image_from;
                                    else
                                        d.ad_id_to_copy_image_from = o.longs === String ? $util.Long.prototype.toString.call(m.ad_id_to_copy_image_from) : o.longs === Number ? new $util.LongBits(m.ad_id_to_copy_image_from.low >>> 0, m.ad_id_to_copy_image_from.high >>> 0).toNumber() : m.ad_id_to_copy_image_from;
                                    if (o.oneofs)
                                        d.image = "ad_id_to_copy_image_from";
                                }
                                if (m.pixel_width != null && m.hasOwnProperty("pixel_width")) {
                                    if (typeof m.pixel_width === "number")
                                        d.pixel_width = o.longs === String ? String(m.pixel_width) : m.pixel_width;
                                    else
                                        d.pixel_width = o.longs === String ? $util.Long.prototype.toString.call(m.pixel_width) : o.longs === Number ? new $util.LongBits(m.pixel_width.low >>> 0, m.pixel_width.high >>> 0).toNumber() : m.pixel_width;
                                    if (o.oneofs)
                                        d._pixel_width = "pixel_width";
                                }
                                if (m.pixel_height != null && m.hasOwnProperty("pixel_height")) {
                                    if (typeof m.pixel_height === "number")
                                        d.pixel_height = o.longs === String ? String(m.pixel_height) : m.pixel_height;
                                    else
                                        d.pixel_height = o.longs === String ? $util.Long.prototype.toString.call(m.pixel_height) : o.longs === Number ? new $util.LongBits(m.pixel_height.low >>> 0, m.pixel_height.high >>> 0).toNumber() : m.pixel_height;
                                    if (o.oneofs)
                                        d._pixel_height = "pixel_height";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                    if (o.oneofs)
                                        d._image_url = "image_url";
                                }
                                if (m.preview_pixel_width != null && m.hasOwnProperty("preview_pixel_width")) {
                                    if (typeof m.preview_pixel_width === "number")
                                        d.preview_pixel_width = o.longs === String ? String(m.preview_pixel_width) : m.preview_pixel_width;
                                    else
                                        d.preview_pixel_width = o.longs === String ? $util.Long.prototype.toString.call(m.preview_pixel_width) : o.longs === Number ? new $util.LongBits(m.preview_pixel_width.low >>> 0, m.preview_pixel_width.high >>> 0).toNumber() : m.preview_pixel_width;
                                    if (o.oneofs)
                                        d._preview_pixel_width = "preview_pixel_width";
                                }
                                if (m.preview_pixel_height != null && m.hasOwnProperty("preview_pixel_height")) {
                                    if (typeof m.preview_pixel_height === "number")
                                        d.preview_pixel_height = o.longs === String ? String(m.preview_pixel_height) : m.preview_pixel_height;
                                    else
                                        d.preview_pixel_height = o.longs === String ? $util.Long.prototype.toString.call(m.preview_pixel_height) : o.longs === Number ? new $util.LongBits(m.preview_pixel_height.low >>> 0, m.preview_pixel_height.high >>> 0).toNumber() : m.preview_pixel_height;
                                    if (o.oneofs)
                                        d._preview_pixel_height = "preview_pixel_height";
                                }
                                if (m.preview_image_url != null && m.hasOwnProperty("preview_image_url")) {
                                    d.preview_image_url = m.preview_image_url;
                                    if (o.oneofs)
                                        d._preview_image_url = "preview_image_url";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ImageAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ImageAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ImageAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ImageAdInfo;
                        })();
    
                        common.VideoBumperInStreamAdInfo = (function() {
    
                            /**
                             * Properties of a VideoBumperInStreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IVideoBumperInStreamAdInfo
                             * @property {google.ads.googleads.v11.common.IAdImageAsset|null} [companion_banner] VideoBumperInStreamAdInfo companion_banner
                             */
    
                            /**
                             * Constructs a new VideoBumperInStreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a VideoBumperInStreamAdInfo.
                             * @implements IVideoBumperInStreamAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IVideoBumperInStreamAdInfo=} [p] Properties to set
                             */
                            function VideoBumperInStreamAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * VideoBumperInStreamAdInfo companion_banner.
                             * @member {google.ads.googleads.v11.common.IAdImageAsset|null|undefined} companion_banner
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @instance
                             */
                            VideoBumperInStreamAdInfo.prototype.companion_banner = null;
    
                            /**
                             * Creates a new VideoBumperInStreamAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoBumperInStreamAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.VideoBumperInStreamAdInfo} VideoBumperInStreamAdInfo instance
                             */
                            VideoBumperInStreamAdInfo.create = function create(properties) {
                                return new VideoBumperInStreamAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified VideoBumperInStreamAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoBumperInStreamAdInfo} m VideoBumperInStreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoBumperInStreamAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.companion_banner != null && Object.hasOwnProperty.call(m, "companion_banner"))
                                    $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.companion_banner, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified VideoBumperInStreamAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoBumperInStreamAdInfo} message VideoBumperInStreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoBumperInStreamAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VideoBumperInStreamAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.VideoBumperInStreamAdInfo} VideoBumperInStreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoBumperInStreamAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VideoBumperInStreamAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.VideoBumperInStreamAdInfo} VideoBumperInStreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoBumperInStreamAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VideoBumperInStreamAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VideoBumperInStreamAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.companion_banner != null && m.hasOwnProperty("companion_banner")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.companion_banner);
                                        if (e)
                                            return "companion_banner." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a VideoBumperInStreamAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.VideoBumperInStreamAdInfo} VideoBumperInStreamAdInfo
                             */
                            VideoBumperInStreamAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo();
                                if (d.companion_banner != null) {
                                    if (typeof d.companion_banner !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.companion_banner: object expected");
                                    m.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.companion_banner);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a VideoBumperInStreamAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.VideoBumperInStreamAdInfo} m VideoBumperInStreamAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VideoBumperInStreamAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.companion_banner = null;
                                }
                                if (m.companion_banner != null && m.hasOwnProperty("companion_banner")) {
                                    d.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.companion_banner, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this VideoBumperInStreamAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.VideoBumperInStreamAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VideoBumperInStreamAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return VideoBumperInStreamAdInfo;
                        })();
    
                        common.VideoNonSkippableInStreamAdInfo = (function() {
    
                            /**
                             * Properties of a VideoNonSkippableInStreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IVideoNonSkippableInStreamAdInfo
                             * @property {google.ads.googleads.v11.common.IAdImageAsset|null} [companion_banner] VideoNonSkippableInStreamAdInfo companion_banner
                             * @property {string|null} [action_button_label] VideoNonSkippableInStreamAdInfo action_button_label
                             * @property {string|null} [action_headline] VideoNonSkippableInStreamAdInfo action_headline
                             */
    
                            /**
                             * Constructs a new VideoNonSkippableInStreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a VideoNonSkippableInStreamAdInfo.
                             * @implements IVideoNonSkippableInStreamAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IVideoNonSkippableInStreamAdInfo=} [p] Properties to set
                             */
                            function VideoNonSkippableInStreamAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * VideoNonSkippableInStreamAdInfo companion_banner.
                             * @member {google.ads.googleads.v11.common.IAdImageAsset|null|undefined} companion_banner
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @instance
                             */
                            VideoNonSkippableInStreamAdInfo.prototype.companion_banner = null;
    
                            /**
                             * VideoNonSkippableInStreamAdInfo action_button_label.
                             * @member {string} action_button_label
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @instance
                             */
                            VideoNonSkippableInStreamAdInfo.prototype.action_button_label = "";
    
                            /**
                             * VideoNonSkippableInStreamAdInfo action_headline.
                             * @member {string} action_headline
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @instance
                             */
                            VideoNonSkippableInStreamAdInfo.prototype.action_headline = "";
    
                            /**
                             * Creates a new VideoNonSkippableInStreamAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoNonSkippableInStreamAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo} VideoNonSkippableInStreamAdInfo instance
                             */
                            VideoNonSkippableInStreamAdInfo.create = function create(properties) {
                                return new VideoNonSkippableInStreamAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified VideoNonSkippableInStreamAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoNonSkippableInStreamAdInfo} m VideoNonSkippableInStreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoNonSkippableInStreamAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.action_button_label != null && Object.hasOwnProperty.call(m, "action_button_label"))
                                    w.uint32(26).string(m.action_button_label);
                                if (m.action_headline != null && Object.hasOwnProperty.call(m, "action_headline"))
                                    w.uint32(34).string(m.action_headline);
                                if (m.companion_banner != null && Object.hasOwnProperty.call(m, "companion_banner"))
                                    $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.companion_banner, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified VideoNonSkippableInStreamAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoNonSkippableInStreamAdInfo} message VideoNonSkippableInStreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoNonSkippableInStreamAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VideoNonSkippableInStreamAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo} VideoNonSkippableInStreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoNonSkippableInStreamAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.action_button_label = r.string();
                                        break;
                                    case 4:
                                        m.action_headline = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VideoNonSkippableInStreamAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo} VideoNonSkippableInStreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoNonSkippableInStreamAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VideoNonSkippableInStreamAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VideoNonSkippableInStreamAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.companion_banner != null && m.hasOwnProperty("companion_banner")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.companion_banner);
                                        if (e)
                                            return "companion_banner." + e;
                                    }
                                }
                                if (m.action_button_label != null && m.hasOwnProperty("action_button_label")) {
                                    if (!$util.isString(m.action_button_label))
                                        return "action_button_label: string expected";
                                }
                                if (m.action_headline != null && m.hasOwnProperty("action_headline")) {
                                    if (!$util.isString(m.action_headline))
                                        return "action_headline: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a VideoNonSkippableInStreamAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo} VideoNonSkippableInStreamAdInfo
                             */
                            VideoNonSkippableInStreamAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo();
                                if (d.companion_banner != null) {
                                    if (typeof d.companion_banner !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.companion_banner: object expected");
                                    m.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.companion_banner);
                                }
                                if (d.action_button_label != null) {
                                    m.action_button_label = String(d.action_button_label);
                                }
                                if (d.action_headline != null) {
                                    m.action_headline = String(d.action_headline);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a VideoNonSkippableInStreamAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo} m VideoNonSkippableInStreamAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VideoNonSkippableInStreamAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.action_button_label = "";
                                    d.action_headline = "";
                                    d.companion_banner = null;
                                }
                                if (m.action_button_label != null && m.hasOwnProperty("action_button_label")) {
                                    d.action_button_label = m.action_button_label;
                                }
                                if (m.action_headline != null && m.hasOwnProperty("action_headline")) {
                                    d.action_headline = m.action_headline;
                                }
                                if (m.companion_banner != null && m.hasOwnProperty("companion_banner")) {
                                    d.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.companion_banner, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this VideoNonSkippableInStreamAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VideoNonSkippableInStreamAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return VideoNonSkippableInStreamAdInfo;
                        })();
    
                        common.VideoTrueViewInStreamAdInfo = (function() {
    
                            /**
                             * Properties of a VideoTrueViewInStreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IVideoTrueViewInStreamAdInfo
                             * @property {string|null} [action_button_label] VideoTrueViewInStreamAdInfo action_button_label
                             * @property {string|null} [action_headline] VideoTrueViewInStreamAdInfo action_headline
                             * @property {google.ads.googleads.v11.common.IAdImageAsset|null} [companion_banner] VideoTrueViewInStreamAdInfo companion_banner
                             */
    
                            /**
                             * Constructs a new VideoTrueViewInStreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a VideoTrueViewInStreamAdInfo.
                             * @implements IVideoTrueViewInStreamAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IVideoTrueViewInStreamAdInfo=} [p] Properties to set
                             */
                            function VideoTrueViewInStreamAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * VideoTrueViewInStreamAdInfo action_button_label.
                             * @member {string} action_button_label
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @instance
                             */
                            VideoTrueViewInStreamAdInfo.prototype.action_button_label = "";
    
                            /**
                             * VideoTrueViewInStreamAdInfo action_headline.
                             * @member {string} action_headline
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @instance
                             */
                            VideoTrueViewInStreamAdInfo.prototype.action_headline = "";
    
                            /**
                             * VideoTrueViewInStreamAdInfo companion_banner.
                             * @member {google.ads.googleads.v11.common.IAdImageAsset|null|undefined} companion_banner
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @instance
                             */
                            VideoTrueViewInStreamAdInfo.prototype.companion_banner = null;
    
                            /**
                             * Creates a new VideoTrueViewInStreamAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoTrueViewInStreamAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo} VideoTrueViewInStreamAdInfo instance
                             */
                            VideoTrueViewInStreamAdInfo.create = function create(properties) {
                                return new VideoTrueViewInStreamAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified VideoTrueViewInStreamAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoTrueViewInStreamAdInfo} m VideoTrueViewInStreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoTrueViewInStreamAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.action_button_label != null && Object.hasOwnProperty.call(m, "action_button_label"))
                                    w.uint32(34).string(m.action_button_label);
                                if (m.action_headline != null && Object.hasOwnProperty.call(m, "action_headline"))
                                    w.uint32(42).string(m.action_headline);
                                if (m.companion_banner != null && Object.hasOwnProperty.call(m, "companion_banner"))
                                    $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.companion_banner, w.uint32(58).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified VideoTrueViewInStreamAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoTrueViewInStreamAdInfo} message VideoTrueViewInStreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoTrueViewInStreamAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VideoTrueViewInStreamAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo} VideoTrueViewInStreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoTrueViewInStreamAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.action_button_label = r.string();
                                        break;
                                    case 5:
                                        m.action_headline = r.string();
                                        break;
                                    case 7:
                                        m.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VideoTrueViewInStreamAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo} VideoTrueViewInStreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoTrueViewInStreamAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VideoTrueViewInStreamAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VideoTrueViewInStreamAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.action_button_label != null && m.hasOwnProperty("action_button_label")) {
                                    if (!$util.isString(m.action_button_label))
                                        return "action_button_label: string expected";
                                }
                                if (m.action_headline != null && m.hasOwnProperty("action_headline")) {
                                    if (!$util.isString(m.action_headline))
                                        return "action_headline: string expected";
                                }
                                if (m.companion_banner != null && m.hasOwnProperty("companion_banner")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.companion_banner);
                                        if (e)
                                            return "companion_banner." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a VideoTrueViewInStreamAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo} VideoTrueViewInStreamAdInfo
                             */
                            VideoTrueViewInStreamAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo();
                                if (d.action_button_label != null) {
                                    m.action_button_label = String(d.action_button_label);
                                }
                                if (d.action_headline != null) {
                                    m.action_headline = String(d.action_headline);
                                }
                                if (d.companion_banner != null) {
                                    if (typeof d.companion_banner !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.companion_banner: object expected");
                                    m.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.companion_banner);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a VideoTrueViewInStreamAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo} m VideoTrueViewInStreamAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VideoTrueViewInStreamAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.action_button_label = "";
                                    d.action_headline = "";
                                    d.companion_banner = null;
                                }
                                if (m.action_button_label != null && m.hasOwnProperty("action_button_label")) {
                                    d.action_button_label = m.action_button_label;
                                }
                                if (m.action_headline != null && m.hasOwnProperty("action_headline")) {
                                    d.action_headline = m.action_headline;
                                }
                                if (m.companion_banner != null && m.hasOwnProperty("companion_banner")) {
                                    d.companion_banner = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.companion_banner, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this VideoTrueViewInStreamAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VideoTrueViewInStreamAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return VideoTrueViewInStreamAdInfo;
                        })();
    
                        common.VideoOutstreamAdInfo = (function() {
    
                            /**
                             * Properties of a VideoOutstreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IVideoOutstreamAdInfo
                             * @property {string|null} [headline] VideoOutstreamAdInfo headline
                             * @property {string|null} [description] VideoOutstreamAdInfo description
                             */
    
                            /**
                             * Constructs a new VideoOutstreamAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a VideoOutstreamAdInfo.
                             * @implements IVideoOutstreamAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IVideoOutstreamAdInfo=} [p] Properties to set
                             */
                            function VideoOutstreamAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * VideoOutstreamAdInfo headline.
                             * @member {string} headline
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @instance
                             */
                            VideoOutstreamAdInfo.prototype.headline = "";
    
                            /**
                             * VideoOutstreamAdInfo description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @instance
                             */
                            VideoOutstreamAdInfo.prototype.description = "";
    
                            /**
                             * Creates a new VideoOutstreamAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoOutstreamAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.VideoOutstreamAdInfo} VideoOutstreamAdInfo instance
                             */
                            VideoOutstreamAdInfo.create = function create(properties) {
                                return new VideoOutstreamAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified VideoOutstreamAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.VideoOutstreamAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoOutstreamAdInfo} m VideoOutstreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoOutstreamAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(26).string(m.headline);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(34).string(m.description);
                                return w;
                            };
    
                            /**
                             * Encodes the specified VideoOutstreamAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.VideoOutstreamAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoOutstreamAdInfo} message VideoOutstreamAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoOutstreamAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VideoOutstreamAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.VideoOutstreamAdInfo} VideoOutstreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoOutstreamAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.headline = r.string();
                                        break;
                                    case 4:
                                        m.description = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VideoOutstreamAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.VideoOutstreamAdInfo} VideoOutstreamAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoOutstreamAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VideoOutstreamAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VideoOutstreamAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a VideoOutstreamAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.VideoOutstreamAdInfo} VideoOutstreamAdInfo
                             */
                            VideoOutstreamAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo();
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a VideoOutstreamAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.VideoOutstreamAdInfo} m VideoOutstreamAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VideoOutstreamAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.headline = "";
                                    d.description = "";
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this VideoOutstreamAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.VideoOutstreamAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VideoOutstreamAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return VideoOutstreamAdInfo;
                        })();
    
                        common.InFeedVideoAdInfo = (function() {
    
                            /**
                             * Properties of an InFeedVideoAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IInFeedVideoAdInfo
                             * @property {string|null} [headline] InFeedVideoAdInfo headline
                             * @property {string|null} [description1] InFeedVideoAdInfo description1
                             * @property {string|null} [description2] InFeedVideoAdInfo description2
                             * @property {google.ads.googleads.v11.enums.VideoThumbnailEnum.VideoThumbnail|null} [thumbnail] InFeedVideoAdInfo thumbnail
                             */
    
                            /**
                             * Constructs a new InFeedVideoAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an InFeedVideoAdInfo.
                             * @implements IInFeedVideoAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IInFeedVideoAdInfo=} [p] Properties to set
                             */
                            function InFeedVideoAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * InFeedVideoAdInfo headline.
                             * @member {string} headline
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @instance
                             */
                            InFeedVideoAdInfo.prototype.headline = "";
    
                            /**
                             * InFeedVideoAdInfo description1.
                             * @member {string} description1
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @instance
                             */
                            InFeedVideoAdInfo.prototype.description1 = "";
    
                            /**
                             * InFeedVideoAdInfo description2.
                             * @member {string} description2
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @instance
                             */
                            InFeedVideoAdInfo.prototype.description2 = "";
    
                            /**
                             * InFeedVideoAdInfo thumbnail.
                             * @member {google.ads.googleads.v11.enums.VideoThumbnailEnum.VideoThumbnail} thumbnail
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @instance
                             */
                            InFeedVideoAdInfo.prototype.thumbnail = 0;
    
                            /**
                             * Creates a new InFeedVideoAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IInFeedVideoAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.InFeedVideoAdInfo} InFeedVideoAdInfo instance
                             */
                            InFeedVideoAdInfo.create = function create(properties) {
                                return new InFeedVideoAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified InFeedVideoAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.InFeedVideoAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IInFeedVideoAdInfo} m InFeedVideoAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InFeedVideoAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(10).string(m.headline);
                                if (m.description1 != null && Object.hasOwnProperty.call(m, "description1"))
                                    w.uint32(18).string(m.description1);
                                if (m.description2 != null && Object.hasOwnProperty.call(m, "description2"))
                                    w.uint32(26).string(m.description2);
                                if (m.thumbnail != null && Object.hasOwnProperty.call(m, "thumbnail"))
                                    w.uint32(32).int32(m.thumbnail);
                                return w;
                            };
    
                            /**
                             * Encodes the specified InFeedVideoAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.InFeedVideoAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IInFeedVideoAdInfo} message InFeedVideoAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InFeedVideoAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an InFeedVideoAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.InFeedVideoAdInfo} InFeedVideoAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InFeedVideoAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.InFeedVideoAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.headline = r.string();
                                        break;
                                    case 2:
                                        m.description1 = r.string();
                                        break;
                                    case 3:
                                        m.description2 = r.string();
                                        break;
                                    case 4:
                                        m.thumbnail = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an InFeedVideoAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.InFeedVideoAdInfo} InFeedVideoAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InFeedVideoAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an InFeedVideoAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            InFeedVideoAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    if (!$util.isString(m.description1))
                                        return "description1: string expected";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    if (!$util.isString(m.description2))
                                        return "description2: string expected";
                                }
                                if (m.thumbnail != null && m.hasOwnProperty("thumbnail")) {
                                    switch (m.thumbnail) {
                                    default:
                                        return "thumbnail: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an InFeedVideoAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.InFeedVideoAdInfo} InFeedVideoAdInfo
                             */
                            InFeedVideoAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.InFeedVideoAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.InFeedVideoAdInfo();
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.description1 != null) {
                                    m.description1 = String(d.description1);
                                }
                                if (d.description2 != null) {
                                    m.description2 = String(d.description2);
                                }
                                switch (d.thumbnail) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.thumbnail = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.thumbnail = 1;
                                    break;
                                case "DEFAULT_THUMBNAIL":
                                case 2:
                                    m.thumbnail = 2;
                                    break;
                                case "THUMBNAIL_1":
                                case 3:
                                    m.thumbnail = 3;
                                    break;
                                case "THUMBNAIL_2":
                                case 4:
                                    m.thumbnail = 4;
                                    break;
                                case "THUMBNAIL_3":
                                case 5:
                                    m.thumbnail = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an InFeedVideoAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.InFeedVideoAdInfo} m InFeedVideoAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            InFeedVideoAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.headline = "";
                                    d.description1 = "";
                                    d.description2 = "";
                                    d.thumbnail = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    d.description1 = m.description1;
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    d.description2 = m.description2;
                                }
                                if (m.thumbnail != null && m.hasOwnProperty("thumbnail")) {
                                    d.thumbnail = o.enums === String ? $root.google.ads.googleads.v11.enums.VideoThumbnailEnum.VideoThumbnail[m.thumbnail] : m.thumbnail;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this InFeedVideoAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.InFeedVideoAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            InFeedVideoAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return InFeedVideoAdInfo;
                        })();
    
                        common.VideoAdInfo = (function() {
    
                            /**
                             * Properties of a VideoAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IVideoAdInfo
                             * @property {google.ads.googleads.v11.common.IAdVideoAsset|null} [video] VideoAdInfo video
                             * @property {google.ads.googleads.v11.common.IVideoTrueViewInStreamAdInfo|null} [in_stream] VideoAdInfo in_stream
                             * @property {google.ads.googleads.v11.common.IVideoBumperInStreamAdInfo|null} [bumper] VideoAdInfo bumper
                             * @property {google.ads.googleads.v11.common.IVideoOutstreamAdInfo|null} [out_stream] VideoAdInfo out_stream
                             * @property {google.ads.googleads.v11.common.IVideoNonSkippableInStreamAdInfo|null} [non_skippable] VideoAdInfo non_skippable
                             * @property {google.ads.googleads.v11.common.IInFeedVideoAdInfo|null} [in_feed] VideoAdInfo in_feed
                             */
    
                            /**
                             * Constructs a new VideoAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a VideoAdInfo.
                             * @implements IVideoAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IVideoAdInfo=} [p] Properties to set
                             */
                            function VideoAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * VideoAdInfo video.
                             * @member {google.ads.googleads.v11.common.IAdVideoAsset|null|undefined} video
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             */
                            VideoAdInfo.prototype.video = null;
    
                            /**
                             * VideoAdInfo in_stream.
                             * @member {google.ads.googleads.v11.common.IVideoTrueViewInStreamAdInfo|null|undefined} in_stream
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             */
                            VideoAdInfo.prototype.in_stream = null;
    
                            /**
                             * VideoAdInfo bumper.
                             * @member {google.ads.googleads.v11.common.IVideoBumperInStreamAdInfo|null|undefined} bumper
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             */
                            VideoAdInfo.prototype.bumper = null;
    
                            /**
                             * VideoAdInfo out_stream.
                             * @member {google.ads.googleads.v11.common.IVideoOutstreamAdInfo|null|undefined} out_stream
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             */
                            VideoAdInfo.prototype.out_stream = null;
    
                            /**
                             * VideoAdInfo non_skippable.
                             * @member {google.ads.googleads.v11.common.IVideoNonSkippableInStreamAdInfo|null|undefined} non_skippable
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             */
                            VideoAdInfo.prototype.non_skippable = null;
    
                            /**
                             * VideoAdInfo in_feed.
                             * @member {google.ads.googleads.v11.common.IInFeedVideoAdInfo|null|undefined} in_feed
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             */
                            VideoAdInfo.prototype.in_feed = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * VideoAdInfo format.
                             * @member {"in_stream"|"bumper"|"out_stream"|"non_skippable"|"in_feed"|undefined} format
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             */
                            Object.defineProperty(VideoAdInfo.prototype, "format", {
                                get: $util.oneOfGetter($oneOfFields = ["in_stream", "bumper", "out_stream", "non_skippable", "in_feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new VideoAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.VideoAdInfo} VideoAdInfo instance
                             */
                            VideoAdInfo.create = function create(properties) {
                                return new VideoAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified VideoAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.VideoAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoAdInfo} m VideoAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.in_stream != null && Object.hasOwnProperty.call(m, "in_stream"))
                                    $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.encode(m.in_stream, w.uint32(18).fork()).ldelim();
                                if (m.bumper != null && Object.hasOwnProperty.call(m, "bumper"))
                                    $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.encode(m.bumper, w.uint32(26).fork()).ldelim();
                                if (m.out_stream != null && Object.hasOwnProperty.call(m, "out_stream"))
                                    $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo.encode(m.out_stream, w.uint32(34).fork()).ldelim();
                                if (m.non_skippable != null && Object.hasOwnProperty.call(m, "non_skippable"))
                                    $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.encode(m.non_skippable, w.uint32(42).fork()).ldelim();
                                if (m.video != null && Object.hasOwnProperty.call(m, "video"))
                                    $root.google.ads.googleads.v11.common.AdVideoAsset.encode(m.video, w.uint32(66).fork()).ldelim();
                                if (m.in_feed != null && Object.hasOwnProperty.call(m, "in_feed"))
                                    $root.google.ads.googleads.v11.common.InFeedVideoAdInfo.encode(m.in_feed, w.uint32(74).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified VideoAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.VideoAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoAdInfo} message VideoAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VideoAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.VideoAdInfo} VideoAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.VideoAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 8:
                                        m.video = $root.google.ads.googleads.v11.common.AdVideoAsset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.in_stream = $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.bumper = $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.out_stream = $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.non_skippable = $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.in_feed = $root.google.ads.googleads.v11.common.InFeedVideoAdInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VideoAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.VideoAdInfo} VideoAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VideoAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VideoAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.video != null && m.hasOwnProperty("video")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdVideoAsset.verify(m.video);
                                        if (e)
                                            return "video." + e;
                                    }
                                }
                                if (m.in_stream != null && m.hasOwnProperty("in_stream")) {
                                    p.format = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.verify(m.in_stream);
                                        if (e)
                                            return "in_stream." + e;
                                    }
                                }
                                if (m.bumper != null && m.hasOwnProperty("bumper")) {
                                    if (p.format === 1)
                                        return "format: multiple values";
                                    p.format = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.verify(m.bumper);
                                        if (e)
                                            return "bumper." + e;
                                    }
                                }
                                if (m.out_stream != null && m.hasOwnProperty("out_stream")) {
                                    if (p.format === 1)
                                        return "format: multiple values";
                                    p.format = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo.verify(m.out_stream);
                                        if (e)
                                            return "out_stream." + e;
                                    }
                                }
                                if (m.non_skippable != null && m.hasOwnProperty("non_skippable")) {
                                    if (p.format === 1)
                                        return "format: multiple values";
                                    p.format = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.verify(m.non_skippable);
                                        if (e)
                                            return "non_skippable." + e;
                                    }
                                }
                                if (m.in_feed != null && m.hasOwnProperty("in_feed")) {
                                    if (p.format === 1)
                                        return "format: multiple values";
                                    p.format = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.InFeedVideoAdInfo.verify(m.in_feed);
                                        if (e)
                                            return "in_feed." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a VideoAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.VideoAdInfo} VideoAdInfo
                             */
                            VideoAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.VideoAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.VideoAdInfo();
                                if (d.video != null) {
                                    if (typeof d.video !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoAdInfo.video: object expected");
                                    m.video = $root.google.ads.googleads.v11.common.AdVideoAsset.fromObject(d.video);
                                }
                                if (d.in_stream != null) {
                                    if (typeof d.in_stream !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoAdInfo.in_stream: object expected");
                                    m.in_stream = $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.fromObject(d.in_stream);
                                }
                                if (d.bumper != null) {
                                    if (typeof d.bumper !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoAdInfo.bumper: object expected");
                                    m.bumper = $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.fromObject(d.bumper);
                                }
                                if (d.out_stream != null) {
                                    if (typeof d.out_stream !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoAdInfo.out_stream: object expected");
                                    m.out_stream = $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo.fromObject(d.out_stream);
                                }
                                if (d.non_skippable != null) {
                                    if (typeof d.non_skippable !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoAdInfo.non_skippable: object expected");
                                    m.non_skippable = $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.fromObject(d.non_skippable);
                                }
                                if (d.in_feed != null) {
                                    if (typeof d.in_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.VideoAdInfo.in_feed: object expected");
                                    m.in_feed = $root.google.ads.googleads.v11.common.InFeedVideoAdInfo.fromObject(d.in_feed);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a VideoAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.VideoAdInfo} m VideoAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VideoAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.video = null;
                                }
                                if (m.in_stream != null && m.hasOwnProperty("in_stream")) {
                                    d.in_stream = $root.google.ads.googleads.v11.common.VideoTrueViewInStreamAdInfo.toObject(m.in_stream, o);
                                    if (o.oneofs)
                                        d.format = "in_stream";
                                }
                                if (m.bumper != null && m.hasOwnProperty("bumper")) {
                                    d.bumper = $root.google.ads.googleads.v11.common.VideoBumperInStreamAdInfo.toObject(m.bumper, o);
                                    if (o.oneofs)
                                        d.format = "bumper";
                                }
                                if (m.out_stream != null && m.hasOwnProperty("out_stream")) {
                                    d.out_stream = $root.google.ads.googleads.v11.common.VideoOutstreamAdInfo.toObject(m.out_stream, o);
                                    if (o.oneofs)
                                        d.format = "out_stream";
                                }
                                if (m.non_skippable != null && m.hasOwnProperty("non_skippable")) {
                                    d.non_skippable = $root.google.ads.googleads.v11.common.VideoNonSkippableInStreamAdInfo.toObject(m.non_skippable, o);
                                    if (o.oneofs)
                                        d.format = "non_skippable";
                                }
                                if (m.video != null && m.hasOwnProperty("video")) {
                                    d.video = $root.google.ads.googleads.v11.common.AdVideoAsset.toObject(m.video, o);
                                }
                                if (m.in_feed != null && m.hasOwnProperty("in_feed")) {
                                    d.in_feed = $root.google.ads.googleads.v11.common.InFeedVideoAdInfo.toObject(m.in_feed, o);
                                    if (o.oneofs)
                                        d.format = "in_feed";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this VideoAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.VideoAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VideoAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return VideoAdInfo;
                        })();
    
                        common.VideoResponsiveAdInfo = (function() {
    
                            /**
                             * Properties of a VideoResponsiveAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IVideoResponsiveAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] VideoResponsiveAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [long_headlines] VideoResponsiveAdInfo long_headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] VideoResponsiveAdInfo descriptions
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [call_to_actions] VideoResponsiveAdInfo call_to_actions
                             * @property {Array.<google.ads.googleads.v11.common.IAdVideoAsset>|null} [videos] VideoResponsiveAdInfo videos
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [companion_banners] VideoResponsiveAdInfo companion_banners
                             * @property {string|null} [breadcrumb1] VideoResponsiveAdInfo breadcrumb1
                             * @property {string|null} [breadcrumb2] VideoResponsiveAdInfo breadcrumb2
                             */
    
                            /**
                             * Constructs a new VideoResponsiveAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a VideoResponsiveAdInfo.
                             * @implements IVideoResponsiveAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IVideoResponsiveAdInfo=} [p] Properties to set
                             */
                            function VideoResponsiveAdInfo(p) {
                                this.headlines = [];
                                this.long_headlines = [];
                                this.descriptions = [];
                                this.call_to_actions = [];
                                this.videos = [];
                                this.companion_banners = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * VideoResponsiveAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * VideoResponsiveAdInfo long_headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} long_headlines
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.long_headlines = $util.emptyArray;
    
                            /**
                             * VideoResponsiveAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * VideoResponsiveAdInfo call_to_actions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} call_to_actions
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.call_to_actions = $util.emptyArray;
    
                            /**
                             * VideoResponsiveAdInfo videos.
                             * @member {Array.<google.ads.googleads.v11.common.IAdVideoAsset>} videos
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.videos = $util.emptyArray;
    
                            /**
                             * VideoResponsiveAdInfo companion_banners.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} companion_banners
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.companion_banners = $util.emptyArray;
    
                            /**
                             * VideoResponsiveAdInfo breadcrumb1.
                             * @member {string} breadcrumb1
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.breadcrumb1 = "";
    
                            /**
                             * VideoResponsiveAdInfo breadcrumb2.
                             * @member {string} breadcrumb2
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             */
                            VideoResponsiveAdInfo.prototype.breadcrumb2 = "";
    
                            /**
                             * Creates a new VideoResponsiveAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoResponsiveAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.VideoResponsiveAdInfo} VideoResponsiveAdInfo instance
                             */
                            VideoResponsiveAdInfo.create = function create(properties) {
                                return new VideoResponsiveAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified VideoResponsiveAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.VideoResponsiveAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoResponsiveAdInfo} m VideoResponsiveAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoResponsiveAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.long_headlines != null && m.long_headlines.length) {
                                    for (var i = 0; i < m.long_headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.long_headlines[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.call_to_actions != null && m.call_to_actions.length) {
                                    for (var i = 0; i < m.call_to_actions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.call_to_actions[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.videos != null && m.videos.length) {
                                    for (var i = 0; i < m.videos.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdVideoAsset.encode(m.videos[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.companion_banners != null && m.companion_banners.length) {
                                    for (var i = 0; i < m.companion_banners.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.companion_banners[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.breadcrumb1 != null && Object.hasOwnProperty.call(m, "breadcrumb1"))
                                    w.uint32(58).string(m.breadcrumb1);
                                if (m.breadcrumb2 != null && Object.hasOwnProperty.call(m, "breadcrumb2"))
                                    w.uint32(66).string(m.breadcrumb2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified VideoResponsiveAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.VideoResponsiveAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IVideoResponsiveAdInfo} message VideoResponsiveAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoResponsiveAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VideoResponsiveAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.VideoResponsiveAdInfo} VideoResponsiveAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoResponsiveAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.long_headlines && m.long_headlines.length))
                                            m.long_headlines = [];
                                        m.long_headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.call_to_actions && m.call_to_actions.length))
                                            m.call_to_actions = [];
                                        m.call_to_actions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        if (!(m.videos && m.videos.length))
                                            m.videos = [];
                                        m.videos.push($root.google.ads.googleads.v11.common.AdVideoAsset.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        if (!(m.companion_banners && m.companion_banners.length))
                                            m.companion_banners = [];
                                        m.companion_banners.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 7:
                                        m.breadcrumb1 = r.string();
                                        break;
                                    case 8:
                                        m.breadcrumb2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VideoResponsiveAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.VideoResponsiveAdInfo} VideoResponsiveAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoResponsiveAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VideoResponsiveAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VideoResponsiveAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.long_headlines != null && m.hasOwnProperty("long_headlines")) {
                                    if (!Array.isArray(m.long_headlines))
                                        return "long_headlines: array expected";
                                    for (var i = 0; i < m.long_headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.long_headlines[i]);
                                            if (e)
                                                return "long_headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.call_to_actions != null && m.hasOwnProperty("call_to_actions")) {
                                    if (!Array.isArray(m.call_to_actions))
                                        return "call_to_actions: array expected";
                                    for (var i = 0; i < m.call_to_actions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.call_to_actions[i]);
                                            if (e)
                                                return "call_to_actions." + e;
                                        }
                                    }
                                }
                                if (m.videos != null && m.hasOwnProperty("videos")) {
                                    if (!Array.isArray(m.videos))
                                        return "videos: array expected";
                                    for (var i = 0; i < m.videos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdVideoAsset.verify(m.videos[i]);
                                            if (e)
                                                return "videos." + e;
                                        }
                                    }
                                }
                                if (m.companion_banners != null && m.hasOwnProperty("companion_banners")) {
                                    if (!Array.isArray(m.companion_banners))
                                        return "companion_banners: array expected";
                                    for (var i = 0; i < m.companion_banners.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.companion_banners[i]);
                                            if (e)
                                                return "companion_banners." + e;
                                        }
                                    }
                                }
                                if (m.breadcrumb1 != null && m.hasOwnProperty("breadcrumb1")) {
                                    if (!$util.isString(m.breadcrumb1))
                                        return "breadcrumb1: string expected";
                                }
                                if (m.breadcrumb2 != null && m.hasOwnProperty("breadcrumb2")) {
                                    if (!$util.isString(m.breadcrumb2))
                                        return "breadcrumb2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a VideoResponsiveAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.VideoResponsiveAdInfo} VideoResponsiveAdInfo
                             */
                            VideoResponsiveAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo();
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.long_headlines) {
                                    if (!Array.isArray(d.long_headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.long_headlines: array expected");
                                    m.long_headlines = [];
                                    for (var i = 0; i < d.long_headlines.length; ++i) {
                                        if (typeof d.long_headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.long_headlines: object expected");
                                        m.long_headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.long_headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.call_to_actions) {
                                    if (!Array.isArray(d.call_to_actions))
                                        throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.call_to_actions: array expected");
                                    m.call_to_actions = [];
                                    for (var i = 0; i < d.call_to_actions.length; ++i) {
                                        if (typeof d.call_to_actions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.call_to_actions: object expected");
                                        m.call_to_actions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.call_to_actions[i]);
                                    }
                                }
                                if (d.videos) {
                                    if (!Array.isArray(d.videos))
                                        throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.videos: array expected");
                                    m.videos = [];
                                    for (var i = 0; i < d.videos.length; ++i) {
                                        if (typeof d.videos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.videos: object expected");
                                        m.videos[i] = $root.google.ads.googleads.v11.common.AdVideoAsset.fromObject(d.videos[i]);
                                    }
                                }
                                if (d.companion_banners) {
                                    if (!Array.isArray(d.companion_banners))
                                        throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.companion_banners: array expected");
                                    m.companion_banners = [];
                                    for (var i = 0; i < d.companion_banners.length; ++i) {
                                        if (typeof d.companion_banners[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.VideoResponsiveAdInfo.companion_banners: object expected");
                                        m.companion_banners[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.companion_banners[i]);
                                    }
                                }
                                if (d.breadcrumb1 != null) {
                                    m.breadcrumb1 = String(d.breadcrumb1);
                                }
                                if (d.breadcrumb2 != null) {
                                    m.breadcrumb2 = String(d.breadcrumb2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a VideoResponsiveAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.VideoResponsiveAdInfo} m VideoResponsiveAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VideoResponsiveAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.headlines = [];
                                    d.long_headlines = [];
                                    d.descriptions = [];
                                    d.call_to_actions = [];
                                    d.videos = [];
                                    d.companion_banners = [];
                                }
                                if (o.defaults) {
                                    d.breadcrumb1 = "";
                                    d.breadcrumb2 = "";
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.long_headlines && m.long_headlines.length) {
                                    d.long_headlines = [];
                                    for (var j = 0; j < m.long_headlines.length; ++j) {
                                        d.long_headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.long_headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.call_to_actions && m.call_to_actions.length) {
                                    d.call_to_actions = [];
                                    for (var j = 0; j < m.call_to_actions.length; ++j) {
                                        d.call_to_actions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.call_to_actions[j], o);
                                    }
                                }
                                if (m.videos && m.videos.length) {
                                    d.videos = [];
                                    for (var j = 0; j < m.videos.length; ++j) {
                                        d.videos[j] = $root.google.ads.googleads.v11.common.AdVideoAsset.toObject(m.videos[j], o);
                                    }
                                }
                                if (m.companion_banners && m.companion_banners.length) {
                                    d.companion_banners = [];
                                    for (var j = 0; j < m.companion_banners.length; ++j) {
                                        d.companion_banners[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.companion_banners[j], o);
                                    }
                                }
                                if (m.breadcrumb1 != null && m.hasOwnProperty("breadcrumb1")) {
                                    d.breadcrumb1 = m.breadcrumb1;
                                }
                                if (m.breadcrumb2 != null && m.hasOwnProperty("breadcrumb2")) {
                                    d.breadcrumb2 = m.breadcrumb2;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this VideoResponsiveAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.VideoResponsiveAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VideoResponsiveAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return VideoResponsiveAdInfo;
                        })();
    
                        common.ResponsiveSearchAdInfo = (function() {
    
                            /**
                             * Properties of a ResponsiveSearchAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IResponsiveSearchAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] ResponsiveSearchAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] ResponsiveSearchAdInfo descriptions
                             * @property {string|null} [path1] ResponsiveSearchAdInfo path1
                             * @property {string|null} [path2] ResponsiveSearchAdInfo path2
                             */
    
                            /**
                             * Constructs a new ResponsiveSearchAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ResponsiveSearchAdInfo.
                             * @implements IResponsiveSearchAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IResponsiveSearchAdInfo=} [p] Properties to set
                             */
                            function ResponsiveSearchAdInfo(p) {
                                this.headlines = [];
                                this.descriptions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ResponsiveSearchAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @instance
                             */
                            ResponsiveSearchAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * ResponsiveSearchAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @instance
                             */
                            ResponsiveSearchAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * ResponsiveSearchAdInfo path1.
                             * @member {string|null|undefined} path1
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @instance
                             */
                            ResponsiveSearchAdInfo.prototype.path1 = null;
    
                            /**
                             * ResponsiveSearchAdInfo path2.
                             * @member {string|null|undefined} path2
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @instance
                             */
                            ResponsiveSearchAdInfo.prototype.path2 = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ResponsiveSearchAdInfo _path1.
                             * @member {"path1"|undefined} _path1
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveSearchAdInfo.prototype, "_path1", {
                                get: $util.oneOfGetter($oneOfFields = ["path1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ResponsiveSearchAdInfo _path2.
                             * @member {"path2"|undefined} _path2
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveSearchAdInfo.prototype, "_path2", {
                                get: $util.oneOfGetter($oneOfFields = ["path2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ResponsiveSearchAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveSearchAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ResponsiveSearchAdInfo} ResponsiveSearchAdInfo instance
                             */
                            ResponsiveSearchAdInfo.create = function create(properties) {
                                return new ResponsiveSearchAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ResponsiveSearchAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ResponsiveSearchAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveSearchAdInfo} m ResponsiveSearchAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponsiveSearchAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.path1 != null && Object.hasOwnProperty.call(m, "path1"))
                                    w.uint32(42).string(m.path1);
                                if (m.path2 != null && Object.hasOwnProperty.call(m, "path2"))
                                    w.uint32(50).string(m.path2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResponsiveSearchAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ResponsiveSearchAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveSearchAdInfo} message ResponsiveSearchAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponsiveSearchAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResponsiveSearchAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ResponsiveSearchAdInfo} ResponsiveSearchAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponsiveSearchAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        m.path1 = r.string();
                                        break;
                                    case 6:
                                        m.path2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResponsiveSearchAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ResponsiveSearchAdInfo} ResponsiveSearchAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponsiveSearchAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResponsiveSearchAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResponsiveSearchAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    p._path1 = 1;
                                    if (!$util.isString(m.path1))
                                        return "path1: string expected";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    p._path2 = 1;
                                    if (!$util.isString(m.path2))
                                        return "path2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ResponsiveSearchAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ResponsiveSearchAdInfo} ResponsiveSearchAdInfo
                             */
                            ResponsiveSearchAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo();
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveSearchAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveSearchAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveSearchAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveSearchAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.path1 != null) {
                                    m.path1 = String(d.path1);
                                }
                                if (d.path2 != null) {
                                    m.path2 = String(d.path2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ResponsiveSearchAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ResponsiveSearchAdInfo} m ResponsiveSearchAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResponsiveSearchAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.headlines = [];
                                    d.descriptions = [];
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    d.path1 = m.path1;
                                    if (o.oneofs)
                                        d._path1 = "path1";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    d.path2 = m.path2;
                                    if (o.oneofs)
                                        d._path2 = "path2";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ResponsiveSearchAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ResponsiveSearchAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResponsiveSearchAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ResponsiveSearchAdInfo;
                        })();
    
                        common.LegacyResponsiveDisplayAdInfo = (function() {
    
                            /**
                             * Properties of a LegacyResponsiveDisplayAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILegacyResponsiveDisplayAdInfo
                             * @property {string|null} [short_headline] LegacyResponsiveDisplayAdInfo short_headline
                             * @property {string|null} [long_headline] LegacyResponsiveDisplayAdInfo long_headline
                             * @property {string|null} [description] LegacyResponsiveDisplayAdInfo description
                             * @property {string|null} [business_name] LegacyResponsiveDisplayAdInfo business_name
                             * @property {boolean|null} [allow_flexible_color] LegacyResponsiveDisplayAdInfo allow_flexible_color
                             * @property {string|null} [accent_color] LegacyResponsiveDisplayAdInfo accent_color
                             * @property {string|null} [main_color] LegacyResponsiveDisplayAdInfo main_color
                             * @property {string|null} [call_to_action_text] LegacyResponsiveDisplayAdInfo call_to_action_text
                             * @property {string|null} [logo_image] LegacyResponsiveDisplayAdInfo logo_image
                             * @property {string|null} [square_logo_image] LegacyResponsiveDisplayAdInfo square_logo_image
                             * @property {string|null} [marketing_image] LegacyResponsiveDisplayAdInfo marketing_image
                             * @property {string|null} [square_marketing_image] LegacyResponsiveDisplayAdInfo square_marketing_image
                             * @property {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.DisplayAdFormatSetting|null} [format_setting] LegacyResponsiveDisplayAdInfo format_setting
                             * @property {string|null} [price_prefix] LegacyResponsiveDisplayAdInfo price_prefix
                             * @property {string|null} [promo_text] LegacyResponsiveDisplayAdInfo promo_text
                             */
    
                            /**
                             * Constructs a new LegacyResponsiveDisplayAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LegacyResponsiveDisplayAdInfo.
                             * @implements ILegacyResponsiveDisplayAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILegacyResponsiveDisplayAdInfo=} [p] Properties to set
                             */
                            function LegacyResponsiveDisplayAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LegacyResponsiveDisplayAdInfo short_headline.
                             * @member {string|null|undefined} short_headline
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.short_headline = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo long_headline.
                             * @member {string|null|undefined} long_headline
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.long_headline = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.description = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.business_name = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo allow_flexible_color.
                             * @member {boolean|null|undefined} allow_flexible_color
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.allow_flexible_color = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo accent_color.
                             * @member {string|null|undefined} accent_color
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.accent_color = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo main_color.
                             * @member {string|null|undefined} main_color
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.main_color = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo call_to_action_text.
                             * @member {string|null|undefined} call_to_action_text
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.call_to_action_text = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo logo_image.
                             * @member {string|null|undefined} logo_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.logo_image = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo square_logo_image.
                             * @member {string|null|undefined} square_logo_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.square_logo_image = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo marketing_image.
                             * @member {string|null|undefined} marketing_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.marketing_image = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo square_marketing_image.
                             * @member {string|null|undefined} square_marketing_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.square_marketing_image = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo format_setting.
                             * @member {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.DisplayAdFormatSetting} format_setting
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.format_setting = 0;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo price_prefix.
                             * @member {string|null|undefined} price_prefix
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.price_prefix = null;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo promo_text.
                             * @member {string|null|undefined} promo_text
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.promo_text = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _short_headline.
                             * @member {"short_headline"|undefined} _short_headline
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_short_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["short_headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _long_headline.
                             * @member {"long_headline"|undefined} _long_headline
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_long_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["long_headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _business_name.
                             * @member {"business_name"|undefined} _business_name
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_business_name", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _allow_flexible_color.
                             * @member {"allow_flexible_color"|undefined} _allow_flexible_color
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_allow_flexible_color", {
                                get: $util.oneOfGetter($oneOfFields = ["allow_flexible_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _accent_color.
                             * @member {"accent_color"|undefined} _accent_color
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_accent_color", {
                                get: $util.oneOfGetter($oneOfFields = ["accent_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _main_color.
                             * @member {"main_color"|undefined} _main_color
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_main_color", {
                                get: $util.oneOfGetter($oneOfFields = ["main_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _call_to_action_text.
                             * @member {"call_to_action_text"|undefined} _call_to_action_text
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_call_to_action_text", {
                                get: $util.oneOfGetter($oneOfFields = ["call_to_action_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _logo_image.
                             * @member {"logo_image"|undefined} _logo_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_logo_image", {
                                get: $util.oneOfGetter($oneOfFields = ["logo_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _square_logo_image.
                             * @member {"square_logo_image"|undefined} _square_logo_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_square_logo_image", {
                                get: $util.oneOfGetter($oneOfFields = ["square_logo_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _marketing_image.
                             * @member {"marketing_image"|undefined} _marketing_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_marketing_image", {
                                get: $util.oneOfGetter($oneOfFields = ["marketing_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _square_marketing_image.
                             * @member {"square_marketing_image"|undefined} _square_marketing_image
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_square_marketing_image", {
                                get: $util.oneOfGetter($oneOfFields = ["square_marketing_image"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _price_prefix.
                             * @member {"price_prefix"|undefined} _price_prefix
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_price_prefix", {
                                get: $util.oneOfGetter($oneOfFields = ["price_prefix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyResponsiveDisplayAdInfo _promo_text.
                             * @member {"promo_text"|undefined} _promo_text
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyResponsiveDisplayAdInfo.prototype, "_promo_text", {
                                get: $util.oneOfGetter($oneOfFields = ["promo_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LegacyResponsiveDisplayAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILegacyResponsiveDisplayAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo} LegacyResponsiveDisplayAdInfo instance
                             */
                            LegacyResponsiveDisplayAdInfo.create = function create(properties) {
                                return new LegacyResponsiveDisplayAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LegacyResponsiveDisplayAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILegacyResponsiveDisplayAdInfo} m LegacyResponsiveDisplayAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LegacyResponsiveDisplayAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.format_setting != null && Object.hasOwnProperty.call(m, "format_setting"))
                                    w.uint32(104).int32(m.format_setting);
                                if (m.short_headline != null && Object.hasOwnProperty.call(m, "short_headline"))
                                    w.uint32(130).string(m.short_headline);
                                if (m.long_headline != null && Object.hasOwnProperty.call(m, "long_headline"))
                                    w.uint32(138).string(m.long_headline);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(146).string(m.description);
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(154).string(m.business_name);
                                if (m.allow_flexible_color != null && Object.hasOwnProperty.call(m, "allow_flexible_color"))
                                    w.uint32(160).bool(m.allow_flexible_color);
                                if (m.accent_color != null && Object.hasOwnProperty.call(m, "accent_color"))
                                    w.uint32(170).string(m.accent_color);
                                if (m.main_color != null && Object.hasOwnProperty.call(m, "main_color"))
                                    w.uint32(178).string(m.main_color);
                                if (m.call_to_action_text != null && Object.hasOwnProperty.call(m, "call_to_action_text"))
                                    w.uint32(186).string(m.call_to_action_text);
                                if (m.logo_image != null && Object.hasOwnProperty.call(m, "logo_image"))
                                    w.uint32(194).string(m.logo_image);
                                if (m.square_logo_image != null && Object.hasOwnProperty.call(m, "square_logo_image"))
                                    w.uint32(202).string(m.square_logo_image);
                                if (m.marketing_image != null && Object.hasOwnProperty.call(m, "marketing_image"))
                                    w.uint32(210).string(m.marketing_image);
                                if (m.square_marketing_image != null && Object.hasOwnProperty.call(m, "square_marketing_image"))
                                    w.uint32(218).string(m.square_marketing_image);
                                if (m.price_prefix != null && Object.hasOwnProperty.call(m, "price_prefix"))
                                    w.uint32(226).string(m.price_prefix);
                                if (m.promo_text != null && Object.hasOwnProperty.call(m, "promo_text"))
                                    w.uint32(234).string(m.promo_text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LegacyResponsiveDisplayAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILegacyResponsiveDisplayAdInfo} message LegacyResponsiveDisplayAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LegacyResponsiveDisplayAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LegacyResponsiveDisplayAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo} LegacyResponsiveDisplayAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LegacyResponsiveDisplayAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 16:
                                        m.short_headline = r.string();
                                        break;
                                    case 17:
                                        m.long_headline = r.string();
                                        break;
                                    case 18:
                                        m.description = r.string();
                                        break;
                                    case 19:
                                        m.business_name = r.string();
                                        break;
                                    case 20:
                                        m.allow_flexible_color = r.bool();
                                        break;
                                    case 21:
                                        m.accent_color = r.string();
                                        break;
                                    case 22:
                                        m.main_color = r.string();
                                        break;
                                    case 23:
                                        m.call_to_action_text = r.string();
                                        break;
                                    case 24:
                                        m.logo_image = r.string();
                                        break;
                                    case 25:
                                        m.square_logo_image = r.string();
                                        break;
                                    case 26:
                                        m.marketing_image = r.string();
                                        break;
                                    case 27:
                                        m.square_marketing_image = r.string();
                                        break;
                                    case 13:
                                        m.format_setting = r.int32();
                                        break;
                                    case 28:
                                        m.price_prefix = r.string();
                                        break;
                                    case 29:
                                        m.promo_text = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LegacyResponsiveDisplayAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo} LegacyResponsiveDisplayAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LegacyResponsiveDisplayAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LegacyResponsiveDisplayAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LegacyResponsiveDisplayAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.short_headline != null && m.hasOwnProperty("short_headline")) {
                                    p._short_headline = 1;
                                    if (!$util.isString(m.short_headline))
                                        return "short_headline: string expected";
                                }
                                if (m.long_headline != null && m.hasOwnProperty("long_headline")) {
                                    p._long_headline = 1;
                                    if (!$util.isString(m.long_headline))
                                        return "long_headline: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p._business_name = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.allow_flexible_color != null && m.hasOwnProperty("allow_flexible_color")) {
                                    p._allow_flexible_color = 1;
                                    if (typeof m.allow_flexible_color !== "boolean")
                                        return "allow_flexible_color: boolean expected";
                                }
                                if (m.accent_color != null && m.hasOwnProperty("accent_color")) {
                                    p._accent_color = 1;
                                    if (!$util.isString(m.accent_color))
                                        return "accent_color: string expected";
                                }
                                if (m.main_color != null && m.hasOwnProperty("main_color")) {
                                    p._main_color = 1;
                                    if (!$util.isString(m.main_color))
                                        return "main_color: string expected";
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    p._call_to_action_text = 1;
                                    if (!$util.isString(m.call_to_action_text))
                                        return "call_to_action_text: string expected";
                                }
                                if (m.logo_image != null && m.hasOwnProperty("logo_image")) {
                                    p._logo_image = 1;
                                    if (!$util.isString(m.logo_image))
                                        return "logo_image: string expected";
                                }
                                if (m.square_logo_image != null && m.hasOwnProperty("square_logo_image")) {
                                    p._square_logo_image = 1;
                                    if (!$util.isString(m.square_logo_image))
                                        return "square_logo_image: string expected";
                                }
                                if (m.marketing_image != null && m.hasOwnProperty("marketing_image")) {
                                    p._marketing_image = 1;
                                    if (!$util.isString(m.marketing_image))
                                        return "marketing_image: string expected";
                                }
                                if (m.square_marketing_image != null && m.hasOwnProperty("square_marketing_image")) {
                                    p._square_marketing_image = 1;
                                    if (!$util.isString(m.square_marketing_image))
                                        return "square_marketing_image: string expected";
                                }
                                if (m.format_setting != null && m.hasOwnProperty("format_setting")) {
                                    switch (m.format_setting) {
                                    default:
                                        return "format_setting: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.price_prefix != null && m.hasOwnProperty("price_prefix")) {
                                    p._price_prefix = 1;
                                    if (!$util.isString(m.price_prefix))
                                        return "price_prefix: string expected";
                                }
                                if (m.promo_text != null && m.hasOwnProperty("promo_text")) {
                                    p._promo_text = 1;
                                    if (!$util.isString(m.promo_text))
                                        return "promo_text: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LegacyResponsiveDisplayAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo} LegacyResponsiveDisplayAdInfo
                             */
                            LegacyResponsiveDisplayAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo();
                                if (d.short_headline != null) {
                                    m.short_headline = String(d.short_headline);
                                }
                                if (d.long_headline != null) {
                                    m.long_headline = String(d.long_headline);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.allow_flexible_color != null) {
                                    m.allow_flexible_color = Boolean(d.allow_flexible_color);
                                }
                                if (d.accent_color != null) {
                                    m.accent_color = String(d.accent_color);
                                }
                                if (d.main_color != null) {
                                    m.main_color = String(d.main_color);
                                }
                                if (d.call_to_action_text != null) {
                                    m.call_to_action_text = String(d.call_to_action_text);
                                }
                                if (d.logo_image != null) {
                                    m.logo_image = String(d.logo_image);
                                }
                                if (d.square_logo_image != null) {
                                    m.square_logo_image = String(d.square_logo_image);
                                }
                                if (d.marketing_image != null) {
                                    m.marketing_image = String(d.marketing_image);
                                }
                                if (d.square_marketing_image != null) {
                                    m.square_marketing_image = String(d.square_marketing_image);
                                }
                                switch (d.format_setting) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.format_setting = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.format_setting = 1;
                                    break;
                                case "ALL_FORMATS":
                                case 2:
                                    m.format_setting = 2;
                                    break;
                                case "NON_NATIVE":
                                case 3:
                                    m.format_setting = 3;
                                    break;
                                case "NATIVE":
                                case 4:
                                    m.format_setting = 4;
                                    break;
                                }
                                if (d.price_prefix != null) {
                                    m.price_prefix = String(d.price_prefix);
                                }
                                if (d.promo_text != null) {
                                    m.promo_text = String(d.promo_text);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LegacyResponsiveDisplayAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo} m LegacyResponsiveDisplayAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LegacyResponsiveDisplayAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.format_setting = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.format_setting != null && m.hasOwnProperty("format_setting")) {
                                    d.format_setting = o.enums === String ? $root.google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.DisplayAdFormatSetting[m.format_setting] : m.format_setting;
                                }
                                if (m.short_headline != null && m.hasOwnProperty("short_headline")) {
                                    d.short_headline = m.short_headline;
                                    if (o.oneofs)
                                        d._short_headline = "short_headline";
                                }
                                if (m.long_headline != null && m.hasOwnProperty("long_headline")) {
                                    d.long_headline = m.long_headline;
                                    if (o.oneofs)
                                        d._long_headline = "long_headline";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d._business_name = "business_name";
                                }
                                if (m.allow_flexible_color != null && m.hasOwnProperty("allow_flexible_color")) {
                                    d.allow_flexible_color = m.allow_flexible_color;
                                    if (o.oneofs)
                                        d._allow_flexible_color = "allow_flexible_color";
                                }
                                if (m.accent_color != null && m.hasOwnProperty("accent_color")) {
                                    d.accent_color = m.accent_color;
                                    if (o.oneofs)
                                        d._accent_color = "accent_color";
                                }
                                if (m.main_color != null && m.hasOwnProperty("main_color")) {
                                    d.main_color = m.main_color;
                                    if (o.oneofs)
                                        d._main_color = "main_color";
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    d.call_to_action_text = m.call_to_action_text;
                                    if (o.oneofs)
                                        d._call_to_action_text = "call_to_action_text";
                                }
                                if (m.logo_image != null && m.hasOwnProperty("logo_image")) {
                                    d.logo_image = m.logo_image;
                                    if (o.oneofs)
                                        d._logo_image = "logo_image";
                                }
                                if (m.square_logo_image != null && m.hasOwnProperty("square_logo_image")) {
                                    d.square_logo_image = m.square_logo_image;
                                    if (o.oneofs)
                                        d._square_logo_image = "square_logo_image";
                                }
                                if (m.marketing_image != null && m.hasOwnProperty("marketing_image")) {
                                    d.marketing_image = m.marketing_image;
                                    if (o.oneofs)
                                        d._marketing_image = "marketing_image";
                                }
                                if (m.square_marketing_image != null && m.hasOwnProperty("square_marketing_image")) {
                                    d.square_marketing_image = m.square_marketing_image;
                                    if (o.oneofs)
                                        d._square_marketing_image = "square_marketing_image";
                                }
                                if (m.price_prefix != null && m.hasOwnProperty("price_prefix")) {
                                    d.price_prefix = m.price_prefix;
                                    if (o.oneofs)
                                        d._price_prefix = "price_prefix";
                                }
                                if (m.promo_text != null && m.hasOwnProperty("promo_text")) {
                                    d.promo_text = m.promo_text;
                                    if (o.oneofs)
                                        d._promo_text = "promo_text";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LegacyResponsiveDisplayAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LegacyResponsiveDisplayAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LegacyResponsiveDisplayAdInfo;
                        })();
    
                        common.AppAdInfo = (function() {
    
                            /**
                             * Properties of an AppAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAppAdInfo
                             * @property {google.ads.googleads.v11.common.IAdTextAsset|null} [mandatory_ad_text] AppAdInfo mandatory_ad_text
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] AppAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] AppAdInfo descriptions
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [images] AppAdInfo images
                             * @property {Array.<google.ads.googleads.v11.common.IAdVideoAsset>|null} [youtube_videos] AppAdInfo youtube_videos
                             * @property {Array.<google.ads.googleads.v11.common.IAdMediaBundleAsset>|null} [html5_media_bundles] AppAdInfo html5_media_bundles
                             */
    
                            /**
                             * Constructs a new AppAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AppAdInfo.
                             * @implements IAppAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAppAdInfo=} [p] Properties to set
                             */
                            function AppAdInfo(p) {
                                this.headlines = [];
                                this.descriptions = [];
                                this.images = [];
                                this.youtube_videos = [];
                                this.html5_media_bundles = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AppAdInfo mandatory_ad_text.
                             * @member {google.ads.googleads.v11.common.IAdTextAsset|null|undefined} mandatory_ad_text
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @instance
                             */
                            AppAdInfo.prototype.mandatory_ad_text = null;
    
                            /**
                             * AppAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @instance
                             */
                            AppAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * AppAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @instance
                             */
                            AppAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * AppAdInfo images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} images
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @instance
                             */
                            AppAdInfo.prototype.images = $util.emptyArray;
    
                            /**
                             * AppAdInfo youtube_videos.
                             * @member {Array.<google.ads.googleads.v11.common.IAdVideoAsset>} youtube_videos
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @instance
                             */
                            AppAdInfo.prototype.youtube_videos = $util.emptyArray;
    
                            /**
                             * AppAdInfo html5_media_bundles.
                             * @member {Array.<google.ads.googleads.v11.common.IAdMediaBundleAsset>} html5_media_bundles
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @instance
                             */
                            AppAdInfo.prototype.html5_media_bundles = $util.emptyArray;
    
                            /**
                             * Creates a new AppAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AppAdInfo} AppAdInfo instance
                             */
                            AppAdInfo.create = function create(properties) {
                                return new AppAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AppAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AppAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppAdInfo} m AppAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.mandatory_ad_text != null && Object.hasOwnProperty.call(m, "mandatory_ad_text"))
                                    $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.mandatory_ad_text, w.uint32(10).fork()).ldelim();
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.images != null && m.images.length) {
                                    for (var i = 0; i < m.images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.images[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.youtube_videos != null && m.youtube_videos.length) {
                                    for (var i = 0; i < m.youtube_videos.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdVideoAsset.encode(m.youtube_videos[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.html5_media_bundles != null && m.html5_media_bundles.length) {
                                    for (var i = 0; i < m.html5_media_bundles.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdMediaBundleAsset.encode(m.html5_media_bundles[i], w.uint32(50).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AppAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppAdInfo} message AppAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AppAdInfo} AppAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AppAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.mandatory_ad_text = $root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.images && m.images.length))
                                            m.images = [];
                                        m.images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        if (!(m.youtube_videos && m.youtube_videos.length))
                                            m.youtube_videos = [];
                                        m.youtube_videos.push($root.google.ads.googleads.v11.common.AdVideoAsset.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        if (!(m.html5_media_bundles && m.html5_media_bundles.length))
                                            m.html5_media_bundles = [];
                                        m.html5_media_bundles.push($root.google.ads.googleads.v11.common.AdMediaBundleAsset.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AppAdInfo} AppAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.mandatory_ad_text != null && m.hasOwnProperty("mandatory_ad_text")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.mandatory_ad_text);
                                        if (e)
                                            return "mandatory_ad_text." + e;
                                    }
                                }
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.images != null && m.hasOwnProperty("images")) {
                                    if (!Array.isArray(m.images))
                                        return "images: array expected";
                                    for (var i = 0; i < m.images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.images[i]);
                                            if (e)
                                                return "images." + e;
                                        }
                                    }
                                }
                                if (m.youtube_videos != null && m.hasOwnProperty("youtube_videos")) {
                                    if (!Array.isArray(m.youtube_videos))
                                        return "youtube_videos: array expected";
                                    for (var i = 0; i < m.youtube_videos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdVideoAsset.verify(m.youtube_videos[i]);
                                            if (e)
                                                return "youtube_videos." + e;
                                        }
                                    }
                                }
                                if (m.html5_media_bundles != null && m.hasOwnProperty("html5_media_bundles")) {
                                    if (!Array.isArray(m.html5_media_bundles))
                                        return "html5_media_bundles: array expected";
                                    for (var i = 0; i < m.html5_media_bundles.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdMediaBundleAsset.verify(m.html5_media_bundles[i]);
                                            if (e)
                                                return "html5_media_bundles." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AppAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AppAdInfo} AppAdInfo
                             */
                            AppAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AppAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AppAdInfo();
                                if (d.mandatory_ad_text != null) {
                                    if (typeof d.mandatory_ad_text !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.mandatory_ad_text: object expected");
                                    m.mandatory_ad_text = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.mandatory_ad_text);
                                }
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.images) {
                                    if (!Array.isArray(d.images))
                                        throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.images: array expected");
                                    m.images = [];
                                    for (var i = 0; i < d.images.length; ++i) {
                                        if (typeof d.images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.images: object expected");
                                        m.images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.images[i]);
                                    }
                                }
                                if (d.youtube_videos) {
                                    if (!Array.isArray(d.youtube_videos))
                                        throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.youtube_videos: array expected");
                                    m.youtube_videos = [];
                                    for (var i = 0; i < d.youtube_videos.length; ++i) {
                                        if (typeof d.youtube_videos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.youtube_videos: object expected");
                                        m.youtube_videos[i] = $root.google.ads.googleads.v11.common.AdVideoAsset.fromObject(d.youtube_videos[i]);
                                    }
                                }
                                if (d.html5_media_bundles) {
                                    if (!Array.isArray(d.html5_media_bundles))
                                        throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.html5_media_bundles: array expected");
                                    m.html5_media_bundles = [];
                                    for (var i = 0; i < d.html5_media_bundles.length; ++i) {
                                        if (typeof d.html5_media_bundles[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppAdInfo.html5_media_bundles: object expected");
                                        m.html5_media_bundles[i] = $root.google.ads.googleads.v11.common.AdMediaBundleAsset.fromObject(d.html5_media_bundles[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AppAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AppAdInfo} m AppAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.headlines = [];
                                    d.descriptions = [];
                                    d.images = [];
                                    d.youtube_videos = [];
                                    d.html5_media_bundles = [];
                                }
                                if (o.defaults) {
                                    d.mandatory_ad_text = null;
                                }
                                if (m.mandatory_ad_text != null && m.hasOwnProperty("mandatory_ad_text")) {
                                    d.mandatory_ad_text = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.mandatory_ad_text, o);
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.images && m.images.length) {
                                    d.images = [];
                                    for (var j = 0; j < m.images.length; ++j) {
                                        d.images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.images[j], o);
                                    }
                                }
                                if (m.youtube_videos && m.youtube_videos.length) {
                                    d.youtube_videos = [];
                                    for (var j = 0; j < m.youtube_videos.length; ++j) {
                                        d.youtube_videos[j] = $root.google.ads.googleads.v11.common.AdVideoAsset.toObject(m.youtube_videos[j], o);
                                    }
                                }
                                if (m.html5_media_bundles && m.html5_media_bundles.length) {
                                    d.html5_media_bundles = [];
                                    for (var j = 0; j < m.html5_media_bundles.length; ++j) {
                                        d.html5_media_bundles[j] = $root.google.ads.googleads.v11.common.AdMediaBundleAsset.toObject(m.html5_media_bundles[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AppAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AppAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AppAdInfo;
                        })();
    
                        common.AppEngagementAdInfo = (function() {
    
                            /**
                             * Properties of an AppEngagementAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAppEngagementAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] AppEngagementAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] AppEngagementAdInfo descriptions
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [images] AppEngagementAdInfo images
                             * @property {Array.<google.ads.googleads.v11.common.IAdVideoAsset>|null} [videos] AppEngagementAdInfo videos
                             */
    
                            /**
                             * Constructs a new AppEngagementAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AppEngagementAdInfo.
                             * @implements IAppEngagementAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAppEngagementAdInfo=} [p] Properties to set
                             */
                            function AppEngagementAdInfo(p) {
                                this.headlines = [];
                                this.descriptions = [];
                                this.images = [];
                                this.videos = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AppEngagementAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @instance
                             */
                            AppEngagementAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * AppEngagementAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @instance
                             */
                            AppEngagementAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * AppEngagementAdInfo images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} images
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @instance
                             */
                            AppEngagementAdInfo.prototype.images = $util.emptyArray;
    
                            /**
                             * AppEngagementAdInfo videos.
                             * @member {Array.<google.ads.googleads.v11.common.IAdVideoAsset>} videos
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @instance
                             */
                            AppEngagementAdInfo.prototype.videos = $util.emptyArray;
    
                            /**
                             * Creates a new AppEngagementAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppEngagementAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AppEngagementAdInfo} AppEngagementAdInfo instance
                             */
                            AppEngagementAdInfo.create = function create(properties) {
                                return new AppEngagementAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AppEngagementAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AppEngagementAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppEngagementAdInfo} m AppEngagementAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppEngagementAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.images != null && m.images.length) {
                                    for (var i = 0; i < m.images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.images[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.videos != null && m.videos.length) {
                                    for (var i = 0; i < m.videos.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdVideoAsset.encode(m.videos[i], w.uint32(34).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppEngagementAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AppEngagementAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppEngagementAdInfo} message AppEngagementAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppEngagementAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppEngagementAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AppEngagementAdInfo} AppEngagementAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppEngagementAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AppEngagementAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.images && m.images.length))
                                            m.images = [];
                                        m.images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.videos && m.videos.length))
                                            m.videos = [];
                                        m.videos.push($root.google.ads.googleads.v11.common.AdVideoAsset.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppEngagementAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AppEngagementAdInfo} AppEngagementAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppEngagementAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppEngagementAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppEngagementAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.images != null && m.hasOwnProperty("images")) {
                                    if (!Array.isArray(m.images))
                                        return "images: array expected";
                                    for (var i = 0; i < m.images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.images[i]);
                                            if (e)
                                                return "images." + e;
                                        }
                                    }
                                }
                                if (m.videos != null && m.hasOwnProperty("videos")) {
                                    if (!Array.isArray(m.videos))
                                        return "videos: array expected";
                                    for (var i = 0; i < m.videos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdVideoAsset.verify(m.videos[i]);
                                            if (e)
                                                return "videos." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AppEngagementAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AppEngagementAdInfo} AppEngagementAdInfo
                             */
                            AppEngagementAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AppEngagementAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AppEngagementAdInfo();
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.images) {
                                    if (!Array.isArray(d.images))
                                        throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.images: array expected");
                                    m.images = [];
                                    for (var i = 0; i < d.images.length; ++i) {
                                        if (typeof d.images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.images: object expected");
                                        m.images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.images[i]);
                                    }
                                }
                                if (d.videos) {
                                    if (!Array.isArray(d.videos))
                                        throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.videos: array expected");
                                    m.videos = [];
                                    for (var i = 0; i < d.videos.length; ++i) {
                                        if (typeof d.videos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppEngagementAdInfo.videos: object expected");
                                        m.videos[i] = $root.google.ads.googleads.v11.common.AdVideoAsset.fromObject(d.videos[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AppEngagementAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AppEngagementAdInfo} m AppEngagementAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppEngagementAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.headlines = [];
                                    d.descriptions = [];
                                    d.images = [];
                                    d.videos = [];
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.images && m.images.length) {
                                    d.images = [];
                                    for (var j = 0; j < m.images.length; ++j) {
                                        d.images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.images[j], o);
                                    }
                                }
                                if (m.videos && m.videos.length) {
                                    d.videos = [];
                                    for (var j = 0; j < m.videos.length; ++j) {
                                        d.videos[j] = $root.google.ads.googleads.v11.common.AdVideoAsset.toObject(m.videos[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AppEngagementAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AppEngagementAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppEngagementAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AppEngagementAdInfo;
                        })();
    
                        common.AppPreRegistrationAdInfo = (function() {
    
                            /**
                             * Properties of an AppPreRegistrationAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAppPreRegistrationAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] AppPreRegistrationAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] AppPreRegistrationAdInfo descriptions
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [images] AppPreRegistrationAdInfo images
                             * @property {Array.<google.ads.googleads.v11.common.IAdVideoAsset>|null} [youtube_videos] AppPreRegistrationAdInfo youtube_videos
                             */
    
                            /**
                             * Constructs a new AppPreRegistrationAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AppPreRegistrationAdInfo.
                             * @implements IAppPreRegistrationAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAppPreRegistrationAdInfo=} [p] Properties to set
                             */
                            function AppPreRegistrationAdInfo(p) {
                                this.headlines = [];
                                this.descriptions = [];
                                this.images = [];
                                this.youtube_videos = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AppPreRegistrationAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @instance
                             */
                            AppPreRegistrationAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * AppPreRegistrationAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @instance
                             */
                            AppPreRegistrationAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * AppPreRegistrationAdInfo images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} images
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @instance
                             */
                            AppPreRegistrationAdInfo.prototype.images = $util.emptyArray;
    
                            /**
                             * AppPreRegistrationAdInfo youtube_videos.
                             * @member {Array.<google.ads.googleads.v11.common.IAdVideoAsset>} youtube_videos
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @instance
                             */
                            AppPreRegistrationAdInfo.prototype.youtube_videos = $util.emptyArray;
    
                            /**
                             * Creates a new AppPreRegistrationAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppPreRegistrationAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AppPreRegistrationAdInfo} AppPreRegistrationAdInfo instance
                             */
                            AppPreRegistrationAdInfo.create = function create(properties) {
                                return new AppPreRegistrationAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AppPreRegistrationAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AppPreRegistrationAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppPreRegistrationAdInfo} m AppPreRegistrationAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPreRegistrationAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.images != null && m.images.length) {
                                    for (var i = 0; i < m.images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.images[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.youtube_videos != null && m.youtube_videos.length) {
                                    for (var i = 0; i < m.youtube_videos.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdVideoAsset.encode(m.youtube_videos[i], w.uint32(34).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppPreRegistrationAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AppPreRegistrationAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppPreRegistrationAdInfo} message AppPreRegistrationAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPreRegistrationAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppPreRegistrationAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AppPreRegistrationAdInfo} AppPreRegistrationAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPreRegistrationAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.images && m.images.length))
                                            m.images = [];
                                        m.images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.youtube_videos && m.youtube_videos.length))
                                            m.youtube_videos = [];
                                        m.youtube_videos.push($root.google.ads.googleads.v11.common.AdVideoAsset.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppPreRegistrationAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AppPreRegistrationAdInfo} AppPreRegistrationAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPreRegistrationAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppPreRegistrationAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppPreRegistrationAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.images != null && m.hasOwnProperty("images")) {
                                    if (!Array.isArray(m.images))
                                        return "images: array expected";
                                    for (var i = 0; i < m.images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.images[i]);
                                            if (e)
                                                return "images." + e;
                                        }
                                    }
                                }
                                if (m.youtube_videos != null && m.hasOwnProperty("youtube_videos")) {
                                    if (!Array.isArray(m.youtube_videos))
                                        return "youtube_videos: array expected";
                                    for (var i = 0; i < m.youtube_videos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdVideoAsset.verify(m.youtube_videos[i]);
                                            if (e)
                                                return "youtube_videos." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AppPreRegistrationAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AppPreRegistrationAdInfo} AppPreRegistrationAdInfo
                             */
                            AppPreRegistrationAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo();
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.images) {
                                    if (!Array.isArray(d.images))
                                        throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.images: array expected");
                                    m.images = [];
                                    for (var i = 0; i < d.images.length; ++i) {
                                        if (typeof d.images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.images: object expected");
                                        m.images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.images[i]);
                                    }
                                }
                                if (d.youtube_videos) {
                                    if (!Array.isArray(d.youtube_videos))
                                        throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.youtube_videos: array expected");
                                    m.youtube_videos = [];
                                    for (var i = 0; i < d.youtube_videos.length; ++i) {
                                        if (typeof d.youtube_videos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppPreRegistrationAdInfo.youtube_videos: object expected");
                                        m.youtube_videos[i] = $root.google.ads.googleads.v11.common.AdVideoAsset.fromObject(d.youtube_videos[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AppPreRegistrationAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AppPreRegistrationAdInfo} m AppPreRegistrationAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppPreRegistrationAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.headlines = [];
                                    d.descriptions = [];
                                    d.images = [];
                                    d.youtube_videos = [];
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.images && m.images.length) {
                                    d.images = [];
                                    for (var j = 0; j < m.images.length; ++j) {
                                        d.images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.images[j], o);
                                    }
                                }
                                if (m.youtube_videos && m.youtube_videos.length) {
                                    d.youtube_videos = [];
                                    for (var j = 0; j < m.youtube_videos.length; ++j) {
                                        d.youtube_videos[j] = $root.google.ads.googleads.v11.common.AdVideoAsset.toObject(m.youtube_videos[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AppPreRegistrationAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AppPreRegistrationAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppPreRegistrationAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AppPreRegistrationAdInfo;
                        })();
    
                        common.LegacyAppInstallAdInfo = (function() {
    
                            /**
                             * Properties of a LegacyAppInstallAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILegacyAppInstallAdInfo
                             * @property {string|null} [app_id] LegacyAppInstallAdInfo app_id
                             * @property {google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum.LegacyAppInstallAdAppStore|null} [app_store] LegacyAppInstallAdInfo app_store
                             * @property {string|null} [headline] LegacyAppInstallAdInfo headline
                             * @property {string|null} [description1] LegacyAppInstallAdInfo description1
                             * @property {string|null} [description2] LegacyAppInstallAdInfo description2
                             */
    
                            /**
                             * Constructs a new LegacyAppInstallAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LegacyAppInstallAdInfo.
                             * @implements ILegacyAppInstallAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILegacyAppInstallAdInfo=} [p] Properties to set
                             */
                            function LegacyAppInstallAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LegacyAppInstallAdInfo app_id.
                             * @member {string|null|undefined} app_id
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            LegacyAppInstallAdInfo.prototype.app_id = null;
    
                            /**
                             * LegacyAppInstallAdInfo app_store.
                             * @member {google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum.LegacyAppInstallAdAppStore} app_store
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            LegacyAppInstallAdInfo.prototype.app_store = 0;
    
                            /**
                             * LegacyAppInstallAdInfo headline.
                             * @member {string|null|undefined} headline
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            LegacyAppInstallAdInfo.prototype.headline = null;
    
                            /**
                             * LegacyAppInstallAdInfo description1.
                             * @member {string|null|undefined} description1
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            LegacyAppInstallAdInfo.prototype.description1 = null;
    
                            /**
                             * LegacyAppInstallAdInfo description2.
                             * @member {string|null|undefined} description2
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            LegacyAppInstallAdInfo.prototype.description2 = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LegacyAppInstallAdInfo _app_id.
                             * @member {"app_id"|undefined} _app_id
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyAppInstallAdInfo.prototype, "_app_id", {
                                get: $util.oneOfGetter($oneOfFields = ["app_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyAppInstallAdInfo _headline.
                             * @member {"headline"|undefined} _headline
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyAppInstallAdInfo.prototype, "_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyAppInstallAdInfo _description1.
                             * @member {"description1"|undefined} _description1
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyAppInstallAdInfo.prototype, "_description1", {
                                get: $util.oneOfGetter($oneOfFields = ["description1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LegacyAppInstallAdInfo _description2.
                             * @member {"description2"|undefined} _description2
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             */
                            Object.defineProperty(LegacyAppInstallAdInfo.prototype, "_description2", {
                                get: $util.oneOfGetter($oneOfFields = ["description2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LegacyAppInstallAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILegacyAppInstallAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LegacyAppInstallAdInfo} LegacyAppInstallAdInfo instance
                             */
                            LegacyAppInstallAdInfo.create = function create(properties) {
                                return new LegacyAppInstallAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LegacyAppInstallAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LegacyAppInstallAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILegacyAppInstallAdInfo} m LegacyAppInstallAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LegacyAppInstallAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.app_store != null && Object.hasOwnProperty.call(m, "app_store"))
                                    w.uint32(16).int32(m.app_store);
                                if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                    w.uint32(50).string(m.app_id);
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(58).string(m.headline);
                                if (m.description1 != null && Object.hasOwnProperty.call(m, "description1"))
                                    w.uint32(66).string(m.description1);
                                if (m.description2 != null && Object.hasOwnProperty.call(m, "description2"))
                                    w.uint32(74).string(m.description2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LegacyAppInstallAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LegacyAppInstallAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILegacyAppInstallAdInfo} message LegacyAppInstallAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LegacyAppInstallAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LegacyAppInstallAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LegacyAppInstallAdInfo} LegacyAppInstallAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LegacyAppInstallAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.app_id = r.string();
                                        break;
                                    case 2:
                                        m.app_store = r.int32();
                                        break;
                                    case 7:
                                        m.headline = r.string();
                                        break;
                                    case 8:
                                        m.description1 = r.string();
                                        break;
                                    case 9:
                                        m.description2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LegacyAppInstallAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LegacyAppInstallAdInfo} LegacyAppInstallAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LegacyAppInstallAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LegacyAppInstallAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LegacyAppInstallAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    p._app_id = 1;
                                    if (!$util.isString(m.app_id))
                                        return "app_id: string expected";
                                }
                                if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                    switch (m.app_store) {
                                    default:
                                        return "app_store: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    p._headline = 1;
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    p._description1 = 1;
                                    if (!$util.isString(m.description1))
                                        return "description1: string expected";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    p._description2 = 1;
                                    if (!$util.isString(m.description2))
                                        return "description2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LegacyAppInstallAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LegacyAppInstallAdInfo} LegacyAppInstallAdInfo
                             */
                            LegacyAppInstallAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo();
                                if (d.app_id != null) {
                                    m.app_id = String(d.app_id);
                                }
                                switch (d.app_store) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.app_store = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.app_store = 1;
                                    break;
                                case "APPLE_APP_STORE":
                                case 2:
                                    m.app_store = 2;
                                    break;
                                case "GOOGLE_PLAY":
                                case 3:
                                    m.app_store = 3;
                                    break;
                                case "WINDOWS_STORE":
                                case 4:
                                    m.app_store = 4;
                                    break;
                                case "WINDOWS_PHONE_STORE":
                                case 5:
                                    m.app_store = 5;
                                    break;
                                case "CN_APP_STORE":
                                case 6:
                                    m.app_store = 6;
                                    break;
                                }
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.description1 != null) {
                                    m.description1 = String(d.description1);
                                }
                                if (d.description2 != null) {
                                    m.description2 = String(d.description2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LegacyAppInstallAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LegacyAppInstallAdInfo} m LegacyAppInstallAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LegacyAppInstallAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.app_store = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                    d.app_store = o.enums === String ? $root.google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum.LegacyAppInstallAdAppStore[m.app_store] : m.app_store;
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    d.app_id = m.app_id;
                                    if (o.oneofs)
                                        d._app_id = "app_id";
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                    if (o.oneofs)
                                        d._headline = "headline";
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    d.description1 = m.description1;
                                    if (o.oneofs)
                                        d._description1 = "description1";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    d.description2 = m.description2;
                                    if (o.oneofs)
                                        d._description2 = "description2";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LegacyAppInstallAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LegacyAppInstallAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LegacyAppInstallAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LegacyAppInstallAdInfo;
                        })();
    
                        common.ResponsiveDisplayAdInfo = (function() {
    
                            /**
                             * Properties of a ResponsiveDisplayAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IResponsiveDisplayAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [marketing_images] ResponsiveDisplayAdInfo marketing_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [square_marketing_images] ResponsiveDisplayAdInfo square_marketing_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [logo_images] ResponsiveDisplayAdInfo logo_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [square_logo_images] ResponsiveDisplayAdInfo square_logo_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] ResponsiveDisplayAdInfo headlines
                             * @property {google.ads.googleads.v11.common.IAdTextAsset|null} [long_headline] ResponsiveDisplayAdInfo long_headline
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] ResponsiveDisplayAdInfo descriptions
                             * @property {Array.<google.ads.googleads.v11.common.IAdVideoAsset>|null} [youtube_videos] ResponsiveDisplayAdInfo youtube_videos
                             * @property {string|null} [business_name] ResponsiveDisplayAdInfo business_name
                             * @property {string|null} [main_color] ResponsiveDisplayAdInfo main_color
                             * @property {string|null} [accent_color] ResponsiveDisplayAdInfo accent_color
                             * @property {boolean|null} [allow_flexible_color] ResponsiveDisplayAdInfo allow_flexible_color
                             * @property {string|null} [call_to_action_text] ResponsiveDisplayAdInfo call_to_action_text
                             * @property {string|null} [price_prefix] ResponsiveDisplayAdInfo price_prefix
                             * @property {string|null} [promo_text] ResponsiveDisplayAdInfo promo_text
                             * @property {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.DisplayAdFormatSetting|null} [format_setting] ResponsiveDisplayAdInfo format_setting
                             * @property {google.ads.googleads.v11.common.IResponsiveDisplayAdControlSpec|null} [control_spec] ResponsiveDisplayAdInfo control_spec
                             */
    
                            /**
                             * Constructs a new ResponsiveDisplayAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ResponsiveDisplayAdInfo.
                             * @implements IResponsiveDisplayAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdInfo=} [p] Properties to set
                             */
                            function ResponsiveDisplayAdInfo(p) {
                                this.marketing_images = [];
                                this.square_marketing_images = [];
                                this.logo_images = [];
                                this.square_logo_images = [];
                                this.headlines = [];
                                this.descriptions = [];
                                this.youtube_videos = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ResponsiveDisplayAdInfo marketing_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} marketing_images
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.marketing_images = $util.emptyArray;
    
                            /**
                             * ResponsiveDisplayAdInfo square_marketing_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} square_marketing_images
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.square_marketing_images = $util.emptyArray;
    
                            /**
                             * ResponsiveDisplayAdInfo logo_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} logo_images
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.logo_images = $util.emptyArray;
    
                            /**
                             * ResponsiveDisplayAdInfo square_logo_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} square_logo_images
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.square_logo_images = $util.emptyArray;
    
                            /**
                             * ResponsiveDisplayAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * ResponsiveDisplayAdInfo long_headline.
                             * @member {google.ads.googleads.v11.common.IAdTextAsset|null|undefined} long_headline
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.long_headline = null;
    
                            /**
                             * ResponsiveDisplayAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * ResponsiveDisplayAdInfo youtube_videos.
                             * @member {Array.<google.ads.googleads.v11.common.IAdVideoAsset>} youtube_videos
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.youtube_videos = $util.emptyArray;
    
                            /**
                             * ResponsiveDisplayAdInfo business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.business_name = null;
    
                            /**
                             * ResponsiveDisplayAdInfo main_color.
                             * @member {string|null|undefined} main_color
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.main_color = null;
    
                            /**
                             * ResponsiveDisplayAdInfo accent_color.
                             * @member {string|null|undefined} accent_color
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.accent_color = null;
    
                            /**
                             * ResponsiveDisplayAdInfo allow_flexible_color.
                             * @member {boolean|null|undefined} allow_flexible_color
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.allow_flexible_color = null;
    
                            /**
                             * ResponsiveDisplayAdInfo call_to_action_text.
                             * @member {string|null|undefined} call_to_action_text
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.call_to_action_text = null;
    
                            /**
                             * ResponsiveDisplayAdInfo price_prefix.
                             * @member {string|null|undefined} price_prefix
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.price_prefix = null;
    
                            /**
                             * ResponsiveDisplayAdInfo promo_text.
                             * @member {string|null|undefined} promo_text
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.promo_text = null;
    
                            /**
                             * ResponsiveDisplayAdInfo format_setting.
                             * @member {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.DisplayAdFormatSetting} format_setting
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.format_setting = 0;
    
                            /**
                             * ResponsiveDisplayAdInfo control_spec.
                             * @member {google.ads.googleads.v11.common.IResponsiveDisplayAdControlSpec|null|undefined} control_spec
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            ResponsiveDisplayAdInfo.prototype.control_spec = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ResponsiveDisplayAdInfo _business_name.
                             * @member {"business_name"|undefined} _business_name
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveDisplayAdInfo.prototype, "_business_name", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ResponsiveDisplayAdInfo _main_color.
                             * @member {"main_color"|undefined} _main_color
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveDisplayAdInfo.prototype, "_main_color", {
                                get: $util.oneOfGetter($oneOfFields = ["main_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ResponsiveDisplayAdInfo _accent_color.
                             * @member {"accent_color"|undefined} _accent_color
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveDisplayAdInfo.prototype, "_accent_color", {
                                get: $util.oneOfGetter($oneOfFields = ["accent_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ResponsiveDisplayAdInfo _allow_flexible_color.
                             * @member {"allow_flexible_color"|undefined} _allow_flexible_color
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveDisplayAdInfo.prototype, "_allow_flexible_color", {
                                get: $util.oneOfGetter($oneOfFields = ["allow_flexible_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ResponsiveDisplayAdInfo _call_to_action_text.
                             * @member {"call_to_action_text"|undefined} _call_to_action_text
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveDisplayAdInfo.prototype, "_call_to_action_text", {
                                get: $util.oneOfGetter($oneOfFields = ["call_to_action_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ResponsiveDisplayAdInfo _price_prefix.
                             * @member {"price_prefix"|undefined} _price_prefix
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveDisplayAdInfo.prototype, "_price_prefix", {
                                get: $util.oneOfGetter($oneOfFields = ["price_prefix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ResponsiveDisplayAdInfo _promo_text.
                             * @member {"promo_text"|undefined} _promo_text
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             */
                            Object.defineProperty(ResponsiveDisplayAdInfo.prototype, "_promo_text", {
                                get: $util.oneOfGetter($oneOfFields = ["promo_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ResponsiveDisplayAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdInfo} ResponsiveDisplayAdInfo instance
                             */
                            ResponsiveDisplayAdInfo.create = function create(properties) {
                                return new ResponsiveDisplayAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ResponsiveDisplayAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdInfo} m ResponsiveDisplayAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponsiveDisplayAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.marketing_images != null && m.marketing_images.length) {
                                    for (var i = 0; i < m.marketing_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.marketing_images[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.square_marketing_images != null && m.square_marketing_images.length) {
                                    for (var i = 0; i < m.square_marketing_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.square_marketing_images[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.logo_images != null && m.logo_images.length) {
                                    for (var i = 0; i < m.logo_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.logo_images[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.square_logo_images != null && m.square_logo_images.length) {
                                    for (var i = 0; i < m.square_logo_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.square_logo_images[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.long_headline != null && Object.hasOwnProperty.call(m, "long_headline"))
                                    $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.long_headline, w.uint32(50).fork()).ldelim();
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.youtube_videos != null && m.youtube_videos.length) {
                                    for (var i = 0; i < m.youtube_videos.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdVideoAsset.encode(m.youtube_videos[i], w.uint32(66).fork()).ldelim();
                                }
                                if (m.format_setting != null && Object.hasOwnProperty.call(m, "format_setting"))
                                    w.uint32(128).int32(m.format_setting);
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(138).string(m.business_name);
                                if (m.main_color != null && Object.hasOwnProperty.call(m, "main_color"))
                                    w.uint32(146).string(m.main_color);
                                if (m.accent_color != null && Object.hasOwnProperty.call(m, "accent_color"))
                                    w.uint32(154).string(m.accent_color);
                                if (m.allow_flexible_color != null && Object.hasOwnProperty.call(m, "allow_flexible_color"))
                                    w.uint32(160).bool(m.allow_flexible_color);
                                if (m.call_to_action_text != null && Object.hasOwnProperty.call(m, "call_to_action_text"))
                                    w.uint32(170).string(m.call_to_action_text);
                                if (m.price_prefix != null && Object.hasOwnProperty.call(m, "price_prefix"))
                                    w.uint32(178).string(m.price_prefix);
                                if (m.promo_text != null && Object.hasOwnProperty.call(m, "promo_text"))
                                    w.uint32(186).string(m.promo_text);
                                if (m.control_spec != null && Object.hasOwnProperty.call(m, "control_spec"))
                                    $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec.encode(m.control_spec, w.uint32(194).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResponsiveDisplayAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdInfo} message ResponsiveDisplayAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponsiveDisplayAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResponsiveDisplayAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdInfo} ResponsiveDisplayAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponsiveDisplayAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.marketing_images && m.marketing_images.length))
                                            m.marketing_images = [];
                                        m.marketing_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.square_marketing_images && m.square_marketing_images.length))
                                            m.square_marketing_images = [];
                                        m.square_marketing_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.logo_images && m.logo_images.length))
                                            m.logo_images = [];
                                        m.logo_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.square_logo_images && m.square_logo_images.length))
                                            m.square_logo_images = [];
                                        m.square_logo_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        m.long_headline = $root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 8:
                                        if (!(m.youtube_videos && m.youtube_videos.length))
                                            m.youtube_videos = [];
                                        m.youtube_videos.push($root.google.ads.googleads.v11.common.AdVideoAsset.decode(r, r.uint32()));
                                        break;
                                    case 17:
                                        m.business_name = r.string();
                                        break;
                                    case 18:
                                        m.main_color = r.string();
                                        break;
                                    case 19:
                                        m.accent_color = r.string();
                                        break;
                                    case 20:
                                        m.allow_flexible_color = r.bool();
                                        break;
                                    case 21:
                                        m.call_to_action_text = r.string();
                                        break;
                                    case 22:
                                        m.price_prefix = r.string();
                                        break;
                                    case 23:
                                        m.promo_text = r.string();
                                        break;
                                    case 16:
                                        m.format_setting = r.int32();
                                        break;
                                    case 24:
                                        m.control_spec = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResponsiveDisplayAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdInfo} ResponsiveDisplayAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponsiveDisplayAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResponsiveDisplayAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResponsiveDisplayAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.marketing_images != null && m.hasOwnProperty("marketing_images")) {
                                    if (!Array.isArray(m.marketing_images))
                                        return "marketing_images: array expected";
                                    for (var i = 0; i < m.marketing_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.marketing_images[i]);
                                            if (e)
                                                return "marketing_images." + e;
                                        }
                                    }
                                }
                                if (m.square_marketing_images != null && m.hasOwnProperty("square_marketing_images")) {
                                    if (!Array.isArray(m.square_marketing_images))
                                        return "square_marketing_images: array expected";
                                    for (var i = 0; i < m.square_marketing_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.square_marketing_images[i]);
                                            if (e)
                                                return "square_marketing_images." + e;
                                        }
                                    }
                                }
                                if (m.logo_images != null && m.hasOwnProperty("logo_images")) {
                                    if (!Array.isArray(m.logo_images))
                                        return "logo_images: array expected";
                                    for (var i = 0; i < m.logo_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.logo_images[i]);
                                            if (e)
                                                return "logo_images." + e;
                                        }
                                    }
                                }
                                if (m.square_logo_images != null && m.hasOwnProperty("square_logo_images")) {
                                    if (!Array.isArray(m.square_logo_images))
                                        return "square_logo_images: array expected";
                                    for (var i = 0; i < m.square_logo_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.square_logo_images[i]);
                                            if (e)
                                                return "square_logo_images." + e;
                                        }
                                    }
                                }
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.long_headline != null && m.hasOwnProperty("long_headline")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.long_headline);
                                        if (e)
                                            return "long_headline." + e;
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.youtube_videos != null && m.hasOwnProperty("youtube_videos")) {
                                    if (!Array.isArray(m.youtube_videos))
                                        return "youtube_videos: array expected";
                                    for (var i = 0; i < m.youtube_videos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdVideoAsset.verify(m.youtube_videos[i]);
                                            if (e)
                                                return "youtube_videos." + e;
                                        }
                                    }
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p._business_name = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.main_color != null && m.hasOwnProperty("main_color")) {
                                    p._main_color = 1;
                                    if (!$util.isString(m.main_color))
                                        return "main_color: string expected";
                                }
                                if (m.accent_color != null && m.hasOwnProperty("accent_color")) {
                                    p._accent_color = 1;
                                    if (!$util.isString(m.accent_color))
                                        return "accent_color: string expected";
                                }
                                if (m.allow_flexible_color != null && m.hasOwnProperty("allow_flexible_color")) {
                                    p._allow_flexible_color = 1;
                                    if (typeof m.allow_flexible_color !== "boolean")
                                        return "allow_flexible_color: boolean expected";
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    p._call_to_action_text = 1;
                                    if (!$util.isString(m.call_to_action_text))
                                        return "call_to_action_text: string expected";
                                }
                                if (m.price_prefix != null && m.hasOwnProperty("price_prefix")) {
                                    p._price_prefix = 1;
                                    if (!$util.isString(m.price_prefix))
                                        return "price_prefix: string expected";
                                }
                                if (m.promo_text != null && m.hasOwnProperty("promo_text")) {
                                    p._promo_text = 1;
                                    if (!$util.isString(m.promo_text))
                                        return "promo_text: string expected";
                                }
                                if (m.format_setting != null && m.hasOwnProperty("format_setting")) {
                                    switch (m.format_setting) {
                                    default:
                                        return "format_setting: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.control_spec != null && m.hasOwnProperty("control_spec")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec.verify(m.control_spec);
                                        if (e)
                                            return "control_spec." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ResponsiveDisplayAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdInfo} ResponsiveDisplayAdInfo
                             */
                            ResponsiveDisplayAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo();
                                if (d.marketing_images) {
                                    if (!Array.isArray(d.marketing_images))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.marketing_images: array expected");
                                    m.marketing_images = [];
                                    for (var i = 0; i < d.marketing_images.length; ++i) {
                                        if (typeof d.marketing_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.marketing_images: object expected");
                                        m.marketing_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.marketing_images[i]);
                                    }
                                }
                                if (d.square_marketing_images) {
                                    if (!Array.isArray(d.square_marketing_images))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.square_marketing_images: array expected");
                                    m.square_marketing_images = [];
                                    for (var i = 0; i < d.square_marketing_images.length; ++i) {
                                        if (typeof d.square_marketing_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.square_marketing_images: object expected");
                                        m.square_marketing_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.square_marketing_images[i]);
                                    }
                                }
                                if (d.logo_images) {
                                    if (!Array.isArray(d.logo_images))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.logo_images: array expected");
                                    m.logo_images = [];
                                    for (var i = 0; i < d.logo_images.length; ++i) {
                                        if (typeof d.logo_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.logo_images: object expected");
                                        m.logo_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.logo_images[i]);
                                    }
                                }
                                if (d.square_logo_images) {
                                    if (!Array.isArray(d.square_logo_images))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.square_logo_images: array expected");
                                    m.square_logo_images = [];
                                    for (var i = 0; i < d.square_logo_images.length; ++i) {
                                        if (typeof d.square_logo_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.square_logo_images: object expected");
                                        m.square_logo_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.square_logo_images[i]);
                                    }
                                }
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.long_headline != null) {
                                    if (typeof d.long_headline !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.long_headline: object expected");
                                    m.long_headline = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.long_headline);
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.youtube_videos) {
                                    if (!Array.isArray(d.youtube_videos))
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.youtube_videos: array expected");
                                    m.youtube_videos = [];
                                    for (var i = 0; i < d.youtube_videos.length; ++i) {
                                        if (typeof d.youtube_videos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.youtube_videos: object expected");
                                        m.youtube_videos[i] = $root.google.ads.googleads.v11.common.AdVideoAsset.fromObject(d.youtube_videos[i]);
                                    }
                                }
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.main_color != null) {
                                    m.main_color = String(d.main_color);
                                }
                                if (d.accent_color != null) {
                                    m.accent_color = String(d.accent_color);
                                }
                                if (d.allow_flexible_color != null) {
                                    m.allow_flexible_color = Boolean(d.allow_flexible_color);
                                }
                                if (d.call_to_action_text != null) {
                                    m.call_to_action_text = String(d.call_to_action_text);
                                }
                                if (d.price_prefix != null) {
                                    m.price_prefix = String(d.price_prefix);
                                }
                                if (d.promo_text != null) {
                                    m.promo_text = String(d.promo_text);
                                }
                                switch (d.format_setting) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.format_setting = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.format_setting = 1;
                                    break;
                                case "ALL_FORMATS":
                                case 2:
                                    m.format_setting = 2;
                                    break;
                                case "NON_NATIVE":
                                case 3:
                                    m.format_setting = 3;
                                    break;
                                case "NATIVE":
                                case 4:
                                    m.format_setting = 4;
                                    break;
                                }
                                if (d.control_spec != null) {
                                    if (typeof d.control_spec !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.control_spec: object expected");
                                    m.control_spec = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec.fromObject(d.control_spec);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ResponsiveDisplayAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ResponsiveDisplayAdInfo} m ResponsiveDisplayAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResponsiveDisplayAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.marketing_images = [];
                                    d.square_marketing_images = [];
                                    d.logo_images = [];
                                    d.square_logo_images = [];
                                    d.headlines = [];
                                    d.descriptions = [];
                                    d.youtube_videos = [];
                                }
                                if (o.defaults) {
                                    d.long_headline = null;
                                    d.format_setting = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.control_spec = null;
                                }
                                if (m.marketing_images && m.marketing_images.length) {
                                    d.marketing_images = [];
                                    for (var j = 0; j < m.marketing_images.length; ++j) {
                                        d.marketing_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.marketing_images[j], o);
                                    }
                                }
                                if (m.square_marketing_images && m.square_marketing_images.length) {
                                    d.square_marketing_images = [];
                                    for (var j = 0; j < m.square_marketing_images.length; ++j) {
                                        d.square_marketing_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.square_marketing_images[j], o);
                                    }
                                }
                                if (m.logo_images && m.logo_images.length) {
                                    d.logo_images = [];
                                    for (var j = 0; j < m.logo_images.length; ++j) {
                                        d.logo_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.logo_images[j], o);
                                    }
                                }
                                if (m.square_logo_images && m.square_logo_images.length) {
                                    d.square_logo_images = [];
                                    for (var j = 0; j < m.square_logo_images.length; ++j) {
                                        d.square_logo_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.square_logo_images[j], o);
                                    }
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.long_headline != null && m.hasOwnProperty("long_headline")) {
                                    d.long_headline = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.long_headline, o);
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.youtube_videos && m.youtube_videos.length) {
                                    d.youtube_videos = [];
                                    for (var j = 0; j < m.youtube_videos.length; ++j) {
                                        d.youtube_videos[j] = $root.google.ads.googleads.v11.common.AdVideoAsset.toObject(m.youtube_videos[j], o);
                                    }
                                }
                                if (m.format_setting != null && m.hasOwnProperty("format_setting")) {
                                    d.format_setting = o.enums === String ? $root.google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.DisplayAdFormatSetting[m.format_setting] : m.format_setting;
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d._business_name = "business_name";
                                }
                                if (m.main_color != null && m.hasOwnProperty("main_color")) {
                                    d.main_color = m.main_color;
                                    if (o.oneofs)
                                        d._main_color = "main_color";
                                }
                                if (m.accent_color != null && m.hasOwnProperty("accent_color")) {
                                    d.accent_color = m.accent_color;
                                    if (o.oneofs)
                                        d._accent_color = "accent_color";
                                }
                                if (m.allow_flexible_color != null && m.hasOwnProperty("allow_flexible_color")) {
                                    d.allow_flexible_color = m.allow_flexible_color;
                                    if (o.oneofs)
                                        d._allow_flexible_color = "allow_flexible_color";
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    d.call_to_action_text = m.call_to_action_text;
                                    if (o.oneofs)
                                        d._call_to_action_text = "call_to_action_text";
                                }
                                if (m.price_prefix != null && m.hasOwnProperty("price_prefix")) {
                                    d.price_prefix = m.price_prefix;
                                    if (o.oneofs)
                                        d._price_prefix = "price_prefix";
                                }
                                if (m.promo_text != null && m.hasOwnProperty("promo_text")) {
                                    d.promo_text = m.promo_text;
                                    if (o.oneofs)
                                        d._promo_text = "promo_text";
                                }
                                if (m.control_spec != null && m.hasOwnProperty("control_spec")) {
                                    d.control_spec = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec.toObject(m.control_spec, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ResponsiveDisplayAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResponsiveDisplayAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ResponsiveDisplayAdInfo;
                        })();
    
                        common.LocalAdInfo = (function() {
    
                            /**
                             * Properties of a LocalAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILocalAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] LocalAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] LocalAdInfo descriptions
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [call_to_actions] LocalAdInfo call_to_actions
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [marketing_images] LocalAdInfo marketing_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [logo_images] LocalAdInfo logo_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdVideoAsset>|null} [videos] LocalAdInfo videos
                             * @property {string|null} [path1] LocalAdInfo path1
                             * @property {string|null} [path2] LocalAdInfo path2
                             */
    
                            /**
                             * Constructs a new LocalAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LocalAdInfo.
                             * @implements ILocalAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILocalAdInfo=} [p] Properties to set
                             */
                            function LocalAdInfo(p) {
                                this.headlines = [];
                                this.descriptions = [];
                                this.call_to_actions = [];
                                this.marketing_images = [];
                                this.logo_images = [];
                                this.videos = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LocalAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * LocalAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * LocalAdInfo call_to_actions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} call_to_actions
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.call_to_actions = $util.emptyArray;
    
                            /**
                             * LocalAdInfo marketing_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} marketing_images
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.marketing_images = $util.emptyArray;
    
                            /**
                             * LocalAdInfo logo_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} logo_images
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.logo_images = $util.emptyArray;
    
                            /**
                             * LocalAdInfo videos.
                             * @member {Array.<google.ads.googleads.v11.common.IAdVideoAsset>} videos
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.videos = $util.emptyArray;
    
                            /**
                             * LocalAdInfo path1.
                             * @member {string|null|undefined} path1
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.path1 = null;
    
                            /**
                             * LocalAdInfo path2.
                             * @member {string|null|undefined} path2
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            LocalAdInfo.prototype.path2 = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LocalAdInfo _path1.
                             * @member {"path1"|undefined} _path1
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            Object.defineProperty(LocalAdInfo.prototype, "_path1", {
                                get: $util.oneOfGetter($oneOfFields = ["path1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocalAdInfo _path2.
                             * @member {"path2"|undefined} _path2
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             */
                            Object.defineProperty(LocalAdInfo.prototype, "_path2", {
                                get: $util.oneOfGetter($oneOfFields = ["path2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LocalAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocalAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LocalAdInfo} LocalAdInfo instance
                             */
                            LocalAdInfo.create = function create(properties) {
                                return new LocalAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LocalAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LocalAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocalAdInfo} m LocalAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocalAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.call_to_actions != null && m.call_to_actions.length) {
                                    for (var i = 0; i < m.call_to_actions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.call_to_actions[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.marketing_images != null && m.marketing_images.length) {
                                    for (var i = 0; i < m.marketing_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.marketing_images[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.logo_images != null && m.logo_images.length) {
                                    for (var i = 0; i < m.logo_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.logo_images[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.videos != null && m.videos.length) {
                                    for (var i = 0; i < m.videos.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdVideoAsset.encode(m.videos[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.path1 != null && Object.hasOwnProperty.call(m, "path1"))
                                    w.uint32(74).string(m.path1);
                                if (m.path2 != null && Object.hasOwnProperty.call(m, "path2"))
                                    w.uint32(82).string(m.path2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocalAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LocalAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocalAdInfo} message LocalAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocalAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocalAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LocalAdInfo} LocalAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocalAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LocalAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.call_to_actions && m.call_to_actions.length))
                                            m.call_to_actions = [];
                                        m.call_to_actions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.marketing_images && m.marketing_images.length))
                                            m.marketing_images = [];
                                        m.marketing_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        if (!(m.logo_images && m.logo_images.length))
                                            m.logo_images = [];
                                        m.logo_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        if (!(m.videos && m.videos.length))
                                            m.videos = [];
                                        m.videos.push($root.google.ads.googleads.v11.common.AdVideoAsset.decode(r, r.uint32()));
                                        break;
                                    case 9:
                                        m.path1 = r.string();
                                        break;
                                    case 10:
                                        m.path2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocalAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LocalAdInfo} LocalAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocalAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocalAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocalAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.call_to_actions != null && m.hasOwnProperty("call_to_actions")) {
                                    if (!Array.isArray(m.call_to_actions))
                                        return "call_to_actions: array expected";
                                    for (var i = 0; i < m.call_to_actions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.call_to_actions[i]);
                                            if (e)
                                                return "call_to_actions." + e;
                                        }
                                    }
                                }
                                if (m.marketing_images != null && m.hasOwnProperty("marketing_images")) {
                                    if (!Array.isArray(m.marketing_images))
                                        return "marketing_images: array expected";
                                    for (var i = 0; i < m.marketing_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.marketing_images[i]);
                                            if (e)
                                                return "marketing_images." + e;
                                        }
                                    }
                                }
                                if (m.logo_images != null && m.hasOwnProperty("logo_images")) {
                                    if (!Array.isArray(m.logo_images))
                                        return "logo_images: array expected";
                                    for (var i = 0; i < m.logo_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.logo_images[i]);
                                            if (e)
                                                return "logo_images." + e;
                                        }
                                    }
                                }
                                if (m.videos != null && m.hasOwnProperty("videos")) {
                                    if (!Array.isArray(m.videos))
                                        return "videos: array expected";
                                    for (var i = 0; i < m.videos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdVideoAsset.verify(m.videos[i]);
                                            if (e)
                                                return "videos." + e;
                                        }
                                    }
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    p._path1 = 1;
                                    if (!$util.isString(m.path1))
                                        return "path1: string expected";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    p._path2 = 1;
                                    if (!$util.isString(m.path2))
                                        return "path2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LocalAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LocalAdInfo} LocalAdInfo
                             */
                            LocalAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LocalAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LocalAdInfo();
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.call_to_actions) {
                                    if (!Array.isArray(d.call_to_actions))
                                        throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.call_to_actions: array expected");
                                    m.call_to_actions = [];
                                    for (var i = 0; i < d.call_to_actions.length; ++i) {
                                        if (typeof d.call_to_actions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.call_to_actions: object expected");
                                        m.call_to_actions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.call_to_actions[i]);
                                    }
                                }
                                if (d.marketing_images) {
                                    if (!Array.isArray(d.marketing_images))
                                        throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.marketing_images: array expected");
                                    m.marketing_images = [];
                                    for (var i = 0; i < d.marketing_images.length; ++i) {
                                        if (typeof d.marketing_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.marketing_images: object expected");
                                        m.marketing_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.marketing_images[i]);
                                    }
                                }
                                if (d.logo_images) {
                                    if (!Array.isArray(d.logo_images))
                                        throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.logo_images: array expected");
                                    m.logo_images = [];
                                    for (var i = 0; i < d.logo_images.length; ++i) {
                                        if (typeof d.logo_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.logo_images: object expected");
                                        m.logo_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.logo_images[i]);
                                    }
                                }
                                if (d.videos) {
                                    if (!Array.isArray(d.videos))
                                        throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.videos: array expected");
                                    m.videos = [];
                                    for (var i = 0; i < d.videos.length; ++i) {
                                        if (typeof d.videos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LocalAdInfo.videos: object expected");
                                        m.videos[i] = $root.google.ads.googleads.v11.common.AdVideoAsset.fromObject(d.videos[i]);
                                    }
                                }
                                if (d.path1 != null) {
                                    m.path1 = String(d.path1);
                                }
                                if (d.path2 != null) {
                                    m.path2 = String(d.path2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LocalAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LocalAdInfo} m LocalAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocalAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.headlines = [];
                                    d.descriptions = [];
                                    d.call_to_actions = [];
                                    d.marketing_images = [];
                                    d.logo_images = [];
                                    d.videos = [];
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.call_to_actions && m.call_to_actions.length) {
                                    d.call_to_actions = [];
                                    for (var j = 0; j < m.call_to_actions.length; ++j) {
                                        d.call_to_actions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.call_to_actions[j], o);
                                    }
                                }
                                if (m.marketing_images && m.marketing_images.length) {
                                    d.marketing_images = [];
                                    for (var j = 0; j < m.marketing_images.length; ++j) {
                                        d.marketing_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.marketing_images[j], o);
                                    }
                                }
                                if (m.logo_images && m.logo_images.length) {
                                    d.logo_images = [];
                                    for (var j = 0; j < m.logo_images.length; ++j) {
                                        d.logo_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.logo_images[j], o);
                                    }
                                }
                                if (m.videos && m.videos.length) {
                                    d.videos = [];
                                    for (var j = 0; j < m.videos.length; ++j) {
                                        d.videos[j] = $root.google.ads.googleads.v11.common.AdVideoAsset.toObject(m.videos[j], o);
                                    }
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    d.path1 = m.path1;
                                    if (o.oneofs)
                                        d._path1 = "path1";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    d.path2 = m.path2;
                                    if (o.oneofs)
                                        d._path2 = "path2";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LocalAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LocalAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocalAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LocalAdInfo;
                        })();
    
                        common.DisplayUploadAdInfo = (function() {
    
                            /**
                             * Properties of a DisplayUploadAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDisplayUploadAdInfo
                             * @property {google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum.DisplayUploadProductType|null} [display_upload_product_type] DisplayUploadAdInfo display_upload_product_type
                             * @property {google.ads.googleads.v11.common.IAdMediaBundleAsset|null} [media_bundle] DisplayUploadAdInfo media_bundle
                             */
    
                            /**
                             * Constructs a new DisplayUploadAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DisplayUploadAdInfo.
                             * @implements IDisplayUploadAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDisplayUploadAdInfo=} [p] Properties to set
                             */
                            function DisplayUploadAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DisplayUploadAdInfo display_upload_product_type.
                             * @member {google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum.DisplayUploadProductType} display_upload_product_type
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @instance
                             */
                            DisplayUploadAdInfo.prototype.display_upload_product_type = 0;
    
                            /**
                             * DisplayUploadAdInfo media_bundle.
                             * @member {google.ads.googleads.v11.common.IAdMediaBundleAsset|null|undefined} media_bundle
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @instance
                             */
                            DisplayUploadAdInfo.prototype.media_bundle = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DisplayUploadAdInfo media_asset.
                             * @member {"media_bundle"|undefined} media_asset
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @instance
                             */
                            Object.defineProperty(DisplayUploadAdInfo.prototype, "media_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["media_bundle"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DisplayUploadAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDisplayUploadAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DisplayUploadAdInfo} DisplayUploadAdInfo instance
                             */
                            DisplayUploadAdInfo.create = function create(properties) {
                                return new DisplayUploadAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified DisplayUploadAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.DisplayUploadAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDisplayUploadAdInfo} m DisplayUploadAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayUploadAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.display_upload_product_type != null && Object.hasOwnProperty.call(m, "display_upload_product_type"))
                                    w.uint32(8).int32(m.display_upload_product_type);
                                if (m.media_bundle != null && Object.hasOwnProperty.call(m, "media_bundle"))
                                    $root.google.ads.googleads.v11.common.AdMediaBundleAsset.encode(m.media_bundle, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DisplayUploadAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DisplayUploadAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDisplayUploadAdInfo} message DisplayUploadAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayUploadAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DisplayUploadAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DisplayUploadAdInfo} DisplayUploadAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayUploadAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DisplayUploadAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.display_upload_product_type = r.int32();
                                        break;
                                    case 2:
                                        m.media_bundle = $root.google.ads.googleads.v11.common.AdMediaBundleAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DisplayUploadAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DisplayUploadAdInfo} DisplayUploadAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayUploadAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DisplayUploadAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DisplayUploadAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.display_upload_product_type != null && m.hasOwnProperty("display_upload_product_type")) {
                                    switch (m.display_upload_product_type) {
                                    default:
                                        return "display_upload_product_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                if (m.media_bundle != null && m.hasOwnProperty("media_bundle")) {
                                    p.media_asset = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdMediaBundleAsset.verify(m.media_bundle);
                                        if (e)
                                            return "media_bundle." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DisplayUploadAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DisplayUploadAdInfo} DisplayUploadAdInfo
                             */
                            DisplayUploadAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DisplayUploadAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DisplayUploadAdInfo();
                                switch (d.display_upload_product_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.display_upload_product_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.display_upload_product_type = 1;
                                    break;
                                case "HTML5_UPLOAD_AD":
                                case 2:
                                    m.display_upload_product_type = 2;
                                    break;
                                case "DYNAMIC_HTML5_EDUCATION_AD":
                                case 3:
                                    m.display_upload_product_type = 3;
                                    break;
                                case "DYNAMIC_HTML5_FLIGHT_AD":
                                case 4:
                                    m.display_upload_product_type = 4;
                                    break;
                                case "DYNAMIC_HTML5_HOTEL_RENTAL_AD":
                                case 5:
                                    m.display_upload_product_type = 5;
                                    break;
                                case "DYNAMIC_HTML5_JOB_AD":
                                case 6:
                                    m.display_upload_product_type = 6;
                                    break;
                                case "DYNAMIC_HTML5_LOCAL_AD":
                                case 7:
                                    m.display_upload_product_type = 7;
                                    break;
                                case "DYNAMIC_HTML5_REAL_ESTATE_AD":
                                case 8:
                                    m.display_upload_product_type = 8;
                                    break;
                                case "DYNAMIC_HTML5_CUSTOM_AD":
                                case 9:
                                    m.display_upload_product_type = 9;
                                    break;
                                case "DYNAMIC_HTML5_TRAVEL_AD":
                                case 10:
                                    m.display_upload_product_type = 10;
                                    break;
                                case "DYNAMIC_HTML5_HOTEL_AD":
                                case 11:
                                    m.display_upload_product_type = 11;
                                    break;
                                }
                                if (d.media_bundle != null) {
                                    if (typeof d.media_bundle !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.DisplayUploadAdInfo.media_bundle: object expected");
                                    m.media_bundle = $root.google.ads.googleads.v11.common.AdMediaBundleAsset.fromObject(d.media_bundle);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DisplayUploadAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.DisplayUploadAdInfo} m DisplayUploadAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DisplayUploadAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.display_upload_product_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.display_upload_product_type != null && m.hasOwnProperty("display_upload_product_type")) {
                                    d.display_upload_product_type = o.enums === String ? $root.google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum.DisplayUploadProductType[m.display_upload_product_type] : m.display_upload_product_type;
                                }
                                if (m.media_bundle != null && m.hasOwnProperty("media_bundle")) {
                                    d.media_bundle = $root.google.ads.googleads.v11.common.AdMediaBundleAsset.toObject(m.media_bundle, o);
                                    if (o.oneofs)
                                        d.media_asset = "media_bundle";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DisplayUploadAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DisplayUploadAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DisplayUploadAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DisplayUploadAdInfo;
                        })();
    
                        common.ResponsiveDisplayAdControlSpec = (function() {
    
                            /**
                             * Properties of a ResponsiveDisplayAdControlSpec.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IResponsiveDisplayAdControlSpec
                             * @property {boolean|null} [enable_asset_enhancements] ResponsiveDisplayAdControlSpec enable_asset_enhancements
                             * @property {boolean|null} [enable_autogen_video] ResponsiveDisplayAdControlSpec enable_autogen_video
                             */
    
                            /**
                             * Constructs a new ResponsiveDisplayAdControlSpec.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ResponsiveDisplayAdControlSpec.
                             * @implements IResponsiveDisplayAdControlSpec
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdControlSpec=} [p] Properties to set
                             */
                            function ResponsiveDisplayAdControlSpec(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ResponsiveDisplayAdControlSpec enable_asset_enhancements.
                             * @member {boolean} enable_asset_enhancements
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @instance
                             */
                            ResponsiveDisplayAdControlSpec.prototype.enable_asset_enhancements = false;
    
                            /**
                             * ResponsiveDisplayAdControlSpec enable_autogen_video.
                             * @member {boolean} enable_autogen_video
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @instance
                             */
                            ResponsiveDisplayAdControlSpec.prototype.enable_autogen_video = false;
    
                            /**
                             * Creates a new ResponsiveDisplayAdControlSpec instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdControlSpec=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec} ResponsiveDisplayAdControlSpec instance
                             */
                            ResponsiveDisplayAdControlSpec.create = function create(properties) {
                                return new ResponsiveDisplayAdControlSpec(properties);
                            };
    
                            /**
                             * Encodes the specified ResponsiveDisplayAdControlSpec message. Does not implicitly {@link google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdControlSpec} m ResponsiveDisplayAdControlSpec message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponsiveDisplayAdControlSpec.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.enable_asset_enhancements != null && Object.hasOwnProperty.call(m, "enable_asset_enhancements"))
                                    w.uint32(8).bool(m.enable_asset_enhancements);
                                if (m.enable_autogen_video != null && Object.hasOwnProperty.call(m, "enable_autogen_video"))
                                    w.uint32(16).bool(m.enable_autogen_video);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResponsiveDisplayAdControlSpec message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {google.ads.googleads.v11.common.IResponsiveDisplayAdControlSpec} message ResponsiveDisplayAdControlSpec message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponsiveDisplayAdControlSpec.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResponsiveDisplayAdControlSpec message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec} ResponsiveDisplayAdControlSpec
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponsiveDisplayAdControlSpec.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.enable_asset_enhancements = r.bool();
                                        break;
                                    case 2:
                                        m.enable_autogen_video = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResponsiveDisplayAdControlSpec message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec} ResponsiveDisplayAdControlSpec
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponsiveDisplayAdControlSpec.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResponsiveDisplayAdControlSpec message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResponsiveDisplayAdControlSpec.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.enable_asset_enhancements != null && m.hasOwnProperty("enable_asset_enhancements")) {
                                    if (typeof m.enable_asset_enhancements !== "boolean")
                                        return "enable_asset_enhancements: boolean expected";
                                }
                                if (m.enable_autogen_video != null && m.hasOwnProperty("enable_autogen_video")) {
                                    if (typeof m.enable_autogen_video !== "boolean")
                                        return "enable_autogen_video: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ResponsiveDisplayAdControlSpec message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec} ResponsiveDisplayAdControlSpec
                             */
                            ResponsiveDisplayAdControlSpec.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec();
                                if (d.enable_asset_enhancements != null) {
                                    m.enable_asset_enhancements = Boolean(d.enable_asset_enhancements);
                                }
                                if (d.enable_autogen_video != null) {
                                    m.enable_autogen_video = Boolean(d.enable_autogen_video);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ResponsiveDisplayAdControlSpec message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @static
                             * @param {google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec} m ResponsiveDisplayAdControlSpec
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResponsiveDisplayAdControlSpec.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.enable_asset_enhancements = false;
                                    d.enable_autogen_video = false;
                                }
                                if (m.enable_asset_enhancements != null && m.hasOwnProperty("enable_asset_enhancements")) {
                                    d.enable_asset_enhancements = m.enable_asset_enhancements;
                                }
                                if (m.enable_autogen_video != null && m.hasOwnProperty("enable_autogen_video")) {
                                    d.enable_autogen_video = m.enable_autogen_video;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ResponsiveDisplayAdControlSpec to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ResponsiveDisplayAdControlSpec
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResponsiveDisplayAdControlSpec.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ResponsiveDisplayAdControlSpec;
                        })();
    
                        common.SmartCampaignAdInfo = (function() {
    
                            /**
                             * Properties of a SmartCampaignAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ISmartCampaignAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] SmartCampaignAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] SmartCampaignAdInfo descriptions
                             */
    
                            /**
                             * Constructs a new SmartCampaignAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a SmartCampaignAdInfo.
                             * @implements ISmartCampaignAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ISmartCampaignAdInfo=} [p] Properties to set
                             */
                            function SmartCampaignAdInfo(p) {
                                this.headlines = [];
                                this.descriptions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SmartCampaignAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @instance
                             */
                            SmartCampaignAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * SmartCampaignAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @instance
                             */
                            SmartCampaignAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * Creates a new SmartCampaignAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ISmartCampaignAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.SmartCampaignAdInfo} SmartCampaignAdInfo instance
                             */
                            SmartCampaignAdInfo.create = function create(properties) {
                                return new SmartCampaignAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified SmartCampaignAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.SmartCampaignAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ISmartCampaignAdInfo} m SmartCampaignAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified SmartCampaignAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.SmartCampaignAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ISmartCampaignAdInfo} message SmartCampaignAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SmartCampaignAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.SmartCampaignAdInfo} SmartCampaignAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.SmartCampaignAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SmartCampaignAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.SmartCampaignAdInfo} SmartCampaignAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SmartCampaignAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SmartCampaignAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SmartCampaignAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.SmartCampaignAdInfo} SmartCampaignAdInfo
                             */
                            SmartCampaignAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.SmartCampaignAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.SmartCampaignAdInfo();
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.SmartCampaignAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.SmartCampaignAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.SmartCampaignAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.SmartCampaignAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SmartCampaignAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.SmartCampaignAdInfo} m SmartCampaignAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SmartCampaignAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.headlines = [];
                                    d.descriptions = [];
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SmartCampaignAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.SmartCampaignAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SmartCampaignAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SmartCampaignAdInfo;
                        })();
    
                        common.CallAdInfo = (function() {
    
                            /**
                             * Properties of a CallAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICallAdInfo
                             * @property {string|null} [country_code] CallAdInfo country_code
                             * @property {string|null} [phone_number] CallAdInfo phone_number
                             * @property {string|null} [business_name] CallAdInfo business_name
                             * @property {string|null} [headline1] CallAdInfo headline1
                             * @property {string|null} [headline2] CallAdInfo headline2
                             * @property {string|null} [description1] CallAdInfo description1
                             * @property {string|null} [description2] CallAdInfo description2
                             * @property {boolean|null} [call_tracked] CallAdInfo call_tracked
                             * @property {boolean|null} [disable_call_conversion] CallAdInfo disable_call_conversion
                             * @property {string|null} [phone_number_verification_url] CallAdInfo phone_number_verification_url
                             * @property {string|null} [conversion_action] CallAdInfo conversion_action
                             * @property {google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState|null} [conversion_reporting_state] CallAdInfo conversion_reporting_state
                             * @property {string|null} [path1] CallAdInfo path1
                             * @property {string|null} [path2] CallAdInfo path2
                             */
    
                            /**
                             * Constructs a new CallAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CallAdInfo.
                             * @implements ICallAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICallAdInfo=} [p] Properties to set
                             */
                            function CallAdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallAdInfo country_code.
                             * @member {string} country_code
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.country_code = "";
    
                            /**
                             * CallAdInfo phone_number.
                             * @member {string} phone_number
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.phone_number = "";
    
                            /**
                             * CallAdInfo business_name.
                             * @member {string} business_name
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.business_name = "";
    
                            /**
                             * CallAdInfo headline1.
                             * @member {string} headline1
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.headline1 = "";
    
                            /**
                             * CallAdInfo headline2.
                             * @member {string} headline2
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.headline2 = "";
    
                            /**
                             * CallAdInfo description1.
                             * @member {string} description1
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.description1 = "";
    
                            /**
                             * CallAdInfo description2.
                             * @member {string} description2
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.description2 = "";
    
                            /**
                             * CallAdInfo call_tracked.
                             * @member {boolean} call_tracked
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.call_tracked = false;
    
                            /**
                             * CallAdInfo disable_call_conversion.
                             * @member {boolean} disable_call_conversion
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.disable_call_conversion = false;
    
                            /**
                             * CallAdInfo phone_number_verification_url.
                             * @member {string} phone_number_verification_url
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.phone_number_verification_url = "";
    
                            /**
                             * CallAdInfo conversion_action.
                             * @member {string} conversion_action
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.conversion_action = "";
    
                            /**
                             * CallAdInfo conversion_reporting_state.
                             * @member {google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState} conversion_reporting_state
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.conversion_reporting_state = 0;
    
                            /**
                             * CallAdInfo path1.
                             * @member {string} path1
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.path1 = "";
    
                            /**
                             * CallAdInfo path2.
                             * @member {string} path2
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             */
                            CallAdInfo.prototype.path2 = "";
    
                            /**
                             * Creates a new CallAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CallAdInfo} CallAdInfo instance
                             */
                            CallAdInfo.create = function create(properties) {
                                return new CallAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CallAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CallAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallAdInfo} m CallAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(10).string(m.country_code);
                                if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                    w.uint32(18).string(m.phone_number);
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(26).string(m.business_name);
                                if (m.description1 != null && Object.hasOwnProperty.call(m, "description1"))
                                    w.uint32(34).string(m.description1);
                                if (m.description2 != null && Object.hasOwnProperty.call(m, "description2"))
                                    w.uint32(42).string(m.description2);
                                if (m.call_tracked != null && Object.hasOwnProperty.call(m, "call_tracked"))
                                    w.uint32(48).bool(m.call_tracked);
                                if (m.disable_call_conversion != null && Object.hasOwnProperty.call(m, "disable_call_conversion"))
                                    w.uint32(56).bool(m.disable_call_conversion);
                                if (m.phone_number_verification_url != null && Object.hasOwnProperty.call(m, "phone_number_verification_url"))
                                    w.uint32(66).string(m.phone_number_verification_url);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(74).string(m.conversion_action);
                                if (m.conversion_reporting_state != null && Object.hasOwnProperty.call(m, "conversion_reporting_state"))
                                    w.uint32(80).int32(m.conversion_reporting_state);
                                if (m.headline1 != null && Object.hasOwnProperty.call(m, "headline1"))
                                    w.uint32(90).string(m.headline1);
                                if (m.headline2 != null && Object.hasOwnProperty.call(m, "headline2"))
                                    w.uint32(98).string(m.headline2);
                                if (m.path1 != null && Object.hasOwnProperty.call(m, "path1"))
                                    w.uint32(106).string(m.path1);
                                if (m.path2 != null && Object.hasOwnProperty.call(m, "path2"))
                                    w.uint32(114).string(m.path2);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CallAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallAdInfo} message CallAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CallAdInfo} CallAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CallAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.country_code = r.string();
                                        break;
                                    case 2:
                                        m.phone_number = r.string();
                                        break;
                                    case 3:
                                        m.business_name = r.string();
                                        break;
                                    case 11:
                                        m.headline1 = r.string();
                                        break;
                                    case 12:
                                        m.headline2 = r.string();
                                        break;
                                    case 4:
                                        m.description1 = r.string();
                                        break;
                                    case 5:
                                        m.description2 = r.string();
                                        break;
                                    case 6:
                                        m.call_tracked = r.bool();
                                        break;
                                    case 7:
                                        m.disable_call_conversion = r.bool();
                                        break;
                                    case 8:
                                        m.phone_number_verification_url = r.string();
                                        break;
                                    case 9:
                                        m.conversion_action = r.string();
                                        break;
                                    case 10:
                                        m.conversion_reporting_state = r.int32();
                                        break;
                                    case 13:
                                        m.path1 = r.string();
                                        break;
                                    case 14:
                                        m.path2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CallAdInfo} CallAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    if (!$util.isString(m.phone_number))
                                        return "phone_number: string expected";
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.headline1 != null && m.hasOwnProperty("headline1")) {
                                    if (!$util.isString(m.headline1))
                                        return "headline1: string expected";
                                }
                                if (m.headline2 != null && m.hasOwnProperty("headline2")) {
                                    if (!$util.isString(m.headline2))
                                        return "headline2: string expected";
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    if (!$util.isString(m.description1))
                                        return "description1: string expected";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    if (!$util.isString(m.description2))
                                        return "description2: string expected";
                                }
                                if (m.call_tracked != null && m.hasOwnProperty("call_tracked")) {
                                    if (typeof m.call_tracked !== "boolean")
                                        return "call_tracked: boolean expected";
                                }
                                if (m.disable_call_conversion != null && m.hasOwnProperty("disable_call_conversion")) {
                                    if (typeof m.disable_call_conversion !== "boolean")
                                        return "disable_call_conversion: boolean expected";
                                }
                                if (m.phone_number_verification_url != null && m.hasOwnProperty("phone_number_verification_url")) {
                                    if (!$util.isString(m.phone_number_verification_url))
                                        return "phone_number_verification_url: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.conversion_reporting_state != null && m.hasOwnProperty("conversion_reporting_state")) {
                                    switch (m.conversion_reporting_state) {
                                    default:
                                        return "conversion_reporting_state: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    if (!$util.isString(m.path1))
                                        return "path1: string expected";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    if (!$util.isString(m.path2))
                                        return "path2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CallAdInfo} CallAdInfo
                             */
                            CallAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CallAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CallAdInfo();
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.phone_number != null) {
                                    m.phone_number = String(d.phone_number);
                                }
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.headline1 != null) {
                                    m.headline1 = String(d.headline1);
                                }
                                if (d.headline2 != null) {
                                    m.headline2 = String(d.headline2);
                                }
                                if (d.description1 != null) {
                                    m.description1 = String(d.description1);
                                }
                                if (d.description2 != null) {
                                    m.description2 = String(d.description2);
                                }
                                if (d.call_tracked != null) {
                                    m.call_tracked = Boolean(d.call_tracked);
                                }
                                if (d.disable_call_conversion != null) {
                                    m.disable_call_conversion = Boolean(d.disable_call_conversion);
                                }
                                if (d.phone_number_verification_url != null) {
                                    m.phone_number_verification_url = String(d.phone_number_verification_url);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                switch (d.conversion_reporting_state) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_reporting_state = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_reporting_state = 1;
                                    break;
                                case "DISABLED":
                                case 2:
                                    m.conversion_reporting_state = 2;
                                    break;
                                case "USE_ACCOUNT_LEVEL_CALL_CONVERSION_ACTION":
                                case 3:
                                    m.conversion_reporting_state = 3;
                                    break;
                                case "USE_RESOURCE_LEVEL_CALL_CONVERSION_ACTION":
                                case 4:
                                    m.conversion_reporting_state = 4;
                                    break;
                                }
                                if (d.path1 != null) {
                                    m.path1 = String(d.path1);
                                }
                                if (d.path2 != null) {
                                    m.path2 = String(d.path2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CallAdInfo} m CallAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.country_code = "";
                                    d.phone_number = "";
                                    d.business_name = "";
                                    d.description1 = "";
                                    d.description2 = "";
                                    d.call_tracked = false;
                                    d.disable_call_conversion = false;
                                    d.phone_number_verification_url = "";
                                    d.conversion_action = "";
                                    d.conversion_reporting_state = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.headline1 = "";
                                    d.headline2 = "";
                                    d.path1 = "";
                                    d.path2 = "";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    d.phone_number = m.phone_number;
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    d.description1 = m.description1;
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    d.description2 = m.description2;
                                }
                                if (m.call_tracked != null && m.hasOwnProperty("call_tracked")) {
                                    d.call_tracked = m.call_tracked;
                                }
                                if (m.disable_call_conversion != null && m.hasOwnProperty("disable_call_conversion")) {
                                    d.disable_call_conversion = m.disable_call_conversion;
                                }
                                if (m.phone_number_verification_url != null && m.hasOwnProperty("phone_number_verification_url")) {
                                    d.phone_number_verification_url = m.phone_number_verification_url;
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                }
                                if (m.conversion_reporting_state != null && m.hasOwnProperty("conversion_reporting_state")) {
                                    d.conversion_reporting_state = o.enums === String ? $root.google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState[m.conversion_reporting_state] : m.conversion_reporting_state;
                                }
                                if (m.headline1 != null && m.hasOwnProperty("headline1")) {
                                    d.headline1 = m.headline1;
                                }
                                if (m.headline2 != null && m.hasOwnProperty("headline2")) {
                                    d.headline2 = m.headline2;
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    d.path1 = m.path1;
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    d.path2 = m.path2;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CallAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallAdInfo;
                        })();
    
                        common.DiscoveryMultiAssetAdInfo = (function() {
    
                            /**
                             * Properties of a DiscoveryMultiAssetAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDiscoveryMultiAssetAdInfo
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [marketing_images] DiscoveryMultiAssetAdInfo marketing_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [square_marketing_images] DiscoveryMultiAssetAdInfo square_marketing_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [portrait_marketing_images] DiscoveryMultiAssetAdInfo portrait_marketing_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdImageAsset>|null} [logo_images] DiscoveryMultiAssetAdInfo logo_images
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [headlines] DiscoveryMultiAssetAdInfo headlines
                             * @property {Array.<google.ads.googleads.v11.common.IAdTextAsset>|null} [descriptions] DiscoveryMultiAssetAdInfo descriptions
                             * @property {string|null} [business_name] DiscoveryMultiAssetAdInfo business_name
                             * @property {string|null} [call_to_action_text] DiscoveryMultiAssetAdInfo call_to_action_text
                             * @property {boolean|null} [lead_form_only] DiscoveryMultiAssetAdInfo lead_form_only
                             */
    
                            /**
                             * Constructs a new DiscoveryMultiAssetAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DiscoveryMultiAssetAdInfo.
                             * @implements IDiscoveryMultiAssetAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDiscoveryMultiAssetAdInfo=} [p] Properties to set
                             */
                            function DiscoveryMultiAssetAdInfo(p) {
                                this.marketing_images = [];
                                this.square_marketing_images = [];
                                this.portrait_marketing_images = [];
                                this.logo_images = [];
                                this.headlines = [];
                                this.descriptions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DiscoveryMultiAssetAdInfo marketing_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} marketing_images
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.marketing_images = $util.emptyArray;
    
                            /**
                             * DiscoveryMultiAssetAdInfo square_marketing_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} square_marketing_images
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.square_marketing_images = $util.emptyArray;
    
                            /**
                             * DiscoveryMultiAssetAdInfo portrait_marketing_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} portrait_marketing_images
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.portrait_marketing_images = $util.emptyArray;
    
                            /**
                             * DiscoveryMultiAssetAdInfo logo_images.
                             * @member {Array.<google.ads.googleads.v11.common.IAdImageAsset>} logo_images
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.logo_images = $util.emptyArray;
    
                            /**
                             * DiscoveryMultiAssetAdInfo headlines.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} headlines
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.headlines = $util.emptyArray;
    
                            /**
                             * DiscoveryMultiAssetAdInfo descriptions.
                             * @member {Array.<google.ads.googleads.v11.common.IAdTextAsset>} descriptions
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.descriptions = $util.emptyArray;
    
                            /**
                             * DiscoveryMultiAssetAdInfo business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.business_name = null;
    
                            /**
                             * DiscoveryMultiAssetAdInfo call_to_action_text.
                             * @member {string|null|undefined} call_to_action_text
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.call_to_action_text = null;
    
                            /**
                             * DiscoveryMultiAssetAdInfo lead_form_only.
                             * @member {boolean|null|undefined} lead_form_only
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            DiscoveryMultiAssetAdInfo.prototype.lead_form_only = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DiscoveryMultiAssetAdInfo _business_name.
                             * @member {"business_name"|undefined} _business_name
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            Object.defineProperty(DiscoveryMultiAssetAdInfo.prototype, "_business_name", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DiscoveryMultiAssetAdInfo _call_to_action_text.
                             * @member {"call_to_action_text"|undefined} _call_to_action_text
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            Object.defineProperty(DiscoveryMultiAssetAdInfo.prototype, "_call_to_action_text", {
                                get: $util.oneOfGetter($oneOfFields = ["call_to_action_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DiscoveryMultiAssetAdInfo _lead_form_only.
                             * @member {"lead_form_only"|undefined} _lead_form_only
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             */
                            Object.defineProperty(DiscoveryMultiAssetAdInfo.prototype, "_lead_form_only", {
                                get: $util.oneOfGetter($oneOfFields = ["lead_form_only"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DiscoveryMultiAssetAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryMultiAssetAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo} DiscoveryMultiAssetAdInfo instance
                             */
                            DiscoveryMultiAssetAdInfo.create = function create(properties) {
                                return new DiscoveryMultiAssetAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified DiscoveryMultiAssetAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryMultiAssetAdInfo} m DiscoveryMultiAssetAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DiscoveryMultiAssetAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.marketing_images != null && m.marketing_images.length) {
                                    for (var i = 0; i < m.marketing_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.marketing_images[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.square_marketing_images != null && m.square_marketing_images.length) {
                                    for (var i = 0; i < m.square_marketing_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.square_marketing_images[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.portrait_marketing_images != null && m.portrait_marketing_images.length) {
                                    for (var i = 0; i < m.portrait_marketing_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.portrait_marketing_images[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.logo_images != null && m.logo_images.length) {
                                    for (var i = 0; i < m.logo_images.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.logo_images[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.headlines != null && m.headlines.length) {
                                    for (var i = 0; i < m.headlines.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headlines[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.descriptions != null && m.descriptions.length) {
                                    for (var i = 0; i < m.descriptions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.descriptions[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(58).string(m.business_name);
                                if (m.call_to_action_text != null && Object.hasOwnProperty.call(m, "call_to_action_text"))
                                    w.uint32(66).string(m.call_to_action_text);
                                if (m.lead_form_only != null && Object.hasOwnProperty.call(m, "lead_form_only"))
                                    w.uint32(72).bool(m.lead_form_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DiscoveryMultiAssetAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryMultiAssetAdInfo} message DiscoveryMultiAssetAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DiscoveryMultiAssetAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DiscoveryMultiAssetAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo} DiscoveryMultiAssetAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DiscoveryMultiAssetAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.marketing_images && m.marketing_images.length))
                                            m.marketing_images = [];
                                        m.marketing_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.square_marketing_images && m.square_marketing_images.length))
                                            m.square_marketing_images = [];
                                        m.square_marketing_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.portrait_marketing_images && m.portrait_marketing_images.length))
                                            m.portrait_marketing_images = [];
                                        m.portrait_marketing_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.logo_images && m.logo_images.length))
                                            m.logo_images = [];
                                        m.logo_images.push($root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        if (!(m.headlines && m.headlines.length))
                                            m.headlines = [];
                                        m.headlines.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        if (!(m.descriptions && m.descriptions.length))
                                            m.descriptions = [];
                                        m.descriptions.push($root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32()));
                                        break;
                                    case 7:
                                        m.business_name = r.string();
                                        break;
                                    case 8:
                                        m.call_to_action_text = r.string();
                                        break;
                                    case 9:
                                        m.lead_form_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DiscoveryMultiAssetAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo} DiscoveryMultiAssetAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DiscoveryMultiAssetAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DiscoveryMultiAssetAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DiscoveryMultiAssetAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.marketing_images != null && m.hasOwnProperty("marketing_images")) {
                                    if (!Array.isArray(m.marketing_images))
                                        return "marketing_images: array expected";
                                    for (var i = 0; i < m.marketing_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.marketing_images[i]);
                                            if (e)
                                                return "marketing_images." + e;
                                        }
                                    }
                                }
                                if (m.square_marketing_images != null && m.hasOwnProperty("square_marketing_images")) {
                                    if (!Array.isArray(m.square_marketing_images))
                                        return "square_marketing_images: array expected";
                                    for (var i = 0; i < m.square_marketing_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.square_marketing_images[i]);
                                            if (e)
                                                return "square_marketing_images." + e;
                                        }
                                    }
                                }
                                if (m.portrait_marketing_images != null && m.hasOwnProperty("portrait_marketing_images")) {
                                    if (!Array.isArray(m.portrait_marketing_images))
                                        return "portrait_marketing_images: array expected";
                                    for (var i = 0; i < m.portrait_marketing_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.portrait_marketing_images[i]);
                                            if (e)
                                                return "portrait_marketing_images." + e;
                                        }
                                    }
                                }
                                if (m.logo_images != null && m.hasOwnProperty("logo_images")) {
                                    if (!Array.isArray(m.logo_images))
                                        return "logo_images: array expected";
                                    for (var i = 0; i < m.logo_images.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.logo_images[i]);
                                            if (e)
                                                return "logo_images." + e;
                                        }
                                    }
                                }
                                if (m.headlines != null && m.hasOwnProperty("headlines")) {
                                    if (!Array.isArray(m.headlines))
                                        return "headlines: array expected";
                                    for (var i = 0; i < m.headlines.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headlines[i]);
                                            if (e)
                                                return "headlines." + e;
                                        }
                                    }
                                }
                                if (m.descriptions != null && m.hasOwnProperty("descriptions")) {
                                    if (!Array.isArray(m.descriptions))
                                        return "descriptions: array expected";
                                    for (var i = 0; i < m.descriptions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.descriptions[i]);
                                            if (e)
                                                return "descriptions." + e;
                                        }
                                    }
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p._business_name = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    p._call_to_action_text = 1;
                                    if (!$util.isString(m.call_to_action_text))
                                        return "call_to_action_text: string expected";
                                }
                                if (m.lead_form_only != null && m.hasOwnProperty("lead_form_only")) {
                                    p._lead_form_only = 1;
                                    if (typeof m.lead_form_only !== "boolean")
                                        return "lead_form_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DiscoveryMultiAssetAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo} DiscoveryMultiAssetAdInfo
                             */
                            DiscoveryMultiAssetAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo();
                                if (d.marketing_images) {
                                    if (!Array.isArray(d.marketing_images))
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.marketing_images: array expected");
                                    m.marketing_images = [];
                                    for (var i = 0; i < d.marketing_images.length; ++i) {
                                        if (typeof d.marketing_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.marketing_images: object expected");
                                        m.marketing_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.marketing_images[i]);
                                    }
                                }
                                if (d.square_marketing_images) {
                                    if (!Array.isArray(d.square_marketing_images))
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.square_marketing_images: array expected");
                                    m.square_marketing_images = [];
                                    for (var i = 0; i < d.square_marketing_images.length; ++i) {
                                        if (typeof d.square_marketing_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.square_marketing_images: object expected");
                                        m.square_marketing_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.square_marketing_images[i]);
                                    }
                                }
                                if (d.portrait_marketing_images) {
                                    if (!Array.isArray(d.portrait_marketing_images))
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.portrait_marketing_images: array expected");
                                    m.portrait_marketing_images = [];
                                    for (var i = 0; i < d.portrait_marketing_images.length; ++i) {
                                        if (typeof d.portrait_marketing_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.portrait_marketing_images: object expected");
                                        m.portrait_marketing_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.portrait_marketing_images[i]);
                                    }
                                }
                                if (d.logo_images) {
                                    if (!Array.isArray(d.logo_images))
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.logo_images: array expected");
                                    m.logo_images = [];
                                    for (var i = 0; i < d.logo_images.length; ++i) {
                                        if (typeof d.logo_images[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.logo_images: object expected");
                                        m.logo_images[i] = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.logo_images[i]);
                                    }
                                }
                                if (d.headlines) {
                                    if (!Array.isArray(d.headlines))
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.headlines: array expected");
                                    m.headlines = [];
                                    for (var i = 0; i < d.headlines.length; ++i) {
                                        if (typeof d.headlines[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.headlines: object expected");
                                        m.headlines[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headlines[i]);
                                    }
                                }
                                if (d.descriptions) {
                                    if (!Array.isArray(d.descriptions))
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.descriptions: array expected");
                                    m.descriptions = [];
                                    for (var i = 0; i < d.descriptions.length; ++i) {
                                        if (typeof d.descriptions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.descriptions: object expected");
                                        m.descriptions[i] = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.descriptions[i]);
                                    }
                                }
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.call_to_action_text != null) {
                                    m.call_to_action_text = String(d.call_to_action_text);
                                }
                                if (d.lead_form_only != null) {
                                    m.lead_form_only = Boolean(d.lead_form_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DiscoveryMultiAssetAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo} m DiscoveryMultiAssetAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DiscoveryMultiAssetAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.marketing_images = [];
                                    d.square_marketing_images = [];
                                    d.portrait_marketing_images = [];
                                    d.logo_images = [];
                                    d.headlines = [];
                                    d.descriptions = [];
                                }
                                if (m.marketing_images && m.marketing_images.length) {
                                    d.marketing_images = [];
                                    for (var j = 0; j < m.marketing_images.length; ++j) {
                                        d.marketing_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.marketing_images[j], o);
                                    }
                                }
                                if (m.square_marketing_images && m.square_marketing_images.length) {
                                    d.square_marketing_images = [];
                                    for (var j = 0; j < m.square_marketing_images.length; ++j) {
                                        d.square_marketing_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.square_marketing_images[j], o);
                                    }
                                }
                                if (m.portrait_marketing_images && m.portrait_marketing_images.length) {
                                    d.portrait_marketing_images = [];
                                    for (var j = 0; j < m.portrait_marketing_images.length; ++j) {
                                        d.portrait_marketing_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.portrait_marketing_images[j], o);
                                    }
                                }
                                if (m.logo_images && m.logo_images.length) {
                                    d.logo_images = [];
                                    for (var j = 0; j < m.logo_images.length; ++j) {
                                        d.logo_images[j] = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.logo_images[j], o);
                                    }
                                }
                                if (m.headlines && m.headlines.length) {
                                    d.headlines = [];
                                    for (var j = 0; j < m.headlines.length; ++j) {
                                        d.headlines[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headlines[j], o);
                                    }
                                }
                                if (m.descriptions && m.descriptions.length) {
                                    d.descriptions = [];
                                    for (var j = 0; j < m.descriptions.length; ++j) {
                                        d.descriptions[j] = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.descriptions[j], o);
                                    }
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d._business_name = "business_name";
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    d.call_to_action_text = m.call_to_action_text;
                                    if (o.oneofs)
                                        d._call_to_action_text = "call_to_action_text";
                                }
                                if (m.lead_form_only != null && m.hasOwnProperty("lead_form_only")) {
                                    d.lead_form_only = m.lead_form_only;
                                    if (o.oneofs)
                                        d._lead_form_only = "lead_form_only";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DiscoveryMultiAssetAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DiscoveryMultiAssetAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DiscoveryMultiAssetAdInfo;
                        })();
    
                        common.DiscoveryCarouselAdInfo = (function() {
    
                            /**
                             * Properties of a DiscoveryCarouselAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDiscoveryCarouselAdInfo
                             * @property {string|null} [business_name] DiscoveryCarouselAdInfo business_name
                             * @property {google.ads.googleads.v11.common.IAdImageAsset|null} [logo_image] DiscoveryCarouselAdInfo logo_image
                             * @property {google.ads.googleads.v11.common.IAdTextAsset|null} [headline] DiscoveryCarouselAdInfo headline
                             * @property {google.ads.googleads.v11.common.IAdTextAsset|null} [description] DiscoveryCarouselAdInfo description
                             * @property {string|null} [call_to_action_text] DiscoveryCarouselAdInfo call_to_action_text
                             * @property {Array.<google.ads.googleads.v11.common.IAdDiscoveryCarouselCardAsset>|null} [carousel_cards] DiscoveryCarouselAdInfo carousel_cards
                             */
    
                            /**
                             * Constructs a new DiscoveryCarouselAdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DiscoveryCarouselAdInfo.
                             * @implements IDiscoveryCarouselAdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselAdInfo=} [p] Properties to set
                             */
                            function DiscoveryCarouselAdInfo(p) {
                                this.carousel_cards = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DiscoveryCarouselAdInfo business_name.
                             * @member {string} business_name
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @instance
                             */
                            DiscoveryCarouselAdInfo.prototype.business_name = "";
    
                            /**
                             * DiscoveryCarouselAdInfo logo_image.
                             * @member {google.ads.googleads.v11.common.IAdImageAsset|null|undefined} logo_image
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @instance
                             */
                            DiscoveryCarouselAdInfo.prototype.logo_image = null;
    
                            /**
                             * DiscoveryCarouselAdInfo headline.
                             * @member {google.ads.googleads.v11.common.IAdTextAsset|null|undefined} headline
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @instance
                             */
                            DiscoveryCarouselAdInfo.prototype.headline = null;
    
                            /**
                             * DiscoveryCarouselAdInfo description.
                             * @member {google.ads.googleads.v11.common.IAdTextAsset|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @instance
                             */
                            DiscoveryCarouselAdInfo.prototype.description = null;
    
                            /**
                             * DiscoveryCarouselAdInfo call_to_action_text.
                             * @member {string} call_to_action_text
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @instance
                             */
                            DiscoveryCarouselAdInfo.prototype.call_to_action_text = "";
    
                            /**
                             * DiscoveryCarouselAdInfo carousel_cards.
                             * @member {Array.<google.ads.googleads.v11.common.IAdDiscoveryCarouselCardAsset>} carousel_cards
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @instance
                             */
                            DiscoveryCarouselAdInfo.prototype.carousel_cards = $util.emptyArray;
    
                            /**
                             * Creates a new DiscoveryCarouselAdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselAdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselAdInfo} DiscoveryCarouselAdInfo instance
                             */
                            DiscoveryCarouselAdInfo.create = function create(properties) {
                                return new DiscoveryCarouselAdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified DiscoveryCarouselAdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselAdInfo} m DiscoveryCarouselAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DiscoveryCarouselAdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(10).string(m.business_name);
                                if (m.logo_image != null && Object.hasOwnProperty.call(m, "logo_image"))
                                    $root.google.ads.googleads.v11.common.AdImageAsset.encode(m.logo_image, w.uint32(18).fork()).ldelim();
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.headline, w.uint32(26).fork()).ldelim();
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    $root.google.ads.googleads.v11.common.AdTextAsset.encode(m.description, w.uint32(34).fork()).ldelim();
                                if (m.call_to_action_text != null && Object.hasOwnProperty.call(m, "call_to_action_text"))
                                    w.uint32(42).string(m.call_to_action_text);
                                if (m.carousel_cards != null && m.carousel_cards.length) {
                                    for (var i = 0; i < m.carousel_cards.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset.encode(m.carousel_cards[i], w.uint32(50).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified DiscoveryCarouselAdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselAdInfo} message DiscoveryCarouselAdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DiscoveryCarouselAdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DiscoveryCarouselAdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselAdInfo} DiscoveryCarouselAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DiscoveryCarouselAdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.business_name = r.string();
                                        break;
                                    case 2:
                                        m.logo_image = $root.google.ads.googleads.v11.common.AdImageAsset.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.headline = $root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.description = $root.google.ads.googleads.v11.common.AdTextAsset.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.call_to_action_text = r.string();
                                        break;
                                    case 6:
                                        if (!(m.carousel_cards && m.carousel_cards.length))
                                            m.carousel_cards = [];
                                        m.carousel_cards.push($root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DiscoveryCarouselAdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselAdInfo} DiscoveryCarouselAdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DiscoveryCarouselAdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DiscoveryCarouselAdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DiscoveryCarouselAdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.logo_image != null && m.hasOwnProperty("logo_image")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdImageAsset.verify(m.logo_image);
                                        if (e)
                                            return "logo_image." + e;
                                    }
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.headline);
                                        if (e)
                                            return "headline." + e;
                                    }
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdTextAsset.verify(m.description);
                                        if (e)
                                            return "description." + e;
                                    }
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    if (!$util.isString(m.call_to_action_text))
                                        return "call_to_action_text: string expected";
                                }
                                if (m.carousel_cards != null && m.hasOwnProperty("carousel_cards")) {
                                    if (!Array.isArray(m.carousel_cards))
                                        return "carousel_cards: array expected";
                                    for (var i = 0; i < m.carousel_cards.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset.verify(m.carousel_cards[i]);
                                            if (e)
                                                return "carousel_cards." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DiscoveryCarouselAdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselAdInfo} DiscoveryCarouselAdInfo
                             */
                            DiscoveryCarouselAdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo();
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.logo_image != null) {
                                    if (typeof d.logo_image !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.logo_image: object expected");
                                    m.logo_image = $root.google.ads.googleads.v11.common.AdImageAsset.fromObject(d.logo_image);
                                }
                                if (d.headline != null) {
                                    if (typeof d.headline !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.headline: object expected");
                                    m.headline = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.headline);
                                }
                                if (d.description != null) {
                                    if (typeof d.description !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.description: object expected");
                                    m.description = $root.google.ads.googleads.v11.common.AdTextAsset.fromObject(d.description);
                                }
                                if (d.call_to_action_text != null) {
                                    m.call_to_action_text = String(d.call_to_action_text);
                                }
                                if (d.carousel_cards) {
                                    if (!Array.isArray(d.carousel_cards))
                                        throw TypeError(".google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.carousel_cards: array expected");
                                    m.carousel_cards = [];
                                    for (var i = 0; i < d.carousel_cards.length; ++i) {
                                        if (typeof d.carousel_cards[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.carousel_cards: object expected");
                                        m.carousel_cards[i] = $root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset.fromObject(d.carousel_cards[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DiscoveryCarouselAdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.DiscoveryCarouselAdInfo} m DiscoveryCarouselAdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DiscoveryCarouselAdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.carousel_cards = [];
                                }
                                if (o.defaults) {
                                    d.business_name = "";
                                    d.logo_image = null;
                                    d.headline = null;
                                    d.description = null;
                                    d.call_to_action_text = "";
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                }
                                if (m.logo_image != null && m.hasOwnProperty("logo_image")) {
                                    d.logo_image = $root.google.ads.googleads.v11.common.AdImageAsset.toObject(m.logo_image, o);
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.headline, o);
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = $root.google.ads.googleads.v11.common.AdTextAsset.toObject(m.description, o);
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    d.call_to_action_text = m.call_to_action_text;
                                }
                                if (m.carousel_cards && m.carousel_cards.length) {
                                    d.carousel_cards = [];
                                    for (var j = 0; j < m.carousel_cards.length; ++j) {
                                        d.carousel_cards[j] = $root.google.ads.googleads.v11.common.AdDiscoveryCarouselCardAsset.toObject(m.carousel_cards[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DiscoveryCarouselAdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselAdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DiscoveryCarouselAdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DiscoveryCarouselAdInfo;
                        })();
    
                        common.YoutubeVideoAsset = (function() {
    
                            /**
                             * Properties of a YoutubeVideoAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IYoutubeVideoAsset
                             * @property {string|null} [youtube_video_id] YoutubeVideoAsset youtube_video_id
                             * @property {string|null} [youtube_video_title] YoutubeVideoAsset youtube_video_title
                             */
    
                            /**
                             * Constructs a new YoutubeVideoAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a YoutubeVideoAsset.
                             * @implements IYoutubeVideoAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IYoutubeVideoAsset=} [p] Properties to set
                             */
                            function YoutubeVideoAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * YoutubeVideoAsset youtube_video_id.
                             * @member {string|null|undefined} youtube_video_id
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @instance
                             */
                            YoutubeVideoAsset.prototype.youtube_video_id = null;
    
                            /**
                             * YoutubeVideoAsset youtube_video_title.
                             * @member {string} youtube_video_title
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @instance
                             */
                            YoutubeVideoAsset.prototype.youtube_video_title = "";
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * YoutubeVideoAsset _youtube_video_id.
                             * @member {"youtube_video_id"|undefined} _youtube_video_id
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @instance
                             */
                            Object.defineProperty(YoutubeVideoAsset.prototype, "_youtube_video_id", {
                                get: $util.oneOfGetter($oneOfFields = ["youtube_video_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new YoutubeVideoAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IYoutubeVideoAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.YoutubeVideoAsset} YoutubeVideoAsset instance
                             */
                            YoutubeVideoAsset.create = function create(properties) {
                                return new YoutubeVideoAsset(properties);
                            };
    
                            /**
                             * Encodes the specified YoutubeVideoAsset message. Does not implicitly {@link google.ads.googleads.v11.common.YoutubeVideoAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IYoutubeVideoAsset} m YoutubeVideoAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YoutubeVideoAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.youtube_video_id != null && Object.hasOwnProperty.call(m, "youtube_video_id"))
                                    w.uint32(18).string(m.youtube_video_id);
                                if (m.youtube_video_title != null && Object.hasOwnProperty.call(m, "youtube_video_title"))
                                    w.uint32(26).string(m.youtube_video_title);
                                return w;
                            };
    
                            /**
                             * Encodes the specified YoutubeVideoAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.YoutubeVideoAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IYoutubeVideoAsset} message YoutubeVideoAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YoutubeVideoAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a YoutubeVideoAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.YoutubeVideoAsset} YoutubeVideoAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YoutubeVideoAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.YoutubeVideoAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.youtube_video_id = r.string();
                                        break;
                                    case 3:
                                        m.youtube_video_title = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a YoutubeVideoAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.YoutubeVideoAsset} YoutubeVideoAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YoutubeVideoAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a YoutubeVideoAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            YoutubeVideoAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.youtube_video_id != null && m.hasOwnProperty("youtube_video_id")) {
                                    p._youtube_video_id = 1;
                                    if (!$util.isString(m.youtube_video_id))
                                        return "youtube_video_id: string expected";
                                }
                                if (m.youtube_video_title != null && m.hasOwnProperty("youtube_video_title")) {
                                    if (!$util.isString(m.youtube_video_title))
                                        return "youtube_video_title: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a YoutubeVideoAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.YoutubeVideoAsset} YoutubeVideoAsset
                             */
                            YoutubeVideoAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.YoutubeVideoAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.YoutubeVideoAsset();
                                if (d.youtube_video_id != null) {
                                    m.youtube_video_id = String(d.youtube_video_id);
                                }
                                if (d.youtube_video_title != null) {
                                    m.youtube_video_title = String(d.youtube_video_title);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a YoutubeVideoAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.YoutubeVideoAsset} m YoutubeVideoAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            YoutubeVideoAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.youtube_video_title = "";
                                }
                                if (m.youtube_video_id != null && m.hasOwnProperty("youtube_video_id")) {
                                    d.youtube_video_id = m.youtube_video_id;
                                    if (o.oneofs)
                                        d._youtube_video_id = "youtube_video_id";
                                }
                                if (m.youtube_video_title != null && m.hasOwnProperty("youtube_video_title")) {
                                    d.youtube_video_title = m.youtube_video_title;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this YoutubeVideoAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.YoutubeVideoAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            YoutubeVideoAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return YoutubeVideoAsset;
                        })();
    
                        common.MediaBundleAsset = (function() {
    
                            /**
                             * Properties of a MediaBundleAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMediaBundleAsset
                             * @property {Uint8Array|null} [data] MediaBundleAsset data
                             */
    
                            /**
                             * Constructs a new MediaBundleAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MediaBundleAsset.
                             * @implements IMediaBundleAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMediaBundleAsset=} [p] Properties to set
                             */
                            function MediaBundleAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MediaBundleAsset data.
                             * @member {Uint8Array|null|undefined} data
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @instance
                             */
                            MediaBundleAsset.prototype.data = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MediaBundleAsset _data.
                             * @member {"data"|undefined} _data
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @instance
                             */
                            Object.defineProperty(MediaBundleAsset.prototype, "_data", {
                                get: $util.oneOfGetter($oneOfFields = ["data"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MediaBundleAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IMediaBundleAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MediaBundleAsset} MediaBundleAsset instance
                             */
                            MediaBundleAsset.create = function create(properties) {
                                return new MediaBundleAsset(properties);
                            };
    
                            /**
                             * Encodes the specified MediaBundleAsset message. Does not implicitly {@link google.ads.googleads.v11.common.MediaBundleAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IMediaBundleAsset} m MediaBundleAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaBundleAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.data != null && Object.hasOwnProperty.call(m, "data"))
                                    w.uint32(18).bytes(m.data);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaBundleAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MediaBundleAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IMediaBundleAsset} message MediaBundleAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaBundleAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaBundleAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MediaBundleAsset} MediaBundleAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaBundleAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MediaBundleAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.data = r.bytes();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaBundleAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MediaBundleAsset} MediaBundleAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaBundleAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaBundleAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaBundleAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    p._data = 1;
                                    if (!(m.data && typeof m.data.length === "number" || $util.isString(m.data)))
                                        return "data: buffer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MediaBundleAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MediaBundleAsset} MediaBundleAsset
                             */
                            MediaBundleAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MediaBundleAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MediaBundleAsset();
                                if (d.data != null) {
                                    if (typeof d.data === "string")
                                        $util.base64.decode(d.data, m.data = $util.newBuffer($util.base64.length(d.data)), 0);
                                    else if (d.data.length)
                                        m.data = d.data;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MediaBundleAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.MediaBundleAsset} m MediaBundleAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaBundleAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    d.data = o.bytes === String ? $util.base64.encode(m.data, 0, m.data.length) : o.bytes === Array ? Array.prototype.slice.call(m.data) : m.data;
                                    if (o.oneofs)
                                        d._data = "data";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MediaBundleAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MediaBundleAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaBundleAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MediaBundleAsset;
                        })();
    
                        common.ImageAsset = (function() {
    
                            /**
                             * Properties of an ImageAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IImageAsset
                             * @property {Uint8Array|null} [data] ImageAsset data
                             * @property {number|null} [file_size] ImageAsset file_size
                             * @property {google.ads.googleads.v11.enums.MimeTypeEnum.MimeType|null} [mime_type] ImageAsset mime_type
                             * @property {google.ads.googleads.v11.common.IImageDimension|null} [full_size] ImageAsset full_size
                             */
    
                            /**
                             * Constructs a new ImageAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ImageAsset.
                             * @implements IImageAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IImageAsset=} [p] Properties to set
                             */
                            function ImageAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ImageAsset data.
                             * @member {Uint8Array|null|undefined} data
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @instance
                             */
                            ImageAsset.prototype.data = null;
    
                            /**
                             * ImageAsset file_size.
                             * @member {number|null|undefined} file_size
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @instance
                             */
                            ImageAsset.prototype.file_size = null;
    
                            /**
                             * ImageAsset mime_type.
                             * @member {google.ads.googleads.v11.enums.MimeTypeEnum.MimeType} mime_type
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @instance
                             */
                            ImageAsset.prototype.mime_type = 0;
    
                            /**
                             * ImageAsset full_size.
                             * @member {google.ads.googleads.v11.common.IImageDimension|null|undefined} full_size
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @instance
                             */
                            ImageAsset.prototype.full_size = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ImageAsset _data.
                             * @member {"data"|undefined} _data
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @instance
                             */
                            Object.defineProperty(ImageAsset.prototype, "_data", {
                                get: $util.oneOfGetter($oneOfFields = ["data"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageAsset _file_size.
                             * @member {"file_size"|undefined} _file_size
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @instance
                             */
                            Object.defineProperty(ImageAsset.prototype, "_file_size", {
                                get: $util.oneOfGetter($oneOfFields = ["file_size"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ImageAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ImageAsset} ImageAsset instance
                             */
                            ImageAsset.create = function create(properties) {
                                return new ImageAsset(properties);
                            };
    
                            /**
                             * Encodes the specified ImageAsset message. Does not implicitly {@link google.ads.googleads.v11.common.ImageAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageAsset} m ImageAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.mime_type != null && Object.hasOwnProperty.call(m, "mime_type"))
                                    w.uint32(24).int32(m.mime_type);
                                if (m.full_size != null && Object.hasOwnProperty.call(m, "full_size"))
                                    $root.google.ads.googleads.v11.common.ImageDimension.encode(m.full_size, w.uint32(34).fork()).ldelim();
                                if (m.data != null && Object.hasOwnProperty.call(m, "data"))
                                    w.uint32(42).bytes(m.data);
                                if (m.file_size != null && Object.hasOwnProperty.call(m, "file_size"))
                                    w.uint32(48).int64(m.file_size);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ImageAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ImageAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageAsset} message ImageAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ImageAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ImageAsset} ImageAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ImageAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.data = r.bytes();
                                        break;
                                    case 6:
                                        m.file_size = r.int64();
                                        break;
                                    case 3:
                                        m.mime_type = r.int32();
                                        break;
                                    case 4:
                                        m.full_size = $root.google.ads.googleads.v11.common.ImageDimension.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ImageAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ImageAsset} ImageAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ImageAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ImageAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    p._data = 1;
                                    if (!(m.data && typeof m.data.length === "number" || $util.isString(m.data)))
                                        return "data: buffer expected";
                                }
                                if (m.file_size != null && m.hasOwnProperty("file_size")) {
                                    p._file_size = 1;
                                    if (!$util.isInteger(m.file_size) && !(m.file_size && $util.isInteger(m.file_size.low) && $util.isInteger(m.file_size.high)))
                                        return "file_size: integer|Long expected";
                                }
                                if (m.mime_type != null && m.hasOwnProperty("mime_type")) {
                                    switch (m.mime_type) {
                                    default:
                                        return "mime_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.full_size != null && m.hasOwnProperty("full_size")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ImageDimension.verify(m.full_size);
                                        if (e)
                                            return "full_size." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ImageAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ImageAsset} ImageAsset
                             */
                            ImageAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ImageAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ImageAsset();
                                if (d.data != null) {
                                    if (typeof d.data === "string")
                                        $util.base64.decode(d.data, m.data = $util.newBuffer($util.base64.length(d.data)), 0);
                                    else if (d.data.length)
                                        m.data = d.data;
                                }
                                if (d.file_size != null) {
                                    if ($util.Long)
                                        (m.file_size = $util.Long.fromValue(d.file_size)).unsigned = false;
                                    else if (typeof d.file_size === "string")
                                        m.file_size = parseInt(d.file_size, 10);
                                    else if (typeof d.file_size === "number")
                                        m.file_size = d.file_size;
                                    else if (typeof d.file_size === "object")
                                        m.file_size = new $util.LongBits(d.file_size.low >>> 0, d.file_size.high >>> 0).toNumber();
                                }
                                switch (d.mime_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.mime_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.mime_type = 1;
                                    break;
                                case "IMAGE_JPEG":
                                case 2:
                                    m.mime_type = 2;
                                    break;
                                case "IMAGE_GIF":
                                case 3:
                                    m.mime_type = 3;
                                    break;
                                case "IMAGE_PNG":
                                case 4:
                                    m.mime_type = 4;
                                    break;
                                case "FLASH":
                                case 5:
                                    m.mime_type = 5;
                                    break;
                                case "TEXT_HTML":
                                case 6:
                                    m.mime_type = 6;
                                    break;
                                case "PDF":
                                case 7:
                                    m.mime_type = 7;
                                    break;
                                case "MSWORD":
                                case 8:
                                    m.mime_type = 8;
                                    break;
                                case "MSEXCEL":
                                case 9:
                                    m.mime_type = 9;
                                    break;
                                case "RTF":
                                case 10:
                                    m.mime_type = 10;
                                    break;
                                case "AUDIO_WAV":
                                case 11:
                                    m.mime_type = 11;
                                    break;
                                case "AUDIO_MP3":
                                case 12:
                                    m.mime_type = 12;
                                    break;
                                case "HTML5_AD_ZIP":
                                case 13:
                                    m.mime_type = 13;
                                    break;
                                }
                                if (d.full_size != null) {
                                    if (typeof d.full_size !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ImageAsset.full_size: object expected");
                                    m.full_size = $root.google.ads.googleads.v11.common.ImageDimension.fromObject(d.full_size);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ImageAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ImageAsset} m ImageAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ImageAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.mime_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.full_size = null;
                                }
                                if (m.mime_type != null && m.hasOwnProperty("mime_type")) {
                                    d.mime_type = o.enums === String ? $root.google.ads.googleads.v11.enums.MimeTypeEnum.MimeType[m.mime_type] : m.mime_type;
                                }
                                if (m.full_size != null && m.hasOwnProperty("full_size")) {
                                    d.full_size = $root.google.ads.googleads.v11.common.ImageDimension.toObject(m.full_size, o);
                                }
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    d.data = o.bytes === String ? $util.base64.encode(m.data, 0, m.data.length) : o.bytes === Array ? Array.prototype.slice.call(m.data) : m.data;
                                    if (o.oneofs)
                                        d._data = "data";
                                }
                                if (m.file_size != null && m.hasOwnProperty("file_size")) {
                                    if (typeof m.file_size === "number")
                                        d.file_size = o.longs === String ? String(m.file_size) : m.file_size;
                                    else
                                        d.file_size = o.longs === String ? $util.Long.prototype.toString.call(m.file_size) : o.longs === Number ? new $util.LongBits(m.file_size.low >>> 0, m.file_size.high >>> 0).toNumber() : m.file_size;
                                    if (o.oneofs)
                                        d._file_size = "file_size";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ImageAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ImageAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ImageAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ImageAsset;
                        })();
    
                        common.ImageDimension = (function() {
    
                            /**
                             * Properties of an ImageDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IImageDimension
                             * @property {number|null} [height_pixels] ImageDimension height_pixels
                             * @property {number|null} [width_pixels] ImageDimension width_pixels
                             * @property {string|null} [url] ImageDimension url
                             */
    
                            /**
                             * Constructs a new ImageDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ImageDimension.
                             * @implements IImageDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IImageDimension=} [p] Properties to set
                             */
                            function ImageDimension(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ImageDimension height_pixels.
                             * @member {number|null|undefined} height_pixels
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @instance
                             */
                            ImageDimension.prototype.height_pixels = null;
    
                            /**
                             * ImageDimension width_pixels.
                             * @member {number|null|undefined} width_pixels
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @instance
                             */
                            ImageDimension.prototype.width_pixels = null;
    
                            /**
                             * ImageDimension url.
                             * @member {string|null|undefined} url
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @instance
                             */
                            ImageDimension.prototype.url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ImageDimension _height_pixels.
                             * @member {"height_pixels"|undefined} _height_pixels
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @instance
                             */
                            Object.defineProperty(ImageDimension.prototype, "_height_pixels", {
                                get: $util.oneOfGetter($oneOfFields = ["height_pixels"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageDimension _width_pixels.
                             * @member {"width_pixels"|undefined} _width_pixels
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @instance
                             */
                            Object.defineProperty(ImageDimension.prototype, "_width_pixels", {
                                get: $util.oneOfGetter($oneOfFields = ["width_pixels"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ImageDimension _url.
                             * @member {"url"|undefined} _url
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @instance
                             */
                            Object.defineProperty(ImageDimension.prototype, "_url", {
                                get: $util.oneOfGetter($oneOfFields = ["url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ImageDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ImageDimension} ImageDimension instance
                             */
                            ImageDimension.create = function create(properties) {
                                return new ImageDimension(properties);
                            };
    
                            /**
                             * Encodes the specified ImageDimension message. Does not implicitly {@link google.ads.googleads.v11.common.ImageDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageDimension} m ImageDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.height_pixels != null && Object.hasOwnProperty.call(m, "height_pixels"))
                                    w.uint32(32).int64(m.height_pixels);
                                if (m.width_pixels != null && Object.hasOwnProperty.call(m, "width_pixels"))
                                    w.uint32(40).int64(m.width_pixels);
                                if (m.url != null && Object.hasOwnProperty.call(m, "url"))
                                    w.uint32(50).string(m.url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ImageDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ImageDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageDimension} message ImageDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ImageDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ImageDimension} ImageDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ImageDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.height_pixels = r.int64();
                                        break;
                                    case 5:
                                        m.width_pixels = r.int64();
                                        break;
                                    case 6:
                                        m.url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ImageDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ImageDimension} ImageDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ImageDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ImageDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.height_pixels != null && m.hasOwnProperty("height_pixels")) {
                                    p._height_pixels = 1;
                                    if (!$util.isInteger(m.height_pixels) && !(m.height_pixels && $util.isInteger(m.height_pixels.low) && $util.isInteger(m.height_pixels.high)))
                                        return "height_pixels: integer|Long expected";
                                }
                                if (m.width_pixels != null && m.hasOwnProperty("width_pixels")) {
                                    p._width_pixels = 1;
                                    if (!$util.isInteger(m.width_pixels) && !(m.width_pixels && $util.isInteger(m.width_pixels.low) && $util.isInteger(m.width_pixels.high)))
                                        return "width_pixels: integer|Long expected";
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    p._url = 1;
                                    if (!$util.isString(m.url))
                                        return "url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ImageDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ImageDimension} ImageDimension
                             */
                            ImageDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ImageDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ImageDimension();
                                if (d.height_pixels != null) {
                                    if ($util.Long)
                                        (m.height_pixels = $util.Long.fromValue(d.height_pixels)).unsigned = false;
                                    else if (typeof d.height_pixels === "string")
                                        m.height_pixels = parseInt(d.height_pixels, 10);
                                    else if (typeof d.height_pixels === "number")
                                        m.height_pixels = d.height_pixels;
                                    else if (typeof d.height_pixels === "object")
                                        m.height_pixels = new $util.LongBits(d.height_pixels.low >>> 0, d.height_pixels.high >>> 0).toNumber();
                                }
                                if (d.width_pixels != null) {
                                    if ($util.Long)
                                        (m.width_pixels = $util.Long.fromValue(d.width_pixels)).unsigned = false;
                                    else if (typeof d.width_pixels === "string")
                                        m.width_pixels = parseInt(d.width_pixels, 10);
                                    else if (typeof d.width_pixels === "number")
                                        m.width_pixels = d.width_pixels;
                                    else if (typeof d.width_pixels === "object")
                                        m.width_pixels = new $util.LongBits(d.width_pixels.low >>> 0, d.width_pixels.high >>> 0).toNumber();
                                }
                                if (d.url != null) {
                                    m.url = String(d.url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ImageDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.ImageDimension} m ImageDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ImageDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.height_pixels != null && m.hasOwnProperty("height_pixels")) {
                                    if (typeof m.height_pixels === "number")
                                        d.height_pixels = o.longs === String ? String(m.height_pixels) : m.height_pixels;
                                    else
                                        d.height_pixels = o.longs === String ? $util.Long.prototype.toString.call(m.height_pixels) : o.longs === Number ? new $util.LongBits(m.height_pixels.low >>> 0, m.height_pixels.high >>> 0).toNumber() : m.height_pixels;
                                    if (o.oneofs)
                                        d._height_pixels = "height_pixels";
                                }
                                if (m.width_pixels != null && m.hasOwnProperty("width_pixels")) {
                                    if (typeof m.width_pixels === "number")
                                        d.width_pixels = o.longs === String ? String(m.width_pixels) : m.width_pixels;
                                    else
                                        d.width_pixels = o.longs === String ? $util.Long.prototype.toString.call(m.width_pixels) : o.longs === Number ? new $util.LongBits(m.width_pixels.low >>> 0, m.width_pixels.high >>> 0).toNumber() : m.width_pixels;
                                    if (o.oneofs)
                                        d._width_pixels = "width_pixels";
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    d.url = m.url;
                                    if (o.oneofs)
                                        d._url = "url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ImageDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ImageDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ImageDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ImageDimension;
                        })();
    
                        common.TextAsset = (function() {
    
                            /**
                             * Properties of a TextAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITextAsset
                             * @property {string|null} [text] TextAsset text
                             */
    
                            /**
                             * Constructs a new TextAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TextAsset.
                             * @implements ITextAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITextAsset=} [p] Properties to set
                             */
                            function TextAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TextAsset text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @instance
                             */
                            TextAsset.prototype.text = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TextAsset _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @instance
                             */
                            Object.defineProperty(TextAsset.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TextAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TextAsset} TextAsset instance
                             */
                            TextAsset.create = function create(properties) {
                                return new TextAsset(properties);
                            };
    
                            /**
                             * Encodes the specified TextAsset message. Does not implicitly {@link google.ads.googleads.v11.common.TextAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextAsset} m TextAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(18).string(m.text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TextAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TextAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextAsset} message TextAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TextAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TextAsset} TextAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TextAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.text = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TextAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TextAsset} TextAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TextAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TextAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TextAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TextAsset} TextAsset
                             */
                            TextAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TextAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TextAsset();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TextAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.TextAsset} m TextAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TextAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TextAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TextAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TextAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TextAsset;
                        })();
    
                        common.LeadFormAsset = (function() {
    
                            /**
                             * Properties of a LeadFormAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILeadFormAsset
                             * @property {string|null} [business_name] LeadFormAsset business_name
                             * @property {google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum.LeadFormCallToActionType|null} [call_to_action_type] LeadFormAsset call_to_action_type
                             * @property {string|null} [call_to_action_description] LeadFormAsset call_to_action_description
                             * @property {string|null} [headline] LeadFormAsset headline
                             * @property {string|null} [description] LeadFormAsset description
                             * @property {string|null} [privacy_policy_url] LeadFormAsset privacy_policy_url
                             * @property {string|null} [post_submit_headline] LeadFormAsset post_submit_headline
                             * @property {string|null} [post_submit_description] LeadFormAsset post_submit_description
                             * @property {Array.<google.ads.googleads.v11.common.ILeadFormField>|null} [fields] LeadFormAsset fields
                             * @property {Array.<google.ads.googleads.v11.common.ILeadFormCustomQuestionField>|null} [custom_question_fields] LeadFormAsset custom_question_fields
                             * @property {Array.<google.ads.googleads.v11.common.ILeadFormDeliveryMethod>|null} [delivery_methods] LeadFormAsset delivery_methods
                             * @property {google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum.LeadFormPostSubmitCallToActionType|null} [post_submit_call_to_action_type] LeadFormAsset post_submit_call_to_action_type
                             * @property {string|null} [background_image_asset] LeadFormAsset background_image_asset
                             * @property {google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum.LeadFormDesiredIntent|null} [desired_intent] LeadFormAsset desired_intent
                             * @property {string|null} [custom_disclosure] LeadFormAsset custom_disclosure
                             */
    
                            /**
                             * Constructs a new LeadFormAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LeadFormAsset.
                             * @implements ILeadFormAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILeadFormAsset=} [p] Properties to set
                             */
                            function LeadFormAsset(p) {
                                this.fields = [];
                                this.custom_question_fields = [];
                                this.delivery_methods = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LeadFormAsset business_name.
                             * @member {string} business_name
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.business_name = "";
    
                            /**
                             * LeadFormAsset call_to_action_type.
                             * @member {google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum.LeadFormCallToActionType} call_to_action_type
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.call_to_action_type = 0;
    
                            /**
                             * LeadFormAsset call_to_action_description.
                             * @member {string} call_to_action_description
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.call_to_action_description = "";
    
                            /**
                             * LeadFormAsset headline.
                             * @member {string} headline
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.headline = "";
    
                            /**
                             * LeadFormAsset description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.description = "";
    
                            /**
                             * LeadFormAsset privacy_policy_url.
                             * @member {string} privacy_policy_url
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.privacy_policy_url = "";
    
                            /**
                             * LeadFormAsset post_submit_headline.
                             * @member {string|null|undefined} post_submit_headline
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.post_submit_headline = null;
    
                            /**
                             * LeadFormAsset post_submit_description.
                             * @member {string|null|undefined} post_submit_description
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.post_submit_description = null;
    
                            /**
                             * LeadFormAsset fields.
                             * @member {Array.<google.ads.googleads.v11.common.ILeadFormField>} fields
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.fields = $util.emptyArray;
    
                            /**
                             * LeadFormAsset custom_question_fields.
                             * @member {Array.<google.ads.googleads.v11.common.ILeadFormCustomQuestionField>} custom_question_fields
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.custom_question_fields = $util.emptyArray;
    
                            /**
                             * LeadFormAsset delivery_methods.
                             * @member {Array.<google.ads.googleads.v11.common.ILeadFormDeliveryMethod>} delivery_methods
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.delivery_methods = $util.emptyArray;
    
                            /**
                             * LeadFormAsset post_submit_call_to_action_type.
                             * @member {google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum.LeadFormPostSubmitCallToActionType} post_submit_call_to_action_type
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.post_submit_call_to_action_type = 0;
    
                            /**
                             * LeadFormAsset background_image_asset.
                             * @member {string|null|undefined} background_image_asset
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.background_image_asset = null;
    
                            /**
                             * LeadFormAsset desired_intent.
                             * @member {google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum.LeadFormDesiredIntent} desired_intent
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.desired_intent = 0;
    
                            /**
                             * LeadFormAsset custom_disclosure.
                             * @member {string|null|undefined} custom_disclosure
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            LeadFormAsset.prototype.custom_disclosure = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LeadFormAsset _post_submit_headline.
                             * @member {"post_submit_headline"|undefined} _post_submit_headline
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            Object.defineProperty(LeadFormAsset.prototype, "_post_submit_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["post_submit_headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LeadFormAsset _post_submit_description.
                             * @member {"post_submit_description"|undefined} _post_submit_description
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            Object.defineProperty(LeadFormAsset.prototype, "_post_submit_description", {
                                get: $util.oneOfGetter($oneOfFields = ["post_submit_description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LeadFormAsset _background_image_asset.
                             * @member {"background_image_asset"|undefined} _background_image_asset
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            Object.defineProperty(LeadFormAsset.prototype, "_background_image_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["background_image_asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LeadFormAsset _custom_disclosure.
                             * @member {"custom_disclosure"|undefined} _custom_disclosure
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             */
                            Object.defineProperty(LeadFormAsset.prototype, "_custom_disclosure", {
                                get: $util.oneOfGetter($oneOfFields = ["custom_disclosure"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LeadFormAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LeadFormAsset} LeadFormAsset instance
                             */
                            LeadFormAsset.create = function create(properties) {
                                return new LeadFormAsset(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormAsset message. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormAsset} m LeadFormAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.fields != null && m.fields.length) {
                                    for (var i = 0; i < m.fields.length; ++i)
                                        $root.google.ads.googleads.v11.common.LeadFormField.encode(m.fields[i], w.uint32(66).fork()).ldelim();
                                }
                                if (m.delivery_methods != null && m.delivery_methods.length) {
                                    for (var i = 0; i < m.delivery_methods.length; ++i)
                                        $root.google.ads.googleads.v11.common.LeadFormDeliveryMethod.encode(m.delivery_methods[i], w.uint32(74).fork()).ldelim();
                                }
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(82).string(m.business_name);
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(98).string(m.headline);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(106).string(m.description);
                                if (m.privacy_policy_url != null && Object.hasOwnProperty.call(m, "privacy_policy_url"))
                                    w.uint32(114).string(m.privacy_policy_url);
                                if (m.post_submit_headline != null && Object.hasOwnProperty.call(m, "post_submit_headline"))
                                    w.uint32(122).string(m.post_submit_headline);
                                if (m.post_submit_description != null && Object.hasOwnProperty.call(m, "post_submit_description"))
                                    w.uint32(130).string(m.post_submit_description);
                                if (m.call_to_action_type != null && Object.hasOwnProperty.call(m, "call_to_action_type"))
                                    w.uint32(136).int32(m.call_to_action_type);
                                if (m.call_to_action_description != null && Object.hasOwnProperty.call(m, "call_to_action_description"))
                                    w.uint32(146).string(m.call_to_action_description);
                                if (m.post_submit_call_to_action_type != null && Object.hasOwnProperty.call(m, "post_submit_call_to_action_type"))
                                    w.uint32(152).int32(m.post_submit_call_to_action_type);
                                if (m.background_image_asset != null && Object.hasOwnProperty.call(m, "background_image_asset"))
                                    w.uint32(162).string(m.background_image_asset);
                                if (m.desired_intent != null && Object.hasOwnProperty.call(m, "desired_intent"))
                                    w.uint32(168).int32(m.desired_intent);
                                if (m.custom_disclosure != null && Object.hasOwnProperty.call(m, "custom_disclosure"))
                                    w.uint32(178).string(m.custom_disclosure);
                                if (m.custom_question_fields != null && m.custom_question_fields.length) {
                                    for (var i = 0; i < m.custom_question_fields.length; ++i)
                                        $root.google.ads.googleads.v11.common.LeadFormCustomQuestionField.encode(m.custom_question_fields[i], w.uint32(186).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormAsset} message LeadFormAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LeadFormAsset} LeadFormAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LeadFormAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 10:
                                        m.business_name = r.string();
                                        break;
                                    case 17:
                                        m.call_to_action_type = r.int32();
                                        break;
                                    case 18:
                                        m.call_to_action_description = r.string();
                                        break;
                                    case 12:
                                        m.headline = r.string();
                                        break;
                                    case 13:
                                        m.description = r.string();
                                        break;
                                    case 14:
                                        m.privacy_policy_url = r.string();
                                        break;
                                    case 15:
                                        m.post_submit_headline = r.string();
                                        break;
                                    case 16:
                                        m.post_submit_description = r.string();
                                        break;
                                    case 8:
                                        if (!(m.fields && m.fields.length))
                                            m.fields = [];
                                        m.fields.push($root.google.ads.googleads.v11.common.LeadFormField.decode(r, r.uint32()));
                                        break;
                                    case 23:
                                        if (!(m.custom_question_fields && m.custom_question_fields.length))
                                            m.custom_question_fields = [];
                                        m.custom_question_fields.push($root.google.ads.googleads.v11.common.LeadFormCustomQuestionField.decode(r, r.uint32()));
                                        break;
                                    case 9:
                                        if (!(m.delivery_methods && m.delivery_methods.length))
                                            m.delivery_methods = [];
                                        m.delivery_methods.push($root.google.ads.googleads.v11.common.LeadFormDeliveryMethod.decode(r, r.uint32()));
                                        break;
                                    case 19:
                                        m.post_submit_call_to_action_type = r.int32();
                                        break;
                                    case 20:
                                        m.background_image_asset = r.string();
                                        break;
                                    case 21:
                                        m.desired_intent = r.int32();
                                        break;
                                    case 22:
                                        m.custom_disclosure = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LeadFormAsset} LeadFormAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.call_to_action_type != null && m.hasOwnProperty("call_to_action_type")) {
                                    switch (m.call_to_action_type) {
                                    default:
                                        return "call_to_action_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                        break;
                                    }
                                }
                                if (m.call_to_action_description != null && m.hasOwnProperty("call_to_action_description")) {
                                    if (!$util.isString(m.call_to_action_description))
                                        return "call_to_action_description: string expected";
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.privacy_policy_url != null && m.hasOwnProperty("privacy_policy_url")) {
                                    if (!$util.isString(m.privacy_policy_url))
                                        return "privacy_policy_url: string expected";
                                }
                                if (m.post_submit_headline != null && m.hasOwnProperty("post_submit_headline")) {
                                    p._post_submit_headline = 1;
                                    if (!$util.isString(m.post_submit_headline))
                                        return "post_submit_headline: string expected";
                                }
                                if (m.post_submit_description != null && m.hasOwnProperty("post_submit_description")) {
                                    p._post_submit_description = 1;
                                    if (!$util.isString(m.post_submit_description))
                                        return "post_submit_description: string expected";
                                }
                                if (m.fields != null && m.hasOwnProperty("fields")) {
                                    if (!Array.isArray(m.fields))
                                        return "fields: array expected";
                                    for (var i = 0; i < m.fields.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.LeadFormField.verify(m.fields[i]);
                                            if (e)
                                                return "fields." + e;
                                        }
                                    }
                                }
                                if (m.custom_question_fields != null && m.hasOwnProperty("custom_question_fields")) {
                                    if (!Array.isArray(m.custom_question_fields))
                                        return "custom_question_fields: array expected";
                                    for (var i = 0; i < m.custom_question_fields.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.LeadFormCustomQuestionField.verify(m.custom_question_fields[i]);
                                            if (e)
                                                return "custom_question_fields." + e;
                                        }
                                    }
                                }
                                if (m.delivery_methods != null && m.hasOwnProperty("delivery_methods")) {
                                    if (!Array.isArray(m.delivery_methods))
                                        return "delivery_methods: array expected";
                                    for (var i = 0; i < m.delivery_methods.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.LeadFormDeliveryMethod.verify(m.delivery_methods[i]);
                                            if (e)
                                                return "delivery_methods." + e;
                                        }
                                    }
                                }
                                if (m.post_submit_call_to_action_type != null && m.hasOwnProperty("post_submit_call_to_action_type")) {
                                    switch (m.post_submit_call_to_action_type) {
                                    default:
                                        return "post_submit_call_to_action_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.background_image_asset != null && m.hasOwnProperty("background_image_asset")) {
                                    p._background_image_asset = 1;
                                    if (!$util.isString(m.background_image_asset))
                                        return "background_image_asset: string expected";
                                }
                                if (m.desired_intent != null && m.hasOwnProperty("desired_intent")) {
                                    switch (m.desired_intent) {
                                    default:
                                        return "desired_intent: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.custom_disclosure != null && m.hasOwnProperty("custom_disclosure")) {
                                    p._custom_disclosure = 1;
                                    if (!$util.isString(m.custom_disclosure))
                                        return "custom_disclosure: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LeadFormAsset} LeadFormAsset
                             */
                            LeadFormAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LeadFormAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LeadFormAsset();
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                switch (d.call_to_action_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.call_to_action_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.call_to_action_type = 1;
                                    break;
                                case "LEARN_MORE":
                                case 2:
                                    m.call_to_action_type = 2;
                                    break;
                                case "GET_QUOTE":
                                case 3:
                                    m.call_to_action_type = 3;
                                    break;
                                case "APPLY_NOW":
                                case 4:
                                    m.call_to_action_type = 4;
                                    break;
                                case "SIGN_UP":
                                case 5:
                                    m.call_to_action_type = 5;
                                    break;
                                case "CONTACT_US":
                                case 6:
                                    m.call_to_action_type = 6;
                                    break;
                                case "SUBSCRIBE":
                                case 7:
                                    m.call_to_action_type = 7;
                                    break;
                                case "DOWNLOAD":
                                case 8:
                                    m.call_to_action_type = 8;
                                    break;
                                case "BOOK_NOW":
                                case 9:
                                    m.call_to_action_type = 9;
                                    break;
                                case "GET_OFFER":
                                case 10:
                                    m.call_to_action_type = 10;
                                    break;
                                case "REGISTER":
                                case 11:
                                    m.call_to_action_type = 11;
                                    break;
                                case "GET_INFO":
                                case 12:
                                    m.call_to_action_type = 12;
                                    break;
                                case "REQUEST_DEMO":
                                case 13:
                                    m.call_to_action_type = 13;
                                    break;
                                case "JOIN_NOW":
                                case 14:
                                    m.call_to_action_type = 14;
                                    break;
                                case "GET_STARTED":
                                case 15:
                                    m.call_to_action_type = 15;
                                    break;
                                }
                                if (d.call_to_action_description != null) {
                                    m.call_to_action_description = String(d.call_to_action_description);
                                }
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.privacy_policy_url != null) {
                                    m.privacy_policy_url = String(d.privacy_policy_url);
                                }
                                if (d.post_submit_headline != null) {
                                    m.post_submit_headline = String(d.post_submit_headline);
                                }
                                if (d.post_submit_description != null) {
                                    m.post_submit_description = String(d.post_submit_description);
                                }
                                if (d.fields) {
                                    if (!Array.isArray(d.fields))
                                        throw TypeError(".google.ads.googleads.v11.common.LeadFormAsset.fields: array expected");
                                    m.fields = [];
                                    for (var i = 0; i < d.fields.length; ++i) {
                                        if (typeof d.fields[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LeadFormAsset.fields: object expected");
                                        m.fields[i] = $root.google.ads.googleads.v11.common.LeadFormField.fromObject(d.fields[i]);
                                    }
                                }
                                if (d.custom_question_fields) {
                                    if (!Array.isArray(d.custom_question_fields))
                                        throw TypeError(".google.ads.googleads.v11.common.LeadFormAsset.custom_question_fields: array expected");
                                    m.custom_question_fields = [];
                                    for (var i = 0; i < d.custom_question_fields.length; ++i) {
                                        if (typeof d.custom_question_fields[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LeadFormAsset.custom_question_fields: object expected");
                                        m.custom_question_fields[i] = $root.google.ads.googleads.v11.common.LeadFormCustomQuestionField.fromObject(d.custom_question_fields[i]);
                                    }
                                }
                                if (d.delivery_methods) {
                                    if (!Array.isArray(d.delivery_methods))
                                        throw TypeError(".google.ads.googleads.v11.common.LeadFormAsset.delivery_methods: array expected");
                                    m.delivery_methods = [];
                                    for (var i = 0; i < d.delivery_methods.length; ++i) {
                                        if (typeof d.delivery_methods[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LeadFormAsset.delivery_methods: object expected");
                                        m.delivery_methods[i] = $root.google.ads.googleads.v11.common.LeadFormDeliveryMethod.fromObject(d.delivery_methods[i]);
                                    }
                                }
                                switch (d.post_submit_call_to_action_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.post_submit_call_to_action_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.post_submit_call_to_action_type = 1;
                                    break;
                                case "VISIT_SITE":
                                case 2:
                                    m.post_submit_call_to_action_type = 2;
                                    break;
                                case "DOWNLOAD":
                                case 3:
                                    m.post_submit_call_to_action_type = 3;
                                    break;
                                case "LEARN_MORE":
                                case 4:
                                    m.post_submit_call_to_action_type = 4;
                                    break;
                                case "SHOP_NOW":
                                case 5:
                                    m.post_submit_call_to_action_type = 5;
                                    break;
                                }
                                if (d.background_image_asset != null) {
                                    m.background_image_asset = String(d.background_image_asset);
                                }
                                switch (d.desired_intent) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.desired_intent = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.desired_intent = 1;
                                    break;
                                case "LOW_INTENT":
                                case 2:
                                    m.desired_intent = 2;
                                    break;
                                case "HIGH_INTENT":
                                case 3:
                                    m.desired_intent = 3;
                                    break;
                                }
                                if (d.custom_disclosure != null) {
                                    m.custom_disclosure = String(d.custom_disclosure);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LeadFormAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.LeadFormAsset} m LeadFormAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.fields = [];
                                    d.delivery_methods = [];
                                    d.custom_question_fields = [];
                                }
                                if (o.defaults) {
                                    d.business_name = "";
                                    d.headline = "";
                                    d.description = "";
                                    d.privacy_policy_url = "";
                                    d.call_to_action_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.call_to_action_description = "";
                                    d.post_submit_call_to_action_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.desired_intent = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.fields && m.fields.length) {
                                    d.fields = [];
                                    for (var j = 0; j < m.fields.length; ++j) {
                                        d.fields[j] = $root.google.ads.googleads.v11.common.LeadFormField.toObject(m.fields[j], o);
                                    }
                                }
                                if (m.delivery_methods && m.delivery_methods.length) {
                                    d.delivery_methods = [];
                                    for (var j = 0; j < m.delivery_methods.length; ++j) {
                                        d.delivery_methods[j] = $root.google.ads.googleads.v11.common.LeadFormDeliveryMethod.toObject(m.delivery_methods[j], o);
                                    }
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.privacy_policy_url != null && m.hasOwnProperty("privacy_policy_url")) {
                                    d.privacy_policy_url = m.privacy_policy_url;
                                }
                                if (m.post_submit_headline != null && m.hasOwnProperty("post_submit_headline")) {
                                    d.post_submit_headline = m.post_submit_headline;
                                    if (o.oneofs)
                                        d._post_submit_headline = "post_submit_headline";
                                }
                                if (m.post_submit_description != null && m.hasOwnProperty("post_submit_description")) {
                                    d.post_submit_description = m.post_submit_description;
                                    if (o.oneofs)
                                        d._post_submit_description = "post_submit_description";
                                }
                                if (m.call_to_action_type != null && m.hasOwnProperty("call_to_action_type")) {
                                    d.call_to_action_type = o.enums === String ? $root.google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum.LeadFormCallToActionType[m.call_to_action_type] : m.call_to_action_type;
                                }
                                if (m.call_to_action_description != null && m.hasOwnProperty("call_to_action_description")) {
                                    d.call_to_action_description = m.call_to_action_description;
                                }
                                if (m.post_submit_call_to_action_type != null && m.hasOwnProperty("post_submit_call_to_action_type")) {
                                    d.post_submit_call_to_action_type = o.enums === String ? $root.google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum.LeadFormPostSubmitCallToActionType[m.post_submit_call_to_action_type] : m.post_submit_call_to_action_type;
                                }
                                if (m.background_image_asset != null && m.hasOwnProperty("background_image_asset")) {
                                    d.background_image_asset = m.background_image_asset;
                                    if (o.oneofs)
                                        d._background_image_asset = "background_image_asset";
                                }
                                if (m.desired_intent != null && m.hasOwnProperty("desired_intent")) {
                                    d.desired_intent = o.enums === String ? $root.google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum.LeadFormDesiredIntent[m.desired_intent] : m.desired_intent;
                                }
                                if (m.custom_disclosure != null && m.hasOwnProperty("custom_disclosure")) {
                                    d.custom_disclosure = m.custom_disclosure;
                                    if (o.oneofs)
                                        d._custom_disclosure = "custom_disclosure";
                                }
                                if (m.custom_question_fields && m.custom_question_fields.length) {
                                    d.custom_question_fields = [];
                                    for (var j = 0; j < m.custom_question_fields.length; ++j) {
                                        d.custom_question_fields[j] = $root.google.ads.googleads.v11.common.LeadFormCustomQuestionField.toObject(m.custom_question_fields[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LeadFormAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LeadFormAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LeadFormAsset;
                        })();
    
                        common.LeadFormField = (function() {
    
                            /**
                             * Properties of a LeadFormField.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILeadFormField
                             * @property {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType|null} [input_type] LeadFormField input_type
                             * @property {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers|null} [single_choice_answers] LeadFormField single_choice_answers
                             */
    
                            /**
                             * Constructs a new LeadFormField.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LeadFormField.
                             * @implements ILeadFormField
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILeadFormField=} [p] Properties to set
                             */
                            function LeadFormField(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LeadFormField input_type.
                             * @member {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType} input_type
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @instance
                             */
                            LeadFormField.prototype.input_type = 0;
    
                            /**
                             * LeadFormField single_choice_answers.
                             * @member {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers|null|undefined} single_choice_answers
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @instance
                             */
                            LeadFormField.prototype.single_choice_answers = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LeadFormField answers.
                             * @member {"single_choice_answers"|undefined} answers
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @instance
                             */
                            Object.defineProperty(LeadFormField.prototype, "answers", {
                                get: $util.oneOfGetter($oneOfFields = ["single_choice_answers"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LeadFormField instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormField=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LeadFormField} LeadFormField instance
                             */
                            LeadFormField.create = function create(properties) {
                                return new LeadFormField(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormField message. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormField.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormField} m LeadFormField message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormField.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.input_type != null && Object.hasOwnProperty.call(m, "input_type"))
                                    w.uint32(8).int32(m.input_type);
                                if (m.single_choice_answers != null && Object.hasOwnProperty.call(m, "single_choice_answers"))
                                    $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.encode(m.single_choice_answers, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormField message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormField.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormField} message LeadFormField message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormField.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormField message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LeadFormField} LeadFormField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormField.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LeadFormField();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.input_type = r.int32();
                                        break;
                                    case 2:
                                        m.single_choice_answers = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormField message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LeadFormField} LeadFormField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormField.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormField message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormField.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.input_type != null && m.hasOwnProperty("input_type")) {
                                    switch (m.input_type) {
                                    default:
                                        return "input_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 1001:
                                    case 1002:
                                    case 1003:
                                    case 1004:
                                    case 1005:
                                    case 1009:
                                    case 1010:
                                    case 1006:
                                    case 1007:
                                    case 1008:
                                    case 1011:
                                    case 1012:
                                    case 1013:
                                    case 1014:
                                    case 1016:
                                    case 1017:
                                    case 1018:
                                    case 1019:
                                    case 1020:
                                    case 1021:
                                    case 1022:
                                    case 1023:
                                    case 1048:
                                    case 1049:
                                    case 1050:
                                    case 1024:
                                    case 1025:
                                    case 1026:
                                    case 1027:
                                    case 1028:
                                    case 1029:
                                    case 1030:
                                    case 1031:
                                    case 1033:
                                    case 1034:
                                    case 1035:
                                    case 1036:
                                    case 1038:
                                    case 1039:
                                    case 1040:
                                    case 1041:
                                    case 1042:
                                    case 1043:
                                    case 1044:
                                    case 1045:
                                    case 1046:
                                    case 1047:
                                        break;
                                    }
                                }
                                if (m.single_choice_answers != null && m.hasOwnProperty("single_choice_answers")) {
                                    p.answers = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.verify(m.single_choice_answers);
                                        if (e)
                                            return "single_choice_answers." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormField message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LeadFormField} LeadFormField
                             */
                            LeadFormField.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LeadFormField)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LeadFormField();
                                switch (d.input_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.input_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.input_type = 1;
                                    break;
                                case "FULL_NAME":
                                case 2:
                                    m.input_type = 2;
                                    break;
                                case "EMAIL":
                                case 3:
                                    m.input_type = 3;
                                    break;
                                case "PHONE_NUMBER":
                                case 4:
                                    m.input_type = 4;
                                    break;
                                case "POSTAL_CODE":
                                case 5:
                                    m.input_type = 5;
                                    break;
                                case "CITY":
                                case 9:
                                    m.input_type = 9;
                                    break;
                                case "REGION":
                                case 10:
                                    m.input_type = 10;
                                    break;
                                case "COUNTRY":
                                case 11:
                                    m.input_type = 11;
                                    break;
                                case "WORK_EMAIL":
                                case 12:
                                    m.input_type = 12;
                                    break;
                                case "COMPANY_NAME":
                                case 13:
                                    m.input_type = 13;
                                    break;
                                case "WORK_PHONE":
                                case 14:
                                    m.input_type = 14;
                                    break;
                                case "JOB_TITLE":
                                case 15:
                                    m.input_type = 15;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_CPF_BR":
                                case 16:
                                    m.input_type = 16;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_DNI_AR":
                                case 17:
                                    m.input_type = 17;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_DNI_PE":
                                case 18:
                                    m.input_type = 18;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_RUT_CL":
                                case 19:
                                    m.input_type = 19;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_CC_CO":
                                case 20:
                                    m.input_type = 20;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_CI_EC":
                                case 21:
                                    m.input_type = 21;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_RFC_MX":
                                case 22:
                                    m.input_type = 22;
                                    break;
                                case "FIRST_NAME":
                                case 23:
                                    m.input_type = 23;
                                    break;
                                case "LAST_NAME":
                                case 24:
                                    m.input_type = 24;
                                    break;
                                case "VEHICLE_MODEL":
                                case 1001:
                                    m.input_type = 1001;
                                    break;
                                case "VEHICLE_TYPE":
                                case 1002:
                                    m.input_type = 1002;
                                    break;
                                case "PREFERRED_DEALERSHIP":
                                case 1003:
                                    m.input_type = 1003;
                                    break;
                                case "VEHICLE_PURCHASE_TIMELINE":
                                case 1004:
                                    m.input_type = 1004;
                                    break;
                                case "VEHICLE_OWNERSHIP":
                                case 1005:
                                    m.input_type = 1005;
                                    break;
                                case "VEHICLE_PAYMENT_TYPE":
                                case 1009:
                                    m.input_type = 1009;
                                    break;
                                case "VEHICLE_CONDITION":
                                case 1010:
                                    m.input_type = 1010;
                                    break;
                                case "COMPANY_SIZE":
                                case 1006:
                                    m.input_type = 1006;
                                    break;
                                case "ANNUAL_SALES":
                                case 1007:
                                    m.input_type = 1007;
                                    break;
                                case "YEARS_IN_BUSINESS":
                                case 1008:
                                    m.input_type = 1008;
                                    break;
                                case "JOB_DEPARTMENT":
                                case 1011:
                                    m.input_type = 1011;
                                    break;
                                case "JOB_ROLE":
                                case 1012:
                                    m.input_type = 1012;
                                    break;
                                case "EDUCATION_PROGRAM":
                                case 1013:
                                    m.input_type = 1013;
                                    break;
                                case "EDUCATION_COURSE":
                                case 1014:
                                    m.input_type = 1014;
                                    break;
                                case "PRODUCT":
                                case 1016:
                                    m.input_type = 1016;
                                    break;
                                case "SERVICE":
                                case 1017:
                                    m.input_type = 1017;
                                    break;
                                case "OFFER":
                                case 1018:
                                    m.input_type = 1018;
                                    break;
                                case "CATEGORY":
                                case 1019:
                                    m.input_type = 1019;
                                    break;
                                case "PREFERRED_CONTACT_METHOD":
                                case 1020:
                                    m.input_type = 1020;
                                    break;
                                case "PREFERRED_LOCATION":
                                case 1021:
                                    m.input_type = 1021;
                                    break;
                                case "PREFERRED_CONTACT_TIME":
                                case 1022:
                                    m.input_type = 1022;
                                    break;
                                case "PURCHASE_TIMELINE":
                                case 1023:
                                    m.input_type = 1023;
                                    break;
                                case "YEARS_OF_EXPERIENCE":
                                case 1048:
                                    m.input_type = 1048;
                                    break;
                                case "JOB_INDUSTRY":
                                case 1049:
                                    m.input_type = 1049;
                                    break;
                                case "LEVEL_OF_EDUCATION":
                                case 1050:
                                    m.input_type = 1050;
                                    break;
                                case "PROPERTY_TYPE":
                                case 1024:
                                    m.input_type = 1024;
                                    break;
                                case "REALTOR_HELP_GOAL":
                                case 1025:
                                    m.input_type = 1025;
                                    break;
                                case "PROPERTY_COMMUNITY":
                                case 1026:
                                    m.input_type = 1026;
                                    break;
                                case "PRICE_RANGE":
                                case 1027:
                                    m.input_type = 1027;
                                    break;
                                case "NUMBER_OF_BEDROOMS":
                                case 1028:
                                    m.input_type = 1028;
                                    break;
                                case "FURNISHED_PROPERTY":
                                case 1029:
                                    m.input_type = 1029;
                                    break;
                                case "PETS_ALLOWED_PROPERTY":
                                case 1030:
                                    m.input_type = 1030;
                                    break;
                                case "NEXT_PLANNED_PURCHASE":
                                case 1031:
                                    m.input_type = 1031;
                                    break;
                                case "EVENT_SIGNUP_INTEREST":
                                case 1033:
                                    m.input_type = 1033;
                                    break;
                                case "PREFERRED_SHOPPING_PLACES":
                                case 1034:
                                    m.input_type = 1034;
                                    break;
                                case "FAVORITE_BRAND":
                                case 1035:
                                    m.input_type = 1035;
                                    break;
                                case "TRANSPORTATION_COMMERCIAL_LICENSE_TYPE":
                                case 1036:
                                    m.input_type = 1036;
                                    break;
                                case "EVENT_BOOKING_INTEREST":
                                case 1038:
                                    m.input_type = 1038;
                                    break;
                                case "DESTINATION_COUNTRY":
                                case 1039:
                                    m.input_type = 1039;
                                    break;
                                case "DESTINATION_CITY":
                                case 1040:
                                    m.input_type = 1040;
                                    break;
                                case "DEPARTURE_COUNTRY":
                                case 1041:
                                    m.input_type = 1041;
                                    break;
                                case "DEPARTURE_CITY":
                                case 1042:
                                    m.input_type = 1042;
                                    break;
                                case "DEPARTURE_DATE":
                                case 1043:
                                    m.input_type = 1043;
                                    break;
                                case "RETURN_DATE":
                                case 1044:
                                    m.input_type = 1044;
                                    break;
                                case "NUMBER_OF_TRAVELERS":
                                case 1045:
                                    m.input_type = 1045;
                                    break;
                                case "TRAVEL_BUDGET":
                                case 1046:
                                    m.input_type = 1046;
                                    break;
                                case "TRAVEL_ACCOMMODATION":
                                case 1047:
                                    m.input_type = 1047;
                                    break;
                                }
                                if (d.single_choice_answers != null) {
                                    if (typeof d.single_choice_answers !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.LeadFormField.single_choice_answers: object expected");
                                    m.single_choice_answers = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.fromObject(d.single_choice_answers);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LeadFormField message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @static
                             * @param {google.ads.googleads.v11.common.LeadFormField} m LeadFormField
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormField.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.input_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.input_type != null && m.hasOwnProperty("input_type")) {
                                    d.input_type = o.enums === String ? $root.google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType[m.input_type] : m.input_type;
                                }
                                if (m.single_choice_answers != null && m.hasOwnProperty("single_choice_answers")) {
                                    d.single_choice_answers = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.toObject(m.single_choice_answers, o);
                                    if (o.oneofs)
                                        d.answers = "single_choice_answers";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LeadFormField to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LeadFormField
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormField.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LeadFormField;
                        })();
    
                        common.LeadFormCustomQuestionField = (function() {
    
                            /**
                             * Properties of a LeadFormCustomQuestionField.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILeadFormCustomQuestionField
                             * @property {string|null} [custom_question_text] LeadFormCustomQuestionField custom_question_text
                             * @property {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers|null} [single_choice_answers] LeadFormCustomQuestionField single_choice_answers
                             */
    
                            /**
                             * Constructs a new LeadFormCustomQuestionField.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LeadFormCustomQuestionField.
                             * @implements ILeadFormCustomQuestionField
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILeadFormCustomQuestionField=} [p] Properties to set
                             */
                            function LeadFormCustomQuestionField(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LeadFormCustomQuestionField custom_question_text.
                             * @member {string} custom_question_text
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @instance
                             */
                            LeadFormCustomQuestionField.prototype.custom_question_text = "";
    
                            /**
                             * LeadFormCustomQuestionField single_choice_answers.
                             * @member {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers|null|undefined} single_choice_answers
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @instance
                             */
                            LeadFormCustomQuestionField.prototype.single_choice_answers = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LeadFormCustomQuestionField answers.
                             * @member {"single_choice_answers"|undefined} answers
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @instance
                             */
                            Object.defineProperty(LeadFormCustomQuestionField.prototype, "answers", {
                                get: $util.oneOfGetter($oneOfFields = ["single_choice_answers"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LeadFormCustomQuestionField instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormCustomQuestionField=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LeadFormCustomQuestionField} LeadFormCustomQuestionField instance
                             */
                            LeadFormCustomQuestionField.create = function create(properties) {
                                return new LeadFormCustomQuestionField(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormCustomQuestionField message. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormCustomQuestionField.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormCustomQuestionField} m LeadFormCustomQuestionField message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormCustomQuestionField.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.custom_question_text != null && Object.hasOwnProperty.call(m, "custom_question_text"))
                                    w.uint32(10).string(m.custom_question_text);
                                if (m.single_choice_answers != null && Object.hasOwnProperty.call(m, "single_choice_answers"))
                                    $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.encode(m.single_choice_answers, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormCustomQuestionField message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormCustomQuestionField.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormCustomQuestionField} message LeadFormCustomQuestionField message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormCustomQuestionField.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormCustomQuestionField message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LeadFormCustomQuestionField} LeadFormCustomQuestionField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormCustomQuestionField.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LeadFormCustomQuestionField();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.custom_question_text = r.string();
                                        break;
                                    case 2:
                                        m.single_choice_answers = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormCustomQuestionField message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LeadFormCustomQuestionField} LeadFormCustomQuestionField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormCustomQuestionField.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormCustomQuestionField message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormCustomQuestionField.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.custom_question_text != null && m.hasOwnProperty("custom_question_text")) {
                                    if (!$util.isString(m.custom_question_text))
                                        return "custom_question_text: string expected";
                                }
                                if (m.single_choice_answers != null && m.hasOwnProperty("single_choice_answers")) {
                                    p.answers = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.verify(m.single_choice_answers);
                                        if (e)
                                            return "single_choice_answers." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormCustomQuestionField message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LeadFormCustomQuestionField} LeadFormCustomQuestionField
                             */
                            LeadFormCustomQuestionField.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LeadFormCustomQuestionField)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LeadFormCustomQuestionField();
                                if (d.custom_question_text != null) {
                                    m.custom_question_text = String(d.custom_question_text);
                                }
                                if (d.single_choice_answers != null) {
                                    if (typeof d.single_choice_answers !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.LeadFormCustomQuestionField.single_choice_answers: object expected");
                                    m.single_choice_answers = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.fromObject(d.single_choice_answers);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LeadFormCustomQuestionField message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @static
                             * @param {google.ads.googleads.v11.common.LeadFormCustomQuestionField} m LeadFormCustomQuestionField
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormCustomQuestionField.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.custom_question_text = "";
                                }
                                if (m.custom_question_text != null && m.hasOwnProperty("custom_question_text")) {
                                    d.custom_question_text = m.custom_question_text;
                                }
                                if (m.single_choice_answers != null && m.hasOwnProperty("single_choice_answers")) {
                                    d.single_choice_answers = $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.toObject(m.single_choice_answers, o);
                                    if (o.oneofs)
                                        d.answers = "single_choice_answers";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LeadFormCustomQuestionField to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LeadFormCustomQuestionField
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormCustomQuestionField.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LeadFormCustomQuestionField;
                        })();
    
                        common.LeadFormSingleChoiceAnswers = (function() {
    
                            /**
                             * Properties of a LeadFormSingleChoiceAnswers.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILeadFormSingleChoiceAnswers
                             * @property {Array.<string>|null} [answers] LeadFormSingleChoiceAnswers answers
                             */
    
                            /**
                             * Constructs a new LeadFormSingleChoiceAnswers.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LeadFormSingleChoiceAnswers.
                             * @implements ILeadFormSingleChoiceAnswers
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers=} [p] Properties to set
                             */
                            function LeadFormSingleChoiceAnswers(p) {
                                this.answers = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LeadFormSingleChoiceAnswers answers.
                             * @member {Array.<string>} answers
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @instance
                             */
                            LeadFormSingleChoiceAnswers.prototype.answers = $util.emptyArray;
    
                            /**
                             * Creates a new LeadFormSingleChoiceAnswers instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers} LeadFormSingleChoiceAnswers instance
                             */
                            LeadFormSingleChoiceAnswers.create = function create(properties) {
                                return new LeadFormSingleChoiceAnswers(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormSingleChoiceAnswers message. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers} m LeadFormSingleChoiceAnswers message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormSingleChoiceAnswers.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.answers != null && m.answers.length) {
                                    for (var i = 0; i < m.answers.length; ++i)
                                        w.uint32(10).string(m.answers[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormSingleChoiceAnswers message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormSingleChoiceAnswers} message LeadFormSingleChoiceAnswers message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormSingleChoiceAnswers.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormSingleChoiceAnswers message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers} LeadFormSingleChoiceAnswers
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormSingleChoiceAnswers.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.answers && m.answers.length))
                                            m.answers = [];
                                        m.answers.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormSingleChoiceAnswers message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers} LeadFormSingleChoiceAnswers
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormSingleChoiceAnswers.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormSingleChoiceAnswers message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormSingleChoiceAnswers.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.answers != null && m.hasOwnProperty("answers")) {
                                    if (!Array.isArray(m.answers))
                                        return "answers: array expected";
                                    for (var i = 0; i < m.answers.length; ++i) {
                                        if (!$util.isString(m.answers[i]))
                                            return "answers: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormSingleChoiceAnswers message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers} LeadFormSingleChoiceAnswers
                             */
                            LeadFormSingleChoiceAnswers.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers();
                                if (d.answers) {
                                    if (!Array.isArray(d.answers))
                                        throw TypeError(".google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers.answers: array expected");
                                    m.answers = [];
                                    for (var i = 0; i < d.answers.length; ++i) {
                                        m.answers[i] = String(d.answers[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LeadFormSingleChoiceAnswers message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @static
                             * @param {google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers} m LeadFormSingleChoiceAnswers
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormSingleChoiceAnswers.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.answers = [];
                                }
                                if (m.answers && m.answers.length) {
                                    d.answers = [];
                                    for (var j = 0; j < m.answers.length; ++j) {
                                        d.answers[j] = m.answers[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LeadFormSingleChoiceAnswers to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LeadFormSingleChoiceAnswers
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormSingleChoiceAnswers.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LeadFormSingleChoiceAnswers;
                        })();
    
                        common.LeadFormDeliveryMethod = (function() {
    
                            /**
                             * Properties of a LeadFormDeliveryMethod.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILeadFormDeliveryMethod
                             * @property {google.ads.googleads.v11.common.IWebhookDelivery|null} [webhook] LeadFormDeliveryMethod webhook
                             */
    
                            /**
                             * Constructs a new LeadFormDeliveryMethod.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LeadFormDeliveryMethod.
                             * @implements ILeadFormDeliveryMethod
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILeadFormDeliveryMethod=} [p] Properties to set
                             */
                            function LeadFormDeliveryMethod(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LeadFormDeliveryMethod webhook.
                             * @member {google.ads.googleads.v11.common.IWebhookDelivery|null|undefined} webhook
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @instance
                             */
                            LeadFormDeliveryMethod.prototype.webhook = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LeadFormDeliveryMethod delivery_details.
                             * @member {"webhook"|undefined} delivery_details
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @instance
                             */
                            Object.defineProperty(LeadFormDeliveryMethod.prototype, "delivery_details", {
                                get: $util.oneOfGetter($oneOfFields = ["webhook"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LeadFormDeliveryMethod instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormDeliveryMethod=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LeadFormDeliveryMethod} LeadFormDeliveryMethod instance
                             */
                            LeadFormDeliveryMethod.create = function create(properties) {
                                return new LeadFormDeliveryMethod(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormDeliveryMethod message. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormDeliveryMethod.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormDeliveryMethod} m LeadFormDeliveryMethod message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormDeliveryMethod.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.webhook != null && Object.hasOwnProperty.call(m, "webhook"))
                                    $root.google.ads.googleads.v11.common.WebhookDelivery.encode(m.webhook, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormDeliveryMethod message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LeadFormDeliveryMethod.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {google.ads.googleads.v11.common.ILeadFormDeliveryMethod} message LeadFormDeliveryMethod message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormDeliveryMethod.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormDeliveryMethod message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LeadFormDeliveryMethod} LeadFormDeliveryMethod
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormDeliveryMethod.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LeadFormDeliveryMethod();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.webhook = $root.google.ads.googleads.v11.common.WebhookDelivery.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormDeliveryMethod message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LeadFormDeliveryMethod} LeadFormDeliveryMethod
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormDeliveryMethod.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormDeliveryMethod message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormDeliveryMethod.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.webhook != null && m.hasOwnProperty("webhook")) {
                                    p.delivery_details = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.WebhookDelivery.verify(m.webhook);
                                        if (e)
                                            return "webhook." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormDeliveryMethod message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LeadFormDeliveryMethod} LeadFormDeliveryMethod
                             */
                            LeadFormDeliveryMethod.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LeadFormDeliveryMethod)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LeadFormDeliveryMethod();
                                if (d.webhook != null) {
                                    if (typeof d.webhook !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.LeadFormDeliveryMethod.webhook: object expected");
                                    m.webhook = $root.google.ads.googleads.v11.common.WebhookDelivery.fromObject(d.webhook);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LeadFormDeliveryMethod message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @static
                             * @param {google.ads.googleads.v11.common.LeadFormDeliveryMethod} m LeadFormDeliveryMethod
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormDeliveryMethod.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.webhook != null && m.hasOwnProperty("webhook")) {
                                    d.webhook = $root.google.ads.googleads.v11.common.WebhookDelivery.toObject(m.webhook, o);
                                    if (o.oneofs)
                                        d.delivery_details = "webhook";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LeadFormDeliveryMethod to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LeadFormDeliveryMethod
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormDeliveryMethod.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LeadFormDeliveryMethod;
                        })();
    
                        common.WebhookDelivery = (function() {
    
                            /**
                             * Properties of a WebhookDelivery.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IWebhookDelivery
                             * @property {string|null} [advertiser_webhook_url] WebhookDelivery advertiser_webhook_url
                             * @property {string|null} [google_secret] WebhookDelivery google_secret
                             * @property {number|null} [payload_schema_version] WebhookDelivery payload_schema_version
                             */
    
                            /**
                             * Constructs a new WebhookDelivery.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a WebhookDelivery.
                             * @implements IWebhookDelivery
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IWebhookDelivery=} [p] Properties to set
                             */
                            function WebhookDelivery(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * WebhookDelivery advertiser_webhook_url.
                             * @member {string|null|undefined} advertiser_webhook_url
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @instance
                             */
                            WebhookDelivery.prototype.advertiser_webhook_url = null;
    
                            /**
                             * WebhookDelivery google_secret.
                             * @member {string|null|undefined} google_secret
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @instance
                             */
                            WebhookDelivery.prototype.google_secret = null;
    
                            /**
                             * WebhookDelivery payload_schema_version.
                             * @member {number|null|undefined} payload_schema_version
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @instance
                             */
                            WebhookDelivery.prototype.payload_schema_version = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * WebhookDelivery _advertiser_webhook_url.
                             * @member {"advertiser_webhook_url"|undefined} _advertiser_webhook_url
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @instance
                             */
                            Object.defineProperty(WebhookDelivery.prototype, "_advertiser_webhook_url", {
                                get: $util.oneOfGetter($oneOfFields = ["advertiser_webhook_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * WebhookDelivery _google_secret.
                             * @member {"google_secret"|undefined} _google_secret
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @instance
                             */
                            Object.defineProperty(WebhookDelivery.prototype, "_google_secret", {
                                get: $util.oneOfGetter($oneOfFields = ["google_secret"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * WebhookDelivery _payload_schema_version.
                             * @member {"payload_schema_version"|undefined} _payload_schema_version
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @instance
                             */
                            Object.defineProperty(WebhookDelivery.prototype, "_payload_schema_version", {
                                get: $util.oneOfGetter($oneOfFields = ["payload_schema_version"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new WebhookDelivery instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebhookDelivery=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.WebhookDelivery} WebhookDelivery instance
                             */
                            WebhookDelivery.create = function create(properties) {
                                return new WebhookDelivery(properties);
                            };
    
                            /**
                             * Encodes the specified WebhookDelivery message. Does not implicitly {@link google.ads.googleads.v11.common.WebhookDelivery.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebhookDelivery} m WebhookDelivery message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebhookDelivery.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.advertiser_webhook_url != null && Object.hasOwnProperty.call(m, "advertiser_webhook_url"))
                                    w.uint32(34).string(m.advertiser_webhook_url);
                                if (m.google_secret != null && Object.hasOwnProperty.call(m, "google_secret"))
                                    w.uint32(42).string(m.google_secret);
                                if (m.payload_schema_version != null && Object.hasOwnProperty.call(m, "payload_schema_version"))
                                    w.uint32(48).int64(m.payload_schema_version);
                                return w;
                            };
    
                            /**
                             * Encodes the specified WebhookDelivery message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.WebhookDelivery.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebhookDelivery} message WebhookDelivery message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebhookDelivery.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a WebhookDelivery message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.WebhookDelivery} WebhookDelivery
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebhookDelivery.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.WebhookDelivery();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.advertiser_webhook_url = r.string();
                                        break;
                                    case 5:
                                        m.google_secret = r.string();
                                        break;
                                    case 6:
                                        m.payload_schema_version = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a WebhookDelivery message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.WebhookDelivery} WebhookDelivery
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebhookDelivery.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a WebhookDelivery message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            WebhookDelivery.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.advertiser_webhook_url != null && m.hasOwnProperty("advertiser_webhook_url")) {
                                    p._advertiser_webhook_url = 1;
                                    if (!$util.isString(m.advertiser_webhook_url))
                                        return "advertiser_webhook_url: string expected";
                                }
                                if (m.google_secret != null && m.hasOwnProperty("google_secret")) {
                                    p._google_secret = 1;
                                    if (!$util.isString(m.google_secret))
                                        return "google_secret: string expected";
                                }
                                if (m.payload_schema_version != null && m.hasOwnProperty("payload_schema_version")) {
                                    p._payload_schema_version = 1;
                                    if (!$util.isInteger(m.payload_schema_version) && !(m.payload_schema_version && $util.isInteger(m.payload_schema_version.low) && $util.isInteger(m.payload_schema_version.high)))
                                        return "payload_schema_version: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a WebhookDelivery message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.WebhookDelivery} WebhookDelivery
                             */
                            WebhookDelivery.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.WebhookDelivery)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.WebhookDelivery();
                                if (d.advertiser_webhook_url != null) {
                                    m.advertiser_webhook_url = String(d.advertiser_webhook_url);
                                }
                                if (d.google_secret != null) {
                                    m.google_secret = String(d.google_secret);
                                }
                                if (d.payload_schema_version != null) {
                                    if ($util.Long)
                                        (m.payload_schema_version = $util.Long.fromValue(d.payload_schema_version)).unsigned = false;
                                    else if (typeof d.payload_schema_version === "string")
                                        m.payload_schema_version = parseInt(d.payload_schema_version, 10);
                                    else if (typeof d.payload_schema_version === "number")
                                        m.payload_schema_version = d.payload_schema_version;
                                    else if (typeof d.payload_schema_version === "object")
                                        m.payload_schema_version = new $util.LongBits(d.payload_schema_version.low >>> 0, d.payload_schema_version.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a WebhookDelivery message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @static
                             * @param {google.ads.googleads.v11.common.WebhookDelivery} m WebhookDelivery
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            WebhookDelivery.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.advertiser_webhook_url != null && m.hasOwnProperty("advertiser_webhook_url")) {
                                    d.advertiser_webhook_url = m.advertiser_webhook_url;
                                    if (o.oneofs)
                                        d._advertiser_webhook_url = "advertiser_webhook_url";
                                }
                                if (m.google_secret != null && m.hasOwnProperty("google_secret")) {
                                    d.google_secret = m.google_secret;
                                    if (o.oneofs)
                                        d._google_secret = "google_secret";
                                }
                                if (m.payload_schema_version != null && m.hasOwnProperty("payload_schema_version")) {
                                    if (typeof m.payload_schema_version === "number")
                                        d.payload_schema_version = o.longs === String ? String(m.payload_schema_version) : m.payload_schema_version;
                                    else
                                        d.payload_schema_version = o.longs === String ? $util.Long.prototype.toString.call(m.payload_schema_version) : o.longs === Number ? new $util.LongBits(m.payload_schema_version.low >>> 0, m.payload_schema_version.high >>> 0).toNumber() : m.payload_schema_version;
                                    if (o.oneofs)
                                        d._payload_schema_version = "payload_schema_version";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this WebhookDelivery to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.WebhookDelivery
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            WebhookDelivery.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return WebhookDelivery;
                        })();
    
                        common.BookOnGoogleAsset = (function() {
    
                            /**
                             * Properties of a BookOnGoogleAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBookOnGoogleAsset
                             */
    
                            /**
                             * Constructs a new BookOnGoogleAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BookOnGoogleAsset.
                             * @implements IBookOnGoogleAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBookOnGoogleAsset=} [p] Properties to set
                             */
                            function BookOnGoogleAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BookOnGoogleAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IBookOnGoogleAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BookOnGoogleAsset} BookOnGoogleAsset instance
                             */
                            BookOnGoogleAsset.create = function create(properties) {
                                return new BookOnGoogleAsset(properties);
                            };
    
                            /**
                             * Encodes the specified BookOnGoogleAsset message. Does not implicitly {@link google.ads.googleads.v11.common.BookOnGoogleAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IBookOnGoogleAsset} m BookOnGoogleAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BookOnGoogleAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BookOnGoogleAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BookOnGoogleAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IBookOnGoogleAsset} message BookOnGoogleAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BookOnGoogleAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BookOnGoogleAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BookOnGoogleAsset} BookOnGoogleAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BookOnGoogleAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BookOnGoogleAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BookOnGoogleAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BookOnGoogleAsset} BookOnGoogleAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BookOnGoogleAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BookOnGoogleAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BookOnGoogleAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BookOnGoogleAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BookOnGoogleAsset} BookOnGoogleAsset
                             */
                            BookOnGoogleAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BookOnGoogleAsset)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.BookOnGoogleAsset();
                            };
    
                            /**
                             * Creates a plain object from a BookOnGoogleAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.BookOnGoogleAsset} m BookOnGoogleAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BookOnGoogleAsset.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BookOnGoogleAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BookOnGoogleAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BookOnGoogleAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BookOnGoogleAsset;
                        })();
    
                        common.PromotionAsset = (function() {
    
                            /**
                             * Properties of a PromotionAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPromotionAsset
                             * @property {string|null} [promotion_target] PromotionAsset promotion_target
                             * @property {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier|null} [discount_modifier] PromotionAsset discount_modifier
                             * @property {string|null} [redemption_start_date] PromotionAsset redemption_start_date
                             * @property {string|null} [redemption_end_date] PromotionAsset redemption_end_date
                             * @property {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.PromotionExtensionOccasion|null} [occasion] PromotionAsset occasion
                             * @property {string|null} [language_code] PromotionAsset language_code
                             * @property {string|null} [start_date] PromotionAsset start_date
                             * @property {string|null} [end_date] PromotionAsset end_date
                             * @property {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>|null} [ad_schedule_targets] PromotionAsset ad_schedule_targets
                             * @property {number|null} [percent_off] PromotionAsset percent_off
                             * @property {google.ads.googleads.v11.common.IMoney|null} [money_amount_off] PromotionAsset money_amount_off
                             * @property {string|null} [promotion_code] PromotionAsset promotion_code
                             * @property {google.ads.googleads.v11.common.IMoney|null} [orders_over_amount] PromotionAsset orders_over_amount
                             */
    
                            /**
                             * Constructs a new PromotionAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PromotionAsset.
                             * @implements IPromotionAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPromotionAsset=} [p] Properties to set
                             */
                            function PromotionAsset(p) {
                                this.ad_schedule_targets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PromotionAsset promotion_target.
                             * @member {string} promotion_target
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.promotion_target = "";
    
                            /**
                             * PromotionAsset discount_modifier.
                             * @member {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier} discount_modifier
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.discount_modifier = 0;
    
                            /**
                             * PromotionAsset redemption_start_date.
                             * @member {string} redemption_start_date
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.redemption_start_date = "";
    
                            /**
                             * PromotionAsset redemption_end_date.
                             * @member {string} redemption_end_date
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.redemption_end_date = "";
    
                            /**
                             * PromotionAsset occasion.
                             * @member {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.PromotionExtensionOccasion} occasion
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.occasion = 0;
    
                            /**
                             * PromotionAsset language_code.
                             * @member {string} language_code
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.language_code = "";
    
                            /**
                             * PromotionAsset start_date.
                             * @member {string} start_date
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.start_date = "";
    
                            /**
                             * PromotionAsset end_date.
                             * @member {string} end_date
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.end_date = "";
    
                            /**
                             * PromotionAsset ad_schedule_targets.
                             * @member {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>} ad_schedule_targets
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.ad_schedule_targets = $util.emptyArray;
    
                            /**
                             * PromotionAsset percent_off.
                             * @member {number|null|undefined} percent_off
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.percent_off = null;
    
                            /**
                             * PromotionAsset money_amount_off.
                             * @member {google.ads.googleads.v11.common.IMoney|null|undefined} money_amount_off
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.money_amount_off = null;
    
                            /**
                             * PromotionAsset promotion_code.
                             * @member {string|null|undefined} promotion_code
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.promotion_code = null;
    
                            /**
                             * PromotionAsset orders_over_amount.
                             * @member {google.ads.googleads.v11.common.IMoney|null|undefined} orders_over_amount
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            PromotionAsset.prototype.orders_over_amount = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PromotionAsset discount_type.
                             * @member {"percent_off"|"money_amount_off"|undefined} discount_type
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            Object.defineProperty(PromotionAsset.prototype, "discount_type", {
                                get: $util.oneOfGetter($oneOfFields = ["percent_off", "money_amount_off"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionAsset promotion_trigger.
                             * @member {"promotion_code"|"orders_over_amount"|undefined} promotion_trigger
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             */
                            Object.defineProperty(PromotionAsset.prototype, "promotion_trigger", {
                                get: $util.oneOfGetter($oneOfFields = ["promotion_code", "orders_over_amount"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PromotionAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPromotionAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PromotionAsset} PromotionAsset instance
                             */
                            PromotionAsset.create = function create(properties) {
                                return new PromotionAsset(properties);
                            };
    
                            /**
                             * Encodes the specified PromotionAsset message. Does not implicitly {@link google.ads.googleads.v11.common.PromotionAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPromotionAsset} m PromotionAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.promotion_target != null && Object.hasOwnProperty.call(m, "promotion_target"))
                                    w.uint32(10).string(m.promotion_target);
                                if (m.discount_modifier != null && Object.hasOwnProperty.call(m, "discount_modifier"))
                                    w.uint32(16).int32(m.discount_modifier);
                                if (m.percent_off != null && Object.hasOwnProperty.call(m, "percent_off"))
                                    w.uint32(24).int64(m.percent_off);
                                if (m.money_amount_off != null && Object.hasOwnProperty.call(m, "money_amount_off"))
                                    $root.google.ads.googleads.v11.common.Money.encode(m.money_amount_off, w.uint32(34).fork()).ldelim();
                                if (m.promotion_code != null && Object.hasOwnProperty.call(m, "promotion_code"))
                                    w.uint32(42).string(m.promotion_code);
                                if (m.orders_over_amount != null && Object.hasOwnProperty.call(m, "orders_over_amount"))
                                    $root.google.ads.googleads.v11.common.Money.encode(m.orders_over_amount, w.uint32(50).fork()).ldelim();
                                if (m.redemption_start_date != null && Object.hasOwnProperty.call(m, "redemption_start_date"))
                                    w.uint32(58).string(m.redemption_start_date);
                                if (m.redemption_end_date != null && Object.hasOwnProperty.call(m, "redemption_end_date"))
                                    w.uint32(66).string(m.redemption_end_date);
                                if (m.occasion != null && Object.hasOwnProperty.call(m, "occasion"))
                                    w.uint32(72).int32(m.occasion);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(82).string(m.language_code);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(90).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(98).string(m.end_date);
                                if (m.ad_schedule_targets != null && m.ad_schedule_targets.length) {
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedule_targets[i], w.uint32(106).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromotionAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PromotionAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPromotionAsset} message PromotionAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromotionAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PromotionAsset} PromotionAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PromotionAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.promotion_target = r.string();
                                        break;
                                    case 2:
                                        m.discount_modifier = r.int32();
                                        break;
                                    case 7:
                                        m.redemption_start_date = r.string();
                                        break;
                                    case 8:
                                        m.redemption_end_date = r.string();
                                        break;
                                    case 9:
                                        m.occasion = r.int32();
                                        break;
                                    case 10:
                                        m.language_code = r.string();
                                        break;
                                    case 11:
                                        m.start_date = r.string();
                                        break;
                                    case 12:
                                        m.end_date = r.string();
                                        break;
                                    case 13:
                                        if (!(m.ad_schedule_targets && m.ad_schedule_targets.length))
                                            m.ad_schedule_targets = [];
                                        m.ad_schedule_targets.push($root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.percent_off = r.int64();
                                        break;
                                    case 4:
                                        m.money_amount_off = $root.google.ads.googleads.v11.common.Money.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.promotion_code = r.string();
                                        break;
                                    case 6:
                                        m.orders_over_amount = $root.google.ads.googleads.v11.common.Money.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromotionAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PromotionAsset} PromotionAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromotionAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromotionAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.promotion_target != null && m.hasOwnProperty("promotion_target")) {
                                    if (!$util.isString(m.promotion_target))
                                        return "promotion_target: string expected";
                                }
                                if (m.discount_modifier != null && m.hasOwnProperty("discount_modifier")) {
                                    switch (m.discount_modifier) {
                                    default:
                                        return "discount_modifier: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.redemption_start_date != null && m.hasOwnProperty("redemption_start_date")) {
                                    if (!$util.isString(m.redemption_start_date))
                                        return "redemption_start_date: string expected";
                                }
                                if (m.redemption_end_date != null && m.hasOwnProperty("redemption_end_date")) {
                                    if (!$util.isString(m.redemption_end_date))
                                        return "redemption_end_date: string expected";
                                }
                                if (m.occasion != null && m.hasOwnProperty("occasion")) {
                                    switch (m.occasion) {
                                    default:
                                        return "occasion: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                        break;
                                    }
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.ad_schedule_targets != null && m.hasOwnProperty("ad_schedule_targets")) {
                                    if (!Array.isArray(m.ad_schedule_targets))
                                        return "ad_schedule_targets: array expected";
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedule_targets[i]);
                                            if (e)
                                                return "ad_schedule_targets." + e;
                                        }
                                    }
                                }
                                if (m.percent_off != null && m.hasOwnProperty("percent_off")) {
                                    p.discount_type = 1;
                                    if (!$util.isInteger(m.percent_off) && !(m.percent_off && $util.isInteger(m.percent_off.low) && $util.isInteger(m.percent_off.high)))
                                        return "percent_off: integer|Long expected";
                                }
                                if (m.money_amount_off != null && m.hasOwnProperty("money_amount_off")) {
                                    if (p.discount_type === 1)
                                        return "discount_type: multiple values";
                                    p.discount_type = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Money.verify(m.money_amount_off);
                                        if (e)
                                            return "money_amount_off." + e;
                                    }
                                }
                                if (m.promotion_code != null && m.hasOwnProperty("promotion_code")) {
                                    p.promotion_trigger = 1;
                                    if (!$util.isString(m.promotion_code))
                                        return "promotion_code: string expected";
                                }
                                if (m.orders_over_amount != null && m.hasOwnProperty("orders_over_amount")) {
                                    if (p.promotion_trigger === 1)
                                        return "promotion_trigger: multiple values";
                                    p.promotion_trigger = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Money.verify(m.orders_over_amount);
                                        if (e)
                                            return "orders_over_amount." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PromotionAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PromotionAsset} PromotionAsset
                             */
                            PromotionAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PromotionAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PromotionAsset();
                                if (d.promotion_target != null) {
                                    m.promotion_target = String(d.promotion_target);
                                }
                                switch (d.discount_modifier) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.discount_modifier = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.discount_modifier = 1;
                                    break;
                                case "UP_TO":
                                case 2:
                                    m.discount_modifier = 2;
                                    break;
                                }
                                if (d.redemption_start_date != null) {
                                    m.redemption_start_date = String(d.redemption_start_date);
                                }
                                if (d.redemption_end_date != null) {
                                    m.redemption_end_date = String(d.redemption_end_date);
                                }
                                switch (d.occasion) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.occasion = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.occasion = 1;
                                    break;
                                case "NEW_YEARS":
                                case 2:
                                    m.occasion = 2;
                                    break;
                                case "CHINESE_NEW_YEAR":
                                case 3:
                                    m.occasion = 3;
                                    break;
                                case "VALENTINES_DAY":
                                case 4:
                                    m.occasion = 4;
                                    break;
                                case "EASTER":
                                case 5:
                                    m.occasion = 5;
                                    break;
                                case "MOTHERS_DAY":
                                case 6:
                                    m.occasion = 6;
                                    break;
                                case "FATHERS_DAY":
                                case 7:
                                    m.occasion = 7;
                                    break;
                                case "LABOR_DAY":
                                case 8:
                                    m.occasion = 8;
                                    break;
                                case "BACK_TO_SCHOOL":
                                case 9:
                                    m.occasion = 9;
                                    break;
                                case "HALLOWEEN":
                                case 10:
                                    m.occasion = 10;
                                    break;
                                case "BLACK_FRIDAY":
                                case 11:
                                    m.occasion = 11;
                                    break;
                                case "CYBER_MONDAY":
                                case 12:
                                    m.occasion = 12;
                                    break;
                                case "CHRISTMAS":
                                case 13:
                                    m.occasion = 13;
                                    break;
                                case "BOXING_DAY":
                                case 14:
                                    m.occasion = 14;
                                    break;
                                case "INDEPENDENCE_DAY":
                                case 15:
                                    m.occasion = 15;
                                    break;
                                case "NATIONAL_DAY":
                                case 16:
                                    m.occasion = 16;
                                    break;
                                case "END_OF_SEASON":
                                case 17:
                                    m.occasion = 17;
                                    break;
                                case "WINTER_SALE":
                                case 18:
                                    m.occasion = 18;
                                    break;
                                case "SUMMER_SALE":
                                case 19:
                                    m.occasion = 19;
                                    break;
                                case "FALL_SALE":
                                case 20:
                                    m.occasion = 20;
                                    break;
                                case "SPRING_SALE":
                                case 21:
                                    m.occasion = 21;
                                    break;
                                case "RAMADAN":
                                case 22:
                                    m.occasion = 22;
                                    break;
                                case "EID_AL_FITR":
                                case 23:
                                    m.occasion = 23;
                                    break;
                                case "EID_AL_ADHA":
                                case 24:
                                    m.occasion = 24;
                                    break;
                                case "SINGLES_DAY":
                                case 25:
                                    m.occasion = 25;
                                    break;
                                case "WOMENS_DAY":
                                case 26:
                                    m.occasion = 26;
                                    break;
                                case "HOLI":
                                case 27:
                                    m.occasion = 27;
                                    break;
                                case "PARENTS_DAY":
                                case 28:
                                    m.occasion = 28;
                                    break;
                                case "ST_NICHOLAS_DAY":
                                case 29:
                                    m.occasion = 29;
                                    break;
                                case "CARNIVAL":
                                case 30:
                                    m.occasion = 30;
                                    break;
                                case "EPIPHANY":
                                case 31:
                                    m.occasion = 31;
                                    break;
                                case "ROSH_HASHANAH":
                                case 32:
                                    m.occasion = 32;
                                    break;
                                case "PASSOVER":
                                case 33:
                                    m.occasion = 33;
                                    break;
                                case "HANUKKAH":
                                case 34:
                                    m.occasion = 34;
                                    break;
                                case "DIWALI":
                                case 35:
                                    m.occasion = 35;
                                    break;
                                case "NAVRATRI":
                                case 36:
                                    m.occasion = 36;
                                    break;
                                case "SONGKRAN":
                                case 37:
                                    m.occasion = 37;
                                    break;
                                case "YEAR_END_GIFT":
                                case 38:
                                    m.occasion = 38;
                                    break;
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.ad_schedule_targets) {
                                    if (!Array.isArray(d.ad_schedule_targets))
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionAsset.ad_schedule_targets: array expected");
                                    m.ad_schedule_targets = [];
                                    for (var i = 0; i < d.ad_schedule_targets.length; ++i) {
                                        if (typeof d.ad_schedule_targets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PromotionAsset.ad_schedule_targets: object expected");
                                        m.ad_schedule_targets[i] = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedule_targets[i]);
                                    }
                                }
                                if (d.percent_off != null) {
                                    if ($util.Long)
                                        (m.percent_off = $util.Long.fromValue(d.percent_off)).unsigned = false;
                                    else if (typeof d.percent_off === "string")
                                        m.percent_off = parseInt(d.percent_off, 10);
                                    else if (typeof d.percent_off === "number")
                                        m.percent_off = d.percent_off;
                                    else if (typeof d.percent_off === "object")
                                        m.percent_off = new $util.LongBits(d.percent_off.low >>> 0, d.percent_off.high >>> 0).toNumber();
                                }
                                if (d.money_amount_off != null) {
                                    if (typeof d.money_amount_off !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionAsset.money_amount_off: object expected");
                                    m.money_amount_off = $root.google.ads.googleads.v11.common.Money.fromObject(d.money_amount_off);
                                }
                                if (d.promotion_code != null) {
                                    m.promotion_code = String(d.promotion_code);
                                }
                                if (d.orders_over_amount != null) {
                                    if (typeof d.orders_over_amount !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionAsset.orders_over_amount: object expected");
                                    m.orders_over_amount = $root.google.ads.googleads.v11.common.Money.fromObject(d.orders_over_amount);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PromotionAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.PromotionAsset} m PromotionAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromotionAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.ad_schedule_targets = [];
                                }
                                if (o.defaults) {
                                    d.promotion_target = "";
                                    d.discount_modifier = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.redemption_start_date = "";
                                    d.redemption_end_date = "";
                                    d.occasion = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.language_code = "";
                                    d.start_date = "";
                                    d.end_date = "";
                                }
                                if (m.promotion_target != null && m.hasOwnProperty("promotion_target")) {
                                    d.promotion_target = m.promotion_target;
                                }
                                if (m.discount_modifier != null && m.hasOwnProperty("discount_modifier")) {
                                    d.discount_modifier = o.enums === String ? $root.google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier[m.discount_modifier] : m.discount_modifier;
                                }
                                if (m.percent_off != null && m.hasOwnProperty("percent_off")) {
                                    if (typeof m.percent_off === "number")
                                        d.percent_off = o.longs === String ? String(m.percent_off) : m.percent_off;
                                    else
                                        d.percent_off = o.longs === String ? $util.Long.prototype.toString.call(m.percent_off) : o.longs === Number ? new $util.LongBits(m.percent_off.low >>> 0, m.percent_off.high >>> 0).toNumber() : m.percent_off;
                                    if (o.oneofs)
                                        d.discount_type = "percent_off";
                                }
                                if (m.money_amount_off != null && m.hasOwnProperty("money_amount_off")) {
                                    d.money_amount_off = $root.google.ads.googleads.v11.common.Money.toObject(m.money_amount_off, o);
                                    if (o.oneofs)
                                        d.discount_type = "money_amount_off";
                                }
                                if (m.promotion_code != null && m.hasOwnProperty("promotion_code")) {
                                    d.promotion_code = m.promotion_code;
                                    if (o.oneofs)
                                        d.promotion_trigger = "promotion_code";
                                }
                                if (m.orders_over_amount != null && m.hasOwnProperty("orders_over_amount")) {
                                    d.orders_over_amount = $root.google.ads.googleads.v11.common.Money.toObject(m.orders_over_amount, o);
                                    if (o.oneofs)
                                        d.promotion_trigger = "orders_over_amount";
                                }
                                if (m.redemption_start_date != null && m.hasOwnProperty("redemption_start_date")) {
                                    d.redemption_start_date = m.redemption_start_date;
                                }
                                if (m.redemption_end_date != null && m.hasOwnProperty("redemption_end_date")) {
                                    d.redemption_end_date = m.redemption_end_date;
                                }
                                if (m.occasion != null && m.hasOwnProperty("occasion")) {
                                    d.occasion = o.enums === String ? $root.google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.PromotionExtensionOccasion[m.occasion] : m.occasion;
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                }
                                if (m.ad_schedule_targets && m.ad_schedule_targets.length) {
                                    d.ad_schedule_targets = [];
                                    for (var j = 0; j < m.ad_schedule_targets.length; ++j) {
                                        d.ad_schedule_targets[j] = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedule_targets[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PromotionAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PromotionAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromotionAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PromotionAsset;
                        })();
    
                        common.CalloutAsset = (function() {
    
                            /**
                             * Properties of a CalloutAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICalloutAsset
                             * @property {string|null} [callout_text] CalloutAsset callout_text
                             * @property {string|null} [start_date] CalloutAsset start_date
                             * @property {string|null} [end_date] CalloutAsset end_date
                             * @property {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>|null} [ad_schedule_targets] CalloutAsset ad_schedule_targets
                             */
    
                            /**
                             * Constructs a new CalloutAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CalloutAsset.
                             * @implements ICalloutAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICalloutAsset=} [p] Properties to set
                             */
                            function CalloutAsset(p) {
                                this.ad_schedule_targets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CalloutAsset callout_text.
                             * @member {string} callout_text
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @instance
                             */
                            CalloutAsset.prototype.callout_text = "";
    
                            /**
                             * CalloutAsset start_date.
                             * @member {string} start_date
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @instance
                             */
                            CalloutAsset.prototype.start_date = "";
    
                            /**
                             * CalloutAsset end_date.
                             * @member {string} end_date
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @instance
                             */
                            CalloutAsset.prototype.end_date = "";
    
                            /**
                             * CalloutAsset ad_schedule_targets.
                             * @member {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>} ad_schedule_targets
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @instance
                             */
                            CalloutAsset.prototype.ad_schedule_targets = $util.emptyArray;
    
                            /**
                             * Creates a new CalloutAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICalloutAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CalloutAsset} CalloutAsset instance
                             */
                            CalloutAsset.create = function create(properties) {
                                return new CalloutAsset(properties);
                            };
    
                            /**
                             * Encodes the specified CalloutAsset message. Does not implicitly {@link google.ads.googleads.v11.common.CalloutAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICalloutAsset} m CalloutAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CalloutAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.callout_text != null && Object.hasOwnProperty.call(m, "callout_text"))
                                    w.uint32(10).string(m.callout_text);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(18).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(26).string(m.end_date);
                                if (m.ad_schedule_targets != null && m.ad_schedule_targets.length) {
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedule_targets[i], w.uint32(34).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CalloutAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CalloutAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICalloutAsset} message CalloutAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CalloutAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CalloutAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CalloutAsset} CalloutAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CalloutAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CalloutAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.callout_text = r.string();
                                        break;
                                    case 2:
                                        m.start_date = r.string();
                                        break;
                                    case 3:
                                        m.end_date = r.string();
                                        break;
                                    case 4:
                                        if (!(m.ad_schedule_targets && m.ad_schedule_targets.length))
                                            m.ad_schedule_targets = [];
                                        m.ad_schedule_targets.push($root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CalloutAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CalloutAsset} CalloutAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CalloutAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CalloutAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CalloutAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.callout_text != null && m.hasOwnProperty("callout_text")) {
                                    if (!$util.isString(m.callout_text))
                                        return "callout_text: string expected";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.ad_schedule_targets != null && m.hasOwnProperty("ad_schedule_targets")) {
                                    if (!Array.isArray(m.ad_schedule_targets))
                                        return "ad_schedule_targets: array expected";
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedule_targets[i]);
                                            if (e)
                                                return "ad_schedule_targets." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CalloutAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CalloutAsset} CalloutAsset
                             */
                            CalloutAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CalloutAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CalloutAsset();
                                if (d.callout_text != null) {
                                    m.callout_text = String(d.callout_text);
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.ad_schedule_targets) {
                                    if (!Array.isArray(d.ad_schedule_targets))
                                        throw TypeError(".google.ads.googleads.v11.common.CalloutAsset.ad_schedule_targets: array expected");
                                    m.ad_schedule_targets = [];
                                    for (var i = 0; i < d.ad_schedule_targets.length; ++i) {
                                        if (typeof d.ad_schedule_targets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.CalloutAsset.ad_schedule_targets: object expected");
                                        m.ad_schedule_targets[i] = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedule_targets[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CalloutAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.CalloutAsset} m CalloutAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CalloutAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.ad_schedule_targets = [];
                                }
                                if (o.defaults) {
                                    d.callout_text = "";
                                    d.start_date = "";
                                    d.end_date = "";
                                }
                                if (m.callout_text != null && m.hasOwnProperty("callout_text")) {
                                    d.callout_text = m.callout_text;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                }
                                if (m.ad_schedule_targets && m.ad_schedule_targets.length) {
                                    d.ad_schedule_targets = [];
                                    for (var j = 0; j < m.ad_schedule_targets.length; ++j) {
                                        d.ad_schedule_targets[j] = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedule_targets[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CalloutAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CalloutAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CalloutAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CalloutAsset;
                        })();
    
                        common.StructuredSnippetAsset = (function() {
    
                            /**
                             * Properties of a StructuredSnippetAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IStructuredSnippetAsset
                             * @property {string|null} [header] StructuredSnippetAsset header
                             * @property {Array.<string>|null} [values] StructuredSnippetAsset values
                             */
    
                            /**
                             * Constructs a new StructuredSnippetAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a StructuredSnippetAsset.
                             * @implements IStructuredSnippetAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetAsset=} [p] Properties to set
                             */
                            function StructuredSnippetAsset(p) {
                                this.values = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * StructuredSnippetAsset header.
                             * @member {string} header
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @instance
                             */
                            StructuredSnippetAsset.prototype.header = "";
    
                            /**
                             * StructuredSnippetAsset values.
                             * @member {Array.<string>} values
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @instance
                             */
                            StructuredSnippetAsset.prototype.values = $util.emptyArray;
    
                            /**
                             * Creates a new StructuredSnippetAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetAsset} StructuredSnippetAsset instance
                             */
                            StructuredSnippetAsset.create = function create(properties) {
                                return new StructuredSnippetAsset(properties);
                            };
    
                            /**
                             * Encodes the specified StructuredSnippetAsset message. Does not implicitly {@link google.ads.googleads.v11.common.StructuredSnippetAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetAsset} m StructuredSnippetAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StructuredSnippetAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.header != null && Object.hasOwnProperty.call(m, "header"))
                                    w.uint32(10).string(m.header);
                                if (m.values != null && m.values.length) {
                                    for (var i = 0; i < m.values.length; ++i)
                                        w.uint32(18).string(m.values[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified StructuredSnippetAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.StructuredSnippetAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetAsset} message StructuredSnippetAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StructuredSnippetAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StructuredSnippetAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetAsset} StructuredSnippetAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StructuredSnippetAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.StructuredSnippetAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.header = r.string();
                                        break;
                                    case 2:
                                        if (!(m.values && m.values.length))
                                            m.values = [];
                                        m.values.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StructuredSnippetAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetAsset} StructuredSnippetAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StructuredSnippetAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StructuredSnippetAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StructuredSnippetAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    if (!$util.isString(m.header))
                                        return "header: string expected";
                                }
                                if (m.values != null && m.hasOwnProperty("values")) {
                                    if (!Array.isArray(m.values))
                                        return "values: array expected";
                                    for (var i = 0; i < m.values.length; ++i) {
                                        if (!$util.isString(m.values[i]))
                                            return "values: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a StructuredSnippetAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetAsset} StructuredSnippetAsset
                             */
                            StructuredSnippetAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.StructuredSnippetAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.StructuredSnippetAsset();
                                if (d.header != null) {
                                    m.header = String(d.header);
                                }
                                if (d.values) {
                                    if (!Array.isArray(d.values))
                                        throw TypeError(".google.ads.googleads.v11.common.StructuredSnippetAsset.values: array expected");
                                    m.values = [];
                                    for (var i = 0; i < d.values.length; ++i) {
                                        m.values[i] = String(d.values[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a StructuredSnippetAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.StructuredSnippetAsset} m StructuredSnippetAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StructuredSnippetAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.values = [];
                                }
                                if (o.defaults) {
                                    d.header = "";
                                }
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    d.header = m.header;
                                }
                                if (m.values && m.values.length) {
                                    d.values = [];
                                    for (var j = 0; j < m.values.length; ++j) {
                                        d.values[j] = m.values[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this StructuredSnippetAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StructuredSnippetAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return StructuredSnippetAsset;
                        })();
    
                        common.SitelinkAsset = (function() {
    
                            /**
                             * Properties of a SitelinkAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ISitelinkAsset
                             * @property {string|null} [link_text] SitelinkAsset link_text
                             * @property {string|null} [description1] SitelinkAsset description1
                             * @property {string|null} [description2] SitelinkAsset description2
                             * @property {string|null} [start_date] SitelinkAsset start_date
                             * @property {string|null} [end_date] SitelinkAsset end_date
                             * @property {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>|null} [ad_schedule_targets] SitelinkAsset ad_schedule_targets
                             */
    
                            /**
                             * Constructs a new SitelinkAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a SitelinkAsset.
                             * @implements ISitelinkAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ISitelinkAsset=} [p] Properties to set
                             */
                            function SitelinkAsset(p) {
                                this.ad_schedule_targets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SitelinkAsset link_text.
                             * @member {string} link_text
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @instance
                             */
                            SitelinkAsset.prototype.link_text = "";
    
                            /**
                             * SitelinkAsset description1.
                             * @member {string} description1
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @instance
                             */
                            SitelinkAsset.prototype.description1 = "";
    
                            /**
                             * SitelinkAsset description2.
                             * @member {string} description2
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @instance
                             */
                            SitelinkAsset.prototype.description2 = "";
    
                            /**
                             * SitelinkAsset start_date.
                             * @member {string} start_date
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @instance
                             */
                            SitelinkAsset.prototype.start_date = "";
    
                            /**
                             * SitelinkAsset end_date.
                             * @member {string} end_date
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @instance
                             */
                            SitelinkAsset.prototype.end_date = "";
    
                            /**
                             * SitelinkAsset ad_schedule_targets.
                             * @member {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>} ad_schedule_targets
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @instance
                             */
                            SitelinkAsset.prototype.ad_schedule_targets = $util.emptyArray;
    
                            /**
                             * Creates a new SitelinkAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ISitelinkAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.SitelinkAsset} SitelinkAsset instance
                             */
                            SitelinkAsset.create = function create(properties) {
                                return new SitelinkAsset(properties);
                            };
    
                            /**
                             * Encodes the specified SitelinkAsset message. Does not implicitly {@link google.ads.googleads.v11.common.SitelinkAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ISitelinkAsset} m SitelinkAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SitelinkAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.link_text != null && Object.hasOwnProperty.call(m, "link_text"))
                                    w.uint32(10).string(m.link_text);
                                if (m.description1 != null && Object.hasOwnProperty.call(m, "description1"))
                                    w.uint32(18).string(m.description1);
                                if (m.description2 != null && Object.hasOwnProperty.call(m, "description2"))
                                    w.uint32(26).string(m.description2);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(34).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(42).string(m.end_date);
                                if (m.ad_schedule_targets != null && m.ad_schedule_targets.length) {
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedule_targets[i], w.uint32(50).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified SitelinkAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.SitelinkAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ISitelinkAsset} message SitelinkAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SitelinkAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SitelinkAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.SitelinkAsset} SitelinkAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SitelinkAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.SitelinkAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.link_text = r.string();
                                        break;
                                    case 2:
                                        m.description1 = r.string();
                                        break;
                                    case 3:
                                        m.description2 = r.string();
                                        break;
                                    case 4:
                                        m.start_date = r.string();
                                        break;
                                    case 5:
                                        m.end_date = r.string();
                                        break;
                                    case 6:
                                        if (!(m.ad_schedule_targets && m.ad_schedule_targets.length))
                                            m.ad_schedule_targets = [];
                                        m.ad_schedule_targets.push($root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SitelinkAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.SitelinkAsset} SitelinkAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SitelinkAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SitelinkAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SitelinkAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    if (!$util.isString(m.link_text))
                                        return "link_text: string expected";
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    if (!$util.isString(m.description1))
                                        return "description1: string expected";
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    if (!$util.isString(m.description2))
                                        return "description2: string expected";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.ad_schedule_targets != null && m.hasOwnProperty("ad_schedule_targets")) {
                                    if (!Array.isArray(m.ad_schedule_targets))
                                        return "ad_schedule_targets: array expected";
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedule_targets[i]);
                                            if (e)
                                                return "ad_schedule_targets." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SitelinkAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.SitelinkAsset} SitelinkAsset
                             */
                            SitelinkAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.SitelinkAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.SitelinkAsset();
                                if (d.link_text != null) {
                                    m.link_text = String(d.link_text);
                                }
                                if (d.description1 != null) {
                                    m.description1 = String(d.description1);
                                }
                                if (d.description2 != null) {
                                    m.description2 = String(d.description2);
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.ad_schedule_targets) {
                                    if (!Array.isArray(d.ad_schedule_targets))
                                        throw TypeError(".google.ads.googleads.v11.common.SitelinkAsset.ad_schedule_targets: array expected");
                                    m.ad_schedule_targets = [];
                                    for (var i = 0; i < d.ad_schedule_targets.length; ++i) {
                                        if (typeof d.ad_schedule_targets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.SitelinkAsset.ad_schedule_targets: object expected");
                                        m.ad_schedule_targets[i] = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedule_targets[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SitelinkAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.SitelinkAsset} m SitelinkAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SitelinkAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.ad_schedule_targets = [];
                                }
                                if (o.defaults) {
                                    d.link_text = "";
                                    d.description1 = "";
                                    d.description2 = "";
                                    d.start_date = "";
                                    d.end_date = "";
                                }
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    d.link_text = m.link_text;
                                }
                                if (m.description1 != null && m.hasOwnProperty("description1")) {
                                    d.description1 = m.description1;
                                }
                                if (m.description2 != null && m.hasOwnProperty("description2")) {
                                    d.description2 = m.description2;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                }
                                if (m.ad_schedule_targets && m.ad_schedule_targets.length) {
                                    d.ad_schedule_targets = [];
                                    for (var j = 0; j < m.ad_schedule_targets.length; ++j) {
                                        d.ad_schedule_targets[j] = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedule_targets[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SitelinkAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.SitelinkAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SitelinkAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SitelinkAsset;
                        })();
    
                        common.PageFeedAsset = (function() {
    
                            /**
                             * Properties of a PageFeedAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPageFeedAsset
                             * @property {string|null} [page_url] PageFeedAsset page_url
                             * @property {Array.<string>|null} [labels] PageFeedAsset labels
                             */
    
                            /**
                             * Constructs a new PageFeedAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PageFeedAsset.
                             * @implements IPageFeedAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPageFeedAsset=} [p] Properties to set
                             */
                            function PageFeedAsset(p) {
                                this.labels = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PageFeedAsset page_url.
                             * @member {string} page_url
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @instance
                             */
                            PageFeedAsset.prototype.page_url = "";
    
                            /**
                             * PageFeedAsset labels.
                             * @member {Array.<string>} labels
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @instance
                             */
                            PageFeedAsset.prototype.labels = $util.emptyArray;
    
                            /**
                             * Creates a new PageFeedAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPageFeedAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PageFeedAsset} PageFeedAsset instance
                             */
                            PageFeedAsset.create = function create(properties) {
                                return new PageFeedAsset(properties);
                            };
    
                            /**
                             * Encodes the specified PageFeedAsset message. Does not implicitly {@link google.ads.googleads.v11.common.PageFeedAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPageFeedAsset} m PageFeedAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PageFeedAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.page_url != null && Object.hasOwnProperty.call(m, "page_url"))
                                    w.uint32(10).string(m.page_url);
                                if (m.labels != null && m.labels.length) {
                                    for (var i = 0; i < m.labels.length; ++i)
                                        w.uint32(18).string(m.labels[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PageFeedAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PageFeedAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPageFeedAsset} message PageFeedAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PageFeedAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PageFeedAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PageFeedAsset} PageFeedAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PageFeedAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PageFeedAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.page_url = r.string();
                                        break;
                                    case 2:
                                        if (!(m.labels && m.labels.length))
                                            m.labels = [];
                                        m.labels.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PageFeedAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PageFeedAsset} PageFeedAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PageFeedAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PageFeedAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PageFeedAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.page_url != null && m.hasOwnProperty("page_url")) {
                                    if (!$util.isString(m.page_url))
                                        return "page_url: string expected";
                                }
                                if (m.labels != null && m.hasOwnProperty("labels")) {
                                    if (!Array.isArray(m.labels))
                                        return "labels: array expected";
                                    for (var i = 0; i < m.labels.length; ++i) {
                                        if (!$util.isString(m.labels[i]))
                                            return "labels: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PageFeedAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PageFeedAsset} PageFeedAsset
                             */
                            PageFeedAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PageFeedAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PageFeedAsset();
                                if (d.page_url != null) {
                                    m.page_url = String(d.page_url);
                                }
                                if (d.labels) {
                                    if (!Array.isArray(d.labels))
                                        throw TypeError(".google.ads.googleads.v11.common.PageFeedAsset.labels: array expected");
                                    m.labels = [];
                                    for (var i = 0; i < d.labels.length; ++i) {
                                        m.labels[i] = String(d.labels[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PageFeedAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.PageFeedAsset} m PageFeedAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PageFeedAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.labels = [];
                                }
                                if (o.defaults) {
                                    d.page_url = "";
                                }
                                if (m.page_url != null && m.hasOwnProperty("page_url")) {
                                    d.page_url = m.page_url;
                                }
                                if (m.labels && m.labels.length) {
                                    d.labels = [];
                                    for (var j = 0; j < m.labels.length; ++j) {
                                        d.labels[j] = m.labels[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PageFeedAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PageFeedAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PageFeedAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PageFeedAsset;
                        })();
    
                        common.DynamicEducationAsset = (function() {
    
                            /**
                             * Properties of a DynamicEducationAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicEducationAsset
                             * @property {string|null} [program_id] DynamicEducationAsset program_id
                             * @property {string|null} [location_id] DynamicEducationAsset location_id
                             * @property {string|null} [program_name] DynamicEducationAsset program_name
                             * @property {string|null} [subject] DynamicEducationAsset subject
                             * @property {string|null} [program_description] DynamicEducationAsset program_description
                             * @property {string|null} [school_name] DynamicEducationAsset school_name
                             * @property {string|null} [address] DynamicEducationAsset address
                             * @property {Array.<string>|null} [contextual_keywords] DynamicEducationAsset contextual_keywords
                             * @property {string|null} [android_app_link] DynamicEducationAsset android_app_link
                             * @property {Array.<string>|null} [similar_program_ids] DynamicEducationAsset similar_program_ids
                             * @property {string|null} [ios_app_link] DynamicEducationAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicEducationAsset ios_app_store_id
                             * @property {string|null} [thumbnail_image_url] DynamicEducationAsset thumbnail_image_url
                             * @property {string|null} [image_url] DynamicEducationAsset image_url
                             */
    
                            /**
                             * Constructs a new DynamicEducationAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicEducationAsset.
                             * @implements IDynamicEducationAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicEducationAsset=} [p] Properties to set
                             */
                            function DynamicEducationAsset(p) {
                                this.contextual_keywords = [];
                                this.similar_program_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicEducationAsset program_id.
                             * @member {string} program_id
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.program_id = "";
    
                            /**
                             * DynamicEducationAsset location_id.
                             * @member {string} location_id
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.location_id = "";
    
                            /**
                             * DynamicEducationAsset program_name.
                             * @member {string} program_name
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.program_name = "";
    
                            /**
                             * DynamicEducationAsset subject.
                             * @member {string} subject
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.subject = "";
    
                            /**
                             * DynamicEducationAsset program_description.
                             * @member {string} program_description
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.program_description = "";
    
                            /**
                             * DynamicEducationAsset school_name.
                             * @member {string} school_name
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.school_name = "";
    
                            /**
                             * DynamicEducationAsset address.
                             * @member {string} address
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.address = "";
    
                            /**
                             * DynamicEducationAsset contextual_keywords.
                             * @member {Array.<string>} contextual_keywords
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.contextual_keywords = $util.emptyArray;
    
                            /**
                             * DynamicEducationAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicEducationAsset similar_program_ids.
                             * @member {Array.<string>} similar_program_ids
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.similar_program_ids = $util.emptyArray;
    
                            /**
                             * DynamicEducationAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicEducationAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * DynamicEducationAsset thumbnail_image_url.
                             * @member {string} thumbnail_image_url
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.thumbnail_image_url = "";
    
                            /**
                             * DynamicEducationAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             */
                            DynamicEducationAsset.prototype.image_url = "";
    
                            /**
                             * Creates a new DynamicEducationAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicEducationAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicEducationAsset} DynamicEducationAsset instance
                             */
                            DynamicEducationAsset.create = function create(properties) {
                                return new DynamicEducationAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicEducationAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicEducationAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicEducationAsset} m DynamicEducationAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicEducationAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.program_id != null && Object.hasOwnProperty.call(m, "program_id"))
                                    w.uint32(10).string(m.program_id);
                                if (m.location_id != null && Object.hasOwnProperty.call(m, "location_id"))
                                    w.uint32(18).string(m.location_id);
                                if (m.program_name != null && Object.hasOwnProperty.call(m, "program_name"))
                                    w.uint32(26).string(m.program_name);
                                if (m.subject != null && Object.hasOwnProperty.call(m, "subject"))
                                    w.uint32(34).string(m.subject);
                                if (m.program_description != null && Object.hasOwnProperty.call(m, "program_description"))
                                    w.uint32(42).string(m.program_description);
                                if (m.school_name != null && Object.hasOwnProperty.call(m, "school_name"))
                                    w.uint32(50).string(m.school_name);
                                if (m.address != null && Object.hasOwnProperty.call(m, "address"))
                                    w.uint32(58).string(m.address);
                                if (m.contextual_keywords != null && m.contextual_keywords.length) {
                                    for (var i = 0; i < m.contextual_keywords.length; ++i)
                                        w.uint32(66).string(m.contextual_keywords[i]);
                                }
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(74).string(m.android_app_link);
                                if (m.similar_program_ids != null && m.similar_program_ids.length) {
                                    for (var i = 0; i < m.similar_program_ids.length; ++i)
                                        w.uint32(82).string(m.similar_program_ids[i]);
                                }
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(90).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(96).int64(m.ios_app_store_id);
                                if (m.thumbnail_image_url != null && Object.hasOwnProperty.call(m, "thumbnail_image_url"))
                                    w.uint32(106).string(m.thumbnail_image_url);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(114).string(m.image_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicEducationAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicEducationAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicEducationAsset} message DynamicEducationAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicEducationAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicEducationAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicEducationAsset} DynamicEducationAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicEducationAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicEducationAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.program_id = r.string();
                                        break;
                                    case 2:
                                        m.location_id = r.string();
                                        break;
                                    case 3:
                                        m.program_name = r.string();
                                        break;
                                    case 4:
                                        m.subject = r.string();
                                        break;
                                    case 5:
                                        m.program_description = r.string();
                                        break;
                                    case 6:
                                        m.school_name = r.string();
                                        break;
                                    case 7:
                                        m.address = r.string();
                                        break;
                                    case 8:
                                        if (!(m.contextual_keywords && m.contextual_keywords.length))
                                            m.contextual_keywords = [];
                                        m.contextual_keywords.push(r.string());
                                        break;
                                    case 9:
                                        m.android_app_link = r.string();
                                        break;
                                    case 10:
                                        if (!(m.similar_program_ids && m.similar_program_ids.length))
                                            m.similar_program_ids = [];
                                        m.similar_program_ids.push(r.string());
                                        break;
                                    case 11:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 12:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    case 13:
                                        m.thumbnail_image_url = r.string();
                                        break;
                                    case 14:
                                        m.image_url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicEducationAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicEducationAsset} DynamicEducationAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicEducationAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicEducationAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicEducationAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.program_id != null && m.hasOwnProperty("program_id")) {
                                    if (!$util.isString(m.program_id))
                                        return "program_id: string expected";
                                }
                                if (m.location_id != null && m.hasOwnProperty("location_id")) {
                                    if (!$util.isString(m.location_id))
                                        return "location_id: string expected";
                                }
                                if (m.program_name != null && m.hasOwnProperty("program_name")) {
                                    if (!$util.isString(m.program_name))
                                        return "program_name: string expected";
                                }
                                if (m.subject != null && m.hasOwnProperty("subject")) {
                                    if (!$util.isString(m.subject))
                                        return "subject: string expected";
                                }
                                if (m.program_description != null && m.hasOwnProperty("program_description")) {
                                    if (!$util.isString(m.program_description))
                                        return "program_description: string expected";
                                }
                                if (m.school_name != null && m.hasOwnProperty("school_name")) {
                                    if (!$util.isString(m.school_name))
                                        return "school_name: string expected";
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    if (!$util.isString(m.address))
                                        return "address: string expected";
                                }
                                if (m.contextual_keywords != null && m.hasOwnProperty("contextual_keywords")) {
                                    if (!Array.isArray(m.contextual_keywords))
                                        return "contextual_keywords: array expected";
                                    for (var i = 0; i < m.contextual_keywords.length; ++i) {
                                        if (!$util.isString(m.contextual_keywords[i]))
                                            return "contextual_keywords: string[] expected";
                                    }
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.similar_program_ids != null && m.hasOwnProperty("similar_program_ids")) {
                                    if (!Array.isArray(m.similar_program_ids))
                                        return "similar_program_ids: array expected";
                                    for (var i = 0; i < m.similar_program_ids.length; ++i) {
                                        if (!$util.isString(m.similar_program_ids[i]))
                                            return "similar_program_ids: string[] expected";
                                    }
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                if (m.thumbnail_image_url != null && m.hasOwnProperty("thumbnail_image_url")) {
                                    if (!$util.isString(m.thumbnail_image_url))
                                        return "thumbnail_image_url: string expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicEducationAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicEducationAsset} DynamicEducationAsset
                             */
                            DynamicEducationAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicEducationAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicEducationAsset();
                                if (d.program_id != null) {
                                    m.program_id = String(d.program_id);
                                }
                                if (d.location_id != null) {
                                    m.location_id = String(d.location_id);
                                }
                                if (d.program_name != null) {
                                    m.program_name = String(d.program_name);
                                }
                                if (d.subject != null) {
                                    m.subject = String(d.subject);
                                }
                                if (d.program_description != null) {
                                    m.program_description = String(d.program_description);
                                }
                                if (d.school_name != null) {
                                    m.school_name = String(d.school_name);
                                }
                                if (d.address != null) {
                                    m.address = String(d.address);
                                }
                                if (d.contextual_keywords) {
                                    if (!Array.isArray(d.contextual_keywords))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicEducationAsset.contextual_keywords: array expected");
                                    m.contextual_keywords = [];
                                    for (var i = 0; i < d.contextual_keywords.length; ++i) {
                                        m.contextual_keywords[i] = String(d.contextual_keywords[i]);
                                    }
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.similar_program_ids) {
                                    if (!Array.isArray(d.similar_program_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicEducationAsset.similar_program_ids: array expected");
                                    m.similar_program_ids = [];
                                    for (var i = 0; i < d.similar_program_ids.length; ++i) {
                                        m.similar_program_ids[i] = String(d.similar_program_ids[i]);
                                    }
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                if (d.thumbnail_image_url != null) {
                                    m.thumbnail_image_url = String(d.thumbnail_image_url);
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicEducationAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicEducationAsset} m DynamicEducationAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicEducationAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.contextual_keywords = [];
                                    d.similar_program_ids = [];
                                }
                                if (o.defaults) {
                                    d.program_id = "";
                                    d.location_id = "";
                                    d.program_name = "";
                                    d.subject = "";
                                    d.program_description = "";
                                    d.school_name = "";
                                    d.address = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                    d.thumbnail_image_url = "";
                                    d.image_url = "";
                                }
                                if (m.program_id != null && m.hasOwnProperty("program_id")) {
                                    d.program_id = m.program_id;
                                }
                                if (m.location_id != null && m.hasOwnProperty("location_id")) {
                                    d.location_id = m.location_id;
                                }
                                if (m.program_name != null && m.hasOwnProperty("program_name")) {
                                    d.program_name = m.program_name;
                                }
                                if (m.subject != null && m.hasOwnProperty("subject")) {
                                    d.subject = m.subject;
                                }
                                if (m.program_description != null && m.hasOwnProperty("program_description")) {
                                    d.program_description = m.program_description;
                                }
                                if (m.school_name != null && m.hasOwnProperty("school_name")) {
                                    d.school_name = m.school_name;
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    d.address = m.address;
                                }
                                if (m.contextual_keywords && m.contextual_keywords.length) {
                                    d.contextual_keywords = [];
                                    for (var j = 0; j < m.contextual_keywords.length; ++j) {
                                        d.contextual_keywords[j] = m.contextual_keywords[j];
                                    }
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.similar_program_ids && m.similar_program_ids.length) {
                                    d.similar_program_ids = [];
                                    for (var j = 0; j < m.similar_program_ids.length; ++j) {
                                        d.similar_program_ids[j] = m.similar_program_ids[j];
                                    }
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                if (m.thumbnail_image_url != null && m.hasOwnProperty("thumbnail_image_url")) {
                                    d.thumbnail_image_url = m.thumbnail_image_url;
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicEducationAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicEducationAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicEducationAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicEducationAsset;
                        })();
    
                        common.MobileAppAsset = (function() {
    
                            /**
                             * Properties of a MobileAppAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMobileAppAsset
                             * @property {string|null} [app_id] MobileAppAsset app_id
                             * @property {google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor|null} [app_store] MobileAppAsset app_store
                             * @property {string|null} [link_text] MobileAppAsset link_text
                             * @property {string|null} [start_date] MobileAppAsset start_date
                             * @property {string|null} [end_date] MobileAppAsset end_date
                             */
    
                            /**
                             * Constructs a new MobileAppAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MobileAppAsset.
                             * @implements IMobileAppAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMobileAppAsset=} [p] Properties to set
                             */
                            function MobileAppAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MobileAppAsset app_id.
                             * @member {string} app_id
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @instance
                             */
                            MobileAppAsset.prototype.app_id = "";
    
                            /**
                             * MobileAppAsset app_store.
                             * @member {google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor} app_store
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @instance
                             */
                            MobileAppAsset.prototype.app_store = 0;
    
                            /**
                             * MobileAppAsset link_text.
                             * @member {string} link_text
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @instance
                             */
                            MobileAppAsset.prototype.link_text = "";
    
                            /**
                             * MobileAppAsset start_date.
                             * @member {string} start_date
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @instance
                             */
                            MobileAppAsset.prototype.start_date = "";
    
                            /**
                             * MobileAppAsset end_date.
                             * @member {string} end_date
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @instance
                             */
                            MobileAppAsset.prototype.end_date = "";
    
                            /**
                             * Creates a new MobileAppAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileAppAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MobileAppAsset} MobileAppAsset instance
                             */
                            MobileAppAsset.create = function create(properties) {
                                return new MobileAppAsset(properties);
                            };
    
                            /**
                             * Encodes the specified MobileAppAsset message. Does not implicitly {@link google.ads.googleads.v11.common.MobileAppAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileAppAsset} m MobileAppAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                    w.uint32(10).string(m.app_id);
                                if (m.app_store != null && Object.hasOwnProperty.call(m, "app_store"))
                                    w.uint32(16).int32(m.app_store);
                                if (m.link_text != null && Object.hasOwnProperty.call(m, "link_text"))
                                    w.uint32(26).string(m.link_text);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(34).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(42).string(m.end_date);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileAppAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MobileAppAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileAppAsset} message MobileAppAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileAppAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MobileAppAsset} MobileAppAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MobileAppAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.app_id = r.string();
                                        break;
                                    case 2:
                                        m.app_store = r.int32();
                                        break;
                                    case 3:
                                        m.link_text = r.string();
                                        break;
                                    case 4:
                                        m.start_date = r.string();
                                        break;
                                    case 5:
                                        m.end_date = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileAppAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MobileAppAsset} MobileAppAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileAppAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileAppAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    if (!$util.isString(m.app_id))
                                        return "app_id: string expected";
                                }
                                if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                    switch (m.app_store) {
                                    default:
                                        return "app_store: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    if (!$util.isString(m.link_text))
                                        return "link_text: string expected";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MobileAppAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MobileAppAsset} MobileAppAsset
                             */
                            MobileAppAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MobileAppAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MobileAppAsset();
                                if (d.app_id != null) {
                                    m.app_id = String(d.app_id);
                                }
                                switch (d.app_store) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.app_store = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.app_store = 1;
                                    break;
                                case "APPLE_APP_STORE":
                                case 2:
                                    m.app_store = 2;
                                    break;
                                case "GOOGLE_APP_STORE":
                                case 3:
                                    m.app_store = 3;
                                    break;
                                }
                                if (d.link_text != null) {
                                    m.link_text = String(d.link_text);
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MobileAppAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.MobileAppAsset} m MobileAppAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileAppAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.app_id = "";
                                    d.app_store = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.link_text = "";
                                    d.start_date = "";
                                    d.end_date = "";
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    d.app_id = m.app_id;
                                }
                                if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                    d.app_store = o.enums === String ? $root.google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor[m.app_store] : m.app_store;
                                }
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    d.link_text = m.link_text;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MobileAppAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MobileAppAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileAppAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MobileAppAsset;
                        })();
    
                        common.HotelCalloutAsset = (function() {
    
                            /**
                             * Properties of a HotelCalloutAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelCalloutAsset
                             * @property {string|null} [text] HotelCalloutAsset text
                             * @property {string|null} [language_code] HotelCalloutAsset language_code
                             */
    
                            /**
                             * Constructs a new HotelCalloutAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelCalloutAsset.
                             * @implements IHotelCalloutAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelCalloutAsset=} [p] Properties to set
                             */
                            function HotelCalloutAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelCalloutAsset text.
                             * @member {string} text
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @instance
                             */
                            HotelCalloutAsset.prototype.text = "";
    
                            /**
                             * HotelCalloutAsset language_code.
                             * @member {string} language_code
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @instance
                             */
                            HotelCalloutAsset.prototype.language_code = "";
    
                            /**
                             * Creates a new HotelCalloutAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCalloutAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelCalloutAsset} HotelCalloutAsset instance
                             */
                            HotelCalloutAsset.create = function create(properties) {
                                return new HotelCalloutAsset(properties);
                            };
    
                            /**
                             * Encodes the specified HotelCalloutAsset message. Does not implicitly {@link google.ads.googleads.v11.common.HotelCalloutAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCalloutAsset} m HotelCalloutAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCalloutAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(10).string(m.text);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(18).string(m.language_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelCalloutAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelCalloutAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCalloutAsset} message HotelCalloutAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCalloutAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelCalloutAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelCalloutAsset} HotelCalloutAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCalloutAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelCalloutAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.text = r.string();
                                        break;
                                    case 2:
                                        m.language_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelCalloutAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelCalloutAsset} HotelCalloutAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCalloutAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelCalloutAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelCalloutAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelCalloutAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelCalloutAsset} HotelCalloutAsset
                             */
                            HotelCalloutAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelCalloutAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelCalloutAsset();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelCalloutAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelCalloutAsset} m HotelCalloutAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelCalloutAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.text = "";
                                    d.language_code = "";
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelCalloutAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelCalloutAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelCalloutAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelCalloutAsset;
                        })();
    
                        common.CallAsset = (function() {
    
                            /**
                             * Properties of a CallAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICallAsset
                             * @property {string|null} [country_code] CallAsset country_code
                             * @property {string|null} [phone_number] CallAsset phone_number
                             * @property {google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState|null} [call_conversion_reporting_state] CallAsset call_conversion_reporting_state
                             * @property {string|null} [call_conversion_action] CallAsset call_conversion_action
                             * @property {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>|null} [ad_schedule_targets] CallAsset ad_schedule_targets
                             */
    
                            /**
                             * Constructs a new CallAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CallAsset.
                             * @implements ICallAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICallAsset=} [p] Properties to set
                             */
                            function CallAsset(p) {
                                this.ad_schedule_targets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallAsset country_code.
                             * @member {string} country_code
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @instance
                             */
                            CallAsset.prototype.country_code = "";
    
                            /**
                             * CallAsset phone_number.
                             * @member {string} phone_number
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @instance
                             */
                            CallAsset.prototype.phone_number = "";
    
                            /**
                             * CallAsset call_conversion_reporting_state.
                             * @member {google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState} call_conversion_reporting_state
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @instance
                             */
                            CallAsset.prototype.call_conversion_reporting_state = 0;
    
                            /**
                             * CallAsset call_conversion_action.
                             * @member {string} call_conversion_action
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @instance
                             */
                            CallAsset.prototype.call_conversion_action = "";
    
                            /**
                             * CallAsset ad_schedule_targets.
                             * @member {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>} ad_schedule_targets
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @instance
                             */
                            CallAsset.prototype.ad_schedule_targets = $util.emptyArray;
    
                            /**
                             * Creates a new CallAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CallAsset} CallAsset instance
                             */
                            CallAsset.create = function create(properties) {
                                return new CallAsset(properties);
                            };
    
                            /**
                             * Encodes the specified CallAsset message. Does not implicitly {@link google.ads.googleads.v11.common.CallAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallAsset} m CallAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(10).string(m.country_code);
                                if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                    w.uint32(18).string(m.phone_number);
                                if (m.call_conversion_reporting_state != null && Object.hasOwnProperty.call(m, "call_conversion_reporting_state"))
                                    w.uint32(24).int32(m.call_conversion_reporting_state);
                                if (m.call_conversion_action != null && Object.hasOwnProperty.call(m, "call_conversion_action"))
                                    w.uint32(34).string(m.call_conversion_action);
                                if (m.ad_schedule_targets != null && m.ad_schedule_targets.length) {
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedule_targets[i], w.uint32(42).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CallAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallAsset} message CallAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CallAsset} CallAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CallAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.country_code = r.string();
                                        break;
                                    case 2:
                                        m.phone_number = r.string();
                                        break;
                                    case 3:
                                        m.call_conversion_reporting_state = r.int32();
                                        break;
                                    case 4:
                                        m.call_conversion_action = r.string();
                                        break;
                                    case 5:
                                        if (!(m.ad_schedule_targets && m.ad_schedule_targets.length))
                                            m.ad_schedule_targets = [];
                                        m.ad_schedule_targets.push($root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CallAsset} CallAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    if (!$util.isString(m.phone_number))
                                        return "phone_number: string expected";
                                }
                                if (m.call_conversion_reporting_state != null && m.hasOwnProperty("call_conversion_reporting_state")) {
                                    switch (m.call_conversion_reporting_state) {
                                    default:
                                        return "call_conversion_reporting_state: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.call_conversion_action != null && m.hasOwnProperty("call_conversion_action")) {
                                    if (!$util.isString(m.call_conversion_action))
                                        return "call_conversion_action: string expected";
                                }
                                if (m.ad_schedule_targets != null && m.hasOwnProperty("ad_schedule_targets")) {
                                    if (!Array.isArray(m.ad_schedule_targets))
                                        return "ad_schedule_targets: array expected";
                                    for (var i = 0; i < m.ad_schedule_targets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedule_targets[i]);
                                            if (e)
                                                return "ad_schedule_targets." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CallAsset} CallAsset
                             */
                            CallAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CallAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CallAsset();
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.phone_number != null) {
                                    m.phone_number = String(d.phone_number);
                                }
                                switch (d.call_conversion_reporting_state) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.call_conversion_reporting_state = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.call_conversion_reporting_state = 1;
                                    break;
                                case "DISABLED":
                                case 2:
                                    m.call_conversion_reporting_state = 2;
                                    break;
                                case "USE_ACCOUNT_LEVEL_CALL_CONVERSION_ACTION":
                                case 3:
                                    m.call_conversion_reporting_state = 3;
                                    break;
                                case "USE_RESOURCE_LEVEL_CALL_CONVERSION_ACTION":
                                case 4:
                                    m.call_conversion_reporting_state = 4;
                                    break;
                                }
                                if (d.call_conversion_action != null) {
                                    m.call_conversion_action = String(d.call_conversion_action);
                                }
                                if (d.ad_schedule_targets) {
                                    if (!Array.isArray(d.ad_schedule_targets))
                                        throw TypeError(".google.ads.googleads.v11.common.CallAsset.ad_schedule_targets: array expected");
                                    m.ad_schedule_targets = [];
                                    for (var i = 0; i < d.ad_schedule_targets.length; ++i) {
                                        if (typeof d.ad_schedule_targets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.CallAsset.ad_schedule_targets: object expected");
                                        m.ad_schedule_targets[i] = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedule_targets[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.CallAsset} m CallAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.ad_schedule_targets = [];
                                }
                                if (o.defaults) {
                                    d.country_code = "";
                                    d.phone_number = "";
                                    d.call_conversion_reporting_state = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.call_conversion_action = "";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    d.phone_number = m.phone_number;
                                }
                                if (m.call_conversion_reporting_state != null && m.hasOwnProperty("call_conversion_reporting_state")) {
                                    d.call_conversion_reporting_state = o.enums === String ? $root.google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState[m.call_conversion_reporting_state] : m.call_conversion_reporting_state;
                                }
                                if (m.call_conversion_action != null && m.hasOwnProperty("call_conversion_action")) {
                                    d.call_conversion_action = m.call_conversion_action;
                                }
                                if (m.ad_schedule_targets && m.ad_schedule_targets.length) {
                                    d.ad_schedule_targets = [];
                                    for (var j = 0; j < m.ad_schedule_targets.length; ++j) {
                                        d.ad_schedule_targets[j] = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedule_targets[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CallAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallAsset;
                        })();
    
                        common.PriceAsset = (function() {
    
                            /**
                             * Properties of a PriceAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPriceAsset
                             * @property {google.ads.googleads.v11.enums.PriceExtensionTypeEnum.PriceExtensionType|null} [type] PriceAsset type
                             * @property {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier|null} [price_qualifier] PriceAsset price_qualifier
                             * @property {string|null} [language_code] PriceAsset language_code
                             * @property {Array.<google.ads.googleads.v11.common.IPriceOffering>|null} [price_offerings] PriceAsset price_offerings
                             */
    
                            /**
                             * Constructs a new PriceAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PriceAsset.
                             * @implements IPriceAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPriceAsset=} [p] Properties to set
                             */
                            function PriceAsset(p) {
                                this.price_offerings = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PriceAsset type.
                             * @member {google.ads.googleads.v11.enums.PriceExtensionTypeEnum.PriceExtensionType} type
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @instance
                             */
                            PriceAsset.prototype.type = 0;
    
                            /**
                             * PriceAsset price_qualifier.
                             * @member {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier} price_qualifier
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @instance
                             */
                            PriceAsset.prototype.price_qualifier = 0;
    
                            /**
                             * PriceAsset language_code.
                             * @member {string} language_code
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @instance
                             */
                            PriceAsset.prototype.language_code = "";
    
                            /**
                             * PriceAsset price_offerings.
                             * @member {Array.<google.ads.googleads.v11.common.IPriceOffering>} price_offerings
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @instance
                             */
                            PriceAsset.prototype.price_offerings = $util.emptyArray;
    
                            /**
                             * Creates a new PriceAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PriceAsset} PriceAsset instance
                             */
                            PriceAsset.create = function create(properties) {
                                return new PriceAsset(properties);
                            };
    
                            /**
                             * Encodes the specified PriceAsset message. Does not implicitly {@link google.ads.googleads.v11.common.PriceAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceAsset} m PriceAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                if (m.price_qualifier != null && Object.hasOwnProperty.call(m, "price_qualifier"))
                                    w.uint32(16).int32(m.price_qualifier);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(26).string(m.language_code);
                                if (m.price_offerings != null && m.price_offerings.length) {
                                    for (var i = 0; i < m.price_offerings.length; ++i)
                                        $root.google.ads.googleads.v11.common.PriceOffering.encode(m.price_offerings[i], w.uint32(34).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PriceAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PriceAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceAsset} message PriceAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PriceAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PriceAsset} PriceAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PriceAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    case 2:
                                        m.price_qualifier = r.int32();
                                        break;
                                    case 3:
                                        m.language_code = r.string();
                                        break;
                                    case 4:
                                        if (!(m.price_offerings && m.price_offerings.length))
                                            m.price_offerings = [];
                                        m.price_offerings.push($root.google.ads.googleads.v11.common.PriceOffering.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PriceAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PriceAsset} PriceAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PriceAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PriceAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.price_qualifier != null && m.hasOwnProperty("price_qualifier")) {
                                    switch (m.price_qualifier) {
                                    default:
                                        return "price_qualifier: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.price_offerings != null && m.hasOwnProperty("price_offerings")) {
                                    if (!Array.isArray(m.price_offerings))
                                        return "price_offerings: array expected";
                                    for (var i = 0; i < m.price_offerings.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PriceOffering.verify(m.price_offerings[i]);
                                            if (e)
                                                return "price_offerings." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PriceAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PriceAsset} PriceAsset
                             */
                            PriceAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PriceAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PriceAsset();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "BRANDS":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "EVENTS":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "LOCATIONS":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "NEIGHBORHOODS":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "PRODUCT_CATEGORIES":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PRODUCT_TIERS":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "SERVICES":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "SERVICE_CATEGORIES":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "SERVICE_TIERS":
                                case 10:
                                    m.type = 10;
                                    break;
                                }
                                switch (d.price_qualifier) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.price_qualifier = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.price_qualifier = 1;
                                    break;
                                case "FROM":
                                case 2:
                                    m.price_qualifier = 2;
                                    break;
                                case "UP_TO":
                                case 3:
                                    m.price_qualifier = 3;
                                    break;
                                case "AVERAGE":
                                case 4:
                                    m.price_qualifier = 4;
                                    break;
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.price_offerings) {
                                    if (!Array.isArray(d.price_offerings))
                                        throw TypeError(".google.ads.googleads.v11.common.PriceAsset.price_offerings: array expected");
                                    m.price_offerings = [];
                                    for (var i = 0; i < d.price_offerings.length; ++i) {
                                        if (typeof d.price_offerings[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PriceAsset.price_offerings: object expected");
                                        m.price_offerings[i] = $root.google.ads.googleads.v11.common.PriceOffering.fromObject(d.price_offerings[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PriceAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.PriceAsset} m PriceAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PriceAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.price_offerings = [];
                                }
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.price_qualifier = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.language_code = "";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.PriceExtensionTypeEnum.PriceExtensionType[m.type] : m.type;
                                }
                                if (m.price_qualifier != null && m.hasOwnProperty("price_qualifier")) {
                                    d.price_qualifier = o.enums === String ? $root.google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier[m.price_qualifier] : m.price_qualifier;
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                }
                                if (m.price_offerings && m.price_offerings.length) {
                                    d.price_offerings = [];
                                    for (var j = 0; j < m.price_offerings.length; ++j) {
                                        d.price_offerings[j] = $root.google.ads.googleads.v11.common.PriceOffering.toObject(m.price_offerings[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PriceAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PriceAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PriceAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PriceAsset;
                        })();
    
                        common.PriceOffering = (function() {
    
                            /**
                             * Properties of a PriceOffering.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPriceOffering
                             * @property {string|null} [header] PriceOffering header
                             * @property {string|null} [description] PriceOffering description
                             * @property {google.ads.googleads.v11.common.IMoney|null} [price] PriceOffering price
                             * @property {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit|null} [unit] PriceOffering unit
                             * @property {string|null} [final_url] PriceOffering final_url
                             * @property {string|null} [final_mobile_url] PriceOffering final_mobile_url
                             */
    
                            /**
                             * Constructs a new PriceOffering.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PriceOffering.
                             * @implements IPriceOffering
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPriceOffering=} [p] Properties to set
                             */
                            function PriceOffering(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PriceOffering header.
                             * @member {string} header
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @instance
                             */
                            PriceOffering.prototype.header = "";
    
                            /**
                             * PriceOffering description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @instance
                             */
                            PriceOffering.prototype.description = "";
    
                            /**
                             * PriceOffering price.
                             * @member {google.ads.googleads.v11.common.IMoney|null|undefined} price
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @instance
                             */
                            PriceOffering.prototype.price = null;
    
                            /**
                             * PriceOffering unit.
                             * @member {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit} unit
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @instance
                             */
                            PriceOffering.prototype.unit = 0;
    
                            /**
                             * PriceOffering final_url.
                             * @member {string} final_url
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @instance
                             */
                            PriceOffering.prototype.final_url = "";
    
                            /**
                             * PriceOffering final_mobile_url.
                             * @member {string} final_mobile_url
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @instance
                             */
                            PriceOffering.prototype.final_mobile_url = "";
    
                            /**
                             * Creates a new PriceOffering instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceOffering=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PriceOffering} PriceOffering instance
                             */
                            PriceOffering.create = function create(properties) {
                                return new PriceOffering(properties);
                            };
    
                            /**
                             * Encodes the specified PriceOffering message. Does not implicitly {@link google.ads.googleads.v11.common.PriceOffering.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceOffering} m PriceOffering message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceOffering.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.header != null && Object.hasOwnProperty.call(m, "header"))
                                    w.uint32(10).string(m.header);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(18).string(m.description);
                                if (m.price != null && Object.hasOwnProperty.call(m, "price"))
                                    $root.google.ads.googleads.v11.common.Money.encode(m.price, w.uint32(26).fork()).ldelim();
                                if (m.unit != null && Object.hasOwnProperty.call(m, "unit"))
                                    w.uint32(32).int32(m.unit);
                                if (m.final_url != null && Object.hasOwnProperty.call(m, "final_url"))
                                    w.uint32(42).string(m.final_url);
                                if (m.final_mobile_url != null && Object.hasOwnProperty.call(m, "final_mobile_url"))
                                    w.uint32(50).string(m.final_mobile_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PriceOffering message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PriceOffering.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceOffering} message PriceOffering message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceOffering.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PriceOffering message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PriceOffering} PriceOffering
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceOffering.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PriceOffering();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.header = r.string();
                                        break;
                                    case 2:
                                        m.description = r.string();
                                        break;
                                    case 3:
                                        m.price = $root.google.ads.googleads.v11.common.Money.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.unit = r.int32();
                                        break;
                                    case 5:
                                        m.final_url = r.string();
                                        break;
                                    case 6:
                                        m.final_mobile_url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PriceOffering message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PriceOffering} PriceOffering
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceOffering.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PriceOffering message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PriceOffering.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    if (!$util.isString(m.header))
                                        return "header: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Money.verify(m.price);
                                        if (e)
                                            return "price." + e;
                                    }
                                }
                                if (m.unit != null && m.hasOwnProperty("unit")) {
                                    switch (m.unit) {
                                    default:
                                        return "unit: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.final_url != null && m.hasOwnProperty("final_url")) {
                                    if (!$util.isString(m.final_url))
                                        return "final_url: string expected";
                                }
                                if (m.final_mobile_url != null && m.hasOwnProperty("final_mobile_url")) {
                                    if (!$util.isString(m.final_mobile_url))
                                        return "final_mobile_url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PriceOffering message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PriceOffering} PriceOffering
                             */
                            PriceOffering.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PriceOffering)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PriceOffering();
                                if (d.header != null) {
                                    m.header = String(d.header);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.price != null) {
                                    if (typeof d.price !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PriceOffering.price: object expected");
                                    m.price = $root.google.ads.googleads.v11.common.Money.fromObject(d.price);
                                }
                                switch (d.unit) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.unit = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.unit = 1;
                                    break;
                                case "PER_HOUR":
                                case 2:
                                    m.unit = 2;
                                    break;
                                case "PER_DAY":
                                case 3:
                                    m.unit = 3;
                                    break;
                                case "PER_WEEK":
                                case 4:
                                    m.unit = 4;
                                    break;
                                case "PER_MONTH":
                                case 5:
                                    m.unit = 5;
                                    break;
                                case "PER_YEAR":
                                case 6:
                                    m.unit = 6;
                                    break;
                                case "PER_NIGHT":
                                case 7:
                                    m.unit = 7;
                                    break;
                                }
                                if (d.final_url != null) {
                                    m.final_url = String(d.final_url);
                                }
                                if (d.final_mobile_url != null) {
                                    m.final_mobile_url = String(d.final_mobile_url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PriceOffering message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @static
                             * @param {google.ads.googleads.v11.common.PriceOffering} m PriceOffering
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PriceOffering.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.header = "";
                                    d.description = "";
                                    d.price = null;
                                    d.unit = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.final_url = "";
                                    d.final_mobile_url = "";
                                }
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    d.header = m.header;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    d.price = $root.google.ads.googleads.v11.common.Money.toObject(m.price, o);
                                }
                                if (m.unit != null && m.hasOwnProperty("unit")) {
                                    d.unit = o.enums === String ? $root.google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit[m.unit] : m.unit;
                                }
                                if (m.final_url != null && m.hasOwnProperty("final_url")) {
                                    d.final_url = m.final_url;
                                }
                                if (m.final_mobile_url != null && m.hasOwnProperty("final_mobile_url")) {
                                    d.final_mobile_url = m.final_mobile_url;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PriceOffering to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PriceOffering
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PriceOffering.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PriceOffering;
                        })();
    
                        common.CallToActionAsset = (function() {
    
                            /**
                             * Properties of a CallToActionAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICallToActionAsset
                             * @property {google.ads.googleads.v11.enums.CallToActionTypeEnum.CallToActionType|null} [call_to_action] CallToActionAsset call_to_action
                             */
    
                            /**
                             * Constructs a new CallToActionAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CallToActionAsset.
                             * @implements ICallToActionAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICallToActionAsset=} [p] Properties to set
                             */
                            function CallToActionAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallToActionAsset call_to_action.
                             * @member {google.ads.googleads.v11.enums.CallToActionTypeEnum.CallToActionType} call_to_action
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @instance
                             */
                            CallToActionAsset.prototype.call_to_action = 0;
    
                            /**
                             * Creates a new CallToActionAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallToActionAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CallToActionAsset} CallToActionAsset instance
                             */
                            CallToActionAsset.create = function create(properties) {
                                return new CallToActionAsset(properties);
                            };
    
                            /**
                             * Encodes the specified CallToActionAsset message. Does not implicitly {@link google.ads.googleads.v11.common.CallToActionAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallToActionAsset} m CallToActionAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallToActionAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.call_to_action != null && Object.hasOwnProperty.call(m, "call_to_action"))
                                    w.uint32(8).int32(m.call_to_action);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallToActionAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CallToActionAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallToActionAsset} message CallToActionAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallToActionAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallToActionAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CallToActionAsset} CallToActionAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallToActionAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CallToActionAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.call_to_action = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallToActionAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CallToActionAsset} CallToActionAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallToActionAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallToActionAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallToActionAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.call_to_action != null && m.hasOwnProperty("call_to_action")) {
                                    switch (m.call_to_action) {
                                    default:
                                        return "call_to_action: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallToActionAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CallToActionAsset} CallToActionAsset
                             */
                            CallToActionAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CallToActionAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CallToActionAsset();
                                switch (d.call_to_action) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.call_to_action = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.call_to_action = 1;
                                    break;
                                case "LEARN_MORE":
                                case 2:
                                    m.call_to_action = 2;
                                    break;
                                case "GET_QUOTE":
                                case 3:
                                    m.call_to_action = 3;
                                    break;
                                case "APPLY_NOW":
                                case 4:
                                    m.call_to_action = 4;
                                    break;
                                case "SIGN_UP":
                                case 5:
                                    m.call_to_action = 5;
                                    break;
                                case "CONTACT_US":
                                case 6:
                                    m.call_to_action = 6;
                                    break;
                                case "SUBSCRIBE":
                                case 7:
                                    m.call_to_action = 7;
                                    break;
                                case "DOWNLOAD":
                                case 8:
                                    m.call_to_action = 8;
                                    break;
                                case "BOOK_NOW":
                                case 9:
                                    m.call_to_action = 9;
                                    break;
                                case "SHOP_NOW":
                                case 10:
                                    m.call_to_action = 10;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallToActionAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.CallToActionAsset} m CallToActionAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallToActionAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.call_to_action = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.call_to_action != null && m.hasOwnProperty("call_to_action")) {
                                    d.call_to_action = o.enums === String ? $root.google.ads.googleads.v11.enums.CallToActionTypeEnum.CallToActionType[m.call_to_action] : m.call_to_action;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallToActionAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CallToActionAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallToActionAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallToActionAsset;
                        })();
    
                        common.DynamicRealEstateAsset = (function() {
    
                            /**
                             * Properties of a DynamicRealEstateAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicRealEstateAsset
                             * @property {string|null} [listing_id] DynamicRealEstateAsset listing_id
                             * @property {string|null} [listing_name] DynamicRealEstateAsset listing_name
                             * @property {string|null} [city_name] DynamicRealEstateAsset city_name
                             * @property {string|null} [description] DynamicRealEstateAsset description
                             * @property {string|null} [address] DynamicRealEstateAsset address
                             * @property {string|null} [price] DynamicRealEstateAsset price
                             * @property {string|null} [image_url] DynamicRealEstateAsset image_url
                             * @property {string|null} [property_type] DynamicRealEstateAsset property_type
                             * @property {string|null} [listing_type] DynamicRealEstateAsset listing_type
                             * @property {Array.<string>|null} [contextual_keywords] DynamicRealEstateAsset contextual_keywords
                             * @property {string|null} [formatted_price] DynamicRealEstateAsset formatted_price
                             * @property {string|null} [android_app_link] DynamicRealEstateAsset android_app_link
                             * @property {string|null} [ios_app_link] DynamicRealEstateAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicRealEstateAsset ios_app_store_id
                             * @property {Array.<string>|null} [similar_listing_ids] DynamicRealEstateAsset similar_listing_ids
                             */
    
                            /**
                             * Constructs a new DynamicRealEstateAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicRealEstateAsset.
                             * @implements IDynamicRealEstateAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicRealEstateAsset=} [p] Properties to set
                             */
                            function DynamicRealEstateAsset(p) {
                                this.contextual_keywords = [];
                                this.similar_listing_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicRealEstateAsset listing_id.
                             * @member {string} listing_id
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.listing_id = "";
    
                            /**
                             * DynamicRealEstateAsset listing_name.
                             * @member {string} listing_name
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.listing_name = "";
    
                            /**
                             * DynamicRealEstateAsset city_name.
                             * @member {string} city_name
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.city_name = "";
    
                            /**
                             * DynamicRealEstateAsset description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.description = "";
    
                            /**
                             * DynamicRealEstateAsset address.
                             * @member {string} address
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.address = "";
    
                            /**
                             * DynamicRealEstateAsset price.
                             * @member {string} price
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.price = "";
    
                            /**
                             * DynamicRealEstateAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.image_url = "";
    
                            /**
                             * DynamicRealEstateAsset property_type.
                             * @member {string} property_type
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.property_type = "";
    
                            /**
                             * DynamicRealEstateAsset listing_type.
                             * @member {string} listing_type
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.listing_type = "";
    
                            /**
                             * DynamicRealEstateAsset contextual_keywords.
                             * @member {Array.<string>} contextual_keywords
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.contextual_keywords = $util.emptyArray;
    
                            /**
                             * DynamicRealEstateAsset formatted_price.
                             * @member {string} formatted_price
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.formatted_price = "";
    
                            /**
                             * DynamicRealEstateAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicRealEstateAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicRealEstateAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * DynamicRealEstateAsset similar_listing_ids.
                             * @member {Array.<string>} similar_listing_ids
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             */
                            DynamicRealEstateAsset.prototype.similar_listing_ids = $util.emptyArray;
    
                            /**
                             * Creates a new DynamicRealEstateAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicRealEstateAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicRealEstateAsset} DynamicRealEstateAsset instance
                             */
                            DynamicRealEstateAsset.create = function create(properties) {
                                return new DynamicRealEstateAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicRealEstateAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicRealEstateAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicRealEstateAsset} m DynamicRealEstateAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicRealEstateAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.listing_id != null && Object.hasOwnProperty.call(m, "listing_id"))
                                    w.uint32(10).string(m.listing_id);
                                if (m.listing_name != null && Object.hasOwnProperty.call(m, "listing_name"))
                                    w.uint32(18).string(m.listing_name);
                                if (m.city_name != null && Object.hasOwnProperty.call(m, "city_name"))
                                    w.uint32(26).string(m.city_name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(34).string(m.description);
                                if (m.address != null && Object.hasOwnProperty.call(m, "address"))
                                    w.uint32(42).string(m.address);
                                if (m.price != null && Object.hasOwnProperty.call(m, "price"))
                                    w.uint32(50).string(m.price);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(58).string(m.image_url);
                                if (m.property_type != null && Object.hasOwnProperty.call(m, "property_type"))
                                    w.uint32(66).string(m.property_type);
                                if (m.listing_type != null && Object.hasOwnProperty.call(m, "listing_type"))
                                    w.uint32(74).string(m.listing_type);
                                if (m.contextual_keywords != null && m.contextual_keywords.length) {
                                    for (var i = 0; i < m.contextual_keywords.length; ++i)
                                        w.uint32(82).string(m.contextual_keywords[i]);
                                }
                                if (m.formatted_price != null && Object.hasOwnProperty.call(m, "formatted_price"))
                                    w.uint32(90).string(m.formatted_price);
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(98).string(m.android_app_link);
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(106).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(112).int64(m.ios_app_store_id);
                                if (m.similar_listing_ids != null && m.similar_listing_ids.length) {
                                    for (var i = 0; i < m.similar_listing_ids.length; ++i)
                                        w.uint32(122).string(m.similar_listing_ids[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicRealEstateAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicRealEstateAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicRealEstateAsset} message DynamicRealEstateAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicRealEstateAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicRealEstateAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicRealEstateAsset} DynamicRealEstateAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicRealEstateAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicRealEstateAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.listing_id = r.string();
                                        break;
                                    case 2:
                                        m.listing_name = r.string();
                                        break;
                                    case 3:
                                        m.city_name = r.string();
                                        break;
                                    case 4:
                                        m.description = r.string();
                                        break;
                                    case 5:
                                        m.address = r.string();
                                        break;
                                    case 6:
                                        m.price = r.string();
                                        break;
                                    case 7:
                                        m.image_url = r.string();
                                        break;
                                    case 8:
                                        m.property_type = r.string();
                                        break;
                                    case 9:
                                        m.listing_type = r.string();
                                        break;
                                    case 10:
                                        if (!(m.contextual_keywords && m.contextual_keywords.length))
                                            m.contextual_keywords = [];
                                        m.contextual_keywords.push(r.string());
                                        break;
                                    case 11:
                                        m.formatted_price = r.string();
                                        break;
                                    case 12:
                                        m.android_app_link = r.string();
                                        break;
                                    case 13:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 14:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    case 15:
                                        if (!(m.similar_listing_ids && m.similar_listing_ids.length))
                                            m.similar_listing_ids = [];
                                        m.similar_listing_ids.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicRealEstateAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicRealEstateAsset} DynamicRealEstateAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicRealEstateAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicRealEstateAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicRealEstateAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.listing_id != null && m.hasOwnProperty("listing_id")) {
                                    if (!$util.isString(m.listing_id))
                                        return "listing_id: string expected";
                                }
                                if (m.listing_name != null && m.hasOwnProperty("listing_name")) {
                                    if (!$util.isString(m.listing_name))
                                        return "listing_name: string expected";
                                }
                                if (m.city_name != null && m.hasOwnProperty("city_name")) {
                                    if (!$util.isString(m.city_name))
                                        return "city_name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    if (!$util.isString(m.address))
                                        return "address: string expected";
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    if (!$util.isString(m.price))
                                        return "price: string expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.property_type != null && m.hasOwnProperty("property_type")) {
                                    if (!$util.isString(m.property_type))
                                        return "property_type: string expected";
                                }
                                if (m.listing_type != null && m.hasOwnProperty("listing_type")) {
                                    if (!$util.isString(m.listing_type))
                                        return "listing_type: string expected";
                                }
                                if (m.contextual_keywords != null && m.hasOwnProperty("contextual_keywords")) {
                                    if (!Array.isArray(m.contextual_keywords))
                                        return "contextual_keywords: array expected";
                                    for (var i = 0; i < m.contextual_keywords.length; ++i) {
                                        if (!$util.isString(m.contextual_keywords[i]))
                                            return "contextual_keywords: string[] expected";
                                    }
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    if (!$util.isString(m.formatted_price))
                                        return "formatted_price: string expected";
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                if (m.similar_listing_ids != null && m.hasOwnProperty("similar_listing_ids")) {
                                    if (!Array.isArray(m.similar_listing_ids))
                                        return "similar_listing_ids: array expected";
                                    for (var i = 0; i < m.similar_listing_ids.length; ++i) {
                                        if (!$util.isString(m.similar_listing_ids[i]))
                                            return "similar_listing_ids: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicRealEstateAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicRealEstateAsset} DynamicRealEstateAsset
                             */
                            DynamicRealEstateAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicRealEstateAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicRealEstateAsset();
                                if (d.listing_id != null) {
                                    m.listing_id = String(d.listing_id);
                                }
                                if (d.listing_name != null) {
                                    m.listing_name = String(d.listing_name);
                                }
                                if (d.city_name != null) {
                                    m.city_name = String(d.city_name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.address != null) {
                                    m.address = String(d.address);
                                }
                                if (d.price != null) {
                                    m.price = String(d.price);
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.property_type != null) {
                                    m.property_type = String(d.property_type);
                                }
                                if (d.listing_type != null) {
                                    m.listing_type = String(d.listing_type);
                                }
                                if (d.contextual_keywords) {
                                    if (!Array.isArray(d.contextual_keywords))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicRealEstateAsset.contextual_keywords: array expected");
                                    m.contextual_keywords = [];
                                    for (var i = 0; i < d.contextual_keywords.length; ++i) {
                                        m.contextual_keywords[i] = String(d.contextual_keywords[i]);
                                    }
                                }
                                if (d.formatted_price != null) {
                                    m.formatted_price = String(d.formatted_price);
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                if (d.similar_listing_ids) {
                                    if (!Array.isArray(d.similar_listing_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicRealEstateAsset.similar_listing_ids: array expected");
                                    m.similar_listing_ids = [];
                                    for (var i = 0; i < d.similar_listing_ids.length; ++i) {
                                        m.similar_listing_ids[i] = String(d.similar_listing_ids[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicRealEstateAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicRealEstateAsset} m DynamicRealEstateAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicRealEstateAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.contextual_keywords = [];
                                    d.similar_listing_ids = [];
                                }
                                if (o.defaults) {
                                    d.listing_id = "";
                                    d.listing_name = "";
                                    d.city_name = "";
                                    d.description = "";
                                    d.address = "";
                                    d.price = "";
                                    d.image_url = "";
                                    d.property_type = "";
                                    d.listing_type = "";
                                    d.formatted_price = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                }
                                if (m.listing_id != null && m.hasOwnProperty("listing_id")) {
                                    d.listing_id = m.listing_id;
                                }
                                if (m.listing_name != null && m.hasOwnProperty("listing_name")) {
                                    d.listing_name = m.listing_name;
                                }
                                if (m.city_name != null && m.hasOwnProperty("city_name")) {
                                    d.city_name = m.city_name;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    d.address = m.address;
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    d.price = m.price;
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                if (m.property_type != null && m.hasOwnProperty("property_type")) {
                                    d.property_type = m.property_type;
                                }
                                if (m.listing_type != null && m.hasOwnProperty("listing_type")) {
                                    d.listing_type = m.listing_type;
                                }
                                if (m.contextual_keywords && m.contextual_keywords.length) {
                                    d.contextual_keywords = [];
                                    for (var j = 0; j < m.contextual_keywords.length; ++j) {
                                        d.contextual_keywords[j] = m.contextual_keywords[j];
                                    }
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    d.formatted_price = m.formatted_price;
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                if (m.similar_listing_ids && m.similar_listing_ids.length) {
                                    d.similar_listing_ids = [];
                                    for (var j = 0; j < m.similar_listing_ids.length; ++j) {
                                        d.similar_listing_ids[j] = m.similar_listing_ids[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicRealEstateAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicRealEstateAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicRealEstateAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicRealEstateAsset;
                        })();
    
                        common.DynamicCustomAsset = (function() {
    
                            /**
                             * Properties of a DynamicCustomAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicCustomAsset
                             * @property {string|null} [id] DynamicCustomAsset id
                             * @property {string|null} [id2] DynamicCustomAsset id2
                             * @property {string|null} [item_title] DynamicCustomAsset item_title
                             * @property {string|null} [item_subtitle] DynamicCustomAsset item_subtitle
                             * @property {string|null} [item_description] DynamicCustomAsset item_description
                             * @property {string|null} [item_address] DynamicCustomAsset item_address
                             * @property {string|null} [item_category] DynamicCustomAsset item_category
                             * @property {string|null} [price] DynamicCustomAsset price
                             * @property {string|null} [sale_price] DynamicCustomAsset sale_price
                             * @property {string|null} [formatted_price] DynamicCustomAsset formatted_price
                             * @property {string|null} [formatted_sale_price] DynamicCustomAsset formatted_sale_price
                             * @property {string|null} [image_url] DynamicCustomAsset image_url
                             * @property {Array.<string>|null} [contextual_keywords] DynamicCustomAsset contextual_keywords
                             * @property {string|null} [android_app_link] DynamicCustomAsset android_app_link
                             * @property {string|null} [ios_app_link] DynamicCustomAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicCustomAsset ios_app_store_id
                             * @property {Array.<string>|null} [similar_ids] DynamicCustomAsset similar_ids
                             */
    
                            /**
                             * Constructs a new DynamicCustomAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicCustomAsset.
                             * @implements IDynamicCustomAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicCustomAsset=} [p] Properties to set
                             */
                            function DynamicCustomAsset(p) {
                                this.contextual_keywords = [];
                                this.similar_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicCustomAsset id.
                             * @member {string} id
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.id = "";
    
                            /**
                             * DynamicCustomAsset id2.
                             * @member {string} id2
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.id2 = "";
    
                            /**
                             * DynamicCustomAsset item_title.
                             * @member {string} item_title
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.item_title = "";
    
                            /**
                             * DynamicCustomAsset item_subtitle.
                             * @member {string} item_subtitle
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.item_subtitle = "";
    
                            /**
                             * DynamicCustomAsset item_description.
                             * @member {string} item_description
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.item_description = "";
    
                            /**
                             * DynamicCustomAsset item_address.
                             * @member {string} item_address
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.item_address = "";
    
                            /**
                             * DynamicCustomAsset item_category.
                             * @member {string} item_category
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.item_category = "";
    
                            /**
                             * DynamicCustomAsset price.
                             * @member {string} price
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.price = "";
    
                            /**
                             * DynamicCustomAsset sale_price.
                             * @member {string} sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.sale_price = "";
    
                            /**
                             * DynamicCustomAsset formatted_price.
                             * @member {string} formatted_price
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.formatted_price = "";
    
                            /**
                             * DynamicCustomAsset formatted_sale_price.
                             * @member {string} formatted_sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.formatted_sale_price = "";
    
                            /**
                             * DynamicCustomAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.image_url = "";
    
                            /**
                             * DynamicCustomAsset contextual_keywords.
                             * @member {Array.<string>} contextual_keywords
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.contextual_keywords = $util.emptyArray;
    
                            /**
                             * DynamicCustomAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicCustomAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicCustomAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * DynamicCustomAsset similar_ids.
                             * @member {Array.<string>} similar_ids
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             */
                            DynamicCustomAsset.prototype.similar_ids = $util.emptyArray;
    
                            /**
                             * Creates a new DynamicCustomAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicCustomAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicCustomAsset} DynamicCustomAsset instance
                             */
                            DynamicCustomAsset.create = function create(properties) {
                                return new DynamicCustomAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicCustomAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicCustomAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicCustomAsset} m DynamicCustomAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicCustomAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(10).string(m.id);
                                if (m.id2 != null && Object.hasOwnProperty.call(m, "id2"))
                                    w.uint32(18).string(m.id2);
                                if (m.item_title != null && Object.hasOwnProperty.call(m, "item_title"))
                                    w.uint32(26).string(m.item_title);
                                if (m.item_subtitle != null && Object.hasOwnProperty.call(m, "item_subtitle"))
                                    w.uint32(34).string(m.item_subtitle);
                                if (m.item_description != null && Object.hasOwnProperty.call(m, "item_description"))
                                    w.uint32(42).string(m.item_description);
                                if (m.item_address != null && Object.hasOwnProperty.call(m, "item_address"))
                                    w.uint32(50).string(m.item_address);
                                if (m.item_category != null && Object.hasOwnProperty.call(m, "item_category"))
                                    w.uint32(58).string(m.item_category);
                                if (m.price != null && Object.hasOwnProperty.call(m, "price"))
                                    w.uint32(66).string(m.price);
                                if (m.sale_price != null && Object.hasOwnProperty.call(m, "sale_price"))
                                    w.uint32(74).string(m.sale_price);
                                if (m.formatted_price != null && Object.hasOwnProperty.call(m, "formatted_price"))
                                    w.uint32(82).string(m.formatted_price);
                                if (m.formatted_sale_price != null && Object.hasOwnProperty.call(m, "formatted_sale_price"))
                                    w.uint32(90).string(m.formatted_sale_price);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(98).string(m.image_url);
                                if (m.contextual_keywords != null && m.contextual_keywords.length) {
                                    for (var i = 0; i < m.contextual_keywords.length; ++i)
                                        w.uint32(106).string(m.contextual_keywords[i]);
                                }
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(114).string(m.android_app_link);
                                if (m.similar_ids != null && m.similar_ids.length) {
                                    for (var i = 0; i < m.similar_ids.length; ++i)
                                        w.uint32(122).string(m.similar_ids[i]);
                                }
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(130).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(136).int64(m.ios_app_store_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicCustomAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicCustomAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicCustomAsset} message DynamicCustomAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicCustomAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicCustomAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicCustomAsset} DynamicCustomAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicCustomAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicCustomAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.id = r.string();
                                        break;
                                    case 2:
                                        m.id2 = r.string();
                                        break;
                                    case 3:
                                        m.item_title = r.string();
                                        break;
                                    case 4:
                                        m.item_subtitle = r.string();
                                        break;
                                    case 5:
                                        m.item_description = r.string();
                                        break;
                                    case 6:
                                        m.item_address = r.string();
                                        break;
                                    case 7:
                                        m.item_category = r.string();
                                        break;
                                    case 8:
                                        m.price = r.string();
                                        break;
                                    case 9:
                                        m.sale_price = r.string();
                                        break;
                                    case 10:
                                        m.formatted_price = r.string();
                                        break;
                                    case 11:
                                        m.formatted_sale_price = r.string();
                                        break;
                                    case 12:
                                        m.image_url = r.string();
                                        break;
                                    case 13:
                                        if (!(m.contextual_keywords && m.contextual_keywords.length))
                                            m.contextual_keywords = [];
                                        m.contextual_keywords.push(r.string());
                                        break;
                                    case 14:
                                        m.android_app_link = r.string();
                                        break;
                                    case 16:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 17:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    case 15:
                                        if (!(m.similar_ids && m.similar_ids.length))
                                            m.similar_ids = [];
                                        m.similar_ids.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicCustomAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicCustomAsset} DynamicCustomAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicCustomAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicCustomAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicCustomAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isString(m.id))
                                        return "id: string expected";
                                }
                                if (m.id2 != null && m.hasOwnProperty("id2")) {
                                    if (!$util.isString(m.id2))
                                        return "id2: string expected";
                                }
                                if (m.item_title != null && m.hasOwnProperty("item_title")) {
                                    if (!$util.isString(m.item_title))
                                        return "item_title: string expected";
                                }
                                if (m.item_subtitle != null && m.hasOwnProperty("item_subtitle")) {
                                    if (!$util.isString(m.item_subtitle))
                                        return "item_subtitle: string expected";
                                }
                                if (m.item_description != null && m.hasOwnProperty("item_description")) {
                                    if (!$util.isString(m.item_description))
                                        return "item_description: string expected";
                                }
                                if (m.item_address != null && m.hasOwnProperty("item_address")) {
                                    if (!$util.isString(m.item_address))
                                        return "item_address: string expected";
                                }
                                if (m.item_category != null && m.hasOwnProperty("item_category")) {
                                    if (!$util.isString(m.item_category))
                                        return "item_category: string expected";
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    if (!$util.isString(m.price))
                                        return "price: string expected";
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    if (!$util.isString(m.sale_price))
                                        return "sale_price: string expected";
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    if (!$util.isString(m.formatted_price))
                                        return "formatted_price: string expected";
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    if (!$util.isString(m.formatted_sale_price))
                                        return "formatted_sale_price: string expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.contextual_keywords != null && m.hasOwnProperty("contextual_keywords")) {
                                    if (!Array.isArray(m.contextual_keywords))
                                        return "contextual_keywords: array expected";
                                    for (var i = 0; i < m.contextual_keywords.length; ++i) {
                                        if (!$util.isString(m.contextual_keywords[i]))
                                            return "contextual_keywords: string[] expected";
                                    }
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                if (m.similar_ids != null && m.hasOwnProperty("similar_ids")) {
                                    if (!Array.isArray(m.similar_ids))
                                        return "similar_ids: array expected";
                                    for (var i = 0; i < m.similar_ids.length; ++i) {
                                        if (!$util.isString(m.similar_ids[i]))
                                            return "similar_ids: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicCustomAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicCustomAsset} DynamicCustomAsset
                             */
                            DynamicCustomAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicCustomAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicCustomAsset();
                                if (d.id != null) {
                                    m.id = String(d.id);
                                }
                                if (d.id2 != null) {
                                    m.id2 = String(d.id2);
                                }
                                if (d.item_title != null) {
                                    m.item_title = String(d.item_title);
                                }
                                if (d.item_subtitle != null) {
                                    m.item_subtitle = String(d.item_subtitle);
                                }
                                if (d.item_description != null) {
                                    m.item_description = String(d.item_description);
                                }
                                if (d.item_address != null) {
                                    m.item_address = String(d.item_address);
                                }
                                if (d.item_category != null) {
                                    m.item_category = String(d.item_category);
                                }
                                if (d.price != null) {
                                    m.price = String(d.price);
                                }
                                if (d.sale_price != null) {
                                    m.sale_price = String(d.sale_price);
                                }
                                if (d.formatted_price != null) {
                                    m.formatted_price = String(d.formatted_price);
                                }
                                if (d.formatted_sale_price != null) {
                                    m.formatted_sale_price = String(d.formatted_sale_price);
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.contextual_keywords) {
                                    if (!Array.isArray(d.contextual_keywords))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicCustomAsset.contextual_keywords: array expected");
                                    m.contextual_keywords = [];
                                    for (var i = 0; i < d.contextual_keywords.length; ++i) {
                                        m.contextual_keywords[i] = String(d.contextual_keywords[i]);
                                    }
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                if (d.similar_ids) {
                                    if (!Array.isArray(d.similar_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicCustomAsset.similar_ids: array expected");
                                    m.similar_ids = [];
                                    for (var i = 0; i < d.similar_ids.length; ++i) {
                                        m.similar_ids[i] = String(d.similar_ids[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicCustomAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicCustomAsset} m DynamicCustomAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicCustomAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.contextual_keywords = [];
                                    d.similar_ids = [];
                                }
                                if (o.defaults) {
                                    d.id = "";
                                    d.id2 = "";
                                    d.item_title = "";
                                    d.item_subtitle = "";
                                    d.item_description = "";
                                    d.item_address = "";
                                    d.item_category = "";
                                    d.price = "";
                                    d.sale_price = "";
                                    d.formatted_price = "";
                                    d.formatted_sale_price = "";
                                    d.image_url = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    d.id = m.id;
                                }
                                if (m.id2 != null && m.hasOwnProperty("id2")) {
                                    d.id2 = m.id2;
                                }
                                if (m.item_title != null && m.hasOwnProperty("item_title")) {
                                    d.item_title = m.item_title;
                                }
                                if (m.item_subtitle != null && m.hasOwnProperty("item_subtitle")) {
                                    d.item_subtitle = m.item_subtitle;
                                }
                                if (m.item_description != null && m.hasOwnProperty("item_description")) {
                                    d.item_description = m.item_description;
                                }
                                if (m.item_address != null && m.hasOwnProperty("item_address")) {
                                    d.item_address = m.item_address;
                                }
                                if (m.item_category != null && m.hasOwnProperty("item_category")) {
                                    d.item_category = m.item_category;
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    d.price = m.price;
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    d.sale_price = m.sale_price;
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    d.formatted_price = m.formatted_price;
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    d.formatted_sale_price = m.formatted_sale_price;
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                if (m.contextual_keywords && m.contextual_keywords.length) {
                                    d.contextual_keywords = [];
                                    for (var j = 0; j < m.contextual_keywords.length; ++j) {
                                        d.contextual_keywords[j] = m.contextual_keywords[j];
                                    }
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.similar_ids && m.similar_ids.length) {
                                    d.similar_ids = [];
                                    for (var j = 0; j < m.similar_ids.length; ++j) {
                                        d.similar_ids[j] = m.similar_ids[j];
                                    }
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicCustomAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicCustomAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicCustomAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicCustomAsset;
                        })();
    
                        common.DynamicHotelsAndRentalsAsset = (function() {
    
                            /**
                             * Properties of a DynamicHotelsAndRentalsAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicHotelsAndRentalsAsset
                             * @property {string|null} [property_id] DynamicHotelsAndRentalsAsset property_id
                             * @property {string|null} [property_name] DynamicHotelsAndRentalsAsset property_name
                             * @property {string|null} [image_url] DynamicHotelsAndRentalsAsset image_url
                             * @property {string|null} [destination_name] DynamicHotelsAndRentalsAsset destination_name
                             * @property {string|null} [description] DynamicHotelsAndRentalsAsset description
                             * @property {string|null} [price] DynamicHotelsAndRentalsAsset price
                             * @property {string|null} [sale_price] DynamicHotelsAndRentalsAsset sale_price
                             * @property {number|null} [star_rating] DynamicHotelsAndRentalsAsset star_rating
                             * @property {string|null} [category] DynamicHotelsAndRentalsAsset category
                             * @property {Array.<string>|null} [contextual_keywords] DynamicHotelsAndRentalsAsset contextual_keywords
                             * @property {string|null} [address] DynamicHotelsAndRentalsAsset address
                             * @property {string|null} [android_app_link] DynamicHotelsAndRentalsAsset android_app_link
                             * @property {string|null} [ios_app_link] DynamicHotelsAndRentalsAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicHotelsAndRentalsAsset ios_app_store_id
                             * @property {string|null} [formatted_price] DynamicHotelsAndRentalsAsset formatted_price
                             * @property {string|null} [formatted_sale_price] DynamicHotelsAndRentalsAsset formatted_sale_price
                             * @property {Array.<string>|null} [similar_property_ids] DynamicHotelsAndRentalsAsset similar_property_ids
                             */
    
                            /**
                             * Constructs a new DynamicHotelsAndRentalsAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicHotelsAndRentalsAsset.
                             * @implements IDynamicHotelsAndRentalsAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicHotelsAndRentalsAsset=} [p] Properties to set
                             */
                            function DynamicHotelsAndRentalsAsset(p) {
                                this.contextual_keywords = [];
                                this.similar_property_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicHotelsAndRentalsAsset property_id.
                             * @member {string} property_id
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.property_id = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset property_name.
                             * @member {string} property_name
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.property_name = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.image_url = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset destination_name.
                             * @member {string} destination_name
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.destination_name = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.description = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset price.
                             * @member {string} price
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.price = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset sale_price.
                             * @member {string} sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.sale_price = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset star_rating.
                             * @member {number} star_rating
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.star_rating = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * DynamicHotelsAndRentalsAsset category.
                             * @member {string} category
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.category = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset contextual_keywords.
                             * @member {Array.<string>} contextual_keywords
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.contextual_keywords = $util.emptyArray;
    
                            /**
                             * DynamicHotelsAndRentalsAsset address.
                             * @member {string} address
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.address = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * DynamicHotelsAndRentalsAsset formatted_price.
                             * @member {string} formatted_price
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.formatted_price = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset formatted_sale_price.
                             * @member {string} formatted_sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.formatted_sale_price = "";
    
                            /**
                             * DynamicHotelsAndRentalsAsset similar_property_ids.
                             * @member {Array.<string>} similar_property_ids
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             */
                            DynamicHotelsAndRentalsAsset.prototype.similar_property_ids = $util.emptyArray;
    
                            /**
                             * Creates a new DynamicHotelsAndRentalsAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicHotelsAndRentalsAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset} DynamicHotelsAndRentalsAsset instance
                             */
                            DynamicHotelsAndRentalsAsset.create = function create(properties) {
                                return new DynamicHotelsAndRentalsAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicHotelsAndRentalsAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicHotelsAndRentalsAsset} m DynamicHotelsAndRentalsAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicHotelsAndRentalsAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.property_id != null && Object.hasOwnProperty.call(m, "property_id"))
                                    w.uint32(10).string(m.property_id);
                                if (m.property_name != null && Object.hasOwnProperty.call(m, "property_name"))
                                    w.uint32(18).string(m.property_name);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(26).string(m.image_url);
                                if (m.destination_name != null && Object.hasOwnProperty.call(m, "destination_name"))
                                    w.uint32(34).string(m.destination_name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(42).string(m.description);
                                if (m.price != null && Object.hasOwnProperty.call(m, "price"))
                                    w.uint32(50).string(m.price);
                                if (m.sale_price != null && Object.hasOwnProperty.call(m, "sale_price"))
                                    w.uint32(58).string(m.sale_price);
                                if (m.star_rating != null && Object.hasOwnProperty.call(m, "star_rating"))
                                    w.uint32(64).int64(m.star_rating);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(74).string(m.category);
                                if (m.contextual_keywords != null && m.contextual_keywords.length) {
                                    for (var i = 0; i < m.contextual_keywords.length; ++i)
                                        w.uint32(82).string(m.contextual_keywords[i]);
                                }
                                if (m.address != null && Object.hasOwnProperty.call(m, "address"))
                                    w.uint32(90).string(m.address);
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(98).string(m.android_app_link);
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(106).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(112).int64(m.ios_app_store_id);
                                if (m.formatted_price != null && Object.hasOwnProperty.call(m, "formatted_price"))
                                    w.uint32(122).string(m.formatted_price);
                                if (m.formatted_sale_price != null && Object.hasOwnProperty.call(m, "formatted_sale_price"))
                                    w.uint32(130).string(m.formatted_sale_price);
                                if (m.similar_property_ids != null && m.similar_property_ids.length) {
                                    for (var i = 0; i < m.similar_property_ids.length; ++i)
                                        w.uint32(138).string(m.similar_property_ids[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicHotelsAndRentalsAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicHotelsAndRentalsAsset} message DynamicHotelsAndRentalsAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicHotelsAndRentalsAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicHotelsAndRentalsAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset} DynamicHotelsAndRentalsAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicHotelsAndRentalsAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.property_id = r.string();
                                        break;
                                    case 2:
                                        m.property_name = r.string();
                                        break;
                                    case 3:
                                        m.image_url = r.string();
                                        break;
                                    case 4:
                                        m.destination_name = r.string();
                                        break;
                                    case 5:
                                        m.description = r.string();
                                        break;
                                    case 6:
                                        m.price = r.string();
                                        break;
                                    case 7:
                                        m.sale_price = r.string();
                                        break;
                                    case 8:
                                        m.star_rating = r.int64();
                                        break;
                                    case 9:
                                        m.category = r.string();
                                        break;
                                    case 10:
                                        if (!(m.contextual_keywords && m.contextual_keywords.length))
                                            m.contextual_keywords = [];
                                        m.contextual_keywords.push(r.string());
                                        break;
                                    case 11:
                                        m.address = r.string();
                                        break;
                                    case 12:
                                        m.android_app_link = r.string();
                                        break;
                                    case 13:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 14:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    case 15:
                                        m.formatted_price = r.string();
                                        break;
                                    case 16:
                                        m.formatted_sale_price = r.string();
                                        break;
                                    case 17:
                                        if (!(m.similar_property_ids && m.similar_property_ids.length))
                                            m.similar_property_ids = [];
                                        m.similar_property_ids.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicHotelsAndRentalsAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset} DynamicHotelsAndRentalsAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicHotelsAndRentalsAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicHotelsAndRentalsAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicHotelsAndRentalsAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.property_id != null && m.hasOwnProperty("property_id")) {
                                    if (!$util.isString(m.property_id))
                                        return "property_id: string expected";
                                }
                                if (m.property_name != null && m.hasOwnProperty("property_name")) {
                                    if (!$util.isString(m.property_name))
                                        return "property_name: string expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.destination_name != null && m.hasOwnProperty("destination_name")) {
                                    if (!$util.isString(m.destination_name))
                                        return "destination_name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    if (!$util.isString(m.price))
                                        return "price: string expected";
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    if (!$util.isString(m.sale_price))
                                        return "sale_price: string expected";
                                }
                                if (m.star_rating != null && m.hasOwnProperty("star_rating")) {
                                    if (!$util.isInteger(m.star_rating) && !(m.star_rating && $util.isInteger(m.star_rating.low) && $util.isInteger(m.star_rating.high)))
                                        return "star_rating: integer|Long expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    if (!$util.isString(m.category))
                                        return "category: string expected";
                                }
                                if (m.contextual_keywords != null && m.hasOwnProperty("contextual_keywords")) {
                                    if (!Array.isArray(m.contextual_keywords))
                                        return "contextual_keywords: array expected";
                                    for (var i = 0; i < m.contextual_keywords.length; ++i) {
                                        if (!$util.isString(m.contextual_keywords[i]))
                                            return "contextual_keywords: string[] expected";
                                    }
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    if (!$util.isString(m.address))
                                        return "address: string expected";
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    if (!$util.isString(m.formatted_price))
                                        return "formatted_price: string expected";
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    if (!$util.isString(m.formatted_sale_price))
                                        return "formatted_sale_price: string expected";
                                }
                                if (m.similar_property_ids != null && m.hasOwnProperty("similar_property_ids")) {
                                    if (!Array.isArray(m.similar_property_ids))
                                        return "similar_property_ids: array expected";
                                    for (var i = 0; i < m.similar_property_ids.length; ++i) {
                                        if (!$util.isString(m.similar_property_ids[i]))
                                            return "similar_property_ids: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicHotelsAndRentalsAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset} DynamicHotelsAndRentalsAsset
                             */
                            DynamicHotelsAndRentalsAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset();
                                if (d.property_id != null) {
                                    m.property_id = String(d.property_id);
                                }
                                if (d.property_name != null) {
                                    m.property_name = String(d.property_name);
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.destination_name != null) {
                                    m.destination_name = String(d.destination_name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.price != null) {
                                    m.price = String(d.price);
                                }
                                if (d.sale_price != null) {
                                    m.sale_price = String(d.sale_price);
                                }
                                if (d.star_rating != null) {
                                    if ($util.Long)
                                        (m.star_rating = $util.Long.fromValue(d.star_rating)).unsigned = false;
                                    else if (typeof d.star_rating === "string")
                                        m.star_rating = parseInt(d.star_rating, 10);
                                    else if (typeof d.star_rating === "number")
                                        m.star_rating = d.star_rating;
                                    else if (typeof d.star_rating === "object")
                                        m.star_rating = new $util.LongBits(d.star_rating.low >>> 0, d.star_rating.high >>> 0).toNumber();
                                }
                                if (d.category != null) {
                                    m.category = String(d.category);
                                }
                                if (d.contextual_keywords) {
                                    if (!Array.isArray(d.contextual_keywords))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.contextual_keywords: array expected");
                                    m.contextual_keywords = [];
                                    for (var i = 0; i < d.contextual_keywords.length; ++i) {
                                        m.contextual_keywords[i] = String(d.contextual_keywords[i]);
                                    }
                                }
                                if (d.address != null) {
                                    m.address = String(d.address);
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                if (d.formatted_price != null) {
                                    m.formatted_price = String(d.formatted_price);
                                }
                                if (d.formatted_sale_price != null) {
                                    m.formatted_sale_price = String(d.formatted_sale_price);
                                }
                                if (d.similar_property_ids) {
                                    if (!Array.isArray(d.similar_property_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.similar_property_ids: array expected");
                                    m.similar_property_ids = [];
                                    for (var i = 0; i < d.similar_property_ids.length; ++i) {
                                        m.similar_property_ids[i] = String(d.similar_property_ids[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicHotelsAndRentalsAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset} m DynamicHotelsAndRentalsAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicHotelsAndRentalsAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.contextual_keywords = [];
                                    d.similar_property_ids = [];
                                }
                                if (o.defaults) {
                                    d.property_id = "";
                                    d.property_name = "";
                                    d.image_url = "";
                                    d.destination_name = "";
                                    d.description = "";
                                    d.price = "";
                                    d.sale_price = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.star_rating = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.star_rating = o.longs === String ? "0" : 0;
                                    d.category = "";
                                    d.address = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                    d.formatted_price = "";
                                    d.formatted_sale_price = "";
                                }
                                if (m.property_id != null && m.hasOwnProperty("property_id")) {
                                    d.property_id = m.property_id;
                                }
                                if (m.property_name != null && m.hasOwnProperty("property_name")) {
                                    d.property_name = m.property_name;
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                if (m.destination_name != null && m.hasOwnProperty("destination_name")) {
                                    d.destination_name = m.destination_name;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    d.price = m.price;
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    d.sale_price = m.sale_price;
                                }
                                if (m.star_rating != null && m.hasOwnProperty("star_rating")) {
                                    if (typeof m.star_rating === "number")
                                        d.star_rating = o.longs === String ? String(m.star_rating) : m.star_rating;
                                    else
                                        d.star_rating = o.longs === String ? $util.Long.prototype.toString.call(m.star_rating) : o.longs === Number ? new $util.LongBits(m.star_rating.low >>> 0, m.star_rating.high >>> 0).toNumber() : m.star_rating;
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = m.category;
                                }
                                if (m.contextual_keywords && m.contextual_keywords.length) {
                                    d.contextual_keywords = [];
                                    for (var j = 0; j < m.contextual_keywords.length; ++j) {
                                        d.contextual_keywords[j] = m.contextual_keywords[j];
                                    }
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    d.address = m.address;
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    d.formatted_price = m.formatted_price;
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    d.formatted_sale_price = m.formatted_sale_price;
                                }
                                if (m.similar_property_ids && m.similar_property_ids.length) {
                                    d.similar_property_ids = [];
                                    for (var j = 0; j < m.similar_property_ids.length; ++j) {
                                        d.similar_property_ids[j] = m.similar_property_ids[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicHotelsAndRentalsAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicHotelsAndRentalsAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicHotelsAndRentalsAsset;
                        })();
    
                        common.DynamicFlightsAsset = (function() {
    
                            /**
                             * Properties of a DynamicFlightsAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicFlightsAsset
                             * @property {string|null} [destination_id] DynamicFlightsAsset destination_id
                             * @property {string|null} [origin_id] DynamicFlightsAsset origin_id
                             * @property {string|null} [flight_description] DynamicFlightsAsset flight_description
                             * @property {string|null} [image_url] DynamicFlightsAsset image_url
                             * @property {string|null} [destination_name] DynamicFlightsAsset destination_name
                             * @property {string|null} [origin_name] DynamicFlightsAsset origin_name
                             * @property {string|null} [flight_price] DynamicFlightsAsset flight_price
                             * @property {string|null} [flight_sale_price] DynamicFlightsAsset flight_sale_price
                             * @property {string|null} [formatted_price] DynamicFlightsAsset formatted_price
                             * @property {string|null} [formatted_sale_price] DynamicFlightsAsset formatted_sale_price
                             * @property {string|null} [android_app_link] DynamicFlightsAsset android_app_link
                             * @property {string|null} [ios_app_link] DynamicFlightsAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicFlightsAsset ios_app_store_id
                             * @property {Array.<string>|null} [similar_destination_ids] DynamicFlightsAsset similar_destination_ids
                             * @property {string|null} [custom_mapping] DynamicFlightsAsset custom_mapping
                             */
    
                            /**
                             * Constructs a new DynamicFlightsAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicFlightsAsset.
                             * @implements IDynamicFlightsAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicFlightsAsset=} [p] Properties to set
                             */
                            function DynamicFlightsAsset(p) {
                                this.similar_destination_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicFlightsAsset destination_id.
                             * @member {string} destination_id
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.destination_id = "";
    
                            /**
                             * DynamicFlightsAsset origin_id.
                             * @member {string} origin_id
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.origin_id = "";
    
                            /**
                             * DynamicFlightsAsset flight_description.
                             * @member {string} flight_description
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.flight_description = "";
    
                            /**
                             * DynamicFlightsAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.image_url = "";
    
                            /**
                             * DynamicFlightsAsset destination_name.
                             * @member {string} destination_name
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.destination_name = "";
    
                            /**
                             * DynamicFlightsAsset origin_name.
                             * @member {string} origin_name
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.origin_name = "";
    
                            /**
                             * DynamicFlightsAsset flight_price.
                             * @member {string} flight_price
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.flight_price = "";
    
                            /**
                             * DynamicFlightsAsset flight_sale_price.
                             * @member {string} flight_sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.flight_sale_price = "";
    
                            /**
                             * DynamicFlightsAsset formatted_price.
                             * @member {string} formatted_price
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.formatted_price = "";
    
                            /**
                             * DynamicFlightsAsset formatted_sale_price.
                             * @member {string} formatted_sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.formatted_sale_price = "";
    
                            /**
                             * DynamicFlightsAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicFlightsAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicFlightsAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * DynamicFlightsAsset similar_destination_ids.
                             * @member {Array.<string>} similar_destination_ids
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.similar_destination_ids = $util.emptyArray;
    
                            /**
                             * DynamicFlightsAsset custom_mapping.
                             * @member {string} custom_mapping
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             */
                            DynamicFlightsAsset.prototype.custom_mapping = "";
    
                            /**
                             * Creates a new DynamicFlightsAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicFlightsAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicFlightsAsset} DynamicFlightsAsset instance
                             */
                            DynamicFlightsAsset.create = function create(properties) {
                                return new DynamicFlightsAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicFlightsAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicFlightsAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicFlightsAsset} m DynamicFlightsAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicFlightsAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.destination_id != null && Object.hasOwnProperty.call(m, "destination_id"))
                                    w.uint32(10).string(m.destination_id);
                                if (m.origin_id != null && Object.hasOwnProperty.call(m, "origin_id"))
                                    w.uint32(18).string(m.origin_id);
                                if (m.flight_description != null && Object.hasOwnProperty.call(m, "flight_description"))
                                    w.uint32(26).string(m.flight_description);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(34).string(m.image_url);
                                if (m.destination_name != null && Object.hasOwnProperty.call(m, "destination_name"))
                                    w.uint32(42).string(m.destination_name);
                                if (m.origin_name != null && Object.hasOwnProperty.call(m, "origin_name"))
                                    w.uint32(50).string(m.origin_name);
                                if (m.flight_price != null && Object.hasOwnProperty.call(m, "flight_price"))
                                    w.uint32(58).string(m.flight_price);
                                if (m.flight_sale_price != null && Object.hasOwnProperty.call(m, "flight_sale_price"))
                                    w.uint32(66).string(m.flight_sale_price);
                                if (m.formatted_price != null && Object.hasOwnProperty.call(m, "formatted_price"))
                                    w.uint32(74).string(m.formatted_price);
                                if (m.formatted_sale_price != null && Object.hasOwnProperty.call(m, "formatted_sale_price"))
                                    w.uint32(82).string(m.formatted_sale_price);
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(90).string(m.android_app_link);
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(98).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(104).int64(m.ios_app_store_id);
                                if (m.similar_destination_ids != null && m.similar_destination_ids.length) {
                                    for (var i = 0; i < m.similar_destination_ids.length; ++i)
                                        w.uint32(114).string(m.similar_destination_ids[i]);
                                }
                                if (m.custom_mapping != null && Object.hasOwnProperty.call(m, "custom_mapping"))
                                    w.uint32(122).string(m.custom_mapping);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicFlightsAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicFlightsAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicFlightsAsset} message DynamicFlightsAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicFlightsAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicFlightsAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicFlightsAsset} DynamicFlightsAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicFlightsAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicFlightsAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.destination_id = r.string();
                                        break;
                                    case 2:
                                        m.origin_id = r.string();
                                        break;
                                    case 3:
                                        m.flight_description = r.string();
                                        break;
                                    case 4:
                                        m.image_url = r.string();
                                        break;
                                    case 5:
                                        m.destination_name = r.string();
                                        break;
                                    case 6:
                                        m.origin_name = r.string();
                                        break;
                                    case 7:
                                        m.flight_price = r.string();
                                        break;
                                    case 8:
                                        m.flight_sale_price = r.string();
                                        break;
                                    case 9:
                                        m.formatted_price = r.string();
                                        break;
                                    case 10:
                                        m.formatted_sale_price = r.string();
                                        break;
                                    case 11:
                                        m.android_app_link = r.string();
                                        break;
                                    case 12:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 13:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    case 14:
                                        if (!(m.similar_destination_ids && m.similar_destination_ids.length))
                                            m.similar_destination_ids = [];
                                        m.similar_destination_ids.push(r.string());
                                        break;
                                    case 15:
                                        m.custom_mapping = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicFlightsAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicFlightsAsset} DynamicFlightsAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicFlightsAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicFlightsAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicFlightsAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.destination_id != null && m.hasOwnProperty("destination_id")) {
                                    if (!$util.isString(m.destination_id))
                                        return "destination_id: string expected";
                                }
                                if (m.origin_id != null && m.hasOwnProperty("origin_id")) {
                                    if (!$util.isString(m.origin_id))
                                        return "origin_id: string expected";
                                }
                                if (m.flight_description != null && m.hasOwnProperty("flight_description")) {
                                    if (!$util.isString(m.flight_description))
                                        return "flight_description: string expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.destination_name != null && m.hasOwnProperty("destination_name")) {
                                    if (!$util.isString(m.destination_name))
                                        return "destination_name: string expected";
                                }
                                if (m.origin_name != null && m.hasOwnProperty("origin_name")) {
                                    if (!$util.isString(m.origin_name))
                                        return "origin_name: string expected";
                                }
                                if (m.flight_price != null && m.hasOwnProperty("flight_price")) {
                                    if (!$util.isString(m.flight_price))
                                        return "flight_price: string expected";
                                }
                                if (m.flight_sale_price != null && m.hasOwnProperty("flight_sale_price")) {
                                    if (!$util.isString(m.flight_sale_price))
                                        return "flight_sale_price: string expected";
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    if (!$util.isString(m.formatted_price))
                                        return "formatted_price: string expected";
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    if (!$util.isString(m.formatted_sale_price))
                                        return "formatted_sale_price: string expected";
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                if (m.similar_destination_ids != null && m.hasOwnProperty("similar_destination_ids")) {
                                    if (!Array.isArray(m.similar_destination_ids))
                                        return "similar_destination_ids: array expected";
                                    for (var i = 0; i < m.similar_destination_ids.length; ++i) {
                                        if (!$util.isString(m.similar_destination_ids[i]))
                                            return "similar_destination_ids: string[] expected";
                                    }
                                }
                                if (m.custom_mapping != null && m.hasOwnProperty("custom_mapping")) {
                                    if (!$util.isString(m.custom_mapping))
                                        return "custom_mapping: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicFlightsAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicFlightsAsset} DynamicFlightsAsset
                             */
                            DynamicFlightsAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicFlightsAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicFlightsAsset();
                                if (d.destination_id != null) {
                                    m.destination_id = String(d.destination_id);
                                }
                                if (d.origin_id != null) {
                                    m.origin_id = String(d.origin_id);
                                }
                                if (d.flight_description != null) {
                                    m.flight_description = String(d.flight_description);
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.destination_name != null) {
                                    m.destination_name = String(d.destination_name);
                                }
                                if (d.origin_name != null) {
                                    m.origin_name = String(d.origin_name);
                                }
                                if (d.flight_price != null) {
                                    m.flight_price = String(d.flight_price);
                                }
                                if (d.flight_sale_price != null) {
                                    m.flight_sale_price = String(d.flight_sale_price);
                                }
                                if (d.formatted_price != null) {
                                    m.formatted_price = String(d.formatted_price);
                                }
                                if (d.formatted_sale_price != null) {
                                    m.formatted_sale_price = String(d.formatted_sale_price);
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                if (d.similar_destination_ids) {
                                    if (!Array.isArray(d.similar_destination_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicFlightsAsset.similar_destination_ids: array expected");
                                    m.similar_destination_ids = [];
                                    for (var i = 0; i < d.similar_destination_ids.length; ++i) {
                                        m.similar_destination_ids[i] = String(d.similar_destination_ids[i]);
                                    }
                                }
                                if (d.custom_mapping != null) {
                                    m.custom_mapping = String(d.custom_mapping);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicFlightsAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicFlightsAsset} m DynamicFlightsAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicFlightsAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.similar_destination_ids = [];
                                }
                                if (o.defaults) {
                                    d.destination_id = "";
                                    d.origin_id = "";
                                    d.flight_description = "";
                                    d.image_url = "";
                                    d.destination_name = "";
                                    d.origin_name = "";
                                    d.flight_price = "";
                                    d.flight_sale_price = "";
                                    d.formatted_price = "";
                                    d.formatted_sale_price = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                    d.custom_mapping = "";
                                }
                                if (m.destination_id != null && m.hasOwnProperty("destination_id")) {
                                    d.destination_id = m.destination_id;
                                }
                                if (m.origin_id != null && m.hasOwnProperty("origin_id")) {
                                    d.origin_id = m.origin_id;
                                }
                                if (m.flight_description != null && m.hasOwnProperty("flight_description")) {
                                    d.flight_description = m.flight_description;
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                if (m.destination_name != null && m.hasOwnProperty("destination_name")) {
                                    d.destination_name = m.destination_name;
                                }
                                if (m.origin_name != null && m.hasOwnProperty("origin_name")) {
                                    d.origin_name = m.origin_name;
                                }
                                if (m.flight_price != null && m.hasOwnProperty("flight_price")) {
                                    d.flight_price = m.flight_price;
                                }
                                if (m.flight_sale_price != null && m.hasOwnProperty("flight_sale_price")) {
                                    d.flight_sale_price = m.flight_sale_price;
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    d.formatted_price = m.formatted_price;
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    d.formatted_sale_price = m.formatted_sale_price;
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                if (m.similar_destination_ids && m.similar_destination_ids.length) {
                                    d.similar_destination_ids = [];
                                    for (var j = 0; j < m.similar_destination_ids.length; ++j) {
                                        d.similar_destination_ids[j] = m.similar_destination_ids[j];
                                    }
                                }
                                if (m.custom_mapping != null && m.hasOwnProperty("custom_mapping")) {
                                    d.custom_mapping = m.custom_mapping;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicFlightsAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicFlightsAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicFlightsAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicFlightsAsset;
                        })();
    
                        common.DiscoveryCarouselCardAsset = (function() {
    
                            /**
                             * Properties of a DiscoveryCarouselCardAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDiscoveryCarouselCardAsset
                             * @property {string|null} [marketing_image_asset] DiscoveryCarouselCardAsset marketing_image_asset
                             * @property {string|null} [square_marketing_image_asset] DiscoveryCarouselCardAsset square_marketing_image_asset
                             * @property {string|null} [portrait_marketing_image_asset] DiscoveryCarouselCardAsset portrait_marketing_image_asset
                             * @property {string|null} [headline] DiscoveryCarouselCardAsset headline
                             * @property {string|null} [call_to_action_text] DiscoveryCarouselCardAsset call_to_action_text
                             */
    
                            /**
                             * Constructs a new DiscoveryCarouselCardAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DiscoveryCarouselCardAsset.
                             * @implements IDiscoveryCarouselCardAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselCardAsset=} [p] Properties to set
                             */
                            function DiscoveryCarouselCardAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DiscoveryCarouselCardAsset marketing_image_asset.
                             * @member {string} marketing_image_asset
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @instance
                             */
                            DiscoveryCarouselCardAsset.prototype.marketing_image_asset = "";
    
                            /**
                             * DiscoveryCarouselCardAsset square_marketing_image_asset.
                             * @member {string} square_marketing_image_asset
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @instance
                             */
                            DiscoveryCarouselCardAsset.prototype.square_marketing_image_asset = "";
    
                            /**
                             * DiscoveryCarouselCardAsset portrait_marketing_image_asset.
                             * @member {string} portrait_marketing_image_asset
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @instance
                             */
                            DiscoveryCarouselCardAsset.prototype.portrait_marketing_image_asset = "";
    
                            /**
                             * DiscoveryCarouselCardAsset headline.
                             * @member {string} headline
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @instance
                             */
                            DiscoveryCarouselCardAsset.prototype.headline = "";
    
                            /**
                             * DiscoveryCarouselCardAsset call_to_action_text.
                             * @member {string} call_to_action_text
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @instance
                             */
                            DiscoveryCarouselCardAsset.prototype.call_to_action_text = "";
    
                            /**
                             * Creates a new DiscoveryCarouselCardAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselCardAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselCardAsset} DiscoveryCarouselCardAsset instance
                             */
                            DiscoveryCarouselCardAsset.create = function create(properties) {
                                return new DiscoveryCarouselCardAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DiscoveryCarouselCardAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DiscoveryCarouselCardAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselCardAsset} m DiscoveryCarouselCardAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DiscoveryCarouselCardAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.marketing_image_asset != null && Object.hasOwnProperty.call(m, "marketing_image_asset"))
                                    w.uint32(10).string(m.marketing_image_asset);
                                if (m.square_marketing_image_asset != null && Object.hasOwnProperty.call(m, "square_marketing_image_asset"))
                                    w.uint32(18).string(m.square_marketing_image_asset);
                                if (m.portrait_marketing_image_asset != null && Object.hasOwnProperty.call(m, "portrait_marketing_image_asset"))
                                    w.uint32(26).string(m.portrait_marketing_image_asset);
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(34).string(m.headline);
                                if (m.call_to_action_text != null && Object.hasOwnProperty.call(m, "call_to_action_text"))
                                    w.uint32(42).string(m.call_to_action_text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DiscoveryCarouselCardAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DiscoveryCarouselCardAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDiscoveryCarouselCardAsset} message DiscoveryCarouselCardAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DiscoveryCarouselCardAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DiscoveryCarouselCardAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselCardAsset} DiscoveryCarouselCardAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DiscoveryCarouselCardAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.marketing_image_asset = r.string();
                                        break;
                                    case 2:
                                        m.square_marketing_image_asset = r.string();
                                        break;
                                    case 3:
                                        m.portrait_marketing_image_asset = r.string();
                                        break;
                                    case 4:
                                        m.headline = r.string();
                                        break;
                                    case 5:
                                        m.call_to_action_text = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DiscoveryCarouselCardAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselCardAsset} DiscoveryCarouselCardAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DiscoveryCarouselCardAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DiscoveryCarouselCardAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DiscoveryCarouselCardAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.marketing_image_asset != null && m.hasOwnProperty("marketing_image_asset")) {
                                    if (!$util.isString(m.marketing_image_asset))
                                        return "marketing_image_asset: string expected";
                                }
                                if (m.square_marketing_image_asset != null && m.hasOwnProperty("square_marketing_image_asset")) {
                                    if (!$util.isString(m.square_marketing_image_asset))
                                        return "square_marketing_image_asset: string expected";
                                }
                                if (m.portrait_marketing_image_asset != null && m.hasOwnProperty("portrait_marketing_image_asset")) {
                                    if (!$util.isString(m.portrait_marketing_image_asset))
                                        return "portrait_marketing_image_asset: string expected";
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    if (!$util.isString(m.call_to_action_text))
                                        return "call_to_action_text: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DiscoveryCarouselCardAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DiscoveryCarouselCardAsset} DiscoveryCarouselCardAsset
                             */
                            DiscoveryCarouselCardAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset();
                                if (d.marketing_image_asset != null) {
                                    m.marketing_image_asset = String(d.marketing_image_asset);
                                }
                                if (d.square_marketing_image_asset != null) {
                                    m.square_marketing_image_asset = String(d.square_marketing_image_asset);
                                }
                                if (d.portrait_marketing_image_asset != null) {
                                    m.portrait_marketing_image_asset = String(d.portrait_marketing_image_asset);
                                }
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.call_to_action_text != null) {
                                    m.call_to_action_text = String(d.call_to_action_text);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DiscoveryCarouselCardAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DiscoveryCarouselCardAsset} m DiscoveryCarouselCardAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DiscoveryCarouselCardAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.marketing_image_asset = "";
                                    d.square_marketing_image_asset = "";
                                    d.portrait_marketing_image_asset = "";
                                    d.headline = "";
                                    d.call_to_action_text = "";
                                }
                                if (m.marketing_image_asset != null && m.hasOwnProperty("marketing_image_asset")) {
                                    d.marketing_image_asset = m.marketing_image_asset;
                                }
                                if (m.square_marketing_image_asset != null && m.hasOwnProperty("square_marketing_image_asset")) {
                                    d.square_marketing_image_asset = m.square_marketing_image_asset;
                                }
                                if (m.portrait_marketing_image_asset != null && m.hasOwnProperty("portrait_marketing_image_asset")) {
                                    d.portrait_marketing_image_asset = m.portrait_marketing_image_asset;
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                }
                                if (m.call_to_action_text != null && m.hasOwnProperty("call_to_action_text")) {
                                    d.call_to_action_text = m.call_to_action_text;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DiscoveryCarouselCardAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DiscoveryCarouselCardAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DiscoveryCarouselCardAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DiscoveryCarouselCardAsset;
                        })();
    
                        common.DynamicTravelAsset = (function() {
    
                            /**
                             * Properties of a DynamicTravelAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicTravelAsset
                             * @property {string|null} [destination_id] DynamicTravelAsset destination_id
                             * @property {string|null} [origin_id] DynamicTravelAsset origin_id
                             * @property {string|null} [title] DynamicTravelAsset title
                             * @property {string|null} [destination_name] DynamicTravelAsset destination_name
                             * @property {string|null} [destination_address] DynamicTravelAsset destination_address
                             * @property {string|null} [origin_name] DynamicTravelAsset origin_name
                             * @property {string|null} [price] DynamicTravelAsset price
                             * @property {string|null} [sale_price] DynamicTravelAsset sale_price
                             * @property {string|null} [formatted_price] DynamicTravelAsset formatted_price
                             * @property {string|null} [formatted_sale_price] DynamicTravelAsset formatted_sale_price
                             * @property {string|null} [category] DynamicTravelAsset category
                             * @property {Array.<string>|null} [contextual_keywords] DynamicTravelAsset contextual_keywords
                             * @property {Array.<string>|null} [similar_destination_ids] DynamicTravelAsset similar_destination_ids
                             * @property {string|null} [image_url] DynamicTravelAsset image_url
                             * @property {string|null} [android_app_link] DynamicTravelAsset android_app_link
                             * @property {string|null} [ios_app_link] DynamicTravelAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicTravelAsset ios_app_store_id
                             */
    
                            /**
                             * Constructs a new DynamicTravelAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicTravelAsset.
                             * @implements IDynamicTravelAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicTravelAsset=} [p] Properties to set
                             */
                            function DynamicTravelAsset(p) {
                                this.contextual_keywords = [];
                                this.similar_destination_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicTravelAsset destination_id.
                             * @member {string} destination_id
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.destination_id = "";
    
                            /**
                             * DynamicTravelAsset origin_id.
                             * @member {string} origin_id
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.origin_id = "";
    
                            /**
                             * DynamicTravelAsset title.
                             * @member {string} title
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.title = "";
    
                            /**
                             * DynamicTravelAsset destination_name.
                             * @member {string} destination_name
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.destination_name = "";
    
                            /**
                             * DynamicTravelAsset destination_address.
                             * @member {string} destination_address
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.destination_address = "";
    
                            /**
                             * DynamicTravelAsset origin_name.
                             * @member {string} origin_name
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.origin_name = "";
    
                            /**
                             * DynamicTravelAsset price.
                             * @member {string} price
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.price = "";
    
                            /**
                             * DynamicTravelAsset sale_price.
                             * @member {string} sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.sale_price = "";
    
                            /**
                             * DynamicTravelAsset formatted_price.
                             * @member {string} formatted_price
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.formatted_price = "";
    
                            /**
                             * DynamicTravelAsset formatted_sale_price.
                             * @member {string} formatted_sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.formatted_sale_price = "";
    
                            /**
                             * DynamicTravelAsset category.
                             * @member {string} category
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.category = "";
    
                            /**
                             * DynamicTravelAsset contextual_keywords.
                             * @member {Array.<string>} contextual_keywords
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.contextual_keywords = $util.emptyArray;
    
                            /**
                             * DynamicTravelAsset similar_destination_ids.
                             * @member {Array.<string>} similar_destination_ids
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.similar_destination_ids = $util.emptyArray;
    
                            /**
                             * DynamicTravelAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.image_url = "";
    
                            /**
                             * DynamicTravelAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicTravelAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicTravelAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             */
                            DynamicTravelAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new DynamicTravelAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicTravelAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicTravelAsset} DynamicTravelAsset instance
                             */
                            DynamicTravelAsset.create = function create(properties) {
                                return new DynamicTravelAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicTravelAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicTravelAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicTravelAsset} m DynamicTravelAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicTravelAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.destination_id != null && Object.hasOwnProperty.call(m, "destination_id"))
                                    w.uint32(10).string(m.destination_id);
                                if (m.origin_id != null && Object.hasOwnProperty.call(m, "origin_id"))
                                    w.uint32(18).string(m.origin_id);
                                if (m.title != null && Object.hasOwnProperty.call(m, "title"))
                                    w.uint32(26).string(m.title);
                                if (m.destination_name != null && Object.hasOwnProperty.call(m, "destination_name"))
                                    w.uint32(34).string(m.destination_name);
                                if (m.destination_address != null && Object.hasOwnProperty.call(m, "destination_address"))
                                    w.uint32(42).string(m.destination_address);
                                if (m.origin_name != null && Object.hasOwnProperty.call(m, "origin_name"))
                                    w.uint32(50).string(m.origin_name);
                                if (m.price != null && Object.hasOwnProperty.call(m, "price"))
                                    w.uint32(58).string(m.price);
                                if (m.sale_price != null && Object.hasOwnProperty.call(m, "sale_price"))
                                    w.uint32(66).string(m.sale_price);
                                if (m.formatted_price != null && Object.hasOwnProperty.call(m, "formatted_price"))
                                    w.uint32(74).string(m.formatted_price);
                                if (m.formatted_sale_price != null && Object.hasOwnProperty.call(m, "formatted_sale_price"))
                                    w.uint32(82).string(m.formatted_sale_price);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(90).string(m.category);
                                if (m.contextual_keywords != null && m.contextual_keywords.length) {
                                    for (var i = 0; i < m.contextual_keywords.length; ++i)
                                        w.uint32(98).string(m.contextual_keywords[i]);
                                }
                                if (m.similar_destination_ids != null && m.similar_destination_ids.length) {
                                    for (var i = 0; i < m.similar_destination_ids.length; ++i)
                                        w.uint32(106).string(m.similar_destination_ids[i]);
                                }
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(114).string(m.image_url);
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(122).string(m.android_app_link);
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(130).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(136).int64(m.ios_app_store_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicTravelAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicTravelAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicTravelAsset} message DynamicTravelAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicTravelAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicTravelAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicTravelAsset} DynamicTravelAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicTravelAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicTravelAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.destination_id = r.string();
                                        break;
                                    case 2:
                                        m.origin_id = r.string();
                                        break;
                                    case 3:
                                        m.title = r.string();
                                        break;
                                    case 4:
                                        m.destination_name = r.string();
                                        break;
                                    case 5:
                                        m.destination_address = r.string();
                                        break;
                                    case 6:
                                        m.origin_name = r.string();
                                        break;
                                    case 7:
                                        m.price = r.string();
                                        break;
                                    case 8:
                                        m.sale_price = r.string();
                                        break;
                                    case 9:
                                        m.formatted_price = r.string();
                                        break;
                                    case 10:
                                        m.formatted_sale_price = r.string();
                                        break;
                                    case 11:
                                        m.category = r.string();
                                        break;
                                    case 12:
                                        if (!(m.contextual_keywords && m.contextual_keywords.length))
                                            m.contextual_keywords = [];
                                        m.contextual_keywords.push(r.string());
                                        break;
                                    case 13:
                                        if (!(m.similar_destination_ids && m.similar_destination_ids.length))
                                            m.similar_destination_ids = [];
                                        m.similar_destination_ids.push(r.string());
                                        break;
                                    case 14:
                                        m.image_url = r.string();
                                        break;
                                    case 15:
                                        m.android_app_link = r.string();
                                        break;
                                    case 16:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 17:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicTravelAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicTravelAsset} DynamicTravelAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicTravelAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicTravelAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicTravelAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.destination_id != null && m.hasOwnProperty("destination_id")) {
                                    if (!$util.isString(m.destination_id))
                                        return "destination_id: string expected";
                                }
                                if (m.origin_id != null && m.hasOwnProperty("origin_id")) {
                                    if (!$util.isString(m.origin_id))
                                        return "origin_id: string expected";
                                }
                                if (m.title != null && m.hasOwnProperty("title")) {
                                    if (!$util.isString(m.title))
                                        return "title: string expected";
                                }
                                if (m.destination_name != null && m.hasOwnProperty("destination_name")) {
                                    if (!$util.isString(m.destination_name))
                                        return "destination_name: string expected";
                                }
                                if (m.destination_address != null && m.hasOwnProperty("destination_address")) {
                                    if (!$util.isString(m.destination_address))
                                        return "destination_address: string expected";
                                }
                                if (m.origin_name != null && m.hasOwnProperty("origin_name")) {
                                    if (!$util.isString(m.origin_name))
                                        return "origin_name: string expected";
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    if (!$util.isString(m.price))
                                        return "price: string expected";
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    if (!$util.isString(m.sale_price))
                                        return "sale_price: string expected";
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    if (!$util.isString(m.formatted_price))
                                        return "formatted_price: string expected";
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    if (!$util.isString(m.formatted_sale_price))
                                        return "formatted_sale_price: string expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    if (!$util.isString(m.category))
                                        return "category: string expected";
                                }
                                if (m.contextual_keywords != null && m.hasOwnProperty("contextual_keywords")) {
                                    if (!Array.isArray(m.contextual_keywords))
                                        return "contextual_keywords: array expected";
                                    for (var i = 0; i < m.contextual_keywords.length; ++i) {
                                        if (!$util.isString(m.contextual_keywords[i]))
                                            return "contextual_keywords: string[] expected";
                                    }
                                }
                                if (m.similar_destination_ids != null && m.hasOwnProperty("similar_destination_ids")) {
                                    if (!Array.isArray(m.similar_destination_ids))
                                        return "similar_destination_ids: array expected";
                                    for (var i = 0; i < m.similar_destination_ids.length; ++i) {
                                        if (!$util.isString(m.similar_destination_ids[i]))
                                            return "similar_destination_ids: string[] expected";
                                    }
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicTravelAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicTravelAsset} DynamicTravelAsset
                             */
                            DynamicTravelAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicTravelAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicTravelAsset();
                                if (d.destination_id != null) {
                                    m.destination_id = String(d.destination_id);
                                }
                                if (d.origin_id != null) {
                                    m.origin_id = String(d.origin_id);
                                }
                                if (d.title != null) {
                                    m.title = String(d.title);
                                }
                                if (d.destination_name != null) {
                                    m.destination_name = String(d.destination_name);
                                }
                                if (d.destination_address != null) {
                                    m.destination_address = String(d.destination_address);
                                }
                                if (d.origin_name != null) {
                                    m.origin_name = String(d.origin_name);
                                }
                                if (d.price != null) {
                                    m.price = String(d.price);
                                }
                                if (d.sale_price != null) {
                                    m.sale_price = String(d.sale_price);
                                }
                                if (d.formatted_price != null) {
                                    m.formatted_price = String(d.formatted_price);
                                }
                                if (d.formatted_sale_price != null) {
                                    m.formatted_sale_price = String(d.formatted_sale_price);
                                }
                                if (d.category != null) {
                                    m.category = String(d.category);
                                }
                                if (d.contextual_keywords) {
                                    if (!Array.isArray(d.contextual_keywords))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicTravelAsset.contextual_keywords: array expected");
                                    m.contextual_keywords = [];
                                    for (var i = 0; i < d.contextual_keywords.length; ++i) {
                                        m.contextual_keywords[i] = String(d.contextual_keywords[i]);
                                    }
                                }
                                if (d.similar_destination_ids) {
                                    if (!Array.isArray(d.similar_destination_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicTravelAsset.similar_destination_ids: array expected");
                                    m.similar_destination_ids = [];
                                    for (var i = 0; i < d.similar_destination_ids.length; ++i) {
                                        m.similar_destination_ids[i] = String(d.similar_destination_ids[i]);
                                    }
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicTravelAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicTravelAsset} m DynamicTravelAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicTravelAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.contextual_keywords = [];
                                    d.similar_destination_ids = [];
                                }
                                if (o.defaults) {
                                    d.destination_id = "";
                                    d.origin_id = "";
                                    d.title = "";
                                    d.destination_name = "";
                                    d.destination_address = "";
                                    d.origin_name = "";
                                    d.price = "";
                                    d.sale_price = "";
                                    d.formatted_price = "";
                                    d.formatted_sale_price = "";
                                    d.category = "";
                                    d.image_url = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                }
                                if (m.destination_id != null && m.hasOwnProperty("destination_id")) {
                                    d.destination_id = m.destination_id;
                                }
                                if (m.origin_id != null && m.hasOwnProperty("origin_id")) {
                                    d.origin_id = m.origin_id;
                                }
                                if (m.title != null && m.hasOwnProperty("title")) {
                                    d.title = m.title;
                                }
                                if (m.destination_name != null && m.hasOwnProperty("destination_name")) {
                                    d.destination_name = m.destination_name;
                                }
                                if (m.destination_address != null && m.hasOwnProperty("destination_address")) {
                                    d.destination_address = m.destination_address;
                                }
                                if (m.origin_name != null && m.hasOwnProperty("origin_name")) {
                                    d.origin_name = m.origin_name;
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    d.price = m.price;
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    d.sale_price = m.sale_price;
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    d.formatted_price = m.formatted_price;
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    d.formatted_sale_price = m.formatted_sale_price;
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = m.category;
                                }
                                if (m.contextual_keywords && m.contextual_keywords.length) {
                                    d.contextual_keywords = [];
                                    for (var j = 0; j < m.contextual_keywords.length; ++j) {
                                        d.contextual_keywords[j] = m.contextual_keywords[j];
                                    }
                                }
                                if (m.similar_destination_ids && m.similar_destination_ids.length) {
                                    d.similar_destination_ids = [];
                                    for (var j = 0; j < m.similar_destination_ids.length; ++j) {
                                        d.similar_destination_ids[j] = m.similar_destination_ids[j];
                                    }
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicTravelAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicTravelAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicTravelAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicTravelAsset;
                        })();
    
                        common.DynamicLocalAsset = (function() {
    
                            /**
                             * Properties of a DynamicLocalAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicLocalAsset
                             * @property {string|null} [deal_id] DynamicLocalAsset deal_id
                             * @property {string|null} [deal_name] DynamicLocalAsset deal_name
                             * @property {string|null} [subtitle] DynamicLocalAsset subtitle
                             * @property {string|null} [description] DynamicLocalAsset description
                             * @property {string|null} [price] DynamicLocalAsset price
                             * @property {string|null} [sale_price] DynamicLocalAsset sale_price
                             * @property {string|null} [image_url] DynamicLocalAsset image_url
                             * @property {string|null} [address] DynamicLocalAsset address
                             * @property {string|null} [category] DynamicLocalAsset category
                             * @property {Array.<string>|null} [contextual_keywords] DynamicLocalAsset contextual_keywords
                             * @property {string|null} [formatted_price] DynamicLocalAsset formatted_price
                             * @property {string|null} [formatted_sale_price] DynamicLocalAsset formatted_sale_price
                             * @property {string|null} [android_app_link] DynamicLocalAsset android_app_link
                             * @property {Array.<string>|null} [similar_deal_ids] DynamicLocalAsset similar_deal_ids
                             * @property {string|null} [ios_app_link] DynamicLocalAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicLocalAsset ios_app_store_id
                             */
    
                            /**
                             * Constructs a new DynamicLocalAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicLocalAsset.
                             * @implements IDynamicLocalAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicLocalAsset=} [p] Properties to set
                             */
                            function DynamicLocalAsset(p) {
                                this.contextual_keywords = [];
                                this.similar_deal_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicLocalAsset deal_id.
                             * @member {string} deal_id
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.deal_id = "";
    
                            /**
                             * DynamicLocalAsset deal_name.
                             * @member {string} deal_name
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.deal_name = "";
    
                            /**
                             * DynamicLocalAsset subtitle.
                             * @member {string} subtitle
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.subtitle = "";
    
                            /**
                             * DynamicLocalAsset description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.description = "";
    
                            /**
                             * DynamicLocalAsset price.
                             * @member {string} price
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.price = "";
    
                            /**
                             * DynamicLocalAsset sale_price.
                             * @member {string} sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.sale_price = "";
    
                            /**
                             * DynamicLocalAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.image_url = "";
    
                            /**
                             * DynamicLocalAsset address.
                             * @member {string} address
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.address = "";
    
                            /**
                             * DynamicLocalAsset category.
                             * @member {string} category
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.category = "";
    
                            /**
                             * DynamicLocalAsset contextual_keywords.
                             * @member {Array.<string>} contextual_keywords
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.contextual_keywords = $util.emptyArray;
    
                            /**
                             * DynamicLocalAsset formatted_price.
                             * @member {string} formatted_price
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.formatted_price = "";
    
                            /**
                             * DynamicLocalAsset formatted_sale_price.
                             * @member {string} formatted_sale_price
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.formatted_sale_price = "";
    
                            /**
                             * DynamicLocalAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicLocalAsset similar_deal_ids.
                             * @member {Array.<string>} similar_deal_ids
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.similar_deal_ids = $util.emptyArray;
    
                            /**
                             * DynamicLocalAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicLocalAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             */
                            DynamicLocalAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new DynamicLocalAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicLocalAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicLocalAsset} DynamicLocalAsset instance
                             */
                            DynamicLocalAsset.create = function create(properties) {
                                return new DynamicLocalAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicLocalAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicLocalAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicLocalAsset} m DynamicLocalAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicLocalAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.deal_id != null && Object.hasOwnProperty.call(m, "deal_id"))
                                    w.uint32(10).string(m.deal_id);
                                if (m.deal_name != null && Object.hasOwnProperty.call(m, "deal_name"))
                                    w.uint32(18).string(m.deal_name);
                                if (m.subtitle != null && Object.hasOwnProperty.call(m, "subtitle"))
                                    w.uint32(26).string(m.subtitle);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(34).string(m.description);
                                if (m.price != null && Object.hasOwnProperty.call(m, "price"))
                                    w.uint32(42).string(m.price);
                                if (m.sale_price != null && Object.hasOwnProperty.call(m, "sale_price"))
                                    w.uint32(50).string(m.sale_price);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(58).string(m.image_url);
                                if (m.address != null && Object.hasOwnProperty.call(m, "address"))
                                    w.uint32(66).string(m.address);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(74).string(m.category);
                                if (m.contextual_keywords != null && m.contextual_keywords.length) {
                                    for (var i = 0; i < m.contextual_keywords.length; ++i)
                                        w.uint32(82).string(m.contextual_keywords[i]);
                                }
                                if (m.formatted_price != null && Object.hasOwnProperty.call(m, "formatted_price"))
                                    w.uint32(90).string(m.formatted_price);
                                if (m.formatted_sale_price != null && Object.hasOwnProperty.call(m, "formatted_sale_price"))
                                    w.uint32(98).string(m.formatted_sale_price);
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(106).string(m.android_app_link);
                                if (m.similar_deal_ids != null && m.similar_deal_ids.length) {
                                    for (var i = 0; i < m.similar_deal_ids.length; ++i)
                                        w.uint32(114).string(m.similar_deal_ids[i]);
                                }
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(122).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(128).int64(m.ios_app_store_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicLocalAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicLocalAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicLocalAsset} message DynamicLocalAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicLocalAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicLocalAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicLocalAsset} DynamicLocalAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicLocalAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicLocalAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.deal_id = r.string();
                                        break;
                                    case 2:
                                        m.deal_name = r.string();
                                        break;
                                    case 3:
                                        m.subtitle = r.string();
                                        break;
                                    case 4:
                                        m.description = r.string();
                                        break;
                                    case 5:
                                        m.price = r.string();
                                        break;
                                    case 6:
                                        m.sale_price = r.string();
                                        break;
                                    case 7:
                                        m.image_url = r.string();
                                        break;
                                    case 8:
                                        m.address = r.string();
                                        break;
                                    case 9:
                                        m.category = r.string();
                                        break;
                                    case 10:
                                        if (!(m.contextual_keywords && m.contextual_keywords.length))
                                            m.contextual_keywords = [];
                                        m.contextual_keywords.push(r.string());
                                        break;
                                    case 11:
                                        m.formatted_price = r.string();
                                        break;
                                    case 12:
                                        m.formatted_sale_price = r.string();
                                        break;
                                    case 13:
                                        m.android_app_link = r.string();
                                        break;
                                    case 14:
                                        if (!(m.similar_deal_ids && m.similar_deal_ids.length))
                                            m.similar_deal_ids = [];
                                        m.similar_deal_ids.push(r.string());
                                        break;
                                    case 15:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 16:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicLocalAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicLocalAsset} DynamicLocalAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicLocalAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicLocalAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicLocalAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.deal_id != null && m.hasOwnProperty("deal_id")) {
                                    if (!$util.isString(m.deal_id))
                                        return "deal_id: string expected";
                                }
                                if (m.deal_name != null && m.hasOwnProperty("deal_name")) {
                                    if (!$util.isString(m.deal_name))
                                        return "deal_name: string expected";
                                }
                                if (m.subtitle != null && m.hasOwnProperty("subtitle")) {
                                    if (!$util.isString(m.subtitle))
                                        return "subtitle: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    if (!$util.isString(m.price))
                                        return "price: string expected";
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    if (!$util.isString(m.sale_price))
                                        return "sale_price: string expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    if (!$util.isString(m.address))
                                        return "address: string expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    if (!$util.isString(m.category))
                                        return "category: string expected";
                                }
                                if (m.contextual_keywords != null && m.hasOwnProperty("contextual_keywords")) {
                                    if (!Array.isArray(m.contextual_keywords))
                                        return "contextual_keywords: array expected";
                                    for (var i = 0; i < m.contextual_keywords.length; ++i) {
                                        if (!$util.isString(m.contextual_keywords[i]))
                                            return "contextual_keywords: string[] expected";
                                    }
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    if (!$util.isString(m.formatted_price))
                                        return "formatted_price: string expected";
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    if (!$util.isString(m.formatted_sale_price))
                                        return "formatted_sale_price: string expected";
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.similar_deal_ids != null && m.hasOwnProperty("similar_deal_ids")) {
                                    if (!Array.isArray(m.similar_deal_ids))
                                        return "similar_deal_ids: array expected";
                                    for (var i = 0; i < m.similar_deal_ids.length; ++i) {
                                        if (!$util.isString(m.similar_deal_ids[i]))
                                            return "similar_deal_ids: string[] expected";
                                    }
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicLocalAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicLocalAsset} DynamicLocalAsset
                             */
                            DynamicLocalAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicLocalAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicLocalAsset();
                                if (d.deal_id != null) {
                                    m.deal_id = String(d.deal_id);
                                }
                                if (d.deal_name != null) {
                                    m.deal_name = String(d.deal_name);
                                }
                                if (d.subtitle != null) {
                                    m.subtitle = String(d.subtitle);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.price != null) {
                                    m.price = String(d.price);
                                }
                                if (d.sale_price != null) {
                                    m.sale_price = String(d.sale_price);
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.address != null) {
                                    m.address = String(d.address);
                                }
                                if (d.category != null) {
                                    m.category = String(d.category);
                                }
                                if (d.contextual_keywords) {
                                    if (!Array.isArray(d.contextual_keywords))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicLocalAsset.contextual_keywords: array expected");
                                    m.contextual_keywords = [];
                                    for (var i = 0; i < d.contextual_keywords.length; ++i) {
                                        m.contextual_keywords[i] = String(d.contextual_keywords[i]);
                                    }
                                }
                                if (d.formatted_price != null) {
                                    m.formatted_price = String(d.formatted_price);
                                }
                                if (d.formatted_sale_price != null) {
                                    m.formatted_sale_price = String(d.formatted_sale_price);
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.similar_deal_ids) {
                                    if (!Array.isArray(d.similar_deal_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicLocalAsset.similar_deal_ids: array expected");
                                    m.similar_deal_ids = [];
                                    for (var i = 0; i < d.similar_deal_ids.length; ++i) {
                                        m.similar_deal_ids[i] = String(d.similar_deal_ids[i]);
                                    }
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicLocalAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicLocalAsset} m DynamicLocalAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicLocalAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.contextual_keywords = [];
                                    d.similar_deal_ids = [];
                                }
                                if (o.defaults) {
                                    d.deal_id = "";
                                    d.deal_name = "";
                                    d.subtitle = "";
                                    d.description = "";
                                    d.price = "";
                                    d.sale_price = "";
                                    d.image_url = "";
                                    d.address = "";
                                    d.category = "";
                                    d.formatted_price = "";
                                    d.formatted_sale_price = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                }
                                if (m.deal_id != null && m.hasOwnProperty("deal_id")) {
                                    d.deal_id = m.deal_id;
                                }
                                if (m.deal_name != null && m.hasOwnProperty("deal_name")) {
                                    d.deal_name = m.deal_name;
                                }
                                if (m.subtitle != null && m.hasOwnProperty("subtitle")) {
                                    d.subtitle = m.subtitle;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    d.price = m.price;
                                }
                                if (m.sale_price != null && m.hasOwnProperty("sale_price")) {
                                    d.sale_price = m.sale_price;
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    d.address = m.address;
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = m.category;
                                }
                                if (m.contextual_keywords && m.contextual_keywords.length) {
                                    d.contextual_keywords = [];
                                    for (var j = 0; j < m.contextual_keywords.length; ++j) {
                                        d.contextual_keywords[j] = m.contextual_keywords[j];
                                    }
                                }
                                if (m.formatted_price != null && m.hasOwnProperty("formatted_price")) {
                                    d.formatted_price = m.formatted_price;
                                }
                                if (m.formatted_sale_price != null && m.hasOwnProperty("formatted_sale_price")) {
                                    d.formatted_sale_price = m.formatted_sale_price;
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.similar_deal_ids && m.similar_deal_ids.length) {
                                    d.similar_deal_ids = [];
                                    for (var j = 0; j < m.similar_deal_ids.length; ++j) {
                                        d.similar_deal_ids[j] = m.similar_deal_ids[j];
                                    }
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicLocalAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicLocalAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicLocalAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicLocalAsset;
                        })();
    
                        common.DynamicJobsAsset = (function() {
    
                            /**
                             * Properties of a DynamicJobsAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicJobsAsset
                             * @property {string|null} [job_id] DynamicJobsAsset job_id
                             * @property {string|null} [location_id] DynamicJobsAsset location_id
                             * @property {string|null} [job_title] DynamicJobsAsset job_title
                             * @property {string|null} [job_subtitle] DynamicJobsAsset job_subtitle
                             * @property {string|null} [description] DynamicJobsAsset description
                             * @property {string|null} [image_url] DynamicJobsAsset image_url
                             * @property {string|null} [job_category] DynamicJobsAsset job_category
                             * @property {Array.<string>|null} [contextual_keywords] DynamicJobsAsset contextual_keywords
                             * @property {string|null} [address] DynamicJobsAsset address
                             * @property {string|null} [salary] DynamicJobsAsset salary
                             * @property {string|null} [android_app_link] DynamicJobsAsset android_app_link
                             * @property {Array.<string>|null} [similar_job_ids] DynamicJobsAsset similar_job_ids
                             * @property {string|null} [ios_app_link] DynamicJobsAsset ios_app_link
                             * @property {number|null} [ios_app_store_id] DynamicJobsAsset ios_app_store_id
                             */
    
                            /**
                             * Constructs a new DynamicJobsAsset.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicJobsAsset.
                             * @implements IDynamicJobsAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicJobsAsset=} [p] Properties to set
                             */
                            function DynamicJobsAsset(p) {
                                this.contextual_keywords = [];
                                this.similar_job_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicJobsAsset job_id.
                             * @member {string} job_id
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.job_id = "";
    
                            /**
                             * DynamicJobsAsset location_id.
                             * @member {string} location_id
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.location_id = "";
    
                            /**
                             * DynamicJobsAsset job_title.
                             * @member {string} job_title
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.job_title = "";
    
                            /**
                             * DynamicJobsAsset job_subtitle.
                             * @member {string} job_subtitle
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.job_subtitle = "";
    
                            /**
                             * DynamicJobsAsset description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.description = "";
    
                            /**
                             * DynamicJobsAsset image_url.
                             * @member {string} image_url
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.image_url = "";
    
                            /**
                             * DynamicJobsAsset job_category.
                             * @member {string} job_category
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.job_category = "";
    
                            /**
                             * DynamicJobsAsset contextual_keywords.
                             * @member {Array.<string>} contextual_keywords
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.contextual_keywords = $util.emptyArray;
    
                            /**
                             * DynamicJobsAsset address.
                             * @member {string} address
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.address = "";
    
                            /**
                             * DynamicJobsAsset salary.
                             * @member {string} salary
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.salary = "";
    
                            /**
                             * DynamicJobsAsset android_app_link.
                             * @member {string} android_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.android_app_link = "";
    
                            /**
                             * DynamicJobsAsset similar_job_ids.
                             * @member {Array.<string>} similar_job_ids
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.similar_job_ids = $util.emptyArray;
    
                            /**
                             * DynamicJobsAsset ios_app_link.
                             * @member {string} ios_app_link
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.ios_app_link = "";
    
                            /**
                             * DynamicJobsAsset ios_app_store_id.
                             * @member {number} ios_app_store_id
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             */
                            DynamicJobsAsset.prototype.ios_app_store_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new DynamicJobsAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicJobsAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicJobsAsset} DynamicJobsAsset instance
                             */
                            DynamicJobsAsset.create = function create(properties) {
                                return new DynamicJobsAsset(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicJobsAsset message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicJobsAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicJobsAsset} m DynamicJobsAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicJobsAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.job_id != null && Object.hasOwnProperty.call(m, "job_id"))
                                    w.uint32(10).string(m.job_id);
                                if (m.location_id != null && Object.hasOwnProperty.call(m, "location_id"))
                                    w.uint32(18).string(m.location_id);
                                if (m.job_title != null && Object.hasOwnProperty.call(m, "job_title"))
                                    w.uint32(26).string(m.job_title);
                                if (m.job_subtitle != null && Object.hasOwnProperty.call(m, "job_subtitle"))
                                    w.uint32(34).string(m.job_subtitle);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(42).string(m.description);
                                if (m.image_url != null && Object.hasOwnProperty.call(m, "image_url"))
                                    w.uint32(50).string(m.image_url);
                                if (m.job_category != null && Object.hasOwnProperty.call(m, "job_category"))
                                    w.uint32(58).string(m.job_category);
                                if (m.contextual_keywords != null && m.contextual_keywords.length) {
                                    for (var i = 0; i < m.contextual_keywords.length; ++i)
                                        w.uint32(66).string(m.contextual_keywords[i]);
                                }
                                if (m.address != null && Object.hasOwnProperty.call(m, "address"))
                                    w.uint32(74).string(m.address);
                                if (m.salary != null && Object.hasOwnProperty.call(m, "salary"))
                                    w.uint32(82).string(m.salary);
                                if (m.android_app_link != null && Object.hasOwnProperty.call(m, "android_app_link"))
                                    w.uint32(90).string(m.android_app_link);
                                if (m.similar_job_ids != null && m.similar_job_ids.length) {
                                    for (var i = 0; i < m.similar_job_ids.length; ++i)
                                        w.uint32(98).string(m.similar_job_ids[i]);
                                }
                                if (m.ios_app_link != null && Object.hasOwnProperty.call(m, "ios_app_link"))
                                    w.uint32(106).string(m.ios_app_link);
                                if (m.ios_app_store_id != null && Object.hasOwnProperty.call(m, "ios_app_store_id"))
                                    w.uint32(112).int64(m.ios_app_store_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicJobsAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicJobsAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicJobsAsset} message DynamicJobsAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicJobsAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicJobsAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicJobsAsset} DynamicJobsAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicJobsAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicJobsAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.job_id = r.string();
                                        break;
                                    case 2:
                                        m.location_id = r.string();
                                        break;
                                    case 3:
                                        m.job_title = r.string();
                                        break;
                                    case 4:
                                        m.job_subtitle = r.string();
                                        break;
                                    case 5:
                                        m.description = r.string();
                                        break;
                                    case 6:
                                        m.image_url = r.string();
                                        break;
                                    case 7:
                                        m.job_category = r.string();
                                        break;
                                    case 8:
                                        if (!(m.contextual_keywords && m.contextual_keywords.length))
                                            m.contextual_keywords = [];
                                        m.contextual_keywords.push(r.string());
                                        break;
                                    case 9:
                                        m.address = r.string();
                                        break;
                                    case 10:
                                        m.salary = r.string();
                                        break;
                                    case 11:
                                        m.android_app_link = r.string();
                                        break;
                                    case 12:
                                        if (!(m.similar_job_ids && m.similar_job_ids.length))
                                            m.similar_job_ids = [];
                                        m.similar_job_ids.push(r.string());
                                        break;
                                    case 13:
                                        m.ios_app_link = r.string();
                                        break;
                                    case 14:
                                        m.ios_app_store_id = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicJobsAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicJobsAsset} DynamicJobsAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicJobsAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicJobsAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicJobsAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.job_id != null && m.hasOwnProperty("job_id")) {
                                    if (!$util.isString(m.job_id))
                                        return "job_id: string expected";
                                }
                                if (m.location_id != null && m.hasOwnProperty("location_id")) {
                                    if (!$util.isString(m.location_id))
                                        return "location_id: string expected";
                                }
                                if (m.job_title != null && m.hasOwnProperty("job_title")) {
                                    if (!$util.isString(m.job_title))
                                        return "job_title: string expected";
                                }
                                if (m.job_subtitle != null && m.hasOwnProperty("job_subtitle")) {
                                    if (!$util.isString(m.job_subtitle))
                                        return "job_subtitle: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    if (!$util.isString(m.image_url))
                                        return "image_url: string expected";
                                }
                                if (m.job_category != null && m.hasOwnProperty("job_category")) {
                                    if (!$util.isString(m.job_category))
                                        return "job_category: string expected";
                                }
                                if (m.contextual_keywords != null && m.hasOwnProperty("contextual_keywords")) {
                                    if (!Array.isArray(m.contextual_keywords))
                                        return "contextual_keywords: array expected";
                                    for (var i = 0; i < m.contextual_keywords.length; ++i) {
                                        if (!$util.isString(m.contextual_keywords[i]))
                                            return "contextual_keywords: string[] expected";
                                    }
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    if (!$util.isString(m.address))
                                        return "address: string expected";
                                }
                                if (m.salary != null && m.hasOwnProperty("salary")) {
                                    if (!$util.isString(m.salary))
                                        return "salary: string expected";
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    if (!$util.isString(m.android_app_link))
                                        return "android_app_link: string expected";
                                }
                                if (m.similar_job_ids != null && m.hasOwnProperty("similar_job_ids")) {
                                    if (!Array.isArray(m.similar_job_ids))
                                        return "similar_job_ids: array expected";
                                    for (var i = 0; i < m.similar_job_ids.length; ++i) {
                                        if (!$util.isString(m.similar_job_ids[i]))
                                            return "similar_job_ids: string[] expected";
                                    }
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    if (!$util.isString(m.ios_app_link))
                                        return "ios_app_link: string expected";
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (!$util.isInteger(m.ios_app_store_id) && !(m.ios_app_store_id && $util.isInteger(m.ios_app_store_id.low) && $util.isInteger(m.ios_app_store_id.high)))
                                        return "ios_app_store_id: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicJobsAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicJobsAsset} DynamicJobsAsset
                             */
                            DynamicJobsAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicJobsAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicJobsAsset();
                                if (d.job_id != null) {
                                    m.job_id = String(d.job_id);
                                }
                                if (d.location_id != null) {
                                    m.location_id = String(d.location_id);
                                }
                                if (d.job_title != null) {
                                    m.job_title = String(d.job_title);
                                }
                                if (d.job_subtitle != null) {
                                    m.job_subtitle = String(d.job_subtitle);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.image_url != null) {
                                    m.image_url = String(d.image_url);
                                }
                                if (d.job_category != null) {
                                    m.job_category = String(d.job_category);
                                }
                                if (d.contextual_keywords) {
                                    if (!Array.isArray(d.contextual_keywords))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicJobsAsset.contextual_keywords: array expected");
                                    m.contextual_keywords = [];
                                    for (var i = 0; i < d.contextual_keywords.length; ++i) {
                                        m.contextual_keywords[i] = String(d.contextual_keywords[i]);
                                    }
                                }
                                if (d.address != null) {
                                    m.address = String(d.address);
                                }
                                if (d.salary != null) {
                                    m.salary = String(d.salary);
                                }
                                if (d.android_app_link != null) {
                                    m.android_app_link = String(d.android_app_link);
                                }
                                if (d.similar_job_ids) {
                                    if (!Array.isArray(d.similar_job_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicJobsAsset.similar_job_ids: array expected");
                                    m.similar_job_ids = [];
                                    for (var i = 0; i < d.similar_job_ids.length; ++i) {
                                        m.similar_job_ids[i] = String(d.similar_job_ids[i]);
                                    }
                                }
                                if (d.ios_app_link != null) {
                                    m.ios_app_link = String(d.ios_app_link);
                                }
                                if (d.ios_app_store_id != null) {
                                    if ($util.Long)
                                        (m.ios_app_store_id = $util.Long.fromValue(d.ios_app_store_id)).unsigned = false;
                                    else if (typeof d.ios_app_store_id === "string")
                                        m.ios_app_store_id = parseInt(d.ios_app_store_id, 10);
                                    else if (typeof d.ios_app_store_id === "number")
                                        m.ios_app_store_id = d.ios_app_store_id;
                                    else if (typeof d.ios_app_store_id === "object")
                                        m.ios_app_store_id = new $util.LongBits(d.ios_app_store_id.low >>> 0, d.ios_app_store_id.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicJobsAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicJobsAsset} m DynamicJobsAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicJobsAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.contextual_keywords = [];
                                    d.similar_job_ids = [];
                                }
                                if (o.defaults) {
                                    d.job_id = "";
                                    d.location_id = "";
                                    d.job_title = "";
                                    d.job_subtitle = "";
                                    d.description = "";
                                    d.image_url = "";
                                    d.job_category = "";
                                    d.address = "";
                                    d.salary = "";
                                    d.android_app_link = "";
                                    d.ios_app_link = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.ios_app_store_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.ios_app_store_id = o.longs === String ? "0" : 0;
                                }
                                if (m.job_id != null && m.hasOwnProperty("job_id")) {
                                    d.job_id = m.job_id;
                                }
                                if (m.location_id != null && m.hasOwnProperty("location_id")) {
                                    d.location_id = m.location_id;
                                }
                                if (m.job_title != null && m.hasOwnProperty("job_title")) {
                                    d.job_title = m.job_title;
                                }
                                if (m.job_subtitle != null && m.hasOwnProperty("job_subtitle")) {
                                    d.job_subtitle = m.job_subtitle;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.image_url != null && m.hasOwnProperty("image_url")) {
                                    d.image_url = m.image_url;
                                }
                                if (m.job_category != null && m.hasOwnProperty("job_category")) {
                                    d.job_category = m.job_category;
                                }
                                if (m.contextual_keywords && m.contextual_keywords.length) {
                                    d.contextual_keywords = [];
                                    for (var j = 0; j < m.contextual_keywords.length; ++j) {
                                        d.contextual_keywords[j] = m.contextual_keywords[j];
                                    }
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    d.address = m.address;
                                }
                                if (m.salary != null && m.hasOwnProperty("salary")) {
                                    d.salary = m.salary;
                                }
                                if (m.android_app_link != null && m.hasOwnProperty("android_app_link")) {
                                    d.android_app_link = m.android_app_link;
                                }
                                if (m.similar_job_ids && m.similar_job_ids.length) {
                                    d.similar_job_ids = [];
                                    for (var j = 0; j < m.similar_job_ids.length; ++j) {
                                        d.similar_job_ids[j] = m.similar_job_ids[j];
                                    }
                                }
                                if (m.ios_app_link != null && m.hasOwnProperty("ios_app_link")) {
                                    d.ios_app_link = m.ios_app_link;
                                }
                                if (m.ios_app_store_id != null && m.hasOwnProperty("ios_app_store_id")) {
                                    if (typeof m.ios_app_store_id === "number")
                                        d.ios_app_store_id = o.longs === String ? String(m.ios_app_store_id) : m.ios_app_store_id;
                                    else
                                        d.ios_app_store_id = o.longs === String ? $util.Long.prototype.toString.call(m.ios_app_store_id) : o.longs === Number ? new $util.LongBits(m.ios_app_store_id.low >>> 0, m.ios_app_store_id.high >>> 0).toNumber() : m.ios_app_store_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicJobsAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicJobsAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicJobsAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicJobsAsset;
                        })();
    
                        common.KeywordInfo = (function() {
    
                            /**
                             * Properties of a KeywordInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordInfo
                             * @property {string|null} [text] KeywordInfo text
                             * @property {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType|null} [match_type] KeywordInfo match_type
                             */
    
                            /**
                             * Constructs a new KeywordInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordInfo.
                             * @implements IKeywordInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordInfo=} [p] Properties to set
                             */
                            function KeywordInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordInfo text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @instance
                             */
                            KeywordInfo.prototype.text = null;
    
                            /**
                             * KeywordInfo match_type.
                             * @member {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType} match_type
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @instance
                             */
                            KeywordInfo.prototype.match_type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordInfo _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @instance
                             */
                            Object.defineProperty(KeywordInfo.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordInfo} KeywordInfo instance
                             */
                            KeywordInfo.create = function create(properties) {
                                return new KeywordInfo(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordInfo message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordInfo} m KeywordInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.match_type != null && Object.hasOwnProperty.call(m, "match_type"))
                                    w.uint32(16).int32(m.match_type);
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(26).string(m.text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordInfo} message KeywordInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordInfo} KeywordInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.text = r.string();
                                        break;
                                    case 2:
                                        m.match_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordInfo} KeywordInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                    switch (m.match_type) {
                                    default:
                                        return "match_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordInfo} KeywordInfo
                             */
                            KeywordInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordInfo();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                switch (d.match_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.match_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.match_type = 1;
                                    break;
                                case "EXACT":
                                case 2:
                                    m.match_type = 2;
                                    break;
                                case "PHRASE":
                                case 3:
                                    m.match_type = 3;
                                    break;
                                case "BROAD":
                                case 4:
                                    m.match_type = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordInfo} m KeywordInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                    d.match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType[m.match_type] : m.match_type;
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordInfo;
                        })();
    
                        common.PlacementInfo = (function() {
    
                            /**
                             * Properties of a PlacementInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPlacementInfo
                             * @property {string|null} [url] PlacementInfo url
                             */
    
                            /**
                             * Constructs a new PlacementInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PlacementInfo.
                             * @implements IPlacementInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPlacementInfo=} [p] Properties to set
                             */
                            function PlacementInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PlacementInfo url.
                             * @member {string|null|undefined} url
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @instance
                             */
                            PlacementInfo.prototype.url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PlacementInfo _url.
                             * @member {"url"|undefined} _url
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @instance
                             */
                            Object.defineProperty(PlacementInfo.prototype, "_url", {
                                get: $util.oneOfGetter($oneOfFields = ["url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PlacementInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IPlacementInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PlacementInfo} PlacementInfo instance
                             */
                            PlacementInfo.create = function create(properties) {
                                return new PlacementInfo(properties);
                            };
    
                            /**
                             * Encodes the specified PlacementInfo message. Does not implicitly {@link google.ads.googleads.v11.common.PlacementInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IPlacementInfo} m PlacementInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlacementInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.url != null && Object.hasOwnProperty.call(m, "url"))
                                    w.uint32(18).string(m.url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlacementInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PlacementInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IPlacementInfo} message PlacementInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlacementInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlacementInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PlacementInfo} PlacementInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlacementInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PlacementInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlacementInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PlacementInfo} PlacementInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlacementInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlacementInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlacementInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    p._url = 1;
                                    if (!$util.isString(m.url))
                                        return "url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PlacementInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PlacementInfo} PlacementInfo
                             */
                            PlacementInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PlacementInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PlacementInfo();
                                if (d.url != null) {
                                    m.url = String(d.url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PlacementInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.PlacementInfo} m PlacementInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlacementInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    d.url = m.url;
                                    if (o.oneofs)
                                        d._url = "url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PlacementInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PlacementInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlacementInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PlacementInfo;
                        })();
    
                        common.MobileAppCategoryInfo = (function() {
    
                            /**
                             * Properties of a MobileAppCategoryInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMobileAppCategoryInfo
                             * @property {string|null} [mobile_app_category_constant] MobileAppCategoryInfo mobile_app_category_constant
                             */
    
                            /**
                             * Constructs a new MobileAppCategoryInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MobileAppCategoryInfo.
                             * @implements IMobileAppCategoryInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMobileAppCategoryInfo=} [p] Properties to set
                             */
                            function MobileAppCategoryInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MobileAppCategoryInfo mobile_app_category_constant.
                             * @member {string|null|undefined} mobile_app_category_constant
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @instance
                             */
                            MobileAppCategoryInfo.prototype.mobile_app_category_constant = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MobileAppCategoryInfo _mobile_app_category_constant.
                             * @member {"mobile_app_category_constant"|undefined} _mobile_app_category_constant
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @instance
                             */
                            Object.defineProperty(MobileAppCategoryInfo.prototype, "_mobile_app_category_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["mobile_app_category_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MobileAppCategoryInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileAppCategoryInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MobileAppCategoryInfo} MobileAppCategoryInfo instance
                             */
                            MobileAppCategoryInfo.create = function create(properties) {
                                return new MobileAppCategoryInfo(properties);
                            };
    
                            /**
                             * Encodes the specified MobileAppCategoryInfo message. Does not implicitly {@link google.ads.googleads.v11.common.MobileAppCategoryInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileAppCategoryInfo} m MobileAppCategoryInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppCategoryInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.mobile_app_category_constant != null && Object.hasOwnProperty.call(m, "mobile_app_category_constant"))
                                    w.uint32(18).string(m.mobile_app_category_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileAppCategoryInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MobileAppCategoryInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileAppCategoryInfo} message MobileAppCategoryInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppCategoryInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileAppCategoryInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MobileAppCategoryInfo} MobileAppCategoryInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppCategoryInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MobileAppCategoryInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.mobile_app_category_constant = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileAppCategoryInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MobileAppCategoryInfo} MobileAppCategoryInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppCategoryInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileAppCategoryInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileAppCategoryInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.mobile_app_category_constant != null && m.hasOwnProperty("mobile_app_category_constant")) {
                                    p._mobile_app_category_constant = 1;
                                    if (!$util.isString(m.mobile_app_category_constant))
                                        return "mobile_app_category_constant: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MobileAppCategoryInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MobileAppCategoryInfo} MobileAppCategoryInfo
                             */
                            MobileAppCategoryInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MobileAppCategoryInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MobileAppCategoryInfo();
                                if (d.mobile_app_category_constant != null) {
                                    m.mobile_app_category_constant = String(d.mobile_app_category_constant);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MobileAppCategoryInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.MobileAppCategoryInfo} m MobileAppCategoryInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileAppCategoryInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.mobile_app_category_constant != null && m.hasOwnProperty("mobile_app_category_constant")) {
                                    d.mobile_app_category_constant = m.mobile_app_category_constant;
                                    if (o.oneofs)
                                        d._mobile_app_category_constant = "mobile_app_category_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MobileAppCategoryInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MobileAppCategoryInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileAppCategoryInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MobileAppCategoryInfo;
                        })();
    
                        common.MobileApplicationInfo = (function() {
    
                            /**
                             * Properties of a MobileApplicationInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMobileApplicationInfo
                             * @property {string|null} [app_id] MobileApplicationInfo app_id
                             * @property {string|null} [name] MobileApplicationInfo name
                             */
    
                            /**
                             * Constructs a new MobileApplicationInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MobileApplicationInfo.
                             * @implements IMobileApplicationInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMobileApplicationInfo=} [p] Properties to set
                             */
                            function MobileApplicationInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MobileApplicationInfo app_id.
                             * @member {string|null|undefined} app_id
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @instance
                             */
                            MobileApplicationInfo.prototype.app_id = null;
    
                            /**
                             * MobileApplicationInfo name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @instance
                             */
                            MobileApplicationInfo.prototype.name = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MobileApplicationInfo _app_id.
                             * @member {"app_id"|undefined} _app_id
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @instance
                             */
                            Object.defineProperty(MobileApplicationInfo.prototype, "_app_id", {
                                get: $util.oneOfGetter($oneOfFields = ["app_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MobileApplicationInfo _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @instance
                             */
                            Object.defineProperty(MobileApplicationInfo.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MobileApplicationInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileApplicationInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MobileApplicationInfo} MobileApplicationInfo instance
                             */
                            MobileApplicationInfo.create = function create(properties) {
                                return new MobileApplicationInfo(properties);
                            };
    
                            /**
                             * Encodes the specified MobileApplicationInfo message. Does not implicitly {@link google.ads.googleads.v11.common.MobileApplicationInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileApplicationInfo} m MobileApplicationInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileApplicationInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                    w.uint32(34).string(m.app_id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(42).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileApplicationInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MobileApplicationInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileApplicationInfo} message MobileApplicationInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileApplicationInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileApplicationInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MobileApplicationInfo} MobileApplicationInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileApplicationInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MobileApplicationInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.app_id = r.string();
                                        break;
                                    case 5:
                                        m.name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileApplicationInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MobileApplicationInfo} MobileApplicationInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileApplicationInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileApplicationInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileApplicationInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    p._app_id = 1;
                                    if (!$util.isString(m.app_id))
                                        return "app_id: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MobileApplicationInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MobileApplicationInfo} MobileApplicationInfo
                             */
                            MobileApplicationInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MobileApplicationInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MobileApplicationInfo();
                                if (d.app_id != null) {
                                    m.app_id = String(d.app_id);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MobileApplicationInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.MobileApplicationInfo} m MobileApplicationInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileApplicationInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    d.app_id = m.app_id;
                                    if (o.oneofs)
                                        d._app_id = "app_id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MobileApplicationInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MobileApplicationInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileApplicationInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MobileApplicationInfo;
                        })();
    
                        common.LocationInfo = (function() {
    
                            /**
                             * Properties of a LocationInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILocationInfo
                             * @property {string|null} [geo_target_constant] LocationInfo geo_target_constant
                             */
    
                            /**
                             * Constructs a new LocationInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LocationInfo.
                             * @implements ILocationInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILocationInfo=} [p] Properties to set
                             */
                            function LocationInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LocationInfo geo_target_constant.
                             * @member {string|null|undefined} geo_target_constant
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @instance
                             */
                            LocationInfo.prototype.geo_target_constant = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LocationInfo _geo_target_constant.
                             * @member {"geo_target_constant"|undefined} _geo_target_constant
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @instance
                             */
                            Object.defineProperty(LocationInfo.prototype, "_geo_target_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LocationInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LocationInfo} LocationInfo instance
                             */
                            LocationInfo.create = function create(properties) {
                                return new LocationInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LocationInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LocationInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationInfo} m LocationInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.geo_target_constant != null && Object.hasOwnProperty.call(m, "geo_target_constant"))
                                    w.uint32(18).string(m.geo_target_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LocationInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationInfo} message LocationInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LocationInfo} LocationInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LocationInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.geo_target_constant = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LocationInfo} LocationInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    p._geo_target_constant = 1;
                                    if (!$util.isString(m.geo_target_constant))
                                        return "geo_target_constant: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LocationInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LocationInfo} LocationInfo
                             */
                            LocationInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LocationInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LocationInfo();
                                if (d.geo_target_constant != null) {
                                    m.geo_target_constant = String(d.geo_target_constant);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LocationInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LocationInfo} m LocationInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    d.geo_target_constant = m.geo_target_constant;
                                    if (o.oneofs)
                                        d._geo_target_constant = "geo_target_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LocationInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LocationInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LocationInfo;
                        })();
    
                        common.DeviceInfo = (function() {
    
                            /**
                             * Properties of a DeviceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDeviceInfo
                             * @property {google.ads.googleads.v11.enums.DeviceEnum.Device|null} [type] DeviceInfo type
                             */
    
                            /**
                             * Constructs a new DeviceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DeviceInfo.
                             * @implements IDeviceInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDeviceInfo=} [p] Properties to set
                             */
                            function DeviceInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DeviceInfo type.
                             * @member {google.ads.googleads.v11.enums.DeviceEnum.Device} type
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @instance
                             */
                            DeviceInfo.prototype.type = 0;
    
                            /**
                             * Creates a new DeviceInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDeviceInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DeviceInfo} DeviceInfo instance
                             */
                            DeviceInfo.create = function create(properties) {
                                return new DeviceInfo(properties);
                            };
    
                            /**
                             * Encodes the specified DeviceInfo message. Does not implicitly {@link google.ads.googleads.v11.common.DeviceInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDeviceInfo} m DeviceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DeviceInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DeviceInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DeviceInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IDeviceInfo} message DeviceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DeviceInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DeviceInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DeviceInfo} DeviceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DeviceInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DeviceInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DeviceInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DeviceInfo} DeviceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DeviceInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DeviceInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DeviceInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DeviceInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DeviceInfo} DeviceInfo
                             */
                            DeviceInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DeviceInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DeviceInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "TABLET":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "DESKTOP":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "CONNECTED_TV":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "OTHER":
                                case 5:
                                    m.type = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DeviceInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.DeviceInfo} m DeviceInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DeviceInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.DeviceEnum.Device[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DeviceInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DeviceInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DeviceInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DeviceInfo;
                        })();
    
                        common.PreferredContentInfo = (function() {
    
                            /**
                             * Properties of a PreferredContentInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPreferredContentInfo
                             * @property {google.ads.googleads.v11.enums.PreferredContentTypeEnum.PreferredContentType|null} [type] PreferredContentInfo type
                             */
    
                            /**
                             * Constructs a new PreferredContentInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PreferredContentInfo.
                             * @implements IPreferredContentInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPreferredContentInfo=} [p] Properties to set
                             */
                            function PreferredContentInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PreferredContentInfo type.
                             * @member {google.ads.googleads.v11.enums.PreferredContentTypeEnum.PreferredContentType} type
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @instance
                             */
                            PreferredContentInfo.prototype.type = 0;
    
                            /**
                             * Creates a new PreferredContentInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IPreferredContentInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PreferredContentInfo} PreferredContentInfo instance
                             */
                            PreferredContentInfo.create = function create(properties) {
                                return new PreferredContentInfo(properties);
                            };
    
                            /**
                             * Encodes the specified PreferredContentInfo message. Does not implicitly {@link google.ads.googleads.v11.common.PreferredContentInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IPreferredContentInfo} m PreferredContentInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PreferredContentInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(16).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PreferredContentInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PreferredContentInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IPreferredContentInfo} message PreferredContentInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PreferredContentInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PreferredContentInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PreferredContentInfo} PreferredContentInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PreferredContentInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PreferredContentInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PreferredContentInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PreferredContentInfo} PreferredContentInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PreferredContentInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PreferredContentInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PreferredContentInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 400:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PreferredContentInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PreferredContentInfo} PreferredContentInfo
                             */
                            PreferredContentInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PreferredContentInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PreferredContentInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "YOUTUBE_TOP_CONTENT":
                                case 400:
                                    m.type = 400;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PreferredContentInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.PreferredContentInfo} m PreferredContentInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PreferredContentInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.PreferredContentTypeEnum.PreferredContentType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PreferredContentInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PreferredContentInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PreferredContentInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PreferredContentInfo;
                        })();
    
                        common.ListingGroupInfo = (function() {
    
                            /**
                             * Properties of a ListingGroupInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IListingGroupInfo
                             * @property {google.ads.googleads.v11.enums.ListingGroupTypeEnum.ListingGroupType|null} [type] ListingGroupInfo type
                             * @property {google.ads.googleads.v11.common.IListingDimensionInfo|null} [case_value] ListingGroupInfo case_value
                             * @property {string|null} [parent_ad_group_criterion] ListingGroupInfo parent_ad_group_criterion
                             */
    
                            /**
                             * Constructs a new ListingGroupInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ListingGroupInfo.
                             * @implements IListingGroupInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IListingGroupInfo=} [p] Properties to set
                             */
                            function ListingGroupInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListingGroupInfo type.
                             * @member {google.ads.googleads.v11.enums.ListingGroupTypeEnum.ListingGroupType} type
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @instance
                             */
                            ListingGroupInfo.prototype.type = 0;
    
                            /**
                             * ListingGroupInfo case_value.
                             * @member {google.ads.googleads.v11.common.IListingDimensionInfo|null|undefined} case_value
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @instance
                             */
                            ListingGroupInfo.prototype.case_value = null;
    
                            /**
                             * ListingGroupInfo parent_ad_group_criterion.
                             * @member {string|null|undefined} parent_ad_group_criterion
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @instance
                             */
                            ListingGroupInfo.prototype.parent_ad_group_criterion = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ListingGroupInfo _parent_ad_group_criterion.
                             * @member {"parent_ad_group_criterion"|undefined} _parent_ad_group_criterion
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @instance
                             */
                            Object.defineProperty(ListingGroupInfo.prototype, "_parent_ad_group_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_ad_group_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ListingGroupInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingGroupInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ListingGroupInfo} ListingGroupInfo instance
                             */
                            ListingGroupInfo.create = function create(properties) {
                                return new ListingGroupInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ListingGroupInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingGroupInfo} m ListingGroupInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                if (m.case_value != null && Object.hasOwnProperty.call(m, "case_value"))
                                    $root.google.ads.googleads.v11.common.ListingDimensionInfo.encode(m.case_value, w.uint32(18).fork()).ldelim();
                                if (m.parent_ad_group_criterion != null && Object.hasOwnProperty.call(m, "parent_ad_group_criterion"))
                                    w.uint32(34).string(m.parent_ad_group_criterion);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ListingGroupInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingGroupInfo} message ListingGroupInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ListingGroupInfo} ListingGroupInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ListingGroupInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    case 2:
                                        m.case_value = $root.google.ads.googleads.v11.common.ListingDimensionInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.parent_ad_group_criterion = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ListingGroupInfo} ListingGroupInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.case_value != null && m.hasOwnProperty("case_value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ListingDimensionInfo.verify(m.case_value);
                                        if (e)
                                            return "case_value." + e;
                                    }
                                }
                                if (m.parent_ad_group_criterion != null && m.hasOwnProperty("parent_ad_group_criterion")) {
                                    p._parent_ad_group_criterion = 1;
                                    if (!$util.isString(m.parent_ad_group_criterion))
                                        return "parent_ad_group_criterion: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ListingGroupInfo} ListingGroupInfo
                             */
                            ListingGroupInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ListingGroupInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ListingGroupInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "SUBDIVISION":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "UNIT":
                                case 3:
                                    m.type = 3;
                                    break;
                                }
                                if (d.case_value != null) {
                                    if (typeof d.case_value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingGroupInfo.case_value: object expected");
                                    m.case_value = $root.google.ads.googleads.v11.common.ListingDimensionInfo.fromObject(d.case_value);
                                }
                                if (d.parent_ad_group_criterion != null) {
                                    m.parent_ad_group_criterion = String(d.parent_ad_group_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ListingGroupInfo} m ListingGroupInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.case_value = null;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupTypeEnum.ListingGroupType[m.type] : m.type;
                                }
                                if (m.case_value != null && m.hasOwnProperty("case_value")) {
                                    d.case_value = $root.google.ads.googleads.v11.common.ListingDimensionInfo.toObject(m.case_value, o);
                                }
                                if (m.parent_ad_group_criterion != null && m.hasOwnProperty("parent_ad_group_criterion")) {
                                    d.parent_ad_group_criterion = m.parent_ad_group_criterion;
                                    if (o.oneofs)
                                        d._parent_ad_group_criterion = "parent_ad_group_criterion";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListingGroupInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ListingGroupInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListingGroupInfo;
                        })();
    
                        common.ListingScopeInfo = (function() {
    
                            /**
                             * Properties of a ListingScopeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IListingScopeInfo
                             * @property {Array.<google.ads.googleads.v11.common.IListingDimensionInfo>|null} [dimensions] ListingScopeInfo dimensions
                             */
    
                            /**
                             * Constructs a new ListingScopeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ListingScopeInfo.
                             * @implements IListingScopeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IListingScopeInfo=} [p] Properties to set
                             */
                            function ListingScopeInfo(p) {
                                this.dimensions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListingScopeInfo dimensions.
                             * @member {Array.<google.ads.googleads.v11.common.IListingDimensionInfo>} dimensions
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @instance
                             */
                            ListingScopeInfo.prototype.dimensions = $util.emptyArray;
    
                            /**
                             * Creates a new ListingScopeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingScopeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ListingScopeInfo} ListingScopeInfo instance
                             */
                            ListingScopeInfo.create = function create(properties) {
                                return new ListingScopeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ListingScopeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ListingScopeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingScopeInfo} m ListingScopeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingScopeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.dimensions != null && m.dimensions.length) {
                                    for (var i = 0; i < m.dimensions.length; ++i)
                                        $root.google.ads.googleads.v11.common.ListingDimensionInfo.encode(m.dimensions[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingScopeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ListingScopeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingScopeInfo} message ListingScopeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingScopeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingScopeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ListingScopeInfo} ListingScopeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingScopeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ListingScopeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        if (!(m.dimensions && m.dimensions.length))
                                            m.dimensions = [];
                                        m.dimensions.push($root.google.ads.googleads.v11.common.ListingDimensionInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingScopeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ListingScopeInfo} ListingScopeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingScopeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingScopeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingScopeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.dimensions != null && m.hasOwnProperty("dimensions")) {
                                    if (!Array.isArray(m.dimensions))
                                        return "dimensions: array expected";
                                    for (var i = 0; i < m.dimensions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.ListingDimensionInfo.verify(m.dimensions[i]);
                                            if (e)
                                                return "dimensions." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListingScopeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ListingScopeInfo} ListingScopeInfo
                             */
                            ListingScopeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ListingScopeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ListingScopeInfo();
                                if (d.dimensions) {
                                    if (!Array.isArray(d.dimensions))
                                        throw TypeError(".google.ads.googleads.v11.common.ListingScopeInfo.dimensions: array expected");
                                    m.dimensions = [];
                                    for (var i = 0; i < d.dimensions.length; ++i) {
                                        if (typeof d.dimensions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.ListingScopeInfo.dimensions: object expected");
                                        m.dimensions[i] = $root.google.ads.googleads.v11.common.ListingDimensionInfo.fromObject(d.dimensions[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListingScopeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ListingScopeInfo} m ListingScopeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingScopeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.dimensions = [];
                                }
                                if (m.dimensions && m.dimensions.length) {
                                    d.dimensions = [];
                                    for (var j = 0; j < m.dimensions.length; ++j) {
                                        d.dimensions[j] = $root.google.ads.googleads.v11.common.ListingDimensionInfo.toObject(m.dimensions[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListingScopeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ListingScopeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingScopeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListingScopeInfo;
                        })();
    
                        common.ListingDimensionInfo = (function() {
    
                            /**
                             * Properties of a ListingDimensionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IListingDimensionInfo
                             * @property {google.ads.googleads.v11.common.IHotelIdInfo|null} [hotel_id] ListingDimensionInfo hotel_id
                             * @property {google.ads.googleads.v11.common.IHotelClassInfo|null} [hotel_class] ListingDimensionInfo hotel_class
                             * @property {google.ads.googleads.v11.common.IHotelCountryRegionInfo|null} [hotel_country_region] ListingDimensionInfo hotel_country_region
                             * @property {google.ads.googleads.v11.common.IHotelStateInfo|null} [hotel_state] ListingDimensionInfo hotel_state
                             * @property {google.ads.googleads.v11.common.IHotelCityInfo|null} [hotel_city] ListingDimensionInfo hotel_city
                             * @property {google.ads.googleads.v11.common.IProductBiddingCategoryInfo|null} [product_bidding_category] ListingDimensionInfo product_bidding_category
                             * @property {google.ads.googleads.v11.common.IProductBrandInfo|null} [product_brand] ListingDimensionInfo product_brand
                             * @property {google.ads.googleads.v11.common.IProductChannelInfo|null} [product_channel] ListingDimensionInfo product_channel
                             * @property {google.ads.googleads.v11.common.IProductChannelExclusivityInfo|null} [product_channel_exclusivity] ListingDimensionInfo product_channel_exclusivity
                             * @property {google.ads.googleads.v11.common.IProductConditionInfo|null} [product_condition] ListingDimensionInfo product_condition
                             * @property {google.ads.googleads.v11.common.IProductCustomAttributeInfo|null} [product_custom_attribute] ListingDimensionInfo product_custom_attribute
                             * @property {google.ads.googleads.v11.common.IProductItemIdInfo|null} [product_item_id] ListingDimensionInfo product_item_id
                             * @property {google.ads.googleads.v11.common.IProductTypeInfo|null} [product_type] ListingDimensionInfo product_type
                             * @property {google.ads.googleads.v11.common.IProductGroupingInfo|null} [product_grouping] ListingDimensionInfo product_grouping
                             * @property {google.ads.googleads.v11.common.IProductLabelsInfo|null} [product_labels] ListingDimensionInfo product_labels
                             * @property {google.ads.googleads.v11.common.IProductLegacyConditionInfo|null} [product_legacy_condition] ListingDimensionInfo product_legacy_condition
                             * @property {google.ads.googleads.v11.common.IProductTypeFullInfo|null} [product_type_full] ListingDimensionInfo product_type_full
                             * @property {google.ads.googleads.v11.common.IUnknownListingDimensionInfo|null} [unknown_listing_dimension] ListingDimensionInfo unknown_listing_dimension
                             */
    
                            /**
                             * Constructs a new ListingDimensionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ListingDimensionInfo.
                             * @implements IListingDimensionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IListingDimensionInfo=} [p] Properties to set
                             */
                            function ListingDimensionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListingDimensionInfo hotel_id.
                             * @member {google.ads.googleads.v11.common.IHotelIdInfo|null|undefined} hotel_id
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.hotel_id = null;
    
                            /**
                             * ListingDimensionInfo hotel_class.
                             * @member {google.ads.googleads.v11.common.IHotelClassInfo|null|undefined} hotel_class
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.hotel_class = null;
    
                            /**
                             * ListingDimensionInfo hotel_country_region.
                             * @member {google.ads.googleads.v11.common.IHotelCountryRegionInfo|null|undefined} hotel_country_region
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.hotel_country_region = null;
    
                            /**
                             * ListingDimensionInfo hotel_state.
                             * @member {google.ads.googleads.v11.common.IHotelStateInfo|null|undefined} hotel_state
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.hotel_state = null;
    
                            /**
                             * ListingDimensionInfo hotel_city.
                             * @member {google.ads.googleads.v11.common.IHotelCityInfo|null|undefined} hotel_city
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.hotel_city = null;
    
                            /**
                             * ListingDimensionInfo product_bidding_category.
                             * @member {google.ads.googleads.v11.common.IProductBiddingCategoryInfo|null|undefined} product_bidding_category
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_bidding_category = null;
    
                            /**
                             * ListingDimensionInfo product_brand.
                             * @member {google.ads.googleads.v11.common.IProductBrandInfo|null|undefined} product_brand
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_brand = null;
    
                            /**
                             * ListingDimensionInfo product_channel.
                             * @member {google.ads.googleads.v11.common.IProductChannelInfo|null|undefined} product_channel
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_channel = null;
    
                            /**
                             * ListingDimensionInfo product_channel_exclusivity.
                             * @member {google.ads.googleads.v11.common.IProductChannelExclusivityInfo|null|undefined} product_channel_exclusivity
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_channel_exclusivity = null;
    
                            /**
                             * ListingDimensionInfo product_condition.
                             * @member {google.ads.googleads.v11.common.IProductConditionInfo|null|undefined} product_condition
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_condition = null;
    
                            /**
                             * ListingDimensionInfo product_custom_attribute.
                             * @member {google.ads.googleads.v11.common.IProductCustomAttributeInfo|null|undefined} product_custom_attribute
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_custom_attribute = null;
    
                            /**
                             * ListingDimensionInfo product_item_id.
                             * @member {google.ads.googleads.v11.common.IProductItemIdInfo|null|undefined} product_item_id
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_item_id = null;
    
                            /**
                             * ListingDimensionInfo product_type.
                             * @member {google.ads.googleads.v11.common.IProductTypeInfo|null|undefined} product_type
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_type = null;
    
                            /**
                             * ListingDimensionInfo product_grouping.
                             * @member {google.ads.googleads.v11.common.IProductGroupingInfo|null|undefined} product_grouping
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_grouping = null;
    
                            /**
                             * ListingDimensionInfo product_labels.
                             * @member {google.ads.googleads.v11.common.IProductLabelsInfo|null|undefined} product_labels
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_labels = null;
    
                            /**
                             * ListingDimensionInfo product_legacy_condition.
                             * @member {google.ads.googleads.v11.common.IProductLegacyConditionInfo|null|undefined} product_legacy_condition
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_legacy_condition = null;
    
                            /**
                             * ListingDimensionInfo product_type_full.
                             * @member {google.ads.googleads.v11.common.IProductTypeFullInfo|null|undefined} product_type_full
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.product_type_full = null;
    
                            /**
                             * ListingDimensionInfo unknown_listing_dimension.
                             * @member {google.ads.googleads.v11.common.IUnknownListingDimensionInfo|null|undefined} unknown_listing_dimension
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            ListingDimensionInfo.prototype.unknown_listing_dimension = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ListingDimensionInfo dimension.
                             * @member {"hotel_id"|"hotel_class"|"hotel_country_region"|"hotel_state"|"hotel_city"|"product_bidding_category"|"product_brand"|"product_channel"|"product_channel_exclusivity"|"product_condition"|"product_custom_attribute"|"product_item_id"|"product_type"|"product_grouping"|"product_labels"|"product_legacy_condition"|"product_type_full"|"unknown_listing_dimension"|undefined} dimension
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             */
                            Object.defineProperty(ListingDimensionInfo.prototype, "dimension", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_id", "hotel_class", "hotel_country_region", "hotel_state", "hotel_city", "product_bidding_category", "product_brand", "product_channel", "product_channel_exclusivity", "product_condition", "product_custom_attribute", "product_item_id", "product_type", "product_grouping", "product_labels", "product_legacy_condition", "product_type_full", "unknown_listing_dimension"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ListingDimensionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingDimensionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ListingDimensionInfo} ListingDimensionInfo instance
                             */
                            ListingDimensionInfo.create = function create(properties) {
                                return new ListingDimensionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ListingDimensionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ListingDimensionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingDimensionInfo} m ListingDimensionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingDimensionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.hotel_id != null && Object.hasOwnProperty.call(m, "hotel_id"))
                                    $root.google.ads.googleads.v11.common.HotelIdInfo.encode(m.hotel_id, w.uint32(18).fork()).ldelim();
                                if (m.hotel_class != null && Object.hasOwnProperty.call(m, "hotel_class"))
                                    $root.google.ads.googleads.v11.common.HotelClassInfo.encode(m.hotel_class, w.uint32(26).fork()).ldelim();
                                if (m.hotel_country_region != null && Object.hasOwnProperty.call(m, "hotel_country_region"))
                                    $root.google.ads.googleads.v11.common.HotelCountryRegionInfo.encode(m.hotel_country_region, w.uint32(34).fork()).ldelim();
                                if (m.hotel_state != null && Object.hasOwnProperty.call(m, "hotel_state"))
                                    $root.google.ads.googleads.v11.common.HotelStateInfo.encode(m.hotel_state, w.uint32(42).fork()).ldelim();
                                if (m.hotel_city != null && Object.hasOwnProperty.call(m, "hotel_city"))
                                    $root.google.ads.googleads.v11.common.HotelCityInfo.encode(m.hotel_city, w.uint32(50).fork()).ldelim();
                                if (m.product_channel != null && Object.hasOwnProperty.call(m, "product_channel"))
                                    $root.google.ads.googleads.v11.common.ProductChannelInfo.encode(m.product_channel, w.uint32(66).fork()).ldelim();
                                if (m.product_channel_exclusivity != null && Object.hasOwnProperty.call(m, "product_channel_exclusivity"))
                                    $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo.encode(m.product_channel_exclusivity, w.uint32(74).fork()).ldelim();
                                if (m.product_condition != null && Object.hasOwnProperty.call(m, "product_condition"))
                                    $root.google.ads.googleads.v11.common.ProductConditionInfo.encode(m.product_condition, w.uint32(82).fork()).ldelim();
                                if (m.product_item_id != null && Object.hasOwnProperty.call(m, "product_item_id"))
                                    $root.google.ads.googleads.v11.common.ProductItemIdInfo.encode(m.product_item_id, w.uint32(90).fork()).ldelim();
                                if (m.product_type != null && Object.hasOwnProperty.call(m, "product_type"))
                                    $root.google.ads.googleads.v11.common.ProductTypeInfo.encode(m.product_type, w.uint32(98).fork()).ldelim();
                                if (m.product_bidding_category != null && Object.hasOwnProperty.call(m, "product_bidding_category"))
                                    $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo.encode(m.product_bidding_category, w.uint32(106).fork()).ldelim();
                                if (m.unknown_listing_dimension != null && Object.hasOwnProperty.call(m, "unknown_listing_dimension"))
                                    $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo.encode(m.unknown_listing_dimension, w.uint32(114).fork()).ldelim();
                                if (m.product_brand != null && Object.hasOwnProperty.call(m, "product_brand"))
                                    $root.google.ads.googleads.v11.common.ProductBrandInfo.encode(m.product_brand, w.uint32(122).fork()).ldelim();
                                if (m.product_custom_attribute != null && Object.hasOwnProperty.call(m, "product_custom_attribute"))
                                    $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo.encode(m.product_custom_attribute, w.uint32(130).fork()).ldelim();
                                if (m.product_grouping != null && Object.hasOwnProperty.call(m, "product_grouping"))
                                    $root.google.ads.googleads.v11.common.ProductGroupingInfo.encode(m.product_grouping, w.uint32(138).fork()).ldelim();
                                if (m.product_labels != null && Object.hasOwnProperty.call(m, "product_labels"))
                                    $root.google.ads.googleads.v11.common.ProductLabelsInfo.encode(m.product_labels, w.uint32(146).fork()).ldelim();
                                if (m.product_legacy_condition != null && Object.hasOwnProperty.call(m, "product_legacy_condition"))
                                    $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo.encode(m.product_legacy_condition, w.uint32(154).fork()).ldelim();
                                if (m.product_type_full != null && Object.hasOwnProperty.call(m, "product_type_full"))
                                    $root.google.ads.googleads.v11.common.ProductTypeFullInfo.encode(m.product_type_full, w.uint32(162).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingDimensionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ListingDimensionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IListingDimensionInfo} message ListingDimensionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingDimensionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingDimensionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ListingDimensionInfo} ListingDimensionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingDimensionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ListingDimensionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.hotel_id = $root.google.ads.googleads.v11.common.HotelIdInfo.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.hotel_class = $root.google.ads.googleads.v11.common.HotelClassInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.hotel_country_region = $root.google.ads.googleads.v11.common.HotelCountryRegionInfo.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.hotel_state = $root.google.ads.googleads.v11.common.HotelStateInfo.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.hotel_city = $root.google.ads.googleads.v11.common.HotelCityInfo.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.product_bidding_category = $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.product_brand = $root.google.ads.googleads.v11.common.ProductBrandInfo.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.product_channel = $root.google.ads.googleads.v11.common.ProductChannelInfo.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.product_channel_exclusivity = $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.product_condition = $root.google.ads.googleads.v11.common.ProductConditionInfo.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.product_custom_attribute = $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.product_item_id = $root.google.ads.googleads.v11.common.ProductItemIdInfo.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.product_type = $root.google.ads.googleads.v11.common.ProductTypeInfo.decode(r, r.uint32());
                                        break;
                                    case 17:
                                        m.product_grouping = $root.google.ads.googleads.v11.common.ProductGroupingInfo.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.product_labels = $root.google.ads.googleads.v11.common.ProductLabelsInfo.decode(r, r.uint32());
                                        break;
                                    case 19:
                                        m.product_legacy_condition = $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.product_type_full = $root.google.ads.googleads.v11.common.ProductTypeFullInfo.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.unknown_listing_dimension = $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingDimensionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ListingDimensionInfo} ListingDimensionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingDimensionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingDimensionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingDimensionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.hotel_id != null && m.hasOwnProperty("hotel_id")) {
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelIdInfo.verify(m.hotel_id);
                                        if (e)
                                            return "hotel_id." + e;
                                    }
                                }
                                if (m.hotel_class != null && m.hasOwnProperty("hotel_class")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelClassInfo.verify(m.hotel_class);
                                        if (e)
                                            return "hotel_class." + e;
                                    }
                                }
                                if (m.hotel_country_region != null && m.hasOwnProperty("hotel_country_region")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelCountryRegionInfo.verify(m.hotel_country_region);
                                        if (e)
                                            return "hotel_country_region." + e;
                                    }
                                }
                                if (m.hotel_state != null && m.hasOwnProperty("hotel_state")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelStateInfo.verify(m.hotel_state);
                                        if (e)
                                            return "hotel_state." + e;
                                    }
                                }
                                if (m.hotel_city != null && m.hasOwnProperty("hotel_city")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelCityInfo.verify(m.hotel_city);
                                        if (e)
                                            return "hotel_city." + e;
                                    }
                                }
                                if (m.product_bidding_category != null && m.hasOwnProperty("product_bidding_category")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo.verify(m.product_bidding_category);
                                        if (e)
                                            return "product_bidding_category." + e;
                                    }
                                }
                                if (m.product_brand != null && m.hasOwnProperty("product_brand")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductBrandInfo.verify(m.product_brand);
                                        if (e)
                                            return "product_brand." + e;
                                    }
                                }
                                if (m.product_channel != null && m.hasOwnProperty("product_channel")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductChannelInfo.verify(m.product_channel);
                                        if (e)
                                            return "product_channel." + e;
                                    }
                                }
                                if (m.product_channel_exclusivity != null && m.hasOwnProperty("product_channel_exclusivity")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo.verify(m.product_channel_exclusivity);
                                        if (e)
                                            return "product_channel_exclusivity." + e;
                                    }
                                }
                                if (m.product_condition != null && m.hasOwnProperty("product_condition")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductConditionInfo.verify(m.product_condition);
                                        if (e)
                                            return "product_condition." + e;
                                    }
                                }
                                if (m.product_custom_attribute != null && m.hasOwnProperty("product_custom_attribute")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo.verify(m.product_custom_attribute);
                                        if (e)
                                            return "product_custom_attribute." + e;
                                    }
                                }
                                if (m.product_item_id != null && m.hasOwnProperty("product_item_id")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductItemIdInfo.verify(m.product_item_id);
                                        if (e)
                                            return "product_item_id." + e;
                                    }
                                }
                                if (m.product_type != null && m.hasOwnProperty("product_type")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductTypeInfo.verify(m.product_type);
                                        if (e)
                                            return "product_type." + e;
                                    }
                                }
                                if (m.product_grouping != null && m.hasOwnProperty("product_grouping")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductGroupingInfo.verify(m.product_grouping);
                                        if (e)
                                            return "product_grouping." + e;
                                    }
                                }
                                if (m.product_labels != null && m.hasOwnProperty("product_labels")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductLabelsInfo.verify(m.product_labels);
                                        if (e)
                                            return "product_labels." + e;
                                    }
                                }
                                if (m.product_legacy_condition != null && m.hasOwnProperty("product_legacy_condition")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo.verify(m.product_legacy_condition);
                                        if (e)
                                            return "product_legacy_condition." + e;
                                    }
                                }
                                if (m.product_type_full != null && m.hasOwnProperty("product_type_full")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProductTypeFullInfo.verify(m.product_type_full);
                                        if (e)
                                            return "product_type_full." + e;
                                    }
                                }
                                if (m.unknown_listing_dimension != null && m.hasOwnProperty("unknown_listing_dimension")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo.verify(m.unknown_listing_dimension);
                                        if (e)
                                            return "unknown_listing_dimension." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListingDimensionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ListingDimensionInfo} ListingDimensionInfo
                             */
                            ListingDimensionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ListingDimensionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ListingDimensionInfo();
                                if (d.hotel_id != null) {
                                    if (typeof d.hotel_id !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.hotel_id: object expected");
                                    m.hotel_id = $root.google.ads.googleads.v11.common.HotelIdInfo.fromObject(d.hotel_id);
                                }
                                if (d.hotel_class != null) {
                                    if (typeof d.hotel_class !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.hotel_class: object expected");
                                    m.hotel_class = $root.google.ads.googleads.v11.common.HotelClassInfo.fromObject(d.hotel_class);
                                }
                                if (d.hotel_country_region != null) {
                                    if (typeof d.hotel_country_region !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.hotel_country_region: object expected");
                                    m.hotel_country_region = $root.google.ads.googleads.v11.common.HotelCountryRegionInfo.fromObject(d.hotel_country_region);
                                }
                                if (d.hotel_state != null) {
                                    if (typeof d.hotel_state !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.hotel_state: object expected");
                                    m.hotel_state = $root.google.ads.googleads.v11.common.HotelStateInfo.fromObject(d.hotel_state);
                                }
                                if (d.hotel_city != null) {
                                    if (typeof d.hotel_city !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.hotel_city: object expected");
                                    m.hotel_city = $root.google.ads.googleads.v11.common.HotelCityInfo.fromObject(d.hotel_city);
                                }
                                if (d.product_bidding_category != null) {
                                    if (typeof d.product_bidding_category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_bidding_category: object expected");
                                    m.product_bidding_category = $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo.fromObject(d.product_bidding_category);
                                }
                                if (d.product_brand != null) {
                                    if (typeof d.product_brand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_brand: object expected");
                                    m.product_brand = $root.google.ads.googleads.v11.common.ProductBrandInfo.fromObject(d.product_brand);
                                }
                                if (d.product_channel != null) {
                                    if (typeof d.product_channel !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_channel: object expected");
                                    m.product_channel = $root.google.ads.googleads.v11.common.ProductChannelInfo.fromObject(d.product_channel);
                                }
                                if (d.product_channel_exclusivity != null) {
                                    if (typeof d.product_channel_exclusivity !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_channel_exclusivity: object expected");
                                    m.product_channel_exclusivity = $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo.fromObject(d.product_channel_exclusivity);
                                }
                                if (d.product_condition != null) {
                                    if (typeof d.product_condition !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_condition: object expected");
                                    m.product_condition = $root.google.ads.googleads.v11.common.ProductConditionInfo.fromObject(d.product_condition);
                                }
                                if (d.product_custom_attribute != null) {
                                    if (typeof d.product_custom_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_custom_attribute: object expected");
                                    m.product_custom_attribute = $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo.fromObject(d.product_custom_attribute);
                                }
                                if (d.product_item_id != null) {
                                    if (typeof d.product_item_id !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_item_id: object expected");
                                    m.product_item_id = $root.google.ads.googleads.v11.common.ProductItemIdInfo.fromObject(d.product_item_id);
                                }
                                if (d.product_type != null) {
                                    if (typeof d.product_type !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_type: object expected");
                                    m.product_type = $root.google.ads.googleads.v11.common.ProductTypeInfo.fromObject(d.product_type);
                                }
                                if (d.product_grouping != null) {
                                    if (typeof d.product_grouping !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_grouping: object expected");
                                    m.product_grouping = $root.google.ads.googleads.v11.common.ProductGroupingInfo.fromObject(d.product_grouping);
                                }
                                if (d.product_labels != null) {
                                    if (typeof d.product_labels !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_labels: object expected");
                                    m.product_labels = $root.google.ads.googleads.v11.common.ProductLabelsInfo.fromObject(d.product_labels);
                                }
                                if (d.product_legacy_condition != null) {
                                    if (typeof d.product_legacy_condition !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_legacy_condition: object expected");
                                    m.product_legacy_condition = $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo.fromObject(d.product_legacy_condition);
                                }
                                if (d.product_type_full != null) {
                                    if (typeof d.product_type_full !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.product_type_full: object expected");
                                    m.product_type_full = $root.google.ads.googleads.v11.common.ProductTypeFullInfo.fromObject(d.product_type_full);
                                }
                                if (d.unknown_listing_dimension != null) {
                                    if (typeof d.unknown_listing_dimension !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ListingDimensionInfo.unknown_listing_dimension: object expected");
                                    m.unknown_listing_dimension = $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo.fromObject(d.unknown_listing_dimension);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListingDimensionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ListingDimensionInfo} m ListingDimensionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingDimensionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.hotel_id != null && m.hasOwnProperty("hotel_id")) {
                                    d.hotel_id = $root.google.ads.googleads.v11.common.HotelIdInfo.toObject(m.hotel_id, o);
                                    if (o.oneofs)
                                        d.dimension = "hotel_id";
                                }
                                if (m.hotel_class != null && m.hasOwnProperty("hotel_class")) {
                                    d.hotel_class = $root.google.ads.googleads.v11.common.HotelClassInfo.toObject(m.hotel_class, o);
                                    if (o.oneofs)
                                        d.dimension = "hotel_class";
                                }
                                if (m.hotel_country_region != null && m.hasOwnProperty("hotel_country_region")) {
                                    d.hotel_country_region = $root.google.ads.googleads.v11.common.HotelCountryRegionInfo.toObject(m.hotel_country_region, o);
                                    if (o.oneofs)
                                        d.dimension = "hotel_country_region";
                                }
                                if (m.hotel_state != null && m.hasOwnProperty("hotel_state")) {
                                    d.hotel_state = $root.google.ads.googleads.v11.common.HotelStateInfo.toObject(m.hotel_state, o);
                                    if (o.oneofs)
                                        d.dimension = "hotel_state";
                                }
                                if (m.hotel_city != null && m.hasOwnProperty("hotel_city")) {
                                    d.hotel_city = $root.google.ads.googleads.v11.common.HotelCityInfo.toObject(m.hotel_city, o);
                                    if (o.oneofs)
                                        d.dimension = "hotel_city";
                                }
                                if (m.product_channel != null && m.hasOwnProperty("product_channel")) {
                                    d.product_channel = $root.google.ads.googleads.v11.common.ProductChannelInfo.toObject(m.product_channel, o);
                                    if (o.oneofs)
                                        d.dimension = "product_channel";
                                }
                                if (m.product_channel_exclusivity != null && m.hasOwnProperty("product_channel_exclusivity")) {
                                    d.product_channel_exclusivity = $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo.toObject(m.product_channel_exclusivity, o);
                                    if (o.oneofs)
                                        d.dimension = "product_channel_exclusivity";
                                }
                                if (m.product_condition != null && m.hasOwnProperty("product_condition")) {
                                    d.product_condition = $root.google.ads.googleads.v11.common.ProductConditionInfo.toObject(m.product_condition, o);
                                    if (o.oneofs)
                                        d.dimension = "product_condition";
                                }
                                if (m.product_item_id != null && m.hasOwnProperty("product_item_id")) {
                                    d.product_item_id = $root.google.ads.googleads.v11.common.ProductItemIdInfo.toObject(m.product_item_id, o);
                                    if (o.oneofs)
                                        d.dimension = "product_item_id";
                                }
                                if (m.product_type != null && m.hasOwnProperty("product_type")) {
                                    d.product_type = $root.google.ads.googleads.v11.common.ProductTypeInfo.toObject(m.product_type, o);
                                    if (o.oneofs)
                                        d.dimension = "product_type";
                                }
                                if (m.product_bidding_category != null && m.hasOwnProperty("product_bidding_category")) {
                                    d.product_bidding_category = $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo.toObject(m.product_bidding_category, o);
                                    if (o.oneofs)
                                        d.dimension = "product_bidding_category";
                                }
                                if (m.unknown_listing_dimension != null && m.hasOwnProperty("unknown_listing_dimension")) {
                                    d.unknown_listing_dimension = $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo.toObject(m.unknown_listing_dimension, o);
                                    if (o.oneofs)
                                        d.dimension = "unknown_listing_dimension";
                                }
                                if (m.product_brand != null && m.hasOwnProperty("product_brand")) {
                                    d.product_brand = $root.google.ads.googleads.v11.common.ProductBrandInfo.toObject(m.product_brand, o);
                                    if (o.oneofs)
                                        d.dimension = "product_brand";
                                }
                                if (m.product_custom_attribute != null && m.hasOwnProperty("product_custom_attribute")) {
                                    d.product_custom_attribute = $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo.toObject(m.product_custom_attribute, o);
                                    if (o.oneofs)
                                        d.dimension = "product_custom_attribute";
                                }
                                if (m.product_grouping != null && m.hasOwnProperty("product_grouping")) {
                                    d.product_grouping = $root.google.ads.googleads.v11.common.ProductGroupingInfo.toObject(m.product_grouping, o);
                                    if (o.oneofs)
                                        d.dimension = "product_grouping";
                                }
                                if (m.product_labels != null && m.hasOwnProperty("product_labels")) {
                                    d.product_labels = $root.google.ads.googleads.v11.common.ProductLabelsInfo.toObject(m.product_labels, o);
                                    if (o.oneofs)
                                        d.dimension = "product_labels";
                                }
                                if (m.product_legacy_condition != null && m.hasOwnProperty("product_legacy_condition")) {
                                    d.product_legacy_condition = $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo.toObject(m.product_legacy_condition, o);
                                    if (o.oneofs)
                                        d.dimension = "product_legacy_condition";
                                }
                                if (m.product_type_full != null && m.hasOwnProperty("product_type_full")) {
                                    d.product_type_full = $root.google.ads.googleads.v11.common.ProductTypeFullInfo.toObject(m.product_type_full, o);
                                    if (o.oneofs)
                                        d.dimension = "product_type_full";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListingDimensionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ListingDimensionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingDimensionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListingDimensionInfo;
                        })();
    
                        common.HotelIdInfo = (function() {
    
                            /**
                             * Properties of a HotelIdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelIdInfo
                             * @property {string|null} [value] HotelIdInfo value
                             */
    
                            /**
                             * Constructs a new HotelIdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelIdInfo.
                             * @implements IHotelIdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelIdInfo=} [p] Properties to set
                             */
                            function HotelIdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelIdInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @instance
                             */
                            HotelIdInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelIdInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @instance
                             */
                            Object.defineProperty(HotelIdInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelIdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelIdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelIdInfo} HotelIdInfo instance
                             */
                            HotelIdInfo.create = function create(properties) {
                                return new HotelIdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelIdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelIdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelIdInfo} m HotelIdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelIdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(18).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelIdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelIdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelIdInfo} message HotelIdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelIdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelIdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelIdInfo} HotelIdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelIdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelIdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelIdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelIdInfo} HotelIdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelIdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelIdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelIdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelIdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelIdInfo} HotelIdInfo
                             */
                            HotelIdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelIdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelIdInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelIdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelIdInfo} m HotelIdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelIdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelIdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelIdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelIdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelIdInfo;
                        })();
    
                        common.HotelClassInfo = (function() {
    
                            /**
                             * Properties of a HotelClassInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelClassInfo
                             * @property {number|null} [value] HotelClassInfo value
                             */
    
                            /**
                             * Constructs a new HotelClassInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelClassInfo.
                             * @implements IHotelClassInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelClassInfo=} [p] Properties to set
                             */
                            function HotelClassInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelClassInfo value.
                             * @member {number|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @instance
                             */
                            HotelClassInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelClassInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @instance
                             */
                            Object.defineProperty(HotelClassInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelClassInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelClassInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelClassInfo} HotelClassInfo instance
                             */
                            HotelClassInfo.create = function create(properties) {
                                return new HotelClassInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelClassInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelClassInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelClassInfo} m HotelClassInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelClassInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(16).int64(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelClassInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelClassInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelClassInfo} message HotelClassInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelClassInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelClassInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelClassInfo} HotelClassInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelClassInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelClassInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.value = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelClassInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelClassInfo} HotelClassInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelClassInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelClassInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelClassInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isInteger(m.value) && !(m.value && $util.isInteger(m.value.low) && $util.isInteger(m.value.high)))
                                        return "value: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelClassInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelClassInfo} HotelClassInfo
                             */
                            HotelClassInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelClassInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelClassInfo();
                                if (d.value != null) {
                                    if ($util.Long)
                                        (m.value = $util.Long.fromValue(d.value)).unsigned = false;
                                    else if (typeof d.value === "string")
                                        m.value = parseInt(d.value, 10);
                                    else if (typeof d.value === "number")
                                        m.value = d.value;
                                    else if (typeof d.value === "object")
                                        m.value = new $util.LongBits(d.value.low >>> 0, d.value.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelClassInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelClassInfo} m HotelClassInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelClassInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    if (typeof m.value === "number")
                                        d.value = o.longs === String ? String(m.value) : m.value;
                                    else
                                        d.value = o.longs === String ? $util.Long.prototype.toString.call(m.value) : o.longs === Number ? new $util.LongBits(m.value.low >>> 0, m.value.high >>> 0).toNumber() : m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelClassInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelClassInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelClassInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelClassInfo;
                        })();
    
                        common.HotelCountryRegionInfo = (function() {
    
                            /**
                             * Properties of a HotelCountryRegionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelCountryRegionInfo
                             * @property {string|null} [country_region_criterion] HotelCountryRegionInfo country_region_criterion
                             */
    
                            /**
                             * Constructs a new HotelCountryRegionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelCountryRegionInfo.
                             * @implements IHotelCountryRegionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelCountryRegionInfo=} [p] Properties to set
                             */
                            function HotelCountryRegionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelCountryRegionInfo country_region_criterion.
                             * @member {string|null|undefined} country_region_criterion
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @instance
                             */
                            HotelCountryRegionInfo.prototype.country_region_criterion = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelCountryRegionInfo _country_region_criterion.
                             * @member {"country_region_criterion"|undefined} _country_region_criterion
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @instance
                             */
                            Object.defineProperty(HotelCountryRegionInfo.prototype, "_country_region_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["country_region_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelCountryRegionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCountryRegionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelCountryRegionInfo} HotelCountryRegionInfo instance
                             */
                            HotelCountryRegionInfo.create = function create(properties) {
                                return new HotelCountryRegionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelCountryRegionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelCountryRegionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCountryRegionInfo} m HotelCountryRegionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCountryRegionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.country_region_criterion != null && Object.hasOwnProperty.call(m, "country_region_criterion"))
                                    w.uint32(18).string(m.country_region_criterion);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelCountryRegionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelCountryRegionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCountryRegionInfo} message HotelCountryRegionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCountryRegionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelCountryRegionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelCountryRegionInfo} HotelCountryRegionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCountryRegionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelCountryRegionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.country_region_criterion = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelCountryRegionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelCountryRegionInfo} HotelCountryRegionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCountryRegionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelCountryRegionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelCountryRegionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.country_region_criterion != null && m.hasOwnProperty("country_region_criterion")) {
                                    p._country_region_criterion = 1;
                                    if (!$util.isString(m.country_region_criterion))
                                        return "country_region_criterion: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelCountryRegionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelCountryRegionInfo} HotelCountryRegionInfo
                             */
                            HotelCountryRegionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelCountryRegionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelCountryRegionInfo();
                                if (d.country_region_criterion != null) {
                                    m.country_region_criterion = String(d.country_region_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelCountryRegionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelCountryRegionInfo} m HotelCountryRegionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelCountryRegionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.country_region_criterion != null && m.hasOwnProperty("country_region_criterion")) {
                                    d.country_region_criterion = m.country_region_criterion;
                                    if (o.oneofs)
                                        d._country_region_criterion = "country_region_criterion";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelCountryRegionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelCountryRegionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelCountryRegionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelCountryRegionInfo;
                        })();
    
                        common.HotelStateInfo = (function() {
    
                            /**
                             * Properties of a HotelStateInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelStateInfo
                             * @property {string|null} [state_criterion] HotelStateInfo state_criterion
                             */
    
                            /**
                             * Constructs a new HotelStateInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelStateInfo.
                             * @implements IHotelStateInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelStateInfo=} [p] Properties to set
                             */
                            function HotelStateInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelStateInfo state_criterion.
                             * @member {string|null|undefined} state_criterion
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @instance
                             */
                            HotelStateInfo.prototype.state_criterion = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelStateInfo _state_criterion.
                             * @member {"state_criterion"|undefined} _state_criterion
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @instance
                             */
                            Object.defineProperty(HotelStateInfo.prototype, "_state_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["state_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelStateInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelStateInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelStateInfo} HotelStateInfo instance
                             */
                            HotelStateInfo.create = function create(properties) {
                                return new HotelStateInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelStateInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelStateInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelStateInfo} m HotelStateInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelStateInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.state_criterion != null && Object.hasOwnProperty.call(m, "state_criterion"))
                                    w.uint32(18).string(m.state_criterion);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelStateInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelStateInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelStateInfo} message HotelStateInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelStateInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelStateInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelStateInfo} HotelStateInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelStateInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelStateInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.state_criterion = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelStateInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelStateInfo} HotelStateInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelStateInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelStateInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelStateInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.state_criterion != null && m.hasOwnProperty("state_criterion")) {
                                    p._state_criterion = 1;
                                    if (!$util.isString(m.state_criterion))
                                        return "state_criterion: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelStateInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelStateInfo} HotelStateInfo
                             */
                            HotelStateInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelStateInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelStateInfo();
                                if (d.state_criterion != null) {
                                    m.state_criterion = String(d.state_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelStateInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelStateInfo} m HotelStateInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelStateInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.state_criterion != null && m.hasOwnProperty("state_criterion")) {
                                    d.state_criterion = m.state_criterion;
                                    if (o.oneofs)
                                        d._state_criterion = "state_criterion";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelStateInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelStateInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelStateInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelStateInfo;
                        })();
    
                        common.HotelCityInfo = (function() {
    
                            /**
                             * Properties of a HotelCityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelCityInfo
                             * @property {string|null} [city_criterion] HotelCityInfo city_criterion
                             */
    
                            /**
                             * Constructs a new HotelCityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelCityInfo.
                             * @implements IHotelCityInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelCityInfo=} [p] Properties to set
                             */
                            function HotelCityInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelCityInfo city_criterion.
                             * @member {string|null|undefined} city_criterion
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @instance
                             */
                            HotelCityInfo.prototype.city_criterion = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelCityInfo _city_criterion.
                             * @member {"city_criterion"|undefined} _city_criterion
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @instance
                             */
                            Object.defineProperty(HotelCityInfo.prototype, "_city_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["city_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelCityInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCityInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelCityInfo} HotelCityInfo instance
                             */
                            HotelCityInfo.create = function create(properties) {
                                return new HotelCityInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelCityInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelCityInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCityInfo} m HotelCityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCityInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.city_criterion != null && Object.hasOwnProperty.call(m, "city_criterion"))
                                    w.uint32(18).string(m.city_criterion);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelCityInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelCityInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCityInfo} message HotelCityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCityInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelCityInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelCityInfo} HotelCityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCityInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelCityInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.city_criterion = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelCityInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelCityInfo} HotelCityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCityInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelCityInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelCityInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.city_criterion != null && m.hasOwnProperty("city_criterion")) {
                                    p._city_criterion = 1;
                                    if (!$util.isString(m.city_criterion))
                                        return "city_criterion: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelCityInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelCityInfo} HotelCityInfo
                             */
                            HotelCityInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelCityInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelCityInfo();
                                if (d.city_criterion != null) {
                                    m.city_criterion = String(d.city_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelCityInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelCityInfo} m HotelCityInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelCityInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.city_criterion != null && m.hasOwnProperty("city_criterion")) {
                                    d.city_criterion = m.city_criterion;
                                    if (o.oneofs)
                                        d._city_criterion = "city_criterion";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelCityInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelCityInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelCityInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelCityInfo;
                        })();
    
                        common.ProductBiddingCategoryInfo = (function() {
    
                            /**
                             * Properties of a ProductBiddingCategoryInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductBiddingCategoryInfo
                             * @property {number|null} [id] ProductBiddingCategoryInfo id
                             * @property {string|null} [country_code] ProductBiddingCategoryInfo country_code
                             * @property {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel|null} [level] ProductBiddingCategoryInfo level
                             */
    
                            /**
                             * Constructs a new ProductBiddingCategoryInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductBiddingCategoryInfo.
                             * @implements IProductBiddingCategoryInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductBiddingCategoryInfo=} [p] Properties to set
                             */
                            function ProductBiddingCategoryInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductBiddingCategoryInfo id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @instance
                             */
                            ProductBiddingCategoryInfo.prototype.id = null;
    
                            /**
                             * ProductBiddingCategoryInfo country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @instance
                             */
                            ProductBiddingCategoryInfo.prototype.country_code = null;
    
                            /**
                             * ProductBiddingCategoryInfo level.
                             * @member {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel} level
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @instance
                             */
                            ProductBiddingCategoryInfo.prototype.level = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductBiddingCategoryInfo _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @instance
                             */
                            Object.defineProperty(ProductBiddingCategoryInfo.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ProductBiddingCategoryInfo _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @instance
                             */
                            Object.defineProperty(ProductBiddingCategoryInfo.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductBiddingCategoryInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductBiddingCategoryInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductBiddingCategoryInfo} ProductBiddingCategoryInfo instance
                             */
                            ProductBiddingCategoryInfo.create = function create(properties) {
                                return new ProductBiddingCategoryInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductBiddingCategoryInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductBiddingCategoryInfo} m ProductBiddingCategoryInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.level != null && Object.hasOwnProperty.call(m, "level"))
                                    w.uint32(24).int32(m.level);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(32).int64(m.id);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(42).string(m.country_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductBiddingCategoryInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductBiddingCategoryInfo} message ProductBiddingCategoryInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductBiddingCategoryInfo} ProductBiddingCategoryInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.id = r.int64();
                                        break;
                                    case 5:
                                        m.country_code = r.string();
                                        break;
                                    case 3:
                                        m.level = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductBiddingCategoryInfo} ProductBiddingCategoryInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductBiddingCategoryInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductBiddingCategoryInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    switch (m.level) {
                                    default:
                                        return "level: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductBiddingCategoryInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductBiddingCategoryInfo} ProductBiddingCategoryInfo
                             */
                            ProductBiddingCategoryInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductBiddingCategoryInfo();
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                switch (d.level) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.level = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.level = 1;
                                    break;
                                case "LEVEL1":
                                case 2:
                                    m.level = 2;
                                    break;
                                case "LEVEL2":
                                case 3:
                                    m.level = 3;
                                    break;
                                case "LEVEL3":
                                case 4:
                                    m.level = 4;
                                    break;
                                case "LEVEL4":
                                case 5:
                                    m.level = 5;
                                    break;
                                case "LEVEL5":
                                case 6:
                                    m.level = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductBiddingCategoryInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductBiddingCategoryInfo} m ProductBiddingCategoryInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductBiddingCategoryInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.level = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    d.level = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel[m.level] : m.level;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductBiddingCategoryInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductBiddingCategoryInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductBiddingCategoryInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductBiddingCategoryInfo;
                        })();
    
                        common.ProductBrandInfo = (function() {
    
                            /**
                             * Properties of a ProductBrandInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductBrandInfo
                             * @property {string|null} [value] ProductBrandInfo value
                             */
    
                            /**
                             * Constructs a new ProductBrandInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductBrandInfo.
                             * @implements IProductBrandInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductBrandInfo=} [p] Properties to set
                             */
                            function ProductBrandInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductBrandInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @instance
                             */
                            ProductBrandInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductBrandInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @instance
                             */
                            Object.defineProperty(ProductBrandInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductBrandInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductBrandInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductBrandInfo} ProductBrandInfo instance
                             */
                            ProductBrandInfo.create = function create(properties) {
                                return new ProductBrandInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductBrandInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductBrandInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductBrandInfo} m ProductBrandInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBrandInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(18).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductBrandInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductBrandInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductBrandInfo} message ProductBrandInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBrandInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductBrandInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductBrandInfo} ProductBrandInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBrandInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductBrandInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductBrandInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductBrandInfo} ProductBrandInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBrandInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductBrandInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductBrandInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductBrandInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductBrandInfo} ProductBrandInfo
                             */
                            ProductBrandInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductBrandInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductBrandInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductBrandInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductBrandInfo} m ProductBrandInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductBrandInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductBrandInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductBrandInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductBrandInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductBrandInfo;
                        })();
    
                        common.ProductChannelInfo = (function() {
    
                            /**
                             * Properties of a ProductChannelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductChannelInfo
                             * @property {google.ads.googleads.v11.enums.ProductChannelEnum.ProductChannel|null} [channel] ProductChannelInfo channel
                             */
    
                            /**
                             * Constructs a new ProductChannelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductChannelInfo.
                             * @implements IProductChannelInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductChannelInfo=} [p] Properties to set
                             */
                            function ProductChannelInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductChannelInfo channel.
                             * @member {google.ads.googleads.v11.enums.ProductChannelEnum.ProductChannel} channel
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @instance
                             */
                            ProductChannelInfo.prototype.channel = 0;
    
                            /**
                             * Creates a new ProductChannelInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductChannelInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductChannelInfo} ProductChannelInfo instance
                             */
                            ProductChannelInfo.create = function create(properties) {
                                return new ProductChannelInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductChannelInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductChannelInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductChannelInfo} m ProductChannelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.channel != null && Object.hasOwnProperty.call(m, "channel"))
                                    w.uint32(8).int32(m.channel);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductChannelInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductChannelInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductChannelInfo} message ProductChannelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductChannelInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductChannelInfo} ProductChannelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductChannelInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.channel = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductChannelInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductChannelInfo} ProductChannelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductChannelInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductChannelInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.channel != null && m.hasOwnProperty("channel")) {
                                    switch (m.channel) {
                                    default:
                                        return "channel: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductChannelInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductChannelInfo} ProductChannelInfo
                             */
                            ProductChannelInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductChannelInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductChannelInfo();
                                switch (d.channel) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.channel = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.channel = 1;
                                    break;
                                case "ONLINE":
                                case 2:
                                    m.channel = 2;
                                    break;
                                case "LOCAL":
                                case 3:
                                    m.channel = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductChannelInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductChannelInfo} m ProductChannelInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductChannelInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.channel = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.channel != null && m.hasOwnProperty("channel")) {
                                    d.channel = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductChannelEnum.ProductChannel[m.channel] : m.channel;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductChannelInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductChannelInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductChannelInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductChannelInfo;
                        })();
    
                        common.ProductChannelExclusivityInfo = (function() {
    
                            /**
                             * Properties of a ProductChannelExclusivityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductChannelExclusivityInfo
                             * @property {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.ProductChannelExclusivity|null} [channel_exclusivity] ProductChannelExclusivityInfo channel_exclusivity
                             */
    
                            /**
                             * Constructs a new ProductChannelExclusivityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductChannelExclusivityInfo.
                             * @implements IProductChannelExclusivityInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductChannelExclusivityInfo=} [p] Properties to set
                             */
                            function ProductChannelExclusivityInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductChannelExclusivityInfo channel_exclusivity.
                             * @member {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.ProductChannelExclusivity} channel_exclusivity
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @instance
                             */
                            ProductChannelExclusivityInfo.prototype.channel_exclusivity = 0;
    
                            /**
                             * Creates a new ProductChannelExclusivityInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductChannelExclusivityInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductChannelExclusivityInfo} ProductChannelExclusivityInfo instance
                             */
                            ProductChannelExclusivityInfo.create = function create(properties) {
                                return new ProductChannelExclusivityInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductChannelExclusivityInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductChannelExclusivityInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductChannelExclusivityInfo} m ProductChannelExclusivityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelExclusivityInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.channel_exclusivity != null && Object.hasOwnProperty.call(m, "channel_exclusivity"))
                                    w.uint32(8).int32(m.channel_exclusivity);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductChannelExclusivityInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductChannelExclusivityInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductChannelExclusivityInfo} message ProductChannelExclusivityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelExclusivityInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductChannelExclusivityInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductChannelExclusivityInfo} ProductChannelExclusivityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelExclusivityInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.channel_exclusivity = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductChannelExclusivityInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductChannelExclusivityInfo} ProductChannelExclusivityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelExclusivityInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductChannelExclusivityInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductChannelExclusivityInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.channel_exclusivity != null && m.hasOwnProperty("channel_exclusivity")) {
                                    switch (m.channel_exclusivity) {
                                    default:
                                        return "channel_exclusivity: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductChannelExclusivityInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductChannelExclusivityInfo} ProductChannelExclusivityInfo
                             */
                            ProductChannelExclusivityInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductChannelExclusivityInfo();
                                switch (d.channel_exclusivity) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.channel_exclusivity = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.channel_exclusivity = 1;
                                    break;
                                case "SINGLE_CHANNEL":
                                case 2:
                                    m.channel_exclusivity = 2;
                                    break;
                                case "MULTI_CHANNEL":
                                case 3:
                                    m.channel_exclusivity = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductChannelExclusivityInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductChannelExclusivityInfo} m ProductChannelExclusivityInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductChannelExclusivityInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.channel_exclusivity = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.channel_exclusivity != null && m.hasOwnProperty("channel_exclusivity")) {
                                    d.channel_exclusivity = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.ProductChannelExclusivity[m.channel_exclusivity] : m.channel_exclusivity;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductChannelExclusivityInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductChannelExclusivityInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductChannelExclusivityInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductChannelExclusivityInfo;
                        })();
    
                        common.ProductConditionInfo = (function() {
    
                            /**
                             * Properties of a ProductConditionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductConditionInfo
                             * @property {google.ads.googleads.v11.enums.ProductConditionEnum.ProductCondition|null} [condition] ProductConditionInfo condition
                             */
    
                            /**
                             * Constructs a new ProductConditionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductConditionInfo.
                             * @implements IProductConditionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductConditionInfo=} [p] Properties to set
                             */
                            function ProductConditionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductConditionInfo condition.
                             * @member {google.ads.googleads.v11.enums.ProductConditionEnum.ProductCondition} condition
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @instance
                             */
                            ProductConditionInfo.prototype.condition = 0;
    
                            /**
                             * Creates a new ProductConditionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductConditionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductConditionInfo} ProductConditionInfo instance
                             */
                            ProductConditionInfo.create = function create(properties) {
                                return new ProductConditionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductConditionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductConditionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductConditionInfo} m ProductConditionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductConditionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.condition != null && Object.hasOwnProperty.call(m, "condition"))
                                    w.uint32(8).int32(m.condition);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductConditionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductConditionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductConditionInfo} message ProductConditionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductConditionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductConditionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductConditionInfo} ProductConditionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductConditionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductConditionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.condition = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductConditionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductConditionInfo} ProductConditionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductConditionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductConditionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductConditionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.condition != null && m.hasOwnProperty("condition")) {
                                    switch (m.condition) {
                                    default:
                                        return "condition: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductConditionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductConditionInfo} ProductConditionInfo
                             */
                            ProductConditionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductConditionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductConditionInfo();
                                switch (d.condition) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.condition = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.condition = 1;
                                    break;
                                case "NEW":
                                case 3:
                                    m.condition = 3;
                                    break;
                                case "REFURBISHED":
                                case 4:
                                    m.condition = 4;
                                    break;
                                case "USED":
                                case 5:
                                    m.condition = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductConditionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductConditionInfo} m ProductConditionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductConditionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.condition = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.condition != null && m.hasOwnProperty("condition")) {
                                    d.condition = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductConditionEnum.ProductCondition[m.condition] : m.condition;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductConditionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductConditionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductConditionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductConditionInfo;
                        })();
    
                        common.ProductCustomAttributeInfo = (function() {
    
                            /**
                             * Properties of a ProductCustomAttributeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductCustomAttributeInfo
                             * @property {string|null} [value] ProductCustomAttributeInfo value
                             * @property {google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum.ProductCustomAttributeIndex|null} [index] ProductCustomAttributeInfo index
                             */
    
                            /**
                             * Constructs a new ProductCustomAttributeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductCustomAttributeInfo.
                             * @implements IProductCustomAttributeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductCustomAttributeInfo=} [p] Properties to set
                             */
                            function ProductCustomAttributeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductCustomAttributeInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @instance
                             */
                            ProductCustomAttributeInfo.prototype.value = null;
    
                            /**
                             * ProductCustomAttributeInfo index.
                             * @member {google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum.ProductCustomAttributeIndex} index
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @instance
                             */
                            ProductCustomAttributeInfo.prototype.index = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductCustomAttributeInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @instance
                             */
                            Object.defineProperty(ProductCustomAttributeInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductCustomAttributeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductCustomAttributeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductCustomAttributeInfo} ProductCustomAttributeInfo instance
                             */
                            ProductCustomAttributeInfo.create = function create(properties) {
                                return new ProductCustomAttributeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductCustomAttributeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductCustomAttributeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductCustomAttributeInfo} m ProductCustomAttributeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductCustomAttributeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.index != null && Object.hasOwnProperty.call(m, "index"))
                                    w.uint32(16).int32(m.index);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(26).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductCustomAttributeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductCustomAttributeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductCustomAttributeInfo} message ProductCustomAttributeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductCustomAttributeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductCustomAttributeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductCustomAttributeInfo} ProductCustomAttributeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductCustomAttributeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.value = r.string();
                                        break;
                                    case 2:
                                        m.index = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductCustomAttributeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductCustomAttributeInfo} ProductCustomAttributeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductCustomAttributeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductCustomAttributeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductCustomAttributeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                if (m.index != null && m.hasOwnProperty("index")) {
                                    switch (m.index) {
                                    default:
                                        return "index: enum value expected";
                                    case 0:
                                    case 1:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductCustomAttributeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductCustomAttributeInfo} ProductCustomAttributeInfo
                             */
                            ProductCustomAttributeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductCustomAttributeInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                switch (d.index) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.index = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.index = 1;
                                    break;
                                case "INDEX0":
                                case 7:
                                    m.index = 7;
                                    break;
                                case "INDEX1":
                                case 8:
                                    m.index = 8;
                                    break;
                                case "INDEX2":
                                case 9:
                                    m.index = 9;
                                    break;
                                case "INDEX3":
                                case 10:
                                    m.index = 10;
                                    break;
                                case "INDEX4":
                                case 11:
                                    m.index = 11;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductCustomAttributeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductCustomAttributeInfo} m ProductCustomAttributeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductCustomAttributeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.index = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.index != null && m.hasOwnProperty("index")) {
                                    d.index = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum.ProductCustomAttributeIndex[m.index] : m.index;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductCustomAttributeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductCustomAttributeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductCustomAttributeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductCustomAttributeInfo;
                        })();
    
                        common.ProductItemIdInfo = (function() {
    
                            /**
                             * Properties of a ProductItemIdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductItemIdInfo
                             * @property {string|null} [value] ProductItemIdInfo value
                             */
    
                            /**
                             * Constructs a new ProductItemIdInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductItemIdInfo.
                             * @implements IProductItemIdInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductItemIdInfo=} [p] Properties to set
                             */
                            function ProductItemIdInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductItemIdInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @instance
                             */
                            ProductItemIdInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductItemIdInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @instance
                             */
                            Object.defineProperty(ProductItemIdInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductItemIdInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductItemIdInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductItemIdInfo} ProductItemIdInfo instance
                             */
                            ProductItemIdInfo.create = function create(properties) {
                                return new ProductItemIdInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductItemIdInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductItemIdInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductItemIdInfo} m ProductItemIdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductItemIdInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(18).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductItemIdInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductItemIdInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductItemIdInfo} message ProductItemIdInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductItemIdInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductItemIdInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductItemIdInfo} ProductItemIdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductItemIdInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductItemIdInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductItemIdInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductItemIdInfo} ProductItemIdInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductItemIdInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductItemIdInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductItemIdInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductItemIdInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductItemIdInfo} ProductItemIdInfo
                             */
                            ProductItemIdInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductItemIdInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductItemIdInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductItemIdInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductItemIdInfo} m ProductItemIdInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductItemIdInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductItemIdInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductItemIdInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductItemIdInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductItemIdInfo;
                        })();
    
                        common.ProductTypeInfo = (function() {
    
                            /**
                             * Properties of a ProductTypeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductTypeInfo
                             * @property {string|null} [value] ProductTypeInfo value
                             * @property {google.ads.googleads.v11.enums.ProductTypeLevelEnum.ProductTypeLevel|null} [level] ProductTypeInfo level
                             */
    
                            /**
                             * Constructs a new ProductTypeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductTypeInfo.
                             * @implements IProductTypeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductTypeInfo=} [p] Properties to set
                             */
                            function ProductTypeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductTypeInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @instance
                             */
                            ProductTypeInfo.prototype.value = null;
    
                            /**
                             * ProductTypeInfo level.
                             * @member {google.ads.googleads.v11.enums.ProductTypeLevelEnum.ProductTypeLevel} level
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @instance
                             */
                            ProductTypeInfo.prototype.level = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductTypeInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @instance
                             */
                            Object.defineProperty(ProductTypeInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductTypeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductTypeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductTypeInfo} ProductTypeInfo instance
                             */
                            ProductTypeInfo.create = function create(properties) {
                                return new ProductTypeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductTypeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductTypeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductTypeInfo} m ProductTypeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductTypeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.level != null && Object.hasOwnProperty.call(m, "level"))
                                    w.uint32(16).int32(m.level);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(26).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductTypeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductTypeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductTypeInfo} message ProductTypeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductTypeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductTypeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductTypeInfo} ProductTypeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductTypeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductTypeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.value = r.string();
                                        break;
                                    case 2:
                                        m.level = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductTypeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductTypeInfo} ProductTypeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductTypeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductTypeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductTypeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    switch (m.level) {
                                    default:
                                        return "level: enum value expected";
                                    case 0:
                                    case 1:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductTypeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductTypeInfo} ProductTypeInfo
                             */
                            ProductTypeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductTypeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductTypeInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                switch (d.level) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.level = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.level = 1;
                                    break;
                                case "LEVEL1":
                                case 7:
                                    m.level = 7;
                                    break;
                                case "LEVEL2":
                                case 8:
                                    m.level = 8;
                                    break;
                                case "LEVEL3":
                                case 9:
                                    m.level = 9;
                                    break;
                                case "LEVEL4":
                                case 10:
                                    m.level = 10;
                                    break;
                                case "LEVEL5":
                                case 11:
                                    m.level = 11;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductTypeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductTypeInfo} m ProductTypeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductTypeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.level = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    d.level = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductTypeLevelEnum.ProductTypeLevel[m.level] : m.level;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductTypeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductTypeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductTypeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductTypeInfo;
                        })();
    
                        common.ProductGroupingInfo = (function() {
    
                            /**
                             * Properties of a ProductGroupingInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductGroupingInfo
                             * @property {string|null} [value] ProductGroupingInfo value
                             */
    
                            /**
                             * Constructs a new ProductGroupingInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductGroupingInfo.
                             * @implements IProductGroupingInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductGroupingInfo=} [p] Properties to set
                             */
                            function ProductGroupingInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductGroupingInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @instance
                             */
                            ProductGroupingInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductGroupingInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @instance
                             */
                            Object.defineProperty(ProductGroupingInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductGroupingInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductGroupingInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductGroupingInfo} ProductGroupingInfo instance
                             */
                            ProductGroupingInfo.create = function create(properties) {
                                return new ProductGroupingInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductGroupingInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductGroupingInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductGroupingInfo} m ProductGroupingInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductGroupingInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(10).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductGroupingInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductGroupingInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductGroupingInfo} message ProductGroupingInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductGroupingInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductGroupingInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductGroupingInfo} ProductGroupingInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductGroupingInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductGroupingInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductGroupingInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductGroupingInfo} ProductGroupingInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductGroupingInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductGroupingInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductGroupingInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductGroupingInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductGroupingInfo} ProductGroupingInfo
                             */
                            ProductGroupingInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductGroupingInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductGroupingInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductGroupingInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductGroupingInfo} m ProductGroupingInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductGroupingInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductGroupingInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductGroupingInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductGroupingInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductGroupingInfo;
                        })();
    
                        common.ProductLabelsInfo = (function() {
    
                            /**
                             * Properties of a ProductLabelsInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductLabelsInfo
                             * @property {string|null} [value] ProductLabelsInfo value
                             */
    
                            /**
                             * Constructs a new ProductLabelsInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductLabelsInfo.
                             * @implements IProductLabelsInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductLabelsInfo=} [p] Properties to set
                             */
                            function ProductLabelsInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductLabelsInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @instance
                             */
                            ProductLabelsInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductLabelsInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @instance
                             */
                            Object.defineProperty(ProductLabelsInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductLabelsInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductLabelsInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductLabelsInfo} ProductLabelsInfo instance
                             */
                            ProductLabelsInfo.create = function create(properties) {
                                return new ProductLabelsInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductLabelsInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductLabelsInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductLabelsInfo} m ProductLabelsInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductLabelsInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(10).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductLabelsInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductLabelsInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductLabelsInfo} message ProductLabelsInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductLabelsInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductLabelsInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductLabelsInfo} ProductLabelsInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductLabelsInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductLabelsInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductLabelsInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductLabelsInfo} ProductLabelsInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductLabelsInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductLabelsInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductLabelsInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductLabelsInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductLabelsInfo} ProductLabelsInfo
                             */
                            ProductLabelsInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductLabelsInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductLabelsInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductLabelsInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductLabelsInfo} m ProductLabelsInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductLabelsInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductLabelsInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductLabelsInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductLabelsInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductLabelsInfo;
                        })();
    
                        common.ProductLegacyConditionInfo = (function() {
    
                            /**
                             * Properties of a ProductLegacyConditionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductLegacyConditionInfo
                             * @property {string|null} [value] ProductLegacyConditionInfo value
                             */
    
                            /**
                             * Constructs a new ProductLegacyConditionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductLegacyConditionInfo.
                             * @implements IProductLegacyConditionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductLegacyConditionInfo=} [p] Properties to set
                             */
                            function ProductLegacyConditionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductLegacyConditionInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @instance
                             */
                            ProductLegacyConditionInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductLegacyConditionInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @instance
                             */
                            Object.defineProperty(ProductLegacyConditionInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductLegacyConditionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductLegacyConditionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductLegacyConditionInfo} ProductLegacyConditionInfo instance
                             */
                            ProductLegacyConditionInfo.create = function create(properties) {
                                return new ProductLegacyConditionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductLegacyConditionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductLegacyConditionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductLegacyConditionInfo} m ProductLegacyConditionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductLegacyConditionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(10).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductLegacyConditionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductLegacyConditionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductLegacyConditionInfo} message ProductLegacyConditionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductLegacyConditionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductLegacyConditionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductLegacyConditionInfo} ProductLegacyConditionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductLegacyConditionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductLegacyConditionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductLegacyConditionInfo} ProductLegacyConditionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductLegacyConditionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductLegacyConditionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductLegacyConditionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductLegacyConditionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductLegacyConditionInfo} ProductLegacyConditionInfo
                             */
                            ProductLegacyConditionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductLegacyConditionInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductLegacyConditionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductLegacyConditionInfo} m ProductLegacyConditionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductLegacyConditionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductLegacyConditionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductLegacyConditionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductLegacyConditionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductLegacyConditionInfo;
                        })();
    
                        common.ProductTypeFullInfo = (function() {
    
                            /**
                             * Properties of a ProductTypeFullInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProductTypeFullInfo
                             * @property {string|null} [value] ProductTypeFullInfo value
                             */
    
                            /**
                             * Constructs a new ProductTypeFullInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProductTypeFullInfo.
                             * @implements IProductTypeFullInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProductTypeFullInfo=} [p] Properties to set
                             */
                            function ProductTypeFullInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductTypeFullInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @instance
                             */
                            ProductTypeFullInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductTypeFullInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @instance
                             */
                            Object.defineProperty(ProductTypeFullInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductTypeFullInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductTypeFullInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProductTypeFullInfo} ProductTypeFullInfo instance
                             */
                            ProductTypeFullInfo.create = function create(properties) {
                                return new ProductTypeFullInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProductTypeFullInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProductTypeFullInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductTypeFullInfo} m ProductTypeFullInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductTypeFullInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(10).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductTypeFullInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProductTypeFullInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProductTypeFullInfo} message ProductTypeFullInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductTypeFullInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductTypeFullInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProductTypeFullInfo} ProductTypeFullInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductTypeFullInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProductTypeFullInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductTypeFullInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProductTypeFullInfo} ProductTypeFullInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductTypeFullInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductTypeFullInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductTypeFullInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductTypeFullInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProductTypeFullInfo} ProductTypeFullInfo
                             */
                            ProductTypeFullInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProductTypeFullInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProductTypeFullInfo();
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductTypeFullInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProductTypeFullInfo} m ProductTypeFullInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductTypeFullInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductTypeFullInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProductTypeFullInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductTypeFullInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductTypeFullInfo;
                        })();
    
                        common.UnknownListingDimensionInfo = (function() {
    
                            /**
                             * Properties of an UnknownListingDimensionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUnknownListingDimensionInfo
                             */
    
                            /**
                             * Constructs a new UnknownListingDimensionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an UnknownListingDimensionInfo.
                             * @implements IUnknownListingDimensionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUnknownListingDimensionInfo=} [p] Properties to set
                             */
                            function UnknownListingDimensionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UnknownListingDimensionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUnknownListingDimensionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UnknownListingDimensionInfo} UnknownListingDimensionInfo instance
                             */
                            UnknownListingDimensionInfo.create = function create(properties) {
                                return new UnknownListingDimensionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UnknownListingDimensionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UnknownListingDimensionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUnknownListingDimensionInfo} m UnknownListingDimensionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UnknownListingDimensionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UnknownListingDimensionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UnknownListingDimensionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUnknownListingDimensionInfo} message UnknownListingDimensionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UnknownListingDimensionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UnknownListingDimensionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UnknownListingDimensionInfo} UnknownListingDimensionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UnknownListingDimensionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UnknownListingDimensionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UnknownListingDimensionInfo} UnknownListingDimensionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UnknownListingDimensionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UnknownListingDimensionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UnknownListingDimensionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an UnknownListingDimensionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UnknownListingDimensionInfo} UnknownListingDimensionInfo
                             */
                            UnknownListingDimensionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.UnknownListingDimensionInfo();
                            };
    
                            /**
                             * Creates a plain object from an UnknownListingDimensionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UnknownListingDimensionInfo} m UnknownListingDimensionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UnknownListingDimensionInfo.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UnknownListingDimensionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UnknownListingDimensionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UnknownListingDimensionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UnknownListingDimensionInfo;
                        })();
    
                        common.HotelDateSelectionTypeInfo = (function() {
    
                            /**
                             * Properties of a HotelDateSelectionTypeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelDateSelectionTypeInfo
                             * @property {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.HotelDateSelectionType|null} [type] HotelDateSelectionTypeInfo type
                             */
    
                            /**
                             * Constructs a new HotelDateSelectionTypeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelDateSelectionTypeInfo.
                             * @implements IHotelDateSelectionTypeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelDateSelectionTypeInfo=} [p] Properties to set
                             */
                            function HotelDateSelectionTypeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelDateSelectionTypeInfo type.
                             * @member {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.HotelDateSelectionType} type
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @instance
                             */
                            HotelDateSelectionTypeInfo.prototype.type = 0;
    
                            /**
                             * Creates a new HotelDateSelectionTypeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelDateSelectionTypeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelDateSelectionTypeInfo} HotelDateSelectionTypeInfo instance
                             */
                            HotelDateSelectionTypeInfo.create = function create(properties) {
                                return new HotelDateSelectionTypeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelDateSelectionTypeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelDateSelectionTypeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelDateSelectionTypeInfo} m HotelDateSelectionTypeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelDateSelectionTypeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelDateSelectionTypeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelDateSelectionTypeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelDateSelectionTypeInfo} message HotelDateSelectionTypeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelDateSelectionTypeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelDateSelectionTypeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelDateSelectionTypeInfo} HotelDateSelectionTypeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelDateSelectionTypeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelDateSelectionTypeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelDateSelectionTypeInfo} HotelDateSelectionTypeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelDateSelectionTypeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelDateSelectionTypeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelDateSelectionTypeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 50:
                                    case 51:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelDateSelectionTypeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelDateSelectionTypeInfo} HotelDateSelectionTypeInfo
                             */
                            HotelDateSelectionTypeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "DEFAULT_SELECTION":
                                case 50:
                                    m.type = 50;
                                    break;
                                case "USER_SELECTED":
                                case 51:
                                    m.type = 51;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelDateSelectionTypeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelDateSelectionTypeInfo} m HotelDateSelectionTypeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelDateSelectionTypeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.HotelDateSelectionType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelDateSelectionTypeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelDateSelectionTypeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelDateSelectionTypeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelDateSelectionTypeInfo;
                        })();
    
                        common.HotelAdvanceBookingWindowInfo = (function() {
    
                            /**
                             * Properties of a HotelAdvanceBookingWindowInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelAdvanceBookingWindowInfo
                             * @property {number|null} [min_days] HotelAdvanceBookingWindowInfo min_days
                             * @property {number|null} [max_days] HotelAdvanceBookingWindowInfo max_days
                             */
    
                            /**
                             * Constructs a new HotelAdvanceBookingWindowInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelAdvanceBookingWindowInfo.
                             * @implements IHotelAdvanceBookingWindowInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelAdvanceBookingWindowInfo=} [p] Properties to set
                             */
                            function HotelAdvanceBookingWindowInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelAdvanceBookingWindowInfo min_days.
                             * @member {number|null|undefined} min_days
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @instance
                             */
                            HotelAdvanceBookingWindowInfo.prototype.min_days = null;
    
                            /**
                             * HotelAdvanceBookingWindowInfo max_days.
                             * @member {number|null|undefined} max_days
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @instance
                             */
                            HotelAdvanceBookingWindowInfo.prototype.max_days = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelAdvanceBookingWindowInfo _min_days.
                             * @member {"min_days"|undefined} _min_days
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @instance
                             */
                            Object.defineProperty(HotelAdvanceBookingWindowInfo.prototype, "_min_days", {
                                get: $util.oneOfGetter($oneOfFields = ["min_days"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * HotelAdvanceBookingWindowInfo _max_days.
                             * @member {"max_days"|undefined} _max_days
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @instance
                             */
                            Object.defineProperty(HotelAdvanceBookingWindowInfo.prototype, "_max_days", {
                                get: $util.oneOfGetter($oneOfFields = ["max_days"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelAdvanceBookingWindowInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelAdvanceBookingWindowInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo} HotelAdvanceBookingWindowInfo instance
                             */
                            HotelAdvanceBookingWindowInfo.create = function create(properties) {
                                return new HotelAdvanceBookingWindowInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelAdvanceBookingWindowInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelAdvanceBookingWindowInfo} m HotelAdvanceBookingWindowInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelAdvanceBookingWindowInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.min_days != null && Object.hasOwnProperty.call(m, "min_days"))
                                    w.uint32(24).int64(m.min_days);
                                if (m.max_days != null && Object.hasOwnProperty.call(m, "max_days"))
                                    w.uint32(32).int64(m.max_days);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelAdvanceBookingWindowInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelAdvanceBookingWindowInfo} message HotelAdvanceBookingWindowInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelAdvanceBookingWindowInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelAdvanceBookingWindowInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo} HotelAdvanceBookingWindowInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelAdvanceBookingWindowInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.min_days = r.int64();
                                        break;
                                    case 4:
                                        m.max_days = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelAdvanceBookingWindowInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo} HotelAdvanceBookingWindowInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelAdvanceBookingWindowInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelAdvanceBookingWindowInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelAdvanceBookingWindowInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.min_days != null && m.hasOwnProperty("min_days")) {
                                    p._min_days = 1;
                                    if (!$util.isInteger(m.min_days) && !(m.min_days && $util.isInteger(m.min_days.low) && $util.isInteger(m.min_days.high)))
                                        return "min_days: integer|Long expected";
                                }
                                if (m.max_days != null && m.hasOwnProperty("max_days")) {
                                    p._max_days = 1;
                                    if (!$util.isInteger(m.max_days) && !(m.max_days && $util.isInteger(m.max_days.low) && $util.isInteger(m.max_days.high)))
                                        return "max_days: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelAdvanceBookingWindowInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo} HotelAdvanceBookingWindowInfo
                             */
                            HotelAdvanceBookingWindowInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo();
                                if (d.min_days != null) {
                                    if ($util.Long)
                                        (m.min_days = $util.Long.fromValue(d.min_days)).unsigned = false;
                                    else if (typeof d.min_days === "string")
                                        m.min_days = parseInt(d.min_days, 10);
                                    else if (typeof d.min_days === "number")
                                        m.min_days = d.min_days;
                                    else if (typeof d.min_days === "object")
                                        m.min_days = new $util.LongBits(d.min_days.low >>> 0, d.min_days.high >>> 0).toNumber();
                                }
                                if (d.max_days != null) {
                                    if ($util.Long)
                                        (m.max_days = $util.Long.fromValue(d.max_days)).unsigned = false;
                                    else if (typeof d.max_days === "string")
                                        m.max_days = parseInt(d.max_days, 10);
                                    else if (typeof d.max_days === "number")
                                        m.max_days = d.max_days;
                                    else if (typeof d.max_days === "object")
                                        m.max_days = new $util.LongBits(d.max_days.low >>> 0, d.max_days.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelAdvanceBookingWindowInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo} m HotelAdvanceBookingWindowInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelAdvanceBookingWindowInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.min_days != null && m.hasOwnProperty("min_days")) {
                                    if (typeof m.min_days === "number")
                                        d.min_days = o.longs === String ? String(m.min_days) : m.min_days;
                                    else
                                        d.min_days = o.longs === String ? $util.Long.prototype.toString.call(m.min_days) : o.longs === Number ? new $util.LongBits(m.min_days.low >>> 0, m.min_days.high >>> 0).toNumber() : m.min_days;
                                    if (o.oneofs)
                                        d._min_days = "min_days";
                                }
                                if (m.max_days != null && m.hasOwnProperty("max_days")) {
                                    if (typeof m.max_days === "number")
                                        d.max_days = o.longs === String ? String(m.max_days) : m.max_days;
                                    else
                                        d.max_days = o.longs === String ? $util.Long.prototype.toString.call(m.max_days) : o.longs === Number ? new $util.LongBits(m.max_days.low >>> 0, m.max_days.high >>> 0).toNumber() : m.max_days;
                                    if (o.oneofs)
                                        d._max_days = "max_days";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelAdvanceBookingWindowInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelAdvanceBookingWindowInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelAdvanceBookingWindowInfo;
                        })();
    
                        common.HotelLengthOfStayInfo = (function() {
    
                            /**
                             * Properties of a HotelLengthOfStayInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelLengthOfStayInfo
                             * @property {number|null} [min_nights] HotelLengthOfStayInfo min_nights
                             * @property {number|null} [max_nights] HotelLengthOfStayInfo max_nights
                             */
    
                            /**
                             * Constructs a new HotelLengthOfStayInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelLengthOfStayInfo.
                             * @implements IHotelLengthOfStayInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelLengthOfStayInfo=} [p] Properties to set
                             */
                            function HotelLengthOfStayInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelLengthOfStayInfo min_nights.
                             * @member {number|null|undefined} min_nights
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @instance
                             */
                            HotelLengthOfStayInfo.prototype.min_nights = null;
    
                            /**
                             * HotelLengthOfStayInfo max_nights.
                             * @member {number|null|undefined} max_nights
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @instance
                             */
                            HotelLengthOfStayInfo.prototype.max_nights = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelLengthOfStayInfo _min_nights.
                             * @member {"min_nights"|undefined} _min_nights
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @instance
                             */
                            Object.defineProperty(HotelLengthOfStayInfo.prototype, "_min_nights", {
                                get: $util.oneOfGetter($oneOfFields = ["min_nights"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * HotelLengthOfStayInfo _max_nights.
                             * @member {"max_nights"|undefined} _max_nights
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @instance
                             */
                            Object.defineProperty(HotelLengthOfStayInfo.prototype, "_max_nights", {
                                get: $util.oneOfGetter($oneOfFields = ["max_nights"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelLengthOfStayInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelLengthOfStayInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelLengthOfStayInfo} HotelLengthOfStayInfo instance
                             */
                            HotelLengthOfStayInfo.create = function create(properties) {
                                return new HotelLengthOfStayInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelLengthOfStayInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelLengthOfStayInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelLengthOfStayInfo} m HotelLengthOfStayInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelLengthOfStayInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.min_nights != null && Object.hasOwnProperty.call(m, "min_nights"))
                                    w.uint32(24).int64(m.min_nights);
                                if (m.max_nights != null && Object.hasOwnProperty.call(m, "max_nights"))
                                    w.uint32(32).int64(m.max_nights);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelLengthOfStayInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelLengthOfStayInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelLengthOfStayInfo} message HotelLengthOfStayInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelLengthOfStayInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelLengthOfStayInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelLengthOfStayInfo} HotelLengthOfStayInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelLengthOfStayInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.min_nights = r.int64();
                                        break;
                                    case 4:
                                        m.max_nights = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelLengthOfStayInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelLengthOfStayInfo} HotelLengthOfStayInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelLengthOfStayInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelLengthOfStayInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelLengthOfStayInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.min_nights != null && m.hasOwnProperty("min_nights")) {
                                    p._min_nights = 1;
                                    if (!$util.isInteger(m.min_nights) && !(m.min_nights && $util.isInteger(m.min_nights.low) && $util.isInteger(m.min_nights.high)))
                                        return "min_nights: integer|Long expected";
                                }
                                if (m.max_nights != null && m.hasOwnProperty("max_nights")) {
                                    p._max_nights = 1;
                                    if (!$util.isInteger(m.max_nights) && !(m.max_nights && $util.isInteger(m.max_nights.low) && $util.isInteger(m.max_nights.high)))
                                        return "max_nights: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelLengthOfStayInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelLengthOfStayInfo} HotelLengthOfStayInfo
                             */
                            HotelLengthOfStayInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo();
                                if (d.min_nights != null) {
                                    if ($util.Long)
                                        (m.min_nights = $util.Long.fromValue(d.min_nights)).unsigned = false;
                                    else if (typeof d.min_nights === "string")
                                        m.min_nights = parseInt(d.min_nights, 10);
                                    else if (typeof d.min_nights === "number")
                                        m.min_nights = d.min_nights;
                                    else if (typeof d.min_nights === "object")
                                        m.min_nights = new $util.LongBits(d.min_nights.low >>> 0, d.min_nights.high >>> 0).toNumber();
                                }
                                if (d.max_nights != null) {
                                    if ($util.Long)
                                        (m.max_nights = $util.Long.fromValue(d.max_nights)).unsigned = false;
                                    else if (typeof d.max_nights === "string")
                                        m.max_nights = parseInt(d.max_nights, 10);
                                    else if (typeof d.max_nights === "number")
                                        m.max_nights = d.max_nights;
                                    else if (typeof d.max_nights === "object")
                                        m.max_nights = new $util.LongBits(d.max_nights.low >>> 0, d.max_nights.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelLengthOfStayInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelLengthOfStayInfo} m HotelLengthOfStayInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelLengthOfStayInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.min_nights != null && m.hasOwnProperty("min_nights")) {
                                    if (typeof m.min_nights === "number")
                                        d.min_nights = o.longs === String ? String(m.min_nights) : m.min_nights;
                                    else
                                        d.min_nights = o.longs === String ? $util.Long.prototype.toString.call(m.min_nights) : o.longs === Number ? new $util.LongBits(m.min_nights.low >>> 0, m.min_nights.high >>> 0).toNumber() : m.min_nights;
                                    if (o.oneofs)
                                        d._min_nights = "min_nights";
                                }
                                if (m.max_nights != null && m.hasOwnProperty("max_nights")) {
                                    if (typeof m.max_nights === "number")
                                        d.max_nights = o.longs === String ? String(m.max_nights) : m.max_nights;
                                    else
                                        d.max_nights = o.longs === String ? $util.Long.prototype.toString.call(m.max_nights) : o.longs === Number ? new $util.LongBits(m.max_nights.low >>> 0, m.max_nights.high >>> 0).toNumber() : m.max_nights;
                                    if (o.oneofs)
                                        d._max_nights = "max_nights";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelLengthOfStayInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelLengthOfStayInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelLengthOfStayInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelLengthOfStayInfo;
                        })();
    
                        common.HotelCheckInDateRangeInfo = (function() {
    
                            /**
                             * Properties of a HotelCheckInDateRangeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelCheckInDateRangeInfo
                             * @property {string|null} [start_date] HotelCheckInDateRangeInfo start_date
                             * @property {string|null} [end_date] HotelCheckInDateRangeInfo end_date
                             */
    
                            /**
                             * Constructs a new HotelCheckInDateRangeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelCheckInDateRangeInfo.
                             * @implements IHotelCheckInDateRangeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDateRangeInfo=} [p] Properties to set
                             */
                            function HotelCheckInDateRangeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelCheckInDateRangeInfo start_date.
                             * @member {string} start_date
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @instance
                             */
                            HotelCheckInDateRangeInfo.prototype.start_date = "";
    
                            /**
                             * HotelCheckInDateRangeInfo end_date.
                             * @member {string} end_date
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @instance
                             */
                            HotelCheckInDateRangeInfo.prototype.end_date = "";
    
                            /**
                             * Creates a new HotelCheckInDateRangeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDateRangeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDateRangeInfo} HotelCheckInDateRangeInfo instance
                             */
                            HotelCheckInDateRangeInfo.create = function create(properties) {
                                return new HotelCheckInDateRangeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelCheckInDateRangeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelCheckInDateRangeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDateRangeInfo} m HotelCheckInDateRangeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCheckInDateRangeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(10).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(18).string(m.end_date);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelCheckInDateRangeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelCheckInDateRangeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDateRangeInfo} message HotelCheckInDateRangeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCheckInDateRangeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelCheckInDateRangeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDateRangeInfo} HotelCheckInDateRangeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCheckInDateRangeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.start_date = r.string();
                                        break;
                                    case 2:
                                        m.end_date = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelCheckInDateRangeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDateRangeInfo} HotelCheckInDateRangeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCheckInDateRangeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelCheckInDateRangeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelCheckInDateRangeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelCheckInDateRangeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDateRangeInfo} HotelCheckInDateRangeInfo
                             */
                            HotelCheckInDateRangeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo();
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelCheckInDateRangeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelCheckInDateRangeInfo} m HotelCheckInDateRangeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelCheckInDateRangeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.start_date = "";
                                    d.end_date = "";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelCheckInDateRangeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDateRangeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelCheckInDateRangeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelCheckInDateRangeInfo;
                        })();
    
                        common.HotelCheckInDayInfo = (function() {
    
                            /**
                             * Properties of a HotelCheckInDayInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelCheckInDayInfo
                             * @property {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek|null} [day_of_week] HotelCheckInDayInfo day_of_week
                             */
    
                            /**
                             * Constructs a new HotelCheckInDayInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelCheckInDayInfo.
                             * @implements IHotelCheckInDayInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDayInfo=} [p] Properties to set
                             */
                            function HotelCheckInDayInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelCheckInDayInfo day_of_week.
                             * @member {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek} day_of_week
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @instance
                             */
                            HotelCheckInDayInfo.prototype.day_of_week = 0;
    
                            /**
                             * Creates a new HotelCheckInDayInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDayInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDayInfo} HotelCheckInDayInfo instance
                             */
                            HotelCheckInDayInfo.create = function create(properties) {
                                return new HotelCheckInDayInfo(properties);
                            };
    
                            /**
                             * Encodes the specified HotelCheckInDayInfo message. Does not implicitly {@link google.ads.googleads.v11.common.HotelCheckInDayInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDayInfo} m HotelCheckInDayInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCheckInDayInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.day_of_week != null && Object.hasOwnProperty.call(m, "day_of_week"))
                                    w.uint32(8).int32(m.day_of_week);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelCheckInDayInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelCheckInDayInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCheckInDayInfo} message HotelCheckInDayInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCheckInDayInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelCheckInDayInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDayInfo} HotelCheckInDayInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCheckInDayInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelCheckInDayInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.day_of_week = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelCheckInDayInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDayInfo} HotelCheckInDayInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCheckInDayInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelCheckInDayInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelCheckInDayInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.day_of_week != null && m.hasOwnProperty("day_of_week")) {
                                    switch (m.day_of_week) {
                                    default:
                                        return "day_of_week: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelCheckInDayInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelCheckInDayInfo} HotelCheckInDayInfo
                             */
                            HotelCheckInDayInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelCheckInDayInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelCheckInDayInfo();
                                switch (d.day_of_week) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.day_of_week = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.day_of_week = 1;
                                    break;
                                case "MONDAY":
                                case 2:
                                    m.day_of_week = 2;
                                    break;
                                case "TUESDAY":
                                case 3:
                                    m.day_of_week = 3;
                                    break;
                                case "WEDNESDAY":
                                case 4:
                                    m.day_of_week = 4;
                                    break;
                                case "THURSDAY":
                                case 5:
                                    m.day_of_week = 5;
                                    break;
                                case "FRIDAY":
                                case 6:
                                    m.day_of_week = 6;
                                    break;
                                case "SATURDAY":
                                case 7:
                                    m.day_of_week = 7;
                                    break;
                                case "SUNDAY":
                                case 8:
                                    m.day_of_week = 8;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelCheckInDayInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelCheckInDayInfo} m HotelCheckInDayInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelCheckInDayInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.day_of_week = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.day_of_week != null && m.hasOwnProperty("day_of_week")) {
                                    d.day_of_week = o.enums === String ? $root.google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek[m.day_of_week] : m.day_of_week;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelCheckInDayInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelCheckInDayInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelCheckInDayInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelCheckInDayInfo;
                        })();
    
                        common.InteractionTypeInfo = (function() {
    
                            /**
                             * Properties of an InteractionTypeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IInteractionTypeInfo
                             * @property {google.ads.googleads.v11.enums.InteractionTypeEnum.InteractionType|null} [type] InteractionTypeInfo type
                             */
    
                            /**
                             * Constructs a new InteractionTypeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an InteractionTypeInfo.
                             * @implements IInteractionTypeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IInteractionTypeInfo=} [p] Properties to set
                             */
                            function InteractionTypeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * InteractionTypeInfo type.
                             * @member {google.ads.googleads.v11.enums.InteractionTypeEnum.InteractionType} type
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @instance
                             */
                            InteractionTypeInfo.prototype.type = 0;
    
                            /**
                             * Creates a new InteractionTypeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IInteractionTypeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.InteractionTypeInfo} InteractionTypeInfo instance
                             */
                            InteractionTypeInfo.create = function create(properties) {
                                return new InteractionTypeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified InteractionTypeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.InteractionTypeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IInteractionTypeInfo} m InteractionTypeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InteractionTypeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified InteractionTypeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.InteractionTypeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IInteractionTypeInfo} message InteractionTypeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InteractionTypeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an InteractionTypeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.InteractionTypeInfo} InteractionTypeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InteractionTypeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.InteractionTypeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an InteractionTypeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.InteractionTypeInfo} InteractionTypeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InteractionTypeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an InteractionTypeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            InteractionTypeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 8000:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an InteractionTypeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.InteractionTypeInfo} InteractionTypeInfo
                             */
                            InteractionTypeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.InteractionTypeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.InteractionTypeInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CALLS":
                                case 8000:
                                    m.type = 8000;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an InteractionTypeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.InteractionTypeInfo} m InteractionTypeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            InteractionTypeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.InteractionTypeEnum.InteractionType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this InteractionTypeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.InteractionTypeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            InteractionTypeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return InteractionTypeInfo;
                        })();
    
                        common.AdScheduleInfo = (function() {
    
                            /**
                             * Properties of an AdScheduleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAdScheduleInfo
                             * @property {google.ads.googleads.v11.enums.MinuteOfHourEnum.MinuteOfHour|null} [start_minute] AdScheduleInfo start_minute
                             * @property {google.ads.googleads.v11.enums.MinuteOfHourEnum.MinuteOfHour|null} [end_minute] AdScheduleInfo end_minute
                             * @property {number|null} [start_hour] AdScheduleInfo start_hour
                             * @property {number|null} [end_hour] AdScheduleInfo end_hour
                             * @property {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek|null} [day_of_week] AdScheduleInfo day_of_week
                             */
    
                            /**
                             * Constructs a new AdScheduleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AdScheduleInfo.
                             * @implements IAdScheduleInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAdScheduleInfo=} [p] Properties to set
                             */
                            function AdScheduleInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdScheduleInfo start_minute.
                             * @member {google.ads.googleads.v11.enums.MinuteOfHourEnum.MinuteOfHour} start_minute
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             */
                            AdScheduleInfo.prototype.start_minute = 0;
    
                            /**
                             * AdScheduleInfo end_minute.
                             * @member {google.ads.googleads.v11.enums.MinuteOfHourEnum.MinuteOfHour} end_minute
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             */
                            AdScheduleInfo.prototype.end_minute = 0;
    
                            /**
                             * AdScheduleInfo start_hour.
                             * @member {number|null|undefined} start_hour
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             */
                            AdScheduleInfo.prototype.start_hour = null;
    
                            /**
                             * AdScheduleInfo end_hour.
                             * @member {number|null|undefined} end_hour
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             */
                            AdScheduleInfo.prototype.end_hour = null;
    
                            /**
                             * AdScheduleInfo day_of_week.
                             * @member {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek} day_of_week
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             */
                            AdScheduleInfo.prototype.day_of_week = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdScheduleInfo _start_hour.
                             * @member {"start_hour"|undefined} _start_hour
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             */
                            Object.defineProperty(AdScheduleInfo.prototype, "_start_hour", {
                                get: $util.oneOfGetter($oneOfFields = ["start_hour"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdScheduleInfo _end_hour.
                             * @member {"end_hour"|undefined} _end_hour
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             */
                            Object.defineProperty(AdScheduleInfo.prototype, "_end_hour", {
                                get: $util.oneOfGetter($oneOfFields = ["end_hour"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdScheduleInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdScheduleInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AdScheduleInfo} AdScheduleInfo instance
                             */
                            AdScheduleInfo.create = function create(properties) {
                                return new AdScheduleInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AdScheduleInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AdScheduleInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdScheduleInfo} m AdScheduleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdScheduleInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.start_minute != null && Object.hasOwnProperty.call(m, "start_minute"))
                                    w.uint32(8).int32(m.start_minute);
                                if (m.end_minute != null && Object.hasOwnProperty.call(m, "end_minute"))
                                    w.uint32(16).int32(m.end_minute);
                                if (m.day_of_week != null && Object.hasOwnProperty.call(m, "day_of_week"))
                                    w.uint32(40).int32(m.day_of_week);
                                if (m.start_hour != null && Object.hasOwnProperty.call(m, "start_hour"))
                                    w.uint32(48).int32(m.start_hour);
                                if (m.end_hour != null && Object.hasOwnProperty.call(m, "end_hour"))
                                    w.uint32(56).int32(m.end_hour);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdScheduleInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AdScheduleInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAdScheduleInfo} message AdScheduleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdScheduleInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdScheduleInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AdScheduleInfo} AdScheduleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdScheduleInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AdScheduleInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.start_minute = r.int32();
                                        break;
                                    case 2:
                                        m.end_minute = r.int32();
                                        break;
                                    case 6:
                                        m.start_hour = r.int32();
                                        break;
                                    case 7:
                                        m.end_hour = r.int32();
                                        break;
                                    case 5:
                                        m.day_of_week = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdScheduleInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AdScheduleInfo} AdScheduleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdScheduleInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdScheduleInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdScheduleInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.start_minute != null && m.hasOwnProperty("start_minute")) {
                                    switch (m.start_minute) {
                                    default:
                                        return "start_minute: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.end_minute != null && m.hasOwnProperty("end_minute")) {
                                    switch (m.end_minute) {
                                    default:
                                        return "end_minute: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.start_hour != null && m.hasOwnProperty("start_hour")) {
                                    p._start_hour = 1;
                                    if (!$util.isInteger(m.start_hour))
                                        return "start_hour: integer expected";
                                }
                                if (m.end_hour != null && m.hasOwnProperty("end_hour")) {
                                    p._end_hour = 1;
                                    if (!$util.isInteger(m.end_hour))
                                        return "end_hour: integer expected";
                                }
                                if (m.day_of_week != null && m.hasOwnProperty("day_of_week")) {
                                    switch (m.day_of_week) {
                                    default:
                                        return "day_of_week: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdScheduleInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AdScheduleInfo} AdScheduleInfo
                             */
                            AdScheduleInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AdScheduleInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AdScheduleInfo();
                                switch (d.start_minute) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.start_minute = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.start_minute = 1;
                                    break;
                                case "ZERO":
                                case 2:
                                    m.start_minute = 2;
                                    break;
                                case "FIFTEEN":
                                case 3:
                                    m.start_minute = 3;
                                    break;
                                case "THIRTY":
                                case 4:
                                    m.start_minute = 4;
                                    break;
                                case "FORTY_FIVE":
                                case 5:
                                    m.start_minute = 5;
                                    break;
                                }
                                switch (d.end_minute) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.end_minute = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.end_minute = 1;
                                    break;
                                case "ZERO":
                                case 2:
                                    m.end_minute = 2;
                                    break;
                                case "FIFTEEN":
                                case 3:
                                    m.end_minute = 3;
                                    break;
                                case "THIRTY":
                                case 4:
                                    m.end_minute = 4;
                                    break;
                                case "FORTY_FIVE":
                                case 5:
                                    m.end_minute = 5;
                                    break;
                                }
                                if (d.start_hour != null) {
                                    m.start_hour = d.start_hour | 0;
                                }
                                if (d.end_hour != null) {
                                    m.end_hour = d.end_hour | 0;
                                }
                                switch (d.day_of_week) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.day_of_week = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.day_of_week = 1;
                                    break;
                                case "MONDAY":
                                case 2:
                                    m.day_of_week = 2;
                                    break;
                                case "TUESDAY":
                                case 3:
                                    m.day_of_week = 3;
                                    break;
                                case "WEDNESDAY":
                                case 4:
                                    m.day_of_week = 4;
                                    break;
                                case "THURSDAY":
                                case 5:
                                    m.day_of_week = 5;
                                    break;
                                case "FRIDAY":
                                case 6:
                                    m.day_of_week = 6;
                                    break;
                                case "SATURDAY":
                                case 7:
                                    m.day_of_week = 7;
                                    break;
                                case "SUNDAY":
                                case 8:
                                    m.day_of_week = 8;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdScheduleInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AdScheduleInfo} m AdScheduleInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdScheduleInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.start_minute = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.end_minute = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.day_of_week = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.start_minute != null && m.hasOwnProperty("start_minute")) {
                                    d.start_minute = o.enums === String ? $root.google.ads.googleads.v11.enums.MinuteOfHourEnum.MinuteOfHour[m.start_minute] : m.start_minute;
                                }
                                if (m.end_minute != null && m.hasOwnProperty("end_minute")) {
                                    d.end_minute = o.enums === String ? $root.google.ads.googleads.v11.enums.MinuteOfHourEnum.MinuteOfHour[m.end_minute] : m.end_minute;
                                }
                                if (m.day_of_week != null && m.hasOwnProperty("day_of_week")) {
                                    d.day_of_week = o.enums === String ? $root.google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek[m.day_of_week] : m.day_of_week;
                                }
                                if (m.start_hour != null && m.hasOwnProperty("start_hour")) {
                                    d.start_hour = m.start_hour;
                                    if (o.oneofs)
                                        d._start_hour = "start_hour";
                                }
                                if (m.end_hour != null && m.hasOwnProperty("end_hour")) {
                                    d.end_hour = m.end_hour;
                                    if (o.oneofs)
                                        d._end_hour = "end_hour";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdScheduleInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AdScheduleInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdScheduleInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdScheduleInfo;
                        })();
    
                        common.AgeRangeInfo = (function() {
    
                            /**
                             * Properties of an AgeRangeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAgeRangeInfo
                             * @property {google.ads.googleads.v11.enums.AgeRangeTypeEnum.AgeRangeType|null} [type] AgeRangeInfo type
                             */
    
                            /**
                             * Constructs a new AgeRangeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AgeRangeInfo.
                             * @implements IAgeRangeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAgeRangeInfo=} [p] Properties to set
                             */
                            function AgeRangeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AgeRangeInfo type.
                             * @member {google.ads.googleads.v11.enums.AgeRangeTypeEnum.AgeRangeType} type
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @instance
                             */
                            AgeRangeInfo.prototype.type = 0;
    
                            /**
                             * Creates a new AgeRangeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeRangeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AgeRangeInfo} AgeRangeInfo instance
                             */
                            AgeRangeInfo.create = function create(properties) {
                                return new AgeRangeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AgeRangeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AgeRangeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeRangeInfo} m AgeRangeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeRangeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AgeRangeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AgeRangeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeRangeInfo} message AgeRangeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeRangeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AgeRangeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AgeRangeInfo} AgeRangeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeRangeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AgeRangeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AgeRangeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AgeRangeInfo} AgeRangeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeRangeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AgeRangeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AgeRangeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 503001:
                                    case 503002:
                                    case 503003:
                                    case 503004:
                                    case 503005:
                                    case 503006:
                                    case 503999:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AgeRangeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AgeRangeInfo} AgeRangeInfo
                             */
                            AgeRangeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AgeRangeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AgeRangeInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "AGE_RANGE_18_24":
                                case 503001:
                                    m.type = 503001;
                                    break;
                                case "AGE_RANGE_25_34":
                                case 503002:
                                    m.type = 503002;
                                    break;
                                case "AGE_RANGE_35_44":
                                case 503003:
                                    m.type = 503003;
                                    break;
                                case "AGE_RANGE_45_54":
                                case 503004:
                                    m.type = 503004;
                                    break;
                                case "AGE_RANGE_55_64":
                                case 503005:
                                    m.type = 503005;
                                    break;
                                case "AGE_RANGE_65_UP":
                                case 503006:
                                    m.type = 503006;
                                    break;
                                case "AGE_RANGE_UNDETERMINED":
                                case 503999:
                                    m.type = 503999;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AgeRangeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AgeRangeInfo} m AgeRangeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AgeRangeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.AgeRangeTypeEnum.AgeRangeType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AgeRangeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AgeRangeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AgeRangeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AgeRangeInfo;
                        })();
    
                        common.GenderInfo = (function() {
    
                            /**
                             * Properties of a GenderInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IGenderInfo
                             * @property {google.ads.googleads.v11.enums.GenderTypeEnum.GenderType|null} [type] GenderInfo type
                             */
    
                            /**
                             * Constructs a new GenderInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a GenderInfo.
                             * @implements IGenderInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IGenderInfo=} [p] Properties to set
                             */
                            function GenderInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenderInfo type.
                             * @member {google.ads.googleads.v11.enums.GenderTypeEnum.GenderType} type
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @instance
                             */
                            GenderInfo.prototype.type = 0;
    
                            /**
                             * Creates a new GenderInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGenderInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.GenderInfo} GenderInfo instance
                             */
                            GenderInfo.create = function create(properties) {
                                return new GenderInfo(properties);
                            };
    
                            /**
                             * Encodes the specified GenderInfo message. Does not implicitly {@link google.ads.googleads.v11.common.GenderInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGenderInfo} m GenderInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenderInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.GenderInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGenderInfo} message GenderInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenderInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.GenderInfo} GenderInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.GenderInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenderInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.GenderInfo} GenderInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenderInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenderInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 10:
                                    case 11:
                                    case 20:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenderInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.GenderInfo} GenderInfo
                             */
                            GenderInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.GenderInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.GenderInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "MALE":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "FEMALE":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "UNDETERMINED":
                                case 20:
                                    m.type = 20;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenderInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.GenderInfo} m GenderInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenderInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.GenderTypeEnum.GenderType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenderInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.GenderInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenderInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenderInfo;
                        })();
    
                        common.IncomeRangeInfo = (function() {
    
                            /**
                             * Properties of an IncomeRangeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IIncomeRangeInfo
                             * @property {google.ads.googleads.v11.enums.IncomeRangeTypeEnum.IncomeRangeType|null} [type] IncomeRangeInfo type
                             */
    
                            /**
                             * Constructs a new IncomeRangeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an IncomeRangeInfo.
                             * @implements IIncomeRangeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IIncomeRangeInfo=} [p] Properties to set
                             */
                            function IncomeRangeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * IncomeRangeInfo type.
                             * @member {google.ads.googleads.v11.enums.IncomeRangeTypeEnum.IncomeRangeType} type
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @instance
                             */
                            IncomeRangeInfo.prototype.type = 0;
    
                            /**
                             * Creates a new IncomeRangeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IIncomeRangeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.IncomeRangeInfo} IncomeRangeInfo instance
                             */
                            IncomeRangeInfo.create = function create(properties) {
                                return new IncomeRangeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified IncomeRangeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.IncomeRangeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IIncomeRangeInfo} m IncomeRangeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IncomeRangeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified IncomeRangeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.IncomeRangeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IIncomeRangeInfo} message IncomeRangeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IncomeRangeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an IncomeRangeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.IncomeRangeInfo} IncomeRangeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IncomeRangeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.IncomeRangeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an IncomeRangeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.IncomeRangeInfo} IncomeRangeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IncomeRangeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an IncomeRangeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            IncomeRangeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 510001:
                                    case 510002:
                                    case 510003:
                                    case 510004:
                                    case 510005:
                                    case 510006:
                                    case 510000:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an IncomeRangeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.IncomeRangeInfo} IncomeRangeInfo
                             */
                            IncomeRangeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.IncomeRangeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.IncomeRangeInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "INCOME_RANGE_0_50":
                                case 510001:
                                    m.type = 510001;
                                    break;
                                case "INCOME_RANGE_50_60":
                                case 510002:
                                    m.type = 510002;
                                    break;
                                case "INCOME_RANGE_60_70":
                                case 510003:
                                    m.type = 510003;
                                    break;
                                case "INCOME_RANGE_70_80":
                                case 510004:
                                    m.type = 510004;
                                    break;
                                case "INCOME_RANGE_80_90":
                                case 510005:
                                    m.type = 510005;
                                    break;
                                case "INCOME_RANGE_90_UP":
                                case 510006:
                                    m.type = 510006;
                                    break;
                                case "INCOME_RANGE_UNDETERMINED":
                                case 510000:
                                    m.type = 510000;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an IncomeRangeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IncomeRangeInfo} m IncomeRangeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            IncomeRangeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.IncomeRangeTypeEnum.IncomeRangeType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this IncomeRangeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.IncomeRangeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            IncomeRangeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return IncomeRangeInfo;
                        })();
    
                        common.ParentalStatusInfo = (function() {
    
                            /**
                             * Properties of a ParentalStatusInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IParentalStatusInfo
                             * @property {google.ads.googleads.v11.enums.ParentalStatusTypeEnum.ParentalStatusType|null} [type] ParentalStatusInfo type
                             */
    
                            /**
                             * Constructs a new ParentalStatusInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ParentalStatusInfo.
                             * @implements IParentalStatusInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IParentalStatusInfo=} [p] Properties to set
                             */
                            function ParentalStatusInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ParentalStatusInfo type.
                             * @member {google.ads.googleads.v11.enums.ParentalStatusTypeEnum.ParentalStatusType} type
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @instance
                             */
                            ParentalStatusInfo.prototype.type = 0;
    
                            /**
                             * Creates a new ParentalStatusInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IParentalStatusInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ParentalStatusInfo} ParentalStatusInfo instance
                             */
                            ParentalStatusInfo.create = function create(properties) {
                                return new ParentalStatusInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ParentalStatusInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ParentalStatusInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IParentalStatusInfo} m ParentalStatusInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ParentalStatusInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ParentalStatusInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IParentalStatusInfo} message ParentalStatusInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ParentalStatusInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ParentalStatusInfo} ParentalStatusInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ParentalStatusInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ParentalStatusInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ParentalStatusInfo} ParentalStatusInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ParentalStatusInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ParentalStatusInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 300:
                                    case 301:
                                    case 302:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ParentalStatusInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ParentalStatusInfo} ParentalStatusInfo
                             */
                            ParentalStatusInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ParentalStatusInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ParentalStatusInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "PARENT":
                                case 300:
                                    m.type = 300;
                                    break;
                                case "NOT_A_PARENT":
                                case 301:
                                    m.type = 301;
                                    break;
                                case "UNDETERMINED":
                                case 302:
                                    m.type = 302;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ParentalStatusInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ParentalStatusInfo} m ParentalStatusInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ParentalStatusInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.ParentalStatusTypeEnum.ParentalStatusType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ParentalStatusInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ParentalStatusInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ParentalStatusInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ParentalStatusInfo;
                        })();
    
                        common.YouTubeVideoInfo = (function() {
    
                            /**
                             * Properties of a YouTubeVideoInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IYouTubeVideoInfo
                             * @property {string|null} [video_id] YouTubeVideoInfo video_id
                             */
    
                            /**
                             * Constructs a new YouTubeVideoInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a YouTubeVideoInfo.
                             * @implements IYouTubeVideoInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IYouTubeVideoInfo=} [p] Properties to set
                             */
                            function YouTubeVideoInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * YouTubeVideoInfo video_id.
                             * @member {string|null|undefined} video_id
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @instance
                             */
                            YouTubeVideoInfo.prototype.video_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * YouTubeVideoInfo _video_id.
                             * @member {"video_id"|undefined} _video_id
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @instance
                             */
                            Object.defineProperty(YouTubeVideoInfo.prototype, "_video_id", {
                                get: $util.oneOfGetter($oneOfFields = ["video_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new YouTubeVideoInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IYouTubeVideoInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.YouTubeVideoInfo} YouTubeVideoInfo instance
                             */
                            YouTubeVideoInfo.create = function create(properties) {
                                return new YouTubeVideoInfo(properties);
                            };
    
                            /**
                             * Encodes the specified YouTubeVideoInfo message. Does not implicitly {@link google.ads.googleads.v11.common.YouTubeVideoInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IYouTubeVideoInfo} m YouTubeVideoInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YouTubeVideoInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.video_id != null && Object.hasOwnProperty.call(m, "video_id"))
                                    w.uint32(18).string(m.video_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified YouTubeVideoInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.YouTubeVideoInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IYouTubeVideoInfo} message YouTubeVideoInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YouTubeVideoInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a YouTubeVideoInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.YouTubeVideoInfo} YouTubeVideoInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YouTubeVideoInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.YouTubeVideoInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.video_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a YouTubeVideoInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.YouTubeVideoInfo} YouTubeVideoInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YouTubeVideoInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a YouTubeVideoInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            YouTubeVideoInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.video_id != null && m.hasOwnProperty("video_id")) {
                                    p._video_id = 1;
                                    if (!$util.isString(m.video_id))
                                        return "video_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a YouTubeVideoInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.YouTubeVideoInfo} YouTubeVideoInfo
                             */
                            YouTubeVideoInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.YouTubeVideoInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.YouTubeVideoInfo();
                                if (d.video_id != null) {
                                    m.video_id = String(d.video_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a YouTubeVideoInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.YouTubeVideoInfo} m YouTubeVideoInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            YouTubeVideoInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.video_id != null && m.hasOwnProperty("video_id")) {
                                    d.video_id = m.video_id;
                                    if (o.oneofs)
                                        d._video_id = "video_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this YouTubeVideoInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.YouTubeVideoInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            YouTubeVideoInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return YouTubeVideoInfo;
                        })();
    
                        common.YouTubeChannelInfo = (function() {
    
                            /**
                             * Properties of a YouTubeChannelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IYouTubeChannelInfo
                             * @property {string|null} [channel_id] YouTubeChannelInfo channel_id
                             */
    
                            /**
                             * Constructs a new YouTubeChannelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a YouTubeChannelInfo.
                             * @implements IYouTubeChannelInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IYouTubeChannelInfo=} [p] Properties to set
                             */
                            function YouTubeChannelInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * YouTubeChannelInfo channel_id.
                             * @member {string|null|undefined} channel_id
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @instance
                             */
                            YouTubeChannelInfo.prototype.channel_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * YouTubeChannelInfo _channel_id.
                             * @member {"channel_id"|undefined} _channel_id
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @instance
                             */
                            Object.defineProperty(YouTubeChannelInfo.prototype, "_channel_id", {
                                get: $util.oneOfGetter($oneOfFields = ["channel_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new YouTubeChannelInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IYouTubeChannelInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.YouTubeChannelInfo} YouTubeChannelInfo instance
                             */
                            YouTubeChannelInfo.create = function create(properties) {
                                return new YouTubeChannelInfo(properties);
                            };
    
                            /**
                             * Encodes the specified YouTubeChannelInfo message. Does not implicitly {@link google.ads.googleads.v11.common.YouTubeChannelInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IYouTubeChannelInfo} m YouTubeChannelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YouTubeChannelInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.channel_id != null && Object.hasOwnProperty.call(m, "channel_id"))
                                    w.uint32(18).string(m.channel_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified YouTubeChannelInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.YouTubeChannelInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IYouTubeChannelInfo} message YouTubeChannelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YouTubeChannelInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a YouTubeChannelInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.YouTubeChannelInfo} YouTubeChannelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YouTubeChannelInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.YouTubeChannelInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.channel_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a YouTubeChannelInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.YouTubeChannelInfo} YouTubeChannelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YouTubeChannelInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a YouTubeChannelInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            YouTubeChannelInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.channel_id != null && m.hasOwnProperty("channel_id")) {
                                    p._channel_id = 1;
                                    if (!$util.isString(m.channel_id))
                                        return "channel_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a YouTubeChannelInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.YouTubeChannelInfo} YouTubeChannelInfo
                             */
                            YouTubeChannelInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.YouTubeChannelInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.YouTubeChannelInfo();
                                if (d.channel_id != null) {
                                    m.channel_id = String(d.channel_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a YouTubeChannelInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.YouTubeChannelInfo} m YouTubeChannelInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            YouTubeChannelInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.channel_id != null && m.hasOwnProperty("channel_id")) {
                                    d.channel_id = m.channel_id;
                                    if (o.oneofs)
                                        d._channel_id = "channel_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this YouTubeChannelInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.YouTubeChannelInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            YouTubeChannelInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return YouTubeChannelInfo;
                        })();
    
                        common.UserListInfo = (function() {
    
                            /**
                             * Properties of a UserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListInfo
                             * @property {string|null} [user_list] UserListInfo user_list
                             */
    
                            /**
                             * Constructs a new UserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListInfo.
                             * @implements IUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListInfo=} [p] Properties to set
                             */
                            function UserListInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListInfo user_list.
                             * @member {string|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @instance
                             */
                            UserListInfo.prototype.user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListInfo _user_list.
                             * @member {"user_list"|undefined} _user_list
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @instance
                             */
                            Object.defineProperty(UserListInfo.prototype, "_user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListInfo} UserListInfo instance
                             */
                            UserListInfo.create = function create(properties) {
                                return new UserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListInfo} m UserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    w.uint32(18).string(m.user_list);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListInfo} message UserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListInfo} UserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.user_list = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListInfo} UserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    p._user_list = 1;
                                    if (!$util.isString(m.user_list))
                                        return "user_list: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListInfo} UserListInfo
                             */
                            UserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListInfo();
                                if (d.user_list != null) {
                                    m.user_list = String(d.user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListInfo} m UserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = m.user_list;
                                    if (o.oneofs)
                                        d._user_list = "user_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListInfo;
                        })();
    
                        common.ProximityInfo = (function() {
    
                            /**
                             * Properties of a ProximityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IProximityInfo
                             * @property {google.ads.googleads.v11.common.IGeoPointInfo|null} [geo_point] ProximityInfo geo_point
                             * @property {number|null} [radius] ProximityInfo radius
                             * @property {google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum.ProximityRadiusUnits|null} [radius_units] ProximityInfo radius_units
                             * @property {google.ads.googleads.v11.common.IAddressInfo|null} [address] ProximityInfo address
                             */
    
                            /**
                             * Constructs a new ProximityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ProximityInfo.
                             * @implements IProximityInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IProximityInfo=} [p] Properties to set
                             */
                            function ProximityInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProximityInfo geo_point.
                             * @member {google.ads.googleads.v11.common.IGeoPointInfo|null|undefined} geo_point
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @instance
                             */
                            ProximityInfo.prototype.geo_point = null;
    
                            /**
                             * ProximityInfo radius.
                             * @member {number|null|undefined} radius
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @instance
                             */
                            ProximityInfo.prototype.radius = null;
    
                            /**
                             * ProximityInfo radius_units.
                             * @member {google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum.ProximityRadiusUnits} radius_units
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @instance
                             */
                            ProximityInfo.prototype.radius_units = 0;
    
                            /**
                             * ProximityInfo address.
                             * @member {google.ads.googleads.v11.common.IAddressInfo|null|undefined} address
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @instance
                             */
                            ProximityInfo.prototype.address = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProximityInfo _radius.
                             * @member {"radius"|undefined} _radius
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @instance
                             */
                            Object.defineProperty(ProximityInfo.prototype, "_radius", {
                                get: $util.oneOfGetter($oneOfFields = ["radius"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProximityInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProximityInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ProximityInfo} ProximityInfo instance
                             */
                            ProximityInfo.create = function create(properties) {
                                return new ProximityInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ProximityInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ProximityInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProximityInfo} m ProximityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProximityInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.geo_point != null && Object.hasOwnProperty.call(m, "geo_point"))
                                    $root.google.ads.googleads.v11.common.GeoPointInfo.encode(m.geo_point, w.uint32(10).fork()).ldelim();
                                if (m.radius_units != null && Object.hasOwnProperty.call(m, "radius_units"))
                                    w.uint32(24).int32(m.radius_units);
                                if (m.address != null && Object.hasOwnProperty.call(m, "address"))
                                    $root.google.ads.googleads.v11.common.AddressInfo.encode(m.address, w.uint32(34).fork()).ldelim();
                                if (m.radius != null && Object.hasOwnProperty.call(m, "radius"))
                                    w.uint32(41).double(m.radius);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProximityInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ProximityInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IProximityInfo} message ProximityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProximityInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProximityInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ProximityInfo} ProximityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProximityInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ProximityInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.geo_point = $root.google.ads.googleads.v11.common.GeoPointInfo.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.radius = r.double();
                                        break;
                                    case 3:
                                        m.radius_units = r.int32();
                                        break;
                                    case 4:
                                        m.address = $root.google.ads.googleads.v11.common.AddressInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProximityInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ProximityInfo} ProximityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProximityInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProximityInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProximityInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.geo_point != null && m.hasOwnProperty("geo_point")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GeoPointInfo.verify(m.geo_point);
                                        if (e)
                                            return "geo_point." + e;
                                    }
                                }
                                if (m.radius != null && m.hasOwnProperty("radius")) {
                                    p._radius = 1;
                                    if (typeof m.radius !== "number")
                                        return "radius: number expected";
                                }
                                if (m.radius_units != null && m.hasOwnProperty("radius_units")) {
                                    switch (m.radius_units) {
                                    default:
                                        return "radius_units: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AddressInfo.verify(m.address);
                                        if (e)
                                            return "address." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProximityInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ProximityInfo} ProximityInfo
                             */
                            ProximityInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ProximityInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ProximityInfo();
                                if (d.geo_point != null) {
                                    if (typeof d.geo_point !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ProximityInfo.geo_point: object expected");
                                    m.geo_point = $root.google.ads.googleads.v11.common.GeoPointInfo.fromObject(d.geo_point);
                                }
                                if (d.radius != null) {
                                    m.radius = Number(d.radius);
                                }
                                switch (d.radius_units) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.radius_units = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.radius_units = 1;
                                    break;
                                case "MILES":
                                case 2:
                                    m.radius_units = 2;
                                    break;
                                case "KILOMETERS":
                                case 3:
                                    m.radius_units = 3;
                                    break;
                                }
                                if (d.address != null) {
                                    if (typeof d.address !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ProximityInfo.address: object expected");
                                    m.address = $root.google.ads.googleads.v11.common.AddressInfo.fromObject(d.address);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProximityInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ProximityInfo} m ProximityInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProximityInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.geo_point = null;
                                    d.radius_units = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.address = null;
                                }
                                if (m.geo_point != null && m.hasOwnProperty("geo_point")) {
                                    d.geo_point = $root.google.ads.googleads.v11.common.GeoPointInfo.toObject(m.geo_point, o);
                                }
                                if (m.radius_units != null && m.hasOwnProperty("radius_units")) {
                                    d.radius_units = o.enums === String ? $root.google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum.ProximityRadiusUnits[m.radius_units] : m.radius_units;
                                }
                                if (m.address != null && m.hasOwnProperty("address")) {
                                    d.address = $root.google.ads.googleads.v11.common.AddressInfo.toObject(m.address, o);
                                }
                                if (m.radius != null && m.hasOwnProperty("radius")) {
                                    d.radius = o.json && !isFinite(m.radius) ? String(m.radius) : m.radius;
                                    if (o.oneofs)
                                        d._radius = "radius";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProximityInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ProximityInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProximityInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProximityInfo;
                        })();
    
                        common.GeoPointInfo = (function() {
    
                            /**
                             * Properties of a GeoPointInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IGeoPointInfo
                             * @property {number|null} [longitude_in_micro_degrees] GeoPointInfo longitude_in_micro_degrees
                             * @property {number|null} [latitude_in_micro_degrees] GeoPointInfo latitude_in_micro_degrees
                             */
    
                            /**
                             * Constructs a new GeoPointInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a GeoPointInfo.
                             * @implements IGeoPointInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IGeoPointInfo=} [p] Properties to set
                             */
                            function GeoPointInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GeoPointInfo longitude_in_micro_degrees.
                             * @member {number|null|undefined} longitude_in_micro_degrees
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @instance
                             */
                            GeoPointInfo.prototype.longitude_in_micro_degrees = null;
    
                            /**
                             * GeoPointInfo latitude_in_micro_degrees.
                             * @member {number|null|undefined} latitude_in_micro_degrees
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @instance
                             */
                            GeoPointInfo.prototype.latitude_in_micro_degrees = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GeoPointInfo _longitude_in_micro_degrees.
                             * @member {"longitude_in_micro_degrees"|undefined} _longitude_in_micro_degrees
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @instance
                             */
                            Object.defineProperty(GeoPointInfo.prototype, "_longitude_in_micro_degrees", {
                                get: $util.oneOfGetter($oneOfFields = ["longitude_in_micro_degrees"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoPointInfo _latitude_in_micro_degrees.
                             * @member {"latitude_in_micro_degrees"|undefined} _latitude_in_micro_degrees
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @instance
                             */
                            Object.defineProperty(GeoPointInfo.prototype, "_latitude_in_micro_degrees", {
                                get: $util.oneOfGetter($oneOfFields = ["latitude_in_micro_degrees"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GeoPointInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGeoPointInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.GeoPointInfo} GeoPointInfo instance
                             */
                            GeoPointInfo.create = function create(properties) {
                                return new GeoPointInfo(properties);
                            };
    
                            /**
                             * Encodes the specified GeoPointInfo message. Does not implicitly {@link google.ads.googleads.v11.common.GeoPointInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGeoPointInfo} m GeoPointInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoPointInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.longitude_in_micro_degrees != null && Object.hasOwnProperty.call(m, "longitude_in_micro_degrees"))
                                    w.uint32(24).int32(m.longitude_in_micro_degrees);
                                if (m.latitude_in_micro_degrees != null && Object.hasOwnProperty.call(m, "latitude_in_micro_degrees"))
                                    w.uint32(32).int32(m.latitude_in_micro_degrees);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeoPointInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.GeoPointInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IGeoPointInfo} message GeoPointInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoPointInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeoPointInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.GeoPointInfo} GeoPointInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoPointInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.GeoPointInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.longitude_in_micro_degrees = r.int32();
                                        break;
                                    case 4:
                                        m.latitude_in_micro_degrees = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeoPointInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.GeoPointInfo} GeoPointInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoPointInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeoPointInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeoPointInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.longitude_in_micro_degrees != null && m.hasOwnProperty("longitude_in_micro_degrees")) {
                                    p._longitude_in_micro_degrees = 1;
                                    if (!$util.isInteger(m.longitude_in_micro_degrees))
                                        return "longitude_in_micro_degrees: integer expected";
                                }
                                if (m.latitude_in_micro_degrees != null && m.hasOwnProperty("latitude_in_micro_degrees")) {
                                    p._latitude_in_micro_degrees = 1;
                                    if (!$util.isInteger(m.latitude_in_micro_degrees))
                                        return "latitude_in_micro_degrees: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GeoPointInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.GeoPointInfo} GeoPointInfo
                             */
                            GeoPointInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.GeoPointInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.GeoPointInfo();
                                if (d.longitude_in_micro_degrees != null) {
                                    m.longitude_in_micro_degrees = d.longitude_in_micro_degrees | 0;
                                }
                                if (d.latitude_in_micro_degrees != null) {
                                    m.latitude_in_micro_degrees = d.latitude_in_micro_degrees | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GeoPointInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.GeoPointInfo} m GeoPointInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeoPointInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.longitude_in_micro_degrees != null && m.hasOwnProperty("longitude_in_micro_degrees")) {
                                    d.longitude_in_micro_degrees = m.longitude_in_micro_degrees;
                                    if (o.oneofs)
                                        d._longitude_in_micro_degrees = "longitude_in_micro_degrees";
                                }
                                if (m.latitude_in_micro_degrees != null && m.hasOwnProperty("latitude_in_micro_degrees")) {
                                    d.latitude_in_micro_degrees = m.latitude_in_micro_degrees;
                                    if (o.oneofs)
                                        d._latitude_in_micro_degrees = "latitude_in_micro_degrees";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GeoPointInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.GeoPointInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeoPointInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GeoPointInfo;
                        })();
    
                        common.AddressInfo = (function() {
    
                            /**
                             * Properties of an AddressInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAddressInfo
                             * @property {string|null} [postal_code] AddressInfo postal_code
                             * @property {string|null} [province_code] AddressInfo province_code
                             * @property {string|null} [country_code] AddressInfo country_code
                             * @property {string|null} [province_name] AddressInfo province_name
                             * @property {string|null} [street_address] AddressInfo street_address
                             * @property {string|null} [street_address2] AddressInfo street_address2
                             * @property {string|null} [city_name] AddressInfo city_name
                             */
    
                            /**
                             * Constructs a new AddressInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AddressInfo.
                             * @implements IAddressInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAddressInfo=} [p] Properties to set
                             */
                            function AddressInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AddressInfo postal_code.
                             * @member {string|null|undefined} postal_code
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            AddressInfo.prototype.postal_code = null;
    
                            /**
                             * AddressInfo province_code.
                             * @member {string|null|undefined} province_code
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            AddressInfo.prototype.province_code = null;
    
                            /**
                             * AddressInfo country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            AddressInfo.prototype.country_code = null;
    
                            /**
                             * AddressInfo province_name.
                             * @member {string|null|undefined} province_name
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            AddressInfo.prototype.province_name = null;
    
                            /**
                             * AddressInfo street_address.
                             * @member {string|null|undefined} street_address
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            AddressInfo.prototype.street_address = null;
    
                            /**
                             * AddressInfo street_address2.
                             * @member {string|null|undefined} street_address2
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            AddressInfo.prototype.street_address2 = null;
    
                            /**
                             * AddressInfo city_name.
                             * @member {string|null|undefined} city_name
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            AddressInfo.prototype.city_name = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AddressInfo _postal_code.
                             * @member {"postal_code"|undefined} _postal_code
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            Object.defineProperty(AddressInfo.prototype, "_postal_code", {
                                get: $util.oneOfGetter($oneOfFields = ["postal_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AddressInfo _province_code.
                             * @member {"province_code"|undefined} _province_code
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            Object.defineProperty(AddressInfo.prototype, "_province_code", {
                                get: $util.oneOfGetter($oneOfFields = ["province_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AddressInfo _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            Object.defineProperty(AddressInfo.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AddressInfo _province_name.
                             * @member {"province_name"|undefined} _province_name
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            Object.defineProperty(AddressInfo.prototype, "_province_name", {
                                get: $util.oneOfGetter($oneOfFields = ["province_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AddressInfo _street_address.
                             * @member {"street_address"|undefined} _street_address
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            Object.defineProperty(AddressInfo.prototype, "_street_address", {
                                get: $util.oneOfGetter($oneOfFields = ["street_address"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AddressInfo _street_address2.
                             * @member {"street_address2"|undefined} _street_address2
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            Object.defineProperty(AddressInfo.prototype, "_street_address2", {
                                get: $util.oneOfGetter($oneOfFields = ["street_address2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AddressInfo _city_name.
                             * @member {"city_name"|undefined} _city_name
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             */
                            Object.defineProperty(AddressInfo.prototype, "_city_name", {
                                get: $util.oneOfGetter($oneOfFields = ["city_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AddressInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAddressInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AddressInfo} AddressInfo instance
                             */
                            AddressInfo.create = function create(properties) {
                                return new AddressInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AddressInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AddressInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAddressInfo} m AddressInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddressInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.postal_code != null && Object.hasOwnProperty.call(m, "postal_code"))
                                    w.uint32(66).string(m.postal_code);
                                if (m.province_code != null && Object.hasOwnProperty.call(m, "province_code"))
                                    w.uint32(74).string(m.province_code);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(82).string(m.country_code);
                                if (m.province_name != null && Object.hasOwnProperty.call(m, "province_name"))
                                    w.uint32(90).string(m.province_name);
                                if (m.street_address != null && Object.hasOwnProperty.call(m, "street_address"))
                                    w.uint32(98).string(m.street_address);
                                if (m.street_address2 != null && Object.hasOwnProperty.call(m, "street_address2"))
                                    w.uint32(106).string(m.street_address2);
                                if (m.city_name != null && Object.hasOwnProperty.call(m, "city_name"))
                                    w.uint32(114).string(m.city_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AddressInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AddressInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAddressInfo} message AddressInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddressInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AddressInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AddressInfo} AddressInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddressInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AddressInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 8:
                                        m.postal_code = r.string();
                                        break;
                                    case 9:
                                        m.province_code = r.string();
                                        break;
                                    case 10:
                                        m.country_code = r.string();
                                        break;
                                    case 11:
                                        m.province_name = r.string();
                                        break;
                                    case 12:
                                        m.street_address = r.string();
                                        break;
                                    case 13:
                                        m.street_address2 = r.string();
                                        break;
                                    case 14:
                                        m.city_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AddressInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AddressInfo} AddressInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddressInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AddressInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AddressInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    p._postal_code = 1;
                                    if (!$util.isString(m.postal_code))
                                        return "postal_code: string expected";
                                }
                                if (m.province_code != null && m.hasOwnProperty("province_code")) {
                                    p._province_code = 1;
                                    if (!$util.isString(m.province_code))
                                        return "province_code: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.province_name != null && m.hasOwnProperty("province_name")) {
                                    p._province_name = 1;
                                    if (!$util.isString(m.province_name))
                                        return "province_name: string expected";
                                }
                                if (m.street_address != null && m.hasOwnProperty("street_address")) {
                                    p._street_address = 1;
                                    if (!$util.isString(m.street_address))
                                        return "street_address: string expected";
                                }
                                if (m.street_address2 != null && m.hasOwnProperty("street_address2")) {
                                    p._street_address2 = 1;
                                    if (!$util.isString(m.street_address2))
                                        return "street_address2: string expected";
                                }
                                if (m.city_name != null && m.hasOwnProperty("city_name")) {
                                    p._city_name = 1;
                                    if (!$util.isString(m.city_name))
                                        return "city_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AddressInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AddressInfo} AddressInfo
                             */
                            AddressInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AddressInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AddressInfo();
                                if (d.postal_code != null) {
                                    m.postal_code = String(d.postal_code);
                                }
                                if (d.province_code != null) {
                                    m.province_code = String(d.province_code);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.province_name != null) {
                                    m.province_name = String(d.province_name);
                                }
                                if (d.street_address != null) {
                                    m.street_address = String(d.street_address);
                                }
                                if (d.street_address2 != null) {
                                    m.street_address2 = String(d.street_address2);
                                }
                                if (d.city_name != null) {
                                    m.city_name = String(d.city_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AddressInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AddressInfo} m AddressInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AddressInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    d.postal_code = m.postal_code;
                                    if (o.oneofs)
                                        d._postal_code = "postal_code";
                                }
                                if (m.province_code != null && m.hasOwnProperty("province_code")) {
                                    d.province_code = m.province_code;
                                    if (o.oneofs)
                                        d._province_code = "province_code";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.province_name != null && m.hasOwnProperty("province_name")) {
                                    d.province_name = m.province_name;
                                    if (o.oneofs)
                                        d._province_name = "province_name";
                                }
                                if (m.street_address != null && m.hasOwnProperty("street_address")) {
                                    d.street_address = m.street_address;
                                    if (o.oneofs)
                                        d._street_address = "street_address";
                                }
                                if (m.street_address2 != null && m.hasOwnProperty("street_address2")) {
                                    d.street_address2 = m.street_address2;
                                    if (o.oneofs)
                                        d._street_address2 = "street_address2";
                                }
                                if (m.city_name != null && m.hasOwnProperty("city_name")) {
                                    d.city_name = m.city_name;
                                    if (o.oneofs)
                                        d._city_name = "city_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AddressInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AddressInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AddressInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AddressInfo;
                        })();
    
                        common.TopicInfo = (function() {
    
                            /**
                             * Properties of a TopicInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITopicInfo
                             * @property {string|null} [topic_constant] TopicInfo topic_constant
                             * @property {Array.<string>|null} [path] TopicInfo path
                             */
    
                            /**
                             * Constructs a new TopicInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TopicInfo.
                             * @implements ITopicInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITopicInfo=} [p] Properties to set
                             */
                            function TopicInfo(p) {
                                this.path = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TopicInfo topic_constant.
                             * @member {string|null|undefined} topic_constant
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @instance
                             */
                            TopicInfo.prototype.topic_constant = null;
    
                            /**
                             * TopicInfo path.
                             * @member {Array.<string>} path
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @instance
                             */
                            TopicInfo.prototype.path = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TopicInfo _topic_constant.
                             * @member {"topic_constant"|undefined} _topic_constant
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @instance
                             */
                            Object.defineProperty(TopicInfo.prototype, "_topic_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["topic_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TopicInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ITopicInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TopicInfo} TopicInfo instance
                             */
                            TopicInfo.create = function create(properties) {
                                return new TopicInfo(properties);
                            };
    
                            /**
                             * Encodes the specified TopicInfo message. Does not implicitly {@link google.ads.googleads.v11.common.TopicInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ITopicInfo} m TopicInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TopicInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.topic_constant != null && Object.hasOwnProperty.call(m, "topic_constant"))
                                    w.uint32(26).string(m.topic_constant);
                                if (m.path != null && m.path.length) {
                                    for (var i = 0; i < m.path.length; ++i)
                                        w.uint32(34).string(m.path[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified TopicInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TopicInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ITopicInfo} message TopicInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TopicInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TopicInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TopicInfo} TopicInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TopicInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TopicInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.topic_constant = r.string();
                                        break;
                                    case 4:
                                        if (!(m.path && m.path.length))
                                            m.path = [];
                                        m.path.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TopicInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TopicInfo} TopicInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TopicInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TopicInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TopicInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.topic_constant != null && m.hasOwnProperty("topic_constant")) {
                                    p._topic_constant = 1;
                                    if (!$util.isString(m.topic_constant))
                                        return "topic_constant: string expected";
                                }
                                if (m.path != null && m.hasOwnProperty("path")) {
                                    if (!Array.isArray(m.path))
                                        return "path: array expected";
                                    for (var i = 0; i < m.path.length; ++i) {
                                        if (!$util.isString(m.path[i]))
                                            return "path: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TopicInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TopicInfo} TopicInfo
                             */
                            TopicInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TopicInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TopicInfo();
                                if (d.topic_constant != null) {
                                    m.topic_constant = String(d.topic_constant);
                                }
                                if (d.path) {
                                    if (!Array.isArray(d.path))
                                        throw TypeError(".google.ads.googleads.v11.common.TopicInfo.path: array expected");
                                    m.path = [];
                                    for (var i = 0; i < d.path.length; ++i) {
                                        m.path[i] = String(d.path[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TopicInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.TopicInfo} m TopicInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TopicInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.path = [];
                                }
                                if (m.topic_constant != null && m.hasOwnProperty("topic_constant")) {
                                    d.topic_constant = m.topic_constant;
                                    if (o.oneofs)
                                        d._topic_constant = "topic_constant";
                                }
                                if (m.path && m.path.length) {
                                    d.path = [];
                                    for (var j = 0; j < m.path.length; ++j) {
                                        d.path[j] = m.path[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TopicInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TopicInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TopicInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TopicInfo;
                        })();
    
                        common.LanguageInfo = (function() {
    
                            /**
                             * Properties of a LanguageInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILanguageInfo
                             * @property {string|null} [language_constant] LanguageInfo language_constant
                             */
    
                            /**
                             * Constructs a new LanguageInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LanguageInfo.
                             * @implements ILanguageInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILanguageInfo=} [p] Properties to set
                             */
                            function LanguageInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LanguageInfo language_constant.
                             * @member {string|null|undefined} language_constant
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @instance
                             */
                            LanguageInfo.prototype.language_constant = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LanguageInfo _language_constant.
                             * @member {"language_constant"|undefined} _language_constant
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @instance
                             */
                            Object.defineProperty(LanguageInfo.prototype, "_language_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["language_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LanguageInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILanguageInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LanguageInfo} LanguageInfo instance
                             */
                            LanguageInfo.create = function create(properties) {
                                return new LanguageInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LanguageInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LanguageInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILanguageInfo} m LanguageInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LanguageInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.language_constant != null && Object.hasOwnProperty.call(m, "language_constant"))
                                    w.uint32(18).string(m.language_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LanguageInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LanguageInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILanguageInfo} message LanguageInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LanguageInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LanguageInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LanguageInfo} LanguageInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LanguageInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LanguageInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.language_constant = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LanguageInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LanguageInfo} LanguageInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LanguageInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LanguageInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LanguageInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.language_constant != null && m.hasOwnProperty("language_constant")) {
                                    p._language_constant = 1;
                                    if (!$util.isString(m.language_constant))
                                        return "language_constant: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LanguageInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LanguageInfo} LanguageInfo
                             */
                            LanguageInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LanguageInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LanguageInfo();
                                if (d.language_constant != null) {
                                    m.language_constant = String(d.language_constant);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LanguageInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LanguageInfo} m LanguageInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LanguageInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.language_constant != null && m.hasOwnProperty("language_constant")) {
                                    d.language_constant = m.language_constant;
                                    if (o.oneofs)
                                        d._language_constant = "language_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LanguageInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LanguageInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LanguageInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LanguageInfo;
                        })();
    
                        common.IpBlockInfo = (function() {
    
                            /**
                             * Properties of an IpBlockInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IIpBlockInfo
                             * @property {string|null} [ip_address] IpBlockInfo ip_address
                             */
    
                            /**
                             * Constructs a new IpBlockInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an IpBlockInfo.
                             * @implements IIpBlockInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IIpBlockInfo=} [p] Properties to set
                             */
                            function IpBlockInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * IpBlockInfo ip_address.
                             * @member {string|null|undefined} ip_address
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @instance
                             */
                            IpBlockInfo.prototype.ip_address = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * IpBlockInfo _ip_address.
                             * @member {"ip_address"|undefined} _ip_address
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @instance
                             */
                            Object.defineProperty(IpBlockInfo.prototype, "_ip_address", {
                                get: $util.oneOfGetter($oneOfFields = ["ip_address"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new IpBlockInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IIpBlockInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.IpBlockInfo} IpBlockInfo instance
                             */
                            IpBlockInfo.create = function create(properties) {
                                return new IpBlockInfo(properties);
                            };
    
                            /**
                             * Encodes the specified IpBlockInfo message. Does not implicitly {@link google.ads.googleads.v11.common.IpBlockInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IIpBlockInfo} m IpBlockInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IpBlockInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ip_address != null && Object.hasOwnProperty.call(m, "ip_address"))
                                    w.uint32(18).string(m.ip_address);
                                return w;
                            };
    
                            /**
                             * Encodes the specified IpBlockInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.IpBlockInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IIpBlockInfo} message IpBlockInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IpBlockInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an IpBlockInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.IpBlockInfo} IpBlockInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IpBlockInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.IpBlockInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.ip_address = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an IpBlockInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.IpBlockInfo} IpBlockInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IpBlockInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an IpBlockInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            IpBlockInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.ip_address != null && m.hasOwnProperty("ip_address")) {
                                    p._ip_address = 1;
                                    if (!$util.isString(m.ip_address))
                                        return "ip_address: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an IpBlockInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.IpBlockInfo} IpBlockInfo
                             */
                            IpBlockInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.IpBlockInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.IpBlockInfo();
                                if (d.ip_address != null) {
                                    m.ip_address = String(d.ip_address);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an IpBlockInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IpBlockInfo} m IpBlockInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            IpBlockInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.ip_address != null && m.hasOwnProperty("ip_address")) {
                                    d.ip_address = m.ip_address;
                                    if (o.oneofs)
                                        d._ip_address = "ip_address";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this IpBlockInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.IpBlockInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            IpBlockInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return IpBlockInfo;
                        })();
    
                        common.ContentLabelInfo = (function() {
    
                            /**
                             * Properties of a ContentLabelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IContentLabelInfo
                             * @property {google.ads.googleads.v11.enums.ContentLabelTypeEnum.ContentLabelType|null} [type] ContentLabelInfo type
                             */
    
                            /**
                             * Constructs a new ContentLabelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ContentLabelInfo.
                             * @implements IContentLabelInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IContentLabelInfo=} [p] Properties to set
                             */
                            function ContentLabelInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ContentLabelInfo type.
                             * @member {google.ads.googleads.v11.enums.ContentLabelTypeEnum.ContentLabelType} type
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @instance
                             */
                            ContentLabelInfo.prototype.type = 0;
    
                            /**
                             * Creates a new ContentLabelInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IContentLabelInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ContentLabelInfo} ContentLabelInfo instance
                             */
                            ContentLabelInfo.create = function create(properties) {
                                return new ContentLabelInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ContentLabelInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ContentLabelInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IContentLabelInfo} m ContentLabelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ContentLabelInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ContentLabelInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ContentLabelInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IContentLabelInfo} message ContentLabelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ContentLabelInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ContentLabelInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ContentLabelInfo} ContentLabelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ContentLabelInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ContentLabelInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ContentLabelInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ContentLabelInfo} ContentLabelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ContentLabelInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ContentLabelInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ContentLabelInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ContentLabelInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ContentLabelInfo} ContentLabelInfo
                             */
                            ContentLabelInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ContentLabelInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ContentLabelInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "SEXUALLY_SUGGESTIVE":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "BELOW_THE_FOLD":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "PARKED_DOMAIN":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "JUVENILE":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PROFANITY":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TRAGEDY":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "VIDEO":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "VIDEO_RATING_DV_G":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "VIDEO_RATING_DV_PG":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "VIDEO_RATING_DV_T":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "VIDEO_RATING_DV_MA":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "VIDEO_NOT_YET_RATED":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "EMBEDDED_VIDEO":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "LIVE_STREAMING_VIDEO":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "SOCIAL_ISSUES":
                                case 17:
                                    m.type = 17;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ContentLabelInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ContentLabelInfo} m ContentLabelInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ContentLabelInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.ContentLabelTypeEnum.ContentLabelType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ContentLabelInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ContentLabelInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ContentLabelInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ContentLabelInfo;
                        })();
    
                        common.CarrierInfo = (function() {
    
                            /**
                             * Properties of a CarrierInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICarrierInfo
                             * @property {string|null} [carrier_constant] CarrierInfo carrier_constant
                             */
    
                            /**
                             * Constructs a new CarrierInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CarrierInfo.
                             * @implements ICarrierInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICarrierInfo=} [p] Properties to set
                             */
                            function CarrierInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CarrierInfo carrier_constant.
                             * @member {string|null|undefined} carrier_constant
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @instance
                             */
                            CarrierInfo.prototype.carrier_constant = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CarrierInfo _carrier_constant.
                             * @member {"carrier_constant"|undefined} _carrier_constant
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @instance
                             */
                            Object.defineProperty(CarrierInfo.prototype, "_carrier_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["carrier_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CarrierInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICarrierInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CarrierInfo} CarrierInfo instance
                             */
                            CarrierInfo.create = function create(properties) {
                                return new CarrierInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CarrierInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CarrierInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICarrierInfo} m CarrierInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CarrierInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.carrier_constant != null && Object.hasOwnProperty.call(m, "carrier_constant"))
                                    w.uint32(18).string(m.carrier_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CarrierInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CarrierInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICarrierInfo} message CarrierInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CarrierInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CarrierInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CarrierInfo} CarrierInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CarrierInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CarrierInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.carrier_constant = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CarrierInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CarrierInfo} CarrierInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CarrierInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CarrierInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CarrierInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.carrier_constant != null && m.hasOwnProperty("carrier_constant")) {
                                    p._carrier_constant = 1;
                                    if (!$util.isString(m.carrier_constant))
                                        return "carrier_constant: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CarrierInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CarrierInfo} CarrierInfo
                             */
                            CarrierInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CarrierInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CarrierInfo();
                                if (d.carrier_constant != null) {
                                    m.carrier_constant = String(d.carrier_constant);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CarrierInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CarrierInfo} m CarrierInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CarrierInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.carrier_constant != null && m.hasOwnProperty("carrier_constant")) {
                                    d.carrier_constant = m.carrier_constant;
                                    if (o.oneofs)
                                        d._carrier_constant = "carrier_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CarrierInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CarrierInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CarrierInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CarrierInfo;
                        })();
    
                        common.UserInterestInfo = (function() {
    
                            /**
                             * Properties of a UserInterestInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserInterestInfo
                             * @property {string|null} [user_interest_category] UserInterestInfo user_interest_category
                             */
    
                            /**
                             * Constructs a new UserInterestInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserInterestInfo.
                             * @implements IUserInterestInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserInterestInfo=} [p] Properties to set
                             */
                            function UserInterestInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserInterestInfo user_interest_category.
                             * @member {string|null|undefined} user_interest_category
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @instance
                             */
                            UserInterestInfo.prototype.user_interest_category = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserInterestInfo _user_interest_category.
                             * @member {"user_interest_category"|undefined} _user_interest_category
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @instance
                             */
                            Object.defineProperty(UserInterestInfo.prototype, "_user_interest_category", {
                                get: $util.oneOfGetter($oneOfFields = ["user_interest_category"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserInterestInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserInterestInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserInterestInfo} UserInterestInfo instance
                             */
                            UserInterestInfo.create = function create(properties) {
                                return new UserInterestInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserInterestInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserInterestInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserInterestInfo} m UserInterestInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterestInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_interest_category != null && Object.hasOwnProperty.call(m, "user_interest_category"))
                                    w.uint32(18).string(m.user_interest_category);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserInterestInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserInterestInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserInterestInfo} message UserInterestInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterestInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserInterestInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserInterestInfo} UserInterestInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterestInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserInterestInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.user_interest_category = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserInterestInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserInterestInfo} UserInterestInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterestInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserInterestInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserInterestInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_interest_category != null && m.hasOwnProperty("user_interest_category")) {
                                    p._user_interest_category = 1;
                                    if (!$util.isString(m.user_interest_category))
                                        return "user_interest_category: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserInterestInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserInterestInfo} UserInterestInfo
                             */
                            UserInterestInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserInterestInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserInterestInfo();
                                if (d.user_interest_category != null) {
                                    m.user_interest_category = String(d.user_interest_category);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserInterestInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserInterestInfo} m UserInterestInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserInterestInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_interest_category != null && m.hasOwnProperty("user_interest_category")) {
                                    d.user_interest_category = m.user_interest_category;
                                    if (o.oneofs)
                                        d._user_interest_category = "user_interest_category";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserInterestInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserInterestInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserInterestInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserInterestInfo;
                        })();
    
                        common.WebpageInfo = (function() {
    
                            /**
                             * Properties of a WebpageInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IWebpageInfo
                             * @property {string|null} [criterion_name] WebpageInfo criterion_name
                             * @property {Array.<google.ads.googleads.v11.common.IWebpageConditionInfo>|null} [conditions] WebpageInfo conditions
                             * @property {number|null} [coverage_percentage] WebpageInfo coverage_percentage
                             * @property {google.ads.googleads.v11.common.IWebpageSampleInfo|null} [sample] WebpageInfo sample
                             */
    
                            /**
                             * Constructs a new WebpageInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a WebpageInfo.
                             * @implements IWebpageInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IWebpageInfo=} [p] Properties to set
                             */
                            function WebpageInfo(p) {
                                this.conditions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * WebpageInfo criterion_name.
                             * @member {string|null|undefined} criterion_name
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @instance
                             */
                            WebpageInfo.prototype.criterion_name = null;
    
                            /**
                             * WebpageInfo conditions.
                             * @member {Array.<google.ads.googleads.v11.common.IWebpageConditionInfo>} conditions
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @instance
                             */
                            WebpageInfo.prototype.conditions = $util.emptyArray;
    
                            /**
                             * WebpageInfo coverage_percentage.
                             * @member {number} coverage_percentage
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @instance
                             */
                            WebpageInfo.prototype.coverage_percentage = 0;
    
                            /**
                             * WebpageInfo sample.
                             * @member {google.ads.googleads.v11.common.IWebpageSampleInfo|null|undefined} sample
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @instance
                             */
                            WebpageInfo.prototype.sample = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * WebpageInfo _criterion_name.
                             * @member {"criterion_name"|undefined} _criterion_name
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @instance
                             */
                            Object.defineProperty(WebpageInfo.prototype, "_criterion_name", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new WebpageInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.WebpageInfo} WebpageInfo instance
                             */
                            WebpageInfo.create = function create(properties) {
                                return new WebpageInfo(properties);
                            };
    
                            /**
                             * Encodes the specified WebpageInfo message. Does not implicitly {@link google.ads.googleads.v11.common.WebpageInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageInfo} m WebpageInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.conditions != null && m.conditions.length) {
                                    for (var i = 0; i < m.conditions.length; ++i)
                                        $root.google.ads.googleads.v11.common.WebpageConditionInfo.encode(m.conditions[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.criterion_name != null && Object.hasOwnProperty.call(m, "criterion_name"))
                                    w.uint32(26).string(m.criterion_name);
                                if (m.coverage_percentage != null && Object.hasOwnProperty.call(m, "coverage_percentage"))
                                    w.uint32(33).double(m.coverage_percentage);
                                if (m.sample != null && Object.hasOwnProperty.call(m, "sample"))
                                    $root.google.ads.googleads.v11.common.WebpageSampleInfo.encode(m.sample, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified WebpageInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.WebpageInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageInfo} message WebpageInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a WebpageInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.WebpageInfo} WebpageInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.WebpageInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.criterion_name = r.string();
                                        break;
                                    case 2:
                                        if (!(m.conditions && m.conditions.length))
                                            m.conditions = [];
                                        m.conditions.push($root.google.ads.googleads.v11.common.WebpageConditionInfo.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        m.coverage_percentage = r.double();
                                        break;
                                    case 5:
                                        m.sample = $root.google.ads.googleads.v11.common.WebpageSampleInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a WebpageInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.WebpageInfo} WebpageInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a WebpageInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            WebpageInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.criterion_name != null && m.hasOwnProperty("criterion_name")) {
                                    p._criterion_name = 1;
                                    if (!$util.isString(m.criterion_name))
                                        return "criterion_name: string expected";
                                }
                                if (m.conditions != null && m.hasOwnProperty("conditions")) {
                                    if (!Array.isArray(m.conditions))
                                        return "conditions: array expected";
                                    for (var i = 0; i < m.conditions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.WebpageConditionInfo.verify(m.conditions[i]);
                                            if (e)
                                                return "conditions." + e;
                                        }
                                    }
                                }
                                if (m.coverage_percentage != null && m.hasOwnProperty("coverage_percentage")) {
                                    if (typeof m.coverage_percentage !== "number")
                                        return "coverage_percentage: number expected";
                                }
                                if (m.sample != null && m.hasOwnProperty("sample")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.WebpageSampleInfo.verify(m.sample);
                                        if (e)
                                            return "sample." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a WebpageInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.WebpageInfo} WebpageInfo
                             */
                            WebpageInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.WebpageInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.WebpageInfo();
                                if (d.criterion_name != null) {
                                    m.criterion_name = String(d.criterion_name);
                                }
                                if (d.conditions) {
                                    if (!Array.isArray(d.conditions))
                                        throw TypeError(".google.ads.googleads.v11.common.WebpageInfo.conditions: array expected");
                                    m.conditions = [];
                                    for (var i = 0; i < d.conditions.length; ++i) {
                                        if (typeof d.conditions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.WebpageInfo.conditions: object expected");
                                        m.conditions[i] = $root.google.ads.googleads.v11.common.WebpageConditionInfo.fromObject(d.conditions[i]);
                                    }
                                }
                                if (d.coverage_percentage != null) {
                                    m.coverage_percentage = Number(d.coverage_percentage);
                                }
                                if (d.sample != null) {
                                    if (typeof d.sample !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.WebpageInfo.sample: object expected");
                                    m.sample = $root.google.ads.googleads.v11.common.WebpageSampleInfo.fromObject(d.sample);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a WebpageInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.WebpageInfo} m WebpageInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            WebpageInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.conditions = [];
                                }
                                if (o.defaults) {
                                    d.coverage_percentage = 0;
                                    d.sample = null;
                                }
                                if (m.conditions && m.conditions.length) {
                                    d.conditions = [];
                                    for (var j = 0; j < m.conditions.length; ++j) {
                                        d.conditions[j] = $root.google.ads.googleads.v11.common.WebpageConditionInfo.toObject(m.conditions[j], o);
                                    }
                                }
                                if (m.criterion_name != null && m.hasOwnProperty("criterion_name")) {
                                    d.criterion_name = m.criterion_name;
                                    if (o.oneofs)
                                        d._criterion_name = "criterion_name";
                                }
                                if (m.coverage_percentage != null && m.hasOwnProperty("coverage_percentage")) {
                                    d.coverage_percentage = o.json && !isFinite(m.coverage_percentage) ? String(m.coverage_percentage) : m.coverage_percentage;
                                }
                                if (m.sample != null && m.hasOwnProperty("sample")) {
                                    d.sample = $root.google.ads.googleads.v11.common.WebpageSampleInfo.toObject(m.sample, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this WebpageInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.WebpageInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            WebpageInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return WebpageInfo;
                        })();
    
                        common.WebpageConditionInfo = (function() {
    
                            /**
                             * Properties of a WebpageConditionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IWebpageConditionInfo
                             * @property {google.ads.googleads.v11.enums.WebpageConditionOperandEnum.WebpageConditionOperand|null} [operand] WebpageConditionInfo operand
                             * @property {google.ads.googleads.v11.enums.WebpageConditionOperatorEnum.WebpageConditionOperator|null} [operator] WebpageConditionInfo operator
                             * @property {string|null} [argument] WebpageConditionInfo argument
                             */
    
                            /**
                             * Constructs a new WebpageConditionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a WebpageConditionInfo.
                             * @implements IWebpageConditionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IWebpageConditionInfo=} [p] Properties to set
                             */
                            function WebpageConditionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * WebpageConditionInfo operand.
                             * @member {google.ads.googleads.v11.enums.WebpageConditionOperandEnum.WebpageConditionOperand} operand
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @instance
                             */
                            WebpageConditionInfo.prototype.operand = 0;
    
                            /**
                             * WebpageConditionInfo operator.
                             * @member {google.ads.googleads.v11.enums.WebpageConditionOperatorEnum.WebpageConditionOperator} operator
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @instance
                             */
                            WebpageConditionInfo.prototype.operator = 0;
    
                            /**
                             * WebpageConditionInfo argument.
                             * @member {string|null|undefined} argument
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @instance
                             */
                            WebpageConditionInfo.prototype.argument = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * WebpageConditionInfo _argument.
                             * @member {"argument"|undefined} _argument
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @instance
                             */
                            Object.defineProperty(WebpageConditionInfo.prototype, "_argument", {
                                get: $util.oneOfGetter($oneOfFields = ["argument"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new WebpageConditionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageConditionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.WebpageConditionInfo} WebpageConditionInfo instance
                             */
                            WebpageConditionInfo.create = function create(properties) {
                                return new WebpageConditionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified WebpageConditionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.WebpageConditionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageConditionInfo} m WebpageConditionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageConditionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operand != null && Object.hasOwnProperty.call(m, "operand"))
                                    w.uint32(8).int32(m.operand);
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(16).int32(m.operator);
                                if (m.argument != null && Object.hasOwnProperty.call(m, "argument"))
                                    w.uint32(34).string(m.argument);
                                return w;
                            };
    
                            /**
                             * Encodes the specified WebpageConditionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.WebpageConditionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageConditionInfo} message WebpageConditionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageConditionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a WebpageConditionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.WebpageConditionInfo} WebpageConditionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageConditionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.WebpageConditionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operand = r.int32();
                                        break;
                                    case 2:
                                        m.operator = r.int32();
                                        break;
                                    case 4:
                                        m.argument = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a WebpageConditionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.WebpageConditionInfo} WebpageConditionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageConditionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a WebpageConditionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            WebpageConditionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.operand != null && m.hasOwnProperty("operand")) {
                                    switch (m.operand) {
                                    default:
                                        return "operand: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.argument != null && m.hasOwnProperty("argument")) {
                                    p._argument = 1;
                                    if (!$util.isString(m.argument))
                                        return "argument: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a WebpageConditionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.WebpageConditionInfo} WebpageConditionInfo
                             */
                            WebpageConditionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.WebpageConditionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.WebpageConditionInfo();
                                switch (d.operand) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operand = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operand = 1;
                                    break;
                                case "URL":
                                case 2:
                                    m.operand = 2;
                                    break;
                                case "CATEGORY":
                                case 3:
                                    m.operand = 3;
                                    break;
                                case "PAGE_TITLE":
                                case 4:
                                    m.operand = 4;
                                    break;
                                case "PAGE_CONTENT":
                                case 5:
                                    m.operand = 5;
                                    break;
                                case "CUSTOM_LABEL":
                                case 6:
                                    m.operand = 6;
                                    break;
                                }
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "EQUALS":
                                case 2:
                                    m.operator = 2;
                                    break;
                                case "CONTAINS":
                                case 3:
                                    m.operator = 3;
                                    break;
                                }
                                if (d.argument != null) {
                                    m.argument = String(d.argument);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a WebpageConditionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.WebpageConditionInfo} m WebpageConditionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            WebpageConditionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.operand = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.operand != null && m.hasOwnProperty("operand")) {
                                    d.operand = o.enums === String ? $root.google.ads.googleads.v11.enums.WebpageConditionOperandEnum.WebpageConditionOperand[m.operand] : m.operand;
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.enums.WebpageConditionOperatorEnum.WebpageConditionOperator[m.operator] : m.operator;
                                }
                                if (m.argument != null && m.hasOwnProperty("argument")) {
                                    d.argument = m.argument;
                                    if (o.oneofs)
                                        d._argument = "argument";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this WebpageConditionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.WebpageConditionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            WebpageConditionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return WebpageConditionInfo;
                        })();
    
                        common.WebpageSampleInfo = (function() {
    
                            /**
                             * Properties of a WebpageSampleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IWebpageSampleInfo
                             * @property {Array.<string>|null} [sample_urls] WebpageSampleInfo sample_urls
                             */
    
                            /**
                             * Constructs a new WebpageSampleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a WebpageSampleInfo.
                             * @implements IWebpageSampleInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IWebpageSampleInfo=} [p] Properties to set
                             */
                            function WebpageSampleInfo(p) {
                                this.sample_urls = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * WebpageSampleInfo sample_urls.
                             * @member {Array.<string>} sample_urls
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @instance
                             */
                            WebpageSampleInfo.prototype.sample_urls = $util.emptyArray;
    
                            /**
                             * Creates a new WebpageSampleInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageSampleInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.WebpageSampleInfo} WebpageSampleInfo instance
                             */
                            WebpageSampleInfo.create = function create(properties) {
                                return new WebpageSampleInfo(properties);
                            };
    
                            /**
                             * Encodes the specified WebpageSampleInfo message. Does not implicitly {@link google.ads.googleads.v11.common.WebpageSampleInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageSampleInfo} m WebpageSampleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageSampleInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.sample_urls != null && m.sample_urls.length) {
                                    for (var i = 0; i < m.sample_urls.length; ++i)
                                        w.uint32(10).string(m.sample_urls[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified WebpageSampleInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.WebpageSampleInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IWebpageSampleInfo} message WebpageSampleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageSampleInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a WebpageSampleInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.WebpageSampleInfo} WebpageSampleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageSampleInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.WebpageSampleInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.sample_urls && m.sample_urls.length))
                                            m.sample_urls = [];
                                        m.sample_urls.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a WebpageSampleInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.WebpageSampleInfo} WebpageSampleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageSampleInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a WebpageSampleInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            WebpageSampleInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.sample_urls != null && m.hasOwnProperty("sample_urls")) {
                                    if (!Array.isArray(m.sample_urls))
                                        return "sample_urls: array expected";
                                    for (var i = 0; i < m.sample_urls.length; ++i) {
                                        if (!$util.isString(m.sample_urls[i]))
                                            return "sample_urls: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a WebpageSampleInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.WebpageSampleInfo} WebpageSampleInfo
                             */
                            WebpageSampleInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.WebpageSampleInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.WebpageSampleInfo();
                                if (d.sample_urls) {
                                    if (!Array.isArray(d.sample_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.WebpageSampleInfo.sample_urls: array expected");
                                    m.sample_urls = [];
                                    for (var i = 0; i < d.sample_urls.length; ++i) {
                                        m.sample_urls[i] = String(d.sample_urls[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a WebpageSampleInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.WebpageSampleInfo} m WebpageSampleInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            WebpageSampleInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.sample_urls = [];
                                }
                                if (m.sample_urls && m.sample_urls.length) {
                                    d.sample_urls = [];
                                    for (var j = 0; j < m.sample_urls.length; ++j) {
                                        d.sample_urls[j] = m.sample_urls[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this WebpageSampleInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.WebpageSampleInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            WebpageSampleInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return WebpageSampleInfo;
                        })();
    
                        common.OperatingSystemVersionInfo = (function() {
    
                            /**
                             * Properties of an OperatingSystemVersionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IOperatingSystemVersionInfo
                             * @property {string|null} [operating_system_version_constant] OperatingSystemVersionInfo operating_system_version_constant
                             */
    
                            /**
                             * Constructs a new OperatingSystemVersionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an OperatingSystemVersionInfo.
                             * @implements IOperatingSystemVersionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IOperatingSystemVersionInfo=} [p] Properties to set
                             */
                            function OperatingSystemVersionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * OperatingSystemVersionInfo operating_system_version_constant.
                             * @member {string|null|undefined} operating_system_version_constant
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @instance
                             */
                            OperatingSystemVersionInfo.prototype.operating_system_version_constant = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * OperatingSystemVersionInfo _operating_system_version_constant.
                             * @member {"operating_system_version_constant"|undefined} _operating_system_version_constant
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @instance
                             */
                            Object.defineProperty(OperatingSystemVersionInfo.prototype, "_operating_system_version_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["operating_system_version_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new OperatingSystemVersionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IOperatingSystemVersionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.OperatingSystemVersionInfo} OperatingSystemVersionInfo instance
                             */
                            OperatingSystemVersionInfo.create = function create(properties) {
                                return new OperatingSystemVersionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified OperatingSystemVersionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.OperatingSystemVersionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IOperatingSystemVersionInfo} m OperatingSystemVersionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatingSystemVersionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operating_system_version_constant != null && Object.hasOwnProperty.call(m, "operating_system_version_constant"))
                                    w.uint32(18).string(m.operating_system_version_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified OperatingSystemVersionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.OperatingSystemVersionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IOperatingSystemVersionInfo} message OperatingSystemVersionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatingSystemVersionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OperatingSystemVersionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.OperatingSystemVersionInfo} OperatingSystemVersionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatingSystemVersionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.operating_system_version_constant = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OperatingSystemVersionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.OperatingSystemVersionInfo} OperatingSystemVersionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatingSystemVersionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OperatingSystemVersionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OperatingSystemVersionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.operating_system_version_constant != null && m.hasOwnProperty("operating_system_version_constant")) {
                                    p._operating_system_version_constant = 1;
                                    if (!$util.isString(m.operating_system_version_constant))
                                        return "operating_system_version_constant: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an OperatingSystemVersionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.OperatingSystemVersionInfo} OperatingSystemVersionInfo
                             */
                            OperatingSystemVersionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo();
                                if (d.operating_system_version_constant != null) {
                                    m.operating_system_version_constant = String(d.operating_system_version_constant);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an OperatingSystemVersionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.OperatingSystemVersionInfo} m OperatingSystemVersionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OperatingSystemVersionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.operating_system_version_constant != null && m.hasOwnProperty("operating_system_version_constant")) {
                                    d.operating_system_version_constant = m.operating_system_version_constant;
                                    if (o.oneofs)
                                        d._operating_system_version_constant = "operating_system_version_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this OperatingSystemVersionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.OperatingSystemVersionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OperatingSystemVersionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return OperatingSystemVersionInfo;
                        })();
    
                        common.AppPaymentModelInfo = (function() {
    
                            /**
                             * Properties of an AppPaymentModelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAppPaymentModelInfo
                             * @property {google.ads.googleads.v11.enums.AppPaymentModelTypeEnum.AppPaymentModelType|null} [type] AppPaymentModelInfo type
                             */
    
                            /**
                             * Constructs a new AppPaymentModelInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AppPaymentModelInfo.
                             * @implements IAppPaymentModelInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAppPaymentModelInfo=} [p] Properties to set
                             */
                            function AppPaymentModelInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AppPaymentModelInfo type.
                             * @member {google.ads.googleads.v11.enums.AppPaymentModelTypeEnum.AppPaymentModelType} type
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @instance
                             */
                            AppPaymentModelInfo.prototype.type = 0;
    
                            /**
                             * Creates a new AppPaymentModelInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppPaymentModelInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AppPaymentModelInfo} AppPaymentModelInfo instance
                             */
                            AppPaymentModelInfo.create = function create(properties) {
                                return new AppPaymentModelInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AppPaymentModelInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AppPaymentModelInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppPaymentModelInfo} m AppPaymentModelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPaymentModelInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppPaymentModelInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AppPaymentModelInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppPaymentModelInfo} message AppPaymentModelInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPaymentModelInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppPaymentModelInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AppPaymentModelInfo} AppPaymentModelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPaymentModelInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AppPaymentModelInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppPaymentModelInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AppPaymentModelInfo} AppPaymentModelInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPaymentModelInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppPaymentModelInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppPaymentModelInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 30:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AppPaymentModelInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AppPaymentModelInfo} AppPaymentModelInfo
                             */
                            AppPaymentModelInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AppPaymentModelInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AppPaymentModelInfo();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "PAID":
                                case 30:
                                    m.type = 30;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AppPaymentModelInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AppPaymentModelInfo} m AppPaymentModelInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppPaymentModelInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.AppPaymentModelTypeEnum.AppPaymentModelType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AppPaymentModelInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AppPaymentModelInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppPaymentModelInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AppPaymentModelInfo;
                        })();
    
                        common.MobileDeviceInfo = (function() {
    
                            /**
                             * Properties of a MobileDeviceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMobileDeviceInfo
                             * @property {string|null} [mobile_device_constant] MobileDeviceInfo mobile_device_constant
                             */
    
                            /**
                             * Constructs a new MobileDeviceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MobileDeviceInfo.
                             * @implements IMobileDeviceInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMobileDeviceInfo=} [p] Properties to set
                             */
                            function MobileDeviceInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MobileDeviceInfo mobile_device_constant.
                             * @member {string|null|undefined} mobile_device_constant
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @instance
                             */
                            MobileDeviceInfo.prototype.mobile_device_constant = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MobileDeviceInfo _mobile_device_constant.
                             * @member {"mobile_device_constant"|undefined} _mobile_device_constant
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @instance
                             */
                            Object.defineProperty(MobileDeviceInfo.prototype, "_mobile_device_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["mobile_device_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MobileDeviceInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileDeviceInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MobileDeviceInfo} MobileDeviceInfo instance
                             */
                            MobileDeviceInfo.create = function create(properties) {
                                return new MobileDeviceInfo(properties);
                            };
    
                            /**
                             * Encodes the specified MobileDeviceInfo message. Does not implicitly {@link google.ads.googleads.v11.common.MobileDeviceInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileDeviceInfo} m MobileDeviceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileDeviceInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.mobile_device_constant != null && Object.hasOwnProperty.call(m, "mobile_device_constant"))
                                    w.uint32(18).string(m.mobile_device_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileDeviceInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MobileDeviceInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IMobileDeviceInfo} message MobileDeviceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileDeviceInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileDeviceInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MobileDeviceInfo} MobileDeviceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileDeviceInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MobileDeviceInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.mobile_device_constant = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileDeviceInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MobileDeviceInfo} MobileDeviceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileDeviceInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileDeviceInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileDeviceInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.mobile_device_constant != null && m.hasOwnProperty("mobile_device_constant")) {
                                    p._mobile_device_constant = 1;
                                    if (!$util.isString(m.mobile_device_constant))
                                        return "mobile_device_constant: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MobileDeviceInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MobileDeviceInfo} MobileDeviceInfo
                             */
                            MobileDeviceInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MobileDeviceInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MobileDeviceInfo();
                                if (d.mobile_device_constant != null) {
                                    m.mobile_device_constant = String(d.mobile_device_constant);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MobileDeviceInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.MobileDeviceInfo} m MobileDeviceInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileDeviceInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.mobile_device_constant != null && m.hasOwnProperty("mobile_device_constant")) {
                                    d.mobile_device_constant = m.mobile_device_constant;
                                    if (o.oneofs)
                                        d._mobile_device_constant = "mobile_device_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MobileDeviceInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MobileDeviceInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileDeviceInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MobileDeviceInfo;
                        })();
    
                        common.CustomAffinityInfo = (function() {
    
                            /**
                             * Properties of a CustomAffinityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICustomAffinityInfo
                             * @property {string|null} [custom_affinity] CustomAffinityInfo custom_affinity
                             */
    
                            /**
                             * Constructs a new CustomAffinityInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CustomAffinityInfo.
                             * @implements ICustomAffinityInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICustomAffinityInfo=} [p] Properties to set
                             */
                            function CustomAffinityInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomAffinityInfo custom_affinity.
                             * @member {string|null|undefined} custom_affinity
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @instance
                             */
                            CustomAffinityInfo.prototype.custom_affinity = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomAffinityInfo _custom_affinity.
                             * @member {"custom_affinity"|undefined} _custom_affinity
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @instance
                             */
                            Object.defineProperty(CustomAffinityInfo.prototype, "_custom_affinity", {
                                get: $util.oneOfGetter($oneOfFields = ["custom_affinity"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomAffinityInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAffinityInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CustomAffinityInfo} CustomAffinityInfo instance
                             */
                            CustomAffinityInfo.create = function create(properties) {
                                return new CustomAffinityInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAffinityInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CustomAffinityInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAffinityInfo} m CustomAffinityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAffinityInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.custom_affinity != null && Object.hasOwnProperty.call(m, "custom_affinity"))
                                    w.uint32(18).string(m.custom_affinity);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAffinityInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CustomAffinityInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAffinityInfo} message CustomAffinityInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAffinityInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAffinityInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CustomAffinityInfo} CustomAffinityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAffinityInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CustomAffinityInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.custom_affinity = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAffinityInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CustomAffinityInfo} CustomAffinityInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAffinityInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAffinityInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAffinityInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.custom_affinity != null && m.hasOwnProperty("custom_affinity")) {
                                    p._custom_affinity = 1;
                                    if (!$util.isString(m.custom_affinity))
                                        return "custom_affinity: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomAffinityInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CustomAffinityInfo} CustomAffinityInfo
                             */
                            CustomAffinityInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CustomAffinityInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CustomAffinityInfo();
                                if (d.custom_affinity != null) {
                                    m.custom_affinity = String(d.custom_affinity);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomAffinityInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CustomAffinityInfo} m CustomAffinityInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAffinityInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.custom_affinity != null && m.hasOwnProperty("custom_affinity")) {
                                    d.custom_affinity = m.custom_affinity;
                                    if (o.oneofs)
                                        d._custom_affinity = "custom_affinity";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomAffinityInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CustomAffinityInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAffinityInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomAffinityInfo;
                        })();
    
                        common.CustomIntentInfo = (function() {
    
                            /**
                             * Properties of a CustomIntentInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICustomIntentInfo
                             * @property {string|null} [custom_intent] CustomIntentInfo custom_intent
                             */
    
                            /**
                             * Constructs a new CustomIntentInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CustomIntentInfo.
                             * @implements ICustomIntentInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICustomIntentInfo=} [p] Properties to set
                             */
                            function CustomIntentInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomIntentInfo custom_intent.
                             * @member {string|null|undefined} custom_intent
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @instance
                             */
                            CustomIntentInfo.prototype.custom_intent = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomIntentInfo _custom_intent.
                             * @member {"custom_intent"|undefined} _custom_intent
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @instance
                             */
                            Object.defineProperty(CustomIntentInfo.prototype, "_custom_intent", {
                                get: $util.oneOfGetter($oneOfFields = ["custom_intent"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomIntentInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomIntentInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CustomIntentInfo} CustomIntentInfo instance
                             */
                            CustomIntentInfo.create = function create(properties) {
                                return new CustomIntentInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CustomIntentInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CustomIntentInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomIntentInfo} m CustomIntentInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomIntentInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.custom_intent != null && Object.hasOwnProperty.call(m, "custom_intent"))
                                    w.uint32(18).string(m.custom_intent);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomIntentInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CustomIntentInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomIntentInfo} message CustomIntentInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomIntentInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomIntentInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CustomIntentInfo} CustomIntentInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomIntentInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CustomIntentInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.custom_intent = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomIntentInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CustomIntentInfo} CustomIntentInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomIntentInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomIntentInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomIntentInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.custom_intent != null && m.hasOwnProperty("custom_intent")) {
                                    p._custom_intent = 1;
                                    if (!$util.isString(m.custom_intent))
                                        return "custom_intent: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomIntentInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CustomIntentInfo} CustomIntentInfo
                             */
                            CustomIntentInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CustomIntentInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CustomIntentInfo();
                                if (d.custom_intent != null) {
                                    m.custom_intent = String(d.custom_intent);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomIntentInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CustomIntentInfo} m CustomIntentInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomIntentInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.custom_intent != null && m.hasOwnProperty("custom_intent")) {
                                    d.custom_intent = m.custom_intent;
                                    if (o.oneofs)
                                        d._custom_intent = "custom_intent";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomIntentInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CustomIntentInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomIntentInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomIntentInfo;
                        })();
    
                        common.LocationGroupInfo = (function() {
    
                            /**
                             * Properties of a LocationGroupInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILocationGroupInfo
                             * @property {string|null} [feed] LocationGroupInfo feed
                             * @property {Array.<string>|null} [geo_target_constants] LocationGroupInfo geo_target_constants
                             * @property {number|null} [radius] LocationGroupInfo radius
                             * @property {google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum.LocationGroupRadiusUnits|null} [radius_units] LocationGroupInfo radius_units
                             * @property {Array.<string>|null} [feed_item_sets] LocationGroupInfo feed_item_sets
                             */
    
                            /**
                             * Constructs a new LocationGroupInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LocationGroupInfo.
                             * @implements ILocationGroupInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILocationGroupInfo=} [p] Properties to set
                             */
                            function LocationGroupInfo(p) {
                                this.geo_target_constants = [];
                                this.feed_item_sets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LocationGroupInfo feed.
                             * @member {string|null|undefined} feed
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             */
                            LocationGroupInfo.prototype.feed = null;
    
                            /**
                             * LocationGroupInfo geo_target_constants.
                             * @member {Array.<string>} geo_target_constants
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             */
                            LocationGroupInfo.prototype.geo_target_constants = $util.emptyArray;
    
                            /**
                             * LocationGroupInfo radius.
                             * @member {number|null|undefined} radius
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             */
                            LocationGroupInfo.prototype.radius = null;
    
                            /**
                             * LocationGroupInfo radius_units.
                             * @member {google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum.LocationGroupRadiusUnits} radius_units
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             */
                            LocationGroupInfo.prototype.radius_units = 0;
    
                            /**
                             * LocationGroupInfo feed_item_sets.
                             * @member {Array.<string>} feed_item_sets
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             */
                            LocationGroupInfo.prototype.feed_item_sets = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LocationGroupInfo _feed.
                             * @member {"feed"|undefined} _feed
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             */
                            Object.defineProperty(LocationGroupInfo.prototype, "_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationGroupInfo _radius.
                             * @member {"radius"|undefined} _radius
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             */
                            Object.defineProperty(LocationGroupInfo.prototype, "_radius", {
                                get: $util.oneOfGetter($oneOfFields = ["radius"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LocationGroupInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationGroupInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LocationGroupInfo} LocationGroupInfo instance
                             */
                            LocationGroupInfo.create = function create(properties) {
                                return new LocationGroupInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LocationGroupInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LocationGroupInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationGroupInfo} m LocationGroupInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationGroupInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.radius_units != null && Object.hasOwnProperty.call(m, "radius_units"))
                                    w.uint32(32).int32(m.radius_units);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(42).string(m.feed);
                                if (m.geo_target_constants != null && m.geo_target_constants.length) {
                                    for (var i = 0; i < m.geo_target_constants.length; ++i)
                                        w.uint32(50).string(m.geo_target_constants[i]);
                                }
                                if (m.radius != null && Object.hasOwnProperty.call(m, "radius"))
                                    w.uint32(56).int64(m.radius);
                                if (m.feed_item_sets != null && m.feed_item_sets.length) {
                                    for (var i = 0; i < m.feed_item_sets.length; ++i)
                                        w.uint32(66).string(m.feed_item_sets[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationGroupInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LocationGroupInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationGroupInfo} message LocationGroupInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationGroupInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationGroupInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LocationGroupInfo} LocationGroupInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationGroupInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LocationGroupInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.feed = r.string();
                                        break;
                                    case 6:
                                        if (!(m.geo_target_constants && m.geo_target_constants.length))
                                            m.geo_target_constants = [];
                                        m.geo_target_constants.push(r.string());
                                        break;
                                    case 7:
                                        m.radius = r.int64();
                                        break;
                                    case 4:
                                        m.radius_units = r.int32();
                                        break;
                                    case 8:
                                        if (!(m.feed_item_sets && m.feed_item_sets.length))
                                            m.feed_item_sets = [];
                                        m.feed_item_sets.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationGroupInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LocationGroupInfo} LocationGroupInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationGroupInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationGroupInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationGroupInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    p._feed = 1;
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.geo_target_constants != null && m.hasOwnProperty("geo_target_constants")) {
                                    if (!Array.isArray(m.geo_target_constants))
                                        return "geo_target_constants: array expected";
                                    for (var i = 0; i < m.geo_target_constants.length; ++i) {
                                        if (!$util.isString(m.geo_target_constants[i]))
                                            return "geo_target_constants: string[] expected";
                                    }
                                }
                                if (m.radius != null && m.hasOwnProperty("radius")) {
                                    p._radius = 1;
                                    if (!$util.isInteger(m.radius) && !(m.radius && $util.isInteger(m.radius.low) && $util.isInteger(m.radius.high)))
                                        return "radius: integer|Long expected";
                                }
                                if (m.radius_units != null && m.hasOwnProperty("radius_units")) {
                                    switch (m.radius_units) {
                                    default:
                                        return "radius_units: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.feed_item_sets != null && m.hasOwnProperty("feed_item_sets")) {
                                    if (!Array.isArray(m.feed_item_sets))
                                        return "feed_item_sets: array expected";
                                    for (var i = 0; i < m.feed_item_sets.length; ++i) {
                                        if (!$util.isString(m.feed_item_sets[i]))
                                            return "feed_item_sets: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LocationGroupInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LocationGroupInfo} LocationGroupInfo
                             */
                            LocationGroupInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LocationGroupInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LocationGroupInfo();
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.geo_target_constants) {
                                    if (!Array.isArray(d.geo_target_constants))
                                        throw TypeError(".google.ads.googleads.v11.common.LocationGroupInfo.geo_target_constants: array expected");
                                    m.geo_target_constants = [];
                                    for (var i = 0; i < d.geo_target_constants.length; ++i) {
                                        m.geo_target_constants[i] = String(d.geo_target_constants[i]);
                                    }
                                }
                                if (d.radius != null) {
                                    if ($util.Long)
                                        (m.radius = $util.Long.fromValue(d.radius)).unsigned = false;
                                    else if (typeof d.radius === "string")
                                        m.radius = parseInt(d.radius, 10);
                                    else if (typeof d.radius === "number")
                                        m.radius = d.radius;
                                    else if (typeof d.radius === "object")
                                        m.radius = new $util.LongBits(d.radius.low >>> 0, d.radius.high >>> 0).toNumber();
                                }
                                switch (d.radius_units) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.radius_units = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.radius_units = 1;
                                    break;
                                case "METERS":
                                case 2:
                                    m.radius_units = 2;
                                    break;
                                case "MILES":
                                case 3:
                                    m.radius_units = 3;
                                    break;
                                case "MILLI_MILES":
                                case 4:
                                    m.radius_units = 4;
                                    break;
                                }
                                if (d.feed_item_sets) {
                                    if (!Array.isArray(d.feed_item_sets))
                                        throw TypeError(".google.ads.googleads.v11.common.LocationGroupInfo.feed_item_sets: array expected");
                                    m.feed_item_sets = [];
                                    for (var i = 0; i < d.feed_item_sets.length; ++i) {
                                        m.feed_item_sets[i] = String(d.feed_item_sets[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LocationGroupInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LocationGroupInfo} m LocationGroupInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationGroupInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.geo_target_constants = [];
                                    d.feed_item_sets = [];
                                }
                                if (o.defaults) {
                                    d.radius_units = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.radius_units != null && m.hasOwnProperty("radius_units")) {
                                    d.radius_units = o.enums === String ? $root.google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum.LocationGroupRadiusUnits[m.radius_units] : m.radius_units;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                    if (o.oneofs)
                                        d._feed = "feed";
                                }
                                if (m.geo_target_constants && m.geo_target_constants.length) {
                                    d.geo_target_constants = [];
                                    for (var j = 0; j < m.geo_target_constants.length; ++j) {
                                        d.geo_target_constants[j] = m.geo_target_constants[j];
                                    }
                                }
                                if (m.radius != null && m.hasOwnProperty("radius")) {
                                    if (typeof m.radius === "number")
                                        d.radius = o.longs === String ? String(m.radius) : m.radius;
                                    else
                                        d.radius = o.longs === String ? $util.Long.prototype.toString.call(m.radius) : o.longs === Number ? new $util.LongBits(m.radius.low >>> 0, m.radius.high >>> 0).toNumber() : m.radius;
                                    if (o.oneofs)
                                        d._radius = "radius";
                                }
                                if (m.feed_item_sets && m.feed_item_sets.length) {
                                    d.feed_item_sets = [];
                                    for (var j = 0; j < m.feed_item_sets.length; ++j) {
                                        d.feed_item_sets[j] = m.feed_item_sets[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LocationGroupInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LocationGroupInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationGroupInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LocationGroupInfo;
                        })();
    
                        common.CustomAudienceInfo = (function() {
    
                            /**
                             * Properties of a CustomAudienceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICustomAudienceInfo
                             * @property {string|null} [custom_audience] CustomAudienceInfo custom_audience
                             */
    
                            /**
                             * Constructs a new CustomAudienceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CustomAudienceInfo.
                             * @implements ICustomAudienceInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICustomAudienceInfo=} [p] Properties to set
                             */
                            function CustomAudienceInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomAudienceInfo custom_audience.
                             * @member {string} custom_audience
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @instance
                             */
                            CustomAudienceInfo.prototype.custom_audience = "";
    
                            /**
                             * Creates a new CustomAudienceInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAudienceInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CustomAudienceInfo} CustomAudienceInfo instance
                             */
                            CustomAudienceInfo.create = function create(properties) {
                                return new CustomAudienceInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CustomAudienceInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAudienceInfo} m CustomAudienceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.custom_audience != null && Object.hasOwnProperty.call(m, "custom_audience"))
                                    w.uint32(10).string(m.custom_audience);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CustomAudienceInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAudienceInfo} message CustomAudienceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CustomAudienceInfo} CustomAudienceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CustomAudienceInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.custom_audience = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CustomAudienceInfo} CustomAudienceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    if (!$util.isString(m.custom_audience))
                                        return "custom_audience: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CustomAudienceInfo} CustomAudienceInfo
                             */
                            CustomAudienceInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CustomAudienceInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CustomAudienceInfo();
                                if (d.custom_audience != null) {
                                    m.custom_audience = String(d.custom_audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CustomAudienceInfo} m CustomAudienceInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.custom_audience = "";
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    d.custom_audience = m.custom_audience;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomAudienceInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CustomAudienceInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomAudienceInfo;
                        })();
    
                        common.CombinedAudienceInfo = (function() {
    
                            /**
                             * Properties of a CombinedAudienceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICombinedAudienceInfo
                             * @property {string|null} [combined_audience] CombinedAudienceInfo combined_audience
                             */
    
                            /**
                             * Constructs a new CombinedAudienceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CombinedAudienceInfo.
                             * @implements ICombinedAudienceInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICombinedAudienceInfo=} [p] Properties to set
                             */
                            function CombinedAudienceInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CombinedAudienceInfo combined_audience.
                             * @member {string} combined_audience
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @instance
                             */
                            CombinedAudienceInfo.prototype.combined_audience = "";
    
                            /**
                             * Creates a new CombinedAudienceInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICombinedAudienceInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CombinedAudienceInfo} CombinedAudienceInfo instance
                             */
                            CombinedAudienceInfo.create = function create(properties) {
                                return new CombinedAudienceInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CombinedAudienceInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CombinedAudienceInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICombinedAudienceInfo} m CombinedAudienceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedAudienceInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.combined_audience != null && Object.hasOwnProperty.call(m, "combined_audience"))
                                    w.uint32(10).string(m.combined_audience);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CombinedAudienceInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CombinedAudienceInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICombinedAudienceInfo} message CombinedAudienceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedAudienceInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CombinedAudienceInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CombinedAudienceInfo} CombinedAudienceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedAudienceInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CombinedAudienceInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.combined_audience = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CombinedAudienceInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CombinedAudienceInfo} CombinedAudienceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedAudienceInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CombinedAudienceInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CombinedAudienceInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    if (!$util.isString(m.combined_audience))
                                        return "combined_audience: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CombinedAudienceInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CombinedAudienceInfo} CombinedAudienceInfo
                             */
                            CombinedAudienceInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CombinedAudienceInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CombinedAudienceInfo();
                                if (d.combined_audience != null) {
                                    m.combined_audience = String(d.combined_audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CombinedAudienceInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CombinedAudienceInfo} m CombinedAudienceInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CombinedAudienceInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.combined_audience = "";
                                }
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    d.combined_audience = m.combined_audience;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CombinedAudienceInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CombinedAudienceInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CombinedAudienceInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CombinedAudienceInfo;
                        })();
    
                        common.AudienceInfo = (function() {
    
                            /**
                             * Properties of an AudienceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAudienceInfo
                             * @property {string|null} [audience] AudienceInfo audience
                             */
    
                            /**
                             * Constructs a new AudienceInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AudienceInfo.
                             * @implements IAudienceInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAudienceInfo=} [p] Properties to set
                             */
                            function AudienceInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceInfo audience.
                             * @member {string} audience
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @instance
                             */
                            AudienceInfo.prototype.audience = "";
    
                            /**
                             * Creates a new AudienceInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AudienceInfo} AudienceInfo instance
                             */
                            AudienceInfo.create = function create(properties) {
                                return new AudienceInfo(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceInfo message. Does not implicitly {@link google.ads.googleads.v11.common.AudienceInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceInfo} m AudienceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.audience != null && Object.hasOwnProperty.call(m, "audience"))
                                    w.uint32(10).string(m.audience);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AudienceInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceInfo} message AudienceInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AudienceInfo} AudienceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AudienceInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.audience = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AudienceInfo} AudienceInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    if (!$util.isString(m.audience))
                                        return "audience: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AudienceInfo} AudienceInfo
                             */
                            AudienceInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AudienceInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AudienceInfo();
                                if (d.audience != null) {
                                    m.audience = String(d.audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.AudienceInfo} m AudienceInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.audience = "";
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    d.audience = m.audience;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AudienceInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceInfo;
                        })();
    
                        common.KeywordThemeInfo = (function() {
    
                            /**
                             * Properties of a KeywordThemeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordThemeInfo
                             * @property {string|null} [keyword_theme_constant] KeywordThemeInfo keyword_theme_constant
                             * @property {string|null} [free_form_keyword_theme] KeywordThemeInfo free_form_keyword_theme
                             */
    
                            /**
                             * Constructs a new KeywordThemeInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordThemeInfo.
                             * @implements IKeywordThemeInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordThemeInfo=} [p] Properties to set
                             */
                            function KeywordThemeInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordThemeInfo keyword_theme_constant.
                             * @member {string|null|undefined} keyword_theme_constant
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @instance
                             */
                            KeywordThemeInfo.prototype.keyword_theme_constant = null;
    
                            /**
                             * KeywordThemeInfo free_form_keyword_theme.
                             * @member {string|null|undefined} free_form_keyword_theme
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @instance
                             */
                            KeywordThemeInfo.prototype.free_form_keyword_theme = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordThemeInfo keyword_theme.
                             * @member {"keyword_theme_constant"|"free_form_keyword_theme"|undefined} keyword_theme
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @instance
                             */
                            Object.defineProperty(KeywordThemeInfo.prototype, "keyword_theme", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_theme_constant", "free_form_keyword_theme"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordThemeInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordThemeInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordThemeInfo} KeywordThemeInfo instance
                             */
                            KeywordThemeInfo.create = function create(properties) {
                                return new KeywordThemeInfo(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordThemeInfo message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordThemeInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordThemeInfo} m KeywordThemeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordThemeInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_theme_constant != null && Object.hasOwnProperty.call(m, "keyword_theme_constant"))
                                    w.uint32(10).string(m.keyword_theme_constant);
                                if (m.free_form_keyword_theme != null && Object.hasOwnProperty.call(m, "free_form_keyword_theme"))
                                    w.uint32(18).string(m.free_form_keyword_theme);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordThemeInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordThemeInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordThemeInfo} message KeywordThemeInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordThemeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordThemeInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordThemeInfo} KeywordThemeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordThemeInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordThemeInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.keyword_theme_constant = r.string();
                                        break;
                                    case 2:
                                        m.free_form_keyword_theme = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordThemeInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordThemeInfo} KeywordThemeInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordThemeInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordThemeInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordThemeInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.keyword_theme_constant != null && m.hasOwnProperty("keyword_theme_constant")) {
                                    p.keyword_theme = 1;
                                    if (!$util.isString(m.keyword_theme_constant))
                                        return "keyword_theme_constant: string expected";
                                }
                                if (m.free_form_keyword_theme != null && m.hasOwnProperty("free_form_keyword_theme")) {
                                    if (p.keyword_theme === 1)
                                        return "keyword_theme: multiple values";
                                    p.keyword_theme = 1;
                                    if (!$util.isString(m.free_form_keyword_theme))
                                        return "free_form_keyword_theme: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordThemeInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordThemeInfo} KeywordThemeInfo
                             */
                            KeywordThemeInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordThemeInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordThemeInfo();
                                if (d.keyword_theme_constant != null) {
                                    m.keyword_theme_constant = String(d.keyword_theme_constant);
                                }
                                if (d.free_form_keyword_theme != null) {
                                    m.free_form_keyword_theme = String(d.free_form_keyword_theme);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordThemeInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordThemeInfo} m KeywordThemeInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordThemeInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.keyword_theme_constant != null && m.hasOwnProperty("keyword_theme_constant")) {
                                    d.keyword_theme_constant = m.keyword_theme_constant;
                                    if (o.oneofs)
                                        d.keyword_theme = "keyword_theme_constant";
                                }
                                if (m.free_form_keyword_theme != null && m.hasOwnProperty("free_form_keyword_theme")) {
                                    d.free_form_keyword_theme = m.free_form_keyword_theme;
                                    if (o.oneofs)
                                        d.keyword_theme = "free_form_keyword_theme";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordThemeInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordThemeInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordThemeInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordThemeInfo;
                        })();
    
                        common.Money = (function() {
    
                            /**
                             * Properties of a Money.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMoney
                             * @property {string|null} [currency_code] Money currency_code
                             * @property {number|null} [amount_micros] Money amount_micros
                             */
    
                            /**
                             * Constructs a new Money.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a Money.
                             * @implements IMoney
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMoney=} [p] Properties to set
                             */
                            function Money(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Money currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.common.Money
                             * @instance
                             */
                            Money.prototype.currency_code = null;
    
                            /**
                             * Money amount_micros.
                             * @member {number|null|undefined} amount_micros
                             * @memberof google.ads.googleads.v11.common.Money
                             * @instance
                             */
                            Money.prototype.amount_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Money _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.common.Money
                             * @instance
                             */
                            Object.defineProperty(Money.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Money _amount_micros.
                             * @member {"amount_micros"|undefined} _amount_micros
                             * @memberof google.ads.googleads.v11.common.Money
                             * @instance
                             */
                            Object.defineProperty(Money.prototype, "_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Money instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {google.ads.googleads.v11.common.IMoney=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.Money} Money instance
                             */
                            Money.create = function create(properties) {
                                return new Money(properties);
                            };
    
                            /**
                             * Encodes the specified Money message. Does not implicitly {@link google.ads.googleads.v11.common.Money.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {google.ads.googleads.v11.common.IMoney} m Money message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Money.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(26).string(m.currency_code);
                                if (m.amount_micros != null && Object.hasOwnProperty.call(m, "amount_micros"))
                                    w.uint32(32).int64(m.amount_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Money message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Money.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {google.ads.googleads.v11.common.IMoney} message Money message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Money.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Money message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.Money} Money
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Money.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Money();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.currency_code = r.string();
                                        break;
                                    case 4:
                                        m.amount_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Money message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.Money} Money
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Money.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Money message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Money.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.amount_micros != null && m.hasOwnProperty("amount_micros")) {
                                    p._amount_micros = 1;
                                    if (!$util.isInteger(m.amount_micros) && !(m.amount_micros && $util.isInteger(m.amount_micros.low) && $util.isInteger(m.amount_micros.high)))
                                        return "amount_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Money message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.Money} Money
                             */
                            Money.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.Money)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.Money();
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.amount_micros != null) {
                                    if ($util.Long)
                                        (m.amount_micros = $util.Long.fromValue(d.amount_micros)).unsigned = false;
                                    else if (typeof d.amount_micros === "string")
                                        m.amount_micros = parseInt(d.amount_micros, 10);
                                    else if (typeof d.amount_micros === "number")
                                        m.amount_micros = d.amount_micros;
                                    else if (typeof d.amount_micros === "object")
                                        m.amount_micros = new $util.LongBits(d.amount_micros.low >>> 0, d.amount_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Money message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.Money
                             * @static
                             * @param {google.ads.googleads.v11.common.Money} m Money
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Money.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.amount_micros != null && m.hasOwnProperty("amount_micros")) {
                                    if (typeof m.amount_micros === "number")
                                        d.amount_micros = o.longs === String ? String(m.amount_micros) : m.amount_micros;
                                    else
                                        d.amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.amount_micros) : o.longs === Number ? new $util.LongBits(m.amount_micros.low >>> 0, m.amount_micros.high >>> 0).toNumber() : m.amount_micros;
                                    if (o.oneofs)
                                        d._amount_micros = "amount_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Money to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.Money
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Money.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Money;
                        })();
    
                        common.AssetUsage = (function() {
    
                            /**
                             * Properties of an AssetUsage.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAssetUsage
                             * @property {string|null} [asset] AssetUsage asset
                             * @property {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.ServedAssetFieldType|null} [served_asset_field_type] AssetUsage served_asset_field_type
                             */
    
                            /**
                             * Constructs a new AssetUsage.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AssetUsage.
                             * @implements IAssetUsage
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAssetUsage=} [p] Properties to set
                             */
                            function AssetUsage(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetUsage asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @instance
                             */
                            AssetUsage.prototype.asset = "";
    
                            /**
                             * AssetUsage served_asset_field_type.
                             * @member {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.ServedAssetFieldType} served_asset_field_type
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @instance
                             */
                            AssetUsage.prototype.served_asset_field_type = 0;
    
                            /**
                             * Creates a new AssetUsage instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {google.ads.googleads.v11.common.IAssetUsage=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AssetUsage} AssetUsage instance
                             */
                            AssetUsage.create = function create(properties) {
                                return new AssetUsage(properties);
                            };
    
                            /**
                             * Encodes the specified AssetUsage message. Does not implicitly {@link google.ads.googleads.v11.common.AssetUsage.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {google.ads.googleads.v11.common.IAssetUsage} m AssetUsage message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetUsage.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(10).string(m.asset);
                                if (m.served_asset_field_type != null && Object.hasOwnProperty.call(m, "served_asset_field_type"))
                                    w.uint32(16).int32(m.served_asset_field_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetUsage message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AssetUsage.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {google.ads.googleads.v11.common.IAssetUsage} message AssetUsage message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetUsage.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetUsage message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AssetUsage} AssetUsage
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetUsage.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AssetUsage();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.asset = r.string();
                                        break;
                                    case 2:
                                        m.served_asset_field_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetUsage message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AssetUsage} AssetUsage
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetUsage.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetUsage message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetUsage.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.served_asset_field_type != null && m.hasOwnProperty("served_asset_field_type")) {
                                    switch (m.served_asset_field_type) {
                                    default:
                                        return "served_asset_field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetUsage message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AssetUsage} AssetUsage
                             */
                            AssetUsage.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AssetUsage)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AssetUsage();
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                switch (d.served_asset_field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.served_asset_field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.served_asset_field_type = 1;
                                    break;
                                case "HEADLINE_1":
                                case 2:
                                    m.served_asset_field_type = 2;
                                    break;
                                case "HEADLINE_2":
                                case 3:
                                    m.served_asset_field_type = 3;
                                    break;
                                case "HEADLINE_3":
                                case 4:
                                    m.served_asset_field_type = 4;
                                    break;
                                case "DESCRIPTION_1":
                                case 5:
                                    m.served_asset_field_type = 5;
                                    break;
                                case "DESCRIPTION_2":
                                case 6:
                                    m.served_asset_field_type = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetUsage message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @static
                             * @param {google.ads.googleads.v11.common.AssetUsage} m AssetUsage
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetUsage.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.asset = "";
                                    d.served_asset_field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.served_asset_field_type != null && m.hasOwnProperty("served_asset_field_type")) {
                                    d.served_asset_field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.ServedAssetFieldType[m.served_asset_field_type] : m.served_asset_field_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetUsage to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AssetUsage
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetUsage.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetUsage;
                        })();
    
                        common.AudienceDimension = (function() {
    
                            /**
                             * Properties of an AudienceDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAudienceDimension
                             * @property {google.ads.googleads.v11.common.IAgeDimension|null} [age] AudienceDimension age
                             * @property {google.ads.googleads.v11.common.IGenderDimension|null} [gender] AudienceDimension gender
                             * @property {google.ads.googleads.v11.common.IHouseholdIncomeDimension|null} [household_income] AudienceDimension household_income
                             * @property {google.ads.googleads.v11.common.IParentalStatusDimension|null} [parental_status] AudienceDimension parental_status
                             * @property {google.ads.googleads.v11.common.IAudienceSegmentDimension|null} [audience_segments] AudienceDimension audience_segments
                             */
    
                            /**
                             * Constructs a new AudienceDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AudienceDimension.
                             * @implements IAudienceDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAudienceDimension=} [p] Properties to set
                             */
                            function AudienceDimension(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceDimension age.
                             * @member {google.ads.googleads.v11.common.IAgeDimension|null|undefined} age
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @instance
                             */
                            AudienceDimension.prototype.age = null;
    
                            /**
                             * AudienceDimension gender.
                             * @member {google.ads.googleads.v11.common.IGenderDimension|null|undefined} gender
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @instance
                             */
                            AudienceDimension.prototype.gender = null;
    
                            /**
                             * AudienceDimension household_income.
                             * @member {google.ads.googleads.v11.common.IHouseholdIncomeDimension|null|undefined} household_income
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @instance
                             */
                            AudienceDimension.prototype.household_income = null;
    
                            /**
                             * AudienceDimension parental_status.
                             * @member {google.ads.googleads.v11.common.IParentalStatusDimension|null|undefined} parental_status
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @instance
                             */
                            AudienceDimension.prototype.parental_status = null;
    
                            /**
                             * AudienceDimension audience_segments.
                             * @member {google.ads.googleads.v11.common.IAudienceSegmentDimension|null|undefined} audience_segments
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @instance
                             */
                            AudienceDimension.prototype.audience_segments = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AudienceDimension dimension.
                             * @member {"age"|"gender"|"household_income"|"parental_status"|"audience_segments"|undefined} dimension
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @instance
                             */
                            Object.defineProperty(AudienceDimension.prototype, "dimension", {
                                get: $util.oneOfGetter($oneOfFields = ["age", "gender", "household_income", "parental_status", "audience_segments"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AudienceDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AudienceDimension} AudienceDimension instance
                             */
                            AudienceDimension.create = function create(properties) {
                                return new AudienceDimension(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceDimension message. Does not implicitly {@link google.ads.googleads.v11.common.AudienceDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceDimension} m AudienceDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.age != null && Object.hasOwnProperty.call(m, "age"))
                                    $root.google.ads.googleads.v11.common.AgeDimension.encode(m.age, w.uint32(10).fork()).ldelim();
                                if (m.gender != null && Object.hasOwnProperty.call(m, "gender"))
                                    $root.google.ads.googleads.v11.common.GenderDimension.encode(m.gender, w.uint32(18).fork()).ldelim();
                                if (m.household_income != null && Object.hasOwnProperty.call(m, "household_income"))
                                    $root.google.ads.googleads.v11.common.HouseholdIncomeDimension.encode(m.household_income, w.uint32(26).fork()).ldelim();
                                if (m.parental_status != null && Object.hasOwnProperty.call(m, "parental_status"))
                                    $root.google.ads.googleads.v11.common.ParentalStatusDimension.encode(m.parental_status, w.uint32(34).fork()).ldelim();
                                if (m.audience_segments != null && Object.hasOwnProperty.call(m, "audience_segments"))
                                    $root.google.ads.googleads.v11.common.AudienceSegmentDimension.encode(m.audience_segments, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AudienceDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceDimension} message AudienceDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AudienceDimension} AudienceDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AudienceDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.age = $root.google.ads.googleads.v11.common.AgeDimension.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.gender = $root.google.ads.googleads.v11.common.GenderDimension.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.household_income = $root.google.ads.googleads.v11.common.HouseholdIncomeDimension.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusDimension.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.audience_segments = $root.google.ads.googleads.v11.common.AudienceSegmentDimension.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AudienceDimension} AudienceDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.age != null && m.hasOwnProperty("age")) {
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AgeDimension.verify(m.age);
                                        if (e)
                                            return "age." + e;
                                    }
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GenderDimension.verify(m.gender);
                                        if (e)
                                            return "gender." + e;
                                    }
                                }
                                if (m.household_income != null && m.hasOwnProperty("household_income")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HouseholdIncomeDimension.verify(m.household_income);
                                        if (e)
                                            return "household_income." + e;
                                    }
                                }
                                if (m.parental_status != null && m.hasOwnProperty("parental_status")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ParentalStatusDimension.verify(m.parental_status);
                                        if (e)
                                            return "parental_status." + e;
                                    }
                                }
                                if (m.audience_segments != null && m.hasOwnProperty("audience_segments")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AudienceSegmentDimension.verify(m.audience_segments);
                                        if (e)
                                            return "audience_segments." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AudienceDimension} AudienceDimension
                             */
                            AudienceDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AudienceDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AudienceDimension();
                                if (d.age != null) {
                                    if (typeof d.age !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceDimension.age: object expected");
                                    m.age = $root.google.ads.googleads.v11.common.AgeDimension.fromObject(d.age);
                                }
                                if (d.gender != null) {
                                    if (typeof d.gender !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceDimension.gender: object expected");
                                    m.gender = $root.google.ads.googleads.v11.common.GenderDimension.fromObject(d.gender);
                                }
                                if (d.household_income != null) {
                                    if (typeof d.household_income !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceDimension.household_income: object expected");
                                    m.household_income = $root.google.ads.googleads.v11.common.HouseholdIncomeDimension.fromObject(d.household_income);
                                }
                                if (d.parental_status != null) {
                                    if (typeof d.parental_status !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceDimension.parental_status: object expected");
                                    m.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusDimension.fromObject(d.parental_status);
                                }
                                if (d.audience_segments != null) {
                                    if (typeof d.audience_segments !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceDimension.audience_segments: object expected");
                                    m.audience_segments = $root.google.ads.googleads.v11.common.AudienceSegmentDimension.fromObject(d.audience_segments);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.AudienceDimension} m AudienceDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.age != null && m.hasOwnProperty("age")) {
                                    d.age = $root.google.ads.googleads.v11.common.AgeDimension.toObject(m.age, o);
                                    if (o.oneofs)
                                        d.dimension = "age";
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    d.gender = $root.google.ads.googleads.v11.common.GenderDimension.toObject(m.gender, o);
                                    if (o.oneofs)
                                        d.dimension = "gender";
                                }
                                if (m.household_income != null && m.hasOwnProperty("household_income")) {
                                    d.household_income = $root.google.ads.googleads.v11.common.HouseholdIncomeDimension.toObject(m.household_income, o);
                                    if (o.oneofs)
                                        d.dimension = "household_income";
                                }
                                if (m.parental_status != null && m.hasOwnProperty("parental_status")) {
                                    d.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusDimension.toObject(m.parental_status, o);
                                    if (o.oneofs)
                                        d.dimension = "parental_status";
                                }
                                if (m.audience_segments != null && m.hasOwnProperty("audience_segments")) {
                                    d.audience_segments = $root.google.ads.googleads.v11.common.AudienceSegmentDimension.toObject(m.audience_segments, o);
                                    if (o.oneofs)
                                        d.dimension = "audience_segments";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AudienceDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceDimension;
                        })();
    
                        common.AudienceExclusionDimension = (function() {
    
                            /**
                             * Properties of an AudienceExclusionDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAudienceExclusionDimension
                             * @property {Array.<google.ads.googleads.v11.common.IExclusionSegment>|null} [exclusions] AudienceExclusionDimension exclusions
                             */
    
                            /**
                             * Constructs a new AudienceExclusionDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AudienceExclusionDimension.
                             * @implements IAudienceExclusionDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAudienceExclusionDimension=} [p] Properties to set
                             */
                            function AudienceExclusionDimension(p) {
                                this.exclusions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceExclusionDimension exclusions.
                             * @member {Array.<google.ads.googleads.v11.common.IExclusionSegment>} exclusions
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @instance
                             */
                            AudienceExclusionDimension.prototype.exclusions = $util.emptyArray;
    
                            /**
                             * Creates a new AudienceExclusionDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceExclusionDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AudienceExclusionDimension} AudienceExclusionDimension instance
                             */
                            AudienceExclusionDimension.create = function create(properties) {
                                return new AudienceExclusionDimension(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceExclusionDimension message. Does not implicitly {@link google.ads.googleads.v11.common.AudienceExclusionDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceExclusionDimension} m AudienceExclusionDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceExclusionDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.exclusions != null && m.exclusions.length) {
                                    for (var i = 0; i < m.exclusions.length; ++i)
                                        $root.google.ads.googleads.v11.common.ExclusionSegment.encode(m.exclusions[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceExclusionDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AudienceExclusionDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceExclusionDimension} message AudienceExclusionDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceExclusionDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceExclusionDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AudienceExclusionDimension} AudienceExclusionDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceExclusionDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AudienceExclusionDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.exclusions && m.exclusions.length))
                                            m.exclusions = [];
                                        m.exclusions.push($root.google.ads.googleads.v11.common.ExclusionSegment.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceExclusionDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AudienceExclusionDimension} AudienceExclusionDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceExclusionDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceExclusionDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceExclusionDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.exclusions != null && m.hasOwnProperty("exclusions")) {
                                    if (!Array.isArray(m.exclusions))
                                        return "exclusions: array expected";
                                    for (var i = 0; i < m.exclusions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.ExclusionSegment.verify(m.exclusions[i]);
                                            if (e)
                                                return "exclusions." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceExclusionDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AudienceExclusionDimension} AudienceExclusionDimension
                             */
                            AudienceExclusionDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AudienceExclusionDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AudienceExclusionDimension();
                                if (d.exclusions) {
                                    if (!Array.isArray(d.exclusions))
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceExclusionDimension.exclusions: array expected");
                                    m.exclusions = [];
                                    for (var i = 0; i < d.exclusions.length; ++i) {
                                        if (typeof d.exclusions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AudienceExclusionDimension.exclusions: object expected");
                                        m.exclusions[i] = $root.google.ads.googleads.v11.common.ExclusionSegment.fromObject(d.exclusions[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceExclusionDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.AudienceExclusionDimension} m AudienceExclusionDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceExclusionDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.exclusions = [];
                                }
                                if (m.exclusions && m.exclusions.length) {
                                    d.exclusions = [];
                                    for (var j = 0; j < m.exclusions.length; ++j) {
                                        d.exclusions[j] = $root.google.ads.googleads.v11.common.ExclusionSegment.toObject(m.exclusions[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceExclusionDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AudienceExclusionDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceExclusionDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceExclusionDimension;
                        })();
    
                        common.ExclusionSegment = (function() {
    
                            /**
                             * Properties of an ExclusionSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IExclusionSegment
                             * @property {google.ads.googleads.v11.common.IUserListSegment|null} [user_list] ExclusionSegment user_list
                             */
    
                            /**
                             * Constructs a new ExclusionSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ExclusionSegment.
                             * @implements IExclusionSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IExclusionSegment=} [p] Properties to set
                             */
                            function ExclusionSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExclusionSegment user_list.
                             * @member {google.ads.googleads.v11.common.IUserListSegment|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @instance
                             */
                            ExclusionSegment.prototype.user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExclusionSegment segment.
                             * @member {"user_list"|undefined} segment
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @instance
                             */
                            Object.defineProperty(ExclusionSegment.prototype, "segment", {
                                get: $util.oneOfGetter($oneOfFields = ["user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExclusionSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IExclusionSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ExclusionSegment} ExclusionSegment instance
                             */
                            ExclusionSegment.create = function create(properties) {
                                return new ExclusionSegment(properties);
                            };
    
                            /**
                             * Encodes the specified ExclusionSegment message. Does not implicitly {@link google.ads.googleads.v11.common.ExclusionSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IExclusionSegment} m ExclusionSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExclusionSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    $root.google.ads.googleads.v11.common.UserListSegment.encode(m.user_list, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExclusionSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ExclusionSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IExclusionSegment} message ExclusionSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExclusionSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExclusionSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ExclusionSegment} ExclusionSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExclusionSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ExclusionSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.user_list = $root.google.ads.googleads.v11.common.UserListSegment.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExclusionSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ExclusionSegment} ExclusionSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExclusionSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExclusionSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExclusionSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    p.segment = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListSegment.verify(m.user_list);
                                        if (e)
                                            return "user_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExclusionSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ExclusionSegment} ExclusionSegment
                             */
                            ExclusionSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ExclusionSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ExclusionSegment();
                                if (d.user_list != null) {
                                    if (typeof d.user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ExclusionSegment.user_list: object expected");
                                    m.user_list = $root.google.ads.googleads.v11.common.UserListSegment.fromObject(d.user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExclusionSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.ExclusionSegment} m ExclusionSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExclusionSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = $root.google.ads.googleads.v11.common.UserListSegment.toObject(m.user_list, o);
                                    if (o.oneofs)
                                        d.segment = "user_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExclusionSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ExclusionSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExclusionSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExclusionSegment;
                        })();
    
                        common.AgeDimension = (function() {
    
                            /**
                             * Properties of an AgeDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAgeDimension
                             * @property {Array.<google.ads.googleads.v11.common.IAgeSegment>|null} [age_ranges] AgeDimension age_ranges
                             * @property {boolean|null} [include_undetermined] AgeDimension include_undetermined
                             */
    
                            /**
                             * Constructs a new AgeDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AgeDimension.
                             * @implements IAgeDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAgeDimension=} [p] Properties to set
                             */
                            function AgeDimension(p) {
                                this.age_ranges = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AgeDimension age_ranges.
                             * @member {Array.<google.ads.googleads.v11.common.IAgeSegment>} age_ranges
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @instance
                             */
                            AgeDimension.prototype.age_ranges = $util.emptyArray;
    
                            /**
                             * AgeDimension include_undetermined.
                             * @member {boolean|null|undefined} include_undetermined
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @instance
                             */
                            AgeDimension.prototype.include_undetermined = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AgeDimension _include_undetermined.
                             * @member {"include_undetermined"|undefined} _include_undetermined
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @instance
                             */
                            Object.defineProperty(AgeDimension.prototype, "_include_undetermined", {
                                get: $util.oneOfGetter($oneOfFields = ["include_undetermined"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AgeDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AgeDimension} AgeDimension instance
                             */
                            AgeDimension.create = function create(properties) {
                                return new AgeDimension(properties);
                            };
    
                            /**
                             * Encodes the specified AgeDimension message. Does not implicitly {@link google.ads.googleads.v11.common.AgeDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeDimension} m AgeDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.age_ranges != null && m.age_ranges.length) {
                                    for (var i = 0; i < m.age_ranges.length; ++i)
                                        $root.google.ads.googleads.v11.common.AgeSegment.encode(m.age_ranges[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.include_undetermined != null && Object.hasOwnProperty.call(m, "include_undetermined"))
                                    w.uint32(16).bool(m.include_undetermined);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AgeDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AgeDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeDimension} message AgeDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AgeDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AgeDimension} AgeDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AgeDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.age_ranges && m.age_ranges.length))
                                            m.age_ranges = [];
                                        m.age_ranges.push($root.google.ads.googleads.v11.common.AgeSegment.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.include_undetermined = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AgeDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AgeDimension} AgeDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AgeDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AgeDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.age_ranges != null && m.hasOwnProperty("age_ranges")) {
                                    if (!Array.isArray(m.age_ranges))
                                        return "age_ranges: array expected";
                                    for (var i = 0; i < m.age_ranges.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AgeSegment.verify(m.age_ranges[i]);
                                            if (e)
                                                return "age_ranges." + e;
                                        }
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    p._include_undetermined = 1;
                                    if (typeof m.include_undetermined !== "boolean")
                                        return "include_undetermined: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AgeDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AgeDimension} AgeDimension
                             */
                            AgeDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AgeDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AgeDimension();
                                if (d.age_ranges) {
                                    if (!Array.isArray(d.age_ranges))
                                        throw TypeError(".google.ads.googleads.v11.common.AgeDimension.age_ranges: array expected");
                                    m.age_ranges = [];
                                    for (var i = 0; i < d.age_ranges.length; ++i) {
                                        if (typeof d.age_ranges[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AgeDimension.age_ranges: object expected");
                                        m.age_ranges[i] = $root.google.ads.googleads.v11.common.AgeSegment.fromObject(d.age_ranges[i]);
                                    }
                                }
                                if (d.include_undetermined != null) {
                                    m.include_undetermined = Boolean(d.include_undetermined);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AgeDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.AgeDimension} m AgeDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AgeDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.age_ranges = [];
                                }
                                if (m.age_ranges && m.age_ranges.length) {
                                    d.age_ranges = [];
                                    for (var j = 0; j < m.age_ranges.length; ++j) {
                                        d.age_ranges[j] = $root.google.ads.googleads.v11.common.AgeSegment.toObject(m.age_ranges[j], o);
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    d.include_undetermined = m.include_undetermined;
                                    if (o.oneofs)
                                        d._include_undetermined = "include_undetermined";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AgeDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AgeDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AgeDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AgeDimension;
                        })();
    
                        common.AgeSegment = (function() {
    
                            /**
                             * Properties of an AgeSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAgeSegment
                             * @property {number|null} [min_age] AgeSegment min_age
                             * @property {number|null} [max_age] AgeSegment max_age
                             */
    
                            /**
                             * Constructs a new AgeSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AgeSegment.
                             * @implements IAgeSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAgeSegment=} [p] Properties to set
                             */
                            function AgeSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AgeSegment min_age.
                             * @member {number|null|undefined} min_age
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @instance
                             */
                            AgeSegment.prototype.min_age = null;
    
                            /**
                             * AgeSegment max_age.
                             * @member {number|null|undefined} max_age
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @instance
                             */
                            AgeSegment.prototype.max_age = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AgeSegment _min_age.
                             * @member {"min_age"|undefined} _min_age
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @instance
                             */
                            Object.defineProperty(AgeSegment.prototype, "_min_age", {
                                get: $util.oneOfGetter($oneOfFields = ["min_age"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AgeSegment _max_age.
                             * @member {"max_age"|undefined} _max_age
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @instance
                             */
                            Object.defineProperty(AgeSegment.prototype, "_max_age", {
                                get: $util.oneOfGetter($oneOfFields = ["max_age"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AgeSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AgeSegment} AgeSegment instance
                             */
                            AgeSegment.create = function create(properties) {
                                return new AgeSegment(properties);
                            };
    
                            /**
                             * Encodes the specified AgeSegment message. Does not implicitly {@link google.ads.googleads.v11.common.AgeSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeSegment} m AgeSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.min_age != null && Object.hasOwnProperty.call(m, "min_age"))
                                    w.uint32(8).int32(m.min_age);
                                if (m.max_age != null && Object.hasOwnProperty.call(m, "max_age"))
                                    w.uint32(16).int32(m.max_age);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AgeSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AgeSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IAgeSegment} message AgeSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AgeSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AgeSegment} AgeSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AgeSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.min_age = r.int32();
                                        break;
                                    case 2:
                                        m.max_age = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AgeSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AgeSegment} AgeSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AgeSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AgeSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.min_age != null && m.hasOwnProperty("min_age")) {
                                    p._min_age = 1;
                                    if (!$util.isInteger(m.min_age))
                                        return "min_age: integer expected";
                                }
                                if (m.max_age != null && m.hasOwnProperty("max_age")) {
                                    p._max_age = 1;
                                    if (!$util.isInteger(m.max_age))
                                        return "max_age: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AgeSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AgeSegment} AgeSegment
                             */
                            AgeSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AgeSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AgeSegment();
                                if (d.min_age != null) {
                                    m.min_age = d.min_age | 0;
                                }
                                if (d.max_age != null) {
                                    m.max_age = d.max_age | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AgeSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.AgeSegment} m AgeSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AgeSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.min_age != null && m.hasOwnProperty("min_age")) {
                                    d.min_age = m.min_age;
                                    if (o.oneofs)
                                        d._min_age = "min_age";
                                }
                                if (m.max_age != null && m.hasOwnProperty("max_age")) {
                                    d.max_age = m.max_age;
                                    if (o.oneofs)
                                        d._max_age = "max_age";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AgeSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AgeSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AgeSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AgeSegment;
                        })();
    
                        common.GenderDimension = (function() {
    
                            /**
                             * Properties of a GenderDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IGenderDimension
                             * @property {Array.<google.ads.googleads.v11.enums.GenderTypeEnum.GenderType>|null} [genders] GenderDimension genders
                             * @property {boolean|null} [include_undetermined] GenderDimension include_undetermined
                             */
    
                            /**
                             * Constructs a new GenderDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a GenderDimension.
                             * @implements IGenderDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IGenderDimension=} [p] Properties to set
                             */
                            function GenderDimension(p) {
                                this.genders = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenderDimension genders.
                             * @member {Array.<google.ads.googleads.v11.enums.GenderTypeEnum.GenderType>} genders
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @instance
                             */
                            GenderDimension.prototype.genders = $util.emptyArray;
    
                            /**
                             * GenderDimension include_undetermined.
                             * @member {boolean|null|undefined} include_undetermined
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @instance
                             */
                            GenderDimension.prototype.include_undetermined = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GenderDimension _include_undetermined.
                             * @member {"include_undetermined"|undefined} _include_undetermined
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @instance
                             */
                            Object.defineProperty(GenderDimension.prototype, "_include_undetermined", {
                                get: $util.oneOfGetter($oneOfFields = ["include_undetermined"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GenderDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IGenderDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.GenderDimension} GenderDimension instance
                             */
                            GenderDimension.create = function create(properties) {
                                return new GenderDimension(properties);
                            };
    
                            /**
                             * Encodes the specified GenderDimension message. Does not implicitly {@link google.ads.googleads.v11.common.GenderDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IGenderDimension} m GenderDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.genders != null && m.genders.length) {
                                    w.uint32(10).fork();
                                    for (var i = 0; i < m.genders.length; ++i)
                                        w.int32(m.genders[i]);
                                    w.ldelim();
                                }
                                if (m.include_undetermined != null && Object.hasOwnProperty.call(m, "include_undetermined"))
                                    w.uint32(16).bool(m.include_undetermined);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenderDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.GenderDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IGenderDimension} message GenderDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenderDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.GenderDimension} GenderDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.GenderDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.genders && m.genders.length))
                                            m.genders = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.genders.push(r.int32());
                                        } else
                                            m.genders.push(r.int32());
                                        break;
                                    case 2:
                                        m.include_undetermined = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenderDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.GenderDimension} GenderDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenderDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenderDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.genders != null && m.hasOwnProperty("genders")) {
                                    if (!Array.isArray(m.genders))
                                        return "genders: array expected";
                                    for (var i = 0; i < m.genders.length; ++i) {
                                        switch (m.genders[i]) {
                                        default:
                                            return "genders: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 10:
                                        case 11:
                                        case 20:
                                            break;
                                        }
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    p._include_undetermined = 1;
                                    if (typeof m.include_undetermined !== "boolean")
                                        return "include_undetermined: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenderDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.GenderDimension} GenderDimension
                             */
                            GenderDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.GenderDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.GenderDimension();
                                if (d.genders) {
                                    if (!Array.isArray(d.genders))
                                        throw TypeError(".google.ads.googleads.v11.common.GenderDimension.genders: array expected");
                                    m.genders = [];
                                    for (var i = 0; i < d.genders.length; ++i) {
                                        switch (d.genders[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.genders[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.genders[i] = 1;
                                            break;
                                        case "MALE":
                                        case 10:
                                            m.genders[i] = 10;
                                            break;
                                        case "FEMALE":
                                        case 11:
                                            m.genders[i] = 11;
                                            break;
                                        case "UNDETERMINED":
                                        case 20:
                                            m.genders[i] = 20;
                                            break;
                                        }
                                    }
                                }
                                if (d.include_undetermined != null) {
                                    m.include_undetermined = Boolean(d.include_undetermined);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenderDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.GenderDimension} m GenderDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenderDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.genders = [];
                                }
                                if (m.genders && m.genders.length) {
                                    d.genders = [];
                                    for (var j = 0; j < m.genders.length; ++j) {
                                        d.genders[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.GenderTypeEnum.GenderType[m.genders[j]] : m.genders[j];
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    d.include_undetermined = m.include_undetermined;
                                    if (o.oneofs)
                                        d._include_undetermined = "include_undetermined";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenderDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.GenderDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenderDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenderDimension;
                        })();
    
                        common.HouseholdIncomeDimension = (function() {
    
                            /**
                             * Properties of an HouseholdIncomeDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHouseholdIncomeDimension
                             * @property {Array.<google.ads.googleads.v11.enums.IncomeRangeTypeEnum.IncomeRangeType>|null} [income_ranges] HouseholdIncomeDimension income_ranges
                             * @property {boolean|null} [include_undetermined] HouseholdIncomeDimension include_undetermined
                             */
    
                            /**
                             * Constructs a new HouseholdIncomeDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an HouseholdIncomeDimension.
                             * @implements IHouseholdIncomeDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHouseholdIncomeDimension=} [p] Properties to set
                             */
                            function HouseholdIncomeDimension(p) {
                                this.income_ranges = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HouseholdIncomeDimension income_ranges.
                             * @member {Array.<google.ads.googleads.v11.enums.IncomeRangeTypeEnum.IncomeRangeType>} income_ranges
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @instance
                             */
                            HouseholdIncomeDimension.prototype.income_ranges = $util.emptyArray;
    
                            /**
                             * HouseholdIncomeDimension include_undetermined.
                             * @member {boolean|null|undefined} include_undetermined
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @instance
                             */
                            HouseholdIncomeDimension.prototype.include_undetermined = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HouseholdIncomeDimension _include_undetermined.
                             * @member {"include_undetermined"|undefined} _include_undetermined
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @instance
                             */
                            Object.defineProperty(HouseholdIncomeDimension.prototype, "_include_undetermined", {
                                get: $util.oneOfGetter($oneOfFields = ["include_undetermined"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HouseholdIncomeDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IHouseholdIncomeDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HouseholdIncomeDimension} HouseholdIncomeDimension instance
                             */
                            HouseholdIncomeDimension.create = function create(properties) {
                                return new HouseholdIncomeDimension(properties);
                            };
    
                            /**
                             * Encodes the specified HouseholdIncomeDimension message. Does not implicitly {@link google.ads.googleads.v11.common.HouseholdIncomeDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IHouseholdIncomeDimension} m HouseholdIncomeDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HouseholdIncomeDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.income_ranges != null && m.income_ranges.length) {
                                    w.uint32(10).fork();
                                    for (var i = 0; i < m.income_ranges.length; ++i)
                                        w.int32(m.income_ranges[i]);
                                    w.ldelim();
                                }
                                if (m.include_undetermined != null && Object.hasOwnProperty.call(m, "include_undetermined"))
                                    w.uint32(16).bool(m.include_undetermined);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HouseholdIncomeDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HouseholdIncomeDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IHouseholdIncomeDimension} message HouseholdIncomeDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HouseholdIncomeDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an HouseholdIncomeDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HouseholdIncomeDimension} HouseholdIncomeDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HouseholdIncomeDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HouseholdIncomeDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.income_ranges && m.income_ranges.length))
                                            m.income_ranges = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.income_ranges.push(r.int32());
                                        } else
                                            m.income_ranges.push(r.int32());
                                        break;
                                    case 2:
                                        m.include_undetermined = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an HouseholdIncomeDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HouseholdIncomeDimension} HouseholdIncomeDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HouseholdIncomeDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an HouseholdIncomeDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HouseholdIncomeDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.income_ranges != null && m.hasOwnProperty("income_ranges")) {
                                    if (!Array.isArray(m.income_ranges))
                                        return "income_ranges: array expected";
                                    for (var i = 0; i < m.income_ranges.length; ++i) {
                                        switch (m.income_ranges[i]) {
                                        default:
                                            return "income_ranges: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 510001:
                                        case 510002:
                                        case 510003:
                                        case 510004:
                                        case 510005:
                                        case 510006:
                                        case 510000:
                                            break;
                                        }
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    p._include_undetermined = 1;
                                    if (typeof m.include_undetermined !== "boolean")
                                        return "include_undetermined: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an HouseholdIncomeDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HouseholdIncomeDimension} HouseholdIncomeDimension
                             */
                            HouseholdIncomeDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HouseholdIncomeDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HouseholdIncomeDimension();
                                if (d.income_ranges) {
                                    if (!Array.isArray(d.income_ranges))
                                        throw TypeError(".google.ads.googleads.v11.common.HouseholdIncomeDimension.income_ranges: array expected");
                                    m.income_ranges = [];
                                    for (var i = 0; i < d.income_ranges.length; ++i) {
                                        switch (d.income_ranges[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.income_ranges[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.income_ranges[i] = 1;
                                            break;
                                        case "INCOME_RANGE_0_50":
                                        case 510001:
                                            m.income_ranges[i] = 510001;
                                            break;
                                        case "INCOME_RANGE_50_60":
                                        case 510002:
                                            m.income_ranges[i] = 510002;
                                            break;
                                        case "INCOME_RANGE_60_70":
                                        case 510003:
                                            m.income_ranges[i] = 510003;
                                            break;
                                        case "INCOME_RANGE_70_80":
                                        case 510004:
                                            m.income_ranges[i] = 510004;
                                            break;
                                        case "INCOME_RANGE_80_90":
                                        case 510005:
                                            m.income_ranges[i] = 510005;
                                            break;
                                        case "INCOME_RANGE_90_UP":
                                        case 510006:
                                            m.income_ranges[i] = 510006;
                                            break;
                                        case "INCOME_RANGE_UNDETERMINED":
                                        case 510000:
                                            m.income_ranges[i] = 510000;
                                            break;
                                        }
                                    }
                                }
                                if (d.include_undetermined != null) {
                                    m.include_undetermined = Boolean(d.include_undetermined);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an HouseholdIncomeDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.HouseholdIncomeDimension} m HouseholdIncomeDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HouseholdIncomeDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.income_ranges = [];
                                }
                                if (m.income_ranges && m.income_ranges.length) {
                                    d.income_ranges = [];
                                    for (var j = 0; j < m.income_ranges.length; ++j) {
                                        d.income_ranges[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.IncomeRangeTypeEnum.IncomeRangeType[m.income_ranges[j]] : m.income_ranges[j];
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    d.include_undetermined = m.include_undetermined;
                                    if (o.oneofs)
                                        d._include_undetermined = "include_undetermined";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HouseholdIncomeDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HouseholdIncomeDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HouseholdIncomeDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HouseholdIncomeDimension;
                        })();
    
                        common.ParentalStatusDimension = (function() {
    
                            /**
                             * Properties of a ParentalStatusDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IParentalStatusDimension
                             * @property {Array.<google.ads.googleads.v11.enums.ParentalStatusTypeEnum.ParentalStatusType>|null} [parental_statuses] ParentalStatusDimension parental_statuses
                             * @property {boolean|null} [include_undetermined] ParentalStatusDimension include_undetermined
                             */
    
                            /**
                             * Constructs a new ParentalStatusDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ParentalStatusDimension.
                             * @implements IParentalStatusDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IParentalStatusDimension=} [p] Properties to set
                             */
                            function ParentalStatusDimension(p) {
                                this.parental_statuses = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ParentalStatusDimension parental_statuses.
                             * @member {Array.<google.ads.googleads.v11.enums.ParentalStatusTypeEnum.ParentalStatusType>} parental_statuses
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @instance
                             */
                            ParentalStatusDimension.prototype.parental_statuses = $util.emptyArray;
    
                            /**
                             * ParentalStatusDimension include_undetermined.
                             * @member {boolean|null|undefined} include_undetermined
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @instance
                             */
                            ParentalStatusDimension.prototype.include_undetermined = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ParentalStatusDimension _include_undetermined.
                             * @member {"include_undetermined"|undefined} _include_undetermined
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @instance
                             */
                            Object.defineProperty(ParentalStatusDimension.prototype, "_include_undetermined", {
                                get: $util.oneOfGetter($oneOfFields = ["include_undetermined"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ParentalStatusDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IParentalStatusDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ParentalStatusDimension} ParentalStatusDimension instance
                             */
                            ParentalStatusDimension.create = function create(properties) {
                                return new ParentalStatusDimension(properties);
                            };
    
                            /**
                             * Encodes the specified ParentalStatusDimension message. Does not implicitly {@link google.ads.googleads.v11.common.ParentalStatusDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IParentalStatusDimension} m ParentalStatusDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.parental_statuses != null && m.parental_statuses.length) {
                                    w.uint32(10).fork();
                                    for (var i = 0; i < m.parental_statuses.length; ++i)
                                        w.int32(m.parental_statuses[i]);
                                    w.ldelim();
                                }
                                if (m.include_undetermined != null && Object.hasOwnProperty.call(m, "include_undetermined"))
                                    w.uint32(16).bool(m.include_undetermined);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ParentalStatusDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ParentalStatusDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IParentalStatusDimension} message ParentalStatusDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ParentalStatusDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ParentalStatusDimension} ParentalStatusDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ParentalStatusDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.parental_statuses && m.parental_statuses.length))
                                            m.parental_statuses = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.parental_statuses.push(r.int32());
                                        } else
                                            m.parental_statuses.push(r.int32());
                                        break;
                                    case 2:
                                        m.include_undetermined = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ParentalStatusDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ParentalStatusDimension} ParentalStatusDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ParentalStatusDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ParentalStatusDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.parental_statuses != null && m.hasOwnProperty("parental_statuses")) {
                                    if (!Array.isArray(m.parental_statuses))
                                        return "parental_statuses: array expected";
                                    for (var i = 0; i < m.parental_statuses.length; ++i) {
                                        switch (m.parental_statuses[i]) {
                                        default:
                                            return "parental_statuses: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 300:
                                        case 301:
                                        case 302:
                                            break;
                                        }
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    p._include_undetermined = 1;
                                    if (typeof m.include_undetermined !== "boolean")
                                        return "include_undetermined: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ParentalStatusDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ParentalStatusDimension} ParentalStatusDimension
                             */
                            ParentalStatusDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ParentalStatusDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ParentalStatusDimension();
                                if (d.parental_statuses) {
                                    if (!Array.isArray(d.parental_statuses))
                                        throw TypeError(".google.ads.googleads.v11.common.ParentalStatusDimension.parental_statuses: array expected");
                                    m.parental_statuses = [];
                                    for (var i = 0; i < d.parental_statuses.length; ++i) {
                                        switch (d.parental_statuses[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.parental_statuses[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.parental_statuses[i] = 1;
                                            break;
                                        case "PARENT":
                                        case 300:
                                            m.parental_statuses[i] = 300;
                                            break;
                                        case "NOT_A_PARENT":
                                        case 301:
                                            m.parental_statuses[i] = 301;
                                            break;
                                        case "UNDETERMINED":
                                        case 302:
                                            m.parental_statuses[i] = 302;
                                            break;
                                        }
                                    }
                                }
                                if (d.include_undetermined != null) {
                                    m.include_undetermined = Boolean(d.include_undetermined);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ParentalStatusDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.ParentalStatusDimension} m ParentalStatusDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ParentalStatusDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.parental_statuses = [];
                                }
                                if (m.parental_statuses && m.parental_statuses.length) {
                                    d.parental_statuses = [];
                                    for (var j = 0; j < m.parental_statuses.length; ++j) {
                                        d.parental_statuses[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.ParentalStatusTypeEnum.ParentalStatusType[m.parental_statuses[j]] : m.parental_statuses[j];
                                    }
                                }
                                if (m.include_undetermined != null && m.hasOwnProperty("include_undetermined")) {
                                    d.include_undetermined = m.include_undetermined;
                                    if (o.oneofs)
                                        d._include_undetermined = "include_undetermined";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ParentalStatusDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ParentalStatusDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ParentalStatusDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ParentalStatusDimension;
                        })();
    
                        common.AudienceSegmentDimension = (function() {
    
                            /**
                             * Properties of an AudienceSegmentDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAudienceSegmentDimension
                             * @property {Array.<google.ads.googleads.v11.common.IAudienceSegment>|null} [segments] AudienceSegmentDimension segments
                             */
    
                            /**
                             * Constructs a new AudienceSegmentDimension.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AudienceSegmentDimension.
                             * @implements IAudienceSegmentDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAudienceSegmentDimension=} [p] Properties to set
                             */
                            function AudienceSegmentDimension(p) {
                                this.segments = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceSegmentDimension segments.
                             * @member {Array.<google.ads.googleads.v11.common.IAudienceSegment>} segments
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @instance
                             */
                            AudienceSegmentDimension.prototype.segments = $util.emptyArray;
    
                            /**
                             * Creates a new AudienceSegmentDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceSegmentDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AudienceSegmentDimension} AudienceSegmentDimension instance
                             */
                            AudienceSegmentDimension.create = function create(properties) {
                                return new AudienceSegmentDimension(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceSegmentDimension message. Does not implicitly {@link google.ads.googleads.v11.common.AudienceSegmentDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceSegmentDimension} m AudienceSegmentDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceSegmentDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.segments != null && m.segments.length) {
                                    for (var i = 0; i < m.segments.length; ++i)
                                        $root.google.ads.googleads.v11.common.AudienceSegment.encode(m.segments[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceSegmentDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AudienceSegmentDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceSegmentDimension} message AudienceSegmentDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceSegmentDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceSegmentDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AudienceSegmentDimension} AudienceSegmentDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceSegmentDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AudienceSegmentDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.segments && m.segments.length))
                                            m.segments = [];
                                        m.segments.push($root.google.ads.googleads.v11.common.AudienceSegment.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceSegmentDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AudienceSegmentDimension} AudienceSegmentDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceSegmentDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceSegmentDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceSegmentDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.segments != null && m.hasOwnProperty("segments")) {
                                    if (!Array.isArray(m.segments))
                                        return "segments: array expected";
                                    for (var i = 0; i < m.segments.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AudienceSegment.verify(m.segments[i]);
                                            if (e)
                                                return "segments." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceSegmentDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AudienceSegmentDimension} AudienceSegmentDimension
                             */
                            AudienceSegmentDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AudienceSegmentDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AudienceSegmentDimension();
                                if (d.segments) {
                                    if (!Array.isArray(d.segments))
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceSegmentDimension.segments: array expected");
                                    m.segments = [];
                                    for (var i = 0; i < d.segments.length; ++i) {
                                        if (typeof d.segments[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AudienceSegmentDimension.segments: object expected");
                                        m.segments[i] = $root.google.ads.googleads.v11.common.AudienceSegment.fromObject(d.segments[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceSegmentDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @static
                             * @param {google.ads.googleads.v11.common.AudienceSegmentDimension} m AudienceSegmentDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceSegmentDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.segments = [];
                                }
                                if (m.segments && m.segments.length) {
                                    d.segments = [];
                                    for (var j = 0; j < m.segments.length; ++j) {
                                        d.segments[j] = $root.google.ads.googleads.v11.common.AudienceSegment.toObject(m.segments[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceSegmentDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AudienceSegmentDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceSegmentDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceSegmentDimension;
                        })();
    
                        common.AudienceSegment = (function() {
    
                            /**
                             * Properties of an AudienceSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAudienceSegment
                             * @property {google.ads.googleads.v11.common.IUserListSegment|null} [user_list] AudienceSegment user_list
                             * @property {google.ads.googleads.v11.common.IUserInterestSegment|null} [user_interest] AudienceSegment user_interest
                             * @property {google.ads.googleads.v11.common.ILifeEventSegment|null} [life_event] AudienceSegment life_event
                             * @property {google.ads.googleads.v11.common.IDetailedDemographicSegment|null} [detailed_demographic] AudienceSegment detailed_demographic
                             * @property {google.ads.googleads.v11.common.ICustomAudienceSegment|null} [custom_audience] AudienceSegment custom_audience
                             */
    
                            /**
                             * Constructs a new AudienceSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AudienceSegment.
                             * @implements IAudienceSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAudienceSegment=} [p] Properties to set
                             */
                            function AudienceSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceSegment user_list.
                             * @member {google.ads.googleads.v11.common.IUserListSegment|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @instance
                             */
                            AudienceSegment.prototype.user_list = null;
    
                            /**
                             * AudienceSegment user_interest.
                             * @member {google.ads.googleads.v11.common.IUserInterestSegment|null|undefined} user_interest
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @instance
                             */
                            AudienceSegment.prototype.user_interest = null;
    
                            /**
                             * AudienceSegment life_event.
                             * @member {google.ads.googleads.v11.common.ILifeEventSegment|null|undefined} life_event
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @instance
                             */
                            AudienceSegment.prototype.life_event = null;
    
                            /**
                             * AudienceSegment detailed_demographic.
                             * @member {google.ads.googleads.v11.common.IDetailedDemographicSegment|null|undefined} detailed_demographic
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @instance
                             */
                            AudienceSegment.prototype.detailed_demographic = null;
    
                            /**
                             * AudienceSegment custom_audience.
                             * @member {google.ads.googleads.v11.common.ICustomAudienceSegment|null|undefined} custom_audience
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @instance
                             */
                            AudienceSegment.prototype.custom_audience = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AudienceSegment segment.
                             * @member {"user_list"|"user_interest"|"life_event"|"detailed_demographic"|"custom_audience"|undefined} segment
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @instance
                             */
                            Object.defineProperty(AudienceSegment.prototype, "segment", {
                                get: $util.oneOfGetter($oneOfFields = ["user_list", "user_interest", "life_event", "detailed_demographic", "custom_audience"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AudienceSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AudienceSegment} AudienceSegment instance
                             */
                            AudienceSegment.create = function create(properties) {
                                return new AudienceSegment(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceSegment message. Does not implicitly {@link google.ads.googleads.v11.common.AudienceSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceSegment} m AudienceSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    $root.google.ads.googleads.v11.common.UserListSegment.encode(m.user_list, w.uint32(10).fork()).ldelim();
                                if (m.user_interest != null && Object.hasOwnProperty.call(m, "user_interest"))
                                    $root.google.ads.googleads.v11.common.UserInterestSegment.encode(m.user_interest, w.uint32(18).fork()).ldelim();
                                if (m.life_event != null && Object.hasOwnProperty.call(m, "life_event"))
                                    $root.google.ads.googleads.v11.common.LifeEventSegment.encode(m.life_event, w.uint32(26).fork()).ldelim();
                                if (m.detailed_demographic != null && Object.hasOwnProperty.call(m, "detailed_demographic"))
                                    $root.google.ads.googleads.v11.common.DetailedDemographicSegment.encode(m.detailed_demographic, w.uint32(34).fork()).ldelim();
                                if (m.custom_audience != null && Object.hasOwnProperty.call(m, "custom_audience"))
                                    $root.google.ads.googleads.v11.common.CustomAudienceSegment.encode(m.custom_audience, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AudienceSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IAudienceSegment} message AudienceSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AudienceSegment} AudienceSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AudienceSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.user_list = $root.google.ads.googleads.v11.common.UserListSegment.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.user_interest = $root.google.ads.googleads.v11.common.UserInterestSegment.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.life_event = $root.google.ads.googleads.v11.common.LifeEventSegment.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.detailed_demographic = $root.google.ads.googleads.v11.common.DetailedDemographicSegment.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceSegment.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AudienceSegment} AudienceSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    p.segment = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListSegment.verify(m.user_list);
                                        if (e)
                                            return "user_list." + e;
                                    }
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    if (p.segment === 1)
                                        return "segment: multiple values";
                                    p.segment = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserInterestSegment.verify(m.user_interest);
                                        if (e)
                                            return "user_interest." + e;
                                    }
                                }
                                if (m.life_event != null && m.hasOwnProperty("life_event")) {
                                    if (p.segment === 1)
                                        return "segment: multiple values";
                                    p.segment = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LifeEventSegment.verify(m.life_event);
                                        if (e)
                                            return "life_event." + e;
                                    }
                                }
                                if (m.detailed_demographic != null && m.hasOwnProperty("detailed_demographic")) {
                                    if (p.segment === 1)
                                        return "segment: multiple values";
                                    p.segment = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DetailedDemographicSegment.verify(m.detailed_demographic);
                                        if (e)
                                            return "detailed_demographic." + e;
                                    }
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    if (p.segment === 1)
                                        return "segment: multiple values";
                                    p.segment = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomAudienceSegment.verify(m.custom_audience);
                                        if (e)
                                            return "custom_audience." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AudienceSegment} AudienceSegment
                             */
                            AudienceSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AudienceSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AudienceSegment();
                                if (d.user_list != null) {
                                    if (typeof d.user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceSegment.user_list: object expected");
                                    m.user_list = $root.google.ads.googleads.v11.common.UserListSegment.fromObject(d.user_list);
                                }
                                if (d.user_interest != null) {
                                    if (typeof d.user_interest !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceSegment.user_interest: object expected");
                                    m.user_interest = $root.google.ads.googleads.v11.common.UserInterestSegment.fromObject(d.user_interest);
                                }
                                if (d.life_event != null) {
                                    if (typeof d.life_event !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceSegment.life_event: object expected");
                                    m.life_event = $root.google.ads.googleads.v11.common.LifeEventSegment.fromObject(d.life_event);
                                }
                                if (d.detailed_demographic != null) {
                                    if (typeof d.detailed_demographic !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceSegment.detailed_demographic: object expected");
                                    m.detailed_demographic = $root.google.ads.googleads.v11.common.DetailedDemographicSegment.fromObject(d.detailed_demographic);
                                }
                                if (d.custom_audience != null) {
                                    if (typeof d.custom_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.AudienceSegment.custom_audience: object expected");
                                    m.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceSegment.fromObject(d.custom_audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.AudienceSegment} m AudienceSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = $root.google.ads.googleads.v11.common.UserListSegment.toObject(m.user_list, o);
                                    if (o.oneofs)
                                        d.segment = "user_list";
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    d.user_interest = $root.google.ads.googleads.v11.common.UserInterestSegment.toObject(m.user_interest, o);
                                    if (o.oneofs)
                                        d.segment = "user_interest";
                                }
                                if (m.life_event != null && m.hasOwnProperty("life_event")) {
                                    d.life_event = $root.google.ads.googleads.v11.common.LifeEventSegment.toObject(m.life_event, o);
                                    if (o.oneofs)
                                        d.segment = "life_event";
                                }
                                if (m.detailed_demographic != null && m.hasOwnProperty("detailed_demographic")) {
                                    d.detailed_demographic = $root.google.ads.googleads.v11.common.DetailedDemographicSegment.toObject(m.detailed_demographic, o);
                                    if (o.oneofs)
                                        d.segment = "detailed_demographic";
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    d.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceSegment.toObject(m.custom_audience, o);
                                    if (o.oneofs)
                                        d.segment = "custom_audience";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AudienceSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceSegment;
                        })();
    
                        common.UserListSegment = (function() {
    
                            /**
                             * Properties of a UserListSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListSegment
                             * @property {string|null} [user_list] UserListSegment user_list
                             */
    
                            /**
                             * Constructs a new UserListSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListSegment.
                             * @implements IUserListSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListSegment=} [p] Properties to set
                             */
                            function UserListSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListSegment user_list.
                             * @member {string|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @instance
                             */
                            UserListSegment.prototype.user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListSegment _user_list.
                             * @member {"user_list"|undefined} _user_list
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @instance
                             */
                            Object.defineProperty(UserListSegment.prototype, "_user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListSegment} UserListSegment instance
                             */
                            UserListSegment.create = function create(properties) {
                                return new UserListSegment(properties);
                            };
    
                            /**
                             * Encodes the specified UserListSegment message. Does not implicitly {@link google.ads.googleads.v11.common.UserListSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListSegment} m UserListSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    w.uint32(10).string(m.user_list);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListSegment} message UserListSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListSegment} UserListSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.user_list = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListSegment} UserListSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    p._user_list = 1;
                                    if (!$util.isString(m.user_list))
                                        return "user_list: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListSegment} UserListSegment
                             */
                            UserListSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListSegment();
                                if (d.user_list != null) {
                                    m.user_list = String(d.user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListSegment} m UserListSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = m.user_list;
                                    if (o.oneofs)
                                        d._user_list = "user_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListSegment;
                        })();
    
                        common.UserInterestSegment = (function() {
    
                            /**
                             * Properties of a UserInterestSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserInterestSegment
                             * @property {string|null} [user_interest_category] UserInterestSegment user_interest_category
                             */
    
                            /**
                             * Constructs a new UserInterestSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserInterestSegment.
                             * @implements IUserInterestSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserInterestSegment=} [p] Properties to set
                             */
                            function UserInterestSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserInterestSegment user_interest_category.
                             * @member {string|null|undefined} user_interest_category
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @instance
                             */
                            UserInterestSegment.prototype.user_interest_category = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserInterestSegment _user_interest_category.
                             * @member {"user_interest_category"|undefined} _user_interest_category
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @instance
                             */
                            Object.defineProperty(UserInterestSegment.prototype, "_user_interest_category", {
                                get: $util.oneOfGetter($oneOfFields = ["user_interest_category"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserInterestSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserInterestSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserInterestSegment} UserInterestSegment instance
                             */
                            UserInterestSegment.create = function create(properties) {
                                return new UserInterestSegment(properties);
                            };
    
                            /**
                             * Encodes the specified UserInterestSegment message. Does not implicitly {@link google.ads.googleads.v11.common.UserInterestSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserInterestSegment} m UserInterestSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterestSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_interest_category != null && Object.hasOwnProperty.call(m, "user_interest_category"))
                                    w.uint32(10).string(m.user_interest_category);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserInterestSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserInterestSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserInterestSegment} message UserInterestSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterestSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserInterestSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserInterestSegment} UserInterestSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterestSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserInterestSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.user_interest_category = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserInterestSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserInterestSegment} UserInterestSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterestSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserInterestSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserInterestSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_interest_category != null && m.hasOwnProperty("user_interest_category")) {
                                    p._user_interest_category = 1;
                                    if (!$util.isString(m.user_interest_category))
                                        return "user_interest_category: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserInterestSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserInterestSegment} UserInterestSegment
                             */
                            UserInterestSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserInterestSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserInterestSegment();
                                if (d.user_interest_category != null) {
                                    m.user_interest_category = String(d.user_interest_category);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserInterestSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.UserInterestSegment} m UserInterestSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserInterestSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_interest_category != null && m.hasOwnProperty("user_interest_category")) {
                                    d.user_interest_category = m.user_interest_category;
                                    if (o.oneofs)
                                        d._user_interest_category = "user_interest_category";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserInterestSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserInterestSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserInterestSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserInterestSegment;
                        })();
    
                        common.LifeEventSegment = (function() {
    
                            /**
                             * Properties of a LifeEventSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILifeEventSegment
                             * @property {string|null} [life_event] LifeEventSegment life_event
                             */
    
                            /**
                             * Constructs a new LifeEventSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LifeEventSegment.
                             * @implements ILifeEventSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILifeEventSegment=} [p] Properties to set
                             */
                            function LifeEventSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LifeEventSegment life_event.
                             * @member {string|null|undefined} life_event
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @instance
                             */
                            LifeEventSegment.prototype.life_event = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LifeEventSegment _life_event.
                             * @member {"life_event"|undefined} _life_event
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @instance
                             */
                            Object.defineProperty(LifeEventSegment.prototype, "_life_event", {
                                get: $util.oneOfGetter($oneOfFields = ["life_event"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LifeEventSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.ILifeEventSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LifeEventSegment} LifeEventSegment instance
                             */
                            LifeEventSegment.create = function create(properties) {
                                return new LifeEventSegment(properties);
                            };
    
                            /**
                             * Encodes the specified LifeEventSegment message. Does not implicitly {@link google.ads.googleads.v11.common.LifeEventSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.ILifeEventSegment} m LifeEventSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LifeEventSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.life_event != null && Object.hasOwnProperty.call(m, "life_event"))
                                    w.uint32(10).string(m.life_event);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LifeEventSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LifeEventSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.ILifeEventSegment} message LifeEventSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LifeEventSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LifeEventSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LifeEventSegment} LifeEventSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LifeEventSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LifeEventSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.life_event = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LifeEventSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LifeEventSegment} LifeEventSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LifeEventSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LifeEventSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LifeEventSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.life_event != null && m.hasOwnProperty("life_event")) {
                                    p._life_event = 1;
                                    if (!$util.isString(m.life_event))
                                        return "life_event: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LifeEventSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LifeEventSegment} LifeEventSegment
                             */
                            LifeEventSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LifeEventSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LifeEventSegment();
                                if (d.life_event != null) {
                                    m.life_event = String(d.life_event);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LifeEventSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.LifeEventSegment} m LifeEventSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LifeEventSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.life_event != null && m.hasOwnProperty("life_event")) {
                                    d.life_event = m.life_event;
                                    if (o.oneofs)
                                        d._life_event = "life_event";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LifeEventSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LifeEventSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LifeEventSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LifeEventSegment;
                        })();
    
                        common.DetailedDemographicSegment = (function() {
    
                            /**
                             * Properties of a DetailedDemographicSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDetailedDemographicSegment
                             * @property {string|null} [detailed_demographic] DetailedDemographicSegment detailed_demographic
                             */
    
                            /**
                             * Constructs a new DetailedDemographicSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DetailedDemographicSegment.
                             * @implements IDetailedDemographicSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDetailedDemographicSegment=} [p] Properties to set
                             */
                            function DetailedDemographicSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DetailedDemographicSegment detailed_demographic.
                             * @member {string|null|undefined} detailed_demographic
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @instance
                             */
                            DetailedDemographicSegment.prototype.detailed_demographic = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DetailedDemographicSegment _detailed_demographic.
                             * @member {"detailed_demographic"|undefined} _detailed_demographic
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @instance
                             */
                            Object.defineProperty(DetailedDemographicSegment.prototype, "_detailed_demographic", {
                                get: $util.oneOfGetter($oneOfFields = ["detailed_demographic"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DetailedDemographicSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IDetailedDemographicSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DetailedDemographicSegment} DetailedDemographicSegment instance
                             */
                            DetailedDemographicSegment.create = function create(properties) {
                                return new DetailedDemographicSegment(properties);
                            };
    
                            /**
                             * Encodes the specified DetailedDemographicSegment message. Does not implicitly {@link google.ads.googleads.v11.common.DetailedDemographicSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IDetailedDemographicSegment} m DetailedDemographicSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DetailedDemographicSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.detailed_demographic != null && Object.hasOwnProperty.call(m, "detailed_demographic"))
                                    w.uint32(10).string(m.detailed_demographic);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DetailedDemographicSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DetailedDemographicSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.IDetailedDemographicSegment} message DetailedDemographicSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DetailedDemographicSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DetailedDemographicSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DetailedDemographicSegment} DetailedDemographicSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DetailedDemographicSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DetailedDemographicSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.detailed_demographic = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DetailedDemographicSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DetailedDemographicSegment} DetailedDemographicSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DetailedDemographicSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DetailedDemographicSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DetailedDemographicSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.detailed_demographic != null && m.hasOwnProperty("detailed_demographic")) {
                                    p._detailed_demographic = 1;
                                    if (!$util.isString(m.detailed_demographic))
                                        return "detailed_demographic: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DetailedDemographicSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DetailedDemographicSegment} DetailedDemographicSegment
                             */
                            DetailedDemographicSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DetailedDemographicSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DetailedDemographicSegment();
                                if (d.detailed_demographic != null) {
                                    m.detailed_demographic = String(d.detailed_demographic);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DetailedDemographicSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.DetailedDemographicSegment} m DetailedDemographicSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DetailedDemographicSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.detailed_demographic != null && m.hasOwnProperty("detailed_demographic")) {
                                    d.detailed_demographic = m.detailed_demographic;
                                    if (o.oneofs)
                                        d._detailed_demographic = "detailed_demographic";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DetailedDemographicSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DetailedDemographicSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DetailedDemographicSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DetailedDemographicSegment;
                        })();
    
                        common.CustomAudienceSegment = (function() {
    
                            /**
                             * Properties of a CustomAudienceSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICustomAudienceSegment
                             * @property {string|null} [custom_audience] CustomAudienceSegment custom_audience
                             */
    
                            /**
                             * Constructs a new CustomAudienceSegment.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CustomAudienceSegment.
                             * @implements ICustomAudienceSegment
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICustomAudienceSegment=} [p] Properties to set
                             */
                            function CustomAudienceSegment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomAudienceSegment custom_audience.
                             * @member {string|null|undefined} custom_audience
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @instance
                             */
                            CustomAudienceSegment.prototype.custom_audience = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomAudienceSegment _custom_audience.
                             * @member {"custom_audience"|undefined} _custom_audience
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @instance
                             */
                            Object.defineProperty(CustomAudienceSegment.prototype, "_custom_audience", {
                                get: $util.oneOfGetter($oneOfFields = ["custom_audience"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomAudienceSegment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAudienceSegment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CustomAudienceSegment} CustomAudienceSegment instance
                             */
                            CustomAudienceSegment.create = function create(properties) {
                                return new CustomAudienceSegment(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceSegment message. Does not implicitly {@link google.ads.googleads.v11.common.CustomAudienceSegment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAudienceSegment} m CustomAudienceSegment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceSegment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.custom_audience != null && Object.hasOwnProperty.call(m, "custom_audience"))
                                    w.uint32(10).string(m.custom_audience);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceSegment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CustomAudienceSegment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomAudienceSegment} message CustomAudienceSegment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceSegment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceSegment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CustomAudienceSegment} CustomAudienceSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceSegment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CustomAudienceSegment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.custom_audience = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceSegment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CustomAudienceSegment} CustomAudienceSegment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceSegment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceSegment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceSegment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    p._custom_audience = 1;
                                    if (!$util.isString(m.custom_audience))
                                        return "custom_audience: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceSegment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CustomAudienceSegment} CustomAudienceSegment
                             */
                            CustomAudienceSegment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CustomAudienceSegment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CustomAudienceSegment();
                                if (d.custom_audience != null) {
                                    m.custom_audience = String(d.custom_audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceSegment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @static
                             * @param {google.ads.googleads.v11.common.CustomAudienceSegment} m CustomAudienceSegment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceSegment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    d.custom_audience = m.custom_audience;
                                    if (o.oneofs)
                                        d._custom_audience = "custom_audience";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomAudienceSegment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CustomAudienceSegment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceSegment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomAudienceSegment;
                        })();
    
                        common.Commission = (function() {
    
                            /**
                             * Properties of a Commission.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICommission
                             * @property {number|null} [commission_rate_micros] Commission commission_rate_micros
                             */
    
                            /**
                             * Constructs a new Commission.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a Commission.
                             * @implements ICommission
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICommission=} [p] Properties to set
                             */
                            function Commission(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Commission commission_rate_micros.
                             * @member {number|null|undefined} commission_rate_micros
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @instance
                             */
                            Commission.prototype.commission_rate_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Commission _commission_rate_micros.
                             * @member {"commission_rate_micros"|undefined} _commission_rate_micros
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @instance
                             */
                            Object.defineProperty(Commission.prototype, "_commission_rate_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["commission_rate_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Commission instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {google.ads.googleads.v11.common.ICommission=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.Commission} Commission instance
                             */
                            Commission.create = function create(properties) {
                                return new Commission(properties);
                            };
    
                            /**
                             * Encodes the specified Commission message. Does not implicitly {@link google.ads.googleads.v11.common.Commission.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {google.ads.googleads.v11.common.ICommission} m Commission message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Commission.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.commission_rate_micros != null && Object.hasOwnProperty.call(m, "commission_rate_micros"))
                                    w.uint32(16).int64(m.commission_rate_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Commission message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Commission.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {google.ads.googleads.v11.common.ICommission} message Commission message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Commission.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Commission message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.Commission} Commission
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Commission.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Commission();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.commission_rate_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Commission message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.Commission} Commission
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Commission.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Commission message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Commission.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.commission_rate_micros != null && m.hasOwnProperty("commission_rate_micros")) {
                                    p._commission_rate_micros = 1;
                                    if (!$util.isInteger(m.commission_rate_micros) && !(m.commission_rate_micros && $util.isInteger(m.commission_rate_micros.low) && $util.isInteger(m.commission_rate_micros.high)))
                                        return "commission_rate_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Commission message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.Commission} Commission
                             */
                            Commission.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.Commission)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.Commission();
                                if (d.commission_rate_micros != null) {
                                    if ($util.Long)
                                        (m.commission_rate_micros = $util.Long.fromValue(d.commission_rate_micros)).unsigned = false;
                                    else if (typeof d.commission_rate_micros === "string")
                                        m.commission_rate_micros = parseInt(d.commission_rate_micros, 10);
                                    else if (typeof d.commission_rate_micros === "number")
                                        m.commission_rate_micros = d.commission_rate_micros;
                                    else if (typeof d.commission_rate_micros === "object")
                                        m.commission_rate_micros = new $util.LongBits(d.commission_rate_micros.low >>> 0, d.commission_rate_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Commission message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @static
                             * @param {google.ads.googleads.v11.common.Commission} m Commission
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Commission.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.commission_rate_micros != null && m.hasOwnProperty("commission_rate_micros")) {
                                    if (typeof m.commission_rate_micros === "number")
                                        d.commission_rate_micros = o.longs === String ? String(m.commission_rate_micros) : m.commission_rate_micros;
                                    else
                                        d.commission_rate_micros = o.longs === String ? $util.Long.prototype.toString.call(m.commission_rate_micros) : o.longs === Number ? new $util.LongBits(m.commission_rate_micros.low >>> 0, m.commission_rate_micros.high >>> 0).toNumber() : m.commission_rate_micros;
                                    if (o.oneofs)
                                        d._commission_rate_micros = "commission_rate_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Commission to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.Commission
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Commission.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Commission;
                        })();
    
                        common.EnhancedCpc = (function() {
    
                            /**
                             * Properties of an EnhancedCpc.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IEnhancedCpc
                             */
    
                            /**
                             * Constructs a new EnhancedCpc.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an EnhancedCpc.
                             * @implements IEnhancedCpc
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IEnhancedCpc=} [p] Properties to set
                             */
                            function EnhancedCpc(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new EnhancedCpc instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IEnhancedCpc=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.EnhancedCpc} EnhancedCpc instance
                             */
                            EnhancedCpc.create = function create(properties) {
                                return new EnhancedCpc(properties);
                            };
    
                            /**
                             * Encodes the specified EnhancedCpc message. Does not implicitly {@link google.ads.googleads.v11.common.EnhancedCpc.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IEnhancedCpc} m EnhancedCpc message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EnhancedCpc.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified EnhancedCpc message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.EnhancedCpc.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IEnhancedCpc} message EnhancedCpc message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EnhancedCpc.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EnhancedCpc message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.EnhancedCpc} EnhancedCpc
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EnhancedCpc.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.EnhancedCpc();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an EnhancedCpc message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.EnhancedCpc} EnhancedCpc
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EnhancedCpc.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EnhancedCpc message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EnhancedCpc.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an EnhancedCpc message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.EnhancedCpc} EnhancedCpc
                             */
                            EnhancedCpc.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.EnhancedCpc)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.EnhancedCpc();
                            };
    
                            /**
                             * Creates a plain object from an EnhancedCpc message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.EnhancedCpc} m EnhancedCpc
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EnhancedCpc.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this EnhancedCpc to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.EnhancedCpc
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EnhancedCpc.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return EnhancedCpc;
                        })();
    
                        common.ManualCpa = (function() {
    
                            /**
                             * Properties of a ManualCpa.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IManualCpa
                             */
    
                            /**
                             * Constructs a new ManualCpa.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ManualCpa.
                             * @implements IManualCpa
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IManualCpa=} [p] Properties to set
                             */
                            function ManualCpa(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ManualCpa instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpa=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ManualCpa} ManualCpa instance
                             */
                            ManualCpa.create = function create(properties) {
                                return new ManualCpa(properties);
                            };
    
                            /**
                             * Encodes the specified ManualCpa message. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpa.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpa} m ManualCpa message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpa.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ManualCpa message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpa.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpa} message ManualCpa message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpa.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ManualCpa message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ManualCpa} ManualCpa
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpa.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ManualCpa();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ManualCpa message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ManualCpa} ManualCpa
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpa.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ManualCpa message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ManualCpa.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ManualCpa message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ManualCpa} ManualCpa
                             */
                            ManualCpa.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ManualCpa)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.ManualCpa();
                            };
    
                            /**
                             * Creates a plain object from a ManualCpa message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.ManualCpa} m ManualCpa
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ManualCpa.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ManualCpa to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ManualCpa
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ManualCpa.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ManualCpa;
                        })();
    
                        common.ManualCpc = (function() {
    
                            /**
                             * Properties of a ManualCpc.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IManualCpc
                             * @property {boolean|null} [enhanced_cpc_enabled] ManualCpc enhanced_cpc_enabled
                             */
    
                            /**
                             * Constructs a new ManualCpc.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ManualCpc.
                             * @implements IManualCpc
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IManualCpc=} [p] Properties to set
                             */
                            function ManualCpc(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ManualCpc enhanced_cpc_enabled.
                             * @member {boolean|null|undefined} enhanced_cpc_enabled
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @instance
                             */
                            ManualCpc.prototype.enhanced_cpc_enabled = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ManualCpc _enhanced_cpc_enabled.
                             * @member {"enhanced_cpc_enabled"|undefined} _enhanced_cpc_enabled
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @instance
                             */
                            Object.defineProperty(ManualCpc.prototype, "_enhanced_cpc_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["enhanced_cpc_enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ManualCpc instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpc=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ManualCpc} ManualCpc instance
                             */
                            ManualCpc.create = function create(properties) {
                                return new ManualCpc(properties);
                            };
    
                            /**
                             * Encodes the specified ManualCpc message. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpc.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpc} m ManualCpc message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpc.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.enhanced_cpc_enabled != null && Object.hasOwnProperty.call(m, "enhanced_cpc_enabled"))
                                    w.uint32(16).bool(m.enhanced_cpc_enabled);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ManualCpc message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpc.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpc} message ManualCpc message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpc.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ManualCpc message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ManualCpc} ManualCpc
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpc.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ManualCpc();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.enhanced_cpc_enabled = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ManualCpc message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ManualCpc} ManualCpc
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpc.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ManualCpc message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ManualCpc.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.enhanced_cpc_enabled != null && m.hasOwnProperty("enhanced_cpc_enabled")) {
                                    p._enhanced_cpc_enabled = 1;
                                    if (typeof m.enhanced_cpc_enabled !== "boolean")
                                        return "enhanced_cpc_enabled: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ManualCpc message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ManualCpc} ManualCpc
                             */
                            ManualCpc.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ManualCpc)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ManualCpc();
                                if (d.enhanced_cpc_enabled != null) {
                                    m.enhanced_cpc_enabled = Boolean(d.enhanced_cpc_enabled);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ManualCpc message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.ManualCpc} m ManualCpc
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ManualCpc.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.enhanced_cpc_enabled != null && m.hasOwnProperty("enhanced_cpc_enabled")) {
                                    d.enhanced_cpc_enabled = m.enhanced_cpc_enabled;
                                    if (o.oneofs)
                                        d._enhanced_cpc_enabled = "enhanced_cpc_enabled";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ManualCpc to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ManualCpc
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ManualCpc.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ManualCpc;
                        })();
    
                        common.ManualCpm = (function() {
    
                            /**
                             * Properties of a ManualCpm.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IManualCpm
                             */
    
                            /**
                             * Constructs a new ManualCpm.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ManualCpm.
                             * @implements IManualCpm
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IManualCpm=} [p] Properties to set
                             */
                            function ManualCpm(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ManualCpm instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpm=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ManualCpm} ManualCpm instance
                             */
                            ManualCpm.create = function create(properties) {
                                return new ManualCpm(properties);
                            };
    
                            /**
                             * Encodes the specified ManualCpm message. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpm.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpm} m ManualCpm message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpm.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ManualCpm message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpm.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpm} message ManualCpm message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpm.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ManualCpm message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ManualCpm} ManualCpm
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpm.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ManualCpm();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ManualCpm message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ManualCpm} ManualCpm
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpm.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ManualCpm message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ManualCpm.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ManualCpm message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ManualCpm} ManualCpm
                             */
                            ManualCpm.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ManualCpm)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.ManualCpm();
                            };
    
                            /**
                             * Creates a plain object from a ManualCpm message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.ManualCpm} m ManualCpm
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ManualCpm.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ManualCpm to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ManualCpm
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ManualCpm.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ManualCpm;
                        })();
    
                        common.ManualCpv = (function() {
    
                            /**
                             * Properties of a ManualCpv.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IManualCpv
                             */
    
                            /**
                             * Constructs a new ManualCpv.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ManualCpv.
                             * @implements IManualCpv
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IManualCpv=} [p] Properties to set
                             */
                            function ManualCpv(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ManualCpv instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpv=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ManualCpv} ManualCpv instance
                             */
                            ManualCpv.create = function create(properties) {
                                return new ManualCpv(properties);
                            };
    
                            /**
                             * Encodes the specified ManualCpv message. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpv.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpv} m ManualCpv message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpv.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ManualCpv message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ManualCpv.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {google.ads.googleads.v11.common.IManualCpv} message ManualCpv message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManualCpv.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ManualCpv message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ManualCpv} ManualCpv
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpv.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ManualCpv();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ManualCpv message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ManualCpv} ManualCpv
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManualCpv.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ManualCpv message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ManualCpv.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ManualCpv message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ManualCpv} ManualCpv
                             */
                            ManualCpv.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ManualCpv)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.ManualCpv();
                            };
    
                            /**
                             * Creates a plain object from a ManualCpv message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @static
                             * @param {google.ads.googleads.v11.common.ManualCpv} m ManualCpv
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ManualCpv.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ManualCpv to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ManualCpv
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ManualCpv.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ManualCpv;
                        })();
    
                        common.MaximizeConversions = (function() {
    
                            /**
                             * Properties of a MaximizeConversions.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMaximizeConversions
                             * @property {number|null} [cpc_bid_ceiling_micros] MaximizeConversions cpc_bid_ceiling_micros
                             * @property {number|null} [cpc_bid_floor_micros] MaximizeConversions cpc_bid_floor_micros
                             * @property {number|null} [target_cpa_micros] MaximizeConversions target_cpa_micros
                             */
    
                            /**
                             * Constructs a new MaximizeConversions.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MaximizeConversions.
                             * @implements IMaximizeConversions
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMaximizeConversions=} [p] Properties to set
                             */
                            function MaximizeConversions(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MaximizeConversions cpc_bid_ceiling_micros.
                             * @member {number} cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @instance
                             */
                            MaximizeConversions.prototype.cpc_bid_ceiling_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * MaximizeConversions cpc_bid_floor_micros.
                             * @member {number} cpc_bid_floor_micros
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @instance
                             */
                            MaximizeConversions.prototype.cpc_bid_floor_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * MaximizeConversions target_cpa_micros.
                             * @member {number} target_cpa_micros
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @instance
                             */
                            MaximizeConversions.prototype.target_cpa_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new MaximizeConversions instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {google.ads.googleads.v11.common.IMaximizeConversions=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MaximizeConversions} MaximizeConversions instance
                             */
                            MaximizeConversions.create = function create(properties) {
                                return new MaximizeConversions(properties);
                            };
    
                            /**
                             * Encodes the specified MaximizeConversions message. Does not implicitly {@link google.ads.googleads.v11.common.MaximizeConversions.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {google.ads.googleads.v11.common.IMaximizeConversions} m MaximizeConversions message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MaximizeConversions.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                    w.uint32(16).int64(m.cpc_bid_ceiling_micros);
                                if (m.cpc_bid_floor_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_floor_micros"))
                                    w.uint32(24).int64(m.cpc_bid_floor_micros);
                                if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                    w.uint32(32).int64(m.target_cpa_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MaximizeConversions message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MaximizeConversions.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {google.ads.googleads.v11.common.IMaximizeConversions} message MaximizeConversions message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MaximizeConversions.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MaximizeConversions message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MaximizeConversions} MaximizeConversions
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MaximizeConversions.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MaximizeConversions();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    case 3:
                                        m.cpc_bid_floor_micros = r.int64();
                                        break;
                                    case 4:
                                        m.target_cpa_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MaximizeConversions message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MaximizeConversions} MaximizeConversions
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MaximizeConversions.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MaximizeConversions message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MaximizeConversions.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                        return "cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    if (!$util.isInteger(m.cpc_bid_floor_micros) && !(m.cpc_bid_floor_micros && $util.isInteger(m.cpc_bid_floor_micros.low) && $util.isInteger(m.cpc_bid_floor_micros.high)))
                                        return "cpc_bid_floor_micros: integer|Long expected";
                                }
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                        return "target_cpa_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MaximizeConversions message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MaximizeConversions} MaximizeConversions
                             */
                            MaximizeConversions.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MaximizeConversions)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MaximizeConversions();
                                if (d.cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_ceiling_micros === "string")
                                        m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.cpc_bid_ceiling_micros === "number")
                                        m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                    else if (typeof d.cpc_bid_ceiling_micros === "object")
                                        m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_floor_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_floor_micros = $util.Long.fromValue(d.cpc_bid_floor_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_floor_micros === "string")
                                        m.cpc_bid_floor_micros = parseInt(d.cpc_bid_floor_micros, 10);
                                    else if (typeof d.cpc_bid_floor_micros === "number")
                                        m.cpc_bid_floor_micros = d.cpc_bid_floor_micros;
                                    else if (typeof d.cpc_bid_floor_micros === "object")
                                        m.cpc_bid_floor_micros = new $util.LongBits(d.cpc_bid_floor_micros.low >>> 0, d.cpc_bid_floor_micros.high >>> 0).toNumber();
                                }
                                if (d.target_cpa_micros != null) {
                                    if ($util.Long)
                                        (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                    else if (typeof d.target_cpa_micros === "string")
                                        m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                    else if (typeof d.target_cpa_micros === "number")
                                        m.target_cpa_micros = d.target_cpa_micros;
                                    else if (typeof d.target_cpa_micros === "object")
                                        m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MaximizeConversions message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @static
                             * @param {google.ads.googleads.v11.common.MaximizeConversions} m MaximizeConversions
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MaximizeConversions.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cpc_bid_ceiling_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cpc_bid_floor_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cpc_bid_floor_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.target_cpa_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.target_cpa_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (typeof m.cpc_bid_ceiling_micros === "number")
                                        d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                    else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    if (typeof m.cpc_bid_floor_micros === "number")
                                        d.cpc_bid_floor_micros = o.longs === String ? String(m.cpc_bid_floor_micros) : m.cpc_bid_floor_micros;
                                    else
                                        d.cpc_bid_floor_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_floor_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_floor_micros.low >>> 0, m.cpc_bid_floor_micros.high >>> 0).toNumber() : m.cpc_bid_floor_micros;
                                }
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    if (typeof m.target_cpa_micros === "number")
                                        d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                    else
                                        d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MaximizeConversions to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MaximizeConversions
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MaximizeConversions.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MaximizeConversions;
                        })();
    
                        common.MaximizeConversionValue = (function() {
    
                            /**
                             * Properties of a MaximizeConversionValue.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMaximizeConversionValue
                             * @property {number|null} [target_roas] MaximizeConversionValue target_roas
                             * @property {number|null} [cpc_bid_ceiling_micros] MaximizeConversionValue cpc_bid_ceiling_micros
                             * @property {number|null} [cpc_bid_floor_micros] MaximizeConversionValue cpc_bid_floor_micros
                             */
    
                            /**
                             * Constructs a new MaximizeConversionValue.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MaximizeConversionValue.
                             * @implements IMaximizeConversionValue
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMaximizeConversionValue=} [p] Properties to set
                             */
                            function MaximizeConversionValue(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MaximizeConversionValue target_roas.
                             * @member {number} target_roas
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @instance
                             */
                            MaximizeConversionValue.prototype.target_roas = 0;
    
                            /**
                             * MaximizeConversionValue cpc_bid_ceiling_micros.
                             * @member {number} cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @instance
                             */
                            MaximizeConversionValue.prototype.cpc_bid_ceiling_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * MaximizeConversionValue cpc_bid_floor_micros.
                             * @member {number} cpc_bid_floor_micros
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @instance
                             */
                            MaximizeConversionValue.prototype.cpc_bid_floor_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new MaximizeConversionValue instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {google.ads.googleads.v11.common.IMaximizeConversionValue=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MaximizeConversionValue} MaximizeConversionValue instance
                             */
                            MaximizeConversionValue.create = function create(properties) {
                                return new MaximizeConversionValue(properties);
                            };
    
                            /**
                             * Encodes the specified MaximizeConversionValue message. Does not implicitly {@link google.ads.googleads.v11.common.MaximizeConversionValue.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {google.ads.googleads.v11.common.IMaximizeConversionValue} m MaximizeConversionValue message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MaximizeConversionValue.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                    w.uint32(17).double(m.target_roas);
                                if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                    w.uint32(24).int64(m.cpc_bid_ceiling_micros);
                                if (m.cpc_bid_floor_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_floor_micros"))
                                    w.uint32(32).int64(m.cpc_bid_floor_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MaximizeConversionValue message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MaximizeConversionValue.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {google.ads.googleads.v11.common.IMaximizeConversionValue} message MaximizeConversionValue message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MaximizeConversionValue.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MaximizeConversionValue message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MaximizeConversionValue} MaximizeConversionValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MaximizeConversionValue.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MaximizeConversionValue();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.target_roas = r.double();
                                        break;
                                    case 3:
                                        m.cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    case 4:
                                        m.cpc_bid_floor_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MaximizeConversionValue message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MaximizeConversionValue} MaximizeConversionValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MaximizeConversionValue.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MaximizeConversionValue message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MaximizeConversionValue.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    if (typeof m.target_roas !== "number")
                                        return "target_roas: number expected";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                        return "cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    if (!$util.isInteger(m.cpc_bid_floor_micros) && !(m.cpc_bid_floor_micros && $util.isInteger(m.cpc_bid_floor_micros.low) && $util.isInteger(m.cpc_bid_floor_micros.high)))
                                        return "cpc_bid_floor_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MaximizeConversionValue message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MaximizeConversionValue} MaximizeConversionValue
                             */
                            MaximizeConversionValue.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MaximizeConversionValue)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MaximizeConversionValue();
                                if (d.target_roas != null) {
                                    m.target_roas = Number(d.target_roas);
                                }
                                if (d.cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_ceiling_micros === "string")
                                        m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.cpc_bid_ceiling_micros === "number")
                                        m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                    else if (typeof d.cpc_bid_ceiling_micros === "object")
                                        m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_floor_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_floor_micros = $util.Long.fromValue(d.cpc_bid_floor_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_floor_micros === "string")
                                        m.cpc_bid_floor_micros = parseInt(d.cpc_bid_floor_micros, 10);
                                    else if (typeof d.cpc_bid_floor_micros === "number")
                                        m.cpc_bid_floor_micros = d.cpc_bid_floor_micros;
                                    else if (typeof d.cpc_bid_floor_micros === "object")
                                        m.cpc_bid_floor_micros = new $util.LongBits(d.cpc_bid_floor_micros.low >>> 0, d.cpc_bid_floor_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MaximizeConversionValue message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @static
                             * @param {google.ads.googleads.v11.common.MaximizeConversionValue} m MaximizeConversionValue
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MaximizeConversionValue.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.target_roas = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cpc_bid_ceiling_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cpc_bid_floor_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cpc_bid_floor_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    d.target_roas = o.json && !isFinite(m.target_roas) ? String(m.target_roas) : m.target_roas;
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (typeof m.cpc_bid_ceiling_micros === "number")
                                        d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                    else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    if (typeof m.cpc_bid_floor_micros === "number")
                                        d.cpc_bid_floor_micros = o.longs === String ? String(m.cpc_bid_floor_micros) : m.cpc_bid_floor_micros;
                                    else
                                        d.cpc_bid_floor_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_floor_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_floor_micros.low >>> 0, m.cpc_bid_floor_micros.high >>> 0).toNumber() : m.cpc_bid_floor_micros;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MaximizeConversionValue to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MaximizeConversionValue
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MaximizeConversionValue.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MaximizeConversionValue;
                        })();
    
                        common.TargetCpa = (function() {
    
                            /**
                             * Properties of a TargetCpa.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetCpa
                             * @property {number|null} [target_cpa_micros] TargetCpa target_cpa_micros
                             * @property {number|null} [cpc_bid_ceiling_micros] TargetCpa cpc_bid_ceiling_micros
                             * @property {number|null} [cpc_bid_floor_micros] TargetCpa cpc_bid_floor_micros
                             */
    
                            /**
                             * Constructs a new TargetCpa.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetCpa.
                             * @implements ITargetCpa
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetCpa=} [p] Properties to set
                             */
                            function TargetCpa(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetCpa target_cpa_micros.
                             * @member {number|null|undefined} target_cpa_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @instance
                             */
                            TargetCpa.prototype.target_cpa_micros = null;
    
                            /**
                             * TargetCpa cpc_bid_ceiling_micros.
                             * @member {number|null|undefined} cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @instance
                             */
                            TargetCpa.prototype.cpc_bid_ceiling_micros = null;
    
                            /**
                             * TargetCpa cpc_bid_floor_micros.
                             * @member {number|null|undefined} cpc_bid_floor_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @instance
                             */
                            TargetCpa.prototype.cpc_bid_floor_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TargetCpa _target_cpa_micros.
                             * @member {"target_cpa_micros"|undefined} _target_cpa_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @instance
                             */
                            Object.defineProperty(TargetCpa.prototype, "_target_cpa_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["target_cpa_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpa _cpc_bid_ceiling_micros.
                             * @member {"cpc_bid_ceiling_micros"|undefined} _cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @instance
                             */
                            Object.defineProperty(TargetCpa.prototype, "_cpc_bid_ceiling_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_ceiling_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpa _cpc_bid_floor_micros.
                             * @member {"cpc_bid_floor_micros"|undefined} _cpc_bid_floor_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @instance
                             */
                            Object.defineProperty(TargetCpa.prototype, "_cpc_bid_floor_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_floor_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TargetCpa instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpa=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetCpa} TargetCpa instance
                             */
                            TargetCpa.create = function create(properties) {
                                return new TargetCpa(properties);
                            };
    
                            /**
                             * Encodes the specified TargetCpa message. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpa.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpa} m TargetCpa message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpa.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                    w.uint32(32).int64(m.target_cpa_micros);
                                if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                    w.uint32(40).int64(m.cpc_bid_ceiling_micros);
                                if (m.cpc_bid_floor_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_floor_micros"))
                                    w.uint32(48).int64(m.cpc_bid_floor_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetCpa message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpa.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpa} message TargetCpa message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpa.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetCpa message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetCpa} TargetCpa
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpa.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetCpa();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.target_cpa_micros = r.int64();
                                        break;
                                    case 5:
                                        m.cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    case 6:
                                        m.cpc_bid_floor_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetCpa message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetCpa} TargetCpa
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpa.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetCpa message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetCpa.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    p._target_cpa_micros = 1;
                                    if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                        return "target_cpa_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    p._cpc_bid_ceiling_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                        return "cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    p._cpc_bid_floor_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_floor_micros) && !(m.cpc_bid_floor_micros && $util.isInteger(m.cpc_bid_floor_micros.low) && $util.isInteger(m.cpc_bid_floor_micros.high)))
                                        return "cpc_bid_floor_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetCpa message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetCpa} TargetCpa
                             */
                            TargetCpa.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetCpa)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetCpa();
                                if (d.target_cpa_micros != null) {
                                    if ($util.Long)
                                        (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                    else if (typeof d.target_cpa_micros === "string")
                                        m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                    else if (typeof d.target_cpa_micros === "number")
                                        m.target_cpa_micros = d.target_cpa_micros;
                                    else if (typeof d.target_cpa_micros === "object")
                                        m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_ceiling_micros === "string")
                                        m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.cpc_bid_ceiling_micros === "number")
                                        m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                    else if (typeof d.cpc_bid_ceiling_micros === "object")
                                        m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_floor_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_floor_micros = $util.Long.fromValue(d.cpc_bid_floor_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_floor_micros === "string")
                                        m.cpc_bid_floor_micros = parseInt(d.cpc_bid_floor_micros, 10);
                                    else if (typeof d.cpc_bid_floor_micros === "number")
                                        m.cpc_bid_floor_micros = d.cpc_bid_floor_micros;
                                    else if (typeof d.cpc_bid_floor_micros === "object")
                                        m.cpc_bid_floor_micros = new $util.LongBits(d.cpc_bid_floor_micros.low >>> 0, d.cpc_bid_floor_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetCpa message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetCpa} m TargetCpa
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetCpa.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    if (typeof m.target_cpa_micros === "number")
                                        d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                    else
                                        d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                    if (o.oneofs)
                                        d._target_cpa_micros = "target_cpa_micros";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (typeof m.cpc_bid_ceiling_micros === "number")
                                        d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                    else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_ceiling_micros = "cpc_bid_ceiling_micros";
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    if (typeof m.cpc_bid_floor_micros === "number")
                                        d.cpc_bid_floor_micros = o.longs === String ? String(m.cpc_bid_floor_micros) : m.cpc_bid_floor_micros;
                                    else
                                        d.cpc_bid_floor_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_floor_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_floor_micros.low >>> 0, m.cpc_bid_floor_micros.high >>> 0).toNumber() : m.cpc_bid_floor_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_floor_micros = "cpc_bid_floor_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetCpa to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetCpa
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetCpa.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetCpa;
                        })();
    
                        common.TargetCpm = (function() {
    
                            /**
                             * Properties of a TargetCpm.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetCpm
                             */
    
                            /**
                             * Constructs a new TargetCpm.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetCpm.
                             * @implements ITargetCpm
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetCpm=} [p] Properties to set
                             */
                            function TargetCpm(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TargetCpm instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpm=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetCpm} TargetCpm instance
                             */
                            TargetCpm.create = function create(properties) {
                                return new TargetCpm(properties);
                            };
    
                            /**
                             * Encodes the specified TargetCpm message. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpm.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpm} m TargetCpm message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpm.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetCpm message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpm.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpm} message TargetCpm message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpm.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetCpm message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetCpm} TargetCpm
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpm.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetCpm();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetCpm message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetCpm} TargetCpm
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpm.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetCpm message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetCpm.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TargetCpm message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetCpm} TargetCpm
                             */
                            TargetCpm.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetCpm)
                                    return d;
                                return new $root.google.ads.googleads.v11.common.TargetCpm();
                            };
    
                            /**
                             * Creates a plain object from a TargetCpm message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetCpm} m TargetCpm
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetCpm.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TargetCpm to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetCpm
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetCpm.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetCpm;
                        })();
    
                        common.TargetImpressionShare = (function() {
    
                            /**
                             * Properties of a TargetImpressionShare.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetImpressionShare
                             * @property {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.TargetImpressionShareLocation|null} [location] TargetImpressionShare location
                             * @property {number|null} [location_fraction_micros] TargetImpressionShare location_fraction_micros
                             * @property {number|null} [cpc_bid_ceiling_micros] TargetImpressionShare cpc_bid_ceiling_micros
                             */
    
                            /**
                             * Constructs a new TargetImpressionShare.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetImpressionShare.
                             * @implements ITargetImpressionShare
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShare=} [p] Properties to set
                             */
                            function TargetImpressionShare(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetImpressionShare location.
                             * @member {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.TargetImpressionShareLocation} location
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @instance
                             */
                            TargetImpressionShare.prototype.location = 0;
    
                            /**
                             * TargetImpressionShare location_fraction_micros.
                             * @member {number|null|undefined} location_fraction_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @instance
                             */
                            TargetImpressionShare.prototype.location_fraction_micros = null;
    
                            /**
                             * TargetImpressionShare cpc_bid_ceiling_micros.
                             * @member {number|null|undefined} cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @instance
                             */
                            TargetImpressionShare.prototype.cpc_bid_ceiling_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TargetImpressionShare _location_fraction_micros.
                             * @member {"location_fraction_micros"|undefined} _location_fraction_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @instance
                             */
                            Object.defineProperty(TargetImpressionShare.prototype, "_location_fraction_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["location_fraction_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetImpressionShare _cpc_bid_ceiling_micros.
                             * @member {"cpc_bid_ceiling_micros"|undefined} _cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @instance
                             */
                            Object.defineProperty(TargetImpressionShare.prototype, "_cpc_bid_ceiling_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_ceiling_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TargetImpressionShare instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShare=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShare} TargetImpressionShare instance
                             */
                            TargetImpressionShare.create = function create(properties) {
                                return new TargetImpressionShare(properties);
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShare message. Does not implicitly {@link google.ads.googleads.v11.common.TargetImpressionShare.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShare} m TargetImpressionShare message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShare.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.location != null && Object.hasOwnProperty.call(m, "location"))
                                    w.uint32(8).int32(m.location);
                                if (m.location_fraction_micros != null && Object.hasOwnProperty.call(m, "location_fraction_micros"))
                                    w.uint32(32).int64(m.location_fraction_micros);
                                if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                    w.uint32(40).int64(m.cpc_bid_ceiling_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShare message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetImpressionShare.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShare} message TargetImpressionShare message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShare.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetImpressionShare message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShare} TargetImpressionShare
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShare.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetImpressionShare();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.location = r.int32();
                                        break;
                                    case 4:
                                        m.location_fraction_micros = r.int64();
                                        break;
                                    case 5:
                                        m.cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetImpressionShare message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShare} TargetImpressionShare
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShare.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetImpressionShare message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetImpressionShare.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    switch (m.location) {
                                    default:
                                        return "location: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.location_fraction_micros != null && m.hasOwnProperty("location_fraction_micros")) {
                                    p._location_fraction_micros = 1;
                                    if (!$util.isInteger(m.location_fraction_micros) && !(m.location_fraction_micros && $util.isInteger(m.location_fraction_micros.low) && $util.isInteger(m.location_fraction_micros.high)))
                                        return "location_fraction_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    p._cpc_bid_ceiling_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                        return "cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetImpressionShare message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShare} TargetImpressionShare
                             */
                            TargetImpressionShare.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetImpressionShare)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetImpressionShare();
                                switch (d.location) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.location = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.location = 1;
                                    break;
                                case "ANYWHERE_ON_PAGE":
                                case 2:
                                    m.location = 2;
                                    break;
                                case "TOP_OF_PAGE":
                                case 3:
                                    m.location = 3;
                                    break;
                                case "ABSOLUTE_TOP_OF_PAGE":
                                case 4:
                                    m.location = 4;
                                    break;
                                }
                                if (d.location_fraction_micros != null) {
                                    if ($util.Long)
                                        (m.location_fraction_micros = $util.Long.fromValue(d.location_fraction_micros)).unsigned = false;
                                    else if (typeof d.location_fraction_micros === "string")
                                        m.location_fraction_micros = parseInt(d.location_fraction_micros, 10);
                                    else if (typeof d.location_fraction_micros === "number")
                                        m.location_fraction_micros = d.location_fraction_micros;
                                    else if (typeof d.location_fraction_micros === "object")
                                        m.location_fraction_micros = new $util.LongBits(d.location_fraction_micros.low >>> 0, d.location_fraction_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_ceiling_micros === "string")
                                        m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.cpc_bid_ceiling_micros === "number")
                                        m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                    else if (typeof d.cpc_bid_ceiling_micros === "object")
                                        m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetImpressionShare message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetImpressionShare} m TargetImpressionShare
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetImpressionShare.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.location = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    d.location = o.enums === String ? $root.google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.TargetImpressionShareLocation[m.location] : m.location;
                                }
                                if (m.location_fraction_micros != null && m.hasOwnProperty("location_fraction_micros")) {
                                    if (typeof m.location_fraction_micros === "number")
                                        d.location_fraction_micros = o.longs === String ? String(m.location_fraction_micros) : m.location_fraction_micros;
                                    else
                                        d.location_fraction_micros = o.longs === String ? $util.Long.prototype.toString.call(m.location_fraction_micros) : o.longs === Number ? new $util.LongBits(m.location_fraction_micros.low >>> 0, m.location_fraction_micros.high >>> 0).toNumber() : m.location_fraction_micros;
                                    if (o.oneofs)
                                        d._location_fraction_micros = "location_fraction_micros";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (typeof m.cpc_bid_ceiling_micros === "number")
                                        d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                    else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_ceiling_micros = "cpc_bid_ceiling_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetImpressionShare to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShare
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetImpressionShare.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetImpressionShare;
                        })();
    
                        common.TargetRoas = (function() {
    
                            /**
                             * Properties of a TargetRoas.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetRoas
                             * @property {number|null} [target_roas] TargetRoas target_roas
                             * @property {number|null} [cpc_bid_ceiling_micros] TargetRoas cpc_bid_ceiling_micros
                             * @property {number|null} [cpc_bid_floor_micros] TargetRoas cpc_bid_floor_micros
                             */
    
                            /**
                             * Constructs a new TargetRoas.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetRoas.
                             * @implements ITargetRoas
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetRoas=} [p] Properties to set
                             */
                            function TargetRoas(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetRoas target_roas.
                             * @member {number|null|undefined} target_roas
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @instance
                             */
                            TargetRoas.prototype.target_roas = null;
    
                            /**
                             * TargetRoas cpc_bid_ceiling_micros.
                             * @member {number|null|undefined} cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @instance
                             */
                            TargetRoas.prototype.cpc_bid_ceiling_micros = null;
    
                            /**
                             * TargetRoas cpc_bid_floor_micros.
                             * @member {number|null|undefined} cpc_bid_floor_micros
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @instance
                             */
                            TargetRoas.prototype.cpc_bid_floor_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TargetRoas _target_roas.
                             * @member {"target_roas"|undefined} _target_roas
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @instance
                             */
                            Object.defineProperty(TargetRoas.prototype, "_target_roas", {
                                get: $util.oneOfGetter($oneOfFields = ["target_roas"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoas _cpc_bid_ceiling_micros.
                             * @member {"cpc_bid_ceiling_micros"|undefined} _cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @instance
                             */
                            Object.defineProperty(TargetRoas.prototype, "_cpc_bid_ceiling_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_ceiling_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoas _cpc_bid_floor_micros.
                             * @member {"cpc_bid_floor_micros"|undefined} _cpc_bid_floor_micros
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @instance
                             */
                            Object.defineProperty(TargetRoas.prototype, "_cpc_bid_floor_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_floor_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TargetRoas instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoas=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetRoas} TargetRoas instance
                             */
                            TargetRoas.create = function create(properties) {
                                return new TargetRoas(properties);
                            };
    
                            /**
                             * Encodes the specified TargetRoas message. Does not implicitly {@link google.ads.googleads.v11.common.TargetRoas.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoas} m TargetRoas message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRoas.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                    w.uint32(33).double(m.target_roas);
                                if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                    w.uint32(40).int64(m.cpc_bid_ceiling_micros);
                                if (m.cpc_bid_floor_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_floor_micros"))
                                    w.uint32(48).int64(m.cpc_bid_floor_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetRoas message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetRoas.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoas} message TargetRoas message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRoas.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetRoas message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetRoas} TargetRoas
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRoas.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetRoas();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.target_roas = r.double();
                                        break;
                                    case 5:
                                        m.cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    case 6:
                                        m.cpc_bid_floor_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetRoas message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetRoas} TargetRoas
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRoas.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetRoas message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetRoas.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    p._target_roas = 1;
                                    if (typeof m.target_roas !== "number")
                                        return "target_roas: number expected";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    p._cpc_bid_ceiling_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                        return "cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    p._cpc_bid_floor_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_floor_micros) && !(m.cpc_bid_floor_micros && $util.isInteger(m.cpc_bid_floor_micros.low) && $util.isInteger(m.cpc_bid_floor_micros.high)))
                                        return "cpc_bid_floor_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetRoas message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetRoas} TargetRoas
                             */
                            TargetRoas.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetRoas)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetRoas();
                                if (d.target_roas != null) {
                                    m.target_roas = Number(d.target_roas);
                                }
                                if (d.cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_ceiling_micros === "string")
                                        m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.cpc_bid_ceiling_micros === "number")
                                        m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                    else if (typeof d.cpc_bid_ceiling_micros === "object")
                                        m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_floor_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_floor_micros = $util.Long.fromValue(d.cpc_bid_floor_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_floor_micros === "string")
                                        m.cpc_bid_floor_micros = parseInt(d.cpc_bid_floor_micros, 10);
                                    else if (typeof d.cpc_bid_floor_micros === "number")
                                        m.cpc_bid_floor_micros = d.cpc_bid_floor_micros;
                                    else if (typeof d.cpc_bid_floor_micros === "object")
                                        m.cpc_bid_floor_micros = new $util.LongBits(d.cpc_bid_floor_micros.low >>> 0, d.cpc_bid_floor_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetRoas message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetRoas} m TargetRoas
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetRoas.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    d.target_roas = o.json && !isFinite(m.target_roas) ? String(m.target_roas) : m.target_roas;
                                    if (o.oneofs)
                                        d._target_roas = "target_roas";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (typeof m.cpc_bid_ceiling_micros === "number")
                                        d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                    else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_ceiling_micros = "cpc_bid_ceiling_micros";
                                }
                                if (m.cpc_bid_floor_micros != null && m.hasOwnProperty("cpc_bid_floor_micros")) {
                                    if (typeof m.cpc_bid_floor_micros === "number")
                                        d.cpc_bid_floor_micros = o.longs === String ? String(m.cpc_bid_floor_micros) : m.cpc_bid_floor_micros;
                                    else
                                        d.cpc_bid_floor_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_floor_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_floor_micros.low >>> 0, m.cpc_bid_floor_micros.high >>> 0).toNumber() : m.cpc_bid_floor_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_floor_micros = "cpc_bid_floor_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetRoas to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetRoas
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetRoas.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetRoas;
                        })();
    
                        common.TargetSpend = (function() {
    
                            /**
                             * Properties of a TargetSpend.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetSpend
                             * @property {number|null} [target_spend_micros] TargetSpend target_spend_micros
                             * @property {number|null} [cpc_bid_ceiling_micros] TargetSpend cpc_bid_ceiling_micros
                             */
    
                            /**
                             * Constructs a new TargetSpend.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetSpend.
                             * @implements ITargetSpend
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetSpend=} [p] Properties to set
                             */
                            function TargetSpend(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetSpend target_spend_micros.
                             * @member {number|null|undefined} target_spend_micros
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @instance
                             */
                            TargetSpend.prototype.target_spend_micros = null;
    
                            /**
                             * TargetSpend cpc_bid_ceiling_micros.
                             * @member {number|null|undefined} cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @instance
                             */
                            TargetSpend.prototype.cpc_bid_ceiling_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TargetSpend _target_spend_micros.
                             * @member {"target_spend_micros"|undefined} _target_spend_micros
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @instance
                             */
                            Object.defineProperty(TargetSpend.prototype, "_target_spend_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["target_spend_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetSpend _cpc_bid_ceiling_micros.
                             * @member {"cpc_bid_ceiling_micros"|undefined} _cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @instance
                             */
                            Object.defineProperty(TargetSpend.prototype, "_cpc_bid_ceiling_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_ceiling_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TargetSpend instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetSpend=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetSpend} TargetSpend instance
                             */
                            TargetSpend.create = function create(properties) {
                                return new TargetSpend(properties);
                            };
    
                            /**
                             * Encodes the specified TargetSpend message. Does not implicitly {@link google.ads.googleads.v11.common.TargetSpend.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetSpend} m TargetSpend message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetSpend.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.target_spend_micros != null && Object.hasOwnProperty.call(m, "target_spend_micros"))
                                    w.uint32(24).int64(m.target_spend_micros);
                                if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                    w.uint32(32).int64(m.cpc_bid_ceiling_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetSpend message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetSpend.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetSpend} message TargetSpend message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetSpend.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetSpend message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetSpend} TargetSpend
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetSpend.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetSpend();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.target_spend_micros = r.int64();
                                        break;
                                    case 4:
                                        m.cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetSpend message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetSpend} TargetSpend
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetSpend.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetSpend message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetSpend.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.target_spend_micros != null && m.hasOwnProperty("target_spend_micros")) {
                                    p._target_spend_micros = 1;
                                    if (!$util.isInteger(m.target_spend_micros) && !(m.target_spend_micros && $util.isInteger(m.target_spend_micros.low) && $util.isInteger(m.target_spend_micros.high)))
                                        return "target_spend_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    p._cpc_bid_ceiling_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                        return "cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetSpend message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetSpend} TargetSpend
                             */
                            TargetSpend.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetSpend)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetSpend();
                                if (d.target_spend_micros != null) {
                                    if ($util.Long)
                                        (m.target_spend_micros = $util.Long.fromValue(d.target_spend_micros)).unsigned = false;
                                    else if (typeof d.target_spend_micros === "string")
                                        m.target_spend_micros = parseInt(d.target_spend_micros, 10);
                                    else if (typeof d.target_spend_micros === "number")
                                        m.target_spend_micros = d.target_spend_micros;
                                    else if (typeof d.target_spend_micros === "object")
                                        m.target_spend_micros = new $util.LongBits(d.target_spend_micros.low >>> 0, d.target_spend_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_ceiling_micros === "string")
                                        m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.cpc_bid_ceiling_micros === "number")
                                        m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                    else if (typeof d.cpc_bid_ceiling_micros === "object")
                                        m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetSpend message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetSpend} m TargetSpend
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetSpend.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.target_spend_micros != null && m.hasOwnProperty("target_spend_micros")) {
                                    if (typeof m.target_spend_micros === "number")
                                        d.target_spend_micros = o.longs === String ? String(m.target_spend_micros) : m.target_spend_micros;
                                    else
                                        d.target_spend_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_spend_micros) : o.longs === Number ? new $util.LongBits(m.target_spend_micros.low >>> 0, m.target_spend_micros.high >>> 0).toNumber() : m.target_spend_micros;
                                    if (o.oneofs)
                                        d._target_spend_micros = "target_spend_micros";
                                }
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (typeof m.cpc_bid_ceiling_micros === "number")
                                        d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                    else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_ceiling_micros = "cpc_bid_ceiling_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetSpend to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetSpend
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetSpend.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetSpend;
                        })();
    
                        common.PercentCpc = (function() {
    
                            /**
                             * Properties of a PercentCpc.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPercentCpc
                             * @property {number|null} [cpc_bid_ceiling_micros] PercentCpc cpc_bid_ceiling_micros
                             * @property {boolean|null} [enhanced_cpc_enabled] PercentCpc enhanced_cpc_enabled
                             */
    
                            /**
                             * Constructs a new PercentCpc.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PercentCpc.
                             * @implements IPercentCpc
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPercentCpc=} [p] Properties to set
                             */
                            function PercentCpc(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PercentCpc cpc_bid_ceiling_micros.
                             * @member {number|null|undefined} cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @instance
                             */
                            PercentCpc.prototype.cpc_bid_ceiling_micros = null;
    
                            /**
                             * PercentCpc enhanced_cpc_enabled.
                             * @member {boolean|null|undefined} enhanced_cpc_enabled
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @instance
                             */
                            PercentCpc.prototype.enhanced_cpc_enabled = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PercentCpc _cpc_bid_ceiling_micros.
                             * @member {"cpc_bid_ceiling_micros"|undefined} _cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @instance
                             */
                            Object.defineProperty(PercentCpc.prototype, "_cpc_bid_ceiling_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_ceiling_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PercentCpc _enhanced_cpc_enabled.
                             * @member {"enhanced_cpc_enabled"|undefined} _enhanced_cpc_enabled
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @instance
                             */
                            Object.defineProperty(PercentCpc.prototype, "_enhanced_cpc_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["enhanced_cpc_enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PercentCpc instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpc=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PercentCpc} PercentCpc instance
                             */
                            PercentCpc.create = function create(properties) {
                                return new PercentCpc(properties);
                            };
    
                            /**
                             * Encodes the specified PercentCpc message. Does not implicitly {@link google.ads.googleads.v11.common.PercentCpc.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpc} m PercentCpc message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PercentCpc.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                    w.uint32(24).int64(m.cpc_bid_ceiling_micros);
                                if (m.enhanced_cpc_enabled != null && Object.hasOwnProperty.call(m, "enhanced_cpc_enabled"))
                                    w.uint32(32).bool(m.enhanced_cpc_enabled);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PercentCpc message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PercentCpc.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpc} message PercentCpc message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PercentCpc.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PercentCpc message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PercentCpc} PercentCpc
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PercentCpc.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PercentCpc();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    case 4:
                                        m.enhanced_cpc_enabled = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PercentCpc message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PercentCpc} PercentCpc
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PercentCpc.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PercentCpc message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PercentCpc.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    p._cpc_bid_ceiling_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                        return "cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                if (m.enhanced_cpc_enabled != null && m.hasOwnProperty("enhanced_cpc_enabled")) {
                                    p._enhanced_cpc_enabled = 1;
                                    if (typeof m.enhanced_cpc_enabled !== "boolean")
                                        return "enhanced_cpc_enabled: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PercentCpc message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PercentCpc} PercentCpc
                             */
                            PercentCpc.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PercentCpc)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PercentCpc();
                                if (d.cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_ceiling_micros === "string")
                                        m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.cpc_bid_ceiling_micros === "number")
                                        m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                    else if (typeof d.cpc_bid_ceiling_micros === "object")
                                        m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                if (d.enhanced_cpc_enabled != null) {
                                    m.enhanced_cpc_enabled = Boolean(d.enhanced_cpc_enabled);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PercentCpc message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @static
                             * @param {google.ads.googleads.v11.common.PercentCpc} m PercentCpc
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PercentCpc.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                    if (typeof m.cpc_bid_ceiling_micros === "number")
                                        d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                    else
                                        d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_ceiling_micros = "cpc_bid_ceiling_micros";
                                }
                                if (m.enhanced_cpc_enabled != null && m.hasOwnProperty("enhanced_cpc_enabled")) {
                                    d.enhanced_cpc_enabled = m.enhanced_cpc_enabled;
                                    if (o.oneofs)
                                        d._enhanced_cpc_enabled = "enhanced_cpc_enabled";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PercentCpc to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PercentCpc
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PercentCpc.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PercentCpc;
                        })();
    
                        common.ClickLocation = (function() {
    
                            /**
                             * Properties of a ClickLocation.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IClickLocation
                             * @property {string|null} [city] ClickLocation city
                             * @property {string|null} [country] ClickLocation country
                             * @property {string|null} [metro] ClickLocation metro
                             * @property {string|null} [most_specific] ClickLocation most_specific
                             * @property {string|null} [region] ClickLocation region
                             */
    
                            /**
                             * Constructs a new ClickLocation.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ClickLocation.
                             * @implements IClickLocation
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IClickLocation=} [p] Properties to set
                             */
                            function ClickLocation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ClickLocation city.
                             * @member {string|null|undefined} city
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            ClickLocation.prototype.city = null;
    
                            /**
                             * ClickLocation country.
                             * @member {string|null|undefined} country
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            ClickLocation.prototype.country = null;
    
                            /**
                             * ClickLocation metro.
                             * @member {string|null|undefined} metro
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            ClickLocation.prototype.metro = null;
    
                            /**
                             * ClickLocation most_specific.
                             * @member {string|null|undefined} most_specific
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            ClickLocation.prototype.most_specific = null;
    
                            /**
                             * ClickLocation region.
                             * @member {string|null|undefined} region
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            ClickLocation.prototype.region = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ClickLocation _city.
                             * @member {"city"|undefined} _city
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            Object.defineProperty(ClickLocation.prototype, "_city", {
                                get: $util.oneOfGetter($oneOfFields = ["city"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickLocation _country.
                             * @member {"country"|undefined} _country
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            Object.defineProperty(ClickLocation.prototype, "_country", {
                                get: $util.oneOfGetter($oneOfFields = ["country"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickLocation _metro.
                             * @member {"metro"|undefined} _metro
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            Object.defineProperty(ClickLocation.prototype, "_metro", {
                                get: $util.oneOfGetter($oneOfFields = ["metro"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickLocation _most_specific.
                             * @member {"most_specific"|undefined} _most_specific
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            Object.defineProperty(ClickLocation.prototype, "_most_specific", {
                                get: $util.oneOfGetter($oneOfFields = ["most_specific"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickLocation _region.
                             * @member {"region"|undefined} _region
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             */
                            Object.defineProperty(ClickLocation.prototype, "_region", {
                                get: $util.oneOfGetter($oneOfFields = ["region"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ClickLocation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {google.ads.googleads.v11.common.IClickLocation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ClickLocation} ClickLocation instance
                             */
                            ClickLocation.create = function create(properties) {
                                return new ClickLocation(properties);
                            };
    
                            /**
                             * Encodes the specified ClickLocation message. Does not implicitly {@link google.ads.googleads.v11.common.ClickLocation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {google.ads.googleads.v11.common.IClickLocation} m ClickLocation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickLocation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.city != null && Object.hasOwnProperty.call(m, "city"))
                                    w.uint32(50).string(m.city);
                                if (m.country != null && Object.hasOwnProperty.call(m, "country"))
                                    w.uint32(58).string(m.country);
                                if (m.metro != null && Object.hasOwnProperty.call(m, "metro"))
                                    w.uint32(66).string(m.metro);
                                if (m.most_specific != null && Object.hasOwnProperty.call(m, "most_specific"))
                                    w.uint32(74).string(m.most_specific);
                                if (m.region != null && Object.hasOwnProperty.call(m, "region"))
                                    w.uint32(82).string(m.region);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ClickLocation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ClickLocation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {google.ads.googleads.v11.common.IClickLocation} message ClickLocation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickLocation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClickLocation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ClickLocation} ClickLocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickLocation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ClickLocation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.city = r.string();
                                        break;
                                    case 7:
                                        m.country = r.string();
                                        break;
                                    case 8:
                                        m.metro = r.string();
                                        break;
                                    case 9:
                                        m.most_specific = r.string();
                                        break;
                                    case 10:
                                        m.region = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ClickLocation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ClickLocation} ClickLocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickLocation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClickLocation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClickLocation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    p._city = 1;
                                    if (!$util.isString(m.city))
                                        return "city: string expected";
                                }
                                if (m.country != null && m.hasOwnProperty("country")) {
                                    p._country = 1;
                                    if (!$util.isString(m.country))
                                        return "country: string expected";
                                }
                                if (m.metro != null && m.hasOwnProperty("metro")) {
                                    p._metro = 1;
                                    if (!$util.isString(m.metro))
                                        return "metro: string expected";
                                }
                                if (m.most_specific != null && m.hasOwnProperty("most_specific")) {
                                    p._most_specific = 1;
                                    if (!$util.isString(m.most_specific))
                                        return "most_specific: string expected";
                                }
                                if (m.region != null && m.hasOwnProperty("region")) {
                                    p._region = 1;
                                    if (!$util.isString(m.region))
                                        return "region: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ClickLocation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ClickLocation} ClickLocation
                             */
                            ClickLocation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ClickLocation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ClickLocation();
                                if (d.city != null) {
                                    m.city = String(d.city);
                                }
                                if (d.country != null) {
                                    m.country = String(d.country);
                                }
                                if (d.metro != null) {
                                    m.metro = String(d.metro);
                                }
                                if (d.most_specific != null) {
                                    m.most_specific = String(d.most_specific);
                                }
                                if (d.region != null) {
                                    m.region = String(d.region);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ClickLocation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @static
                             * @param {google.ads.googleads.v11.common.ClickLocation} m ClickLocation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClickLocation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    d.city = m.city;
                                    if (o.oneofs)
                                        d._city = "city";
                                }
                                if (m.country != null && m.hasOwnProperty("country")) {
                                    d.country = m.country;
                                    if (o.oneofs)
                                        d._country = "country";
                                }
                                if (m.metro != null && m.hasOwnProperty("metro")) {
                                    d.metro = m.metro;
                                    if (o.oneofs)
                                        d._metro = "metro";
                                }
                                if (m.most_specific != null && m.hasOwnProperty("most_specific")) {
                                    d.most_specific = m.most_specific;
                                    if (o.oneofs)
                                        d._most_specific = "most_specific";
                                }
                                if (m.region != null && m.hasOwnProperty("region")) {
                                    d.region = m.region;
                                    if (o.oneofs)
                                        d._region = "region";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ClickLocation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ClickLocation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClickLocation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ClickLocation;
                        })();
    
                        common.CriterionCategoryAvailability = (function() {
    
                            /**
                             * Properties of a CriterionCategoryAvailability.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICriterionCategoryAvailability
                             * @property {google.ads.googleads.v11.common.ICriterionCategoryChannelAvailability|null} [channel] CriterionCategoryAvailability channel
                             * @property {Array.<google.ads.googleads.v11.common.ICriterionCategoryLocaleAvailability>|null} [locale] CriterionCategoryAvailability locale
                             */
    
                            /**
                             * Constructs a new CriterionCategoryAvailability.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CriterionCategoryAvailability.
                             * @implements ICriterionCategoryAvailability
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryAvailability=} [p] Properties to set
                             */
                            function CriterionCategoryAvailability(p) {
                                this.locale = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CriterionCategoryAvailability channel.
                             * @member {google.ads.googleads.v11.common.ICriterionCategoryChannelAvailability|null|undefined} channel
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @instance
                             */
                            CriterionCategoryAvailability.prototype.channel = null;
    
                            /**
                             * CriterionCategoryAvailability locale.
                             * @member {Array.<google.ads.googleads.v11.common.ICriterionCategoryLocaleAvailability>} locale
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @instance
                             */
                            CriterionCategoryAvailability.prototype.locale = $util.emptyArray;
    
                            /**
                             * Creates a new CriterionCategoryAvailability instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryAvailability=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryAvailability} CriterionCategoryAvailability instance
                             */
                            CriterionCategoryAvailability.create = function create(properties) {
                                return new CriterionCategoryAvailability(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryAvailability message. Does not implicitly {@link google.ads.googleads.v11.common.CriterionCategoryAvailability.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryAvailability} m CriterionCategoryAvailability message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryAvailability.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.channel != null && Object.hasOwnProperty.call(m, "channel"))
                                    $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.encode(m.channel, w.uint32(10).fork()).ldelim();
                                if (m.locale != null && m.locale.length) {
                                    for (var i = 0; i < m.locale.length; ++i)
                                        $root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability.encode(m.locale[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryAvailability message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CriterionCategoryAvailability.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryAvailability} message CriterionCategoryAvailability message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryAvailability.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionCategoryAvailability message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryAvailability} CriterionCategoryAvailability
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryAvailability.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CriterionCategoryAvailability();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.channel = $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.locale && m.locale.length))
                                            m.locale = [];
                                        m.locale.push($root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionCategoryAvailability message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryAvailability} CriterionCategoryAvailability
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryAvailability.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionCategoryAvailability message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionCategoryAvailability.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.channel != null && m.hasOwnProperty("channel")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.verify(m.channel);
                                        if (e)
                                            return "channel." + e;
                                    }
                                }
                                if (m.locale != null && m.hasOwnProperty("locale")) {
                                    if (!Array.isArray(m.locale))
                                        return "locale: array expected";
                                    for (var i = 0; i < m.locale.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability.verify(m.locale[i]);
                                            if (e)
                                                return "locale." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CriterionCategoryAvailability message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryAvailability} CriterionCategoryAvailability
                             */
                            CriterionCategoryAvailability.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CriterionCategoryAvailability)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CriterionCategoryAvailability();
                                if (d.channel != null) {
                                    if (typeof d.channel !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.CriterionCategoryAvailability.channel: object expected");
                                    m.channel = $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.fromObject(d.channel);
                                }
                                if (d.locale) {
                                    if (!Array.isArray(d.locale))
                                        throw TypeError(".google.ads.googleads.v11.common.CriterionCategoryAvailability.locale: array expected");
                                    m.locale = [];
                                    for (var i = 0; i < d.locale.length; ++i) {
                                        if (typeof d.locale[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.CriterionCategoryAvailability.locale: object expected");
                                        m.locale[i] = $root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability.fromObject(d.locale[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CriterionCategoryAvailability message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.CriterionCategoryAvailability} m CriterionCategoryAvailability
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionCategoryAvailability.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.locale = [];
                                }
                                if (o.defaults) {
                                    d.channel = null;
                                }
                                if (m.channel != null && m.hasOwnProperty("channel")) {
                                    d.channel = $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.toObject(m.channel, o);
                                }
                                if (m.locale && m.locale.length) {
                                    d.locale = [];
                                    for (var j = 0; j < m.locale.length; ++j) {
                                        d.locale[j] = $root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability.toObject(m.locale[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CriterionCategoryAvailability to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryAvailability
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionCategoryAvailability.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CriterionCategoryAvailability;
                        })();
    
                        common.CriterionCategoryChannelAvailability = (function() {
    
                            /**
                             * Properties of a CriterionCategoryChannelAvailability.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICriterionCategoryChannelAvailability
                             * @property {google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum.CriterionCategoryChannelAvailabilityMode|null} [availability_mode] CriterionCategoryChannelAvailability availability_mode
                             * @property {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType|null} [advertising_channel_type] CriterionCategoryChannelAvailability advertising_channel_type
                             * @property {Array.<google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType>|null} [advertising_channel_sub_type] CriterionCategoryChannelAvailability advertising_channel_sub_type
                             * @property {boolean|null} [include_default_channel_sub_type] CriterionCategoryChannelAvailability include_default_channel_sub_type
                             */
    
                            /**
                             * Constructs a new CriterionCategoryChannelAvailability.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CriterionCategoryChannelAvailability.
                             * @implements ICriterionCategoryChannelAvailability
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryChannelAvailability=} [p] Properties to set
                             */
                            function CriterionCategoryChannelAvailability(p) {
                                this.advertising_channel_sub_type = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CriterionCategoryChannelAvailability availability_mode.
                             * @member {google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum.CriterionCategoryChannelAvailabilityMode} availability_mode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @instance
                             */
                            CriterionCategoryChannelAvailability.prototype.availability_mode = 0;
    
                            /**
                             * CriterionCategoryChannelAvailability advertising_channel_type.
                             * @member {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType} advertising_channel_type
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @instance
                             */
                            CriterionCategoryChannelAvailability.prototype.advertising_channel_type = 0;
    
                            /**
                             * CriterionCategoryChannelAvailability advertising_channel_sub_type.
                             * @member {Array.<google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType>} advertising_channel_sub_type
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @instance
                             */
                            CriterionCategoryChannelAvailability.prototype.advertising_channel_sub_type = $util.emptyArray;
    
                            /**
                             * CriterionCategoryChannelAvailability include_default_channel_sub_type.
                             * @member {boolean|null|undefined} include_default_channel_sub_type
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @instance
                             */
                            CriterionCategoryChannelAvailability.prototype.include_default_channel_sub_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CriterionCategoryChannelAvailability _include_default_channel_sub_type.
                             * @member {"include_default_channel_sub_type"|undefined} _include_default_channel_sub_type
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @instance
                             */
                            Object.defineProperty(CriterionCategoryChannelAvailability.prototype, "_include_default_channel_sub_type", {
                                get: $util.oneOfGetter($oneOfFields = ["include_default_channel_sub_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CriterionCategoryChannelAvailability instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryChannelAvailability=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryChannelAvailability} CriterionCategoryChannelAvailability instance
                             */
                            CriterionCategoryChannelAvailability.create = function create(properties) {
                                return new CriterionCategoryChannelAvailability(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryChannelAvailability message. Does not implicitly {@link google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryChannelAvailability} m CriterionCategoryChannelAvailability message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryChannelAvailability.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.availability_mode != null && Object.hasOwnProperty.call(m, "availability_mode"))
                                    w.uint32(8).int32(m.availability_mode);
                                if (m.advertising_channel_type != null && Object.hasOwnProperty.call(m, "advertising_channel_type"))
                                    w.uint32(16).int32(m.advertising_channel_type);
                                if (m.advertising_channel_sub_type != null && m.advertising_channel_sub_type.length) {
                                    w.uint32(26).fork();
                                    for (var i = 0; i < m.advertising_channel_sub_type.length; ++i)
                                        w.int32(m.advertising_channel_sub_type[i]);
                                    w.ldelim();
                                }
                                if (m.include_default_channel_sub_type != null && Object.hasOwnProperty.call(m, "include_default_channel_sub_type"))
                                    w.uint32(40).bool(m.include_default_channel_sub_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryChannelAvailability message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryChannelAvailability} message CriterionCategoryChannelAvailability message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryChannelAvailability.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionCategoryChannelAvailability message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryChannelAvailability} CriterionCategoryChannelAvailability
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryChannelAvailability.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.availability_mode = r.int32();
                                        break;
                                    case 2:
                                        m.advertising_channel_type = r.int32();
                                        break;
                                    case 3:
                                        if (!(m.advertising_channel_sub_type && m.advertising_channel_sub_type.length))
                                            m.advertising_channel_sub_type = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.advertising_channel_sub_type.push(r.int32());
                                        } else
                                            m.advertising_channel_sub_type.push(r.int32());
                                        break;
                                    case 5:
                                        m.include_default_channel_sub_type = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionCategoryChannelAvailability message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryChannelAvailability} CriterionCategoryChannelAvailability
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryChannelAvailability.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionCategoryChannelAvailability message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionCategoryChannelAvailability.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.availability_mode != null && m.hasOwnProperty("availability_mode")) {
                                    switch (m.availability_mode) {
                                    default:
                                        return "availability_mode: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.advertising_channel_type != null && m.hasOwnProperty("advertising_channel_type")) {
                                    switch (m.advertising_channel_type) {
                                    default:
                                        return "advertising_channel_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.advertising_channel_sub_type != null && m.hasOwnProperty("advertising_channel_sub_type")) {
                                    if (!Array.isArray(m.advertising_channel_sub_type))
                                        return "advertising_channel_sub_type: array expected";
                                    for (var i = 0; i < m.advertising_channel_sub_type.length; ++i) {
                                        switch (m.advertising_channel_sub_type[i]) {
                                        default:
                                            return "advertising_channel_sub_type: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 17:
                                        case 18:
                                            break;
                                        }
                                    }
                                }
                                if (m.include_default_channel_sub_type != null && m.hasOwnProperty("include_default_channel_sub_type")) {
                                    p._include_default_channel_sub_type = 1;
                                    if (typeof m.include_default_channel_sub_type !== "boolean")
                                        return "include_default_channel_sub_type: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CriterionCategoryChannelAvailability message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryChannelAvailability} CriterionCategoryChannelAvailability
                             */
                            CriterionCategoryChannelAvailability.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CriterionCategoryChannelAvailability();
                                switch (d.availability_mode) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.availability_mode = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.availability_mode = 1;
                                    break;
                                case "ALL_CHANNELS":
                                case 2:
                                    m.availability_mode = 2;
                                    break;
                                case "CHANNEL_TYPE_AND_ALL_SUBTYPES":
                                case 3:
                                    m.availability_mode = 3;
                                    break;
                                case "CHANNEL_TYPE_AND_SUBSET_SUBTYPES":
                                case 4:
                                    m.availability_mode = 4;
                                    break;
                                }
                                switch (d.advertising_channel_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.advertising_channel_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.advertising_channel_type = 1;
                                    break;
                                case "SEARCH":
                                case 2:
                                    m.advertising_channel_type = 2;
                                    break;
                                case "DISPLAY":
                                case 3:
                                    m.advertising_channel_type = 3;
                                    break;
                                case "SHOPPING":
                                case 4:
                                    m.advertising_channel_type = 4;
                                    break;
                                case "HOTEL":
                                case 5:
                                    m.advertising_channel_type = 5;
                                    break;
                                case "VIDEO":
                                case 6:
                                    m.advertising_channel_type = 6;
                                    break;
                                case "MULTI_CHANNEL":
                                case 7:
                                    m.advertising_channel_type = 7;
                                    break;
                                case "LOCAL":
                                case 8:
                                    m.advertising_channel_type = 8;
                                    break;
                                case "SMART":
                                case 9:
                                    m.advertising_channel_type = 9;
                                    break;
                                case "PERFORMANCE_MAX":
                                case 10:
                                    m.advertising_channel_type = 10;
                                    break;
                                case "LOCAL_SERVICES":
                                case 11:
                                    m.advertising_channel_type = 11;
                                    break;
                                case "DISCOVERY":
                                case 12:
                                    m.advertising_channel_type = 12;
                                    break;
                                }
                                if (d.advertising_channel_sub_type) {
                                    if (!Array.isArray(d.advertising_channel_sub_type))
                                        throw TypeError(".google.ads.googleads.v11.common.CriterionCategoryChannelAvailability.advertising_channel_sub_type: array expected");
                                    m.advertising_channel_sub_type = [];
                                    for (var i = 0; i < d.advertising_channel_sub_type.length; ++i) {
                                        switch (d.advertising_channel_sub_type[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.advertising_channel_sub_type[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.advertising_channel_sub_type[i] = 1;
                                            break;
                                        case "SEARCH_MOBILE_APP":
                                        case 2:
                                            m.advertising_channel_sub_type[i] = 2;
                                            break;
                                        case "DISPLAY_MOBILE_APP":
                                        case 3:
                                            m.advertising_channel_sub_type[i] = 3;
                                            break;
                                        case "SEARCH_EXPRESS":
                                        case 4:
                                            m.advertising_channel_sub_type[i] = 4;
                                            break;
                                        case "DISPLAY_EXPRESS":
                                        case 5:
                                            m.advertising_channel_sub_type[i] = 5;
                                            break;
                                        case "SHOPPING_SMART_ADS":
                                        case 6:
                                            m.advertising_channel_sub_type[i] = 6;
                                            break;
                                        case "DISPLAY_GMAIL_AD":
                                        case 7:
                                            m.advertising_channel_sub_type[i] = 7;
                                            break;
                                        case "DISPLAY_SMART_CAMPAIGN":
                                        case 8:
                                            m.advertising_channel_sub_type[i] = 8;
                                            break;
                                        case "VIDEO_OUTSTREAM":
                                        case 9:
                                            m.advertising_channel_sub_type[i] = 9;
                                            break;
                                        case "VIDEO_ACTION":
                                        case 10:
                                            m.advertising_channel_sub_type[i] = 10;
                                            break;
                                        case "VIDEO_NON_SKIPPABLE":
                                        case 11:
                                            m.advertising_channel_sub_type[i] = 11;
                                            break;
                                        case "APP_CAMPAIGN":
                                        case 12:
                                            m.advertising_channel_sub_type[i] = 12;
                                            break;
                                        case "APP_CAMPAIGN_FOR_ENGAGEMENT":
                                        case 13:
                                            m.advertising_channel_sub_type[i] = 13;
                                            break;
                                        case "LOCAL_CAMPAIGN":
                                        case 14:
                                            m.advertising_channel_sub_type[i] = 14;
                                            break;
                                        case "SHOPPING_COMPARISON_LISTING_ADS":
                                        case 15:
                                            m.advertising_channel_sub_type[i] = 15;
                                            break;
                                        case "SMART_CAMPAIGN":
                                        case 16:
                                            m.advertising_channel_sub_type[i] = 16;
                                            break;
                                        case "VIDEO_SEQUENCE":
                                        case 17:
                                            m.advertising_channel_sub_type[i] = 17;
                                            break;
                                        case "APP_CAMPAIGN_FOR_PRE_REGISTRATION":
                                        case 18:
                                            m.advertising_channel_sub_type[i] = 18;
                                            break;
                                        }
                                    }
                                }
                                if (d.include_default_channel_sub_type != null) {
                                    m.include_default_channel_sub_type = Boolean(d.include_default_channel_sub_type);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CriterionCategoryChannelAvailability message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.CriterionCategoryChannelAvailability} m CriterionCategoryChannelAvailability
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionCategoryChannelAvailability.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.advertising_channel_sub_type = [];
                                }
                                if (o.defaults) {
                                    d.availability_mode = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.advertising_channel_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.availability_mode != null && m.hasOwnProperty("availability_mode")) {
                                    d.availability_mode = o.enums === String ? $root.google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum.CriterionCategoryChannelAvailabilityMode[m.availability_mode] : m.availability_mode;
                                }
                                if (m.advertising_channel_type != null && m.hasOwnProperty("advertising_channel_type")) {
                                    d.advertising_channel_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType[m.advertising_channel_type] : m.advertising_channel_type;
                                }
                                if (m.advertising_channel_sub_type && m.advertising_channel_sub_type.length) {
                                    d.advertising_channel_sub_type = [];
                                    for (var j = 0; j < m.advertising_channel_sub_type.length; ++j) {
                                        d.advertising_channel_sub_type[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType[m.advertising_channel_sub_type[j]] : m.advertising_channel_sub_type[j];
                                    }
                                }
                                if (m.include_default_channel_sub_type != null && m.hasOwnProperty("include_default_channel_sub_type")) {
                                    d.include_default_channel_sub_type = m.include_default_channel_sub_type;
                                    if (o.oneofs)
                                        d._include_default_channel_sub_type = "include_default_channel_sub_type";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CriterionCategoryChannelAvailability to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryChannelAvailability
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionCategoryChannelAvailability.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CriterionCategoryChannelAvailability;
                        })();
    
                        common.CriterionCategoryLocaleAvailability = (function() {
    
                            /**
                             * Properties of a CriterionCategoryLocaleAvailability.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICriterionCategoryLocaleAvailability
                             * @property {google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum.CriterionCategoryLocaleAvailabilityMode|null} [availability_mode] CriterionCategoryLocaleAvailability availability_mode
                             * @property {string|null} [country_code] CriterionCategoryLocaleAvailability country_code
                             * @property {string|null} [language_code] CriterionCategoryLocaleAvailability language_code
                             */
    
                            /**
                             * Constructs a new CriterionCategoryLocaleAvailability.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CriterionCategoryLocaleAvailability.
                             * @implements ICriterionCategoryLocaleAvailability
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryLocaleAvailability=} [p] Properties to set
                             */
                            function CriterionCategoryLocaleAvailability(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CriterionCategoryLocaleAvailability availability_mode.
                             * @member {google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum.CriterionCategoryLocaleAvailabilityMode} availability_mode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @instance
                             */
                            CriterionCategoryLocaleAvailability.prototype.availability_mode = 0;
    
                            /**
                             * CriterionCategoryLocaleAvailability country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @instance
                             */
                            CriterionCategoryLocaleAvailability.prototype.country_code = null;
    
                            /**
                             * CriterionCategoryLocaleAvailability language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @instance
                             */
                            CriterionCategoryLocaleAvailability.prototype.language_code = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CriterionCategoryLocaleAvailability _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @instance
                             */
                            Object.defineProperty(CriterionCategoryLocaleAvailability.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CriterionCategoryLocaleAvailability _language_code.
                             * @member {"language_code"|undefined} _language_code
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @instance
                             */
                            Object.defineProperty(CriterionCategoryLocaleAvailability.prototype, "_language_code", {
                                get: $util.oneOfGetter($oneOfFields = ["language_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CriterionCategoryLocaleAvailability instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryLocaleAvailability=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability} CriterionCategoryLocaleAvailability instance
                             */
                            CriterionCategoryLocaleAvailability.create = function create(properties) {
                                return new CriterionCategoryLocaleAvailability(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryLocaleAvailability message. Does not implicitly {@link google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryLocaleAvailability} m CriterionCategoryLocaleAvailability message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryLocaleAvailability.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.availability_mode != null && Object.hasOwnProperty.call(m, "availability_mode"))
                                    w.uint32(8).int32(m.availability_mode);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(34).string(m.country_code);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(42).string(m.language_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryLocaleAvailability message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.ICriterionCategoryLocaleAvailability} message CriterionCategoryLocaleAvailability message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryLocaleAvailability.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionCategoryLocaleAvailability message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability} CriterionCategoryLocaleAvailability
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryLocaleAvailability.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.availability_mode = r.int32();
                                        break;
                                    case 4:
                                        m.country_code = r.string();
                                        break;
                                    case 5:
                                        m.language_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionCategoryLocaleAvailability message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability} CriterionCategoryLocaleAvailability
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryLocaleAvailability.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionCategoryLocaleAvailability message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionCategoryLocaleAvailability.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.availability_mode != null && m.hasOwnProperty("availability_mode")) {
                                    switch (m.availability_mode) {
                                    default:
                                        return "availability_mode: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    p._language_code = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CriterionCategoryLocaleAvailability message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability} CriterionCategoryLocaleAvailability
                             */
                            CriterionCategoryLocaleAvailability.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability();
                                switch (d.availability_mode) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.availability_mode = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.availability_mode = 1;
                                    break;
                                case "ALL_LOCALES":
                                case 2:
                                    m.availability_mode = 2;
                                    break;
                                case "COUNTRY_AND_ALL_LANGUAGES":
                                case 3:
                                    m.availability_mode = 3;
                                    break;
                                case "LANGUAGE_AND_ALL_COUNTRIES":
                                case 4:
                                    m.availability_mode = 4;
                                    break;
                                case "COUNTRY_AND_LANGUAGE":
                                case 5:
                                    m.availability_mode = 5;
                                    break;
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CriterionCategoryLocaleAvailability message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @static
                             * @param {google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability} m CriterionCategoryLocaleAvailability
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionCategoryLocaleAvailability.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.availability_mode = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.availability_mode != null && m.hasOwnProperty("availability_mode")) {
                                    d.availability_mode = o.enums === String ? $root.google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum.CriterionCategoryLocaleAvailabilityMode[m.availability_mode] : m.availability_mode;
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d._language_code = "language_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CriterionCategoryLocaleAvailability to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CriterionCategoryLocaleAvailability
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionCategoryLocaleAvailability.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CriterionCategoryLocaleAvailability;
                        })();
    
                        common.CustomParameter = (function() {
    
                            /**
                             * Properties of a CustomParameter.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICustomParameter
                             * @property {string|null} [key] CustomParameter key
                             * @property {string|null} [value] CustomParameter value
                             */
    
                            /**
                             * Constructs a new CustomParameter.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CustomParameter.
                             * @implements ICustomParameter
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICustomParameter=} [p] Properties to set
                             */
                            function CustomParameter(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomParameter key.
                             * @member {string|null|undefined} key
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @instance
                             */
                            CustomParameter.prototype.key = null;
    
                            /**
                             * CustomParameter value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @instance
                             */
                            CustomParameter.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomParameter _key.
                             * @member {"key"|undefined} _key
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @instance
                             */
                            Object.defineProperty(CustomParameter.prototype, "_key", {
                                get: $util.oneOfGetter($oneOfFields = ["key"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomParameter _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @instance
                             */
                            Object.defineProperty(CustomParameter.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomParameter instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomParameter=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CustomParameter} CustomParameter instance
                             */
                            CustomParameter.create = function create(properties) {
                                return new CustomParameter(properties);
                            };
    
                            /**
                             * Encodes the specified CustomParameter message. Does not implicitly {@link google.ads.googleads.v11.common.CustomParameter.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomParameter} m CustomParameter message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomParameter.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.key != null && Object.hasOwnProperty.call(m, "key"))
                                    w.uint32(26).string(m.key);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(34).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomParameter message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CustomParameter.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomParameter} message CustomParameter message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomParameter.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomParameter message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CustomParameter} CustomParameter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomParameter.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CustomParameter();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.key = r.string();
                                        break;
                                    case 4:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomParameter message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CustomParameter} CustomParameter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomParameter.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomParameter message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomParameter.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.key != null && m.hasOwnProperty("key")) {
                                    p._key = 1;
                                    if (!$util.isString(m.key))
                                        return "key: string expected";
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomParameter message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CustomParameter} CustomParameter
                             */
                            CustomParameter.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CustomParameter)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CustomParameter();
                                if (d.key != null) {
                                    m.key = String(d.key);
                                }
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomParameter message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @static
                             * @param {google.ads.googleads.v11.common.CustomParameter} m CustomParameter
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomParameter.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.key != null && m.hasOwnProperty("key")) {
                                    d.key = m.key;
                                    if (o.oneofs)
                                        d._key = "key";
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomParameter to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CustomParameter
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomParameter.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomParameter;
                        })();
    
                        common.CustomizerValue = (function() {
    
                            /**
                             * Properties of a CustomizerValue.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICustomizerValue
                             * @property {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.CustomizerAttributeType|null} [type] CustomizerValue type
                             * @property {string|null} [string_value] CustomizerValue string_value
                             */
    
                            /**
                             * Constructs a new CustomizerValue.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CustomizerValue.
                             * @implements ICustomizerValue
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICustomizerValue=} [p] Properties to set
                             */
                            function CustomizerValue(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomizerValue type.
                             * @member {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.CustomizerAttributeType} type
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @instance
                             */
                            CustomizerValue.prototype.type = 0;
    
                            /**
                             * CustomizerValue string_value.
                             * @member {string} string_value
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @instance
                             */
                            CustomizerValue.prototype.string_value = "";
    
                            /**
                             * Creates a new CustomizerValue instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomizerValue=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CustomizerValue} CustomizerValue instance
                             */
                            CustomizerValue.create = function create(properties) {
                                return new CustomizerValue(properties);
                            };
    
                            /**
                             * Encodes the specified CustomizerValue message. Does not implicitly {@link google.ads.googleads.v11.common.CustomizerValue.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomizerValue} m CustomizerValue message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerValue.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                if (m.string_value != null && Object.hasOwnProperty.call(m, "string_value"))
                                    w.uint32(18).string(m.string_value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomizerValue message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CustomizerValue.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomizerValue} message CustomizerValue message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerValue.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomizerValue message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CustomizerValue} CustomizerValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerValue.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CustomizerValue();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    case 2:
                                        m.string_value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomizerValue message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CustomizerValue} CustomizerValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerValue.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomizerValue message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomizerValue.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                    if (!$util.isString(m.string_value))
                                        return "string_value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomizerValue message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CustomizerValue} CustomizerValue
                             */
                            CustomizerValue.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CustomizerValue)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CustomizerValue();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "TEXT":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "NUMBER":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "PRICE":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "PERCENT":
                                case 5:
                                    m.type = 5;
                                    break;
                                }
                                if (d.string_value != null) {
                                    m.string_value = String(d.string_value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomizerValue message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @static
                             * @param {google.ads.googleads.v11.common.CustomizerValue} m CustomizerValue
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomizerValue.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.string_value = "";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.CustomizerAttributeType[m.type] : m.type;
                                }
                                if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                    d.string_value = m.string_value;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomizerValue to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CustomizerValue
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomizerValue.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomizerValue;
                        })();
    
                        common.DateRange = (function() {
    
                            /**
                             * Properties of a DateRange.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDateRange
                             * @property {string|null} [start_date] DateRange start_date
                             * @property {string|null} [end_date] DateRange end_date
                             */
    
                            /**
                             * Constructs a new DateRange.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DateRange.
                             * @implements IDateRange
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDateRange=} [p] Properties to set
                             */
                            function DateRange(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DateRange start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @instance
                             */
                            DateRange.prototype.start_date = null;
    
                            /**
                             * DateRange end_date.
                             * @member {string|null|undefined} end_date
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @instance
                             */
                            DateRange.prototype.end_date = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DateRange _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @instance
                             */
                            Object.defineProperty(DateRange.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DateRange _end_date.
                             * @member {"end_date"|undefined} _end_date
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @instance
                             */
                            Object.defineProperty(DateRange.prototype, "_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DateRange instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {google.ads.googleads.v11.common.IDateRange=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DateRange} DateRange instance
                             */
                            DateRange.create = function create(properties) {
                                return new DateRange(properties);
                            };
    
                            /**
                             * Encodes the specified DateRange message. Does not implicitly {@link google.ads.googleads.v11.common.DateRange.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {google.ads.googleads.v11.common.IDateRange} m DateRange message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DateRange.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(26).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(34).string(m.end_date);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DateRange message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DateRange.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {google.ads.googleads.v11.common.IDateRange} message DateRange message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DateRange.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DateRange message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DateRange} DateRange
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DateRange.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DateRange();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.start_date = r.string();
                                        break;
                                    case 4:
                                        m.end_date = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DateRange message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DateRange} DateRange
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DateRange.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DateRange message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DateRange.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    p._end_date = 1;
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DateRange message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DateRange} DateRange
                             */
                            DateRange.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DateRange)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DateRange();
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DateRange message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @static
                             * @param {google.ads.googleads.v11.common.DateRange} m DateRange
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DateRange.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                    if (o.oneofs)
                                        d._end_date = "end_date";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DateRange to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DateRange
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DateRange.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DateRange;
                        })();
    
                        common.YearMonthRange = (function() {
    
                            /**
                             * Properties of a YearMonthRange.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IYearMonthRange
                             * @property {google.ads.googleads.v11.common.IYearMonth|null} [start] YearMonthRange start
                             * @property {google.ads.googleads.v11.common.IYearMonth|null} [end] YearMonthRange end
                             */
    
                            /**
                             * Constructs a new YearMonthRange.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a YearMonthRange.
                             * @implements IYearMonthRange
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IYearMonthRange=} [p] Properties to set
                             */
                            function YearMonthRange(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * YearMonthRange start.
                             * @member {google.ads.googleads.v11.common.IYearMonth|null|undefined} start
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @instance
                             */
                            YearMonthRange.prototype.start = null;
    
                            /**
                             * YearMonthRange end.
                             * @member {google.ads.googleads.v11.common.IYearMonth|null|undefined} end
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @instance
                             */
                            YearMonthRange.prototype.end = null;
    
                            /**
                             * Creates a new YearMonthRange instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {google.ads.googleads.v11.common.IYearMonthRange=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.YearMonthRange} YearMonthRange instance
                             */
                            YearMonthRange.create = function create(properties) {
                                return new YearMonthRange(properties);
                            };
    
                            /**
                             * Encodes the specified YearMonthRange message. Does not implicitly {@link google.ads.googleads.v11.common.YearMonthRange.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {google.ads.googleads.v11.common.IYearMonthRange} m YearMonthRange message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YearMonthRange.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.start != null && Object.hasOwnProperty.call(m, "start"))
                                    $root.google.ads.googleads.v11.common.YearMonth.encode(m.start, w.uint32(10).fork()).ldelim();
                                if (m.end != null && Object.hasOwnProperty.call(m, "end"))
                                    $root.google.ads.googleads.v11.common.YearMonth.encode(m.end, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified YearMonthRange message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.YearMonthRange.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {google.ads.googleads.v11.common.IYearMonthRange} message YearMonthRange message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YearMonthRange.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a YearMonthRange message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.YearMonthRange} YearMonthRange
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YearMonthRange.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.YearMonthRange();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.start = $root.google.ads.googleads.v11.common.YearMonth.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.end = $root.google.ads.googleads.v11.common.YearMonth.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a YearMonthRange message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.YearMonthRange} YearMonthRange
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YearMonthRange.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a YearMonthRange message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            YearMonthRange.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.start != null && m.hasOwnProperty("start")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YearMonth.verify(m.start);
                                        if (e)
                                            return "start." + e;
                                    }
                                }
                                if (m.end != null && m.hasOwnProperty("end")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YearMonth.verify(m.end);
                                        if (e)
                                            return "end." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a YearMonthRange message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.YearMonthRange} YearMonthRange
                             */
                            YearMonthRange.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.YearMonthRange)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.YearMonthRange();
                                if (d.start != null) {
                                    if (typeof d.start !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.YearMonthRange.start: object expected");
                                    m.start = $root.google.ads.googleads.v11.common.YearMonth.fromObject(d.start);
                                }
                                if (d.end != null) {
                                    if (typeof d.end !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.YearMonthRange.end: object expected");
                                    m.end = $root.google.ads.googleads.v11.common.YearMonth.fromObject(d.end);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a YearMonthRange message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @static
                             * @param {google.ads.googleads.v11.common.YearMonthRange} m YearMonthRange
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            YearMonthRange.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.start = null;
                                    d.end = null;
                                }
                                if (m.start != null && m.hasOwnProperty("start")) {
                                    d.start = $root.google.ads.googleads.v11.common.YearMonth.toObject(m.start, o);
                                }
                                if (m.end != null && m.hasOwnProperty("end")) {
                                    d.end = $root.google.ads.googleads.v11.common.YearMonth.toObject(m.end, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this YearMonthRange to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.YearMonthRange
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            YearMonthRange.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return YearMonthRange;
                        })();
    
                        common.YearMonth = (function() {
    
                            /**
                             * Properties of a YearMonth.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IYearMonth
                             * @property {number|null} [year] YearMonth year
                             * @property {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear|null} [month] YearMonth month
                             */
    
                            /**
                             * Constructs a new YearMonth.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a YearMonth.
                             * @implements IYearMonth
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IYearMonth=} [p] Properties to set
                             */
                            function YearMonth(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * YearMonth year.
                             * @member {number} year
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @instance
                             */
                            YearMonth.prototype.year = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * YearMonth month.
                             * @member {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear} month
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @instance
                             */
                            YearMonth.prototype.month = 0;
    
                            /**
                             * Creates a new YearMonth instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {google.ads.googleads.v11.common.IYearMonth=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.YearMonth} YearMonth instance
                             */
                            YearMonth.create = function create(properties) {
                                return new YearMonth(properties);
                            };
    
                            /**
                             * Encodes the specified YearMonth message. Does not implicitly {@link google.ads.googleads.v11.common.YearMonth.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {google.ads.googleads.v11.common.IYearMonth} m YearMonth message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YearMonth.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.year != null && Object.hasOwnProperty.call(m, "year"))
                                    w.uint32(8).int64(m.year);
                                if (m.month != null && Object.hasOwnProperty.call(m, "month"))
                                    w.uint32(16).int32(m.month);
                                return w;
                            };
    
                            /**
                             * Encodes the specified YearMonth message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.YearMonth.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {google.ads.googleads.v11.common.IYearMonth} message YearMonth message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YearMonth.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a YearMonth message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.YearMonth} YearMonth
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YearMonth.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.YearMonth();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.year = r.int64();
                                        break;
                                    case 2:
                                        m.month = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a YearMonth message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.YearMonth} YearMonth
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YearMonth.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a YearMonth message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            YearMonth.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.year != null && m.hasOwnProperty("year")) {
                                    if (!$util.isInteger(m.year) && !(m.year && $util.isInteger(m.year.low) && $util.isInteger(m.year.high)))
                                        return "year: integer|Long expected";
                                }
                                if (m.month != null && m.hasOwnProperty("month")) {
                                    switch (m.month) {
                                    default:
                                        return "month: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a YearMonth message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.YearMonth} YearMonth
                             */
                            YearMonth.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.YearMonth)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.YearMonth();
                                if (d.year != null) {
                                    if ($util.Long)
                                        (m.year = $util.Long.fromValue(d.year)).unsigned = false;
                                    else if (typeof d.year === "string")
                                        m.year = parseInt(d.year, 10);
                                    else if (typeof d.year === "number")
                                        m.year = d.year;
                                    else if (typeof d.year === "object")
                                        m.year = new $util.LongBits(d.year.low >>> 0, d.year.high >>> 0).toNumber();
                                }
                                switch (d.month) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.month = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.month = 1;
                                    break;
                                case "JANUARY":
                                case 2:
                                    m.month = 2;
                                    break;
                                case "FEBRUARY":
                                case 3:
                                    m.month = 3;
                                    break;
                                case "MARCH":
                                case 4:
                                    m.month = 4;
                                    break;
                                case "APRIL":
                                case 5:
                                    m.month = 5;
                                    break;
                                case "MAY":
                                case 6:
                                    m.month = 6;
                                    break;
                                case "JUNE":
                                case 7:
                                    m.month = 7;
                                    break;
                                case "JULY":
                                case 8:
                                    m.month = 8;
                                    break;
                                case "AUGUST":
                                case 9:
                                    m.month = 9;
                                    break;
                                case "SEPTEMBER":
                                case 10:
                                    m.month = 10;
                                    break;
                                case "OCTOBER":
                                case 11:
                                    m.month = 11;
                                    break;
                                case "NOVEMBER":
                                case 12:
                                    m.month = 12;
                                    break;
                                case "DECEMBER":
                                case 13:
                                    m.month = 13;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a YearMonth message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @static
                             * @param {google.ads.googleads.v11.common.YearMonth} m YearMonth
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            YearMonth.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.year = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.year = o.longs === String ? "0" : 0;
                                    d.month = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.year != null && m.hasOwnProperty("year")) {
                                    if (typeof m.year === "number")
                                        d.year = o.longs === String ? String(m.year) : m.year;
                                    else
                                        d.year = o.longs === String ? $util.Long.prototype.toString.call(m.year) : o.longs === Number ? new $util.LongBits(m.year.low >>> 0, m.year.high >>> 0).toNumber() : m.year;
                                }
                                if (m.month != null && m.hasOwnProperty("month")) {
                                    d.month = o.enums === String ? $root.google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear[m.month] : m.month;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this YearMonth to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.YearMonth
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            YearMonth.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return YearMonth;
                        })();
    
                        common.ExplorerAutoOptimizerSetting = (function() {
    
                            /**
                             * Properties of an ExplorerAutoOptimizerSetting.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IExplorerAutoOptimizerSetting
                             * @property {boolean|null} [opt_in] ExplorerAutoOptimizerSetting opt_in
                             */
    
                            /**
                             * Constructs a new ExplorerAutoOptimizerSetting.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ExplorerAutoOptimizerSetting.
                             * @implements IExplorerAutoOptimizerSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IExplorerAutoOptimizerSetting=} [p] Properties to set
                             */
                            function ExplorerAutoOptimizerSetting(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExplorerAutoOptimizerSetting opt_in.
                             * @member {boolean|null|undefined} opt_in
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @instance
                             */
                            ExplorerAutoOptimizerSetting.prototype.opt_in = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExplorerAutoOptimizerSetting _opt_in.
                             * @member {"opt_in"|undefined} _opt_in
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @instance
                             */
                            Object.defineProperty(ExplorerAutoOptimizerSetting.prototype, "_opt_in", {
                                get: $util.oneOfGetter($oneOfFields = ["opt_in"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExplorerAutoOptimizerSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.IExplorerAutoOptimizerSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting} ExplorerAutoOptimizerSetting instance
                             */
                            ExplorerAutoOptimizerSetting.create = function create(properties) {
                                return new ExplorerAutoOptimizerSetting(properties);
                            };
    
                            /**
                             * Encodes the specified ExplorerAutoOptimizerSetting message. Does not implicitly {@link google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.IExplorerAutoOptimizerSetting} m ExplorerAutoOptimizerSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExplorerAutoOptimizerSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.opt_in != null && Object.hasOwnProperty.call(m, "opt_in"))
                                    w.uint32(16).bool(m.opt_in);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExplorerAutoOptimizerSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.IExplorerAutoOptimizerSetting} message ExplorerAutoOptimizerSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExplorerAutoOptimizerSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExplorerAutoOptimizerSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting} ExplorerAutoOptimizerSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExplorerAutoOptimizerSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.opt_in = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExplorerAutoOptimizerSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting} ExplorerAutoOptimizerSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExplorerAutoOptimizerSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExplorerAutoOptimizerSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExplorerAutoOptimizerSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.opt_in != null && m.hasOwnProperty("opt_in")) {
                                    p._opt_in = 1;
                                    if (typeof m.opt_in !== "boolean")
                                        return "opt_in: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExplorerAutoOptimizerSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting} ExplorerAutoOptimizerSetting
                             */
                            ExplorerAutoOptimizerSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting();
                                if (d.opt_in != null) {
                                    m.opt_in = Boolean(d.opt_in);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExplorerAutoOptimizerSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting} m ExplorerAutoOptimizerSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExplorerAutoOptimizerSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.opt_in != null && m.hasOwnProperty("opt_in")) {
                                    d.opt_in = m.opt_in;
                                    if (o.oneofs)
                                        d._opt_in = "opt_in";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExplorerAutoOptimizerSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExplorerAutoOptimizerSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExplorerAutoOptimizerSetting;
                        })();
    
                        common.AppFeedItem = (function() {
    
                            /**
                             * Properties of an AppFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAppFeedItem
                             * @property {string|null} [link_text] AppFeedItem link_text
                             * @property {string|null} [app_id] AppFeedItem app_id
                             * @property {google.ads.googleads.v11.enums.AppStoreEnum.AppStore|null} [app_store] AppFeedItem app_store
                             * @property {Array.<string>|null} [final_urls] AppFeedItem final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] AppFeedItem final_mobile_urls
                             * @property {string|null} [tracking_url_template] AppFeedItem tracking_url_template
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] AppFeedItem url_custom_parameters
                             * @property {string|null} [final_url_suffix] AppFeedItem final_url_suffix
                             */
    
                            /**
                             * Constructs a new AppFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AppFeedItem.
                             * @implements IAppFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAppFeedItem=} [p] Properties to set
                             */
                            function AppFeedItem(p) {
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                this.url_custom_parameters = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AppFeedItem link_text.
                             * @member {string|null|undefined} link_text
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.link_text = null;
    
                            /**
                             * AppFeedItem app_id.
                             * @member {string|null|undefined} app_id
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.app_id = null;
    
                            /**
                             * AppFeedItem app_store.
                             * @member {google.ads.googleads.v11.enums.AppStoreEnum.AppStore} app_store
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.app_store = 0;
    
                            /**
                             * AppFeedItem final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * AppFeedItem final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * AppFeedItem tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.tracking_url_template = null;
    
                            /**
                             * AppFeedItem url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * AppFeedItem final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            AppFeedItem.prototype.final_url_suffix = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AppFeedItem _link_text.
                             * @member {"link_text"|undefined} _link_text
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            Object.defineProperty(AppFeedItem.prototype, "_link_text", {
                                get: $util.oneOfGetter($oneOfFields = ["link_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AppFeedItem _app_id.
                             * @member {"app_id"|undefined} _app_id
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            Object.defineProperty(AppFeedItem.prototype, "_app_id", {
                                get: $util.oneOfGetter($oneOfFields = ["app_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AppFeedItem _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            Object.defineProperty(AppFeedItem.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AppFeedItem _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             */
                            Object.defineProperty(AppFeedItem.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AppFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AppFeedItem} AppFeedItem instance
                             */
                            AppFeedItem.create = function create(properties) {
                                return new AppFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified AppFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.AppFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppFeedItem} m AppFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.app_store != null && Object.hasOwnProperty.call(m, "app_store"))
                                    w.uint32(24).int32(m.app_store);
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.link_text != null && Object.hasOwnProperty.call(m, "link_text"))
                                    w.uint32(74).string(m.link_text);
                                if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                    w.uint32(82).string(m.app_id);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(90).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(98).string(m.final_mobile_urls[i]);
                                }
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(106).string(m.tracking_url_template);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(114).string(m.final_url_suffix);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AppFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IAppFeedItem} message AppFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AppFeedItem} AppFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AppFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 9:
                                        m.link_text = r.string();
                                        break;
                                    case 10:
                                        m.app_id = r.string();
                                        break;
                                    case 3:
                                        m.app_store = r.int32();
                                        break;
                                    case 11:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 12:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 13:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 7:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 14:
                                        m.final_url_suffix = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AppFeedItem} AppFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    p._link_text = 1;
                                    if (!$util.isString(m.link_text))
                                        return "link_text: string expected";
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    p._app_id = 1;
                                    if (!$util.isString(m.app_id))
                                        return "app_id: string expected";
                                }
                                if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                    switch (m.app_store) {
                                    default:
                                        return "app_store: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AppFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AppFeedItem} AppFeedItem
                             */
                            AppFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AppFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AppFeedItem();
                                if (d.link_text != null) {
                                    m.link_text = String(d.link_text);
                                }
                                if (d.app_id != null) {
                                    m.app_id = String(d.app_id);
                                }
                                switch (d.app_store) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.app_store = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.app_store = 1;
                                    break;
                                case "APPLE_ITUNES":
                                case 2:
                                    m.app_store = 2;
                                    break;
                                case "GOOGLE_PLAY":
                                case 3:
                                    m.app_store = 3;
                                    break;
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.AppFeedItem.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.AppFeedItem.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.common.AppFeedItem.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.AppFeedItem.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AppFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.AppFeedItem} m AppFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.url_custom_parameters = [];
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (o.defaults) {
                                    d.app_store = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                    d.app_store = o.enums === String ? $root.google.ads.googleads.v11.enums.AppStoreEnum.AppStore[m.app_store] : m.app_store;
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    d.link_text = m.link_text;
                                    if (o.oneofs)
                                        d._link_text = "link_text";
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    d.app_id = m.app_id;
                                    if (o.oneofs)
                                        d._app_id = "app_id";
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AppFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AppFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AppFeedItem;
                        })();
    
                        common.CallFeedItem = (function() {
    
                            /**
                             * Properties of a CallFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICallFeedItem
                             * @property {string|null} [phone_number] CallFeedItem phone_number
                             * @property {string|null} [country_code] CallFeedItem country_code
                             * @property {boolean|null} [call_tracking_enabled] CallFeedItem call_tracking_enabled
                             * @property {string|null} [call_conversion_action] CallFeedItem call_conversion_action
                             * @property {boolean|null} [call_conversion_tracking_disabled] CallFeedItem call_conversion_tracking_disabled
                             * @property {google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState|null} [call_conversion_reporting_state] CallFeedItem call_conversion_reporting_state
                             */
    
                            /**
                             * Constructs a new CallFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CallFeedItem.
                             * @implements ICallFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICallFeedItem=} [p] Properties to set
                             */
                            function CallFeedItem(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallFeedItem phone_number.
                             * @member {string|null|undefined} phone_number
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            CallFeedItem.prototype.phone_number = null;
    
                            /**
                             * CallFeedItem country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            CallFeedItem.prototype.country_code = null;
    
                            /**
                             * CallFeedItem call_tracking_enabled.
                             * @member {boolean|null|undefined} call_tracking_enabled
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            CallFeedItem.prototype.call_tracking_enabled = null;
    
                            /**
                             * CallFeedItem call_conversion_action.
                             * @member {string|null|undefined} call_conversion_action
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            CallFeedItem.prototype.call_conversion_action = null;
    
                            /**
                             * CallFeedItem call_conversion_tracking_disabled.
                             * @member {boolean|null|undefined} call_conversion_tracking_disabled
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            CallFeedItem.prototype.call_conversion_tracking_disabled = null;
    
                            /**
                             * CallFeedItem call_conversion_reporting_state.
                             * @member {google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState} call_conversion_reporting_state
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            CallFeedItem.prototype.call_conversion_reporting_state = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CallFeedItem _phone_number.
                             * @member {"phone_number"|undefined} _phone_number
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            Object.defineProperty(CallFeedItem.prototype, "_phone_number", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_number"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallFeedItem _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            Object.defineProperty(CallFeedItem.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallFeedItem _call_tracking_enabled.
                             * @member {"call_tracking_enabled"|undefined} _call_tracking_enabled
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            Object.defineProperty(CallFeedItem.prototype, "_call_tracking_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["call_tracking_enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallFeedItem _call_conversion_action.
                             * @member {"call_conversion_action"|undefined} _call_conversion_action
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            Object.defineProperty(CallFeedItem.prototype, "_call_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["call_conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallFeedItem _call_conversion_tracking_disabled.
                             * @member {"call_conversion_tracking_disabled"|undefined} _call_conversion_tracking_disabled
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             */
                            Object.defineProperty(CallFeedItem.prototype, "_call_conversion_tracking_disabled", {
                                get: $util.oneOfGetter($oneOfFields = ["call_conversion_tracking_disabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CallFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CallFeedItem} CallFeedItem instance
                             */
                            CallFeedItem.create = function create(properties) {
                                return new CallFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified CallFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.CallFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallFeedItem} m CallFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.call_conversion_reporting_state != null && Object.hasOwnProperty.call(m, "call_conversion_reporting_state"))
                                    w.uint32(48).int32(m.call_conversion_reporting_state);
                                if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                    w.uint32(58).string(m.phone_number);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(66).string(m.country_code);
                                if (m.call_tracking_enabled != null && Object.hasOwnProperty.call(m, "call_tracking_enabled"))
                                    w.uint32(72).bool(m.call_tracking_enabled);
                                if (m.call_conversion_action != null && Object.hasOwnProperty.call(m, "call_conversion_action"))
                                    w.uint32(82).string(m.call_conversion_action);
                                if (m.call_conversion_tracking_disabled != null && Object.hasOwnProperty.call(m, "call_conversion_tracking_disabled"))
                                    w.uint32(88).bool(m.call_conversion_tracking_disabled);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CallFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ICallFeedItem} message CallFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CallFeedItem} CallFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CallFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 7:
                                        m.phone_number = r.string();
                                        break;
                                    case 8:
                                        m.country_code = r.string();
                                        break;
                                    case 9:
                                        m.call_tracking_enabled = r.bool();
                                        break;
                                    case 10:
                                        m.call_conversion_action = r.string();
                                        break;
                                    case 11:
                                        m.call_conversion_tracking_disabled = r.bool();
                                        break;
                                    case 6:
                                        m.call_conversion_reporting_state = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CallFeedItem} CallFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    p._phone_number = 1;
                                    if (!$util.isString(m.phone_number))
                                        return "phone_number: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.call_tracking_enabled != null && m.hasOwnProperty("call_tracking_enabled")) {
                                    p._call_tracking_enabled = 1;
                                    if (typeof m.call_tracking_enabled !== "boolean")
                                        return "call_tracking_enabled: boolean expected";
                                }
                                if (m.call_conversion_action != null && m.hasOwnProperty("call_conversion_action")) {
                                    p._call_conversion_action = 1;
                                    if (!$util.isString(m.call_conversion_action))
                                        return "call_conversion_action: string expected";
                                }
                                if (m.call_conversion_tracking_disabled != null && m.hasOwnProperty("call_conversion_tracking_disabled")) {
                                    p._call_conversion_tracking_disabled = 1;
                                    if (typeof m.call_conversion_tracking_disabled !== "boolean")
                                        return "call_conversion_tracking_disabled: boolean expected";
                                }
                                if (m.call_conversion_reporting_state != null && m.hasOwnProperty("call_conversion_reporting_state")) {
                                    switch (m.call_conversion_reporting_state) {
                                    default:
                                        return "call_conversion_reporting_state: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CallFeedItem} CallFeedItem
                             */
                            CallFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CallFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CallFeedItem();
                                if (d.phone_number != null) {
                                    m.phone_number = String(d.phone_number);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.call_tracking_enabled != null) {
                                    m.call_tracking_enabled = Boolean(d.call_tracking_enabled);
                                }
                                if (d.call_conversion_action != null) {
                                    m.call_conversion_action = String(d.call_conversion_action);
                                }
                                if (d.call_conversion_tracking_disabled != null) {
                                    m.call_conversion_tracking_disabled = Boolean(d.call_conversion_tracking_disabled);
                                }
                                switch (d.call_conversion_reporting_state) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.call_conversion_reporting_state = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.call_conversion_reporting_state = 1;
                                    break;
                                case "DISABLED":
                                case 2:
                                    m.call_conversion_reporting_state = 2;
                                    break;
                                case "USE_ACCOUNT_LEVEL_CALL_CONVERSION_ACTION":
                                case 3:
                                    m.call_conversion_reporting_state = 3;
                                    break;
                                case "USE_RESOURCE_LEVEL_CALL_CONVERSION_ACTION":
                                case 4:
                                    m.call_conversion_reporting_state = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.CallFeedItem} m CallFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.call_conversion_reporting_state = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.call_conversion_reporting_state != null && m.hasOwnProperty("call_conversion_reporting_state")) {
                                    d.call_conversion_reporting_state = o.enums === String ? $root.google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState[m.call_conversion_reporting_state] : m.call_conversion_reporting_state;
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    d.phone_number = m.phone_number;
                                    if (o.oneofs)
                                        d._phone_number = "phone_number";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.call_tracking_enabled != null && m.hasOwnProperty("call_tracking_enabled")) {
                                    d.call_tracking_enabled = m.call_tracking_enabled;
                                    if (o.oneofs)
                                        d._call_tracking_enabled = "call_tracking_enabled";
                                }
                                if (m.call_conversion_action != null && m.hasOwnProperty("call_conversion_action")) {
                                    d.call_conversion_action = m.call_conversion_action;
                                    if (o.oneofs)
                                        d._call_conversion_action = "call_conversion_action";
                                }
                                if (m.call_conversion_tracking_disabled != null && m.hasOwnProperty("call_conversion_tracking_disabled")) {
                                    d.call_conversion_tracking_disabled = m.call_conversion_tracking_disabled;
                                    if (o.oneofs)
                                        d._call_conversion_tracking_disabled = "call_conversion_tracking_disabled";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CallFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallFeedItem;
                        })();
    
                        common.CalloutFeedItem = (function() {
    
                            /**
                             * Properties of a CalloutFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICalloutFeedItem
                             * @property {string|null} [callout_text] CalloutFeedItem callout_text
                             */
    
                            /**
                             * Constructs a new CalloutFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CalloutFeedItem.
                             * @implements ICalloutFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICalloutFeedItem=} [p] Properties to set
                             */
                            function CalloutFeedItem(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CalloutFeedItem callout_text.
                             * @member {string|null|undefined} callout_text
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @instance
                             */
                            CalloutFeedItem.prototype.callout_text = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CalloutFeedItem _callout_text.
                             * @member {"callout_text"|undefined} _callout_text
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @instance
                             */
                            Object.defineProperty(CalloutFeedItem.prototype, "_callout_text", {
                                get: $util.oneOfGetter($oneOfFields = ["callout_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CalloutFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ICalloutFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CalloutFeedItem} CalloutFeedItem instance
                             */
                            CalloutFeedItem.create = function create(properties) {
                                return new CalloutFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified CalloutFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.CalloutFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ICalloutFeedItem} m CalloutFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CalloutFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.callout_text != null && Object.hasOwnProperty.call(m, "callout_text"))
                                    w.uint32(18).string(m.callout_text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CalloutFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CalloutFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ICalloutFeedItem} message CalloutFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CalloutFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CalloutFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CalloutFeedItem} CalloutFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CalloutFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CalloutFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.callout_text = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CalloutFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CalloutFeedItem} CalloutFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CalloutFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CalloutFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CalloutFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.callout_text != null && m.hasOwnProperty("callout_text")) {
                                    p._callout_text = 1;
                                    if (!$util.isString(m.callout_text))
                                        return "callout_text: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CalloutFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CalloutFeedItem} CalloutFeedItem
                             */
                            CalloutFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CalloutFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CalloutFeedItem();
                                if (d.callout_text != null) {
                                    m.callout_text = String(d.callout_text);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CalloutFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.CalloutFeedItem} m CalloutFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CalloutFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.callout_text != null && m.hasOwnProperty("callout_text")) {
                                    d.callout_text = m.callout_text;
                                    if (o.oneofs)
                                        d._callout_text = "callout_text";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CalloutFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CalloutFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CalloutFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CalloutFeedItem;
                        })();
    
                        common.LocationFeedItem = (function() {
    
                            /**
                             * Properties of a LocationFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILocationFeedItem
                             * @property {string|null} [business_name] LocationFeedItem business_name
                             * @property {string|null} [address_line_1] LocationFeedItem address_line_1
                             * @property {string|null} [address_line_2] LocationFeedItem address_line_2
                             * @property {string|null} [city] LocationFeedItem city
                             * @property {string|null} [province] LocationFeedItem province
                             * @property {string|null} [postal_code] LocationFeedItem postal_code
                             * @property {string|null} [country_code] LocationFeedItem country_code
                             * @property {string|null} [phone_number] LocationFeedItem phone_number
                             */
    
                            /**
                             * Constructs a new LocationFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LocationFeedItem.
                             * @implements ILocationFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILocationFeedItem=} [p] Properties to set
                             */
                            function LocationFeedItem(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LocationFeedItem business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.business_name = null;
    
                            /**
                             * LocationFeedItem address_line_1.
                             * @member {string|null|undefined} address_line_1
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.address_line_1 = null;
    
                            /**
                             * LocationFeedItem address_line_2.
                             * @member {string|null|undefined} address_line_2
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.address_line_2 = null;
    
                            /**
                             * LocationFeedItem city.
                             * @member {string|null|undefined} city
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.city = null;
    
                            /**
                             * LocationFeedItem province.
                             * @member {string|null|undefined} province
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.province = null;
    
                            /**
                             * LocationFeedItem postal_code.
                             * @member {string|null|undefined} postal_code
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.postal_code = null;
    
                            /**
                             * LocationFeedItem country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.country_code = null;
    
                            /**
                             * LocationFeedItem phone_number.
                             * @member {string|null|undefined} phone_number
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            LocationFeedItem.prototype.phone_number = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LocationFeedItem _business_name.
                             * @member {"business_name"|undefined} _business_name
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_business_name", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationFeedItem _address_line_1.
                             * @member {"address_line_1"|undefined} _address_line_1
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_address_line_1", {
                                get: $util.oneOfGetter($oneOfFields = ["address_line_1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationFeedItem _address_line_2.
                             * @member {"address_line_2"|undefined} _address_line_2
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_address_line_2", {
                                get: $util.oneOfGetter($oneOfFields = ["address_line_2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationFeedItem _city.
                             * @member {"city"|undefined} _city
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_city", {
                                get: $util.oneOfGetter($oneOfFields = ["city"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationFeedItem _province.
                             * @member {"province"|undefined} _province
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_province", {
                                get: $util.oneOfGetter($oneOfFields = ["province"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationFeedItem _postal_code.
                             * @member {"postal_code"|undefined} _postal_code
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_postal_code", {
                                get: $util.oneOfGetter($oneOfFields = ["postal_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationFeedItem _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LocationFeedItem _phone_number.
                             * @member {"phone_number"|undefined} _phone_number
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(LocationFeedItem.prototype, "_phone_number", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_number"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LocationFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LocationFeedItem} LocationFeedItem instance
                             */
                            LocationFeedItem.create = function create(properties) {
                                return new LocationFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified LocationFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.LocationFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationFeedItem} m LocationFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(74).string(m.business_name);
                                if (m.address_line_1 != null && Object.hasOwnProperty.call(m, "address_line_1"))
                                    w.uint32(82).string(m.address_line_1);
                                if (m.address_line_2 != null && Object.hasOwnProperty.call(m, "address_line_2"))
                                    w.uint32(90).string(m.address_line_2);
                                if (m.city != null && Object.hasOwnProperty.call(m, "city"))
                                    w.uint32(98).string(m.city);
                                if (m.province != null && Object.hasOwnProperty.call(m, "province"))
                                    w.uint32(106).string(m.province);
                                if (m.postal_code != null && Object.hasOwnProperty.call(m, "postal_code"))
                                    w.uint32(114).string(m.postal_code);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(122).string(m.country_code);
                                if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                    w.uint32(130).string(m.phone_number);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LocationFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ILocationFeedItem} message LocationFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LocationFeedItem} LocationFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LocationFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 9:
                                        m.business_name = r.string();
                                        break;
                                    case 10:
                                        m.address_line_1 = r.string();
                                        break;
                                    case 11:
                                        m.address_line_2 = r.string();
                                        break;
                                    case 12:
                                        m.city = r.string();
                                        break;
                                    case 13:
                                        m.province = r.string();
                                        break;
                                    case 14:
                                        m.postal_code = r.string();
                                        break;
                                    case 15:
                                        m.country_code = r.string();
                                        break;
                                    case 16:
                                        m.phone_number = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LocationFeedItem} LocationFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p._business_name = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.address_line_1 != null && m.hasOwnProperty("address_line_1")) {
                                    p._address_line_1 = 1;
                                    if (!$util.isString(m.address_line_1))
                                        return "address_line_1: string expected";
                                }
                                if (m.address_line_2 != null && m.hasOwnProperty("address_line_2")) {
                                    p._address_line_2 = 1;
                                    if (!$util.isString(m.address_line_2))
                                        return "address_line_2: string expected";
                                }
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    p._city = 1;
                                    if (!$util.isString(m.city))
                                        return "city: string expected";
                                }
                                if (m.province != null && m.hasOwnProperty("province")) {
                                    p._province = 1;
                                    if (!$util.isString(m.province))
                                        return "province: string expected";
                                }
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    p._postal_code = 1;
                                    if (!$util.isString(m.postal_code))
                                        return "postal_code: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    p._phone_number = 1;
                                    if (!$util.isString(m.phone_number))
                                        return "phone_number: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LocationFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LocationFeedItem} LocationFeedItem
                             */
                            LocationFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LocationFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LocationFeedItem();
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.address_line_1 != null) {
                                    m.address_line_1 = String(d.address_line_1);
                                }
                                if (d.address_line_2 != null) {
                                    m.address_line_2 = String(d.address_line_2);
                                }
                                if (d.city != null) {
                                    m.city = String(d.city);
                                }
                                if (d.province != null) {
                                    m.province = String(d.province);
                                }
                                if (d.postal_code != null) {
                                    m.postal_code = String(d.postal_code);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.phone_number != null) {
                                    m.phone_number = String(d.phone_number);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LocationFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.LocationFeedItem} m LocationFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d._business_name = "business_name";
                                }
                                if (m.address_line_1 != null && m.hasOwnProperty("address_line_1")) {
                                    d.address_line_1 = m.address_line_1;
                                    if (o.oneofs)
                                        d._address_line_1 = "address_line_1";
                                }
                                if (m.address_line_2 != null && m.hasOwnProperty("address_line_2")) {
                                    d.address_line_2 = m.address_line_2;
                                    if (o.oneofs)
                                        d._address_line_2 = "address_line_2";
                                }
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    d.city = m.city;
                                    if (o.oneofs)
                                        d._city = "city";
                                }
                                if (m.province != null && m.hasOwnProperty("province")) {
                                    d.province = m.province;
                                    if (o.oneofs)
                                        d._province = "province";
                                }
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    d.postal_code = m.postal_code;
                                    if (o.oneofs)
                                        d._postal_code = "postal_code";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    d.phone_number = m.phone_number;
                                    if (o.oneofs)
                                        d._phone_number = "phone_number";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LocationFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LocationFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LocationFeedItem;
                        })();
    
                        common.AffiliateLocationFeedItem = (function() {
    
                            /**
                             * Properties of an AffiliateLocationFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAffiliateLocationFeedItem
                             * @property {string|null} [business_name] AffiliateLocationFeedItem business_name
                             * @property {string|null} [address_line_1] AffiliateLocationFeedItem address_line_1
                             * @property {string|null} [address_line_2] AffiliateLocationFeedItem address_line_2
                             * @property {string|null} [city] AffiliateLocationFeedItem city
                             * @property {string|null} [province] AffiliateLocationFeedItem province
                             * @property {string|null} [postal_code] AffiliateLocationFeedItem postal_code
                             * @property {string|null} [country_code] AffiliateLocationFeedItem country_code
                             * @property {string|null} [phone_number] AffiliateLocationFeedItem phone_number
                             * @property {number|null} [chain_id] AffiliateLocationFeedItem chain_id
                             * @property {string|null} [chain_name] AffiliateLocationFeedItem chain_name
                             */
    
                            /**
                             * Constructs a new AffiliateLocationFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AffiliateLocationFeedItem.
                             * @implements IAffiliateLocationFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAffiliateLocationFeedItem=} [p] Properties to set
                             */
                            function AffiliateLocationFeedItem(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AffiliateLocationFeedItem business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.business_name = null;
    
                            /**
                             * AffiliateLocationFeedItem address_line_1.
                             * @member {string|null|undefined} address_line_1
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.address_line_1 = null;
    
                            /**
                             * AffiliateLocationFeedItem address_line_2.
                             * @member {string|null|undefined} address_line_2
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.address_line_2 = null;
    
                            /**
                             * AffiliateLocationFeedItem city.
                             * @member {string|null|undefined} city
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.city = null;
    
                            /**
                             * AffiliateLocationFeedItem province.
                             * @member {string|null|undefined} province
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.province = null;
    
                            /**
                             * AffiliateLocationFeedItem postal_code.
                             * @member {string|null|undefined} postal_code
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.postal_code = null;
    
                            /**
                             * AffiliateLocationFeedItem country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.country_code = null;
    
                            /**
                             * AffiliateLocationFeedItem phone_number.
                             * @member {string|null|undefined} phone_number
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.phone_number = null;
    
                            /**
                             * AffiliateLocationFeedItem chain_id.
                             * @member {number|null|undefined} chain_id
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.chain_id = null;
    
                            /**
                             * AffiliateLocationFeedItem chain_name.
                             * @member {string|null|undefined} chain_name
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            AffiliateLocationFeedItem.prototype.chain_name = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AffiliateLocationFeedItem _business_name.
                             * @member {"business_name"|undefined} _business_name
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_business_name", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _address_line_1.
                             * @member {"address_line_1"|undefined} _address_line_1
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_address_line_1", {
                                get: $util.oneOfGetter($oneOfFields = ["address_line_1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _address_line_2.
                             * @member {"address_line_2"|undefined} _address_line_2
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_address_line_2", {
                                get: $util.oneOfGetter($oneOfFields = ["address_line_2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _city.
                             * @member {"city"|undefined} _city
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_city", {
                                get: $util.oneOfGetter($oneOfFields = ["city"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _province.
                             * @member {"province"|undefined} _province
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_province", {
                                get: $util.oneOfGetter($oneOfFields = ["province"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _postal_code.
                             * @member {"postal_code"|undefined} _postal_code
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_postal_code", {
                                get: $util.oneOfGetter($oneOfFields = ["postal_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _phone_number.
                             * @member {"phone_number"|undefined} _phone_number
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_phone_number", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_number"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _chain_id.
                             * @member {"chain_id"|undefined} _chain_id
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_chain_id", {
                                get: $util.oneOfGetter($oneOfFields = ["chain_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AffiliateLocationFeedItem _chain_name.
                             * @member {"chain_name"|undefined} _chain_name
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             */
                            Object.defineProperty(AffiliateLocationFeedItem.prototype, "_chain_name", {
                                get: $util.oneOfGetter($oneOfFields = ["chain_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AffiliateLocationFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IAffiliateLocationFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AffiliateLocationFeedItem} AffiliateLocationFeedItem instance
                             */
                            AffiliateLocationFeedItem.create = function create(properties) {
                                return new AffiliateLocationFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified AffiliateLocationFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.AffiliateLocationFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IAffiliateLocationFeedItem} m AffiliateLocationFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AffiliateLocationFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(90).string(m.business_name);
                                if (m.address_line_1 != null && Object.hasOwnProperty.call(m, "address_line_1"))
                                    w.uint32(98).string(m.address_line_1);
                                if (m.address_line_2 != null && Object.hasOwnProperty.call(m, "address_line_2"))
                                    w.uint32(106).string(m.address_line_2);
                                if (m.city != null && Object.hasOwnProperty.call(m, "city"))
                                    w.uint32(114).string(m.city);
                                if (m.province != null && Object.hasOwnProperty.call(m, "province"))
                                    w.uint32(122).string(m.province);
                                if (m.postal_code != null && Object.hasOwnProperty.call(m, "postal_code"))
                                    w.uint32(130).string(m.postal_code);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(138).string(m.country_code);
                                if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                    w.uint32(146).string(m.phone_number);
                                if (m.chain_id != null && Object.hasOwnProperty.call(m, "chain_id"))
                                    w.uint32(152).int64(m.chain_id);
                                if (m.chain_name != null && Object.hasOwnProperty.call(m, "chain_name"))
                                    w.uint32(162).string(m.chain_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AffiliateLocationFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AffiliateLocationFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IAffiliateLocationFeedItem} message AffiliateLocationFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AffiliateLocationFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AffiliateLocationFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AffiliateLocationFeedItem} AffiliateLocationFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AffiliateLocationFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 11:
                                        m.business_name = r.string();
                                        break;
                                    case 12:
                                        m.address_line_1 = r.string();
                                        break;
                                    case 13:
                                        m.address_line_2 = r.string();
                                        break;
                                    case 14:
                                        m.city = r.string();
                                        break;
                                    case 15:
                                        m.province = r.string();
                                        break;
                                    case 16:
                                        m.postal_code = r.string();
                                        break;
                                    case 17:
                                        m.country_code = r.string();
                                        break;
                                    case 18:
                                        m.phone_number = r.string();
                                        break;
                                    case 19:
                                        m.chain_id = r.int64();
                                        break;
                                    case 20:
                                        m.chain_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AffiliateLocationFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AffiliateLocationFeedItem} AffiliateLocationFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AffiliateLocationFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AffiliateLocationFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AffiliateLocationFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p._business_name = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.address_line_1 != null && m.hasOwnProperty("address_line_1")) {
                                    p._address_line_1 = 1;
                                    if (!$util.isString(m.address_line_1))
                                        return "address_line_1: string expected";
                                }
                                if (m.address_line_2 != null && m.hasOwnProperty("address_line_2")) {
                                    p._address_line_2 = 1;
                                    if (!$util.isString(m.address_line_2))
                                        return "address_line_2: string expected";
                                }
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    p._city = 1;
                                    if (!$util.isString(m.city))
                                        return "city: string expected";
                                }
                                if (m.province != null && m.hasOwnProperty("province")) {
                                    p._province = 1;
                                    if (!$util.isString(m.province))
                                        return "province: string expected";
                                }
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    p._postal_code = 1;
                                    if (!$util.isString(m.postal_code))
                                        return "postal_code: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    p._phone_number = 1;
                                    if (!$util.isString(m.phone_number))
                                        return "phone_number: string expected";
                                }
                                if (m.chain_id != null && m.hasOwnProperty("chain_id")) {
                                    p._chain_id = 1;
                                    if (!$util.isInteger(m.chain_id) && !(m.chain_id && $util.isInteger(m.chain_id.low) && $util.isInteger(m.chain_id.high)))
                                        return "chain_id: integer|Long expected";
                                }
                                if (m.chain_name != null && m.hasOwnProperty("chain_name")) {
                                    p._chain_name = 1;
                                    if (!$util.isString(m.chain_name))
                                        return "chain_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AffiliateLocationFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AffiliateLocationFeedItem} AffiliateLocationFeedItem
                             */
                            AffiliateLocationFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem();
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.address_line_1 != null) {
                                    m.address_line_1 = String(d.address_line_1);
                                }
                                if (d.address_line_2 != null) {
                                    m.address_line_2 = String(d.address_line_2);
                                }
                                if (d.city != null) {
                                    m.city = String(d.city);
                                }
                                if (d.province != null) {
                                    m.province = String(d.province);
                                }
                                if (d.postal_code != null) {
                                    m.postal_code = String(d.postal_code);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.phone_number != null) {
                                    m.phone_number = String(d.phone_number);
                                }
                                if (d.chain_id != null) {
                                    if ($util.Long)
                                        (m.chain_id = $util.Long.fromValue(d.chain_id)).unsigned = false;
                                    else if (typeof d.chain_id === "string")
                                        m.chain_id = parseInt(d.chain_id, 10);
                                    else if (typeof d.chain_id === "number")
                                        m.chain_id = d.chain_id;
                                    else if (typeof d.chain_id === "object")
                                        m.chain_id = new $util.LongBits(d.chain_id.low >>> 0, d.chain_id.high >>> 0).toNumber();
                                }
                                if (d.chain_name != null) {
                                    m.chain_name = String(d.chain_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AffiliateLocationFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.AffiliateLocationFeedItem} m AffiliateLocationFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AffiliateLocationFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d._business_name = "business_name";
                                }
                                if (m.address_line_1 != null && m.hasOwnProperty("address_line_1")) {
                                    d.address_line_1 = m.address_line_1;
                                    if (o.oneofs)
                                        d._address_line_1 = "address_line_1";
                                }
                                if (m.address_line_2 != null && m.hasOwnProperty("address_line_2")) {
                                    d.address_line_2 = m.address_line_2;
                                    if (o.oneofs)
                                        d._address_line_2 = "address_line_2";
                                }
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    d.city = m.city;
                                    if (o.oneofs)
                                        d._city = "city";
                                }
                                if (m.province != null && m.hasOwnProperty("province")) {
                                    d.province = m.province;
                                    if (o.oneofs)
                                        d._province = "province";
                                }
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    d.postal_code = m.postal_code;
                                    if (o.oneofs)
                                        d._postal_code = "postal_code";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    d.phone_number = m.phone_number;
                                    if (o.oneofs)
                                        d._phone_number = "phone_number";
                                }
                                if (m.chain_id != null && m.hasOwnProperty("chain_id")) {
                                    if (typeof m.chain_id === "number")
                                        d.chain_id = o.longs === String ? String(m.chain_id) : m.chain_id;
                                    else
                                        d.chain_id = o.longs === String ? $util.Long.prototype.toString.call(m.chain_id) : o.longs === Number ? new $util.LongBits(m.chain_id.low >>> 0, m.chain_id.high >>> 0).toNumber() : m.chain_id;
                                    if (o.oneofs)
                                        d._chain_id = "chain_id";
                                }
                                if (m.chain_name != null && m.hasOwnProperty("chain_name")) {
                                    d.chain_name = m.chain_name;
                                    if (o.oneofs)
                                        d._chain_name = "chain_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AffiliateLocationFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AffiliateLocationFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AffiliateLocationFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AffiliateLocationFeedItem;
                        })();
    
                        common.TextMessageFeedItem = (function() {
    
                            /**
                             * Properties of a TextMessageFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITextMessageFeedItem
                             * @property {string|null} [business_name] TextMessageFeedItem business_name
                             * @property {string|null} [country_code] TextMessageFeedItem country_code
                             * @property {string|null} [phone_number] TextMessageFeedItem phone_number
                             * @property {string|null} [text] TextMessageFeedItem text
                             * @property {string|null} [extension_text] TextMessageFeedItem extension_text
                             */
    
                            /**
                             * Constructs a new TextMessageFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TextMessageFeedItem.
                             * @implements ITextMessageFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITextMessageFeedItem=} [p] Properties to set
                             */
                            function TextMessageFeedItem(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TextMessageFeedItem business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            TextMessageFeedItem.prototype.business_name = null;
    
                            /**
                             * TextMessageFeedItem country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            TextMessageFeedItem.prototype.country_code = null;
    
                            /**
                             * TextMessageFeedItem phone_number.
                             * @member {string|null|undefined} phone_number
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            TextMessageFeedItem.prototype.phone_number = null;
    
                            /**
                             * TextMessageFeedItem text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            TextMessageFeedItem.prototype.text = null;
    
                            /**
                             * TextMessageFeedItem extension_text.
                             * @member {string|null|undefined} extension_text
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            TextMessageFeedItem.prototype.extension_text = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TextMessageFeedItem _business_name.
                             * @member {"business_name"|undefined} _business_name
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            Object.defineProperty(TextMessageFeedItem.prototype, "_business_name", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TextMessageFeedItem _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            Object.defineProperty(TextMessageFeedItem.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TextMessageFeedItem _phone_number.
                             * @member {"phone_number"|undefined} _phone_number
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            Object.defineProperty(TextMessageFeedItem.prototype, "_phone_number", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_number"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TextMessageFeedItem _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            Object.defineProperty(TextMessageFeedItem.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TextMessageFeedItem _extension_text.
                             * @member {"extension_text"|undefined} _extension_text
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             */
                            Object.defineProperty(TextMessageFeedItem.prototype, "_extension_text", {
                                get: $util.oneOfGetter($oneOfFields = ["extension_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TextMessageFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextMessageFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TextMessageFeedItem} TextMessageFeedItem instance
                             */
                            TextMessageFeedItem.create = function create(properties) {
                                return new TextMessageFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified TextMessageFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.TextMessageFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextMessageFeedItem} m TextMessageFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextMessageFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(50).string(m.business_name);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(58).string(m.country_code);
                                if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                    w.uint32(66).string(m.phone_number);
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(74).string(m.text);
                                if (m.extension_text != null && Object.hasOwnProperty.call(m, "extension_text"))
                                    w.uint32(82).string(m.extension_text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TextMessageFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TextMessageFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextMessageFeedItem} message TextMessageFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextMessageFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TextMessageFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TextMessageFeedItem} TextMessageFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextMessageFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TextMessageFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.business_name = r.string();
                                        break;
                                    case 7:
                                        m.country_code = r.string();
                                        break;
                                    case 8:
                                        m.phone_number = r.string();
                                        break;
                                    case 9:
                                        m.text = r.string();
                                        break;
                                    case 10:
                                        m.extension_text = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TextMessageFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TextMessageFeedItem} TextMessageFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextMessageFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TextMessageFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TextMessageFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p._business_name = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    p._phone_number = 1;
                                    if (!$util.isString(m.phone_number))
                                        return "phone_number: string expected";
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.extension_text != null && m.hasOwnProperty("extension_text")) {
                                    p._extension_text = 1;
                                    if (!$util.isString(m.extension_text))
                                        return "extension_text: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TextMessageFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TextMessageFeedItem} TextMessageFeedItem
                             */
                            TextMessageFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TextMessageFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TextMessageFeedItem();
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.phone_number != null) {
                                    m.phone_number = String(d.phone_number);
                                }
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                if (d.extension_text != null) {
                                    m.extension_text = String(d.extension_text);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TextMessageFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.TextMessageFeedItem} m TextMessageFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TextMessageFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d._business_name = "business_name";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    d.phone_number = m.phone_number;
                                    if (o.oneofs)
                                        d._phone_number = "phone_number";
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.extension_text != null && m.hasOwnProperty("extension_text")) {
                                    d.extension_text = m.extension_text;
                                    if (o.oneofs)
                                        d._extension_text = "extension_text";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TextMessageFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TextMessageFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TextMessageFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TextMessageFeedItem;
                        })();
    
                        common.PriceFeedItem = (function() {
    
                            /**
                             * Properties of a PriceFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPriceFeedItem
                             * @property {google.ads.googleads.v11.enums.PriceExtensionTypeEnum.PriceExtensionType|null} [type] PriceFeedItem type
                             * @property {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier|null} [price_qualifier] PriceFeedItem price_qualifier
                             * @property {string|null} [tracking_url_template] PriceFeedItem tracking_url_template
                             * @property {string|null} [language_code] PriceFeedItem language_code
                             * @property {Array.<google.ads.googleads.v11.common.IPriceOffer>|null} [price_offerings] PriceFeedItem price_offerings
                             * @property {string|null} [final_url_suffix] PriceFeedItem final_url_suffix
                             */
    
                            /**
                             * Constructs a new PriceFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PriceFeedItem.
                             * @implements IPriceFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPriceFeedItem=} [p] Properties to set
                             */
                            function PriceFeedItem(p) {
                                this.price_offerings = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PriceFeedItem type.
                             * @member {google.ads.googleads.v11.enums.PriceExtensionTypeEnum.PriceExtensionType} type
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            PriceFeedItem.prototype.type = 0;
    
                            /**
                             * PriceFeedItem price_qualifier.
                             * @member {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier} price_qualifier
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            PriceFeedItem.prototype.price_qualifier = 0;
    
                            /**
                             * PriceFeedItem tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            PriceFeedItem.prototype.tracking_url_template = null;
    
                            /**
                             * PriceFeedItem language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            PriceFeedItem.prototype.language_code = null;
    
                            /**
                             * PriceFeedItem price_offerings.
                             * @member {Array.<google.ads.googleads.v11.common.IPriceOffer>} price_offerings
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            PriceFeedItem.prototype.price_offerings = $util.emptyArray;
    
                            /**
                             * PriceFeedItem final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            PriceFeedItem.prototype.final_url_suffix = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PriceFeedItem _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            Object.defineProperty(PriceFeedItem.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PriceFeedItem _language_code.
                             * @member {"language_code"|undefined} _language_code
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            Object.defineProperty(PriceFeedItem.prototype, "_language_code", {
                                get: $util.oneOfGetter($oneOfFields = ["language_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PriceFeedItem _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             */
                            Object.defineProperty(PriceFeedItem.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PriceFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PriceFeedItem} PriceFeedItem instance
                             */
                            PriceFeedItem.create = function create(properties) {
                                return new PriceFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified PriceFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.PriceFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceFeedItem} m PriceFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                if (m.price_qualifier != null && Object.hasOwnProperty.call(m, "price_qualifier"))
                                    w.uint32(16).int32(m.price_qualifier);
                                if (m.price_offerings != null && m.price_offerings.length) {
                                    for (var i = 0; i < m.price_offerings.length; ++i)
                                        $root.google.ads.googleads.v11.common.PriceOffer.encode(m.price_offerings[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(58).string(m.tracking_url_template);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(66).string(m.language_code);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(74).string(m.final_url_suffix);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PriceFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PriceFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceFeedItem} message PriceFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PriceFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PriceFeedItem} PriceFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PriceFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    case 2:
                                        m.price_qualifier = r.int32();
                                        break;
                                    case 7:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 8:
                                        m.language_code = r.string();
                                        break;
                                    case 5:
                                        if (!(m.price_offerings && m.price_offerings.length))
                                            m.price_offerings = [];
                                        m.price_offerings.push($root.google.ads.googleads.v11.common.PriceOffer.decode(r, r.uint32()));
                                        break;
                                    case 9:
                                        m.final_url_suffix = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PriceFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PriceFeedItem} PriceFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PriceFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PriceFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.price_qualifier != null && m.hasOwnProperty("price_qualifier")) {
                                    switch (m.price_qualifier) {
                                    default:
                                        return "price_qualifier: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    p._language_code = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.price_offerings != null && m.hasOwnProperty("price_offerings")) {
                                    if (!Array.isArray(m.price_offerings))
                                        return "price_offerings: array expected";
                                    for (var i = 0; i < m.price_offerings.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PriceOffer.verify(m.price_offerings[i]);
                                            if (e)
                                                return "price_offerings." + e;
                                        }
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PriceFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PriceFeedItem} PriceFeedItem
                             */
                            PriceFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PriceFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PriceFeedItem();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "BRANDS":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "EVENTS":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "LOCATIONS":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "NEIGHBORHOODS":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "PRODUCT_CATEGORIES":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PRODUCT_TIERS":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "SERVICES":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "SERVICE_CATEGORIES":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "SERVICE_TIERS":
                                case 10:
                                    m.type = 10;
                                    break;
                                }
                                switch (d.price_qualifier) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.price_qualifier = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.price_qualifier = 1;
                                    break;
                                case "FROM":
                                case 2:
                                    m.price_qualifier = 2;
                                    break;
                                case "UP_TO":
                                case 3:
                                    m.price_qualifier = 3;
                                    break;
                                case "AVERAGE":
                                case 4:
                                    m.price_qualifier = 4;
                                    break;
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.price_offerings) {
                                    if (!Array.isArray(d.price_offerings))
                                        throw TypeError(".google.ads.googleads.v11.common.PriceFeedItem.price_offerings: array expected");
                                    m.price_offerings = [];
                                    for (var i = 0; i < d.price_offerings.length; ++i) {
                                        if (typeof d.price_offerings[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PriceFeedItem.price_offerings: object expected");
                                        m.price_offerings[i] = $root.google.ads.googleads.v11.common.PriceOffer.fromObject(d.price_offerings[i]);
                                    }
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PriceFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.PriceFeedItem} m PriceFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PriceFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.price_offerings = [];
                                }
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.price_qualifier = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.PriceExtensionTypeEnum.PriceExtensionType[m.type] : m.type;
                                }
                                if (m.price_qualifier != null && m.hasOwnProperty("price_qualifier")) {
                                    d.price_qualifier = o.enums === String ? $root.google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier[m.price_qualifier] : m.price_qualifier;
                                }
                                if (m.price_offerings && m.price_offerings.length) {
                                    d.price_offerings = [];
                                    for (var j = 0; j < m.price_offerings.length; ++j) {
                                        d.price_offerings[j] = $root.google.ads.googleads.v11.common.PriceOffer.toObject(m.price_offerings[j], o);
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d._language_code = "language_code";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PriceFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PriceFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PriceFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PriceFeedItem;
                        })();
    
                        common.PriceOffer = (function() {
    
                            /**
                             * Properties of a PriceOffer.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPriceOffer
                             * @property {string|null} [header] PriceOffer header
                             * @property {string|null} [description] PriceOffer description
                             * @property {google.ads.googleads.v11.common.IMoney|null} [price] PriceOffer price
                             * @property {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit|null} [unit] PriceOffer unit
                             * @property {Array.<string>|null} [final_urls] PriceOffer final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] PriceOffer final_mobile_urls
                             */
    
                            /**
                             * Constructs a new PriceOffer.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PriceOffer.
                             * @implements IPriceOffer
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPriceOffer=} [p] Properties to set
                             */
                            function PriceOffer(p) {
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PriceOffer header.
                             * @member {string|null|undefined} header
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            PriceOffer.prototype.header = null;
    
                            /**
                             * PriceOffer description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            PriceOffer.prototype.description = null;
    
                            /**
                             * PriceOffer price.
                             * @member {google.ads.googleads.v11.common.IMoney|null|undefined} price
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            PriceOffer.prototype.price = null;
    
                            /**
                             * PriceOffer unit.
                             * @member {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit} unit
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            PriceOffer.prototype.unit = 0;
    
                            /**
                             * PriceOffer final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            PriceOffer.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * PriceOffer final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            PriceOffer.prototype.final_mobile_urls = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PriceOffer _header.
                             * @member {"header"|undefined} _header
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            Object.defineProperty(PriceOffer.prototype, "_header", {
                                get: $util.oneOfGetter($oneOfFields = ["header"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PriceOffer _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             */
                            Object.defineProperty(PriceOffer.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PriceOffer instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceOffer=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PriceOffer} PriceOffer instance
                             */
                            PriceOffer.create = function create(properties) {
                                return new PriceOffer(properties);
                            };
    
                            /**
                             * Encodes the specified PriceOffer message. Does not implicitly {@link google.ads.googleads.v11.common.PriceOffer.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceOffer} m PriceOffer message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceOffer.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.price != null && Object.hasOwnProperty.call(m, "price"))
                                    $root.google.ads.googleads.v11.common.Money.encode(m.price, w.uint32(26).fork()).ldelim();
                                if (m.unit != null && Object.hasOwnProperty.call(m, "unit"))
                                    w.uint32(32).int32(m.unit);
                                if (m.header != null && Object.hasOwnProperty.call(m, "header"))
                                    w.uint32(58).string(m.header);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(66).string(m.description);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(74).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(82).string(m.final_mobile_urls[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PriceOffer message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PriceOffer.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {google.ads.googleads.v11.common.IPriceOffer} message PriceOffer message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceOffer.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PriceOffer message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PriceOffer} PriceOffer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceOffer.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PriceOffer();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 7:
                                        m.header = r.string();
                                        break;
                                    case 8:
                                        m.description = r.string();
                                        break;
                                    case 3:
                                        m.price = $root.google.ads.googleads.v11.common.Money.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.unit = r.int32();
                                        break;
                                    case 9:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 10:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PriceOffer message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PriceOffer} PriceOffer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceOffer.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PriceOffer message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PriceOffer.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    p._header = 1;
                                    if (!$util.isString(m.header))
                                        return "header: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Money.verify(m.price);
                                        if (e)
                                            return "price." + e;
                                    }
                                }
                                if (m.unit != null && m.hasOwnProperty("unit")) {
                                    switch (m.unit) {
                                    default:
                                        return "unit: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PriceOffer message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PriceOffer} PriceOffer
                             */
                            PriceOffer.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PriceOffer)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PriceOffer();
                                if (d.header != null) {
                                    m.header = String(d.header);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.price != null) {
                                    if (typeof d.price !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PriceOffer.price: object expected");
                                    m.price = $root.google.ads.googleads.v11.common.Money.fromObject(d.price);
                                }
                                switch (d.unit) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.unit = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.unit = 1;
                                    break;
                                case "PER_HOUR":
                                case 2:
                                    m.unit = 2;
                                    break;
                                case "PER_DAY":
                                case 3:
                                    m.unit = 3;
                                    break;
                                case "PER_WEEK":
                                case 4:
                                    m.unit = 4;
                                    break;
                                case "PER_MONTH":
                                case 5:
                                    m.unit = 5;
                                    break;
                                case "PER_YEAR":
                                case 6:
                                    m.unit = 6;
                                    break;
                                case "PER_NIGHT":
                                case 7:
                                    m.unit = 7;
                                    break;
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.PriceOffer.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.PriceOffer.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PriceOffer message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @static
                             * @param {google.ads.googleads.v11.common.PriceOffer} m PriceOffer
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PriceOffer.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (o.defaults) {
                                    d.price = null;
                                    d.unit = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.price != null && m.hasOwnProperty("price")) {
                                    d.price = $root.google.ads.googleads.v11.common.Money.toObject(m.price, o);
                                }
                                if (m.unit != null && m.hasOwnProperty("unit")) {
                                    d.unit = o.enums === String ? $root.google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit[m.unit] : m.unit;
                                }
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    d.header = m.header;
                                    if (o.oneofs)
                                        d._header = "header";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PriceOffer to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PriceOffer
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PriceOffer.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PriceOffer;
                        })();
    
                        common.PromotionFeedItem = (function() {
    
                            /**
                             * Properties of a PromotionFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPromotionFeedItem
                             * @property {string|null} [promotion_target] PromotionFeedItem promotion_target
                             * @property {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier|null} [discount_modifier] PromotionFeedItem discount_modifier
                             * @property {string|null} [promotion_start_date] PromotionFeedItem promotion_start_date
                             * @property {string|null} [promotion_end_date] PromotionFeedItem promotion_end_date
                             * @property {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.PromotionExtensionOccasion|null} [occasion] PromotionFeedItem occasion
                             * @property {Array.<string>|null} [final_urls] PromotionFeedItem final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] PromotionFeedItem final_mobile_urls
                             * @property {string|null} [tracking_url_template] PromotionFeedItem tracking_url_template
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] PromotionFeedItem url_custom_parameters
                             * @property {string|null} [final_url_suffix] PromotionFeedItem final_url_suffix
                             * @property {string|null} [language_code] PromotionFeedItem language_code
                             * @property {number|null} [percent_off] PromotionFeedItem percent_off
                             * @property {google.ads.googleads.v11.common.IMoney|null} [money_amount_off] PromotionFeedItem money_amount_off
                             * @property {string|null} [promotion_code] PromotionFeedItem promotion_code
                             * @property {google.ads.googleads.v11.common.IMoney|null} [orders_over_amount] PromotionFeedItem orders_over_amount
                             */
    
                            /**
                             * Constructs a new PromotionFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PromotionFeedItem.
                             * @implements IPromotionFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPromotionFeedItem=} [p] Properties to set
                             */
                            function PromotionFeedItem(p) {
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                this.url_custom_parameters = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PromotionFeedItem promotion_target.
                             * @member {string|null|undefined} promotion_target
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.promotion_target = null;
    
                            /**
                             * PromotionFeedItem discount_modifier.
                             * @member {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier} discount_modifier
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.discount_modifier = 0;
    
                            /**
                             * PromotionFeedItem promotion_start_date.
                             * @member {string|null|undefined} promotion_start_date
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.promotion_start_date = null;
    
                            /**
                             * PromotionFeedItem promotion_end_date.
                             * @member {string|null|undefined} promotion_end_date
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.promotion_end_date = null;
    
                            /**
                             * PromotionFeedItem occasion.
                             * @member {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.PromotionExtensionOccasion} occasion
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.occasion = 0;
    
                            /**
                             * PromotionFeedItem final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * PromotionFeedItem final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * PromotionFeedItem tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.tracking_url_template = null;
    
                            /**
                             * PromotionFeedItem url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * PromotionFeedItem final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.final_url_suffix = null;
    
                            /**
                             * PromotionFeedItem language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.language_code = null;
    
                            /**
                             * PromotionFeedItem percent_off.
                             * @member {number|null|undefined} percent_off
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.percent_off = null;
    
                            /**
                             * PromotionFeedItem money_amount_off.
                             * @member {google.ads.googleads.v11.common.IMoney|null|undefined} money_amount_off
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.money_amount_off = null;
    
                            /**
                             * PromotionFeedItem promotion_code.
                             * @member {string|null|undefined} promotion_code
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.promotion_code = null;
    
                            /**
                             * PromotionFeedItem orders_over_amount.
                             * @member {google.ads.googleads.v11.common.IMoney|null|undefined} orders_over_amount
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            PromotionFeedItem.prototype.orders_over_amount = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PromotionFeedItem _promotion_target.
                             * @member {"promotion_target"|undefined} _promotion_target
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "_promotion_target", {
                                get: $util.oneOfGetter($oneOfFields = ["promotion_target"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionFeedItem _promotion_start_date.
                             * @member {"promotion_start_date"|undefined} _promotion_start_date
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "_promotion_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["promotion_start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionFeedItem _promotion_end_date.
                             * @member {"promotion_end_date"|undefined} _promotion_end_date
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "_promotion_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["promotion_end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionFeedItem _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionFeedItem _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionFeedItem _language_code.
                             * @member {"language_code"|undefined} _language_code
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "_language_code", {
                                get: $util.oneOfGetter($oneOfFields = ["language_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionFeedItem discount_type.
                             * @member {"percent_off"|"money_amount_off"|undefined} discount_type
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "discount_type", {
                                get: $util.oneOfGetter($oneOfFields = ["percent_off", "money_amount_off"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PromotionFeedItem promotion_trigger.
                             * @member {"promotion_code"|"orders_over_amount"|undefined} promotion_trigger
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             */
                            Object.defineProperty(PromotionFeedItem.prototype, "promotion_trigger", {
                                get: $util.oneOfGetter($oneOfFields = ["promotion_code", "orders_over_amount"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PromotionFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IPromotionFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PromotionFeedItem} PromotionFeedItem instance
                             */
                            PromotionFeedItem.create = function create(properties) {
                                return new PromotionFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified PromotionFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.PromotionFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IPromotionFeedItem} m PromotionFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.discount_modifier != null && Object.hasOwnProperty.call(m, "discount_modifier"))
                                    w.uint32(16).int32(m.discount_modifier);
                                if (m.money_amount_off != null && Object.hasOwnProperty.call(m, "money_amount_off"))
                                    $root.google.ads.googleads.v11.common.Money.encode(m.money_amount_off, w.uint32(34).fork()).ldelim();
                                if (m.orders_over_amount != null && Object.hasOwnProperty.call(m, "orders_over_amount"))
                                    $root.google.ads.googleads.v11.common.Money.encode(m.orders_over_amount, w.uint32(50).fork()).ldelim();
                                if (m.occasion != null && Object.hasOwnProperty.call(m, "occasion"))
                                    w.uint32(72).int32(m.occasion);
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(106).fork()).ldelim();
                                }
                                if (m.promotion_target != null && Object.hasOwnProperty.call(m, "promotion_target"))
                                    w.uint32(130).string(m.promotion_target);
                                if (m.percent_off != null && Object.hasOwnProperty.call(m, "percent_off"))
                                    w.uint32(136).int64(m.percent_off);
                                if (m.promotion_code != null && Object.hasOwnProperty.call(m, "promotion_code"))
                                    w.uint32(146).string(m.promotion_code);
                                if (m.promotion_start_date != null && Object.hasOwnProperty.call(m, "promotion_start_date"))
                                    w.uint32(154).string(m.promotion_start_date);
                                if (m.promotion_end_date != null && Object.hasOwnProperty.call(m, "promotion_end_date"))
                                    w.uint32(162).string(m.promotion_end_date);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(170).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(178).string(m.final_mobile_urls[i]);
                                }
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(186).string(m.tracking_url_template);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(194).string(m.final_url_suffix);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(202).string(m.language_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromotionFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PromotionFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IPromotionFeedItem} message PromotionFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromotionFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PromotionFeedItem} PromotionFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PromotionFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 16:
                                        m.promotion_target = r.string();
                                        break;
                                    case 2:
                                        m.discount_modifier = r.int32();
                                        break;
                                    case 19:
                                        m.promotion_start_date = r.string();
                                        break;
                                    case 20:
                                        m.promotion_end_date = r.string();
                                        break;
                                    case 9:
                                        m.occasion = r.int32();
                                        break;
                                    case 21:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 22:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 23:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 13:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 24:
                                        m.final_url_suffix = r.string();
                                        break;
                                    case 25:
                                        m.language_code = r.string();
                                        break;
                                    case 17:
                                        m.percent_off = r.int64();
                                        break;
                                    case 4:
                                        m.money_amount_off = $root.google.ads.googleads.v11.common.Money.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.promotion_code = r.string();
                                        break;
                                    case 6:
                                        m.orders_over_amount = $root.google.ads.googleads.v11.common.Money.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromotionFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PromotionFeedItem} PromotionFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromotionFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromotionFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.promotion_target != null && m.hasOwnProperty("promotion_target")) {
                                    p._promotion_target = 1;
                                    if (!$util.isString(m.promotion_target))
                                        return "promotion_target: string expected";
                                }
                                if (m.discount_modifier != null && m.hasOwnProperty("discount_modifier")) {
                                    switch (m.discount_modifier) {
                                    default:
                                        return "discount_modifier: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.promotion_start_date != null && m.hasOwnProperty("promotion_start_date")) {
                                    p._promotion_start_date = 1;
                                    if (!$util.isString(m.promotion_start_date))
                                        return "promotion_start_date: string expected";
                                }
                                if (m.promotion_end_date != null && m.hasOwnProperty("promotion_end_date")) {
                                    p._promotion_end_date = 1;
                                    if (!$util.isString(m.promotion_end_date))
                                        return "promotion_end_date: string expected";
                                }
                                if (m.occasion != null && m.hasOwnProperty("occasion")) {
                                    switch (m.occasion) {
                                    default:
                                        return "occasion: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                        break;
                                    }
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    p._language_code = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.percent_off != null && m.hasOwnProperty("percent_off")) {
                                    p.discount_type = 1;
                                    if (!$util.isInteger(m.percent_off) && !(m.percent_off && $util.isInteger(m.percent_off.low) && $util.isInteger(m.percent_off.high)))
                                        return "percent_off: integer|Long expected";
                                }
                                if (m.money_amount_off != null && m.hasOwnProperty("money_amount_off")) {
                                    if (p.discount_type === 1)
                                        return "discount_type: multiple values";
                                    p.discount_type = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Money.verify(m.money_amount_off);
                                        if (e)
                                            return "money_amount_off." + e;
                                    }
                                }
                                if (m.promotion_code != null && m.hasOwnProperty("promotion_code")) {
                                    p.promotion_trigger = 1;
                                    if (!$util.isString(m.promotion_code))
                                        return "promotion_code: string expected";
                                }
                                if (m.orders_over_amount != null && m.hasOwnProperty("orders_over_amount")) {
                                    if (p.promotion_trigger === 1)
                                        return "promotion_trigger: multiple values";
                                    p.promotion_trigger = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Money.verify(m.orders_over_amount);
                                        if (e)
                                            return "orders_over_amount." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PromotionFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PromotionFeedItem} PromotionFeedItem
                             */
                            PromotionFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PromotionFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PromotionFeedItem();
                                if (d.promotion_target != null) {
                                    m.promotion_target = String(d.promotion_target);
                                }
                                switch (d.discount_modifier) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.discount_modifier = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.discount_modifier = 1;
                                    break;
                                case "UP_TO":
                                case 2:
                                    m.discount_modifier = 2;
                                    break;
                                }
                                if (d.promotion_start_date != null) {
                                    m.promotion_start_date = String(d.promotion_start_date);
                                }
                                if (d.promotion_end_date != null) {
                                    m.promotion_end_date = String(d.promotion_end_date);
                                }
                                switch (d.occasion) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.occasion = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.occasion = 1;
                                    break;
                                case "NEW_YEARS":
                                case 2:
                                    m.occasion = 2;
                                    break;
                                case "CHINESE_NEW_YEAR":
                                case 3:
                                    m.occasion = 3;
                                    break;
                                case "VALENTINES_DAY":
                                case 4:
                                    m.occasion = 4;
                                    break;
                                case "EASTER":
                                case 5:
                                    m.occasion = 5;
                                    break;
                                case "MOTHERS_DAY":
                                case 6:
                                    m.occasion = 6;
                                    break;
                                case "FATHERS_DAY":
                                case 7:
                                    m.occasion = 7;
                                    break;
                                case "LABOR_DAY":
                                case 8:
                                    m.occasion = 8;
                                    break;
                                case "BACK_TO_SCHOOL":
                                case 9:
                                    m.occasion = 9;
                                    break;
                                case "HALLOWEEN":
                                case 10:
                                    m.occasion = 10;
                                    break;
                                case "BLACK_FRIDAY":
                                case 11:
                                    m.occasion = 11;
                                    break;
                                case "CYBER_MONDAY":
                                case 12:
                                    m.occasion = 12;
                                    break;
                                case "CHRISTMAS":
                                case 13:
                                    m.occasion = 13;
                                    break;
                                case "BOXING_DAY":
                                case 14:
                                    m.occasion = 14;
                                    break;
                                case "INDEPENDENCE_DAY":
                                case 15:
                                    m.occasion = 15;
                                    break;
                                case "NATIONAL_DAY":
                                case 16:
                                    m.occasion = 16;
                                    break;
                                case "END_OF_SEASON":
                                case 17:
                                    m.occasion = 17;
                                    break;
                                case "WINTER_SALE":
                                case 18:
                                    m.occasion = 18;
                                    break;
                                case "SUMMER_SALE":
                                case 19:
                                    m.occasion = 19;
                                    break;
                                case "FALL_SALE":
                                case 20:
                                    m.occasion = 20;
                                    break;
                                case "SPRING_SALE":
                                case 21:
                                    m.occasion = 21;
                                    break;
                                case "RAMADAN":
                                case 22:
                                    m.occasion = 22;
                                    break;
                                case "EID_AL_FITR":
                                case 23:
                                    m.occasion = 23;
                                    break;
                                case "EID_AL_ADHA":
                                case 24:
                                    m.occasion = 24;
                                    break;
                                case "SINGLES_DAY":
                                case 25:
                                    m.occasion = 25;
                                    break;
                                case "WOMENS_DAY":
                                case 26:
                                    m.occasion = 26;
                                    break;
                                case "HOLI":
                                case 27:
                                    m.occasion = 27;
                                    break;
                                case "PARENTS_DAY":
                                case 28:
                                    m.occasion = 28;
                                    break;
                                case "ST_NICHOLAS_DAY":
                                case 29:
                                    m.occasion = 29;
                                    break;
                                case "CARNIVAL":
                                case 30:
                                    m.occasion = 30;
                                    break;
                                case "EPIPHANY":
                                case 31:
                                    m.occasion = 31;
                                    break;
                                case "ROSH_HASHANAH":
                                case 32:
                                    m.occasion = 32;
                                    break;
                                case "PASSOVER":
                                case 33:
                                    m.occasion = 33;
                                    break;
                                case "HANUKKAH":
                                case 34:
                                    m.occasion = 34;
                                    break;
                                case "DIWALI":
                                case 35:
                                    m.occasion = 35;
                                    break;
                                case "NAVRATRI":
                                case 36:
                                    m.occasion = 36;
                                    break;
                                case "SONGKRAN":
                                case 37:
                                    m.occasion = 37;
                                    break;
                                case "YEAR_END_GIFT":
                                case 38:
                                    m.occasion = 38;
                                    break;
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionFeedItem.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionFeedItem.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionFeedItem.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PromotionFeedItem.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.percent_off != null) {
                                    if ($util.Long)
                                        (m.percent_off = $util.Long.fromValue(d.percent_off)).unsigned = false;
                                    else if (typeof d.percent_off === "string")
                                        m.percent_off = parseInt(d.percent_off, 10);
                                    else if (typeof d.percent_off === "number")
                                        m.percent_off = d.percent_off;
                                    else if (typeof d.percent_off === "object")
                                        m.percent_off = new $util.LongBits(d.percent_off.low >>> 0, d.percent_off.high >>> 0).toNumber();
                                }
                                if (d.money_amount_off != null) {
                                    if (typeof d.money_amount_off !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionFeedItem.money_amount_off: object expected");
                                    m.money_amount_off = $root.google.ads.googleads.v11.common.Money.fromObject(d.money_amount_off);
                                }
                                if (d.promotion_code != null) {
                                    m.promotion_code = String(d.promotion_code);
                                }
                                if (d.orders_over_amount != null) {
                                    if (typeof d.orders_over_amount !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.PromotionFeedItem.orders_over_amount: object expected");
                                    m.orders_over_amount = $root.google.ads.googleads.v11.common.Money.fromObject(d.orders_over_amount);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PromotionFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.PromotionFeedItem} m PromotionFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromotionFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.url_custom_parameters = [];
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (o.defaults) {
                                    d.discount_modifier = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.occasion = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.discount_modifier != null && m.hasOwnProperty("discount_modifier")) {
                                    d.discount_modifier = o.enums === String ? $root.google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier[m.discount_modifier] : m.discount_modifier;
                                }
                                if (m.money_amount_off != null && m.hasOwnProperty("money_amount_off")) {
                                    d.money_amount_off = $root.google.ads.googleads.v11.common.Money.toObject(m.money_amount_off, o);
                                    if (o.oneofs)
                                        d.discount_type = "money_amount_off";
                                }
                                if (m.orders_over_amount != null && m.hasOwnProperty("orders_over_amount")) {
                                    d.orders_over_amount = $root.google.ads.googleads.v11.common.Money.toObject(m.orders_over_amount, o);
                                    if (o.oneofs)
                                        d.promotion_trigger = "orders_over_amount";
                                }
                                if (m.occasion != null && m.hasOwnProperty("occasion")) {
                                    d.occasion = o.enums === String ? $root.google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.PromotionExtensionOccasion[m.occasion] : m.occasion;
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.promotion_target != null && m.hasOwnProperty("promotion_target")) {
                                    d.promotion_target = m.promotion_target;
                                    if (o.oneofs)
                                        d._promotion_target = "promotion_target";
                                }
                                if (m.percent_off != null && m.hasOwnProperty("percent_off")) {
                                    if (typeof m.percent_off === "number")
                                        d.percent_off = o.longs === String ? String(m.percent_off) : m.percent_off;
                                    else
                                        d.percent_off = o.longs === String ? $util.Long.prototype.toString.call(m.percent_off) : o.longs === Number ? new $util.LongBits(m.percent_off.low >>> 0, m.percent_off.high >>> 0).toNumber() : m.percent_off;
                                    if (o.oneofs)
                                        d.discount_type = "percent_off";
                                }
                                if (m.promotion_code != null && m.hasOwnProperty("promotion_code")) {
                                    d.promotion_code = m.promotion_code;
                                    if (o.oneofs)
                                        d.promotion_trigger = "promotion_code";
                                }
                                if (m.promotion_start_date != null && m.hasOwnProperty("promotion_start_date")) {
                                    d.promotion_start_date = m.promotion_start_date;
                                    if (o.oneofs)
                                        d._promotion_start_date = "promotion_start_date";
                                }
                                if (m.promotion_end_date != null && m.hasOwnProperty("promotion_end_date")) {
                                    d.promotion_end_date = m.promotion_end_date;
                                    if (o.oneofs)
                                        d._promotion_end_date = "promotion_end_date";
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d._language_code = "language_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PromotionFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PromotionFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromotionFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PromotionFeedItem;
                        })();
    
                        common.StructuredSnippetFeedItem = (function() {
    
                            /**
                             * Properties of a StructuredSnippetFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IStructuredSnippetFeedItem
                             * @property {string|null} [header] StructuredSnippetFeedItem header
                             * @property {Array.<string>|null} [values] StructuredSnippetFeedItem values
                             */
    
                            /**
                             * Constructs a new StructuredSnippetFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a StructuredSnippetFeedItem.
                             * @implements IStructuredSnippetFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetFeedItem=} [p] Properties to set
                             */
                            function StructuredSnippetFeedItem(p) {
                                this.values = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * StructuredSnippetFeedItem header.
                             * @member {string|null|undefined} header
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @instance
                             */
                            StructuredSnippetFeedItem.prototype.header = null;
    
                            /**
                             * StructuredSnippetFeedItem values.
                             * @member {Array.<string>} values
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @instance
                             */
                            StructuredSnippetFeedItem.prototype.values = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * StructuredSnippetFeedItem _header.
                             * @member {"header"|undefined} _header
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @instance
                             */
                            Object.defineProperty(StructuredSnippetFeedItem.prototype, "_header", {
                                get: $util.oneOfGetter($oneOfFields = ["header"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new StructuredSnippetFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetFeedItem} StructuredSnippetFeedItem instance
                             */
                            StructuredSnippetFeedItem.create = function create(properties) {
                                return new StructuredSnippetFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified StructuredSnippetFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.StructuredSnippetFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetFeedItem} m StructuredSnippetFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StructuredSnippetFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.header != null && Object.hasOwnProperty.call(m, "header"))
                                    w.uint32(26).string(m.header);
                                if (m.values != null && m.values.length) {
                                    for (var i = 0; i < m.values.length; ++i)
                                        w.uint32(34).string(m.values[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified StructuredSnippetFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.StructuredSnippetFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IStructuredSnippetFeedItem} message StructuredSnippetFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StructuredSnippetFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StructuredSnippetFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetFeedItem} StructuredSnippetFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StructuredSnippetFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.header = r.string();
                                        break;
                                    case 4:
                                        if (!(m.values && m.values.length))
                                            m.values = [];
                                        m.values.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StructuredSnippetFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetFeedItem} StructuredSnippetFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StructuredSnippetFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StructuredSnippetFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StructuredSnippetFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    p._header = 1;
                                    if (!$util.isString(m.header))
                                        return "header: string expected";
                                }
                                if (m.values != null && m.hasOwnProperty("values")) {
                                    if (!Array.isArray(m.values))
                                        return "values: array expected";
                                    for (var i = 0; i < m.values.length; ++i) {
                                        if (!$util.isString(m.values[i]))
                                            return "values: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a StructuredSnippetFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.StructuredSnippetFeedItem} StructuredSnippetFeedItem
                             */
                            StructuredSnippetFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem();
                                if (d.header != null) {
                                    m.header = String(d.header);
                                }
                                if (d.values) {
                                    if (!Array.isArray(d.values))
                                        throw TypeError(".google.ads.googleads.v11.common.StructuredSnippetFeedItem.values: array expected");
                                    m.values = [];
                                    for (var i = 0; i < d.values.length; ++i) {
                                        m.values[i] = String(d.values[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a StructuredSnippetFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.StructuredSnippetFeedItem} m StructuredSnippetFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StructuredSnippetFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.values = [];
                                }
                                if (m.header != null && m.hasOwnProperty("header")) {
                                    d.header = m.header;
                                    if (o.oneofs)
                                        d._header = "header";
                                }
                                if (m.values && m.values.length) {
                                    d.values = [];
                                    for (var j = 0; j < m.values.length; ++j) {
                                        d.values[j] = m.values[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this StructuredSnippetFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.StructuredSnippetFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StructuredSnippetFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return StructuredSnippetFeedItem;
                        })();
    
                        common.SitelinkFeedItem = (function() {
    
                            /**
                             * Properties of a SitelinkFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ISitelinkFeedItem
                             * @property {string|null} [link_text] SitelinkFeedItem link_text
                             * @property {string|null} [line1] SitelinkFeedItem line1
                             * @property {string|null} [line2] SitelinkFeedItem line2
                             * @property {Array.<string>|null} [final_urls] SitelinkFeedItem final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] SitelinkFeedItem final_mobile_urls
                             * @property {string|null} [tracking_url_template] SitelinkFeedItem tracking_url_template
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] SitelinkFeedItem url_custom_parameters
                             * @property {string|null} [final_url_suffix] SitelinkFeedItem final_url_suffix
                             */
    
                            /**
                             * Constructs a new SitelinkFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a SitelinkFeedItem.
                             * @implements ISitelinkFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ISitelinkFeedItem=} [p] Properties to set
                             */
                            function SitelinkFeedItem(p) {
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                this.url_custom_parameters = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SitelinkFeedItem link_text.
                             * @member {string|null|undefined} link_text
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.link_text = null;
    
                            /**
                             * SitelinkFeedItem line1.
                             * @member {string|null|undefined} line1
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.line1 = null;
    
                            /**
                             * SitelinkFeedItem line2.
                             * @member {string|null|undefined} line2
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.line2 = null;
    
                            /**
                             * SitelinkFeedItem final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * SitelinkFeedItem final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * SitelinkFeedItem tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.tracking_url_template = null;
    
                            /**
                             * SitelinkFeedItem url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * SitelinkFeedItem final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            SitelinkFeedItem.prototype.final_url_suffix = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SitelinkFeedItem _link_text.
                             * @member {"link_text"|undefined} _link_text
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            Object.defineProperty(SitelinkFeedItem.prototype, "_link_text", {
                                get: $util.oneOfGetter($oneOfFields = ["link_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SitelinkFeedItem _line1.
                             * @member {"line1"|undefined} _line1
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            Object.defineProperty(SitelinkFeedItem.prototype, "_line1", {
                                get: $util.oneOfGetter($oneOfFields = ["line1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SitelinkFeedItem _line2.
                             * @member {"line2"|undefined} _line2
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            Object.defineProperty(SitelinkFeedItem.prototype, "_line2", {
                                get: $util.oneOfGetter($oneOfFields = ["line2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SitelinkFeedItem _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            Object.defineProperty(SitelinkFeedItem.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SitelinkFeedItem _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             */
                            Object.defineProperty(SitelinkFeedItem.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SitelinkFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ISitelinkFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.SitelinkFeedItem} SitelinkFeedItem instance
                             */
                            SitelinkFeedItem.create = function create(properties) {
                                return new SitelinkFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified SitelinkFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.SitelinkFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ISitelinkFeedItem} m SitelinkFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SitelinkFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.link_text != null && Object.hasOwnProperty.call(m, "link_text"))
                                    w.uint32(74).string(m.link_text);
                                if (m.line1 != null && Object.hasOwnProperty.call(m, "line1"))
                                    w.uint32(82).string(m.line1);
                                if (m.line2 != null && Object.hasOwnProperty.call(m, "line2"))
                                    w.uint32(90).string(m.line2);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(98).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(106).string(m.final_mobile_urls[i]);
                                }
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(114).string(m.tracking_url_template);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(122).string(m.final_url_suffix);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SitelinkFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.SitelinkFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ISitelinkFeedItem} message SitelinkFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SitelinkFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SitelinkFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.SitelinkFeedItem} SitelinkFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SitelinkFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.SitelinkFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 9:
                                        m.link_text = r.string();
                                        break;
                                    case 10:
                                        m.line1 = r.string();
                                        break;
                                    case 11:
                                        m.line2 = r.string();
                                        break;
                                    case 12:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 13:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 14:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 7:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 15:
                                        m.final_url_suffix = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SitelinkFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.SitelinkFeedItem} SitelinkFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SitelinkFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SitelinkFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SitelinkFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    p._link_text = 1;
                                    if (!$util.isString(m.link_text))
                                        return "link_text: string expected";
                                }
                                if (m.line1 != null && m.hasOwnProperty("line1")) {
                                    p._line1 = 1;
                                    if (!$util.isString(m.line1))
                                        return "line1: string expected";
                                }
                                if (m.line2 != null && m.hasOwnProperty("line2")) {
                                    p._line2 = 1;
                                    if (!$util.isString(m.line2))
                                        return "line2: string expected";
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SitelinkFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.SitelinkFeedItem} SitelinkFeedItem
                             */
                            SitelinkFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.SitelinkFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.SitelinkFeedItem();
                                if (d.link_text != null) {
                                    m.link_text = String(d.link_text);
                                }
                                if (d.line1 != null) {
                                    m.line1 = String(d.line1);
                                }
                                if (d.line2 != null) {
                                    m.line2 = String(d.line2);
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.SitelinkFeedItem.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.SitelinkFeedItem.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.common.SitelinkFeedItem.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.SitelinkFeedItem.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SitelinkFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.SitelinkFeedItem} m SitelinkFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SitelinkFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.url_custom_parameters = [];
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.link_text != null && m.hasOwnProperty("link_text")) {
                                    d.link_text = m.link_text;
                                    if (o.oneofs)
                                        d._link_text = "link_text";
                                }
                                if (m.line1 != null && m.hasOwnProperty("line1")) {
                                    d.line1 = m.line1;
                                    if (o.oneofs)
                                        d._line1 = "line1";
                                }
                                if (m.line2 != null && m.hasOwnProperty("line2")) {
                                    d.line2 = m.line2;
                                    if (o.oneofs)
                                        d._line2 = "line2";
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SitelinkFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.SitelinkFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SitelinkFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SitelinkFeedItem;
                        })();
    
                        common.HotelCalloutFeedItem = (function() {
    
                            /**
                             * Properties of a HotelCalloutFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHotelCalloutFeedItem
                             * @property {string|null} [text] HotelCalloutFeedItem text
                             * @property {string|null} [language_code] HotelCalloutFeedItem language_code
                             */
    
                            /**
                             * Constructs a new HotelCalloutFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HotelCalloutFeedItem.
                             * @implements IHotelCalloutFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHotelCalloutFeedItem=} [p] Properties to set
                             */
                            function HotelCalloutFeedItem(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelCalloutFeedItem text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @instance
                             */
                            HotelCalloutFeedItem.prototype.text = null;
    
                            /**
                             * HotelCalloutFeedItem language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @instance
                             */
                            HotelCalloutFeedItem.prototype.language_code = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HotelCalloutFeedItem _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @instance
                             */
                            Object.defineProperty(HotelCalloutFeedItem.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * HotelCalloutFeedItem _language_code.
                             * @member {"language_code"|undefined} _language_code
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @instance
                             */
                            Object.defineProperty(HotelCalloutFeedItem.prototype, "_language_code", {
                                get: $util.oneOfGetter($oneOfFields = ["language_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HotelCalloutFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCalloutFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HotelCalloutFeedItem} HotelCalloutFeedItem instance
                             */
                            HotelCalloutFeedItem.create = function create(properties) {
                                return new HotelCalloutFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified HotelCalloutFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.HotelCalloutFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCalloutFeedItem} m HotelCalloutFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCalloutFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(26).string(m.text);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(34).string(m.language_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelCalloutFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HotelCalloutFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IHotelCalloutFeedItem} message HotelCalloutFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCalloutFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelCalloutFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HotelCalloutFeedItem} HotelCalloutFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCalloutFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HotelCalloutFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.text = r.string();
                                        break;
                                    case 4:
                                        m.language_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelCalloutFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HotelCalloutFeedItem} HotelCalloutFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCalloutFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelCalloutFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelCalloutFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    p._language_code = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelCalloutFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HotelCalloutFeedItem} HotelCalloutFeedItem
                             */
                            HotelCalloutFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HotelCalloutFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HotelCalloutFeedItem();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelCalloutFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.HotelCalloutFeedItem} m HotelCalloutFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelCalloutFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d._language_code = "language_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelCalloutFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HotelCalloutFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelCalloutFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelCalloutFeedItem;
                        })();
    
                        common.ImageFeedItem = (function() {
    
                            /**
                             * Properties of an ImageFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IImageFeedItem
                             * @property {string|null} [image_asset] ImageFeedItem image_asset
                             */
    
                            /**
                             * Constructs a new ImageFeedItem.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ImageFeedItem.
                             * @implements IImageFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IImageFeedItem=} [p] Properties to set
                             */
                            function ImageFeedItem(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ImageFeedItem image_asset.
                             * @member {string} image_asset
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @instance
                             */
                            ImageFeedItem.prototype.image_asset = "";
    
                            /**
                             * Creates a new ImageFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ImageFeedItem} ImageFeedItem instance
                             */
                            ImageFeedItem.create = function create(properties) {
                                return new ImageFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified ImageFeedItem message. Does not implicitly {@link google.ads.googleads.v11.common.ImageFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageFeedItem} m ImageFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.image_asset != null && Object.hasOwnProperty.call(m, "image_asset"))
                                    w.uint32(10).string(m.image_asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ImageFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ImageFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.IImageFeedItem} message ImageFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ImageFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ImageFeedItem} ImageFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ImageFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.image_asset = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ImageFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ImageFeedItem} ImageFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ImageFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ImageFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.image_asset != null && m.hasOwnProperty("image_asset")) {
                                    if (!$util.isString(m.image_asset))
                                        return "image_asset: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ImageFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ImageFeedItem} ImageFeedItem
                             */
                            ImageFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ImageFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ImageFeedItem();
                                if (d.image_asset != null) {
                                    m.image_asset = String(d.image_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ImageFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.common.ImageFeedItem} m ImageFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ImageFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.image_asset = "";
                                }
                                if (m.image_asset != null && m.hasOwnProperty("image_asset")) {
                                    d.image_asset = m.image_asset;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ImageFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ImageFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ImageFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ImageFeedItem;
                        })();
    
                        common.DynamicLocationSetFilter = (function() {
    
                            /**
                             * Properties of a DynamicLocationSetFilter.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicLocationSetFilter
                             * @property {Array.<string>|null} [labels] DynamicLocationSetFilter labels
                             * @property {google.ads.googleads.v11.common.IBusinessNameFilter|null} [business_name_filter] DynamicLocationSetFilter business_name_filter
                             */
    
                            /**
                             * Constructs a new DynamicLocationSetFilter.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicLocationSetFilter.
                             * @implements IDynamicLocationSetFilter
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicLocationSetFilter=} [p] Properties to set
                             */
                            function DynamicLocationSetFilter(p) {
                                this.labels = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicLocationSetFilter labels.
                             * @member {Array.<string>} labels
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @instance
                             */
                            DynamicLocationSetFilter.prototype.labels = $util.emptyArray;
    
                            /**
                             * DynamicLocationSetFilter business_name_filter.
                             * @member {google.ads.googleads.v11.common.IBusinessNameFilter|null|undefined} business_name_filter
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @instance
                             */
                            DynamicLocationSetFilter.prototype.business_name_filter = null;
    
                            /**
                             * Creates a new DynamicLocationSetFilter instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicLocationSetFilter=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicLocationSetFilter} DynamicLocationSetFilter instance
                             */
                            DynamicLocationSetFilter.create = function create(properties) {
                                return new DynamicLocationSetFilter(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicLocationSetFilter message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicLocationSetFilter.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicLocationSetFilter} m DynamicLocationSetFilter message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicLocationSetFilter.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.labels != null && m.labels.length) {
                                    for (var i = 0; i < m.labels.length; ++i)
                                        w.uint32(10).string(m.labels[i]);
                                }
                                if (m.business_name_filter != null && Object.hasOwnProperty.call(m, "business_name_filter"))
                                    $root.google.ads.googleads.v11.common.BusinessNameFilter.encode(m.business_name_filter, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicLocationSetFilter message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicLocationSetFilter.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicLocationSetFilter} message DynamicLocationSetFilter message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicLocationSetFilter.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicLocationSetFilter message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicLocationSetFilter} DynamicLocationSetFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicLocationSetFilter.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicLocationSetFilter();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.labels && m.labels.length))
                                            m.labels = [];
                                        m.labels.push(r.string());
                                        break;
                                    case 2:
                                        m.business_name_filter = $root.google.ads.googleads.v11.common.BusinessNameFilter.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicLocationSetFilter message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicLocationSetFilter} DynamicLocationSetFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicLocationSetFilter.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicLocationSetFilter message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicLocationSetFilter.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.labels != null && m.hasOwnProperty("labels")) {
                                    if (!Array.isArray(m.labels))
                                        return "labels: array expected";
                                    for (var i = 0; i < m.labels.length; ++i) {
                                        if (!$util.isString(m.labels[i]))
                                            return "labels: string[] expected";
                                    }
                                }
                                if (m.business_name_filter != null && m.hasOwnProperty("business_name_filter")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.BusinessNameFilter.verify(m.business_name_filter);
                                        if (e)
                                            return "business_name_filter." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicLocationSetFilter message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicLocationSetFilter} DynamicLocationSetFilter
                             */
                            DynamicLocationSetFilter.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicLocationSetFilter)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicLocationSetFilter();
                                if (d.labels) {
                                    if (!Array.isArray(d.labels))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicLocationSetFilter.labels: array expected");
                                    m.labels = [];
                                    for (var i = 0; i < d.labels.length; ++i) {
                                        m.labels[i] = String(d.labels[i]);
                                    }
                                }
                                if (d.business_name_filter != null) {
                                    if (typeof d.business_name_filter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicLocationSetFilter.business_name_filter: object expected");
                                    m.business_name_filter = $root.google.ads.googleads.v11.common.BusinessNameFilter.fromObject(d.business_name_filter);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicLocationSetFilter message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicLocationSetFilter} m DynamicLocationSetFilter
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicLocationSetFilter.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.labels = [];
                                }
                                if (o.defaults) {
                                    d.business_name_filter = null;
                                }
                                if (m.labels && m.labels.length) {
                                    d.labels = [];
                                    for (var j = 0; j < m.labels.length; ++j) {
                                        d.labels[j] = m.labels[j];
                                    }
                                }
                                if (m.business_name_filter != null && m.hasOwnProperty("business_name_filter")) {
                                    d.business_name_filter = $root.google.ads.googleads.v11.common.BusinessNameFilter.toObject(m.business_name_filter, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicLocationSetFilter to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicLocationSetFilter
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicLocationSetFilter.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicLocationSetFilter;
                        })();
    
                        common.BusinessNameFilter = (function() {
    
                            /**
                             * Properties of a BusinessNameFilter.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBusinessNameFilter
                             * @property {string|null} [business_name] BusinessNameFilter business_name
                             * @property {google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum.FeedItemSetStringFilterType|null} [filter_type] BusinessNameFilter filter_type
                             */
    
                            /**
                             * Constructs a new BusinessNameFilter.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BusinessNameFilter.
                             * @implements IBusinessNameFilter
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBusinessNameFilter=} [p] Properties to set
                             */
                            function BusinessNameFilter(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BusinessNameFilter business_name.
                             * @member {string} business_name
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @instance
                             */
                            BusinessNameFilter.prototype.business_name = "";
    
                            /**
                             * BusinessNameFilter filter_type.
                             * @member {google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum.FeedItemSetStringFilterType} filter_type
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @instance
                             */
                            BusinessNameFilter.prototype.filter_type = 0;
    
                            /**
                             * Creates a new BusinessNameFilter instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IBusinessNameFilter=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BusinessNameFilter} BusinessNameFilter instance
                             */
                            BusinessNameFilter.create = function create(properties) {
                                return new BusinessNameFilter(properties);
                            };
    
                            /**
                             * Encodes the specified BusinessNameFilter message. Does not implicitly {@link google.ads.googleads.v11.common.BusinessNameFilter.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IBusinessNameFilter} m BusinessNameFilter message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BusinessNameFilter.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(10).string(m.business_name);
                                if (m.filter_type != null && Object.hasOwnProperty.call(m, "filter_type"))
                                    w.uint32(16).int32(m.filter_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified BusinessNameFilter message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BusinessNameFilter.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IBusinessNameFilter} message BusinessNameFilter message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BusinessNameFilter.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BusinessNameFilter message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BusinessNameFilter} BusinessNameFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BusinessNameFilter.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BusinessNameFilter();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.business_name = r.string();
                                        break;
                                    case 2:
                                        m.filter_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BusinessNameFilter message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BusinessNameFilter} BusinessNameFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BusinessNameFilter.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BusinessNameFilter message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BusinessNameFilter.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.filter_type != null && m.hasOwnProperty("filter_type")) {
                                    switch (m.filter_type) {
                                    default:
                                        return "filter_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BusinessNameFilter message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BusinessNameFilter} BusinessNameFilter
                             */
                            BusinessNameFilter.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BusinessNameFilter)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.BusinessNameFilter();
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                switch (d.filter_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.filter_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.filter_type = 1;
                                    break;
                                case "EXACT":
                                case 2:
                                    m.filter_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BusinessNameFilter message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.BusinessNameFilter} m BusinessNameFilter
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BusinessNameFilter.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.business_name = "";
                                    d.filter_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                }
                                if (m.filter_type != null && m.hasOwnProperty("filter_type")) {
                                    d.filter_type = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum.FeedItemSetStringFilterType[m.filter_type] : m.filter_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BusinessNameFilter to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BusinessNameFilter
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BusinessNameFilter.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BusinessNameFilter;
                        })();
    
                        common.DynamicAffiliateLocationSetFilter = (function() {
    
                            /**
                             * Properties of a DynamicAffiliateLocationSetFilter.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IDynamicAffiliateLocationSetFilter
                             * @property {Array.<number>|null} [chain_ids] DynamicAffiliateLocationSetFilter chain_ids
                             */
    
                            /**
                             * Constructs a new DynamicAffiliateLocationSetFilter.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a DynamicAffiliateLocationSetFilter.
                             * @implements IDynamicAffiliateLocationSetFilter
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IDynamicAffiliateLocationSetFilter=} [p] Properties to set
                             */
                            function DynamicAffiliateLocationSetFilter(p) {
                                this.chain_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicAffiliateLocationSetFilter chain_ids.
                             * @member {Array.<number>} chain_ids
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @instance
                             */
                            DynamicAffiliateLocationSetFilter.prototype.chain_ids = $util.emptyArray;
    
                            /**
                             * Creates a new DynamicAffiliateLocationSetFilter instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicAffiliateLocationSetFilter=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter} DynamicAffiliateLocationSetFilter instance
                             */
                            DynamicAffiliateLocationSetFilter.create = function create(properties) {
                                return new DynamicAffiliateLocationSetFilter(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicAffiliateLocationSetFilter message. Does not implicitly {@link google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicAffiliateLocationSetFilter} m DynamicAffiliateLocationSetFilter message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicAffiliateLocationSetFilter.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.chain_ids != null && m.chain_ids.length) {
                                    w.uint32(10).fork();
                                    for (var i = 0; i < m.chain_ids.length; ++i)
                                        w.int64(m.chain_ids[i]);
                                    w.ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicAffiliateLocationSetFilter message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.IDynamicAffiliateLocationSetFilter} message DynamicAffiliateLocationSetFilter message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicAffiliateLocationSetFilter.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicAffiliateLocationSetFilter message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter} DynamicAffiliateLocationSetFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicAffiliateLocationSetFilter.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.chain_ids && m.chain_ids.length))
                                            m.chain_ids = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.chain_ids.push(r.int64());
                                        } else
                                            m.chain_ids.push(r.int64());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicAffiliateLocationSetFilter message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter} DynamicAffiliateLocationSetFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicAffiliateLocationSetFilter.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicAffiliateLocationSetFilter message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicAffiliateLocationSetFilter.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.chain_ids != null && m.hasOwnProperty("chain_ids")) {
                                    if (!Array.isArray(m.chain_ids))
                                        return "chain_ids: array expected";
                                    for (var i = 0; i < m.chain_ids.length; ++i) {
                                        if (!$util.isInteger(m.chain_ids[i]) && !(m.chain_ids[i] && $util.isInteger(m.chain_ids[i].low) && $util.isInteger(m.chain_ids[i].high)))
                                            return "chain_ids: integer|Long[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicAffiliateLocationSetFilter message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter} DynamicAffiliateLocationSetFilter
                             */
                            DynamicAffiliateLocationSetFilter.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter();
                                if (d.chain_ids) {
                                    if (!Array.isArray(d.chain_ids))
                                        throw TypeError(".google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.chain_ids: array expected");
                                    m.chain_ids = [];
                                    for (var i = 0; i < d.chain_ids.length; ++i) {
                                        if ($util.Long)
                                            (m.chain_ids[i] = $util.Long.fromValue(d.chain_ids[i])).unsigned = false;
                                        else if (typeof d.chain_ids[i] === "string")
                                            m.chain_ids[i] = parseInt(d.chain_ids[i], 10);
                                        else if (typeof d.chain_ids[i] === "number")
                                            m.chain_ids[i] = d.chain_ids[i];
                                        else if (typeof d.chain_ids[i] === "object")
                                            m.chain_ids[i] = new $util.LongBits(d.chain_ids[i].low >>> 0, d.chain_ids[i].high >>> 0).toNumber();
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicAffiliateLocationSetFilter message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @static
                             * @param {google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter} m DynamicAffiliateLocationSetFilter
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicAffiliateLocationSetFilter.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.chain_ids = [];
                                }
                                if (m.chain_ids && m.chain_ids.length) {
                                    d.chain_ids = [];
                                    for (var j = 0; j < m.chain_ids.length; ++j) {
                                        if (typeof m.chain_ids[j] === "number")
                                            d.chain_ids[j] = o.longs === String ? String(m.chain_ids[j]) : m.chain_ids[j];
                                        else
                                            d.chain_ids[j] = o.longs === String ? $util.Long.prototype.toString.call(m.chain_ids[j]) : o.longs === Number ? new $util.LongBits(m.chain_ids[j].low >>> 0, m.chain_ids[j].high >>> 0).toNumber() : m.chain_ids[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicAffiliateLocationSetFilter to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicAffiliateLocationSetFilter.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicAffiliateLocationSetFilter;
                        })();
    
                        common.FinalAppUrl = (function() {
    
                            /**
                             * Properties of a FinalAppUrl.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IFinalAppUrl
                             * @property {google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType|null} [os_type] FinalAppUrl os_type
                             * @property {string|null} [url] FinalAppUrl url
                             */
    
                            /**
                             * Constructs a new FinalAppUrl.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a FinalAppUrl.
                             * @implements IFinalAppUrl
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IFinalAppUrl=} [p] Properties to set
                             */
                            function FinalAppUrl(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FinalAppUrl os_type.
                             * @member {google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType} os_type
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @instance
                             */
                            FinalAppUrl.prototype.os_type = 0;
    
                            /**
                             * FinalAppUrl url.
                             * @member {string|null|undefined} url
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @instance
                             */
                            FinalAppUrl.prototype.url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FinalAppUrl _url.
                             * @member {"url"|undefined} _url
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @instance
                             */
                            Object.defineProperty(FinalAppUrl.prototype, "_url", {
                                get: $util.oneOfGetter($oneOfFields = ["url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FinalAppUrl instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {google.ads.googleads.v11.common.IFinalAppUrl=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.FinalAppUrl} FinalAppUrl instance
                             */
                            FinalAppUrl.create = function create(properties) {
                                return new FinalAppUrl(properties);
                            };
    
                            /**
                             * Encodes the specified FinalAppUrl message. Does not implicitly {@link google.ads.googleads.v11.common.FinalAppUrl.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {google.ads.googleads.v11.common.IFinalAppUrl} m FinalAppUrl message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FinalAppUrl.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.os_type != null && Object.hasOwnProperty.call(m, "os_type"))
                                    w.uint32(8).int32(m.os_type);
                                if (m.url != null && Object.hasOwnProperty.call(m, "url"))
                                    w.uint32(26).string(m.url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FinalAppUrl message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.FinalAppUrl.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {google.ads.googleads.v11.common.IFinalAppUrl} message FinalAppUrl message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FinalAppUrl.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FinalAppUrl message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.FinalAppUrl} FinalAppUrl
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FinalAppUrl.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.FinalAppUrl();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.os_type = r.int32();
                                        break;
                                    case 3:
                                        m.url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FinalAppUrl message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.FinalAppUrl} FinalAppUrl
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FinalAppUrl.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FinalAppUrl message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FinalAppUrl.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.os_type != null && m.hasOwnProperty("os_type")) {
                                    switch (m.os_type) {
                                    default:
                                        return "os_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    p._url = 1;
                                    if (!$util.isString(m.url))
                                        return "url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FinalAppUrl message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.FinalAppUrl} FinalAppUrl
                             */
                            FinalAppUrl.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.FinalAppUrl)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.FinalAppUrl();
                                switch (d.os_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.os_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.os_type = 1;
                                    break;
                                case "IOS":
                                case 2:
                                    m.os_type = 2;
                                    break;
                                case "ANDROID":
                                case 3:
                                    m.os_type = 3;
                                    break;
                                }
                                if (d.url != null) {
                                    m.url = String(d.url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FinalAppUrl message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @static
                             * @param {google.ads.googleads.v11.common.FinalAppUrl} m FinalAppUrl
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FinalAppUrl.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.os_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.os_type != null && m.hasOwnProperty("os_type")) {
                                    d.os_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType[m.os_type] : m.os_type;
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    d.url = m.url;
                                    if (o.oneofs)
                                        d._url = "url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FinalAppUrl to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.FinalAppUrl
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FinalAppUrl.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FinalAppUrl;
                        })();
    
                        common.FrequencyCapEntry = (function() {
    
                            /**
                             * Properties of a FrequencyCapEntry.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IFrequencyCapEntry
                             * @property {google.ads.googleads.v11.common.IFrequencyCapKey|null} [key] FrequencyCapEntry key
                             * @property {number|null} [cap] FrequencyCapEntry cap
                             */
    
                            /**
                             * Constructs a new FrequencyCapEntry.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a FrequencyCapEntry.
                             * @implements IFrequencyCapEntry
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IFrequencyCapEntry=} [p] Properties to set
                             */
                            function FrequencyCapEntry(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FrequencyCapEntry key.
                             * @member {google.ads.googleads.v11.common.IFrequencyCapKey|null|undefined} key
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @instance
                             */
                            FrequencyCapEntry.prototype.key = null;
    
                            /**
                             * FrequencyCapEntry cap.
                             * @member {number|null|undefined} cap
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @instance
                             */
                            FrequencyCapEntry.prototype.cap = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FrequencyCapEntry _cap.
                             * @member {"cap"|undefined} _cap
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @instance
                             */
                            Object.defineProperty(FrequencyCapEntry.prototype, "_cap", {
                                get: $util.oneOfGetter($oneOfFields = ["cap"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FrequencyCapEntry instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.IFrequencyCapEntry=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.FrequencyCapEntry} FrequencyCapEntry instance
                             */
                            FrequencyCapEntry.create = function create(properties) {
                                return new FrequencyCapEntry(properties);
                            };
    
                            /**
                             * Encodes the specified FrequencyCapEntry message. Does not implicitly {@link google.ads.googleads.v11.common.FrequencyCapEntry.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.IFrequencyCapEntry} m FrequencyCapEntry message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapEntry.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.key != null && Object.hasOwnProperty.call(m, "key"))
                                    $root.google.ads.googleads.v11.common.FrequencyCapKey.encode(m.key, w.uint32(10).fork()).ldelim();
                                if (m.cap != null && Object.hasOwnProperty.call(m, "cap"))
                                    w.uint32(24).int32(m.cap);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FrequencyCapEntry message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.FrequencyCapEntry.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.IFrequencyCapEntry} message FrequencyCapEntry message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapEntry.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FrequencyCapEntry message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.FrequencyCapEntry} FrequencyCapEntry
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapEntry.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.FrequencyCapEntry();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.key = $root.google.ads.googleads.v11.common.FrequencyCapKey.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.cap = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FrequencyCapEntry message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.FrequencyCapEntry} FrequencyCapEntry
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapEntry.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FrequencyCapEntry message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FrequencyCapEntry.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.key != null && m.hasOwnProperty("key")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.FrequencyCapKey.verify(m.key);
                                        if (e)
                                            return "key." + e;
                                    }
                                }
                                if (m.cap != null && m.hasOwnProperty("cap")) {
                                    p._cap = 1;
                                    if (!$util.isInteger(m.cap))
                                        return "cap: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FrequencyCapEntry message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.FrequencyCapEntry} FrequencyCapEntry
                             */
                            FrequencyCapEntry.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.FrequencyCapEntry)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.FrequencyCapEntry();
                                if (d.key != null) {
                                    if (typeof d.key !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.FrequencyCapEntry.key: object expected");
                                    m.key = $root.google.ads.googleads.v11.common.FrequencyCapKey.fromObject(d.key);
                                }
                                if (d.cap != null) {
                                    m.cap = d.cap | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FrequencyCapEntry message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @static
                             * @param {google.ads.googleads.v11.common.FrequencyCapEntry} m FrequencyCapEntry
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FrequencyCapEntry.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.key = null;
                                }
                                if (m.key != null && m.hasOwnProperty("key")) {
                                    d.key = $root.google.ads.googleads.v11.common.FrequencyCapKey.toObject(m.key, o);
                                }
                                if (m.cap != null && m.hasOwnProperty("cap")) {
                                    d.cap = m.cap;
                                    if (o.oneofs)
                                        d._cap = "cap";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FrequencyCapEntry to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.FrequencyCapEntry
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FrequencyCapEntry.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FrequencyCapEntry;
                        })();
    
                        common.FrequencyCapKey = (function() {
    
                            /**
                             * Properties of a FrequencyCapKey.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IFrequencyCapKey
                             * @property {google.ads.googleads.v11.enums.FrequencyCapLevelEnum.FrequencyCapLevel|null} [level] FrequencyCapKey level
                             * @property {google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum.FrequencyCapEventType|null} [event_type] FrequencyCapKey event_type
                             * @property {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit|null} [time_unit] FrequencyCapKey time_unit
                             * @property {number|null} [time_length] FrequencyCapKey time_length
                             */
    
                            /**
                             * Constructs a new FrequencyCapKey.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a FrequencyCapKey.
                             * @implements IFrequencyCapKey
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IFrequencyCapKey=} [p] Properties to set
                             */
                            function FrequencyCapKey(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FrequencyCapKey level.
                             * @member {google.ads.googleads.v11.enums.FrequencyCapLevelEnum.FrequencyCapLevel} level
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @instance
                             */
                            FrequencyCapKey.prototype.level = 0;
    
                            /**
                             * FrequencyCapKey event_type.
                             * @member {google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum.FrequencyCapEventType} event_type
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @instance
                             */
                            FrequencyCapKey.prototype.event_type = 0;
    
                            /**
                             * FrequencyCapKey time_unit.
                             * @member {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit} time_unit
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @instance
                             */
                            FrequencyCapKey.prototype.time_unit = 0;
    
                            /**
                             * FrequencyCapKey time_length.
                             * @member {number|null|undefined} time_length
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @instance
                             */
                            FrequencyCapKey.prototype.time_length = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FrequencyCapKey _time_length.
                             * @member {"time_length"|undefined} _time_length
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @instance
                             */
                            Object.defineProperty(FrequencyCapKey.prototype, "_time_length", {
                                get: $util.oneOfGetter($oneOfFields = ["time_length"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FrequencyCapKey instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {google.ads.googleads.v11.common.IFrequencyCapKey=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.FrequencyCapKey} FrequencyCapKey instance
                             */
                            FrequencyCapKey.create = function create(properties) {
                                return new FrequencyCapKey(properties);
                            };
    
                            /**
                             * Encodes the specified FrequencyCapKey message. Does not implicitly {@link google.ads.googleads.v11.common.FrequencyCapKey.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {google.ads.googleads.v11.common.IFrequencyCapKey} m FrequencyCapKey message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapKey.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.level != null && Object.hasOwnProperty.call(m, "level"))
                                    w.uint32(8).int32(m.level);
                                if (m.time_unit != null && Object.hasOwnProperty.call(m, "time_unit"))
                                    w.uint32(16).int32(m.time_unit);
                                if (m.event_type != null && Object.hasOwnProperty.call(m, "event_type"))
                                    w.uint32(24).int32(m.event_type);
                                if (m.time_length != null && Object.hasOwnProperty.call(m, "time_length"))
                                    w.uint32(40).int32(m.time_length);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FrequencyCapKey message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.FrequencyCapKey.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {google.ads.googleads.v11.common.IFrequencyCapKey} message FrequencyCapKey message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapKey.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FrequencyCapKey message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.FrequencyCapKey} FrequencyCapKey
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapKey.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.FrequencyCapKey();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.level = r.int32();
                                        break;
                                    case 3:
                                        m.event_type = r.int32();
                                        break;
                                    case 2:
                                        m.time_unit = r.int32();
                                        break;
                                    case 5:
                                        m.time_length = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FrequencyCapKey message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.FrequencyCapKey} FrequencyCapKey
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapKey.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FrequencyCapKey message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FrequencyCapKey.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    switch (m.level) {
                                    default:
                                        return "level: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.event_type != null && m.hasOwnProperty("event_type")) {
                                    switch (m.event_type) {
                                    default:
                                        return "event_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.time_unit != null && m.hasOwnProperty("time_unit")) {
                                    switch (m.time_unit) {
                                    default:
                                        return "time_unit: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.time_length != null && m.hasOwnProperty("time_length")) {
                                    p._time_length = 1;
                                    if (!$util.isInteger(m.time_length))
                                        return "time_length: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FrequencyCapKey message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.FrequencyCapKey} FrequencyCapKey
                             */
                            FrequencyCapKey.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.FrequencyCapKey)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.FrequencyCapKey();
                                switch (d.level) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.level = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.level = 1;
                                    break;
                                case "AD_GROUP_AD":
                                case 2:
                                    m.level = 2;
                                    break;
                                case "AD_GROUP":
                                case 3:
                                    m.level = 3;
                                    break;
                                case "CAMPAIGN":
                                case 4:
                                    m.level = 4;
                                    break;
                                }
                                switch (d.event_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.event_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.event_type = 1;
                                    break;
                                case "IMPRESSION":
                                case 2:
                                    m.event_type = 2;
                                    break;
                                case "VIDEO_VIEW":
                                case 3:
                                    m.event_type = 3;
                                    break;
                                }
                                switch (d.time_unit) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.time_unit = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.time_unit = 1;
                                    break;
                                case "DAY":
                                case 2:
                                    m.time_unit = 2;
                                    break;
                                case "WEEK":
                                case 3:
                                    m.time_unit = 3;
                                    break;
                                case "MONTH":
                                case 4:
                                    m.time_unit = 4;
                                    break;
                                }
                                if (d.time_length != null) {
                                    m.time_length = d.time_length | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FrequencyCapKey message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @static
                             * @param {google.ads.googleads.v11.common.FrequencyCapKey} m FrequencyCapKey
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FrequencyCapKey.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.level = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.time_unit = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.event_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    d.level = o.enums === String ? $root.google.ads.googleads.v11.enums.FrequencyCapLevelEnum.FrequencyCapLevel[m.level] : m.level;
                                }
                                if (m.time_unit != null && m.hasOwnProperty("time_unit")) {
                                    d.time_unit = o.enums === String ? $root.google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit[m.time_unit] : m.time_unit;
                                }
                                if (m.event_type != null && m.hasOwnProperty("event_type")) {
                                    d.event_type = o.enums === String ? $root.google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum.FrequencyCapEventType[m.event_type] : m.event_type;
                                }
                                if (m.time_length != null && m.hasOwnProperty("time_length")) {
                                    d.time_length = m.time_length;
                                    if (o.oneofs)
                                        d._time_length = "time_length";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FrequencyCapKey to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.FrequencyCapKey
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FrequencyCapKey.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FrequencyCapKey;
                        })();
    
                        common.KeywordPlanHistoricalMetrics = (function() {
    
                            /**
                             * Properties of a KeywordPlanHistoricalMetrics.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordPlanHistoricalMetrics
                             * @property {number|null} [avg_monthly_searches] KeywordPlanHistoricalMetrics avg_monthly_searches
                             * @property {Array.<google.ads.googleads.v11.common.IMonthlySearchVolume>|null} [monthly_search_volumes] KeywordPlanHistoricalMetrics monthly_search_volumes
                             * @property {google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel|null} [competition] KeywordPlanHistoricalMetrics competition
                             * @property {number|null} [competition_index] KeywordPlanHistoricalMetrics competition_index
                             * @property {number|null} [low_top_of_page_bid_micros] KeywordPlanHistoricalMetrics low_top_of_page_bid_micros
                             * @property {number|null} [high_top_of_page_bid_micros] KeywordPlanHistoricalMetrics high_top_of_page_bid_micros
                             * @property {number|null} [average_cpc_micros] KeywordPlanHistoricalMetrics average_cpc_micros
                             */
    
                            /**
                             * Constructs a new KeywordPlanHistoricalMetrics.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordPlanHistoricalMetrics.
                             * @implements IKeywordPlanHistoricalMetrics
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics=} [p] Properties to set
                             */
                            function KeywordPlanHistoricalMetrics(p) {
                                this.monthly_search_volumes = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanHistoricalMetrics avg_monthly_searches.
                             * @member {number|null|undefined} avg_monthly_searches
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanHistoricalMetrics.prototype.avg_monthly_searches = null;
    
                            /**
                             * KeywordPlanHistoricalMetrics monthly_search_volumes.
                             * @member {Array.<google.ads.googleads.v11.common.IMonthlySearchVolume>} monthly_search_volumes
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanHistoricalMetrics.prototype.monthly_search_volumes = $util.emptyArray;
    
                            /**
                             * KeywordPlanHistoricalMetrics competition.
                             * @member {google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel} competition
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanHistoricalMetrics.prototype.competition = 0;
    
                            /**
                             * KeywordPlanHistoricalMetrics competition_index.
                             * @member {number|null|undefined} competition_index
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanHistoricalMetrics.prototype.competition_index = null;
    
                            /**
                             * KeywordPlanHistoricalMetrics low_top_of_page_bid_micros.
                             * @member {number|null|undefined} low_top_of_page_bid_micros
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanHistoricalMetrics.prototype.low_top_of_page_bid_micros = null;
    
                            /**
                             * KeywordPlanHistoricalMetrics high_top_of_page_bid_micros.
                             * @member {number|null|undefined} high_top_of_page_bid_micros
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanHistoricalMetrics.prototype.high_top_of_page_bid_micros = null;
    
                            /**
                             * KeywordPlanHistoricalMetrics average_cpc_micros.
                             * @member {number|null|undefined} average_cpc_micros
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanHistoricalMetrics.prototype.average_cpc_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanHistoricalMetrics _avg_monthly_searches.
                             * @member {"avg_monthly_searches"|undefined} _avg_monthly_searches
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanHistoricalMetrics.prototype, "_avg_monthly_searches", {
                                get: $util.oneOfGetter($oneOfFields = ["avg_monthly_searches"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanHistoricalMetrics _competition_index.
                             * @member {"competition_index"|undefined} _competition_index
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanHistoricalMetrics.prototype, "_competition_index", {
                                get: $util.oneOfGetter($oneOfFields = ["competition_index"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanHistoricalMetrics _low_top_of_page_bid_micros.
                             * @member {"low_top_of_page_bid_micros"|undefined} _low_top_of_page_bid_micros
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanHistoricalMetrics.prototype, "_low_top_of_page_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["low_top_of_page_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanHistoricalMetrics _high_top_of_page_bid_micros.
                             * @member {"high_top_of_page_bid_micros"|undefined} _high_top_of_page_bid_micros
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanHistoricalMetrics.prototype, "_high_top_of_page_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["high_top_of_page_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanHistoricalMetrics _average_cpc_micros.
                             * @member {"average_cpc_micros"|undefined} _average_cpc_micros
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanHistoricalMetrics.prototype, "_average_cpc_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["average_cpc_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanHistoricalMetrics instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics} KeywordPlanHistoricalMetrics instance
                             */
                            KeywordPlanHistoricalMetrics.create = function create(properties) {
                                return new KeywordPlanHistoricalMetrics(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanHistoricalMetrics message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics} m KeywordPlanHistoricalMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanHistoricalMetrics.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.competition != null && Object.hasOwnProperty.call(m, "competition"))
                                    w.uint32(16).int32(m.competition);
                                if (m.monthly_search_volumes != null && m.monthly_search_volumes.length) {
                                    for (var i = 0; i < m.monthly_search_volumes.length; ++i)
                                        $root.google.ads.googleads.v11.common.MonthlySearchVolume.encode(m.monthly_search_volumes[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.avg_monthly_searches != null && Object.hasOwnProperty.call(m, "avg_monthly_searches"))
                                    w.uint32(56).int64(m.avg_monthly_searches);
                                if (m.competition_index != null && Object.hasOwnProperty.call(m, "competition_index"))
                                    w.uint32(64).int64(m.competition_index);
                                if (m.low_top_of_page_bid_micros != null && Object.hasOwnProperty.call(m, "low_top_of_page_bid_micros"))
                                    w.uint32(72).int64(m.low_top_of_page_bid_micros);
                                if (m.high_top_of_page_bid_micros != null && Object.hasOwnProperty.call(m, "high_top_of_page_bid_micros"))
                                    w.uint32(80).int64(m.high_top_of_page_bid_micros);
                                if (m.average_cpc_micros != null && Object.hasOwnProperty.call(m, "average_cpc_micros"))
                                    w.uint32(88).int64(m.average_cpc_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanHistoricalMetrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics} message KeywordPlanHistoricalMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanHistoricalMetrics.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanHistoricalMetrics message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics} KeywordPlanHistoricalMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanHistoricalMetrics.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 7:
                                        m.avg_monthly_searches = r.int64();
                                        break;
                                    case 6:
                                        if (!(m.monthly_search_volumes && m.monthly_search_volumes.length))
                                            m.monthly_search_volumes = [];
                                        m.monthly_search_volumes.push($root.google.ads.googleads.v11.common.MonthlySearchVolume.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.competition = r.int32();
                                        break;
                                    case 8:
                                        m.competition_index = r.int64();
                                        break;
                                    case 9:
                                        m.low_top_of_page_bid_micros = r.int64();
                                        break;
                                    case 10:
                                        m.high_top_of_page_bid_micros = r.int64();
                                        break;
                                    case 11:
                                        m.average_cpc_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanHistoricalMetrics message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics} KeywordPlanHistoricalMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanHistoricalMetrics.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanHistoricalMetrics message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanHistoricalMetrics.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.avg_monthly_searches != null && m.hasOwnProperty("avg_monthly_searches")) {
                                    p._avg_monthly_searches = 1;
                                    if (!$util.isInteger(m.avg_monthly_searches) && !(m.avg_monthly_searches && $util.isInteger(m.avg_monthly_searches.low) && $util.isInteger(m.avg_monthly_searches.high)))
                                        return "avg_monthly_searches: integer|Long expected";
                                }
                                if (m.monthly_search_volumes != null && m.hasOwnProperty("monthly_search_volumes")) {
                                    if (!Array.isArray(m.monthly_search_volumes))
                                        return "monthly_search_volumes: array expected";
                                    for (var i = 0; i < m.monthly_search_volumes.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.MonthlySearchVolume.verify(m.monthly_search_volumes[i]);
                                            if (e)
                                                return "monthly_search_volumes." + e;
                                        }
                                    }
                                }
                                if (m.competition != null && m.hasOwnProperty("competition")) {
                                    switch (m.competition) {
                                    default:
                                        return "competition: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.competition_index != null && m.hasOwnProperty("competition_index")) {
                                    p._competition_index = 1;
                                    if (!$util.isInteger(m.competition_index) && !(m.competition_index && $util.isInteger(m.competition_index.low) && $util.isInteger(m.competition_index.high)))
                                        return "competition_index: integer|Long expected";
                                }
                                if (m.low_top_of_page_bid_micros != null && m.hasOwnProperty("low_top_of_page_bid_micros")) {
                                    p._low_top_of_page_bid_micros = 1;
                                    if (!$util.isInteger(m.low_top_of_page_bid_micros) && !(m.low_top_of_page_bid_micros && $util.isInteger(m.low_top_of_page_bid_micros.low) && $util.isInteger(m.low_top_of_page_bid_micros.high)))
                                        return "low_top_of_page_bid_micros: integer|Long expected";
                                }
                                if (m.high_top_of_page_bid_micros != null && m.hasOwnProperty("high_top_of_page_bid_micros")) {
                                    p._high_top_of_page_bid_micros = 1;
                                    if (!$util.isInteger(m.high_top_of_page_bid_micros) && !(m.high_top_of_page_bid_micros && $util.isInteger(m.high_top_of_page_bid_micros.low) && $util.isInteger(m.high_top_of_page_bid_micros.high)))
                                        return "high_top_of_page_bid_micros: integer|Long expected";
                                }
                                if (m.average_cpc_micros != null && m.hasOwnProperty("average_cpc_micros")) {
                                    p._average_cpc_micros = 1;
                                    if (!$util.isInteger(m.average_cpc_micros) && !(m.average_cpc_micros && $util.isInteger(m.average_cpc_micros.low) && $util.isInteger(m.average_cpc_micros.high)))
                                        return "average_cpc_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanHistoricalMetrics message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics} KeywordPlanHistoricalMetrics
                             */
                            KeywordPlanHistoricalMetrics.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics();
                                if (d.avg_monthly_searches != null) {
                                    if ($util.Long)
                                        (m.avg_monthly_searches = $util.Long.fromValue(d.avg_monthly_searches)).unsigned = false;
                                    else if (typeof d.avg_monthly_searches === "string")
                                        m.avg_monthly_searches = parseInt(d.avg_monthly_searches, 10);
                                    else if (typeof d.avg_monthly_searches === "number")
                                        m.avg_monthly_searches = d.avg_monthly_searches;
                                    else if (typeof d.avg_monthly_searches === "object")
                                        m.avg_monthly_searches = new $util.LongBits(d.avg_monthly_searches.low >>> 0, d.avg_monthly_searches.high >>> 0).toNumber();
                                }
                                if (d.monthly_search_volumes) {
                                    if (!Array.isArray(d.monthly_search_volumes))
                                        throw TypeError(".google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.monthly_search_volumes: array expected");
                                    m.monthly_search_volumes = [];
                                    for (var i = 0; i < d.monthly_search_volumes.length; ++i) {
                                        if (typeof d.monthly_search_volumes[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.monthly_search_volumes: object expected");
                                        m.monthly_search_volumes[i] = $root.google.ads.googleads.v11.common.MonthlySearchVolume.fromObject(d.monthly_search_volumes[i]);
                                    }
                                }
                                switch (d.competition) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.competition = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.competition = 1;
                                    break;
                                case "LOW":
                                case 2:
                                    m.competition = 2;
                                    break;
                                case "MEDIUM":
                                case 3:
                                    m.competition = 3;
                                    break;
                                case "HIGH":
                                case 4:
                                    m.competition = 4;
                                    break;
                                }
                                if (d.competition_index != null) {
                                    if ($util.Long)
                                        (m.competition_index = $util.Long.fromValue(d.competition_index)).unsigned = false;
                                    else if (typeof d.competition_index === "string")
                                        m.competition_index = parseInt(d.competition_index, 10);
                                    else if (typeof d.competition_index === "number")
                                        m.competition_index = d.competition_index;
                                    else if (typeof d.competition_index === "object")
                                        m.competition_index = new $util.LongBits(d.competition_index.low >>> 0, d.competition_index.high >>> 0).toNumber();
                                }
                                if (d.low_top_of_page_bid_micros != null) {
                                    if ($util.Long)
                                        (m.low_top_of_page_bid_micros = $util.Long.fromValue(d.low_top_of_page_bid_micros)).unsigned = false;
                                    else if (typeof d.low_top_of_page_bid_micros === "string")
                                        m.low_top_of_page_bid_micros = parseInt(d.low_top_of_page_bid_micros, 10);
                                    else if (typeof d.low_top_of_page_bid_micros === "number")
                                        m.low_top_of_page_bid_micros = d.low_top_of_page_bid_micros;
                                    else if (typeof d.low_top_of_page_bid_micros === "object")
                                        m.low_top_of_page_bid_micros = new $util.LongBits(d.low_top_of_page_bid_micros.low >>> 0, d.low_top_of_page_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.high_top_of_page_bid_micros != null) {
                                    if ($util.Long)
                                        (m.high_top_of_page_bid_micros = $util.Long.fromValue(d.high_top_of_page_bid_micros)).unsigned = false;
                                    else if (typeof d.high_top_of_page_bid_micros === "string")
                                        m.high_top_of_page_bid_micros = parseInt(d.high_top_of_page_bid_micros, 10);
                                    else if (typeof d.high_top_of_page_bid_micros === "number")
                                        m.high_top_of_page_bid_micros = d.high_top_of_page_bid_micros;
                                    else if (typeof d.high_top_of_page_bid_micros === "object")
                                        m.high_top_of_page_bid_micros = new $util.LongBits(d.high_top_of_page_bid_micros.low >>> 0, d.high_top_of_page_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.average_cpc_micros != null) {
                                    if ($util.Long)
                                        (m.average_cpc_micros = $util.Long.fromValue(d.average_cpc_micros)).unsigned = false;
                                    else if (typeof d.average_cpc_micros === "string")
                                        m.average_cpc_micros = parseInt(d.average_cpc_micros, 10);
                                    else if (typeof d.average_cpc_micros === "number")
                                        m.average_cpc_micros = d.average_cpc_micros;
                                    else if (typeof d.average_cpc_micros === "object")
                                        m.average_cpc_micros = new $util.LongBits(d.average_cpc_micros.low >>> 0, d.average_cpc_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanHistoricalMetrics message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics} m KeywordPlanHistoricalMetrics
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanHistoricalMetrics.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.monthly_search_volumes = [];
                                }
                                if (o.defaults) {
                                    d.competition = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.competition != null && m.hasOwnProperty("competition")) {
                                    d.competition = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel[m.competition] : m.competition;
                                }
                                if (m.monthly_search_volumes && m.monthly_search_volumes.length) {
                                    d.monthly_search_volumes = [];
                                    for (var j = 0; j < m.monthly_search_volumes.length; ++j) {
                                        d.monthly_search_volumes[j] = $root.google.ads.googleads.v11.common.MonthlySearchVolume.toObject(m.monthly_search_volumes[j], o);
                                    }
                                }
                                if (m.avg_monthly_searches != null && m.hasOwnProperty("avg_monthly_searches")) {
                                    if (typeof m.avg_monthly_searches === "number")
                                        d.avg_monthly_searches = o.longs === String ? String(m.avg_monthly_searches) : m.avg_monthly_searches;
                                    else
                                        d.avg_monthly_searches = o.longs === String ? $util.Long.prototype.toString.call(m.avg_monthly_searches) : o.longs === Number ? new $util.LongBits(m.avg_monthly_searches.low >>> 0, m.avg_monthly_searches.high >>> 0).toNumber() : m.avg_monthly_searches;
                                    if (o.oneofs)
                                        d._avg_monthly_searches = "avg_monthly_searches";
                                }
                                if (m.competition_index != null && m.hasOwnProperty("competition_index")) {
                                    if (typeof m.competition_index === "number")
                                        d.competition_index = o.longs === String ? String(m.competition_index) : m.competition_index;
                                    else
                                        d.competition_index = o.longs === String ? $util.Long.prototype.toString.call(m.competition_index) : o.longs === Number ? new $util.LongBits(m.competition_index.low >>> 0, m.competition_index.high >>> 0).toNumber() : m.competition_index;
                                    if (o.oneofs)
                                        d._competition_index = "competition_index";
                                }
                                if (m.low_top_of_page_bid_micros != null && m.hasOwnProperty("low_top_of_page_bid_micros")) {
                                    if (typeof m.low_top_of_page_bid_micros === "number")
                                        d.low_top_of_page_bid_micros = o.longs === String ? String(m.low_top_of_page_bid_micros) : m.low_top_of_page_bid_micros;
                                    else
                                        d.low_top_of_page_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.low_top_of_page_bid_micros) : o.longs === Number ? new $util.LongBits(m.low_top_of_page_bid_micros.low >>> 0, m.low_top_of_page_bid_micros.high >>> 0).toNumber() : m.low_top_of_page_bid_micros;
                                    if (o.oneofs)
                                        d._low_top_of_page_bid_micros = "low_top_of_page_bid_micros";
                                }
                                if (m.high_top_of_page_bid_micros != null && m.hasOwnProperty("high_top_of_page_bid_micros")) {
                                    if (typeof m.high_top_of_page_bid_micros === "number")
                                        d.high_top_of_page_bid_micros = o.longs === String ? String(m.high_top_of_page_bid_micros) : m.high_top_of_page_bid_micros;
                                    else
                                        d.high_top_of_page_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.high_top_of_page_bid_micros) : o.longs === Number ? new $util.LongBits(m.high_top_of_page_bid_micros.low >>> 0, m.high_top_of_page_bid_micros.high >>> 0).toNumber() : m.high_top_of_page_bid_micros;
                                    if (o.oneofs)
                                        d._high_top_of_page_bid_micros = "high_top_of_page_bid_micros";
                                }
                                if (m.average_cpc_micros != null && m.hasOwnProperty("average_cpc_micros")) {
                                    if (typeof m.average_cpc_micros === "number")
                                        d.average_cpc_micros = o.longs === String ? String(m.average_cpc_micros) : m.average_cpc_micros;
                                    else
                                        d.average_cpc_micros = o.longs === String ? $util.Long.prototype.toString.call(m.average_cpc_micros) : o.longs === Number ? new $util.LongBits(m.average_cpc_micros.low >>> 0, m.average_cpc_micros.high >>> 0).toNumber() : m.average_cpc_micros;
                                    if (o.oneofs)
                                        d._average_cpc_micros = "average_cpc_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanHistoricalMetrics to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanHistoricalMetrics.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanHistoricalMetrics;
                        })();
    
                        common.HistoricalMetricsOptions = (function() {
    
                            /**
                             * Properties of a HistoricalMetricsOptions.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IHistoricalMetricsOptions
                             * @property {google.ads.googleads.v11.common.IYearMonthRange|null} [year_month_range] HistoricalMetricsOptions year_month_range
                             * @property {boolean|null} [include_average_cpc] HistoricalMetricsOptions include_average_cpc
                             */
    
                            /**
                             * Constructs a new HistoricalMetricsOptions.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a HistoricalMetricsOptions.
                             * @implements IHistoricalMetricsOptions
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IHistoricalMetricsOptions=} [p] Properties to set
                             */
                            function HistoricalMetricsOptions(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HistoricalMetricsOptions year_month_range.
                             * @member {google.ads.googleads.v11.common.IYearMonthRange|null|undefined} year_month_range
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @instance
                             */
                            HistoricalMetricsOptions.prototype.year_month_range = null;
    
                            /**
                             * HistoricalMetricsOptions include_average_cpc.
                             * @member {boolean} include_average_cpc
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @instance
                             */
                            HistoricalMetricsOptions.prototype.include_average_cpc = false;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * HistoricalMetricsOptions _year_month_range.
                             * @member {"year_month_range"|undefined} _year_month_range
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @instance
                             */
                            Object.defineProperty(HistoricalMetricsOptions.prototype, "_year_month_range", {
                                get: $util.oneOfGetter($oneOfFields = ["year_month_range"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new HistoricalMetricsOptions instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {google.ads.googleads.v11.common.IHistoricalMetricsOptions=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.HistoricalMetricsOptions} HistoricalMetricsOptions instance
                             */
                            HistoricalMetricsOptions.create = function create(properties) {
                                return new HistoricalMetricsOptions(properties);
                            };
    
                            /**
                             * Encodes the specified HistoricalMetricsOptions message. Does not implicitly {@link google.ads.googleads.v11.common.HistoricalMetricsOptions.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {google.ads.googleads.v11.common.IHistoricalMetricsOptions} m HistoricalMetricsOptions message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HistoricalMetricsOptions.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.year_month_range != null && Object.hasOwnProperty.call(m, "year_month_range"))
                                    $root.google.ads.googleads.v11.common.YearMonthRange.encode(m.year_month_range, w.uint32(10).fork()).ldelim();
                                if (m.include_average_cpc != null && Object.hasOwnProperty.call(m, "include_average_cpc"))
                                    w.uint32(16).bool(m.include_average_cpc);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HistoricalMetricsOptions message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.HistoricalMetricsOptions.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {google.ads.googleads.v11.common.IHistoricalMetricsOptions} message HistoricalMetricsOptions message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HistoricalMetricsOptions.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HistoricalMetricsOptions message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.HistoricalMetricsOptions} HistoricalMetricsOptions
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HistoricalMetricsOptions.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.HistoricalMetricsOptions();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.year_month_range = $root.google.ads.googleads.v11.common.YearMonthRange.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.include_average_cpc = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HistoricalMetricsOptions message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.HistoricalMetricsOptions} HistoricalMetricsOptions
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HistoricalMetricsOptions.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HistoricalMetricsOptions message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HistoricalMetricsOptions.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.year_month_range != null && m.hasOwnProperty("year_month_range")) {
                                    p._year_month_range = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YearMonthRange.verify(m.year_month_range);
                                        if (e)
                                            return "year_month_range." + e;
                                    }
                                }
                                if (m.include_average_cpc != null && m.hasOwnProperty("include_average_cpc")) {
                                    if (typeof m.include_average_cpc !== "boolean")
                                        return "include_average_cpc: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HistoricalMetricsOptions message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.HistoricalMetricsOptions} HistoricalMetricsOptions
                             */
                            HistoricalMetricsOptions.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.HistoricalMetricsOptions)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.HistoricalMetricsOptions();
                                if (d.year_month_range != null) {
                                    if (typeof d.year_month_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.HistoricalMetricsOptions.year_month_range: object expected");
                                    m.year_month_range = $root.google.ads.googleads.v11.common.YearMonthRange.fromObject(d.year_month_range);
                                }
                                if (d.include_average_cpc != null) {
                                    m.include_average_cpc = Boolean(d.include_average_cpc);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HistoricalMetricsOptions message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @static
                             * @param {google.ads.googleads.v11.common.HistoricalMetricsOptions} m HistoricalMetricsOptions
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HistoricalMetricsOptions.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.include_average_cpc = false;
                                }
                                if (m.year_month_range != null && m.hasOwnProperty("year_month_range")) {
                                    d.year_month_range = $root.google.ads.googleads.v11.common.YearMonthRange.toObject(m.year_month_range, o);
                                    if (o.oneofs)
                                        d._year_month_range = "year_month_range";
                                }
                                if (m.include_average_cpc != null && m.hasOwnProperty("include_average_cpc")) {
                                    d.include_average_cpc = m.include_average_cpc;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HistoricalMetricsOptions to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.HistoricalMetricsOptions
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HistoricalMetricsOptions.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HistoricalMetricsOptions;
                        })();
    
                        common.MonthlySearchVolume = (function() {
    
                            /**
                             * Properties of a MonthlySearchVolume.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMonthlySearchVolume
                             * @property {number|null} [year] MonthlySearchVolume year
                             * @property {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear|null} [month] MonthlySearchVolume month
                             * @property {number|null} [monthly_searches] MonthlySearchVolume monthly_searches
                             */
    
                            /**
                             * Constructs a new MonthlySearchVolume.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MonthlySearchVolume.
                             * @implements IMonthlySearchVolume
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMonthlySearchVolume=} [p] Properties to set
                             */
                            function MonthlySearchVolume(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MonthlySearchVolume year.
                             * @member {number|null|undefined} year
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @instance
                             */
                            MonthlySearchVolume.prototype.year = null;
    
                            /**
                             * MonthlySearchVolume month.
                             * @member {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear} month
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @instance
                             */
                            MonthlySearchVolume.prototype.month = 0;
    
                            /**
                             * MonthlySearchVolume monthly_searches.
                             * @member {number|null|undefined} monthly_searches
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @instance
                             */
                            MonthlySearchVolume.prototype.monthly_searches = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MonthlySearchVolume _year.
                             * @member {"year"|undefined} _year
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @instance
                             */
                            Object.defineProperty(MonthlySearchVolume.prototype, "_year", {
                                get: $util.oneOfGetter($oneOfFields = ["year"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MonthlySearchVolume _monthly_searches.
                             * @member {"monthly_searches"|undefined} _monthly_searches
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @instance
                             */
                            Object.defineProperty(MonthlySearchVolume.prototype, "_monthly_searches", {
                                get: $util.oneOfGetter($oneOfFields = ["monthly_searches"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MonthlySearchVolume instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {google.ads.googleads.v11.common.IMonthlySearchVolume=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MonthlySearchVolume} MonthlySearchVolume instance
                             */
                            MonthlySearchVolume.create = function create(properties) {
                                return new MonthlySearchVolume(properties);
                            };
    
                            /**
                             * Encodes the specified MonthlySearchVolume message. Does not implicitly {@link google.ads.googleads.v11.common.MonthlySearchVolume.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {google.ads.googleads.v11.common.IMonthlySearchVolume} m MonthlySearchVolume message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MonthlySearchVolume.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.month != null && Object.hasOwnProperty.call(m, "month"))
                                    w.uint32(16).int32(m.month);
                                if (m.year != null && Object.hasOwnProperty.call(m, "year"))
                                    w.uint32(32).int64(m.year);
                                if (m.monthly_searches != null && Object.hasOwnProperty.call(m, "monthly_searches"))
                                    w.uint32(40).int64(m.monthly_searches);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MonthlySearchVolume message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MonthlySearchVolume.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {google.ads.googleads.v11.common.IMonthlySearchVolume} message MonthlySearchVolume message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MonthlySearchVolume.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MonthlySearchVolume message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MonthlySearchVolume} MonthlySearchVolume
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MonthlySearchVolume.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MonthlySearchVolume();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.year = r.int64();
                                        break;
                                    case 2:
                                        m.month = r.int32();
                                        break;
                                    case 5:
                                        m.monthly_searches = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MonthlySearchVolume message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MonthlySearchVolume} MonthlySearchVolume
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MonthlySearchVolume.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MonthlySearchVolume message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MonthlySearchVolume.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.year != null && m.hasOwnProperty("year")) {
                                    p._year = 1;
                                    if (!$util.isInteger(m.year) && !(m.year && $util.isInteger(m.year.low) && $util.isInteger(m.year.high)))
                                        return "year: integer|Long expected";
                                }
                                if (m.month != null && m.hasOwnProperty("month")) {
                                    switch (m.month) {
                                    default:
                                        return "month: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.monthly_searches != null && m.hasOwnProperty("monthly_searches")) {
                                    p._monthly_searches = 1;
                                    if (!$util.isInteger(m.monthly_searches) && !(m.monthly_searches && $util.isInteger(m.monthly_searches.low) && $util.isInteger(m.monthly_searches.high)))
                                        return "monthly_searches: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MonthlySearchVolume message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MonthlySearchVolume} MonthlySearchVolume
                             */
                            MonthlySearchVolume.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MonthlySearchVolume)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MonthlySearchVolume();
                                if (d.year != null) {
                                    if ($util.Long)
                                        (m.year = $util.Long.fromValue(d.year)).unsigned = false;
                                    else if (typeof d.year === "string")
                                        m.year = parseInt(d.year, 10);
                                    else if (typeof d.year === "number")
                                        m.year = d.year;
                                    else if (typeof d.year === "object")
                                        m.year = new $util.LongBits(d.year.low >>> 0, d.year.high >>> 0).toNumber();
                                }
                                switch (d.month) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.month = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.month = 1;
                                    break;
                                case "JANUARY":
                                case 2:
                                    m.month = 2;
                                    break;
                                case "FEBRUARY":
                                case 3:
                                    m.month = 3;
                                    break;
                                case "MARCH":
                                case 4:
                                    m.month = 4;
                                    break;
                                case "APRIL":
                                case 5:
                                    m.month = 5;
                                    break;
                                case "MAY":
                                case 6:
                                    m.month = 6;
                                    break;
                                case "JUNE":
                                case 7:
                                    m.month = 7;
                                    break;
                                case "JULY":
                                case 8:
                                    m.month = 8;
                                    break;
                                case "AUGUST":
                                case 9:
                                    m.month = 9;
                                    break;
                                case "SEPTEMBER":
                                case 10:
                                    m.month = 10;
                                    break;
                                case "OCTOBER":
                                case 11:
                                    m.month = 11;
                                    break;
                                case "NOVEMBER":
                                case 12:
                                    m.month = 12;
                                    break;
                                case "DECEMBER":
                                case 13:
                                    m.month = 13;
                                    break;
                                }
                                if (d.monthly_searches != null) {
                                    if ($util.Long)
                                        (m.monthly_searches = $util.Long.fromValue(d.monthly_searches)).unsigned = false;
                                    else if (typeof d.monthly_searches === "string")
                                        m.monthly_searches = parseInt(d.monthly_searches, 10);
                                    else if (typeof d.monthly_searches === "number")
                                        m.monthly_searches = d.monthly_searches;
                                    else if (typeof d.monthly_searches === "object")
                                        m.monthly_searches = new $util.LongBits(d.monthly_searches.low >>> 0, d.monthly_searches.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MonthlySearchVolume message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @static
                             * @param {google.ads.googleads.v11.common.MonthlySearchVolume} m MonthlySearchVolume
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MonthlySearchVolume.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.month = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.month != null && m.hasOwnProperty("month")) {
                                    d.month = o.enums === String ? $root.google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear[m.month] : m.month;
                                }
                                if (m.year != null && m.hasOwnProperty("year")) {
                                    if (typeof m.year === "number")
                                        d.year = o.longs === String ? String(m.year) : m.year;
                                    else
                                        d.year = o.longs === String ? $util.Long.prototype.toString.call(m.year) : o.longs === Number ? new $util.LongBits(m.year.low >>> 0, m.year.high >>> 0).toNumber() : m.year;
                                    if (o.oneofs)
                                        d._year = "year";
                                }
                                if (m.monthly_searches != null && m.hasOwnProperty("monthly_searches")) {
                                    if (typeof m.monthly_searches === "number")
                                        d.monthly_searches = o.longs === String ? String(m.monthly_searches) : m.monthly_searches;
                                    else
                                        d.monthly_searches = o.longs === String ? $util.Long.prototype.toString.call(m.monthly_searches) : o.longs === Number ? new $util.LongBits(m.monthly_searches.low >>> 0, m.monthly_searches.high >>> 0).toNumber() : m.monthly_searches;
                                    if (o.oneofs)
                                        d._monthly_searches = "monthly_searches";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MonthlySearchVolume to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MonthlySearchVolume
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MonthlySearchVolume.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MonthlySearchVolume;
                        })();
    
                        common.KeywordPlanAggregateMetrics = (function() {
    
                            /**
                             * Properties of a KeywordPlanAggregateMetrics.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordPlanAggregateMetrics
                             * @property {Array.<google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType>|null} [aggregate_metric_types] KeywordPlanAggregateMetrics aggregate_metric_types
                             */
    
                            /**
                             * Constructs a new KeywordPlanAggregateMetrics.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordPlanAggregateMetrics.
                             * @implements IKeywordPlanAggregateMetrics
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics=} [p] Properties to set
                             */
                            function KeywordPlanAggregateMetrics(p) {
                                this.aggregate_metric_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanAggregateMetrics aggregate_metric_types.
                             * @member {Array.<google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType>} aggregate_metric_types
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @instance
                             */
                            KeywordPlanAggregateMetrics.prototype.aggregate_metric_types = $util.emptyArray;
    
                            /**
                             * Creates a new KeywordPlanAggregateMetrics instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetrics} KeywordPlanAggregateMetrics instance
                             */
                            KeywordPlanAggregateMetrics.create = function create(properties) {
                                return new KeywordPlanAggregateMetrics(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAggregateMetrics message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics} m KeywordPlanAggregateMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAggregateMetrics.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.aggregate_metric_types != null && m.aggregate_metric_types.length) {
                                    w.uint32(10).fork();
                                    for (var i = 0; i < m.aggregate_metric_types.length; ++i)
                                        w.int32(m.aggregate_metric_types[i]);
                                    w.ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAggregateMetrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics} message KeywordPlanAggregateMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAggregateMetrics.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAggregateMetrics message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetrics} KeywordPlanAggregateMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAggregateMetrics.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.aggregate_metric_types && m.aggregate_metric_types.length))
                                            m.aggregate_metric_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.aggregate_metric_types.push(r.int32());
                                        } else
                                            m.aggregate_metric_types.push(r.int32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAggregateMetrics message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetrics} KeywordPlanAggregateMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAggregateMetrics.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAggregateMetrics message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAggregateMetrics.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.aggregate_metric_types != null && m.hasOwnProperty("aggregate_metric_types")) {
                                    if (!Array.isArray(m.aggregate_metric_types))
                                        return "aggregate_metric_types: array expected";
                                    for (var i = 0; i < m.aggregate_metric_types.length; ++i) {
                                        switch (m.aggregate_metric_types[i]) {
                                        default:
                                            return "aggregate_metric_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                            break;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAggregateMetrics message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetrics} KeywordPlanAggregateMetrics
                             */
                            KeywordPlanAggregateMetrics.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics();
                                if (d.aggregate_metric_types) {
                                    if (!Array.isArray(d.aggregate_metric_types))
                                        throw TypeError(".google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.aggregate_metric_types: array expected");
                                    m.aggregate_metric_types = [];
                                    for (var i = 0; i < d.aggregate_metric_types.length; ++i) {
                                        switch (d.aggregate_metric_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.aggregate_metric_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.aggregate_metric_types[i] = 1;
                                            break;
                                        case "DEVICE":
                                        case 2:
                                            m.aggregate_metric_types[i] = 2;
                                            break;
                                        }
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAggregateMetrics message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordPlanAggregateMetrics} m KeywordPlanAggregateMetrics
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAggregateMetrics.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.aggregate_metric_types = [];
                                }
                                if (m.aggregate_metric_types && m.aggregate_metric_types.length) {
                                    d.aggregate_metric_types = [];
                                    for (var j = 0; j < m.aggregate_metric_types.length; ++j) {
                                        d.aggregate_metric_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType[m.aggregate_metric_types[j]] : m.aggregate_metric_types[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanAggregateMetrics to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetrics
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAggregateMetrics.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanAggregateMetrics;
                        })();
    
                        common.KeywordPlanAggregateMetricResults = (function() {
    
                            /**
                             * Properties of a KeywordPlanAggregateMetricResults.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordPlanAggregateMetricResults
                             * @property {Array.<google.ads.googleads.v11.common.IKeywordPlanDeviceSearches>|null} [device_searches] KeywordPlanAggregateMetricResults device_searches
                             */
    
                            /**
                             * Constructs a new KeywordPlanAggregateMetricResults.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordPlanAggregateMetricResults.
                             * @implements IKeywordPlanAggregateMetricResults
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults=} [p] Properties to set
                             */
                            function KeywordPlanAggregateMetricResults(p) {
                                this.device_searches = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanAggregateMetricResults device_searches.
                             * @member {Array.<google.ads.googleads.v11.common.IKeywordPlanDeviceSearches>} device_searches
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @instance
                             */
                            KeywordPlanAggregateMetricResults.prototype.device_searches = $util.emptyArray;
    
                            /**
                             * Creates a new KeywordPlanAggregateMetricResults instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults} KeywordPlanAggregateMetricResults instance
                             */
                            KeywordPlanAggregateMetricResults.create = function create(properties) {
                                return new KeywordPlanAggregateMetricResults(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAggregateMetricResults message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults} m KeywordPlanAggregateMetricResults message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAggregateMetricResults.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.device_searches != null && m.device_searches.length) {
                                    for (var i = 0; i < m.device_searches.length; ++i)
                                        $root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches.encode(m.device_searches[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAggregateMetricResults message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults} message KeywordPlanAggregateMetricResults message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAggregateMetricResults.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAggregateMetricResults message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults} KeywordPlanAggregateMetricResults
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAggregateMetricResults.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.device_searches && m.device_searches.length))
                                            m.device_searches = [];
                                        m.device_searches.push($root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAggregateMetricResults message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults} KeywordPlanAggregateMetricResults
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAggregateMetricResults.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAggregateMetricResults message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAggregateMetricResults.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.device_searches != null && m.hasOwnProperty("device_searches")) {
                                    if (!Array.isArray(m.device_searches))
                                        return "device_searches: array expected";
                                    for (var i = 0; i < m.device_searches.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches.verify(m.device_searches[i]);
                                            if (e)
                                                return "device_searches." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAggregateMetricResults message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults} KeywordPlanAggregateMetricResults
                             */
                            KeywordPlanAggregateMetricResults.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults();
                                if (d.device_searches) {
                                    if (!Array.isArray(d.device_searches))
                                        throw TypeError(".google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.device_searches: array expected");
                                    m.device_searches = [];
                                    for (var i = 0; i < d.device_searches.length; ++i) {
                                        if (typeof d.device_searches[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.device_searches: object expected");
                                        m.device_searches[i] = $root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches.fromObject(d.device_searches[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAggregateMetricResults message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults} m KeywordPlanAggregateMetricResults
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAggregateMetricResults.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.device_searches = [];
                                }
                                if (m.device_searches && m.device_searches.length) {
                                    d.device_searches = [];
                                    for (var j = 0; j < m.device_searches.length; ++j) {
                                        d.device_searches[j] = $root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches.toObject(m.device_searches[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanAggregateMetricResults to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAggregateMetricResults.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanAggregateMetricResults;
                        })();
    
                        common.KeywordPlanDeviceSearches = (function() {
    
                            /**
                             * Properties of a KeywordPlanDeviceSearches.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordPlanDeviceSearches
                             * @property {google.ads.googleads.v11.enums.DeviceEnum.Device|null} [device] KeywordPlanDeviceSearches device
                             * @property {number|null} [search_count] KeywordPlanDeviceSearches search_count
                             */
    
                            /**
                             * Constructs a new KeywordPlanDeviceSearches.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordPlanDeviceSearches.
                             * @implements IKeywordPlanDeviceSearches
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordPlanDeviceSearches=} [p] Properties to set
                             */
                            function KeywordPlanDeviceSearches(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanDeviceSearches device.
                             * @member {google.ads.googleads.v11.enums.DeviceEnum.Device} device
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @instance
                             */
                            KeywordPlanDeviceSearches.prototype.device = 0;
    
                            /**
                             * KeywordPlanDeviceSearches search_count.
                             * @member {number|null|undefined} search_count
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @instance
                             */
                            KeywordPlanDeviceSearches.prototype.search_count = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanDeviceSearches _search_count.
                             * @member {"search_count"|undefined} _search_count
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanDeviceSearches.prototype, "_search_count", {
                                get: $util.oneOfGetter($oneOfFields = ["search_count"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanDeviceSearches instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanDeviceSearches=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordPlanDeviceSearches} KeywordPlanDeviceSearches instance
                             */
                            KeywordPlanDeviceSearches.create = function create(properties) {
                                return new KeywordPlanDeviceSearches(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanDeviceSearches message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanDeviceSearches.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanDeviceSearches} m KeywordPlanDeviceSearches message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanDeviceSearches.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    w.uint32(8).int32(m.device);
                                if (m.search_count != null && Object.hasOwnProperty.call(m, "search_count"))
                                    w.uint32(16).int64(m.search_count);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanDeviceSearches message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordPlanDeviceSearches.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordPlanDeviceSearches} message KeywordPlanDeviceSearches message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanDeviceSearches.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanDeviceSearches message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordPlanDeviceSearches} KeywordPlanDeviceSearches
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanDeviceSearches.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.device = r.int32();
                                        break;
                                    case 2:
                                        m.search_count = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanDeviceSearches message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordPlanDeviceSearches} KeywordPlanDeviceSearches
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanDeviceSearches.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanDeviceSearches message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanDeviceSearches.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    switch (m.device) {
                                    default:
                                        return "device: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.search_count != null && m.hasOwnProperty("search_count")) {
                                    p._search_count = 1;
                                    if (!$util.isInteger(m.search_count) && !(m.search_count && $util.isInteger(m.search_count.low) && $util.isInteger(m.search_count.high)))
                                        return "search_count: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanDeviceSearches message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordPlanDeviceSearches} KeywordPlanDeviceSearches
                             */
                            KeywordPlanDeviceSearches.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordPlanDeviceSearches();
                                switch (d.device) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device = 2;
                                    break;
                                case "TABLET":
                                case 3:
                                    m.device = 3;
                                    break;
                                case "DESKTOP":
                                case 4:
                                    m.device = 4;
                                    break;
                                case "CONNECTED_TV":
                                case 6:
                                    m.device = 6;
                                    break;
                                case "OTHER":
                                case 5:
                                    m.device = 5;
                                    break;
                                }
                                if (d.search_count != null) {
                                    if ($util.Long)
                                        (m.search_count = $util.Long.fromValue(d.search_count)).unsigned = false;
                                    else if (typeof d.search_count === "string")
                                        m.search_count = parseInt(d.search_count, 10);
                                    else if (typeof d.search_count === "number")
                                        m.search_count = d.search_count;
                                    else if (typeof d.search_count === "object")
                                        m.search_count = new $util.LongBits(d.search_count.low >>> 0, d.search_count.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanDeviceSearches message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordPlanDeviceSearches} m KeywordPlanDeviceSearches
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanDeviceSearches.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.device = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.DeviceEnum.Device[m.device] : m.device;
                                }
                                if (m.search_count != null && m.hasOwnProperty("search_count")) {
                                    if (typeof m.search_count === "number")
                                        d.search_count = o.longs === String ? String(m.search_count) : m.search_count;
                                    else
                                        d.search_count = o.longs === String ? $util.Long.prototype.toString.call(m.search_count) : o.longs === Number ? new $util.LongBits(m.search_count.low >>> 0, m.search_count.high >>> 0).toNumber() : m.search_count;
                                    if (o.oneofs)
                                        d._search_count = "search_count";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanDeviceSearches to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordPlanDeviceSearches
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanDeviceSearches.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanDeviceSearches;
                        })();
    
                        common.KeywordAnnotations = (function() {
    
                            /**
                             * Properties of a KeywordAnnotations.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordAnnotations
                             * @property {Array.<google.ads.googleads.v11.common.IKeywordConcept>|null} [concepts] KeywordAnnotations concepts
                             */
    
                            /**
                             * Constructs a new KeywordAnnotations.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordAnnotations.
                             * @implements IKeywordAnnotations
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordAnnotations=} [p] Properties to set
                             */
                            function KeywordAnnotations(p) {
                                this.concepts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordAnnotations concepts.
                             * @member {Array.<google.ads.googleads.v11.common.IKeywordConcept>} concepts
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @instance
                             */
                            KeywordAnnotations.prototype.concepts = $util.emptyArray;
    
                            /**
                             * Creates a new KeywordAnnotations instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordAnnotations=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordAnnotations} KeywordAnnotations instance
                             */
                            KeywordAnnotations.create = function create(properties) {
                                return new KeywordAnnotations(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordAnnotations message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordAnnotations.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordAnnotations} m KeywordAnnotations message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordAnnotations.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.concepts != null && m.concepts.length) {
                                    for (var i = 0; i < m.concepts.length; ++i)
                                        $root.google.ads.googleads.v11.common.KeywordConcept.encode(m.concepts[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordAnnotations message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordAnnotations.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordAnnotations} message KeywordAnnotations message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordAnnotations.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordAnnotations message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordAnnotations} KeywordAnnotations
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordAnnotations.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordAnnotations();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.concepts && m.concepts.length))
                                            m.concepts = [];
                                        m.concepts.push($root.google.ads.googleads.v11.common.KeywordConcept.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordAnnotations message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordAnnotations} KeywordAnnotations
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordAnnotations.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordAnnotations message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordAnnotations.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.concepts != null && m.hasOwnProperty("concepts")) {
                                    if (!Array.isArray(m.concepts))
                                        return "concepts: array expected";
                                    for (var i = 0; i < m.concepts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.KeywordConcept.verify(m.concepts[i]);
                                            if (e)
                                                return "concepts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordAnnotations message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordAnnotations} KeywordAnnotations
                             */
                            KeywordAnnotations.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordAnnotations)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordAnnotations();
                                if (d.concepts) {
                                    if (!Array.isArray(d.concepts))
                                        throw TypeError(".google.ads.googleads.v11.common.KeywordAnnotations.concepts: array expected");
                                    m.concepts = [];
                                    for (var i = 0; i < d.concepts.length; ++i) {
                                        if (typeof d.concepts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.KeywordAnnotations.concepts: object expected");
                                        m.concepts[i] = $root.google.ads.googleads.v11.common.KeywordConcept.fromObject(d.concepts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordAnnotations message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordAnnotations} m KeywordAnnotations
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordAnnotations.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.concepts = [];
                                }
                                if (m.concepts && m.concepts.length) {
                                    d.concepts = [];
                                    for (var j = 0; j < m.concepts.length; ++j) {
                                        d.concepts[j] = $root.google.ads.googleads.v11.common.KeywordConcept.toObject(m.concepts[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordAnnotations to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordAnnotations
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordAnnotations.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordAnnotations;
                        })();
    
                        common.KeywordConcept = (function() {
    
                            /**
                             * Properties of a KeywordConcept.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeywordConcept
                             * @property {string|null} [name] KeywordConcept name
                             * @property {google.ads.googleads.v11.common.IConceptGroup|null} [concept_group] KeywordConcept concept_group
                             */
    
                            /**
                             * Constructs a new KeywordConcept.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a KeywordConcept.
                             * @implements IKeywordConcept
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeywordConcept=} [p] Properties to set
                             */
                            function KeywordConcept(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordConcept name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @instance
                             */
                            KeywordConcept.prototype.name = "";
    
                            /**
                             * KeywordConcept concept_group.
                             * @member {google.ads.googleads.v11.common.IConceptGroup|null|undefined} concept_group
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @instance
                             */
                            KeywordConcept.prototype.concept_group = null;
    
                            /**
                             * Creates a new KeywordConcept instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordConcept=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.KeywordConcept} KeywordConcept instance
                             */
                            KeywordConcept.create = function create(properties) {
                                return new KeywordConcept(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordConcept message. Does not implicitly {@link google.ads.googleads.v11.common.KeywordConcept.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordConcept} m KeywordConcept message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordConcept.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(10).string(m.name);
                                if (m.concept_group != null && Object.hasOwnProperty.call(m, "concept_group"))
                                    $root.google.ads.googleads.v11.common.ConceptGroup.encode(m.concept_group, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordConcept message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.KeywordConcept.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeywordConcept} message KeywordConcept message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordConcept.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordConcept message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.KeywordConcept} KeywordConcept
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordConcept.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.KeywordConcept();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.name = r.string();
                                        break;
                                    case 2:
                                        m.concept_group = $root.google.ads.googleads.v11.common.ConceptGroup.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordConcept message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.KeywordConcept} KeywordConcept
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordConcept.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordConcept message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordConcept.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.concept_group != null && m.hasOwnProperty("concept_group")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ConceptGroup.verify(m.concept_group);
                                        if (e)
                                            return "concept_group." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordConcept message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.KeywordConcept} KeywordConcept
                             */
                            KeywordConcept.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.KeywordConcept)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.KeywordConcept();
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.concept_group != null) {
                                    if (typeof d.concept_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.KeywordConcept.concept_group: object expected");
                                    m.concept_group = $root.google.ads.googleads.v11.common.ConceptGroup.fromObject(d.concept_group);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordConcept message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @static
                             * @param {google.ads.googleads.v11.common.KeywordConcept} m KeywordConcept
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordConcept.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.name = "";
                                    d.concept_group = null;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.concept_group != null && m.hasOwnProperty("concept_group")) {
                                    d.concept_group = $root.google.ads.googleads.v11.common.ConceptGroup.toObject(m.concept_group, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordConcept to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.KeywordConcept
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordConcept.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordConcept;
                        })();
    
                        common.ConceptGroup = (function() {
    
                            /**
                             * Properties of a ConceptGroup.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IConceptGroup
                             * @property {string|null} [name] ConceptGroup name
                             * @property {google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType|null} [type] ConceptGroup type
                             */
    
                            /**
                             * Constructs a new ConceptGroup.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ConceptGroup.
                             * @implements IConceptGroup
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IConceptGroup=} [p] Properties to set
                             */
                            function ConceptGroup(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConceptGroup name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @instance
                             */
                            ConceptGroup.prototype.name = "";
    
                            /**
                             * ConceptGroup type.
                             * @member {google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType} type
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @instance
                             */
                            ConceptGroup.prototype.type = 0;
    
                            /**
                             * Creates a new ConceptGroup instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {google.ads.googleads.v11.common.IConceptGroup=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ConceptGroup} ConceptGroup instance
                             */
                            ConceptGroup.create = function create(properties) {
                                return new ConceptGroup(properties);
                            };
    
                            /**
                             * Encodes the specified ConceptGroup message. Does not implicitly {@link google.ads.googleads.v11.common.ConceptGroup.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {google.ads.googleads.v11.common.IConceptGroup} m ConceptGroup message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConceptGroup.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(10).string(m.name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(16).int32(m.type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConceptGroup message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ConceptGroup.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {google.ads.googleads.v11.common.IConceptGroup} message ConceptGroup message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConceptGroup.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConceptGroup message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ConceptGroup} ConceptGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConceptGroup.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ConceptGroup();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.name = r.string();
                                        break;
                                    case 2:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConceptGroup message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ConceptGroup} ConceptGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConceptGroup.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConceptGroup message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConceptGroup.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConceptGroup message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ConceptGroup} ConceptGroup
                             */
                            ConceptGroup.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ConceptGroup)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ConceptGroup();
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "BRAND":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "OTHER_BRANDS":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "NON_BRAND":
                                case 4:
                                    m.type = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConceptGroup message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @static
                             * @param {google.ads.googleads.v11.common.ConceptGroup} m ConceptGroup
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConceptGroup.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType[m.type] : m.type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConceptGroup to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ConceptGroup
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConceptGroup.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConceptGroup;
                        })();
    
                        common.MatchingFunction = (function() {
    
                            /**
                             * Properties of a MatchingFunction.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMatchingFunction
                             * @property {string|null} [function_string] MatchingFunction function_string
                             * @property {google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum.MatchingFunctionOperator|null} [operator] MatchingFunction operator
                             * @property {Array.<google.ads.googleads.v11.common.IOperand>|null} [left_operands] MatchingFunction left_operands
                             * @property {Array.<google.ads.googleads.v11.common.IOperand>|null} [right_operands] MatchingFunction right_operands
                             */
    
                            /**
                             * Constructs a new MatchingFunction.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MatchingFunction.
                             * @implements IMatchingFunction
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMatchingFunction=} [p] Properties to set
                             */
                            function MatchingFunction(p) {
                                this.left_operands = [];
                                this.right_operands = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MatchingFunction function_string.
                             * @member {string|null|undefined} function_string
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @instance
                             */
                            MatchingFunction.prototype.function_string = null;
    
                            /**
                             * MatchingFunction operator.
                             * @member {google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum.MatchingFunctionOperator} operator
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @instance
                             */
                            MatchingFunction.prototype.operator = 0;
    
                            /**
                             * MatchingFunction left_operands.
                             * @member {Array.<google.ads.googleads.v11.common.IOperand>} left_operands
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @instance
                             */
                            MatchingFunction.prototype.left_operands = $util.emptyArray;
    
                            /**
                             * MatchingFunction right_operands.
                             * @member {Array.<google.ads.googleads.v11.common.IOperand>} right_operands
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @instance
                             */
                            MatchingFunction.prototype.right_operands = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MatchingFunction _function_string.
                             * @member {"function_string"|undefined} _function_string
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @instance
                             */
                            Object.defineProperty(MatchingFunction.prototype, "_function_string", {
                                get: $util.oneOfGetter($oneOfFields = ["function_string"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MatchingFunction instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {google.ads.googleads.v11.common.IMatchingFunction=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MatchingFunction} MatchingFunction instance
                             */
                            MatchingFunction.create = function create(properties) {
                                return new MatchingFunction(properties);
                            };
    
                            /**
                             * Encodes the specified MatchingFunction message. Does not implicitly {@link google.ads.googleads.v11.common.MatchingFunction.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {google.ads.googleads.v11.common.IMatchingFunction} m MatchingFunction message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MatchingFunction.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.left_operands != null && m.left_operands.length) {
                                    for (var i = 0; i < m.left_operands.length; ++i)
                                        $root.google.ads.googleads.v11.common.Operand.encode(m.left_operands[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.right_operands != null && m.right_operands.length) {
                                    for (var i = 0; i < m.right_operands.length; ++i)
                                        $root.google.ads.googleads.v11.common.Operand.encode(m.right_operands[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(32).int32(m.operator);
                                if (m.function_string != null && Object.hasOwnProperty.call(m, "function_string"))
                                    w.uint32(42).string(m.function_string);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MatchingFunction message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MatchingFunction.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {google.ads.googleads.v11.common.IMatchingFunction} message MatchingFunction message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MatchingFunction.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MatchingFunction message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MatchingFunction} MatchingFunction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MatchingFunction.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MatchingFunction();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.function_string = r.string();
                                        break;
                                    case 4:
                                        m.operator = r.int32();
                                        break;
                                    case 2:
                                        if (!(m.left_operands && m.left_operands.length))
                                            m.left_operands = [];
                                        m.left_operands.push($root.google.ads.googleads.v11.common.Operand.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.right_operands && m.right_operands.length))
                                            m.right_operands = [];
                                        m.right_operands.push($root.google.ads.googleads.v11.common.Operand.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MatchingFunction message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MatchingFunction} MatchingFunction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MatchingFunction.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MatchingFunction message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MatchingFunction.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.function_string != null && m.hasOwnProperty("function_string")) {
                                    p._function_string = 1;
                                    if (!$util.isString(m.function_string))
                                        return "function_string: string expected";
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.left_operands != null && m.hasOwnProperty("left_operands")) {
                                    if (!Array.isArray(m.left_operands))
                                        return "left_operands: array expected";
                                    for (var i = 0; i < m.left_operands.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.Operand.verify(m.left_operands[i]);
                                            if (e)
                                                return "left_operands." + e;
                                        }
                                    }
                                }
                                if (m.right_operands != null && m.hasOwnProperty("right_operands")) {
                                    if (!Array.isArray(m.right_operands))
                                        return "right_operands: array expected";
                                    for (var i = 0; i < m.right_operands.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.Operand.verify(m.right_operands[i]);
                                            if (e)
                                                return "right_operands." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MatchingFunction message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MatchingFunction} MatchingFunction
                             */
                            MatchingFunction.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MatchingFunction)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MatchingFunction();
                                if (d.function_string != null) {
                                    m.function_string = String(d.function_string);
                                }
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "IN":
                                case 2:
                                    m.operator = 2;
                                    break;
                                case "IDENTITY":
                                case 3:
                                    m.operator = 3;
                                    break;
                                case "EQUALS":
                                case 4:
                                    m.operator = 4;
                                    break;
                                case "AND":
                                case 5:
                                    m.operator = 5;
                                    break;
                                case "CONTAINS_ANY":
                                case 6:
                                    m.operator = 6;
                                    break;
                                }
                                if (d.left_operands) {
                                    if (!Array.isArray(d.left_operands))
                                        throw TypeError(".google.ads.googleads.v11.common.MatchingFunction.left_operands: array expected");
                                    m.left_operands = [];
                                    for (var i = 0; i < d.left_operands.length; ++i) {
                                        if (typeof d.left_operands[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.MatchingFunction.left_operands: object expected");
                                        m.left_operands[i] = $root.google.ads.googleads.v11.common.Operand.fromObject(d.left_operands[i]);
                                    }
                                }
                                if (d.right_operands) {
                                    if (!Array.isArray(d.right_operands))
                                        throw TypeError(".google.ads.googleads.v11.common.MatchingFunction.right_operands: array expected");
                                    m.right_operands = [];
                                    for (var i = 0; i < d.right_operands.length; ++i) {
                                        if (typeof d.right_operands[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.MatchingFunction.right_operands: object expected");
                                        m.right_operands[i] = $root.google.ads.googleads.v11.common.Operand.fromObject(d.right_operands[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MatchingFunction message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @static
                             * @param {google.ads.googleads.v11.common.MatchingFunction} m MatchingFunction
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MatchingFunction.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.left_operands = [];
                                    d.right_operands = [];
                                }
                                if (o.defaults) {
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.left_operands && m.left_operands.length) {
                                    d.left_operands = [];
                                    for (var j = 0; j < m.left_operands.length; ++j) {
                                        d.left_operands[j] = $root.google.ads.googleads.v11.common.Operand.toObject(m.left_operands[j], o);
                                    }
                                }
                                if (m.right_operands && m.right_operands.length) {
                                    d.right_operands = [];
                                    for (var j = 0; j < m.right_operands.length; ++j) {
                                        d.right_operands[j] = $root.google.ads.googleads.v11.common.Operand.toObject(m.right_operands[j], o);
                                    }
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum.MatchingFunctionOperator[m.operator] : m.operator;
                                }
                                if (m.function_string != null && m.hasOwnProperty("function_string")) {
                                    d.function_string = m.function_string;
                                    if (o.oneofs)
                                        d._function_string = "function_string";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MatchingFunction to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MatchingFunction
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MatchingFunction.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MatchingFunction;
                        })();
    
                        common.Operand = (function() {
    
                            /**
                             * Properties of an Operand.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IOperand
                             * @property {google.ads.googleads.v11.common.Operand.IConstantOperand|null} [constant_operand] Operand constant_operand
                             * @property {google.ads.googleads.v11.common.Operand.IFeedAttributeOperand|null} [feed_attribute_operand] Operand feed_attribute_operand
                             * @property {google.ads.googleads.v11.common.Operand.IFunctionOperand|null} [function_operand] Operand function_operand
                             * @property {google.ads.googleads.v11.common.Operand.IRequestContextOperand|null} [request_context_operand] Operand request_context_operand
                             */
    
                            /**
                             * Constructs a new Operand.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an Operand.
                             * @implements IOperand
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IOperand=} [p] Properties to set
                             */
                            function Operand(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Operand constant_operand.
                             * @member {google.ads.googleads.v11.common.Operand.IConstantOperand|null|undefined} constant_operand
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @instance
                             */
                            Operand.prototype.constant_operand = null;
    
                            /**
                             * Operand feed_attribute_operand.
                             * @member {google.ads.googleads.v11.common.Operand.IFeedAttributeOperand|null|undefined} feed_attribute_operand
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @instance
                             */
                            Operand.prototype.feed_attribute_operand = null;
    
                            /**
                             * Operand function_operand.
                             * @member {google.ads.googleads.v11.common.Operand.IFunctionOperand|null|undefined} function_operand
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @instance
                             */
                            Operand.prototype.function_operand = null;
    
                            /**
                             * Operand request_context_operand.
                             * @member {google.ads.googleads.v11.common.Operand.IRequestContextOperand|null|undefined} request_context_operand
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @instance
                             */
                            Operand.prototype.request_context_operand = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Operand function_argument_operand.
                             * @member {"constant_operand"|"feed_attribute_operand"|"function_operand"|"request_context_operand"|undefined} function_argument_operand
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @instance
                             */
                            Object.defineProperty(Operand.prototype, "function_argument_operand", {
                                get: $util.oneOfGetter($oneOfFields = ["constant_operand", "feed_attribute_operand", "function_operand", "request_context_operand"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Operand instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {google.ads.googleads.v11.common.IOperand=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.Operand} Operand instance
                             */
                            Operand.create = function create(properties) {
                                return new Operand(properties);
                            };
    
                            /**
                             * Encodes the specified Operand message. Does not implicitly {@link google.ads.googleads.v11.common.Operand.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {google.ads.googleads.v11.common.IOperand} m Operand message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Operand.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.constant_operand != null && Object.hasOwnProperty.call(m, "constant_operand"))
                                    $root.google.ads.googleads.v11.common.Operand.ConstantOperand.encode(m.constant_operand, w.uint32(10).fork()).ldelim();
                                if (m.feed_attribute_operand != null && Object.hasOwnProperty.call(m, "feed_attribute_operand"))
                                    $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand.encode(m.feed_attribute_operand, w.uint32(18).fork()).ldelim();
                                if (m.function_operand != null && Object.hasOwnProperty.call(m, "function_operand"))
                                    $root.google.ads.googleads.v11.common.Operand.FunctionOperand.encode(m.function_operand, w.uint32(26).fork()).ldelim();
                                if (m.request_context_operand != null && Object.hasOwnProperty.call(m, "request_context_operand"))
                                    $root.google.ads.googleads.v11.common.Operand.RequestContextOperand.encode(m.request_context_operand, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified Operand message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Operand.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {google.ads.googleads.v11.common.IOperand} message Operand message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Operand.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Operand message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.Operand} Operand
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Operand.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Operand();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.constant_operand = $root.google.ads.googleads.v11.common.Operand.ConstantOperand.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.feed_attribute_operand = $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.function_operand = $root.google.ads.googleads.v11.common.Operand.FunctionOperand.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.request_context_operand = $root.google.ads.googleads.v11.common.Operand.RequestContextOperand.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an Operand message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.Operand} Operand
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Operand.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Operand message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Operand.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.constant_operand != null && m.hasOwnProperty("constant_operand")) {
                                    p.function_argument_operand = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Operand.ConstantOperand.verify(m.constant_operand);
                                        if (e)
                                            return "constant_operand." + e;
                                    }
                                }
                                if (m.feed_attribute_operand != null && m.hasOwnProperty("feed_attribute_operand")) {
                                    if (p.function_argument_operand === 1)
                                        return "function_argument_operand: multiple values";
                                    p.function_argument_operand = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand.verify(m.feed_attribute_operand);
                                        if (e)
                                            return "feed_attribute_operand." + e;
                                    }
                                }
                                if (m.function_operand != null && m.hasOwnProperty("function_operand")) {
                                    if (p.function_argument_operand === 1)
                                        return "function_argument_operand: multiple values";
                                    p.function_argument_operand = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Operand.FunctionOperand.verify(m.function_operand);
                                        if (e)
                                            return "function_operand." + e;
                                    }
                                }
                                if (m.request_context_operand != null && m.hasOwnProperty("request_context_operand")) {
                                    if (p.function_argument_operand === 1)
                                        return "function_argument_operand: multiple values";
                                    p.function_argument_operand = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Operand.RequestContextOperand.verify(m.request_context_operand);
                                        if (e)
                                            return "request_context_operand." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Operand message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.Operand} Operand
                             */
                            Operand.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.Operand)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.Operand();
                                if (d.constant_operand != null) {
                                    if (typeof d.constant_operand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Operand.constant_operand: object expected");
                                    m.constant_operand = $root.google.ads.googleads.v11.common.Operand.ConstantOperand.fromObject(d.constant_operand);
                                }
                                if (d.feed_attribute_operand != null) {
                                    if (typeof d.feed_attribute_operand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Operand.feed_attribute_operand: object expected");
                                    m.feed_attribute_operand = $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand.fromObject(d.feed_attribute_operand);
                                }
                                if (d.function_operand != null) {
                                    if (typeof d.function_operand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Operand.function_operand: object expected");
                                    m.function_operand = $root.google.ads.googleads.v11.common.Operand.FunctionOperand.fromObject(d.function_operand);
                                }
                                if (d.request_context_operand != null) {
                                    if (typeof d.request_context_operand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Operand.request_context_operand: object expected");
                                    m.request_context_operand = $root.google.ads.googleads.v11.common.Operand.RequestContextOperand.fromObject(d.request_context_operand);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an Operand message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @static
                             * @param {google.ads.googleads.v11.common.Operand} m Operand
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Operand.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.constant_operand != null && m.hasOwnProperty("constant_operand")) {
                                    d.constant_operand = $root.google.ads.googleads.v11.common.Operand.ConstantOperand.toObject(m.constant_operand, o);
                                    if (o.oneofs)
                                        d.function_argument_operand = "constant_operand";
                                }
                                if (m.feed_attribute_operand != null && m.hasOwnProperty("feed_attribute_operand")) {
                                    d.feed_attribute_operand = $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand.toObject(m.feed_attribute_operand, o);
                                    if (o.oneofs)
                                        d.function_argument_operand = "feed_attribute_operand";
                                }
                                if (m.function_operand != null && m.hasOwnProperty("function_operand")) {
                                    d.function_operand = $root.google.ads.googleads.v11.common.Operand.FunctionOperand.toObject(m.function_operand, o);
                                    if (o.oneofs)
                                        d.function_argument_operand = "function_operand";
                                }
                                if (m.request_context_operand != null && m.hasOwnProperty("request_context_operand")) {
                                    d.request_context_operand = $root.google.ads.googleads.v11.common.Operand.RequestContextOperand.toObject(m.request_context_operand, o);
                                    if (o.oneofs)
                                        d.function_argument_operand = "request_context_operand";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Operand to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.Operand
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Operand.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            Operand.ConstantOperand = (function() {
    
                                /**
                                 * Properties of a ConstantOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @interface IConstantOperand
                                 * @property {string|null} [string_value] ConstantOperand string_value
                                 * @property {number|null} [long_value] ConstantOperand long_value
                                 * @property {boolean|null} [boolean_value] ConstantOperand boolean_value
                                 * @property {number|null} [double_value] ConstantOperand double_value
                                 */
    
                                /**
                                 * Constructs a new ConstantOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @classdesc Represents a ConstantOperand.
                                 * @implements IConstantOperand
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.Operand.IConstantOperand=} [p] Properties to set
                                 */
                                function ConstantOperand(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ConstantOperand string_value.
                                 * @member {string|null|undefined} string_value
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @instance
                                 */
                                ConstantOperand.prototype.string_value = null;
    
                                /**
                                 * ConstantOperand long_value.
                                 * @member {number|null|undefined} long_value
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @instance
                                 */
                                ConstantOperand.prototype.long_value = null;
    
                                /**
                                 * ConstantOperand boolean_value.
                                 * @member {boolean|null|undefined} boolean_value
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @instance
                                 */
                                ConstantOperand.prototype.boolean_value = null;
    
                                /**
                                 * ConstantOperand double_value.
                                 * @member {number|null|undefined} double_value
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @instance
                                 */
                                ConstantOperand.prototype.double_value = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ConstantOperand constant_operand_value.
                                 * @member {"string_value"|"long_value"|"boolean_value"|"double_value"|undefined} constant_operand_value
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @instance
                                 */
                                Object.defineProperty(ConstantOperand.prototype, "constant_operand_value", {
                                    get: $util.oneOfGetter($oneOfFields = ["string_value", "long_value", "boolean_value", "double_value"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ConstantOperand instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IConstantOperand=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.Operand.ConstantOperand} ConstantOperand instance
                                 */
                                ConstantOperand.create = function create(properties) {
                                    return new ConstantOperand(properties);
                                };
    
                                /**
                                 * Encodes the specified ConstantOperand message. Does not implicitly {@link google.ads.googleads.v11.common.Operand.ConstantOperand.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IConstantOperand} m ConstantOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ConstantOperand.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.string_value != null && Object.hasOwnProperty.call(m, "string_value"))
                                        w.uint32(42).string(m.string_value);
                                    if (m.long_value != null && Object.hasOwnProperty.call(m, "long_value"))
                                        w.uint32(48).int64(m.long_value);
                                    if (m.boolean_value != null && Object.hasOwnProperty.call(m, "boolean_value"))
                                        w.uint32(56).bool(m.boolean_value);
                                    if (m.double_value != null && Object.hasOwnProperty.call(m, "double_value"))
                                        w.uint32(65).double(m.double_value);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ConstantOperand message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Operand.ConstantOperand.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IConstantOperand} message ConstantOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ConstantOperand.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ConstantOperand message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.Operand.ConstantOperand} ConstantOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ConstantOperand.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Operand.ConstantOperand();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 5:
                                            m.string_value = r.string();
                                            break;
                                        case 6:
                                            m.long_value = r.int64();
                                            break;
                                        case 7:
                                            m.boolean_value = r.bool();
                                            break;
                                        case 8:
                                            m.double_value = r.double();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ConstantOperand message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.Operand.ConstantOperand} ConstantOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ConstantOperand.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ConstantOperand message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ConstantOperand.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                        p.constant_operand_value = 1;
                                        if (!$util.isString(m.string_value))
                                            return "string_value: string expected";
                                    }
                                    if (m.long_value != null && m.hasOwnProperty("long_value")) {
                                        if (p.constant_operand_value === 1)
                                            return "constant_operand_value: multiple values";
                                        p.constant_operand_value = 1;
                                        if (!$util.isInteger(m.long_value) && !(m.long_value && $util.isInteger(m.long_value.low) && $util.isInteger(m.long_value.high)))
                                            return "long_value: integer|Long expected";
                                    }
                                    if (m.boolean_value != null && m.hasOwnProperty("boolean_value")) {
                                        if (p.constant_operand_value === 1)
                                            return "constant_operand_value: multiple values";
                                        p.constant_operand_value = 1;
                                        if (typeof m.boolean_value !== "boolean")
                                            return "boolean_value: boolean expected";
                                    }
                                    if (m.double_value != null && m.hasOwnProperty("double_value")) {
                                        if (p.constant_operand_value === 1)
                                            return "constant_operand_value: multiple values";
                                        p.constant_operand_value = 1;
                                        if (typeof m.double_value !== "number")
                                            return "double_value: number expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ConstantOperand message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.Operand.ConstantOperand} ConstantOperand
                                 */
                                ConstantOperand.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.Operand.ConstantOperand)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.Operand.ConstantOperand();
                                    if (d.string_value != null) {
                                        m.string_value = String(d.string_value);
                                    }
                                    if (d.long_value != null) {
                                        if ($util.Long)
                                            (m.long_value = $util.Long.fromValue(d.long_value)).unsigned = false;
                                        else if (typeof d.long_value === "string")
                                            m.long_value = parseInt(d.long_value, 10);
                                        else if (typeof d.long_value === "number")
                                            m.long_value = d.long_value;
                                        else if (typeof d.long_value === "object")
                                            m.long_value = new $util.LongBits(d.long_value.low >>> 0, d.long_value.high >>> 0).toNumber();
                                    }
                                    if (d.boolean_value != null) {
                                        m.boolean_value = Boolean(d.boolean_value);
                                    }
                                    if (d.double_value != null) {
                                        m.double_value = Number(d.double_value);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ConstantOperand message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.ConstantOperand} m ConstantOperand
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ConstantOperand.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                        d.string_value = m.string_value;
                                        if (o.oneofs)
                                            d.constant_operand_value = "string_value";
                                    }
                                    if (m.long_value != null && m.hasOwnProperty("long_value")) {
                                        if (typeof m.long_value === "number")
                                            d.long_value = o.longs === String ? String(m.long_value) : m.long_value;
                                        else
                                            d.long_value = o.longs === String ? $util.Long.prototype.toString.call(m.long_value) : o.longs === Number ? new $util.LongBits(m.long_value.low >>> 0, m.long_value.high >>> 0).toNumber() : m.long_value;
                                        if (o.oneofs)
                                            d.constant_operand_value = "long_value";
                                    }
                                    if (m.boolean_value != null && m.hasOwnProperty("boolean_value")) {
                                        d.boolean_value = m.boolean_value;
                                        if (o.oneofs)
                                            d.constant_operand_value = "boolean_value";
                                    }
                                    if (m.double_value != null && m.hasOwnProperty("double_value")) {
                                        d.double_value = o.json && !isFinite(m.double_value) ? String(m.double_value) : m.double_value;
                                        if (o.oneofs)
                                            d.constant_operand_value = "double_value";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ConstantOperand to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.Operand.ConstantOperand
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ConstantOperand.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ConstantOperand;
                            })();
    
                            Operand.FeedAttributeOperand = (function() {
    
                                /**
                                 * Properties of a FeedAttributeOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @interface IFeedAttributeOperand
                                 * @property {number|null} [feed_id] FeedAttributeOperand feed_id
                                 * @property {number|null} [feed_attribute_id] FeedAttributeOperand feed_attribute_id
                                 */
    
                                /**
                                 * Constructs a new FeedAttributeOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @classdesc Represents a FeedAttributeOperand.
                                 * @implements IFeedAttributeOperand
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.Operand.IFeedAttributeOperand=} [p] Properties to set
                                 */
                                function FeedAttributeOperand(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * FeedAttributeOperand feed_id.
                                 * @member {number|null|undefined} feed_id
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @instance
                                 */
                                FeedAttributeOperand.prototype.feed_id = null;
    
                                /**
                                 * FeedAttributeOperand feed_attribute_id.
                                 * @member {number|null|undefined} feed_attribute_id
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @instance
                                 */
                                FeedAttributeOperand.prototype.feed_attribute_id = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * FeedAttributeOperand _feed_id.
                                 * @member {"feed_id"|undefined} _feed_id
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @instance
                                 */
                                Object.defineProperty(FeedAttributeOperand.prototype, "_feed_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["feed_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * FeedAttributeOperand _feed_attribute_id.
                                 * @member {"feed_attribute_id"|undefined} _feed_attribute_id
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @instance
                                 */
                                Object.defineProperty(FeedAttributeOperand.prototype, "_feed_attribute_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["feed_attribute_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new FeedAttributeOperand instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IFeedAttributeOperand=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.Operand.FeedAttributeOperand} FeedAttributeOperand instance
                                 */
                                FeedAttributeOperand.create = function create(properties) {
                                    return new FeedAttributeOperand(properties);
                                };
    
                                /**
                                 * Encodes the specified FeedAttributeOperand message. Does not implicitly {@link google.ads.googleads.v11.common.Operand.FeedAttributeOperand.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IFeedAttributeOperand} m FeedAttributeOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FeedAttributeOperand.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.feed_id != null && Object.hasOwnProperty.call(m, "feed_id"))
                                        w.uint32(24).int64(m.feed_id);
                                    if (m.feed_attribute_id != null && Object.hasOwnProperty.call(m, "feed_attribute_id"))
                                        w.uint32(32).int64(m.feed_attribute_id);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified FeedAttributeOperand message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Operand.FeedAttributeOperand.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IFeedAttributeOperand} message FeedAttributeOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FeedAttributeOperand.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a FeedAttributeOperand message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.Operand.FeedAttributeOperand} FeedAttributeOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FeedAttributeOperand.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 3:
                                            m.feed_id = r.int64();
                                            break;
                                        case 4:
                                            m.feed_attribute_id = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a FeedAttributeOperand message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.Operand.FeedAttributeOperand} FeedAttributeOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FeedAttributeOperand.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a FeedAttributeOperand message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                FeedAttributeOperand.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.feed_id != null && m.hasOwnProperty("feed_id")) {
                                        p._feed_id = 1;
                                        if (!$util.isInteger(m.feed_id) && !(m.feed_id && $util.isInteger(m.feed_id.low) && $util.isInteger(m.feed_id.high)))
                                            return "feed_id: integer|Long expected";
                                    }
                                    if (m.feed_attribute_id != null && m.hasOwnProperty("feed_attribute_id")) {
                                        p._feed_attribute_id = 1;
                                        if (!$util.isInteger(m.feed_attribute_id) && !(m.feed_attribute_id && $util.isInteger(m.feed_attribute_id.low) && $util.isInteger(m.feed_attribute_id.high)))
                                            return "feed_attribute_id: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a FeedAttributeOperand message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.Operand.FeedAttributeOperand} FeedAttributeOperand
                                 */
                                FeedAttributeOperand.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.Operand.FeedAttributeOperand();
                                    if (d.feed_id != null) {
                                        if ($util.Long)
                                            (m.feed_id = $util.Long.fromValue(d.feed_id)).unsigned = false;
                                        else if (typeof d.feed_id === "string")
                                            m.feed_id = parseInt(d.feed_id, 10);
                                        else if (typeof d.feed_id === "number")
                                            m.feed_id = d.feed_id;
                                        else if (typeof d.feed_id === "object")
                                            m.feed_id = new $util.LongBits(d.feed_id.low >>> 0, d.feed_id.high >>> 0).toNumber();
                                    }
                                    if (d.feed_attribute_id != null) {
                                        if ($util.Long)
                                            (m.feed_attribute_id = $util.Long.fromValue(d.feed_attribute_id)).unsigned = false;
                                        else if (typeof d.feed_attribute_id === "string")
                                            m.feed_attribute_id = parseInt(d.feed_attribute_id, 10);
                                        else if (typeof d.feed_attribute_id === "number")
                                            m.feed_attribute_id = d.feed_attribute_id;
                                        else if (typeof d.feed_attribute_id === "object")
                                            m.feed_attribute_id = new $util.LongBits(d.feed_attribute_id.low >>> 0, d.feed_attribute_id.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a FeedAttributeOperand message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.FeedAttributeOperand} m FeedAttributeOperand
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                FeedAttributeOperand.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.feed_id != null && m.hasOwnProperty("feed_id")) {
                                        if (typeof m.feed_id === "number")
                                            d.feed_id = o.longs === String ? String(m.feed_id) : m.feed_id;
                                        else
                                            d.feed_id = o.longs === String ? $util.Long.prototype.toString.call(m.feed_id) : o.longs === Number ? new $util.LongBits(m.feed_id.low >>> 0, m.feed_id.high >>> 0).toNumber() : m.feed_id;
                                        if (o.oneofs)
                                            d._feed_id = "feed_id";
                                    }
                                    if (m.feed_attribute_id != null && m.hasOwnProperty("feed_attribute_id")) {
                                        if (typeof m.feed_attribute_id === "number")
                                            d.feed_attribute_id = o.longs === String ? String(m.feed_attribute_id) : m.feed_attribute_id;
                                        else
                                            d.feed_attribute_id = o.longs === String ? $util.Long.prototype.toString.call(m.feed_attribute_id) : o.longs === Number ? new $util.LongBits(m.feed_attribute_id.low >>> 0, m.feed_attribute_id.high >>> 0).toNumber() : m.feed_attribute_id;
                                        if (o.oneofs)
                                            d._feed_attribute_id = "feed_attribute_id";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this FeedAttributeOperand to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.Operand.FeedAttributeOperand
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                FeedAttributeOperand.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return FeedAttributeOperand;
                            })();
    
                            Operand.FunctionOperand = (function() {
    
                                /**
                                 * Properties of a FunctionOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @interface IFunctionOperand
                                 * @property {google.ads.googleads.v11.common.IMatchingFunction|null} [matching_function] FunctionOperand matching_function
                                 */
    
                                /**
                                 * Constructs a new FunctionOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @classdesc Represents a FunctionOperand.
                                 * @implements IFunctionOperand
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.Operand.IFunctionOperand=} [p] Properties to set
                                 */
                                function FunctionOperand(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * FunctionOperand matching_function.
                                 * @member {google.ads.googleads.v11.common.IMatchingFunction|null|undefined} matching_function
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @instance
                                 */
                                FunctionOperand.prototype.matching_function = null;
    
                                /**
                                 * Creates a new FunctionOperand instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IFunctionOperand=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.Operand.FunctionOperand} FunctionOperand instance
                                 */
                                FunctionOperand.create = function create(properties) {
                                    return new FunctionOperand(properties);
                                };
    
                                /**
                                 * Encodes the specified FunctionOperand message. Does not implicitly {@link google.ads.googleads.v11.common.Operand.FunctionOperand.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IFunctionOperand} m FunctionOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FunctionOperand.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.matching_function != null && Object.hasOwnProperty.call(m, "matching_function"))
                                        $root.google.ads.googleads.v11.common.MatchingFunction.encode(m.matching_function, w.uint32(10).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified FunctionOperand message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Operand.FunctionOperand.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IFunctionOperand} message FunctionOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FunctionOperand.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a FunctionOperand message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.Operand.FunctionOperand} FunctionOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FunctionOperand.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Operand.FunctionOperand();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a FunctionOperand message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.Operand.FunctionOperand} FunctionOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FunctionOperand.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a FunctionOperand message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                FunctionOperand.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.MatchingFunction.verify(m.matching_function);
                                            if (e)
                                                return "matching_function." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a FunctionOperand message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.Operand.FunctionOperand} FunctionOperand
                                 */
                                FunctionOperand.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.Operand.FunctionOperand)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.Operand.FunctionOperand();
                                    if (d.matching_function != null) {
                                        if (typeof d.matching_function !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.Operand.FunctionOperand.matching_function: object expected");
                                        m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.fromObject(d.matching_function);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a FunctionOperand message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.FunctionOperand} m FunctionOperand
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                FunctionOperand.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.matching_function = null;
                                    }
                                    if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                        d.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.toObject(m.matching_function, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this FunctionOperand to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.Operand.FunctionOperand
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                FunctionOperand.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return FunctionOperand;
                            })();
    
                            Operand.RequestContextOperand = (function() {
    
                                /**
                                 * Properties of a RequestContextOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @interface IRequestContextOperand
                                 * @property {google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum.MatchingFunctionContextType|null} [context_type] RequestContextOperand context_type
                                 */
    
                                /**
                                 * Constructs a new RequestContextOperand.
                                 * @memberof google.ads.googleads.v11.common.Operand
                                 * @classdesc Represents a RequestContextOperand.
                                 * @implements IRequestContextOperand
                                 * @constructor
                                 * @param {google.ads.googleads.v11.common.Operand.IRequestContextOperand=} [p] Properties to set
                                 */
                                function RequestContextOperand(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * RequestContextOperand context_type.
                                 * @member {google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum.MatchingFunctionContextType} context_type
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @instance
                                 */
                                RequestContextOperand.prototype.context_type = 0;
    
                                /**
                                 * Creates a new RequestContextOperand instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IRequestContextOperand=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.common.Operand.RequestContextOperand} RequestContextOperand instance
                                 */
                                RequestContextOperand.create = function create(properties) {
                                    return new RequestContextOperand(properties);
                                };
    
                                /**
                                 * Encodes the specified RequestContextOperand message. Does not implicitly {@link google.ads.googleads.v11.common.Operand.RequestContextOperand.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IRequestContextOperand} m RequestContextOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                RequestContextOperand.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.context_type != null && Object.hasOwnProperty.call(m, "context_type"))
                                        w.uint32(8).int32(m.context_type);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified RequestContextOperand message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Operand.RequestContextOperand.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.IRequestContextOperand} message RequestContextOperand message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                RequestContextOperand.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a RequestContextOperand message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.common.Operand.RequestContextOperand} RequestContextOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                RequestContextOperand.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Operand.RequestContextOperand();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.context_type = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a RequestContextOperand message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.common.Operand.RequestContextOperand} RequestContextOperand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                RequestContextOperand.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a RequestContextOperand message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                RequestContextOperand.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.context_type != null && m.hasOwnProperty("context_type")) {
                                        switch (m.context_type) {
                                        default:
                                            return "context_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a RequestContextOperand message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.common.Operand.RequestContextOperand} RequestContextOperand
                                 */
                                RequestContextOperand.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.common.Operand.RequestContextOperand)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.common.Operand.RequestContextOperand();
                                    switch (d.context_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.context_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.context_type = 1;
                                        break;
                                    case "FEED_ITEM_ID":
                                    case 2:
                                        m.context_type = 2;
                                        break;
                                    case "DEVICE_NAME":
                                    case 3:
                                        m.context_type = 3;
                                        break;
                                    case "FEED_ITEM_SET_ID":
                                    case 4:
                                        m.context_type = 4;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a RequestContextOperand message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @static
                                 * @param {google.ads.googleads.v11.common.Operand.RequestContextOperand} m RequestContextOperand
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                RequestContextOperand.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.context_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.context_type != null && m.hasOwnProperty("context_type")) {
                                        d.context_type = o.enums === String ? $root.google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum.MatchingFunctionContextType[m.context_type] : m.context_type;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this RequestContextOperand to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.common.Operand.RequestContextOperand
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                RequestContextOperand.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return RequestContextOperand;
                            })();
    
                            return Operand;
                        })();
    
                        common.MetricGoal = (function() {
    
                            /**
                             * Properties of a MetricGoal.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMetricGoal
                             * @property {google.ads.googleads.v11.enums.ExperimentMetricEnum.ExperimentMetric|null} [metric] MetricGoal metric
                             * @property {google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum.ExperimentMetricDirection|null} [direction] MetricGoal direction
                             */
    
                            /**
                             * Constructs a new MetricGoal.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a MetricGoal.
                             * @implements IMetricGoal
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMetricGoal=} [p] Properties to set
                             */
                            function MetricGoal(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MetricGoal metric.
                             * @member {google.ads.googleads.v11.enums.ExperimentMetricEnum.ExperimentMetric} metric
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @instance
                             */
                            MetricGoal.prototype.metric = 0;
    
                            /**
                             * MetricGoal direction.
                             * @member {google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum.ExperimentMetricDirection} direction
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @instance
                             */
                            MetricGoal.prototype.direction = 0;
    
                            /**
                             * Creates a new MetricGoal instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {google.ads.googleads.v11.common.IMetricGoal=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.MetricGoal} MetricGoal instance
                             */
                            MetricGoal.create = function create(properties) {
                                return new MetricGoal(properties);
                            };
    
                            /**
                             * Encodes the specified MetricGoal message. Does not implicitly {@link google.ads.googleads.v11.common.MetricGoal.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {google.ads.googleads.v11.common.IMetricGoal} m MetricGoal message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MetricGoal.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.metric != null && Object.hasOwnProperty.call(m, "metric"))
                                    w.uint32(8).int32(m.metric);
                                if (m.direction != null && Object.hasOwnProperty.call(m, "direction"))
                                    w.uint32(16).int32(m.direction);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MetricGoal message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.MetricGoal.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {google.ads.googleads.v11.common.IMetricGoal} message MetricGoal message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MetricGoal.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MetricGoal message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.MetricGoal} MetricGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MetricGoal.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.MetricGoal();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.metric = r.int32();
                                        break;
                                    case 2:
                                        m.direction = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MetricGoal message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.MetricGoal} MetricGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MetricGoal.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MetricGoal message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MetricGoal.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.metric != null && m.hasOwnProperty("metric")) {
                                    switch (m.metric) {
                                    default:
                                        return "metric: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                        break;
                                    }
                                }
                                if (m.direction != null && m.hasOwnProperty("direction")) {
                                    switch (m.direction) {
                                    default:
                                        return "direction: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MetricGoal message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.MetricGoal} MetricGoal
                             */
                            MetricGoal.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.MetricGoal)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.MetricGoal();
                                switch (d.metric) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.metric = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.metric = 1;
                                    break;
                                case "CLICKS":
                                case 2:
                                    m.metric = 2;
                                    break;
                                case "IMPRESSIONS":
                                case 3:
                                    m.metric = 3;
                                    break;
                                case "COST":
                                case 4:
                                    m.metric = 4;
                                    break;
                                case "CONVERSIONS_PER_INTERACTION_RATE":
                                case 5:
                                    m.metric = 5;
                                    break;
                                case "COST_PER_CONVERSION":
                                case 6:
                                    m.metric = 6;
                                    break;
                                case "CONVERSIONS_VALUE_PER_COST":
                                case 7:
                                    m.metric = 7;
                                    break;
                                case "AVERAGE_CPC":
                                case 8:
                                    m.metric = 8;
                                    break;
                                case "CTR":
                                case 9:
                                    m.metric = 9;
                                    break;
                                case "INCREMENTAL_CONVERSIONS":
                                case 10:
                                    m.metric = 10;
                                    break;
                                case "COMPLETED_VIDEO_VIEWS":
                                case 11:
                                    m.metric = 11;
                                    break;
                                case "CUSTOM_ALGORITHMS":
                                case 12:
                                    m.metric = 12;
                                    break;
                                case "CONVERSIONS":
                                case 13:
                                    m.metric = 13;
                                    break;
                                case "CONVERSION_VALUE":
                                case 14:
                                    m.metric = 14;
                                    break;
                                }
                                switch (d.direction) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.direction = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.direction = 1;
                                    break;
                                case "NO_CHANGE":
                                case 2:
                                    m.direction = 2;
                                    break;
                                case "INCREASE":
                                case 3:
                                    m.direction = 3;
                                    break;
                                case "DECREASE":
                                case 4:
                                    m.direction = 4;
                                    break;
                                case "NO_CHANGE_OR_INCREASE":
                                case 5:
                                    m.direction = 5;
                                    break;
                                case "NO_CHANGE_OR_DECREASE":
                                case 6:
                                    m.direction = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MetricGoal message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @static
                             * @param {google.ads.googleads.v11.common.MetricGoal} m MetricGoal
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MetricGoal.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.metric = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.direction = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.metric != null && m.hasOwnProperty("metric")) {
                                    d.metric = o.enums === String ? $root.google.ads.googleads.v11.enums.ExperimentMetricEnum.ExperimentMetric[m.metric] : m.metric;
                                }
                                if (m.direction != null && m.hasOwnProperty("direction")) {
                                    d.direction = o.enums === String ? $root.google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum.ExperimentMetricDirection[m.direction] : m.direction;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MetricGoal to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.MetricGoal
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MetricGoal.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MetricGoal;
                        })();
    
                        common.Metrics = (function() {
    
                            /**
                             * Properties of a Metrics.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IMetrics
                             * @property {number|null} [absolute_top_impression_percentage] Metrics absolute_top_impression_percentage
                             * @property {number|null} [active_view_cpm] Metrics active_view_cpm
                             * @property {number|null} [active_view_ctr] Metrics active_view_ctr
                             * @property {number|null} [active_view_impressions] Metrics active_view_impressions
                             * @property {number|null} [active_view_measurability] Metrics active_view_measurability
                             * @property {number|null} [active_view_measurable_cost_micros] Metrics active_view_measurable_cost_micros
                             * @property {number|null} [active_view_measurable_impressions] Metrics active_view_measurable_impressions
                             * @property {number|null} [active_view_viewability] Metrics active_view_viewability
                             * @property {number|null} [all_conversions_from_interactions_rate] Metrics all_conversions_from_interactions_rate
                             * @property {number|null} [all_conversions_value] Metrics all_conversions_value
                             * @property {number|null} [all_conversions_value_by_conversion_date] Metrics all_conversions_value_by_conversion_date
                             * @property {number|null} [all_conversions] Metrics all_conversions
                             * @property {number|null} [all_conversions_by_conversion_date] Metrics all_conversions_by_conversion_date
                             * @property {number|null} [all_conversions_value_per_cost] Metrics all_conversions_value_per_cost
                             * @property {number|null} [all_conversions_from_click_to_call] Metrics all_conversions_from_click_to_call
                             * @property {number|null} [all_conversions_from_directions] Metrics all_conversions_from_directions
                             * @property {number|null} [all_conversions_from_interactions_value_per_interaction] Metrics all_conversions_from_interactions_value_per_interaction
                             * @property {number|null} [all_conversions_from_menu] Metrics all_conversions_from_menu
                             * @property {number|null} [all_conversions_from_order] Metrics all_conversions_from_order
                             * @property {number|null} [all_conversions_from_other_engagement] Metrics all_conversions_from_other_engagement
                             * @property {number|null} [all_conversions_from_store_visit] Metrics all_conversions_from_store_visit
                             * @property {number|null} [all_conversions_from_store_website] Metrics all_conversions_from_store_website
                             * @property {number|null} [auction_insight_search_absolute_top_impression_percentage] Metrics auction_insight_search_absolute_top_impression_percentage
                             * @property {number|null} [auction_insight_search_impression_share] Metrics auction_insight_search_impression_share
                             * @property {number|null} [auction_insight_search_outranking_share] Metrics auction_insight_search_outranking_share
                             * @property {number|null} [auction_insight_search_overlap_rate] Metrics auction_insight_search_overlap_rate
                             * @property {number|null} [auction_insight_search_position_above_rate] Metrics auction_insight_search_position_above_rate
                             * @property {number|null} [auction_insight_search_top_impression_percentage] Metrics auction_insight_search_top_impression_percentage
                             * @property {number|null} [average_cost] Metrics average_cost
                             * @property {number|null} [average_cpc] Metrics average_cpc
                             * @property {number|null} [average_cpe] Metrics average_cpe
                             * @property {number|null} [average_cpm] Metrics average_cpm
                             * @property {number|null} [average_cpv] Metrics average_cpv
                             * @property {number|null} [average_page_views] Metrics average_page_views
                             * @property {number|null} [average_time_on_site] Metrics average_time_on_site
                             * @property {number|null} [benchmark_average_max_cpc] Metrics benchmark_average_max_cpc
                             * @property {number|null} [biddable_app_install_conversions] Metrics biddable_app_install_conversions
                             * @property {number|null} [biddable_app_post_install_conversions] Metrics biddable_app_post_install_conversions
                             * @property {number|null} [benchmark_ctr] Metrics benchmark_ctr
                             * @property {number|null} [bounce_rate] Metrics bounce_rate
                             * @property {number|null} [clicks] Metrics clicks
                             * @property {number|null} [combined_clicks] Metrics combined_clicks
                             * @property {number|null} [combined_clicks_per_query] Metrics combined_clicks_per_query
                             * @property {number|null} [combined_queries] Metrics combined_queries
                             * @property {number|null} [content_budget_lost_impression_share] Metrics content_budget_lost_impression_share
                             * @property {number|null} [content_impression_share] Metrics content_impression_share
                             * @property {string|null} [conversion_last_received_request_date_time] Metrics conversion_last_received_request_date_time
                             * @property {string|null} [conversion_last_conversion_date] Metrics conversion_last_conversion_date
                             * @property {number|null} [content_rank_lost_impression_share] Metrics content_rank_lost_impression_share
                             * @property {number|null} [conversions_from_interactions_rate] Metrics conversions_from_interactions_rate
                             * @property {number|null} [conversions_value] Metrics conversions_value
                             * @property {number|null} [conversions_value_by_conversion_date] Metrics conversions_value_by_conversion_date
                             * @property {number|null} [conversions_value_per_cost] Metrics conversions_value_per_cost
                             * @property {number|null} [conversions_from_interactions_value_per_interaction] Metrics conversions_from_interactions_value_per_interaction
                             * @property {number|null} [conversions] Metrics conversions
                             * @property {number|null} [conversions_by_conversion_date] Metrics conversions_by_conversion_date
                             * @property {number|null} [cost_micros] Metrics cost_micros
                             * @property {number|null} [cost_per_all_conversions] Metrics cost_per_all_conversions
                             * @property {number|null} [cost_per_conversion] Metrics cost_per_conversion
                             * @property {number|null} [cost_per_current_model_attributed_conversion] Metrics cost_per_current_model_attributed_conversion
                             * @property {number|null} [cross_device_conversions] Metrics cross_device_conversions
                             * @property {number|null} [ctr] Metrics ctr
                             * @property {number|null} [current_model_attributed_conversions] Metrics current_model_attributed_conversions
                             * @property {number|null} [current_model_attributed_conversions_from_interactions_rate] Metrics current_model_attributed_conversions_from_interactions_rate
                             * @property {number|null} [current_model_attributed_conversions_from_interactions_value_per_interaction] Metrics current_model_attributed_conversions_from_interactions_value_per_interaction
                             * @property {number|null} [current_model_attributed_conversions_value] Metrics current_model_attributed_conversions_value
                             * @property {number|null} [current_model_attributed_conversions_value_per_cost] Metrics current_model_attributed_conversions_value_per_cost
                             * @property {number|null} [engagement_rate] Metrics engagement_rate
                             * @property {number|null} [engagements] Metrics engagements
                             * @property {number|null} [hotel_average_lead_value_micros] Metrics hotel_average_lead_value_micros
                             * @property {number|null} [hotel_commission_rate_micros] Metrics hotel_commission_rate_micros
                             * @property {number|null} [hotel_expected_commission_cost] Metrics hotel_expected_commission_cost
                             * @property {number|null} [hotel_price_difference_percentage] Metrics hotel_price_difference_percentage
                             * @property {number|null} [hotel_eligible_impressions] Metrics hotel_eligible_impressions
                             * @property {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket|null} [historical_creative_quality_score] Metrics historical_creative_quality_score
                             * @property {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket|null} [historical_landing_page_quality_score] Metrics historical_landing_page_quality_score
                             * @property {number|null} [historical_quality_score] Metrics historical_quality_score
                             * @property {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket|null} [historical_search_predicted_ctr] Metrics historical_search_predicted_ctr
                             * @property {number|null} [gmail_forwards] Metrics gmail_forwards
                             * @property {number|null} [gmail_saves] Metrics gmail_saves
                             * @property {number|null} [gmail_secondary_clicks] Metrics gmail_secondary_clicks
                             * @property {number|null} [impressions_from_store_reach] Metrics impressions_from_store_reach
                             * @property {number|null} [impressions] Metrics impressions
                             * @property {number|null} [interaction_rate] Metrics interaction_rate
                             * @property {number|null} [interactions] Metrics interactions
                             * @property {Array.<google.ads.googleads.v11.enums.InteractionEventTypeEnum.InteractionEventType>|null} [interaction_event_types] Metrics interaction_event_types
                             * @property {number|null} [invalid_click_rate] Metrics invalid_click_rate
                             * @property {number|null} [invalid_clicks] Metrics invalid_clicks
                             * @property {number|null} [message_chats] Metrics message_chats
                             * @property {number|null} [message_impressions] Metrics message_impressions
                             * @property {number|null} [message_chat_rate] Metrics message_chat_rate
                             * @property {number|null} [mobile_friendly_clicks_percentage] Metrics mobile_friendly_clicks_percentage
                             * @property {number|null} [optimization_score_uplift] Metrics optimization_score_uplift
                             * @property {string|null} [optimization_score_url] Metrics optimization_score_url
                             * @property {number|null} [organic_clicks] Metrics organic_clicks
                             * @property {number|null} [organic_clicks_per_query] Metrics organic_clicks_per_query
                             * @property {number|null} [organic_impressions] Metrics organic_impressions
                             * @property {number|null} [organic_impressions_per_query] Metrics organic_impressions_per_query
                             * @property {number|null} [organic_queries] Metrics organic_queries
                             * @property {number|null} [percent_new_visitors] Metrics percent_new_visitors
                             * @property {number|null} [phone_calls] Metrics phone_calls
                             * @property {number|null} [phone_impressions] Metrics phone_impressions
                             * @property {number|null} [phone_through_rate] Metrics phone_through_rate
                             * @property {number|null} [relative_ctr] Metrics relative_ctr
                             * @property {number|null} [search_absolute_top_impression_share] Metrics search_absolute_top_impression_share
                             * @property {number|null} [search_budget_lost_absolute_top_impression_share] Metrics search_budget_lost_absolute_top_impression_share
                             * @property {number|null} [search_budget_lost_impression_share] Metrics search_budget_lost_impression_share
                             * @property {number|null} [search_budget_lost_top_impression_share] Metrics search_budget_lost_top_impression_share
                             * @property {number|null} [search_click_share] Metrics search_click_share
                             * @property {number|null} [search_exact_match_impression_share] Metrics search_exact_match_impression_share
                             * @property {number|null} [search_impression_share] Metrics search_impression_share
                             * @property {number|null} [search_rank_lost_absolute_top_impression_share] Metrics search_rank_lost_absolute_top_impression_share
                             * @property {number|null} [search_rank_lost_impression_share] Metrics search_rank_lost_impression_share
                             * @property {number|null} [search_rank_lost_top_impression_share] Metrics search_rank_lost_top_impression_share
                             * @property {number|null} [search_top_impression_share] Metrics search_top_impression_share
                             * @property {number|null} [speed_score] Metrics speed_score
                             * @property {number|null} [top_impression_percentage] Metrics top_impression_percentage
                             * @property {number|null} [valid_accelerated_mobile_pages_clicks_percentage] Metrics valid_accelerated_mobile_pages_clicks_percentage
                             * @property {number|null} [value_per_all_conversions] Metrics value_per_all_conversions
                             * @property {number|null} [value_per_all_conversions_by_conversion_date] Metrics value_per_all_conversions_by_conversion_date
                             * @property {number|null} [value_per_conversion] Metrics value_per_conversion
                             * @property {number|null} [value_per_conversions_by_conversion_date] Metrics value_per_conversions_by_conversion_date
                             * @property {number|null} [value_per_current_model_attributed_conversion] Metrics value_per_current_model_attributed_conversion
                             * @property {number|null} [video_quartile_p100_rate] Metrics video_quartile_p100_rate
                             * @property {number|null} [video_quartile_p25_rate] Metrics video_quartile_p25_rate
                             * @property {number|null} [video_quartile_p50_rate] Metrics video_quartile_p50_rate
                             * @property {number|null} [video_quartile_p75_rate] Metrics video_quartile_p75_rate
                             * @property {number|null} [video_view_rate] Metrics video_view_rate
                             * @property {number|null} [video_views] Metrics video_views
                             * @property {number|null} [view_through_conversions] Metrics view_through_conversions
                             * @property {number|null} [sk_ad_network_conversions] Metrics sk_ad_network_conversions
                             */
    
                            /**
                             * Constructs a new Metrics.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a Metrics.
                             * @implements IMetrics
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IMetrics=} [p] Properties to set
                             */
                            function Metrics(p) {
                                this.interaction_event_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Metrics absolute_top_impression_percentage.
                             * @member {number|null|undefined} absolute_top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.absolute_top_impression_percentage = null;
    
                            /**
                             * Metrics active_view_cpm.
                             * @member {number|null|undefined} active_view_cpm
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.active_view_cpm = null;
    
                            /**
                             * Metrics active_view_ctr.
                             * @member {number|null|undefined} active_view_ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.active_view_ctr = null;
    
                            /**
                             * Metrics active_view_impressions.
                             * @member {number|null|undefined} active_view_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.active_view_impressions = null;
    
                            /**
                             * Metrics active_view_measurability.
                             * @member {number|null|undefined} active_view_measurability
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.active_view_measurability = null;
    
                            /**
                             * Metrics active_view_measurable_cost_micros.
                             * @member {number|null|undefined} active_view_measurable_cost_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.active_view_measurable_cost_micros = null;
    
                            /**
                             * Metrics active_view_measurable_impressions.
                             * @member {number|null|undefined} active_view_measurable_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.active_view_measurable_impressions = null;
    
                            /**
                             * Metrics active_view_viewability.
                             * @member {number|null|undefined} active_view_viewability
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.active_view_viewability = null;
    
                            /**
                             * Metrics all_conversions_from_interactions_rate.
                             * @member {number|null|undefined} all_conversions_from_interactions_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_interactions_rate = null;
    
                            /**
                             * Metrics all_conversions_value.
                             * @member {number|null|undefined} all_conversions_value
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_value = null;
    
                            /**
                             * Metrics all_conversions_value_by_conversion_date.
                             * @member {number} all_conversions_value_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_value_by_conversion_date = 0;
    
                            /**
                             * Metrics all_conversions.
                             * @member {number|null|undefined} all_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions = null;
    
                            /**
                             * Metrics all_conversions_by_conversion_date.
                             * @member {number} all_conversions_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_by_conversion_date = 0;
    
                            /**
                             * Metrics all_conversions_value_per_cost.
                             * @member {number|null|undefined} all_conversions_value_per_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_value_per_cost = null;
    
                            /**
                             * Metrics all_conversions_from_click_to_call.
                             * @member {number|null|undefined} all_conversions_from_click_to_call
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_click_to_call = null;
    
                            /**
                             * Metrics all_conversions_from_directions.
                             * @member {number|null|undefined} all_conversions_from_directions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_directions = null;
    
                            /**
                             * Metrics all_conversions_from_interactions_value_per_interaction.
                             * @member {number|null|undefined} all_conversions_from_interactions_value_per_interaction
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_interactions_value_per_interaction = null;
    
                            /**
                             * Metrics all_conversions_from_menu.
                             * @member {number|null|undefined} all_conversions_from_menu
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_menu = null;
    
                            /**
                             * Metrics all_conversions_from_order.
                             * @member {number|null|undefined} all_conversions_from_order
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_order = null;
    
                            /**
                             * Metrics all_conversions_from_other_engagement.
                             * @member {number|null|undefined} all_conversions_from_other_engagement
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_other_engagement = null;
    
                            /**
                             * Metrics all_conversions_from_store_visit.
                             * @member {number|null|undefined} all_conversions_from_store_visit
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_store_visit = null;
    
                            /**
                             * Metrics all_conversions_from_store_website.
                             * @member {number|null|undefined} all_conversions_from_store_website
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.all_conversions_from_store_website = null;
    
                            /**
                             * Metrics auction_insight_search_absolute_top_impression_percentage.
                             * @member {number|null|undefined} auction_insight_search_absolute_top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.auction_insight_search_absolute_top_impression_percentage = null;
    
                            /**
                             * Metrics auction_insight_search_impression_share.
                             * @member {number|null|undefined} auction_insight_search_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.auction_insight_search_impression_share = null;
    
                            /**
                             * Metrics auction_insight_search_outranking_share.
                             * @member {number|null|undefined} auction_insight_search_outranking_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.auction_insight_search_outranking_share = null;
    
                            /**
                             * Metrics auction_insight_search_overlap_rate.
                             * @member {number|null|undefined} auction_insight_search_overlap_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.auction_insight_search_overlap_rate = null;
    
                            /**
                             * Metrics auction_insight_search_position_above_rate.
                             * @member {number|null|undefined} auction_insight_search_position_above_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.auction_insight_search_position_above_rate = null;
    
                            /**
                             * Metrics auction_insight_search_top_impression_percentage.
                             * @member {number|null|undefined} auction_insight_search_top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.auction_insight_search_top_impression_percentage = null;
    
                            /**
                             * Metrics average_cost.
                             * @member {number|null|undefined} average_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.average_cost = null;
    
                            /**
                             * Metrics average_cpc.
                             * @member {number|null|undefined} average_cpc
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.average_cpc = null;
    
                            /**
                             * Metrics average_cpe.
                             * @member {number|null|undefined} average_cpe
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.average_cpe = null;
    
                            /**
                             * Metrics average_cpm.
                             * @member {number|null|undefined} average_cpm
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.average_cpm = null;
    
                            /**
                             * Metrics average_cpv.
                             * @member {number|null|undefined} average_cpv
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.average_cpv = null;
    
                            /**
                             * Metrics average_page_views.
                             * @member {number|null|undefined} average_page_views
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.average_page_views = null;
    
                            /**
                             * Metrics average_time_on_site.
                             * @member {number|null|undefined} average_time_on_site
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.average_time_on_site = null;
    
                            /**
                             * Metrics benchmark_average_max_cpc.
                             * @member {number|null|undefined} benchmark_average_max_cpc
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.benchmark_average_max_cpc = null;
    
                            /**
                             * Metrics biddable_app_install_conversions.
                             * @member {number|null|undefined} biddable_app_install_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.biddable_app_install_conversions = null;
    
                            /**
                             * Metrics biddable_app_post_install_conversions.
                             * @member {number|null|undefined} biddable_app_post_install_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.biddable_app_post_install_conversions = null;
    
                            /**
                             * Metrics benchmark_ctr.
                             * @member {number|null|undefined} benchmark_ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.benchmark_ctr = null;
    
                            /**
                             * Metrics bounce_rate.
                             * @member {number|null|undefined} bounce_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.bounce_rate = null;
    
                            /**
                             * Metrics clicks.
                             * @member {number|null|undefined} clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.clicks = null;
    
                            /**
                             * Metrics combined_clicks.
                             * @member {number|null|undefined} combined_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.combined_clicks = null;
    
                            /**
                             * Metrics combined_clicks_per_query.
                             * @member {number|null|undefined} combined_clicks_per_query
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.combined_clicks_per_query = null;
    
                            /**
                             * Metrics combined_queries.
                             * @member {number|null|undefined} combined_queries
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.combined_queries = null;
    
                            /**
                             * Metrics content_budget_lost_impression_share.
                             * @member {number|null|undefined} content_budget_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.content_budget_lost_impression_share = null;
    
                            /**
                             * Metrics content_impression_share.
                             * @member {number|null|undefined} content_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.content_impression_share = null;
    
                            /**
                             * Metrics conversion_last_received_request_date_time.
                             * @member {string|null|undefined} conversion_last_received_request_date_time
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversion_last_received_request_date_time = null;
    
                            /**
                             * Metrics conversion_last_conversion_date.
                             * @member {string|null|undefined} conversion_last_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversion_last_conversion_date = null;
    
                            /**
                             * Metrics content_rank_lost_impression_share.
                             * @member {number|null|undefined} content_rank_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.content_rank_lost_impression_share = null;
    
                            /**
                             * Metrics conversions_from_interactions_rate.
                             * @member {number|null|undefined} conversions_from_interactions_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversions_from_interactions_rate = null;
    
                            /**
                             * Metrics conversions_value.
                             * @member {number|null|undefined} conversions_value
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversions_value = null;
    
                            /**
                             * Metrics conversions_value_by_conversion_date.
                             * @member {number} conversions_value_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversions_value_by_conversion_date = 0;
    
                            /**
                             * Metrics conversions_value_per_cost.
                             * @member {number|null|undefined} conversions_value_per_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversions_value_per_cost = null;
    
                            /**
                             * Metrics conversions_from_interactions_value_per_interaction.
                             * @member {number|null|undefined} conversions_from_interactions_value_per_interaction
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversions_from_interactions_value_per_interaction = null;
    
                            /**
                             * Metrics conversions.
                             * @member {number|null|undefined} conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversions = null;
    
                            /**
                             * Metrics conversions_by_conversion_date.
                             * @member {number} conversions_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.conversions_by_conversion_date = 0;
    
                            /**
                             * Metrics cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.cost_micros = null;
    
                            /**
                             * Metrics cost_per_all_conversions.
                             * @member {number|null|undefined} cost_per_all_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.cost_per_all_conversions = null;
    
                            /**
                             * Metrics cost_per_conversion.
                             * @member {number|null|undefined} cost_per_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.cost_per_conversion = null;
    
                            /**
                             * Metrics cost_per_current_model_attributed_conversion.
                             * @member {number|null|undefined} cost_per_current_model_attributed_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.cost_per_current_model_attributed_conversion = null;
    
                            /**
                             * Metrics cross_device_conversions.
                             * @member {number|null|undefined} cross_device_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.cross_device_conversions = null;
    
                            /**
                             * Metrics ctr.
                             * @member {number|null|undefined} ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.ctr = null;
    
                            /**
                             * Metrics current_model_attributed_conversions.
                             * @member {number|null|undefined} current_model_attributed_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.current_model_attributed_conversions = null;
    
                            /**
                             * Metrics current_model_attributed_conversions_from_interactions_rate.
                             * @member {number|null|undefined} current_model_attributed_conversions_from_interactions_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.current_model_attributed_conversions_from_interactions_rate = null;
    
                            /**
                             * Metrics current_model_attributed_conversions_from_interactions_value_per_interaction.
                             * @member {number|null|undefined} current_model_attributed_conversions_from_interactions_value_per_interaction
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.current_model_attributed_conversions_from_interactions_value_per_interaction = null;
    
                            /**
                             * Metrics current_model_attributed_conversions_value.
                             * @member {number|null|undefined} current_model_attributed_conversions_value
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.current_model_attributed_conversions_value = null;
    
                            /**
                             * Metrics current_model_attributed_conversions_value_per_cost.
                             * @member {number|null|undefined} current_model_attributed_conversions_value_per_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.current_model_attributed_conversions_value_per_cost = null;
    
                            /**
                             * Metrics engagement_rate.
                             * @member {number|null|undefined} engagement_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.engagement_rate = null;
    
                            /**
                             * Metrics engagements.
                             * @member {number|null|undefined} engagements
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.engagements = null;
    
                            /**
                             * Metrics hotel_average_lead_value_micros.
                             * @member {number|null|undefined} hotel_average_lead_value_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.hotel_average_lead_value_micros = null;
    
                            /**
                             * Metrics hotel_commission_rate_micros.
                             * @member {number|null|undefined} hotel_commission_rate_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.hotel_commission_rate_micros = null;
    
                            /**
                             * Metrics hotel_expected_commission_cost.
                             * @member {number|null|undefined} hotel_expected_commission_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.hotel_expected_commission_cost = null;
    
                            /**
                             * Metrics hotel_price_difference_percentage.
                             * @member {number|null|undefined} hotel_price_difference_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.hotel_price_difference_percentage = null;
    
                            /**
                             * Metrics hotel_eligible_impressions.
                             * @member {number|null|undefined} hotel_eligible_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.hotel_eligible_impressions = null;
    
                            /**
                             * Metrics historical_creative_quality_score.
                             * @member {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket} historical_creative_quality_score
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.historical_creative_quality_score = 0;
    
                            /**
                             * Metrics historical_landing_page_quality_score.
                             * @member {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket} historical_landing_page_quality_score
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.historical_landing_page_quality_score = 0;
    
                            /**
                             * Metrics historical_quality_score.
                             * @member {number|null|undefined} historical_quality_score
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.historical_quality_score = null;
    
                            /**
                             * Metrics historical_search_predicted_ctr.
                             * @member {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket} historical_search_predicted_ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.historical_search_predicted_ctr = 0;
    
                            /**
                             * Metrics gmail_forwards.
                             * @member {number|null|undefined} gmail_forwards
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.gmail_forwards = null;
    
                            /**
                             * Metrics gmail_saves.
                             * @member {number|null|undefined} gmail_saves
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.gmail_saves = null;
    
                            /**
                             * Metrics gmail_secondary_clicks.
                             * @member {number|null|undefined} gmail_secondary_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.gmail_secondary_clicks = null;
    
                            /**
                             * Metrics impressions_from_store_reach.
                             * @member {number|null|undefined} impressions_from_store_reach
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.impressions_from_store_reach = null;
    
                            /**
                             * Metrics impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.impressions = null;
    
                            /**
                             * Metrics interaction_rate.
                             * @member {number|null|undefined} interaction_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.interaction_rate = null;
    
                            /**
                             * Metrics interactions.
                             * @member {number|null|undefined} interactions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.interactions = null;
    
                            /**
                             * Metrics interaction_event_types.
                             * @member {Array.<google.ads.googleads.v11.enums.InteractionEventTypeEnum.InteractionEventType>} interaction_event_types
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.interaction_event_types = $util.emptyArray;
    
                            /**
                             * Metrics invalid_click_rate.
                             * @member {number|null|undefined} invalid_click_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.invalid_click_rate = null;
    
                            /**
                             * Metrics invalid_clicks.
                             * @member {number|null|undefined} invalid_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.invalid_clicks = null;
    
                            /**
                             * Metrics message_chats.
                             * @member {number|null|undefined} message_chats
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.message_chats = null;
    
                            /**
                             * Metrics message_impressions.
                             * @member {number|null|undefined} message_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.message_impressions = null;
    
                            /**
                             * Metrics message_chat_rate.
                             * @member {number|null|undefined} message_chat_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.message_chat_rate = null;
    
                            /**
                             * Metrics mobile_friendly_clicks_percentage.
                             * @member {number|null|undefined} mobile_friendly_clicks_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.mobile_friendly_clicks_percentage = null;
    
                            /**
                             * Metrics optimization_score_uplift.
                             * @member {number|null|undefined} optimization_score_uplift
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.optimization_score_uplift = null;
    
                            /**
                             * Metrics optimization_score_url.
                             * @member {string|null|undefined} optimization_score_url
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.optimization_score_url = null;
    
                            /**
                             * Metrics organic_clicks.
                             * @member {number|null|undefined} organic_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.organic_clicks = null;
    
                            /**
                             * Metrics organic_clicks_per_query.
                             * @member {number|null|undefined} organic_clicks_per_query
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.organic_clicks_per_query = null;
    
                            /**
                             * Metrics organic_impressions.
                             * @member {number|null|undefined} organic_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.organic_impressions = null;
    
                            /**
                             * Metrics organic_impressions_per_query.
                             * @member {number|null|undefined} organic_impressions_per_query
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.organic_impressions_per_query = null;
    
                            /**
                             * Metrics organic_queries.
                             * @member {number|null|undefined} organic_queries
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.organic_queries = null;
    
                            /**
                             * Metrics percent_new_visitors.
                             * @member {number|null|undefined} percent_new_visitors
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.percent_new_visitors = null;
    
                            /**
                             * Metrics phone_calls.
                             * @member {number|null|undefined} phone_calls
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.phone_calls = null;
    
                            /**
                             * Metrics phone_impressions.
                             * @member {number|null|undefined} phone_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.phone_impressions = null;
    
                            /**
                             * Metrics phone_through_rate.
                             * @member {number|null|undefined} phone_through_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.phone_through_rate = null;
    
                            /**
                             * Metrics relative_ctr.
                             * @member {number|null|undefined} relative_ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.relative_ctr = null;
    
                            /**
                             * Metrics search_absolute_top_impression_share.
                             * @member {number|null|undefined} search_absolute_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_absolute_top_impression_share = null;
    
                            /**
                             * Metrics search_budget_lost_absolute_top_impression_share.
                             * @member {number|null|undefined} search_budget_lost_absolute_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_budget_lost_absolute_top_impression_share = null;
    
                            /**
                             * Metrics search_budget_lost_impression_share.
                             * @member {number|null|undefined} search_budget_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_budget_lost_impression_share = null;
    
                            /**
                             * Metrics search_budget_lost_top_impression_share.
                             * @member {number|null|undefined} search_budget_lost_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_budget_lost_top_impression_share = null;
    
                            /**
                             * Metrics search_click_share.
                             * @member {number|null|undefined} search_click_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_click_share = null;
    
                            /**
                             * Metrics search_exact_match_impression_share.
                             * @member {number|null|undefined} search_exact_match_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_exact_match_impression_share = null;
    
                            /**
                             * Metrics search_impression_share.
                             * @member {number|null|undefined} search_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_impression_share = null;
    
                            /**
                             * Metrics search_rank_lost_absolute_top_impression_share.
                             * @member {number|null|undefined} search_rank_lost_absolute_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_rank_lost_absolute_top_impression_share = null;
    
                            /**
                             * Metrics search_rank_lost_impression_share.
                             * @member {number|null|undefined} search_rank_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_rank_lost_impression_share = null;
    
                            /**
                             * Metrics search_rank_lost_top_impression_share.
                             * @member {number|null|undefined} search_rank_lost_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_rank_lost_top_impression_share = null;
    
                            /**
                             * Metrics search_top_impression_share.
                             * @member {number|null|undefined} search_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.search_top_impression_share = null;
    
                            /**
                             * Metrics speed_score.
                             * @member {number|null|undefined} speed_score
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.speed_score = null;
    
                            /**
                             * Metrics top_impression_percentage.
                             * @member {number|null|undefined} top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.top_impression_percentage = null;
    
                            /**
                             * Metrics valid_accelerated_mobile_pages_clicks_percentage.
                             * @member {number|null|undefined} valid_accelerated_mobile_pages_clicks_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.valid_accelerated_mobile_pages_clicks_percentage = null;
    
                            /**
                             * Metrics value_per_all_conversions.
                             * @member {number|null|undefined} value_per_all_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.value_per_all_conversions = null;
    
                            /**
                             * Metrics value_per_all_conversions_by_conversion_date.
                             * @member {number|null|undefined} value_per_all_conversions_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.value_per_all_conversions_by_conversion_date = null;
    
                            /**
                             * Metrics value_per_conversion.
                             * @member {number|null|undefined} value_per_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.value_per_conversion = null;
    
                            /**
                             * Metrics value_per_conversions_by_conversion_date.
                             * @member {number|null|undefined} value_per_conversions_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.value_per_conversions_by_conversion_date = null;
    
                            /**
                             * Metrics value_per_current_model_attributed_conversion.
                             * @member {number|null|undefined} value_per_current_model_attributed_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.value_per_current_model_attributed_conversion = null;
    
                            /**
                             * Metrics video_quartile_p100_rate.
                             * @member {number|null|undefined} video_quartile_p100_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.video_quartile_p100_rate = null;
    
                            /**
                             * Metrics video_quartile_p25_rate.
                             * @member {number|null|undefined} video_quartile_p25_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.video_quartile_p25_rate = null;
    
                            /**
                             * Metrics video_quartile_p50_rate.
                             * @member {number|null|undefined} video_quartile_p50_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.video_quartile_p50_rate = null;
    
                            /**
                             * Metrics video_quartile_p75_rate.
                             * @member {number|null|undefined} video_quartile_p75_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.video_quartile_p75_rate = null;
    
                            /**
                             * Metrics video_view_rate.
                             * @member {number|null|undefined} video_view_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.video_view_rate = null;
    
                            /**
                             * Metrics video_views.
                             * @member {number|null|undefined} video_views
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.video_views = null;
    
                            /**
                             * Metrics view_through_conversions.
                             * @member {number|null|undefined} view_through_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.view_through_conversions = null;
    
                            /**
                             * Metrics sk_ad_network_conversions.
                             * @member {number} sk_ad_network_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Metrics.prototype.sk_ad_network_conversions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Metrics _absolute_top_impression_percentage.
                             * @member {"absolute_top_impression_percentage"|undefined} _absolute_top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_absolute_top_impression_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["absolute_top_impression_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _active_view_cpm.
                             * @member {"active_view_cpm"|undefined} _active_view_cpm
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_active_view_cpm", {
                                get: $util.oneOfGetter($oneOfFields = ["active_view_cpm"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _active_view_ctr.
                             * @member {"active_view_ctr"|undefined} _active_view_ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_active_view_ctr", {
                                get: $util.oneOfGetter($oneOfFields = ["active_view_ctr"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _active_view_impressions.
                             * @member {"active_view_impressions"|undefined} _active_view_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_active_view_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["active_view_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _active_view_measurability.
                             * @member {"active_view_measurability"|undefined} _active_view_measurability
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_active_view_measurability", {
                                get: $util.oneOfGetter($oneOfFields = ["active_view_measurability"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _active_view_measurable_cost_micros.
                             * @member {"active_view_measurable_cost_micros"|undefined} _active_view_measurable_cost_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_active_view_measurable_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["active_view_measurable_cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _active_view_measurable_impressions.
                             * @member {"active_view_measurable_impressions"|undefined} _active_view_measurable_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_active_view_measurable_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["active_view_measurable_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _active_view_viewability.
                             * @member {"active_view_viewability"|undefined} _active_view_viewability
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_active_view_viewability", {
                                get: $util.oneOfGetter($oneOfFields = ["active_view_viewability"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_interactions_rate.
                             * @member {"all_conversions_from_interactions_rate"|undefined} _all_conversions_from_interactions_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_interactions_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_interactions_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_value.
                             * @member {"all_conversions_value"|undefined} _all_conversions_value
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions.
                             * @member {"all_conversions"|undefined} _all_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_value_per_cost.
                             * @member {"all_conversions_value_per_cost"|undefined} _all_conversions_value_per_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_value_per_cost", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_value_per_cost"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_click_to_call.
                             * @member {"all_conversions_from_click_to_call"|undefined} _all_conversions_from_click_to_call
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_click_to_call", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_click_to_call"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_directions.
                             * @member {"all_conversions_from_directions"|undefined} _all_conversions_from_directions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_directions", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_directions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_interactions_value_per_interaction.
                             * @member {"all_conversions_from_interactions_value_per_interaction"|undefined} _all_conversions_from_interactions_value_per_interaction
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_interactions_value_per_interaction", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_interactions_value_per_interaction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_menu.
                             * @member {"all_conversions_from_menu"|undefined} _all_conversions_from_menu
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_menu", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_menu"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_order.
                             * @member {"all_conversions_from_order"|undefined} _all_conversions_from_order
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_order", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_order"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_other_engagement.
                             * @member {"all_conversions_from_other_engagement"|undefined} _all_conversions_from_other_engagement
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_other_engagement", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_other_engagement"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_store_visit.
                             * @member {"all_conversions_from_store_visit"|undefined} _all_conversions_from_store_visit
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_store_visit", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_store_visit"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _all_conversions_from_store_website.
                             * @member {"all_conversions_from_store_website"|undefined} _all_conversions_from_store_website
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_all_conversions_from_store_website", {
                                get: $util.oneOfGetter($oneOfFields = ["all_conversions_from_store_website"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _auction_insight_search_absolute_top_impression_percentage.
                             * @member {"auction_insight_search_absolute_top_impression_percentage"|undefined} _auction_insight_search_absolute_top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_auction_insight_search_absolute_top_impression_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["auction_insight_search_absolute_top_impression_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _auction_insight_search_impression_share.
                             * @member {"auction_insight_search_impression_share"|undefined} _auction_insight_search_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_auction_insight_search_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["auction_insight_search_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _auction_insight_search_outranking_share.
                             * @member {"auction_insight_search_outranking_share"|undefined} _auction_insight_search_outranking_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_auction_insight_search_outranking_share", {
                                get: $util.oneOfGetter($oneOfFields = ["auction_insight_search_outranking_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _auction_insight_search_overlap_rate.
                             * @member {"auction_insight_search_overlap_rate"|undefined} _auction_insight_search_overlap_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_auction_insight_search_overlap_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["auction_insight_search_overlap_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _auction_insight_search_position_above_rate.
                             * @member {"auction_insight_search_position_above_rate"|undefined} _auction_insight_search_position_above_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_auction_insight_search_position_above_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["auction_insight_search_position_above_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _auction_insight_search_top_impression_percentage.
                             * @member {"auction_insight_search_top_impression_percentage"|undefined} _auction_insight_search_top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_auction_insight_search_top_impression_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["auction_insight_search_top_impression_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _average_cost.
                             * @member {"average_cost"|undefined} _average_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_average_cost", {
                                get: $util.oneOfGetter($oneOfFields = ["average_cost"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _average_cpc.
                             * @member {"average_cpc"|undefined} _average_cpc
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_average_cpc", {
                                get: $util.oneOfGetter($oneOfFields = ["average_cpc"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _average_cpe.
                             * @member {"average_cpe"|undefined} _average_cpe
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_average_cpe", {
                                get: $util.oneOfGetter($oneOfFields = ["average_cpe"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _average_cpm.
                             * @member {"average_cpm"|undefined} _average_cpm
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_average_cpm", {
                                get: $util.oneOfGetter($oneOfFields = ["average_cpm"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _average_cpv.
                             * @member {"average_cpv"|undefined} _average_cpv
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_average_cpv", {
                                get: $util.oneOfGetter($oneOfFields = ["average_cpv"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _average_page_views.
                             * @member {"average_page_views"|undefined} _average_page_views
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_average_page_views", {
                                get: $util.oneOfGetter($oneOfFields = ["average_page_views"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _average_time_on_site.
                             * @member {"average_time_on_site"|undefined} _average_time_on_site
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_average_time_on_site", {
                                get: $util.oneOfGetter($oneOfFields = ["average_time_on_site"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _benchmark_average_max_cpc.
                             * @member {"benchmark_average_max_cpc"|undefined} _benchmark_average_max_cpc
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_benchmark_average_max_cpc", {
                                get: $util.oneOfGetter($oneOfFields = ["benchmark_average_max_cpc"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _biddable_app_install_conversions.
                             * @member {"biddable_app_install_conversions"|undefined} _biddable_app_install_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_biddable_app_install_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_app_install_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _biddable_app_post_install_conversions.
                             * @member {"biddable_app_post_install_conversions"|undefined} _biddable_app_post_install_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_biddable_app_post_install_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_app_post_install_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _benchmark_ctr.
                             * @member {"benchmark_ctr"|undefined} _benchmark_ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_benchmark_ctr", {
                                get: $util.oneOfGetter($oneOfFields = ["benchmark_ctr"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _bounce_rate.
                             * @member {"bounce_rate"|undefined} _bounce_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_bounce_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["bounce_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _clicks.
                             * @member {"clicks"|undefined} _clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _combined_clicks.
                             * @member {"combined_clicks"|undefined} _combined_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_combined_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["combined_clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _combined_clicks_per_query.
                             * @member {"combined_clicks_per_query"|undefined} _combined_clicks_per_query
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_combined_clicks_per_query", {
                                get: $util.oneOfGetter($oneOfFields = ["combined_clicks_per_query"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _combined_queries.
                             * @member {"combined_queries"|undefined} _combined_queries
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_combined_queries", {
                                get: $util.oneOfGetter($oneOfFields = ["combined_queries"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _content_budget_lost_impression_share.
                             * @member {"content_budget_lost_impression_share"|undefined} _content_budget_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_content_budget_lost_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["content_budget_lost_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _content_impression_share.
                             * @member {"content_impression_share"|undefined} _content_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_content_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["content_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _conversion_last_received_request_date_time.
                             * @member {"conversion_last_received_request_date_time"|undefined} _conversion_last_received_request_date_time
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_conversion_last_received_request_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_last_received_request_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _conversion_last_conversion_date.
                             * @member {"conversion_last_conversion_date"|undefined} _conversion_last_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_conversion_last_conversion_date", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_last_conversion_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _content_rank_lost_impression_share.
                             * @member {"content_rank_lost_impression_share"|undefined} _content_rank_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_content_rank_lost_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["content_rank_lost_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _conversions_from_interactions_rate.
                             * @member {"conversions_from_interactions_rate"|undefined} _conversions_from_interactions_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_conversions_from_interactions_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["conversions_from_interactions_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _conversions_value.
                             * @member {"conversions_value"|undefined} _conversions_value
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _conversions_value_per_cost.
                             * @member {"conversions_value_per_cost"|undefined} _conversions_value_per_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_conversions_value_per_cost", {
                                get: $util.oneOfGetter($oneOfFields = ["conversions_value_per_cost"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _conversions_from_interactions_value_per_interaction.
                             * @member {"conversions_from_interactions_value_per_interaction"|undefined} _conversions_from_interactions_value_per_interaction
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_conversions_from_interactions_value_per_interaction", {
                                get: $util.oneOfGetter($oneOfFields = ["conversions_from_interactions_value_per_interaction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _conversions.
                             * @member {"conversions"|undefined} _conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _cost_per_all_conversions.
                             * @member {"cost_per_all_conversions"|undefined} _cost_per_all_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_cost_per_all_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_per_all_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _cost_per_conversion.
                             * @member {"cost_per_conversion"|undefined} _cost_per_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_cost_per_conversion", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_per_conversion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _cost_per_current_model_attributed_conversion.
                             * @member {"cost_per_current_model_attributed_conversion"|undefined} _cost_per_current_model_attributed_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_cost_per_current_model_attributed_conversion", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_per_current_model_attributed_conversion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _cross_device_conversions.
                             * @member {"cross_device_conversions"|undefined} _cross_device_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_cross_device_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["cross_device_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _ctr.
                             * @member {"ctr"|undefined} _ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_ctr", {
                                get: $util.oneOfGetter($oneOfFields = ["ctr"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _current_model_attributed_conversions.
                             * @member {"current_model_attributed_conversions"|undefined} _current_model_attributed_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_current_model_attributed_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["current_model_attributed_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _current_model_attributed_conversions_from_interactions_rate.
                             * @member {"current_model_attributed_conversions_from_interactions_rate"|undefined} _current_model_attributed_conversions_from_interactions_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_current_model_attributed_conversions_from_interactions_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["current_model_attributed_conversions_from_interactions_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _current_model_attributed_conversions_from_interactions_value_per_interaction.
                             * @member {"current_model_attributed_conversions_from_interactions_value_per_interaction"|undefined} _current_model_attributed_conversions_from_interactions_value_per_interaction
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_current_model_attributed_conversions_from_interactions_value_per_interaction", {
                                get: $util.oneOfGetter($oneOfFields = ["current_model_attributed_conversions_from_interactions_value_per_interaction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _current_model_attributed_conversions_value.
                             * @member {"current_model_attributed_conversions_value"|undefined} _current_model_attributed_conversions_value
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_current_model_attributed_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["current_model_attributed_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _current_model_attributed_conversions_value_per_cost.
                             * @member {"current_model_attributed_conversions_value_per_cost"|undefined} _current_model_attributed_conversions_value_per_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_current_model_attributed_conversions_value_per_cost", {
                                get: $util.oneOfGetter($oneOfFields = ["current_model_attributed_conversions_value_per_cost"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _engagement_rate.
                             * @member {"engagement_rate"|undefined} _engagement_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_engagement_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["engagement_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _engagements.
                             * @member {"engagements"|undefined} _engagements
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_engagements", {
                                get: $util.oneOfGetter($oneOfFields = ["engagements"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _hotel_average_lead_value_micros.
                             * @member {"hotel_average_lead_value_micros"|undefined} _hotel_average_lead_value_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_hotel_average_lead_value_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_average_lead_value_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _hotel_commission_rate_micros.
                             * @member {"hotel_commission_rate_micros"|undefined} _hotel_commission_rate_micros
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_hotel_commission_rate_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_commission_rate_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _hotel_expected_commission_cost.
                             * @member {"hotel_expected_commission_cost"|undefined} _hotel_expected_commission_cost
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_hotel_expected_commission_cost", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_expected_commission_cost"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _hotel_price_difference_percentage.
                             * @member {"hotel_price_difference_percentage"|undefined} _hotel_price_difference_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_hotel_price_difference_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_price_difference_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _hotel_eligible_impressions.
                             * @member {"hotel_eligible_impressions"|undefined} _hotel_eligible_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_hotel_eligible_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_eligible_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _historical_quality_score.
                             * @member {"historical_quality_score"|undefined} _historical_quality_score
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_historical_quality_score", {
                                get: $util.oneOfGetter($oneOfFields = ["historical_quality_score"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _gmail_forwards.
                             * @member {"gmail_forwards"|undefined} _gmail_forwards
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_gmail_forwards", {
                                get: $util.oneOfGetter($oneOfFields = ["gmail_forwards"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _gmail_saves.
                             * @member {"gmail_saves"|undefined} _gmail_saves
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_gmail_saves", {
                                get: $util.oneOfGetter($oneOfFields = ["gmail_saves"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _gmail_secondary_clicks.
                             * @member {"gmail_secondary_clicks"|undefined} _gmail_secondary_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_gmail_secondary_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["gmail_secondary_clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _impressions_from_store_reach.
                             * @member {"impressions_from_store_reach"|undefined} _impressions_from_store_reach
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_impressions_from_store_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions_from_store_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _interaction_rate.
                             * @member {"interaction_rate"|undefined} _interaction_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_interaction_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["interaction_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _interactions.
                             * @member {"interactions"|undefined} _interactions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_interactions", {
                                get: $util.oneOfGetter($oneOfFields = ["interactions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _invalid_click_rate.
                             * @member {"invalid_click_rate"|undefined} _invalid_click_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_invalid_click_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["invalid_click_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _invalid_clicks.
                             * @member {"invalid_clicks"|undefined} _invalid_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_invalid_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["invalid_clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _message_chats.
                             * @member {"message_chats"|undefined} _message_chats
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_message_chats", {
                                get: $util.oneOfGetter($oneOfFields = ["message_chats"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _message_impressions.
                             * @member {"message_impressions"|undefined} _message_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_message_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["message_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _message_chat_rate.
                             * @member {"message_chat_rate"|undefined} _message_chat_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_message_chat_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["message_chat_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _mobile_friendly_clicks_percentage.
                             * @member {"mobile_friendly_clicks_percentage"|undefined} _mobile_friendly_clicks_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_mobile_friendly_clicks_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["mobile_friendly_clicks_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _optimization_score_uplift.
                             * @member {"optimization_score_uplift"|undefined} _optimization_score_uplift
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_optimization_score_uplift", {
                                get: $util.oneOfGetter($oneOfFields = ["optimization_score_uplift"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _optimization_score_url.
                             * @member {"optimization_score_url"|undefined} _optimization_score_url
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_optimization_score_url", {
                                get: $util.oneOfGetter($oneOfFields = ["optimization_score_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _organic_clicks.
                             * @member {"organic_clicks"|undefined} _organic_clicks
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_organic_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["organic_clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _organic_clicks_per_query.
                             * @member {"organic_clicks_per_query"|undefined} _organic_clicks_per_query
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_organic_clicks_per_query", {
                                get: $util.oneOfGetter($oneOfFields = ["organic_clicks_per_query"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _organic_impressions.
                             * @member {"organic_impressions"|undefined} _organic_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_organic_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["organic_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _organic_impressions_per_query.
                             * @member {"organic_impressions_per_query"|undefined} _organic_impressions_per_query
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_organic_impressions_per_query", {
                                get: $util.oneOfGetter($oneOfFields = ["organic_impressions_per_query"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _organic_queries.
                             * @member {"organic_queries"|undefined} _organic_queries
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_organic_queries", {
                                get: $util.oneOfGetter($oneOfFields = ["organic_queries"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _percent_new_visitors.
                             * @member {"percent_new_visitors"|undefined} _percent_new_visitors
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_percent_new_visitors", {
                                get: $util.oneOfGetter($oneOfFields = ["percent_new_visitors"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _phone_calls.
                             * @member {"phone_calls"|undefined} _phone_calls
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_phone_calls", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_calls"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _phone_impressions.
                             * @member {"phone_impressions"|undefined} _phone_impressions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_phone_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _phone_through_rate.
                             * @member {"phone_through_rate"|undefined} _phone_through_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_phone_through_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_through_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _relative_ctr.
                             * @member {"relative_ctr"|undefined} _relative_ctr
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_relative_ctr", {
                                get: $util.oneOfGetter($oneOfFields = ["relative_ctr"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_absolute_top_impression_share.
                             * @member {"search_absolute_top_impression_share"|undefined} _search_absolute_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_absolute_top_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_absolute_top_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_budget_lost_absolute_top_impression_share.
                             * @member {"search_budget_lost_absolute_top_impression_share"|undefined} _search_budget_lost_absolute_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_budget_lost_absolute_top_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_budget_lost_absolute_top_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_budget_lost_impression_share.
                             * @member {"search_budget_lost_impression_share"|undefined} _search_budget_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_budget_lost_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_budget_lost_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_budget_lost_top_impression_share.
                             * @member {"search_budget_lost_top_impression_share"|undefined} _search_budget_lost_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_budget_lost_top_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_budget_lost_top_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_click_share.
                             * @member {"search_click_share"|undefined} _search_click_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_click_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_click_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_exact_match_impression_share.
                             * @member {"search_exact_match_impression_share"|undefined} _search_exact_match_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_exact_match_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_exact_match_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_impression_share.
                             * @member {"search_impression_share"|undefined} _search_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_rank_lost_absolute_top_impression_share.
                             * @member {"search_rank_lost_absolute_top_impression_share"|undefined} _search_rank_lost_absolute_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_rank_lost_absolute_top_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_rank_lost_absolute_top_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_rank_lost_impression_share.
                             * @member {"search_rank_lost_impression_share"|undefined} _search_rank_lost_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_rank_lost_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_rank_lost_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_rank_lost_top_impression_share.
                             * @member {"search_rank_lost_top_impression_share"|undefined} _search_rank_lost_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_rank_lost_top_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_rank_lost_top_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _search_top_impression_share.
                             * @member {"search_top_impression_share"|undefined} _search_top_impression_share
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_search_top_impression_share", {
                                get: $util.oneOfGetter($oneOfFields = ["search_top_impression_share"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _speed_score.
                             * @member {"speed_score"|undefined} _speed_score
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_speed_score", {
                                get: $util.oneOfGetter($oneOfFields = ["speed_score"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _top_impression_percentage.
                             * @member {"top_impression_percentage"|undefined} _top_impression_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_top_impression_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["top_impression_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _valid_accelerated_mobile_pages_clicks_percentage.
                             * @member {"valid_accelerated_mobile_pages_clicks_percentage"|undefined} _valid_accelerated_mobile_pages_clicks_percentage
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_valid_accelerated_mobile_pages_clicks_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["valid_accelerated_mobile_pages_clicks_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _value_per_all_conversions.
                             * @member {"value_per_all_conversions"|undefined} _value_per_all_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_value_per_all_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["value_per_all_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _value_per_all_conversions_by_conversion_date.
                             * @member {"value_per_all_conversions_by_conversion_date"|undefined} _value_per_all_conversions_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_value_per_all_conversions_by_conversion_date", {
                                get: $util.oneOfGetter($oneOfFields = ["value_per_all_conversions_by_conversion_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _value_per_conversion.
                             * @member {"value_per_conversion"|undefined} _value_per_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_value_per_conversion", {
                                get: $util.oneOfGetter($oneOfFields = ["value_per_conversion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _value_per_conversions_by_conversion_date.
                             * @member {"value_per_conversions_by_conversion_date"|undefined} _value_per_conversions_by_conversion_date
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_value_per_conversions_by_conversion_date", {
                                get: $util.oneOfGetter($oneOfFields = ["value_per_conversions_by_conversion_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _value_per_current_model_attributed_conversion.
                             * @member {"value_per_current_model_attributed_conversion"|undefined} _value_per_current_model_attributed_conversion
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_value_per_current_model_attributed_conversion", {
                                get: $util.oneOfGetter($oneOfFields = ["value_per_current_model_attributed_conversion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _video_quartile_p100_rate.
                             * @member {"video_quartile_p100_rate"|undefined} _video_quartile_p100_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_video_quartile_p100_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["video_quartile_p100_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _video_quartile_p25_rate.
                             * @member {"video_quartile_p25_rate"|undefined} _video_quartile_p25_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_video_quartile_p25_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["video_quartile_p25_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _video_quartile_p50_rate.
                             * @member {"video_quartile_p50_rate"|undefined} _video_quartile_p50_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_video_quartile_p50_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["video_quartile_p50_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _video_quartile_p75_rate.
                             * @member {"video_quartile_p75_rate"|undefined} _video_quartile_p75_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_video_quartile_p75_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["video_quartile_p75_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _video_view_rate.
                             * @member {"video_view_rate"|undefined} _video_view_rate
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_video_view_rate", {
                                get: $util.oneOfGetter($oneOfFields = ["video_view_rate"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _video_views.
                             * @member {"video_views"|undefined} _video_views
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_video_views", {
                                get: $util.oneOfGetter($oneOfFields = ["video_views"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Metrics _view_through_conversions.
                             * @member {"view_through_conversions"|undefined} _view_through_conversions
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             */
                            Object.defineProperty(Metrics.prototype, "_view_through_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["view_through_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Metrics instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IMetrics=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.Metrics} Metrics instance
                             */
                            Metrics.create = function create(properties) {
                                return new Metrics(properties);
                            };
    
                            /**
                             * Encodes the specified Metrics message. Does not implicitly {@link google.ads.googleads.v11.common.Metrics.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IMetrics} m Metrics message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Metrics.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.historical_creative_quality_score != null && Object.hasOwnProperty.call(m, "historical_creative_quality_score"))
                                    w.uint32(640).int32(m.historical_creative_quality_score);
                                if (m.historical_landing_page_quality_score != null && Object.hasOwnProperty.call(m, "historical_landing_page_quality_score"))
                                    w.uint32(648).int32(m.historical_landing_page_quality_score);
                                if (m.historical_search_predicted_ctr != null && Object.hasOwnProperty.call(m, "historical_search_predicted_ctr"))
                                    w.uint32(664).int32(m.historical_search_predicted_ctr);
                                if (m.interaction_event_types != null && m.interaction_event_types.length) {
                                    w.uint32(802).fork();
                                    for (var i = 0; i < m.interaction_event_types.length; ++i)
                                        w.int32(m.interaction_event_types[i]);
                                    w.ldelim();
                                }
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(1048).int64(m.clicks);
                                if (m.video_quartile_p100_rate != null && Object.hasOwnProperty.call(m, "video_quartile_p100_rate"))
                                    w.uint32(1057).double(m.video_quartile_p100_rate);
                                if (m.video_quartile_p25_rate != null && Object.hasOwnProperty.call(m, "video_quartile_p25_rate"))
                                    w.uint32(1065).double(m.video_quartile_p25_rate);
                                if (m.video_quartile_p50_rate != null && Object.hasOwnProperty.call(m, "video_quartile_p50_rate"))
                                    w.uint32(1073).double(m.video_quartile_p50_rate);
                                if (m.video_quartile_p75_rate != null && Object.hasOwnProperty.call(m, "video_quartile_p75_rate"))
                                    w.uint32(1081).double(m.video_quartile_p75_rate);
                                if (m.search_absolute_top_impression_share != null && Object.hasOwnProperty.call(m, "search_absolute_top_impression_share"))
                                    w.uint32(1089).double(m.search_absolute_top_impression_share);
                                if (m.search_budget_lost_absolute_top_impression_share != null && Object.hasOwnProperty.call(m, "search_budget_lost_absolute_top_impression_share"))
                                    w.uint32(1097).double(m.search_budget_lost_absolute_top_impression_share);
                                if (m.search_budget_lost_impression_share != null && Object.hasOwnProperty.call(m, "search_budget_lost_impression_share"))
                                    w.uint32(1105).double(m.search_budget_lost_impression_share);
                                if (m.search_budget_lost_top_impression_share != null && Object.hasOwnProperty.call(m, "search_budget_lost_top_impression_share"))
                                    w.uint32(1113).double(m.search_budget_lost_top_impression_share);
                                if (m.search_click_share != null && Object.hasOwnProperty.call(m, "search_click_share"))
                                    w.uint32(1121).double(m.search_click_share);
                                if (m.search_exact_match_impression_share != null && Object.hasOwnProperty.call(m, "search_exact_match_impression_share"))
                                    w.uint32(1129).double(m.search_exact_match_impression_share);
                                if (m.search_impression_share != null && Object.hasOwnProperty.call(m, "search_impression_share"))
                                    w.uint32(1137).double(m.search_impression_share);
                                if (m.search_rank_lost_absolute_top_impression_share != null && Object.hasOwnProperty.call(m, "search_rank_lost_absolute_top_impression_share"))
                                    w.uint32(1145).double(m.search_rank_lost_absolute_top_impression_share);
                                if (m.search_rank_lost_impression_share != null && Object.hasOwnProperty.call(m, "search_rank_lost_impression_share"))
                                    w.uint32(1153).double(m.search_rank_lost_impression_share);
                                if (m.search_rank_lost_top_impression_share != null && Object.hasOwnProperty.call(m, "search_rank_lost_top_impression_share"))
                                    w.uint32(1161).double(m.search_rank_lost_top_impression_share);
                                if (m.search_top_impression_share != null && Object.hasOwnProperty.call(m, "search_top_impression_share"))
                                    w.uint32(1169).double(m.search_top_impression_share);
                                if (m.speed_score != null && Object.hasOwnProperty.call(m, "speed_score"))
                                    w.uint32(1176).int64(m.speed_score);
                                if (m.top_impression_percentage != null && Object.hasOwnProperty.call(m, "top_impression_percentage"))
                                    w.uint32(1185).double(m.top_impression_percentage);
                                if (m.valid_accelerated_mobile_pages_clicks_percentage != null && Object.hasOwnProperty.call(m, "valid_accelerated_mobile_pages_clicks_percentage"))
                                    w.uint32(1193).double(m.valid_accelerated_mobile_pages_clicks_percentage);
                                if (m.value_per_all_conversions != null && Object.hasOwnProperty.call(m, "value_per_all_conversions"))
                                    w.uint32(1201).double(m.value_per_all_conversions);
                                if (m.value_per_conversion != null && Object.hasOwnProperty.call(m, "value_per_conversion"))
                                    w.uint32(1209).double(m.value_per_conversion);
                                if (m.value_per_current_model_attributed_conversion != null && Object.hasOwnProperty.call(m, "value_per_current_model_attributed_conversion"))
                                    w.uint32(1217).double(m.value_per_current_model_attributed_conversion);
                                if (m.video_view_rate != null && Object.hasOwnProperty.call(m, "video_view_rate"))
                                    w.uint32(1225).double(m.video_view_rate);
                                if (m.video_views != null && Object.hasOwnProperty.call(m, "video_views"))
                                    w.uint32(1232).int64(m.video_views);
                                if (m.view_through_conversions != null && Object.hasOwnProperty.call(m, "view_through_conversions"))
                                    w.uint32(1240).int64(m.view_through_conversions);
                                if (m.combined_clicks != null && Object.hasOwnProperty.call(m, "combined_clicks"))
                                    w.uint32(1248).int64(m.combined_clicks);
                                if (m.combined_clicks_per_query != null && Object.hasOwnProperty.call(m, "combined_clicks_per_query"))
                                    w.uint32(1257).double(m.combined_clicks_per_query);
                                if (m.combined_queries != null && Object.hasOwnProperty.call(m, "combined_queries"))
                                    w.uint32(1264).int64(m.combined_queries);
                                if (m.content_budget_lost_impression_share != null && Object.hasOwnProperty.call(m, "content_budget_lost_impression_share"))
                                    w.uint32(1273).double(m.content_budget_lost_impression_share);
                                if (m.content_impression_share != null && Object.hasOwnProperty.call(m, "content_impression_share"))
                                    w.uint32(1281).double(m.content_impression_share);
                                if (m.conversion_last_received_request_date_time != null && Object.hasOwnProperty.call(m, "conversion_last_received_request_date_time"))
                                    w.uint32(1290).string(m.conversion_last_received_request_date_time);
                                if (m.conversion_last_conversion_date != null && Object.hasOwnProperty.call(m, "conversion_last_conversion_date"))
                                    w.uint32(1298).string(m.conversion_last_conversion_date);
                                if (m.content_rank_lost_impression_share != null && Object.hasOwnProperty.call(m, "content_rank_lost_impression_share"))
                                    w.uint32(1305).double(m.content_rank_lost_impression_share);
                                if (m.conversions_from_interactions_rate != null && Object.hasOwnProperty.call(m, "conversions_from_interactions_rate"))
                                    w.uint32(1313).double(m.conversions_from_interactions_rate);
                                if (m.conversions_value != null && Object.hasOwnProperty.call(m, "conversions_value"))
                                    w.uint32(1321).double(m.conversions_value);
                                if (m.conversions_value_per_cost != null && Object.hasOwnProperty.call(m, "conversions_value_per_cost"))
                                    w.uint32(1329).double(m.conversions_value_per_cost);
                                if (m.conversions_from_interactions_value_per_interaction != null && Object.hasOwnProperty.call(m, "conversions_from_interactions_value_per_interaction"))
                                    w.uint32(1337).double(m.conversions_from_interactions_value_per_interaction);
                                if (m.conversions != null && Object.hasOwnProperty.call(m, "conversions"))
                                    w.uint32(1345).double(m.conversions);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(1352).int64(m.cost_micros);
                                if (m.cost_per_all_conversions != null && Object.hasOwnProperty.call(m, "cost_per_all_conversions"))
                                    w.uint32(1361).double(m.cost_per_all_conversions);
                                if (m.cost_per_conversion != null && Object.hasOwnProperty.call(m, "cost_per_conversion"))
                                    w.uint32(1369).double(m.cost_per_conversion);
                                if (m.cost_per_current_model_attributed_conversion != null && Object.hasOwnProperty.call(m, "cost_per_current_model_attributed_conversion"))
                                    w.uint32(1377).double(m.cost_per_current_model_attributed_conversion);
                                if (m.cross_device_conversions != null && Object.hasOwnProperty.call(m, "cross_device_conversions"))
                                    w.uint32(1385).double(m.cross_device_conversions);
                                if (m.ctr != null && Object.hasOwnProperty.call(m, "ctr"))
                                    w.uint32(1393).double(m.ctr);
                                if (m.current_model_attributed_conversions != null && Object.hasOwnProperty.call(m, "current_model_attributed_conversions"))
                                    w.uint32(1401).double(m.current_model_attributed_conversions);
                                if (m.current_model_attributed_conversions_from_interactions_rate != null && Object.hasOwnProperty.call(m, "current_model_attributed_conversions_from_interactions_rate"))
                                    w.uint32(1409).double(m.current_model_attributed_conversions_from_interactions_rate);
                                if (m.current_model_attributed_conversions_from_interactions_value_per_interaction != null && Object.hasOwnProperty.call(m, "current_model_attributed_conversions_from_interactions_value_per_interaction"))
                                    w.uint32(1417).double(m.current_model_attributed_conversions_from_interactions_value_per_interaction);
                                if (m.current_model_attributed_conversions_value != null && Object.hasOwnProperty.call(m, "current_model_attributed_conversions_value"))
                                    w.uint32(1425).double(m.current_model_attributed_conversions_value);
                                if (m.current_model_attributed_conversions_value_per_cost != null && Object.hasOwnProperty.call(m, "current_model_attributed_conversions_value_per_cost"))
                                    w.uint32(1433).double(m.current_model_attributed_conversions_value_per_cost);
                                if (m.engagement_rate != null && Object.hasOwnProperty.call(m, "engagement_rate"))
                                    w.uint32(1441).double(m.engagement_rate);
                                if (m.engagements != null && Object.hasOwnProperty.call(m, "engagements"))
                                    w.uint32(1448).int64(m.engagements);
                                if (m.absolute_top_impression_percentage != null && Object.hasOwnProperty.call(m, "absolute_top_impression_percentage"))
                                    w.uint32(1465).double(m.absolute_top_impression_percentage);
                                if (m.active_view_cpm != null && Object.hasOwnProperty.call(m, "active_view_cpm"))
                                    w.uint32(1473).double(m.active_view_cpm);
                                if (m.active_view_ctr != null && Object.hasOwnProperty.call(m, "active_view_ctr"))
                                    w.uint32(1481).double(m.active_view_ctr);
                                if (m.active_view_impressions != null && Object.hasOwnProperty.call(m, "active_view_impressions"))
                                    w.uint32(1488).int64(m.active_view_impressions);
                                if (m.active_view_measurability != null && Object.hasOwnProperty.call(m, "active_view_measurability"))
                                    w.uint32(1497).double(m.active_view_measurability);
                                if (m.active_view_measurable_cost_micros != null && Object.hasOwnProperty.call(m, "active_view_measurable_cost_micros"))
                                    w.uint32(1504).int64(m.active_view_measurable_cost_micros);
                                if (m.active_view_measurable_impressions != null && Object.hasOwnProperty.call(m, "active_view_measurable_impressions"))
                                    w.uint32(1512).int64(m.active_view_measurable_impressions);
                                if (m.active_view_viewability != null && Object.hasOwnProperty.call(m, "active_view_viewability"))
                                    w.uint32(1521).double(m.active_view_viewability);
                                if (m.all_conversions_from_interactions_rate != null && Object.hasOwnProperty.call(m, "all_conversions_from_interactions_rate"))
                                    w.uint32(1529).double(m.all_conversions_from_interactions_rate);
                                if (m.all_conversions_value != null && Object.hasOwnProperty.call(m, "all_conversions_value"))
                                    w.uint32(1537).double(m.all_conversions_value);
                                if (m.all_conversions != null && Object.hasOwnProperty.call(m, "all_conversions"))
                                    w.uint32(1545).double(m.all_conversions);
                                if (m.all_conversions_value_per_cost != null && Object.hasOwnProperty.call(m, "all_conversions_value_per_cost"))
                                    w.uint32(1553).double(m.all_conversions_value_per_cost);
                                if (m.all_conversions_from_click_to_call != null && Object.hasOwnProperty.call(m, "all_conversions_from_click_to_call"))
                                    w.uint32(1561).double(m.all_conversions_from_click_to_call);
                                if (m.all_conversions_from_directions != null && Object.hasOwnProperty.call(m, "all_conversions_from_directions"))
                                    w.uint32(1569).double(m.all_conversions_from_directions);
                                if (m.all_conversions_from_interactions_value_per_interaction != null && Object.hasOwnProperty.call(m, "all_conversions_from_interactions_value_per_interaction"))
                                    w.uint32(1577).double(m.all_conversions_from_interactions_value_per_interaction);
                                if (m.all_conversions_from_menu != null && Object.hasOwnProperty.call(m, "all_conversions_from_menu"))
                                    w.uint32(1585).double(m.all_conversions_from_menu);
                                if (m.all_conversions_from_order != null && Object.hasOwnProperty.call(m, "all_conversions_from_order"))
                                    w.uint32(1593).double(m.all_conversions_from_order);
                                if (m.all_conversions_from_other_engagement != null && Object.hasOwnProperty.call(m, "all_conversions_from_other_engagement"))
                                    w.uint32(1601).double(m.all_conversions_from_other_engagement);
                                if (m.all_conversions_from_store_visit != null && Object.hasOwnProperty.call(m, "all_conversions_from_store_visit"))
                                    w.uint32(1609).double(m.all_conversions_from_store_visit);
                                if (m.all_conversions_from_store_website != null && Object.hasOwnProperty.call(m, "all_conversions_from_store_website"))
                                    w.uint32(1617).double(m.all_conversions_from_store_website);
                                if (m.average_cost != null && Object.hasOwnProperty.call(m, "average_cost"))
                                    w.uint32(1625).double(m.average_cost);
                                if (m.average_cpc != null && Object.hasOwnProperty.call(m, "average_cpc"))
                                    w.uint32(1633).double(m.average_cpc);
                                if (m.average_cpe != null && Object.hasOwnProperty.call(m, "average_cpe"))
                                    w.uint32(1641).double(m.average_cpe);
                                if (m.average_cpm != null && Object.hasOwnProperty.call(m, "average_cpm"))
                                    w.uint32(1649).double(m.average_cpm);
                                if (m.average_cpv != null && Object.hasOwnProperty.call(m, "average_cpv"))
                                    w.uint32(1657).double(m.average_cpv);
                                if (m.average_page_views != null && Object.hasOwnProperty.call(m, "average_page_views"))
                                    w.uint32(1665).double(m.average_page_views);
                                if (m.average_time_on_site != null && Object.hasOwnProperty.call(m, "average_time_on_site"))
                                    w.uint32(1673).double(m.average_time_on_site);
                                if (m.benchmark_average_max_cpc != null && Object.hasOwnProperty.call(m, "benchmark_average_max_cpc"))
                                    w.uint32(1681).double(m.benchmark_average_max_cpc);
                                if (m.benchmark_ctr != null && Object.hasOwnProperty.call(m, "benchmark_ctr"))
                                    w.uint32(1689).double(m.benchmark_ctr);
                                if (m.bounce_rate != null && Object.hasOwnProperty.call(m, "bounce_rate"))
                                    w.uint32(1697).double(m.bounce_rate);
                                if (m.hotel_average_lead_value_micros != null && Object.hasOwnProperty.call(m, "hotel_average_lead_value_micros"))
                                    w.uint32(1705).double(m.hotel_average_lead_value_micros);
                                if (m.hotel_price_difference_percentage != null && Object.hasOwnProperty.call(m, "hotel_price_difference_percentage"))
                                    w.uint32(1713).double(m.hotel_price_difference_percentage);
                                if (m.hotel_eligible_impressions != null && Object.hasOwnProperty.call(m, "hotel_eligible_impressions"))
                                    w.uint32(1720).int64(m.hotel_eligible_impressions);
                                if (m.historical_quality_score != null && Object.hasOwnProperty.call(m, "historical_quality_score"))
                                    w.uint32(1728).int64(m.historical_quality_score);
                                if (m.gmail_forwards != null && Object.hasOwnProperty.call(m, "gmail_forwards"))
                                    w.uint32(1736).int64(m.gmail_forwards);
                                if (m.gmail_saves != null && Object.hasOwnProperty.call(m, "gmail_saves"))
                                    w.uint32(1744).int64(m.gmail_saves);
                                if (m.gmail_secondary_clicks != null && Object.hasOwnProperty.call(m, "gmail_secondary_clicks"))
                                    w.uint32(1752).int64(m.gmail_secondary_clicks);
                                if (m.impressions_from_store_reach != null && Object.hasOwnProperty.call(m, "impressions_from_store_reach"))
                                    w.uint32(1760).int64(m.impressions_from_store_reach);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(1768).int64(m.impressions);
                                if (m.interaction_rate != null && Object.hasOwnProperty.call(m, "interaction_rate"))
                                    w.uint32(1777).double(m.interaction_rate);
                                if (m.interactions != null && Object.hasOwnProperty.call(m, "interactions"))
                                    w.uint32(1784).int64(m.interactions);
                                if (m.invalid_click_rate != null && Object.hasOwnProperty.call(m, "invalid_click_rate"))
                                    w.uint32(1793).double(m.invalid_click_rate);
                                if (m.invalid_clicks != null && Object.hasOwnProperty.call(m, "invalid_clicks"))
                                    w.uint32(1800).int64(m.invalid_clicks);
                                if (m.message_chats != null && Object.hasOwnProperty.call(m, "message_chats"))
                                    w.uint32(1808).int64(m.message_chats);
                                if (m.message_impressions != null && Object.hasOwnProperty.call(m, "message_impressions"))
                                    w.uint32(1816).int64(m.message_impressions);
                                if (m.message_chat_rate != null && Object.hasOwnProperty.call(m, "message_chat_rate"))
                                    w.uint32(1825).double(m.message_chat_rate);
                                if (m.mobile_friendly_clicks_percentage != null && Object.hasOwnProperty.call(m, "mobile_friendly_clicks_percentage"))
                                    w.uint32(1833).double(m.mobile_friendly_clicks_percentage);
                                if (m.organic_clicks != null && Object.hasOwnProperty.call(m, "organic_clicks"))
                                    w.uint32(1840).int64(m.organic_clicks);
                                if (m.organic_clicks_per_query != null && Object.hasOwnProperty.call(m, "organic_clicks_per_query"))
                                    w.uint32(1849).double(m.organic_clicks_per_query);
                                if (m.organic_impressions != null && Object.hasOwnProperty.call(m, "organic_impressions"))
                                    w.uint32(1856).int64(m.organic_impressions);
                                if (m.organic_impressions_per_query != null && Object.hasOwnProperty.call(m, "organic_impressions_per_query"))
                                    w.uint32(1865).double(m.organic_impressions_per_query);
                                if (m.organic_queries != null && Object.hasOwnProperty.call(m, "organic_queries"))
                                    w.uint32(1872).int64(m.organic_queries);
                                if (m.percent_new_visitors != null && Object.hasOwnProperty.call(m, "percent_new_visitors"))
                                    w.uint32(1881).double(m.percent_new_visitors);
                                if (m.phone_calls != null && Object.hasOwnProperty.call(m, "phone_calls"))
                                    w.uint32(1888).int64(m.phone_calls);
                                if (m.phone_impressions != null && Object.hasOwnProperty.call(m, "phone_impressions"))
                                    w.uint32(1896).int64(m.phone_impressions);
                                if (m.phone_through_rate != null && Object.hasOwnProperty.call(m, "phone_through_rate"))
                                    w.uint32(1905).double(m.phone_through_rate);
                                if (m.relative_ctr != null && Object.hasOwnProperty.call(m, "relative_ctr"))
                                    w.uint32(1913).double(m.relative_ctr);
                                if (m.all_conversions_value_by_conversion_date != null && Object.hasOwnProperty.call(m, "all_conversions_value_by_conversion_date"))
                                    w.uint32(1921).double(m.all_conversions_value_by_conversion_date);
                                if (m.all_conversions_by_conversion_date != null && Object.hasOwnProperty.call(m, "all_conversions_by_conversion_date"))
                                    w.uint32(1929).double(m.all_conversions_by_conversion_date);
                                if (m.conversions_value_by_conversion_date != null && Object.hasOwnProperty.call(m, "conversions_value_by_conversion_date"))
                                    w.uint32(1937).double(m.conversions_value_by_conversion_date);
                                if (m.conversions_by_conversion_date != null && Object.hasOwnProperty.call(m, "conversions_by_conversion_date"))
                                    w.uint32(1945).double(m.conversions_by_conversion_date);
                                if (m.value_per_all_conversions_by_conversion_date != null && Object.hasOwnProperty.call(m, "value_per_all_conversions_by_conversion_date"))
                                    w.uint32(1953).double(m.value_per_all_conversions_by_conversion_date);
                                if (m.value_per_conversions_by_conversion_date != null && Object.hasOwnProperty.call(m, "value_per_conversions_by_conversion_date"))
                                    w.uint32(1961).double(m.value_per_conversions_by_conversion_date);
                                if (m.sk_ad_network_conversions != null && Object.hasOwnProperty.call(m, "sk_ad_network_conversions"))
                                    w.uint32(1968).int64(m.sk_ad_network_conversions);
                                if (m.optimization_score_uplift != null && Object.hasOwnProperty.call(m, "optimization_score_uplift"))
                                    w.uint32(1977).double(m.optimization_score_uplift);
                                if (m.optimization_score_url != null && Object.hasOwnProperty.call(m, "optimization_score_url"))
                                    w.uint32(1986).string(m.optimization_score_url);
                                if (m.biddable_app_install_conversions != null && Object.hasOwnProperty.call(m, "biddable_app_install_conversions"))
                                    w.uint32(2033).double(m.biddable_app_install_conversions);
                                if (m.biddable_app_post_install_conversions != null && Object.hasOwnProperty.call(m, "biddable_app_post_install_conversions"))
                                    w.uint32(2041).double(m.biddable_app_post_install_conversions);
                                if (m.hotel_commission_rate_micros != null && Object.hasOwnProperty.call(m, "hotel_commission_rate_micros"))
                                    w.uint32(2048).int64(m.hotel_commission_rate_micros);
                                if (m.hotel_expected_commission_cost != null && Object.hasOwnProperty.call(m, "hotel_expected_commission_cost"))
                                    w.uint32(2057).double(m.hotel_expected_commission_cost);
                                if (m.auction_insight_search_absolute_top_impression_percentage != null && Object.hasOwnProperty.call(m, "auction_insight_search_absolute_top_impression_percentage"))
                                    w.uint32(2065).double(m.auction_insight_search_absolute_top_impression_percentage);
                                if (m.auction_insight_search_impression_share != null && Object.hasOwnProperty.call(m, "auction_insight_search_impression_share"))
                                    w.uint32(2073).double(m.auction_insight_search_impression_share);
                                if (m.auction_insight_search_outranking_share != null && Object.hasOwnProperty.call(m, "auction_insight_search_outranking_share"))
                                    w.uint32(2081).double(m.auction_insight_search_outranking_share);
                                if (m.auction_insight_search_overlap_rate != null && Object.hasOwnProperty.call(m, "auction_insight_search_overlap_rate"))
                                    w.uint32(2089).double(m.auction_insight_search_overlap_rate);
                                if (m.auction_insight_search_position_above_rate != null && Object.hasOwnProperty.call(m, "auction_insight_search_position_above_rate"))
                                    w.uint32(2097).double(m.auction_insight_search_position_above_rate);
                                if (m.auction_insight_search_top_impression_percentage != null && Object.hasOwnProperty.call(m, "auction_insight_search_top_impression_percentage"))
                                    w.uint32(2105).double(m.auction_insight_search_top_impression_percentage);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Metrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Metrics.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {google.ads.googleads.v11.common.IMetrics} message Metrics message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Metrics.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Metrics message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.Metrics} Metrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Metrics.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Metrics();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 183:
                                        m.absolute_top_impression_percentage = r.double();
                                        break;
                                    case 184:
                                        m.active_view_cpm = r.double();
                                        break;
                                    case 185:
                                        m.active_view_ctr = r.double();
                                        break;
                                    case 186:
                                        m.active_view_impressions = r.int64();
                                        break;
                                    case 187:
                                        m.active_view_measurability = r.double();
                                        break;
                                    case 188:
                                        m.active_view_measurable_cost_micros = r.int64();
                                        break;
                                    case 189:
                                        m.active_view_measurable_impressions = r.int64();
                                        break;
                                    case 190:
                                        m.active_view_viewability = r.double();
                                        break;
                                    case 191:
                                        m.all_conversions_from_interactions_rate = r.double();
                                        break;
                                    case 192:
                                        m.all_conversions_value = r.double();
                                        break;
                                    case 240:
                                        m.all_conversions_value_by_conversion_date = r.double();
                                        break;
                                    case 193:
                                        m.all_conversions = r.double();
                                        break;
                                    case 241:
                                        m.all_conversions_by_conversion_date = r.double();
                                        break;
                                    case 194:
                                        m.all_conversions_value_per_cost = r.double();
                                        break;
                                    case 195:
                                        m.all_conversions_from_click_to_call = r.double();
                                        break;
                                    case 196:
                                        m.all_conversions_from_directions = r.double();
                                        break;
                                    case 197:
                                        m.all_conversions_from_interactions_value_per_interaction = r.double();
                                        break;
                                    case 198:
                                        m.all_conversions_from_menu = r.double();
                                        break;
                                    case 199:
                                        m.all_conversions_from_order = r.double();
                                        break;
                                    case 200:
                                        m.all_conversions_from_other_engagement = r.double();
                                        break;
                                    case 201:
                                        m.all_conversions_from_store_visit = r.double();
                                        break;
                                    case 202:
                                        m.all_conversions_from_store_website = r.double();
                                        break;
                                    case 258:
                                        m.auction_insight_search_absolute_top_impression_percentage = r.double();
                                        break;
                                    case 259:
                                        m.auction_insight_search_impression_share = r.double();
                                        break;
                                    case 260:
                                        m.auction_insight_search_outranking_share = r.double();
                                        break;
                                    case 261:
                                        m.auction_insight_search_overlap_rate = r.double();
                                        break;
                                    case 262:
                                        m.auction_insight_search_position_above_rate = r.double();
                                        break;
                                    case 263:
                                        m.auction_insight_search_top_impression_percentage = r.double();
                                        break;
                                    case 203:
                                        m.average_cost = r.double();
                                        break;
                                    case 204:
                                        m.average_cpc = r.double();
                                        break;
                                    case 205:
                                        m.average_cpe = r.double();
                                        break;
                                    case 206:
                                        m.average_cpm = r.double();
                                        break;
                                    case 207:
                                        m.average_cpv = r.double();
                                        break;
                                    case 208:
                                        m.average_page_views = r.double();
                                        break;
                                    case 209:
                                        m.average_time_on_site = r.double();
                                        break;
                                    case 210:
                                        m.benchmark_average_max_cpc = r.double();
                                        break;
                                    case 254:
                                        m.biddable_app_install_conversions = r.double();
                                        break;
                                    case 255:
                                        m.biddable_app_post_install_conversions = r.double();
                                        break;
                                    case 211:
                                        m.benchmark_ctr = r.double();
                                        break;
                                    case 212:
                                        m.bounce_rate = r.double();
                                        break;
                                    case 131:
                                        m.clicks = r.int64();
                                        break;
                                    case 156:
                                        m.combined_clicks = r.int64();
                                        break;
                                    case 157:
                                        m.combined_clicks_per_query = r.double();
                                        break;
                                    case 158:
                                        m.combined_queries = r.int64();
                                        break;
                                    case 159:
                                        m.content_budget_lost_impression_share = r.double();
                                        break;
                                    case 160:
                                        m.content_impression_share = r.double();
                                        break;
                                    case 161:
                                        m.conversion_last_received_request_date_time = r.string();
                                        break;
                                    case 162:
                                        m.conversion_last_conversion_date = r.string();
                                        break;
                                    case 163:
                                        m.content_rank_lost_impression_share = r.double();
                                        break;
                                    case 164:
                                        m.conversions_from_interactions_rate = r.double();
                                        break;
                                    case 165:
                                        m.conversions_value = r.double();
                                        break;
                                    case 242:
                                        m.conversions_value_by_conversion_date = r.double();
                                        break;
                                    case 166:
                                        m.conversions_value_per_cost = r.double();
                                        break;
                                    case 167:
                                        m.conversions_from_interactions_value_per_interaction = r.double();
                                        break;
                                    case 168:
                                        m.conversions = r.double();
                                        break;
                                    case 243:
                                        m.conversions_by_conversion_date = r.double();
                                        break;
                                    case 169:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 170:
                                        m.cost_per_all_conversions = r.double();
                                        break;
                                    case 171:
                                        m.cost_per_conversion = r.double();
                                        break;
                                    case 172:
                                        m.cost_per_current_model_attributed_conversion = r.double();
                                        break;
                                    case 173:
                                        m.cross_device_conversions = r.double();
                                        break;
                                    case 174:
                                        m.ctr = r.double();
                                        break;
                                    case 175:
                                        m.current_model_attributed_conversions = r.double();
                                        break;
                                    case 176:
                                        m.current_model_attributed_conversions_from_interactions_rate = r.double();
                                        break;
                                    case 177:
                                        m.current_model_attributed_conversions_from_interactions_value_per_interaction = r.double();
                                        break;
                                    case 178:
                                        m.current_model_attributed_conversions_value = r.double();
                                        break;
                                    case 179:
                                        m.current_model_attributed_conversions_value_per_cost = r.double();
                                        break;
                                    case 180:
                                        m.engagement_rate = r.double();
                                        break;
                                    case 181:
                                        m.engagements = r.int64();
                                        break;
                                    case 213:
                                        m.hotel_average_lead_value_micros = r.double();
                                        break;
                                    case 256:
                                        m.hotel_commission_rate_micros = r.int64();
                                        break;
                                    case 257:
                                        m.hotel_expected_commission_cost = r.double();
                                        break;
                                    case 214:
                                        m.hotel_price_difference_percentage = r.double();
                                        break;
                                    case 215:
                                        m.hotel_eligible_impressions = r.int64();
                                        break;
                                    case 80:
                                        m.historical_creative_quality_score = r.int32();
                                        break;
                                    case 81:
                                        m.historical_landing_page_quality_score = r.int32();
                                        break;
                                    case 216:
                                        m.historical_quality_score = r.int64();
                                        break;
                                    case 83:
                                        m.historical_search_predicted_ctr = r.int32();
                                        break;
                                    case 217:
                                        m.gmail_forwards = r.int64();
                                        break;
                                    case 218:
                                        m.gmail_saves = r.int64();
                                        break;
                                    case 219:
                                        m.gmail_secondary_clicks = r.int64();
                                        break;
                                    case 220:
                                        m.impressions_from_store_reach = r.int64();
                                        break;
                                    case 221:
                                        m.impressions = r.int64();
                                        break;
                                    case 222:
                                        m.interaction_rate = r.double();
                                        break;
                                    case 223:
                                        m.interactions = r.int64();
                                        break;
                                    case 100:
                                        if (!(m.interaction_event_types && m.interaction_event_types.length))
                                            m.interaction_event_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.interaction_event_types.push(r.int32());
                                        } else
                                            m.interaction_event_types.push(r.int32());
                                        break;
                                    case 224:
                                        m.invalid_click_rate = r.double();
                                        break;
                                    case 225:
                                        m.invalid_clicks = r.int64();
                                        break;
                                    case 226:
                                        m.message_chats = r.int64();
                                        break;
                                    case 227:
                                        m.message_impressions = r.int64();
                                        break;
                                    case 228:
                                        m.message_chat_rate = r.double();
                                        break;
                                    case 229:
                                        m.mobile_friendly_clicks_percentage = r.double();
                                        break;
                                    case 247:
                                        m.optimization_score_uplift = r.double();
                                        break;
                                    case 248:
                                        m.optimization_score_url = r.string();
                                        break;
                                    case 230:
                                        m.organic_clicks = r.int64();
                                        break;
                                    case 231:
                                        m.organic_clicks_per_query = r.double();
                                        break;
                                    case 232:
                                        m.organic_impressions = r.int64();
                                        break;
                                    case 233:
                                        m.organic_impressions_per_query = r.double();
                                        break;
                                    case 234:
                                        m.organic_queries = r.int64();
                                        break;
                                    case 235:
                                        m.percent_new_visitors = r.double();
                                        break;
                                    case 236:
                                        m.phone_calls = r.int64();
                                        break;
                                    case 237:
                                        m.phone_impressions = r.int64();
                                        break;
                                    case 238:
                                        m.phone_through_rate = r.double();
                                        break;
                                    case 239:
                                        m.relative_ctr = r.double();
                                        break;
                                    case 136:
                                        m.search_absolute_top_impression_share = r.double();
                                        break;
                                    case 137:
                                        m.search_budget_lost_absolute_top_impression_share = r.double();
                                        break;
                                    case 138:
                                        m.search_budget_lost_impression_share = r.double();
                                        break;
                                    case 139:
                                        m.search_budget_lost_top_impression_share = r.double();
                                        break;
                                    case 140:
                                        m.search_click_share = r.double();
                                        break;
                                    case 141:
                                        m.search_exact_match_impression_share = r.double();
                                        break;
                                    case 142:
                                        m.search_impression_share = r.double();
                                        break;
                                    case 143:
                                        m.search_rank_lost_absolute_top_impression_share = r.double();
                                        break;
                                    case 144:
                                        m.search_rank_lost_impression_share = r.double();
                                        break;
                                    case 145:
                                        m.search_rank_lost_top_impression_share = r.double();
                                        break;
                                    case 146:
                                        m.search_top_impression_share = r.double();
                                        break;
                                    case 147:
                                        m.speed_score = r.int64();
                                        break;
                                    case 148:
                                        m.top_impression_percentage = r.double();
                                        break;
                                    case 149:
                                        m.valid_accelerated_mobile_pages_clicks_percentage = r.double();
                                        break;
                                    case 150:
                                        m.value_per_all_conversions = r.double();
                                        break;
                                    case 244:
                                        m.value_per_all_conversions_by_conversion_date = r.double();
                                        break;
                                    case 151:
                                        m.value_per_conversion = r.double();
                                        break;
                                    case 245:
                                        m.value_per_conversions_by_conversion_date = r.double();
                                        break;
                                    case 152:
                                        m.value_per_current_model_attributed_conversion = r.double();
                                        break;
                                    case 132:
                                        m.video_quartile_p100_rate = r.double();
                                        break;
                                    case 133:
                                        m.video_quartile_p25_rate = r.double();
                                        break;
                                    case 134:
                                        m.video_quartile_p50_rate = r.double();
                                        break;
                                    case 135:
                                        m.video_quartile_p75_rate = r.double();
                                        break;
                                    case 153:
                                        m.video_view_rate = r.double();
                                        break;
                                    case 154:
                                        m.video_views = r.int64();
                                        break;
                                    case 155:
                                        m.view_through_conversions = r.int64();
                                        break;
                                    case 246:
                                        m.sk_ad_network_conversions = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Metrics message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.Metrics} Metrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Metrics.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Metrics message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Metrics.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.absolute_top_impression_percentage != null && m.hasOwnProperty("absolute_top_impression_percentage")) {
                                    p._absolute_top_impression_percentage = 1;
                                    if (typeof m.absolute_top_impression_percentage !== "number")
                                        return "absolute_top_impression_percentage: number expected";
                                }
                                if (m.active_view_cpm != null && m.hasOwnProperty("active_view_cpm")) {
                                    p._active_view_cpm = 1;
                                    if (typeof m.active_view_cpm !== "number")
                                        return "active_view_cpm: number expected";
                                }
                                if (m.active_view_ctr != null && m.hasOwnProperty("active_view_ctr")) {
                                    p._active_view_ctr = 1;
                                    if (typeof m.active_view_ctr !== "number")
                                        return "active_view_ctr: number expected";
                                }
                                if (m.active_view_impressions != null && m.hasOwnProperty("active_view_impressions")) {
                                    p._active_view_impressions = 1;
                                    if (!$util.isInteger(m.active_view_impressions) && !(m.active_view_impressions && $util.isInteger(m.active_view_impressions.low) && $util.isInteger(m.active_view_impressions.high)))
                                        return "active_view_impressions: integer|Long expected";
                                }
                                if (m.active_view_measurability != null && m.hasOwnProperty("active_view_measurability")) {
                                    p._active_view_measurability = 1;
                                    if (typeof m.active_view_measurability !== "number")
                                        return "active_view_measurability: number expected";
                                }
                                if (m.active_view_measurable_cost_micros != null && m.hasOwnProperty("active_view_measurable_cost_micros")) {
                                    p._active_view_measurable_cost_micros = 1;
                                    if (!$util.isInteger(m.active_view_measurable_cost_micros) && !(m.active_view_measurable_cost_micros && $util.isInteger(m.active_view_measurable_cost_micros.low) && $util.isInteger(m.active_view_measurable_cost_micros.high)))
                                        return "active_view_measurable_cost_micros: integer|Long expected";
                                }
                                if (m.active_view_measurable_impressions != null && m.hasOwnProperty("active_view_measurable_impressions")) {
                                    p._active_view_measurable_impressions = 1;
                                    if (!$util.isInteger(m.active_view_measurable_impressions) && !(m.active_view_measurable_impressions && $util.isInteger(m.active_view_measurable_impressions.low) && $util.isInteger(m.active_view_measurable_impressions.high)))
                                        return "active_view_measurable_impressions: integer|Long expected";
                                }
                                if (m.active_view_viewability != null && m.hasOwnProperty("active_view_viewability")) {
                                    p._active_view_viewability = 1;
                                    if (typeof m.active_view_viewability !== "number")
                                        return "active_view_viewability: number expected";
                                }
                                if (m.all_conversions_from_interactions_rate != null && m.hasOwnProperty("all_conversions_from_interactions_rate")) {
                                    p._all_conversions_from_interactions_rate = 1;
                                    if (typeof m.all_conversions_from_interactions_rate !== "number")
                                        return "all_conversions_from_interactions_rate: number expected";
                                }
                                if (m.all_conversions_value != null && m.hasOwnProperty("all_conversions_value")) {
                                    p._all_conversions_value = 1;
                                    if (typeof m.all_conversions_value !== "number")
                                        return "all_conversions_value: number expected";
                                }
                                if (m.all_conversions_value_by_conversion_date != null && m.hasOwnProperty("all_conversions_value_by_conversion_date")) {
                                    if (typeof m.all_conversions_value_by_conversion_date !== "number")
                                        return "all_conversions_value_by_conversion_date: number expected";
                                }
                                if (m.all_conversions != null && m.hasOwnProperty("all_conversions")) {
                                    p._all_conversions = 1;
                                    if (typeof m.all_conversions !== "number")
                                        return "all_conversions: number expected";
                                }
                                if (m.all_conversions_by_conversion_date != null && m.hasOwnProperty("all_conversions_by_conversion_date")) {
                                    if (typeof m.all_conversions_by_conversion_date !== "number")
                                        return "all_conversions_by_conversion_date: number expected";
                                }
                                if (m.all_conversions_value_per_cost != null && m.hasOwnProperty("all_conversions_value_per_cost")) {
                                    p._all_conversions_value_per_cost = 1;
                                    if (typeof m.all_conversions_value_per_cost !== "number")
                                        return "all_conversions_value_per_cost: number expected";
                                }
                                if (m.all_conversions_from_click_to_call != null && m.hasOwnProperty("all_conversions_from_click_to_call")) {
                                    p._all_conversions_from_click_to_call = 1;
                                    if (typeof m.all_conversions_from_click_to_call !== "number")
                                        return "all_conversions_from_click_to_call: number expected";
                                }
                                if (m.all_conversions_from_directions != null && m.hasOwnProperty("all_conversions_from_directions")) {
                                    p._all_conversions_from_directions = 1;
                                    if (typeof m.all_conversions_from_directions !== "number")
                                        return "all_conversions_from_directions: number expected";
                                }
                                if (m.all_conversions_from_interactions_value_per_interaction != null && m.hasOwnProperty("all_conversions_from_interactions_value_per_interaction")) {
                                    p._all_conversions_from_interactions_value_per_interaction = 1;
                                    if (typeof m.all_conversions_from_interactions_value_per_interaction !== "number")
                                        return "all_conversions_from_interactions_value_per_interaction: number expected";
                                }
                                if (m.all_conversions_from_menu != null && m.hasOwnProperty("all_conversions_from_menu")) {
                                    p._all_conversions_from_menu = 1;
                                    if (typeof m.all_conversions_from_menu !== "number")
                                        return "all_conversions_from_menu: number expected";
                                }
                                if (m.all_conversions_from_order != null && m.hasOwnProperty("all_conversions_from_order")) {
                                    p._all_conversions_from_order = 1;
                                    if (typeof m.all_conversions_from_order !== "number")
                                        return "all_conversions_from_order: number expected";
                                }
                                if (m.all_conversions_from_other_engagement != null && m.hasOwnProperty("all_conversions_from_other_engagement")) {
                                    p._all_conversions_from_other_engagement = 1;
                                    if (typeof m.all_conversions_from_other_engagement !== "number")
                                        return "all_conversions_from_other_engagement: number expected";
                                }
                                if (m.all_conversions_from_store_visit != null && m.hasOwnProperty("all_conversions_from_store_visit")) {
                                    p._all_conversions_from_store_visit = 1;
                                    if (typeof m.all_conversions_from_store_visit !== "number")
                                        return "all_conversions_from_store_visit: number expected";
                                }
                                if (m.all_conversions_from_store_website != null && m.hasOwnProperty("all_conversions_from_store_website")) {
                                    p._all_conversions_from_store_website = 1;
                                    if (typeof m.all_conversions_from_store_website !== "number")
                                        return "all_conversions_from_store_website: number expected";
                                }
                                if (m.auction_insight_search_absolute_top_impression_percentage != null && m.hasOwnProperty("auction_insight_search_absolute_top_impression_percentage")) {
                                    p._auction_insight_search_absolute_top_impression_percentage = 1;
                                    if (typeof m.auction_insight_search_absolute_top_impression_percentage !== "number")
                                        return "auction_insight_search_absolute_top_impression_percentage: number expected";
                                }
                                if (m.auction_insight_search_impression_share != null && m.hasOwnProperty("auction_insight_search_impression_share")) {
                                    p._auction_insight_search_impression_share = 1;
                                    if (typeof m.auction_insight_search_impression_share !== "number")
                                        return "auction_insight_search_impression_share: number expected";
                                }
                                if (m.auction_insight_search_outranking_share != null && m.hasOwnProperty("auction_insight_search_outranking_share")) {
                                    p._auction_insight_search_outranking_share = 1;
                                    if (typeof m.auction_insight_search_outranking_share !== "number")
                                        return "auction_insight_search_outranking_share: number expected";
                                }
                                if (m.auction_insight_search_overlap_rate != null && m.hasOwnProperty("auction_insight_search_overlap_rate")) {
                                    p._auction_insight_search_overlap_rate = 1;
                                    if (typeof m.auction_insight_search_overlap_rate !== "number")
                                        return "auction_insight_search_overlap_rate: number expected";
                                }
                                if (m.auction_insight_search_position_above_rate != null && m.hasOwnProperty("auction_insight_search_position_above_rate")) {
                                    p._auction_insight_search_position_above_rate = 1;
                                    if (typeof m.auction_insight_search_position_above_rate !== "number")
                                        return "auction_insight_search_position_above_rate: number expected";
                                }
                                if (m.auction_insight_search_top_impression_percentage != null && m.hasOwnProperty("auction_insight_search_top_impression_percentage")) {
                                    p._auction_insight_search_top_impression_percentage = 1;
                                    if (typeof m.auction_insight_search_top_impression_percentage !== "number")
                                        return "auction_insight_search_top_impression_percentage: number expected";
                                }
                                if (m.average_cost != null && m.hasOwnProperty("average_cost")) {
                                    p._average_cost = 1;
                                    if (typeof m.average_cost !== "number")
                                        return "average_cost: number expected";
                                }
                                if (m.average_cpc != null && m.hasOwnProperty("average_cpc")) {
                                    p._average_cpc = 1;
                                    if (typeof m.average_cpc !== "number")
                                        return "average_cpc: number expected";
                                }
                                if (m.average_cpe != null && m.hasOwnProperty("average_cpe")) {
                                    p._average_cpe = 1;
                                    if (typeof m.average_cpe !== "number")
                                        return "average_cpe: number expected";
                                }
                                if (m.average_cpm != null && m.hasOwnProperty("average_cpm")) {
                                    p._average_cpm = 1;
                                    if (typeof m.average_cpm !== "number")
                                        return "average_cpm: number expected";
                                }
                                if (m.average_cpv != null && m.hasOwnProperty("average_cpv")) {
                                    p._average_cpv = 1;
                                    if (typeof m.average_cpv !== "number")
                                        return "average_cpv: number expected";
                                }
                                if (m.average_page_views != null && m.hasOwnProperty("average_page_views")) {
                                    p._average_page_views = 1;
                                    if (typeof m.average_page_views !== "number")
                                        return "average_page_views: number expected";
                                }
                                if (m.average_time_on_site != null && m.hasOwnProperty("average_time_on_site")) {
                                    p._average_time_on_site = 1;
                                    if (typeof m.average_time_on_site !== "number")
                                        return "average_time_on_site: number expected";
                                }
                                if (m.benchmark_average_max_cpc != null && m.hasOwnProperty("benchmark_average_max_cpc")) {
                                    p._benchmark_average_max_cpc = 1;
                                    if (typeof m.benchmark_average_max_cpc !== "number")
                                        return "benchmark_average_max_cpc: number expected";
                                }
                                if (m.biddable_app_install_conversions != null && m.hasOwnProperty("biddable_app_install_conversions")) {
                                    p._biddable_app_install_conversions = 1;
                                    if (typeof m.biddable_app_install_conversions !== "number")
                                        return "biddable_app_install_conversions: number expected";
                                }
                                if (m.biddable_app_post_install_conversions != null && m.hasOwnProperty("biddable_app_post_install_conversions")) {
                                    p._biddable_app_post_install_conversions = 1;
                                    if (typeof m.biddable_app_post_install_conversions !== "number")
                                        return "biddable_app_post_install_conversions: number expected";
                                }
                                if (m.benchmark_ctr != null && m.hasOwnProperty("benchmark_ctr")) {
                                    p._benchmark_ctr = 1;
                                    if (typeof m.benchmark_ctr !== "number")
                                        return "benchmark_ctr: number expected";
                                }
                                if (m.bounce_rate != null && m.hasOwnProperty("bounce_rate")) {
                                    p._bounce_rate = 1;
                                    if (typeof m.bounce_rate !== "number")
                                        return "bounce_rate: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    p._clicks = 1;
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.combined_clicks != null && m.hasOwnProperty("combined_clicks")) {
                                    p._combined_clicks = 1;
                                    if (!$util.isInteger(m.combined_clicks) && !(m.combined_clicks && $util.isInteger(m.combined_clicks.low) && $util.isInteger(m.combined_clicks.high)))
                                        return "combined_clicks: integer|Long expected";
                                }
                                if (m.combined_clicks_per_query != null && m.hasOwnProperty("combined_clicks_per_query")) {
                                    p._combined_clicks_per_query = 1;
                                    if (typeof m.combined_clicks_per_query !== "number")
                                        return "combined_clicks_per_query: number expected";
                                }
                                if (m.combined_queries != null && m.hasOwnProperty("combined_queries")) {
                                    p._combined_queries = 1;
                                    if (!$util.isInteger(m.combined_queries) && !(m.combined_queries && $util.isInteger(m.combined_queries.low) && $util.isInteger(m.combined_queries.high)))
                                        return "combined_queries: integer|Long expected";
                                }
                                if (m.content_budget_lost_impression_share != null && m.hasOwnProperty("content_budget_lost_impression_share")) {
                                    p._content_budget_lost_impression_share = 1;
                                    if (typeof m.content_budget_lost_impression_share !== "number")
                                        return "content_budget_lost_impression_share: number expected";
                                }
                                if (m.content_impression_share != null && m.hasOwnProperty("content_impression_share")) {
                                    p._content_impression_share = 1;
                                    if (typeof m.content_impression_share !== "number")
                                        return "content_impression_share: number expected";
                                }
                                if (m.conversion_last_received_request_date_time != null && m.hasOwnProperty("conversion_last_received_request_date_time")) {
                                    p._conversion_last_received_request_date_time = 1;
                                    if (!$util.isString(m.conversion_last_received_request_date_time))
                                        return "conversion_last_received_request_date_time: string expected";
                                }
                                if (m.conversion_last_conversion_date != null && m.hasOwnProperty("conversion_last_conversion_date")) {
                                    p._conversion_last_conversion_date = 1;
                                    if (!$util.isString(m.conversion_last_conversion_date))
                                        return "conversion_last_conversion_date: string expected";
                                }
                                if (m.content_rank_lost_impression_share != null && m.hasOwnProperty("content_rank_lost_impression_share")) {
                                    p._content_rank_lost_impression_share = 1;
                                    if (typeof m.content_rank_lost_impression_share !== "number")
                                        return "content_rank_lost_impression_share: number expected";
                                }
                                if (m.conversions_from_interactions_rate != null && m.hasOwnProperty("conversions_from_interactions_rate")) {
                                    p._conversions_from_interactions_rate = 1;
                                    if (typeof m.conversions_from_interactions_rate !== "number")
                                        return "conversions_from_interactions_rate: number expected";
                                }
                                if (m.conversions_value != null && m.hasOwnProperty("conversions_value")) {
                                    p._conversions_value = 1;
                                    if (typeof m.conversions_value !== "number")
                                        return "conversions_value: number expected";
                                }
                                if (m.conversions_value_by_conversion_date != null && m.hasOwnProperty("conversions_value_by_conversion_date")) {
                                    if (typeof m.conversions_value_by_conversion_date !== "number")
                                        return "conversions_value_by_conversion_date: number expected";
                                }
                                if (m.conversions_value_per_cost != null && m.hasOwnProperty("conversions_value_per_cost")) {
                                    p._conversions_value_per_cost = 1;
                                    if (typeof m.conversions_value_per_cost !== "number")
                                        return "conversions_value_per_cost: number expected";
                                }
                                if (m.conversions_from_interactions_value_per_interaction != null && m.hasOwnProperty("conversions_from_interactions_value_per_interaction")) {
                                    p._conversions_from_interactions_value_per_interaction = 1;
                                    if (typeof m.conversions_from_interactions_value_per_interaction !== "number")
                                        return "conversions_from_interactions_value_per_interaction: number expected";
                                }
                                if (m.conversions != null && m.hasOwnProperty("conversions")) {
                                    p._conversions = 1;
                                    if (typeof m.conversions !== "number")
                                        return "conversions: number expected";
                                }
                                if (m.conversions_by_conversion_date != null && m.hasOwnProperty("conversions_by_conversion_date")) {
                                    if (typeof m.conversions_by_conversion_date !== "number")
                                        return "conversions_by_conversion_date: number expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.cost_per_all_conversions != null && m.hasOwnProperty("cost_per_all_conversions")) {
                                    p._cost_per_all_conversions = 1;
                                    if (typeof m.cost_per_all_conversions !== "number")
                                        return "cost_per_all_conversions: number expected";
                                }
                                if (m.cost_per_conversion != null && m.hasOwnProperty("cost_per_conversion")) {
                                    p._cost_per_conversion = 1;
                                    if (typeof m.cost_per_conversion !== "number")
                                        return "cost_per_conversion: number expected";
                                }
                                if (m.cost_per_current_model_attributed_conversion != null && m.hasOwnProperty("cost_per_current_model_attributed_conversion")) {
                                    p._cost_per_current_model_attributed_conversion = 1;
                                    if (typeof m.cost_per_current_model_attributed_conversion !== "number")
                                        return "cost_per_current_model_attributed_conversion: number expected";
                                }
                                if (m.cross_device_conversions != null && m.hasOwnProperty("cross_device_conversions")) {
                                    p._cross_device_conversions = 1;
                                    if (typeof m.cross_device_conversions !== "number")
                                        return "cross_device_conversions: number expected";
                                }
                                if (m.ctr != null && m.hasOwnProperty("ctr")) {
                                    p._ctr = 1;
                                    if (typeof m.ctr !== "number")
                                        return "ctr: number expected";
                                }
                                if (m.current_model_attributed_conversions != null && m.hasOwnProperty("current_model_attributed_conversions")) {
                                    p._current_model_attributed_conversions = 1;
                                    if (typeof m.current_model_attributed_conversions !== "number")
                                        return "current_model_attributed_conversions: number expected";
                                }
                                if (m.current_model_attributed_conversions_from_interactions_rate != null && m.hasOwnProperty("current_model_attributed_conversions_from_interactions_rate")) {
                                    p._current_model_attributed_conversions_from_interactions_rate = 1;
                                    if (typeof m.current_model_attributed_conversions_from_interactions_rate !== "number")
                                        return "current_model_attributed_conversions_from_interactions_rate: number expected";
                                }
                                if (m.current_model_attributed_conversions_from_interactions_value_per_interaction != null && m.hasOwnProperty("current_model_attributed_conversions_from_interactions_value_per_interaction")) {
                                    p._current_model_attributed_conversions_from_interactions_value_per_interaction = 1;
                                    if (typeof m.current_model_attributed_conversions_from_interactions_value_per_interaction !== "number")
                                        return "current_model_attributed_conversions_from_interactions_value_per_interaction: number expected";
                                }
                                if (m.current_model_attributed_conversions_value != null && m.hasOwnProperty("current_model_attributed_conversions_value")) {
                                    p._current_model_attributed_conversions_value = 1;
                                    if (typeof m.current_model_attributed_conversions_value !== "number")
                                        return "current_model_attributed_conversions_value: number expected";
                                }
                                if (m.current_model_attributed_conversions_value_per_cost != null && m.hasOwnProperty("current_model_attributed_conversions_value_per_cost")) {
                                    p._current_model_attributed_conversions_value_per_cost = 1;
                                    if (typeof m.current_model_attributed_conversions_value_per_cost !== "number")
                                        return "current_model_attributed_conversions_value_per_cost: number expected";
                                }
                                if (m.engagement_rate != null && m.hasOwnProperty("engagement_rate")) {
                                    p._engagement_rate = 1;
                                    if (typeof m.engagement_rate !== "number")
                                        return "engagement_rate: number expected";
                                }
                                if (m.engagements != null && m.hasOwnProperty("engagements")) {
                                    p._engagements = 1;
                                    if (!$util.isInteger(m.engagements) && !(m.engagements && $util.isInteger(m.engagements.low) && $util.isInteger(m.engagements.high)))
                                        return "engagements: integer|Long expected";
                                }
                                if (m.hotel_average_lead_value_micros != null && m.hasOwnProperty("hotel_average_lead_value_micros")) {
                                    p._hotel_average_lead_value_micros = 1;
                                    if (typeof m.hotel_average_lead_value_micros !== "number")
                                        return "hotel_average_lead_value_micros: number expected";
                                }
                                if (m.hotel_commission_rate_micros != null && m.hasOwnProperty("hotel_commission_rate_micros")) {
                                    p._hotel_commission_rate_micros = 1;
                                    if (!$util.isInteger(m.hotel_commission_rate_micros) && !(m.hotel_commission_rate_micros && $util.isInteger(m.hotel_commission_rate_micros.low) && $util.isInteger(m.hotel_commission_rate_micros.high)))
                                        return "hotel_commission_rate_micros: integer|Long expected";
                                }
                                if (m.hotel_expected_commission_cost != null && m.hasOwnProperty("hotel_expected_commission_cost")) {
                                    p._hotel_expected_commission_cost = 1;
                                    if (typeof m.hotel_expected_commission_cost !== "number")
                                        return "hotel_expected_commission_cost: number expected";
                                }
                                if (m.hotel_price_difference_percentage != null && m.hasOwnProperty("hotel_price_difference_percentage")) {
                                    p._hotel_price_difference_percentage = 1;
                                    if (typeof m.hotel_price_difference_percentage !== "number")
                                        return "hotel_price_difference_percentage: number expected";
                                }
                                if (m.hotel_eligible_impressions != null && m.hasOwnProperty("hotel_eligible_impressions")) {
                                    p._hotel_eligible_impressions = 1;
                                    if (!$util.isInteger(m.hotel_eligible_impressions) && !(m.hotel_eligible_impressions && $util.isInteger(m.hotel_eligible_impressions.low) && $util.isInteger(m.hotel_eligible_impressions.high)))
                                        return "hotel_eligible_impressions: integer|Long expected";
                                }
                                if (m.historical_creative_quality_score != null && m.hasOwnProperty("historical_creative_quality_score")) {
                                    switch (m.historical_creative_quality_score) {
                                    default:
                                        return "historical_creative_quality_score: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.historical_landing_page_quality_score != null && m.hasOwnProperty("historical_landing_page_quality_score")) {
                                    switch (m.historical_landing_page_quality_score) {
                                    default:
                                        return "historical_landing_page_quality_score: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.historical_quality_score != null && m.hasOwnProperty("historical_quality_score")) {
                                    p._historical_quality_score = 1;
                                    if (!$util.isInteger(m.historical_quality_score) && !(m.historical_quality_score && $util.isInteger(m.historical_quality_score.low) && $util.isInteger(m.historical_quality_score.high)))
                                        return "historical_quality_score: integer|Long expected";
                                }
                                if (m.historical_search_predicted_ctr != null && m.hasOwnProperty("historical_search_predicted_ctr")) {
                                    switch (m.historical_search_predicted_ctr) {
                                    default:
                                        return "historical_search_predicted_ctr: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.gmail_forwards != null && m.hasOwnProperty("gmail_forwards")) {
                                    p._gmail_forwards = 1;
                                    if (!$util.isInteger(m.gmail_forwards) && !(m.gmail_forwards && $util.isInteger(m.gmail_forwards.low) && $util.isInteger(m.gmail_forwards.high)))
                                        return "gmail_forwards: integer|Long expected";
                                }
                                if (m.gmail_saves != null && m.hasOwnProperty("gmail_saves")) {
                                    p._gmail_saves = 1;
                                    if (!$util.isInteger(m.gmail_saves) && !(m.gmail_saves && $util.isInteger(m.gmail_saves.low) && $util.isInteger(m.gmail_saves.high)))
                                        return "gmail_saves: integer|Long expected";
                                }
                                if (m.gmail_secondary_clicks != null && m.hasOwnProperty("gmail_secondary_clicks")) {
                                    p._gmail_secondary_clicks = 1;
                                    if (!$util.isInteger(m.gmail_secondary_clicks) && !(m.gmail_secondary_clicks && $util.isInteger(m.gmail_secondary_clicks.low) && $util.isInteger(m.gmail_secondary_clicks.high)))
                                        return "gmail_secondary_clicks: integer|Long expected";
                                }
                                if (m.impressions_from_store_reach != null && m.hasOwnProperty("impressions_from_store_reach")) {
                                    p._impressions_from_store_reach = 1;
                                    if (!$util.isInteger(m.impressions_from_store_reach) && !(m.impressions_from_store_reach && $util.isInteger(m.impressions_from_store_reach.low) && $util.isInteger(m.impressions_from_store_reach.high)))
                                        return "impressions_from_store_reach: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.interaction_rate != null && m.hasOwnProperty("interaction_rate")) {
                                    p._interaction_rate = 1;
                                    if (typeof m.interaction_rate !== "number")
                                        return "interaction_rate: number expected";
                                }
                                if (m.interactions != null && m.hasOwnProperty("interactions")) {
                                    p._interactions = 1;
                                    if (!$util.isInteger(m.interactions) && !(m.interactions && $util.isInteger(m.interactions.low) && $util.isInteger(m.interactions.high)))
                                        return "interactions: integer|Long expected";
                                }
                                if (m.interaction_event_types != null && m.hasOwnProperty("interaction_event_types")) {
                                    if (!Array.isArray(m.interaction_event_types))
                                        return "interaction_event_types: array expected";
                                    for (var i = 0; i < m.interaction_event_types.length; ++i) {
                                        switch (m.interaction_event_types[i]) {
                                        default:
                                            return "interaction_event_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                            break;
                                        }
                                    }
                                }
                                if (m.invalid_click_rate != null && m.hasOwnProperty("invalid_click_rate")) {
                                    p._invalid_click_rate = 1;
                                    if (typeof m.invalid_click_rate !== "number")
                                        return "invalid_click_rate: number expected";
                                }
                                if (m.invalid_clicks != null && m.hasOwnProperty("invalid_clicks")) {
                                    p._invalid_clicks = 1;
                                    if (!$util.isInteger(m.invalid_clicks) && !(m.invalid_clicks && $util.isInteger(m.invalid_clicks.low) && $util.isInteger(m.invalid_clicks.high)))
                                        return "invalid_clicks: integer|Long expected";
                                }
                                if (m.message_chats != null && m.hasOwnProperty("message_chats")) {
                                    p._message_chats = 1;
                                    if (!$util.isInteger(m.message_chats) && !(m.message_chats && $util.isInteger(m.message_chats.low) && $util.isInteger(m.message_chats.high)))
                                        return "message_chats: integer|Long expected";
                                }
                                if (m.message_impressions != null && m.hasOwnProperty("message_impressions")) {
                                    p._message_impressions = 1;
                                    if (!$util.isInteger(m.message_impressions) && !(m.message_impressions && $util.isInteger(m.message_impressions.low) && $util.isInteger(m.message_impressions.high)))
                                        return "message_impressions: integer|Long expected";
                                }
                                if (m.message_chat_rate != null && m.hasOwnProperty("message_chat_rate")) {
                                    p._message_chat_rate = 1;
                                    if (typeof m.message_chat_rate !== "number")
                                        return "message_chat_rate: number expected";
                                }
                                if (m.mobile_friendly_clicks_percentage != null && m.hasOwnProperty("mobile_friendly_clicks_percentage")) {
                                    p._mobile_friendly_clicks_percentage = 1;
                                    if (typeof m.mobile_friendly_clicks_percentage !== "number")
                                        return "mobile_friendly_clicks_percentage: number expected";
                                }
                                if (m.optimization_score_uplift != null && m.hasOwnProperty("optimization_score_uplift")) {
                                    p._optimization_score_uplift = 1;
                                    if (typeof m.optimization_score_uplift !== "number")
                                        return "optimization_score_uplift: number expected";
                                }
                                if (m.optimization_score_url != null && m.hasOwnProperty("optimization_score_url")) {
                                    p._optimization_score_url = 1;
                                    if (!$util.isString(m.optimization_score_url))
                                        return "optimization_score_url: string expected";
                                }
                                if (m.organic_clicks != null && m.hasOwnProperty("organic_clicks")) {
                                    p._organic_clicks = 1;
                                    if (!$util.isInteger(m.organic_clicks) && !(m.organic_clicks && $util.isInteger(m.organic_clicks.low) && $util.isInteger(m.organic_clicks.high)))
                                        return "organic_clicks: integer|Long expected";
                                }
                                if (m.organic_clicks_per_query != null && m.hasOwnProperty("organic_clicks_per_query")) {
                                    p._organic_clicks_per_query = 1;
                                    if (typeof m.organic_clicks_per_query !== "number")
                                        return "organic_clicks_per_query: number expected";
                                }
                                if (m.organic_impressions != null && m.hasOwnProperty("organic_impressions")) {
                                    p._organic_impressions = 1;
                                    if (!$util.isInteger(m.organic_impressions) && !(m.organic_impressions && $util.isInteger(m.organic_impressions.low) && $util.isInteger(m.organic_impressions.high)))
                                        return "organic_impressions: integer|Long expected";
                                }
                                if (m.organic_impressions_per_query != null && m.hasOwnProperty("organic_impressions_per_query")) {
                                    p._organic_impressions_per_query = 1;
                                    if (typeof m.organic_impressions_per_query !== "number")
                                        return "organic_impressions_per_query: number expected";
                                }
                                if (m.organic_queries != null && m.hasOwnProperty("organic_queries")) {
                                    p._organic_queries = 1;
                                    if (!$util.isInteger(m.organic_queries) && !(m.organic_queries && $util.isInteger(m.organic_queries.low) && $util.isInteger(m.organic_queries.high)))
                                        return "organic_queries: integer|Long expected";
                                }
                                if (m.percent_new_visitors != null && m.hasOwnProperty("percent_new_visitors")) {
                                    p._percent_new_visitors = 1;
                                    if (typeof m.percent_new_visitors !== "number")
                                        return "percent_new_visitors: number expected";
                                }
                                if (m.phone_calls != null && m.hasOwnProperty("phone_calls")) {
                                    p._phone_calls = 1;
                                    if (!$util.isInteger(m.phone_calls) && !(m.phone_calls && $util.isInteger(m.phone_calls.low) && $util.isInteger(m.phone_calls.high)))
                                        return "phone_calls: integer|Long expected";
                                }
                                if (m.phone_impressions != null && m.hasOwnProperty("phone_impressions")) {
                                    p._phone_impressions = 1;
                                    if (!$util.isInteger(m.phone_impressions) && !(m.phone_impressions && $util.isInteger(m.phone_impressions.low) && $util.isInteger(m.phone_impressions.high)))
                                        return "phone_impressions: integer|Long expected";
                                }
                                if (m.phone_through_rate != null && m.hasOwnProperty("phone_through_rate")) {
                                    p._phone_through_rate = 1;
                                    if (typeof m.phone_through_rate !== "number")
                                        return "phone_through_rate: number expected";
                                }
                                if (m.relative_ctr != null && m.hasOwnProperty("relative_ctr")) {
                                    p._relative_ctr = 1;
                                    if (typeof m.relative_ctr !== "number")
                                        return "relative_ctr: number expected";
                                }
                                if (m.search_absolute_top_impression_share != null && m.hasOwnProperty("search_absolute_top_impression_share")) {
                                    p._search_absolute_top_impression_share = 1;
                                    if (typeof m.search_absolute_top_impression_share !== "number")
                                        return "search_absolute_top_impression_share: number expected";
                                }
                                if (m.search_budget_lost_absolute_top_impression_share != null && m.hasOwnProperty("search_budget_lost_absolute_top_impression_share")) {
                                    p._search_budget_lost_absolute_top_impression_share = 1;
                                    if (typeof m.search_budget_lost_absolute_top_impression_share !== "number")
                                        return "search_budget_lost_absolute_top_impression_share: number expected";
                                }
                                if (m.search_budget_lost_impression_share != null && m.hasOwnProperty("search_budget_lost_impression_share")) {
                                    p._search_budget_lost_impression_share = 1;
                                    if (typeof m.search_budget_lost_impression_share !== "number")
                                        return "search_budget_lost_impression_share: number expected";
                                }
                                if (m.search_budget_lost_top_impression_share != null && m.hasOwnProperty("search_budget_lost_top_impression_share")) {
                                    p._search_budget_lost_top_impression_share = 1;
                                    if (typeof m.search_budget_lost_top_impression_share !== "number")
                                        return "search_budget_lost_top_impression_share: number expected";
                                }
                                if (m.search_click_share != null && m.hasOwnProperty("search_click_share")) {
                                    p._search_click_share = 1;
                                    if (typeof m.search_click_share !== "number")
                                        return "search_click_share: number expected";
                                }
                                if (m.search_exact_match_impression_share != null && m.hasOwnProperty("search_exact_match_impression_share")) {
                                    p._search_exact_match_impression_share = 1;
                                    if (typeof m.search_exact_match_impression_share !== "number")
                                        return "search_exact_match_impression_share: number expected";
                                }
                                if (m.search_impression_share != null && m.hasOwnProperty("search_impression_share")) {
                                    p._search_impression_share = 1;
                                    if (typeof m.search_impression_share !== "number")
                                        return "search_impression_share: number expected";
                                }
                                if (m.search_rank_lost_absolute_top_impression_share != null && m.hasOwnProperty("search_rank_lost_absolute_top_impression_share")) {
                                    p._search_rank_lost_absolute_top_impression_share = 1;
                                    if (typeof m.search_rank_lost_absolute_top_impression_share !== "number")
                                        return "search_rank_lost_absolute_top_impression_share: number expected";
                                }
                                if (m.search_rank_lost_impression_share != null && m.hasOwnProperty("search_rank_lost_impression_share")) {
                                    p._search_rank_lost_impression_share = 1;
                                    if (typeof m.search_rank_lost_impression_share !== "number")
                                        return "search_rank_lost_impression_share: number expected";
                                }
                                if (m.search_rank_lost_top_impression_share != null && m.hasOwnProperty("search_rank_lost_top_impression_share")) {
                                    p._search_rank_lost_top_impression_share = 1;
                                    if (typeof m.search_rank_lost_top_impression_share !== "number")
                                        return "search_rank_lost_top_impression_share: number expected";
                                }
                                if (m.search_top_impression_share != null && m.hasOwnProperty("search_top_impression_share")) {
                                    p._search_top_impression_share = 1;
                                    if (typeof m.search_top_impression_share !== "number")
                                        return "search_top_impression_share: number expected";
                                }
                                if (m.speed_score != null && m.hasOwnProperty("speed_score")) {
                                    p._speed_score = 1;
                                    if (!$util.isInteger(m.speed_score) && !(m.speed_score && $util.isInteger(m.speed_score.low) && $util.isInteger(m.speed_score.high)))
                                        return "speed_score: integer|Long expected";
                                }
                                if (m.top_impression_percentage != null && m.hasOwnProperty("top_impression_percentage")) {
                                    p._top_impression_percentage = 1;
                                    if (typeof m.top_impression_percentage !== "number")
                                        return "top_impression_percentage: number expected";
                                }
                                if (m.valid_accelerated_mobile_pages_clicks_percentage != null && m.hasOwnProperty("valid_accelerated_mobile_pages_clicks_percentage")) {
                                    p._valid_accelerated_mobile_pages_clicks_percentage = 1;
                                    if (typeof m.valid_accelerated_mobile_pages_clicks_percentage !== "number")
                                        return "valid_accelerated_mobile_pages_clicks_percentage: number expected";
                                }
                                if (m.value_per_all_conversions != null && m.hasOwnProperty("value_per_all_conversions")) {
                                    p._value_per_all_conversions = 1;
                                    if (typeof m.value_per_all_conversions !== "number")
                                        return "value_per_all_conversions: number expected";
                                }
                                if (m.value_per_all_conversions_by_conversion_date != null && m.hasOwnProperty("value_per_all_conversions_by_conversion_date")) {
                                    p._value_per_all_conversions_by_conversion_date = 1;
                                    if (typeof m.value_per_all_conversions_by_conversion_date !== "number")
                                        return "value_per_all_conversions_by_conversion_date: number expected";
                                }
                                if (m.value_per_conversion != null && m.hasOwnProperty("value_per_conversion")) {
                                    p._value_per_conversion = 1;
                                    if (typeof m.value_per_conversion !== "number")
                                        return "value_per_conversion: number expected";
                                }
                                if (m.value_per_conversions_by_conversion_date != null && m.hasOwnProperty("value_per_conversions_by_conversion_date")) {
                                    p._value_per_conversions_by_conversion_date = 1;
                                    if (typeof m.value_per_conversions_by_conversion_date !== "number")
                                        return "value_per_conversions_by_conversion_date: number expected";
                                }
                                if (m.value_per_current_model_attributed_conversion != null && m.hasOwnProperty("value_per_current_model_attributed_conversion")) {
                                    p._value_per_current_model_attributed_conversion = 1;
                                    if (typeof m.value_per_current_model_attributed_conversion !== "number")
                                        return "value_per_current_model_attributed_conversion: number expected";
                                }
                                if (m.video_quartile_p100_rate != null && m.hasOwnProperty("video_quartile_p100_rate")) {
                                    p._video_quartile_p100_rate = 1;
                                    if (typeof m.video_quartile_p100_rate !== "number")
                                        return "video_quartile_p100_rate: number expected";
                                }
                                if (m.video_quartile_p25_rate != null && m.hasOwnProperty("video_quartile_p25_rate")) {
                                    p._video_quartile_p25_rate = 1;
                                    if (typeof m.video_quartile_p25_rate !== "number")
                                        return "video_quartile_p25_rate: number expected";
                                }
                                if (m.video_quartile_p50_rate != null && m.hasOwnProperty("video_quartile_p50_rate")) {
                                    p._video_quartile_p50_rate = 1;
                                    if (typeof m.video_quartile_p50_rate !== "number")
                                        return "video_quartile_p50_rate: number expected";
                                }
                                if (m.video_quartile_p75_rate != null && m.hasOwnProperty("video_quartile_p75_rate")) {
                                    p._video_quartile_p75_rate = 1;
                                    if (typeof m.video_quartile_p75_rate !== "number")
                                        return "video_quartile_p75_rate: number expected";
                                }
                                if (m.video_view_rate != null && m.hasOwnProperty("video_view_rate")) {
                                    p._video_view_rate = 1;
                                    if (typeof m.video_view_rate !== "number")
                                        return "video_view_rate: number expected";
                                }
                                if (m.video_views != null && m.hasOwnProperty("video_views")) {
                                    p._video_views = 1;
                                    if (!$util.isInteger(m.video_views) && !(m.video_views && $util.isInteger(m.video_views.low) && $util.isInteger(m.video_views.high)))
                                        return "video_views: integer|Long expected";
                                }
                                if (m.view_through_conversions != null && m.hasOwnProperty("view_through_conversions")) {
                                    p._view_through_conversions = 1;
                                    if (!$util.isInteger(m.view_through_conversions) && !(m.view_through_conversions && $util.isInteger(m.view_through_conversions.low) && $util.isInteger(m.view_through_conversions.high)))
                                        return "view_through_conversions: integer|Long expected";
                                }
                                if (m.sk_ad_network_conversions != null && m.hasOwnProperty("sk_ad_network_conversions")) {
                                    if (!$util.isInteger(m.sk_ad_network_conversions) && !(m.sk_ad_network_conversions && $util.isInteger(m.sk_ad_network_conversions.low) && $util.isInteger(m.sk_ad_network_conversions.high)))
                                        return "sk_ad_network_conversions: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Metrics message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.Metrics} Metrics
                             */
                            Metrics.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.Metrics)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.Metrics();
                                if (d.absolute_top_impression_percentage != null) {
                                    m.absolute_top_impression_percentage = Number(d.absolute_top_impression_percentage);
                                }
                                if (d.active_view_cpm != null) {
                                    m.active_view_cpm = Number(d.active_view_cpm);
                                }
                                if (d.active_view_ctr != null) {
                                    m.active_view_ctr = Number(d.active_view_ctr);
                                }
                                if (d.active_view_impressions != null) {
                                    if ($util.Long)
                                        (m.active_view_impressions = $util.Long.fromValue(d.active_view_impressions)).unsigned = false;
                                    else if (typeof d.active_view_impressions === "string")
                                        m.active_view_impressions = parseInt(d.active_view_impressions, 10);
                                    else if (typeof d.active_view_impressions === "number")
                                        m.active_view_impressions = d.active_view_impressions;
                                    else if (typeof d.active_view_impressions === "object")
                                        m.active_view_impressions = new $util.LongBits(d.active_view_impressions.low >>> 0, d.active_view_impressions.high >>> 0).toNumber();
                                }
                                if (d.active_view_measurability != null) {
                                    m.active_view_measurability = Number(d.active_view_measurability);
                                }
                                if (d.active_view_measurable_cost_micros != null) {
                                    if ($util.Long)
                                        (m.active_view_measurable_cost_micros = $util.Long.fromValue(d.active_view_measurable_cost_micros)).unsigned = false;
                                    else if (typeof d.active_view_measurable_cost_micros === "string")
                                        m.active_view_measurable_cost_micros = parseInt(d.active_view_measurable_cost_micros, 10);
                                    else if (typeof d.active_view_measurable_cost_micros === "number")
                                        m.active_view_measurable_cost_micros = d.active_view_measurable_cost_micros;
                                    else if (typeof d.active_view_measurable_cost_micros === "object")
                                        m.active_view_measurable_cost_micros = new $util.LongBits(d.active_view_measurable_cost_micros.low >>> 0, d.active_view_measurable_cost_micros.high >>> 0).toNumber();
                                }
                                if (d.active_view_measurable_impressions != null) {
                                    if ($util.Long)
                                        (m.active_view_measurable_impressions = $util.Long.fromValue(d.active_view_measurable_impressions)).unsigned = false;
                                    else if (typeof d.active_view_measurable_impressions === "string")
                                        m.active_view_measurable_impressions = parseInt(d.active_view_measurable_impressions, 10);
                                    else if (typeof d.active_view_measurable_impressions === "number")
                                        m.active_view_measurable_impressions = d.active_view_measurable_impressions;
                                    else if (typeof d.active_view_measurable_impressions === "object")
                                        m.active_view_measurable_impressions = new $util.LongBits(d.active_view_measurable_impressions.low >>> 0, d.active_view_measurable_impressions.high >>> 0).toNumber();
                                }
                                if (d.active_view_viewability != null) {
                                    m.active_view_viewability = Number(d.active_view_viewability);
                                }
                                if (d.all_conversions_from_interactions_rate != null) {
                                    m.all_conversions_from_interactions_rate = Number(d.all_conversions_from_interactions_rate);
                                }
                                if (d.all_conversions_value != null) {
                                    m.all_conversions_value = Number(d.all_conversions_value);
                                }
                                if (d.all_conversions_value_by_conversion_date != null) {
                                    m.all_conversions_value_by_conversion_date = Number(d.all_conversions_value_by_conversion_date);
                                }
                                if (d.all_conversions != null) {
                                    m.all_conversions = Number(d.all_conversions);
                                }
                                if (d.all_conversions_by_conversion_date != null) {
                                    m.all_conversions_by_conversion_date = Number(d.all_conversions_by_conversion_date);
                                }
                                if (d.all_conversions_value_per_cost != null) {
                                    m.all_conversions_value_per_cost = Number(d.all_conversions_value_per_cost);
                                }
                                if (d.all_conversions_from_click_to_call != null) {
                                    m.all_conversions_from_click_to_call = Number(d.all_conversions_from_click_to_call);
                                }
                                if (d.all_conversions_from_directions != null) {
                                    m.all_conversions_from_directions = Number(d.all_conversions_from_directions);
                                }
                                if (d.all_conversions_from_interactions_value_per_interaction != null) {
                                    m.all_conversions_from_interactions_value_per_interaction = Number(d.all_conversions_from_interactions_value_per_interaction);
                                }
                                if (d.all_conversions_from_menu != null) {
                                    m.all_conversions_from_menu = Number(d.all_conversions_from_menu);
                                }
                                if (d.all_conversions_from_order != null) {
                                    m.all_conversions_from_order = Number(d.all_conversions_from_order);
                                }
                                if (d.all_conversions_from_other_engagement != null) {
                                    m.all_conversions_from_other_engagement = Number(d.all_conversions_from_other_engagement);
                                }
                                if (d.all_conversions_from_store_visit != null) {
                                    m.all_conversions_from_store_visit = Number(d.all_conversions_from_store_visit);
                                }
                                if (d.all_conversions_from_store_website != null) {
                                    m.all_conversions_from_store_website = Number(d.all_conversions_from_store_website);
                                }
                                if (d.auction_insight_search_absolute_top_impression_percentage != null) {
                                    m.auction_insight_search_absolute_top_impression_percentage = Number(d.auction_insight_search_absolute_top_impression_percentage);
                                }
                                if (d.auction_insight_search_impression_share != null) {
                                    m.auction_insight_search_impression_share = Number(d.auction_insight_search_impression_share);
                                }
                                if (d.auction_insight_search_outranking_share != null) {
                                    m.auction_insight_search_outranking_share = Number(d.auction_insight_search_outranking_share);
                                }
                                if (d.auction_insight_search_overlap_rate != null) {
                                    m.auction_insight_search_overlap_rate = Number(d.auction_insight_search_overlap_rate);
                                }
                                if (d.auction_insight_search_position_above_rate != null) {
                                    m.auction_insight_search_position_above_rate = Number(d.auction_insight_search_position_above_rate);
                                }
                                if (d.auction_insight_search_top_impression_percentage != null) {
                                    m.auction_insight_search_top_impression_percentage = Number(d.auction_insight_search_top_impression_percentage);
                                }
                                if (d.average_cost != null) {
                                    m.average_cost = Number(d.average_cost);
                                }
                                if (d.average_cpc != null) {
                                    m.average_cpc = Number(d.average_cpc);
                                }
                                if (d.average_cpe != null) {
                                    m.average_cpe = Number(d.average_cpe);
                                }
                                if (d.average_cpm != null) {
                                    m.average_cpm = Number(d.average_cpm);
                                }
                                if (d.average_cpv != null) {
                                    m.average_cpv = Number(d.average_cpv);
                                }
                                if (d.average_page_views != null) {
                                    m.average_page_views = Number(d.average_page_views);
                                }
                                if (d.average_time_on_site != null) {
                                    m.average_time_on_site = Number(d.average_time_on_site);
                                }
                                if (d.benchmark_average_max_cpc != null) {
                                    m.benchmark_average_max_cpc = Number(d.benchmark_average_max_cpc);
                                }
                                if (d.biddable_app_install_conversions != null) {
                                    m.biddable_app_install_conversions = Number(d.biddable_app_install_conversions);
                                }
                                if (d.biddable_app_post_install_conversions != null) {
                                    m.biddable_app_post_install_conversions = Number(d.biddable_app_post_install_conversions);
                                }
                                if (d.benchmark_ctr != null) {
                                    m.benchmark_ctr = Number(d.benchmark_ctr);
                                }
                                if (d.bounce_rate != null) {
                                    m.bounce_rate = Number(d.bounce_rate);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.combined_clicks != null) {
                                    if ($util.Long)
                                        (m.combined_clicks = $util.Long.fromValue(d.combined_clicks)).unsigned = false;
                                    else if (typeof d.combined_clicks === "string")
                                        m.combined_clicks = parseInt(d.combined_clicks, 10);
                                    else if (typeof d.combined_clicks === "number")
                                        m.combined_clicks = d.combined_clicks;
                                    else if (typeof d.combined_clicks === "object")
                                        m.combined_clicks = new $util.LongBits(d.combined_clicks.low >>> 0, d.combined_clicks.high >>> 0).toNumber();
                                }
                                if (d.combined_clicks_per_query != null) {
                                    m.combined_clicks_per_query = Number(d.combined_clicks_per_query);
                                }
                                if (d.combined_queries != null) {
                                    if ($util.Long)
                                        (m.combined_queries = $util.Long.fromValue(d.combined_queries)).unsigned = false;
                                    else if (typeof d.combined_queries === "string")
                                        m.combined_queries = parseInt(d.combined_queries, 10);
                                    else if (typeof d.combined_queries === "number")
                                        m.combined_queries = d.combined_queries;
                                    else if (typeof d.combined_queries === "object")
                                        m.combined_queries = new $util.LongBits(d.combined_queries.low >>> 0, d.combined_queries.high >>> 0).toNumber();
                                }
                                if (d.content_budget_lost_impression_share != null) {
                                    m.content_budget_lost_impression_share = Number(d.content_budget_lost_impression_share);
                                }
                                if (d.content_impression_share != null) {
                                    m.content_impression_share = Number(d.content_impression_share);
                                }
                                if (d.conversion_last_received_request_date_time != null) {
                                    m.conversion_last_received_request_date_time = String(d.conversion_last_received_request_date_time);
                                }
                                if (d.conversion_last_conversion_date != null) {
                                    m.conversion_last_conversion_date = String(d.conversion_last_conversion_date);
                                }
                                if (d.content_rank_lost_impression_share != null) {
                                    m.content_rank_lost_impression_share = Number(d.content_rank_lost_impression_share);
                                }
                                if (d.conversions_from_interactions_rate != null) {
                                    m.conversions_from_interactions_rate = Number(d.conversions_from_interactions_rate);
                                }
                                if (d.conversions_value != null) {
                                    m.conversions_value = Number(d.conversions_value);
                                }
                                if (d.conversions_value_by_conversion_date != null) {
                                    m.conversions_value_by_conversion_date = Number(d.conversions_value_by_conversion_date);
                                }
                                if (d.conversions_value_per_cost != null) {
                                    m.conversions_value_per_cost = Number(d.conversions_value_per_cost);
                                }
                                if (d.conversions_from_interactions_value_per_interaction != null) {
                                    m.conversions_from_interactions_value_per_interaction = Number(d.conversions_from_interactions_value_per_interaction);
                                }
                                if (d.conversions != null) {
                                    m.conversions = Number(d.conversions);
                                }
                                if (d.conversions_by_conversion_date != null) {
                                    m.conversions_by_conversion_date = Number(d.conversions_by_conversion_date);
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.cost_per_all_conversions != null) {
                                    m.cost_per_all_conversions = Number(d.cost_per_all_conversions);
                                }
                                if (d.cost_per_conversion != null) {
                                    m.cost_per_conversion = Number(d.cost_per_conversion);
                                }
                                if (d.cost_per_current_model_attributed_conversion != null) {
                                    m.cost_per_current_model_attributed_conversion = Number(d.cost_per_current_model_attributed_conversion);
                                }
                                if (d.cross_device_conversions != null) {
                                    m.cross_device_conversions = Number(d.cross_device_conversions);
                                }
                                if (d.ctr != null) {
                                    m.ctr = Number(d.ctr);
                                }
                                if (d.current_model_attributed_conversions != null) {
                                    m.current_model_attributed_conversions = Number(d.current_model_attributed_conversions);
                                }
                                if (d.current_model_attributed_conversions_from_interactions_rate != null) {
                                    m.current_model_attributed_conversions_from_interactions_rate = Number(d.current_model_attributed_conversions_from_interactions_rate);
                                }
                                if (d.current_model_attributed_conversions_from_interactions_value_per_interaction != null) {
                                    m.current_model_attributed_conversions_from_interactions_value_per_interaction = Number(d.current_model_attributed_conversions_from_interactions_value_per_interaction);
                                }
                                if (d.current_model_attributed_conversions_value != null) {
                                    m.current_model_attributed_conversions_value = Number(d.current_model_attributed_conversions_value);
                                }
                                if (d.current_model_attributed_conversions_value_per_cost != null) {
                                    m.current_model_attributed_conversions_value_per_cost = Number(d.current_model_attributed_conversions_value_per_cost);
                                }
                                if (d.engagement_rate != null) {
                                    m.engagement_rate = Number(d.engagement_rate);
                                }
                                if (d.engagements != null) {
                                    if ($util.Long)
                                        (m.engagements = $util.Long.fromValue(d.engagements)).unsigned = false;
                                    else if (typeof d.engagements === "string")
                                        m.engagements = parseInt(d.engagements, 10);
                                    else if (typeof d.engagements === "number")
                                        m.engagements = d.engagements;
                                    else if (typeof d.engagements === "object")
                                        m.engagements = new $util.LongBits(d.engagements.low >>> 0, d.engagements.high >>> 0).toNumber();
                                }
                                if (d.hotel_average_lead_value_micros != null) {
                                    m.hotel_average_lead_value_micros = Number(d.hotel_average_lead_value_micros);
                                }
                                if (d.hotel_commission_rate_micros != null) {
                                    if ($util.Long)
                                        (m.hotel_commission_rate_micros = $util.Long.fromValue(d.hotel_commission_rate_micros)).unsigned = false;
                                    else if (typeof d.hotel_commission_rate_micros === "string")
                                        m.hotel_commission_rate_micros = parseInt(d.hotel_commission_rate_micros, 10);
                                    else if (typeof d.hotel_commission_rate_micros === "number")
                                        m.hotel_commission_rate_micros = d.hotel_commission_rate_micros;
                                    else if (typeof d.hotel_commission_rate_micros === "object")
                                        m.hotel_commission_rate_micros = new $util.LongBits(d.hotel_commission_rate_micros.low >>> 0, d.hotel_commission_rate_micros.high >>> 0).toNumber();
                                }
                                if (d.hotel_expected_commission_cost != null) {
                                    m.hotel_expected_commission_cost = Number(d.hotel_expected_commission_cost);
                                }
                                if (d.hotel_price_difference_percentage != null) {
                                    m.hotel_price_difference_percentage = Number(d.hotel_price_difference_percentage);
                                }
                                if (d.hotel_eligible_impressions != null) {
                                    if ($util.Long)
                                        (m.hotel_eligible_impressions = $util.Long.fromValue(d.hotel_eligible_impressions)).unsigned = false;
                                    else if (typeof d.hotel_eligible_impressions === "string")
                                        m.hotel_eligible_impressions = parseInt(d.hotel_eligible_impressions, 10);
                                    else if (typeof d.hotel_eligible_impressions === "number")
                                        m.hotel_eligible_impressions = d.hotel_eligible_impressions;
                                    else if (typeof d.hotel_eligible_impressions === "object")
                                        m.hotel_eligible_impressions = new $util.LongBits(d.hotel_eligible_impressions.low >>> 0, d.hotel_eligible_impressions.high >>> 0).toNumber();
                                }
                                switch (d.historical_creative_quality_score) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.historical_creative_quality_score = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.historical_creative_quality_score = 1;
                                    break;
                                case "BELOW_AVERAGE":
                                case 2:
                                    m.historical_creative_quality_score = 2;
                                    break;
                                case "AVERAGE":
                                case 3:
                                    m.historical_creative_quality_score = 3;
                                    break;
                                case "ABOVE_AVERAGE":
                                case 4:
                                    m.historical_creative_quality_score = 4;
                                    break;
                                }
                                switch (d.historical_landing_page_quality_score) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.historical_landing_page_quality_score = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.historical_landing_page_quality_score = 1;
                                    break;
                                case "BELOW_AVERAGE":
                                case 2:
                                    m.historical_landing_page_quality_score = 2;
                                    break;
                                case "AVERAGE":
                                case 3:
                                    m.historical_landing_page_quality_score = 3;
                                    break;
                                case "ABOVE_AVERAGE":
                                case 4:
                                    m.historical_landing_page_quality_score = 4;
                                    break;
                                }
                                if (d.historical_quality_score != null) {
                                    if ($util.Long)
                                        (m.historical_quality_score = $util.Long.fromValue(d.historical_quality_score)).unsigned = false;
                                    else if (typeof d.historical_quality_score === "string")
                                        m.historical_quality_score = parseInt(d.historical_quality_score, 10);
                                    else if (typeof d.historical_quality_score === "number")
                                        m.historical_quality_score = d.historical_quality_score;
                                    else if (typeof d.historical_quality_score === "object")
                                        m.historical_quality_score = new $util.LongBits(d.historical_quality_score.low >>> 0, d.historical_quality_score.high >>> 0).toNumber();
                                }
                                switch (d.historical_search_predicted_ctr) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.historical_search_predicted_ctr = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.historical_search_predicted_ctr = 1;
                                    break;
                                case "BELOW_AVERAGE":
                                case 2:
                                    m.historical_search_predicted_ctr = 2;
                                    break;
                                case "AVERAGE":
                                case 3:
                                    m.historical_search_predicted_ctr = 3;
                                    break;
                                case "ABOVE_AVERAGE":
                                case 4:
                                    m.historical_search_predicted_ctr = 4;
                                    break;
                                }
                                if (d.gmail_forwards != null) {
                                    if ($util.Long)
                                        (m.gmail_forwards = $util.Long.fromValue(d.gmail_forwards)).unsigned = false;
                                    else if (typeof d.gmail_forwards === "string")
                                        m.gmail_forwards = parseInt(d.gmail_forwards, 10);
                                    else if (typeof d.gmail_forwards === "number")
                                        m.gmail_forwards = d.gmail_forwards;
                                    else if (typeof d.gmail_forwards === "object")
                                        m.gmail_forwards = new $util.LongBits(d.gmail_forwards.low >>> 0, d.gmail_forwards.high >>> 0).toNumber();
                                }
                                if (d.gmail_saves != null) {
                                    if ($util.Long)
                                        (m.gmail_saves = $util.Long.fromValue(d.gmail_saves)).unsigned = false;
                                    else if (typeof d.gmail_saves === "string")
                                        m.gmail_saves = parseInt(d.gmail_saves, 10);
                                    else if (typeof d.gmail_saves === "number")
                                        m.gmail_saves = d.gmail_saves;
                                    else if (typeof d.gmail_saves === "object")
                                        m.gmail_saves = new $util.LongBits(d.gmail_saves.low >>> 0, d.gmail_saves.high >>> 0).toNumber();
                                }
                                if (d.gmail_secondary_clicks != null) {
                                    if ($util.Long)
                                        (m.gmail_secondary_clicks = $util.Long.fromValue(d.gmail_secondary_clicks)).unsigned = false;
                                    else if (typeof d.gmail_secondary_clicks === "string")
                                        m.gmail_secondary_clicks = parseInt(d.gmail_secondary_clicks, 10);
                                    else if (typeof d.gmail_secondary_clicks === "number")
                                        m.gmail_secondary_clicks = d.gmail_secondary_clicks;
                                    else if (typeof d.gmail_secondary_clicks === "object")
                                        m.gmail_secondary_clicks = new $util.LongBits(d.gmail_secondary_clicks.low >>> 0, d.gmail_secondary_clicks.high >>> 0).toNumber();
                                }
                                if (d.impressions_from_store_reach != null) {
                                    if ($util.Long)
                                        (m.impressions_from_store_reach = $util.Long.fromValue(d.impressions_from_store_reach)).unsigned = false;
                                    else if (typeof d.impressions_from_store_reach === "string")
                                        m.impressions_from_store_reach = parseInt(d.impressions_from_store_reach, 10);
                                    else if (typeof d.impressions_from_store_reach === "number")
                                        m.impressions_from_store_reach = d.impressions_from_store_reach;
                                    else if (typeof d.impressions_from_store_reach === "object")
                                        m.impressions_from_store_reach = new $util.LongBits(d.impressions_from_store_reach.low >>> 0, d.impressions_from_store_reach.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.interaction_rate != null) {
                                    m.interaction_rate = Number(d.interaction_rate);
                                }
                                if (d.interactions != null) {
                                    if ($util.Long)
                                        (m.interactions = $util.Long.fromValue(d.interactions)).unsigned = false;
                                    else if (typeof d.interactions === "string")
                                        m.interactions = parseInt(d.interactions, 10);
                                    else if (typeof d.interactions === "number")
                                        m.interactions = d.interactions;
                                    else if (typeof d.interactions === "object")
                                        m.interactions = new $util.LongBits(d.interactions.low >>> 0, d.interactions.high >>> 0).toNumber();
                                }
                                if (d.interaction_event_types) {
                                    if (!Array.isArray(d.interaction_event_types))
                                        throw TypeError(".google.ads.googleads.v11.common.Metrics.interaction_event_types: array expected");
                                    m.interaction_event_types = [];
                                    for (var i = 0; i < d.interaction_event_types.length; ++i) {
                                        switch (d.interaction_event_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.interaction_event_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.interaction_event_types[i] = 1;
                                            break;
                                        case "CLICK":
                                        case 2:
                                            m.interaction_event_types[i] = 2;
                                            break;
                                        case "ENGAGEMENT":
                                        case 3:
                                            m.interaction_event_types[i] = 3;
                                            break;
                                        case "VIDEO_VIEW":
                                        case 4:
                                            m.interaction_event_types[i] = 4;
                                            break;
                                        case "NONE":
                                        case 5:
                                            m.interaction_event_types[i] = 5;
                                            break;
                                        }
                                    }
                                }
                                if (d.invalid_click_rate != null) {
                                    m.invalid_click_rate = Number(d.invalid_click_rate);
                                }
                                if (d.invalid_clicks != null) {
                                    if ($util.Long)
                                        (m.invalid_clicks = $util.Long.fromValue(d.invalid_clicks)).unsigned = false;
                                    else if (typeof d.invalid_clicks === "string")
                                        m.invalid_clicks = parseInt(d.invalid_clicks, 10);
                                    else if (typeof d.invalid_clicks === "number")
                                        m.invalid_clicks = d.invalid_clicks;
                                    else if (typeof d.invalid_clicks === "object")
                                        m.invalid_clicks = new $util.LongBits(d.invalid_clicks.low >>> 0, d.invalid_clicks.high >>> 0).toNumber();
                                }
                                if (d.message_chats != null) {
                                    if ($util.Long)
                                        (m.message_chats = $util.Long.fromValue(d.message_chats)).unsigned = false;
                                    else if (typeof d.message_chats === "string")
                                        m.message_chats = parseInt(d.message_chats, 10);
                                    else if (typeof d.message_chats === "number")
                                        m.message_chats = d.message_chats;
                                    else if (typeof d.message_chats === "object")
                                        m.message_chats = new $util.LongBits(d.message_chats.low >>> 0, d.message_chats.high >>> 0).toNumber();
                                }
                                if (d.message_impressions != null) {
                                    if ($util.Long)
                                        (m.message_impressions = $util.Long.fromValue(d.message_impressions)).unsigned = false;
                                    else if (typeof d.message_impressions === "string")
                                        m.message_impressions = parseInt(d.message_impressions, 10);
                                    else if (typeof d.message_impressions === "number")
                                        m.message_impressions = d.message_impressions;
                                    else if (typeof d.message_impressions === "object")
                                        m.message_impressions = new $util.LongBits(d.message_impressions.low >>> 0, d.message_impressions.high >>> 0).toNumber();
                                }
                                if (d.message_chat_rate != null) {
                                    m.message_chat_rate = Number(d.message_chat_rate);
                                }
                                if (d.mobile_friendly_clicks_percentage != null) {
                                    m.mobile_friendly_clicks_percentage = Number(d.mobile_friendly_clicks_percentage);
                                }
                                if (d.optimization_score_uplift != null) {
                                    m.optimization_score_uplift = Number(d.optimization_score_uplift);
                                }
                                if (d.optimization_score_url != null) {
                                    m.optimization_score_url = String(d.optimization_score_url);
                                }
                                if (d.organic_clicks != null) {
                                    if ($util.Long)
                                        (m.organic_clicks = $util.Long.fromValue(d.organic_clicks)).unsigned = false;
                                    else if (typeof d.organic_clicks === "string")
                                        m.organic_clicks = parseInt(d.organic_clicks, 10);
                                    else if (typeof d.organic_clicks === "number")
                                        m.organic_clicks = d.organic_clicks;
                                    else if (typeof d.organic_clicks === "object")
                                        m.organic_clicks = new $util.LongBits(d.organic_clicks.low >>> 0, d.organic_clicks.high >>> 0).toNumber();
                                }
                                if (d.organic_clicks_per_query != null) {
                                    m.organic_clicks_per_query = Number(d.organic_clicks_per_query);
                                }
                                if (d.organic_impressions != null) {
                                    if ($util.Long)
                                        (m.organic_impressions = $util.Long.fromValue(d.organic_impressions)).unsigned = false;
                                    else if (typeof d.organic_impressions === "string")
                                        m.organic_impressions = parseInt(d.organic_impressions, 10);
                                    else if (typeof d.organic_impressions === "number")
                                        m.organic_impressions = d.organic_impressions;
                                    else if (typeof d.organic_impressions === "object")
                                        m.organic_impressions = new $util.LongBits(d.organic_impressions.low >>> 0, d.organic_impressions.high >>> 0).toNumber();
                                }
                                if (d.organic_impressions_per_query != null) {
                                    m.organic_impressions_per_query = Number(d.organic_impressions_per_query);
                                }
                                if (d.organic_queries != null) {
                                    if ($util.Long)
                                        (m.organic_queries = $util.Long.fromValue(d.organic_queries)).unsigned = false;
                                    else if (typeof d.organic_queries === "string")
                                        m.organic_queries = parseInt(d.organic_queries, 10);
                                    else if (typeof d.organic_queries === "number")
                                        m.organic_queries = d.organic_queries;
                                    else if (typeof d.organic_queries === "object")
                                        m.organic_queries = new $util.LongBits(d.organic_queries.low >>> 0, d.organic_queries.high >>> 0).toNumber();
                                }
                                if (d.percent_new_visitors != null) {
                                    m.percent_new_visitors = Number(d.percent_new_visitors);
                                }
                                if (d.phone_calls != null) {
                                    if ($util.Long)
                                        (m.phone_calls = $util.Long.fromValue(d.phone_calls)).unsigned = false;
                                    else if (typeof d.phone_calls === "string")
                                        m.phone_calls = parseInt(d.phone_calls, 10);
                                    else if (typeof d.phone_calls === "number")
                                        m.phone_calls = d.phone_calls;
                                    else if (typeof d.phone_calls === "object")
                                        m.phone_calls = new $util.LongBits(d.phone_calls.low >>> 0, d.phone_calls.high >>> 0).toNumber();
                                }
                                if (d.phone_impressions != null) {
                                    if ($util.Long)
                                        (m.phone_impressions = $util.Long.fromValue(d.phone_impressions)).unsigned = false;
                                    else if (typeof d.phone_impressions === "string")
                                        m.phone_impressions = parseInt(d.phone_impressions, 10);
                                    else if (typeof d.phone_impressions === "number")
                                        m.phone_impressions = d.phone_impressions;
                                    else if (typeof d.phone_impressions === "object")
                                        m.phone_impressions = new $util.LongBits(d.phone_impressions.low >>> 0, d.phone_impressions.high >>> 0).toNumber();
                                }
                                if (d.phone_through_rate != null) {
                                    m.phone_through_rate = Number(d.phone_through_rate);
                                }
                                if (d.relative_ctr != null) {
                                    m.relative_ctr = Number(d.relative_ctr);
                                }
                                if (d.search_absolute_top_impression_share != null) {
                                    m.search_absolute_top_impression_share = Number(d.search_absolute_top_impression_share);
                                }
                                if (d.search_budget_lost_absolute_top_impression_share != null) {
                                    m.search_budget_lost_absolute_top_impression_share = Number(d.search_budget_lost_absolute_top_impression_share);
                                }
                                if (d.search_budget_lost_impression_share != null) {
                                    m.search_budget_lost_impression_share = Number(d.search_budget_lost_impression_share);
                                }
                                if (d.search_budget_lost_top_impression_share != null) {
                                    m.search_budget_lost_top_impression_share = Number(d.search_budget_lost_top_impression_share);
                                }
                                if (d.search_click_share != null) {
                                    m.search_click_share = Number(d.search_click_share);
                                }
                                if (d.search_exact_match_impression_share != null) {
                                    m.search_exact_match_impression_share = Number(d.search_exact_match_impression_share);
                                }
                                if (d.search_impression_share != null) {
                                    m.search_impression_share = Number(d.search_impression_share);
                                }
                                if (d.search_rank_lost_absolute_top_impression_share != null) {
                                    m.search_rank_lost_absolute_top_impression_share = Number(d.search_rank_lost_absolute_top_impression_share);
                                }
                                if (d.search_rank_lost_impression_share != null) {
                                    m.search_rank_lost_impression_share = Number(d.search_rank_lost_impression_share);
                                }
                                if (d.search_rank_lost_top_impression_share != null) {
                                    m.search_rank_lost_top_impression_share = Number(d.search_rank_lost_top_impression_share);
                                }
                                if (d.search_top_impression_share != null) {
                                    m.search_top_impression_share = Number(d.search_top_impression_share);
                                }
                                if (d.speed_score != null) {
                                    if ($util.Long)
                                        (m.speed_score = $util.Long.fromValue(d.speed_score)).unsigned = false;
                                    else if (typeof d.speed_score === "string")
                                        m.speed_score = parseInt(d.speed_score, 10);
                                    else if (typeof d.speed_score === "number")
                                        m.speed_score = d.speed_score;
                                    else if (typeof d.speed_score === "object")
                                        m.speed_score = new $util.LongBits(d.speed_score.low >>> 0, d.speed_score.high >>> 0).toNumber();
                                }
                                if (d.top_impression_percentage != null) {
                                    m.top_impression_percentage = Number(d.top_impression_percentage);
                                }
                                if (d.valid_accelerated_mobile_pages_clicks_percentage != null) {
                                    m.valid_accelerated_mobile_pages_clicks_percentage = Number(d.valid_accelerated_mobile_pages_clicks_percentage);
                                }
                                if (d.value_per_all_conversions != null) {
                                    m.value_per_all_conversions = Number(d.value_per_all_conversions);
                                }
                                if (d.value_per_all_conversions_by_conversion_date != null) {
                                    m.value_per_all_conversions_by_conversion_date = Number(d.value_per_all_conversions_by_conversion_date);
                                }
                                if (d.value_per_conversion != null) {
                                    m.value_per_conversion = Number(d.value_per_conversion);
                                }
                                if (d.value_per_conversions_by_conversion_date != null) {
                                    m.value_per_conversions_by_conversion_date = Number(d.value_per_conversions_by_conversion_date);
                                }
                                if (d.value_per_current_model_attributed_conversion != null) {
                                    m.value_per_current_model_attributed_conversion = Number(d.value_per_current_model_attributed_conversion);
                                }
                                if (d.video_quartile_p100_rate != null) {
                                    m.video_quartile_p100_rate = Number(d.video_quartile_p100_rate);
                                }
                                if (d.video_quartile_p25_rate != null) {
                                    m.video_quartile_p25_rate = Number(d.video_quartile_p25_rate);
                                }
                                if (d.video_quartile_p50_rate != null) {
                                    m.video_quartile_p50_rate = Number(d.video_quartile_p50_rate);
                                }
                                if (d.video_quartile_p75_rate != null) {
                                    m.video_quartile_p75_rate = Number(d.video_quartile_p75_rate);
                                }
                                if (d.video_view_rate != null) {
                                    m.video_view_rate = Number(d.video_view_rate);
                                }
                                if (d.video_views != null) {
                                    if ($util.Long)
                                        (m.video_views = $util.Long.fromValue(d.video_views)).unsigned = false;
                                    else if (typeof d.video_views === "string")
                                        m.video_views = parseInt(d.video_views, 10);
                                    else if (typeof d.video_views === "number")
                                        m.video_views = d.video_views;
                                    else if (typeof d.video_views === "object")
                                        m.video_views = new $util.LongBits(d.video_views.low >>> 0, d.video_views.high >>> 0).toNumber();
                                }
                                if (d.view_through_conversions != null) {
                                    if ($util.Long)
                                        (m.view_through_conversions = $util.Long.fromValue(d.view_through_conversions)).unsigned = false;
                                    else if (typeof d.view_through_conversions === "string")
                                        m.view_through_conversions = parseInt(d.view_through_conversions, 10);
                                    else if (typeof d.view_through_conversions === "number")
                                        m.view_through_conversions = d.view_through_conversions;
                                    else if (typeof d.view_through_conversions === "object")
                                        m.view_through_conversions = new $util.LongBits(d.view_through_conversions.low >>> 0, d.view_through_conversions.high >>> 0).toNumber();
                                }
                                if (d.sk_ad_network_conversions != null) {
                                    if ($util.Long)
                                        (m.sk_ad_network_conversions = $util.Long.fromValue(d.sk_ad_network_conversions)).unsigned = false;
                                    else if (typeof d.sk_ad_network_conversions === "string")
                                        m.sk_ad_network_conversions = parseInt(d.sk_ad_network_conversions, 10);
                                    else if (typeof d.sk_ad_network_conversions === "number")
                                        m.sk_ad_network_conversions = d.sk_ad_network_conversions;
                                    else if (typeof d.sk_ad_network_conversions === "object")
                                        m.sk_ad_network_conversions = new $util.LongBits(d.sk_ad_network_conversions.low >>> 0, d.sk_ad_network_conversions.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Metrics message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @static
                             * @param {google.ads.googleads.v11.common.Metrics} m Metrics
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Metrics.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.interaction_event_types = [];
                                }
                                if (o.defaults) {
                                    d.historical_creative_quality_score = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.historical_landing_page_quality_score = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.historical_search_predicted_ctr = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.all_conversions_value_by_conversion_date = 0;
                                    d.all_conversions_by_conversion_date = 0;
                                    d.conversions_value_by_conversion_date = 0;
                                    d.conversions_by_conversion_date = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.sk_ad_network_conversions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.sk_ad_network_conversions = o.longs === String ? "0" : 0;
                                }
                                if (m.historical_creative_quality_score != null && m.hasOwnProperty("historical_creative_quality_score")) {
                                    d.historical_creative_quality_score = o.enums === String ? $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket[m.historical_creative_quality_score] : m.historical_creative_quality_score;
                                }
                                if (m.historical_landing_page_quality_score != null && m.hasOwnProperty("historical_landing_page_quality_score")) {
                                    d.historical_landing_page_quality_score = o.enums === String ? $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket[m.historical_landing_page_quality_score] : m.historical_landing_page_quality_score;
                                }
                                if (m.historical_search_predicted_ctr != null && m.hasOwnProperty("historical_search_predicted_ctr")) {
                                    d.historical_search_predicted_ctr = o.enums === String ? $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket[m.historical_search_predicted_ctr] : m.historical_search_predicted_ctr;
                                }
                                if (m.interaction_event_types && m.interaction_event_types.length) {
                                    d.interaction_event_types = [];
                                    for (var j = 0; j < m.interaction_event_types.length; ++j) {
                                        d.interaction_event_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.InteractionEventTypeEnum.InteractionEventType[m.interaction_event_types[j]] : m.interaction_event_types[j];
                                    }
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                    if (o.oneofs)
                                        d._clicks = "clicks";
                                }
                                if (m.video_quartile_p100_rate != null && m.hasOwnProperty("video_quartile_p100_rate")) {
                                    d.video_quartile_p100_rate = o.json && !isFinite(m.video_quartile_p100_rate) ? String(m.video_quartile_p100_rate) : m.video_quartile_p100_rate;
                                    if (o.oneofs)
                                        d._video_quartile_p100_rate = "video_quartile_p100_rate";
                                }
                                if (m.video_quartile_p25_rate != null && m.hasOwnProperty("video_quartile_p25_rate")) {
                                    d.video_quartile_p25_rate = o.json && !isFinite(m.video_quartile_p25_rate) ? String(m.video_quartile_p25_rate) : m.video_quartile_p25_rate;
                                    if (o.oneofs)
                                        d._video_quartile_p25_rate = "video_quartile_p25_rate";
                                }
                                if (m.video_quartile_p50_rate != null && m.hasOwnProperty("video_quartile_p50_rate")) {
                                    d.video_quartile_p50_rate = o.json && !isFinite(m.video_quartile_p50_rate) ? String(m.video_quartile_p50_rate) : m.video_quartile_p50_rate;
                                    if (o.oneofs)
                                        d._video_quartile_p50_rate = "video_quartile_p50_rate";
                                }
                                if (m.video_quartile_p75_rate != null && m.hasOwnProperty("video_quartile_p75_rate")) {
                                    d.video_quartile_p75_rate = o.json && !isFinite(m.video_quartile_p75_rate) ? String(m.video_quartile_p75_rate) : m.video_quartile_p75_rate;
                                    if (o.oneofs)
                                        d._video_quartile_p75_rate = "video_quartile_p75_rate";
                                }
                                if (m.search_absolute_top_impression_share != null && m.hasOwnProperty("search_absolute_top_impression_share")) {
                                    d.search_absolute_top_impression_share = o.json && !isFinite(m.search_absolute_top_impression_share) ? String(m.search_absolute_top_impression_share) : m.search_absolute_top_impression_share;
                                    if (o.oneofs)
                                        d._search_absolute_top_impression_share = "search_absolute_top_impression_share";
                                }
                                if (m.search_budget_lost_absolute_top_impression_share != null && m.hasOwnProperty("search_budget_lost_absolute_top_impression_share")) {
                                    d.search_budget_lost_absolute_top_impression_share = o.json && !isFinite(m.search_budget_lost_absolute_top_impression_share) ? String(m.search_budget_lost_absolute_top_impression_share) : m.search_budget_lost_absolute_top_impression_share;
                                    if (o.oneofs)
                                        d._search_budget_lost_absolute_top_impression_share = "search_budget_lost_absolute_top_impression_share";
                                }
                                if (m.search_budget_lost_impression_share != null && m.hasOwnProperty("search_budget_lost_impression_share")) {
                                    d.search_budget_lost_impression_share = o.json && !isFinite(m.search_budget_lost_impression_share) ? String(m.search_budget_lost_impression_share) : m.search_budget_lost_impression_share;
                                    if (o.oneofs)
                                        d._search_budget_lost_impression_share = "search_budget_lost_impression_share";
                                }
                                if (m.search_budget_lost_top_impression_share != null && m.hasOwnProperty("search_budget_lost_top_impression_share")) {
                                    d.search_budget_lost_top_impression_share = o.json && !isFinite(m.search_budget_lost_top_impression_share) ? String(m.search_budget_lost_top_impression_share) : m.search_budget_lost_top_impression_share;
                                    if (o.oneofs)
                                        d._search_budget_lost_top_impression_share = "search_budget_lost_top_impression_share";
                                }
                                if (m.search_click_share != null && m.hasOwnProperty("search_click_share")) {
                                    d.search_click_share = o.json && !isFinite(m.search_click_share) ? String(m.search_click_share) : m.search_click_share;
                                    if (o.oneofs)
                                        d._search_click_share = "search_click_share";
                                }
                                if (m.search_exact_match_impression_share != null && m.hasOwnProperty("search_exact_match_impression_share")) {
                                    d.search_exact_match_impression_share = o.json && !isFinite(m.search_exact_match_impression_share) ? String(m.search_exact_match_impression_share) : m.search_exact_match_impression_share;
                                    if (o.oneofs)
                                        d._search_exact_match_impression_share = "search_exact_match_impression_share";
                                }
                                if (m.search_impression_share != null && m.hasOwnProperty("search_impression_share")) {
                                    d.search_impression_share = o.json && !isFinite(m.search_impression_share) ? String(m.search_impression_share) : m.search_impression_share;
                                    if (o.oneofs)
                                        d._search_impression_share = "search_impression_share";
                                }
                                if (m.search_rank_lost_absolute_top_impression_share != null && m.hasOwnProperty("search_rank_lost_absolute_top_impression_share")) {
                                    d.search_rank_lost_absolute_top_impression_share = o.json && !isFinite(m.search_rank_lost_absolute_top_impression_share) ? String(m.search_rank_lost_absolute_top_impression_share) : m.search_rank_lost_absolute_top_impression_share;
                                    if (o.oneofs)
                                        d._search_rank_lost_absolute_top_impression_share = "search_rank_lost_absolute_top_impression_share";
                                }
                                if (m.search_rank_lost_impression_share != null && m.hasOwnProperty("search_rank_lost_impression_share")) {
                                    d.search_rank_lost_impression_share = o.json && !isFinite(m.search_rank_lost_impression_share) ? String(m.search_rank_lost_impression_share) : m.search_rank_lost_impression_share;
                                    if (o.oneofs)
                                        d._search_rank_lost_impression_share = "search_rank_lost_impression_share";
                                }
                                if (m.search_rank_lost_top_impression_share != null && m.hasOwnProperty("search_rank_lost_top_impression_share")) {
                                    d.search_rank_lost_top_impression_share = o.json && !isFinite(m.search_rank_lost_top_impression_share) ? String(m.search_rank_lost_top_impression_share) : m.search_rank_lost_top_impression_share;
                                    if (o.oneofs)
                                        d._search_rank_lost_top_impression_share = "search_rank_lost_top_impression_share";
                                }
                                if (m.search_top_impression_share != null && m.hasOwnProperty("search_top_impression_share")) {
                                    d.search_top_impression_share = o.json && !isFinite(m.search_top_impression_share) ? String(m.search_top_impression_share) : m.search_top_impression_share;
                                    if (o.oneofs)
                                        d._search_top_impression_share = "search_top_impression_share";
                                }
                                if (m.speed_score != null && m.hasOwnProperty("speed_score")) {
                                    if (typeof m.speed_score === "number")
                                        d.speed_score = o.longs === String ? String(m.speed_score) : m.speed_score;
                                    else
                                        d.speed_score = o.longs === String ? $util.Long.prototype.toString.call(m.speed_score) : o.longs === Number ? new $util.LongBits(m.speed_score.low >>> 0, m.speed_score.high >>> 0).toNumber() : m.speed_score;
                                    if (o.oneofs)
                                        d._speed_score = "speed_score";
                                }
                                if (m.top_impression_percentage != null && m.hasOwnProperty("top_impression_percentage")) {
                                    d.top_impression_percentage = o.json && !isFinite(m.top_impression_percentage) ? String(m.top_impression_percentage) : m.top_impression_percentage;
                                    if (o.oneofs)
                                        d._top_impression_percentage = "top_impression_percentage";
                                }
                                if (m.valid_accelerated_mobile_pages_clicks_percentage != null && m.hasOwnProperty("valid_accelerated_mobile_pages_clicks_percentage")) {
                                    d.valid_accelerated_mobile_pages_clicks_percentage = o.json && !isFinite(m.valid_accelerated_mobile_pages_clicks_percentage) ? String(m.valid_accelerated_mobile_pages_clicks_percentage) : m.valid_accelerated_mobile_pages_clicks_percentage;
                                    if (o.oneofs)
                                        d._valid_accelerated_mobile_pages_clicks_percentage = "valid_accelerated_mobile_pages_clicks_percentage";
                                }
                                if (m.value_per_all_conversions != null && m.hasOwnProperty("value_per_all_conversions")) {
                                    d.value_per_all_conversions = o.json && !isFinite(m.value_per_all_conversions) ? String(m.value_per_all_conversions) : m.value_per_all_conversions;
                                    if (o.oneofs)
                                        d._value_per_all_conversions = "value_per_all_conversions";
                                }
                                if (m.value_per_conversion != null && m.hasOwnProperty("value_per_conversion")) {
                                    d.value_per_conversion = o.json && !isFinite(m.value_per_conversion) ? String(m.value_per_conversion) : m.value_per_conversion;
                                    if (o.oneofs)
                                        d._value_per_conversion = "value_per_conversion";
                                }
                                if (m.value_per_current_model_attributed_conversion != null && m.hasOwnProperty("value_per_current_model_attributed_conversion")) {
                                    d.value_per_current_model_attributed_conversion = o.json && !isFinite(m.value_per_current_model_attributed_conversion) ? String(m.value_per_current_model_attributed_conversion) : m.value_per_current_model_attributed_conversion;
                                    if (o.oneofs)
                                        d._value_per_current_model_attributed_conversion = "value_per_current_model_attributed_conversion";
                                }
                                if (m.video_view_rate != null && m.hasOwnProperty("video_view_rate")) {
                                    d.video_view_rate = o.json && !isFinite(m.video_view_rate) ? String(m.video_view_rate) : m.video_view_rate;
                                    if (o.oneofs)
                                        d._video_view_rate = "video_view_rate";
                                }
                                if (m.video_views != null && m.hasOwnProperty("video_views")) {
                                    if (typeof m.video_views === "number")
                                        d.video_views = o.longs === String ? String(m.video_views) : m.video_views;
                                    else
                                        d.video_views = o.longs === String ? $util.Long.prototype.toString.call(m.video_views) : o.longs === Number ? new $util.LongBits(m.video_views.low >>> 0, m.video_views.high >>> 0).toNumber() : m.video_views;
                                    if (o.oneofs)
                                        d._video_views = "video_views";
                                }
                                if (m.view_through_conversions != null && m.hasOwnProperty("view_through_conversions")) {
                                    if (typeof m.view_through_conversions === "number")
                                        d.view_through_conversions = o.longs === String ? String(m.view_through_conversions) : m.view_through_conversions;
                                    else
                                        d.view_through_conversions = o.longs === String ? $util.Long.prototype.toString.call(m.view_through_conversions) : o.longs === Number ? new $util.LongBits(m.view_through_conversions.low >>> 0, m.view_through_conversions.high >>> 0).toNumber() : m.view_through_conversions;
                                    if (o.oneofs)
                                        d._view_through_conversions = "view_through_conversions";
                                }
                                if (m.combined_clicks != null && m.hasOwnProperty("combined_clicks")) {
                                    if (typeof m.combined_clicks === "number")
                                        d.combined_clicks = o.longs === String ? String(m.combined_clicks) : m.combined_clicks;
                                    else
                                        d.combined_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.combined_clicks) : o.longs === Number ? new $util.LongBits(m.combined_clicks.low >>> 0, m.combined_clicks.high >>> 0).toNumber() : m.combined_clicks;
                                    if (o.oneofs)
                                        d._combined_clicks = "combined_clicks";
                                }
                                if (m.combined_clicks_per_query != null && m.hasOwnProperty("combined_clicks_per_query")) {
                                    d.combined_clicks_per_query = o.json && !isFinite(m.combined_clicks_per_query) ? String(m.combined_clicks_per_query) : m.combined_clicks_per_query;
                                    if (o.oneofs)
                                        d._combined_clicks_per_query = "combined_clicks_per_query";
                                }
                                if (m.combined_queries != null && m.hasOwnProperty("combined_queries")) {
                                    if (typeof m.combined_queries === "number")
                                        d.combined_queries = o.longs === String ? String(m.combined_queries) : m.combined_queries;
                                    else
                                        d.combined_queries = o.longs === String ? $util.Long.prototype.toString.call(m.combined_queries) : o.longs === Number ? new $util.LongBits(m.combined_queries.low >>> 0, m.combined_queries.high >>> 0).toNumber() : m.combined_queries;
                                    if (o.oneofs)
                                        d._combined_queries = "combined_queries";
                                }
                                if (m.content_budget_lost_impression_share != null && m.hasOwnProperty("content_budget_lost_impression_share")) {
                                    d.content_budget_lost_impression_share = o.json && !isFinite(m.content_budget_lost_impression_share) ? String(m.content_budget_lost_impression_share) : m.content_budget_lost_impression_share;
                                    if (o.oneofs)
                                        d._content_budget_lost_impression_share = "content_budget_lost_impression_share";
                                }
                                if (m.content_impression_share != null && m.hasOwnProperty("content_impression_share")) {
                                    d.content_impression_share = o.json && !isFinite(m.content_impression_share) ? String(m.content_impression_share) : m.content_impression_share;
                                    if (o.oneofs)
                                        d._content_impression_share = "content_impression_share";
                                }
                                if (m.conversion_last_received_request_date_time != null && m.hasOwnProperty("conversion_last_received_request_date_time")) {
                                    d.conversion_last_received_request_date_time = m.conversion_last_received_request_date_time;
                                    if (o.oneofs)
                                        d._conversion_last_received_request_date_time = "conversion_last_received_request_date_time";
                                }
                                if (m.conversion_last_conversion_date != null && m.hasOwnProperty("conversion_last_conversion_date")) {
                                    d.conversion_last_conversion_date = m.conversion_last_conversion_date;
                                    if (o.oneofs)
                                        d._conversion_last_conversion_date = "conversion_last_conversion_date";
                                }
                                if (m.content_rank_lost_impression_share != null && m.hasOwnProperty("content_rank_lost_impression_share")) {
                                    d.content_rank_lost_impression_share = o.json && !isFinite(m.content_rank_lost_impression_share) ? String(m.content_rank_lost_impression_share) : m.content_rank_lost_impression_share;
                                    if (o.oneofs)
                                        d._content_rank_lost_impression_share = "content_rank_lost_impression_share";
                                }
                                if (m.conversions_from_interactions_rate != null && m.hasOwnProperty("conversions_from_interactions_rate")) {
                                    d.conversions_from_interactions_rate = o.json && !isFinite(m.conversions_from_interactions_rate) ? String(m.conversions_from_interactions_rate) : m.conversions_from_interactions_rate;
                                    if (o.oneofs)
                                        d._conversions_from_interactions_rate = "conversions_from_interactions_rate";
                                }
                                if (m.conversions_value != null && m.hasOwnProperty("conversions_value")) {
                                    d.conversions_value = o.json && !isFinite(m.conversions_value) ? String(m.conversions_value) : m.conversions_value;
                                    if (o.oneofs)
                                        d._conversions_value = "conversions_value";
                                }
                                if (m.conversions_value_per_cost != null && m.hasOwnProperty("conversions_value_per_cost")) {
                                    d.conversions_value_per_cost = o.json && !isFinite(m.conversions_value_per_cost) ? String(m.conversions_value_per_cost) : m.conversions_value_per_cost;
                                    if (o.oneofs)
                                        d._conversions_value_per_cost = "conversions_value_per_cost";
                                }
                                if (m.conversions_from_interactions_value_per_interaction != null && m.hasOwnProperty("conversions_from_interactions_value_per_interaction")) {
                                    d.conversions_from_interactions_value_per_interaction = o.json && !isFinite(m.conversions_from_interactions_value_per_interaction) ? String(m.conversions_from_interactions_value_per_interaction) : m.conversions_from_interactions_value_per_interaction;
                                    if (o.oneofs)
                                        d._conversions_from_interactions_value_per_interaction = "conversions_from_interactions_value_per_interaction";
                                }
                                if (m.conversions != null && m.hasOwnProperty("conversions")) {
                                    d.conversions = o.json && !isFinite(m.conversions) ? String(m.conversions) : m.conversions;
                                    if (o.oneofs)
                                        d._conversions = "conversions";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                if (m.cost_per_all_conversions != null && m.hasOwnProperty("cost_per_all_conversions")) {
                                    d.cost_per_all_conversions = o.json && !isFinite(m.cost_per_all_conversions) ? String(m.cost_per_all_conversions) : m.cost_per_all_conversions;
                                    if (o.oneofs)
                                        d._cost_per_all_conversions = "cost_per_all_conversions";
                                }
                                if (m.cost_per_conversion != null && m.hasOwnProperty("cost_per_conversion")) {
                                    d.cost_per_conversion = o.json && !isFinite(m.cost_per_conversion) ? String(m.cost_per_conversion) : m.cost_per_conversion;
                                    if (o.oneofs)
                                        d._cost_per_conversion = "cost_per_conversion";
                                }
                                if (m.cost_per_current_model_attributed_conversion != null && m.hasOwnProperty("cost_per_current_model_attributed_conversion")) {
                                    d.cost_per_current_model_attributed_conversion = o.json && !isFinite(m.cost_per_current_model_attributed_conversion) ? String(m.cost_per_current_model_attributed_conversion) : m.cost_per_current_model_attributed_conversion;
                                    if (o.oneofs)
                                        d._cost_per_current_model_attributed_conversion = "cost_per_current_model_attributed_conversion";
                                }
                                if (m.cross_device_conversions != null && m.hasOwnProperty("cross_device_conversions")) {
                                    d.cross_device_conversions = o.json && !isFinite(m.cross_device_conversions) ? String(m.cross_device_conversions) : m.cross_device_conversions;
                                    if (o.oneofs)
                                        d._cross_device_conversions = "cross_device_conversions";
                                }
                                if (m.ctr != null && m.hasOwnProperty("ctr")) {
                                    d.ctr = o.json && !isFinite(m.ctr) ? String(m.ctr) : m.ctr;
                                    if (o.oneofs)
                                        d._ctr = "ctr";
                                }
                                if (m.current_model_attributed_conversions != null && m.hasOwnProperty("current_model_attributed_conversions")) {
                                    d.current_model_attributed_conversions = o.json && !isFinite(m.current_model_attributed_conversions) ? String(m.current_model_attributed_conversions) : m.current_model_attributed_conversions;
                                    if (o.oneofs)
                                        d._current_model_attributed_conversions = "current_model_attributed_conversions";
                                }
                                if (m.current_model_attributed_conversions_from_interactions_rate != null && m.hasOwnProperty("current_model_attributed_conversions_from_interactions_rate")) {
                                    d.current_model_attributed_conversions_from_interactions_rate = o.json && !isFinite(m.current_model_attributed_conversions_from_interactions_rate) ? String(m.current_model_attributed_conversions_from_interactions_rate) : m.current_model_attributed_conversions_from_interactions_rate;
                                    if (o.oneofs)
                                        d._current_model_attributed_conversions_from_interactions_rate = "current_model_attributed_conversions_from_interactions_rate";
                                }
                                if (m.current_model_attributed_conversions_from_interactions_value_per_interaction != null && m.hasOwnProperty("current_model_attributed_conversions_from_interactions_value_per_interaction")) {
                                    d.current_model_attributed_conversions_from_interactions_value_per_interaction = o.json && !isFinite(m.current_model_attributed_conversions_from_interactions_value_per_interaction) ? String(m.current_model_attributed_conversions_from_interactions_value_per_interaction) : m.current_model_attributed_conversions_from_interactions_value_per_interaction;
                                    if (o.oneofs)
                                        d._current_model_attributed_conversions_from_interactions_value_per_interaction = "current_model_attributed_conversions_from_interactions_value_per_interaction";
                                }
                                if (m.current_model_attributed_conversions_value != null && m.hasOwnProperty("current_model_attributed_conversions_value")) {
                                    d.current_model_attributed_conversions_value = o.json && !isFinite(m.current_model_attributed_conversions_value) ? String(m.current_model_attributed_conversions_value) : m.current_model_attributed_conversions_value;
                                    if (o.oneofs)
                                        d._current_model_attributed_conversions_value = "current_model_attributed_conversions_value";
                                }
                                if (m.current_model_attributed_conversions_value_per_cost != null && m.hasOwnProperty("current_model_attributed_conversions_value_per_cost")) {
                                    d.current_model_attributed_conversions_value_per_cost = o.json && !isFinite(m.current_model_attributed_conversions_value_per_cost) ? String(m.current_model_attributed_conversions_value_per_cost) : m.current_model_attributed_conversions_value_per_cost;
                                    if (o.oneofs)
                                        d._current_model_attributed_conversions_value_per_cost = "current_model_attributed_conversions_value_per_cost";
                                }
                                if (m.engagement_rate != null && m.hasOwnProperty("engagement_rate")) {
                                    d.engagement_rate = o.json && !isFinite(m.engagement_rate) ? String(m.engagement_rate) : m.engagement_rate;
                                    if (o.oneofs)
                                        d._engagement_rate = "engagement_rate";
                                }
                                if (m.engagements != null && m.hasOwnProperty("engagements")) {
                                    if (typeof m.engagements === "number")
                                        d.engagements = o.longs === String ? String(m.engagements) : m.engagements;
                                    else
                                        d.engagements = o.longs === String ? $util.Long.prototype.toString.call(m.engagements) : o.longs === Number ? new $util.LongBits(m.engagements.low >>> 0, m.engagements.high >>> 0).toNumber() : m.engagements;
                                    if (o.oneofs)
                                        d._engagements = "engagements";
                                }
                                if (m.absolute_top_impression_percentage != null && m.hasOwnProperty("absolute_top_impression_percentage")) {
                                    d.absolute_top_impression_percentage = o.json && !isFinite(m.absolute_top_impression_percentage) ? String(m.absolute_top_impression_percentage) : m.absolute_top_impression_percentage;
                                    if (o.oneofs)
                                        d._absolute_top_impression_percentage = "absolute_top_impression_percentage";
                                }
                                if (m.active_view_cpm != null && m.hasOwnProperty("active_view_cpm")) {
                                    d.active_view_cpm = o.json && !isFinite(m.active_view_cpm) ? String(m.active_view_cpm) : m.active_view_cpm;
                                    if (o.oneofs)
                                        d._active_view_cpm = "active_view_cpm";
                                }
                                if (m.active_view_ctr != null && m.hasOwnProperty("active_view_ctr")) {
                                    d.active_view_ctr = o.json && !isFinite(m.active_view_ctr) ? String(m.active_view_ctr) : m.active_view_ctr;
                                    if (o.oneofs)
                                        d._active_view_ctr = "active_view_ctr";
                                }
                                if (m.active_view_impressions != null && m.hasOwnProperty("active_view_impressions")) {
                                    if (typeof m.active_view_impressions === "number")
                                        d.active_view_impressions = o.longs === String ? String(m.active_view_impressions) : m.active_view_impressions;
                                    else
                                        d.active_view_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.active_view_impressions) : o.longs === Number ? new $util.LongBits(m.active_view_impressions.low >>> 0, m.active_view_impressions.high >>> 0).toNumber() : m.active_view_impressions;
                                    if (o.oneofs)
                                        d._active_view_impressions = "active_view_impressions";
                                }
                                if (m.active_view_measurability != null && m.hasOwnProperty("active_view_measurability")) {
                                    d.active_view_measurability = o.json && !isFinite(m.active_view_measurability) ? String(m.active_view_measurability) : m.active_view_measurability;
                                    if (o.oneofs)
                                        d._active_view_measurability = "active_view_measurability";
                                }
                                if (m.active_view_measurable_cost_micros != null && m.hasOwnProperty("active_view_measurable_cost_micros")) {
                                    if (typeof m.active_view_measurable_cost_micros === "number")
                                        d.active_view_measurable_cost_micros = o.longs === String ? String(m.active_view_measurable_cost_micros) : m.active_view_measurable_cost_micros;
                                    else
                                        d.active_view_measurable_cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.active_view_measurable_cost_micros) : o.longs === Number ? new $util.LongBits(m.active_view_measurable_cost_micros.low >>> 0, m.active_view_measurable_cost_micros.high >>> 0).toNumber() : m.active_view_measurable_cost_micros;
                                    if (o.oneofs)
                                        d._active_view_measurable_cost_micros = "active_view_measurable_cost_micros";
                                }
                                if (m.active_view_measurable_impressions != null && m.hasOwnProperty("active_view_measurable_impressions")) {
                                    if (typeof m.active_view_measurable_impressions === "number")
                                        d.active_view_measurable_impressions = o.longs === String ? String(m.active_view_measurable_impressions) : m.active_view_measurable_impressions;
                                    else
                                        d.active_view_measurable_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.active_view_measurable_impressions) : o.longs === Number ? new $util.LongBits(m.active_view_measurable_impressions.low >>> 0, m.active_view_measurable_impressions.high >>> 0).toNumber() : m.active_view_measurable_impressions;
                                    if (o.oneofs)
                                        d._active_view_measurable_impressions = "active_view_measurable_impressions";
                                }
                                if (m.active_view_viewability != null && m.hasOwnProperty("active_view_viewability")) {
                                    d.active_view_viewability = o.json && !isFinite(m.active_view_viewability) ? String(m.active_view_viewability) : m.active_view_viewability;
                                    if (o.oneofs)
                                        d._active_view_viewability = "active_view_viewability";
                                }
                                if (m.all_conversions_from_interactions_rate != null && m.hasOwnProperty("all_conversions_from_interactions_rate")) {
                                    d.all_conversions_from_interactions_rate = o.json && !isFinite(m.all_conversions_from_interactions_rate) ? String(m.all_conversions_from_interactions_rate) : m.all_conversions_from_interactions_rate;
                                    if (o.oneofs)
                                        d._all_conversions_from_interactions_rate = "all_conversions_from_interactions_rate";
                                }
                                if (m.all_conversions_value != null && m.hasOwnProperty("all_conversions_value")) {
                                    d.all_conversions_value = o.json && !isFinite(m.all_conversions_value) ? String(m.all_conversions_value) : m.all_conversions_value;
                                    if (o.oneofs)
                                        d._all_conversions_value = "all_conversions_value";
                                }
                                if (m.all_conversions != null && m.hasOwnProperty("all_conversions")) {
                                    d.all_conversions = o.json && !isFinite(m.all_conversions) ? String(m.all_conversions) : m.all_conversions;
                                    if (o.oneofs)
                                        d._all_conversions = "all_conversions";
                                }
                                if (m.all_conversions_value_per_cost != null && m.hasOwnProperty("all_conversions_value_per_cost")) {
                                    d.all_conversions_value_per_cost = o.json && !isFinite(m.all_conversions_value_per_cost) ? String(m.all_conversions_value_per_cost) : m.all_conversions_value_per_cost;
                                    if (o.oneofs)
                                        d._all_conversions_value_per_cost = "all_conversions_value_per_cost";
                                }
                                if (m.all_conversions_from_click_to_call != null && m.hasOwnProperty("all_conversions_from_click_to_call")) {
                                    d.all_conversions_from_click_to_call = o.json && !isFinite(m.all_conversions_from_click_to_call) ? String(m.all_conversions_from_click_to_call) : m.all_conversions_from_click_to_call;
                                    if (o.oneofs)
                                        d._all_conversions_from_click_to_call = "all_conversions_from_click_to_call";
                                }
                                if (m.all_conversions_from_directions != null && m.hasOwnProperty("all_conversions_from_directions")) {
                                    d.all_conversions_from_directions = o.json && !isFinite(m.all_conversions_from_directions) ? String(m.all_conversions_from_directions) : m.all_conversions_from_directions;
                                    if (o.oneofs)
                                        d._all_conversions_from_directions = "all_conversions_from_directions";
                                }
                                if (m.all_conversions_from_interactions_value_per_interaction != null && m.hasOwnProperty("all_conversions_from_interactions_value_per_interaction")) {
                                    d.all_conversions_from_interactions_value_per_interaction = o.json && !isFinite(m.all_conversions_from_interactions_value_per_interaction) ? String(m.all_conversions_from_interactions_value_per_interaction) : m.all_conversions_from_interactions_value_per_interaction;
                                    if (o.oneofs)
                                        d._all_conversions_from_interactions_value_per_interaction = "all_conversions_from_interactions_value_per_interaction";
                                }
                                if (m.all_conversions_from_menu != null && m.hasOwnProperty("all_conversions_from_menu")) {
                                    d.all_conversions_from_menu = o.json && !isFinite(m.all_conversions_from_menu) ? String(m.all_conversions_from_menu) : m.all_conversions_from_menu;
                                    if (o.oneofs)
                                        d._all_conversions_from_menu = "all_conversions_from_menu";
                                }
                                if (m.all_conversions_from_order != null && m.hasOwnProperty("all_conversions_from_order")) {
                                    d.all_conversions_from_order = o.json && !isFinite(m.all_conversions_from_order) ? String(m.all_conversions_from_order) : m.all_conversions_from_order;
                                    if (o.oneofs)
                                        d._all_conversions_from_order = "all_conversions_from_order";
                                }
                                if (m.all_conversions_from_other_engagement != null && m.hasOwnProperty("all_conversions_from_other_engagement")) {
                                    d.all_conversions_from_other_engagement = o.json && !isFinite(m.all_conversions_from_other_engagement) ? String(m.all_conversions_from_other_engagement) : m.all_conversions_from_other_engagement;
                                    if (o.oneofs)
                                        d._all_conversions_from_other_engagement = "all_conversions_from_other_engagement";
                                }
                                if (m.all_conversions_from_store_visit != null && m.hasOwnProperty("all_conversions_from_store_visit")) {
                                    d.all_conversions_from_store_visit = o.json && !isFinite(m.all_conversions_from_store_visit) ? String(m.all_conversions_from_store_visit) : m.all_conversions_from_store_visit;
                                    if (o.oneofs)
                                        d._all_conversions_from_store_visit = "all_conversions_from_store_visit";
                                }
                                if (m.all_conversions_from_store_website != null && m.hasOwnProperty("all_conversions_from_store_website")) {
                                    d.all_conversions_from_store_website = o.json && !isFinite(m.all_conversions_from_store_website) ? String(m.all_conversions_from_store_website) : m.all_conversions_from_store_website;
                                    if (o.oneofs)
                                        d._all_conversions_from_store_website = "all_conversions_from_store_website";
                                }
                                if (m.average_cost != null && m.hasOwnProperty("average_cost")) {
                                    d.average_cost = o.json && !isFinite(m.average_cost) ? String(m.average_cost) : m.average_cost;
                                    if (o.oneofs)
                                        d._average_cost = "average_cost";
                                }
                                if (m.average_cpc != null && m.hasOwnProperty("average_cpc")) {
                                    d.average_cpc = o.json && !isFinite(m.average_cpc) ? String(m.average_cpc) : m.average_cpc;
                                    if (o.oneofs)
                                        d._average_cpc = "average_cpc";
                                }
                                if (m.average_cpe != null && m.hasOwnProperty("average_cpe")) {
                                    d.average_cpe = o.json && !isFinite(m.average_cpe) ? String(m.average_cpe) : m.average_cpe;
                                    if (o.oneofs)
                                        d._average_cpe = "average_cpe";
                                }
                                if (m.average_cpm != null && m.hasOwnProperty("average_cpm")) {
                                    d.average_cpm = o.json && !isFinite(m.average_cpm) ? String(m.average_cpm) : m.average_cpm;
                                    if (o.oneofs)
                                        d._average_cpm = "average_cpm";
                                }
                                if (m.average_cpv != null && m.hasOwnProperty("average_cpv")) {
                                    d.average_cpv = o.json && !isFinite(m.average_cpv) ? String(m.average_cpv) : m.average_cpv;
                                    if (o.oneofs)
                                        d._average_cpv = "average_cpv";
                                }
                                if (m.average_page_views != null && m.hasOwnProperty("average_page_views")) {
                                    d.average_page_views = o.json && !isFinite(m.average_page_views) ? String(m.average_page_views) : m.average_page_views;
                                    if (o.oneofs)
                                        d._average_page_views = "average_page_views";
                                }
                                if (m.average_time_on_site != null && m.hasOwnProperty("average_time_on_site")) {
                                    d.average_time_on_site = o.json && !isFinite(m.average_time_on_site) ? String(m.average_time_on_site) : m.average_time_on_site;
                                    if (o.oneofs)
                                        d._average_time_on_site = "average_time_on_site";
                                }
                                if (m.benchmark_average_max_cpc != null && m.hasOwnProperty("benchmark_average_max_cpc")) {
                                    d.benchmark_average_max_cpc = o.json && !isFinite(m.benchmark_average_max_cpc) ? String(m.benchmark_average_max_cpc) : m.benchmark_average_max_cpc;
                                    if (o.oneofs)
                                        d._benchmark_average_max_cpc = "benchmark_average_max_cpc";
                                }
                                if (m.benchmark_ctr != null && m.hasOwnProperty("benchmark_ctr")) {
                                    d.benchmark_ctr = o.json && !isFinite(m.benchmark_ctr) ? String(m.benchmark_ctr) : m.benchmark_ctr;
                                    if (o.oneofs)
                                        d._benchmark_ctr = "benchmark_ctr";
                                }
                                if (m.bounce_rate != null && m.hasOwnProperty("bounce_rate")) {
                                    d.bounce_rate = o.json && !isFinite(m.bounce_rate) ? String(m.bounce_rate) : m.bounce_rate;
                                    if (o.oneofs)
                                        d._bounce_rate = "bounce_rate";
                                }
                                if (m.hotel_average_lead_value_micros != null && m.hasOwnProperty("hotel_average_lead_value_micros")) {
                                    d.hotel_average_lead_value_micros = o.json && !isFinite(m.hotel_average_lead_value_micros) ? String(m.hotel_average_lead_value_micros) : m.hotel_average_lead_value_micros;
                                    if (o.oneofs)
                                        d._hotel_average_lead_value_micros = "hotel_average_lead_value_micros";
                                }
                                if (m.hotel_price_difference_percentage != null && m.hasOwnProperty("hotel_price_difference_percentage")) {
                                    d.hotel_price_difference_percentage = o.json && !isFinite(m.hotel_price_difference_percentage) ? String(m.hotel_price_difference_percentage) : m.hotel_price_difference_percentage;
                                    if (o.oneofs)
                                        d._hotel_price_difference_percentage = "hotel_price_difference_percentage";
                                }
                                if (m.hotel_eligible_impressions != null && m.hasOwnProperty("hotel_eligible_impressions")) {
                                    if (typeof m.hotel_eligible_impressions === "number")
                                        d.hotel_eligible_impressions = o.longs === String ? String(m.hotel_eligible_impressions) : m.hotel_eligible_impressions;
                                    else
                                        d.hotel_eligible_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.hotel_eligible_impressions) : o.longs === Number ? new $util.LongBits(m.hotel_eligible_impressions.low >>> 0, m.hotel_eligible_impressions.high >>> 0).toNumber() : m.hotel_eligible_impressions;
                                    if (o.oneofs)
                                        d._hotel_eligible_impressions = "hotel_eligible_impressions";
                                }
                                if (m.historical_quality_score != null && m.hasOwnProperty("historical_quality_score")) {
                                    if (typeof m.historical_quality_score === "number")
                                        d.historical_quality_score = o.longs === String ? String(m.historical_quality_score) : m.historical_quality_score;
                                    else
                                        d.historical_quality_score = o.longs === String ? $util.Long.prototype.toString.call(m.historical_quality_score) : o.longs === Number ? new $util.LongBits(m.historical_quality_score.low >>> 0, m.historical_quality_score.high >>> 0).toNumber() : m.historical_quality_score;
                                    if (o.oneofs)
                                        d._historical_quality_score = "historical_quality_score";
                                }
                                if (m.gmail_forwards != null && m.hasOwnProperty("gmail_forwards")) {
                                    if (typeof m.gmail_forwards === "number")
                                        d.gmail_forwards = o.longs === String ? String(m.gmail_forwards) : m.gmail_forwards;
                                    else
                                        d.gmail_forwards = o.longs === String ? $util.Long.prototype.toString.call(m.gmail_forwards) : o.longs === Number ? new $util.LongBits(m.gmail_forwards.low >>> 0, m.gmail_forwards.high >>> 0).toNumber() : m.gmail_forwards;
                                    if (o.oneofs)
                                        d._gmail_forwards = "gmail_forwards";
                                }
                                if (m.gmail_saves != null && m.hasOwnProperty("gmail_saves")) {
                                    if (typeof m.gmail_saves === "number")
                                        d.gmail_saves = o.longs === String ? String(m.gmail_saves) : m.gmail_saves;
                                    else
                                        d.gmail_saves = o.longs === String ? $util.Long.prototype.toString.call(m.gmail_saves) : o.longs === Number ? new $util.LongBits(m.gmail_saves.low >>> 0, m.gmail_saves.high >>> 0).toNumber() : m.gmail_saves;
                                    if (o.oneofs)
                                        d._gmail_saves = "gmail_saves";
                                }
                                if (m.gmail_secondary_clicks != null && m.hasOwnProperty("gmail_secondary_clicks")) {
                                    if (typeof m.gmail_secondary_clicks === "number")
                                        d.gmail_secondary_clicks = o.longs === String ? String(m.gmail_secondary_clicks) : m.gmail_secondary_clicks;
                                    else
                                        d.gmail_secondary_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.gmail_secondary_clicks) : o.longs === Number ? new $util.LongBits(m.gmail_secondary_clicks.low >>> 0, m.gmail_secondary_clicks.high >>> 0).toNumber() : m.gmail_secondary_clicks;
                                    if (o.oneofs)
                                        d._gmail_secondary_clicks = "gmail_secondary_clicks";
                                }
                                if (m.impressions_from_store_reach != null && m.hasOwnProperty("impressions_from_store_reach")) {
                                    if (typeof m.impressions_from_store_reach === "number")
                                        d.impressions_from_store_reach = o.longs === String ? String(m.impressions_from_store_reach) : m.impressions_from_store_reach;
                                    else
                                        d.impressions_from_store_reach = o.longs === String ? $util.Long.prototype.toString.call(m.impressions_from_store_reach) : o.longs === Number ? new $util.LongBits(m.impressions_from_store_reach.low >>> 0, m.impressions_from_store_reach.high >>> 0).toNumber() : m.impressions_from_store_reach;
                                    if (o.oneofs)
                                        d._impressions_from_store_reach = "impressions_from_store_reach";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.interaction_rate != null && m.hasOwnProperty("interaction_rate")) {
                                    d.interaction_rate = o.json && !isFinite(m.interaction_rate) ? String(m.interaction_rate) : m.interaction_rate;
                                    if (o.oneofs)
                                        d._interaction_rate = "interaction_rate";
                                }
                                if (m.interactions != null && m.hasOwnProperty("interactions")) {
                                    if (typeof m.interactions === "number")
                                        d.interactions = o.longs === String ? String(m.interactions) : m.interactions;
                                    else
                                        d.interactions = o.longs === String ? $util.Long.prototype.toString.call(m.interactions) : o.longs === Number ? new $util.LongBits(m.interactions.low >>> 0, m.interactions.high >>> 0).toNumber() : m.interactions;
                                    if (o.oneofs)
                                        d._interactions = "interactions";
                                }
                                if (m.invalid_click_rate != null && m.hasOwnProperty("invalid_click_rate")) {
                                    d.invalid_click_rate = o.json && !isFinite(m.invalid_click_rate) ? String(m.invalid_click_rate) : m.invalid_click_rate;
                                    if (o.oneofs)
                                        d._invalid_click_rate = "invalid_click_rate";
                                }
                                if (m.invalid_clicks != null && m.hasOwnProperty("invalid_clicks")) {
                                    if (typeof m.invalid_clicks === "number")
                                        d.invalid_clicks = o.longs === String ? String(m.invalid_clicks) : m.invalid_clicks;
                                    else
                                        d.invalid_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.invalid_clicks) : o.longs === Number ? new $util.LongBits(m.invalid_clicks.low >>> 0, m.invalid_clicks.high >>> 0).toNumber() : m.invalid_clicks;
                                    if (o.oneofs)
                                        d._invalid_clicks = "invalid_clicks";
                                }
                                if (m.message_chats != null && m.hasOwnProperty("message_chats")) {
                                    if (typeof m.message_chats === "number")
                                        d.message_chats = o.longs === String ? String(m.message_chats) : m.message_chats;
                                    else
                                        d.message_chats = o.longs === String ? $util.Long.prototype.toString.call(m.message_chats) : o.longs === Number ? new $util.LongBits(m.message_chats.low >>> 0, m.message_chats.high >>> 0).toNumber() : m.message_chats;
                                    if (o.oneofs)
                                        d._message_chats = "message_chats";
                                }
                                if (m.message_impressions != null && m.hasOwnProperty("message_impressions")) {
                                    if (typeof m.message_impressions === "number")
                                        d.message_impressions = o.longs === String ? String(m.message_impressions) : m.message_impressions;
                                    else
                                        d.message_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.message_impressions) : o.longs === Number ? new $util.LongBits(m.message_impressions.low >>> 0, m.message_impressions.high >>> 0).toNumber() : m.message_impressions;
                                    if (o.oneofs)
                                        d._message_impressions = "message_impressions";
                                }
                                if (m.message_chat_rate != null && m.hasOwnProperty("message_chat_rate")) {
                                    d.message_chat_rate = o.json && !isFinite(m.message_chat_rate) ? String(m.message_chat_rate) : m.message_chat_rate;
                                    if (o.oneofs)
                                        d._message_chat_rate = "message_chat_rate";
                                }
                                if (m.mobile_friendly_clicks_percentage != null && m.hasOwnProperty("mobile_friendly_clicks_percentage")) {
                                    d.mobile_friendly_clicks_percentage = o.json && !isFinite(m.mobile_friendly_clicks_percentage) ? String(m.mobile_friendly_clicks_percentage) : m.mobile_friendly_clicks_percentage;
                                    if (o.oneofs)
                                        d._mobile_friendly_clicks_percentage = "mobile_friendly_clicks_percentage";
                                }
                                if (m.organic_clicks != null && m.hasOwnProperty("organic_clicks")) {
                                    if (typeof m.organic_clicks === "number")
                                        d.organic_clicks = o.longs === String ? String(m.organic_clicks) : m.organic_clicks;
                                    else
                                        d.organic_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.organic_clicks) : o.longs === Number ? new $util.LongBits(m.organic_clicks.low >>> 0, m.organic_clicks.high >>> 0).toNumber() : m.organic_clicks;
                                    if (o.oneofs)
                                        d._organic_clicks = "organic_clicks";
                                }
                                if (m.organic_clicks_per_query != null && m.hasOwnProperty("organic_clicks_per_query")) {
                                    d.organic_clicks_per_query = o.json && !isFinite(m.organic_clicks_per_query) ? String(m.organic_clicks_per_query) : m.organic_clicks_per_query;
                                    if (o.oneofs)
                                        d._organic_clicks_per_query = "organic_clicks_per_query";
                                }
                                if (m.organic_impressions != null && m.hasOwnProperty("organic_impressions")) {
                                    if (typeof m.organic_impressions === "number")
                                        d.organic_impressions = o.longs === String ? String(m.organic_impressions) : m.organic_impressions;
                                    else
                                        d.organic_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.organic_impressions) : o.longs === Number ? new $util.LongBits(m.organic_impressions.low >>> 0, m.organic_impressions.high >>> 0).toNumber() : m.organic_impressions;
                                    if (o.oneofs)
                                        d._organic_impressions = "organic_impressions";
                                }
                                if (m.organic_impressions_per_query != null && m.hasOwnProperty("organic_impressions_per_query")) {
                                    d.organic_impressions_per_query = o.json && !isFinite(m.organic_impressions_per_query) ? String(m.organic_impressions_per_query) : m.organic_impressions_per_query;
                                    if (o.oneofs)
                                        d._organic_impressions_per_query = "organic_impressions_per_query";
                                }
                                if (m.organic_queries != null && m.hasOwnProperty("organic_queries")) {
                                    if (typeof m.organic_queries === "number")
                                        d.organic_queries = o.longs === String ? String(m.organic_queries) : m.organic_queries;
                                    else
                                        d.organic_queries = o.longs === String ? $util.Long.prototype.toString.call(m.organic_queries) : o.longs === Number ? new $util.LongBits(m.organic_queries.low >>> 0, m.organic_queries.high >>> 0).toNumber() : m.organic_queries;
                                    if (o.oneofs)
                                        d._organic_queries = "organic_queries";
                                }
                                if (m.percent_new_visitors != null && m.hasOwnProperty("percent_new_visitors")) {
                                    d.percent_new_visitors = o.json && !isFinite(m.percent_new_visitors) ? String(m.percent_new_visitors) : m.percent_new_visitors;
                                    if (o.oneofs)
                                        d._percent_new_visitors = "percent_new_visitors";
                                }
                                if (m.phone_calls != null && m.hasOwnProperty("phone_calls")) {
                                    if (typeof m.phone_calls === "number")
                                        d.phone_calls = o.longs === String ? String(m.phone_calls) : m.phone_calls;
                                    else
                                        d.phone_calls = o.longs === String ? $util.Long.prototype.toString.call(m.phone_calls) : o.longs === Number ? new $util.LongBits(m.phone_calls.low >>> 0, m.phone_calls.high >>> 0).toNumber() : m.phone_calls;
                                    if (o.oneofs)
                                        d._phone_calls = "phone_calls";
                                }
                                if (m.phone_impressions != null && m.hasOwnProperty("phone_impressions")) {
                                    if (typeof m.phone_impressions === "number")
                                        d.phone_impressions = o.longs === String ? String(m.phone_impressions) : m.phone_impressions;
                                    else
                                        d.phone_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.phone_impressions) : o.longs === Number ? new $util.LongBits(m.phone_impressions.low >>> 0, m.phone_impressions.high >>> 0).toNumber() : m.phone_impressions;
                                    if (o.oneofs)
                                        d._phone_impressions = "phone_impressions";
                                }
                                if (m.phone_through_rate != null && m.hasOwnProperty("phone_through_rate")) {
                                    d.phone_through_rate = o.json && !isFinite(m.phone_through_rate) ? String(m.phone_through_rate) : m.phone_through_rate;
                                    if (o.oneofs)
                                        d._phone_through_rate = "phone_through_rate";
                                }
                                if (m.relative_ctr != null && m.hasOwnProperty("relative_ctr")) {
                                    d.relative_ctr = o.json && !isFinite(m.relative_ctr) ? String(m.relative_ctr) : m.relative_ctr;
                                    if (o.oneofs)
                                        d._relative_ctr = "relative_ctr";
                                }
                                if (m.all_conversions_value_by_conversion_date != null && m.hasOwnProperty("all_conversions_value_by_conversion_date")) {
                                    d.all_conversions_value_by_conversion_date = o.json && !isFinite(m.all_conversions_value_by_conversion_date) ? String(m.all_conversions_value_by_conversion_date) : m.all_conversions_value_by_conversion_date;
                                }
                                if (m.all_conversions_by_conversion_date != null && m.hasOwnProperty("all_conversions_by_conversion_date")) {
                                    d.all_conversions_by_conversion_date = o.json && !isFinite(m.all_conversions_by_conversion_date) ? String(m.all_conversions_by_conversion_date) : m.all_conversions_by_conversion_date;
                                }
                                if (m.conversions_value_by_conversion_date != null && m.hasOwnProperty("conversions_value_by_conversion_date")) {
                                    d.conversions_value_by_conversion_date = o.json && !isFinite(m.conversions_value_by_conversion_date) ? String(m.conversions_value_by_conversion_date) : m.conversions_value_by_conversion_date;
                                }
                                if (m.conversions_by_conversion_date != null && m.hasOwnProperty("conversions_by_conversion_date")) {
                                    d.conversions_by_conversion_date = o.json && !isFinite(m.conversions_by_conversion_date) ? String(m.conversions_by_conversion_date) : m.conversions_by_conversion_date;
                                }
                                if (m.value_per_all_conversions_by_conversion_date != null && m.hasOwnProperty("value_per_all_conversions_by_conversion_date")) {
                                    d.value_per_all_conversions_by_conversion_date = o.json && !isFinite(m.value_per_all_conversions_by_conversion_date) ? String(m.value_per_all_conversions_by_conversion_date) : m.value_per_all_conversions_by_conversion_date;
                                    if (o.oneofs)
                                        d._value_per_all_conversions_by_conversion_date = "value_per_all_conversions_by_conversion_date";
                                }
                                if (m.value_per_conversions_by_conversion_date != null && m.hasOwnProperty("value_per_conversions_by_conversion_date")) {
                                    d.value_per_conversions_by_conversion_date = o.json && !isFinite(m.value_per_conversions_by_conversion_date) ? String(m.value_per_conversions_by_conversion_date) : m.value_per_conversions_by_conversion_date;
                                    if (o.oneofs)
                                        d._value_per_conversions_by_conversion_date = "value_per_conversions_by_conversion_date";
                                }
                                if (m.sk_ad_network_conversions != null && m.hasOwnProperty("sk_ad_network_conversions")) {
                                    if (typeof m.sk_ad_network_conversions === "number")
                                        d.sk_ad_network_conversions = o.longs === String ? String(m.sk_ad_network_conversions) : m.sk_ad_network_conversions;
                                    else
                                        d.sk_ad_network_conversions = o.longs === String ? $util.Long.prototype.toString.call(m.sk_ad_network_conversions) : o.longs === Number ? new $util.LongBits(m.sk_ad_network_conversions.low >>> 0, m.sk_ad_network_conversions.high >>> 0).toNumber() : m.sk_ad_network_conversions;
                                }
                                if (m.optimization_score_uplift != null && m.hasOwnProperty("optimization_score_uplift")) {
                                    d.optimization_score_uplift = o.json && !isFinite(m.optimization_score_uplift) ? String(m.optimization_score_uplift) : m.optimization_score_uplift;
                                    if (o.oneofs)
                                        d._optimization_score_uplift = "optimization_score_uplift";
                                }
                                if (m.optimization_score_url != null && m.hasOwnProperty("optimization_score_url")) {
                                    d.optimization_score_url = m.optimization_score_url;
                                    if (o.oneofs)
                                        d._optimization_score_url = "optimization_score_url";
                                }
                                if (m.biddable_app_install_conversions != null && m.hasOwnProperty("biddable_app_install_conversions")) {
                                    d.biddable_app_install_conversions = o.json && !isFinite(m.biddable_app_install_conversions) ? String(m.biddable_app_install_conversions) : m.biddable_app_install_conversions;
                                    if (o.oneofs)
                                        d._biddable_app_install_conversions = "biddable_app_install_conversions";
                                }
                                if (m.biddable_app_post_install_conversions != null && m.hasOwnProperty("biddable_app_post_install_conversions")) {
                                    d.biddable_app_post_install_conversions = o.json && !isFinite(m.biddable_app_post_install_conversions) ? String(m.biddable_app_post_install_conversions) : m.biddable_app_post_install_conversions;
                                    if (o.oneofs)
                                        d._biddable_app_post_install_conversions = "biddable_app_post_install_conversions";
                                }
                                if (m.hotel_commission_rate_micros != null && m.hasOwnProperty("hotel_commission_rate_micros")) {
                                    if (typeof m.hotel_commission_rate_micros === "number")
                                        d.hotel_commission_rate_micros = o.longs === String ? String(m.hotel_commission_rate_micros) : m.hotel_commission_rate_micros;
                                    else
                                        d.hotel_commission_rate_micros = o.longs === String ? $util.Long.prototype.toString.call(m.hotel_commission_rate_micros) : o.longs === Number ? new $util.LongBits(m.hotel_commission_rate_micros.low >>> 0, m.hotel_commission_rate_micros.high >>> 0).toNumber() : m.hotel_commission_rate_micros;
                                    if (o.oneofs)
                                        d._hotel_commission_rate_micros = "hotel_commission_rate_micros";
                                }
                                if (m.hotel_expected_commission_cost != null && m.hasOwnProperty("hotel_expected_commission_cost")) {
                                    d.hotel_expected_commission_cost = o.json && !isFinite(m.hotel_expected_commission_cost) ? String(m.hotel_expected_commission_cost) : m.hotel_expected_commission_cost;
                                    if (o.oneofs)
                                        d._hotel_expected_commission_cost = "hotel_expected_commission_cost";
                                }
                                if (m.auction_insight_search_absolute_top_impression_percentage != null && m.hasOwnProperty("auction_insight_search_absolute_top_impression_percentage")) {
                                    d.auction_insight_search_absolute_top_impression_percentage = o.json && !isFinite(m.auction_insight_search_absolute_top_impression_percentage) ? String(m.auction_insight_search_absolute_top_impression_percentage) : m.auction_insight_search_absolute_top_impression_percentage;
                                    if (o.oneofs)
                                        d._auction_insight_search_absolute_top_impression_percentage = "auction_insight_search_absolute_top_impression_percentage";
                                }
                                if (m.auction_insight_search_impression_share != null && m.hasOwnProperty("auction_insight_search_impression_share")) {
                                    d.auction_insight_search_impression_share = o.json && !isFinite(m.auction_insight_search_impression_share) ? String(m.auction_insight_search_impression_share) : m.auction_insight_search_impression_share;
                                    if (o.oneofs)
                                        d._auction_insight_search_impression_share = "auction_insight_search_impression_share";
                                }
                                if (m.auction_insight_search_outranking_share != null && m.hasOwnProperty("auction_insight_search_outranking_share")) {
                                    d.auction_insight_search_outranking_share = o.json && !isFinite(m.auction_insight_search_outranking_share) ? String(m.auction_insight_search_outranking_share) : m.auction_insight_search_outranking_share;
                                    if (o.oneofs)
                                        d._auction_insight_search_outranking_share = "auction_insight_search_outranking_share";
                                }
                                if (m.auction_insight_search_overlap_rate != null && m.hasOwnProperty("auction_insight_search_overlap_rate")) {
                                    d.auction_insight_search_overlap_rate = o.json && !isFinite(m.auction_insight_search_overlap_rate) ? String(m.auction_insight_search_overlap_rate) : m.auction_insight_search_overlap_rate;
                                    if (o.oneofs)
                                        d._auction_insight_search_overlap_rate = "auction_insight_search_overlap_rate";
                                }
                                if (m.auction_insight_search_position_above_rate != null && m.hasOwnProperty("auction_insight_search_position_above_rate")) {
                                    d.auction_insight_search_position_above_rate = o.json && !isFinite(m.auction_insight_search_position_above_rate) ? String(m.auction_insight_search_position_above_rate) : m.auction_insight_search_position_above_rate;
                                    if (o.oneofs)
                                        d._auction_insight_search_position_above_rate = "auction_insight_search_position_above_rate";
                                }
                                if (m.auction_insight_search_top_impression_percentage != null && m.hasOwnProperty("auction_insight_search_top_impression_percentage")) {
                                    d.auction_insight_search_top_impression_percentage = o.json && !isFinite(m.auction_insight_search_top_impression_percentage) ? String(m.auction_insight_search_top_impression_percentage) : m.auction_insight_search_top_impression_percentage;
                                    if (o.oneofs)
                                        d._auction_insight_search_top_impression_percentage = "auction_insight_search_top_impression_percentage";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Metrics to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.Metrics
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Metrics.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Metrics;
                        })();
    
                        common.OfflineUserAddressInfo = (function() {
    
                            /**
                             * Properties of an OfflineUserAddressInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IOfflineUserAddressInfo
                             * @property {string|null} [hashed_first_name] OfflineUserAddressInfo hashed_first_name
                             * @property {string|null} [hashed_last_name] OfflineUserAddressInfo hashed_last_name
                             * @property {string|null} [city] OfflineUserAddressInfo city
                             * @property {string|null} [state] OfflineUserAddressInfo state
                             * @property {string|null} [country_code] OfflineUserAddressInfo country_code
                             * @property {string|null} [postal_code] OfflineUserAddressInfo postal_code
                             * @property {string|null} [hashed_street_address] OfflineUserAddressInfo hashed_street_address
                             */
    
                            /**
                             * Constructs a new OfflineUserAddressInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an OfflineUserAddressInfo.
                             * @implements IOfflineUserAddressInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IOfflineUserAddressInfo=} [p] Properties to set
                             */
                            function OfflineUserAddressInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * OfflineUserAddressInfo hashed_first_name.
                             * @member {string|null|undefined} hashed_first_name
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            OfflineUserAddressInfo.prototype.hashed_first_name = null;
    
                            /**
                             * OfflineUserAddressInfo hashed_last_name.
                             * @member {string|null|undefined} hashed_last_name
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            OfflineUserAddressInfo.prototype.hashed_last_name = null;
    
                            /**
                             * OfflineUserAddressInfo city.
                             * @member {string|null|undefined} city
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            OfflineUserAddressInfo.prototype.city = null;
    
                            /**
                             * OfflineUserAddressInfo state.
                             * @member {string|null|undefined} state
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            OfflineUserAddressInfo.prototype.state = null;
    
                            /**
                             * OfflineUserAddressInfo country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            OfflineUserAddressInfo.prototype.country_code = null;
    
                            /**
                             * OfflineUserAddressInfo postal_code.
                             * @member {string|null|undefined} postal_code
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            OfflineUserAddressInfo.prototype.postal_code = null;
    
                            /**
                             * OfflineUserAddressInfo hashed_street_address.
                             * @member {string|null|undefined} hashed_street_address
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            OfflineUserAddressInfo.prototype.hashed_street_address = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * OfflineUserAddressInfo _hashed_first_name.
                             * @member {"hashed_first_name"|undefined} _hashed_first_name
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            Object.defineProperty(OfflineUserAddressInfo.prototype, "_hashed_first_name", {
                                get: $util.oneOfGetter($oneOfFields = ["hashed_first_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserAddressInfo _hashed_last_name.
                             * @member {"hashed_last_name"|undefined} _hashed_last_name
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            Object.defineProperty(OfflineUserAddressInfo.prototype, "_hashed_last_name", {
                                get: $util.oneOfGetter($oneOfFields = ["hashed_last_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserAddressInfo _city.
                             * @member {"city"|undefined} _city
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            Object.defineProperty(OfflineUserAddressInfo.prototype, "_city", {
                                get: $util.oneOfGetter($oneOfFields = ["city"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserAddressInfo _state.
                             * @member {"state"|undefined} _state
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            Object.defineProperty(OfflineUserAddressInfo.prototype, "_state", {
                                get: $util.oneOfGetter($oneOfFields = ["state"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserAddressInfo _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            Object.defineProperty(OfflineUserAddressInfo.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserAddressInfo _postal_code.
                             * @member {"postal_code"|undefined} _postal_code
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            Object.defineProperty(OfflineUserAddressInfo.prototype, "_postal_code", {
                                get: $util.oneOfGetter($oneOfFields = ["postal_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserAddressInfo _hashed_street_address.
                             * @member {"hashed_street_address"|undefined} _hashed_street_address
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             */
                            Object.defineProperty(OfflineUserAddressInfo.prototype, "_hashed_street_address", {
                                get: $util.oneOfGetter($oneOfFields = ["hashed_street_address"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new OfflineUserAddressInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IOfflineUserAddressInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.OfflineUserAddressInfo} OfflineUserAddressInfo instance
                             */
                            OfflineUserAddressInfo.create = function create(properties) {
                                return new OfflineUserAddressInfo(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserAddressInfo message. Does not implicitly {@link google.ads.googleads.v11.common.OfflineUserAddressInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IOfflineUserAddressInfo} m OfflineUserAddressInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserAddressInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.hashed_first_name != null && Object.hasOwnProperty.call(m, "hashed_first_name"))
                                    w.uint32(58).string(m.hashed_first_name);
                                if (m.hashed_last_name != null && Object.hasOwnProperty.call(m, "hashed_last_name"))
                                    w.uint32(66).string(m.hashed_last_name);
                                if (m.city != null && Object.hasOwnProperty.call(m, "city"))
                                    w.uint32(74).string(m.city);
                                if (m.state != null && Object.hasOwnProperty.call(m, "state"))
                                    w.uint32(82).string(m.state);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(90).string(m.country_code);
                                if (m.postal_code != null && Object.hasOwnProperty.call(m, "postal_code"))
                                    w.uint32(98).string(m.postal_code);
                                if (m.hashed_street_address != null && Object.hasOwnProperty.call(m, "hashed_street_address"))
                                    w.uint32(106).string(m.hashed_street_address);
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserAddressInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.OfflineUserAddressInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IOfflineUserAddressInfo} message OfflineUserAddressInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserAddressInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserAddressInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.OfflineUserAddressInfo} OfflineUserAddressInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserAddressInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.OfflineUserAddressInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 7:
                                        m.hashed_first_name = r.string();
                                        break;
                                    case 8:
                                        m.hashed_last_name = r.string();
                                        break;
                                    case 9:
                                        m.city = r.string();
                                        break;
                                    case 10:
                                        m.state = r.string();
                                        break;
                                    case 11:
                                        m.country_code = r.string();
                                        break;
                                    case 12:
                                        m.postal_code = r.string();
                                        break;
                                    case 13:
                                        m.hashed_street_address = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserAddressInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.OfflineUserAddressInfo} OfflineUserAddressInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserAddressInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserAddressInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserAddressInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.hashed_first_name != null && m.hasOwnProperty("hashed_first_name")) {
                                    p._hashed_first_name = 1;
                                    if (!$util.isString(m.hashed_first_name))
                                        return "hashed_first_name: string expected";
                                }
                                if (m.hashed_last_name != null && m.hasOwnProperty("hashed_last_name")) {
                                    p._hashed_last_name = 1;
                                    if (!$util.isString(m.hashed_last_name))
                                        return "hashed_last_name: string expected";
                                }
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    p._city = 1;
                                    if (!$util.isString(m.city))
                                        return "city: string expected";
                                }
                                if (m.state != null && m.hasOwnProperty("state")) {
                                    p._state = 1;
                                    if (!$util.isString(m.state))
                                        return "state: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    p._postal_code = 1;
                                    if (!$util.isString(m.postal_code))
                                        return "postal_code: string expected";
                                }
                                if (m.hashed_street_address != null && m.hasOwnProperty("hashed_street_address")) {
                                    p._hashed_street_address = 1;
                                    if (!$util.isString(m.hashed_street_address))
                                        return "hashed_street_address: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserAddressInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.OfflineUserAddressInfo} OfflineUserAddressInfo
                             */
                            OfflineUserAddressInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.OfflineUserAddressInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.OfflineUserAddressInfo();
                                if (d.hashed_first_name != null) {
                                    m.hashed_first_name = String(d.hashed_first_name);
                                }
                                if (d.hashed_last_name != null) {
                                    m.hashed_last_name = String(d.hashed_last_name);
                                }
                                if (d.city != null) {
                                    m.city = String(d.city);
                                }
                                if (d.state != null) {
                                    m.state = String(d.state);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.postal_code != null) {
                                    m.postal_code = String(d.postal_code);
                                }
                                if (d.hashed_street_address != null) {
                                    m.hashed_street_address = String(d.hashed_street_address);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserAddressInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.OfflineUserAddressInfo} m OfflineUserAddressInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserAddressInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.hashed_first_name != null && m.hasOwnProperty("hashed_first_name")) {
                                    d.hashed_first_name = m.hashed_first_name;
                                    if (o.oneofs)
                                        d._hashed_first_name = "hashed_first_name";
                                }
                                if (m.hashed_last_name != null && m.hasOwnProperty("hashed_last_name")) {
                                    d.hashed_last_name = m.hashed_last_name;
                                    if (o.oneofs)
                                        d._hashed_last_name = "hashed_last_name";
                                }
                                if (m.city != null && m.hasOwnProperty("city")) {
                                    d.city = m.city;
                                    if (o.oneofs)
                                        d._city = "city";
                                }
                                if (m.state != null && m.hasOwnProperty("state")) {
                                    d.state = m.state;
                                    if (o.oneofs)
                                        d._state = "state";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.postal_code != null && m.hasOwnProperty("postal_code")) {
                                    d.postal_code = m.postal_code;
                                    if (o.oneofs)
                                        d._postal_code = "postal_code";
                                }
                                if (m.hashed_street_address != null && m.hasOwnProperty("hashed_street_address")) {
                                    d.hashed_street_address = m.hashed_street_address;
                                    if (o.oneofs)
                                        d._hashed_street_address = "hashed_street_address";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this OfflineUserAddressInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.OfflineUserAddressInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserAddressInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return OfflineUserAddressInfo;
                        })();
    
                        common.UserIdentifier = (function() {
    
                            /**
                             * Properties of a UserIdentifier.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserIdentifier
                             * @property {google.ads.googleads.v11.enums.UserIdentifierSourceEnum.UserIdentifierSource|null} [user_identifier_source] UserIdentifier user_identifier_source
                             * @property {string|null} [hashed_email] UserIdentifier hashed_email
                             * @property {string|null} [hashed_phone_number] UserIdentifier hashed_phone_number
                             * @property {string|null} [mobile_id] UserIdentifier mobile_id
                             * @property {string|null} [third_party_user_id] UserIdentifier third_party_user_id
                             * @property {google.ads.googleads.v11.common.IOfflineUserAddressInfo|null} [address_info] UserIdentifier address_info
                             */
    
                            /**
                             * Constructs a new UserIdentifier.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserIdentifier.
                             * @implements IUserIdentifier
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserIdentifier=} [p] Properties to set
                             */
                            function UserIdentifier(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserIdentifier user_identifier_source.
                             * @member {google.ads.googleads.v11.enums.UserIdentifierSourceEnum.UserIdentifierSource} user_identifier_source
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             */
                            UserIdentifier.prototype.user_identifier_source = 0;
    
                            /**
                             * UserIdentifier hashed_email.
                             * @member {string|null|undefined} hashed_email
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             */
                            UserIdentifier.prototype.hashed_email = null;
    
                            /**
                             * UserIdentifier hashed_phone_number.
                             * @member {string|null|undefined} hashed_phone_number
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             */
                            UserIdentifier.prototype.hashed_phone_number = null;
    
                            /**
                             * UserIdentifier mobile_id.
                             * @member {string|null|undefined} mobile_id
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             */
                            UserIdentifier.prototype.mobile_id = null;
    
                            /**
                             * UserIdentifier third_party_user_id.
                             * @member {string|null|undefined} third_party_user_id
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             */
                            UserIdentifier.prototype.third_party_user_id = null;
    
                            /**
                             * UserIdentifier address_info.
                             * @member {google.ads.googleads.v11.common.IOfflineUserAddressInfo|null|undefined} address_info
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             */
                            UserIdentifier.prototype.address_info = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserIdentifier identifier.
                             * @member {"hashed_email"|"hashed_phone_number"|"mobile_id"|"third_party_user_id"|"address_info"|undefined} identifier
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             */
                            Object.defineProperty(UserIdentifier.prototype, "identifier", {
                                get: $util.oneOfGetter($oneOfFields = ["hashed_email", "hashed_phone_number", "mobile_id", "third_party_user_id", "address_info"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserIdentifier instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserIdentifier=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserIdentifier} UserIdentifier instance
                             */
                            UserIdentifier.create = function create(properties) {
                                return new UserIdentifier(properties);
                            };
    
                            /**
                             * Encodes the specified UserIdentifier message. Does not implicitly {@link google.ads.googleads.v11.common.UserIdentifier.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserIdentifier} m UserIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserIdentifier.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.address_info != null && Object.hasOwnProperty.call(m, "address_info"))
                                    $root.google.ads.googleads.v11.common.OfflineUserAddressInfo.encode(m.address_info, w.uint32(42).fork()).ldelim();
                                if (m.user_identifier_source != null && Object.hasOwnProperty.call(m, "user_identifier_source"))
                                    w.uint32(48).int32(m.user_identifier_source);
                                if (m.hashed_email != null && Object.hasOwnProperty.call(m, "hashed_email"))
                                    w.uint32(58).string(m.hashed_email);
                                if (m.hashed_phone_number != null && Object.hasOwnProperty.call(m, "hashed_phone_number"))
                                    w.uint32(66).string(m.hashed_phone_number);
                                if (m.mobile_id != null && Object.hasOwnProperty.call(m, "mobile_id"))
                                    w.uint32(74).string(m.mobile_id);
                                if (m.third_party_user_id != null && Object.hasOwnProperty.call(m, "third_party_user_id"))
                                    w.uint32(82).string(m.third_party_user_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserIdentifier message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserIdentifier.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserIdentifier} message UserIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserIdentifier.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserIdentifier message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserIdentifier} UserIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserIdentifier.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserIdentifier();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.user_identifier_source = r.int32();
                                        break;
                                    case 7:
                                        m.hashed_email = r.string();
                                        break;
                                    case 8:
                                        m.hashed_phone_number = r.string();
                                        break;
                                    case 9:
                                        m.mobile_id = r.string();
                                        break;
                                    case 10:
                                        m.third_party_user_id = r.string();
                                        break;
                                    case 5:
                                        m.address_info = $root.google.ads.googleads.v11.common.OfflineUserAddressInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserIdentifier message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserIdentifier} UserIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserIdentifier.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserIdentifier message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserIdentifier.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_identifier_source != null && m.hasOwnProperty("user_identifier_source")) {
                                    switch (m.user_identifier_source) {
                                    default:
                                        return "user_identifier_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.hashed_email != null && m.hasOwnProperty("hashed_email")) {
                                    p.identifier = 1;
                                    if (!$util.isString(m.hashed_email))
                                        return "hashed_email: string expected";
                                }
                                if (m.hashed_phone_number != null && m.hasOwnProperty("hashed_phone_number")) {
                                    if (p.identifier === 1)
                                        return "identifier: multiple values";
                                    p.identifier = 1;
                                    if (!$util.isString(m.hashed_phone_number))
                                        return "hashed_phone_number: string expected";
                                }
                                if (m.mobile_id != null && m.hasOwnProperty("mobile_id")) {
                                    if (p.identifier === 1)
                                        return "identifier: multiple values";
                                    p.identifier = 1;
                                    if (!$util.isString(m.mobile_id))
                                        return "mobile_id: string expected";
                                }
                                if (m.third_party_user_id != null && m.hasOwnProperty("third_party_user_id")) {
                                    if (p.identifier === 1)
                                        return "identifier: multiple values";
                                    p.identifier = 1;
                                    if (!$util.isString(m.third_party_user_id))
                                        return "third_party_user_id: string expected";
                                }
                                if (m.address_info != null && m.hasOwnProperty("address_info")) {
                                    if (p.identifier === 1)
                                        return "identifier: multiple values";
                                    p.identifier = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.OfflineUserAddressInfo.verify(m.address_info);
                                        if (e)
                                            return "address_info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserIdentifier message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserIdentifier} UserIdentifier
                             */
                            UserIdentifier.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserIdentifier)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserIdentifier();
                                switch (d.user_identifier_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.user_identifier_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.user_identifier_source = 1;
                                    break;
                                case "FIRST_PARTY":
                                case 2:
                                    m.user_identifier_source = 2;
                                    break;
                                case "THIRD_PARTY":
                                case 3:
                                    m.user_identifier_source = 3;
                                    break;
                                }
                                if (d.hashed_email != null) {
                                    m.hashed_email = String(d.hashed_email);
                                }
                                if (d.hashed_phone_number != null) {
                                    m.hashed_phone_number = String(d.hashed_phone_number);
                                }
                                if (d.mobile_id != null) {
                                    m.mobile_id = String(d.mobile_id);
                                }
                                if (d.third_party_user_id != null) {
                                    m.third_party_user_id = String(d.third_party_user_id);
                                }
                                if (d.address_info != null) {
                                    if (typeof d.address_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.UserIdentifier.address_info: object expected");
                                    m.address_info = $root.google.ads.googleads.v11.common.OfflineUserAddressInfo.fromObject(d.address_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserIdentifier message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.common.UserIdentifier} m UserIdentifier
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserIdentifier.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.user_identifier_source = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.address_info != null && m.hasOwnProperty("address_info")) {
                                    d.address_info = $root.google.ads.googleads.v11.common.OfflineUserAddressInfo.toObject(m.address_info, o);
                                    if (o.oneofs)
                                        d.identifier = "address_info";
                                }
                                if (m.user_identifier_source != null && m.hasOwnProperty("user_identifier_source")) {
                                    d.user_identifier_source = o.enums === String ? $root.google.ads.googleads.v11.enums.UserIdentifierSourceEnum.UserIdentifierSource[m.user_identifier_source] : m.user_identifier_source;
                                }
                                if (m.hashed_email != null && m.hasOwnProperty("hashed_email")) {
                                    d.hashed_email = m.hashed_email;
                                    if (o.oneofs)
                                        d.identifier = "hashed_email";
                                }
                                if (m.hashed_phone_number != null && m.hasOwnProperty("hashed_phone_number")) {
                                    d.hashed_phone_number = m.hashed_phone_number;
                                    if (o.oneofs)
                                        d.identifier = "hashed_phone_number";
                                }
                                if (m.mobile_id != null && m.hasOwnProperty("mobile_id")) {
                                    d.mobile_id = m.mobile_id;
                                    if (o.oneofs)
                                        d.identifier = "mobile_id";
                                }
                                if (m.third_party_user_id != null && m.hasOwnProperty("third_party_user_id")) {
                                    d.third_party_user_id = m.third_party_user_id;
                                    if (o.oneofs)
                                        d.identifier = "third_party_user_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserIdentifier to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserIdentifier
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserIdentifier.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserIdentifier;
                        })();
    
                        common.TransactionAttribute = (function() {
    
                            /**
                             * Properties of a TransactionAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITransactionAttribute
                             * @property {string|null} [transaction_date_time] TransactionAttribute transaction_date_time
                             * @property {number|null} [transaction_amount_micros] TransactionAttribute transaction_amount_micros
                             * @property {string|null} [currency_code] TransactionAttribute currency_code
                             * @property {string|null} [conversion_action] TransactionAttribute conversion_action
                             * @property {string|null} [order_id] TransactionAttribute order_id
                             * @property {google.ads.googleads.v11.common.IStoreAttribute|null} [store_attribute] TransactionAttribute store_attribute
                             * @property {string|null} [custom_value] TransactionAttribute custom_value
                             * @property {google.ads.googleads.v11.common.IItemAttribute|null} [item_attribute] TransactionAttribute item_attribute
                             */
    
                            /**
                             * Constructs a new TransactionAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TransactionAttribute.
                             * @implements ITransactionAttribute
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITransactionAttribute=} [p] Properties to set
                             */
                            function TransactionAttribute(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TransactionAttribute transaction_date_time.
                             * @member {string|null|undefined} transaction_date_time
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.transaction_date_time = null;
    
                            /**
                             * TransactionAttribute transaction_amount_micros.
                             * @member {number|null|undefined} transaction_amount_micros
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.transaction_amount_micros = null;
    
                            /**
                             * TransactionAttribute currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.currency_code = null;
    
                            /**
                             * TransactionAttribute conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.conversion_action = null;
    
                            /**
                             * TransactionAttribute order_id.
                             * @member {string|null|undefined} order_id
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.order_id = null;
    
                            /**
                             * TransactionAttribute store_attribute.
                             * @member {google.ads.googleads.v11.common.IStoreAttribute|null|undefined} store_attribute
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.store_attribute = null;
    
                            /**
                             * TransactionAttribute custom_value.
                             * @member {string|null|undefined} custom_value
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.custom_value = null;
    
                            /**
                             * TransactionAttribute item_attribute.
                             * @member {google.ads.googleads.v11.common.IItemAttribute|null|undefined} item_attribute
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            TransactionAttribute.prototype.item_attribute = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TransactionAttribute _transaction_date_time.
                             * @member {"transaction_date_time"|undefined} _transaction_date_time
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            Object.defineProperty(TransactionAttribute.prototype, "_transaction_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["transaction_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TransactionAttribute _transaction_amount_micros.
                             * @member {"transaction_amount_micros"|undefined} _transaction_amount_micros
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            Object.defineProperty(TransactionAttribute.prototype, "_transaction_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["transaction_amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TransactionAttribute _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            Object.defineProperty(TransactionAttribute.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TransactionAttribute _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            Object.defineProperty(TransactionAttribute.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TransactionAttribute _order_id.
                             * @member {"order_id"|undefined} _order_id
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            Object.defineProperty(TransactionAttribute.prototype, "_order_id", {
                                get: $util.oneOfGetter($oneOfFields = ["order_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TransactionAttribute _custom_value.
                             * @member {"custom_value"|undefined} _custom_value
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             */
                            Object.defineProperty(TransactionAttribute.prototype, "_custom_value", {
                                get: $util.oneOfGetter($oneOfFields = ["custom_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TransactionAttribute instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.ITransactionAttribute=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TransactionAttribute} TransactionAttribute instance
                             */
                            TransactionAttribute.create = function create(properties) {
                                return new TransactionAttribute(properties);
                            };
    
                            /**
                             * Encodes the specified TransactionAttribute message. Does not implicitly {@link google.ads.googleads.v11.common.TransactionAttribute.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.ITransactionAttribute} m TransactionAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TransactionAttribute.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.store_attribute != null && Object.hasOwnProperty.call(m, "store_attribute"))
                                    $root.google.ads.googleads.v11.common.StoreAttribute.encode(m.store_attribute, w.uint32(50).fork()).ldelim();
                                if (m.transaction_date_time != null && Object.hasOwnProperty.call(m, "transaction_date_time"))
                                    w.uint32(66).string(m.transaction_date_time);
                                if (m.transaction_amount_micros != null && Object.hasOwnProperty.call(m, "transaction_amount_micros"))
                                    w.uint32(73).double(m.transaction_amount_micros);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(82).string(m.currency_code);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(90).string(m.conversion_action);
                                if (m.order_id != null && Object.hasOwnProperty.call(m, "order_id"))
                                    w.uint32(98).string(m.order_id);
                                if (m.custom_value != null && Object.hasOwnProperty.call(m, "custom_value"))
                                    w.uint32(106).string(m.custom_value);
                                if (m.item_attribute != null && Object.hasOwnProperty.call(m, "item_attribute"))
                                    $root.google.ads.googleads.v11.common.ItemAttribute.encode(m.item_attribute, w.uint32(114).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TransactionAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TransactionAttribute.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.ITransactionAttribute} message TransactionAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TransactionAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TransactionAttribute message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TransactionAttribute} TransactionAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TransactionAttribute.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TransactionAttribute();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 8:
                                        m.transaction_date_time = r.string();
                                        break;
                                    case 9:
                                        m.transaction_amount_micros = r.double();
                                        break;
                                    case 10:
                                        m.currency_code = r.string();
                                        break;
                                    case 11:
                                        m.conversion_action = r.string();
                                        break;
                                    case 12:
                                        m.order_id = r.string();
                                        break;
                                    case 6:
                                        m.store_attribute = $root.google.ads.googleads.v11.common.StoreAttribute.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.custom_value = r.string();
                                        break;
                                    case 14:
                                        m.item_attribute = $root.google.ads.googleads.v11.common.ItemAttribute.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TransactionAttribute message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TransactionAttribute} TransactionAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TransactionAttribute.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TransactionAttribute message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TransactionAttribute.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.transaction_date_time != null && m.hasOwnProperty("transaction_date_time")) {
                                    p._transaction_date_time = 1;
                                    if (!$util.isString(m.transaction_date_time))
                                        return "transaction_date_time: string expected";
                                }
                                if (m.transaction_amount_micros != null && m.hasOwnProperty("transaction_amount_micros")) {
                                    p._transaction_amount_micros = 1;
                                    if (typeof m.transaction_amount_micros !== "number")
                                        return "transaction_amount_micros: number expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    p._order_id = 1;
                                    if (!$util.isString(m.order_id))
                                        return "order_id: string expected";
                                }
                                if (m.store_attribute != null && m.hasOwnProperty("store_attribute")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.StoreAttribute.verify(m.store_attribute);
                                        if (e)
                                            return "store_attribute." + e;
                                    }
                                }
                                if (m.custom_value != null && m.hasOwnProperty("custom_value")) {
                                    p._custom_value = 1;
                                    if (!$util.isString(m.custom_value))
                                        return "custom_value: string expected";
                                }
                                if (m.item_attribute != null && m.hasOwnProperty("item_attribute")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ItemAttribute.verify(m.item_attribute);
                                        if (e)
                                            return "item_attribute." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TransactionAttribute message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TransactionAttribute} TransactionAttribute
                             */
                            TransactionAttribute.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TransactionAttribute)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TransactionAttribute();
                                if (d.transaction_date_time != null) {
                                    m.transaction_date_time = String(d.transaction_date_time);
                                }
                                if (d.transaction_amount_micros != null) {
                                    m.transaction_amount_micros = Number(d.transaction_amount_micros);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.order_id != null) {
                                    m.order_id = String(d.order_id);
                                }
                                if (d.store_attribute != null) {
                                    if (typeof d.store_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.TransactionAttribute.store_attribute: object expected");
                                    m.store_attribute = $root.google.ads.googleads.v11.common.StoreAttribute.fromObject(d.store_attribute);
                                }
                                if (d.custom_value != null) {
                                    m.custom_value = String(d.custom_value);
                                }
                                if (d.item_attribute != null) {
                                    if (typeof d.item_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.TransactionAttribute.item_attribute: object expected");
                                    m.item_attribute = $root.google.ads.googleads.v11.common.ItemAttribute.fromObject(d.item_attribute);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TransactionAttribute message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.TransactionAttribute} m TransactionAttribute
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TransactionAttribute.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.store_attribute = null;
                                    d.item_attribute = null;
                                }
                                if (m.store_attribute != null && m.hasOwnProperty("store_attribute")) {
                                    d.store_attribute = $root.google.ads.googleads.v11.common.StoreAttribute.toObject(m.store_attribute, o);
                                }
                                if (m.transaction_date_time != null && m.hasOwnProperty("transaction_date_time")) {
                                    d.transaction_date_time = m.transaction_date_time;
                                    if (o.oneofs)
                                        d._transaction_date_time = "transaction_date_time";
                                }
                                if (m.transaction_amount_micros != null && m.hasOwnProperty("transaction_amount_micros")) {
                                    d.transaction_amount_micros = o.json && !isFinite(m.transaction_amount_micros) ? String(m.transaction_amount_micros) : m.transaction_amount_micros;
                                    if (o.oneofs)
                                        d._transaction_amount_micros = "transaction_amount_micros";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    d.order_id = m.order_id;
                                    if (o.oneofs)
                                        d._order_id = "order_id";
                                }
                                if (m.custom_value != null && m.hasOwnProperty("custom_value")) {
                                    d.custom_value = m.custom_value;
                                    if (o.oneofs)
                                        d._custom_value = "custom_value";
                                }
                                if (m.item_attribute != null && m.hasOwnProperty("item_attribute")) {
                                    d.item_attribute = $root.google.ads.googleads.v11.common.ItemAttribute.toObject(m.item_attribute, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TransactionAttribute to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TransactionAttribute
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TransactionAttribute.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TransactionAttribute;
                        })();
    
                        common.StoreAttribute = (function() {
    
                            /**
                             * Properties of a StoreAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IStoreAttribute
                             * @property {string|null} [store_code] StoreAttribute store_code
                             */
    
                            /**
                             * Constructs a new StoreAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a StoreAttribute.
                             * @implements IStoreAttribute
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IStoreAttribute=} [p] Properties to set
                             */
                            function StoreAttribute(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * StoreAttribute store_code.
                             * @member {string|null|undefined} store_code
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @instance
                             */
                            StoreAttribute.prototype.store_code = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * StoreAttribute _store_code.
                             * @member {"store_code"|undefined} _store_code
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @instance
                             */
                            Object.defineProperty(StoreAttribute.prototype, "_store_code", {
                                get: $util.oneOfGetter($oneOfFields = ["store_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new StoreAttribute instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreAttribute=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.StoreAttribute} StoreAttribute instance
                             */
                            StoreAttribute.create = function create(properties) {
                                return new StoreAttribute(properties);
                            };
    
                            /**
                             * Encodes the specified StoreAttribute message. Does not implicitly {@link google.ads.googleads.v11.common.StoreAttribute.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreAttribute} m StoreAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StoreAttribute.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.store_code != null && Object.hasOwnProperty.call(m, "store_code"))
                                    w.uint32(18).string(m.store_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified StoreAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.StoreAttribute.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreAttribute} message StoreAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StoreAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StoreAttribute message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.StoreAttribute} StoreAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StoreAttribute.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.StoreAttribute();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.store_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StoreAttribute message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.StoreAttribute} StoreAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StoreAttribute.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StoreAttribute message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StoreAttribute.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.store_code != null && m.hasOwnProperty("store_code")) {
                                    p._store_code = 1;
                                    if (!$util.isString(m.store_code))
                                        return "store_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a StoreAttribute message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.StoreAttribute} StoreAttribute
                             */
                            StoreAttribute.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.StoreAttribute)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.StoreAttribute();
                                if (d.store_code != null) {
                                    m.store_code = String(d.store_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a StoreAttribute message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.StoreAttribute} m StoreAttribute
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StoreAttribute.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.store_code != null && m.hasOwnProperty("store_code")) {
                                    d.store_code = m.store_code;
                                    if (o.oneofs)
                                        d._store_code = "store_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this StoreAttribute to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.StoreAttribute
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StoreAttribute.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return StoreAttribute;
                        })();
    
                        common.ItemAttribute = (function() {
    
                            /**
                             * Properties of an ItemAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IItemAttribute
                             * @property {string|null} [item_id] ItemAttribute item_id
                             * @property {number|null} [merchant_id] ItemAttribute merchant_id
                             * @property {string|null} [country_code] ItemAttribute country_code
                             * @property {string|null} [language_code] ItemAttribute language_code
                             * @property {number|null} [quantity] ItemAttribute quantity
                             */
    
                            /**
                             * Constructs a new ItemAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ItemAttribute.
                             * @implements IItemAttribute
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IItemAttribute=} [p] Properties to set
                             */
                            function ItemAttribute(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ItemAttribute item_id.
                             * @member {string} item_id
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @instance
                             */
                            ItemAttribute.prototype.item_id = "";
    
                            /**
                             * ItemAttribute merchant_id.
                             * @member {number|null|undefined} merchant_id
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @instance
                             */
                            ItemAttribute.prototype.merchant_id = null;
    
                            /**
                             * ItemAttribute country_code.
                             * @member {string} country_code
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @instance
                             */
                            ItemAttribute.prototype.country_code = "";
    
                            /**
                             * ItemAttribute language_code.
                             * @member {string} language_code
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @instance
                             */
                            ItemAttribute.prototype.language_code = "";
    
                            /**
                             * ItemAttribute quantity.
                             * @member {number} quantity
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @instance
                             */
                            ItemAttribute.prototype.quantity = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ItemAttribute _merchant_id.
                             * @member {"merchant_id"|undefined} _merchant_id
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @instance
                             */
                            Object.defineProperty(ItemAttribute.prototype, "_merchant_id", {
                                get: $util.oneOfGetter($oneOfFields = ["merchant_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ItemAttribute instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IItemAttribute=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ItemAttribute} ItemAttribute instance
                             */
                            ItemAttribute.create = function create(properties) {
                                return new ItemAttribute(properties);
                            };
    
                            /**
                             * Encodes the specified ItemAttribute message. Does not implicitly {@link google.ads.googleads.v11.common.ItemAttribute.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IItemAttribute} m ItemAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ItemAttribute.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.item_id != null && Object.hasOwnProperty.call(m, "item_id"))
                                    w.uint32(10).string(m.item_id);
                                if (m.merchant_id != null && Object.hasOwnProperty.call(m, "merchant_id"))
                                    w.uint32(16).int64(m.merchant_id);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(26).string(m.country_code);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(34).string(m.language_code);
                                if (m.quantity != null && Object.hasOwnProperty.call(m, "quantity"))
                                    w.uint32(40).int64(m.quantity);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ItemAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ItemAttribute.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IItemAttribute} message ItemAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ItemAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ItemAttribute message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ItemAttribute} ItemAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ItemAttribute.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ItemAttribute();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.item_id = r.string();
                                        break;
                                    case 2:
                                        m.merchant_id = r.int64();
                                        break;
                                    case 3:
                                        m.country_code = r.string();
                                        break;
                                    case 4:
                                        m.language_code = r.string();
                                        break;
                                    case 5:
                                        m.quantity = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ItemAttribute message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ItemAttribute} ItemAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ItemAttribute.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ItemAttribute message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ItemAttribute.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.item_id != null && m.hasOwnProperty("item_id")) {
                                    if (!$util.isString(m.item_id))
                                        return "item_id: string expected";
                                }
                                if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                    p._merchant_id = 1;
                                    if (!$util.isInteger(m.merchant_id) && !(m.merchant_id && $util.isInteger(m.merchant_id.low) && $util.isInteger(m.merchant_id.high)))
                                        return "merchant_id: integer|Long expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.quantity != null && m.hasOwnProperty("quantity")) {
                                    if (!$util.isInteger(m.quantity) && !(m.quantity && $util.isInteger(m.quantity.low) && $util.isInteger(m.quantity.high)))
                                        return "quantity: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ItemAttribute message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ItemAttribute} ItemAttribute
                             */
                            ItemAttribute.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ItemAttribute)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ItemAttribute();
                                if (d.item_id != null) {
                                    m.item_id = String(d.item_id);
                                }
                                if (d.merchant_id != null) {
                                    if ($util.Long)
                                        (m.merchant_id = $util.Long.fromValue(d.merchant_id)).unsigned = false;
                                    else if (typeof d.merchant_id === "string")
                                        m.merchant_id = parseInt(d.merchant_id, 10);
                                    else if (typeof d.merchant_id === "number")
                                        m.merchant_id = d.merchant_id;
                                    else if (typeof d.merchant_id === "object")
                                        m.merchant_id = new $util.LongBits(d.merchant_id.low >>> 0, d.merchant_id.high >>> 0).toNumber();
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.quantity != null) {
                                    if ($util.Long)
                                        (m.quantity = $util.Long.fromValue(d.quantity)).unsigned = false;
                                    else if (typeof d.quantity === "string")
                                        m.quantity = parseInt(d.quantity, 10);
                                    else if (typeof d.quantity === "number")
                                        m.quantity = d.quantity;
                                    else if (typeof d.quantity === "object")
                                        m.quantity = new $util.LongBits(d.quantity.low >>> 0, d.quantity.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ItemAttribute message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.ItemAttribute} m ItemAttribute
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ItemAttribute.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.item_id = "";
                                    d.country_code = "";
                                    d.language_code = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.quantity = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.quantity = o.longs === String ? "0" : 0;
                                }
                                if (m.item_id != null && m.hasOwnProperty("item_id")) {
                                    d.item_id = m.item_id;
                                }
                                if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                    if (typeof m.merchant_id === "number")
                                        d.merchant_id = o.longs === String ? String(m.merchant_id) : m.merchant_id;
                                    else
                                        d.merchant_id = o.longs === String ? $util.Long.prototype.toString.call(m.merchant_id) : o.longs === Number ? new $util.LongBits(m.merchant_id.low >>> 0, m.merchant_id.high >>> 0).toNumber() : m.merchant_id;
                                    if (o.oneofs)
                                        d._merchant_id = "merchant_id";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                }
                                if (m.quantity != null && m.hasOwnProperty("quantity")) {
                                    if (typeof m.quantity === "number")
                                        d.quantity = o.longs === String ? String(m.quantity) : m.quantity;
                                    else
                                        d.quantity = o.longs === String ? $util.Long.prototype.toString.call(m.quantity) : o.longs === Number ? new $util.LongBits(m.quantity.low >>> 0, m.quantity.high >>> 0).toNumber() : m.quantity;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ItemAttribute to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ItemAttribute
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ItemAttribute.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ItemAttribute;
                        })();
    
                        common.UserData = (function() {
    
                            /**
                             * Properties of a UserData.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserData
                             * @property {Array.<google.ads.googleads.v11.common.IUserIdentifier>|null} [user_identifiers] UserData user_identifiers
                             * @property {google.ads.googleads.v11.common.ITransactionAttribute|null} [transaction_attribute] UserData transaction_attribute
                             * @property {google.ads.googleads.v11.common.IUserAttribute|null} [user_attribute] UserData user_attribute
                             */
    
                            /**
                             * Constructs a new UserData.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserData.
                             * @implements IUserData
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserData=} [p] Properties to set
                             */
                            function UserData(p) {
                                this.user_identifiers = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserData user_identifiers.
                             * @member {Array.<google.ads.googleads.v11.common.IUserIdentifier>} user_identifiers
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @instance
                             */
                            UserData.prototype.user_identifiers = $util.emptyArray;
    
                            /**
                             * UserData transaction_attribute.
                             * @member {google.ads.googleads.v11.common.ITransactionAttribute|null|undefined} transaction_attribute
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @instance
                             */
                            UserData.prototype.transaction_attribute = null;
    
                            /**
                             * UserData user_attribute.
                             * @member {google.ads.googleads.v11.common.IUserAttribute|null|undefined} user_attribute
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @instance
                             */
                            UserData.prototype.user_attribute = null;
    
                            /**
                             * Creates a new UserData instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserData=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserData} UserData instance
                             */
                            UserData.create = function create(properties) {
                                return new UserData(properties);
                            };
    
                            /**
                             * Encodes the specified UserData message. Does not implicitly {@link google.ads.googleads.v11.common.UserData.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserData} m UserData message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserData.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_identifiers != null && m.user_identifiers.length) {
                                    for (var i = 0; i < m.user_identifiers.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserIdentifier.encode(m.user_identifiers[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.transaction_attribute != null && Object.hasOwnProperty.call(m, "transaction_attribute"))
                                    $root.google.ads.googleads.v11.common.TransactionAttribute.encode(m.transaction_attribute, w.uint32(18).fork()).ldelim();
                                if (m.user_attribute != null && Object.hasOwnProperty.call(m, "user_attribute"))
                                    $root.google.ads.googleads.v11.common.UserAttribute.encode(m.user_attribute, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserData message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserData.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserData} message UserData message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserData.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserData message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserData} UserData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserData.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserData();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.user_identifiers && m.user_identifiers.length))
                                            m.user_identifiers = [];
                                        m.user_identifiers.push($root.google.ads.googleads.v11.common.UserIdentifier.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.transaction_attribute = $root.google.ads.googleads.v11.common.TransactionAttribute.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.user_attribute = $root.google.ads.googleads.v11.common.UserAttribute.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserData message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserData} UserData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserData.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserData message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserData.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.user_identifiers != null && m.hasOwnProperty("user_identifiers")) {
                                    if (!Array.isArray(m.user_identifiers))
                                        return "user_identifiers: array expected";
                                    for (var i = 0; i < m.user_identifiers.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserIdentifier.verify(m.user_identifiers[i]);
                                            if (e)
                                                return "user_identifiers." + e;
                                        }
                                    }
                                }
                                if (m.transaction_attribute != null && m.hasOwnProperty("transaction_attribute")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TransactionAttribute.verify(m.transaction_attribute);
                                        if (e)
                                            return "transaction_attribute." + e;
                                    }
                                }
                                if (m.user_attribute != null && m.hasOwnProperty("user_attribute")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserAttribute.verify(m.user_attribute);
                                        if (e)
                                            return "user_attribute." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserData message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserData} UserData
                             */
                            UserData.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserData)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserData();
                                if (d.user_identifiers) {
                                    if (!Array.isArray(d.user_identifiers))
                                        throw TypeError(".google.ads.googleads.v11.common.UserData.user_identifiers: array expected");
                                    m.user_identifiers = [];
                                    for (var i = 0; i < d.user_identifiers.length; ++i) {
                                        if (typeof d.user_identifiers[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.UserData.user_identifiers: object expected");
                                        m.user_identifiers[i] = $root.google.ads.googleads.v11.common.UserIdentifier.fromObject(d.user_identifiers[i]);
                                    }
                                }
                                if (d.transaction_attribute != null) {
                                    if (typeof d.transaction_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.UserData.transaction_attribute: object expected");
                                    m.transaction_attribute = $root.google.ads.googleads.v11.common.TransactionAttribute.fromObject(d.transaction_attribute);
                                }
                                if (d.user_attribute != null) {
                                    if (typeof d.user_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.UserData.user_attribute: object expected");
                                    m.user_attribute = $root.google.ads.googleads.v11.common.UserAttribute.fromObject(d.user_attribute);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserData message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @static
                             * @param {google.ads.googleads.v11.common.UserData} m UserData
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserData.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.user_identifiers = [];
                                }
                                if (o.defaults) {
                                    d.transaction_attribute = null;
                                    d.user_attribute = null;
                                }
                                if (m.user_identifiers && m.user_identifiers.length) {
                                    d.user_identifiers = [];
                                    for (var j = 0; j < m.user_identifiers.length; ++j) {
                                        d.user_identifiers[j] = $root.google.ads.googleads.v11.common.UserIdentifier.toObject(m.user_identifiers[j], o);
                                    }
                                }
                                if (m.transaction_attribute != null && m.hasOwnProperty("transaction_attribute")) {
                                    d.transaction_attribute = $root.google.ads.googleads.v11.common.TransactionAttribute.toObject(m.transaction_attribute, o);
                                }
                                if (m.user_attribute != null && m.hasOwnProperty("user_attribute")) {
                                    d.user_attribute = $root.google.ads.googleads.v11.common.UserAttribute.toObject(m.user_attribute, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserData to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserData
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserData.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserData;
                        })();
    
                        common.UserAttribute = (function() {
    
                            /**
                             * Properties of a UserAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserAttribute
                             * @property {number|null} [lifetime_value_micros] UserAttribute lifetime_value_micros
                             * @property {number|null} [lifetime_value_bucket] UserAttribute lifetime_value_bucket
                             * @property {string|null} [last_purchase_date_time] UserAttribute last_purchase_date_time
                             * @property {number|null} [average_purchase_count] UserAttribute average_purchase_count
                             * @property {number|null} [average_purchase_value_micros] UserAttribute average_purchase_value_micros
                             * @property {string|null} [acquisition_date_time] UserAttribute acquisition_date_time
                             * @property {google.ads.googleads.v11.common.IShoppingLoyalty|null} [shopping_loyalty] UserAttribute shopping_loyalty
                             */
    
                            /**
                             * Constructs a new UserAttribute.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserAttribute.
                             * @implements IUserAttribute
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserAttribute=} [p] Properties to set
                             */
                            function UserAttribute(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserAttribute lifetime_value_micros.
                             * @member {number|null|undefined} lifetime_value_micros
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            UserAttribute.prototype.lifetime_value_micros = null;
    
                            /**
                             * UserAttribute lifetime_value_bucket.
                             * @member {number|null|undefined} lifetime_value_bucket
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            UserAttribute.prototype.lifetime_value_bucket = null;
    
                            /**
                             * UserAttribute last_purchase_date_time.
                             * @member {string} last_purchase_date_time
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            UserAttribute.prototype.last_purchase_date_time = "";
    
                            /**
                             * UserAttribute average_purchase_count.
                             * @member {number} average_purchase_count
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            UserAttribute.prototype.average_purchase_count = 0;
    
                            /**
                             * UserAttribute average_purchase_value_micros.
                             * @member {number} average_purchase_value_micros
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            UserAttribute.prototype.average_purchase_value_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * UserAttribute acquisition_date_time.
                             * @member {string} acquisition_date_time
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            UserAttribute.prototype.acquisition_date_time = "";
    
                            /**
                             * UserAttribute shopping_loyalty.
                             * @member {google.ads.googleads.v11.common.IShoppingLoyalty|null|undefined} shopping_loyalty
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            UserAttribute.prototype.shopping_loyalty = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserAttribute _lifetime_value_micros.
                             * @member {"lifetime_value_micros"|undefined} _lifetime_value_micros
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            Object.defineProperty(UserAttribute.prototype, "_lifetime_value_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["lifetime_value_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserAttribute _lifetime_value_bucket.
                             * @member {"lifetime_value_bucket"|undefined} _lifetime_value_bucket
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            Object.defineProperty(UserAttribute.prototype, "_lifetime_value_bucket", {
                                get: $util.oneOfGetter($oneOfFields = ["lifetime_value_bucket"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserAttribute _shopping_loyalty.
                             * @member {"shopping_loyalty"|undefined} _shopping_loyalty
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             */
                            Object.defineProperty(UserAttribute.prototype, "_shopping_loyalty", {
                                get: $util.oneOfGetter($oneOfFields = ["shopping_loyalty"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserAttribute instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserAttribute=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserAttribute} UserAttribute instance
                             */
                            UserAttribute.create = function create(properties) {
                                return new UserAttribute(properties);
                            };
    
                            /**
                             * Encodes the specified UserAttribute message. Does not implicitly {@link google.ads.googleads.v11.common.UserAttribute.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserAttribute} m UserAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserAttribute.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.lifetime_value_micros != null && Object.hasOwnProperty.call(m, "lifetime_value_micros"))
                                    w.uint32(8).int64(m.lifetime_value_micros);
                                if (m.lifetime_value_bucket != null && Object.hasOwnProperty.call(m, "lifetime_value_bucket"))
                                    w.uint32(16).int32(m.lifetime_value_bucket);
                                if (m.last_purchase_date_time != null && Object.hasOwnProperty.call(m, "last_purchase_date_time"))
                                    w.uint32(26).string(m.last_purchase_date_time);
                                if (m.average_purchase_count != null && Object.hasOwnProperty.call(m, "average_purchase_count"))
                                    w.uint32(32).int32(m.average_purchase_count);
                                if (m.average_purchase_value_micros != null && Object.hasOwnProperty.call(m, "average_purchase_value_micros"))
                                    w.uint32(40).int64(m.average_purchase_value_micros);
                                if (m.acquisition_date_time != null && Object.hasOwnProperty.call(m, "acquisition_date_time"))
                                    w.uint32(50).string(m.acquisition_date_time);
                                if (m.shopping_loyalty != null && Object.hasOwnProperty.call(m, "shopping_loyalty"))
                                    $root.google.ads.googleads.v11.common.ShoppingLoyalty.encode(m.shopping_loyalty, w.uint32(58).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserAttribute.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserAttribute} message UserAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserAttribute message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserAttribute} UserAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserAttribute.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserAttribute();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.lifetime_value_micros = r.int64();
                                        break;
                                    case 2:
                                        m.lifetime_value_bucket = r.int32();
                                        break;
                                    case 3:
                                        m.last_purchase_date_time = r.string();
                                        break;
                                    case 4:
                                        m.average_purchase_count = r.int32();
                                        break;
                                    case 5:
                                        m.average_purchase_value_micros = r.int64();
                                        break;
                                    case 6:
                                        m.acquisition_date_time = r.string();
                                        break;
                                    case 7:
                                        m.shopping_loyalty = $root.google.ads.googleads.v11.common.ShoppingLoyalty.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserAttribute message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserAttribute} UserAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserAttribute.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserAttribute message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserAttribute.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.lifetime_value_micros != null && m.hasOwnProperty("lifetime_value_micros")) {
                                    p._lifetime_value_micros = 1;
                                    if (!$util.isInteger(m.lifetime_value_micros) && !(m.lifetime_value_micros && $util.isInteger(m.lifetime_value_micros.low) && $util.isInteger(m.lifetime_value_micros.high)))
                                        return "lifetime_value_micros: integer|Long expected";
                                }
                                if (m.lifetime_value_bucket != null && m.hasOwnProperty("lifetime_value_bucket")) {
                                    p._lifetime_value_bucket = 1;
                                    if (!$util.isInteger(m.lifetime_value_bucket))
                                        return "lifetime_value_bucket: integer expected";
                                }
                                if (m.last_purchase_date_time != null && m.hasOwnProperty("last_purchase_date_time")) {
                                    if (!$util.isString(m.last_purchase_date_time))
                                        return "last_purchase_date_time: string expected";
                                }
                                if (m.average_purchase_count != null && m.hasOwnProperty("average_purchase_count")) {
                                    if (!$util.isInteger(m.average_purchase_count))
                                        return "average_purchase_count: integer expected";
                                }
                                if (m.average_purchase_value_micros != null && m.hasOwnProperty("average_purchase_value_micros")) {
                                    if (!$util.isInteger(m.average_purchase_value_micros) && !(m.average_purchase_value_micros && $util.isInteger(m.average_purchase_value_micros.low) && $util.isInteger(m.average_purchase_value_micros.high)))
                                        return "average_purchase_value_micros: integer|Long expected";
                                }
                                if (m.acquisition_date_time != null && m.hasOwnProperty("acquisition_date_time")) {
                                    if (!$util.isString(m.acquisition_date_time))
                                        return "acquisition_date_time: string expected";
                                }
                                if (m.shopping_loyalty != null && m.hasOwnProperty("shopping_loyalty")) {
                                    p._shopping_loyalty = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ShoppingLoyalty.verify(m.shopping_loyalty);
                                        if (e)
                                            return "shopping_loyalty." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserAttribute message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserAttribute} UserAttribute
                             */
                            UserAttribute.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserAttribute)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserAttribute();
                                if (d.lifetime_value_micros != null) {
                                    if ($util.Long)
                                        (m.lifetime_value_micros = $util.Long.fromValue(d.lifetime_value_micros)).unsigned = false;
                                    else if (typeof d.lifetime_value_micros === "string")
                                        m.lifetime_value_micros = parseInt(d.lifetime_value_micros, 10);
                                    else if (typeof d.lifetime_value_micros === "number")
                                        m.lifetime_value_micros = d.lifetime_value_micros;
                                    else if (typeof d.lifetime_value_micros === "object")
                                        m.lifetime_value_micros = new $util.LongBits(d.lifetime_value_micros.low >>> 0, d.lifetime_value_micros.high >>> 0).toNumber();
                                }
                                if (d.lifetime_value_bucket != null) {
                                    m.lifetime_value_bucket = d.lifetime_value_bucket | 0;
                                }
                                if (d.last_purchase_date_time != null) {
                                    m.last_purchase_date_time = String(d.last_purchase_date_time);
                                }
                                if (d.average_purchase_count != null) {
                                    m.average_purchase_count = d.average_purchase_count | 0;
                                }
                                if (d.average_purchase_value_micros != null) {
                                    if ($util.Long)
                                        (m.average_purchase_value_micros = $util.Long.fromValue(d.average_purchase_value_micros)).unsigned = false;
                                    else if (typeof d.average_purchase_value_micros === "string")
                                        m.average_purchase_value_micros = parseInt(d.average_purchase_value_micros, 10);
                                    else if (typeof d.average_purchase_value_micros === "number")
                                        m.average_purchase_value_micros = d.average_purchase_value_micros;
                                    else if (typeof d.average_purchase_value_micros === "object")
                                        m.average_purchase_value_micros = new $util.LongBits(d.average_purchase_value_micros.low >>> 0, d.average_purchase_value_micros.high >>> 0).toNumber();
                                }
                                if (d.acquisition_date_time != null) {
                                    m.acquisition_date_time = String(d.acquisition_date_time);
                                }
                                if (d.shopping_loyalty != null) {
                                    if (typeof d.shopping_loyalty !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.UserAttribute.shopping_loyalty: object expected");
                                    m.shopping_loyalty = $root.google.ads.googleads.v11.common.ShoppingLoyalty.fromObject(d.shopping_loyalty);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserAttribute message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @static
                             * @param {google.ads.googleads.v11.common.UserAttribute} m UserAttribute
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserAttribute.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.last_purchase_date_time = "";
                                    d.average_purchase_count = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.average_purchase_value_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.average_purchase_value_micros = o.longs === String ? "0" : 0;
                                    d.acquisition_date_time = "";
                                }
                                if (m.lifetime_value_micros != null && m.hasOwnProperty("lifetime_value_micros")) {
                                    if (typeof m.lifetime_value_micros === "number")
                                        d.lifetime_value_micros = o.longs === String ? String(m.lifetime_value_micros) : m.lifetime_value_micros;
                                    else
                                        d.lifetime_value_micros = o.longs === String ? $util.Long.prototype.toString.call(m.lifetime_value_micros) : o.longs === Number ? new $util.LongBits(m.lifetime_value_micros.low >>> 0, m.lifetime_value_micros.high >>> 0).toNumber() : m.lifetime_value_micros;
                                    if (o.oneofs)
                                        d._lifetime_value_micros = "lifetime_value_micros";
                                }
                                if (m.lifetime_value_bucket != null && m.hasOwnProperty("lifetime_value_bucket")) {
                                    d.lifetime_value_bucket = m.lifetime_value_bucket;
                                    if (o.oneofs)
                                        d._lifetime_value_bucket = "lifetime_value_bucket";
                                }
                                if (m.last_purchase_date_time != null && m.hasOwnProperty("last_purchase_date_time")) {
                                    d.last_purchase_date_time = m.last_purchase_date_time;
                                }
                                if (m.average_purchase_count != null && m.hasOwnProperty("average_purchase_count")) {
                                    d.average_purchase_count = m.average_purchase_count;
                                }
                                if (m.average_purchase_value_micros != null && m.hasOwnProperty("average_purchase_value_micros")) {
                                    if (typeof m.average_purchase_value_micros === "number")
                                        d.average_purchase_value_micros = o.longs === String ? String(m.average_purchase_value_micros) : m.average_purchase_value_micros;
                                    else
                                        d.average_purchase_value_micros = o.longs === String ? $util.Long.prototype.toString.call(m.average_purchase_value_micros) : o.longs === Number ? new $util.LongBits(m.average_purchase_value_micros.low >>> 0, m.average_purchase_value_micros.high >>> 0).toNumber() : m.average_purchase_value_micros;
                                }
                                if (m.acquisition_date_time != null && m.hasOwnProperty("acquisition_date_time")) {
                                    d.acquisition_date_time = m.acquisition_date_time;
                                }
                                if (m.shopping_loyalty != null && m.hasOwnProperty("shopping_loyalty")) {
                                    d.shopping_loyalty = $root.google.ads.googleads.v11.common.ShoppingLoyalty.toObject(m.shopping_loyalty, o);
                                    if (o.oneofs)
                                        d._shopping_loyalty = "shopping_loyalty";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserAttribute to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserAttribute
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserAttribute.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserAttribute;
                        })();
    
                        common.ShoppingLoyalty = (function() {
    
                            /**
                             * Properties of a ShoppingLoyalty.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IShoppingLoyalty
                             * @property {string|null} [loyalty_tier] ShoppingLoyalty loyalty_tier
                             */
    
                            /**
                             * Constructs a new ShoppingLoyalty.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a ShoppingLoyalty.
                             * @implements IShoppingLoyalty
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IShoppingLoyalty=} [p] Properties to set
                             */
                            function ShoppingLoyalty(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ShoppingLoyalty loyalty_tier.
                             * @member {string|null|undefined} loyalty_tier
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @instance
                             */
                            ShoppingLoyalty.prototype.loyalty_tier = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ShoppingLoyalty _loyalty_tier.
                             * @member {"loyalty_tier"|undefined} _loyalty_tier
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @instance
                             */
                            Object.defineProperty(ShoppingLoyalty.prototype, "_loyalty_tier", {
                                get: $util.oneOfGetter($oneOfFields = ["loyalty_tier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ShoppingLoyalty instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingLoyalty=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ShoppingLoyalty} ShoppingLoyalty instance
                             */
                            ShoppingLoyalty.create = function create(properties) {
                                return new ShoppingLoyalty(properties);
                            };
    
                            /**
                             * Encodes the specified ShoppingLoyalty message. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingLoyalty.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingLoyalty} m ShoppingLoyalty message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingLoyalty.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.loyalty_tier != null && Object.hasOwnProperty.call(m, "loyalty_tier"))
                                    w.uint32(10).string(m.loyalty_tier);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ShoppingLoyalty message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ShoppingLoyalty.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {google.ads.googleads.v11.common.IShoppingLoyalty} message ShoppingLoyalty message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingLoyalty.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ShoppingLoyalty message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ShoppingLoyalty} ShoppingLoyalty
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingLoyalty.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ShoppingLoyalty();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.loyalty_tier = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ShoppingLoyalty message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ShoppingLoyalty} ShoppingLoyalty
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingLoyalty.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ShoppingLoyalty message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ShoppingLoyalty.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.loyalty_tier != null && m.hasOwnProperty("loyalty_tier")) {
                                    p._loyalty_tier = 1;
                                    if (!$util.isString(m.loyalty_tier))
                                        return "loyalty_tier: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ShoppingLoyalty message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ShoppingLoyalty} ShoppingLoyalty
                             */
                            ShoppingLoyalty.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ShoppingLoyalty)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ShoppingLoyalty();
                                if (d.loyalty_tier != null) {
                                    m.loyalty_tier = String(d.loyalty_tier);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ShoppingLoyalty message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @static
                             * @param {google.ads.googleads.v11.common.ShoppingLoyalty} m ShoppingLoyalty
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ShoppingLoyalty.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.loyalty_tier != null && m.hasOwnProperty("loyalty_tier")) {
                                    d.loyalty_tier = m.loyalty_tier;
                                    if (o.oneofs)
                                        d._loyalty_tier = "loyalty_tier";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ShoppingLoyalty to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ShoppingLoyalty
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ShoppingLoyalty.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ShoppingLoyalty;
                        })();
    
                        common.CustomerMatchUserListMetadata = (function() {
    
                            /**
                             * Properties of a CustomerMatchUserListMetadata.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICustomerMatchUserListMetadata
                             * @property {string|null} [user_list] CustomerMatchUserListMetadata user_list
                             */
    
                            /**
                             * Constructs a new CustomerMatchUserListMetadata.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CustomerMatchUserListMetadata.
                             * @implements ICustomerMatchUserListMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata=} [p] Properties to set
                             */
                            function CustomerMatchUserListMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerMatchUserListMetadata user_list.
                             * @member {string|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @instance
                             */
                            CustomerMatchUserListMetadata.prototype.user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerMatchUserListMetadata _user_list.
                             * @member {"user_list"|undefined} _user_list
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @instance
                             */
                            Object.defineProperty(CustomerMatchUserListMetadata.prototype, "_user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerMatchUserListMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CustomerMatchUserListMetadata} CustomerMatchUserListMetadata instance
                             */
                            CustomerMatchUserListMetadata.create = function create(properties) {
                                return new CustomerMatchUserListMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerMatchUserListMetadata message. Does not implicitly {@link google.ads.googleads.v11.common.CustomerMatchUserListMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata} m CustomerMatchUserListMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerMatchUserListMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    w.uint32(18).string(m.user_list);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerMatchUserListMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CustomerMatchUserListMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata} message CustomerMatchUserListMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerMatchUserListMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerMatchUserListMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CustomerMatchUserListMetadata} CustomerMatchUserListMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerMatchUserListMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.user_list = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerMatchUserListMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CustomerMatchUserListMetadata} CustomerMatchUserListMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerMatchUserListMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerMatchUserListMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerMatchUserListMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    p._user_list = 1;
                                    if (!$util.isString(m.user_list))
                                        return "user_list: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerMatchUserListMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CustomerMatchUserListMetadata} CustomerMatchUserListMetadata
                             */
                            CustomerMatchUserListMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata();
                                if (d.user_list != null) {
                                    m.user_list = String(d.user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerMatchUserListMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.CustomerMatchUserListMetadata} m CustomerMatchUserListMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerMatchUserListMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = m.user_list;
                                    if (o.oneofs)
                                        d._user_list = "user_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerMatchUserListMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CustomerMatchUserListMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerMatchUserListMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerMatchUserListMetadata;
                        })();
    
                        common.StoreSalesMetadata = (function() {
    
                            /**
                             * Properties of a StoreSalesMetadata.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IStoreSalesMetadata
                             * @property {number|null} [loyalty_fraction] StoreSalesMetadata loyalty_fraction
                             * @property {number|null} [transaction_upload_fraction] StoreSalesMetadata transaction_upload_fraction
                             * @property {string|null} [custom_key] StoreSalesMetadata custom_key
                             * @property {google.ads.googleads.v11.common.IStoreSalesThirdPartyMetadata|null} [third_party_metadata] StoreSalesMetadata third_party_metadata
                             */
    
                            /**
                             * Constructs a new StoreSalesMetadata.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a StoreSalesMetadata.
                             * @implements IStoreSalesMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IStoreSalesMetadata=} [p] Properties to set
                             */
                            function StoreSalesMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * StoreSalesMetadata loyalty_fraction.
                             * @member {number|null|undefined} loyalty_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             */
                            StoreSalesMetadata.prototype.loyalty_fraction = null;
    
                            /**
                             * StoreSalesMetadata transaction_upload_fraction.
                             * @member {number|null|undefined} transaction_upload_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             */
                            StoreSalesMetadata.prototype.transaction_upload_fraction = null;
    
                            /**
                             * StoreSalesMetadata custom_key.
                             * @member {string|null|undefined} custom_key
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             */
                            StoreSalesMetadata.prototype.custom_key = null;
    
                            /**
                             * StoreSalesMetadata third_party_metadata.
                             * @member {google.ads.googleads.v11.common.IStoreSalesThirdPartyMetadata|null|undefined} third_party_metadata
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             */
                            StoreSalesMetadata.prototype.third_party_metadata = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * StoreSalesMetadata _loyalty_fraction.
                             * @member {"loyalty_fraction"|undefined} _loyalty_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesMetadata.prototype, "_loyalty_fraction", {
                                get: $util.oneOfGetter($oneOfFields = ["loyalty_fraction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * StoreSalesMetadata _transaction_upload_fraction.
                             * @member {"transaction_upload_fraction"|undefined} _transaction_upload_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesMetadata.prototype, "_transaction_upload_fraction", {
                                get: $util.oneOfGetter($oneOfFields = ["transaction_upload_fraction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * StoreSalesMetadata _custom_key.
                             * @member {"custom_key"|undefined} _custom_key
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesMetadata.prototype, "_custom_key", {
                                get: $util.oneOfGetter($oneOfFields = ["custom_key"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new StoreSalesMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreSalesMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.StoreSalesMetadata} StoreSalesMetadata instance
                             */
                            StoreSalesMetadata.create = function create(properties) {
                                return new StoreSalesMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified StoreSalesMetadata message. Does not implicitly {@link google.ads.googleads.v11.common.StoreSalesMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreSalesMetadata} m StoreSalesMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StoreSalesMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.third_party_metadata != null && Object.hasOwnProperty.call(m, "third_party_metadata"))
                                    $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata.encode(m.third_party_metadata, w.uint32(26).fork()).ldelim();
                                if (m.loyalty_fraction != null && Object.hasOwnProperty.call(m, "loyalty_fraction"))
                                    w.uint32(41).double(m.loyalty_fraction);
                                if (m.transaction_upload_fraction != null && Object.hasOwnProperty.call(m, "transaction_upload_fraction"))
                                    w.uint32(49).double(m.transaction_upload_fraction);
                                if (m.custom_key != null && Object.hasOwnProperty.call(m, "custom_key"))
                                    w.uint32(58).string(m.custom_key);
                                return w;
                            };
    
                            /**
                             * Encodes the specified StoreSalesMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.StoreSalesMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreSalesMetadata} message StoreSalesMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StoreSalesMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StoreSalesMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.StoreSalesMetadata} StoreSalesMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StoreSalesMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.StoreSalesMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.loyalty_fraction = r.double();
                                        break;
                                    case 6:
                                        m.transaction_upload_fraction = r.double();
                                        break;
                                    case 7:
                                        m.custom_key = r.string();
                                        break;
                                    case 3:
                                        m.third_party_metadata = $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StoreSalesMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.StoreSalesMetadata} StoreSalesMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StoreSalesMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StoreSalesMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StoreSalesMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.loyalty_fraction != null && m.hasOwnProperty("loyalty_fraction")) {
                                    p._loyalty_fraction = 1;
                                    if (typeof m.loyalty_fraction !== "number")
                                        return "loyalty_fraction: number expected";
                                }
                                if (m.transaction_upload_fraction != null && m.hasOwnProperty("transaction_upload_fraction")) {
                                    p._transaction_upload_fraction = 1;
                                    if (typeof m.transaction_upload_fraction !== "number")
                                        return "transaction_upload_fraction: number expected";
                                }
                                if (m.custom_key != null && m.hasOwnProperty("custom_key")) {
                                    p._custom_key = 1;
                                    if (!$util.isString(m.custom_key))
                                        return "custom_key: string expected";
                                }
                                if (m.third_party_metadata != null && m.hasOwnProperty("third_party_metadata")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata.verify(m.third_party_metadata);
                                        if (e)
                                            return "third_party_metadata." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a StoreSalesMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.StoreSalesMetadata} StoreSalesMetadata
                             */
                            StoreSalesMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.StoreSalesMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.StoreSalesMetadata();
                                if (d.loyalty_fraction != null) {
                                    m.loyalty_fraction = Number(d.loyalty_fraction);
                                }
                                if (d.transaction_upload_fraction != null) {
                                    m.transaction_upload_fraction = Number(d.transaction_upload_fraction);
                                }
                                if (d.custom_key != null) {
                                    m.custom_key = String(d.custom_key);
                                }
                                if (d.third_party_metadata != null) {
                                    if (typeof d.third_party_metadata !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.StoreSalesMetadata.third_party_metadata: object expected");
                                    m.third_party_metadata = $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata.fromObject(d.third_party_metadata);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a StoreSalesMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.StoreSalesMetadata} m StoreSalesMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StoreSalesMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.third_party_metadata = null;
                                }
                                if (m.third_party_metadata != null && m.hasOwnProperty("third_party_metadata")) {
                                    d.third_party_metadata = $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata.toObject(m.third_party_metadata, o);
                                }
                                if (m.loyalty_fraction != null && m.hasOwnProperty("loyalty_fraction")) {
                                    d.loyalty_fraction = o.json && !isFinite(m.loyalty_fraction) ? String(m.loyalty_fraction) : m.loyalty_fraction;
                                    if (o.oneofs)
                                        d._loyalty_fraction = "loyalty_fraction";
                                }
                                if (m.transaction_upload_fraction != null && m.hasOwnProperty("transaction_upload_fraction")) {
                                    d.transaction_upload_fraction = o.json && !isFinite(m.transaction_upload_fraction) ? String(m.transaction_upload_fraction) : m.transaction_upload_fraction;
                                    if (o.oneofs)
                                        d._transaction_upload_fraction = "transaction_upload_fraction";
                                }
                                if (m.custom_key != null && m.hasOwnProperty("custom_key")) {
                                    d.custom_key = m.custom_key;
                                    if (o.oneofs)
                                        d._custom_key = "custom_key";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this StoreSalesMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.StoreSalesMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StoreSalesMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return StoreSalesMetadata;
                        })();
    
                        common.StoreSalesThirdPartyMetadata = (function() {
    
                            /**
                             * Properties of a StoreSalesThirdPartyMetadata.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IStoreSalesThirdPartyMetadata
                             * @property {string|null} [advertiser_upload_date_time] StoreSalesThirdPartyMetadata advertiser_upload_date_time
                             * @property {number|null} [valid_transaction_fraction] StoreSalesThirdPartyMetadata valid_transaction_fraction
                             * @property {number|null} [partner_match_fraction] StoreSalesThirdPartyMetadata partner_match_fraction
                             * @property {number|null} [partner_upload_fraction] StoreSalesThirdPartyMetadata partner_upload_fraction
                             * @property {string|null} [bridge_map_version_id] StoreSalesThirdPartyMetadata bridge_map_version_id
                             * @property {number|null} [partner_id] StoreSalesThirdPartyMetadata partner_id
                             */
    
                            /**
                             * Constructs a new StoreSalesThirdPartyMetadata.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a StoreSalesThirdPartyMetadata.
                             * @implements IStoreSalesThirdPartyMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IStoreSalesThirdPartyMetadata=} [p] Properties to set
                             */
                            function StoreSalesThirdPartyMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * StoreSalesThirdPartyMetadata advertiser_upload_date_time.
                             * @member {string|null|undefined} advertiser_upload_date_time
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            StoreSalesThirdPartyMetadata.prototype.advertiser_upload_date_time = null;
    
                            /**
                             * StoreSalesThirdPartyMetadata valid_transaction_fraction.
                             * @member {number|null|undefined} valid_transaction_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            StoreSalesThirdPartyMetadata.prototype.valid_transaction_fraction = null;
    
                            /**
                             * StoreSalesThirdPartyMetadata partner_match_fraction.
                             * @member {number|null|undefined} partner_match_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            StoreSalesThirdPartyMetadata.prototype.partner_match_fraction = null;
    
                            /**
                             * StoreSalesThirdPartyMetadata partner_upload_fraction.
                             * @member {number|null|undefined} partner_upload_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            StoreSalesThirdPartyMetadata.prototype.partner_upload_fraction = null;
    
                            /**
                             * StoreSalesThirdPartyMetadata bridge_map_version_id.
                             * @member {string|null|undefined} bridge_map_version_id
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            StoreSalesThirdPartyMetadata.prototype.bridge_map_version_id = null;
    
                            /**
                             * StoreSalesThirdPartyMetadata partner_id.
                             * @member {number|null|undefined} partner_id
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            StoreSalesThirdPartyMetadata.prototype.partner_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * StoreSalesThirdPartyMetadata _advertiser_upload_date_time.
                             * @member {"advertiser_upload_date_time"|undefined} _advertiser_upload_date_time
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesThirdPartyMetadata.prototype, "_advertiser_upload_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["advertiser_upload_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * StoreSalesThirdPartyMetadata _valid_transaction_fraction.
                             * @member {"valid_transaction_fraction"|undefined} _valid_transaction_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesThirdPartyMetadata.prototype, "_valid_transaction_fraction", {
                                get: $util.oneOfGetter($oneOfFields = ["valid_transaction_fraction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * StoreSalesThirdPartyMetadata _partner_match_fraction.
                             * @member {"partner_match_fraction"|undefined} _partner_match_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesThirdPartyMetadata.prototype, "_partner_match_fraction", {
                                get: $util.oneOfGetter($oneOfFields = ["partner_match_fraction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * StoreSalesThirdPartyMetadata _partner_upload_fraction.
                             * @member {"partner_upload_fraction"|undefined} _partner_upload_fraction
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesThirdPartyMetadata.prototype, "_partner_upload_fraction", {
                                get: $util.oneOfGetter($oneOfFields = ["partner_upload_fraction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * StoreSalesThirdPartyMetadata _bridge_map_version_id.
                             * @member {"bridge_map_version_id"|undefined} _bridge_map_version_id
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesThirdPartyMetadata.prototype, "_bridge_map_version_id", {
                                get: $util.oneOfGetter($oneOfFields = ["bridge_map_version_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * StoreSalesThirdPartyMetadata _partner_id.
                             * @member {"partner_id"|undefined} _partner_id
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             */
                            Object.defineProperty(StoreSalesThirdPartyMetadata.prototype, "_partner_id", {
                                get: $util.oneOfGetter($oneOfFields = ["partner_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new StoreSalesThirdPartyMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreSalesThirdPartyMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata} StoreSalesThirdPartyMetadata instance
                             */
                            StoreSalesThirdPartyMetadata.create = function create(properties) {
                                return new StoreSalesThirdPartyMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified StoreSalesThirdPartyMetadata message. Does not implicitly {@link google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreSalesThirdPartyMetadata} m StoreSalesThirdPartyMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StoreSalesThirdPartyMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.advertiser_upload_date_time != null && Object.hasOwnProperty.call(m, "advertiser_upload_date_time"))
                                    w.uint32(58).string(m.advertiser_upload_date_time);
                                if (m.valid_transaction_fraction != null && Object.hasOwnProperty.call(m, "valid_transaction_fraction"))
                                    w.uint32(65).double(m.valid_transaction_fraction);
                                if (m.partner_match_fraction != null && Object.hasOwnProperty.call(m, "partner_match_fraction"))
                                    w.uint32(73).double(m.partner_match_fraction);
                                if (m.partner_upload_fraction != null && Object.hasOwnProperty.call(m, "partner_upload_fraction"))
                                    w.uint32(81).double(m.partner_upload_fraction);
                                if (m.bridge_map_version_id != null && Object.hasOwnProperty.call(m, "bridge_map_version_id"))
                                    w.uint32(90).string(m.bridge_map_version_id);
                                if (m.partner_id != null && Object.hasOwnProperty.call(m, "partner_id"))
                                    w.uint32(96).int64(m.partner_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified StoreSalesThirdPartyMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.IStoreSalesThirdPartyMetadata} message StoreSalesThirdPartyMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StoreSalesThirdPartyMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StoreSalesThirdPartyMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata} StoreSalesThirdPartyMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StoreSalesThirdPartyMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 7:
                                        m.advertiser_upload_date_time = r.string();
                                        break;
                                    case 8:
                                        m.valid_transaction_fraction = r.double();
                                        break;
                                    case 9:
                                        m.partner_match_fraction = r.double();
                                        break;
                                    case 10:
                                        m.partner_upload_fraction = r.double();
                                        break;
                                    case 11:
                                        m.bridge_map_version_id = r.string();
                                        break;
                                    case 12:
                                        m.partner_id = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StoreSalesThirdPartyMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata} StoreSalesThirdPartyMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StoreSalesThirdPartyMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StoreSalesThirdPartyMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StoreSalesThirdPartyMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.advertiser_upload_date_time != null && m.hasOwnProperty("advertiser_upload_date_time")) {
                                    p._advertiser_upload_date_time = 1;
                                    if (!$util.isString(m.advertiser_upload_date_time))
                                        return "advertiser_upload_date_time: string expected";
                                }
                                if (m.valid_transaction_fraction != null && m.hasOwnProperty("valid_transaction_fraction")) {
                                    p._valid_transaction_fraction = 1;
                                    if (typeof m.valid_transaction_fraction !== "number")
                                        return "valid_transaction_fraction: number expected";
                                }
                                if (m.partner_match_fraction != null && m.hasOwnProperty("partner_match_fraction")) {
                                    p._partner_match_fraction = 1;
                                    if (typeof m.partner_match_fraction !== "number")
                                        return "partner_match_fraction: number expected";
                                }
                                if (m.partner_upload_fraction != null && m.hasOwnProperty("partner_upload_fraction")) {
                                    p._partner_upload_fraction = 1;
                                    if (typeof m.partner_upload_fraction !== "number")
                                        return "partner_upload_fraction: number expected";
                                }
                                if (m.bridge_map_version_id != null && m.hasOwnProperty("bridge_map_version_id")) {
                                    p._bridge_map_version_id = 1;
                                    if (!$util.isString(m.bridge_map_version_id))
                                        return "bridge_map_version_id: string expected";
                                }
                                if (m.partner_id != null && m.hasOwnProperty("partner_id")) {
                                    p._partner_id = 1;
                                    if (!$util.isInteger(m.partner_id) && !(m.partner_id && $util.isInteger(m.partner_id.low) && $util.isInteger(m.partner_id.high)))
                                        return "partner_id: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a StoreSalesThirdPartyMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata} StoreSalesThirdPartyMetadata
                             */
                            StoreSalesThirdPartyMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata();
                                if (d.advertiser_upload_date_time != null) {
                                    m.advertiser_upload_date_time = String(d.advertiser_upload_date_time);
                                }
                                if (d.valid_transaction_fraction != null) {
                                    m.valid_transaction_fraction = Number(d.valid_transaction_fraction);
                                }
                                if (d.partner_match_fraction != null) {
                                    m.partner_match_fraction = Number(d.partner_match_fraction);
                                }
                                if (d.partner_upload_fraction != null) {
                                    m.partner_upload_fraction = Number(d.partner_upload_fraction);
                                }
                                if (d.bridge_map_version_id != null) {
                                    m.bridge_map_version_id = String(d.bridge_map_version_id);
                                }
                                if (d.partner_id != null) {
                                    if ($util.Long)
                                        (m.partner_id = $util.Long.fromValue(d.partner_id)).unsigned = false;
                                    else if (typeof d.partner_id === "string")
                                        m.partner_id = parseInt(d.partner_id, 10);
                                    else if (typeof d.partner_id === "number")
                                        m.partner_id = d.partner_id;
                                    else if (typeof d.partner_id === "object")
                                        m.partner_id = new $util.LongBits(d.partner_id.low >>> 0, d.partner_id.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a StoreSalesThirdPartyMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @static
                             * @param {google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata} m StoreSalesThirdPartyMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StoreSalesThirdPartyMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.advertiser_upload_date_time != null && m.hasOwnProperty("advertiser_upload_date_time")) {
                                    d.advertiser_upload_date_time = m.advertiser_upload_date_time;
                                    if (o.oneofs)
                                        d._advertiser_upload_date_time = "advertiser_upload_date_time";
                                }
                                if (m.valid_transaction_fraction != null && m.hasOwnProperty("valid_transaction_fraction")) {
                                    d.valid_transaction_fraction = o.json && !isFinite(m.valid_transaction_fraction) ? String(m.valid_transaction_fraction) : m.valid_transaction_fraction;
                                    if (o.oneofs)
                                        d._valid_transaction_fraction = "valid_transaction_fraction";
                                }
                                if (m.partner_match_fraction != null && m.hasOwnProperty("partner_match_fraction")) {
                                    d.partner_match_fraction = o.json && !isFinite(m.partner_match_fraction) ? String(m.partner_match_fraction) : m.partner_match_fraction;
                                    if (o.oneofs)
                                        d._partner_match_fraction = "partner_match_fraction";
                                }
                                if (m.partner_upload_fraction != null && m.hasOwnProperty("partner_upload_fraction")) {
                                    d.partner_upload_fraction = o.json && !isFinite(m.partner_upload_fraction) ? String(m.partner_upload_fraction) : m.partner_upload_fraction;
                                    if (o.oneofs)
                                        d._partner_upload_fraction = "partner_upload_fraction";
                                }
                                if (m.bridge_map_version_id != null && m.hasOwnProperty("bridge_map_version_id")) {
                                    d.bridge_map_version_id = m.bridge_map_version_id;
                                    if (o.oneofs)
                                        d._bridge_map_version_id = "bridge_map_version_id";
                                }
                                if (m.partner_id != null && m.hasOwnProperty("partner_id")) {
                                    if (typeof m.partner_id === "number")
                                        d.partner_id = o.longs === String ? String(m.partner_id) : m.partner_id;
                                    else
                                        d.partner_id = o.longs === String ? $util.Long.prototype.toString.call(m.partner_id) : o.longs === Number ? new $util.LongBits(m.partner_id.low >>> 0, m.partner_id.high >>> 0).toNumber() : m.partner_id;
                                    if (o.oneofs)
                                        d._partner_id = "partner_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this StoreSalesThirdPartyMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.StoreSalesThirdPartyMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StoreSalesThirdPartyMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return StoreSalesThirdPartyMetadata;
                        })();
    
                        common.PolicySummary = (function() {
    
                            /**
                             * Properties of a PolicySummary.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPolicySummary
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>|null} [policy_topic_entries] PolicySummary policy_topic_entries
                             * @property {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus|null} [review_status] PolicySummary review_status
                             * @property {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus|null} [approval_status] PolicySummary approval_status
                             */
    
                            /**
                             * Constructs a new PolicySummary.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PolicySummary.
                             * @implements IPolicySummary
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPolicySummary=} [p] Properties to set
                             */
                            function PolicySummary(p) {
                                this.policy_topic_entries = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicySummary policy_topic_entries.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>} policy_topic_entries
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @instance
                             */
                            PolicySummary.prototype.policy_topic_entries = $util.emptyArray;
    
                            /**
                             * PolicySummary review_status.
                             * @member {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus} review_status
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @instance
                             */
                            PolicySummary.prototype.review_status = 0;
    
                            /**
                             * PolicySummary approval_status.
                             * @member {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus} approval_status
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @instance
                             */
                            PolicySummary.prototype.approval_status = 0;
    
                            /**
                             * Creates a new PolicySummary instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicySummary=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PolicySummary} PolicySummary instance
                             */
                            PolicySummary.create = function create(properties) {
                                return new PolicySummary(properties);
                            };
    
                            /**
                             * Encodes the specified PolicySummary message. Does not implicitly {@link google.ads.googleads.v11.common.PolicySummary.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicySummary} m PolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicySummary.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.policy_topic_entries != null && m.policy_topic_entries.length) {
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEntry.encode(m.policy_topic_entries[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.review_status != null && Object.hasOwnProperty.call(m, "review_status"))
                                    w.uint32(16).int32(m.review_status);
                                if (m.approval_status != null && Object.hasOwnProperty.call(m, "approval_status"))
                                    w.uint32(24).int32(m.approval_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicySummary message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PolicySummary.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.IPolicySummary} message PolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicySummary.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicySummary message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PolicySummary} PolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicySummary.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PolicySummary();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.policy_topic_entries && m.policy_topic_entries.length))
                                            m.policy_topic_entries = [];
                                        m.policy_topic_entries.push($root.google.ads.googleads.v11.common.PolicyTopicEntry.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.review_status = r.int32();
                                        break;
                                    case 3:
                                        m.approval_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicySummary message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PolicySummary} PolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicySummary.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicySummary message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicySummary.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.policy_topic_entries != null && m.hasOwnProperty("policy_topic_entries")) {
                                    if (!Array.isArray(m.policy_topic_entries))
                                        return "policy_topic_entries: array expected";
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEntry.verify(m.policy_topic_entries[i]);
                                            if (e)
                                                return "policy_topic_entries." + e;
                                        }
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    switch (m.review_status) {
                                    default:
                                        return "review_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    switch (m.approval_status) {
                                    default:
                                        return "approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicySummary message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PolicySummary} PolicySummary
                             */
                            PolicySummary.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PolicySummary)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PolicySummary();
                                if (d.policy_topic_entries) {
                                    if (!Array.isArray(d.policy_topic_entries))
                                        throw TypeError(".google.ads.googleads.v11.common.PolicySummary.policy_topic_entries: array expected");
                                    m.policy_topic_entries = [];
                                    for (var i = 0; i < d.policy_topic_entries.length; ++i) {
                                        if (typeof d.policy_topic_entries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PolicySummary.policy_topic_entries: object expected");
                                        m.policy_topic_entries[i] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.fromObject(d.policy_topic_entries[i]);
                                    }
                                }
                                switch (d.review_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.review_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.review_status = 1;
                                    break;
                                case "REVIEW_IN_PROGRESS":
                                case 2:
                                    m.review_status = 2;
                                    break;
                                case "REVIEWED":
                                case 3:
                                    m.review_status = 3;
                                    break;
                                case "UNDER_APPEAL":
                                case 4:
                                    m.review_status = 4;
                                    break;
                                case "ELIGIBLE_MAY_SERVE":
                                case 5:
                                    m.review_status = 5;
                                    break;
                                }
                                switch (d.approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approval_status = 1;
                                    break;
                                case "DISAPPROVED":
                                case 2:
                                    m.approval_status = 2;
                                    break;
                                case "APPROVED_LIMITED":
                                case 3:
                                    m.approval_status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.approval_status = 4;
                                    break;
                                case "AREA_OF_INTEREST_ONLY":
                                case 5:
                                    m.approval_status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicySummary message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.common.PolicySummary} m PolicySummary
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicySummary.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.policy_topic_entries = [];
                                }
                                if (o.defaults) {
                                    d.review_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.policy_topic_entries && m.policy_topic_entries.length) {
                                    d.policy_topic_entries = [];
                                    for (var j = 0; j < m.policy_topic_entries.length; ++j) {
                                        d.policy_topic_entries[j] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.toObject(m.policy_topic_entries[j], o);
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    d.review_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus[m.review_status] : m.review_status;
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    d.approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus[m.approval_status] : m.approval_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicySummary to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PolicySummary
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicySummary.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PolicySummary;
                        })();
    
                        common.RealTimeBiddingSetting = (function() {
    
                            /**
                             * Properties of a RealTimeBiddingSetting.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IRealTimeBiddingSetting
                             * @property {boolean|null} [opt_in] RealTimeBiddingSetting opt_in
                             */
    
                            /**
                             * Constructs a new RealTimeBiddingSetting.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a RealTimeBiddingSetting.
                             * @implements IRealTimeBiddingSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IRealTimeBiddingSetting=} [p] Properties to set
                             */
                            function RealTimeBiddingSetting(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RealTimeBiddingSetting opt_in.
                             * @member {boolean|null|undefined} opt_in
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @instance
                             */
                            RealTimeBiddingSetting.prototype.opt_in = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * RealTimeBiddingSetting _opt_in.
                             * @member {"opt_in"|undefined} _opt_in
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @instance
                             */
                            Object.defineProperty(RealTimeBiddingSetting.prototype, "_opt_in", {
                                get: $util.oneOfGetter($oneOfFields = ["opt_in"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new RealTimeBiddingSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.IRealTimeBiddingSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.RealTimeBiddingSetting} RealTimeBiddingSetting instance
                             */
                            RealTimeBiddingSetting.create = function create(properties) {
                                return new RealTimeBiddingSetting(properties);
                            };
    
                            /**
                             * Encodes the specified RealTimeBiddingSetting message. Does not implicitly {@link google.ads.googleads.v11.common.RealTimeBiddingSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.IRealTimeBiddingSetting} m RealTimeBiddingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RealTimeBiddingSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.opt_in != null && Object.hasOwnProperty.call(m, "opt_in"))
                                    w.uint32(16).bool(m.opt_in);
                                return w;
                            };
    
                            /**
                             * Encodes the specified RealTimeBiddingSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.RealTimeBiddingSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.IRealTimeBiddingSetting} message RealTimeBiddingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RealTimeBiddingSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RealTimeBiddingSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.RealTimeBiddingSetting} RealTimeBiddingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RealTimeBiddingSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.RealTimeBiddingSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.opt_in = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RealTimeBiddingSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.RealTimeBiddingSetting} RealTimeBiddingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RealTimeBiddingSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RealTimeBiddingSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RealTimeBiddingSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.opt_in != null && m.hasOwnProperty("opt_in")) {
                                    p._opt_in = 1;
                                    if (typeof m.opt_in !== "boolean")
                                        return "opt_in: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RealTimeBiddingSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.RealTimeBiddingSetting} RealTimeBiddingSetting
                             */
                            RealTimeBiddingSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.RealTimeBiddingSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.RealTimeBiddingSetting();
                                if (d.opt_in != null) {
                                    m.opt_in = Boolean(d.opt_in);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RealTimeBiddingSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.RealTimeBiddingSetting} m RealTimeBiddingSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RealTimeBiddingSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.opt_in != null && m.hasOwnProperty("opt_in")) {
                                    d.opt_in = m.opt_in;
                                    if (o.oneofs)
                                        d._opt_in = "opt_in";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RealTimeBiddingSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.RealTimeBiddingSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RealTimeBiddingSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RealTimeBiddingSetting;
                        })();
    
                        common.Segments = (function() {
    
                            /**
                             * Properties of a Segments.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ISegments
                             * @property {google.ads.googleads.v11.enums.AdDestinationTypeEnum.AdDestinationType|null} [ad_destination_type] Segments ad_destination_type
                             * @property {google.ads.googleads.v11.enums.AdNetworkTypeEnum.AdNetworkType|null} [ad_network_type] Segments ad_network_type
                             * @property {string|null} [auction_insight_domain] Segments auction_insight_domain
                             * @property {google.ads.googleads.v11.common.IBudgetCampaignAssociationStatus|null} [budget_campaign_association_status] Segments budget_campaign_association_status
                             * @property {google.ads.googleads.v11.enums.ClickTypeEnum.ClickType|null} [click_type] Segments click_type
                             * @property {string|null} [conversion_action] Segments conversion_action
                             * @property {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory|null} [conversion_action_category] Segments conversion_action_category
                             * @property {string|null} [conversion_action_name] Segments conversion_action_name
                             * @property {boolean|null} [conversion_adjustment] Segments conversion_adjustment
                             * @property {google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum.ConversionAttributionEventType|null} [conversion_attribution_event_type] Segments conversion_attribution_event_type
                             * @property {google.ads.googleads.v11.enums.ConversionLagBucketEnum.ConversionLagBucket|null} [conversion_lag_bucket] Segments conversion_lag_bucket
                             * @property {google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket|null} [conversion_or_adjustment_lag_bucket] Segments conversion_or_adjustment_lag_bucket
                             * @property {string|null} [date] Segments date
                             * @property {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek|null} [day_of_week] Segments day_of_week
                             * @property {google.ads.googleads.v11.enums.DeviceEnum.Device|null} [device] Segments device
                             * @property {google.ads.googleads.v11.enums.ExternalConversionSourceEnum.ExternalConversionSource|null} [external_conversion_source] Segments external_conversion_source
                             * @property {string|null} [geo_target_airport] Segments geo_target_airport
                             * @property {string|null} [geo_target_canton] Segments geo_target_canton
                             * @property {string|null} [geo_target_city] Segments geo_target_city
                             * @property {string|null} [geo_target_country] Segments geo_target_country
                             * @property {string|null} [geo_target_county] Segments geo_target_county
                             * @property {string|null} [geo_target_district] Segments geo_target_district
                             * @property {string|null} [geo_target_metro] Segments geo_target_metro
                             * @property {string|null} [geo_target_most_specific_location] Segments geo_target_most_specific_location
                             * @property {string|null} [geo_target_postal_code] Segments geo_target_postal_code
                             * @property {string|null} [geo_target_province] Segments geo_target_province
                             * @property {string|null} [geo_target_region] Segments geo_target_region
                             * @property {string|null} [geo_target_state] Segments geo_target_state
                             * @property {number|null} [hotel_booking_window_days] Segments hotel_booking_window_days
                             * @property {number|null} [hotel_center_id] Segments hotel_center_id
                             * @property {string|null} [hotel_check_in_date] Segments hotel_check_in_date
                             * @property {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek|null} [hotel_check_in_day_of_week] Segments hotel_check_in_day_of_week
                             * @property {string|null} [hotel_city] Segments hotel_city
                             * @property {number|null} [hotel_class] Segments hotel_class
                             * @property {string|null} [hotel_country] Segments hotel_country
                             * @property {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.HotelDateSelectionType|null} [hotel_date_selection_type] Segments hotel_date_selection_type
                             * @property {number|null} [hotel_length_of_stay] Segments hotel_length_of_stay
                             * @property {string|null} [hotel_rate_rule_id] Segments hotel_rate_rule_id
                             * @property {google.ads.googleads.v11.enums.HotelRateTypeEnum.HotelRateType|null} [hotel_rate_type] Segments hotel_rate_type
                             * @property {google.ads.googleads.v11.enums.HotelPriceBucketEnum.HotelPriceBucket|null} [hotel_price_bucket] Segments hotel_price_bucket
                             * @property {string|null} [hotel_state] Segments hotel_state
                             * @property {number|null} [hour] Segments hour
                             * @property {boolean|null} [interaction_on_this_extension] Segments interaction_on_this_extension
                             * @property {google.ads.googleads.v11.common.IKeyword|null} [keyword] Segments keyword
                             * @property {string|null} [month] Segments month
                             * @property {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear|null} [month_of_year] Segments month_of_year
                             * @property {string|null} [partner_hotel_id] Segments partner_hotel_id
                             * @property {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType|null} [placeholder_type] Segments placeholder_type
                             * @property {number|null} [product_aggregator_id] Segments product_aggregator_id
                             * @property {string|null} [product_bidding_category_level1] Segments product_bidding_category_level1
                             * @property {string|null} [product_bidding_category_level2] Segments product_bidding_category_level2
                             * @property {string|null} [product_bidding_category_level3] Segments product_bidding_category_level3
                             * @property {string|null} [product_bidding_category_level4] Segments product_bidding_category_level4
                             * @property {string|null} [product_bidding_category_level5] Segments product_bidding_category_level5
                             * @property {string|null} [product_brand] Segments product_brand
                             * @property {google.ads.googleads.v11.enums.ProductChannelEnum.ProductChannel|null} [product_channel] Segments product_channel
                             * @property {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.ProductChannelExclusivity|null} [product_channel_exclusivity] Segments product_channel_exclusivity
                             * @property {google.ads.googleads.v11.enums.ProductConditionEnum.ProductCondition|null} [product_condition] Segments product_condition
                             * @property {string|null} [product_country] Segments product_country
                             * @property {string|null} [product_custom_attribute0] Segments product_custom_attribute0
                             * @property {string|null} [product_custom_attribute1] Segments product_custom_attribute1
                             * @property {string|null} [product_custom_attribute2] Segments product_custom_attribute2
                             * @property {string|null} [product_custom_attribute3] Segments product_custom_attribute3
                             * @property {string|null} [product_custom_attribute4] Segments product_custom_attribute4
                             * @property {string|null} [product_item_id] Segments product_item_id
                             * @property {string|null} [product_language] Segments product_language
                             * @property {number|null} [product_merchant_id] Segments product_merchant_id
                             * @property {string|null} [product_store_id] Segments product_store_id
                             * @property {string|null} [product_title] Segments product_title
                             * @property {string|null} [product_type_l1] Segments product_type_l1
                             * @property {string|null} [product_type_l2] Segments product_type_l2
                             * @property {string|null} [product_type_l3] Segments product_type_l3
                             * @property {string|null} [product_type_l4] Segments product_type_l4
                             * @property {string|null} [product_type_l5] Segments product_type_l5
                             * @property {string|null} [quarter] Segments quarter
                             * @property {google.ads.googleads.v11.enums.RecommendationTypeEnum.RecommendationType|null} [recommendation_type] Segments recommendation_type
                             * @property {google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType|null} [search_engine_results_page_type] Segments search_engine_results_page_type
                             * @property {google.ads.googleads.v11.enums.SearchTermMatchTypeEnum.SearchTermMatchType|null} [search_term_match_type] Segments search_term_match_type
                             * @property {google.ads.googleads.v11.enums.SlotEnum.Slot|null} [slot] Segments slot
                             * @property {google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension|null} [conversion_value_rule_primary_dimension] Segments conversion_value_rule_primary_dimension
                             * @property {string|null} [webpage] Segments webpage
                             * @property {string|null} [week] Segments week
                             * @property {number|null} [year] Segments year
                             * @property {number|null} [sk_ad_network_conversion_value] Segments sk_ad_network_conversion_value
                             * @property {google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum.SkAdNetworkUserType|null} [sk_ad_network_user_type] Segments sk_ad_network_user_type
                             * @property {google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType|null} [sk_ad_network_ad_event_type] Segments sk_ad_network_ad_event_type
                             * @property {google.ads.googleads.v11.common.ISkAdNetworkSourceApp|null} [sk_ad_network_source_app] Segments sk_ad_network_source_app
                             * @property {google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit|null} [sk_ad_network_attribution_credit] Segments sk_ad_network_attribution_credit
                             * @property {google.ads.googleads.v11.common.IAssetInteractionTarget|null} [asset_interaction_target] Segments asset_interaction_target
                             */
    
                            /**
                             * Constructs a new Segments.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a Segments.
                             * @implements ISegments
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ISegments=} [p] Properties to set
                             */
                            function Segments(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Segments ad_destination_type.
                             * @member {google.ads.googleads.v11.enums.AdDestinationTypeEnum.AdDestinationType} ad_destination_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.ad_destination_type = 0;
    
                            /**
                             * Segments ad_network_type.
                             * @member {google.ads.googleads.v11.enums.AdNetworkTypeEnum.AdNetworkType} ad_network_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.ad_network_type = 0;
    
                            /**
                             * Segments auction_insight_domain.
                             * @member {string|null|undefined} auction_insight_domain
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.auction_insight_domain = null;
    
                            /**
                             * Segments budget_campaign_association_status.
                             * @member {google.ads.googleads.v11.common.IBudgetCampaignAssociationStatus|null|undefined} budget_campaign_association_status
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.budget_campaign_association_status = null;
    
                            /**
                             * Segments click_type.
                             * @member {google.ads.googleads.v11.enums.ClickTypeEnum.ClickType} click_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.click_type = 0;
    
                            /**
                             * Segments conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_action = null;
    
                            /**
                             * Segments conversion_action_category.
                             * @member {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory} conversion_action_category
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_action_category = 0;
    
                            /**
                             * Segments conversion_action_name.
                             * @member {string|null|undefined} conversion_action_name
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_action_name = null;
    
                            /**
                             * Segments conversion_adjustment.
                             * @member {boolean|null|undefined} conversion_adjustment
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_adjustment = null;
    
                            /**
                             * Segments conversion_attribution_event_type.
                             * @member {google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum.ConversionAttributionEventType} conversion_attribution_event_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_attribution_event_type = 0;
    
                            /**
                             * Segments conversion_lag_bucket.
                             * @member {google.ads.googleads.v11.enums.ConversionLagBucketEnum.ConversionLagBucket} conversion_lag_bucket
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_lag_bucket = 0;
    
                            /**
                             * Segments conversion_or_adjustment_lag_bucket.
                             * @member {google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket} conversion_or_adjustment_lag_bucket
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_or_adjustment_lag_bucket = 0;
    
                            /**
                             * Segments date.
                             * @member {string|null|undefined} date
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.date = null;
    
                            /**
                             * Segments day_of_week.
                             * @member {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek} day_of_week
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.day_of_week = 0;
    
                            /**
                             * Segments device.
                             * @member {google.ads.googleads.v11.enums.DeviceEnum.Device} device
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.device = 0;
    
                            /**
                             * Segments external_conversion_source.
                             * @member {google.ads.googleads.v11.enums.ExternalConversionSourceEnum.ExternalConversionSource} external_conversion_source
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.external_conversion_source = 0;
    
                            /**
                             * Segments geo_target_airport.
                             * @member {string|null|undefined} geo_target_airport
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_airport = null;
    
                            /**
                             * Segments geo_target_canton.
                             * @member {string|null|undefined} geo_target_canton
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_canton = null;
    
                            /**
                             * Segments geo_target_city.
                             * @member {string|null|undefined} geo_target_city
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_city = null;
    
                            /**
                             * Segments geo_target_country.
                             * @member {string|null|undefined} geo_target_country
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_country = null;
    
                            /**
                             * Segments geo_target_county.
                             * @member {string|null|undefined} geo_target_county
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_county = null;
    
                            /**
                             * Segments geo_target_district.
                             * @member {string|null|undefined} geo_target_district
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_district = null;
    
                            /**
                             * Segments geo_target_metro.
                             * @member {string|null|undefined} geo_target_metro
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_metro = null;
    
                            /**
                             * Segments geo_target_most_specific_location.
                             * @member {string|null|undefined} geo_target_most_specific_location
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_most_specific_location = null;
    
                            /**
                             * Segments geo_target_postal_code.
                             * @member {string|null|undefined} geo_target_postal_code
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_postal_code = null;
    
                            /**
                             * Segments geo_target_province.
                             * @member {string|null|undefined} geo_target_province
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_province = null;
    
                            /**
                             * Segments geo_target_region.
                             * @member {string|null|undefined} geo_target_region
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_region = null;
    
                            /**
                             * Segments geo_target_state.
                             * @member {string|null|undefined} geo_target_state
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.geo_target_state = null;
    
                            /**
                             * Segments hotel_booking_window_days.
                             * @member {number|null|undefined} hotel_booking_window_days
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_booking_window_days = null;
    
                            /**
                             * Segments hotel_center_id.
                             * @member {number|null|undefined} hotel_center_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_center_id = null;
    
                            /**
                             * Segments hotel_check_in_date.
                             * @member {string|null|undefined} hotel_check_in_date
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_check_in_date = null;
    
                            /**
                             * Segments hotel_check_in_day_of_week.
                             * @member {google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek} hotel_check_in_day_of_week
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_check_in_day_of_week = 0;
    
                            /**
                             * Segments hotel_city.
                             * @member {string|null|undefined} hotel_city
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_city = null;
    
                            /**
                             * Segments hotel_class.
                             * @member {number|null|undefined} hotel_class
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_class = null;
    
                            /**
                             * Segments hotel_country.
                             * @member {string|null|undefined} hotel_country
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_country = null;
    
                            /**
                             * Segments hotel_date_selection_type.
                             * @member {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.HotelDateSelectionType} hotel_date_selection_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_date_selection_type = 0;
    
                            /**
                             * Segments hotel_length_of_stay.
                             * @member {number|null|undefined} hotel_length_of_stay
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_length_of_stay = null;
    
                            /**
                             * Segments hotel_rate_rule_id.
                             * @member {string|null|undefined} hotel_rate_rule_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_rate_rule_id = null;
    
                            /**
                             * Segments hotel_rate_type.
                             * @member {google.ads.googleads.v11.enums.HotelRateTypeEnum.HotelRateType} hotel_rate_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_rate_type = 0;
    
                            /**
                             * Segments hotel_price_bucket.
                             * @member {google.ads.googleads.v11.enums.HotelPriceBucketEnum.HotelPriceBucket} hotel_price_bucket
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_price_bucket = 0;
    
                            /**
                             * Segments hotel_state.
                             * @member {string|null|undefined} hotel_state
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hotel_state = null;
    
                            /**
                             * Segments hour.
                             * @member {number|null|undefined} hour
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.hour = null;
    
                            /**
                             * Segments interaction_on_this_extension.
                             * @member {boolean|null|undefined} interaction_on_this_extension
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.interaction_on_this_extension = null;
    
                            /**
                             * Segments keyword.
                             * @member {google.ads.googleads.v11.common.IKeyword|null|undefined} keyword
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.keyword = null;
    
                            /**
                             * Segments month.
                             * @member {string|null|undefined} month
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.month = null;
    
                            /**
                             * Segments month_of_year.
                             * @member {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear} month_of_year
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.month_of_year = 0;
    
                            /**
                             * Segments partner_hotel_id.
                             * @member {string|null|undefined} partner_hotel_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.partner_hotel_id = null;
    
                            /**
                             * Segments placeholder_type.
                             * @member {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType} placeholder_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.placeholder_type = 0;
    
                            /**
                             * Segments product_aggregator_id.
                             * @member {number|null|undefined} product_aggregator_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_aggregator_id = null;
    
                            /**
                             * Segments product_bidding_category_level1.
                             * @member {string|null|undefined} product_bidding_category_level1
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_bidding_category_level1 = null;
    
                            /**
                             * Segments product_bidding_category_level2.
                             * @member {string|null|undefined} product_bidding_category_level2
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_bidding_category_level2 = null;
    
                            /**
                             * Segments product_bidding_category_level3.
                             * @member {string|null|undefined} product_bidding_category_level3
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_bidding_category_level3 = null;
    
                            /**
                             * Segments product_bidding_category_level4.
                             * @member {string|null|undefined} product_bidding_category_level4
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_bidding_category_level4 = null;
    
                            /**
                             * Segments product_bidding_category_level5.
                             * @member {string|null|undefined} product_bidding_category_level5
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_bidding_category_level5 = null;
    
                            /**
                             * Segments product_brand.
                             * @member {string|null|undefined} product_brand
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_brand = null;
    
                            /**
                             * Segments product_channel.
                             * @member {google.ads.googleads.v11.enums.ProductChannelEnum.ProductChannel} product_channel
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_channel = 0;
    
                            /**
                             * Segments product_channel_exclusivity.
                             * @member {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.ProductChannelExclusivity} product_channel_exclusivity
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_channel_exclusivity = 0;
    
                            /**
                             * Segments product_condition.
                             * @member {google.ads.googleads.v11.enums.ProductConditionEnum.ProductCondition} product_condition
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_condition = 0;
    
                            /**
                             * Segments product_country.
                             * @member {string|null|undefined} product_country
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_country = null;
    
                            /**
                             * Segments product_custom_attribute0.
                             * @member {string|null|undefined} product_custom_attribute0
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_custom_attribute0 = null;
    
                            /**
                             * Segments product_custom_attribute1.
                             * @member {string|null|undefined} product_custom_attribute1
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_custom_attribute1 = null;
    
                            /**
                             * Segments product_custom_attribute2.
                             * @member {string|null|undefined} product_custom_attribute2
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_custom_attribute2 = null;
    
                            /**
                             * Segments product_custom_attribute3.
                             * @member {string|null|undefined} product_custom_attribute3
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_custom_attribute3 = null;
    
                            /**
                             * Segments product_custom_attribute4.
                             * @member {string|null|undefined} product_custom_attribute4
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_custom_attribute4 = null;
    
                            /**
                             * Segments product_item_id.
                             * @member {string|null|undefined} product_item_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_item_id = null;
    
                            /**
                             * Segments product_language.
                             * @member {string|null|undefined} product_language
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_language = null;
    
                            /**
                             * Segments product_merchant_id.
                             * @member {number|null|undefined} product_merchant_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_merchant_id = null;
    
                            /**
                             * Segments product_store_id.
                             * @member {string|null|undefined} product_store_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_store_id = null;
    
                            /**
                             * Segments product_title.
                             * @member {string|null|undefined} product_title
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_title = null;
    
                            /**
                             * Segments product_type_l1.
                             * @member {string|null|undefined} product_type_l1
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_type_l1 = null;
    
                            /**
                             * Segments product_type_l2.
                             * @member {string|null|undefined} product_type_l2
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_type_l2 = null;
    
                            /**
                             * Segments product_type_l3.
                             * @member {string|null|undefined} product_type_l3
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_type_l3 = null;
    
                            /**
                             * Segments product_type_l4.
                             * @member {string|null|undefined} product_type_l4
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_type_l4 = null;
    
                            /**
                             * Segments product_type_l5.
                             * @member {string|null|undefined} product_type_l5
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.product_type_l5 = null;
    
                            /**
                             * Segments quarter.
                             * @member {string|null|undefined} quarter
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.quarter = null;
    
                            /**
                             * Segments recommendation_type.
                             * @member {google.ads.googleads.v11.enums.RecommendationTypeEnum.RecommendationType} recommendation_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.recommendation_type = 0;
    
                            /**
                             * Segments search_engine_results_page_type.
                             * @member {google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType} search_engine_results_page_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.search_engine_results_page_type = 0;
    
                            /**
                             * Segments search_term_match_type.
                             * @member {google.ads.googleads.v11.enums.SearchTermMatchTypeEnum.SearchTermMatchType} search_term_match_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.search_term_match_type = 0;
    
                            /**
                             * Segments slot.
                             * @member {google.ads.googleads.v11.enums.SlotEnum.Slot} slot
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.slot = 0;
    
                            /**
                             * Segments conversion_value_rule_primary_dimension.
                             * @member {google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension} conversion_value_rule_primary_dimension
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.conversion_value_rule_primary_dimension = 0;
    
                            /**
                             * Segments webpage.
                             * @member {string|null|undefined} webpage
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.webpage = null;
    
                            /**
                             * Segments week.
                             * @member {string|null|undefined} week
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.week = null;
    
                            /**
                             * Segments year.
                             * @member {number|null|undefined} year
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.year = null;
    
                            /**
                             * Segments sk_ad_network_conversion_value.
                             * @member {number|null|undefined} sk_ad_network_conversion_value
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.sk_ad_network_conversion_value = null;
    
                            /**
                             * Segments sk_ad_network_user_type.
                             * @member {google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum.SkAdNetworkUserType} sk_ad_network_user_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.sk_ad_network_user_type = 0;
    
                            /**
                             * Segments sk_ad_network_ad_event_type.
                             * @member {google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType} sk_ad_network_ad_event_type
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.sk_ad_network_ad_event_type = 0;
    
                            /**
                             * Segments sk_ad_network_source_app.
                             * @member {google.ads.googleads.v11.common.ISkAdNetworkSourceApp|null|undefined} sk_ad_network_source_app
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.sk_ad_network_source_app = null;
    
                            /**
                             * Segments sk_ad_network_attribution_credit.
                             * @member {google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit} sk_ad_network_attribution_credit
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.sk_ad_network_attribution_credit = 0;
    
                            /**
                             * Segments asset_interaction_target.
                             * @member {google.ads.googleads.v11.common.IAssetInteractionTarget|null|undefined} asset_interaction_target
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Segments.prototype.asset_interaction_target = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Segments _auction_insight_domain.
                             * @member {"auction_insight_domain"|undefined} _auction_insight_domain
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_auction_insight_domain", {
                                get: $util.oneOfGetter($oneOfFields = ["auction_insight_domain"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _conversion_action_name.
                             * @member {"conversion_action_name"|undefined} _conversion_action_name
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_conversion_action_name", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _conversion_adjustment.
                             * @member {"conversion_adjustment"|undefined} _conversion_adjustment
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_conversion_adjustment", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_adjustment"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _date.
                             * @member {"date"|undefined} _date
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_date", {
                                get: $util.oneOfGetter($oneOfFields = ["date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_airport.
                             * @member {"geo_target_airport"|undefined} _geo_target_airport
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_airport", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_airport"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_canton.
                             * @member {"geo_target_canton"|undefined} _geo_target_canton
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_canton", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_canton"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_city.
                             * @member {"geo_target_city"|undefined} _geo_target_city
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_city", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_city"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_country.
                             * @member {"geo_target_country"|undefined} _geo_target_country
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_country", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_country"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_county.
                             * @member {"geo_target_county"|undefined} _geo_target_county
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_county", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_county"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_district.
                             * @member {"geo_target_district"|undefined} _geo_target_district
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_district", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_district"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_metro.
                             * @member {"geo_target_metro"|undefined} _geo_target_metro
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_metro", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_metro"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_most_specific_location.
                             * @member {"geo_target_most_specific_location"|undefined} _geo_target_most_specific_location
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_most_specific_location", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_most_specific_location"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_postal_code.
                             * @member {"geo_target_postal_code"|undefined} _geo_target_postal_code
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_postal_code", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_postal_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_province.
                             * @member {"geo_target_province"|undefined} _geo_target_province
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_province", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_province"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_region.
                             * @member {"geo_target_region"|undefined} _geo_target_region
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_region", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_region"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _geo_target_state.
                             * @member {"geo_target_state"|undefined} _geo_target_state
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_geo_target_state", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_state"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_booking_window_days.
                             * @member {"hotel_booking_window_days"|undefined} _hotel_booking_window_days
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_booking_window_days", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_booking_window_days"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_center_id.
                             * @member {"hotel_center_id"|undefined} _hotel_center_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_center_id", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_center_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_check_in_date.
                             * @member {"hotel_check_in_date"|undefined} _hotel_check_in_date
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_check_in_date", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_check_in_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_city.
                             * @member {"hotel_city"|undefined} _hotel_city
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_city", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_city"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_class.
                             * @member {"hotel_class"|undefined} _hotel_class
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_class", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_class"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_country.
                             * @member {"hotel_country"|undefined} _hotel_country
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_country", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_country"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_length_of_stay.
                             * @member {"hotel_length_of_stay"|undefined} _hotel_length_of_stay
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_length_of_stay", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_length_of_stay"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_rate_rule_id.
                             * @member {"hotel_rate_rule_id"|undefined} _hotel_rate_rule_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_rate_rule_id", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_rate_rule_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hotel_state.
                             * @member {"hotel_state"|undefined} _hotel_state
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hotel_state", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_state"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _hour.
                             * @member {"hour"|undefined} _hour
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_hour", {
                                get: $util.oneOfGetter($oneOfFields = ["hour"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _interaction_on_this_extension.
                             * @member {"interaction_on_this_extension"|undefined} _interaction_on_this_extension
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_interaction_on_this_extension", {
                                get: $util.oneOfGetter($oneOfFields = ["interaction_on_this_extension"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _month.
                             * @member {"month"|undefined} _month
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_month", {
                                get: $util.oneOfGetter($oneOfFields = ["month"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _partner_hotel_id.
                             * @member {"partner_hotel_id"|undefined} _partner_hotel_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_partner_hotel_id", {
                                get: $util.oneOfGetter($oneOfFields = ["partner_hotel_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_aggregator_id.
                             * @member {"product_aggregator_id"|undefined} _product_aggregator_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_aggregator_id", {
                                get: $util.oneOfGetter($oneOfFields = ["product_aggregator_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_bidding_category_level1.
                             * @member {"product_bidding_category_level1"|undefined} _product_bidding_category_level1
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_bidding_category_level1", {
                                get: $util.oneOfGetter($oneOfFields = ["product_bidding_category_level1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_bidding_category_level2.
                             * @member {"product_bidding_category_level2"|undefined} _product_bidding_category_level2
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_bidding_category_level2", {
                                get: $util.oneOfGetter($oneOfFields = ["product_bidding_category_level2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_bidding_category_level3.
                             * @member {"product_bidding_category_level3"|undefined} _product_bidding_category_level3
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_bidding_category_level3", {
                                get: $util.oneOfGetter($oneOfFields = ["product_bidding_category_level3"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_bidding_category_level4.
                             * @member {"product_bidding_category_level4"|undefined} _product_bidding_category_level4
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_bidding_category_level4", {
                                get: $util.oneOfGetter($oneOfFields = ["product_bidding_category_level4"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_bidding_category_level5.
                             * @member {"product_bidding_category_level5"|undefined} _product_bidding_category_level5
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_bidding_category_level5", {
                                get: $util.oneOfGetter($oneOfFields = ["product_bidding_category_level5"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_brand.
                             * @member {"product_brand"|undefined} _product_brand
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_brand", {
                                get: $util.oneOfGetter($oneOfFields = ["product_brand"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_country.
                             * @member {"product_country"|undefined} _product_country
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_country", {
                                get: $util.oneOfGetter($oneOfFields = ["product_country"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_custom_attribute0.
                             * @member {"product_custom_attribute0"|undefined} _product_custom_attribute0
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_custom_attribute0", {
                                get: $util.oneOfGetter($oneOfFields = ["product_custom_attribute0"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_custom_attribute1.
                             * @member {"product_custom_attribute1"|undefined} _product_custom_attribute1
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_custom_attribute1", {
                                get: $util.oneOfGetter($oneOfFields = ["product_custom_attribute1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_custom_attribute2.
                             * @member {"product_custom_attribute2"|undefined} _product_custom_attribute2
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_custom_attribute2", {
                                get: $util.oneOfGetter($oneOfFields = ["product_custom_attribute2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_custom_attribute3.
                             * @member {"product_custom_attribute3"|undefined} _product_custom_attribute3
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_custom_attribute3", {
                                get: $util.oneOfGetter($oneOfFields = ["product_custom_attribute3"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_custom_attribute4.
                             * @member {"product_custom_attribute4"|undefined} _product_custom_attribute4
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_custom_attribute4", {
                                get: $util.oneOfGetter($oneOfFields = ["product_custom_attribute4"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_item_id.
                             * @member {"product_item_id"|undefined} _product_item_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_item_id", {
                                get: $util.oneOfGetter($oneOfFields = ["product_item_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_language.
                             * @member {"product_language"|undefined} _product_language
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_language", {
                                get: $util.oneOfGetter($oneOfFields = ["product_language"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_merchant_id.
                             * @member {"product_merchant_id"|undefined} _product_merchant_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_merchant_id", {
                                get: $util.oneOfGetter($oneOfFields = ["product_merchant_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_store_id.
                             * @member {"product_store_id"|undefined} _product_store_id
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_store_id", {
                                get: $util.oneOfGetter($oneOfFields = ["product_store_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_title.
                             * @member {"product_title"|undefined} _product_title
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_title", {
                                get: $util.oneOfGetter($oneOfFields = ["product_title"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_type_l1.
                             * @member {"product_type_l1"|undefined} _product_type_l1
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_type_l1", {
                                get: $util.oneOfGetter($oneOfFields = ["product_type_l1"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_type_l2.
                             * @member {"product_type_l2"|undefined} _product_type_l2
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_type_l2", {
                                get: $util.oneOfGetter($oneOfFields = ["product_type_l2"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_type_l3.
                             * @member {"product_type_l3"|undefined} _product_type_l3
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_type_l3", {
                                get: $util.oneOfGetter($oneOfFields = ["product_type_l3"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_type_l4.
                             * @member {"product_type_l4"|undefined} _product_type_l4
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_type_l4", {
                                get: $util.oneOfGetter($oneOfFields = ["product_type_l4"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _product_type_l5.
                             * @member {"product_type_l5"|undefined} _product_type_l5
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_product_type_l5", {
                                get: $util.oneOfGetter($oneOfFields = ["product_type_l5"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _quarter.
                             * @member {"quarter"|undefined} _quarter
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_quarter", {
                                get: $util.oneOfGetter($oneOfFields = ["quarter"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _webpage.
                             * @member {"webpage"|undefined} _webpage
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_webpage", {
                                get: $util.oneOfGetter($oneOfFields = ["webpage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _week.
                             * @member {"week"|undefined} _week
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_week", {
                                get: $util.oneOfGetter($oneOfFields = ["week"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _year.
                             * @member {"year"|undefined} _year
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_year", {
                                get: $util.oneOfGetter($oneOfFields = ["year"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _sk_ad_network_conversion_value.
                             * @member {"sk_ad_network_conversion_value"|undefined} _sk_ad_network_conversion_value
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_sk_ad_network_conversion_value", {
                                get: $util.oneOfGetter($oneOfFields = ["sk_ad_network_conversion_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _sk_ad_network_source_app.
                             * @member {"sk_ad_network_source_app"|undefined} _sk_ad_network_source_app
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_sk_ad_network_source_app", {
                                get: $util.oneOfGetter($oneOfFields = ["sk_ad_network_source_app"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Segments _asset_interaction_target.
                             * @member {"asset_interaction_target"|undefined} _asset_interaction_target
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             */
                            Object.defineProperty(Segments.prototype, "_asset_interaction_target", {
                                get: $util.oneOfGetter($oneOfFields = ["asset_interaction_target"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Segments instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {google.ads.googleads.v11.common.ISegments=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.Segments} Segments instance
                             */
                            Segments.create = function create(properties) {
                                return new Segments(properties);
                            };
    
                            /**
                             * Encodes the specified Segments message. Does not implicitly {@link google.ads.googleads.v11.common.Segments.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {google.ads.googleads.v11.common.ISegments} m Segments message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Segments.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    w.uint32(8).int32(m.device);
                                if (m.conversion_attribution_event_type != null && Object.hasOwnProperty.call(m, "conversion_attribution_event_type"))
                                    w.uint32(16).int32(m.conversion_attribution_event_type);
                                if (m.ad_network_type != null && Object.hasOwnProperty.call(m, "ad_network_type"))
                                    w.uint32(24).int32(m.ad_network_type);
                                if (m.day_of_week != null && Object.hasOwnProperty.call(m, "day_of_week"))
                                    w.uint32(40).int32(m.day_of_week);
                                if (m.hotel_check_in_day_of_week != null && Object.hasOwnProperty.call(m, "hotel_check_in_day_of_week"))
                                    w.uint32(72).int32(m.hotel_check_in_day_of_week);
                                if (m.hotel_date_selection_type != null && Object.hasOwnProperty.call(m, "hotel_date_selection_type"))
                                    w.uint32(104).int32(m.hotel_date_selection_type);
                                if (m.month_of_year != null && Object.hasOwnProperty.call(m, "month_of_year"))
                                    w.uint32(144).int32(m.month_of_year);
                                if (m.placeholder_type != null && Object.hasOwnProperty.call(m, "placeholder_type"))
                                    w.uint32(160).int32(m.placeholder_type);
                                if (m.search_term_match_type != null && Object.hasOwnProperty.call(m, "search_term_match_type"))
                                    w.uint32(176).int32(m.search_term_match_type);
                                if (m.slot != null && Object.hasOwnProperty.call(m, "slot"))
                                    w.uint32(184).int32(m.slot);
                                if (m.click_type != null && Object.hasOwnProperty.call(m, "click_type"))
                                    w.uint32(208).int32(m.click_type);
                                if (m.product_channel != null && Object.hasOwnProperty.call(m, "product_channel"))
                                    w.uint32(240).int32(m.product_channel);
                                if (m.product_channel_exclusivity != null && Object.hasOwnProperty.call(m, "product_channel_exclusivity"))
                                    w.uint32(248).int32(m.product_channel_exclusivity);
                                if (m.product_condition != null && Object.hasOwnProperty.call(m, "product_condition"))
                                    w.uint32(256).int32(m.product_condition);
                                if (m.conversion_lag_bucket != null && Object.hasOwnProperty.call(m, "conversion_lag_bucket"))
                                    w.uint32(400).int32(m.conversion_lag_bucket);
                                if (m.conversion_or_adjustment_lag_bucket != null && Object.hasOwnProperty.call(m, "conversion_or_adjustment_lag_bucket"))
                                    w.uint32(408).int32(m.conversion_or_adjustment_lag_bucket);
                                if (m.conversion_action_category != null && Object.hasOwnProperty.call(m, "conversion_action_category"))
                                    w.uint32(424).int32(m.conversion_action_category);
                                if (m.external_conversion_source != null && Object.hasOwnProperty.call(m, "external_conversion_source"))
                                    w.uint32(440).int32(m.external_conversion_source);
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    $root.google.ads.googleads.v11.common.Keyword.encode(m.keyword, w.uint32(490).fork()).ldelim();
                                if (m.search_engine_results_page_type != null && Object.hasOwnProperty.call(m, "search_engine_results_page_type"))
                                    w.uint32(560).int32(m.search_engine_results_page_type);
                                if (m.hotel_rate_type != null && Object.hasOwnProperty.call(m, "hotel_rate_type"))
                                    w.uint32(592).int32(m.hotel_rate_type);
                                if (m.hotel_price_bucket != null && Object.hasOwnProperty.call(m, "hotel_price_bucket"))
                                    w.uint32(624).int32(m.hotel_price_bucket);
                                if (m.date != null && Object.hasOwnProperty.call(m, "date"))
                                    w.uint32(634).string(m.date);
                                if (m.hotel_center_id != null && Object.hasOwnProperty.call(m, "hotel_center_id"))
                                    w.uint32(640).int64(m.hotel_center_id);
                                if (m.hotel_check_in_date != null && Object.hasOwnProperty.call(m, "hotel_check_in_date"))
                                    w.uint32(650).string(m.hotel_check_in_date);
                                if (m.hotel_city != null && Object.hasOwnProperty.call(m, "hotel_city"))
                                    w.uint32(658).string(m.hotel_city);
                                if (m.hotel_class != null && Object.hasOwnProperty.call(m, "hotel_class"))
                                    w.uint32(664).int32(m.hotel_class);
                                if (m.hotel_country != null && Object.hasOwnProperty.call(m, "hotel_country"))
                                    w.uint32(674).string(m.hotel_country);
                                if (m.hotel_length_of_stay != null && Object.hasOwnProperty.call(m, "hotel_length_of_stay"))
                                    w.uint32(680).int32(m.hotel_length_of_stay);
                                if (m.hotel_rate_rule_id != null && Object.hasOwnProperty.call(m, "hotel_rate_rule_id"))
                                    w.uint32(690).string(m.hotel_rate_rule_id);
                                if (m.hotel_state != null && Object.hasOwnProperty.call(m, "hotel_state"))
                                    w.uint32(698).string(m.hotel_state);
                                if (m.hour != null && Object.hasOwnProperty.call(m, "hour"))
                                    w.uint32(704).int32(m.hour);
                                if (m.interaction_on_this_extension != null && Object.hasOwnProperty.call(m, "interaction_on_this_extension"))
                                    w.uint32(712).bool(m.interaction_on_this_extension);
                                if (m.month != null && Object.hasOwnProperty.call(m, "month"))
                                    w.uint32(722).string(m.month);
                                if (m.partner_hotel_id != null && Object.hasOwnProperty.call(m, "partner_hotel_id"))
                                    w.uint32(730).string(m.partner_hotel_id);
                                if (m.product_bidding_category_level1 != null && Object.hasOwnProperty.call(m, "product_bidding_category_level1"))
                                    w.uint32(738).string(m.product_bidding_category_level1);
                                if (m.product_bidding_category_level2 != null && Object.hasOwnProperty.call(m, "product_bidding_category_level2"))
                                    w.uint32(746).string(m.product_bidding_category_level2);
                                if (m.product_bidding_category_level3 != null && Object.hasOwnProperty.call(m, "product_bidding_category_level3"))
                                    w.uint32(754).string(m.product_bidding_category_level3);
                                if (m.product_bidding_category_level4 != null && Object.hasOwnProperty.call(m, "product_bidding_category_level4"))
                                    w.uint32(762).string(m.product_bidding_category_level4);
                                if (m.product_bidding_category_level5 != null && Object.hasOwnProperty.call(m, "product_bidding_category_level5"))
                                    w.uint32(770).string(m.product_bidding_category_level5);
                                if (m.product_brand != null && Object.hasOwnProperty.call(m, "product_brand"))
                                    w.uint32(778).string(m.product_brand);
                                if (m.product_country != null && Object.hasOwnProperty.call(m, "product_country"))
                                    w.uint32(786).string(m.product_country);
                                if (m.product_custom_attribute0 != null && Object.hasOwnProperty.call(m, "product_custom_attribute0"))
                                    w.uint32(794).string(m.product_custom_attribute0);
                                if (m.product_custom_attribute1 != null && Object.hasOwnProperty.call(m, "product_custom_attribute1"))
                                    w.uint32(802).string(m.product_custom_attribute1);
                                if (m.product_custom_attribute2 != null && Object.hasOwnProperty.call(m, "product_custom_attribute2"))
                                    w.uint32(810).string(m.product_custom_attribute2);
                                if (m.product_custom_attribute3 != null && Object.hasOwnProperty.call(m, "product_custom_attribute3"))
                                    w.uint32(818).string(m.product_custom_attribute3);
                                if (m.product_custom_attribute4 != null && Object.hasOwnProperty.call(m, "product_custom_attribute4"))
                                    w.uint32(826).string(m.product_custom_attribute4);
                                if (m.product_item_id != null && Object.hasOwnProperty.call(m, "product_item_id"))
                                    w.uint32(834).string(m.product_item_id);
                                if (m.product_language != null && Object.hasOwnProperty.call(m, "product_language"))
                                    w.uint32(842).string(m.product_language);
                                if (m.product_store_id != null && Object.hasOwnProperty.call(m, "product_store_id"))
                                    w.uint32(850).string(m.product_store_id);
                                if (m.product_title != null && Object.hasOwnProperty.call(m, "product_title"))
                                    w.uint32(858).string(m.product_title);
                                if (m.product_type_l1 != null && Object.hasOwnProperty.call(m, "product_type_l1"))
                                    w.uint32(866).string(m.product_type_l1);
                                if (m.product_type_l2 != null && Object.hasOwnProperty.call(m, "product_type_l2"))
                                    w.uint32(874).string(m.product_type_l2);
                                if (m.product_type_l3 != null && Object.hasOwnProperty.call(m, "product_type_l3"))
                                    w.uint32(882).string(m.product_type_l3);
                                if (m.product_type_l4 != null && Object.hasOwnProperty.call(m, "product_type_l4"))
                                    w.uint32(890).string(m.product_type_l4);
                                if (m.product_type_l5 != null && Object.hasOwnProperty.call(m, "product_type_l5"))
                                    w.uint32(898).string(m.product_type_l5);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(906).string(m.conversion_action);
                                if (m.conversion_action_name != null && Object.hasOwnProperty.call(m, "conversion_action_name"))
                                    w.uint32(914).string(m.conversion_action_name);
                                if (m.conversion_adjustment != null && Object.hasOwnProperty.call(m, "conversion_adjustment"))
                                    w.uint32(920).bool(m.conversion_adjustment);
                                if (m.geo_target_airport != null && Object.hasOwnProperty.call(m, "geo_target_airport"))
                                    w.uint32(930).string(m.geo_target_airport);
                                if (m.geo_target_canton != null && Object.hasOwnProperty.call(m, "geo_target_canton"))
                                    w.uint32(938).string(m.geo_target_canton);
                                if (m.geo_target_city != null && Object.hasOwnProperty.call(m, "geo_target_city"))
                                    w.uint32(946).string(m.geo_target_city);
                                if (m.geo_target_country != null && Object.hasOwnProperty.call(m, "geo_target_country"))
                                    w.uint32(954).string(m.geo_target_country);
                                if (m.geo_target_county != null && Object.hasOwnProperty.call(m, "geo_target_county"))
                                    w.uint32(962).string(m.geo_target_county);
                                if (m.geo_target_district != null && Object.hasOwnProperty.call(m, "geo_target_district"))
                                    w.uint32(970).string(m.geo_target_district);
                                if (m.geo_target_metro != null && Object.hasOwnProperty.call(m, "geo_target_metro"))
                                    w.uint32(978).string(m.geo_target_metro);
                                if (m.geo_target_most_specific_location != null && Object.hasOwnProperty.call(m, "geo_target_most_specific_location"))
                                    w.uint32(986).string(m.geo_target_most_specific_location);
                                if (m.geo_target_postal_code != null && Object.hasOwnProperty.call(m, "geo_target_postal_code"))
                                    w.uint32(994).string(m.geo_target_postal_code);
                                if (m.geo_target_province != null && Object.hasOwnProperty.call(m, "geo_target_province"))
                                    w.uint32(1002).string(m.geo_target_province);
                                if (m.geo_target_region != null && Object.hasOwnProperty.call(m, "geo_target_region"))
                                    w.uint32(1010).string(m.geo_target_region);
                                if (m.geo_target_state != null && Object.hasOwnProperty.call(m, "geo_target_state"))
                                    w.uint32(1018).string(m.geo_target_state);
                                if (m.quarter != null && Object.hasOwnProperty.call(m, "quarter"))
                                    w.uint32(1026).string(m.quarter);
                                if (m.webpage != null && Object.hasOwnProperty.call(m, "webpage"))
                                    w.uint32(1034).string(m.webpage);
                                if (m.week != null && Object.hasOwnProperty.call(m, "week"))
                                    w.uint32(1042).string(m.week);
                                if (m.year != null && Object.hasOwnProperty.call(m, "year"))
                                    w.uint32(1048).int32(m.year);
                                if (m.product_aggregator_id != null && Object.hasOwnProperty.call(m, "product_aggregator_id"))
                                    w.uint32(1056).int64(m.product_aggregator_id);
                                if (m.product_merchant_id != null && Object.hasOwnProperty.call(m, "product_merchant_id"))
                                    w.uint32(1064).int64(m.product_merchant_id);
                                if (m.budget_campaign_association_status != null && Object.hasOwnProperty.call(m, "budget_campaign_association_status"))
                                    $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus.encode(m.budget_campaign_association_status, w.uint32(1074).fork()).ldelim();
                                if (m.hotel_booking_window_days != null && Object.hasOwnProperty.call(m, "hotel_booking_window_days"))
                                    w.uint32(1080).int64(m.hotel_booking_window_days);
                                if (m.ad_destination_type != null && Object.hasOwnProperty.call(m, "ad_destination_type"))
                                    w.uint32(1088).int32(m.ad_destination_type);
                                if (m.sk_ad_network_conversion_value != null && Object.hasOwnProperty.call(m, "sk_ad_network_conversion_value"))
                                    w.uint32(1096).int64(m.sk_ad_network_conversion_value);
                                if (m.conversion_value_rule_primary_dimension != null && Object.hasOwnProperty.call(m, "conversion_value_rule_primary_dimension"))
                                    w.uint32(1104).int32(m.conversion_value_rule_primary_dimension);
                                if (m.asset_interaction_target != null && Object.hasOwnProperty.call(m, "asset_interaction_target"))
                                    $root.google.ads.googleads.v11.common.AssetInteractionTarget.encode(m.asset_interaction_target, w.uint32(1114).fork()).ldelim();
                                if (m.recommendation_type != null && Object.hasOwnProperty.call(m, "recommendation_type"))
                                    w.uint32(1120).int32(m.recommendation_type);
                                if (m.sk_ad_network_user_type != null && Object.hasOwnProperty.call(m, "sk_ad_network_user_type"))
                                    w.uint32(1128).int32(m.sk_ad_network_user_type);
                                if (m.sk_ad_network_ad_event_type != null && Object.hasOwnProperty.call(m, "sk_ad_network_ad_event_type"))
                                    w.uint32(1136).int32(m.sk_ad_network_ad_event_type);
                                if (m.sk_ad_network_source_app != null && Object.hasOwnProperty.call(m, "sk_ad_network_source_app"))
                                    $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp.encode(m.sk_ad_network_source_app, w.uint32(1146).fork()).ldelim();
                                if (m.sk_ad_network_attribution_credit != null && Object.hasOwnProperty.call(m, "sk_ad_network_attribution_credit"))
                                    w.uint32(1152).int32(m.sk_ad_network_attribution_credit);
                                if (m.auction_insight_domain != null && Object.hasOwnProperty.call(m, "auction_insight_domain"))
                                    w.uint32(1162).string(m.auction_insight_domain);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Segments message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Segments.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {google.ads.googleads.v11.common.ISegments} message Segments message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Segments.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Segments message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.Segments} Segments
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Segments.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Segments();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 136:
                                        m.ad_destination_type = r.int32();
                                        break;
                                    case 3:
                                        m.ad_network_type = r.int32();
                                        break;
                                    case 145:
                                        m.auction_insight_domain = r.string();
                                        break;
                                    case 134:
                                        m.budget_campaign_association_status = $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus.decode(r, r.uint32());
                                        break;
                                    case 26:
                                        m.click_type = r.int32();
                                        break;
                                    case 113:
                                        m.conversion_action = r.string();
                                        break;
                                    case 53:
                                        m.conversion_action_category = r.int32();
                                        break;
                                    case 114:
                                        m.conversion_action_name = r.string();
                                        break;
                                    case 115:
                                        m.conversion_adjustment = r.bool();
                                        break;
                                    case 2:
                                        m.conversion_attribution_event_type = r.int32();
                                        break;
                                    case 50:
                                        m.conversion_lag_bucket = r.int32();
                                        break;
                                    case 51:
                                        m.conversion_or_adjustment_lag_bucket = r.int32();
                                        break;
                                    case 79:
                                        m.date = r.string();
                                        break;
                                    case 5:
                                        m.day_of_week = r.int32();
                                        break;
                                    case 1:
                                        m.device = r.int32();
                                        break;
                                    case 55:
                                        m.external_conversion_source = r.int32();
                                        break;
                                    case 116:
                                        m.geo_target_airport = r.string();
                                        break;
                                    case 117:
                                        m.geo_target_canton = r.string();
                                        break;
                                    case 118:
                                        m.geo_target_city = r.string();
                                        break;
                                    case 119:
                                        m.geo_target_country = r.string();
                                        break;
                                    case 120:
                                        m.geo_target_county = r.string();
                                        break;
                                    case 121:
                                        m.geo_target_district = r.string();
                                        break;
                                    case 122:
                                        m.geo_target_metro = r.string();
                                        break;
                                    case 123:
                                        m.geo_target_most_specific_location = r.string();
                                        break;
                                    case 124:
                                        m.geo_target_postal_code = r.string();
                                        break;
                                    case 125:
                                        m.geo_target_province = r.string();
                                        break;
                                    case 126:
                                        m.geo_target_region = r.string();
                                        break;
                                    case 127:
                                        m.geo_target_state = r.string();
                                        break;
                                    case 135:
                                        m.hotel_booking_window_days = r.int64();
                                        break;
                                    case 80:
                                        m.hotel_center_id = r.int64();
                                        break;
                                    case 81:
                                        m.hotel_check_in_date = r.string();
                                        break;
                                    case 9:
                                        m.hotel_check_in_day_of_week = r.int32();
                                        break;
                                    case 82:
                                        m.hotel_city = r.string();
                                        break;
                                    case 83:
                                        m.hotel_class = r.int32();
                                        break;
                                    case 84:
                                        m.hotel_country = r.string();
                                        break;
                                    case 13:
                                        m.hotel_date_selection_type = r.int32();
                                        break;
                                    case 85:
                                        m.hotel_length_of_stay = r.int32();
                                        break;
                                    case 86:
                                        m.hotel_rate_rule_id = r.string();
                                        break;
                                    case 74:
                                        m.hotel_rate_type = r.int32();
                                        break;
                                    case 78:
                                        m.hotel_price_bucket = r.int32();
                                        break;
                                    case 87:
                                        m.hotel_state = r.string();
                                        break;
                                    case 88:
                                        m.hour = r.int32();
                                        break;
                                    case 89:
                                        m.interaction_on_this_extension = r.bool();
                                        break;
                                    case 61:
                                        m.keyword = $root.google.ads.googleads.v11.common.Keyword.decode(r, r.uint32());
                                        break;
                                    case 90:
                                        m.month = r.string();
                                        break;
                                    case 18:
                                        m.month_of_year = r.int32();
                                        break;
                                    case 91:
                                        m.partner_hotel_id = r.string();
                                        break;
                                    case 20:
                                        m.placeholder_type = r.int32();
                                        break;
                                    case 132:
                                        m.product_aggregator_id = r.int64();
                                        break;
                                    case 92:
                                        m.product_bidding_category_level1 = r.string();
                                        break;
                                    case 93:
                                        m.product_bidding_category_level2 = r.string();
                                        break;
                                    case 94:
                                        m.product_bidding_category_level3 = r.string();
                                        break;
                                    case 95:
                                        m.product_bidding_category_level4 = r.string();
                                        break;
                                    case 96:
                                        m.product_bidding_category_level5 = r.string();
                                        break;
                                    case 97:
                                        m.product_brand = r.string();
                                        break;
                                    case 30:
                                        m.product_channel = r.int32();
                                        break;
                                    case 31:
                                        m.product_channel_exclusivity = r.int32();
                                        break;
                                    case 32:
                                        m.product_condition = r.int32();
                                        break;
                                    case 98:
                                        m.product_country = r.string();
                                        break;
                                    case 99:
                                        m.product_custom_attribute0 = r.string();
                                        break;
                                    case 100:
                                        m.product_custom_attribute1 = r.string();
                                        break;
                                    case 101:
                                        m.product_custom_attribute2 = r.string();
                                        break;
                                    case 102:
                                        m.product_custom_attribute3 = r.string();
                                        break;
                                    case 103:
                                        m.product_custom_attribute4 = r.string();
                                        break;
                                    case 104:
                                        m.product_item_id = r.string();
                                        break;
                                    case 105:
                                        m.product_language = r.string();
                                        break;
                                    case 133:
                                        m.product_merchant_id = r.int64();
                                        break;
                                    case 106:
                                        m.product_store_id = r.string();
                                        break;
                                    case 107:
                                        m.product_title = r.string();
                                        break;
                                    case 108:
                                        m.product_type_l1 = r.string();
                                        break;
                                    case 109:
                                        m.product_type_l2 = r.string();
                                        break;
                                    case 110:
                                        m.product_type_l3 = r.string();
                                        break;
                                    case 111:
                                        m.product_type_l4 = r.string();
                                        break;
                                    case 112:
                                        m.product_type_l5 = r.string();
                                        break;
                                    case 128:
                                        m.quarter = r.string();
                                        break;
                                    case 140:
                                        m.recommendation_type = r.int32();
                                        break;
                                    case 70:
                                        m.search_engine_results_page_type = r.int32();
                                        break;
                                    case 22:
                                        m.search_term_match_type = r.int32();
                                        break;
                                    case 23:
                                        m.slot = r.int32();
                                        break;
                                    case 138:
                                        m.conversion_value_rule_primary_dimension = r.int32();
                                        break;
                                    case 129:
                                        m.webpage = r.string();
                                        break;
                                    case 130:
                                        m.week = r.string();
                                        break;
                                    case 131:
                                        m.year = r.int32();
                                        break;
                                    case 137:
                                        m.sk_ad_network_conversion_value = r.int64();
                                        break;
                                    case 141:
                                        m.sk_ad_network_user_type = r.int32();
                                        break;
                                    case 142:
                                        m.sk_ad_network_ad_event_type = r.int32();
                                        break;
                                    case 143:
                                        m.sk_ad_network_source_app = $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp.decode(r, r.uint32());
                                        break;
                                    case 144:
                                        m.sk_ad_network_attribution_credit = r.int32();
                                        break;
                                    case 139:
                                        m.asset_interaction_target = $root.google.ads.googleads.v11.common.AssetInteractionTarget.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Segments message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.Segments} Segments
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Segments.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Segments message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Segments.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.ad_destination_type != null && m.hasOwnProperty("ad_destination_type")) {
                                    switch (m.ad_destination_type) {
                                    default:
                                        return "ad_destination_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.ad_network_type != null && m.hasOwnProperty("ad_network_type")) {
                                    switch (m.ad_network_type) {
                                    default:
                                        return "ad_network_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.auction_insight_domain != null && m.hasOwnProperty("auction_insight_domain")) {
                                    p._auction_insight_domain = 1;
                                    if (!$util.isString(m.auction_insight_domain))
                                        return "auction_insight_domain: string expected";
                                }
                                if (m.budget_campaign_association_status != null && m.hasOwnProperty("budget_campaign_association_status")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus.verify(m.budget_campaign_association_status);
                                        if (e)
                                            return "budget_campaign_association_status." + e;
                                    }
                                }
                                if (m.click_type != null && m.hasOwnProperty("click_type")) {
                                    switch (m.click_type) {
                                    default:
                                        return "click_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 46:
                                    case 47:
                                    case 48:
                                    case 49:
                                    case 50:
                                    case 51:
                                    case 52:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                        break;
                                    }
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.conversion_action_category != null && m.hasOwnProperty("conversion_action_category")) {
                                    switch (m.conversion_action_category) {
                                    default:
                                        return "conversion_action_category: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                        break;
                                    }
                                }
                                if (m.conversion_action_name != null && m.hasOwnProperty("conversion_action_name")) {
                                    p._conversion_action_name = 1;
                                    if (!$util.isString(m.conversion_action_name))
                                        return "conversion_action_name: string expected";
                                }
                                if (m.conversion_adjustment != null && m.hasOwnProperty("conversion_adjustment")) {
                                    p._conversion_adjustment = 1;
                                    if (typeof m.conversion_adjustment !== "boolean")
                                        return "conversion_adjustment: boolean expected";
                                }
                                if (m.conversion_attribution_event_type != null && m.hasOwnProperty("conversion_attribution_event_type")) {
                                    switch (m.conversion_attribution_event_type) {
                                    default:
                                        return "conversion_attribution_event_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.conversion_lag_bucket != null && m.hasOwnProperty("conversion_lag_bucket")) {
                                    switch (m.conversion_lag_bucket) {
                                    default:
                                        return "conversion_lag_bucket: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                        break;
                                    }
                                }
                                if (m.conversion_or_adjustment_lag_bucket != null && m.hasOwnProperty("conversion_or_adjustment_lag_bucket")) {
                                    switch (m.conversion_or_adjustment_lag_bucket) {
                                    default:
                                        return "conversion_or_adjustment_lag_bucket: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                        break;
                                    }
                                }
                                if (m.date != null && m.hasOwnProperty("date")) {
                                    p._date = 1;
                                    if (!$util.isString(m.date))
                                        return "date: string expected";
                                }
                                if (m.day_of_week != null && m.hasOwnProperty("day_of_week")) {
                                    switch (m.day_of_week) {
                                    default:
                                        return "day_of_week: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    switch (m.device) {
                                    default:
                                        return "device: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.external_conversion_source != null && m.hasOwnProperty("external_conversion_source")) {
                                    switch (m.external_conversion_source) {
                                    default:
                                        return "external_conversion_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 27:
                                    case 29:
                                    case 31:
                                    case 33:
                                    case 34:
                                        break;
                                    }
                                }
                                if (m.geo_target_airport != null && m.hasOwnProperty("geo_target_airport")) {
                                    p._geo_target_airport = 1;
                                    if (!$util.isString(m.geo_target_airport))
                                        return "geo_target_airport: string expected";
                                }
                                if (m.geo_target_canton != null && m.hasOwnProperty("geo_target_canton")) {
                                    p._geo_target_canton = 1;
                                    if (!$util.isString(m.geo_target_canton))
                                        return "geo_target_canton: string expected";
                                }
                                if (m.geo_target_city != null && m.hasOwnProperty("geo_target_city")) {
                                    p._geo_target_city = 1;
                                    if (!$util.isString(m.geo_target_city))
                                        return "geo_target_city: string expected";
                                }
                                if (m.geo_target_country != null && m.hasOwnProperty("geo_target_country")) {
                                    p._geo_target_country = 1;
                                    if (!$util.isString(m.geo_target_country))
                                        return "geo_target_country: string expected";
                                }
                                if (m.geo_target_county != null && m.hasOwnProperty("geo_target_county")) {
                                    p._geo_target_county = 1;
                                    if (!$util.isString(m.geo_target_county))
                                        return "geo_target_county: string expected";
                                }
                                if (m.geo_target_district != null && m.hasOwnProperty("geo_target_district")) {
                                    p._geo_target_district = 1;
                                    if (!$util.isString(m.geo_target_district))
                                        return "geo_target_district: string expected";
                                }
                                if (m.geo_target_metro != null && m.hasOwnProperty("geo_target_metro")) {
                                    p._geo_target_metro = 1;
                                    if (!$util.isString(m.geo_target_metro))
                                        return "geo_target_metro: string expected";
                                }
                                if (m.geo_target_most_specific_location != null && m.hasOwnProperty("geo_target_most_specific_location")) {
                                    p._geo_target_most_specific_location = 1;
                                    if (!$util.isString(m.geo_target_most_specific_location))
                                        return "geo_target_most_specific_location: string expected";
                                }
                                if (m.geo_target_postal_code != null && m.hasOwnProperty("geo_target_postal_code")) {
                                    p._geo_target_postal_code = 1;
                                    if (!$util.isString(m.geo_target_postal_code))
                                        return "geo_target_postal_code: string expected";
                                }
                                if (m.geo_target_province != null && m.hasOwnProperty("geo_target_province")) {
                                    p._geo_target_province = 1;
                                    if (!$util.isString(m.geo_target_province))
                                        return "geo_target_province: string expected";
                                }
                                if (m.geo_target_region != null && m.hasOwnProperty("geo_target_region")) {
                                    p._geo_target_region = 1;
                                    if (!$util.isString(m.geo_target_region))
                                        return "geo_target_region: string expected";
                                }
                                if (m.geo_target_state != null && m.hasOwnProperty("geo_target_state")) {
                                    p._geo_target_state = 1;
                                    if (!$util.isString(m.geo_target_state))
                                        return "geo_target_state: string expected";
                                }
                                if (m.hotel_booking_window_days != null && m.hasOwnProperty("hotel_booking_window_days")) {
                                    p._hotel_booking_window_days = 1;
                                    if (!$util.isInteger(m.hotel_booking_window_days) && !(m.hotel_booking_window_days && $util.isInteger(m.hotel_booking_window_days.low) && $util.isInteger(m.hotel_booking_window_days.high)))
                                        return "hotel_booking_window_days: integer|Long expected";
                                }
                                if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                    p._hotel_center_id = 1;
                                    if (!$util.isInteger(m.hotel_center_id) && !(m.hotel_center_id && $util.isInteger(m.hotel_center_id.low) && $util.isInteger(m.hotel_center_id.high)))
                                        return "hotel_center_id: integer|Long expected";
                                }
                                if (m.hotel_check_in_date != null && m.hasOwnProperty("hotel_check_in_date")) {
                                    p._hotel_check_in_date = 1;
                                    if (!$util.isString(m.hotel_check_in_date))
                                        return "hotel_check_in_date: string expected";
                                }
                                if (m.hotel_check_in_day_of_week != null && m.hasOwnProperty("hotel_check_in_day_of_week")) {
                                    switch (m.hotel_check_in_day_of_week) {
                                    default:
                                        return "hotel_check_in_day_of_week: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.hotel_city != null && m.hasOwnProperty("hotel_city")) {
                                    p._hotel_city = 1;
                                    if (!$util.isString(m.hotel_city))
                                        return "hotel_city: string expected";
                                }
                                if (m.hotel_class != null && m.hasOwnProperty("hotel_class")) {
                                    p._hotel_class = 1;
                                    if (!$util.isInteger(m.hotel_class))
                                        return "hotel_class: integer expected";
                                }
                                if (m.hotel_country != null && m.hasOwnProperty("hotel_country")) {
                                    p._hotel_country = 1;
                                    if (!$util.isString(m.hotel_country))
                                        return "hotel_country: string expected";
                                }
                                if (m.hotel_date_selection_type != null && m.hasOwnProperty("hotel_date_selection_type")) {
                                    switch (m.hotel_date_selection_type) {
                                    default:
                                        return "hotel_date_selection_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 50:
                                    case 51:
                                        break;
                                    }
                                }
                                if (m.hotel_length_of_stay != null && m.hasOwnProperty("hotel_length_of_stay")) {
                                    p._hotel_length_of_stay = 1;
                                    if (!$util.isInteger(m.hotel_length_of_stay))
                                        return "hotel_length_of_stay: integer expected";
                                }
                                if (m.hotel_rate_rule_id != null && m.hasOwnProperty("hotel_rate_rule_id")) {
                                    p._hotel_rate_rule_id = 1;
                                    if (!$util.isString(m.hotel_rate_rule_id))
                                        return "hotel_rate_rule_id: string expected";
                                }
                                if (m.hotel_rate_type != null && m.hasOwnProperty("hotel_rate_type")) {
                                    switch (m.hotel_rate_type) {
                                    default:
                                        return "hotel_rate_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.hotel_price_bucket != null && m.hasOwnProperty("hotel_price_bucket")) {
                                    switch (m.hotel_price_bucket) {
                                    default:
                                        return "hotel_price_bucket: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.hotel_state != null && m.hasOwnProperty("hotel_state")) {
                                    p._hotel_state = 1;
                                    if (!$util.isString(m.hotel_state))
                                        return "hotel_state: string expected";
                                }
                                if (m.hour != null && m.hasOwnProperty("hour")) {
                                    p._hour = 1;
                                    if (!$util.isInteger(m.hour))
                                        return "hour: integer expected";
                                }
                                if (m.interaction_on_this_extension != null && m.hasOwnProperty("interaction_on_this_extension")) {
                                    p._interaction_on_this_extension = 1;
                                    if (typeof m.interaction_on_this_extension !== "boolean")
                                        return "interaction_on_this_extension: boolean expected";
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Keyword.verify(m.keyword);
                                        if (e)
                                            return "keyword." + e;
                                    }
                                }
                                if (m.month != null && m.hasOwnProperty("month")) {
                                    p._month = 1;
                                    if (!$util.isString(m.month))
                                        return "month: string expected";
                                }
                                if (m.month_of_year != null && m.hasOwnProperty("month_of_year")) {
                                    switch (m.month_of_year) {
                                    default:
                                        return "month_of_year: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.partner_hotel_id != null && m.hasOwnProperty("partner_hotel_id")) {
                                    p._partner_hotel_id = 1;
                                    if (!$util.isString(m.partner_hotel_id))
                                        return "partner_hotel_id: string expected";
                                }
                                if (m.placeholder_type != null && m.hasOwnProperty("placeholder_type")) {
                                    switch (m.placeholder_type) {
                                    default:
                                        return "placeholder_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.product_aggregator_id != null && m.hasOwnProperty("product_aggregator_id")) {
                                    p._product_aggregator_id = 1;
                                    if (!$util.isInteger(m.product_aggregator_id) && !(m.product_aggregator_id && $util.isInteger(m.product_aggregator_id.low) && $util.isInteger(m.product_aggregator_id.high)))
                                        return "product_aggregator_id: integer|Long expected";
                                }
                                if (m.product_bidding_category_level1 != null && m.hasOwnProperty("product_bidding_category_level1")) {
                                    p._product_bidding_category_level1 = 1;
                                    if (!$util.isString(m.product_bidding_category_level1))
                                        return "product_bidding_category_level1: string expected";
                                }
                                if (m.product_bidding_category_level2 != null && m.hasOwnProperty("product_bidding_category_level2")) {
                                    p._product_bidding_category_level2 = 1;
                                    if (!$util.isString(m.product_bidding_category_level2))
                                        return "product_bidding_category_level2: string expected";
                                }
                                if (m.product_bidding_category_level3 != null && m.hasOwnProperty("product_bidding_category_level3")) {
                                    p._product_bidding_category_level3 = 1;
                                    if (!$util.isString(m.product_bidding_category_level3))
                                        return "product_bidding_category_level3: string expected";
                                }
                                if (m.product_bidding_category_level4 != null && m.hasOwnProperty("product_bidding_category_level4")) {
                                    p._product_bidding_category_level4 = 1;
                                    if (!$util.isString(m.product_bidding_category_level4))
                                        return "product_bidding_category_level4: string expected";
                                }
                                if (m.product_bidding_category_level5 != null && m.hasOwnProperty("product_bidding_category_level5")) {
                                    p._product_bidding_category_level5 = 1;
                                    if (!$util.isString(m.product_bidding_category_level5))
                                        return "product_bidding_category_level5: string expected";
                                }
                                if (m.product_brand != null && m.hasOwnProperty("product_brand")) {
                                    p._product_brand = 1;
                                    if (!$util.isString(m.product_brand))
                                        return "product_brand: string expected";
                                }
                                if (m.product_channel != null && m.hasOwnProperty("product_channel")) {
                                    switch (m.product_channel) {
                                    default:
                                        return "product_channel: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.product_channel_exclusivity != null && m.hasOwnProperty("product_channel_exclusivity")) {
                                    switch (m.product_channel_exclusivity) {
                                    default:
                                        return "product_channel_exclusivity: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.product_condition != null && m.hasOwnProperty("product_condition")) {
                                    switch (m.product_condition) {
                                    default:
                                        return "product_condition: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.product_country != null && m.hasOwnProperty("product_country")) {
                                    p._product_country = 1;
                                    if (!$util.isString(m.product_country))
                                        return "product_country: string expected";
                                }
                                if (m.product_custom_attribute0 != null && m.hasOwnProperty("product_custom_attribute0")) {
                                    p._product_custom_attribute0 = 1;
                                    if (!$util.isString(m.product_custom_attribute0))
                                        return "product_custom_attribute0: string expected";
                                }
                                if (m.product_custom_attribute1 != null && m.hasOwnProperty("product_custom_attribute1")) {
                                    p._product_custom_attribute1 = 1;
                                    if (!$util.isString(m.product_custom_attribute1))
                                        return "product_custom_attribute1: string expected";
                                }
                                if (m.product_custom_attribute2 != null && m.hasOwnProperty("product_custom_attribute2")) {
                                    p._product_custom_attribute2 = 1;
                                    if (!$util.isString(m.product_custom_attribute2))
                                        return "product_custom_attribute2: string expected";
                                }
                                if (m.product_custom_attribute3 != null && m.hasOwnProperty("product_custom_attribute3")) {
                                    p._product_custom_attribute3 = 1;
                                    if (!$util.isString(m.product_custom_attribute3))
                                        return "product_custom_attribute3: string expected";
                                }
                                if (m.product_custom_attribute4 != null && m.hasOwnProperty("product_custom_attribute4")) {
                                    p._product_custom_attribute4 = 1;
                                    if (!$util.isString(m.product_custom_attribute4))
                                        return "product_custom_attribute4: string expected";
                                }
                                if (m.product_item_id != null && m.hasOwnProperty("product_item_id")) {
                                    p._product_item_id = 1;
                                    if (!$util.isString(m.product_item_id))
                                        return "product_item_id: string expected";
                                }
                                if (m.product_language != null && m.hasOwnProperty("product_language")) {
                                    p._product_language = 1;
                                    if (!$util.isString(m.product_language))
                                        return "product_language: string expected";
                                }
                                if (m.product_merchant_id != null && m.hasOwnProperty("product_merchant_id")) {
                                    p._product_merchant_id = 1;
                                    if (!$util.isInteger(m.product_merchant_id) && !(m.product_merchant_id && $util.isInteger(m.product_merchant_id.low) && $util.isInteger(m.product_merchant_id.high)))
                                        return "product_merchant_id: integer|Long expected";
                                }
                                if (m.product_store_id != null && m.hasOwnProperty("product_store_id")) {
                                    p._product_store_id = 1;
                                    if (!$util.isString(m.product_store_id))
                                        return "product_store_id: string expected";
                                }
                                if (m.product_title != null && m.hasOwnProperty("product_title")) {
                                    p._product_title = 1;
                                    if (!$util.isString(m.product_title))
                                        return "product_title: string expected";
                                }
                                if (m.product_type_l1 != null && m.hasOwnProperty("product_type_l1")) {
                                    p._product_type_l1 = 1;
                                    if (!$util.isString(m.product_type_l1))
                                        return "product_type_l1: string expected";
                                }
                                if (m.product_type_l2 != null && m.hasOwnProperty("product_type_l2")) {
                                    p._product_type_l2 = 1;
                                    if (!$util.isString(m.product_type_l2))
                                        return "product_type_l2: string expected";
                                }
                                if (m.product_type_l3 != null && m.hasOwnProperty("product_type_l3")) {
                                    p._product_type_l3 = 1;
                                    if (!$util.isString(m.product_type_l3))
                                        return "product_type_l3: string expected";
                                }
                                if (m.product_type_l4 != null && m.hasOwnProperty("product_type_l4")) {
                                    p._product_type_l4 = 1;
                                    if (!$util.isString(m.product_type_l4))
                                        return "product_type_l4: string expected";
                                }
                                if (m.product_type_l5 != null && m.hasOwnProperty("product_type_l5")) {
                                    p._product_type_l5 = 1;
                                    if (!$util.isString(m.product_type_l5))
                                        return "product_type_l5: string expected";
                                }
                                if (m.quarter != null && m.hasOwnProperty("quarter")) {
                                    p._quarter = 1;
                                    if (!$util.isString(m.quarter))
                                        return "quarter: string expected";
                                }
                                if (m.recommendation_type != null && m.hasOwnProperty("recommendation_type")) {
                                    switch (m.recommendation_type) {
                                    default:
                                        return "recommendation_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                        break;
                                    }
                                }
                                if (m.search_engine_results_page_type != null && m.hasOwnProperty("search_engine_results_page_type")) {
                                    switch (m.search_engine_results_page_type) {
                                    default:
                                        return "search_engine_results_page_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.search_term_match_type != null && m.hasOwnProperty("search_term_match_type")) {
                                    switch (m.search_term_match_type) {
                                    default:
                                        return "search_term_match_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.slot != null && m.hasOwnProperty("slot")) {
                                    switch (m.slot) {
                                    default:
                                        return "slot: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.conversion_value_rule_primary_dimension != null && m.hasOwnProperty("conversion_value_rule_primary_dimension")) {
                                    switch (m.conversion_value_rule_primary_dimension) {
                                    default:
                                        return "conversion_value_rule_primary_dimension: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.webpage != null && m.hasOwnProperty("webpage")) {
                                    p._webpage = 1;
                                    if (!$util.isString(m.webpage))
                                        return "webpage: string expected";
                                }
                                if (m.week != null && m.hasOwnProperty("week")) {
                                    p._week = 1;
                                    if (!$util.isString(m.week))
                                        return "week: string expected";
                                }
                                if (m.year != null && m.hasOwnProperty("year")) {
                                    p._year = 1;
                                    if (!$util.isInteger(m.year))
                                        return "year: integer expected";
                                }
                                if (m.sk_ad_network_conversion_value != null && m.hasOwnProperty("sk_ad_network_conversion_value")) {
                                    p._sk_ad_network_conversion_value = 1;
                                    if (!$util.isInteger(m.sk_ad_network_conversion_value) && !(m.sk_ad_network_conversion_value && $util.isInteger(m.sk_ad_network_conversion_value.low) && $util.isInteger(m.sk_ad_network_conversion_value.high)))
                                        return "sk_ad_network_conversion_value: integer|Long expected";
                                }
                                if (m.sk_ad_network_user_type != null && m.hasOwnProperty("sk_ad_network_user_type")) {
                                    switch (m.sk_ad_network_user_type) {
                                    default:
                                        return "sk_ad_network_user_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.sk_ad_network_ad_event_type != null && m.hasOwnProperty("sk_ad_network_ad_event_type")) {
                                    switch (m.sk_ad_network_ad_event_type) {
                                    default:
                                        return "sk_ad_network_ad_event_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.sk_ad_network_source_app != null && m.hasOwnProperty("sk_ad_network_source_app")) {
                                    p._sk_ad_network_source_app = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp.verify(m.sk_ad_network_source_app);
                                        if (e)
                                            return "sk_ad_network_source_app." + e;
                                    }
                                }
                                if (m.sk_ad_network_attribution_credit != null && m.hasOwnProperty("sk_ad_network_attribution_credit")) {
                                    switch (m.sk_ad_network_attribution_credit) {
                                    default:
                                        return "sk_ad_network_attribution_credit: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.asset_interaction_target != null && m.hasOwnProperty("asset_interaction_target")) {
                                    p._asset_interaction_target = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AssetInteractionTarget.verify(m.asset_interaction_target);
                                        if (e)
                                            return "asset_interaction_target." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Segments message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.Segments} Segments
                             */
                            Segments.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.Segments)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.Segments();
                                switch (d.ad_destination_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_destination_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_destination_type = 1;
                                    break;
                                case "NOT_APPLICABLE":
                                case 2:
                                    m.ad_destination_type = 2;
                                    break;
                                case "WEBSITE":
                                case 3:
                                    m.ad_destination_type = 3;
                                    break;
                                case "APP_DEEP_LINK":
                                case 4:
                                    m.ad_destination_type = 4;
                                    break;
                                case "APP_STORE":
                                case 5:
                                    m.ad_destination_type = 5;
                                    break;
                                case "PHONE_CALL":
                                case 6:
                                    m.ad_destination_type = 6;
                                    break;
                                case "MAP_DIRECTIONS":
                                case 7:
                                    m.ad_destination_type = 7;
                                    break;
                                case "LOCATION_LISTING":
                                case 8:
                                    m.ad_destination_type = 8;
                                    break;
                                case "MESSAGE":
                                case 9:
                                    m.ad_destination_type = 9;
                                    break;
                                case "LEAD_FORM":
                                case 10:
                                    m.ad_destination_type = 10;
                                    break;
                                case "YOUTUBE":
                                case 11:
                                    m.ad_destination_type = 11;
                                    break;
                                case "UNMODELED_FOR_CONVERSIONS":
                                case 12:
                                    m.ad_destination_type = 12;
                                    break;
                                }
                                switch (d.ad_network_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_network_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_network_type = 1;
                                    break;
                                case "SEARCH":
                                case 2:
                                    m.ad_network_type = 2;
                                    break;
                                case "SEARCH_PARTNERS":
                                case 3:
                                    m.ad_network_type = 3;
                                    break;
                                case "CONTENT":
                                case 4:
                                    m.ad_network_type = 4;
                                    break;
                                case "YOUTUBE_SEARCH":
                                case 5:
                                    m.ad_network_type = 5;
                                    break;
                                case "YOUTUBE_WATCH":
                                case 6:
                                    m.ad_network_type = 6;
                                    break;
                                case "MIXED":
                                case 7:
                                    m.ad_network_type = 7;
                                    break;
                                }
                                if (d.auction_insight_domain != null) {
                                    m.auction_insight_domain = String(d.auction_insight_domain);
                                }
                                if (d.budget_campaign_association_status != null) {
                                    if (typeof d.budget_campaign_association_status !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Segments.budget_campaign_association_status: object expected");
                                    m.budget_campaign_association_status = $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus.fromObject(d.budget_campaign_association_status);
                                }
                                switch (d.click_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.click_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.click_type = 1;
                                    break;
                                case "APP_DEEPLINK":
                                case 2:
                                    m.click_type = 2;
                                    break;
                                case "BREADCRUMBS":
                                case 3:
                                    m.click_type = 3;
                                    break;
                                case "BROADBAND_PLAN":
                                case 4:
                                    m.click_type = 4;
                                    break;
                                case "CALL_TRACKING":
                                case 5:
                                    m.click_type = 5;
                                    break;
                                case "CALLS":
                                case 6:
                                    m.click_type = 6;
                                    break;
                                case "CLICK_ON_ENGAGEMENT_AD":
                                case 7:
                                    m.click_type = 7;
                                    break;
                                case "GET_DIRECTIONS":
                                case 8:
                                    m.click_type = 8;
                                    break;
                                case "LOCATION_EXPANSION":
                                case 9:
                                    m.click_type = 9;
                                    break;
                                case "LOCATION_FORMAT_CALL":
                                case 10:
                                    m.click_type = 10;
                                    break;
                                case "LOCATION_FORMAT_DIRECTIONS":
                                case 11:
                                    m.click_type = 11;
                                    break;
                                case "LOCATION_FORMAT_IMAGE":
                                case 12:
                                    m.click_type = 12;
                                    break;
                                case "LOCATION_FORMAT_LANDING_PAGE":
                                case 13:
                                    m.click_type = 13;
                                    break;
                                case "LOCATION_FORMAT_MAP":
                                case 14:
                                    m.click_type = 14;
                                    break;
                                case "LOCATION_FORMAT_STORE_INFO":
                                case 15:
                                    m.click_type = 15;
                                    break;
                                case "LOCATION_FORMAT_TEXT":
                                case 16:
                                    m.click_type = 16;
                                    break;
                                case "MOBILE_CALL_TRACKING":
                                case 17:
                                    m.click_type = 17;
                                    break;
                                case "OFFER_PRINTS":
                                case 18:
                                    m.click_type = 18;
                                    break;
                                case "OTHER":
                                case 19:
                                    m.click_type = 19;
                                    break;
                                case "PRODUCT_EXTENSION_CLICKS":
                                case 20:
                                    m.click_type = 20;
                                    break;
                                case "PRODUCT_LISTING_AD_CLICKS":
                                case 21:
                                    m.click_type = 21;
                                    break;
                                case "SITELINKS":
                                case 22:
                                    m.click_type = 22;
                                    break;
                                case "STORE_LOCATOR":
                                case 23:
                                    m.click_type = 23;
                                    break;
                                case "URL_CLICKS":
                                case 25:
                                    m.click_type = 25;
                                    break;
                                case "VIDEO_APP_STORE_CLICKS":
                                case 26:
                                    m.click_type = 26;
                                    break;
                                case "VIDEO_CALL_TO_ACTION_CLICKS":
                                case 27:
                                    m.click_type = 27;
                                    break;
                                case "VIDEO_CARD_ACTION_HEADLINE_CLICKS":
                                case 28:
                                    m.click_type = 28;
                                    break;
                                case "VIDEO_END_CAP_CLICKS":
                                case 29:
                                    m.click_type = 29;
                                    break;
                                case "VIDEO_WEBSITE_CLICKS":
                                case 30:
                                    m.click_type = 30;
                                    break;
                                case "VISUAL_SITELINKS":
                                case 31:
                                    m.click_type = 31;
                                    break;
                                case "WIRELESS_PLAN":
                                case 32:
                                    m.click_type = 32;
                                    break;
                                case "PRODUCT_LISTING_AD_LOCAL":
                                case 33:
                                    m.click_type = 33;
                                    break;
                                case "PRODUCT_LISTING_AD_MULTICHANNEL_LOCAL":
                                case 34:
                                    m.click_type = 34;
                                    break;
                                case "PRODUCT_LISTING_AD_MULTICHANNEL_ONLINE":
                                case 35:
                                    m.click_type = 35;
                                    break;
                                case "PRODUCT_LISTING_ADS_COUPON":
                                case 36:
                                    m.click_type = 36;
                                    break;
                                case "PRODUCT_LISTING_AD_TRANSACTABLE":
                                case 37:
                                    m.click_type = 37;
                                    break;
                                case "PRODUCT_AD_APP_DEEPLINK":
                                case 38:
                                    m.click_type = 38;
                                    break;
                                case "SHOWCASE_AD_CATEGORY_LINK":
                                case 39:
                                    m.click_type = 39;
                                    break;
                                case "SHOWCASE_AD_LOCAL_STOREFRONT_LINK":
                                case 40:
                                    m.click_type = 40;
                                    break;
                                case "SHOWCASE_AD_ONLINE_PRODUCT_LINK":
                                case 42:
                                    m.click_type = 42;
                                    break;
                                case "SHOWCASE_AD_LOCAL_PRODUCT_LINK":
                                case 43:
                                    m.click_type = 43;
                                    break;
                                case "PROMOTION_EXTENSION":
                                case 44:
                                    m.click_type = 44;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_HEADLINE":
                                case 45:
                                    m.click_type = 45;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_SWIPES":
                                case 46:
                                    m.click_type = 46;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_SEE_MORE":
                                case 47:
                                    m.click_type = 47;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_SITELINK_ONE":
                                case 48:
                                    m.click_type = 48;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_SITELINK_TWO":
                                case 49:
                                    m.click_type = 49;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_SITELINK_THREE":
                                case 50:
                                    m.click_type = 50;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_SITELINK_FOUR":
                                case 51:
                                    m.click_type = 51;
                                    break;
                                case "SWIPEABLE_GALLERY_AD_SITELINK_FIVE":
                                case 52:
                                    m.click_type = 52;
                                    break;
                                case "HOTEL_PRICE":
                                case 53:
                                    m.click_type = 53;
                                    break;
                                case "PRICE_EXTENSION":
                                case 54:
                                    m.click_type = 54;
                                    break;
                                case "HOTEL_BOOK_ON_GOOGLE_ROOM_SELECTION":
                                case 55:
                                    m.click_type = 55;
                                    break;
                                case "SHOPPING_COMPARISON_LISTING":
                                case 56:
                                    m.click_type = 56;
                                    break;
                                case "CROSS_NETWORK":
                                case 57:
                                    m.click_type = 57;
                                    break;
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                switch (d.conversion_action_category) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_action_category = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_action_category = 1;
                                    break;
                                case "DEFAULT":
                                case 2:
                                    m.conversion_action_category = 2;
                                    break;
                                case "PAGE_VIEW":
                                case 3:
                                    m.conversion_action_category = 3;
                                    break;
                                case "PURCHASE":
                                case 4:
                                    m.conversion_action_category = 4;
                                    break;
                                case "SIGNUP":
                                case 5:
                                    m.conversion_action_category = 5;
                                    break;
                                case "DOWNLOAD":
                                case 7:
                                    m.conversion_action_category = 7;
                                    break;
                                case "ADD_TO_CART":
                                case 8:
                                    m.conversion_action_category = 8;
                                    break;
                                case "BEGIN_CHECKOUT":
                                case 9:
                                    m.conversion_action_category = 9;
                                    break;
                                case "SUBSCRIBE_PAID":
                                case 10:
                                    m.conversion_action_category = 10;
                                    break;
                                case "PHONE_CALL_LEAD":
                                case 11:
                                    m.conversion_action_category = 11;
                                    break;
                                case "IMPORTED_LEAD":
                                case 12:
                                    m.conversion_action_category = 12;
                                    break;
                                case "SUBMIT_LEAD_FORM":
                                case 13:
                                    m.conversion_action_category = 13;
                                    break;
                                case "BOOK_APPOINTMENT":
                                case 14:
                                    m.conversion_action_category = 14;
                                    break;
                                case "REQUEST_QUOTE":
                                case 15:
                                    m.conversion_action_category = 15;
                                    break;
                                case "GET_DIRECTIONS":
                                case 16:
                                    m.conversion_action_category = 16;
                                    break;
                                case "OUTBOUND_CLICK":
                                case 17:
                                    m.conversion_action_category = 17;
                                    break;
                                case "CONTACT":
                                case 18:
                                    m.conversion_action_category = 18;
                                    break;
                                case "ENGAGEMENT":
                                case 19:
                                    m.conversion_action_category = 19;
                                    break;
                                case "STORE_VISIT":
                                case 20:
                                    m.conversion_action_category = 20;
                                    break;
                                case "STORE_SALE":
                                case 21:
                                    m.conversion_action_category = 21;
                                    break;
                                case "QUALIFIED_LEAD":
                                case 22:
                                    m.conversion_action_category = 22;
                                    break;
                                case "CONVERTED_LEAD":
                                case 23:
                                    m.conversion_action_category = 23;
                                    break;
                                }
                                if (d.conversion_action_name != null) {
                                    m.conversion_action_name = String(d.conversion_action_name);
                                }
                                if (d.conversion_adjustment != null) {
                                    m.conversion_adjustment = Boolean(d.conversion_adjustment);
                                }
                                switch (d.conversion_attribution_event_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_attribution_event_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_attribution_event_type = 1;
                                    break;
                                case "IMPRESSION":
                                case 2:
                                    m.conversion_attribution_event_type = 2;
                                    break;
                                case "INTERACTION":
                                case 3:
                                    m.conversion_attribution_event_type = 3;
                                    break;
                                }
                                switch (d.conversion_lag_bucket) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_lag_bucket = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_lag_bucket = 1;
                                    break;
                                case "LESS_THAN_ONE_DAY":
                                case 2:
                                    m.conversion_lag_bucket = 2;
                                    break;
                                case "ONE_TO_TWO_DAYS":
                                case 3:
                                    m.conversion_lag_bucket = 3;
                                    break;
                                case "TWO_TO_THREE_DAYS":
                                case 4:
                                    m.conversion_lag_bucket = 4;
                                    break;
                                case "THREE_TO_FOUR_DAYS":
                                case 5:
                                    m.conversion_lag_bucket = 5;
                                    break;
                                case "FOUR_TO_FIVE_DAYS":
                                case 6:
                                    m.conversion_lag_bucket = 6;
                                    break;
                                case "FIVE_TO_SIX_DAYS":
                                case 7:
                                    m.conversion_lag_bucket = 7;
                                    break;
                                case "SIX_TO_SEVEN_DAYS":
                                case 8:
                                    m.conversion_lag_bucket = 8;
                                    break;
                                case "SEVEN_TO_EIGHT_DAYS":
                                case 9:
                                    m.conversion_lag_bucket = 9;
                                    break;
                                case "EIGHT_TO_NINE_DAYS":
                                case 10:
                                    m.conversion_lag_bucket = 10;
                                    break;
                                case "NINE_TO_TEN_DAYS":
                                case 11:
                                    m.conversion_lag_bucket = 11;
                                    break;
                                case "TEN_TO_ELEVEN_DAYS":
                                case 12:
                                    m.conversion_lag_bucket = 12;
                                    break;
                                case "ELEVEN_TO_TWELVE_DAYS":
                                case 13:
                                    m.conversion_lag_bucket = 13;
                                    break;
                                case "TWELVE_TO_THIRTEEN_DAYS":
                                case 14:
                                    m.conversion_lag_bucket = 14;
                                    break;
                                case "THIRTEEN_TO_FOURTEEN_DAYS":
                                case 15:
                                    m.conversion_lag_bucket = 15;
                                    break;
                                case "FOURTEEN_TO_TWENTY_ONE_DAYS":
                                case 16:
                                    m.conversion_lag_bucket = 16;
                                    break;
                                case "TWENTY_ONE_TO_THIRTY_DAYS":
                                case 17:
                                    m.conversion_lag_bucket = 17;
                                    break;
                                case "THIRTY_TO_FORTY_FIVE_DAYS":
                                case 18:
                                    m.conversion_lag_bucket = 18;
                                    break;
                                case "FORTY_FIVE_TO_SIXTY_DAYS":
                                case 19:
                                    m.conversion_lag_bucket = 19;
                                    break;
                                case "SIXTY_TO_NINETY_DAYS":
                                case 20:
                                    m.conversion_lag_bucket = 20;
                                    break;
                                }
                                switch (d.conversion_or_adjustment_lag_bucket) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_or_adjustment_lag_bucket = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_or_adjustment_lag_bucket = 1;
                                    break;
                                case "CONVERSION_LESS_THAN_ONE_DAY":
                                case 2:
                                    m.conversion_or_adjustment_lag_bucket = 2;
                                    break;
                                case "CONVERSION_ONE_TO_TWO_DAYS":
                                case 3:
                                    m.conversion_or_adjustment_lag_bucket = 3;
                                    break;
                                case "CONVERSION_TWO_TO_THREE_DAYS":
                                case 4:
                                    m.conversion_or_adjustment_lag_bucket = 4;
                                    break;
                                case "CONVERSION_THREE_TO_FOUR_DAYS":
                                case 5:
                                    m.conversion_or_adjustment_lag_bucket = 5;
                                    break;
                                case "CONVERSION_FOUR_TO_FIVE_DAYS":
                                case 6:
                                    m.conversion_or_adjustment_lag_bucket = 6;
                                    break;
                                case "CONVERSION_FIVE_TO_SIX_DAYS":
                                case 7:
                                    m.conversion_or_adjustment_lag_bucket = 7;
                                    break;
                                case "CONVERSION_SIX_TO_SEVEN_DAYS":
                                case 8:
                                    m.conversion_or_adjustment_lag_bucket = 8;
                                    break;
                                case "CONVERSION_SEVEN_TO_EIGHT_DAYS":
                                case 9:
                                    m.conversion_or_adjustment_lag_bucket = 9;
                                    break;
                                case "CONVERSION_EIGHT_TO_NINE_DAYS":
                                case 10:
                                    m.conversion_or_adjustment_lag_bucket = 10;
                                    break;
                                case "CONVERSION_NINE_TO_TEN_DAYS":
                                case 11:
                                    m.conversion_or_adjustment_lag_bucket = 11;
                                    break;
                                case "CONVERSION_TEN_TO_ELEVEN_DAYS":
                                case 12:
                                    m.conversion_or_adjustment_lag_bucket = 12;
                                    break;
                                case "CONVERSION_ELEVEN_TO_TWELVE_DAYS":
                                case 13:
                                    m.conversion_or_adjustment_lag_bucket = 13;
                                    break;
                                case "CONVERSION_TWELVE_TO_THIRTEEN_DAYS":
                                case 14:
                                    m.conversion_or_adjustment_lag_bucket = 14;
                                    break;
                                case "CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS":
                                case 15:
                                    m.conversion_or_adjustment_lag_bucket = 15;
                                    break;
                                case "CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS":
                                case 16:
                                    m.conversion_or_adjustment_lag_bucket = 16;
                                    break;
                                case "CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS":
                                case 17:
                                    m.conversion_or_adjustment_lag_bucket = 17;
                                    break;
                                case "CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS":
                                case 18:
                                    m.conversion_or_adjustment_lag_bucket = 18;
                                    break;
                                case "CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS":
                                case 19:
                                    m.conversion_or_adjustment_lag_bucket = 19;
                                    break;
                                case "CONVERSION_SIXTY_TO_NINETY_DAYS":
                                case 20:
                                    m.conversion_or_adjustment_lag_bucket = 20;
                                    break;
                                case "ADJUSTMENT_LESS_THAN_ONE_DAY":
                                case 21:
                                    m.conversion_or_adjustment_lag_bucket = 21;
                                    break;
                                case "ADJUSTMENT_ONE_TO_TWO_DAYS":
                                case 22:
                                    m.conversion_or_adjustment_lag_bucket = 22;
                                    break;
                                case "ADJUSTMENT_TWO_TO_THREE_DAYS":
                                case 23:
                                    m.conversion_or_adjustment_lag_bucket = 23;
                                    break;
                                case "ADJUSTMENT_THREE_TO_FOUR_DAYS":
                                case 24:
                                    m.conversion_or_adjustment_lag_bucket = 24;
                                    break;
                                case "ADJUSTMENT_FOUR_TO_FIVE_DAYS":
                                case 25:
                                    m.conversion_or_adjustment_lag_bucket = 25;
                                    break;
                                case "ADJUSTMENT_FIVE_TO_SIX_DAYS":
                                case 26:
                                    m.conversion_or_adjustment_lag_bucket = 26;
                                    break;
                                case "ADJUSTMENT_SIX_TO_SEVEN_DAYS":
                                case 27:
                                    m.conversion_or_adjustment_lag_bucket = 27;
                                    break;
                                case "ADJUSTMENT_SEVEN_TO_EIGHT_DAYS":
                                case 28:
                                    m.conversion_or_adjustment_lag_bucket = 28;
                                    break;
                                case "ADJUSTMENT_EIGHT_TO_NINE_DAYS":
                                case 29:
                                    m.conversion_or_adjustment_lag_bucket = 29;
                                    break;
                                case "ADJUSTMENT_NINE_TO_TEN_DAYS":
                                case 30:
                                    m.conversion_or_adjustment_lag_bucket = 30;
                                    break;
                                case "ADJUSTMENT_TEN_TO_ELEVEN_DAYS":
                                case 31:
                                    m.conversion_or_adjustment_lag_bucket = 31;
                                    break;
                                case "ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS":
                                case 32:
                                    m.conversion_or_adjustment_lag_bucket = 32;
                                    break;
                                case "ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS":
                                case 33:
                                    m.conversion_or_adjustment_lag_bucket = 33;
                                    break;
                                case "ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS":
                                case 34:
                                    m.conversion_or_adjustment_lag_bucket = 34;
                                    break;
                                case "ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS":
                                case 35:
                                    m.conversion_or_adjustment_lag_bucket = 35;
                                    break;
                                case "ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS":
                                case 36:
                                    m.conversion_or_adjustment_lag_bucket = 36;
                                    break;
                                case "ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS":
                                case 37:
                                    m.conversion_or_adjustment_lag_bucket = 37;
                                    break;
                                case "ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS":
                                case 38:
                                    m.conversion_or_adjustment_lag_bucket = 38;
                                    break;
                                case "ADJUSTMENT_SIXTY_TO_NINETY_DAYS":
                                case 39:
                                    m.conversion_or_adjustment_lag_bucket = 39;
                                    break;
                                case "ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS":
                                case 40:
                                    m.conversion_or_adjustment_lag_bucket = 40;
                                    break;
                                case "CONVERSION_UNKNOWN":
                                case 41:
                                    m.conversion_or_adjustment_lag_bucket = 41;
                                    break;
                                case "ADJUSTMENT_UNKNOWN":
                                case 42:
                                    m.conversion_or_adjustment_lag_bucket = 42;
                                    break;
                                }
                                if (d.date != null) {
                                    m.date = String(d.date);
                                }
                                switch (d.day_of_week) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.day_of_week = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.day_of_week = 1;
                                    break;
                                case "MONDAY":
                                case 2:
                                    m.day_of_week = 2;
                                    break;
                                case "TUESDAY":
                                case 3:
                                    m.day_of_week = 3;
                                    break;
                                case "WEDNESDAY":
                                case 4:
                                    m.day_of_week = 4;
                                    break;
                                case "THURSDAY":
                                case 5:
                                    m.day_of_week = 5;
                                    break;
                                case "FRIDAY":
                                case 6:
                                    m.day_of_week = 6;
                                    break;
                                case "SATURDAY":
                                case 7:
                                    m.day_of_week = 7;
                                    break;
                                case "SUNDAY":
                                case 8:
                                    m.day_of_week = 8;
                                    break;
                                }
                                switch (d.device) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device = 2;
                                    break;
                                case "TABLET":
                                case 3:
                                    m.device = 3;
                                    break;
                                case "DESKTOP":
                                case 4:
                                    m.device = 4;
                                    break;
                                case "CONNECTED_TV":
                                case 6:
                                    m.device = 6;
                                    break;
                                case "OTHER":
                                case 5:
                                    m.device = 5;
                                    break;
                                }
                                switch (d.external_conversion_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.external_conversion_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.external_conversion_source = 1;
                                    break;
                                case "WEBPAGE":
                                case 2:
                                    m.external_conversion_source = 2;
                                    break;
                                case "ANALYTICS":
                                case 3:
                                    m.external_conversion_source = 3;
                                    break;
                                case "UPLOAD":
                                case 4:
                                    m.external_conversion_source = 4;
                                    break;
                                case "AD_CALL_METRICS":
                                case 5:
                                    m.external_conversion_source = 5;
                                    break;
                                case "WEBSITE_CALL_METRICS":
                                case 6:
                                    m.external_conversion_source = 6;
                                    break;
                                case "STORE_VISITS":
                                case 7:
                                    m.external_conversion_source = 7;
                                    break;
                                case "ANDROID_IN_APP":
                                case 8:
                                    m.external_conversion_source = 8;
                                    break;
                                case "IOS_IN_APP":
                                case 9:
                                    m.external_conversion_source = 9;
                                    break;
                                case "IOS_FIRST_OPEN":
                                case 10:
                                    m.external_conversion_source = 10;
                                    break;
                                case "APP_UNSPECIFIED":
                                case 11:
                                    m.external_conversion_source = 11;
                                    break;
                                case "ANDROID_FIRST_OPEN":
                                case 12:
                                    m.external_conversion_source = 12;
                                    break;
                                case "UPLOAD_CALLS":
                                case 13:
                                    m.external_conversion_source = 13;
                                    break;
                                case "FIREBASE":
                                case 14:
                                    m.external_conversion_source = 14;
                                    break;
                                case "CLICK_TO_CALL":
                                case 15:
                                    m.external_conversion_source = 15;
                                    break;
                                case "SALESFORCE":
                                case 16:
                                    m.external_conversion_source = 16;
                                    break;
                                case "STORE_SALES_CRM":
                                case 17:
                                    m.external_conversion_source = 17;
                                    break;
                                case "STORE_SALES_PAYMENT_NETWORK":
                                case 18:
                                    m.external_conversion_source = 18;
                                    break;
                                case "GOOGLE_PLAY":
                                case 19:
                                    m.external_conversion_source = 19;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS":
                                case 20:
                                    m.external_conversion_source = 20;
                                    break;
                                case "GOOGLE_ATTRIBUTION":
                                case 21:
                                    m.external_conversion_source = 21;
                                    break;
                                case "STORE_SALES_DIRECT_UPLOAD":
                                case 23:
                                    m.external_conversion_source = 23;
                                    break;
                                case "STORE_SALES":
                                case 24:
                                    m.external_conversion_source = 24;
                                    break;
                                case "SEARCH_ADS_360":
                                case 25:
                                    m.external_conversion_source = 25;
                                    break;
                                case "GOOGLE_HOSTED":
                                case 27:
                                    m.external_conversion_source = 27;
                                    break;
                                case "FLOODLIGHT":
                                case 29:
                                    m.external_conversion_source = 29;
                                    break;
                                case "ANALYTICS_SEARCH_ADS_360":
                                case 31:
                                    m.external_conversion_source = 31;
                                    break;
                                case "FIREBASE_SEARCH_ADS_360":
                                case 33:
                                    m.external_conversion_source = 33;
                                    break;
                                case "DISPLAY_AND_VIDEO_360_FLOODLIGHT":
                                case 34:
                                    m.external_conversion_source = 34;
                                    break;
                                }
                                if (d.geo_target_airport != null) {
                                    m.geo_target_airport = String(d.geo_target_airport);
                                }
                                if (d.geo_target_canton != null) {
                                    m.geo_target_canton = String(d.geo_target_canton);
                                }
                                if (d.geo_target_city != null) {
                                    m.geo_target_city = String(d.geo_target_city);
                                }
                                if (d.geo_target_country != null) {
                                    m.geo_target_country = String(d.geo_target_country);
                                }
                                if (d.geo_target_county != null) {
                                    m.geo_target_county = String(d.geo_target_county);
                                }
                                if (d.geo_target_district != null) {
                                    m.geo_target_district = String(d.geo_target_district);
                                }
                                if (d.geo_target_metro != null) {
                                    m.geo_target_metro = String(d.geo_target_metro);
                                }
                                if (d.geo_target_most_specific_location != null) {
                                    m.geo_target_most_specific_location = String(d.geo_target_most_specific_location);
                                }
                                if (d.geo_target_postal_code != null) {
                                    m.geo_target_postal_code = String(d.geo_target_postal_code);
                                }
                                if (d.geo_target_province != null) {
                                    m.geo_target_province = String(d.geo_target_province);
                                }
                                if (d.geo_target_region != null) {
                                    m.geo_target_region = String(d.geo_target_region);
                                }
                                if (d.geo_target_state != null) {
                                    m.geo_target_state = String(d.geo_target_state);
                                }
                                if (d.hotel_booking_window_days != null) {
                                    if ($util.Long)
                                        (m.hotel_booking_window_days = $util.Long.fromValue(d.hotel_booking_window_days)).unsigned = false;
                                    else if (typeof d.hotel_booking_window_days === "string")
                                        m.hotel_booking_window_days = parseInt(d.hotel_booking_window_days, 10);
                                    else if (typeof d.hotel_booking_window_days === "number")
                                        m.hotel_booking_window_days = d.hotel_booking_window_days;
                                    else if (typeof d.hotel_booking_window_days === "object")
                                        m.hotel_booking_window_days = new $util.LongBits(d.hotel_booking_window_days.low >>> 0, d.hotel_booking_window_days.high >>> 0).toNumber();
                                }
                                if (d.hotel_center_id != null) {
                                    if ($util.Long)
                                        (m.hotel_center_id = $util.Long.fromValue(d.hotel_center_id)).unsigned = false;
                                    else if (typeof d.hotel_center_id === "string")
                                        m.hotel_center_id = parseInt(d.hotel_center_id, 10);
                                    else if (typeof d.hotel_center_id === "number")
                                        m.hotel_center_id = d.hotel_center_id;
                                    else if (typeof d.hotel_center_id === "object")
                                        m.hotel_center_id = new $util.LongBits(d.hotel_center_id.low >>> 0, d.hotel_center_id.high >>> 0).toNumber();
                                }
                                if (d.hotel_check_in_date != null) {
                                    m.hotel_check_in_date = String(d.hotel_check_in_date);
                                }
                                switch (d.hotel_check_in_day_of_week) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.hotel_check_in_day_of_week = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.hotel_check_in_day_of_week = 1;
                                    break;
                                case "MONDAY":
                                case 2:
                                    m.hotel_check_in_day_of_week = 2;
                                    break;
                                case "TUESDAY":
                                case 3:
                                    m.hotel_check_in_day_of_week = 3;
                                    break;
                                case "WEDNESDAY":
                                case 4:
                                    m.hotel_check_in_day_of_week = 4;
                                    break;
                                case "THURSDAY":
                                case 5:
                                    m.hotel_check_in_day_of_week = 5;
                                    break;
                                case "FRIDAY":
                                case 6:
                                    m.hotel_check_in_day_of_week = 6;
                                    break;
                                case "SATURDAY":
                                case 7:
                                    m.hotel_check_in_day_of_week = 7;
                                    break;
                                case "SUNDAY":
                                case 8:
                                    m.hotel_check_in_day_of_week = 8;
                                    break;
                                }
                                if (d.hotel_city != null) {
                                    m.hotel_city = String(d.hotel_city);
                                }
                                if (d.hotel_class != null) {
                                    m.hotel_class = d.hotel_class | 0;
                                }
                                if (d.hotel_country != null) {
                                    m.hotel_country = String(d.hotel_country);
                                }
                                switch (d.hotel_date_selection_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.hotel_date_selection_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.hotel_date_selection_type = 1;
                                    break;
                                case "DEFAULT_SELECTION":
                                case 50:
                                    m.hotel_date_selection_type = 50;
                                    break;
                                case "USER_SELECTED":
                                case 51:
                                    m.hotel_date_selection_type = 51;
                                    break;
                                }
                                if (d.hotel_length_of_stay != null) {
                                    m.hotel_length_of_stay = d.hotel_length_of_stay | 0;
                                }
                                if (d.hotel_rate_rule_id != null) {
                                    m.hotel_rate_rule_id = String(d.hotel_rate_rule_id);
                                }
                                switch (d.hotel_rate_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.hotel_rate_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.hotel_rate_type = 1;
                                    break;
                                case "UNAVAILABLE":
                                case 2:
                                    m.hotel_rate_type = 2;
                                    break;
                                case "PUBLIC_RATE":
                                case 3:
                                    m.hotel_rate_type = 3;
                                    break;
                                case "QUALIFIED_RATE":
                                case 4:
                                    m.hotel_rate_type = 4;
                                    break;
                                case "PRIVATE_RATE":
                                case 5:
                                    m.hotel_rate_type = 5;
                                    break;
                                }
                                switch (d.hotel_price_bucket) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.hotel_price_bucket = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.hotel_price_bucket = 1;
                                    break;
                                case "LOWEST_UNIQUE":
                                case 2:
                                    m.hotel_price_bucket = 2;
                                    break;
                                case "LOWEST_TIED":
                                case 3:
                                    m.hotel_price_bucket = 3;
                                    break;
                                case "NOT_LOWEST":
                                case 4:
                                    m.hotel_price_bucket = 4;
                                    break;
                                case "ONLY_PARTNER_SHOWN":
                                case 5:
                                    m.hotel_price_bucket = 5;
                                    break;
                                }
                                if (d.hotel_state != null) {
                                    m.hotel_state = String(d.hotel_state);
                                }
                                if (d.hour != null) {
                                    m.hour = d.hour | 0;
                                }
                                if (d.interaction_on_this_extension != null) {
                                    m.interaction_on_this_extension = Boolean(d.interaction_on_this_extension);
                                }
                                if (d.keyword != null) {
                                    if (typeof d.keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Segments.keyword: object expected");
                                    m.keyword = $root.google.ads.googleads.v11.common.Keyword.fromObject(d.keyword);
                                }
                                if (d.month != null) {
                                    m.month = String(d.month);
                                }
                                switch (d.month_of_year) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.month_of_year = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.month_of_year = 1;
                                    break;
                                case "JANUARY":
                                case 2:
                                    m.month_of_year = 2;
                                    break;
                                case "FEBRUARY":
                                case 3:
                                    m.month_of_year = 3;
                                    break;
                                case "MARCH":
                                case 4:
                                    m.month_of_year = 4;
                                    break;
                                case "APRIL":
                                case 5:
                                    m.month_of_year = 5;
                                    break;
                                case "MAY":
                                case 6:
                                    m.month_of_year = 6;
                                    break;
                                case "JUNE":
                                case 7:
                                    m.month_of_year = 7;
                                    break;
                                case "JULY":
                                case 8:
                                    m.month_of_year = 8;
                                    break;
                                case "AUGUST":
                                case 9:
                                    m.month_of_year = 9;
                                    break;
                                case "SEPTEMBER":
                                case 10:
                                    m.month_of_year = 10;
                                    break;
                                case "OCTOBER":
                                case 11:
                                    m.month_of_year = 11;
                                    break;
                                case "NOVEMBER":
                                case 12:
                                    m.month_of_year = 12;
                                    break;
                                case "DECEMBER":
                                case 13:
                                    m.month_of_year = 13;
                                    break;
                                }
                                if (d.partner_hotel_id != null) {
                                    m.partner_hotel_id = String(d.partner_hotel_id);
                                }
                                switch (d.placeholder_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.placeholder_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.placeholder_type = 1;
                                    break;
                                case "SITELINK":
                                case 2:
                                    m.placeholder_type = 2;
                                    break;
                                case "CALL":
                                case 3:
                                    m.placeholder_type = 3;
                                    break;
                                case "APP":
                                case 4:
                                    m.placeholder_type = 4;
                                    break;
                                case "LOCATION":
                                case 5:
                                    m.placeholder_type = 5;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 6:
                                    m.placeholder_type = 6;
                                    break;
                                case "CALLOUT":
                                case 7:
                                    m.placeholder_type = 7;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 8:
                                    m.placeholder_type = 8;
                                    break;
                                case "MESSAGE":
                                case 9:
                                    m.placeholder_type = 9;
                                    break;
                                case "PRICE":
                                case 10:
                                    m.placeholder_type = 10;
                                    break;
                                case "PROMOTION":
                                case 11:
                                    m.placeholder_type = 11;
                                    break;
                                case "AD_CUSTOMIZER":
                                case 12:
                                    m.placeholder_type = 12;
                                    break;
                                case "DYNAMIC_EDUCATION":
                                case 13:
                                    m.placeholder_type = 13;
                                    break;
                                case "DYNAMIC_FLIGHT":
                                case 14:
                                    m.placeholder_type = 14;
                                    break;
                                case "DYNAMIC_CUSTOM":
                                case 15:
                                    m.placeholder_type = 15;
                                    break;
                                case "DYNAMIC_HOTEL":
                                case 16:
                                    m.placeholder_type = 16;
                                    break;
                                case "DYNAMIC_REAL_ESTATE":
                                case 17:
                                    m.placeholder_type = 17;
                                    break;
                                case "DYNAMIC_TRAVEL":
                                case 18:
                                    m.placeholder_type = 18;
                                    break;
                                case "DYNAMIC_LOCAL":
                                case 19:
                                    m.placeholder_type = 19;
                                    break;
                                case "DYNAMIC_JOB":
                                case 20:
                                    m.placeholder_type = 20;
                                    break;
                                case "IMAGE":
                                case 21:
                                    m.placeholder_type = 21;
                                    break;
                                }
                                if (d.product_aggregator_id != null) {
                                    if ($util.Long)
                                        (m.product_aggregator_id = $util.Long.fromValue(d.product_aggregator_id)).unsigned = false;
                                    else if (typeof d.product_aggregator_id === "string")
                                        m.product_aggregator_id = parseInt(d.product_aggregator_id, 10);
                                    else if (typeof d.product_aggregator_id === "number")
                                        m.product_aggregator_id = d.product_aggregator_id;
                                    else if (typeof d.product_aggregator_id === "object")
                                        m.product_aggregator_id = new $util.LongBits(d.product_aggregator_id.low >>> 0, d.product_aggregator_id.high >>> 0).toNumber();
                                }
                                if (d.product_bidding_category_level1 != null) {
                                    m.product_bidding_category_level1 = String(d.product_bidding_category_level1);
                                }
                                if (d.product_bidding_category_level2 != null) {
                                    m.product_bidding_category_level2 = String(d.product_bidding_category_level2);
                                }
                                if (d.product_bidding_category_level3 != null) {
                                    m.product_bidding_category_level3 = String(d.product_bidding_category_level3);
                                }
                                if (d.product_bidding_category_level4 != null) {
                                    m.product_bidding_category_level4 = String(d.product_bidding_category_level4);
                                }
                                if (d.product_bidding_category_level5 != null) {
                                    m.product_bidding_category_level5 = String(d.product_bidding_category_level5);
                                }
                                if (d.product_brand != null) {
                                    m.product_brand = String(d.product_brand);
                                }
                                switch (d.product_channel) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.product_channel = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.product_channel = 1;
                                    break;
                                case "ONLINE":
                                case 2:
                                    m.product_channel = 2;
                                    break;
                                case "LOCAL":
                                case 3:
                                    m.product_channel = 3;
                                    break;
                                }
                                switch (d.product_channel_exclusivity) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.product_channel_exclusivity = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.product_channel_exclusivity = 1;
                                    break;
                                case "SINGLE_CHANNEL":
                                case 2:
                                    m.product_channel_exclusivity = 2;
                                    break;
                                case "MULTI_CHANNEL":
                                case 3:
                                    m.product_channel_exclusivity = 3;
                                    break;
                                }
                                switch (d.product_condition) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.product_condition = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.product_condition = 1;
                                    break;
                                case "NEW":
                                case 3:
                                    m.product_condition = 3;
                                    break;
                                case "REFURBISHED":
                                case 4:
                                    m.product_condition = 4;
                                    break;
                                case "USED":
                                case 5:
                                    m.product_condition = 5;
                                    break;
                                }
                                if (d.product_country != null) {
                                    m.product_country = String(d.product_country);
                                }
                                if (d.product_custom_attribute0 != null) {
                                    m.product_custom_attribute0 = String(d.product_custom_attribute0);
                                }
                                if (d.product_custom_attribute1 != null) {
                                    m.product_custom_attribute1 = String(d.product_custom_attribute1);
                                }
                                if (d.product_custom_attribute2 != null) {
                                    m.product_custom_attribute2 = String(d.product_custom_attribute2);
                                }
                                if (d.product_custom_attribute3 != null) {
                                    m.product_custom_attribute3 = String(d.product_custom_attribute3);
                                }
                                if (d.product_custom_attribute4 != null) {
                                    m.product_custom_attribute4 = String(d.product_custom_attribute4);
                                }
                                if (d.product_item_id != null) {
                                    m.product_item_id = String(d.product_item_id);
                                }
                                if (d.product_language != null) {
                                    m.product_language = String(d.product_language);
                                }
                                if (d.product_merchant_id != null) {
                                    if ($util.Long)
                                        (m.product_merchant_id = $util.Long.fromValue(d.product_merchant_id)).unsigned = false;
                                    else if (typeof d.product_merchant_id === "string")
                                        m.product_merchant_id = parseInt(d.product_merchant_id, 10);
                                    else if (typeof d.product_merchant_id === "number")
                                        m.product_merchant_id = d.product_merchant_id;
                                    else if (typeof d.product_merchant_id === "object")
                                        m.product_merchant_id = new $util.LongBits(d.product_merchant_id.low >>> 0, d.product_merchant_id.high >>> 0).toNumber();
                                }
                                if (d.product_store_id != null) {
                                    m.product_store_id = String(d.product_store_id);
                                }
                                if (d.product_title != null) {
                                    m.product_title = String(d.product_title);
                                }
                                if (d.product_type_l1 != null) {
                                    m.product_type_l1 = String(d.product_type_l1);
                                }
                                if (d.product_type_l2 != null) {
                                    m.product_type_l2 = String(d.product_type_l2);
                                }
                                if (d.product_type_l3 != null) {
                                    m.product_type_l3 = String(d.product_type_l3);
                                }
                                if (d.product_type_l4 != null) {
                                    m.product_type_l4 = String(d.product_type_l4);
                                }
                                if (d.product_type_l5 != null) {
                                    m.product_type_l5 = String(d.product_type_l5);
                                }
                                if (d.quarter != null) {
                                    m.quarter = String(d.quarter);
                                }
                                switch (d.recommendation_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.recommendation_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.recommendation_type = 1;
                                    break;
                                case "CAMPAIGN_BUDGET":
                                case 2:
                                    m.recommendation_type = 2;
                                    break;
                                case "KEYWORD":
                                case 3:
                                    m.recommendation_type = 3;
                                    break;
                                case "TEXT_AD":
                                case 4:
                                    m.recommendation_type = 4;
                                    break;
                                case "TARGET_CPA_OPT_IN":
                                case 5:
                                    m.recommendation_type = 5;
                                    break;
                                case "MAXIMIZE_CONVERSIONS_OPT_IN":
                                case 6:
                                    m.recommendation_type = 6;
                                    break;
                                case "ENHANCED_CPC_OPT_IN":
                                case 7:
                                    m.recommendation_type = 7;
                                    break;
                                case "SEARCH_PARTNERS_OPT_IN":
                                case 8:
                                    m.recommendation_type = 8;
                                    break;
                                case "MAXIMIZE_CLICKS_OPT_IN":
                                case 9:
                                    m.recommendation_type = 9;
                                    break;
                                case "OPTIMIZE_AD_ROTATION":
                                case 10:
                                    m.recommendation_type = 10;
                                    break;
                                case "CALLOUT_EXTENSION":
                                case 11:
                                    m.recommendation_type = 11;
                                    break;
                                case "SITELINK_EXTENSION":
                                case 12:
                                    m.recommendation_type = 12;
                                    break;
                                case "CALL_EXTENSION":
                                case 13:
                                    m.recommendation_type = 13;
                                    break;
                                case "KEYWORD_MATCH_TYPE":
                                case 14:
                                    m.recommendation_type = 14;
                                    break;
                                case "MOVE_UNUSED_BUDGET":
                                case 15:
                                    m.recommendation_type = 15;
                                    break;
                                case "FORECASTING_CAMPAIGN_BUDGET":
                                case 16:
                                    m.recommendation_type = 16;
                                    break;
                                case "TARGET_ROAS_OPT_IN":
                                case 17:
                                    m.recommendation_type = 17;
                                    break;
                                case "RESPONSIVE_SEARCH_AD":
                                case 18:
                                    m.recommendation_type = 18;
                                    break;
                                case "MARGINAL_ROI_CAMPAIGN_BUDGET":
                                case 19:
                                    m.recommendation_type = 19;
                                    break;
                                case "USE_BROAD_MATCH_KEYWORD":
                                case 20:
                                    m.recommendation_type = 20;
                                    break;
                                case "RESPONSIVE_SEARCH_AD_ASSET":
                                case 21:
                                    m.recommendation_type = 21;
                                    break;
                                case "UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX":
                                case 22:
                                    m.recommendation_type = 22;
                                    break;
                                case "RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH":
                                case 23:
                                    m.recommendation_type = 23;
                                    break;
                                }
                                switch (d.search_engine_results_page_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.search_engine_results_page_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.search_engine_results_page_type = 1;
                                    break;
                                case "ADS_ONLY":
                                case 2:
                                    m.search_engine_results_page_type = 2;
                                    break;
                                case "ORGANIC_ONLY":
                                case 3:
                                    m.search_engine_results_page_type = 3;
                                    break;
                                case "ADS_AND_ORGANIC":
                                case 4:
                                    m.search_engine_results_page_type = 4;
                                    break;
                                }
                                switch (d.search_term_match_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.search_term_match_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.search_term_match_type = 1;
                                    break;
                                case "BROAD":
                                case 2:
                                    m.search_term_match_type = 2;
                                    break;
                                case "EXACT":
                                case 3:
                                    m.search_term_match_type = 3;
                                    break;
                                case "PHRASE":
                                case 4:
                                    m.search_term_match_type = 4;
                                    break;
                                case "NEAR_EXACT":
                                case 5:
                                    m.search_term_match_type = 5;
                                    break;
                                case "NEAR_PHRASE":
                                case 6:
                                    m.search_term_match_type = 6;
                                    break;
                                }
                                switch (d.slot) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.slot = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.slot = 1;
                                    break;
                                case "SEARCH_SIDE":
                                case 2:
                                    m.slot = 2;
                                    break;
                                case "SEARCH_TOP":
                                case 3:
                                    m.slot = 3;
                                    break;
                                case "SEARCH_OTHER":
                                case 4:
                                    m.slot = 4;
                                    break;
                                case "CONTENT":
                                case 5:
                                    m.slot = 5;
                                    break;
                                case "SEARCH_PARTNER_TOP":
                                case 6:
                                    m.slot = 6;
                                    break;
                                case "SEARCH_PARTNER_OTHER":
                                case 7:
                                    m.slot = 7;
                                    break;
                                case "MIXED":
                                case 8:
                                    m.slot = 8;
                                    break;
                                }
                                switch (d.conversion_value_rule_primary_dimension) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_value_rule_primary_dimension = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_value_rule_primary_dimension = 1;
                                    break;
                                case "NO_RULE_APPLIED":
                                case 2:
                                    m.conversion_value_rule_primary_dimension = 2;
                                    break;
                                case "ORIGINAL":
                                case 3:
                                    m.conversion_value_rule_primary_dimension = 3;
                                    break;
                                case "NEW_VS_RETURNING_USER":
                                case 4:
                                    m.conversion_value_rule_primary_dimension = 4;
                                    break;
                                case "GEO_LOCATION":
                                case 5:
                                    m.conversion_value_rule_primary_dimension = 5;
                                    break;
                                case "DEVICE":
                                case 6:
                                    m.conversion_value_rule_primary_dimension = 6;
                                    break;
                                case "AUDIENCE":
                                case 7:
                                    m.conversion_value_rule_primary_dimension = 7;
                                    break;
                                case "MULTIPLE":
                                case 8:
                                    m.conversion_value_rule_primary_dimension = 8;
                                    break;
                                }
                                if (d.webpage != null) {
                                    m.webpage = String(d.webpage);
                                }
                                if (d.week != null) {
                                    m.week = String(d.week);
                                }
                                if (d.year != null) {
                                    m.year = d.year | 0;
                                }
                                if (d.sk_ad_network_conversion_value != null) {
                                    if ($util.Long)
                                        (m.sk_ad_network_conversion_value = $util.Long.fromValue(d.sk_ad_network_conversion_value)).unsigned = false;
                                    else if (typeof d.sk_ad_network_conversion_value === "string")
                                        m.sk_ad_network_conversion_value = parseInt(d.sk_ad_network_conversion_value, 10);
                                    else if (typeof d.sk_ad_network_conversion_value === "number")
                                        m.sk_ad_network_conversion_value = d.sk_ad_network_conversion_value;
                                    else if (typeof d.sk_ad_network_conversion_value === "object")
                                        m.sk_ad_network_conversion_value = new $util.LongBits(d.sk_ad_network_conversion_value.low >>> 0, d.sk_ad_network_conversion_value.high >>> 0).toNumber();
                                }
                                switch (d.sk_ad_network_user_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.sk_ad_network_user_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.sk_ad_network_user_type = 1;
                                    break;
                                case "UNAVAILABLE":
                                case 2:
                                    m.sk_ad_network_user_type = 2;
                                    break;
                                case "NEW_INSTALLER":
                                case 3:
                                    m.sk_ad_network_user_type = 3;
                                    break;
                                case "REINSTALLER":
                                case 4:
                                    m.sk_ad_network_user_type = 4;
                                    break;
                                }
                                switch (d.sk_ad_network_ad_event_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.sk_ad_network_ad_event_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.sk_ad_network_ad_event_type = 1;
                                    break;
                                case "UNAVAILABLE":
                                case 2:
                                    m.sk_ad_network_ad_event_type = 2;
                                    break;
                                case "INTERACTION":
                                case 3:
                                    m.sk_ad_network_ad_event_type = 3;
                                    break;
                                case "VIEW":
                                case 4:
                                    m.sk_ad_network_ad_event_type = 4;
                                    break;
                                }
                                if (d.sk_ad_network_source_app != null) {
                                    if (typeof d.sk_ad_network_source_app !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Segments.sk_ad_network_source_app: object expected");
                                    m.sk_ad_network_source_app = $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp.fromObject(d.sk_ad_network_source_app);
                                }
                                switch (d.sk_ad_network_attribution_credit) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.sk_ad_network_attribution_credit = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.sk_ad_network_attribution_credit = 1;
                                    break;
                                case "UNAVAILABLE":
                                case 2:
                                    m.sk_ad_network_attribution_credit = 2;
                                    break;
                                case "WON":
                                case 3:
                                    m.sk_ad_network_attribution_credit = 3;
                                    break;
                                case "CONTRIBUTED":
                                case 4:
                                    m.sk_ad_network_attribution_credit = 4;
                                    break;
                                }
                                if (d.asset_interaction_target != null) {
                                    if (typeof d.asset_interaction_target !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Segments.asset_interaction_target: object expected");
                                    m.asset_interaction_target = $root.google.ads.googleads.v11.common.AssetInteractionTarget.fromObject(d.asset_interaction_target);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Segments message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @static
                             * @param {google.ads.googleads.v11.common.Segments} m Segments
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Segments.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.device = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.conversion_attribution_event_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.ad_network_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.day_of_week = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.hotel_check_in_day_of_week = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.hotel_date_selection_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.month_of_year = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.placeholder_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.search_term_match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.slot = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.click_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.product_channel = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.product_channel_exclusivity = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.product_condition = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.conversion_lag_bucket = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.conversion_or_adjustment_lag_bucket = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.conversion_action_category = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.external_conversion_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.keyword = null;
                                    d.search_engine_results_page_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.hotel_rate_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.hotel_price_bucket = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.budget_campaign_association_status = null;
                                    d.ad_destination_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.conversion_value_rule_primary_dimension = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.recommendation_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.sk_ad_network_user_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.sk_ad_network_ad_event_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.sk_ad_network_attribution_credit = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.DeviceEnum.Device[m.device] : m.device;
                                }
                                if (m.conversion_attribution_event_type != null && m.hasOwnProperty("conversion_attribution_event_type")) {
                                    d.conversion_attribution_event_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum.ConversionAttributionEventType[m.conversion_attribution_event_type] : m.conversion_attribution_event_type;
                                }
                                if (m.ad_network_type != null && m.hasOwnProperty("ad_network_type")) {
                                    d.ad_network_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AdNetworkTypeEnum.AdNetworkType[m.ad_network_type] : m.ad_network_type;
                                }
                                if (m.day_of_week != null && m.hasOwnProperty("day_of_week")) {
                                    d.day_of_week = o.enums === String ? $root.google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek[m.day_of_week] : m.day_of_week;
                                }
                                if (m.hotel_check_in_day_of_week != null && m.hasOwnProperty("hotel_check_in_day_of_week")) {
                                    d.hotel_check_in_day_of_week = o.enums === String ? $root.google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek[m.hotel_check_in_day_of_week] : m.hotel_check_in_day_of_week;
                                }
                                if (m.hotel_date_selection_type != null && m.hasOwnProperty("hotel_date_selection_type")) {
                                    d.hotel_date_selection_type = o.enums === String ? $root.google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.HotelDateSelectionType[m.hotel_date_selection_type] : m.hotel_date_selection_type;
                                }
                                if (m.month_of_year != null && m.hasOwnProperty("month_of_year")) {
                                    d.month_of_year = o.enums === String ? $root.google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear[m.month_of_year] : m.month_of_year;
                                }
                                if (m.placeholder_type != null && m.hasOwnProperty("placeholder_type")) {
                                    d.placeholder_type = o.enums === String ? $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType[m.placeholder_type] : m.placeholder_type;
                                }
                                if (m.search_term_match_type != null && m.hasOwnProperty("search_term_match_type")) {
                                    d.search_term_match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SearchTermMatchTypeEnum.SearchTermMatchType[m.search_term_match_type] : m.search_term_match_type;
                                }
                                if (m.slot != null && m.hasOwnProperty("slot")) {
                                    d.slot = o.enums === String ? $root.google.ads.googleads.v11.enums.SlotEnum.Slot[m.slot] : m.slot;
                                }
                                if (m.click_type != null && m.hasOwnProperty("click_type")) {
                                    d.click_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ClickTypeEnum.ClickType[m.click_type] : m.click_type;
                                }
                                if (m.product_channel != null && m.hasOwnProperty("product_channel")) {
                                    d.product_channel = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductChannelEnum.ProductChannel[m.product_channel] : m.product_channel;
                                }
                                if (m.product_channel_exclusivity != null && m.hasOwnProperty("product_channel_exclusivity")) {
                                    d.product_channel_exclusivity = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.ProductChannelExclusivity[m.product_channel_exclusivity] : m.product_channel_exclusivity;
                                }
                                if (m.product_condition != null && m.hasOwnProperty("product_condition")) {
                                    d.product_condition = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductConditionEnum.ProductCondition[m.product_condition] : m.product_condition;
                                }
                                if (m.conversion_lag_bucket != null && m.hasOwnProperty("conversion_lag_bucket")) {
                                    d.conversion_lag_bucket = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionLagBucketEnum.ConversionLagBucket[m.conversion_lag_bucket] : m.conversion_lag_bucket;
                                }
                                if (m.conversion_or_adjustment_lag_bucket != null && m.hasOwnProperty("conversion_or_adjustment_lag_bucket")) {
                                    d.conversion_or_adjustment_lag_bucket = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket[m.conversion_or_adjustment_lag_bucket] : m.conversion_or_adjustment_lag_bucket;
                                }
                                if (m.conversion_action_category != null && m.hasOwnProperty("conversion_action_category")) {
                                    d.conversion_action_category = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory[m.conversion_action_category] : m.conversion_action_category;
                                }
                                if (m.external_conversion_source != null && m.hasOwnProperty("external_conversion_source")) {
                                    d.external_conversion_source = o.enums === String ? $root.google.ads.googleads.v11.enums.ExternalConversionSourceEnum.ExternalConversionSource[m.external_conversion_source] : m.external_conversion_source;
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = $root.google.ads.googleads.v11.common.Keyword.toObject(m.keyword, o);
                                }
                                if (m.search_engine_results_page_type != null && m.hasOwnProperty("search_engine_results_page_type")) {
                                    d.search_engine_results_page_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType[m.search_engine_results_page_type] : m.search_engine_results_page_type;
                                }
                                if (m.hotel_rate_type != null && m.hasOwnProperty("hotel_rate_type")) {
                                    d.hotel_rate_type = o.enums === String ? $root.google.ads.googleads.v11.enums.HotelRateTypeEnum.HotelRateType[m.hotel_rate_type] : m.hotel_rate_type;
                                }
                                if (m.hotel_price_bucket != null && m.hasOwnProperty("hotel_price_bucket")) {
                                    d.hotel_price_bucket = o.enums === String ? $root.google.ads.googleads.v11.enums.HotelPriceBucketEnum.HotelPriceBucket[m.hotel_price_bucket] : m.hotel_price_bucket;
                                }
                                if (m.date != null && m.hasOwnProperty("date")) {
                                    d.date = m.date;
                                    if (o.oneofs)
                                        d._date = "date";
                                }
                                if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                    if (typeof m.hotel_center_id === "number")
                                        d.hotel_center_id = o.longs === String ? String(m.hotel_center_id) : m.hotel_center_id;
                                    else
                                        d.hotel_center_id = o.longs === String ? $util.Long.prototype.toString.call(m.hotel_center_id) : o.longs === Number ? new $util.LongBits(m.hotel_center_id.low >>> 0, m.hotel_center_id.high >>> 0).toNumber() : m.hotel_center_id;
                                    if (o.oneofs)
                                        d._hotel_center_id = "hotel_center_id";
                                }
                                if (m.hotel_check_in_date != null && m.hasOwnProperty("hotel_check_in_date")) {
                                    d.hotel_check_in_date = m.hotel_check_in_date;
                                    if (o.oneofs)
                                        d._hotel_check_in_date = "hotel_check_in_date";
                                }
                                if (m.hotel_city != null && m.hasOwnProperty("hotel_city")) {
                                    d.hotel_city = m.hotel_city;
                                    if (o.oneofs)
                                        d._hotel_city = "hotel_city";
                                }
                                if (m.hotel_class != null && m.hasOwnProperty("hotel_class")) {
                                    d.hotel_class = m.hotel_class;
                                    if (o.oneofs)
                                        d._hotel_class = "hotel_class";
                                }
                                if (m.hotel_country != null && m.hasOwnProperty("hotel_country")) {
                                    d.hotel_country = m.hotel_country;
                                    if (o.oneofs)
                                        d._hotel_country = "hotel_country";
                                }
                                if (m.hotel_length_of_stay != null && m.hasOwnProperty("hotel_length_of_stay")) {
                                    d.hotel_length_of_stay = m.hotel_length_of_stay;
                                    if (o.oneofs)
                                        d._hotel_length_of_stay = "hotel_length_of_stay";
                                }
                                if (m.hotel_rate_rule_id != null && m.hasOwnProperty("hotel_rate_rule_id")) {
                                    d.hotel_rate_rule_id = m.hotel_rate_rule_id;
                                    if (o.oneofs)
                                        d._hotel_rate_rule_id = "hotel_rate_rule_id";
                                }
                                if (m.hotel_state != null && m.hasOwnProperty("hotel_state")) {
                                    d.hotel_state = m.hotel_state;
                                    if (o.oneofs)
                                        d._hotel_state = "hotel_state";
                                }
                                if (m.hour != null && m.hasOwnProperty("hour")) {
                                    d.hour = m.hour;
                                    if (o.oneofs)
                                        d._hour = "hour";
                                }
                                if (m.interaction_on_this_extension != null && m.hasOwnProperty("interaction_on_this_extension")) {
                                    d.interaction_on_this_extension = m.interaction_on_this_extension;
                                    if (o.oneofs)
                                        d._interaction_on_this_extension = "interaction_on_this_extension";
                                }
                                if (m.month != null && m.hasOwnProperty("month")) {
                                    d.month = m.month;
                                    if (o.oneofs)
                                        d._month = "month";
                                }
                                if (m.partner_hotel_id != null && m.hasOwnProperty("partner_hotel_id")) {
                                    d.partner_hotel_id = m.partner_hotel_id;
                                    if (o.oneofs)
                                        d._partner_hotel_id = "partner_hotel_id";
                                }
                                if (m.product_bidding_category_level1 != null && m.hasOwnProperty("product_bidding_category_level1")) {
                                    d.product_bidding_category_level1 = m.product_bidding_category_level1;
                                    if (o.oneofs)
                                        d._product_bidding_category_level1 = "product_bidding_category_level1";
                                }
                                if (m.product_bidding_category_level2 != null && m.hasOwnProperty("product_bidding_category_level2")) {
                                    d.product_bidding_category_level2 = m.product_bidding_category_level2;
                                    if (o.oneofs)
                                        d._product_bidding_category_level2 = "product_bidding_category_level2";
                                }
                                if (m.product_bidding_category_level3 != null && m.hasOwnProperty("product_bidding_category_level3")) {
                                    d.product_bidding_category_level3 = m.product_bidding_category_level3;
                                    if (o.oneofs)
                                        d._product_bidding_category_level3 = "product_bidding_category_level3";
                                }
                                if (m.product_bidding_category_level4 != null && m.hasOwnProperty("product_bidding_category_level4")) {
                                    d.product_bidding_category_level4 = m.product_bidding_category_level4;
                                    if (o.oneofs)
                                        d._product_bidding_category_level4 = "product_bidding_category_level4";
                                }
                                if (m.product_bidding_category_level5 != null && m.hasOwnProperty("product_bidding_category_level5")) {
                                    d.product_bidding_category_level5 = m.product_bidding_category_level5;
                                    if (o.oneofs)
                                        d._product_bidding_category_level5 = "product_bidding_category_level5";
                                }
                                if (m.product_brand != null && m.hasOwnProperty("product_brand")) {
                                    d.product_brand = m.product_brand;
                                    if (o.oneofs)
                                        d._product_brand = "product_brand";
                                }
                                if (m.product_country != null && m.hasOwnProperty("product_country")) {
                                    d.product_country = m.product_country;
                                    if (o.oneofs)
                                        d._product_country = "product_country";
                                }
                                if (m.product_custom_attribute0 != null && m.hasOwnProperty("product_custom_attribute0")) {
                                    d.product_custom_attribute0 = m.product_custom_attribute0;
                                    if (o.oneofs)
                                        d._product_custom_attribute0 = "product_custom_attribute0";
                                }
                                if (m.product_custom_attribute1 != null && m.hasOwnProperty("product_custom_attribute1")) {
                                    d.product_custom_attribute1 = m.product_custom_attribute1;
                                    if (o.oneofs)
                                        d._product_custom_attribute1 = "product_custom_attribute1";
                                }
                                if (m.product_custom_attribute2 != null && m.hasOwnProperty("product_custom_attribute2")) {
                                    d.product_custom_attribute2 = m.product_custom_attribute2;
                                    if (o.oneofs)
                                        d._product_custom_attribute2 = "product_custom_attribute2";
                                }
                                if (m.product_custom_attribute3 != null && m.hasOwnProperty("product_custom_attribute3")) {
                                    d.product_custom_attribute3 = m.product_custom_attribute3;
                                    if (o.oneofs)
                                        d._product_custom_attribute3 = "product_custom_attribute3";
                                }
                                if (m.product_custom_attribute4 != null && m.hasOwnProperty("product_custom_attribute4")) {
                                    d.product_custom_attribute4 = m.product_custom_attribute4;
                                    if (o.oneofs)
                                        d._product_custom_attribute4 = "product_custom_attribute4";
                                }
                                if (m.product_item_id != null && m.hasOwnProperty("product_item_id")) {
                                    d.product_item_id = m.product_item_id;
                                    if (o.oneofs)
                                        d._product_item_id = "product_item_id";
                                }
                                if (m.product_language != null && m.hasOwnProperty("product_language")) {
                                    d.product_language = m.product_language;
                                    if (o.oneofs)
                                        d._product_language = "product_language";
                                }
                                if (m.product_store_id != null && m.hasOwnProperty("product_store_id")) {
                                    d.product_store_id = m.product_store_id;
                                    if (o.oneofs)
                                        d._product_store_id = "product_store_id";
                                }
                                if (m.product_title != null && m.hasOwnProperty("product_title")) {
                                    d.product_title = m.product_title;
                                    if (o.oneofs)
                                        d._product_title = "product_title";
                                }
                                if (m.product_type_l1 != null && m.hasOwnProperty("product_type_l1")) {
                                    d.product_type_l1 = m.product_type_l1;
                                    if (o.oneofs)
                                        d._product_type_l1 = "product_type_l1";
                                }
                                if (m.product_type_l2 != null && m.hasOwnProperty("product_type_l2")) {
                                    d.product_type_l2 = m.product_type_l2;
                                    if (o.oneofs)
                                        d._product_type_l2 = "product_type_l2";
                                }
                                if (m.product_type_l3 != null && m.hasOwnProperty("product_type_l3")) {
                                    d.product_type_l3 = m.product_type_l3;
                                    if (o.oneofs)
                                        d._product_type_l3 = "product_type_l3";
                                }
                                if (m.product_type_l4 != null && m.hasOwnProperty("product_type_l4")) {
                                    d.product_type_l4 = m.product_type_l4;
                                    if (o.oneofs)
                                        d._product_type_l4 = "product_type_l4";
                                }
                                if (m.product_type_l5 != null && m.hasOwnProperty("product_type_l5")) {
                                    d.product_type_l5 = m.product_type_l5;
                                    if (o.oneofs)
                                        d._product_type_l5 = "product_type_l5";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.conversion_action_name != null && m.hasOwnProperty("conversion_action_name")) {
                                    d.conversion_action_name = m.conversion_action_name;
                                    if (o.oneofs)
                                        d._conversion_action_name = "conversion_action_name";
                                }
                                if (m.conversion_adjustment != null && m.hasOwnProperty("conversion_adjustment")) {
                                    d.conversion_adjustment = m.conversion_adjustment;
                                    if (o.oneofs)
                                        d._conversion_adjustment = "conversion_adjustment";
                                }
                                if (m.geo_target_airport != null && m.hasOwnProperty("geo_target_airport")) {
                                    d.geo_target_airport = m.geo_target_airport;
                                    if (o.oneofs)
                                        d._geo_target_airport = "geo_target_airport";
                                }
                                if (m.geo_target_canton != null && m.hasOwnProperty("geo_target_canton")) {
                                    d.geo_target_canton = m.geo_target_canton;
                                    if (o.oneofs)
                                        d._geo_target_canton = "geo_target_canton";
                                }
                                if (m.geo_target_city != null && m.hasOwnProperty("geo_target_city")) {
                                    d.geo_target_city = m.geo_target_city;
                                    if (o.oneofs)
                                        d._geo_target_city = "geo_target_city";
                                }
                                if (m.geo_target_country != null && m.hasOwnProperty("geo_target_country")) {
                                    d.geo_target_country = m.geo_target_country;
                                    if (o.oneofs)
                                        d._geo_target_country = "geo_target_country";
                                }
                                if (m.geo_target_county != null && m.hasOwnProperty("geo_target_county")) {
                                    d.geo_target_county = m.geo_target_county;
                                    if (o.oneofs)
                                        d._geo_target_county = "geo_target_county";
                                }
                                if (m.geo_target_district != null && m.hasOwnProperty("geo_target_district")) {
                                    d.geo_target_district = m.geo_target_district;
                                    if (o.oneofs)
                                        d._geo_target_district = "geo_target_district";
                                }
                                if (m.geo_target_metro != null && m.hasOwnProperty("geo_target_metro")) {
                                    d.geo_target_metro = m.geo_target_metro;
                                    if (o.oneofs)
                                        d._geo_target_metro = "geo_target_metro";
                                }
                                if (m.geo_target_most_specific_location != null && m.hasOwnProperty("geo_target_most_specific_location")) {
                                    d.geo_target_most_specific_location = m.geo_target_most_specific_location;
                                    if (o.oneofs)
                                        d._geo_target_most_specific_location = "geo_target_most_specific_location";
                                }
                                if (m.geo_target_postal_code != null && m.hasOwnProperty("geo_target_postal_code")) {
                                    d.geo_target_postal_code = m.geo_target_postal_code;
                                    if (o.oneofs)
                                        d._geo_target_postal_code = "geo_target_postal_code";
                                }
                                if (m.geo_target_province != null && m.hasOwnProperty("geo_target_province")) {
                                    d.geo_target_province = m.geo_target_province;
                                    if (o.oneofs)
                                        d._geo_target_province = "geo_target_province";
                                }
                                if (m.geo_target_region != null && m.hasOwnProperty("geo_target_region")) {
                                    d.geo_target_region = m.geo_target_region;
                                    if (o.oneofs)
                                        d._geo_target_region = "geo_target_region";
                                }
                                if (m.geo_target_state != null && m.hasOwnProperty("geo_target_state")) {
                                    d.geo_target_state = m.geo_target_state;
                                    if (o.oneofs)
                                        d._geo_target_state = "geo_target_state";
                                }
                                if (m.quarter != null && m.hasOwnProperty("quarter")) {
                                    d.quarter = m.quarter;
                                    if (o.oneofs)
                                        d._quarter = "quarter";
                                }
                                if (m.webpage != null && m.hasOwnProperty("webpage")) {
                                    d.webpage = m.webpage;
                                    if (o.oneofs)
                                        d._webpage = "webpage";
                                }
                                if (m.week != null && m.hasOwnProperty("week")) {
                                    d.week = m.week;
                                    if (o.oneofs)
                                        d._week = "week";
                                }
                                if (m.year != null && m.hasOwnProperty("year")) {
                                    d.year = m.year;
                                    if (o.oneofs)
                                        d._year = "year";
                                }
                                if (m.product_aggregator_id != null && m.hasOwnProperty("product_aggregator_id")) {
                                    if (typeof m.product_aggregator_id === "number")
                                        d.product_aggregator_id = o.longs === String ? String(m.product_aggregator_id) : m.product_aggregator_id;
                                    else
                                        d.product_aggregator_id = o.longs === String ? $util.Long.prototype.toString.call(m.product_aggregator_id) : o.longs === Number ? new $util.LongBits(m.product_aggregator_id.low >>> 0, m.product_aggregator_id.high >>> 0).toNumber() : m.product_aggregator_id;
                                    if (o.oneofs)
                                        d._product_aggregator_id = "product_aggregator_id";
                                }
                                if (m.product_merchant_id != null && m.hasOwnProperty("product_merchant_id")) {
                                    if (typeof m.product_merchant_id === "number")
                                        d.product_merchant_id = o.longs === String ? String(m.product_merchant_id) : m.product_merchant_id;
                                    else
                                        d.product_merchant_id = o.longs === String ? $util.Long.prototype.toString.call(m.product_merchant_id) : o.longs === Number ? new $util.LongBits(m.product_merchant_id.low >>> 0, m.product_merchant_id.high >>> 0).toNumber() : m.product_merchant_id;
                                    if (o.oneofs)
                                        d._product_merchant_id = "product_merchant_id";
                                }
                                if (m.budget_campaign_association_status != null && m.hasOwnProperty("budget_campaign_association_status")) {
                                    d.budget_campaign_association_status = $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus.toObject(m.budget_campaign_association_status, o);
                                }
                                if (m.hotel_booking_window_days != null && m.hasOwnProperty("hotel_booking_window_days")) {
                                    if (typeof m.hotel_booking_window_days === "number")
                                        d.hotel_booking_window_days = o.longs === String ? String(m.hotel_booking_window_days) : m.hotel_booking_window_days;
                                    else
                                        d.hotel_booking_window_days = o.longs === String ? $util.Long.prototype.toString.call(m.hotel_booking_window_days) : o.longs === Number ? new $util.LongBits(m.hotel_booking_window_days.low >>> 0, m.hotel_booking_window_days.high >>> 0).toNumber() : m.hotel_booking_window_days;
                                    if (o.oneofs)
                                        d._hotel_booking_window_days = "hotel_booking_window_days";
                                }
                                if (m.ad_destination_type != null && m.hasOwnProperty("ad_destination_type")) {
                                    d.ad_destination_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AdDestinationTypeEnum.AdDestinationType[m.ad_destination_type] : m.ad_destination_type;
                                }
                                if (m.sk_ad_network_conversion_value != null && m.hasOwnProperty("sk_ad_network_conversion_value")) {
                                    if (typeof m.sk_ad_network_conversion_value === "number")
                                        d.sk_ad_network_conversion_value = o.longs === String ? String(m.sk_ad_network_conversion_value) : m.sk_ad_network_conversion_value;
                                    else
                                        d.sk_ad_network_conversion_value = o.longs === String ? $util.Long.prototype.toString.call(m.sk_ad_network_conversion_value) : o.longs === Number ? new $util.LongBits(m.sk_ad_network_conversion_value.low >>> 0, m.sk_ad_network_conversion_value.high >>> 0).toNumber() : m.sk_ad_network_conversion_value;
                                    if (o.oneofs)
                                        d._sk_ad_network_conversion_value = "sk_ad_network_conversion_value";
                                }
                                if (m.conversion_value_rule_primary_dimension != null && m.hasOwnProperty("conversion_value_rule_primary_dimension")) {
                                    d.conversion_value_rule_primary_dimension = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension[m.conversion_value_rule_primary_dimension] : m.conversion_value_rule_primary_dimension;
                                }
                                if (m.asset_interaction_target != null && m.hasOwnProperty("asset_interaction_target")) {
                                    d.asset_interaction_target = $root.google.ads.googleads.v11.common.AssetInteractionTarget.toObject(m.asset_interaction_target, o);
                                    if (o.oneofs)
                                        d._asset_interaction_target = "asset_interaction_target";
                                }
                                if (m.recommendation_type != null && m.hasOwnProperty("recommendation_type")) {
                                    d.recommendation_type = o.enums === String ? $root.google.ads.googleads.v11.enums.RecommendationTypeEnum.RecommendationType[m.recommendation_type] : m.recommendation_type;
                                }
                                if (m.sk_ad_network_user_type != null && m.hasOwnProperty("sk_ad_network_user_type")) {
                                    d.sk_ad_network_user_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum.SkAdNetworkUserType[m.sk_ad_network_user_type] : m.sk_ad_network_user_type;
                                }
                                if (m.sk_ad_network_ad_event_type != null && m.hasOwnProperty("sk_ad_network_ad_event_type")) {
                                    d.sk_ad_network_ad_event_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType[m.sk_ad_network_ad_event_type] : m.sk_ad_network_ad_event_type;
                                }
                                if (m.sk_ad_network_source_app != null && m.hasOwnProperty("sk_ad_network_source_app")) {
                                    d.sk_ad_network_source_app = $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp.toObject(m.sk_ad_network_source_app, o);
                                    if (o.oneofs)
                                        d._sk_ad_network_source_app = "sk_ad_network_source_app";
                                }
                                if (m.sk_ad_network_attribution_credit != null && m.hasOwnProperty("sk_ad_network_attribution_credit")) {
                                    d.sk_ad_network_attribution_credit = o.enums === String ? $root.google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit[m.sk_ad_network_attribution_credit] : m.sk_ad_network_attribution_credit;
                                }
                                if (m.auction_insight_domain != null && m.hasOwnProperty("auction_insight_domain")) {
                                    d.auction_insight_domain = m.auction_insight_domain;
                                    if (o.oneofs)
                                        d._auction_insight_domain = "auction_insight_domain";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Segments to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.Segments
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Segments.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Segments;
                        })();
    
                        common.Keyword = (function() {
    
                            /**
                             * Properties of a Keyword.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IKeyword
                             * @property {string|null} [ad_group_criterion] Keyword ad_group_criterion
                             * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [info] Keyword info
                             */
    
                            /**
                             * Constructs a new Keyword.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a Keyword.
                             * @implements IKeyword
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IKeyword=} [p] Properties to set
                             */
                            function Keyword(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Keyword ad_group_criterion.
                             * @member {string|null|undefined} ad_group_criterion
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @instance
                             */
                            Keyword.prototype.ad_group_criterion = null;
    
                            /**
                             * Keyword info.
                             * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} info
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @instance
                             */
                            Keyword.prototype.info = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Keyword _ad_group_criterion.
                             * @member {"ad_group_criterion"|undefined} _ad_group_criterion
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @instance
                             */
                            Object.defineProperty(Keyword.prototype, "_ad_group_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Keyword instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeyword=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.Keyword} Keyword instance
                             */
                            Keyword.create = function create(properties) {
                                return new Keyword(properties);
                            };
    
                            /**
                             * Encodes the specified Keyword message. Does not implicitly {@link google.ads.googleads.v11.common.Keyword.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeyword} m Keyword message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Keyword.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.info != null && Object.hasOwnProperty.call(m, "info"))
                                    $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.info, w.uint32(18).fork()).ldelim();
                                if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                    w.uint32(26).string(m.ad_group_criterion);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Keyword message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Keyword.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {google.ads.googleads.v11.common.IKeyword} message Keyword message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Keyword.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Keyword message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.Keyword} Keyword
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Keyword.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Keyword();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.ad_group_criterion = r.string();
                                        break;
                                    case 2:
                                        m.info = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Keyword message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.Keyword} Keyword
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Keyword.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Keyword message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Keyword.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    p._ad_group_criterion = 1;
                                    if (!$util.isString(m.ad_group_criterion))
                                        return "ad_group_criterion: string expected";
                                }
                                if (m.info != null && m.hasOwnProperty("info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.info);
                                        if (e)
                                            return "info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Keyword message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.Keyword} Keyword
                             */
                            Keyword.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.Keyword)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.Keyword();
                                if (d.ad_group_criterion != null) {
                                    m.ad_group_criterion = String(d.ad_group_criterion);
                                }
                                if (d.info != null) {
                                    if (typeof d.info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.Keyword.info: object expected");
                                    m.info = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Keyword message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @static
                             * @param {google.ads.googleads.v11.common.Keyword} m Keyword
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Keyword.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.info = null;
                                }
                                if (m.info != null && m.hasOwnProperty("info")) {
                                    d.info = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.info, o);
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    d.ad_group_criterion = m.ad_group_criterion;
                                    if (o.oneofs)
                                        d._ad_group_criterion = "ad_group_criterion";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Keyword to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.Keyword
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Keyword.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Keyword;
                        })();
    
                        common.BudgetCampaignAssociationStatus = (function() {
    
                            /**
                             * Properties of a BudgetCampaignAssociationStatus.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBudgetCampaignAssociationStatus
                             * @property {string|null} [campaign] BudgetCampaignAssociationStatus campaign
                             * @property {google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus|null} [status] BudgetCampaignAssociationStatus status
                             */
    
                            /**
                             * Constructs a new BudgetCampaignAssociationStatus.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BudgetCampaignAssociationStatus.
                             * @implements IBudgetCampaignAssociationStatus
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBudgetCampaignAssociationStatus=} [p] Properties to set
                             */
                            function BudgetCampaignAssociationStatus(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BudgetCampaignAssociationStatus campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @instance
                             */
                            BudgetCampaignAssociationStatus.prototype.campaign = null;
    
                            /**
                             * BudgetCampaignAssociationStatus status.
                             * @member {google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus} status
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @instance
                             */
                            BudgetCampaignAssociationStatus.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BudgetCampaignAssociationStatus _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @instance
                             */
                            Object.defineProperty(BudgetCampaignAssociationStatus.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BudgetCampaignAssociationStatus instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetCampaignAssociationStatus=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BudgetCampaignAssociationStatus} BudgetCampaignAssociationStatus instance
                             */
                            BudgetCampaignAssociationStatus.create = function create(properties) {
                                return new BudgetCampaignAssociationStatus(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetCampaignAssociationStatus message. Does not implicitly {@link google.ads.googleads.v11.common.BudgetCampaignAssociationStatus.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetCampaignAssociationStatus} m BudgetCampaignAssociationStatus message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetCampaignAssociationStatus.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(10).string(m.campaign);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(16).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetCampaignAssociationStatus message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BudgetCampaignAssociationStatus.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetCampaignAssociationStatus} message BudgetCampaignAssociationStatus message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetCampaignAssociationStatus.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetCampaignAssociationStatus message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BudgetCampaignAssociationStatus} BudgetCampaignAssociationStatus
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetCampaignAssociationStatus.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.campaign = r.string();
                                        break;
                                    case 2:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetCampaignAssociationStatus message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BudgetCampaignAssociationStatus} BudgetCampaignAssociationStatus
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetCampaignAssociationStatus.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetCampaignAssociationStatus message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetCampaignAssociationStatus.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BudgetCampaignAssociationStatus message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BudgetCampaignAssociationStatus} BudgetCampaignAssociationStatus
                             */
                            BudgetCampaignAssociationStatus.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.BudgetCampaignAssociationStatus();
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BudgetCampaignAssociationStatus message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @static
                             * @param {google.ads.googleads.v11.common.BudgetCampaignAssociationStatus} m BudgetCampaignAssociationStatus
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetCampaignAssociationStatus.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BudgetCampaignAssociationStatus to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BudgetCampaignAssociationStatus
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetCampaignAssociationStatus.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BudgetCampaignAssociationStatus;
                        })();
    
                        common.AssetInteractionTarget = (function() {
    
                            /**
                             * Properties of an AssetInteractionTarget.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IAssetInteractionTarget
                             * @property {string|null} [asset] AssetInteractionTarget asset
                             * @property {boolean|null} [interaction_on_this_asset] AssetInteractionTarget interaction_on_this_asset
                             */
    
                            /**
                             * Constructs a new AssetInteractionTarget.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an AssetInteractionTarget.
                             * @implements IAssetInteractionTarget
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IAssetInteractionTarget=} [p] Properties to set
                             */
                            function AssetInteractionTarget(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetInteractionTarget asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @instance
                             */
                            AssetInteractionTarget.prototype.asset = "";
    
                            /**
                             * AssetInteractionTarget interaction_on_this_asset.
                             * @member {boolean} interaction_on_this_asset
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @instance
                             */
                            AssetInteractionTarget.prototype.interaction_on_this_asset = false;
    
                            /**
                             * Creates a new AssetInteractionTarget instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {google.ads.googleads.v11.common.IAssetInteractionTarget=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.AssetInteractionTarget} AssetInteractionTarget instance
                             */
                            AssetInteractionTarget.create = function create(properties) {
                                return new AssetInteractionTarget(properties);
                            };
    
                            /**
                             * Encodes the specified AssetInteractionTarget message. Does not implicitly {@link google.ads.googleads.v11.common.AssetInteractionTarget.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {google.ads.googleads.v11.common.IAssetInteractionTarget} m AssetInteractionTarget message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetInteractionTarget.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(10).string(m.asset);
                                if (m.interaction_on_this_asset != null && Object.hasOwnProperty.call(m, "interaction_on_this_asset"))
                                    w.uint32(16).bool(m.interaction_on_this_asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetInteractionTarget message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.AssetInteractionTarget.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {google.ads.googleads.v11.common.IAssetInteractionTarget} message AssetInteractionTarget message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetInteractionTarget.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetInteractionTarget message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.AssetInteractionTarget} AssetInteractionTarget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetInteractionTarget.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.AssetInteractionTarget();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.asset = r.string();
                                        break;
                                    case 2:
                                        m.interaction_on_this_asset = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetInteractionTarget message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.AssetInteractionTarget} AssetInteractionTarget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetInteractionTarget.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetInteractionTarget message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetInteractionTarget.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.interaction_on_this_asset != null && m.hasOwnProperty("interaction_on_this_asset")) {
                                    if (typeof m.interaction_on_this_asset !== "boolean")
                                        return "interaction_on_this_asset: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetInteractionTarget message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.AssetInteractionTarget} AssetInteractionTarget
                             */
                            AssetInteractionTarget.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.AssetInteractionTarget)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.AssetInteractionTarget();
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                if (d.interaction_on_this_asset != null) {
                                    m.interaction_on_this_asset = Boolean(d.interaction_on_this_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetInteractionTarget message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @static
                             * @param {google.ads.googleads.v11.common.AssetInteractionTarget} m AssetInteractionTarget
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetInteractionTarget.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.asset = "";
                                    d.interaction_on_this_asset = false;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.interaction_on_this_asset != null && m.hasOwnProperty("interaction_on_this_asset")) {
                                    d.interaction_on_this_asset = m.interaction_on_this_asset;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetInteractionTarget to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.AssetInteractionTarget
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetInteractionTarget.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetInteractionTarget;
                        })();
    
                        common.SkAdNetworkSourceApp = (function() {
    
                            /**
                             * Properties of a SkAdNetworkSourceApp.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ISkAdNetworkSourceApp
                             * @property {string|null} [sk_ad_network_source_app_id] SkAdNetworkSourceApp sk_ad_network_source_app_id
                             */
    
                            /**
                             * Constructs a new SkAdNetworkSourceApp.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a SkAdNetworkSourceApp.
                             * @implements ISkAdNetworkSourceApp
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ISkAdNetworkSourceApp=} [p] Properties to set
                             */
                            function SkAdNetworkSourceApp(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SkAdNetworkSourceApp sk_ad_network_source_app_id.
                             * @member {string|null|undefined} sk_ad_network_source_app_id
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @instance
                             */
                            SkAdNetworkSourceApp.prototype.sk_ad_network_source_app_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SkAdNetworkSourceApp _sk_ad_network_source_app_id.
                             * @member {"sk_ad_network_source_app_id"|undefined} _sk_ad_network_source_app_id
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @instance
                             */
                            Object.defineProperty(SkAdNetworkSourceApp.prototype, "_sk_ad_network_source_app_id", {
                                get: $util.oneOfGetter($oneOfFields = ["sk_ad_network_source_app_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SkAdNetworkSourceApp instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {google.ads.googleads.v11.common.ISkAdNetworkSourceApp=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.SkAdNetworkSourceApp} SkAdNetworkSourceApp instance
                             */
                            SkAdNetworkSourceApp.create = function create(properties) {
                                return new SkAdNetworkSourceApp(properties);
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkSourceApp message. Does not implicitly {@link google.ads.googleads.v11.common.SkAdNetworkSourceApp.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {google.ads.googleads.v11.common.ISkAdNetworkSourceApp} m SkAdNetworkSourceApp message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkSourceApp.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.sk_ad_network_source_app_id != null && Object.hasOwnProperty.call(m, "sk_ad_network_source_app_id"))
                                    w.uint32(10).string(m.sk_ad_network_source_app_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkSourceApp message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.SkAdNetworkSourceApp.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {google.ads.googleads.v11.common.ISkAdNetworkSourceApp} message SkAdNetworkSourceApp message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkSourceApp.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SkAdNetworkSourceApp message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.SkAdNetworkSourceApp} SkAdNetworkSourceApp
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkSourceApp.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.sk_ad_network_source_app_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SkAdNetworkSourceApp message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.SkAdNetworkSourceApp} SkAdNetworkSourceApp
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkSourceApp.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SkAdNetworkSourceApp message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SkAdNetworkSourceApp.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.sk_ad_network_source_app_id != null && m.hasOwnProperty("sk_ad_network_source_app_id")) {
                                    p._sk_ad_network_source_app_id = 1;
                                    if (!$util.isString(m.sk_ad_network_source_app_id))
                                        return "sk_ad_network_source_app_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SkAdNetworkSourceApp message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.SkAdNetworkSourceApp} SkAdNetworkSourceApp
                             */
                            SkAdNetworkSourceApp.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.SkAdNetworkSourceApp();
                                if (d.sk_ad_network_source_app_id != null) {
                                    m.sk_ad_network_source_app_id = String(d.sk_ad_network_source_app_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SkAdNetworkSourceApp message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @static
                             * @param {google.ads.googleads.v11.common.SkAdNetworkSourceApp} m SkAdNetworkSourceApp
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SkAdNetworkSourceApp.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.sk_ad_network_source_app_id != null && m.hasOwnProperty("sk_ad_network_source_app_id")) {
                                    d.sk_ad_network_source_app_id = m.sk_ad_network_source_app_id;
                                    if (o.oneofs)
                                        d._sk_ad_network_source_app_id = "sk_ad_network_source_app_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SkAdNetworkSourceApp to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.SkAdNetworkSourceApp
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SkAdNetworkSourceApp.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SkAdNetworkSourceApp;
                        })();
    
                        common.BidModifierSimulationPointList = (function() {
    
                            /**
                             * Properties of a BidModifierSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBidModifierSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.IBidModifierSimulationPoint>|null} [points] BidModifierSimulationPointList points
                             */
    
                            /**
                             * Constructs a new BidModifierSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BidModifierSimulationPointList.
                             * @implements IBidModifierSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPointList=} [p] Properties to set
                             */
                            function BidModifierSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BidModifierSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.IBidModifierSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @instance
                             */
                            BidModifierSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new BidModifierSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPointList} BidModifierSimulationPointList instance
                             */
                            BidModifierSimulationPointList.create = function create(properties) {
                                return new BidModifierSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified BidModifierSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.BidModifierSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPointList} m BidModifierSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BidModifierSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.BidModifierSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified BidModifierSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BidModifierSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPointList} message BidModifierSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BidModifierSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BidModifierSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPointList} BidModifierSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BidModifierSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BidModifierSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.BidModifierSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BidModifierSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPointList} BidModifierSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BidModifierSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BidModifierSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BidModifierSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.BidModifierSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BidModifierSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPointList} BidModifierSimulationPointList
                             */
                            BidModifierSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BidModifierSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.BidModifierSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.BidModifierSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.BidModifierSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.BidModifierSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BidModifierSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.BidModifierSimulationPointList} m BidModifierSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BidModifierSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.BidModifierSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BidModifierSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BidModifierSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BidModifierSimulationPointList;
                        })();
    
                        common.CpcBidSimulationPointList = (function() {
    
                            /**
                             * Properties of a CpcBidSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICpcBidSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.ICpcBidSimulationPoint>|null} [points] CpcBidSimulationPointList points
                             */
    
                            /**
                             * Constructs a new CpcBidSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CpcBidSimulationPointList.
                             * @implements ICpcBidSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPointList=} [p] Properties to set
                             */
                            function CpcBidSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CpcBidSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.ICpcBidSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @instance
                             */
                            CpcBidSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new CpcBidSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPointList} CpcBidSimulationPointList instance
                             */
                            CpcBidSimulationPointList.create = function create(properties) {
                                return new CpcBidSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified CpcBidSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.CpcBidSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPointList} m CpcBidSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpcBidSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.CpcBidSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CpcBidSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CpcBidSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPointList} message CpcBidSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpcBidSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CpcBidSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPointList} CpcBidSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpcBidSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CpcBidSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.CpcBidSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CpcBidSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPointList} CpcBidSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpcBidSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CpcBidSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CpcBidSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CpcBidSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CpcBidSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPointList} CpcBidSimulationPointList
                             */
                            CpcBidSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CpcBidSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CpcBidSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.CpcBidSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.CpcBidSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.CpcBidSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CpcBidSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.CpcBidSimulationPointList} m CpcBidSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CpcBidSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.CpcBidSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CpcBidSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CpcBidSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CpcBidSimulationPointList;
                        })();
    
                        common.CpvBidSimulationPointList = (function() {
    
                            /**
                             * Properties of a CpvBidSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICpvBidSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.ICpvBidSimulationPoint>|null} [points] CpvBidSimulationPointList points
                             */
    
                            /**
                             * Constructs a new CpvBidSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CpvBidSimulationPointList.
                             * @implements ICpvBidSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPointList=} [p] Properties to set
                             */
                            function CpvBidSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CpvBidSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.ICpvBidSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @instance
                             */
                            CpvBidSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new CpvBidSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPointList} CpvBidSimulationPointList instance
                             */
                            CpvBidSimulationPointList.create = function create(properties) {
                                return new CpvBidSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified CpvBidSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.CpvBidSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPointList} m CpvBidSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpvBidSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.CpvBidSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CpvBidSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CpvBidSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPointList} message CpvBidSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpvBidSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CpvBidSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPointList} CpvBidSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpvBidSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CpvBidSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.CpvBidSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CpvBidSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPointList} CpvBidSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpvBidSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CpvBidSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CpvBidSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CpvBidSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CpvBidSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPointList} CpvBidSimulationPointList
                             */
                            CpvBidSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CpvBidSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CpvBidSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.CpvBidSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.CpvBidSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.CpvBidSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CpvBidSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.CpvBidSimulationPointList} m CpvBidSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CpvBidSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.CpvBidSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CpvBidSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CpvBidSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CpvBidSimulationPointList;
                        })();
    
                        common.TargetCpaSimulationPointList = (function() {
    
                            /**
                             * Properties of a TargetCpaSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetCpaSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.ITargetCpaSimulationPoint>|null} [points] TargetCpaSimulationPointList points
                             */
    
                            /**
                             * Constructs a new TargetCpaSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetCpaSimulationPointList.
                             * @implements ITargetCpaSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPointList=} [p] Properties to set
                             */
                            function TargetCpaSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetCpaSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.ITargetCpaSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @instance
                             */
                            TargetCpaSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new TargetCpaSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPointList} TargetCpaSimulationPointList instance
                             */
                            TargetCpaSimulationPointList.create = function create(properties) {
                                return new TargetCpaSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified TargetCpaSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpaSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPointList} m TargetCpaSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpaSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.TargetCpaSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetCpaSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpaSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPointList} message TargetCpaSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpaSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetCpaSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPointList} TargetCpaSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpaSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.TargetCpaSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetCpaSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPointList} TargetCpaSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpaSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetCpaSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetCpaSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.TargetCpaSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetCpaSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPointList} TargetCpaSimulationPointList
                             */
                            TargetCpaSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.TargetCpaSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.TargetCpaSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.TargetCpaSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetCpaSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetCpaSimulationPointList} m TargetCpaSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetCpaSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.TargetCpaSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetCpaSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetCpaSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetCpaSimulationPointList;
                        })();
    
                        common.TargetRoasSimulationPointList = (function() {
    
                            /**
                             * Properties of a TargetRoasSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetRoasSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.ITargetRoasSimulationPoint>|null} [points] TargetRoasSimulationPointList points
                             */
    
                            /**
                             * Constructs a new TargetRoasSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetRoasSimulationPointList.
                             * @implements ITargetRoasSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPointList=} [p] Properties to set
                             */
                            function TargetRoasSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetRoasSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.ITargetRoasSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @instance
                             */
                            TargetRoasSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new TargetRoasSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPointList} TargetRoasSimulationPointList instance
                             */
                            TargetRoasSimulationPointList.create = function create(properties) {
                                return new TargetRoasSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified TargetRoasSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.TargetRoasSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPointList} m TargetRoasSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRoasSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.TargetRoasSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetRoasSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetRoasSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPointList} message TargetRoasSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRoasSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetRoasSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPointList} TargetRoasSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRoasSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.TargetRoasSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetRoasSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPointList} TargetRoasSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRoasSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetRoasSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetRoasSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.TargetRoasSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetRoasSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPointList} TargetRoasSimulationPointList
                             */
                            TargetRoasSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.TargetRoasSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.TargetRoasSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.TargetRoasSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetRoasSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetRoasSimulationPointList} m TargetRoasSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetRoasSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.TargetRoasSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetRoasSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetRoasSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetRoasSimulationPointList;
                        })();
    
                        common.PercentCpcBidSimulationPointList = (function() {
    
                            /**
                             * Properties of a PercentCpcBidSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPercentCpcBidSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.IPercentCpcBidSimulationPoint>|null} [points] PercentCpcBidSimulationPointList points
                             */
    
                            /**
                             * Constructs a new PercentCpcBidSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PercentCpcBidSimulationPointList.
                             * @implements IPercentCpcBidSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPointList=} [p] Properties to set
                             */
                            function PercentCpcBidSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PercentCpcBidSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.IPercentCpcBidSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @instance
                             */
                            PercentCpcBidSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new PercentCpcBidSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPointList} PercentCpcBidSimulationPointList instance
                             */
                            PercentCpcBidSimulationPointList.create = function create(properties) {
                                return new PercentCpcBidSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified PercentCpcBidSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPointList} m PercentCpcBidSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PercentCpcBidSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PercentCpcBidSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPointList} message PercentCpcBidSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PercentCpcBidSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PercentCpcBidSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPointList} PercentCpcBidSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PercentCpcBidSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PercentCpcBidSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPointList} PercentCpcBidSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PercentCpcBidSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PercentCpcBidSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PercentCpcBidSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PercentCpcBidSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPointList} PercentCpcBidSimulationPointList
                             */
                            PercentCpcBidSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PercentCpcBidSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.PercentCpcBidSimulationPointList} m PercentCpcBidSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PercentCpcBidSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PercentCpcBidSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PercentCpcBidSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PercentCpcBidSimulationPointList;
                        })();
    
                        common.BudgetSimulationPointList = (function() {
    
                            /**
                             * Properties of a BudgetSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBudgetSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.IBudgetSimulationPoint>|null} [points] BudgetSimulationPointList points
                             */
    
                            /**
                             * Constructs a new BudgetSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BudgetSimulationPointList.
                             * @implements IBudgetSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPointList=} [p] Properties to set
                             */
                            function BudgetSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BudgetSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.IBudgetSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @instance
                             */
                            BudgetSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new BudgetSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPointList} BudgetSimulationPointList instance
                             */
                            BudgetSimulationPointList.create = function create(properties) {
                                return new BudgetSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.BudgetSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPointList} m BudgetSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.BudgetSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BudgetSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPointList} message BudgetSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPointList} BudgetSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BudgetSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.BudgetSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPointList} BudgetSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.BudgetSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BudgetSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPointList} BudgetSimulationPointList
                             */
                            BudgetSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BudgetSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.BudgetSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.BudgetSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.BudgetSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.BudgetSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BudgetSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.BudgetSimulationPointList} m BudgetSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.BudgetSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BudgetSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BudgetSimulationPointList;
                        })();
    
                        common.TargetImpressionShareSimulationPointList = (function() {
    
                            /**
                             * Properties of a TargetImpressionShareSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetImpressionShareSimulationPointList
                             * @property {Array.<google.ads.googleads.v11.common.ITargetImpressionShareSimulationPoint>|null} [points] TargetImpressionShareSimulationPointList points
                             */
    
                            /**
                             * Constructs a new TargetImpressionShareSimulationPointList.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetImpressionShareSimulationPointList.
                             * @implements ITargetImpressionShareSimulationPointList
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPointList=} [p] Properties to set
                             */
                            function TargetImpressionShareSimulationPointList(p) {
                                this.points = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetImpressionShareSimulationPointList points.
                             * @member {Array.<google.ads.googleads.v11.common.ITargetImpressionShareSimulationPoint>} points
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @instance
                             */
                            TargetImpressionShareSimulationPointList.prototype.points = $util.emptyArray;
    
                            /**
                             * Creates a new TargetImpressionShareSimulationPointList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPointList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList} TargetImpressionShareSimulationPointList instance
                             */
                            TargetImpressionShareSimulationPointList.create = function create(properties) {
                                return new TargetImpressionShareSimulationPointList(properties);
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShareSimulationPointList message. Does not implicitly {@link google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPointList} m TargetImpressionShareSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShareSimulationPointList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.points != null && m.points.length) {
                                    for (var i = 0; i < m.points.length; ++i)
                                        $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint.encode(m.points[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShareSimulationPointList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPointList} message TargetImpressionShareSimulationPointList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShareSimulationPointList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetImpressionShareSimulationPointList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList} TargetImpressionShareSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShareSimulationPointList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.points && m.points.length))
                                            m.points = [];
                                        m.points.push($root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetImpressionShareSimulationPointList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList} TargetImpressionShareSimulationPointList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShareSimulationPointList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetImpressionShareSimulationPointList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetImpressionShareSimulationPointList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.points != null && m.hasOwnProperty("points")) {
                                    if (!Array.isArray(m.points))
                                        return "points: array expected";
                                    for (var i = 0; i < m.points.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint.verify(m.points[i]);
                                            if (e)
                                                return "points." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetImpressionShareSimulationPointList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList} TargetImpressionShareSimulationPointList
                             */
                            TargetImpressionShareSimulationPointList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList();
                                if (d.points) {
                                    if (!Array.isArray(d.points))
                                        throw TypeError(".google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.points: array expected");
                                    m.points = [];
                                    for (var i = 0; i < d.points.length; ++i) {
                                        if (typeof d.points[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.points: object expected");
                                        m.points[i] = $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint.fromObject(d.points[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetImpressionShareSimulationPointList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList} m TargetImpressionShareSimulationPointList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetImpressionShareSimulationPointList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.points = [];
                                }
                                if (m.points && m.points.length) {
                                    d.points = [];
                                    for (var j = 0; j < m.points.length; ++j) {
                                        d.points[j] = $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint.toObject(m.points[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetImpressionShareSimulationPointList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetImpressionShareSimulationPointList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetImpressionShareSimulationPointList;
                        })();
    
                        common.BidModifierSimulationPoint = (function() {
    
                            /**
                             * Properties of a BidModifierSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBidModifierSimulationPoint
                             * @property {number|null} [bid_modifier] BidModifierSimulationPoint bid_modifier
                             * @property {number|null} [biddable_conversions] BidModifierSimulationPoint biddable_conversions
                             * @property {number|null} [biddable_conversions_value] BidModifierSimulationPoint biddable_conversions_value
                             * @property {number|null} [clicks] BidModifierSimulationPoint clicks
                             * @property {number|null} [cost_micros] BidModifierSimulationPoint cost_micros
                             * @property {number|null} [impressions] BidModifierSimulationPoint impressions
                             * @property {number|null} [top_slot_impressions] BidModifierSimulationPoint top_slot_impressions
                             * @property {number|null} [parent_biddable_conversions] BidModifierSimulationPoint parent_biddable_conversions
                             * @property {number|null} [parent_biddable_conversions_value] BidModifierSimulationPoint parent_biddable_conversions_value
                             * @property {number|null} [parent_clicks] BidModifierSimulationPoint parent_clicks
                             * @property {number|null} [parent_cost_micros] BidModifierSimulationPoint parent_cost_micros
                             * @property {number|null} [parent_impressions] BidModifierSimulationPoint parent_impressions
                             * @property {number|null} [parent_top_slot_impressions] BidModifierSimulationPoint parent_top_slot_impressions
                             * @property {number|null} [parent_required_budget_micros] BidModifierSimulationPoint parent_required_budget_micros
                             */
    
                            /**
                             * Constructs a new BidModifierSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BidModifierSimulationPoint.
                             * @implements IBidModifierSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPoint=} [p] Properties to set
                             */
                            function BidModifierSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BidModifierSimulationPoint bid_modifier.
                             * @member {number|null|undefined} bid_modifier
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.bid_modifier = null;
    
                            /**
                             * BidModifierSimulationPoint biddable_conversions.
                             * @member {number|null|undefined} biddable_conversions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.biddable_conversions = null;
    
                            /**
                             * BidModifierSimulationPoint biddable_conversions_value.
                             * @member {number|null|undefined} biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.biddable_conversions_value = null;
    
                            /**
                             * BidModifierSimulationPoint clicks.
                             * @member {number|null|undefined} clicks
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.clicks = null;
    
                            /**
                             * BidModifierSimulationPoint cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.cost_micros = null;
    
                            /**
                             * BidModifierSimulationPoint impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.impressions = null;
    
                            /**
                             * BidModifierSimulationPoint top_slot_impressions.
                             * @member {number|null|undefined} top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.top_slot_impressions = null;
    
                            /**
                             * BidModifierSimulationPoint parent_biddable_conversions.
                             * @member {number|null|undefined} parent_biddable_conversions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.parent_biddable_conversions = null;
    
                            /**
                             * BidModifierSimulationPoint parent_biddable_conversions_value.
                             * @member {number|null|undefined} parent_biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.parent_biddable_conversions_value = null;
    
                            /**
                             * BidModifierSimulationPoint parent_clicks.
                             * @member {number|null|undefined} parent_clicks
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.parent_clicks = null;
    
                            /**
                             * BidModifierSimulationPoint parent_cost_micros.
                             * @member {number|null|undefined} parent_cost_micros
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.parent_cost_micros = null;
    
                            /**
                             * BidModifierSimulationPoint parent_impressions.
                             * @member {number|null|undefined} parent_impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.parent_impressions = null;
    
                            /**
                             * BidModifierSimulationPoint parent_top_slot_impressions.
                             * @member {number|null|undefined} parent_top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.parent_top_slot_impressions = null;
    
                            /**
                             * BidModifierSimulationPoint parent_required_budget_micros.
                             * @member {number|null|undefined} parent_required_budget_micros
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            BidModifierSimulationPoint.prototype.parent_required_budget_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BidModifierSimulationPoint _bid_modifier.
                             * @member {"bid_modifier"|undefined} _bid_modifier
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_bid_modifier", {
                                get: $util.oneOfGetter($oneOfFields = ["bid_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _biddable_conversions.
                             * @member {"biddable_conversions"|undefined} _biddable_conversions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_biddable_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _biddable_conversions_value.
                             * @member {"biddable_conversions_value"|undefined} _biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_biddable_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _clicks.
                             * @member {"clicks"|undefined} _clicks
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _top_slot_impressions.
                             * @member {"top_slot_impressions"|undefined} _top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_top_slot_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["top_slot_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _parent_biddable_conversions.
                             * @member {"parent_biddable_conversions"|undefined} _parent_biddable_conversions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_parent_biddable_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_biddable_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _parent_biddable_conversions_value.
                             * @member {"parent_biddable_conversions_value"|undefined} _parent_biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_parent_biddable_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_biddable_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _parent_clicks.
                             * @member {"parent_clicks"|undefined} _parent_clicks
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_parent_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _parent_cost_micros.
                             * @member {"parent_cost_micros"|undefined} _parent_cost_micros
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_parent_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _parent_impressions.
                             * @member {"parent_impressions"|undefined} _parent_impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_parent_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _parent_top_slot_impressions.
                             * @member {"parent_top_slot_impressions"|undefined} _parent_top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_parent_top_slot_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_top_slot_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BidModifierSimulationPoint _parent_required_budget_micros.
                             * @member {"parent_required_budget_micros"|undefined} _parent_required_budget_micros
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(BidModifierSimulationPoint.prototype, "_parent_required_budget_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_required_budget_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BidModifierSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPoint} BidModifierSimulationPoint instance
                             */
                            BidModifierSimulationPoint.create = function create(properties) {
                                return new BidModifierSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified BidModifierSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.BidModifierSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPoint} m BidModifierSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BidModifierSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.bid_modifier != null && Object.hasOwnProperty.call(m, "bid_modifier"))
                                    w.uint32(121).double(m.bid_modifier);
                                if (m.biddable_conversions != null && Object.hasOwnProperty.call(m, "biddable_conversions"))
                                    w.uint32(129).double(m.biddable_conversions);
                                if (m.biddable_conversions_value != null && Object.hasOwnProperty.call(m, "biddable_conversions_value"))
                                    w.uint32(137).double(m.biddable_conversions_value);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(144).int64(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(152).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(160).int64(m.impressions);
                                if (m.top_slot_impressions != null && Object.hasOwnProperty.call(m, "top_slot_impressions"))
                                    w.uint32(168).int64(m.top_slot_impressions);
                                if (m.parent_biddable_conversions != null && Object.hasOwnProperty.call(m, "parent_biddable_conversions"))
                                    w.uint32(177).double(m.parent_biddable_conversions);
                                if (m.parent_biddable_conversions_value != null && Object.hasOwnProperty.call(m, "parent_biddable_conversions_value"))
                                    w.uint32(185).double(m.parent_biddable_conversions_value);
                                if (m.parent_clicks != null && Object.hasOwnProperty.call(m, "parent_clicks"))
                                    w.uint32(192).int64(m.parent_clicks);
                                if (m.parent_cost_micros != null && Object.hasOwnProperty.call(m, "parent_cost_micros"))
                                    w.uint32(200).int64(m.parent_cost_micros);
                                if (m.parent_impressions != null && Object.hasOwnProperty.call(m, "parent_impressions"))
                                    w.uint32(208).int64(m.parent_impressions);
                                if (m.parent_top_slot_impressions != null && Object.hasOwnProperty.call(m, "parent_top_slot_impressions"))
                                    w.uint32(216).int64(m.parent_top_slot_impressions);
                                if (m.parent_required_budget_micros != null && Object.hasOwnProperty.call(m, "parent_required_budget_micros"))
                                    w.uint32(224).int64(m.parent_required_budget_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified BidModifierSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BidModifierSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IBidModifierSimulationPoint} message BidModifierSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BidModifierSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BidModifierSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPoint} BidModifierSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BidModifierSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BidModifierSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 15:
                                        m.bid_modifier = r.double();
                                        break;
                                    case 16:
                                        m.biddable_conversions = r.double();
                                        break;
                                    case 17:
                                        m.biddable_conversions_value = r.double();
                                        break;
                                    case 18:
                                        m.clicks = r.int64();
                                        break;
                                    case 19:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 20:
                                        m.impressions = r.int64();
                                        break;
                                    case 21:
                                        m.top_slot_impressions = r.int64();
                                        break;
                                    case 22:
                                        m.parent_biddable_conversions = r.double();
                                        break;
                                    case 23:
                                        m.parent_biddable_conversions_value = r.double();
                                        break;
                                    case 24:
                                        m.parent_clicks = r.int64();
                                        break;
                                    case 25:
                                        m.parent_cost_micros = r.int64();
                                        break;
                                    case 26:
                                        m.parent_impressions = r.int64();
                                        break;
                                    case 27:
                                        m.parent_top_slot_impressions = r.int64();
                                        break;
                                    case 28:
                                        m.parent_required_budget_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BidModifierSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPoint} BidModifierSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BidModifierSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BidModifierSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BidModifierSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    p._bid_modifier = 1;
                                    if (typeof m.bid_modifier !== "number")
                                        return "bid_modifier: number expected";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    p._biddable_conversions = 1;
                                    if (typeof m.biddable_conversions !== "number")
                                        return "biddable_conversions: number expected";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    p._biddable_conversions_value = 1;
                                    if (typeof m.biddable_conversions_value !== "number")
                                        return "biddable_conversions_value: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    p._clicks = 1;
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    p._top_slot_impressions = 1;
                                    if (!$util.isInteger(m.top_slot_impressions) && !(m.top_slot_impressions && $util.isInteger(m.top_slot_impressions.low) && $util.isInteger(m.top_slot_impressions.high)))
                                        return "top_slot_impressions: integer|Long expected";
                                }
                                if (m.parent_biddable_conversions != null && m.hasOwnProperty("parent_biddable_conversions")) {
                                    p._parent_biddable_conversions = 1;
                                    if (typeof m.parent_biddable_conversions !== "number")
                                        return "parent_biddable_conversions: number expected";
                                }
                                if (m.parent_biddable_conversions_value != null && m.hasOwnProperty("parent_biddable_conversions_value")) {
                                    p._parent_biddable_conversions_value = 1;
                                    if (typeof m.parent_biddable_conversions_value !== "number")
                                        return "parent_biddable_conversions_value: number expected";
                                }
                                if (m.parent_clicks != null && m.hasOwnProperty("parent_clicks")) {
                                    p._parent_clicks = 1;
                                    if (!$util.isInteger(m.parent_clicks) && !(m.parent_clicks && $util.isInteger(m.parent_clicks.low) && $util.isInteger(m.parent_clicks.high)))
                                        return "parent_clicks: integer|Long expected";
                                }
                                if (m.parent_cost_micros != null && m.hasOwnProperty("parent_cost_micros")) {
                                    p._parent_cost_micros = 1;
                                    if (!$util.isInteger(m.parent_cost_micros) && !(m.parent_cost_micros && $util.isInteger(m.parent_cost_micros.low) && $util.isInteger(m.parent_cost_micros.high)))
                                        return "parent_cost_micros: integer|Long expected";
                                }
                                if (m.parent_impressions != null && m.hasOwnProperty("parent_impressions")) {
                                    p._parent_impressions = 1;
                                    if (!$util.isInteger(m.parent_impressions) && !(m.parent_impressions && $util.isInteger(m.parent_impressions.low) && $util.isInteger(m.parent_impressions.high)))
                                        return "parent_impressions: integer|Long expected";
                                }
                                if (m.parent_top_slot_impressions != null && m.hasOwnProperty("parent_top_slot_impressions")) {
                                    p._parent_top_slot_impressions = 1;
                                    if (!$util.isInteger(m.parent_top_slot_impressions) && !(m.parent_top_slot_impressions && $util.isInteger(m.parent_top_slot_impressions.low) && $util.isInteger(m.parent_top_slot_impressions.high)))
                                        return "parent_top_slot_impressions: integer|Long expected";
                                }
                                if (m.parent_required_budget_micros != null && m.hasOwnProperty("parent_required_budget_micros")) {
                                    p._parent_required_budget_micros = 1;
                                    if (!$util.isInteger(m.parent_required_budget_micros) && !(m.parent_required_budget_micros && $util.isInteger(m.parent_required_budget_micros.low) && $util.isInteger(m.parent_required_budget_micros.high)))
                                        return "parent_required_budget_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BidModifierSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BidModifierSimulationPoint} BidModifierSimulationPoint
                             */
                            BidModifierSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BidModifierSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.BidModifierSimulationPoint();
                                if (d.bid_modifier != null) {
                                    m.bid_modifier = Number(d.bid_modifier);
                                }
                                if (d.biddable_conversions != null) {
                                    m.biddable_conversions = Number(d.biddable_conversions);
                                }
                                if (d.biddable_conversions_value != null) {
                                    m.biddable_conversions_value = Number(d.biddable_conversions_value);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.top_slot_impressions = $util.Long.fromValue(d.top_slot_impressions)).unsigned = false;
                                    else if (typeof d.top_slot_impressions === "string")
                                        m.top_slot_impressions = parseInt(d.top_slot_impressions, 10);
                                    else if (typeof d.top_slot_impressions === "number")
                                        m.top_slot_impressions = d.top_slot_impressions;
                                    else if (typeof d.top_slot_impressions === "object")
                                        m.top_slot_impressions = new $util.LongBits(d.top_slot_impressions.low >>> 0, d.top_slot_impressions.high >>> 0).toNumber();
                                }
                                if (d.parent_biddable_conversions != null) {
                                    m.parent_biddable_conversions = Number(d.parent_biddable_conversions);
                                }
                                if (d.parent_biddable_conversions_value != null) {
                                    m.parent_biddable_conversions_value = Number(d.parent_biddable_conversions_value);
                                }
                                if (d.parent_clicks != null) {
                                    if ($util.Long)
                                        (m.parent_clicks = $util.Long.fromValue(d.parent_clicks)).unsigned = false;
                                    else if (typeof d.parent_clicks === "string")
                                        m.parent_clicks = parseInt(d.parent_clicks, 10);
                                    else if (typeof d.parent_clicks === "number")
                                        m.parent_clicks = d.parent_clicks;
                                    else if (typeof d.parent_clicks === "object")
                                        m.parent_clicks = new $util.LongBits(d.parent_clicks.low >>> 0, d.parent_clicks.high >>> 0).toNumber();
                                }
                                if (d.parent_cost_micros != null) {
                                    if ($util.Long)
                                        (m.parent_cost_micros = $util.Long.fromValue(d.parent_cost_micros)).unsigned = false;
                                    else if (typeof d.parent_cost_micros === "string")
                                        m.parent_cost_micros = parseInt(d.parent_cost_micros, 10);
                                    else if (typeof d.parent_cost_micros === "number")
                                        m.parent_cost_micros = d.parent_cost_micros;
                                    else if (typeof d.parent_cost_micros === "object")
                                        m.parent_cost_micros = new $util.LongBits(d.parent_cost_micros.low >>> 0, d.parent_cost_micros.high >>> 0).toNumber();
                                }
                                if (d.parent_impressions != null) {
                                    if ($util.Long)
                                        (m.parent_impressions = $util.Long.fromValue(d.parent_impressions)).unsigned = false;
                                    else if (typeof d.parent_impressions === "string")
                                        m.parent_impressions = parseInt(d.parent_impressions, 10);
                                    else if (typeof d.parent_impressions === "number")
                                        m.parent_impressions = d.parent_impressions;
                                    else if (typeof d.parent_impressions === "object")
                                        m.parent_impressions = new $util.LongBits(d.parent_impressions.low >>> 0, d.parent_impressions.high >>> 0).toNumber();
                                }
                                if (d.parent_top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.parent_top_slot_impressions = $util.Long.fromValue(d.parent_top_slot_impressions)).unsigned = false;
                                    else if (typeof d.parent_top_slot_impressions === "string")
                                        m.parent_top_slot_impressions = parseInt(d.parent_top_slot_impressions, 10);
                                    else if (typeof d.parent_top_slot_impressions === "number")
                                        m.parent_top_slot_impressions = d.parent_top_slot_impressions;
                                    else if (typeof d.parent_top_slot_impressions === "object")
                                        m.parent_top_slot_impressions = new $util.LongBits(d.parent_top_slot_impressions.low >>> 0, d.parent_top_slot_impressions.high >>> 0).toNumber();
                                }
                                if (d.parent_required_budget_micros != null) {
                                    if ($util.Long)
                                        (m.parent_required_budget_micros = $util.Long.fromValue(d.parent_required_budget_micros)).unsigned = false;
                                    else if (typeof d.parent_required_budget_micros === "string")
                                        m.parent_required_budget_micros = parseInt(d.parent_required_budget_micros, 10);
                                    else if (typeof d.parent_required_budget_micros === "number")
                                        m.parent_required_budget_micros = d.parent_required_budget_micros;
                                    else if (typeof d.parent_required_budget_micros === "object")
                                        m.parent_required_budget_micros = new $util.LongBits(d.parent_required_budget_micros.low >>> 0, d.parent_required_budget_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BidModifierSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.BidModifierSimulationPoint} m BidModifierSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BidModifierSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    d.bid_modifier = o.json && !isFinite(m.bid_modifier) ? String(m.bid_modifier) : m.bid_modifier;
                                    if (o.oneofs)
                                        d._bid_modifier = "bid_modifier";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    d.biddable_conversions = o.json && !isFinite(m.biddable_conversions) ? String(m.biddable_conversions) : m.biddable_conversions;
                                    if (o.oneofs)
                                        d._biddable_conversions = "biddable_conversions";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    d.biddable_conversions_value = o.json && !isFinite(m.biddable_conversions_value) ? String(m.biddable_conversions_value) : m.biddable_conversions_value;
                                    if (o.oneofs)
                                        d._biddable_conversions_value = "biddable_conversions_value";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                    if (o.oneofs)
                                        d._clicks = "clicks";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (typeof m.top_slot_impressions === "number")
                                        d.top_slot_impressions = o.longs === String ? String(m.top_slot_impressions) : m.top_slot_impressions;
                                    else
                                        d.top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.top_slot_impressions.low >>> 0, m.top_slot_impressions.high >>> 0).toNumber() : m.top_slot_impressions;
                                    if (o.oneofs)
                                        d._top_slot_impressions = "top_slot_impressions";
                                }
                                if (m.parent_biddable_conversions != null && m.hasOwnProperty("parent_biddable_conversions")) {
                                    d.parent_biddable_conversions = o.json && !isFinite(m.parent_biddable_conversions) ? String(m.parent_biddable_conversions) : m.parent_biddable_conversions;
                                    if (o.oneofs)
                                        d._parent_biddable_conversions = "parent_biddable_conversions";
                                }
                                if (m.parent_biddable_conversions_value != null && m.hasOwnProperty("parent_biddable_conversions_value")) {
                                    d.parent_biddable_conversions_value = o.json && !isFinite(m.parent_biddable_conversions_value) ? String(m.parent_biddable_conversions_value) : m.parent_biddable_conversions_value;
                                    if (o.oneofs)
                                        d._parent_biddable_conversions_value = "parent_biddable_conversions_value";
                                }
                                if (m.parent_clicks != null && m.hasOwnProperty("parent_clicks")) {
                                    if (typeof m.parent_clicks === "number")
                                        d.parent_clicks = o.longs === String ? String(m.parent_clicks) : m.parent_clicks;
                                    else
                                        d.parent_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.parent_clicks) : o.longs === Number ? new $util.LongBits(m.parent_clicks.low >>> 0, m.parent_clicks.high >>> 0).toNumber() : m.parent_clicks;
                                    if (o.oneofs)
                                        d._parent_clicks = "parent_clicks";
                                }
                                if (m.parent_cost_micros != null && m.hasOwnProperty("parent_cost_micros")) {
                                    if (typeof m.parent_cost_micros === "number")
                                        d.parent_cost_micros = o.longs === String ? String(m.parent_cost_micros) : m.parent_cost_micros;
                                    else
                                        d.parent_cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.parent_cost_micros) : o.longs === Number ? new $util.LongBits(m.parent_cost_micros.low >>> 0, m.parent_cost_micros.high >>> 0).toNumber() : m.parent_cost_micros;
                                    if (o.oneofs)
                                        d._parent_cost_micros = "parent_cost_micros";
                                }
                                if (m.parent_impressions != null && m.hasOwnProperty("parent_impressions")) {
                                    if (typeof m.parent_impressions === "number")
                                        d.parent_impressions = o.longs === String ? String(m.parent_impressions) : m.parent_impressions;
                                    else
                                        d.parent_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.parent_impressions) : o.longs === Number ? new $util.LongBits(m.parent_impressions.low >>> 0, m.parent_impressions.high >>> 0).toNumber() : m.parent_impressions;
                                    if (o.oneofs)
                                        d._parent_impressions = "parent_impressions";
                                }
                                if (m.parent_top_slot_impressions != null && m.hasOwnProperty("parent_top_slot_impressions")) {
                                    if (typeof m.parent_top_slot_impressions === "number")
                                        d.parent_top_slot_impressions = o.longs === String ? String(m.parent_top_slot_impressions) : m.parent_top_slot_impressions;
                                    else
                                        d.parent_top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.parent_top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.parent_top_slot_impressions.low >>> 0, m.parent_top_slot_impressions.high >>> 0).toNumber() : m.parent_top_slot_impressions;
                                    if (o.oneofs)
                                        d._parent_top_slot_impressions = "parent_top_slot_impressions";
                                }
                                if (m.parent_required_budget_micros != null && m.hasOwnProperty("parent_required_budget_micros")) {
                                    if (typeof m.parent_required_budget_micros === "number")
                                        d.parent_required_budget_micros = o.longs === String ? String(m.parent_required_budget_micros) : m.parent_required_budget_micros;
                                    else
                                        d.parent_required_budget_micros = o.longs === String ? $util.Long.prototype.toString.call(m.parent_required_budget_micros) : o.longs === Number ? new $util.LongBits(m.parent_required_budget_micros.low >>> 0, m.parent_required_budget_micros.high >>> 0).toNumber() : m.parent_required_budget_micros;
                                    if (o.oneofs)
                                        d._parent_required_budget_micros = "parent_required_budget_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BidModifierSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BidModifierSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BidModifierSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BidModifierSimulationPoint;
                        })();
    
                        common.CpcBidSimulationPoint = (function() {
    
                            /**
                             * Properties of a CpcBidSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICpcBidSimulationPoint
                             * @property {number|null} [required_budget_amount_micros] CpcBidSimulationPoint required_budget_amount_micros
                             * @property {number|null} [biddable_conversions] CpcBidSimulationPoint biddable_conversions
                             * @property {number|null} [biddable_conversions_value] CpcBidSimulationPoint biddable_conversions_value
                             * @property {number|null} [clicks] CpcBidSimulationPoint clicks
                             * @property {number|null} [cost_micros] CpcBidSimulationPoint cost_micros
                             * @property {number|null} [impressions] CpcBidSimulationPoint impressions
                             * @property {number|null} [top_slot_impressions] CpcBidSimulationPoint top_slot_impressions
                             * @property {number|null} [cpc_bid_micros] CpcBidSimulationPoint cpc_bid_micros
                             * @property {number|null} [cpc_bid_scaling_modifier] CpcBidSimulationPoint cpc_bid_scaling_modifier
                             */
    
                            /**
                             * Constructs a new CpcBidSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CpcBidSimulationPoint.
                             * @implements ICpcBidSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPoint=} [p] Properties to set
                             */
                            function CpcBidSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CpcBidSimulationPoint required_budget_amount_micros.
                             * @member {number} required_budget_amount_micros
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.required_budget_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CpcBidSimulationPoint biddable_conversions.
                             * @member {number|null|undefined} biddable_conversions
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.biddable_conversions = null;
    
                            /**
                             * CpcBidSimulationPoint biddable_conversions_value.
                             * @member {number|null|undefined} biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.biddable_conversions_value = null;
    
                            /**
                             * CpcBidSimulationPoint clicks.
                             * @member {number|null|undefined} clicks
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.clicks = null;
    
                            /**
                             * CpcBidSimulationPoint cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.cost_micros = null;
    
                            /**
                             * CpcBidSimulationPoint impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.impressions = null;
    
                            /**
                             * CpcBidSimulationPoint top_slot_impressions.
                             * @member {number|null|undefined} top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.top_slot_impressions = null;
    
                            /**
                             * CpcBidSimulationPoint cpc_bid_micros.
                             * @member {number|null|undefined} cpc_bid_micros
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.cpc_bid_micros = null;
    
                            /**
                             * CpcBidSimulationPoint cpc_bid_scaling_modifier.
                             * @member {number|null|undefined} cpc_bid_scaling_modifier
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            CpcBidSimulationPoint.prototype.cpc_bid_scaling_modifier = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CpcBidSimulationPoint _biddable_conversions.
                             * @member {"biddable_conversions"|undefined} _biddable_conversions
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpcBidSimulationPoint.prototype, "_biddable_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpcBidSimulationPoint _biddable_conversions_value.
                             * @member {"biddable_conversions_value"|undefined} _biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpcBidSimulationPoint.prototype, "_biddable_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpcBidSimulationPoint _clicks.
                             * @member {"clicks"|undefined} _clicks
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpcBidSimulationPoint.prototype, "_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpcBidSimulationPoint _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpcBidSimulationPoint.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpcBidSimulationPoint _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpcBidSimulationPoint.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpcBidSimulationPoint _top_slot_impressions.
                             * @member {"top_slot_impressions"|undefined} _top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpcBidSimulationPoint.prototype, "_top_slot_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["top_slot_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpcBidSimulationPoint cpc_simulation_key_value.
                             * @member {"cpc_bid_micros"|"cpc_bid_scaling_modifier"|undefined} cpc_simulation_key_value
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpcBidSimulationPoint.prototype, "cpc_simulation_key_value", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_micros", "cpc_bid_scaling_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CpcBidSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPoint} CpcBidSimulationPoint instance
                             */
                            CpcBidSimulationPoint.create = function create(properties) {
                                return new CpcBidSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified CpcBidSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.CpcBidSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPoint} m CpcBidSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpcBidSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.biddable_conversions != null && Object.hasOwnProperty.call(m, "biddable_conversions"))
                                    w.uint32(73).double(m.biddable_conversions);
                                if (m.biddable_conversions_value != null && Object.hasOwnProperty.call(m, "biddable_conversions_value"))
                                    w.uint32(81).double(m.biddable_conversions_value);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(88).int64(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(96).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(104).int64(m.impressions);
                                if (m.top_slot_impressions != null && Object.hasOwnProperty.call(m, "top_slot_impressions"))
                                    w.uint32(112).int64(m.top_slot_impressions);
                                if (m.cpc_bid_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_micros"))
                                    w.uint32(120).int64(m.cpc_bid_micros);
                                if (m.cpc_bid_scaling_modifier != null && Object.hasOwnProperty.call(m, "cpc_bid_scaling_modifier"))
                                    w.uint32(129).double(m.cpc_bid_scaling_modifier);
                                if (m.required_budget_amount_micros != null && Object.hasOwnProperty.call(m, "required_budget_amount_micros"))
                                    w.uint32(136).int64(m.required_budget_amount_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CpcBidSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CpcBidSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpcBidSimulationPoint} message CpcBidSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpcBidSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CpcBidSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPoint} CpcBidSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpcBidSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CpcBidSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 17:
                                        m.required_budget_amount_micros = r.int64();
                                        break;
                                    case 9:
                                        m.biddable_conversions = r.double();
                                        break;
                                    case 10:
                                        m.biddable_conversions_value = r.double();
                                        break;
                                    case 11:
                                        m.clicks = r.int64();
                                        break;
                                    case 12:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 13:
                                        m.impressions = r.int64();
                                        break;
                                    case 14:
                                        m.top_slot_impressions = r.int64();
                                        break;
                                    case 15:
                                        m.cpc_bid_micros = r.int64();
                                        break;
                                    case 16:
                                        m.cpc_bid_scaling_modifier = r.double();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CpcBidSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPoint} CpcBidSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpcBidSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CpcBidSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CpcBidSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (!$util.isInteger(m.required_budget_amount_micros) && !(m.required_budget_amount_micros && $util.isInteger(m.required_budget_amount_micros.low) && $util.isInteger(m.required_budget_amount_micros.high)))
                                        return "required_budget_amount_micros: integer|Long expected";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    p._biddable_conversions = 1;
                                    if (typeof m.biddable_conversions !== "number")
                                        return "biddable_conversions: number expected";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    p._biddable_conversions_value = 1;
                                    if (typeof m.biddable_conversions_value !== "number")
                                        return "biddable_conversions_value: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    p._clicks = 1;
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    p._top_slot_impressions = 1;
                                    if (!$util.isInteger(m.top_slot_impressions) && !(m.top_slot_impressions && $util.isInteger(m.top_slot_impressions.low) && $util.isInteger(m.top_slot_impressions.high)))
                                        return "top_slot_impressions: integer|Long expected";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    p.cpc_simulation_key_value = 1;
                                    if (!$util.isInteger(m.cpc_bid_micros) && !(m.cpc_bid_micros && $util.isInteger(m.cpc_bid_micros.low) && $util.isInteger(m.cpc_bid_micros.high)))
                                        return "cpc_bid_micros: integer|Long expected";
                                }
                                if (m.cpc_bid_scaling_modifier != null && m.hasOwnProperty("cpc_bid_scaling_modifier")) {
                                    if (p.cpc_simulation_key_value === 1)
                                        return "cpc_simulation_key_value: multiple values";
                                    p.cpc_simulation_key_value = 1;
                                    if (typeof m.cpc_bid_scaling_modifier !== "number")
                                        return "cpc_bid_scaling_modifier: number expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CpcBidSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CpcBidSimulationPoint} CpcBidSimulationPoint
                             */
                            CpcBidSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CpcBidSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CpcBidSimulationPoint();
                                if (d.required_budget_amount_micros != null) {
                                    if ($util.Long)
                                        (m.required_budget_amount_micros = $util.Long.fromValue(d.required_budget_amount_micros)).unsigned = false;
                                    else if (typeof d.required_budget_amount_micros === "string")
                                        m.required_budget_amount_micros = parseInt(d.required_budget_amount_micros, 10);
                                    else if (typeof d.required_budget_amount_micros === "number")
                                        m.required_budget_amount_micros = d.required_budget_amount_micros;
                                    else if (typeof d.required_budget_amount_micros === "object")
                                        m.required_budget_amount_micros = new $util.LongBits(d.required_budget_amount_micros.low >>> 0, d.required_budget_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.biddable_conversions != null) {
                                    m.biddable_conversions = Number(d.biddable_conversions);
                                }
                                if (d.biddable_conversions_value != null) {
                                    m.biddable_conversions_value = Number(d.biddable_conversions_value);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.top_slot_impressions = $util.Long.fromValue(d.top_slot_impressions)).unsigned = false;
                                    else if (typeof d.top_slot_impressions === "string")
                                        m.top_slot_impressions = parseInt(d.top_slot_impressions, 10);
                                    else if (typeof d.top_slot_impressions === "number")
                                        m.top_slot_impressions = d.top_slot_impressions;
                                    else if (typeof d.top_slot_impressions === "object")
                                        m.top_slot_impressions = new $util.LongBits(d.top_slot_impressions.low >>> 0, d.top_slot_impressions.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_micros = $util.Long.fromValue(d.cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_micros === "string")
                                        m.cpc_bid_micros = parseInt(d.cpc_bid_micros, 10);
                                    else if (typeof d.cpc_bid_micros === "number")
                                        m.cpc_bid_micros = d.cpc_bid_micros;
                                    else if (typeof d.cpc_bid_micros === "object")
                                        m.cpc_bid_micros = new $util.LongBits(d.cpc_bid_micros.low >>> 0, d.cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.cpc_bid_scaling_modifier != null) {
                                    m.cpc_bid_scaling_modifier = Number(d.cpc_bid_scaling_modifier);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CpcBidSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.CpcBidSimulationPoint} m CpcBidSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CpcBidSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.required_budget_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.required_budget_amount_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    d.biddable_conversions = o.json && !isFinite(m.biddable_conversions) ? String(m.biddable_conversions) : m.biddable_conversions;
                                    if (o.oneofs)
                                        d._biddable_conversions = "biddable_conversions";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    d.biddable_conversions_value = o.json && !isFinite(m.biddable_conversions_value) ? String(m.biddable_conversions_value) : m.biddable_conversions_value;
                                    if (o.oneofs)
                                        d._biddable_conversions_value = "biddable_conversions_value";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                    if (o.oneofs)
                                        d._clicks = "clicks";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (typeof m.top_slot_impressions === "number")
                                        d.top_slot_impressions = o.longs === String ? String(m.top_slot_impressions) : m.top_slot_impressions;
                                    else
                                        d.top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.top_slot_impressions.low >>> 0, m.top_slot_impressions.high >>> 0).toNumber() : m.top_slot_impressions;
                                    if (o.oneofs)
                                        d._top_slot_impressions = "top_slot_impressions";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    if (typeof m.cpc_bid_micros === "number")
                                        d.cpc_bid_micros = o.longs === String ? String(m.cpc_bid_micros) : m.cpc_bid_micros;
                                    else
                                        d.cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_micros.low >>> 0, m.cpc_bid_micros.high >>> 0).toNumber() : m.cpc_bid_micros;
                                    if (o.oneofs)
                                        d.cpc_simulation_key_value = "cpc_bid_micros";
                                }
                                if (m.cpc_bid_scaling_modifier != null && m.hasOwnProperty("cpc_bid_scaling_modifier")) {
                                    d.cpc_bid_scaling_modifier = o.json && !isFinite(m.cpc_bid_scaling_modifier) ? String(m.cpc_bid_scaling_modifier) : m.cpc_bid_scaling_modifier;
                                    if (o.oneofs)
                                        d.cpc_simulation_key_value = "cpc_bid_scaling_modifier";
                                }
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (typeof m.required_budget_amount_micros === "number")
                                        d.required_budget_amount_micros = o.longs === String ? String(m.required_budget_amount_micros) : m.required_budget_amount_micros;
                                    else
                                        d.required_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.required_budget_amount_micros.low >>> 0, m.required_budget_amount_micros.high >>> 0).toNumber() : m.required_budget_amount_micros;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CpcBidSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CpcBidSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CpcBidSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CpcBidSimulationPoint;
                        })();
    
                        common.CpvBidSimulationPoint = (function() {
    
                            /**
                             * Properties of a CpvBidSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICpvBidSimulationPoint
                             * @property {number|null} [cpv_bid_micros] CpvBidSimulationPoint cpv_bid_micros
                             * @property {number|null} [cost_micros] CpvBidSimulationPoint cost_micros
                             * @property {number|null} [impressions] CpvBidSimulationPoint impressions
                             * @property {number|null} [views] CpvBidSimulationPoint views
                             */
    
                            /**
                             * Constructs a new CpvBidSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CpvBidSimulationPoint.
                             * @implements ICpvBidSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPoint=} [p] Properties to set
                             */
                            function CpvBidSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CpvBidSimulationPoint cpv_bid_micros.
                             * @member {number|null|undefined} cpv_bid_micros
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            CpvBidSimulationPoint.prototype.cpv_bid_micros = null;
    
                            /**
                             * CpvBidSimulationPoint cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            CpvBidSimulationPoint.prototype.cost_micros = null;
    
                            /**
                             * CpvBidSimulationPoint impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            CpvBidSimulationPoint.prototype.impressions = null;
    
                            /**
                             * CpvBidSimulationPoint views.
                             * @member {number|null|undefined} views
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            CpvBidSimulationPoint.prototype.views = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CpvBidSimulationPoint _cpv_bid_micros.
                             * @member {"cpv_bid_micros"|undefined} _cpv_bid_micros
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpvBidSimulationPoint.prototype, "_cpv_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpv_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpvBidSimulationPoint _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpvBidSimulationPoint.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpvBidSimulationPoint _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpvBidSimulationPoint.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CpvBidSimulationPoint _views.
                             * @member {"views"|undefined} _views
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(CpvBidSimulationPoint.prototype, "_views", {
                                get: $util.oneOfGetter($oneOfFields = ["views"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CpvBidSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPoint} CpvBidSimulationPoint instance
                             */
                            CpvBidSimulationPoint.create = function create(properties) {
                                return new CpvBidSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified CpvBidSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.CpvBidSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPoint} m CpvBidSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpvBidSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.cpv_bid_micros != null && Object.hasOwnProperty.call(m, "cpv_bid_micros"))
                                    w.uint32(40).int64(m.cpv_bid_micros);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(48).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(56).int64(m.impressions);
                                if (m.views != null && Object.hasOwnProperty.call(m, "views"))
                                    w.uint32(64).int64(m.views);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CpvBidSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CpvBidSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ICpvBidSimulationPoint} message CpvBidSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CpvBidSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CpvBidSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPoint} CpvBidSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpvBidSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CpvBidSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.cpv_bid_micros = r.int64();
                                        break;
                                    case 6:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 7:
                                        m.impressions = r.int64();
                                        break;
                                    case 8:
                                        m.views = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CpvBidSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPoint} CpvBidSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CpvBidSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CpvBidSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CpvBidSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.cpv_bid_micros != null && m.hasOwnProperty("cpv_bid_micros")) {
                                    p._cpv_bid_micros = 1;
                                    if (!$util.isInteger(m.cpv_bid_micros) && !(m.cpv_bid_micros && $util.isInteger(m.cpv_bid_micros.low) && $util.isInteger(m.cpv_bid_micros.high)))
                                        return "cpv_bid_micros: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.views != null && m.hasOwnProperty("views")) {
                                    p._views = 1;
                                    if (!$util.isInteger(m.views) && !(m.views && $util.isInteger(m.views.low) && $util.isInteger(m.views.high)))
                                        return "views: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CpvBidSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CpvBidSimulationPoint} CpvBidSimulationPoint
                             */
                            CpvBidSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CpvBidSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CpvBidSimulationPoint();
                                if (d.cpv_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpv_bid_micros = $util.Long.fromValue(d.cpv_bid_micros)).unsigned = false;
                                    else if (typeof d.cpv_bid_micros === "string")
                                        m.cpv_bid_micros = parseInt(d.cpv_bid_micros, 10);
                                    else if (typeof d.cpv_bid_micros === "number")
                                        m.cpv_bid_micros = d.cpv_bid_micros;
                                    else if (typeof d.cpv_bid_micros === "object")
                                        m.cpv_bid_micros = new $util.LongBits(d.cpv_bid_micros.low >>> 0, d.cpv_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.views != null) {
                                    if ($util.Long)
                                        (m.views = $util.Long.fromValue(d.views)).unsigned = false;
                                    else if (typeof d.views === "string")
                                        m.views = parseInt(d.views, 10);
                                    else if (typeof d.views === "number")
                                        m.views = d.views;
                                    else if (typeof d.views === "object")
                                        m.views = new $util.LongBits(d.views.low >>> 0, d.views.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CpvBidSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.CpvBidSimulationPoint} m CpvBidSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CpvBidSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.cpv_bid_micros != null && m.hasOwnProperty("cpv_bid_micros")) {
                                    if (typeof m.cpv_bid_micros === "number")
                                        d.cpv_bid_micros = o.longs === String ? String(m.cpv_bid_micros) : m.cpv_bid_micros;
                                    else
                                        d.cpv_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpv_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpv_bid_micros.low >>> 0, m.cpv_bid_micros.high >>> 0).toNumber() : m.cpv_bid_micros;
                                    if (o.oneofs)
                                        d._cpv_bid_micros = "cpv_bid_micros";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.views != null && m.hasOwnProperty("views")) {
                                    if (typeof m.views === "number")
                                        d.views = o.longs === String ? String(m.views) : m.views;
                                    else
                                        d.views = o.longs === String ? $util.Long.prototype.toString.call(m.views) : o.longs === Number ? new $util.LongBits(m.views.low >>> 0, m.views.high >>> 0).toNumber() : m.views;
                                    if (o.oneofs)
                                        d._views = "views";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CpvBidSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CpvBidSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CpvBidSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CpvBidSimulationPoint;
                        })();
    
                        common.TargetCpaSimulationPoint = (function() {
    
                            /**
                             * Properties of a TargetCpaSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetCpaSimulationPoint
                             * @property {number|null} [required_budget_amount_micros] TargetCpaSimulationPoint required_budget_amount_micros
                             * @property {number|null} [biddable_conversions] TargetCpaSimulationPoint biddable_conversions
                             * @property {number|null} [biddable_conversions_value] TargetCpaSimulationPoint biddable_conversions_value
                             * @property {number|null} [app_installs] TargetCpaSimulationPoint app_installs
                             * @property {number|null} [in_app_actions] TargetCpaSimulationPoint in_app_actions
                             * @property {number|null} [clicks] TargetCpaSimulationPoint clicks
                             * @property {number|null} [cost_micros] TargetCpaSimulationPoint cost_micros
                             * @property {number|null} [impressions] TargetCpaSimulationPoint impressions
                             * @property {number|null} [top_slot_impressions] TargetCpaSimulationPoint top_slot_impressions
                             * @property {number|null} [target_cpa_micros] TargetCpaSimulationPoint target_cpa_micros
                             * @property {number|null} [target_cpa_scaling_modifier] TargetCpaSimulationPoint target_cpa_scaling_modifier
                             */
    
                            /**
                             * Constructs a new TargetCpaSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetCpaSimulationPoint.
                             * @implements ITargetCpaSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPoint=} [p] Properties to set
                             */
                            function TargetCpaSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetCpaSimulationPoint required_budget_amount_micros.
                             * @member {number} required_budget_amount_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.required_budget_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetCpaSimulationPoint biddable_conversions.
                             * @member {number|null|undefined} biddable_conversions
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.biddable_conversions = null;
    
                            /**
                             * TargetCpaSimulationPoint biddable_conversions_value.
                             * @member {number|null|undefined} biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.biddable_conversions_value = null;
    
                            /**
                             * TargetCpaSimulationPoint app_installs.
                             * @member {number} app_installs
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.app_installs = 0;
    
                            /**
                             * TargetCpaSimulationPoint in_app_actions.
                             * @member {number} in_app_actions
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.in_app_actions = 0;
    
                            /**
                             * TargetCpaSimulationPoint clicks.
                             * @member {number|null|undefined} clicks
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.clicks = null;
    
                            /**
                             * TargetCpaSimulationPoint cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.cost_micros = null;
    
                            /**
                             * TargetCpaSimulationPoint impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.impressions = null;
    
                            /**
                             * TargetCpaSimulationPoint top_slot_impressions.
                             * @member {number|null|undefined} top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.top_slot_impressions = null;
    
                            /**
                             * TargetCpaSimulationPoint target_cpa_micros.
                             * @member {number|null|undefined} target_cpa_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.target_cpa_micros = null;
    
                            /**
                             * TargetCpaSimulationPoint target_cpa_scaling_modifier.
                             * @member {number|null|undefined} target_cpa_scaling_modifier
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            TargetCpaSimulationPoint.prototype.target_cpa_scaling_modifier = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TargetCpaSimulationPoint _biddable_conversions.
                             * @member {"biddable_conversions"|undefined} _biddable_conversions
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetCpaSimulationPoint.prototype, "_biddable_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpaSimulationPoint _biddable_conversions_value.
                             * @member {"biddable_conversions_value"|undefined} _biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetCpaSimulationPoint.prototype, "_biddable_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpaSimulationPoint _clicks.
                             * @member {"clicks"|undefined} _clicks
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetCpaSimulationPoint.prototype, "_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpaSimulationPoint _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetCpaSimulationPoint.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpaSimulationPoint _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetCpaSimulationPoint.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpaSimulationPoint _top_slot_impressions.
                             * @member {"top_slot_impressions"|undefined} _top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetCpaSimulationPoint.prototype, "_top_slot_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["top_slot_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetCpaSimulationPoint target_cpa_simulation_key_value.
                             * @member {"target_cpa_micros"|"target_cpa_scaling_modifier"|undefined} target_cpa_simulation_key_value
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetCpaSimulationPoint.prototype, "target_cpa_simulation_key_value", {
                                get: $util.oneOfGetter($oneOfFields = ["target_cpa_micros", "target_cpa_scaling_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TargetCpaSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPoint} TargetCpaSimulationPoint instance
                             */
                            TargetCpaSimulationPoint.create = function create(properties) {
                                return new TargetCpaSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified TargetCpaSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpaSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPoint} m TargetCpaSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpaSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.biddable_conversions != null && Object.hasOwnProperty.call(m, "biddable_conversions"))
                                    w.uint32(73).double(m.biddable_conversions);
                                if (m.biddable_conversions_value != null && Object.hasOwnProperty.call(m, "biddable_conversions_value"))
                                    w.uint32(81).double(m.biddable_conversions_value);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(88).int64(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(96).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(104).int64(m.impressions);
                                if (m.top_slot_impressions != null && Object.hasOwnProperty.call(m, "top_slot_impressions"))
                                    w.uint32(112).int64(m.top_slot_impressions);
                                if (m.app_installs != null && Object.hasOwnProperty.call(m, "app_installs"))
                                    w.uint32(121).double(m.app_installs);
                                if (m.in_app_actions != null && Object.hasOwnProperty.call(m, "in_app_actions"))
                                    w.uint32(129).double(m.in_app_actions);
                                if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                    w.uint32(136).int64(m.target_cpa_micros);
                                if (m.target_cpa_scaling_modifier != null && Object.hasOwnProperty.call(m, "target_cpa_scaling_modifier"))
                                    w.uint32(145).double(m.target_cpa_scaling_modifier);
                                if (m.required_budget_amount_micros != null && Object.hasOwnProperty.call(m, "required_budget_amount_micros"))
                                    w.uint32(152).int64(m.required_budget_amount_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetCpaSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetCpaSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetCpaSimulationPoint} message TargetCpaSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpaSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetCpaSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPoint} TargetCpaSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpaSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetCpaSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 19:
                                        m.required_budget_amount_micros = r.int64();
                                        break;
                                    case 9:
                                        m.biddable_conversions = r.double();
                                        break;
                                    case 10:
                                        m.biddable_conversions_value = r.double();
                                        break;
                                    case 15:
                                        m.app_installs = r.double();
                                        break;
                                    case 16:
                                        m.in_app_actions = r.double();
                                        break;
                                    case 11:
                                        m.clicks = r.int64();
                                        break;
                                    case 12:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 13:
                                        m.impressions = r.int64();
                                        break;
                                    case 14:
                                        m.top_slot_impressions = r.int64();
                                        break;
                                    case 17:
                                        m.target_cpa_micros = r.int64();
                                        break;
                                    case 18:
                                        m.target_cpa_scaling_modifier = r.double();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetCpaSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPoint} TargetCpaSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpaSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetCpaSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetCpaSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (!$util.isInteger(m.required_budget_amount_micros) && !(m.required_budget_amount_micros && $util.isInteger(m.required_budget_amount_micros.low) && $util.isInteger(m.required_budget_amount_micros.high)))
                                        return "required_budget_amount_micros: integer|Long expected";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    p._biddable_conversions = 1;
                                    if (typeof m.biddable_conversions !== "number")
                                        return "biddable_conversions: number expected";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    p._biddable_conversions_value = 1;
                                    if (typeof m.biddable_conversions_value !== "number")
                                        return "biddable_conversions_value: number expected";
                                }
                                if (m.app_installs != null && m.hasOwnProperty("app_installs")) {
                                    if (typeof m.app_installs !== "number")
                                        return "app_installs: number expected";
                                }
                                if (m.in_app_actions != null && m.hasOwnProperty("in_app_actions")) {
                                    if (typeof m.in_app_actions !== "number")
                                        return "in_app_actions: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    p._clicks = 1;
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    p._top_slot_impressions = 1;
                                    if (!$util.isInteger(m.top_slot_impressions) && !(m.top_slot_impressions && $util.isInteger(m.top_slot_impressions.low) && $util.isInteger(m.top_slot_impressions.high)))
                                        return "top_slot_impressions: integer|Long expected";
                                }
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    p.target_cpa_simulation_key_value = 1;
                                    if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                        return "target_cpa_micros: integer|Long expected";
                                }
                                if (m.target_cpa_scaling_modifier != null && m.hasOwnProperty("target_cpa_scaling_modifier")) {
                                    if (p.target_cpa_simulation_key_value === 1)
                                        return "target_cpa_simulation_key_value: multiple values";
                                    p.target_cpa_simulation_key_value = 1;
                                    if (typeof m.target_cpa_scaling_modifier !== "number")
                                        return "target_cpa_scaling_modifier: number expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetCpaSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetCpaSimulationPoint} TargetCpaSimulationPoint
                             */
                            TargetCpaSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetCpaSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetCpaSimulationPoint();
                                if (d.required_budget_amount_micros != null) {
                                    if ($util.Long)
                                        (m.required_budget_amount_micros = $util.Long.fromValue(d.required_budget_amount_micros)).unsigned = false;
                                    else if (typeof d.required_budget_amount_micros === "string")
                                        m.required_budget_amount_micros = parseInt(d.required_budget_amount_micros, 10);
                                    else if (typeof d.required_budget_amount_micros === "number")
                                        m.required_budget_amount_micros = d.required_budget_amount_micros;
                                    else if (typeof d.required_budget_amount_micros === "object")
                                        m.required_budget_amount_micros = new $util.LongBits(d.required_budget_amount_micros.low >>> 0, d.required_budget_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.biddable_conversions != null) {
                                    m.biddable_conversions = Number(d.biddable_conversions);
                                }
                                if (d.biddable_conversions_value != null) {
                                    m.biddable_conversions_value = Number(d.biddable_conversions_value);
                                }
                                if (d.app_installs != null) {
                                    m.app_installs = Number(d.app_installs);
                                }
                                if (d.in_app_actions != null) {
                                    m.in_app_actions = Number(d.in_app_actions);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.top_slot_impressions = $util.Long.fromValue(d.top_slot_impressions)).unsigned = false;
                                    else if (typeof d.top_slot_impressions === "string")
                                        m.top_slot_impressions = parseInt(d.top_slot_impressions, 10);
                                    else if (typeof d.top_slot_impressions === "number")
                                        m.top_slot_impressions = d.top_slot_impressions;
                                    else if (typeof d.top_slot_impressions === "object")
                                        m.top_slot_impressions = new $util.LongBits(d.top_slot_impressions.low >>> 0, d.top_slot_impressions.high >>> 0).toNumber();
                                }
                                if (d.target_cpa_micros != null) {
                                    if ($util.Long)
                                        (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                    else if (typeof d.target_cpa_micros === "string")
                                        m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                    else if (typeof d.target_cpa_micros === "number")
                                        m.target_cpa_micros = d.target_cpa_micros;
                                    else if (typeof d.target_cpa_micros === "object")
                                        m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                }
                                if (d.target_cpa_scaling_modifier != null) {
                                    m.target_cpa_scaling_modifier = Number(d.target_cpa_scaling_modifier);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetCpaSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetCpaSimulationPoint} m TargetCpaSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetCpaSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.app_installs = 0;
                                    d.in_app_actions = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.required_budget_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.required_budget_amount_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    d.biddable_conversions = o.json && !isFinite(m.biddable_conversions) ? String(m.biddable_conversions) : m.biddable_conversions;
                                    if (o.oneofs)
                                        d._biddable_conversions = "biddable_conversions";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    d.biddable_conversions_value = o.json && !isFinite(m.biddable_conversions_value) ? String(m.biddable_conversions_value) : m.biddable_conversions_value;
                                    if (o.oneofs)
                                        d._biddable_conversions_value = "biddable_conversions_value";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                    if (o.oneofs)
                                        d._clicks = "clicks";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (typeof m.top_slot_impressions === "number")
                                        d.top_slot_impressions = o.longs === String ? String(m.top_slot_impressions) : m.top_slot_impressions;
                                    else
                                        d.top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.top_slot_impressions.low >>> 0, m.top_slot_impressions.high >>> 0).toNumber() : m.top_slot_impressions;
                                    if (o.oneofs)
                                        d._top_slot_impressions = "top_slot_impressions";
                                }
                                if (m.app_installs != null && m.hasOwnProperty("app_installs")) {
                                    d.app_installs = o.json && !isFinite(m.app_installs) ? String(m.app_installs) : m.app_installs;
                                }
                                if (m.in_app_actions != null && m.hasOwnProperty("in_app_actions")) {
                                    d.in_app_actions = o.json && !isFinite(m.in_app_actions) ? String(m.in_app_actions) : m.in_app_actions;
                                }
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    if (typeof m.target_cpa_micros === "number")
                                        d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                    else
                                        d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                    if (o.oneofs)
                                        d.target_cpa_simulation_key_value = "target_cpa_micros";
                                }
                                if (m.target_cpa_scaling_modifier != null && m.hasOwnProperty("target_cpa_scaling_modifier")) {
                                    d.target_cpa_scaling_modifier = o.json && !isFinite(m.target_cpa_scaling_modifier) ? String(m.target_cpa_scaling_modifier) : m.target_cpa_scaling_modifier;
                                    if (o.oneofs)
                                        d.target_cpa_simulation_key_value = "target_cpa_scaling_modifier";
                                }
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (typeof m.required_budget_amount_micros === "number")
                                        d.required_budget_amount_micros = o.longs === String ? String(m.required_budget_amount_micros) : m.required_budget_amount_micros;
                                    else
                                        d.required_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.required_budget_amount_micros.low >>> 0, m.required_budget_amount_micros.high >>> 0).toNumber() : m.required_budget_amount_micros;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetCpaSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetCpaSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetCpaSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetCpaSimulationPoint;
                        })();
    
                        common.TargetRoasSimulationPoint = (function() {
    
                            /**
                             * Properties of a TargetRoasSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetRoasSimulationPoint
                             * @property {number|null} [target_roas] TargetRoasSimulationPoint target_roas
                             * @property {number|null} [required_budget_amount_micros] TargetRoasSimulationPoint required_budget_amount_micros
                             * @property {number|null} [biddable_conversions] TargetRoasSimulationPoint biddable_conversions
                             * @property {number|null} [biddable_conversions_value] TargetRoasSimulationPoint biddable_conversions_value
                             * @property {number|null} [clicks] TargetRoasSimulationPoint clicks
                             * @property {number|null} [cost_micros] TargetRoasSimulationPoint cost_micros
                             * @property {number|null} [impressions] TargetRoasSimulationPoint impressions
                             * @property {number|null} [top_slot_impressions] TargetRoasSimulationPoint top_slot_impressions
                             */
    
                            /**
                             * Constructs a new TargetRoasSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetRoasSimulationPoint.
                             * @implements ITargetRoasSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPoint=} [p] Properties to set
                             */
                            function TargetRoasSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetRoasSimulationPoint target_roas.
                             * @member {number|null|undefined} target_roas
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.target_roas = null;
    
                            /**
                             * TargetRoasSimulationPoint required_budget_amount_micros.
                             * @member {number} required_budget_amount_micros
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.required_budget_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetRoasSimulationPoint biddable_conversions.
                             * @member {number|null|undefined} biddable_conversions
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.biddable_conversions = null;
    
                            /**
                             * TargetRoasSimulationPoint biddable_conversions_value.
                             * @member {number|null|undefined} biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.biddable_conversions_value = null;
    
                            /**
                             * TargetRoasSimulationPoint clicks.
                             * @member {number|null|undefined} clicks
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.clicks = null;
    
                            /**
                             * TargetRoasSimulationPoint cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.cost_micros = null;
    
                            /**
                             * TargetRoasSimulationPoint impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.impressions = null;
    
                            /**
                             * TargetRoasSimulationPoint top_slot_impressions.
                             * @member {number|null|undefined} top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            TargetRoasSimulationPoint.prototype.top_slot_impressions = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TargetRoasSimulationPoint _target_roas.
                             * @member {"target_roas"|undefined} _target_roas
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetRoasSimulationPoint.prototype, "_target_roas", {
                                get: $util.oneOfGetter($oneOfFields = ["target_roas"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoasSimulationPoint _biddable_conversions.
                             * @member {"biddable_conversions"|undefined} _biddable_conversions
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetRoasSimulationPoint.prototype, "_biddable_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoasSimulationPoint _biddable_conversions_value.
                             * @member {"biddable_conversions_value"|undefined} _biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetRoasSimulationPoint.prototype, "_biddable_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoasSimulationPoint _clicks.
                             * @member {"clicks"|undefined} _clicks
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetRoasSimulationPoint.prototype, "_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoasSimulationPoint _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetRoasSimulationPoint.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoasSimulationPoint _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetRoasSimulationPoint.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TargetRoasSimulationPoint _top_slot_impressions.
                             * @member {"top_slot_impressions"|undefined} _top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(TargetRoasSimulationPoint.prototype, "_top_slot_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["top_slot_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TargetRoasSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPoint} TargetRoasSimulationPoint instance
                             */
                            TargetRoasSimulationPoint.create = function create(properties) {
                                return new TargetRoasSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified TargetRoasSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.TargetRoasSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPoint} m TargetRoasSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRoasSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                    w.uint32(65).double(m.target_roas);
                                if (m.biddable_conversions != null && Object.hasOwnProperty.call(m, "biddable_conversions"))
                                    w.uint32(73).double(m.biddable_conversions);
                                if (m.biddable_conversions_value != null && Object.hasOwnProperty.call(m, "biddable_conversions_value"))
                                    w.uint32(81).double(m.biddable_conversions_value);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(88).int64(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(96).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(104).int64(m.impressions);
                                if (m.top_slot_impressions != null && Object.hasOwnProperty.call(m, "top_slot_impressions"))
                                    w.uint32(112).int64(m.top_slot_impressions);
                                if (m.required_budget_amount_micros != null && Object.hasOwnProperty.call(m, "required_budget_amount_micros"))
                                    w.uint32(120).int64(m.required_budget_amount_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetRoasSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetRoasSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRoasSimulationPoint} message TargetRoasSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRoasSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetRoasSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPoint} TargetRoasSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRoasSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetRoasSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 8:
                                        m.target_roas = r.double();
                                        break;
                                    case 15:
                                        m.required_budget_amount_micros = r.int64();
                                        break;
                                    case 9:
                                        m.biddable_conversions = r.double();
                                        break;
                                    case 10:
                                        m.biddable_conversions_value = r.double();
                                        break;
                                    case 11:
                                        m.clicks = r.int64();
                                        break;
                                    case 12:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 13:
                                        m.impressions = r.int64();
                                        break;
                                    case 14:
                                        m.top_slot_impressions = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetRoasSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPoint} TargetRoasSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRoasSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetRoasSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetRoasSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    p._target_roas = 1;
                                    if (typeof m.target_roas !== "number")
                                        return "target_roas: number expected";
                                }
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (!$util.isInteger(m.required_budget_amount_micros) && !(m.required_budget_amount_micros && $util.isInteger(m.required_budget_amount_micros.low) && $util.isInteger(m.required_budget_amount_micros.high)))
                                        return "required_budget_amount_micros: integer|Long expected";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    p._biddable_conversions = 1;
                                    if (typeof m.biddable_conversions !== "number")
                                        return "biddable_conversions: number expected";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    p._biddable_conversions_value = 1;
                                    if (typeof m.biddable_conversions_value !== "number")
                                        return "biddable_conversions_value: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    p._clicks = 1;
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    p._top_slot_impressions = 1;
                                    if (!$util.isInteger(m.top_slot_impressions) && !(m.top_slot_impressions && $util.isInteger(m.top_slot_impressions.low) && $util.isInteger(m.top_slot_impressions.high)))
                                        return "top_slot_impressions: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetRoasSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetRoasSimulationPoint} TargetRoasSimulationPoint
                             */
                            TargetRoasSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetRoasSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetRoasSimulationPoint();
                                if (d.target_roas != null) {
                                    m.target_roas = Number(d.target_roas);
                                }
                                if (d.required_budget_amount_micros != null) {
                                    if ($util.Long)
                                        (m.required_budget_amount_micros = $util.Long.fromValue(d.required_budget_amount_micros)).unsigned = false;
                                    else if (typeof d.required_budget_amount_micros === "string")
                                        m.required_budget_amount_micros = parseInt(d.required_budget_amount_micros, 10);
                                    else if (typeof d.required_budget_amount_micros === "number")
                                        m.required_budget_amount_micros = d.required_budget_amount_micros;
                                    else if (typeof d.required_budget_amount_micros === "object")
                                        m.required_budget_amount_micros = new $util.LongBits(d.required_budget_amount_micros.low >>> 0, d.required_budget_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.biddable_conversions != null) {
                                    m.biddable_conversions = Number(d.biddable_conversions);
                                }
                                if (d.biddable_conversions_value != null) {
                                    m.biddable_conversions_value = Number(d.biddable_conversions_value);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.top_slot_impressions = $util.Long.fromValue(d.top_slot_impressions)).unsigned = false;
                                    else if (typeof d.top_slot_impressions === "string")
                                        m.top_slot_impressions = parseInt(d.top_slot_impressions, 10);
                                    else if (typeof d.top_slot_impressions === "number")
                                        m.top_slot_impressions = d.top_slot_impressions;
                                    else if (typeof d.top_slot_impressions === "object")
                                        m.top_slot_impressions = new $util.LongBits(d.top_slot_impressions.low >>> 0, d.top_slot_impressions.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetRoasSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetRoasSimulationPoint} m TargetRoasSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetRoasSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.required_budget_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.required_budget_amount_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    d.target_roas = o.json && !isFinite(m.target_roas) ? String(m.target_roas) : m.target_roas;
                                    if (o.oneofs)
                                        d._target_roas = "target_roas";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    d.biddable_conversions = o.json && !isFinite(m.biddable_conversions) ? String(m.biddable_conversions) : m.biddable_conversions;
                                    if (o.oneofs)
                                        d._biddable_conversions = "biddable_conversions";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    d.biddable_conversions_value = o.json && !isFinite(m.biddable_conversions_value) ? String(m.biddable_conversions_value) : m.biddable_conversions_value;
                                    if (o.oneofs)
                                        d._biddable_conversions_value = "biddable_conversions_value";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                    if (o.oneofs)
                                        d._clicks = "clicks";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (typeof m.top_slot_impressions === "number")
                                        d.top_slot_impressions = o.longs === String ? String(m.top_slot_impressions) : m.top_slot_impressions;
                                    else
                                        d.top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.top_slot_impressions.low >>> 0, m.top_slot_impressions.high >>> 0).toNumber() : m.top_slot_impressions;
                                    if (o.oneofs)
                                        d._top_slot_impressions = "top_slot_impressions";
                                }
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (typeof m.required_budget_amount_micros === "number")
                                        d.required_budget_amount_micros = o.longs === String ? String(m.required_budget_amount_micros) : m.required_budget_amount_micros;
                                    else
                                        d.required_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.required_budget_amount_micros.low >>> 0, m.required_budget_amount_micros.high >>> 0).toNumber() : m.required_budget_amount_micros;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetRoasSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetRoasSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetRoasSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetRoasSimulationPoint;
                        })();
    
                        common.PercentCpcBidSimulationPoint = (function() {
    
                            /**
                             * Properties of a PercentCpcBidSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IPercentCpcBidSimulationPoint
                             * @property {number|null} [percent_cpc_bid_micros] PercentCpcBidSimulationPoint percent_cpc_bid_micros
                             * @property {number|null} [biddable_conversions] PercentCpcBidSimulationPoint biddable_conversions
                             * @property {number|null} [biddable_conversions_value] PercentCpcBidSimulationPoint biddable_conversions_value
                             * @property {number|null} [clicks] PercentCpcBidSimulationPoint clicks
                             * @property {number|null} [cost_micros] PercentCpcBidSimulationPoint cost_micros
                             * @property {number|null} [impressions] PercentCpcBidSimulationPoint impressions
                             * @property {number|null} [top_slot_impressions] PercentCpcBidSimulationPoint top_slot_impressions
                             */
    
                            /**
                             * Constructs a new PercentCpcBidSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a PercentCpcBidSimulationPoint.
                             * @implements IPercentCpcBidSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPoint=} [p] Properties to set
                             */
                            function PercentCpcBidSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PercentCpcBidSimulationPoint percent_cpc_bid_micros.
                             * @member {number|null|undefined} percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            PercentCpcBidSimulationPoint.prototype.percent_cpc_bid_micros = null;
    
                            /**
                             * PercentCpcBidSimulationPoint biddable_conversions.
                             * @member {number|null|undefined} biddable_conversions
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            PercentCpcBidSimulationPoint.prototype.biddable_conversions = null;
    
                            /**
                             * PercentCpcBidSimulationPoint biddable_conversions_value.
                             * @member {number|null|undefined} biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            PercentCpcBidSimulationPoint.prototype.biddable_conversions_value = null;
    
                            /**
                             * PercentCpcBidSimulationPoint clicks.
                             * @member {number|null|undefined} clicks
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            PercentCpcBidSimulationPoint.prototype.clicks = null;
    
                            /**
                             * PercentCpcBidSimulationPoint cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            PercentCpcBidSimulationPoint.prototype.cost_micros = null;
    
                            /**
                             * PercentCpcBidSimulationPoint impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            PercentCpcBidSimulationPoint.prototype.impressions = null;
    
                            /**
                             * PercentCpcBidSimulationPoint top_slot_impressions.
                             * @member {number|null|undefined} top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            PercentCpcBidSimulationPoint.prototype.top_slot_impressions = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PercentCpcBidSimulationPoint _percent_cpc_bid_micros.
                             * @member {"percent_cpc_bid_micros"|undefined} _percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(PercentCpcBidSimulationPoint.prototype, "_percent_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["percent_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PercentCpcBidSimulationPoint _biddable_conversions.
                             * @member {"biddable_conversions"|undefined} _biddable_conversions
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(PercentCpcBidSimulationPoint.prototype, "_biddable_conversions", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PercentCpcBidSimulationPoint _biddable_conversions_value.
                             * @member {"biddable_conversions_value"|undefined} _biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(PercentCpcBidSimulationPoint.prototype, "_biddable_conversions_value", {
                                get: $util.oneOfGetter($oneOfFields = ["biddable_conversions_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PercentCpcBidSimulationPoint _clicks.
                             * @member {"clicks"|undefined} _clicks
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(PercentCpcBidSimulationPoint.prototype, "_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PercentCpcBidSimulationPoint _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(PercentCpcBidSimulationPoint.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PercentCpcBidSimulationPoint _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(PercentCpcBidSimulationPoint.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PercentCpcBidSimulationPoint _top_slot_impressions.
                             * @member {"top_slot_impressions"|undefined} _top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             */
                            Object.defineProperty(PercentCpcBidSimulationPoint.prototype, "_top_slot_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["top_slot_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PercentCpcBidSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPoint} PercentCpcBidSimulationPoint instance
                             */
                            PercentCpcBidSimulationPoint.create = function create(properties) {
                                return new PercentCpcBidSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified PercentCpcBidSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.PercentCpcBidSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPoint} m PercentCpcBidSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PercentCpcBidSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.percent_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "percent_cpc_bid_micros"))
                                    w.uint32(8).int64(m.percent_cpc_bid_micros);
                                if (m.biddable_conversions != null && Object.hasOwnProperty.call(m, "biddable_conversions"))
                                    w.uint32(17).double(m.biddable_conversions);
                                if (m.biddable_conversions_value != null && Object.hasOwnProperty.call(m, "biddable_conversions_value"))
                                    w.uint32(25).double(m.biddable_conversions_value);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(32).int64(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(40).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(48).int64(m.impressions);
                                if (m.top_slot_impressions != null && Object.hasOwnProperty.call(m, "top_slot_impressions"))
                                    w.uint32(56).int64(m.top_slot_impressions);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PercentCpcBidSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.PercentCpcBidSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IPercentCpcBidSimulationPoint} message PercentCpcBidSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PercentCpcBidSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PercentCpcBidSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPoint} PercentCpcBidSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PercentCpcBidSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.percent_cpc_bid_micros = r.int64();
                                        break;
                                    case 2:
                                        m.biddable_conversions = r.double();
                                        break;
                                    case 3:
                                        m.biddable_conversions_value = r.double();
                                        break;
                                    case 4:
                                        m.clicks = r.int64();
                                        break;
                                    case 5:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 6:
                                        m.impressions = r.int64();
                                        break;
                                    case 7:
                                        m.top_slot_impressions = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PercentCpcBidSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPoint} PercentCpcBidSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PercentCpcBidSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PercentCpcBidSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PercentCpcBidSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.percent_cpc_bid_micros != null && m.hasOwnProperty("percent_cpc_bid_micros")) {
                                    p._percent_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.percent_cpc_bid_micros) && !(m.percent_cpc_bid_micros && $util.isInteger(m.percent_cpc_bid_micros.low) && $util.isInteger(m.percent_cpc_bid_micros.high)))
                                        return "percent_cpc_bid_micros: integer|Long expected";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    p._biddable_conversions = 1;
                                    if (typeof m.biddable_conversions !== "number")
                                        return "biddable_conversions: number expected";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    p._biddable_conversions_value = 1;
                                    if (typeof m.biddable_conversions_value !== "number")
                                        return "biddable_conversions_value: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    p._clicks = 1;
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    p._top_slot_impressions = 1;
                                    if (!$util.isInteger(m.top_slot_impressions) && !(m.top_slot_impressions && $util.isInteger(m.top_slot_impressions.low) && $util.isInteger(m.top_slot_impressions.high)))
                                        return "top_slot_impressions: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PercentCpcBidSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.PercentCpcBidSimulationPoint} PercentCpcBidSimulationPoint
                             */
                            PercentCpcBidSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPoint();
                                if (d.percent_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.percent_cpc_bid_micros = $util.Long.fromValue(d.percent_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.percent_cpc_bid_micros === "string")
                                        m.percent_cpc_bid_micros = parseInt(d.percent_cpc_bid_micros, 10);
                                    else if (typeof d.percent_cpc_bid_micros === "number")
                                        m.percent_cpc_bid_micros = d.percent_cpc_bid_micros;
                                    else if (typeof d.percent_cpc_bid_micros === "object")
                                        m.percent_cpc_bid_micros = new $util.LongBits(d.percent_cpc_bid_micros.low >>> 0, d.percent_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.biddable_conversions != null) {
                                    m.biddable_conversions = Number(d.biddable_conversions);
                                }
                                if (d.biddable_conversions_value != null) {
                                    m.biddable_conversions_value = Number(d.biddable_conversions_value);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.top_slot_impressions = $util.Long.fromValue(d.top_slot_impressions)).unsigned = false;
                                    else if (typeof d.top_slot_impressions === "string")
                                        m.top_slot_impressions = parseInt(d.top_slot_impressions, 10);
                                    else if (typeof d.top_slot_impressions === "number")
                                        m.top_slot_impressions = d.top_slot_impressions;
                                    else if (typeof d.top_slot_impressions === "object")
                                        m.top_slot_impressions = new $util.LongBits(d.top_slot_impressions.low >>> 0, d.top_slot_impressions.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PercentCpcBidSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.PercentCpcBidSimulationPoint} m PercentCpcBidSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PercentCpcBidSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.percent_cpc_bid_micros != null && m.hasOwnProperty("percent_cpc_bid_micros")) {
                                    if (typeof m.percent_cpc_bid_micros === "number")
                                        d.percent_cpc_bid_micros = o.longs === String ? String(m.percent_cpc_bid_micros) : m.percent_cpc_bid_micros;
                                    else
                                        d.percent_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.percent_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.percent_cpc_bid_micros.low >>> 0, m.percent_cpc_bid_micros.high >>> 0).toNumber() : m.percent_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._percent_cpc_bid_micros = "percent_cpc_bid_micros";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    d.biddable_conversions = o.json && !isFinite(m.biddable_conversions) ? String(m.biddable_conversions) : m.biddable_conversions;
                                    if (o.oneofs)
                                        d._biddable_conversions = "biddable_conversions";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    d.biddable_conversions_value = o.json && !isFinite(m.biddable_conversions_value) ? String(m.biddable_conversions_value) : m.biddable_conversions_value;
                                    if (o.oneofs)
                                        d._biddable_conversions_value = "biddable_conversions_value";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                    if (o.oneofs)
                                        d._clicks = "clicks";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (typeof m.top_slot_impressions === "number")
                                        d.top_slot_impressions = o.longs === String ? String(m.top_slot_impressions) : m.top_slot_impressions;
                                    else
                                        d.top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.top_slot_impressions.low >>> 0, m.top_slot_impressions.high >>> 0).toNumber() : m.top_slot_impressions;
                                    if (o.oneofs)
                                        d._top_slot_impressions = "top_slot_impressions";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PercentCpcBidSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.PercentCpcBidSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PercentCpcBidSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PercentCpcBidSimulationPoint;
                        })();
    
                        common.BudgetSimulationPoint = (function() {
    
                            /**
                             * Properties of a BudgetSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBudgetSimulationPoint
                             * @property {number|null} [budget_amount_micros] BudgetSimulationPoint budget_amount_micros
                             * @property {number|null} [required_cpc_bid_ceiling_micros] BudgetSimulationPoint required_cpc_bid_ceiling_micros
                             * @property {number|null} [biddable_conversions] BudgetSimulationPoint biddable_conversions
                             * @property {number|null} [biddable_conversions_value] BudgetSimulationPoint biddable_conversions_value
                             * @property {number|null} [clicks] BudgetSimulationPoint clicks
                             * @property {number|null} [cost_micros] BudgetSimulationPoint cost_micros
                             * @property {number|null} [impressions] BudgetSimulationPoint impressions
                             * @property {number|null} [top_slot_impressions] BudgetSimulationPoint top_slot_impressions
                             */
    
                            /**
                             * Constructs a new BudgetSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BudgetSimulationPoint.
                             * @implements IBudgetSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPoint=} [p] Properties to set
                             */
                            function BudgetSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BudgetSimulationPoint budget_amount_micros.
                             * @member {number} budget_amount_micros
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.budget_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BudgetSimulationPoint required_cpc_bid_ceiling_micros.
                             * @member {number} required_cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.required_cpc_bid_ceiling_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BudgetSimulationPoint biddable_conversions.
                             * @member {number} biddable_conversions
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.biddable_conversions = 0;
    
                            /**
                             * BudgetSimulationPoint biddable_conversions_value.
                             * @member {number} biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.biddable_conversions_value = 0;
    
                            /**
                             * BudgetSimulationPoint clicks.
                             * @member {number} clicks
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.clicks = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BudgetSimulationPoint cost_micros.
                             * @member {number} cost_micros
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.cost_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BudgetSimulationPoint impressions.
                             * @member {number} impressions
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.impressions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BudgetSimulationPoint top_slot_impressions.
                             * @member {number} top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             */
                            BudgetSimulationPoint.prototype.top_slot_impressions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new BudgetSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPoint} BudgetSimulationPoint instance
                             */
                            BudgetSimulationPoint.create = function create(properties) {
                                return new BudgetSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.BudgetSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPoint} m BudgetSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.budget_amount_micros != null && Object.hasOwnProperty.call(m, "budget_amount_micros"))
                                    w.uint32(8).int64(m.budget_amount_micros);
                                if (m.required_cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "required_cpc_bid_ceiling_micros"))
                                    w.uint32(16).int64(m.required_cpc_bid_ceiling_micros);
                                if (m.biddable_conversions != null && Object.hasOwnProperty.call(m, "biddable_conversions"))
                                    w.uint32(25).double(m.biddable_conversions);
                                if (m.biddable_conversions_value != null && Object.hasOwnProperty.call(m, "biddable_conversions_value"))
                                    w.uint32(33).double(m.biddable_conversions_value);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(40).int64(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(48).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(56).int64(m.impressions);
                                if (m.top_slot_impressions != null && Object.hasOwnProperty.call(m, "top_slot_impressions"))
                                    w.uint32(64).int64(m.top_slot_impressions);
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BudgetSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.IBudgetSimulationPoint} message BudgetSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPoint} BudgetSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BudgetSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.budget_amount_micros = r.int64();
                                        break;
                                    case 2:
                                        m.required_cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    case 3:
                                        m.biddable_conversions = r.double();
                                        break;
                                    case 4:
                                        m.biddable_conversions_value = r.double();
                                        break;
                                    case 5:
                                        m.clicks = r.int64();
                                        break;
                                    case 6:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 7:
                                        m.impressions = r.int64();
                                        break;
                                    case 8:
                                        m.top_slot_impressions = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPoint} BudgetSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.budget_amount_micros != null && m.hasOwnProperty("budget_amount_micros")) {
                                    if (!$util.isInteger(m.budget_amount_micros) && !(m.budget_amount_micros && $util.isInteger(m.budget_amount_micros.low) && $util.isInteger(m.budget_amount_micros.high)))
                                        return "budget_amount_micros: integer|Long expected";
                                }
                                if (m.required_cpc_bid_ceiling_micros != null && m.hasOwnProperty("required_cpc_bid_ceiling_micros")) {
                                    if (!$util.isInteger(m.required_cpc_bid_ceiling_micros) && !(m.required_cpc_bid_ceiling_micros && $util.isInteger(m.required_cpc_bid_ceiling_micros.low) && $util.isInteger(m.required_cpc_bid_ceiling_micros.high)))
                                        return "required_cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    if (typeof m.biddable_conversions !== "number")
                                        return "biddable_conversions: number expected";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    if (typeof m.biddable_conversions_value !== "number")
                                        return "biddable_conversions_value: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (!$util.isInteger(m.top_slot_impressions) && !(m.top_slot_impressions && $util.isInteger(m.top_slot_impressions.low) && $util.isInteger(m.top_slot_impressions.high)))
                                        return "top_slot_impressions: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BudgetSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BudgetSimulationPoint} BudgetSimulationPoint
                             */
                            BudgetSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BudgetSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.BudgetSimulationPoint();
                                if (d.budget_amount_micros != null) {
                                    if ($util.Long)
                                        (m.budget_amount_micros = $util.Long.fromValue(d.budget_amount_micros)).unsigned = false;
                                    else if (typeof d.budget_amount_micros === "string")
                                        m.budget_amount_micros = parseInt(d.budget_amount_micros, 10);
                                    else if (typeof d.budget_amount_micros === "number")
                                        m.budget_amount_micros = d.budget_amount_micros;
                                    else if (typeof d.budget_amount_micros === "object")
                                        m.budget_amount_micros = new $util.LongBits(d.budget_amount_micros.low >>> 0, d.budget_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.required_cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.required_cpc_bid_ceiling_micros = $util.Long.fromValue(d.required_cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.required_cpc_bid_ceiling_micros === "string")
                                        m.required_cpc_bid_ceiling_micros = parseInt(d.required_cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.required_cpc_bid_ceiling_micros === "number")
                                        m.required_cpc_bid_ceiling_micros = d.required_cpc_bid_ceiling_micros;
                                    else if (typeof d.required_cpc_bid_ceiling_micros === "object")
                                        m.required_cpc_bid_ceiling_micros = new $util.LongBits(d.required_cpc_bid_ceiling_micros.low >>> 0, d.required_cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                if (d.biddable_conversions != null) {
                                    m.biddable_conversions = Number(d.biddable_conversions);
                                }
                                if (d.biddable_conversions_value != null) {
                                    m.biddable_conversions_value = Number(d.biddable_conversions_value);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.top_slot_impressions = $util.Long.fromValue(d.top_slot_impressions)).unsigned = false;
                                    else if (typeof d.top_slot_impressions === "string")
                                        m.top_slot_impressions = parseInt(d.top_slot_impressions, 10);
                                    else if (typeof d.top_slot_impressions === "number")
                                        m.top_slot_impressions = d.top_slot_impressions;
                                    else if (typeof d.top_slot_impressions === "object")
                                        m.top_slot_impressions = new $util.LongBits(d.top_slot_impressions.low >>> 0, d.top_slot_impressions.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BudgetSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.BudgetSimulationPoint} m BudgetSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.budget_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.budget_amount_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? "0" : 0;
                                    d.biddable_conversions = 0;
                                    d.biddable_conversions_value = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.clicks = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.clicks = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cost_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cost_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.impressions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.impressions = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.top_slot_impressions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.top_slot_impressions = o.longs === String ? "0" : 0;
                                }
                                if (m.budget_amount_micros != null && m.hasOwnProperty("budget_amount_micros")) {
                                    if (typeof m.budget_amount_micros === "number")
                                        d.budget_amount_micros = o.longs === String ? String(m.budget_amount_micros) : m.budget_amount_micros;
                                    else
                                        d.budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.budget_amount_micros.low >>> 0, m.budget_amount_micros.high >>> 0).toNumber() : m.budget_amount_micros;
                                }
                                if (m.required_cpc_bid_ceiling_micros != null && m.hasOwnProperty("required_cpc_bid_ceiling_micros")) {
                                    if (typeof m.required_cpc_bid_ceiling_micros === "number")
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? String(m.required_cpc_bid_ceiling_micros) : m.required_cpc_bid_ceiling_micros;
                                    else
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.required_cpc_bid_ceiling_micros.low >>> 0, m.required_cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.required_cpc_bid_ceiling_micros;
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    d.biddable_conversions = o.json && !isFinite(m.biddable_conversions) ? String(m.biddable_conversions) : m.biddable_conversions;
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    d.biddable_conversions_value = o.json && !isFinite(m.biddable_conversions_value) ? String(m.biddable_conversions_value) : m.biddable_conversions_value;
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (typeof m.top_slot_impressions === "number")
                                        d.top_slot_impressions = o.longs === String ? String(m.top_slot_impressions) : m.top_slot_impressions;
                                    else
                                        d.top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.top_slot_impressions.low >>> 0, m.top_slot_impressions.high >>> 0).toNumber() : m.top_slot_impressions;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BudgetSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BudgetSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BudgetSimulationPoint;
                        })();
    
                        common.TargetImpressionShareSimulationPoint = (function() {
    
                            /**
                             * Properties of a TargetImpressionShareSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetImpressionShareSimulationPoint
                             * @property {number|null} [target_impression_share_micros] TargetImpressionShareSimulationPoint target_impression_share_micros
                             * @property {number|null} [required_cpc_bid_ceiling_micros] TargetImpressionShareSimulationPoint required_cpc_bid_ceiling_micros
                             * @property {number|null} [required_budget_amount_micros] TargetImpressionShareSimulationPoint required_budget_amount_micros
                             * @property {number|null} [biddable_conversions] TargetImpressionShareSimulationPoint biddable_conversions
                             * @property {number|null} [biddable_conversions_value] TargetImpressionShareSimulationPoint biddable_conversions_value
                             * @property {number|null} [clicks] TargetImpressionShareSimulationPoint clicks
                             * @property {number|null} [cost_micros] TargetImpressionShareSimulationPoint cost_micros
                             * @property {number|null} [impressions] TargetImpressionShareSimulationPoint impressions
                             * @property {number|null} [top_slot_impressions] TargetImpressionShareSimulationPoint top_slot_impressions
                             * @property {number|null} [absolute_top_impressions] TargetImpressionShareSimulationPoint absolute_top_impressions
                             */
    
                            /**
                             * Constructs a new TargetImpressionShareSimulationPoint.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetImpressionShareSimulationPoint.
                             * @implements ITargetImpressionShareSimulationPoint
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPoint=} [p] Properties to set
                             */
                            function TargetImpressionShareSimulationPoint(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetImpressionShareSimulationPoint target_impression_share_micros.
                             * @member {number} target_impression_share_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.target_impression_share_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint required_cpc_bid_ceiling_micros.
                             * @member {number} required_cpc_bid_ceiling_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.required_cpc_bid_ceiling_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint required_budget_amount_micros.
                             * @member {number} required_budget_amount_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.required_budget_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint biddable_conversions.
                             * @member {number} biddable_conversions
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.biddable_conversions = 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint biddable_conversions_value.
                             * @member {number} biddable_conversions_value
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.biddable_conversions_value = 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint clicks.
                             * @member {number} clicks
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.clicks = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint cost_micros.
                             * @member {number} cost_micros
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.cost_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint impressions.
                             * @member {number} impressions
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.impressions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint top_slot_impressions.
                             * @member {number} top_slot_impressions
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.top_slot_impressions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TargetImpressionShareSimulationPoint absolute_top_impressions.
                             * @member {number} absolute_top_impressions
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             */
                            TargetImpressionShareSimulationPoint.prototype.absolute_top_impressions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new TargetImpressionShareSimulationPoint instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPoint=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint} TargetImpressionShareSimulationPoint instance
                             */
                            TargetImpressionShareSimulationPoint.create = function create(properties) {
                                return new TargetImpressionShareSimulationPoint(properties);
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShareSimulationPoint message. Does not implicitly {@link google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPoint} m TargetImpressionShareSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShareSimulationPoint.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.target_impression_share_micros != null && Object.hasOwnProperty.call(m, "target_impression_share_micros"))
                                    w.uint32(8).int64(m.target_impression_share_micros);
                                if (m.required_cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "required_cpc_bid_ceiling_micros"))
                                    w.uint32(16).int64(m.required_cpc_bid_ceiling_micros);
                                if (m.required_budget_amount_micros != null && Object.hasOwnProperty.call(m, "required_budget_amount_micros"))
                                    w.uint32(24).int64(m.required_budget_amount_micros);
                                if (m.biddable_conversions != null && Object.hasOwnProperty.call(m, "biddable_conversions"))
                                    w.uint32(33).double(m.biddable_conversions);
                                if (m.biddable_conversions_value != null && Object.hasOwnProperty.call(m, "biddable_conversions_value"))
                                    w.uint32(41).double(m.biddable_conversions_value);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(48).int64(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(56).int64(m.cost_micros);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(64).int64(m.impressions);
                                if (m.top_slot_impressions != null && Object.hasOwnProperty.call(m, "top_slot_impressions"))
                                    w.uint32(72).int64(m.top_slot_impressions);
                                if (m.absolute_top_impressions != null && Object.hasOwnProperty.call(m, "absolute_top_impressions"))
                                    w.uint32(80).int64(m.absolute_top_impressions);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShareSimulationPoint message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPoint} message TargetImpressionShareSimulationPoint message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShareSimulationPoint.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetImpressionShareSimulationPoint message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint} TargetImpressionShareSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShareSimulationPoint.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.target_impression_share_micros = r.int64();
                                        break;
                                    case 2:
                                        m.required_cpc_bid_ceiling_micros = r.int64();
                                        break;
                                    case 3:
                                        m.required_budget_amount_micros = r.int64();
                                        break;
                                    case 4:
                                        m.biddable_conversions = r.double();
                                        break;
                                    case 5:
                                        m.biddable_conversions_value = r.double();
                                        break;
                                    case 6:
                                        m.clicks = r.int64();
                                        break;
                                    case 7:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 8:
                                        m.impressions = r.int64();
                                        break;
                                    case 9:
                                        m.top_slot_impressions = r.int64();
                                        break;
                                    case 10:
                                        m.absolute_top_impressions = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetImpressionShareSimulationPoint message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint} TargetImpressionShareSimulationPoint
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShareSimulationPoint.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetImpressionShareSimulationPoint message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetImpressionShareSimulationPoint.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.target_impression_share_micros != null && m.hasOwnProperty("target_impression_share_micros")) {
                                    if (!$util.isInteger(m.target_impression_share_micros) && !(m.target_impression_share_micros && $util.isInteger(m.target_impression_share_micros.low) && $util.isInteger(m.target_impression_share_micros.high)))
                                        return "target_impression_share_micros: integer|Long expected";
                                }
                                if (m.required_cpc_bid_ceiling_micros != null && m.hasOwnProperty("required_cpc_bid_ceiling_micros")) {
                                    if (!$util.isInteger(m.required_cpc_bid_ceiling_micros) && !(m.required_cpc_bid_ceiling_micros && $util.isInteger(m.required_cpc_bid_ceiling_micros.low) && $util.isInteger(m.required_cpc_bid_ceiling_micros.high)))
                                        return "required_cpc_bid_ceiling_micros: integer|Long expected";
                                }
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (!$util.isInteger(m.required_budget_amount_micros) && !(m.required_budget_amount_micros && $util.isInteger(m.required_budget_amount_micros.low) && $util.isInteger(m.required_budget_amount_micros.high)))
                                        return "required_budget_amount_micros: integer|Long expected";
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    if (typeof m.biddable_conversions !== "number")
                                        return "biddable_conversions: number expected";
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    if (typeof m.biddable_conversions_value !== "number")
                                        return "biddable_conversions_value: number expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (!$util.isInteger(m.clicks) && !(m.clicks && $util.isInteger(m.clicks.low) && $util.isInteger(m.clicks.high)))
                                        return "clicks: integer|Long expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (!$util.isInteger(m.impressions) && !(m.impressions && $util.isInteger(m.impressions.low) && $util.isInteger(m.impressions.high)))
                                        return "impressions: integer|Long expected";
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (!$util.isInteger(m.top_slot_impressions) && !(m.top_slot_impressions && $util.isInteger(m.top_slot_impressions.low) && $util.isInteger(m.top_slot_impressions.high)))
                                        return "top_slot_impressions: integer|Long expected";
                                }
                                if (m.absolute_top_impressions != null && m.hasOwnProperty("absolute_top_impressions")) {
                                    if (!$util.isInteger(m.absolute_top_impressions) && !(m.absolute_top_impressions && $util.isInteger(m.absolute_top_impressions.low) && $util.isInteger(m.absolute_top_impressions.high)))
                                        return "absolute_top_impressions: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetImpressionShareSimulationPoint message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint} TargetImpressionShareSimulationPoint
                             */
                            TargetImpressionShareSimulationPoint.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint();
                                if (d.target_impression_share_micros != null) {
                                    if ($util.Long)
                                        (m.target_impression_share_micros = $util.Long.fromValue(d.target_impression_share_micros)).unsigned = false;
                                    else if (typeof d.target_impression_share_micros === "string")
                                        m.target_impression_share_micros = parseInt(d.target_impression_share_micros, 10);
                                    else if (typeof d.target_impression_share_micros === "number")
                                        m.target_impression_share_micros = d.target_impression_share_micros;
                                    else if (typeof d.target_impression_share_micros === "object")
                                        m.target_impression_share_micros = new $util.LongBits(d.target_impression_share_micros.low >>> 0, d.target_impression_share_micros.high >>> 0).toNumber();
                                }
                                if (d.required_cpc_bid_ceiling_micros != null) {
                                    if ($util.Long)
                                        (m.required_cpc_bid_ceiling_micros = $util.Long.fromValue(d.required_cpc_bid_ceiling_micros)).unsigned = false;
                                    else if (typeof d.required_cpc_bid_ceiling_micros === "string")
                                        m.required_cpc_bid_ceiling_micros = parseInt(d.required_cpc_bid_ceiling_micros, 10);
                                    else if (typeof d.required_cpc_bid_ceiling_micros === "number")
                                        m.required_cpc_bid_ceiling_micros = d.required_cpc_bid_ceiling_micros;
                                    else if (typeof d.required_cpc_bid_ceiling_micros === "object")
                                        m.required_cpc_bid_ceiling_micros = new $util.LongBits(d.required_cpc_bid_ceiling_micros.low >>> 0, d.required_cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                }
                                if (d.required_budget_amount_micros != null) {
                                    if ($util.Long)
                                        (m.required_budget_amount_micros = $util.Long.fromValue(d.required_budget_amount_micros)).unsigned = false;
                                    else if (typeof d.required_budget_amount_micros === "string")
                                        m.required_budget_amount_micros = parseInt(d.required_budget_amount_micros, 10);
                                    else if (typeof d.required_budget_amount_micros === "number")
                                        m.required_budget_amount_micros = d.required_budget_amount_micros;
                                    else if (typeof d.required_budget_amount_micros === "object")
                                        m.required_budget_amount_micros = new $util.LongBits(d.required_budget_amount_micros.low >>> 0, d.required_budget_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.biddable_conversions != null) {
                                    m.biddable_conversions = Number(d.biddable_conversions);
                                }
                                if (d.biddable_conversions_value != null) {
                                    m.biddable_conversions_value = Number(d.biddable_conversions_value);
                                }
                                if (d.clicks != null) {
                                    if ($util.Long)
                                        (m.clicks = $util.Long.fromValue(d.clicks)).unsigned = false;
                                    else if (typeof d.clicks === "string")
                                        m.clicks = parseInt(d.clicks, 10);
                                    else if (typeof d.clicks === "number")
                                        m.clicks = d.clicks;
                                    else if (typeof d.clicks === "object")
                                        m.clicks = new $util.LongBits(d.clicks.low >>> 0, d.clicks.high >>> 0).toNumber();
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.impressions != null) {
                                    if ($util.Long)
                                        (m.impressions = $util.Long.fromValue(d.impressions)).unsigned = false;
                                    else if (typeof d.impressions === "string")
                                        m.impressions = parseInt(d.impressions, 10);
                                    else if (typeof d.impressions === "number")
                                        m.impressions = d.impressions;
                                    else if (typeof d.impressions === "object")
                                        m.impressions = new $util.LongBits(d.impressions.low >>> 0, d.impressions.high >>> 0).toNumber();
                                }
                                if (d.top_slot_impressions != null) {
                                    if ($util.Long)
                                        (m.top_slot_impressions = $util.Long.fromValue(d.top_slot_impressions)).unsigned = false;
                                    else if (typeof d.top_slot_impressions === "string")
                                        m.top_slot_impressions = parseInt(d.top_slot_impressions, 10);
                                    else if (typeof d.top_slot_impressions === "number")
                                        m.top_slot_impressions = d.top_slot_impressions;
                                    else if (typeof d.top_slot_impressions === "object")
                                        m.top_slot_impressions = new $util.LongBits(d.top_slot_impressions.low >>> 0, d.top_slot_impressions.high >>> 0).toNumber();
                                }
                                if (d.absolute_top_impressions != null) {
                                    if ($util.Long)
                                        (m.absolute_top_impressions = $util.Long.fromValue(d.absolute_top_impressions)).unsigned = false;
                                    else if (typeof d.absolute_top_impressions === "string")
                                        m.absolute_top_impressions = parseInt(d.absolute_top_impressions, 10);
                                    else if (typeof d.absolute_top_impressions === "number")
                                        m.absolute_top_impressions = d.absolute_top_impressions;
                                    else if (typeof d.absolute_top_impressions === "object")
                                        m.absolute_top_impressions = new $util.LongBits(d.absolute_top_impressions.low >>> 0, d.absolute_top_impressions.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetImpressionShareSimulationPoint message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint} m TargetImpressionShareSimulationPoint
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetImpressionShareSimulationPoint.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.target_impression_share_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.target_impression_share_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.required_budget_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.required_budget_amount_micros = o.longs === String ? "0" : 0;
                                    d.biddable_conversions = 0;
                                    d.biddable_conversions_value = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.clicks = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.clicks = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cost_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cost_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.impressions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.impressions = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.top_slot_impressions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.top_slot_impressions = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.absolute_top_impressions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.absolute_top_impressions = o.longs === String ? "0" : 0;
                                }
                                if (m.target_impression_share_micros != null && m.hasOwnProperty("target_impression_share_micros")) {
                                    if (typeof m.target_impression_share_micros === "number")
                                        d.target_impression_share_micros = o.longs === String ? String(m.target_impression_share_micros) : m.target_impression_share_micros;
                                    else
                                        d.target_impression_share_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_impression_share_micros) : o.longs === Number ? new $util.LongBits(m.target_impression_share_micros.low >>> 0, m.target_impression_share_micros.high >>> 0).toNumber() : m.target_impression_share_micros;
                                }
                                if (m.required_cpc_bid_ceiling_micros != null && m.hasOwnProperty("required_cpc_bid_ceiling_micros")) {
                                    if (typeof m.required_cpc_bid_ceiling_micros === "number")
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? String(m.required_cpc_bid_ceiling_micros) : m.required_cpc_bid_ceiling_micros;
                                    else
                                        d.required_cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.required_cpc_bid_ceiling_micros.low >>> 0, m.required_cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.required_cpc_bid_ceiling_micros;
                                }
                                if (m.required_budget_amount_micros != null && m.hasOwnProperty("required_budget_amount_micros")) {
                                    if (typeof m.required_budget_amount_micros === "number")
                                        d.required_budget_amount_micros = o.longs === String ? String(m.required_budget_amount_micros) : m.required_budget_amount_micros;
                                    else
                                        d.required_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.required_budget_amount_micros.low >>> 0, m.required_budget_amount_micros.high >>> 0).toNumber() : m.required_budget_amount_micros;
                                }
                                if (m.biddable_conversions != null && m.hasOwnProperty("biddable_conversions")) {
                                    d.biddable_conversions = o.json && !isFinite(m.biddable_conversions) ? String(m.biddable_conversions) : m.biddable_conversions;
                                }
                                if (m.biddable_conversions_value != null && m.hasOwnProperty("biddable_conversions_value")) {
                                    d.biddable_conversions_value = o.json && !isFinite(m.biddable_conversions_value) ? String(m.biddable_conversions_value) : m.biddable_conversions_value;
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    if (typeof m.clicks === "number")
                                        d.clicks = o.longs === String ? String(m.clicks) : m.clicks;
                                    else
                                        d.clicks = o.longs === String ? $util.Long.prototype.toString.call(m.clicks) : o.longs === Number ? new $util.LongBits(m.clicks.low >>> 0, m.clicks.high >>> 0).toNumber() : m.clicks;
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (typeof m.impressions === "number")
                                        d.impressions = o.longs === String ? String(m.impressions) : m.impressions;
                                    else
                                        d.impressions = o.longs === String ? $util.Long.prototype.toString.call(m.impressions) : o.longs === Number ? new $util.LongBits(m.impressions.low >>> 0, m.impressions.high >>> 0).toNumber() : m.impressions;
                                }
                                if (m.top_slot_impressions != null && m.hasOwnProperty("top_slot_impressions")) {
                                    if (typeof m.top_slot_impressions === "number")
                                        d.top_slot_impressions = o.longs === String ? String(m.top_slot_impressions) : m.top_slot_impressions;
                                    else
                                        d.top_slot_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.top_slot_impressions) : o.longs === Number ? new $util.LongBits(m.top_slot_impressions.low >>> 0, m.top_slot_impressions.high >>> 0).toNumber() : m.top_slot_impressions;
                                }
                                if (m.absolute_top_impressions != null && m.hasOwnProperty("absolute_top_impressions")) {
                                    if (typeof m.absolute_top_impressions === "number")
                                        d.absolute_top_impressions = o.longs === String ? String(m.absolute_top_impressions) : m.absolute_top_impressions;
                                    else
                                        d.absolute_top_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.absolute_top_impressions) : o.longs === Number ? new $util.LongBits(m.absolute_top_impressions.low >>> 0, m.absolute_top_impressions.high >>> 0).toNumber() : m.absolute_top_impressions;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetImpressionShareSimulationPoint to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetImpressionShareSimulationPoint
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetImpressionShareSimulationPoint.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetImpressionShareSimulationPoint;
                        })();
    
                        common.TagSnippet = (function() {
    
                            /**
                             * Properties of a TagSnippet.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITagSnippet
                             * @property {google.ads.googleads.v11.enums.TrackingCodeTypeEnum.TrackingCodeType|null} [type] TagSnippet type
                             * @property {google.ads.googleads.v11.enums.TrackingCodePageFormatEnum.TrackingCodePageFormat|null} [page_format] TagSnippet page_format
                             * @property {string|null} [global_site_tag] TagSnippet global_site_tag
                             * @property {string|null} [event_snippet] TagSnippet event_snippet
                             */
    
                            /**
                             * Constructs a new TagSnippet.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TagSnippet.
                             * @implements ITagSnippet
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITagSnippet=} [p] Properties to set
                             */
                            function TagSnippet(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TagSnippet type.
                             * @member {google.ads.googleads.v11.enums.TrackingCodeTypeEnum.TrackingCodeType} type
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @instance
                             */
                            TagSnippet.prototype.type = 0;
    
                            /**
                             * TagSnippet page_format.
                             * @member {google.ads.googleads.v11.enums.TrackingCodePageFormatEnum.TrackingCodePageFormat} page_format
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @instance
                             */
                            TagSnippet.prototype.page_format = 0;
    
                            /**
                             * TagSnippet global_site_tag.
                             * @member {string|null|undefined} global_site_tag
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @instance
                             */
                            TagSnippet.prototype.global_site_tag = null;
    
                            /**
                             * TagSnippet event_snippet.
                             * @member {string|null|undefined} event_snippet
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @instance
                             */
                            TagSnippet.prototype.event_snippet = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TagSnippet _global_site_tag.
                             * @member {"global_site_tag"|undefined} _global_site_tag
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @instance
                             */
                            Object.defineProperty(TagSnippet.prototype, "_global_site_tag", {
                                get: $util.oneOfGetter($oneOfFields = ["global_site_tag"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TagSnippet _event_snippet.
                             * @member {"event_snippet"|undefined} _event_snippet
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @instance
                             */
                            Object.defineProperty(TagSnippet.prototype, "_event_snippet", {
                                get: $util.oneOfGetter($oneOfFields = ["event_snippet"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TagSnippet instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {google.ads.googleads.v11.common.ITagSnippet=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TagSnippet} TagSnippet instance
                             */
                            TagSnippet.create = function create(properties) {
                                return new TagSnippet(properties);
                            };
    
                            /**
                             * Encodes the specified TagSnippet message. Does not implicitly {@link google.ads.googleads.v11.common.TagSnippet.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {google.ads.googleads.v11.common.ITagSnippet} m TagSnippet message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TagSnippet.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(8).int32(m.type);
                                if (m.page_format != null && Object.hasOwnProperty.call(m, "page_format"))
                                    w.uint32(16).int32(m.page_format);
                                if (m.global_site_tag != null && Object.hasOwnProperty.call(m, "global_site_tag"))
                                    w.uint32(42).string(m.global_site_tag);
                                if (m.event_snippet != null && Object.hasOwnProperty.call(m, "event_snippet"))
                                    w.uint32(50).string(m.event_snippet);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TagSnippet message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TagSnippet.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {google.ads.googleads.v11.common.ITagSnippet} message TagSnippet message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TagSnippet.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TagSnippet message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TagSnippet} TagSnippet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TagSnippet.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TagSnippet();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.type = r.int32();
                                        break;
                                    case 2:
                                        m.page_format = r.int32();
                                        break;
                                    case 5:
                                        m.global_site_tag = r.string();
                                        break;
                                    case 6:
                                        m.event_snippet = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TagSnippet message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TagSnippet} TagSnippet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TagSnippet.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TagSnippet message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TagSnippet.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.page_format != null && m.hasOwnProperty("page_format")) {
                                    switch (m.page_format) {
                                    default:
                                        return "page_format: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.global_site_tag != null && m.hasOwnProperty("global_site_tag")) {
                                    p._global_site_tag = 1;
                                    if (!$util.isString(m.global_site_tag))
                                        return "global_site_tag: string expected";
                                }
                                if (m.event_snippet != null && m.hasOwnProperty("event_snippet")) {
                                    p._event_snippet = 1;
                                    if (!$util.isString(m.event_snippet))
                                        return "event_snippet: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TagSnippet message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TagSnippet} TagSnippet
                             */
                            TagSnippet.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TagSnippet)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TagSnippet();
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "WEBPAGE":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "WEBPAGE_ONCLICK":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "CLICK_TO_CALL":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "WEBSITE_CALL":
                                case 5:
                                    m.type = 5;
                                    break;
                                }
                                switch (d.page_format) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.page_format = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.page_format = 1;
                                    break;
                                case "HTML":
                                case 2:
                                    m.page_format = 2;
                                    break;
                                case "AMP":
                                case 3:
                                    m.page_format = 3;
                                    break;
                                }
                                if (d.global_site_tag != null) {
                                    m.global_site_tag = String(d.global_site_tag);
                                }
                                if (d.event_snippet != null) {
                                    m.event_snippet = String(d.event_snippet);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TagSnippet message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @static
                             * @param {google.ads.googleads.v11.common.TagSnippet} m TagSnippet
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TagSnippet.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.page_format = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.TrackingCodeTypeEnum.TrackingCodeType[m.type] : m.type;
                                }
                                if (m.page_format != null && m.hasOwnProperty("page_format")) {
                                    d.page_format = o.enums === String ? $root.google.ads.googleads.v11.enums.TrackingCodePageFormatEnum.TrackingCodePageFormat[m.page_format] : m.page_format;
                                }
                                if (m.global_site_tag != null && m.hasOwnProperty("global_site_tag")) {
                                    d.global_site_tag = m.global_site_tag;
                                    if (o.oneofs)
                                        d._global_site_tag = "global_site_tag";
                                }
                                if (m.event_snippet != null && m.hasOwnProperty("event_snippet")) {
                                    d.event_snippet = m.event_snippet;
                                    if (o.oneofs)
                                        d._event_snippet = "event_snippet";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TagSnippet to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TagSnippet
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TagSnippet.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TagSnippet;
                        })();
    
                        common.TargetingSetting = (function() {
    
                            /**
                             * Properties of a TargetingSetting.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetingSetting
                             * @property {Array.<google.ads.googleads.v11.common.ITargetRestriction>|null} [target_restrictions] TargetingSetting target_restrictions
                             * @property {Array.<google.ads.googleads.v11.common.ITargetRestrictionOperation>|null} [target_restriction_operations] TargetingSetting target_restriction_operations
                             */
    
                            /**
                             * Constructs a new TargetingSetting.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetingSetting.
                             * @implements ITargetingSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetingSetting=} [p] Properties to set
                             */
                            function TargetingSetting(p) {
                                this.target_restrictions = [];
                                this.target_restriction_operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetingSetting target_restrictions.
                             * @member {Array.<google.ads.googleads.v11.common.ITargetRestriction>} target_restrictions
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @instance
                             */
                            TargetingSetting.prototype.target_restrictions = $util.emptyArray;
    
                            /**
                             * TargetingSetting target_restriction_operations.
                             * @member {Array.<google.ads.googleads.v11.common.ITargetRestrictionOperation>} target_restriction_operations
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @instance
                             */
                            TargetingSetting.prototype.target_restriction_operations = $util.emptyArray;
    
                            /**
                             * Creates a new TargetingSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetingSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetingSetting} TargetingSetting instance
                             */
                            TargetingSetting.create = function create(properties) {
                                return new TargetingSetting(properties);
                            };
    
                            /**
                             * Encodes the specified TargetingSetting message. Does not implicitly {@link google.ads.googleads.v11.common.TargetingSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetingSetting} m TargetingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetingSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.target_restrictions != null && m.target_restrictions.length) {
                                    for (var i = 0; i < m.target_restrictions.length; ++i)
                                        $root.google.ads.googleads.v11.common.TargetRestriction.encode(m.target_restrictions[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.target_restriction_operations != null && m.target_restriction_operations.length) {
                                    for (var i = 0; i < m.target_restriction_operations.length; ++i)
                                        $root.google.ads.googleads.v11.common.TargetRestrictionOperation.encode(m.target_restriction_operations[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetingSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetingSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetingSetting} message TargetingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetingSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetingSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetingSetting} TargetingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetingSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetingSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.target_restrictions && m.target_restrictions.length))
                                            m.target_restrictions = [];
                                        m.target_restrictions.push($root.google.ads.googleads.v11.common.TargetRestriction.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.target_restriction_operations && m.target_restriction_operations.length))
                                            m.target_restriction_operations = [];
                                        m.target_restriction_operations.push($root.google.ads.googleads.v11.common.TargetRestrictionOperation.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetingSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetingSetting} TargetingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetingSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetingSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetingSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.target_restrictions != null && m.hasOwnProperty("target_restrictions")) {
                                    if (!Array.isArray(m.target_restrictions))
                                        return "target_restrictions: array expected";
                                    for (var i = 0; i < m.target_restrictions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.TargetRestriction.verify(m.target_restrictions[i]);
                                            if (e)
                                                return "target_restrictions." + e;
                                        }
                                    }
                                }
                                if (m.target_restriction_operations != null && m.hasOwnProperty("target_restriction_operations")) {
                                    if (!Array.isArray(m.target_restriction_operations))
                                        return "target_restriction_operations: array expected";
                                    for (var i = 0; i < m.target_restriction_operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.TargetRestrictionOperation.verify(m.target_restriction_operations[i]);
                                            if (e)
                                                return "target_restriction_operations." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetingSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetingSetting} TargetingSetting
                             */
                            TargetingSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetingSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetingSetting();
                                if (d.target_restrictions) {
                                    if (!Array.isArray(d.target_restrictions))
                                        throw TypeError(".google.ads.googleads.v11.common.TargetingSetting.target_restrictions: array expected");
                                    m.target_restrictions = [];
                                    for (var i = 0; i < d.target_restrictions.length; ++i) {
                                        if (typeof d.target_restrictions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.TargetingSetting.target_restrictions: object expected");
                                        m.target_restrictions[i] = $root.google.ads.googleads.v11.common.TargetRestriction.fromObject(d.target_restrictions[i]);
                                    }
                                }
                                if (d.target_restriction_operations) {
                                    if (!Array.isArray(d.target_restriction_operations))
                                        throw TypeError(".google.ads.googleads.v11.common.TargetingSetting.target_restriction_operations: array expected");
                                    m.target_restriction_operations = [];
                                    for (var i = 0; i < d.target_restriction_operations.length; ++i) {
                                        if (typeof d.target_restriction_operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.TargetingSetting.target_restriction_operations: object expected");
                                        m.target_restriction_operations[i] = $root.google.ads.googleads.v11.common.TargetRestrictionOperation.fromObject(d.target_restriction_operations[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetingSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetingSetting} m TargetingSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetingSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.target_restrictions = [];
                                    d.target_restriction_operations = [];
                                }
                                if (m.target_restrictions && m.target_restrictions.length) {
                                    d.target_restrictions = [];
                                    for (var j = 0; j < m.target_restrictions.length; ++j) {
                                        d.target_restrictions[j] = $root.google.ads.googleads.v11.common.TargetRestriction.toObject(m.target_restrictions[j], o);
                                    }
                                }
                                if (m.target_restriction_operations && m.target_restriction_operations.length) {
                                    d.target_restriction_operations = [];
                                    for (var j = 0; j < m.target_restriction_operations.length; ++j) {
                                        d.target_restriction_operations[j] = $root.google.ads.googleads.v11.common.TargetRestrictionOperation.toObject(m.target_restriction_operations[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetingSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetingSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetingSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetingSetting;
                        })();
    
                        common.TargetRestriction = (function() {
    
                            /**
                             * Properties of a TargetRestriction.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetRestriction
                             * @property {google.ads.googleads.v11.enums.TargetingDimensionEnum.TargetingDimension|null} [targeting_dimension] TargetRestriction targeting_dimension
                             * @property {boolean|null} [bid_only] TargetRestriction bid_only
                             */
    
                            /**
                             * Constructs a new TargetRestriction.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetRestriction.
                             * @implements ITargetRestriction
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetRestriction=} [p] Properties to set
                             */
                            function TargetRestriction(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetRestriction targeting_dimension.
                             * @member {google.ads.googleads.v11.enums.TargetingDimensionEnum.TargetingDimension} targeting_dimension
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @instance
                             */
                            TargetRestriction.prototype.targeting_dimension = 0;
    
                            /**
                             * TargetRestriction bid_only.
                             * @member {boolean|null|undefined} bid_only
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @instance
                             */
                            TargetRestriction.prototype.bid_only = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TargetRestriction _bid_only.
                             * @member {"bid_only"|undefined} _bid_only
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @instance
                             */
                            Object.defineProperty(TargetRestriction.prototype, "_bid_only", {
                                get: $util.oneOfGetter($oneOfFields = ["bid_only"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TargetRestriction instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRestriction=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetRestriction} TargetRestriction instance
                             */
                            TargetRestriction.create = function create(properties) {
                                return new TargetRestriction(properties);
                            };
    
                            /**
                             * Encodes the specified TargetRestriction message. Does not implicitly {@link google.ads.googleads.v11.common.TargetRestriction.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRestriction} m TargetRestriction message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRestriction.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.targeting_dimension != null && Object.hasOwnProperty.call(m, "targeting_dimension"))
                                    w.uint32(8).int32(m.targeting_dimension);
                                if (m.bid_only != null && Object.hasOwnProperty.call(m, "bid_only"))
                                    w.uint32(24).bool(m.bid_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetRestriction message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetRestriction.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRestriction} message TargetRestriction message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRestriction.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetRestriction message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetRestriction} TargetRestriction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRestriction.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetRestriction();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.targeting_dimension = r.int32();
                                        break;
                                    case 3:
                                        m.bid_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetRestriction message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetRestriction} TargetRestriction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRestriction.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetRestriction message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetRestriction.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.targeting_dimension != null && m.hasOwnProperty("targeting_dimension")) {
                                    switch (m.targeting_dimension) {
                                    default:
                                        return "targeting_dimension: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.bid_only != null && m.hasOwnProperty("bid_only")) {
                                    p._bid_only = 1;
                                    if (typeof m.bid_only !== "boolean")
                                        return "bid_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetRestriction message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetRestriction} TargetRestriction
                             */
                            TargetRestriction.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetRestriction)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetRestriction();
                                switch (d.targeting_dimension) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.targeting_dimension = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.targeting_dimension = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.targeting_dimension = 2;
                                    break;
                                case "AUDIENCE":
                                case 3:
                                    m.targeting_dimension = 3;
                                    break;
                                case "TOPIC":
                                case 4:
                                    m.targeting_dimension = 4;
                                    break;
                                case "GENDER":
                                case 5:
                                    m.targeting_dimension = 5;
                                    break;
                                case "AGE_RANGE":
                                case 6:
                                    m.targeting_dimension = 6;
                                    break;
                                case "PLACEMENT":
                                case 7:
                                    m.targeting_dimension = 7;
                                    break;
                                case "PARENTAL_STATUS":
                                case 8:
                                    m.targeting_dimension = 8;
                                    break;
                                case "INCOME_RANGE":
                                case 9:
                                    m.targeting_dimension = 9;
                                    break;
                                }
                                if (d.bid_only != null) {
                                    m.bid_only = Boolean(d.bid_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetRestriction message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetRestriction} m TargetRestriction
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetRestriction.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.targeting_dimension = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.targeting_dimension != null && m.hasOwnProperty("targeting_dimension")) {
                                    d.targeting_dimension = o.enums === String ? $root.google.ads.googleads.v11.enums.TargetingDimensionEnum.TargetingDimension[m.targeting_dimension] : m.targeting_dimension;
                                }
                                if (m.bid_only != null && m.hasOwnProperty("bid_only")) {
                                    d.bid_only = m.bid_only;
                                    if (o.oneofs)
                                        d._bid_only = "bid_only";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetRestriction to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetRestriction
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetRestriction.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TargetRestriction;
                        })();
    
                        common.TargetRestrictionOperation = (function() {
    
                            /**
                             * Properties of a TargetRestrictionOperation.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITargetRestrictionOperation
                             * @property {google.ads.googleads.v11.common.TargetRestrictionOperation.Operator|null} [operator] TargetRestrictionOperation operator
                             * @property {google.ads.googleads.v11.common.ITargetRestriction|null} [value] TargetRestrictionOperation value
                             */
    
                            /**
                             * Constructs a new TargetRestrictionOperation.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TargetRestrictionOperation.
                             * @implements ITargetRestrictionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITargetRestrictionOperation=} [p] Properties to set
                             */
                            function TargetRestrictionOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TargetRestrictionOperation operator.
                             * @member {google.ads.googleads.v11.common.TargetRestrictionOperation.Operator} operator
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @instance
                             */
                            TargetRestrictionOperation.prototype.operator = 0;
    
                            /**
                             * TargetRestrictionOperation value.
                             * @member {google.ads.googleads.v11.common.ITargetRestriction|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @instance
                             */
                            TargetRestrictionOperation.prototype.value = null;
    
                            /**
                             * Creates a new TargetRestrictionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRestrictionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TargetRestrictionOperation} TargetRestrictionOperation instance
                             */
                            TargetRestrictionOperation.create = function create(properties) {
                                return new TargetRestrictionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified TargetRestrictionOperation message. Does not implicitly {@link google.ads.googleads.v11.common.TargetRestrictionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRestrictionOperation} m TargetRestrictionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRestrictionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(8).int32(m.operator);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    $root.google.ads.googleads.v11.common.TargetRestriction.encode(m.value, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetRestrictionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TargetRestrictionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {google.ads.googleads.v11.common.ITargetRestrictionOperation} message TargetRestrictionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetRestrictionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetRestrictionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TargetRestrictionOperation} TargetRestrictionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRestrictionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TargetRestrictionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operator = r.int32();
                                        break;
                                    case 2:
                                        m.value = $root.google.ads.googleads.v11.common.TargetRestriction.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetRestrictionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TargetRestrictionOperation} TargetRestrictionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetRestrictionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetRestrictionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetRestrictionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetRestriction.verify(m.value);
                                        if (e)
                                            return "value." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TargetRestrictionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TargetRestrictionOperation} TargetRestrictionOperation
                             */
                            TargetRestrictionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TargetRestrictionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TargetRestrictionOperation();
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "ADD":
                                case 2:
                                    m.operator = 2;
                                    break;
                                case "REMOVE":
                                case 3:
                                    m.operator = 3;
                                    break;
                                }
                                if (d.value != null) {
                                    if (typeof d.value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.TargetRestrictionOperation.value: object expected");
                                    m.value = $root.google.ads.googleads.v11.common.TargetRestriction.fromObject(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TargetRestrictionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @static
                             * @param {google.ads.googleads.v11.common.TargetRestrictionOperation} m TargetRestrictionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetRestrictionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.value = null;
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.common.TargetRestrictionOperation.Operator[m.operator] : m.operator;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = $root.google.ads.googleads.v11.common.TargetRestriction.toObject(m.value, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TargetRestrictionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TargetRestrictionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetRestrictionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Operator enum.
                             * @name google.ads.googleads.v11.common.TargetRestrictionOperation.Operator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADD=2 ADD value
                             * @property {number} REMOVE=3 REMOVE value
                             */
                            TargetRestrictionOperation.Operator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADD"] = 2;
                                values[valuesById[3] = "REMOVE"] = 3;
                                return values;
                            })();
    
                            return TargetRestrictionOperation;
                        })();
    
                        common.TextLabel = (function() {
    
                            /**
                             * Properties of a TextLabel.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ITextLabel
                             * @property {string|null} [background_color] TextLabel background_color
                             * @property {string|null} [description] TextLabel description
                             */
    
                            /**
                             * Constructs a new TextLabel.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a TextLabel.
                             * @implements ITextLabel
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ITextLabel=} [p] Properties to set
                             */
                            function TextLabel(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TextLabel background_color.
                             * @member {string|null|undefined} background_color
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @instance
                             */
                            TextLabel.prototype.background_color = null;
    
                            /**
                             * TextLabel description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @instance
                             */
                            TextLabel.prototype.description = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TextLabel _background_color.
                             * @member {"background_color"|undefined} _background_color
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @instance
                             */
                            Object.defineProperty(TextLabel.prototype, "_background_color", {
                                get: $util.oneOfGetter($oneOfFields = ["background_color"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TextLabel _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @instance
                             */
                            Object.defineProperty(TextLabel.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TextLabel instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextLabel=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.TextLabel} TextLabel instance
                             */
                            TextLabel.create = function create(properties) {
                                return new TextLabel(properties);
                            };
    
                            /**
                             * Encodes the specified TextLabel message. Does not implicitly {@link google.ads.googleads.v11.common.TextLabel.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextLabel} m TextLabel message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextLabel.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.background_color != null && Object.hasOwnProperty.call(m, "background_color"))
                                    w.uint32(26).string(m.background_color);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(34).string(m.description);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TextLabel message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.TextLabel.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {google.ads.googleads.v11.common.ITextLabel} message TextLabel message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TextLabel.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TextLabel message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.TextLabel} TextLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextLabel.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.TextLabel();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.background_color = r.string();
                                        break;
                                    case 4:
                                        m.description = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TextLabel message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.TextLabel} TextLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TextLabel.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TextLabel message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TextLabel.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.background_color != null && m.hasOwnProperty("background_color")) {
                                    p._background_color = 1;
                                    if (!$util.isString(m.background_color))
                                        return "background_color: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TextLabel message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.TextLabel} TextLabel
                             */
                            TextLabel.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.TextLabel)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.TextLabel();
                                if (d.background_color != null) {
                                    m.background_color = String(d.background_color);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TextLabel message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @static
                             * @param {google.ads.googleads.v11.common.TextLabel} m TextLabel
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TextLabel.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.background_color != null && m.hasOwnProperty("background_color")) {
                                    d.background_color = m.background_color;
                                    if (o.oneofs)
                                        d._background_color = "background_color";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TextLabel to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.TextLabel
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TextLabel.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TextLabel;
                        })();
    
                        common.UrlCollection = (function() {
    
                            /**
                             * Properties of an UrlCollection.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUrlCollection
                             * @property {string|null} [url_collection_id] UrlCollection url_collection_id
                             * @property {Array.<string>|null} [final_urls] UrlCollection final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] UrlCollection final_mobile_urls
                             * @property {string|null} [tracking_url_template] UrlCollection tracking_url_template
                             */
    
                            /**
                             * Constructs a new UrlCollection.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an UrlCollection.
                             * @implements IUrlCollection
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUrlCollection=} [p] Properties to set
                             */
                            function UrlCollection(p) {
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UrlCollection url_collection_id.
                             * @member {string|null|undefined} url_collection_id
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @instance
                             */
                            UrlCollection.prototype.url_collection_id = null;
    
                            /**
                             * UrlCollection final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @instance
                             */
                            UrlCollection.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * UrlCollection final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @instance
                             */
                            UrlCollection.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * UrlCollection tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @instance
                             */
                            UrlCollection.prototype.tracking_url_template = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UrlCollection _url_collection_id.
                             * @member {"url_collection_id"|undefined} _url_collection_id
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @instance
                             */
                            Object.defineProperty(UrlCollection.prototype, "_url_collection_id", {
                                get: $util.oneOfGetter($oneOfFields = ["url_collection_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UrlCollection _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @instance
                             */
                            Object.defineProperty(UrlCollection.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UrlCollection instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {google.ads.googleads.v11.common.IUrlCollection=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UrlCollection} UrlCollection instance
                             */
                            UrlCollection.create = function create(properties) {
                                return new UrlCollection(properties);
                            };
    
                            /**
                             * Encodes the specified UrlCollection message. Does not implicitly {@link google.ads.googleads.v11.common.UrlCollection.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {google.ads.googleads.v11.common.IUrlCollection} m UrlCollection message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UrlCollection.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.url_collection_id != null && Object.hasOwnProperty.call(m, "url_collection_id"))
                                    w.uint32(42).string(m.url_collection_id);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(50).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(58).string(m.final_mobile_urls[i]);
                                }
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(66).string(m.tracking_url_template);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UrlCollection message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UrlCollection.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {google.ads.googleads.v11.common.IUrlCollection} message UrlCollection message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UrlCollection.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UrlCollection message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UrlCollection} UrlCollection
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UrlCollection.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UrlCollection();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.url_collection_id = r.string();
                                        break;
                                    case 6:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 7:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 8:
                                        m.tracking_url_template = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UrlCollection message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UrlCollection} UrlCollection
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UrlCollection.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UrlCollection message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UrlCollection.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.url_collection_id != null && m.hasOwnProperty("url_collection_id")) {
                                    p._url_collection_id = 1;
                                    if (!$util.isString(m.url_collection_id))
                                        return "url_collection_id: string expected";
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UrlCollection message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UrlCollection} UrlCollection
                             */
                            UrlCollection.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UrlCollection)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UrlCollection();
                                if (d.url_collection_id != null) {
                                    m.url_collection_id = String(d.url_collection_id);
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.UrlCollection.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.common.UrlCollection.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UrlCollection message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @static
                             * @param {google.ads.googleads.v11.common.UrlCollection} m UrlCollection
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UrlCollection.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (m.url_collection_id != null && m.hasOwnProperty("url_collection_id")) {
                                    d.url_collection_id = m.url_collection_id;
                                    if (o.oneofs)
                                        d._url_collection_id = "url_collection_id";
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UrlCollection to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UrlCollection
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UrlCollection.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UrlCollection;
                        })();
    
                        common.SimilarUserListInfo = (function() {
    
                            /**
                             * Properties of a SimilarUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ISimilarUserListInfo
                             * @property {string|null} [seed_user_list] SimilarUserListInfo seed_user_list
                             */
    
                            /**
                             * Constructs a new SimilarUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a SimilarUserListInfo.
                             * @implements ISimilarUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ISimilarUserListInfo=} [p] Properties to set
                             */
                            function SimilarUserListInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SimilarUserListInfo seed_user_list.
                             * @member {string|null|undefined} seed_user_list
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @instance
                             */
                            SimilarUserListInfo.prototype.seed_user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SimilarUserListInfo _seed_user_list.
                             * @member {"seed_user_list"|undefined} _seed_user_list
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @instance
                             */
                            Object.defineProperty(SimilarUserListInfo.prototype, "_seed_user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["seed_user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SimilarUserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ISimilarUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.SimilarUserListInfo} SimilarUserListInfo instance
                             */
                            SimilarUserListInfo.create = function create(properties) {
                                return new SimilarUserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified SimilarUserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.SimilarUserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ISimilarUserListInfo} m SimilarUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SimilarUserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.seed_user_list != null && Object.hasOwnProperty.call(m, "seed_user_list"))
                                    w.uint32(18).string(m.seed_user_list);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SimilarUserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.SimilarUserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ISimilarUserListInfo} message SimilarUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SimilarUserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SimilarUserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.SimilarUserListInfo} SimilarUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SimilarUserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.SimilarUserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.seed_user_list = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SimilarUserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.SimilarUserListInfo} SimilarUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SimilarUserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SimilarUserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SimilarUserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.seed_user_list != null && m.hasOwnProperty("seed_user_list")) {
                                    p._seed_user_list = 1;
                                    if (!$util.isString(m.seed_user_list))
                                        return "seed_user_list: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SimilarUserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.SimilarUserListInfo} SimilarUserListInfo
                             */
                            SimilarUserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.SimilarUserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.SimilarUserListInfo();
                                if (d.seed_user_list != null) {
                                    m.seed_user_list = String(d.seed_user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SimilarUserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.SimilarUserListInfo} m SimilarUserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SimilarUserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.seed_user_list != null && m.hasOwnProperty("seed_user_list")) {
                                    d.seed_user_list = m.seed_user_list;
                                    if (o.oneofs)
                                        d._seed_user_list = "seed_user_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SimilarUserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.SimilarUserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SimilarUserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SimilarUserListInfo;
                        })();
    
                        common.CrmBasedUserListInfo = (function() {
    
                            /**
                             * Properties of a CrmBasedUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICrmBasedUserListInfo
                             * @property {string|null} [app_id] CrmBasedUserListInfo app_id
                             * @property {google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType|null} [upload_key_type] CrmBasedUserListInfo upload_key_type
                             * @property {google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType|null} [data_source_type] CrmBasedUserListInfo data_source_type
                             */
    
                            /**
                             * Constructs a new CrmBasedUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CrmBasedUserListInfo.
                             * @implements ICrmBasedUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICrmBasedUserListInfo=} [p] Properties to set
                             */
                            function CrmBasedUserListInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CrmBasedUserListInfo app_id.
                             * @member {string|null|undefined} app_id
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @instance
                             */
                            CrmBasedUserListInfo.prototype.app_id = null;
    
                            /**
                             * CrmBasedUserListInfo upload_key_type.
                             * @member {google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType} upload_key_type
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @instance
                             */
                            CrmBasedUserListInfo.prototype.upload_key_type = 0;
    
                            /**
                             * CrmBasedUserListInfo data_source_type.
                             * @member {google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType} data_source_type
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @instance
                             */
                            CrmBasedUserListInfo.prototype.data_source_type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CrmBasedUserListInfo _app_id.
                             * @member {"app_id"|undefined} _app_id
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @instance
                             */
                            Object.defineProperty(CrmBasedUserListInfo.prototype, "_app_id", {
                                get: $util.oneOfGetter($oneOfFields = ["app_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CrmBasedUserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICrmBasedUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CrmBasedUserListInfo} CrmBasedUserListInfo instance
                             */
                            CrmBasedUserListInfo.create = function create(properties) {
                                return new CrmBasedUserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CrmBasedUserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CrmBasedUserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICrmBasedUserListInfo} m CrmBasedUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CrmBasedUserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.upload_key_type != null && Object.hasOwnProperty.call(m, "upload_key_type"))
                                    w.uint32(16).int32(m.upload_key_type);
                                if (m.data_source_type != null && Object.hasOwnProperty.call(m, "data_source_type"))
                                    w.uint32(24).int32(m.data_source_type);
                                if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                    w.uint32(34).string(m.app_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CrmBasedUserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CrmBasedUserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICrmBasedUserListInfo} message CrmBasedUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CrmBasedUserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CrmBasedUserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CrmBasedUserListInfo} CrmBasedUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CrmBasedUserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CrmBasedUserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.app_id = r.string();
                                        break;
                                    case 2:
                                        m.upload_key_type = r.int32();
                                        break;
                                    case 3:
                                        m.data_source_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CrmBasedUserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CrmBasedUserListInfo} CrmBasedUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CrmBasedUserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CrmBasedUserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CrmBasedUserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    p._app_id = 1;
                                    if (!$util.isString(m.app_id))
                                        return "app_id: string expected";
                                }
                                if (m.upload_key_type != null && m.hasOwnProperty("upload_key_type")) {
                                    switch (m.upload_key_type) {
                                    default:
                                        return "upload_key_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.data_source_type != null && m.hasOwnProperty("data_source_type")) {
                                    switch (m.data_source_type) {
                                    default:
                                        return "data_source_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CrmBasedUserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CrmBasedUserListInfo} CrmBasedUserListInfo
                             */
                            CrmBasedUserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CrmBasedUserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CrmBasedUserListInfo();
                                if (d.app_id != null) {
                                    m.app_id = String(d.app_id);
                                }
                                switch (d.upload_key_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.upload_key_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.upload_key_type = 1;
                                    break;
                                case "CONTACT_INFO":
                                case 2:
                                    m.upload_key_type = 2;
                                    break;
                                case "CRM_ID":
                                case 3:
                                    m.upload_key_type = 3;
                                    break;
                                case "MOBILE_ADVERTISING_ID":
                                case 4:
                                    m.upload_key_type = 4;
                                    break;
                                }
                                switch (d.data_source_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.data_source_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.data_source_type = 1;
                                    break;
                                case "FIRST_PARTY":
                                case 2:
                                    m.data_source_type = 2;
                                    break;
                                case "THIRD_PARTY_CREDIT_BUREAU":
                                case 3:
                                    m.data_source_type = 3;
                                    break;
                                case "THIRD_PARTY_VOTER_FILE":
                                case 4:
                                    m.data_source_type = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CrmBasedUserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CrmBasedUserListInfo} m CrmBasedUserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CrmBasedUserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.upload_key_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.data_source_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.upload_key_type != null && m.hasOwnProperty("upload_key_type")) {
                                    d.upload_key_type = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType[m.upload_key_type] : m.upload_key_type;
                                }
                                if (m.data_source_type != null && m.hasOwnProperty("data_source_type")) {
                                    d.data_source_type = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType[m.data_source_type] : m.data_source_type;
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    d.app_id = m.app_id;
                                    if (o.oneofs)
                                        d._app_id = "app_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CrmBasedUserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CrmBasedUserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CrmBasedUserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CrmBasedUserListInfo;
                        })();
    
                        common.UserListRuleInfo = (function() {
    
                            /**
                             * Properties of a UserListRuleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListRuleInfo
                             * @property {google.ads.googleads.v11.enums.UserListRuleTypeEnum.UserListRuleType|null} [rule_type] UserListRuleInfo rule_type
                             * @property {Array.<google.ads.googleads.v11.common.IUserListRuleItemGroupInfo>|null} [rule_item_groups] UserListRuleInfo rule_item_groups
                             */
    
                            /**
                             * Constructs a new UserListRuleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListRuleInfo.
                             * @implements IUserListRuleInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListRuleInfo=} [p] Properties to set
                             */
                            function UserListRuleInfo(p) {
                                this.rule_item_groups = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListRuleInfo rule_type.
                             * @member {google.ads.googleads.v11.enums.UserListRuleTypeEnum.UserListRuleType} rule_type
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @instance
                             */
                            UserListRuleInfo.prototype.rule_type = 0;
    
                            /**
                             * UserListRuleInfo rule_item_groups.
                             * @member {Array.<google.ads.googleads.v11.common.IUserListRuleItemGroupInfo>} rule_item_groups
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @instance
                             */
                            UserListRuleInfo.prototype.rule_item_groups = $util.emptyArray;
    
                            /**
                             * Creates a new UserListRuleInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListRuleInfo} UserListRuleInfo instance
                             */
                            UserListRuleInfo.create = function create(properties) {
                                return new UserListRuleInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListRuleInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListRuleInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleInfo} m UserListRuleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.rule_type != null && Object.hasOwnProperty.call(m, "rule_type"))
                                    w.uint32(8).int32(m.rule_type);
                                if (m.rule_item_groups != null && m.rule_item_groups.length) {
                                    for (var i = 0; i < m.rule_item_groups.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo.encode(m.rule_item_groups[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListRuleInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListRuleInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleInfo} message UserListRuleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListRuleInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListRuleInfo} UserListRuleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListRuleInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.rule_type = r.int32();
                                        break;
                                    case 2:
                                        if (!(m.rule_item_groups && m.rule_item_groups.length))
                                            m.rule_item_groups = [];
                                        m.rule_item_groups.push($root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListRuleInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListRuleInfo} UserListRuleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListRuleInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListRuleInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.rule_type != null && m.hasOwnProperty("rule_type")) {
                                    switch (m.rule_type) {
                                    default:
                                        return "rule_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.rule_item_groups != null && m.hasOwnProperty("rule_item_groups")) {
                                    if (!Array.isArray(m.rule_item_groups))
                                        return "rule_item_groups: array expected";
                                    for (var i = 0; i < m.rule_item_groups.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo.verify(m.rule_item_groups[i]);
                                            if (e)
                                                return "rule_item_groups." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListRuleInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListRuleInfo} UserListRuleInfo
                             */
                            UserListRuleInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListRuleInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListRuleInfo();
                                switch (d.rule_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.rule_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.rule_type = 1;
                                    break;
                                case "AND_OF_ORS":
                                case 2:
                                    m.rule_type = 2;
                                    break;
                                case "OR_OF_ANDS":
                                case 3:
                                    m.rule_type = 3;
                                    break;
                                }
                                if (d.rule_item_groups) {
                                    if (!Array.isArray(d.rule_item_groups))
                                        throw TypeError(".google.ads.googleads.v11.common.UserListRuleInfo.rule_item_groups: array expected");
                                    m.rule_item_groups = [];
                                    for (var i = 0; i < d.rule_item_groups.length; ++i) {
                                        if (typeof d.rule_item_groups[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.UserListRuleInfo.rule_item_groups: object expected");
                                        m.rule_item_groups[i] = $root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo.fromObject(d.rule_item_groups[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListRuleInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListRuleInfo} m UserListRuleInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListRuleInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.rule_item_groups = [];
                                }
                                if (o.defaults) {
                                    d.rule_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.rule_type != null && m.hasOwnProperty("rule_type")) {
                                    d.rule_type = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListRuleTypeEnum.UserListRuleType[m.rule_type] : m.rule_type;
                                }
                                if (m.rule_item_groups && m.rule_item_groups.length) {
                                    d.rule_item_groups = [];
                                    for (var j = 0; j < m.rule_item_groups.length; ++j) {
                                        d.rule_item_groups[j] = $root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo.toObject(m.rule_item_groups[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListRuleInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListRuleInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListRuleInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListRuleInfo;
                        })();
    
                        common.UserListRuleItemGroupInfo = (function() {
    
                            /**
                             * Properties of a UserListRuleItemGroupInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListRuleItemGroupInfo
                             * @property {Array.<google.ads.googleads.v11.common.IUserListRuleItemInfo>|null} [rule_items] UserListRuleItemGroupInfo rule_items
                             */
    
                            /**
                             * Constructs a new UserListRuleItemGroupInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListRuleItemGroupInfo.
                             * @implements IUserListRuleItemGroupInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemGroupInfo=} [p] Properties to set
                             */
                            function UserListRuleItemGroupInfo(p) {
                                this.rule_items = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListRuleItemGroupInfo rule_items.
                             * @member {Array.<google.ads.googleads.v11.common.IUserListRuleItemInfo>} rule_items
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @instance
                             */
                            UserListRuleItemGroupInfo.prototype.rule_items = $util.emptyArray;
    
                            /**
                             * Creates a new UserListRuleItemGroupInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemGroupInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemGroupInfo} UserListRuleItemGroupInfo instance
                             */
                            UserListRuleItemGroupInfo.create = function create(properties) {
                                return new UserListRuleItemGroupInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListRuleItemGroupInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListRuleItemGroupInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemGroupInfo} m UserListRuleItemGroupInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleItemGroupInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.rule_items != null && m.rule_items.length) {
                                    for (var i = 0; i < m.rule_items.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserListRuleItemInfo.encode(m.rule_items[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListRuleItemGroupInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListRuleItemGroupInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemGroupInfo} message UserListRuleItemGroupInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleItemGroupInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListRuleItemGroupInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemGroupInfo} UserListRuleItemGroupInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleItemGroupInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.rule_items && m.rule_items.length))
                                            m.rule_items = [];
                                        m.rule_items.push($root.google.ads.googleads.v11.common.UserListRuleItemInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListRuleItemGroupInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemGroupInfo} UserListRuleItemGroupInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleItemGroupInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListRuleItemGroupInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListRuleItemGroupInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.rule_items != null && m.hasOwnProperty("rule_items")) {
                                    if (!Array.isArray(m.rule_items))
                                        return "rule_items: array expected";
                                    for (var i = 0; i < m.rule_items.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserListRuleItemInfo.verify(m.rule_items[i]);
                                            if (e)
                                                return "rule_items." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListRuleItemGroupInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemGroupInfo} UserListRuleItemGroupInfo
                             */
                            UserListRuleItemGroupInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListRuleItemGroupInfo();
                                if (d.rule_items) {
                                    if (!Array.isArray(d.rule_items))
                                        throw TypeError(".google.ads.googleads.v11.common.UserListRuleItemGroupInfo.rule_items: array expected");
                                    m.rule_items = [];
                                    for (var i = 0; i < d.rule_items.length; ++i) {
                                        if (typeof d.rule_items[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.UserListRuleItemGroupInfo.rule_items: object expected");
                                        m.rule_items[i] = $root.google.ads.googleads.v11.common.UserListRuleItemInfo.fromObject(d.rule_items[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListRuleItemGroupInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListRuleItemGroupInfo} m UserListRuleItemGroupInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListRuleItemGroupInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.rule_items = [];
                                }
                                if (m.rule_items && m.rule_items.length) {
                                    d.rule_items = [];
                                    for (var j = 0; j < m.rule_items.length; ++j) {
                                        d.rule_items[j] = $root.google.ads.googleads.v11.common.UserListRuleItemInfo.toObject(m.rule_items[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListRuleItemGroupInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemGroupInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListRuleItemGroupInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListRuleItemGroupInfo;
                        })();
    
                        common.UserListRuleItemInfo = (function() {
    
                            /**
                             * Properties of a UserListRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListRuleItemInfo
                             * @property {string|null} [name] UserListRuleItemInfo name
                             * @property {google.ads.googleads.v11.common.IUserListNumberRuleItemInfo|null} [number_rule_item] UserListRuleItemInfo number_rule_item
                             * @property {google.ads.googleads.v11.common.IUserListStringRuleItemInfo|null} [string_rule_item] UserListRuleItemInfo string_rule_item
                             * @property {google.ads.googleads.v11.common.IUserListDateRuleItemInfo|null} [date_rule_item] UserListRuleItemInfo date_rule_item
                             */
    
                            /**
                             * Constructs a new UserListRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListRuleItemInfo.
                             * @implements IUserListRuleItemInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemInfo=} [p] Properties to set
                             */
                            function UserListRuleItemInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListRuleItemInfo name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @instance
                             */
                            UserListRuleItemInfo.prototype.name = null;
    
                            /**
                             * UserListRuleItemInfo number_rule_item.
                             * @member {google.ads.googleads.v11.common.IUserListNumberRuleItemInfo|null|undefined} number_rule_item
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @instance
                             */
                            UserListRuleItemInfo.prototype.number_rule_item = null;
    
                            /**
                             * UserListRuleItemInfo string_rule_item.
                             * @member {google.ads.googleads.v11.common.IUserListStringRuleItemInfo|null|undefined} string_rule_item
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @instance
                             */
                            UserListRuleItemInfo.prototype.string_rule_item = null;
    
                            /**
                             * UserListRuleItemInfo date_rule_item.
                             * @member {google.ads.googleads.v11.common.IUserListDateRuleItemInfo|null|undefined} date_rule_item
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @instance
                             */
                            UserListRuleItemInfo.prototype.date_rule_item = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListRuleItemInfo _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @instance
                             */
                            Object.defineProperty(UserListRuleItemInfo.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserListRuleItemInfo rule_item.
                             * @member {"number_rule_item"|"string_rule_item"|"date_rule_item"|undefined} rule_item
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @instance
                             */
                            Object.defineProperty(UserListRuleItemInfo.prototype, "rule_item", {
                                get: $util.oneOfGetter($oneOfFields = ["number_rule_item", "string_rule_item", "date_rule_item"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListRuleItemInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemInfo} UserListRuleItemInfo instance
                             */
                            UserListRuleItemInfo.create = function create(properties) {
                                return new UserListRuleItemInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListRuleItemInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListRuleItemInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemInfo} m UserListRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleItemInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.number_rule_item != null && Object.hasOwnProperty.call(m, "number_rule_item"))
                                    $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo.encode(m.number_rule_item, w.uint32(18).fork()).ldelim();
                                if (m.string_rule_item != null && Object.hasOwnProperty.call(m, "string_rule_item"))
                                    $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo.encode(m.string_rule_item, w.uint32(26).fork()).ldelim();
                                if (m.date_rule_item != null && Object.hasOwnProperty.call(m, "date_rule_item"))
                                    $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo.encode(m.date_rule_item, w.uint32(34).fork()).ldelim();
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(42).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListRuleItemInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListRuleItemInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListRuleItemInfo} message UserListRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleItemInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListRuleItemInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemInfo} UserListRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleItemInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListRuleItemInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.name = r.string();
                                        break;
                                    case 2:
                                        m.number_rule_item = $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.string_rule_item = $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.date_rule_item = $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListRuleItemInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemInfo} UserListRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleItemInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListRuleItemInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListRuleItemInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.number_rule_item != null && m.hasOwnProperty("number_rule_item")) {
                                    p.rule_item = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo.verify(m.number_rule_item);
                                        if (e)
                                            return "number_rule_item." + e;
                                    }
                                }
                                if (m.string_rule_item != null && m.hasOwnProperty("string_rule_item")) {
                                    if (p.rule_item === 1)
                                        return "rule_item: multiple values";
                                    p.rule_item = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo.verify(m.string_rule_item);
                                        if (e)
                                            return "string_rule_item." + e;
                                    }
                                }
                                if (m.date_rule_item != null && m.hasOwnProperty("date_rule_item")) {
                                    if (p.rule_item === 1)
                                        return "rule_item: multiple values";
                                    p.rule_item = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo.verify(m.date_rule_item);
                                        if (e)
                                            return "date_rule_item." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListRuleItemInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListRuleItemInfo} UserListRuleItemInfo
                             */
                            UserListRuleItemInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListRuleItemInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListRuleItemInfo();
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.number_rule_item != null) {
                                    if (typeof d.number_rule_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.UserListRuleItemInfo.number_rule_item: object expected");
                                    m.number_rule_item = $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo.fromObject(d.number_rule_item);
                                }
                                if (d.string_rule_item != null) {
                                    if (typeof d.string_rule_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.UserListRuleItemInfo.string_rule_item: object expected");
                                    m.string_rule_item = $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo.fromObject(d.string_rule_item);
                                }
                                if (d.date_rule_item != null) {
                                    if (typeof d.date_rule_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.UserListRuleItemInfo.date_rule_item: object expected");
                                    m.date_rule_item = $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo.fromObject(d.date_rule_item);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListRuleItemInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListRuleItemInfo} m UserListRuleItemInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListRuleItemInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.number_rule_item != null && m.hasOwnProperty("number_rule_item")) {
                                    d.number_rule_item = $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo.toObject(m.number_rule_item, o);
                                    if (o.oneofs)
                                        d.rule_item = "number_rule_item";
                                }
                                if (m.string_rule_item != null && m.hasOwnProperty("string_rule_item")) {
                                    d.string_rule_item = $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo.toObject(m.string_rule_item, o);
                                    if (o.oneofs)
                                        d.rule_item = "string_rule_item";
                                }
                                if (m.date_rule_item != null && m.hasOwnProperty("date_rule_item")) {
                                    d.date_rule_item = $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo.toObject(m.date_rule_item, o);
                                    if (o.oneofs)
                                        d.rule_item = "date_rule_item";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListRuleItemInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListRuleItemInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListRuleItemInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListRuleItemInfo;
                        })();
    
                        common.UserListDateRuleItemInfo = (function() {
    
                            /**
                             * Properties of a UserListDateRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListDateRuleItemInfo
                             * @property {google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator|null} [operator] UserListDateRuleItemInfo operator
                             * @property {string|null} [value] UserListDateRuleItemInfo value
                             * @property {number|null} [offset_in_days] UserListDateRuleItemInfo offset_in_days
                             */
    
                            /**
                             * Constructs a new UserListDateRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListDateRuleItemInfo.
                             * @implements IUserListDateRuleItemInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListDateRuleItemInfo=} [p] Properties to set
                             */
                            function UserListDateRuleItemInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListDateRuleItemInfo operator.
                             * @member {google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator} operator
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @instance
                             */
                            UserListDateRuleItemInfo.prototype.operator = 0;
    
                            /**
                             * UserListDateRuleItemInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @instance
                             */
                            UserListDateRuleItemInfo.prototype.value = null;
    
                            /**
                             * UserListDateRuleItemInfo offset_in_days.
                             * @member {number|null|undefined} offset_in_days
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @instance
                             */
                            UserListDateRuleItemInfo.prototype.offset_in_days = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListDateRuleItemInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @instance
                             */
                            Object.defineProperty(UserListDateRuleItemInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserListDateRuleItemInfo _offset_in_days.
                             * @member {"offset_in_days"|undefined} _offset_in_days
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @instance
                             */
                            Object.defineProperty(UserListDateRuleItemInfo.prototype, "_offset_in_days", {
                                get: $util.oneOfGetter($oneOfFields = ["offset_in_days"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListDateRuleItemInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListDateRuleItemInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListDateRuleItemInfo} UserListDateRuleItemInfo instance
                             */
                            UserListDateRuleItemInfo.create = function create(properties) {
                                return new UserListDateRuleItemInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListDateRuleItemInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListDateRuleItemInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListDateRuleItemInfo} m UserListDateRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListDateRuleItemInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(8).int32(m.operator);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(34).string(m.value);
                                if (m.offset_in_days != null && Object.hasOwnProperty.call(m, "offset_in_days"))
                                    w.uint32(40).int64(m.offset_in_days);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListDateRuleItemInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListDateRuleItemInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListDateRuleItemInfo} message UserListDateRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListDateRuleItemInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListDateRuleItemInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListDateRuleItemInfo} UserListDateRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListDateRuleItemInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operator = r.int32();
                                        break;
                                    case 4:
                                        m.value = r.string();
                                        break;
                                    case 5:
                                        m.offset_in_days = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListDateRuleItemInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListDateRuleItemInfo} UserListDateRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListDateRuleItemInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListDateRuleItemInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListDateRuleItemInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                if (m.offset_in_days != null && m.hasOwnProperty("offset_in_days")) {
                                    p._offset_in_days = 1;
                                    if (!$util.isInteger(m.offset_in_days) && !(m.offset_in_days && $util.isInteger(m.offset_in_days.low) && $util.isInteger(m.offset_in_days.high)))
                                        return "offset_in_days: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListDateRuleItemInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListDateRuleItemInfo} UserListDateRuleItemInfo
                             */
                            UserListDateRuleItemInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListDateRuleItemInfo();
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "EQUALS":
                                case 2:
                                    m.operator = 2;
                                    break;
                                case "NOT_EQUALS":
                                case 3:
                                    m.operator = 3;
                                    break;
                                case "BEFORE":
                                case 4:
                                    m.operator = 4;
                                    break;
                                case "AFTER":
                                case 5:
                                    m.operator = 5;
                                    break;
                                }
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                if (d.offset_in_days != null) {
                                    if ($util.Long)
                                        (m.offset_in_days = $util.Long.fromValue(d.offset_in_days)).unsigned = false;
                                    else if (typeof d.offset_in_days === "string")
                                        m.offset_in_days = parseInt(d.offset_in_days, 10);
                                    else if (typeof d.offset_in_days === "number")
                                        m.offset_in_days = d.offset_in_days;
                                    else if (typeof d.offset_in_days === "object")
                                        m.offset_in_days = new $util.LongBits(d.offset_in_days.low >>> 0, d.offset_in_days.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListDateRuleItemInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListDateRuleItemInfo} m UserListDateRuleItemInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListDateRuleItemInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator[m.operator] : m.operator;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                if (m.offset_in_days != null && m.hasOwnProperty("offset_in_days")) {
                                    if (typeof m.offset_in_days === "number")
                                        d.offset_in_days = o.longs === String ? String(m.offset_in_days) : m.offset_in_days;
                                    else
                                        d.offset_in_days = o.longs === String ? $util.Long.prototype.toString.call(m.offset_in_days) : o.longs === Number ? new $util.LongBits(m.offset_in_days.low >>> 0, m.offset_in_days.high >>> 0).toNumber() : m.offset_in_days;
                                    if (o.oneofs)
                                        d._offset_in_days = "offset_in_days";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListDateRuleItemInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListDateRuleItemInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListDateRuleItemInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListDateRuleItemInfo;
                        })();
    
                        common.UserListNumberRuleItemInfo = (function() {
    
                            /**
                             * Properties of a UserListNumberRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListNumberRuleItemInfo
                             * @property {google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator|null} [operator] UserListNumberRuleItemInfo operator
                             * @property {number|null} [value] UserListNumberRuleItemInfo value
                             */
    
                            /**
                             * Constructs a new UserListNumberRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListNumberRuleItemInfo.
                             * @implements IUserListNumberRuleItemInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListNumberRuleItemInfo=} [p] Properties to set
                             */
                            function UserListNumberRuleItemInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListNumberRuleItemInfo operator.
                             * @member {google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator} operator
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @instance
                             */
                            UserListNumberRuleItemInfo.prototype.operator = 0;
    
                            /**
                             * UserListNumberRuleItemInfo value.
                             * @member {number|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @instance
                             */
                            UserListNumberRuleItemInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListNumberRuleItemInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @instance
                             */
                            Object.defineProperty(UserListNumberRuleItemInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListNumberRuleItemInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListNumberRuleItemInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListNumberRuleItemInfo} UserListNumberRuleItemInfo instance
                             */
                            UserListNumberRuleItemInfo.create = function create(properties) {
                                return new UserListNumberRuleItemInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListNumberRuleItemInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListNumberRuleItemInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListNumberRuleItemInfo} m UserListNumberRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListNumberRuleItemInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(8).int32(m.operator);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(25).double(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListNumberRuleItemInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListNumberRuleItemInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListNumberRuleItemInfo} message UserListNumberRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListNumberRuleItemInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListNumberRuleItemInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListNumberRuleItemInfo} UserListNumberRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListNumberRuleItemInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operator = r.int32();
                                        break;
                                    case 3:
                                        m.value = r.double();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListNumberRuleItemInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListNumberRuleItemInfo} UserListNumberRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListNumberRuleItemInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListNumberRuleItemInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListNumberRuleItemInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (typeof m.value !== "number")
                                        return "value: number expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListNumberRuleItemInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListNumberRuleItemInfo} UserListNumberRuleItemInfo
                             */
                            UserListNumberRuleItemInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListNumberRuleItemInfo();
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "GREATER_THAN":
                                case 2:
                                    m.operator = 2;
                                    break;
                                case "GREATER_THAN_OR_EQUAL":
                                case 3:
                                    m.operator = 3;
                                    break;
                                case "EQUALS":
                                case 4:
                                    m.operator = 4;
                                    break;
                                case "NOT_EQUALS":
                                case 5:
                                    m.operator = 5;
                                    break;
                                case "LESS_THAN":
                                case 6:
                                    m.operator = 6;
                                    break;
                                case "LESS_THAN_OR_EQUAL":
                                case 7:
                                    m.operator = 7;
                                    break;
                                }
                                if (d.value != null) {
                                    m.value = Number(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListNumberRuleItemInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListNumberRuleItemInfo} m UserListNumberRuleItemInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListNumberRuleItemInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator[m.operator] : m.operator;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = o.json && !isFinite(m.value) ? String(m.value) : m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListNumberRuleItemInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListNumberRuleItemInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListNumberRuleItemInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListNumberRuleItemInfo;
                        })();
    
                        common.UserListStringRuleItemInfo = (function() {
    
                            /**
                             * Properties of a UserListStringRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListStringRuleItemInfo
                             * @property {google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator|null} [operator] UserListStringRuleItemInfo operator
                             * @property {string|null} [value] UserListStringRuleItemInfo value
                             */
    
                            /**
                             * Constructs a new UserListStringRuleItemInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListStringRuleItemInfo.
                             * @implements IUserListStringRuleItemInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListStringRuleItemInfo=} [p] Properties to set
                             */
                            function UserListStringRuleItemInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListStringRuleItemInfo operator.
                             * @member {google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator} operator
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @instance
                             */
                            UserListStringRuleItemInfo.prototype.operator = 0;
    
                            /**
                             * UserListStringRuleItemInfo value.
                             * @member {string|null|undefined} value
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @instance
                             */
                            UserListStringRuleItemInfo.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListStringRuleItemInfo _value.
                             * @member {"value"|undefined} _value
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @instance
                             */
                            Object.defineProperty(UserListStringRuleItemInfo.prototype, "_value", {
                                get: $util.oneOfGetter($oneOfFields = ["value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListStringRuleItemInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListStringRuleItemInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListStringRuleItemInfo} UserListStringRuleItemInfo instance
                             */
                            UserListStringRuleItemInfo.create = function create(properties) {
                                return new UserListStringRuleItemInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListStringRuleItemInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListStringRuleItemInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListStringRuleItemInfo} m UserListStringRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListStringRuleItemInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(8).int32(m.operator);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(26).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListStringRuleItemInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListStringRuleItemInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListStringRuleItemInfo} message UserListStringRuleItemInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListStringRuleItemInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListStringRuleItemInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListStringRuleItemInfo} UserListStringRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListStringRuleItemInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operator = r.int32();
                                        break;
                                    case 3:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListStringRuleItemInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListStringRuleItemInfo} UserListStringRuleItemInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListStringRuleItemInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListStringRuleItemInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListStringRuleItemInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    p._value = 1;
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListStringRuleItemInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListStringRuleItemInfo} UserListStringRuleItemInfo
                             */
                            UserListStringRuleItemInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListStringRuleItemInfo();
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "CONTAINS":
                                case 2:
                                    m.operator = 2;
                                    break;
                                case "EQUALS":
                                case 3:
                                    m.operator = 3;
                                    break;
                                case "STARTS_WITH":
                                case 4:
                                    m.operator = 4;
                                    break;
                                case "ENDS_WITH":
                                case 5:
                                    m.operator = 5;
                                    break;
                                case "NOT_EQUALS":
                                case 6:
                                    m.operator = 6;
                                    break;
                                case "NOT_CONTAINS":
                                case 7:
                                    m.operator = 7;
                                    break;
                                case "NOT_STARTS_WITH":
                                case 8:
                                    m.operator = 8;
                                    break;
                                case "NOT_ENDS_WITH":
                                case 9:
                                    m.operator = 9;
                                    break;
                                }
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListStringRuleItemInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListStringRuleItemInfo} m UserListStringRuleItemInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListStringRuleItemInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator[m.operator] : m.operator;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                    if (o.oneofs)
                                        d._value = "value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListStringRuleItemInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListStringRuleItemInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListStringRuleItemInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListStringRuleItemInfo;
                        })();
    
                        common.CombinedRuleUserListInfo = (function() {
    
                            /**
                             * Properties of a CombinedRuleUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ICombinedRuleUserListInfo
                             * @property {google.ads.googleads.v11.common.IUserListRuleInfo|null} [left_operand] CombinedRuleUserListInfo left_operand
                             * @property {google.ads.googleads.v11.common.IUserListRuleInfo|null} [right_operand] CombinedRuleUserListInfo right_operand
                             * @property {google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum.UserListCombinedRuleOperator|null} [rule_operator] CombinedRuleUserListInfo rule_operator
                             */
    
                            /**
                             * Constructs a new CombinedRuleUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a CombinedRuleUserListInfo.
                             * @implements ICombinedRuleUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ICombinedRuleUserListInfo=} [p] Properties to set
                             */
                            function CombinedRuleUserListInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CombinedRuleUserListInfo left_operand.
                             * @member {google.ads.googleads.v11.common.IUserListRuleInfo|null|undefined} left_operand
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @instance
                             */
                            CombinedRuleUserListInfo.prototype.left_operand = null;
    
                            /**
                             * CombinedRuleUserListInfo right_operand.
                             * @member {google.ads.googleads.v11.common.IUserListRuleInfo|null|undefined} right_operand
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @instance
                             */
                            CombinedRuleUserListInfo.prototype.right_operand = null;
    
                            /**
                             * CombinedRuleUserListInfo rule_operator.
                             * @member {google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum.UserListCombinedRuleOperator} rule_operator
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @instance
                             */
                            CombinedRuleUserListInfo.prototype.rule_operator = 0;
    
                            /**
                             * Creates a new CombinedRuleUserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICombinedRuleUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.CombinedRuleUserListInfo} CombinedRuleUserListInfo instance
                             */
                            CombinedRuleUserListInfo.create = function create(properties) {
                                return new CombinedRuleUserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified CombinedRuleUserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.CombinedRuleUserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICombinedRuleUserListInfo} m CombinedRuleUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedRuleUserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.left_operand != null && Object.hasOwnProperty.call(m, "left_operand"))
                                    $root.google.ads.googleads.v11.common.UserListRuleInfo.encode(m.left_operand, w.uint32(10).fork()).ldelim();
                                if (m.right_operand != null && Object.hasOwnProperty.call(m, "right_operand"))
                                    $root.google.ads.googleads.v11.common.UserListRuleInfo.encode(m.right_operand, w.uint32(18).fork()).ldelim();
                                if (m.rule_operator != null && Object.hasOwnProperty.call(m, "rule_operator"))
                                    w.uint32(24).int32(m.rule_operator);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CombinedRuleUserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.CombinedRuleUserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ICombinedRuleUserListInfo} message CombinedRuleUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedRuleUserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CombinedRuleUserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.CombinedRuleUserListInfo} CombinedRuleUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedRuleUserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.left_operand = $root.google.ads.googleads.v11.common.UserListRuleInfo.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.right_operand = $root.google.ads.googleads.v11.common.UserListRuleInfo.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.rule_operator = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CombinedRuleUserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.CombinedRuleUserListInfo} CombinedRuleUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedRuleUserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CombinedRuleUserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CombinedRuleUserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.left_operand != null && m.hasOwnProperty("left_operand")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListRuleInfo.verify(m.left_operand);
                                        if (e)
                                            return "left_operand." + e;
                                    }
                                }
                                if (m.right_operand != null && m.hasOwnProperty("right_operand")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListRuleInfo.verify(m.right_operand);
                                        if (e)
                                            return "right_operand." + e;
                                    }
                                }
                                if (m.rule_operator != null && m.hasOwnProperty("rule_operator")) {
                                    switch (m.rule_operator) {
                                    default:
                                        return "rule_operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CombinedRuleUserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.CombinedRuleUserListInfo} CombinedRuleUserListInfo
                             */
                            CombinedRuleUserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo();
                                if (d.left_operand != null) {
                                    if (typeof d.left_operand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.CombinedRuleUserListInfo.left_operand: object expected");
                                    m.left_operand = $root.google.ads.googleads.v11.common.UserListRuleInfo.fromObject(d.left_operand);
                                }
                                if (d.right_operand != null) {
                                    if (typeof d.right_operand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.CombinedRuleUserListInfo.right_operand: object expected");
                                    m.right_operand = $root.google.ads.googleads.v11.common.UserListRuleInfo.fromObject(d.right_operand);
                                }
                                switch (d.rule_operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.rule_operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.rule_operator = 1;
                                    break;
                                case "AND":
                                case 2:
                                    m.rule_operator = 2;
                                    break;
                                case "AND_NOT":
                                case 3:
                                    m.rule_operator = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CombinedRuleUserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.CombinedRuleUserListInfo} m CombinedRuleUserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CombinedRuleUserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.left_operand = null;
                                    d.right_operand = null;
                                    d.rule_operator = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.left_operand != null && m.hasOwnProperty("left_operand")) {
                                    d.left_operand = $root.google.ads.googleads.v11.common.UserListRuleInfo.toObject(m.left_operand, o);
                                }
                                if (m.right_operand != null && m.hasOwnProperty("right_operand")) {
                                    d.right_operand = $root.google.ads.googleads.v11.common.UserListRuleInfo.toObject(m.right_operand, o);
                                }
                                if (m.rule_operator != null && m.hasOwnProperty("rule_operator")) {
                                    d.rule_operator = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum.UserListCombinedRuleOperator[m.rule_operator] : m.rule_operator;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CombinedRuleUserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.CombinedRuleUserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CombinedRuleUserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CombinedRuleUserListInfo;
                        })();
    
                        common.ExpressionRuleUserListInfo = (function() {
    
                            /**
                             * Properties of an ExpressionRuleUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IExpressionRuleUserListInfo
                             * @property {google.ads.googleads.v11.common.IUserListRuleInfo|null} [rule] ExpressionRuleUserListInfo rule
                             */
    
                            /**
                             * Constructs a new ExpressionRuleUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents an ExpressionRuleUserListInfo.
                             * @implements IExpressionRuleUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IExpressionRuleUserListInfo=} [p] Properties to set
                             */
                            function ExpressionRuleUserListInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExpressionRuleUserListInfo rule.
                             * @member {google.ads.googleads.v11.common.IUserListRuleInfo|null|undefined} rule
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @instance
                             */
                            ExpressionRuleUserListInfo.prototype.rule = null;
    
                            /**
                             * Creates a new ExpressionRuleUserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpressionRuleUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.ExpressionRuleUserListInfo} ExpressionRuleUserListInfo instance
                             */
                            ExpressionRuleUserListInfo.create = function create(properties) {
                                return new ExpressionRuleUserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ExpressionRuleUserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.ExpressionRuleUserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpressionRuleUserListInfo} m ExpressionRuleUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpressionRuleUserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.rule != null && Object.hasOwnProperty.call(m, "rule"))
                                    $root.google.ads.googleads.v11.common.UserListRuleInfo.encode(m.rule, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExpressionRuleUserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.ExpressionRuleUserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IExpressionRuleUserListInfo} message ExpressionRuleUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpressionRuleUserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExpressionRuleUserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.ExpressionRuleUserListInfo} ExpressionRuleUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpressionRuleUserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.rule = $root.google.ads.googleads.v11.common.UserListRuleInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExpressionRuleUserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.ExpressionRuleUserListInfo} ExpressionRuleUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpressionRuleUserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExpressionRuleUserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExpressionRuleUserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.rule != null && m.hasOwnProperty("rule")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListRuleInfo.verify(m.rule);
                                        if (e)
                                            return "rule." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExpressionRuleUserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.ExpressionRuleUserListInfo} ExpressionRuleUserListInfo
                             */
                            ExpressionRuleUserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo();
                                if (d.rule != null) {
                                    if (typeof d.rule !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.ExpressionRuleUserListInfo.rule: object expected");
                                    m.rule = $root.google.ads.googleads.v11.common.UserListRuleInfo.fromObject(d.rule);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExpressionRuleUserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ExpressionRuleUserListInfo} m ExpressionRuleUserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExpressionRuleUserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.rule = null;
                                }
                                if (m.rule != null && m.hasOwnProperty("rule")) {
                                    d.rule = $root.google.ads.googleads.v11.common.UserListRuleInfo.toObject(m.rule, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExpressionRuleUserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.ExpressionRuleUserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExpressionRuleUserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExpressionRuleUserListInfo;
                        })();
    
                        common.RuleBasedUserListInfo = (function() {
    
                            /**
                             * Properties of a RuleBasedUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IRuleBasedUserListInfo
                             * @property {google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum.UserListPrepopulationStatus|null} [prepopulation_status] RuleBasedUserListInfo prepopulation_status
                             * @property {google.ads.googleads.v11.common.ICombinedRuleUserListInfo|null} [combined_rule_user_list] RuleBasedUserListInfo combined_rule_user_list
                             * @property {google.ads.googleads.v11.common.IExpressionRuleUserListInfo|null} [expression_rule_user_list] RuleBasedUserListInfo expression_rule_user_list
                             */
    
                            /**
                             * Constructs a new RuleBasedUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a RuleBasedUserListInfo.
                             * @implements IRuleBasedUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IRuleBasedUserListInfo=} [p] Properties to set
                             */
                            function RuleBasedUserListInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RuleBasedUserListInfo prepopulation_status.
                             * @member {google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum.UserListPrepopulationStatus} prepopulation_status
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @instance
                             */
                            RuleBasedUserListInfo.prototype.prepopulation_status = 0;
    
                            /**
                             * RuleBasedUserListInfo combined_rule_user_list.
                             * @member {google.ads.googleads.v11.common.ICombinedRuleUserListInfo|null|undefined} combined_rule_user_list
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @instance
                             */
                            RuleBasedUserListInfo.prototype.combined_rule_user_list = null;
    
                            /**
                             * RuleBasedUserListInfo expression_rule_user_list.
                             * @member {google.ads.googleads.v11.common.IExpressionRuleUserListInfo|null|undefined} expression_rule_user_list
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @instance
                             */
                            RuleBasedUserListInfo.prototype.expression_rule_user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * RuleBasedUserListInfo rule_based_user_list.
                             * @member {"combined_rule_user_list"|"expression_rule_user_list"|undefined} rule_based_user_list
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @instance
                             */
                            Object.defineProperty(RuleBasedUserListInfo.prototype, "rule_based_user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["combined_rule_user_list", "expression_rule_user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new RuleBasedUserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IRuleBasedUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.RuleBasedUserListInfo} RuleBasedUserListInfo instance
                             */
                            RuleBasedUserListInfo.create = function create(properties) {
                                return new RuleBasedUserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified RuleBasedUserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.RuleBasedUserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IRuleBasedUserListInfo} m RuleBasedUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RuleBasedUserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.prepopulation_status != null && Object.hasOwnProperty.call(m, "prepopulation_status"))
                                    w.uint32(8).int32(m.prepopulation_status);
                                if (m.combined_rule_user_list != null && Object.hasOwnProperty.call(m, "combined_rule_user_list"))
                                    $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo.encode(m.combined_rule_user_list, w.uint32(18).fork()).ldelim();
                                if (m.expression_rule_user_list != null && Object.hasOwnProperty.call(m, "expression_rule_user_list"))
                                    $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo.encode(m.expression_rule_user_list, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RuleBasedUserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.RuleBasedUserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IRuleBasedUserListInfo} message RuleBasedUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RuleBasedUserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RuleBasedUserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.RuleBasedUserListInfo} RuleBasedUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RuleBasedUserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.RuleBasedUserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.prepopulation_status = r.int32();
                                        break;
                                    case 2:
                                        m.combined_rule_user_list = $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.expression_rule_user_list = $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RuleBasedUserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.RuleBasedUserListInfo} RuleBasedUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RuleBasedUserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RuleBasedUserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RuleBasedUserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.prepopulation_status != null && m.hasOwnProperty("prepopulation_status")) {
                                    switch (m.prepopulation_status) {
                                    default:
                                        return "prepopulation_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.combined_rule_user_list != null && m.hasOwnProperty("combined_rule_user_list")) {
                                    p.rule_based_user_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo.verify(m.combined_rule_user_list);
                                        if (e)
                                            return "combined_rule_user_list." + e;
                                    }
                                }
                                if (m.expression_rule_user_list != null && m.hasOwnProperty("expression_rule_user_list")) {
                                    if (p.rule_based_user_list === 1)
                                        return "rule_based_user_list: multiple values";
                                    p.rule_based_user_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo.verify(m.expression_rule_user_list);
                                        if (e)
                                            return "expression_rule_user_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RuleBasedUserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.RuleBasedUserListInfo} RuleBasedUserListInfo
                             */
                            RuleBasedUserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.RuleBasedUserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.RuleBasedUserListInfo();
                                switch (d.prepopulation_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.prepopulation_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.prepopulation_status = 1;
                                    break;
                                case "REQUESTED":
                                case 2:
                                    m.prepopulation_status = 2;
                                    break;
                                case "FINISHED":
                                case 3:
                                    m.prepopulation_status = 3;
                                    break;
                                case "FAILED":
                                case 4:
                                    m.prepopulation_status = 4;
                                    break;
                                }
                                if (d.combined_rule_user_list != null) {
                                    if (typeof d.combined_rule_user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.RuleBasedUserListInfo.combined_rule_user_list: object expected");
                                    m.combined_rule_user_list = $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo.fromObject(d.combined_rule_user_list);
                                }
                                if (d.expression_rule_user_list != null) {
                                    if (typeof d.expression_rule_user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.common.RuleBasedUserListInfo.expression_rule_user_list: object expected");
                                    m.expression_rule_user_list = $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo.fromObject(d.expression_rule_user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RuleBasedUserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.RuleBasedUserListInfo} m RuleBasedUserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RuleBasedUserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.prepopulation_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.prepopulation_status != null && m.hasOwnProperty("prepopulation_status")) {
                                    d.prepopulation_status = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum.UserListPrepopulationStatus[m.prepopulation_status] : m.prepopulation_status;
                                }
                                if (m.combined_rule_user_list != null && m.hasOwnProperty("combined_rule_user_list")) {
                                    d.combined_rule_user_list = $root.google.ads.googleads.v11.common.CombinedRuleUserListInfo.toObject(m.combined_rule_user_list, o);
                                    if (o.oneofs)
                                        d.rule_based_user_list = "combined_rule_user_list";
                                }
                                if (m.expression_rule_user_list != null && m.hasOwnProperty("expression_rule_user_list")) {
                                    d.expression_rule_user_list = $root.google.ads.googleads.v11.common.ExpressionRuleUserListInfo.toObject(m.expression_rule_user_list, o);
                                    if (o.oneofs)
                                        d.rule_based_user_list = "expression_rule_user_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RuleBasedUserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.RuleBasedUserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RuleBasedUserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RuleBasedUserListInfo;
                        })();
    
                        common.LogicalUserListInfo = (function() {
    
                            /**
                             * Properties of a LogicalUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILogicalUserListInfo
                             * @property {Array.<google.ads.googleads.v11.common.IUserListLogicalRuleInfo>|null} [rules] LogicalUserListInfo rules
                             */
    
                            /**
                             * Constructs a new LogicalUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LogicalUserListInfo.
                             * @implements ILogicalUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILogicalUserListInfo=} [p] Properties to set
                             */
                            function LogicalUserListInfo(p) {
                                this.rules = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LogicalUserListInfo rules.
                             * @member {Array.<google.ads.googleads.v11.common.IUserListLogicalRuleInfo>} rules
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @instance
                             */
                            LogicalUserListInfo.prototype.rules = $util.emptyArray;
    
                            /**
                             * Creates a new LogicalUserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILogicalUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LogicalUserListInfo} LogicalUserListInfo instance
                             */
                            LogicalUserListInfo.create = function create(properties) {
                                return new LogicalUserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LogicalUserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LogicalUserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILogicalUserListInfo} m LogicalUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LogicalUserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.rules != null && m.rules.length) {
                                    for (var i = 0; i < m.rules.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserListLogicalRuleInfo.encode(m.rules[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified LogicalUserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LogicalUserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILogicalUserListInfo} message LogicalUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LogicalUserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LogicalUserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LogicalUserListInfo} LogicalUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LogicalUserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LogicalUserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.rules && m.rules.length))
                                            m.rules = [];
                                        m.rules.push($root.google.ads.googleads.v11.common.UserListLogicalRuleInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LogicalUserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LogicalUserListInfo} LogicalUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LogicalUserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LogicalUserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LogicalUserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.rules != null && m.hasOwnProperty("rules")) {
                                    if (!Array.isArray(m.rules))
                                        return "rules: array expected";
                                    for (var i = 0; i < m.rules.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserListLogicalRuleInfo.verify(m.rules[i]);
                                            if (e)
                                                return "rules." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LogicalUserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LogicalUserListInfo} LogicalUserListInfo
                             */
                            LogicalUserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LogicalUserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LogicalUserListInfo();
                                if (d.rules) {
                                    if (!Array.isArray(d.rules))
                                        throw TypeError(".google.ads.googleads.v11.common.LogicalUserListInfo.rules: array expected");
                                    m.rules = [];
                                    for (var i = 0; i < d.rules.length; ++i) {
                                        if (typeof d.rules[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.LogicalUserListInfo.rules: object expected");
                                        m.rules[i] = $root.google.ads.googleads.v11.common.UserListLogicalRuleInfo.fromObject(d.rules[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LogicalUserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LogicalUserListInfo} m LogicalUserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LogicalUserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.rules = [];
                                }
                                if (m.rules && m.rules.length) {
                                    d.rules = [];
                                    for (var j = 0; j < m.rules.length; ++j) {
                                        d.rules[j] = $root.google.ads.googleads.v11.common.UserListLogicalRuleInfo.toObject(m.rules[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LogicalUserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LogicalUserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LogicalUserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LogicalUserListInfo;
                        })();
    
                        common.UserListLogicalRuleInfo = (function() {
    
                            /**
                             * Properties of a UserListLogicalRuleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListLogicalRuleInfo
                             * @property {google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator|null} [operator] UserListLogicalRuleInfo operator
                             * @property {Array.<google.ads.googleads.v11.common.ILogicalUserListOperandInfo>|null} [rule_operands] UserListLogicalRuleInfo rule_operands
                             */
    
                            /**
                             * Constructs a new UserListLogicalRuleInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListLogicalRuleInfo.
                             * @implements IUserListLogicalRuleInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListLogicalRuleInfo=} [p] Properties to set
                             */
                            function UserListLogicalRuleInfo(p) {
                                this.rule_operands = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListLogicalRuleInfo operator.
                             * @member {google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator} operator
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @instance
                             */
                            UserListLogicalRuleInfo.prototype.operator = 0;
    
                            /**
                             * UserListLogicalRuleInfo rule_operands.
                             * @member {Array.<google.ads.googleads.v11.common.ILogicalUserListOperandInfo>} rule_operands
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @instance
                             */
                            UserListLogicalRuleInfo.prototype.rule_operands = $util.emptyArray;
    
                            /**
                             * Creates a new UserListLogicalRuleInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListLogicalRuleInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListLogicalRuleInfo} UserListLogicalRuleInfo instance
                             */
                            UserListLogicalRuleInfo.create = function create(properties) {
                                return new UserListLogicalRuleInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListLogicalRuleInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListLogicalRuleInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListLogicalRuleInfo} m UserListLogicalRuleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListLogicalRuleInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(8).int32(m.operator);
                                if (m.rule_operands != null && m.rule_operands.length) {
                                    for (var i = 0; i < m.rule_operands.length; ++i)
                                        $root.google.ads.googleads.v11.common.LogicalUserListOperandInfo.encode(m.rule_operands[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListLogicalRuleInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListLogicalRuleInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListLogicalRuleInfo} message UserListLogicalRuleInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListLogicalRuleInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListLogicalRuleInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListLogicalRuleInfo} UserListLogicalRuleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListLogicalRuleInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListLogicalRuleInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operator = r.int32();
                                        break;
                                    case 2:
                                        if (!(m.rule_operands && m.rule_operands.length))
                                            m.rule_operands = [];
                                        m.rule_operands.push($root.google.ads.googleads.v11.common.LogicalUserListOperandInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListLogicalRuleInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListLogicalRuleInfo} UserListLogicalRuleInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListLogicalRuleInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListLogicalRuleInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListLogicalRuleInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.rule_operands != null && m.hasOwnProperty("rule_operands")) {
                                    if (!Array.isArray(m.rule_operands))
                                        return "rule_operands: array expected";
                                    for (var i = 0; i < m.rule_operands.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.LogicalUserListOperandInfo.verify(m.rule_operands[i]);
                                            if (e)
                                                return "rule_operands." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListLogicalRuleInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListLogicalRuleInfo} UserListLogicalRuleInfo
                             */
                            UserListLogicalRuleInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListLogicalRuleInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListLogicalRuleInfo();
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "ALL":
                                case 2:
                                    m.operator = 2;
                                    break;
                                case "ANY":
                                case 3:
                                    m.operator = 3;
                                    break;
                                case "NONE":
                                case 4:
                                    m.operator = 4;
                                    break;
                                }
                                if (d.rule_operands) {
                                    if (!Array.isArray(d.rule_operands))
                                        throw TypeError(".google.ads.googleads.v11.common.UserListLogicalRuleInfo.rule_operands: array expected");
                                    m.rule_operands = [];
                                    for (var i = 0; i < d.rule_operands.length; ++i) {
                                        if (typeof d.rule_operands[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.UserListLogicalRuleInfo.rule_operands: object expected");
                                        m.rule_operands[i] = $root.google.ads.googleads.v11.common.LogicalUserListOperandInfo.fromObject(d.rule_operands[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListLogicalRuleInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListLogicalRuleInfo} m UserListLogicalRuleInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListLogicalRuleInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.rule_operands = [];
                                }
                                if (o.defaults) {
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator[m.operator] : m.operator;
                                }
                                if (m.rule_operands && m.rule_operands.length) {
                                    d.rule_operands = [];
                                    for (var j = 0; j < m.rule_operands.length; ++j) {
                                        d.rule_operands[j] = $root.google.ads.googleads.v11.common.LogicalUserListOperandInfo.toObject(m.rule_operands[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListLogicalRuleInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListLogicalRuleInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListLogicalRuleInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListLogicalRuleInfo;
                        })();
    
                        common.LogicalUserListOperandInfo = (function() {
    
                            /**
                             * Properties of a LogicalUserListOperandInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface ILogicalUserListOperandInfo
                             * @property {string|null} [user_list] LogicalUserListOperandInfo user_list
                             */
    
                            /**
                             * Constructs a new LogicalUserListOperandInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a LogicalUserListOperandInfo.
                             * @implements ILogicalUserListOperandInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.ILogicalUserListOperandInfo=} [p] Properties to set
                             */
                            function LogicalUserListOperandInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LogicalUserListOperandInfo user_list.
                             * @member {string|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @instance
                             */
                            LogicalUserListOperandInfo.prototype.user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LogicalUserListOperandInfo _user_list.
                             * @member {"user_list"|undefined} _user_list
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @instance
                             */
                            Object.defineProperty(LogicalUserListOperandInfo.prototype, "_user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LogicalUserListOperandInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILogicalUserListOperandInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.LogicalUserListOperandInfo} LogicalUserListOperandInfo instance
                             */
                            LogicalUserListOperandInfo.create = function create(properties) {
                                return new LogicalUserListOperandInfo(properties);
                            };
    
                            /**
                             * Encodes the specified LogicalUserListOperandInfo message. Does not implicitly {@link google.ads.googleads.v11.common.LogicalUserListOperandInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILogicalUserListOperandInfo} m LogicalUserListOperandInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LogicalUserListOperandInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    w.uint32(18).string(m.user_list);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LogicalUserListOperandInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.LogicalUserListOperandInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.ILogicalUserListOperandInfo} message LogicalUserListOperandInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LogicalUserListOperandInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LogicalUserListOperandInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.LogicalUserListOperandInfo} LogicalUserListOperandInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LogicalUserListOperandInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.LogicalUserListOperandInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.user_list = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LogicalUserListOperandInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.LogicalUserListOperandInfo} LogicalUserListOperandInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LogicalUserListOperandInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LogicalUserListOperandInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LogicalUserListOperandInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    p._user_list = 1;
                                    if (!$util.isString(m.user_list))
                                        return "user_list: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LogicalUserListOperandInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.LogicalUserListOperandInfo} LogicalUserListOperandInfo
                             */
                            LogicalUserListOperandInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.LogicalUserListOperandInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.LogicalUserListOperandInfo();
                                if (d.user_list != null) {
                                    m.user_list = String(d.user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LogicalUserListOperandInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.LogicalUserListOperandInfo} m LogicalUserListOperandInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LogicalUserListOperandInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = m.user_list;
                                    if (o.oneofs)
                                        d._user_list = "user_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LogicalUserListOperandInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.LogicalUserListOperandInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LogicalUserListOperandInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LogicalUserListOperandInfo;
                        })();
    
                        common.BasicUserListInfo = (function() {
    
                            /**
                             * Properties of a BasicUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IBasicUserListInfo
                             * @property {Array.<google.ads.googleads.v11.common.IUserListActionInfo>|null} [actions] BasicUserListInfo actions
                             */
    
                            /**
                             * Constructs a new BasicUserListInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a BasicUserListInfo.
                             * @implements IBasicUserListInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IBasicUserListInfo=} [p] Properties to set
                             */
                            function BasicUserListInfo(p) {
                                this.actions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BasicUserListInfo actions.
                             * @member {Array.<google.ads.googleads.v11.common.IUserListActionInfo>} actions
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @instance
                             */
                            BasicUserListInfo.prototype.actions = $util.emptyArray;
    
                            /**
                             * Creates a new BasicUserListInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IBasicUserListInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.BasicUserListInfo} BasicUserListInfo instance
                             */
                            BasicUserListInfo.create = function create(properties) {
                                return new BasicUserListInfo(properties);
                            };
    
                            /**
                             * Encodes the specified BasicUserListInfo message. Does not implicitly {@link google.ads.googleads.v11.common.BasicUserListInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IBasicUserListInfo} m BasicUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BasicUserListInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.actions != null && m.actions.length) {
                                    for (var i = 0; i < m.actions.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserListActionInfo.encode(m.actions[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified BasicUserListInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.BasicUserListInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IBasicUserListInfo} message BasicUserListInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BasicUserListInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BasicUserListInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.BasicUserListInfo} BasicUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BasicUserListInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.BasicUserListInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.actions && m.actions.length))
                                            m.actions = [];
                                        m.actions.push($root.google.ads.googleads.v11.common.UserListActionInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BasicUserListInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.BasicUserListInfo} BasicUserListInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BasicUserListInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BasicUserListInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BasicUserListInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.actions != null && m.hasOwnProperty("actions")) {
                                    if (!Array.isArray(m.actions))
                                        return "actions: array expected";
                                    for (var i = 0; i < m.actions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserListActionInfo.verify(m.actions[i]);
                                            if (e)
                                                return "actions." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BasicUserListInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.BasicUserListInfo} BasicUserListInfo
                             */
                            BasicUserListInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.BasicUserListInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.BasicUserListInfo();
                                if (d.actions) {
                                    if (!Array.isArray(d.actions))
                                        throw TypeError(".google.ads.googleads.v11.common.BasicUserListInfo.actions: array expected");
                                    m.actions = [];
                                    for (var i = 0; i < d.actions.length; ++i) {
                                        if (typeof d.actions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.common.BasicUserListInfo.actions: object expected");
                                        m.actions[i] = $root.google.ads.googleads.v11.common.UserListActionInfo.fromObject(d.actions[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BasicUserListInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.BasicUserListInfo} m BasicUserListInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BasicUserListInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.actions = [];
                                }
                                if (m.actions && m.actions.length) {
                                    d.actions = [];
                                    for (var j = 0; j < m.actions.length; ++j) {
                                        d.actions[j] = $root.google.ads.googleads.v11.common.UserListActionInfo.toObject(m.actions[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BasicUserListInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.BasicUserListInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BasicUserListInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BasicUserListInfo;
                        })();
    
                        common.UserListActionInfo = (function() {
    
                            /**
                             * Properties of a UserListActionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IUserListActionInfo
                             * @property {string|null} [conversion_action] UserListActionInfo conversion_action
                             * @property {string|null} [remarketing_action] UserListActionInfo remarketing_action
                             */
    
                            /**
                             * Constructs a new UserListActionInfo.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a UserListActionInfo.
                             * @implements IUserListActionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IUserListActionInfo=} [p] Properties to set
                             */
                            function UserListActionInfo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListActionInfo conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @instance
                             */
                            UserListActionInfo.prototype.conversion_action = null;
    
                            /**
                             * UserListActionInfo remarketing_action.
                             * @member {string|null|undefined} remarketing_action
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @instance
                             */
                            UserListActionInfo.prototype.remarketing_action = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListActionInfo user_list_action.
                             * @member {"conversion_action"|"remarketing_action"|undefined} user_list_action
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @instance
                             */
                            Object.defineProperty(UserListActionInfo.prototype, "user_list_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action", "remarketing_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListActionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListActionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.UserListActionInfo} UserListActionInfo instance
                             */
                            UserListActionInfo.create = function create(properties) {
                                return new UserListActionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified UserListActionInfo message. Does not implicitly {@link google.ads.googleads.v11.common.UserListActionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListActionInfo} m UserListActionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListActionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(26).string(m.conversion_action);
                                if (m.remarketing_action != null && Object.hasOwnProperty.call(m, "remarketing_action"))
                                    w.uint32(34).string(m.remarketing_action);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListActionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.UserListActionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.IUserListActionInfo} message UserListActionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListActionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListActionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.UserListActionInfo} UserListActionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListActionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.UserListActionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.conversion_action = r.string();
                                        break;
                                    case 4:
                                        m.remarketing_action = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListActionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.UserListActionInfo} UserListActionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListActionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListActionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListActionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p.user_list_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.remarketing_action != null && m.hasOwnProperty("remarketing_action")) {
                                    if (p.user_list_action === 1)
                                        return "user_list_action: multiple values";
                                    p.user_list_action = 1;
                                    if (!$util.isString(m.remarketing_action))
                                        return "remarketing_action: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListActionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.UserListActionInfo} UserListActionInfo
                             */
                            UserListActionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.UserListActionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.UserListActionInfo();
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.remarketing_action != null) {
                                    m.remarketing_action = String(d.remarketing_action);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListActionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @static
                             * @param {google.ads.googleads.v11.common.UserListActionInfo} m UserListActionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListActionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d.user_list_action = "conversion_action";
                                }
                                if (m.remarketing_action != null && m.hasOwnProperty("remarketing_action")) {
                                    d.remarketing_action = m.remarketing_action;
                                    if (o.oneofs)
                                        d.user_list_action = "remarketing_action";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListActionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.UserListActionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListActionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListActionInfo;
                        })();
    
                        common.Value = (function() {
    
                            /**
                             * Properties of a Value.
                             * @memberof google.ads.googleads.v11.common
                             * @interface IValue
                             * @property {boolean|null} [boolean_value] Value boolean_value
                             * @property {number|null} [int64_value] Value int64_value
                             * @property {number|null} [float_value] Value float_value
                             * @property {number|null} [double_value] Value double_value
                             * @property {string|null} [string_value] Value string_value
                             */
    
                            /**
                             * Constructs a new Value.
                             * @memberof google.ads.googleads.v11.common
                             * @classdesc Represents a Value.
                             * @implements IValue
                             * @constructor
                             * @param {google.ads.googleads.v11.common.IValue=} [p] Properties to set
                             */
                            function Value(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Value boolean_value.
                             * @member {boolean|null|undefined} boolean_value
                             * @memberof google.ads.googleads.v11.common.Value
                             * @instance
                             */
                            Value.prototype.boolean_value = null;
    
                            /**
                             * Value int64_value.
                             * @member {number|null|undefined} int64_value
                             * @memberof google.ads.googleads.v11.common.Value
                             * @instance
                             */
                            Value.prototype.int64_value = null;
    
                            /**
                             * Value float_value.
                             * @member {number|null|undefined} float_value
                             * @memberof google.ads.googleads.v11.common.Value
                             * @instance
                             */
                            Value.prototype.float_value = null;
    
                            /**
                             * Value double_value.
                             * @member {number|null|undefined} double_value
                             * @memberof google.ads.googleads.v11.common.Value
                             * @instance
                             */
                            Value.prototype.double_value = null;
    
                            /**
                             * Value string_value.
                             * @member {string|null|undefined} string_value
                             * @memberof google.ads.googleads.v11.common.Value
                             * @instance
                             */
                            Value.prototype.string_value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Value value.
                             * @member {"boolean_value"|"int64_value"|"float_value"|"double_value"|"string_value"|undefined} value
                             * @memberof google.ads.googleads.v11.common.Value
                             * @instance
                             */
                            Object.defineProperty(Value.prototype, "value", {
                                get: $util.oneOfGetter($oneOfFields = ["boolean_value", "int64_value", "float_value", "double_value", "string_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Value instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {google.ads.googleads.v11.common.IValue=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.common.Value} Value instance
                             */
                            Value.create = function create(properties) {
                                return new Value(properties);
                            };
    
                            /**
                             * Encodes the specified Value message. Does not implicitly {@link google.ads.googleads.v11.common.Value.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {google.ads.googleads.v11.common.IValue} m Value message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Value.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.boolean_value != null && Object.hasOwnProperty.call(m, "boolean_value"))
                                    w.uint32(8).bool(m.boolean_value);
                                if (m.int64_value != null && Object.hasOwnProperty.call(m, "int64_value"))
                                    w.uint32(16).int64(m.int64_value);
                                if (m.float_value != null && Object.hasOwnProperty.call(m, "float_value"))
                                    w.uint32(29).float(m.float_value);
                                if (m.double_value != null && Object.hasOwnProperty.call(m, "double_value"))
                                    w.uint32(33).double(m.double_value);
                                if (m.string_value != null && Object.hasOwnProperty.call(m, "string_value"))
                                    w.uint32(42).string(m.string_value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Value message, length delimited. Does not implicitly {@link google.ads.googleads.v11.common.Value.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {google.ads.googleads.v11.common.IValue} message Value message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Value.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Value message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.common.Value} Value
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Value.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.common.Value();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.boolean_value = r.bool();
                                        break;
                                    case 2:
                                        m.int64_value = r.int64();
                                        break;
                                    case 3:
                                        m.float_value = r.float();
                                        break;
                                    case 4:
                                        m.double_value = r.double();
                                        break;
                                    case 5:
                                        m.string_value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Value message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.common.Value} Value
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Value.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Value message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Value.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.boolean_value != null && m.hasOwnProperty("boolean_value")) {
                                    p.value = 1;
                                    if (typeof m.boolean_value !== "boolean")
                                        return "boolean_value: boolean expected";
                                }
                                if (m.int64_value != null && m.hasOwnProperty("int64_value")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (!$util.isInteger(m.int64_value) && !(m.int64_value && $util.isInteger(m.int64_value.low) && $util.isInteger(m.int64_value.high)))
                                        return "int64_value: integer|Long expected";
                                }
                                if (m.float_value != null && m.hasOwnProperty("float_value")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (typeof m.float_value !== "number")
                                        return "float_value: number expected";
                                }
                                if (m.double_value != null && m.hasOwnProperty("double_value")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (typeof m.double_value !== "number")
                                        return "double_value: number expected";
                                }
                                if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (!$util.isString(m.string_value))
                                        return "string_value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Value message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.common.Value} Value
                             */
                            Value.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.common.Value)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.common.Value();
                                if (d.boolean_value != null) {
                                    m.boolean_value = Boolean(d.boolean_value);
                                }
                                if (d.int64_value != null) {
                                    if ($util.Long)
                                        (m.int64_value = $util.Long.fromValue(d.int64_value)).unsigned = false;
                                    else if (typeof d.int64_value === "string")
                                        m.int64_value = parseInt(d.int64_value, 10);
                                    else if (typeof d.int64_value === "number")
                                        m.int64_value = d.int64_value;
                                    else if (typeof d.int64_value === "object")
                                        m.int64_value = new $util.LongBits(d.int64_value.low >>> 0, d.int64_value.high >>> 0).toNumber();
                                }
                                if (d.float_value != null) {
                                    m.float_value = Number(d.float_value);
                                }
                                if (d.double_value != null) {
                                    m.double_value = Number(d.double_value);
                                }
                                if (d.string_value != null) {
                                    m.string_value = String(d.string_value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Value message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.common.Value
                             * @static
                             * @param {google.ads.googleads.v11.common.Value} m Value
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Value.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.boolean_value != null && m.hasOwnProperty("boolean_value")) {
                                    d.boolean_value = m.boolean_value;
                                    if (o.oneofs)
                                        d.value = "boolean_value";
                                }
                                if (m.int64_value != null && m.hasOwnProperty("int64_value")) {
                                    if (typeof m.int64_value === "number")
                                        d.int64_value = o.longs === String ? String(m.int64_value) : m.int64_value;
                                    else
                                        d.int64_value = o.longs === String ? $util.Long.prototype.toString.call(m.int64_value) : o.longs === Number ? new $util.LongBits(m.int64_value.low >>> 0, m.int64_value.high >>> 0).toNumber() : m.int64_value;
                                    if (o.oneofs)
                                        d.value = "int64_value";
                                }
                                if (m.float_value != null && m.hasOwnProperty("float_value")) {
                                    d.float_value = o.json && !isFinite(m.float_value) ? String(m.float_value) : m.float_value;
                                    if (o.oneofs)
                                        d.value = "float_value";
                                }
                                if (m.double_value != null && m.hasOwnProperty("double_value")) {
                                    d.double_value = o.json && !isFinite(m.double_value) ? String(m.double_value) : m.double_value;
                                    if (o.oneofs)
                                        d.value = "double_value";
                                }
                                if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                    d.string_value = m.string_value;
                                    if (o.oneofs)
                                        d.value = "string_value";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Value to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.common.Value
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Value.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Value;
                        })();
    
                        return common;
                    })();
    
                    v11.enums = (function() {
    
                        /**
                         * Namespace enums.
                         * @memberof google.ads.googleads.v11
                         * @namespace
                         */
                        var enums = {};
    
                        enums.PolicyTopicEntryTypeEnum = (function() {
    
                            /**
                             * Properties of a PolicyTopicEntryTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPolicyTopicEntryTypeEnum
                             */
    
                            /**
                             * Constructs a new PolicyTopicEntryTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PolicyTopicEntryTypeEnum.
                             * @implements IPolicyTopicEntryTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEntryTypeEnum=} [p] Properties to set
                             */
                            function PolicyTopicEntryTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyTopicEntryTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEntryTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum} PolicyTopicEntryTypeEnum instance
                             */
                            PolicyTopicEntryTypeEnum.create = function create(properties) {
                                return new PolicyTopicEntryTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEntryTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEntryTypeEnum} m PolicyTopicEntryTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEntryTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEntryTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEntryTypeEnum} message PolicyTopicEntryTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEntryTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyTopicEntryTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum} PolicyTopicEntryTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEntryTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyTopicEntryTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum} PolicyTopicEntryTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEntryTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyTopicEntryTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyTopicEntryTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyTopicEntryTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum} PolicyTopicEntryTypeEnum
                             */
                            PolicyTopicEntryTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyTopicEntryTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum} m PolicyTopicEntryTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyTopicEntryTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyTopicEntryTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyTopicEntryTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyTopicEntryType enum.
                             * @name google.ads.googleads.v11.enums.PolicyTopicEntryTypeEnum.PolicyTopicEntryType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PROHIBITED=2 PROHIBITED value
                             * @property {number} LIMITED=4 LIMITED value
                             * @property {number} FULLY_LIMITED=8 FULLY_LIMITED value
                             * @property {number} DESCRIPTIVE=5 DESCRIPTIVE value
                             * @property {number} BROADENING=6 BROADENING value
                             * @property {number} AREA_OF_INTEREST_ONLY=7 AREA_OF_INTEREST_ONLY value
                             */
                            PolicyTopicEntryTypeEnum.PolicyTopicEntryType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PROHIBITED"] = 2;
                                values[valuesById[4] = "LIMITED"] = 4;
                                values[valuesById[8] = "FULLY_LIMITED"] = 8;
                                values[valuesById[5] = "DESCRIPTIVE"] = 5;
                                values[valuesById[6] = "BROADENING"] = 6;
                                values[valuesById[7] = "AREA_OF_INTEREST_ONLY"] = 7;
                                return values;
                            })();
    
                            return PolicyTopicEntryTypeEnum;
                        })();
    
                        enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum = (function() {
    
                            /**
                             * Properties of a PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             */
    
                            /**
                             * Constructs a new PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.
                             * @implements IPolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationMismatchUrlTypeEnum=} [p] Properties to set
                             */
                            function PolicyTopicEvidenceDestinationMismatchUrlTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyTopicEvidenceDestinationMismatchUrlTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationMismatchUrlTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum} PolicyTopicEvidenceDestinationMismatchUrlTypeEnum instance
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.create = function create(properties) {
                                return new PolicyTopicEvidenceDestinationMismatchUrlTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationMismatchUrlTypeEnum} m PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationMismatchUrlTypeEnum} message PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum} PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum} PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum} PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyTopicEvidenceDestinationMismatchUrlTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum} m PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyTopicEvidenceDestinationMismatchUrlTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyTopicEvidenceDestinationMismatchUrlType enum.
                             * @name google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DISPLAY_URL=2 DISPLAY_URL value
                             * @property {number} FINAL_URL=3 FINAL_URL value
                             * @property {number} FINAL_MOBILE_URL=4 FINAL_MOBILE_URL value
                             * @property {number} TRACKING_URL=5 TRACKING_URL value
                             * @property {number} MOBILE_TRACKING_URL=6 MOBILE_TRACKING_URL value
                             */
                            PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DISPLAY_URL"] = 2;
                                values[valuesById[3] = "FINAL_URL"] = 3;
                                values[valuesById[4] = "FINAL_MOBILE_URL"] = 4;
                                values[valuesById[5] = "TRACKING_URL"] = 5;
                                values[valuesById[6] = "MOBILE_TRACKING_URL"] = 6;
                                return values;
                            })();
    
                            return PolicyTopicEvidenceDestinationMismatchUrlTypeEnum;
                        })();
    
                        enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum = (function() {
    
                            /**
                             * Properties of a PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             */
    
                            /**
                             * Constructs a new PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.
                             * @implements IPolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDeviceEnum=} [p] Properties to set
                             */
                            function PolicyTopicEvidenceDestinationNotWorkingDeviceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyTopicEvidenceDestinationNotWorkingDeviceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDeviceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum} PolicyTopicEvidenceDestinationNotWorkingDeviceEnum instance
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.create = function create(properties) {
                                return new PolicyTopicEvidenceDestinationNotWorkingDeviceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDeviceEnum} m PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDeviceEnum} message PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum} PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum} PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum} PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyTopicEvidenceDestinationNotWorkingDeviceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum} m PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyTopicEvidenceDestinationNotWorkingDeviceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyTopicEvidenceDestinationNotWorkingDevice enum.
                             * @name google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DESKTOP=2 DESKTOP value
                             * @property {number} ANDROID=3 ANDROID value
                             * @property {number} IOS=4 IOS value
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DESKTOP"] = 2;
                                values[valuesById[3] = "ANDROID"] = 3;
                                values[valuesById[4] = "IOS"] = 4;
                                return values;
                            })();
    
                            return PolicyTopicEvidenceDestinationNotWorkingDeviceEnum;
                        })();
    
                        enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum = (function() {
    
                            /**
                             * Properties of a PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             */
    
                            /**
                             * Constructs a new PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.
                             * @implements IPolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum=} [p] Properties to set
                             */
                            function PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum} PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum instance
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.create = function create(properties) {
                                return new PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum} m PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum} message PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum} PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum} PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum} PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum} m PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyTopicEvidenceDestinationNotWorkingDnsErrorType enum.
                             * @name google.ads.googleads.v11.enums.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} HOSTNAME_NOT_FOUND=2 HOSTNAME_NOT_FOUND value
                             * @property {number} GOOGLE_CRAWLER_DNS_ISSUE=3 GOOGLE_CRAWLER_DNS_ISSUE value
                             */
                            PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "HOSTNAME_NOT_FOUND"] = 2;
                                values[valuesById[3] = "GOOGLE_CRAWLER_DNS_ISSUE"] = 3;
                                return values;
                            })();
    
                            return PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum;
                        })();
    
                        enums.PolicyApprovalStatusEnum = (function() {
    
                            /**
                             * Properties of a PolicyApprovalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPolicyApprovalStatusEnum
                             */
    
                            /**
                             * Constructs a new PolicyApprovalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PolicyApprovalStatusEnum.
                             * @implements IPolicyApprovalStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPolicyApprovalStatusEnum=} [p] Properties to set
                             */
                            function PolicyApprovalStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyApprovalStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyApprovalStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum} PolicyApprovalStatusEnum instance
                             */
                            PolicyApprovalStatusEnum.create = function create(properties) {
                                return new PolicyApprovalStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyApprovalStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyApprovalStatusEnum} m PolicyApprovalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyApprovalStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyApprovalStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyApprovalStatusEnum} message PolicyApprovalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyApprovalStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyApprovalStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum} PolicyApprovalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyApprovalStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyApprovalStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum} PolicyApprovalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyApprovalStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyApprovalStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyApprovalStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyApprovalStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum} PolicyApprovalStatusEnum
                             */
                            PolicyApprovalStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyApprovalStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum} m PolicyApprovalStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyApprovalStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyApprovalStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PolicyApprovalStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyApprovalStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyApprovalStatus enum.
                             * @name google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DISAPPROVED=2 DISAPPROVED value
                             * @property {number} APPROVED_LIMITED=3 APPROVED_LIMITED value
                             * @property {number} APPROVED=4 APPROVED value
                             * @property {number} AREA_OF_INTEREST_ONLY=5 AREA_OF_INTEREST_ONLY value
                             */
                            PolicyApprovalStatusEnum.PolicyApprovalStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DISAPPROVED"] = 2;
                                values[valuesById[3] = "APPROVED_LIMITED"] = 3;
                                values[valuesById[4] = "APPROVED"] = 4;
                                values[valuesById[5] = "AREA_OF_INTEREST_ONLY"] = 5;
                                return values;
                            })();
    
                            return PolicyApprovalStatusEnum;
                        })();
    
                        enums.PolicyReviewStatusEnum = (function() {
    
                            /**
                             * Properties of a PolicyReviewStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPolicyReviewStatusEnum
                             */
    
                            /**
                             * Constructs a new PolicyReviewStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PolicyReviewStatusEnum.
                             * @implements IPolicyReviewStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPolicyReviewStatusEnum=} [p] Properties to set
                             */
                            function PolicyReviewStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyReviewStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyReviewStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PolicyReviewStatusEnum} PolicyReviewStatusEnum instance
                             */
                            PolicyReviewStatusEnum.create = function create(properties) {
                                return new PolicyReviewStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyReviewStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyReviewStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyReviewStatusEnum} m PolicyReviewStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyReviewStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyReviewStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PolicyReviewStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPolicyReviewStatusEnum} message PolicyReviewStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyReviewStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyReviewStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PolicyReviewStatusEnum} PolicyReviewStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyReviewStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyReviewStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PolicyReviewStatusEnum} PolicyReviewStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyReviewStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyReviewStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyReviewStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyReviewStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PolicyReviewStatusEnum} PolicyReviewStatusEnum
                             */
                            PolicyReviewStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyReviewStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PolicyReviewStatusEnum} m PolicyReviewStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyReviewStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyReviewStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PolicyReviewStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyReviewStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyReviewStatus enum.
                             * @name google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} REVIEW_IN_PROGRESS=2 REVIEW_IN_PROGRESS value
                             * @property {number} REVIEWED=3 REVIEWED value
                             * @property {number} UNDER_APPEAL=4 UNDER_APPEAL value
                             * @property {number} ELIGIBLE_MAY_SERVE=5 ELIGIBLE_MAY_SERVE value
                             */
                            PolicyReviewStatusEnum.PolicyReviewStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "REVIEW_IN_PROGRESS"] = 2;
                                values[valuesById[3] = "REVIEWED"] = 3;
                                values[valuesById[4] = "UNDER_APPEAL"] = 4;
                                values[valuesById[5] = "ELIGIBLE_MAY_SERVE"] = 5;
                                return values;
                            })();
    
                            return PolicyReviewStatusEnum;
                        })();
    
                        enums.AssetPerformanceLabelEnum = (function() {
    
                            /**
                             * Properties of an AssetPerformanceLabelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetPerformanceLabelEnum
                             */
    
                            /**
                             * Constructs a new AssetPerformanceLabelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetPerformanceLabelEnum.
                             * @implements IAssetPerformanceLabelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetPerformanceLabelEnum=} [p] Properties to set
                             */
                            function AssetPerformanceLabelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetPerformanceLabelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetPerformanceLabelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum} AssetPerformanceLabelEnum instance
                             */
                            AssetPerformanceLabelEnum.create = function create(properties) {
                                return new AssetPerformanceLabelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetPerformanceLabelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetPerformanceLabelEnum} m AssetPerformanceLabelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetPerformanceLabelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetPerformanceLabelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetPerformanceLabelEnum} message AssetPerformanceLabelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetPerformanceLabelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetPerformanceLabelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum} AssetPerformanceLabelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetPerformanceLabelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetPerformanceLabelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetPerformanceLabelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum} AssetPerformanceLabelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetPerformanceLabelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetPerformanceLabelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetPerformanceLabelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetPerformanceLabelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum} AssetPerformanceLabelEnum
                             */
                            AssetPerformanceLabelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetPerformanceLabelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetPerformanceLabelEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetPerformanceLabelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum} m AssetPerformanceLabelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetPerformanceLabelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetPerformanceLabelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetPerformanceLabelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetPerformanceLabelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetPerformanceLabel enum.
                             * @name google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} LEARNING=3 LEARNING value
                             * @property {number} LOW=4 LOW value
                             * @property {number} GOOD=5 GOOD value
                             * @property {number} BEST=6 BEST value
                             */
                            AssetPerformanceLabelEnum.AssetPerformanceLabel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "LEARNING"] = 3;
                                values[valuesById[4] = "LOW"] = 4;
                                values[valuesById[5] = "GOOD"] = 5;
                                values[valuesById[6] = "BEST"] = 6;
                                return values;
                            })();
    
                            return AssetPerformanceLabelEnum;
                        })();
    
                        enums.ServedAssetFieldTypeEnum = (function() {
    
                            /**
                             * Properties of a ServedAssetFieldTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IServedAssetFieldTypeEnum
                             */
    
                            /**
                             * Constructs a new ServedAssetFieldTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ServedAssetFieldTypeEnum.
                             * @implements IServedAssetFieldTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IServedAssetFieldTypeEnum=} [p] Properties to set
                             */
                            function ServedAssetFieldTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ServedAssetFieldTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IServedAssetFieldTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum} ServedAssetFieldTypeEnum instance
                             */
                            ServedAssetFieldTypeEnum.create = function create(properties) {
                                return new ServedAssetFieldTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ServedAssetFieldTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IServedAssetFieldTypeEnum} m ServedAssetFieldTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ServedAssetFieldTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ServedAssetFieldTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IServedAssetFieldTypeEnum} message ServedAssetFieldTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ServedAssetFieldTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ServedAssetFieldTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum} ServedAssetFieldTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ServedAssetFieldTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ServedAssetFieldTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum} ServedAssetFieldTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ServedAssetFieldTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ServedAssetFieldTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ServedAssetFieldTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ServedAssetFieldTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum} ServedAssetFieldTypeEnum
                             */
                            ServedAssetFieldTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ServedAssetFieldTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum} m ServedAssetFieldTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ServedAssetFieldTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ServedAssetFieldTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ServedAssetFieldTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ServedAssetFieldType enum.
                             * @name google.ads.googleads.v11.enums.ServedAssetFieldTypeEnum.ServedAssetFieldType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} HEADLINE_1=2 HEADLINE_1 value
                             * @property {number} HEADLINE_2=3 HEADLINE_2 value
                             * @property {number} HEADLINE_3=4 HEADLINE_3 value
                             * @property {number} DESCRIPTION_1=5 DESCRIPTION_1 value
                             * @property {number} DESCRIPTION_2=6 DESCRIPTION_2 value
                             */
                            ServedAssetFieldTypeEnum.ServedAssetFieldType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "HEADLINE_1"] = 2;
                                values[valuesById[3] = "HEADLINE_2"] = 3;
                                values[valuesById[4] = "HEADLINE_3"] = 4;
                                values[valuesById[5] = "DESCRIPTION_1"] = 5;
                                values[valuesById[6] = "DESCRIPTION_2"] = 6;
                                return values;
                            })();
    
                            return ServedAssetFieldTypeEnum;
                        })();
    
                        enums.CallConversionReportingStateEnum = (function() {
    
                            /**
                             * Properties of a CallConversionReportingStateEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICallConversionReportingStateEnum
                             */
    
                            /**
                             * Constructs a new CallConversionReportingStateEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CallConversionReportingStateEnum.
                             * @implements ICallConversionReportingStateEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICallConversionReportingStateEnum=} [p] Properties to set
                             */
                            function CallConversionReportingStateEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CallConversionReportingStateEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallConversionReportingStateEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CallConversionReportingStateEnum} CallConversionReportingStateEnum instance
                             */
                            CallConversionReportingStateEnum.create = function create(properties) {
                                return new CallConversionReportingStateEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CallConversionReportingStateEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CallConversionReportingStateEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallConversionReportingStateEnum} m CallConversionReportingStateEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallConversionReportingStateEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallConversionReportingStateEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CallConversionReportingStateEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallConversionReportingStateEnum} message CallConversionReportingStateEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallConversionReportingStateEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallConversionReportingStateEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CallConversionReportingStateEnum} CallConversionReportingStateEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallConversionReportingStateEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CallConversionReportingStateEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallConversionReportingStateEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CallConversionReportingStateEnum} CallConversionReportingStateEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallConversionReportingStateEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallConversionReportingStateEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallConversionReportingStateEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CallConversionReportingStateEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CallConversionReportingStateEnum} CallConversionReportingStateEnum
                             */
                            CallConversionReportingStateEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CallConversionReportingStateEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CallConversionReportingStateEnum();
                            };
    
                            /**
                             * Creates a plain object from a CallConversionReportingStateEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CallConversionReportingStateEnum} m CallConversionReportingStateEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallConversionReportingStateEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CallConversionReportingStateEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CallConversionReportingStateEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallConversionReportingStateEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CallConversionReportingState enum.
                             * @name google.ads.googleads.v11.enums.CallConversionReportingStateEnum.CallConversionReportingState
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DISABLED=2 DISABLED value
                             * @property {number} USE_ACCOUNT_LEVEL_CALL_CONVERSION_ACTION=3 USE_ACCOUNT_LEVEL_CALL_CONVERSION_ACTION value
                             * @property {number} USE_RESOURCE_LEVEL_CALL_CONVERSION_ACTION=4 USE_RESOURCE_LEVEL_CALL_CONVERSION_ACTION value
                             */
                            CallConversionReportingStateEnum.CallConversionReportingState = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DISABLED"] = 2;
                                values[valuesById[3] = "USE_ACCOUNT_LEVEL_CALL_CONVERSION_ACTION"] = 3;
                                values[valuesById[4] = "USE_RESOURCE_LEVEL_CALL_CONVERSION_ACTION"] = 4;
                                return values;
                            })();
    
                            return CallConversionReportingStateEnum;
                        })();
    
                        enums.DisplayAdFormatSettingEnum = (function() {
    
                            /**
                             * Properties of a DisplayAdFormatSettingEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IDisplayAdFormatSettingEnum
                             */
    
                            /**
                             * Constructs a new DisplayAdFormatSettingEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a DisplayAdFormatSettingEnum.
                             * @implements IDisplayAdFormatSettingEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IDisplayAdFormatSettingEnum=} [p] Properties to set
                             */
                            function DisplayAdFormatSettingEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DisplayAdFormatSettingEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDisplayAdFormatSettingEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum} DisplayAdFormatSettingEnum instance
                             */
                            DisplayAdFormatSettingEnum.create = function create(properties) {
                                return new DisplayAdFormatSettingEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DisplayAdFormatSettingEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDisplayAdFormatSettingEnum} m DisplayAdFormatSettingEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayAdFormatSettingEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DisplayAdFormatSettingEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDisplayAdFormatSettingEnum} message DisplayAdFormatSettingEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayAdFormatSettingEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DisplayAdFormatSettingEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum} DisplayAdFormatSettingEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayAdFormatSettingEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DisplayAdFormatSettingEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum} DisplayAdFormatSettingEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayAdFormatSettingEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DisplayAdFormatSettingEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DisplayAdFormatSettingEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DisplayAdFormatSettingEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum} DisplayAdFormatSettingEnum
                             */
                            DisplayAdFormatSettingEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum();
                            };
    
                            /**
                             * Creates a plain object from a DisplayAdFormatSettingEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum} m DisplayAdFormatSettingEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DisplayAdFormatSettingEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DisplayAdFormatSettingEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DisplayAdFormatSettingEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DisplayAdFormatSetting enum.
                             * @name google.ads.googleads.v11.enums.DisplayAdFormatSettingEnum.DisplayAdFormatSetting
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ALL_FORMATS=2 ALL_FORMATS value
                             * @property {number} NON_NATIVE=3 NON_NATIVE value
                             * @property {number} NATIVE=4 NATIVE value
                             */
                            DisplayAdFormatSettingEnum.DisplayAdFormatSetting = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ALL_FORMATS"] = 2;
                                values[valuesById[3] = "NON_NATIVE"] = 3;
                                values[valuesById[4] = "NATIVE"] = 4;
                                return values;
                            })();
    
                            return DisplayAdFormatSettingEnum;
                        })();
    
                        enums.DisplayUploadProductTypeEnum = (function() {
    
                            /**
                             * Properties of a DisplayUploadProductTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IDisplayUploadProductTypeEnum
                             */
    
                            /**
                             * Constructs a new DisplayUploadProductTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a DisplayUploadProductTypeEnum.
                             * @implements IDisplayUploadProductTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IDisplayUploadProductTypeEnum=} [p] Properties to set
                             */
                            function DisplayUploadProductTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DisplayUploadProductTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDisplayUploadProductTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum} DisplayUploadProductTypeEnum instance
                             */
                            DisplayUploadProductTypeEnum.create = function create(properties) {
                                return new DisplayUploadProductTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DisplayUploadProductTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDisplayUploadProductTypeEnum} m DisplayUploadProductTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayUploadProductTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DisplayUploadProductTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDisplayUploadProductTypeEnum} message DisplayUploadProductTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayUploadProductTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DisplayUploadProductTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum} DisplayUploadProductTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayUploadProductTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DisplayUploadProductTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum} DisplayUploadProductTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayUploadProductTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DisplayUploadProductTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DisplayUploadProductTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DisplayUploadProductTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum} DisplayUploadProductTypeEnum
                             */
                            DisplayUploadProductTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a DisplayUploadProductTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum} m DisplayUploadProductTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DisplayUploadProductTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DisplayUploadProductTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DisplayUploadProductTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DisplayUploadProductType enum.
                             * @name google.ads.googleads.v11.enums.DisplayUploadProductTypeEnum.DisplayUploadProductType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} HTML5_UPLOAD_AD=2 HTML5_UPLOAD_AD value
                             * @property {number} DYNAMIC_HTML5_EDUCATION_AD=3 DYNAMIC_HTML5_EDUCATION_AD value
                             * @property {number} DYNAMIC_HTML5_FLIGHT_AD=4 DYNAMIC_HTML5_FLIGHT_AD value
                             * @property {number} DYNAMIC_HTML5_HOTEL_RENTAL_AD=5 DYNAMIC_HTML5_HOTEL_RENTAL_AD value
                             * @property {number} DYNAMIC_HTML5_JOB_AD=6 DYNAMIC_HTML5_JOB_AD value
                             * @property {number} DYNAMIC_HTML5_LOCAL_AD=7 DYNAMIC_HTML5_LOCAL_AD value
                             * @property {number} DYNAMIC_HTML5_REAL_ESTATE_AD=8 DYNAMIC_HTML5_REAL_ESTATE_AD value
                             * @property {number} DYNAMIC_HTML5_CUSTOM_AD=9 DYNAMIC_HTML5_CUSTOM_AD value
                             * @property {number} DYNAMIC_HTML5_TRAVEL_AD=10 DYNAMIC_HTML5_TRAVEL_AD value
                             * @property {number} DYNAMIC_HTML5_HOTEL_AD=11 DYNAMIC_HTML5_HOTEL_AD value
                             */
                            DisplayUploadProductTypeEnum.DisplayUploadProductType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "HTML5_UPLOAD_AD"] = 2;
                                values[valuesById[3] = "DYNAMIC_HTML5_EDUCATION_AD"] = 3;
                                values[valuesById[4] = "DYNAMIC_HTML5_FLIGHT_AD"] = 4;
                                values[valuesById[5] = "DYNAMIC_HTML5_HOTEL_RENTAL_AD"] = 5;
                                values[valuesById[6] = "DYNAMIC_HTML5_JOB_AD"] = 6;
                                values[valuesById[7] = "DYNAMIC_HTML5_LOCAL_AD"] = 7;
                                values[valuesById[8] = "DYNAMIC_HTML5_REAL_ESTATE_AD"] = 8;
                                values[valuesById[9] = "DYNAMIC_HTML5_CUSTOM_AD"] = 9;
                                values[valuesById[10] = "DYNAMIC_HTML5_TRAVEL_AD"] = 10;
                                values[valuesById[11] = "DYNAMIC_HTML5_HOTEL_AD"] = 11;
                                return values;
                            })();
    
                            return DisplayUploadProductTypeEnum;
                        })();
    
                        enums.LegacyAppInstallAdAppStoreEnum = (function() {
    
                            /**
                             * Properties of a LegacyAppInstallAdAppStoreEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILegacyAppInstallAdAppStoreEnum
                             */
    
                            /**
                             * Constructs a new LegacyAppInstallAdAppStoreEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LegacyAppInstallAdAppStoreEnum.
                             * @implements ILegacyAppInstallAdAppStoreEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILegacyAppInstallAdAppStoreEnum=} [p] Properties to set
                             */
                            function LegacyAppInstallAdAppStoreEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LegacyAppInstallAdAppStoreEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILegacyAppInstallAdAppStoreEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum} LegacyAppInstallAdAppStoreEnum instance
                             */
                            LegacyAppInstallAdAppStoreEnum.create = function create(properties) {
                                return new LegacyAppInstallAdAppStoreEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LegacyAppInstallAdAppStoreEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILegacyAppInstallAdAppStoreEnum} m LegacyAppInstallAdAppStoreEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LegacyAppInstallAdAppStoreEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LegacyAppInstallAdAppStoreEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILegacyAppInstallAdAppStoreEnum} message LegacyAppInstallAdAppStoreEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LegacyAppInstallAdAppStoreEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LegacyAppInstallAdAppStoreEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum} LegacyAppInstallAdAppStoreEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LegacyAppInstallAdAppStoreEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LegacyAppInstallAdAppStoreEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum} LegacyAppInstallAdAppStoreEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LegacyAppInstallAdAppStoreEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LegacyAppInstallAdAppStoreEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LegacyAppInstallAdAppStoreEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LegacyAppInstallAdAppStoreEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum} LegacyAppInstallAdAppStoreEnum
                             */
                            LegacyAppInstallAdAppStoreEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum();
                            };
    
                            /**
                             * Creates a plain object from a LegacyAppInstallAdAppStoreEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum} m LegacyAppInstallAdAppStoreEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LegacyAppInstallAdAppStoreEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LegacyAppInstallAdAppStoreEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LegacyAppInstallAdAppStoreEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LegacyAppInstallAdAppStore enum.
                             * @name google.ads.googleads.v11.enums.LegacyAppInstallAdAppStoreEnum.LegacyAppInstallAdAppStore
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APPLE_APP_STORE=2 APPLE_APP_STORE value
                             * @property {number} GOOGLE_PLAY=3 GOOGLE_PLAY value
                             * @property {number} WINDOWS_STORE=4 WINDOWS_STORE value
                             * @property {number} WINDOWS_PHONE_STORE=5 WINDOWS_PHONE_STORE value
                             * @property {number} CN_APP_STORE=6 CN_APP_STORE value
                             */
                            LegacyAppInstallAdAppStoreEnum.LegacyAppInstallAdAppStore = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APPLE_APP_STORE"] = 2;
                                values[valuesById[3] = "GOOGLE_PLAY"] = 3;
                                values[valuesById[4] = "WINDOWS_STORE"] = 4;
                                values[valuesById[5] = "WINDOWS_PHONE_STORE"] = 5;
                                values[valuesById[6] = "CN_APP_STORE"] = 6;
                                return values;
                            })();
    
                            return LegacyAppInstallAdAppStoreEnum;
                        })();
    
                        enums.MimeTypeEnum = (function() {
    
                            /**
                             * Properties of a MimeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMimeTypeEnum
                             */
    
                            /**
                             * Constructs a new MimeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MimeTypeEnum.
                             * @implements IMimeTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMimeTypeEnum=} [p] Properties to set
                             */
                            function MimeTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MimeTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMimeTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MimeTypeEnum} MimeTypeEnum instance
                             */
                            MimeTypeEnum.create = function create(properties) {
                                return new MimeTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MimeTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MimeTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMimeTypeEnum} m MimeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MimeTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MimeTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MimeTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMimeTypeEnum} message MimeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MimeTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MimeTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MimeTypeEnum} MimeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MimeTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MimeTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MimeTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MimeTypeEnum} MimeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MimeTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MimeTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MimeTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MimeTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MimeTypeEnum} MimeTypeEnum
                             */
                            MimeTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MimeTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MimeTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a MimeTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MimeTypeEnum} m MimeTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MimeTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MimeTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MimeTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MimeTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MimeType enum.
                             * @name google.ads.googleads.v11.enums.MimeTypeEnum.MimeType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} IMAGE_JPEG=2 IMAGE_JPEG value
                             * @property {number} IMAGE_GIF=3 IMAGE_GIF value
                             * @property {number} IMAGE_PNG=4 IMAGE_PNG value
                             * @property {number} FLASH=5 FLASH value
                             * @property {number} TEXT_HTML=6 TEXT_HTML value
                             * @property {number} PDF=7 PDF value
                             * @property {number} MSWORD=8 MSWORD value
                             * @property {number} MSEXCEL=9 MSEXCEL value
                             * @property {number} RTF=10 RTF value
                             * @property {number} AUDIO_WAV=11 AUDIO_WAV value
                             * @property {number} AUDIO_MP3=12 AUDIO_MP3 value
                             * @property {number} HTML5_AD_ZIP=13 HTML5_AD_ZIP value
                             */
                            MimeTypeEnum.MimeType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "IMAGE_JPEG"] = 2;
                                values[valuesById[3] = "IMAGE_GIF"] = 3;
                                values[valuesById[4] = "IMAGE_PNG"] = 4;
                                values[valuesById[5] = "FLASH"] = 5;
                                values[valuesById[6] = "TEXT_HTML"] = 6;
                                values[valuesById[7] = "PDF"] = 7;
                                values[valuesById[8] = "MSWORD"] = 8;
                                values[valuesById[9] = "MSEXCEL"] = 9;
                                values[valuesById[10] = "RTF"] = 10;
                                values[valuesById[11] = "AUDIO_WAV"] = 11;
                                values[valuesById[12] = "AUDIO_MP3"] = 12;
                                values[valuesById[13] = "HTML5_AD_ZIP"] = 13;
                                return values;
                            })();
    
                            return MimeTypeEnum;
                        })();
    
                        enums.VideoThumbnailEnum = (function() {
    
                            /**
                             * Properties of a VideoThumbnailEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IVideoThumbnailEnum
                             */
    
                            /**
                             * Constructs a new VideoThumbnailEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a VideoThumbnailEnum.
                             * @implements IVideoThumbnailEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IVideoThumbnailEnum=} [p] Properties to set
                             */
                            function VideoThumbnailEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new VideoThumbnailEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVideoThumbnailEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.VideoThumbnailEnum} VideoThumbnailEnum instance
                             */
                            VideoThumbnailEnum.create = function create(properties) {
                                return new VideoThumbnailEnum(properties);
                            };
    
                            /**
                             * Encodes the specified VideoThumbnailEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.VideoThumbnailEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVideoThumbnailEnum} m VideoThumbnailEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoThumbnailEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified VideoThumbnailEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.VideoThumbnailEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVideoThumbnailEnum} message VideoThumbnailEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VideoThumbnailEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VideoThumbnailEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.VideoThumbnailEnum} VideoThumbnailEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoThumbnailEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.VideoThumbnailEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VideoThumbnailEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.VideoThumbnailEnum} VideoThumbnailEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VideoThumbnailEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VideoThumbnailEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VideoThumbnailEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a VideoThumbnailEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.VideoThumbnailEnum} VideoThumbnailEnum
                             */
                            VideoThumbnailEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.VideoThumbnailEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.VideoThumbnailEnum();
                            };
    
                            /**
                             * Creates a plain object from a VideoThumbnailEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.VideoThumbnailEnum} m VideoThumbnailEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VideoThumbnailEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this VideoThumbnailEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.VideoThumbnailEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VideoThumbnailEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * VideoThumbnail enum.
                             * @name google.ads.googleads.v11.enums.VideoThumbnailEnum.VideoThumbnail
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DEFAULT_THUMBNAIL=2 DEFAULT_THUMBNAIL value
                             * @property {number} THUMBNAIL_1=3 THUMBNAIL_1 value
                             * @property {number} THUMBNAIL_2=4 THUMBNAIL_2 value
                             * @property {number} THUMBNAIL_3=5 THUMBNAIL_3 value
                             */
                            VideoThumbnailEnum.VideoThumbnail = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DEFAULT_THUMBNAIL"] = 2;
                                values[valuesById[3] = "THUMBNAIL_1"] = 3;
                                values[valuesById[4] = "THUMBNAIL_2"] = 4;
                                values[valuesById[5] = "THUMBNAIL_3"] = 5;
                                return values;
                            })();
    
                            return VideoThumbnailEnum;
                        })();
    
                        enums.AgeRangeTypeEnum = (function() {
    
                            /**
                             * Properties of an AgeRangeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAgeRangeTypeEnum
                             */
    
                            /**
                             * Constructs a new AgeRangeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AgeRangeTypeEnum.
                             * @implements IAgeRangeTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAgeRangeTypeEnum=} [p] Properties to set
                             */
                            function AgeRangeTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AgeRangeTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAgeRangeTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AgeRangeTypeEnum} AgeRangeTypeEnum instance
                             */
                            AgeRangeTypeEnum.create = function create(properties) {
                                return new AgeRangeTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AgeRangeTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AgeRangeTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAgeRangeTypeEnum} m AgeRangeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeRangeTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AgeRangeTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AgeRangeTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAgeRangeTypeEnum} message AgeRangeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeRangeTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AgeRangeTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AgeRangeTypeEnum} AgeRangeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeRangeTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AgeRangeTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AgeRangeTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AgeRangeTypeEnum} AgeRangeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeRangeTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AgeRangeTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AgeRangeTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AgeRangeTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AgeRangeTypeEnum} AgeRangeTypeEnum
                             */
                            AgeRangeTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AgeRangeTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AgeRangeTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AgeRangeTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AgeRangeTypeEnum} m AgeRangeTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AgeRangeTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AgeRangeTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AgeRangeTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AgeRangeTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AgeRangeType enum.
                             * @name google.ads.googleads.v11.enums.AgeRangeTypeEnum.AgeRangeType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AGE_RANGE_18_24=503001 AGE_RANGE_18_24 value
                             * @property {number} AGE_RANGE_25_34=503002 AGE_RANGE_25_34 value
                             * @property {number} AGE_RANGE_35_44=503003 AGE_RANGE_35_44 value
                             * @property {number} AGE_RANGE_45_54=503004 AGE_RANGE_45_54 value
                             * @property {number} AGE_RANGE_55_64=503005 AGE_RANGE_55_64 value
                             * @property {number} AGE_RANGE_65_UP=503006 AGE_RANGE_65_UP value
                             * @property {number} AGE_RANGE_UNDETERMINED=503999 AGE_RANGE_UNDETERMINED value
                             */
                            AgeRangeTypeEnum.AgeRangeType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[503001] = "AGE_RANGE_18_24"] = 503001;
                                values[valuesById[503002] = "AGE_RANGE_25_34"] = 503002;
                                values[valuesById[503003] = "AGE_RANGE_35_44"] = 503003;
                                values[valuesById[503004] = "AGE_RANGE_45_54"] = 503004;
                                values[valuesById[503005] = "AGE_RANGE_55_64"] = 503005;
                                values[valuesById[503006] = "AGE_RANGE_65_UP"] = 503006;
                                values[valuesById[503999] = "AGE_RANGE_UNDETERMINED"] = 503999;
                                return values;
                            })();
    
                            return AgeRangeTypeEnum;
                        })();
    
                        enums.AppPaymentModelTypeEnum = (function() {
    
                            /**
                             * Properties of an AppPaymentModelTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAppPaymentModelTypeEnum
                             */
    
                            /**
                             * Constructs a new AppPaymentModelTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AppPaymentModelTypeEnum.
                             * @implements IAppPaymentModelTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAppPaymentModelTypeEnum=} [p] Properties to set
                             */
                            function AppPaymentModelTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AppPaymentModelTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppPaymentModelTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AppPaymentModelTypeEnum} AppPaymentModelTypeEnum instance
                             */
                            AppPaymentModelTypeEnum.create = function create(properties) {
                                return new AppPaymentModelTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AppPaymentModelTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AppPaymentModelTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppPaymentModelTypeEnum} m AppPaymentModelTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPaymentModelTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppPaymentModelTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AppPaymentModelTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppPaymentModelTypeEnum} message AppPaymentModelTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPaymentModelTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppPaymentModelTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AppPaymentModelTypeEnum} AppPaymentModelTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPaymentModelTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AppPaymentModelTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppPaymentModelTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AppPaymentModelTypeEnum} AppPaymentModelTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPaymentModelTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppPaymentModelTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppPaymentModelTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AppPaymentModelTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AppPaymentModelTypeEnum} AppPaymentModelTypeEnum
                             */
                            AppPaymentModelTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AppPaymentModelTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AppPaymentModelTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AppPaymentModelTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AppPaymentModelTypeEnum} m AppPaymentModelTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppPaymentModelTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AppPaymentModelTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AppPaymentModelTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppPaymentModelTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AppPaymentModelType enum.
                             * @name google.ads.googleads.v11.enums.AppPaymentModelTypeEnum.AppPaymentModelType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PAID=30 PAID value
                             */
                            AppPaymentModelTypeEnum.AppPaymentModelType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[30] = "PAID"] = 30;
                                return values;
                            })();
    
                            return AppPaymentModelTypeEnum;
                        })();
    
                        enums.ContentLabelTypeEnum = (function() {
    
                            /**
                             * Properties of a ContentLabelTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IContentLabelTypeEnum
                             */
    
                            /**
                             * Constructs a new ContentLabelTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ContentLabelTypeEnum.
                             * @implements IContentLabelTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IContentLabelTypeEnum=} [p] Properties to set
                             */
                            function ContentLabelTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ContentLabelTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IContentLabelTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ContentLabelTypeEnum} ContentLabelTypeEnum instance
                             */
                            ContentLabelTypeEnum.create = function create(properties) {
                                return new ContentLabelTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ContentLabelTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ContentLabelTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IContentLabelTypeEnum} m ContentLabelTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ContentLabelTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ContentLabelTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ContentLabelTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IContentLabelTypeEnum} message ContentLabelTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ContentLabelTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ContentLabelTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ContentLabelTypeEnum} ContentLabelTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ContentLabelTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ContentLabelTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ContentLabelTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ContentLabelTypeEnum} ContentLabelTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ContentLabelTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ContentLabelTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ContentLabelTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ContentLabelTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ContentLabelTypeEnum} ContentLabelTypeEnum
                             */
                            ContentLabelTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ContentLabelTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ContentLabelTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ContentLabelTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ContentLabelTypeEnum} m ContentLabelTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ContentLabelTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ContentLabelTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ContentLabelTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ContentLabelTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ContentLabelType enum.
                             * @name google.ads.googleads.v11.enums.ContentLabelTypeEnum.ContentLabelType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SEXUALLY_SUGGESTIVE=2 SEXUALLY_SUGGESTIVE value
                             * @property {number} BELOW_THE_FOLD=3 BELOW_THE_FOLD value
                             * @property {number} PARKED_DOMAIN=4 PARKED_DOMAIN value
                             * @property {number} JUVENILE=6 JUVENILE value
                             * @property {number} PROFANITY=7 PROFANITY value
                             * @property {number} TRAGEDY=8 TRAGEDY value
                             * @property {number} VIDEO=9 VIDEO value
                             * @property {number} VIDEO_RATING_DV_G=10 VIDEO_RATING_DV_G value
                             * @property {number} VIDEO_RATING_DV_PG=11 VIDEO_RATING_DV_PG value
                             * @property {number} VIDEO_RATING_DV_T=12 VIDEO_RATING_DV_T value
                             * @property {number} VIDEO_RATING_DV_MA=13 VIDEO_RATING_DV_MA value
                             * @property {number} VIDEO_NOT_YET_RATED=14 VIDEO_NOT_YET_RATED value
                             * @property {number} EMBEDDED_VIDEO=15 EMBEDDED_VIDEO value
                             * @property {number} LIVE_STREAMING_VIDEO=16 LIVE_STREAMING_VIDEO value
                             * @property {number} SOCIAL_ISSUES=17 SOCIAL_ISSUES value
                             */
                            ContentLabelTypeEnum.ContentLabelType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SEXUALLY_SUGGESTIVE"] = 2;
                                values[valuesById[3] = "BELOW_THE_FOLD"] = 3;
                                values[valuesById[4] = "PARKED_DOMAIN"] = 4;
                                values[valuesById[6] = "JUVENILE"] = 6;
                                values[valuesById[7] = "PROFANITY"] = 7;
                                values[valuesById[8] = "TRAGEDY"] = 8;
                                values[valuesById[9] = "VIDEO"] = 9;
                                values[valuesById[10] = "VIDEO_RATING_DV_G"] = 10;
                                values[valuesById[11] = "VIDEO_RATING_DV_PG"] = 11;
                                values[valuesById[12] = "VIDEO_RATING_DV_T"] = 12;
                                values[valuesById[13] = "VIDEO_RATING_DV_MA"] = 13;
                                values[valuesById[14] = "VIDEO_NOT_YET_RATED"] = 14;
                                values[valuesById[15] = "EMBEDDED_VIDEO"] = 15;
                                values[valuesById[16] = "LIVE_STREAMING_VIDEO"] = 16;
                                values[valuesById[17] = "SOCIAL_ISSUES"] = 17;
                                return values;
                            })();
    
                            return ContentLabelTypeEnum;
                        })();
    
                        enums.DayOfWeekEnum = (function() {
    
                            /**
                             * Properties of a DayOfWeekEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IDayOfWeekEnum
                             */
    
                            /**
                             * Constructs a new DayOfWeekEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a DayOfWeekEnum.
                             * @implements IDayOfWeekEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IDayOfWeekEnum=} [p] Properties to set
                             */
                            function DayOfWeekEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DayOfWeekEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDayOfWeekEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.DayOfWeekEnum} DayOfWeekEnum instance
                             */
                            DayOfWeekEnum.create = function create(properties) {
                                return new DayOfWeekEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DayOfWeekEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.DayOfWeekEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDayOfWeekEnum} m DayOfWeekEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DayOfWeekEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DayOfWeekEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.DayOfWeekEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDayOfWeekEnum} message DayOfWeekEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DayOfWeekEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DayOfWeekEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.DayOfWeekEnum} DayOfWeekEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DayOfWeekEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.DayOfWeekEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DayOfWeekEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.DayOfWeekEnum} DayOfWeekEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DayOfWeekEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DayOfWeekEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DayOfWeekEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DayOfWeekEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.DayOfWeekEnum} DayOfWeekEnum
                             */
                            DayOfWeekEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.DayOfWeekEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.DayOfWeekEnum();
                            };
    
                            /**
                             * Creates a plain object from a DayOfWeekEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.DayOfWeekEnum} m DayOfWeekEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DayOfWeekEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DayOfWeekEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.DayOfWeekEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DayOfWeekEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DayOfWeek enum.
                             * @name google.ads.googleads.v11.enums.DayOfWeekEnum.DayOfWeek
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MONDAY=2 MONDAY value
                             * @property {number} TUESDAY=3 TUESDAY value
                             * @property {number} WEDNESDAY=4 WEDNESDAY value
                             * @property {number} THURSDAY=5 THURSDAY value
                             * @property {number} FRIDAY=6 FRIDAY value
                             * @property {number} SATURDAY=7 SATURDAY value
                             * @property {number} SUNDAY=8 SUNDAY value
                             */
                            DayOfWeekEnum.DayOfWeek = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MONDAY"] = 2;
                                values[valuesById[3] = "TUESDAY"] = 3;
                                values[valuesById[4] = "WEDNESDAY"] = 4;
                                values[valuesById[5] = "THURSDAY"] = 5;
                                values[valuesById[6] = "FRIDAY"] = 6;
                                values[valuesById[7] = "SATURDAY"] = 7;
                                values[valuesById[8] = "SUNDAY"] = 8;
                                return values;
                            })();
    
                            return DayOfWeekEnum;
                        })();
    
                        enums.DeviceEnum = (function() {
    
                            /**
                             * Properties of a DeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IDeviceEnum
                             */
    
                            /**
                             * Constructs a new DeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a DeviceEnum.
                             * @implements IDeviceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IDeviceEnum=} [p] Properties to set
                             */
                            function DeviceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DeviceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDeviceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.DeviceEnum} DeviceEnum instance
                             */
                            DeviceEnum.create = function create(properties) {
                                return new DeviceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DeviceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.DeviceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDeviceEnum} m DeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DeviceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DeviceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.DeviceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDeviceEnum} message DeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DeviceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DeviceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.DeviceEnum} DeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DeviceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.DeviceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DeviceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.DeviceEnum} DeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DeviceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DeviceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DeviceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DeviceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.DeviceEnum} DeviceEnum
                             */
                            DeviceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.DeviceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.DeviceEnum();
                            };
    
                            /**
                             * Creates a plain object from a DeviceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.DeviceEnum} m DeviceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DeviceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DeviceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.DeviceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DeviceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Device enum.
                             * @name google.ads.googleads.v11.enums.DeviceEnum.Device
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MOBILE=2 MOBILE value
                             * @property {number} TABLET=3 TABLET value
                             * @property {number} DESKTOP=4 DESKTOP value
                             * @property {number} CONNECTED_TV=6 CONNECTED_TV value
                             * @property {number} OTHER=5 OTHER value
                             */
                            DeviceEnum.Device = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MOBILE"] = 2;
                                values[valuesById[3] = "TABLET"] = 3;
                                values[valuesById[4] = "DESKTOP"] = 4;
                                values[valuesById[6] = "CONNECTED_TV"] = 6;
                                values[valuesById[5] = "OTHER"] = 5;
                                return values;
                            })();
    
                            return DeviceEnum;
                        })();
    
                        enums.GenderTypeEnum = (function() {
    
                            /**
                             * Properties of a GenderTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGenderTypeEnum
                             */
    
                            /**
                             * Constructs a new GenderTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GenderTypeEnum.
                             * @implements IGenderTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGenderTypeEnum=} [p] Properties to set
                             */
                            function GenderTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GenderTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGenderTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GenderTypeEnum} GenderTypeEnum instance
                             */
                            GenderTypeEnum.create = function create(properties) {
                                return new GenderTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GenderTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GenderTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGenderTypeEnum} m GenderTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenderTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GenderTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGenderTypeEnum} message GenderTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenderTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GenderTypeEnum} GenderTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GenderTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenderTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GenderTypeEnum} GenderTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenderTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenderTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GenderTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GenderTypeEnum} GenderTypeEnum
                             */
                            GenderTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GenderTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GenderTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a GenderTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GenderTypeEnum} m GenderTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenderTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GenderTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GenderTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenderTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GenderType enum.
                             * @name google.ads.googleads.v11.enums.GenderTypeEnum.GenderType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MALE=10 MALE value
                             * @property {number} FEMALE=11 FEMALE value
                             * @property {number} UNDETERMINED=20 UNDETERMINED value
                             */
                            GenderTypeEnum.GenderType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[10] = "MALE"] = 10;
                                values[valuesById[11] = "FEMALE"] = 11;
                                values[valuesById[20] = "UNDETERMINED"] = 20;
                                return values;
                            })();
    
                            return GenderTypeEnum;
                        })();
    
                        enums.HotelDateSelectionTypeEnum = (function() {
    
                            /**
                             * Properties of a HotelDateSelectionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IHotelDateSelectionTypeEnum
                             */
    
                            /**
                             * Constructs a new HotelDateSelectionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a HotelDateSelectionTypeEnum.
                             * @implements IHotelDateSelectionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IHotelDateSelectionTypeEnum=} [p] Properties to set
                             */
                            function HotelDateSelectionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new HotelDateSelectionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelDateSelectionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum} HotelDateSelectionTypeEnum instance
                             */
                            HotelDateSelectionTypeEnum.create = function create(properties) {
                                return new HotelDateSelectionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified HotelDateSelectionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelDateSelectionTypeEnum} m HotelDateSelectionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelDateSelectionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelDateSelectionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelDateSelectionTypeEnum} message HotelDateSelectionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelDateSelectionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelDateSelectionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum} HotelDateSelectionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelDateSelectionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelDateSelectionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum} HotelDateSelectionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelDateSelectionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelDateSelectionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelDateSelectionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a HotelDateSelectionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum} HotelDateSelectionTypeEnum
                             */
                            HotelDateSelectionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a HotelDateSelectionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum} m HotelDateSelectionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelDateSelectionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this HotelDateSelectionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelDateSelectionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * HotelDateSelectionType enum.
                             * @name google.ads.googleads.v11.enums.HotelDateSelectionTypeEnum.HotelDateSelectionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DEFAULT_SELECTION=50 DEFAULT_SELECTION value
                             * @property {number} USER_SELECTED=51 USER_SELECTED value
                             */
                            HotelDateSelectionTypeEnum.HotelDateSelectionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[50] = "DEFAULT_SELECTION"] = 50;
                                values[valuesById[51] = "USER_SELECTED"] = 51;
                                return values;
                            })();
    
                            return HotelDateSelectionTypeEnum;
                        })();
    
                        enums.IncomeRangeTypeEnum = (function() {
    
                            /**
                             * Properties of an IncomeRangeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IIncomeRangeTypeEnum
                             */
    
                            /**
                             * Constructs a new IncomeRangeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an IncomeRangeTypeEnum.
                             * @implements IIncomeRangeTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IIncomeRangeTypeEnum=} [p] Properties to set
                             */
                            function IncomeRangeTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new IncomeRangeTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IIncomeRangeTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.IncomeRangeTypeEnum} IncomeRangeTypeEnum instance
                             */
                            IncomeRangeTypeEnum.create = function create(properties) {
                                return new IncomeRangeTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified IncomeRangeTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.IncomeRangeTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IIncomeRangeTypeEnum} m IncomeRangeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IncomeRangeTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified IncomeRangeTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.IncomeRangeTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IIncomeRangeTypeEnum} message IncomeRangeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IncomeRangeTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an IncomeRangeTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.IncomeRangeTypeEnum} IncomeRangeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IncomeRangeTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.IncomeRangeTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an IncomeRangeTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.IncomeRangeTypeEnum} IncomeRangeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IncomeRangeTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an IncomeRangeTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            IncomeRangeTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an IncomeRangeTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.IncomeRangeTypeEnum} IncomeRangeTypeEnum
                             */
                            IncomeRangeTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.IncomeRangeTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.IncomeRangeTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an IncomeRangeTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IncomeRangeTypeEnum} m IncomeRangeTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            IncomeRangeTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this IncomeRangeTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.IncomeRangeTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            IncomeRangeTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * IncomeRangeType enum.
                             * @name google.ads.googleads.v11.enums.IncomeRangeTypeEnum.IncomeRangeType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INCOME_RANGE_0_50=510001 INCOME_RANGE_0_50 value
                             * @property {number} INCOME_RANGE_50_60=510002 INCOME_RANGE_50_60 value
                             * @property {number} INCOME_RANGE_60_70=510003 INCOME_RANGE_60_70 value
                             * @property {number} INCOME_RANGE_70_80=510004 INCOME_RANGE_70_80 value
                             * @property {number} INCOME_RANGE_80_90=510005 INCOME_RANGE_80_90 value
                             * @property {number} INCOME_RANGE_90_UP=510006 INCOME_RANGE_90_UP value
                             * @property {number} INCOME_RANGE_UNDETERMINED=510000 INCOME_RANGE_UNDETERMINED value
                             */
                            IncomeRangeTypeEnum.IncomeRangeType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[510001] = "INCOME_RANGE_0_50"] = 510001;
                                values[valuesById[510002] = "INCOME_RANGE_50_60"] = 510002;
                                values[valuesById[510003] = "INCOME_RANGE_60_70"] = 510003;
                                values[valuesById[510004] = "INCOME_RANGE_70_80"] = 510004;
                                values[valuesById[510005] = "INCOME_RANGE_80_90"] = 510005;
                                values[valuesById[510006] = "INCOME_RANGE_90_UP"] = 510006;
                                values[valuesById[510000] = "INCOME_RANGE_UNDETERMINED"] = 510000;
                                return values;
                            })();
    
                            return IncomeRangeTypeEnum;
                        })();
    
                        enums.InteractionTypeEnum = (function() {
    
                            /**
                             * Properties of an InteractionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IInteractionTypeEnum
                             */
    
                            /**
                             * Constructs a new InteractionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an InteractionTypeEnum.
                             * @implements IInteractionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IInteractionTypeEnum=} [p] Properties to set
                             */
                            function InteractionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new InteractionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInteractionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.InteractionTypeEnum} InteractionTypeEnum instance
                             */
                            InteractionTypeEnum.create = function create(properties) {
                                return new InteractionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified InteractionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.InteractionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInteractionTypeEnum} m InteractionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InteractionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified InteractionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.InteractionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInteractionTypeEnum} message InteractionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InteractionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an InteractionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.InteractionTypeEnum} InteractionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InteractionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.InteractionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an InteractionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.InteractionTypeEnum} InteractionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InteractionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an InteractionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            InteractionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an InteractionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.InteractionTypeEnum} InteractionTypeEnum
                             */
                            InteractionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.InteractionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.InteractionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an InteractionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.InteractionTypeEnum} m InteractionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            InteractionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this InteractionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.InteractionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            InteractionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * InteractionType enum.
                             * @name google.ads.googleads.v11.enums.InteractionTypeEnum.InteractionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CALLS=8000 CALLS value
                             */
                            InteractionTypeEnum.InteractionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[8000] = "CALLS"] = 8000;
                                return values;
                            })();
    
                            return InteractionTypeEnum;
                        })();
    
                        enums.KeywordMatchTypeEnum = (function() {
    
                            /**
                             * Properties of a KeywordMatchTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IKeywordMatchTypeEnum
                             */
    
                            /**
                             * Constructs a new KeywordMatchTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a KeywordMatchTypeEnum.
                             * @implements IKeywordMatchTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IKeywordMatchTypeEnum=} [p] Properties to set
                             */
                            function KeywordMatchTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordMatchTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordMatchTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.KeywordMatchTypeEnum} KeywordMatchTypeEnum instance
                             */
                            KeywordMatchTypeEnum.create = function create(properties) {
                                return new KeywordMatchTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordMatchTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordMatchTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordMatchTypeEnum} m KeywordMatchTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordMatchTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordMatchTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordMatchTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordMatchTypeEnum} message KeywordMatchTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordMatchTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordMatchTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.KeywordMatchTypeEnum} KeywordMatchTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordMatchTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordMatchTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.KeywordMatchTypeEnum} KeywordMatchTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordMatchTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordMatchTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordMatchTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordMatchTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.KeywordMatchTypeEnum} KeywordMatchTypeEnum
                             */
                            KeywordMatchTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordMatchTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.KeywordMatchTypeEnum} m KeywordMatchTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordMatchTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordMatchTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.KeywordMatchTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordMatchTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordMatchType enum.
                             * @name google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EXACT=2 EXACT value
                             * @property {number} PHRASE=3 PHRASE value
                             * @property {number} BROAD=4 BROAD value
                             */
                            KeywordMatchTypeEnum.KeywordMatchType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EXACT"] = 2;
                                values[valuesById[3] = "PHRASE"] = 3;
                                values[valuesById[4] = "BROAD"] = 4;
                                return values;
                            })();
    
                            return KeywordMatchTypeEnum;
                        })();
    
                        enums.ListingGroupTypeEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupTypeEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupTypeEnum.
                             * @implements IListingGroupTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupTypeEnum=} [p] Properties to set
                             */
                            function ListingGroupTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupTypeEnum} ListingGroupTypeEnum instance
                             */
                            ListingGroupTypeEnum.create = function create(properties) {
                                return new ListingGroupTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupTypeEnum} m ListingGroupTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupTypeEnum} message ListingGroupTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupTypeEnum} ListingGroupTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupTypeEnum} ListingGroupTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupTypeEnum} ListingGroupTypeEnum
                             */
                            ListingGroupTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupTypeEnum} m ListingGroupTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupType enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupTypeEnum.ListingGroupType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SUBDIVISION=2 SUBDIVISION value
                             * @property {number} UNIT=3 UNIT value
                             */
                            ListingGroupTypeEnum.ListingGroupType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SUBDIVISION"] = 2;
                                values[valuesById[3] = "UNIT"] = 3;
                                return values;
                            })();
    
                            return ListingGroupTypeEnum;
                        })();
    
                        enums.LocationGroupRadiusUnitsEnum = (function() {
    
                            /**
                             * Properties of a LocationGroupRadiusUnitsEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILocationGroupRadiusUnitsEnum
                             */
    
                            /**
                             * Constructs a new LocationGroupRadiusUnitsEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LocationGroupRadiusUnitsEnum.
                             * @implements ILocationGroupRadiusUnitsEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILocationGroupRadiusUnitsEnum=} [p] Properties to set
                             */
                            function LocationGroupRadiusUnitsEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LocationGroupRadiusUnitsEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationGroupRadiusUnitsEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum} LocationGroupRadiusUnitsEnum instance
                             */
                            LocationGroupRadiusUnitsEnum.create = function create(properties) {
                                return new LocationGroupRadiusUnitsEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LocationGroupRadiusUnitsEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationGroupRadiusUnitsEnum} m LocationGroupRadiusUnitsEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationGroupRadiusUnitsEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationGroupRadiusUnitsEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationGroupRadiusUnitsEnum} message LocationGroupRadiusUnitsEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationGroupRadiusUnitsEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationGroupRadiusUnitsEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum} LocationGroupRadiusUnitsEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationGroupRadiusUnitsEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationGroupRadiusUnitsEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum} LocationGroupRadiusUnitsEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationGroupRadiusUnitsEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationGroupRadiusUnitsEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationGroupRadiusUnitsEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LocationGroupRadiusUnitsEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum} LocationGroupRadiusUnitsEnum
                             */
                            LocationGroupRadiusUnitsEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum();
                            };
    
                            /**
                             * Creates a plain object from a LocationGroupRadiusUnitsEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum} m LocationGroupRadiusUnitsEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationGroupRadiusUnitsEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LocationGroupRadiusUnitsEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationGroupRadiusUnitsEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LocationGroupRadiusUnits enum.
                             * @name google.ads.googleads.v11.enums.LocationGroupRadiusUnitsEnum.LocationGroupRadiusUnits
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} METERS=2 METERS value
                             * @property {number} MILES=3 MILES value
                             * @property {number} MILLI_MILES=4 MILLI_MILES value
                             */
                            LocationGroupRadiusUnitsEnum.LocationGroupRadiusUnits = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "METERS"] = 2;
                                values[valuesById[3] = "MILES"] = 3;
                                values[valuesById[4] = "MILLI_MILES"] = 4;
                                return values;
                            })();
    
                            return LocationGroupRadiusUnitsEnum;
                        })();
    
                        enums.MinuteOfHourEnum = (function() {
    
                            /**
                             * Properties of a MinuteOfHourEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMinuteOfHourEnum
                             */
    
                            /**
                             * Constructs a new MinuteOfHourEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MinuteOfHourEnum.
                             * @implements IMinuteOfHourEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMinuteOfHourEnum=} [p] Properties to set
                             */
                            function MinuteOfHourEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MinuteOfHourEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMinuteOfHourEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MinuteOfHourEnum} MinuteOfHourEnum instance
                             */
                            MinuteOfHourEnum.create = function create(properties) {
                                return new MinuteOfHourEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MinuteOfHourEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MinuteOfHourEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMinuteOfHourEnum} m MinuteOfHourEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MinuteOfHourEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MinuteOfHourEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MinuteOfHourEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMinuteOfHourEnum} message MinuteOfHourEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MinuteOfHourEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MinuteOfHourEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MinuteOfHourEnum} MinuteOfHourEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MinuteOfHourEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MinuteOfHourEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MinuteOfHourEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MinuteOfHourEnum} MinuteOfHourEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MinuteOfHourEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MinuteOfHourEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MinuteOfHourEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MinuteOfHourEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MinuteOfHourEnum} MinuteOfHourEnum
                             */
                            MinuteOfHourEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MinuteOfHourEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MinuteOfHourEnum();
                            };
    
                            /**
                             * Creates a plain object from a MinuteOfHourEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MinuteOfHourEnum} m MinuteOfHourEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MinuteOfHourEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MinuteOfHourEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MinuteOfHourEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MinuteOfHourEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MinuteOfHour enum.
                             * @name google.ads.googleads.v11.enums.MinuteOfHourEnum.MinuteOfHour
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ZERO=2 ZERO value
                             * @property {number} FIFTEEN=3 FIFTEEN value
                             * @property {number} THIRTY=4 THIRTY value
                             * @property {number} FORTY_FIVE=5 FORTY_FIVE value
                             */
                            MinuteOfHourEnum.MinuteOfHour = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ZERO"] = 2;
                                values[valuesById[3] = "FIFTEEN"] = 3;
                                values[valuesById[4] = "THIRTY"] = 4;
                                values[valuesById[5] = "FORTY_FIVE"] = 5;
                                return values;
                            })();
    
                            return MinuteOfHourEnum;
                        })();
    
                        enums.ParentalStatusTypeEnum = (function() {
    
                            /**
                             * Properties of a ParentalStatusTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IParentalStatusTypeEnum
                             */
    
                            /**
                             * Constructs a new ParentalStatusTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ParentalStatusTypeEnum.
                             * @implements IParentalStatusTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IParentalStatusTypeEnum=} [p] Properties to set
                             */
                            function ParentalStatusTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ParentalStatusTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IParentalStatusTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ParentalStatusTypeEnum} ParentalStatusTypeEnum instance
                             */
                            ParentalStatusTypeEnum.create = function create(properties) {
                                return new ParentalStatusTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ParentalStatusTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ParentalStatusTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IParentalStatusTypeEnum} m ParentalStatusTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ParentalStatusTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ParentalStatusTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IParentalStatusTypeEnum} message ParentalStatusTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ParentalStatusTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ParentalStatusTypeEnum} ParentalStatusTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ParentalStatusTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ParentalStatusTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ParentalStatusTypeEnum} ParentalStatusTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ParentalStatusTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ParentalStatusTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ParentalStatusTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ParentalStatusTypeEnum} ParentalStatusTypeEnum
                             */
                            ParentalStatusTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ParentalStatusTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ParentalStatusTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ParentalStatusTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ParentalStatusTypeEnum} m ParentalStatusTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ParentalStatusTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ParentalStatusTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ParentalStatusTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ParentalStatusTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ParentalStatusType enum.
                             * @name google.ads.googleads.v11.enums.ParentalStatusTypeEnum.ParentalStatusType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PARENT=300 PARENT value
                             * @property {number} NOT_A_PARENT=301 NOT_A_PARENT value
                             * @property {number} UNDETERMINED=302 UNDETERMINED value
                             */
                            ParentalStatusTypeEnum.ParentalStatusType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[300] = "PARENT"] = 300;
                                values[valuesById[301] = "NOT_A_PARENT"] = 301;
                                values[valuesById[302] = "UNDETERMINED"] = 302;
                                return values;
                            })();
    
                            return ParentalStatusTypeEnum;
                        })();
    
                        enums.PreferredContentTypeEnum = (function() {
    
                            /**
                             * Properties of a PreferredContentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPreferredContentTypeEnum
                             */
    
                            /**
                             * Constructs a new PreferredContentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PreferredContentTypeEnum.
                             * @implements IPreferredContentTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPreferredContentTypeEnum=} [p] Properties to set
                             */
                            function PreferredContentTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PreferredContentTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPreferredContentTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PreferredContentTypeEnum} PreferredContentTypeEnum instance
                             */
                            PreferredContentTypeEnum.create = function create(properties) {
                                return new PreferredContentTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PreferredContentTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PreferredContentTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPreferredContentTypeEnum} m PreferredContentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PreferredContentTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PreferredContentTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PreferredContentTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPreferredContentTypeEnum} message PreferredContentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PreferredContentTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PreferredContentTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PreferredContentTypeEnum} PreferredContentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PreferredContentTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PreferredContentTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PreferredContentTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PreferredContentTypeEnum} PreferredContentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PreferredContentTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PreferredContentTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PreferredContentTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PreferredContentTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PreferredContentTypeEnum} PreferredContentTypeEnum
                             */
                            PreferredContentTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PreferredContentTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PreferredContentTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PreferredContentTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PreferredContentTypeEnum} m PreferredContentTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PreferredContentTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PreferredContentTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PreferredContentTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PreferredContentTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PreferredContentType enum.
                             * @name google.ads.googleads.v11.enums.PreferredContentTypeEnum.PreferredContentType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} YOUTUBE_TOP_CONTENT=400 YOUTUBE_TOP_CONTENT value
                             */
                            PreferredContentTypeEnum.PreferredContentType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[400] = "YOUTUBE_TOP_CONTENT"] = 400;
                                return values;
                            })();
    
                            return PreferredContentTypeEnum;
                        })();
    
                        enums.ProductBiddingCategoryLevelEnum = (function() {
    
                            /**
                             * Properties of a ProductBiddingCategoryLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProductBiddingCategoryLevelEnum
                             */
    
                            /**
                             * Constructs a new ProductBiddingCategoryLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProductBiddingCategoryLevelEnum.
                             * @implements IProductBiddingCategoryLevelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryLevelEnum=} [p] Properties to set
                             */
                            function ProductBiddingCategoryLevelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProductBiddingCategoryLevelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryLevelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum} ProductBiddingCategoryLevelEnum instance
                             */
                            ProductBiddingCategoryLevelEnum.create = function create(properties) {
                                return new ProductBiddingCategoryLevelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryLevelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryLevelEnum} m ProductBiddingCategoryLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryLevelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryLevelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryLevelEnum} message ProductBiddingCategoryLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryLevelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryLevelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum} ProductBiddingCategoryLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryLevelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryLevelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum} ProductBiddingCategoryLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryLevelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductBiddingCategoryLevelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductBiddingCategoryLevelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProductBiddingCategoryLevelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum} ProductBiddingCategoryLevelEnum
                             */
                            ProductBiddingCategoryLevelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProductBiddingCategoryLevelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum} m ProductBiddingCategoryLevelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductBiddingCategoryLevelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProductBiddingCategoryLevelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductBiddingCategoryLevelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProductBiddingCategoryLevel enum.
                             * @name google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LEVEL1=2 LEVEL1 value
                             * @property {number} LEVEL2=3 LEVEL2 value
                             * @property {number} LEVEL3=4 LEVEL3 value
                             * @property {number} LEVEL4=5 LEVEL4 value
                             * @property {number} LEVEL5=6 LEVEL5 value
                             */
                            ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LEVEL1"] = 2;
                                values[valuesById[3] = "LEVEL2"] = 3;
                                values[valuesById[4] = "LEVEL3"] = 4;
                                values[valuesById[5] = "LEVEL4"] = 5;
                                values[valuesById[6] = "LEVEL5"] = 6;
                                return values;
                            })();
    
                            return ProductBiddingCategoryLevelEnum;
                        })();
    
                        enums.ProductChannelEnum = (function() {
    
                            /**
                             * Properties of a ProductChannelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProductChannelEnum
                             */
    
                            /**
                             * Constructs a new ProductChannelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProductChannelEnum.
                             * @implements IProductChannelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProductChannelEnum=} [p] Properties to set
                             */
                            function ProductChannelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProductChannelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductChannelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProductChannelEnum} ProductChannelEnum instance
                             */
                            ProductChannelEnum.create = function create(properties) {
                                return new ProductChannelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProductChannelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProductChannelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductChannelEnum} m ProductChannelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductChannelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProductChannelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductChannelEnum} message ProductChannelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductChannelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProductChannelEnum} ProductChannelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProductChannelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductChannelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProductChannelEnum} ProductChannelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductChannelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductChannelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProductChannelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProductChannelEnum} ProductChannelEnum
                             */
                            ProductChannelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProductChannelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProductChannelEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProductChannelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProductChannelEnum} m ProductChannelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductChannelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProductChannelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProductChannelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductChannelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProductChannel enum.
                             * @name google.ads.googleads.v11.enums.ProductChannelEnum.ProductChannel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ONLINE=2 ONLINE value
                             * @property {number} LOCAL=3 LOCAL value
                             */
                            ProductChannelEnum.ProductChannel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ONLINE"] = 2;
                                values[valuesById[3] = "LOCAL"] = 3;
                                return values;
                            })();
    
                            return ProductChannelEnum;
                        })();
    
                        enums.ProductChannelExclusivityEnum = (function() {
    
                            /**
                             * Properties of a ProductChannelExclusivityEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProductChannelExclusivityEnum
                             */
    
                            /**
                             * Constructs a new ProductChannelExclusivityEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProductChannelExclusivityEnum.
                             * @implements IProductChannelExclusivityEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProductChannelExclusivityEnum=} [p] Properties to set
                             */
                            function ProductChannelExclusivityEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProductChannelExclusivityEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductChannelExclusivityEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum} ProductChannelExclusivityEnum instance
                             */
                            ProductChannelExclusivityEnum.create = function create(properties) {
                                return new ProductChannelExclusivityEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProductChannelExclusivityEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductChannelExclusivityEnum} m ProductChannelExclusivityEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelExclusivityEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductChannelExclusivityEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductChannelExclusivityEnum} message ProductChannelExclusivityEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductChannelExclusivityEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductChannelExclusivityEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum} ProductChannelExclusivityEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelExclusivityEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProductChannelExclusivityEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductChannelExclusivityEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum} ProductChannelExclusivityEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductChannelExclusivityEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductChannelExclusivityEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductChannelExclusivityEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProductChannelExclusivityEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum} ProductChannelExclusivityEnum
                             */
                            ProductChannelExclusivityEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProductChannelExclusivityEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProductChannelExclusivityEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProductChannelExclusivityEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProductChannelExclusivityEnum} m ProductChannelExclusivityEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductChannelExclusivityEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProductChannelExclusivityEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProductChannelExclusivityEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductChannelExclusivityEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProductChannelExclusivity enum.
                             * @name google.ads.googleads.v11.enums.ProductChannelExclusivityEnum.ProductChannelExclusivity
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SINGLE_CHANNEL=2 SINGLE_CHANNEL value
                             * @property {number} MULTI_CHANNEL=3 MULTI_CHANNEL value
                             */
                            ProductChannelExclusivityEnum.ProductChannelExclusivity = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SINGLE_CHANNEL"] = 2;
                                values[valuesById[3] = "MULTI_CHANNEL"] = 3;
                                return values;
                            })();
    
                            return ProductChannelExclusivityEnum;
                        })();
    
                        enums.ProductConditionEnum = (function() {
    
                            /**
                             * Properties of a ProductConditionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProductConditionEnum
                             */
    
                            /**
                             * Constructs a new ProductConditionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProductConditionEnum.
                             * @implements IProductConditionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProductConditionEnum=} [p] Properties to set
                             */
                            function ProductConditionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProductConditionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductConditionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProductConditionEnum} ProductConditionEnum instance
                             */
                            ProductConditionEnum.create = function create(properties) {
                                return new ProductConditionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProductConditionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProductConditionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductConditionEnum} m ProductConditionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductConditionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductConditionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProductConditionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductConditionEnum} message ProductConditionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductConditionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductConditionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProductConditionEnum} ProductConditionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductConditionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProductConditionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductConditionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProductConditionEnum} ProductConditionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductConditionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductConditionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductConditionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProductConditionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProductConditionEnum} ProductConditionEnum
                             */
                            ProductConditionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProductConditionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProductConditionEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProductConditionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProductConditionEnum} m ProductConditionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductConditionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProductConditionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProductConditionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductConditionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProductCondition enum.
                             * @name google.ads.googleads.v11.enums.ProductConditionEnum.ProductCondition
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NEW=3 NEW value
                             * @property {number} REFURBISHED=4 REFURBISHED value
                             * @property {number} USED=5 USED value
                             */
                            ProductConditionEnum.ProductCondition = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "NEW"] = 3;
                                values[valuesById[4] = "REFURBISHED"] = 4;
                                values[valuesById[5] = "USED"] = 5;
                                return values;
                            })();
    
                            return ProductConditionEnum;
                        })();
    
                        enums.ProductCustomAttributeIndexEnum = (function() {
    
                            /**
                             * Properties of a ProductCustomAttributeIndexEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProductCustomAttributeIndexEnum
                             */
    
                            /**
                             * Constructs a new ProductCustomAttributeIndexEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProductCustomAttributeIndexEnum.
                             * @implements IProductCustomAttributeIndexEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProductCustomAttributeIndexEnum=} [p] Properties to set
                             */
                            function ProductCustomAttributeIndexEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProductCustomAttributeIndexEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductCustomAttributeIndexEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum} ProductCustomAttributeIndexEnum instance
                             */
                            ProductCustomAttributeIndexEnum.create = function create(properties) {
                                return new ProductCustomAttributeIndexEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProductCustomAttributeIndexEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductCustomAttributeIndexEnum} m ProductCustomAttributeIndexEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductCustomAttributeIndexEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductCustomAttributeIndexEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductCustomAttributeIndexEnum} message ProductCustomAttributeIndexEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductCustomAttributeIndexEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductCustomAttributeIndexEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum} ProductCustomAttributeIndexEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductCustomAttributeIndexEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductCustomAttributeIndexEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum} ProductCustomAttributeIndexEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductCustomAttributeIndexEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductCustomAttributeIndexEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductCustomAttributeIndexEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProductCustomAttributeIndexEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum} ProductCustomAttributeIndexEnum
                             */
                            ProductCustomAttributeIndexEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProductCustomAttributeIndexEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum} m ProductCustomAttributeIndexEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductCustomAttributeIndexEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProductCustomAttributeIndexEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductCustomAttributeIndexEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProductCustomAttributeIndex enum.
                             * @name google.ads.googleads.v11.enums.ProductCustomAttributeIndexEnum.ProductCustomAttributeIndex
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INDEX0=7 INDEX0 value
                             * @property {number} INDEX1=8 INDEX1 value
                             * @property {number} INDEX2=9 INDEX2 value
                             * @property {number} INDEX3=10 INDEX3 value
                             * @property {number} INDEX4=11 INDEX4 value
                             */
                            ProductCustomAttributeIndexEnum.ProductCustomAttributeIndex = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[7] = "INDEX0"] = 7;
                                values[valuesById[8] = "INDEX1"] = 8;
                                values[valuesById[9] = "INDEX2"] = 9;
                                values[valuesById[10] = "INDEX3"] = 10;
                                values[valuesById[11] = "INDEX4"] = 11;
                                return values;
                            })();
    
                            return ProductCustomAttributeIndexEnum;
                        })();
    
                        enums.ProductTypeLevelEnum = (function() {
    
                            /**
                             * Properties of a ProductTypeLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProductTypeLevelEnum
                             */
    
                            /**
                             * Constructs a new ProductTypeLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProductTypeLevelEnum.
                             * @implements IProductTypeLevelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProductTypeLevelEnum=} [p] Properties to set
                             */
                            function ProductTypeLevelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProductTypeLevelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductTypeLevelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProductTypeLevelEnum} ProductTypeLevelEnum instance
                             */
                            ProductTypeLevelEnum.create = function create(properties) {
                                return new ProductTypeLevelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProductTypeLevelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProductTypeLevelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductTypeLevelEnum} m ProductTypeLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductTypeLevelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductTypeLevelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProductTypeLevelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductTypeLevelEnum} message ProductTypeLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductTypeLevelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductTypeLevelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProductTypeLevelEnum} ProductTypeLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductTypeLevelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProductTypeLevelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductTypeLevelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProductTypeLevelEnum} ProductTypeLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductTypeLevelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductTypeLevelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductTypeLevelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProductTypeLevelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProductTypeLevelEnum} ProductTypeLevelEnum
                             */
                            ProductTypeLevelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProductTypeLevelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProductTypeLevelEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProductTypeLevelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProductTypeLevelEnum} m ProductTypeLevelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductTypeLevelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProductTypeLevelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProductTypeLevelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductTypeLevelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProductTypeLevel enum.
                             * @name google.ads.googleads.v11.enums.ProductTypeLevelEnum.ProductTypeLevel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LEVEL1=7 LEVEL1 value
                             * @property {number} LEVEL2=8 LEVEL2 value
                             * @property {number} LEVEL3=9 LEVEL3 value
                             * @property {number} LEVEL4=10 LEVEL4 value
                             * @property {number} LEVEL5=11 LEVEL5 value
                             */
                            ProductTypeLevelEnum.ProductTypeLevel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[7] = "LEVEL1"] = 7;
                                values[valuesById[8] = "LEVEL2"] = 8;
                                values[valuesById[9] = "LEVEL3"] = 9;
                                values[valuesById[10] = "LEVEL4"] = 10;
                                values[valuesById[11] = "LEVEL5"] = 11;
                                return values;
                            })();
    
                            return ProductTypeLevelEnum;
                        })();
    
                        enums.ProximityRadiusUnitsEnum = (function() {
    
                            /**
                             * Properties of a ProximityRadiusUnitsEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProximityRadiusUnitsEnum
                             */
    
                            /**
                             * Constructs a new ProximityRadiusUnitsEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProximityRadiusUnitsEnum.
                             * @implements IProximityRadiusUnitsEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProximityRadiusUnitsEnum=} [p] Properties to set
                             */
                            function ProximityRadiusUnitsEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProximityRadiusUnitsEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProximityRadiusUnitsEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum} ProximityRadiusUnitsEnum instance
                             */
                            ProximityRadiusUnitsEnum.create = function create(properties) {
                                return new ProximityRadiusUnitsEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProximityRadiusUnitsEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProximityRadiusUnitsEnum} m ProximityRadiusUnitsEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProximityRadiusUnitsEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProximityRadiusUnitsEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProximityRadiusUnitsEnum} message ProximityRadiusUnitsEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProximityRadiusUnitsEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProximityRadiusUnitsEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum} ProximityRadiusUnitsEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProximityRadiusUnitsEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProximityRadiusUnitsEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum} ProximityRadiusUnitsEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProximityRadiusUnitsEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProximityRadiusUnitsEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProximityRadiusUnitsEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProximityRadiusUnitsEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum} ProximityRadiusUnitsEnum
                             */
                            ProximityRadiusUnitsEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProximityRadiusUnitsEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum} m ProximityRadiusUnitsEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProximityRadiusUnitsEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProximityRadiusUnitsEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProximityRadiusUnitsEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProximityRadiusUnits enum.
                             * @name google.ads.googleads.v11.enums.ProximityRadiusUnitsEnum.ProximityRadiusUnits
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MILES=2 MILES value
                             * @property {number} KILOMETERS=3 KILOMETERS value
                             */
                            ProximityRadiusUnitsEnum.ProximityRadiusUnits = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MILES"] = 2;
                                values[valuesById[3] = "KILOMETERS"] = 3;
                                return values;
                            })();
    
                            return ProximityRadiusUnitsEnum;
                        })();
    
                        enums.WebpageConditionOperandEnum = (function() {
    
                            /**
                             * Properties of a WebpageConditionOperandEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IWebpageConditionOperandEnum
                             */
    
                            /**
                             * Constructs a new WebpageConditionOperandEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a WebpageConditionOperandEnum.
                             * @implements IWebpageConditionOperandEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperandEnum=} [p] Properties to set
                             */
                            function WebpageConditionOperandEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new WebpageConditionOperandEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperandEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperandEnum} WebpageConditionOperandEnum instance
                             */
                            WebpageConditionOperandEnum.create = function create(properties) {
                                return new WebpageConditionOperandEnum(properties);
                            };
    
                            /**
                             * Encodes the specified WebpageConditionOperandEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.WebpageConditionOperandEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperandEnum} m WebpageConditionOperandEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageConditionOperandEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified WebpageConditionOperandEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.WebpageConditionOperandEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperandEnum} message WebpageConditionOperandEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageConditionOperandEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a WebpageConditionOperandEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperandEnum} WebpageConditionOperandEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageConditionOperandEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.WebpageConditionOperandEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a WebpageConditionOperandEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperandEnum} WebpageConditionOperandEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageConditionOperandEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a WebpageConditionOperandEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            WebpageConditionOperandEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a WebpageConditionOperandEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperandEnum} WebpageConditionOperandEnum
                             */
                            WebpageConditionOperandEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.WebpageConditionOperandEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.WebpageConditionOperandEnum();
                            };
    
                            /**
                             * Creates a plain object from a WebpageConditionOperandEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.WebpageConditionOperandEnum} m WebpageConditionOperandEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            WebpageConditionOperandEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this WebpageConditionOperandEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperandEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            WebpageConditionOperandEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * WebpageConditionOperand enum.
                             * @name google.ads.googleads.v11.enums.WebpageConditionOperandEnum.WebpageConditionOperand
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} URL=2 URL value
                             * @property {number} CATEGORY=3 CATEGORY value
                             * @property {number} PAGE_TITLE=4 PAGE_TITLE value
                             * @property {number} PAGE_CONTENT=5 PAGE_CONTENT value
                             * @property {number} CUSTOM_LABEL=6 CUSTOM_LABEL value
                             */
                            WebpageConditionOperandEnum.WebpageConditionOperand = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "URL"] = 2;
                                values[valuesById[3] = "CATEGORY"] = 3;
                                values[valuesById[4] = "PAGE_TITLE"] = 4;
                                values[valuesById[5] = "PAGE_CONTENT"] = 5;
                                values[valuesById[6] = "CUSTOM_LABEL"] = 6;
                                return values;
                            })();
    
                            return WebpageConditionOperandEnum;
                        })();
    
                        enums.WebpageConditionOperatorEnum = (function() {
    
                            /**
                             * Properties of a WebpageConditionOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IWebpageConditionOperatorEnum
                             */
    
                            /**
                             * Constructs a new WebpageConditionOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a WebpageConditionOperatorEnum.
                             * @implements IWebpageConditionOperatorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperatorEnum=} [p] Properties to set
                             */
                            function WebpageConditionOperatorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new WebpageConditionOperatorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperatorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperatorEnum} WebpageConditionOperatorEnum instance
                             */
                            WebpageConditionOperatorEnum.create = function create(properties) {
                                return new WebpageConditionOperatorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified WebpageConditionOperatorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.WebpageConditionOperatorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperatorEnum} m WebpageConditionOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageConditionOperatorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified WebpageConditionOperatorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.WebpageConditionOperatorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IWebpageConditionOperatorEnum} message WebpageConditionOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageConditionOperatorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a WebpageConditionOperatorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperatorEnum} WebpageConditionOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageConditionOperatorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.WebpageConditionOperatorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a WebpageConditionOperatorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperatorEnum} WebpageConditionOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageConditionOperatorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a WebpageConditionOperatorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            WebpageConditionOperatorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a WebpageConditionOperatorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.WebpageConditionOperatorEnum} WebpageConditionOperatorEnum
                             */
                            WebpageConditionOperatorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.WebpageConditionOperatorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.WebpageConditionOperatorEnum();
                            };
    
                            /**
                             * Creates a plain object from a WebpageConditionOperatorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.WebpageConditionOperatorEnum} m WebpageConditionOperatorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            WebpageConditionOperatorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this WebpageConditionOperatorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.WebpageConditionOperatorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            WebpageConditionOperatorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * WebpageConditionOperator enum.
                             * @name google.ads.googleads.v11.enums.WebpageConditionOperatorEnum.WebpageConditionOperator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EQUALS=2 EQUALS value
                             * @property {number} CONTAINS=3 CONTAINS value
                             */
                            WebpageConditionOperatorEnum.WebpageConditionOperator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EQUALS"] = 2;
                                values[valuesById[3] = "CONTAINS"] = 3;
                                return values;
                            })();
    
                            return WebpageConditionOperatorEnum;
                        })();
    
                        enums.CallToActionTypeEnum = (function() {
    
                            /**
                             * Properties of a CallToActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICallToActionTypeEnum
                             */
    
                            /**
                             * Constructs a new CallToActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CallToActionTypeEnum.
                             * @implements ICallToActionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICallToActionTypeEnum=} [p] Properties to set
                             */
                            function CallToActionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CallToActionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallToActionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CallToActionTypeEnum} CallToActionTypeEnum instance
                             */
                            CallToActionTypeEnum.create = function create(properties) {
                                return new CallToActionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CallToActionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CallToActionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallToActionTypeEnum} m CallToActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallToActionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallToActionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CallToActionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallToActionTypeEnum} message CallToActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallToActionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallToActionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CallToActionTypeEnum} CallToActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallToActionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CallToActionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallToActionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CallToActionTypeEnum} CallToActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallToActionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallToActionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallToActionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CallToActionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CallToActionTypeEnum} CallToActionTypeEnum
                             */
                            CallToActionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CallToActionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CallToActionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CallToActionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CallToActionTypeEnum} m CallToActionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallToActionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CallToActionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CallToActionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallToActionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CallToActionType enum.
                             * @name google.ads.googleads.v11.enums.CallToActionTypeEnum.CallToActionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LEARN_MORE=2 LEARN_MORE value
                             * @property {number} GET_QUOTE=3 GET_QUOTE value
                             * @property {number} APPLY_NOW=4 APPLY_NOW value
                             * @property {number} SIGN_UP=5 SIGN_UP value
                             * @property {number} CONTACT_US=6 CONTACT_US value
                             * @property {number} SUBSCRIBE=7 SUBSCRIBE value
                             * @property {number} DOWNLOAD=8 DOWNLOAD value
                             * @property {number} BOOK_NOW=9 BOOK_NOW value
                             * @property {number} SHOP_NOW=10 SHOP_NOW value
                             */
                            CallToActionTypeEnum.CallToActionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LEARN_MORE"] = 2;
                                values[valuesById[3] = "GET_QUOTE"] = 3;
                                values[valuesById[4] = "APPLY_NOW"] = 4;
                                values[valuesById[5] = "SIGN_UP"] = 5;
                                values[valuesById[6] = "CONTACT_US"] = 6;
                                values[valuesById[7] = "SUBSCRIBE"] = 7;
                                values[valuesById[8] = "DOWNLOAD"] = 8;
                                values[valuesById[9] = "BOOK_NOW"] = 9;
                                values[valuesById[10] = "SHOP_NOW"] = 10;
                                return values;
                            })();
    
                            return CallToActionTypeEnum;
                        })();
    
                        enums.LeadFormCallToActionTypeEnum = (function() {
    
                            /**
                             * Properties of a LeadFormCallToActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILeadFormCallToActionTypeEnum
                             */
    
                            /**
                             * Constructs a new LeadFormCallToActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LeadFormCallToActionTypeEnum.
                             * @implements ILeadFormCallToActionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILeadFormCallToActionTypeEnum=} [p] Properties to set
                             */
                            function LeadFormCallToActionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LeadFormCallToActionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormCallToActionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum} LeadFormCallToActionTypeEnum instance
                             */
                            LeadFormCallToActionTypeEnum.create = function create(properties) {
                                return new LeadFormCallToActionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormCallToActionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormCallToActionTypeEnum} m LeadFormCallToActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormCallToActionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormCallToActionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormCallToActionTypeEnum} message LeadFormCallToActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormCallToActionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormCallToActionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum} LeadFormCallToActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormCallToActionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormCallToActionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum} LeadFormCallToActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormCallToActionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormCallToActionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormCallToActionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormCallToActionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum} LeadFormCallToActionTypeEnum
                             */
                            LeadFormCallToActionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a LeadFormCallToActionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum} m LeadFormCallToActionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormCallToActionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LeadFormCallToActionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormCallToActionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LeadFormCallToActionType enum.
                             * @name google.ads.googleads.v11.enums.LeadFormCallToActionTypeEnum.LeadFormCallToActionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LEARN_MORE=2 LEARN_MORE value
                             * @property {number} GET_QUOTE=3 GET_QUOTE value
                             * @property {number} APPLY_NOW=4 APPLY_NOW value
                             * @property {number} SIGN_UP=5 SIGN_UP value
                             * @property {number} CONTACT_US=6 CONTACT_US value
                             * @property {number} SUBSCRIBE=7 SUBSCRIBE value
                             * @property {number} DOWNLOAD=8 DOWNLOAD value
                             * @property {number} BOOK_NOW=9 BOOK_NOW value
                             * @property {number} GET_OFFER=10 GET_OFFER value
                             * @property {number} REGISTER=11 REGISTER value
                             * @property {number} GET_INFO=12 GET_INFO value
                             * @property {number} REQUEST_DEMO=13 REQUEST_DEMO value
                             * @property {number} JOIN_NOW=14 JOIN_NOW value
                             * @property {number} GET_STARTED=15 GET_STARTED value
                             */
                            LeadFormCallToActionTypeEnum.LeadFormCallToActionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LEARN_MORE"] = 2;
                                values[valuesById[3] = "GET_QUOTE"] = 3;
                                values[valuesById[4] = "APPLY_NOW"] = 4;
                                values[valuesById[5] = "SIGN_UP"] = 5;
                                values[valuesById[6] = "CONTACT_US"] = 6;
                                values[valuesById[7] = "SUBSCRIBE"] = 7;
                                values[valuesById[8] = "DOWNLOAD"] = 8;
                                values[valuesById[9] = "BOOK_NOW"] = 9;
                                values[valuesById[10] = "GET_OFFER"] = 10;
                                values[valuesById[11] = "REGISTER"] = 11;
                                values[valuesById[12] = "GET_INFO"] = 12;
                                values[valuesById[13] = "REQUEST_DEMO"] = 13;
                                values[valuesById[14] = "JOIN_NOW"] = 14;
                                values[valuesById[15] = "GET_STARTED"] = 15;
                                return values;
                            })();
    
                            return LeadFormCallToActionTypeEnum;
                        })();
    
                        enums.LeadFormDesiredIntentEnum = (function() {
    
                            /**
                             * Properties of a LeadFormDesiredIntentEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILeadFormDesiredIntentEnum
                             */
    
                            /**
                             * Constructs a new LeadFormDesiredIntentEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LeadFormDesiredIntentEnum.
                             * @implements ILeadFormDesiredIntentEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILeadFormDesiredIntentEnum=} [p] Properties to set
                             */
                            function LeadFormDesiredIntentEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LeadFormDesiredIntentEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormDesiredIntentEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum} LeadFormDesiredIntentEnum instance
                             */
                            LeadFormDesiredIntentEnum.create = function create(properties) {
                                return new LeadFormDesiredIntentEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormDesiredIntentEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormDesiredIntentEnum} m LeadFormDesiredIntentEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormDesiredIntentEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormDesiredIntentEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormDesiredIntentEnum} message LeadFormDesiredIntentEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormDesiredIntentEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormDesiredIntentEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum} LeadFormDesiredIntentEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormDesiredIntentEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormDesiredIntentEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum} LeadFormDesiredIntentEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormDesiredIntentEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormDesiredIntentEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormDesiredIntentEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormDesiredIntentEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum} LeadFormDesiredIntentEnum
                             */
                            LeadFormDesiredIntentEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum();
                            };
    
                            /**
                             * Creates a plain object from a LeadFormDesiredIntentEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum} m LeadFormDesiredIntentEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormDesiredIntentEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LeadFormDesiredIntentEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormDesiredIntentEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LeadFormDesiredIntent enum.
                             * @name google.ads.googleads.v11.enums.LeadFormDesiredIntentEnum.LeadFormDesiredIntent
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LOW_INTENT=2 LOW_INTENT value
                             * @property {number} HIGH_INTENT=3 HIGH_INTENT value
                             */
                            LeadFormDesiredIntentEnum.LeadFormDesiredIntent = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LOW_INTENT"] = 2;
                                values[valuesById[3] = "HIGH_INTENT"] = 3;
                                return values;
                            })();
    
                            return LeadFormDesiredIntentEnum;
                        })();
    
                        enums.LeadFormFieldUserInputTypeEnum = (function() {
    
                            /**
                             * Properties of a LeadFormFieldUserInputTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILeadFormFieldUserInputTypeEnum
                             */
    
                            /**
                             * Constructs a new LeadFormFieldUserInputTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LeadFormFieldUserInputTypeEnum.
                             * @implements ILeadFormFieldUserInputTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILeadFormFieldUserInputTypeEnum=} [p] Properties to set
                             */
                            function LeadFormFieldUserInputTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LeadFormFieldUserInputTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormFieldUserInputTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum} LeadFormFieldUserInputTypeEnum instance
                             */
                            LeadFormFieldUserInputTypeEnum.create = function create(properties) {
                                return new LeadFormFieldUserInputTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormFieldUserInputTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormFieldUserInputTypeEnum} m LeadFormFieldUserInputTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormFieldUserInputTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormFieldUserInputTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormFieldUserInputTypeEnum} message LeadFormFieldUserInputTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormFieldUserInputTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormFieldUserInputTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum} LeadFormFieldUserInputTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormFieldUserInputTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormFieldUserInputTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum} LeadFormFieldUserInputTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormFieldUserInputTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormFieldUserInputTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormFieldUserInputTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormFieldUserInputTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum} LeadFormFieldUserInputTypeEnum
                             */
                            LeadFormFieldUserInputTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a LeadFormFieldUserInputTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum} m LeadFormFieldUserInputTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormFieldUserInputTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LeadFormFieldUserInputTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormFieldUserInputTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LeadFormFieldUserInputType enum.
                             * @name google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FULL_NAME=2 FULL_NAME value
                             * @property {number} EMAIL=3 EMAIL value
                             * @property {number} PHONE_NUMBER=4 PHONE_NUMBER value
                             * @property {number} POSTAL_CODE=5 POSTAL_CODE value
                             * @property {number} CITY=9 CITY value
                             * @property {number} REGION=10 REGION value
                             * @property {number} COUNTRY=11 COUNTRY value
                             * @property {number} WORK_EMAIL=12 WORK_EMAIL value
                             * @property {number} COMPANY_NAME=13 COMPANY_NAME value
                             * @property {number} WORK_PHONE=14 WORK_PHONE value
                             * @property {number} JOB_TITLE=15 JOB_TITLE value
                             * @property {number} GOVERNMENT_ISSUED_ID_CPF_BR=16 GOVERNMENT_ISSUED_ID_CPF_BR value
                             * @property {number} GOVERNMENT_ISSUED_ID_DNI_AR=17 GOVERNMENT_ISSUED_ID_DNI_AR value
                             * @property {number} GOVERNMENT_ISSUED_ID_DNI_PE=18 GOVERNMENT_ISSUED_ID_DNI_PE value
                             * @property {number} GOVERNMENT_ISSUED_ID_RUT_CL=19 GOVERNMENT_ISSUED_ID_RUT_CL value
                             * @property {number} GOVERNMENT_ISSUED_ID_CC_CO=20 GOVERNMENT_ISSUED_ID_CC_CO value
                             * @property {number} GOVERNMENT_ISSUED_ID_CI_EC=21 GOVERNMENT_ISSUED_ID_CI_EC value
                             * @property {number} GOVERNMENT_ISSUED_ID_RFC_MX=22 GOVERNMENT_ISSUED_ID_RFC_MX value
                             * @property {number} FIRST_NAME=23 FIRST_NAME value
                             * @property {number} LAST_NAME=24 LAST_NAME value
                             * @property {number} VEHICLE_MODEL=1001 VEHICLE_MODEL value
                             * @property {number} VEHICLE_TYPE=1002 VEHICLE_TYPE value
                             * @property {number} PREFERRED_DEALERSHIP=1003 PREFERRED_DEALERSHIP value
                             * @property {number} VEHICLE_PURCHASE_TIMELINE=1004 VEHICLE_PURCHASE_TIMELINE value
                             * @property {number} VEHICLE_OWNERSHIP=1005 VEHICLE_OWNERSHIP value
                             * @property {number} VEHICLE_PAYMENT_TYPE=1009 VEHICLE_PAYMENT_TYPE value
                             * @property {number} VEHICLE_CONDITION=1010 VEHICLE_CONDITION value
                             * @property {number} COMPANY_SIZE=1006 COMPANY_SIZE value
                             * @property {number} ANNUAL_SALES=1007 ANNUAL_SALES value
                             * @property {number} YEARS_IN_BUSINESS=1008 YEARS_IN_BUSINESS value
                             * @property {number} JOB_DEPARTMENT=1011 JOB_DEPARTMENT value
                             * @property {number} JOB_ROLE=1012 JOB_ROLE value
                             * @property {number} EDUCATION_PROGRAM=1013 EDUCATION_PROGRAM value
                             * @property {number} EDUCATION_COURSE=1014 EDUCATION_COURSE value
                             * @property {number} PRODUCT=1016 PRODUCT value
                             * @property {number} SERVICE=1017 SERVICE value
                             * @property {number} OFFER=1018 OFFER value
                             * @property {number} CATEGORY=1019 CATEGORY value
                             * @property {number} PREFERRED_CONTACT_METHOD=1020 PREFERRED_CONTACT_METHOD value
                             * @property {number} PREFERRED_LOCATION=1021 PREFERRED_LOCATION value
                             * @property {number} PREFERRED_CONTACT_TIME=1022 PREFERRED_CONTACT_TIME value
                             * @property {number} PURCHASE_TIMELINE=1023 PURCHASE_TIMELINE value
                             * @property {number} YEARS_OF_EXPERIENCE=1048 YEARS_OF_EXPERIENCE value
                             * @property {number} JOB_INDUSTRY=1049 JOB_INDUSTRY value
                             * @property {number} LEVEL_OF_EDUCATION=1050 LEVEL_OF_EDUCATION value
                             * @property {number} PROPERTY_TYPE=1024 PROPERTY_TYPE value
                             * @property {number} REALTOR_HELP_GOAL=1025 REALTOR_HELP_GOAL value
                             * @property {number} PROPERTY_COMMUNITY=1026 PROPERTY_COMMUNITY value
                             * @property {number} PRICE_RANGE=1027 PRICE_RANGE value
                             * @property {number} NUMBER_OF_BEDROOMS=1028 NUMBER_OF_BEDROOMS value
                             * @property {number} FURNISHED_PROPERTY=1029 FURNISHED_PROPERTY value
                             * @property {number} PETS_ALLOWED_PROPERTY=1030 PETS_ALLOWED_PROPERTY value
                             * @property {number} NEXT_PLANNED_PURCHASE=1031 NEXT_PLANNED_PURCHASE value
                             * @property {number} EVENT_SIGNUP_INTEREST=1033 EVENT_SIGNUP_INTEREST value
                             * @property {number} PREFERRED_SHOPPING_PLACES=1034 PREFERRED_SHOPPING_PLACES value
                             * @property {number} FAVORITE_BRAND=1035 FAVORITE_BRAND value
                             * @property {number} TRANSPORTATION_COMMERCIAL_LICENSE_TYPE=1036 TRANSPORTATION_COMMERCIAL_LICENSE_TYPE value
                             * @property {number} EVENT_BOOKING_INTEREST=1038 EVENT_BOOKING_INTEREST value
                             * @property {number} DESTINATION_COUNTRY=1039 DESTINATION_COUNTRY value
                             * @property {number} DESTINATION_CITY=1040 DESTINATION_CITY value
                             * @property {number} DEPARTURE_COUNTRY=1041 DEPARTURE_COUNTRY value
                             * @property {number} DEPARTURE_CITY=1042 DEPARTURE_CITY value
                             * @property {number} DEPARTURE_DATE=1043 DEPARTURE_DATE value
                             * @property {number} RETURN_DATE=1044 RETURN_DATE value
                             * @property {number} NUMBER_OF_TRAVELERS=1045 NUMBER_OF_TRAVELERS value
                             * @property {number} TRAVEL_BUDGET=1046 TRAVEL_BUDGET value
                             * @property {number} TRAVEL_ACCOMMODATION=1047 TRAVEL_ACCOMMODATION value
                             */
                            LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FULL_NAME"] = 2;
                                values[valuesById[3] = "EMAIL"] = 3;
                                values[valuesById[4] = "PHONE_NUMBER"] = 4;
                                values[valuesById[5] = "POSTAL_CODE"] = 5;
                                values[valuesById[9] = "CITY"] = 9;
                                values[valuesById[10] = "REGION"] = 10;
                                values[valuesById[11] = "COUNTRY"] = 11;
                                values[valuesById[12] = "WORK_EMAIL"] = 12;
                                values[valuesById[13] = "COMPANY_NAME"] = 13;
                                values[valuesById[14] = "WORK_PHONE"] = 14;
                                values[valuesById[15] = "JOB_TITLE"] = 15;
                                values[valuesById[16] = "GOVERNMENT_ISSUED_ID_CPF_BR"] = 16;
                                values[valuesById[17] = "GOVERNMENT_ISSUED_ID_DNI_AR"] = 17;
                                values[valuesById[18] = "GOVERNMENT_ISSUED_ID_DNI_PE"] = 18;
                                values[valuesById[19] = "GOVERNMENT_ISSUED_ID_RUT_CL"] = 19;
                                values[valuesById[20] = "GOVERNMENT_ISSUED_ID_CC_CO"] = 20;
                                values[valuesById[21] = "GOVERNMENT_ISSUED_ID_CI_EC"] = 21;
                                values[valuesById[22] = "GOVERNMENT_ISSUED_ID_RFC_MX"] = 22;
                                values[valuesById[23] = "FIRST_NAME"] = 23;
                                values[valuesById[24] = "LAST_NAME"] = 24;
                                values[valuesById[1001] = "VEHICLE_MODEL"] = 1001;
                                values[valuesById[1002] = "VEHICLE_TYPE"] = 1002;
                                values[valuesById[1003] = "PREFERRED_DEALERSHIP"] = 1003;
                                values[valuesById[1004] = "VEHICLE_PURCHASE_TIMELINE"] = 1004;
                                values[valuesById[1005] = "VEHICLE_OWNERSHIP"] = 1005;
                                values[valuesById[1009] = "VEHICLE_PAYMENT_TYPE"] = 1009;
                                values[valuesById[1010] = "VEHICLE_CONDITION"] = 1010;
                                values[valuesById[1006] = "COMPANY_SIZE"] = 1006;
                                values[valuesById[1007] = "ANNUAL_SALES"] = 1007;
                                values[valuesById[1008] = "YEARS_IN_BUSINESS"] = 1008;
                                values[valuesById[1011] = "JOB_DEPARTMENT"] = 1011;
                                values[valuesById[1012] = "JOB_ROLE"] = 1012;
                                values[valuesById[1013] = "EDUCATION_PROGRAM"] = 1013;
                                values[valuesById[1014] = "EDUCATION_COURSE"] = 1014;
                                values[valuesById[1016] = "PRODUCT"] = 1016;
                                values[valuesById[1017] = "SERVICE"] = 1017;
                                values[valuesById[1018] = "OFFER"] = 1018;
                                values[valuesById[1019] = "CATEGORY"] = 1019;
                                values[valuesById[1020] = "PREFERRED_CONTACT_METHOD"] = 1020;
                                values[valuesById[1021] = "PREFERRED_LOCATION"] = 1021;
                                values[valuesById[1022] = "PREFERRED_CONTACT_TIME"] = 1022;
                                values[valuesById[1023] = "PURCHASE_TIMELINE"] = 1023;
                                values[valuesById[1048] = "YEARS_OF_EXPERIENCE"] = 1048;
                                values[valuesById[1049] = "JOB_INDUSTRY"] = 1049;
                                values[valuesById[1050] = "LEVEL_OF_EDUCATION"] = 1050;
                                values[valuesById[1024] = "PROPERTY_TYPE"] = 1024;
                                values[valuesById[1025] = "REALTOR_HELP_GOAL"] = 1025;
                                values[valuesById[1026] = "PROPERTY_COMMUNITY"] = 1026;
                                values[valuesById[1027] = "PRICE_RANGE"] = 1027;
                                values[valuesById[1028] = "NUMBER_OF_BEDROOMS"] = 1028;
                                values[valuesById[1029] = "FURNISHED_PROPERTY"] = 1029;
                                values[valuesById[1030] = "PETS_ALLOWED_PROPERTY"] = 1030;
                                values[valuesById[1031] = "NEXT_PLANNED_PURCHASE"] = 1031;
                                values[valuesById[1033] = "EVENT_SIGNUP_INTEREST"] = 1033;
                                values[valuesById[1034] = "PREFERRED_SHOPPING_PLACES"] = 1034;
                                values[valuesById[1035] = "FAVORITE_BRAND"] = 1035;
                                values[valuesById[1036] = "TRANSPORTATION_COMMERCIAL_LICENSE_TYPE"] = 1036;
                                values[valuesById[1038] = "EVENT_BOOKING_INTEREST"] = 1038;
                                values[valuesById[1039] = "DESTINATION_COUNTRY"] = 1039;
                                values[valuesById[1040] = "DESTINATION_CITY"] = 1040;
                                values[valuesById[1041] = "DEPARTURE_COUNTRY"] = 1041;
                                values[valuesById[1042] = "DEPARTURE_CITY"] = 1042;
                                values[valuesById[1043] = "DEPARTURE_DATE"] = 1043;
                                values[valuesById[1044] = "RETURN_DATE"] = 1044;
                                values[valuesById[1045] = "NUMBER_OF_TRAVELERS"] = 1045;
                                values[valuesById[1046] = "TRAVEL_BUDGET"] = 1046;
                                values[valuesById[1047] = "TRAVEL_ACCOMMODATION"] = 1047;
                                return values;
                            })();
    
                            return LeadFormFieldUserInputTypeEnum;
                        })();
    
                        enums.LeadFormPostSubmitCallToActionTypeEnum = (function() {
    
                            /**
                             * Properties of a LeadFormPostSubmitCallToActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILeadFormPostSubmitCallToActionTypeEnum
                             */
    
                            /**
                             * Constructs a new LeadFormPostSubmitCallToActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LeadFormPostSubmitCallToActionTypeEnum.
                             * @implements ILeadFormPostSubmitCallToActionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILeadFormPostSubmitCallToActionTypeEnum=} [p] Properties to set
                             */
                            function LeadFormPostSubmitCallToActionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LeadFormPostSubmitCallToActionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormPostSubmitCallToActionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum} LeadFormPostSubmitCallToActionTypeEnum instance
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.create = function create(properties) {
                                return new LeadFormPostSubmitCallToActionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormPostSubmitCallToActionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormPostSubmitCallToActionTypeEnum} m LeadFormPostSubmitCallToActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormPostSubmitCallToActionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILeadFormPostSubmitCallToActionTypeEnum} message LeadFormPostSubmitCallToActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormPostSubmitCallToActionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum} LeadFormPostSubmitCallToActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormPostSubmitCallToActionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum} LeadFormPostSubmitCallToActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormPostSubmitCallToActionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormPostSubmitCallToActionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum} LeadFormPostSubmitCallToActionTypeEnum
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a LeadFormPostSubmitCallToActionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum} m LeadFormPostSubmitCallToActionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LeadFormPostSubmitCallToActionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LeadFormPostSubmitCallToActionType enum.
                             * @name google.ads.googleads.v11.enums.LeadFormPostSubmitCallToActionTypeEnum.LeadFormPostSubmitCallToActionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} VISIT_SITE=2 VISIT_SITE value
                             * @property {number} DOWNLOAD=3 DOWNLOAD value
                             * @property {number} LEARN_MORE=4 LEARN_MORE value
                             * @property {number} SHOP_NOW=5 SHOP_NOW value
                             */
                            LeadFormPostSubmitCallToActionTypeEnum.LeadFormPostSubmitCallToActionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "VISIT_SITE"] = 2;
                                values[valuesById[3] = "DOWNLOAD"] = 3;
                                values[valuesById[4] = "LEARN_MORE"] = 4;
                                values[valuesById[5] = "SHOP_NOW"] = 5;
                                return values;
                            })();
    
                            return LeadFormPostSubmitCallToActionTypeEnum;
                        })();
    
                        enums.MobileAppVendorEnum = (function() {
    
                            /**
                             * Properties of a MobileAppVendorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMobileAppVendorEnum
                             */
    
                            /**
                             * Constructs a new MobileAppVendorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MobileAppVendorEnum.
                             * @implements IMobileAppVendorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMobileAppVendorEnum=} [p] Properties to set
                             */
                            function MobileAppVendorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MobileAppVendorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMobileAppVendorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MobileAppVendorEnum} MobileAppVendorEnum instance
                             */
                            MobileAppVendorEnum.create = function create(properties) {
                                return new MobileAppVendorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MobileAppVendorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MobileAppVendorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMobileAppVendorEnum} m MobileAppVendorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppVendorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileAppVendorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MobileAppVendorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMobileAppVendorEnum} message MobileAppVendorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppVendorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileAppVendorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MobileAppVendorEnum} MobileAppVendorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppVendorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MobileAppVendorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileAppVendorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MobileAppVendorEnum} MobileAppVendorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppVendorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileAppVendorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileAppVendorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MobileAppVendorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MobileAppVendorEnum} MobileAppVendorEnum
                             */
                            MobileAppVendorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MobileAppVendorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MobileAppVendorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MobileAppVendorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MobileAppVendorEnum} m MobileAppVendorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileAppVendorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MobileAppVendorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MobileAppVendorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileAppVendorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MobileAppVendor enum.
                             * @name google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APPLE_APP_STORE=2 APPLE_APP_STORE value
                             * @property {number} GOOGLE_APP_STORE=3 GOOGLE_APP_STORE value
                             */
                            MobileAppVendorEnum.MobileAppVendor = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APPLE_APP_STORE"] = 2;
                                values[valuesById[3] = "GOOGLE_APP_STORE"] = 3;
                                return values;
                            })();
    
                            return MobileAppVendorEnum;
                        })();
    
                        enums.PriceExtensionPriceQualifierEnum = (function() {
    
                            /**
                             * Properties of a PriceExtensionPriceQualifierEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPriceExtensionPriceQualifierEnum
                             */
    
                            /**
                             * Constructs a new PriceExtensionPriceQualifierEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PriceExtensionPriceQualifierEnum.
                             * @implements IPriceExtensionPriceQualifierEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceQualifierEnum=} [p] Properties to set
                             */
                            function PriceExtensionPriceQualifierEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PriceExtensionPriceQualifierEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceQualifierEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum} PriceExtensionPriceQualifierEnum instance
                             */
                            PriceExtensionPriceQualifierEnum.create = function create(properties) {
                                return new PriceExtensionPriceQualifierEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PriceExtensionPriceQualifierEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceQualifierEnum} m PriceExtensionPriceQualifierEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceExtensionPriceQualifierEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PriceExtensionPriceQualifierEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceQualifierEnum} message PriceExtensionPriceQualifierEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceExtensionPriceQualifierEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PriceExtensionPriceQualifierEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum} PriceExtensionPriceQualifierEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceExtensionPriceQualifierEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PriceExtensionPriceQualifierEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum} PriceExtensionPriceQualifierEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceExtensionPriceQualifierEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PriceExtensionPriceQualifierEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PriceExtensionPriceQualifierEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PriceExtensionPriceQualifierEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum} PriceExtensionPriceQualifierEnum
                             */
                            PriceExtensionPriceQualifierEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum();
                            };
    
                            /**
                             * Creates a plain object from a PriceExtensionPriceQualifierEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum} m PriceExtensionPriceQualifierEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PriceExtensionPriceQualifierEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PriceExtensionPriceQualifierEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PriceExtensionPriceQualifierEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PriceExtensionPriceQualifier enum.
                             * @name google.ads.googleads.v11.enums.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FROM=2 FROM value
                             * @property {number} UP_TO=3 UP_TO value
                             * @property {number} AVERAGE=4 AVERAGE value
                             */
                            PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FROM"] = 2;
                                values[valuesById[3] = "UP_TO"] = 3;
                                values[valuesById[4] = "AVERAGE"] = 4;
                                return values;
                            })();
    
                            return PriceExtensionPriceQualifierEnum;
                        })();
    
                        enums.PriceExtensionPriceUnitEnum = (function() {
    
                            /**
                             * Properties of a PriceExtensionPriceUnitEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPriceExtensionPriceUnitEnum
                             */
    
                            /**
                             * Constructs a new PriceExtensionPriceUnitEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PriceExtensionPriceUnitEnum.
                             * @implements IPriceExtensionPriceUnitEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceUnitEnum=} [p] Properties to set
                             */
                            function PriceExtensionPriceUnitEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PriceExtensionPriceUnitEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceUnitEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum} PriceExtensionPriceUnitEnum instance
                             */
                            PriceExtensionPriceUnitEnum.create = function create(properties) {
                                return new PriceExtensionPriceUnitEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PriceExtensionPriceUnitEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceUnitEnum} m PriceExtensionPriceUnitEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceExtensionPriceUnitEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PriceExtensionPriceUnitEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionPriceUnitEnum} message PriceExtensionPriceUnitEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceExtensionPriceUnitEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PriceExtensionPriceUnitEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum} PriceExtensionPriceUnitEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceExtensionPriceUnitEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PriceExtensionPriceUnitEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum} PriceExtensionPriceUnitEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceExtensionPriceUnitEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PriceExtensionPriceUnitEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PriceExtensionPriceUnitEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PriceExtensionPriceUnitEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum} PriceExtensionPriceUnitEnum
                             */
                            PriceExtensionPriceUnitEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum();
                            };
    
                            /**
                             * Creates a plain object from a PriceExtensionPriceUnitEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum} m PriceExtensionPriceUnitEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PriceExtensionPriceUnitEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PriceExtensionPriceUnitEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PriceExtensionPriceUnitEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PriceExtensionPriceUnit enum.
                             * @name google.ads.googleads.v11.enums.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PER_HOUR=2 PER_HOUR value
                             * @property {number} PER_DAY=3 PER_DAY value
                             * @property {number} PER_WEEK=4 PER_WEEK value
                             * @property {number} PER_MONTH=5 PER_MONTH value
                             * @property {number} PER_YEAR=6 PER_YEAR value
                             * @property {number} PER_NIGHT=7 PER_NIGHT value
                             */
                            PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PER_HOUR"] = 2;
                                values[valuesById[3] = "PER_DAY"] = 3;
                                values[valuesById[4] = "PER_WEEK"] = 4;
                                values[valuesById[5] = "PER_MONTH"] = 5;
                                values[valuesById[6] = "PER_YEAR"] = 6;
                                values[valuesById[7] = "PER_NIGHT"] = 7;
                                return values;
                            })();
    
                            return PriceExtensionPriceUnitEnum;
                        })();
    
                        enums.PriceExtensionTypeEnum = (function() {
    
                            /**
                             * Properties of a PriceExtensionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPriceExtensionTypeEnum
                             */
    
                            /**
                             * Constructs a new PriceExtensionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PriceExtensionTypeEnum.
                             * @implements IPriceExtensionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionTypeEnum=} [p] Properties to set
                             */
                            function PriceExtensionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PriceExtensionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionTypeEnum} PriceExtensionTypeEnum instance
                             */
                            PriceExtensionTypeEnum.create = function create(properties) {
                                return new PriceExtensionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PriceExtensionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PriceExtensionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionTypeEnum} m PriceExtensionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceExtensionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PriceExtensionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PriceExtensionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPriceExtensionTypeEnum} message PriceExtensionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PriceExtensionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PriceExtensionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionTypeEnum} PriceExtensionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceExtensionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PriceExtensionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PriceExtensionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionTypeEnum} PriceExtensionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PriceExtensionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PriceExtensionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PriceExtensionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PriceExtensionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PriceExtensionTypeEnum} PriceExtensionTypeEnum
                             */
                            PriceExtensionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PriceExtensionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PriceExtensionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PriceExtensionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PriceExtensionTypeEnum} m PriceExtensionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PriceExtensionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PriceExtensionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PriceExtensionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PriceExtensionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PriceExtensionType enum.
                             * @name google.ads.googleads.v11.enums.PriceExtensionTypeEnum.PriceExtensionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BRANDS=2 BRANDS value
                             * @property {number} EVENTS=3 EVENTS value
                             * @property {number} LOCATIONS=4 LOCATIONS value
                             * @property {number} NEIGHBORHOODS=5 NEIGHBORHOODS value
                             * @property {number} PRODUCT_CATEGORIES=6 PRODUCT_CATEGORIES value
                             * @property {number} PRODUCT_TIERS=7 PRODUCT_TIERS value
                             * @property {number} SERVICES=8 SERVICES value
                             * @property {number} SERVICE_CATEGORIES=9 SERVICE_CATEGORIES value
                             * @property {number} SERVICE_TIERS=10 SERVICE_TIERS value
                             */
                            PriceExtensionTypeEnum.PriceExtensionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BRANDS"] = 2;
                                values[valuesById[3] = "EVENTS"] = 3;
                                values[valuesById[4] = "LOCATIONS"] = 4;
                                values[valuesById[5] = "NEIGHBORHOODS"] = 5;
                                values[valuesById[6] = "PRODUCT_CATEGORIES"] = 6;
                                values[valuesById[7] = "PRODUCT_TIERS"] = 7;
                                values[valuesById[8] = "SERVICES"] = 8;
                                values[valuesById[9] = "SERVICE_CATEGORIES"] = 9;
                                values[valuesById[10] = "SERVICE_TIERS"] = 10;
                                return values;
                            })();
    
                            return PriceExtensionTypeEnum;
                        })();
    
                        enums.PromotionExtensionDiscountModifierEnum = (function() {
    
                            /**
                             * Properties of a PromotionExtensionDiscountModifierEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPromotionExtensionDiscountModifierEnum
                             */
    
                            /**
                             * Constructs a new PromotionExtensionDiscountModifierEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PromotionExtensionDiscountModifierEnum.
                             * @implements IPromotionExtensionDiscountModifierEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionDiscountModifierEnum=} [p] Properties to set
                             */
                            function PromotionExtensionDiscountModifierEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PromotionExtensionDiscountModifierEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionDiscountModifierEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum} PromotionExtensionDiscountModifierEnum instance
                             */
                            PromotionExtensionDiscountModifierEnum.create = function create(properties) {
                                return new PromotionExtensionDiscountModifierEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PromotionExtensionDiscountModifierEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionDiscountModifierEnum} m PromotionExtensionDiscountModifierEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionExtensionDiscountModifierEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromotionExtensionDiscountModifierEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionDiscountModifierEnum} message PromotionExtensionDiscountModifierEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionExtensionDiscountModifierEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromotionExtensionDiscountModifierEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum} PromotionExtensionDiscountModifierEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionExtensionDiscountModifierEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromotionExtensionDiscountModifierEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum} PromotionExtensionDiscountModifierEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionExtensionDiscountModifierEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromotionExtensionDiscountModifierEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromotionExtensionDiscountModifierEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PromotionExtensionDiscountModifierEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum} PromotionExtensionDiscountModifierEnum
                             */
                            PromotionExtensionDiscountModifierEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum();
                            };
    
                            /**
                             * Creates a plain object from a PromotionExtensionDiscountModifierEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum} m PromotionExtensionDiscountModifierEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromotionExtensionDiscountModifierEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PromotionExtensionDiscountModifierEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromotionExtensionDiscountModifierEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PromotionExtensionDiscountModifier enum.
                             * @name google.ads.googleads.v11.enums.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UP_TO=2 UP_TO value
                             */
                            PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UP_TO"] = 2;
                                return values;
                            })();
    
                            return PromotionExtensionDiscountModifierEnum;
                        })();
    
                        enums.PromotionExtensionOccasionEnum = (function() {
    
                            /**
                             * Properties of a PromotionExtensionOccasionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPromotionExtensionOccasionEnum
                             */
    
                            /**
                             * Constructs a new PromotionExtensionOccasionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PromotionExtensionOccasionEnum.
                             * @implements IPromotionExtensionOccasionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionOccasionEnum=} [p] Properties to set
                             */
                            function PromotionExtensionOccasionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PromotionExtensionOccasionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionOccasionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum} PromotionExtensionOccasionEnum instance
                             */
                            PromotionExtensionOccasionEnum.create = function create(properties) {
                                return new PromotionExtensionOccasionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PromotionExtensionOccasionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionOccasionEnum} m PromotionExtensionOccasionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionExtensionOccasionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromotionExtensionOccasionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionExtensionOccasionEnum} message PromotionExtensionOccasionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionExtensionOccasionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromotionExtensionOccasionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum} PromotionExtensionOccasionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionExtensionOccasionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromotionExtensionOccasionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum} PromotionExtensionOccasionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionExtensionOccasionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromotionExtensionOccasionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromotionExtensionOccasionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PromotionExtensionOccasionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum} PromotionExtensionOccasionEnum
                             */
                            PromotionExtensionOccasionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum();
                            };
    
                            /**
                             * Creates a plain object from a PromotionExtensionOccasionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum} m PromotionExtensionOccasionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromotionExtensionOccasionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PromotionExtensionOccasionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromotionExtensionOccasionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PromotionExtensionOccasion enum.
                             * @name google.ads.googleads.v11.enums.PromotionExtensionOccasionEnum.PromotionExtensionOccasion
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NEW_YEARS=2 NEW_YEARS value
                             * @property {number} CHINESE_NEW_YEAR=3 CHINESE_NEW_YEAR value
                             * @property {number} VALENTINES_DAY=4 VALENTINES_DAY value
                             * @property {number} EASTER=5 EASTER value
                             * @property {number} MOTHERS_DAY=6 MOTHERS_DAY value
                             * @property {number} FATHERS_DAY=7 FATHERS_DAY value
                             * @property {number} LABOR_DAY=8 LABOR_DAY value
                             * @property {number} BACK_TO_SCHOOL=9 BACK_TO_SCHOOL value
                             * @property {number} HALLOWEEN=10 HALLOWEEN value
                             * @property {number} BLACK_FRIDAY=11 BLACK_FRIDAY value
                             * @property {number} CYBER_MONDAY=12 CYBER_MONDAY value
                             * @property {number} CHRISTMAS=13 CHRISTMAS value
                             * @property {number} BOXING_DAY=14 BOXING_DAY value
                             * @property {number} INDEPENDENCE_DAY=15 INDEPENDENCE_DAY value
                             * @property {number} NATIONAL_DAY=16 NATIONAL_DAY value
                             * @property {number} END_OF_SEASON=17 END_OF_SEASON value
                             * @property {number} WINTER_SALE=18 WINTER_SALE value
                             * @property {number} SUMMER_SALE=19 SUMMER_SALE value
                             * @property {number} FALL_SALE=20 FALL_SALE value
                             * @property {number} SPRING_SALE=21 SPRING_SALE value
                             * @property {number} RAMADAN=22 RAMADAN value
                             * @property {number} EID_AL_FITR=23 EID_AL_FITR value
                             * @property {number} EID_AL_ADHA=24 EID_AL_ADHA value
                             * @property {number} SINGLES_DAY=25 SINGLES_DAY value
                             * @property {number} WOMENS_DAY=26 WOMENS_DAY value
                             * @property {number} HOLI=27 HOLI value
                             * @property {number} PARENTS_DAY=28 PARENTS_DAY value
                             * @property {number} ST_NICHOLAS_DAY=29 ST_NICHOLAS_DAY value
                             * @property {number} CARNIVAL=30 CARNIVAL value
                             * @property {number} EPIPHANY=31 EPIPHANY value
                             * @property {number} ROSH_HASHANAH=32 ROSH_HASHANAH value
                             * @property {number} PASSOVER=33 PASSOVER value
                             * @property {number} HANUKKAH=34 HANUKKAH value
                             * @property {number} DIWALI=35 DIWALI value
                             * @property {number} NAVRATRI=36 NAVRATRI value
                             * @property {number} SONGKRAN=37 SONGKRAN value
                             * @property {number} YEAR_END_GIFT=38 YEAR_END_GIFT value
                             */
                            PromotionExtensionOccasionEnum.PromotionExtensionOccasion = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NEW_YEARS"] = 2;
                                values[valuesById[3] = "CHINESE_NEW_YEAR"] = 3;
                                values[valuesById[4] = "VALENTINES_DAY"] = 4;
                                values[valuesById[5] = "EASTER"] = 5;
                                values[valuesById[6] = "MOTHERS_DAY"] = 6;
                                values[valuesById[7] = "FATHERS_DAY"] = 7;
                                values[valuesById[8] = "LABOR_DAY"] = 8;
                                values[valuesById[9] = "BACK_TO_SCHOOL"] = 9;
                                values[valuesById[10] = "HALLOWEEN"] = 10;
                                values[valuesById[11] = "BLACK_FRIDAY"] = 11;
                                values[valuesById[12] = "CYBER_MONDAY"] = 12;
                                values[valuesById[13] = "CHRISTMAS"] = 13;
                                values[valuesById[14] = "BOXING_DAY"] = 14;
                                values[valuesById[15] = "INDEPENDENCE_DAY"] = 15;
                                values[valuesById[16] = "NATIONAL_DAY"] = 16;
                                values[valuesById[17] = "END_OF_SEASON"] = 17;
                                values[valuesById[18] = "WINTER_SALE"] = 18;
                                values[valuesById[19] = "SUMMER_SALE"] = 19;
                                values[valuesById[20] = "FALL_SALE"] = 20;
                                values[valuesById[21] = "SPRING_SALE"] = 21;
                                values[valuesById[22] = "RAMADAN"] = 22;
                                values[valuesById[23] = "EID_AL_FITR"] = 23;
                                values[valuesById[24] = "EID_AL_ADHA"] = 24;
                                values[valuesById[25] = "SINGLES_DAY"] = 25;
                                values[valuesById[26] = "WOMENS_DAY"] = 26;
                                values[valuesById[27] = "HOLI"] = 27;
                                values[valuesById[28] = "PARENTS_DAY"] = 28;
                                values[valuesById[29] = "ST_NICHOLAS_DAY"] = 29;
                                values[valuesById[30] = "CARNIVAL"] = 30;
                                values[valuesById[31] = "EPIPHANY"] = 31;
                                values[valuesById[32] = "ROSH_HASHANAH"] = 32;
                                values[valuesById[33] = "PASSOVER"] = 33;
                                values[valuesById[34] = "HANUKKAH"] = 34;
                                values[valuesById[35] = "DIWALI"] = 35;
                                values[valuesById[36] = "NAVRATRI"] = 36;
                                values[valuesById[37] = "SONGKRAN"] = 37;
                                values[valuesById[38] = "YEAR_END_GIFT"] = 38;
                                return values;
                            })();
    
                            return PromotionExtensionOccasionEnum;
                        })();
    
                        enums.TargetImpressionShareLocationEnum = (function() {
    
                            /**
                             * Properties of a TargetImpressionShareLocationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ITargetImpressionShareLocationEnum
                             */
    
                            /**
                             * Constructs a new TargetImpressionShareLocationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a TargetImpressionShareLocationEnum.
                             * @implements ITargetImpressionShareLocationEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ITargetImpressionShareLocationEnum=} [p] Properties to set
                             */
                            function TargetImpressionShareLocationEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TargetImpressionShareLocationEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetImpressionShareLocationEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum} TargetImpressionShareLocationEnum instance
                             */
                            TargetImpressionShareLocationEnum.create = function create(properties) {
                                return new TargetImpressionShareLocationEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShareLocationEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetImpressionShareLocationEnum} m TargetImpressionShareLocationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShareLocationEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetImpressionShareLocationEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetImpressionShareLocationEnum} message TargetImpressionShareLocationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetImpressionShareLocationEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetImpressionShareLocationEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum} TargetImpressionShareLocationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShareLocationEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetImpressionShareLocationEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum} TargetImpressionShareLocationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetImpressionShareLocationEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetImpressionShareLocationEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetImpressionShareLocationEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TargetImpressionShareLocationEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum} TargetImpressionShareLocationEnum
                             */
                            TargetImpressionShareLocationEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum();
                            };
    
                            /**
                             * Creates a plain object from a TargetImpressionShareLocationEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum} m TargetImpressionShareLocationEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetImpressionShareLocationEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TargetImpressionShareLocationEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetImpressionShareLocationEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TargetImpressionShareLocation enum.
                             * @name google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.TargetImpressionShareLocation
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ANYWHERE_ON_PAGE=2 ANYWHERE_ON_PAGE value
                             * @property {number} TOP_OF_PAGE=3 TOP_OF_PAGE value
                             * @property {number} ABSOLUTE_TOP_OF_PAGE=4 ABSOLUTE_TOP_OF_PAGE value
                             */
                            TargetImpressionShareLocationEnum.TargetImpressionShareLocation = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ANYWHERE_ON_PAGE"] = 2;
                                values[valuesById[3] = "TOP_OF_PAGE"] = 3;
                                values[valuesById[4] = "ABSOLUTE_TOP_OF_PAGE"] = 4;
                                return values;
                            })();
    
                            return TargetImpressionShareLocationEnum;
                        })();
    
                        enums.AdvertisingChannelSubTypeEnum = (function() {
    
                            /**
                             * Properties of an AdvertisingChannelSubTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdvertisingChannelSubTypeEnum
                             */
    
                            /**
                             * Constructs a new AdvertisingChannelSubTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdvertisingChannelSubTypeEnum.
                             * @implements IAdvertisingChannelSubTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelSubTypeEnum=} [p] Properties to set
                             */
                            function AdvertisingChannelSubTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdvertisingChannelSubTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelSubTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum} AdvertisingChannelSubTypeEnum instance
                             */
                            AdvertisingChannelSubTypeEnum.create = function create(properties) {
                                return new AdvertisingChannelSubTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdvertisingChannelSubTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelSubTypeEnum} m AdvertisingChannelSubTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdvertisingChannelSubTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdvertisingChannelSubTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelSubTypeEnum} message AdvertisingChannelSubTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdvertisingChannelSubTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdvertisingChannelSubTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum} AdvertisingChannelSubTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdvertisingChannelSubTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdvertisingChannelSubTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum} AdvertisingChannelSubTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdvertisingChannelSubTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdvertisingChannelSubTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdvertisingChannelSubTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdvertisingChannelSubTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum} AdvertisingChannelSubTypeEnum
                             */
                            AdvertisingChannelSubTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdvertisingChannelSubTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum} m AdvertisingChannelSubTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdvertisingChannelSubTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdvertisingChannelSubTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdvertisingChannelSubTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdvertisingChannelSubType enum.
                             * @name google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SEARCH_MOBILE_APP=2 SEARCH_MOBILE_APP value
                             * @property {number} DISPLAY_MOBILE_APP=3 DISPLAY_MOBILE_APP value
                             * @property {number} SEARCH_EXPRESS=4 SEARCH_EXPRESS value
                             * @property {number} DISPLAY_EXPRESS=5 DISPLAY_EXPRESS value
                             * @property {number} SHOPPING_SMART_ADS=6 SHOPPING_SMART_ADS value
                             * @property {number} DISPLAY_GMAIL_AD=7 DISPLAY_GMAIL_AD value
                             * @property {number} DISPLAY_SMART_CAMPAIGN=8 DISPLAY_SMART_CAMPAIGN value
                             * @property {number} VIDEO_OUTSTREAM=9 VIDEO_OUTSTREAM value
                             * @property {number} VIDEO_ACTION=10 VIDEO_ACTION value
                             * @property {number} VIDEO_NON_SKIPPABLE=11 VIDEO_NON_SKIPPABLE value
                             * @property {number} APP_CAMPAIGN=12 APP_CAMPAIGN value
                             * @property {number} APP_CAMPAIGN_FOR_ENGAGEMENT=13 APP_CAMPAIGN_FOR_ENGAGEMENT value
                             * @property {number} LOCAL_CAMPAIGN=14 LOCAL_CAMPAIGN value
                             * @property {number} SHOPPING_COMPARISON_LISTING_ADS=15 SHOPPING_COMPARISON_LISTING_ADS value
                             * @property {number} SMART_CAMPAIGN=16 SMART_CAMPAIGN value
                             * @property {number} VIDEO_SEQUENCE=17 VIDEO_SEQUENCE value
                             * @property {number} APP_CAMPAIGN_FOR_PRE_REGISTRATION=18 APP_CAMPAIGN_FOR_PRE_REGISTRATION value
                             */
                            AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SEARCH_MOBILE_APP"] = 2;
                                values[valuesById[3] = "DISPLAY_MOBILE_APP"] = 3;
                                values[valuesById[4] = "SEARCH_EXPRESS"] = 4;
                                values[valuesById[5] = "DISPLAY_EXPRESS"] = 5;
                                values[valuesById[6] = "SHOPPING_SMART_ADS"] = 6;
                                values[valuesById[7] = "DISPLAY_GMAIL_AD"] = 7;
                                values[valuesById[8] = "DISPLAY_SMART_CAMPAIGN"] = 8;
                                values[valuesById[9] = "VIDEO_OUTSTREAM"] = 9;
                                values[valuesById[10] = "VIDEO_ACTION"] = 10;
                                values[valuesById[11] = "VIDEO_NON_SKIPPABLE"] = 11;
                                values[valuesById[12] = "APP_CAMPAIGN"] = 12;
                                values[valuesById[13] = "APP_CAMPAIGN_FOR_ENGAGEMENT"] = 13;
                                values[valuesById[14] = "LOCAL_CAMPAIGN"] = 14;
                                values[valuesById[15] = "SHOPPING_COMPARISON_LISTING_ADS"] = 15;
                                values[valuesById[16] = "SMART_CAMPAIGN"] = 16;
                                values[valuesById[17] = "VIDEO_SEQUENCE"] = 17;
                                values[valuesById[18] = "APP_CAMPAIGN_FOR_PRE_REGISTRATION"] = 18;
                                return values;
                            })();
    
                            return AdvertisingChannelSubTypeEnum;
                        })();
    
                        enums.AdvertisingChannelTypeEnum = (function() {
    
                            /**
                             * Properties of an AdvertisingChannelTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdvertisingChannelTypeEnum
                             */
    
                            /**
                             * Constructs a new AdvertisingChannelTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdvertisingChannelTypeEnum.
                             * @implements IAdvertisingChannelTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelTypeEnum=} [p] Properties to set
                             */
                            function AdvertisingChannelTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdvertisingChannelTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum} AdvertisingChannelTypeEnum instance
                             */
                            AdvertisingChannelTypeEnum.create = function create(properties) {
                                return new AdvertisingChannelTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdvertisingChannelTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelTypeEnum} m AdvertisingChannelTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdvertisingChannelTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdvertisingChannelTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdvertisingChannelTypeEnum} message AdvertisingChannelTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdvertisingChannelTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdvertisingChannelTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum} AdvertisingChannelTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdvertisingChannelTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdvertisingChannelTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum} AdvertisingChannelTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdvertisingChannelTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdvertisingChannelTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdvertisingChannelTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdvertisingChannelTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum} AdvertisingChannelTypeEnum
                             */
                            AdvertisingChannelTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdvertisingChannelTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum} m AdvertisingChannelTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdvertisingChannelTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdvertisingChannelTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdvertisingChannelTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdvertisingChannelType enum.
                             * @name google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SEARCH=2 SEARCH value
                             * @property {number} DISPLAY=3 DISPLAY value
                             * @property {number} SHOPPING=4 SHOPPING value
                             * @property {number} HOTEL=5 HOTEL value
                             * @property {number} VIDEO=6 VIDEO value
                             * @property {number} MULTI_CHANNEL=7 MULTI_CHANNEL value
                             * @property {number} LOCAL=8 LOCAL value
                             * @property {number} SMART=9 SMART value
                             * @property {number} PERFORMANCE_MAX=10 PERFORMANCE_MAX value
                             * @property {number} LOCAL_SERVICES=11 LOCAL_SERVICES value
                             * @property {number} DISCOVERY=12 DISCOVERY value
                             */
                            AdvertisingChannelTypeEnum.AdvertisingChannelType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SEARCH"] = 2;
                                values[valuesById[3] = "DISPLAY"] = 3;
                                values[valuesById[4] = "SHOPPING"] = 4;
                                values[valuesById[5] = "HOTEL"] = 5;
                                values[valuesById[6] = "VIDEO"] = 6;
                                values[valuesById[7] = "MULTI_CHANNEL"] = 7;
                                values[valuesById[8] = "LOCAL"] = 8;
                                values[valuesById[9] = "SMART"] = 9;
                                values[valuesById[10] = "PERFORMANCE_MAX"] = 10;
                                values[valuesById[11] = "LOCAL_SERVICES"] = 11;
                                values[valuesById[12] = "DISCOVERY"] = 12;
                                return values;
                            })();
    
                            return AdvertisingChannelTypeEnum;
                        })();
    
                        enums.CriterionCategoryChannelAvailabilityModeEnum = (function() {
    
                            /**
                             * Properties of a CriterionCategoryChannelAvailabilityModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICriterionCategoryChannelAvailabilityModeEnum
                             */
    
                            /**
                             * Constructs a new CriterionCategoryChannelAvailabilityModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CriterionCategoryChannelAvailabilityModeEnum.
                             * @implements ICriterionCategoryChannelAvailabilityModeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryChannelAvailabilityModeEnum=} [p] Properties to set
                             */
                            function CriterionCategoryChannelAvailabilityModeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CriterionCategoryChannelAvailabilityModeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryChannelAvailabilityModeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum} CriterionCategoryChannelAvailabilityModeEnum instance
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.create = function create(properties) {
                                return new CriterionCategoryChannelAvailabilityModeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryChannelAvailabilityModeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryChannelAvailabilityModeEnum} m CriterionCategoryChannelAvailabilityModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryChannelAvailabilityModeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryChannelAvailabilityModeEnum} message CriterionCategoryChannelAvailabilityModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionCategoryChannelAvailabilityModeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum} CriterionCategoryChannelAvailabilityModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionCategoryChannelAvailabilityModeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum} CriterionCategoryChannelAvailabilityModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionCategoryChannelAvailabilityModeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CriterionCategoryChannelAvailabilityModeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum} CriterionCategoryChannelAvailabilityModeEnum
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CriterionCategoryChannelAvailabilityModeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum} m CriterionCategoryChannelAvailabilityModeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CriterionCategoryChannelAvailabilityModeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CriterionCategoryChannelAvailabilityMode enum.
                             * @name google.ads.googleads.v11.enums.CriterionCategoryChannelAvailabilityModeEnum.CriterionCategoryChannelAvailabilityMode
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ALL_CHANNELS=2 ALL_CHANNELS value
                             * @property {number} CHANNEL_TYPE_AND_ALL_SUBTYPES=3 CHANNEL_TYPE_AND_ALL_SUBTYPES value
                             * @property {number} CHANNEL_TYPE_AND_SUBSET_SUBTYPES=4 CHANNEL_TYPE_AND_SUBSET_SUBTYPES value
                             */
                            CriterionCategoryChannelAvailabilityModeEnum.CriterionCategoryChannelAvailabilityMode = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ALL_CHANNELS"] = 2;
                                values[valuesById[3] = "CHANNEL_TYPE_AND_ALL_SUBTYPES"] = 3;
                                values[valuesById[4] = "CHANNEL_TYPE_AND_SUBSET_SUBTYPES"] = 4;
                                return values;
                            })();
    
                            return CriterionCategoryChannelAvailabilityModeEnum;
                        })();
    
                        enums.CriterionCategoryLocaleAvailabilityModeEnum = (function() {
    
                            /**
                             * Properties of a CriterionCategoryLocaleAvailabilityModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICriterionCategoryLocaleAvailabilityModeEnum
                             */
    
                            /**
                             * Constructs a new CriterionCategoryLocaleAvailabilityModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CriterionCategoryLocaleAvailabilityModeEnum.
                             * @implements ICriterionCategoryLocaleAvailabilityModeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryLocaleAvailabilityModeEnum=} [p] Properties to set
                             */
                            function CriterionCategoryLocaleAvailabilityModeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CriterionCategoryLocaleAvailabilityModeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryLocaleAvailabilityModeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum} CriterionCategoryLocaleAvailabilityModeEnum instance
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.create = function create(properties) {
                                return new CriterionCategoryLocaleAvailabilityModeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryLocaleAvailabilityModeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryLocaleAvailabilityModeEnum} m CriterionCategoryLocaleAvailabilityModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionCategoryLocaleAvailabilityModeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionCategoryLocaleAvailabilityModeEnum} message CriterionCategoryLocaleAvailabilityModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionCategoryLocaleAvailabilityModeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum} CriterionCategoryLocaleAvailabilityModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionCategoryLocaleAvailabilityModeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum} CriterionCategoryLocaleAvailabilityModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionCategoryLocaleAvailabilityModeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CriterionCategoryLocaleAvailabilityModeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum} CriterionCategoryLocaleAvailabilityModeEnum
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CriterionCategoryLocaleAvailabilityModeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum} m CriterionCategoryLocaleAvailabilityModeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CriterionCategoryLocaleAvailabilityModeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CriterionCategoryLocaleAvailabilityMode enum.
                             * @name google.ads.googleads.v11.enums.CriterionCategoryLocaleAvailabilityModeEnum.CriterionCategoryLocaleAvailabilityMode
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ALL_LOCALES=2 ALL_LOCALES value
                             * @property {number} COUNTRY_AND_ALL_LANGUAGES=3 COUNTRY_AND_ALL_LANGUAGES value
                             * @property {number} LANGUAGE_AND_ALL_COUNTRIES=4 LANGUAGE_AND_ALL_COUNTRIES value
                             * @property {number} COUNTRY_AND_LANGUAGE=5 COUNTRY_AND_LANGUAGE value
                             */
                            CriterionCategoryLocaleAvailabilityModeEnum.CriterionCategoryLocaleAvailabilityMode = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ALL_LOCALES"] = 2;
                                values[valuesById[3] = "COUNTRY_AND_ALL_LANGUAGES"] = 3;
                                values[valuesById[4] = "LANGUAGE_AND_ALL_COUNTRIES"] = 4;
                                values[valuesById[5] = "COUNTRY_AND_LANGUAGE"] = 5;
                                return values;
                            })();
    
                            return CriterionCategoryLocaleAvailabilityModeEnum;
                        })();
    
                        enums.CustomizerAttributeTypeEnum = (function() {
    
                            /**
                             * Properties of a CustomizerAttributeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomizerAttributeTypeEnum
                             */
    
                            /**
                             * Constructs a new CustomizerAttributeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomizerAttributeTypeEnum.
                             * @implements ICustomizerAttributeTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeTypeEnum=} [p] Properties to set
                             */
                            function CustomizerAttributeTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomizerAttributeTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum} CustomizerAttributeTypeEnum instance
                             */
                            CustomizerAttributeTypeEnum.create = function create(properties) {
                                return new CustomizerAttributeTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeTypeEnum} m CustomizerAttributeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeTypeEnum} message CustomizerAttributeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomizerAttributeTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum} CustomizerAttributeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomizerAttributeTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum} CustomizerAttributeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomizerAttributeTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomizerAttributeTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomizerAttributeTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum} CustomizerAttributeTypeEnum
                             */
                            CustomizerAttributeTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomizerAttributeTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum} m CustomizerAttributeTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomizerAttributeTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomizerAttributeTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomizerAttributeTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomizerAttributeType enum.
                             * @name google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.CustomizerAttributeType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TEXT=2 TEXT value
                             * @property {number} NUMBER=3 NUMBER value
                             * @property {number} PRICE=4 PRICE value
                             * @property {number} PERCENT=5 PERCENT value
                             */
                            CustomizerAttributeTypeEnum.CustomizerAttributeType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TEXT"] = 2;
                                values[valuesById[3] = "NUMBER"] = 3;
                                values[valuesById[4] = "PRICE"] = 4;
                                values[valuesById[5] = "PERCENT"] = 5;
                                return values;
                            })();
    
                            return CustomizerAttributeTypeEnum;
                        })();
    
                        enums.MonthOfYearEnum = (function() {
    
                            /**
                             * Properties of a MonthOfYearEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMonthOfYearEnum
                             */
    
                            /**
                             * Constructs a new MonthOfYearEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MonthOfYearEnum.
                             * @implements IMonthOfYearEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMonthOfYearEnum=} [p] Properties to set
                             */
                            function MonthOfYearEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MonthOfYearEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMonthOfYearEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MonthOfYearEnum} MonthOfYearEnum instance
                             */
                            MonthOfYearEnum.create = function create(properties) {
                                return new MonthOfYearEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MonthOfYearEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MonthOfYearEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMonthOfYearEnum} m MonthOfYearEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MonthOfYearEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MonthOfYearEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MonthOfYearEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMonthOfYearEnum} message MonthOfYearEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MonthOfYearEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MonthOfYearEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MonthOfYearEnum} MonthOfYearEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MonthOfYearEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MonthOfYearEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MonthOfYearEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MonthOfYearEnum} MonthOfYearEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MonthOfYearEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MonthOfYearEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MonthOfYearEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MonthOfYearEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MonthOfYearEnum} MonthOfYearEnum
                             */
                            MonthOfYearEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MonthOfYearEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MonthOfYearEnum();
                            };
    
                            /**
                             * Creates a plain object from a MonthOfYearEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MonthOfYearEnum} m MonthOfYearEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MonthOfYearEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MonthOfYearEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MonthOfYearEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MonthOfYearEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MonthOfYear enum.
                             * @name google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} JANUARY=2 JANUARY value
                             * @property {number} FEBRUARY=3 FEBRUARY value
                             * @property {number} MARCH=4 MARCH value
                             * @property {number} APRIL=5 APRIL value
                             * @property {number} MAY=6 MAY value
                             * @property {number} JUNE=7 JUNE value
                             * @property {number} JULY=8 JULY value
                             * @property {number} AUGUST=9 AUGUST value
                             * @property {number} SEPTEMBER=10 SEPTEMBER value
                             * @property {number} OCTOBER=11 OCTOBER value
                             * @property {number} NOVEMBER=12 NOVEMBER value
                             * @property {number} DECEMBER=13 DECEMBER value
                             */
                            MonthOfYearEnum.MonthOfYear = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "JANUARY"] = 2;
                                values[valuesById[3] = "FEBRUARY"] = 3;
                                values[valuesById[4] = "MARCH"] = 4;
                                values[valuesById[5] = "APRIL"] = 5;
                                values[valuesById[6] = "MAY"] = 6;
                                values[valuesById[7] = "JUNE"] = 7;
                                values[valuesById[8] = "JULY"] = 8;
                                values[valuesById[9] = "AUGUST"] = 9;
                                values[valuesById[10] = "SEPTEMBER"] = 10;
                                values[valuesById[11] = "OCTOBER"] = 11;
                                values[valuesById[12] = "NOVEMBER"] = 12;
                                values[valuesById[13] = "DECEMBER"] = 13;
                                return values;
                            })();
    
                            return MonthOfYearEnum;
                        })();
    
                        enums.AppStoreEnum = (function() {
    
                            /**
                             * Properties of an AppStoreEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAppStoreEnum
                             */
    
                            /**
                             * Constructs a new AppStoreEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AppStoreEnum.
                             * @implements IAppStoreEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAppStoreEnum=} [p] Properties to set
                             */
                            function AppStoreEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AppStoreEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppStoreEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AppStoreEnum} AppStoreEnum instance
                             */
                            AppStoreEnum.create = function create(properties) {
                                return new AppStoreEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AppStoreEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AppStoreEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppStoreEnum} m AppStoreEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppStoreEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppStoreEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AppStoreEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppStoreEnum} message AppStoreEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppStoreEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppStoreEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AppStoreEnum} AppStoreEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppStoreEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AppStoreEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppStoreEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AppStoreEnum} AppStoreEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppStoreEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppStoreEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppStoreEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AppStoreEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AppStoreEnum} AppStoreEnum
                             */
                            AppStoreEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AppStoreEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AppStoreEnum();
                            };
    
                            /**
                             * Creates a plain object from an AppStoreEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AppStoreEnum} m AppStoreEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppStoreEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AppStoreEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AppStoreEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppStoreEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AppStore enum.
                             * @name google.ads.googleads.v11.enums.AppStoreEnum.AppStore
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APPLE_ITUNES=2 APPLE_ITUNES value
                             * @property {number} GOOGLE_PLAY=3 GOOGLE_PLAY value
                             */
                            AppStoreEnum.AppStore = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APPLE_ITUNES"] = 2;
                                values[valuesById[3] = "GOOGLE_PLAY"] = 3;
                                return values;
                            })();
    
                            return AppStoreEnum;
                        })();
    
                        enums.FeedItemSetStringFilterTypeEnum = (function() {
    
                            /**
                             * Properties of a FeedItemSetStringFilterTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemSetStringFilterTypeEnum
                             */
    
                            /**
                             * Constructs a new FeedItemSetStringFilterTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemSetStringFilterTypeEnum.
                             * @implements IFeedItemSetStringFilterTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStringFilterTypeEnum=} [p] Properties to set
                             */
                            function FeedItemSetStringFilterTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemSetStringFilterTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStringFilterTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum} FeedItemSetStringFilterTypeEnum instance
                             */
                            FeedItemSetStringFilterTypeEnum.create = function create(properties) {
                                return new FeedItemSetStringFilterTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSetStringFilterTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStringFilterTypeEnum} m FeedItemSetStringFilterTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetStringFilterTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSetStringFilterTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStringFilterTypeEnum} message FeedItemSetStringFilterTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetStringFilterTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSetStringFilterTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum} FeedItemSetStringFilterTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetStringFilterTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSetStringFilterTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum} FeedItemSetStringFilterTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetStringFilterTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSetStringFilterTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSetStringFilterTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSetStringFilterTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum} FeedItemSetStringFilterTypeEnum
                             */
                            FeedItemSetStringFilterTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSetStringFilterTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum} m FeedItemSetStringFilterTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSetStringFilterTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemSetStringFilterTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSetStringFilterTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemSetStringFilterType enum.
                             * @name google.ads.googleads.v11.enums.FeedItemSetStringFilterTypeEnum.FeedItemSetStringFilterType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EXACT=2 EXACT value
                             */
                            FeedItemSetStringFilterTypeEnum.FeedItemSetStringFilterType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EXACT"] = 2;
                                return values;
                            })();
    
                            return FeedItemSetStringFilterTypeEnum;
                        })();
    
                        enums.AppUrlOperatingSystemTypeEnum = (function() {
    
                            /**
                             * Properties of an AppUrlOperatingSystemTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAppUrlOperatingSystemTypeEnum
                             */
    
                            /**
                             * Constructs a new AppUrlOperatingSystemTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AppUrlOperatingSystemTypeEnum.
                             * @implements IAppUrlOperatingSystemTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAppUrlOperatingSystemTypeEnum=} [p] Properties to set
                             */
                            function AppUrlOperatingSystemTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AppUrlOperatingSystemTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppUrlOperatingSystemTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum} AppUrlOperatingSystemTypeEnum instance
                             */
                            AppUrlOperatingSystemTypeEnum.create = function create(properties) {
                                return new AppUrlOperatingSystemTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AppUrlOperatingSystemTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppUrlOperatingSystemTypeEnum} m AppUrlOperatingSystemTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppUrlOperatingSystemTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppUrlOperatingSystemTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppUrlOperatingSystemTypeEnum} message AppUrlOperatingSystemTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppUrlOperatingSystemTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppUrlOperatingSystemTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum} AppUrlOperatingSystemTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppUrlOperatingSystemTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppUrlOperatingSystemTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum} AppUrlOperatingSystemTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppUrlOperatingSystemTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppUrlOperatingSystemTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppUrlOperatingSystemTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AppUrlOperatingSystemTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum} AppUrlOperatingSystemTypeEnum
                             */
                            AppUrlOperatingSystemTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AppUrlOperatingSystemTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum} m AppUrlOperatingSystemTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppUrlOperatingSystemTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AppUrlOperatingSystemTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppUrlOperatingSystemTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AppUrlOperatingSystemType enum.
                             * @name google.ads.googleads.v11.enums.AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} IOS=2 IOS value
                             * @property {number} ANDROID=3 ANDROID value
                             */
                            AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "IOS"] = 2;
                                values[valuesById[3] = "ANDROID"] = 3;
                                return values;
                            })();
    
                            return AppUrlOperatingSystemTypeEnum;
                        })();
    
                        enums.FrequencyCapEventTypeEnum = (function() {
    
                            /**
                             * Properties of a FrequencyCapEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFrequencyCapEventTypeEnum
                             */
    
                            /**
                             * Constructs a new FrequencyCapEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FrequencyCapEventTypeEnum.
                             * @implements IFrequencyCapEventTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapEventTypeEnum=} [p] Properties to set
                             */
                            function FrequencyCapEventTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FrequencyCapEventTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapEventTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum} FrequencyCapEventTypeEnum instance
                             */
                            FrequencyCapEventTypeEnum.create = function create(properties) {
                                return new FrequencyCapEventTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FrequencyCapEventTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapEventTypeEnum} m FrequencyCapEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapEventTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FrequencyCapEventTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapEventTypeEnum} message FrequencyCapEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapEventTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FrequencyCapEventTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum} FrequencyCapEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapEventTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FrequencyCapEventTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum} FrequencyCapEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapEventTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FrequencyCapEventTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FrequencyCapEventTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FrequencyCapEventTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum} FrequencyCapEventTypeEnum
                             */
                            FrequencyCapEventTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a FrequencyCapEventTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum} m FrequencyCapEventTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FrequencyCapEventTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FrequencyCapEventTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FrequencyCapEventTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FrequencyCapEventType enum.
                             * @name google.ads.googleads.v11.enums.FrequencyCapEventTypeEnum.FrequencyCapEventType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} IMPRESSION=2 IMPRESSION value
                             * @property {number} VIDEO_VIEW=3 VIDEO_VIEW value
                             */
                            FrequencyCapEventTypeEnum.FrequencyCapEventType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "IMPRESSION"] = 2;
                                values[valuesById[3] = "VIDEO_VIEW"] = 3;
                                return values;
                            })();
    
                            return FrequencyCapEventTypeEnum;
                        })();
    
                        enums.FrequencyCapLevelEnum = (function() {
    
                            /**
                             * Properties of a FrequencyCapLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFrequencyCapLevelEnum
                             */
    
                            /**
                             * Constructs a new FrequencyCapLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FrequencyCapLevelEnum.
                             * @implements IFrequencyCapLevelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapLevelEnum=} [p] Properties to set
                             */
                            function FrequencyCapLevelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FrequencyCapLevelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapLevelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapLevelEnum} FrequencyCapLevelEnum instance
                             */
                            FrequencyCapLevelEnum.create = function create(properties) {
                                return new FrequencyCapLevelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FrequencyCapLevelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FrequencyCapLevelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapLevelEnum} m FrequencyCapLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapLevelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FrequencyCapLevelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FrequencyCapLevelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapLevelEnum} message FrequencyCapLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapLevelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FrequencyCapLevelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapLevelEnum} FrequencyCapLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapLevelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FrequencyCapLevelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FrequencyCapLevelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapLevelEnum} FrequencyCapLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapLevelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FrequencyCapLevelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FrequencyCapLevelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FrequencyCapLevelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapLevelEnum} FrequencyCapLevelEnum
                             */
                            FrequencyCapLevelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FrequencyCapLevelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FrequencyCapLevelEnum();
                            };
    
                            /**
                             * Creates a plain object from a FrequencyCapLevelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FrequencyCapLevelEnum} m FrequencyCapLevelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FrequencyCapLevelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FrequencyCapLevelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapLevelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FrequencyCapLevelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FrequencyCapLevel enum.
                             * @name google.ads.googleads.v11.enums.FrequencyCapLevelEnum.FrequencyCapLevel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_GROUP_AD=2 AD_GROUP_AD value
                             * @property {number} AD_GROUP=3 AD_GROUP value
                             * @property {number} CAMPAIGN=4 CAMPAIGN value
                             */
                            FrequencyCapLevelEnum.FrequencyCapLevel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_GROUP_AD"] = 2;
                                values[valuesById[3] = "AD_GROUP"] = 3;
                                values[valuesById[4] = "CAMPAIGN"] = 4;
                                return values;
                            })();
    
                            return FrequencyCapLevelEnum;
                        })();
    
                        enums.FrequencyCapTimeUnitEnum = (function() {
    
                            /**
                             * Properties of a FrequencyCapTimeUnitEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFrequencyCapTimeUnitEnum
                             */
    
                            /**
                             * Constructs a new FrequencyCapTimeUnitEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FrequencyCapTimeUnitEnum.
                             * @implements IFrequencyCapTimeUnitEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapTimeUnitEnum=} [p] Properties to set
                             */
                            function FrequencyCapTimeUnitEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FrequencyCapTimeUnitEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapTimeUnitEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum} FrequencyCapTimeUnitEnum instance
                             */
                            FrequencyCapTimeUnitEnum.create = function create(properties) {
                                return new FrequencyCapTimeUnitEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FrequencyCapTimeUnitEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapTimeUnitEnum} m FrequencyCapTimeUnitEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapTimeUnitEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FrequencyCapTimeUnitEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFrequencyCapTimeUnitEnum} message FrequencyCapTimeUnitEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCapTimeUnitEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FrequencyCapTimeUnitEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum} FrequencyCapTimeUnitEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapTimeUnitEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FrequencyCapTimeUnitEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum} FrequencyCapTimeUnitEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCapTimeUnitEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FrequencyCapTimeUnitEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FrequencyCapTimeUnitEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FrequencyCapTimeUnitEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum} FrequencyCapTimeUnitEnum
                             */
                            FrequencyCapTimeUnitEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum();
                            };
    
                            /**
                             * Creates a plain object from a FrequencyCapTimeUnitEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum} m FrequencyCapTimeUnitEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FrequencyCapTimeUnitEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FrequencyCapTimeUnitEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FrequencyCapTimeUnitEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FrequencyCapTimeUnit enum.
                             * @name google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DAY=2 DAY value
                             * @property {number} WEEK=3 WEEK value
                             * @property {number} MONTH=4 MONTH value
                             */
                            FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DAY"] = 2;
                                values[valuesById[3] = "WEEK"] = 3;
                                values[valuesById[4] = "MONTH"] = 4;
                                return values;
                            })();
    
                            return FrequencyCapTimeUnitEnum;
                        })();
    
                        enums.KeywordPlanAggregateMetricTypeEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanAggregateMetricTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IKeywordPlanAggregateMetricTypeEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanAggregateMetricTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a KeywordPlanAggregateMetricTypeEnum.
                             * @implements IKeywordPlanAggregateMetricTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanAggregateMetricTypeEnum=} [p] Properties to set
                             */
                            function KeywordPlanAggregateMetricTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanAggregateMetricTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanAggregateMetricTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum} KeywordPlanAggregateMetricTypeEnum instance
                             */
                            KeywordPlanAggregateMetricTypeEnum.create = function create(properties) {
                                return new KeywordPlanAggregateMetricTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAggregateMetricTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanAggregateMetricTypeEnum} m KeywordPlanAggregateMetricTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAggregateMetricTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAggregateMetricTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanAggregateMetricTypeEnum} message KeywordPlanAggregateMetricTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAggregateMetricTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAggregateMetricTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum} KeywordPlanAggregateMetricTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAggregateMetricTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAggregateMetricTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum} KeywordPlanAggregateMetricTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAggregateMetricTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAggregateMetricTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAggregateMetricTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAggregateMetricTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum} KeywordPlanAggregateMetricTypeEnum
                             */
                            KeywordPlanAggregateMetricTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAggregateMetricTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum} m KeywordPlanAggregateMetricTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAggregateMetricTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanAggregateMetricTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAggregateMetricTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanAggregateMetricType enum.
                             * @name google.ads.googleads.v11.enums.KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DEVICE=2 DEVICE value
                             */
                            KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DEVICE"] = 2;
                                return values;
                            })();
    
                            return KeywordPlanAggregateMetricTypeEnum;
                        })();
    
                        enums.KeywordPlanCompetitionLevelEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanCompetitionLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IKeywordPlanCompetitionLevelEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanCompetitionLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a KeywordPlanCompetitionLevelEnum.
                             * @implements IKeywordPlanCompetitionLevelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanCompetitionLevelEnum=} [p] Properties to set
                             */
                            function KeywordPlanCompetitionLevelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanCompetitionLevelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanCompetitionLevelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum} KeywordPlanCompetitionLevelEnum instance
                             */
                            KeywordPlanCompetitionLevelEnum.create = function create(properties) {
                                return new KeywordPlanCompetitionLevelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCompetitionLevelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanCompetitionLevelEnum} m KeywordPlanCompetitionLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCompetitionLevelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCompetitionLevelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanCompetitionLevelEnum} message KeywordPlanCompetitionLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCompetitionLevelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCompetitionLevelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum} KeywordPlanCompetitionLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCompetitionLevelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCompetitionLevelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum} KeywordPlanCompetitionLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCompetitionLevelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCompetitionLevelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCompetitionLevelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCompetitionLevelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum} KeywordPlanCompetitionLevelEnum
                             */
                            KeywordPlanCompetitionLevelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCompetitionLevelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum} m KeywordPlanCompetitionLevelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCompetitionLevelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanCompetitionLevelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCompetitionLevelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanCompetitionLevel enum.
                             * @name google.ads.googleads.v11.enums.KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LOW=2 LOW value
                             * @property {number} MEDIUM=3 MEDIUM value
                             * @property {number} HIGH=4 HIGH value
                             */
                            KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LOW"] = 2;
                                values[valuesById[3] = "MEDIUM"] = 3;
                                values[valuesById[4] = "HIGH"] = 4;
                                return values;
                            })();
    
                            return KeywordPlanCompetitionLevelEnum;
                        })();
    
                        enums.KeywordPlanConceptGroupTypeEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanConceptGroupTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IKeywordPlanConceptGroupTypeEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanConceptGroupTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a KeywordPlanConceptGroupTypeEnum.
                             * @implements IKeywordPlanConceptGroupTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanConceptGroupTypeEnum=} [p] Properties to set
                             */
                            function KeywordPlanConceptGroupTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanConceptGroupTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanConceptGroupTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum} KeywordPlanConceptGroupTypeEnum instance
                             */
                            KeywordPlanConceptGroupTypeEnum.create = function create(properties) {
                                return new KeywordPlanConceptGroupTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanConceptGroupTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanConceptGroupTypeEnum} m KeywordPlanConceptGroupTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanConceptGroupTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanConceptGroupTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanConceptGroupTypeEnum} message KeywordPlanConceptGroupTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanConceptGroupTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanConceptGroupTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum} KeywordPlanConceptGroupTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanConceptGroupTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanConceptGroupTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum} KeywordPlanConceptGroupTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanConceptGroupTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanConceptGroupTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanConceptGroupTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanConceptGroupTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum} KeywordPlanConceptGroupTypeEnum
                             */
                            KeywordPlanConceptGroupTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanConceptGroupTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum} m KeywordPlanConceptGroupTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanConceptGroupTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanConceptGroupTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanConceptGroupTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanConceptGroupType enum.
                             * @name google.ads.googleads.v11.enums.KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BRAND=2 BRAND value
                             * @property {number} OTHER_BRANDS=3 OTHER_BRANDS value
                             * @property {number} NON_BRAND=4 NON_BRAND value
                             */
                            KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BRAND"] = 2;
                                values[valuesById[3] = "OTHER_BRANDS"] = 3;
                                values[valuesById[4] = "NON_BRAND"] = 4;
                                return values;
                            })();
    
                            return KeywordPlanConceptGroupTypeEnum;
                        })();
    
                        enums.MatchingFunctionContextTypeEnum = (function() {
    
                            /**
                             * Properties of a MatchingFunctionContextTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMatchingFunctionContextTypeEnum
                             */
    
                            /**
                             * Constructs a new MatchingFunctionContextTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MatchingFunctionContextTypeEnum.
                             * @implements IMatchingFunctionContextTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionContextTypeEnum=} [p] Properties to set
                             */
                            function MatchingFunctionContextTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MatchingFunctionContextTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionContextTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum} MatchingFunctionContextTypeEnum instance
                             */
                            MatchingFunctionContextTypeEnum.create = function create(properties) {
                                return new MatchingFunctionContextTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MatchingFunctionContextTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionContextTypeEnum} m MatchingFunctionContextTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MatchingFunctionContextTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MatchingFunctionContextTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionContextTypeEnum} message MatchingFunctionContextTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MatchingFunctionContextTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MatchingFunctionContextTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum} MatchingFunctionContextTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MatchingFunctionContextTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MatchingFunctionContextTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum} MatchingFunctionContextTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MatchingFunctionContextTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MatchingFunctionContextTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MatchingFunctionContextTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MatchingFunctionContextTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum} MatchingFunctionContextTypeEnum
                             */
                            MatchingFunctionContextTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a MatchingFunctionContextTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum} m MatchingFunctionContextTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MatchingFunctionContextTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MatchingFunctionContextTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MatchingFunctionContextTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MatchingFunctionContextType enum.
                             * @name google.ads.googleads.v11.enums.MatchingFunctionContextTypeEnum.MatchingFunctionContextType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FEED_ITEM_ID=2 FEED_ITEM_ID value
                             * @property {number} DEVICE_NAME=3 DEVICE_NAME value
                             * @property {number} FEED_ITEM_SET_ID=4 FEED_ITEM_SET_ID value
                             */
                            MatchingFunctionContextTypeEnum.MatchingFunctionContextType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FEED_ITEM_ID"] = 2;
                                values[valuesById[3] = "DEVICE_NAME"] = 3;
                                values[valuesById[4] = "FEED_ITEM_SET_ID"] = 4;
                                return values;
                            })();
    
                            return MatchingFunctionContextTypeEnum;
                        })();
    
                        enums.MatchingFunctionOperatorEnum = (function() {
    
                            /**
                             * Properties of a MatchingFunctionOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMatchingFunctionOperatorEnum
                             */
    
                            /**
                             * Constructs a new MatchingFunctionOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MatchingFunctionOperatorEnum.
                             * @implements IMatchingFunctionOperatorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionOperatorEnum=} [p] Properties to set
                             */
                            function MatchingFunctionOperatorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MatchingFunctionOperatorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionOperatorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum} MatchingFunctionOperatorEnum instance
                             */
                            MatchingFunctionOperatorEnum.create = function create(properties) {
                                return new MatchingFunctionOperatorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MatchingFunctionOperatorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionOperatorEnum} m MatchingFunctionOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MatchingFunctionOperatorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MatchingFunctionOperatorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMatchingFunctionOperatorEnum} message MatchingFunctionOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MatchingFunctionOperatorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MatchingFunctionOperatorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum} MatchingFunctionOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MatchingFunctionOperatorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MatchingFunctionOperatorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum} MatchingFunctionOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MatchingFunctionOperatorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MatchingFunctionOperatorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MatchingFunctionOperatorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MatchingFunctionOperatorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum} MatchingFunctionOperatorEnum
                             */
                            MatchingFunctionOperatorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MatchingFunctionOperatorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum} m MatchingFunctionOperatorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MatchingFunctionOperatorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MatchingFunctionOperatorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MatchingFunctionOperatorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MatchingFunctionOperator enum.
                             * @name google.ads.googleads.v11.enums.MatchingFunctionOperatorEnum.MatchingFunctionOperator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} IN=2 IN value
                             * @property {number} IDENTITY=3 IDENTITY value
                             * @property {number} EQUALS=4 EQUALS value
                             * @property {number} AND=5 AND value
                             * @property {number} CONTAINS_ANY=6 CONTAINS_ANY value
                             */
                            MatchingFunctionOperatorEnum.MatchingFunctionOperator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "IN"] = 2;
                                values[valuesById[3] = "IDENTITY"] = 3;
                                values[valuesById[4] = "EQUALS"] = 4;
                                values[valuesById[5] = "AND"] = 5;
                                values[valuesById[6] = "CONTAINS_ANY"] = 6;
                                return values;
                            })();
    
                            return MatchingFunctionOperatorEnum;
                        })();
    
                        enums.ExperimentMetricEnum = (function() {
    
                            /**
                             * Properties of an ExperimentMetricEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IExperimentMetricEnum
                             */
    
                            /**
                             * Constructs a new ExperimentMetricEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ExperimentMetricEnum.
                             * @implements IExperimentMetricEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricEnum=} [p] Properties to set
                             */
                            function ExperimentMetricEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExperimentMetricEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricEnum} ExperimentMetricEnum instance
                             */
                            ExperimentMetricEnum.create = function create(properties) {
                                return new ExperimentMetricEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentMetricEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentMetricEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricEnum} m ExperimentMetricEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentMetricEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentMetricEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentMetricEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricEnum} message ExperimentMetricEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentMetricEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentMetricEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricEnum} ExperimentMetricEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentMetricEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ExperimentMetricEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentMetricEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricEnum} ExperimentMetricEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentMetricEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentMetricEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentMetricEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentMetricEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricEnum} ExperimentMetricEnum
                             */
                            ExperimentMetricEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ExperimentMetricEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ExperimentMetricEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExperimentMetricEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ExperimentMetricEnum} m ExperimentMetricEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentMetricEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExperimentMetricEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentMetricEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExperimentMetric enum.
                             * @name google.ads.googleads.v11.enums.ExperimentMetricEnum.ExperimentMetric
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CLICKS=2 CLICKS value
                             * @property {number} IMPRESSIONS=3 IMPRESSIONS value
                             * @property {number} COST=4 COST value
                             * @property {number} CONVERSIONS_PER_INTERACTION_RATE=5 CONVERSIONS_PER_INTERACTION_RATE value
                             * @property {number} COST_PER_CONVERSION=6 COST_PER_CONVERSION value
                             * @property {number} CONVERSIONS_VALUE_PER_COST=7 CONVERSIONS_VALUE_PER_COST value
                             * @property {number} AVERAGE_CPC=8 AVERAGE_CPC value
                             * @property {number} CTR=9 CTR value
                             * @property {number} INCREMENTAL_CONVERSIONS=10 INCREMENTAL_CONVERSIONS value
                             * @property {number} COMPLETED_VIDEO_VIEWS=11 COMPLETED_VIDEO_VIEWS value
                             * @property {number} CUSTOM_ALGORITHMS=12 CUSTOM_ALGORITHMS value
                             * @property {number} CONVERSIONS=13 CONVERSIONS value
                             * @property {number} CONVERSION_VALUE=14 CONVERSION_VALUE value
                             */
                            ExperimentMetricEnum.ExperimentMetric = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CLICKS"] = 2;
                                values[valuesById[3] = "IMPRESSIONS"] = 3;
                                values[valuesById[4] = "COST"] = 4;
                                values[valuesById[5] = "CONVERSIONS_PER_INTERACTION_RATE"] = 5;
                                values[valuesById[6] = "COST_PER_CONVERSION"] = 6;
                                values[valuesById[7] = "CONVERSIONS_VALUE_PER_COST"] = 7;
                                values[valuesById[8] = "AVERAGE_CPC"] = 8;
                                values[valuesById[9] = "CTR"] = 9;
                                values[valuesById[10] = "INCREMENTAL_CONVERSIONS"] = 10;
                                values[valuesById[11] = "COMPLETED_VIDEO_VIEWS"] = 11;
                                values[valuesById[12] = "CUSTOM_ALGORITHMS"] = 12;
                                values[valuesById[13] = "CONVERSIONS"] = 13;
                                values[valuesById[14] = "CONVERSION_VALUE"] = 14;
                                return values;
                            })();
    
                            return ExperimentMetricEnum;
                        })();
    
                        enums.ExperimentMetricDirectionEnum = (function() {
    
                            /**
                             * Properties of an ExperimentMetricDirectionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IExperimentMetricDirectionEnum
                             */
    
                            /**
                             * Constructs a new ExperimentMetricDirectionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ExperimentMetricDirectionEnum.
                             * @implements IExperimentMetricDirectionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricDirectionEnum=} [p] Properties to set
                             */
                            function ExperimentMetricDirectionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExperimentMetricDirectionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricDirectionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum} ExperimentMetricDirectionEnum instance
                             */
                            ExperimentMetricDirectionEnum.create = function create(properties) {
                                return new ExperimentMetricDirectionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentMetricDirectionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricDirectionEnum} m ExperimentMetricDirectionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentMetricDirectionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentMetricDirectionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentMetricDirectionEnum} message ExperimentMetricDirectionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentMetricDirectionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentMetricDirectionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum} ExperimentMetricDirectionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentMetricDirectionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentMetricDirectionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum} ExperimentMetricDirectionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentMetricDirectionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentMetricDirectionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentMetricDirectionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentMetricDirectionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum} ExperimentMetricDirectionEnum
                             */
                            ExperimentMetricDirectionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExperimentMetricDirectionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum} m ExperimentMetricDirectionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentMetricDirectionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExperimentMetricDirectionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentMetricDirectionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExperimentMetricDirection enum.
                             * @name google.ads.googleads.v11.enums.ExperimentMetricDirectionEnum.ExperimentMetricDirection
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NO_CHANGE=2 NO_CHANGE value
                             * @property {number} INCREASE=3 INCREASE value
                             * @property {number} DECREASE=4 DECREASE value
                             * @property {number} NO_CHANGE_OR_INCREASE=5 NO_CHANGE_OR_INCREASE value
                             * @property {number} NO_CHANGE_OR_DECREASE=6 NO_CHANGE_OR_DECREASE value
                             */
                            ExperimentMetricDirectionEnum.ExperimentMetricDirection = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NO_CHANGE"] = 2;
                                values[valuesById[3] = "INCREASE"] = 3;
                                values[valuesById[4] = "DECREASE"] = 4;
                                values[valuesById[5] = "NO_CHANGE_OR_INCREASE"] = 5;
                                values[valuesById[6] = "NO_CHANGE_OR_DECREASE"] = 6;
                                return values;
                            })();
    
                            return ExperimentMetricDirectionEnum;
                        })();
    
                        enums.InteractionEventTypeEnum = (function() {
    
                            /**
                             * Properties of an InteractionEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IInteractionEventTypeEnum
                             */
    
                            /**
                             * Constructs a new InteractionEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an InteractionEventTypeEnum.
                             * @implements IInteractionEventTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IInteractionEventTypeEnum=} [p] Properties to set
                             */
                            function InteractionEventTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new InteractionEventTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInteractionEventTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.InteractionEventTypeEnum} InteractionEventTypeEnum instance
                             */
                            InteractionEventTypeEnum.create = function create(properties) {
                                return new InteractionEventTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified InteractionEventTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.InteractionEventTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInteractionEventTypeEnum} m InteractionEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InteractionEventTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified InteractionEventTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.InteractionEventTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInteractionEventTypeEnum} message InteractionEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InteractionEventTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an InteractionEventTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.InteractionEventTypeEnum} InteractionEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InteractionEventTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.InteractionEventTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an InteractionEventTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.InteractionEventTypeEnum} InteractionEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InteractionEventTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an InteractionEventTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            InteractionEventTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an InteractionEventTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.InteractionEventTypeEnum} InteractionEventTypeEnum
                             */
                            InteractionEventTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.InteractionEventTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.InteractionEventTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an InteractionEventTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.InteractionEventTypeEnum} m InteractionEventTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            InteractionEventTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this InteractionEventTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.InteractionEventTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            InteractionEventTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * InteractionEventType enum.
                             * @name google.ads.googleads.v11.enums.InteractionEventTypeEnum.InteractionEventType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CLICK=2 CLICK value
                             * @property {number} ENGAGEMENT=3 ENGAGEMENT value
                             * @property {number} VIDEO_VIEW=4 VIDEO_VIEW value
                             * @property {number} NONE=5 NONE value
                             */
                            InteractionEventTypeEnum.InteractionEventType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CLICK"] = 2;
                                values[valuesById[3] = "ENGAGEMENT"] = 3;
                                values[valuesById[4] = "VIDEO_VIEW"] = 4;
                                values[valuesById[5] = "NONE"] = 5;
                                return values;
                            })();
    
                            return InteractionEventTypeEnum;
                        })();
    
                        enums.QualityScoreBucketEnum = (function() {
    
                            /**
                             * Properties of a QualityScoreBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IQualityScoreBucketEnum
                             */
    
                            /**
                             * Constructs a new QualityScoreBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a QualityScoreBucketEnum.
                             * @implements IQualityScoreBucketEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IQualityScoreBucketEnum=} [p] Properties to set
                             */
                            function QualityScoreBucketEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new QualityScoreBucketEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IQualityScoreBucketEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.QualityScoreBucketEnum} QualityScoreBucketEnum instance
                             */
                            QualityScoreBucketEnum.create = function create(properties) {
                                return new QualityScoreBucketEnum(properties);
                            };
    
                            /**
                             * Encodes the specified QualityScoreBucketEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.QualityScoreBucketEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IQualityScoreBucketEnum} m QualityScoreBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QualityScoreBucketEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified QualityScoreBucketEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.QualityScoreBucketEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IQualityScoreBucketEnum} message QualityScoreBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QualityScoreBucketEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a QualityScoreBucketEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.QualityScoreBucketEnum} QualityScoreBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QualityScoreBucketEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a QualityScoreBucketEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.QualityScoreBucketEnum} QualityScoreBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QualityScoreBucketEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a QualityScoreBucketEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            QualityScoreBucketEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a QualityScoreBucketEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.QualityScoreBucketEnum} QualityScoreBucketEnum
                             */
                            QualityScoreBucketEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum();
                            };
    
                            /**
                             * Creates a plain object from a QualityScoreBucketEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.QualityScoreBucketEnum} m QualityScoreBucketEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            QualityScoreBucketEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this QualityScoreBucketEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.QualityScoreBucketEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            QualityScoreBucketEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * QualityScoreBucket enum.
                             * @name google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BELOW_AVERAGE=2 BELOW_AVERAGE value
                             * @property {number} AVERAGE=3 AVERAGE value
                             * @property {number} ABOVE_AVERAGE=4 ABOVE_AVERAGE value
                             */
                            QualityScoreBucketEnum.QualityScoreBucket = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BELOW_AVERAGE"] = 2;
                                values[valuesById[3] = "AVERAGE"] = 3;
                                values[valuesById[4] = "ABOVE_AVERAGE"] = 4;
                                return values;
                            })();
    
                            return QualityScoreBucketEnum;
                        })();
    
                        enums.UserIdentifierSourceEnum = (function() {
    
                            /**
                             * Properties of a UserIdentifierSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserIdentifierSourceEnum
                             */
    
                            /**
                             * Constructs a new UserIdentifierSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserIdentifierSourceEnum.
                             * @implements IUserIdentifierSourceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserIdentifierSourceEnum=} [p] Properties to set
                             */
                            function UserIdentifierSourceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserIdentifierSourceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserIdentifierSourceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserIdentifierSourceEnum} UserIdentifierSourceEnum instance
                             */
                            UserIdentifierSourceEnum.create = function create(properties) {
                                return new UserIdentifierSourceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserIdentifierSourceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserIdentifierSourceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserIdentifierSourceEnum} m UserIdentifierSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserIdentifierSourceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserIdentifierSourceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserIdentifierSourceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserIdentifierSourceEnum} message UserIdentifierSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserIdentifierSourceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserIdentifierSourceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserIdentifierSourceEnum} UserIdentifierSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserIdentifierSourceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserIdentifierSourceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserIdentifierSourceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserIdentifierSourceEnum} UserIdentifierSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserIdentifierSourceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserIdentifierSourceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserIdentifierSourceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserIdentifierSourceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserIdentifierSourceEnum} UserIdentifierSourceEnum
                             */
                            UserIdentifierSourceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserIdentifierSourceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserIdentifierSourceEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserIdentifierSourceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserIdentifierSourceEnum} m UserIdentifierSourceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserIdentifierSourceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserIdentifierSourceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserIdentifierSourceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserIdentifierSourceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserIdentifierSource enum.
                             * @name google.ads.googleads.v11.enums.UserIdentifierSourceEnum.UserIdentifierSource
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FIRST_PARTY=2 FIRST_PARTY value
                             * @property {number} THIRD_PARTY=3 THIRD_PARTY value
                             */
                            UserIdentifierSourceEnum.UserIdentifierSource = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FIRST_PARTY"] = 2;
                                values[valuesById[3] = "THIRD_PARTY"] = 3;
                                return values;
                            })();
    
                            return UserIdentifierSourceEnum;
                        })();
    
                        enums.AdDestinationTypeEnum = (function() {
    
                            /**
                             * Properties of an AdDestinationTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdDestinationTypeEnum
                             */
    
                            /**
                             * Constructs a new AdDestinationTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdDestinationTypeEnum.
                             * @implements IAdDestinationTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdDestinationTypeEnum=} [p] Properties to set
                             */
                            function AdDestinationTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdDestinationTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdDestinationTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdDestinationTypeEnum} AdDestinationTypeEnum instance
                             */
                            AdDestinationTypeEnum.create = function create(properties) {
                                return new AdDestinationTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdDestinationTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdDestinationTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdDestinationTypeEnum} m AdDestinationTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdDestinationTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdDestinationTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdDestinationTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdDestinationTypeEnum} message AdDestinationTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdDestinationTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdDestinationTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdDestinationTypeEnum} AdDestinationTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdDestinationTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdDestinationTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdDestinationTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdDestinationTypeEnum} AdDestinationTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdDestinationTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdDestinationTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdDestinationTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdDestinationTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdDestinationTypeEnum} AdDestinationTypeEnum
                             */
                            AdDestinationTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdDestinationTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdDestinationTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdDestinationTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdDestinationTypeEnum} m AdDestinationTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdDestinationTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdDestinationTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdDestinationTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdDestinationTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdDestinationType enum.
                             * @name google.ads.googleads.v11.enums.AdDestinationTypeEnum.AdDestinationType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOT_APPLICABLE=2 NOT_APPLICABLE value
                             * @property {number} WEBSITE=3 WEBSITE value
                             * @property {number} APP_DEEP_LINK=4 APP_DEEP_LINK value
                             * @property {number} APP_STORE=5 APP_STORE value
                             * @property {number} PHONE_CALL=6 PHONE_CALL value
                             * @property {number} MAP_DIRECTIONS=7 MAP_DIRECTIONS value
                             * @property {number} LOCATION_LISTING=8 LOCATION_LISTING value
                             * @property {number} MESSAGE=9 MESSAGE value
                             * @property {number} LEAD_FORM=10 LEAD_FORM value
                             * @property {number} YOUTUBE=11 YOUTUBE value
                             * @property {number} UNMODELED_FOR_CONVERSIONS=12 UNMODELED_FOR_CONVERSIONS value
                             */
                            AdDestinationTypeEnum.AdDestinationType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOT_APPLICABLE"] = 2;
                                values[valuesById[3] = "WEBSITE"] = 3;
                                values[valuesById[4] = "APP_DEEP_LINK"] = 4;
                                values[valuesById[5] = "APP_STORE"] = 5;
                                values[valuesById[6] = "PHONE_CALL"] = 6;
                                values[valuesById[7] = "MAP_DIRECTIONS"] = 7;
                                values[valuesById[8] = "LOCATION_LISTING"] = 8;
                                values[valuesById[9] = "MESSAGE"] = 9;
                                values[valuesById[10] = "LEAD_FORM"] = 10;
                                values[valuesById[11] = "YOUTUBE"] = 11;
                                values[valuesById[12] = "UNMODELED_FOR_CONVERSIONS"] = 12;
                                return values;
                            })();
    
                            return AdDestinationTypeEnum;
                        })();
    
                        enums.AdNetworkTypeEnum = (function() {
    
                            /**
                             * Properties of an AdNetworkTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdNetworkTypeEnum
                             */
    
                            /**
                             * Constructs a new AdNetworkTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdNetworkTypeEnum.
                             * @implements IAdNetworkTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdNetworkTypeEnum=} [p] Properties to set
                             */
                            function AdNetworkTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdNetworkTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdNetworkTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdNetworkTypeEnum} AdNetworkTypeEnum instance
                             */
                            AdNetworkTypeEnum.create = function create(properties) {
                                return new AdNetworkTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdNetworkTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdNetworkTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdNetworkTypeEnum} m AdNetworkTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdNetworkTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdNetworkTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdNetworkTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdNetworkTypeEnum} message AdNetworkTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdNetworkTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdNetworkTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdNetworkTypeEnum} AdNetworkTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdNetworkTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdNetworkTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdNetworkTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdNetworkTypeEnum} AdNetworkTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdNetworkTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdNetworkTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdNetworkTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdNetworkTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdNetworkTypeEnum} AdNetworkTypeEnum
                             */
                            AdNetworkTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdNetworkTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdNetworkTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdNetworkTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdNetworkTypeEnum} m AdNetworkTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdNetworkTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdNetworkTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdNetworkTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdNetworkTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdNetworkType enum.
                             * @name google.ads.googleads.v11.enums.AdNetworkTypeEnum.AdNetworkType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SEARCH=2 SEARCH value
                             * @property {number} SEARCH_PARTNERS=3 SEARCH_PARTNERS value
                             * @property {number} CONTENT=4 CONTENT value
                             * @property {number} YOUTUBE_SEARCH=5 YOUTUBE_SEARCH value
                             * @property {number} YOUTUBE_WATCH=6 YOUTUBE_WATCH value
                             * @property {number} MIXED=7 MIXED value
                             */
                            AdNetworkTypeEnum.AdNetworkType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SEARCH"] = 2;
                                values[valuesById[3] = "SEARCH_PARTNERS"] = 3;
                                values[valuesById[4] = "CONTENT"] = 4;
                                values[valuesById[5] = "YOUTUBE_SEARCH"] = 5;
                                values[valuesById[6] = "YOUTUBE_WATCH"] = 6;
                                values[valuesById[7] = "MIXED"] = 7;
                                return values;
                            })();
    
                            return AdNetworkTypeEnum;
                        })();
    
                        enums.BudgetCampaignAssociationStatusEnum = (function() {
    
                            /**
                             * Properties of a BudgetCampaignAssociationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBudgetCampaignAssociationStatusEnum
                             */
    
                            /**
                             * Constructs a new BudgetCampaignAssociationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BudgetCampaignAssociationStatusEnum.
                             * @implements IBudgetCampaignAssociationStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBudgetCampaignAssociationStatusEnum=} [p] Properties to set
                             */
                            function BudgetCampaignAssociationStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BudgetCampaignAssociationStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetCampaignAssociationStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum} BudgetCampaignAssociationStatusEnum instance
                             */
                            BudgetCampaignAssociationStatusEnum.create = function create(properties) {
                                return new BudgetCampaignAssociationStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetCampaignAssociationStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetCampaignAssociationStatusEnum} m BudgetCampaignAssociationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetCampaignAssociationStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetCampaignAssociationStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetCampaignAssociationStatusEnum} message BudgetCampaignAssociationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetCampaignAssociationStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetCampaignAssociationStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum} BudgetCampaignAssociationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetCampaignAssociationStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetCampaignAssociationStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum} BudgetCampaignAssociationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetCampaignAssociationStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetCampaignAssociationStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetCampaignAssociationStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BudgetCampaignAssociationStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum} BudgetCampaignAssociationStatusEnum
                             */
                            BudgetCampaignAssociationStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a BudgetCampaignAssociationStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum} m BudgetCampaignAssociationStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetCampaignAssociationStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BudgetCampaignAssociationStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetCampaignAssociationStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BudgetCampaignAssociationStatus enum.
                             * @name google.ads.googleads.v11.enums.BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return BudgetCampaignAssociationStatusEnum;
                        })();
    
                        enums.ClickTypeEnum = (function() {
    
                            /**
                             * Properties of a ClickTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IClickTypeEnum
                             */
    
                            /**
                             * Constructs a new ClickTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ClickTypeEnum.
                             * @implements IClickTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IClickTypeEnum=} [p] Properties to set
                             */
                            function ClickTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ClickTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IClickTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ClickTypeEnum} ClickTypeEnum instance
                             */
                            ClickTypeEnum.create = function create(properties) {
                                return new ClickTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ClickTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ClickTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IClickTypeEnum} m ClickTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ClickTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ClickTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IClickTypeEnum} message ClickTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClickTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ClickTypeEnum} ClickTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ClickTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ClickTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ClickTypeEnum} ClickTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClickTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClickTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ClickTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ClickTypeEnum} ClickTypeEnum
                             */
                            ClickTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ClickTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ClickTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ClickTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ClickTypeEnum} m ClickTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClickTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ClickTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ClickTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClickTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ClickType enum.
                             * @name google.ads.googleads.v11.enums.ClickTypeEnum.ClickType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APP_DEEPLINK=2 APP_DEEPLINK value
                             * @property {number} BREADCRUMBS=3 BREADCRUMBS value
                             * @property {number} BROADBAND_PLAN=4 BROADBAND_PLAN value
                             * @property {number} CALL_TRACKING=5 CALL_TRACKING value
                             * @property {number} CALLS=6 CALLS value
                             * @property {number} CLICK_ON_ENGAGEMENT_AD=7 CLICK_ON_ENGAGEMENT_AD value
                             * @property {number} GET_DIRECTIONS=8 GET_DIRECTIONS value
                             * @property {number} LOCATION_EXPANSION=9 LOCATION_EXPANSION value
                             * @property {number} LOCATION_FORMAT_CALL=10 LOCATION_FORMAT_CALL value
                             * @property {number} LOCATION_FORMAT_DIRECTIONS=11 LOCATION_FORMAT_DIRECTIONS value
                             * @property {number} LOCATION_FORMAT_IMAGE=12 LOCATION_FORMAT_IMAGE value
                             * @property {number} LOCATION_FORMAT_LANDING_PAGE=13 LOCATION_FORMAT_LANDING_PAGE value
                             * @property {number} LOCATION_FORMAT_MAP=14 LOCATION_FORMAT_MAP value
                             * @property {number} LOCATION_FORMAT_STORE_INFO=15 LOCATION_FORMAT_STORE_INFO value
                             * @property {number} LOCATION_FORMAT_TEXT=16 LOCATION_FORMAT_TEXT value
                             * @property {number} MOBILE_CALL_TRACKING=17 MOBILE_CALL_TRACKING value
                             * @property {number} OFFER_PRINTS=18 OFFER_PRINTS value
                             * @property {number} OTHER=19 OTHER value
                             * @property {number} PRODUCT_EXTENSION_CLICKS=20 PRODUCT_EXTENSION_CLICKS value
                             * @property {number} PRODUCT_LISTING_AD_CLICKS=21 PRODUCT_LISTING_AD_CLICKS value
                             * @property {number} SITELINKS=22 SITELINKS value
                             * @property {number} STORE_LOCATOR=23 STORE_LOCATOR value
                             * @property {number} URL_CLICKS=25 URL_CLICKS value
                             * @property {number} VIDEO_APP_STORE_CLICKS=26 VIDEO_APP_STORE_CLICKS value
                             * @property {number} VIDEO_CALL_TO_ACTION_CLICKS=27 VIDEO_CALL_TO_ACTION_CLICKS value
                             * @property {number} VIDEO_CARD_ACTION_HEADLINE_CLICKS=28 VIDEO_CARD_ACTION_HEADLINE_CLICKS value
                             * @property {number} VIDEO_END_CAP_CLICKS=29 VIDEO_END_CAP_CLICKS value
                             * @property {number} VIDEO_WEBSITE_CLICKS=30 VIDEO_WEBSITE_CLICKS value
                             * @property {number} VISUAL_SITELINKS=31 VISUAL_SITELINKS value
                             * @property {number} WIRELESS_PLAN=32 WIRELESS_PLAN value
                             * @property {number} PRODUCT_LISTING_AD_LOCAL=33 PRODUCT_LISTING_AD_LOCAL value
                             * @property {number} PRODUCT_LISTING_AD_MULTICHANNEL_LOCAL=34 PRODUCT_LISTING_AD_MULTICHANNEL_LOCAL value
                             * @property {number} PRODUCT_LISTING_AD_MULTICHANNEL_ONLINE=35 PRODUCT_LISTING_AD_MULTICHANNEL_ONLINE value
                             * @property {number} PRODUCT_LISTING_ADS_COUPON=36 PRODUCT_LISTING_ADS_COUPON value
                             * @property {number} PRODUCT_LISTING_AD_TRANSACTABLE=37 PRODUCT_LISTING_AD_TRANSACTABLE value
                             * @property {number} PRODUCT_AD_APP_DEEPLINK=38 PRODUCT_AD_APP_DEEPLINK value
                             * @property {number} SHOWCASE_AD_CATEGORY_LINK=39 SHOWCASE_AD_CATEGORY_LINK value
                             * @property {number} SHOWCASE_AD_LOCAL_STOREFRONT_LINK=40 SHOWCASE_AD_LOCAL_STOREFRONT_LINK value
                             * @property {number} SHOWCASE_AD_ONLINE_PRODUCT_LINK=42 SHOWCASE_AD_ONLINE_PRODUCT_LINK value
                             * @property {number} SHOWCASE_AD_LOCAL_PRODUCT_LINK=43 SHOWCASE_AD_LOCAL_PRODUCT_LINK value
                             * @property {number} PROMOTION_EXTENSION=44 PROMOTION_EXTENSION value
                             * @property {number} SWIPEABLE_GALLERY_AD_HEADLINE=45 SWIPEABLE_GALLERY_AD_HEADLINE value
                             * @property {number} SWIPEABLE_GALLERY_AD_SWIPES=46 SWIPEABLE_GALLERY_AD_SWIPES value
                             * @property {number} SWIPEABLE_GALLERY_AD_SEE_MORE=47 SWIPEABLE_GALLERY_AD_SEE_MORE value
                             * @property {number} SWIPEABLE_GALLERY_AD_SITELINK_ONE=48 SWIPEABLE_GALLERY_AD_SITELINK_ONE value
                             * @property {number} SWIPEABLE_GALLERY_AD_SITELINK_TWO=49 SWIPEABLE_GALLERY_AD_SITELINK_TWO value
                             * @property {number} SWIPEABLE_GALLERY_AD_SITELINK_THREE=50 SWIPEABLE_GALLERY_AD_SITELINK_THREE value
                             * @property {number} SWIPEABLE_GALLERY_AD_SITELINK_FOUR=51 SWIPEABLE_GALLERY_AD_SITELINK_FOUR value
                             * @property {number} SWIPEABLE_GALLERY_AD_SITELINK_FIVE=52 SWIPEABLE_GALLERY_AD_SITELINK_FIVE value
                             * @property {number} HOTEL_PRICE=53 HOTEL_PRICE value
                             * @property {number} PRICE_EXTENSION=54 PRICE_EXTENSION value
                             * @property {number} HOTEL_BOOK_ON_GOOGLE_ROOM_SELECTION=55 HOTEL_BOOK_ON_GOOGLE_ROOM_SELECTION value
                             * @property {number} SHOPPING_COMPARISON_LISTING=56 SHOPPING_COMPARISON_LISTING value
                             * @property {number} CROSS_NETWORK=57 CROSS_NETWORK value
                             */
                            ClickTypeEnum.ClickType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APP_DEEPLINK"] = 2;
                                values[valuesById[3] = "BREADCRUMBS"] = 3;
                                values[valuesById[4] = "BROADBAND_PLAN"] = 4;
                                values[valuesById[5] = "CALL_TRACKING"] = 5;
                                values[valuesById[6] = "CALLS"] = 6;
                                values[valuesById[7] = "CLICK_ON_ENGAGEMENT_AD"] = 7;
                                values[valuesById[8] = "GET_DIRECTIONS"] = 8;
                                values[valuesById[9] = "LOCATION_EXPANSION"] = 9;
                                values[valuesById[10] = "LOCATION_FORMAT_CALL"] = 10;
                                values[valuesById[11] = "LOCATION_FORMAT_DIRECTIONS"] = 11;
                                values[valuesById[12] = "LOCATION_FORMAT_IMAGE"] = 12;
                                values[valuesById[13] = "LOCATION_FORMAT_LANDING_PAGE"] = 13;
                                values[valuesById[14] = "LOCATION_FORMAT_MAP"] = 14;
                                values[valuesById[15] = "LOCATION_FORMAT_STORE_INFO"] = 15;
                                values[valuesById[16] = "LOCATION_FORMAT_TEXT"] = 16;
                                values[valuesById[17] = "MOBILE_CALL_TRACKING"] = 17;
                                values[valuesById[18] = "OFFER_PRINTS"] = 18;
                                values[valuesById[19] = "OTHER"] = 19;
                                values[valuesById[20] = "PRODUCT_EXTENSION_CLICKS"] = 20;
                                values[valuesById[21] = "PRODUCT_LISTING_AD_CLICKS"] = 21;
                                values[valuesById[22] = "SITELINKS"] = 22;
                                values[valuesById[23] = "STORE_LOCATOR"] = 23;
                                values[valuesById[25] = "URL_CLICKS"] = 25;
                                values[valuesById[26] = "VIDEO_APP_STORE_CLICKS"] = 26;
                                values[valuesById[27] = "VIDEO_CALL_TO_ACTION_CLICKS"] = 27;
                                values[valuesById[28] = "VIDEO_CARD_ACTION_HEADLINE_CLICKS"] = 28;
                                values[valuesById[29] = "VIDEO_END_CAP_CLICKS"] = 29;
                                values[valuesById[30] = "VIDEO_WEBSITE_CLICKS"] = 30;
                                values[valuesById[31] = "VISUAL_SITELINKS"] = 31;
                                values[valuesById[32] = "WIRELESS_PLAN"] = 32;
                                values[valuesById[33] = "PRODUCT_LISTING_AD_LOCAL"] = 33;
                                values[valuesById[34] = "PRODUCT_LISTING_AD_MULTICHANNEL_LOCAL"] = 34;
                                values[valuesById[35] = "PRODUCT_LISTING_AD_MULTICHANNEL_ONLINE"] = 35;
                                values[valuesById[36] = "PRODUCT_LISTING_ADS_COUPON"] = 36;
                                values[valuesById[37] = "PRODUCT_LISTING_AD_TRANSACTABLE"] = 37;
                                values[valuesById[38] = "PRODUCT_AD_APP_DEEPLINK"] = 38;
                                values[valuesById[39] = "SHOWCASE_AD_CATEGORY_LINK"] = 39;
                                values[valuesById[40] = "SHOWCASE_AD_LOCAL_STOREFRONT_LINK"] = 40;
                                values[valuesById[42] = "SHOWCASE_AD_ONLINE_PRODUCT_LINK"] = 42;
                                values[valuesById[43] = "SHOWCASE_AD_LOCAL_PRODUCT_LINK"] = 43;
                                values[valuesById[44] = "PROMOTION_EXTENSION"] = 44;
                                values[valuesById[45] = "SWIPEABLE_GALLERY_AD_HEADLINE"] = 45;
                                values[valuesById[46] = "SWIPEABLE_GALLERY_AD_SWIPES"] = 46;
                                values[valuesById[47] = "SWIPEABLE_GALLERY_AD_SEE_MORE"] = 47;
                                values[valuesById[48] = "SWIPEABLE_GALLERY_AD_SITELINK_ONE"] = 48;
                                values[valuesById[49] = "SWIPEABLE_GALLERY_AD_SITELINK_TWO"] = 49;
                                values[valuesById[50] = "SWIPEABLE_GALLERY_AD_SITELINK_THREE"] = 50;
                                values[valuesById[51] = "SWIPEABLE_GALLERY_AD_SITELINK_FOUR"] = 51;
                                values[valuesById[52] = "SWIPEABLE_GALLERY_AD_SITELINK_FIVE"] = 52;
                                values[valuesById[53] = "HOTEL_PRICE"] = 53;
                                values[valuesById[54] = "PRICE_EXTENSION"] = 54;
                                values[valuesById[55] = "HOTEL_BOOK_ON_GOOGLE_ROOM_SELECTION"] = 55;
                                values[valuesById[56] = "SHOPPING_COMPARISON_LISTING"] = 56;
                                values[valuesById[57] = "CROSS_NETWORK"] = 57;
                                return values;
                            })();
    
                            return ClickTypeEnum;
                        })();
    
                        enums.ConversionActionCategoryEnum = (function() {
    
                            /**
                             * Properties of a ConversionActionCategoryEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionActionCategoryEnum
                             */
    
                            /**
                             * Constructs a new ConversionActionCategoryEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionActionCategoryEnum.
                             * @implements IConversionActionCategoryEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionActionCategoryEnum=} [p] Properties to set
                             */
                            function ConversionActionCategoryEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionActionCategoryEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionCategoryEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCategoryEnum} ConversionActionCategoryEnum instance
                             */
                            ConversionActionCategoryEnum.create = function create(properties) {
                                return new ConversionActionCategoryEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionActionCategoryEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionCategoryEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionCategoryEnum} m ConversionActionCategoryEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionCategoryEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionActionCategoryEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionCategoryEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionCategoryEnum} message ConversionActionCategoryEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionCategoryEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionActionCategoryEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCategoryEnum} ConversionActionCategoryEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionCategoryEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionActionCategoryEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCategoryEnum} ConversionActionCategoryEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionCategoryEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionActionCategoryEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionActionCategoryEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionActionCategoryEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCategoryEnum} ConversionActionCategoryEnum
                             */
                            ConversionActionCategoryEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionActionCategoryEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionActionCategoryEnum} m ConversionActionCategoryEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionActionCategoryEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionActionCategoryEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCategoryEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionActionCategoryEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionActionCategory enum.
                             * @name google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DEFAULT=2 DEFAULT value
                             * @property {number} PAGE_VIEW=3 PAGE_VIEW value
                             * @property {number} PURCHASE=4 PURCHASE value
                             * @property {number} SIGNUP=5 SIGNUP value
                             * @property {number} DOWNLOAD=7 DOWNLOAD value
                             * @property {number} ADD_TO_CART=8 ADD_TO_CART value
                             * @property {number} BEGIN_CHECKOUT=9 BEGIN_CHECKOUT value
                             * @property {number} SUBSCRIBE_PAID=10 SUBSCRIBE_PAID value
                             * @property {number} PHONE_CALL_LEAD=11 PHONE_CALL_LEAD value
                             * @property {number} IMPORTED_LEAD=12 IMPORTED_LEAD value
                             * @property {number} SUBMIT_LEAD_FORM=13 SUBMIT_LEAD_FORM value
                             * @property {number} BOOK_APPOINTMENT=14 BOOK_APPOINTMENT value
                             * @property {number} REQUEST_QUOTE=15 REQUEST_QUOTE value
                             * @property {number} GET_DIRECTIONS=16 GET_DIRECTIONS value
                             * @property {number} OUTBOUND_CLICK=17 OUTBOUND_CLICK value
                             * @property {number} CONTACT=18 CONTACT value
                             * @property {number} ENGAGEMENT=19 ENGAGEMENT value
                             * @property {number} STORE_VISIT=20 STORE_VISIT value
                             * @property {number} STORE_SALE=21 STORE_SALE value
                             * @property {number} QUALIFIED_LEAD=22 QUALIFIED_LEAD value
                             * @property {number} CONVERTED_LEAD=23 CONVERTED_LEAD value
                             */
                            ConversionActionCategoryEnum.ConversionActionCategory = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DEFAULT"] = 2;
                                values[valuesById[3] = "PAGE_VIEW"] = 3;
                                values[valuesById[4] = "PURCHASE"] = 4;
                                values[valuesById[5] = "SIGNUP"] = 5;
                                values[valuesById[7] = "DOWNLOAD"] = 7;
                                values[valuesById[8] = "ADD_TO_CART"] = 8;
                                values[valuesById[9] = "BEGIN_CHECKOUT"] = 9;
                                values[valuesById[10] = "SUBSCRIBE_PAID"] = 10;
                                values[valuesById[11] = "PHONE_CALL_LEAD"] = 11;
                                values[valuesById[12] = "IMPORTED_LEAD"] = 12;
                                values[valuesById[13] = "SUBMIT_LEAD_FORM"] = 13;
                                values[valuesById[14] = "BOOK_APPOINTMENT"] = 14;
                                values[valuesById[15] = "REQUEST_QUOTE"] = 15;
                                values[valuesById[16] = "GET_DIRECTIONS"] = 16;
                                values[valuesById[17] = "OUTBOUND_CLICK"] = 17;
                                values[valuesById[18] = "CONTACT"] = 18;
                                values[valuesById[19] = "ENGAGEMENT"] = 19;
                                values[valuesById[20] = "STORE_VISIT"] = 20;
                                values[valuesById[21] = "STORE_SALE"] = 21;
                                values[valuesById[22] = "QUALIFIED_LEAD"] = 22;
                                values[valuesById[23] = "CONVERTED_LEAD"] = 23;
                                return values;
                            })();
    
                            return ConversionActionCategoryEnum;
                        })();
    
                        enums.ConversionAttributionEventTypeEnum = (function() {
    
                            /**
                             * Properties of a ConversionAttributionEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionAttributionEventTypeEnum
                             */
    
                            /**
                             * Constructs a new ConversionAttributionEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionAttributionEventTypeEnum.
                             * @implements IConversionAttributionEventTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionAttributionEventTypeEnum=} [p] Properties to set
                             */
                            function ConversionAttributionEventTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionAttributionEventTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionAttributionEventTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum} ConversionAttributionEventTypeEnum instance
                             */
                            ConversionAttributionEventTypeEnum.create = function create(properties) {
                                return new ConversionAttributionEventTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionAttributionEventTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionAttributionEventTypeEnum} m ConversionAttributionEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAttributionEventTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionAttributionEventTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionAttributionEventTypeEnum} message ConversionAttributionEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAttributionEventTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionAttributionEventTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum} ConversionAttributionEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAttributionEventTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionAttributionEventTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum} ConversionAttributionEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAttributionEventTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionAttributionEventTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionAttributionEventTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionAttributionEventTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum} ConversionAttributionEventTypeEnum
                             */
                            ConversionAttributionEventTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionAttributionEventTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum} m ConversionAttributionEventTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionAttributionEventTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionAttributionEventTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionAttributionEventTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionAttributionEventType enum.
                             * @name google.ads.googleads.v11.enums.ConversionAttributionEventTypeEnum.ConversionAttributionEventType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} IMPRESSION=2 IMPRESSION value
                             * @property {number} INTERACTION=3 INTERACTION value
                             */
                            ConversionAttributionEventTypeEnum.ConversionAttributionEventType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "IMPRESSION"] = 2;
                                values[valuesById[3] = "INTERACTION"] = 3;
                                return values;
                            })();
    
                            return ConversionAttributionEventTypeEnum;
                        })();
    
                        enums.ConversionLagBucketEnum = (function() {
    
                            /**
                             * Properties of a ConversionLagBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionLagBucketEnum
                             */
    
                            /**
                             * Constructs a new ConversionLagBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionLagBucketEnum.
                             * @implements IConversionLagBucketEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionLagBucketEnum=} [p] Properties to set
                             */
                            function ConversionLagBucketEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionLagBucketEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionLagBucketEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionLagBucketEnum} ConversionLagBucketEnum instance
                             */
                            ConversionLagBucketEnum.create = function create(properties) {
                                return new ConversionLagBucketEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionLagBucketEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionLagBucketEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionLagBucketEnum} m ConversionLagBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionLagBucketEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionLagBucketEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionLagBucketEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionLagBucketEnum} message ConversionLagBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionLagBucketEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionLagBucketEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionLagBucketEnum} ConversionLagBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionLagBucketEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionLagBucketEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionLagBucketEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionLagBucketEnum} ConversionLagBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionLagBucketEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionLagBucketEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionLagBucketEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionLagBucketEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionLagBucketEnum} ConversionLagBucketEnum
                             */
                            ConversionLagBucketEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionLagBucketEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionLagBucketEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionLagBucketEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionLagBucketEnum} m ConversionLagBucketEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionLagBucketEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionLagBucketEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionLagBucketEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionLagBucketEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionLagBucket enum.
                             * @name google.ads.googleads.v11.enums.ConversionLagBucketEnum.ConversionLagBucket
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LESS_THAN_ONE_DAY=2 LESS_THAN_ONE_DAY value
                             * @property {number} ONE_TO_TWO_DAYS=3 ONE_TO_TWO_DAYS value
                             * @property {number} TWO_TO_THREE_DAYS=4 TWO_TO_THREE_DAYS value
                             * @property {number} THREE_TO_FOUR_DAYS=5 THREE_TO_FOUR_DAYS value
                             * @property {number} FOUR_TO_FIVE_DAYS=6 FOUR_TO_FIVE_DAYS value
                             * @property {number} FIVE_TO_SIX_DAYS=7 FIVE_TO_SIX_DAYS value
                             * @property {number} SIX_TO_SEVEN_DAYS=8 SIX_TO_SEVEN_DAYS value
                             * @property {number} SEVEN_TO_EIGHT_DAYS=9 SEVEN_TO_EIGHT_DAYS value
                             * @property {number} EIGHT_TO_NINE_DAYS=10 EIGHT_TO_NINE_DAYS value
                             * @property {number} NINE_TO_TEN_DAYS=11 NINE_TO_TEN_DAYS value
                             * @property {number} TEN_TO_ELEVEN_DAYS=12 TEN_TO_ELEVEN_DAYS value
                             * @property {number} ELEVEN_TO_TWELVE_DAYS=13 ELEVEN_TO_TWELVE_DAYS value
                             * @property {number} TWELVE_TO_THIRTEEN_DAYS=14 TWELVE_TO_THIRTEEN_DAYS value
                             * @property {number} THIRTEEN_TO_FOURTEEN_DAYS=15 THIRTEEN_TO_FOURTEEN_DAYS value
                             * @property {number} FOURTEEN_TO_TWENTY_ONE_DAYS=16 FOURTEEN_TO_TWENTY_ONE_DAYS value
                             * @property {number} TWENTY_ONE_TO_THIRTY_DAYS=17 TWENTY_ONE_TO_THIRTY_DAYS value
                             * @property {number} THIRTY_TO_FORTY_FIVE_DAYS=18 THIRTY_TO_FORTY_FIVE_DAYS value
                             * @property {number} FORTY_FIVE_TO_SIXTY_DAYS=19 FORTY_FIVE_TO_SIXTY_DAYS value
                             * @property {number} SIXTY_TO_NINETY_DAYS=20 SIXTY_TO_NINETY_DAYS value
                             */
                            ConversionLagBucketEnum.ConversionLagBucket = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LESS_THAN_ONE_DAY"] = 2;
                                values[valuesById[3] = "ONE_TO_TWO_DAYS"] = 3;
                                values[valuesById[4] = "TWO_TO_THREE_DAYS"] = 4;
                                values[valuesById[5] = "THREE_TO_FOUR_DAYS"] = 5;
                                values[valuesById[6] = "FOUR_TO_FIVE_DAYS"] = 6;
                                values[valuesById[7] = "FIVE_TO_SIX_DAYS"] = 7;
                                values[valuesById[8] = "SIX_TO_SEVEN_DAYS"] = 8;
                                values[valuesById[9] = "SEVEN_TO_EIGHT_DAYS"] = 9;
                                values[valuesById[10] = "EIGHT_TO_NINE_DAYS"] = 10;
                                values[valuesById[11] = "NINE_TO_TEN_DAYS"] = 11;
                                values[valuesById[12] = "TEN_TO_ELEVEN_DAYS"] = 12;
                                values[valuesById[13] = "ELEVEN_TO_TWELVE_DAYS"] = 13;
                                values[valuesById[14] = "TWELVE_TO_THIRTEEN_DAYS"] = 14;
                                values[valuesById[15] = "THIRTEEN_TO_FOURTEEN_DAYS"] = 15;
                                values[valuesById[16] = "FOURTEEN_TO_TWENTY_ONE_DAYS"] = 16;
                                values[valuesById[17] = "TWENTY_ONE_TO_THIRTY_DAYS"] = 17;
                                values[valuesById[18] = "THIRTY_TO_FORTY_FIVE_DAYS"] = 18;
                                values[valuesById[19] = "FORTY_FIVE_TO_SIXTY_DAYS"] = 19;
                                values[valuesById[20] = "SIXTY_TO_NINETY_DAYS"] = 20;
                                return values;
                            })();
    
                            return ConversionLagBucketEnum;
                        })();
    
                        enums.ConversionOrAdjustmentLagBucketEnum = (function() {
    
                            /**
                             * Properties of a ConversionOrAdjustmentLagBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionOrAdjustmentLagBucketEnum
                             */
    
                            /**
                             * Constructs a new ConversionOrAdjustmentLagBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionOrAdjustmentLagBucketEnum.
                             * @implements IConversionOrAdjustmentLagBucketEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionOrAdjustmentLagBucketEnum=} [p] Properties to set
                             */
                            function ConversionOrAdjustmentLagBucketEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionOrAdjustmentLagBucketEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionOrAdjustmentLagBucketEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum} ConversionOrAdjustmentLagBucketEnum instance
                             */
                            ConversionOrAdjustmentLagBucketEnum.create = function create(properties) {
                                return new ConversionOrAdjustmentLagBucketEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionOrAdjustmentLagBucketEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionOrAdjustmentLagBucketEnum} m ConversionOrAdjustmentLagBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionOrAdjustmentLagBucketEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionOrAdjustmentLagBucketEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionOrAdjustmentLagBucketEnum} message ConversionOrAdjustmentLagBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionOrAdjustmentLagBucketEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionOrAdjustmentLagBucketEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum} ConversionOrAdjustmentLagBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionOrAdjustmentLagBucketEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionOrAdjustmentLagBucketEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum} ConversionOrAdjustmentLagBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionOrAdjustmentLagBucketEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionOrAdjustmentLagBucketEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionOrAdjustmentLagBucketEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionOrAdjustmentLagBucketEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum} ConversionOrAdjustmentLagBucketEnum
                             */
                            ConversionOrAdjustmentLagBucketEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionOrAdjustmentLagBucketEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum} m ConversionOrAdjustmentLagBucketEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionOrAdjustmentLagBucketEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionOrAdjustmentLagBucketEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionOrAdjustmentLagBucketEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionOrAdjustmentLagBucket enum.
                             * @name google.ads.googleads.v11.enums.ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CONVERSION_LESS_THAN_ONE_DAY=2 CONVERSION_LESS_THAN_ONE_DAY value
                             * @property {number} CONVERSION_ONE_TO_TWO_DAYS=3 CONVERSION_ONE_TO_TWO_DAYS value
                             * @property {number} CONVERSION_TWO_TO_THREE_DAYS=4 CONVERSION_TWO_TO_THREE_DAYS value
                             * @property {number} CONVERSION_THREE_TO_FOUR_DAYS=5 CONVERSION_THREE_TO_FOUR_DAYS value
                             * @property {number} CONVERSION_FOUR_TO_FIVE_DAYS=6 CONVERSION_FOUR_TO_FIVE_DAYS value
                             * @property {number} CONVERSION_FIVE_TO_SIX_DAYS=7 CONVERSION_FIVE_TO_SIX_DAYS value
                             * @property {number} CONVERSION_SIX_TO_SEVEN_DAYS=8 CONVERSION_SIX_TO_SEVEN_DAYS value
                             * @property {number} CONVERSION_SEVEN_TO_EIGHT_DAYS=9 CONVERSION_SEVEN_TO_EIGHT_DAYS value
                             * @property {number} CONVERSION_EIGHT_TO_NINE_DAYS=10 CONVERSION_EIGHT_TO_NINE_DAYS value
                             * @property {number} CONVERSION_NINE_TO_TEN_DAYS=11 CONVERSION_NINE_TO_TEN_DAYS value
                             * @property {number} CONVERSION_TEN_TO_ELEVEN_DAYS=12 CONVERSION_TEN_TO_ELEVEN_DAYS value
                             * @property {number} CONVERSION_ELEVEN_TO_TWELVE_DAYS=13 CONVERSION_ELEVEN_TO_TWELVE_DAYS value
                             * @property {number} CONVERSION_TWELVE_TO_THIRTEEN_DAYS=14 CONVERSION_TWELVE_TO_THIRTEEN_DAYS value
                             * @property {number} CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS=15 CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS value
                             * @property {number} CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS=16 CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS value
                             * @property {number} CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS=17 CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS value
                             * @property {number} CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS=18 CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS value
                             * @property {number} CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS=19 CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS value
                             * @property {number} CONVERSION_SIXTY_TO_NINETY_DAYS=20 CONVERSION_SIXTY_TO_NINETY_DAYS value
                             * @property {number} ADJUSTMENT_LESS_THAN_ONE_DAY=21 ADJUSTMENT_LESS_THAN_ONE_DAY value
                             * @property {number} ADJUSTMENT_ONE_TO_TWO_DAYS=22 ADJUSTMENT_ONE_TO_TWO_DAYS value
                             * @property {number} ADJUSTMENT_TWO_TO_THREE_DAYS=23 ADJUSTMENT_TWO_TO_THREE_DAYS value
                             * @property {number} ADJUSTMENT_THREE_TO_FOUR_DAYS=24 ADJUSTMENT_THREE_TO_FOUR_DAYS value
                             * @property {number} ADJUSTMENT_FOUR_TO_FIVE_DAYS=25 ADJUSTMENT_FOUR_TO_FIVE_DAYS value
                             * @property {number} ADJUSTMENT_FIVE_TO_SIX_DAYS=26 ADJUSTMENT_FIVE_TO_SIX_DAYS value
                             * @property {number} ADJUSTMENT_SIX_TO_SEVEN_DAYS=27 ADJUSTMENT_SIX_TO_SEVEN_DAYS value
                             * @property {number} ADJUSTMENT_SEVEN_TO_EIGHT_DAYS=28 ADJUSTMENT_SEVEN_TO_EIGHT_DAYS value
                             * @property {number} ADJUSTMENT_EIGHT_TO_NINE_DAYS=29 ADJUSTMENT_EIGHT_TO_NINE_DAYS value
                             * @property {number} ADJUSTMENT_NINE_TO_TEN_DAYS=30 ADJUSTMENT_NINE_TO_TEN_DAYS value
                             * @property {number} ADJUSTMENT_TEN_TO_ELEVEN_DAYS=31 ADJUSTMENT_TEN_TO_ELEVEN_DAYS value
                             * @property {number} ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS=32 ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS value
                             * @property {number} ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS=33 ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS value
                             * @property {number} ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS=34 ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS value
                             * @property {number} ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS=35 ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS value
                             * @property {number} ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS=36 ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS value
                             * @property {number} ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS=37 ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS value
                             * @property {number} ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS=38 ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS value
                             * @property {number} ADJUSTMENT_SIXTY_TO_NINETY_DAYS=39 ADJUSTMENT_SIXTY_TO_NINETY_DAYS value
                             * @property {number} ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS=40 ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS value
                             * @property {number} CONVERSION_UNKNOWN=41 CONVERSION_UNKNOWN value
                             * @property {number} ADJUSTMENT_UNKNOWN=42 ADJUSTMENT_UNKNOWN value
                             */
                            ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CONVERSION_LESS_THAN_ONE_DAY"] = 2;
                                values[valuesById[3] = "CONVERSION_ONE_TO_TWO_DAYS"] = 3;
                                values[valuesById[4] = "CONVERSION_TWO_TO_THREE_DAYS"] = 4;
                                values[valuesById[5] = "CONVERSION_THREE_TO_FOUR_DAYS"] = 5;
                                values[valuesById[6] = "CONVERSION_FOUR_TO_FIVE_DAYS"] = 6;
                                values[valuesById[7] = "CONVERSION_FIVE_TO_SIX_DAYS"] = 7;
                                values[valuesById[8] = "CONVERSION_SIX_TO_SEVEN_DAYS"] = 8;
                                values[valuesById[9] = "CONVERSION_SEVEN_TO_EIGHT_DAYS"] = 9;
                                values[valuesById[10] = "CONVERSION_EIGHT_TO_NINE_DAYS"] = 10;
                                values[valuesById[11] = "CONVERSION_NINE_TO_TEN_DAYS"] = 11;
                                values[valuesById[12] = "CONVERSION_TEN_TO_ELEVEN_DAYS"] = 12;
                                values[valuesById[13] = "CONVERSION_ELEVEN_TO_TWELVE_DAYS"] = 13;
                                values[valuesById[14] = "CONVERSION_TWELVE_TO_THIRTEEN_DAYS"] = 14;
                                values[valuesById[15] = "CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS"] = 15;
                                values[valuesById[16] = "CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS"] = 16;
                                values[valuesById[17] = "CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS"] = 17;
                                values[valuesById[18] = "CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS"] = 18;
                                values[valuesById[19] = "CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS"] = 19;
                                values[valuesById[20] = "CONVERSION_SIXTY_TO_NINETY_DAYS"] = 20;
                                values[valuesById[21] = "ADJUSTMENT_LESS_THAN_ONE_DAY"] = 21;
                                values[valuesById[22] = "ADJUSTMENT_ONE_TO_TWO_DAYS"] = 22;
                                values[valuesById[23] = "ADJUSTMENT_TWO_TO_THREE_DAYS"] = 23;
                                values[valuesById[24] = "ADJUSTMENT_THREE_TO_FOUR_DAYS"] = 24;
                                values[valuesById[25] = "ADJUSTMENT_FOUR_TO_FIVE_DAYS"] = 25;
                                values[valuesById[26] = "ADJUSTMENT_FIVE_TO_SIX_DAYS"] = 26;
                                values[valuesById[27] = "ADJUSTMENT_SIX_TO_SEVEN_DAYS"] = 27;
                                values[valuesById[28] = "ADJUSTMENT_SEVEN_TO_EIGHT_DAYS"] = 28;
                                values[valuesById[29] = "ADJUSTMENT_EIGHT_TO_NINE_DAYS"] = 29;
                                values[valuesById[30] = "ADJUSTMENT_NINE_TO_TEN_DAYS"] = 30;
                                values[valuesById[31] = "ADJUSTMENT_TEN_TO_ELEVEN_DAYS"] = 31;
                                values[valuesById[32] = "ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS"] = 32;
                                values[valuesById[33] = "ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS"] = 33;
                                values[valuesById[34] = "ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS"] = 34;
                                values[valuesById[35] = "ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS"] = 35;
                                values[valuesById[36] = "ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS"] = 36;
                                values[valuesById[37] = "ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS"] = 37;
                                values[valuesById[38] = "ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS"] = 38;
                                values[valuesById[39] = "ADJUSTMENT_SIXTY_TO_NINETY_DAYS"] = 39;
                                values[valuesById[40] = "ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS"] = 40;
                                values[valuesById[41] = "CONVERSION_UNKNOWN"] = 41;
                                values[valuesById[42] = "ADJUSTMENT_UNKNOWN"] = 42;
                                return values;
                            })();
    
                            return ConversionOrAdjustmentLagBucketEnum;
                        })();
    
                        enums.ConversionValueRulePrimaryDimensionEnum = (function() {
    
                            /**
                             * Properties of a ConversionValueRulePrimaryDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionValueRulePrimaryDimensionEnum
                             */
    
                            /**
                             * Constructs a new ConversionValueRulePrimaryDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionValueRulePrimaryDimensionEnum.
                             * @implements IConversionValueRulePrimaryDimensionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionValueRulePrimaryDimensionEnum=} [p] Properties to set
                             */
                            function ConversionValueRulePrimaryDimensionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionValueRulePrimaryDimensionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRulePrimaryDimensionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum} ConversionValueRulePrimaryDimensionEnum instance
                             */
                            ConversionValueRulePrimaryDimensionEnum.create = function create(properties) {
                                return new ConversionValueRulePrimaryDimensionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRulePrimaryDimensionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRulePrimaryDimensionEnum} m ConversionValueRulePrimaryDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRulePrimaryDimensionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRulePrimaryDimensionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRulePrimaryDimensionEnum} message ConversionValueRulePrimaryDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRulePrimaryDimensionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRulePrimaryDimensionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum} ConversionValueRulePrimaryDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRulePrimaryDimensionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRulePrimaryDimensionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum} ConversionValueRulePrimaryDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRulePrimaryDimensionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRulePrimaryDimensionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRulePrimaryDimensionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRulePrimaryDimensionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum} ConversionValueRulePrimaryDimensionEnum
                             */
                            ConversionValueRulePrimaryDimensionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRulePrimaryDimensionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum} m ConversionValueRulePrimaryDimensionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRulePrimaryDimensionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionValueRulePrimaryDimensionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRulePrimaryDimensionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionValueRulePrimaryDimension enum.
                             * @name google.ads.googleads.v11.enums.ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NO_RULE_APPLIED=2 NO_RULE_APPLIED value
                             * @property {number} ORIGINAL=3 ORIGINAL value
                             * @property {number} NEW_VS_RETURNING_USER=4 NEW_VS_RETURNING_USER value
                             * @property {number} GEO_LOCATION=5 GEO_LOCATION value
                             * @property {number} DEVICE=6 DEVICE value
                             * @property {number} AUDIENCE=7 AUDIENCE value
                             * @property {number} MULTIPLE=8 MULTIPLE value
                             */
                            ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NO_RULE_APPLIED"] = 2;
                                values[valuesById[3] = "ORIGINAL"] = 3;
                                values[valuesById[4] = "NEW_VS_RETURNING_USER"] = 4;
                                values[valuesById[5] = "GEO_LOCATION"] = 5;
                                values[valuesById[6] = "DEVICE"] = 6;
                                values[valuesById[7] = "AUDIENCE"] = 7;
                                values[valuesById[8] = "MULTIPLE"] = 8;
                                return values;
                            })();
    
                            return ConversionValueRulePrimaryDimensionEnum;
                        })();
    
                        enums.ExternalConversionSourceEnum = (function() {
    
                            /**
                             * Properties of an ExternalConversionSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IExternalConversionSourceEnum
                             */
    
                            /**
                             * Constructs a new ExternalConversionSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ExternalConversionSourceEnum.
                             * @implements IExternalConversionSourceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IExternalConversionSourceEnum=} [p] Properties to set
                             */
                            function ExternalConversionSourceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExternalConversionSourceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExternalConversionSourceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ExternalConversionSourceEnum} ExternalConversionSourceEnum instance
                             */
                            ExternalConversionSourceEnum.create = function create(properties) {
                                return new ExternalConversionSourceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExternalConversionSourceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ExternalConversionSourceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExternalConversionSourceEnum} m ExternalConversionSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExternalConversionSourceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExternalConversionSourceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ExternalConversionSourceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExternalConversionSourceEnum} message ExternalConversionSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExternalConversionSourceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExternalConversionSourceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ExternalConversionSourceEnum} ExternalConversionSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExternalConversionSourceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ExternalConversionSourceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExternalConversionSourceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ExternalConversionSourceEnum} ExternalConversionSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExternalConversionSourceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExternalConversionSourceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExternalConversionSourceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExternalConversionSourceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ExternalConversionSourceEnum} ExternalConversionSourceEnum
                             */
                            ExternalConversionSourceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ExternalConversionSourceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ExternalConversionSourceEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExternalConversionSourceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ExternalConversionSourceEnum} m ExternalConversionSourceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExternalConversionSourceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExternalConversionSourceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ExternalConversionSourceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExternalConversionSourceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExternalConversionSource enum.
                             * @name google.ads.googleads.v11.enums.ExternalConversionSourceEnum.ExternalConversionSource
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} WEBPAGE=2 WEBPAGE value
                             * @property {number} ANALYTICS=3 ANALYTICS value
                             * @property {number} UPLOAD=4 UPLOAD value
                             * @property {number} AD_CALL_METRICS=5 AD_CALL_METRICS value
                             * @property {number} WEBSITE_CALL_METRICS=6 WEBSITE_CALL_METRICS value
                             * @property {number} STORE_VISITS=7 STORE_VISITS value
                             * @property {number} ANDROID_IN_APP=8 ANDROID_IN_APP value
                             * @property {number} IOS_IN_APP=9 IOS_IN_APP value
                             * @property {number} IOS_FIRST_OPEN=10 IOS_FIRST_OPEN value
                             * @property {number} APP_UNSPECIFIED=11 APP_UNSPECIFIED value
                             * @property {number} ANDROID_FIRST_OPEN=12 ANDROID_FIRST_OPEN value
                             * @property {number} UPLOAD_CALLS=13 UPLOAD_CALLS value
                             * @property {number} FIREBASE=14 FIREBASE value
                             * @property {number} CLICK_TO_CALL=15 CLICK_TO_CALL value
                             * @property {number} SALESFORCE=16 SALESFORCE value
                             * @property {number} STORE_SALES_CRM=17 STORE_SALES_CRM value
                             * @property {number} STORE_SALES_PAYMENT_NETWORK=18 STORE_SALES_PAYMENT_NETWORK value
                             * @property {number} GOOGLE_PLAY=19 GOOGLE_PLAY value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS=20 THIRD_PARTY_APP_ANALYTICS value
                             * @property {number} GOOGLE_ATTRIBUTION=21 GOOGLE_ATTRIBUTION value
                             * @property {number} STORE_SALES_DIRECT_UPLOAD=23 STORE_SALES_DIRECT_UPLOAD value
                             * @property {number} STORE_SALES=24 STORE_SALES value
                             * @property {number} SEARCH_ADS_360=25 SEARCH_ADS_360 value
                             * @property {number} GOOGLE_HOSTED=27 GOOGLE_HOSTED value
                             * @property {number} FLOODLIGHT=29 FLOODLIGHT value
                             * @property {number} ANALYTICS_SEARCH_ADS_360=31 ANALYTICS_SEARCH_ADS_360 value
                             * @property {number} FIREBASE_SEARCH_ADS_360=33 FIREBASE_SEARCH_ADS_360 value
                             * @property {number} DISPLAY_AND_VIDEO_360_FLOODLIGHT=34 DISPLAY_AND_VIDEO_360_FLOODLIGHT value
                             */
                            ExternalConversionSourceEnum.ExternalConversionSource = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "WEBPAGE"] = 2;
                                values[valuesById[3] = "ANALYTICS"] = 3;
                                values[valuesById[4] = "UPLOAD"] = 4;
                                values[valuesById[5] = "AD_CALL_METRICS"] = 5;
                                values[valuesById[6] = "WEBSITE_CALL_METRICS"] = 6;
                                values[valuesById[7] = "STORE_VISITS"] = 7;
                                values[valuesById[8] = "ANDROID_IN_APP"] = 8;
                                values[valuesById[9] = "IOS_IN_APP"] = 9;
                                values[valuesById[10] = "IOS_FIRST_OPEN"] = 10;
                                values[valuesById[11] = "APP_UNSPECIFIED"] = 11;
                                values[valuesById[12] = "ANDROID_FIRST_OPEN"] = 12;
                                values[valuesById[13] = "UPLOAD_CALLS"] = 13;
                                values[valuesById[14] = "FIREBASE"] = 14;
                                values[valuesById[15] = "CLICK_TO_CALL"] = 15;
                                values[valuesById[16] = "SALESFORCE"] = 16;
                                values[valuesById[17] = "STORE_SALES_CRM"] = 17;
                                values[valuesById[18] = "STORE_SALES_PAYMENT_NETWORK"] = 18;
                                values[valuesById[19] = "GOOGLE_PLAY"] = 19;
                                values[valuesById[20] = "THIRD_PARTY_APP_ANALYTICS"] = 20;
                                values[valuesById[21] = "GOOGLE_ATTRIBUTION"] = 21;
                                values[valuesById[23] = "STORE_SALES_DIRECT_UPLOAD"] = 23;
                                values[valuesById[24] = "STORE_SALES"] = 24;
                                values[valuesById[25] = "SEARCH_ADS_360"] = 25;
                                values[valuesById[27] = "GOOGLE_HOSTED"] = 27;
                                values[valuesById[29] = "FLOODLIGHT"] = 29;
                                values[valuesById[31] = "ANALYTICS_SEARCH_ADS_360"] = 31;
                                values[valuesById[33] = "FIREBASE_SEARCH_ADS_360"] = 33;
                                values[valuesById[34] = "DISPLAY_AND_VIDEO_360_FLOODLIGHT"] = 34;
                                return values;
                            })();
    
                            return ExternalConversionSourceEnum;
                        })();
    
                        enums.HotelPriceBucketEnum = (function() {
    
                            /**
                             * Properties of a HotelPriceBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IHotelPriceBucketEnum
                             */
    
                            /**
                             * Constructs a new HotelPriceBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a HotelPriceBucketEnum.
                             * @implements IHotelPriceBucketEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IHotelPriceBucketEnum=} [p] Properties to set
                             */
                            function HotelPriceBucketEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new HotelPriceBucketEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelPriceBucketEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.HotelPriceBucketEnum} HotelPriceBucketEnum instance
                             */
                            HotelPriceBucketEnum.create = function create(properties) {
                                return new HotelPriceBucketEnum(properties);
                            };
    
                            /**
                             * Encodes the specified HotelPriceBucketEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.HotelPriceBucketEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelPriceBucketEnum} m HotelPriceBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelPriceBucketEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelPriceBucketEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.HotelPriceBucketEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelPriceBucketEnum} message HotelPriceBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelPriceBucketEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelPriceBucketEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.HotelPriceBucketEnum} HotelPriceBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelPriceBucketEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.HotelPriceBucketEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelPriceBucketEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.HotelPriceBucketEnum} HotelPriceBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelPriceBucketEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelPriceBucketEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelPriceBucketEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a HotelPriceBucketEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.HotelPriceBucketEnum} HotelPriceBucketEnum
                             */
                            HotelPriceBucketEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.HotelPriceBucketEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.HotelPriceBucketEnum();
                            };
    
                            /**
                             * Creates a plain object from a HotelPriceBucketEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.HotelPriceBucketEnum} m HotelPriceBucketEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelPriceBucketEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this HotelPriceBucketEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.HotelPriceBucketEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelPriceBucketEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * HotelPriceBucket enum.
                             * @name google.ads.googleads.v11.enums.HotelPriceBucketEnum.HotelPriceBucket
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LOWEST_UNIQUE=2 LOWEST_UNIQUE value
                             * @property {number} LOWEST_TIED=3 LOWEST_TIED value
                             * @property {number} NOT_LOWEST=4 NOT_LOWEST value
                             * @property {number} ONLY_PARTNER_SHOWN=5 ONLY_PARTNER_SHOWN value
                             */
                            HotelPriceBucketEnum.HotelPriceBucket = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LOWEST_UNIQUE"] = 2;
                                values[valuesById[3] = "LOWEST_TIED"] = 3;
                                values[valuesById[4] = "NOT_LOWEST"] = 4;
                                values[valuesById[5] = "ONLY_PARTNER_SHOWN"] = 5;
                                return values;
                            })();
    
                            return HotelPriceBucketEnum;
                        })();
    
                        enums.HotelRateTypeEnum = (function() {
    
                            /**
                             * Properties of a HotelRateTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IHotelRateTypeEnum
                             */
    
                            /**
                             * Constructs a new HotelRateTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a HotelRateTypeEnum.
                             * @implements IHotelRateTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IHotelRateTypeEnum=} [p] Properties to set
                             */
                            function HotelRateTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new HotelRateTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelRateTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.HotelRateTypeEnum} HotelRateTypeEnum instance
                             */
                            HotelRateTypeEnum.create = function create(properties) {
                                return new HotelRateTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified HotelRateTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.HotelRateTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelRateTypeEnum} m HotelRateTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelRateTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelRateTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.HotelRateTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelRateTypeEnum} message HotelRateTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelRateTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelRateTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.HotelRateTypeEnum} HotelRateTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelRateTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.HotelRateTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelRateTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.HotelRateTypeEnum} HotelRateTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelRateTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelRateTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelRateTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a HotelRateTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.HotelRateTypeEnum} HotelRateTypeEnum
                             */
                            HotelRateTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.HotelRateTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.HotelRateTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a HotelRateTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.HotelRateTypeEnum} m HotelRateTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelRateTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this HotelRateTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.HotelRateTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelRateTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * HotelRateType enum.
                             * @name google.ads.googleads.v11.enums.HotelRateTypeEnum.HotelRateType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNAVAILABLE=2 UNAVAILABLE value
                             * @property {number} PUBLIC_RATE=3 PUBLIC_RATE value
                             * @property {number} QUALIFIED_RATE=4 QUALIFIED_RATE value
                             * @property {number} PRIVATE_RATE=5 PRIVATE_RATE value
                             */
                            HotelRateTypeEnum.HotelRateType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNAVAILABLE"] = 2;
                                values[valuesById[3] = "PUBLIC_RATE"] = 3;
                                values[valuesById[4] = "QUALIFIED_RATE"] = 4;
                                values[valuesById[5] = "PRIVATE_RATE"] = 5;
                                return values;
                            })();
    
                            return HotelRateTypeEnum;
                        })();
    
                        enums.PlaceholderTypeEnum = (function() {
    
                            /**
                             * Properties of a PlaceholderTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPlaceholderTypeEnum
                             */
    
                            /**
                             * Constructs a new PlaceholderTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PlaceholderTypeEnum.
                             * @implements IPlaceholderTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPlaceholderTypeEnum=} [p] Properties to set
                             */
                            function PlaceholderTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PlaceholderTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPlaceholderTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PlaceholderTypeEnum} PlaceholderTypeEnum instance
                             */
                            PlaceholderTypeEnum.create = function create(properties) {
                                return new PlaceholderTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PlaceholderTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PlaceholderTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPlaceholderTypeEnum} m PlaceholderTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlaceholderTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlaceholderTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PlaceholderTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPlaceholderTypeEnum} message PlaceholderTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlaceholderTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlaceholderTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PlaceholderTypeEnum} PlaceholderTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlaceholderTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlaceholderTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PlaceholderTypeEnum} PlaceholderTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlaceholderTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlaceholderTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlaceholderTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PlaceholderTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PlaceholderTypeEnum} PlaceholderTypeEnum
                             */
                            PlaceholderTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PlaceholderTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PlaceholderTypeEnum} m PlaceholderTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlaceholderTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PlaceholderTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PlaceholderTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlaceholderTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PlaceholderType enum.
                             * @name google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SITELINK=2 SITELINK value
                             * @property {number} CALL=3 CALL value
                             * @property {number} APP=4 APP value
                             * @property {number} LOCATION=5 LOCATION value
                             * @property {number} AFFILIATE_LOCATION=6 AFFILIATE_LOCATION value
                             * @property {number} CALLOUT=7 CALLOUT value
                             * @property {number} STRUCTURED_SNIPPET=8 STRUCTURED_SNIPPET value
                             * @property {number} MESSAGE=9 MESSAGE value
                             * @property {number} PRICE=10 PRICE value
                             * @property {number} PROMOTION=11 PROMOTION value
                             * @property {number} AD_CUSTOMIZER=12 AD_CUSTOMIZER value
                             * @property {number} DYNAMIC_EDUCATION=13 DYNAMIC_EDUCATION value
                             * @property {number} DYNAMIC_FLIGHT=14 DYNAMIC_FLIGHT value
                             * @property {number} DYNAMIC_CUSTOM=15 DYNAMIC_CUSTOM value
                             * @property {number} DYNAMIC_HOTEL=16 DYNAMIC_HOTEL value
                             * @property {number} DYNAMIC_REAL_ESTATE=17 DYNAMIC_REAL_ESTATE value
                             * @property {number} DYNAMIC_TRAVEL=18 DYNAMIC_TRAVEL value
                             * @property {number} DYNAMIC_LOCAL=19 DYNAMIC_LOCAL value
                             * @property {number} DYNAMIC_JOB=20 DYNAMIC_JOB value
                             * @property {number} IMAGE=21 IMAGE value
                             */
                            PlaceholderTypeEnum.PlaceholderType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SITELINK"] = 2;
                                values[valuesById[3] = "CALL"] = 3;
                                values[valuesById[4] = "APP"] = 4;
                                values[valuesById[5] = "LOCATION"] = 5;
                                values[valuesById[6] = "AFFILIATE_LOCATION"] = 6;
                                values[valuesById[7] = "CALLOUT"] = 7;
                                values[valuesById[8] = "STRUCTURED_SNIPPET"] = 8;
                                values[valuesById[9] = "MESSAGE"] = 9;
                                values[valuesById[10] = "PRICE"] = 10;
                                values[valuesById[11] = "PROMOTION"] = 11;
                                values[valuesById[12] = "AD_CUSTOMIZER"] = 12;
                                values[valuesById[13] = "DYNAMIC_EDUCATION"] = 13;
                                values[valuesById[14] = "DYNAMIC_FLIGHT"] = 14;
                                values[valuesById[15] = "DYNAMIC_CUSTOM"] = 15;
                                values[valuesById[16] = "DYNAMIC_HOTEL"] = 16;
                                values[valuesById[17] = "DYNAMIC_REAL_ESTATE"] = 17;
                                values[valuesById[18] = "DYNAMIC_TRAVEL"] = 18;
                                values[valuesById[19] = "DYNAMIC_LOCAL"] = 19;
                                values[valuesById[20] = "DYNAMIC_JOB"] = 20;
                                values[valuesById[21] = "IMAGE"] = 21;
                                return values;
                            })();
    
                            return PlaceholderTypeEnum;
                        })();
    
                        enums.RecommendationTypeEnum = (function() {
    
                            /**
                             * Properties of a RecommendationTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IRecommendationTypeEnum
                             */
    
                            /**
                             * Constructs a new RecommendationTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a RecommendationTypeEnum.
                             * @implements IRecommendationTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IRecommendationTypeEnum=} [p] Properties to set
                             */
                            function RecommendationTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new RecommendationTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IRecommendationTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.RecommendationTypeEnum} RecommendationTypeEnum instance
                             */
                            RecommendationTypeEnum.create = function create(properties) {
                                return new RecommendationTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified RecommendationTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.RecommendationTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IRecommendationTypeEnum} m RecommendationTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RecommendationTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RecommendationTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.RecommendationTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IRecommendationTypeEnum} message RecommendationTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RecommendationTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RecommendationTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.RecommendationTypeEnum} RecommendationTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RecommendationTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.RecommendationTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RecommendationTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.RecommendationTypeEnum} RecommendationTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RecommendationTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RecommendationTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RecommendationTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a RecommendationTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.RecommendationTypeEnum} RecommendationTypeEnum
                             */
                            RecommendationTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.RecommendationTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.RecommendationTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a RecommendationTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.RecommendationTypeEnum} m RecommendationTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RecommendationTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this RecommendationTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.RecommendationTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RecommendationTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * RecommendationType enum.
                             * @name google.ads.googleads.v11.enums.RecommendationTypeEnum.RecommendationType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CAMPAIGN_BUDGET=2 CAMPAIGN_BUDGET value
                             * @property {number} KEYWORD=3 KEYWORD value
                             * @property {number} TEXT_AD=4 TEXT_AD value
                             * @property {number} TARGET_CPA_OPT_IN=5 TARGET_CPA_OPT_IN value
                             * @property {number} MAXIMIZE_CONVERSIONS_OPT_IN=6 MAXIMIZE_CONVERSIONS_OPT_IN value
                             * @property {number} ENHANCED_CPC_OPT_IN=7 ENHANCED_CPC_OPT_IN value
                             * @property {number} SEARCH_PARTNERS_OPT_IN=8 SEARCH_PARTNERS_OPT_IN value
                             * @property {number} MAXIMIZE_CLICKS_OPT_IN=9 MAXIMIZE_CLICKS_OPT_IN value
                             * @property {number} OPTIMIZE_AD_ROTATION=10 OPTIMIZE_AD_ROTATION value
                             * @property {number} CALLOUT_EXTENSION=11 CALLOUT_EXTENSION value
                             * @property {number} SITELINK_EXTENSION=12 SITELINK_EXTENSION value
                             * @property {number} CALL_EXTENSION=13 CALL_EXTENSION value
                             * @property {number} KEYWORD_MATCH_TYPE=14 KEYWORD_MATCH_TYPE value
                             * @property {number} MOVE_UNUSED_BUDGET=15 MOVE_UNUSED_BUDGET value
                             * @property {number} FORECASTING_CAMPAIGN_BUDGET=16 FORECASTING_CAMPAIGN_BUDGET value
                             * @property {number} TARGET_ROAS_OPT_IN=17 TARGET_ROAS_OPT_IN value
                             * @property {number} RESPONSIVE_SEARCH_AD=18 RESPONSIVE_SEARCH_AD value
                             * @property {number} MARGINAL_ROI_CAMPAIGN_BUDGET=19 MARGINAL_ROI_CAMPAIGN_BUDGET value
                             * @property {number} USE_BROAD_MATCH_KEYWORD=20 USE_BROAD_MATCH_KEYWORD value
                             * @property {number} RESPONSIVE_SEARCH_AD_ASSET=21 RESPONSIVE_SEARCH_AD_ASSET value
                             * @property {number} UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX=22 UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX value
                             * @property {number} RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH=23 RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH value
                             */
                            RecommendationTypeEnum.RecommendationType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CAMPAIGN_BUDGET"] = 2;
                                values[valuesById[3] = "KEYWORD"] = 3;
                                values[valuesById[4] = "TEXT_AD"] = 4;
                                values[valuesById[5] = "TARGET_CPA_OPT_IN"] = 5;
                                values[valuesById[6] = "MAXIMIZE_CONVERSIONS_OPT_IN"] = 6;
                                values[valuesById[7] = "ENHANCED_CPC_OPT_IN"] = 7;
                                values[valuesById[8] = "SEARCH_PARTNERS_OPT_IN"] = 8;
                                values[valuesById[9] = "MAXIMIZE_CLICKS_OPT_IN"] = 9;
                                values[valuesById[10] = "OPTIMIZE_AD_ROTATION"] = 10;
                                values[valuesById[11] = "CALLOUT_EXTENSION"] = 11;
                                values[valuesById[12] = "SITELINK_EXTENSION"] = 12;
                                values[valuesById[13] = "CALL_EXTENSION"] = 13;
                                values[valuesById[14] = "KEYWORD_MATCH_TYPE"] = 14;
                                values[valuesById[15] = "MOVE_UNUSED_BUDGET"] = 15;
                                values[valuesById[16] = "FORECASTING_CAMPAIGN_BUDGET"] = 16;
                                values[valuesById[17] = "TARGET_ROAS_OPT_IN"] = 17;
                                values[valuesById[18] = "RESPONSIVE_SEARCH_AD"] = 18;
                                values[valuesById[19] = "MARGINAL_ROI_CAMPAIGN_BUDGET"] = 19;
                                values[valuesById[20] = "USE_BROAD_MATCH_KEYWORD"] = 20;
                                values[valuesById[21] = "RESPONSIVE_SEARCH_AD_ASSET"] = 21;
                                values[valuesById[22] = "UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX"] = 22;
                                values[valuesById[23] = "RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH"] = 23;
                                return values;
                            })();
    
                            return RecommendationTypeEnum;
                        })();
    
                        enums.SearchEngineResultsPageTypeEnum = (function() {
    
                            /**
                             * Properties of a SearchEngineResultsPageTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISearchEngineResultsPageTypeEnum
                             */
    
                            /**
                             * Constructs a new SearchEngineResultsPageTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SearchEngineResultsPageTypeEnum.
                             * @implements ISearchEngineResultsPageTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISearchEngineResultsPageTypeEnum=} [p] Properties to set
                             */
                            function SearchEngineResultsPageTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SearchEngineResultsPageTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchEngineResultsPageTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum} SearchEngineResultsPageTypeEnum instance
                             */
                            SearchEngineResultsPageTypeEnum.create = function create(properties) {
                                return new SearchEngineResultsPageTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SearchEngineResultsPageTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchEngineResultsPageTypeEnum} m SearchEngineResultsPageTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchEngineResultsPageTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchEngineResultsPageTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchEngineResultsPageTypeEnum} message SearchEngineResultsPageTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchEngineResultsPageTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchEngineResultsPageTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum} SearchEngineResultsPageTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchEngineResultsPageTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchEngineResultsPageTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum} SearchEngineResultsPageTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchEngineResultsPageTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchEngineResultsPageTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchEngineResultsPageTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SearchEngineResultsPageTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum} SearchEngineResultsPageTypeEnum
                             */
                            SearchEngineResultsPageTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SearchEngineResultsPageTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum} m SearchEngineResultsPageTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchEngineResultsPageTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SearchEngineResultsPageTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchEngineResultsPageTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SearchEngineResultsPageType enum.
                             * @name google.ads.googleads.v11.enums.SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADS_ONLY=2 ADS_ONLY value
                             * @property {number} ORGANIC_ONLY=3 ORGANIC_ONLY value
                             * @property {number} ADS_AND_ORGANIC=4 ADS_AND_ORGANIC value
                             */
                            SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADS_ONLY"] = 2;
                                values[valuesById[3] = "ORGANIC_ONLY"] = 3;
                                values[valuesById[4] = "ADS_AND_ORGANIC"] = 4;
                                return values;
                            })();
    
                            return SearchEngineResultsPageTypeEnum;
                        })();
    
                        enums.SearchTermMatchTypeEnum = (function() {
    
                            /**
                             * Properties of a SearchTermMatchTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISearchTermMatchTypeEnum
                             */
    
                            /**
                             * Constructs a new SearchTermMatchTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SearchTermMatchTypeEnum.
                             * @implements ISearchTermMatchTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISearchTermMatchTypeEnum=} [p] Properties to set
                             */
                            function SearchTermMatchTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SearchTermMatchTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchTermMatchTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SearchTermMatchTypeEnum} SearchTermMatchTypeEnum instance
                             */
                            SearchTermMatchTypeEnum.create = function create(properties) {
                                return new SearchTermMatchTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SearchTermMatchTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SearchTermMatchTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchTermMatchTypeEnum} m SearchTermMatchTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchTermMatchTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchTermMatchTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SearchTermMatchTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchTermMatchTypeEnum} message SearchTermMatchTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchTermMatchTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchTermMatchTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SearchTermMatchTypeEnum} SearchTermMatchTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchTermMatchTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SearchTermMatchTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchTermMatchTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SearchTermMatchTypeEnum} SearchTermMatchTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchTermMatchTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchTermMatchTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchTermMatchTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SearchTermMatchTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SearchTermMatchTypeEnum} SearchTermMatchTypeEnum
                             */
                            SearchTermMatchTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SearchTermMatchTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SearchTermMatchTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SearchTermMatchTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SearchTermMatchTypeEnum} m SearchTermMatchTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchTermMatchTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SearchTermMatchTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SearchTermMatchTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchTermMatchTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SearchTermMatchType enum.
                             * @name google.ads.googleads.v11.enums.SearchTermMatchTypeEnum.SearchTermMatchType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BROAD=2 BROAD value
                             * @property {number} EXACT=3 EXACT value
                             * @property {number} PHRASE=4 PHRASE value
                             * @property {number} NEAR_EXACT=5 NEAR_EXACT value
                             * @property {number} NEAR_PHRASE=6 NEAR_PHRASE value
                             */
                            SearchTermMatchTypeEnum.SearchTermMatchType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BROAD"] = 2;
                                values[valuesById[3] = "EXACT"] = 3;
                                values[valuesById[4] = "PHRASE"] = 4;
                                values[valuesById[5] = "NEAR_EXACT"] = 5;
                                values[valuesById[6] = "NEAR_PHRASE"] = 6;
                                return values;
                            })();
    
                            return SearchTermMatchTypeEnum;
                        })();
    
                        enums.SkAdNetworkAdEventTypeEnum = (function() {
    
                            /**
                             * Properties of a SkAdNetworkAdEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISkAdNetworkAdEventTypeEnum
                             */
    
                            /**
                             * Constructs a new SkAdNetworkAdEventTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SkAdNetworkAdEventTypeEnum.
                             * @implements ISkAdNetworkAdEventTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAdEventTypeEnum=} [p] Properties to set
                             */
                            function SkAdNetworkAdEventTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SkAdNetworkAdEventTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAdEventTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum} SkAdNetworkAdEventTypeEnum instance
                             */
                            SkAdNetworkAdEventTypeEnum.create = function create(properties) {
                                return new SkAdNetworkAdEventTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkAdEventTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAdEventTypeEnum} m SkAdNetworkAdEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkAdEventTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkAdEventTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAdEventTypeEnum} message SkAdNetworkAdEventTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkAdEventTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SkAdNetworkAdEventTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum} SkAdNetworkAdEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkAdEventTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SkAdNetworkAdEventTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum} SkAdNetworkAdEventTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkAdEventTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SkAdNetworkAdEventTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SkAdNetworkAdEventTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SkAdNetworkAdEventTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum} SkAdNetworkAdEventTypeEnum
                             */
                            SkAdNetworkAdEventTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SkAdNetworkAdEventTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum} m SkAdNetworkAdEventTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SkAdNetworkAdEventTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SkAdNetworkAdEventTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SkAdNetworkAdEventTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SkAdNetworkAdEventType enum.
                             * @name google.ads.googleads.v11.enums.SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNAVAILABLE=2 UNAVAILABLE value
                             * @property {number} INTERACTION=3 INTERACTION value
                             * @property {number} VIEW=4 VIEW value
                             */
                            SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNAVAILABLE"] = 2;
                                values[valuesById[3] = "INTERACTION"] = 3;
                                values[valuesById[4] = "VIEW"] = 4;
                                return values;
                            })();
    
                            return SkAdNetworkAdEventTypeEnum;
                        })();
    
                        enums.SkAdNetworkAttributionCreditEnum = (function() {
    
                            /**
                             * Properties of a SkAdNetworkAttributionCreditEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISkAdNetworkAttributionCreditEnum
                             */
    
                            /**
                             * Constructs a new SkAdNetworkAttributionCreditEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SkAdNetworkAttributionCreditEnum.
                             * @implements ISkAdNetworkAttributionCreditEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAttributionCreditEnum=} [p] Properties to set
                             */
                            function SkAdNetworkAttributionCreditEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SkAdNetworkAttributionCreditEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAttributionCreditEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum} SkAdNetworkAttributionCreditEnum instance
                             */
                            SkAdNetworkAttributionCreditEnum.create = function create(properties) {
                                return new SkAdNetworkAttributionCreditEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkAttributionCreditEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAttributionCreditEnum} m SkAdNetworkAttributionCreditEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkAttributionCreditEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkAttributionCreditEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkAttributionCreditEnum} message SkAdNetworkAttributionCreditEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkAttributionCreditEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SkAdNetworkAttributionCreditEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum} SkAdNetworkAttributionCreditEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkAttributionCreditEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SkAdNetworkAttributionCreditEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum} SkAdNetworkAttributionCreditEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkAttributionCreditEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SkAdNetworkAttributionCreditEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SkAdNetworkAttributionCreditEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SkAdNetworkAttributionCreditEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum} SkAdNetworkAttributionCreditEnum
                             */
                            SkAdNetworkAttributionCreditEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum();
                            };
    
                            /**
                             * Creates a plain object from a SkAdNetworkAttributionCreditEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum} m SkAdNetworkAttributionCreditEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SkAdNetworkAttributionCreditEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SkAdNetworkAttributionCreditEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SkAdNetworkAttributionCreditEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SkAdNetworkAttributionCredit enum.
                             * @name google.ads.googleads.v11.enums.SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNAVAILABLE=2 UNAVAILABLE value
                             * @property {number} WON=3 WON value
                             * @property {number} CONTRIBUTED=4 CONTRIBUTED value
                             */
                            SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNAVAILABLE"] = 2;
                                values[valuesById[3] = "WON"] = 3;
                                values[valuesById[4] = "CONTRIBUTED"] = 4;
                                return values;
                            })();
    
                            return SkAdNetworkAttributionCreditEnum;
                        })();
    
                        enums.SkAdNetworkUserTypeEnum = (function() {
    
                            /**
                             * Properties of a SkAdNetworkUserTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISkAdNetworkUserTypeEnum
                             */
    
                            /**
                             * Constructs a new SkAdNetworkUserTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SkAdNetworkUserTypeEnum.
                             * @implements ISkAdNetworkUserTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkUserTypeEnum=} [p] Properties to set
                             */
                            function SkAdNetworkUserTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SkAdNetworkUserTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkUserTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum} SkAdNetworkUserTypeEnum instance
                             */
                            SkAdNetworkUserTypeEnum.create = function create(properties) {
                                return new SkAdNetworkUserTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkUserTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkUserTypeEnum} m SkAdNetworkUserTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkUserTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SkAdNetworkUserTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISkAdNetworkUserTypeEnum} message SkAdNetworkUserTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SkAdNetworkUserTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SkAdNetworkUserTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum} SkAdNetworkUserTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkUserTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SkAdNetworkUserTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum} SkAdNetworkUserTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SkAdNetworkUserTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SkAdNetworkUserTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SkAdNetworkUserTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SkAdNetworkUserTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum} SkAdNetworkUserTypeEnum
                             */
                            SkAdNetworkUserTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SkAdNetworkUserTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum} m SkAdNetworkUserTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SkAdNetworkUserTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SkAdNetworkUserTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SkAdNetworkUserTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SkAdNetworkUserType enum.
                             * @name google.ads.googleads.v11.enums.SkAdNetworkUserTypeEnum.SkAdNetworkUserType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNAVAILABLE=2 UNAVAILABLE value
                             * @property {number} NEW_INSTALLER=3 NEW_INSTALLER value
                             * @property {number} REINSTALLER=4 REINSTALLER value
                             */
                            SkAdNetworkUserTypeEnum.SkAdNetworkUserType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNAVAILABLE"] = 2;
                                values[valuesById[3] = "NEW_INSTALLER"] = 3;
                                values[valuesById[4] = "REINSTALLER"] = 4;
                                return values;
                            })();
    
                            return SkAdNetworkUserTypeEnum;
                        })();
    
                        enums.SlotEnum = (function() {
    
                            /**
                             * Properties of a SlotEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISlotEnum
                             */
    
                            /**
                             * Constructs a new SlotEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SlotEnum.
                             * @implements ISlotEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISlotEnum=} [p] Properties to set
                             */
                            function SlotEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SlotEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISlotEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SlotEnum} SlotEnum instance
                             */
                            SlotEnum.create = function create(properties) {
                                return new SlotEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SlotEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SlotEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISlotEnum} m SlotEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SlotEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SlotEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SlotEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISlotEnum} message SlotEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SlotEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SlotEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SlotEnum} SlotEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SlotEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SlotEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SlotEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SlotEnum} SlotEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SlotEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SlotEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SlotEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SlotEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SlotEnum} SlotEnum
                             */
                            SlotEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SlotEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SlotEnum();
                            };
    
                            /**
                             * Creates a plain object from a SlotEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SlotEnum} m SlotEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SlotEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SlotEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SlotEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SlotEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Slot enum.
                             * @name google.ads.googleads.v11.enums.SlotEnum.Slot
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SEARCH_SIDE=2 SEARCH_SIDE value
                             * @property {number} SEARCH_TOP=3 SEARCH_TOP value
                             * @property {number} SEARCH_OTHER=4 SEARCH_OTHER value
                             * @property {number} CONTENT=5 CONTENT value
                             * @property {number} SEARCH_PARTNER_TOP=6 SEARCH_PARTNER_TOP value
                             * @property {number} SEARCH_PARTNER_OTHER=7 SEARCH_PARTNER_OTHER value
                             * @property {number} MIXED=8 MIXED value
                             */
                            SlotEnum.Slot = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SEARCH_SIDE"] = 2;
                                values[valuesById[3] = "SEARCH_TOP"] = 3;
                                values[valuesById[4] = "SEARCH_OTHER"] = 4;
                                values[valuesById[5] = "CONTENT"] = 5;
                                values[valuesById[6] = "SEARCH_PARTNER_TOP"] = 6;
                                values[valuesById[7] = "SEARCH_PARTNER_OTHER"] = 7;
                                values[valuesById[8] = "MIXED"] = 8;
                                return values;
                            })();
    
                            return SlotEnum;
                        })();
    
                        enums.TrackingCodePageFormatEnum = (function() {
    
                            /**
                             * Properties of a TrackingCodePageFormatEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ITrackingCodePageFormatEnum
                             */
    
                            /**
                             * Constructs a new TrackingCodePageFormatEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a TrackingCodePageFormatEnum.
                             * @implements ITrackingCodePageFormatEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ITrackingCodePageFormatEnum=} [p] Properties to set
                             */
                            function TrackingCodePageFormatEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TrackingCodePageFormatEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITrackingCodePageFormatEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.TrackingCodePageFormatEnum} TrackingCodePageFormatEnum instance
                             */
                            TrackingCodePageFormatEnum.create = function create(properties) {
                                return new TrackingCodePageFormatEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TrackingCodePageFormatEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.TrackingCodePageFormatEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITrackingCodePageFormatEnum} m TrackingCodePageFormatEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TrackingCodePageFormatEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TrackingCodePageFormatEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.TrackingCodePageFormatEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITrackingCodePageFormatEnum} message TrackingCodePageFormatEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TrackingCodePageFormatEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TrackingCodePageFormatEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.TrackingCodePageFormatEnum} TrackingCodePageFormatEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TrackingCodePageFormatEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.TrackingCodePageFormatEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TrackingCodePageFormatEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.TrackingCodePageFormatEnum} TrackingCodePageFormatEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TrackingCodePageFormatEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TrackingCodePageFormatEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TrackingCodePageFormatEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TrackingCodePageFormatEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.TrackingCodePageFormatEnum} TrackingCodePageFormatEnum
                             */
                            TrackingCodePageFormatEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.TrackingCodePageFormatEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.TrackingCodePageFormatEnum();
                            };
    
                            /**
                             * Creates a plain object from a TrackingCodePageFormatEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.TrackingCodePageFormatEnum} m TrackingCodePageFormatEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TrackingCodePageFormatEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TrackingCodePageFormatEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.TrackingCodePageFormatEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TrackingCodePageFormatEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TrackingCodePageFormat enum.
                             * @name google.ads.googleads.v11.enums.TrackingCodePageFormatEnum.TrackingCodePageFormat
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} HTML=2 HTML value
                             * @property {number} AMP=3 AMP value
                             */
                            TrackingCodePageFormatEnum.TrackingCodePageFormat = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "HTML"] = 2;
                                values[valuesById[3] = "AMP"] = 3;
                                return values;
                            })();
    
                            return TrackingCodePageFormatEnum;
                        })();
    
                        enums.TrackingCodeTypeEnum = (function() {
    
                            /**
                             * Properties of a TrackingCodeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ITrackingCodeTypeEnum
                             */
    
                            /**
                             * Constructs a new TrackingCodeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a TrackingCodeTypeEnum.
                             * @implements ITrackingCodeTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ITrackingCodeTypeEnum=} [p] Properties to set
                             */
                            function TrackingCodeTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TrackingCodeTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITrackingCodeTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.TrackingCodeTypeEnum} TrackingCodeTypeEnum instance
                             */
                            TrackingCodeTypeEnum.create = function create(properties) {
                                return new TrackingCodeTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TrackingCodeTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.TrackingCodeTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITrackingCodeTypeEnum} m TrackingCodeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TrackingCodeTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TrackingCodeTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.TrackingCodeTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITrackingCodeTypeEnum} message TrackingCodeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TrackingCodeTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TrackingCodeTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.TrackingCodeTypeEnum} TrackingCodeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TrackingCodeTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.TrackingCodeTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TrackingCodeTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.TrackingCodeTypeEnum} TrackingCodeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TrackingCodeTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TrackingCodeTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TrackingCodeTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TrackingCodeTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.TrackingCodeTypeEnum} TrackingCodeTypeEnum
                             */
                            TrackingCodeTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.TrackingCodeTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.TrackingCodeTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a TrackingCodeTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.TrackingCodeTypeEnum} m TrackingCodeTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TrackingCodeTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TrackingCodeTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.TrackingCodeTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TrackingCodeTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TrackingCodeType enum.
                             * @name google.ads.googleads.v11.enums.TrackingCodeTypeEnum.TrackingCodeType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} WEBPAGE=2 WEBPAGE value
                             * @property {number} WEBPAGE_ONCLICK=3 WEBPAGE_ONCLICK value
                             * @property {number} CLICK_TO_CALL=4 CLICK_TO_CALL value
                             * @property {number} WEBSITE_CALL=5 WEBSITE_CALL value
                             */
                            TrackingCodeTypeEnum.TrackingCodeType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "WEBPAGE"] = 2;
                                values[valuesById[3] = "WEBPAGE_ONCLICK"] = 3;
                                values[valuesById[4] = "CLICK_TO_CALL"] = 4;
                                values[valuesById[5] = "WEBSITE_CALL"] = 5;
                                return values;
                            })();
    
                            return TrackingCodeTypeEnum;
                        })();
    
                        enums.TargetingDimensionEnum = (function() {
    
                            /**
                             * Properties of a TargetingDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ITargetingDimensionEnum
                             */
    
                            /**
                             * Constructs a new TargetingDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a TargetingDimensionEnum.
                             * @implements ITargetingDimensionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ITargetingDimensionEnum=} [p] Properties to set
                             */
                            function TargetingDimensionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TargetingDimensionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetingDimensionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.TargetingDimensionEnum} TargetingDimensionEnum instance
                             */
                            TargetingDimensionEnum.create = function create(properties) {
                                return new TargetingDimensionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TargetingDimensionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.TargetingDimensionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetingDimensionEnum} m TargetingDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetingDimensionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetingDimensionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.TargetingDimensionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetingDimensionEnum} message TargetingDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetingDimensionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetingDimensionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.TargetingDimensionEnum} TargetingDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetingDimensionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.TargetingDimensionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetingDimensionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.TargetingDimensionEnum} TargetingDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetingDimensionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetingDimensionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetingDimensionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TargetingDimensionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.TargetingDimensionEnum} TargetingDimensionEnum
                             */
                            TargetingDimensionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.TargetingDimensionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.TargetingDimensionEnum();
                            };
    
                            /**
                             * Creates a plain object from a TargetingDimensionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.TargetingDimensionEnum} m TargetingDimensionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetingDimensionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TargetingDimensionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.TargetingDimensionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetingDimensionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TargetingDimension enum.
                             * @name google.ads.googleads.v11.enums.TargetingDimensionEnum.TargetingDimension
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} KEYWORD=2 KEYWORD value
                             * @property {number} AUDIENCE=3 AUDIENCE value
                             * @property {number} TOPIC=4 TOPIC value
                             * @property {number} GENDER=5 GENDER value
                             * @property {number} AGE_RANGE=6 AGE_RANGE value
                             * @property {number} PLACEMENT=7 PLACEMENT value
                             * @property {number} PARENTAL_STATUS=8 PARENTAL_STATUS value
                             * @property {number} INCOME_RANGE=9 INCOME_RANGE value
                             */
                            TargetingDimensionEnum.TargetingDimension = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "KEYWORD"] = 2;
                                values[valuesById[3] = "AUDIENCE"] = 3;
                                values[valuesById[4] = "TOPIC"] = 4;
                                values[valuesById[5] = "GENDER"] = 5;
                                values[valuesById[6] = "AGE_RANGE"] = 6;
                                values[valuesById[7] = "PLACEMENT"] = 7;
                                values[valuesById[8] = "PARENTAL_STATUS"] = 8;
                                values[valuesById[9] = "INCOME_RANGE"] = 9;
                                return values;
                            })();
    
                            return TargetingDimensionEnum;
                        })();
    
                        enums.CustomerMatchUploadKeyTypeEnum = (function() {
    
                            /**
                             * Properties of a CustomerMatchUploadKeyTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomerMatchUploadKeyTypeEnum
                             */
    
                            /**
                             * Constructs a new CustomerMatchUploadKeyTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomerMatchUploadKeyTypeEnum.
                             * @implements ICustomerMatchUploadKeyTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomerMatchUploadKeyTypeEnum=} [p] Properties to set
                             */
                            function CustomerMatchUploadKeyTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerMatchUploadKeyTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerMatchUploadKeyTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum} CustomerMatchUploadKeyTypeEnum instance
                             */
                            CustomerMatchUploadKeyTypeEnum.create = function create(properties) {
                                return new CustomerMatchUploadKeyTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerMatchUploadKeyTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerMatchUploadKeyTypeEnum} m CustomerMatchUploadKeyTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerMatchUploadKeyTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerMatchUploadKeyTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerMatchUploadKeyTypeEnum} message CustomerMatchUploadKeyTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerMatchUploadKeyTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerMatchUploadKeyTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum} CustomerMatchUploadKeyTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerMatchUploadKeyTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerMatchUploadKeyTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum} CustomerMatchUploadKeyTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerMatchUploadKeyTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerMatchUploadKeyTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerMatchUploadKeyTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerMatchUploadKeyTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum} CustomerMatchUploadKeyTypeEnum
                             */
                            CustomerMatchUploadKeyTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerMatchUploadKeyTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum} m CustomerMatchUploadKeyTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerMatchUploadKeyTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerMatchUploadKeyTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerMatchUploadKeyTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerMatchUploadKeyType enum.
                             * @name google.ads.googleads.v11.enums.CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CONTACT_INFO=2 CONTACT_INFO value
                             * @property {number} CRM_ID=3 CRM_ID value
                             * @property {number} MOBILE_ADVERTISING_ID=4 MOBILE_ADVERTISING_ID value
                             */
                            CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CONTACT_INFO"] = 2;
                                values[valuesById[3] = "CRM_ID"] = 3;
                                values[valuesById[4] = "MOBILE_ADVERTISING_ID"] = 4;
                                return values;
                            })();
    
                            return CustomerMatchUploadKeyTypeEnum;
                        })();
    
                        enums.UserListCombinedRuleOperatorEnum = (function() {
    
                            /**
                             * Properties of a UserListCombinedRuleOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListCombinedRuleOperatorEnum
                             */
    
                            /**
                             * Constructs a new UserListCombinedRuleOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListCombinedRuleOperatorEnum.
                             * @implements IUserListCombinedRuleOperatorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListCombinedRuleOperatorEnum=} [p] Properties to set
                             */
                            function UserListCombinedRuleOperatorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListCombinedRuleOperatorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListCombinedRuleOperatorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum} UserListCombinedRuleOperatorEnum instance
                             */
                            UserListCombinedRuleOperatorEnum.create = function create(properties) {
                                return new UserListCombinedRuleOperatorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListCombinedRuleOperatorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListCombinedRuleOperatorEnum} m UserListCombinedRuleOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListCombinedRuleOperatorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListCombinedRuleOperatorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListCombinedRuleOperatorEnum} message UserListCombinedRuleOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListCombinedRuleOperatorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListCombinedRuleOperatorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum} UserListCombinedRuleOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListCombinedRuleOperatorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListCombinedRuleOperatorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum} UserListCombinedRuleOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListCombinedRuleOperatorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListCombinedRuleOperatorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListCombinedRuleOperatorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListCombinedRuleOperatorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum} UserListCombinedRuleOperatorEnum
                             */
                            UserListCombinedRuleOperatorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListCombinedRuleOperatorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum} m UserListCombinedRuleOperatorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListCombinedRuleOperatorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListCombinedRuleOperatorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListCombinedRuleOperatorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListCombinedRuleOperator enum.
                             * @name google.ads.googleads.v11.enums.UserListCombinedRuleOperatorEnum.UserListCombinedRuleOperator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AND=2 AND value
                             * @property {number} AND_NOT=3 AND_NOT value
                             */
                            UserListCombinedRuleOperatorEnum.UserListCombinedRuleOperator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AND"] = 2;
                                values[valuesById[3] = "AND_NOT"] = 3;
                                return values;
                            })();
    
                            return UserListCombinedRuleOperatorEnum;
                        })();
    
                        enums.UserListCrmDataSourceTypeEnum = (function() {
    
                            /**
                             * Properties of a UserListCrmDataSourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListCrmDataSourceTypeEnum
                             */
    
                            /**
                             * Constructs a new UserListCrmDataSourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListCrmDataSourceTypeEnum.
                             * @implements IUserListCrmDataSourceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListCrmDataSourceTypeEnum=} [p] Properties to set
                             */
                            function UserListCrmDataSourceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListCrmDataSourceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListCrmDataSourceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum} UserListCrmDataSourceTypeEnum instance
                             */
                            UserListCrmDataSourceTypeEnum.create = function create(properties) {
                                return new UserListCrmDataSourceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListCrmDataSourceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListCrmDataSourceTypeEnum} m UserListCrmDataSourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListCrmDataSourceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListCrmDataSourceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListCrmDataSourceTypeEnum} message UserListCrmDataSourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListCrmDataSourceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListCrmDataSourceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum} UserListCrmDataSourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListCrmDataSourceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListCrmDataSourceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum} UserListCrmDataSourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListCrmDataSourceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListCrmDataSourceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListCrmDataSourceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListCrmDataSourceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum} UserListCrmDataSourceTypeEnum
                             */
                            UserListCrmDataSourceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListCrmDataSourceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum} m UserListCrmDataSourceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListCrmDataSourceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListCrmDataSourceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListCrmDataSourceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListCrmDataSourceType enum.
                             * @name google.ads.googleads.v11.enums.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FIRST_PARTY=2 FIRST_PARTY value
                             * @property {number} THIRD_PARTY_CREDIT_BUREAU=3 THIRD_PARTY_CREDIT_BUREAU value
                             * @property {number} THIRD_PARTY_VOTER_FILE=4 THIRD_PARTY_VOTER_FILE value
                             */
                            UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FIRST_PARTY"] = 2;
                                values[valuesById[3] = "THIRD_PARTY_CREDIT_BUREAU"] = 3;
                                values[valuesById[4] = "THIRD_PARTY_VOTER_FILE"] = 4;
                                return values;
                            })();
    
                            return UserListCrmDataSourceTypeEnum;
                        })();
    
                        enums.UserListDateRuleItemOperatorEnum = (function() {
    
                            /**
                             * Properties of a UserListDateRuleItemOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListDateRuleItemOperatorEnum
                             */
    
                            /**
                             * Constructs a new UserListDateRuleItemOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListDateRuleItemOperatorEnum.
                             * @implements IUserListDateRuleItemOperatorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListDateRuleItemOperatorEnum=} [p] Properties to set
                             */
                            function UserListDateRuleItemOperatorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListDateRuleItemOperatorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListDateRuleItemOperatorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum} UserListDateRuleItemOperatorEnum instance
                             */
                            UserListDateRuleItemOperatorEnum.create = function create(properties) {
                                return new UserListDateRuleItemOperatorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListDateRuleItemOperatorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListDateRuleItemOperatorEnum} m UserListDateRuleItemOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListDateRuleItemOperatorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListDateRuleItemOperatorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListDateRuleItemOperatorEnum} message UserListDateRuleItemOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListDateRuleItemOperatorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListDateRuleItemOperatorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum} UserListDateRuleItemOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListDateRuleItemOperatorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListDateRuleItemOperatorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum} UserListDateRuleItemOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListDateRuleItemOperatorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListDateRuleItemOperatorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListDateRuleItemOperatorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListDateRuleItemOperatorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum} UserListDateRuleItemOperatorEnum
                             */
                            UserListDateRuleItemOperatorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListDateRuleItemOperatorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum} m UserListDateRuleItemOperatorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListDateRuleItemOperatorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListDateRuleItemOperatorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListDateRuleItemOperatorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListDateRuleItemOperator enum.
                             * @name google.ads.googleads.v11.enums.UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EQUALS=2 EQUALS value
                             * @property {number} NOT_EQUALS=3 NOT_EQUALS value
                             * @property {number} BEFORE=4 BEFORE value
                             * @property {number} AFTER=5 AFTER value
                             */
                            UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EQUALS"] = 2;
                                values[valuesById[3] = "NOT_EQUALS"] = 3;
                                values[valuesById[4] = "BEFORE"] = 4;
                                values[valuesById[5] = "AFTER"] = 5;
                                return values;
                            })();
    
                            return UserListDateRuleItemOperatorEnum;
                        })();
    
                        enums.UserListLogicalRuleOperatorEnum = (function() {
    
                            /**
                             * Properties of a UserListLogicalRuleOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListLogicalRuleOperatorEnum
                             */
    
                            /**
                             * Constructs a new UserListLogicalRuleOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListLogicalRuleOperatorEnum.
                             * @implements IUserListLogicalRuleOperatorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListLogicalRuleOperatorEnum=} [p] Properties to set
                             */
                            function UserListLogicalRuleOperatorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListLogicalRuleOperatorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListLogicalRuleOperatorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum} UserListLogicalRuleOperatorEnum instance
                             */
                            UserListLogicalRuleOperatorEnum.create = function create(properties) {
                                return new UserListLogicalRuleOperatorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListLogicalRuleOperatorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListLogicalRuleOperatorEnum} m UserListLogicalRuleOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListLogicalRuleOperatorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListLogicalRuleOperatorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListLogicalRuleOperatorEnum} message UserListLogicalRuleOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListLogicalRuleOperatorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListLogicalRuleOperatorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum} UserListLogicalRuleOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListLogicalRuleOperatorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListLogicalRuleOperatorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum} UserListLogicalRuleOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListLogicalRuleOperatorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListLogicalRuleOperatorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListLogicalRuleOperatorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListLogicalRuleOperatorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum} UserListLogicalRuleOperatorEnum
                             */
                            UserListLogicalRuleOperatorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListLogicalRuleOperatorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum} m UserListLogicalRuleOperatorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListLogicalRuleOperatorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListLogicalRuleOperatorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListLogicalRuleOperatorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListLogicalRuleOperator enum.
                             * @name google.ads.googleads.v11.enums.UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ALL=2 ALL value
                             * @property {number} ANY=3 ANY value
                             * @property {number} NONE=4 NONE value
                             */
                            UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ALL"] = 2;
                                values[valuesById[3] = "ANY"] = 3;
                                values[valuesById[4] = "NONE"] = 4;
                                return values;
                            })();
    
                            return UserListLogicalRuleOperatorEnum;
                        })();
    
                        enums.UserListNumberRuleItemOperatorEnum = (function() {
    
                            /**
                             * Properties of a UserListNumberRuleItemOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListNumberRuleItemOperatorEnum
                             */
    
                            /**
                             * Constructs a new UserListNumberRuleItemOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListNumberRuleItemOperatorEnum.
                             * @implements IUserListNumberRuleItemOperatorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListNumberRuleItemOperatorEnum=} [p] Properties to set
                             */
                            function UserListNumberRuleItemOperatorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListNumberRuleItemOperatorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListNumberRuleItemOperatorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum} UserListNumberRuleItemOperatorEnum instance
                             */
                            UserListNumberRuleItemOperatorEnum.create = function create(properties) {
                                return new UserListNumberRuleItemOperatorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListNumberRuleItemOperatorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListNumberRuleItemOperatorEnum} m UserListNumberRuleItemOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListNumberRuleItemOperatorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListNumberRuleItemOperatorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListNumberRuleItemOperatorEnum} message UserListNumberRuleItemOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListNumberRuleItemOperatorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListNumberRuleItemOperatorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum} UserListNumberRuleItemOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListNumberRuleItemOperatorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListNumberRuleItemOperatorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum} UserListNumberRuleItemOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListNumberRuleItemOperatorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListNumberRuleItemOperatorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListNumberRuleItemOperatorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListNumberRuleItemOperatorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum} UserListNumberRuleItemOperatorEnum
                             */
                            UserListNumberRuleItemOperatorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListNumberRuleItemOperatorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum} m UserListNumberRuleItemOperatorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListNumberRuleItemOperatorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListNumberRuleItemOperatorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListNumberRuleItemOperatorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListNumberRuleItemOperator enum.
                             * @name google.ads.googleads.v11.enums.UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} GREATER_THAN=2 GREATER_THAN value
                             * @property {number} GREATER_THAN_OR_EQUAL=3 GREATER_THAN_OR_EQUAL value
                             * @property {number} EQUALS=4 EQUALS value
                             * @property {number} NOT_EQUALS=5 NOT_EQUALS value
                             * @property {number} LESS_THAN=6 LESS_THAN value
                             * @property {number} LESS_THAN_OR_EQUAL=7 LESS_THAN_OR_EQUAL value
                             */
                            UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "GREATER_THAN"] = 2;
                                values[valuesById[3] = "GREATER_THAN_OR_EQUAL"] = 3;
                                values[valuesById[4] = "EQUALS"] = 4;
                                values[valuesById[5] = "NOT_EQUALS"] = 5;
                                values[valuesById[6] = "LESS_THAN"] = 6;
                                values[valuesById[7] = "LESS_THAN_OR_EQUAL"] = 7;
                                return values;
                            })();
    
                            return UserListNumberRuleItemOperatorEnum;
                        })();
    
                        enums.UserListPrepopulationStatusEnum = (function() {
    
                            /**
                             * Properties of a UserListPrepopulationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListPrepopulationStatusEnum
                             */
    
                            /**
                             * Constructs a new UserListPrepopulationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListPrepopulationStatusEnum.
                             * @implements IUserListPrepopulationStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListPrepopulationStatusEnum=} [p] Properties to set
                             */
                            function UserListPrepopulationStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListPrepopulationStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListPrepopulationStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum} UserListPrepopulationStatusEnum instance
                             */
                            UserListPrepopulationStatusEnum.create = function create(properties) {
                                return new UserListPrepopulationStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListPrepopulationStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListPrepopulationStatusEnum} m UserListPrepopulationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListPrepopulationStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListPrepopulationStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListPrepopulationStatusEnum} message UserListPrepopulationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListPrepopulationStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListPrepopulationStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum} UserListPrepopulationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListPrepopulationStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListPrepopulationStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum} UserListPrepopulationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListPrepopulationStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListPrepopulationStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListPrepopulationStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListPrepopulationStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum} UserListPrepopulationStatusEnum
                             */
                            UserListPrepopulationStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListPrepopulationStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum} m UserListPrepopulationStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListPrepopulationStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListPrepopulationStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListPrepopulationStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListPrepopulationStatus enum.
                             * @name google.ads.googleads.v11.enums.UserListPrepopulationStatusEnum.UserListPrepopulationStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} REQUESTED=2 REQUESTED value
                             * @property {number} FINISHED=3 FINISHED value
                             * @property {number} FAILED=4 FAILED value
                             */
                            UserListPrepopulationStatusEnum.UserListPrepopulationStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "REQUESTED"] = 2;
                                values[valuesById[3] = "FINISHED"] = 3;
                                values[valuesById[4] = "FAILED"] = 4;
                                return values;
                            })();
    
                            return UserListPrepopulationStatusEnum;
                        })();
    
                        enums.UserListRuleTypeEnum = (function() {
    
                            /**
                             * Properties of a UserListRuleTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListRuleTypeEnum
                             */
    
                            /**
                             * Constructs a new UserListRuleTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListRuleTypeEnum.
                             * @implements IUserListRuleTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListRuleTypeEnum=} [p] Properties to set
                             */
                            function UserListRuleTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListRuleTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListRuleTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListRuleTypeEnum} UserListRuleTypeEnum instance
                             */
                            UserListRuleTypeEnum.create = function create(properties) {
                                return new UserListRuleTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListRuleTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListRuleTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListRuleTypeEnum} m UserListRuleTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListRuleTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListRuleTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListRuleTypeEnum} message UserListRuleTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListRuleTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListRuleTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListRuleTypeEnum} UserListRuleTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListRuleTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListRuleTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListRuleTypeEnum} UserListRuleTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListRuleTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListRuleTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListRuleTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListRuleTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListRuleTypeEnum} UserListRuleTypeEnum
                             */
                            UserListRuleTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListRuleTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListRuleTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListRuleTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListRuleTypeEnum} m UserListRuleTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListRuleTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListRuleTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListRuleTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListRuleTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListRuleType enum.
                             * @name google.ads.googleads.v11.enums.UserListRuleTypeEnum.UserListRuleType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AND_OF_ORS=2 AND_OF_ORS value
                             * @property {number} OR_OF_ANDS=3 OR_OF_ANDS value
                             */
                            UserListRuleTypeEnum.UserListRuleType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AND_OF_ORS"] = 2;
                                values[valuesById[3] = "OR_OF_ANDS"] = 3;
                                return values;
                            })();
    
                            return UserListRuleTypeEnum;
                        })();
    
                        enums.UserListStringRuleItemOperatorEnum = (function() {
    
                            /**
                             * Properties of a UserListStringRuleItemOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListStringRuleItemOperatorEnum
                             */
    
                            /**
                             * Constructs a new UserListStringRuleItemOperatorEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListStringRuleItemOperatorEnum.
                             * @implements IUserListStringRuleItemOperatorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListStringRuleItemOperatorEnum=} [p] Properties to set
                             */
                            function UserListStringRuleItemOperatorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListStringRuleItemOperatorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListStringRuleItemOperatorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum} UserListStringRuleItemOperatorEnum instance
                             */
                            UserListStringRuleItemOperatorEnum.create = function create(properties) {
                                return new UserListStringRuleItemOperatorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListStringRuleItemOperatorEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListStringRuleItemOperatorEnum} m UserListStringRuleItemOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListStringRuleItemOperatorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListStringRuleItemOperatorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListStringRuleItemOperatorEnum} message UserListStringRuleItemOperatorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListStringRuleItemOperatorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListStringRuleItemOperatorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum} UserListStringRuleItemOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListStringRuleItemOperatorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListStringRuleItemOperatorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum} UserListStringRuleItemOperatorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListStringRuleItemOperatorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListStringRuleItemOperatorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListStringRuleItemOperatorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListStringRuleItemOperatorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum} UserListStringRuleItemOperatorEnum
                             */
                            UserListStringRuleItemOperatorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListStringRuleItemOperatorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum} m UserListStringRuleItemOperatorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListStringRuleItemOperatorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListStringRuleItemOperatorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListStringRuleItemOperatorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListStringRuleItemOperator enum.
                             * @name google.ads.googleads.v11.enums.UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CONTAINS=2 CONTAINS value
                             * @property {number} EQUALS=3 EQUALS value
                             * @property {number} STARTS_WITH=4 STARTS_WITH value
                             * @property {number} ENDS_WITH=5 ENDS_WITH value
                             * @property {number} NOT_EQUALS=6 NOT_EQUALS value
                             * @property {number} NOT_CONTAINS=7 NOT_CONTAINS value
                             * @property {number} NOT_STARTS_WITH=8 NOT_STARTS_WITH value
                             * @property {number} NOT_ENDS_WITH=9 NOT_ENDS_WITH value
                             */
                            UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CONTAINS"] = 2;
                                values[valuesById[3] = "EQUALS"] = 3;
                                values[valuesById[4] = "STARTS_WITH"] = 4;
                                values[valuesById[5] = "ENDS_WITH"] = 5;
                                values[valuesById[6] = "NOT_EQUALS"] = 6;
                                values[valuesById[7] = "NOT_CONTAINS"] = 7;
                                values[valuesById[8] = "NOT_STARTS_WITH"] = 8;
                                values[valuesById[9] = "NOT_ENDS_WITH"] = 9;
                                return values;
                            })();
    
                            return UserListStringRuleItemOperatorEnum;
                        })();
    
                        enums.AccessInvitationStatusEnum = (function() {
    
                            /**
                             * Properties of an AccessInvitationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAccessInvitationStatusEnum
                             */
    
                            /**
                             * Constructs a new AccessInvitationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AccessInvitationStatusEnum.
                             * @implements IAccessInvitationStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAccessInvitationStatusEnum=} [p] Properties to set
                             */
                            function AccessInvitationStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccessInvitationStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessInvitationStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AccessInvitationStatusEnum} AccessInvitationStatusEnum instance
                             */
                            AccessInvitationStatusEnum.create = function create(properties) {
                                return new AccessInvitationStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccessInvitationStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AccessInvitationStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessInvitationStatusEnum} m AccessInvitationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessInvitationStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccessInvitationStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AccessInvitationStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessInvitationStatusEnum} message AccessInvitationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessInvitationStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccessInvitationStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AccessInvitationStatusEnum} AccessInvitationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessInvitationStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AccessInvitationStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccessInvitationStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AccessInvitationStatusEnum} AccessInvitationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessInvitationStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccessInvitationStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccessInvitationStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccessInvitationStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AccessInvitationStatusEnum} AccessInvitationStatusEnum
                             */
                            AccessInvitationStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AccessInvitationStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AccessInvitationStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccessInvitationStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AccessInvitationStatusEnum} m AccessInvitationStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccessInvitationStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccessInvitationStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AccessInvitationStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccessInvitationStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccessInvitationStatus enum.
                             * @name google.ads.googleads.v11.enums.AccessInvitationStatusEnum.AccessInvitationStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} DECLINED=3 DECLINED value
                             * @property {number} EXPIRED=4 EXPIRED value
                             */
                            AccessInvitationStatusEnum.AccessInvitationStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "DECLINED"] = 3;
                                values[valuesById[4] = "EXPIRED"] = 4;
                                return values;
                            })();
    
                            return AccessInvitationStatusEnum;
                        })();
    
                        enums.AccessReasonEnum = (function() {
    
                            /**
                             * Properties of an AccessReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAccessReasonEnum
                             */
    
                            /**
                             * Constructs a new AccessReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AccessReasonEnum.
                             * @implements IAccessReasonEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAccessReasonEnum=} [p] Properties to set
                             */
                            function AccessReasonEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccessReasonEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessReasonEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AccessReasonEnum} AccessReasonEnum instance
                             */
                            AccessReasonEnum.create = function create(properties) {
                                return new AccessReasonEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccessReasonEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AccessReasonEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessReasonEnum} m AccessReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessReasonEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccessReasonEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AccessReasonEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessReasonEnum} message AccessReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessReasonEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccessReasonEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AccessReasonEnum} AccessReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessReasonEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AccessReasonEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccessReasonEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AccessReasonEnum} AccessReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessReasonEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccessReasonEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccessReasonEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccessReasonEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AccessReasonEnum} AccessReasonEnum
                             */
                            AccessReasonEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AccessReasonEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AccessReasonEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccessReasonEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AccessReasonEnum} m AccessReasonEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccessReasonEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccessReasonEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AccessReasonEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccessReasonEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccessReason enum.
                             * @name google.ads.googleads.v11.enums.AccessReasonEnum.AccessReason
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OWNED=2 OWNED value
                             * @property {number} SHARED=3 SHARED value
                             * @property {number} LICENSED=4 LICENSED value
                             * @property {number} SUBSCRIBED=5 SUBSCRIBED value
                             * @property {number} AFFILIATED=6 AFFILIATED value
                             */
                            AccessReasonEnum.AccessReason = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OWNED"] = 2;
                                values[valuesById[3] = "SHARED"] = 3;
                                values[valuesById[4] = "LICENSED"] = 4;
                                values[valuesById[5] = "SUBSCRIBED"] = 5;
                                values[valuesById[6] = "AFFILIATED"] = 6;
                                return values;
                            })();
    
                            return AccessReasonEnum;
                        })();
    
                        enums.AccessRoleEnum = (function() {
    
                            /**
                             * Properties of an AccessRoleEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAccessRoleEnum
                             */
    
                            /**
                             * Constructs a new AccessRoleEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AccessRoleEnum.
                             * @implements IAccessRoleEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAccessRoleEnum=} [p] Properties to set
                             */
                            function AccessRoleEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccessRoleEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessRoleEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AccessRoleEnum} AccessRoleEnum instance
                             */
                            AccessRoleEnum.create = function create(properties) {
                                return new AccessRoleEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccessRoleEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AccessRoleEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessRoleEnum} m AccessRoleEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessRoleEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccessRoleEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AccessRoleEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccessRoleEnum} message AccessRoleEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessRoleEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccessRoleEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AccessRoleEnum} AccessRoleEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessRoleEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AccessRoleEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccessRoleEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AccessRoleEnum} AccessRoleEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessRoleEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccessRoleEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccessRoleEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccessRoleEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AccessRoleEnum} AccessRoleEnum
                             */
                            AccessRoleEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AccessRoleEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AccessRoleEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccessRoleEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AccessRoleEnum} m AccessRoleEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccessRoleEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccessRoleEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AccessRoleEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccessRoleEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccessRole enum.
                             * @name google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADMIN=2 ADMIN value
                             * @property {number} STANDARD=3 STANDARD value
                             * @property {number} READ_ONLY=4 READ_ONLY value
                             * @property {number} EMAIL_ONLY=5 EMAIL_ONLY value
                             */
                            AccessRoleEnum.AccessRole = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADMIN"] = 2;
                                values[valuesById[3] = "STANDARD"] = 3;
                                values[valuesById[4] = "READ_ONLY"] = 4;
                                values[valuesById[5] = "EMAIL_ONLY"] = 5;
                                return values;
                            })();
    
                            return AccessRoleEnum;
                        })();
    
                        enums.AccountBudgetProposalStatusEnum = (function() {
    
                            /**
                             * Properties of an AccountBudgetProposalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAccountBudgetProposalStatusEnum
                             */
    
                            /**
                             * Constructs a new AccountBudgetProposalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AccountBudgetProposalStatusEnum.
                             * @implements IAccountBudgetProposalStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalStatusEnum=} [p] Properties to set
                             */
                            function AccountBudgetProposalStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccountBudgetProposalStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum} AccountBudgetProposalStatusEnum instance
                             */
                            AccountBudgetProposalStatusEnum.create = function create(properties) {
                                return new AccountBudgetProposalStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalStatusEnum} m AccountBudgetProposalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalStatusEnum} message AccountBudgetProposalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum} AccountBudgetProposalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum} AccountBudgetProposalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountBudgetProposalStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountBudgetProposalStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccountBudgetProposalStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum} AccountBudgetProposalStatusEnum
                             */
                            AccountBudgetProposalStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccountBudgetProposalStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum} m AccountBudgetProposalStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountBudgetProposalStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccountBudgetProposalStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountBudgetProposalStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccountBudgetProposalStatus enum.
                             * @name google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} APPROVED_HELD=3 APPROVED_HELD value
                             * @property {number} APPROVED=4 APPROVED value
                             * @property {number} CANCELLED=5 CANCELLED value
                             * @property {number} REJECTED=6 REJECTED value
                             */
                            AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "APPROVED_HELD"] = 3;
                                values[valuesById[4] = "APPROVED"] = 4;
                                values[valuesById[5] = "CANCELLED"] = 5;
                                values[valuesById[6] = "REJECTED"] = 6;
                                return values;
                            })();
    
                            return AccountBudgetProposalStatusEnum;
                        })();
    
                        enums.AccountBudgetProposalTypeEnum = (function() {
    
                            /**
                             * Properties of an AccountBudgetProposalTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAccountBudgetProposalTypeEnum
                             */
    
                            /**
                             * Constructs a new AccountBudgetProposalTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AccountBudgetProposalTypeEnum.
                             * @implements IAccountBudgetProposalTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalTypeEnum=} [p] Properties to set
                             */
                            function AccountBudgetProposalTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccountBudgetProposalTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum} AccountBudgetProposalTypeEnum instance
                             */
                            AccountBudgetProposalTypeEnum.create = function create(properties) {
                                return new AccountBudgetProposalTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalTypeEnum} m AccountBudgetProposalTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetProposalTypeEnum} message AccountBudgetProposalTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum} AccountBudgetProposalTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum} AccountBudgetProposalTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountBudgetProposalTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountBudgetProposalTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccountBudgetProposalTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum} AccountBudgetProposalTypeEnum
                             */
                            AccountBudgetProposalTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccountBudgetProposalTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum} m AccountBudgetProposalTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountBudgetProposalTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccountBudgetProposalTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountBudgetProposalTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccountBudgetProposalType enum.
                             * @name google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.AccountBudgetProposalType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CREATE=2 CREATE value
                             * @property {number} UPDATE=3 UPDATE value
                             * @property {number} END=4 END value
                             * @property {number} REMOVE=5 REMOVE value
                             */
                            AccountBudgetProposalTypeEnum.AccountBudgetProposalType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CREATE"] = 2;
                                values[valuesById[3] = "UPDATE"] = 3;
                                values[valuesById[4] = "END"] = 4;
                                values[valuesById[5] = "REMOVE"] = 5;
                                return values;
                            })();
    
                            return AccountBudgetProposalTypeEnum;
                        })();
    
                        enums.AccountBudgetStatusEnum = (function() {
    
                            /**
                             * Properties of an AccountBudgetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAccountBudgetStatusEnum
                             */
    
                            /**
                             * Constructs a new AccountBudgetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AccountBudgetStatusEnum.
                             * @implements IAccountBudgetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetStatusEnum=} [p] Properties to set
                             */
                            function AccountBudgetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccountBudgetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetStatusEnum} AccountBudgetStatusEnum instance
                             */
                            AccountBudgetStatusEnum.create = function create(properties) {
                                return new AccountBudgetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccountBudgetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AccountBudgetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetStatusEnum} m AccountBudgetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountBudgetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AccountBudgetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountBudgetStatusEnum} message AccountBudgetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountBudgetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetStatusEnum} AccountBudgetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AccountBudgetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountBudgetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetStatusEnum} AccountBudgetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountBudgetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountBudgetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccountBudgetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AccountBudgetStatusEnum} AccountBudgetStatusEnum
                             */
                            AccountBudgetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AccountBudgetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AccountBudgetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccountBudgetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AccountBudgetStatusEnum} m AccountBudgetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountBudgetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccountBudgetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AccountBudgetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountBudgetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccountBudgetStatus enum.
                             * @name google.ads.googleads.v11.enums.AccountBudgetStatusEnum.AccountBudgetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} APPROVED=3 APPROVED value
                             * @property {number} CANCELLED=4 CANCELLED value
                             */
                            AccountBudgetStatusEnum.AccountBudgetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "APPROVED"] = 3;
                                values[valuesById[4] = "CANCELLED"] = 4;
                                return values;
                            })();
    
                            return AccountBudgetStatusEnum;
                        })();
    
                        enums.AccountLinkStatusEnum = (function() {
    
                            /**
                             * Properties of an AccountLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAccountLinkStatusEnum
                             */
    
                            /**
                             * Constructs a new AccountLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AccountLinkStatusEnum.
                             * @implements IAccountLinkStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAccountLinkStatusEnum=} [p] Properties to set
                             */
                            function AccountLinkStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccountLinkStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountLinkStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AccountLinkStatusEnum} AccountLinkStatusEnum instance
                             */
                            AccountLinkStatusEnum.create = function create(properties) {
                                return new AccountLinkStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccountLinkStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AccountLinkStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountLinkStatusEnum} m AccountLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLinkStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountLinkStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AccountLinkStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAccountLinkStatusEnum} message AccountLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLinkStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountLinkStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AccountLinkStatusEnum} AccountLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLinkStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AccountLinkStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountLinkStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AccountLinkStatusEnum} AccountLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLinkStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountLinkStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountLinkStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccountLinkStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AccountLinkStatusEnum} AccountLinkStatusEnum
                             */
                            AccountLinkStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AccountLinkStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AccountLinkStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccountLinkStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AccountLinkStatusEnum} m AccountLinkStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountLinkStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccountLinkStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AccountLinkStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountLinkStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccountLinkStatus enum.
                             * @name google.ads.googleads.v11.enums.AccountLinkStatusEnum.AccountLinkStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             * @property {number} REQUESTED=4 REQUESTED value
                             * @property {number} PENDING_APPROVAL=5 PENDING_APPROVAL value
                             * @property {number} REJECTED=6 REJECTED value
                             * @property {number} REVOKED=7 REVOKED value
                             */
                            AccountLinkStatusEnum.AccountLinkStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                values[valuesById[4] = "REQUESTED"] = 4;
                                values[valuesById[5] = "PENDING_APPROVAL"] = 5;
                                values[valuesById[6] = "REJECTED"] = 6;
                                values[valuesById[7] = "REVOKED"] = 7;
                                return values;
                            })();
    
                            return AccountLinkStatusEnum;
                        })();
    
                        enums.AdCustomizerPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of an AdCustomizerPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdCustomizerPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new AdCustomizerPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdCustomizerPlaceholderFieldEnum.
                             * @implements IAdCustomizerPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdCustomizerPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function AdCustomizerPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdCustomizerPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdCustomizerPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum} AdCustomizerPlaceholderFieldEnum instance
                             */
                            AdCustomizerPlaceholderFieldEnum.create = function create(properties) {
                                return new AdCustomizerPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdCustomizerPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdCustomizerPlaceholderFieldEnum} m AdCustomizerPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdCustomizerPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdCustomizerPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdCustomizerPlaceholderFieldEnum} message AdCustomizerPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdCustomizerPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdCustomizerPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum} AdCustomizerPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdCustomizerPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdCustomizerPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum} AdCustomizerPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdCustomizerPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdCustomizerPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdCustomizerPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdCustomizerPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum} AdCustomizerPlaceholderFieldEnum
                             */
                            AdCustomizerPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdCustomizerPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum} m AdCustomizerPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdCustomizerPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdCustomizerPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdCustomizerPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdCustomizerPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INTEGER=2 INTEGER value
                             * @property {number} PRICE=3 PRICE value
                             * @property {number} DATE=4 DATE value
                             * @property {number} STRING=5 STRING value
                             */
                            AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INTEGER"] = 2;
                                values[valuesById[3] = "PRICE"] = 3;
                                values[valuesById[4] = "DATE"] = 4;
                                values[valuesById[5] = "STRING"] = 5;
                                return values;
                            })();
    
                            return AdCustomizerPlaceholderFieldEnum;
                        })();
    
                        enums.AdGroupAdRotationModeEnum = (function() {
    
                            /**
                             * Properties of an AdGroupAdRotationModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdGroupAdRotationModeEnum
                             */
    
                            /**
                             * Constructs a new AdGroupAdRotationModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdGroupAdRotationModeEnum.
                             * @implements IAdGroupAdRotationModeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdRotationModeEnum=} [p] Properties to set
                             */
                            function AdGroupAdRotationModeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupAdRotationModeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdRotationModeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum} AdGroupAdRotationModeEnum instance
                             */
                            AdGroupAdRotationModeEnum.create = function create(properties) {
                                return new AdGroupAdRotationModeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdRotationModeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdRotationModeEnum} m AdGroupAdRotationModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdRotationModeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdRotationModeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdRotationModeEnum} message AdGroupAdRotationModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdRotationModeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdRotationModeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum} AdGroupAdRotationModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdRotationModeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdRotationModeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum} AdGroupAdRotationModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdRotationModeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdRotationModeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdRotationModeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdRotationModeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum} AdGroupAdRotationModeEnum
                             */
                            AdGroupAdRotationModeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdRotationModeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum} m AdGroupAdRotationModeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdRotationModeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupAdRotationModeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdRotationModeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupAdRotationMode enum.
                             * @name google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum.AdGroupAdRotationMode
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OPTIMIZE=2 OPTIMIZE value
                             * @property {number} ROTATE_FOREVER=3 ROTATE_FOREVER value
                             */
                            AdGroupAdRotationModeEnum.AdGroupAdRotationMode = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OPTIMIZE"] = 2;
                                values[valuesById[3] = "ROTATE_FOREVER"] = 3;
                                return values;
                            })();
    
                            return AdGroupAdRotationModeEnum;
                        })();
    
                        enums.AdGroupAdStatusEnum = (function() {
    
                            /**
                             * Properties of an AdGroupAdStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdGroupAdStatusEnum
                             */
    
                            /**
                             * Constructs a new AdGroupAdStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdGroupAdStatusEnum.
                             * @implements IAdGroupAdStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdStatusEnum=} [p] Properties to set
                             */
                            function AdGroupAdStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupAdStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdStatusEnum} AdGroupAdStatusEnum instance
                             */
                            AdGroupAdStatusEnum.create = function create(properties) {
                                return new AdGroupAdStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupAdStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdStatusEnum} m AdGroupAdStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupAdStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupAdStatusEnum} message AdGroupAdStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdStatusEnum} AdGroupAdStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdGroupAdStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdStatusEnum} AdGroupAdStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdGroupAdStatusEnum} AdGroupAdStatusEnum
                             */
                            AdGroupAdStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdGroupAdStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdGroupAdStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdGroupAdStatusEnum} m AdGroupAdStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupAdStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdGroupAdStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupAdStatus enum.
                             * @name google.ads.googleads.v11.enums.AdGroupAdStatusEnum.AdGroupAdStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} PAUSED=3 PAUSED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            AdGroupAdStatusEnum.AdGroupAdStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "PAUSED"] = 3;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return AdGroupAdStatusEnum;
                        })();
    
                        enums.AdGroupCriterionApprovalStatusEnum = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionApprovalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdGroupCriterionApprovalStatusEnum
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionApprovalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdGroupCriterionApprovalStatusEnum.
                             * @implements IAdGroupCriterionApprovalStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionApprovalStatusEnum=} [p] Properties to set
                             */
                            function AdGroupCriterionApprovalStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupCriterionApprovalStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionApprovalStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum} AdGroupCriterionApprovalStatusEnum instance
                             */
                            AdGroupCriterionApprovalStatusEnum.create = function create(properties) {
                                return new AdGroupCriterionApprovalStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionApprovalStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionApprovalStatusEnum} m AdGroupCriterionApprovalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionApprovalStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionApprovalStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionApprovalStatusEnum} message AdGroupCriterionApprovalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionApprovalStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionApprovalStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum} AdGroupCriterionApprovalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionApprovalStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionApprovalStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum} AdGroupCriterionApprovalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionApprovalStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionApprovalStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionApprovalStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionApprovalStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum} AdGroupCriterionApprovalStatusEnum
                             */
                            AdGroupCriterionApprovalStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionApprovalStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum} m AdGroupCriterionApprovalStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionApprovalStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupCriterionApprovalStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionApprovalStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupCriterionApprovalStatus enum.
                             * @name google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum.AdGroupCriterionApprovalStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APPROVED=2 APPROVED value
                             * @property {number} DISAPPROVED=3 DISAPPROVED value
                             * @property {number} PENDING_REVIEW=4 PENDING_REVIEW value
                             * @property {number} UNDER_REVIEW=5 UNDER_REVIEW value
                             */
                            AdGroupCriterionApprovalStatusEnum.AdGroupCriterionApprovalStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APPROVED"] = 2;
                                values[valuesById[3] = "DISAPPROVED"] = 3;
                                values[valuesById[4] = "PENDING_REVIEW"] = 4;
                                values[valuesById[5] = "UNDER_REVIEW"] = 5;
                                return values;
                            })();
    
                            return AdGroupCriterionApprovalStatusEnum;
                        })();
    
                        enums.AdGroupCriterionStatusEnum = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdGroupCriterionStatusEnum
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdGroupCriterionStatusEnum.
                             * @implements IAdGroupCriterionStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionStatusEnum=} [p] Properties to set
                             */
                            function AdGroupCriterionStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupCriterionStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum} AdGroupCriterionStatusEnum instance
                             */
                            AdGroupCriterionStatusEnum.create = function create(properties) {
                                return new AdGroupCriterionStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionStatusEnum} m AdGroupCriterionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupCriterionStatusEnum} message AdGroupCriterionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum} AdGroupCriterionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum} AdGroupCriterionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum} AdGroupCriterionStatusEnum
                             */
                            AdGroupCriterionStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum} m AdGroupCriterionStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupCriterionStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupCriterionStatus enum.
                             * @name google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum.AdGroupCriterionStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} PAUSED=3 PAUSED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            AdGroupCriterionStatusEnum.AdGroupCriterionStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "PAUSED"] = 3;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return AdGroupCriterionStatusEnum;
                        })();
    
                        enums.AdGroupStatusEnum = (function() {
    
                            /**
                             * Properties of an AdGroupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdGroupStatusEnum
                             */
    
                            /**
                             * Constructs a new AdGroupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdGroupStatusEnum.
                             * @implements IAdGroupStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdGroupStatusEnum=} [p] Properties to set
                             */
                            function AdGroupStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdGroupStatusEnum} AdGroupStatusEnum instance
                             */
                            AdGroupStatusEnum.create = function create(properties) {
                                return new AdGroupStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupStatusEnum} m AdGroupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupStatusEnum} message AdGroupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdGroupStatusEnum} AdGroupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdGroupStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdGroupStatusEnum} AdGroupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdGroupStatusEnum} AdGroupStatusEnum
                             */
                            AdGroupStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdGroupStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdGroupStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdGroupStatusEnum} m AdGroupStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdGroupStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupStatus enum.
                             * @name google.ads.googleads.v11.enums.AdGroupStatusEnum.AdGroupStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} PAUSED=3 PAUSED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            AdGroupStatusEnum.AdGroupStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "PAUSED"] = 3;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return AdGroupStatusEnum;
                        })();
    
                        enums.AdGroupTypeEnum = (function() {
    
                            /**
                             * Properties of an AdGroupTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdGroupTypeEnum
                             */
    
                            /**
                             * Constructs a new AdGroupTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdGroupTypeEnum.
                             * @implements IAdGroupTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdGroupTypeEnum=} [p] Properties to set
                             */
                            function AdGroupTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdGroupTypeEnum} AdGroupTypeEnum instance
                             */
                            AdGroupTypeEnum.create = function create(properties) {
                                return new AdGroupTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupTypeEnum} m AdGroupTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdGroupTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdGroupTypeEnum} message AdGroupTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdGroupTypeEnum} AdGroupTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdGroupTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdGroupTypeEnum} AdGroupTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdGroupTypeEnum} AdGroupTypeEnum
                             */
                            AdGroupTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdGroupTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdGroupTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdGroupTypeEnum} m AdGroupTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdGroupTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupType enum.
                             * @name google.ads.googleads.v11.enums.AdGroupTypeEnum.AdGroupType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SEARCH_STANDARD=2 SEARCH_STANDARD value
                             * @property {number} DISPLAY_STANDARD=3 DISPLAY_STANDARD value
                             * @property {number} SHOPPING_PRODUCT_ADS=4 SHOPPING_PRODUCT_ADS value
                             * @property {number} HOTEL_ADS=6 HOTEL_ADS value
                             * @property {number} SHOPPING_SMART_ADS=7 SHOPPING_SMART_ADS value
                             * @property {number} VIDEO_BUMPER=8 VIDEO_BUMPER value
                             * @property {number} VIDEO_TRUE_VIEW_IN_STREAM=9 VIDEO_TRUE_VIEW_IN_STREAM value
                             * @property {number} VIDEO_TRUE_VIEW_IN_DISPLAY=10 VIDEO_TRUE_VIEW_IN_DISPLAY value
                             * @property {number} VIDEO_NON_SKIPPABLE_IN_STREAM=11 VIDEO_NON_SKIPPABLE_IN_STREAM value
                             * @property {number} VIDEO_OUTSTREAM=12 VIDEO_OUTSTREAM value
                             * @property {number} SEARCH_DYNAMIC_ADS=13 SEARCH_DYNAMIC_ADS value
                             * @property {number} SHOPPING_COMPARISON_LISTING_ADS=14 SHOPPING_COMPARISON_LISTING_ADS value
                             * @property {number} PROMOTED_HOTEL_ADS=15 PROMOTED_HOTEL_ADS value
                             * @property {number} VIDEO_RESPONSIVE=16 VIDEO_RESPONSIVE value
                             * @property {number} VIDEO_EFFICIENT_REACH=17 VIDEO_EFFICIENT_REACH value
                             * @property {number} SMART_CAMPAIGN_ADS=18 SMART_CAMPAIGN_ADS value
                             */
                            AdGroupTypeEnum.AdGroupType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SEARCH_STANDARD"] = 2;
                                values[valuesById[3] = "DISPLAY_STANDARD"] = 3;
                                values[valuesById[4] = "SHOPPING_PRODUCT_ADS"] = 4;
                                values[valuesById[6] = "HOTEL_ADS"] = 6;
                                values[valuesById[7] = "SHOPPING_SMART_ADS"] = 7;
                                values[valuesById[8] = "VIDEO_BUMPER"] = 8;
                                values[valuesById[9] = "VIDEO_TRUE_VIEW_IN_STREAM"] = 9;
                                values[valuesById[10] = "VIDEO_TRUE_VIEW_IN_DISPLAY"] = 10;
                                values[valuesById[11] = "VIDEO_NON_SKIPPABLE_IN_STREAM"] = 11;
                                values[valuesById[12] = "VIDEO_OUTSTREAM"] = 12;
                                values[valuesById[13] = "SEARCH_DYNAMIC_ADS"] = 13;
                                values[valuesById[14] = "SHOPPING_COMPARISON_LISTING_ADS"] = 14;
                                values[valuesById[15] = "PROMOTED_HOTEL_ADS"] = 15;
                                values[valuesById[16] = "VIDEO_RESPONSIVE"] = 16;
                                values[valuesById[17] = "VIDEO_EFFICIENT_REACH"] = 17;
                                values[valuesById[18] = "SMART_CAMPAIGN_ADS"] = 18;
                                return values;
                            })();
    
                            return AdGroupTypeEnum;
                        })();
    
                        enums.AdServingOptimizationStatusEnum = (function() {
    
                            /**
                             * Properties of an AdServingOptimizationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdServingOptimizationStatusEnum
                             */
    
                            /**
                             * Constructs a new AdServingOptimizationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdServingOptimizationStatusEnum.
                             * @implements IAdServingOptimizationStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdServingOptimizationStatusEnum=} [p] Properties to set
                             */
                            function AdServingOptimizationStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdServingOptimizationStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdServingOptimizationStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum} AdServingOptimizationStatusEnum instance
                             */
                            AdServingOptimizationStatusEnum.create = function create(properties) {
                                return new AdServingOptimizationStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdServingOptimizationStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdServingOptimizationStatusEnum} m AdServingOptimizationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdServingOptimizationStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdServingOptimizationStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdServingOptimizationStatusEnum} message AdServingOptimizationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdServingOptimizationStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdServingOptimizationStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum} AdServingOptimizationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdServingOptimizationStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdServingOptimizationStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum} AdServingOptimizationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdServingOptimizationStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdServingOptimizationStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdServingOptimizationStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdServingOptimizationStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum} AdServingOptimizationStatusEnum
                             */
                            AdServingOptimizationStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdServingOptimizationStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum} m AdServingOptimizationStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdServingOptimizationStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdServingOptimizationStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdServingOptimizationStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdServingOptimizationStatus enum.
                             * @name google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum.AdServingOptimizationStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OPTIMIZE=2 OPTIMIZE value
                             * @property {number} CONVERSION_OPTIMIZE=3 CONVERSION_OPTIMIZE value
                             * @property {number} ROTATE=4 ROTATE value
                             * @property {number} ROTATE_INDEFINITELY=5 ROTATE_INDEFINITELY value
                             * @property {number} UNAVAILABLE=6 UNAVAILABLE value
                             */
                            AdServingOptimizationStatusEnum.AdServingOptimizationStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OPTIMIZE"] = 2;
                                values[valuesById[3] = "CONVERSION_OPTIMIZE"] = 3;
                                values[valuesById[4] = "ROTATE"] = 4;
                                values[valuesById[5] = "ROTATE_INDEFINITELY"] = 5;
                                values[valuesById[6] = "UNAVAILABLE"] = 6;
                                return values;
                            })();
    
                            return AdServingOptimizationStatusEnum;
                        })();
    
                        enums.AdStrengthEnum = (function() {
    
                            /**
                             * Properties of an AdStrengthEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdStrengthEnum
                             */
    
                            /**
                             * Constructs a new AdStrengthEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdStrengthEnum.
                             * @implements IAdStrengthEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdStrengthEnum=} [p] Properties to set
                             */
                            function AdStrengthEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdStrengthEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdStrengthEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdStrengthEnum} AdStrengthEnum instance
                             */
                            AdStrengthEnum.create = function create(properties) {
                                return new AdStrengthEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdStrengthEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdStrengthEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdStrengthEnum} m AdStrengthEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdStrengthEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdStrengthEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdStrengthEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdStrengthEnum} message AdStrengthEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdStrengthEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdStrengthEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdStrengthEnum} AdStrengthEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdStrengthEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdStrengthEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdStrengthEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdStrengthEnum} AdStrengthEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdStrengthEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdStrengthEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdStrengthEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdStrengthEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdStrengthEnum} AdStrengthEnum
                             */
                            AdStrengthEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdStrengthEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdStrengthEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdStrengthEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdStrengthEnum} m AdStrengthEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdStrengthEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdStrengthEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdStrengthEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdStrengthEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdStrength enum.
                             * @name google.ads.googleads.v11.enums.AdStrengthEnum.AdStrength
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} NO_ADS=3 NO_ADS value
                             * @property {number} POOR=4 POOR value
                             * @property {number} AVERAGE=5 AVERAGE value
                             * @property {number} GOOD=6 GOOD value
                             * @property {number} EXCELLENT=7 EXCELLENT value
                             */
                            AdStrengthEnum.AdStrength = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "NO_ADS"] = 3;
                                values[valuesById[4] = "POOR"] = 4;
                                values[valuesById[5] = "AVERAGE"] = 5;
                                values[valuesById[6] = "GOOD"] = 6;
                                values[valuesById[7] = "EXCELLENT"] = 7;
                                return values;
                            })();
    
                            return AdStrengthEnum;
                        })();
    
                        enums.AdTypeEnum = (function() {
    
                            /**
                             * Properties of an AdTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAdTypeEnum
                             */
    
                            /**
                             * Constructs a new AdTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AdTypeEnum.
                             * @implements IAdTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAdTypeEnum=} [p] Properties to set
                             */
                            function AdTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AdTypeEnum} AdTypeEnum instance
                             */
                            AdTypeEnum.create = function create(properties) {
                                return new AdTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AdTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdTypeEnum} m AdTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AdTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAdTypeEnum} message AdTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AdTypeEnum} AdTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AdTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AdTypeEnum} AdTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AdTypeEnum} AdTypeEnum
                             */
                            AdTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AdTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AdTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AdTypeEnum} m AdTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AdTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdType enum.
                             * @name google.ads.googleads.v11.enums.AdTypeEnum.AdType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TEXT_AD=2 TEXT_AD value
                             * @property {number} EXPANDED_TEXT_AD=3 EXPANDED_TEXT_AD value
                             * @property {number} EXPANDED_DYNAMIC_SEARCH_AD=7 EXPANDED_DYNAMIC_SEARCH_AD value
                             * @property {number} HOTEL_AD=8 HOTEL_AD value
                             * @property {number} SHOPPING_SMART_AD=9 SHOPPING_SMART_AD value
                             * @property {number} SHOPPING_PRODUCT_AD=10 SHOPPING_PRODUCT_AD value
                             * @property {number} VIDEO_AD=12 VIDEO_AD value
                             * @property {number} GMAIL_AD=13 GMAIL_AD value
                             * @property {number} IMAGE_AD=14 IMAGE_AD value
                             * @property {number} RESPONSIVE_SEARCH_AD=15 RESPONSIVE_SEARCH_AD value
                             * @property {number} LEGACY_RESPONSIVE_DISPLAY_AD=16 LEGACY_RESPONSIVE_DISPLAY_AD value
                             * @property {number} APP_AD=17 APP_AD value
                             * @property {number} LEGACY_APP_INSTALL_AD=18 LEGACY_APP_INSTALL_AD value
                             * @property {number} RESPONSIVE_DISPLAY_AD=19 RESPONSIVE_DISPLAY_AD value
                             * @property {number} LOCAL_AD=20 LOCAL_AD value
                             * @property {number} HTML5_UPLOAD_AD=21 HTML5_UPLOAD_AD value
                             * @property {number} DYNAMIC_HTML5_AD=22 DYNAMIC_HTML5_AD value
                             * @property {number} APP_ENGAGEMENT_AD=23 APP_ENGAGEMENT_AD value
                             * @property {number} SHOPPING_COMPARISON_LISTING_AD=24 SHOPPING_COMPARISON_LISTING_AD value
                             * @property {number} VIDEO_BUMPER_AD=25 VIDEO_BUMPER_AD value
                             * @property {number} VIDEO_NON_SKIPPABLE_IN_STREAM_AD=26 VIDEO_NON_SKIPPABLE_IN_STREAM_AD value
                             * @property {number} VIDEO_OUTSTREAM_AD=27 VIDEO_OUTSTREAM_AD value
                             * @property {number} VIDEO_TRUEVIEW_IN_STREAM_AD=29 VIDEO_TRUEVIEW_IN_STREAM_AD value
                             * @property {number} VIDEO_RESPONSIVE_AD=30 VIDEO_RESPONSIVE_AD value
                             * @property {number} SMART_CAMPAIGN_AD=31 SMART_CAMPAIGN_AD value
                             * @property {number} CALL_AD=32 CALL_AD value
                             * @property {number} APP_PRE_REGISTRATION_AD=33 APP_PRE_REGISTRATION_AD value
                             * @property {number} IN_FEED_VIDEO_AD=34 IN_FEED_VIDEO_AD value
                             * @property {number} DISCOVERY_MULTI_ASSET_AD=35 DISCOVERY_MULTI_ASSET_AD value
                             * @property {number} DISCOVERY_CAROUSEL_AD=36 DISCOVERY_CAROUSEL_AD value
                             */
                            AdTypeEnum.AdType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TEXT_AD"] = 2;
                                values[valuesById[3] = "EXPANDED_TEXT_AD"] = 3;
                                values[valuesById[7] = "EXPANDED_DYNAMIC_SEARCH_AD"] = 7;
                                values[valuesById[8] = "HOTEL_AD"] = 8;
                                values[valuesById[9] = "SHOPPING_SMART_AD"] = 9;
                                values[valuesById[10] = "SHOPPING_PRODUCT_AD"] = 10;
                                values[valuesById[12] = "VIDEO_AD"] = 12;
                                values[valuesById[13] = "GMAIL_AD"] = 13;
                                values[valuesById[14] = "IMAGE_AD"] = 14;
                                values[valuesById[15] = "RESPONSIVE_SEARCH_AD"] = 15;
                                values[valuesById[16] = "LEGACY_RESPONSIVE_DISPLAY_AD"] = 16;
                                values[valuesById[17] = "APP_AD"] = 17;
                                values[valuesById[18] = "LEGACY_APP_INSTALL_AD"] = 18;
                                values[valuesById[19] = "RESPONSIVE_DISPLAY_AD"] = 19;
                                values[valuesById[20] = "LOCAL_AD"] = 20;
                                values[valuesById[21] = "HTML5_UPLOAD_AD"] = 21;
                                values[valuesById[22] = "DYNAMIC_HTML5_AD"] = 22;
                                values[valuesById[23] = "APP_ENGAGEMENT_AD"] = 23;
                                values[valuesById[24] = "SHOPPING_COMPARISON_LISTING_AD"] = 24;
                                values[valuesById[25] = "VIDEO_BUMPER_AD"] = 25;
                                values[valuesById[26] = "VIDEO_NON_SKIPPABLE_IN_STREAM_AD"] = 26;
                                values[valuesById[27] = "VIDEO_OUTSTREAM_AD"] = 27;
                                values[valuesById[29] = "VIDEO_TRUEVIEW_IN_STREAM_AD"] = 29;
                                values[valuesById[30] = "VIDEO_RESPONSIVE_AD"] = 30;
                                values[valuesById[31] = "SMART_CAMPAIGN_AD"] = 31;
                                values[valuesById[32] = "CALL_AD"] = 32;
                                values[valuesById[33] = "APP_PRE_REGISTRATION_AD"] = 33;
                                values[valuesById[34] = "IN_FEED_VIDEO_AD"] = 34;
                                values[valuesById[35] = "DISCOVERY_MULTI_ASSET_AD"] = 35;
                                values[valuesById[36] = "DISCOVERY_CAROUSEL_AD"] = 36;
                                return values;
                            })();
    
                            return AdTypeEnum;
                        })();
    
                        enums.AffiliateLocationFeedRelationshipTypeEnum = (function() {
    
                            /**
                             * Properties of an AffiliateLocationFeedRelationshipTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAffiliateLocationFeedRelationshipTypeEnum
                             */
    
                            /**
                             * Constructs a new AffiliateLocationFeedRelationshipTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AffiliateLocationFeedRelationshipTypeEnum.
                             * @implements IAffiliateLocationFeedRelationshipTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationFeedRelationshipTypeEnum=} [p] Properties to set
                             */
                            function AffiliateLocationFeedRelationshipTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AffiliateLocationFeedRelationshipTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationFeedRelationshipTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum} AffiliateLocationFeedRelationshipTypeEnum instance
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.create = function create(properties) {
                                return new AffiliateLocationFeedRelationshipTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AffiliateLocationFeedRelationshipTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationFeedRelationshipTypeEnum} m AffiliateLocationFeedRelationshipTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AffiliateLocationFeedRelationshipTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationFeedRelationshipTypeEnum} message AffiliateLocationFeedRelationshipTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AffiliateLocationFeedRelationshipTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum} AffiliateLocationFeedRelationshipTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AffiliateLocationFeedRelationshipTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum} AffiliateLocationFeedRelationshipTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AffiliateLocationFeedRelationshipTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AffiliateLocationFeedRelationshipTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum} AffiliateLocationFeedRelationshipTypeEnum
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AffiliateLocationFeedRelationshipTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum} m AffiliateLocationFeedRelationshipTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AffiliateLocationFeedRelationshipTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AffiliateLocationFeedRelationshipType enum.
                             * @name google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum.AffiliateLocationFeedRelationshipType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} GENERAL_RETAILER=2 GENERAL_RETAILER value
                             */
                            AffiliateLocationFeedRelationshipTypeEnum.AffiliateLocationFeedRelationshipType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "GENERAL_RETAILER"] = 2;
                                return values;
                            })();
    
                            return AffiliateLocationFeedRelationshipTypeEnum;
                        })();
    
                        enums.AffiliateLocationPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of an AffiliateLocationPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAffiliateLocationPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new AffiliateLocationPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AffiliateLocationPlaceholderFieldEnum.
                             * @implements IAffiliateLocationPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function AffiliateLocationPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AffiliateLocationPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum} AffiliateLocationPlaceholderFieldEnum instance
                             */
                            AffiliateLocationPlaceholderFieldEnum.create = function create(properties) {
                                return new AffiliateLocationPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AffiliateLocationPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationPlaceholderFieldEnum} m AffiliateLocationPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AffiliateLocationPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AffiliateLocationPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAffiliateLocationPlaceholderFieldEnum} message AffiliateLocationPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AffiliateLocationPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AffiliateLocationPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum} AffiliateLocationPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AffiliateLocationPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AffiliateLocationPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum} AffiliateLocationPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AffiliateLocationPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AffiliateLocationPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AffiliateLocationPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AffiliateLocationPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum} AffiliateLocationPlaceholderFieldEnum
                             */
                            AffiliateLocationPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from an AffiliateLocationPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum} m AffiliateLocationPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AffiliateLocationPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AffiliateLocationPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AffiliateLocationPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AffiliateLocationPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BUSINESS_NAME=2 BUSINESS_NAME value
                             * @property {number} ADDRESS_LINE_1=3 ADDRESS_LINE_1 value
                             * @property {number} ADDRESS_LINE_2=4 ADDRESS_LINE_2 value
                             * @property {number} CITY=5 CITY value
                             * @property {number} PROVINCE=6 PROVINCE value
                             * @property {number} POSTAL_CODE=7 POSTAL_CODE value
                             * @property {number} COUNTRY_CODE=8 COUNTRY_CODE value
                             * @property {number} PHONE_NUMBER=9 PHONE_NUMBER value
                             * @property {number} LANGUAGE_CODE=10 LANGUAGE_CODE value
                             * @property {number} CHAIN_ID=11 CHAIN_ID value
                             * @property {number} CHAIN_NAME=12 CHAIN_NAME value
                             */
                            AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BUSINESS_NAME"] = 2;
                                values[valuesById[3] = "ADDRESS_LINE_1"] = 3;
                                values[valuesById[4] = "ADDRESS_LINE_2"] = 4;
                                values[valuesById[5] = "CITY"] = 5;
                                values[valuesById[6] = "PROVINCE"] = 6;
                                values[valuesById[7] = "POSTAL_CODE"] = 7;
                                values[valuesById[8] = "COUNTRY_CODE"] = 8;
                                values[valuesById[9] = "PHONE_NUMBER"] = 9;
                                values[valuesById[10] = "LANGUAGE_CODE"] = 10;
                                values[valuesById[11] = "CHAIN_ID"] = 11;
                                values[valuesById[12] = "CHAIN_NAME"] = 12;
                                return values;
                            })();
    
                            return AffiliateLocationPlaceholderFieldEnum;
                        })();
    
                        enums.AppCampaignAppStoreEnum = (function() {
    
                            /**
                             * Properties of an AppCampaignAppStoreEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAppCampaignAppStoreEnum
                             */
    
                            /**
                             * Constructs a new AppCampaignAppStoreEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AppCampaignAppStoreEnum.
                             * @implements IAppCampaignAppStoreEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAppCampaignAppStoreEnum=} [p] Properties to set
                             */
                            function AppCampaignAppStoreEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AppCampaignAppStoreEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppCampaignAppStoreEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AppCampaignAppStoreEnum} AppCampaignAppStoreEnum instance
                             */
                            AppCampaignAppStoreEnum.create = function create(properties) {
                                return new AppCampaignAppStoreEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AppCampaignAppStoreEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AppCampaignAppStoreEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppCampaignAppStoreEnum} m AppCampaignAppStoreEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppCampaignAppStoreEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppCampaignAppStoreEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AppCampaignAppStoreEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppCampaignAppStoreEnum} message AppCampaignAppStoreEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppCampaignAppStoreEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppCampaignAppStoreEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AppCampaignAppStoreEnum} AppCampaignAppStoreEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppCampaignAppStoreEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AppCampaignAppStoreEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppCampaignAppStoreEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AppCampaignAppStoreEnum} AppCampaignAppStoreEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppCampaignAppStoreEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppCampaignAppStoreEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppCampaignAppStoreEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AppCampaignAppStoreEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AppCampaignAppStoreEnum} AppCampaignAppStoreEnum
                             */
                            AppCampaignAppStoreEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AppCampaignAppStoreEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AppCampaignAppStoreEnum();
                            };
    
                            /**
                             * Creates a plain object from an AppCampaignAppStoreEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AppCampaignAppStoreEnum} m AppCampaignAppStoreEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppCampaignAppStoreEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AppCampaignAppStoreEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AppCampaignAppStoreEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppCampaignAppStoreEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AppCampaignAppStore enum.
                             * @name google.ads.googleads.v11.enums.AppCampaignAppStoreEnum.AppCampaignAppStore
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APPLE_APP_STORE=2 APPLE_APP_STORE value
                             * @property {number} GOOGLE_APP_STORE=3 GOOGLE_APP_STORE value
                             */
                            AppCampaignAppStoreEnum.AppCampaignAppStore = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APPLE_APP_STORE"] = 2;
                                values[valuesById[3] = "GOOGLE_APP_STORE"] = 3;
                                return values;
                            })();
    
                            return AppCampaignAppStoreEnum;
                        })();
    
                        enums.AppCampaignBiddingStrategyGoalTypeEnum = (function() {
    
                            /**
                             * Properties of an AppCampaignBiddingStrategyGoalTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAppCampaignBiddingStrategyGoalTypeEnum
                             */
    
                            /**
                             * Constructs a new AppCampaignBiddingStrategyGoalTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AppCampaignBiddingStrategyGoalTypeEnum.
                             * @implements IAppCampaignBiddingStrategyGoalTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAppCampaignBiddingStrategyGoalTypeEnum=} [p] Properties to set
                             */
                            function AppCampaignBiddingStrategyGoalTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AppCampaignBiddingStrategyGoalTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppCampaignBiddingStrategyGoalTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum} AppCampaignBiddingStrategyGoalTypeEnum instance
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.create = function create(properties) {
                                return new AppCampaignBiddingStrategyGoalTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AppCampaignBiddingStrategyGoalTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppCampaignBiddingStrategyGoalTypeEnum} m AppCampaignBiddingStrategyGoalTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppCampaignBiddingStrategyGoalTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppCampaignBiddingStrategyGoalTypeEnum} message AppCampaignBiddingStrategyGoalTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppCampaignBiddingStrategyGoalTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum} AppCampaignBiddingStrategyGoalTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppCampaignBiddingStrategyGoalTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum} AppCampaignBiddingStrategyGoalTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppCampaignBiddingStrategyGoalTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AppCampaignBiddingStrategyGoalTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum} AppCampaignBiddingStrategyGoalTypeEnum
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AppCampaignBiddingStrategyGoalTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum} m AppCampaignBiddingStrategyGoalTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AppCampaignBiddingStrategyGoalTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AppCampaignBiddingStrategyGoalType enum.
                             * @name google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OPTIMIZE_INSTALLS_TARGET_INSTALL_COST=2 OPTIMIZE_INSTALLS_TARGET_INSTALL_COST value
                             * @property {number} OPTIMIZE_IN_APP_CONVERSIONS_TARGET_INSTALL_COST=3 OPTIMIZE_IN_APP_CONVERSIONS_TARGET_INSTALL_COST value
                             * @property {number} OPTIMIZE_IN_APP_CONVERSIONS_TARGET_CONVERSION_COST=4 OPTIMIZE_IN_APP_CONVERSIONS_TARGET_CONVERSION_COST value
                             * @property {number} OPTIMIZE_RETURN_ON_ADVERTISING_SPEND=5 OPTIMIZE_RETURN_ON_ADVERTISING_SPEND value
                             * @property {number} OPTIMIZE_PRE_REGISTRATION_CONVERSION_VOLUME=6 OPTIMIZE_PRE_REGISTRATION_CONVERSION_VOLUME value
                             * @property {number} OPTIMIZE_INSTALLS_WITHOUT_TARGET_INSTALL_COST=7 OPTIMIZE_INSTALLS_WITHOUT_TARGET_INSTALL_COST value
                             */
                            AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OPTIMIZE_INSTALLS_TARGET_INSTALL_COST"] = 2;
                                values[valuesById[3] = "OPTIMIZE_IN_APP_CONVERSIONS_TARGET_INSTALL_COST"] = 3;
                                values[valuesById[4] = "OPTIMIZE_IN_APP_CONVERSIONS_TARGET_CONVERSION_COST"] = 4;
                                values[valuesById[5] = "OPTIMIZE_RETURN_ON_ADVERTISING_SPEND"] = 5;
                                values[valuesById[6] = "OPTIMIZE_PRE_REGISTRATION_CONVERSION_VOLUME"] = 6;
                                values[valuesById[7] = "OPTIMIZE_INSTALLS_WITHOUT_TARGET_INSTALL_COST"] = 7;
                                return values;
                            })();
    
                            return AppCampaignBiddingStrategyGoalTypeEnum;
                        })();
    
                        enums.AppPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of an AppPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAppPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new AppPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AppPlaceholderFieldEnum.
                             * @implements IAppPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAppPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function AppPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AppPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AppPlaceholderFieldEnum} AppPlaceholderFieldEnum instance
                             */
                            AppPlaceholderFieldEnum.create = function create(properties) {
                                return new AppPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AppPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AppPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppPlaceholderFieldEnum} m AppPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AppPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AppPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAppPlaceholderFieldEnum} message AppPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AppPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AppPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AppPlaceholderFieldEnum} AppPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AppPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AppPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AppPlaceholderFieldEnum} AppPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AppPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AppPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AppPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AppPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AppPlaceholderFieldEnum} AppPlaceholderFieldEnum
                             */
                            AppPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AppPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AppPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from an AppPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AppPlaceholderFieldEnum} m AppPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AppPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AppPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AppPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AppPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AppPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.AppPlaceholderFieldEnum.AppPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} STORE=2 STORE value
                             * @property {number} ID=3 ID value
                             * @property {number} LINK_TEXT=4 LINK_TEXT value
                             * @property {number} URL=5 URL value
                             * @property {number} FINAL_URLS=6 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=7 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=8 TRACKING_URL value
                             * @property {number} FINAL_URL_SUFFIX=9 FINAL_URL_SUFFIX value
                             */
                            AppPlaceholderFieldEnum.AppPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "STORE"] = 2;
                                values[valuesById[3] = "ID"] = 3;
                                values[valuesById[4] = "LINK_TEXT"] = 4;
                                values[valuesById[5] = "URL"] = 5;
                                values[valuesById[6] = "FINAL_URLS"] = 6;
                                values[valuesById[7] = "FINAL_MOBILE_URLS"] = 7;
                                values[valuesById[8] = "TRACKING_URL"] = 8;
                                values[valuesById[9] = "FINAL_URL_SUFFIX"] = 9;
                                return values;
                            })();
    
                            return AppPlaceholderFieldEnum;
                        })();
    
                        enums.AssetFieldTypeEnum = (function() {
    
                            /**
                             * Properties of an AssetFieldTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetFieldTypeEnum
                             */
    
                            /**
                             * Constructs a new AssetFieldTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetFieldTypeEnum.
                             * @implements IAssetFieldTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetFieldTypeEnum=} [p] Properties to set
                             */
                            function AssetFieldTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetFieldTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetFieldTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetFieldTypeEnum} AssetFieldTypeEnum instance
                             */
                            AssetFieldTypeEnum.create = function create(properties) {
                                return new AssetFieldTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetFieldTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetFieldTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetFieldTypeEnum} m AssetFieldTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetFieldTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetFieldTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetFieldTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetFieldTypeEnum} message AssetFieldTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetFieldTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetFieldTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetFieldTypeEnum} AssetFieldTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetFieldTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetFieldTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetFieldTypeEnum} AssetFieldTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetFieldTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetFieldTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetFieldTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetFieldTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetFieldTypeEnum} AssetFieldTypeEnum
                             */
                            AssetFieldTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetFieldTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetFieldTypeEnum} m AssetFieldTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetFieldTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetFieldTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetFieldTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetFieldTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetFieldType enum.
                             * @name google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} HEADLINE=2 HEADLINE value
                             * @property {number} DESCRIPTION=3 DESCRIPTION value
                             * @property {number} MANDATORY_AD_TEXT=4 MANDATORY_AD_TEXT value
                             * @property {number} MARKETING_IMAGE=5 MARKETING_IMAGE value
                             * @property {number} MEDIA_BUNDLE=6 MEDIA_BUNDLE value
                             * @property {number} YOUTUBE_VIDEO=7 YOUTUBE_VIDEO value
                             * @property {number} BOOK_ON_GOOGLE=8 BOOK_ON_GOOGLE value
                             * @property {number} LEAD_FORM=9 LEAD_FORM value
                             * @property {number} PROMOTION=10 PROMOTION value
                             * @property {number} CALLOUT=11 CALLOUT value
                             * @property {number} STRUCTURED_SNIPPET=12 STRUCTURED_SNIPPET value
                             * @property {number} SITELINK=13 SITELINK value
                             * @property {number} MOBILE_APP=14 MOBILE_APP value
                             * @property {number} HOTEL_CALLOUT=15 HOTEL_CALLOUT value
                             * @property {number} CALL=16 CALL value
                             * @property {number} PRICE=24 PRICE value
                             * @property {number} LONG_HEADLINE=17 LONG_HEADLINE value
                             * @property {number} BUSINESS_NAME=18 BUSINESS_NAME value
                             * @property {number} SQUARE_MARKETING_IMAGE=19 SQUARE_MARKETING_IMAGE value
                             * @property {number} PORTRAIT_MARKETING_IMAGE=20 PORTRAIT_MARKETING_IMAGE value
                             * @property {number} LOGO=21 LOGO value
                             * @property {number} LANDSCAPE_LOGO=22 LANDSCAPE_LOGO value
                             * @property {number} VIDEO=23 VIDEO value
                             * @property {number} CALL_TO_ACTION_SELECTION=25 CALL_TO_ACTION_SELECTION value
                             */
                            AssetFieldTypeEnum.AssetFieldType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "HEADLINE"] = 2;
                                values[valuesById[3] = "DESCRIPTION"] = 3;
                                values[valuesById[4] = "MANDATORY_AD_TEXT"] = 4;
                                values[valuesById[5] = "MARKETING_IMAGE"] = 5;
                                values[valuesById[6] = "MEDIA_BUNDLE"] = 6;
                                values[valuesById[7] = "YOUTUBE_VIDEO"] = 7;
                                values[valuesById[8] = "BOOK_ON_GOOGLE"] = 8;
                                values[valuesById[9] = "LEAD_FORM"] = 9;
                                values[valuesById[10] = "PROMOTION"] = 10;
                                values[valuesById[11] = "CALLOUT"] = 11;
                                values[valuesById[12] = "STRUCTURED_SNIPPET"] = 12;
                                values[valuesById[13] = "SITELINK"] = 13;
                                values[valuesById[14] = "MOBILE_APP"] = 14;
                                values[valuesById[15] = "HOTEL_CALLOUT"] = 15;
                                values[valuesById[16] = "CALL"] = 16;
                                values[valuesById[24] = "PRICE"] = 24;
                                values[valuesById[17] = "LONG_HEADLINE"] = 17;
                                values[valuesById[18] = "BUSINESS_NAME"] = 18;
                                values[valuesById[19] = "SQUARE_MARKETING_IMAGE"] = 19;
                                values[valuesById[20] = "PORTRAIT_MARKETING_IMAGE"] = 20;
                                values[valuesById[21] = "LOGO"] = 21;
                                values[valuesById[22] = "LANDSCAPE_LOGO"] = 22;
                                values[valuesById[23] = "VIDEO"] = 23;
                                values[valuesById[25] = "CALL_TO_ACTION_SELECTION"] = 25;
                                return values;
                            })();
    
                            return AssetFieldTypeEnum;
                        })();
    
                        enums.AssetGroupStatusEnum = (function() {
    
                            /**
                             * Properties of an AssetGroupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetGroupStatusEnum
                             */
    
                            /**
                             * Constructs a new AssetGroupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetGroupStatusEnum.
                             * @implements IAssetGroupStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetGroupStatusEnum=} [p] Properties to set
                             */
                            function AssetGroupStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetGroupStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetGroupStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetGroupStatusEnum} AssetGroupStatusEnum instance
                             */
                            AssetGroupStatusEnum.create = function create(properties) {
                                return new AssetGroupStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetGroupStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetGroupStatusEnum} m AssetGroupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetGroupStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetGroupStatusEnum} message AssetGroupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetGroupStatusEnum} AssetGroupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetGroupStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetGroupStatusEnum} AssetGroupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetGroupStatusEnum} AssetGroupStatusEnum
                             */
                            AssetGroupStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetGroupStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetGroupStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetGroupStatusEnum} m AssetGroupStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetGroupStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetGroupStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetGroupStatus enum.
                             * @name google.ads.googleads.v11.enums.AssetGroupStatusEnum.AssetGroupStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} PAUSED=3 PAUSED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            AssetGroupStatusEnum.AssetGroupStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "PAUSED"] = 3;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return AssetGroupStatusEnum;
                        })();
    
                        enums.AssetLinkStatusEnum = (function() {
    
                            /**
                             * Properties of an AssetLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetLinkStatusEnum
                             */
    
                            /**
                             * Constructs a new AssetLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetLinkStatusEnum.
                             * @implements IAssetLinkStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetLinkStatusEnum=} [p] Properties to set
                             */
                            function AssetLinkStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetLinkStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetLinkStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetLinkStatusEnum} AssetLinkStatusEnum instance
                             */
                            AssetLinkStatusEnum.create = function create(properties) {
                                return new AssetLinkStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetLinkStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetLinkStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetLinkStatusEnum} m AssetLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetLinkStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetLinkStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetLinkStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetLinkStatusEnum} message AssetLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetLinkStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetLinkStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetLinkStatusEnum} AssetLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetLinkStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetLinkStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetLinkStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetLinkStatusEnum} AssetLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetLinkStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetLinkStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetLinkStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetLinkStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetLinkStatusEnum} AssetLinkStatusEnum
                             */
                            AssetLinkStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetLinkStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetLinkStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetLinkStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetLinkStatusEnum} m AssetLinkStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetLinkStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetLinkStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetLinkStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetLinkStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetLinkStatus enum.
                             * @name google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             * @property {number} PAUSED=4 PAUSED value
                             */
                            AssetLinkStatusEnum.AssetLinkStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                values[valuesById[4] = "PAUSED"] = 4;
                                return values;
                            })();
    
                            return AssetLinkStatusEnum;
                        })();
    
                        enums.AssetSetAssetStatusEnum = (function() {
    
                            /**
                             * Properties of an AssetSetAssetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetSetAssetStatusEnum
                             */
    
                            /**
                             * Constructs a new AssetSetAssetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetSetAssetStatusEnum.
                             * @implements IAssetSetAssetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetSetAssetStatusEnum=} [p] Properties to set
                             */
                            function AssetSetAssetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSetAssetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetAssetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetSetAssetStatusEnum} AssetSetAssetStatusEnum instance
                             */
                            AssetSetAssetStatusEnum.create = function create(properties) {
                                return new AssetSetAssetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetAssetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetAssetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetAssetStatusEnum} m AssetSetAssetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAssetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetAssetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetAssetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetAssetStatusEnum} message AssetSetAssetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAssetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetAssetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetSetAssetStatusEnum} AssetSetAssetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAssetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetSetAssetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetAssetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetSetAssetStatusEnum} AssetSetAssetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAssetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetAssetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetAssetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetAssetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetSetAssetStatusEnum} AssetSetAssetStatusEnum
                             */
                            AssetSetAssetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetSetAssetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetSetAssetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSetAssetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetSetAssetStatusEnum} m AssetSetAssetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetAssetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSetAssetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetSetAssetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetAssetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSetAssetStatus enum.
                             * @name google.ads.googleads.v11.enums.AssetSetAssetStatusEnum.AssetSetAssetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            AssetSetAssetStatusEnum.AssetSetAssetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return AssetSetAssetStatusEnum;
                        })();
    
                        enums.AssetSetLinkStatusEnum = (function() {
    
                            /**
                             * Properties of an AssetSetLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetSetLinkStatusEnum
                             */
    
                            /**
                             * Constructs a new AssetSetLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetSetLinkStatusEnum.
                             * @implements IAssetSetLinkStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetSetLinkStatusEnum=} [p] Properties to set
                             */
                            function AssetSetLinkStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSetLinkStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetLinkStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetSetLinkStatusEnum} AssetSetLinkStatusEnum instance
                             */
                            AssetSetLinkStatusEnum.create = function create(properties) {
                                return new AssetSetLinkStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetLinkStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetLinkStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetLinkStatusEnum} m AssetSetLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetLinkStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetLinkStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetLinkStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetLinkStatusEnum} message AssetSetLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetLinkStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetLinkStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetSetLinkStatusEnum} AssetSetLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetLinkStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetSetLinkStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetLinkStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetSetLinkStatusEnum} AssetSetLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetLinkStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetLinkStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetLinkStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetLinkStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetSetLinkStatusEnum} AssetSetLinkStatusEnum
                             */
                            AssetSetLinkStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetSetLinkStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetSetLinkStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSetLinkStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetSetLinkStatusEnum} m AssetSetLinkStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetLinkStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSetLinkStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetSetLinkStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetLinkStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSetLinkStatus enum.
                             * @name google.ads.googleads.v11.enums.AssetSetLinkStatusEnum.AssetSetLinkStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            AssetSetLinkStatusEnum.AssetSetLinkStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return AssetSetLinkStatusEnum;
                        })();
    
                        enums.AssetSetStatusEnum = (function() {
    
                            /**
                             * Properties of an AssetSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetSetStatusEnum
                             */
    
                            /**
                             * Constructs a new AssetSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetSetStatusEnum.
                             * @implements IAssetSetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetSetStatusEnum=} [p] Properties to set
                             */
                            function AssetSetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetSetStatusEnum} AssetSetStatusEnum instance
                             */
                            AssetSetStatusEnum.create = function create(properties) {
                                return new AssetSetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetStatusEnum} m AssetSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetStatusEnum} message AssetSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetSetStatusEnum} AssetSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetSetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetSetStatusEnum} AssetSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetSetStatusEnum} AssetSetStatusEnum
                             */
                            AssetSetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetSetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetSetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetSetStatusEnum} m AssetSetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetSetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSetStatus enum.
                             * @name google.ads.googleads.v11.enums.AssetSetStatusEnum.AssetSetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            AssetSetStatusEnum.AssetSetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return AssetSetStatusEnum;
                        })();
    
                        enums.AssetSetTypeEnum = (function() {
    
                            /**
                             * Properties of an AssetSetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetSetTypeEnum
                             */
    
                            /**
                             * Constructs a new AssetSetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetSetTypeEnum.
                             * @implements IAssetSetTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetSetTypeEnum=} [p] Properties to set
                             */
                            function AssetSetTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSetTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetSetTypeEnum} AssetSetTypeEnum instance
                             */
                            AssetSetTypeEnum.create = function create(properties) {
                                return new AssetSetTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetTypeEnum} m AssetSetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSetTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSetTypeEnum} message AssetSetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetSetTypeEnum} AssetSetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetSetTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetSetTypeEnum} AssetSetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetSetTypeEnum} AssetSetTypeEnum
                             */
                            AssetSetTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetSetTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetSetTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSetTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetSetTypeEnum} m AssetSetTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSetTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetSetTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSetType enum.
                             * @name google.ads.googleads.v11.enums.AssetSetTypeEnum.AssetSetType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PAGE_FEED=2 PAGE_FEED value
                             * @property {number} DYNAMIC_EDUCATION=3 DYNAMIC_EDUCATION value
                             * @property {number} MERCHANT_CENTER_FEED=4 MERCHANT_CENTER_FEED value
                             * @property {number} DYNAMIC_REAL_ESTATE=5 DYNAMIC_REAL_ESTATE value
                             * @property {number} DYNAMIC_CUSTOM=6 DYNAMIC_CUSTOM value
                             * @property {number} DYNAMIC_HOTELS_AND_RENTALS=7 DYNAMIC_HOTELS_AND_RENTALS value
                             * @property {number} DYNAMIC_FLIGHTS=8 DYNAMIC_FLIGHTS value
                             * @property {number} DYNAMIC_TRAVEL=9 DYNAMIC_TRAVEL value
                             * @property {number} DYNAMIC_LOCAL=10 DYNAMIC_LOCAL value
                             * @property {number} DYNAMIC_JOBS=11 DYNAMIC_JOBS value
                             */
                            AssetSetTypeEnum.AssetSetType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PAGE_FEED"] = 2;
                                values[valuesById[3] = "DYNAMIC_EDUCATION"] = 3;
                                values[valuesById[4] = "MERCHANT_CENTER_FEED"] = 4;
                                values[valuesById[5] = "DYNAMIC_REAL_ESTATE"] = 5;
                                values[valuesById[6] = "DYNAMIC_CUSTOM"] = 6;
                                values[valuesById[7] = "DYNAMIC_HOTELS_AND_RENTALS"] = 7;
                                values[valuesById[8] = "DYNAMIC_FLIGHTS"] = 8;
                                values[valuesById[9] = "DYNAMIC_TRAVEL"] = 9;
                                values[valuesById[10] = "DYNAMIC_LOCAL"] = 10;
                                values[valuesById[11] = "DYNAMIC_JOBS"] = 11;
                                return values;
                            })();
    
                            return AssetSetTypeEnum;
                        })();
    
                        enums.AssetSourceEnum = (function() {
    
                            /**
                             * Properties of an AssetSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetSourceEnum
                             */
    
                            /**
                             * Constructs a new AssetSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetSourceEnum.
                             * @implements IAssetSourceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetSourceEnum=} [p] Properties to set
                             */
                            function AssetSourceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSourceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSourceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetSourceEnum} AssetSourceEnum instance
                             */
                            AssetSourceEnum.create = function create(properties) {
                                return new AssetSourceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSourceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSourceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSourceEnum} m AssetSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSourceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSourceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetSourceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetSourceEnum} message AssetSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSourceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSourceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetSourceEnum} AssetSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSourceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetSourceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSourceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetSourceEnum} AssetSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSourceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSourceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSourceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSourceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetSourceEnum} AssetSourceEnum
                             */
                            AssetSourceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetSourceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetSourceEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSourceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetSourceEnum} m AssetSourceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSourceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSourceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetSourceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSourceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSource enum.
                             * @name google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADVERTISER=2 ADVERTISER value
                             * @property {number} AUTOMATICALLY_CREATED=3 AUTOMATICALLY_CREATED value
                             */
                            AssetSourceEnum.AssetSource = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADVERTISER"] = 2;
                                values[valuesById[3] = "AUTOMATICALLY_CREATED"] = 3;
                                return values;
                            })();
    
                            return AssetSourceEnum;
                        })();
    
                        enums.AssetTypeEnum = (function() {
    
                            /**
                             * Properties of an AssetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAssetTypeEnum
                             */
    
                            /**
                             * Constructs a new AssetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AssetTypeEnum.
                             * @implements IAssetTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAssetTypeEnum=} [p] Properties to set
                             */
                            function AssetTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AssetTypeEnum} AssetTypeEnum instance
                             */
                            AssetTypeEnum.create = function create(properties) {
                                return new AssetTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AssetTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetTypeEnum} m AssetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AssetTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAssetTypeEnum} message AssetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AssetTypeEnum} AssetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AssetTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AssetTypeEnum} AssetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AssetTypeEnum} AssetTypeEnum
                             */
                            AssetTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AssetTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AssetTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AssetTypeEnum} m AssetTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AssetTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetType enum.
                             * @name google.ads.googleads.v11.enums.AssetTypeEnum.AssetType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} YOUTUBE_VIDEO=2 YOUTUBE_VIDEO value
                             * @property {number} MEDIA_BUNDLE=3 MEDIA_BUNDLE value
                             * @property {number} IMAGE=4 IMAGE value
                             * @property {number} TEXT=5 TEXT value
                             * @property {number} LEAD_FORM=6 LEAD_FORM value
                             * @property {number} BOOK_ON_GOOGLE=7 BOOK_ON_GOOGLE value
                             * @property {number} PROMOTION=8 PROMOTION value
                             * @property {number} CALLOUT=9 CALLOUT value
                             * @property {number} STRUCTURED_SNIPPET=10 STRUCTURED_SNIPPET value
                             * @property {number} SITELINK=11 SITELINK value
                             * @property {number} PAGE_FEED=12 PAGE_FEED value
                             * @property {number} DYNAMIC_EDUCATION=13 DYNAMIC_EDUCATION value
                             * @property {number} MOBILE_APP=14 MOBILE_APP value
                             * @property {number} HOTEL_CALLOUT=15 HOTEL_CALLOUT value
                             * @property {number} CALL=16 CALL value
                             * @property {number} PRICE=17 PRICE value
                             * @property {number} CALL_TO_ACTION=18 CALL_TO_ACTION value
                             * @property {number} DYNAMIC_REAL_ESTATE=19 DYNAMIC_REAL_ESTATE value
                             * @property {number} DYNAMIC_CUSTOM=20 DYNAMIC_CUSTOM value
                             * @property {number} DYNAMIC_HOTELS_AND_RENTALS=21 DYNAMIC_HOTELS_AND_RENTALS value
                             * @property {number} DYNAMIC_FLIGHTS=22 DYNAMIC_FLIGHTS value
                             * @property {number} DISCOVERY_CAROUSEL_CARD=23 DISCOVERY_CAROUSEL_CARD value
                             * @property {number} DYNAMIC_TRAVEL=24 DYNAMIC_TRAVEL value
                             * @property {number} DYNAMIC_LOCAL=25 DYNAMIC_LOCAL value
                             * @property {number} DYNAMIC_JOBS=26 DYNAMIC_JOBS value
                             */
                            AssetTypeEnum.AssetType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "YOUTUBE_VIDEO"] = 2;
                                values[valuesById[3] = "MEDIA_BUNDLE"] = 3;
                                values[valuesById[4] = "IMAGE"] = 4;
                                values[valuesById[5] = "TEXT"] = 5;
                                values[valuesById[6] = "LEAD_FORM"] = 6;
                                values[valuesById[7] = "BOOK_ON_GOOGLE"] = 7;
                                values[valuesById[8] = "PROMOTION"] = 8;
                                values[valuesById[9] = "CALLOUT"] = 9;
                                values[valuesById[10] = "STRUCTURED_SNIPPET"] = 10;
                                values[valuesById[11] = "SITELINK"] = 11;
                                values[valuesById[12] = "PAGE_FEED"] = 12;
                                values[valuesById[13] = "DYNAMIC_EDUCATION"] = 13;
                                values[valuesById[14] = "MOBILE_APP"] = 14;
                                values[valuesById[15] = "HOTEL_CALLOUT"] = 15;
                                values[valuesById[16] = "CALL"] = 16;
                                values[valuesById[17] = "PRICE"] = 17;
                                values[valuesById[18] = "CALL_TO_ACTION"] = 18;
                                values[valuesById[19] = "DYNAMIC_REAL_ESTATE"] = 19;
                                values[valuesById[20] = "DYNAMIC_CUSTOM"] = 20;
                                values[valuesById[21] = "DYNAMIC_HOTELS_AND_RENTALS"] = 21;
                                values[valuesById[22] = "DYNAMIC_FLIGHTS"] = 22;
                                values[valuesById[23] = "DISCOVERY_CAROUSEL_CARD"] = 23;
                                values[valuesById[24] = "DYNAMIC_TRAVEL"] = 24;
                                values[valuesById[25] = "DYNAMIC_LOCAL"] = 25;
                                values[valuesById[26] = "DYNAMIC_JOBS"] = 26;
                                return values;
                            })();
    
                            return AssetTypeEnum;
                        })();
    
                        enums.AsyncActionStatusEnum = (function() {
    
                            /**
                             * Properties of an AsyncActionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAsyncActionStatusEnum
                             */
    
                            /**
                             * Constructs a new AsyncActionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AsyncActionStatusEnum.
                             * @implements IAsyncActionStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAsyncActionStatusEnum=} [p] Properties to set
                             */
                            function AsyncActionStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AsyncActionStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAsyncActionStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AsyncActionStatusEnum} AsyncActionStatusEnum instance
                             */
                            AsyncActionStatusEnum.create = function create(properties) {
                                return new AsyncActionStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AsyncActionStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AsyncActionStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAsyncActionStatusEnum} m AsyncActionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AsyncActionStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AsyncActionStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AsyncActionStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAsyncActionStatusEnum} message AsyncActionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AsyncActionStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AsyncActionStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AsyncActionStatusEnum} AsyncActionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AsyncActionStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AsyncActionStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AsyncActionStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AsyncActionStatusEnum} AsyncActionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AsyncActionStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AsyncActionStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AsyncActionStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AsyncActionStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AsyncActionStatusEnum} AsyncActionStatusEnum
                             */
                            AsyncActionStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AsyncActionStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AsyncActionStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AsyncActionStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AsyncActionStatusEnum} m AsyncActionStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AsyncActionStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AsyncActionStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AsyncActionStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AsyncActionStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AsyncActionStatus enum.
                             * @name google.ads.googleads.v11.enums.AsyncActionStatusEnum.AsyncActionStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOT_STARTED=2 NOT_STARTED value
                             * @property {number} IN_PROGRESS=3 IN_PROGRESS value
                             * @property {number} COMPLETED=4 COMPLETED value
                             * @property {number} FAILED=5 FAILED value
                             * @property {number} COMPLETED_WITH_WARNING=6 COMPLETED_WITH_WARNING value
                             */
                            AsyncActionStatusEnum.AsyncActionStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOT_STARTED"] = 2;
                                values[valuesById[3] = "IN_PROGRESS"] = 3;
                                values[valuesById[4] = "COMPLETED"] = 4;
                                values[valuesById[5] = "FAILED"] = 5;
                                values[valuesById[6] = "COMPLETED_WITH_WARNING"] = 6;
                                return values;
                            })();
    
                            return AsyncActionStatusEnum;
                        })();
    
                        enums.AttributionModelEnum = (function() {
    
                            /**
                             * Properties of an AttributionModelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAttributionModelEnum
                             */
    
                            /**
                             * Constructs a new AttributionModelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AttributionModelEnum.
                             * @implements IAttributionModelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAttributionModelEnum=} [p] Properties to set
                             */
                            function AttributionModelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AttributionModelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAttributionModelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AttributionModelEnum} AttributionModelEnum instance
                             */
                            AttributionModelEnum.create = function create(properties) {
                                return new AttributionModelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AttributionModelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AttributionModelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAttributionModelEnum} m AttributionModelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AttributionModelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AttributionModelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AttributionModelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAttributionModelEnum} message AttributionModelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AttributionModelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AttributionModelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AttributionModelEnum} AttributionModelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AttributionModelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AttributionModelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AttributionModelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AttributionModelEnum} AttributionModelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AttributionModelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AttributionModelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AttributionModelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AttributionModelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AttributionModelEnum} AttributionModelEnum
                             */
                            AttributionModelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AttributionModelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AttributionModelEnum();
                            };
    
                            /**
                             * Creates a plain object from an AttributionModelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AttributionModelEnum} m AttributionModelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AttributionModelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AttributionModelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AttributionModelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AttributionModelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AttributionModel enum.
                             * @name google.ads.googleads.v11.enums.AttributionModelEnum.AttributionModel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EXTERNAL=100 EXTERNAL value
                             * @property {number} GOOGLE_ADS_LAST_CLICK=101 GOOGLE_ADS_LAST_CLICK value
                             * @property {number} GOOGLE_SEARCH_ATTRIBUTION_FIRST_CLICK=102 GOOGLE_SEARCH_ATTRIBUTION_FIRST_CLICK value
                             * @property {number} GOOGLE_SEARCH_ATTRIBUTION_LINEAR=103 GOOGLE_SEARCH_ATTRIBUTION_LINEAR value
                             * @property {number} GOOGLE_SEARCH_ATTRIBUTION_TIME_DECAY=104 GOOGLE_SEARCH_ATTRIBUTION_TIME_DECAY value
                             * @property {number} GOOGLE_SEARCH_ATTRIBUTION_POSITION_BASED=105 GOOGLE_SEARCH_ATTRIBUTION_POSITION_BASED value
                             * @property {number} GOOGLE_SEARCH_ATTRIBUTION_DATA_DRIVEN=106 GOOGLE_SEARCH_ATTRIBUTION_DATA_DRIVEN value
                             */
                            AttributionModelEnum.AttributionModel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[100] = "EXTERNAL"] = 100;
                                values[valuesById[101] = "GOOGLE_ADS_LAST_CLICK"] = 101;
                                values[valuesById[102] = "GOOGLE_SEARCH_ATTRIBUTION_FIRST_CLICK"] = 102;
                                values[valuesById[103] = "GOOGLE_SEARCH_ATTRIBUTION_LINEAR"] = 103;
                                values[valuesById[104] = "GOOGLE_SEARCH_ATTRIBUTION_TIME_DECAY"] = 104;
                                values[valuesById[105] = "GOOGLE_SEARCH_ATTRIBUTION_POSITION_BASED"] = 105;
                                values[valuesById[106] = "GOOGLE_SEARCH_ATTRIBUTION_DATA_DRIVEN"] = 106;
                                return values;
                            })();
    
                            return AttributionModelEnum;
                        })();
    
                        enums.AudienceInsightsDimensionEnum = (function() {
    
                            /**
                             * Properties of an AudienceInsightsDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAudienceInsightsDimensionEnum
                             */
    
                            /**
                             * Constructs a new AudienceInsightsDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AudienceInsightsDimensionEnum.
                             * @implements IAudienceInsightsDimensionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAudienceInsightsDimensionEnum=} [p] Properties to set
                             */
                            function AudienceInsightsDimensionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AudienceInsightsDimensionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAudienceInsightsDimensionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum} AudienceInsightsDimensionEnum instance
                             */
                            AudienceInsightsDimensionEnum.create = function create(properties) {
                                return new AudienceInsightsDimensionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsDimensionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAudienceInsightsDimensionEnum} m AudienceInsightsDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsDimensionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsDimensionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAudienceInsightsDimensionEnum} message AudienceInsightsDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsDimensionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceInsightsDimensionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum} AudienceInsightsDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsDimensionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceInsightsDimensionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum} AudienceInsightsDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsDimensionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceInsightsDimensionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceInsightsDimensionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AudienceInsightsDimensionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum} AudienceInsightsDimensionEnum
                             */
                            AudienceInsightsDimensionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum();
                            };
    
                            /**
                             * Creates a plain object from an AudienceInsightsDimensionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum} m AudienceInsightsDimensionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceInsightsDimensionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AudienceInsightsDimensionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceInsightsDimensionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AudienceInsightsDimension enum.
                             * @name google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.AudienceInsightsDimension
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CATEGORY=2 CATEGORY value
                             * @property {number} KNOWLEDGE_GRAPH=3 KNOWLEDGE_GRAPH value
                             */
                            AudienceInsightsDimensionEnum.AudienceInsightsDimension = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CATEGORY"] = 2;
                                values[valuesById[3] = "KNOWLEDGE_GRAPH"] = 3;
                                return values;
                            })();
    
                            return AudienceInsightsDimensionEnum;
                        })();
    
                        enums.AudienceStatusEnum = (function() {
    
                            /**
                             * Properties of an AudienceStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IAudienceStatusEnum
                             */
    
                            /**
                             * Constructs a new AudienceStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an AudienceStatusEnum.
                             * @implements IAudienceStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IAudienceStatusEnum=} [p] Properties to set
                             */
                            function AudienceStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AudienceStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAudienceStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.AudienceStatusEnum} AudienceStatusEnum instance
                             */
                            AudienceStatusEnum.create = function create(properties) {
                                return new AudienceStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.AudienceStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAudienceStatusEnum} m AudienceStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.AudienceStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IAudienceStatusEnum} message AudienceStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.AudienceStatusEnum} AudienceStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.AudienceStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.AudienceStatusEnum} AudienceStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AudienceStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.AudienceStatusEnum} AudienceStatusEnum
                             */
                            AudienceStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.AudienceStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.AudienceStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an AudienceStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.AudienceStatusEnum} m AudienceStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AudienceStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.AudienceStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AudienceStatus enum.
                             * @name google.ads.googleads.v11.enums.AudienceStatusEnum.AudienceStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            AudienceStatusEnum.AudienceStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return AudienceStatusEnum;
                        })();
    
                        enums.BatchJobStatusEnum = (function() {
    
                            /**
                             * Properties of a BatchJobStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBatchJobStatusEnum
                             */
    
                            /**
                             * Constructs a new BatchJobStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BatchJobStatusEnum.
                             * @implements IBatchJobStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBatchJobStatusEnum=} [p] Properties to set
                             */
                            function BatchJobStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BatchJobStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBatchJobStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BatchJobStatusEnum} BatchJobStatusEnum instance
                             */
                            BatchJobStatusEnum.create = function create(properties) {
                                return new BatchJobStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BatchJobStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BatchJobStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBatchJobStatusEnum} m BatchJobStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BatchJobStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BatchJobStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBatchJobStatusEnum} message BatchJobStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BatchJobStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BatchJobStatusEnum} BatchJobStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BatchJobStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BatchJobStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BatchJobStatusEnum} BatchJobStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BatchJobStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BatchJobStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BatchJobStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BatchJobStatusEnum} BatchJobStatusEnum
                             */
                            BatchJobStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BatchJobStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BatchJobStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a BatchJobStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BatchJobStatusEnum} m BatchJobStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BatchJobStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BatchJobStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BatchJobStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BatchJobStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BatchJobStatus enum.
                             * @name google.ads.googleads.v11.enums.BatchJobStatusEnum.BatchJobStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} RUNNING=3 RUNNING value
                             * @property {number} DONE=4 DONE value
                             */
                            BatchJobStatusEnum.BatchJobStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "RUNNING"] = 3;
                                values[valuesById[4] = "DONE"] = 4;
                                return values;
                            })();
    
                            return BatchJobStatusEnum;
                        })();
    
                        enums.BidModifierSourceEnum = (function() {
    
                            /**
                             * Properties of a BidModifierSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBidModifierSourceEnum
                             */
    
                            /**
                             * Constructs a new BidModifierSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BidModifierSourceEnum.
                             * @implements IBidModifierSourceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBidModifierSourceEnum=} [p] Properties to set
                             */
                            function BidModifierSourceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BidModifierSourceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBidModifierSourceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BidModifierSourceEnum} BidModifierSourceEnum instance
                             */
                            BidModifierSourceEnum.create = function create(properties) {
                                return new BidModifierSourceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BidModifierSourceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BidModifierSourceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBidModifierSourceEnum} m BidModifierSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BidModifierSourceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BidModifierSourceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BidModifierSourceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBidModifierSourceEnum} message BidModifierSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BidModifierSourceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BidModifierSourceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BidModifierSourceEnum} BidModifierSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BidModifierSourceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BidModifierSourceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BidModifierSourceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BidModifierSourceEnum} BidModifierSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BidModifierSourceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BidModifierSourceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BidModifierSourceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BidModifierSourceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BidModifierSourceEnum} BidModifierSourceEnum
                             */
                            BidModifierSourceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BidModifierSourceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BidModifierSourceEnum();
                            };
    
                            /**
                             * Creates a plain object from a BidModifierSourceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BidModifierSourceEnum} m BidModifierSourceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BidModifierSourceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BidModifierSourceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BidModifierSourceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BidModifierSourceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BidModifierSource enum.
                             * @name google.ads.googleads.v11.enums.BidModifierSourceEnum.BidModifierSource
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CAMPAIGN=2 CAMPAIGN value
                             * @property {number} AD_GROUP=3 AD_GROUP value
                             */
                            BidModifierSourceEnum.BidModifierSource = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CAMPAIGN"] = 2;
                                values[valuesById[3] = "AD_GROUP"] = 3;
                                return values;
                            })();
    
                            return BidModifierSourceEnum;
                        })();
    
                        enums.BiddingSourceEnum = (function() {
    
                            /**
                             * Properties of a BiddingSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBiddingSourceEnum
                             */
    
                            /**
                             * Constructs a new BiddingSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BiddingSourceEnum.
                             * @implements IBiddingSourceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBiddingSourceEnum=} [p] Properties to set
                             */
                            function BiddingSourceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BiddingSourceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingSourceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BiddingSourceEnum} BiddingSourceEnum instance
                             */
                            BiddingSourceEnum.create = function create(properties) {
                                return new BiddingSourceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingSourceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BiddingSourceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingSourceEnum} m BiddingSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingSourceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingSourceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BiddingSourceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingSourceEnum} message BiddingSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingSourceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingSourceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BiddingSourceEnum} BiddingSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingSourceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BiddingSourceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingSourceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BiddingSourceEnum} BiddingSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingSourceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingSourceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingSourceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BiddingSourceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BiddingSourceEnum} BiddingSourceEnum
                             */
                            BiddingSourceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BiddingSourceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BiddingSourceEnum();
                            };
    
                            /**
                             * Creates a plain object from a BiddingSourceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BiddingSourceEnum} m BiddingSourceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingSourceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BiddingSourceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BiddingSourceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingSourceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BiddingSource enum.
                             * @name google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CAMPAIGN_BIDDING_STRATEGY=5 CAMPAIGN_BIDDING_STRATEGY value
                             * @property {number} AD_GROUP=6 AD_GROUP value
                             * @property {number} AD_GROUP_CRITERION=7 AD_GROUP_CRITERION value
                             */
                            BiddingSourceEnum.BiddingSource = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[5] = "CAMPAIGN_BIDDING_STRATEGY"] = 5;
                                values[valuesById[6] = "AD_GROUP"] = 6;
                                values[valuesById[7] = "AD_GROUP_CRITERION"] = 7;
                                return values;
                            })();
    
                            return BiddingSourceEnum;
                        })();
    
                        enums.BiddingStrategyStatusEnum = (function() {
    
                            /**
                             * Properties of a BiddingStrategyStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBiddingStrategyStatusEnum
                             */
    
                            /**
                             * Constructs a new BiddingStrategyStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BiddingStrategyStatusEnum.
                             * @implements IBiddingStrategyStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyStatusEnum=} [p] Properties to set
                             */
                            function BiddingStrategyStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BiddingStrategyStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyStatusEnum} BiddingStrategyStatusEnum instance
                             */
                            BiddingStrategyStatusEnum.create = function create(properties) {
                                return new BiddingStrategyStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BiddingStrategyStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyStatusEnum} m BiddingStrategyStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BiddingStrategyStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyStatusEnum} message BiddingStrategyStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingStrategyStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyStatusEnum} BiddingStrategyStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BiddingStrategyStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingStrategyStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyStatusEnum} BiddingStrategyStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingStrategyStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingStrategyStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BiddingStrategyStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyStatusEnum} BiddingStrategyStatusEnum
                             */
                            BiddingStrategyStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BiddingStrategyStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BiddingStrategyStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a BiddingStrategyStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BiddingStrategyStatusEnum} m BiddingStrategyStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingStrategyStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BiddingStrategyStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingStrategyStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BiddingStrategyStatus enum.
                             * @name google.ads.googleads.v11.enums.BiddingStrategyStatusEnum.BiddingStrategyStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            BiddingStrategyStatusEnum.BiddingStrategyStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return BiddingStrategyStatusEnum;
                        })();
    
                        enums.BiddingStrategyTypeEnum = (function() {
    
                            /**
                             * Properties of a BiddingStrategyTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBiddingStrategyTypeEnum
                             */
    
                            /**
                             * Constructs a new BiddingStrategyTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BiddingStrategyTypeEnum.
                             * @implements IBiddingStrategyTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyTypeEnum=} [p] Properties to set
                             */
                            function BiddingStrategyTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BiddingStrategyTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum} BiddingStrategyTypeEnum instance
                             */
                            BiddingStrategyTypeEnum.create = function create(properties) {
                                return new BiddingStrategyTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyTypeEnum} m BiddingStrategyTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBiddingStrategyTypeEnum} message BiddingStrategyTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingStrategyTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum} BiddingStrategyTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BiddingStrategyTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingStrategyTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum} BiddingStrategyTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingStrategyTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingStrategyTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BiddingStrategyTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum} BiddingStrategyTypeEnum
                             */
                            BiddingStrategyTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BiddingStrategyTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BiddingStrategyTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a BiddingStrategyTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum} m BiddingStrategyTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingStrategyTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BiddingStrategyTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BiddingStrategyTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingStrategyTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BiddingStrategyType enum.
                             * @name google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} COMMISSION=16 COMMISSION value
                             * @property {number} ENHANCED_CPC=2 ENHANCED_CPC value
                             * @property {number} INVALID=17 INVALID value
                             * @property {number} MANUAL_CPA=18 MANUAL_CPA value
                             * @property {number} MANUAL_CPC=3 MANUAL_CPC value
                             * @property {number} MANUAL_CPM=4 MANUAL_CPM value
                             * @property {number} MANUAL_CPV=13 MANUAL_CPV value
                             * @property {number} MAXIMIZE_CONVERSIONS=10 MAXIMIZE_CONVERSIONS value
                             * @property {number} MAXIMIZE_CONVERSION_VALUE=11 MAXIMIZE_CONVERSION_VALUE value
                             * @property {number} PAGE_ONE_PROMOTED=5 PAGE_ONE_PROMOTED value
                             * @property {number} PERCENT_CPC=12 PERCENT_CPC value
                             * @property {number} TARGET_CPA=6 TARGET_CPA value
                             * @property {number} TARGET_CPM=14 TARGET_CPM value
                             * @property {number} TARGET_IMPRESSION_SHARE=15 TARGET_IMPRESSION_SHARE value
                             * @property {number} TARGET_OUTRANK_SHARE=7 TARGET_OUTRANK_SHARE value
                             * @property {number} TARGET_ROAS=8 TARGET_ROAS value
                             * @property {number} TARGET_SPEND=9 TARGET_SPEND value
                             */
                            BiddingStrategyTypeEnum.BiddingStrategyType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[16] = "COMMISSION"] = 16;
                                values[valuesById[2] = "ENHANCED_CPC"] = 2;
                                values[valuesById[17] = "INVALID"] = 17;
                                values[valuesById[18] = "MANUAL_CPA"] = 18;
                                values[valuesById[3] = "MANUAL_CPC"] = 3;
                                values[valuesById[4] = "MANUAL_CPM"] = 4;
                                values[valuesById[13] = "MANUAL_CPV"] = 13;
                                values[valuesById[10] = "MAXIMIZE_CONVERSIONS"] = 10;
                                values[valuesById[11] = "MAXIMIZE_CONVERSION_VALUE"] = 11;
                                values[valuesById[5] = "PAGE_ONE_PROMOTED"] = 5;
                                values[valuesById[12] = "PERCENT_CPC"] = 12;
                                values[valuesById[6] = "TARGET_CPA"] = 6;
                                values[valuesById[14] = "TARGET_CPM"] = 14;
                                values[valuesById[15] = "TARGET_IMPRESSION_SHARE"] = 15;
                                values[valuesById[7] = "TARGET_OUTRANK_SHARE"] = 7;
                                values[valuesById[8] = "TARGET_ROAS"] = 8;
                                values[valuesById[9] = "TARGET_SPEND"] = 9;
                                return values;
                            })();
    
                            return BiddingStrategyTypeEnum;
                        })();
    
                        enums.BillingSetupStatusEnum = (function() {
    
                            /**
                             * Properties of a BillingSetupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBillingSetupStatusEnum
                             */
    
                            /**
                             * Constructs a new BillingSetupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BillingSetupStatusEnum.
                             * @implements IBillingSetupStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBillingSetupStatusEnum=} [p] Properties to set
                             */
                            function BillingSetupStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BillingSetupStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBillingSetupStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BillingSetupStatusEnum} BillingSetupStatusEnum instance
                             */
                            BillingSetupStatusEnum.create = function create(properties) {
                                return new BillingSetupStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BillingSetupStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BillingSetupStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBillingSetupStatusEnum} m BillingSetupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetupStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BillingSetupStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BillingSetupStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBillingSetupStatusEnum} message BillingSetupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetupStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BillingSetupStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BillingSetupStatusEnum} BillingSetupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetupStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BillingSetupStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BillingSetupStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BillingSetupStatusEnum} BillingSetupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetupStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BillingSetupStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BillingSetupStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BillingSetupStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BillingSetupStatusEnum} BillingSetupStatusEnum
                             */
                            BillingSetupStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BillingSetupStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BillingSetupStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a BillingSetupStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BillingSetupStatusEnum} m BillingSetupStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BillingSetupStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BillingSetupStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BillingSetupStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BillingSetupStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BillingSetupStatus enum.
                             * @name google.ads.googleads.v11.enums.BillingSetupStatusEnum.BillingSetupStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} APPROVED_HELD=3 APPROVED_HELD value
                             * @property {number} APPROVED=4 APPROVED value
                             * @property {number} CANCELLED=5 CANCELLED value
                             */
                            BillingSetupStatusEnum.BillingSetupStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "APPROVED_HELD"] = 3;
                                values[valuesById[4] = "APPROVED"] = 4;
                                values[valuesById[5] = "CANCELLED"] = 5;
                                return values;
                            })();
    
                            return BillingSetupStatusEnum;
                        })();
    
                        enums.BrandSafetySuitabilityEnum = (function() {
    
                            /**
                             * Properties of a BrandSafetySuitabilityEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBrandSafetySuitabilityEnum
                             */
    
                            /**
                             * Constructs a new BrandSafetySuitabilityEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BrandSafetySuitabilityEnum.
                             * @implements IBrandSafetySuitabilityEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBrandSafetySuitabilityEnum=} [p] Properties to set
                             */
                            function BrandSafetySuitabilityEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BrandSafetySuitabilityEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBrandSafetySuitabilityEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum} BrandSafetySuitabilityEnum instance
                             */
                            BrandSafetySuitabilityEnum.create = function create(properties) {
                                return new BrandSafetySuitabilityEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BrandSafetySuitabilityEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBrandSafetySuitabilityEnum} m BrandSafetySuitabilityEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BrandSafetySuitabilityEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BrandSafetySuitabilityEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBrandSafetySuitabilityEnum} message BrandSafetySuitabilityEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BrandSafetySuitabilityEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BrandSafetySuitabilityEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum} BrandSafetySuitabilityEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BrandSafetySuitabilityEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BrandSafetySuitabilityEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum} BrandSafetySuitabilityEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BrandSafetySuitabilityEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BrandSafetySuitabilityEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BrandSafetySuitabilityEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BrandSafetySuitabilityEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum} BrandSafetySuitabilityEnum
                             */
                            BrandSafetySuitabilityEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum();
                            };
    
                            /**
                             * Creates a plain object from a BrandSafetySuitabilityEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum} m BrandSafetySuitabilityEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BrandSafetySuitabilityEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BrandSafetySuitabilityEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BrandSafetySuitabilityEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BrandSafetySuitability enum.
                             * @name google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum.BrandSafetySuitability
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EXPANDED_INVENTORY=2 EXPANDED_INVENTORY value
                             * @property {number} STANDARD_INVENTORY=3 STANDARD_INVENTORY value
                             * @property {number} LIMITED_INVENTORY=4 LIMITED_INVENTORY value
                             */
                            BrandSafetySuitabilityEnum.BrandSafetySuitability = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EXPANDED_INVENTORY"] = 2;
                                values[valuesById[3] = "STANDARD_INVENTORY"] = 3;
                                values[valuesById[4] = "LIMITED_INVENTORY"] = 4;
                                return values;
                            })();
    
                            return BrandSafetySuitabilityEnum;
                        })();
    
                        enums.BudgetDeliveryMethodEnum = (function() {
    
                            /**
                             * Properties of a BudgetDeliveryMethodEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBudgetDeliveryMethodEnum
                             */
    
                            /**
                             * Constructs a new BudgetDeliveryMethodEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BudgetDeliveryMethodEnum.
                             * @implements IBudgetDeliveryMethodEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBudgetDeliveryMethodEnum=} [p] Properties to set
                             */
                            function BudgetDeliveryMethodEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BudgetDeliveryMethodEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetDeliveryMethodEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum} BudgetDeliveryMethodEnum instance
                             */
                            BudgetDeliveryMethodEnum.create = function create(properties) {
                                return new BudgetDeliveryMethodEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetDeliveryMethodEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetDeliveryMethodEnum} m BudgetDeliveryMethodEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetDeliveryMethodEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetDeliveryMethodEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetDeliveryMethodEnum} message BudgetDeliveryMethodEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetDeliveryMethodEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetDeliveryMethodEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum} BudgetDeliveryMethodEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetDeliveryMethodEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetDeliveryMethodEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum} BudgetDeliveryMethodEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetDeliveryMethodEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetDeliveryMethodEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetDeliveryMethodEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BudgetDeliveryMethodEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum} BudgetDeliveryMethodEnum
                             */
                            BudgetDeliveryMethodEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum();
                            };
    
                            /**
                             * Creates a plain object from a BudgetDeliveryMethodEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum} m BudgetDeliveryMethodEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetDeliveryMethodEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BudgetDeliveryMethodEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetDeliveryMethodEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BudgetDeliveryMethod enum.
                             * @name google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum.BudgetDeliveryMethod
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} STANDARD=2 STANDARD value
                             * @property {number} ACCELERATED=3 ACCELERATED value
                             */
                            BudgetDeliveryMethodEnum.BudgetDeliveryMethod = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "STANDARD"] = 2;
                                values[valuesById[3] = "ACCELERATED"] = 3;
                                return values;
                            })();
    
                            return BudgetDeliveryMethodEnum;
                        })();
    
                        enums.BudgetPeriodEnum = (function() {
    
                            /**
                             * Properties of a BudgetPeriodEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBudgetPeriodEnum
                             */
    
                            /**
                             * Constructs a new BudgetPeriodEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BudgetPeriodEnum.
                             * @implements IBudgetPeriodEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBudgetPeriodEnum=} [p] Properties to set
                             */
                            function BudgetPeriodEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BudgetPeriodEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetPeriodEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BudgetPeriodEnum} BudgetPeriodEnum instance
                             */
                            BudgetPeriodEnum.create = function create(properties) {
                                return new BudgetPeriodEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetPeriodEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetPeriodEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetPeriodEnum} m BudgetPeriodEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetPeriodEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetPeriodEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetPeriodEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetPeriodEnum} message BudgetPeriodEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetPeriodEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetPeriodEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BudgetPeriodEnum} BudgetPeriodEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetPeriodEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BudgetPeriodEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetPeriodEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BudgetPeriodEnum} BudgetPeriodEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetPeriodEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetPeriodEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetPeriodEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BudgetPeriodEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BudgetPeriodEnum} BudgetPeriodEnum
                             */
                            BudgetPeriodEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BudgetPeriodEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BudgetPeriodEnum();
                            };
    
                            /**
                             * Creates a plain object from a BudgetPeriodEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BudgetPeriodEnum} m BudgetPeriodEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetPeriodEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BudgetPeriodEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BudgetPeriodEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetPeriodEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BudgetPeriod enum.
                             * @name google.ads.googleads.v11.enums.BudgetPeriodEnum.BudgetPeriod
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DAILY=2 DAILY value
                             * @property {number} CUSTOM_PERIOD=5 CUSTOM_PERIOD value
                             */
                            BudgetPeriodEnum.BudgetPeriod = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DAILY"] = 2;
                                values[valuesById[5] = "CUSTOM_PERIOD"] = 5;
                                return values;
                            })();
    
                            return BudgetPeriodEnum;
                        })();
    
                        enums.BudgetStatusEnum = (function() {
    
                            /**
                             * Properties of a BudgetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBudgetStatusEnum
                             */
    
                            /**
                             * Constructs a new BudgetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BudgetStatusEnum.
                             * @implements IBudgetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBudgetStatusEnum=} [p] Properties to set
                             */
                            function BudgetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BudgetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BudgetStatusEnum} BudgetStatusEnum instance
                             */
                            BudgetStatusEnum.create = function create(properties) {
                                return new BudgetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetStatusEnum} m BudgetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetStatusEnum} message BudgetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BudgetStatusEnum} BudgetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BudgetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BudgetStatusEnum} BudgetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BudgetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BudgetStatusEnum} BudgetStatusEnum
                             */
                            BudgetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BudgetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BudgetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a BudgetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BudgetStatusEnum} m BudgetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BudgetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BudgetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BudgetStatus enum.
                             * @name google.ads.googleads.v11.enums.BudgetStatusEnum.BudgetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            BudgetStatusEnum.BudgetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return BudgetStatusEnum;
                        })();
    
                        enums.BudgetTypeEnum = (function() {
    
                            /**
                             * Properties of a BudgetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IBudgetTypeEnum
                             */
    
                            /**
                             * Constructs a new BudgetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a BudgetTypeEnum.
                             * @implements IBudgetTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IBudgetTypeEnum=} [p] Properties to set
                             */
                            function BudgetTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BudgetTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.BudgetTypeEnum} BudgetTypeEnum instance
                             */
                            BudgetTypeEnum.create = function create(properties) {
                                return new BudgetTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BudgetTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetTypeEnum} m BudgetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BudgetTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.BudgetTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IBudgetTypeEnum} message BudgetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BudgetTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BudgetTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.BudgetTypeEnum} BudgetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.BudgetTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BudgetTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.BudgetTypeEnum} BudgetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BudgetTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BudgetTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BudgetTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BudgetTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.BudgetTypeEnum} BudgetTypeEnum
                             */
                            BudgetTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.BudgetTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.BudgetTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a BudgetTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.BudgetTypeEnum} m BudgetTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BudgetTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BudgetTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.BudgetTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BudgetTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BudgetType enum.
                             * @name google.ads.googleads.v11.enums.BudgetTypeEnum.BudgetType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} STANDARD=2 STANDARD value
                             * @property {number} FIXED_CPA=4 FIXED_CPA value
                             * @property {number} SMART_CAMPAIGN=5 SMART_CAMPAIGN value
                             * @property {number} LOCAL_SERVICES=6 LOCAL_SERVICES value
                             */
                            BudgetTypeEnum.BudgetType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "STANDARD"] = 2;
                                values[valuesById[4] = "FIXED_CPA"] = 4;
                                values[valuesById[5] = "SMART_CAMPAIGN"] = 5;
                                values[valuesById[6] = "LOCAL_SERVICES"] = 6;
                                return values;
                            })();
    
                            return BudgetTypeEnum;
                        })();
    
                        enums.CallPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a CallPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICallPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new CallPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CallPlaceholderFieldEnum.
                             * @implements ICallPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICallPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function CallPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CallPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CallPlaceholderFieldEnum} CallPlaceholderFieldEnum instance
                             */
                            CallPlaceholderFieldEnum.create = function create(properties) {
                                return new CallPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CallPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CallPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallPlaceholderFieldEnum} m CallPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CallPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallPlaceholderFieldEnum} message CallPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CallPlaceholderFieldEnum} CallPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CallPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CallPlaceholderFieldEnum} CallPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CallPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CallPlaceholderFieldEnum} CallPlaceholderFieldEnum
                             */
                            CallPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CallPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CallPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a CallPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CallPlaceholderFieldEnum} m CallPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CallPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CallPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CallPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.CallPlaceholderFieldEnum.CallPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PHONE_NUMBER=2 PHONE_NUMBER value
                             * @property {number} COUNTRY_CODE=3 COUNTRY_CODE value
                             * @property {number} TRACKED=4 TRACKED value
                             * @property {number} CONVERSION_TYPE_ID=5 CONVERSION_TYPE_ID value
                             * @property {number} CONVERSION_REPORTING_STATE=6 CONVERSION_REPORTING_STATE value
                             */
                            CallPlaceholderFieldEnum.CallPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PHONE_NUMBER"] = 2;
                                values[valuesById[3] = "COUNTRY_CODE"] = 3;
                                values[valuesById[4] = "TRACKED"] = 4;
                                values[valuesById[5] = "CONVERSION_TYPE_ID"] = 5;
                                values[valuesById[6] = "CONVERSION_REPORTING_STATE"] = 6;
                                return values;
                            })();
    
                            return CallPlaceholderFieldEnum;
                        })();
    
                        enums.CallTrackingDisplayLocationEnum = (function() {
    
                            /**
                             * Properties of a CallTrackingDisplayLocationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICallTrackingDisplayLocationEnum
                             */
    
                            /**
                             * Constructs a new CallTrackingDisplayLocationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CallTrackingDisplayLocationEnum.
                             * @implements ICallTrackingDisplayLocationEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICallTrackingDisplayLocationEnum=} [p] Properties to set
                             */
                            function CallTrackingDisplayLocationEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CallTrackingDisplayLocationEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallTrackingDisplayLocationEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum} CallTrackingDisplayLocationEnum instance
                             */
                            CallTrackingDisplayLocationEnum.create = function create(properties) {
                                return new CallTrackingDisplayLocationEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CallTrackingDisplayLocationEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallTrackingDisplayLocationEnum} m CallTrackingDisplayLocationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallTrackingDisplayLocationEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallTrackingDisplayLocationEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallTrackingDisplayLocationEnum} message CallTrackingDisplayLocationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallTrackingDisplayLocationEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallTrackingDisplayLocationEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum} CallTrackingDisplayLocationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallTrackingDisplayLocationEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallTrackingDisplayLocationEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum} CallTrackingDisplayLocationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallTrackingDisplayLocationEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallTrackingDisplayLocationEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallTrackingDisplayLocationEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CallTrackingDisplayLocationEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum} CallTrackingDisplayLocationEnum
                             */
                            CallTrackingDisplayLocationEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum();
                            };
    
                            /**
                             * Creates a plain object from a CallTrackingDisplayLocationEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum} m CallTrackingDisplayLocationEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallTrackingDisplayLocationEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CallTrackingDisplayLocationEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallTrackingDisplayLocationEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CallTrackingDisplayLocation enum.
                             * @name google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD=2 AD value
                             * @property {number} LANDING_PAGE=3 LANDING_PAGE value
                             */
                            CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD"] = 2;
                                values[valuesById[3] = "LANDING_PAGE"] = 3;
                                return values;
                            })();
    
                            return CallTrackingDisplayLocationEnum;
                        })();
    
                        enums.CallTypeEnum = (function() {
    
                            /**
                             * Properties of a CallTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICallTypeEnum
                             */
    
                            /**
                             * Constructs a new CallTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CallTypeEnum.
                             * @implements ICallTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICallTypeEnum=} [p] Properties to set
                             */
                            function CallTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CallTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CallTypeEnum} CallTypeEnum instance
                             */
                            CallTypeEnum.create = function create(properties) {
                                return new CallTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CallTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CallTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallTypeEnum} m CallTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CallTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICallTypeEnum} message CallTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CallTypeEnum} CallTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CallTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CallTypeEnum} CallTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CallTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CallTypeEnum} CallTypeEnum
                             */
                            CallTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CallTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CallTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CallTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CallTypeEnum} m CallTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CallTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CallTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CallType enum.
                             * @name google.ads.googleads.v11.enums.CallTypeEnum.CallType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MANUALLY_DIALED=2 MANUALLY_DIALED value
                             * @property {number} HIGH_END_MOBILE_SEARCH=3 HIGH_END_MOBILE_SEARCH value
                             */
                            CallTypeEnum.CallType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MANUALLY_DIALED"] = 2;
                                values[valuesById[3] = "HIGH_END_MOBILE_SEARCH"] = 3;
                                return values;
                            })();
    
                            return CallTypeEnum;
                        })();
    
                        enums.CalloutPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a CalloutPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICalloutPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new CalloutPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CalloutPlaceholderFieldEnum.
                             * @implements ICalloutPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICalloutPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function CalloutPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CalloutPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICalloutPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum} CalloutPlaceholderFieldEnum instance
                             */
                            CalloutPlaceholderFieldEnum.create = function create(properties) {
                                return new CalloutPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CalloutPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICalloutPlaceholderFieldEnum} m CalloutPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CalloutPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CalloutPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICalloutPlaceholderFieldEnum} message CalloutPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CalloutPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CalloutPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum} CalloutPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CalloutPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CalloutPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum} CalloutPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CalloutPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CalloutPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CalloutPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CalloutPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum} CalloutPlaceholderFieldEnum
                             */
                            CalloutPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a CalloutPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum} m CalloutPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CalloutPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CalloutPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CalloutPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CalloutPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum.CalloutPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CALLOUT_TEXT=2 CALLOUT_TEXT value
                             */
                            CalloutPlaceholderFieldEnum.CalloutPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CALLOUT_TEXT"] = 2;
                                return values;
                            })();
    
                            return CalloutPlaceholderFieldEnum;
                        })();
    
                        enums.CampaignCriterionStatusEnum = (function() {
    
                            /**
                             * Properties of a CampaignCriterionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignCriterionStatusEnum
                             */
    
                            /**
                             * Constructs a new CampaignCriterionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignCriterionStatusEnum.
                             * @implements ICampaignCriterionStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignCriterionStatusEnum=} [p] Properties to set
                             */
                            function CampaignCriterionStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignCriterionStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignCriterionStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignCriterionStatusEnum} CampaignCriterionStatusEnum instance
                             */
                            CampaignCriterionStatusEnum.create = function create(properties) {
                                return new CampaignCriterionStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignCriterionStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignCriterionStatusEnum} m CampaignCriterionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignCriterionStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignCriterionStatusEnum} message CampaignCriterionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCriterionStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignCriterionStatusEnum} CampaignCriterionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignCriterionStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCriterionStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignCriterionStatusEnum} CampaignCriterionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCriterionStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCriterionStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCriterionStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignCriterionStatusEnum} CampaignCriterionStatusEnum
                             */
                            CampaignCriterionStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignCriterionStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignCriterionStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignCriterionStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignCriterionStatusEnum} m CampaignCriterionStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCriterionStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignCriterionStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignCriterionStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCriterionStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignCriterionStatus enum.
                             * @name google.ads.googleads.v11.enums.CampaignCriterionStatusEnum.CampaignCriterionStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} PAUSED=3 PAUSED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            CampaignCriterionStatusEnum.CampaignCriterionStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "PAUSED"] = 3;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return CampaignCriterionStatusEnum;
                        })();
    
                        enums.CampaignDraftStatusEnum = (function() {
    
                            /**
                             * Properties of a CampaignDraftStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignDraftStatusEnum
                             */
    
                            /**
                             * Constructs a new CampaignDraftStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignDraftStatusEnum.
                             * @implements ICampaignDraftStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignDraftStatusEnum=} [p] Properties to set
                             */
                            function CampaignDraftStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignDraftStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignDraftStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignDraftStatusEnum} CampaignDraftStatusEnum instance
                             */
                            CampaignDraftStatusEnum.create = function create(properties) {
                                return new CampaignDraftStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignDraftStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignDraftStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignDraftStatusEnum} m CampaignDraftStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraftStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignDraftStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignDraftStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignDraftStatusEnum} message CampaignDraftStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraftStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignDraftStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignDraftStatusEnum} CampaignDraftStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraftStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignDraftStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignDraftStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignDraftStatusEnum} CampaignDraftStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraftStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignDraftStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignDraftStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignDraftStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignDraftStatusEnum} CampaignDraftStatusEnum
                             */
                            CampaignDraftStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignDraftStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignDraftStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignDraftStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignDraftStatusEnum} m CampaignDraftStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignDraftStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignDraftStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignDraftStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignDraftStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignDraftStatus enum.
                             * @name google.ads.googleads.v11.enums.CampaignDraftStatusEnum.CampaignDraftStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PROPOSED=2 PROPOSED value
                             * @property {number} REMOVED=3 REMOVED value
                             * @property {number} PROMOTING=5 PROMOTING value
                             * @property {number} PROMOTED=4 PROMOTED value
                             * @property {number} PROMOTE_FAILED=6 PROMOTE_FAILED value
                             */
                            CampaignDraftStatusEnum.CampaignDraftStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PROPOSED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                values[valuesById[5] = "PROMOTING"] = 5;
                                values[valuesById[4] = "PROMOTED"] = 4;
                                values[valuesById[6] = "PROMOTE_FAILED"] = 6;
                                return values;
                            })();
    
                            return CampaignDraftStatusEnum;
                        })();
    
                        enums.CampaignExperimentStatusEnum = (function() {
    
                            /**
                             * Properties of a CampaignExperimentStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignExperimentStatusEnum
                             */
    
                            /**
                             * Constructs a new CampaignExperimentStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignExperimentStatusEnum.
                             * @implements ICampaignExperimentStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentStatusEnum=} [p] Properties to set
                             */
                            function CampaignExperimentStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignExperimentStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentStatusEnum} CampaignExperimentStatusEnum instance
                             */
                            CampaignExperimentStatusEnum.create = function create(properties) {
                                return new CampaignExperimentStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignExperimentStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentStatusEnum} m CampaignExperimentStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignExperimentStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentStatusEnum} message CampaignExperimentStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExperimentStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentStatusEnum} CampaignExperimentStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignExperimentStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExperimentStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentStatusEnum} CampaignExperimentStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExperimentStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExperimentStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExperimentStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentStatusEnum} CampaignExperimentStatusEnum
                             */
                            CampaignExperimentStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignExperimentStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignExperimentStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignExperimentStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignExperimentStatusEnum} m CampaignExperimentStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExperimentStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignExperimentStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExperimentStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignExperimentStatus enum.
                             * @name google.ads.googleads.v11.enums.CampaignExperimentStatusEnum.CampaignExperimentStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INITIALIZING=2 INITIALIZING value
                             * @property {number} INITIALIZATION_FAILED=8 INITIALIZATION_FAILED value
                             * @property {number} ENABLED=3 ENABLED value
                             * @property {number} GRADUATED=4 GRADUATED value
                             * @property {number} REMOVED=5 REMOVED value
                             * @property {number} PROMOTING=6 PROMOTING value
                             * @property {number} PROMOTION_FAILED=9 PROMOTION_FAILED value
                             * @property {number} PROMOTED=7 PROMOTED value
                             * @property {number} ENDED_MANUALLY=10 ENDED_MANUALLY value
                             */
                            CampaignExperimentStatusEnum.CampaignExperimentStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INITIALIZING"] = 2;
                                values[valuesById[8] = "INITIALIZATION_FAILED"] = 8;
                                values[valuesById[3] = "ENABLED"] = 3;
                                values[valuesById[4] = "GRADUATED"] = 4;
                                values[valuesById[5] = "REMOVED"] = 5;
                                values[valuesById[6] = "PROMOTING"] = 6;
                                values[valuesById[9] = "PROMOTION_FAILED"] = 9;
                                values[valuesById[7] = "PROMOTED"] = 7;
                                values[valuesById[10] = "ENDED_MANUALLY"] = 10;
                                return values;
                            })();
    
                            return CampaignExperimentStatusEnum;
                        })();
    
                        enums.CampaignExperimentTrafficSplitTypeEnum = (function() {
    
                            /**
                             * Properties of a CampaignExperimentTrafficSplitTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignExperimentTrafficSplitTypeEnum
                             */
    
                            /**
                             * Constructs a new CampaignExperimentTrafficSplitTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignExperimentTrafficSplitTypeEnum.
                             * @implements ICampaignExperimentTrafficSplitTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTrafficSplitTypeEnum=} [p] Properties to set
                             */
                            function CampaignExperimentTrafficSplitTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignExperimentTrafficSplitTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTrafficSplitTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum} CampaignExperimentTrafficSplitTypeEnum instance
                             */
                            CampaignExperimentTrafficSplitTypeEnum.create = function create(properties) {
                                return new CampaignExperimentTrafficSplitTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentTrafficSplitTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTrafficSplitTypeEnum} m CampaignExperimentTrafficSplitTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentTrafficSplitTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentTrafficSplitTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTrafficSplitTypeEnum} message CampaignExperimentTrafficSplitTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentTrafficSplitTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExperimentTrafficSplitTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum} CampaignExperimentTrafficSplitTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentTrafficSplitTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExperimentTrafficSplitTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum} CampaignExperimentTrafficSplitTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentTrafficSplitTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExperimentTrafficSplitTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExperimentTrafficSplitTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExperimentTrafficSplitTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum} CampaignExperimentTrafficSplitTypeEnum
                             */
                            CampaignExperimentTrafficSplitTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignExperimentTrafficSplitTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum} m CampaignExperimentTrafficSplitTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExperimentTrafficSplitTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignExperimentTrafficSplitTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExperimentTrafficSplitTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignExperimentTrafficSplitType enum.
                             * @name google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum.CampaignExperimentTrafficSplitType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} RANDOM_QUERY=2 RANDOM_QUERY value
                             * @property {number} COOKIE=3 COOKIE value
                             */
                            CampaignExperimentTrafficSplitTypeEnum.CampaignExperimentTrafficSplitType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "RANDOM_QUERY"] = 2;
                                values[valuesById[3] = "COOKIE"] = 3;
                                return values;
                            })();
    
                            return CampaignExperimentTrafficSplitTypeEnum;
                        })();
    
                        enums.CampaignExperimentTypeEnum = (function() {
    
                            /**
                             * Properties of a CampaignExperimentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignExperimentTypeEnum
                             */
    
                            /**
                             * Constructs a new CampaignExperimentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignExperimentTypeEnum.
                             * @implements ICampaignExperimentTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTypeEnum=} [p] Properties to set
                             */
                            function CampaignExperimentTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignExperimentTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTypeEnum} CampaignExperimentTypeEnum instance
                             */
                            CampaignExperimentTypeEnum.create = function create(properties) {
                                return new CampaignExperimentTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignExperimentTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTypeEnum} m CampaignExperimentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignExperimentTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignExperimentTypeEnum} message CampaignExperimentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExperimentTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTypeEnum} CampaignExperimentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignExperimentTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExperimentTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTypeEnum} CampaignExperimentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExperimentTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExperimentTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExperimentTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignExperimentTypeEnum} CampaignExperimentTypeEnum
                             */
                            CampaignExperimentTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignExperimentTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignExperimentTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignExperimentTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignExperimentTypeEnum} m CampaignExperimentTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExperimentTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignExperimentTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignExperimentTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExperimentTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignExperimentType enum.
                             * @name google.ads.googleads.v11.enums.CampaignExperimentTypeEnum.CampaignExperimentType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BASE=2 BASE value
                             * @property {number} DRAFT=3 DRAFT value
                             * @property {number} EXPERIMENT=4 EXPERIMENT value
                             */
                            CampaignExperimentTypeEnum.CampaignExperimentType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BASE"] = 2;
                                values[valuesById[3] = "DRAFT"] = 3;
                                values[valuesById[4] = "EXPERIMENT"] = 4;
                                return values;
                            })();
    
                            return CampaignExperimentTypeEnum;
                        })();
    
                        enums.CampaignGroupStatusEnum = (function() {
    
                            /**
                             * Properties of a CampaignGroupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignGroupStatusEnum
                             */
    
                            /**
                             * Constructs a new CampaignGroupStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignGroupStatusEnum.
                             * @implements ICampaignGroupStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignGroupStatusEnum=} [p] Properties to set
                             */
                            function CampaignGroupStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignGroupStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignGroupStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignGroupStatusEnum} CampaignGroupStatusEnum instance
                             */
                            CampaignGroupStatusEnum.create = function create(properties) {
                                return new CampaignGroupStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignGroupStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignGroupStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignGroupStatusEnum} m CampaignGroupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignGroupStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignGroupStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignGroupStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignGroupStatusEnum} message CampaignGroupStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignGroupStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignGroupStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignGroupStatusEnum} CampaignGroupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignGroupStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignGroupStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignGroupStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignGroupStatusEnum} CampaignGroupStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignGroupStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignGroupStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignGroupStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignGroupStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignGroupStatusEnum} CampaignGroupStatusEnum
                             */
                            CampaignGroupStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignGroupStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignGroupStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignGroupStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignGroupStatusEnum} m CampaignGroupStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignGroupStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignGroupStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignGroupStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignGroupStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignGroupStatus enum.
                             * @name google.ads.googleads.v11.enums.CampaignGroupStatusEnum.CampaignGroupStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CampaignGroupStatusEnum.CampaignGroupStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CampaignGroupStatusEnum;
                        })();
    
                        enums.CampaignServingStatusEnum = (function() {
    
                            /**
                             * Properties of a CampaignServingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignServingStatusEnum
                             */
    
                            /**
                             * Constructs a new CampaignServingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignServingStatusEnum.
                             * @implements ICampaignServingStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignServingStatusEnum=} [p] Properties to set
                             */
                            function CampaignServingStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignServingStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignServingStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignServingStatusEnum} CampaignServingStatusEnum instance
                             */
                            CampaignServingStatusEnum.create = function create(properties) {
                                return new CampaignServingStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignServingStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignServingStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignServingStatusEnum} m CampaignServingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignServingStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignServingStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignServingStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignServingStatusEnum} message CampaignServingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignServingStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignServingStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignServingStatusEnum} CampaignServingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignServingStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignServingStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignServingStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignServingStatusEnum} CampaignServingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignServingStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignServingStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignServingStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignServingStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignServingStatusEnum} CampaignServingStatusEnum
                             */
                            CampaignServingStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignServingStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignServingStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignServingStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignServingStatusEnum} m CampaignServingStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignServingStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignServingStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignServingStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignServingStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignServingStatus enum.
                             * @name google.ads.googleads.v11.enums.CampaignServingStatusEnum.CampaignServingStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SERVING=2 SERVING value
                             * @property {number} NONE=3 NONE value
                             * @property {number} ENDED=4 ENDED value
                             * @property {number} PENDING=5 PENDING value
                             * @property {number} SUSPENDED=6 SUSPENDED value
                             */
                            CampaignServingStatusEnum.CampaignServingStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SERVING"] = 2;
                                values[valuesById[3] = "NONE"] = 3;
                                values[valuesById[4] = "ENDED"] = 4;
                                values[valuesById[5] = "PENDING"] = 5;
                                values[valuesById[6] = "SUSPENDED"] = 6;
                                return values;
                            })();
    
                            return CampaignServingStatusEnum;
                        })();
    
                        enums.CampaignSharedSetStatusEnum = (function() {
    
                            /**
                             * Properties of a CampaignSharedSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignSharedSetStatusEnum
                             */
    
                            /**
                             * Constructs a new CampaignSharedSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignSharedSetStatusEnum.
                             * @implements ICampaignSharedSetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignSharedSetStatusEnum=} [p] Properties to set
                             */
                            function CampaignSharedSetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignSharedSetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignSharedSetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum} CampaignSharedSetStatusEnum instance
                             */
                            CampaignSharedSetStatusEnum.create = function create(properties) {
                                return new CampaignSharedSetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignSharedSetStatusEnum} m CampaignSharedSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignSharedSetStatusEnum} message CampaignSharedSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignSharedSetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum} CampaignSharedSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignSharedSetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum} CampaignSharedSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignSharedSetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignSharedSetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignSharedSetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum} CampaignSharedSetStatusEnum
                             */
                            CampaignSharedSetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignSharedSetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum} m CampaignSharedSetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignSharedSetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignSharedSetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignSharedSetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignSharedSetStatus enum.
                             * @name google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum.CampaignSharedSetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CampaignSharedSetStatusEnum.CampaignSharedSetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CampaignSharedSetStatusEnum;
                        })();
    
                        enums.CampaignStatusEnum = (function() {
    
                            /**
                             * Properties of a CampaignStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICampaignStatusEnum
                             */
    
                            /**
                             * Constructs a new CampaignStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CampaignStatusEnum.
                             * @implements ICampaignStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICampaignStatusEnum=} [p] Properties to set
                             */
                            function CampaignStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CampaignStatusEnum} CampaignStatusEnum instance
                             */
                            CampaignStatusEnum.create = function create(properties) {
                                return new CampaignStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignStatusEnum} m CampaignStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CampaignStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICampaignStatusEnum} message CampaignStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CampaignStatusEnum} CampaignStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CampaignStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CampaignStatusEnum} CampaignStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CampaignStatusEnum} CampaignStatusEnum
                             */
                            CampaignStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CampaignStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CampaignStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CampaignStatusEnum} m CampaignStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CampaignStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignStatus enum.
                             * @name google.ads.googleads.v11.enums.CampaignStatusEnum.CampaignStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} PAUSED=3 PAUSED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            CampaignStatusEnum.CampaignStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "PAUSED"] = 3;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return CampaignStatusEnum;
                        })();
    
                        enums.ChangeClientTypeEnum = (function() {
    
                            /**
                             * Properties of a ChangeClientTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IChangeClientTypeEnum
                             */
    
                            /**
                             * Constructs a new ChangeClientTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ChangeClientTypeEnum.
                             * @implements IChangeClientTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IChangeClientTypeEnum=} [p] Properties to set
                             */
                            function ChangeClientTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ChangeClientTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeClientTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ChangeClientTypeEnum} ChangeClientTypeEnum instance
                             */
                            ChangeClientTypeEnum.create = function create(properties) {
                                return new ChangeClientTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeClientTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeClientTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeClientTypeEnum} m ChangeClientTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeClientTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeClientTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeClientTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeClientTypeEnum} message ChangeClientTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeClientTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeClientTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ChangeClientTypeEnum} ChangeClientTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeClientTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ChangeClientTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeClientTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ChangeClientTypeEnum} ChangeClientTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeClientTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeClientTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeClientTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ChangeClientTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ChangeClientTypeEnum} ChangeClientTypeEnum
                             */
                            ChangeClientTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ChangeClientTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ChangeClientTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ChangeClientTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ChangeClientTypeEnum} m ChangeClientTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeClientTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ChangeClientTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ChangeClientTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeClientTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ChangeClientType enum.
                             * @name google.ads.googleads.v11.enums.ChangeClientTypeEnum.ChangeClientType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} GOOGLE_ADS_WEB_CLIENT=2 GOOGLE_ADS_WEB_CLIENT value
                             * @property {number} GOOGLE_ADS_AUTOMATED_RULE=3 GOOGLE_ADS_AUTOMATED_RULE value
                             * @property {number} GOOGLE_ADS_SCRIPTS=4 GOOGLE_ADS_SCRIPTS value
                             * @property {number} GOOGLE_ADS_BULK_UPLOAD=5 GOOGLE_ADS_BULK_UPLOAD value
                             * @property {number} GOOGLE_ADS_API=6 GOOGLE_ADS_API value
                             * @property {number} GOOGLE_ADS_EDITOR=7 GOOGLE_ADS_EDITOR value
                             * @property {number} GOOGLE_ADS_MOBILE_APP=8 GOOGLE_ADS_MOBILE_APP value
                             * @property {number} GOOGLE_ADS_RECOMMENDATIONS=9 GOOGLE_ADS_RECOMMENDATIONS value
                             * @property {number} SEARCH_ADS_360_SYNC=10 SEARCH_ADS_360_SYNC value
                             * @property {number} SEARCH_ADS_360_POST=11 SEARCH_ADS_360_POST value
                             * @property {number} INTERNAL_TOOL=12 INTERNAL_TOOL value
                             * @property {number} OTHER=13 OTHER value
                             */
                            ChangeClientTypeEnum.ChangeClientType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "GOOGLE_ADS_WEB_CLIENT"] = 2;
                                values[valuesById[3] = "GOOGLE_ADS_AUTOMATED_RULE"] = 3;
                                values[valuesById[4] = "GOOGLE_ADS_SCRIPTS"] = 4;
                                values[valuesById[5] = "GOOGLE_ADS_BULK_UPLOAD"] = 5;
                                values[valuesById[6] = "GOOGLE_ADS_API"] = 6;
                                values[valuesById[7] = "GOOGLE_ADS_EDITOR"] = 7;
                                values[valuesById[8] = "GOOGLE_ADS_MOBILE_APP"] = 8;
                                values[valuesById[9] = "GOOGLE_ADS_RECOMMENDATIONS"] = 9;
                                values[valuesById[10] = "SEARCH_ADS_360_SYNC"] = 10;
                                values[valuesById[11] = "SEARCH_ADS_360_POST"] = 11;
                                values[valuesById[12] = "INTERNAL_TOOL"] = 12;
                                values[valuesById[13] = "OTHER"] = 13;
                                return values;
                            })();
    
                            return ChangeClientTypeEnum;
                        })();
    
                        enums.ChangeEventResourceTypeEnum = (function() {
    
                            /**
                             * Properties of a ChangeEventResourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IChangeEventResourceTypeEnum
                             */
    
                            /**
                             * Constructs a new ChangeEventResourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ChangeEventResourceTypeEnum.
                             * @implements IChangeEventResourceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IChangeEventResourceTypeEnum=} [p] Properties to set
                             */
                            function ChangeEventResourceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ChangeEventResourceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeEventResourceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum} ChangeEventResourceTypeEnum instance
                             */
                            ChangeEventResourceTypeEnum.create = function create(properties) {
                                return new ChangeEventResourceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeEventResourceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeEventResourceTypeEnum} m ChangeEventResourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeEventResourceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeEventResourceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeEventResourceTypeEnum} message ChangeEventResourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeEventResourceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeEventResourceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum} ChangeEventResourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeEventResourceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeEventResourceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum} ChangeEventResourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeEventResourceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeEventResourceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeEventResourceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ChangeEventResourceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum} ChangeEventResourceTypeEnum
                             */
                            ChangeEventResourceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ChangeEventResourceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum} m ChangeEventResourceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeEventResourceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ChangeEventResourceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeEventResourceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ChangeEventResourceType enum.
                             * @name google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum.ChangeEventResourceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD=2 AD value
                             * @property {number} AD_GROUP=3 AD_GROUP value
                             * @property {number} AD_GROUP_CRITERION=4 AD_GROUP_CRITERION value
                             * @property {number} CAMPAIGN=5 CAMPAIGN value
                             * @property {number} CAMPAIGN_BUDGET=6 CAMPAIGN_BUDGET value
                             * @property {number} AD_GROUP_BID_MODIFIER=7 AD_GROUP_BID_MODIFIER value
                             * @property {number} CAMPAIGN_CRITERION=8 CAMPAIGN_CRITERION value
                             * @property {number} FEED=9 FEED value
                             * @property {number} FEED_ITEM=10 FEED_ITEM value
                             * @property {number} CAMPAIGN_FEED=11 CAMPAIGN_FEED value
                             * @property {number} AD_GROUP_FEED=12 AD_GROUP_FEED value
                             * @property {number} AD_GROUP_AD=13 AD_GROUP_AD value
                             * @property {number} ASSET=14 ASSET value
                             * @property {number} CUSTOMER_ASSET=15 CUSTOMER_ASSET value
                             * @property {number} CAMPAIGN_ASSET=16 CAMPAIGN_ASSET value
                             * @property {number} AD_GROUP_ASSET=17 AD_GROUP_ASSET value
                             * @property {number} ASSET_SET=18 ASSET_SET value
                             * @property {number} ASSET_SET_ASSET=19 ASSET_SET_ASSET value
                             * @property {number} CAMPAIGN_ASSET_SET=20 CAMPAIGN_ASSET_SET value
                             */
                            ChangeEventResourceTypeEnum.ChangeEventResourceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD"] = 2;
                                values[valuesById[3] = "AD_GROUP"] = 3;
                                values[valuesById[4] = "AD_GROUP_CRITERION"] = 4;
                                values[valuesById[5] = "CAMPAIGN"] = 5;
                                values[valuesById[6] = "CAMPAIGN_BUDGET"] = 6;
                                values[valuesById[7] = "AD_GROUP_BID_MODIFIER"] = 7;
                                values[valuesById[8] = "CAMPAIGN_CRITERION"] = 8;
                                values[valuesById[9] = "FEED"] = 9;
                                values[valuesById[10] = "FEED_ITEM"] = 10;
                                values[valuesById[11] = "CAMPAIGN_FEED"] = 11;
                                values[valuesById[12] = "AD_GROUP_FEED"] = 12;
                                values[valuesById[13] = "AD_GROUP_AD"] = 13;
                                values[valuesById[14] = "ASSET"] = 14;
                                values[valuesById[15] = "CUSTOMER_ASSET"] = 15;
                                values[valuesById[16] = "CAMPAIGN_ASSET"] = 16;
                                values[valuesById[17] = "AD_GROUP_ASSET"] = 17;
                                values[valuesById[18] = "ASSET_SET"] = 18;
                                values[valuesById[19] = "ASSET_SET_ASSET"] = 19;
                                values[valuesById[20] = "CAMPAIGN_ASSET_SET"] = 20;
                                return values;
                            })();
    
                            return ChangeEventResourceTypeEnum;
                        })();
    
                        enums.ChangeStatusOperationEnum = (function() {
    
                            /**
                             * Properties of a ChangeStatusOperationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IChangeStatusOperationEnum
                             */
    
                            /**
                             * Constructs a new ChangeStatusOperationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ChangeStatusOperationEnum.
                             * @implements IChangeStatusOperationEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IChangeStatusOperationEnum=} [p] Properties to set
                             */
                            function ChangeStatusOperationEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ChangeStatusOperationEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeStatusOperationEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusOperationEnum} ChangeStatusOperationEnum instance
                             */
                            ChangeStatusOperationEnum.create = function create(properties) {
                                return new ChangeStatusOperationEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeStatusOperationEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeStatusOperationEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeStatusOperationEnum} m ChangeStatusOperationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatusOperationEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeStatusOperationEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeStatusOperationEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeStatusOperationEnum} message ChangeStatusOperationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatusOperationEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeStatusOperationEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusOperationEnum} ChangeStatusOperationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatusOperationEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ChangeStatusOperationEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeStatusOperationEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusOperationEnum} ChangeStatusOperationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatusOperationEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeStatusOperationEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeStatusOperationEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ChangeStatusOperationEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusOperationEnum} ChangeStatusOperationEnum
                             */
                            ChangeStatusOperationEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ChangeStatusOperationEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ChangeStatusOperationEnum();
                            };
    
                            /**
                             * Creates a plain object from a ChangeStatusOperationEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ChangeStatusOperationEnum} m ChangeStatusOperationEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeStatusOperationEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ChangeStatusOperationEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusOperationEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeStatusOperationEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ChangeStatusOperation enum.
                             * @name google.ads.googleads.v11.enums.ChangeStatusOperationEnum.ChangeStatusOperation
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADDED=2 ADDED value
                             * @property {number} CHANGED=3 CHANGED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            ChangeStatusOperationEnum.ChangeStatusOperation = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADDED"] = 2;
                                values[valuesById[3] = "CHANGED"] = 3;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return ChangeStatusOperationEnum;
                        })();
    
                        enums.ChangeStatusResourceTypeEnum = (function() {
    
                            /**
                             * Properties of a ChangeStatusResourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IChangeStatusResourceTypeEnum
                             */
    
                            /**
                             * Constructs a new ChangeStatusResourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ChangeStatusResourceTypeEnum.
                             * @implements IChangeStatusResourceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IChangeStatusResourceTypeEnum=} [p] Properties to set
                             */
                            function ChangeStatusResourceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ChangeStatusResourceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeStatusResourceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum} ChangeStatusResourceTypeEnum instance
                             */
                            ChangeStatusResourceTypeEnum.create = function create(properties) {
                                return new ChangeStatusResourceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeStatusResourceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeStatusResourceTypeEnum} m ChangeStatusResourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatusResourceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeStatusResourceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IChangeStatusResourceTypeEnum} message ChangeStatusResourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatusResourceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeStatusResourceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum} ChangeStatusResourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatusResourceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeStatusResourceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum} ChangeStatusResourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatusResourceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeStatusResourceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeStatusResourceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ChangeStatusResourceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum} ChangeStatusResourceTypeEnum
                             */
                            ChangeStatusResourceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ChangeStatusResourceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum} m ChangeStatusResourceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeStatusResourceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ChangeStatusResourceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeStatusResourceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ChangeStatusResourceType enum.
                             * @name google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum.ChangeStatusResourceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_GROUP=3 AD_GROUP value
                             * @property {number} AD_GROUP_AD=4 AD_GROUP_AD value
                             * @property {number} AD_GROUP_CRITERION=5 AD_GROUP_CRITERION value
                             * @property {number} CAMPAIGN=6 CAMPAIGN value
                             * @property {number} CAMPAIGN_CRITERION=7 CAMPAIGN_CRITERION value
                             * @property {number} FEED=9 FEED value
                             * @property {number} FEED_ITEM=10 FEED_ITEM value
                             * @property {number} AD_GROUP_FEED=11 AD_GROUP_FEED value
                             * @property {number} CAMPAIGN_FEED=12 CAMPAIGN_FEED value
                             * @property {number} AD_GROUP_BID_MODIFIER=13 AD_GROUP_BID_MODIFIER value
                             * @property {number} SHARED_SET=14 SHARED_SET value
                             * @property {number} CAMPAIGN_SHARED_SET=15 CAMPAIGN_SHARED_SET value
                             * @property {number} ASSET=16 ASSET value
                             * @property {number} CUSTOMER_ASSET=17 CUSTOMER_ASSET value
                             * @property {number} CAMPAIGN_ASSET=18 CAMPAIGN_ASSET value
                             * @property {number} AD_GROUP_ASSET=19 AD_GROUP_ASSET value
                             */
                            ChangeStatusResourceTypeEnum.ChangeStatusResourceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "AD_GROUP"] = 3;
                                values[valuesById[4] = "AD_GROUP_AD"] = 4;
                                values[valuesById[5] = "AD_GROUP_CRITERION"] = 5;
                                values[valuesById[6] = "CAMPAIGN"] = 6;
                                values[valuesById[7] = "CAMPAIGN_CRITERION"] = 7;
                                values[valuesById[9] = "FEED"] = 9;
                                values[valuesById[10] = "FEED_ITEM"] = 10;
                                values[valuesById[11] = "AD_GROUP_FEED"] = 11;
                                values[valuesById[12] = "CAMPAIGN_FEED"] = 12;
                                values[valuesById[13] = "AD_GROUP_BID_MODIFIER"] = 13;
                                values[valuesById[14] = "SHARED_SET"] = 14;
                                values[valuesById[15] = "CAMPAIGN_SHARED_SET"] = 15;
                                values[valuesById[16] = "ASSET"] = 16;
                                values[valuesById[17] = "CUSTOMER_ASSET"] = 17;
                                values[valuesById[18] = "CAMPAIGN_ASSET"] = 18;
                                values[valuesById[19] = "AD_GROUP_ASSET"] = 19;
                                return values;
                            })();
    
                            return ChangeStatusResourceTypeEnum;
                        })();
    
                        enums.CombinedAudienceStatusEnum = (function() {
    
                            /**
                             * Properties of a CombinedAudienceStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICombinedAudienceStatusEnum
                             */
    
                            /**
                             * Constructs a new CombinedAudienceStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CombinedAudienceStatusEnum.
                             * @implements ICombinedAudienceStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICombinedAudienceStatusEnum=} [p] Properties to set
                             */
                            function CombinedAudienceStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CombinedAudienceStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICombinedAudienceStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CombinedAudienceStatusEnum} CombinedAudienceStatusEnum instance
                             */
                            CombinedAudienceStatusEnum.create = function create(properties) {
                                return new CombinedAudienceStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CombinedAudienceStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CombinedAudienceStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICombinedAudienceStatusEnum} m CombinedAudienceStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedAudienceStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CombinedAudienceStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CombinedAudienceStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICombinedAudienceStatusEnum} message CombinedAudienceStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedAudienceStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CombinedAudienceStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CombinedAudienceStatusEnum} CombinedAudienceStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedAudienceStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CombinedAudienceStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CombinedAudienceStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CombinedAudienceStatusEnum} CombinedAudienceStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedAudienceStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CombinedAudienceStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CombinedAudienceStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CombinedAudienceStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CombinedAudienceStatusEnum} CombinedAudienceStatusEnum
                             */
                            CombinedAudienceStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CombinedAudienceStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CombinedAudienceStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CombinedAudienceStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CombinedAudienceStatusEnum} m CombinedAudienceStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CombinedAudienceStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CombinedAudienceStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CombinedAudienceStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CombinedAudienceStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CombinedAudienceStatus enum.
                             * @name google.ads.googleads.v11.enums.CombinedAudienceStatusEnum.CombinedAudienceStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CombinedAudienceStatusEnum.CombinedAudienceStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CombinedAudienceStatusEnum;
                        })();
    
                        enums.ConversionActionCountingTypeEnum = (function() {
    
                            /**
                             * Properties of a ConversionActionCountingTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionActionCountingTypeEnum
                             */
    
                            /**
                             * Constructs a new ConversionActionCountingTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionActionCountingTypeEnum.
                             * @implements IConversionActionCountingTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionActionCountingTypeEnum=} [p] Properties to set
                             */
                            function ConversionActionCountingTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionActionCountingTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionCountingTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum} ConversionActionCountingTypeEnum instance
                             */
                            ConversionActionCountingTypeEnum.create = function create(properties) {
                                return new ConversionActionCountingTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionActionCountingTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionCountingTypeEnum} m ConversionActionCountingTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionCountingTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionActionCountingTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionCountingTypeEnum} message ConversionActionCountingTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionCountingTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionActionCountingTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum} ConversionActionCountingTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionCountingTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionActionCountingTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum} ConversionActionCountingTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionCountingTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionActionCountingTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionActionCountingTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionActionCountingTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum} ConversionActionCountingTypeEnum
                             */
                            ConversionActionCountingTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionActionCountingTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum} m ConversionActionCountingTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionActionCountingTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionActionCountingTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionActionCountingTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionActionCountingType enum.
                             * @name google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum.ConversionActionCountingType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ONE_PER_CLICK=2 ONE_PER_CLICK value
                             * @property {number} MANY_PER_CLICK=3 MANY_PER_CLICK value
                             */
                            ConversionActionCountingTypeEnum.ConversionActionCountingType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ONE_PER_CLICK"] = 2;
                                values[valuesById[3] = "MANY_PER_CLICK"] = 3;
                                return values;
                            })();
    
                            return ConversionActionCountingTypeEnum;
                        })();
    
                        enums.ConversionActionStatusEnum = (function() {
    
                            /**
                             * Properties of a ConversionActionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionActionStatusEnum
                             */
    
                            /**
                             * Constructs a new ConversionActionStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionActionStatusEnum.
                             * @implements IConversionActionStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionActionStatusEnum=} [p] Properties to set
                             */
                            function ConversionActionStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionActionStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionActionStatusEnum} ConversionActionStatusEnum instance
                             */
                            ConversionActionStatusEnum.create = function create(properties) {
                                return new ConversionActionStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionActionStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionStatusEnum} m ConversionActionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionActionStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionStatusEnum} message ConversionActionStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionActionStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionActionStatusEnum} ConversionActionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionActionStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionActionStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionActionStatusEnum} ConversionActionStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionActionStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionActionStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionActionStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionActionStatusEnum} ConversionActionStatusEnum
                             */
                            ConversionActionStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionActionStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionActionStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionActionStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionActionStatusEnum} m ConversionActionStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionActionStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionActionStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionActionStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionActionStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionActionStatus enum.
                             * @name google.ads.googleads.v11.enums.ConversionActionStatusEnum.ConversionActionStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             * @property {number} HIDDEN=4 HIDDEN value
                             */
                            ConversionActionStatusEnum.ConversionActionStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                values[valuesById[4] = "HIDDEN"] = 4;
                                return values;
                            })();
    
                            return ConversionActionStatusEnum;
                        })();
    
                        enums.ConversionActionTypeEnum = (function() {
    
                            /**
                             * Properties of a ConversionActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionActionTypeEnum
                             */
    
                            /**
                             * Constructs a new ConversionActionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionActionTypeEnum.
                             * @implements IConversionActionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionActionTypeEnum=} [p] Properties to set
                             */
                            function ConversionActionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionActionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionActionTypeEnum} ConversionActionTypeEnum instance
                             */
                            ConversionActionTypeEnum.create = function create(properties) {
                                return new ConversionActionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionActionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionTypeEnum} m ConversionActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionActionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionActionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionActionTypeEnum} message ConversionActionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionActionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionActionTypeEnum} ConversionActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionActionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionActionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionActionTypeEnum} ConversionActionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionActionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionActionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionActionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionActionTypeEnum} ConversionActionTypeEnum
                             */
                            ConversionActionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionActionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionActionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionActionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionActionTypeEnum} m ConversionActionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionActionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionActionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionActionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionActionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionActionType enum.
                             * @name google.ads.googleads.v11.enums.ConversionActionTypeEnum.ConversionActionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_CALL=2 AD_CALL value
                             * @property {number} CLICK_TO_CALL=3 CLICK_TO_CALL value
                             * @property {number} GOOGLE_PLAY_DOWNLOAD=4 GOOGLE_PLAY_DOWNLOAD value
                             * @property {number} GOOGLE_PLAY_IN_APP_PURCHASE=5 GOOGLE_PLAY_IN_APP_PURCHASE value
                             * @property {number} UPLOAD_CALLS=6 UPLOAD_CALLS value
                             * @property {number} UPLOAD_CLICKS=7 UPLOAD_CLICKS value
                             * @property {number} WEBPAGE=8 WEBPAGE value
                             * @property {number} WEBSITE_CALL=9 WEBSITE_CALL value
                             * @property {number} STORE_SALES_DIRECT_UPLOAD=10 STORE_SALES_DIRECT_UPLOAD value
                             * @property {number} STORE_SALES=11 STORE_SALES value
                             * @property {number} FIREBASE_ANDROID_FIRST_OPEN=12 FIREBASE_ANDROID_FIRST_OPEN value
                             * @property {number} FIREBASE_ANDROID_IN_APP_PURCHASE=13 FIREBASE_ANDROID_IN_APP_PURCHASE value
                             * @property {number} FIREBASE_ANDROID_CUSTOM=14 FIREBASE_ANDROID_CUSTOM value
                             * @property {number} FIREBASE_IOS_FIRST_OPEN=15 FIREBASE_IOS_FIRST_OPEN value
                             * @property {number} FIREBASE_IOS_IN_APP_PURCHASE=16 FIREBASE_IOS_IN_APP_PURCHASE value
                             * @property {number} FIREBASE_IOS_CUSTOM=17 FIREBASE_IOS_CUSTOM value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS_ANDROID_FIRST_OPEN=18 THIRD_PARTY_APP_ANALYTICS_ANDROID_FIRST_OPEN value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS_ANDROID_IN_APP_PURCHASE=19 THIRD_PARTY_APP_ANALYTICS_ANDROID_IN_APP_PURCHASE value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS_ANDROID_CUSTOM=20 THIRD_PARTY_APP_ANALYTICS_ANDROID_CUSTOM value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS_IOS_FIRST_OPEN=21 THIRD_PARTY_APP_ANALYTICS_IOS_FIRST_OPEN value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS_IOS_IN_APP_PURCHASE=22 THIRD_PARTY_APP_ANALYTICS_IOS_IN_APP_PURCHASE value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS_IOS_CUSTOM=23 THIRD_PARTY_APP_ANALYTICS_IOS_CUSTOM value
                             * @property {number} ANDROID_APP_PRE_REGISTRATION=24 ANDROID_APP_PRE_REGISTRATION value
                             * @property {number} ANDROID_INSTALLS_ALL_OTHER_APPS=25 ANDROID_INSTALLS_ALL_OTHER_APPS value
                             * @property {number} FLOODLIGHT_ACTION=26 FLOODLIGHT_ACTION value
                             * @property {number} FLOODLIGHT_TRANSACTION=27 FLOODLIGHT_TRANSACTION value
                             * @property {number} GOOGLE_HOSTED=28 GOOGLE_HOSTED value
                             * @property {number} LEAD_FORM_SUBMIT=29 LEAD_FORM_SUBMIT value
                             * @property {number} SALESFORCE=30 SALESFORCE value
                             * @property {number} SEARCH_ADS_360=31 SEARCH_ADS_360 value
                             * @property {number} SMART_CAMPAIGN_AD_CLICKS_TO_CALL=32 SMART_CAMPAIGN_AD_CLICKS_TO_CALL value
                             * @property {number} SMART_CAMPAIGN_MAP_CLICKS_TO_CALL=33 SMART_CAMPAIGN_MAP_CLICKS_TO_CALL value
                             * @property {number} SMART_CAMPAIGN_MAP_DIRECTIONS=34 SMART_CAMPAIGN_MAP_DIRECTIONS value
                             * @property {number} SMART_CAMPAIGN_TRACKED_CALLS=35 SMART_CAMPAIGN_TRACKED_CALLS value
                             * @property {number} STORE_VISITS=36 STORE_VISITS value
                             */
                            ConversionActionTypeEnum.ConversionActionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_CALL"] = 2;
                                values[valuesById[3] = "CLICK_TO_CALL"] = 3;
                                values[valuesById[4] = "GOOGLE_PLAY_DOWNLOAD"] = 4;
                                values[valuesById[5] = "GOOGLE_PLAY_IN_APP_PURCHASE"] = 5;
                                values[valuesById[6] = "UPLOAD_CALLS"] = 6;
                                values[valuesById[7] = "UPLOAD_CLICKS"] = 7;
                                values[valuesById[8] = "WEBPAGE"] = 8;
                                values[valuesById[9] = "WEBSITE_CALL"] = 9;
                                values[valuesById[10] = "STORE_SALES_DIRECT_UPLOAD"] = 10;
                                values[valuesById[11] = "STORE_SALES"] = 11;
                                values[valuesById[12] = "FIREBASE_ANDROID_FIRST_OPEN"] = 12;
                                values[valuesById[13] = "FIREBASE_ANDROID_IN_APP_PURCHASE"] = 13;
                                values[valuesById[14] = "FIREBASE_ANDROID_CUSTOM"] = 14;
                                values[valuesById[15] = "FIREBASE_IOS_FIRST_OPEN"] = 15;
                                values[valuesById[16] = "FIREBASE_IOS_IN_APP_PURCHASE"] = 16;
                                values[valuesById[17] = "FIREBASE_IOS_CUSTOM"] = 17;
                                values[valuesById[18] = "THIRD_PARTY_APP_ANALYTICS_ANDROID_FIRST_OPEN"] = 18;
                                values[valuesById[19] = "THIRD_PARTY_APP_ANALYTICS_ANDROID_IN_APP_PURCHASE"] = 19;
                                values[valuesById[20] = "THIRD_PARTY_APP_ANALYTICS_ANDROID_CUSTOM"] = 20;
                                values[valuesById[21] = "THIRD_PARTY_APP_ANALYTICS_IOS_FIRST_OPEN"] = 21;
                                values[valuesById[22] = "THIRD_PARTY_APP_ANALYTICS_IOS_IN_APP_PURCHASE"] = 22;
                                values[valuesById[23] = "THIRD_PARTY_APP_ANALYTICS_IOS_CUSTOM"] = 23;
                                values[valuesById[24] = "ANDROID_APP_PRE_REGISTRATION"] = 24;
                                values[valuesById[25] = "ANDROID_INSTALLS_ALL_OTHER_APPS"] = 25;
                                values[valuesById[26] = "FLOODLIGHT_ACTION"] = 26;
                                values[valuesById[27] = "FLOODLIGHT_TRANSACTION"] = 27;
                                values[valuesById[28] = "GOOGLE_HOSTED"] = 28;
                                values[valuesById[29] = "LEAD_FORM_SUBMIT"] = 29;
                                values[valuesById[30] = "SALESFORCE"] = 30;
                                values[valuesById[31] = "SEARCH_ADS_360"] = 31;
                                values[valuesById[32] = "SMART_CAMPAIGN_AD_CLICKS_TO_CALL"] = 32;
                                values[valuesById[33] = "SMART_CAMPAIGN_MAP_CLICKS_TO_CALL"] = 33;
                                values[valuesById[34] = "SMART_CAMPAIGN_MAP_DIRECTIONS"] = 34;
                                values[valuesById[35] = "SMART_CAMPAIGN_TRACKED_CALLS"] = 35;
                                values[valuesById[36] = "STORE_VISITS"] = 36;
                                return values;
                            })();
    
                            return ConversionActionTypeEnum;
                        })();
    
                        enums.ConversionAdjustmentTypeEnum = (function() {
    
                            /**
                             * Properties of a ConversionAdjustmentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionAdjustmentTypeEnum
                             */
    
                            /**
                             * Constructs a new ConversionAdjustmentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionAdjustmentTypeEnum.
                             * @implements IConversionAdjustmentTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionAdjustmentTypeEnum=} [p] Properties to set
                             */
                            function ConversionAdjustmentTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionAdjustmentTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionAdjustmentTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum} ConversionAdjustmentTypeEnum instance
                             */
                            ConversionAdjustmentTypeEnum.create = function create(properties) {
                                return new ConversionAdjustmentTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustmentTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionAdjustmentTypeEnum} m ConversionAdjustmentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustmentTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustmentTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionAdjustmentTypeEnum} message ConversionAdjustmentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustmentTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionAdjustmentTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum} ConversionAdjustmentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustmentTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionAdjustmentTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum} ConversionAdjustmentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustmentTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionAdjustmentTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionAdjustmentTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionAdjustmentTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum} ConversionAdjustmentTypeEnum
                             */
                            ConversionAdjustmentTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionAdjustmentTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum} m ConversionAdjustmentTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionAdjustmentTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionAdjustmentTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionAdjustmentTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionAdjustmentType enum.
                             * @name google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.ConversionAdjustmentType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} RETRACTION=2 RETRACTION value
                             * @property {number} RESTATEMENT=3 RESTATEMENT value
                             * @property {number} ENHANCEMENT=4 ENHANCEMENT value
                             */
                            ConversionAdjustmentTypeEnum.ConversionAdjustmentType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "RETRACTION"] = 2;
                                values[valuesById[3] = "RESTATEMENT"] = 3;
                                values[valuesById[4] = "ENHANCEMENT"] = 4;
                                return values;
                            })();
    
                            return ConversionAdjustmentTypeEnum;
                        })();
    
                        enums.ConversionCustomVariableStatusEnum = (function() {
    
                            /**
                             * Properties of a ConversionCustomVariableStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionCustomVariableStatusEnum
                             */
    
                            /**
                             * Constructs a new ConversionCustomVariableStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionCustomVariableStatusEnum.
                             * @implements IConversionCustomVariableStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionCustomVariableStatusEnum=} [p] Properties to set
                             */
                            function ConversionCustomVariableStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionCustomVariableStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionCustomVariableStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum} ConversionCustomVariableStatusEnum instance
                             */
                            ConversionCustomVariableStatusEnum.create = function create(properties) {
                                return new ConversionCustomVariableStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariableStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionCustomVariableStatusEnum} m ConversionCustomVariableStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariableStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariableStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionCustomVariableStatusEnum} message ConversionCustomVariableStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariableStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionCustomVariableStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum} ConversionCustomVariableStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariableStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionCustomVariableStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum} ConversionCustomVariableStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariableStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionCustomVariableStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionCustomVariableStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionCustomVariableStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum} ConversionCustomVariableStatusEnum
                             */
                            ConversionCustomVariableStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionCustomVariableStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum} m ConversionCustomVariableStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionCustomVariableStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionCustomVariableStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionCustomVariableStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionCustomVariableStatus enum.
                             * @name google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ACTIVATION_NEEDED=2 ACTIVATION_NEEDED value
                             * @property {number} ENABLED=3 ENABLED value
                             * @property {number} PAUSED=4 PAUSED value
                             */
                            ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ACTIVATION_NEEDED"] = 2;
                                values[valuesById[3] = "ENABLED"] = 3;
                                values[valuesById[4] = "PAUSED"] = 4;
                                return values;
                            })();
    
                            return ConversionCustomVariableStatusEnum;
                        })();
    
                        enums.ConversionEnvironmentEnum = (function() {
    
                            /**
                             * Properties of a ConversionEnvironmentEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionEnvironmentEnum
                             */
    
                            /**
                             * Constructs a new ConversionEnvironmentEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionEnvironmentEnum.
                             * @implements IConversionEnvironmentEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionEnvironmentEnum=} [p] Properties to set
                             */
                            function ConversionEnvironmentEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionEnvironmentEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionEnvironmentEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionEnvironmentEnum} ConversionEnvironmentEnum instance
                             */
                            ConversionEnvironmentEnum.create = function create(properties) {
                                return new ConversionEnvironmentEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionEnvironmentEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionEnvironmentEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionEnvironmentEnum} m ConversionEnvironmentEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionEnvironmentEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionEnvironmentEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionEnvironmentEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionEnvironmentEnum} message ConversionEnvironmentEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionEnvironmentEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionEnvironmentEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionEnvironmentEnum} ConversionEnvironmentEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionEnvironmentEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionEnvironmentEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionEnvironmentEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionEnvironmentEnum} ConversionEnvironmentEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionEnvironmentEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionEnvironmentEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionEnvironmentEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionEnvironmentEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionEnvironmentEnum} ConversionEnvironmentEnum
                             */
                            ConversionEnvironmentEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionEnvironmentEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionEnvironmentEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionEnvironmentEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionEnvironmentEnum} m ConversionEnvironmentEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionEnvironmentEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionEnvironmentEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionEnvironmentEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionEnvironmentEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionEnvironment enum.
                             * @name google.ads.googleads.v11.enums.ConversionEnvironmentEnum.ConversionEnvironment
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APP=2 APP value
                             * @property {number} WEB=3 WEB value
                             */
                            ConversionEnvironmentEnum.ConversionEnvironment = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APP"] = 2;
                                values[valuesById[3] = "WEB"] = 3;
                                return values;
                            })();
    
                            return ConversionEnvironmentEnum;
                        })();
    
                        enums.ConversionOriginEnum = (function() {
    
                            /**
                             * Properties of a ConversionOriginEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionOriginEnum
                             */
    
                            /**
                             * Constructs a new ConversionOriginEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionOriginEnum.
                             * @implements IConversionOriginEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionOriginEnum=} [p] Properties to set
                             */
                            function ConversionOriginEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionOriginEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionOriginEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionOriginEnum} ConversionOriginEnum instance
                             */
                            ConversionOriginEnum.create = function create(properties) {
                                return new ConversionOriginEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionOriginEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionOriginEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionOriginEnum} m ConversionOriginEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionOriginEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionOriginEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionOriginEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionOriginEnum} message ConversionOriginEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionOriginEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionOriginEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionOriginEnum} ConversionOriginEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionOriginEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionOriginEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionOriginEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionOriginEnum} ConversionOriginEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionOriginEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionOriginEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionOriginEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionOriginEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionOriginEnum} ConversionOriginEnum
                             */
                            ConversionOriginEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionOriginEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionOriginEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionOriginEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionOriginEnum} m ConversionOriginEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionOriginEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionOriginEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionOriginEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionOriginEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionOrigin enum.
                             * @name google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} WEBSITE=2 WEBSITE value
                             * @property {number} GOOGLE_HOSTED=3 GOOGLE_HOSTED value
                             * @property {number} APP=4 APP value
                             * @property {number} CALL_FROM_ADS=5 CALL_FROM_ADS value
                             * @property {number} STORE=6 STORE value
                             * @property {number} YOUTUBE_HOSTED=7 YOUTUBE_HOSTED value
                             */
                            ConversionOriginEnum.ConversionOrigin = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "WEBSITE"] = 2;
                                values[valuesById[3] = "GOOGLE_HOSTED"] = 3;
                                values[valuesById[4] = "APP"] = 4;
                                values[valuesById[5] = "CALL_FROM_ADS"] = 5;
                                values[valuesById[6] = "STORE"] = 6;
                                values[valuesById[7] = "YOUTUBE_HOSTED"] = 7;
                                return values;
                            })();
    
                            return ConversionOriginEnum;
                        })();
    
                        enums.ConversionTrackingStatusEnum = (function() {
    
                            /**
                             * Properties of a ConversionTrackingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionTrackingStatusEnum
                             */
    
                            /**
                             * Constructs a new ConversionTrackingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionTrackingStatusEnum.
                             * @implements IConversionTrackingStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionTrackingStatusEnum=} [p] Properties to set
                             */
                            function ConversionTrackingStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionTrackingStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionTrackingStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionTrackingStatusEnum} ConversionTrackingStatusEnum instance
                             */
                            ConversionTrackingStatusEnum.create = function create(properties) {
                                return new ConversionTrackingStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionTrackingStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionTrackingStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionTrackingStatusEnum} m ConversionTrackingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionTrackingStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionTrackingStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionTrackingStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionTrackingStatusEnum} message ConversionTrackingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionTrackingStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionTrackingStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionTrackingStatusEnum} ConversionTrackingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionTrackingStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionTrackingStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionTrackingStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionTrackingStatusEnum} ConversionTrackingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionTrackingStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionTrackingStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionTrackingStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionTrackingStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionTrackingStatusEnum} ConversionTrackingStatusEnum
                             */
                            ConversionTrackingStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionTrackingStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionTrackingStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionTrackingStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionTrackingStatusEnum} m ConversionTrackingStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionTrackingStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionTrackingStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionTrackingStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionTrackingStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionTrackingStatus enum.
                             * @name google.ads.googleads.v11.enums.ConversionTrackingStatusEnum.ConversionTrackingStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOT_CONVERSION_TRACKED=2 NOT_CONVERSION_TRACKED value
                             * @property {number} CONVERSION_TRACKING_MANAGED_BY_SELF=3 CONVERSION_TRACKING_MANAGED_BY_SELF value
                             * @property {number} CONVERSION_TRACKING_MANAGED_BY_THIS_MANAGER=4 CONVERSION_TRACKING_MANAGED_BY_THIS_MANAGER value
                             * @property {number} CONVERSION_TRACKING_MANAGED_BY_ANOTHER_MANAGER=5 CONVERSION_TRACKING_MANAGED_BY_ANOTHER_MANAGER value
                             */
                            ConversionTrackingStatusEnum.ConversionTrackingStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOT_CONVERSION_TRACKED"] = 2;
                                values[valuesById[3] = "CONVERSION_TRACKING_MANAGED_BY_SELF"] = 3;
                                values[valuesById[4] = "CONVERSION_TRACKING_MANAGED_BY_THIS_MANAGER"] = 4;
                                values[valuesById[5] = "CONVERSION_TRACKING_MANAGED_BY_ANOTHER_MANAGER"] = 5;
                                return values;
                            })();
    
                            return ConversionTrackingStatusEnum;
                        })();
    
                        enums.ConversionValueRuleSetStatusEnum = (function() {
    
                            /**
                             * Properties of a ConversionValueRuleSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionValueRuleSetStatusEnum
                             */
    
                            /**
                             * Constructs a new ConversionValueRuleSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionValueRuleSetStatusEnum.
                             * @implements IConversionValueRuleSetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleSetStatusEnum=} [p] Properties to set
                             */
                            function ConversionValueRuleSetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionValueRuleSetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleSetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum} ConversionValueRuleSetStatusEnum instance
                             */
                            ConversionValueRuleSetStatusEnum.create = function create(properties) {
                                return new ConversionValueRuleSetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleSetStatusEnum} m ConversionValueRuleSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleSetStatusEnum} message ConversionValueRuleSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum} ConversionValueRuleSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum} ConversionValueRuleSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRuleSetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRuleSetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRuleSetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum} ConversionValueRuleSetStatusEnum
                             */
                            ConversionValueRuleSetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRuleSetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum} m ConversionValueRuleSetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRuleSetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionValueRuleSetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRuleSetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionValueRuleSetStatus enum.
                             * @name google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             * @property {number} PAUSED=4 PAUSED value
                             */
                            ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                values[valuesById[4] = "PAUSED"] = 4;
                                return values;
                            })();
    
                            return ConversionValueRuleSetStatusEnum;
                        })();
    
                        enums.ConversionValueRuleStatusEnum = (function() {
    
                            /**
                             * Properties of a ConversionValueRuleStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IConversionValueRuleStatusEnum
                             */
    
                            /**
                             * Constructs a new ConversionValueRuleStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ConversionValueRuleStatusEnum.
                             * @implements IConversionValueRuleStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleStatusEnum=} [p] Properties to set
                             */
                            function ConversionValueRuleStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionValueRuleStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum} ConversionValueRuleStatusEnum instance
                             */
                            ConversionValueRuleStatusEnum.create = function create(properties) {
                                return new ConversionValueRuleStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleStatusEnum} m ConversionValueRuleStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IConversionValueRuleStatusEnum} message ConversionValueRuleStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRuleStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum} ConversionValueRuleStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRuleStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum} ConversionValueRuleStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRuleStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRuleStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRuleStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum} ConversionValueRuleStatusEnum
                             */
                            ConversionValueRuleStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRuleStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum} m ConversionValueRuleStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRuleStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionValueRuleStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRuleStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionValueRuleStatus enum.
                             * @name google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum.ConversionValueRuleStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             * @property {number} PAUSED=4 PAUSED value
                             */
                            ConversionValueRuleStatusEnum.ConversionValueRuleStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                values[valuesById[4] = "PAUSED"] = 4;
                                return values;
                            })();
    
                            return ConversionValueRuleStatusEnum;
                        })();
    
                        enums.CriterionSystemServingStatusEnum = (function() {
    
                            /**
                             * Properties of a CriterionSystemServingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICriterionSystemServingStatusEnum
                             */
    
                            /**
                             * Constructs a new CriterionSystemServingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CriterionSystemServingStatusEnum.
                             * @implements ICriterionSystemServingStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICriterionSystemServingStatusEnum=} [p] Properties to set
                             */
                            function CriterionSystemServingStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CriterionSystemServingStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionSystemServingStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum} CriterionSystemServingStatusEnum instance
                             */
                            CriterionSystemServingStatusEnum.create = function create(properties) {
                                return new CriterionSystemServingStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionSystemServingStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionSystemServingStatusEnum} m CriterionSystemServingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionSystemServingStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionSystemServingStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionSystemServingStatusEnum} message CriterionSystemServingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionSystemServingStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionSystemServingStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum} CriterionSystemServingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionSystemServingStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionSystemServingStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum} CriterionSystemServingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionSystemServingStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionSystemServingStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionSystemServingStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CriterionSystemServingStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum} CriterionSystemServingStatusEnum
                             */
                            CriterionSystemServingStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CriterionSystemServingStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum} m CriterionSystemServingStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionSystemServingStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CriterionSystemServingStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionSystemServingStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CriterionSystemServingStatus enum.
                             * @name google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum.CriterionSystemServingStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ELIGIBLE=2 ELIGIBLE value
                             * @property {number} RARELY_SERVED=3 RARELY_SERVED value
                             */
                            CriterionSystemServingStatusEnum.CriterionSystemServingStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ELIGIBLE"] = 2;
                                values[valuesById[3] = "RARELY_SERVED"] = 3;
                                return values;
                            })();
    
                            return CriterionSystemServingStatusEnum;
                        })();
    
                        enums.CriterionTypeEnum = (function() {
    
                            /**
                             * Properties of a CriterionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICriterionTypeEnum
                             */
    
                            /**
                             * Constructs a new CriterionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CriterionTypeEnum.
                             * @implements ICriterionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICriterionTypeEnum=} [p] Properties to set
                             */
                            function CriterionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CriterionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CriterionTypeEnum} CriterionTypeEnum instance
                             */
                            CriterionTypeEnum.create = function create(properties) {
                                return new CriterionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionTypeEnum} m CriterionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CriterionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICriterionTypeEnum} message CriterionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CriterionTypeEnum} CriterionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CriterionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CriterionTypeEnum} CriterionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CriterionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CriterionTypeEnum} CriterionTypeEnum
                             */
                            CriterionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CriterionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CriterionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CriterionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CriterionTypeEnum} m CriterionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CriterionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CriterionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CriterionType enum.
                             * @name google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} KEYWORD=2 KEYWORD value
                             * @property {number} PLACEMENT=3 PLACEMENT value
                             * @property {number} MOBILE_APP_CATEGORY=4 MOBILE_APP_CATEGORY value
                             * @property {number} MOBILE_APPLICATION=5 MOBILE_APPLICATION value
                             * @property {number} DEVICE=6 DEVICE value
                             * @property {number} LOCATION=7 LOCATION value
                             * @property {number} LISTING_GROUP=8 LISTING_GROUP value
                             * @property {number} AD_SCHEDULE=9 AD_SCHEDULE value
                             * @property {number} AGE_RANGE=10 AGE_RANGE value
                             * @property {number} GENDER=11 GENDER value
                             * @property {number} INCOME_RANGE=12 INCOME_RANGE value
                             * @property {number} PARENTAL_STATUS=13 PARENTAL_STATUS value
                             * @property {number} YOUTUBE_VIDEO=14 YOUTUBE_VIDEO value
                             * @property {number} YOUTUBE_CHANNEL=15 YOUTUBE_CHANNEL value
                             * @property {number} USER_LIST=16 USER_LIST value
                             * @property {number} PROXIMITY=17 PROXIMITY value
                             * @property {number} TOPIC=18 TOPIC value
                             * @property {number} LISTING_SCOPE=19 LISTING_SCOPE value
                             * @property {number} LANGUAGE=20 LANGUAGE value
                             * @property {number} IP_BLOCK=21 IP_BLOCK value
                             * @property {number} CONTENT_LABEL=22 CONTENT_LABEL value
                             * @property {number} CARRIER=23 CARRIER value
                             * @property {number} USER_INTEREST=24 USER_INTEREST value
                             * @property {number} WEBPAGE=25 WEBPAGE value
                             * @property {number} OPERATING_SYSTEM_VERSION=26 OPERATING_SYSTEM_VERSION value
                             * @property {number} APP_PAYMENT_MODEL=27 APP_PAYMENT_MODEL value
                             * @property {number} MOBILE_DEVICE=28 MOBILE_DEVICE value
                             * @property {number} CUSTOM_AFFINITY=29 CUSTOM_AFFINITY value
                             * @property {number} CUSTOM_INTENT=30 CUSTOM_INTENT value
                             * @property {number} LOCATION_GROUP=31 LOCATION_GROUP value
                             * @property {number} CUSTOM_AUDIENCE=32 CUSTOM_AUDIENCE value
                             * @property {number} COMBINED_AUDIENCE=33 COMBINED_AUDIENCE value
                             * @property {number} KEYWORD_THEME=34 KEYWORD_THEME value
                             * @property {number} AUDIENCE=35 AUDIENCE value
                             */
                            CriterionTypeEnum.CriterionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "KEYWORD"] = 2;
                                values[valuesById[3] = "PLACEMENT"] = 3;
                                values[valuesById[4] = "MOBILE_APP_CATEGORY"] = 4;
                                values[valuesById[5] = "MOBILE_APPLICATION"] = 5;
                                values[valuesById[6] = "DEVICE"] = 6;
                                values[valuesById[7] = "LOCATION"] = 7;
                                values[valuesById[8] = "LISTING_GROUP"] = 8;
                                values[valuesById[9] = "AD_SCHEDULE"] = 9;
                                values[valuesById[10] = "AGE_RANGE"] = 10;
                                values[valuesById[11] = "GENDER"] = 11;
                                values[valuesById[12] = "INCOME_RANGE"] = 12;
                                values[valuesById[13] = "PARENTAL_STATUS"] = 13;
                                values[valuesById[14] = "YOUTUBE_VIDEO"] = 14;
                                values[valuesById[15] = "YOUTUBE_CHANNEL"] = 15;
                                values[valuesById[16] = "USER_LIST"] = 16;
                                values[valuesById[17] = "PROXIMITY"] = 17;
                                values[valuesById[18] = "TOPIC"] = 18;
                                values[valuesById[19] = "LISTING_SCOPE"] = 19;
                                values[valuesById[20] = "LANGUAGE"] = 20;
                                values[valuesById[21] = "IP_BLOCK"] = 21;
                                values[valuesById[22] = "CONTENT_LABEL"] = 22;
                                values[valuesById[23] = "CARRIER"] = 23;
                                values[valuesById[24] = "USER_INTEREST"] = 24;
                                values[valuesById[25] = "WEBPAGE"] = 25;
                                values[valuesById[26] = "OPERATING_SYSTEM_VERSION"] = 26;
                                values[valuesById[27] = "APP_PAYMENT_MODEL"] = 27;
                                values[valuesById[28] = "MOBILE_DEVICE"] = 28;
                                values[valuesById[29] = "CUSTOM_AFFINITY"] = 29;
                                values[valuesById[30] = "CUSTOM_INTENT"] = 30;
                                values[valuesById[31] = "LOCATION_GROUP"] = 31;
                                values[valuesById[32] = "CUSTOM_AUDIENCE"] = 32;
                                values[valuesById[33] = "COMBINED_AUDIENCE"] = 33;
                                values[valuesById[34] = "KEYWORD_THEME"] = 34;
                                values[valuesById[35] = "AUDIENCE"] = 35;
                                return values;
                            })();
    
                            return CriterionTypeEnum;
                        })();
    
                        enums.CustomAudienceMemberTypeEnum = (function() {
    
                            /**
                             * Properties of a CustomAudienceMemberTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomAudienceMemberTypeEnum
                             */
    
                            /**
                             * Constructs a new CustomAudienceMemberTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomAudienceMemberTypeEnum.
                             * @implements ICustomAudienceMemberTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceMemberTypeEnum=} [p] Properties to set
                             */
                            function CustomAudienceMemberTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomAudienceMemberTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceMemberTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum} CustomAudienceMemberTypeEnum instance
                             */
                            CustomAudienceMemberTypeEnum.create = function create(properties) {
                                return new CustomAudienceMemberTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceMemberTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceMemberTypeEnum} m CustomAudienceMemberTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceMemberTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceMemberTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceMemberTypeEnum} message CustomAudienceMemberTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceMemberTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceMemberTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum} CustomAudienceMemberTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceMemberTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceMemberTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum} CustomAudienceMemberTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceMemberTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceMemberTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceMemberTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceMemberTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum} CustomAudienceMemberTypeEnum
                             */
                            CustomAudienceMemberTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceMemberTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum} m CustomAudienceMemberTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceMemberTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomAudienceMemberTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceMemberTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomAudienceMemberType enum.
                             * @name google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum.CustomAudienceMemberType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} KEYWORD=2 KEYWORD value
                             * @property {number} URL=3 URL value
                             * @property {number} PLACE_CATEGORY=4 PLACE_CATEGORY value
                             * @property {number} APP=5 APP value
                             */
                            CustomAudienceMemberTypeEnum.CustomAudienceMemberType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "KEYWORD"] = 2;
                                values[valuesById[3] = "URL"] = 3;
                                values[valuesById[4] = "PLACE_CATEGORY"] = 4;
                                values[valuesById[5] = "APP"] = 5;
                                return values;
                            })();
    
                            return CustomAudienceMemberTypeEnum;
                        })();
    
                        enums.CustomAudienceStatusEnum = (function() {
    
                            /**
                             * Properties of a CustomAudienceStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomAudienceStatusEnum
                             */
    
                            /**
                             * Constructs a new CustomAudienceStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomAudienceStatusEnum.
                             * @implements ICustomAudienceStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceStatusEnum=} [p] Properties to set
                             */
                            function CustomAudienceStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomAudienceStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceStatusEnum} CustomAudienceStatusEnum instance
                             */
                            CustomAudienceStatusEnum.create = function create(properties) {
                                return new CustomAudienceStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomAudienceStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceStatusEnum} m CustomAudienceStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomAudienceStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceStatusEnum} message CustomAudienceStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceStatusEnum} CustomAudienceStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomAudienceStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceStatusEnum} CustomAudienceStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceStatusEnum} CustomAudienceStatusEnum
                             */
                            CustomAudienceStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomAudienceStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomAudienceStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomAudienceStatusEnum} m CustomAudienceStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomAudienceStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomAudienceStatus enum.
                             * @name google.ads.googleads.v11.enums.CustomAudienceStatusEnum.CustomAudienceStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CustomAudienceStatusEnum.CustomAudienceStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CustomAudienceStatusEnum;
                        })();
    
                        enums.CustomAudienceTypeEnum = (function() {
    
                            /**
                             * Properties of a CustomAudienceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomAudienceTypeEnum
                             */
    
                            /**
                             * Constructs a new CustomAudienceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomAudienceTypeEnum.
                             * @implements ICustomAudienceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceTypeEnum=} [p] Properties to set
                             */
                            function CustomAudienceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomAudienceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceTypeEnum} CustomAudienceTypeEnum instance
                             */
                            CustomAudienceTypeEnum.create = function create(properties) {
                                return new CustomAudienceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomAudienceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceTypeEnum} m CustomAudienceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomAudienceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomAudienceTypeEnum} message CustomAudienceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceTypeEnum} CustomAudienceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomAudienceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceTypeEnum} CustomAudienceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomAudienceTypeEnum} CustomAudienceTypeEnum
                             */
                            CustomAudienceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomAudienceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomAudienceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomAudienceTypeEnum} m CustomAudienceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomAudienceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomAudienceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomAudienceType enum.
                             * @name google.ads.googleads.v11.enums.CustomAudienceTypeEnum.CustomAudienceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AUTO=2 AUTO value
                             * @property {number} INTEREST=3 INTEREST value
                             * @property {number} PURCHASE_INTENT=4 PURCHASE_INTENT value
                             * @property {number} SEARCH=5 SEARCH value
                             */
                            CustomAudienceTypeEnum.CustomAudienceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AUTO"] = 2;
                                values[valuesById[3] = "INTEREST"] = 3;
                                values[valuesById[4] = "PURCHASE_INTENT"] = 4;
                                values[valuesById[5] = "SEARCH"] = 5;
                                return values;
                            })();
    
                            return CustomAudienceTypeEnum;
                        })();
    
                        enums.CustomConversionGoalStatusEnum = (function() {
    
                            /**
                             * Properties of a CustomConversionGoalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomConversionGoalStatusEnum
                             */
    
                            /**
                             * Constructs a new CustomConversionGoalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomConversionGoalStatusEnum.
                             * @implements ICustomConversionGoalStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomConversionGoalStatusEnum=} [p] Properties to set
                             */
                            function CustomConversionGoalStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomConversionGoalStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomConversionGoalStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum} CustomConversionGoalStatusEnum instance
                             */
                            CustomConversionGoalStatusEnum.create = function create(properties) {
                                return new CustomConversionGoalStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoalStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomConversionGoalStatusEnum} m CustomConversionGoalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoalStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoalStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomConversionGoalStatusEnum} message CustomConversionGoalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoalStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomConversionGoalStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum} CustomConversionGoalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoalStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomConversionGoalStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum} CustomConversionGoalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoalStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomConversionGoalStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomConversionGoalStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomConversionGoalStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum} CustomConversionGoalStatusEnum
                             */
                            CustomConversionGoalStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomConversionGoalStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum} m CustomConversionGoalStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomConversionGoalStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomConversionGoalStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomConversionGoalStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomConversionGoalStatus enum.
                             * @name google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum.CustomConversionGoalStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CustomConversionGoalStatusEnum.CustomConversionGoalStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CustomConversionGoalStatusEnum;
                        })();
    
                        enums.CustomInterestMemberTypeEnum = (function() {
    
                            /**
                             * Properties of a CustomInterestMemberTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomInterestMemberTypeEnum
                             */
    
                            /**
                             * Constructs a new CustomInterestMemberTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomInterestMemberTypeEnum.
                             * @implements ICustomInterestMemberTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomInterestMemberTypeEnum=} [p] Properties to set
                             */
                            function CustomInterestMemberTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomInterestMemberTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestMemberTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum} CustomInterestMemberTypeEnum instance
                             */
                            CustomInterestMemberTypeEnum.create = function create(properties) {
                                return new CustomInterestMemberTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomInterestMemberTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestMemberTypeEnum} m CustomInterestMemberTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestMemberTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomInterestMemberTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestMemberTypeEnum} message CustomInterestMemberTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestMemberTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomInterestMemberTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum} CustomInterestMemberTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestMemberTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomInterestMemberTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum} CustomInterestMemberTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestMemberTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomInterestMemberTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomInterestMemberTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomInterestMemberTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum} CustomInterestMemberTypeEnum
                             */
                            CustomInterestMemberTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomInterestMemberTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum} m CustomInterestMemberTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomInterestMemberTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomInterestMemberTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomInterestMemberTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomInterestMemberType enum.
                             * @name google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum.CustomInterestMemberType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} KEYWORD=2 KEYWORD value
                             * @property {number} URL=3 URL value
                             */
                            CustomInterestMemberTypeEnum.CustomInterestMemberType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "KEYWORD"] = 2;
                                values[valuesById[3] = "URL"] = 3;
                                return values;
                            })();
    
                            return CustomInterestMemberTypeEnum;
                        })();
    
                        enums.CustomInterestStatusEnum = (function() {
    
                            /**
                             * Properties of a CustomInterestStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomInterestStatusEnum
                             */
    
                            /**
                             * Constructs a new CustomInterestStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomInterestStatusEnum.
                             * @implements ICustomInterestStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomInterestStatusEnum=} [p] Properties to set
                             */
                            function CustomInterestStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomInterestStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomInterestStatusEnum} CustomInterestStatusEnum instance
                             */
                            CustomInterestStatusEnum.create = function create(properties) {
                                return new CustomInterestStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomInterestStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomInterestStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestStatusEnum} m CustomInterestStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomInterestStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomInterestStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestStatusEnum} message CustomInterestStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomInterestStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomInterestStatusEnum} CustomInterestStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomInterestStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomInterestStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomInterestStatusEnum} CustomInterestStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomInterestStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomInterestStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomInterestStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomInterestStatusEnum} CustomInterestStatusEnum
                             */
                            CustomInterestStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomInterestStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomInterestStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomInterestStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomInterestStatusEnum} m CustomInterestStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomInterestStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomInterestStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomInterestStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomInterestStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomInterestStatus enum.
                             * @name google.ads.googleads.v11.enums.CustomInterestStatusEnum.CustomInterestStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CustomInterestStatusEnum.CustomInterestStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CustomInterestStatusEnum;
                        })();
    
                        enums.CustomInterestTypeEnum = (function() {
    
                            /**
                             * Properties of a CustomInterestTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomInterestTypeEnum
                             */
    
                            /**
                             * Constructs a new CustomInterestTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomInterestTypeEnum.
                             * @implements ICustomInterestTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomInterestTypeEnum=} [p] Properties to set
                             */
                            function CustomInterestTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomInterestTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomInterestTypeEnum} CustomInterestTypeEnum instance
                             */
                            CustomInterestTypeEnum.create = function create(properties) {
                                return new CustomInterestTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomInterestTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomInterestTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestTypeEnum} m CustomInterestTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomInterestTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomInterestTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomInterestTypeEnum} message CustomInterestTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomInterestTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomInterestTypeEnum} CustomInterestTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomInterestTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomInterestTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomInterestTypeEnum} CustomInterestTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomInterestTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomInterestTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomInterestTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomInterestTypeEnum} CustomInterestTypeEnum
                             */
                            CustomInterestTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomInterestTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomInterestTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomInterestTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomInterestTypeEnum} m CustomInterestTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomInterestTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomInterestTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomInterestTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomInterestTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomInterestType enum.
                             * @name google.ads.googleads.v11.enums.CustomInterestTypeEnum.CustomInterestType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CUSTOM_AFFINITY=2 CUSTOM_AFFINITY value
                             * @property {number} CUSTOM_INTENT=3 CUSTOM_INTENT value
                             */
                            CustomInterestTypeEnum.CustomInterestType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CUSTOM_AFFINITY"] = 2;
                                values[valuesById[3] = "CUSTOM_INTENT"] = 3;
                                return values;
                            })();
    
                            return CustomInterestTypeEnum;
                        })();
    
                        enums.CustomPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a CustomPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new CustomPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomPlaceholderFieldEnum.
                             * @implements ICustomPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function CustomPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum} CustomPlaceholderFieldEnum instance
                             */
                            CustomPlaceholderFieldEnum.create = function create(properties) {
                                return new CustomPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomPlaceholderFieldEnum} m CustomPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomPlaceholderFieldEnum} message CustomPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum} CustomPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum} CustomPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum} CustomPlaceholderFieldEnum
                             */
                            CustomPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum} m CustomPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum.CustomPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ID=2 ID value
                             * @property {number} ID2=3 ID2 value
                             * @property {number} ITEM_TITLE=4 ITEM_TITLE value
                             * @property {number} ITEM_SUBTITLE=5 ITEM_SUBTITLE value
                             * @property {number} ITEM_DESCRIPTION=6 ITEM_DESCRIPTION value
                             * @property {number} ITEM_ADDRESS=7 ITEM_ADDRESS value
                             * @property {number} PRICE=8 PRICE value
                             * @property {number} FORMATTED_PRICE=9 FORMATTED_PRICE value
                             * @property {number} SALE_PRICE=10 SALE_PRICE value
                             * @property {number} FORMATTED_SALE_PRICE=11 FORMATTED_SALE_PRICE value
                             * @property {number} IMAGE_URL=12 IMAGE_URL value
                             * @property {number} ITEM_CATEGORY=13 ITEM_CATEGORY value
                             * @property {number} FINAL_URLS=14 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=15 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=16 TRACKING_URL value
                             * @property {number} CONTEXTUAL_KEYWORDS=17 CONTEXTUAL_KEYWORDS value
                             * @property {number} ANDROID_APP_LINK=18 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_IDS=19 SIMILAR_IDS value
                             * @property {number} IOS_APP_LINK=20 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=21 IOS_APP_STORE_ID value
                             */
                            CustomPlaceholderFieldEnum.CustomPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ID"] = 2;
                                values[valuesById[3] = "ID2"] = 3;
                                values[valuesById[4] = "ITEM_TITLE"] = 4;
                                values[valuesById[5] = "ITEM_SUBTITLE"] = 5;
                                values[valuesById[6] = "ITEM_DESCRIPTION"] = 6;
                                values[valuesById[7] = "ITEM_ADDRESS"] = 7;
                                values[valuesById[8] = "PRICE"] = 8;
                                values[valuesById[9] = "FORMATTED_PRICE"] = 9;
                                values[valuesById[10] = "SALE_PRICE"] = 10;
                                values[valuesById[11] = "FORMATTED_SALE_PRICE"] = 11;
                                values[valuesById[12] = "IMAGE_URL"] = 12;
                                values[valuesById[13] = "ITEM_CATEGORY"] = 13;
                                values[valuesById[14] = "FINAL_URLS"] = 14;
                                values[valuesById[15] = "FINAL_MOBILE_URLS"] = 15;
                                values[valuesById[16] = "TRACKING_URL"] = 16;
                                values[valuesById[17] = "CONTEXTUAL_KEYWORDS"] = 17;
                                values[valuesById[18] = "ANDROID_APP_LINK"] = 18;
                                values[valuesById[19] = "SIMILAR_IDS"] = 19;
                                values[valuesById[20] = "IOS_APP_LINK"] = 20;
                                values[valuesById[21] = "IOS_APP_STORE_ID"] = 21;
                                return values;
                            })();
    
                            return CustomPlaceholderFieldEnum;
                        })();
    
                        enums.CustomerPayPerConversionEligibilityFailureReasonEnum = (function() {
    
                            /**
                             * Properties of a CustomerPayPerConversionEligibilityFailureReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomerPayPerConversionEligibilityFailureReasonEnum
                             */
    
                            /**
                             * Constructs a new CustomerPayPerConversionEligibilityFailureReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomerPayPerConversionEligibilityFailureReasonEnum.
                             * @implements ICustomerPayPerConversionEligibilityFailureReasonEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomerPayPerConversionEligibilityFailureReasonEnum=} [p] Properties to set
                             */
                            function CustomerPayPerConversionEligibilityFailureReasonEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerPayPerConversionEligibilityFailureReasonEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerPayPerConversionEligibilityFailureReasonEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum} CustomerPayPerConversionEligibilityFailureReasonEnum instance
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.create = function create(properties) {
                                return new CustomerPayPerConversionEligibilityFailureReasonEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerPayPerConversionEligibilityFailureReasonEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerPayPerConversionEligibilityFailureReasonEnum} m CustomerPayPerConversionEligibilityFailureReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerPayPerConversionEligibilityFailureReasonEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerPayPerConversionEligibilityFailureReasonEnum} message CustomerPayPerConversionEligibilityFailureReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerPayPerConversionEligibilityFailureReasonEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum} CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerPayPerConversionEligibilityFailureReasonEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum} CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerPayPerConversionEligibilityFailureReasonEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerPayPerConversionEligibilityFailureReasonEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum} CustomerPayPerConversionEligibilityFailureReasonEnum
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerPayPerConversionEligibilityFailureReasonEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum} m CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerPayPerConversionEligibilityFailureReasonEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerPayPerConversionEligibilityFailureReason enum.
                             * @name google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOT_ENOUGH_CONVERSIONS=2 NOT_ENOUGH_CONVERSIONS value
                             * @property {number} CONVERSION_LAG_TOO_HIGH=3 CONVERSION_LAG_TOO_HIGH value
                             * @property {number} HAS_CAMPAIGN_WITH_SHARED_BUDGET=4 HAS_CAMPAIGN_WITH_SHARED_BUDGET value
                             * @property {number} HAS_UPLOAD_CLICKS_CONVERSION=5 HAS_UPLOAD_CLICKS_CONVERSION value
                             * @property {number} AVERAGE_DAILY_SPEND_TOO_HIGH=6 AVERAGE_DAILY_SPEND_TOO_HIGH value
                             * @property {number} ANALYSIS_NOT_COMPLETE=7 ANALYSIS_NOT_COMPLETE value
                             * @property {number} OTHER=8 OTHER value
                             */
                            CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOT_ENOUGH_CONVERSIONS"] = 2;
                                values[valuesById[3] = "CONVERSION_LAG_TOO_HIGH"] = 3;
                                values[valuesById[4] = "HAS_CAMPAIGN_WITH_SHARED_BUDGET"] = 4;
                                values[valuesById[5] = "HAS_UPLOAD_CLICKS_CONVERSION"] = 5;
                                values[valuesById[6] = "AVERAGE_DAILY_SPEND_TOO_HIGH"] = 6;
                                values[valuesById[7] = "ANALYSIS_NOT_COMPLETE"] = 7;
                                values[valuesById[8] = "OTHER"] = 8;
                                return values;
                            })();
    
                            return CustomerPayPerConversionEligibilityFailureReasonEnum;
                        })();
    
                        enums.CustomerStatusEnum = (function() {
    
                            /**
                             * Properties of a CustomerStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomerStatusEnum
                             */
    
                            /**
                             * Constructs a new CustomerStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomerStatusEnum.
                             * @implements ICustomerStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomerStatusEnum=} [p] Properties to set
                             */
                            function CustomerStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomerStatusEnum} CustomerStatusEnum instance
                             */
                            CustomerStatusEnum.create = function create(properties) {
                                return new CustomerStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomerStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerStatusEnum} m CustomerStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomerStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomerStatusEnum} message CustomerStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomerStatusEnum} CustomerStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomerStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomerStatusEnum} CustomerStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomerStatusEnum} CustomerStatusEnum
                             */
                            CustomerStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomerStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomerStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomerStatusEnum} m CustomerStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomerStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerStatus enum.
                             * @name google.ads.googleads.v11.enums.CustomerStatusEnum.CustomerStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} CANCELED=3 CANCELED value
                             * @property {number} SUSPENDED=4 SUSPENDED value
                             * @property {number} CLOSED=5 CLOSED value
                             */
                            CustomerStatusEnum.CustomerStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "CANCELED"] = 3;
                                values[valuesById[4] = "SUSPENDED"] = 4;
                                values[valuesById[5] = "CLOSED"] = 5;
                                return values;
                            })();
    
                            return CustomerStatusEnum;
                        })();
    
                        enums.CustomizerAttributeStatusEnum = (function() {
    
                            /**
                             * Properties of a CustomizerAttributeStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomizerAttributeStatusEnum
                             */
    
                            /**
                             * Constructs a new CustomizerAttributeStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomizerAttributeStatusEnum.
                             * @implements ICustomizerAttributeStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeStatusEnum=} [p] Properties to set
                             */
                            function CustomizerAttributeStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomizerAttributeStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum} CustomizerAttributeStatusEnum instance
                             */
                            CustomizerAttributeStatusEnum.create = function create(properties) {
                                return new CustomizerAttributeStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeStatusEnum} m CustomizerAttributeStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerAttributeStatusEnum} message CustomizerAttributeStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomizerAttributeStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum} CustomizerAttributeStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomizerAttributeStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum} CustomizerAttributeStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomizerAttributeStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomizerAttributeStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomizerAttributeStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum} CustomizerAttributeStatusEnum
                             */
                            CustomizerAttributeStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomizerAttributeStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum} m CustomizerAttributeStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomizerAttributeStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomizerAttributeStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomizerAttributeStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomizerAttributeStatus enum.
                             * @name google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum.CustomizerAttributeStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CustomizerAttributeStatusEnum.CustomizerAttributeStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CustomizerAttributeStatusEnum;
                        })();
    
                        enums.CustomizerValueStatusEnum = (function() {
    
                            /**
                             * Properties of a CustomizerValueStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ICustomizerValueStatusEnum
                             */
    
                            /**
                             * Constructs a new CustomizerValueStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a CustomizerValueStatusEnum.
                             * @implements ICustomizerValueStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ICustomizerValueStatusEnum=} [p] Properties to set
                             */
                            function CustomizerValueStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomizerValueStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerValueStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.CustomizerValueStatusEnum} CustomizerValueStatusEnum instance
                             */
                            CustomizerValueStatusEnum.create = function create(properties) {
                                return new CustomizerValueStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomizerValueStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.CustomizerValueStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerValueStatusEnum} m CustomizerValueStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerValueStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomizerValueStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.CustomizerValueStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ICustomizerValueStatusEnum} message CustomizerValueStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerValueStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomizerValueStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.CustomizerValueStatusEnum} CustomizerValueStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerValueStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.CustomizerValueStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomizerValueStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.CustomizerValueStatusEnum} CustomizerValueStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerValueStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomizerValueStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomizerValueStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomizerValueStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.CustomizerValueStatusEnum} CustomizerValueStatusEnum
                             */
                            CustomizerValueStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.CustomizerValueStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.CustomizerValueStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomizerValueStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.CustomizerValueStatusEnum} m CustomizerValueStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomizerValueStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomizerValueStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.CustomizerValueStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomizerValueStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomizerValueStatus enum.
                             * @name google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            CustomizerValueStatusEnum.CustomizerValueStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return CustomizerValueStatusEnum;
                        })();
    
                        enums.DataDrivenModelStatusEnum = (function() {
    
                            /**
                             * Properties of a DataDrivenModelStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IDataDrivenModelStatusEnum
                             */
    
                            /**
                             * Constructs a new DataDrivenModelStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a DataDrivenModelStatusEnum.
                             * @implements IDataDrivenModelStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IDataDrivenModelStatusEnum=} [p] Properties to set
                             */
                            function DataDrivenModelStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DataDrivenModelStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDataDrivenModelStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.DataDrivenModelStatusEnum} DataDrivenModelStatusEnum instance
                             */
                            DataDrivenModelStatusEnum.create = function create(properties) {
                                return new DataDrivenModelStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DataDrivenModelStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.DataDrivenModelStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDataDrivenModelStatusEnum} m DataDrivenModelStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DataDrivenModelStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DataDrivenModelStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.DataDrivenModelStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDataDrivenModelStatusEnum} message DataDrivenModelStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DataDrivenModelStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DataDrivenModelStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.DataDrivenModelStatusEnum} DataDrivenModelStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DataDrivenModelStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.DataDrivenModelStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DataDrivenModelStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.DataDrivenModelStatusEnum} DataDrivenModelStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DataDrivenModelStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DataDrivenModelStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DataDrivenModelStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DataDrivenModelStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.DataDrivenModelStatusEnum} DataDrivenModelStatusEnum
                             */
                            DataDrivenModelStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.DataDrivenModelStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.DataDrivenModelStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a DataDrivenModelStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.DataDrivenModelStatusEnum} m DataDrivenModelStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DataDrivenModelStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DataDrivenModelStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.DataDrivenModelStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DataDrivenModelStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DataDrivenModelStatus enum.
                             * @name google.ads.googleads.v11.enums.DataDrivenModelStatusEnum.DataDrivenModelStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AVAILABLE=2 AVAILABLE value
                             * @property {number} STALE=3 STALE value
                             * @property {number} EXPIRED=4 EXPIRED value
                             * @property {number} NEVER_GENERATED=5 NEVER_GENERATED value
                             */
                            DataDrivenModelStatusEnum.DataDrivenModelStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AVAILABLE"] = 2;
                                values[valuesById[3] = "STALE"] = 3;
                                values[valuesById[4] = "EXPIRED"] = 4;
                                values[valuesById[5] = "NEVER_GENERATED"] = 5;
                                return values;
                            })();
    
                            return DataDrivenModelStatusEnum;
                        })();
    
                        enums.DistanceBucketEnum = (function() {
    
                            /**
                             * Properties of a DistanceBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IDistanceBucketEnum
                             */
    
                            /**
                             * Constructs a new DistanceBucketEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a DistanceBucketEnum.
                             * @implements IDistanceBucketEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IDistanceBucketEnum=} [p] Properties to set
                             */
                            function DistanceBucketEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DistanceBucketEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDistanceBucketEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.DistanceBucketEnum} DistanceBucketEnum instance
                             */
                            DistanceBucketEnum.create = function create(properties) {
                                return new DistanceBucketEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DistanceBucketEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.DistanceBucketEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDistanceBucketEnum} m DistanceBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DistanceBucketEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DistanceBucketEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.DistanceBucketEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDistanceBucketEnum} message DistanceBucketEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DistanceBucketEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DistanceBucketEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.DistanceBucketEnum} DistanceBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DistanceBucketEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.DistanceBucketEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DistanceBucketEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.DistanceBucketEnum} DistanceBucketEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DistanceBucketEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DistanceBucketEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DistanceBucketEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DistanceBucketEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.DistanceBucketEnum} DistanceBucketEnum
                             */
                            DistanceBucketEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.DistanceBucketEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.DistanceBucketEnum();
                            };
    
                            /**
                             * Creates a plain object from a DistanceBucketEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.DistanceBucketEnum} m DistanceBucketEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DistanceBucketEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DistanceBucketEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.DistanceBucketEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DistanceBucketEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DistanceBucket enum.
                             * @name google.ads.googleads.v11.enums.DistanceBucketEnum.DistanceBucket
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} WITHIN_700M=2 WITHIN_700M value
                             * @property {number} WITHIN_1KM=3 WITHIN_1KM value
                             * @property {number} WITHIN_5KM=4 WITHIN_5KM value
                             * @property {number} WITHIN_10KM=5 WITHIN_10KM value
                             * @property {number} WITHIN_15KM=6 WITHIN_15KM value
                             * @property {number} WITHIN_20KM=7 WITHIN_20KM value
                             * @property {number} WITHIN_25KM=8 WITHIN_25KM value
                             * @property {number} WITHIN_30KM=9 WITHIN_30KM value
                             * @property {number} WITHIN_35KM=10 WITHIN_35KM value
                             * @property {number} WITHIN_40KM=11 WITHIN_40KM value
                             * @property {number} WITHIN_45KM=12 WITHIN_45KM value
                             * @property {number} WITHIN_50KM=13 WITHIN_50KM value
                             * @property {number} WITHIN_55KM=14 WITHIN_55KM value
                             * @property {number} WITHIN_60KM=15 WITHIN_60KM value
                             * @property {number} WITHIN_65KM=16 WITHIN_65KM value
                             * @property {number} BEYOND_65KM=17 BEYOND_65KM value
                             * @property {number} WITHIN_0_7MILES=18 WITHIN_0_7MILES value
                             * @property {number} WITHIN_1MILE=19 WITHIN_1MILE value
                             * @property {number} WITHIN_5MILES=20 WITHIN_5MILES value
                             * @property {number} WITHIN_10MILES=21 WITHIN_10MILES value
                             * @property {number} WITHIN_15MILES=22 WITHIN_15MILES value
                             * @property {number} WITHIN_20MILES=23 WITHIN_20MILES value
                             * @property {number} WITHIN_25MILES=24 WITHIN_25MILES value
                             * @property {number} WITHIN_30MILES=25 WITHIN_30MILES value
                             * @property {number} WITHIN_35MILES=26 WITHIN_35MILES value
                             * @property {number} WITHIN_40MILES=27 WITHIN_40MILES value
                             * @property {number} BEYOND_40MILES=28 BEYOND_40MILES value
                             */
                            DistanceBucketEnum.DistanceBucket = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "WITHIN_700M"] = 2;
                                values[valuesById[3] = "WITHIN_1KM"] = 3;
                                values[valuesById[4] = "WITHIN_5KM"] = 4;
                                values[valuesById[5] = "WITHIN_10KM"] = 5;
                                values[valuesById[6] = "WITHIN_15KM"] = 6;
                                values[valuesById[7] = "WITHIN_20KM"] = 7;
                                values[valuesById[8] = "WITHIN_25KM"] = 8;
                                values[valuesById[9] = "WITHIN_30KM"] = 9;
                                values[valuesById[10] = "WITHIN_35KM"] = 10;
                                values[valuesById[11] = "WITHIN_40KM"] = 11;
                                values[valuesById[12] = "WITHIN_45KM"] = 12;
                                values[valuesById[13] = "WITHIN_50KM"] = 13;
                                values[valuesById[14] = "WITHIN_55KM"] = 14;
                                values[valuesById[15] = "WITHIN_60KM"] = 15;
                                values[valuesById[16] = "WITHIN_65KM"] = 16;
                                values[valuesById[17] = "BEYOND_65KM"] = 17;
                                values[valuesById[18] = "WITHIN_0_7MILES"] = 18;
                                values[valuesById[19] = "WITHIN_1MILE"] = 19;
                                values[valuesById[20] = "WITHIN_5MILES"] = 20;
                                values[valuesById[21] = "WITHIN_10MILES"] = 21;
                                values[valuesById[22] = "WITHIN_15MILES"] = 22;
                                values[valuesById[23] = "WITHIN_20MILES"] = 23;
                                values[valuesById[24] = "WITHIN_25MILES"] = 24;
                                values[valuesById[25] = "WITHIN_30MILES"] = 25;
                                values[valuesById[26] = "WITHIN_35MILES"] = 26;
                                values[valuesById[27] = "WITHIN_40MILES"] = 27;
                                values[valuesById[28] = "BEYOND_40MILES"] = 28;
                                return values;
                            })();
    
                            return DistanceBucketEnum;
                        })();
    
                        enums.DsaPageFeedCriterionFieldEnum = (function() {
    
                            /**
                             * Properties of a DsaPageFeedCriterionFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IDsaPageFeedCriterionFieldEnum
                             */
    
                            /**
                             * Constructs a new DsaPageFeedCriterionFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a DsaPageFeedCriterionFieldEnum.
                             * @implements IDsaPageFeedCriterionFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IDsaPageFeedCriterionFieldEnum=} [p] Properties to set
                             */
                            function DsaPageFeedCriterionFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DsaPageFeedCriterionFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDsaPageFeedCriterionFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum} DsaPageFeedCriterionFieldEnum instance
                             */
                            DsaPageFeedCriterionFieldEnum.create = function create(properties) {
                                return new DsaPageFeedCriterionFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DsaPageFeedCriterionFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDsaPageFeedCriterionFieldEnum} m DsaPageFeedCriterionFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DsaPageFeedCriterionFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DsaPageFeedCriterionFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IDsaPageFeedCriterionFieldEnum} message DsaPageFeedCriterionFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DsaPageFeedCriterionFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DsaPageFeedCriterionFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum} DsaPageFeedCriterionFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DsaPageFeedCriterionFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DsaPageFeedCriterionFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum} DsaPageFeedCriterionFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DsaPageFeedCriterionFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DsaPageFeedCriterionFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DsaPageFeedCriterionFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DsaPageFeedCriterionFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum} DsaPageFeedCriterionFieldEnum
                             */
                            DsaPageFeedCriterionFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a DsaPageFeedCriterionFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum} m DsaPageFeedCriterionFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DsaPageFeedCriterionFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DsaPageFeedCriterionFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DsaPageFeedCriterionFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DsaPageFeedCriterionField enum.
                             * @name google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PAGE_URL=2 PAGE_URL value
                             * @property {number} LABEL=3 LABEL value
                             */
                            DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PAGE_URL"] = 2;
                                values[valuesById[3] = "LABEL"] = 3;
                                return values;
                            })();
    
                            return DsaPageFeedCriterionFieldEnum;
                        })();
    
                        enums.EducationPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of an EducationPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IEducationPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new EducationPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an EducationPlaceholderFieldEnum.
                             * @implements IEducationPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IEducationPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function EducationPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new EducationPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IEducationPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum} EducationPlaceholderFieldEnum instance
                             */
                            EducationPlaceholderFieldEnum.create = function create(properties) {
                                return new EducationPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified EducationPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IEducationPlaceholderFieldEnum} m EducationPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EducationPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified EducationPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IEducationPlaceholderFieldEnum} message EducationPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EducationPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EducationPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum} EducationPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EducationPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an EducationPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum} EducationPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EducationPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EducationPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EducationPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an EducationPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum} EducationPlaceholderFieldEnum
                             */
                            EducationPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from an EducationPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum} m EducationPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EducationPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this EducationPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EducationPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * EducationPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum.EducationPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PROGRAM_ID=2 PROGRAM_ID value
                             * @property {number} LOCATION_ID=3 LOCATION_ID value
                             * @property {number} PROGRAM_NAME=4 PROGRAM_NAME value
                             * @property {number} AREA_OF_STUDY=5 AREA_OF_STUDY value
                             * @property {number} PROGRAM_DESCRIPTION=6 PROGRAM_DESCRIPTION value
                             * @property {number} SCHOOL_NAME=7 SCHOOL_NAME value
                             * @property {number} ADDRESS=8 ADDRESS value
                             * @property {number} THUMBNAIL_IMAGE_URL=9 THUMBNAIL_IMAGE_URL value
                             * @property {number} ALTERNATIVE_THUMBNAIL_IMAGE_URL=10 ALTERNATIVE_THUMBNAIL_IMAGE_URL value
                             * @property {number} FINAL_URLS=11 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=12 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=13 TRACKING_URL value
                             * @property {number} CONTEXTUAL_KEYWORDS=14 CONTEXTUAL_KEYWORDS value
                             * @property {number} ANDROID_APP_LINK=15 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_PROGRAM_IDS=16 SIMILAR_PROGRAM_IDS value
                             * @property {number} IOS_APP_LINK=17 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=18 IOS_APP_STORE_ID value
                             */
                            EducationPlaceholderFieldEnum.EducationPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PROGRAM_ID"] = 2;
                                values[valuesById[3] = "LOCATION_ID"] = 3;
                                values[valuesById[4] = "PROGRAM_NAME"] = 4;
                                values[valuesById[5] = "AREA_OF_STUDY"] = 5;
                                values[valuesById[6] = "PROGRAM_DESCRIPTION"] = 6;
                                values[valuesById[7] = "SCHOOL_NAME"] = 7;
                                values[valuesById[8] = "ADDRESS"] = 8;
                                values[valuesById[9] = "THUMBNAIL_IMAGE_URL"] = 9;
                                values[valuesById[10] = "ALTERNATIVE_THUMBNAIL_IMAGE_URL"] = 10;
                                values[valuesById[11] = "FINAL_URLS"] = 11;
                                values[valuesById[12] = "FINAL_MOBILE_URLS"] = 12;
                                values[valuesById[13] = "TRACKING_URL"] = 13;
                                values[valuesById[14] = "CONTEXTUAL_KEYWORDS"] = 14;
                                values[valuesById[15] = "ANDROID_APP_LINK"] = 15;
                                values[valuesById[16] = "SIMILAR_PROGRAM_IDS"] = 16;
                                values[valuesById[17] = "IOS_APP_LINK"] = 17;
                                values[valuesById[18] = "IOS_APP_STORE_ID"] = 18;
                                return values;
                            })();
    
                            return EducationPlaceholderFieldEnum;
                        })();
    
                        enums.ExperimentStatusEnum = (function() {
    
                            /**
                             * Properties of an ExperimentStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IExperimentStatusEnum
                             */
    
                            /**
                             * Constructs a new ExperimentStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ExperimentStatusEnum.
                             * @implements IExperimentStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IExperimentStatusEnum=} [p] Properties to set
                             */
                            function ExperimentStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExperimentStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ExperimentStatusEnum} ExperimentStatusEnum instance
                             */
                            ExperimentStatusEnum.create = function create(properties) {
                                return new ExperimentStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentStatusEnum} m ExperimentStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentStatusEnum} message ExperimentStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ExperimentStatusEnum} ExperimentStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ExperimentStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ExperimentStatusEnum} ExperimentStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ExperimentStatusEnum} ExperimentStatusEnum
                             */
                            ExperimentStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ExperimentStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ExperimentStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExperimentStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ExperimentStatusEnum} m ExperimentStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExperimentStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ExperimentStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExperimentStatus enum.
                             * @name google.ads.googleads.v11.enums.ExperimentStatusEnum.ExperimentStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             * @property {number} HALTED=4 HALTED value
                             * @property {number} PROMOTED=5 PROMOTED value
                             * @property {number} SETUP=6 SETUP value
                             * @property {number} INITIATED=7 INITIATED value
                             * @property {number} GRADUATED=8 GRADUATED value
                             */
                            ExperimentStatusEnum.ExperimentStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                values[valuesById[4] = "HALTED"] = 4;
                                values[valuesById[5] = "PROMOTED"] = 5;
                                values[valuesById[6] = "SETUP"] = 6;
                                values[valuesById[7] = "INITIATED"] = 7;
                                values[valuesById[8] = "GRADUATED"] = 8;
                                return values;
                            })();
    
                            return ExperimentStatusEnum;
                        })();
    
                        enums.ExperimentTypeEnum = (function() {
    
                            /**
                             * Properties of an ExperimentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IExperimentTypeEnum
                             */
    
                            /**
                             * Constructs a new ExperimentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ExperimentTypeEnum.
                             * @implements IExperimentTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IExperimentTypeEnum=} [p] Properties to set
                             */
                            function ExperimentTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExperimentTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ExperimentTypeEnum} ExperimentTypeEnum instance
                             */
                            ExperimentTypeEnum.create = function create(properties) {
                                return new ExperimentTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentTypeEnum} m ExperimentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ExperimentTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExperimentTypeEnum} message ExperimentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ExperimentTypeEnum} ExperimentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ExperimentTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ExperimentTypeEnum} ExperimentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ExperimentTypeEnum} ExperimentTypeEnum
                             */
                            ExperimentTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ExperimentTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ExperimentTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExperimentTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ExperimentTypeEnum} m ExperimentTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExperimentTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ExperimentTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExperimentType enum.
                             * @name google.ads.googleads.v11.enums.ExperimentTypeEnum.ExperimentType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DISPLAY_AND_VIDEO_360=2 DISPLAY_AND_VIDEO_360 value
                             * @property {number} AD_VARIATION=3 AD_VARIATION value
                             * @property {number} YOUTUBE_CUSTOM=5 YOUTUBE_CUSTOM value
                             * @property {number} DISPLAY_CUSTOM=6 DISPLAY_CUSTOM value
                             * @property {number} SEARCH_CUSTOM=7 SEARCH_CUSTOM value
                             * @property {number} DISPLAY_AUTOMATED_BIDDING_STRATEGY=8 DISPLAY_AUTOMATED_BIDDING_STRATEGY value
                             * @property {number} SEARCH_AUTOMATED_BIDDING_STRATEGY=9 SEARCH_AUTOMATED_BIDDING_STRATEGY value
                             * @property {number} SHOPPING_AUTOMATED_BIDDING_STRATEGY=10 SHOPPING_AUTOMATED_BIDDING_STRATEGY value
                             * @property {number} SMART_MATCHING=11 SMART_MATCHING value
                             * @property {number} HOTEL_CUSTOM=12 HOTEL_CUSTOM value
                             */
                            ExperimentTypeEnum.ExperimentType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DISPLAY_AND_VIDEO_360"] = 2;
                                values[valuesById[3] = "AD_VARIATION"] = 3;
                                values[valuesById[5] = "YOUTUBE_CUSTOM"] = 5;
                                values[valuesById[6] = "DISPLAY_CUSTOM"] = 6;
                                values[valuesById[7] = "SEARCH_CUSTOM"] = 7;
                                values[valuesById[8] = "DISPLAY_AUTOMATED_BIDDING_STRATEGY"] = 8;
                                values[valuesById[9] = "SEARCH_AUTOMATED_BIDDING_STRATEGY"] = 9;
                                values[valuesById[10] = "SHOPPING_AUTOMATED_BIDDING_STRATEGY"] = 10;
                                values[valuesById[11] = "SMART_MATCHING"] = 11;
                                values[valuesById[12] = "HOTEL_CUSTOM"] = 12;
                                return values;
                            })();
    
                            return ExperimentTypeEnum;
                        })();
    
                        enums.ExtensionSettingDeviceEnum = (function() {
    
                            /**
                             * Properties of an ExtensionSettingDeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IExtensionSettingDeviceEnum
                             */
    
                            /**
                             * Constructs a new ExtensionSettingDeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ExtensionSettingDeviceEnum.
                             * @implements IExtensionSettingDeviceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IExtensionSettingDeviceEnum=} [p] Properties to set
                             */
                            function ExtensionSettingDeviceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExtensionSettingDeviceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExtensionSettingDeviceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum} ExtensionSettingDeviceEnum instance
                             */
                            ExtensionSettingDeviceEnum.create = function create(properties) {
                                return new ExtensionSettingDeviceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExtensionSettingDeviceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExtensionSettingDeviceEnum} m ExtensionSettingDeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionSettingDeviceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExtensionSettingDeviceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExtensionSettingDeviceEnum} message ExtensionSettingDeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionSettingDeviceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExtensionSettingDeviceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum} ExtensionSettingDeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionSettingDeviceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExtensionSettingDeviceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum} ExtensionSettingDeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionSettingDeviceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExtensionSettingDeviceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExtensionSettingDeviceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExtensionSettingDeviceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum} ExtensionSettingDeviceEnum
                             */
                            ExtensionSettingDeviceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExtensionSettingDeviceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum} m ExtensionSettingDeviceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExtensionSettingDeviceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExtensionSettingDeviceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExtensionSettingDeviceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExtensionSettingDevice enum.
                             * @name google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MOBILE=2 MOBILE value
                             * @property {number} DESKTOP=3 DESKTOP value
                             */
                            ExtensionSettingDeviceEnum.ExtensionSettingDevice = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MOBILE"] = 2;
                                values[valuesById[3] = "DESKTOP"] = 3;
                                return values;
                            })();
    
                            return ExtensionSettingDeviceEnum;
                        })();
    
                        enums.ExtensionTypeEnum = (function() {
    
                            /**
                             * Properties of an ExtensionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IExtensionTypeEnum
                             */
    
                            /**
                             * Constructs a new ExtensionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ExtensionTypeEnum.
                             * @implements IExtensionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IExtensionTypeEnum=} [p] Properties to set
                             */
                            function ExtensionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExtensionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExtensionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ExtensionTypeEnum} ExtensionTypeEnum instance
                             */
                            ExtensionTypeEnum.create = function create(properties) {
                                return new ExtensionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExtensionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ExtensionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExtensionTypeEnum} m ExtensionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExtensionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ExtensionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IExtensionTypeEnum} message ExtensionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExtensionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ExtensionTypeEnum} ExtensionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ExtensionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExtensionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ExtensionTypeEnum} ExtensionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExtensionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExtensionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExtensionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ExtensionTypeEnum} ExtensionTypeEnum
                             */
                            ExtensionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ExtensionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ExtensionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExtensionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ExtensionTypeEnum} m ExtensionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExtensionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExtensionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ExtensionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExtensionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExtensionType enum.
                             * @name google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NONE=2 NONE value
                             * @property {number} APP=3 APP value
                             * @property {number} CALL=4 CALL value
                             * @property {number} CALLOUT=5 CALLOUT value
                             * @property {number} MESSAGE=6 MESSAGE value
                             * @property {number} PRICE=7 PRICE value
                             * @property {number} PROMOTION=8 PROMOTION value
                             * @property {number} SITELINK=10 SITELINK value
                             * @property {number} STRUCTURED_SNIPPET=11 STRUCTURED_SNIPPET value
                             * @property {number} LOCATION=12 LOCATION value
                             * @property {number} AFFILIATE_LOCATION=13 AFFILIATE_LOCATION value
                             * @property {number} HOTEL_CALLOUT=15 HOTEL_CALLOUT value
                             * @property {number} IMAGE=16 IMAGE value
                             */
                            ExtensionTypeEnum.ExtensionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NONE"] = 2;
                                values[valuesById[3] = "APP"] = 3;
                                values[valuesById[4] = "CALL"] = 4;
                                values[valuesById[5] = "CALLOUT"] = 5;
                                values[valuesById[6] = "MESSAGE"] = 6;
                                values[valuesById[7] = "PRICE"] = 7;
                                values[valuesById[8] = "PROMOTION"] = 8;
                                values[valuesById[10] = "SITELINK"] = 10;
                                values[valuesById[11] = "STRUCTURED_SNIPPET"] = 11;
                                values[valuesById[12] = "LOCATION"] = 12;
                                values[valuesById[13] = "AFFILIATE_LOCATION"] = 13;
                                values[valuesById[15] = "HOTEL_CALLOUT"] = 15;
                                values[valuesById[16] = "IMAGE"] = 16;
                                return values;
                            })();
    
                            return ExtensionTypeEnum;
                        })();
    
                        enums.FeedAttributeTypeEnum = (function() {
    
                            /**
                             * Properties of a FeedAttributeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedAttributeTypeEnum
                             */
    
                            /**
                             * Constructs a new FeedAttributeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedAttributeTypeEnum.
                             * @implements IFeedAttributeTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedAttributeTypeEnum=} [p] Properties to set
                             */
                            function FeedAttributeTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedAttributeTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedAttributeTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedAttributeTypeEnum} FeedAttributeTypeEnum instance
                             */
                            FeedAttributeTypeEnum.create = function create(properties) {
                                return new FeedAttributeTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedAttributeTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedAttributeTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedAttributeTypeEnum} m FeedAttributeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttributeTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedAttributeTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedAttributeTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedAttributeTypeEnum} message FeedAttributeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttributeTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedAttributeTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedAttributeTypeEnum} FeedAttributeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttributeTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedAttributeTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedAttributeTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedAttributeTypeEnum} FeedAttributeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttributeTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedAttributeTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedAttributeTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedAttributeTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedAttributeTypeEnum} FeedAttributeTypeEnum
                             */
                            FeedAttributeTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedAttributeTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedAttributeTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedAttributeTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedAttributeTypeEnum} m FeedAttributeTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedAttributeTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedAttributeTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedAttributeTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedAttributeTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedAttributeType enum.
                             * @name google.ads.googleads.v11.enums.FeedAttributeTypeEnum.FeedAttributeType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INT64=2 INT64 value
                             * @property {number} DOUBLE=3 DOUBLE value
                             * @property {number} STRING=4 STRING value
                             * @property {number} BOOLEAN=5 BOOLEAN value
                             * @property {number} URL=6 URL value
                             * @property {number} DATE_TIME=7 DATE_TIME value
                             * @property {number} INT64_LIST=8 INT64_LIST value
                             * @property {number} DOUBLE_LIST=9 DOUBLE_LIST value
                             * @property {number} STRING_LIST=10 STRING_LIST value
                             * @property {number} BOOLEAN_LIST=11 BOOLEAN_LIST value
                             * @property {number} URL_LIST=12 URL_LIST value
                             * @property {number} DATE_TIME_LIST=13 DATE_TIME_LIST value
                             * @property {number} PRICE=14 PRICE value
                             */
                            FeedAttributeTypeEnum.FeedAttributeType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INT64"] = 2;
                                values[valuesById[3] = "DOUBLE"] = 3;
                                values[valuesById[4] = "STRING"] = 4;
                                values[valuesById[5] = "BOOLEAN"] = 5;
                                values[valuesById[6] = "URL"] = 6;
                                values[valuesById[7] = "DATE_TIME"] = 7;
                                values[valuesById[8] = "INT64_LIST"] = 8;
                                values[valuesById[9] = "DOUBLE_LIST"] = 9;
                                values[valuesById[10] = "STRING_LIST"] = 10;
                                values[valuesById[11] = "BOOLEAN_LIST"] = 11;
                                values[valuesById[12] = "URL_LIST"] = 12;
                                values[valuesById[13] = "DATE_TIME_LIST"] = 13;
                                values[valuesById[14] = "PRICE"] = 14;
                                return values;
                            })();
    
                            return FeedAttributeTypeEnum;
                        })();
    
                        enums.FeedItemQualityApprovalStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedItemQualityApprovalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemQualityApprovalStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedItemQualityApprovalStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemQualityApprovalStatusEnum.
                             * @implements IFeedItemQualityApprovalStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityApprovalStatusEnum=} [p] Properties to set
                             */
                            function FeedItemQualityApprovalStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemQualityApprovalStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityApprovalStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum} FeedItemQualityApprovalStatusEnum instance
                             */
                            FeedItemQualityApprovalStatusEnum.create = function create(properties) {
                                return new FeedItemQualityApprovalStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemQualityApprovalStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityApprovalStatusEnum} m FeedItemQualityApprovalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemQualityApprovalStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemQualityApprovalStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityApprovalStatusEnum} message FeedItemQualityApprovalStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemQualityApprovalStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemQualityApprovalStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum} FeedItemQualityApprovalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemQualityApprovalStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemQualityApprovalStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum} FeedItemQualityApprovalStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemQualityApprovalStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemQualityApprovalStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemQualityApprovalStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemQualityApprovalStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum} FeedItemQualityApprovalStatusEnum
                             */
                            FeedItemQualityApprovalStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemQualityApprovalStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum} m FeedItemQualityApprovalStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemQualityApprovalStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemQualityApprovalStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemQualityApprovalStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemQualityApprovalStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum.FeedItemQualityApprovalStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} APPROVED=2 APPROVED value
                             * @property {number} DISAPPROVED=3 DISAPPROVED value
                             */
                            FeedItemQualityApprovalStatusEnum.FeedItemQualityApprovalStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "APPROVED"] = 2;
                                values[valuesById[3] = "DISAPPROVED"] = 3;
                                return values;
                            })();
    
                            return FeedItemQualityApprovalStatusEnum;
                        })();
    
                        enums.FeedItemQualityDisapprovalReasonEnum = (function() {
    
                            /**
                             * Properties of a FeedItemQualityDisapprovalReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemQualityDisapprovalReasonEnum
                             */
    
                            /**
                             * Constructs a new FeedItemQualityDisapprovalReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemQualityDisapprovalReasonEnum.
                             * @implements IFeedItemQualityDisapprovalReasonEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityDisapprovalReasonEnum=} [p] Properties to set
                             */
                            function FeedItemQualityDisapprovalReasonEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemQualityDisapprovalReasonEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityDisapprovalReasonEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum} FeedItemQualityDisapprovalReasonEnum instance
                             */
                            FeedItemQualityDisapprovalReasonEnum.create = function create(properties) {
                                return new FeedItemQualityDisapprovalReasonEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemQualityDisapprovalReasonEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityDisapprovalReasonEnum} m FeedItemQualityDisapprovalReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemQualityDisapprovalReasonEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemQualityDisapprovalReasonEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemQualityDisapprovalReasonEnum} message FeedItemQualityDisapprovalReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemQualityDisapprovalReasonEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemQualityDisapprovalReasonEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum} FeedItemQualityDisapprovalReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemQualityDisapprovalReasonEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemQualityDisapprovalReasonEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum} FeedItemQualityDisapprovalReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemQualityDisapprovalReasonEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemQualityDisapprovalReasonEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemQualityDisapprovalReasonEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemQualityDisapprovalReasonEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum} FeedItemQualityDisapprovalReasonEnum
                             */
                            FeedItemQualityDisapprovalReasonEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemQualityDisapprovalReasonEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum} m FeedItemQualityDisapprovalReasonEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemQualityDisapprovalReasonEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemQualityDisapprovalReasonEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemQualityDisapprovalReasonEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemQualityDisapprovalReason enum.
                             * @name google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum.FeedItemQualityDisapprovalReason
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PRICE_TABLE_REPETITIVE_HEADERS=2 PRICE_TABLE_REPETITIVE_HEADERS value
                             * @property {number} PRICE_TABLE_REPETITIVE_DESCRIPTION=3 PRICE_TABLE_REPETITIVE_DESCRIPTION value
                             * @property {number} PRICE_TABLE_INCONSISTENT_ROWS=4 PRICE_TABLE_INCONSISTENT_ROWS value
                             * @property {number} PRICE_DESCRIPTION_HAS_PRICE_QUALIFIERS=5 PRICE_DESCRIPTION_HAS_PRICE_QUALIFIERS value
                             * @property {number} PRICE_UNSUPPORTED_LANGUAGE=6 PRICE_UNSUPPORTED_LANGUAGE value
                             * @property {number} PRICE_TABLE_ROW_HEADER_TABLE_TYPE_MISMATCH=7 PRICE_TABLE_ROW_HEADER_TABLE_TYPE_MISMATCH value
                             * @property {number} PRICE_TABLE_ROW_HEADER_HAS_PROMOTIONAL_TEXT=8 PRICE_TABLE_ROW_HEADER_HAS_PROMOTIONAL_TEXT value
                             * @property {number} PRICE_TABLE_ROW_DESCRIPTION_NOT_RELEVANT=9 PRICE_TABLE_ROW_DESCRIPTION_NOT_RELEVANT value
                             * @property {number} PRICE_TABLE_ROW_DESCRIPTION_HAS_PROMOTIONAL_TEXT=10 PRICE_TABLE_ROW_DESCRIPTION_HAS_PROMOTIONAL_TEXT value
                             * @property {number} PRICE_TABLE_ROW_HEADER_DESCRIPTION_REPETITIVE=11 PRICE_TABLE_ROW_HEADER_DESCRIPTION_REPETITIVE value
                             * @property {number} PRICE_TABLE_ROW_UNRATEABLE=12 PRICE_TABLE_ROW_UNRATEABLE value
                             * @property {number} PRICE_TABLE_ROW_PRICE_INVALID=13 PRICE_TABLE_ROW_PRICE_INVALID value
                             * @property {number} PRICE_TABLE_ROW_URL_INVALID=14 PRICE_TABLE_ROW_URL_INVALID value
                             * @property {number} PRICE_HEADER_OR_DESCRIPTION_HAS_PRICE=15 PRICE_HEADER_OR_DESCRIPTION_HAS_PRICE value
                             * @property {number} STRUCTURED_SNIPPETS_HEADER_POLICY_VIOLATED=16 STRUCTURED_SNIPPETS_HEADER_POLICY_VIOLATED value
                             * @property {number} STRUCTURED_SNIPPETS_REPEATED_VALUES=17 STRUCTURED_SNIPPETS_REPEATED_VALUES value
                             * @property {number} STRUCTURED_SNIPPETS_EDITORIAL_GUIDELINES=18 STRUCTURED_SNIPPETS_EDITORIAL_GUIDELINES value
                             * @property {number} STRUCTURED_SNIPPETS_HAS_PROMOTIONAL_TEXT=19 STRUCTURED_SNIPPETS_HAS_PROMOTIONAL_TEXT value
                             */
                            FeedItemQualityDisapprovalReasonEnum.FeedItemQualityDisapprovalReason = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PRICE_TABLE_REPETITIVE_HEADERS"] = 2;
                                values[valuesById[3] = "PRICE_TABLE_REPETITIVE_DESCRIPTION"] = 3;
                                values[valuesById[4] = "PRICE_TABLE_INCONSISTENT_ROWS"] = 4;
                                values[valuesById[5] = "PRICE_DESCRIPTION_HAS_PRICE_QUALIFIERS"] = 5;
                                values[valuesById[6] = "PRICE_UNSUPPORTED_LANGUAGE"] = 6;
                                values[valuesById[7] = "PRICE_TABLE_ROW_HEADER_TABLE_TYPE_MISMATCH"] = 7;
                                values[valuesById[8] = "PRICE_TABLE_ROW_HEADER_HAS_PROMOTIONAL_TEXT"] = 8;
                                values[valuesById[9] = "PRICE_TABLE_ROW_DESCRIPTION_NOT_RELEVANT"] = 9;
                                values[valuesById[10] = "PRICE_TABLE_ROW_DESCRIPTION_HAS_PROMOTIONAL_TEXT"] = 10;
                                values[valuesById[11] = "PRICE_TABLE_ROW_HEADER_DESCRIPTION_REPETITIVE"] = 11;
                                values[valuesById[12] = "PRICE_TABLE_ROW_UNRATEABLE"] = 12;
                                values[valuesById[13] = "PRICE_TABLE_ROW_PRICE_INVALID"] = 13;
                                values[valuesById[14] = "PRICE_TABLE_ROW_URL_INVALID"] = 14;
                                values[valuesById[15] = "PRICE_HEADER_OR_DESCRIPTION_HAS_PRICE"] = 15;
                                values[valuesById[16] = "STRUCTURED_SNIPPETS_HEADER_POLICY_VIOLATED"] = 16;
                                values[valuesById[17] = "STRUCTURED_SNIPPETS_REPEATED_VALUES"] = 17;
                                values[valuesById[18] = "STRUCTURED_SNIPPETS_EDITORIAL_GUIDELINES"] = 18;
                                values[valuesById[19] = "STRUCTURED_SNIPPETS_HAS_PROMOTIONAL_TEXT"] = 19;
                                return values;
                            })();
    
                            return FeedItemQualityDisapprovalReasonEnum;
                        })();
    
                        enums.FeedItemSetStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedItemSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemSetStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedItemSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemSetStatusEnum.
                             * @implements IFeedItemSetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStatusEnum=} [p] Properties to set
                             */
                            function FeedItemSetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemSetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStatusEnum} FeedItemSetStatusEnum instance
                             */
                            FeedItemSetStatusEnum.create = function create(properties) {
                                return new FeedItemSetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemSetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStatusEnum} m FeedItemSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemSetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemSetStatusEnum} message FeedItemSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStatusEnum} FeedItemSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemSetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStatusEnum} FeedItemSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemSetStatusEnum} FeedItemSetStatusEnum
                             */
                            FeedItemSetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemSetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemSetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemSetStatusEnum} m FeedItemSetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemSetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemSetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemSetStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedItemSetStatusEnum.FeedItemSetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            FeedItemSetStatusEnum.FeedItemSetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return FeedItemSetStatusEnum;
                        })();
    
                        enums.FeedItemStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedItemStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedItemStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemStatusEnum.
                             * @implements IFeedItemStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemStatusEnum=} [p] Properties to set
                             */
                            function FeedItemStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemStatusEnum} FeedItemStatusEnum instance
                             */
                            FeedItemStatusEnum.create = function create(properties) {
                                return new FeedItemStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemStatusEnum} m FeedItemStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemStatusEnum} message FeedItemStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemStatusEnum} FeedItemStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemStatusEnum} FeedItemStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemStatusEnum} FeedItemStatusEnum
                             */
                            FeedItemStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemStatusEnum} m FeedItemStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedItemStatusEnum.FeedItemStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            FeedItemStatusEnum.FeedItemStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return FeedItemStatusEnum;
                        })();
    
                        enums.FeedItemTargetDeviceEnum = (function() {
    
                            /**
                             * Properties of a FeedItemTargetDeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemTargetDeviceEnum
                             */
    
                            /**
                             * Constructs a new FeedItemTargetDeviceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemTargetDeviceEnum.
                             * @implements IFeedItemTargetDeviceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetDeviceEnum=} [p] Properties to set
                             */
                            function FeedItemTargetDeviceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemTargetDeviceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetDeviceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum} FeedItemTargetDeviceEnum instance
                             */
                            FeedItemTargetDeviceEnum.create = function create(properties) {
                                return new FeedItemTargetDeviceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetDeviceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetDeviceEnum} m FeedItemTargetDeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetDeviceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetDeviceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetDeviceEnum} message FeedItemTargetDeviceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetDeviceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemTargetDeviceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum} FeedItemTargetDeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetDeviceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemTargetDeviceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum} FeedItemTargetDeviceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetDeviceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemTargetDeviceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemTargetDeviceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemTargetDeviceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum} FeedItemTargetDeviceEnum
                             */
                            FeedItemTargetDeviceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemTargetDeviceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum} m FeedItemTargetDeviceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemTargetDeviceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemTargetDeviceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemTargetDeviceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemTargetDevice enum.
                             * @name google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.FeedItemTargetDevice
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MOBILE=2 MOBILE value
                             */
                            FeedItemTargetDeviceEnum.FeedItemTargetDevice = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MOBILE"] = 2;
                                return values;
                            })();
    
                            return FeedItemTargetDeviceEnum;
                        })();
    
                        enums.FeedItemTargetStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedItemTargetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemTargetStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedItemTargetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemTargetStatusEnum.
                             * @implements IFeedItemTargetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetStatusEnum=} [p] Properties to set
                             */
                            function FeedItemTargetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemTargetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetStatusEnum} FeedItemTargetStatusEnum instance
                             */
                            FeedItemTargetStatusEnum.create = function create(properties) {
                                return new FeedItemTargetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemTargetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetStatusEnum} m FeedItemTargetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemTargetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetStatusEnum} message FeedItemTargetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemTargetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetStatusEnum} FeedItemTargetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemTargetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemTargetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetStatusEnum} FeedItemTargetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemTargetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemTargetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemTargetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetStatusEnum} FeedItemTargetStatusEnum
                             */
                            FeedItemTargetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemTargetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemTargetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemTargetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemTargetStatusEnum} m FeedItemTargetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemTargetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemTargetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemTargetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemTargetStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedItemTargetStatusEnum.FeedItemTargetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            FeedItemTargetStatusEnum.FeedItemTargetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return FeedItemTargetStatusEnum;
                        })();
    
                        enums.FeedItemTargetTypeEnum = (function() {
    
                            /**
                             * Properties of a FeedItemTargetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemTargetTypeEnum
                             */
    
                            /**
                             * Constructs a new FeedItemTargetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemTargetTypeEnum.
                             * @implements IFeedItemTargetTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetTypeEnum=} [p] Properties to set
                             */
                            function FeedItemTargetTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemTargetTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetTypeEnum} FeedItemTargetTypeEnum instance
                             */
                            FeedItemTargetTypeEnum.create = function create(properties) {
                                return new FeedItemTargetTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemTargetTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetTypeEnum} m FeedItemTargetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemTargetTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemTargetTypeEnum} message FeedItemTargetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemTargetTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetTypeEnum} FeedItemTargetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemTargetTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemTargetTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetTypeEnum} FeedItemTargetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemTargetTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemTargetTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemTargetTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemTargetTypeEnum} FeedItemTargetTypeEnum
                             */
                            FeedItemTargetTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemTargetTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemTargetTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemTargetTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemTargetTypeEnum} m FeedItemTargetTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemTargetTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemTargetTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemTargetTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemTargetTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemTargetType enum.
                             * @name google.ads.googleads.v11.enums.FeedItemTargetTypeEnum.FeedItemTargetType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CAMPAIGN=2 CAMPAIGN value
                             * @property {number} AD_GROUP=3 AD_GROUP value
                             * @property {number} CRITERION=4 CRITERION value
                             */
                            FeedItemTargetTypeEnum.FeedItemTargetType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CAMPAIGN"] = 2;
                                values[valuesById[3] = "AD_GROUP"] = 3;
                                values[valuesById[4] = "CRITERION"] = 4;
                                return values;
                            })();
    
                            return FeedItemTargetTypeEnum;
                        })();
    
                        enums.FeedItemValidationStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedItemValidationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedItemValidationStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedItemValidationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedItemValidationStatusEnum.
                             * @implements IFeedItemValidationStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedItemValidationStatusEnum=} [p] Properties to set
                             */
                            function FeedItemValidationStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemValidationStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemValidationStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedItemValidationStatusEnum} FeedItemValidationStatusEnum instance
                             */
                            FeedItemValidationStatusEnum.create = function create(properties) {
                                return new FeedItemValidationStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemValidationStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemValidationStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemValidationStatusEnum} m FeedItemValidationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemValidationStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemValidationStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedItemValidationStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedItemValidationStatusEnum} message FeedItemValidationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemValidationStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemValidationStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedItemValidationStatusEnum} FeedItemValidationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemValidationStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedItemValidationStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemValidationStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedItemValidationStatusEnum} FeedItemValidationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemValidationStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemValidationStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemValidationStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemValidationStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedItemValidationStatusEnum} FeedItemValidationStatusEnum
                             */
                            FeedItemValidationStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedItemValidationStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedItemValidationStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemValidationStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedItemValidationStatusEnum} m FeedItemValidationStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemValidationStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemValidationStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedItemValidationStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemValidationStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemValidationStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedItemValidationStatusEnum.FeedItemValidationStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} INVALID=3 INVALID value
                             * @property {number} VALID=4 VALID value
                             */
                            FeedItemValidationStatusEnum.FeedItemValidationStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "INVALID"] = 3;
                                values[valuesById[4] = "VALID"] = 4;
                                return values;
                            })();
    
                            return FeedItemValidationStatusEnum;
                        })();
    
                        enums.FeedLinkStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedLinkStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedLinkStatusEnum.
                             * @implements IFeedLinkStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedLinkStatusEnum=} [p] Properties to set
                             */
                            function FeedLinkStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedLinkStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedLinkStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedLinkStatusEnum} FeedLinkStatusEnum instance
                             */
                            FeedLinkStatusEnum.create = function create(properties) {
                                return new FeedLinkStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedLinkStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedLinkStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedLinkStatusEnum} m FeedLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedLinkStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedLinkStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedLinkStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedLinkStatusEnum} message FeedLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedLinkStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedLinkStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedLinkStatusEnum} FeedLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedLinkStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedLinkStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedLinkStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedLinkStatusEnum} FeedLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedLinkStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedLinkStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedLinkStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedLinkStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedLinkStatusEnum} FeedLinkStatusEnum
                             */
                            FeedLinkStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedLinkStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedLinkStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedLinkStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedLinkStatusEnum} m FeedLinkStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedLinkStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedLinkStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedLinkStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedLinkStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedLinkStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            FeedLinkStatusEnum.FeedLinkStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return FeedLinkStatusEnum;
                        })();
    
                        enums.FeedMappingCriterionTypeEnum = (function() {
    
                            /**
                             * Properties of a FeedMappingCriterionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedMappingCriterionTypeEnum
                             */
    
                            /**
                             * Constructs a new FeedMappingCriterionTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedMappingCriterionTypeEnum.
                             * @implements IFeedMappingCriterionTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedMappingCriterionTypeEnum=} [p] Properties to set
                             */
                            function FeedMappingCriterionTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedMappingCriterionTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedMappingCriterionTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum} FeedMappingCriterionTypeEnum instance
                             */
                            FeedMappingCriterionTypeEnum.create = function create(properties) {
                                return new FeedMappingCriterionTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedMappingCriterionTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedMappingCriterionTypeEnum} m FeedMappingCriterionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingCriterionTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedMappingCriterionTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedMappingCriterionTypeEnum} message FeedMappingCriterionTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingCriterionTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedMappingCriterionTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum} FeedMappingCriterionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingCriterionTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedMappingCriterionTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum} FeedMappingCriterionTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingCriterionTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedMappingCriterionTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedMappingCriterionTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedMappingCriterionTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum} FeedMappingCriterionTypeEnum
                             */
                            FeedMappingCriterionTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedMappingCriterionTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum} m FeedMappingCriterionTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedMappingCriterionTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedMappingCriterionTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedMappingCriterionTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedMappingCriterionType enum.
                             * @name google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum.FeedMappingCriterionType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LOCATION_EXTENSION_TARGETING=4 LOCATION_EXTENSION_TARGETING value
                             * @property {number} DSA_PAGE_FEED=3 DSA_PAGE_FEED value
                             */
                            FeedMappingCriterionTypeEnum.FeedMappingCriterionType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[4] = "LOCATION_EXTENSION_TARGETING"] = 4;
                                values[valuesById[3] = "DSA_PAGE_FEED"] = 3;
                                return values;
                            })();
    
                            return FeedMappingCriterionTypeEnum;
                        })();
    
                        enums.FeedMappingStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedMappingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedMappingStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedMappingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedMappingStatusEnum.
                             * @implements IFeedMappingStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedMappingStatusEnum=} [p] Properties to set
                             */
                            function FeedMappingStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedMappingStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedMappingStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedMappingStatusEnum} FeedMappingStatusEnum instance
                             */
                            FeedMappingStatusEnum.create = function create(properties) {
                                return new FeedMappingStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedMappingStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedMappingStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedMappingStatusEnum} m FeedMappingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedMappingStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedMappingStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedMappingStatusEnum} message FeedMappingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedMappingStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedMappingStatusEnum} FeedMappingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedMappingStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedMappingStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedMappingStatusEnum} FeedMappingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedMappingStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedMappingStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedMappingStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedMappingStatusEnum} FeedMappingStatusEnum
                             */
                            FeedMappingStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedMappingStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedMappingStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedMappingStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedMappingStatusEnum} m FeedMappingStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedMappingStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedMappingStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedMappingStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedMappingStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedMappingStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedMappingStatusEnum.FeedMappingStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            FeedMappingStatusEnum.FeedMappingStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return FeedMappingStatusEnum;
                        })();
    
                        enums.FeedOriginEnum = (function() {
    
                            /**
                             * Properties of a FeedOriginEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedOriginEnum
                             */
    
                            /**
                             * Constructs a new FeedOriginEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedOriginEnum.
                             * @implements IFeedOriginEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedOriginEnum=} [p] Properties to set
                             */
                            function FeedOriginEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedOriginEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedOriginEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedOriginEnum} FeedOriginEnum instance
                             */
                            FeedOriginEnum.create = function create(properties) {
                                return new FeedOriginEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedOriginEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedOriginEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedOriginEnum} m FeedOriginEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedOriginEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedOriginEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedOriginEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedOriginEnum} message FeedOriginEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedOriginEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedOriginEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedOriginEnum} FeedOriginEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedOriginEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedOriginEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedOriginEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedOriginEnum} FeedOriginEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedOriginEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedOriginEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedOriginEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedOriginEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedOriginEnum} FeedOriginEnum
                             */
                            FeedOriginEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedOriginEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedOriginEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedOriginEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedOriginEnum} m FeedOriginEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedOriginEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedOriginEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedOriginEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedOriginEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedOrigin enum.
                             * @name google.ads.googleads.v11.enums.FeedOriginEnum.FeedOrigin
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} USER=2 USER value
                             * @property {number} GOOGLE=3 GOOGLE value
                             */
                            FeedOriginEnum.FeedOrigin = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "USER"] = 2;
                                values[valuesById[3] = "GOOGLE"] = 3;
                                return values;
                            })();
    
                            return FeedOriginEnum;
                        })();
    
                        enums.FeedStatusEnum = (function() {
    
                            /**
                             * Properties of a FeedStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFeedStatusEnum
                             */
    
                            /**
                             * Constructs a new FeedStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FeedStatusEnum.
                             * @implements IFeedStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFeedStatusEnum=} [p] Properties to set
                             */
                            function FeedStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FeedStatusEnum} FeedStatusEnum instance
                             */
                            FeedStatusEnum.create = function create(properties) {
                                return new FeedStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FeedStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedStatusEnum} m FeedStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FeedStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFeedStatusEnum} message FeedStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FeedStatusEnum} FeedStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FeedStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FeedStatusEnum} FeedStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FeedStatusEnum} FeedStatusEnum
                             */
                            FeedStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FeedStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FeedStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FeedStatusEnum} m FeedStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FeedStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedStatus enum.
                             * @name google.ads.googleads.v11.enums.FeedStatusEnum.FeedStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            FeedStatusEnum.FeedStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return FeedStatusEnum;
                        })();
    
                        enums.FlightPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a FlightPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IFlightPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new FlightPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a FlightPlaceholderFieldEnum.
                             * @implements IFlightPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IFlightPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function FlightPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FlightPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFlightPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum} FlightPlaceholderFieldEnum instance
                             */
                            FlightPlaceholderFieldEnum.create = function create(properties) {
                                return new FlightPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FlightPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFlightPlaceholderFieldEnum} m FlightPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FlightPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FlightPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IFlightPlaceholderFieldEnum} message FlightPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FlightPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FlightPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum} FlightPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FlightPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FlightPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum} FlightPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FlightPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FlightPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FlightPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FlightPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum} FlightPlaceholderFieldEnum
                             */
                            FlightPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a FlightPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum} m FlightPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FlightPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FlightPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FlightPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FlightPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum.FlightPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DESTINATION_ID=2 DESTINATION_ID value
                             * @property {number} ORIGIN_ID=3 ORIGIN_ID value
                             * @property {number} FLIGHT_DESCRIPTION=4 FLIGHT_DESCRIPTION value
                             * @property {number} ORIGIN_NAME=5 ORIGIN_NAME value
                             * @property {number} DESTINATION_NAME=6 DESTINATION_NAME value
                             * @property {number} FLIGHT_PRICE=7 FLIGHT_PRICE value
                             * @property {number} FORMATTED_PRICE=8 FORMATTED_PRICE value
                             * @property {number} FLIGHT_SALE_PRICE=9 FLIGHT_SALE_PRICE value
                             * @property {number} FORMATTED_SALE_PRICE=10 FORMATTED_SALE_PRICE value
                             * @property {number} IMAGE_URL=11 IMAGE_URL value
                             * @property {number} FINAL_URLS=12 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=13 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=14 TRACKING_URL value
                             * @property {number} ANDROID_APP_LINK=15 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_DESTINATION_IDS=16 SIMILAR_DESTINATION_IDS value
                             * @property {number} IOS_APP_LINK=17 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=18 IOS_APP_STORE_ID value
                             */
                            FlightPlaceholderFieldEnum.FlightPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DESTINATION_ID"] = 2;
                                values[valuesById[3] = "ORIGIN_ID"] = 3;
                                values[valuesById[4] = "FLIGHT_DESCRIPTION"] = 4;
                                values[valuesById[5] = "ORIGIN_NAME"] = 5;
                                values[valuesById[6] = "DESTINATION_NAME"] = 6;
                                values[valuesById[7] = "FLIGHT_PRICE"] = 7;
                                values[valuesById[8] = "FORMATTED_PRICE"] = 8;
                                values[valuesById[9] = "FLIGHT_SALE_PRICE"] = 9;
                                values[valuesById[10] = "FORMATTED_SALE_PRICE"] = 10;
                                values[valuesById[11] = "IMAGE_URL"] = 11;
                                values[valuesById[12] = "FINAL_URLS"] = 12;
                                values[valuesById[13] = "FINAL_MOBILE_URLS"] = 13;
                                values[valuesById[14] = "TRACKING_URL"] = 14;
                                values[valuesById[15] = "ANDROID_APP_LINK"] = 15;
                                values[valuesById[16] = "SIMILAR_DESTINATION_IDS"] = 16;
                                values[valuesById[17] = "IOS_APP_LINK"] = 17;
                                values[valuesById[18] = "IOS_APP_STORE_ID"] = 18;
                                return values;
                            })();
    
                            return FlightPlaceholderFieldEnum;
                        })();
    
                        enums.GeoTargetConstantStatusEnum = (function() {
    
                            /**
                             * Properties of a GeoTargetConstantStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGeoTargetConstantStatusEnum
                             */
    
                            /**
                             * Constructs a new GeoTargetConstantStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GeoTargetConstantStatusEnum.
                             * @implements IGeoTargetConstantStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGeoTargetConstantStatusEnum=} [p] Properties to set
                             */
                            function GeoTargetConstantStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GeoTargetConstantStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetConstantStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum} GeoTargetConstantStatusEnum instance
                             */
                            GeoTargetConstantStatusEnum.create = function create(properties) {
                                return new GeoTargetConstantStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstantStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetConstantStatusEnum} m GeoTargetConstantStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstantStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstantStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetConstantStatusEnum} message GeoTargetConstantStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstantStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeoTargetConstantStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum} GeoTargetConstantStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstantStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeoTargetConstantStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum} GeoTargetConstantStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstantStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeoTargetConstantStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeoTargetConstantStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GeoTargetConstantStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum} GeoTargetConstantStatusEnum
                             */
                            GeoTargetConstantStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a GeoTargetConstantStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum} m GeoTargetConstantStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeoTargetConstantStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GeoTargetConstantStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeoTargetConstantStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GeoTargetConstantStatus enum.
                             * @name google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum.GeoTargetConstantStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVAL_PLANNED=3 REMOVAL_PLANNED value
                             */
                            GeoTargetConstantStatusEnum.GeoTargetConstantStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVAL_PLANNED"] = 3;
                                return values;
                            })();
    
                            return GeoTargetConstantStatusEnum;
                        })();
    
                        enums.GeoTargetingRestrictionEnum = (function() {
    
                            /**
                             * Properties of a GeoTargetingRestrictionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGeoTargetingRestrictionEnum
                             */
    
                            /**
                             * Constructs a new GeoTargetingRestrictionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GeoTargetingRestrictionEnum.
                             * @implements IGeoTargetingRestrictionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingRestrictionEnum=} [p] Properties to set
                             */
                            function GeoTargetingRestrictionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GeoTargetingRestrictionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingRestrictionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum} GeoTargetingRestrictionEnum instance
                             */
                            GeoTargetingRestrictionEnum.create = function create(properties) {
                                return new GeoTargetingRestrictionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GeoTargetingRestrictionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingRestrictionEnum} m GeoTargetingRestrictionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetingRestrictionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeoTargetingRestrictionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingRestrictionEnum} message GeoTargetingRestrictionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetingRestrictionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeoTargetingRestrictionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum} GeoTargetingRestrictionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetingRestrictionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeoTargetingRestrictionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum} GeoTargetingRestrictionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetingRestrictionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeoTargetingRestrictionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeoTargetingRestrictionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GeoTargetingRestrictionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum} GeoTargetingRestrictionEnum
                             */
                            GeoTargetingRestrictionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum();
                            };
    
                            /**
                             * Creates a plain object from a GeoTargetingRestrictionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum} m GeoTargetingRestrictionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeoTargetingRestrictionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GeoTargetingRestrictionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeoTargetingRestrictionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GeoTargetingRestriction enum.
                             * @name google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum.GeoTargetingRestriction
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LOCATION_OF_PRESENCE=2 LOCATION_OF_PRESENCE value
                             */
                            GeoTargetingRestrictionEnum.GeoTargetingRestriction = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LOCATION_OF_PRESENCE"] = 2;
                                return values;
                            })();
    
                            return GeoTargetingRestrictionEnum;
                        })();
    
                        enums.GeoTargetingTypeEnum = (function() {
    
                            /**
                             * Properties of a GeoTargetingTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGeoTargetingTypeEnum
                             */
    
                            /**
                             * Constructs a new GeoTargetingTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GeoTargetingTypeEnum.
                             * @implements IGeoTargetingTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingTypeEnum=} [p] Properties to set
                             */
                            function GeoTargetingTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GeoTargetingTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingTypeEnum} GeoTargetingTypeEnum instance
                             */
                            GeoTargetingTypeEnum.create = function create(properties) {
                                return new GeoTargetingTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GeoTargetingTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GeoTargetingTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingTypeEnum} m GeoTargetingTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetingTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeoTargetingTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GeoTargetingTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGeoTargetingTypeEnum} message GeoTargetingTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetingTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeoTargetingTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingTypeEnum} GeoTargetingTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetingTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GeoTargetingTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeoTargetingTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingTypeEnum} GeoTargetingTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetingTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeoTargetingTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeoTargetingTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GeoTargetingTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GeoTargetingTypeEnum} GeoTargetingTypeEnum
                             */
                            GeoTargetingTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GeoTargetingTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GeoTargetingTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a GeoTargetingTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GeoTargetingTypeEnum} m GeoTargetingTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeoTargetingTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GeoTargetingTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GeoTargetingTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeoTargetingTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GeoTargetingType enum.
                             * @name google.ads.googleads.v11.enums.GeoTargetingTypeEnum.GeoTargetingType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AREA_OF_INTEREST=2 AREA_OF_INTEREST value
                             * @property {number} LOCATION_OF_PRESENCE=3 LOCATION_OF_PRESENCE value
                             */
                            GeoTargetingTypeEnum.GeoTargetingType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AREA_OF_INTEREST"] = 2;
                                values[valuesById[3] = "LOCATION_OF_PRESENCE"] = 3;
                                return values;
                            })();
    
                            return GeoTargetingTypeEnum;
                        })();
    
                        enums.GoalConfigLevelEnum = (function() {
    
                            /**
                             * Properties of a GoalConfigLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGoalConfigLevelEnum
                             */
    
                            /**
                             * Constructs a new GoalConfigLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GoalConfigLevelEnum.
                             * @implements IGoalConfigLevelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGoalConfigLevelEnum=} [p] Properties to set
                             */
                            function GoalConfigLevelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GoalConfigLevelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoalConfigLevelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GoalConfigLevelEnum} GoalConfigLevelEnum instance
                             */
                            GoalConfigLevelEnum.create = function create(properties) {
                                return new GoalConfigLevelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GoalConfigLevelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GoalConfigLevelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoalConfigLevelEnum} m GoalConfigLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoalConfigLevelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoalConfigLevelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GoalConfigLevelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoalConfigLevelEnum} message GoalConfigLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoalConfigLevelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoalConfigLevelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GoalConfigLevelEnum} GoalConfigLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoalConfigLevelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GoalConfigLevelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoalConfigLevelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GoalConfigLevelEnum} GoalConfigLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoalConfigLevelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoalConfigLevelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoalConfigLevelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GoalConfigLevelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GoalConfigLevelEnum} GoalConfigLevelEnum
                             */
                            GoalConfigLevelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GoalConfigLevelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GoalConfigLevelEnum();
                            };
    
                            /**
                             * Creates a plain object from a GoalConfigLevelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GoalConfigLevelEnum} m GoalConfigLevelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoalConfigLevelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GoalConfigLevelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GoalConfigLevelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoalConfigLevelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GoalConfigLevel enum.
                             * @name google.ads.googleads.v11.enums.GoalConfigLevelEnum.GoalConfigLevel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CUSTOMER=2 CUSTOMER value
                             * @property {number} CAMPAIGN=3 CAMPAIGN value
                             */
                            GoalConfigLevelEnum.GoalConfigLevel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CUSTOMER"] = 2;
                                values[valuesById[3] = "CAMPAIGN"] = 3;
                                return values;
                            })();
    
                            return GoalConfigLevelEnum;
                        })();
    
                        enums.GoogleAdsFieldCategoryEnum = (function() {
    
                            /**
                             * Properties of a GoogleAdsFieldCategoryEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGoogleAdsFieldCategoryEnum
                             */
    
                            /**
                             * Constructs a new GoogleAdsFieldCategoryEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GoogleAdsFieldCategoryEnum.
                             * @implements IGoogleAdsFieldCategoryEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldCategoryEnum=} [p] Properties to set
                             */
                            function GoogleAdsFieldCategoryEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GoogleAdsFieldCategoryEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldCategoryEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum} GoogleAdsFieldCategoryEnum instance
                             */
                            GoogleAdsFieldCategoryEnum.create = function create(properties) {
                                return new GoogleAdsFieldCategoryEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleAdsFieldCategoryEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldCategoryEnum} m GoogleAdsFieldCategoryEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsFieldCategoryEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleAdsFieldCategoryEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldCategoryEnum} message GoogleAdsFieldCategoryEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsFieldCategoryEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleAdsFieldCategoryEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum} GoogleAdsFieldCategoryEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsFieldCategoryEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleAdsFieldCategoryEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum} GoogleAdsFieldCategoryEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsFieldCategoryEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleAdsFieldCategoryEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleAdsFieldCategoryEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GoogleAdsFieldCategoryEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum} GoogleAdsFieldCategoryEnum
                             */
                            GoogleAdsFieldCategoryEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum();
                            };
    
                            /**
                             * Creates a plain object from a GoogleAdsFieldCategoryEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum} m GoogleAdsFieldCategoryEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleAdsFieldCategoryEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GoogleAdsFieldCategoryEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleAdsFieldCategoryEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GoogleAdsFieldCategory enum.
                             * @name google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} RESOURCE=2 RESOURCE value
                             * @property {number} ATTRIBUTE=3 ATTRIBUTE value
                             * @property {number} SEGMENT=5 SEGMENT value
                             * @property {number} METRIC=6 METRIC value
                             */
                            GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "RESOURCE"] = 2;
                                values[valuesById[3] = "ATTRIBUTE"] = 3;
                                values[valuesById[5] = "SEGMENT"] = 5;
                                values[valuesById[6] = "METRIC"] = 6;
                                return values;
                            })();
    
                            return GoogleAdsFieldCategoryEnum;
                        })();
    
                        enums.GoogleAdsFieldDataTypeEnum = (function() {
    
                            /**
                             * Properties of a GoogleAdsFieldDataTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGoogleAdsFieldDataTypeEnum
                             */
    
                            /**
                             * Constructs a new GoogleAdsFieldDataTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GoogleAdsFieldDataTypeEnum.
                             * @implements IGoogleAdsFieldDataTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldDataTypeEnum=} [p] Properties to set
                             */
                            function GoogleAdsFieldDataTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GoogleAdsFieldDataTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldDataTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum} GoogleAdsFieldDataTypeEnum instance
                             */
                            GoogleAdsFieldDataTypeEnum.create = function create(properties) {
                                return new GoogleAdsFieldDataTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleAdsFieldDataTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldDataTypeEnum} m GoogleAdsFieldDataTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsFieldDataTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleAdsFieldDataTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleAdsFieldDataTypeEnum} message GoogleAdsFieldDataTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsFieldDataTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleAdsFieldDataTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum} GoogleAdsFieldDataTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsFieldDataTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleAdsFieldDataTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum} GoogleAdsFieldDataTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsFieldDataTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleAdsFieldDataTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleAdsFieldDataTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GoogleAdsFieldDataTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum} GoogleAdsFieldDataTypeEnum
                             */
                            GoogleAdsFieldDataTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a GoogleAdsFieldDataTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum} m GoogleAdsFieldDataTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleAdsFieldDataTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GoogleAdsFieldDataTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleAdsFieldDataTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GoogleAdsFieldDataType enum.
                             * @name google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BOOLEAN=2 BOOLEAN value
                             * @property {number} DATE=3 DATE value
                             * @property {number} DOUBLE=4 DOUBLE value
                             * @property {number} ENUM=5 ENUM value
                             * @property {number} FLOAT=6 FLOAT value
                             * @property {number} INT32=7 INT32 value
                             * @property {number} INT64=8 INT64 value
                             * @property {number} MESSAGE=9 MESSAGE value
                             * @property {number} RESOURCE_NAME=10 RESOURCE_NAME value
                             * @property {number} STRING=11 STRING value
                             * @property {number} UINT64=12 UINT64 value
                             */
                            GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BOOLEAN"] = 2;
                                values[valuesById[3] = "DATE"] = 3;
                                values[valuesById[4] = "DOUBLE"] = 4;
                                values[valuesById[5] = "ENUM"] = 5;
                                values[valuesById[6] = "FLOAT"] = 6;
                                values[valuesById[7] = "INT32"] = 7;
                                values[valuesById[8] = "INT64"] = 8;
                                values[valuesById[9] = "MESSAGE"] = 9;
                                values[valuesById[10] = "RESOURCE_NAME"] = 10;
                                values[valuesById[11] = "STRING"] = 11;
                                values[valuesById[12] = "UINT64"] = 12;
                                return values;
                            })();
    
                            return GoogleAdsFieldDataTypeEnum;
                        })();
    
                        enums.GoogleVoiceCallStatusEnum = (function() {
    
                            /**
                             * Properties of a GoogleVoiceCallStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IGoogleVoiceCallStatusEnum
                             */
    
                            /**
                             * Constructs a new GoogleVoiceCallStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a GoogleVoiceCallStatusEnum.
                             * @implements IGoogleVoiceCallStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IGoogleVoiceCallStatusEnum=} [p] Properties to set
                             */
                            function GoogleVoiceCallStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GoogleVoiceCallStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleVoiceCallStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum} GoogleVoiceCallStatusEnum instance
                             */
                            GoogleVoiceCallStatusEnum.create = function create(properties) {
                                return new GoogleVoiceCallStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleVoiceCallStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleVoiceCallStatusEnum} m GoogleVoiceCallStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleVoiceCallStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleVoiceCallStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IGoogleVoiceCallStatusEnum} message GoogleVoiceCallStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleVoiceCallStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleVoiceCallStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum} GoogleVoiceCallStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleVoiceCallStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleVoiceCallStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum} GoogleVoiceCallStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleVoiceCallStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleVoiceCallStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleVoiceCallStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GoogleVoiceCallStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum} GoogleVoiceCallStatusEnum
                             */
                            GoogleVoiceCallStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a GoogleVoiceCallStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum} m GoogleVoiceCallStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleVoiceCallStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GoogleVoiceCallStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleVoiceCallStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GoogleVoiceCallStatus enum.
                             * @name google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MISSED=2 MISSED value
                             * @property {number} RECEIVED=3 RECEIVED value
                             */
                            GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MISSED"] = 2;
                                values[valuesById[3] = "RECEIVED"] = 3;
                                return values;
                            })();
    
                            return GoogleVoiceCallStatusEnum;
                        })();
    
                        enums.HotelPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a HotelPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IHotelPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new HotelPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a HotelPlaceholderFieldEnum.
                             * @implements IHotelPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IHotelPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function HotelPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new HotelPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum} HotelPlaceholderFieldEnum instance
                             */
                            HotelPlaceholderFieldEnum.create = function create(properties) {
                                return new HotelPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified HotelPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelPlaceholderFieldEnum} m HotelPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelPlaceholderFieldEnum} message HotelPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum} HotelPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum} HotelPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a HotelPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum} HotelPlaceholderFieldEnum
                             */
                            HotelPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a HotelPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum} m HotelPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this HotelPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * HotelPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum.HotelPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PROPERTY_ID=2 PROPERTY_ID value
                             * @property {number} PROPERTY_NAME=3 PROPERTY_NAME value
                             * @property {number} DESTINATION_NAME=4 DESTINATION_NAME value
                             * @property {number} DESCRIPTION=5 DESCRIPTION value
                             * @property {number} ADDRESS=6 ADDRESS value
                             * @property {number} PRICE=7 PRICE value
                             * @property {number} FORMATTED_PRICE=8 FORMATTED_PRICE value
                             * @property {number} SALE_PRICE=9 SALE_PRICE value
                             * @property {number} FORMATTED_SALE_PRICE=10 FORMATTED_SALE_PRICE value
                             * @property {number} IMAGE_URL=11 IMAGE_URL value
                             * @property {number} CATEGORY=12 CATEGORY value
                             * @property {number} STAR_RATING=13 STAR_RATING value
                             * @property {number} CONTEXTUAL_KEYWORDS=14 CONTEXTUAL_KEYWORDS value
                             * @property {number} FINAL_URLS=15 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=16 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=17 TRACKING_URL value
                             * @property {number} ANDROID_APP_LINK=18 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_PROPERTY_IDS=19 SIMILAR_PROPERTY_IDS value
                             * @property {number} IOS_APP_LINK=20 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=21 IOS_APP_STORE_ID value
                             */
                            HotelPlaceholderFieldEnum.HotelPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PROPERTY_ID"] = 2;
                                values[valuesById[3] = "PROPERTY_NAME"] = 3;
                                values[valuesById[4] = "DESTINATION_NAME"] = 4;
                                values[valuesById[5] = "DESCRIPTION"] = 5;
                                values[valuesById[6] = "ADDRESS"] = 6;
                                values[valuesById[7] = "PRICE"] = 7;
                                values[valuesById[8] = "FORMATTED_PRICE"] = 8;
                                values[valuesById[9] = "SALE_PRICE"] = 9;
                                values[valuesById[10] = "FORMATTED_SALE_PRICE"] = 10;
                                values[valuesById[11] = "IMAGE_URL"] = 11;
                                values[valuesById[12] = "CATEGORY"] = 12;
                                values[valuesById[13] = "STAR_RATING"] = 13;
                                values[valuesById[14] = "CONTEXTUAL_KEYWORDS"] = 14;
                                values[valuesById[15] = "FINAL_URLS"] = 15;
                                values[valuesById[16] = "FINAL_MOBILE_URLS"] = 16;
                                values[valuesById[17] = "TRACKING_URL"] = 17;
                                values[valuesById[18] = "ANDROID_APP_LINK"] = 18;
                                values[valuesById[19] = "SIMILAR_PROPERTY_IDS"] = 19;
                                values[valuesById[20] = "IOS_APP_LINK"] = 20;
                                values[valuesById[21] = "IOS_APP_STORE_ID"] = 21;
                                return values;
                            })();
    
                            return HotelPlaceholderFieldEnum;
                        })();
    
                        enums.HotelReconciliationStatusEnum = (function() {
    
                            /**
                             * Properties of a HotelReconciliationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IHotelReconciliationStatusEnum
                             */
    
                            /**
                             * Constructs a new HotelReconciliationStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a HotelReconciliationStatusEnum.
                             * @implements IHotelReconciliationStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IHotelReconciliationStatusEnum=} [p] Properties to set
                             */
                            function HotelReconciliationStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new HotelReconciliationStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelReconciliationStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.HotelReconciliationStatusEnum} HotelReconciliationStatusEnum instance
                             */
                            HotelReconciliationStatusEnum.create = function create(properties) {
                                return new HotelReconciliationStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified HotelReconciliationStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.HotelReconciliationStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelReconciliationStatusEnum} m HotelReconciliationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelReconciliationStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelReconciliationStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.HotelReconciliationStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IHotelReconciliationStatusEnum} message HotelReconciliationStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelReconciliationStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelReconciliationStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.HotelReconciliationStatusEnum} HotelReconciliationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelReconciliationStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.HotelReconciliationStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelReconciliationStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.HotelReconciliationStatusEnum} HotelReconciliationStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelReconciliationStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelReconciliationStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelReconciliationStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a HotelReconciliationStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.HotelReconciliationStatusEnum} HotelReconciliationStatusEnum
                             */
                            HotelReconciliationStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.HotelReconciliationStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.HotelReconciliationStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a HotelReconciliationStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.HotelReconciliationStatusEnum} m HotelReconciliationStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelReconciliationStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this HotelReconciliationStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.HotelReconciliationStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelReconciliationStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * HotelReconciliationStatus enum.
                             * @name google.ads.googleads.v11.enums.HotelReconciliationStatusEnum.HotelReconciliationStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} RESERVATION_ENABLED=2 RESERVATION_ENABLED value
                             * @property {number} RECONCILIATION_NEEDED=3 RECONCILIATION_NEEDED value
                             * @property {number} RECONCILED=4 RECONCILED value
                             * @property {number} CANCELED=5 CANCELED value
                             */
                            HotelReconciliationStatusEnum.HotelReconciliationStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "RESERVATION_ENABLED"] = 2;
                                values[valuesById[3] = "RECONCILIATION_NEEDED"] = 3;
                                values[valuesById[4] = "RECONCILED"] = 4;
                                values[valuesById[5] = "CANCELED"] = 5;
                                return values;
                            })();
    
                            return HotelReconciliationStatusEnum;
                        })();
    
                        enums.ImagePlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of an ImagePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IImagePlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new ImagePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an ImagePlaceholderFieldEnum.
                             * @implements IImagePlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IImagePlaceholderFieldEnum=} [p] Properties to set
                             */
                            function ImagePlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ImagePlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IImagePlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum} ImagePlaceholderFieldEnum instance
                             */
                            ImagePlaceholderFieldEnum.create = function create(properties) {
                                return new ImagePlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ImagePlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IImagePlaceholderFieldEnum} m ImagePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImagePlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ImagePlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IImagePlaceholderFieldEnum} message ImagePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImagePlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ImagePlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum} ImagePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImagePlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ImagePlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum} ImagePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImagePlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ImagePlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ImagePlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ImagePlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum} ImagePlaceholderFieldEnum
                             */
                            ImagePlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from an ImagePlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum} m ImagePlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ImagePlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ImagePlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ImagePlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ImagePlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum.ImagePlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ASSET_ID=2 ASSET_ID value
                             */
                            ImagePlaceholderFieldEnum.ImagePlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ASSET_ID"] = 2;
                                return values;
                            })();
    
                            return ImagePlaceholderFieldEnum;
                        })();
    
                        enums.InvoiceTypeEnum = (function() {
    
                            /**
                             * Properties of an InvoiceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IInvoiceTypeEnum
                             */
    
                            /**
                             * Constructs a new InvoiceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an InvoiceTypeEnum.
                             * @implements IInvoiceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IInvoiceTypeEnum=} [p] Properties to set
                             */
                            function InvoiceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new InvoiceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInvoiceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.InvoiceTypeEnum} InvoiceTypeEnum instance
                             */
                            InvoiceTypeEnum.create = function create(properties) {
                                return new InvoiceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified InvoiceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.InvoiceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInvoiceTypeEnum} m InvoiceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InvoiceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified InvoiceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.InvoiceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IInvoiceTypeEnum} message InvoiceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InvoiceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an InvoiceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.InvoiceTypeEnum} InvoiceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InvoiceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.InvoiceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an InvoiceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.InvoiceTypeEnum} InvoiceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InvoiceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an InvoiceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            InvoiceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an InvoiceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.InvoiceTypeEnum} InvoiceTypeEnum
                             */
                            InvoiceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.InvoiceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.InvoiceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an InvoiceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.InvoiceTypeEnum} m InvoiceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            InvoiceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this InvoiceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.InvoiceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            InvoiceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * InvoiceType enum.
                             * @name google.ads.googleads.v11.enums.InvoiceTypeEnum.InvoiceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CREDIT_MEMO=2 CREDIT_MEMO value
                             * @property {number} INVOICE=3 INVOICE value
                             */
                            InvoiceTypeEnum.InvoiceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CREDIT_MEMO"] = 2;
                                values[valuesById[3] = "INVOICE"] = 3;
                                return values;
                            })();
    
                            return InvoiceTypeEnum;
                        })();
    
                        enums.JobPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a JobPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IJobPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new JobPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a JobPlaceholderFieldEnum.
                             * @implements IJobPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IJobPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function JobPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new JobPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IJobPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.JobPlaceholderFieldEnum} JobPlaceholderFieldEnum instance
                             */
                            JobPlaceholderFieldEnum.create = function create(properties) {
                                return new JobPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified JobPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.JobPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IJobPlaceholderFieldEnum} m JobPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            JobPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified JobPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.JobPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IJobPlaceholderFieldEnum} message JobPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            JobPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a JobPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.JobPlaceholderFieldEnum} JobPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            JobPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.JobPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a JobPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.JobPlaceholderFieldEnum} JobPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            JobPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a JobPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            JobPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a JobPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.JobPlaceholderFieldEnum} JobPlaceholderFieldEnum
                             */
                            JobPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.JobPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.JobPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a JobPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.JobPlaceholderFieldEnum} m JobPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            JobPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this JobPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.JobPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            JobPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * JobPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.JobPlaceholderFieldEnum.JobPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} JOB_ID=2 JOB_ID value
                             * @property {number} LOCATION_ID=3 LOCATION_ID value
                             * @property {number} TITLE=4 TITLE value
                             * @property {number} SUBTITLE=5 SUBTITLE value
                             * @property {number} DESCRIPTION=6 DESCRIPTION value
                             * @property {number} IMAGE_URL=7 IMAGE_URL value
                             * @property {number} CATEGORY=8 CATEGORY value
                             * @property {number} CONTEXTUAL_KEYWORDS=9 CONTEXTUAL_KEYWORDS value
                             * @property {number} ADDRESS=10 ADDRESS value
                             * @property {number} SALARY=11 SALARY value
                             * @property {number} FINAL_URLS=12 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=14 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=15 TRACKING_URL value
                             * @property {number} ANDROID_APP_LINK=16 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_JOB_IDS=17 SIMILAR_JOB_IDS value
                             * @property {number} IOS_APP_LINK=18 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=19 IOS_APP_STORE_ID value
                             */
                            JobPlaceholderFieldEnum.JobPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "JOB_ID"] = 2;
                                values[valuesById[3] = "LOCATION_ID"] = 3;
                                values[valuesById[4] = "TITLE"] = 4;
                                values[valuesById[5] = "SUBTITLE"] = 5;
                                values[valuesById[6] = "DESCRIPTION"] = 6;
                                values[valuesById[7] = "IMAGE_URL"] = 7;
                                values[valuesById[8] = "CATEGORY"] = 8;
                                values[valuesById[9] = "CONTEXTUAL_KEYWORDS"] = 9;
                                values[valuesById[10] = "ADDRESS"] = 10;
                                values[valuesById[11] = "SALARY"] = 11;
                                values[valuesById[12] = "FINAL_URLS"] = 12;
                                values[valuesById[14] = "FINAL_MOBILE_URLS"] = 14;
                                values[valuesById[15] = "TRACKING_URL"] = 15;
                                values[valuesById[16] = "ANDROID_APP_LINK"] = 16;
                                values[valuesById[17] = "SIMILAR_JOB_IDS"] = 17;
                                values[valuesById[18] = "IOS_APP_LINK"] = 18;
                                values[valuesById[19] = "IOS_APP_STORE_ID"] = 19;
                                return values;
                            })();
    
                            return JobPlaceholderFieldEnum;
                        })();
    
                        enums.KeywordPlanForecastIntervalEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanForecastIntervalEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IKeywordPlanForecastIntervalEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanForecastIntervalEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a KeywordPlanForecastIntervalEnum.
                             * @implements IKeywordPlanForecastIntervalEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanForecastIntervalEnum=} [p] Properties to set
                             */
                            function KeywordPlanForecastIntervalEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanForecastIntervalEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanForecastIntervalEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum} KeywordPlanForecastIntervalEnum instance
                             */
                            KeywordPlanForecastIntervalEnum.create = function create(properties) {
                                return new KeywordPlanForecastIntervalEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanForecastIntervalEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanForecastIntervalEnum} m KeywordPlanForecastIntervalEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanForecastIntervalEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanForecastIntervalEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanForecastIntervalEnum} message KeywordPlanForecastIntervalEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanForecastIntervalEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanForecastIntervalEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum} KeywordPlanForecastIntervalEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanForecastIntervalEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanForecastIntervalEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum} KeywordPlanForecastIntervalEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanForecastIntervalEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanForecastIntervalEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanForecastIntervalEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanForecastIntervalEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum} KeywordPlanForecastIntervalEnum
                             */
                            KeywordPlanForecastIntervalEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanForecastIntervalEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum} m KeywordPlanForecastIntervalEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanForecastIntervalEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanForecastIntervalEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanForecastIntervalEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanForecastInterval enum.
                             * @name google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NEXT_WEEK=3 NEXT_WEEK value
                             * @property {number} NEXT_MONTH=4 NEXT_MONTH value
                             * @property {number} NEXT_QUARTER=5 NEXT_QUARTER value
                             */
                            KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "NEXT_WEEK"] = 3;
                                values[valuesById[4] = "NEXT_MONTH"] = 4;
                                values[valuesById[5] = "NEXT_QUARTER"] = 5;
                                return values;
                            })();
    
                            return KeywordPlanForecastIntervalEnum;
                        })();
    
                        enums.KeywordPlanKeywordAnnotationEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanKeywordAnnotationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IKeywordPlanKeywordAnnotationEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanKeywordAnnotationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a KeywordPlanKeywordAnnotationEnum.
                             * @implements IKeywordPlanKeywordAnnotationEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanKeywordAnnotationEnum=} [p] Properties to set
                             */
                            function KeywordPlanKeywordAnnotationEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanKeywordAnnotationEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanKeywordAnnotationEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum} KeywordPlanKeywordAnnotationEnum instance
                             */
                            KeywordPlanKeywordAnnotationEnum.create = function create(properties) {
                                return new KeywordPlanKeywordAnnotationEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanKeywordAnnotationEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanKeywordAnnotationEnum} m KeywordPlanKeywordAnnotationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanKeywordAnnotationEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanKeywordAnnotationEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanKeywordAnnotationEnum} message KeywordPlanKeywordAnnotationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanKeywordAnnotationEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanKeywordAnnotationEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum} KeywordPlanKeywordAnnotationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanKeywordAnnotationEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanKeywordAnnotationEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum} KeywordPlanKeywordAnnotationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanKeywordAnnotationEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanKeywordAnnotationEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanKeywordAnnotationEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanKeywordAnnotationEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum} KeywordPlanKeywordAnnotationEnum
                             */
                            KeywordPlanKeywordAnnotationEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanKeywordAnnotationEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum} m KeywordPlanKeywordAnnotationEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanKeywordAnnotationEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanKeywordAnnotationEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanKeywordAnnotationEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanKeywordAnnotation enum.
                             * @name google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} KEYWORD_CONCEPT=2 KEYWORD_CONCEPT value
                             */
                            KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "KEYWORD_CONCEPT"] = 2;
                                return values;
                            })();
    
                            return KeywordPlanKeywordAnnotationEnum;
                        })();
    
                        enums.KeywordPlanNetworkEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanNetworkEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IKeywordPlanNetworkEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanNetworkEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a KeywordPlanNetworkEnum.
                             * @implements IKeywordPlanNetworkEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanNetworkEnum=} [p] Properties to set
                             */
                            function KeywordPlanNetworkEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanNetworkEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanNetworkEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum} KeywordPlanNetworkEnum instance
                             */
                            KeywordPlanNetworkEnum.create = function create(properties) {
                                return new KeywordPlanNetworkEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanNetworkEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanNetworkEnum} m KeywordPlanNetworkEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanNetworkEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanNetworkEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IKeywordPlanNetworkEnum} message KeywordPlanNetworkEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanNetworkEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanNetworkEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum} KeywordPlanNetworkEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanNetworkEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.KeywordPlanNetworkEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanNetworkEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum} KeywordPlanNetworkEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanNetworkEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanNetworkEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanNetworkEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanNetworkEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum} KeywordPlanNetworkEnum
                             */
                            KeywordPlanNetworkEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.KeywordPlanNetworkEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.KeywordPlanNetworkEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanNetworkEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum} m KeywordPlanNetworkEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanNetworkEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanNetworkEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.KeywordPlanNetworkEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanNetworkEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanNetwork enum.
                             * @name google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} GOOGLE_SEARCH=2 GOOGLE_SEARCH value
                             * @property {number} GOOGLE_SEARCH_AND_PARTNERS=3 GOOGLE_SEARCH_AND_PARTNERS value
                             */
                            KeywordPlanNetworkEnum.KeywordPlanNetwork = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "GOOGLE_SEARCH"] = 2;
                                values[valuesById[3] = "GOOGLE_SEARCH_AND_PARTNERS"] = 3;
                                return values;
                            })();
    
                            return KeywordPlanNetworkEnum;
                        })();
    
                        enums.LabelStatusEnum = (function() {
    
                            /**
                             * Properties of a LabelStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILabelStatusEnum
                             */
    
                            /**
                             * Constructs a new LabelStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LabelStatusEnum.
                             * @implements ILabelStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILabelStatusEnum=} [p] Properties to set
                             */
                            function LabelStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LabelStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILabelStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LabelStatusEnum} LabelStatusEnum instance
                             */
                            LabelStatusEnum.create = function create(properties) {
                                return new LabelStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LabelStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LabelStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILabelStatusEnum} m LabelStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LabelStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LabelStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LabelStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILabelStatusEnum} message LabelStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LabelStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LabelStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LabelStatusEnum} LabelStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LabelStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LabelStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LabelStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LabelStatusEnum} LabelStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LabelStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LabelStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LabelStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LabelStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LabelStatusEnum} LabelStatusEnum
                             */
                            LabelStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LabelStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LabelStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a LabelStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LabelStatusEnum} m LabelStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LabelStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LabelStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LabelStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LabelStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LabelStatus enum.
                             * @name google.ads.googleads.v11.enums.LabelStatusEnum.LabelStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            LabelStatusEnum.LabelStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return LabelStatusEnum;
                        })();
    
                        enums.LinkedAccountTypeEnum = (function() {
    
                            /**
                             * Properties of a LinkedAccountTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILinkedAccountTypeEnum
                             */
    
                            /**
                             * Constructs a new LinkedAccountTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LinkedAccountTypeEnum.
                             * @implements ILinkedAccountTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILinkedAccountTypeEnum=} [p] Properties to set
                             */
                            function LinkedAccountTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LinkedAccountTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILinkedAccountTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LinkedAccountTypeEnum} LinkedAccountTypeEnum instance
                             */
                            LinkedAccountTypeEnum.create = function create(properties) {
                                return new LinkedAccountTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LinkedAccountTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LinkedAccountTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILinkedAccountTypeEnum} m LinkedAccountTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LinkedAccountTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LinkedAccountTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LinkedAccountTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILinkedAccountTypeEnum} message LinkedAccountTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LinkedAccountTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LinkedAccountTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LinkedAccountTypeEnum} LinkedAccountTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LinkedAccountTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LinkedAccountTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LinkedAccountTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LinkedAccountTypeEnum} LinkedAccountTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LinkedAccountTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LinkedAccountTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LinkedAccountTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LinkedAccountTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LinkedAccountTypeEnum} LinkedAccountTypeEnum
                             */
                            LinkedAccountTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LinkedAccountTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LinkedAccountTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a LinkedAccountTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LinkedAccountTypeEnum} m LinkedAccountTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LinkedAccountTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LinkedAccountTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LinkedAccountTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LinkedAccountTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LinkedAccountType enum.
                             * @name google.ads.googleads.v11.enums.LinkedAccountTypeEnum.LinkedAccountType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} THIRD_PARTY_APP_ANALYTICS=2 THIRD_PARTY_APP_ANALYTICS value
                             * @property {number} DATA_PARTNER=3 DATA_PARTNER value
                             * @property {number} GOOGLE_ADS=4 GOOGLE_ADS value
                             * @property {number} HOTEL_CENTER=5 HOTEL_CENTER value
                             */
                            LinkedAccountTypeEnum.LinkedAccountType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "THIRD_PARTY_APP_ANALYTICS"] = 2;
                                values[valuesById[3] = "DATA_PARTNER"] = 3;
                                values[valuesById[4] = "GOOGLE_ADS"] = 4;
                                values[valuesById[5] = "HOTEL_CENTER"] = 5;
                                return values;
                            })();
    
                            return LinkedAccountTypeEnum;
                        })();
    
                        enums.ListingGroupFilterBiddingCategoryLevelEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterBiddingCategoryLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupFilterBiddingCategoryLevelEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterBiddingCategoryLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupFilterBiddingCategoryLevelEnum.
                             * @implements IListingGroupFilterBiddingCategoryLevelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterBiddingCategoryLevelEnum=} [p] Properties to set
                             */
                            function ListingGroupFilterBiddingCategoryLevelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupFilterBiddingCategoryLevelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterBiddingCategoryLevelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum} ListingGroupFilterBiddingCategoryLevelEnum instance
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.create = function create(properties) {
                                return new ListingGroupFilterBiddingCategoryLevelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterBiddingCategoryLevelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterBiddingCategoryLevelEnum} m ListingGroupFilterBiddingCategoryLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterBiddingCategoryLevelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterBiddingCategoryLevelEnum} message ListingGroupFilterBiddingCategoryLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterBiddingCategoryLevelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum} ListingGroupFilterBiddingCategoryLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterBiddingCategoryLevelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum} ListingGroupFilterBiddingCategoryLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterBiddingCategoryLevelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterBiddingCategoryLevelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum} ListingGroupFilterBiddingCategoryLevelEnum
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterBiddingCategoryLevelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum} m ListingGroupFilterBiddingCategoryLevelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupFilterBiddingCategoryLevelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupFilterBiddingCategoryLevel enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum.ListingGroupFilterBiddingCategoryLevel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LEVEL1=2 LEVEL1 value
                             * @property {number} LEVEL2=3 LEVEL2 value
                             * @property {number} LEVEL3=4 LEVEL3 value
                             * @property {number} LEVEL4=5 LEVEL4 value
                             * @property {number} LEVEL5=6 LEVEL5 value
                             */
                            ListingGroupFilterBiddingCategoryLevelEnum.ListingGroupFilterBiddingCategoryLevel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LEVEL1"] = 2;
                                values[valuesById[3] = "LEVEL2"] = 3;
                                values[valuesById[4] = "LEVEL3"] = 4;
                                values[valuesById[5] = "LEVEL4"] = 5;
                                values[valuesById[6] = "LEVEL5"] = 6;
                                return values;
                            })();
    
                            return ListingGroupFilterBiddingCategoryLevelEnum;
                        })();
    
                        enums.ListingGroupFilterCustomAttributeIndexEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterCustomAttributeIndexEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupFilterCustomAttributeIndexEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterCustomAttributeIndexEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupFilterCustomAttributeIndexEnum.
                             * @implements IListingGroupFilterCustomAttributeIndexEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterCustomAttributeIndexEnum=} [p] Properties to set
                             */
                            function ListingGroupFilterCustomAttributeIndexEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupFilterCustomAttributeIndexEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterCustomAttributeIndexEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum} ListingGroupFilterCustomAttributeIndexEnum instance
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.create = function create(properties) {
                                return new ListingGroupFilterCustomAttributeIndexEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterCustomAttributeIndexEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterCustomAttributeIndexEnum} m ListingGroupFilterCustomAttributeIndexEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterCustomAttributeIndexEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterCustomAttributeIndexEnum} message ListingGroupFilterCustomAttributeIndexEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterCustomAttributeIndexEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum} ListingGroupFilterCustomAttributeIndexEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterCustomAttributeIndexEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum} ListingGroupFilterCustomAttributeIndexEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterCustomAttributeIndexEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterCustomAttributeIndexEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum} ListingGroupFilterCustomAttributeIndexEnum
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterCustomAttributeIndexEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum} m ListingGroupFilterCustomAttributeIndexEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupFilterCustomAttributeIndexEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupFilterCustomAttributeIndex enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INDEX0=2 INDEX0 value
                             * @property {number} INDEX1=3 INDEX1 value
                             * @property {number} INDEX2=4 INDEX2 value
                             * @property {number} INDEX3=5 INDEX3 value
                             * @property {number} INDEX4=6 INDEX4 value
                             */
                            ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INDEX0"] = 2;
                                values[valuesById[3] = "INDEX1"] = 3;
                                values[valuesById[4] = "INDEX2"] = 4;
                                values[valuesById[5] = "INDEX3"] = 5;
                                values[valuesById[6] = "INDEX4"] = 6;
                                return values;
                            })();
    
                            return ListingGroupFilterCustomAttributeIndexEnum;
                        })();
    
                        enums.ListingGroupFilterProductChannelEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterProductChannelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupFilterProductChannelEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterProductChannelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupFilterProductChannelEnum.
                             * @implements IListingGroupFilterProductChannelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductChannelEnum=} [p] Properties to set
                             */
                            function ListingGroupFilterProductChannelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupFilterProductChannelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductChannelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum} ListingGroupFilterProductChannelEnum instance
                             */
                            ListingGroupFilterProductChannelEnum.create = function create(properties) {
                                return new ListingGroupFilterProductChannelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterProductChannelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductChannelEnum} m ListingGroupFilterProductChannelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterProductChannelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterProductChannelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductChannelEnum} message ListingGroupFilterProductChannelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterProductChannelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterProductChannelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum} ListingGroupFilterProductChannelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterProductChannelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterProductChannelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum} ListingGroupFilterProductChannelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterProductChannelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterProductChannelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterProductChannelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterProductChannelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum} ListingGroupFilterProductChannelEnum
                             */
                            ListingGroupFilterProductChannelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterProductChannelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum} m ListingGroupFilterProductChannelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterProductChannelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupFilterProductChannelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterProductChannelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupFilterProductChannel enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ONLINE=2 ONLINE value
                             * @property {number} LOCAL=3 LOCAL value
                             */
                            ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ONLINE"] = 2;
                                values[valuesById[3] = "LOCAL"] = 3;
                                return values;
                            })();
    
                            return ListingGroupFilterProductChannelEnum;
                        })();
    
                        enums.ListingGroupFilterProductConditionEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterProductConditionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupFilterProductConditionEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterProductConditionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupFilterProductConditionEnum.
                             * @implements IListingGroupFilterProductConditionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductConditionEnum=} [p] Properties to set
                             */
                            function ListingGroupFilterProductConditionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupFilterProductConditionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductConditionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum} ListingGroupFilterProductConditionEnum instance
                             */
                            ListingGroupFilterProductConditionEnum.create = function create(properties) {
                                return new ListingGroupFilterProductConditionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterProductConditionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductConditionEnum} m ListingGroupFilterProductConditionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterProductConditionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterProductConditionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductConditionEnum} message ListingGroupFilterProductConditionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterProductConditionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterProductConditionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum} ListingGroupFilterProductConditionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterProductConditionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterProductConditionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum} ListingGroupFilterProductConditionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterProductConditionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterProductConditionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterProductConditionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterProductConditionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum} ListingGroupFilterProductConditionEnum
                             */
                            ListingGroupFilterProductConditionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterProductConditionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum} m ListingGroupFilterProductConditionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterProductConditionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupFilterProductConditionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterProductConditionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupFilterProductCondition enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NEW=2 NEW value
                             * @property {number} REFURBISHED=3 REFURBISHED value
                             * @property {number} USED=4 USED value
                             */
                            ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NEW"] = 2;
                                values[valuesById[3] = "REFURBISHED"] = 3;
                                values[valuesById[4] = "USED"] = 4;
                                return values;
                            })();
    
                            return ListingGroupFilterProductConditionEnum;
                        })();
    
                        enums.ListingGroupFilterProductTypeLevelEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterProductTypeLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupFilterProductTypeLevelEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterProductTypeLevelEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupFilterProductTypeLevelEnum.
                             * @implements IListingGroupFilterProductTypeLevelEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductTypeLevelEnum=} [p] Properties to set
                             */
                            function ListingGroupFilterProductTypeLevelEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupFilterProductTypeLevelEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductTypeLevelEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum} ListingGroupFilterProductTypeLevelEnum instance
                             */
                            ListingGroupFilterProductTypeLevelEnum.create = function create(properties) {
                                return new ListingGroupFilterProductTypeLevelEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterProductTypeLevelEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductTypeLevelEnum} m ListingGroupFilterProductTypeLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterProductTypeLevelEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterProductTypeLevelEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterProductTypeLevelEnum} message ListingGroupFilterProductTypeLevelEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterProductTypeLevelEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterProductTypeLevelEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum} ListingGroupFilterProductTypeLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterProductTypeLevelEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterProductTypeLevelEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum} ListingGroupFilterProductTypeLevelEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterProductTypeLevelEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterProductTypeLevelEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterProductTypeLevelEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterProductTypeLevelEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum} ListingGroupFilterProductTypeLevelEnum
                             */
                            ListingGroupFilterProductTypeLevelEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterProductTypeLevelEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum} m ListingGroupFilterProductTypeLevelEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterProductTypeLevelEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupFilterProductTypeLevelEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterProductTypeLevelEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupFilterProductTypeLevel enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LEVEL1=2 LEVEL1 value
                             * @property {number} LEVEL2=3 LEVEL2 value
                             * @property {number} LEVEL3=4 LEVEL3 value
                             * @property {number} LEVEL4=5 LEVEL4 value
                             * @property {number} LEVEL5=6 LEVEL5 value
                             */
                            ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LEVEL1"] = 2;
                                values[valuesById[3] = "LEVEL2"] = 3;
                                values[valuesById[4] = "LEVEL3"] = 4;
                                values[valuesById[5] = "LEVEL4"] = 5;
                                values[valuesById[6] = "LEVEL5"] = 6;
                                return values;
                            })();
    
                            return ListingGroupFilterProductTypeLevelEnum;
                        })();
    
                        enums.ListingGroupFilterTypeEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupFilterTypeEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupFilterTypeEnum.
                             * @implements IListingGroupFilterTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterTypeEnum=} [p] Properties to set
                             */
                            function ListingGroupFilterTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupFilterTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum} ListingGroupFilterTypeEnum instance
                             */
                            ListingGroupFilterTypeEnum.create = function create(properties) {
                                return new ListingGroupFilterTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterTypeEnum} m ListingGroupFilterTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterTypeEnum} message ListingGroupFilterTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum} ListingGroupFilterTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum} ListingGroupFilterTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum} ListingGroupFilterTypeEnum
                             */
                            ListingGroupFilterTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum} m ListingGroupFilterTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupFilterTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupFilterType enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum.ListingGroupFilterType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SUBDIVISION=2 SUBDIVISION value
                             * @property {number} UNIT_INCLUDED=3 UNIT_INCLUDED value
                             * @property {number} UNIT_EXCLUDED=4 UNIT_EXCLUDED value
                             */
                            ListingGroupFilterTypeEnum.ListingGroupFilterType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SUBDIVISION"] = 2;
                                values[valuesById[3] = "UNIT_INCLUDED"] = 3;
                                values[valuesById[4] = "UNIT_EXCLUDED"] = 4;
                                return values;
                            })();
    
                            return ListingGroupFilterTypeEnum;
                        })();
    
                        enums.ListingGroupFilterVerticalEnum = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterVerticalEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IListingGroupFilterVerticalEnum
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterVerticalEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ListingGroupFilterVerticalEnum.
                             * @implements IListingGroupFilterVerticalEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterVerticalEnum=} [p] Properties to set
                             */
                            function ListingGroupFilterVerticalEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListingGroupFilterVerticalEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterVerticalEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum} ListingGroupFilterVerticalEnum instance
                             */
                            ListingGroupFilterVerticalEnum.create = function create(properties) {
                                return new ListingGroupFilterVerticalEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterVerticalEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterVerticalEnum} m ListingGroupFilterVerticalEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterVerticalEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterVerticalEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IListingGroupFilterVerticalEnum} message ListingGroupFilterVerticalEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterVerticalEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterVerticalEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum} ListingGroupFilterVerticalEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterVerticalEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterVerticalEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum} ListingGroupFilterVerticalEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterVerticalEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterVerticalEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterVerticalEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterVerticalEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum} ListingGroupFilterVerticalEnum
                             */
                            ListingGroupFilterVerticalEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterVerticalEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum} m ListingGroupFilterVerticalEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterVerticalEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListingGroupFilterVerticalEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterVerticalEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListingGroupFilterVertical enum.
                             * @name google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum.ListingGroupFilterVertical
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SHOPPING=2 SHOPPING value
                             */
                            ListingGroupFilterVerticalEnum.ListingGroupFilterVertical = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SHOPPING"] = 2;
                                return values;
                            })();
    
                            return ListingGroupFilterVerticalEnum;
                        })();
    
                        enums.LocalPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a LocalPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILocalPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new LocalPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LocalPlaceholderFieldEnum.
                             * @implements ILocalPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILocalPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function LocalPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LocalPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocalPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum} LocalPlaceholderFieldEnum instance
                             */
                            LocalPlaceholderFieldEnum.create = function create(properties) {
                                return new LocalPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LocalPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocalPlaceholderFieldEnum} m LocalPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocalPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocalPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocalPlaceholderFieldEnum} message LocalPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocalPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocalPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum} LocalPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocalPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocalPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum} LocalPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocalPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocalPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocalPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LocalPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum} LocalPlaceholderFieldEnum
                             */
                            LocalPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a LocalPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum} m LocalPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocalPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LocalPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocalPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LocalPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum.LocalPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DEAL_ID=2 DEAL_ID value
                             * @property {number} DEAL_NAME=3 DEAL_NAME value
                             * @property {number} SUBTITLE=4 SUBTITLE value
                             * @property {number} DESCRIPTION=5 DESCRIPTION value
                             * @property {number} PRICE=6 PRICE value
                             * @property {number} FORMATTED_PRICE=7 FORMATTED_PRICE value
                             * @property {number} SALE_PRICE=8 SALE_PRICE value
                             * @property {number} FORMATTED_SALE_PRICE=9 FORMATTED_SALE_PRICE value
                             * @property {number} IMAGE_URL=10 IMAGE_URL value
                             * @property {number} ADDRESS=11 ADDRESS value
                             * @property {number} CATEGORY=12 CATEGORY value
                             * @property {number} CONTEXTUAL_KEYWORDS=13 CONTEXTUAL_KEYWORDS value
                             * @property {number} FINAL_URLS=14 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=15 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=16 TRACKING_URL value
                             * @property {number} ANDROID_APP_LINK=17 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_DEAL_IDS=18 SIMILAR_DEAL_IDS value
                             * @property {number} IOS_APP_LINK=19 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=20 IOS_APP_STORE_ID value
                             */
                            LocalPlaceholderFieldEnum.LocalPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DEAL_ID"] = 2;
                                values[valuesById[3] = "DEAL_NAME"] = 3;
                                values[valuesById[4] = "SUBTITLE"] = 4;
                                values[valuesById[5] = "DESCRIPTION"] = 5;
                                values[valuesById[6] = "PRICE"] = 6;
                                values[valuesById[7] = "FORMATTED_PRICE"] = 7;
                                values[valuesById[8] = "SALE_PRICE"] = 8;
                                values[valuesById[9] = "FORMATTED_SALE_PRICE"] = 9;
                                values[valuesById[10] = "IMAGE_URL"] = 10;
                                values[valuesById[11] = "ADDRESS"] = 11;
                                values[valuesById[12] = "CATEGORY"] = 12;
                                values[valuesById[13] = "CONTEXTUAL_KEYWORDS"] = 13;
                                values[valuesById[14] = "FINAL_URLS"] = 14;
                                values[valuesById[15] = "FINAL_MOBILE_URLS"] = 15;
                                values[valuesById[16] = "TRACKING_URL"] = 16;
                                values[valuesById[17] = "ANDROID_APP_LINK"] = 17;
                                values[valuesById[18] = "SIMILAR_DEAL_IDS"] = 18;
                                values[valuesById[19] = "IOS_APP_LINK"] = 19;
                                values[valuesById[20] = "IOS_APP_STORE_ID"] = 20;
                                return values;
                            })();
    
                            return LocalPlaceholderFieldEnum;
                        })();
    
                        enums.LocationExtensionTargetingCriterionFieldEnum = (function() {
    
                            /**
                             * Properties of a LocationExtensionTargetingCriterionFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILocationExtensionTargetingCriterionFieldEnum
                             */
    
                            /**
                             * Constructs a new LocationExtensionTargetingCriterionFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LocationExtensionTargetingCriterionFieldEnum.
                             * @implements ILocationExtensionTargetingCriterionFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILocationExtensionTargetingCriterionFieldEnum=} [p] Properties to set
                             */
                            function LocationExtensionTargetingCriterionFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LocationExtensionTargetingCriterionFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationExtensionTargetingCriterionFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum} LocationExtensionTargetingCriterionFieldEnum instance
                             */
                            LocationExtensionTargetingCriterionFieldEnum.create = function create(properties) {
                                return new LocationExtensionTargetingCriterionFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LocationExtensionTargetingCriterionFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationExtensionTargetingCriterionFieldEnum} m LocationExtensionTargetingCriterionFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationExtensionTargetingCriterionFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationExtensionTargetingCriterionFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationExtensionTargetingCriterionFieldEnum} message LocationExtensionTargetingCriterionFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationExtensionTargetingCriterionFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationExtensionTargetingCriterionFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum} LocationExtensionTargetingCriterionFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationExtensionTargetingCriterionFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationExtensionTargetingCriterionFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum} LocationExtensionTargetingCriterionFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationExtensionTargetingCriterionFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationExtensionTargetingCriterionFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationExtensionTargetingCriterionFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LocationExtensionTargetingCriterionFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum} LocationExtensionTargetingCriterionFieldEnum
                             */
                            LocationExtensionTargetingCriterionFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a LocationExtensionTargetingCriterionFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum} m LocationExtensionTargetingCriterionFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationExtensionTargetingCriterionFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LocationExtensionTargetingCriterionFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationExtensionTargetingCriterionFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LocationExtensionTargetingCriterionField enum.
                             * @name google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADDRESS_LINE_1=2 ADDRESS_LINE_1 value
                             * @property {number} ADDRESS_LINE_2=3 ADDRESS_LINE_2 value
                             * @property {number} CITY=4 CITY value
                             * @property {number} PROVINCE=5 PROVINCE value
                             * @property {number} POSTAL_CODE=6 POSTAL_CODE value
                             * @property {number} COUNTRY_CODE=7 COUNTRY_CODE value
                             */
                            LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADDRESS_LINE_1"] = 2;
                                values[valuesById[3] = "ADDRESS_LINE_2"] = 3;
                                values[valuesById[4] = "CITY"] = 4;
                                values[valuesById[5] = "PROVINCE"] = 5;
                                values[valuesById[6] = "POSTAL_CODE"] = 6;
                                values[valuesById[7] = "COUNTRY_CODE"] = 7;
                                return values;
                            })();
    
                            return LocationExtensionTargetingCriterionFieldEnum;
                        })();
    
                        enums.LocationPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a LocationPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILocationPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new LocationPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LocationPlaceholderFieldEnum.
                             * @implements ILocationPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILocationPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function LocationPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LocationPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum} LocationPlaceholderFieldEnum instance
                             */
                            LocationPlaceholderFieldEnum.create = function create(properties) {
                                return new LocationPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LocationPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationPlaceholderFieldEnum} m LocationPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationPlaceholderFieldEnum} message LocationPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum} LocationPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum} LocationPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LocationPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum} LocationPlaceholderFieldEnum
                             */
                            LocationPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a LocationPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum} m LocationPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LocationPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LocationPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum.LocationPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BUSINESS_NAME=2 BUSINESS_NAME value
                             * @property {number} ADDRESS_LINE_1=3 ADDRESS_LINE_1 value
                             * @property {number} ADDRESS_LINE_2=4 ADDRESS_LINE_2 value
                             * @property {number} CITY=5 CITY value
                             * @property {number} PROVINCE=6 PROVINCE value
                             * @property {number} POSTAL_CODE=7 POSTAL_CODE value
                             * @property {number} COUNTRY_CODE=8 COUNTRY_CODE value
                             * @property {number} PHONE_NUMBER=9 PHONE_NUMBER value
                             */
                            LocationPlaceholderFieldEnum.LocationPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BUSINESS_NAME"] = 2;
                                values[valuesById[3] = "ADDRESS_LINE_1"] = 3;
                                values[valuesById[4] = "ADDRESS_LINE_2"] = 4;
                                values[valuesById[5] = "CITY"] = 5;
                                values[valuesById[6] = "PROVINCE"] = 6;
                                values[valuesById[7] = "POSTAL_CODE"] = 7;
                                values[valuesById[8] = "COUNTRY_CODE"] = 8;
                                values[valuesById[9] = "PHONE_NUMBER"] = 9;
                                return values;
                            })();
    
                            return LocationPlaceholderFieldEnum;
                        })();
    
                        enums.LocationSourceTypeEnum = (function() {
    
                            /**
                             * Properties of a LocationSourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ILocationSourceTypeEnum
                             */
    
                            /**
                             * Constructs a new LocationSourceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a LocationSourceTypeEnum.
                             * @implements ILocationSourceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ILocationSourceTypeEnum=} [p] Properties to set
                             */
                            function LocationSourceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LocationSourceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationSourceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.LocationSourceTypeEnum} LocationSourceTypeEnum instance
                             */
                            LocationSourceTypeEnum.create = function create(properties) {
                                return new LocationSourceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LocationSourceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.LocationSourceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationSourceTypeEnum} m LocationSourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationSourceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationSourceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.LocationSourceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ILocationSourceTypeEnum} message LocationSourceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationSourceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationSourceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.LocationSourceTypeEnum} LocationSourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationSourceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.LocationSourceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationSourceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.LocationSourceTypeEnum} LocationSourceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationSourceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationSourceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationSourceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LocationSourceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.LocationSourceTypeEnum} LocationSourceTypeEnum
                             */
                            LocationSourceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.LocationSourceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.LocationSourceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a LocationSourceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.LocationSourceTypeEnum} m LocationSourceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationSourceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LocationSourceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.LocationSourceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationSourceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LocationSourceType enum.
                             * @name google.ads.googleads.v11.enums.LocationSourceTypeEnum.LocationSourceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} GOOGLE_MY_BUSINESS=2 GOOGLE_MY_BUSINESS value
                             * @property {number} AFFILIATE=3 AFFILIATE value
                             */
                            LocationSourceTypeEnum.LocationSourceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "GOOGLE_MY_BUSINESS"] = 2;
                                values[valuesById[3] = "AFFILIATE"] = 3;
                                return values;
                            })();
    
                            return LocationSourceTypeEnum;
                        })();
    
                        enums.ManagerLinkStatusEnum = (function() {
    
                            /**
                             * Properties of a ManagerLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IManagerLinkStatusEnum
                             */
    
                            /**
                             * Constructs a new ManagerLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ManagerLinkStatusEnum.
                             * @implements IManagerLinkStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IManagerLinkStatusEnum=} [p] Properties to set
                             */
                            function ManagerLinkStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ManagerLinkStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IManagerLinkStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ManagerLinkStatusEnum} ManagerLinkStatusEnum instance
                             */
                            ManagerLinkStatusEnum.create = function create(properties) {
                                return new ManagerLinkStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ManagerLinkStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ManagerLinkStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IManagerLinkStatusEnum} m ManagerLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManagerLinkStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ManagerLinkStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ManagerLinkStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IManagerLinkStatusEnum} message ManagerLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManagerLinkStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ManagerLinkStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ManagerLinkStatusEnum} ManagerLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManagerLinkStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ManagerLinkStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ManagerLinkStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ManagerLinkStatusEnum} ManagerLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManagerLinkStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ManagerLinkStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ManagerLinkStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ManagerLinkStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ManagerLinkStatusEnum} ManagerLinkStatusEnum
                             */
                            ManagerLinkStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ManagerLinkStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ManagerLinkStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a ManagerLinkStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ManagerLinkStatusEnum} m ManagerLinkStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ManagerLinkStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ManagerLinkStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ManagerLinkStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ManagerLinkStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ManagerLinkStatus enum.
                             * @name google.ads.googleads.v11.enums.ManagerLinkStatusEnum.ManagerLinkStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ACTIVE=2 ACTIVE value
                             * @property {number} INACTIVE=3 INACTIVE value
                             * @property {number} PENDING=4 PENDING value
                             * @property {number} REFUSED=5 REFUSED value
                             * @property {number} CANCELED=6 CANCELED value
                             */
                            ManagerLinkStatusEnum.ManagerLinkStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ACTIVE"] = 2;
                                values[valuesById[3] = "INACTIVE"] = 3;
                                values[valuesById[4] = "PENDING"] = 4;
                                values[valuesById[5] = "REFUSED"] = 5;
                                values[valuesById[6] = "CANCELED"] = 6;
                                return values;
                            })();
    
                            return ManagerLinkStatusEnum;
                        })();
    
                        enums.MediaTypeEnum = (function() {
    
                            /**
                             * Properties of a MediaTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMediaTypeEnum
                             */
    
                            /**
                             * Constructs a new MediaTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MediaTypeEnum.
                             * @implements IMediaTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMediaTypeEnum=} [p] Properties to set
                             */
                            function MediaTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MediaTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMediaTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MediaTypeEnum} MediaTypeEnum instance
                             */
                            MediaTypeEnum.create = function create(properties) {
                                return new MediaTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MediaTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MediaTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMediaTypeEnum} m MediaTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MediaTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMediaTypeEnum} message MediaTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MediaTypeEnum} MediaTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MediaTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MediaTypeEnum} MediaTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MediaTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MediaTypeEnum} MediaTypeEnum
                             */
                            MediaTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MediaTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MediaTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a MediaTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MediaTypeEnum} m MediaTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MediaTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MediaTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MediaType enum.
                             * @name google.ads.googleads.v11.enums.MediaTypeEnum.MediaType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} IMAGE=2 IMAGE value
                             * @property {number} ICON=3 ICON value
                             * @property {number} MEDIA_BUNDLE=4 MEDIA_BUNDLE value
                             * @property {number} AUDIO=5 AUDIO value
                             * @property {number} VIDEO=6 VIDEO value
                             * @property {number} DYNAMIC_IMAGE=7 DYNAMIC_IMAGE value
                             */
                            MediaTypeEnum.MediaType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "IMAGE"] = 2;
                                values[valuesById[3] = "ICON"] = 3;
                                values[valuesById[4] = "MEDIA_BUNDLE"] = 4;
                                values[valuesById[5] = "AUDIO"] = 5;
                                values[valuesById[6] = "VIDEO"] = 6;
                                values[valuesById[7] = "DYNAMIC_IMAGE"] = 7;
                                return values;
                            })();
    
                            return MediaTypeEnum;
                        })();
    
                        enums.MerchantCenterLinkStatusEnum = (function() {
    
                            /**
                             * Properties of a MerchantCenterLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMerchantCenterLinkStatusEnum
                             */
    
                            /**
                             * Constructs a new MerchantCenterLinkStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MerchantCenterLinkStatusEnum.
                             * @implements IMerchantCenterLinkStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMerchantCenterLinkStatusEnum=} [p] Properties to set
                             */
                            function MerchantCenterLinkStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MerchantCenterLinkStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMerchantCenterLinkStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum} MerchantCenterLinkStatusEnum instance
                             */
                            MerchantCenterLinkStatusEnum.create = function create(properties) {
                                return new MerchantCenterLinkStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MerchantCenterLinkStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMerchantCenterLinkStatusEnum} m MerchantCenterLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterLinkStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MerchantCenterLinkStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMerchantCenterLinkStatusEnum} message MerchantCenterLinkStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterLinkStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MerchantCenterLinkStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum} MerchantCenterLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterLinkStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MerchantCenterLinkStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum} MerchantCenterLinkStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterLinkStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MerchantCenterLinkStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MerchantCenterLinkStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MerchantCenterLinkStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum} MerchantCenterLinkStatusEnum
                             */
                            MerchantCenterLinkStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a MerchantCenterLinkStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum} m MerchantCenterLinkStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MerchantCenterLinkStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MerchantCenterLinkStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MerchantCenterLinkStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MerchantCenterLinkStatus enum.
                             * @name google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum.MerchantCenterLinkStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} PENDING=3 PENDING value
                             */
                            MerchantCenterLinkStatusEnum.MerchantCenterLinkStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "PENDING"] = 3;
                                return values;
                            })();
    
                            return MerchantCenterLinkStatusEnum;
                        })();
    
                        enums.MessagePlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a MessagePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMessagePlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new MessagePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MessagePlaceholderFieldEnum.
                             * @implements IMessagePlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMessagePlaceholderFieldEnum=} [p] Properties to set
                             */
                            function MessagePlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MessagePlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMessagePlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum} MessagePlaceholderFieldEnum instance
                             */
                            MessagePlaceholderFieldEnum.create = function create(properties) {
                                return new MessagePlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MessagePlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMessagePlaceholderFieldEnum} m MessagePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MessagePlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MessagePlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMessagePlaceholderFieldEnum} message MessagePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MessagePlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MessagePlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum} MessagePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MessagePlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MessagePlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum} MessagePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MessagePlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MessagePlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MessagePlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MessagePlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum} MessagePlaceholderFieldEnum
                             */
                            MessagePlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a MessagePlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum} m MessagePlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MessagePlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MessagePlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MessagePlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MessagePlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum.MessagePlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BUSINESS_NAME=2 BUSINESS_NAME value
                             * @property {number} COUNTRY_CODE=3 COUNTRY_CODE value
                             * @property {number} PHONE_NUMBER=4 PHONE_NUMBER value
                             * @property {number} MESSAGE_EXTENSION_TEXT=5 MESSAGE_EXTENSION_TEXT value
                             * @property {number} MESSAGE_TEXT=6 MESSAGE_TEXT value
                             */
                            MessagePlaceholderFieldEnum.MessagePlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BUSINESS_NAME"] = 2;
                                values[valuesById[3] = "COUNTRY_CODE"] = 3;
                                values[valuesById[4] = "PHONE_NUMBER"] = 4;
                                values[valuesById[5] = "MESSAGE_EXTENSION_TEXT"] = 5;
                                values[valuesById[6] = "MESSAGE_TEXT"] = 6;
                                return values;
                            })();
    
                            return MessagePlaceholderFieldEnum;
                        })();
    
                        enums.MobileDeviceTypeEnum = (function() {
    
                            /**
                             * Properties of a MobileDeviceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IMobileDeviceTypeEnum
                             */
    
                            /**
                             * Constructs a new MobileDeviceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a MobileDeviceTypeEnum.
                             * @implements IMobileDeviceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IMobileDeviceTypeEnum=} [p] Properties to set
                             */
                            function MobileDeviceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MobileDeviceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMobileDeviceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.MobileDeviceTypeEnum} MobileDeviceTypeEnum instance
                             */
                            MobileDeviceTypeEnum.create = function create(properties) {
                                return new MobileDeviceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MobileDeviceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.MobileDeviceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMobileDeviceTypeEnum} m MobileDeviceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileDeviceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileDeviceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.MobileDeviceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IMobileDeviceTypeEnum} message MobileDeviceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileDeviceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileDeviceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.MobileDeviceTypeEnum} MobileDeviceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileDeviceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.MobileDeviceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileDeviceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.MobileDeviceTypeEnum} MobileDeviceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileDeviceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileDeviceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileDeviceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MobileDeviceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.MobileDeviceTypeEnum} MobileDeviceTypeEnum
                             */
                            MobileDeviceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.MobileDeviceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.MobileDeviceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a MobileDeviceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.MobileDeviceTypeEnum} m MobileDeviceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileDeviceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MobileDeviceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.MobileDeviceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileDeviceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MobileDeviceType enum.
                             * @name google.ads.googleads.v11.enums.MobileDeviceTypeEnum.MobileDeviceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MOBILE=2 MOBILE value
                             * @property {number} TABLET=3 TABLET value
                             */
                            MobileDeviceTypeEnum.MobileDeviceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MOBILE"] = 2;
                                values[valuesById[3] = "TABLET"] = 3;
                                return values;
                            })();
    
                            return MobileDeviceTypeEnum;
                        })();
    
                        enums.NegativeGeoTargetTypeEnum = (function() {
    
                            /**
                             * Properties of a NegativeGeoTargetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface INegativeGeoTargetTypeEnum
                             */
    
                            /**
                             * Constructs a new NegativeGeoTargetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a NegativeGeoTargetTypeEnum.
                             * @implements INegativeGeoTargetTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.INegativeGeoTargetTypeEnum=} [p] Properties to set
                             */
                            function NegativeGeoTargetTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new NegativeGeoTargetTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.INegativeGeoTargetTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum} NegativeGeoTargetTypeEnum instance
                             */
                            NegativeGeoTargetTypeEnum.create = function create(properties) {
                                return new NegativeGeoTargetTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified NegativeGeoTargetTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.INegativeGeoTargetTypeEnum} m NegativeGeoTargetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NegativeGeoTargetTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified NegativeGeoTargetTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.INegativeGeoTargetTypeEnum} message NegativeGeoTargetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NegativeGeoTargetTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a NegativeGeoTargetTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum} NegativeGeoTargetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NegativeGeoTargetTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a NegativeGeoTargetTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum} NegativeGeoTargetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NegativeGeoTargetTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a NegativeGeoTargetTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            NegativeGeoTargetTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a NegativeGeoTargetTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum} NegativeGeoTargetTypeEnum
                             */
                            NegativeGeoTargetTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a NegativeGeoTargetTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum} m NegativeGeoTargetTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            NegativeGeoTargetTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this NegativeGeoTargetTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            NegativeGeoTargetTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * NegativeGeoTargetType enum.
                             * @name google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum.NegativeGeoTargetType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PRESENCE_OR_INTEREST=4 PRESENCE_OR_INTEREST value
                             * @property {number} PRESENCE=5 PRESENCE value
                             */
                            NegativeGeoTargetTypeEnum.NegativeGeoTargetType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[4] = "PRESENCE_OR_INTEREST"] = 4;
                                values[valuesById[5] = "PRESENCE"] = 5;
                                return values;
                            })();
    
                            return NegativeGeoTargetTypeEnum;
                        })();
    
                        enums.OfflineUserDataJobFailureReasonEnum = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJobFailureReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IOfflineUserDataJobFailureReasonEnum
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJobFailureReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an OfflineUserDataJobFailureReasonEnum.
                             * @implements IOfflineUserDataJobFailureReasonEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobFailureReasonEnum=} [p] Properties to set
                             */
                            function OfflineUserDataJobFailureReasonEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OfflineUserDataJobFailureReasonEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobFailureReasonEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum} OfflineUserDataJobFailureReasonEnum instance
                             */
                            OfflineUserDataJobFailureReasonEnum.create = function create(properties) {
                                return new OfflineUserDataJobFailureReasonEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobFailureReasonEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobFailureReasonEnum} m OfflineUserDataJobFailureReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobFailureReasonEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobFailureReasonEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobFailureReasonEnum} message OfflineUserDataJobFailureReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobFailureReasonEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobFailureReasonEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum} OfflineUserDataJobFailureReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobFailureReasonEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobFailureReasonEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum} OfflineUserDataJobFailureReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobFailureReasonEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJobFailureReasonEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJobFailureReasonEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJobFailureReasonEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum} OfflineUserDataJobFailureReasonEnum
                             */
                            OfflineUserDataJobFailureReasonEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum();
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJobFailureReasonEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum} m OfflineUserDataJobFailureReasonEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJobFailureReasonEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OfflineUserDataJobFailureReasonEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJobFailureReasonEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OfflineUserDataJobFailureReason enum.
                             * @name google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INSUFFICIENT_MATCHED_TRANSACTIONS=2 INSUFFICIENT_MATCHED_TRANSACTIONS value
                             * @property {number} INSUFFICIENT_TRANSACTIONS=3 INSUFFICIENT_TRANSACTIONS value
                             */
                            OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INSUFFICIENT_MATCHED_TRANSACTIONS"] = 2;
                                values[valuesById[3] = "INSUFFICIENT_TRANSACTIONS"] = 3;
                                return values;
                            })();
    
                            return OfflineUserDataJobFailureReasonEnum;
                        })();
    
                        enums.OfflineUserDataJobMatchRateRangeEnum = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJobMatchRateRangeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IOfflineUserDataJobMatchRateRangeEnum
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJobMatchRateRangeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an OfflineUserDataJobMatchRateRangeEnum.
                             * @implements IOfflineUserDataJobMatchRateRangeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobMatchRateRangeEnum=} [p] Properties to set
                             */
                            function OfflineUserDataJobMatchRateRangeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OfflineUserDataJobMatchRateRangeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobMatchRateRangeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum} OfflineUserDataJobMatchRateRangeEnum instance
                             */
                            OfflineUserDataJobMatchRateRangeEnum.create = function create(properties) {
                                return new OfflineUserDataJobMatchRateRangeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobMatchRateRangeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobMatchRateRangeEnum} m OfflineUserDataJobMatchRateRangeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobMatchRateRangeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobMatchRateRangeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobMatchRateRangeEnum} message OfflineUserDataJobMatchRateRangeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobMatchRateRangeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobMatchRateRangeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum} OfflineUserDataJobMatchRateRangeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobMatchRateRangeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobMatchRateRangeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum} OfflineUserDataJobMatchRateRangeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobMatchRateRangeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJobMatchRateRangeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJobMatchRateRangeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJobMatchRateRangeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum} OfflineUserDataJobMatchRateRangeEnum
                             */
                            OfflineUserDataJobMatchRateRangeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum();
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJobMatchRateRangeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum} m OfflineUserDataJobMatchRateRangeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJobMatchRateRangeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OfflineUserDataJobMatchRateRangeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJobMatchRateRangeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OfflineUserDataJobMatchRateRange enum.
                             * @name google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MATCH_RANGE_LESS_THAN_20=2 MATCH_RANGE_LESS_THAN_20 value
                             * @property {number} MATCH_RANGE_20_TO_30=3 MATCH_RANGE_20_TO_30 value
                             * @property {number} MATCH_RANGE_31_TO_40=4 MATCH_RANGE_31_TO_40 value
                             * @property {number} MATCH_RANGE_41_TO_50=5 MATCH_RANGE_41_TO_50 value
                             * @property {number} MATCH_RANGE_51_TO_60=6 MATCH_RANGE_51_TO_60 value
                             * @property {number} MATCH_RANGE_61_TO_70=7 MATCH_RANGE_61_TO_70 value
                             * @property {number} MATCH_RANGE_71_TO_80=8 MATCH_RANGE_71_TO_80 value
                             * @property {number} MATCH_RANGE_81_TO_90=9 MATCH_RANGE_81_TO_90 value
                             * @property {number} MATCH_RANGE_91_TO_100=10 MATCH_RANGE_91_TO_100 value
                             */
                            OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MATCH_RANGE_LESS_THAN_20"] = 2;
                                values[valuesById[3] = "MATCH_RANGE_20_TO_30"] = 3;
                                values[valuesById[4] = "MATCH_RANGE_31_TO_40"] = 4;
                                values[valuesById[5] = "MATCH_RANGE_41_TO_50"] = 5;
                                values[valuesById[6] = "MATCH_RANGE_51_TO_60"] = 6;
                                values[valuesById[7] = "MATCH_RANGE_61_TO_70"] = 7;
                                values[valuesById[8] = "MATCH_RANGE_71_TO_80"] = 8;
                                values[valuesById[9] = "MATCH_RANGE_81_TO_90"] = 9;
                                values[valuesById[10] = "MATCH_RANGE_91_TO_100"] = 10;
                                return values;
                            })();
    
                            return OfflineUserDataJobMatchRateRangeEnum;
                        })();
    
                        enums.OfflineUserDataJobStatusEnum = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJobStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IOfflineUserDataJobStatusEnum
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJobStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an OfflineUserDataJobStatusEnum.
                             * @implements IOfflineUserDataJobStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobStatusEnum=} [p] Properties to set
                             */
                            function OfflineUserDataJobStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OfflineUserDataJobStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum} OfflineUserDataJobStatusEnum instance
                             */
                            OfflineUserDataJobStatusEnum.create = function create(properties) {
                                return new OfflineUserDataJobStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobStatusEnum} m OfflineUserDataJobStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobStatusEnum} message OfflineUserDataJobStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum} OfflineUserDataJobStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum} OfflineUserDataJobStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJobStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJobStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJobStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum} OfflineUserDataJobStatusEnum
                             */
                            OfflineUserDataJobStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJobStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum} m OfflineUserDataJobStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJobStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OfflineUserDataJobStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJobStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OfflineUserDataJobStatus enum.
                             * @name google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PENDING=2 PENDING value
                             * @property {number} RUNNING=3 RUNNING value
                             * @property {number} SUCCESS=4 SUCCESS value
                             * @property {number} FAILED=5 FAILED value
                             */
                            OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PENDING"] = 2;
                                values[valuesById[3] = "RUNNING"] = 3;
                                values[valuesById[4] = "SUCCESS"] = 4;
                                values[valuesById[5] = "FAILED"] = 5;
                                return values;
                            })();
    
                            return OfflineUserDataJobStatusEnum;
                        })();
    
                        enums.OfflineUserDataJobTypeEnum = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJobTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IOfflineUserDataJobTypeEnum
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJobTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an OfflineUserDataJobTypeEnum.
                             * @implements IOfflineUserDataJobTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobTypeEnum=} [p] Properties to set
                             */
                            function OfflineUserDataJobTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OfflineUserDataJobTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum} OfflineUserDataJobTypeEnum instance
                             */
                            OfflineUserDataJobTypeEnum.create = function create(properties) {
                                return new OfflineUserDataJobTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobTypeEnum} m OfflineUserDataJobTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOfflineUserDataJobTypeEnum} message OfflineUserDataJobTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum} OfflineUserDataJobTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum} OfflineUserDataJobTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJobTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJobTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJobTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum} OfflineUserDataJobTypeEnum
                             */
                            OfflineUserDataJobTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJobTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum} m OfflineUserDataJobTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJobTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OfflineUserDataJobTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJobTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OfflineUserDataJobType enum.
                             * @name google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum.OfflineUserDataJobType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} STORE_SALES_UPLOAD_FIRST_PARTY=2 STORE_SALES_UPLOAD_FIRST_PARTY value
                             * @property {number} STORE_SALES_UPLOAD_THIRD_PARTY=3 STORE_SALES_UPLOAD_THIRD_PARTY value
                             * @property {number} CUSTOMER_MATCH_USER_LIST=4 CUSTOMER_MATCH_USER_LIST value
                             * @property {number} CUSTOMER_MATCH_WITH_ATTRIBUTES=5 CUSTOMER_MATCH_WITH_ATTRIBUTES value
                             */
                            OfflineUserDataJobTypeEnum.OfflineUserDataJobType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "STORE_SALES_UPLOAD_FIRST_PARTY"] = 2;
                                values[valuesById[3] = "STORE_SALES_UPLOAD_THIRD_PARTY"] = 3;
                                values[valuesById[4] = "CUSTOMER_MATCH_USER_LIST"] = 4;
                                values[valuesById[5] = "CUSTOMER_MATCH_WITH_ATTRIBUTES"] = 5;
                                return values;
                            })();
    
                            return OfflineUserDataJobTypeEnum;
                        })();
    
                        enums.OperatingSystemVersionOperatorTypeEnum = (function() {
    
                            /**
                             * Properties of an OperatingSystemVersionOperatorTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IOperatingSystemVersionOperatorTypeEnum
                             */
    
                            /**
                             * Constructs a new OperatingSystemVersionOperatorTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an OperatingSystemVersionOperatorTypeEnum.
                             * @implements IOperatingSystemVersionOperatorTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IOperatingSystemVersionOperatorTypeEnum=} [p] Properties to set
                             */
                            function OperatingSystemVersionOperatorTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OperatingSystemVersionOperatorTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOperatingSystemVersionOperatorTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum} OperatingSystemVersionOperatorTypeEnum instance
                             */
                            OperatingSystemVersionOperatorTypeEnum.create = function create(properties) {
                                return new OperatingSystemVersionOperatorTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OperatingSystemVersionOperatorTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOperatingSystemVersionOperatorTypeEnum} m OperatingSystemVersionOperatorTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatingSystemVersionOperatorTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OperatingSystemVersionOperatorTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOperatingSystemVersionOperatorTypeEnum} message OperatingSystemVersionOperatorTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatingSystemVersionOperatorTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OperatingSystemVersionOperatorTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum} OperatingSystemVersionOperatorTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatingSystemVersionOperatorTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OperatingSystemVersionOperatorTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum} OperatingSystemVersionOperatorTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatingSystemVersionOperatorTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OperatingSystemVersionOperatorTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OperatingSystemVersionOperatorTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OperatingSystemVersionOperatorTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum} OperatingSystemVersionOperatorTypeEnum
                             */
                            OperatingSystemVersionOperatorTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an OperatingSystemVersionOperatorTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum} m OperatingSystemVersionOperatorTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OperatingSystemVersionOperatorTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OperatingSystemVersionOperatorTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OperatingSystemVersionOperatorTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OperatingSystemVersionOperatorType enum.
                             * @name google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EQUALS_TO=2 EQUALS_TO value
                             * @property {number} GREATER_THAN_EQUALS_TO=4 GREATER_THAN_EQUALS_TO value
                             */
                            OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EQUALS_TO"] = 2;
                                values[valuesById[4] = "GREATER_THAN_EQUALS_TO"] = 4;
                                return values;
                            })();
    
                            return OperatingSystemVersionOperatorTypeEnum;
                        })();
    
                        enums.OptimizationGoalTypeEnum = (function() {
    
                            /**
                             * Properties of an OptimizationGoalTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IOptimizationGoalTypeEnum
                             */
    
                            /**
                             * Constructs a new OptimizationGoalTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents an OptimizationGoalTypeEnum.
                             * @implements IOptimizationGoalTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IOptimizationGoalTypeEnum=} [p] Properties to set
                             */
                            function OptimizationGoalTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OptimizationGoalTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOptimizationGoalTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.OptimizationGoalTypeEnum} OptimizationGoalTypeEnum instance
                             */
                            OptimizationGoalTypeEnum.create = function create(properties) {
                                return new OptimizationGoalTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OptimizationGoalTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.OptimizationGoalTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOptimizationGoalTypeEnum} m OptimizationGoalTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OptimizationGoalTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OptimizationGoalTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.OptimizationGoalTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IOptimizationGoalTypeEnum} message OptimizationGoalTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OptimizationGoalTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OptimizationGoalTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.OptimizationGoalTypeEnum} OptimizationGoalTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OptimizationGoalTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.OptimizationGoalTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OptimizationGoalTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.OptimizationGoalTypeEnum} OptimizationGoalTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OptimizationGoalTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OptimizationGoalTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OptimizationGoalTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OptimizationGoalTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.OptimizationGoalTypeEnum} OptimizationGoalTypeEnum
                             */
                            OptimizationGoalTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.OptimizationGoalTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.OptimizationGoalTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from an OptimizationGoalTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.OptimizationGoalTypeEnum} m OptimizationGoalTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OptimizationGoalTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OptimizationGoalTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.OptimizationGoalTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OptimizationGoalTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OptimizationGoalType enum.
                             * @name google.ads.googleads.v11.enums.OptimizationGoalTypeEnum.OptimizationGoalType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CALL_CLICKS=2 CALL_CLICKS value
                             * @property {number} DRIVING_DIRECTIONS=3 DRIVING_DIRECTIONS value
                             * @property {number} APP_PRE_REGISTRATION=4 APP_PRE_REGISTRATION value
                             */
                            OptimizationGoalTypeEnum.OptimizationGoalType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CALL_CLICKS"] = 2;
                                values[valuesById[3] = "DRIVING_DIRECTIONS"] = 3;
                                values[valuesById[4] = "APP_PRE_REGISTRATION"] = 4;
                                return values;
                            })();
    
                            return OptimizationGoalTypeEnum;
                        })();
    
                        enums.PaymentModeEnum = (function() {
    
                            /**
                             * Properties of a PaymentModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPaymentModeEnum
                             */
    
                            /**
                             * Constructs a new PaymentModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PaymentModeEnum.
                             * @implements IPaymentModeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPaymentModeEnum=} [p] Properties to set
                             */
                            function PaymentModeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PaymentModeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPaymentModeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PaymentModeEnum} PaymentModeEnum instance
                             */
                            PaymentModeEnum.create = function create(properties) {
                                return new PaymentModeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PaymentModeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PaymentModeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPaymentModeEnum} m PaymentModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaymentModeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PaymentModeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PaymentModeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPaymentModeEnum} message PaymentModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaymentModeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PaymentModeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PaymentModeEnum} PaymentModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaymentModeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PaymentModeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PaymentModeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PaymentModeEnum} PaymentModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaymentModeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PaymentModeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PaymentModeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PaymentModeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PaymentModeEnum} PaymentModeEnum
                             */
                            PaymentModeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PaymentModeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PaymentModeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PaymentModeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PaymentModeEnum} m PaymentModeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PaymentModeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PaymentModeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PaymentModeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PaymentModeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PaymentMode enum.
                             * @name google.ads.googleads.v11.enums.PaymentModeEnum.PaymentMode
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CLICKS=4 CLICKS value
                             * @property {number} CONVERSION_VALUE=5 CONVERSION_VALUE value
                             * @property {number} CONVERSIONS=6 CONVERSIONS value
                             * @property {number} GUEST_STAY=7 GUEST_STAY value
                             */
                            PaymentModeEnum.PaymentMode = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[4] = "CLICKS"] = 4;
                                values[valuesById[5] = "CONVERSION_VALUE"] = 5;
                                values[valuesById[6] = "CONVERSIONS"] = 6;
                                values[valuesById[7] = "GUEST_STAY"] = 7;
                                return values;
                            })();
    
                            return PaymentModeEnum;
                        })();
    
                        enums.PerformanceMaxUpgradeStatusEnum = (function() {
    
                            /**
                             * Properties of a PerformanceMaxUpgradeStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPerformanceMaxUpgradeStatusEnum
                             */
    
                            /**
                             * Constructs a new PerformanceMaxUpgradeStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PerformanceMaxUpgradeStatusEnum.
                             * @implements IPerformanceMaxUpgradeStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPerformanceMaxUpgradeStatusEnum=} [p] Properties to set
                             */
                            function PerformanceMaxUpgradeStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PerformanceMaxUpgradeStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPerformanceMaxUpgradeStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum} PerformanceMaxUpgradeStatusEnum instance
                             */
                            PerformanceMaxUpgradeStatusEnum.create = function create(properties) {
                                return new PerformanceMaxUpgradeStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PerformanceMaxUpgradeStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPerformanceMaxUpgradeStatusEnum} m PerformanceMaxUpgradeStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PerformanceMaxUpgradeStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PerformanceMaxUpgradeStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPerformanceMaxUpgradeStatusEnum} message PerformanceMaxUpgradeStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PerformanceMaxUpgradeStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PerformanceMaxUpgradeStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum} PerformanceMaxUpgradeStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PerformanceMaxUpgradeStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PerformanceMaxUpgradeStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum} PerformanceMaxUpgradeStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PerformanceMaxUpgradeStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PerformanceMaxUpgradeStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PerformanceMaxUpgradeStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PerformanceMaxUpgradeStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum} PerformanceMaxUpgradeStatusEnum
                             */
                            PerformanceMaxUpgradeStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a PerformanceMaxUpgradeStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum} m PerformanceMaxUpgradeStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PerformanceMaxUpgradeStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PerformanceMaxUpgradeStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PerformanceMaxUpgradeStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PerformanceMaxUpgradeStatus enum.
                             * @name google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UPGRADE_ELIBIGLE=2 UPGRADE_ELIBIGLE value
                             * @property {number} UPGRADE_IN_PROGRESS=3 UPGRADE_IN_PROGRESS value
                             * @property {number} UPGRADE_COMPLETE=4 UPGRADE_COMPLETE value
                             * @property {number} UPGRADE_FAILED=5 UPGRADE_FAILED value
                             */
                            PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UPGRADE_ELIBIGLE"] = 2;
                                values[valuesById[3] = "UPGRADE_IN_PROGRESS"] = 3;
                                values[valuesById[4] = "UPGRADE_COMPLETE"] = 4;
                                values[valuesById[5] = "UPGRADE_FAILED"] = 5;
                                return values;
                            })();
    
                            return PerformanceMaxUpgradeStatusEnum;
                        })();
    
                        enums.PlacementTypeEnum = (function() {
    
                            /**
                             * Properties of a PlacementTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPlacementTypeEnum
                             */
    
                            /**
                             * Constructs a new PlacementTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PlacementTypeEnum.
                             * @implements IPlacementTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPlacementTypeEnum=} [p] Properties to set
                             */
                            function PlacementTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PlacementTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPlacementTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PlacementTypeEnum} PlacementTypeEnum instance
                             */
                            PlacementTypeEnum.create = function create(properties) {
                                return new PlacementTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PlacementTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PlacementTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPlacementTypeEnum} m PlacementTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlacementTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlacementTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PlacementTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPlacementTypeEnum} message PlacementTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlacementTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlacementTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PlacementTypeEnum} PlacementTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlacementTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PlacementTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlacementTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PlacementTypeEnum} PlacementTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlacementTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlacementTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlacementTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PlacementTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PlacementTypeEnum} PlacementTypeEnum
                             */
                            PlacementTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PlacementTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PlacementTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PlacementTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PlacementTypeEnum} m PlacementTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlacementTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PlacementTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PlacementTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlacementTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PlacementType enum.
                             * @name google.ads.googleads.v11.enums.PlacementTypeEnum.PlacementType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} WEBSITE=2 WEBSITE value
                             * @property {number} MOBILE_APP_CATEGORY=3 MOBILE_APP_CATEGORY value
                             * @property {number} MOBILE_APPLICATION=4 MOBILE_APPLICATION value
                             * @property {number} YOUTUBE_VIDEO=5 YOUTUBE_VIDEO value
                             * @property {number} YOUTUBE_CHANNEL=6 YOUTUBE_CHANNEL value
                             */
                            PlacementTypeEnum.PlacementType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "WEBSITE"] = 2;
                                values[valuesById[3] = "MOBILE_APP_CATEGORY"] = 3;
                                values[valuesById[4] = "MOBILE_APPLICATION"] = 4;
                                values[valuesById[5] = "YOUTUBE_VIDEO"] = 5;
                                values[valuesById[6] = "YOUTUBE_CHANNEL"] = 6;
                                return values;
                            })();
    
                            return PlacementTypeEnum;
                        })();
    
                        enums.PositiveGeoTargetTypeEnum = (function() {
    
                            /**
                             * Properties of a PositiveGeoTargetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPositiveGeoTargetTypeEnum
                             */
    
                            /**
                             * Constructs a new PositiveGeoTargetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PositiveGeoTargetTypeEnum.
                             * @implements IPositiveGeoTargetTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPositiveGeoTargetTypeEnum=} [p] Properties to set
                             */
                            function PositiveGeoTargetTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PositiveGeoTargetTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPositiveGeoTargetTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum} PositiveGeoTargetTypeEnum instance
                             */
                            PositiveGeoTargetTypeEnum.create = function create(properties) {
                                return new PositiveGeoTargetTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PositiveGeoTargetTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPositiveGeoTargetTypeEnum} m PositiveGeoTargetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PositiveGeoTargetTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PositiveGeoTargetTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPositiveGeoTargetTypeEnum} message PositiveGeoTargetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PositiveGeoTargetTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PositiveGeoTargetTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum} PositiveGeoTargetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PositiveGeoTargetTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PositiveGeoTargetTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum} PositiveGeoTargetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PositiveGeoTargetTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PositiveGeoTargetTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PositiveGeoTargetTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PositiveGeoTargetTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum} PositiveGeoTargetTypeEnum
                             */
                            PositiveGeoTargetTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a PositiveGeoTargetTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum} m PositiveGeoTargetTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PositiveGeoTargetTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PositiveGeoTargetTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PositiveGeoTargetTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PositiveGeoTargetType enum.
                             * @name google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum.PositiveGeoTargetType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PRESENCE_OR_INTEREST=5 PRESENCE_OR_INTEREST value
                             * @property {number} SEARCH_INTEREST=6 SEARCH_INTEREST value
                             * @property {number} PRESENCE=7 PRESENCE value
                             */
                            PositiveGeoTargetTypeEnum.PositiveGeoTargetType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[5] = "PRESENCE_OR_INTEREST"] = 5;
                                values[valuesById[6] = "SEARCH_INTEREST"] = 6;
                                values[valuesById[7] = "PRESENCE"] = 7;
                                return values;
                            })();
    
                            return PositiveGeoTargetTypeEnum;
                        })();
    
                        enums.PricePlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a PricePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPricePlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new PricePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PricePlaceholderFieldEnum.
                             * @implements IPricePlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPricePlaceholderFieldEnum=} [p] Properties to set
                             */
                            function PricePlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PricePlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPricePlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PricePlaceholderFieldEnum} PricePlaceholderFieldEnum instance
                             */
                            PricePlaceholderFieldEnum.create = function create(properties) {
                                return new PricePlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PricePlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PricePlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPricePlaceholderFieldEnum} m PricePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PricePlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PricePlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PricePlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPricePlaceholderFieldEnum} message PricePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PricePlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PricePlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PricePlaceholderFieldEnum} PricePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PricePlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PricePlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PricePlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PricePlaceholderFieldEnum} PricePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PricePlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PricePlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PricePlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PricePlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PricePlaceholderFieldEnum} PricePlaceholderFieldEnum
                             */
                            PricePlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PricePlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PricePlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a PricePlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PricePlaceholderFieldEnum} m PricePlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PricePlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PricePlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PricePlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PricePlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PricePlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.PricePlaceholderFieldEnum.PricePlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TYPE=2 TYPE value
                             * @property {number} PRICE_QUALIFIER=3 PRICE_QUALIFIER value
                             * @property {number} TRACKING_TEMPLATE=4 TRACKING_TEMPLATE value
                             * @property {number} LANGUAGE=5 LANGUAGE value
                             * @property {number} FINAL_URL_SUFFIX=6 FINAL_URL_SUFFIX value
                             * @property {number} ITEM_1_HEADER=100 ITEM_1_HEADER value
                             * @property {number} ITEM_1_DESCRIPTION=101 ITEM_1_DESCRIPTION value
                             * @property {number} ITEM_1_PRICE=102 ITEM_1_PRICE value
                             * @property {number} ITEM_1_UNIT=103 ITEM_1_UNIT value
                             * @property {number} ITEM_1_FINAL_URLS=104 ITEM_1_FINAL_URLS value
                             * @property {number} ITEM_1_FINAL_MOBILE_URLS=105 ITEM_1_FINAL_MOBILE_URLS value
                             * @property {number} ITEM_2_HEADER=200 ITEM_2_HEADER value
                             * @property {number} ITEM_2_DESCRIPTION=201 ITEM_2_DESCRIPTION value
                             * @property {number} ITEM_2_PRICE=202 ITEM_2_PRICE value
                             * @property {number} ITEM_2_UNIT=203 ITEM_2_UNIT value
                             * @property {number} ITEM_2_FINAL_URLS=204 ITEM_2_FINAL_URLS value
                             * @property {number} ITEM_2_FINAL_MOBILE_URLS=205 ITEM_2_FINAL_MOBILE_URLS value
                             * @property {number} ITEM_3_HEADER=300 ITEM_3_HEADER value
                             * @property {number} ITEM_3_DESCRIPTION=301 ITEM_3_DESCRIPTION value
                             * @property {number} ITEM_3_PRICE=302 ITEM_3_PRICE value
                             * @property {number} ITEM_3_UNIT=303 ITEM_3_UNIT value
                             * @property {number} ITEM_3_FINAL_URLS=304 ITEM_3_FINAL_URLS value
                             * @property {number} ITEM_3_FINAL_MOBILE_URLS=305 ITEM_3_FINAL_MOBILE_URLS value
                             * @property {number} ITEM_4_HEADER=400 ITEM_4_HEADER value
                             * @property {number} ITEM_4_DESCRIPTION=401 ITEM_4_DESCRIPTION value
                             * @property {number} ITEM_4_PRICE=402 ITEM_4_PRICE value
                             * @property {number} ITEM_4_UNIT=403 ITEM_4_UNIT value
                             * @property {number} ITEM_4_FINAL_URLS=404 ITEM_4_FINAL_URLS value
                             * @property {number} ITEM_4_FINAL_MOBILE_URLS=405 ITEM_4_FINAL_MOBILE_URLS value
                             * @property {number} ITEM_5_HEADER=500 ITEM_5_HEADER value
                             * @property {number} ITEM_5_DESCRIPTION=501 ITEM_5_DESCRIPTION value
                             * @property {number} ITEM_5_PRICE=502 ITEM_5_PRICE value
                             * @property {number} ITEM_5_UNIT=503 ITEM_5_UNIT value
                             * @property {number} ITEM_5_FINAL_URLS=504 ITEM_5_FINAL_URLS value
                             * @property {number} ITEM_5_FINAL_MOBILE_URLS=505 ITEM_5_FINAL_MOBILE_URLS value
                             * @property {number} ITEM_6_HEADER=600 ITEM_6_HEADER value
                             * @property {number} ITEM_6_DESCRIPTION=601 ITEM_6_DESCRIPTION value
                             * @property {number} ITEM_6_PRICE=602 ITEM_6_PRICE value
                             * @property {number} ITEM_6_UNIT=603 ITEM_6_UNIT value
                             * @property {number} ITEM_6_FINAL_URLS=604 ITEM_6_FINAL_URLS value
                             * @property {number} ITEM_6_FINAL_MOBILE_URLS=605 ITEM_6_FINAL_MOBILE_URLS value
                             * @property {number} ITEM_7_HEADER=700 ITEM_7_HEADER value
                             * @property {number} ITEM_7_DESCRIPTION=701 ITEM_7_DESCRIPTION value
                             * @property {number} ITEM_7_PRICE=702 ITEM_7_PRICE value
                             * @property {number} ITEM_7_UNIT=703 ITEM_7_UNIT value
                             * @property {number} ITEM_7_FINAL_URLS=704 ITEM_7_FINAL_URLS value
                             * @property {number} ITEM_7_FINAL_MOBILE_URLS=705 ITEM_7_FINAL_MOBILE_URLS value
                             * @property {number} ITEM_8_HEADER=800 ITEM_8_HEADER value
                             * @property {number} ITEM_8_DESCRIPTION=801 ITEM_8_DESCRIPTION value
                             * @property {number} ITEM_8_PRICE=802 ITEM_8_PRICE value
                             * @property {number} ITEM_8_UNIT=803 ITEM_8_UNIT value
                             * @property {number} ITEM_8_FINAL_URLS=804 ITEM_8_FINAL_URLS value
                             * @property {number} ITEM_8_FINAL_MOBILE_URLS=805 ITEM_8_FINAL_MOBILE_URLS value
                             */
                            PricePlaceholderFieldEnum.PricePlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TYPE"] = 2;
                                values[valuesById[3] = "PRICE_QUALIFIER"] = 3;
                                values[valuesById[4] = "TRACKING_TEMPLATE"] = 4;
                                values[valuesById[5] = "LANGUAGE"] = 5;
                                values[valuesById[6] = "FINAL_URL_SUFFIX"] = 6;
                                values[valuesById[100] = "ITEM_1_HEADER"] = 100;
                                values[valuesById[101] = "ITEM_1_DESCRIPTION"] = 101;
                                values[valuesById[102] = "ITEM_1_PRICE"] = 102;
                                values[valuesById[103] = "ITEM_1_UNIT"] = 103;
                                values[valuesById[104] = "ITEM_1_FINAL_URLS"] = 104;
                                values[valuesById[105] = "ITEM_1_FINAL_MOBILE_URLS"] = 105;
                                values[valuesById[200] = "ITEM_2_HEADER"] = 200;
                                values[valuesById[201] = "ITEM_2_DESCRIPTION"] = 201;
                                values[valuesById[202] = "ITEM_2_PRICE"] = 202;
                                values[valuesById[203] = "ITEM_2_UNIT"] = 203;
                                values[valuesById[204] = "ITEM_2_FINAL_URLS"] = 204;
                                values[valuesById[205] = "ITEM_2_FINAL_MOBILE_URLS"] = 205;
                                values[valuesById[300] = "ITEM_3_HEADER"] = 300;
                                values[valuesById[301] = "ITEM_3_DESCRIPTION"] = 301;
                                values[valuesById[302] = "ITEM_3_PRICE"] = 302;
                                values[valuesById[303] = "ITEM_3_UNIT"] = 303;
                                values[valuesById[304] = "ITEM_3_FINAL_URLS"] = 304;
                                values[valuesById[305] = "ITEM_3_FINAL_MOBILE_URLS"] = 305;
                                values[valuesById[400] = "ITEM_4_HEADER"] = 400;
                                values[valuesById[401] = "ITEM_4_DESCRIPTION"] = 401;
                                values[valuesById[402] = "ITEM_4_PRICE"] = 402;
                                values[valuesById[403] = "ITEM_4_UNIT"] = 403;
                                values[valuesById[404] = "ITEM_4_FINAL_URLS"] = 404;
                                values[valuesById[405] = "ITEM_4_FINAL_MOBILE_URLS"] = 405;
                                values[valuesById[500] = "ITEM_5_HEADER"] = 500;
                                values[valuesById[501] = "ITEM_5_DESCRIPTION"] = 501;
                                values[valuesById[502] = "ITEM_5_PRICE"] = 502;
                                values[valuesById[503] = "ITEM_5_UNIT"] = 503;
                                values[valuesById[504] = "ITEM_5_FINAL_URLS"] = 504;
                                values[valuesById[505] = "ITEM_5_FINAL_MOBILE_URLS"] = 505;
                                values[valuesById[600] = "ITEM_6_HEADER"] = 600;
                                values[valuesById[601] = "ITEM_6_DESCRIPTION"] = 601;
                                values[valuesById[602] = "ITEM_6_PRICE"] = 602;
                                values[valuesById[603] = "ITEM_6_UNIT"] = 603;
                                values[valuesById[604] = "ITEM_6_FINAL_URLS"] = 604;
                                values[valuesById[605] = "ITEM_6_FINAL_MOBILE_URLS"] = 605;
                                values[valuesById[700] = "ITEM_7_HEADER"] = 700;
                                values[valuesById[701] = "ITEM_7_DESCRIPTION"] = 701;
                                values[valuesById[702] = "ITEM_7_PRICE"] = 702;
                                values[valuesById[703] = "ITEM_7_UNIT"] = 703;
                                values[valuesById[704] = "ITEM_7_FINAL_URLS"] = 704;
                                values[valuesById[705] = "ITEM_7_FINAL_MOBILE_URLS"] = 705;
                                values[valuesById[800] = "ITEM_8_HEADER"] = 800;
                                values[valuesById[801] = "ITEM_8_DESCRIPTION"] = 801;
                                values[valuesById[802] = "ITEM_8_PRICE"] = 802;
                                values[valuesById[803] = "ITEM_8_UNIT"] = 803;
                                values[valuesById[804] = "ITEM_8_FINAL_URLS"] = 804;
                                values[valuesById[805] = "ITEM_8_FINAL_MOBILE_URLS"] = 805;
                                return values;
                            })();
    
                            return PricePlaceholderFieldEnum;
                        })();
    
                        enums.ProductBiddingCategoryStatusEnum = (function() {
    
                            /**
                             * Properties of a ProductBiddingCategoryStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IProductBiddingCategoryStatusEnum
                             */
    
                            /**
                             * Constructs a new ProductBiddingCategoryStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ProductBiddingCategoryStatusEnum.
                             * @implements IProductBiddingCategoryStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryStatusEnum=} [p] Properties to set
                             */
                            function ProductBiddingCategoryStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ProductBiddingCategoryStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum} ProductBiddingCategoryStatusEnum instance
                             */
                            ProductBiddingCategoryStatusEnum.create = function create(properties) {
                                return new ProductBiddingCategoryStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryStatusEnum} m ProductBiddingCategoryStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IProductBiddingCategoryStatusEnum} message ProductBiddingCategoryStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum} ProductBiddingCategoryStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum} ProductBiddingCategoryStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductBiddingCategoryStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductBiddingCategoryStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ProductBiddingCategoryStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum} ProductBiddingCategoryStatusEnum
                             */
                            ProductBiddingCategoryStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a ProductBiddingCategoryStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum} m ProductBiddingCategoryStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductBiddingCategoryStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ProductBiddingCategoryStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductBiddingCategoryStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ProductBiddingCategoryStatus enum.
                             * @name google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum.ProductBiddingCategoryStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ACTIVE=2 ACTIVE value
                             * @property {number} OBSOLETE=3 OBSOLETE value
                             */
                            ProductBiddingCategoryStatusEnum.ProductBiddingCategoryStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ACTIVE"] = 2;
                                values[valuesById[3] = "OBSOLETE"] = 3;
                                return values;
                            })();
    
                            return ProductBiddingCategoryStatusEnum;
                        })();
    
                        enums.PromotionPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a PromotionPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IPromotionPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new PromotionPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a PromotionPlaceholderFieldEnum.
                             * @implements IPromotionPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IPromotionPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function PromotionPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PromotionPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum} PromotionPlaceholderFieldEnum instance
                             */
                            PromotionPlaceholderFieldEnum.create = function create(properties) {
                                return new PromotionPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PromotionPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionPlaceholderFieldEnum} m PromotionPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromotionPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IPromotionPlaceholderFieldEnum} message PromotionPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromotionPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromotionPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum} PromotionPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromotionPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum} PromotionPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromotionPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromotionPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromotionPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PromotionPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum} PromotionPlaceholderFieldEnum
                             */
                            PromotionPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a PromotionPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum} m PromotionPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromotionPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PromotionPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromotionPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PromotionPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum.PromotionPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PROMOTION_TARGET=2 PROMOTION_TARGET value
                             * @property {number} DISCOUNT_MODIFIER=3 DISCOUNT_MODIFIER value
                             * @property {number} PERCENT_OFF=4 PERCENT_OFF value
                             * @property {number} MONEY_AMOUNT_OFF=5 MONEY_AMOUNT_OFF value
                             * @property {number} PROMOTION_CODE=6 PROMOTION_CODE value
                             * @property {number} ORDERS_OVER_AMOUNT=7 ORDERS_OVER_AMOUNT value
                             * @property {number} PROMOTION_START=8 PROMOTION_START value
                             * @property {number} PROMOTION_END=9 PROMOTION_END value
                             * @property {number} OCCASION=10 OCCASION value
                             * @property {number} FINAL_URLS=11 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=12 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=13 TRACKING_URL value
                             * @property {number} LANGUAGE=14 LANGUAGE value
                             * @property {number} FINAL_URL_SUFFIX=15 FINAL_URL_SUFFIX value
                             */
                            PromotionPlaceholderFieldEnum.PromotionPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PROMOTION_TARGET"] = 2;
                                values[valuesById[3] = "DISCOUNT_MODIFIER"] = 3;
                                values[valuesById[4] = "PERCENT_OFF"] = 4;
                                values[valuesById[5] = "MONEY_AMOUNT_OFF"] = 5;
                                values[valuesById[6] = "PROMOTION_CODE"] = 6;
                                values[valuesById[7] = "ORDERS_OVER_AMOUNT"] = 7;
                                values[valuesById[8] = "PROMOTION_START"] = 8;
                                values[valuesById[9] = "PROMOTION_END"] = 9;
                                values[valuesById[10] = "OCCASION"] = 10;
                                values[valuesById[11] = "FINAL_URLS"] = 11;
                                values[valuesById[12] = "FINAL_MOBILE_URLS"] = 12;
                                values[valuesById[13] = "TRACKING_URL"] = 13;
                                values[valuesById[14] = "LANGUAGE"] = 14;
                                values[valuesById[15] = "FINAL_URL_SUFFIX"] = 15;
                                return values;
                            })();
    
                            return PromotionPlaceholderFieldEnum;
                        })();
    
                        enums.ReachPlanAdLengthEnum = (function() {
    
                            /**
                             * Properties of a ReachPlanAdLengthEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IReachPlanAdLengthEnum
                             */
    
                            /**
                             * Constructs a new ReachPlanAdLengthEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ReachPlanAdLengthEnum.
                             * @implements IReachPlanAdLengthEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IReachPlanAdLengthEnum=} [p] Properties to set
                             */
                            function ReachPlanAdLengthEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ReachPlanAdLengthEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanAdLengthEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAdLengthEnum} ReachPlanAdLengthEnum instance
                             */
                            ReachPlanAdLengthEnum.create = function create(properties) {
                                return new ReachPlanAdLengthEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ReachPlanAdLengthEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ReachPlanAdLengthEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanAdLengthEnum} m ReachPlanAdLengthEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanAdLengthEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ReachPlanAdLengthEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ReachPlanAdLengthEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanAdLengthEnum} message ReachPlanAdLengthEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanAdLengthEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ReachPlanAdLengthEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAdLengthEnum} ReachPlanAdLengthEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanAdLengthEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ReachPlanAdLengthEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ReachPlanAdLengthEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAdLengthEnum} ReachPlanAdLengthEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanAdLengthEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ReachPlanAdLengthEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ReachPlanAdLengthEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ReachPlanAdLengthEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAdLengthEnum} ReachPlanAdLengthEnum
                             */
                            ReachPlanAdLengthEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ReachPlanAdLengthEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ReachPlanAdLengthEnum();
                            };
    
                            /**
                             * Creates a plain object from a ReachPlanAdLengthEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ReachPlanAdLengthEnum} m ReachPlanAdLengthEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ReachPlanAdLengthEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ReachPlanAdLengthEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAdLengthEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ReachPlanAdLengthEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ReachPlanAdLength enum.
                             * @name google.ads.googleads.v11.enums.ReachPlanAdLengthEnum.ReachPlanAdLength
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SIX_SECONDS=2 SIX_SECONDS value
                             * @property {number} FIFTEEN_OR_TWENTY_SECONDS=3 FIFTEEN_OR_TWENTY_SECONDS value
                             * @property {number} TWENTY_SECONDS_OR_MORE=4 TWENTY_SECONDS_OR_MORE value
                             */
                            ReachPlanAdLengthEnum.ReachPlanAdLength = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SIX_SECONDS"] = 2;
                                values[valuesById[3] = "FIFTEEN_OR_TWENTY_SECONDS"] = 3;
                                values[valuesById[4] = "TWENTY_SECONDS_OR_MORE"] = 4;
                                return values;
                            })();
    
                            return ReachPlanAdLengthEnum;
                        })();
    
                        enums.ReachPlanAgeRangeEnum = (function() {
    
                            /**
                             * Properties of a ReachPlanAgeRangeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IReachPlanAgeRangeEnum
                             */
    
                            /**
                             * Constructs a new ReachPlanAgeRangeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ReachPlanAgeRangeEnum.
                             * @implements IReachPlanAgeRangeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IReachPlanAgeRangeEnum=} [p] Properties to set
                             */
                            function ReachPlanAgeRangeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ReachPlanAgeRangeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanAgeRangeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum} ReachPlanAgeRangeEnum instance
                             */
                            ReachPlanAgeRangeEnum.create = function create(properties) {
                                return new ReachPlanAgeRangeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ReachPlanAgeRangeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanAgeRangeEnum} m ReachPlanAgeRangeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanAgeRangeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ReachPlanAgeRangeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanAgeRangeEnum} message ReachPlanAgeRangeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanAgeRangeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ReachPlanAgeRangeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum} ReachPlanAgeRangeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanAgeRangeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ReachPlanAgeRangeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum} ReachPlanAgeRangeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanAgeRangeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ReachPlanAgeRangeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ReachPlanAgeRangeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ReachPlanAgeRangeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum} ReachPlanAgeRangeEnum
                             */
                            ReachPlanAgeRangeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ReachPlanAgeRangeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum} m ReachPlanAgeRangeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ReachPlanAgeRangeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ReachPlanAgeRangeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ReachPlanAgeRangeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ReachPlanAgeRange enum.
                             * @name google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.ReachPlanAgeRange
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AGE_RANGE_18_24=503001 AGE_RANGE_18_24 value
                             * @property {number} AGE_RANGE_18_34=2 AGE_RANGE_18_34 value
                             * @property {number} AGE_RANGE_18_44=3 AGE_RANGE_18_44 value
                             * @property {number} AGE_RANGE_18_49=4 AGE_RANGE_18_49 value
                             * @property {number} AGE_RANGE_18_54=5 AGE_RANGE_18_54 value
                             * @property {number} AGE_RANGE_18_64=6 AGE_RANGE_18_64 value
                             * @property {number} AGE_RANGE_18_65_UP=7 AGE_RANGE_18_65_UP value
                             * @property {number} AGE_RANGE_21_34=8 AGE_RANGE_21_34 value
                             * @property {number} AGE_RANGE_25_34=503002 AGE_RANGE_25_34 value
                             * @property {number} AGE_RANGE_25_44=9 AGE_RANGE_25_44 value
                             * @property {number} AGE_RANGE_25_49=10 AGE_RANGE_25_49 value
                             * @property {number} AGE_RANGE_25_54=11 AGE_RANGE_25_54 value
                             * @property {number} AGE_RANGE_25_64=12 AGE_RANGE_25_64 value
                             * @property {number} AGE_RANGE_25_65_UP=13 AGE_RANGE_25_65_UP value
                             * @property {number} AGE_RANGE_35_44=503003 AGE_RANGE_35_44 value
                             * @property {number} AGE_RANGE_35_49=14 AGE_RANGE_35_49 value
                             * @property {number} AGE_RANGE_35_54=15 AGE_RANGE_35_54 value
                             * @property {number} AGE_RANGE_35_64=16 AGE_RANGE_35_64 value
                             * @property {number} AGE_RANGE_35_65_UP=17 AGE_RANGE_35_65_UP value
                             * @property {number} AGE_RANGE_45_54=503004 AGE_RANGE_45_54 value
                             * @property {number} AGE_RANGE_45_64=18 AGE_RANGE_45_64 value
                             * @property {number} AGE_RANGE_45_65_UP=19 AGE_RANGE_45_65_UP value
                             * @property {number} AGE_RANGE_50_65_UP=20 AGE_RANGE_50_65_UP value
                             * @property {number} AGE_RANGE_55_64=503005 AGE_RANGE_55_64 value
                             * @property {number} AGE_RANGE_55_65_UP=21 AGE_RANGE_55_65_UP value
                             * @property {number} AGE_RANGE_65_UP=503006 AGE_RANGE_65_UP value
                             */
                            ReachPlanAgeRangeEnum.ReachPlanAgeRange = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[503001] = "AGE_RANGE_18_24"] = 503001;
                                values[valuesById[2] = "AGE_RANGE_18_34"] = 2;
                                values[valuesById[3] = "AGE_RANGE_18_44"] = 3;
                                values[valuesById[4] = "AGE_RANGE_18_49"] = 4;
                                values[valuesById[5] = "AGE_RANGE_18_54"] = 5;
                                values[valuesById[6] = "AGE_RANGE_18_64"] = 6;
                                values[valuesById[7] = "AGE_RANGE_18_65_UP"] = 7;
                                values[valuesById[8] = "AGE_RANGE_21_34"] = 8;
                                values[valuesById[503002] = "AGE_RANGE_25_34"] = 503002;
                                values[valuesById[9] = "AGE_RANGE_25_44"] = 9;
                                values[valuesById[10] = "AGE_RANGE_25_49"] = 10;
                                values[valuesById[11] = "AGE_RANGE_25_54"] = 11;
                                values[valuesById[12] = "AGE_RANGE_25_64"] = 12;
                                values[valuesById[13] = "AGE_RANGE_25_65_UP"] = 13;
                                values[valuesById[503003] = "AGE_RANGE_35_44"] = 503003;
                                values[valuesById[14] = "AGE_RANGE_35_49"] = 14;
                                values[valuesById[15] = "AGE_RANGE_35_54"] = 15;
                                values[valuesById[16] = "AGE_RANGE_35_64"] = 16;
                                values[valuesById[17] = "AGE_RANGE_35_65_UP"] = 17;
                                values[valuesById[503004] = "AGE_RANGE_45_54"] = 503004;
                                values[valuesById[18] = "AGE_RANGE_45_64"] = 18;
                                values[valuesById[19] = "AGE_RANGE_45_65_UP"] = 19;
                                values[valuesById[20] = "AGE_RANGE_50_65_UP"] = 20;
                                values[valuesById[503005] = "AGE_RANGE_55_64"] = 503005;
                                values[valuesById[21] = "AGE_RANGE_55_65_UP"] = 21;
                                values[valuesById[503006] = "AGE_RANGE_65_UP"] = 503006;
                                return values;
                            })();
    
                            return ReachPlanAgeRangeEnum;
                        })();
    
                        enums.ReachPlanNetworkEnum = (function() {
    
                            /**
                             * Properties of a ReachPlanNetworkEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IReachPlanNetworkEnum
                             */
    
                            /**
                             * Constructs a new ReachPlanNetworkEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ReachPlanNetworkEnum.
                             * @implements IReachPlanNetworkEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IReachPlanNetworkEnum=} [p] Properties to set
                             */
                            function ReachPlanNetworkEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ReachPlanNetworkEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanNetworkEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ReachPlanNetworkEnum} ReachPlanNetworkEnum instance
                             */
                            ReachPlanNetworkEnum.create = function create(properties) {
                                return new ReachPlanNetworkEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ReachPlanNetworkEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ReachPlanNetworkEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanNetworkEnum} m ReachPlanNetworkEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanNetworkEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ReachPlanNetworkEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ReachPlanNetworkEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IReachPlanNetworkEnum} message ReachPlanNetworkEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanNetworkEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ReachPlanNetworkEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ReachPlanNetworkEnum} ReachPlanNetworkEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanNetworkEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ReachPlanNetworkEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ReachPlanNetworkEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ReachPlanNetworkEnum} ReachPlanNetworkEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanNetworkEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ReachPlanNetworkEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ReachPlanNetworkEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ReachPlanNetworkEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ReachPlanNetworkEnum} ReachPlanNetworkEnum
                             */
                            ReachPlanNetworkEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ReachPlanNetworkEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ReachPlanNetworkEnum();
                            };
    
                            /**
                             * Creates a plain object from a ReachPlanNetworkEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ReachPlanNetworkEnum} m ReachPlanNetworkEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ReachPlanNetworkEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ReachPlanNetworkEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ReachPlanNetworkEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ReachPlanNetworkEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ReachPlanNetwork enum.
                             * @name google.ads.googleads.v11.enums.ReachPlanNetworkEnum.ReachPlanNetwork
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} YOUTUBE=2 YOUTUBE value
                             * @property {number} GOOGLE_VIDEO_PARTNERS=3 GOOGLE_VIDEO_PARTNERS value
                             * @property {number} YOUTUBE_AND_GOOGLE_VIDEO_PARTNERS=4 YOUTUBE_AND_GOOGLE_VIDEO_PARTNERS value
                             */
                            ReachPlanNetworkEnum.ReachPlanNetwork = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "YOUTUBE"] = 2;
                                values[valuesById[3] = "GOOGLE_VIDEO_PARTNERS"] = 3;
                                values[valuesById[4] = "YOUTUBE_AND_GOOGLE_VIDEO_PARTNERS"] = 4;
                                return values;
                            })();
    
                            return ReachPlanNetworkEnum;
                        })();
    
                        enums.RealEstatePlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a RealEstatePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IRealEstatePlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new RealEstatePlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a RealEstatePlaceholderFieldEnum.
                             * @implements IRealEstatePlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IRealEstatePlaceholderFieldEnum=} [p] Properties to set
                             */
                            function RealEstatePlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new RealEstatePlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IRealEstatePlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum} RealEstatePlaceholderFieldEnum instance
                             */
                            RealEstatePlaceholderFieldEnum.create = function create(properties) {
                                return new RealEstatePlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified RealEstatePlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IRealEstatePlaceholderFieldEnum} m RealEstatePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RealEstatePlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RealEstatePlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IRealEstatePlaceholderFieldEnum} message RealEstatePlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RealEstatePlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RealEstatePlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum} RealEstatePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RealEstatePlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RealEstatePlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum} RealEstatePlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RealEstatePlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RealEstatePlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RealEstatePlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a RealEstatePlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum} RealEstatePlaceholderFieldEnum
                             */
                            RealEstatePlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a RealEstatePlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum} m RealEstatePlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RealEstatePlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this RealEstatePlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RealEstatePlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * RealEstatePlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LISTING_ID=2 LISTING_ID value
                             * @property {number} LISTING_NAME=3 LISTING_NAME value
                             * @property {number} CITY_NAME=4 CITY_NAME value
                             * @property {number} DESCRIPTION=5 DESCRIPTION value
                             * @property {number} ADDRESS=6 ADDRESS value
                             * @property {number} PRICE=7 PRICE value
                             * @property {number} FORMATTED_PRICE=8 FORMATTED_PRICE value
                             * @property {number} IMAGE_URL=9 IMAGE_URL value
                             * @property {number} PROPERTY_TYPE=10 PROPERTY_TYPE value
                             * @property {number} LISTING_TYPE=11 LISTING_TYPE value
                             * @property {number} CONTEXTUAL_KEYWORDS=12 CONTEXTUAL_KEYWORDS value
                             * @property {number} FINAL_URLS=13 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=14 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=15 TRACKING_URL value
                             * @property {number} ANDROID_APP_LINK=16 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_LISTING_IDS=17 SIMILAR_LISTING_IDS value
                             * @property {number} IOS_APP_LINK=18 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=19 IOS_APP_STORE_ID value
                             */
                            RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LISTING_ID"] = 2;
                                values[valuesById[3] = "LISTING_NAME"] = 3;
                                values[valuesById[4] = "CITY_NAME"] = 4;
                                values[valuesById[5] = "DESCRIPTION"] = 5;
                                values[valuesById[6] = "ADDRESS"] = 6;
                                values[valuesById[7] = "PRICE"] = 7;
                                values[valuesById[8] = "FORMATTED_PRICE"] = 8;
                                values[valuesById[9] = "IMAGE_URL"] = 9;
                                values[valuesById[10] = "PROPERTY_TYPE"] = 10;
                                values[valuesById[11] = "LISTING_TYPE"] = 11;
                                values[valuesById[12] = "CONTEXTUAL_KEYWORDS"] = 12;
                                values[valuesById[13] = "FINAL_URLS"] = 13;
                                values[valuesById[14] = "FINAL_MOBILE_URLS"] = 14;
                                values[valuesById[15] = "TRACKING_URL"] = 15;
                                values[valuesById[16] = "ANDROID_APP_LINK"] = 16;
                                values[valuesById[17] = "SIMILAR_LISTING_IDS"] = 17;
                                values[valuesById[18] = "IOS_APP_LINK"] = 18;
                                values[valuesById[19] = "IOS_APP_STORE_ID"] = 19;
                                return values;
                            })();
    
                            return RealEstatePlaceholderFieldEnum;
                        })();
    
                        enums.ResourceChangeOperationEnum = (function() {
    
                            /**
                             * Properties of a ResourceChangeOperationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IResourceChangeOperationEnum
                             */
    
                            /**
                             * Constructs a new ResourceChangeOperationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ResourceChangeOperationEnum.
                             * @implements IResourceChangeOperationEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IResourceChangeOperationEnum=} [p] Properties to set
                             */
                            function ResourceChangeOperationEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ResourceChangeOperationEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResourceChangeOperationEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ResourceChangeOperationEnum} ResourceChangeOperationEnum instance
                             */
                            ResourceChangeOperationEnum.create = function create(properties) {
                                return new ResourceChangeOperationEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ResourceChangeOperationEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ResourceChangeOperationEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResourceChangeOperationEnum} m ResourceChangeOperationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceChangeOperationEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResourceChangeOperationEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ResourceChangeOperationEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResourceChangeOperationEnum} message ResourceChangeOperationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceChangeOperationEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResourceChangeOperationEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ResourceChangeOperationEnum} ResourceChangeOperationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceChangeOperationEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ResourceChangeOperationEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResourceChangeOperationEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ResourceChangeOperationEnum} ResourceChangeOperationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceChangeOperationEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResourceChangeOperationEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResourceChangeOperationEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ResourceChangeOperationEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ResourceChangeOperationEnum} ResourceChangeOperationEnum
                             */
                            ResourceChangeOperationEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ResourceChangeOperationEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ResourceChangeOperationEnum();
                            };
    
                            /**
                             * Creates a plain object from a ResourceChangeOperationEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ResourceChangeOperationEnum} m ResourceChangeOperationEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResourceChangeOperationEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ResourceChangeOperationEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ResourceChangeOperationEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResourceChangeOperationEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ResourceChangeOperation enum.
                             * @name google.ads.googleads.v11.enums.ResourceChangeOperationEnum.ResourceChangeOperation
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CREATE=2 CREATE value
                             * @property {number} UPDATE=3 UPDATE value
                             * @property {number} REMOVE=4 REMOVE value
                             */
                            ResourceChangeOperationEnum.ResourceChangeOperation = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CREATE"] = 2;
                                values[valuesById[3] = "UPDATE"] = 3;
                                values[valuesById[4] = "REMOVE"] = 4;
                                return values;
                            })();
    
                            return ResourceChangeOperationEnum;
                        })();
    
                        enums.ResourceLimitTypeEnum = (function() {
    
                            /**
                             * Properties of a ResourceLimitTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IResourceLimitTypeEnum
                             */
    
                            /**
                             * Constructs a new ResourceLimitTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ResourceLimitTypeEnum.
                             * @implements IResourceLimitTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IResourceLimitTypeEnum=} [p] Properties to set
                             */
                            function ResourceLimitTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ResourceLimitTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResourceLimitTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ResourceLimitTypeEnum} ResourceLimitTypeEnum instance
                             */
                            ResourceLimitTypeEnum.create = function create(properties) {
                                return new ResourceLimitTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ResourceLimitTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ResourceLimitTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResourceLimitTypeEnum} m ResourceLimitTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceLimitTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResourceLimitTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ResourceLimitTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResourceLimitTypeEnum} message ResourceLimitTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceLimitTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResourceLimitTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ResourceLimitTypeEnum} ResourceLimitTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceLimitTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ResourceLimitTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResourceLimitTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ResourceLimitTypeEnum} ResourceLimitTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceLimitTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResourceLimitTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResourceLimitTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ResourceLimitTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ResourceLimitTypeEnum} ResourceLimitTypeEnum
                             */
                            ResourceLimitTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ResourceLimitTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ResourceLimitTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ResourceLimitTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ResourceLimitTypeEnum} m ResourceLimitTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResourceLimitTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ResourceLimitTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ResourceLimitTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResourceLimitTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ResourceLimitType enum.
                             * @name google.ads.googleads.v11.enums.ResourceLimitTypeEnum.ResourceLimitType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CAMPAIGNS_PER_CUSTOMER=2 CAMPAIGNS_PER_CUSTOMER value
                             * @property {number} BASE_CAMPAIGNS_PER_CUSTOMER=3 BASE_CAMPAIGNS_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_CAMPAIGNS_PER_CUSTOMER=105 EXPERIMENT_CAMPAIGNS_PER_CUSTOMER value
                             * @property {number} HOTEL_CAMPAIGNS_PER_CUSTOMER=4 HOTEL_CAMPAIGNS_PER_CUSTOMER value
                             * @property {number} SMART_SHOPPING_CAMPAIGNS_PER_CUSTOMER=5 SMART_SHOPPING_CAMPAIGNS_PER_CUSTOMER value
                             * @property {number} AD_GROUPS_PER_CAMPAIGN=6 AD_GROUPS_PER_CAMPAIGN value
                             * @property {number} AD_GROUPS_PER_SHOPPING_CAMPAIGN=8 AD_GROUPS_PER_SHOPPING_CAMPAIGN value
                             * @property {number} AD_GROUPS_PER_HOTEL_CAMPAIGN=9 AD_GROUPS_PER_HOTEL_CAMPAIGN value
                             * @property {number} REPORTING_AD_GROUPS_PER_LOCAL_CAMPAIGN=10 REPORTING_AD_GROUPS_PER_LOCAL_CAMPAIGN value
                             * @property {number} REPORTING_AD_GROUPS_PER_APP_CAMPAIGN=11 REPORTING_AD_GROUPS_PER_APP_CAMPAIGN value
                             * @property {number} MANAGED_AD_GROUPS_PER_SMART_CAMPAIGN=52 MANAGED_AD_GROUPS_PER_SMART_CAMPAIGN value
                             * @property {number} AD_GROUP_CRITERIA_PER_CUSTOMER=12 AD_GROUP_CRITERIA_PER_CUSTOMER value
                             * @property {number} BASE_AD_GROUP_CRITERIA_PER_CUSTOMER=13 BASE_AD_GROUP_CRITERIA_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_AD_GROUP_CRITERIA_PER_CUSTOMER=107 EXPERIMENT_AD_GROUP_CRITERIA_PER_CUSTOMER value
                             * @property {number} AD_GROUP_CRITERIA_PER_CAMPAIGN=14 AD_GROUP_CRITERIA_PER_CAMPAIGN value
                             * @property {number} CAMPAIGN_CRITERIA_PER_CUSTOMER=15 CAMPAIGN_CRITERIA_PER_CUSTOMER value
                             * @property {number} BASE_CAMPAIGN_CRITERIA_PER_CUSTOMER=16 BASE_CAMPAIGN_CRITERIA_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_CAMPAIGN_CRITERIA_PER_CUSTOMER=108 EXPERIMENT_CAMPAIGN_CRITERIA_PER_CUSTOMER value
                             * @property {number} WEBPAGE_CRITERIA_PER_CUSTOMER=17 WEBPAGE_CRITERIA_PER_CUSTOMER value
                             * @property {number} BASE_WEBPAGE_CRITERIA_PER_CUSTOMER=18 BASE_WEBPAGE_CRITERIA_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_WEBPAGE_CRITERIA_PER_CUSTOMER=19 EXPERIMENT_WEBPAGE_CRITERIA_PER_CUSTOMER value
                             * @property {number} COMBINED_AUDIENCE_CRITERIA_PER_AD_GROUP=20 COMBINED_AUDIENCE_CRITERIA_PER_AD_GROUP value
                             * @property {number} CUSTOMER_NEGATIVE_PLACEMENT_CRITERIA_PER_CUSTOMER=21 CUSTOMER_NEGATIVE_PLACEMENT_CRITERIA_PER_CUSTOMER value
                             * @property {number} CUSTOMER_NEGATIVE_YOUTUBE_CHANNEL_CRITERIA_PER_CUSTOMER=22 CUSTOMER_NEGATIVE_YOUTUBE_CHANNEL_CRITERIA_PER_CUSTOMER value
                             * @property {number} CRITERIA_PER_AD_GROUP=23 CRITERIA_PER_AD_GROUP value
                             * @property {number} LISTING_GROUPS_PER_AD_GROUP=24 LISTING_GROUPS_PER_AD_GROUP value
                             * @property {number} EXPLICITLY_SHARED_BUDGETS_PER_CUSTOMER=25 EXPLICITLY_SHARED_BUDGETS_PER_CUSTOMER value
                             * @property {number} IMPLICITLY_SHARED_BUDGETS_PER_CUSTOMER=26 IMPLICITLY_SHARED_BUDGETS_PER_CUSTOMER value
                             * @property {number} COMBINED_AUDIENCE_CRITERIA_PER_CAMPAIGN=27 COMBINED_AUDIENCE_CRITERIA_PER_CAMPAIGN value
                             * @property {number} NEGATIVE_KEYWORDS_PER_CAMPAIGN=28 NEGATIVE_KEYWORDS_PER_CAMPAIGN value
                             * @property {number} NEGATIVE_PLACEMENTS_PER_CAMPAIGN=29 NEGATIVE_PLACEMENTS_PER_CAMPAIGN value
                             * @property {number} GEO_TARGETS_PER_CAMPAIGN=30 GEO_TARGETS_PER_CAMPAIGN value
                             * @property {number} NEGATIVE_IP_BLOCKS_PER_CAMPAIGN=32 NEGATIVE_IP_BLOCKS_PER_CAMPAIGN value
                             * @property {number} PROXIMITIES_PER_CAMPAIGN=33 PROXIMITIES_PER_CAMPAIGN value
                             * @property {number} LISTING_SCOPES_PER_SHOPPING_CAMPAIGN=34 LISTING_SCOPES_PER_SHOPPING_CAMPAIGN value
                             * @property {number} LISTING_SCOPES_PER_NON_SHOPPING_CAMPAIGN=35 LISTING_SCOPES_PER_NON_SHOPPING_CAMPAIGN value
                             * @property {number} NEGATIVE_KEYWORDS_PER_SHARED_SET=36 NEGATIVE_KEYWORDS_PER_SHARED_SET value
                             * @property {number} NEGATIVE_PLACEMENTS_PER_SHARED_SET=37 NEGATIVE_PLACEMENTS_PER_SHARED_SET value
                             * @property {number} SHARED_SETS_PER_CUSTOMER_FOR_TYPE_DEFAULT=40 SHARED_SETS_PER_CUSTOMER_FOR_TYPE_DEFAULT value
                             * @property {number} SHARED_SETS_PER_CUSTOMER_FOR_NEGATIVE_PLACEMENT_LIST_LOWER=41 SHARED_SETS_PER_CUSTOMER_FOR_NEGATIVE_PLACEMENT_LIST_LOWER value
                             * @property {number} HOTEL_ADVANCE_BOOKING_WINDOW_BID_MODIFIERS_PER_AD_GROUP=44 HOTEL_ADVANCE_BOOKING_WINDOW_BID_MODIFIERS_PER_AD_GROUP value
                             * @property {number} BIDDING_STRATEGIES_PER_CUSTOMER=45 BIDDING_STRATEGIES_PER_CUSTOMER value
                             * @property {number} BASIC_USER_LISTS_PER_CUSTOMER=47 BASIC_USER_LISTS_PER_CUSTOMER value
                             * @property {number} LOGICAL_USER_LISTS_PER_CUSTOMER=48 LOGICAL_USER_LISTS_PER_CUSTOMER value
                             * @property {number} RULE_BASED_USER_LISTS_PER_CUSTOMER=153 RULE_BASED_USER_LISTS_PER_CUSTOMER value
                             * @property {number} BASE_AD_GROUP_ADS_PER_CUSTOMER=53 BASE_AD_GROUP_ADS_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_AD_GROUP_ADS_PER_CUSTOMER=54 EXPERIMENT_AD_GROUP_ADS_PER_CUSTOMER value
                             * @property {number} AD_GROUP_ADS_PER_CAMPAIGN=55 AD_GROUP_ADS_PER_CAMPAIGN value
                             * @property {number} TEXT_AND_OTHER_ADS_PER_AD_GROUP=56 TEXT_AND_OTHER_ADS_PER_AD_GROUP value
                             * @property {number} IMAGE_ADS_PER_AD_GROUP=57 IMAGE_ADS_PER_AD_GROUP value
                             * @property {number} SHOPPING_SMART_ADS_PER_AD_GROUP=58 SHOPPING_SMART_ADS_PER_AD_GROUP value
                             * @property {number} RESPONSIVE_SEARCH_ADS_PER_AD_GROUP=59 RESPONSIVE_SEARCH_ADS_PER_AD_GROUP value
                             * @property {number} APP_ADS_PER_AD_GROUP=60 APP_ADS_PER_AD_GROUP value
                             * @property {number} APP_ENGAGEMENT_ADS_PER_AD_GROUP=61 APP_ENGAGEMENT_ADS_PER_AD_GROUP value
                             * @property {number} LOCAL_ADS_PER_AD_GROUP=62 LOCAL_ADS_PER_AD_GROUP value
                             * @property {number} VIDEO_ADS_PER_AD_GROUP=63 VIDEO_ADS_PER_AD_GROUP value
                             * @property {number} LEAD_FORM_CAMPAIGN_ASSETS_PER_CAMPAIGN=143 LEAD_FORM_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} PROMOTION_CUSTOMER_ASSETS_PER_CUSTOMER=79 PROMOTION_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} PROMOTION_CAMPAIGN_ASSETS_PER_CAMPAIGN=80 PROMOTION_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} PROMOTION_AD_GROUP_ASSETS_PER_AD_GROUP=81 PROMOTION_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER=134 CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN=135 CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP=136 CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} SITELINK_CUSTOMER_ASSETS_PER_CUSTOMER=137 SITELINK_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} SITELINK_CAMPAIGN_ASSETS_PER_CAMPAIGN=138 SITELINK_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} SITELINK_AD_GROUP_ASSETS_PER_AD_GROUP=139 SITELINK_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} STRUCTURED_SNIPPET_CUSTOMER_ASSETS_PER_CUSTOMER=140 STRUCTURED_SNIPPET_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} STRUCTURED_SNIPPET_CAMPAIGN_ASSETS_PER_CAMPAIGN=141 STRUCTURED_SNIPPET_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} STRUCTURED_SNIPPET_AD_GROUP_ASSETS_PER_AD_GROUP=142 STRUCTURED_SNIPPET_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} MOBILE_APP_CUSTOMER_ASSETS_PER_CUSTOMER=144 MOBILE_APP_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} MOBILE_APP_CAMPAIGN_ASSETS_PER_CAMPAIGN=145 MOBILE_APP_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} MOBILE_APP_AD_GROUP_ASSETS_PER_AD_GROUP=146 MOBILE_APP_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} HOTEL_CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER=147 HOTEL_CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} HOTEL_CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN=148 HOTEL_CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} HOTEL_CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP=149 HOTEL_CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} CALL_CUSTOMER_ASSETS_PER_CUSTOMER=150 CALL_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} CALL_CAMPAIGN_ASSETS_PER_CAMPAIGN=151 CALL_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} CALL_AD_GROUP_ASSETS_PER_AD_GROUP=152 CALL_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} PRICE_CUSTOMER_ASSETS_PER_CUSTOMER=154 PRICE_CUSTOMER_ASSETS_PER_CUSTOMER value
                             * @property {number} PRICE_CAMPAIGN_ASSETS_PER_CAMPAIGN=155 PRICE_CAMPAIGN_ASSETS_PER_CAMPAIGN value
                             * @property {number} PRICE_AD_GROUP_ASSETS_PER_AD_GROUP=156 PRICE_AD_GROUP_ASSETS_PER_AD_GROUP value
                             * @property {number} PAGE_FEED_ASSET_SETS_PER_CUSTOMER=157 PAGE_FEED_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} DYNAMIC_EDUCATION_FEED_ASSET_SETS_PER_CUSTOMER=158 DYNAMIC_EDUCATION_FEED_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_PAGE_FEED_ASSET_SET=159 ASSETS_PER_PAGE_FEED_ASSET_SET value
                             * @property {number} ASSETS_PER_DYNAMIC_EDUCATION_FEED_ASSET_SET=160 ASSETS_PER_DYNAMIC_EDUCATION_FEED_ASSET_SET value
                             * @property {number} DYNAMIC_REAL_ESTATE_ASSET_SETS_PER_CUSTOMER=161 DYNAMIC_REAL_ESTATE_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_DYNAMIC_REAL_ESTATE_ASSET_SET=162 ASSETS_PER_DYNAMIC_REAL_ESTATE_ASSET_SET value
                             * @property {number} DYNAMIC_CUSTOM_ASSET_SETS_PER_CUSTOMER=163 DYNAMIC_CUSTOM_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_DYNAMIC_CUSTOM_ASSET_SET=164 ASSETS_PER_DYNAMIC_CUSTOM_ASSET_SET value
                             * @property {number} DYNAMIC_HOTELS_AND_RENTALS_ASSET_SETS_PER_CUSTOMER=165 DYNAMIC_HOTELS_AND_RENTALS_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_DYNAMIC_HOTELS_AND_RENTALS_ASSET_SET=166 ASSETS_PER_DYNAMIC_HOTELS_AND_RENTALS_ASSET_SET value
                             * @property {number} DYNAMIC_LOCAL_ASSET_SETS_PER_CUSTOMER=167 DYNAMIC_LOCAL_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_DYNAMIC_LOCAL_ASSET_SET=168 ASSETS_PER_DYNAMIC_LOCAL_ASSET_SET value
                             * @property {number} DYNAMIC_FLIGHTS_ASSET_SETS_PER_CUSTOMER=169 DYNAMIC_FLIGHTS_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_DYNAMIC_FLIGHTS_ASSET_SET=170 ASSETS_PER_DYNAMIC_FLIGHTS_ASSET_SET value
                             * @property {number} DYNAMIC_TRAVEL_ASSET_SETS_PER_CUSTOMER=171 DYNAMIC_TRAVEL_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_DYNAMIC_TRAVEL_ASSET_SET=172 ASSETS_PER_DYNAMIC_TRAVEL_ASSET_SET value
                             * @property {number} DYNAMIC_JOBS_ASSET_SETS_PER_CUSTOMER=173 DYNAMIC_JOBS_ASSET_SETS_PER_CUSTOMER value
                             * @property {number} ASSETS_PER_DYNAMIC_JOBS_ASSET_SET=174 ASSETS_PER_DYNAMIC_JOBS_ASSET_SET value
                             * @property {number} VERSIONS_PER_AD=82 VERSIONS_PER_AD value
                             * @property {number} USER_FEEDS_PER_CUSTOMER=90 USER_FEEDS_PER_CUSTOMER value
                             * @property {number} SYSTEM_FEEDS_PER_CUSTOMER=91 SYSTEM_FEEDS_PER_CUSTOMER value
                             * @property {number} FEED_ATTRIBUTES_PER_FEED=92 FEED_ATTRIBUTES_PER_FEED value
                             * @property {number} FEED_ITEMS_PER_CUSTOMER=94 FEED_ITEMS_PER_CUSTOMER value
                             * @property {number} CAMPAIGN_FEEDS_PER_CUSTOMER=95 CAMPAIGN_FEEDS_PER_CUSTOMER value
                             * @property {number} BASE_CAMPAIGN_FEEDS_PER_CUSTOMER=96 BASE_CAMPAIGN_FEEDS_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_CAMPAIGN_FEEDS_PER_CUSTOMER=109 EXPERIMENT_CAMPAIGN_FEEDS_PER_CUSTOMER value
                             * @property {number} AD_GROUP_FEEDS_PER_CUSTOMER=97 AD_GROUP_FEEDS_PER_CUSTOMER value
                             * @property {number} BASE_AD_GROUP_FEEDS_PER_CUSTOMER=98 BASE_AD_GROUP_FEEDS_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_AD_GROUP_FEEDS_PER_CUSTOMER=110 EXPERIMENT_AD_GROUP_FEEDS_PER_CUSTOMER value
                             * @property {number} AD_GROUP_FEEDS_PER_CAMPAIGN=99 AD_GROUP_FEEDS_PER_CAMPAIGN value
                             * @property {number} FEED_ITEM_SETS_PER_CUSTOMER=100 FEED_ITEM_SETS_PER_CUSTOMER value
                             * @property {number} FEED_ITEMS_PER_FEED_ITEM_SET=101 FEED_ITEMS_PER_FEED_ITEM_SET value
                             * @property {number} CAMPAIGN_EXPERIMENTS_PER_CUSTOMER=112 CAMPAIGN_EXPERIMENTS_PER_CUSTOMER value
                             * @property {number} EXPERIMENT_ARMS_PER_VIDEO_EXPERIMENT=113 EXPERIMENT_ARMS_PER_VIDEO_EXPERIMENT value
                             * @property {number} OWNED_LABELS_PER_CUSTOMER=115 OWNED_LABELS_PER_CUSTOMER value
                             * @property {number} LABELS_PER_CAMPAIGN=117 LABELS_PER_CAMPAIGN value
                             * @property {number} LABELS_PER_AD_GROUP=118 LABELS_PER_AD_GROUP value
                             * @property {number} LABELS_PER_AD_GROUP_AD=119 LABELS_PER_AD_GROUP_AD value
                             * @property {number} LABELS_PER_AD_GROUP_CRITERION=120 LABELS_PER_AD_GROUP_CRITERION value
                             * @property {number} TARGET_CUSTOMERS_PER_LABEL=121 TARGET_CUSTOMERS_PER_LABEL value
                             * @property {number} KEYWORD_PLANS_PER_USER_PER_CUSTOMER=122 KEYWORD_PLANS_PER_USER_PER_CUSTOMER value
                             * @property {number} KEYWORD_PLAN_AD_GROUP_KEYWORDS_PER_KEYWORD_PLAN=123 KEYWORD_PLAN_AD_GROUP_KEYWORDS_PER_KEYWORD_PLAN value
                             * @property {number} KEYWORD_PLAN_AD_GROUPS_PER_KEYWORD_PLAN=124 KEYWORD_PLAN_AD_GROUPS_PER_KEYWORD_PLAN value
                             * @property {number} KEYWORD_PLAN_NEGATIVE_KEYWORDS_PER_KEYWORD_PLAN=125 KEYWORD_PLAN_NEGATIVE_KEYWORDS_PER_KEYWORD_PLAN value
                             * @property {number} KEYWORD_PLAN_CAMPAIGNS_PER_KEYWORD_PLAN=126 KEYWORD_PLAN_CAMPAIGNS_PER_KEYWORD_PLAN value
                             * @property {number} CONVERSION_ACTIONS_PER_CUSTOMER=128 CONVERSION_ACTIONS_PER_CUSTOMER value
                             * @property {number} BATCH_JOB_OPERATIONS_PER_JOB=130 BATCH_JOB_OPERATIONS_PER_JOB value
                             * @property {number} BATCH_JOBS_PER_CUSTOMER=131 BATCH_JOBS_PER_CUSTOMER value
                             * @property {number} HOTEL_CHECK_IN_DATE_RANGE_BID_MODIFIERS_PER_AD_GROUP=132 HOTEL_CHECK_IN_DATE_RANGE_BID_MODIFIERS_PER_AD_GROUP value
                             */
                            ResourceLimitTypeEnum.ResourceLimitType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CAMPAIGNS_PER_CUSTOMER"] = 2;
                                values[valuesById[3] = "BASE_CAMPAIGNS_PER_CUSTOMER"] = 3;
                                values[valuesById[105] = "EXPERIMENT_CAMPAIGNS_PER_CUSTOMER"] = 105;
                                values[valuesById[4] = "HOTEL_CAMPAIGNS_PER_CUSTOMER"] = 4;
                                values[valuesById[5] = "SMART_SHOPPING_CAMPAIGNS_PER_CUSTOMER"] = 5;
                                values[valuesById[6] = "AD_GROUPS_PER_CAMPAIGN"] = 6;
                                values[valuesById[8] = "AD_GROUPS_PER_SHOPPING_CAMPAIGN"] = 8;
                                values[valuesById[9] = "AD_GROUPS_PER_HOTEL_CAMPAIGN"] = 9;
                                values[valuesById[10] = "REPORTING_AD_GROUPS_PER_LOCAL_CAMPAIGN"] = 10;
                                values[valuesById[11] = "REPORTING_AD_GROUPS_PER_APP_CAMPAIGN"] = 11;
                                values[valuesById[52] = "MANAGED_AD_GROUPS_PER_SMART_CAMPAIGN"] = 52;
                                values[valuesById[12] = "AD_GROUP_CRITERIA_PER_CUSTOMER"] = 12;
                                values[valuesById[13] = "BASE_AD_GROUP_CRITERIA_PER_CUSTOMER"] = 13;
                                values[valuesById[107] = "EXPERIMENT_AD_GROUP_CRITERIA_PER_CUSTOMER"] = 107;
                                values[valuesById[14] = "AD_GROUP_CRITERIA_PER_CAMPAIGN"] = 14;
                                values[valuesById[15] = "CAMPAIGN_CRITERIA_PER_CUSTOMER"] = 15;
                                values[valuesById[16] = "BASE_CAMPAIGN_CRITERIA_PER_CUSTOMER"] = 16;
                                values[valuesById[108] = "EXPERIMENT_CAMPAIGN_CRITERIA_PER_CUSTOMER"] = 108;
                                values[valuesById[17] = "WEBPAGE_CRITERIA_PER_CUSTOMER"] = 17;
                                values[valuesById[18] = "BASE_WEBPAGE_CRITERIA_PER_CUSTOMER"] = 18;
                                values[valuesById[19] = "EXPERIMENT_WEBPAGE_CRITERIA_PER_CUSTOMER"] = 19;
                                values[valuesById[20] = "COMBINED_AUDIENCE_CRITERIA_PER_AD_GROUP"] = 20;
                                values[valuesById[21] = "CUSTOMER_NEGATIVE_PLACEMENT_CRITERIA_PER_CUSTOMER"] = 21;
                                values[valuesById[22] = "CUSTOMER_NEGATIVE_YOUTUBE_CHANNEL_CRITERIA_PER_CUSTOMER"] = 22;
                                values[valuesById[23] = "CRITERIA_PER_AD_GROUP"] = 23;
                                values[valuesById[24] = "LISTING_GROUPS_PER_AD_GROUP"] = 24;
                                values[valuesById[25] = "EXPLICITLY_SHARED_BUDGETS_PER_CUSTOMER"] = 25;
                                values[valuesById[26] = "IMPLICITLY_SHARED_BUDGETS_PER_CUSTOMER"] = 26;
                                values[valuesById[27] = "COMBINED_AUDIENCE_CRITERIA_PER_CAMPAIGN"] = 27;
                                values[valuesById[28] = "NEGATIVE_KEYWORDS_PER_CAMPAIGN"] = 28;
                                values[valuesById[29] = "NEGATIVE_PLACEMENTS_PER_CAMPAIGN"] = 29;
                                values[valuesById[30] = "GEO_TARGETS_PER_CAMPAIGN"] = 30;
                                values[valuesById[32] = "NEGATIVE_IP_BLOCKS_PER_CAMPAIGN"] = 32;
                                values[valuesById[33] = "PROXIMITIES_PER_CAMPAIGN"] = 33;
                                values[valuesById[34] = "LISTING_SCOPES_PER_SHOPPING_CAMPAIGN"] = 34;
                                values[valuesById[35] = "LISTING_SCOPES_PER_NON_SHOPPING_CAMPAIGN"] = 35;
                                values[valuesById[36] = "NEGATIVE_KEYWORDS_PER_SHARED_SET"] = 36;
                                values[valuesById[37] = "NEGATIVE_PLACEMENTS_PER_SHARED_SET"] = 37;
                                values[valuesById[40] = "SHARED_SETS_PER_CUSTOMER_FOR_TYPE_DEFAULT"] = 40;
                                values[valuesById[41] = "SHARED_SETS_PER_CUSTOMER_FOR_NEGATIVE_PLACEMENT_LIST_LOWER"] = 41;
                                values[valuesById[44] = "HOTEL_ADVANCE_BOOKING_WINDOW_BID_MODIFIERS_PER_AD_GROUP"] = 44;
                                values[valuesById[45] = "BIDDING_STRATEGIES_PER_CUSTOMER"] = 45;
                                values[valuesById[47] = "BASIC_USER_LISTS_PER_CUSTOMER"] = 47;
                                values[valuesById[48] = "LOGICAL_USER_LISTS_PER_CUSTOMER"] = 48;
                                values[valuesById[153] = "RULE_BASED_USER_LISTS_PER_CUSTOMER"] = 153;
                                values[valuesById[53] = "BASE_AD_GROUP_ADS_PER_CUSTOMER"] = 53;
                                values[valuesById[54] = "EXPERIMENT_AD_GROUP_ADS_PER_CUSTOMER"] = 54;
                                values[valuesById[55] = "AD_GROUP_ADS_PER_CAMPAIGN"] = 55;
                                values[valuesById[56] = "TEXT_AND_OTHER_ADS_PER_AD_GROUP"] = 56;
                                values[valuesById[57] = "IMAGE_ADS_PER_AD_GROUP"] = 57;
                                values[valuesById[58] = "SHOPPING_SMART_ADS_PER_AD_GROUP"] = 58;
                                values[valuesById[59] = "RESPONSIVE_SEARCH_ADS_PER_AD_GROUP"] = 59;
                                values[valuesById[60] = "APP_ADS_PER_AD_GROUP"] = 60;
                                values[valuesById[61] = "APP_ENGAGEMENT_ADS_PER_AD_GROUP"] = 61;
                                values[valuesById[62] = "LOCAL_ADS_PER_AD_GROUP"] = 62;
                                values[valuesById[63] = "VIDEO_ADS_PER_AD_GROUP"] = 63;
                                values[valuesById[143] = "LEAD_FORM_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 143;
                                values[valuesById[79] = "PROMOTION_CUSTOMER_ASSETS_PER_CUSTOMER"] = 79;
                                values[valuesById[80] = "PROMOTION_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 80;
                                values[valuesById[81] = "PROMOTION_AD_GROUP_ASSETS_PER_AD_GROUP"] = 81;
                                values[valuesById[134] = "CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER"] = 134;
                                values[valuesById[135] = "CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 135;
                                values[valuesById[136] = "CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP"] = 136;
                                values[valuesById[137] = "SITELINK_CUSTOMER_ASSETS_PER_CUSTOMER"] = 137;
                                values[valuesById[138] = "SITELINK_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 138;
                                values[valuesById[139] = "SITELINK_AD_GROUP_ASSETS_PER_AD_GROUP"] = 139;
                                values[valuesById[140] = "STRUCTURED_SNIPPET_CUSTOMER_ASSETS_PER_CUSTOMER"] = 140;
                                values[valuesById[141] = "STRUCTURED_SNIPPET_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 141;
                                values[valuesById[142] = "STRUCTURED_SNIPPET_AD_GROUP_ASSETS_PER_AD_GROUP"] = 142;
                                values[valuesById[144] = "MOBILE_APP_CUSTOMER_ASSETS_PER_CUSTOMER"] = 144;
                                values[valuesById[145] = "MOBILE_APP_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 145;
                                values[valuesById[146] = "MOBILE_APP_AD_GROUP_ASSETS_PER_AD_GROUP"] = 146;
                                values[valuesById[147] = "HOTEL_CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER"] = 147;
                                values[valuesById[148] = "HOTEL_CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 148;
                                values[valuesById[149] = "HOTEL_CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP"] = 149;
                                values[valuesById[150] = "CALL_CUSTOMER_ASSETS_PER_CUSTOMER"] = 150;
                                values[valuesById[151] = "CALL_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 151;
                                values[valuesById[152] = "CALL_AD_GROUP_ASSETS_PER_AD_GROUP"] = 152;
                                values[valuesById[154] = "PRICE_CUSTOMER_ASSETS_PER_CUSTOMER"] = 154;
                                values[valuesById[155] = "PRICE_CAMPAIGN_ASSETS_PER_CAMPAIGN"] = 155;
                                values[valuesById[156] = "PRICE_AD_GROUP_ASSETS_PER_AD_GROUP"] = 156;
                                values[valuesById[157] = "PAGE_FEED_ASSET_SETS_PER_CUSTOMER"] = 157;
                                values[valuesById[158] = "DYNAMIC_EDUCATION_FEED_ASSET_SETS_PER_CUSTOMER"] = 158;
                                values[valuesById[159] = "ASSETS_PER_PAGE_FEED_ASSET_SET"] = 159;
                                values[valuesById[160] = "ASSETS_PER_DYNAMIC_EDUCATION_FEED_ASSET_SET"] = 160;
                                values[valuesById[161] = "DYNAMIC_REAL_ESTATE_ASSET_SETS_PER_CUSTOMER"] = 161;
                                values[valuesById[162] = "ASSETS_PER_DYNAMIC_REAL_ESTATE_ASSET_SET"] = 162;
                                values[valuesById[163] = "DYNAMIC_CUSTOM_ASSET_SETS_PER_CUSTOMER"] = 163;
                                values[valuesById[164] = "ASSETS_PER_DYNAMIC_CUSTOM_ASSET_SET"] = 164;
                                values[valuesById[165] = "DYNAMIC_HOTELS_AND_RENTALS_ASSET_SETS_PER_CUSTOMER"] = 165;
                                values[valuesById[166] = "ASSETS_PER_DYNAMIC_HOTELS_AND_RENTALS_ASSET_SET"] = 166;
                                values[valuesById[167] = "DYNAMIC_LOCAL_ASSET_SETS_PER_CUSTOMER"] = 167;
                                values[valuesById[168] = "ASSETS_PER_DYNAMIC_LOCAL_ASSET_SET"] = 168;
                                values[valuesById[169] = "DYNAMIC_FLIGHTS_ASSET_SETS_PER_CUSTOMER"] = 169;
                                values[valuesById[170] = "ASSETS_PER_DYNAMIC_FLIGHTS_ASSET_SET"] = 170;
                                values[valuesById[171] = "DYNAMIC_TRAVEL_ASSET_SETS_PER_CUSTOMER"] = 171;
                                values[valuesById[172] = "ASSETS_PER_DYNAMIC_TRAVEL_ASSET_SET"] = 172;
                                values[valuesById[173] = "DYNAMIC_JOBS_ASSET_SETS_PER_CUSTOMER"] = 173;
                                values[valuesById[174] = "ASSETS_PER_DYNAMIC_JOBS_ASSET_SET"] = 174;
                                values[valuesById[82] = "VERSIONS_PER_AD"] = 82;
                                values[valuesById[90] = "USER_FEEDS_PER_CUSTOMER"] = 90;
                                values[valuesById[91] = "SYSTEM_FEEDS_PER_CUSTOMER"] = 91;
                                values[valuesById[92] = "FEED_ATTRIBUTES_PER_FEED"] = 92;
                                values[valuesById[94] = "FEED_ITEMS_PER_CUSTOMER"] = 94;
                                values[valuesById[95] = "CAMPAIGN_FEEDS_PER_CUSTOMER"] = 95;
                                values[valuesById[96] = "BASE_CAMPAIGN_FEEDS_PER_CUSTOMER"] = 96;
                                values[valuesById[109] = "EXPERIMENT_CAMPAIGN_FEEDS_PER_CUSTOMER"] = 109;
                                values[valuesById[97] = "AD_GROUP_FEEDS_PER_CUSTOMER"] = 97;
                                values[valuesById[98] = "BASE_AD_GROUP_FEEDS_PER_CUSTOMER"] = 98;
                                values[valuesById[110] = "EXPERIMENT_AD_GROUP_FEEDS_PER_CUSTOMER"] = 110;
                                values[valuesById[99] = "AD_GROUP_FEEDS_PER_CAMPAIGN"] = 99;
                                values[valuesById[100] = "FEED_ITEM_SETS_PER_CUSTOMER"] = 100;
                                values[valuesById[101] = "FEED_ITEMS_PER_FEED_ITEM_SET"] = 101;
                                values[valuesById[112] = "CAMPAIGN_EXPERIMENTS_PER_CUSTOMER"] = 112;
                                values[valuesById[113] = "EXPERIMENT_ARMS_PER_VIDEO_EXPERIMENT"] = 113;
                                values[valuesById[115] = "OWNED_LABELS_PER_CUSTOMER"] = 115;
                                values[valuesById[117] = "LABELS_PER_CAMPAIGN"] = 117;
                                values[valuesById[118] = "LABELS_PER_AD_GROUP"] = 118;
                                values[valuesById[119] = "LABELS_PER_AD_GROUP_AD"] = 119;
                                values[valuesById[120] = "LABELS_PER_AD_GROUP_CRITERION"] = 120;
                                values[valuesById[121] = "TARGET_CUSTOMERS_PER_LABEL"] = 121;
                                values[valuesById[122] = "KEYWORD_PLANS_PER_USER_PER_CUSTOMER"] = 122;
                                values[valuesById[123] = "KEYWORD_PLAN_AD_GROUP_KEYWORDS_PER_KEYWORD_PLAN"] = 123;
                                values[valuesById[124] = "KEYWORD_PLAN_AD_GROUPS_PER_KEYWORD_PLAN"] = 124;
                                values[valuesById[125] = "KEYWORD_PLAN_NEGATIVE_KEYWORDS_PER_KEYWORD_PLAN"] = 125;
                                values[valuesById[126] = "KEYWORD_PLAN_CAMPAIGNS_PER_KEYWORD_PLAN"] = 126;
                                values[valuesById[128] = "CONVERSION_ACTIONS_PER_CUSTOMER"] = 128;
                                values[valuesById[130] = "BATCH_JOB_OPERATIONS_PER_JOB"] = 130;
                                values[valuesById[131] = "BATCH_JOBS_PER_CUSTOMER"] = 131;
                                values[valuesById[132] = "HOTEL_CHECK_IN_DATE_RANGE_BID_MODIFIERS_PER_AD_GROUP"] = 132;
                                return values;
                            })();
    
                            return ResourceLimitTypeEnum;
                        })();
    
                        enums.ResponseContentTypeEnum = (function() {
    
                            /**
                             * Properties of a ResponseContentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IResponseContentTypeEnum
                             */
    
                            /**
                             * Constructs a new ResponseContentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ResponseContentTypeEnum.
                             * @implements IResponseContentTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IResponseContentTypeEnum=} [p] Properties to set
                             */
                            function ResponseContentTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ResponseContentTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResponseContentTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ResponseContentTypeEnum} ResponseContentTypeEnum instance
                             */
                            ResponseContentTypeEnum.create = function create(properties) {
                                return new ResponseContentTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ResponseContentTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ResponseContentTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResponseContentTypeEnum} m ResponseContentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponseContentTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResponseContentTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ResponseContentTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IResponseContentTypeEnum} message ResponseContentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResponseContentTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResponseContentTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ResponseContentTypeEnum} ResponseContentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponseContentTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResponseContentTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ResponseContentTypeEnum} ResponseContentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResponseContentTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResponseContentTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResponseContentTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ResponseContentTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ResponseContentTypeEnum} ResponseContentTypeEnum
                             */
                            ResponseContentTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ResponseContentTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ResponseContentTypeEnum} m ResponseContentTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResponseContentTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ResponseContentTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ResponseContentTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResponseContentTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ResponseContentType enum.
                             * @name google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} RESOURCE_NAME_ONLY=1 RESOURCE_NAME_ONLY value
                             * @property {number} MUTABLE_RESOURCE=2 MUTABLE_RESOURCE value
                             */
                            ResponseContentTypeEnum.ResponseContentType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "RESOURCE_NAME_ONLY"] = 1;
                                values[valuesById[2] = "MUTABLE_RESOURCE"] = 2;
                                return values;
                            })();
    
                            return ResponseContentTypeEnum;
                        })();
    
                        enums.SearchTermTargetingStatusEnum = (function() {
    
                            /**
                             * Properties of a SearchTermTargetingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISearchTermTargetingStatusEnum
                             */
    
                            /**
                             * Constructs a new SearchTermTargetingStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SearchTermTargetingStatusEnum.
                             * @implements ISearchTermTargetingStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISearchTermTargetingStatusEnum=} [p] Properties to set
                             */
                            function SearchTermTargetingStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SearchTermTargetingStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchTermTargetingStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum} SearchTermTargetingStatusEnum instance
                             */
                            SearchTermTargetingStatusEnum.create = function create(properties) {
                                return new SearchTermTargetingStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SearchTermTargetingStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchTermTargetingStatusEnum} m SearchTermTargetingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchTermTargetingStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchTermTargetingStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISearchTermTargetingStatusEnum} message SearchTermTargetingStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchTermTargetingStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchTermTargetingStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum} SearchTermTargetingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchTermTargetingStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchTermTargetingStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum} SearchTermTargetingStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchTermTargetingStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchTermTargetingStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchTermTargetingStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SearchTermTargetingStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum} SearchTermTargetingStatusEnum
                             */
                            SearchTermTargetingStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a SearchTermTargetingStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum} m SearchTermTargetingStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchTermTargetingStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SearchTermTargetingStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchTermTargetingStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SearchTermTargetingStatus enum.
                             * @name google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum.SearchTermTargetingStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADDED=2 ADDED value
                             * @property {number} EXCLUDED=3 EXCLUDED value
                             * @property {number} ADDED_EXCLUDED=4 ADDED_EXCLUDED value
                             * @property {number} NONE=5 NONE value
                             */
                            SearchTermTargetingStatusEnum.SearchTermTargetingStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADDED"] = 2;
                                values[valuesById[3] = "EXCLUDED"] = 3;
                                values[valuesById[4] = "ADDED_EXCLUDED"] = 4;
                                values[valuesById[5] = "NONE"] = 5;
                                return values;
                            })();
    
                            return SearchTermTargetingStatusEnum;
                        })();
    
                        enums.SeasonalityEventScopeEnum = (function() {
    
                            /**
                             * Properties of a SeasonalityEventScopeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISeasonalityEventScopeEnum
                             */
    
                            /**
                             * Constructs a new SeasonalityEventScopeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SeasonalityEventScopeEnum.
                             * @implements ISeasonalityEventScopeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventScopeEnum=} [p] Properties to set
                             */
                            function SeasonalityEventScopeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SeasonalityEventScopeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventScopeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum} SeasonalityEventScopeEnum instance
                             */
                            SeasonalityEventScopeEnum.create = function create(properties) {
                                return new SeasonalityEventScopeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SeasonalityEventScopeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventScopeEnum} m SeasonalityEventScopeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SeasonalityEventScopeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SeasonalityEventScopeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventScopeEnum} message SeasonalityEventScopeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SeasonalityEventScopeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SeasonalityEventScopeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum} SeasonalityEventScopeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SeasonalityEventScopeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SeasonalityEventScopeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SeasonalityEventScopeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum} SeasonalityEventScopeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SeasonalityEventScopeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SeasonalityEventScopeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SeasonalityEventScopeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SeasonalityEventScopeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum} SeasonalityEventScopeEnum
                             */
                            SeasonalityEventScopeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SeasonalityEventScopeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SeasonalityEventScopeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SeasonalityEventScopeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum} m SeasonalityEventScopeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SeasonalityEventScopeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SeasonalityEventScopeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventScopeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SeasonalityEventScopeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SeasonalityEventScope enum.
                             * @name google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.SeasonalityEventScope
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CUSTOMER=2 CUSTOMER value
                             * @property {number} CAMPAIGN=4 CAMPAIGN value
                             * @property {number} CHANNEL=5 CHANNEL value
                             */
                            SeasonalityEventScopeEnum.SeasonalityEventScope = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CUSTOMER"] = 2;
                                values[valuesById[4] = "CAMPAIGN"] = 4;
                                values[valuesById[5] = "CHANNEL"] = 5;
                                return values;
                            })();
    
                            return SeasonalityEventScopeEnum;
                        })();
    
                        enums.SeasonalityEventStatusEnum = (function() {
    
                            /**
                             * Properties of a SeasonalityEventStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISeasonalityEventStatusEnum
                             */
    
                            /**
                             * Constructs a new SeasonalityEventStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SeasonalityEventStatusEnum.
                             * @implements ISeasonalityEventStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventStatusEnum=} [p] Properties to set
                             */
                            function SeasonalityEventStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SeasonalityEventStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum} SeasonalityEventStatusEnum instance
                             */
                            SeasonalityEventStatusEnum.create = function create(properties) {
                                return new SeasonalityEventStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SeasonalityEventStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventStatusEnum} m SeasonalityEventStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SeasonalityEventStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SeasonalityEventStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISeasonalityEventStatusEnum} message SeasonalityEventStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SeasonalityEventStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SeasonalityEventStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum} SeasonalityEventStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SeasonalityEventStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SeasonalityEventStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SeasonalityEventStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum} SeasonalityEventStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SeasonalityEventStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SeasonalityEventStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SeasonalityEventStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SeasonalityEventStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum} SeasonalityEventStatusEnum
                             */
                            SeasonalityEventStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SeasonalityEventStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SeasonalityEventStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a SeasonalityEventStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum} m SeasonalityEventStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SeasonalityEventStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SeasonalityEventStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SeasonalityEventStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SeasonalityEventStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SeasonalityEventStatus enum.
                             * @name google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.SeasonalityEventStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=4 REMOVED value
                             */
                            SeasonalityEventStatusEnum.SeasonalityEventStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[4] = "REMOVED"] = 4;
                                return values;
                            })();
    
                            return SeasonalityEventStatusEnum;
                        })();
    
                        enums.SharedSetStatusEnum = (function() {
    
                            /**
                             * Properties of a SharedSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISharedSetStatusEnum
                             */
    
                            /**
                             * Constructs a new SharedSetStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SharedSetStatusEnum.
                             * @implements ISharedSetStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISharedSetStatusEnum=} [p] Properties to set
                             */
                            function SharedSetStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SharedSetStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISharedSetStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SharedSetStatusEnum} SharedSetStatusEnum instance
                             */
                            SharedSetStatusEnum.create = function create(properties) {
                                return new SharedSetStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SharedSetStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SharedSetStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISharedSetStatusEnum} m SharedSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedSetStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SharedSetStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISharedSetStatusEnum} message SharedSetStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedSetStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SharedSetStatusEnum} SharedSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SharedSetStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedSetStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SharedSetStatusEnum} SharedSetStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedSetStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedSetStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SharedSetStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SharedSetStatusEnum} SharedSetStatusEnum
                             */
                            SharedSetStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SharedSetStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SharedSetStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a SharedSetStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SharedSetStatusEnum} m SharedSetStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedSetStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SharedSetStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SharedSetStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedSetStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SharedSetStatus enum.
                             * @name google.ads.googleads.v11.enums.SharedSetStatusEnum.SharedSetStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} REMOVED=3 REMOVED value
                             */
                            SharedSetStatusEnum.SharedSetStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "REMOVED"] = 3;
                                return values;
                            })();
    
                            return SharedSetStatusEnum;
                        })();
    
                        enums.SharedSetTypeEnum = (function() {
    
                            /**
                             * Properties of a SharedSetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISharedSetTypeEnum
                             */
    
                            /**
                             * Constructs a new SharedSetTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SharedSetTypeEnum.
                             * @implements ISharedSetTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISharedSetTypeEnum=} [p] Properties to set
                             */
                            function SharedSetTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SharedSetTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISharedSetTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SharedSetTypeEnum} SharedSetTypeEnum instance
                             */
                            SharedSetTypeEnum.create = function create(properties) {
                                return new SharedSetTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SharedSetTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SharedSetTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISharedSetTypeEnum} m SharedSetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedSetTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SharedSetTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISharedSetTypeEnum} message SharedSetTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedSetTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SharedSetTypeEnum} SharedSetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SharedSetTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedSetTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SharedSetTypeEnum} SharedSetTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedSetTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedSetTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SharedSetTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SharedSetTypeEnum} SharedSetTypeEnum
                             */
                            SharedSetTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SharedSetTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SharedSetTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SharedSetTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SharedSetTypeEnum} m SharedSetTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedSetTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SharedSetTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SharedSetTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedSetTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SharedSetType enum.
                             * @name google.ads.googleads.v11.enums.SharedSetTypeEnum.SharedSetType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NEGATIVE_KEYWORDS=2 NEGATIVE_KEYWORDS value
                             * @property {number} NEGATIVE_PLACEMENTS=3 NEGATIVE_PLACEMENTS value
                             */
                            SharedSetTypeEnum.SharedSetType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NEGATIVE_KEYWORDS"] = 2;
                                values[valuesById[3] = "NEGATIVE_PLACEMENTS"] = 3;
                                return values;
                            })();
    
                            return SharedSetTypeEnum;
                        })();
    
                        enums.SimulationModificationMethodEnum = (function() {
    
                            /**
                             * Properties of a SimulationModificationMethodEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISimulationModificationMethodEnum
                             */
    
                            /**
                             * Constructs a new SimulationModificationMethodEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SimulationModificationMethodEnum.
                             * @implements ISimulationModificationMethodEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISimulationModificationMethodEnum=} [p] Properties to set
                             */
                            function SimulationModificationMethodEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SimulationModificationMethodEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISimulationModificationMethodEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SimulationModificationMethodEnum} SimulationModificationMethodEnum instance
                             */
                            SimulationModificationMethodEnum.create = function create(properties) {
                                return new SimulationModificationMethodEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SimulationModificationMethodEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SimulationModificationMethodEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISimulationModificationMethodEnum} m SimulationModificationMethodEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SimulationModificationMethodEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SimulationModificationMethodEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SimulationModificationMethodEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISimulationModificationMethodEnum} message SimulationModificationMethodEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SimulationModificationMethodEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SimulationModificationMethodEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SimulationModificationMethodEnum} SimulationModificationMethodEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SimulationModificationMethodEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SimulationModificationMethodEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SimulationModificationMethodEnum} SimulationModificationMethodEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SimulationModificationMethodEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SimulationModificationMethodEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SimulationModificationMethodEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SimulationModificationMethodEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SimulationModificationMethodEnum} SimulationModificationMethodEnum
                             */
                            SimulationModificationMethodEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum();
                            };
    
                            /**
                             * Creates a plain object from a SimulationModificationMethodEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SimulationModificationMethodEnum} m SimulationModificationMethodEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SimulationModificationMethodEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SimulationModificationMethodEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SimulationModificationMethodEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SimulationModificationMethodEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SimulationModificationMethod enum.
                             * @name google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNIFORM=2 UNIFORM value
                             * @property {number} DEFAULT=3 DEFAULT value
                             * @property {number} SCALING=4 SCALING value
                             */
                            SimulationModificationMethodEnum.SimulationModificationMethod = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNIFORM"] = 2;
                                values[valuesById[3] = "DEFAULT"] = 3;
                                values[valuesById[4] = "SCALING"] = 4;
                                return values;
                            })();
    
                            return SimulationModificationMethodEnum;
                        })();
    
                        enums.SimulationTypeEnum = (function() {
    
                            /**
                             * Properties of a SimulationTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISimulationTypeEnum
                             */
    
                            /**
                             * Constructs a new SimulationTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SimulationTypeEnum.
                             * @implements ISimulationTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISimulationTypeEnum=} [p] Properties to set
                             */
                            function SimulationTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SimulationTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISimulationTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SimulationTypeEnum} SimulationTypeEnum instance
                             */
                            SimulationTypeEnum.create = function create(properties) {
                                return new SimulationTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SimulationTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SimulationTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISimulationTypeEnum} m SimulationTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SimulationTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SimulationTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SimulationTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISimulationTypeEnum} message SimulationTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SimulationTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SimulationTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SimulationTypeEnum} SimulationTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SimulationTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SimulationTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SimulationTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SimulationTypeEnum} SimulationTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SimulationTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SimulationTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SimulationTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SimulationTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SimulationTypeEnum} SimulationTypeEnum
                             */
                            SimulationTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SimulationTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SimulationTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SimulationTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SimulationTypeEnum} m SimulationTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SimulationTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SimulationTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SimulationTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SimulationTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SimulationType enum.
                             * @name google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CPC_BID=2 CPC_BID value
                             * @property {number} CPV_BID=3 CPV_BID value
                             * @property {number} TARGET_CPA=4 TARGET_CPA value
                             * @property {number} BID_MODIFIER=5 BID_MODIFIER value
                             * @property {number} TARGET_ROAS=6 TARGET_ROAS value
                             * @property {number} PERCENT_CPC_BID=7 PERCENT_CPC_BID value
                             * @property {number} TARGET_IMPRESSION_SHARE=8 TARGET_IMPRESSION_SHARE value
                             * @property {number} BUDGET=9 BUDGET value
                             */
                            SimulationTypeEnum.SimulationType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CPC_BID"] = 2;
                                values[valuesById[3] = "CPV_BID"] = 3;
                                values[valuesById[4] = "TARGET_CPA"] = 4;
                                values[valuesById[5] = "BID_MODIFIER"] = 5;
                                values[valuesById[6] = "TARGET_ROAS"] = 6;
                                values[valuesById[7] = "PERCENT_CPC_BID"] = 7;
                                values[valuesById[8] = "TARGET_IMPRESSION_SHARE"] = 8;
                                values[valuesById[9] = "BUDGET"] = 9;
                                return values;
                            })();
    
                            return SimulationTypeEnum;
                        })();
    
                        enums.SitelinkPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a SitelinkPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISitelinkPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new SitelinkPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SitelinkPlaceholderFieldEnum.
                             * @implements ISitelinkPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISitelinkPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function SitelinkPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SitelinkPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISitelinkPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum} SitelinkPlaceholderFieldEnum instance
                             */
                            SitelinkPlaceholderFieldEnum.create = function create(properties) {
                                return new SitelinkPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SitelinkPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISitelinkPlaceholderFieldEnum} m SitelinkPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SitelinkPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SitelinkPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISitelinkPlaceholderFieldEnum} message SitelinkPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SitelinkPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SitelinkPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum} SitelinkPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SitelinkPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SitelinkPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum} SitelinkPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SitelinkPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SitelinkPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SitelinkPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SitelinkPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum} SitelinkPlaceholderFieldEnum
                             */
                            SitelinkPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a SitelinkPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum} m SitelinkPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SitelinkPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SitelinkPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SitelinkPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SitelinkPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TEXT=2 TEXT value
                             * @property {number} LINE_1=3 LINE_1 value
                             * @property {number} LINE_2=4 LINE_2 value
                             * @property {number} FINAL_URLS=5 FINAL_URLS value
                             * @property {number} FINAL_MOBILE_URLS=6 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=7 TRACKING_URL value
                             * @property {number} FINAL_URL_SUFFIX=8 FINAL_URL_SUFFIX value
                             */
                            SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TEXT"] = 2;
                                values[valuesById[3] = "LINE_1"] = 3;
                                values[valuesById[4] = "LINE_2"] = 4;
                                values[valuesById[5] = "FINAL_URLS"] = 5;
                                values[valuesById[6] = "FINAL_MOBILE_URLS"] = 6;
                                values[valuesById[7] = "TRACKING_URL"] = 7;
                                values[valuesById[8] = "FINAL_URL_SUFFIX"] = 8;
                                return values;
                            })();
    
                            return SitelinkPlaceholderFieldEnum;
                        })();
    
                        enums.SpendingLimitTypeEnum = (function() {
    
                            /**
                             * Properties of a SpendingLimitTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISpendingLimitTypeEnum
                             */
    
                            /**
                             * Constructs a new SpendingLimitTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SpendingLimitTypeEnum.
                             * @implements ISpendingLimitTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISpendingLimitTypeEnum=} [p] Properties to set
                             */
                            function SpendingLimitTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SpendingLimitTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISpendingLimitTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SpendingLimitTypeEnum} SpendingLimitTypeEnum instance
                             */
                            SpendingLimitTypeEnum.create = function create(properties) {
                                return new SpendingLimitTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SpendingLimitTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SpendingLimitTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISpendingLimitTypeEnum} m SpendingLimitTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SpendingLimitTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SpendingLimitTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SpendingLimitTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISpendingLimitTypeEnum} message SpendingLimitTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SpendingLimitTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SpendingLimitTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SpendingLimitTypeEnum} SpendingLimitTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SpendingLimitTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SpendingLimitTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SpendingLimitTypeEnum} SpendingLimitTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SpendingLimitTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SpendingLimitTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SpendingLimitTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SpendingLimitTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SpendingLimitTypeEnum} SpendingLimitTypeEnum
                             */
                            SpendingLimitTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a SpendingLimitTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SpendingLimitTypeEnum} m SpendingLimitTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SpendingLimitTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SpendingLimitTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SpendingLimitTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SpendingLimitTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SpendingLimitType enum.
                             * @name google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INFINITE=2 INFINITE value
                             */
                            SpendingLimitTypeEnum.SpendingLimitType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INFINITE"] = 2;
                                return values;
                            })();
    
                            return SpendingLimitTypeEnum;
                        })();
    
                        enums.StructuredSnippetPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a StructuredSnippetPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IStructuredSnippetPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new StructuredSnippetPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a StructuredSnippetPlaceholderFieldEnum.
                             * @implements IStructuredSnippetPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IStructuredSnippetPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function StructuredSnippetPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new StructuredSnippetPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IStructuredSnippetPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum} StructuredSnippetPlaceholderFieldEnum instance
                             */
                            StructuredSnippetPlaceholderFieldEnum.create = function create(properties) {
                                return new StructuredSnippetPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified StructuredSnippetPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IStructuredSnippetPlaceholderFieldEnum} m StructuredSnippetPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StructuredSnippetPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified StructuredSnippetPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IStructuredSnippetPlaceholderFieldEnum} message StructuredSnippetPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StructuredSnippetPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StructuredSnippetPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum} StructuredSnippetPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StructuredSnippetPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StructuredSnippetPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum} StructuredSnippetPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StructuredSnippetPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StructuredSnippetPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StructuredSnippetPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a StructuredSnippetPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum} StructuredSnippetPlaceholderFieldEnum
                             */
                            StructuredSnippetPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a StructuredSnippetPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum} m StructuredSnippetPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StructuredSnippetPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this StructuredSnippetPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StructuredSnippetPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * StructuredSnippetPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} HEADER=2 HEADER value
                             * @property {number} SNIPPETS=3 SNIPPETS value
                             */
                            StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "HEADER"] = 2;
                                values[valuesById[3] = "SNIPPETS"] = 3;
                                return values;
                            })();
    
                            return StructuredSnippetPlaceholderFieldEnum;
                        })();
    
                        enums.SummaryRowSettingEnum = (function() {
    
                            /**
                             * Properties of a SummaryRowSettingEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISummaryRowSettingEnum
                             */
    
                            /**
                             * Constructs a new SummaryRowSettingEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SummaryRowSettingEnum.
                             * @implements ISummaryRowSettingEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISummaryRowSettingEnum=} [p] Properties to set
                             */
                            function SummaryRowSettingEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SummaryRowSettingEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISummaryRowSettingEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SummaryRowSettingEnum} SummaryRowSettingEnum instance
                             */
                            SummaryRowSettingEnum.create = function create(properties) {
                                return new SummaryRowSettingEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SummaryRowSettingEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SummaryRowSettingEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISummaryRowSettingEnum} m SummaryRowSettingEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SummaryRowSettingEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SummaryRowSettingEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SummaryRowSettingEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISummaryRowSettingEnum} message SummaryRowSettingEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SummaryRowSettingEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SummaryRowSettingEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SummaryRowSettingEnum} SummaryRowSettingEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SummaryRowSettingEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SummaryRowSettingEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SummaryRowSettingEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SummaryRowSettingEnum} SummaryRowSettingEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SummaryRowSettingEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SummaryRowSettingEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SummaryRowSettingEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SummaryRowSettingEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SummaryRowSettingEnum} SummaryRowSettingEnum
                             */
                            SummaryRowSettingEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SummaryRowSettingEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SummaryRowSettingEnum();
                            };
    
                            /**
                             * Creates a plain object from a SummaryRowSettingEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SummaryRowSettingEnum} m SummaryRowSettingEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SummaryRowSettingEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SummaryRowSettingEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SummaryRowSettingEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SummaryRowSettingEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SummaryRowSetting enum.
                             * @name google.ads.googleads.v11.enums.SummaryRowSettingEnum.SummaryRowSetting
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NO_SUMMARY_ROW=2 NO_SUMMARY_ROW value
                             * @property {number} SUMMARY_ROW_WITH_RESULTS=3 SUMMARY_ROW_WITH_RESULTS value
                             * @property {number} SUMMARY_ROW_ONLY=4 SUMMARY_ROW_ONLY value
                             */
                            SummaryRowSettingEnum.SummaryRowSetting = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NO_SUMMARY_ROW"] = 2;
                                values[valuesById[3] = "SUMMARY_ROW_WITH_RESULTS"] = 3;
                                values[valuesById[4] = "SUMMARY_ROW_ONLY"] = 4;
                                return values;
                            })();
    
                            return SummaryRowSettingEnum;
                        })();
    
                        enums.SystemManagedResourceSourceEnum = (function() {
    
                            /**
                             * Properties of a SystemManagedResourceSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ISystemManagedResourceSourceEnum
                             */
    
                            /**
                             * Constructs a new SystemManagedResourceSourceEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a SystemManagedResourceSourceEnum.
                             * @implements ISystemManagedResourceSourceEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ISystemManagedResourceSourceEnum=} [p] Properties to set
                             */
                            function SystemManagedResourceSourceEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SystemManagedResourceSourceEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISystemManagedResourceSourceEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum} SystemManagedResourceSourceEnum instance
                             */
                            SystemManagedResourceSourceEnum.create = function create(properties) {
                                return new SystemManagedResourceSourceEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SystemManagedResourceSourceEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISystemManagedResourceSourceEnum} m SystemManagedResourceSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SystemManagedResourceSourceEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SystemManagedResourceSourceEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ISystemManagedResourceSourceEnum} message SystemManagedResourceSourceEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SystemManagedResourceSourceEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SystemManagedResourceSourceEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum} SystemManagedResourceSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SystemManagedResourceSourceEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SystemManagedResourceSourceEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum} SystemManagedResourceSourceEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SystemManagedResourceSourceEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SystemManagedResourceSourceEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SystemManagedResourceSourceEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SystemManagedResourceSourceEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum} SystemManagedResourceSourceEnum
                             */
                            SystemManagedResourceSourceEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum();
                            };
    
                            /**
                             * Creates a plain object from a SystemManagedResourceSourceEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum} m SystemManagedResourceSourceEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SystemManagedResourceSourceEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SystemManagedResourceSourceEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SystemManagedResourceSourceEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SystemManagedResourceSource enum.
                             * @name google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum.SystemManagedResourceSource
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_VARIATIONS=2 AD_VARIATIONS value
                             */
                            SystemManagedResourceSourceEnum.SystemManagedResourceSource = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_VARIATIONS"] = 2;
                                return values;
                            })();
    
                            return SystemManagedResourceSourceEnum;
                        })();
    
                        enums.TargetCpaOptInRecommendationGoalEnum = (function() {
    
                            /**
                             * Properties of a TargetCpaOptInRecommendationGoalEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ITargetCpaOptInRecommendationGoalEnum
                             */
    
                            /**
                             * Constructs a new TargetCpaOptInRecommendationGoalEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a TargetCpaOptInRecommendationGoalEnum.
                             * @implements ITargetCpaOptInRecommendationGoalEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ITargetCpaOptInRecommendationGoalEnum=} [p] Properties to set
                             */
                            function TargetCpaOptInRecommendationGoalEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TargetCpaOptInRecommendationGoalEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetCpaOptInRecommendationGoalEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum} TargetCpaOptInRecommendationGoalEnum instance
                             */
                            TargetCpaOptInRecommendationGoalEnum.create = function create(properties) {
                                return new TargetCpaOptInRecommendationGoalEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TargetCpaOptInRecommendationGoalEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetCpaOptInRecommendationGoalEnum} m TargetCpaOptInRecommendationGoalEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpaOptInRecommendationGoalEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TargetCpaOptInRecommendationGoalEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITargetCpaOptInRecommendationGoalEnum} message TargetCpaOptInRecommendationGoalEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TargetCpaOptInRecommendationGoalEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TargetCpaOptInRecommendationGoalEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum} TargetCpaOptInRecommendationGoalEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpaOptInRecommendationGoalEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TargetCpaOptInRecommendationGoalEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum} TargetCpaOptInRecommendationGoalEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TargetCpaOptInRecommendationGoalEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TargetCpaOptInRecommendationGoalEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TargetCpaOptInRecommendationGoalEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TargetCpaOptInRecommendationGoalEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum} TargetCpaOptInRecommendationGoalEnum
                             */
                            TargetCpaOptInRecommendationGoalEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum();
                            };
    
                            /**
                             * Creates a plain object from a TargetCpaOptInRecommendationGoalEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum} m TargetCpaOptInRecommendationGoalEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TargetCpaOptInRecommendationGoalEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TargetCpaOptInRecommendationGoalEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TargetCpaOptInRecommendationGoalEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TargetCpaOptInRecommendationGoal enum.
                             * @name google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SAME_COST=2 SAME_COST value
                             * @property {number} SAME_CONVERSIONS=3 SAME_CONVERSIONS value
                             * @property {number} SAME_CPA=4 SAME_CPA value
                             * @property {number} CLOSEST_CPA=5 CLOSEST_CPA value
                             */
                            TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SAME_COST"] = 2;
                                values[valuesById[3] = "SAME_CONVERSIONS"] = 3;
                                values[valuesById[4] = "SAME_CPA"] = 4;
                                values[valuesById[5] = "CLOSEST_CPA"] = 5;
                                return values;
                            })();
    
                            return TargetCpaOptInRecommendationGoalEnum;
                        })();
    
                        enums.TimeTypeEnum = (function() {
    
                            /**
                             * Properties of a TimeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ITimeTypeEnum
                             */
    
                            /**
                             * Constructs a new TimeTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a TimeTypeEnum.
                             * @implements ITimeTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ITimeTypeEnum=} [p] Properties to set
                             */
                            function TimeTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TimeTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITimeTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.TimeTypeEnum} TimeTypeEnum instance
                             */
                            TimeTypeEnum.create = function create(properties) {
                                return new TimeTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TimeTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.TimeTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITimeTypeEnum} m TimeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TimeTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TimeTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.TimeTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITimeTypeEnum} message TimeTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TimeTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TimeTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.TimeTypeEnum} TimeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TimeTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.TimeTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TimeTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.TimeTypeEnum} TimeTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TimeTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TimeTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TimeTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TimeTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.TimeTypeEnum} TimeTypeEnum
                             */
                            TimeTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.TimeTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.TimeTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a TimeTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.TimeTypeEnum} m TimeTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TimeTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TimeTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.TimeTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TimeTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TimeType enum.
                             * @name google.ads.googleads.v11.enums.TimeTypeEnum.TimeType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOW=2 NOW value
                             * @property {number} FOREVER=3 FOREVER value
                             */
                            TimeTypeEnum.TimeType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOW"] = 2;
                                values[valuesById[3] = "FOREVER"] = 3;
                                return values;
                            })();
    
                            return TimeTypeEnum;
                        })();
    
                        enums.TravelPlaceholderFieldEnum = (function() {
    
                            /**
                             * Properties of a TravelPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface ITravelPlaceholderFieldEnum
                             */
    
                            /**
                             * Constructs a new TravelPlaceholderFieldEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a TravelPlaceholderFieldEnum.
                             * @implements ITravelPlaceholderFieldEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.ITravelPlaceholderFieldEnum=} [p] Properties to set
                             */
                            function TravelPlaceholderFieldEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TravelPlaceholderFieldEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITravelPlaceholderFieldEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum} TravelPlaceholderFieldEnum instance
                             */
                            TravelPlaceholderFieldEnum.create = function create(properties) {
                                return new TravelPlaceholderFieldEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TravelPlaceholderFieldEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITravelPlaceholderFieldEnum} m TravelPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TravelPlaceholderFieldEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TravelPlaceholderFieldEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ITravelPlaceholderFieldEnum} message TravelPlaceholderFieldEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TravelPlaceholderFieldEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TravelPlaceholderFieldEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum} TravelPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TravelPlaceholderFieldEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TravelPlaceholderFieldEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum} TravelPlaceholderFieldEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TravelPlaceholderFieldEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TravelPlaceholderFieldEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TravelPlaceholderFieldEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TravelPlaceholderFieldEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum} TravelPlaceholderFieldEnum
                             */
                            TravelPlaceholderFieldEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum();
                            };
    
                            /**
                             * Creates a plain object from a TravelPlaceholderFieldEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum} m TravelPlaceholderFieldEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TravelPlaceholderFieldEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TravelPlaceholderFieldEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TravelPlaceholderFieldEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TravelPlaceholderField enum.
                             * @name google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum.TravelPlaceholderField
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DESTINATION_ID=2 DESTINATION_ID value
                             * @property {number} ORIGIN_ID=3 ORIGIN_ID value
                             * @property {number} TITLE=4 TITLE value
                             * @property {number} DESTINATION_NAME=5 DESTINATION_NAME value
                             * @property {number} ORIGIN_NAME=6 ORIGIN_NAME value
                             * @property {number} PRICE=7 PRICE value
                             * @property {number} FORMATTED_PRICE=8 FORMATTED_PRICE value
                             * @property {number} SALE_PRICE=9 SALE_PRICE value
                             * @property {number} FORMATTED_SALE_PRICE=10 FORMATTED_SALE_PRICE value
                             * @property {number} IMAGE_URL=11 IMAGE_URL value
                             * @property {number} CATEGORY=12 CATEGORY value
                             * @property {number} CONTEXTUAL_KEYWORDS=13 CONTEXTUAL_KEYWORDS value
                             * @property {number} DESTINATION_ADDRESS=14 DESTINATION_ADDRESS value
                             * @property {number} FINAL_URL=15 FINAL_URL value
                             * @property {number} FINAL_MOBILE_URLS=16 FINAL_MOBILE_URLS value
                             * @property {number} TRACKING_URL=17 TRACKING_URL value
                             * @property {number} ANDROID_APP_LINK=18 ANDROID_APP_LINK value
                             * @property {number} SIMILAR_DESTINATION_IDS=19 SIMILAR_DESTINATION_IDS value
                             * @property {number} IOS_APP_LINK=20 IOS_APP_LINK value
                             * @property {number} IOS_APP_STORE_ID=21 IOS_APP_STORE_ID value
                             */
                            TravelPlaceholderFieldEnum.TravelPlaceholderField = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DESTINATION_ID"] = 2;
                                values[valuesById[3] = "ORIGIN_ID"] = 3;
                                values[valuesById[4] = "TITLE"] = 4;
                                values[valuesById[5] = "DESTINATION_NAME"] = 5;
                                values[valuesById[6] = "ORIGIN_NAME"] = 6;
                                values[valuesById[7] = "PRICE"] = 7;
                                values[valuesById[8] = "FORMATTED_PRICE"] = 8;
                                values[valuesById[9] = "SALE_PRICE"] = 9;
                                values[valuesById[10] = "FORMATTED_SALE_PRICE"] = 10;
                                values[valuesById[11] = "IMAGE_URL"] = 11;
                                values[valuesById[12] = "CATEGORY"] = 12;
                                values[valuesById[13] = "CONTEXTUAL_KEYWORDS"] = 13;
                                values[valuesById[14] = "DESTINATION_ADDRESS"] = 14;
                                values[valuesById[15] = "FINAL_URL"] = 15;
                                values[valuesById[16] = "FINAL_MOBILE_URLS"] = 16;
                                values[valuesById[17] = "TRACKING_URL"] = 17;
                                values[valuesById[18] = "ANDROID_APP_LINK"] = 18;
                                values[valuesById[19] = "SIMILAR_DESTINATION_IDS"] = 19;
                                values[valuesById[20] = "IOS_APP_LINK"] = 20;
                                values[valuesById[21] = "IOS_APP_STORE_ID"] = 21;
                                return values;
                            })();
    
                            return TravelPlaceholderFieldEnum;
                        })();
    
                        enums.UserInterestTaxonomyTypeEnum = (function() {
    
                            /**
                             * Properties of a UserInterestTaxonomyTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserInterestTaxonomyTypeEnum
                             */
    
                            /**
                             * Constructs a new UserInterestTaxonomyTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserInterestTaxonomyTypeEnum.
                             * @implements IUserInterestTaxonomyTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserInterestTaxonomyTypeEnum=} [p] Properties to set
                             */
                            function UserInterestTaxonomyTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserInterestTaxonomyTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserInterestTaxonomyTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum} UserInterestTaxonomyTypeEnum instance
                             */
                            UserInterestTaxonomyTypeEnum.create = function create(properties) {
                                return new UserInterestTaxonomyTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserInterestTaxonomyTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserInterestTaxonomyTypeEnum} m UserInterestTaxonomyTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterestTaxonomyTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserInterestTaxonomyTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserInterestTaxonomyTypeEnum} message UserInterestTaxonomyTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterestTaxonomyTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserInterestTaxonomyTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum} UserInterestTaxonomyTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterestTaxonomyTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserInterestTaxonomyTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum} UserInterestTaxonomyTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterestTaxonomyTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserInterestTaxonomyTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserInterestTaxonomyTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserInterestTaxonomyTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum} UserInterestTaxonomyTypeEnum
                             */
                            UserInterestTaxonomyTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserInterestTaxonomyTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum} m UserInterestTaxonomyTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserInterestTaxonomyTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserInterestTaxonomyTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserInterestTaxonomyTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserInterestTaxonomyType enum.
                             * @name google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AFFINITY=2 AFFINITY value
                             * @property {number} IN_MARKET=3 IN_MARKET value
                             * @property {number} MOBILE_APP_INSTALL_USER=4 MOBILE_APP_INSTALL_USER value
                             * @property {number} VERTICAL_GEO=5 VERTICAL_GEO value
                             * @property {number} NEW_SMART_PHONE_USER=6 NEW_SMART_PHONE_USER value
                             */
                            UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AFFINITY"] = 2;
                                values[valuesById[3] = "IN_MARKET"] = 3;
                                values[valuesById[4] = "MOBILE_APP_INSTALL_USER"] = 4;
                                values[valuesById[5] = "VERTICAL_GEO"] = 5;
                                values[valuesById[6] = "NEW_SMART_PHONE_USER"] = 6;
                                return values;
                            })();
    
                            return UserInterestTaxonomyTypeEnum;
                        })();
    
                        enums.UserListAccessStatusEnum = (function() {
    
                            /**
                             * Properties of a UserListAccessStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListAccessStatusEnum
                             */
    
                            /**
                             * Constructs a new UserListAccessStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListAccessStatusEnum.
                             * @implements IUserListAccessStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListAccessStatusEnum=} [p] Properties to set
                             */
                            function UserListAccessStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListAccessStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListAccessStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListAccessStatusEnum} UserListAccessStatusEnum instance
                             */
                            UserListAccessStatusEnum.create = function create(properties) {
                                return new UserListAccessStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListAccessStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListAccessStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListAccessStatusEnum} m UserListAccessStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListAccessStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListAccessStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListAccessStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListAccessStatusEnum} message UserListAccessStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListAccessStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListAccessStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListAccessStatusEnum} UserListAccessStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListAccessStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListAccessStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListAccessStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListAccessStatusEnum} UserListAccessStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListAccessStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListAccessStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListAccessStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListAccessStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListAccessStatusEnum} UserListAccessStatusEnum
                             */
                            UserListAccessStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListAccessStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListAccessStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListAccessStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListAccessStatusEnum} m UserListAccessStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListAccessStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListAccessStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListAccessStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListAccessStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListAccessStatus enum.
                             * @name google.ads.googleads.v11.enums.UserListAccessStatusEnum.UserListAccessStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENABLED=2 ENABLED value
                             * @property {number} DISABLED=3 DISABLED value
                             */
                            UserListAccessStatusEnum.UserListAccessStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ENABLED"] = 2;
                                values[valuesById[3] = "DISABLED"] = 3;
                                return values;
                            })();
    
                            return UserListAccessStatusEnum;
                        })();
    
                        enums.UserListClosingReasonEnum = (function() {
    
                            /**
                             * Properties of a UserListClosingReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListClosingReasonEnum
                             */
    
                            /**
                             * Constructs a new UserListClosingReasonEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListClosingReasonEnum.
                             * @implements IUserListClosingReasonEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListClosingReasonEnum=} [p] Properties to set
                             */
                            function UserListClosingReasonEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListClosingReasonEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListClosingReasonEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListClosingReasonEnum} UserListClosingReasonEnum instance
                             */
                            UserListClosingReasonEnum.create = function create(properties) {
                                return new UserListClosingReasonEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListClosingReasonEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListClosingReasonEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListClosingReasonEnum} m UserListClosingReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListClosingReasonEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListClosingReasonEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListClosingReasonEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListClosingReasonEnum} message UserListClosingReasonEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListClosingReasonEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListClosingReasonEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListClosingReasonEnum} UserListClosingReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListClosingReasonEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListClosingReasonEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListClosingReasonEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListClosingReasonEnum} UserListClosingReasonEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListClosingReasonEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListClosingReasonEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListClosingReasonEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListClosingReasonEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListClosingReasonEnum} UserListClosingReasonEnum
                             */
                            UserListClosingReasonEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListClosingReasonEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListClosingReasonEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListClosingReasonEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListClosingReasonEnum} m UserListClosingReasonEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListClosingReasonEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListClosingReasonEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListClosingReasonEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListClosingReasonEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListClosingReason enum.
                             * @name google.ads.googleads.v11.enums.UserListClosingReasonEnum.UserListClosingReason
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNUSED=2 UNUSED value
                             */
                            UserListClosingReasonEnum.UserListClosingReason = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNUSED"] = 2;
                                return values;
                            })();
    
                            return UserListClosingReasonEnum;
                        })();
    
                        enums.UserListMembershipStatusEnum = (function() {
    
                            /**
                             * Properties of a UserListMembershipStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListMembershipStatusEnum
                             */
    
                            /**
                             * Constructs a new UserListMembershipStatusEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListMembershipStatusEnum.
                             * @implements IUserListMembershipStatusEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListMembershipStatusEnum=} [p] Properties to set
                             */
                            function UserListMembershipStatusEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListMembershipStatusEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListMembershipStatusEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListMembershipStatusEnum} UserListMembershipStatusEnum instance
                             */
                            UserListMembershipStatusEnum.create = function create(properties) {
                                return new UserListMembershipStatusEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListMembershipStatusEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListMembershipStatusEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListMembershipStatusEnum} m UserListMembershipStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListMembershipStatusEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListMembershipStatusEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListMembershipStatusEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListMembershipStatusEnum} message UserListMembershipStatusEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListMembershipStatusEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListMembershipStatusEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListMembershipStatusEnum} UserListMembershipStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListMembershipStatusEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListMembershipStatusEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListMembershipStatusEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListMembershipStatusEnum} UserListMembershipStatusEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListMembershipStatusEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListMembershipStatusEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListMembershipStatusEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListMembershipStatusEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListMembershipStatusEnum} UserListMembershipStatusEnum
                             */
                            UserListMembershipStatusEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListMembershipStatusEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListMembershipStatusEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListMembershipStatusEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListMembershipStatusEnum} m UserListMembershipStatusEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListMembershipStatusEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListMembershipStatusEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListMembershipStatusEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListMembershipStatusEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListMembershipStatus enum.
                             * @name google.ads.googleads.v11.enums.UserListMembershipStatusEnum.UserListMembershipStatus
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OPEN=2 OPEN value
                             * @property {number} CLOSED=3 CLOSED value
                             */
                            UserListMembershipStatusEnum.UserListMembershipStatus = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OPEN"] = 2;
                                values[valuesById[3] = "CLOSED"] = 3;
                                return values;
                            })();
    
                            return UserListMembershipStatusEnum;
                        })();
    
                        enums.UserListSizeRangeEnum = (function() {
    
                            /**
                             * Properties of a UserListSizeRangeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListSizeRangeEnum
                             */
    
                            /**
                             * Constructs a new UserListSizeRangeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListSizeRangeEnum.
                             * @implements IUserListSizeRangeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListSizeRangeEnum=} [p] Properties to set
                             */
                            function UserListSizeRangeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListSizeRangeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListSizeRangeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListSizeRangeEnum} UserListSizeRangeEnum instance
                             */
                            UserListSizeRangeEnum.create = function create(properties) {
                                return new UserListSizeRangeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListSizeRangeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListSizeRangeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListSizeRangeEnum} m UserListSizeRangeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListSizeRangeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListSizeRangeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListSizeRangeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListSizeRangeEnum} message UserListSizeRangeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListSizeRangeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListSizeRangeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListSizeRangeEnum} UserListSizeRangeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListSizeRangeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListSizeRangeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListSizeRangeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListSizeRangeEnum} UserListSizeRangeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListSizeRangeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListSizeRangeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListSizeRangeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListSizeRangeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListSizeRangeEnum} UserListSizeRangeEnum
                             */
                            UserListSizeRangeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListSizeRangeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListSizeRangeEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListSizeRangeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListSizeRangeEnum} m UserListSizeRangeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListSizeRangeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListSizeRangeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListSizeRangeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListSizeRangeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListSizeRange enum.
                             * @name google.ads.googleads.v11.enums.UserListSizeRangeEnum.UserListSizeRange
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LESS_THAN_FIVE_HUNDRED=2 LESS_THAN_FIVE_HUNDRED value
                             * @property {number} LESS_THAN_ONE_THOUSAND=3 LESS_THAN_ONE_THOUSAND value
                             * @property {number} ONE_THOUSAND_TO_TEN_THOUSAND=4 ONE_THOUSAND_TO_TEN_THOUSAND value
                             * @property {number} TEN_THOUSAND_TO_FIFTY_THOUSAND=5 TEN_THOUSAND_TO_FIFTY_THOUSAND value
                             * @property {number} FIFTY_THOUSAND_TO_ONE_HUNDRED_THOUSAND=6 FIFTY_THOUSAND_TO_ONE_HUNDRED_THOUSAND value
                             * @property {number} ONE_HUNDRED_THOUSAND_TO_THREE_HUNDRED_THOUSAND=7 ONE_HUNDRED_THOUSAND_TO_THREE_HUNDRED_THOUSAND value
                             * @property {number} THREE_HUNDRED_THOUSAND_TO_FIVE_HUNDRED_THOUSAND=8 THREE_HUNDRED_THOUSAND_TO_FIVE_HUNDRED_THOUSAND value
                             * @property {number} FIVE_HUNDRED_THOUSAND_TO_ONE_MILLION=9 FIVE_HUNDRED_THOUSAND_TO_ONE_MILLION value
                             * @property {number} ONE_MILLION_TO_TWO_MILLION=10 ONE_MILLION_TO_TWO_MILLION value
                             * @property {number} TWO_MILLION_TO_THREE_MILLION=11 TWO_MILLION_TO_THREE_MILLION value
                             * @property {number} THREE_MILLION_TO_FIVE_MILLION=12 THREE_MILLION_TO_FIVE_MILLION value
                             * @property {number} FIVE_MILLION_TO_TEN_MILLION=13 FIVE_MILLION_TO_TEN_MILLION value
                             * @property {number} TEN_MILLION_TO_TWENTY_MILLION=14 TEN_MILLION_TO_TWENTY_MILLION value
                             * @property {number} TWENTY_MILLION_TO_THIRTY_MILLION=15 TWENTY_MILLION_TO_THIRTY_MILLION value
                             * @property {number} THIRTY_MILLION_TO_FIFTY_MILLION=16 THIRTY_MILLION_TO_FIFTY_MILLION value
                             * @property {number} OVER_FIFTY_MILLION=17 OVER_FIFTY_MILLION value
                             */
                            UserListSizeRangeEnum.UserListSizeRange = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LESS_THAN_FIVE_HUNDRED"] = 2;
                                values[valuesById[3] = "LESS_THAN_ONE_THOUSAND"] = 3;
                                values[valuesById[4] = "ONE_THOUSAND_TO_TEN_THOUSAND"] = 4;
                                values[valuesById[5] = "TEN_THOUSAND_TO_FIFTY_THOUSAND"] = 5;
                                values[valuesById[6] = "FIFTY_THOUSAND_TO_ONE_HUNDRED_THOUSAND"] = 6;
                                values[valuesById[7] = "ONE_HUNDRED_THOUSAND_TO_THREE_HUNDRED_THOUSAND"] = 7;
                                values[valuesById[8] = "THREE_HUNDRED_THOUSAND_TO_FIVE_HUNDRED_THOUSAND"] = 8;
                                values[valuesById[9] = "FIVE_HUNDRED_THOUSAND_TO_ONE_MILLION"] = 9;
                                values[valuesById[10] = "ONE_MILLION_TO_TWO_MILLION"] = 10;
                                values[valuesById[11] = "TWO_MILLION_TO_THREE_MILLION"] = 11;
                                values[valuesById[12] = "THREE_MILLION_TO_FIVE_MILLION"] = 12;
                                values[valuesById[13] = "FIVE_MILLION_TO_TEN_MILLION"] = 13;
                                values[valuesById[14] = "TEN_MILLION_TO_TWENTY_MILLION"] = 14;
                                values[valuesById[15] = "TWENTY_MILLION_TO_THIRTY_MILLION"] = 15;
                                values[valuesById[16] = "THIRTY_MILLION_TO_FIFTY_MILLION"] = 16;
                                values[valuesById[17] = "OVER_FIFTY_MILLION"] = 17;
                                return values;
                            })();
    
                            return UserListSizeRangeEnum;
                        })();
    
                        enums.UserListTypeEnum = (function() {
    
                            /**
                             * Properties of a UserListTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IUserListTypeEnum
                             */
    
                            /**
                             * Constructs a new UserListTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a UserListTypeEnum.
                             * @implements IUserListTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IUserListTypeEnum=} [p] Properties to set
                             */
                            function UserListTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.UserListTypeEnum} UserListTypeEnum instance
                             */
                            UserListTypeEnum.create = function create(properties) {
                                return new UserListTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.UserListTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListTypeEnum} m UserListTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.UserListTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IUserListTypeEnum} message UserListTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.UserListTypeEnum} UserListTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.UserListTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.UserListTypeEnum} UserListTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.UserListTypeEnum} UserListTypeEnum
                             */
                            UserListTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.UserListTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.UserListTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.UserListTypeEnum} m UserListTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.UserListTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListType enum.
                             * @name google.ads.googleads.v11.enums.UserListTypeEnum.UserListType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} REMARKETING=2 REMARKETING value
                             * @property {number} LOGICAL=3 LOGICAL value
                             * @property {number} EXTERNAL_REMARKETING=4 EXTERNAL_REMARKETING value
                             * @property {number} RULE_BASED=5 RULE_BASED value
                             * @property {number} SIMILAR=6 SIMILAR value
                             * @property {number} CRM_BASED=7 CRM_BASED value
                             */
                            UserListTypeEnum.UserListType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "REMARKETING"] = 2;
                                values[valuesById[3] = "LOGICAL"] = 3;
                                values[valuesById[4] = "EXTERNAL_REMARKETING"] = 4;
                                values[valuesById[5] = "RULE_BASED"] = 5;
                                values[valuesById[6] = "SIMILAR"] = 6;
                                values[valuesById[7] = "CRM_BASED"] = 7;
                                return values;
                            })();
    
                            return UserListTypeEnum;
                        })();
    
                        enums.ValueRuleDeviceTypeEnum = (function() {
    
                            /**
                             * Properties of a ValueRuleDeviceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IValueRuleDeviceTypeEnum
                             */
    
                            /**
                             * Constructs a new ValueRuleDeviceTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ValueRuleDeviceTypeEnum.
                             * @implements IValueRuleDeviceTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IValueRuleDeviceTypeEnum=} [p] Properties to set
                             */
                            function ValueRuleDeviceTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ValueRuleDeviceTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleDeviceTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum} ValueRuleDeviceTypeEnum instance
                             */
                            ValueRuleDeviceTypeEnum.create = function create(properties) {
                                return new ValueRuleDeviceTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ValueRuleDeviceTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleDeviceTypeEnum} m ValueRuleDeviceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleDeviceTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ValueRuleDeviceTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleDeviceTypeEnum} message ValueRuleDeviceTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleDeviceTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ValueRuleDeviceTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum} ValueRuleDeviceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleDeviceTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ValueRuleDeviceTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum} ValueRuleDeviceTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleDeviceTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ValueRuleDeviceTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ValueRuleDeviceTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ValueRuleDeviceTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum} ValueRuleDeviceTypeEnum
                             */
                            ValueRuleDeviceTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ValueRuleDeviceTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum} m ValueRuleDeviceTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ValueRuleDeviceTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ValueRuleDeviceTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ValueRuleDeviceTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ValueRuleDeviceType enum.
                             * @name google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum.ValueRuleDeviceType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MOBILE=2 MOBILE value
                             * @property {number} DESKTOP=3 DESKTOP value
                             * @property {number} TABLET=4 TABLET value
                             */
                            ValueRuleDeviceTypeEnum.ValueRuleDeviceType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MOBILE"] = 2;
                                values[valuesById[3] = "DESKTOP"] = 3;
                                values[valuesById[4] = "TABLET"] = 4;
                                return values;
                            })();
    
                            return ValueRuleDeviceTypeEnum;
                        })();
    
                        enums.ValueRuleGeoLocationMatchTypeEnum = (function() {
    
                            /**
                             * Properties of a ValueRuleGeoLocationMatchTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IValueRuleGeoLocationMatchTypeEnum
                             */
    
                            /**
                             * Constructs a new ValueRuleGeoLocationMatchTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ValueRuleGeoLocationMatchTypeEnum.
                             * @implements IValueRuleGeoLocationMatchTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IValueRuleGeoLocationMatchTypeEnum=} [p] Properties to set
                             */
                            function ValueRuleGeoLocationMatchTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ValueRuleGeoLocationMatchTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleGeoLocationMatchTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum} ValueRuleGeoLocationMatchTypeEnum instance
                             */
                            ValueRuleGeoLocationMatchTypeEnum.create = function create(properties) {
                                return new ValueRuleGeoLocationMatchTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ValueRuleGeoLocationMatchTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleGeoLocationMatchTypeEnum} m ValueRuleGeoLocationMatchTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleGeoLocationMatchTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ValueRuleGeoLocationMatchTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleGeoLocationMatchTypeEnum} message ValueRuleGeoLocationMatchTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleGeoLocationMatchTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ValueRuleGeoLocationMatchTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum} ValueRuleGeoLocationMatchTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleGeoLocationMatchTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ValueRuleGeoLocationMatchTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum} ValueRuleGeoLocationMatchTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleGeoLocationMatchTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ValueRuleGeoLocationMatchTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ValueRuleGeoLocationMatchTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ValueRuleGeoLocationMatchTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum} ValueRuleGeoLocationMatchTypeEnum
                             */
                            ValueRuleGeoLocationMatchTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ValueRuleGeoLocationMatchTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum} m ValueRuleGeoLocationMatchTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ValueRuleGeoLocationMatchTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ValueRuleGeoLocationMatchTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ValueRuleGeoLocationMatchTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ValueRuleGeoLocationMatchType enum.
                             * @name google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ANY=2 ANY value
                             * @property {number} LOCATION_OF_PRESENCE=3 LOCATION_OF_PRESENCE value
                             */
                            ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ANY"] = 2;
                                values[valuesById[3] = "LOCATION_OF_PRESENCE"] = 3;
                                return values;
                            })();
    
                            return ValueRuleGeoLocationMatchTypeEnum;
                        })();
    
                        enums.ValueRuleOperationEnum = (function() {
    
                            /**
                             * Properties of a ValueRuleOperationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IValueRuleOperationEnum
                             */
    
                            /**
                             * Constructs a new ValueRuleOperationEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ValueRuleOperationEnum.
                             * @implements IValueRuleOperationEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IValueRuleOperationEnum=} [p] Properties to set
                             */
                            function ValueRuleOperationEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ValueRuleOperationEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleOperationEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ValueRuleOperationEnum} ValueRuleOperationEnum instance
                             */
                            ValueRuleOperationEnum.create = function create(properties) {
                                return new ValueRuleOperationEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ValueRuleOperationEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleOperationEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleOperationEnum} m ValueRuleOperationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleOperationEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ValueRuleOperationEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleOperationEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleOperationEnum} message ValueRuleOperationEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleOperationEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ValueRuleOperationEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ValueRuleOperationEnum} ValueRuleOperationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleOperationEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ValueRuleOperationEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ValueRuleOperationEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ValueRuleOperationEnum} ValueRuleOperationEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleOperationEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ValueRuleOperationEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ValueRuleOperationEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ValueRuleOperationEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ValueRuleOperationEnum} ValueRuleOperationEnum
                             */
                            ValueRuleOperationEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ValueRuleOperationEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ValueRuleOperationEnum();
                            };
    
                            /**
                             * Creates a plain object from a ValueRuleOperationEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ValueRuleOperationEnum} m ValueRuleOperationEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ValueRuleOperationEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ValueRuleOperationEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ValueRuleOperationEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ValueRuleOperationEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ValueRuleOperation enum.
                             * @name google.ads.googleads.v11.enums.ValueRuleOperationEnum.ValueRuleOperation
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADD=2 ADD value
                             * @property {number} MULTIPLY=3 MULTIPLY value
                             * @property {number} SET=4 SET value
                             */
                            ValueRuleOperationEnum.ValueRuleOperation = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADD"] = 2;
                                values[valuesById[3] = "MULTIPLY"] = 3;
                                values[valuesById[4] = "SET"] = 4;
                                return values;
                            })();
    
                            return ValueRuleOperationEnum;
                        })();
    
                        enums.ValueRuleSetAttachmentTypeEnum = (function() {
    
                            /**
                             * Properties of a ValueRuleSetAttachmentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IValueRuleSetAttachmentTypeEnum
                             */
    
                            /**
                             * Constructs a new ValueRuleSetAttachmentTypeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ValueRuleSetAttachmentTypeEnum.
                             * @implements IValueRuleSetAttachmentTypeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetAttachmentTypeEnum=} [p] Properties to set
                             */
                            function ValueRuleSetAttachmentTypeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ValueRuleSetAttachmentTypeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetAttachmentTypeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum} ValueRuleSetAttachmentTypeEnum instance
                             */
                            ValueRuleSetAttachmentTypeEnum.create = function create(properties) {
                                return new ValueRuleSetAttachmentTypeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ValueRuleSetAttachmentTypeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetAttachmentTypeEnum} m ValueRuleSetAttachmentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleSetAttachmentTypeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ValueRuleSetAttachmentTypeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetAttachmentTypeEnum} message ValueRuleSetAttachmentTypeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleSetAttachmentTypeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ValueRuleSetAttachmentTypeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum} ValueRuleSetAttachmentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleSetAttachmentTypeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ValueRuleSetAttachmentTypeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum} ValueRuleSetAttachmentTypeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleSetAttachmentTypeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ValueRuleSetAttachmentTypeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ValueRuleSetAttachmentTypeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ValueRuleSetAttachmentTypeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum} ValueRuleSetAttachmentTypeEnum
                             */
                            ValueRuleSetAttachmentTypeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum();
                            };
    
                            /**
                             * Creates a plain object from a ValueRuleSetAttachmentTypeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum} m ValueRuleSetAttachmentTypeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ValueRuleSetAttachmentTypeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ValueRuleSetAttachmentTypeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ValueRuleSetAttachmentTypeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ValueRuleSetAttachmentType enum.
                             * @name google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CUSTOMER=2 CUSTOMER value
                             * @property {number} CAMPAIGN=3 CAMPAIGN value
                             */
                            ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CUSTOMER"] = 2;
                                values[valuesById[3] = "CAMPAIGN"] = 3;
                                return values;
                            })();
    
                            return ValueRuleSetAttachmentTypeEnum;
                        })();
    
                        enums.ValueRuleSetDimensionEnum = (function() {
    
                            /**
                             * Properties of a ValueRuleSetDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IValueRuleSetDimensionEnum
                             */
    
                            /**
                             * Constructs a new ValueRuleSetDimensionEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a ValueRuleSetDimensionEnum.
                             * @implements IValueRuleSetDimensionEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetDimensionEnum=} [p] Properties to set
                             */
                            function ValueRuleSetDimensionEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ValueRuleSetDimensionEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetDimensionEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum} ValueRuleSetDimensionEnum instance
                             */
                            ValueRuleSetDimensionEnum.create = function create(properties) {
                                return new ValueRuleSetDimensionEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ValueRuleSetDimensionEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetDimensionEnum} m ValueRuleSetDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleSetDimensionEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ValueRuleSetDimensionEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IValueRuleSetDimensionEnum} message ValueRuleSetDimensionEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ValueRuleSetDimensionEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ValueRuleSetDimensionEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum} ValueRuleSetDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleSetDimensionEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ValueRuleSetDimensionEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum} ValueRuleSetDimensionEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ValueRuleSetDimensionEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ValueRuleSetDimensionEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ValueRuleSetDimensionEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ValueRuleSetDimensionEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum} ValueRuleSetDimensionEnum
                             */
                            ValueRuleSetDimensionEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum();
                            };
    
                            /**
                             * Creates a plain object from a ValueRuleSetDimensionEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum} m ValueRuleSetDimensionEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ValueRuleSetDimensionEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ValueRuleSetDimensionEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ValueRuleSetDimensionEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ValueRuleSetDimension enum.
                             * @name google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum.ValueRuleSetDimension
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} GEO_LOCATION=2 GEO_LOCATION value
                             * @property {number} DEVICE=3 DEVICE value
                             * @property {number} AUDIENCE=4 AUDIENCE value
                             * @property {number} NO_CONDITION=5 NO_CONDITION value
                             */
                            ValueRuleSetDimensionEnum.ValueRuleSetDimension = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "GEO_LOCATION"] = 2;
                                values[valuesById[3] = "DEVICE"] = 3;
                                values[valuesById[4] = "AUDIENCE"] = 4;
                                values[valuesById[5] = "NO_CONDITION"] = 5;
                                return values;
                            })();
    
                            return ValueRuleSetDimensionEnum;
                        })();
    
                        enums.VanityPharmaDisplayUrlModeEnum = (function() {
    
                            /**
                             * Properties of a VanityPharmaDisplayUrlModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IVanityPharmaDisplayUrlModeEnum
                             */
    
                            /**
                             * Constructs a new VanityPharmaDisplayUrlModeEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a VanityPharmaDisplayUrlModeEnum.
                             * @implements IVanityPharmaDisplayUrlModeEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaDisplayUrlModeEnum=} [p] Properties to set
                             */
                            function VanityPharmaDisplayUrlModeEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new VanityPharmaDisplayUrlModeEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaDisplayUrlModeEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum} VanityPharmaDisplayUrlModeEnum instance
                             */
                            VanityPharmaDisplayUrlModeEnum.create = function create(properties) {
                                return new VanityPharmaDisplayUrlModeEnum(properties);
                            };
    
                            /**
                             * Encodes the specified VanityPharmaDisplayUrlModeEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaDisplayUrlModeEnum} m VanityPharmaDisplayUrlModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VanityPharmaDisplayUrlModeEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified VanityPharmaDisplayUrlModeEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaDisplayUrlModeEnum} message VanityPharmaDisplayUrlModeEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VanityPharmaDisplayUrlModeEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VanityPharmaDisplayUrlModeEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum} VanityPharmaDisplayUrlModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VanityPharmaDisplayUrlModeEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VanityPharmaDisplayUrlModeEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum} VanityPharmaDisplayUrlModeEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VanityPharmaDisplayUrlModeEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VanityPharmaDisplayUrlModeEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VanityPharmaDisplayUrlModeEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a VanityPharmaDisplayUrlModeEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum} VanityPharmaDisplayUrlModeEnum
                             */
                            VanityPharmaDisplayUrlModeEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum();
                            };
    
                            /**
                             * Creates a plain object from a VanityPharmaDisplayUrlModeEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum} m VanityPharmaDisplayUrlModeEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VanityPharmaDisplayUrlModeEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this VanityPharmaDisplayUrlModeEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VanityPharmaDisplayUrlModeEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * VanityPharmaDisplayUrlMode enum.
                             * @name google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MANUFACTURER_WEBSITE_URL=2 MANUFACTURER_WEBSITE_URL value
                             * @property {number} WEBSITE_DESCRIPTION=3 WEBSITE_DESCRIPTION value
                             */
                            VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MANUFACTURER_WEBSITE_URL"] = 2;
                                values[valuesById[3] = "WEBSITE_DESCRIPTION"] = 3;
                                return values;
                            })();
    
                            return VanityPharmaDisplayUrlModeEnum;
                        })();
    
                        enums.VanityPharmaTextEnum = (function() {
    
                            /**
                             * Properties of a VanityPharmaTextEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @interface IVanityPharmaTextEnum
                             */
    
                            /**
                             * Constructs a new VanityPharmaTextEnum.
                             * @memberof google.ads.googleads.v11.enums
                             * @classdesc Represents a VanityPharmaTextEnum.
                             * @implements IVanityPharmaTextEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaTextEnum=} [p] Properties to set
                             */
                            function VanityPharmaTextEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new VanityPharmaTextEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaTextEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaTextEnum} VanityPharmaTextEnum instance
                             */
                            VanityPharmaTextEnum.create = function create(properties) {
                                return new VanityPharmaTextEnum(properties);
                            };
    
                            /**
                             * Encodes the specified VanityPharmaTextEnum message. Does not implicitly {@link google.ads.googleads.v11.enums.VanityPharmaTextEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaTextEnum} m VanityPharmaTextEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VanityPharmaTextEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified VanityPharmaTextEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.enums.VanityPharmaTextEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.IVanityPharmaTextEnum} message VanityPharmaTextEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            VanityPharmaTextEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a VanityPharmaTextEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaTextEnum} VanityPharmaTextEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VanityPharmaTextEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.enums.VanityPharmaTextEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a VanityPharmaTextEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaTextEnum} VanityPharmaTextEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            VanityPharmaTextEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a VanityPharmaTextEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            VanityPharmaTextEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a VanityPharmaTextEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.enums.VanityPharmaTextEnum} VanityPharmaTextEnum
                             */
                            VanityPharmaTextEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.enums.VanityPharmaTextEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.enums.VanityPharmaTextEnum();
                            };
    
                            /**
                             * Creates a plain object from a VanityPharmaTextEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @static
                             * @param {google.ads.googleads.v11.enums.VanityPharmaTextEnum} m VanityPharmaTextEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            VanityPharmaTextEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this VanityPharmaTextEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.enums.VanityPharmaTextEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            VanityPharmaTextEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * VanityPharmaText enum.
                             * @name google.ads.googleads.v11.enums.VanityPharmaTextEnum.VanityPharmaText
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PRESCRIPTION_TREATMENT_WEBSITE_EN=2 PRESCRIPTION_TREATMENT_WEBSITE_EN value
                             * @property {number} PRESCRIPTION_TREATMENT_WEBSITE_ES=3 PRESCRIPTION_TREATMENT_WEBSITE_ES value
                             * @property {number} PRESCRIPTION_DEVICE_WEBSITE_EN=4 PRESCRIPTION_DEVICE_WEBSITE_EN value
                             * @property {number} PRESCRIPTION_DEVICE_WEBSITE_ES=5 PRESCRIPTION_DEVICE_WEBSITE_ES value
                             * @property {number} MEDICAL_DEVICE_WEBSITE_EN=6 MEDICAL_DEVICE_WEBSITE_EN value
                             * @property {number} MEDICAL_DEVICE_WEBSITE_ES=7 MEDICAL_DEVICE_WEBSITE_ES value
                             * @property {number} PREVENTATIVE_TREATMENT_WEBSITE_EN=8 PREVENTATIVE_TREATMENT_WEBSITE_EN value
                             * @property {number} PREVENTATIVE_TREATMENT_WEBSITE_ES=9 PREVENTATIVE_TREATMENT_WEBSITE_ES value
                             * @property {number} PRESCRIPTION_CONTRACEPTION_WEBSITE_EN=10 PRESCRIPTION_CONTRACEPTION_WEBSITE_EN value
                             * @property {number} PRESCRIPTION_CONTRACEPTION_WEBSITE_ES=11 PRESCRIPTION_CONTRACEPTION_WEBSITE_ES value
                             * @property {number} PRESCRIPTION_VACCINE_WEBSITE_EN=12 PRESCRIPTION_VACCINE_WEBSITE_EN value
                             * @property {number} PRESCRIPTION_VACCINE_WEBSITE_ES=13 PRESCRIPTION_VACCINE_WEBSITE_ES value
                             */
                            VanityPharmaTextEnum.VanityPharmaText = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PRESCRIPTION_TREATMENT_WEBSITE_EN"] = 2;
                                values[valuesById[3] = "PRESCRIPTION_TREATMENT_WEBSITE_ES"] = 3;
                                values[valuesById[4] = "PRESCRIPTION_DEVICE_WEBSITE_EN"] = 4;
                                values[valuesById[5] = "PRESCRIPTION_DEVICE_WEBSITE_ES"] = 5;
                                values[valuesById[6] = "MEDICAL_DEVICE_WEBSITE_EN"] = 6;
                                values[valuesById[7] = "MEDICAL_DEVICE_WEBSITE_ES"] = 7;
                                values[valuesById[8] = "PREVENTATIVE_TREATMENT_WEBSITE_EN"] = 8;
                                values[valuesById[9] = "PREVENTATIVE_TREATMENT_WEBSITE_ES"] = 9;
                                values[valuesById[10] = "PRESCRIPTION_CONTRACEPTION_WEBSITE_EN"] = 10;
                                values[valuesById[11] = "PRESCRIPTION_CONTRACEPTION_WEBSITE_ES"] = 11;
                                values[valuesById[12] = "PRESCRIPTION_VACCINE_WEBSITE_EN"] = 12;
                                values[valuesById[13] = "PRESCRIPTION_VACCINE_WEBSITE_ES"] = 13;
                                return values;
                            })();
    
                            return VanityPharmaTextEnum;
                        })();
    
                        return enums;
                    })();
    
                    v11.errors = (function() {
    
                        /**
                         * Namespace errors.
                         * @memberof google.ads.googleads.v11
                         * @namespace
                         */
                        var errors = {};
    
                        errors.AccessInvitationErrorEnum = (function() {
    
                            /**
                             * Properties of an AccessInvitationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAccessInvitationErrorEnum
                             */
    
                            /**
                             * Constructs a new AccessInvitationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AccessInvitationErrorEnum.
                             * @implements IAccessInvitationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAccessInvitationErrorEnum=} [p] Properties to set
                             */
                            function AccessInvitationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccessInvitationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccessInvitationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AccessInvitationErrorEnum} AccessInvitationErrorEnum instance
                             */
                            AccessInvitationErrorEnum.create = function create(properties) {
                                return new AccessInvitationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccessInvitationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AccessInvitationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccessInvitationErrorEnum} m AccessInvitationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessInvitationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccessInvitationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AccessInvitationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccessInvitationErrorEnum} message AccessInvitationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessInvitationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccessInvitationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AccessInvitationErrorEnum} AccessInvitationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessInvitationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AccessInvitationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccessInvitationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AccessInvitationErrorEnum} AccessInvitationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessInvitationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccessInvitationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccessInvitationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccessInvitationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AccessInvitationErrorEnum} AccessInvitationErrorEnum
                             */
                            AccessInvitationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AccessInvitationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AccessInvitationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccessInvitationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AccessInvitationErrorEnum} m AccessInvitationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccessInvitationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccessInvitationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AccessInvitationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccessInvitationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccessInvitationError enum.
                             * @name google.ads.googleads.v11.errors.AccessInvitationErrorEnum.AccessInvitationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_EMAIL_ADDRESS=2 INVALID_EMAIL_ADDRESS value
                             * @property {number} EMAIL_ADDRESS_ALREADY_HAS_ACCESS=3 EMAIL_ADDRESS_ALREADY_HAS_ACCESS value
                             * @property {number} INVALID_INVITATION_STATUS=4 INVALID_INVITATION_STATUS value
                             * @property {number} GOOGLE_CONSUMER_ACCOUNT_NOT_ALLOWED=5 GOOGLE_CONSUMER_ACCOUNT_NOT_ALLOWED value
                             * @property {number} INVALID_INVITATION_ID=6 INVALID_INVITATION_ID value
                             * @property {number} EMAIL_ADDRESS_ALREADY_HAS_PENDING_INVITATION=7 EMAIL_ADDRESS_ALREADY_HAS_PENDING_INVITATION value
                             * @property {number} PENDING_INVITATIONS_LIMIT_EXCEEDED=8 PENDING_INVITATIONS_LIMIT_EXCEEDED value
                             * @property {number} EMAIL_DOMAIN_POLICY_VIOLATED=9 EMAIL_DOMAIN_POLICY_VIOLATED value
                             */
                            AccessInvitationErrorEnum.AccessInvitationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_EMAIL_ADDRESS"] = 2;
                                values[valuesById[3] = "EMAIL_ADDRESS_ALREADY_HAS_ACCESS"] = 3;
                                values[valuesById[4] = "INVALID_INVITATION_STATUS"] = 4;
                                values[valuesById[5] = "GOOGLE_CONSUMER_ACCOUNT_NOT_ALLOWED"] = 5;
                                values[valuesById[6] = "INVALID_INVITATION_ID"] = 6;
                                values[valuesById[7] = "EMAIL_ADDRESS_ALREADY_HAS_PENDING_INVITATION"] = 7;
                                values[valuesById[8] = "PENDING_INVITATIONS_LIMIT_EXCEEDED"] = 8;
                                values[valuesById[9] = "EMAIL_DOMAIN_POLICY_VIOLATED"] = 9;
                                return values;
                            })();
    
                            return AccessInvitationErrorEnum;
                        })();
    
                        errors.AccountBudgetProposalErrorEnum = (function() {
    
                            /**
                             * Properties of an AccountBudgetProposalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAccountBudgetProposalErrorEnum
                             */
    
                            /**
                             * Constructs a new AccountBudgetProposalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AccountBudgetProposalErrorEnum.
                             * @implements IAccountBudgetProposalErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAccountBudgetProposalErrorEnum=} [p] Properties to set
                             */
                            function AccountBudgetProposalErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccountBudgetProposalErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccountBudgetProposalErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum} AccountBudgetProposalErrorEnum instance
                             */
                            AccountBudgetProposalErrorEnum.create = function create(properties) {
                                return new AccountBudgetProposalErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccountBudgetProposalErrorEnum} m AccountBudgetProposalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccountBudgetProposalErrorEnum} message AccountBudgetProposalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum} AccountBudgetProposalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum} AccountBudgetProposalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountBudgetProposalErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountBudgetProposalErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccountBudgetProposalErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum} AccountBudgetProposalErrorEnum
                             */
                            AccountBudgetProposalErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccountBudgetProposalErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum} m AccountBudgetProposalErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountBudgetProposalErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccountBudgetProposalErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountBudgetProposalErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccountBudgetProposalError enum.
                             * @name google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum.AccountBudgetProposalError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FIELD_MASK_NOT_ALLOWED=2 FIELD_MASK_NOT_ALLOWED value
                             * @property {number} IMMUTABLE_FIELD=3 IMMUTABLE_FIELD value
                             * @property {number} REQUIRED_FIELD_MISSING=4 REQUIRED_FIELD_MISSING value
                             * @property {number} CANNOT_CANCEL_APPROVED_PROPOSAL=5 CANNOT_CANCEL_APPROVED_PROPOSAL value
                             * @property {number} CANNOT_REMOVE_UNAPPROVED_BUDGET=6 CANNOT_REMOVE_UNAPPROVED_BUDGET value
                             * @property {number} CANNOT_REMOVE_RUNNING_BUDGET=7 CANNOT_REMOVE_RUNNING_BUDGET value
                             * @property {number} CANNOT_END_UNAPPROVED_BUDGET=8 CANNOT_END_UNAPPROVED_BUDGET value
                             * @property {number} CANNOT_END_INACTIVE_BUDGET=9 CANNOT_END_INACTIVE_BUDGET value
                             * @property {number} BUDGET_NAME_REQUIRED=10 BUDGET_NAME_REQUIRED value
                             * @property {number} CANNOT_UPDATE_OLD_BUDGET=11 CANNOT_UPDATE_OLD_BUDGET value
                             * @property {number} CANNOT_END_IN_PAST=12 CANNOT_END_IN_PAST value
                             * @property {number} CANNOT_EXTEND_END_TIME=13 CANNOT_EXTEND_END_TIME value
                             * @property {number} PURCHASE_ORDER_NUMBER_REQUIRED=14 PURCHASE_ORDER_NUMBER_REQUIRED value
                             * @property {number} PENDING_UPDATE_PROPOSAL_EXISTS=15 PENDING_UPDATE_PROPOSAL_EXISTS value
                             * @property {number} MULTIPLE_BUDGETS_NOT_ALLOWED_FOR_UNAPPROVED_BILLING_SETUP=16 MULTIPLE_BUDGETS_NOT_ALLOWED_FOR_UNAPPROVED_BILLING_SETUP value
                             * @property {number} CANNOT_UPDATE_START_TIME_FOR_STARTED_BUDGET=17 CANNOT_UPDATE_START_TIME_FOR_STARTED_BUDGET value
                             * @property {number} SPENDING_LIMIT_LOWER_THAN_ACCRUED_COST_NOT_ALLOWED=18 SPENDING_LIMIT_LOWER_THAN_ACCRUED_COST_NOT_ALLOWED value
                             * @property {number} UPDATE_IS_NO_OP=19 UPDATE_IS_NO_OP value
                             * @property {number} END_TIME_MUST_FOLLOW_START_TIME=20 END_TIME_MUST_FOLLOW_START_TIME value
                             * @property {number} BUDGET_DATE_RANGE_INCOMPATIBLE_WITH_BILLING_SETUP=21 BUDGET_DATE_RANGE_INCOMPATIBLE_WITH_BILLING_SETUP value
                             * @property {number} NOT_AUTHORIZED=22 NOT_AUTHORIZED value
                             * @property {number} INVALID_BILLING_SETUP=23 INVALID_BILLING_SETUP value
                             * @property {number} OVERLAPS_EXISTING_BUDGET=24 OVERLAPS_EXISTING_BUDGET value
                             * @property {number} CANNOT_CREATE_BUDGET_THROUGH_API=25 CANNOT_CREATE_BUDGET_THROUGH_API value
                             */
                            AccountBudgetProposalErrorEnum.AccountBudgetProposalError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FIELD_MASK_NOT_ALLOWED"] = 2;
                                values[valuesById[3] = "IMMUTABLE_FIELD"] = 3;
                                values[valuesById[4] = "REQUIRED_FIELD_MISSING"] = 4;
                                values[valuesById[5] = "CANNOT_CANCEL_APPROVED_PROPOSAL"] = 5;
                                values[valuesById[6] = "CANNOT_REMOVE_UNAPPROVED_BUDGET"] = 6;
                                values[valuesById[7] = "CANNOT_REMOVE_RUNNING_BUDGET"] = 7;
                                values[valuesById[8] = "CANNOT_END_UNAPPROVED_BUDGET"] = 8;
                                values[valuesById[9] = "CANNOT_END_INACTIVE_BUDGET"] = 9;
                                values[valuesById[10] = "BUDGET_NAME_REQUIRED"] = 10;
                                values[valuesById[11] = "CANNOT_UPDATE_OLD_BUDGET"] = 11;
                                values[valuesById[12] = "CANNOT_END_IN_PAST"] = 12;
                                values[valuesById[13] = "CANNOT_EXTEND_END_TIME"] = 13;
                                values[valuesById[14] = "PURCHASE_ORDER_NUMBER_REQUIRED"] = 14;
                                values[valuesById[15] = "PENDING_UPDATE_PROPOSAL_EXISTS"] = 15;
                                values[valuesById[16] = "MULTIPLE_BUDGETS_NOT_ALLOWED_FOR_UNAPPROVED_BILLING_SETUP"] = 16;
                                values[valuesById[17] = "CANNOT_UPDATE_START_TIME_FOR_STARTED_BUDGET"] = 17;
                                values[valuesById[18] = "SPENDING_LIMIT_LOWER_THAN_ACCRUED_COST_NOT_ALLOWED"] = 18;
                                values[valuesById[19] = "UPDATE_IS_NO_OP"] = 19;
                                values[valuesById[20] = "END_TIME_MUST_FOLLOW_START_TIME"] = 20;
                                values[valuesById[21] = "BUDGET_DATE_RANGE_INCOMPATIBLE_WITH_BILLING_SETUP"] = 21;
                                values[valuesById[22] = "NOT_AUTHORIZED"] = 22;
                                values[valuesById[23] = "INVALID_BILLING_SETUP"] = 23;
                                values[valuesById[24] = "OVERLAPS_EXISTING_BUDGET"] = 24;
                                values[valuesById[25] = "CANNOT_CREATE_BUDGET_THROUGH_API"] = 25;
                                return values;
                            })();
    
                            return AccountBudgetProposalErrorEnum;
                        })();
    
                        errors.AccountLinkErrorEnum = (function() {
    
                            /**
                             * Properties of an AccountLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAccountLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new AccountLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AccountLinkErrorEnum.
                             * @implements IAccountLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAccountLinkErrorEnum=} [p] Properties to set
                             */
                            function AccountLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AccountLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccountLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AccountLinkErrorEnum} AccountLinkErrorEnum instance
                             */
                            AccountLinkErrorEnum.create = function create(properties) {
                                return new AccountLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AccountLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AccountLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccountLinkErrorEnum} m AccountLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AccountLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAccountLinkErrorEnum} message AccountLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AccountLinkErrorEnum} AccountLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AccountLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AccountLinkErrorEnum} AccountLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AccountLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AccountLinkErrorEnum} AccountLinkErrorEnum
                             */
                            AccountLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AccountLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AccountLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AccountLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AccountLinkErrorEnum} m AccountLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AccountLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AccountLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AccountLinkError enum.
                             * @name google.ads.googleads.v11.errors.AccountLinkErrorEnum.AccountLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_STATUS=2 INVALID_STATUS value
                             */
                            AccountLinkErrorEnum.AccountLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_STATUS"] = 2;
                                return values;
                            })();
    
                            return AccountLinkErrorEnum;
                        })();
    
                        errors.AdCustomizerErrorEnum = (function() {
    
                            /**
                             * Properties of an AdCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdCustomizerErrorEnum
                             */
    
                            /**
                             * Constructs a new AdCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdCustomizerErrorEnum.
                             * @implements IAdCustomizerErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdCustomizerErrorEnum=} [p] Properties to set
                             */
                            function AdCustomizerErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdCustomizerErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdCustomizerErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdCustomizerErrorEnum} AdCustomizerErrorEnum instance
                             */
                            AdCustomizerErrorEnum.create = function create(properties) {
                                return new AdCustomizerErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdCustomizerErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdCustomizerErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdCustomizerErrorEnum} m AdCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdCustomizerErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdCustomizerErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdCustomizerErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdCustomizerErrorEnum} message AdCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdCustomizerErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdCustomizerErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdCustomizerErrorEnum} AdCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdCustomizerErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdCustomizerErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdCustomizerErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdCustomizerErrorEnum} AdCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdCustomizerErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdCustomizerErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdCustomizerErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdCustomizerErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdCustomizerErrorEnum} AdCustomizerErrorEnum
                             */
                            AdCustomizerErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdCustomizerErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdCustomizerErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdCustomizerErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdCustomizerErrorEnum} m AdCustomizerErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdCustomizerErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdCustomizerErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdCustomizerErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdCustomizerErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdCustomizerError enum.
                             * @name google.ads.googleads.v11.errors.AdCustomizerErrorEnum.AdCustomizerError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} COUNTDOWN_INVALID_DATE_FORMAT=2 COUNTDOWN_INVALID_DATE_FORMAT value
                             * @property {number} COUNTDOWN_DATE_IN_PAST=3 COUNTDOWN_DATE_IN_PAST value
                             * @property {number} COUNTDOWN_INVALID_LOCALE=4 COUNTDOWN_INVALID_LOCALE value
                             * @property {number} COUNTDOWN_INVALID_START_DAYS_BEFORE=5 COUNTDOWN_INVALID_START_DAYS_BEFORE value
                             * @property {number} UNKNOWN_USER_LIST=6 UNKNOWN_USER_LIST value
                             */
                            AdCustomizerErrorEnum.AdCustomizerError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "COUNTDOWN_INVALID_DATE_FORMAT"] = 2;
                                values[valuesById[3] = "COUNTDOWN_DATE_IN_PAST"] = 3;
                                values[valuesById[4] = "COUNTDOWN_INVALID_LOCALE"] = 4;
                                values[valuesById[5] = "COUNTDOWN_INVALID_START_DAYS_BEFORE"] = 5;
                                values[valuesById[6] = "UNKNOWN_USER_LIST"] = 6;
                                return values;
                            })();
    
                            return AdCustomizerErrorEnum;
                        })();
    
                        errors.AdErrorEnum = (function() {
    
                            /**
                             * Properties of an AdErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdErrorEnum
                             */
    
                            /**
                             * Constructs a new AdErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdErrorEnum.
                             * @implements IAdErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdErrorEnum=} [p] Properties to set
                             */
                            function AdErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdErrorEnum} AdErrorEnum instance
                             */
                            AdErrorEnum.create = function create(properties) {
                                return new AdErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdErrorEnum} m AdErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdErrorEnum} message AdErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdErrorEnum} AdErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdErrorEnum} AdErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdErrorEnum} AdErrorEnum
                             */
                            AdErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdErrorEnum} m AdErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdError enum.
                             * @name google.ads.googleads.v11.errors.AdErrorEnum.AdError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_CUSTOMIZERS_NOT_SUPPORTED_FOR_AD_TYPE=2 AD_CUSTOMIZERS_NOT_SUPPORTED_FOR_AD_TYPE value
                             * @property {number} APPROXIMATELY_TOO_LONG=3 APPROXIMATELY_TOO_LONG value
                             * @property {number} APPROXIMATELY_TOO_SHORT=4 APPROXIMATELY_TOO_SHORT value
                             * @property {number} BAD_SNIPPET=5 BAD_SNIPPET value
                             * @property {number} CANNOT_MODIFY_AD=6 CANNOT_MODIFY_AD value
                             * @property {number} CANNOT_SET_BUSINESS_NAME_IF_URL_SET=7 CANNOT_SET_BUSINESS_NAME_IF_URL_SET value
                             * @property {number} CANNOT_SET_FIELD=8 CANNOT_SET_FIELD value
                             * @property {number} CANNOT_SET_FIELD_WITH_ORIGIN_AD_ID_SET=9 CANNOT_SET_FIELD_WITH_ORIGIN_AD_ID_SET value
                             * @property {number} CANNOT_SET_FIELD_WITH_AD_ID_SET_FOR_SHARING=10 CANNOT_SET_FIELD_WITH_AD_ID_SET_FOR_SHARING value
                             * @property {number} CANNOT_SET_ALLOW_FLEXIBLE_COLOR_FALSE=11 CANNOT_SET_ALLOW_FLEXIBLE_COLOR_FALSE value
                             * @property {number} CANNOT_SET_COLOR_CONTROL_WHEN_NATIVE_FORMAT_SETTING=12 CANNOT_SET_COLOR_CONTROL_WHEN_NATIVE_FORMAT_SETTING value
                             * @property {number} CANNOT_SET_URL=13 CANNOT_SET_URL value
                             * @property {number} CANNOT_SET_WITHOUT_FINAL_URLS=14 CANNOT_SET_WITHOUT_FINAL_URLS value
                             * @property {number} CANNOT_SET_WITH_FINAL_URLS=15 CANNOT_SET_WITH_FINAL_URLS value
                             * @property {number} CANNOT_SET_WITH_URL_DATA=17 CANNOT_SET_WITH_URL_DATA value
                             * @property {number} CANNOT_USE_AD_SUBCLASS_FOR_OPERATOR=18 CANNOT_USE_AD_SUBCLASS_FOR_OPERATOR value
                             * @property {number} CUSTOMER_NOT_APPROVED_MOBILEADS=19 CUSTOMER_NOT_APPROVED_MOBILEADS value
                             * @property {number} CUSTOMER_NOT_APPROVED_THIRDPARTY_ADS=20 CUSTOMER_NOT_APPROVED_THIRDPARTY_ADS value
                             * @property {number} CUSTOMER_NOT_APPROVED_THIRDPARTY_REDIRECT_ADS=21 CUSTOMER_NOT_APPROVED_THIRDPARTY_REDIRECT_ADS value
                             * @property {number} CUSTOMER_NOT_ELIGIBLE=22 CUSTOMER_NOT_ELIGIBLE value
                             * @property {number} CUSTOMER_NOT_ELIGIBLE_FOR_UPDATING_BEACON_URL=23 CUSTOMER_NOT_ELIGIBLE_FOR_UPDATING_BEACON_URL value
                             * @property {number} DIMENSION_ALREADY_IN_UNION=24 DIMENSION_ALREADY_IN_UNION value
                             * @property {number} DIMENSION_MUST_BE_SET=25 DIMENSION_MUST_BE_SET value
                             * @property {number} DIMENSION_NOT_IN_UNION=26 DIMENSION_NOT_IN_UNION value
                             * @property {number} DISPLAY_URL_CANNOT_BE_SPECIFIED=27 DISPLAY_URL_CANNOT_BE_SPECIFIED value
                             * @property {number} DOMESTIC_PHONE_NUMBER_FORMAT=28 DOMESTIC_PHONE_NUMBER_FORMAT value
                             * @property {number} EMERGENCY_PHONE_NUMBER=29 EMERGENCY_PHONE_NUMBER value
                             * @property {number} EMPTY_FIELD=30 EMPTY_FIELD value
                             * @property {number} FEED_ATTRIBUTE_MUST_HAVE_MAPPING_FOR_TYPE_ID=31 FEED_ATTRIBUTE_MUST_HAVE_MAPPING_FOR_TYPE_ID value
                             * @property {number} FEED_ATTRIBUTE_MAPPING_TYPE_MISMATCH=32 FEED_ATTRIBUTE_MAPPING_TYPE_MISMATCH value
                             * @property {number} ILLEGAL_AD_CUSTOMIZER_TAG_USE=33 ILLEGAL_AD_CUSTOMIZER_TAG_USE value
                             * @property {number} ILLEGAL_TAG_USE=34 ILLEGAL_TAG_USE value
                             * @property {number} INCONSISTENT_DIMENSIONS=35 INCONSISTENT_DIMENSIONS value
                             * @property {number} INCONSISTENT_STATUS_IN_TEMPLATE_UNION=36 INCONSISTENT_STATUS_IN_TEMPLATE_UNION value
                             * @property {number} INCORRECT_LENGTH=37 INCORRECT_LENGTH value
                             * @property {number} INELIGIBLE_FOR_UPGRADE=38 INELIGIBLE_FOR_UPGRADE value
                             * @property {number} INVALID_AD_ADDRESS_CAMPAIGN_TARGET=39 INVALID_AD_ADDRESS_CAMPAIGN_TARGET value
                             * @property {number} INVALID_AD_TYPE=40 INVALID_AD_TYPE value
                             * @property {number} INVALID_ATTRIBUTES_FOR_MOBILE_IMAGE=41 INVALID_ATTRIBUTES_FOR_MOBILE_IMAGE value
                             * @property {number} INVALID_ATTRIBUTES_FOR_MOBILE_TEXT=42 INVALID_ATTRIBUTES_FOR_MOBILE_TEXT value
                             * @property {number} INVALID_CALL_TO_ACTION_TEXT=43 INVALID_CALL_TO_ACTION_TEXT value
                             * @property {number} INVALID_CHARACTER_FOR_URL=44 INVALID_CHARACTER_FOR_URL value
                             * @property {number} INVALID_COUNTRY_CODE=45 INVALID_COUNTRY_CODE value
                             * @property {number} INVALID_EXPANDED_DYNAMIC_SEARCH_AD_TAG=47 INVALID_EXPANDED_DYNAMIC_SEARCH_AD_TAG value
                             * @property {number} INVALID_INPUT=48 INVALID_INPUT value
                             * @property {number} INVALID_MARKUP_LANGUAGE=49 INVALID_MARKUP_LANGUAGE value
                             * @property {number} INVALID_MOBILE_CARRIER=50 INVALID_MOBILE_CARRIER value
                             * @property {number} INVALID_MOBILE_CARRIER_TARGET=51 INVALID_MOBILE_CARRIER_TARGET value
                             * @property {number} INVALID_NUMBER_OF_ELEMENTS=52 INVALID_NUMBER_OF_ELEMENTS value
                             * @property {number} INVALID_PHONE_NUMBER_FORMAT=53 INVALID_PHONE_NUMBER_FORMAT value
                             * @property {number} INVALID_RICH_MEDIA_CERTIFIED_VENDOR_FORMAT_ID=54 INVALID_RICH_MEDIA_CERTIFIED_VENDOR_FORMAT_ID value
                             * @property {number} INVALID_TEMPLATE_DATA=55 INVALID_TEMPLATE_DATA value
                             * @property {number} INVALID_TEMPLATE_ELEMENT_FIELD_TYPE=56 INVALID_TEMPLATE_ELEMENT_FIELD_TYPE value
                             * @property {number} INVALID_TEMPLATE_ID=57 INVALID_TEMPLATE_ID value
                             * @property {number} LINE_TOO_WIDE=58 LINE_TOO_WIDE value
                             * @property {number} MISSING_AD_CUSTOMIZER_MAPPING=59 MISSING_AD_CUSTOMIZER_MAPPING value
                             * @property {number} MISSING_ADDRESS_COMPONENT=60 MISSING_ADDRESS_COMPONENT value
                             * @property {number} MISSING_ADVERTISEMENT_NAME=61 MISSING_ADVERTISEMENT_NAME value
                             * @property {number} MISSING_BUSINESS_NAME=62 MISSING_BUSINESS_NAME value
                             * @property {number} MISSING_DESCRIPTION1=63 MISSING_DESCRIPTION1 value
                             * @property {number} MISSING_DESCRIPTION2=64 MISSING_DESCRIPTION2 value
                             * @property {number} MISSING_DESTINATION_URL_TAG=65 MISSING_DESTINATION_URL_TAG value
                             * @property {number} MISSING_LANDING_PAGE_URL_TAG=66 MISSING_LANDING_PAGE_URL_TAG value
                             * @property {number} MISSING_DIMENSION=67 MISSING_DIMENSION value
                             * @property {number} MISSING_DISPLAY_URL=68 MISSING_DISPLAY_URL value
                             * @property {number} MISSING_HEADLINE=69 MISSING_HEADLINE value
                             * @property {number} MISSING_HEIGHT=70 MISSING_HEIGHT value
                             * @property {number} MISSING_IMAGE=71 MISSING_IMAGE value
                             * @property {number} MISSING_MARKETING_IMAGE_OR_PRODUCT_VIDEOS=72 MISSING_MARKETING_IMAGE_OR_PRODUCT_VIDEOS value
                             * @property {number} MISSING_MARKUP_LANGUAGES=73 MISSING_MARKUP_LANGUAGES value
                             * @property {number} MISSING_MOBILE_CARRIER=74 MISSING_MOBILE_CARRIER value
                             * @property {number} MISSING_PHONE=75 MISSING_PHONE value
                             * @property {number} MISSING_REQUIRED_TEMPLATE_FIELDS=76 MISSING_REQUIRED_TEMPLATE_FIELDS value
                             * @property {number} MISSING_TEMPLATE_FIELD_VALUE=77 MISSING_TEMPLATE_FIELD_VALUE value
                             * @property {number} MISSING_TEXT=78 MISSING_TEXT value
                             * @property {number} MISSING_VISIBLE_URL=79 MISSING_VISIBLE_URL value
                             * @property {number} MISSING_WIDTH=80 MISSING_WIDTH value
                             * @property {number} MULTIPLE_DISTINCT_FEEDS_UNSUPPORTED=81 MULTIPLE_DISTINCT_FEEDS_UNSUPPORTED value
                             * @property {number} MUST_USE_TEMP_AD_UNION_ID_ON_ADD=82 MUST_USE_TEMP_AD_UNION_ID_ON_ADD value
                             * @property {number} TOO_LONG=83 TOO_LONG value
                             * @property {number} TOO_SHORT=84 TOO_SHORT value
                             * @property {number} UNION_DIMENSIONS_CANNOT_CHANGE=85 UNION_DIMENSIONS_CANNOT_CHANGE value
                             * @property {number} UNKNOWN_ADDRESS_COMPONENT=86 UNKNOWN_ADDRESS_COMPONENT value
                             * @property {number} UNKNOWN_FIELD_NAME=87 UNKNOWN_FIELD_NAME value
                             * @property {number} UNKNOWN_UNIQUE_NAME=88 UNKNOWN_UNIQUE_NAME value
                             * @property {number} UNSUPPORTED_DIMENSIONS=89 UNSUPPORTED_DIMENSIONS value
                             * @property {number} URL_INVALID_SCHEME=90 URL_INVALID_SCHEME value
                             * @property {number} URL_INVALID_TOP_LEVEL_DOMAIN=91 URL_INVALID_TOP_LEVEL_DOMAIN value
                             * @property {number} URL_MALFORMED=92 URL_MALFORMED value
                             * @property {number} URL_NO_HOST=93 URL_NO_HOST value
                             * @property {number} URL_NOT_EQUIVALENT=94 URL_NOT_EQUIVALENT value
                             * @property {number} URL_HOST_NAME_TOO_LONG=95 URL_HOST_NAME_TOO_LONG value
                             * @property {number} URL_NO_SCHEME=96 URL_NO_SCHEME value
                             * @property {number} URL_NO_TOP_LEVEL_DOMAIN=97 URL_NO_TOP_LEVEL_DOMAIN value
                             * @property {number} URL_PATH_NOT_ALLOWED=98 URL_PATH_NOT_ALLOWED value
                             * @property {number} URL_PORT_NOT_ALLOWED=99 URL_PORT_NOT_ALLOWED value
                             * @property {number} URL_QUERY_NOT_ALLOWED=100 URL_QUERY_NOT_ALLOWED value
                             * @property {number} URL_SCHEME_BEFORE_EXPANDED_DYNAMIC_SEARCH_AD_TAG=102 URL_SCHEME_BEFORE_EXPANDED_DYNAMIC_SEARCH_AD_TAG value
                             * @property {number} USER_DOES_NOT_HAVE_ACCESS_TO_TEMPLATE=103 USER_DOES_NOT_HAVE_ACCESS_TO_TEMPLATE value
                             * @property {number} INCONSISTENT_EXPANDABLE_SETTINGS=104 INCONSISTENT_EXPANDABLE_SETTINGS value
                             * @property {number} INVALID_FORMAT=105 INVALID_FORMAT value
                             * @property {number} INVALID_FIELD_TEXT=106 INVALID_FIELD_TEXT value
                             * @property {number} ELEMENT_NOT_PRESENT=107 ELEMENT_NOT_PRESENT value
                             * @property {number} IMAGE_ERROR=108 IMAGE_ERROR value
                             * @property {number} VALUE_NOT_IN_RANGE=109 VALUE_NOT_IN_RANGE value
                             * @property {number} FIELD_NOT_PRESENT=110 FIELD_NOT_PRESENT value
                             * @property {number} ADDRESS_NOT_COMPLETE=111 ADDRESS_NOT_COMPLETE value
                             * @property {number} ADDRESS_INVALID=112 ADDRESS_INVALID value
                             * @property {number} VIDEO_RETRIEVAL_ERROR=113 VIDEO_RETRIEVAL_ERROR value
                             * @property {number} AUDIO_ERROR=114 AUDIO_ERROR value
                             * @property {number} INVALID_YOUTUBE_DISPLAY_URL=115 INVALID_YOUTUBE_DISPLAY_URL value
                             * @property {number} TOO_MANY_PRODUCT_IMAGES=116 TOO_MANY_PRODUCT_IMAGES value
                             * @property {number} TOO_MANY_PRODUCT_VIDEOS=117 TOO_MANY_PRODUCT_VIDEOS value
                             * @property {number} INCOMPATIBLE_AD_TYPE_AND_DEVICE_PREFERENCE=118 INCOMPATIBLE_AD_TYPE_AND_DEVICE_PREFERENCE value
                             * @property {number} CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY=119 CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED=120 CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED value
                             * @property {number} DISALLOWED_NUMBER_TYPE=121 DISALLOWED_NUMBER_TYPE value
                             * @property {number} PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY=122 PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY=123 PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY value
                             * @property {number} PREMIUM_RATE_NUMBER_NOT_ALLOWED=124 PREMIUM_RATE_NUMBER_NOT_ALLOWED value
                             * @property {number} VANITY_PHONE_NUMBER_NOT_ALLOWED=125 VANITY_PHONE_NUMBER_NOT_ALLOWED value
                             * @property {number} INVALID_CALL_CONVERSION_TYPE_ID=126 INVALID_CALL_CONVERSION_TYPE_ID value
                             * @property {number} CANNOT_DISABLE_CALL_CONVERSION_AND_SET_CONVERSION_TYPE_ID=127 CANNOT_DISABLE_CALL_CONVERSION_AND_SET_CONVERSION_TYPE_ID value
                             * @property {number} CANNOT_SET_PATH2_WITHOUT_PATH1=128 CANNOT_SET_PATH2_WITHOUT_PATH1 value
                             * @property {number} MISSING_DYNAMIC_SEARCH_ADS_SETTING_DOMAIN_NAME=129 MISSING_DYNAMIC_SEARCH_ADS_SETTING_DOMAIN_NAME value
                             * @property {number} INCOMPATIBLE_WITH_RESTRICTION_TYPE=130 INCOMPATIBLE_WITH_RESTRICTION_TYPE value
                             * @property {number} CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED=131 CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED value
                             * @property {number} MISSING_IMAGE_OR_MEDIA_BUNDLE=132 MISSING_IMAGE_OR_MEDIA_BUNDLE value
                             * @property {number} PRODUCT_TYPE_NOT_SUPPORTED_IN_THIS_CAMPAIGN=133 PRODUCT_TYPE_NOT_SUPPORTED_IN_THIS_CAMPAIGN value
                             * @property {number} PLACEHOLDER_CANNOT_HAVE_EMPTY_DEFAULT_VALUE=134 PLACEHOLDER_CANNOT_HAVE_EMPTY_DEFAULT_VALUE value
                             * @property {number} PLACEHOLDER_COUNTDOWN_FUNCTION_CANNOT_HAVE_DEFAULT_VALUE=135 PLACEHOLDER_COUNTDOWN_FUNCTION_CANNOT_HAVE_DEFAULT_VALUE value
                             * @property {number} PLACEHOLDER_DEFAULT_VALUE_MISSING=136 PLACEHOLDER_DEFAULT_VALUE_MISSING value
                             * @property {number} UNEXPECTED_PLACEHOLDER_DEFAULT_VALUE=137 UNEXPECTED_PLACEHOLDER_DEFAULT_VALUE value
                             * @property {number} AD_CUSTOMIZERS_MAY_NOT_BE_ADJACENT=138 AD_CUSTOMIZERS_MAY_NOT_BE_ADJACENT value
                             * @property {number} UPDATING_AD_WITH_NO_ENABLED_ASSOCIATION=139 UPDATING_AD_WITH_NO_ENABLED_ASSOCIATION value
                             * @property {number} CALL_AD_VERIFICATION_URL_FINAL_URL_DOES_NOT_HAVE_SAME_DOMAIN=140 CALL_AD_VERIFICATION_URL_FINAL_URL_DOES_NOT_HAVE_SAME_DOMAIN value
                             * @property {number} CALL_AD_FINAL_URL_AND_VERIFICATION_URL_CANNOT_BOTH_BE_EMPTY=154 CALL_AD_FINAL_URL_AND_VERIFICATION_URL_CANNOT_BOTH_BE_EMPTY value
                             * @property {number} TOO_MANY_AD_CUSTOMIZERS=141 TOO_MANY_AD_CUSTOMIZERS value
                             * @property {number} INVALID_AD_CUSTOMIZER_FORMAT=142 INVALID_AD_CUSTOMIZER_FORMAT value
                             * @property {number} NESTED_AD_CUSTOMIZER_SYNTAX=143 NESTED_AD_CUSTOMIZER_SYNTAX value
                             * @property {number} UNSUPPORTED_AD_CUSTOMIZER_SYNTAX=144 UNSUPPORTED_AD_CUSTOMIZER_SYNTAX value
                             * @property {number} UNPAIRED_BRACE_IN_AD_CUSTOMIZER_TAG=145 UNPAIRED_BRACE_IN_AD_CUSTOMIZER_TAG value
                             * @property {number} MORE_THAN_ONE_COUNTDOWN_TAG_TYPE_EXISTS=146 MORE_THAN_ONE_COUNTDOWN_TAG_TYPE_EXISTS value
                             * @property {number} DATE_TIME_IN_COUNTDOWN_TAG_IS_INVALID=147 DATE_TIME_IN_COUNTDOWN_TAG_IS_INVALID value
                             * @property {number} DATE_TIME_IN_COUNTDOWN_TAG_IS_PAST=148 DATE_TIME_IN_COUNTDOWN_TAG_IS_PAST value
                             * @property {number} UNRECOGNIZED_AD_CUSTOMIZER_TAG_FOUND=149 UNRECOGNIZED_AD_CUSTOMIZER_TAG_FOUND value
                             * @property {number} CUSTOMIZER_TYPE_FORBIDDEN_FOR_FIELD=150 CUSTOMIZER_TYPE_FORBIDDEN_FOR_FIELD value
                             * @property {number} INVALID_CUSTOMIZER_ATTRIBUTE_NAME=151 INVALID_CUSTOMIZER_ATTRIBUTE_NAME value
                             * @property {number} STORE_MISMATCH=152 STORE_MISMATCH value
                             * @property {number} MISSING_REQUIRED_IMAGE_ASPECT_RATIO=153 MISSING_REQUIRED_IMAGE_ASPECT_RATIO value
                             * @property {number} MISMATCHED_ASPECT_RATIOS=155 MISMATCHED_ASPECT_RATIOS value
                             * @property {number} DUPLICATE_IMAGE_ACROSS_CAROUSEL_CARDS=156 DUPLICATE_IMAGE_ACROSS_CAROUSEL_CARDS value
                             */
                            AdErrorEnum.AdError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_CUSTOMIZERS_NOT_SUPPORTED_FOR_AD_TYPE"] = 2;
                                values[valuesById[3] = "APPROXIMATELY_TOO_LONG"] = 3;
                                values[valuesById[4] = "APPROXIMATELY_TOO_SHORT"] = 4;
                                values[valuesById[5] = "BAD_SNIPPET"] = 5;
                                values[valuesById[6] = "CANNOT_MODIFY_AD"] = 6;
                                values[valuesById[7] = "CANNOT_SET_BUSINESS_NAME_IF_URL_SET"] = 7;
                                values[valuesById[8] = "CANNOT_SET_FIELD"] = 8;
                                values[valuesById[9] = "CANNOT_SET_FIELD_WITH_ORIGIN_AD_ID_SET"] = 9;
                                values[valuesById[10] = "CANNOT_SET_FIELD_WITH_AD_ID_SET_FOR_SHARING"] = 10;
                                values[valuesById[11] = "CANNOT_SET_ALLOW_FLEXIBLE_COLOR_FALSE"] = 11;
                                values[valuesById[12] = "CANNOT_SET_COLOR_CONTROL_WHEN_NATIVE_FORMAT_SETTING"] = 12;
                                values[valuesById[13] = "CANNOT_SET_URL"] = 13;
                                values[valuesById[14] = "CANNOT_SET_WITHOUT_FINAL_URLS"] = 14;
                                values[valuesById[15] = "CANNOT_SET_WITH_FINAL_URLS"] = 15;
                                values[valuesById[17] = "CANNOT_SET_WITH_URL_DATA"] = 17;
                                values[valuesById[18] = "CANNOT_USE_AD_SUBCLASS_FOR_OPERATOR"] = 18;
                                values[valuesById[19] = "CUSTOMER_NOT_APPROVED_MOBILEADS"] = 19;
                                values[valuesById[20] = "CUSTOMER_NOT_APPROVED_THIRDPARTY_ADS"] = 20;
                                values[valuesById[21] = "CUSTOMER_NOT_APPROVED_THIRDPARTY_REDIRECT_ADS"] = 21;
                                values[valuesById[22] = "CUSTOMER_NOT_ELIGIBLE"] = 22;
                                values[valuesById[23] = "CUSTOMER_NOT_ELIGIBLE_FOR_UPDATING_BEACON_URL"] = 23;
                                values[valuesById[24] = "DIMENSION_ALREADY_IN_UNION"] = 24;
                                values[valuesById[25] = "DIMENSION_MUST_BE_SET"] = 25;
                                values[valuesById[26] = "DIMENSION_NOT_IN_UNION"] = 26;
                                values[valuesById[27] = "DISPLAY_URL_CANNOT_BE_SPECIFIED"] = 27;
                                values[valuesById[28] = "DOMESTIC_PHONE_NUMBER_FORMAT"] = 28;
                                values[valuesById[29] = "EMERGENCY_PHONE_NUMBER"] = 29;
                                values[valuesById[30] = "EMPTY_FIELD"] = 30;
                                values[valuesById[31] = "FEED_ATTRIBUTE_MUST_HAVE_MAPPING_FOR_TYPE_ID"] = 31;
                                values[valuesById[32] = "FEED_ATTRIBUTE_MAPPING_TYPE_MISMATCH"] = 32;
                                values[valuesById[33] = "ILLEGAL_AD_CUSTOMIZER_TAG_USE"] = 33;
                                values[valuesById[34] = "ILLEGAL_TAG_USE"] = 34;
                                values[valuesById[35] = "INCONSISTENT_DIMENSIONS"] = 35;
                                values[valuesById[36] = "INCONSISTENT_STATUS_IN_TEMPLATE_UNION"] = 36;
                                values[valuesById[37] = "INCORRECT_LENGTH"] = 37;
                                values[valuesById[38] = "INELIGIBLE_FOR_UPGRADE"] = 38;
                                values[valuesById[39] = "INVALID_AD_ADDRESS_CAMPAIGN_TARGET"] = 39;
                                values[valuesById[40] = "INVALID_AD_TYPE"] = 40;
                                values[valuesById[41] = "INVALID_ATTRIBUTES_FOR_MOBILE_IMAGE"] = 41;
                                values[valuesById[42] = "INVALID_ATTRIBUTES_FOR_MOBILE_TEXT"] = 42;
                                values[valuesById[43] = "INVALID_CALL_TO_ACTION_TEXT"] = 43;
                                values[valuesById[44] = "INVALID_CHARACTER_FOR_URL"] = 44;
                                values[valuesById[45] = "INVALID_COUNTRY_CODE"] = 45;
                                values[valuesById[47] = "INVALID_EXPANDED_DYNAMIC_SEARCH_AD_TAG"] = 47;
                                values[valuesById[48] = "INVALID_INPUT"] = 48;
                                values[valuesById[49] = "INVALID_MARKUP_LANGUAGE"] = 49;
                                values[valuesById[50] = "INVALID_MOBILE_CARRIER"] = 50;
                                values[valuesById[51] = "INVALID_MOBILE_CARRIER_TARGET"] = 51;
                                values[valuesById[52] = "INVALID_NUMBER_OF_ELEMENTS"] = 52;
                                values[valuesById[53] = "INVALID_PHONE_NUMBER_FORMAT"] = 53;
                                values[valuesById[54] = "INVALID_RICH_MEDIA_CERTIFIED_VENDOR_FORMAT_ID"] = 54;
                                values[valuesById[55] = "INVALID_TEMPLATE_DATA"] = 55;
                                values[valuesById[56] = "INVALID_TEMPLATE_ELEMENT_FIELD_TYPE"] = 56;
                                values[valuesById[57] = "INVALID_TEMPLATE_ID"] = 57;
                                values[valuesById[58] = "LINE_TOO_WIDE"] = 58;
                                values[valuesById[59] = "MISSING_AD_CUSTOMIZER_MAPPING"] = 59;
                                values[valuesById[60] = "MISSING_ADDRESS_COMPONENT"] = 60;
                                values[valuesById[61] = "MISSING_ADVERTISEMENT_NAME"] = 61;
                                values[valuesById[62] = "MISSING_BUSINESS_NAME"] = 62;
                                values[valuesById[63] = "MISSING_DESCRIPTION1"] = 63;
                                values[valuesById[64] = "MISSING_DESCRIPTION2"] = 64;
                                values[valuesById[65] = "MISSING_DESTINATION_URL_TAG"] = 65;
                                values[valuesById[66] = "MISSING_LANDING_PAGE_URL_TAG"] = 66;
                                values[valuesById[67] = "MISSING_DIMENSION"] = 67;
                                values[valuesById[68] = "MISSING_DISPLAY_URL"] = 68;
                                values[valuesById[69] = "MISSING_HEADLINE"] = 69;
                                values[valuesById[70] = "MISSING_HEIGHT"] = 70;
                                values[valuesById[71] = "MISSING_IMAGE"] = 71;
                                values[valuesById[72] = "MISSING_MARKETING_IMAGE_OR_PRODUCT_VIDEOS"] = 72;
                                values[valuesById[73] = "MISSING_MARKUP_LANGUAGES"] = 73;
                                values[valuesById[74] = "MISSING_MOBILE_CARRIER"] = 74;
                                values[valuesById[75] = "MISSING_PHONE"] = 75;
                                values[valuesById[76] = "MISSING_REQUIRED_TEMPLATE_FIELDS"] = 76;
                                values[valuesById[77] = "MISSING_TEMPLATE_FIELD_VALUE"] = 77;
                                values[valuesById[78] = "MISSING_TEXT"] = 78;
                                values[valuesById[79] = "MISSING_VISIBLE_URL"] = 79;
                                values[valuesById[80] = "MISSING_WIDTH"] = 80;
                                values[valuesById[81] = "MULTIPLE_DISTINCT_FEEDS_UNSUPPORTED"] = 81;
                                values[valuesById[82] = "MUST_USE_TEMP_AD_UNION_ID_ON_ADD"] = 82;
                                values[valuesById[83] = "TOO_LONG"] = 83;
                                values[valuesById[84] = "TOO_SHORT"] = 84;
                                values[valuesById[85] = "UNION_DIMENSIONS_CANNOT_CHANGE"] = 85;
                                values[valuesById[86] = "UNKNOWN_ADDRESS_COMPONENT"] = 86;
                                values[valuesById[87] = "UNKNOWN_FIELD_NAME"] = 87;
                                values[valuesById[88] = "UNKNOWN_UNIQUE_NAME"] = 88;
                                values[valuesById[89] = "UNSUPPORTED_DIMENSIONS"] = 89;
                                values[valuesById[90] = "URL_INVALID_SCHEME"] = 90;
                                values[valuesById[91] = "URL_INVALID_TOP_LEVEL_DOMAIN"] = 91;
                                values[valuesById[92] = "URL_MALFORMED"] = 92;
                                values[valuesById[93] = "URL_NO_HOST"] = 93;
                                values[valuesById[94] = "URL_NOT_EQUIVALENT"] = 94;
                                values[valuesById[95] = "URL_HOST_NAME_TOO_LONG"] = 95;
                                values[valuesById[96] = "URL_NO_SCHEME"] = 96;
                                values[valuesById[97] = "URL_NO_TOP_LEVEL_DOMAIN"] = 97;
                                values[valuesById[98] = "URL_PATH_NOT_ALLOWED"] = 98;
                                values[valuesById[99] = "URL_PORT_NOT_ALLOWED"] = 99;
                                values[valuesById[100] = "URL_QUERY_NOT_ALLOWED"] = 100;
                                values[valuesById[102] = "URL_SCHEME_BEFORE_EXPANDED_DYNAMIC_SEARCH_AD_TAG"] = 102;
                                values[valuesById[103] = "USER_DOES_NOT_HAVE_ACCESS_TO_TEMPLATE"] = 103;
                                values[valuesById[104] = "INCONSISTENT_EXPANDABLE_SETTINGS"] = 104;
                                values[valuesById[105] = "INVALID_FORMAT"] = 105;
                                values[valuesById[106] = "INVALID_FIELD_TEXT"] = 106;
                                values[valuesById[107] = "ELEMENT_NOT_PRESENT"] = 107;
                                values[valuesById[108] = "IMAGE_ERROR"] = 108;
                                values[valuesById[109] = "VALUE_NOT_IN_RANGE"] = 109;
                                values[valuesById[110] = "FIELD_NOT_PRESENT"] = 110;
                                values[valuesById[111] = "ADDRESS_NOT_COMPLETE"] = 111;
                                values[valuesById[112] = "ADDRESS_INVALID"] = 112;
                                values[valuesById[113] = "VIDEO_RETRIEVAL_ERROR"] = 113;
                                values[valuesById[114] = "AUDIO_ERROR"] = 114;
                                values[valuesById[115] = "INVALID_YOUTUBE_DISPLAY_URL"] = 115;
                                values[valuesById[116] = "TOO_MANY_PRODUCT_IMAGES"] = 116;
                                values[valuesById[117] = "TOO_MANY_PRODUCT_VIDEOS"] = 117;
                                values[valuesById[118] = "INCOMPATIBLE_AD_TYPE_AND_DEVICE_PREFERENCE"] = 118;
                                values[valuesById[119] = "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY"] = 119;
                                values[valuesById[120] = "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED"] = 120;
                                values[valuesById[121] = "DISALLOWED_NUMBER_TYPE"] = 121;
                                values[valuesById[122] = "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY"] = 122;
                                values[valuesById[123] = "PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY"] = 123;
                                values[valuesById[124] = "PREMIUM_RATE_NUMBER_NOT_ALLOWED"] = 124;
                                values[valuesById[125] = "VANITY_PHONE_NUMBER_NOT_ALLOWED"] = 125;
                                values[valuesById[126] = "INVALID_CALL_CONVERSION_TYPE_ID"] = 126;
                                values[valuesById[127] = "CANNOT_DISABLE_CALL_CONVERSION_AND_SET_CONVERSION_TYPE_ID"] = 127;
                                values[valuesById[128] = "CANNOT_SET_PATH2_WITHOUT_PATH1"] = 128;
                                values[valuesById[129] = "MISSING_DYNAMIC_SEARCH_ADS_SETTING_DOMAIN_NAME"] = 129;
                                values[valuesById[130] = "INCOMPATIBLE_WITH_RESTRICTION_TYPE"] = 130;
                                values[valuesById[131] = "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED"] = 131;
                                values[valuesById[132] = "MISSING_IMAGE_OR_MEDIA_BUNDLE"] = 132;
                                values[valuesById[133] = "PRODUCT_TYPE_NOT_SUPPORTED_IN_THIS_CAMPAIGN"] = 133;
                                values[valuesById[134] = "PLACEHOLDER_CANNOT_HAVE_EMPTY_DEFAULT_VALUE"] = 134;
                                values[valuesById[135] = "PLACEHOLDER_COUNTDOWN_FUNCTION_CANNOT_HAVE_DEFAULT_VALUE"] = 135;
                                values[valuesById[136] = "PLACEHOLDER_DEFAULT_VALUE_MISSING"] = 136;
                                values[valuesById[137] = "UNEXPECTED_PLACEHOLDER_DEFAULT_VALUE"] = 137;
                                values[valuesById[138] = "AD_CUSTOMIZERS_MAY_NOT_BE_ADJACENT"] = 138;
                                values[valuesById[139] = "UPDATING_AD_WITH_NO_ENABLED_ASSOCIATION"] = 139;
                                values[valuesById[140] = "CALL_AD_VERIFICATION_URL_FINAL_URL_DOES_NOT_HAVE_SAME_DOMAIN"] = 140;
                                values[valuesById[154] = "CALL_AD_FINAL_URL_AND_VERIFICATION_URL_CANNOT_BOTH_BE_EMPTY"] = 154;
                                values[valuesById[141] = "TOO_MANY_AD_CUSTOMIZERS"] = 141;
                                values[valuesById[142] = "INVALID_AD_CUSTOMIZER_FORMAT"] = 142;
                                values[valuesById[143] = "NESTED_AD_CUSTOMIZER_SYNTAX"] = 143;
                                values[valuesById[144] = "UNSUPPORTED_AD_CUSTOMIZER_SYNTAX"] = 144;
                                values[valuesById[145] = "UNPAIRED_BRACE_IN_AD_CUSTOMIZER_TAG"] = 145;
                                values[valuesById[146] = "MORE_THAN_ONE_COUNTDOWN_TAG_TYPE_EXISTS"] = 146;
                                values[valuesById[147] = "DATE_TIME_IN_COUNTDOWN_TAG_IS_INVALID"] = 147;
                                values[valuesById[148] = "DATE_TIME_IN_COUNTDOWN_TAG_IS_PAST"] = 148;
                                values[valuesById[149] = "UNRECOGNIZED_AD_CUSTOMIZER_TAG_FOUND"] = 149;
                                values[valuesById[150] = "CUSTOMIZER_TYPE_FORBIDDEN_FOR_FIELD"] = 150;
                                values[valuesById[151] = "INVALID_CUSTOMIZER_ATTRIBUTE_NAME"] = 151;
                                values[valuesById[152] = "STORE_MISMATCH"] = 152;
                                values[valuesById[153] = "MISSING_REQUIRED_IMAGE_ASPECT_RATIO"] = 153;
                                values[valuesById[155] = "MISMATCHED_ASPECT_RATIOS"] = 155;
                                values[valuesById[156] = "DUPLICATE_IMAGE_ACROSS_CAROUSEL_CARDS"] = 156;
                                return values;
                            })();
    
                            return AdErrorEnum;
                        })();
    
                        errors.AdGroupAdErrorEnum = (function() {
    
                            /**
                             * Properties of an AdGroupAdErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdGroupAdErrorEnum
                             */
    
                            /**
                             * Constructs a new AdGroupAdErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdGroupAdErrorEnum.
                             * @implements IAdGroupAdErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdGroupAdErrorEnum=} [p] Properties to set
                             */
                            function AdGroupAdErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupAdErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupAdErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdGroupAdErrorEnum} AdGroupAdErrorEnum instance
                             */
                            AdGroupAdErrorEnum.create = function create(properties) {
                                return new AdGroupAdErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupAdErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupAdErrorEnum} m AdGroupAdErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupAdErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupAdErrorEnum} message AdGroupAdErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdGroupAdErrorEnum} AdGroupAdErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdGroupAdErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdGroupAdErrorEnum} AdGroupAdErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdGroupAdErrorEnum} AdGroupAdErrorEnum
                             */
                            AdGroupAdErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdGroupAdErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdGroupAdErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdGroupAdErrorEnum} m AdGroupAdErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupAdErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdGroupAdErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupAdError enum.
                             * @name google.ads.googleads.v11.errors.AdGroupAdErrorEnum.AdGroupAdError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_GROUP_AD_LABEL_DOES_NOT_EXIST=2 AD_GROUP_AD_LABEL_DOES_NOT_EXIST value
                             * @property {number} AD_GROUP_AD_LABEL_ALREADY_EXISTS=3 AD_GROUP_AD_LABEL_ALREADY_EXISTS value
                             * @property {number} AD_NOT_UNDER_ADGROUP=4 AD_NOT_UNDER_ADGROUP value
                             * @property {number} CANNOT_OPERATE_ON_REMOVED_ADGROUPAD=5 CANNOT_OPERATE_ON_REMOVED_ADGROUPAD value
                             * @property {number} CANNOT_CREATE_DEPRECATED_ADS=6 CANNOT_CREATE_DEPRECATED_ADS value
                             * @property {number} CANNOT_CREATE_TEXT_ADS=7 CANNOT_CREATE_TEXT_ADS value
                             * @property {number} EMPTY_FIELD=8 EMPTY_FIELD value
                             * @property {number} RESOURCE_REFERENCED_IN_MULTIPLE_OPS=9 RESOURCE_REFERENCED_IN_MULTIPLE_OPS value
                             * @property {number} AD_TYPE_CANNOT_BE_PAUSED=10 AD_TYPE_CANNOT_BE_PAUSED value
                             * @property {number} AD_TYPE_CANNOT_BE_REMOVED=11 AD_TYPE_CANNOT_BE_REMOVED value
                             * @property {number} CANNOT_UPDATE_DEPRECATED_ADS=12 CANNOT_UPDATE_DEPRECATED_ADS value
                             */
                            AdGroupAdErrorEnum.AdGroupAdError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_GROUP_AD_LABEL_DOES_NOT_EXIST"] = 2;
                                values[valuesById[3] = "AD_GROUP_AD_LABEL_ALREADY_EXISTS"] = 3;
                                values[valuesById[4] = "AD_NOT_UNDER_ADGROUP"] = 4;
                                values[valuesById[5] = "CANNOT_OPERATE_ON_REMOVED_ADGROUPAD"] = 5;
                                values[valuesById[6] = "CANNOT_CREATE_DEPRECATED_ADS"] = 6;
                                values[valuesById[7] = "CANNOT_CREATE_TEXT_ADS"] = 7;
                                values[valuesById[8] = "EMPTY_FIELD"] = 8;
                                values[valuesById[9] = "RESOURCE_REFERENCED_IN_MULTIPLE_OPS"] = 9;
                                values[valuesById[10] = "AD_TYPE_CANNOT_BE_PAUSED"] = 10;
                                values[valuesById[11] = "AD_TYPE_CANNOT_BE_REMOVED"] = 11;
                                values[valuesById[12] = "CANNOT_UPDATE_DEPRECATED_ADS"] = 12;
                                return values;
                            })();
    
                            return AdGroupAdErrorEnum;
                        })();
    
                        errors.AdGroupBidModifierErrorEnum = (function() {
    
                            /**
                             * Properties of an AdGroupBidModifierErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdGroupBidModifierErrorEnum
                             */
    
                            /**
                             * Constructs a new AdGroupBidModifierErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdGroupBidModifierErrorEnum.
                             * @implements IAdGroupBidModifierErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdGroupBidModifierErrorEnum=} [p] Properties to set
                             */
                            function AdGroupBidModifierErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupBidModifierErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupBidModifierErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum} AdGroupBidModifierErrorEnum instance
                             */
                            AdGroupBidModifierErrorEnum.create = function create(properties) {
                                return new AdGroupBidModifierErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupBidModifierErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupBidModifierErrorEnum} m AdGroupBidModifierErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupBidModifierErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupBidModifierErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupBidModifierErrorEnum} message AdGroupBidModifierErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupBidModifierErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupBidModifierErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum} AdGroupBidModifierErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupBidModifierErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupBidModifierErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum} AdGroupBidModifierErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupBidModifierErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupBidModifierErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupBidModifierErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupBidModifierErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum} AdGroupBidModifierErrorEnum
                             */
                            AdGroupBidModifierErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupBidModifierErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum} m AdGroupBidModifierErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupBidModifierErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupBidModifierErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupBidModifierErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupBidModifierError enum.
                             * @name google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum.AdGroupBidModifierError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CRITERION_ID_NOT_SUPPORTED=2 CRITERION_ID_NOT_SUPPORTED value
                             * @property {number} CANNOT_OVERRIDE_OPTED_OUT_CAMPAIGN_CRITERION_BID_MODIFIER=3 CANNOT_OVERRIDE_OPTED_OUT_CAMPAIGN_CRITERION_BID_MODIFIER value
                             */
                            AdGroupBidModifierErrorEnum.AdGroupBidModifierError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CRITERION_ID_NOT_SUPPORTED"] = 2;
                                values[valuesById[3] = "CANNOT_OVERRIDE_OPTED_OUT_CAMPAIGN_CRITERION_BID_MODIFIER"] = 3;
                                return values;
                            })();
    
                            return AdGroupBidModifierErrorEnum;
                        })();
    
                        errors.AdGroupCriterionCustomizerErrorEnum = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdGroupCriterionCustomizerErrorEnum
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdGroupCriterionCustomizerErrorEnum.
                             * @implements IAdGroupCriterionCustomizerErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionCustomizerErrorEnum=} [p] Properties to set
                             */
                            function AdGroupCriterionCustomizerErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupCriterionCustomizerErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionCustomizerErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum} AdGroupCriterionCustomizerErrorEnum instance
                             */
                            AdGroupCriterionCustomizerErrorEnum.create = function create(properties) {
                                return new AdGroupCriterionCustomizerErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionCustomizerErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionCustomizerErrorEnum} m AdGroupCriterionCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionCustomizerErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionCustomizerErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionCustomizerErrorEnum} message AdGroupCriterionCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionCustomizerErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionCustomizerErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum} AdGroupCriterionCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionCustomizerErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionCustomizerErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum} AdGroupCriterionCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionCustomizerErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionCustomizerErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionCustomizerErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionCustomizerErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum} AdGroupCriterionCustomizerErrorEnum
                             */
                            AdGroupCriterionCustomizerErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionCustomizerErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum} m AdGroupCriterionCustomizerErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionCustomizerErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupCriterionCustomizerErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionCustomizerErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupCriterionCustomizerError enum.
                             * @name google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum.AdGroupCriterionCustomizerError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CRITERION_IS_NOT_KEYWORD=2 CRITERION_IS_NOT_KEYWORD value
                             */
                            AdGroupCriterionCustomizerErrorEnum.AdGroupCriterionCustomizerError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CRITERION_IS_NOT_KEYWORD"] = 2;
                                return values;
                            })();
    
                            return AdGroupCriterionCustomizerErrorEnum;
                        })();
    
                        errors.AdGroupCriterionErrorEnum = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdGroupCriterionErrorEnum
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdGroupCriterionErrorEnum.
                             * @implements IAdGroupCriterionErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionErrorEnum=} [p] Properties to set
                             */
                            function AdGroupCriterionErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupCriterionErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum} AdGroupCriterionErrorEnum instance
                             */
                            AdGroupCriterionErrorEnum.create = function create(properties) {
                                return new AdGroupCriterionErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionErrorEnum} m AdGroupCriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCriterionErrorEnum} message AdGroupCriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum} AdGroupCriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum} AdGroupCriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum} AdGroupCriterionErrorEnum
                             */
                            AdGroupCriterionErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum} m AdGroupCriterionErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupCriterionErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupCriterionError enum.
                             * @name google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum.AdGroupCriterionError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_GROUP_CRITERION_LABEL_DOES_NOT_EXIST=2 AD_GROUP_CRITERION_LABEL_DOES_NOT_EXIST value
                             * @property {number} AD_GROUP_CRITERION_LABEL_ALREADY_EXISTS=3 AD_GROUP_CRITERION_LABEL_ALREADY_EXISTS value
                             * @property {number} CANNOT_ADD_LABEL_TO_NEGATIVE_CRITERION=4 CANNOT_ADD_LABEL_TO_NEGATIVE_CRITERION value
                             * @property {number} TOO_MANY_OPERATIONS=5 TOO_MANY_OPERATIONS value
                             * @property {number} CANT_UPDATE_NEGATIVE=6 CANT_UPDATE_NEGATIVE value
                             * @property {number} CONCRETE_TYPE_REQUIRED=7 CONCRETE_TYPE_REQUIRED value
                             * @property {number} BID_INCOMPATIBLE_WITH_ADGROUP=8 BID_INCOMPATIBLE_WITH_ADGROUP value
                             * @property {number} CANNOT_TARGET_AND_EXCLUDE=9 CANNOT_TARGET_AND_EXCLUDE value
                             * @property {number} ILLEGAL_URL=10 ILLEGAL_URL value
                             * @property {number} INVALID_KEYWORD_TEXT=11 INVALID_KEYWORD_TEXT value
                             * @property {number} INVALID_DESTINATION_URL=12 INVALID_DESTINATION_URL value
                             * @property {number} MISSING_DESTINATION_URL_TAG=13 MISSING_DESTINATION_URL_TAG value
                             * @property {number} KEYWORD_LEVEL_BID_NOT_SUPPORTED_FOR_MANUALCPM=14 KEYWORD_LEVEL_BID_NOT_SUPPORTED_FOR_MANUALCPM value
                             * @property {number} INVALID_USER_STATUS=15 INVALID_USER_STATUS value
                             * @property {number} CANNOT_ADD_CRITERIA_TYPE=16 CANNOT_ADD_CRITERIA_TYPE value
                             * @property {number} CANNOT_EXCLUDE_CRITERIA_TYPE=17 CANNOT_EXCLUDE_CRITERIA_TYPE value
                             * @property {number} CAMPAIGN_TYPE_NOT_COMPATIBLE_WITH_PARTIAL_FAILURE=27 CAMPAIGN_TYPE_NOT_COMPATIBLE_WITH_PARTIAL_FAILURE value
                             * @property {number} OPERATIONS_FOR_TOO_MANY_SHOPPING_ADGROUPS=28 OPERATIONS_FOR_TOO_MANY_SHOPPING_ADGROUPS value
                             * @property {number} CANNOT_MODIFY_URL_FIELDS_WITH_DUPLICATE_ELEMENTS=29 CANNOT_MODIFY_URL_FIELDS_WITH_DUPLICATE_ELEMENTS value
                             * @property {number} CANNOT_SET_WITHOUT_FINAL_URLS=30 CANNOT_SET_WITHOUT_FINAL_URLS value
                             * @property {number} CANNOT_CLEAR_FINAL_URLS_IF_FINAL_MOBILE_URLS_EXIST=31 CANNOT_CLEAR_FINAL_URLS_IF_FINAL_MOBILE_URLS_EXIST value
                             * @property {number} CANNOT_CLEAR_FINAL_URLS_IF_FINAL_APP_URLS_EXIST=32 CANNOT_CLEAR_FINAL_URLS_IF_FINAL_APP_URLS_EXIST value
                             * @property {number} CANNOT_CLEAR_FINAL_URLS_IF_TRACKING_URL_TEMPLATE_EXISTS=33 CANNOT_CLEAR_FINAL_URLS_IF_TRACKING_URL_TEMPLATE_EXISTS value
                             * @property {number} CANNOT_CLEAR_FINAL_URLS_IF_URL_CUSTOM_PARAMETERS_EXIST=34 CANNOT_CLEAR_FINAL_URLS_IF_URL_CUSTOM_PARAMETERS_EXIST value
                             * @property {number} CANNOT_SET_BOTH_DESTINATION_URL_AND_FINAL_URLS=35 CANNOT_SET_BOTH_DESTINATION_URL_AND_FINAL_URLS value
                             * @property {number} CANNOT_SET_BOTH_DESTINATION_URL_AND_TRACKING_URL_TEMPLATE=36 CANNOT_SET_BOTH_DESTINATION_URL_AND_TRACKING_URL_TEMPLATE value
                             * @property {number} FINAL_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE=37 FINAL_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE value
                             * @property {number} FINAL_MOBILE_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE=38 FINAL_MOBILE_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE value
                             */
                            AdGroupCriterionErrorEnum.AdGroupCriterionError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_GROUP_CRITERION_LABEL_DOES_NOT_EXIST"] = 2;
                                values[valuesById[3] = "AD_GROUP_CRITERION_LABEL_ALREADY_EXISTS"] = 3;
                                values[valuesById[4] = "CANNOT_ADD_LABEL_TO_NEGATIVE_CRITERION"] = 4;
                                values[valuesById[5] = "TOO_MANY_OPERATIONS"] = 5;
                                values[valuesById[6] = "CANT_UPDATE_NEGATIVE"] = 6;
                                values[valuesById[7] = "CONCRETE_TYPE_REQUIRED"] = 7;
                                values[valuesById[8] = "BID_INCOMPATIBLE_WITH_ADGROUP"] = 8;
                                values[valuesById[9] = "CANNOT_TARGET_AND_EXCLUDE"] = 9;
                                values[valuesById[10] = "ILLEGAL_URL"] = 10;
                                values[valuesById[11] = "INVALID_KEYWORD_TEXT"] = 11;
                                values[valuesById[12] = "INVALID_DESTINATION_URL"] = 12;
                                values[valuesById[13] = "MISSING_DESTINATION_URL_TAG"] = 13;
                                values[valuesById[14] = "KEYWORD_LEVEL_BID_NOT_SUPPORTED_FOR_MANUALCPM"] = 14;
                                values[valuesById[15] = "INVALID_USER_STATUS"] = 15;
                                values[valuesById[16] = "CANNOT_ADD_CRITERIA_TYPE"] = 16;
                                values[valuesById[17] = "CANNOT_EXCLUDE_CRITERIA_TYPE"] = 17;
                                values[valuesById[27] = "CAMPAIGN_TYPE_NOT_COMPATIBLE_WITH_PARTIAL_FAILURE"] = 27;
                                values[valuesById[28] = "OPERATIONS_FOR_TOO_MANY_SHOPPING_ADGROUPS"] = 28;
                                values[valuesById[29] = "CANNOT_MODIFY_URL_FIELDS_WITH_DUPLICATE_ELEMENTS"] = 29;
                                values[valuesById[30] = "CANNOT_SET_WITHOUT_FINAL_URLS"] = 30;
                                values[valuesById[31] = "CANNOT_CLEAR_FINAL_URLS_IF_FINAL_MOBILE_URLS_EXIST"] = 31;
                                values[valuesById[32] = "CANNOT_CLEAR_FINAL_URLS_IF_FINAL_APP_URLS_EXIST"] = 32;
                                values[valuesById[33] = "CANNOT_CLEAR_FINAL_URLS_IF_TRACKING_URL_TEMPLATE_EXISTS"] = 33;
                                values[valuesById[34] = "CANNOT_CLEAR_FINAL_URLS_IF_URL_CUSTOM_PARAMETERS_EXIST"] = 34;
                                values[valuesById[35] = "CANNOT_SET_BOTH_DESTINATION_URL_AND_FINAL_URLS"] = 35;
                                values[valuesById[36] = "CANNOT_SET_BOTH_DESTINATION_URL_AND_TRACKING_URL_TEMPLATE"] = 36;
                                values[valuesById[37] = "FINAL_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE"] = 37;
                                values[valuesById[38] = "FINAL_MOBILE_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE"] = 38;
                                return values;
                            })();
    
                            return AdGroupCriterionErrorEnum;
                        })();
    
                        errors.AdGroupCustomizerErrorEnum = (function() {
    
                            /**
                             * Properties of an AdGroupCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdGroupCustomizerErrorEnum
                             */
    
                            /**
                             * Constructs a new AdGroupCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdGroupCustomizerErrorEnum.
                             * @implements IAdGroupCustomizerErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdGroupCustomizerErrorEnum=} [p] Properties to set
                             */
                            function AdGroupCustomizerErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupCustomizerErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCustomizerErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum} AdGroupCustomizerErrorEnum instance
                             */
                            AdGroupCustomizerErrorEnum.create = function create(properties) {
                                return new AdGroupCustomizerErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCustomizerErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCustomizerErrorEnum} m AdGroupCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCustomizerErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCustomizerErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupCustomizerErrorEnum} message AdGroupCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCustomizerErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCustomizerErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum} AdGroupCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCustomizerErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCustomizerErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum} AdGroupCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCustomizerErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCustomizerErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCustomizerErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCustomizerErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum} AdGroupCustomizerErrorEnum
                             */
                            AdGroupCustomizerErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCustomizerErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum} m AdGroupCustomizerErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCustomizerErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupCustomizerErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCustomizerErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupCustomizerError enum.
                             * @name google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum.AdGroupCustomizerError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             */
                            AdGroupCustomizerErrorEnum.AdGroupCustomizerError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                return values;
                            })();
    
                            return AdGroupCustomizerErrorEnum;
                        })();
    
                        errors.AdGroupErrorEnum = (function() {
    
                            /**
                             * Properties of an AdGroupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdGroupErrorEnum
                             */
    
                            /**
                             * Constructs a new AdGroupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdGroupErrorEnum.
                             * @implements IAdGroupErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdGroupErrorEnum=} [p] Properties to set
                             */
                            function AdGroupErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdGroupErrorEnum} AdGroupErrorEnum instance
                             */
                            AdGroupErrorEnum.create = function create(properties) {
                                return new AdGroupErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupErrorEnum} m AdGroupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupErrorEnum} message AdGroupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdGroupErrorEnum} AdGroupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdGroupErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdGroupErrorEnum} AdGroupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdGroupErrorEnum} AdGroupErrorEnum
                             */
                            AdGroupErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdGroupErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdGroupErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdGroupErrorEnum} m AdGroupErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdGroupErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupError enum.
                             * @name google.ads.googleads.v11.errors.AdGroupErrorEnum.AdGroupError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_ADGROUP_NAME=2 DUPLICATE_ADGROUP_NAME value
                             * @property {number} INVALID_ADGROUP_NAME=3 INVALID_ADGROUP_NAME value
                             * @property {number} ADVERTISER_NOT_ON_CONTENT_NETWORK=5 ADVERTISER_NOT_ON_CONTENT_NETWORK value
                             * @property {number} BID_TOO_BIG=6 BID_TOO_BIG value
                             * @property {number} BID_TYPE_AND_BIDDING_STRATEGY_MISMATCH=7 BID_TYPE_AND_BIDDING_STRATEGY_MISMATCH value
                             * @property {number} MISSING_ADGROUP_NAME=8 MISSING_ADGROUP_NAME value
                             * @property {number} ADGROUP_LABEL_DOES_NOT_EXIST=9 ADGROUP_LABEL_DOES_NOT_EXIST value
                             * @property {number} ADGROUP_LABEL_ALREADY_EXISTS=10 ADGROUP_LABEL_ALREADY_EXISTS value
                             * @property {number} INVALID_CONTENT_BID_CRITERION_TYPE_GROUP=11 INVALID_CONTENT_BID_CRITERION_TYPE_GROUP value
                             * @property {number} AD_GROUP_TYPE_NOT_VALID_FOR_ADVERTISING_CHANNEL_TYPE=12 AD_GROUP_TYPE_NOT_VALID_FOR_ADVERTISING_CHANNEL_TYPE value
                             * @property {number} ADGROUP_TYPE_NOT_SUPPORTED_FOR_CAMPAIGN_SALES_COUNTRY=13 ADGROUP_TYPE_NOT_SUPPORTED_FOR_CAMPAIGN_SALES_COUNTRY value
                             * @property {number} CANNOT_ADD_ADGROUP_OF_TYPE_DSA_TO_CAMPAIGN_WITHOUT_DSA_SETTING=14 CANNOT_ADD_ADGROUP_OF_TYPE_DSA_TO_CAMPAIGN_WITHOUT_DSA_SETTING value
                             * @property {number} PROMOTED_HOTEL_AD_GROUPS_NOT_AVAILABLE_FOR_CUSTOMER=15 PROMOTED_HOTEL_AD_GROUPS_NOT_AVAILABLE_FOR_CUSTOMER value
                             * @property {number} INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE=16 INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE value
                             */
                            AdGroupErrorEnum.AdGroupError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_ADGROUP_NAME"] = 2;
                                values[valuesById[3] = "INVALID_ADGROUP_NAME"] = 3;
                                values[valuesById[5] = "ADVERTISER_NOT_ON_CONTENT_NETWORK"] = 5;
                                values[valuesById[6] = "BID_TOO_BIG"] = 6;
                                values[valuesById[7] = "BID_TYPE_AND_BIDDING_STRATEGY_MISMATCH"] = 7;
                                values[valuesById[8] = "MISSING_ADGROUP_NAME"] = 8;
                                values[valuesById[9] = "ADGROUP_LABEL_DOES_NOT_EXIST"] = 9;
                                values[valuesById[10] = "ADGROUP_LABEL_ALREADY_EXISTS"] = 10;
                                values[valuesById[11] = "INVALID_CONTENT_BID_CRITERION_TYPE_GROUP"] = 11;
                                values[valuesById[12] = "AD_GROUP_TYPE_NOT_VALID_FOR_ADVERTISING_CHANNEL_TYPE"] = 12;
                                values[valuesById[13] = "ADGROUP_TYPE_NOT_SUPPORTED_FOR_CAMPAIGN_SALES_COUNTRY"] = 13;
                                values[valuesById[14] = "CANNOT_ADD_ADGROUP_OF_TYPE_DSA_TO_CAMPAIGN_WITHOUT_DSA_SETTING"] = 14;
                                values[valuesById[15] = "PROMOTED_HOTEL_AD_GROUPS_NOT_AVAILABLE_FOR_CUSTOMER"] = 15;
                                values[valuesById[16] = "INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE"] = 16;
                                return values;
                            })();
    
                            return AdGroupErrorEnum;
                        })();
    
                        errors.AdGroupFeedErrorEnum = (function() {
    
                            /**
                             * Properties of an AdGroupFeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdGroupFeedErrorEnum
                             */
    
                            /**
                             * Constructs a new AdGroupFeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdGroupFeedErrorEnum.
                             * @implements IAdGroupFeedErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdGroupFeedErrorEnum=} [p] Properties to set
                             */
                            function AdGroupFeedErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdGroupFeedErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupFeedErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdGroupFeedErrorEnum} AdGroupFeedErrorEnum instance
                             */
                            AdGroupFeedErrorEnum.create = function create(properties) {
                                return new AdGroupFeedErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupFeedErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupFeedErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupFeedErrorEnum} m AdGroupFeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupFeedErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupFeedErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdGroupFeedErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdGroupFeedErrorEnum} message AdGroupFeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupFeedErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupFeedErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdGroupFeedErrorEnum} AdGroupFeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupFeedErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdGroupFeedErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupFeedErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdGroupFeedErrorEnum} AdGroupFeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupFeedErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupFeedErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupFeedErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupFeedErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdGroupFeedErrorEnum} AdGroupFeedErrorEnum
                             */
                            AdGroupFeedErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdGroupFeedErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdGroupFeedErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdGroupFeedErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdGroupFeedErrorEnum} m AdGroupFeedErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupFeedErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdGroupFeedErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdGroupFeedErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupFeedErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdGroupFeedError enum.
                             * @name google.ads.googleads.v11.errors.AdGroupFeedErrorEnum.AdGroupFeedError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE=2 FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE value
                             * @property {number} CANNOT_CREATE_FOR_REMOVED_FEED=3 CANNOT_CREATE_FOR_REMOVED_FEED value
                             * @property {number} ADGROUP_FEED_ALREADY_EXISTS=4 ADGROUP_FEED_ALREADY_EXISTS value
                             * @property {number} CANNOT_OPERATE_ON_REMOVED_ADGROUP_FEED=5 CANNOT_OPERATE_ON_REMOVED_ADGROUP_FEED value
                             * @property {number} INVALID_PLACEHOLDER_TYPE=6 INVALID_PLACEHOLDER_TYPE value
                             * @property {number} MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE=7 MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE value
                             * @property {number} NO_EXISTING_LOCATION_CUSTOMER_FEED=8 NO_EXISTING_LOCATION_CUSTOMER_FEED value
                             */
                            AdGroupFeedErrorEnum.AdGroupFeedError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE"] = 2;
                                values[valuesById[3] = "CANNOT_CREATE_FOR_REMOVED_FEED"] = 3;
                                values[valuesById[4] = "ADGROUP_FEED_ALREADY_EXISTS"] = 4;
                                values[valuesById[5] = "CANNOT_OPERATE_ON_REMOVED_ADGROUP_FEED"] = 5;
                                values[valuesById[6] = "INVALID_PLACEHOLDER_TYPE"] = 6;
                                values[valuesById[7] = "MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE"] = 7;
                                values[valuesById[8] = "NO_EXISTING_LOCATION_CUSTOMER_FEED"] = 8;
                                return values;
                            })();
    
                            return AdGroupFeedErrorEnum;
                        })();
    
                        errors.AdParameterErrorEnum = (function() {
    
                            /**
                             * Properties of an AdParameterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdParameterErrorEnum
                             */
    
                            /**
                             * Constructs a new AdParameterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdParameterErrorEnum.
                             * @implements IAdParameterErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdParameterErrorEnum=} [p] Properties to set
                             */
                            function AdParameterErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdParameterErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdParameterErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdParameterErrorEnum} AdParameterErrorEnum instance
                             */
                            AdParameterErrorEnum.create = function create(properties) {
                                return new AdParameterErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdParameterErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdParameterErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdParameterErrorEnum} m AdParameterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdParameterErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdParameterErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdParameterErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdParameterErrorEnum} message AdParameterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdParameterErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdParameterErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdParameterErrorEnum} AdParameterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdParameterErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdParameterErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdParameterErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdParameterErrorEnum} AdParameterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdParameterErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdParameterErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdParameterErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdParameterErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdParameterErrorEnum} AdParameterErrorEnum
                             */
                            AdParameterErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdParameterErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdParameterErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdParameterErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdParameterErrorEnum} m AdParameterErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdParameterErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdParameterErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdParameterErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdParameterErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdParameterError enum.
                             * @name google.ads.googleads.v11.errors.AdParameterErrorEnum.AdParameterError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_GROUP_CRITERION_MUST_BE_KEYWORD=2 AD_GROUP_CRITERION_MUST_BE_KEYWORD value
                             * @property {number} INVALID_INSERTION_TEXT_FORMAT=3 INVALID_INSERTION_TEXT_FORMAT value
                             */
                            AdParameterErrorEnum.AdParameterError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_GROUP_CRITERION_MUST_BE_KEYWORD"] = 2;
                                values[valuesById[3] = "INVALID_INSERTION_TEXT_FORMAT"] = 3;
                                return values;
                            })();
    
                            return AdParameterErrorEnum;
                        })();
    
                        errors.AdSharingErrorEnum = (function() {
    
                            /**
                             * Properties of an AdSharingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdSharingErrorEnum
                             */
    
                            /**
                             * Constructs a new AdSharingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdSharingErrorEnum.
                             * @implements IAdSharingErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdSharingErrorEnum=} [p] Properties to set
                             */
                            function AdSharingErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdSharingErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdSharingErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdSharingErrorEnum} AdSharingErrorEnum instance
                             */
                            AdSharingErrorEnum.create = function create(properties) {
                                return new AdSharingErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdSharingErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdSharingErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdSharingErrorEnum} m AdSharingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdSharingErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdSharingErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdSharingErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdSharingErrorEnum} message AdSharingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdSharingErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdSharingErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdSharingErrorEnum} AdSharingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdSharingErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdSharingErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdSharingErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdSharingErrorEnum} AdSharingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdSharingErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdSharingErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdSharingErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdSharingErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdSharingErrorEnum} AdSharingErrorEnum
                             */
                            AdSharingErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdSharingErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdSharingErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdSharingErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdSharingErrorEnum} m AdSharingErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdSharingErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdSharingErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdSharingErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdSharingErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdSharingError enum.
                             * @name google.ads.googleads.v11.errors.AdSharingErrorEnum.AdSharingError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AD_GROUP_ALREADY_CONTAINS_AD=2 AD_GROUP_ALREADY_CONTAINS_AD value
                             * @property {number} INCOMPATIBLE_AD_UNDER_AD_GROUP=3 INCOMPATIBLE_AD_UNDER_AD_GROUP value
                             * @property {number} CANNOT_SHARE_INACTIVE_AD=4 CANNOT_SHARE_INACTIVE_AD value
                             */
                            AdSharingErrorEnum.AdSharingError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AD_GROUP_ALREADY_CONTAINS_AD"] = 2;
                                values[valuesById[3] = "INCOMPATIBLE_AD_UNDER_AD_GROUP"] = 3;
                                values[valuesById[4] = "CANNOT_SHARE_INACTIVE_AD"] = 4;
                                return values;
                            })();
    
                            return AdSharingErrorEnum;
                        })();
    
                        errors.AdxErrorEnum = (function() {
    
                            /**
                             * Properties of an AdxErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAdxErrorEnum
                             */
    
                            /**
                             * Constructs a new AdxErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AdxErrorEnum.
                             * @implements IAdxErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAdxErrorEnum=} [p] Properties to set
                             */
                            function AdxErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AdxErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdxErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AdxErrorEnum} AdxErrorEnum instance
                             */
                            AdxErrorEnum.create = function create(properties) {
                                return new AdxErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AdxErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AdxErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdxErrorEnum} m AdxErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdxErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdxErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AdxErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAdxErrorEnum} message AdxErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdxErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdxErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AdxErrorEnum} AdxErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdxErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AdxErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdxErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AdxErrorEnum} AdxErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdxErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdxErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdxErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AdxErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AdxErrorEnum} AdxErrorEnum
                             */
                            AdxErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AdxErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AdxErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AdxErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AdxErrorEnum} m AdxErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdxErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AdxErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AdxErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdxErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AdxError enum.
                             * @name google.ads.googleads.v11.errors.AdxErrorEnum.AdxError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNSUPPORTED_FEATURE=2 UNSUPPORTED_FEATURE value
                             */
                            AdxErrorEnum.AdxError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNSUPPORTED_FEATURE"] = 2;
                                return values;
                            })();
    
                            return AdxErrorEnum;
                        })();
    
                        errors.AssetErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetErrorEnum.
                             * @implements IAssetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetErrorEnum=} [p] Properties to set
                             */
                            function AssetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetErrorEnum} AssetErrorEnum instance
                             */
                            AssetErrorEnum.create = function create(properties) {
                                return new AssetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetErrorEnum} m AssetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetErrorEnum} message AssetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetErrorEnum} AssetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetErrorEnum} AssetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetErrorEnum} AssetErrorEnum
                             */
                            AssetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetErrorEnum} m AssetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetError enum.
                             * @name google.ads.googleads.v11.errors.AssetErrorEnum.AssetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CUSTOMER_NOT_ON_ALLOWLIST_FOR_ASSET_TYPE=13 CUSTOMER_NOT_ON_ALLOWLIST_FOR_ASSET_TYPE value
                             * @property {number} DUPLICATE_ASSET=3 DUPLICATE_ASSET value
                             * @property {number} DUPLICATE_ASSET_NAME=4 DUPLICATE_ASSET_NAME value
                             * @property {number} ASSET_DATA_IS_MISSING=5 ASSET_DATA_IS_MISSING value
                             * @property {number} CANNOT_MODIFY_ASSET_NAME=6 CANNOT_MODIFY_ASSET_NAME value
                             * @property {number} FIELD_INCOMPATIBLE_WITH_ASSET_TYPE=7 FIELD_INCOMPATIBLE_WITH_ASSET_TYPE value
                             * @property {number} INVALID_CALL_TO_ACTION_TEXT=8 INVALID_CALL_TO_ACTION_TEXT value
                             * @property {number} LEAD_FORM_INVALID_FIELDS_COMBINATION=9 LEAD_FORM_INVALID_FIELDS_COMBINATION value
                             * @property {number} LEAD_FORM_MISSING_AGREEMENT=10 LEAD_FORM_MISSING_AGREEMENT value
                             * @property {number} INVALID_ASSET_STATUS=11 INVALID_ASSET_STATUS value
                             * @property {number} FIELD_CANNOT_BE_MODIFIED_FOR_ASSET_TYPE=12 FIELD_CANNOT_BE_MODIFIED_FOR_ASSET_TYPE value
                             * @property {number} SCHEDULES_CANNOT_OVERLAP=14 SCHEDULES_CANNOT_OVERLAP value
                             * @property {number} PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF=15 PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF value
                             * @property {number} PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT=16 PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT value
                             * @property {number} TOO_MANY_DECIMAL_PLACES_SPECIFIED=17 TOO_MANY_DECIMAL_PLACES_SPECIFIED value
                             * @property {number} DUPLICATE_ASSETS_WITH_DIFFERENT_FIELD_VALUE=18 DUPLICATE_ASSETS_WITH_DIFFERENT_FIELD_VALUE value
                             * @property {number} CALL_CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED=19 CALL_CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED value
                             * @property {number} CALL_CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED=20 CALL_CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED value
                             * @property {number} CALL_DISALLOWED_NUMBER_TYPE=21 CALL_DISALLOWED_NUMBER_TYPE value
                             * @property {number} CALL_INVALID_CONVERSION_ACTION=22 CALL_INVALID_CONVERSION_ACTION value
                             * @property {number} CALL_INVALID_COUNTRY_CODE=23 CALL_INVALID_COUNTRY_CODE value
                             * @property {number} CALL_INVALID_DOMESTIC_PHONE_NUMBER_FORMAT=24 CALL_INVALID_DOMESTIC_PHONE_NUMBER_FORMAT value
                             * @property {number} CALL_INVALID_PHONE_NUMBER=25 CALL_INVALID_PHONE_NUMBER value
                             * @property {number} CALL_PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY=26 CALL_PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} CALL_PREMIUM_RATE_NUMBER_NOT_ALLOWED=27 CALL_PREMIUM_RATE_NUMBER_NOT_ALLOWED value
                             * @property {number} CALL_VANITY_PHONE_NUMBER_NOT_ALLOWED=28 CALL_VANITY_PHONE_NUMBER_NOT_ALLOWED value
                             * @property {number} PRICE_HEADER_SAME_AS_DESCRIPTION=29 PRICE_HEADER_SAME_AS_DESCRIPTION value
                             * @property {number} MOBILE_APP_INVALID_APP_ID=30 MOBILE_APP_INVALID_APP_ID value
                             * @property {number} MOBILE_APP_INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL=31 MOBILE_APP_INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL value
                             * @property {number} NAME_REQUIRED_FOR_ASSET_TYPE=32 NAME_REQUIRED_FOR_ASSET_TYPE value
                             * @property {number} LEAD_FORM_LEGACY_QUALIFYING_QUESTIONS_DISALLOWED=33 LEAD_FORM_LEGACY_QUALIFYING_QUESTIONS_DISALLOWED value
                             * @property {number} NAME_CONFLICT_FOR_ASSET_TYPE=34 NAME_CONFLICT_FOR_ASSET_TYPE value
                             * @property {number} CANNOT_MODIFY_ASSET_SOURCE=35 CANNOT_MODIFY_ASSET_SOURCE value
                             * @property {number} CANNOT_MODIFY_AUTOMATICALLY_CREATED_ASSET=36 CANNOT_MODIFY_AUTOMATICALLY_CREATED_ASSET value
                             */
                            AssetErrorEnum.AssetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[13] = "CUSTOMER_NOT_ON_ALLOWLIST_FOR_ASSET_TYPE"] = 13;
                                values[valuesById[3] = "DUPLICATE_ASSET"] = 3;
                                values[valuesById[4] = "DUPLICATE_ASSET_NAME"] = 4;
                                values[valuesById[5] = "ASSET_DATA_IS_MISSING"] = 5;
                                values[valuesById[6] = "CANNOT_MODIFY_ASSET_NAME"] = 6;
                                values[valuesById[7] = "FIELD_INCOMPATIBLE_WITH_ASSET_TYPE"] = 7;
                                values[valuesById[8] = "INVALID_CALL_TO_ACTION_TEXT"] = 8;
                                values[valuesById[9] = "LEAD_FORM_INVALID_FIELDS_COMBINATION"] = 9;
                                values[valuesById[10] = "LEAD_FORM_MISSING_AGREEMENT"] = 10;
                                values[valuesById[11] = "INVALID_ASSET_STATUS"] = 11;
                                values[valuesById[12] = "FIELD_CANNOT_BE_MODIFIED_FOR_ASSET_TYPE"] = 12;
                                values[valuesById[14] = "SCHEDULES_CANNOT_OVERLAP"] = 14;
                                values[valuesById[15] = "PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF"] = 15;
                                values[valuesById[16] = "PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT"] = 16;
                                values[valuesById[17] = "TOO_MANY_DECIMAL_PLACES_SPECIFIED"] = 17;
                                values[valuesById[18] = "DUPLICATE_ASSETS_WITH_DIFFERENT_FIELD_VALUE"] = 18;
                                values[valuesById[19] = "CALL_CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED"] = 19;
                                values[valuesById[20] = "CALL_CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED"] = 20;
                                values[valuesById[21] = "CALL_DISALLOWED_NUMBER_TYPE"] = 21;
                                values[valuesById[22] = "CALL_INVALID_CONVERSION_ACTION"] = 22;
                                values[valuesById[23] = "CALL_INVALID_COUNTRY_CODE"] = 23;
                                values[valuesById[24] = "CALL_INVALID_DOMESTIC_PHONE_NUMBER_FORMAT"] = 24;
                                values[valuesById[25] = "CALL_INVALID_PHONE_NUMBER"] = 25;
                                values[valuesById[26] = "CALL_PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY"] = 26;
                                values[valuesById[27] = "CALL_PREMIUM_RATE_NUMBER_NOT_ALLOWED"] = 27;
                                values[valuesById[28] = "CALL_VANITY_PHONE_NUMBER_NOT_ALLOWED"] = 28;
                                values[valuesById[29] = "PRICE_HEADER_SAME_AS_DESCRIPTION"] = 29;
                                values[valuesById[30] = "MOBILE_APP_INVALID_APP_ID"] = 30;
                                values[valuesById[31] = "MOBILE_APP_INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL"] = 31;
                                values[valuesById[32] = "NAME_REQUIRED_FOR_ASSET_TYPE"] = 32;
                                values[valuesById[33] = "LEAD_FORM_LEGACY_QUALIFYING_QUESTIONS_DISALLOWED"] = 33;
                                values[valuesById[34] = "NAME_CONFLICT_FOR_ASSET_TYPE"] = 34;
                                values[valuesById[35] = "CANNOT_MODIFY_ASSET_SOURCE"] = 35;
                                values[valuesById[36] = "CANNOT_MODIFY_AUTOMATICALLY_CREATED_ASSET"] = 36;
                                return values;
                            })();
    
                            return AssetErrorEnum;
                        })();
    
                        errors.AssetGroupAssetErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetGroupAssetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetGroupAssetErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetGroupAssetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetGroupAssetErrorEnum.
                             * @implements IAssetGroupAssetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetGroupAssetErrorEnum=} [p] Properties to set
                             */
                            function AssetGroupAssetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetGroupAssetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupAssetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum} AssetGroupAssetErrorEnum instance
                             */
                            AssetGroupAssetErrorEnum.create = function create(properties) {
                                return new AssetGroupAssetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupAssetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupAssetErrorEnum} m AssetGroupAssetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupAssetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupAssetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupAssetErrorEnum} message AssetGroupAssetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupAssetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupAssetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum} AssetGroupAssetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupAssetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupAssetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum} AssetGroupAssetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupAssetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupAssetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupAssetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupAssetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum} AssetGroupAssetErrorEnum
                             */
                            AssetGroupAssetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupAssetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum} m AssetGroupAssetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupAssetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetGroupAssetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupAssetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetGroupAssetError enum.
                             * @name google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum.AssetGroupAssetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_RESOURCE=2 DUPLICATE_RESOURCE value
                             * @property {number} EXPANDABLE_TAGS_NOT_ALLOWED_IN_DESCRIPTION=3 EXPANDABLE_TAGS_NOT_ALLOWED_IN_DESCRIPTION value
                             * @property {number} AD_CUSTOMIZER_NOT_SUPPORTED=4 AD_CUSTOMIZER_NOT_SUPPORTED value
                             */
                            AssetGroupAssetErrorEnum.AssetGroupAssetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_RESOURCE"] = 2;
                                values[valuesById[3] = "EXPANDABLE_TAGS_NOT_ALLOWED_IN_DESCRIPTION"] = 3;
                                values[valuesById[4] = "AD_CUSTOMIZER_NOT_SUPPORTED"] = 4;
                                return values;
                            })();
    
                            return AssetGroupAssetErrorEnum;
                        })();
    
                        errors.AssetGroupErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetGroupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetGroupErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetGroupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetGroupErrorEnum.
                             * @implements IAssetGroupErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetGroupErrorEnum=} [p] Properties to set
                             */
                            function AssetGroupErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetGroupErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetGroupErrorEnum} AssetGroupErrorEnum instance
                             */
                            AssetGroupErrorEnum.create = function create(properties) {
                                return new AssetGroupErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetGroupErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupErrorEnum} m AssetGroupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetGroupErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupErrorEnum} message AssetGroupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetGroupErrorEnum} AssetGroupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetGroupErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetGroupErrorEnum} AssetGroupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetGroupErrorEnum} AssetGroupErrorEnum
                             */
                            AssetGroupErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetGroupErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetGroupErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetGroupErrorEnum} m AssetGroupErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetGroupErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetGroupErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetGroupError enum.
                             * @name google.ads.googleads.v11.errors.AssetGroupErrorEnum.AssetGroupError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_NAME=2 DUPLICATE_NAME value
                             * @property {number} CANNOT_ADD_ASSET_GROUP_FOR_CAMPAIGN_TYPE=3 CANNOT_ADD_ASSET_GROUP_FOR_CAMPAIGN_TYPE value
                             * @property {number} NOT_ENOUGH_HEADLINE_ASSET=4 NOT_ENOUGH_HEADLINE_ASSET value
                             * @property {number} NOT_ENOUGH_LONG_HEADLINE_ASSET=5 NOT_ENOUGH_LONG_HEADLINE_ASSET value
                             * @property {number} NOT_ENOUGH_DESCRIPTION_ASSET=6 NOT_ENOUGH_DESCRIPTION_ASSET value
                             * @property {number} NOT_ENOUGH_BUSINESS_NAME_ASSET=7 NOT_ENOUGH_BUSINESS_NAME_ASSET value
                             * @property {number} NOT_ENOUGH_MARKETING_IMAGE_ASSET=8 NOT_ENOUGH_MARKETING_IMAGE_ASSET value
                             * @property {number} NOT_ENOUGH_SQUARE_MARKETING_IMAGE_ASSET=9 NOT_ENOUGH_SQUARE_MARKETING_IMAGE_ASSET value
                             * @property {number} NOT_ENOUGH_LOGO_ASSET=10 NOT_ENOUGH_LOGO_ASSET value
                             * @property {number} FINAL_URL_SHOPPING_MERCHANT_HOME_PAGE_URL_DOMAINS_DIFFER=11 FINAL_URL_SHOPPING_MERCHANT_HOME_PAGE_URL_DOMAINS_DIFFER value
                             */
                            AssetGroupErrorEnum.AssetGroupError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_NAME"] = 2;
                                values[valuesById[3] = "CANNOT_ADD_ASSET_GROUP_FOR_CAMPAIGN_TYPE"] = 3;
                                values[valuesById[4] = "NOT_ENOUGH_HEADLINE_ASSET"] = 4;
                                values[valuesById[5] = "NOT_ENOUGH_LONG_HEADLINE_ASSET"] = 5;
                                values[valuesById[6] = "NOT_ENOUGH_DESCRIPTION_ASSET"] = 6;
                                values[valuesById[7] = "NOT_ENOUGH_BUSINESS_NAME_ASSET"] = 7;
                                values[valuesById[8] = "NOT_ENOUGH_MARKETING_IMAGE_ASSET"] = 8;
                                values[valuesById[9] = "NOT_ENOUGH_SQUARE_MARKETING_IMAGE_ASSET"] = 9;
                                values[valuesById[10] = "NOT_ENOUGH_LOGO_ASSET"] = 10;
                                values[valuesById[11] = "FINAL_URL_SHOPPING_MERCHANT_HOME_PAGE_URL_DOMAINS_DIFFER"] = 11;
                                return values;
                            })();
    
                            return AssetGroupErrorEnum;
                        })();
    
                        errors.AssetGroupListingGroupFilterErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetGroupListingGroupFilterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetGroupListingGroupFilterErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetGroupListingGroupFilterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetGroupListingGroupFilterErrorEnum.
                             * @implements IAssetGroupListingGroupFilterErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetGroupListingGroupFilterErrorEnum=} [p] Properties to set
                             */
                            function AssetGroupListingGroupFilterErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetGroupListingGroupFilterErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupListingGroupFilterErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum} AssetGroupListingGroupFilterErrorEnum instance
                             */
                            AssetGroupListingGroupFilterErrorEnum.create = function create(properties) {
                                return new AssetGroupListingGroupFilterErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupListingGroupFilterErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupListingGroupFilterErrorEnum} m AssetGroupListingGroupFilterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupListingGroupFilterErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupListingGroupFilterErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetGroupListingGroupFilterErrorEnum} message AssetGroupListingGroupFilterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupListingGroupFilterErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupListingGroupFilterErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum} AssetGroupListingGroupFilterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupListingGroupFilterErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupListingGroupFilterErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum} AssetGroupListingGroupFilterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupListingGroupFilterErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupListingGroupFilterErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupListingGroupFilterErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupListingGroupFilterErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum} AssetGroupListingGroupFilterErrorEnum
                             */
                            AssetGroupListingGroupFilterErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupListingGroupFilterErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum} m AssetGroupListingGroupFilterErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupListingGroupFilterErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetGroupListingGroupFilterErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupListingGroupFilterErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetGroupListingGroupFilterError enum.
                             * @name google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum.AssetGroupListingGroupFilterError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TREE_TOO_DEEP=2 TREE_TOO_DEEP value
                             * @property {number} UNIT_CANNOT_HAVE_CHILDREN=3 UNIT_CANNOT_HAVE_CHILDREN value
                             * @property {number} SUBDIVISION_MUST_HAVE_EVERYTHING_ELSE_CHILD=4 SUBDIVISION_MUST_HAVE_EVERYTHING_ELSE_CHILD value
                             * @property {number} DIFFERENT_DIMENSION_TYPE_BETWEEN_SIBLINGS=5 DIFFERENT_DIMENSION_TYPE_BETWEEN_SIBLINGS value
                             * @property {number} SAME_DIMENSION_VALUE_BETWEEN_SIBLINGS=6 SAME_DIMENSION_VALUE_BETWEEN_SIBLINGS value
                             * @property {number} SAME_DIMENSION_TYPE_BETWEEN_ANCESTORS=7 SAME_DIMENSION_TYPE_BETWEEN_ANCESTORS value
                             * @property {number} MULTIPLE_ROOTS=8 MULTIPLE_ROOTS value
                             * @property {number} INVALID_DIMENSION_VALUE=9 INVALID_DIMENSION_VALUE value
                             * @property {number} MUST_REFINE_HIERARCHICAL_PARENT_TYPE=10 MUST_REFINE_HIERARCHICAL_PARENT_TYPE value
                             * @property {number} INVALID_PRODUCT_BIDDING_CATEGORY=11 INVALID_PRODUCT_BIDDING_CATEGORY value
                             * @property {number} CHANGING_CASE_VALUE_WITH_CHILDREN=12 CHANGING_CASE_VALUE_WITH_CHILDREN value
                             * @property {number} SUBDIVISION_HAS_CHILDREN=13 SUBDIVISION_HAS_CHILDREN value
                             * @property {number} CANNOT_REFINE_HIERARCHICAL_EVERYTHING_ELSE=14 CANNOT_REFINE_HIERARCHICAL_EVERYTHING_ELSE value
                             */
                            AssetGroupListingGroupFilterErrorEnum.AssetGroupListingGroupFilterError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TREE_TOO_DEEP"] = 2;
                                values[valuesById[3] = "UNIT_CANNOT_HAVE_CHILDREN"] = 3;
                                values[valuesById[4] = "SUBDIVISION_MUST_HAVE_EVERYTHING_ELSE_CHILD"] = 4;
                                values[valuesById[5] = "DIFFERENT_DIMENSION_TYPE_BETWEEN_SIBLINGS"] = 5;
                                values[valuesById[6] = "SAME_DIMENSION_VALUE_BETWEEN_SIBLINGS"] = 6;
                                values[valuesById[7] = "SAME_DIMENSION_TYPE_BETWEEN_ANCESTORS"] = 7;
                                values[valuesById[8] = "MULTIPLE_ROOTS"] = 8;
                                values[valuesById[9] = "INVALID_DIMENSION_VALUE"] = 9;
                                values[valuesById[10] = "MUST_REFINE_HIERARCHICAL_PARENT_TYPE"] = 10;
                                values[valuesById[11] = "INVALID_PRODUCT_BIDDING_CATEGORY"] = 11;
                                values[valuesById[12] = "CHANGING_CASE_VALUE_WITH_CHILDREN"] = 12;
                                values[valuesById[13] = "SUBDIVISION_HAS_CHILDREN"] = 13;
                                values[valuesById[14] = "CANNOT_REFINE_HIERARCHICAL_EVERYTHING_ELSE"] = 14;
                                return values;
                            })();
    
                            return AssetGroupListingGroupFilterErrorEnum;
                        })();
    
                        errors.AssetLinkErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetLinkErrorEnum.
                             * @implements IAssetLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetLinkErrorEnum=} [p] Properties to set
                             */
                            function AssetLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetLinkErrorEnum} AssetLinkErrorEnum instance
                             */
                            AssetLinkErrorEnum.create = function create(properties) {
                                return new AssetLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetLinkErrorEnum} m AssetLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetLinkErrorEnum} message AssetLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetLinkErrorEnum} AssetLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetLinkErrorEnum} AssetLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetLinkErrorEnum} AssetLinkErrorEnum
                             */
                            AssetLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetLinkErrorEnum} m AssetLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetLinkError enum.
                             * @name google.ads.googleads.v11.errors.AssetLinkErrorEnum.AssetLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PINNING_UNSUPPORTED=2 PINNING_UNSUPPORTED value
                             * @property {number} UNSUPPORTED_FIELD_TYPE=3 UNSUPPORTED_FIELD_TYPE value
                             * @property {number} FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE=4 FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE value
                             * @property {number} FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE=5 FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE value
                             * @property {number} INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE=6 INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE value
                             * @property {number} IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE=7 IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE value
                             * @property {number} INVALID_PINNED_FIELD=8 INVALID_PINNED_FIELD value
                             * @property {number} MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE=9 MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE value
                             * @property {number} NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION=10 NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION value
                             * @property {number} NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK=11 NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK value
                             * @property {number} NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION=12 NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION value
                             * @property {number} YOUTUBE_VIDEO_REMOVED=13 YOUTUBE_VIDEO_REMOVED value
                             * @property {number} YOUTUBE_VIDEO_TOO_LONG=14 YOUTUBE_VIDEO_TOO_LONG value
                             * @property {number} YOUTUBE_VIDEO_TOO_SHORT=15 YOUTUBE_VIDEO_TOO_SHORT value
                             * @property {number} INVALID_STATUS=17 INVALID_STATUS value
                             * @property {number} YOUTUBE_VIDEO_DURATION_NOT_DEFINED=18 YOUTUBE_VIDEO_DURATION_NOT_DEFINED value
                             * @property {number} CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS=19 CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS value
                             * @property {number} CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET=20 CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET value
                             * @property {number} CANNOT_MODIFY_ASSET_LINK_SOURCE=21 CANNOT_MODIFY_ASSET_LINK_SOURCE value
                             */
                            AssetLinkErrorEnum.AssetLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PINNING_UNSUPPORTED"] = 2;
                                values[valuesById[3] = "UNSUPPORTED_FIELD_TYPE"] = 3;
                                values[valuesById[4] = "FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE"] = 4;
                                values[valuesById[5] = "FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE"] = 5;
                                values[valuesById[6] = "INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE"] = 6;
                                values[valuesById[7] = "IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE"] = 7;
                                values[valuesById[8] = "INVALID_PINNED_FIELD"] = 8;
                                values[valuesById[9] = "MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE"] = 9;
                                values[valuesById[10] = "NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION"] = 10;
                                values[valuesById[11] = "NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK"] = 11;
                                values[valuesById[12] = "NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION"] = 12;
                                values[valuesById[13] = "YOUTUBE_VIDEO_REMOVED"] = 13;
                                values[valuesById[14] = "YOUTUBE_VIDEO_TOO_LONG"] = 14;
                                values[valuesById[15] = "YOUTUBE_VIDEO_TOO_SHORT"] = 15;
                                values[valuesById[17] = "INVALID_STATUS"] = 17;
                                values[valuesById[18] = "YOUTUBE_VIDEO_DURATION_NOT_DEFINED"] = 18;
                                values[valuesById[19] = "CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS"] = 19;
                                values[valuesById[20] = "CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET"] = 20;
                                values[valuesById[21] = "CANNOT_MODIFY_ASSET_LINK_SOURCE"] = 21;
                                return values;
                            })();
    
                            return AssetLinkErrorEnum;
                        })();
    
                        errors.AssetSetAssetErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetSetAssetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetSetAssetErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetSetAssetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetSetAssetErrorEnum.
                             * @implements IAssetSetAssetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetSetAssetErrorEnum=} [p] Properties to set
                             */
                            function AssetSetAssetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSetAssetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetAssetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetSetAssetErrorEnum} AssetSetAssetErrorEnum instance
                             */
                            AssetSetAssetErrorEnum.create = function create(properties) {
                                return new AssetSetAssetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetAssetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetSetAssetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetAssetErrorEnum} m AssetSetAssetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAssetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetAssetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetSetAssetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetAssetErrorEnum} message AssetSetAssetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAssetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetAssetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetSetAssetErrorEnum} AssetSetAssetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAssetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetSetAssetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetAssetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetSetAssetErrorEnum} AssetSetAssetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAssetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetAssetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetAssetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetAssetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetSetAssetErrorEnum} AssetSetAssetErrorEnum
                             */
                            AssetSetAssetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetSetAssetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetSetAssetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSetAssetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetSetAssetErrorEnum} m AssetSetAssetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetAssetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSetAssetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetSetAssetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetAssetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSetAssetError enum.
                             * @name google.ads.googleads.v11.errors.AssetSetAssetErrorEnum.AssetSetAssetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_ASSET_TYPE=2 INVALID_ASSET_TYPE value
                             * @property {number} INVALID_ASSET_SET_TYPE=3 INVALID_ASSET_SET_TYPE value
                             * @property {number} DUPLICATE_EXTERNAL_KEY=4 DUPLICATE_EXTERNAL_KEY value
                             */
                            AssetSetAssetErrorEnum.AssetSetAssetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_ASSET_TYPE"] = 2;
                                values[valuesById[3] = "INVALID_ASSET_SET_TYPE"] = 3;
                                values[valuesById[4] = "DUPLICATE_EXTERNAL_KEY"] = 4;
                                return values;
                            })();
    
                            return AssetSetAssetErrorEnum;
                        })();
    
                        errors.AssetSetErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetSetErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetSetErrorEnum.
                             * @implements IAssetSetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetSetErrorEnum=} [p] Properties to set
                             */
                            function AssetSetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetSetErrorEnum} AssetSetErrorEnum instance
                             */
                            AssetSetErrorEnum.create = function create(properties) {
                                return new AssetSetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetSetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetErrorEnum} m AssetSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetSetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetErrorEnum} message AssetSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetSetErrorEnum} AssetSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetSetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetSetErrorEnum} AssetSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetSetErrorEnum} AssetSetErrorEnum
                             */
                            AssetSetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetSetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetSetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetSetErrorEnum} m AssetSetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetSetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSetError enum.
                             * @name google.ads.googleads.v11.errors.AssetSetErrorEnum.AssetSetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_ASSET_SET_NAME=2 DUPLICATE_ASSET_SET_NAME value
                             */
                            AssetSetErrorEnum.AssetSetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_ASSET_SET_NAME"] = 2;
                                return values;
                            })();
    
                            return AssetSetErrorEnum;
                        })();
    
                        errors.AssetSetLinkErrorEnum = (function() {
    
                            /**
                             * Properties of an AssetSetLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAssetSetLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new AssetSetLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AssetSetLinkErrorEnum.
                             * @implements IAssetSetLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAssetSetLinkErrorEnum=} [p] Properties to set
                             */
                            function AssetSetLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AssetSetLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AssetSetLinkErrorEnum} AssetSetLinkErrorEnum instance
                             */
                            AssetSetLinkErrorEnum.create = function create(properties) {
                                return new AssetSetLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AssetSetLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetLinkErrorEnum} m AssetSetLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AssetSetLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAssetSetLinkErrorEnum} message AssetSetLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AssetSetLinkErrorEnum} AssetSetLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AssetSetLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AssetSetLinkErrorEnum} AssetSetLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AssetSetLinkErrorEnum} AssetSetLinkErrorEnum
                             */
                            AssetSetLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AssetSetLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AssetSetLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AssetSetLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AssetSetLinkErrorEnum} m AssetSetLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AssetSetLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AssetSetLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AssetSetLinkError enum.
                             * @name google.ads.googleads.v11.errors.AssetSetLinkErrorEnum.AssetSetLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE=2 INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE value
                             * @property {number} DUPLICATE_FEED_LINK=3 DUPLICATE_FEED_LINK value
                             * @property {number} INCOMPATIBLE_ASSET_SET_TYPE_WITH_CAMPAIGN_TYPE=4 INCOMPATIBLE_ASSET_SET_TYPE_WITH_CAMPAIGN_TYPE value
                             * @property {number} DUPLICATE_ASSET_SET_LINK=5 DUPLICATE_ASSET_SET_LINK value
                             * @property {number} ASSET_SET_LINK_CANNOT_BE_REMOVED=6 ASSET_SET_LINK_CANNOT_BE_REMOVED value
                             */
                            AssetSetLinkErrorEnum.AssetSetLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE"] = 2;
                                values[valuesById[3] = "DUPLICATE_FEED_LINK"] = 3;
                                values[valuesById[4] = "INCOMPATIBLE_ASSET_SET_TYPE_WITH_CAMPAIGN_TYPE"] = 4;
                                values[valuesById[5] = "DUPLICATE_ASSET_SET_LINK"] = 5;
                                values[valuesById[6] = "ASSET_SET_LINK_CANNOT_BE_REMOVED"] = 6;
                                return values;
                            })();
    
                            return AssetSetLinkErrorEnum;
                        })();
    
                        errors.AudienceErrorEnum = (function() {
    
                            /**
                             * Properties of an AudienceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAudienceErrorEnum
                             */
    
                            /**
                             * Constructs a new AudienceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AudienceErrorEnum.
                             * @implements IAudienceErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAudienceErrorEnum=} [p] Properties to set
                             */
                            function AudienceErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AudienceErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAudienceErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AudienceErrorEnum} AudienceErrorEnum instance
                             */
                            AudienceErrorEnum.create = function create(properties) {
                                return new AudienceErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AudienceErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAudienceErrorEnum} m AudienceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AudienceErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAudienceErrorEnum} message AudienceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AudienceErrorEnum} AudienceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AudienceErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AudienceErrorEnum} AudienceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AudienceErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AudienceErrorEnum} AudienceErrorEnum
                             */
                            AudienceErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AudienceErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AudienceErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AudienceErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AudienceErrorEnum} m AudienceErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AudienceErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AudienceErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AudienceError enum.
                             * @name google.ads.googleads.v11.errors.AudienceErrorEnum.AudienceError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NAME_ALREADY_IN_USE=2 NAME_ALREADY_IN_USE value
                             * @property {number} DIMENSION_INVALID=3 DIMENSION_INVALID value
                             * @property {number} AUDIENCE_SEGMENT_NOT_FOUND=4 AUDIENCE_SEGMENT_NOT_FOUND value
                             * @property {number} AUDIENCE_SEGMENT_TYPE_NOT_SUPPORTED=5 AUDIENCE_SEGMENT_TYPE_NOT_SUPPORTED value
                             * @property {number} DUPLICATE_AUDIENCE_SEGMENT=6 DUPLICATE_AUDIENCE_SEGMENT value
                             * @property {number} TOO_MANY_SEGMENTS=7 TOO_MANY_SEGMENTS value
                             * @property {number} TOO_MANY_DIMENSIONS_OF_SAME_TYPE=8 TOO_MANY_DIMENSIONS_OF_SAME_TYPE value
                             * @property {number} IN_USE=9 IN_USE value
                             */
                            AudienceErrorEnum.AudienceError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NAME_ALREADY_IN_USE"] = 2;
                                values[valuesById[3] = "DIMENSION_INVALID"] = 3;
                                values[valuesById[4] = "AUDIENCE_SEGMENT_NOT_FOUND"] = 4;
                                values[valuesById[5] = "AUDIENCE_SEGMENT_TYPE_NOT_SUPPORTED"] = 5;
                                values[valuesById[6] = "DUPLICATE_AUDIENCE_SEGMENT"] = 6;
                                values[valuesById[7] = "TOO_MANY_SEGMENTS"] = 7;
                                values[valuesById[8] = "TOO_MANY_DIMENSIONS_OF_SAME_TYPE"] = 8;
                                values[valuesById[9] = "IN_USE"] = 9;
                                return values;
                            })();
    
                            return AudienceErrorEnum;
                        })();
    
                        errors.AuthenticationErrorEnum = (function() {
    
                            /**
                             * Properties of an AuthenticationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAuthenticationErrorEnum
                             */
    
                            /**
                             * Constructs a new AuthenticationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AuthenticationErrorEnum.
                             * @implements IAuthenticationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAuthenticationErrorEnum=} [p] Properties to set
                             */
                            function AuthenticationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AuthenticationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAuthenticationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AuthenticationErrorEnum} AuthenticationErrorEnum instance
                             */
                            AuthenticationErrorEnum.create = function create(properties) {
                                return new AuthenticationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AuthenticationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AuthenticationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAuthenticationErrorEnum} m AuthenticationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AuthenticationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AuthenticationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AuthenticationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAuthenticationErrorEnum} message AuthenticationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AuthenticationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AuthenticationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AuthenticationErrorEnum} AuthenticationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AuthenticationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AuthenticationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AuthenticationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AuthenticationErrorEnum} AuthenticationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AuthenticationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AuthenticationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AuthenticationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AuthenticationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AuthenticationErrorEnum} AuthenticationErrorEnum
                             */
                            AuthenticationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AuthenticationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AuthenticationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AuthenticationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AuthenticationErrorEnum} m AuthenticationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AuthenticationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AuthenticationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AuthenticationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AuthenticationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AuthenticationError enum.
                             * @name google.ads.googleads.v11.errors.AuthenticationErrorEnum.AuthenticationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} AUTHENTICATION_ERROR=2 AUTHENTICATION_ERROR value
                             * @property {number} CLIENT_CUSTOMER_ID_INVALID=5 CLIENT_CUSTOMER_ID_INVALID value
                             * @property {number} CUSTOMER_NOT_FOUND=8 CUSTOMER_NOT_FOUND value
                             * @property {number} GOOGLE_ACCOUNT_DELETED=9 GOOGLE_ACCOUNT_DELETED value
                             * @property {number} GOOGLE_ACCOUNT_COOKIE_INVALID=10 GOOGLE_ACCOUNT_COOKIE_INVALID value
                             * @property {number} GOOGLE_ACCOUNT_AUTHENTICATION_FAILED=25 GOOGLE_ACCOUNT_AUTHENTICATION_FAILED value
                             * @property {number} GOOGLE_ACCOUNT_USER_AND_ADS_USER_MISMATCH=12 GOOGLE_ACCOUNT_USER_AND_ADS_USER_MISMATCH value
                             * @property {number} LOGIN_COOKIE_REQUIRED=13 LOGIN_COOKIE_REQUIRED value
                             * @property {number} NOT_ADS_USER=14 NOT_ADS_USER value
                             * @property {number} OAUTH_TOKEN_INVALID=15 OAUTH_TOKEN_INVALID value
                             * @property {number} OAUTH_TOKEN_EXPIRED=16 OAUTH_TOKEN_EXPIRED value
                             * @property {number} OAUTH_TOKEN_DISABLED=17 OAUTH_TOKEN_DISABLED value
                             * @property {number} OAUTH_TOKEN_REVOKED=18 OAUTH_TOKEN_REVOKED value
                             * @property {number} OAUTH_TOKEN_HEADER_INVALID=19 OAUTH_TOKEN_HEADER_INVALID value
                             * @property {number} LOGIN_COOKIE_INVALID=20 LOGIN_COOKIE_INVALID value
                             * @property {number} USER_ID_INVALID=22 USER_ID_INVALID value
                             * @property {number} TWO_STEP_VERIFICATION_NOT_ENROLLED=23 TWO_STEP_VERIFICATION_NOT_ENROLLED value
                             * @property {number} ADVANCED_PROTECTION_NOT_ENROLLED=24 ADVANCED_PROTECTION_NOT_ENROLLED value
                             */
                            AuthenticationErrorEnum.AuthenticationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "AUTHENTICATION_ERROR"] = 2;
                                values[valuesById[5] = "CLIENT_CUSTOMER_ID_INVALID"] = 5;
                                values[valuesById[8] = "CUSTOMER_NOT_FOUND"] = 8;
                                values[valuesById[9] = "GOOGLE_ACCOUNT_DELETED"] = 9;
                                values[valuesById[10] = "GOOGLE_ACCOUNT_COOKIE_INVALID"] = 10;
                                values[valuesById[25] = "GOOGLE_ACCOUNT_AUTHENTICATION_FAILED"] = 25;
                                values[valuesById[12] = "GOOGLE_ACCOUNT_USER_AND_ADS_USER_MISMATCH"] = 12;
                                values[valuesById[13] = "LOGIN_COOKIE_REQUIRED"] = 13;
                                values[valuesById[14] = "NOT_ADS_USER"] = 14;
                                values[valuesById[15] = "OAUTH_TOKEN_INVALID"] = 15;
                                values[valuesById[16] = "OAUTH_TOKEN_EXPIRED"] = 16;
                                values[valuesById[17] = "OAUTH_TOKEN_DISABLED"] = 17;
                                values[valuesById[18] = "OAUTH_TOKEN_REVOKED"] = 18;
                                values[valuesById[19] = "OAUTH_TOKEN_HEADER_INVALID"] = 19;
                                values[valuesById[20] = "LOGIN_COOKIE_INVALID"] = 20;
                                values[valuesById[22] = "USER_ID_INVALID"] = 22;
                                values[valuesById[23] = "TWO_STEP_VERIFICATION_NOT_ENROLLED"] = 23;
                                values[valuesById[24] = "ADVANCED_PROTECTION_NOT_ENROLLED"] = 24;
                                return values;
                            })();
    
                            return AuthenticationErrorEnum;
                        })();
    
                        errors.AuthorizationErrorEnum = (function() {
    
                            /**
                             * Properties of an AuthorizationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IAuthorizationErrorEnum
                             */
    
                            /**
                             * Constructs a new AuthorizationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an AuthorizationErrorEnum.
                             * @implements IAuthorizationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IAuthorizationErrorEnum=} [p] Properties to set
                             */
                            function AuthorizationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new AuthorizationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAuthorizationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.AuthorizationErrorEnum} AuthorizationErrorEnum instance
                             */
                            AuthorizationErrorEnum.create = function create(properties) {
                                return new AuthorizationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified AuthorizationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.AuthorizationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAuthorizationErrorEnum} m AuthorizationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AuthorizationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AuthorizationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.AuthorizationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IAuthorizationErrorEnum} message AuthorizationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AuthorizationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AuthorizationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.AuthorizationErrorEnum} AuthorizationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AuthorizationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.AuthorizationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AuthorizationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.AuthorizationErrorEnum} AuthorizationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AuthorizationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AuthorizationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AuthorizationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an AuthorizationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.AuthorizationErrorEnum} AuthorizationErrorEnum
                             */
                            AuthorizationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.AuthorizationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.AuthorizationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an AuthorizationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.AuthorizationErrorEnum} m AuthorizationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AuthorizationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this AuthorizationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.AuthorizationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AuthorizationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * AuthorizationError enum.
                             * @name google.ads.googleads.v11.errors.AuthorizationErrorEnum.AuthorizationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} USER_PERMISSION_DENIED=2 USER_PERMISSION_DENIED value
                             * @property {number} DEVELOPER_TOKEN_NOT_ON_ALLOWLIST=13 DEVELOPER_TOKEN_NOT_ON_ALLOWLIST value
                             * @property {number} DEVELOPER_TOKEN_PROHIBITED=4 DEVELOPER_TOKEN_PROHIBITED value
                             * @property {number} PROJECT_DISABLED=5 PROJECT_DISABLED value
                             * @property {number} AUTHORIZATION_ERROR=6 AUTHORIZATION_ERROR value
                             * @property {number} ACTION_NOT_PERMITTED=7 ACTION_NOT_PERMITTED value
                             * @property {number} INCOMPLETE_SIGNUP=8 INCOMPLETE_SIGNUP value
                             * @property {number} CUSTOMER_NOT_ENABLED=24 CUSTOMER_NOT_ENABLED value
                             * @property {number} MISSING_TOS=9 MISSING_TOS value
                             * @property {number} DEVELOPER_TOKEN_NOT_APPROVED=10 DEVELOPER_TOKEN_NOT_APPROVED value
                             * @property {number} INVALID_LOGIN_CUSTOMER_ID_SERVING_CUSTOMER_ID_COMBINATION=11 INVALID_LOGIN_CUSTOMER_ID_SERVING_CUSTOMER_ID_COMBINATION value
                             * @property {number} SERVICE_ACCESS_DENIED=12 SERVICE_ACCESS_DENIED value
                             * @property {number} ACCESS_DENIED_FOR_ACCOUNT_TYPE=25 ACCESS_DENIED_FOR_ACCOUNT_TYPE value
                             * @property {number} METRIC_ACCESS_DENIED=26 METRIC_ACCESS_DENIED value
                             */
                            AuthorizationErrorEnum.AuthorizationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "USER_PERMISSION_DENIED"] = 2;
                                values[valuesById[13] = "DEVELOPER_TOKEN_NOT_ON_ALLOWLIST"] = 13;
                                values[valuesById[4] = "DEVELOPER_TOKEN_PROHIBITED"] = 4;
                                values[valuesById[5] = "PROJECT_DISABLED"] = 5;
                                values[valuesById[6] = "AUTHORIZATION_ERROR"] = 6;
                                values[valuesById[7] = "ACTION_NOT_PERMITTED"] = 7;
                                values[valuesById[8] = "INCOMPLETE_SIGNUP"] = 8;
                                values[valuesById[24] = "CUSTOMER_NOT_ENABLED"] = 24;
                                values[valuesById[9] = "MISSING_TOS"] = 9;
                                values[valuesById[10] = "DEVELOPER_TOKEN_NOT_APPROVED"] = 10;
                                values[valuesById[11] = "INVALID_LOGIN_CUSTOMER_ID_SERVING_CUSTOMER_ID_COMBINATION"] = 11;
                                values[valuesById[12] = "SERVICE_ACCESS_DENIED"] = 12;
                                values[valuesById[25] = "ACCESS_DENIED_FOR_ACCOUNT_TYPE"] = 25;
                                values[valuesById[26] = "METRIC_ACCESS_DENIED"] = 26;
                                return values;
                            })();
    
                            return AuthorizationErrorEnum;
                        })();
    
                        errors.BatchJobErrorEnum = (function() {
    
                            /**
                             * Properties of a BatchJobErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IBatchJobErrorEnum
                             */
    
                            /**
                             * Constructs a new BatchJobErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a BatchJobErrorEnum.
                             * @implements IBatchJobErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IBatchJobErrorEnum=} [p] Properties to set
                             */
                            function BatchJobErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BatchJobErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBatchJobErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.BatchJobErrorEnum} BatchJobErrorEnum instance
                             */
                            BatchJobErrorEnum.create = function create(properties) {
                                return new BatchJobErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BatchJobErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.BatchJobErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBatchJobErrorEnum} m BatchJobErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BatchJobErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.BatchJobErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBatchJobErrorEnum} message BatchJobErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BatchJobErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.BatchJobErrorEnum} BatchJobErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.BatchJobErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BatchJobErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.BatchJobErrorEnum} BatchJobErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BatchJobErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BatchJobErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BatchJobErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.BatchJobErrorEnum} BatchJobErrorEnum
                             */
                            BatchJobErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.BatchJobErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.BatchJobErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a BatchJobErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.BatchJobErrorEnum} m BatchJobErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BatchJobErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BatchJobErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.BatchJobErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BatchJobErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BatchJobError enum.
                             * @name google.ads.googleads.v11.errors.BatchJobErrorEnum.BatchJobError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_MODIFY_JOB_AFTER_JOB_STARTS_RUNNING=2 CANNOT_MODIFY_JOB_AFTER_JOB_STARTS_RUNNING value
                             * @property {number} EMPTY_OPERATIONS=3 EMPTY_OPERATIONS value
                             * @property {number} INVALID_SEQUENCE_TOKEN=4 INVALID_SEQUENCE_TOKEN value
                             * @property {number} RESULTS_NOT_READY=5 RESULTS_NOT_READY value
                             * @property {number} INVALID_PAGE_SIZE=6 INVALID_PAGE_SIZE value
                             * @property {number} CAN_ONLY_REMOVE_PENDING_JOB=7 CAN_ONLY_REMOVE_PENDING_JOB value
                             */
                            BatchJobErrorEnum.BatchJobError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_MODIFY_JOB_AFTER_JOB_STARTS_RUNNING"] = 2;
                                values[valuesById[3] = "EMPTY_OPERATIONS"] = 3;
                                values[valuesById[4] = "INVALID_SEQUENCE_TOKEN"] = 4;
                                values[valuesById[5] = "RESULTS_NOT_READY"] = 5;
                                values[valuesById[6] = "INVALID_PAGE_SIZE"] = 6;
                                values[valuesById[7] = "CAN_ONLY_REMOVE_PENDING_JOB"] = 7;
                                return values;
                            })();
    
                            return BatchJobErrorEnum;
                        })();
    
                        errors.BiddingErrorEnum = (function() {
    
                            /**
                             * Properties of a BiddingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IBiddingErrorEnum
                             */
    
                            /**
                             * Constructs a new BiddingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a BiddingErrorEnum.
                             * @implements IBiddingErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IBiddingErrorEnum=} [p] Properties to set
                             */
                            function BiddingErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BiddingErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBiddingErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.BiddingErrorEnum} BiddingErrorEnum instance
                             */
                            BiddingErrorEnum.create = function create(properties) {
                                return new BiddingErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.BiddingErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBiddingErrorEnum} m BiddingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.BiddingErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBiddingErrorEnum} message BiddingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.BiddingErrorEnum} BiddingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.BiddingErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.BiddingErrorEnum} BiddingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BiddingErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.BiddingErrorEnum} BiddingErrorEnum
                             */
                            BiddingErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.BiddingErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.BiddingErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a BiddingErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.BiddingErrorEnum} m BiddingErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BiddingErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.BiddingErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BiddingError enum.
                             * @name google.ads.googleads.v11.errors.BiddingErrorEnum.BiddingError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BIDDING_STRATEGY_TRANSITION_NOT_ALLOWED=2 BIDDING_STRATEGY_TRANSITION_NOT_ALLOWED value
                             * @property {number} CANNOT_ATTACH_BIDDING_STRATEGY_TO_CAMPAIGN=7 CANNOT_ATTACH_BIDDING_STRATEGY_TO_CAMPAIGN value
                             * @property {number} INVALID_ANONYMOUS_BIDDING_STRATEGY_TYPE=10 INVALID_ANONYMOUS_BIDDING_STRATEGY_TYPE value
                             * @property {number} INVALID_BIDDING_STRATEGY_TYPE=14 INVALID_BIDDING_STRATEGY_TYPE value
                             * @property {number} INVALID_BID=17 INVALID_BID value
                             * @property {number} BIDDING_STRATEGY_NOT_AVAILABLE_FOR_ACCOUNT_TYPE=18 BIDDING_STRATEGY_NOT_AVAILABLE_FOR_ACCOUNT_TYPE value
                             * @property {number} CONVERSION_TRACKING_NOT_ENABLED=19 CONVERSION_TRACKING_NOT_ENABLED value
                             * @property {number} NOT_ENOUGH_CONVERSIONS=20 NOT_ENOUGH_CONVERSIONS value
                             * @property {number} CANNOT_CREATE_CAMPAIGN_WITH_BIDDING_STRATEGY=21 CANNOT_CREATE_CAMPAIGN_WITH_BIDDING_STRATEGY value
                             * @property {number} CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CAMPAIGN_LEVEL_POP_BIDDING_STRATEGY=23 CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CAMPAIGN_LEVEL_POP_BIDDING_STRATEGY value
                             * @property {number} BIDDING_STRATEGY_NOT_SUPPORTED_WITH_AD_SCHEDULE=24 BIDDING_STRATEGY_NOT_SUPPORTED_WITH_AD_SCHEDULE value
                             * @property {number} PAY_PER_CONVERSION_NOT_AVAILABLE_FOR_CUSTOMER=25 PAY_PER_CONVERSION_NOT_AVAILABLE_FOR_CUSTOMER value
                             * @property {number} PAY_PER_CONVERSION_NOT_ALLOWED_WITH_TARGET_CPA=26 PAY_PER_CONVERSION_NOT_ALLOWED_WITH_TARGET_CPA value
                             * @property {number} BIDDING_STRATEGY_NOT_ALLOWED_FOR_SEARCH_ONLY_CAMPAIGNS=27 BIDDING_STRATEGY_NOT_ALLOWED_FOR_SEARCH_ONLY_CAMPAIGNS value
                             * @property {number} BIDDING_STRATEGY_NOT_SUPPORTED_IN_DRAFTS_OR_EXPERIMENTS=28 BIDDING_STRATEGY_NOT_SUPPORTED_IN_DRAFTS_OR_EXPERIMENTS value
                             * @property {number} BIDDING_STRATEGY_TYPE_DOES_NOT_SUPPORT_PRODUCT_TYPE_ADGROUP_CRITERION=29 BIDDING_STRATEGY_TYPE_DOES_NOT_SUPPORT_PRODUCT_TYPE_ADGROUP_CRITERION value
                             * @property {number} BID_TOO_SMALL=30 BID_TOO_SMALL value
                             * @property {number} BID_TOO_BIG=31 BID_TOO_BIG value
                             * @property {number} BID_TOO_MANY_FRACTIONAL_DIGITS=32 BID_TOO_MANY_FRACTIONAL_DIGITS value
                             * @property {number} INVALID_DOMAIN_NAME=33 INVALID_DOMAIN_NAME value
                             * @property {number} NOT_COMPATIBLE_WITH_PAYMENT_MODE=34 NOT_COMPATIBLE_WITH_PAYMENT_MODE value
                             * @property {number} NOT_COMPATIBLE_WITH_BUDGET_TYPE=35 NOT_COMPATIBLE_WITH_BUDGET_TYPE value
                             * @property {number} NOT_COMPATIBLE_WITH_BIDDING_STRATEGY_TYPE=36 NOT_COMPATIBLE_WITH_BIDDING_STRATEGY_TYPE value
                             * @property {number} BIDDING_STRATEGY_TYPE_INCOMPATIBLE_WITH_SHARED_BUDGET=37 BIDDING_STRATEGY_TYPE_INCOMPATIBLE_WITH_SHARED_BUDGET value
                             * @property {number} BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ALIGNED=38 BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ALIGNED value
                             * @property {number} BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ATTACHED_TO_THE_SAME_CAMPAIGNS_TO_ALIGN=39 BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ATTACHED_TO_THE_SAME_CAMPAIGNS_TO_ALIGN value
                             * @property {number} BIDDING_STRATEGY_AND_BUDGET_MUST_BE_REMOVED_TOGETHER=40 BIDDING_STRATEGY_AND_BUDGET_MUST_BE_REMOVED_TOGETHER value
                             */
                            BiddingErrorEnum.BiddingError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BIDDING_STRATEGY_TRANSITION_NOT_ALLOWED"] = 2;
                                values[valuesById[7] = "CANNOT_ATTACH_BIDDING_STRATEGY_TO_CAMPAIGN"] = 7;
                                values[valuesById[10] = "INVALID_ANONYMOUS_BIDDING_STRATEGY_TYPE"] = 10;
                                values[valuesById[14] = "INVALID_BIDDING_STRATEGY_TYPE"] = 14;
                                values[valuesById[17] = "INVALID_BID"] = 17;
                                values[valuesById[18] = "BIDDING_STRATEGY_NOT_AVAILABLE_FOR_ACCOUNT_TYPE"] = 18;
                                values[valuesById[19] = "CONVERSION_TRACKING_NOT_ENABLED"] = 19;
                                values[valuesById[20] = "NOT_ENOUGH_CONVERSIONS"] = 20;
                                values[valuesById[21] = "CANNOT_CREATE_CAMPAIGN_WITH_BIDDING_STRATEGY"] = 21;
                                values[valuesById[23] = "CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CAMPAIGN_LEVEL_POP_BIDDING_STRATEGY"] = 23;
                                values[valuesById[24] = "BIDDING_STRATEGY_NOT_SUPPORTED_WITH_AD_SCHEDULE"] = 24;
                                values[valuesById[25] = "PAY_PER_CONVERSION_NOT_AVAILABLE_FOR_CUSTOMER"] = 25;
                                values[valuesById[26] = "PAY_PER_CONVERSION_NOT_ALLOWED_WITH_TARGET_CPA"] = 26;
                                values[valuesById[27] = "BIDDING_STRATEGY_NOT_ALLOWED_FOR_SEARCH_ONLY_CAMPAIGNS"] = 27;
                                values[valuesById[28] = "BIDDING_STRATEGY_NOT_SUPPORTED_IN_DRAFTS_OR_EXPERIMENTS"] = 28;
                                values[valuesById[29] = "BIDDING_STRATEGY_TYPE_DOES_NOT_SUPPORT_PRODUCT_TYPE_ADGROUP_CRITERION"] = 29;
                                values[valuesById[30] = "BID_TOO_SMALL"] = 30;
                                values[valuesById[31] = "BID_TOO_BIG"] = 31;
                                values[valuesById[32] = "BID_TOO_MANY_FRACTIONAL_DIGITS"] = 32;
                                values[valuesById[33] = "INVALID_DOMAIN_NAME"] = 33;
                                values[valuesById[34] = "NOT_COMPATIBLE_WITH_PAYMENT_MODE"] = 34;
                                values[valuesById[35] = "NOT_COMPATIBLE_WITH_BUDGET_TYPE"] = 35;
                                values[valuesById[36] = "NOT_COMPATIBLE_WITH_BIDDING_STRATEGY_TYPE"] = 36;
                                values[valuesById[37] = "BIDDING_STRATEGY_TYPE_INCOMPATIBLE_WITH_SHARED_BUDGET"] = 37;
                                values[valuesById[38] = "BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ALIGNED"] = 38;
                                values[valuesById[39] = "BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ATTACHED_TO_THE_SAME_CAMPAIGNS_TO_ALIGN"] = 39;
                                values[valuesById[40] = "BIDDING_STRATEGY_AND_BUDGET_MUST_BE_REMOVED_TOGETHER"] = 40;
                                return values;
                            })();
    
                            return BiddingErrorEnum;
                        })();
    
                        errors.BiddingStrategyErrorEnum = (function() {
    
                            /**
                             * Properties of a BiddingStrategyErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IBiddingStrategyErrorEnum
                             */
    
                            /**
                             * Constructs a new BiddingStrategyErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a BiddingStrategyErrorEnum.
                             * @implements IBiddingStrategyErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IBiddingStrategyErrorEnum=} [p] Properties to set
                             */
                            function BiddingStrategyErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BiddingStrategyErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBiddingStrategyErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.BiddingStrategyErrorEnum} BiddingStrategyErrorEnum instance
                             */
                            BiddingStrategyErrorEnum.create = function create(properties) {
                                return new BiddingStrategyErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.BiddingStrategyErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBiddingStrategyErrorEnum} m BiddingStrategyErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.BiddingStrategyErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBiddingStrategyErrorEnum} message BiddingStrategyErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingStrategyErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.BiddingStrategyErrorEnum} BiddingStrategyErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.BiddingStrategyErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingStrategyErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.BiddingStrategyErrorEnum} BiddingStrategyErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingStrategyErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingStrategyErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BiddingStrategyErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.BiddingStrategyErrorEnum} BiddingStrategyErrorEnum
                             */
                            BiddingStrategyErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.BiddingStrategyErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.BiddingStrategyErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a BiddingStrategyErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.BiddingStrategyErrorEnum} m BiddingStrategyErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingStrategyErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BiddingStrategyErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.BiddingStrategyErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingStrategyErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BiddingStrategyError enum.
                             * @name google.ads.googleads.v11.errors.BiddingStrategyErrorEnum.BiddingStrategyError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_NAME=2 DUPLICATE_NAME value
                             * @property {number} CANNOT_CHANGE_BIDDING_STRATEGY_TYPE=3 CANNOT_CHANGE_BIDDING_STRATEGY_TYPE value
                             * @property {number} CANNOT_REMOVE_ASSOCIATED_STRATEGY=4 CANNOT_REMOVE_ASSOCIATED_STRATEGY value
                             * @property {number} BIDDING_STRATEGY_NOT_SUPPORTED=5 BIDDING_STRATEGY_NOT_SUPPORTED value
                             * @property {number} INCOMPATIBLE_BIDDING_STRATEGY_AND_BIDDING_STRATEGY_GOAL_TYPE=6 INCOMPATIBLE_BIDDING_STRATEGY_AND_BIDDING_STRATEGY_GOAL_TYPE value
                             */
                            BiddingStrategyErrorEnum.BiddingStrategyError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_NAME"] = 2;
                                values[valuesById[3] = "CANNOT_CHANGE_BIDDING_STRATEGY_TYPE"] = 3;
                                values[valuesById[4] = "CANNOT_REMOVE_ASSOCIATED_STRATEGY"] = 4;
                                values[valuesById[5] = "BIDDING_STRATEGY_NOT_SUPPORTED"] = 5;
                                values[valuesById[6] = "INCOMPATIBLE_BIDDING_STRATEGY_AND_BIDDING_STRATEGY_GOAL_TYPE"] = 6;
                                return values;
                            })();
    
                            return BiddingStrategyErrorEnum;
                        })();
    
                        errors.BillingSetupErrorEnum = (function() {
    
                            /**
                             * Properties of a BillingSetupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IBillingSetupErrorEnum
                             */
    
                            /**
                             * Constructs a new BillingSetupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a BillingSetupErrorEnum.
                             * @implements IBillingSetupErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IBillingSetupErrorEnum=} [p] Properties to set
                             */
                            function BillingSetupErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new BillingSetupErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBillingSetupErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.BillingSetupErrorEnum} BillingSetupErrorEnum instance
                             */
                            BillingSetupErrorEnum.create = function create(properties) {
                                return new BillingSetupErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified BillingSetupErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.BillingSetupErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBillingSetupErrorEnum} m BillingSetupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetupErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BillingSetupErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.BillingSetupErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IBillingSetupErrorEnum} message BillingSetupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetupErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BillingSetupErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.BillingSetupErrorEnum} BillingSetupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetupErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.BillingSetupErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BillingSetupErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.BillingSetupErrorEnum} BillingSetupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetupErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BillingSetupErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BillingSetupErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a BillingSetupErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.BillingSetupErrorEnum} BillingSetupErrorEnum
                             */
                            BillingSetupErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.BillingSetupErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.BillingSetupErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a BillingSetupErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.BillingSetupErrorEnum} m BillingSetupErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BillingSetupErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this BillingSetupErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.BillingSetupErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BillingSetupErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * BillingSetupError enum.
                             * @name google.ads.googleads.v11.errors.BillingSetupErrorEnum.BillingSetupError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_USE_EXISTING_AND_NEW_ACCOUNT=2 CANNOT_USE_EXISTING_AND_NEW_ACCOUNT value
                             * @property {number} CANNOT_REMOVE_STARTED_BILLING_SETUP=3 CANNOT_REMOVE_STARTED_BILLING_SETUP value
                             * @property {number} CANNOT_CHANGE_BILLING_TO_SAME_PAYMENTS_ACCOUNT=4 CANNOT_CHANGE_BILLING_TO_SAME_PAYMENTS_ACCOUNT value
                             * @property {number} BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_STATUS=5 BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_STATUS value
                             * @property {number} INVALID_PAYMENTS_ACCOUNT=6 INVALID_PAYMENTS_ACCOUNT value
                             * @property {number} BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_CATEGORY=7 BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_CATEGORY value
                             * @property {number} INVALID_START_TIME_TYPE=8 INVALID_START_TIME_TYPE value
                             * @property {number} THIRD_PARTY_ALREADY_HAS_BILLING=9 THIRD_PARTY_ALREADY_HAS_BILLING value
                             * @property {number} BILLING_SETUP_IN_PROGRESS=10 BILLING_SETUP_IN_PROGRESS value
                             * @property {number} NO_SIGNUP_PERMISSION=11 NO_SIGNUP_PERMISSION value
                             * @property {number} CHANGE_OF_BILL_TO_IN_PROGRESS=12 CHANGE_OF_BILL_TO_IN_PROGRESS value
                             * @property {number} PAYMENTS_PROFILE_NOT_FOUND=13 PAYMENTS_PROFILE_NOT_FOUND value
                             * @property {number} PAYMENTS_ACCOUNT_NOT_FOUND=14 PAYMENTS_ACCOUNT_NOT_FOUND value
                             * @property {number} PAYMENTS_PROFILE_INELIGIBLE=15 PAYMENTS_PROFILE_INELIGIBLE value
                             * @property {number} PAYMENTS_ACCOUNT_INELIGIBLE=16 PAYMENTS_ACCOUNT_INELIGIBLE value
                             * @property {number} CUSTOMER_NEEDS_INTERNAL_APPROVAL=17 CUSTOMER_NEEDS_INTERNAL_APPROVAL value
                             * @property {number} PAYMENTS_ACCOUNT_INELIGIBLE_CURRENCY_CODE_MISMATCH=19 PAYMENTS_ACCOUNT_INELIGIBLE_CURRENCY_CODE_MISMATCH value
                             * @property {number} FUTURE_START_TIME_PROHIBITED=20 FUTURE_START_TIME_PROHIBITED value
                             */
                            BillingSetupErrorEnum.BillingSetupError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_USE_EXISTING_AND_NEW_ACCOUNT"] = 2;
                                values[valuesById[3] = "CANNOT_REMOVE_STARTED_BILLING_SETUP"] = 3;
                                values[valuesById[4] = "CANNOT_CHANGE_BILLING_TO_SAME_PAYMENTS_ACCOUNT"] = 4;
                                values[valuesById[5] = "BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_STATUS"] = 5;
                                values[valuesById[6] = "INVALID_PAYMENTS_ACCOUNT"] = 6;
                                values[valuesById[7] = "BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_CATEGORY"] = 7;
                                values[valuesById[8] = "INVALID_START_TIME_TYPE"] = 8;
                                values[valuesById[9] = "THIRD_PARTY_ALREADY_HAS_BILLING"] = 9;
                                values[valuesById[10] = "BILLING_SETUP_IN_PROGRESS"] = 10;
                                values[valuesById[11] = "NO_SIGNUP_PERMISSION"] = 11;
                                values[valuesById[12] = "CHANGE_OF_BILL_TO_IN_PROGRESS"] = 12;
                                values[valuesById[13] = "PAYMENTS_PROFILE_NOT_FOUND"] = 13;
                                values[valuesById[14] = "PAYMENTS_ACCOUNT_NOT_FOUND"] = 14;
                                values[valuesById[15] = "PAYMENTS_PROFILE_INELIGIBLE"] = 15;
                                values[valuesById[16] = "PAYMENTS_ACCOUNT_INELIGIBLE"] = 16;
                                values[valuesById[17] = "CUSTOMER_NEEDS_INTERNAL_APPROVAL"] = 17;
                                values[valuesById[19] = "PAYMENTS_ACCOUNT_INELIGIBLE_CURRENCY_CODE_MISMATCH"] = 19;
                                values[valuesById[20] = "FUTURE_START_TIME_PROHIBITED"] = 20;
                                return values;
                            })();
    
                            return BillingSetupErrorEnum;
                        })();
    
                        errors.CampaignBudgetErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignBudgetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignBudgetErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignBudgetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignBudgetErrorEnum.
                             * @implements ICampaignBudgetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignBudgetErrorEnum=} [p] Properties to set
                             */
                            function CampaignBudgetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignBudgetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignBudgetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignBudgetErrorEnum} CampaignBudgetErrorEnum instance
                             */
                            CampaignBudgetErrorEnum.create = function create(properties) {
                                return new CampaignBudgetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignBudgetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignBudgetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignBudgetErrorEnum} m CampaignBudgetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudgetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignBudgetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignBudgetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignBudgetErrorEnum} message CampaignBudgetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudgetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignBudgetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignBudgetErrorEnum} CampaignBudgetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudgetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignBudgetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignBudgetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignBudgetErrorEnum} CampaignBudgetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudgetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignBudgetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignBudgetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignBudgetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignBudgetErrorEnum} CampaignBudgetErrorEnum
                             */
                            CampaignBudgetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignBudgetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignBudgetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignBudgetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignBudgetErrorEnum} m CampaignBudgetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignBudgetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignBudgetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignBudgetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignBudgetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignBudgetError enum.
                             * @name google.ads.googleads.v11.errors.CampaignBudgetErrorEnum.CampaignBudgetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CAMPAIGN_BUDGET_CANNOT_BE_SHARED=17 CAMPAIGN_BUDGET_CANNOT_BE_SHARED value
                             * @property {number} CAMPAIGN_BUDGET_REMOVED=2 CAMPAIGN_BUDGET_REMOVED value
                             * @property {number} CAMPAIGN_BUDGET_IN_USE=3 CAMPAIGN_BUDGET_IN_USE value
                             * @property {number} CAMPAIGN_BUDGET_PERIOD_NOT_AVAILABLE=4 CAMPAIGN_BUDGET_PERIOD_NOT_AVAILABLE value
                             * @property {number} CANNOT_MODIFY_FIELD_OF_IMPLICITLY_SHARED_CAMPAIGN_BUDGET=6 CANNOT_MODIFY_FIELD_OF_IMPLICITLY_SHARED_CAMPAIGN_BUDGET value
                             * @property {number} CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_IMPLICITLY_SHARED=7 CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_IMPLICITLY_SHARED value
                             * @property {number} CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED_WITHOUT_NAME=8 CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED_WITHOUT_NAME value
                             * @property {number} CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED=9 CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED value
                             * @property {number} CANNOT_USE_IMPLICITLY_SHARED_CAMPAIGN_BUDGET_WITH_MULTIPLE_CAMPAIGNS=10 CANNOT_USE_IMPLICITLY_SHARED_CAMPAIGN_BUDGET_WITH_MULTIPLE_CAMPAIGNS value
                             * @property {number} DUPLICATE_NAME=11 DUPLICATE_NAME value
                             * @property {number} MONEY_AMOUNT_IN_WRONG_CURRENCY=12 MONEY_AMOUNT_IN_WRONG_CURRENCY value
                             * @property {number} MONEY_AMOUNT_LESS_THAN_CURRENCY_MINIMUM_CPC=13 MONEY_AMOUNT_LESS_THAN_CURRENCY_MINIMUM_CPC value
                             * @property {number} MONEY_AMOUNT_TOO_LARGE=14 MONEY_AMOUNT_TOO_LARGE value
                             * @property {number} NEGATIVE_MONEY_AMOUNT=15 NEGATIVE_MONEY_AMOUNT value
                             * @property {number} NON_MULTIPLE_OF_MINIMUM_CURRENCY_UNIT=16 NON_MULTIPLE_OF_MINIMUM_CURRENCY_UNIT value
                             * @property {number} TOTAL_BUDGET_AMOUNT_MUST_BE_UNSET_FOR_BUDGET_PERIOD_DAILY=18 TOTAL_BUDGET_AMOUNT_MUST_BE_UNSET_FOR_BUDGET_PERIOD_DAILY value
                             * @property {number} INVALID_PERIOD=19 INVALID_PERIOD value
                             */
                            CampaignBudgetErrorEnum.CampaignBudgetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[17] = "CAMPAIGN_BUDGET_CANNOT_BE_SHARED"] = 17;
                                values[valuesById[2] = "CAMPAIGN_BUDGET_REMOVED"] = 2;
                                values[valuesById[3] = "CAMPAIGN_BUDGET_IN_USE"] = 3;
                                values[valuesById[4] = "CAMPAIGN_BUDGET_PERIOD_NOT_AVAILABLE"] = 4;
                                values[valuesById[6] = "CANNOT_MODIFY_FIELD_OF_IMPLICITLY_SHARED_CAMPAIGN_BUDGET"] = 6;
                                values[valuesById[7] = "CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_IMPLICITLY_SHARED"] = 7;
                                values[valuesById[8] = "CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED_WITHOUT_NAME"] = 8;
                                values[valuesById[9] = "CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED"] = 9;
                                values[valuesById[10] = "CANNOT_USE_IMPLICITLY_SHARED_CAMPAIGN_BUDGET_WITH_MULTIPLE_CAMPAIGNS"] = 10;
                                values[valuesById[11] = "DUPLICATE_NAME"] = 11;
                                values[valuesById[12] = "MONEY_AMOUNT_IN_WRONG_CURRENCY"] = 12;
                                values[valuesById[13] = "MONEY_AMOUNT_LESS_THAN_CURRENCY_MINIMUM_CPC"] = 13;
                                values[valuesById[14] = "MONEY_AMOUNT_TOO_LARGE"] = 14;
                                values[valuesById[15] = "NEGATIVE_MONEY_AMOUNT"] = 15;
                                values[valuesById[16] = "NON_MULTIPLE_OF_MINIMUM_CURRENCY_UNIT"] = 16;
                                values[valuesById[18] = "TOTAL_BUDGET_AMOUNT_MUST_BE_UNSET_FOR_BUDGET_PERIOD_DAILY"] = 18;
                                values[valuesById[19] = "INVALID_PERIOD"] = 19;
                                return values;
                            })();
    
                            return CampaignBudgetErrorEnum;
                        })();
    
                        errors.CampaignConversionGoalErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignConversionGoalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignConversionGoalErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignConversionGoalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignConversionGoalErrorEnum.
                             * @implements ICampaignConversionGoalErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignConversionGoalErrorEnum=} [p] Properties to set
                             */
                            function CampaignConversionGoalErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignConversionGoalErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignConversionGoalErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum} CampaignConversionGoalErrorEnum instance
                             */
                            CampaignConversionGoalErrorEnum.create = function create(properties) {
                                return new CampaignConversionGoalErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignConversionGoalErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignConversionGoalErrorEnum} m CampaignConversionGoalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignConversionGoalErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignConversionGoalErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignConversionGoalErrorEnum} message CampaignConversionGoalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignConversionGoalErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignConversionGoalErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum} CampaignConversionGoalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignConversionGoalErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignConversionGoalErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum} CampaignConversionGoalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignConversionGoalErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignConversionGoalErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignConversionGoalErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignConversionGoalErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum} CampaignConversionGoalErrorEnum
                             */
                            CampaignConversionGoalErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignConversionGoalErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum} m CampaignConversionGoalErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignConversionGoalErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignConversionGoalErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignConversionGoalErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignConversionGoalError enum.
                             * @name google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum.CampaignConversionGoalError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN=2 CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN value
                             */
                            CampaignConversionGoalErrorEnum.CampaignConversionGoalError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN"] = 2;
                                return values;
                            })();
    
                            return CampaignConversionGoalErrorEnum;
                        })();
    
                        errors.CampaignCriterionErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignCriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignCriterionErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignCriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignCriterionErrorEnum.
                             * @implements ICampaignCriterionErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignCriterionErrorEnum=} [p] Properties to set
                             */
                            function CampaignCriterionErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignCriterionErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignCriterionErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignCriterionErrorEnum} CampaignCriterionErrorEnum instance
                             */
                            CampaignCriterionErrorEnum.create = function create(properties) {
                                return new CampaignCriterionErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignCriterionErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignCriterionErrorEnum} m CampaignCriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignCriterionErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignCriterionErrorEnum} message CampaignCriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCriterionErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignCriterionErrorEnum} CampaignCriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignCriterionErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCriterionErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignCriterionErrorEnum} CampaignCriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCriterionErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCriterionErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCriterionErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignCriterionErrorEnum} CampaignCriterionErrorEnum
                             */
                            CampaignCriterionErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignCriterionErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignCriterionErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignCriterionErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignCriterionErrorEnum} m CampaignCriterionErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCriterionErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignCriterionErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignCriterionErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCriterionErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignCriterionError enum.
                             * @name google.ads.googleads.v11.errors.CampaignCriterionErrorEnum.CampaignCriterionError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CONCRETE_TYPE_REQUIRED=2 CONCRETE_TYPE_REQUIRED value
                             * @property {number} INVALID_PLACEMENT_URL=3 INVALID_PLACEMENT_URL value
                             * @property {number} CANNOT_EXCLUDE_CRITERIA_TYPE=4 CANNOT_EXCLUDE_CRITERIA_TYPE value
                             * @property {number} CANNOT_SET_STATUS_FOR_CRITERIA_TYPE=5 CANNOT_SET_STATUS_FOR_CRITERIA_TYPE value
                             * @property {number} CANNOT_SET_STATUS_FOR_EXCLUDED_CRITERIA=6 CANNOT_SET_STATUS_FOR_EXCLUDED_CRITERIA value
                             * @property {number} CANNOT_TARGET_AND_EXCLUDE=7 CANNOT_TARGET_AND_EXCLUDE value
                             * @property {number} TOO_MANY_OPERATIONS=8 TOO_MANY_OPERATIONS value
                             * @property {number} OPERATOR_NOT_SUPPORTED_FOR_CRITERION_TYPE=9 OPERATOR_NOT_SUPPORTED_FOR_CRITERION_TYPE value
                             * @property {number} SHOPPING_CAMPAIGN_SALES_COUNTRY_NOT_SUPPORTED_FOR_SALES_CHANNEL=10 SHOPPING_CAMPAIGN_SALES_COUNTRY_NOT_SUPPORTED_FOR_SALES_CHANNEL value
                             * @property {number} CANNOT_ADD_EXISTING_FIELD=11 CANNOT_ADD_EXISTING_FIELD value
                             * @property {number} CANNOT_UPDATE_NEGATIVE_CRITERION=12 CANNOT_UPDATE_NEGATIVE_CRITERION value
                             * @property {number} CANNOT_SET_NEGATIVE_KEYWORD_THEME_CONSTANT_CRITERION=13 CANNOT_SET_NEGATIVE_KEYWORD_THEME_CONSTANT_CRITERION value
                             * @property {number} INVALID_KEYWORD_THEME_CONSTANT=14 INVALID_KEYWORD_THEME_CONSTANT value
                             * @property {number} MISSING_KEYWORD_THEME_CONSTANT_OR_FREE_FORM_KEYWORD_THEME=15 MISSING_KEYWORD_THEME_CONSTANT_OR_FREE_FORM_KEYWORD_THEME value
                             * @property {number} CANNOT_TARGET_BOTH_PROXIMITY_AND_LOCATION_CRITERIA_FOR_SMART_CAMPAIGN=16 CANNOT_TARGET_BOTH_PROXIMITY_AND_LOCATION_CRITERIA_FOR_SMART_CAMPAIGN value
                             * @property {number} CANNOT_TARGET_MULTIPLE_PROXIMITY_CRITERIA_FOR_SMART_CAMPAIGN=17 CANNOT_TARGET_MULTIPLE_PROXIMITY_CRITERIA_FOR_SMART_CAMPAIGN value
                             */
                            CampaignCriterionErrorEnum.CampaignCriterionError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CONCRETE_TYPE_REQUIRED"] = 2;
                                values[valuesById[3] = "INVALID_PLACEMENT_URL"] = 3;
                                values[valuesById[4] = "CANNOT_EXCLUDE_CRITERIA_TYPE"] = 4;
                                values[valuesById[5] = "CANNOT_SET_STATUS_FOR_CRITERIA_TYPE"] = 5;
                                values[valuesById[6] = "CANNOT_SET_STATUS_FOR_EXCLUDED_CRITERIA"] = 6;
                                values[valuesById[7] = "CANNOT_TARGET_AND_EXCLUDE"] = 7;
                                values[valuesById[8] = "TOO_MANY_OPERATIONS"] = 8;
                                values[valuesById[9] = "OPERATOR_NOT_SUPPORTED_FOR_CRITERION_TYPE"] = 9;
                                values[valuesById[10] = "SHOPPING_CAMPAIGN_SALES_COUNTRY_NOT_SUPPORTED_FOR_SALES_CHANNEL"] = 10;
                                values[valuesById[11] = "CANNOT_ADD_EXISTING_FIELD"] = 11;
                                values[valuesById[12] = "CANNOT_UPDATE_NEGATIVE_CRITERION"] = 12;
                                values[valuesById[13] = "CANNOT_SET_NEGATIVE_KEYWORD_THEME_CONSTANT_CRITERION"] = 13;
                                values[valuesById[14] = "INVALID_KEYWORD_THEME_CONSTANT"] = 14;
                                values[valuesById[15] = "MISSING_KEYWORD_THEME_CONSTANT_OR_FREE_FORM_KEYWORD_THEME"] = 15;
                                values[valuesById[16] = "CANNOT_TARGET_BOTH_PROXIMITY_AND_LOCATION_CRITERIA_FOR_SMART_CAMPAIGN"] = 16;
                                values[valuesById[17] = "CANNOT_TARGET_MULTIPLE_PROXIMITY_CRITERIA_FOR_SMART_CAMPAIGN"] = 17;
                                return values;
                            })();
    
                            return CampaignCriterionErrorEnum;
                        })();
    
                        errors.CampaignCustomizerErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignCustomizerErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignCustomizerErrorEnum.
                             * @implements ICampaignCustomizerErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignCustomizerErrorEnum=} [p] Properties to set
                             */
                            function CampaignCustomizerErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignCustomizerErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignCustomizerErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum} CampaignCustomizerErrorEnum instance
                             */
                            CampaignCustomizerErrorEnum.create = function create(properties) {
                                return new CampaignCustomizerErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCustomizerErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignCustomizerErrorEnum} m CampaignCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCustomizerErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCustomizerErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignCustomizerErrorEnum} message CampaignCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCustomizerErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCustomizerErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum} CampaignCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCustomizerErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCustomizerErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum} CampaignCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCustomizerErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCustomizerErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCustomizerErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCustomizerErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum} CampaignCustomizerErrorEnum
                             */
                            CampaignCustomizerErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignCustomizerErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum} m CampaignCustomizerErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCustomizerErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignCustomizerErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCustomizerErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignCustomizerError enum.
                             * @name google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum.CampaignCustomizerError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             */
                            CampaignCustomizerErrorEnum.CampaignCustomizerError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                return values;
                            })();
    
                            return CampaignCustomizerErrorEnum;
                        })();
    
                        errors.CampaignDraftErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignDraftErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignDraftErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignDraftErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignDraftErrorEnum.
                             * @implements ICampaignDraftErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignDraftErrorEnum=} [p] Properties to set
                             */
                            function CampaignDraftErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignDraftErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignDraftErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignDraftErrorEnum} CampaignDraftErrorEnum instance
                             */
                            CampaignDraftErrorEnum.create = function create(properties) {
                                return new CampaignDraftErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignDraftErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignDraftErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignDraftErrorEnum} m CampaignDraftErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraftErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignDraftErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignDraftErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignDraftErrorEnum} message CampaignDraftErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraftErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignDraftErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignDraftErrorEnum} CampaignDraftErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraftErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignDraftErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignDraftErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignDraftErrorEnum} CampaignDraftErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraftErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignDraftErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignDraftErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignDraftErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignDraftErrorEnum} CampaignDraftErrorEnum
                             */
                            CampaignDraftErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignDraftErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignDraftErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignDraftErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignDraftErrorEnum} m CampaignDraftErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignDraftErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignDraftErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignDraftErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignDraftErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignDraftError enum.
                             * @name google.ads.googleads.v11.errors.CampaignDraftErrorEnum.CampaignDraftError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_DRAFT_NAME=2 DUPLICATE_DRAFT_NAME value
                             * @property {number} INVALID_STATUS_TRANSITION_FROM_REMOVED=3 INVALID_STATUS_TRANSITION_FROM_REMOVED value
                             * @property {number} INVALID_STATUS_TRANSITION_FROM_PROMOTED=4 INVALID_STATUS_TRANSITION_FROM_PROMOTED value
                             * @property {number} INVALID_STATUS_TRANSITION_FROM_PROMOTE_FAILED=5 INVALID_STATUS_TRANSITION_FROM_PROMOTE_FAILED value
                             * @property {number} CUSTOMER_CANNOT_CREATE_DRAFT=6 CUSTOMER_CANNOT_CREATE_DRAFT value
                             * @property {number} CAMPAIGN_CANNOT_CREATE_DRAFT=7 CAMPAIGN_CANNOT_CREATE_DRAFT value
                             * @property {number} INVALID_DRAFT_CHANGE=8 INVALID_DRAFT_CHANGE value
                             * @property {number} INVALID_STATUS_TRANSITION=9 INVALID_STATUS_TRANSITION value
                             * @property {number} MAX_NUMBER_OF_DRAFTS_PER_CAMPAIGN_REACHED=10 MAX_NUMBER_OF_DRAFTS_PER_CAMPAIGN_REACHED value
                             * @property {number} LIST_ERRORS_FOR_PROMOTED_DRAFT_ONLY=11 LIST_ERRORS_FOR_PROMOTED_DRAFT_ONLY value
                             */
                            CampaignDraftErrorEnum.CampaignDraftError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_DRAFT_NAME"] = 2;
                                values[valuesById[3] = "INVALID_STATUS_TRANSITION_FROM_REMOVED"] = 3;
                                values[valuesById[4] = "INVALID_STATUS_TRANSITION_FROM_PROMOTED"] = 4;
                                values[valuesById[5] = "INVALID_STATUS_TRANSITION_FROM_PROMOTE_FAILED"] = 5;
                                values[valuesById[6] = "CUSTOMER_CANNOT_CREATE_DRAFT"] = 6;
                                values[valuesById[7] = "CAMPAIGN_CANNOT_CREATE_DRAFT"] = 7;
                                values[valuesById[8] = "INVALID_DRAFT_CHANGE"] = 8;
                                values[valuesById[9] = "INVALID_STATUS_TRANSITION"] = 9;
                                values[valuesById[10] = "MAX_NUMBER_OF_DRAFTS_PER_CAMPAIGN_REACHED"] = 10;
                                values[valuesById[11] = "LIST_ERRORS_FOR_PROMOTED_DRAFT_ONLY"] = 11;
                                return values;
                            })();
    
                            return CampaignDraftErrorEnum;
                        })();
    
                        errors.CampaignErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignErrorEnum.
                             * @implements ICampaignErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignErrorEnum=} [p] Properties to set
                             */
                            function CampaignErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignErrorEnum} CampaignErrorEnum instance
                             */
                            CampaignErrorEnum.create = function create(properties) {
                                return new CampaignErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignErrorEnum} m CampaignErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignErrorEnum} message CampaignErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignErrorEnum} CampaignErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignErrorEnum} CampaignErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignErrorEnum} CampaignErrorEnum
                             */
                            CampaignErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignErrorEnum} m CampaignErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignError enum.
                             * @name google.ads.googleads.v11.errors.CampaignErrorEnum.CampaignError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_TARGET_CONTENT_NETWORK=3 CANNOT_TARGET_CONTENT_NETWORK value
                             * @property {number} CANNOT_TARGET_SEARCH_NETWORK=4 CANNOT_TARGET_SEARCH_NETWORK value
                             * @property {number} CANNOT_TARGET_SEARCH_NETWORK_WITHOUT_GOOGLE_SEARCH=5 CANNOT_TARGET_SEARCH_NETWORK_WITHOUT_GOOGLE_SEARCH value
                             * @property {number} CANNOT_TARGET_GOOGLE_SEARCH_FOR_CPM_CAMPAIGN=6 CANNOT_TARGET_GOOGLE_SEARCH_FOR_CPM_CAMPAIGN value
                             * @property {number} CAMPAIGN_MUST_TARGET_AT_LEAST_ONE_NETWORK=7 CAMPAIGN_MUST_TARGET_AT_LEAST_ONE_NETWORK value
                             * @property {number} CANNOT_TARGET_PARTNER_SEARCH_NETWORK=8 CANNOT_TARGET_PARTNER_SEARCH_NETWORK value
                             * @property {number} CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CRITERIA_LEVEL_BIDDING_STRATEGY=9 CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CRITERIA_LEVEL_BIDDING_STRATEGY value
                             * @property {number} CAMPAIGN_DURATION_MUST_CONTAIN_ALL_RUNNABLE_TRIALS=10 CAMPAIGN_DURATION_MUST_CONTAIN_ALL_RUNNABLE_TRIALS value
                             * @property {number} CANNOT_MODIFY_FOR_TRIAL_CAMPAIGN=11 CANNOT_MODIFY_FOR_TRIAL_CAMPAIGN value
                             * @property {number} DUPLICATE_CAMPAIGN_NAME=12 DUPLICATE_CAMPAIGN_NAME value
                             * @property {number} INCOMPATIBLE_CAMPAIGN_FIELD=13 INCOMPATIBLE_CAMPAIGN_FIELD value
                             * @property {number} INVALID_CAMPAIGN_NAME=14 INVALID_CAMPAIGN_NAME value
                             * @property {number} INVALID_AD_SERVING_OPTIMIZATION_STATUS=15 INVALID_AD_SERVING_OPTIMIZATION_STATUS value
                             * @property {number} INVALID_TRACKING_URL=16 INVALID_TRACKING_URL value
                             * @property {number} CANNOT_SET_BOTH_TRACKING_URL_TEMPLATE_AND_TRACKING_SETTING=17 CANNOT_SET_BOTH_TRACKING_URL_TEMPLATE_AND_TRACKING_SETTING value
                             * @property {number} MAX_IMPRESSIONS_NOT_IN_RANGE=18 MAX_IMPRESSIONS_NOT_IN_RANGE value
                             * @property {number} TIME_UNIT_NOT_SUPPORTED=19 TIME_UNIT_NOT_SUPPORTED value
                             * @property {number} INVALID_OPERATION_IF_SERVING_STATUS_HAS_ENDED=20 INVALID_OPERATION_IF_SERVING_STATUS_HAS_ENDED value
                             * @property {number} BUDGET_CANNOT_BE_SHARED=21 BUDGET_CANNOT_BE_SHARED value
                             * @property {number} CAMPAIGN_CANNOT_USE_SHARED_BUDGET=22 CAMPAIGN_CANNOT_USE_SHARED_BUDGET value
                             * @property {number} CANNOT_CHANGE_BUDGET_ON_CAMPAIGN_WITH_TRIALS=23 CANNOT_CHANGE_BUDGET_ON_CAMPAIGN_WITH_TRIALS value
                             * @property {number} CAMPAIGN_LABEL_DOES_NOT_EXIST=24 CAMPAIGN_LABEL_DOES_NOT_EXIST value
                             * @property {number} CAMPAIGN_LABEL_ALREADY_EXISTS=25 CAMPAIGN_LABEL_ALREADY_EXISTS value
                             * @property {number} MISSING_SHOPPING_SETTING=26 MISSING_SHOPPING_SETTING value
                             * @property {number} INVALID_SHOPPING_SALES_COUNTRY=27 INVALID_SHOPPING_SALES_COUNTRY value
                             * @property {number} ADVERTISING_CHANNEL_TYPE_NOT_AVAILABLE_FOR_ACCOUNT_TYPE=31 ADVERTISING_CHANNEL_TYPE_NOT_AVAILABLE_FOR_ACCOUNT_TYPE value
                             * @property {number} INVALID_ADVERTISING_CHANNEL_SUB_TYPE=32 INVALID_ADVERTISING_CHANNEL_SUB_TYPE value
                             * @property {number} AT_LEAST_ONE_CONVERSION_MUST_BE_SELECTED=33 AT_LEAST_ONE_CONVERSION_MUST_BE_SELECTED value
                             * @property {number} CANNOT_SET_AD_ROTATION_MODE=34 CANNOT_SET_AD_ROTATION_MODE value
                             * @property {number} CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED=35 CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED value
                             * @property {number} CANNOT_SET_DATE_TO_PAST=36 CANNOT_SET_DATE_TO_PAST value
                             * @property {number} MISSING_HOTEL_CUSTOMER_LINK=37 MISSING_HOTEL_CUSTOMER_LINK value
                             * @property {number} INVALID_HOTEL_CUSTOMER_LINK=38 INVALID_HOTEL_CUSTOMER_LINK value
                             * @property {number} MISSING_HOTEL_SETTING=39 MISSING_HOTEL_SETTING value
                             * @property {number} CANNOT_USE_SHARED_CAMPAIGN_BUDGET_WHILE_PART_OF_CAMPAIGN_GROUP=40 CANNOT_USE_SHARED_CAMPAIGN_BUDGET_WHILE_PART_OF_CAMPAIGN_GROUP value
                             * @property {number} APP_NOT_FOUND=41 APP_NOT_FOUND value
                             * @property {number} SHOPPING_ENABLE_LOCAL_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE=42 SHOPPING_ENABLE_LOCAL_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE value
                             * @property {number} MERCHANT_NOT_ALLOWED_FOR_COMPARISON_LISTING_ADS=43 MERCHANT_NOT_ALLOWED_FOR_COMPARISON_LISTING_ADS value
                             * @property {number} INSUFFICIENT_APP_INSTALLS_COUNT=44 INSUFFICIENT_APP_INSTALLS_COUNT value
                             * @property {number} SENSITIVE_CATEGORY_APP=45 SENSITIVE_CATEGORY_APP value
                             * @property {number} HEC_AGREEMENT_REQUIRED=46 HEC_AGREEMENT_REQUIRED value
                             * @property {number} NOT_COMPATIBLE_WITH_VIEW_THROUGH_CONVERSION_OPTIMIZATION=49 NOT_COMPATIBLE_WITH_VIEW_THROUGH_CONVERSION_OPTIMIZATION value
                             * @property {number} INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE=48 INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE value
                             * @property {number} CANNOT_CREATE_APP_PRE_REGISTRATION_FOR_NON_ANDROID_APP=50 CANNOT_CREATE_APP_PRE_REGISTRATION_FOR_NON_ANDROID_APP value
                             * @property {number} APP_NOT_AVAILABLE_TO_CREATE_APP_PRE_REGISTRATION_CAMPAIGN=51 APP_NOT_AVAILABLE_TO_CREATE_APP_PRE_REGISTRATION_CAMPAIGN value
                             * @property {number} INCOMPATIBLE_BUDGET_TYPE=52 INCOMPATIBLE_BUDGET_TYPE value
                             * @property {number} LOCAL_SERVICES_DUPLICATE_CATEGORY_BID=53 LOCAL_SERVICES_DUPLICATE_CATEGORY_BID value
                             * @property {number} LOCAL_SERVICES_INVALID_CATEGORY_BID=54 LOCAL_SERVICES_INVALID_CATEGORY_BID value
                             * @property {number} LOCAL_SERVICES_MISSING_CATEGORY_BID=55 LOCAL_SERVICES_MISSING_CATEGORY_BID value
                             */
                            CampaignErrorEnum.CampaignError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "CANNOT_TARGET_CONTENT_NETWORK"] = 3;
                                values[valuesById[4] = "CANNOT_TARGET_SEARCH_NETWORK"] = 4;
                                values[valuesById[5] = "CANNOT_TARGET_SEARCH_NETWORK_WITHOUT_GOOGLE_SEARCH"] = 5;
                                values[valuesById[6] = "CANNOT_TARGET_GOOGLE_SEARCH_FOR_CPM_CAMPAIGN"] = 6;
                                values[valuesById[7] = "CAMPAIGN_MUST_TARGET_AT_LEAST_ONE_NETWORK"] = 7;
                                values[valuesById[8] = "CANNOT_TARGET_PARTNER_SEARCH_NETWORK"] = 8;
                                values[valuesById[9] = "CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CRITERIA_LEVEL_BIDDING_STRATEGY"] = 9;
                                values[valuesById[10] = "CAMPAIGN_DURATION_MUST_CONTAIN_ALL_RUNNABLE_TRIALS"] = 10;
                                values[valuesById[11] = "CANNOT_MODIFY_FOR_TRIAL_CAMPAIGN"] = 11;
                                values[valuesById[12] = "DUPLICATE_CAMPAIGN_NAME"] = 12;
                                values[valuesById[13] = "INCOMPATIBLE_CAMPAIGN_FIELD"] = 13;
                                values[valuesById[14] = "INVALID_CAMPAIGN_NAME"] = 14;
                                values[valuesById[15] = "INVALID_AD_SERVING_OPTIMIZATION_STATUS"] = 15;
                                values[valuesById[16] = "INVALID_TRACKING_URL"] = 16;
                                values[valuesById[17] = "CANNOT_SET_BOTH_TRACKING_URL_TEMPLATE_AND_TRACKING_SETTING"] = 17;
                                values[valuesById[18] = "MAX_IMPRESSIONS_NOT_IN_RANGE"] = 18;
                                values[valuesById[19] = "TIME_UNIT_NOT_SUPPORTED"] = 19;
                                values[valuesById[20] = "INVALID_OPERATION_IF_SERVING_STATUS_HAS_ENDED"] = 20;
                                values[valuesById[21] = "BUDGET_CANNOT_BE_SHARED"] = 21;
                                values[valuesById[22] = "CAMPAIGN_CANNOT_USE_SHARED_BUDGET"] = 22;
                                values[valuesById[23] = "CANNOT_CHANGE_BUDGET_ON_CAMPAIGN_WITH_TRIALS"] = 23;
                                values[valuesById[24] = "CAMPAIGN_LABEL_DOES_NOT_EXIST"] = 24;
                                values[valuesById[25] = "CAMPAIGN_LABEL_ALREADY_EXISTS"] = 25;
                                values[valuesById[26] = "MISSING_SHOPPING_SETTING"] = 26;
                                values[valuesById[27] = "INVALID_SHOPPING_SALES_COUNTRY"] = 27;
                                values[valuesById[31] = "ADVERTISING_CHANNEL_TYPE_NOT_AVAILABLE_FOR_ACCOUNT_TYPE"] = 31;
                                values[valuesById[32] = "INVALID_ADVERTISING_CHANNEL_SUB_TYPE"] = 32;
                                values[valuesById[33] = "AT_LEAST_ONE_CONVERSION_MUST_BE_SELECTED"] = 33;
                                values[valuesById[34] = "CANNOT_SET_AD_ROTATION_MODE"] = 34;
                                values[valuesById[35] = "CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED"] = 35;
                                values[valuesById[36] = "CANNOT_SET_DATE_TO_PAST"] = 36;
                                values[valuesById[37] = "MISSING_HOTEL_CUSTOMER_LINK"] = 37;
                                values[valuesById[38] = "INVALID_HOTEL_CUSTOMER_LINK"] = 38;
                                values[valuesById[39] = "MISSING_HOTEL_SETTING"] = 39;
                                values[valuesById[40] = "CANNOT_USE_SHARED_CAMPAIGN_BUDGET_WHILE_PART_OF_CAMPAIGN_GROUP"] = 40;
                                values[valuesById[41] = "APP_NOT_FOUND"] = 41;
                                values[valuesById[42] = "SHOPPING_ENABLE_LOCAL_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE"] = 42;
                                values[valuesById[43] = "MERCHANT_NOT_ALLOWED_FOR_COMPARISON_LISTING_ADS"] = 43;
                                values[valuesById[44] = "INSUFFICIENT_APP_INSTALLS_COUNT"] = 44;
                                values[valuesById[45] = "SENSITIVE_CATEGORY_APP"] = 45;
                                values[valuesById[46] = "HEC_AGREEMENT_REQUIRED"] = 46;
                                values[valuesById[49] = "NOT_COMPATIBLE_WITH_VIEW_THROUGH_CONVERSION_OPTIMIZATION"] = 49;
                                values[valuesById[48] = "INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE"] = 48;
                                values[valuesById[50] = "CANNOT_CREATE_APP_PRE_REGISTRATION_FOR_NON_ANDROID_APP"] = 50;
                                values[valuesById[51] = "APP_NOT_AVAILABLE_TO_CREATE_APP_PRE_REGISTRATION_CAMPAIGN"] = 51;
                                values[valuesById[52] = "INCOMPATIBLE_BUDGET_TYPE"] = 52;
                                values[valuesById[53] = "LOCAL_SERVICES_DUPLICATE_CATEGORY_BID"] = 53;
                                values[valuesById[54] = "LOCAL_SERVICES_INVALID_CATEGORY_BID"] = 54;
                                values[valuesById[55] = "LOCAL_SERVICES_MISSING_CATEGORY_BID"] = 55;
                                return values;
                            })();
    
                            return CampaignErrorEnum;
                        })();
    
                        errors.CampaignExperimentErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignExperimentErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignExperimentErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignExperimentErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignExperimentErrorEnum.
                             * @implements ICampaignExperimentErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignExperimentErrorEnum=} [p] Properties to set
                             */
                            function CampaignExperimentErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignExperimentErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignExperimentErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignExperimentErrorEnum} CampaignExperimentErrorEnum instance
                             */
                            CampaignExperimentErrorEnum.create = function create(properties) {
                                return new CampaignExperimentErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignExperimentErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignExperimentErrorEnum} m CampaignExperimentErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignExperimentErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignExperimentErrorEnum} message CampaignExperimentErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExperimentErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignExperimentErrorEnum} CampaignExperimentErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignExperimentErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExperimentErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignExperimentErrorEnum} CampaignExperimentErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExperimentErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExperimentErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExperimentErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignExperimentErrorEnum} CampaignExperimentErrorEnum
                             */
                            CampaignExperimentErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignExperimentErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignExperimentErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignExperimentErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignExperimentErrorEnum} m CampaignExperimentErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExperimentErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignExperimentErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignExperimentErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExperimentErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignExperimentError enum.
                             * @name google.ads.googleads.v11.errors.CampaignExperimentErrorEnum.CampaignExperimentError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_NAME=2 DUPLICATE_NAME value
                             * @property {number} INVALID_TRANSITION=3 INVALID_TRANSITION value
                             * @property {number} CANNOT_CREATE_EXPERIMENT_WITH_SHARED_BUDGET=4 CANNOT_CREATE_EXPERIMENT_WITH_SHARED_BUDGET value
                             * @property {number} CANNOT_CREATE_EXPERIMENT_FOR_REMOVED_BASE_CAMPAIGN=5 CANNOT_CREATE_EXPERIMENT_FOR_REMOVED_BASE_CAMPAIGN value
                             * @property {number} CANNOT_CREATE_EXPERIMENT_FOR_NON_PROPOSED_DRAFT=6 CANNOT_CREATE_EXPERIMENT_FOR_NON_PROPOSED_DRAFT value
                             * @property {number} CUSTOMER_CANNOT_CREATE_EXPERIMENT=7 CUSTOMER_CANNOT_CREATE_EXPERIMENT value
                             * @property {number} CAMPAIGN_CANNOT_CREATE_EXPERIMENT=8 CAMPAIGN_CANNOT_CREATE_EXPERIMENT value
                             * @property {number} EXPERIMENT_DURATIONS_MUST_NOT_OVERLAP=9 EXPERIMENT_DURATIONS_MUST_NOT_OVERLAP value
                             * @property {number} EXPERIMENT_DURATION_MUST_BE_WITHIN_CAMPAIGN_DURATION=10 EXPERIMENT_DURATION_MUST_BE_WITHIN_CAMPAIGN_DURATION value
                             * @property {number} CANNOT_MUTATE_EXPERIMENT_DUE_TO_STATUS=11 CANNOT_MUTATE_EXPERIMENT_DUE_TO_STATUS value
                             */
                            CampaignExperimentErrorEnum.CampaignExperimentError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_NAME"] = 2;
                                values[valuesById[3] = "INVALID_TRANSITION"] = 3;
                                values[valuesById[4] = "CANNOT_CREATE_EXPERIMENT_WITH_SHARED_BUDGET"] = 4;
                                values[valuesById[5] = "CANNOT_CREATE_EXPERIMENT_FOR_REMOVED_BASE_CAMPAIGN"] = 5;
                                values[valuesById[6] = "CANNOT_CREATE_EXPERIMENT_FOR_NON_PROPOSED_DRAFT"] = 6;
                                values[valuesById[7] = "CUSTOMER_CANNOT_CREATE_EXPERIMENT"] = 7;
                                values[valuesById[8] = "CAMPAIGN_CANNOT_CREATE_EXPERIMENT"] = 8;
                                values[valuesById[9] = "EXPERIMENT_DURATIONS_MUST_NOT_OVERLAP"] = 9;
                                values[valuesById[10] = "EXPERIMENT_DURATION_MUST_BE_WITHIN_CAMPAIGN_DURATION"] = 10;
                                values[valuesById[11] = "CANNOT_MUTATE_EXPERIMENT_DUE_TO_STATUS"] = 11;
                                return values;
                            })();
    
                            return CampaignExperimentErrorEnum;
                        })();
    
                        errors.CampaignFeedErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignFeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignFeedErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignFeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignFeedErrorEnum.
                             * @implements ICampaignFeedErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignFeedErrorEnum=} [p] Properties to set
                             */
                            function CampaignFeedErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignFeedErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignFeedErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignFeedErrorEnum} CampaignFeedErrorEnum instance
                             */
                            CampaignFeedErrorEnum.create = function create(properties) {
                                return new CampaignFeedErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignFeedErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignFeedErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignFeedErrorEnum} m CampaignFeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignFeedErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignFeedErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignFeedErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignFeedErrorEnum} message CampaignFeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignFeedErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignFeedErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignFeedErrorEnum} CampaignFeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignFeedErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignFeedErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignFeedErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignFeedErrorEnum} CampaignFeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignFeedErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignFeedErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignFeedErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignFeedErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignFeedErrorEnum} CampaignFeedErrorEnum
                             */
                            CampaignFeedErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignFeedErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignFeedErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignFeedErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignFeedErrorEnum} m CampaignFeedErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignFeedErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignFeedErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignFeedErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignFeedErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignFeedError enum.
                             * @name google.ads.googleads.v11.errors.CampaignFeedErrorEnum.CampaignFeedError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE=2 FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE value
                             * @property {number} CANNOT_CREATE_FOR_REMOVED_FEED=4 CANNOT_CREATE_FOR_REMOVED_FEED value
                             * @property {number} CANNOT_CREATE_ALREADY_EXISTING_CAMPAIGN_FEED=5 CANNOT_CREATE_ALREADY_EXISTING_CAMPAIGN_FEED value
                             * @property {number} CANNOT_MODIFY_REMOVED_CAMPAIGN_FEED=6 CANNOT_MODIFY_REMOVED_CAMPAIGN_FEED value
                             * @property {number} INVALID_PLACEHOLDER_TYPE=7 INVALID_PLACEHOLDER_TYPE value
                             * @property {number} MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE=8 MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE value
                             * @property {number} NO_EXISTING_LOCATION_CUSTOMER_FEED=9 NO_EXISTING_LOCATION_CUSTOMER_FEED value
                             */
                            CampaignFeedErrorEnum.CampaignFeedError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE"] = 2;
                                values[valuesById[4] = "CANNOT_CREATE_FOR_REMOVED_FEED"] = 4;
                                values[valuesById[5] = "CANNOT_CREATE_ALREADY_EXISTING_CAMPAIGN_FEED"] = 5;
                                values[valuesById[6] = "CANNOT_MODIFY_REMOVED_CAMPAIGN_FEED"] = 6;
                                values[valuesById[7] = "INVALID_PLACEHOLDER_TYPE"] = 7;
                                values[valuesById[8] = "MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE"] = 8;
                                values[valuesById[9] = "NO_EXISTING_LOCATION_CUSTOMER_FEED"] = 9;
                                return values;
                            })();
    
                            return CampaignFeedErrorEnum;
                        })();
    
                        errors.CampaignSharedSetErrorEnum = (function() {
    
                            /**
                             * Properties of a CampaignSharedSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICampaignSharedSetErrorEnum
                             */
    
                            /**
                             * Constructs a new CampaignSharedSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CampaignSharedSetErrorEnum.
                             * @implements ICampaignSharedSetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICampaignSharedSetErrorEnum=} [p] Properties to set
                             */
                            function CampaignSharedSetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CampaignSharedSetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignSharedSetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum} CampaignSharedSetErrorEnum instance
                             */
                            CampaignSharedSetErrorEnum.create = function create(properties) {
                                return new CampaignSharedSetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignSharedSetErrorEnum} m CampaignSharedSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICampaignSharedSetErrorEnum} message CampaignSharedSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignSharedSetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum} CampaignSharedSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignSharedSetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum} CampaignSharedSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignSharedSetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignSharedSetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CampaignSharedSetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum} CampaignSharedSetErrorEnum
                             */
                            CampaignSharedSetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CampaignSharedSetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum} m CampaignSharedSetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignSharedSetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CampaignSharedSetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignSharedSetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CampaignSharedSetError enum.
                             * @name google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum.CampaignSharedSetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SHARED_SET_ACCESS_DENIED=2 SHARED_SET_ACCESS_DENIED value
                             */
                            CampaignSharedSetErrorEnum.CampaignSharedSetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "SHARED_SET_ACCESS_DENIED"] = 2;
                                return values;
                            })();
    
                            return CampaignSharedSetErrorEnum;
                        })();
    
                        errors.ChangeEventErrorEnum = (function() {
    
                            /**
                             * Properties of a ChangeEventErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IChangeEventErrorEnum
                             */
    
                            /**
                             * Constructs a new ChangeEventErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ChangeEventErrorEnum.
                             * @implements IChangeEventErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IChangeEventErrorEnum=} [p] Properties to set
                             */
                            function ChangeEventErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ChangeEventErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IChangeEventErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ChangeEventErrorEnum} ChangeEventErrorEnum instance
                             */
                            ChangeEventErrorEnum.create = function create(properties) {
                                return new ChangeEventErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeEventErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ChangeEventErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IChangeEventErrorEnum} m ChangeEventErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeEventErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeEventErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ChangeEventErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IChangeEventErrorEnum} message ChangeEventErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeEventErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeEventErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ChangeEventErrorEnum} ChangeEventErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeEventErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ChangeEventErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeEventErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ChangeEventErrorEnum} ChangeEventErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeEventErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeEventErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeEventErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ChangeEventErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ChangeEventErrorEnum} ChangeEventErrorEnum
                             */
                            ChangeEventErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ChangeEventErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ChangeEventErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ChangeEventErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ChangeEventErrorEnum} m ChangeEventErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeEventErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ChangeEventErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ChangeEventErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeEventErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ChangeEventError enum.
                             * @name google.ads.googleads.v11.errors.ChangeEventErrorEnum.ChangeEventError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} START_DATE_TOO_OLD=2 START_DATE_TOO_OLD value
                             * @property {number} CHANGE_DATE_RANGE_INFINITE=3 CHANGE_DATE_RANGE_INFINITE value
                             * @property {number} CHANGE_DATE_RANGE_NEGATIVE=4 CHANGE_DATE_RANGE_NEGATIVE value
                             * @property {number} LIMIT_NOT_SPECIFIED=5 LIMIT_NOT_SPECIFIED value
                             * @property {number} INVALID_LIMIT_CLAUSE=6 INVALID_LIMIT_CLAUSE value
                             */
                            ChangeEventErrorEnum.ChangeEventError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "START_DATE_TOO_OLD"] = 2;
                                values[valuesById[3] = "CHANGE_DATE_RANGE_INFINITE"] = 3;
                                values[valuesById[4] = "CHANGE_DATE_RANGE_NEGATIVE"] = 4;
                                values[valuesById[5] = "LIMIT_NOT_SPECIFIED"] = 5;
                                values[valuesById[6] = "INVALID_LIMIT_CLAUSE"] = 6;
                                return values;
                            })();
    
                            return ChangeEventErrorEnum;
                        })();
    
                        errors.ChangeStatusErrorEnum = (function() {
    
                            /**
                             * Properties of a ChangeStatusErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IChangeStatusErrorEnum
                             */
    
                            /**
                             * Constructs a new ChangeStatusErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ChangeStatusErrorEnum.
                             * @implements IChangeStatusErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IChangeStatusErrorEnum=} [p] Properties to set
                             */
                            function ChangeStatusErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ChangeStatusErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IChangeStatusErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ChangeStatusErrorEnum} ChangeStatusErrorEnum instance
                             */
                            ChangeStatusErrorEnum.create = function create(properties) {
                                return new ChangeStatusErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeStatusErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ChangeStatusErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IChangeStatusErrorEnum} m ChangeStatusErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatusErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeStatusErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ChangeStatusErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IChangeStatusErrorEnum} message ChangeStatusErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatusErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeStatusErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ChangeStatusErrorEnum} ChangeStatusErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatusErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ChangeStatusErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeStatusErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ChangeStatusErrorEnum} ChangeStatusErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatusErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeStatusErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeStatusErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ChangeStatusErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ChangeStatusErrorEnum} ChangeStatusErrorEnum
                             */
                            ChangeStatusErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ChangeStatusErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ChangeStatusErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ChangeStatusErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ChangeStatusErrorEnum} m ChangeStatusErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeStatusErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ChangeStatusErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ChangeStatusErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeStatusErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ChangeStatusError enum.
                             * @name google.ads.googleads.v11.errors.ChangeStatusErrorEnum.ChangeStatusError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} START_DATE_TOO_OLD=3 START_DATE_TOO_OLD value
                             * @property {number} CHANGE_DATE_RANGE_INFINITE=4 CHANGE_DATE_RANGE_INFINITE value
                             * @property {number} CHANGE_DATE_RANGE_NEGATIVE=5 CHANGE_DATE_RANGE_NEGATIVE value
                             * @property {number} LIMIT_NOT_SPECIFIED=6 LIMIT_NOT_SPECIFIED value
                             * @property {number} INVALID_LIMIT_CLAUSE=7 INVALID_LIMIT_CLAUSE value
                             */
                            ChangeStatusErrorEnum.ChangeStatusError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "START_DATE_TOO_OLD"] = 3;
                                values[valuesById[4] = "CHANGE_DATE_RANGE_INFINITE"] = 4;
                                values[valuesById[5] = "CHANGE_DATE_RANGE_NEGATIVE"] = 5;
                                values[valuesById[6] = "LIMIT_NOT_SPECIFIED"] = 6;
                                values[valuesById[7] = "INVALID_LIMIT_CLAUSE"] = 7;
                                return values;
                            })();
    
                            return ChangeStatusErrorEnum;
                        })();
    
                        errors.CollectionSizeErrorEnum = (function() {
    
                            /**
                             * Properties of a CollectionSizeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICollectionSizeErrorEnum
                             */
    
                            /**
                             * Constructs a new CollectionSizeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CollectionSizeErrorEnum.
                             * @implements ICollectionSizeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICollectionSizeErrorEnum=} [p] Properties to set
                             */
                            function CollectionSizeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CollectionSizeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICollectionSizeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CollectionSizeErrorEnum} CollectionSizeErrorEnum instance
                             */
                            CollectionSizeErrorEnum.create = function create(properties) {
                                return new CollectionSizeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CollectionSizeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CollectionSizeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICollectionSizeErrorEnum} m CollectionSizeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CollectionSizeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CollectionSizeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CollectionSizeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICollectionSizeErrorEnum} message CollectionSizeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CollectionSizeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CollectionSizeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CollectionSizeErrorEnum} CollectionSizeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CollectionSizeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CollectionSizeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CollectionSizeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CollectionSizeErrorEnum} CollectionSizeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CollectionSizeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CollectionSizeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CollectionSizeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CollectionSizeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CollectionSizeErrorEnum} CollectionSizeErrorEnum
                             */
                            CollectionSizeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CollectionSizeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CollectionSizeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CollectionSizeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CollectionSizeErrorEnum} m CollectionSizeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CollectionSizeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CollectionSizeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CollectionSizeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CollectionSizeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CollectionSizeError enum.
                             * @name google.ads.googleads.v11.errors.CollectionSizeErrorEnum.CollectionSizeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TOO_FEW=2 TOO_FEW value
                             * @property {number} TOO_MANY=3 TOO_MANY value
                             */
                            CollectionSizeErrorEnum.CollectionSizeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TOO_FEW"] = 2;
                                values[valuesById[3] = "TOO_MANY"] = 3;
                                return values;
                            })();
    
                            return CollectionSizeErrorEnum;
                        })();
    
                        errors.ContextErrorEnum = (function() {
    
                            /**
                             * Properties of a ContextErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IContextErrorEnum
                             */
    
                            /**
                             * Constructs a new ContextErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ContextErrorEnum.
                             * @implements IContextErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IContextErrorEnum=} [p] Properties to set
                             */
                            function ContextErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ContextErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IContextErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ContextErrorEnum} ContextErrorEnum instance
                             */
                            ContextErrorEnum.create = function create(properties) {
                                return new ContextErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ContextErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ContextErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IContextErrorEnum} m ContextErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ContextErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ContextErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ContextErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IContextErrorEnum} message ContextErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ContextErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ContextErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ContextErrorEnum} ContextErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ContextErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ContextErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ContextErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ContextErrorEnum} ContextErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ContextErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ContextErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ContextErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ContextErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ContextErrorEnum} ContextErrorEnum
                             */
                            ContextErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ContextErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ContextErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ContextErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ContextErrorEnum} m ContextErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ContextErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ContextErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ContextErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ContextErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ContextError enum.
                             * @name google.ads.googleads.v11.errors.ContextErrorEnum.ContextError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OPERATION_NOT_PERMITTED_FOR_CONTEXT=2 OPERATION_NOT_PERMITTED_FOR_CONTEXT value
                             * @property {number} OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE=3 OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE value
                             */
                            ContextErrorEnum.ContextError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OPERATION_NOT_PERMITTED_FOR_CONTEXT"] = 2;
                                values[valuesById[3] = "OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE"] = 3;
                                return values;
                            })();
    
                            return ContextErrorEnum;
                        })();
    
                        errors.ConversionActionErrorEnum = (function() {
    
                            /**
                             * Properties of a ConversionActionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IConversionActionErrorEnum
                             */
    
                            /**
                             * Constructs a new ConversionActionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ConversionActionErrorEnum.
                             * @implements IConversionActionErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IConversionActionErrorEnum=} [p] Properties to set
                             */
                            function ConversionActionErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionActionErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionActionErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ConversionActionErrorEnum} ConversionActionErrorEnum instance
                             */
                            ConversionActionErrorEnum.create = function create(properties) {
                                return new ConversionActionErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionActionErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionActionErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionActionErrorEnum} m ConversionActionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionActionErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionActionErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionActionErrorEnum} message ConversionActionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionActionErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ConversionActionErrorEnum} ConversionActionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ConversionActionErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionActionErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ConversionActionErrorEnum} ConversionActionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionActionErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionActionErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionActionErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ConversionActionErrorEnum} ConversionActionErrorEnum
                             */
                            ConversionActionErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ConversionActionErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ConversionActionErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionActionErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ConversionActionErrorEnum} m ConversionActionErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionActionErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionActionErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ConversionActionErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionActionErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionActionError enum.
                             * @name google.ads.googleads.v11.errors.ConversionActionErrorEnum.ConversionActionError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_NAME=2 DUPLICATE_NAME value
                             * @property {number} DUPLICATE_APP_ID=3 DUPLICATE_APP_ID value
                             * @property {number} TWO_CONVERSION_ACTIONS_BIDDING_ON_SAME_APP_DOWNLOAD=4 TWO_CONVERSION_ACTIONS_BIDDING_ON_SAME_APP_DOWNLOAD value
                             * @property {number} BIDDING_ON_SAME_APP_DOWNLOAD_AS_GLOBAL_ACTION=5 BIDDING_ON_SAME_APP_DOWNLOAD_AS_GLOBAL_ACTION value
                             * @property {number} DATA_DRIVEN_MODEL_WAS_NEVER_GENERATED=6 DATA_DRIVEN_MODEL_WAS_NEVER_GENERATED value
                             * @property {number} DATA_DRIVEN_MODEL_EXPIRED=7 DATA_DRIVEN_MODEL_EXPIRED value
                             * @property {number} DATA_DRIVEN_MODEL_STALE=8 DATA_DRIVEN_MODEL_STALE value
                             * @property {number} DATA_DRIVEN_MODEL_UNKNOWN=9 DATA_DRIVEN_MODEL_UNKNOWN value
                             * @property {number} CREATION_NOT_SUPPORTED=10 CREATION_NOT_SUPPORTED value
                             * @property {number} UPDATE_NOT_SUPPORTED=11 UPDATE_NOT_SUPPORTED value
                             */
                            ConversionActionErrorEnum.ConversionActionError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_NAME"] = 2;
                                values[valuesById[3] = "DUPLICATE_APP_ID"] = 3;
                                values[valuesById[4] = "TWO_CONVERSION_ACTIONS_BIDDING_ON_SAME_APP_DOWNLOAD"] = 4;
                                values[valuesById[5] = "BIDDING_ON_SAME_APP_DOWNLOAD_AS_GLOBAL_ACTION"] = 5;
                                values[valuesById[6] = "DATA_DRIVEN_MODEL_WAS_NEVER_GENERATED"] = 6;
                                values[valuesById[7] = "DATA_DRIVEN_MODEL_EXPIRED"] = 7;
                                values[valuesById[8] = "DATA_DRIVEN_MODEL_STALE"] = 8;
                                values[valuesById[9] = "DATA_DRIVEN_MODEL_UNKNOWN"] = 9;
                                values[valuesById[10] = "CREATION_NOT_SUPPORTED"] = 10;
                                values[valuesById[11] = "UPDATE_NOT_SUPPORTED"] = 11;
                                return values;
                            })();
    
                            return ConversionActionErrorEnum;
                        })();
    
                        errors.ConversionAdjustmentUploadErrorEnum = (function() {
    
                            /**
                             * Properties of a ConversionAdjustmentUploadErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IConversionAdjustmentUploadErrorEnum
                             */
    
                            /**
                             * Constructs a new ConversionAdjustmentUploadErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ConversionAdjustmentUploadErrorEnum.
                             * @implements IConversionAdjustmentUploadErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IConversionAdjustmentUploadErrorEnum=} [p] Properties to set
                             */
                            function ConversionAdjustmentUploadErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionAdjustmentUploadErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionAdjustmentUploadErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum} ConversionAdjustmentUploadErrorEnum instance
                             */
                            ConversionAdjustmentUploadErrorEnum.create = function create(properties) {
                                return new ConversionAdjustmentUploadErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustmentUploadErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionAdjustmentUploadErrorEnum} m ConversionAdjustmentUploadErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustmentUploadErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustmentUploadErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionAdjustmentUploadErrorEnum} message ConversionAdjustmentUploadErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustmentUploadErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionAdjustmentUploadErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum} ConversionAdjustmentUploadErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustmentUploadErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionAdjustmentUploadErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum} ConversionAdjustmentUploadErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustmentUploadErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionAdjustmentUploadErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionAdjustmentUploadErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionAdjustmentUploadErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum} ConversionAdjustmentUploadErrorEnum
                             */
                            ConversionAdjustmentUploadErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionAdjustmentUploadErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum} m ConversionAdjustmentUploadErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionAdjustmentUploadErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionAdjustmentUploadErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionAdjustmentUploadErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionAdjustmentUploadError enum.
                             * @name google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TOO_RECENT_CONVERSION_ACTION=2 TOO_RECENT_CONVERSION_ACTION value
                             * @property {number} INVALID_CONVERSION_ACTION=3 INVALID_CONVERSION_ACTION value
                             * @property {number} CONVERSION_ALREADY_RETRACTED=4 CONVERSION_ALREADY_RETRACTED value
                             * @property {number} CONVERSION_NOT_FOUND=5 CONVERSION_NOT_FOUND value
                             * @property {number} CONVERSION_EXPIRED=6 CONVERSION_EXPIRED value
                             * @property {number} ADJUSTMENT_PRECEDES_CONVERSION=7 ADJUSTMENT_PRECEDES_CONVERSION value
                             * @property {number} MORE_RECENT_RESTATEMENT_FOUND=8 MORE_RECENT_RESTATEMENT_FOUND value
                             * @property {number} TOO_RECENT_CONVERSION=9 TOO_RECENT_CONVERSION value
                             * @property {number} CANNOT_RESTATE_CONVERSION_ACTION_THAT_ALWAYS_USES_DEFAULT_CONVERSION_VALUE=10 CANNOT_RESTATE_CONVERSION_ACTION_THAT_ALWAYS_USES_DEFAULT_CONVERSION_VALUE value
                             * @property {number} TOO_MANY_ADJUSTMENTS_IN_REQUEST=11 TOO_MANY_ADJUSTMENTS_IN_REQUEST value
                             * @property {number} TOO_MANY_ADJUSTMENTS=12 TOO_MANY_ADJUSTMENTS value
                             * @property {number} RESTATEMENT_ALREADY_EXISTS=13 RESTATEMENT_ALREADY_EXISTS value
                             * @property {number} DUPLICATE_ADJUSTMENT_IN_REQUEST=14 DUPLICATE_ADJUSTMENT_IN_REQUEST value
                             * @property {number} CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS=15 CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS value
                             * @property {number} CONVERSION_ACTION_NOT_ELIGIBLE_FOR_ENHANCEMENT=16 CONVERSION_ACTION_NOT_ELIGIBLE_FOR_ENHANCEMENT value
                             * @property {number} INVALID_USER_IDENTIFIER=17 INVALID_USER_IDENTIFIER value
                             * @property {number} UNSUPPORTED_USER_IDENTIFIER=18 UNSUPPORTED_USER_IDENTIFIER value
                             * @property {number} GCLID_DATE_TIME_PAIR_AND_ORDER_ID_BOTH_SET=20 GCLID_DATE_TIME_PAIR_AND_ORDER_ID_BOTH_SET value
                             * @property {number} CONVERSION_ALREADY_ENHANCED=21 CONVERSION_ALREADY_ENHANCED value
                             * @property {number} DUPLICATE_ENHANCEMENT_IN_REQUEST=22 DUPLICATE_ENHANCEMENT_IN_REQUEST value
                             * @property {number} CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCEMENT=23 CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCEMENT value
                             * @property {number} MISSING_ORDER_ID_FOR_WEBPAGE=24 MISSING_ORDER_ID_FOR_WEBPAGE value
                             * @property {number} ORDER_ID_CONTAINS_PII=25 ORDER_ID_CONTAINS_PII value
                             */
                            ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TOO_RECENT_CONVERSION_ACTION"] = 2;
                                values[valuesById[3] = "INVALID_CONVERSION_ACTION"] = 3;
                                values[valuesById[4] = "CONVERSION_ALREADY_RETRACTED"] = 4;
                                values[valuesById[5] = "CONVERSION_NOT_FOUND"] = 5;
                                values[valuesById[6] = "CONVERSION_EXPIRED"] = 6;
                                values[valuesById[7] = "ADJUSTMENT_PRECEDES_CONVERSION"] = 7;
                                values[valuesById[8] = "MORE_RECENT_RESTATEMENT_FOUND"] = 8;
                                values[valuesById[9] = "TOO_RECENT_CONVERSION"] = 9;
                                values[valuesById[10] = "CANNOT_RESTATE_CONVERSION_ACTION_THAT_ALWAYS_USES_DEFAULT_CONVERSION_VALUE"] = 10;
                                values[valuesById[11] = "TOO_MANY_ADJUSTMENTS_IN_REQUEST"] = 11;
                                values[valuesById[12] = "TOO_MANY_ADJUSTMENTS"] = 12;
                                values[valuesById[13] = "RESTATEMENT_ALREADY_EXISTS"] = 13;
                                values[valuesById[14] = "DUPLICATE_ADJUSTMENT_IN_REQUEST"] = 14;
                                values[valuesById[15] = "CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS"] = 15;
                                values[valuesById[16] = "CONVERSION_ACTION_NOT_ELIGIBLE_FOR_ENHANCEMENT"] = 16;
                                values[valuesById[17] = "INVALID_USER_IDENTIFIER"] = 17;
                                values[valuesById[18] = "UNSUPPORTED_USER_IDENTIFIER"] = 18;
                                values[valuesById[20] = "GCLID_DATE_TIME_PAIR_AND_ORDER_ID_BOTH_SET"] = 20;
                                values[valuesById[21] = "CONVERSION_ALREADY_ENHANCED"] = 21;
                                values[valuesById[22] = "DUPLICATE_ENHANCEMENT_IN_REQUEST"] = 22;
                                values[valuesById[23] = "CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCEMENT"] = 23;
                                values[valuesById[24] = "MISSING_ORDER_ID_FOR_WEBPAGE"] = 24;
                                values[valuesById[25] = "ORDER_ID_CONTAINS_PII"] = 25;
                                return values;
                            })();
    
                            return ConversionAdjustmentUploadErrorEnum;
                        })();
    
                        errors.ConversionCustomVariableErrorEnum = (function() {
    
                            /**
                             * Properties of a ConversionCustomVariableErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IConversionCustomVariableErrorEnum
                             */
    
                            /**
                             * Constructs a new ConversionCustomVariableErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ConversionCustomVariableErrorEnum.
                             * @implements IConversionCustomVariableErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IConversionCustomVariableErrorEnum=} [p] Properties to set
                             */
                            function ConversionCustomVariableErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionCustomVariableErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionCustomVariableErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum} ConversionCustomVariableErrorEnum instance
                             */
                            ConversionCustomVariableErrorEnum.create = function create(properties) {
                                return new ConversionCustomVariableErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariableErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionCustomVariableErrorEnum} m ConversionCustomVariableErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariableErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariableErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionCustomVariableErrorEnum} message ConversionCustomVariableErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariableErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionCustomVariableErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum} ConversionCustomVariableErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariableErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionCustomVariableErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum} ConversionCustomVariableErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariableErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionCustomVariableErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionCustomVariableErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionCustomVariableErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum} ConversionCustomVariableErrorEnum
                             */
                            ConversionCustomVariableErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionCustomVariableErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum} m ConversionCustomVariableErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionCustomVariableErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionCustomVariableErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionCustomVariableErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionCustomVariableError enum.
                             * @name google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum.ConversionCustomVariableError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_NAME=2 DUPLICATE_NAME value
                             * @property {number} DUPLICATE_TAG=3 DUPLICATE_TAG value
                             * @property {number} RESERVED_TAG=4 RESERVED_TAG value
                             */
                            ConversionCustomVariableErrorEnum.ConversionCustomVariableError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_NAME"] = 2;
                                values[valuesById[3] = "DUPLICATE_TAG"] = 3;
                                values[valuesById[4] = "RESERVED_TAG"] = 4;
                                return values;
                            })();
    
                            return ConversionCustomVariableErrorEnum;
                        })();
    
                        errors.ConversionGoalCampaignConfigErrorEnum = (function() {
    
                            /**
                             * Properties of a ConversionGoalCampaignConfigErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IConversionGoalCampaignConfigErrorEnum
                             */
    
                            /**
                             * Constructs a new ConversionGoalCampaignConfigErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ConversionGoalCampaignConfigErrorEnum.
                             * @implements IConversionGoalCampaignConfigErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IConversionGoalCampaignConfigErrorEnum=} [p] Properties to set
                             */
                            function ConversionGoalCampaignConfigErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionGoalCampaignConfigErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionGoalCampaignConfigErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum} ConversionGoalCampaignConfigErrorEnum instance
                             */
                            ConversionGoalCampaignConfigErrorEnum.create = function create(properties) {
                                return new ConversionGoalCampaignConfigErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionGoalCampaignConfigErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionGoalCampaignConfigErrorEnum} m ConversionGoalCampaignConfigErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionGoalCampaignConfigErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionGoalCampaignConfigErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionGoalCampaignConfigErrorEnum} message ConversionGoalCampaignConfigErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionGoalCampaignConfigErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionGoalCampaignConfigErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum} ConversionGoalCampaignConfigErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionGoalCampaignConfigErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionGoalCampaignConfigErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum} ConversionGoalCampaignConfigErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionGoalCampaignConfigErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionGoalCampaignConfigErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionGoalCampaignConfigErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionGoalCampaignConfigErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum} ConversionGoalCampaignConfigErrorEnum
                             */
                            ConversionGoalCampaignConfigErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionGoalCampaignConfigErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum} m ConversionGoalCampaignConfigErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionGoalCampaignConfigErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionGoalCampaignConfigErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionGoalCampaignConfigErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionGoalCampaignConfigError enum.
                             * @name google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum.ConversionGoalCampaignConfigError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN=2 CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN value
                             * @property {number} CUSTOM_GOAL_DOES_NOT_BELONG_TO_GOOGLE_ADS_CONVERSION_CUSTOMER=3 CUSTOM_GOAL_DOES_NOT_BELONG_TO_GOOGLE_ADS_CONVERSION_CUSTOMER value
                             * @property {number} CAMPAIGN_CANNOT_USE_UNIFIED_GOALS=4 CAMPAIGN_CANNOT_USE_UNIFIED_GOALS value
                             */
                            ConversionGoalCampaignConfigErrorEnum.ConversionGoalCampaignConfigError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN"] = 2;
                                values[valuesById[3] = "CUSTOM_GOAL_DOES_NOT_BELONG_TO_GOOGLE_ADS_CONVERSION_CUSTOMER"] = 3;
                                values[valuesById[4] = "CAMPAIGN_CANNOT_USE_UNIFIED_GOALS"] = 4;
                                return values;
                            })();
    
                            return ConversionGoalCampaignConfigErrorEnum;
                        })();
    
                        errors.ConversionUploadErrorEnum = (function() {
    
                            /**
                             * Properties of a ConversionUploadErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IConversionUploadErrorEnum
                             */
    
                            /**
                             * Constructs a new ConversionUploadErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ConversionUploadErrorEnum.
                             * @implements IConversionUploadErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IConversionUploadErrorEnum=} [p] Properties to set
                             */
                            function ConversionUploadErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionUploadErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionUploadErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ConversionUploadErrorEnum} ConversionUploadErrorEnum instance
                             */
                            ConversionUploadErrorEnum.create = function create(properties) {
                                return new ConversionUploadErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionUploadErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionUploadErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionUploadErrorEnum} m ConversionUploadErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionUploadErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionUploadErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionUploadErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionUploadErrorEnum} message ConversionUploadErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionUploadErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionUploadErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ConversionUploadErrorEnum} ConversionUploadErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionUploadErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ConversionUploadErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionUploadErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ConversionUploadErrorEnum} ConversionUploadErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionUploadErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionUploadErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionUploadErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionUploadErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ConversionUploadErrorEnum} ConversionUploadErrorEnum
                             */
                            ConversionUploadErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ConversionUploadErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ConversionUploadErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionUploadErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ConversionUploadErrorEnum} m ConversionUploadErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionUploadErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionUploadErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ConversionUploadErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionUploadErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionUploadError enum.
                             * @name google.ads.googleads.v11.errors.ConversionUploadErrorEnum.ConversionUploadError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TOO_MANY_CONVERSIONS_IN_REQUEST=2 TOO_MANY_CONVERSIONS_IN_REQUEST value
                             * @property {number} UNPARSEABLE_GCLID=3 UNPARSEABLE_GCLID value
                             * @property {number} CONVERSION_PRECEDES_EVENT=42 CONVERSION_PRECEDES_EVENT value
                             * @property {number} EXPIRED_EVENT=43 EXPIRED_EVENT value
                             * @property {number} TOO_RECENT_EVENT=44 TOO_RECENT_EVENT value
                             * @property {number} EVENT_NOT_FOUND=45 EVENT_NOT_FOUND value
                             * @property {number} UNAUTHORIZED_CUSTOMER=8 UNAUTHORIZED_CUSTOMER value
                             * @property {number} INVALID_CONVERSION_ACTION=9 INVALID_CONVERSION_ACTION value
                             * @property {number} TOO_RECENT_CONVERSION_ACTION=10 TOO_RECENT_CONVERSION_ACTION value
                             * @property {number} CONVERSION_TRACKING_NOT_ENABLED_AT_IMPRESSION_TIME=11 CONVERSION_TRACKING_NOT_ENABLED_AT_IMPRESSION_TIME value
                             * @property {number} EXTERNAL_ATTRIBUTION_DATA_SET_FOR_NON_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION=12 EXTERNAL_ATTRIBUTION_DATA_SET_FOR_NON_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION value
                             * @property {number} EXTERNAL_ATTRIBUTION_DATA_NOT_SET_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION=13 EXTERNAL_ATTRIBUTION_DATA_NOT_SET_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION value
                             * @property {number} ORDER_ID_NOT_PERMITTED_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION=14 ORDER_ID_NOT_PERMITTED_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION value
                             * @property {number} ORDER_ID_ALREADY_IN_USE=15 ORDER_ID_ALREADY_IN_USE value
                             * @property {number} DUPLICATE_ORDER_ID=16 DUPLICATE_ORDER_ID value
                             * @property {number} TOO_RECENT_CALL=17 TOO_RECENT_CALL value
                             * @property {number} EXPIRED_CALL=18 EXPIRED_CALL value
                             * @property {number} CALL_NOT_FOUND=19 CALL_NOT_FOUND value
                             * @property {number} CONVERSION_PRECEDES_CALL=20 CONVERSION_PRECEDES_CALL value
                             * @property {number} CONVERSION_TRACKING_NOT_ENABLED_AT_CALL_TIME=21 CONVERSION_TRACKING_NOT_ENABLED_AT_CALL_TIME value
                             * @property {number} UNPARSEABLE_CALLERS_PHONE_NUMBER=22 UNPARSEABLE_CALLERS_PHONE_NUMBER value
                             * @property {number} CLICK_CONVERSION_ALREADY_EXISTS=23 CLICK_CONVERSION_ALREADY_EXISTS value
                             * @property {number} CALL_CONVERSION_ALREADY_EXISTS=24 CALL_CONVERSION_ALREADY_EXISTS value
                             * @property {number} DUPLICATE_CLICK_CONVERSION_IN_REQUEST=25 DUPLICATE_CLICK_CONVERSION_IN_REQUEST value
                             * @property {number} DUPLICATE_CALL_CONVERSION_IN_REQUEST=26 DUPLICATE_CALL_CONVERSION_IN_REQUEST value
                             * @property {number} CUSTOM_VARIABLE_NOT_ENABLED=28 CUSTOM_VARIABLE_NOT_ENABLED value
                             * @property {number} CUSTOM_VARIABLE_VALUE_CONTAINS_PII=29 CUSTOM_VARIABLE_VALUE_CONTAINS_PII value
                             * @property {number} INVALID_CUSTOMER_FOR_CLICK=30 INVALID_CUSTOMER_FOR_CLICK value
                             * @property {number} INVALID_CUSTOMER_FOR_CALL=31 INVALID_CUSTOMER_FOR_CALL value
                             * @property {number} CONVERSION_NOT_COMPLIANT_WITH_ATT_POLICY=32 CONVERSION_NOT_COMPLIANT_WITH_ATT_POLICY value
                             * @property {number} CLICK_NOT_FOUND=33 CLICK_NOT_FOUND value
                             * @property {number} INVALID_USER_IDENTIFIER=34 INVALID_USER_IDENTIFIER value
                             * @property {number} EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION_NOT_PERMITTED_WITH_USER_IDENTIFIER=35 EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION_NOT_PERMITTED_WITH_USER_IDENTIFIER value
                             * @property {number} UNSUPPORTED_USER_IDENTIFIER=36 UNSUPPORTED_USER_IDENTIFIER value
                             * @property {number} GBRAID_WBRAID_BOTH_SET=38 GBRAID_WBRAID_BOTH_SET value
                             * @property {number} UNPARSEABLE_WBRAID=39 UNPARSEABLE_WBRAID value
                             * @property {number} UNPARSEABLE_GBRAID=40 UNPARSEABLE_GBRAID value
                             * @property {number} ONE_PER_CLICK_CONVERSION_ACTION_NOT_PERMITTED_WITH_BRAID=46 ONE_PER_CLICK_CONVERSION_ACTION_NOT_PERMITTED_WITH_BRAID value
                             * @property {number} CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCED_CONVERSIONS=47 CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCED_CONVERSIONS value
                             * @property {number} CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS=48 CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS value
                             * @property {number} ORDER_ID_CONTAINS_PII=49 ORDER_ID_CONTAINS_PII value
                             */
                            ConversionUploadErrorEnum.ConversionUploadError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TOO_MANY_CONVERSIONS_IN_REQUEST"] = 2;
                                values[valuesById[3] = "UNPARSEABLE_GCLID"] = 3;
                                values[valuesById[42] = "CONVERSION_PRECEDES_EVENT"] = 42;
                                values[valuesById[43] = "EXPIRED_EVENT"] = 43;
                                values[valuesById[44] = "TOO_RECENT_EVENT"] = 44;
                                values[valuesById[45] = "EVENT_NOT_FOUND"] = 45;
                                values[valuesById[8] = "UNAUTHORIZED_CUSTOMER"] = 8;
                                values[valuesById[9] = "INVALID_CONVERSION_ACTION"] = 9;
                                values[valuesById[10] = "TOO_RECENT_CONVERSION_ACTION"] = 10;
                                values[valuesById[11] = "CONVERSION_TRACKING_NOT_ENABLED_AT_IMPRESSION_TIME"] = 11;
                                values[valuesById[12] = "EXTERNAL_ATTRIBUTION_DATA_SET_FOR_NON_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION"] = 12;
                                values[valuesById[13] = "EXTERNAL_ATTRIBUTION_DATA_NOT_SET_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION"] = 13;
                                values[valuesById[14] = "ORDER_ID_NOT_PERMITTED_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION"] = 14;
                                values[valuesById[15] = "ORDER_ID_ALREADY_IN_USE"] = 15;
                                values[valuesById[16] = "DUPLICATE_ORDER_ID"] = 16;
                                values[valuesById[17] = "TOO_RECENT_CALL"] = 17;
                                values[valuesById[18] = "EXPIRED_CALL"] = 18;
                                values[valuesById[19] = "CALL_NOT_FOUND"] = 19;
                                values[valuesById[20] = "CONVERSION_PRECEDES_CALL"] = 20;
                                values[valuesById[21] = "CONVERSION_TRACKING_NOT_ENABLED_AT_CALL_TIME"] = 21;
                                values[valuesById[22] = "UNPARSEABLE_CALLERS_PHONE_NUMBER"] = 22;
                                values[valuesById[23] = "CLICK_CONVERSION_ALREADY_EXISTS"] = 23;
                                values[valuesById[24] = "CALL_CONVERSION_ALREADY_EXISTS"] = 24;
                                values[valuesById[25] = "DUPLICATE_CLICK_CONVERSION_IN_REQUEST"] = 25;
                                values[valuesById[26] = "DUPLICATE_CALL_CONVERSION_IN_REQUEST"] = 26;
                                values[valuesById[28] = "CUSTOM_VARIABLE_NOT_ENABLED"] = 28;
                                values[valuesById[29] = "CUSTOM_VARIABLE_VALUE_CONTAINS_PII"] = 29;
                                values[valuesById[30] = "INVALID_CUSTOMER_FOR_CLICK"] = 30;
                                values[valuesById[31] = "INVALID_CUSTOMER_FOR_CALL"] = 31;
                                values[valuesById[32] = "CONVERSION_NOT_COMPLIANT_WITH_ATT_POLICY"] = 32;
                                values[valuesById[33] = "CLICK_NOT_FOUND"] = 33;
                                values[valuesById[34] = "INVALID_USER_IDENTIFIER"] = 34;
                                values[valuesById[35] = "EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION_NOT_PERMITTED_WITH_USER_IDENTIFIER"] = 35;
                                values[valuesById[36] = "UNSUPPORTED_USER_IDENTIFIER"] = 36;
                                values[valuesById[38] = "GBRAID_WBRAID_BOTH_SET"] = 38;
                                values[valuesById[39] = "UNPARSEABLE_WBRAID"] = 39;
                                values[valuesById[40] = "UNPARSEABLE_GBRAID"] = 40;
                                values[valuesById[46] = "ONE_PER_CLICK_CONVERSION_ACTION_NOT_PERMITTED_WITH_BRAID"] = 46;
                                values[valuesById[47] = "CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCED_CONVERSIONS"] = 47;
                                values[valuesById[48] = "CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS"] = 48;
                                values[valuesById[49] = "ORDER_ID_CONTAINS_PII"] = 49;
                                return values;
                            })();
    
                            return ConversionUploadErrorEnum;
                        })();
    
                        errors.ConversionValueRuleErrorEnum = (function() {
    
                            /**
                             * Properties of a ConversionValueRuleErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IConversionValueRuleErrorEnum
                             */
    
                            /**
                             * Constructs a new ConversionValueRuleErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ConversionValueRuleErrorEnum.
                             * @implements IConversionValueRuleErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleErrorEnum=} [p] Properties to set
                             */
                            function ConversionValueRuleErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionValueRuleErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum} ConversionValueRuleErrorEnum instance
                             */
                            ConversionValueRuleErrorEnum.create = function create(properties) {
                                return new ConversionValueRuleErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleErrorEnum} m ConversionValueRuleErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleErrorEnum} message ConversionValueRuleErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRuleErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum} ConversionValueRuleErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRuleErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum} ConversionValueRuleErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRuleErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRuleErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRuleErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum} ConversionValueRuleErrorEnum
                             */
                            ConversionValueRuleErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRuleErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum} m ConversionValueRuleErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRuleErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionValueRuleErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRuleErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionValueRuleError enum.
                             * @name google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum.ConversionValueRuleError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_GEO_TARGET_CONSTANT=2 INVALID_GEO_TARGET_CONSTANT value
                             * @property {number} CONFLICTING_INCLUDED_AND_EXCLUDED_GEO_TARGET=3 CONFLICTING_INCLUDED_AND_EXCLUDED_GEO_TARGET value
                             * @property {number} CONFLICTING_CONDITIONS=4 CONFLICTING_CONDITIONS value
                             * @property {number} CANNOT_REMOVE_IF_INCLUDED_IN_VALUE_RULE_SET=5 CANNOT_REMOVE_IF_INCLUDED_IN_VALUE_RULE_SET value
                             * @property {number} CONDITION_NOT_ALLOWED=6 CONDITION_NOT_ALLOWED value
                             * @property {number} FIELD_MUST_BE_UNSET=7 FIELD_MUST_BE_UNSET value
                             * @property {number} CANNOT_PAUSE_UNLESS_VALUE_RULE_SET_IS_PAUSED=8 CANNOT_PAUSE_UNLESS_VALUE_RULE_SET_IS_PAUSED value
                             * @property {number} UNTARGETABLE_GEO_TARGET=9 UNTARGETABLE_GEO_TARGET value
                             * @property {number} INVALID_AUDIENCE_USER_LIST=10 INVALID_AUDIENCE_USER_LIST value
                             * @property {number} INACCESSIBLE_USER_LIST=11 INACCESSIBLE_USER_LIST value
                             * @property {number} INVALID_AUDIENCE_USER_INTEREST=12 INVALID_AUDIENCE_USER_INTEREST value
                             * @property {number} CANNOT_ADD_RULE_WITH_STATUS_REMOVED=13 CANNOT_ADD_RULE_WITH_STATUS_REMOVED value
                             */
                            ConversionValueRuleErrorEnum.ConversionValueRuleError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_GEO_TARGET_CONSTANT"] = 2;
                                values[valuesById[3] = "CONFLICTING_INCLUDED_AND_EXCLUDED_GEO_TARGET"] = 3;
                                values[valuesById[4] = "CONFLICTING_CONDITIONS"] = 4;
                                values[valuesById[5] = "CANNOT_REMOVE_IF_INCLUDED_IN_VALUE_RULE_SET"] = 5;
                                values[valuesById[6] = "CONDITION_NOT_ALLOWED"] = 6;
                                values[valuesById[7] = "FIELD_MUST_BE_UNSET"] = 7;
                                values[valuesById[8] = "CANNOT_PAUSE_UNLESS_VALUE_RULE_SET_IS_PAUSED"] = 8;
                                values[valuesById[9] = "UNTARGETABLE_GEO_TARGET"] = 9;
                                values[valuesById[10] = "INVALID_AUDIENCE_USER_LIST"] = 10;
                                values[valuesById[11] = "INACCESSIBLE_USER_LIST"] = 11;
                                values[valuesById[12] = "INVALID_AUDIENCE_USER_INTEREST"] = 12;
                                values[valuesById[13] = "CANNOT_ADD_RULE_WITH_STATUS_REMOVED"] = 13;
                                return values;
                            })();
    
                            return ConversionValueRuleErrorEnum;
                        })();
    
                        errors.ConversionValueRuleSetErrorEnum = (function() {
    
                            /**
                             * Properties of a ConversionValueRuleSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IConversionValueRuleSetErrorEnum
                             */
    
                            /**
                             * Constructs a new ConversionValueRuleSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ConversionValueRuleSetErrorEnum.
                             * @implements IConversionValueRuleSetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleSetErrorEnum=} [p] Properties to set
                             */
                            function ConversionValueRuleSetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ConversionValueRuleSetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleSetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum} ConversionValueRuleSetErrorEnum instance
                             */
                            ConversionValueRuleSetErrorEnum.create = function create(properties) {
                                return new ConversionValueRuleSetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleSetErrorEnum} m ConversionValueRuleSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IConversionValueRuleSetErrorEnum} message ConversionValueRuleSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum} ConversionValueRuleSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum} ConversionValueRuleSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRuleSetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRuleSetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRuleSetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum} ConversionValueRuleSetErrorEnum
                             */
                            ConversionValueRuleSetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRuleSetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum} m ConversionValueRuleSetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRuleSetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ConversionValueRuleSetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRuleSetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ConversionValueRuleSetError enum.
                             * @name google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CONFLICTING_VALUE_RULE_CONDITIONS=2 CONFLICTING_VALUE_RULE_CONDITIONS value
                             * @property {number} INVALID_VALUE_RULE=3 INVALID_VALUE_RULE value
                             * @property {number} DIMENSIONS_UPDATE_ONLY_ALLOW_APPEND=4 DIMENSIONS_UPDATE_ONLY_ALLOW_APPEND value
                             * @property {number} CONDITION_TYPE_NOT_ALLOWED=5 CONDITION_TYPE_NOT_ALLOWED value
                             * @property {number} DUPLICATE_DIMENSIONS=6 DUPLICATE_DIMENSIONS value
                             * @property {number} INVALID_CAMPAIGN_ID=7 INVALID_CAMPAIGN_ID value
                             * @property {number} CANNOT_PAUSE_UNLESS_ALL_VALUE_RULES_ARE_PAUSED=8 CANNOT_PAUSE_UNLESS_ALL_VALUE_RULES_ARE_PAUSED value
                             * @property {number} SHOULD_PAUSE_WHEN_ALL_VALUE_RULES_ARE_PAUSED=9 SHOULD_PAUSE_WHEN_ALL_VALUE_RULES_ARE_PAUSED value
                             * @property {number} VALUE_RULES_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE=10 VALUE_RULES_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE value
                             * @property {number} INELIGIBLE_CONVERSION_ACTION_CATEGORIES=11 INELIGIBLE_CONVERSION_ACTION_CATEGORIES value
                             * @property {number} DIMENSION_NO_CONDITION_USED_WITH_OTHER_DIMENSIONS=12 DIMENSION_NO_CONDITION_USED_WITH_OTHER_DIMENSIONS value
                             * @property {number} DIMENSION_NO_CONDITION_NOT_ALLOWED=13 DIMENSION_NO_CONDITION_NOT_ALLOWED value
                             * @property {number} UNSUPPORTED_CONVERSION_ACTION_CATEGORIES=14 UNSUPPORTED_CONVERSION_ACTION_CATEGORIES value
                             */
                            ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CONFLICTING_VALUE_RULE_CONDITIONS"] = 2;
                                values[valuesById[3] = "INVALID_VALUE_RULE"] = 3;
                                values[valuesById[4] = "DIMENSIONS_UPDATE_ONLY_ALLOW_APPEND"] = 4;
                                values[valuesById[5] = "CONDITION_TYPE_NOT_ALLOWED"] = 5;
                                values[valuesById[6] = "DUPLICATE_DIMENSIONS"] = 6;
                                values[valuesById[7] = "INVALID_CAMPAIGN_ID"] = 7;
                                values[valuesById[8] = "CANNOT_PAUSE_UNLESS_ALL_VALUE_RULES_ARE_PAUSED"] = 8;
                                values[valuesById[9] = "SHOULD_PAUSE_WHEN_ALL_VALUE_RULES_ARE_PAUSED"] = 9;
                                values[valuesById[10] = "VALUE_RULES_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE"] = 10;
                                values[valuesById[11] = "INELIGIBLE_CONVERSION_ACTION_CATEGORIES"] = 11;
                                values[valuesById[12] = "DIMENSION_NO_CONDITION_USED_WITH_OTHER_DIMENSIONS"] = 12;
                                values[valuesById[13] = "DIMENSION_NO_CONDITION_NOT_ALLOWED"] = 13;
                                values[valuesById[14] = "UNSUPPORTED_CONVERSION_ACTION_CATEGORIES"] = 14;
                                return values;
                            })();
    
                            return ConversionValueRuleSetErrorEnum;
                        })();
    
                        errors.CountryCodeErrorEnum = (function() {
    
                            /**
                             * Properties of a CountryCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICountryCodeErrorEnum
                             */
    
                            /**
                             * Constructs a new CountryCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CountryCodeErrorEnum.
                             * @implements ICountryCodeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICountryCodeErrorEnum=} [p] Properties to set
                             */
                            function CountryCodeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CountryCodeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICountryCodeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CountryCodeErrorEnum} CountryCodeErrorEnum instance
                             */
                            CountryCodeErrorEnum.create = function create(properties) {
                                return new CountryCodeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CountryCodeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CountryCodeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICountryCodeErrorEnum} m CountryCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CountryCodeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CountryCodeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CountryCodeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICountryCodeErrorEnum} message CountryCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CountryCodeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CountryCodeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CountryCodeErrorEnum} CountryCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CountryCodeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CountryCodeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CountryCodeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CountryCodeErrorEnum} CountryCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CountryCodeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CountryCodeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CountryCodeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CountryCodeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CountryCodeErrorEnum} CountryCodeErrorEnum
                             */
                            CountryCodeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CountryCodeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CountryCodeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CountryCodeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CountryCodeErrorEnum} m CountryCodeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CountryCodeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CountryCodeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CountryCodeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CountryCodeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CountryCodeError enum.
                             * @name google.ads.googleads.v11.errors.CountryCodeErrorEnum.CountryCodeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_COUNTRY_CODE=2 INVALID_COUNTRY_CODE value
                             */
                            CountryCodeErrorEnum.CountryCodeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_COUNTRY_CODE"] = 2;
                                return values;
                            })();
    
                            return CountryCodeErrorEnum;
                        })();
    
                        errors.CriterionErrorEnum = (function() {
    
                            /**
                             * Properties of a CriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICriterionErrorEnum
                             */
    
                            /**
                             * Constructs a new CriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CriterionErrorEnum.
                             * @implements ICriterionErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICriterionErrorEnum=} [p] Properties to set
                             */
                            function CriterionErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CriterionErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICriterionErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CriterionErrorEnum} CriterionErrorEnum instance
                             */
                            CriterionErrorEnum.create = function create(properties) {
                                return new CriterionErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CriterionErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CriterionErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICriterionErrorEnum} m CriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CriterionErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CriterionErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICriterionErrorEnum} message CriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CriterionErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CriterionErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CriterionErrorEnum} CriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CriterionErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CriterionErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CriterionErrorEnum} CriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CriterionErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CriterionErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CriterionErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CriterionErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CriterionErrorEnum} CriterionErrorEnum
                             */
                            CriterionErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CriterionErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CriterionErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CriterionErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CriterionErrorEnum} m CriterionErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CriterionErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CriterionErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CriterionErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CriterionErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CriterionError enum.
                             * @name google.ads.googleads.v11.errors.CriterionErrorEnum.CriterionError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CONCRETE_TYPE_REQUIRED=2 CONCRETE_TYPE_REQUIRED value
                             * @property {number} INVALID_EXCLUDED_CATEGORY=3 INVALID_EXCLUDED_CATEGORY value
                             * @property {number} INVALID_KEYWORD_TEXT=4 INVALID_KEYWORD_TEXT value
                             * @property {number} KEYWORD_TEXT_TOO_LONG=5 KEYWORD_TEXT_TOO_LONG value
                             * @property {number} KEYWORD_HAS_TOO_MANY_WORDS=6 KEYWORD_HAS_TOO_MANY_WORDS value
                             * @property {number} KEYWORD_HAS_INVALID_CHARS=7 KEYWORD_HAS_INVALID_CHARS value
                             * @property {number} INVALID_PLACEMENT_URL=8 INVALID_PLACEMENT_URL value
                             * @property {number} INVALID_USER_LIST=9 INVALID_USER_LIST value
                             * @property {number} INVALID_USER_INTEREST=10 INVALID_USER_INTEREST value
                             * @property {number} INVALID_FORMAT_FOR_PLACEMENT_URL=11 INVALID_FORMAT_FOR_PLACEMENT_URL value
                             * @property {number} PLACEMENT_URL_IS_TOO_LONG=12 PLACEMENT_URL_IS_TOO_LONG value
                             * @property {number} PLACEMENT_URL_HAS_ILLEGAL_CHAR=13 PLACEMENT_URL_HAS_ILLEGAL_CHAR value
                             * @property {number} PLACEMENT_URL_HAS_MULTIPLE_SITES_IN_LINE=14 PLACEMENT_URL_HAS_MULTIPLE_SITES_IN_LINE value
                             * @property {number} PLACEMENT_IS_NOT_AVAILABLE_FOR_TARGETING_OR_EXCLUSION=15 PLACEMENT_IS_NOT_AVAILABLE_FOR_TARGETING_OR_EXCLUSION value
                             * @property {number} INVALID_TOPIC_PATH=16 INVALID_TOPIC_PATH value
                             * @property {number} INVALID_YOUTUBE_CHANNEL_ID=17 INVALID_YOUTUBE_CHANNEL_ID value
                             * @property {number} INVALID_YOUTUBE_VIDEO_ID=18 INVALID_YOUTUBE_VIDEO_ID value
                             * @property {number} YOUTUBE_VERTICAL_CHANNEL_DEPRECATED=19 YOUTUBE_VERTICAL_CHANNEL_DEPRECATED value
                             * @property {number} YOUTUBE_DEMOGRAPHIC_CHANNEL_DEPRECATED=20 YOUTUBE_DEMOGRAPHIC_CHANNEL_DEPRECATED value
                             * @property {number} YOUTUBE_URL_UNSUPPORTED=21 YOUTUBE_URL_UNSUPPORTED value
                             * @property {number} CANNOT_EXCLUDE_CRITERIA_TYPE=22 CANNOT_EXCLUDE_CRITERIA_TYPE value
                             * @property {number} CANNOT_ADD_CRITERIA_TYPE=23 CANNOT_ADD_CRITERIA_TYPE value
                             * @property {number} CANNOT_EXCLUDE_SIMILAR_USER_LIST=26 CANNOT_EXCLUDE_SIMILAR_USER_LIST value
                             * @property {number} CANNOT_ADD_CLOSED_USER_LIST=27 CANNOT_ADD_CLOSED_USER_LIST value
                             * @property {number} CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_ONLY_CAMPAIGNS=28 CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_ONLY_CAMPAIGNS value
                             * @property {number} CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_CAMPAIGNS=29 CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_CAMPAIGNS value
                             * @property {number} CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SHOPPING_CAMPAIGNS=30 CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SHOPPING_CAMPAIGNS value
                             * @property {number} CANNOT_ADD_USER_INTERESTS_TO_SEARCH_CAMPAIGNS=31 CANNOT_ADD_USER_INTERESTS_TO_SEARCH_CAMPAIGNS value
                             * @property {number} CANNOT_SET_BIDS_ON_CRITERION_TYPE_IN_SEARCH_CAMPAIGNS=32 CANNOT_SET_BIDS_ON_CRITERION_TYPE_IN_SEARCH_CAMPAIGNS value
                             * @property {number} CANNOT_ADD_URLS_TO_CRITERION_TYPE_FOR_CAMPAIGN_TYPE=33 CANNOT_ADD_URLS_TO_CRITERION_TYPE_FOR_CAMPAIGN_TYPE value
                             * @property {number} INVALID_COMBINED_AUDIENCE=122 INVALID_COMBINED_AUDIENCE value
                             * @property {number} INVALID_CUSTOM_AFFINITY=96 INVALID_CUSTOM_AFFINITY value
                             * @property {number} INVALID_CUSTOM_INTENT=97 INVALID_CUSTOM_INTENT value
                             * @property {number} INVALID_CUSTOM_AUDIENCE=121 INVALID_CUSTOM_AUDIENCE value
                             * @property {number} INVALID_IP_ADDRESS=34 INVALID_IP_ADDRESS value
                             * @property {number} INVALID_IP_FORMAT=35 INVALID_IP_FORMAT value
                             * @property {number} INVALID_MOBILE_APP=36 INVALID_MOBILE_APP value
                             * @property {number} INVALID_MOBILE_APP_CATEGORY=37 INVALID_MOBILE_APP_CATEGORY value
                             * @property {number} INVALID_CRITERION_ID=38 INVALID_CRITERION_ID value
                             * @property {number} CANNOT_TARGET_CRITERION=39 CANNOT_TARGET_CRITERION value
                             * @property {number} CANNOT_TARGET_OBSOLETE_CRITERION=40 CANNOT_TARGET_OBSOLETE_CRITERION value
                             * @property {number} CRITERION_ID_AND_TYPE_MISMATCH=41 CRITERION_ID_AND_TYPE_MISMATCH value
                             * @property {number} INVALID_PROXIMITY_RADIUS=42 INVALID_PROXIMITY_RADIUS value
                             * @property {number} INVALID_PROXIMITY_RADIUS_UNITS=43 INVALID_PROXIMITY_RADIUS_UNITS value
                             * @property {number} INVALID_STREETADDRESS_LENGTH=44 INVALID_STREETADDRESS_LENGTH value
                             * @property {number} INVALID_CITYNAME_LENGTH=45 INVALID_CITYNAME_LENGTH value
                             * @property {number} INVALID_REGIONCODE_LENGTH=46 INVALID_REGIONCODE_LENGTH value
                             * @property {number} INVALID_REGIONNAME_LENGTH=47 INVALID_REGIONNAME_LENGTH value
                             * @property {number} INVALID_POSTALCODE_LENGTH=48 INVALID_POSTALCODE_LENGTH value
                             * @property {number} INVALID_COUNTRY_CODE=49 INVALID_COUNTRY_CODE value
                             * @property {number} INVALID_LATITUDE=50 INVALID_LATITUDE value
                             * @property {number} INVALID_LONGITUDE=51 INVALID_LONGITUDE value
                             * @property {number} PROXIMITY_GEOPOINT_AND_ADDRESS_BOTH_CANNOT_BE_NULL=52 PROXIMITY_GEOPOINT_AND_ADDRESS_BOTH_CANNOT_BE_NULL value
                             * @property {number} INVALID_PROXIMITY_ADDRESS=53 INVALID_PROXIMITY_ADDRESS value
                             * @property {number} INVALID_USER_DOMAIN_NAME=54 INVALID_USER_DOMAIN_NAME value
                             * @property {number} CRITERION_PARAMETER_TOO_LONG=55 CRITERION_PARAMETER_TOO_LONG value
                             * @property {number} AD_SCHEDULE_TIME_INTERVALS_OVERLAP=56 AD_SCHEDULE_TIME_INTERVALS_OVERLAP value
                             * @property {number} AD_SCHEDULE_INTERVAL_CANNOT_SPAN_MULTIPLE_DAYS=57 AD_SCHEDULE_INTERVAL_CANNOT_SPAN_MULTIPLE_DAYS value
                             * @property {number} AD_SCHEDULE_INVALID_TIME_INTERVAL=58 AD_SCHEDULE_INVALID_TIME_INTERVAL value
                             * @property {number} AD_SCHEDULE_EXCEEDED_INTERVALS_PER_DAY_LIMIT=59 AD_SCHEDULE_EXCEEDED_INTERVALS_PER_DAY_LIMIT value
                             * @property {number} AD_SCHEDULE_CRITERION_ID_MISMATCHING_FIELDS=60 AD_SCHEDULE_CRITERION_ID_MISMATCHING_FIELDS value
                             * @property {number} CANNOT_BID_MODIFY_CRITERION_TYPE=61 CANNOT_BID_MODIFY_CRITERION_TYPE value
                             * @property {number} CANNOT_BID_MODIFY_CRITERION_CAMPAIGN_OPTED_OUT=62 CANNOT_BID_MODIFY_CRITERION_CAMPAIGN_OPTED_OUT value
                             * @property {number} CANNOT_BID_MODIFY_NEGATIVE_CRITERION=63 CANNOT_BID_MODIFY_NEGATIVE_CRITERION value
                             * @property {number} BID_MODIFIER_ALREADY_EXISTS=64 BID_MODIFIER_ALREADY_EXISTS value
                             * @property {number} FEED_ID_NOT_ALLOWED=65 FEED_ID_NOT_ALLOWED value
                             * @property {number} ACCOUNT_INELIGIBLE_FOR_CRITERIA_TYPE=66 ACCOUNT_INELIGIBLE_FOR_CRITERIA_TYPE value
                             * @property {number} CRITERIA_TYPE_INVALID_FOR_BIDDING_STRATEGY=67 CRITERIA_TYPE_INVALID_FOR_BIDDING_STRATEGY value
                             * @property {number} CANNOT_EXCLUDE_CRITERION=68 CANNOT_EXCLUDE_CRITERION value
                             * @property {number} CANNOT_REMOVE_CRITERION=69 CANNOT_REMOVE_CRITERION value
                             * @property {number} INVALID_PRODUCT_BIDDING_CATEGORY=76 INVALID_PRODUCT_BIDDING_CATEGORY value
                             * @property {number} MISSING_SHOPPING_SETTING=77 MISSING_SHOPPING_SETTING value
                             * @property {number} INVALID_MATCHING_FUNCTION=78 INVALID_MATCHING_FUNCTION value
                             * @property {number} LOCATION_FILTER_NOT_ALLOWED=79 LOCATION_FILTER_NOT_ALLOWED value
                             * @property {number} INVALID_FEED_FOR_LOCATION_FILTER=98 INVALID_FEED_FOR_LOCATION_FILTER value
                             * @property {number} LOCATION_FILTER_INVALID=80 LOCATION_FILTER_INVALID value
                             * @property {number} CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_FEED_ITEM_SETS=123 CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_FEED_ITEM_SETS value
                             * @property {number} INVALID_LOCATION_GROUP_RADIUS=124 INVALID_LOCATION_GROUP_RADIUS value
                             * @property {number} INVALID_LOCATION_GROUP_RADIUS_UNIT=125 INVALID_LOCATION_GROUP_RADIUS_UNIT value
                             * @property {number} CANNOT_ATTACH_CRITERIA_AT_CAMPAIGN_AND_ADGROUP=81 CANNOT_ATTACH_CRITERIA_AT_CAMPAIGN_AND_ADGROUP value
                             * @property {number} HOTEL_LENGTH_OF_STAY_OVERLAPS_WITH_EXISTING_CRITERION=82 HOTEL_LENGTH_OF_STAY_OVERLAPS_WITH_EXISTING_CRITERION value
                             * @property {number} HOTEL_ADVANCE_BOOKING_WINDOW_OVERLAPS_WITH_EXISTING_CRITERION=83 HOTEL_ADVANCE_BOOKING_WINDOW_OVERLAPS_WITH_EXISTING_CRITERION value
                             * @property {number} FIELD_INCOMPATIBLE_WITH_NEGATIVE_TARGETING=84 FIELD_INCOMPATIBLE_WITH_NEGATIVE_TARGETING value
                             * @property {number} INVALID_WEBPAGE_CONDITION=85 INVALID_WEBPAGE_CONDITION value
                             * @property {number} INVALID_WEBPAGE_CONDITION_URL=86 INVALID_WEBPAGE_CONDITION_URL value
                             * @property {number} WEBPAGE_CONDITION_URL_CANNOT_BE_EMPTY=87 WEBPAGE_CONDITION_URL_CANNOT_BE_EMPTY value
                             * @property {number} WEBPAGE_CONDITION_URL_UNSUPPORTED_PROTOCOL=88 WEBPAGE_CONDITION_URL_UNSUPPORTED_PROTOCOL value
                             * @property {number} WEBPAGE_CONDITION_URL_CANNOT_BE_IP_ADDRESS=89 WEBPAGE_CONDITION_URL_CANNOT_BE_IP_ADDRESS value
                             * @property {number} WEBPAGE_CONDITION_URL_DOMAIN_NOT_CONSISTENT_WITH_CAMPAIGN_SETTING=90 WEBPAGE_CONDITION_URL_DOMAIN_NOT_CONSISTENT_WITH_CAMPAIGN_SETTING value
                             * @property {number} WEBPAGE_CONDITION_URL_CANNOT_BE_PUBLIC_SUFFIX=91 WEBPAGE_CONDITION_URL_CANNOT_BE_PUBLIC_SUFFIX value
                             * @property {number} WEBPAGE_CONDITION_URL_INVALID_PUBLIC_SUFFIX=92 WEBPAGE_CONDITION_URL_INVALID_PUBLIC_SUFFIX value
                             * @property {number} WEBPAGE_CONDITION_URL_VALUE_TRACK_VALUE_NOT_SUPPORTED=93 WEBPAGE_CONDITION_URL_VALUE_TRACK_VALUE_NOT_SUPPORTED value
                             * @property {number} WEBPAGE_CRITERION_URL_EQUALS_CAN_HAVE_ONLY_ONE_CONDITION=94 WEBPAGE_CRITERION_URL_EQUALS_CAN_HAVE_ONLY_ONE_CONDITION value
                             * @property {number} WEBPAGE_CRITERION_NOT_SUPPORTED_ON_NON_DSA_AD_GROUP=95 WEBPAGE_CRITERION_NOT_SUPPORTED_ON_NON_DSA_AD_GROUP value
                             * @property {number} CANNOT_TARGET_USER_LIST_FOR_SMART_DISPLAY_CAMPAIGNS=99 CANNOT_TARGET_USER_LIST_FOR_SMART_DISPLAY_CAMPAIGNS value
                             * @property {number} CANNOT_TARGET_PLACEMENTS_FOR_SEARCH_CAMPAIGNS=126 CANNOT_TARGET_PLACEMENTS_FOR_SEARCH_CAMPAIGNS value
                             * @property {number} LISTING_SCOPE_TOO_MANY_DIMENSION_TYPES=100 LISTING_SCOPE_TOO_MANY_DIMENSION_TYPES value
                             * @property {number} LISTING_SCOPE_TOO_MANY_IN_OPERATORS=101 LISTING_SCOPE_TOO_MANY_IN_OPERATORS value
                             * @property {number} LISTING_SCOPE_IN_OPERATOR_NOT_SUPPORTED=102 LISTING_SCOPE_IN_OPERATOR_NOT_SUPPORTED value
                             * @property {number} DUPLICATE_LISTING_DIMENSION_TYPE=103 DUPLICATE_LISTING_DIMENSION_TYPE value
                             * @property {number} DUPLICATE_LISTING_DIMENSION_VALUE=104 DUPLICATE_LISTING_DIMENSION_VALUE value
                             * @property {number} CANNOT_SET_BIDS_ON_LISTING_GROUP_SUBDIVISION=105 CANNOT_SET_BIDS_ON_LISTING_GROUP_SUBDIVISION value
                             * @property {number} INVALID_LISTING_GROUP_HIERARCHY=106 INVALID_LISTING_GROUP_HIERARCHY value
                             * @property {number} LISTING_GROUP_UNIT_CANNOT_HAVE_CHILDREN=107 LISTING_GROUP_UNIT_CANNOT_HAVE_CHILDREN value
                             * @property {number} LISTING_GROUP_SUBDIVISION_REQUIRES_OTHERS_CASE=108 LISTING_GROUP_SUBDIVISION_REQUIRES_OTHERS_CASE value
                             * @property {number} LISTING_GROUP_REQUIRES_SAME_DIMENSION_TYPE_AS_SIBLINGS=109 LISTING_GROUP_REQUIRES_SAME_DIMENSION_TYPE_AS_SIBLINGS value
                             * @property {number} LISTING_GROUP_ALREADY_EXISTS=110 LISTING_GROUP_ALREADY_EXISTS value
                             * @property {number} LISTING_GROUP_DOES_NOT_EXIST=111 LISTING_GROUP_DOES_NOT_EXIST value
                             * @property {number} LISTING_GROUP_CANNOT_BE_REMOVED=112 LISTING_GROUP_CANNOT_BE_REMOVED value
                             * @property {number} INVALID_LISTING_GROUP_TYPE=113 INVALID_LISTING_GROUP_TYPE value
                             * @property {number} LISTING_GROUP_ADD_MAY_ONLY_USE_TEMP_ID=114 LISTING_GROUP_ADD_MAY_ONLY_USE_TEMP_ID value
                             * @property {number} LISTING_SCOPE_TOO_LONG=115 LISTING_SCOPE_TOO_LONG value
                             * @property {number} LISTING_SCOPE_TOO_MANY_DIMENSIONS=116 LISTING_SCOPE_TOO_MANY_DIMENSIONS value
                             * @property {number} LISTING_GROUP_TOO_LONG=117 LISTING_GROUP_TOO_LONG value
                             * @property {number} LISTING_GROUP_TREE_TOO_DEEP=118 LISTING_GROUP_TREE_TOO_DEEP value
                             * @property {number} INVALID_LISTING_DIMENSION=119 INVALID_LISTING_DIMENSION value
                             * @property {number} INVALID_LISTING_DIMENSION_TYPE=120 INVALID_LISTING_DIMENSION_TYPE value
                             * @property {number} ADVERTISER_NOT_ON_ALLOWLIST_FOR_COMBINED_AUDIENCE_ON_DISPLAY=127 ADVERTISER_NOT_ON_ALLOWLIST_FOR_COMBINED_AUDIENCE_ON_DISPLAY value
                             * @property {number} CANNOT_TARGET_REMOVED_COMBINED_AUDIENCE=128 CANNOT_TARGET_REMOVED_COMBINED_AUDIENCE value
                             * @property {number} INVALID_COMBINED_AUDIENCE_ID=129 INVALID_COMBINED_AUDIENCE_ID value
                             * @property {number} CANNOT_TARGET_REMOVED_CUSTOM_AUDIENCE=130 CANNOT_TARGET_REMOVED_CUSTOM_AUDIENCE value
                             * @property {number} HOTEL_CHECK_IN_DATE_RANGE_OVERLAPS_WITH_EXISTING_CRITERION=131 HOTEL_CHECK_IN_DATE_RANGE_OVERLAPS_WITH_EXISTING_CRITERION value
                             * @property {number} HOTEL_CHECK_IN_DATE_RANGE_START_DATE_TOO_EARLY=132 HOTEL_CHECK_IN_DATE_RANGE_START_DATE_TOO_EARLY value
                             * @property {number} HOTEL_CHECK_IN_DATE_RANGE_END_DATE_TOO_LATE=133 HOTEL_CHECK_IN_DATE_RANGE_END_DATE_TOO_LATE value
                             * @property {number} HOTEL_CHECK_IN_DATE_RANGE_REVERSED=134 HOTEL_CHECK_IN_DATE_RANGE_REVERSED value
                             * @property {number} BROAD_MATCH_MODIFIER_KEYWORD_NOT_ALLOWED=135 BROAD_MATCH_MODIFIER_KEYWORD_NOT_ALLOWED value
                             * @property {number} ONE_AUDIENCE_ALLOWED_PER_ASSET_GROUP=136 ONE_AUDIENCE_ALLOWED_PER_ASSET_GROUP value
                             * @property {number} AUDIENCE_NOT_ELIGIBLE_FOR_CAMPAIGN_TYPE=137 AUDIENCE_NOT_ELIGIBLE_FOR_CAMPAIGN_TYPE value
                             * @property {number} AUDIENCE_NOT_ALLOWED_TO_ATTACH_WHEN_AUDIENCE_GROUPED_SET_TO_FALSE=138 AUDIENCE_NOT_ALLOWED_TO_ATTACH_WHEN_AUDIENCE_GROUPED_SET_TO_FALSE value
                             * @property {number} CANNOT_TARGET_CUSTOMER_MATCH_USER_LIST=139 CANNOT_TARGET_CUSTOMER_MATCH_USER_LIST value
                             */
                            CriterionErrorEnum.CriterionError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CONCRETE_TYPE_REQUIRED"] = 2;
                                values[valuesById[3] = "INVALID_EXCLUDED_CATEGORY"] = 3;
                                values[valuesById[4] = "INVALID_KEYWORD_TEXT"] = 4;
                                values[valuesById[5] = "KEYWORD_TEXT_TOO_LONG"] = 5;
                                values[valuesById[6] = "KEYWORD_HAS_TOO_MANY_WORDS"] = 6;
                                values[valuesById[7] = "KEYWORD_HAS_INVALID_CHARS"] = 7;
                                values[valuesById[8] = "INVALID_PLACEMENT_URL"] = 8;
                                values[valuesById[9] = "INVALID_USER_LIST"] = 9;
                                values[valuesById[10] = "INVALID_USER_INTEREST"] = 10;
                                values[valuesById[11] = "INVALID_FORMAT_FOR_PLACEMENT_URL"] = 11;
                                values[valuesById[12] = "PLACEMENT_URL_IS_TOO_LONG"] = 12;
                                values[valuesById[13] = "PLACEMENT_URL_HAS_ILLEGAL_CHAR"] = 13;
                                values[valuesById[14] = "PLACEMENT_URL_HAS_MULTIPLE_SITES_IN_LINE"] = 14;
                                values[valuesById[15] = "PLACEMENT_IS_NOT_AVAILABLE_FOR_TARGETING_OR_EXCLUSION"] = 15;
                                values[valuesById[16] = "INVALID_TOPIC_PATH"] = 16;
                                values[valuesById[17] = "INVALID_YOUTUBE_CHANNEL_ID"] = 17;
                                values[valuesById[18] = "INVALID_YOUTUBE_VIDEO_ID"] = 18;
                                values[valuesById[19] = "YOUTUBE_VERTICAL_CHANNEL_DEPRECATED"] = 19;
                                values[valuesById[20] = "YOUTUBE_DEMOGRAPHIC_CHANNEL_DEPRECATED"] = 20;
                                values[valuesById[21] = "YOUTUBE_URL_UNSUPPORTED"] = 21;
                                values[valuesById[22] = "CANNOT_EXCLUDE_CRITERIA_TYPE"] = 22;
                                values[valuesById[23] = "CANNOT_ADD_CRITERIA_TYPE"] = 23;
                                values[valuesById[26] = "CANNOT_EXCLUDE_SIMILAR_USER_LIST"] = 26;
                                values[valuesById[27] = "CANNOT_ADD_CLOSED_USER_LIST"] = 27;
                                values[valuesById[28] = "CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_ONLY_CAMPAIGNS"] = 28;
                                values[valuesById[29] = "CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_CAMPAIGNS"] = 29;
                                values[valuesById[30] = "CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SHOPPING_CAMPAIGNS"] = 30;
                                values[valuesById[31] = "CANNOT_ADD_USER_INTERESTS_TO_SEARCH_CAMPAIGNS"] = 31;
                                values[valuesById[32] = "CANNOT_SET_BIDS_ON_CRITERION_TYPE_IN_SEARCH_CAMPAIGNS"] = 32;
                                values[valuesById[33] = "CANNOT_ADD_URLS_TO_CRITERION_TYPE_FOR_CAMPAIGN_TYPE"] = 33;
                                values[valuesById[122] = "INVALID_COMBINED_AUDIENCE"] = 122;
                                values[valuesById[96] = "INVALID_CUSTOM_AFFINITY"] = 96;
                                values[valuesById[97] = "INVALID_CUSTOM_INTENT"] = 97;
                                values[valuesById[121] = "INVALID_CUSTOM_AUDIENCE"] = 121;
                                values[valuesById[34] = "INVALID_IP_ADDRESS"] = 34;
                                values[valuesById[35] = "INVALID_IP_FORMAT"] = 35;
                                values[valuesById[36] = "INVALID_MOBILE_APP"] = 36;
                                values[valuesById[37] = "INVALID_MOBILE_APP_CATEGORY"] = 37;
                                values[valuesById[38] = "INVALID_CRITERION_ID"] = 38;
                                values[valuesById[39] = "CANNOT_TARGET_CRITERION"] = 39;
                                values[valuesById[40] = "CANNOT_TARGET_OBSOLETE_CRITERION"] = 40;
                                values[valuesById[41] = "CRITERION_ID_AND_TYPE_MISMATCH"] = 41;
                                values[valuesById[42] = "INVALID_PROXIMITY_RADIUS"] = 42;
                                values[valuesById[43] = "INVALID_PROXIMITY_RADIUS_UNITS"] = 43;
                                values[valuesById[44] = "INVALID_STREETADDRESS_LENGTH"] = 44;
                                values[valuesById[45] = "INVALID_CITYNAME_LENGTH"] = 45;
                                values[valuesById[46] = "INVALID_REGIONCODE_LENGTH"] = 46;
                                values[valuesById[47] = "INVALID_REGIONNAME_LENGTH"] = 47;
                                values[valuesById[48] = "INVALID_POSTALCODE_LENGTH"] = 48;
                                values[valuesById[49] = "INVALID_COUNTRY_CODE"] = 49;
                                values[valuesById[50] = "INVALID_LATITUDE"] = 50;
                                values[valuesById[51] = "INVALID_LONGITUDE"] = 51;
                                values[valuesById[52] = "PROXIMITY_GEOPOINT_AND_ADDRESS_BOTH_CANNOT_BE_NULL"] = 52;
                                values[valuesById[53] = "INVALID_PROXIMITY_ADDRESS"] = 53;
                                values[valuesById[54] = "INVALID_USER_DOMAIN_NAME"] = 54;
                                values[valuesById[55] = "CRITERION_PARAMETER_TOO_LONG"] = 55;
                                values[valuesById[56] = "AD_SCHEDULE_TIME_INTERVALS_OVERLAP"] = 56;
                                values[valuesById[57] = "AD_SCHEDULE_INTERVAL_CANNOT_SPAN_MULTIPLE_DAYS"] = 57;
                                values[valuesById[58] = "AD_SCHEDULE_INVALID_TIME_INTERVAL"] = 58;
                                values[valuesById[59] = "AD_SCHEDULE_EXCEEDED_INTERVALS_PER_DAY_LIMIT"] = 59;
                                values[valuesById[60] = "AD_SCHEDULE_CRITERION_ID_MISMATCHING_FIELDS"] = 60;
                                values[valuesById[61] = "CANNOT_BID_MODIFY_CRITERION_TYPE"] = 61;
                                values[valuesById[62] = "CANNOT_BID_MODIFY_CRITERION_CAMPAIGN_OPTED_OUT"] = 62;
                                values[valuesById[63] = "CANNOT_BID_MODIFY_NEGATIVE_CRITERION"] = 63;
                                values[valuesById[64] = "BID_MODIFIER_ALREADY_EXISTS"] = 64;
                                values[valuesById[65] = "FEED_ID_NOT_ALLOWED"] = 65;
                                values[valuesById[66] = "ACCOUNT_INELIGIBLE_FOR_CRITERIA_TYPE"] = 66;
                                values[valuesById[67] = "CRITERIA_TYPE_INVALID_FOR_BIDDING_STRATEGY"] = 67;
                                values[valuesById[68] = "CANNOT_EXCLUDE_CRITERION"] = 68;
                                values[valuesById[69] = "CANNOT_REMOVE_CRITERION"] = 69;
                                values[valuesById[76] = "INVALID_PRODUCT_BIDDING_CATEGORY"] = 76;
                                values[valuesById[77] = "MISSING_SHOPPING_SETTING"] = 77;
                                values[valuesById[78] = "INVALID_MATCHING_FUNCTION"] = 78;
                                values[valuesById[79] = "LOCATION_FILTER_NOT_ALLOWED"] = 79;
                                values[valuesById[98] = "INVALID_FEED_FOR_LOCATION_FILTER"] = 98;
                                values[valuesById[80] = "LOCATION_FILTER_INVALID"] = 80;
                                values[valuesById[123] = "CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_FEED_ITEM_SETS"] = 123;
                                values[valuesById[124] = "INVALID_LOCATION_GROUP_RADIUS"] = 124;
                                values[valuesById[125] = "INVALID_LOCATION_GROUP_RADIUS_UNIT"] = 125;
                                values[valuesById[81] = "CANNOT_ATTACH_CRITERIA_AT_CAMPAIGN_AND_ADGROUP"] = 81;
                                values[valuesById[82] = "HOTEL_LENGTH_OF_STAY_OVERLAPS_WITH_EXISTING_CRITERION"] = 82;
                                values[valuesById[83] = "HOTEL_ADVANCE_BOOKING_WINDOW_OVERLAPS_WITH_EXISTING_CRITERION"] = 83;
                                values[valuesById[84] = "FIELD_INCOMPATIBLE_WITH_NEGATIVE_TARGETING"] = 84;
                                values[valuesById[85] = "INVALID_WEBPAGE_CONDITION"] = 85;
                                values[valuesById[86] = "INVALID_WEBPAGE_CONDITION_URL"] = 86;
                                values[valuesById[87] = "WEBPAGE_CONDITION_URL_CANNOT_BE_EMPTY"] = 87;
                                values[valuesById[88] = "WEBPAGE_CONDITION_URL_UNSUPPORTED_PROTOCOL"] = 88;
                                values[valuesById[89] = "WEBPAGE_CONDITION_URL_CANNOT_BE_IP_ADDRESS"] = 89;
                                values[valuesById[90] = "WEBPAGE_CONDITION_URL_DOMAIN_NOT_CONSISTENT_WITH_CAMPAIGN_SETTING"] = 90;
                                values[valuesById[91] = "WEBPAGE_CONDITION_URL_CANNOT_BE_PUBLIC_SUFFIX"] = 91;
                                values[valuesById[92] = "WEBPAGE_CONDITION_URL_INVALID_PUBLIC_SUFFIX"] = 92;
                                values[valuesById[93] = "WEBPAGE_CONDITION_URL_VALUE_TRACK_VALUE_NOT_SUPPORTED"] = 93;
                                values[valuesById[94] = "WEBPAGE_CRITERION_URL_EQUALS_CAN_HAVE_ONLY_ONE_CONDITION"] = 94;
                                values[valuesById[95] = "WEBPAGE_CRITERION_NOT_SUPPORTED_ON_NON_DSA_AD_GROUP"] = 95;
                                values[valuesById[99] = "CANNOT_TARGET_USER_LIST_FOR_SMART_DISPLAY_CAMPAIGNS"] = 99;
                                values[valuesById[126] = "CANNOT_TARGET_PLACEMENTS_FOR_SEARCH_CAMPAIGNS"] = 126;
                                values[valuesById[100] = "LISTING_SCOPE_TOO_MANY_DIMENSION_TYPES"] = 100;
                                values[valuesById[101] = "LISTING_SCOPE_TOO_MANY_IN_OPERATORS"] = 101;
                                values[valuesById[102] = "LISTING_SCOPE_IN_OPERATOR_NOT_SUPPORTED"] = 102;
                                values[valuesById[103] = "DUPLICATE_LISTING_DIMENSION_TYPE"] = 103;
                                values[valuesById[104] = "DUPLICATE_LISTING_DIMENSION_VALUE"] = 104;
                                values[valuesById[105] = "CANNOT_SET_BIDS_ON_LISTING_GROUP_SUBDIVISION"] = 105;
                                values[valuesById[106] = "INVALID_LISTING_GROUP_HIERARCHY"] = 106;
                                values[valuesById[107] = "LISTING_GROUP_UNIT_CANNOT_HAVE_CHILDREN"] = 107;
                                values[valuesById[108] = "LISTING_GROUP_SUBDIVISION_REQUIRES_OTHERS_CASE"] = 108;
                                values[valuesById[109] = "LISTING_GROUP_REQUIRES_SAME_DIMENSION_TYPE_AS_SIBLINGS"] = 109;
                                values[valuesById[110] = "LISTING_GROUP_ALREADY_EXISTS"] = 110;
                                values[valuesById[111] = "LISTING_GROUP_DOES_NOT_EXIST"] = 111;
                                values[valuesById[112] = "LISTING_GROUP_CANNOT_BE_REMOVED"] = 112;
                                values[valuesById[113] = "INVALID_LISTING_GROUP_TYPE"] = 113;
                                values[valuesById[114] = "LISTING_GROUP_ADD_MAY_ONLY_USE_TEMP_ID"] = 114;
                                values[valuesById[115] = "LISTING_SCOPE_TOO_LONG"] = 115;
                                values[valuesById[116] = "LISTING_SCOPE_TOO_MANY_DIMENSIONS"] = 116;
                                values[valuesById[117] = "LISTING_GROUP_TOO_LONG"] = 117;
                                values[valuesById[118] = "LISTING_GROUP_TREE_TOO_DEEP"] = 118;
                                values[valuesById[119] = "INVALID_LISTING_DIMENSION"] = 119;
                                values[valuesById[120] = "INVALID_LISTING_DIMENSION_TYPE"] = 120;
                                values[valuesById[127] = "ADVERTISER_NOT_ON_ALLOWLIST_FOR_COMBINED_AUDIENCE_ON_DISPLAY"] = 127;
                                values[valuesById[128] = "CANNOT_TARGET_REMOVED_COMBINED_AUDIENCE"] = 128;
                                values[valuesById[129] = "INVALID_COMBINED_AUDIENCE_ID"] = 129;
                                values[valuesById[130] = "CANNOT_TARGET_REMOVED_CUSTOM_AUDIENCE"] = 130;
                                values[valuesById[131] = "HOTEL_CHECK_IN_DATE_RANGE_OVERLAPS_WITH_EXISTING_CRITERION"] = 131;
                                values[valuesById[132] = "HOTEL_CHECK_IN_DATE_RANGE_START_DATE_TOO_EARLY"] = 132;
                                values[valuesById[133] = "HOTEL_CHECK_IN_DATE_RANGE_END_DATE_TOO_LATE"] = 133;
                                values[valuesById[134] = "HOTEL_CHECK_IN_DATE_RANGE_REVERSED"] = 134;
                                values[valuesById[135] = "BROAD_MATCH_MODIFIER_KEYWORD_NOT_ALLOWED"] = 135;
                                values[valuesById[136] = "ONE_AUDIENCE_ALLOWED_PER_ASSET_GROUP"] = 136;
                                values[valuesById[137] = "AUDIENCE_NOT_ELIGIBLE_FOR_CAMPAIGN_TYPE"] = 137;
                                values[valuesById[138] = "AUDIENCE_NOT_ALLOWED_TO_ATTACH_WHEN_AUDIENCE_GROUPED_SET_TO_FALSE"] = 138;
                                values[valuesById[139] = "CANNOT_TARGET_CUSTOMER_MATCH_USER_LIST"] = 139;
                                return values;
                            })();
    
                            return CriterionErrorEnum;
                        })();
    
                        errors.CurrencyCodeErrorEnum = (function() {
    
                            /**
                             * Properties of a CurrencyCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICurrencyCodeErrorEnum
                             */
    
                            /**
                             * Constructs a new CurrencyCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CurrencyCodeErrorEnum.
                             * @implements ICurrencyCodeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICurrencyCodeErrorEnum=} [p] Properties to set
                             */
                            function CurrencyCodeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CurrencyCodeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICurrencyCodeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CurrencyCodeErrorEnum} CurrencyCodeErrorEnum instance
                             */
                            CurrencyCodeErrorEnum.create = function create(properties) {
                                return new CurrencyCodeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CurrencyCodeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CurrencyCodeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICurrencyCodeErrorEnum} m CurrencyCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CurrencyCodeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CurrencyCodeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CurrencyCodeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICurrencyCodeErrorEnum} message CurrencyCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CurrencyCodeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CurrencyCodeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CurrencyCodeErrorEnum} CurrencyCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CurrencyCodeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CurrencyCodeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CurrencyCodeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CurrencyCodeErrorEnum} CurrencyCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CurrencyCodeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CurrencyCodeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CurrencyCodeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CurrencyCodeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CurrencyCodeErrorEnum} CurrencyCodeErrorEnum
                             */
                            CurrencyCodeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CurrencyCodeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CurrencyCodeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CurrencyCodeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CurrencyCodeErrorEnum} m CurrencyCodeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CurrencyCodeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CurrencyCodeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CurrencyCodeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CurrencyCodeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CurrencyCodeError enum.
                             * @name google.ads.googleads.v11.errors.CurrencyCodeErrorEnum.CurrencyCodeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNSUPPORTED=2 UNSUPPORTED value
                             */
                            CurrencyCodeErrorEnum.CurrencyCodeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNSUPPORTED"] = 2;
                                return values;
                            })();
    
                            return CurrencyCodeErrorEnum;
                        })();
    
                        errors.CustomAudienceErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomAudienceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomAudienceErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomAudienceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomAudienceErrorEnum.
                             * @implements ICustomAudienceErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomAudienceErrorEnum=} [p] Properties to set
                             */
                            function CustomAudienceErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomAudienceErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomAudienceErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomAudienceErrorEnum} CustomAudienceErrorEnum instance
                             */
                            CustomAudienceErrorEnum.create = function create(properties) {
                                return new CustomAudienceErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomAudienceErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomAudienceErrorEnum} m CustomAudienceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomAudienceErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomAudienceErrorEnum} message CustomAudienceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomAudienceErrorEnum} CustomAudienceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomAudienceErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomAudienceErrorEnum} CustomAudienceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomAudienceErrorEnum} CustomAudienceErrorEnum
                             */
                            CustomAudienceErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomAudienceErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomAudienceErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomAudienceErrorEnum} m CustomAudienceErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomAudienceErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomAudienceErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomAudienceError enum.
                             * @name google.ads.googleads.v11.errors.CustomAudienceErrorEnum.CustomAudienceError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NAME_ALREADY_USED=2 NAME_ALREADY_USED value
                             * @property {number} CANNOT_REMOVE_WHILE_IN_USE=3 CANNOT_REMOVE_WHILE_IN_USE value
                             * @property {number} RESOURCE_ALREADY_REMOVED=4 RESOURCE_ALREADY_REMOVED value
                             * @property {number} MEMBER_TYPE_AND_PARAMETER_ALREADY_EXISTED=5 MEMBER_TYPE_AND_PARAMETER_ALREADY_EXISTED value
                             * @property {number} INVALID_MEMBER_TYPE=6 INVALID_MEMBER_TYPE value
                             * @property {number} MEMBER_TYPE_AND_VALUE_DOES_NOT_MATCH=7 MEMBER_TYPE_AND_VALUE_DOES_NOT_MATCH value
                             * @property {number} POLICY_VIOLATION=8 POLICY_VIOLATION value
                             * @property {number} INVALID_TYPE_CHANGE=9 INVALID_TYPE_CHANGE value
                             */
                            CustomAudienceErrorEnum.CustomAudienceError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NAME_ALREADY_USED"] = 2;
                                values[valuesById[3] = "CANNOT_REMOVE_WHILE_IN_USE"] = 3;
                                values[valuesById[4] = "RESOURCE_ALREADY_REMOVED"] = 4;
                                values[valuesById[5] = "MEMBER_TYPE_AND_PARAMETER_ALREADY_EXISTED"] = 5;
                                values[valuesById[6] = "INVALID_MEMBER_TYPE"] = 6;
                                values[valuesById[7] = "MEMBER_TYPE_AND_VALUE_DOES_NOT_MATCH"] = 7;
                                values[valuesById[8] = "POLICY_VIOLATION"] = 8;
                                values[valuesById[9] = "INVALID_TYPE_CHANGE"] = 9;
                                return values;
                            })();
    
                            return CustomAudienceErrorEnum;
                        })();
    
                        errors.CustomConversionGoalErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomConversionGoalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomConversionGoalErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomConversionGoalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomConversionGoalErrorEnum.
                             * @implements ICustomConversionGoalErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomConversionGoalErrorEnum=} [p] Properties to set
                             */
                            function CustomConversionGoalErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomConversionGoalErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomConversionGoalErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum} CustomConversionGoalErrorEnum instance
                             */
                            CustomConversionGoalErrorEnum.create = function create(properties) {
                                return new CustomConversionGoalErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoalErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomConversionGoalErrorEnum} m CustomConversionGoalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoalErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoalErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomConversionGoalErrorEnum} message CustomConversionGoalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoalErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomConversionGoalErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum} CustomConversionGoalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoalErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomConversionGoalErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum} CustomConversionGoalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoalErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomConversionGoalErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomConversionGoalErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomConversionGoalErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum} CustomConversionGoalErrorEnum
                             */
                            CustomConversionGoalErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomConversionGoalErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum} m CustomConversionGoalErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomConversionGoalErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomConversionGoalErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomConversionGoalErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomConversionGoalError enum.
                             * @name google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum.CustomConversionGoalError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_CONVERSION_ACTION=2 INVALID_CONVERSION_ACTION value
                             * @property {number} CONVERSION_ACTION_NOT_ENABLED=3 CONVERSION_ACTION_NOT_ENABLED value
                             * @property {number} CANNOT_REMOVE_LINKED_CUSTOM_CONVERSION_GOAL=4 CANNOT_REMOVE_LINKED_CUSTOM_CONVERSION_GOAL value
                             * @property {number} CUSTOM_GOAL_DUPLICATE_NAME=5 CUSTOM_GOAL_DUPLICATE_NAME value
                             * @property {number} DUPLICATE_CONVERSION_ACTION_LIST=6 DUPLICATE_CONVERSION_ACTION_LIST value
                             */
                            CustomConversionGoalErrorEnum.CustomConversionGoalError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_CONVERSION_ACTION"] = 2;
                                values[valuesById[3] = "CONVERSION_ACTION_NOT_ENABLED"] = 3;
                                values[valuesById[4] = "CANNOT_REMOVE_LINKED_CUSTOM_CONVERSION_GOAL"] = 4;
                                values[valuesById[5] = "CUSTOM_GOAL_DUPLICATE_NAME"] = 5;
                                values[valuesById[6] = "DUPLICATE_CONVERSION_ACTION_LIST"] = 6;
                                return values;
                            })();
    
                            return CustomConversionGoalErrorEnum;
                        })();
    
                        errors.CustomInterestErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomInterestErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomInterestErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomInterestErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomInterestErrorEnum.
                             * @implements ICustomInterestErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomInterestErrorEnum=} [p] Properties to set
                             */
                            function CustomInterestErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomInterestErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomInterestErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomInterestErrorEnum} CustomInterestErrorEnum instance
                             */
                            CustomInterestErrorEnum.create = function create(properties) {
                                return new CustomInterestErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomInterestErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomInterestErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomInterestErrorEnum} m CustomInterestErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomInterestErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomInterestErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomInterestErrorEnum} message CustomInterestErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomInterestErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomInterestErrorEnum} CustomInterestErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomInterestErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomInterestErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomInterestErrorEnum} CustomInterestErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomInterestErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomInterestErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomInterestErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomInterestErrorEnum} CustomInterestErrorEnum
                             */
                            CustomInterestErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomInterestErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomInterestErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomInterestErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomInterestErrorEnum} m CustomInterestErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomInterestErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomInterestErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomInterestErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomInterestErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomInterestError enum.
                             * @name google.ads.googleads.v11.errors.CustomInterestErrorEnum.CustomInterestError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NAME_ALREADY_USED=2 NAME_ALREADY_USED value
                             * @property {number} CUSTOM_INTEREST_MEMBER_ID_AND_TYPE_PARAMETER_NOT_PRESENT_IN_REMOVE=3 CUSTOM_INTEREST_MEMBER_ID_AND_TYPE_PARAMETER_NOT_PRESENT_IN_REMOVE value
                             * @property {number} TYPE_AND_PARAMETER_NOT_FOUND=4 TYPE_AND_PARAMETER_NOT_FOUND value
                             * @property {number} TYPE_AND_PARAMETER_ALREADY_EXISTED=5 TYPE_AND_PARAMETER_ALREADY_EXISTED value
                             * @property {number} INVALID_CUSTOM_INTEREST_MEMBER_TYPE=6 INVALID_CUSTOM_INTEREST_MEMBER_TYPE value
                             * @property {number} CANNOT_REMOVE_WHILE_IN_USE=7 CANNOT_REMOVE_WHILE_IN_USE value
                             * @property {number} CANNOT_CHANGE_TYPE=8 CANNOT_CHANGE_TYPE value
                             */
                            CustomInterestErrorEnum.CustomInterestError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NAME_ALREADY_USED"] = 2;
                                values[valuesById[3] = "CUSTOM_INTEREST_MEMBER_ID_AND_TYPE_PARAMETER_NOT_PRESENT_IN_REMOVE"] = 3;
                                values[valuesById[4] = "TYPE_AND_PARAMETER_NOT_FOUND"] = 4;
                                values[valuesById[5] = "TYPE_AND_PARAMETER_ALREADY_EXISTED"] = 5;
                                values[valuesById[6] = "INVALID_CUSTOM_INTEREST_MEMBER_TYPE"] = 6;
                                values[valuesById[7] = "CANNOT_REMOVE_WHILE_IN_USE"] = 7;
                                values[valuesById[8] = "CANNOT_CHANGE_TYPE"] = 8;
                                return values;
                            })();
    
                            return CustomInterestErrorEnum;
                        })();
    
                        errors.CustomerClientLinkErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomerClientLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomerClientLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomerClientLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomerClientLinkErrorEnum.
                             * @implements ICustomerClientLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomerClientLinkErrorEnum=} [p] Properties to set
                             */
                            function CustomerClientLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerClientLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerClientLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum} CustomerClientLinkErrorEnum instance
                             */
                            CustomerClientLinkErrorEnum.create = function create(properties) {
                                return new CustomerClientLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerClientLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerClientLinkErrorEnum} m CustomerClientLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClientLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerClientLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerClientLinkErrorEnum} message CustomerClientLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClientLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerClientLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum} CustomerClientLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClientLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerClientLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum} CustomerClientLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClientLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerClientLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerClientLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerClientLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum} CustomerClientLinkErrorEnum
                             */
                            CustomerClientLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerClientLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum} m CustomerClientLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerClientLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerClientLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerClientLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerClientLinkError enum.
                             * @name google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum.CustomerClientLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CLIENT_ALREADY_INVITED_BY_THIS_MANAGER=2 CLIENT_ALREADY_INVITED_BY_THIS_MANAGER value
                             * @property {number} CLIENT_ALREADY_MANAGED_IN_HIERARCHY=3 CLIENT_ALREADY_MANAGED_IN_HIERARCHY value
                             * @property {number} CYCLIC_LINK_NOT_ALLOWED=4 CYCLIC_LINK_NOT_ALLOWED value
                             * @property {number} CUSTOMER_HAS_TOO_MANY_ACCOUNTS=5 CUSTOMER_HAS_TOO_MANY_ACCOUNTS value
                             * @property {number} CLIENT_HAS_TOO_MANY_INVITATIONS=6 CLIENT_HAS_TOO_MANY_INVITATIONS value
                             * @property {number} CANNOT_HIDE_OR_UNHIDE_MANAGER_ACCOUNTS=7 CANNOT_HIDE_OR_UNHIDE_MANAGER_ACCOUNTS value
                             * @property {number} CUSTOMER_HAS_TOO_MANY_ACCOUNTS_AT_MANAGER=8 CUSTOMER_HAS_TOO_MANY_ACCOUNTS_AT_MANAGER value
                             * @property {number} CLIENT_HAS_TOO_MANY_MANAGERS=9 CLIENT_HAS_TOO_MANY_MANAGERS value
                             */
                            CustomerClientLinkErrorEnum.CustomerClientLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CLIENT_ALREADY_INVITED_BY_THIS_MANAGER"] = 2;
                                values[valuesById[3] = "CLIENT_ALREADY_MANAGED_IN_HIERARCHY"] = 3;
                                values[valuesById[4] = "CYCLIC_LINK_NOT_ALLOWED"] = 4;
                                values[valuesById[5] = "CUSTOMER_HAS_TOO_MANY_ACCOUNTS"] = 5;
                                values[valuesById[6] = "CLIENT_HAS_TOO_MANY_INVITATIONS"] = 6;
                                values[valuesById[7] = "CANNOT_HIDE_OR_UNHIDE_MANAGER_ACCOUNTS"] = 7;
                                values[valuesById[8] = "CUSTOMER_HAS_TOO_MANY_ACCOUNTS_AT_MANAGER"] = 8;
                                values[valuesById[9] = "CLIENT_HAS_TOO_MANY_MANAGERS"] = 9;
                                return values;
                            })();
    
                            return CustomerClientLinkErrorEnum;
                        })();
    
                        errors.CustomerCustomizerErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomerCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomerCustomizerErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomerCustomizerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomerCustomizerErrorEnum.
                             * @implements ICustomerCustomizerErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomerCustomizerErrorEnum=} [p] Properties to set
                             */
                            function CustomerCustomizerErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerCustomizerErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerCustomizerErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum} CustomerCustomizerErrorEnum instance
                             */
                            CustomerCustomizerErrorEnum.create = function create(properties) {
                                return new CustomerCustomizerErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerCustomizerErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerCustomizerErrorEnum} m CustomerCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerCustomizerErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerCustomizerErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerCustomizerErrorEnum} message CustomerCustomizerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerCustomizerErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerCustomizerErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum} CustomerCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerCustomizerErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerCustomizerErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum} CustomerCustomizerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerCustomizerErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerCustomizerErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerCustomizerErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerCustomizerErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum} CustomerCustomizerErrorEnum
                             */
                            CustomerCustomizerErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerCustomizerErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum} m CustomerCustomizerErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerCustomizerErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerCustomizerErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerCustomizerErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerCustomizerError enum.
                             * @name google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum.CustomerCustomizerError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             */
                            CustomerCustomizerErrorEnum.CustomerCustomizerError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                return values;
                            })();
    
                            return CustomerCustomizerErrorEnum;
                        })();
    
                        errors.CustomerErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomerErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomerErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomerErrorEnum.
                             * @implements ICustomerErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomerErrorEnum=} [p] Properties to set
                             */
                            function CustomerErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomerErrorEnum} CustomerErrorEnum instance
                             */
                            CustomerErrorEnum.create = function create(properties) {
                                return new CustomerErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerErrorEnum} m CustomerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerErrorEnum} message CustomerErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomerErrorEnum} CustomerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomerErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomerErrorEnum} CustomerErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomerErrorEnum} CustomerErrorEnum
                             */
                            CustomerErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomerErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomerErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomerErrorEnum} m CustomerErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomerErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerError enum.
                             * @name google.ads.googleads.v11.errors.CustomerErrorEnum.CustomerError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} STATUS_CHANGE_DISALLOWED=2 STATUS_CHANGE_DISALLOWED value
                             * @property {number} ACCOUNT_NOT_SET_UP=3 ACCOUNT_NOT_SET_UP value
                             */
                            CustomerErrorEnum.CustomerError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "STATUS_CHANGE_DISALLOWED"] = 2;
                                values[valuesById[3] = "ACCOUNT_NOT_SET_UP"] = 3;
                                return values;
                            })();
    
                            return CustomerErrorEnum;
                        })();
    
                        errors.CustomerFeedErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomerFeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomerFeedErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomerFeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomerFeedErrorEnum.
                             * @implements ICustomerFeedErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomerFeedErrorEnum=} [p] Properties to set
                             */
                            function CustomerFeedErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerFeedErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerFeedErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomerFeedErrorEnum} CustomerFeedErrorEnum instance
                             */
                            CustomerFeedErrorEnum.create = function create(properties) {
                                return new CustomerFeedErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerFeedErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerFeedErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerFeedErrorEnum} m CustomerFeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerFeedErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerFeedErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerFeedErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerFeedErrorEnum} message CustomerFeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerFeedErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerFeedErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomerFeedErrorEnum} CustomerFeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerFeedErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomerFeedErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerFeedErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomerFeedErrorEnum} CustomerFeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerFeedErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerFeedErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerFeedErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerFeedErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomerFeedErrorEnum} CustomerFeedErrorEnum
                             */
                            CustomerFeedErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomerFeedErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomerFeedErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerFeedErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomerFeedErrorEnum} m CustomerFeedErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerFeedErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerFeedErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomerFeedErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerFeedErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerFeedError enum.
                             * @name google.ads.googleads.v11.errors.CustomerFeedErrorEnum.CustomerFeedError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE=2 FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE value
                             * @property {number} CANNOT_CREATE_FOR_REMOVED_FEED=3 CANNOT_CREATE_FOR_REMOVED_FEED value
                             * @property {number} CANNOT_CREATE_ALREADY_EXISTING_CUSTOMER_FEED=4 CANNOT_CREATE_ALREADY_EXISTING_CUSTOMER_FEED value
                             * @property {number} CANNOT_MODIFY_REMOVED_CUSTOMER_FEED=5 CANNOT_MODIFY_REMOVED_CUSTOMER_FEED value
                             * @property {number} INVALID_PLACEHOLDER_TYPE=6 INVALID_PLACEHOLDER_TYPE value
                             * @property {number} MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE=7 MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE value
                             * @property {number} PLACEHOLDER_TYPE_NOT_ALLOWED_ON_CUSTOMER_FEED=8 PLACEHOLDER_TYPE_NOT_ALLOWED_ON_CUSTOMER_FEED value
                             */
                            CustomerFeedErrorEnum.CustomerFeedError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE"] = 2;
                                values[valuesById[3] = "CANNOT_CREATE_FOR_REMOVED_FEED"] = 3;
                                values[valuesById[4] = "CANNOT_CREATE_ALREADY_EXISTING_CUSTOMER_FEED"] = 4;
                                values[valuesById[5] = "CANNOT_MODIFY_REMOVED_CUSTOMER_FEED"] = 5;
                                values[valuesById[6] = "INVALID_PLACEHOLDER_TYPE"] = 6;
                                values[valuesById[7] = "MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE"] = 7;
                                values[valuesById[8] = "PLACEHOLDER_TYPE_NOT_ALLOWED_ON_CUSTOMER_FEED"] = 8;
                                return values;
                            })();
    
                            return CustomerFeedErrorEnum;
                        })();
    
                        errors.CustomerManagerLinkErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomerManagerLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomerManagerLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomerManagerLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomerManagerLinkErrorEnum.
                             * @implements ICustomerManagerLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomerManagerLinkErrorEnum=} [p] Properties to set
                             */
                            function CustomerManagerLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerManagerLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerManagerLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum} CustomerManagerLinkErrorEnum instance
                             */
                            CustomerManagerLinkErrorEnum.create = function create(properties) {
                                return new CustomerManagerLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerManagerLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerManagerLinkErrorEnum} m CustomerManagerLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerManagerLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerManagerLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerManagerLinkErrorEnum} message CustomerManagerLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerManagerLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerManagerLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum} CustomerManagerLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerManagerLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerManagerLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum} CustomerManagerLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerManagerLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerManagerLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerManagerLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerManagerLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum} CustomerManagerLinkErrorEnum
                             */
                            CustomerManagerLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerManagerLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum} m CustomerManagerLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerManagerLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerManagerLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerManagerLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerManagerLinkError enum.
                             * @name google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum.CustomerManagerLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NO_PENDING_INVITE=2 NO_PENDING_INVITE value
                             * @property {number} SAME_CLIENT_MORE_THAN_ONCE_PER_CALL=3 SAME_CLIENT_MORE_THAN_ONCE_PER_CALL value
                             * @property {number} MANAGER_HAS_MAX_NUMBER_OF_LINKED_ACCOUNTS=4 MANAGER_HAS_MAX_NUMBER_OF_LINKED_ACCOUNTS value
                             * @property {number} CANNOT_UNLINK_ACCOUNT_WITHOUT_ACTIVE_USER=5 CANNOT_UNLINK_ACCOUNT_WITHOUT_ACTIVE_USER value
                             * @property {number} CANNOT_REMOVE_LAST_CLIENT_ACCOUNT_OWNER=6 CANNOT_REMOVE_LAST_CLIENT_ACCOUNT_OWNER value
                             * @property {number} CANNOT_CHANGE_ROLE_BY_NON_ACCOUNT_OWNER=7 CANNOT_CHANGE_ROLE_BY_NON_ACCOUNT_OWNER value
                             * @property {number} CANNOT_CHANGE_ROLE_FOR_NON_ACTIVE_LINK_ACCOUNT=8 CANNOT_CHANGE_ROLE_FOR_NON_ACTIVE_LINK_ACCOUNT value
                             * @property {number} DUPLICATE_CHILD_FOUND=9 DUPLICATE_CHILD_FOUND value
                             * @property {number} TEST_ACCOUNT_LINKS_TOO_MANY_CHILD_ACCOUNTS=10 TEST_ACCOUNT_LINKS_TOO_MANY_CHILD_ACCOUNTS value
                             */
                            CustomerManagerLinkErrorEnum.CustomerManagerLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NO_PENDING_INVITE"] = 2;
                                values[valuesById[3] = "SAME_CLIENT_MORE_THAN_ONCE_PER_CALL"] = 3;
                                values[valuesById[4] = "MANAGER_HAS_MAX_NUMBER_OF_LINKED_ACCOUNTS"] = 4;
                                values[valuesById[5] = "CANNOT_UNLINK_ACCOUNT_WITHOUT_ACTIVE_USER"] = 5;
                                values[valuesById[6] = "CANNOT_REMOVE_LAST_CLIENT_ACCOUNT_OWNER"] = 6;
                                values[valuesById[7] = "CANNOT_CHANGE_ROLE_BY_NON_ACCOUNT_OWNER"] = 7;
                                values[valuesById[8] = "CANNOT_CHANGE_ROLE_FOR_NON_ACTIVE_LINK_ACCOUNT"] = 8;
                                values[valuesById[9] = "DUPLICATE_CHILD_FOUND"] = 9;
                                values[valuesById[10] = "TEST_ACCOUNT_LINKS_TOO_MANY_CHILD_ACCOUNTS"] = 10;
                                return values;
                            })();
    
                            return CustomerManagerLinkErrorEnum;
                        })();
    
                        errors.CustomerUserAccessErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomerUserAccessErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomerUserAccessErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomerUserAccessErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomerUserAccessErrorEnum.
                             * @implements ICustomerUserAccessErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomerUserAccessErrorEnum=} [p] Properties to set
                             */
                            function CustomerUserAccessErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomerUserAccessErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerUserAccessErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum} CustomerUserAccessErrorEnum instance
                             */
                            CustomerUserAccessErrorEnum.create = function create(properties) {
                                return new CustomerUserAccessErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerUserAccessErrorEnum} m CustomerUserAccessErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomerUserAccessErrorEnum} message CustomerUserAccessErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerUserAccessErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum} CustomerUserAccessErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerUserAccessErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum} CustomerUserAccessErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerUserAccessErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerUserAccessErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomerUserAccessErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum} CustomerUserAccessErrorEnum
                             */
                            CustomerUserAccessErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomerUserAccessErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum} m CustomerUserAccessErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerUserAccessErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomerUserAccessErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerUserAccessErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomerUserAccessError enum.
                             * @name google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum.CustomerUserAccessError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_USER_ID=2 INVALID_USER_ID value
                             * @property {number} REMOVAL_DISALLOWED=3 REMOVAL_DISALLOWED value
                             * @property {number} DISALLOWED_ACCESS_ROLE=4 DISALLOWED_ACCESS_ROLE value
                             * @property {number} LAST_ADMIN_USER_OF_SERVING_CUSTOMER=5 LAST_ADMIN_USER_OF_SERVING_CUSTOMER value
                             * @property {number} LAST_ADMIN_USER_OF_MANAGER=6 LAST_ADMIN_USER_OF_MANAGER value
                             */
                            CustomerUserAccessErrorEnum.CustomerUserAccessError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_USER_ID"] = 2;
                                values[valuesById[3] = "REMOVAL_DISALLOWED"] = 3;
                                values[valuesById[4] = "DISALLOWED_ACCESS_ROLE"] = 4;
                                values[valuesById[5] = "LAST_ADMIN_USER_OF_SERVING_CUSTOMER"] = 5;
                                values[valuesById[6] = "LAST_ADMIN_USER_OF_MANAGER"] = 6;
                                return values;
                            })();
    
                            return CustomerUserAccessErrorEnum;
                        })();
    
                        errors.CustomizerAttributeErrorEnum = (function() {
    
                            /**
                             * Properties of a CustomizerAttributeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ICustomizerAttributeErrorEnum
                             */
    
                            /**
                             * Constructs a new CustomizerAttributeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a CustomizerAttributeErrorEnum.
                             * @implements ICustomizerAttributeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ICustomizerAttributeErrorEnum=} [p] Properties to set
                             */
                            function CustomizerAttributeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new CustomizerAttributeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomizerAttributeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum} CustomizerAttributeErrorEnum instance
                             */
                            CustomizerAttributeErrorEnum.create = function create(properties) {
                                return new CustomizerAttributeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomizerAttributeErrorEnum} m CustomizerAttributeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ICustomizerAttributeErrorEnum} message CustomizerAttributeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomizerAttributeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum} CustomizerAttributeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomizerAttributeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum} CustomizerAttributeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomizerAttributeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomizerAttributeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a CustomizerAttributeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum} CustomizerAttributeErrorEnum
                             */
                            CustomizerAttributeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a CustomizerAttributeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum} m CustomizerAttributeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomizerAttributeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this CustomizerAttributeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomizerAttributeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * CustomizerAttributeError enum.
                             * @name google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum.CustomizerAttributeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_CUSTOMIZER_ATTRIBUTE_NAME=2 DUPLICATE_CUSTOMIZER_ATTRIBUTE_NAME value
                             */
                            CustomizerAttributeErrorEnum.CustomizerAttributeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_CUSTOMIZER_ATTRIBUTE_NAME"] = 2;
                                return values;
                            })();
    
                            return CustomizerAttributeErrorEnum;
                        })();
    
                        errors.DatabaseErrorEnum = (function() {
    
                            /**
                             * Properties of a DatabaseErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IDatabaseErrorEnum
                             */
    
                            /**
                             * Constructs a new DatabaseErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a DatabaseErrorEnum.
                             * @implements IDatabaseErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IDatabaseErrorEnum=} [p] Properties to set
                             */
                            function DatabaseErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DatabaseErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDatabaseErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.DatabaseErrorEnum} DatabaseErrorEnum instance
                             */
                            DatabaseErrorEnum.create = function create(properties) {
                                return new DatabaseErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DatabaseErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.DatabaseErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDatabaseErrorEnum} m DatabaseErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DatabaseErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DatabaseErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.DatabaseErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDatabaseErrorEnum} message DatabaseErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DatabaseErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DatabaseErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.DatabaseErrorEnum} DatabaseErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DatabaseErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.DatabaseErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DatabaseErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.DatabaseErrorEnum} DatabaseErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DatabaseErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DatabaseErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DatabaseErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DatabaseErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.DatabaseErrorEnum} DatabaseErrorEnum
                             */
                            DatabaseErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.DatabaseErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.DatabaseErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a DatabaseErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.DatabaseErrorEnum} m DatabaseErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DatabaseErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DatabaseErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.DatabaseErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DatabaseErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DatabaseError enum.
                             * @name google.ads.googleads.v11.errors.DatabaseErrorEnum.DatabaseError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CONCURRENT_MODIFICATION=2 CONCURRENT_MODIFICATION value
                             * @property {number} DATA_CONSTRAINT_VIOLATION=3 DATA_CONSTRAINT_VIOLATION value
                             * @property {number} REQUEST_TOO_LARGE=4 REQUEST_TOO_LARGE value
                             */
                            DatabaseErrorEnum.DatabaseError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CONCURRENT_MODIFICATION"] = 2;
                                values[valuesById[3] = "DATA_CONSTRAINT_VIOLATION"] = 3;
                                values[valuesById[4] = "REQUEST_TOO_LARGE"] = 4;
                                return values;
                            })();
    
                            return DatabaseErrorEnum;
                        })();
    
                        errors.DateErrorEnum = (function() {
    
                            /**
                             * Properties of a DateErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IDateErrorEnum
                             */
    
                            /**
                             * Constructs a new DateErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a DateErrorEnum.
                             * @implements IDateErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IDateErrorEnum=} [p] Properties to set
                             */
                            function DateErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DateErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDateErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.DateErrorEnum} DateErrorEnum instance
                             */
                            DateErrorEnum.create = function create(properties) {
                                return new DateErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DateErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.DateErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDateErrorEnum} m DateErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DateErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DateErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.DateErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDateErrorEnum} message DateErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DateErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DateErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.DateErrorEnum} DateErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DateErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.DateErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DateErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.DateErrorEnum} DateErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DateErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DateErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DateErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DateErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.DateErrorEnum} DateErrorEnum
                             */
                            DateErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.DateErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.DateErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a DateErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.DateErrorEnum} m DateErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DateErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DateErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.DateErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DateErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DateError enum.
                             * @name google.ads.googleads.v11.errors.DateErrorEnum.DateError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_FIELD_VALUES_IN_DATE=2 INVALID_FIELD_VALUES_IN_DATE value
                             * @property {number} INVALID_FIELD_VALUES_IN_DATE_TIME=3 INVALID_FIELD_VALUES_IN_DATE_TIME value
                             * @property {number} INVALID_STRING_DATE=4 INVALID_STRING_DATE value
                             * @property {number} INVALID_STRING_DATE_TIME_MICROS=6 INVALID_STRING_DATE_TIME_MICROS value
                             * @property {number} INVALID_STRING_DATE_TIME_SECONDS=11 INVALID_STRING_DATE_TIME_SECONDS value
                             * @property {number} INVALID_STRING_DATE_TIME_SECONDS_WITH_OFFSET=12 INVALID_STRING_DATE_TIME_SECONDS_WITH_OFFSET value
                             * @property {number} EARLIER_THAN_MINIMUM_DATE=7 EARLIER_THAN_MINIMUM_DATE value
                             * @property {number} LATER_THAN_MAXIMUM_DATE=8 LATER_THAN_MAXIMUM_DATE value
                             * @property {number} DATE_RANGE_MINIMUM_DATE_LATER_THAN_MAXIMUM_DATE=9 DATE_RANGE_MINIMUM_DATE_LATER_THAN_MAXIMUM_DATE value
                             * @property {number} DATE_RANGE_MINIMUM_AND_MAXIMUM_DATES_BOTH_NULL=10 DATE_RANGE_MINIMUM_AND_MAXIMUM_DATES_BOTH_NULL value
                             */
                            DateErrorEnum.DateError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_FIELD_VALUES_IN_DATE"] = 2;
                                values[valuesById[3] = "INVALID_FIELD_VALUES_IN_DATE_TIME"] = 3;
                                values[valuesById[4] = "INVALID_STRING_DATE"] = 4;
                                values[valuesById[6] = "INVALID_STRING_DATE_TIME_MICROS"] = 6;
                                values[valuesById[11] = "INVALID_STRING_DATE_TIME_SECONDS"] = 11;
                                values[valuesById[12] = "INVALID_STRING_DATE_TIME_SECONDS_WITH_OFFSET"] = 12;
                                values[valuesById[7] = "EARLIER_THAN_MINIMUM_DATE"] = 7;
                                values[valuesById[8] = "LATER_THAN_MAXIMUM_DATE"] = 8;
                                values[valuesById[9] = "DATE_RANGE_MINIMUM_DATE_LATER_THAN_MAXIMUM_DATE"] = 9;
                                values[valuesById[10] = "DATE_RANGE_MINIMUM_AND_MAXIMUM_DATES_BOTH_NULL"] = 10;
                                return values;
                            })();
    
                            return DateErrorEnum;
                        })();
    
                        errors.DateRangeErrorEnum = (function() {
    
                            /**
                             * Properties of a DateRangeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IDateRangeErrorEnum
                             */
    
                            /**
                             * Constructs a new DateRangeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a DateRangeErrorEnum.
                             * @implements IDateRangeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IDateRangeErrorEnum=} [p] Properties to set
                             */
                            function DateRangeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DateRangeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDateRangeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.DateRangeErrorEnum} DateRangeErrorEnum instance
                             */
                            DateRangeErrorEnum.create = function create(properties) {
                                return new DateRangeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DateRangeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.DateRangeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDateRangeErrorEnum} m DateRangeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DateRangeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DateRangeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.DateRangeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDateRangeErrorEnum} message DateRangeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DateRangeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DateRangeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.DateRangeErrorEnum} DateRangeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DateRangeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.DateRangeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DateRangeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.DateRangeErrorEnum} DateRangeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DateRangeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DateRangeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DateRangeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DateRangeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.DateRangeErrorEnum} DateRangeErrorEnum
                             */
                            DateRangeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.DateRangeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.DateRangeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a DateRangeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.DateRangeErrorEnum} m DateRangeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DateRangeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DateRangeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.DateRangeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DateRangeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DateRangeError enum.
                             * @name google.ads.googleads.v11.errors.DateRangeErrorEnum.DateRangeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_DATE=2 INVALID_DATE value
                             * @property {number} START_DATE_AFTER_END_DATE=3 START_DATE_AFTER_END_DATE value
                             * @property {number} CANNOT_SET_DATE_TO_PAST=4 CANNOT_SET_DATE_TO_PAST value
                             * @property {number} AFTER_MAXIMUM_ALLOWABLE_DATE=5 AFTER_MAXIMUM_ALLOWABLE_DATE value
                             * @property {number} CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED=6 CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED value
                             */
                            DateRangeErrorEnum.DateRangeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_DATE"] = 2;
                                values[valuesById[3] = "START_DATE_AFTER_END_DATE"] = 3;
                                values[valuesById[4] = "CANNOT_SET_DATE_TO_PAST"] = 4;
                                values[valuesById[5] = "AFTER_MAXIMUM_ALLOWABLE_DATE"] = 5;
                                values[valuesById[6] = "CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED"] = 6;
                                return values;
                            })();
    
                            return DateRangeErrorEnum;
                        })();
    
                        errors.DistinctErrorEnum = (function() {
    
                            /**
                             * Properties of a DistinctErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IDistinctErrorEnum
                             */
    
                            /**
                             * Constructs a new DistinctErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a DistinctErrorEnum.
                             * @implements IDistinctErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IDistinctErrorEnum=} [p] Properties to set
                             */
                            function DistinctErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new DistinctErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDistinctErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.DistinctErrorEnum} DistinctErrorEnum instance
                             */
                            DistinctErrorEnum.create = function create(properties) {
                                return new DistinctErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified DistinctErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.DistinctErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDistinctErrorEnum} m DistinctErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DistinctErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DistinctErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.DistinctErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IDistinctErrorEnum} message DistinctErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DistinctErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DistinctErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.DistinctErrorEnum} DistinctErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DistinctErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.DistinctErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DistinctErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.DistinctErrorEnum} DistinctErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DistinctErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DistinctErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DistinctErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a DistinctErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.DistinctErrorEnum} DistinctErrorEnum
                             */
                            DistinctErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.DistinctErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.DistinctErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a DistinctErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.DistinctErrorEnum} m DistinctErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DistinctErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this DistinctErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.DistinctErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DistinctErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * DistinctError enum.
                             * @name google.ads.googleads.v11.errors.DistinctErrorEnum.DistinctError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} DUPLICATE_ELEMENT=2 DUPLICATE_ELEMENT value
                             * @property {number} DUPLICATE_TYPE=3 DUPLICATE_TYPE value
                             */
                            DistinctErrorEnum.DistinctError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "DUPLICATE_ELEMENT"] = 2;
                                values[valuesById[3] = "DUPLICATE_TYPE"] = 3;
                                return values;
                            })();
    
                            return DistinctErrorEnum;
                        })();
    
                        errors.EnumErrorEnum = (function() {
    
                            /**
                             * Properties of an EnumErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IEnumErrorEnum
                             */
    
                            /**
                             * Constructs a new EnumErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an EnumErrorEnum.
                             * @implements IEnumErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IEnumErrorEnum=} [p] Properties to set
                             */
                            function EnumErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new EnumErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IEnumErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.EnumErrorEnum} EnumErrorEnum instance
                             */
                            EnumErrorEnum.create = function create(properties) {
                                return new EnumErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified EnumErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.EnumErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IEnumErrorEnum} m EnumErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EnumErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified EnumErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.EnumErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IEnumErrorEnum} message EnumErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EnumErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EnumErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.EnumErrorEnum} EnumErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EnumErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.EnumErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an EnumErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.EnumErrorEnum} EnumErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EnumErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EnumErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EnumErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an EnumErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.EnumErrorEnum} EnumErrorEnum
                             */
                            EnumErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.EnumErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.EnumErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an EnumErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.EnumErrorEnum} m EnumErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EnumErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this EnumErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.EnumErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EnumErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * EnumError enum.
                             * @name google.ads.googleads.v11.errors.EnumErrorEnum.EnumError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ENUM_VALUE_NOT_PERMITTED=3 ENUM_VALUE_NOT_PERMITTED value
                             */
                            EnumErrorEnum.EnumError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "ENUM_VALUE_NOT_PERMITTED"] = 3;
                                return values;
                            })();
    
                            return EnumErrorEnum;
                        })();
    
                        errors.GoogleAdsFailure = (function() {
    
                            /**
                             * Properties of a GoogleAdsFailure.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IGoogleAdsFailure
                             * @property {Array.<google.ads.googleads.v11.errors.IGoogleAdsError>|null} [errors] GoogleAdsFailure errors
                             * @property {string|null} [request_id] GoogleAdsFailure request_id
                             */
    
                            /**
                             * Constructs a new GoogleAdsFailure.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a GoogleAdsFailure.
                             * @implements IGoogleAdsFailure
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsFailure=} [p] Properties to set
                             */
                            function GoogleAdsFailure(p) {
                                this.errors = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GoogleAdsFailure errors.
                             * @member {Array.<google.ads.googleads.v11.errors.IGoogleAdsError>} errors
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @instance
                             */
                            GoogleAdsFailure.prototype.errors = $util.emptyArray;
    
                            /**
                             * GoogleAdsFailure request_id.
                             * @member {string} request_id
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @instance
                             */
                            GoogleAdsFailure.prototype.request_id = "";
    
                            /**
                             * Creates a new GoogleAdsFailure instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsFailure=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsFailure} GoogleAdsFailure instance
                             */
                            GoogleAdsFailure.create = function create(properties) {
                                return new GoogleAdsFailure(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleAdsFailure message. Does not implicitly {@link google.ads.googleads.v11.errors.GoogleAdsFailure.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsFailure} m GoogleAdsFailure message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsFailure.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.errors != null && m.errors.length) {
                                    for (var i = 0; i < m.errors.length; ++i)
                                        $root.google.ads.googleads.v11.errors.GoogleAdsError.encode(m.errors[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.request_id != null && Object.hasOwnProperty.call(m, "request_id"))
                                    w.uint32(18).string(m.request_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleAdsFailure message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.GoogleAdsFailure.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsFailure} message GoogleAdsFailure message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsFailure.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleAdsFailure message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsFailure} GoogleAdsFailure
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsFailure.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.GoogleAdsFailure();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.errors && m.errors.length))
                                            m.errors = [];
                                        m.errors.push($root.google.ads.googleads.v11.errors.GoogleAdsError.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.request_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleAdsFailure message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsFailure} GoogleAdsFailure
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsFailure.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleAdsFailure message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleAdsFailure.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.errors != null && m.hasOwnProperty("errors")) {
                                    if (!Array.isArray(m.errors))
                                        return "errors: array expected";
                                    for (var i = 0; i < m.errors.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.errors.GoogleAdsError.verify(m.errors[i]);
                                            if (e)
                                                return "errors." + e;
                                        }
                                    }
                                }
                                if (m.request_id != null && m.hasOwnProperty("request_id")) {
                                    if (!$util.isString(m.request_id))
                                        return "request_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GoogleAdsFailure message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsFailure} GoogleAdsFailure
                             */
                            GoogleAdsFailure.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.GoogleAdsFailure)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.GoogleAdsFailure();
                                if (d.errors) {
                                    if (!Array.isArray(d.errors))
                                        throw TypeError(".google.ads.googleads.v11.errors.GoogleAdsFailure.errors: array expected");
                                    m.errors = [];
                                    for (var i = 0; i < d.errors.length; ++i) {
                                        if (typeof d.errors[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.errors.GoogleAdsFailure.errors: object expected");
                                        m.errors[i] = $root.google.ads.googleads.v11.errors.GoogleAdsError.fromObject(d.errors[i]);
                                    }
                                }
                                if (d.request_id != null) {
                                    m.request_id = String(d.request_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GoogleAdsFailure message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @static
                             * @param {google.ads.googleads.v11.errors.GoogleAdsFailure} m GoogleAdsFailure
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleAdsFailure.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.errors = [];
                                }
                                if (o.defaults) {
                                    d.request_id = "";
                                }
                                if (m.errors && m.errors.length) {
                                    d.errors = [];
                                    for (var j = 0; j < m.errors.length; ++j) {
                                        d.errors[j] = $root.google.ads.googleads.v11.errors.GoogleAdsError.toObject(m.errors[j], o);
                                    }
                                }
                                if (m.request_id != null && m.hasOwnProperty("request_id")) {
                                    d.request_id = m.request_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GoogleAdsFailure to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsFailure
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleAdsFailure.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GoogleAdsFailure;
                        })();
    
                        errors.GoogleAdsError = (function() {
    
                            /**
                             * Properties of a GoogleAdsError.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IGoogleAdsError
                             * @property {google.ads.googleads.v11.errors.IErrorCode|null} [error_code] GoogleAdsError error_code
                             * @property {string|null} [message] GoogleAdsError message
                             * @property {google.ads.googleads.v11.common.IValue|null} [trigger] GoogleAdsError trigger
                             * @property {google.ads.googleads.v11.errors.IErrorLocation|null} [location] GoogleAdsError location
                             * @property {google.ads.googleads.v11.errors.IErrorDetails|null} [details] GoogleAdsError details
                             */
    
                            /**
                             * Constructs a new GoogleAdsError.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a GoogleAdsError.
                             * @implements IGoogleAdsError
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsError=} [p] Properties to set
                             */
                            function GoogleAdsError(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GoogleAdsError error_code.
                             * @member {google.ads.googleads.v11.errors.IErrorCode|null|undefined} error_code
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @instance
                             */
                            GoogleAdsError.prototype.error_code = null;
    
                            /**
                             * GoogleAdsError message.
                             * @member {string} message
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @instance
                             */
                            GoogleAdsError.prototype.message = "";
    
                            /**
                             * GoogleAdsError trigger.
                             * @member {google.ads.googleads.v11.common.IValue|null|undefined} trigger
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @instance
                             */
                            GoogleAdsError.prototype.trigger = null;
    
                            /**
                             * GoogleAdsError location.
                             * @member {google.ads.googleads.v11.errors.IErrorLocation|null|undefined} location
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @instance
                             */
                            GoogleAdsError.prototype.location = null;
    
                            /**
                             * GoogleAdsError details.
                             * @member {google.ads.googleads.v11.errors.IErrorDetails|null|undefined} details
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @instance
                             */
                            GoogleAdsError.prototype.details = null;
    
                            /**
                             * Creates a new GoogleAdsError instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsError=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsError} GoogleAdsError instance
                             */
                            GoogleAdsError.create = function create(properties) {
                                return new GoogleAdsError(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleAdsError message. Does not implicitly {@link google.ads.googleads.v11.errors.GoogleAdsError.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsError} m GoogleAdsError message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsError.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.error_code != null && Object.hasOwnProperty.call(m, "error_code"))
                                    $root.google.ads.googleads.v11.errors.ErrorCode.encode(m.error_code, w.uint32(10).fork()).ldelim();
                                if (m.message != null && Object.hasOwnProperty.call(m, "message"))
                                    w.uint32(18).string(m.message);
                                if (m.trigger != null && Object.hasOwnProperty.call(m, "trigger"))
                                    $root.google.ads.googleads.v11.common.Value.encode(m.trigger, w.uint32(26).fork()).ldelim();
                                if (m.location != null && Object.hasOwnProperty.call(m, "location"))
                                    $root.google.ads.googleads.v11.errors.ErrorLocation.encode(m.location, w.uint32(34).fork()).ldelim();
                                if (m.details != null && Object.hasOwnProperty.call(m, "details"))
                                    $root.google.ads.googleads.v11.errors.ErrorDetails.encode(m.details, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleAdsError message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.GoogleAdsError.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGoogleAdsError} message GoogleAdsError message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsError.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleAdsError message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsError} GoogleAdsError
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsError.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.GoogleAdsError();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.error_code = $root.google.ads.googleads.v11.errors.ErrorCode.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.message = r.string();
                                        break;
                                    case 3:
                                        m.trigger = $root.google.ads.googleads.v11.common.Value.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.location = $root.google.ads.googleads.v11.errors.ErrorLocation.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.details = $root.google.ads.googleads.v11.errors.ErrorDetails.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleAdsError message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsError} GoogleAdsError
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsError.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleAdsError message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleAdsError.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.error_code != null && m.hasOwnProperty("error_code")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.errors.ErrorCode.verify(m.error_code);
                                        if (e)
                                            return "error_code." + e;
                                    }
                                }
                                if (m.message != null && m.hasOwnProperty("message")) {
                                    if (!$util.isString(m.message))
                                        return "message: string expected";
                                }
                                if (m.trigger != null && m.hasOwnProperty("trigger")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Value.verify(m.trigger);
                                        if (e)
                                            return "trigger." + e;
                                    }
                                }
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.errors.ErrorLocation.verify(m.location);
                                        if (e)
                                            return "location." + e;
                                    }
                                }
                                if (m.details != null && m.hasOwnProperty("details")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.errors.ErrorDetails.verify(m.details);
                                        if (e)
                                            return "details." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GoogleAdsError message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.GoogleAdsError} GoogleAdsError
                             */
                            GoogleAdsError.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.GoogleAdsError)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.GoogleAdsError();
                                if (d.error_code != null) {
                                    if (typeof d.error_code !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.GoogleAdsError.error_code: object expected");
                                    m.error_code = $root.google.ads.googleads.v11.errors.ErrorCode.fromObject(d.error_code);
                                }
                                if (d.message != null) {
                                    m.message = String(d.message);
                                }
                                if (d.trigger != null) {
                                    if (typeof d.trigger !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.GoogleAdsError.trigger: object expected");
                                    m.trigger = $root.google.ads.googleads.v11.common.Value.fromObject(d.trigger);
                                }
                                if (d.location != null) {
                                    if (typeof d.location !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.GoogleAdsError.location: object expected");
                                    m.location = $root.google.ads.googleads.v11.errors.ErrorLocation.fromObject(d.location);
                                }
                                if (d.details != null) {
                                    if (typeof d.details !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.GoogleAdsError.details: object expected");
                                    m.details = $root.google.ads.googleads.v11.errors.ErrorDetails.fromObject(d.details);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GoogleAdsError message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @static
                             * @param {google.ads.googleads.v11.errors.GoogleAdsError} m GoogleAdsError
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleAdsError.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.error_code = null;
                                    d.message = "";
                                    d.trigger = null;
                                    d.location = null;
                                    d.details = null;
                                }
                                if (m.error_code != null && m.hasOwnProperty("error_code")) {
                                    d.error_code = $root.google.ads.googleads.v11.errors.ErrorCode.toObject(m.error_code, o);
                                }
                                if (m.message != null && m.hasOwnProperty("message")) {
                                    d.message = m.message;
                                }
                                if (m.trigger != null && m.hasOwnProperty("trigger")) {
                                    d.trigger = $root.google.ads.googleads.v11.common.Value.toObject(m.trigger, o);
                                }
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    d.location = $root.google.ads.googleads.v11.errors.ErrorLocation.toObject(m.location, o);
                                }
                                if (m.details != null && m.hasOwnProperty("details")) {
                                    d.details = $root.google.ads.googleads.v11.errors.ErrorDetails.toObject(m.details, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GoogleAdsError to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.GoogleAdsError
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleAdsError.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GoogleAdsError;
                        })();
    
                        errors.ErrorCode = (function() {
    
                            /**
                             * Properties of an ErrorCode.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IErrorCode
                             * @property {google.ads.googleads.v11.errors.RequestErrorEnum.RequestError|null} [request_error] ErrorCode request_error
                             * @property {google.ads.googleads.v11.errors.BiddingStrategyErrorEnum.BiddingStrategyError|null} [bidding_strategy_error] ErrorCode bidding_strategy_error
                             * @property {google.ads.googleads.v11.errors.UrlFieldErrorEnum.UrlFieldError|null} [url_field_error] ErrorCode url_field_error
                             * @property {google.ads.googleads.v11.errors.ListOperationErrorEnum.ListOperationError|null} [list_operation_error] ErrorCode list_operation_error
                             * @property {google.ads.googleads.v11.errors.QueryErrorEnum.QueryError|null} [query_error] ErrorCode query_error
                             * @property {google.ads.googleads.v11.errors.MutateErrorEnum.MutateError|null} [mutate_error] ErrorCode mutate_error
                             * @property {google.ads.googleads.v11.errors.FieldMaskErrorEnum.FieldMaskError|null} [field_mask_error] ErrorCode field_mask_error
                             * @property {google.ads.googleads.v11.errors.AuthorizationErrorEnum.AuthorizationError|null} [authorization_error] ErrorCode authorization_error
                             * @property {google.ads.googleads.v11.errors.InternalErrorEnum.InternalError|null} [internal_error] ErrorCode internal_error
                             * @property {google.ads.googleads.v11.errors.QuotaErrorEnum.QuotaError|null} [quota_error] ErrorCode quota_error
                             * @property {google.ads.googleads.v11.errors.AdErrorEnum.AdError|null} [ad_error] ErrorCode ad_error
                             * @property {google.ads.googleads.v11.errors.AdGroupErrorEnum.AdGroupError|null} [ad_group_error] ErrorCode ad_group_error
                             * @property {google.ads.googleads.v11.errors.CampaignBudgetErrorEnum.CampaignBudgetError|null} [campaign_budget_error] ErrorCode campaign_budget_error
                             * @property {google.ads.googleads.v11.errors.CampaignErrorEnum.CampaignError|null} [campaign_error] ErrorCode campaign_error
                             * @property {google.ads.googleads.v11.errors.AuthenticationErrorEnum.AuthenticationError|null} [authentication_error] ErrorCode authentication_error
                             * @property {google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum.AdGroupCriterionCustomizerError|null} [ad_group_criterion_customizer_error] ErrorCode ad_group_criterion_customizer_error
                             * @property {google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum.AdGroupCriterionError|null} [ad_group_criterion_error] ErrorCode ad_group_criterion_error
                             * @property {google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum.AdGroupCustomizerError|null} [ad_group_customizer_error] ErrorCode ad_group_customizer_error
                             * @property {google.ads.googleads.v11.errors.AdCustomizerErrorEnum.AdCustomizerError|null} [ad_customizer_error] ErrorCode ad_customizer_error
                             * @property {google.ads.googleads.v11.errors.AdGroupAdErrorEnum.AdGroupAdError|null} [ad_group_ad_error] ErrorCode ad_group_ad_error
                             * @property {google.ads.googleads.v11.errors.AdSharingErrorEnum.AdSharingError|null} [ad_sharing_error] ErrorCode ad_sharing_error
                             * @property {google.ads.googleads.v11.errors.AdxErrorEnum.AdxError|null} [adx_error] ErrorCode adx_error
                             * @property {google.ads.googleads.v11.errors.AssetErrorEnum.AssetError|null} [asset_error] ErrorCode asset_error
                             * @property {google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum.AssetGroupAssetError|null} [asset_group_asset_error] ErrorCode asset_group_asset_error
                             * @property {google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum.AssetGroupListingGroupFilterError|null} [asset_group_listing_group_filter_error] ErrorCode asset_group_listing_group_filter_error
                             * @property {google.ads.googleads.v11.errors.AssetGroupErrorEnum.AssetGroupError|null} [asset_group_error] ErrorCode asset_group_error
                             * @property {google.ads.googleads.v11.errors.AssetSetAssetErrorEnum.AssetSetAssetError|null} [asset_set_asset_error] ErrorCode asset_set_asset_error
                             * @property {google.ads.googleads.v11.errors.AssetSetLinkErrorEnum.AssetSetLinkError|null} [asset_set_link_error] ErrorCode asset_set_link_error
                             * @property {google.ads.googleads.v11.errors.AssetSetErrorEnum.AssetSetError|null} [asset_set_error] ErrorCode asset_set_error
                             * @property {google.ads.googleads.v11.errors.BiddingErrorEnum.BiddingError|null} [bidding_error] ErrorCode bidding_error
                             * @property {google.ads.googleads.v11.errors.CampaignCriterionErrorEnum.CampaignCriterionError|null} [campaign_criterion_error] ErrorCode campaign_criterion_error
                             * @property {google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum.CampaignConversionGoalError|null} [campaign_conversion_goal_error] ErrorCode campaign_conversion_goal_error
                             * @property {google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum.CampaignCustomizerError|null} [campaign_customizer_error] ErrorCode campaign_customizer_error
                             * @property {google.ads.googleads.v11.errors.CollectionSizeErrorEnum.CollectionSizeError|null} [collection_size_error] ErrorCode collection_size_error
                             * @property {google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum.ConversionGoalCampaignConfigError|null} [conversion_goal_campaign_config_error] ErrorCode conversion_goal_campaign_config_error
                             * @property {google.ads.googleads.v11.errors.CountryCodeErrorEnum.CountryCodeError|null} [country_code_error] ErrorCode country_code_error
                             * @property {google.ads.googleads.v11.errors.CriterionErrorEnum.CriterionError|null} [criterion_error] ErrorCode criterion_error
                             * @property {google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum.CustomConversionGoalError|null} [custom_conversion_goal_error] ErrorCode custom_conversion_goal_error
                             * @property {google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum.CustomerCustomizerError|null} [customer_customizer_error] ErrorCode customer_customizer_error
                             * @property {google.ads.googleads.v11.errors.CustomerErrorEnum.CustomerError|null} [customer_error] ErrorCode customer_error
                             * @property {google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum.CustomizerAttributeError|null} [customizer_attribute_error] ErrorCode customizer_attribute_error
                             * @property {google.ads.googleads.v11.errors.DateErrorEnum.DateError|null} [date_error] ErrorCode date_error
                             * @property {google.ads.googleads.v11.errors.DateRangeErrorEnum.DateRangeError|null} [date_range_error] ErrorCode date_range_error
                             * @property {google.ads.googleads.v11.errors.DistinctErrorEnum.DistinctError|null} [distinct_error] ErrorCode distinct_error
                             * @property {google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError|null} [feed_attribute_reference_error] ErrorCode feed_attribute_reference_error
                             * @property {google.ads.googleads.v11.errors.FunctionErrorEnum.FunctionError|null} [function_error] ErrorCode function_error
                             * @property {google.ads.googleads.v11.errors.FunctionParsingErrorEnum.FunctionParsingError|null} [function_parsing_error] ErrorCode function_parsing_error
                             * @property {google.ads.googleads.v11.errors.IdErrorEnum.IdError|null} [id_error] ErrorCode id_error
                             * @property {google.ads.googleads.v11.errors.ImageErrorEnum.ImageError|null} [image_error] ErrorCode image_error
                             * @property {google.ads.googleads.v11.errors.LanguageCodeErrorEnum.LanguageCodeError|null} [language_code_error] ErrorCode language_code_error
                             * @property {google.ads.googleads.v11.errors.MediaBundleErrorEnum.MediaBundleError|null} [media_bundle_error] ErrorCode media_bundle_error
                             * @property {google.ads.googleads.v11.errors.MediaUploadErrorEnum.MediaUploadError|null} [media_upload_error] ErrorCode media_upload_error
                             * @property {google.ads.googleads.v11.errors.MediaFileErrorEnum.MediaFileError|null} [media_file_error] ErrorCode media_file_error
                             * @property {google.ads.googleads.v11.errors.MerchantCenterErrorEnum.MerchantCenterError|null} [merchant_center_error] ErrorCode merchant_center_error
                             * @property {google.ads.googleads.v11.errors.MultiplierErrorEnum.MultiplierError|null} [multiplier_error] ErrorCode multiplier_error
                             * @property {google.ads.googleads.v11.errors.NewResourceCreationErrorEnum.NewResourceCreationError|null} [new_resource_creation_error] ErrorCode new_resource_creation_error
                             * @property {google.ads.googleads.v11.errors.NotEmptyErrorEnum.NotEmptyError|null} [not_empty_error] ErrorCode not_empty_error
                             * @property {google.ads.googleads.v11.errors.NullErrorEnum.NullError|null} [null_error] ErrorCode null_error
                             * @property {google.ads.googleads.v11.errors.OperatorErrorEnum.OperatorError|null} [operator_error] ErrorCode operator_error
                             * @property {google.ads.googleads.v11.errors.RangeErrorEnum.RangeError|null} [range_error] ErrorCode range_error
                             * @property {google.ads.googleads.v11.errors.RecommendationErrorEnum.RecommendationError|null} [recommendation_error] ErrorCode recommendation_error
                             * @property {google.ads.googleads.v11.errors.RegionCodeErrorEnum.RegionCodeError|null} [region_code_error] ErrorCode region_code_error
                             * @property {google.ads.googleads.v11.errors.SettingErrorEnum.SettingError|null} [setting_error] ErrorCode setting_error
                             * @property {google.ads.googleads.v11.errors.StringFormatErrorEnum.StringFormatError|null} [string_format_error] ErrorCode string_format_error
                             * @property {google.ads.googleads.v11.errors.StringLengthErrorEnum.StringLengthError|null} [string_length_error] ErrorCode string_length_error
                             * @property {google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum.OperationAccessDeniedError|null} [operation_access_denied_error] ErrorCode operation_access_denied_error
                             * @property {google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError|null} [resource_access_denied_error] ErrorCode resource_access_denied_error
                             * @property {google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError|null} [resource_count_limit_exceeded_error] ErrorCode resource_count_limit_exceeded_error
                             * @property {google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError|null} [youtube_video_registration_error] ErrorCode youtube_video_registration_error
                             * @property {google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum.AdGroupBidModifierError|null} [ad_group_bid_modifier_error] ErrorCode ad_group_bid_modifier_error
                             * @property {google.ads.googleads.v11.errors.ContextErrorEnum.ContextError|null} [context_error] ErrorCode context_error
                             * @property {google.ads.googleads.v11.errors.FieldErrorEnum.FieldError|null} [field_error] ErrorCode field_error
                             * @property {google.ads.googleads.v11.errors.SharedSetErrorEnum.SharedSetError|null} [shared_set_error] ErrorCode shared_set_error
                             * @property {google.ads.googleads.v11.errors.SharedCriterionErrorEnum.SharedCriterionError|null} [shared_criterion_error] ErrorCode shared_criterion_error
                             * @property {google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum.CampaignSharedSetError|null} [campaign_shared_set_error] ErrorCode campaign_shared_set_error
                             * @property {google.ads.googleads.v11.errors.ConversionActionErrorEnum.ConversionActionError|null} [conversion_action_error] ErrorCode conversion_action_error
                             * @property {google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError|null} [conversion_adjustment_upload_error] ErrorCode conversion_adjustment_upload_error
                             * @property {google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum.ConversionCustomVariableError|null} [conversion_custom_variable_error] ErrorCode conversion_custom_variable_error
                             * @property {google.ads.googleads.v11.errors.ConversionUploadErrorEnum.ConversionUploadError|null} [conversion_upload_error] ErrorCode conversion_upload_error
                             * @property {google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum.ConversionValueRuleError|null} [conversion_value_rule_error] ErrorCode conversion_value_rule_error
                             * @property {google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError|null} [conversion_value_rule_set_error] ErrorCode conversion_value_rule_set_error
                             * @property {google.ads.googleads.v11.errors.HeaderErrorEnum.HeaderError|null} [header_error] ErrorCode header_error
                             * @property {google.ads.googleads.v11.errors.DatabaseErrorEnum.DatabaseError|null} [database_error] ErrorCode database_error
                             * @property {google.ads.googleads.v11.errors.PolicyFindingErrorEnum.PolicyFindingError|null} [policy_finding_error] ErrorCode policy_finding_error
                             * @property {google.ads.googleads.v11.errors.EnumErrorEnum.EnumError|null} [enum_error] ErrorCode enum_error
                             * @property {google.ads.googleads.v11.errors.KeywordPlanErrorEnum.KeywordPlanError|null} [keyword_plan_error] ErrorCode keyword_plan_error
                             * @property {google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError|null} [keyword_plan_campaign_error] ErrorCode keyword_plan_campaign_error
                             * @property {google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError|null} [keyword_plan_campaign_keyword_error] ErrorCode keyword_plan_campaign_keyword_error
                             * @property {google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError|null} [keyword_plan_ad_group_error] ErrorCode keyword_plan_ad_group_error
                             * @property {google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError|null} [keyword_plan_ad_group_keyword_error] ErrorCode keyword_plan_ad_group_keyword_error
                             * @property {google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError|null} [keyword_plan_idea_error] ErrorCode keyword_plan_idea_error
                             * @property {google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum.AccountBudgetProposalError|null} [account_budget_proposal_error] ErrorCode account_budget_proposal_error
                             * @property {google.ads.googleads.v11.errors.UserListErrorEnum.UserListError|null} [user_list_error] ErrorCode user_list_error
                             * @property {google.ads.googleads.v11.errors.ChangeEventErrorEnum.ChangeEventError|null} [change_event_error] ErrorCode change_event_error
                             * @property {google.ads.googleads.v11.errors.ChangeStatusErrorEnum.ChangeStatusError|null} [change_status_error] ErrorCode change_status_error
                             * @property {google.ads.googleads.v11.errors.FeedErrorEnum.FeedError|null} [feed_error] ErrorCode feed_error
                             * @property {google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError|null} [geo_target_constant_suggestion_error] ErrorCode geo_target_constant_suggestion_error
                             * @property {google.ads.googleads.v11.errors.CampaignDraftErrorEnum.CampaignDraftError|null} [campaign_draft_error] ErrorCode campaign_draft_error
                             * @property {google.ads.googleads.v11.errors.FeedItemErrorEnum.FeedItemError|null} [feed_item_error] ErrorCode feed_item_error
                             * @property {google.ads.googleads.v11.errors.LabelErrorEnum.LabelError|null} [label_error] ErrorCode label_error
                             * @property {google.ads.googleads.v11.errors.BillingSetupErrorEnum.BillingSetupError|null} [billing_setup_error] ErrorCode billing_setup_error
                             * @property {google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum.CustomerClientLinkError|null} [customer_client_link_error] ErrorCode customer_client_link_error
                             * @property {google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum.CustomerManagerLinkError|null} [customer_manager_link_error] ErrorCode customer_manager_link_error
                             * @property {google.ads.googleads.v11.errors.FeedMappingErrorEnum.FeedMappingError|null} [feed_mapping_error] ErrorCode feed_mapping_error
                             * @property {google.ads.googleads.v11.errors.CustomerFeedErrorEnum.CustomerFeedError|null} [customer_feed_error] ErrorCode customer_feed_error
                             * @property {google.ads.googleads.v11.errors.AdGroupFeedErrorEnum.AdGroupFeedError|null} [ad_group_feed_error] ErrorCode ad_group_feed_error
                             * @property {google.ads.googleads.v11.errors.CampaignFeedErrorEnum.CampaignFeedError|null} [campaign_feed_error] ErrorCode campaign_feed_error
                             * @property {google.ads.googleads.v11.errors.CustomInterestErrorEnum.CustomInterestError|null} [custom_interest_error] ErrorCode custom_interest_error
                             * @property {google.ads.googleads.v11.errors.CampaignExperimentErrorEnum.CampaignExperimentError|null} [campaign_experiment_error] ErrorCode campaign_experiment_error
                             * @property {google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum.ExtensionFeedItemError|null} [extension_feed_item_error] ErrorCode extension_feed_item_error
                             * @property {google.ads.googleads.v11.errors.AdParameterErrorEnum.AdParameterError|null} [ad_parameter_error] ErrorCode ad_parameter_error
                             * @property {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.FeedItemValidationError|null} [feed_item_validation_error] ErrorCode feed_item_validation_error
                             * @property {google.ads.googleads.v11.errors.ExtensionSettingErrorEnum.ExtensionSettingError|null} [extension_setting_error] ErrorCode extension_setting_error
                             * @property {google.ads.googleads.v11.errors.FeedItemSetErrorEnum.FeedItemSetError|null} [feed_item_set_error] ErrorCode feed_item_set_error
                             * @property {google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum.FeedItemSetLinkError|null} [feed_item_set_link_error] ErrorCode feed_item_set_link_error
                             * @property {google.ads.googleads.v11.errors.FeedItemTargetErrorEnum.FeedItemTargetError|null} [feed_item_target_error] ErrorCode feed_item_target_error
                             * @property {google.ads.googleads.v11.errors.PolicyViolationErrorEnum.PolicyViolationError|null} [policy_violation_error] ErrorCode policy_violation_error
                             * @property {google.ads.googleads.v11.errors.PartialFailureErrorEnum.PartialFailureError|null} [partial_failure_error] ErrorCode partial_failure_error
                             * @property {google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum.PolicyValidationParameterError|null} [policy_validation_parameter_error] ErrorCode policy_validation_parameter_error
                             * @property {google.ads.googleads.v11.errors.SizeLimitErrorEnum.SizeLimitError|null} [size_limit_error] ErrorCode size_limit_error
                             * @property {google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum.OfflineUserDataJobError|null} [offline_user_data_job_error] ErrorCode offline_user_data_job_error
                             * @property {google.ads.googleads.v11.errors.NotAllowlistedErrorEnum.NotAllowlistedError|null} [not_allowlisted_error] ErrorCode not_allowlisted_error
                             * @property {google.ads.googleads.v11.errors.ManagerLinkErrorEnum.ManagerLinkError|null} [manager_link_error] ErrorCode manager_link_error
                             * @property {google.ads.googleads.v11.errors.CurrencyCodeErrorEnum.CurrencyCodeError|null} [currency_code_error] ErrorCode currency_code_error
                             * @property {google.ads.googleads.v11.errors.ExperimentErrorEnum.ExperimentError|null} [experiment_error] ErrorCode experiment_error
                             * @property {google.ads.googleads.v11.errors.AccessInvitationErrorEnum.AccessInvitationError|null} [access_invitation_error] ErrorCode access_invitation_error
                             * @property {google.ads.googleads.v11.errors.ReachPlanErrorEnum.ReachPlanError|null} [reach_plan_error] ErrorCode reach_plan_error
                             * @property {google.ads.googleads.v11.errors.InvoiceErrorEnum.InvoiceError|null} [invoice_error] ErrorCode invoice_error
                             * @property {google.ads.googleads.v11.errors.PaymentsAccountErrorEnum.PaymentsAccountError|null} [payments_account_error] ErrorCode payments_account_error
                             * @property {google.ads.googleads.v11.errors.TimeZoneErrorEnum.TimeZoneError|null} [time_zone_error] ErrorCode time_zone_error
                             * @property {google.ads.googleads.v11.errors.AssetLinkErrorEnum.AssetLinkError|null} [asset_link_error] ErrorCode asset_link_error
                             * @property {google.ads.googleads.v11.errors.UserDataErrorEnum.UserDataError|null} [user_data_error] ErrorCode user_data_error
                             * @property {google.ads.googleads.v11.errors.BatchJobErrorEnum.BatchJobError|null} [batch_job_error] ErrorCode batch_job_error
                             * @property {google.ads.googleads.v11.errors.AccountLinkErrorEnum.AccountLinkError|null} [account_link_error] ErrorCode account_link_error
                             * @property {google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError|null} [third_party_app_analytics_link_error] ErrorCode third_party_app_analytics_link_error
                             * @property {google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum.CustomerUserAccessError|null} [customer_user_access_error] ErrorCode customer_user_access_error
                             * @property {google.ads.googleads.v11.errors.CustomAudienceErrorEnum.CustomAudienceError|null} [custom_audience_error] ErrorCode custom_audience_error
                             * @property {google.ads.googleads.v11.errors.AudienceErrorEnum.AudienceError|null} [audience_error] ErrorCode audience_error
                             * @property {google.ads.googleads.v11.errors.ExperimentArmErrorEnum.ExperimentArmError|null} [experiment_arm_error] ErrorCode experiment_arm_error
                             */
    
                            /**
                             * Constructs a new ErrorCode.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ErrorCode.
                             * @implements IErrorCode
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IErrorCode=} [p] Properties to set
                             */
                            function ErrorCode(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ErrorCode request_error.
                             * @member {google.ads.googleads.v11.errors.RequestErrorEnum.RequestError|null|undefined} request_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.request_error = null;
    
                            /**
                             * ErrorCode bidding_strategy_error.
                             * @member {google.ads.googleads.v11.errors.BiddingStrategyErrorEnum.BiddingStrategyError|null|undefined} bidding_strategy_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.bidding_strategy_error = null;
    
                            /**
                             * ErrorCode url_field_error.
                             * @member {google.ads.googleads.v11.errors.UrlFieldErrorEnum.UrlFieldError|null|undefined} url_field_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.url_field_error = null;
    
                            /**
                             * ErrorCode list_operation_error.
                             * @member {google.ads.googleads.v11.errors.ListOperationErrorEnum.ListOperationError|null|undefined} list_operation_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.list_operation_error = null;
    
                            /**
                             * ErrorCode query_error.
                             * @member {google.ads.googleads.v11.errors.QueryErrorEnum.QueryError|null|undefined} query_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.query_error = null;
    
                            /**
                             * ErrorCode mutate_error.
                             * @member {google.ads.googleads.v11.errors.MutateErrorEnum.MutateError|null|undefined} mutate_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.mutate_error = null;
    
                            /**
                             * ErrorCode field_mask_error.
                             * @member {google.ads.googleads.v11.errors.FieldMaskErrorEnum.FieldMaskError|null|undefined} field_mask_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.field_mask_error = null;
    
                            /**
                             * ErrorCode authorization_error.
                             * @member {google.ads.googleads.v11.errors.AuthorizationErrorEnum.AuthorizationError|null|undefined} authorization_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.authorization_error = null;
    
                            /**
                             * ErrorCode internal_error.
                             * @member {google.ads.googleads.v11.errors.InternalErrorEnum.InternalError|null|undefined} internal_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.internal_error = null;
    
                            /**
                             * ErrorCode quota_error.
                             * @member {google.ads.googleads.v11.errors.QuotaErrorEnum.QuotaError|null|undefined} quota_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.quota_error = null;
    
                            /**
                             * ErrorCode ad_error.
                             * @member {google.ads.googleads.v11.errors.AdErrorEnum.AdError|null|undefined} ad_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_error = null;
    
                            /**
                             * ErrorCode ad_group_error.
                             * @member {google.ads.googleads.v11.errors.AdGroupErrorEnum.AdGroupError|null|undefined} ad_group_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_group_error = null;
    
                            /**
                             * ErrorCode campaign_budget_error.
                             * @member {google.ads.googleads.v11.errors.CampaignBudgetErrorEnum.CampaignBudgetError|null|undefined} campaign_budget_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_budget_error = null;
    
                            /**
                             * ErrorCode campaign_error.
                             * @member {google.ads.googleads.v11.errors.CampaignErrorEnum.CampaignError|null|undefined} campaign_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_error = null;
    
                            /**
                             * ErrorCode authentication_error.
                             * @member {google.ads.googleads.v11.errors.AuthenticationErrorEnum.AuthenticationError|null|undefined} authentication_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.authentication_error = null;
    
                            /**
                             * ErrorCode ad_group_criterion_customizer_error.
                             * @member {google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum.AdGroupCriterionCustomizerError|null|undefined} ad_group_criterion_customizer_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_group_criterion_customizer_error = null;
    
                            /**
                             * ErrorCode ad_group_criterion_error.
                             * @member {google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum.AdGroupCriterionError|null|undefined} ad_group_criterion_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_group_criterion_error = null;
    
                            /**
                             * ErrorCode ad_group_customizer_error.
                             * @member {google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum.AdGroupCustomizerError|null|undefined} ad_group_customizer_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_group_customizer_error = null;
    
                            /**
                             * ErrorCode ad_customizer_error.
                             * @member {google.ads.googleads.v11.errors.AdCustomizerErrorEnum.AdCustomizerError|null|undefined} ad_customizer_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_customizer_error = null;
    
                            /**
                             * ErrorCode ad_group_ad_error.
                             * @member {google.ads.googleads.v11.errors.AdGroupAdErrorEnum.AdGroupAdError|null|undefined} ad_group_ad_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_group_ad_error = null;
    
                            /**
                             * ErrorCode ad_sharing_error.
                             * @member {google.ads.googleads.v11.errors.AdSharingErrorEnum.AdSharingError|null|undefined} ad_sharing_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_sharing_error = null;
    
                            /**
                             * ErrorCode adx_error.
                             * @member {google.ads.googleads.v11.errors.AdxErrorEnum.AdxError|null|undefined} adx_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.adx_error = null;
    
                            /**
                             * ErrorCode asset_error.
                             * @member {google.ads.googleads.v11.errors.AssetErrorEnum.AssetError|null|undefined} asset_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_error = null;
    
                            /**
                             * ErrorCode asset_group_asset_error.
                             * @member {google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum.AssetGroupAssetError|null|undefined} asset_group_asset_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_group_asset_error = null;
    
                            /**
                             * ErrorCode asset_group_listing_group_filter_error.
                             * @member {google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum.AssetGroupListingGroupFilterError|null|undefined} asset_group_listing_group_filter_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_group_listing_group_filter_error = null;
    
                            /**
                             * ErrorCode asset_group_error.
                             * @member {google.ads.googleads.v11.errors.AssetGroupErrorEnum.AssetGroupError|null|undefined} asset_group_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_group_error = null;
    
                            /**
                             * ErrorCode asset_set_asset_error.
                             * @member {google.ads.googleads.v11.errors.AssetSetAssetErrorEnum.AssetSetAssetError|null|undefined} asset_set_asset_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_set_asset_error = null;
    
                            /**
                             * ErrorCode asset_set_link_error.
                             * @member {google.ads.googleads.v11.errors.AssetSetLinkErrorEnum.AssetSetLinkError|null|undefined} asset_set_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_set_link_error = null;
    
                            /**
                             * ErrorCode asset_set_error.
                             * @member {google.ads.googleads.v11.errors.AssetSetErrorEnum.AssetSetError|null|undefined} asset_set_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_set_error = null;
    
                            /**
                             * ErrorCode bidding_error.
                             * @member {google.ads.googleads.v11.errors.BiddingErrorEnum.BiddingError|null|undefined} bidding_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.bidding_error = null;
    
                            /**
                             * ErrorCode campaign_criterion_error.
                             * @member {google.ads.googleads.v11.errors.CampaignCriterionErrorEnum.CampaignCriterionError|null|undefined} campaign_criterion_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_criterion_error = null;
    
                            /**
                             * ErrorCode campaign_conversion_goal_error.
                             * @member {google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum.CampaignConversionGoalError|null|undefined} campaign_conversion_goal_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_conversion_goal_error = null;
    
                            /**
                             * ErrorCode campaign_customizer_error.
                             * @member {google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum.CampaignCustomizerError|null|undefined} campaign_customizer_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_customizer_error = null;
    
                            /**
                             * ErrorCode collection_size_error.
                             * @member {google.ads.googleads.v11.errors.CollectionSizeErrorEnum.CollectionSizeError|null|undefined} collection_size_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.collection_size_error = null;
    
                            /**
                             * ErrorCode conversion_goal_campaign_config_error.
                             * @member {google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum.ConversionGoalCampaignConfigError|null|undefined} conversion_goal_campaign_config_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.conversion_goal_campaign_config_error = null;
    
                            /**
                             * ErrorCode country_code_error.
                             * @member {google.ads.googleads.v11.errors.CountryCodeErrorEnum.CountryCodeError|null|undefined} country_code_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.country_code_error = null;
    
                            /**
                             * ErrorCode criterion_error.
                             * @member {google.ads.googleads.v11.errors.CriterionErrorEnum.CriterionError|null|undefined} criterion_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.criterion_error = null;
    
                            /**
                             * ErrorCode custom_conversion_goal_error.
                             * @member {google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum.CustomConversionGoalError|null|undefined} custom_conversion_goal_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.custom_conversion_goal_error = null;
    
                            /**
                             * ErrorCode customer_customizer_error.
                             * @member {google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum.CustomerCustomizerError|null|undefined} customer_customizer_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.customer_customizer_error = null;
    
                            /**
                             * ErrorCode customer_error.
                             * @member {google.ads.googleads.v11.errors.CustomerErrorEnum.CustomerError|null|undefined} customer_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.customer_error = null;
    
                            /**
                             * ErrorCode customizer_attribute_error.
                             * @member {google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum.CustomizerAttributeError|null|undefined} customizer_attribute_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.customizer_attribute_error = null;
    
                            /**
                             * ErrorCode date_error.
                             * @member {google.ads.googleads.v11.errors.DateErrorEnum.DateError|null|undefined} date_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.date_error = null;
    
                            /**
                             * ErrorCode date_range_error.
                             * @member {google.ads.googleads.v11.errors.DateRangeErrorEnum.DateRangeError|null|undefined} date_range_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.date_range_error = null;
    
                            /**
                             * ErrorCode distinct_error.
                             * @member {google.ads.googleads.v11.errors.DistinctErrorEnum.DistinctError|null|undefined} distinct_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.distinct_error = null;
    
                            /**
                             * ErrorCode feed_attribute_reference_error.
                             * @member {google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError|null|undefined} feed_attribute_reference_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_attribute_reference_error = null;
    
                            /**
                             * ErrorCode function_error.
                             * @member {google.ads.googleads.v11.errors.FunctionErrorEnum.FunctionError|null|undefined} function_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.function_error = null;
    
                            /**
                             * ErrorCode function_parsing_error.
                             * @member {google.ads.googleads.v11.errors.FunctionParsingErrorEnum.FunctionParsingError|null|undefined} function_parsing_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.function_parsing_error = null;
    
                            /**
                             * ErrorCode id_error.
                             * @member {google.ads.googleads.v11.errors.IdErrorEnum.IdError|null|undefined} id_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.id_error = null;
    
                            /**
                             * ErrorCode image_error.
                             * @member {google.ads.googleads.v11.errors.ImageErrorEnum.ImageError|null|undefined} image_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.image_error = null;
    
                            /**
                             * ErrorCode language_code_error.
                             * @member {google.ads.googleads.v11.errors.LanguageCodeErrorEnum.LanguageCodeError|null|undefined} language_code_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.language_code_error = null;
    
                            /**
                             * ErrorCode media_bundle_error.
                             * @member {google.ads.googleads.v11.errors.MediaBundleErrorEnum.MediaBundleError|null|undefined} media_bundle_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.media_bundle_error = null;
    
                            /**
                             * ErrorCode media_upload_error.
                             * @member {google.ads.googleads.v11.errors.MediaUploadErrorEnum.MediaUploadError|null|undefined} media_upload_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.media_upload_error = null;
    
                            /**
                             * ErrorCode media_file_error.
                             * @member {google.ads.googleads.v11.errors.MediaFileErrorEnum.MediaFileError|null|undefined} media_file_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.media_file_error = null;
    
                            /**
                             * ErrorCode merchant_center_error.
                             * @member {google.ads.googleads.v11.errors.MerchantCenterErrorEnum.MerchantCenterError|null|undefined} merchant_center_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.merchant_center_error = null;
    
                            /**
                             * ErrorCode multiplier_error.
                             * @member {google.ads.googleads.v11.errors.MultiplierErrorEnum.MultiplierError|null|undefined} multiplier_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.multiplier_error = null;
    
                            /**
                             * ErrorCode new_resource_creation_error.
                             * @member {google.ads.googleads.v11.errors.NewResourceCreationErrorEnum.NewResourceCreationError|null|undefined} new_resource_creation_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.new_resource_creation_error = null;
    
                            /**
                             * ErrorCode not_empty_error.
                             * @member {google.ads.googleads.v11.errors.NotEmptyErrorEnum.NotEmptyError|null|undefined} not_empty_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.not_empty_error = null;
    
                            /**
                             * ErrorCode null_error.
                             * @member {google.ads.googleads.v11.errors.NullErrorEnum.NullError|null|undefined} null_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.null_error = null;
    
                            /**
                             * ErrorCode operator_error.
                             * @member {google.ads.googleads.v11.errors.OperatorErrorEnum.OperatorError|null|undefined} operator_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.operator_error = null;
    
                            /**
                             * ErrorCode range_error.
                             * @member {google.ads.googleads.v11.errors.RangeErrorEnum.RangeError|null|undefined} range_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.range_error = null;
    
                            /**
                             * ErrorCode recommendation_error.
                             * @member {google.ads.googleads.v11.errors.RecommendationErrorEnum.RecommendationError|null|undefined} recommendation_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.recommendation_error = null;
    
                            /**
                             * ErrorCode region_code_error.
                             * @member {google.ads.googleads.v11.errors.RegionCodeErrorEnum.RegionCodeError|null|undefined} region_code_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.region_code_error = null;
    
                            /**
                             * ErrorCode setting_error.
                             * @member {google.ads.googleads.v11.errors.SettingErrorEnum.SettingError|null|undefined} setting_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.setting_error = null;
    
                            /**
                             * ErrorCode string_format_error.
                             * @member {google.ads.googleads.v11.errors.StringFormatErrorEnum.StringFormatError|null|undefined} string_format_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.string_format_error = null;
    
                            /**
                             * ErrorCode string_length_error.
                             * @member {google.ads.googleads.v11.errors.StringLengthErrorEnum.StringLengthError|null|undefined} string_length_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.string_length_error = null;
    
                            /**
                             * ErrorCode operation_access_denied_error.
                             * @member {google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum.OperationAccessDeniedError|null|undefined} operation_access_denied_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.operation_access_denied_error = null;
    
                            /**
                             * ErrorCode resource_access_denied_error.
                             * @member {google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError|null|undefined} resource_access_denied_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.resource_access_denied_error = null;
    
                            /**
                             * ErrorCode resource_count_limit_exceeded_error.
                             * @member {google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError|null|undefined} resource_count_limit_exceeded_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.resource_count_limit_exceeded_error = null;
    
                            /**
                             * ErrorCode youtube_video_registration_error.
                             * @member {google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError|null|undefined} youtube_video_registration_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.youtube_video_registration_error = null;
    
                            /**
                             * ErrorCode ad_group_bid_modifier_error.
                             * @member {google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum.AdGroupBidModifierError|null|undefined} ad_group_bid_modifier_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_group_bid_modifier_error = null;
    
                            /**
                             * ErrorCode context_error.
                             * @member {google.ads.googleads.v11.errors.ContextErrorEnum.ContextError|null|undefined} context_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.context_error = null;
    
                            /**
                             * ErrorCode field_error.
                             * @member {google.ads.googleads.v11.errors.FieldErrorEnum.FieldError|null|undefined} field_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.field_error = null;
    
                            /**
                             * ErrorCode shared_set_error.
                             * @member {google.ads.googleads.v11.errors.SharedSetErrorEnum.SharedSetError|null|undefined} shared_set_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.shared_set_error = null;
    
                            /**
                             * ErrorCode shared_criterion_error.
                             * @member {google.ads.googleads.v11.errors.SharedCriterionErrorEnum.SharedCriterionError|null|undefined} shared_criterion_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.shared_criterion_error = null;
    
                            /**
                             * ErrorCode campaign_shared_set_error.
                             * @member {google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum.CampaignSharedSetError|null|undefined} campaign_shared_set_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_shared_set_error = null;
    
                            /**
                             * ErrorCode conversion_action_error.
                             * @member {google.ads.googleads.v11.errors.ConversionActionErrorEnum.ConversionActionError|null|undefined} conversion_action_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.conversion_action_error = null;
    
                            /**
                             * ErrorCode conversion_adjustment_upload_error.
                             * @member {google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError|null|undefined} conversion_adjustment_upload_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.conversion_adjustment_upload_error = null;
    
                            /**
                             * ErrorCode conversion_custom_variable_error.
                             * @member {google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum.ConversionCustomVariableError|null|undefined} conversion_custom_variable_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.conversion_custom_variable_error = null;
    
                            /**
                             * ErrorCode conversion_upload_error.
                             * @member {google.ads.googleads.v11.errors.ConversionUploadErrorEnum.ConversionUploadError|null|undefined} conversion_upload_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.conversion_upload_error = null;
    
                            /**
                             * ErrorCode conversion_value_rule_error.
                             * @member {google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum.ConversionValueRuleError|null|undefined} conversion_value_rule_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.conversion_value_rule_error = null;
    
                            /**
                             * ErrorCode conversion_value_rule_set_error.
                             * @member {google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError|null|undefined} conversion_value_rule_set_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.conversion_value_rule_set_error = null;
    
                            /**
                             * ErrorCode header_error.
                             * @member {google.ads.googleads.v11.errors.HeaderErrorEnum.HeaderError|null|undefined} header_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.header_error = null;
    
                            /**
                             * ErrorCode database_error.
                             * @member {google.ads.googleads.v11.errors.DatabaseErrorEnum.DatabaseError|null|undefined} database_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.database_error = null;
    
                            /**
                             * ErrorCode policy_finding_error.
                             * @member {google.ads.googleads.v11.errors.PolicyFindingErrorEnum.PolicyFindingError|null|undefined} policy_finding_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.policy_finding_error = null;
    
                            /**
                             * ErrorCode enum_error.
                             * @member {google.ads.googleads.v11.errors.EnumErrorEnum.EnumError|null|undefined} enum_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.enum_error = null;
    
                            /**
                             * ErrorCode keyword_plan_error.
                             * @member {google.ads.googleads.v11.errors.KeywordPlanErrorEnum.KeywordPlanError|null|undefined} keyword_plan_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.keyword_plan_error = null;
    
                            /**
                             * ErrorCode keyword_plan_campaign_error.
                             * @member {google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError|null|undefined} keyword_plan_campaign_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.keyword_plan_campaign_error = null;
    
                            /**
                             * ErrorCode keyword_plan_campaign_keyword_error.
                             * @member {google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError|null|undefined} keyword_plan_campaign_keyword_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.keyword_plan_campaign_keyword_error = null;
    
                            /**
                             * ErrorCode keyword_plan_ad_group_error.
                             * @member {google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError|null|undefined} keyword_plan_ad_group_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.keyword_plan_ad_group_error = null;
    
                            /**
                             * ErrorCode keyword_plan_ad_group_keyword_error.
                             * @member {google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError|null|undefined} keyword_plan_ad_group_keyword_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.keyword_plan_ad_group_keyword_error = null;
    
                            /**
                             * ErrorCode keyword_plan_idea_error.
                             * @member {google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError|null|undefined} keyword_plan_idea_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.keyword_plan_idea_error = null;
    
                            /**
                             * ErrorCode account_budget_proposal_error.
                             * @member {google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum.AccountBudgetProposalError|null|undefined} account_budget_proposal_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.account_budget_proposal_error = null;
    
                            /**
                             * ErrorCode user_list_error.
                             * @member {google.ads.googleads.v11.errors.UserListErrorEnum.UserListError|null|undefined} user_list_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.user_list_error = null;
    
                            /**
                             * ErrorCode change_event_error.
                             * @member {google.ads.googleads.v11.errors.ChangeEventErrorEnum.ChangeEventError|null|undefined} change_event_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.change_event_error = null;
    
                            /**
                             * ErrorCode change_status_error.
                             * @member {google.ads.googleads.v11.errors.ChangeStatusErrorEnum.ChangeStatusError|null|undefined} change_status_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.change_status_error = null;
    
                            /**
                             * ErrorCode feed_error.
                             * @member {google.ads.googleads.v11.errors.FeedErrorEnum.FeedError|null|undefined} feed_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_error = null;
    
                            /**
                             * ErrorCode geo_target_constant_suggestion_error.
                             * @member {google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError|null|undefined} geo_target_constant_suggestion_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.geo_target_constant_suggestion_error = null;
    
                            /**
                             * ErrorCode campaign_draft_error.
                             * @member {google.ads.googleads.v11.errors.CampaignDraftErrorEnum.CampaignDraftError|null|undefined} campaign_draft_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_draft_error = null;
    
                            /**
                             * ErrorCode feed_item_error.
                             * @member {google.ads.googleads.v11.errors.FeedItemErrorEnum.FeedItemError|null|undefined} feed_item_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_item_error = null;
    
                            /**
                             * ErrorCode label_error.
                             * @member {google.ads.googleads.v11.errors.LabelErrorEnum.LabelError|null|undefined} label_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.label_error = null;
    
                            /**
                             * ErrorCode billing_setup_error.
                             * @member {google.ads.googleads.v11.errors.BillingSetupErrorEnum.BillingSetupError|null|undefined} billing_setup_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.billing_setup_error = null;
    
                            /**
                             * ErrorCode customer_client_link_error.
                             * @member {google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum.CustomerClientLinkError|null|undefined} customer_client_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.customer_client_link_error = null;
    
                            /**
                             * ErrorCode customer_manager_link_error.
                             * @member {google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum.CustomerManagerLinkError|null|undefined} customer_manager_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.customer_manager_link_error = null;
    
                            /**
                             * ErrorCode feed_mapping_error.
                             * @member {google.ads.googleads.v11.errors.FeedMappingErrorEnum.FeedMappingError|null|undefined} feed_mapping_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_mapping_error = null;
    
                            /**
                             * ErrorCode customer_feed_error.
                             * @member {google.ads.googleads.v11.errors.CustomerFeedErrorEnum.CustomerFeedError|null|undefined} customer_feed_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.customer_feed_error = null;
    
                            /**
                             * ErrorCode ad_group_feed_error.
                             * @member {google.ads.googleads.v11.errors.AdGroupFeedErrorEnum.AdGroupFeedError|null|undefined} ad_group_feed_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_group_feed_error = null;
    
                            /**
                             * ErrorCode campaign_feed_error.
                             * @member {google.ads.googleads.v11.errors.CampaignFeedErrorEnum.CampaignFeedError|null|undefined} campaign_feed_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_feed_error = null;
    
                            /**
                             * ErrorCode custom_interest_error.
                             * @member {google.ads.googleads.v11.errors.CustomInterestErrorEnum.CustomInterestError|null|undefined} custom_interest_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.custom_interest_error = null;
    
                            /**
                             * ErrorCode campaign_experiment_error.
                             * @member {google.ads.googleads.v11.errors.CampaignExperimentErrorEnum.CampaignExperimentError|null|undefined} campaign_experiment_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.campaign_experiment_error = null;
    
                            /**
                             * ErrorCode extension_feed_item_error.
                             * @member {google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum.ExtensionFeedItemError|null|undefined} extension_feed_item_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.extension_feed_item_error = null;
    
                            /**
                             * ErrorCode ad_parameter_error.
                             * @member {google.ads.googleads.v11.errors.AdParameterErrorEnum.AdParameterError|null|undefined} ad_parameter_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.ad_parameter_error = null;
    
                            /**
                             * ErrorCode feed_item_validation_error.
                             * @member {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.FeedItemValidationError|null|undefined} feed_item_validation_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_item_validation_error = null;
    
                            /**
                             * ErrorCode extension_setting_error.
                             * @member {google.ads.googleads.v11.errors.ExtensionSettingErrorEnum.ExtensionSettingError|null|undefined} extension_setting_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.extension_setting_error = null;
    
                            /**
                             * ErrorCode feed_item_set_error.
                             * @member {google.ads.googleads.v11.errors.FeedItemSetErrorEnum.FeedItemSetError|null|undefined} feed_item_set_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_item_set_error = null;
    
                            /**
                             * ErrorCode feed_item_set_link_error.
                             * @member {google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum.FeedItemSetLinkError|null|undefined} feed_item_set_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_item_set_link_error = null;
    
                            /**
                             * ErrorCode feed_item_target_error.
                             * @member {google.ads.googleads.v11.errors.FeedItemTargetErrorEnum.FeedItemTargetError|null|undefined} feed_item_target_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.feed_item_target_error = null;
    
                            /**
                             * ErrorCode policy_violation_error.
                             * @member {google.ads.googleads.v11.errors.PolicyViolationErrorEnum.PolicyViolationError|null|undefined} policy_violation_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.policy_violation_error = null;
    
                            /**
                             * ErrorCode partial_failure_error.
                             * @member {google.ads.googleads.v11.errors.PartialFailureErrorEnum.PartialFailureError|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.partial_failure_error = null;
    
                            /**
                             * ErrorCode policy_validation_parameter_error.
                             * @member {google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum.PolicyValidationParameterError|null|undefined} policy_validation_parameter_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.policy_validation_parameter_error = null;
    
                            /**
                             * ErrorCode size_limit_error.
                             * @member {google.ads.googleads.v11.errors.SizeLimitErrorEnum.SizeLimitError|null|undefined} size_limit_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.size_limit_error = null;
    
                            /**
                             * ErrorCode offline_user_data_job_error.
                             * @member {google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum.OfflineUserDataJobError|null|undefined} offline_user_data_job_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.offline_user_data_job_error = null;
    
                            /**
                             * ErrorCode not_allowlisted_error.
                             * @member {google.ads.googleads.v11.errors.NotAllowlistedErrorEnum.NotAllowlistedError|null|undefined} not_allowlisted_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.not_allowlisted_error = null;
    
                            /**
                             * ErrorCode manager_link_error.
                             * @member {google.ads.googleads.v11.errors.ManagerLinkErrorEnum.ManagerLinkError|null|undefined} manager_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.manager_link_error = null;
    
                            /**
                             * ErrorCode currency_code_error.
                             * @member {google.ads.googleads.v11.errors.CurrencyCodeErrorEnum.CurrencyCodeError|null|undefined} currency_code_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.currency_code_error = null;
    
                            /**
                             * ErrorCode experiment_error.
                             * @member {google.ads.googleads.v11.errors.ExperimentErrorEnum.ExperimentError|null|undefined} experiment_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.experiment_error = null;
    
                            /**
                             * ErrorCode access_invitation_error.
                             * @member {google.ads.googleads.v11.errors.AccessInvitationErrorEnum.AccessInvitationError|null|undefined} access_invitation_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.access_invitation_error = null;
    
                            /**
                             * ErrorCode reach_plan_error.
                             * @member {google.ads.googleads.v11.errors.ReachPlanErrorEnum.ReachPlanError|null|undefined} reach_plan_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.reach_plan_error = null;
    
                            /**
                             * ErrorCode invoice_error.
                             * @member {google.ads.googleads.v11.errors.InvoiceErrorEnum.InvoiceError|null|undefined} invoice_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.invoice_error = null;
    
                            /**
                             * ErrorCode payments_account_error.
                             * @member {google.ads.googleads.v11.errors.PaymentsAccountErrorEnum.PaymentsAccountError|null|undefined} payments_account_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.payments_account_error = null;
    
                            /**
                             * ErrorCode time_zone_error.
                             * @member {google.ads.googleads.v11.errors.TimeZoneErrorEnum.TimeZoneError|null|undefined} time_zone_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.time_zone_error = null;
    
                            /**
                             * ErrorCode asset_link_error.
                             * @member {google.ads.googleads.v11.errors.AssetLinkErrorEnum.AssetLinkError|null|undefined} asset_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.asset_link_error = null;
    
                            /**
                             * ErrorCode user_data_error.
                             * @member {google.ads.googleads.v11.errors.UserDataErrorEnum.UserDataError|null|undefined} user_data_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.user_data_error = null;
    
                            /**
                             * ErrorCode batch_job_error.
                             * @member {google.ads.googleads.v11.errors.BatchJobErrorEnum.BatchJobError|null|undefined} batch_job_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.batch_job_error = null;
    
                            /**
                             * ErrorCode account_link_error.
                             * @member {google.ads.googleads.v11.errors.AccountLinkErrorEnum.AccountLinkError|null|undefined} account_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.account_link_error = null;
    
                            /**
                             * ErrorCode third_party_app_analytics_link_error.
                             * @member {google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError|null|undefined} third_party_app_analytics_link_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.third_party_app_analytics_link_error = null;
    
                            /**
                             * ErrorCode customer_user_access_error.
                             * @member {google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum.CustomerUserAccessError|null|undefined} customer_user_access_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.customer_user_access_error = null;
    
                            /**
                             * ErrorCode custom_audience_error.
                             * @member {google.ads.googleads.v11.errors.CustomAudienceErrorEnum.CustomAudienceError|null|undefined} custom_audience_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.custom_audience_error = null;
    
                            /**
                             * ErrorCode audience_error.
                             * @member {google.ads.googleads.v11.errors.AudienceErrorEnum.AudienceError|null|undefined} audience_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.audience_error = null;
    
                            /**
                             * ErrorCode experiment_arm_error.
                             * @member {google.ads.googleads.v11.errors.ExperimentArmErrorEnum.ExperimentArmError|null|undefined} experiment_arm_error
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            ErrorCode.prototype.experiment_arm_error = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ErrorCode error_code.
                             * @member {"request_error"|"bidding_strategy_error"|"url_field_error"|"list_operation_error"|"query_error"|"mutate_error"|"field_mask_error"|"authorization_error"|"internal_error"|"quota_error"|"ad_error"|"ad_group_error"|"campaign_budget_error"|"campaign_error"|"authentication_error"|"ad_group_criterion_customizer_error"|"ad_group_criterion_error"|"ad_group_customizer_error"|"ad_customizer_error"|"ad_group_ad_error"|"ad_sharing_error"|"adx_error"|"asset_error"|"asset_group_asset_error"|"asset_group_listing_group_filter_error"|"asset_group_error"|"asset_set_asset_error"|"asset_set_link_error"|"asset_set_error"|"bidding_error"|"campaign_criterion_error"|"campaign_conversion_goal_error"|"campaign_customizer_error"|"collection_size_error"|"conversion_goal_campaign_config_error"|"country_code_error"|"criterion_error"|"custom_conversion_goal_error"|"customer_customizer_error"|"customer_error"|"customizer_attribute_error"|"date_error"|"date_range_error"|"distinct_error"|"feed_attribute_reference_error"|"function_error"|"function_parsing_error"|"id_error"|"image_error"|"language_code_error"|"media_bundle_error"|"media_upload_error"|"media_file_error"|"merchant_center_error"|"multiplier_error"|"new_resource_creation_error"|"not_empty_error"|"null_error"|"operator_error"|"range_error"|"recommendation_error"|"region_code_error"|"setting_error"|"string_format_error"|"string_length_error"|"operation_access_denied_error"|"resource_access_denied_error"|"resource_count_limit_exceeded_error"|"youtube_video_registration_error"|"ad_group_bid_modifier_error"|"context_error"|"field_error"|"shared_set_error"|"shared_criterion_error"|"campaign_shared_set_error"|"conversion_action_error"|"conversion_adjustment_upload_error"|"conversion_custom_variable_error"|"conversion_upload_error"|"conversion_value_rule_error"|"conversion_value_rule_set_error"|"header_error"|"database_error"|"policy_finding_error"|"enum_error"|"keyword_plan_error"|"keyword_plan_campaign_error"|"keyword_plan_campaign_keyword_error"|"keyword_plan_ad_group_error"|"keyword_plan_ad_group_keyword_error"|"keyword_plan_idea_error"|"account_budget_proposal_error"|"user_list_error"|"change_event_error"|"change_status_error"|"feed_error"|"geo_target_constant_suggestion_error"|"campaign_draft_error"|"feed_item_error"|"label_error"|"billing_setup_error"|"customer_client_link_error"|"customer_manager_link_error"|"feed_mapping_error"|"customer_feed_error"|"ad_group_feed_error"|"campaign_feed_error"|"custom_interest_error"|"campaign_experiment_error"|"extension_feed_item_error"|"ad_parameter_error"|"feed_item_validation_error"|"extension_setting_error"|"feed_item_set_error"|"feed_item_set_link_error"|"feed_item_target_error"|"policy_violation_error"|"partial_failure_error"|"policy_validation_parameter_error"|"size_limit_error"|"offline_user_data_job_error"|"not_allowlisted_error"|"manager_link_error"|"currency_code_error"|"experiment_error"|"access_invitation_error"|"reach_plan_error"|"invoice_error"|"payments_account_error"|"time_zone_error"|"asset_link_error"|"user_data_error"|"batch_job_error"|"account_link_error"|"third_party_app_analytics_link_error"|"customer_user_access_error"|"custom_audience_error"|"audience_error"|"experiment_arm_error"|undefined} error_code
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             */
                            Object.defineProperty(ErrorCode.prototype, "error_code", {
                                get: $util.oneOfGetter($oneOfFields = ["request_error", "bidding_strategy_error", "url_field_error", "list_operation_error", "query_error", "mutate_error", "field_mask_error", "authorization_error", "internal_error", "quota_error", "ad_error", "ad_group_error", "campaign_budget_error", "campaign_error", "authentication_error", "ad_group_criterion_customizer_error", "ad_group_criterion_error", "ad_group_customizer_error", "ad_customizer_error", "ad_group_ad_error", "ad_sharing_error", "adx_error", "asset_error", "asset_group_asset_error", "asset_group_listing_group_filter_error", "asset_group_error", "asset_set_asset_error", "asset_set_link_error", "asset_set_error", "bidding_error", "campaign_criterion_error", "campaign_conversion_goal_error", "campaign_customizer_error", "collection_size_error", "conversion_goal_campaign_config_error", "country_code_error", "criterion_error", "custom_conversion_goal_error", "customer_customizer_error", "customer_error", "customizer_attribute_error", "date_error", "date_range_error", "distinct_error", "feed_attribute_reference_error", "function_error", "function_parsing_error", "id_error", "image_error", "language_code_error", "media_bundle_error", "media_upload_error", "media_file_error", "merchant_center_error", "multiplier_error", "new_resource_creation_error", "not_empty_error", "null_error", "operator_error", "range_error", "recommendation_error", "region_code_error", "setting_error", "string_format_error", "string_length_error", "operation_access_denied_error", "resource_access_denied_error", "resource_count_limit_exceeded_error", "youtube_video_registration_error", "ad_group_bid_modifier_error", "context_error", "field_error", "shared_set_error", "shared_criterion_error", "campaign_shared_set_error", "conversion_action_error", "conversion_adjustment_upload_error", "conversion_custom_variable_error", "conversion_upload_error", "conversion_value_rule_error", "conversion_value_rule_set_error", "header_error", "database_error", "policy_finding_error", "enum_error", "keyword_plan_error", "keyword_plan_campaign_error", "keyword_plan_campaign_keyword_error", "keyword_plan_ad_group_error", "keyword_plan_ad_group_keyword_error", "keyword_plan_idea_error", "account_budget_proposal_error", "user_list_error", "change_event_error", "change_status_error", "feed_error", "geo_target_constant_suggestion_error", "campaign_draft_error", "feed_item_error", "label_error", "billing_setup_error", "customer_client_link_error", "customer_manager_link_error", "feed_mapping_error", "customer_feed_error", "ad_group_feed_error", "campaign_feed_error", "custom_interest_error", "campaign_experiment_error", "extension_feed_item_error", "ad_parameter_error", "feed_item_validation_error", "extension_setting_error", "feed_item_set_error", "feed_item_set_link_error", "feed_item_target_error", "policy_violation_error", "partial_failure_error", "policy_validation_parameter_error", "size_limit_error", "offline_user_data_job_error", "not_allowlisted_error", "manager_link_error", "currency_code_error", "experiment_error", "access_invitation_error", "reach_plan_error", "invoice_error", "payments_account_error", "time_zone_error", "asset_link_error", "user_data_error", "batch_job_error", "account_link_error", "third_party_app_analytics_link_error", "customer_user_access_error", "custom_audience_error", "audience_error", "experiment_arm_error"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ErrorCode instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorCode=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ErrorCode} ErrorCode instance
                             */
                            ErrorCode.create = function create(properties) {
                                return new ErrorCode(properties);
                            };
    
                            /**
                             * Encodes the specified ErrorCode message. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorCode.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorCode} m ErrorCode message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ErrorCode.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.request_error != null && Object.hasOwnProperty.call(m, "request_error"))
                                    w.uint32(8).int32(m.request_error);
                                if (m.bidding_strategy_error != null && Object.hasOwnProperty.call(m, "bidding_strategy_error"))
                                    w.uint32(16).int32(m.bidding_strategy_error);
                                if (m.url_field_error != null && Object.hasOwnProperty.call(m, "url_field_error"))
                                    w.uint32(24).int32(m.url_field_error);
                                if (m.list_operation_error != null && Object.hasOwnProperty.call(m, "list_operation_error"))
                                    w.uint32(32).int32(m.list_operation_error);
                                if (m.query_error != null && Object.hasOwnProperty.call(m, "query_error"))
                                    w.uint32(40).int32(m.query_error);
                                if (m.mutate_error != null && Object.hasOwnProperty.call(m, "mutate_error"))
                                    w.uint32(56).int32(m.mutate_error);
                                if (m.field_mask_error != null && Object.hasOwnProperty.call(m, "field_mask_error"))
                                    w.uint32(64).int32(m.field_mask_error);
                                if (m.authorization_error != null && Object.hasOwnProperty.call(m, "authorization_error"))
                                    w.uint32(72).int32(m.authorization_error);
                                if (m.internal_error != null && Object.hasOwnProperty.call(m, "internal_error"))
                                    w.uint32(80).int32(m.internal_error);
                                if (m.quota_error != null && Object.hasOwnProperty.call(m, "quota_error"))
                                    w.uint32(88).int32(m.quota_error);
                                if (m.ad_error != null && Object.hasOwnProperty.call(m, "ad_error"))
                                    w.uint32(96).int32(m.ad_error);
                                if (m.ad_group_error != null && Object.hasOwnProperty.call(m, "ad_group_error"))
                                    w.uint32(104).int32(m.ad_group_error);
                                if (m.campaign_budget_error != null && Object.hasOwnProperty.call(m, "campaign_budget_error"))
                                    w.uint32(112).int32(m.campaign_budget_error);
                                if (m.campaign_error != null && Object.hasOwnProperty.call(m, "campaign_error"))
                                    w.uint32(120).int32(m.campaign_error);
                                if (m.authentication_error != null && Object.hasOwnProperty.call(m, "authentication_error"))
                                    w.uint32(136).int32(m.authentication_error);
                                if (m.ad_group_criterion_error != null && Object.hasOwnProperty.call(m, "ad_group_criterion_error"))
                                    w.uint32(144).int32(m.ad_group_criterion_error);
                                if (m.ad_customizer_error != null && Object.hasOwnProperty.call(m, "ad_customizer_error"))
                                    w.uint32(152).int32(m.ad_customizer_error);
                                if (m.ad_group_ad_error != null && Object.hasOwnProperty.call(m, "ad_group_ad_error"))
                                    w.uint32(168).int32(m.ad_group_ad_error);
                                if (m.ad_sharing_error != null && Object.hasOwnProperty.call(m, "ad_sharing_error"))
                                    w.uint32(192).int32(m.ad_sharing_error);
                                if (m.adx_error != null && Object.hasOwnProperty.call(m, "adx_error"))
                                    w.uint32(200).int32(m.adx_error);
                                if (m.bidding_error != null && Object.hasOwnProperty.call(m, "bidding_error"))
                                    w.uint32(208).int32(m.bidding_error);
                                if (m.campaign_criterion_error != null && Object.hasOwnProperty.call(m, "campaign_criterion_error"))
                                    w.uint32(232).int32(m.campaign_criterion_error);
                                if (m.collection_size_error != null && Object.hasOwnProperty.call(m, "collection_size_error"))
                                    w.uint32(248).int32(m.collection_size_error);
                                if (m.criterion_error != null && Object.hasOwnProperty.call(m, "criterion_error"))
                                    w.uint32(256).int32(m.criterion_error);
                                if (m.date_error != null && Object.hasOwnProperty.call(m, "date_error"))
                                    w.uint32(264).int32(m.date_error);
                                if (m.date_range_error != null && Object.hasOwnProperty.call(m, "date_range_error"))
                                    w.uint32(272).int32(m.date_range_error);
                                if (m.distinct_error != null && Object.hasOwnProperty.call(m, "distinct_error"))
                                    w.uint32(280).int32(m.distinct_error);
                                if (m.feed_attribute_reference_error != null && Object.hasOwnProperty.call(m, "feed_attribute_reference_error"))
                                    w.uint32(288).int32(m.feed_attribute_reference_error);
                                if (m.function_error != null && Object.hasOwnProperty.call(m, "function_error"))
                                    w.uint32(296).int32(m.function_error);
                                if (m.function_parsing_error != null && Object.hasOwnProperty.call(m, "function_parsing_error"))
                                    w.uint32(304).int32(m.function_parsing_error);
                                if (m.id_error != null && Object.hasOwnProperty.call(m, "id_error"))
                                    w.uint32(312).int32(m.id_error);
                                if (m.image_error != null && Object.hasOwnProperty.call(m, "image_error"))
                                    w.uint32(320).int32(m.image_error);
                                if (m.media_bundle_error != null && Object.hasOwnProperty.call(m, "media_bundle_error"))
                                    w.uint32(336).int32(m.media_bundle_error);
                                if (m.multiplier_error != null && Object.hasOwnProperty.call(m, "multiplier_error"))
                                    w.uint32(352).int32(m.multiplier_error);
                                if (m.new_resource_creation_error != null && Object.hasOwnProperty.call(m, "new_resource_creation_error"))
                                    w.uint32(360).int32(m.new_resource_creation_error);
                                if (m.not_empty_error != null && Object.hasOwnProperty.call(m, "not_empty_error"))
                                    w.uint32(368).int32(m.not_empty_error);
                                if (m.null_error != null && Object.hasOwnProperty.call(m, "null_error"))
                                    w.uint32(376).int32(m.null_error);
                                if (m.operator_error != null && Object.hasOwnProperty.call(m, "operator_error"))
                                    w.uint32(384).int32(m.operator_error);
                                if (m.range_error != null && Object.hasOwnProperty.call(m, "range_error"))
                                    w.uint32(392).int32(m.range_error);
                                if (m.region_code_error != null && Object.hasOwnProperty.call(m, "region_code_error"))
                                    w.uint32(408).int32(m.region_code_error);
                                if (m.setting_error != null && Object.hasOwnProperty.call(m, "setting_error"))
                                    w.uint32(416).int32(m.setting_error);
                                if (m.string_format_error != null && Object.hasOwnProperty.call(m, "string_format_error"))
                                    w.uint32(424).int32(m.string_format_error);
                                if (m.string_length_error != null && Object.hasOwnProperty.call(m, "string_length_error"))
                                    w.uint32(432).int32(m.string_length_error);
                                if (m.operation_access_denied_error != null && Object.hasOwnProperty.call(m, "operation_access_denied_error"))
                                    w.uint32(440).int32(m.operation_access_denied_error);
                                if (m.resource_access_denied_error != null && Object.hasOwnProperty.call(m, "resource_access_denied_error"))
                                    w.uint32(448).int32(m.resource_access_denied_error);
                                if (m.resource_count_limit_exceeded_error != null && Object.hasOwnProperty.call(m, "resource_count_limit_exceeded_error"))
                                    w.uint32(456).int32(m.resource_count_limit_exceeded_error);
                                if (m.recommendation_error != null && Object.hasOwnProperty.call(m, "recommendation_error"))
                                    w.uint32(464).int32(m.recommendation_error);
                                if (m.ad_group_bid_modifier_error != null && Object.hasOwnProperty.call(m, "ad_group_bid_modifier_error"))
                                    w.uint32(472).int32(m.ad_group_bid_modifier_error);
                                if (m.context_error != null && Object.hasOwnProperty.call(m, "context_error"))
                                    w.uint32(480).int32(m.context_error);
                                if (m.field_error != null && Object.hasOwnProperty.call(m, "field_error"))
                                    w.uint32(488).int32(m.field_error);
                                if (m.shared_set_error != null && Object.hasOwnProperty.call(m, "shared_set_error"))
                                    w.uint32(496).int32(m.shared_set_error);
                                if (m.shared_criterion_error != null && Object.hasOwnProperty.call(m, "shared_criterion_error"))
                                    w.uint32(504).int32(m.shared_criterion_error);
                                if (m.campaign_shared_set_error != null && Object.hasOwnProperty.call(m, "campaign_shared_set_error"))
                                    w.uint32(512).int32(m.campaign_shared_set_error);
                                if (m.conversion_action_error != null && Object.hasOwnProperty.call(m, "conversion_action_error"))
                                    w.uint32(520).int32(m.conversion_action_error);
                                if (m.header_error != null && Object.hasOwnProperty.call(m, "header_error"))
                                    w.uint32(528).int32(m.header_error);
                                if (m.database_error != null && Object.hasOwnProperty.call(m, "database_error"))
                                    w.uint32(536).int32(m.database_error);
                                if (m.policy_finding_error != null && Object.hasOwnProperty.call(m, "policy_finding_error"))
                                    w.uint32(544).int32(m.policy_finding_error);
                                if (m.enum_error != null && Object.hasOwnProperty.call(m, "enum_error"))
                                    w.uint32(560).int32(m.enum_error);
                                if (m.keyword_plan_error != null && Object.hasOwnProperty.call(m, "keyword_plan_error"))
                                    w.uint32(568).int32(m.keyword_plan_error);
                                if (m.keyword_plan_campaign_error != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign_error"))
                                    w.uint32(576).int32(m.keyword_plan_campaign_error);
                                if (m.keyword_plan_ad_group_error != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_error"))
                                    w.uint32(592).int32(m.keyword_plan_ad_group_error);
                                if (m.keyword_plan_idea_error != null && Object.hasOwnProperty.call(m, "keyword_plan_idea_error"))
                                    w.uint32(608).int32(m.keyword_plan_idea_error);
                                if (m.account_budget_proposal_error != null && Object.hasOwnProperty.call(m, "account_budget_proposal_error"))
                                    w.uint32(616).int32(m.account_budget_proposal_error);
                                if (m.user_list_error != null && Object.hasOwnProperty.call(m, "user_list_error"))
                                    w.uint32(624).int32(m.user_list_error);
                                if (m.change_status_error != null && Object.hasOwnProperty.call(m, "change_status_error"))
                                    w.uint32(632).int32(m.change_status_error);
                                if (m.feed_error != null && Object.hasOwnProperty.call(m, "feed_error"))
                                    w.uint32(640).int32(m.feed_error);
                                if (m.geo_target_constant_suggestion_error != null && Object.hasOwnProperty.call(m, "geo_target_constant_suggestion_error"))
                                    w.uint32(648).int32(m.geo_target_constant_suggestion_error);
                                if (m.campaign_draft_error != null && Object.hasOwnProperty.call(m, "campaign_draft_error"))
                                    w.uint32(656).int32(m.campaign_draft_error);
                                if (m.feed_item_error != null && Object.hasOwnProperty.call(m, "feed_item_error"))
                                    w.uint32(664).int32(m.feed_item_error);
                                if (m.label_error != null && Object.hasOwnProperty.call(m, "label_error"))
                                    w.uint32(672).int32(m.label_error);
                                if (m.media_file_error != null && Object.hasOwnProperty.call(m, "media_file_error"))
                                    w.uint32(688).int32(m.media_file_error);
                                if (m.billing_setup_error != null && Object.hasOwnProperty.call(m, "billing_setup_error"))
                                    w.uint32(696).int32(m.billing_setup_error);
                                if (m.customer_client_link_error != null && Object.hasOwnProperty.call(m, "customer_client_link_error"))
                                    w.uint32(704).int32(m.customer_client_link_error);
                                if (m.customer_error != null && Object.hasOwnProperty.call(m, "customer_error"))
                                    w.uint32(720).int32(m.customer_error);
                                if (m.customer_manager_link_error != null && Object.hasOwnProperty.call(m, "customer_manager_link_error"))
                                    w.uint32(728).int32(m.customer_manager_link_error);
                                if (m.feed_mapping_error != null && Object.hasOwnProperty.call(m, "feed_mapping_error"))
                                    w.uint32(736).int32(m.feed_mapping_error);
                                if (m.customer_feed_error != null && Object.hasOwnProperty.call(m, "customer_feed_error"))
                                    w.uint32(744).int32(m.customer_feed_error);
                                if (m.ad_group_feed_error != null && Object.hasOwnProperty.call(m, "ad_group_feed_error"))
                                    w.uint32(752).int32(m.ad_group_feed_error);
                                if (m.campaign_feed_error != null && Object.hasOwnProperty.call(m, "campaign_feed_error"))
                                    w.uint32(768).int32(m.campaign_feed_error);
                                if (m.custom_interest_error != null && Object.hasOwnProperty.call(m, "custom_interest_error"))
                                    w.uint32(776).int32(m.custom_interest_error);
                                if (m.campaign_experiment_error != null && Object.hasOwnProperty.call(m, "campaign_experiment_error"))
                                    w.uint32(784).int32(m.campaign_experiment_error);
                                if (m.extension_feed_item_error != null && Object.hasOwnProperty.call(m, "extension_feed_item_error"))
                                    w.uint32(800).int32(m.extension_feed_item_error);
                                if (m.ad_parameter_error != null && Object.hasOwnProperty.call(m, "ad_parameter_error"))
                                    w.uint32(808).int32(m.ad_parameter_error);
                                if (m.feed_item_validation_error != null && Object.hasOwnProperty.call(m, "feed_item_validation_error"))
                                    w.uint32(816).int32(m.feed_item_validation_error);
                                if (m.extension_setting_error != null && Object.hasOwnProperty.call(m, "extension_setting_error"))
                                    w.uint32(824).int32(m.extension_setting_error);
                                if (m.feed_item_target_error != null && Object.hasOwnProperty.call(m, "feed_item_target_error"))
                                    w.uint32(832).int32(m.feed_item_target_error);
                                if (m.policy_violation_error != null && Object.hasOwnProperty.call(m, "policy_violation_error"))
                                    w.uint32(840).int32(m.policy_violation_error);
                                if (m.asset_error != null && Object.hasOwnProperty.call(m, "asset_error"))
                                    w.uint32(856).int32(m.asset_error);
                                if (m.country_code_error != null && Object.hasOwnProperty.call(m, "country_code_error"))
                                    w.uint32(872).int32(m.country_code_error);
                                if (m.language_code_error != null && Object.hasOwnProperty.call(m, "language_code_error"))
                                    w.uint32(880).int32(m.language_code_error);
                                if (m.conversion_upload_error != null && Object.hasOwnProperty.call(m, "conversion_upload_error"))
                                    w.uint32(888).int32(m.conversion_upload_error);
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    w.uint32(896).int32(m.partial_failure_error);
                                if (m.policy_validation_parameter_error != null && Object.hasOwnProperty.call(m, "policy_validation_parameter_error"))
                                    w.uint32(912).int32(m.policy_validation_parameter_error);
                                if (m.conversion_adjustment_upload_error != null && Object.hasOwnProperty.call(m, "conversion_adjustment_upload_error"))
                                    w.uint32(920).int32(m.conversion_adjustment_upload_error);
                                if (m.media_upload_error != null && Object.hasOwnProperty.call(m, "media_upload_error"))
                                    w.uint32(928).int32(m.media_upload_error);
                                if (m.youtube_video_registration_error != null && Object.hasOwnProperty.call(m, "youtube_video_registration_error"))
                                    w.uint32(936).int32(m.youtube_video_registration_error);
                                if (m.size_limit_error != null && Object.hasOwnProperty.call(m, "size_limit_error"))
                                    w.uint32(944).int32(m.size_limit_error);
                                if (m.offline_user_data_job_error != null && Object.hasOwnProperty.call(m, "offline_user_data_job_error"))
                                    w.uint32(952).int32(m.offline_user_data_job_error);
                                if (m.manager_link_error != null && Object.hasOwnProperty.call(m, "manager_link_error"))
                                    w.uint32(968).int32(m.manager_link_error);
                                if (m.currency_code_error != null && Object.hasOwnProperty.call(m, "currency_code_error"))
                                    w.uint32(976).int32(m.currency_code_error);
                                if (m.experiment_error != null && Object.hasOwnProperty.call(m, "experiment_error"))
                                    w.uint32(984).int32(m.experiment_error);
                                if (m.access_invitation_error != null && Object.hasOwnProperty.call(m, "access_invitation_error"))
                                    w.uint32(992).int32(m.access_invitation_error);
                                if (m.reach_plan_error != null && Object.hasOwnProperty.call(m, "reach_plan_error"))
                                    w.uint32(1000).int32(m.reach_plan_error);
                                if (m.invoice_error != null && Object.hasOwnProperty.call(m, "invoice_error"))
                                    w.uint32(1008).int32(m.invoice_error);
                                if (m.payments_account_error != null && Object.hasOwnProperty.call(m, "payments_account_error"))
                                    w.uint32(1016).int32(m.payments_account_error);
                                if (m.time_zone_error != null && Object.hasOwnProperty.call(m, "time_zone_error"))
                                    w.uint32(1024).int32(m.time_zone_error);
                                if (m.asset_link_error != null && Object.hasOwnProperty.call(m, "asset_link_error"))
                                    w.uint32(1032).int32(m.asset_link_error);
                                if (m.user_data_error != null && Object.hasOwnProperty.call(m, "user_data_error"))
                                    w.uint32(1040).int32(m.user_data_error);
                                if (m.batch_job_error != null && Object.hasOwnProperty.call(m, "batch_job_error"))
                                    w.uint32(1048).int32(m.batch_job_error);
                                if (m.keyword_plan_campaign_keyword_error != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign_keyword_error"))
                                    w.uint32(1056).int32(m.keyword_plan_campaign_keyword_error);
                                if (m.keyword_plan_ad_group_keyword_error != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_keyword_error"))
                                    w.uint32(1064).int32(m.keyword_plan_ad_group_keyword_error);
                                if (m.account_link_error != null && Object.hasOwnProperty.call(m, "account_link_error"))
                                    w.uint32(1072).int32(m.account_link_error);
                                if (m.third_party_app_analytics_link_error != null && Object.hasOwnProperty.call(m, "third_party_app_analytics_link_error"))
                                    w.uint32(1080).int32(m.third_party_app_analytics_link_error);
                                if (m.change_event_error != null && Object.hasOwnProperty.call(m, "change_event_error"))
                                    w.uint32(1088).int32(m.change_event_error);
                                if (m.not_allowlisted_error != null && Object.hasOwnProperty.call(m, "not_allowlisted_error"))
                                    w.uint32(1096).int32(m.not_allowlisted_error);
                                if (m.customer_user_access_error != null && Object.hasOwnProperty.call(m, "customer_user_access_error"))
                                    w.uint32(1104).int32(m.customer_user_access_error);
                                if (m.custom_audience_error != null && Object.hasOwnProperty.call(m, "custom_audience_error"))
                                    w.uint32(1112).int32(m.custom_audience_error);
                                if (m.feed_item_set_error != null && Object.hasOwnProperty.call(m, "feed_item_set_error"))
                                    w.uint32(1120).int32(m.feed_item_set_error);
                                if (m.feed_item_set_link_error != null && Object.hasOwnProperty.call(m, "feed_item_set_link_error"))
                                    w.uint32(1128).int32(m.feed_item_set_link_error);
                                if (m.conversion_custom_variable_error != null && Object.hasOwnProperty.call(m, "conversion_custom_variable_error"))
                                    w.uint32(1144).int32(m.conversion_custom_variable_error);
                                if (m.conversion_value_rule_error != null && Object.hasOwnProperty.call(m, "conversion_value_rule_error"))
                                    w.uint32(1160).int32(m.conversion_value_rule_error);
                                if (m.conversion_value_rule_set_error != null && Object.hasOwnProperty.call(m, "conversion_value_rule_set_error"))
                                    w.uint32(1168).int32(m.conversion_value_rule_set_error);
                                if (m.asset_group_error != null && Object.hasOwnProperty.call(m, "asset_group_error"))
                                    w.uint32(1184).int32(m.asset_group_error);
                                if (m.asset_group_asset_error != null && Object.hasOwnProperty.call(m, "asset_group_asset_error"))
                                    w.uint32(1192).int32(m.asset_group_asset_error);
                                if (m.custom_conversion_goal_error != null && Object.hasOwnProperty.call(m, "custom_conversion_goal_error"))
                                    w.uint32(1200).int32(m.custom_conversion_goal_error);
                                if (m.customizer_attribute_error != null && Object.hasOwnProperty.call(m, "customizer_attribute_error"))
                                    w.uint32(1208).int32(m.customizer_attribute_error);
                                if (m.asset_set_error != null && Object.hasOwnProperty.call(m, "asset_set_error"))
                                    w.uint32(1216).int32(m.asset_set_error);
                                if (m.asset_set_asset_error != null && Object.hasOwnProperty.call(m, "asset_set_asset_error"))
                                    w.uint32(1224).int32(m.asset_set_asset_error);
                                if (m.asset_set_link_error != null && Object.hasOwnProperty.call(m, "asset_set_link_error"))
                                    w.uint32(1232).int32(m.asset_set_link_error);
                                if (m.asset_group_listing_group_filter_error != null && Object.hasOwnProperty.call(m, "asset_group_listing_group_filter_error"))
                                    w.uint32(1240).int32(m.asset_group_listing_group_filter_error);
                                if (m.experiment_arm_error != null && Object.hasOwnProperty.call(m, "experiment_arm_error"))
                                    w.uint32(1248).int32(m.experiment_arm_error);
                                if (m.customer_customizer_error != null && Object.hasOwnProperty.call(m, "customer_customizer_error"))
                                    w.uint32(1264).int32(m.customer_customizer_error);
                                if (m.ad_group_customizer_error != null && Object.hasOwnProperty.call(m, "ad_group_customizer_error"))
                                    w.uint32(1272).int32(m.ad_group_customizer_error);
                                if (m.campaign_customizer_error != null && Object.hasOwnProperty.call(m, "campaign_customizer_error"))
                                    w.uint32(1280).int32(m.campaign_customizer_error);
                                if (m.ad_group_criterion_customizer_error != null && Object.hasOwnProperty.call(m, "ad_group_criterion_customizer_error"))
                                    w.uint32(1288).int32(m.ad_group_criterion_customizer_error);
                                if (m.merchant_center_error != null && Object.hasOwnProperty.call(m, "merchant_center_error"))
                                    w.uint32(1296).int32(m.merchant_center_error);
                                if (m.audience_error != null && Object.hasOwnProperty.call(m, "audience_error"))
                                    w.uint32(1312).int32(m.audience_error);
                                if (m.conversion_goal_campaign_config_error != null && Object.hasOwnProperty.call(m, "conversion_goal_campaign_config_error"))
                                    w.uint32(1320).int32(m.conversion_goal_campaign_config_error);
                                if (m.campaign_conversion_goal_error != null && Object.hasOwnProperty.call(m, "campaign_conversion_goal_error"))
                                    w.uint32(1328).int32(m.campaign_conversion_goal_error);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ErrorCode message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorCode.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorCode} message ErrorCode message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ErrorCode.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ErrorCode message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ErrorCode} ErrorCode
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ErrorCode.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ErrorCode();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.request_error = r.int32();
                                        break;
                                    case 2:
                                        m.bidding_strategy_error = r.int32();
                                        break;
                                    case 3:
                                        m.url_field_error = r.int32();
                                        break;
                                    case 4:
                                        m.list_operation_error = r.int32();
                                        break;
                                    case 5:
                                        m.query_error = r.int32();
                                        break;
                                    case 7:
                                        m.mutate_error = r.int32();
                                        break;
                                    case 8:
                                        m.field_mask_error = r.int32();
                                        break;
                                    case 9:
                                        m.authorization_error = r.int32();
                                        break;
                                    case 10:
                                        m.internal_error = r.int32();
                                        break;
                                    case 11:
                                        m.quota_error = r.int32();
                                        break;
                                    case 12:
                                        m.ad_error = r.int32();
                                        break;
                                    case 13:
                                        m.ad_group_error = r.int32();
                                        break;
                                    case 14:
                                        m.campaign_budget_error = r.int32();
                                        break;
                                    case 15:
                                        m.campaign_error = r.int32();
                                        break;
                                    case 17:
                                        m.authentication_error = r.int32();
                                        break;
                                    case 161:
                                        m.ad_group_criterion_customizer_error = r.int32();
                                        break;
                                    case 18:
                                        m.ad_group_criterion_error = r.int32();
                                        break;
                                    case 159:
                                        m.ad_group_customizer_error = r.int32();
                                        break;
                                    case 19:
                                        m.ad_customizer_error = r.int32();
                                        break;
                                    case 21:
                                        m.ad_group_ad_error = r.int32();
                                        break;
                                    case 24:
                                        m.ad_sharing_error = r.int32();
                                        break;
                                    case 25:
                                        m.adx_error = r.int32();
                                        break;
                                    case 107:
                                        m.asset_error = r.int32();
                                        break;
                                    case 149:
                                        m.asset_group_asset_error = r.int32();
                                        break;
                                    case 155:
                                        m.asset_group_listing_group_filter_error = r.int32();
                                        break;
                                    case 148:
                                        m.asset_group_error = r.int32();
                                        break;
                                    case 153:
                                        m.asset_set_asset_error = r.int32();
                                        break;
                                    case 154:
                                        m.asset_set_link_error = r.int32();
                                        break;
                                    case 152:
                                        m.asset_set_error = r.int32();
                                        break;
                                    case 26:
                                        m.bidding_error = r.int32();
                                        break;
                                    case 29:
                                        m.campaign_criterion_error = r.int32();
                                        break;
                                    case 166:
                                        m.campaign_conversion_goal_error = r.int32();
                                        break;
                                    case 160:
                                        m.campaign_customizer_error = r.int32();
                                        break;
                                    case 31:
                                        m.collection_size_error = r.int32();
                                        break;
                                    case 165:
                                        m.conversion_goal_campaign_config_error = r.int32();
                                        break;
                                    case 109:
                                        m.country_code_error = r.int32();
                                        break;
                                    case 32:
                                        m.criterion_error = r.int32();
                                        break;
                                    case 150:
                                        m.custom_conversion_goal_error = r.int32();
                                        break;
                                    case 158:
                                        m.customer_customizer_error = r.int32();
                                        break;
                                    case 90:
                                        m.customer_error = r.int32();
                                        break;
                                    case 151:
                                        m.customizer_attribute_error = r.int32();
                                        break;
                                    case 33:
                                        m.date_error = r.int32();
                                        break;
                                    case 34:
                                        m.date_range_error = r.int32();
                                        break;
                                    case 35:
                                        m.distinct_error = r.int32();
                                        break;
                                    case 36:
                                        m.feed_attribute_reference_error = r.int32();
                                        break;
                                    case 37:
                                        m.function_error = r.int32();
                                        break;
                                    case 38:
                                        m.function_parsing_error = r.int32();
                                        break;
                                    case 39:
                                        m.id_error = r.int32();
                                        break;
                                    case 40:
                                        m.image_error = r.int32();
                                        break;
                                    case 110:
                                        m.language_code_error = r.int32();
                                        break;
                                    case 42:
                                        m.media_bundle_error = r.int32();
                                        break;
                                    case 116:
                                        m.media_upload_error = r.int32();
                                        break;
                                    case 86:
                                        m.media_file_error = r.int32();
                                        break;
                                    case 162:
                                        m.merchant_center_error = r.int32();
                                        break;
                                    case 44:
                                        m.multiplier_error = r.int32();
                                        break;
                                    case 45:
                                        m.new_resource_creation_error = r.int32();
                                        break;
                                    case 46:
                                        m.not_empty_error = r.int32();
                                        break;
                                    case 47:
                                        m.null_error = r.int32();
                                        break;
                                    case 48:
                                        m.operator_error = r.int32();
                                        break;
                                    case 49:
                                        m.range_error = r.int32();
                                        break;
                                    case 58:
                                        m.recommendation_error = r.int32();
                                        break;
                                    case 51:
                                        m.region_code_error = r.int32();
                                        break;
                                    case 52:
                                        m.setting_error = r.int32();
                                        break;
                                    case 53:
                                        m.string_format_error = r.int32();
                                        break;
                                    case 54:
                                        m.string_length_error = r.int32();
                                        break;
                                    case 55:
                                        m.operation_access_denied_error = r.int32();
                                        break;
                                    case 56:
                                        m.resource_access_denied_error = r.int32();
                                        break;
                                    case 57:
                                        m.resource_count_limit_exceeded_error = r.int32();
                                        break;
                                    case 117:
                                        m.youtube_video_registration_error = r.int32();
                                        break;
                                    case 59:
                                        m.ad_group_bid_modifier_error = r.int32();
                                        break;
                                    case 60:
                                        m.context_error = r.int32();
                                        break;
                                    case 61:
                                        m.field_error = r.int32();
                                        break;
                                    case 62:
                                        m.shared_set_error = r.int32();
                                        break;
                                    case 63:
                                        m.shared_criterion_error = r.int32();
                                        break;
                                    case 64:
                                        m.campaign_shared_set_error = r.int32();
                                        break;
                                    case 65:
                                        m.conversion_action_error = r.int32();
                                        break;
                                    case 115:
                                        m.conversion_adjustment_upload_error = r.int32();
                                        break;
                                    case 143:
                                        m.conversion_custom_variable_error = r.int32();
                                        break;
                                    case 111:
                                        m.conversion_upload_error = r.int32();
                                        break;
                                    case 145:
                                        m.conversion_value_rule_error = r.int32();
                                        break;
                                    case 146:
                                        m.conversion_value_rule_set_error = r.int32();
                                        break;
                                    case 66:
                                        m.header_error = r.int32();
                                        break;
                                    case 67:
                                        m.database_error = r.int32();
                                        break;
                                    case 68:
                                        m.policy_finding_error = r.int32();
                                        break;
                                    case 70:
                                        m.enum_error = r.int32();
                                        break;
                                    case 71:
                                        m.keyword_plan_error = r.int32();
                                        break;
                                    case 72:
                                        m.keyword_plan_campaign_error = r.int32();
                                        break;
                                    case 132:
                                        m.keyword_plan_campaign_keyword_error = r.int32();
                                        break;
                                    case 74:
                                        m.keyword_plan_ad_group_error = r.int32();
                                        break;
                                    case 133:
                                        m.keyword_plan_ad_group_keyword_error = r.int32();
                                        break;
                                    case 76:
                                        m.keyword_plan_idea_error = r.int32();
                                        break;
                                    case 77:
                                        m.account_budget_proposal_error = r.int32();
                                        break;
                                    case 78:
                                        m.user_list_error = r.int32();
                                        break;
                                    case 136:
                                        m.change_event_error = r.int32();
                                        break;
                                    case 79:
                                        m.change_status_error = r.int32();
                                        break;
                                    case 80:
                                        m.feed_error = r.int32();
                                        break;
                                    case 81:
                                        m.geo_target_constant_suggestion_error = r.int32();
                                        break;
                                    case 82:
                                        m.campaign_draft_error = r.int32();
                                        break;
                                    case 83:
                                        m.feed_item_error = r.int32();
                                        break;
                                    case 84:
                                        m.label_error = r.int32();
                                        break;
                                    case 87:
                                        m.billing_setup_error = r.int32();
                                        break;
                                    case 88:
                                        m.customer_client_link_error = r.int32();
                                        break;
                                    case 91:
                                        m.customer_manager_link_error = r.int32();
                                        break;
                                    case 92:
                                        m.feed_mapping_error = r.int32();
                                        break;
                                    case 93:
                                        m.customer_feed_error = r.int32();
                                        break;
                                    case 94:
                                        m.ad_group_feed_error = r.int32();
                                        break;
                                    case 96:
                                        m.campaign_feed_error = r.int32();
                                        break;
                                    case 97:
                                        m.custom_interest_error = r.int32();
                                        break;
                                    case 98:
                                        m.campaign_experiment_error = r.int32();
                                        break;
                                    case 100:
                                        m.extension_feed_item_error = r.int32();
                                        break;
                                    case 101:
                                        m.ad_parameter_error = r.int32();
                                        break;
                                    case 102:
                                        m.feed_item_validation_error = r.int32();
                                        break;
                                    case 103:
                                        m.extension_setting_error = r.int32();
                                        break;
                                    case 140:
                                        m.feed_item_set_error = r.int32();
                                        break;
                                    case 141:
                                        m.feed_item_set_link_error = r.int32();
                                        break;
                                    case 104:
                                        m.feed_item_target_error = r.int32();
                                        break;
                                    case 105:
                                        m.policy_violation_error = r.int32();
                                        break;
                                    case 112:
                                        m.partial_failure_error = r.int32();
                                        break;
                                    case 114:
                                        m.policy_validation_parameter_error = r.int32();
                                        break;
                                    case 118:
                                        m.size_limit_error = r.int32();
                                        break;
                                    case 119:
                                        m.offline_user_data_job_error = r.int32();
                                        break;
                                    case 137:
                                        m.not_allowlisted_error = r.int32();
                                        break;
                                    case 121:
                                        m.manager_link_error = r.int32();
                                        break;
                                    case 122:
                                        m.currency_code_error = r.int32();
                                        break;
                                    case 123:
                                        m.experiment_error = r.int32();
                                        break;
                                    case 124:
                                        m.access_invitation_error = r.int32();
                                        break;
                                    case 125:
                                        m.reach_plan_error = r.int32();
                                        break;
                                    case 126:
                                        m.invoice_error = r.int32();
                                        break;
                                    case 127:
                                        m.payments_account_error = r.int32();
                                        break;
                                    case 128:
                                        m.time_zone_error = r.int32();
                                        break;
                                    case 129:
                                        m.asset_link_error = r.int32();
                                        break;
                                    case 130:
                                        m.user_data_error = r.int32();
                                        break;
                                    case 131:
                                        m.batch_job_error = r.int32();
                                        break;
                                    case 134:
                                        m.account_link_error = r.int32();
                                        break;
                                    case 135:
                                        m.third_party_app_analytics_link_error = r.int32();
                                        break;
                                    case 138:
                                        m.customer_user_access_error = r.int32();
                                        break;
                                    case 139:
                                        m.custom_audience_error = r.int32();
                                        break;
                                    case 164:
                                        m.audience_error = r.int32();
                                        break;
                                    case 156:
                                        m.experiment_arm_error = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ErrorCode message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ErrorCode} ErrorCode
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ErrorCode.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ErrorCode message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ErrorCode.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.request_error != null && m.hasOwnProperty("request_error")) {
                                    p.error_code = 1;
                                    switch (m.request_error) {
                                    default:
                                        return "request_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 17:
                                    case 16:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 22:
                                    case 9:
                                    case 11:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                        break;
                                    }
                                }
                                if (m.bidding_strategy_error != null && m.hasOwnProperty("bidding_strategy_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.bidding_strategy_error) {
                                    default:
                                        return "bidding_strategy_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.url_field_error != null && m.hasOwnProperty("url_field_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.url_field_error) {
                                    default:
                                        return "url_field_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 52:
                                    case 44:
                                    case 45:
                                    case 46:
                                    case 47:
                                    case 48:
                                    case 49:
                                    case 50:
                                    case 51:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                    case 58:
                                        break;
                                    }
                                }
                                if (m.list_operation_error != null && m.hasOwnProperty("list_operation_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.list_operation_error) {
                                    default:
                                        return "list_operation_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.query_error != null && m.hasOwnProperty("query_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.query_error) {
                                    default:
                                        return "query_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 50:
                                    case 18:
                                    case 7:
                                    case 12:
                                    case 15:
                                    case 5:
                                    case 3:
                                    case 61:
                                    case 62:
                                    case 45:
                                    case 2:
                                    case 4:
                                    case 36:
                                    case 60:
                                    case 30:
                                    case 14:
                                    case 37:
                                    case 55:
                                    case 44:
                                    case 41:
                                    case 16:
                                    case 13:
                                    case 42:
                                    case 29:
                                    case 38:
                                    case 64:
                                    case 57:
                                    case 26:
                                    case 22:
                                    case 56:
                                    case 35:
                                    case 28:
                                    case 54:
                                    case 31:
                                    case 40:
                                    case 23:
                                    case 24:
                                    case 43:
                                    case 48:
                                    case 58:
                                    case 49:
                                    case 51:
                                    case 53:
                                    case 25:
                                    case 8:
                                    case 10:
                                    case 21:
                                    case 6:
                                    case 34:
                                    case 9:
                                    case 47:
                                    case 32:
                                    case 11:
                                    case 59:
                                    case 63:
                                        break;
                                    }
                                }
                                if (m.mutate_error != null && m.hasOwnProperty("mutate_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.mutate_error) {
                                    default:
                                        return "mutate_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 16:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.field_mask_error != null && m.hasOwnProperty("field_mask_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.field_mask_error) {
                                    default:
                                        return "field_mask_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                    case 4:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.authorization_error != null && m.hasOwnProperty("authorization_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.authorization_error) {
                                    default:
                                        return "authorization_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 13:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 24:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 25:
                                    case 26:
                                        break;
                                    }
                                }
                                if (m.internal_error != null && m.hasOwnProperty("internal_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.internal_error) {
                                    default:
                                        return "internal_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.quota_error != null && m.hasOwnProperty("quota_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.quota_error) {
                                    default:
                                        return "quota_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.ad_error != null && m.hasOwnProperty("ad_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_error) {
                                    default:
                                        return "ad_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 47:
                                    case 48:
                                    case 49:
                                    case 50:
                                    case 51:
                                    case 52:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                    case 58:
                                    case 59:
                                    case 60:
                                    case 61:
                                    case 62:
                                    case 63:
                                    case 64:
                                    case 65:
                                    case 66:
                                    case 67:
                                    case 68:
                                    case 69:
                                    case 70:
                                    case 71:
                                    case 72:
                                    case 73:
                                    case 74:
                                    case 75:
                                    case 76:
                                    case 77:
                                    case 78:
                                    case 79:
                                    case 80:
                                    case 81:
                                    case 82:
                                    case 83:
                                    case 84:
                                    case 85:
                                    case 86:
                                    case 87:
                                    case 88:
                                    case 89:
                                    case 90:
                                    case 91:
                                    case 92:
                                    case 93:
                                    case 94:
                                    case 95:
                                    case 96:
                                    case 97:
                                    case 98:
                                    case 99:
                                    case 100:
                                    case 102:
                                    case 103:
                                    case 104:
                                    case 105:
                                    case 106:
                                    case 107:
                                    case 108:
                                    case 109:
                                    case 110:
                                    case 111:
                                    case 112:
                                    case 113:
                                    case 114:
                                    case 115:
                                    case 116:
                                    case 117:
                                    case 118:
                                    case 119:
                                    case 120:
                                    case 121:
                                    case 122:
                                    case 123:
                                    case 124:
                                    case 125:
                                    case 126:
                                    case 127:
                                    case 128:
                                    case 129:
                                    case 130:
                                    case 131:
                                    case 132:
                                    case 133:
                                    case 134:
                                    case 135:
                                    case 136:
                                    case 137:
                                    case 138:
                                    case 139:
                                    case 140:
                                    case 154:
                                    case 141:
                                    case 142:
                                    case 143:
                                    case 144:
                                    case 145:
                                    case 146:
                                    case 147:
                                    case 148:
                                    case 149:
                                    case 150:
                                    case 151:
                                    case 152:
                                    case 153:
                                    case 155:
                                    case 156:
                                        break;
                                    }
                                }
                                if (m.ad_group_error != null && m.hasOwnProperty("ad_group_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_group_error) {
                                    default:
                                        return "ad_group_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                        break;
                                    }
                                }
                                if (m.campaign_budget_error != null && m.hasOwnProperty("campaign_budget_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_budget_error) {
                                    default:
                                        return "campaign_budget_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 17:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 18:
                                    case 19:
                                        break;
                                    }
                                }
                                if (m.campaign_error != null && m.hasOwnProperty("campaign_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_error) {
                                    default:
                                        return "campaign_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 46:
                                    case 49:
                                    case 48:
                                    case 50:
                                    case 51:
                                    case 52:
                                    case 53:
                                    case 54:
                                    case 55:
                                        break;
                                    }
                                }
                                if (m.authentication_error != null && m.hasOwnProperty("authentication_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.authentication_error) {
                                    default:
                                        return "authentication_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 5:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 25:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 22:
                                    case 23:
                                    case 24:
                                        break;
                                    }
                                }
                                if (m.ad_group_criterion_customizer_error != null && m.hasOwnProperty("ad_group_criterion_customizer_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_group_criterion_customizer_error) {
                                    default:
                                        return "ad_group_criterion_customizer_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.ad_group_criterion_error != null && m.hasOwnProperty("ad_group_criterion_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_group_criterion_error) {
                                    default:
                                        return "ad_group_criterion_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                        break;
                                    }
                                }
                                if (m.ad_group_customizer_error != null && m.hasOwnProperty("ad_group_customizer_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_group_customizer_error) {
                                    default:
                                        return "ad_group_customizer_error: enum value expected";
                                    case 0:
                                    case 1:
                                        break;
                                    }
                                }
                                if (m.ad_customizer_error != null && m.hasOwnProperty("ad_customizer_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_customizer_error) {
                                    default:
                                        return "ad_customizer_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.ad_group_ad_error != null && m.hasOwnProperty("ad_group_ad_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_group_ad_error) {
                                    default:
                                        return "ad_group_ad_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.ad_sharing_error != null && m.hasOwnProperty("ad_sharing_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_sharing_error) {
                                    default:
                                        return "ad_sharing_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.adx_error != null && m.hasOwnProperty("adx_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.adx_error) {
                                    default:
                                        return "adx_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.asset_error != null && m.hasOwnProperty("asset_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_error) {
                                    default:
                                        return "asset_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 13:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                        break;
                                    }
                                }
                                if (m.asset_group_asset_error != null && m.hasOwnProperty("asset_group_asset_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_group_asset_error) {
                                    default:
                                        return "asset_group_asset_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.asset_group_listing_group_filter_error != null && m.hasOwnProperty("asset_group_listing_group_filter_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_group_listing_group_filter_error) {
                                    default:
                                        return "asset_group_listing_group_filter_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                        break;
                                    }
                                }
                                if (m.asset_group_error != null && m.hasOwnProperty("asset_group_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_group_error) {
                                    default:
                                        return "asset_group_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                if (m.asset_set_asset_error != null && m.hasOwnProperty("asset_set_asset_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_set_asset_error) {
                                    default:
                                        return "asset_set_asset_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.asset_set_link_error != null && m.hasOwnProperty("asset_set_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_set_link_error) {
                                    default:
                                        return "asset_set_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.asset_set_error != null && m.hasOwnProperty("asset_set_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_set_error) {
                                    default:
                                        return "asset_set_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.bidding_error != null && m.hasOwnProperty("bidding_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.bidding_error) {
                                    default:
                                        return "bidding_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 7:
                                    case 10:
                                    case 14:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                        break;
                                    }
                                }
                                if (m.campaign_criterion_error != null && m.hasOwnProperty("campaign_criterion_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_criterion_error) {
                                    default:
                                        return "campaign_criterion_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                        break;
                                    }
                                }
                                if (m.campaign_conversion_goal_error != null && m.hasOwnProperty("campaign_conversion_goal_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_conversion_goal_error) {
                                    default:
                                        return "campaign_conversion_goal_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.campaign_customizer_error != null && m.hasOwnProperty("campaign_customizer_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_customizer_error) {
                                    default:
                                        return "campaign_customizer_error: enum value expected";
                                    case 0:
                                    case 1:
                                        break;
                                    }
                                }
                                if (m.collection_size_error != null && m.hasOwnProperty("collection_size_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.collection_size_error) {
                                    default:
                                        return "collection_size_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.conversion_goal_campaign_config_error != null && m.hasOwnProperty("conversion_goal_campaign_config_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.conversion_goal_campaign_config_error) {
                                    default:
                                        return "conversion_goal_campaign_config_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.country_code_error != null && m.hasOwnProperty("country_code_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.country_code_error) {
                                    default:
                                        return "country_code_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.criterion_error != null && m.hasOwnProperty("criterion_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.criterion_error) {
                                    default:
                                        return "criterion_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 122:
                                    case 96:
                                    case 97:
                                    case 121:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 46:
                                    case 47:
                                    case 48:
                                    case 49:
                                    case 50:
                                    case 51:
                                    case 52:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                    case 58:
                                    case 59:
                                    case 60:
                                    case 61:
                                    case 62:
                                    case 63:
                                    case 64:
                                    case 65:
                                    case 66:
                                    case 67:
                                    case 68:
                                    case 69:
                                    case 76:
                                    case 77:
                                    case 78:
                                    case 79:
                                    case 98:
                                    case 80:
                                    case 123:
                                    case 124:
                                    case 125:
                                    case 81:
                                    case 82:
                                    case 83:
                                    case 84:
                                    case 85:
                                    case 86:
                                    case 87:
                                    case 88:
                                    case 89:
                                    case 90:
                                    case 91:
                                    case 92:
                                    case 93:
                                    case 94:
                                    case 95:
                                    case 99:
                                    case 126:
                                    case 100:
                                    case 101:
                                    case 102:
                                    case 103:
                                    case 104:
                                    case 105:
                                    case 106:
                                    case 107:
                                    case 108:
                                    case 109:
                                    case 110:
                                    case 111:
                                    case 112:
                                    case 113:
                                    case 114:
                                    case 115:
                                    case 116:
                                    case 117:
                                    case 118:
                                    case 119:
                                    case 120:
                                    case 127:
                                    case 128:
                                    case 129:
                                    case 130:
                                    case 131:
                                    case 132:
                                    case 133:
                                    case 134:
                                    case 135:
                                    case 136:
                                    case 137:
                                    case 138:
                                    case 139:
                                        break;
                                    }
                                }
                                if (m.custom_conversion_goal_error != null && m.hasOwnProperty("custom_conversion_goal_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.custom_conversion_goal_error) {
                                    default:
                                        return "custom_conversion_goal_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.customer_customizer_error != null && m.hasOwnProperty("customer_customizer_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.customer_customizer_error) {
                                    default:
                                        return "customer_customizer_error: enum value expected";
                                    case 0:
                                    case 1:
                                        break;
                                    }
                                }
                                if (m.customer_error != null && m.hasOwnProperty("customer_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.customer_error) {
                                    default:
                                        return "customer_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.customizer_attribute_error != null && m.hasOwnProperty("customizer_attribute_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.customizer_attribute_error) {
                                    default:
                                        return "customizer_attribute_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.date_error != null && m.hasOwnProperty("date_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.date_error) {
                                    default:
                                        return "date_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 11:
                                    case 12:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.date_range_error != null && m.hasOwnProperty("date_range_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.date_range_error) {
                                    default:
                                        return "date_range_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.distinct_error != null && m.hasOwnProperty("distinct_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.distinct_error) {
                                    default:
                                        return "distinct_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.feed_attribute_reference_error != null && m.hasOwnProperty("feed_attribute_reference_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_attribute_reference_error) {
                                    default:
                                        return "feed_attribute_reference_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.function_error != null && m.hasOwnProperty("function_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.function_error) {
                                    default:
                                        return "function_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                        break;
                                    }
                                }
                                if (m.function_parsing_error != null && m.hasOwnProperty("function_parsing_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.function_parsing_error) {
                                    default:
                                        return "function_parsing_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.id_error != null && m.hasOwnProperty("id_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.id_error) {
                                    default:
                                        return "id_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.image_error != null && m.hasOwnProperty("image_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.image_error) {
                                    default:
                                        return "image_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                        break;
                                    }
                                }
                                if (m.language_code_error != null && m.hasOwnProperty("language_code_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.language_code_error) {
                                    default:
                                        return "language_code_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.media_bundle_error != null && m.hasOwnProperty("media_bundle_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.media_bundle_error) {
                                    default:
                                        return "media_bundle_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                        break;
                                    }
                                }
                                if (m.media_upload_error != null && m.hasOwnProperty("media_upload_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.media_upload_error) {
                                    default:
                                        return "media_upload_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                        break;
                                    }
                                }
                                if (m.media_file_error != null && m.hasOwnProperty("media_file_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.media_file_error) {
                                    default:
                                        return "media_file_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                        break;
                                    }
                                }
                                if (m.merchant_center_error != null && m.hasOwnProperty("merchant_center_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.merchant_center_error) {
                                    default:
                                        return "merchant_center_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.multiplier_error != null && m.hasOwnProperty("multiplier_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.multiplier_error) {
                                    default:
                                        return "multiplier_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.new_resource_creation_error != null && m.hasOwnProperty("new_resource_creation_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.new_resource_creation_error) {
                                    default:
                                        return "new_resource_creation_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.not_empty_error != null && m.hasOwnProperty("not_empty_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.not_empty_error) {
                                    default:
                                        return "not_empty_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.null_error != null && m.hasOwnProperty("null_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.null_error) {
                                    default:
                                        return "null_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.operator_error != null && m.hasOwnProperty("operator_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.operator_error) {
                                    default:
                                        return "operator_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.range_error != null && m.hasOwnProperty("range_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.range_error) {
                                    default:
                                        return "range_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.recommendation_error != null && m.hasOwnProperty("recommendation_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.recommendation_error) {
                                    default:
                                        return "recommendation_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                        break;
                                    }
                                }
                                if (m.region_code_error != null && m.hasOwnProperty("region_code_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.region_code_error) {
                                    default:
                                        return "region_code_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.setting_error != null && m.hasOwnProperty("setting_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.setting_error) {
                                    default:
                                        return "setting_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.string_format_error != null && m.hasOwnProperty("string_format_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.string_format_error) {
                                    default:
                                        return "string_format_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.string_length_error != null && m.hasOwnProperty("string_length_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.string_length_error) {
                                    default:
                                        return "string_length_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 4:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.operation_access_denied_error != null && m.hasOwnProperty("operation_access_denied_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.operation_access_denied_error) {
                                    default:
                                        return "operation_access_denied_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                if (m.resource_access_denied_error != null && m.hasOwnProperty("resource_access_denied_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.resource_access_denied_error) {
                                    default:
                                        return "resource_access_denied_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.resource_count_limit_exceeded_error != null && m.hasOwnProperty("resource_count_limit_exceeded_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.resource_count_limit_exceeded_error) {
                                    default:
                                        return "resource_count_limit_exceeded_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.youtube_video_registration_error != null && m.hasOwnProperty("youtube_video_registration_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.youtube_video_registration_error) {
                                    default:
                                        return "youtube_video_registration_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.ad_group_bid_modifier_error != null && m.hasOwnProperty("ad_group_bid_modifier_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_group_bid_modifier_error) {
                                    default:
                                        return "ad_group_bid_modifier_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.context_error != null && m.hasOwnProperty("context_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.context_error) {
                                    default:
                                        return "context_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.field_error != null && m.hasOwnProperty("field_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.field_error) {
                                    default:
                                        return "field_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.shared_set_error != null && m.hasOwnProperty("shared_set_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.shared_set_error) {
                                    default:
                                        return "shared_set_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.shared_criterion_error != null && m.hasOwnProperty("shared_criterion_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.shared_criterion_error) {
                                    default:
                                        return "shared_criterion_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.campaign_shared_set_error != null && m.hasOwnProperty("campaign_shared_set_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_shared_set_error) {
                                    default:
                                        return "campaign_shared_set_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.conversion_action_error != null && m.hasOwnProperty("conversion_action_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.conversion_action_error) {
                                    default:
                                        return "conversion_action_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                if (m.conversion_adjustment_upload_error != null && m.hasOwnProperty("conversion_adjustment_upload_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.conversion_adjustment_upload_error) {
                                    default:
                                        return "conversion_adjustment_upload_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.conversion_custom_variable_error != null && m.hasOwnProperty("conversion_custom_variable_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.conversion_custom_variable_error) {
                                    default:
                                        return "conversion_custom_variable_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.conversion_upload_error != null && m.hasOwnProperty("conversion_upload_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.conversion_upload_error) {
                                    default:
                                        return "conversion_upload_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 46:
                                    case 47:
                                    case 48:
                                    case 49:
                                        break;
                                    }
                                }
                                if (m.conversion_value_rule_error != null && m.hasOwnProperty("conversion_value_rule_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.conversion_value_rule_error) {
                                    default:
                                        return "conversion_value_rule_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.conversion_value_rule_set_error != null && m.hasOwnProperty("conversion_value_rule_set_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.conversion_value_rule_set_error) {
                                    default:
                                        return "conversion_value_rule_set_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                        break;
                                    }
                                }
                                if (m.header_error != null && m.hasOwnProperty("header_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.header_error) {
                                    default:
                                        return "header_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.database_error != null && m.hasOwnProperty("database_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.database_error) {
                                    default:
                                        return "database_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.policy_finding_error != null && m.hasOwnProperty("policy_finding_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.policy_finding_error) {
                                    default:
                                        return "policy_finding_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.enum_error != null && m.hasOwnProperty("enum_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.enum_error) {
                                    default:
                                        return "enum_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.keyword_plan_error != null && m.hasOwnProperty("keyword_plan_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.keyword_plan_error) {
                                    default:
                                        return "keyword_plan_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                        break;
                                    }
                                }
                                if (m.keyword_plan_campaign_error != null && m.hasOwnProperty("keyword_plan_campaign_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.keyword_plan_campaign_error) {
                                    default:
                                        return "keyword_plan_campaign_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.keyword_plan_campaign_keyword_error != null && m.hasOwnProperty("keyword_plan_campaign_keyword_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.keyword_plan_campaign_keyword_error) {
                                    default:
                                        return "keyword_plan_campaign_keyword_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.keyword_plan_ad_group_error != null && m.hasOwnProperty("keyword_plan_ad_group_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.keyword_plan_ad_group_error) {
                                    default:
                                        return "keyword_plan_ad_group_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.keyword_plan_ad_group_keyword_error != null && m.hasOwnProperty("keyword_plan_ad_group_keyword_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.keyword_plan_ad_group_keyword_error) {
                                    default:
                                        return "keyword_plan_ad_group_keyword_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.keyword_plan_idea_error != null && m.hasOwnProperty("keyword_plan_idea_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.keyword_plan_idea_error) {
                                    default:
                                        return "keyword_plan_idea_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.account_budget_proposal_error != null && m.hasOwnProperty("account_budget_proposal_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.account_budget_proposal_error) {
                                    default:
                                        return "account_budget_proposal_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.user_list_error != null && m.hasOwnProperty("user_list_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.user_list_error) {
                                    default:
                                        return "user_list_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 37:
                                    case 34:
                                    case 35:
                                    case 36:
                                        break;
                                    }
                                }
                                if (m.change_event_error != null && m.hasOwnProperty("change_event_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.change_event_error) {
                                    default:
                                        return "change_event_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.change_status_error != null && m.hasOwnProperty("change_status_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.change_status_error) {
                                    default:
                                        return "change_status_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.feed_error != null && m.hasOwnProperty("feed_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_error) {
                                    default:
                                        return "feed_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                        break;
                                    }
                                }
                                if (m.geo_target_constant_suggestion_error != null && m.hasOwnProperty("geo_target_constant_suggestion_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.geo_target_constant_suggestion_error) {
                                    default:
                                        return "geo_target_constant_suggestion_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.campaign_draft_error != null && m.hasOwnProperty("campaign_draft_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_draft_error) {
                                    default:
                                        return "campaign_draft_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                if (m.feed_item_error != null && m.hasOwnProperty("feed_item_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_item_error) {
                                    default:
                                        return "feed_item_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.label_error != null && m.hasOwnProperty("label_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.label_error) {
                                    default:
                                        return "label_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.billing_setup_error != null && m.hasOwnProperty("billing_setup_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.billing_setup_error) {
                                    default:
                                        return "billing_setup_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 19:
                                    case 20:
                                        break;
                                    }
                                }
                                if (m.customer_client_link_error != null && m.hasOwnProperty("customer_client_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.customer_client_link_error) {
                                    default:
                                        return "customer_client_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.customer_manager_link_error != null && m.hasOwnProperty("customer_manager_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.customer_manager_link_error) {
                                    default:
                                        return "customer_manager_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.feed_mapping_error != null && m.hasOwnProperty("feed_mapping_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_mapping_error) {
                                    default:
                                        return "feed_mapping_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                        break;
                                    }
                                }
                                if (m.customer_feed_error != null && m.hasOwnProperty("customer_feed_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.customer_feed_error) {
                                    default:
                                        return "customer_feed_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.ad_group_feed_error != null && m.hasOwnProperty("ad_group_feed_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_group_feed_error) {
                                    default:
                                        return "ad_group_feed_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.campaign_feed_error != null && m.hasOwnProperty("campaign_feed_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_feed_error) {
                                    default:
                                        return "campaign_feed_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.custom_interest_error != null && m.hasOwnProperty("custom_interest_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.custom_interest_error) {
                                    default:
                                        return "custom_interest_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.campaign_experiment_error != null && m.hasOwnProperty("campaign_experiment_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.campaign_experiment_error) {
                                    default:
                                        return "campaign_experiment_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                if (m.extension_feed_item_error != null && m.hasOwnProperty("extension_feed_item_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.extension_feed_item_error) {
                                    default:
                                        return "extension_feed_item_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 47:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 46:
                                        break;
                                    }
                                }
                                if (m.ad_parameter_error != null && m.hasOwnProperty("ad_parameter_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.ad_parameter_error) {
                                    default:
                                        return "ad_parameter_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.feed_item_validation_error != null && m.hasOwnProperty("feed_item_validation_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_item_validation_error) {
                                    default:
                                        return "feed_item_validation_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 99:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 46:
                                    case 47:
                                    case 48:
                                    case 49:
                                    case 50:
                                    case 51:
                                    case 52:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                    case 58:
                                    case 59:
                                    case 60:
                                    case 61:
                                    case 62:
                                    case 63:
                                    case 64:
                                    case 65:
                                    case 66:
                                    case 67:
                                    case 68:
                                    case 69:
                                    case 70:
                                    case 71:
                                    case 72:
                                    case 73:
                                    case 74:
                                    case 75:
                                    case 76:
                                    case 77:
                                    case 78:
                                    case 79:
                                    case 80:
                                    case 81:
                                    case 82:
                                    case 83:
                                    case 84:
                                    case 85:
                                    case 86:
                                    case 87:
                                    case 88:
                                    case 89:
                                    case 90:
                                    case 91:
                                    case 92:
                                    case 93:
                                    case 94:
                                    case 95:
                                    case 96:
                                    case 97:
                                    case 98:
                                    case 100:
                                    case 101:
                                    case 102:
                                    case 103:
                                    case 104:
                                    case 105:
                                    case 106:
                                    case 107:
                                        break;
                                    }
                                }
                                if (m.extension_setting_error != null && m.hasOwnProperty("extension_setting_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.extension_setting_error) {
                                    default:
                                        return "extension_setting_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 69:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 45:
                                    case 47:
                                    case 48:
                                    case 49:
                                    case 50:
                                    case 51:
                                    case 52:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                    case 58:
                                    case 59:
                                    case 60:
                                    case 61:
                                    case 62:
                                    case 63:
                                    case 64:
                                    case 65:
                                    case 66:
                                    case 67:
                                    case 68:
                                        break;
                                    }
                                }
                                if (m.feed_item_set_error != null && m.hasOwnProperty("feed_item_set_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_item_set_error) {
                                    default:
                                        return "feed_item_set_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.feed_item_set_link_error != null && m.hasOwnProperty("feed_item_set_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_item_set_link_error) {
                                    default:
                                        return "feed_item_set_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.feed_item_target_error != null && m.hasOwnProperty("feed_item_target_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.feed_item_target_error) {
                                    default:
                                        return "feed_item_target_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.policy_violation_error != null && m.hasOwnProperty("policy_violation_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.policy_violation_error) {
                                    default:
                                        return "policy_violation_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.partial_failure_error) {
                                    default:
                                        return "partial_failure_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.policy_validation_parameter_error != null && m.hasOwnProperty("policy_validation_parameter_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.policy_validation_parameter_error) {
                                    default:
                                        return "policy_validation_parameter_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.size_limit_error != null && m.hasOwnProperty("size_limit_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.size_limit_error) {
                                    default:
                                        return "size_limit_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.offline_user_data_job_error != null && m.hasOwnProperty("offline_user_data_job_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.offline_user_data_job_error) {
                                    default:
                                        return "offline_user_data_job_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 33:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 31:
                                    case 32:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 29:
                                    case 30:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                        break;
                                    }
                                }
                                if (m.not_allowlisted_error != null && m.hasOwnProperty("not_allowlisted_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.not_allowlisted_error) {
                                    default:
                                        return "not_allowlisted_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.manager_link_error != null && m.hasOwnProperty("manager_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.manager_link_error) {
                                    default:
                                        return "manager_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                        break;
                                    }
                                }
                                if (m.currency_code_error != null && m.hasOwnProperty("currency_code_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.currency_code_error) {
                                    default:
                                        return "currency_code_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.experiment_error != null && m.hasOwnProperty("experiment_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.experiment_error) {
                                    default:
                                        return "experiment_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.access_invitation_error != null && m.hasOwnProperty("access_invitation_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.access_invitation_error) {
                                    default:
                                        return "access_invitation_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.reach_plan_error != null && m.hasOwnProperty("reach_plan_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.reach_plan_error) {
                                    default:
                                        return "reach_plan_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.invoice_error != null && m.hasOwnProperty("invoice_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.invoice_error) {
                                    default:
                                        return "invoice_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.payments_account_error != null && m.hasOwnProperty("payments_account_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.payments_account_error) {
                                    default:
                                        return "payments_account_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.time_zone_error != null && m.hasOwnProperty("time_zone_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.time_zone_error) {
                                    default:
                                        return "time_zone_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.asset_link_error != null && m.hasOwnProperty("asset_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.asset_link_error) {
                                    default:
                                        return "asset_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.user_data_error != null && m.hasOwnProperty("user_data_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.user_data_error) {
                                    default:
                                        return "user_data_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.batch_job_error != null && m.hasOwnProperty("batch_job_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.batch_job_error) {
                                    default:
                                        return "batch_job_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.account_link_error != null && m.hasOwnProperty("account_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.account_link_error) {
                                    default:
                                        return "account_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.third_party_app_analytics_link_error != null && m.hasOwnProperty("third_party_app_analytics_link_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.third_party_app_analytics_link_error) {
                                    default:
                                        return "third_party_app_analytics_link_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.customer_user_access_error != null && m.hasOwnProperty("customer_user_access_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.customer_user_access_error) {
                                    default:
                                        return "customer_user_access_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.custom_audience_error != null && m.hasOwnProperty("custom_audience_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.custom_audience_error) {
                                    default:
                                        return "custom_audience_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.audience_error != null && m.hasOwnProperty("audience_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.audience_error) {
                                    default:
                                        return "audience_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.experiment_arm_error != null && m.hasOwnProperty("experiment_arm_error")) {
                                    if (p.error_code === 1)
                                        return "error_code: multiple values";
                                    p.error_code = 1;
                                    switch (m.experiment_arm_error) {
                                    default:
                                        return "experiment_arm_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ErrorCode message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ErrorCode} ErrorCode
                             */
                            ErrorCode.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ErrorCode)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.ErrorCode();
                                switch (d.request_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.request_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.request_error = 1;
                                    break;
                                case "RESOURCE_NAME_MISSING":
                                case 3:
                                    m.request_error = 3;
                                    break;
                                case "RESOURCE_NAME_MALFORMED":
                                case 4:
                                    m.request_error = 4;
                                    break;
                                case "BAD_RESOURCE_ID":
                                case 17:
                                    m.request_error = 17;
                                    break;
                                case "INVALID_CUSTOMER_ID":
                                case 16:
                                    m.request_error = 16;
                                    break;
                                case "OPERATION_REQUIRED":
                                case 5:
                                    m.request_error = 5;
                                    break;
                                case "RESOURCE_NOT_FOUND":
                                case 6:
                                    m.request_error = 6;
                                    break;
                                case "INVALID_PAGE_TOKEN":
                                case 7:
                                    m.request_error = 7;
                                    break;
                                case "EXPIRED_PAGE_TOKEN":
                                case 8:
                                    m.request_error = 8;
                                    break;
                                case "INVALID_PAGE_SIZE":
                                case 22:
                                    m.request_error = 22;
                                    break;
                                case "REQUIRED_FIELD_MISSING":
                                case 9:
                                    m.request_error = 9;
                                    break;
                                case "IMMUTABLE_FIELD":
                                case 11:
                                    m.request_error = 11;
                                    break;
                                case "TOO_MANY_MUTATE_OPERATIONS":
                                case 13:
                                    m.request_error = 13;
                                    break;
                                case "CANNOT_BE_EXECUTED_BY_MANAGER_ACCOUNT":
                                case 14:
                                    m.request_error = 14;
                                    break;
                                case "CANNOT_MODIFY_FOREIGN_FIELD":
                                case 15:
                                    m.request_error = 15;
                                    break;
                                case "INVALID_ENUM_VALUE":
                                case 18:
                                    m.request_error = 18;
                                    break;
                                case "DEVELOPER_TOKEN_PARAMETER_MISSING":
                                case 19:
                                    m.request_error = 19;
                                    break;
                                case "LOGIN_CUSTOMER_ID_PARAMETER_MISSING":
                                case 20:
                                    m.request_error = 20;
                                    break;
                                case "VALIDATE_ONLY_REQUEST_HAS_PAGE_TOKEN":
                                case 21:
                                    m.request_error = 21;
                                    break;
                                case "CANNOT_RETURN_SUMMARY_ROW_FOR_REQUEST_WITHOUT_METRICS":
                                case 29:
                                    m.request_error = 29;
                                    break;
                                case "CANNOT_RETURN_SUMMARY_ROW_FOR_VALIDATE_ONLY_REQUESTS":
                                case 30:
                                    m.request_error = 30;
                                    break;
                                case "INCONSISTENT_RETURN_SUMMARY_ROW_VALUE":
                                case 31:
                                    m.request_error = 31;
                                    break;
                                case "TOTAL_RESULTS_COUNT_NOT_ORIGINALLY_REQUESTED":
                                case 32:
                                    m.request_error = 32;
                                    break;
                                case "RPC_DEADLINE_TOO_SHORT":
                                case 33:
                                    m.request_error = 33;
                                    break;
                                }
                                switch (d.bidding_strategy_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.bidding_strategy_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.bidding_strategy_error = 1;
                                    break;
                                case "DUPLICATE_NAME":
                                case 2:
                                    m.bidding_strategy_error = 2;
                                    break;
                                case "CANNOT_CHANGE_BIDDING_STRATEGY_TYPE":
                                case 3:
                                    m.bidding_strategy_error = 3;
                                    break;
                                case "CANNOT_REMOVE_ASSOCIATED_STRATEGY":
                                case 4:
                                    m.bidding_strategy_error = 4;
                                    break;
                                case "BIDDING_STRATEGY_NOT_SUPPORTED":
                                case 5:
                                    m.bidding_strategy_error = 5;
                                    break;
                                case "INCOMPATIBLE_BIDDING_STRATEGY_AND_BIDDING_STRATEGY_GOAL_TYPE":
                                case 6:
                                    m.bidding_strategy_error = 6;
                                    break;
                                }
                                switch (d.url_field_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.url_field_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.url_field_error = 1;
                                    break;
                                case "INVALID_TRACKING_URL_TEMPLATE":
                                case 2:
                                    m.url_field_error = 2;
                                    break;
                                case "INVALID_TAG_IN_TRACKING_URL_TEMPLATE":
                                case 3:
                                    m.url_field_error = 3;
                                    break;
                                case "MISSING_TRACKING_URL_TEMPLATE_TAG":
                                case 4:
                                    m.url_field_error = 4;
                                    break;
                                case "MISSING_PROTOCOL_IN_TRACKING_URL_TEMPLATE":
                                case 5:
                                    m.url_field_error = 5;
                                    break;
                                case "INVALID_PROTOCOL_IN_TRACKING_URL_TEMPLATE":
                                case 6:
                                    m.url_field_error = 6;
                                    break;
                                case "MALFORMED_TRACKING_URL_TEMPLATE":
                                case 7:
                                    m.url_field_error = 7;
                                    break;
                                case "MISSING_HOST_IN_TRACKING_URL_TEMPLATE":
                                case 8:
                                    m.url_field_error = 8;
                                    break;
                                case "INVALID_TLD_IN_TRACKING_URL_TEMPLATE":
                                case 9:
                                    m.url_field_error = 9;
                                    break;
                                case "REDUNDANT_NESTED_TRACKING_URL_TEMPLATE_TAG":
                                case 10:
                                    m.url_field_error = 10;
                                    break;
                                case "INVALID_FINAL_URL":
                                case 11:
                                    m.url_field_error = 11;
                                    break;
                                case "INVALID_TAG_IN_FINAL_URL":
                                case 12:
                                    m.url_field_error = 12;
                                    break;
                                case "REDUNDANT_NESTED_FINAL_URL_TAG":
                                case 13:
                                    m.url_field_error = 13;
                                    break;
                                case "MISSING_PROTOCOL_IN_FINAL_URL":
                                case 14:
                                    m.url_field_error = 14;
                                    break;
                                case "INVALID_PROTOCOL_IN_FINAL_URL":
                                case 15:
                                    m.url_field_error = 15;
                                    break;
                                case "MALFORMED_FINAL_URL":
                                case 16:
                                    m.url_field_error = 16;
                                    break;
                                case "MISSING_HOST_IN_FINAL_URL":
                                case 17:
                                    m.url_field_error = 17;
                                    break;
                                case "INVALID_TLD_IN_FINAL_URL":
                                case 18:
                                    m.url_field_error = 18;
                                    break;
                                case "INVALID_FINAL_MOBILE_URL":
                                case 19:
                                    m.url_field_error = 19;
                                    break;
                                case "INVALID_TAG_IN_FINAL_MOBILE_URL":
                                case 20:
                                    m.url_field_error = 20;
                                    break;
                                case "REDUNDANT_NESTED_FINAL_MOBILE_URL_TAG":
                                case 21:
                                    m.url_field_error = 21;
                                    break;
                                case "MISSING_PROTOCOL_IN_FINAL_MOBILE_URL":
                                case 22:
                                    m.url_field_error = 22;
                                    break;
                                case "INVALID_PROTOCOL_IN_FINAL_MOBILE_URL":
                                case 23:
                                    m.url_field_error = 23;
                                    break;
                                case "MALFORMED_FINAL_MOBILE_URL":
                                case 24:
                                    m.url_field_error = 24;
                                    break;
                                case "MISSING_HOST_IN_FINAL_MOBILE_URL":
                                case 25:
                                    m.url_field_error = 25;
                                    break;
                                case "INVALID_TLD_IN_FINAL_MOBILE_URL":
                                case 26:
                                    m.url_field_error = 26;
                                    break;
                                case "INVALID_FINAL_APP_URL":
                                case 27:
                                    m.url_field_error = 27;
                                    break;
                                case "INVALID_TAG_IN_FINAL_APP_URL":
                                case 28:
                                    m.url_field_error = 28;
                                    break;
                                case "REDUNDANT_NESTED_FINAL_APP_URL_TAG":
                                case 29:
                                    m.url_field_error = 29;
                                    break;
                                case "MULTIPLE_APP_URLS_FOR_OSTYPE":
                                case 30:
                                    m.url_field_error = 30;
                                    break;
                                case "INVALID_OSTYPE":
                                case 31:
                                    m.url_field_error = 31;
                                    break;
                                case "INVALID_PROTOCOL_FOR_APP_URL":
                                case 32:
                                    m.url_field_error = 32;
                                    break;
                                case "INVALID_PACKAGE_ID_FOR_APP_URL":
                                case 33:
                                    m.url_field_error = 33;
                                    break;
                                case "URL_CUSTOM_PARAMETERS_COUNT_EXCEEDS_LIMIT":
                                case 34:
                                    m.url_field_error = 34;
                                    break;
                                case "INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_KEY":
                                case 39:
                                    m.url_field_error = 39;
                                    break;
                                case "INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_VALUE":
                                case 40:
                                    m.url_field_error = 40;
                                    break;
                                case "INVALID_TAG_IN_URL_CUSTOM_PARAMETER_VALUE":
                                case 41:
                                    m.url_field_error = 41;
                                    break;
                                case "REDUNDANT_NESTED_URL_CUSTOM_PARAMETER_TAG":
                                case 42:
                                    m.url_field_error = 42;
                                    break;
                                case "MISSING_PROTOCOL":
                                case 43:
                                    m.url_field_error = 43;
                                    break;
                                case "INVALID_PROTOCOL":
                                case 52:
                                    m.url_field_error = 52;
                                    break;
                                case "INVALID_URL":
                                case 44:
                                    m.url_field_error = 44;
                                    break;
                                case "DESTINATION_URL_DEPRECATED":
                                case 45:
                                    m.url_field_error = 45;
                                    break;
                                case "INVALID_TAG_IN_URL":
                                case 46:
                                    m.url_field_error = 46;
                                    break;
                                case "MISSING_URL_TAG":
                                case 47:
                                    m.url_field_error = 47;
                                    break;
                                case "DUPLICATE_URL_ID":
                                case 48:
                                    m.url_field_error = 48;
                                    break;
                                case "INVALID_URL_ID":
                                case 49:
                                    m.url_field_error = 49;
                                    break;
                                case "FINAL_URL_SUFFIX_MALFORMED":
                                case 50:
                                    m.url_field_error = 50;
                                    break;
                                case "INVALID_TAG_IN_FINAL_URL_SUFFIX":
                                case 51:
                                    m.url_field_error = 51;
                                    break;
                                case "INVALID_TOP_LEVEL_DOMAIN":
                                case 53:
                                    m.url_field_error = 53;
                                    break;
                                case "MALFORMED_TOP_LEVEL_DOMAIN":
                                case 54:
                                    m.url_field_error = 54;
                                    break;
                                case "MALFORMED_URL":
                                case 55:
                                    m.url_field_error = 55;
                                    break;
                                case "MISSING_HOST":
                                case 56:
                                    m.url_field_error = 56;
                                    break;
                                case "NULL_CUSTOM_PARAMETER_VALUE":
                                case 57:
                                    m.url_field_error = 57;
                                    break;
                                case "VALUE_TRACK_PARAMETER_NOT_SUPPORTED":
                                case 58:
                                    m.url_field_error = 58;
                                    break;
                                }
                                switch (d.list_operation_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.list_operation_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.list_operation_error = 1;
                                    break;
                                case "REQUIRED_FIELD_MISSING":
                                case 7:
                                    m.list_operation_error = 7;
                                    break;
                                case "DUPLICATE_VALUES":
                                case 8:
                                    m.list_operation_error = 8;
                                    break;
                                }
                                switch (d.query_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.query_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.query_error = 1;
                                    break;
                                case "QUERY_ERROR":
                                case 50:
                                    m.query_error = 50;
                                    break;
                                case "BAD_ENUM_CONSTANT":
                                case 18:
                                    m.query_error = 18;
                                    break;
                                case "BAD_ESCAPE_SEQUENCE":
                                case 7:
                                    m.query_error = 7;
                                    break;
                                case "BAD_FIELD_NAME":
                                case 12:
                                    m.query_error = 12;
                                    break;
                                case "BAD_LIMIT_VALUE":
                                case 15:
                                    m.query_error = 15;
                                    break;
                                case "BAD_NUMBER":
                                case 5:
                                    m.query_error = 5;
                                    break;
                                case "BAD_OPERATOR":
                                case 3:
                                    m.query_error = 3;
                                    break;
                                case "BAD_PARAMETER_NAME":
                                case 61:
                                    m.query_error = 61;
                                    break;
                                case "BAD_PARAMETER_VALUE":
                                case 62:
                                    m.query_error = 62;
                                    break;
                                case "BAD_RESOURCE_TYPE_IN_FROM_CLAUSE":
                                case 45:
                                    m.query_error = 45;
                                    break;
                                case "BAD_SYMBOL":
                                case 2:
                                    m.query_error = 2;
                                    break;
                                case "BAD_VALUE":
                                case 4:
                                    m.query_error = 4;
                                    break;
                                case "DATE_RANGE_TOO_WIDE":
                                case 36:
                                    m.query_error = 36;
                                    break;
                                case "DATE_RANGE_TOO_NARROW":
                                case 60:
                                    m.query_error = 60;
                                    break;
                                case "EXPECTED_AND":
                                case 30:
                                    m.query_error = 30;
                                    break;
                                case "EXPECTED_BY":
                                case 14:
                                    m.query_error = 14;
                                    break;
                                case "EXPECTED_DIMENSION_FIELD_IN_SELECT_CLAUSE":
                                case 37:
                                    m.query_error = 37;
                                    break;
                                case "EXPECTED_FILTERS_ON_DATE_RANGE":
                                case 55:
                                    m.query_error = 55;
                                    break;
                                case "EXPECTED_FROM":
                                case 44:
                                    m.query_error = 44;
                                    break;
                                case "EXPECTED_LIST":
                                case 41:
                                    m.query_error = 41;
                                    break;
                                case "EXPECTED_REFERENCED_FIELD_IN_SELECT_CLAUSE":
                                case 16:
                                    m.query_error = 16;
                                    break;
                                case "EXPECTED_SELECT":
                                case 13:
                                    m.query_error = 13;
                                    break;
                                case "EXPECTED_SINGLE_VALUE":
                                case 42:
                                    m.query_error = 42;
                                    break;
                                case "EXPECTED_VALUE_WITH_BETWEEN_OPERATOR":
                                case 29:
                                    m.query_error = 29;
                                    break;
                                case "INVALID_DATE_FORMAT":
                                case 38:
                                    m.query_error = 38;
                                    break;
                                case "MISALIGNED_DATE_FOR_FILTER":
                                case 64:
                                    m.query_error = 64;
                                    break;
                                case "INVALID_STRING_VALUE":
                                case 57:
                                    m.query_error = 57;
                                    break;
                                case "INVALID_VALUE_WITH_BETWEEN_OPERATOR":
                                case 26:
                                    m.query_error = 26;
                                    break;
                                case "INVALID_VALUE_WITH_DURING_OPERATOR":
                                case 22:
                                    m.query_error = 22;
                                    break;
                                case "INVALID_VALUE_WITH_LIKE_OPERATOR":
                                case 56:
                                    m.query_error = 56;
                                    break;
                                case "OPERATOR_FIELD_MISMATCH":
                                case 35:
                                    m.query_error = 35;
                                    break;
                                case "PROHIBITED_EMPTY_LIST_IN_CONDITION":
                                case 28:
                                    m.query_error = 28;
                                    break;
                                case "PROHIBITED_ENUM_CONSTANT":
                                case 54:
                                    m.query_error = 54;
                                    break;
                                case "PROHIBITED_FIELD_COMBINATION_IN_SELECT_CLAUSE":
                                case 31:
                                    m.query_error = 31;
                                    break;
                                case "PROHIBITED_FIELD_IN_ORDER_BY_CLAUSE":
                                case 40:
                                    m.query_error = 40;
                                    break;
                                case "PROHIBITED_FIELD_IN_SELECT_CLAUSE":
                                case 23:
                                    m.query_error = 23;
                                    break;
                                case "PROHIBITED_FIELD_IN_WHERE_CLAUSE":
                                case 24:
                                    m.query_error = 24;
                                    break;
                                case "PROHIBITED_RESOURCE_TYPE_IN_FROM_CLAUSE":
                                case 43:
                                    m.query_error = 43;
                                    break;
                                case "PROHIBITED_RESOURCE_TYPE_IN_SELECT_CLAUSE":
                                case 48:
                                    m.query_error = 48;
                                    break;
                                case "PROHIBITED_RESOURCE_TYPE_IN_WHERE_CLAUSE":
                                case 58:
                                    m.query_error = 58;
                                    break;
                                case "PROHIBITED_METRIC_IN_SELECT_OR_WHERE_CLAUSE":
                                case 49:
                                    m.query_error = 49;
                                    break;
                                case "PROHIBITED_SEGMENT_IN_SELECT_OR_WHERE_CLAUSE":
                                case 51:
                                    m.query_error = 51;
                                    break;
                                case "PROHIBITED_SEGMENT_WITH_METRIC_IN_SELECT_OR_WHERE_CLAUSE":
                                case 53:
                                    m.query_error = 53;
                                    break;
                                case "LIMIT_VALUE_TOO_LOW":
                                case 25:
                                    m.query_error = 25;
                                    break;
                                case "PROHIBITED_NEWLINE_IN_STRING":
                                case 8:
                                    m.query_error = 8;
                                    break;
                                case "PROHIBITED_VALUE_COMBINATION_IN_LIST":
                                case 10:
                                    m.query_error = 10;
                                    break;
                                case "PROHIBITED_VALUE_COMBINATION_WITH_BETWEEN_OPERATOR":
                                case 21:
                                    m.query_error = 21;
                                    break;
                                case "STRING_NOT_TERMINATED":
                                case 6:
                                    m.query_error = 6;
                                    break;
                                case "TOO_MANY_SEGMENTS":
                                case 34:
                                    m.query_error = 34;
                                    break;
                                case "UNEXPECTED_END_OF_QUERY":
                                case 9:
                                    m.query_error = 9;
                                    break;
                                case "UNEXPECTED_FROM_CLAUSE":
                                case 47:
                                    m.query_error = 47;
                                    break;
                                case "UNRECOGNIZED_FIELD":
                                case 32:
                                    m.query_error = 32;
                                    break;
                                case "UNEXPECTED_INPUT":
                                case 11:
                                    m.query_error = 11;
                                    break;
                                case "REQUESTED_METRICS_FOR_MANAGER":
                                case 59:
                                    m.query_error = 59;
                                    break;
                                case "FILTER_HAS_TOO_MANY_VALUES":
                                case 63:
                                    m.query_error = 63;
                                    break;
                                }
                                switch (d.mutate_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.mutate_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.mutate_error = 1;
                                    break;
                                case "RESOURCE_NOT_FOUND":
                                case 3:
                                    m.mutate_error = 3;
                                    break;
                                case "ID_EXISTS_IN_MULTIPLE_MUTATES":
                                case 7:
                                    m.mutate_error = 7;
                                    break;
                                case "INCONSISTENT_FIELD_VALUES":
                                case 8:
                                    m.mutate_error = 8;
                                    break;
                                case "MUTATE_NOT_ALLOWED":
                                case 9:
                                    m.mutate_error = 9;
                                    break;
                                case "RESOURCE_NOT_IN_GOOGLE_ADS":
                                case 10:
                                    m.mutate_error = 10;
                                    break;
                                case "RESOURCE_ALREADY_EXISTS":
                                case 11:
                                    m.mutate_error = 11;
                                    break;
                                case "RESOURCE_DOES_NOT_SUPPORT_VALIDATE_ONLY":
                                case 12:
                                    m.mutate_error = 12;
                                    break;
                                case "OPERATION_DOES_NOT_SUPPORT_PARTIAL_FAILURE":
                                case 16:
                                    m.mutate_error = 16;
                                    break;
                                case "RESOURCE_READ_ONLY":
                                case 13:
                                    m.mutate_error = 13;
                                    break;
                                }
                                switch (d.field_mask_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_mask_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_mask_error = 1;
                                    break;
                                case "FIELD_MASK_MISSING":
                                case 5:
                                    m.field_mask_error = 5;
                                    break;
                                case "FIELD_MASK_NOT_ALLOWED":
                                case 4:
                                    m.field_mask_error = 4;
                                    break;
                                case "FIELD_NOT_FOUND":
                                case 2:
                                    m.field_mask_error = 2;
                                    break;
                                case "FIELD_HAS_SUBFIELDS":
                                case 3:
                                    m.field_mask_error = 3;
                                    break;
                                }
                                switch (d.authorization_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.authorization_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.authorization_error = 1;
                                    break;
                                case "USER_PERMISSION_DENIED":
                                case 2:
                                    m.authorization_error = 2;
                                    break;
                                case "DEVELOPER_TOKEN_NOT_ON_ALLOWLIST":
                                case 13:
                                    m.authorization_error = 13;
                                    break;
                                case "DEVELOPER_TOKEN_PROHIBITED":
                                case 4:
                                    m.authorization_error = 4;
                                    break;
                                case "PROJECT_DISABLED":
                                case 5:
                                    m.authorization_error = 5;
                                    break;
                                case "AUTHORIZATION_ERROR":
                                case 6:
                                    m.authorization_error = 6;
                                    break;
                                case "ACTION_NOT_PERMITTED":
                                case 7:
                                    m.authorization_error = 7;
                                    break;
                                case "INCOMPLETE_SIGNUP":
                                case 8:
                                    m.authorization_error = 8;
                                    break;
                                case "CUSTOMER_NOT_ENABLED":
                                case 24:
                                    m.authorization_error = 24;
                                    break;
                                case "MISSING_TOS":
                                case 9:
                                    m.authorization_error = 9;
                                    break;
                                case "DEVELOPER_TOKEN_NOT_APPROVED":
                                case 10:
                                    m.authorization_error = 10;
                                    break;
                                case "INVALID_LOGIN_CUSTOMER_ID_SERVING_CUSTOMER_ID_COMBINATION":
                                case 11:
                                    m.authorization_error = 11;
                                    break;
                                case "SERVICE_ACCESS_DENIED":
                                case 12:
                                    m.authorization_error = 12;
                                    break;
                                case "ACCESS_DENIED_FOR_ACCOUNT_TYPE":
                                case 25:
                                    m.authorization_error = 25;
                                    break;
                                case "METRIC_ACCESS_DENIED":
                                case 26:
                                    m.authorization_error = 26;
                                    break;
                                }
                                switch (d.internal_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.internal_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.internal_error = 1;
                                    break;
                                case "INTERNAL_ERROR":
                                case 2:
                                    m.internal_error = 2;
                                    break;
                                case "ERROR_CODE_NOT_PUBLISHED":
                                case 3:
                                    m.internal_error = 3;
                                    break;
                                case "TRANSIENT_ERROR":
                                case 4:
                                    m.internal_error = 4;
                                    break;
                                case "DEADLINE_EXCEEDED":
                                case 5:
                                    m.internal_error = 5;
                                    break;
                                }
                                switch (d.quota_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.quota_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.quota_error = 1;
                                    break;
                                case "RESOURCE_EXHAUSTED":
                                case 2:
                                    m.quota_error = 2;
                                    break;
                                case "ACCESS_PROHIBITED":
                                case 3:
                                    m.quota_error = 3;
                                    break;
                                case "RESOURCE_TEMPORARILY_EXHAUSTED":
                                case 4:
                                    m.quota_error = 4;
                                    break;
                                }
                                switch (d.ad_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_error = 1;
                                    break;
                                case "AD_CUSTOMIZERS_NOT_SUPPORTED_FOR_AD_TYPE":
                                case 2:
                                    m.ad_error = 2;
                                    break;
                                case "APPROXIMATELY_TOO_LONG":
                                case 3:
                                    m.ad_error = 3;
                                    break;
                                case "APPROXIMATELY_TOO_SHORT":
                                case 4:
                                    m.ad_error = 4;
                                    break;
                                case "BAD_SNIPPET":
                                case 5:
                                    m.ad_error = 5;
                                    break;
                                case "CANNOT_MODIFY_AD":
                                case 6:
                                    m.ad_error = 6;
                                    break;
                                case "CANNOT_SET_BUSINESS_NAME_IF_URL_SET":
                                case 7:
                                    m.ad_error = 7;
                                    break;
                                case "CANNOT_SET_FIELD":
                                case 8:
                                    m.ad_error = 8;
                                    break;
                                case "CANNOT_SET_FIELD_WITH_ORIGIN_AD_ID_SET":
                                case 9:
                                    m.ad_error = 9;
                                    break;
                                case "CANNOT_SET_FIELD_WITH_AD_ID_SET_FOR_SHARING":
                                case 10:
                                    m.ad_error = 10;
                                    break;
                                case "CANNOT_SET_ALLOW_FLEXIBLE_COLOR_FALSE":
                                case 11:
                                    m.ad_error = 11;
                                    break;
                                case "CANNOT_SET_COLOR_CONTROL_WHEN_NATIVE_FORMAT_SETTING":
                                case 12:
                                    m.ad_error = 12;
                                    break;
                                case "CANNOT_SET_URL":
                                case 13:
                                    m.ad_error = 13;
                                    break;
                                case "CANNOT_SET_WITHOUT_FINAL_URLS":
                                case 14:
                                    m.ad_error = 14;
                                    break;
                                case "CANNOT_SET_WITH_FINAL_URLS":
                                case 15:
                                    m.ad_error = 15;
                                    break;
                                case "CANNOT_SET_WITH_URL_DATA":
                                case 17:
                                    m.ad_error = 17;
                                    break;
                                case "CANNOT_USE_AD_SUBCLASS_FOR_OPERATOR":
                                case 18:
                                    m.ad_error = 18;
                                    break;
                                case "CUSTOMER_NOT_APPROVED_MOBILEADS":
                                case 19:
                                    m.ad_error = 19;
                                    break;
                                case "CUSTOMER_NOT_APPROVED_THIRDPARTY_ADS":
                                case 20:
                                    m.ad_error = 20;
                                    break;
                                case "CUSTOMER_NOT_APPROVED_THIRDPARTY_REDIRECT_ADS":
                                case 21:
                                    m.ad_error = 21;
                                    break;
                                case "CUSTOMER_NOT_ELIGIBLE":
                                case 22:
                                    m.ad_error = 22;
                                    break;
                                case "CUSTOMER_NOT_ELIGIBLE_FOR_UPDATING_BEACON_URL":
                                case 23:
                                    m.ad_error = 23;
                                    break;
                                case "DIMENSION_ALREADY_IN_UNION":
                                case 24:
                                    m.ad_error = 24;
                                    break;
                                case "DIMENSION_MUST_BE_SET":
                                case 25:
                                    m.ad_error = 25;
                                    break;
                                case "DIMENSION_NOT_IN_UNION":
                                case 26:
                                    m.ad_error = 26;
                                    break;
                                case "DISPLAY_URL_CANNOT_BE_SPECIFIED":
                                case 27:
                                    m.ad_error = 27;
                                    break;
                                case "DOMESTIC_PHONE_NUMBER_FORMAT":
                                case 28:
                                    m.ad_error = 28;
                                    break;
                                case "EMERGENCY_PHONE_NUMBER":
                                case 29:
                                    m.ad_error = 29;
                                    break;
                                case "EMPTY_FIELD":
                                case 30:
                                    m.ad_error = 30;
                                    break;
                                case "FEED_ATTRIBUTE_MUST_HAVE_MAPPING_FOR_TYPE_ID":
                                case 31:
                                    m.ad_error = 31;
                                    break;
                                case "FEED_ATTRIBUTE_MAPPING_TYPE_MISMATCH":
                                case 32:
                                    m.ad_error = 32;
                                    break;
                                case "ILLEGAL_AD_CUSTOMIZER_TAG_USE":
                                case 33:
                                    m.ad_error = 33;
                                    break;
                                case "ILLEGAL_TAG_USE":
                                case 34:
                                    m.ad_error = 34;
                                    break;
                                case "INCONSISTENT_DIMENSIONS":
                                case 35:
                                    m.ad_error = 35;
                                    break;
                                case "INCONSISTENT_STATUS_IN_TEMPLATE_UNION":
                                case 36:
                                    m.ad_error = 36;
                                    break;
                                case "INCORRECT_LENGTH":
                                case 37:
                                    m.ad_error = 37;
                                    break;
                                case "INELIGIBLE_FOR_UPGRADE":
                                case 38:
                                    m.ad_error = 38;
                                    break;
                                case "INVALID_AD_ADDRESS_CAMPAIGN_TARGET":
                                case 39:
                                    m.ad_error = 39;
                                    break;
                                case "INVALID_AD_TYPE":
                                case 40:
                                    m.ad_error = 40;
                                    break;
                                case "INVALID_ATTRIBUTES_FOR_MOBILE_IMAGE":
                                case 41:
                                    m.ad_error = 41;
                                    break;
                                case "INVALID_ATTRIBUTES_FOR_MOBILE_TEXT":
                                case 42:
                                    m.ad_error = 42;
                                    break;
                                case "INVALID_CALL_TO_ACTION_TEXT":
                                case 43:
                                    m.ad_error = 43;
                                    break;
                                case "INVALID_CHARACTER_FOR_URL":
                                case 44:
                                    m.ad_error = 44;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 45:
                                    m.ad_error = 45;
                                    break;
                                case "INVALID_EXPANDED_DYNAMIC_SEARCH_AD_TAG":
                                case 47:
                                    m.ad_error = 47;
                                    break;
                                case "INVALID_INPUT":
                                case 48:
                                    m.ad_error = 48;
                                    break;
                                case "INVALID_MARKUP_LANGUAGE":
                                case 49:
                                    m.ad_error = 49;
                                    break;
                                case "INVALID_MOBILE_CARRIER":
                                case 50:
                                    m.ad_error = 50;
                                    break;
                                case "INVALID_MOBILE_CARRIER_TARGET":
                                case 51:
                                    m.ad_error = 51;
                                    break;
                                case "INVALID_NUMBER_OF_ELEMENTS":
                                case 52:
                                    m.ad_error = 52;
                                    break;
                                case "INVALID_PHONE_NUMBER_FORMAT":
                                case 53:
                                    m.ad_error = 53;
                                    break;
                                case "INVALID_RICH_MEDIA_CERTIFIED_VENDOR_FORMAT_ID":
                                case 54:
                                    m.ad_error = 54;
                                    break;
                                case "INVALID_TEMPLATE_DATA":
                                case 55:
                                    m.ad_error = 55;
                                    break;
                                case "INVALID_TEMPLATE_ELEMENT_FIELD_TYPE":
                                case 56:
                                    m.ad_error = 56;
                                    break;
                                case "INVALID_TEMPLATE_ID":
                                case 57:
                                    m.ad_error = 57;
                                    break;
                                case "LINE_TOO_WIDE":
                                case 58:
                                    m.ad_error = 58;
                                    break;
                                case "MISSING_AD_CUSTOMIZER_MAPPING":
                                case 59:
                                    m.ad_error = 59;
                                    break;
                                case "MISSING_ADDRESS_COMPONENT":
                                case 60:
                                    m.ad_error = 60;
                                    break;
                                case "MISSING_ADVERTISEMENT_NAME":
                                case 61:
                                    m.ad_error = 61;
                                    break;
                                case "MISSING_BUSINESS_NAME":
                                case 62:
                                    m.ad_error = 62;
                                    break;
                                case "MISSING_DESCRIPTION1":
                                case 63:
                                    m.ad_error = 63;
                                    break;
                                case "MISSING_DESCRIPTION2":
                                case 64:
                                    m.ad_error = 64;
                                    break;
                                case "MISSING_DESTINATION_URL_TAG":
                                case 65:
                                    m.ad_error = 65;
                                    break;
                                case "MISSING_LANDING_PAGE_URL_TAG":
                                case 66:
                                    m.ad_error = 66;
                                    break;
                                case "MISSING_DIMENSION":
                                case 67:
                                    m.ad_error = 67;
                                    break;
                                case "MISSING_DISPLAY_URL":
                                case 68:
                                    m.ad_error = 68;
                                    break;
                                case "MISSING_HEADLINE":
                                case 69:
                                    m.ad_error = 69;
                                    break;
                                case "MISSING_HEIGHT":
                                case 70:
                                    m.ad_error = 70;
                                    break;
                                case "MISSING_IMAGE":
                                case 71:
                                    m.ad_error = 71;
                                    break;
                                case "MISSING_MARKETING_IMAGE_OR_PRODUCT_VIDEOS":
                                case 72:
                                    m.ad_error = 72;
                                    break;
                                case "MISSING_MARKUP_LANGUAGES":
                                case 73:
                                    m.ad_error = 73;
                                    break;
                                case "MISSING_MOBILE_CARRIER":
                                case 74:
                                    m.ad_error = 74;
                                    break;
                                case "MISSING_PHONE":
                                case 75:
                                    m.ad_error = 75;
                                    break;
                                case "MISSING_REQUIRED_TEMPLATE_FIELDS":
                                case 76:
                                    m.ad_error = 76;
                                    break;
                                case "MISSING_TEMPLATE_FIELD_VALUE":
                                case 77:
                                    m.ad_error = 77;
                                    break;
                                case "MISSING_TEXT":
                                case 78:
                                    m.ad_error = 78;
                                    break;
                                case "MISSING_VISIBLE_URL":
                                case 79:
                                    m.ad_error = 79;
                                    break;
                                case "MISSING_WIDTH":
                                case 80:
                                    m.ad_error = 80;
                                    break;
                                case "MULTIPLE_DISTINCT_FEEDS_UNSUPPORTED":
                                case 81:
                                    m.ad_error = 81;
                                    break;
                                case "MUST_USE_TEMP_AD_UNION_ID_ON_ADD":
                                case 82:
                                    m.ad_error = 82;
                                    break;
                                case "TOO_LONG":
                                case 83:
                                    m.ad_error = 83;
                                    break;
                                case "TOO_SHORT":
                                case 84:
                                    m.ad_error = 84;
                                    break;
                                case "UNION_DIMENSIONS_CANNOT_CHANGE":
                                case 85:
                                    m.ad_error = 85;
                                    break;
                                case "UNKNOWN_ADDRESS_COMPONENT":
                                case 86:
                                    m.ad_error = 86;
                                    break;
                                case "UNKNOWN_FIELD_NAME":
                                case 87:
                                    m.ad_error = 87;
                                    break;
                                case "UNKNOWN_UNIQUE_NAME":
                                case 88:
                                    m.ad_error = 88;
                                    break;
                                case "UNSUPPORTED_DIMENSIONS":
                                case 89:
                                    m.ad_error = 89;
                                    break;
                                case "URL_INVALID_SCHEME":
                                case 90:
                                    m.ad_error = 90;
                                    break;
                                case "URL_INVALID_TOP_LEVEL_DOMAIN":
                                case 91:
                                    m.ad_error = 91;
                                    break;
                                case "URL_MALFORMED":
                                case 92:
                                    m.ad_error = 92;
                                    break;
                                case "URL_NO_HOST":
                                case 93:
                                    m.ad_error = 93;
                                    break;
                                case "URL_NOT_EQUIVALENT":
                                case 94:
                                    m.ad_error = 94;
                                    break;
                                case "URL_HOST_NAME_TOO_LONG":
                                case 95:
                                    m.ad_error = 95;
                                    break;
                                case "URL_NO_SCHEME":
                                case 96:
                                    m.ad_error = 96;
                                    break;
                                case "URL_NO_TOP_LEVEL_DOMAIN":
                                case 97:
                                    m.ad_error = 97;
                                    break;
                                case "URL_PATH_NOT_ALLOWED":
                                case 98:
                                    m.ad_error = 98;
                                    break;
                                case "URL_PORT_NOT_ALLOWED":
                                case 99:
                                    m.ad_error = 99;
                                    break;
                                case "URL_QUERY_NOT_ALLOWED":
                                case 100:
                                    m.ad_error = 100;
                                    break;
                                case "URL_SCHEME_BEFORE_EXPANDED_DYNAMIC_SEARCH_AD_TAG":
                                case 102:
                                    m.ad_error = 102;
                                    break;
                                case "USER_DOES_NOT_HAVE_ACCESS_TO_TEMPLATE":
                                case 103:
                                    m.ad_error = 103;
                                    break;
                                case "INCONSISTENT_EXPANDABLE_SETTINGS":
                                case 104:
                                    m.ad_error = 104;
                                    break;
                                case "INVALID_FORMAT":
                                case 105:
                                    m.ad_error = 105;
                                    break;
                                case "INVALID_FIELD_TEXT":
                                case 106:
                                    m.ad_error = 106;
                                    break;
                                case "ELEMENT_NOT_PRESENT":
                                case 107:
                                    m.ad_error = 107;
                                    break;
                                case "IMAGE_ERROR":
                                case 108:
                                    m.ad_error = 108;
                                    break;
                                case "VALUE_NOT_IN_RANGE":
                                case 109:
                                    m.ad_error = 109;
                                    break;
                                case "FIELD_NOT_PRESENT":
                                case 110:
                                    m.ad_error = 110;
                                    break;
                                case "ADDRESS_NOT_COMPLETE":
                                case 111:
                                    m.ad_error = 111;
                                    break;
                                case "ADDRESS_INVALID":
                                case 112:
                                    m.ad_error = 112;
                                    break;
                                case "VIDEO_RETRIEVAL_ERROR":
                                case 113:
                                    m.ad_error = 113;
                                    break;
                                case "AUDIO_ERROR":
                                case 114:
                                    m.ad_error = 114;
                                    break;
                                case "INVALID_YOUTUBE_DISPLAY_URL":
                                case 115:
                                    m.ad_error = 115;
                                    break;
                                case "TOO_MANY_PRODUCT_IMAGES":
                                case 116:
                                    m.ad_error = 116;
                                    break;
                                case "TOO_MANY_PRODUCT_VIDEOS":
                                case 117:
                                    m.ad_error = 117;
                                    break;
                                case "INCOMPATIBLE_AD_TYPE_AND_DEVICE_PREFERENCE":
                                case 118:
                                    m.ad_error = 118;
                                    break;
                                case "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY":
                                case 119:
                                    m.ad_error = 119;
                                    break;
                                case "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED":
                                case 120:
                                    m.ad_error = 120;
                                    break;
                                case "DISALLOWED_NUMBER_TYPE":
                                case 121:
                                    m.ad_error = 121;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY":
                                case 122:
                                    m.ad_error = 122;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY":
                                case 123:
                                    m.ad_error = 123;
                                    break;
                                case "PREMIUM_RATE_NUMBER_NOT_ALLOWED":
                                case 124:
                                    m.ad_error = 124;
                                    break;
                                case "VANITY_PHONE_NUMBER_NOT_ALLOWED":
                                case 125:
                                    m.ad_error = 125;
                                    break;
                                case "INVALID_CALL_CONVERSION_TYPE_ID":
                                case 126:
                                    m.ad_error = 126;
                                    break;
                                case "CANNOT_DISABLE_CALL_CONVERSION_AND_SET_CONVERSION_TYPE_ID":
                                case 127:
                                    m.ad_error = 127;
                                    break;
                                case "CANNOT_SET_PATH2_WITHOUT_PATH1":
                                case 128:
                                    m.ad_error = 128;
                                    break;
                                case "MISSING_DYNAMIC_SEARCH_ADS_SETTING_DOMAIN_NAME":
                                case 129:
                                    m.ad_error = 129;
                                    break;
                                case "INCOMPATIBLE_WITH_RESTRICTION_TYPE":
                                case 130:
                                    m.ad_error = 130;
                                    break;
                                case "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED":
                                case 131:
                                    m.ad_error = 131;
                                    break;
                                case "MISSING_IMAGE_OR_MEDIA_BUNDLE":
                                case 132:
                                    m.ad_error = 132;
                                    break;
                                case "PRODUCT_TYPE_NOT_SUPPORTED_IN_THIS_CAMPAIGN":
                                case 133:
                                    m.ad_error = 133;
                                    break;
                                case "PLACEHOLDER_CANNOT_HAVE_EMPTY_DEFAULT_VALUE":
                                case 134:
                                    m.ad_error = 134;
                                    break;
                                case "PLACEHOLDER_COUNTDOWN_FUNCTION_CANNOT_HAVE_DEFAULT_VALUE":
                                case 135:
                                    m.ad_error = 135;
                                    break;
                                case "PLACEHOLDER_DEFAULT_VALUE_MISSING":
                                case 136:
                                    m.ad_error = 136;
                                    break;
                                case "UNEXPECTED_PLACEHOLDER_DEFAULT_VALUE":
                                case 137:
                                    m.ad_error = 137;
                                    break;
                                case "AD_CUSTOMIZERS_MAY_NOT_BE_ADJACENT":
                                case 138:
                                    m.ad_error = 138;
                                    break;
                                case "UPDATING_AD_WITH_NO_ENABLED_ASSOCIATION":
                                case 139:
                                    m.ad_error = 139;
                                    break;
                                case "CALL_AD_VERIFICATION_URL_FINAL_URL_DOES_NOT_HAVE_SAME_DOMAIN":
                                case 140:
                                    m.ad_error = 140;
                                    break;
                                case "CALL_AD_FINAL_URL_AND_VERIFICATION_URL_CANNOT_BOTH_BE_EMPTY":
                                case 154:
                                    m.ad_error = 154;
                                    break;
                                case "TOO_MANY_AD_CUSTOMIZERS":
                                case 141:
                                    m.ad_error = 141;
                                    break;
                                case "INVALID_AD_CUSTOMIZER_FORMAT":
                                case 142:
                                    m.ad_error = 142;
                                    break;
                                case "NESTED_AD_CUSTOMIZER_SYNTAX":
                                case 143:
                                    m.ad_error = 143;
                                    break;
                                case "UNSUPPORTED_AD_CUSTOMIZER_SYNTAX":
                                case 144:
                                    m.ad_error = 144;
                                    break;
                                case "UNPAIRED_BRACE_IN_AD_CUSTOMIZER_TAG":
                                case 145:
                                    m.ad_error = 145;
                                    break;
                                case "MORE_THAN_ONE_COUNTDOWN_TAG_TYPE_EXISTS":
                                case 146:
                                    m.ad_error = 146;
                                    break;
                                case "DATE_TIME_IN_COUNTDOWN_TAG_IS_INVALID":
                                case 147:
                                    m.ad_error = 147;
                                    break;
                                case "DATE_TIME_IN_COUNTDOWN_TAG_IS_PAST":
                                case 148:
                                    m.ad_error = 148;
                                    break;
                                case "UNRECOGNIZED_AD_CUSTOMIZER_TAG_FOUND":
                                case 149:
                                    m.ad_error = 149;
                                    break;
                                case "CUSTOMIZER_TYPE_FORBIDDEN_FOR_FIELD":
                                case 150:
                                    m.ad_error = 150;
                                    break;
                                case "INVALID_CUSTOMIZER_ATTRIBUTE_NAME":
                                case 151:
                                    m.ad_error = 151;
                                    break;
                                case "STORE_MISMATCH":
                                case 152:
                                    m.ad_error = 152;
                                    break;
                                case "MISSING_REQUIRED_IMAGE_ASPECT_RATIO":
                                case 153:
                                    m.ad_error = 153;
                                    break;
                                case "MISMATCHED_ASPECT_RATIOS":
                                case 155:
                                    m.ad_error = 155;
                                    break;
                                case "DUPLICATE_IMAGE_ACROSS_CAROUSEL_CARDS":
                                case 156:
                                    m.ad_error = 156;
                                    break;
                                }
                                switch (d.ad_group_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_group_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_group_error = 1;
                                    break;
                                case "DUPLICATE_ADGROUP_NAME":
                                case 2:
                                    m.ad_group_error = 2;
                                    break;
                                case "INVALID_ADGROUP_NAME":
                                case 3:
                                    m.ad_group_error = 3;
                                    break;
                                case "ADVERTISER_NOT_ON_CONTENT_NETWORK":
                                case 5:
                                    m.ad_group_error = 5;
                                    break;
                                case "BID_TOO_BIG":
                                case 6:
                                    m.ad_group_error = 6;
                                    break;
                                case "BID_TYPE_AND_BIDDING_STRATEGY_MISMATCH":
                                case 7:
                                    m.ad_group_error = 7;
                                    break;
                                case "MISSING_ADGROUP_NAME":
                                case 8:
                                    m.ad_group_error = 8;
                                    break;
                                case "ADGROUP_LABEL_DOES_NOT_EXIST":
                                case 9:
                                    m.ad_group_error = 9;
                                    break;
                                case "ADGROUP_LABEL_ALREADY_EXISTS":
                                case 10:
                                    m.ad_group_error = 10;
                                    break;
                                case "INVALID_CONTENT_BID_CRITERION_TYPE_GROUP":
                                case 11:
                                    m.ad_group_error = 11;
                                    break;
                                case "AD_GROUP_TYPE_NOT_VALID_FOR_ADVERTISING_CHANNEL_TYPE":
                                case 12:
                                    m.ad_group_error = 12;
                                    break;
                                case "ADGROUP_TYPE_NOT_SUPPORTED_FOR_CAMPAIGN_SALES_COUNTRY":
                                case 13:
                                    m.ad_group_error = 13;
                                    break;
                                case "CANNOT_ADD_ADGROUP_OF_TYPE_DSA_TO_CAMPAIGN_WITHOUT_DSA_SETTING":
                                case 14:
                                    m.ad_group_error = 14;
                                    break;
                                case "PROMOTED_HOTEL_AD_GROUPS_NOT_AVAILABLE_FOR_CUSTOMER":
                                case 15:
                                    m.ad_group_error = 15;
                                    break;
                                case "INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE":
                                case 16:
                                    m.ad_group_error = 16;
                                    break;
                                }
                                switch (d.campaign_budget_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_budget_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_budget_error = 1;
                                    break;
                                case "CAMPAIGN_BUDGET_CANNOT_BE_SHARED":
                                case 17:
                                    m.campaign_budget_error = 17;
                                    break;
                                case "CAMPAIGN_BUDGET_REMOVED":
                                case 2:
                                    m.campaign_budget_error = 2;
                                    break;
                                case "CAMPAIGN_BUDGET_IN_USE":
                                case 3:
                                    m.campaign_budget_error = 3;
                                    break;
                                case "CAMPAIGN_BUDGET_PERIOD_NOT_AVAILABLE":
                                case 4:
                                    m.campaign_budget_error = 4;
                                    break;
                                case "CANNOT_MODIFY_FIELD_OF_IMPLICITLY_SHARED_CAMPAIGN_BUDGET":
                                case 6:
                                    m.campaign_budget_error = 6;
                                    break;
                                case "CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_IMPLICITLY_SHARED":
                                case 7:
                                    m.campaign_budget_error = 7;
                                    break;
                                case "CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED_WITHOUT_NAME":
                                case 8:
                                    m.campaign_budget_error = 8;
                                    break;
                                case "CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED":
                                case 9:
                                    m.campaign_budget_error = 9;
                                    break;
                                case "CANNOT_USE_IMPLICITLY_SHARED_CAMPAIGN_BUDGET_WITH_MULTIPLE_CAMPAIGNS":
                                case 10:
                                    m.campaign_budget_error = 10;
                                    break;
                                case "DUPLICATE_NAME":
                                case 11:
                                    m.campaign_budget_error = 11;
                                    break;
                                case "MONEY_AMOUNT_IN_WRONG_CURRENCY":
                                case 12:
                                    m.campaign_budget_error = 12;
                                    break;
                                case "MONEY_AMOUNT_LESS_THAN_CURRENCY_MINIMUM_CPC":
                                case 13:
                                    m.campaign_budget_error = 13;
                                    break;
                                case "MONEY_AMOUNT_TOO_LARGE":
                                case 14:
                                    m.campaign_budget_error = 14;
                                    break;
                                case "NEGATIVE_MONEY_AMOUNT":
                                case 15:
                                    m.campaign_budget_error = 15;
                                    break;
                                case "NON_MULTIPLE_OF_MINIMUM_CURRENCY_UNIT":
                                case 16:
                                    m.campaign_budget_error = 16;
                                    break;
                                case "TOTAL_BUDGET_AMOUNT_MUST_BE_UNSET_FOR_BUDGET_PERIOD_DAILY":
                                case 18:
                                    m.campaign_budget_error = 18;
                                    break;
                                case "INVALID_PERIOD":
                                case 19:
                                    m.campaign_budget_error = 19;
                                    break;
                                }
                                switch (d.campaign_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_error = 1;
                                    break;
                                case "CANNOT_TARGET_CONTENT_NETWORK":
                                case 3:
                                    m.campaign_error = 3;
                                    break;
                                case "CANNOT_TARGET_SEARCH_NETWORK":
                                case 4:
                                    m.campaign_error = 4;
                                    break;
                                case "CANNOT_TARGET_SEARCH_NETWORK_WITHOUT_GOOGLE_SEARCH":
                                case 5:
                                    m.campaign_error = 5;
                                    break;
                                case "CANNOT_TARGET_GOOGLE_SEARCH_FOR_CPM_CAMPAIGN":
                                case 6:
                                    m.campaign_error = 6;
                                    break;
                                case "CAMPAIGN_MUST_TARGET_AT_LEAST_ONE_NETWORK":
                                case 7:
                                    m.campaign_error = 7;
                                    break;
                                case "CANNOT_TARGET_PARTNER_SEARCH_NETWORK":
                                case 8:
                                    m.campaign_error = 8;
                                    break;
                                case "CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CRITERIA_LEVEL_BIDDING_STRATEGY":
                                case 9:
                                    m.campaign_error = 9;
                                    break;
                                case "CAMPAIGN_DURATION_MUST_CONTAIN_ALL_RUNNABLE_TRIALS":
                                case 10:
                                    m.campaign_error = 10;
                                    break;
                                case "CANNOT_MODIFY_FOR_TRIAL_CAMPAIGN":
                                case 11:
                                    m.campaign_error = 11;
                                    break;
                                case "DUPLICATE_CAMPAIGN_NAME":
                                case 12:
                                    m.campaign_error = 12;
                                    break;
                                case "INCOMPATIBLE_CAMPAIGN_FIELD":
                                case 13:
                                    m.campaign_error = 13;
                                    break;
                                case "INVALID_CAMPAIGN_NAME":
                                case 14:
                                    m.campaign_error = 14;
                                    break;
                                case "INVALID_AD_SERVING_OPTIMIZATION_STATUS":
                                case 15:
                                    m.campaign_error = 15;
                                    break;
                                case "INVALID_TRACKING_URL":
                                case 16:
                                    m.campaign_error = 16;
                                    break;
                                case "CANNOT_SET_BOTH_TRACKING_URL_TEMPLATE_AND_TRACKING_SETTING":
                                case 17:
                                    m.campaign_error = 17;
                                    break;
                                case "MAX_IMPRESSIONS_NOT_IN_RANGE":
                                case 18:
                                    m.campaign_error = 18;
                                    break;
                                case "TIME_UNIT_NOT_SUPPORTED":
                                case 19:
                                    m.campaign_error = 19;
                                    break;
                                case "INVALID_OPERATION_IF_SERVING_STATUS_HAS_ENDED":
                                case 20:
                                    m.campaign_error = 20;
                                    break;
                                case "BUDGET_CANNOT_BE_SHARED":
                                case 21:
                                    m.campaign_error = 21;
                                    break;
                                case "CAMPAIGN_CANNOT_USE_SHARED_BUDGET":
                                case 22:
                                    m.campaign_error = 22;
                                    break;
                                case "CANNOT_CHANGE_BUDGET_ON_CAMPAIGN_WITH_TRIALS":
                                case 23:
                                    m.campaign_error = 23;
                                    break;
                                case "CAMPAIGN_LABEL_DOES_NOT_EXIST":
                                case 24:
                                    m.campaign_error = 24;
                                    break;
                                case "CAMPAIGN_LABEL_ALREADY_EXISTS":
                                case 25:
                                    m.campaign_error = 25;
                                    break;
                                case "MISSING_SHOPPING_SETTING":
                                case 26:
                                    m.campaign_error = 26;
                                    break;
                                case "INVALID_SHOPPING_SALES_COUNTRY":
                                case 27:
                                    m.campaign_error = 27;
                                    break;
                                case "ADVERTISING_CHANNEL_TYPE_NOT_AVAILABLE_FOR_ACCOUNT_TYPE":
                                case 31:
                                    m.campaign_error = 31;
                                    break;
                                case "INVALID_ADVERTISING_CHANNEL_SUB_TYPE":
                                case 32:
                                    m.campaign_error = 32;
                                    break;
                                case "AT_LEAST_ONE_CONVERSION_MUST_BE_SELECTED":
                                case 33:
                                    m.campaign_error = 33;
                                    break;
                                case "CANNOT_SET_AD_ROTATION_MODE":
                                case 34:
                                    m.campaign_error = 34;
                                    break;
                                case "CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED":
                                case 35:
                                    m.campaign_error = 35;
                                    break;
                                case "CANNOT_SET_DATE_TO_PAST":
                                case 36:
                                    m.campaign_error = 36;
                                    break;
                                case "MISSING_HOTEL_CUSTOMER_LINK":
                                case 37:
                                    m.campaign_error = 37;
                                    break;
                                case "INVALID_HOTEL_CUSTOMER_LINK":
                                case 38:
                                    m.campaign_error = 38;
                                    break;
                                case "MISSING_HOTEL_SETTING":
                                case 39:
                                    m.campaign_error = 39;
                                    break;
                                case "CANNOT_USE_SHARED_CAMPAIGN_BUDGET_WHILE_PART_OF_CAMPAIGN_GROUP":
                                case 40:
                                    m.campaign_error = 40;
                                    break;
                                case "APP_NOT_FOUND":
                                case 41:
                                    m.campaign_error = 41;
                                    break;
                                case "SHOPPING_ENABLE_LOCAL_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE":
                                case 42:
                                    m.campaign_error = 42;
                                    break;
                                case "MERCHANT_NOT_ALLOWED_FOR_COMPARISON_LISTING_ADS":
                                case 43:
                                    m.campaign_error = 43;
                                    break;
                                case "INSUFFICIENT_APP_INSTALLS_COUNT":
                                case 44:
                                    m.campaign_error = 44;
                                    break;
                                case "SENSITIVE_CATEGORY_APP":
                                case 45:
                                    m.campaign_error = 45;
                                    break;
                                case "HEC_AGREEMENT_REQUIRED":
                                case 46:
                                    m.campaign_error = 46;
                                    break;
                                case "NOT_COMPATIBLE_WITH_VIEW_THROUGH_CONVERSION_OPTIMIZATION":
                                case 49:
                                    m.campaign_error = 49;
                                    break;
                                case "INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE":
                                case 48:
                                    m.campaign_error = 48;
                                    break;
                                case "CANNOT_CREATE_APP_PRE_REGISTRATION_FOR_NON_ANDROID_APP":
                                case 50:
                                    m.campaign_error = 50;
                                    break;
                                case "APP_NOT_AVAILABLE_TO_CREATE_APP_PRE_REGISTRATION_CAMPAIGN":
                                case 51:
                                    m.campaign_error = 51;
                                    break;
                                case "INCOMPATIBLE_BUDGET_TYPE":
                                case 52:
                                    m.campaign_error = 52;
                                    break;
                                case "LOCAL_SERVICES_DUPLICATE_CATEGORY_BID":
                                case 53:
                                    m.campaign_error = 53;
                                    break;
                                case "LOCAL_SERVICES_INVALID_CATEGORY_BID":
                                case 54:
                                    m.campaign_error = 54;
                                    break;
                                case "LOCAL_SERVICES_MISSING_CATEGORY_BID":
                                case 55:
                                    m.campaign_error = 55;
                                    break;
                                }
                                switch (d.authentication_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.authentication_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.authentication_error = 1;
                                    break;
                                case "AUTHENTICATION_ERROR":
                                case 2:
                                    m.authentication_error = 2;
                                    break;
                                case "CLIENT_CUSTOMER_ID_INVALID":
                                case 5:
                                    m.authentication_error = 5;
                                    break;
                                case "CUSTOMER_NOT_FOUND":
                                case 8:
                                    m.authentication_error = 8;
                                    break;
                                case "GOOGLE_ACCOUNT_DELETED":
                                case 9:
                                    m.authentication_error = 9;
                                    break;
                                case "GOOGLE_ACCOUNT_COOKIE_INVALID":
                                case 10:
                                    m.authentication_error = 10;
                                    break;
                                case "GOOGLE_ACCOUNT_AUTHENTICATION_FAILED":
                                case 25:
                                    m.authentication_error = 25;
                                    break;
                                case "GOOGLE_ACCOUNT_USER_AND_ADS_USER_MISMATCH":
                                case 12:
                                    m.authentication_error = 12;
                                    break;
                                case "LOGIN_COOKIE_REQUIRED":
                                case 13:
                                    m.authentication_error = 13;
                                    break;
                                case "NOT_ADS_USER":
                                case 14:
                                    m.authentication_error = 14;
                                    break;
                                case "OAUTH_TOKEN_INVALID":
                                case 15:
                                    m.authentication_error = 15;
                                    break;
                                case "OAUTH_TOKEN_EXPIRED":
                                case 16:
                                    m.authentication_error = 16;
                                    break;
                                case "OAUTH_TOKEN_DISABLED":
                                case 17:
                                    m.authentication_error = 17;
                                    break;
                                case "OAUTH_TOKEN_REVOKED":
                                case 18:
                                    m.authentication_error = 18;
                                    break;
                                case "OAUTH_TOKEN_HEADER_INVALID":
                                case 19:
                                    m.authentication_error = 19;
                                    break;
                                case "LOGIN_COOKIE_INVALID":
                                case 20:
                                    m.authentication_error = 20;
                                    break;
                                case "USER_ID_INVALID":
                                case 22:
                                    m.authentication_error = 22;
                                    break;
                                case "TWO_STEP_VERIFICATION_NOT_ENROLLED":
                                case 23:
                                    m.authentication_error = 23;
                                    break;
                                case "ADVANCED_PROTECTION_NOT_ENROLLED":
                                case 24:
                                    m.authentication_error = 24;
                                    break;
                                }
                                switch (d.ad_group_criterion_customizer_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_group_criterion_customizer_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_group_criterion_customizer_error = 1;
                                    break;
                                case "CRITERION_IS_NOT_KEYWORD":
                                case 2:
                                    m.ad_group_criterion_customizer_error = 2;
                                    break;
                                }
                                switch (d.ad_group_criterion_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_group_criterion_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_group_criterion_error = 1;
                                    break;
                                case "AD_GROUP_CRITERION_LABEL_DOES_NOT_EXIST":
                                case 2:
                                    m.ad_group_criterion_error = 2;
                                    break;
                                case "AD_GROUP_CRITERION_LABEL_ALREADY_EXISTS":
                                case 3:
                                    m.ad_group_criterion_error = 3;
                                    break;
                                case "CANNOT_ADD_LABEL_TO_NEGATIVE_CRITERION":
                                case 4:
                                    m.ad_group_criterion_error = 4;
                                    break;
                                case "TOO_MANY_OPERATIONS":
                                case 5:
                                    m.ad_group_criterion_error = 5;
                                    break;
                                case "CANT_UPDATE_NEGATIVE":
                                case 6:
                                    m.ad_group_criterion_error = 6;
                                    break;
                                case "CONCRETE_TYPE_REQUIRED":
                                case 7:
                                    m.ad_group_criterion_error = 7;
                                    break;
                                case "BID_INCOMPATIBLE_WITH_ADGROUP":
                                case 8:
                                    m.ad_group_criterion_error = 8;
                                    break;
                                case "CANNOT_TARGET_AND_EXCLUDE":
                                case 9:
                                    m.ad_group_criterion_error = 9;
                                    break;
                                case "ILLEGAL_URL":
                                case 10:
                                    m.ad_group_criterion_error = 10;
                                    break;
                                case "INVALID_KEYWORD_TEXT":
                                case 11:
                                    m.ad_group_criterion_error = 11;
                                    break;
                                case "INVALID_DESTINATION_URL":
                                case 12:
                                    m.ad_group_criterion_error = 12;
                                    break;
                                case "MISSING_DESTINATION_URL_TAG":
                                case 13:
                                    m.ad_group_criterion_error = 13;
                                    break;
                                case "KEYWORD_LEVEL_BID_NOT_SUPPORTED_FOR_MANUALCPM":
                                case 14:
                                    m.ad_group_criterion_error = 14;
                                    break;
                                case "INVALID_USER_STATUS":
                                case 15:
                                    m.ad_group_criterion_error = 15;
                                    break;
                                case "CANNOT_ADD_CRITERIA_TYPE":
                                case 16:
                                    m.ad_group_criterion_error = 16;
                                    break;
                                case "CANNOT_EXCLUDE_CRITERIA_TYPE":
                                case 17:
                                    m.ad_group_criterion_error = 17;
                                    break;
                                case "CAMPAIGN_TYPE_NOT_COMPATIBLE_WITH_PARTIAL_FAILURE":
                                case 27:
                                    m.ad_group_criterion_error = 27;
                                    break;
                                case "OPERATIONS_FOR_TOO_MANY_SHOPPING_ADGROUPS":
                                case 28:
                                    m.ad_group_criterion_error = 28;
                                    break;
                                case "CANNOT_MODIFY_URL_FIELDS_WITH_DUPLICATE_ELEMENTS":
                                case 29:
                                    m.ad_group_criterion_error = 29;
                                    break;
                                case "CANNOT_SET_WITHOUT_FINAL_URLS":
                                case 30:
                                    m.ad_group_criterion_error = 30;
                                    break;
                                case "CANNOT_CLEAR_FINAL_URLS_IF_FINAL_MOBILE_URLS_EXIST":
                                case 31:
                                    m.ad_group_criterion_error = 31;
                                    break;
                                case "CANNOT_CLEAR_FINAL_URLS_IF_FINAL_APP_URLS_EXIST":
                                case 32:
                                    m.ad_group_criterion_error = 32;
                                    break;
                                case "CANNOT_CLEAR_FINAL_URLS_IF_TRACKING_URL_TEMPLATE_EXISTS":
                                case 33:
                                    m.ad_group_criterion_error = 33;
                                    break;
                                case "CANNOT_CLEAR_FINAL_URLS_IF_URL_CUSTOM_PARAMETERS_EXIST":
                                case 34:
                                    m.ad_group_criterion_error = 34;
                                    break;
                                case "CANNOT_SET_BOTH_DESTINATION_URL_AND_FINAL_URLS":
                                case 35:
                                    m.ad_group_criterion_error = 35;
                                    break;
                                case "CANNOT_SET_BOTH_DESTINATION_URL_AND_TRACKING_URL_TEMPLATE":
                                case 36:
                                    m.ad_group_criterion_error = 36;
                                    break;
                                case "FINAL_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE":
                                case 37:
                                    m.ad_group_criterion_error = 37;
                                    break;
                                case "FINAL_MOBILE_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE":
                                case 38:
                                    m.ad_group_criterion_error = 38;
                                    break;
                                }
                                switch (d.ad_group_customizer_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_group_customizer_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_group_customizer_error = 1;
                                    break;
                                }
                                switch (d.ad_customizer_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_customizer_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_customizer_error = 1;
                                    break;
                                case "COUNTDOWN_INVALID_DATE_FORMAT":
                                case 2:
                                    m.ad_customizer_error = 2;
                                    break;
                                case "COUNTDOWN_DATE_IN_PAST":
                                case 3:
                                    m.ad_customizer_error = 3;
                                    break;
                                case "COUNTDOWN_INVALID_LOCALE":
                                case 4:
                                    m.ad_customizer_error = 4;
                                    break;
                                case "COUNTDOWN_INVALID_START_DAYS_BEFORE":
                                case 5:
                                    m.ad_customizer_error = 5;
                                    break;
                                case "UNKNOWN_USER_LIST":
                                case 6:
                                    m.ad_customizer_error = 6;
                                    break;
                                }
                                switch (d.ad_group_ad_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_group_ad_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_group_ad_error = 1;
                                    break;
                                case "AD_GROUP_AD_LABEL_DOES_NOT_EXIST":
                                case 2:
                                    m.ad_group_ad_error = 2;
                                    break;
                                case "AD_GROUP_AD_LABEL_ALREADY_EXISTS":
                                case 3:
                                    m.ad_group_ad_error = 3;
                                    break;
                                case "AD_NOT_UNDER_ADGROUP":
                                case 4:
                                    m.ad_group_ad_error = 4;
                                    break;
                                case "CANNOT_OPERATE_ON_REMOVED_ADGROUPAD":
                                case 5:
                                    m.ad_group_ad_error = 5;
                                    break;
                                case "CANNOT_CREATE_DEPRECATED_ADS":
                                case 6:
                                    m.ad_group_ad_error = 6;
                                    break;
                                case "CANNOT_CREATE_TEXT_ADS":
                                case 7:
                                    m.ad_group_ad_error = 7;
                                    break;
                                case "EMPTY_FIELD":
                                case 8:
                                    m.ad_group_ad_error = 8;
                                    break;
                                case "RESOURCE_REFERENCED_IN_MULTIPLE_OPS":
                                case 9:
                                    m.ad_group_ad_error = 9;
                                    break;
                                case "AD_TYPE_CANNOT_BE_PAUSED":
                                case 10:
                                    m.ad_group_ad_error = 10;
                                    break;
                                case "AD_TYPE_CANNOT_BE_REMOVED":
                                case 11:
                                    m.ad_group_ad_error = 11;
                                    break;
                                case "CANNOT_UPDATE_DEPRECATED_ADS":
                                case 12:
                                    m.ad_group_ad_error = 12;
                                    break;
                                }
                                switch (d.ad_sharing_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_sharing_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_sharing_error = 1;
                                    break;
                                case "AD_GROUP_ALREADY_CONTAINS_AD":
                                case 2:
                                    m.ad_sharing_error = 2;
                                    break;
                                case "INCOMPATIBLE_AD_UNDER_AD_GROUP":
                                case 3:
                                    m.ad_sharing_error = 3;
                                    break;
                                case "CANNOT_SHARE_INACTIVE_AD":
                                case 4:
                                    m.ad_sharing_error = 4;
                                    break;
                                }
                                switch (d.adx_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.adx_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.adx_error = 1;
                                    break;
                                case "UNSUPPORTED_FEATURE":
                                case 2:
                                    m.adx_error = 2;
                                    break;
                                }
                                switch (d.asset_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_error = 1;
                                    break;
                                case "CUSTOMER_NOT_ON_ALLOWLIST_FOR_ASSET_TYPE":
                                case 13:
                                    m.asset_error = 13;
                                    break;
                                case "DUPLICATE_ASSET":
                                case 3:
                                    m.asset_error = 3;
                                    break;
                                case "DUPLICATE_ASSET_NAME":
                                case 4:
                                    m.asset_error = 4;
                                    break;
                                case "ASSET_DATA_IS_MISSING":
                                case 5:
                                    m.asset_error = 5;
                                    break;
                                case "CANNOT_MODIFY_ASSET_NAME":
                                case 6:
                                    m.asset_error = 6;
                                    break;
                                case "FIELD_INCOMPATIBLE_WITH_ASSET_TYPE":
                                case 7:
                                    m.asset_error = 7;
                                    break;
                                case "INVALID_CALL_TO_ACTION_TEXT":
                                case 8:
                                    m.asset_error = 8;
                                    break;
                                case "LEAD_FORM_INVALID_FIELDS_COMBINATION":
                                case 9:
                                    m.asset_error = 9;
                                    break;
                                case "LEAD_FORM_MISSING_AGREEMENT":
                                case 10:
                                    m.asset_error = 10;
                                    break;
                                case "INVALID_ASSET_STATUS":
                                case 11:
                                    m.asset_error = 11;
                                    break;
                                case "FIELD_CANNOT_BE_MODIFIED_FOR_ASSET_TYPE":
                                case 12:
                                    m.asset_error = 12;
                                    break;
                                case "SCHEDULES_CANNOT_OVERLAP":
                                case 14:
                                    m.asset_error = 14;
                                    break;
                                case "PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF":
                                case 15:
                                    m.asset_error = 15;
                                    break;
                                case "PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT":
                                case 16:
                                    m.asset_error = 16;
                                    break;
                                case "TOO_MANY_DECIMAL_PLACES_SPECIFIED":
                                case 17:
                                    m.asset_error = 17;
                                    break;
                                case "DUPLICATE_ASSETS_WITH_DIFFERENT_FIELD_VALUE":
                                case 18:
                                    m.asset_error = 18;
                                    break;
                                case "CALL_CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED":
                                case 19:
                                    m.asset_error = 19;
                                    break;
                                case "CALL_CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED":
                                case 20:
                                    m.asset_error = 20;
                                    break;
                                case "CALL_DISALLOWED_NUMBER_TYPE":
                                case 21:
                                    m.asset_error = 21;
                                    break;
                                case "CALL_INVALID_CONVERSION_ACTION":
                                case 22:
                                    m.asset_error = 22;
                                    break;
                                case "CALL_INVALID_COUNTRY_CODE":
                                case 23:
                                    m.asset_error = 23;
                                    break;
                                case "CALL_INVALID_DOMESTIC_PHONE_NUMBER_FORMAT":
                                case 24:
                                    m.asset_error = 24;
                                    break;
                                case "CALL_INVALID_PHONE_NUMBER":
                                case 25:
                                    m.asset_error = 25;
                                    break;
                                case "CALL_PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY":
                                case 26:
                                    m.asset_error = 26;
                                    break;
                                case "CALL_PREMIUM_RATE_NUMBER_NOT_ALLOWED":
                                case 27:
                                    m.asset_error = 27;
                                    break;
                                case "CALL_VANITY_PHONE_NUMBER_NOT_ALLOWED":
                                case 28:
                                    m.asset_error = 28;
                                    break;
                                case "PRICE_HEADER_SAME_AS_DESCRIPTION":
                                case 29:
                                    m.asset_error = 29;
                                    break;
                                case "MOBILE_APP_INVALID_APP_ID":
                                case 30:
                                    m.asset_error = 30;
                                    break;
                                case "MOBILE_APP_INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL":
                                case 31:
                                    m.asset_error = 31;
                                    break;
                                case "NAME_REQUIRED_FOR_ASSET_TYPE":
                                case 32:
                                    m.asset_error = 32;
                                    break;
                                case "LEAD_FORM_LEGACY_QUALIFYING_QUESTIONS_DISALLOWED":
                                case 33:
                                    m.asset_error = 33;
                                    break;
                                case "NAME_CONFLICT_FOR_ASSET_TYPE":
                                case 34:
                                    m.asset_error = 34;
                                    break;
                                case "CANNOT_MODIFY_ASSET_SOURCE":
                                case 35:
                                    m.asset_error = 35;
                                    break;
                                case "CANNOT_MODIFY_AUTOMATICALLY_CREATED_ASSET":
                                case 36:
                                    m.asset_error = 36;
                                    break;
                                }
                                switch (d.asset_group_asset_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_group_asset_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_group_asset_error = 1;
                                    break;
                                case "DUPLICATE_RESOURCE":
                                case 2:
                                    m.asset_group_asset_error = 2;
                                    break;
                                case "EXPANDABLE_TAGS_NOT_ALLOWED_IN_DESCRIPTION":
                                case 3:
                                    m.asset_group_asset_error = 3;
                                    break;
                                case "AD_CUSTOMIZER_NOT_SUPPORTED":
                                case 4:
                                    m.asset_group_asset_error = 4;
                                    break;
                                }
                                switch (d.asset_group_listing_group_filter_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_group_listing_group_filter_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_group_listing_group_filter_error = 1;
                                    break;
                                case "TREE_TOO_DEEP":
                                case 2:
                                    m.asset_group_listing_group_filter_error = 2;
                                    break;
                                case "UNIT_CANNOT_HAVE_CHILDREN":
                                case 3:
                                    m.asset_group_listing_group_filter_error = 3;
                                    break;
                                case "SUBDIVISION_MUST_HAVE_EVERYTHING_ELSE_CHILD":
                                case 4:
                                    m.asset_group_listing_group_filter_error = 4;
                                    break;
                                case "DIFFERENT_DIMENSION_TYPE_BETWEEN_SIBLINGS":
                                case 5:
                                    m.asset_group_listing_group_filter_error = 5;
                                    break;
                                case "SAME_DIMENSION_VALUE_BETWEEN_SIBLINGS":
                                case 6:
                                    m.asset_group_listing_group_filter_error = 6;
                                    break;
                                case "SAME_DIMENSION_TYPE_BETWEEN_ANCESTORS":
                                case 7:
                                    m.asset_group_listing_group_filter_error = 7;
                                    break;
                                case "MULTIPLE_ROOTS":
                                case 8:
                                    m.asset_group_listing_group_filter_error = 8;
                                    break;
                                case "INVALID_DIMENSION_VALUE":
                                case 9:
                                    m.asset_group_listing_group_filter_error = 9;
                                    break;
                                case "MUST_REFINE_HIERARCHICAL_PARENT_TYPE":
                                case 10:
                                    m.asset_group_listing_group_filter_error = 10;
                                    break;
                                case "INVALID_PRODUCT_BIDDING_CATEGORY":
                                case 11:
                                    m.asset_group_listing_group_filter_error = 11;
                                    break;
                                case "CHANGING_CASE_VALUE_WITH_CHILDREN":
                                case 12:
                                    m.asset_group_listing_group_filter_error = 12;
                                    break;
                                case "SUBDIVISION_HAS_CHILDREN":
                                case 13:
                                    m.asset_group_listing_group_filter_error = 13;
                                    break;
                                case "CANNOT_REFINE_HIERARCHICAL_EVERYTHING_ELSE":
                                case 14:
                                    m.asset_group_listing_group_filter_error = 14;
                                    break;
                                }
                                switch (d.asset_group_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_group_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_group_error = 1;
                                    break;
                                case "DUPLICATE_NAME":
                                case 2:
                                    m.asset_group_error = 2;
                                    break;
                                case "CANNOT_ADD_ASSET_GROUP_FOR_CAMPAIGN_TYPE":
                                case 3:
                                    m.asset_group_error = 3;
                                    break;
                                case "NOT_ENOUGH_HEADLINE_ASSET":
                                case 4:
                                    m.asset_group_error = 4;
                                    break;
                                case "NOT_ENOUGH_LONG_HEADLINE_ASSET":
                                case 5:
                                    m.asset_group_error = 5;
                                    break;
                                case "NOT_ENOUGH_DESCRIPTION_ASSET":
                                case 6:
                                    m.asset_group_error = 6;
                                    break;
                                case "NOT_ENOUGH_BUSINESS_NAME_ASSET":
                                case 7:
                                    m.asset_group_error = 7;
                                    break;
                                case "NOT_ENOUGH_MARKETING_IMAGE_ASSET":
                                case 8:
                                    m.asset_group_error = 8;
                                    break;
                                case "NOT_ENOUGH_SQUARE_MARKETING_IMAGE_ASSET":
                                case 9:
                                    m.asset_group_error = 9;
                                    break;
                                case "NOT_ENOUGH_LOGO_ASSET":
                                case 10:
                                    m.asset_group_error = 10;
                                    break;
                                case "FINAL_URL_SHOPPING_MERCHANT_HOME_PAGE_URL_DOMAINS_DIFFER":
                                case 11:
                                    m.asset_group_error = 11;
                                    break;
                                }
                                switch (d.asset_set_asset_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_set_asset_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_set_asset_error = 1;
                                    break;
                                case "INVALID_ASSET_TYPE":
                                case 2:
                                    m.asset_set_asset_error = 2;
                                    break;
                                case "INVALID_ASSET_SET_TYPE":
                                case 3:
                                    m.asset_set_asset_error = 3;
                                    break;
                                case "DUPLICATE_EXTERNAL_KEY":
                                case 4:
                                    m.asset_set_asset_error = 4;
                                    break;
                                }
                                switch (d.asset_set_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_set_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_set_link_error = 1;
                                    break;
                                case "INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE":
                                case 2:
                                    m.asset_set_link_error = 2;
                                    break;
                                case "DUPLICATE_FEED_LINK":
                                case 3:
                                    m.asset_set_link_error = 3;
                                    break;
                                case "INCOMPATIBLE_ASSET_SET_TYPE_WITH_CAMPAIGN_TYPE":
                                case 4:
                                    m.asset_set_link_error = 4;
                                    break;
                                case "DUPLICATE_ASSET_SET_LINK":
                                case 5:
                                    m.asset_set_link_error = 5;
                                    break;
                                case "ASSET_SET_LINK_CANNOT_BE_REMOVED":
                                case 6:
                                    m.asset_set_link_error = 6;
                                    break;
                                }
                                switch (d.asset_set_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_set_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_set_error = 1;
                                    break;
                                case "DUPLICATE_ASSET_SET_NAME":
                                case 2:
                                    m.asset_set_error = 2;
                                    break;
                                }
                                switch (d.bidding_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.bidding_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.bidding_error = 1;
                                    break;
                                case "BIDDING_STRATEGY_TRANSITION_NOT_ALLOWED":
                                case 2:
                                    m.bidding_error = 2;
                                    break;
                                case "CANNOT_ATTACH_BIDDING_STRATEGY_TO_CAMPAIGN":
                                case 7:
                                    m.bidding_error = 7;
                                    break;
                                case "INVALID_ANONYMOUS_BIDDING_STRATEGY_TYPE":
                                case 10:
                                    m.bidding_error = 10;
                                    break;
                                case "INVALID_BIDDING_STRATEGY_TYPE":
                                case 14:
                                    m.bidding_error = 14;
                                    break;
                                case "INVALID_BID":
                                case 17:
                                    m.bidding_error = 17;
                                    break;
                                case "BIDDING_STRATEGY_NOT_AVAILABLE_FOR_ACCOUNT_TYPE":
                                case 18:
                                    m.bidding_error = 18;
                                    break;
                                case "CONVERSION_TRACKING_NOT_ENABLED":
                                case 19:
                                    m.bidding_error = 19;
                                    break;
                                case "NOT_ENOUGH_CONVERSIONS":
                                case 20:
                                    m.bidding_error = 20;
                                    break;
                                case "CANNOT_CREATE_CAMPAIGN_WITH_BIDDING_STRATEGY":
                                case 21:
                                    m.bidding_error = 21;
                                    break;
                                case "CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CAMPAIGN_LEVEL_POP_BIDDING_STRATEGY":
                                case 23:
                                    m.bidding_error = 23;
                                    break;
                                case "BIDDING_STRATEGY_NOT_SUPPORTED_WITH_AD_SCHEDULE":
                                case 24:
                                    m.bidding_error = 24;
                                    break;
                                case "PAY_PER_CONVERSION_NOT_AVAILABLE_FOR_CUSTOMER":
                                case 25:
                                    m.bidding_error = 25;
                                    break;
                                case "PAY_PER_CONVERSION_NOT_ALLOWED_WITH_TARGET_CPA":
                                case 26:
                                    m.bidding_error = 26;
                                    break;
                                case "BIDDING_STRATEGY_NOT_ALLOWED_FOR_SEARCH_ONLY_CAMPAIGNS":
                                case 27:
                                    m.bidding_error = 27;
                                    break;
                                case "BIDDING_STRATEGY_NOT_SUPPORTED_IN_DRAFTS_OR_EXPERIMENTS":
                                case 28:
                                    m.bidding_error = 28;
                                    break;
                                case "BIDDING_STRATEGY_TYPE_DOES_NOT_SUPPORT_PRODUCT_TYPE_ADGROUP_CRITERION":
                                case 29:
                                    m.bidding_error = 29;
                                    break;
                                case "BID_TOO_SMALL":
                                case 30:
                                    m.bidding_error = 30;
                                    break;
                                case "BID_TOO_BIG":
                                case 31:
                                    m.bidding_error = 31;
                                    break;
                                case "BID_TOO_MANY_FRACTIONAL_DIGITS":
                                case 32:
                                    m.bidding_error = 32;
                                    break;
                                case "INVALID_DOMAIN_NAME":
                                case 33:
                                    m.bidding_error = 33;
                                    break;
                                case "NOT_COMPATIBLE_WITH_PAYMENT_MODE":
                                case 34:
                                    m.bidding_error = 34;
                                    break;
                                case "NOT_COMPATIBLE_WITH_BUDGET_TYPE":
                                case 35:
                                    m.bidding_error = 35;
                                    break;
                                case "NOT_COMPATIBLE_WITH_BIDDING_STRATEGY_TYPE":
                                case 36:
                                    m.bidding_error = 36;
                                    break;
                                case "BIDDING_STRATEGY_TYPE_INCOMPATIBLE_WITH_SHARED_BUDGET":
                                case 37:
                                    m.bidding_error = 37;
                                    break;
                                case "BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ALIGNED":
                                case 38:
                                    m.bidding_error = 38;
                                    break;
                                case "BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ATTACHED_TO_THE_SAME_CAMPAIGNS_TO_ALIGN":
                                case 39:
                                    m.bidding_error = 39;
                                    break;
                                case "BIDDING_STRATEGY_AND_BUDGET_MUST_BE_REMOVED_TOGETHER":
                                case 40:
                                    m.bidding_error = 40;
                                    break;
                                }
                                switch (d.campaign_criterion_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_criterion_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_criterion_error = 1;
                                    break;
                                case "CONCRETE_TYPE_REQUIRED":
                                case 2:
                                    m.campaign_criterion_error = 2;
                                    break;
                                case "INVALID_PLACEMENT_URL":
                                case 3:
                                    m.campaign_criterion_error = 3;
                                    break;
                                case "CANNOT_EXCLUDE_CRITERIA_TYPE":
                                case 4:
                                    m.campaign_criterion_error = 4;
                                    break;
                                case "CANNOT_SET_STATUS_FOR_CRITERIA_TYPE":
                                case 5:
                                    m.campaign_criterion_error = 5;
                                    break;
                                case "CANNOT_SET_STATUS_FOR_EXCLUDED_CRITERIA":
                                case 6:
                                    m.campaign_criterion_error = 6;
                                    break;
                                case "CANNOT_TARGET_AND_EXCLUDE":
                                case 7:
                                    m.campaign_criterion_error = 7;
                                    break;
                                case "TOO_MANY_OPERATIONS":
                                case 8:
                                    m.campaign_criterion_error = 8;
                                    break;
                                case "OPERATOR_NOT_SUPPORTED_FOR_CRITERION_TYPE":
                                case 9:
                                    m.campaign_criterion_error = 9;
                                    break;
                                case "SHOPPING_CAMPAIGN_SALES_COUNTRY_NOT_SUPPORTED_FOR_SALES_CHANNEL":
                                case 10:
                                    m.campaign_criterion_error = 10;
                                    break;
                                case "CANNOT_ADD_EXISTING_FIELD":
                                case 11:
                                    m.campaign_criterion_error = 11;
                                    break;
                                case "CANNOT_UPDATE_NEGATIVE_CRITERION":
                                case 12:
                                    m.campaign_criterion_error = 12;
                                    break;
                                case "CANNOT_SET_NEGATIVE_KEYWORD_THEME_CONSTANT_CRITERION":
                                case 13:
                                    m.campaign_criterion_error = 13;
                                    break;
                                case "INVALID_KEYWORD_THEME_CONSTANT":
                                case 14:
                                    m.campaign_criterion_error = 14;
                                    break;
                                case "MISSING_KEYWORD_THEME_CONSTANT_OR_FREE_FORM_KEYWORD_THEME":
                                case 15:
                                    m.campaign_criterion_error = 15;
                                    break;
                                case "CANNOT_TARGET_BOTH_PROXIMITY_AND_LOCATION_CRITERIA_FOR_SMART_CAMPAIGN":
                                case 16:
                                    m.campaign_criterion_error = 16;
                                    break;
                                case "CANNOT_TARGET_MULTIPLE_PROXIMITY_CRITERIA_FOR_SMART_CAMPAIGN":
                                case 17:
                                    m.campaign_criterion_error = 17;
                                    break;
                                }
                                switch (d.campaign_conversion_goal_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_conversion_goal_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_conversion_goal_error = 1;
                                    break;
                                case "CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN":
                                case 2:
                                    m.campaign_conversion_goal_error = 2;
                                    break;
                                }
                                switch (d.campaign_customizer_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_customizer_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_customizer_error = 1;
                                    break;
                                }
                                switch (d.collection_size_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.collection_size_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.collection_size_error = 1;
                                    break;
                                case "TOO_FEW":
                                case 2:
                                    m.collection_size_error = 2;
                                    break;
                                case "TOO_MANY":
                                case 3:
                                    m.collection_size_error = 3;
                                    break;
                                }
                                switch (d.conversion_goal_campaign_config_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_goal_campaign_config_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_goal_campaign_config_error = 1;
                                    break;
                                case "CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN":
                                case 2:
                                    m.conversion_goal_campaign_config_error = 2;
                                    break;
                                case "CUSTOM_GOAL_DOES_NOT_BELONG_TO_GOOGLE_ADS_CONVERSION_CUSTOMER":
                                case 3:
                                    m.conversion_goal_campaign_config_error = 3;
                                    break;
                                case "CAMPAIGN_CANNOT_USE_UNIFIED_GOALS":
                                case 4:
                                    m.conversion_goal_campaign_config_error = 4;
                                    break;
                                }
                                switch (d.country_code_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.country_code_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.country_code_error = 1;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 2:
                                    m.country_code_error = 2;
                                    break;
                                }
                                switch (d.criterion_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.criterion_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.criterion_error = 1;
                                    break;
                                case "CONCRETE_TYPE_REQUIRED":
                                case 2:
                                    m.criterion_error = 2;
                                    break;
                                case "INVALID_EXCLUDED_CATEGORY":
                                case 3:
                                    m.criterion_error = 3;
                                    break;
                                case "INVALID_KEYWORD_TEXT":
                                case 4:
                                    m.criterion_error = 4;
                                    break;
                                case "KEYWORD_TEXT_TOO_LONG":
                                case 5:
                                    m.criterion_error = 5;
                                    break;
                                case "KEYWORD_HAS_TOO_MANY_WORDS":
                                case 6:
                                    m.criterion_error = 6;
                                    break;
                                case "KEYWORD_HAS_INVALID_CHARS":
                                case 7:
                                    m.criterion_error = 7;
                                    break;
                                case "INVALID_PLACEMENT_URL":
                                case 8:
                                    m.criterion_error = 8;
                                    break;
                                case "INVALID_USER_LIST":
                                case 9:
                                    m.criterion_error = 9;
                                    break;
                                case "INVALID_USER_INTEREST":
                                case 10:
                                    m.criterion_error = 10;
                                    break;
                                case "INVALID_FORMAT_FOR_PLACEMENT_URL":
                                case 11:
                                    m.criterion_error = 11;
                                    break;
                                case "PLACEMENT_URL_IS_TOO_LONG":
                                case 12:
                                    m.criterion_error = 12;
                                    break;
                                case "PLACEMENT_URL_HAS_ILLEGAL_CHAR":
                                case 13:
                                    m.criterion_error = 13;
                                    break;
                                case "PLACEMENT_URL_HAS_MULTIPLE_SITES_IN_LINE":
                                case 14:
                                    m.criterion_error = 14;
                                    break;
                                case "PLACEMENT_IS_NOT_AVAILABLE_FOR_TARGETING_OR_EXCLUSION":
                                case 15:
                                    m.criterion_error = 15;
                                    break;
                                case "INVALID_TOPIC_PATH":
                                case 16:
                                    m.criterion_error = 16;
                                    break;
                                case "INVALID_YOUTUBE_CHANNEL_ID":
                                case 17:
                                    m.criterion_error = 17;
                                    break;
                                case "INVALID_YOUTUBE_VIDEO_ID":
                                case 18:
                                    m.criterion_error = 18;
                                    break;
                                case "YOUTUBE_VERTICAL_CHANNEL_DEPRECATED":
                                case 19:
                                    m.criterion_error = 19;
                                    break;
                                case "YOUTUBE_DEMOGRAPHIC_CHANNEL_DEPRECATED":
                                case 20:
                                    m.criterion_error = 20;
                                    break;
                                case "YOUTUBE_URL_UNSUPPORTED":
                                case 21:
                                    m.criterion_error = 21;
                                    break;
                                case "CANNOT_EXCLUDE_CRITERIA_TYPE":
                                case 22:
                                    m.criterion_error = 22;
                                    break;
                                case "CANNOT_ADD_CRITERIA_TYPE":
                                case 23:
                                    m.criterion_error = 23;
                                    break;
                                case "CANNOT_EXCLUDE_SIMILAR_USER_LIST":
                                case 26:
                                    m.criterion_error = 26;
                                    break;
                                case "CANNOT_ADD_CLOSED_USER_LIST":
                                case 27:
                                    m.criterion_error = 27;
                                    break;
                                case "CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_ONLY_CAMPAIGNS":
                                case 28:
                                    m.criterion_error = 28;
                                    break;
                                case "CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_CAMPAIGNS":
                                case 29:
                                    m.criterion_error = 29;
                                    break;
                                case "CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SHOPPING_CAMPAIGNS":
                                case 30:
                                    m.criterion_error = 30;
                                    break;
                                case "CANNOT_ADD_USER_INTERESTS_TO_SEARCH_CAMPAIGNS":
                                case 31:
                                    m.criterion_error = 31;
                                    break;
                                case "CANNOT_SET_BIDS_ON_CRITERION_TYPE_IN_SEARCH_CAMPAIGNS":
                                case 32:
                                    m.criterion_error = 32;
                                    break;
                                case "CANNOT_ADD_URLS_TO_CRITERION_TYPE_FOR_CAMPAIGN_TYPE":
                                case 33:
                                    m.criterion_error = 33;
                                    break;
                                case "INVALID_COMBINED_AUDIENCE":
                                case 122:
                                    m.criterion_error = 122;
                                    break;
                                case "INVALID_CUSTOM_AFFINITY":
                                case 96:
                                    m.criterion_error = 96;
                                    break;
                                case "INVALID_CUSTOM_INTENT":
                                case 97:
                                    m.criterion_error = 97;
                                    break;
                                case "INVALID_CUSTOM_AUDIENCE":
                                case 121:
                                    m.criterion_error = 121;
                                    break;
                                case "INVALID_IP_ADDRESS":
                                case 34:
                                    m.criterion_error = 34;
                                    break;
                                case "INVALID_IP_FORMAT":
                                case 35:
                                    m.criterion_error = 35;
                                    break;
                                case "INVALID_MOBILE_APP":
                                case 36:
                                    m.criterion_error = 36;
                                    break;
                                case "INVALID_MOBILE_APP_CATEGORY":
                                case 37:
                                    m.criterion_error = 37;
                                    break;
                                case "INVALID_CRITERION_ID":
                                case 38:
                                    m.criterion_error = 38;
                                    break;
                                case "CANNOT_TARGET_CRITERION":
                                case 39:
                                    m.criterion_error = 39;
                                    break;
                                case "CANNOT_TARGET_OBSOLETE_CRITERION":
                                case 40:
                                    m.criterion_error = 40;
                                    break;
                                case "CRITERION_ID_AND_TYPE_MISMATCH":
                                case 41:
                                    m.criterion_error = 41;
                                    break;
                                case "INVALID_PROXIMITY_RADIUS":
                                case 42:
                                    m.criterion_error = 42;
                                    break;
                                case "INVALID_PROXIMITY_RADIUS_UNITS":
                                case 43:
                                    m.criterion_error = 43;
                                    break;
                                case "INVALID_STREETADDRESS_LENGTH":
                                case 44:
                                    m.criterion_error = 44;
                                    break;
                                case "INVALID_CITYNAME_LENGTH":
                                case 45:
                                    m.criterion_error = 45;
                                    break;
                                case "INVALID_REGIONCODE_LENGTH":
                                case 46:
                                    m.criterion_error = 46;
                                    break;
                                case "INVALID_REGIONNAME_LENGTH":
                                case 47:
                                    m.criterion_error = 47;
                                    break;
                                case "INVALID_POSTALCODE_LENGTH":
                                case 48:
                                    m.criterion_error = 48;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 49:
                                    m.criterion_error = 49;
                                    break;
                                case "INVALID_LATITUDE":
                                case 50:
                                    m.criterion_error = 50;
                                    break;
                                case "INVALID_LONGITUDE":
                                case 51:
                                    m.criterion_error = 51;
                                    break;
                                case "PROXIMITY_GEOPOINT_AND_ADDRESS_BOTH_CANNOT_BE_NULL":
                                case 52:
                                    m.criterion_error = 52;
                                    break;
                                case "INVALID_PROXIMITY_ADDRESS":
                                case 53:
                                    m.criterion_error = 53;
                                    break;
                                case "INVALID_USER_DOMAIN_NAME":
                                case 54:
                                    m.criterion_error = 54;
                                    break;
                                case "CRITERION_PARAMETER_TOO_LONG":
                                case 55:
                                    m.criterion_error = 55;
                                    break;
                                case "AD_SCHEDULE_TIME_INTERVALS_OVERLAP":
                                case 56:
                                    m.criterion_error = 56;
                                    break;
                                case "AD_SCHEDULE_INTERVAL_CANNOT_SPAN_MULTIPLE_DAYS":
                                case 57:
                                    m.criterion_error = 57;
                                    break;
                                case "AD_SCHEDULE_INVALID_TIME_INTERVAL":
                                case 58:
                                    m.criterion_error = 58;
                                    break;
                                case "AD_SCHEDULE_EXCEEDED_INTERVALS_PER_DAY_LIMIT":
                                case 59:
                                    m.criterion_error = 59;
                                    break;
                                case "AD_SCHEDULE_CRITERION_ID_MISMATCHING_FIELDS":
                                case 60:
                                    m.criterion_error = 60;
                                    break;
                                case "CANNOT_BID_MODIFY_CRITERION_TYPE":
                                case 61:
                                    m.criterion_error = 61;
                                    break;
                                case "CANNOT_BID_MODIFY_CRITERION_CAMPAIGN_OPTED_OUT":
                                case 62:
                                    m.criterion_error = 62;
                                    break;
                                case "CANNOT_BID_MODIFY_NEGATIVE_CRITERION":
                                case 63:
                                    m.criterion_error = 63;
                                    break;
                                case "BID_MODIFIER_ALREADY_EXISTS":
                                case 64:
                                    m.criterion_error = 64;
                                    break;
                                case "FEED_ID_NOT_ALLOWED":
                                case 65:
                                    m.criterion_error = 65;
                                    break;
                                case "ACCOUNT_INELIGIBLE_FOR_CRITERIA_TYPE":
                                case 66:
                                    m.criterion_error = 66;
                                    break;
                                case "CRITERIA_TYPE_INVALID_FOR_BIDDING_STRATEGY":
                                case 67:
                                    m.criterion_error = 67;
                                    break;
                                case "CANNOT_EXCLUDE_CRITERION":
                                case 68:
                                    m.criterion_error = 68;
                                    break;
                                case "CANNOT_REMOVE_CRITERION":
                                case 69:
                                    m.criterion_error = 69;
                                    break;
                                case "INVALID_PRODUCT_BIDDING_CATEGORY":
                                case 76:
                                    m.criterion_error = 76;
                                    break;
                                case "MISSING_SHOPPING_SETTING":
                                case 77:
                                    m.criterion_error = 77;
                                    break;
                                case "INVALID_MATCHING_FUNCTION":
                                case 78:
                                    m.criterion_error = 78;
                                    break;
                                case "LOCATION_FILTER_NOT_ALLOWED":
                                case 79:
                                    m.criterion_error = 79;
                                    break;
                                case "INVALID_FEED_FOR_LOCATION_FILTER":
                                case 98:
                                    m.criterion_error = 98;
                                    break;
                                case "LOCATION_FILTER_INVALID":
                                case 80:
                                    m.criterion_error = 80;
                                    break;
                                case "CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_FEED_ITEM_SETS":
                                case 123:
                                    m.criterion_error = 123;
                                    break;
                                case "INVALID_LOCATION_GROUP_RADIUS":
                                case 124:
                                    m.criterion_error = 124;
                                    break;
                                case "INVALID_LOCATION_GROUP_RADIUS_UNIT":
                                case 125:
                                    m.criterion_error = 125;
                                    break;
                                case "CANNOT_ATTACH_CRITERIA_AT_CAMPAIGN_AND_ADGROUP":
                                case 81:
                                    m.criterion_error = 81;
                                    break;
                                case "HOTEL_LENGTH_OF_STAY_OVERLAPS_WITH_EXISTING_CRITERION":
                                case 82:
                                    m.criterion_error = 82;
                                    break;
                                case "HOTEL_ADVANCE_BOOKING_WINDOW_OVERLAPS_WITH_EXISTING_CRITERION":
                                case 83:
                                    m.criterion_error = 83;
                                    break;
                                case "FIELD_INCOMPATIBLE_WITH_NEGATIVE_TARGETING":
                                case 84:
                                    m.criterion_error = 84;
                                    break;
                                case "INVALID_WEBPAGE_CONDITION":
                                case 85:
                                    m.criterion_error = 85;
                                    break;
                                case "INVALID_WEBPAGE_CONDITION_URL":
                                case 86:
                                    m.criterion_error = 86;
                                    break;
                                case "WEBPAGE_CONDITION_URL_CANNOT_BE_EMPTY":
                                case 87:
                                    m.criterion_error = 87;
                                    break;
                                case "WEBPAGE_CONDITION_URL_UNSUPPORTED_PROTOCOL":
                                case 88:
                                    m.criterion_error = 88;
                                    break;
                                case "WEBPAGE_CONDITION_URL_CANNOT_BE_IP_ADDRESS":
                                case 89:
                                    m.criterion_error = 89;
                                    break;
                                case "WEBPAGE_CONDITION_URL_DOMAIN_NOT_CONSISTENT_WITH_CAMPAIGN_SETTING":
                                case 90:
                                    m.criterion_error = 90;
                                    break;
                                case "WEBPAGE_CONDITION_URL_CANNOT_BE_PUBLIC_SUFFIX":
                                case 91:
                                    m.criterion_error = 91;
                                    break;
                                case "WEBPAGE_CONDITION_URL_INVALID_PUBLIC_SUFFIX":
                                case 92:
                                    m.criterion_error = 92;
                                    break;
                                case "WEBPAGE_CONDITION_URL_VALUE_TRACK_VALUE_NOT_SUPPORTED":
                                case 93:
                                    m.criterion_error = 93;
                                    break;
                                case "WEBPAGE_CRITERION_URL_EQUALS_CAN_HAVE_ONLY_ONE_CONDITION":
                                case 94:
                                    m.criterion_error = 94;
                                    break;
                                case "WEBPAGE_CRITERION_NOT_SUPPORTED_ON_NON_DSA_AD_GROUP":
                                case 95:
                                    m.criterion_error = 95;
                                    break;
                                case "CANNOT_TARGET_USER_LIST_FOR_SMART_DISPLAY_CAMPAIGNS":
                                case 99:
                                    m.criterion_error = 99;
                                    break;
                                case "CANNOT_TARGET_PLACEMENTS_FOR_SEARCH_CAMPAIGNS":
                                case 126:
                                    m.criterion_error = 126;
                                    break;
                                case "LISTING_SCOPE_TOO_MANY_DIMENSION_TYPES":
                                case 100:
                                    m.criterion_error = 100;
                                    break;
                                case "LISTING_SCOPE_TOO_MANY_IN_OPERATORS":
                                case 101:
                                    m.criterion_error = 101;
                                    break;
                                case "LISTING_SCOPE_IN_OPERATOR_NOT_SUPPORTED":
                                case 102:
                                    m.criterion_error = 102;
                                    break;
                                case "DUPLICATE_LISTING_DIMENSION_TYPE":
                                case 103:
                                    m.criterion_error = 103;
                                    break;
                                case "DUPLICATE_LISTING_DIMENSION_VALUE":
                                case 104:
                                    m.criterion_error = 104;
                                    break;
                                case "CANNOT_SET_BIDS_ON_LISTING_GROUP_SUBDIVISION":
                                case 105:
                                    m.criterion_error = 105;
                                    break;
                                case "INVALID_LISTING_GROUP_HIERARCHY":
                                case 106:
                                    m.criterion_error = 106;
                                    break;
                                case "LISTING_GROUP_UNIT_CANNOT_HAVE_CHILDREN":
                                case 107:
                                    m.criterion_error = 107;
                                    break;
                                case "LISTING_GROUP_SUBDIVISION_REQUIRES_OTHERS_CASE":
                                case 108:
                                    m.criterion_error = 108;
                                    break;
                                case "LISTING_GROUP_REQUIRES_SAME_DIMENSION_TYPE_AS_SIBLINGS":
                                case 109:
                                    m.criterion_error = 109;
                                    break;
                                case "LISTING_GROUP_ALREADY_EXISTS":
                                case 110:
                                    m.criterion_error = 110;
                                    break;
                                case "LISTING_GROUP_DOES_NOT_EXIST":
                                case 111:
                                    m.criterion_error = 111;
                                    break;
                                case "LISTING_GROUP_CANNOT_BE_REMOVED":
                                case 112:
                                    m.criterion_error = 112;
                                    break;
                                case "INVALID_LISTING_GROUP_TYPE":
                                case 113:
                                    m.criterion_error = 113;
                                    break;
                                case "LISTING_GROUP_ADD_MAY_ONLY_USE_TEMP_ID":
                                case 114:
                                    m.criterion_error = 114;
                                    break;
                                case "LISTING_SCOPE_TOO_LONG":
                                case 115:
                                    m.criterion_error = 115;
                                    break;
                                case "LISTING_SCOPE_TOO_MANY_DIMENSIONS":
                                case 116:
                                    m.criterion_error = 116;
                                    break;
                                case "LISTING_GROUP_TOO_LONG":
                                case 117:
                                    m.criterion_error = 117;
                                    break;
                                case "LISTING_GROUP_TREE_TOO_DEEP":
                                case 118:
                                    m.criterion_error = 118;
                                    break;
                                case "INVALID_LISTING_DIMENSION":
                                case 119:
                                    m.criterion_error = 119;
                                    break;
                                case "INVALID_LISTING_DIMENSION_TYPE":
                                case 120:
                                    m.criterion_error = 120;
                                    break;
                                case "ADVERTISER_NOT_ON_ALLOWLIST_FOR_COMBINED_AUDIENCE_ON_DISPLAY":
                                case 127:
                                    m.criterion_error = 127;
                                    break;
                                case "CANNOT_TARGET_REMOVED_COMBINED_AUDIENCE":
                                case 128:
                                    m.criterion_error = 128;
                                    break;
                                case "INVALID_COMBINED_AUDIENCE_ID":
                                case 129:
                                    m.criterion_error = 129;
                                    break;
                                case "CANNOT_TARGET_REMOVED_CUSTOM_AUDIENCE":
                                case 130:
                                    m.criterion_error = 130;
                                    break;
                                case "HOTEL_CHECK_IN_DATE_RANGE_OVERLAPS_WITH_EXISTING_CRITERION":
                                case 131:
                                    m.criterion_error = 131;
                                    break;
                                case "HOTEL_CHECK_IN_DATE_RANGE_START_DATE_TOO_EARLY":
                                case 132:
                                    m.criterion_error = 132;
                                    break;
                                case "HOTEL_CHECK_IN_DATE_RANGE_END_DATE_TOO_LATE":
                                case 133:
                                    m.criterion_error = 133;
                                    break;
                                case "HOTEL_CHECK_IN_DATE_RANGE_REVERSED":
                                case 134:
                                    m.criterion_error = 134;
                                    break;
                                case "BROAD_MATCH_MODIFIER_KEYWORD_NOT_ALLOWED":
                                case 135:
                                    m.criterion_error = 135;
                                    break;
                                case "ONE_AUDIENCE_ALLOWED_PER_ASSET_GROUP":
                                case 136:
                                    m.criterion_error = 136;
                                    break;
                                case "AUDIENCE_NOT_ELIGIBLE_FOR_CAMPAIGN_TYPE":
                                case 137:
                                    m.criterion_error = 137;
                                    break;
                                case "AUDIENCE_NOT_ALLOWED_TO_ATTACH_WHEN_AUDIENCE_GROUPED_SET_TO_FALSE":
                                case 138:
                                    m.criterion_error = 138;
                                    break;
                                case "CANNOT_TARGET_CUSTOMER_MATCH_USER_LIST":
                                case 139:
                                    m.criterion_error = 139;
                                    break;
                                }
                                switch (d.custom_conversion_goal_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.custom_conversion_goal_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.custom_conversion_goal_error = 1;
                                    break;
                                case "INVALID_CONVERSION_ACTION":
                                case 2:
                                    m.custom_conversion_goal_error = 2;
                                    break;
                                case "CONVERSION_ACTION_NOT_ENABLED":
                                case 3:
                                    m.custom_conversion_goal_error = 3;
                                    break;
                                case "CANNOT_REMOVE_LINKED_CUSTOM_CONVERSION_GOAL":
                                case 4:
                                    m.custom_conversion_goal_error = 4;
                                    break;
                                case "CUSTOM_GOAL_DUPLICATE_NAME":
                                case 5:
                                    m.custom_conversion_goal_error = 5;
                                    break;
                                case "DUPLICATE_CONVERSION_ACTION_LIST":
                                case 6:
                                    m.custom_conversion_goal_error = 6;
                                    break;
                                }
                                switch (d.customer_customizer_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.customer_customizer_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.customer_customizer_error = 1;
                                    break;
                                }
                                switch (d.customer_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.customer_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.customer_error = 1;
                                    break;
                                case "STATUS_CHANGE_DISALLOWED":
                                case 2:
                                    m.customer_error = 2;
                                    break;
                                case "ACCOUNT_NOT_SET_UP":
                                case 3:
                                    m.customer_error = 3;
                                    break;
                                }
                                switch (d.customizer_attribute_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.customizer_attribute_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.customizer_attribute_error = 1;
                                    break;
                                case "DUPLICATE_CUSTOMIZER_ATTRIBUTE_NAME":
                                case 2:
                                    m.customizer_attribute_error = 2;
                                    break;
                                }
                                switch (d.date_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.date_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.date_error = 1;
                                    break;
                                case "INVALID_FIELD_VALUES_IN_DATE":
                                case 2:
                                    m.date_error = 2;
                                    break;
                                case "INVALID_FIELD_VALUES_IN_DATE_TIME":
                                case 3:
                                    m.date_error = 3;
                                    break;
                                case "INVALID_STRING_DATE":
                                case 4:
                                    m.date_error = 4;
                                    break;
                                case "INVALID_STRING_DATE_TIME_MICROS":
                                case 6:
                                    m.date_error = 6;
                                    break;
                                case "INVALID_STRING_DATE_TIME_SECONDS":
                                case 11:
                                    m.date_error = 11;
                                    break;
                                case "INVALID_STRING_DATE_TIME_SECONDS_WITH_OFFSET":
                                case 12:
                                    m.date_error = 12;
                                    break;
                                case "EARLIER_THAN_MINIMUM_DATE":
                                case 7:
                                    m.date_error = 7;
                                    break;
                                case "LATER_THAN_MAXIMUM_DATE":
                                case 8:
                                    m.date_error = 8;
                                    break;
                                case "DATE_RANGE_MINIMUM_DATE_LATER_THAN_MAXIMUM_DATE":
                                case 9:
                                    m.date_error = 9;
                                    break;
                                case "DATE_RANGE_MINIMUM_AND_MAXIMUM_DATES_BOTH_NULL":
                                case 10:
                                    m.date_error = 10;
                                    break;
                                }
                                switch (d.date_range_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.date_range_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.date_range_error = 1;
                                    break;
                                case "INVALID_DATE":
                                case 2:
                                    m.date_range_error = 2;
                                    break;
                                case "START_DATE_AFTER_END_DATE":
                                case 3:
                                    m.date_range_error = 3;
                                    break;
                                case "CANNOT_SET_DATE_TO_PAST":
                                case 4:
                                    m.date_range_error = 4;
                                    break;
                                case "AFTER_MAXIMUM_ALLOWABLE_DATE":
                                case 5:
                                    m.date_range_error = 5;
                                    break;
                                case "CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED":
                                case 6:
                                    m.date_range_error = 6;
                                    break;
                                }
                                switch (d.distinct_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.distinct_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.distinct_error = 1;
                                    break;
                                case "DUPLICATE_ELEMENT":
                                case 2:
                                    m.distinct_error = 2;
                                    break;
                                case "DUPLICATE_TYPE":
                                case 3:
                                    m.distinct_error = 3;
                                    break;
                                }
                                switch (d.feed_attribute_reference_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_attribute_reference_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_attribute_reference_error = 1;
                                    break;
                                case "CANNOT_REFERENCE_REMOVED_FEED":
                                case 2:
                                    m.feed_attribute_reference_error = 2;
                                    break;
                                case "INVALID_FEED_NAME":
                                case 3:
                                    m.feed_attribute_reference_error = 3;
                                    break;
                                case "INVALID_FEED_ATTRIBUTE_NAME":
                                case 4:
                                    m.feed_attribute_reference_error = 4;
                                    break;
                                }
                                switch (d.function_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.function_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.function_error = 1;
                                    break;
                                case "INVALID_FUNCTION_FORMAT":
                                case 2:
                                    m.function_error = 2;
                                    break;
                                case "DATA_TYPE_MISMATCH":
                                case 3:
                                    m.function_error = 3;
                                    break;
                                case "INVALID_CONJUNCTION_OPERANDS":
                                case 4:
                                    m.function_error = 4;
                                    break;
                                case "INVALID_NUMBER_OF_OPERANDS":
                                case 5:
                                    m.function_error = 5;
                                    break;
                                case "INVALID_OPERAND_TYPE":
                                case 6:
                                    m.function_error = 6;
                                    break;
                                case "INVALID_OPERATOR":
                                case 7:
                                    m.function_error = 7;
                                    break;
                                case "INVALID_REQUEST_CONTEXT_TYPE":
                                case 8:
                                    m.function_error = 8;
                                    break;
                                case "INVALID_FUNCTION_FOR_CALL_PLACEHOLDER":
                                case 9:
                                    m.function_error = 9;
                                    break;
                                case "INVALID_FUNCTION_FOR_PLACEHOLDER":
                                case 10:
                                    m.function_error = 10;
                                    break;
                                case "INVALID_OPERAND":
                                case 11:
                                    m.function_error = 11;
                                    break;
                                case "MISSING_CONSTANT_OPERAND_VALUE":
                                case 12:
                                    m.function_error = 12;
                                    break;
                                case "INVALID_CONSTANT_OPERAND_VALUE":
                                case 13:
                                    m.function_error = 13;
                                    break;
                                case "INVALID_NESTING":
                                case 14:
                                    m.function_error = 14;
                                    break;
                                case "MULTIPLE_FEED_IDS_NOT_SUPPORTED":
                                case 15:
                                    m.function_error = 15;
                                    break;
                                case "INVALID_FUNCTION_FOR_FEED_WITH_FIXED_SCHEMA":
                                case 16:
                                    m.function_error = 16;
                                    break;
                                case "INVALID_ATTRIBUTE_NAME":
                                case 17:
                                    m.function_error = 17;
                                    break;
                                }
                                switch (d.function_parsing_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.function_parsing_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.function_parsing_error = 1;
                                    break;
                                case "NO_MORE_INPUT":
                                case 2:
                                    m.function_parsing_error = 2;
                                    break;
                                case "EXPECTED_CHARACTER":
                                case 3:
                                    m.function_parsing_error = 3;
                                    break;
                                case "UNEXPECTED_SEPARATOR":
                                case 4:
                                    m.function_parsing_error = 4;
                                    break;
                                case "UNMATCHED_LEFT_BRACKET":
                                case 5:
                                    m.function_parsing_error = 5;
                                    break;
                                case "UNMATCHED_RIGHT_BRACKET":
                                case 6:
                                    m.function_parsing_error = 6;
                                    break;
                                case "TOO_MANY_NESTED_FUNCTIONS":
                                case 7:
                                    m.function_parsing_error = 7;
                                    break;
                                case "MISSING_RIGHT_HAND_OPERAND":
                                case 8:
                                    m.function_parsing_error = 8;
                                    break;
                                case "INVALID_OPERATOR_NAME":
                                case 9:
                                    m.function_parsing_error = 9;
                                    break;
                                case "FEED_ATTRIBUTE_OPERAND_ARGUMENT_NOT_INTEGER":
                                case 10:
                                    m.function_parsing_error = 10;
                                    break;
                                case "NO_OPERANDS":
                                case 11:
                                    m.function_parsing_error = 11;
                                    break;
                                case "TOO_MANY_OPERANDS":
                                case 12:
                                    m.function_parsing_error = 12;
                                    break;
                                }
                                switch (d.id_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.id_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.id_error = 1;
                                    break;
                                case "NOT_FOUND":
                                case 2:
                                    m.id_error = 2;
                                    break;
                                }
                                switch (d.image_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.image_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.image_error = 1;
                                    break;
                                case "INVALID_IMAGE":
                                case 2:
                                    m.image_error = 2;
                                    break;
                                case "STORAGE_ERROR":
                                case 3:
                                    m.image_error = 3;
                                    break;
                                case "BAD_REQUEST":
                                case 4:
                                    m.image_error = 4;
                                    break;
                                case "UNEXPECTED_SIZE":
                                case 5:
                                    m.image_error = 5;
                                    break;
                                case "ANIMATED_NOT_ALLOWED":
                                case 6:
                                    m.image_error = 6;
                                    break;
                                case "ANIMATION_TOO_LONG":
                                case 7:
                                    m.image_error = 7;
                                    break;
                                case "SERVER_ERROR":
                                case 8:
                                    m.image_error = 8;
                                    break;
                                case "CMYK_JPEG_NOT_ALLOWED":
                                case 9:
                                    m.image_error = 9;
                                    break;
                                case "FLASH_NOT_ALLOWED":
                                case 10:
                                    m.image_error = 10;
                                    break;
                                case "FLASH_WITHOUT_CLICKTAG":
                                case 11:
                                    m.image_error = 11;
                                    break;
                                case "FLASH_ERROR_AFTER_FIXING_CLICK_TAG":
                                case 12:
                                    m.image_error = 12;
                                    break;
                                case "ANIMATED_VISUAL_EFFECT":
                                case 13:
                                    m.image_error = 13;
                                    break;
                                case "FLASH_ERROR":
                                case 14:
                                    m.image_error = 14;
                                    break;
                                case "LAYOUT_PROBLEM":
                                case 15:
                                    m.image_error = 15;
                                    break;
                                case "PROBLEM_READING_IMAGE_FILE":
                                case 16:
                                    m.image_error = 16;
                                    break;
                                case "ERROR_STORING_IMAGE":
                                case 17:
                                    m.image_error = 17;
                                    break;
                                case "ASPECT_RATIO_NOT_ALLOWED":
                                case 18:
                                    m.image_error = 18;
                                    break;
                                case "FLASH_HAS_NETWORK_OBJECTS":
                                case 19:
                                    m.image_error = 19;
                                    break;
                                case "FLASH_HAS_NETWORK_METHODS":
                                case 20:
                                    m.image_error = 20;
                                    break;
                                case "FLASH_HAS_URL":
                                case 21:
                                    m.image_error = 21;
                                    break;
                                case "FLASH_HAS_MOUSE_TRACKING":
                                case 22:
                                    m.image_error = 22;
                                    break;
                                case "FLASH_HAS_RANDOM_NUM":
                                case 23:
                                    m.image_error = 23;
                                    break;
                                case "FLASH_SELF_TARGETS":
                                case 24:
                                    m.image_error = 24;
                                    break;
                                case "FLASH_BAD_GETURL_TARGET":
                                case 25:
                                    m.image_error = 25;
                                    break;
                                case "FLASH_VERSION_NOT_SUPPORTED":
                                case 26:
                                    m.image_error = 26;
                                    break;
                                case "FLASH_WITHOUT_HARD_CODED_CLICK_URL":
                                case 27:
                                    m.image_error = 27;
                                    break;
                                case "INVALID_FLASH_FILE":
                                case 28:
                                    m.image_error = 28;
                                    break;
                                case "FAILED_TO_FIX_CLICK_TAG_IN_FLASH":
                                case 29:
                                    m.image_error = 29;
                                    break;
                                case "FLASH_ACCESSES_NETWORK_RESOURCES":
                                case 30:
                                    m.image_error = 30;
                                    break;
                                case "FLASH_EXTERNAL_JS_CALL":
                                case 31:
                                    m.image_error = 31;
                                    break;
                                case "FLASH_EXTERNAL_FS_CALL":
                                case 32:
                                    m.image_error = 32;
                                    break;
                                case "FILE_TOO_LARGE":
                                case 33:
                                    m.image_error = 33;
                                    break;
                                case "IMAGE_DATA_TOO_LARGE":
                                case 34:
                                    m.image_error = 34;
                                    break;
                                case "IMAGE_PROCESSING_ERROR":
                                case 35:
                                    m.image_error = 35;
                                    break;
                                case "IMAGE_TOO_SMALL":
                                case 36:
                                    m.image_error = 36;
                                    break;
                                case "INVALID_INPUT":
                                case 37:
                                    m.image_error = 37;
                                    break;
                                case "PROBLEM_READING_FILE":
                                case 38:
                                    m.image_error = 38;
                                    break;
                                case "IMAGE_CONSTRAINTS_VIOLATED":
                                case 39:
                                    m.image_error = 39;
                                    break;
                                case "FORMAT_NOT_ALLOWED":
                                case 40:
                                    m.image_error = 40;
                                    break;
                                }
                                switch (d.language_code_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.language_code_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.language_code_error = 1;
                                    break;
                                case "LANGUAGE_CODE_NOT_FOUND":
                                case 2:
                                    m.language_code_error = 2;
                                    break;
                                case "INVALID_LANGUAGE_CODE":
                                case 3:
                                    m.language_code_error = 3;
                                    break;
                                }
                                switch (d.media_bundle_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.media_bundle_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.media_bundle_error = 1;
                                    break;
                                case "BAD_REQUEST":
                                case 3:
                                    m.media_bundle_error = 3;
                                    break;
                                case "DOUBLECLICK_BUNDLE_NOT_ALLOWED":
                                case 4:
                                    m.media_bundle_error = 4;
                                    break;
                                case "EXTERNAL_URL_NOT_ALLOWED":
                                case 5:
                                    m.media_bundle_error = 5;
                                    break;
                                case "FILE_TOO_LARGE":
                                case 6:
                                    m.media_bundle_error = 6;
                                    break;
                                case "GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED":
                                case 7:
                                    m.media_bundle_error = 7;
                                    break;
                                case "INVALID_INPUT":
                                case 8:
                                    m.media_bundle_error = 8;
                                    break;
                                case "INVALID_MEDIA_BUNDLE":
                                case 9:
                                    m.media_bundle_error = 9;
                                    break;
                                case "INVALID_MEDIA_BUNDLE_ENTRY":
                                case 10:
                                    m.media_bundle_error = 10;
                                    break;
                                case "INVALID_MIME_TYPE":
                                case 11:
                                    m.media_bundle_error = 11;
                                    break;
                                case "INVALID_PATH":
                                case 12:
                                    m.media_bundle_error = 12;
                                    break;
                                case "INVALID_URL_REFERENCE":
                                case 13:
                                    m.media_bundle_error = 13;
                                    break;
                                case "MEDIA_DATA_TOO_LARGE":
                                case 14:
                                    m.media_bundle_error = 14;
                                    break;
                                case "MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY":
                                case 15:
                                    m.media_bundle_error = 15;
                                    break;
                                case "SERVER_ERROR":
                                case 16:
                                    m.media_bundle_error = 16;
                                    break;
                                case "STORAGE_ERROR":
                                case 17:
                                    m.media_bundle_error = 17;
                                    break;
                                case "SWIFFY_BUNDLE_NOT_ALLOWED":
                                case 18:
                                    m.media_bundle_error = 18;
                                    break;
                                case "TOO_MANY_FILES":
                                case 19:
                                    m.media_bundle_error = 19;
                                    break;
                                case "UNEXPECTED_SIZE":
                                case 20:
                                    m.media_bundle_error = 20;
                                    break;
                                case "UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT":
                                case 21:
                                    m.media_bundle_error = 21;
                                    break;
                                case "UNSUPPORTED_HTML5_FEATURE":
                                case 22:
                                    m.media_bundle_error = 22;
                                    break;
                                case "URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT":
                                case 23:
                                    m.media_bundle_error = 23;
                                    break;
                                case "CUSTOM_EXIT_NOT_ALLOWED":
                                case 24:
                                    m.media_bundle_error = 24;
                                    break;
                                }
                                switch (d.media_upload_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.media_upload_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.media_upload_error = 1;
                                    break;
                                case "FILE_TOO_BIG":
                                case 2:
                                    m.media_upload_error = 2;
                                    break;
                                case "UNPARSEABLE_IMAGE":
                                case 3:
                                    m.media_upload_error = 3;
                                    break;
                                case "ANIMATED_IMAGE_NOT_ALLOWED":
                                case 4:
                                    m.media_upload_error = 4;
                                    break;
                                case "FORMAT_NOT_ALLOWED":
                                case 5:
                                    m.media_upload_error = 5;
                                    break;
                                case "EXTERNAL_URL_NOT_ALLOWED":
                                case 6:
                                    m.media_upload_error = 6;
                                    break;
                                case "INVALID_URL_REFERENCE":
                                case 7:
                                    m.media_upload_error = 7;
                                    break;
                                case "MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY":
                                case 8:
                                    m.media_upload_error = 8;
                                    break;
                                case "ANIMATED_VISUAL_EFFECT":
                                case 9:
                                    m.media_upload_error = 9;
                                    break;
                                case "ANIMATION_TOO_LONG":
                                case 10:
                                    m.media_upload_error = 10;
                                    break;
                                case "ASPECT_RATIO_NOT_ALLOWED":
                                case 11:
                                    m.media_upload_error = 11;
                                    break;
                                case "AUDIO_NOT_ALLOWED_IN_MEDIA_BUNDLE":
                                case 12:
                                    m.media_upload_error = 12;
                                    break;
                                case "CMYK_JPEG_NOT_ALLOWED":
                                case 13:
                                    m.media_upload_error = 13;
                                    break;
                                case "FLASH_NOT_ALLOWED":
                                case 14:
                                    m.media_upload_error = 14;
                                    break;
                                case "FRAME_RATE_TOO_HIGH":
                                case 15:
                                    m.media_upload_error = 15;
                                    break;
                                case "GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED":
                                case 16:
                                    m.media_upload_error = 16;
                                    break;
                                case "IMAGE_CONSTRAINTS_VIOLATED":
                                case 17:
                                    m.media_upload_error = 17;
                                    break;
                                case "INVALID_MEDIA_BUNDLE":
                                case 18:
                                    m.media_upload_error = 18;
                                    break;
                                case "INVALID_MEDIA_BUNDLE_ENTRY":
                                case 19:
                                    m.media_upload_error = 19;
                                    break;
                                case "INVALID_MIME_TYPE":
                                case 20:
                                    m.media_upload_error = 20;
                                    break;
                                case "INVALID_PATH":
                                case 21:
                                    m.media_upload_error = 21;
                                    break;
                                case "LAYOUT_PROBLEM":
                                case 22:
                                    m.media_upload_error = 22;
                                    break;
                                case "MALFORMED_URL":
                                case 23:
                                    m.media_upload_error = 23;
                                    break;
                                case "MEDIA_BUNDLE_NOT_ALLOWED":
                                case 24:
                                    m.media_upload_error = 24;
                                    break;
                                case "MEDIA_BUNDLE_NOT_COMPATIBLE_TO_PRODUCT_TYPE":
                                case 25:
                                    m.media_upload_error = 25;
                                    break;
                                case "MEDIA_BUNDLE_REJECTED_BY_MULTIPLE_ASSET_SPECS":
                                case 26:
                                    m.media_upload_error = 26;
                                    break;
                                case "TOO_MANY_FILES_IN_MEDIA_BUNDLE":
                                case 27:
                                    m.media_upload_error = 27;
                                    break;
                                case "UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT":
                                case 28:
                                    m.media_upload_error = 28;
                                    break;
                                case "UNSUPPORTED_HTML5_FEATURE":
                                case 29:
                                    m.media_upload_error = 29;
                                    break;
                                case "URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT":
                                case 30:
                                    m.media_upload_error = 30;
                                    break;
                                case "VIDEO_FILE_NAME_TOO_LONG":
                                case 31:
                                    m.media_upload_error = 31;
                                    break;
                                case "VIDEO_MULTIPLE_FILES_WITH_SAME_NAME":
                                case 32:
                                    m.media_upload_error = 32;
                                    break;
                                case "VIDEO_NOT_ALLOWED_IN_MEDIA_BUNDLE":
                                case 33:
                                    m.media_upload_error = 33;
                                    break;
                                case "CANNOT_UPLOAD_MEDIA_TYPE_THROUGH_API":
                                case 34:
                                    m.media_upload_error = 34;
                                    break;
                                case "DIMENSIONS_NOT_ALLOWED":
                                case 35:
                                    m.media_upload_error = 35;
                                    break;
                                }
                                switch (d.media_file_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.media_file_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.media_file_error = 1;
                                    break;
                                case "CANNOT_CREATE_STANDARD_ICON":
                                case 2:
                                    m.media_file_error = 2;
                                    break;
                                case "CANNOT_SELECT_STANDARD_ICON_WITH_OTHER_TYPES":
                                case 3:
                                    m.media_file_error = 3;
                                    break;
                                case "CANNOT_SPECIFY_MEDIA_FILE_ID_AND_DATA":
                                case 4:
                                    m.media_file_error = 4;
                                    break;
                                case "DUPLICATE_MEDIA":
                                case 5:
                                    m.media_file_error = 5;
                                    break;
                                case "EMPTY_FIELD":
                                case 6:
                                    m.media_file_error = 6;
                                    break;
                                case "RESOURCE_REFERENCED_IN_MULTIPLE_OPS":
                                case 7:
                                    m.media_file_error = 7;
                                    break;
                                case "FIELD_NOT_SUPPORTED_FOR_MEDIA_SUB_TYPE":
                                case 8:
                                    m.media_file_error = 8;
                                    break;
                                case "INVALID_MEDIA_FILE_ID":
                                case 9:
                                    m.media_file_error = 9;
                                    break;
                                case "INVALID_MEDIA_SUB_TYPE":
                                case 10:
                                    m.media_file_error = 10;
                                    break;
                                case "INVALID_MEDIA_FILE_TYPE":
                                case 11:
                                    m.media_file_error = 11;
                                    break;
                                case "INVALID_MIME_TYPE":
                                case 12:
                                    m.media_file_error = 12;
                                    break;
                                case "INVALID_REFERENCE_ID":
                                case 13:
                                    m.media_file_error = 13;
                                    break;
                                case "INVALID_YOU_TUBE_ID":
                                case 14:
                                    m.media_file_error = 14;
                                    break;
                                case "MEDIA_FILE_FAILED_TRANSCODING":
                                case 15:
                                    m.media_file_error = 15;
                                    break;
                                case "MEDIA_NOT_TRANSCODED":
                                case 16:
                                    m.media_file_error = 16;
                                    break;
                                case "MEDIA_TYPE_DOES_NOT_MATCH_MEDIA_FILE_TYPE":
                                case 17:
                                    m.media_file_error = 17;
                                    break;
                                case "NO_FIELDS_SPECIFIED":
                                case 18:
                                    m.media_file_error = 18;
                                    break;
                                case "NULL_REFERENCE_ID_AND_MEDIA_ID":
                                case 19:
                                    m.media_file_error = 19;
                                    break;
                                case "TOO_LONG":
                                case 20:
                                    m.media_file_error = 20;
                                    break;
                                case "UNSUPPORTED_TYPE":
                                case 21:
                                    m.media_file_error = 21;
                                    break;
                                case "YOU_TUBE_SERVICE_UNAVAILABLE":
                                case 22:
                                    m.media_file_error = 22;
                                    break;
                                case "YOU_TUBE_VIDEO_HAS_NON_POSITIVE_DURATION":
                                case 23:
                                    m.media_file_error = 23;
                                    break;
                                case "YOU_TUBE_VIDEO_NOT_FOUND":
                                case 24:
                                    m.media_file_error = 24;
                                    break;
                                }
                                switch (d.merchant_center_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.merchant_center_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.merchant_center_error = 1;
                                    break;
                                case "MERCHANT_ID_CANNOT_BE_ACCESSED":
                                case 2:
                                    m.merchant_center_error = 2;
                                    break;
                                case "CUSTOMER_NOT_ALLOWED_FOR_SHOPPING_PERFORMANCE_MAX":
                                case 3:
                                    m.merchant_center_error = 3;
                                    break;
                                }
                                switch (d.multiplier_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.multiplier_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.multiplier_error = 1;
                                    break;
                                case "MULTIPLIER_TOO_HIGH":
                                case 2:
                                    m.multiplier_error = 2;
                                    break;
                                case "MULTIPLIER_TOO_LOW":
                                case 3:
                                    m.multiplier_error = 3;
                                    break;
                                case "TOO_MANY_FRACTIONAL_DIGITS":
                                case 4:
                                    m.multiplier_error = 4;
                                    break;
                                case "MULTIPLIER_NOT_ALLOWED_FOR_BIDDING_STRATEGY":
                                case 5:
                                    m.multiplier_error = 5;
                                    break;
                                case "MULTIPLIER_NOT_ALLOWED_WHEN_BASE_BID_IS_MISSING":
                                case 6:
                                    m.multiplier_error = 6;
                                    break;
                                case "NO_MULTIPLIER_SPECIFIED":
                                case 7:
                                    m.multiplier_error = 7;
                                    break;
                                case "MULTIPLIER_CAUSES_BID_TO_EXCEED_DAILY_BUDGET":
                                case 8:
                                    m.multiplier_error = 8;
                                    break;
                                case "MULTIPLIER_CAUSES_BID_TO_EXCEED_MONTHLY_BUDGET":
                                case 9:
                                    m.multiplier_error = 9;
                                    break;
                                case "MULTIPLIER_CAUSES_BID_TO_EXCEED_CUSTOM_BUDGET":
                                case 10:
                                    m.multiplier_error = 10;
                                    break;
                                case "MULTIPLIER_CAUSES_BID_TO_EXCEED_MAX_ALLOWED_BID":
                                case 11:
                                    m.multiplier_error = 11;
                                    break;
                                case "BID_LESS_THAN_MIN_ALLOWED_BID_WITH_MULTIPLIER":
                                case 12:
                                    m.multiplier_error = 12;
                                    break;
                                case "MULTIPLIER_AND_BIDDING_STRATEGY_TYPE_MISMATCH":
                                case 13:
                                    m.multiplier_error = 13;
                                    break;
                                }
                                switch (d.new_resource_creation_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.new_resource_creation_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.new_resource_creation_error = 1;
                                    break;
                                case "CANNOT_SET_ID_FOR_CREATE":
                                case 2:
                                    m.new_resource_creation_error = 2;
                                    break;
                                case "DUPLICATE_TEMP_IDS":
                                case 3:
                                    m.new_resource_creation_error = 3;
                                    break;
                                case "TEMP_ID_RESOURCE_HAD_ERRORS":
                                case 4:
                                    m.new_resource_creation_error = 4;
                                    break;
                                }
                                switch (d.not_empty_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.not_empty_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.not_empty_error = 1;
                                    break;
                                case "EMPTY_LIST":
                                case 2:
                                    m.not_empty_error = 2;
                                    break;
                                }
                                switch (d.null_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.null_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.null_error = 1;
                                    break;
                                case "NULL_CONTENT":
                                case 2:
                                    m.null_error = 2;
                                    break;
                                }
                                switch (d.operator_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator_error = 1;
                                    break;
                                case "OPERATOR_NOT_SUPPORTED":
                                case 2:
                                    m.operator_error = 2;
                                    break;
                                }
                                switch (d.range_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.range_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.range_error = 1;
                                    break;
                                case "TOO_LOW":
                                case 2:
                                    m.range_error = 2;
                                    break;
                                case "TOO_HIGH":
                                case 3:
                                    m.range_error = 3;
                                    break;
                                }
                                switch (d.recommendation_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.recommendation_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.recommendation_error = 1;
                                    break;
                                case "BUDGET_AMOUNT_TOO_SMALL":
                                case 2:
                                    m.recommendation_error = 2;
                                    break;
                                case "BUDGET_AMOUNT_TOO_LARGE":
                                case 3:
                                    m.recommendation_error = 3;
                                    break;
                                case "INVALID_BUDGET_AMOUNT":
                                case 4:
                                    m.recommendation_error = 4;
                                    break;
                                case "POLICY_ERROR":
                                case 5:
                                    m.recommendation_error = 5;
                                    break;
                                case "INVALID_BID_AMOUNT":
                                case 6:
                                    m.recommendation_error = 6;
                                    break;
                                case "ADGROUP_KEYWORD_LIMIT":
                                case 7:
                                    m.recommendation_error = 7;
                                    break;
                                case "RECOMMENDATION_ALREADY_APPLIED":
                                case 8:
                                    m.recommendation_error = 8;
                                    break;
                                case "RECOMMENDATION_INVALIDATED":
                                case 9:
                                    m.recommendation_error = 9;
                                    break;
                                case "TOO_MANY_OPERATIONS":
                                case 10:
                                    m.recommendation_error = 10;
                                    break;
                                case "NO_OPERATIONS":
                                case 11:
                                    m.recommendation_error = 11;
                                    break;
                                case "DIFFERENT_TYPES_NOT_SUPPORTED":
                                case 12:
                                    m.recommendation_error = 12;
                                    break;
                                case "DUPLICATE_RESOURCE_NAME":
                                case 13:
                                    m.recommendation_error = 13;
                                    break;
                                case "RECOMMENDATION_ALREADY_DISMISSED":
                                case 14:
                                    m.recommendation_error = 14;
                                    break;
                                case "INVALID_APPLY_REQUEST":
                                case 15:
                                    m.recommendation_error = 15;
                                    break;
                                }
                                switch (d.region_code_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.region_code_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.region_code_error = 1;
                                    break;
                                case "INVALID_REGION_CODE":
                                case 2:
                                    m.region_code_error = 2;
                                    break;
                                }
                                switch (d.setting_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.setting_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.setting_error = 1;
                                    break;
                                case "SETTING_TYPE_IS_NOT_AVAILABLE":
                                case 3:
                                    m.setting_error = 3;
                                    break;
                                case "SETTING_TYPE_IS_NOT_COMPATIBLE_WITH_CAMPAIGN":
                                case 4:
                                    m.setting_error = 4;
                                    break;
                                case "TARGETING_SETTING_CONTAINS_INVALID_CRITERION_TYPE_GROUP":
                                case 5:
                                    m.setting_error = 5;
                                    break;
                                case "TARGETING_SETTING_DEMOGRAPHIC_CRITERION_TYPE_GROUPS_MUST_BE_SET_TO_TARGET_ALL":
                                case 6:
                                    m.setting_error = 6;
                                    break;
                                case "TARGETING_SETTING_CANNOT_CHANGE_TARGET_ALL_TO_FALSE_FOR_DEMOGRAPHIC_CRITERION_TYPE_GROUP":
                                case 7:
                                    m.setting_error = 7;
                                    break;
                                case "DYNAMIC_SEARCH_ADS_SETTING_AT_LEAST_ONE_FEED_ID_MUST_BE_PRESENT":
                                case 8:
                                    m.setting_error = 8;
                                    break;
                                case "DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_DOMAIN_NAME":
                                case 9:
                                    m.setting_error = 9;
                                    break;
                                case "DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_SUBDOMAIN_NAME":
                                case 10:
                                    m.setting_error = 10;
                                    break;
                                case "DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_LANGUAGE_CODE":
                                case 11:
                                    m.setting_error = 11;
                                    break;
                                case "TARGET_ALL_IS_NOT_ALLOWED_FOR_PLACEMENT_IN_SEARCH_CAMPAIGN":
                                case 12:
                                    m.setting_error = 12;
                                    break;
                                case "SETTING_VALUE_NOT_COMPATIBLE_WITH_CAMPAIGN":
                                case 20:
                                    m.setting_error = 20;
                                    break;
                                case "BID_ONLY_IS_NOT_ALLOWED_TO_BE_MODIFIED_WITH_CUSTOMER_MATCH_TARGETING":
                                case 21:
                                    m.setting_error = 21;
                                    break;
                                }
                                switch (d.string_format_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.string_format_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.string_format_error = 1;
                                    break;
                                case "ILLEGAL_CHARS":
                                case 2:
                                    m.string_format_error = 2;
                                    break;
                                case "INVALID_FORMAT":
                                case 3:
                                    m.string_format_error = 3;
                                    break;
                                }
                                switch (d.string_length_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.string_length_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.string_length_error = 1;
                                    break;
                                case "EMPTY":
                                case 4:
                                    m.string_length_error = 4;
                                    break;
                                case "TOO_SHORT":
                                case 2:
                                    m.string_length_error = 2;
                                    break;
                                case "TOO_LONG":
                                case 3:
                                    m.string_length_error = 3;
                                    break;
                                }
                                switch (d.operation_access_denied_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operation_access_denied_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operation_access_denied_error = 1;
                                    break;
                                case "ACTION_NOT_PERMITTED":
                                case 2:
                                    m.operation_access_denied_error = 2;
                                    break;
                                case "CREATE_OPERATION_NOT_PERMITTED":
                                case 3:
                                    m.operation_access_denied_error = 3;
                                    break;
                                case "REMOVE_OPERATION_NOT_PERMITTED":
                                case 4:
                                    m.operation_access_denied_error = 4;
                                    break;
                                case "UPDATE_OPERATION_NOT_PERMITTED":
                                case 5:
                                    m.operation_access_denied_error = 5;
                                    break;
                                case "MUTATE_ACTION_NOT_PERMITTED_FOR_CLIENT":
                                case 6:
                                    m.operation_access_denied_error = 6;
                                    break;
                                case "OPERATION_NOT_PERMITTED_FOR_CAMPAIGN_TYPE":
                                case 7:
                                    m.operation_access_denied_error = 7;
                                    break;
                                case "CREATE_AS_REMOVED_NOT_PERMITTED":
                                case 8:
                                    m.operation_access_denied_error = 8;
                                    break;
                                case "OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE":
                                case 9:
                                    m.operation_access_denied_error = 9;
                                    break;
                                case "OPERATION_NOT_PERMITTED_FOR_AD_GROUP_TYPE":
                                case 10:
                                    m.operation_access_denied_error = 10;
                                    break;
                                case "MUTATE_NOT_PERMITTED_FOR_CUSTOMER":
                                case 11:
                                    m.operation_access_denied_error = 11;
                                    break;
                                }
                                switch (d.resource_access_denied_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.resource_access_denied_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.resource_access_denied_error = 1;
                                    break;
                                case "WRITE_ACCESS_DENIED":
                                case 3:
                                    m.resource_access_denied_error = 3;
                                    break;
                                }
                                switch (d.resource_count_limit_exceeded_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.resource_count_limit_exceeded_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.resource_count_limit_exceeded_error = 1;
                                    break;
                                case "ACCOUNT_LIMIT":
                                case 2:
                                    m.resource_count_limit_exceeded_error = 2;
                                    break;
                                case "CAMPAIGN_LIMIT":
                                case 3:
                                    m.resource_count_limit_exceeded_error = 3;
                                    break;
                                case "ADGROUP_LIMIT":
                                case 4:
                                    m.resource_count_limit_exceeded_error = 4;
                                    break;
                                case "AD_GROUP_AD_LIMIT":
                                case 5:
                                    m.resource_count_limit_exceeded_error = 5;
                                    break;
                                case "AD_GROUP_CRITERION_LIMIT":
                                case 6:
                                    m.resource_count_limit_exceeded_error = 6;
                                    break;
                                case "SHARED_SET_LIMIT":
                                case 7:
                                    m.resource_count_limit_exceeded_error = 7;
                                    break;
                                case "MATCHING_FUNCTION_LIMIT":
                                case 8:
                                    m.resource_count_limit_exceeded_error = 8;
                                    break;
                                case "RESPONSE_ROW_LIMIT_EXCEEDED":
                                case 9:
                                    m.resource_count_limit_exceeded_error = 9;
                                    break;
                                case "RESOURCE_LIMIT":
                                case 10:
                                    m.resource_count_limit_exceeded_error = 10;
                                    break;
                                }
                                switch (d.youtube_video_registration_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.youtube_video_registration_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.youtube_video_registration_error = 1;
                                    break;
                                case "VIDEO_NOT_FOUND":
                                case 2:
                                    m.youtube_video_registration_error = 2;
                                    break;
                                case "VIDEO_NOT_ACCESSIBLE":
                                case 3:
                                    m.youtube_video_registration_error = 3;
                                    break;
                                case "VIDEO_NOT_ELIGIBLE":
                                case 4:
                                    m.youtube_video_registration_error = 4;
                                    break;
                                }
                                switch (d.ad_group_bid_modifier_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_group_bid_modifier_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_group_bid_modifier_error = 1;
                                    break;
                                case "CRITERION_ID_NOT_SUPPORTED":
                                case 2:
                                    m.ad_group_bid_modifier_error = 2;
                                    break;
                                case "CANNOT_OVERRIDE_OPTED_OUT_CAMPAIGN_CRITERION_BID_MODIFIER":
                                case 3:
                                    m.ad_group_bid_modifier_error = 3;
                                    break;
                                }
                                switch (d.context_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.context_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.context_error = 1;
                                    break;
                                case "OPERATION_NOT_PERMITTED_FOR_CONTEXT":
                                case 2:
                                    m.context_error = 2;
                                    break;
                                case "OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE":
                                case 3:
                                    m.context_error = 3;
                                    break;
                                }
                                switch (d.field_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_error = 1;
                                    break;
                                case "REQUIRED":
                                case 2:
                                    m.field_error = 2;
                                    break;
                                case "IMMUTABLE_FIELD":
                                case 3:
                                    m.field_error = 3;
                                    break;
                                case "INVALID_VALUE":
                                case 4:
                                    m.field_error = 4;
                                    break;
                                case "VALUE_MUST_BE_UNSET":
                                case 5:
                                    m.field_error = 5;
                                    break;
                                case "REQUIRED_NONEMPTY_LIST":
                                case 6:
                                    m.field_error = 6;
                                    break;
                                case "FIELD_CANNOT_BE_CLEARED":
                                case 7:
                                    m.field_error = 7;
                                    break;
                                case "BLOCKED_VALUE":
                                case 9:
                                    m.field_error = 9;
                                    break;
                                case "FIELD_CAN_ONLY_BE_CLEARED":
                                case 10:
                                    m.field_error = 10;
                                    break;
                                }
                                switch (d.shared_set_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.shared_set_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.shared_set_error = 1;
                                    break;
                                case "CUSTOMER_CANNOT_CREATE_SHARED_SET_OF_THIS_TYPE":
                                case 2:
                                    m.shared_set_error = 2;
                                    break;
                                case "DUPLICATE_NAME":
                                case 3:
                                    m.shared_set_error = 3;
                                    break;
                                case "SHARED_SET_REMOVED":
                                case 4:
                                    m.shared_set_error = 4;
                                    break;
                                case "SHARED_SET_IN_USE":
                                case 5:
                                    m.shared_set_error = 5;
                                    break;
                                }
                                switch (d.shared_criterion_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.shared_criterion_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.shared_criterion_error = 1;
                                    break;
                                case "CRITERION_TYPE_NOT_ALLOWED_FOR_SHARED_SET_TYPE":
                                case 2:
                                    m.shared_criterion_error = 2;
                                    break;
                                }
                                switch (d.campaign_shared_set_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_shared_set_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_shared_set_error = 1;
                                    break;
                                case "SHARED_SET_ACCESS_DENIED":
                                case 2:
                                    m.campaign_shared_set_error = 2;
                                    break;
                                }
                                switch (d.conversion_action_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_action_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_action_error = 1;
                                    break;
                                case "DUPLICATE_NAME":
                                case 2:
                                    m.conversion_action_error = 2;
                                    break;
                                case "DUPLICATE_APP_ID":
                                case 3:
                                    m.conversion_action_error = 3;
                                    break;
                                case "TWO_CONVERSION_ACTIONS_BIDDING_ON_SAME_APP_DOWNLOAD":
                                case 4:
                                    m.conversion_action_error = 4;
                                    break;
                                case "BIDDING_ON_SAME_APP_DOWNLOAD_AS_GLOBAL_ACTION":
                                case 5:
                                    m.conversion_action_error = 5;
                                    break;
                                case "DATA_DRIVEN_MODEL_WAS_NEVER_GENERATED":
                                case 6:
                                    m.conversion_action_error = 6;
                                    break;
                                case "DATA_DRIVEN_MODEL_EXPIRED":
                                case 7:
                                    m.conversion_action_error = 7;
                                    break;
                                case "DATA_DRIVEN_MODEL_STALE":
                                case 8:
                                    m.conversion_action_error = 8;
                                    break;
                                case "DATA_DRIVEN_MODEL_UNKNOWN":
                                case 9:
                                    m.conversion_action_error = 9;
                                    break;
                                case "CREATION_NOT_SUPPORTED":
                                case 10:
                                    m.conversion_action_error = 10;
                                    break;
                                case "UPDATE_NOT_SUPPORTED":
                                case 11:
                                    m.conversion_action_error = 11;
                                    break;
                                }
                                switch (d.conversion_adjustment_upload_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_adjustment_upload_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_adjustment_upload_error = 1;
                                    break;
                                case "TOO_RECENT_CONVERSION_ACTION":
                                case 2:
                                    m.conversion_adjustment_upload_error = 2;
                                    break;
                                case "INVALID_CONVERSION_ACTION":
                                case 3:
                                    m.conversion_adjustment_upload_error = 3;
                                    break;
                                case "CONVERSION_ALREADY_RETRACTED":
                                case 4:
                                    m.conversion_adjustment_upload_error = 4;
                                    break;
                                case "CONVERSION_NOT_FOUND":
                                case 5:
                                    m.conversion_adjustment_upload_error = 5;
                                    break;
                                case "CONVERSION_EXPIRED":
                                case 6:
                                    m.conversion_adjustment_upload_error = 6;
                                    break;
                                case "ADJUSTMENT_PRECEDES_CONVERSION":
                                case 7:
                                    m.conversion_adjustment_upload_error = 7;
                                    break;
                                case "MORE_RECENT_RESTATEMENT_FOUND":
                                case 8:
                                    m.conversion_adjustment_upload_error = 8;
                                    break;
                                case "TOO_RECENT_CONVERSION":
                                case 9:
                                    m.conversion_adjustment_upload_error = 9;
                                    break;
                                case "CANNOT_RESTATE_CONVERSION_ACTION_THAT_ALWAYS_USES_DEFAULT_CONVERSION_VALUE":
                                case 10:
                                    m.conversion_adjustment_upload_error = 10;
                                    break;
                                case "TOO_MANY_ADJUSTMENTS_IN_REQUEST":
                                case 11:
                                    m.conversion_adjustment_upload_error = 11;
                                    break;
                                case "TOO_MANY_ADJUSTMENTS":
                                case 12:
                                    m.conversion_adjustment_upload_error = 12;
                                    break;
                                case "RESTATEMENT_ALREADY_EXISTS":
                                case 13:
                                    m.conversion_adjustment_upload_error = 13;
                                    break;
                                case "DUPLICATE_ADJUSTMENT_IN_REQUEST":
                                case 14:
                                    m.conversion_adjustment_upload_error = 14;
                                    break;
                                case "CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS":
                                case 15:
                                    m.conversion_adjustment_upload_error = 15;
                                    break;
                                case "CONVERSION_ACTION_NOT_ELIGIBLE_FOR_ENHANCEMENT":
                                case 16:
                                    m.conversion_adjustment_upload_error = 16;
                                    break;
                                case "INVALID_USER_IDENTIFIER":
                                case 17:
                                    m.conversion_adjustment_upload_error = 17;
                                    break;
                                case "UNSUPPORTED_USER_IDENTIFIER":
                                case 18:
                                    m.conversion_adjustment_upload_error = 18;
                                    break;
                                case "GCLID_DATE_TIME_PAIR_AND_ORDER_ID_BOTH_SET":
                                case 20:
                                    m.conversion_adjustment_upload_error = 20;
                                    break;
                                case "CONVERSION_ALREADY_ENHANCED":
                                case 21:
                                    m.conversion_adjustment_upload_error = 21;
                                    break;
                                case "DUPLICATE_ENHANCEMENT_IN_REQUEST":
                                case 22:
                                    m.conversion_adjustment_upload_error = 22;
                                    break;
                                case "CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCEMENT":
                                case 23:
                                    m.conversion_adjustment_upload_error = 23;
                                    break;
                                case "MISSING_ORDER_ID_FOR_WEBPAGE":
                                case 24:
                                    m.conversion_adjustment_upload_error = 24;
                                    break;
                                case "ORDER_ID_CONTAINS_PII":
                                case 25:
                                    m.conversion_adjustment_upload_error = 25;
                                    break;
                                }
                                switch (d.conversion_custom_variable_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_custom_variable_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_custom_variable_error = 1;
                                    break;
                                case "DUPLICATE_NAME":
                                case 2:
                                    m.conversion_custom_variable_error = 2;
                                    break;
                                case "DUPLICATE_TAG":
                                case 3:
                                    m.conversion_custom_variable_error = 3;
                                    break;
                                case "RESERVED_TAG":
                                case 4:
                                    m.conversion_custom_variable_error = 4;
                                    break;
                                }
                                switch (d.conversion_upload_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_upload_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_upload_error = 1;
                                    break;
                                case "TOO_MANY_CONVERSIONS_IN_REQUEST":
                                case 2:
                                    m.conversion_upload_error = 2;
                                    break;
                                case "UNPARSEABLE_GCLID":
                                case 3:
                                    m.conversion_upload_error = 3;
                                    break;
                                case "CONVERSION_PRECEDES_EVENT":
                                case 42:
                                    m.conversion_upload_error = 42;
                                    break;
                                case "EXPIRED_EVENT":
                                case 43:
                                    m.conversion_upload_error = 43;
                                    break;
                                case "TOO_RECENT_EVENT":
                                case 44:
                                    m.conversion_upload_error = 44;
                                    break;
                                case "EVENT_NOT_FOUND":
                                case 45:
                                    m.conversion_upload_error = 45;
                                    break;
                                case "UNAUTHORIZED_CUSTOMER":
                                case 8:
                                    m.conversion_upload_error = 8;
                                    break;
                                case "INVALID_CONVERSION_ACTION":
                                case 9:
                                    m.conversion_upload_error = 9;
                                    break;
                                case "TOO_RECENT_CONVERSION_ACTION":
                                case 10:
                                    m.conversion_upload_error = 10;
                                    break;
                                case "CONVERSION_TRACKING_NOT_ENABLED_AT_IMPRESSION_TIME":
                                case 11:
                                    m.conversion_upload_error = 11;
                                    break;
                                case "EXTERNAL_ATTRIBUTION_DATA_SET_FOR_NON_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION":
                                case 12:
                                    m.conversion_upload_error = 12;
                                    break;
                                case "EXTERNAL_ATTRIBUTION_DATA_NOT_SET_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION":
                                case 13:
                                    m.conversion_upload_error = 13;
                                    break;
                                case "ORDER_ID_NOT_PERMITTED_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION":
                                case 14:
                                    m.conversion_upload_error = 14;
                                    break;
                                case "ORDER_ID_ALREADY_IN_USE":
                                case 15:
                                    m.conversion_upload_error = 15;
                                    break;
                                case "DUPLICATE_ORDER_ID":
                                case 16:
                                    m.conversion_upload_error = 16;
                                    break;
                                case "TOO_RECENT_CALL":
                                case 17:
                                    m.conversion_upload_error = 17;
                                    break;
                                case "EXPIRED_CALL":
                                case 18:
                                    m.conversion_upload_error = 18;
                                    break;
                                case "CALL_NOT_FOUND":
                                case 19:
                                    m.conversion_upload_error = 19;
                                    break;
                                case "CONVERSION_PRECEDES_CALL":
                                case 20:
                                    m.conversion_upload_error = 20;
                                    break;
                                case "CONVERSION_TRACKING_NOT_ENABLED_AT_CALL_TIME":
                                case 21:
                                    m.conversion_upload_error = 21;
                                    break;
                                case "UNPARSEABLE_CALLERS_PHONE_NUMBER":
                                case 22:
                                    m.conversion_upload_error = 22;
                                    break;
                                case "CLICK_CONVERSION_ALREADY_EXISTS":
                                case 23:
                                    m.conversion_upload_error = 23;
                                    break;
                                case "CALL_CONVERSION_ALREADY_EXISTS":
                                case 24:
                                    m.conversion_upload_error = 24;
                                    break;
                                case "DUPLICATE_CLICK_CONVERSION_IN_REQUEST":
                                case 25:
                                    m.conversion_upload_error = 25;
                                    break;
                                case "DUPLICATE_CALL_CONVERSION_IN_REQUEST":
                                case 26:
                                    m.conversion_upload_error = 26;
                                    break;
                                case "CUSTOM_VARIABLE_NOT_ENABLED":
                                case 28:
                                    m.conversion_upload_error = 28;
                                    break;
                                case "CUSTOM_VARIABLE_VALUE_CONTAINS_PII":
                                case 29:
                                    m.conversion_upload_error = 29;
                                    break;
                                case "INVALID_CUSTOMER_FOR_CLICK":
                                case 30:
                                    m.conversion_upload_error = 30;
                                    break;
                                case "INVALID_CUSTOMER_FOR_CALL":
                                case 31:
                                    m.conversion_upload_error = 31;
                                    break;
                                case "CONVERSION_NOT_COMPLIANT_WITH_ATT_POLICY":
                                case 32:
                                    m.conversion_upload_error = 32;
                                    break;
                                case "CLICK_NOT_FOUND":
                                case 33:
                                    m.conversion_upload_error = 33;
                                    break;
                                case "INVALID_USER_IDENTIFIER":
                                case 34:
                                    m.conversion_upload_error = 34;
                                    break;
                                case "EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION_NOT_PERMITTED_WITH_USER_IDENTIFIER":
                                case 35:
                                    m.conversion_upload_error = 35;
                                    break;
                                case "UNSUPPORTED_USER_IDENTIFIER":
                                case 36:
                                    m.conversion_upload_error = 36;
                                    break;
                                case "GBRAID_WBRAID_BOTH_SET":
                                case 38:
                                    m.conversion_upload_error = 38;
                                    break;
                                case "UNPARSEABLE_WBRAID":
                                case 39:
                                    m.conversion_upload_error = 39;
                                    break;
                                case "UNPARSEABLE_GBRAID":
                                case 40:
                                    m.conversion_upload_error = 40;
                                    break;
                                case "ONE_PER_CLICK_CONVERSION_ACTION_NOT_PERMITTED_WITH_BRAID":
                                case 46:
                                    m.conversion_upload_error = 46;
                                    break;
                                case "CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCED_CONVERSIONS":
                                case 47:
                                    m.conversion_upload_error = 47;
                                    break;
                                case "CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS":
                                case 48:
                                    m.conversion_upload_error = 48;
                                    break;
                                case "ORDER_ID_CONTAINS_PII":
                                case 49:
                                    m.conversion_upload_error = 49;
                                    break;
                                }
                                switch (d.conversion_value_rule_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_value_rule_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_value_rule_error = 1;
                                    break;
                                case "INVALID_GEO_TARGET_CONSTANT":
                                case 2:
                                    m.conversion_value_rule_error = 2;
                                    break;
                                case "CONFLICTING_INCLUDED_AND_EXCLUDED_GEO_TARGET":
                                case 3:
                                    m.conversion_value_rule_error = 3;
                                    break;
                                case "CONFLICTING_CONDITIONS":
                                case 4:
                                    m.conversion_value_rule_error = 4;
                                    break;
                                case "CANNOT_REMOVE_IF_INCLUDED_IN_VALUE_RULE_SET":
                                case 5:
                                    m.conversion_value_rule_error = 5;
                                    break;
                                case "CONDITION_NOT_ALLOWED":
                                case 6:
                                    m.conversion_value_rule_error = 6;
                                    break;
                                case "FIELD_MUST_BE_UNSET":
                                case 7:
                                    m.conversion_value_rule_error = 7;
                                    break;
                                case "CANNOT_PAUSE_UNLESS_VALUE_RULE_SET_IS_PAUSED":
                                case 8:
                                    m.conversion_value_rule_error = 8;
                                    break;
                                case "UNTARGETABLE_GEO_TARGET":
                                case 9:
                                    m.conversion_value_rule_error = 9;
                                    break;
                                case "INVALID_AUDIENCE_USER_LIST":
                                case 10:
                                    m.conversion_value_rule_error = 10;
                                    break;
                                case "INACCESSIBLE_USER_LIST":
                                case 11:
                                    m.conversion_value_rule_error = 11;
                                    break;
                                case "INVALID_AUDIENCE_USER_INTEREST":
                                case 12:
                                    m.conversion_value_rule_error = 12;
                                    break;
                                case "CANNOT_ADD_RULE_WITH_STATUS_REMOVED":
                                case 13:
                                    m.conversion_value_rule_error = 13;
                                    break;
                                }
                                switch (d.conversion_value_rule_set_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_value_rule_set_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_value_rule_set_error = 1;
                                    break;
                                case "CONFLICTING_VALUE_RULE_CONDITIONS":
                                case 2:
                                    m.conversion_value_rule_set_error = 2;
                                    break;
                                case "INVALID_VALUE_RULE":
                                case 3:
                                    m.conversion_value_rule_set_error = 3;
                                    break;
                                case "DIMENSIONS_UPDATE_ONLY_ALLOW_APPEND":
                                case 4:
                                    m.conversion_value_rule_set_error = 4;
                                    break;
                                case "CONDITION_TYPE_NOT_ALLOWED":
                                case 5:
                                    m.conversion_value_rule_set_error = 5;
                                    break;
                                case "DUPLICATE_DIMENSIONS":
                                case 6:
                                    m.conversion_value_rule_set_error = 6;
                                    break;
                                case "INVALID_CAMPAIGN_ID":
                                case 7:
                                    m.conversion_value_rule_set_error = 7;
                                    break;
                                case "CANNOT_PAUSE_UNLESS_ALL_VALUE_RULES_ARE_PAUSED":
                                case 8:
                                    m.conversion_value_rule_set_error = 8;
                                    break;
                                case "SHOULD_PAUSE_WHEN_ALL_VALUE_RULES_ARE_PAUSED":
                                case 9:
                                    m.conversion_value_rule_set_error = 9;
                                    break;
                                case "VALUE_RULES_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE":
                                case 10:
                                    m.conversion_value_rule_set_error = 10;
                                    break;
                                case "INELIGIBLE_CONVERSION_ACTION_CATEGORIES":
                                case 11:
                                    m.conversion_value_rule_set_error = 11;
                                    break;
                                case "DIMENSION_NO_CONDITION_USED_WITH_OTHER_DIMENSIONS":
                                case 12:
                                    m.conversion_value_rule_set_error = 12;
                                    break;
                                case "DIMENSION_NO_CONDITION_NOT_ALLOWED":
                                case 13:
                                    m.conversion_value_rule_set_error = 13;
                                    break;
                                case "UNSUPPORTED_CONVERSION_ACTION_CATEGORIES":
                                case 14:
                                    m.conversion_value_rule_set_error = 14;
                                    break;
                                }
                                switch (d.header_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.header_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.header_error = 1;
                                    break;
                                case "INVALID_LOGIN_CUSTOMER_ID":
                                case 3:
                                    m.header_error = 3;
                                    break;
                                case "INVALID_LINKED_CUSTOMER_ID":
                                case 7:
                                    m.header_error = 7;
                                    break;
                                }
                                switch (d.database_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.database_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.database_error = 1;
                                    break;
                                case "CONCURRENT_MODIFICATION":
                                case 2:
                                    m.database_error = 2;
                                    break;
                                case "DATA_CONSTRAINT_VIOLATION":
                                case 3:
                                    m.database_error = 3;
                                    break;
                                case "REQUEST_TOO_LARGE":
                                case 4:
                                    m.database_error = 4;
                                    break;
                                }
                                switch (d.policy_finding_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.policy_finding_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.policy_finding_error = 1;
                                    break;
                                case "POLICY_FINDING":
                                case 2:
                                    m.policy_finding_error = 2;
                                    break;
                                case "POLICY_TOPIC_NOT_FOUND":
                                case 3:
                                    m.policy_finding_error = 3;
                                    break;
                                }
                                switch (d.enum_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.enum_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.enum_error = 1;
                                    break;
                                case "ENUM_VALUE_NOT_PERMITTED":
                                case 3:
                                    m.enum_error = 3;
                                    break;
                                }
                                switch (d.keyword_plan_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_error = 1;
                                    break;
                                case "BID_MULTIPLIER_OUT_OF_RANGE":
                                case 2:
                                    m.keyword_plan_error = 2;
                                    break;
                                case "BID_TOO_HIGH":
                                case 3:
                                    m.keyword_plan_error = 3;
                                    break;
                                case "BID_TOO_LOW":
                                case 4:
                                    m.keyword_plan_error = 4;
                                    break;
                                case "BID_TOO_MANY_FRACTIONAL_DIGITS":
                                case 5:
                                    m.keyword_plan_error = 5;
                                    break;
                                case "DAILY_BUDGET_TOO_LOW":
                                case 6:
                                    m.keyword_plan_error = 6;
                                    break;
                                case "DAILY_BUDGET_TOO_MANY_FRACTIONAL_DIGITS":
                                case 7:
                                    m.keyword_plan_error = 7;
                                    break;
                                case "INVALID_VALUE":
                                case 8:
                                    m.keyword_plan_error = 8;
                                    break;
                                case "KEYWORD_PLAN_HAS_NO_KEYWORDS":
                                case 9:
                                    m.keyword_plan_error = 9;
                                    break;
                                case "KEYWORD_PLAN_NOT_ENABLED":
                                case 10:
                                    m.keyword_plan_error = 10;
                                    break;
                                case "KEYWORD_PLAN_NOT_FOUND":
                                case 11:
                                    m.keyword_plan_error = 11;
                                    break;
                                case "MISSING_BID":
                                case 13:
                                    m.keyword_plan_error = 13;
                                    break;
                                case "MISSING_FORECAST_PERIOD":
                                case 14:
                                    m.keyword_plan_error = 14;
                                    break;
                                case "INVALID_FORECAST_DATE_RANGE":
                                case 15:
                                    m.keyword_plan_error = 15;
                                    break;
                                case "INVALID_NAME":
                                case 16:
                                    m.keyword_plan_error = 16;
                                    break;
                                }
                                switch (d.keyword_plan_campaign_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_campaign_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_campaign_error = 1;
                                    break;
                                case "INVALID_NAME":
                                case 2:
                                    m.keyword_plan_campaign_error = 2;
                                    break;
                                case "INVALID_LANGUAGES":
                                case 3:
                                    m.keyword_plan_campaign_error = 3;
                                    break;
                                case "INVALID_GEOS":
                                case 4:
                                    m.keyword_plan_campaign_error = 4;
                                    break;
                                case "DUPLICATE_NAME":
                                case 5:
                                    m.keyword_plan_campaign_error = 5;
                                    break;
                                case "MAX_GEOS_EXCEEDED":
                                case 6:
                                    m.keyword_plan_campaign_error = 6;
                                    break;
                                case "MAX_LANGUAGES_EXCEEDED":
                                case 7:
                                    m.keyword_plan_campaign_error = 7;
                                    break;
                                }
                                switch (d.keyword_plan_campaign_keyword_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_campaign_keyword_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_campaign_keyword_error = 1;
                                    break;
                                case "CAMPAIGN_KEYWORD_IS_POSITIVE":
                                case 8:
                                    m.keyword_plan_campaign_keyword_error = 8;
                                    break;
                                }
                                switch (d.keyword_plan_ad_group_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_ad_group_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_ad_group_error = 1;
                                    break;
                                case "INVALID_NAME":
                                case 2:
                                    m.keyword_plan_ad_group_error = 2;
                                    break;
                                case "DUPLICATE_NAME":
                                case 3:
                                    m.keyword_plan_ad_group_error = 3;
                                    break;
                                }
                                switch (d.keyword_plan_ad_group_keyword_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_ad_group_keyword_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_ad_group_keyword_error = 1;
                                    break;
                                case "INVALID_KEYWORD_MATCH_TYPE":
                                case 2:
                                    m.keyword_plan_ad_group_keyword_error = 2;
                                    break;
                                case "DUPLICATE_KEYWORD":
                                case 3:
                                    m.keyword_plan_ad_group_keyword_error = 3;
                                    break;
                                case "KEYWORD_TEXT_TOO_LONG":
                                case 4:
                                    m.keyword_plan_ad_group_keyword_error = 4;
                                    break;
                                case "KEYWORD_HAS_INVALID_CHARS":
                                case 5:
                                    m.keyword_plan_ad_group_keyword_error = 5;
                                    break;
                                case "KEYWORD_HAS_TOO_MANY_WORDS":
                                case 6:
                                    m.keyword_plan_ad_group_keyword_error = 6;
                                    break;
                                case "INVALID_KEYWORD_TEXT":
                                case 7:
                                    m.keyword_plan_ad_group_keyword_error = 7;
                                    break;
                                case "NEGATIVE_KEYWORD_HAS_CPC_BID":
                                case 8:
                                    m.keyword_plan_ad_group_keyword_error = 8;
                                    break;
                                case "NEW_BMM_KEYWORDS_NOT_ALLOWED":
                                case 9:
                                    m.keyword_plan_ad_group_keyword_error = 9;
                                    break;
                                }
                                switch (d.keyword_plan_idea_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_idea_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_idea_error = 1;
                                    break;
                                case "URL_CRAWL_ERROR":
                                case 2:
                                    m.keyword_plan_idea_error = 2;
                                    break;
                                case "INVALID_VALUE":
                                case 3:
                                    m.keyword_plan_idea_error = 3;
                                    break;
                                }
                                switch (d.account_budget_proposal_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.account_budget_proposal_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.account_budget_proposal_error = 1;
                                    break;
                                case "FIELD_MASK_NOT_ALLOWED":
                                case 2:
                                    m.account_budget_proposal_error = 2;
                                    break;
                                case "IMMUTABLE_FIELD":
                                case 3:
                                    m.account_budget_proposal_error = 3;
                                    break;
                                case "REQUIRED_FIELD_MISSING":
                                case 4:
                                    m.account_budget_proposal_error = 4;
                                    break;
                                case "CANNOT_CANCEL_APPROVED_PROPOSAL":
                                case 5:
                                    m.account_budget_proposal_error = 5;
                                    break;
                                case "CANNOT_REMOVE_UNAPPROVED_BUDGET":
                                case 6:
                                    m.account_budget_proposal_error = 6;
                                    break;
                                case "CANNOT_REMOVE_RUNNING_BUDGET":
                                case 7:
                                    m.account_budget_proposal_error = 7;
                                    break;
                                case "CANNOT_END_UNAPPROVED_BUDGET":
                                case 8:
                                    m.account_budget_proposal_error = 8;
                                    break;
                                case "CANNOT_END_INACTIVE_BUDGET":
                                case 9:
                                    m.account_budget_proposal_error = 9;
                                    break;
                                case "BUDGET_NAME_REQUIRED":
                                case 10:
                                    m.account_budget_proposal_error = 10;
                                    break;
                                case "CANNOT_UPDATE_OLD_BUDGET":
                                case 11:
                                    m.account_budget_proposal_error = 11;
                                    break;
                                case "CANNOT_END_IN_PAST":
                                case 12:
                                    m.account_budget_proposal_error = 12;
                                    break;
                                case "CANNOT_EXTEND_END_TIME":
                                case 13:
                                    m.account_budget_proposal_error = 13;
                                    break;
                                case "PURCHASE_ORDER_NUMBER_REQUIRED":
                                case 14:
                                    m.account_budget_proposal_error = 14;
                                    break;
                                case "PENDING_UPDATE_PROPOSAL_EXISTS":
                                case 15:
                                    m.account_budget_proposal_error = 15;
                                    break;
                                case "MULTIPLE_BUDGETS_NOT_ALLOWED_FOR_UNAPPROVED_BILLING_SETUP":
                                case 16:
                                    m.account_budget_proposal_error = 16;
                                    break;
                                case "CANNOT_UPDATE_START_TIME_FOR_STARTED_BUDGET":
                                case 17:
                                    m.account_budget_proposal_error = 17;
                                    break;
                                case "SPENDING_LIMIT_LOWER_THAN_ACCRUED_COST_NOT_ALLOWED":
                                case 18:
                                    m.account_budget_proposal_error = 18;
                                    break;
                                case "UPDATE_IS_NO_OP":
                                case 19:
                                    m.account_budget_proposal_error = 19;
                                    break;
                                case "END_TIME_MUST_FOLLOW_START_TIME":
                                case 20:
                                    m.account_budget_proposal_error = 20;
                                    break;
                                case "BUDGET_DATE_RANGE_INCOMPATIBLE_WITH_BILLING_SETUP":
                                case 21:
                                    m.account_budget_proposal_error = 21;
                                    break;
                                case "NOT_AUTHORIZED":
                                case 22:
                                    m.account_budget_proposal_error = 22;
                                    break;
                                case "INVALID_BILLING_SETUP":
                                case 23:
                                    m.account_budget_proposal_error = 23;
                                    break;
                                case "OVERLAPS_EXISTING_BUDGET":
                                case 24:
                                    m.account_budget_proposal_error = 24;
                                    break;
                                case "CANNOT_CREATE_BUDGET_THROUGH_API":
                                case 25:
                                    m.account_budget_proposal_error = 25;
                                    break;
                                }
                                switch (d.user_list_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.user_list_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.user_list_error = 1;
                                    break;
                                case "EXTERNAL_REMARKETING_USER_LIST_MUTATE_NOT_SUPPORTED":
                                case 2:
                                    m.user_list_error = 2;
                                    break;
                                case "CONCRETE_TYPE_REQUIRED":
                                case 3:
                                    m.user_list_error = 3;
                                    break;
                                case "CONVERSION_TYPE_ID_REQUIRED":
                                case 4:
                                    m.user_list_error = 4;
                                    break;
                                case "DUPLICATE_CONVERSION_TYPES":
                                case 5:
                                    m.user_list_error = 5;
                                    break;
                                case "INVALID_CONVERSION_TYPE":
                                case 6:
                                    m.user_list_error = 6;
                                    break;
                                case "INVALID_DESCRIPTION":
                                case 7:
                                    m.user_list_error = 7;
                                    break;
                                case "INVALID_NAME":
                                case 8:
                                    m.user_list_error = 8;
                                    break;
                                case "INVALID_TYPE":
                                case 9:
                                    m.user_list_error = 9;
                                    break;
                                case "CAN_NOT_ADD_LOGICAL_LIST_AS_LOGICAL_LIST_OPERAND":
                                case 10:
                                    m.user_list_error = 10;
                                    break;
                                case "INVALID_USER_LIST_LOGICAL_RULE_OPERAND":
                                case 11:
                                    m.user_list_error = 11;
                                    break;
                                case "NAME_ALREADY_USED":
                                case 12:
                                    m.user_list_error = 12;
                                    break;
                                case "NEW_CONVERSION_TYPE_NAME_REQUIRED":
                                case 13:
                                    m.user_list_error = 13;
                                    break;
                                case "CONVERSION_TYPE_NAME_ALREADY_USED":
                                case 14:
                                    m.user_list_error = 14;
                                    break;
                                case "OWNERSHIP_REQUIRED_FOR_SET":
                                case 15:
                                    m.user_list_error = 15;
                                    break;
                                case "USER_LIST_MUTATE_NOT_SUPPORTED":
                                case 16:
                                    m.user_list_error = 16;
                                    break;
                                case "INVALID_RULE":
                                case 17:
                                    m.user_list_error = 17;
                                    break;
                                case "INVALID_DATE_RANGE":
                                case 27:
                                    m.user_list_error = 27;
                                    break;
                                case "CAN_NOT_MUTATE_SENSITIVE_USERLIST":
                                case 28:
                                    m.user_list_error = 28;
                                    break;
                                case "MAX_NUM_RULEBASED_USERLISTS":
                                case 29:
                                    m.user_list_error = 29;
                                    break;
                                case "CANNOT_MODIFY_BILLABLE_RECORD_COUNT":
                                case 30:
                                    m.user_list_error = 30;
                                    break;
                                case "APP_ID_NOT_SET":
                                case 31:
                                    m.user_list_error = 31;
                                    break;
                                case "USERLIST_NAME_IS_RESERVED_FOR_SYSTEM_LIST":
                                case 32:
                                    m.user_list_error = 32;
                                    break;
                                case "ADVERTISER_NOT_ON_ALLOWLIST_FOR_USING_UPLOADED_DATA":
                                case 37:
                                    m.user_list_error = 37;
                                    break;
                                case "RULE_TYPE_IS_NOT_SUPPORTED":
                                case 34:
                                    m.user_list_error = 34;
                                    break;
                                case "CAN_NOT_ADD_A_SIMILAR_USERLIST_AS_LOGICAL_LIST_OPERAND":
                                case 35:
                                    m.user_list_error = 35;
                                    break;
                                case "CAN_NOT_MIX_CRM_BASED_IN_LOGICAL_LIST_WITH_OTHER_LISTS":
                                case 36:
                                    m.user_list_error = 36;
                                    break;
                                }
                                switch (d.change_event_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.change_event_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.change_event_error = 1;
                                    break;
                                case "START_DATE_TOO_OLD":
                                case 2:
                                    m.change_event_error = 2;
                                    break;
                                case "CHANGE_DATE_RANGE_INFINITE":
                                case 3:
                                    m.change_event_error = 3;
                                    break;
                                case "CHANGE_DATE_RANGE_NEGATIVE":
                                case 4:
                                    m.change_event_error = 4;
                                    break;
                                case "LIMIT_NOT_SPECIFIED":
                                case 5:
                                    m.change_event_error = 5;
                                    break;
                                case "INVALID_LIMIT_CLAUSE":
                                case 6:
                                    m.change_event_error = 6;
                                    break;
                                }
                                switch (d.change_status_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.change_status_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.change_status_error = 1;
                                    break;
                                case "START_DATE_TOO_OLD":
                                case 3:
                                    m.change_status_error = 3;
                                    break;
                                case "CHANGE_DATE_RANGE_INFINITE":
                                case 4:
                                    m.change_status_error = 4;
                                    break;
                                case "CHANGE_DATE_RANGE_NEGATIVE":
                                case 5:
                                    m.change_status_error = 5;
                                    break;
                                case "LIMIT_NOT_SPECIFIED":
                                case 6:
                                    m.change_status_error = 6;
                                    break;
                                case "INVALID_LIMIT_CLAUSE":
                                case 7:
                                    m.change_status_error = 7;
                                    break;
                                }
                                switch (d.feed_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_error = 1;
                                    break;
                                case "ATTRIBUTE_NAMES_NOT_UNIQUE":
                                case 2:
                                    m.feed_error = 2;
                                    break;
                                case "ATTRIBUTES_DO_NOT_MATCH_EXISTING_ATTRIBUTES":
                                case 3:
                                    m.feed_error = 3;
                                    break;
                                case "CANNOT_SPECIFY_USER_ORIGIN_FOR_SYSTEM_FEED":
                                case 4:
                                    m.feed_error = 4;
                                    break;
                                case "CANNOT_SPECIFY_GOOGLE_ORIGIN_FOR_NON_SYSTEM_FEED":
                                case 5:
                                    m.feed_error = 5;
                                    break;
                                case "CANNOT_SPECIFY_FEED_ATTRIBUTES_FOR_SYSTEM_FEED":
                                case 6:
                                    m.feed_error = 6;
                                    break;
                                case "CANNOT_UPDATE_FEED_ATTRIBUTES_WITH_ORIGIN_GOOGLE":
                                case 7:
                                    m.feed_error = 7;
                                    break;
                                case "FEED_REMOVED":
                                case 8:
                                    m.feed_error = 8;
                                    break;
                                case "INVALID_ORIGIN_VALUE":
                                case 9:
                                    m.feed_error = 9;
                                    break;
                                case "FEED_ORIGIN_IS_NOT_USER":
                                case 10:
                                    m.feed_error = 10;
                                    break;
                                case "INVALID_AUTH_TOKEN_FOR_EMAIL":
                                case 11:
                                    m.feed_error = 11;
                                    break;
                                case "INVALID_EMAIL":
                                case 12:
                                    m.feed_error = 12;
                                    break;
                                case "DUPLICATE_FEED_NAME":
                                case 13:
                                    m.feed_error = 13;
                                    break;
                                case "INVALID_FEED_NAME":
                                case 14:
                                    m.feed_error = 14;
                                    break;
                                case "MISSING_OAUTH_INFO":
                                case 15:
                                    m.feed_error = 15;
                                    break;
                                case "NEW_ATTRIBUTE_CANNOT_BE_PART_OF_UNIQUE_KEY":
                                case 16:
                                    m.feed_error = 16;
                                    break;
                                case "TOO_MANY_ATTRIBUTES":
                                case 17:
                                    m.feed_error = 17;
                                    break;
                                case "INVALID_BUSINESS_ACCOUNT":
                                case 18:
                                    m.feed_error = 18;
                                    break;
                                case "BUSINESS_ACCOUNT_CANNOT_ACCESS_LOCATION_ACCOUNT":
                                case 19:
                                    m.feed_error = 19;
                                    break;
                                case "INVALID_AFFILIATE_CHAIN_ID":
                                case 20:
                                    m.feed_error = 20;
                                    break;
                                case "DUPLICATE_SYSTEM_FEED":
                                case 21:
                                    m.feed_error = 21;
                                    break;
                                case "GMB_ACCESS_ERROR":
                                case 22:
                                    m.feed_error = 22;
                                    break;
                                case "CANNOT_HAVE_LOCATION_AND_AFFILIATE_LOCATION_FEEDS":
                                case 23:
                                    m.feed_error = 23;
                                    break;
                                case "LEGACY_EXTENSION_TYPE_READ_ONLY":
                                case 24:
                                    m.feed_error = 24;
                                    break;
                                }
                                switch (d.geo_target_constant_suggestion_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.geo_target_constant_suggestion_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.geo_target_constant_suggestion_error = 1;
                                    break;
                                case "LOCATION_NAME_SIZE_LIMIT":
                                case 2:
                                    m.geo_target_constant_suggestion_error = 2;
                                    break;
                                case "LOCATION_NAME_LIMIT":
                                case 3:
                                    m.geo_target_constant_suggestion_error = 3;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 4:
                                    m.geo_target_constant_suggestion_error = 4;
                                    break;
                                case "REQUEST_PARAMETERS_UNSET":
                                case 5:
                                    m.geo_target_constant_suggestion_error = 5;
                                    break;
                                }
                                switch (d.campaign_draft_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_draft_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_draft_error = 1;
                                    break;
                                case "DUPLICATE_DRAFT_NAME":
                                case 2:
                                    m.campaign_draft_error = 2;
                                    break;
                                case "INVALID_STATUS_TRANSITION_FROM_REMOVED":
                                case 3:
                                    m.campaign_draft_error = 3;
                                    break;
                                case "INVALID_STATUS_TRANSITION_FROM_PROMOTED":
                                case 4:
                                    m.campaign_draft_error = 4;
                                    break;
                                case "INVALID_STATUS_TRANSITION_FROM_PROMOTE_FAILED":
                                case 5:
                                    m.campaign_draft_error = 5;
                                    break;
                                case "CUSTOMER_CANNOT_CREATE_DRAFT":
                                case 6:
                                    m.campaign_draft_error = 6;
                                    break;
                                case "CAMPAIGN_CANNOT_CREATE_DRAFT":
                                case 7:
                                    m.campaign_draft_error = 7;
                                    break;
                                case "INVALID_DRAFT_CHANGE":
                                case 8:
                                    m.campaign_draft_error = 8;
                                    break;
                                case "INVALID_STATUS_TRANSITION":
                                case 9:
                                    m.campaign_draft_error = 9;
                                    break;
                                case "MAX_NUMBER_OF_DRAFTS_PER_CAMPAIGN_REACHED":
                                case 10:
                                    m.campaign_draft_error = 10;
                                    break;
                                case "LIST_ERRORS_FOR_PROMOTED_DRAFT_ONLY":
                                case 11:
                                    m.campaign_draft_error = 11;
                                    break;
                                }
                                switch (d.feed_item_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_item_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_item_error = 1;
                                    break;
                                case "CANNOT_CONVERT_ATTRIBUTE_VALUE_FROM_STRING":
                                case 2:
                                    m.feed_item_error = 2;
                                    break;
                                case "CANNOT_OPERATE_ON_REMOVED_FEED_ITEM":
                                case 3:
                                    m.feed_item_error = 3;
                                    break;
                                case "DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE":
                                case 4:
                                    m.feed_item_error = 4;
                                    break;
                                case "KEY_ATTRIBUTES_NOT_FOUND":
                                case 5:
                                    m.feed_item_error = 5;
                                    break;
                                case "INVALID_URL":
                                case 6:
                                    m.feed_item_error = 6;
                                    break;
                                case "MISSING_KEY_ATTRIBUTES":
                                case 7:
                                    m.feed_item_error = 7;
                                    break;
                                case "KEY_ATTRIBUTES_NOT_UNIQUE":
                                case 8:
                                    m.feed_item_error = 8;
                                    break;
                                case "CANNOT_MODIFY_KEY_ATTRIBUTE_VALUE":
                                case 9:
                                    m.feed_item_error = 9;
                                    break;
                                case "SIZE_TOO_LARGE_FOR_MULTI_VALUE_ATTRIBUTE":
                                case 10:
                                    m.feed_item_error = 10;
                                    break;
                                }
                                switch (d.label_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.label_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.label_error = 1;
                                    break;
                                case "CANNOT_APPLY_INACTIVE_LABEL":
                                case 2:
                                    m.label_error = 2;
                                    break;
                                case "CANNOT_APPLY_LABEL_TO_DISABLED_AD_GROUP_CRITERION":
                                case 3:
                                    m.label_error = 3;
                                    break;
                                case "CANNOT_APPLY_LABEL_TO_NEGATIVE_AD_GROUP_CRITERION":
                                case 4:
                                    m.label_error = 4;
                                    break;
                                case "EXCEEDED_LABEL_LIMIT_PER_TYPE":
                                case 5:
                                    m.label_error = 5;
                                    break;
                                case "INVALID_RESOURCE_FOR_MANAGER_LABEL":
                                case 6:
                                    m.label_error = 6;
                                    break;
                                case "DUPLICATE_NAME":
                                case 7:
                                    m.label_error = 7;
                                    break;
                                case "INVALID_LABEL_NAME":
                                case 8:
                                    m.label_error = 8;
                                    break;
                                case "CANNOT_ATTACH_LABEL_TO_DRAFT":
                                case 9:
                                    m.label_error = 9;
                                    break;
                                case "CANNOT_ATTACH_NON_MANAGER_LABEL_TO_CUSTOMER":
                                case 10:
                                    m.label_error = 10;
                                    break;
                                }
                                switch (d.billing_setup_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.billing_setup_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.billing_setup_error = 1;
                                    break;
                                case "CANNOT_USE_EXISTING_AND_NEW_ACCOUNT":
                                case 2:
                                    m.billing_setup_error = 2;
                                    break;
                                case "CANNOT_REMOVE_STARTED_BILLING_SETUP":
                                case 3:
                                    m.billing_setup_error = 3;
                                    break;
                                case "CANNOT_CHANGE_BILLING_TO_SAME_PAYMENTS_ACCOUNT":
                                case 4:
                                    m.billing_setup_error = 4;
                                    break;
                                case "BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_STATUS":
                                case 5:
                                    m.billing_setup_error = 5;
                                    break;
                                case "INVALID_PAYMENTS_ACCOUNT":
                                case 6:
                                    m.billing_setup_error = 6;
                                    break;
                                case "BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_CATEGORY":
                                case 7:
                                    m.billing_setup_error = 7;
                                    break;
                                case "INVALID_START_TIME_TYPE":
                                case 8:
                                    m.billing_setup_error = 8;
                                    break;
                                case "THIRD_PARTY_ALREADY_HAS_BILLING":
                                case 9:
                                    m.billing_setup_error = 9;
                                    break;
                                case "BILLING_SETUP_IN_PROGRESS":
                                case 10:
                                    m.billing_setup_error = 10;
                                    break;
                                case "NO_SIGNUP_PERMISSION":
                                case 11:
                                    m.billing_setup_error = 11;
                                    break;
                                case "CHANGE_OF_BILL_TO_IN_PROGRESS":
                                case 12:
                                    m.billing_setup_error = 12;
                                    break;
                                case "PAYMENTS_PROFILE_NOT_FOUND":
                                case 13:
                                    m.billing_setup_error = 13;
                                    break;
                                case "PAYMENTS_ACCOUNT_NOT_FOUND":
                                case 14:
                                    m.billing_setup_error = 14;
                                    break;
                                case "PAYMENTS_PROFILE_INELIGIBLE":
                                case 15:
                                    m.billing_setup_error = 15;
                                    break;
                                case "PAYMENTS_ACCOUNT_INELIGIBLE":
                                case 16:
                                    m.billing_setup_error = 16;
                                    break;
                                case "CUSTOMER_NEEDS_INTERNAL_APPROVAL":
                                case 17:
                                    m.billing_setup_error = 17;
                                    break;
                                case "PAYMENTS_ACCOUNT_INELIGIBLE_CURRENCY_CODE_MISMATCH":
                                case 19:
                                    m.billing_setup_error = 19;
                                    break;
                                case "FUTURE_START_TIME_PROHIBITED":
                                case 20:
                                    m.billing_setup_error = 20;
                                    break;
                                }
                                switch (d.customer_client_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.customer_client_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.customer_client_link_error = 1;
                                    break;
                                case "CLIENT_ALREADY_INVITED_BY_THIS_MANAGER":
                                case 2:
                                    m.customer_client_link_error = 2;
                                    break;
                                case "CLIENT_ALREADY_MANAGED_IN_HIERARCHY":
                                case 3:
                                    m.customer_client_link_error = 3;
                                    break;
                                case "CYCLIC_LINK_NOT_ALLOWED":
                                case 4:
                                    m.customer_client_link_error = 4;
                                    break;
                                case "CUSTOMER_HAS_TOO_MANY_ACCOUNTS":
                                case 5:
                                    m.customer_client_link_error = 5;
                                    break;
                                case "CLIENT_HAS_TOO_MANY_INVITATIONS":
                                case 6:
                                    m.customer_client_link_error = 6;
                                    break;
                                case "CANNOT_HIDE_OR_UNHIDE_MANAGER_ACCOUNTS":
                                case 7:
                                    m.customer_client_link_error = 7;
                                    break;
                                case "CUSTOMER_HAS_TOO_MANY_ACCOUNTS_AT_MANAGER":
                                case 8:
                                    m.customer_client_link_error = 8;
                                    break;
                                case "CLIENT_HAS_TOO_MANY_MANAGERS":
                                case 9:
                                    m.customer_client_link_error = 9;
                                    break;
                                }
                                switch (d.customer_manager_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.customer_manager_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.customer_manager_link_error = 1;
                                    break;
                                case "NO_PENDING_INVITE":
                                case 2:
                                    m.customer_manager_link_error = 2;
                                    break;
                                case "SAME_CLIENT_MORE_THAN_ONCE_PER_CALL":
                                case 3:
                                    m.customer_manager_link_error = 3;
                                    break;
                                case "MANAGER_HAS_MAX_NUMBER_OF_LINKED_ACCOUNTS":
                                case 4:
                                    m.customer_manager_link_error = 4;
                                    break;
                                case "CANNOT_UNLINK_ACCOUNT_WITHOUT_ACTIVE_USER":
                                case 5:
                                    m.customer_manager_link_error = 5;
                                    break;
                                case "CANNOT_REMOVE_LAST_CLIENT_ACCOUNT_OWNER":
                                case 6:
                                    m.customer_manager_link_error = 6;
                                    break;
                                case "CANNOT_CHANGE_ROLE_BY_NON_ACCOUNT_OWNER":
                                case 7:
                                    m.customer_manager_link_error = 7;
                                    break;
                                case "CANNOT_CHANGE_ROLE_FOR_NON_ACTIVE_LINK_ACCOUNT":
                                case 8:
                                    m.customer_manager_link_error = 8;
                                    break;
                                case "DUPLICATE_CHILD_FOUND":
                                case 9:
                                    m.customer_manager_link_error = 9;
                                    break;
                                case "TEST_ACCOUNT_LINKS_TOO_MANY_CHILD_ACCOUNTS":
                                case 10:
                                    m.customer_manager_link_error = 10;
                                    break;
                                }
                                switch (d.feed_mapping_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_mapping_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_mapping_error = 1;
                                    break;
                                case "INVALID_PLACEHOLDER_FIELD":
                                case 2:
                                    m.feed_mapping_error = 2;
                                    break;
                                case "INVALID_CRITERION_FIELD":
                                case 3:
                                    m.feed_mapping_error = 3;
                                    break;
                                case "INVALID_PLACEHOLDER_TYPE":
                                case 4:
                                    m.feed_mapping_error = 4;
                                    break;
                                case "INVALID_CRITERION_TYPE":
                                case 5:
                                    m.feed_mapping_error = 5;
                                    break;
                                case "NO_ATTRIBUTE_FIELD_MAPPINGS":
                                case 7:
                                    m.feed_mapping_error = 7;
                                    break;
                                case "FEED_ATTRIBUTE_TYPE_MISMATCH":
                                case 8:
                                    m.feed_mapping_error = 8;
                                    break;
                                case "CANNOT_OPERATE_ON_MAPPINGS_FOR_SYSTEM_GENERATED_FEED":
                                case 9:
                                    m.feed_mapping_error = 9;
                                    break;
                                case "MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_TYPE":
                                case 10:
                                    m.feed_mapping_error = 10;
                                    break;
                                case "MULTIPLE_MAPPINGS_FOR_CRITERION_TYPE":
                                case 11:
                                    m.feed_mapping_error = 11;
                                    break;
                                case "MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_FIELD":
                                case 12:
                                    m.feed_mapping_error = 12;
                                    break;
                                case "MULTIPLE_MAPPINGS_FOR_CRITERION_FIELD":
                                case 13:
                                    m.feed_mapping_error = 13;
                                    break;
                                case "UNEXPECTED_ATTRIBUTE_FIELD_MAPPINGS":
                                case 14:
                                    m.feed_mapping_error = 14;
                                    break;
                                case "LOCATION_PLACEHOLDER_ONLY_FOR_PLACES_FEEDS":
                                case 15:
                                    m.feed_mapping_error = 15;
                                    break;
                                case "CANNOT_MODIFY_MAPPINGS_FOR_TYPED_FEED":
                                case 16:
                                    m.feed_mapping_error = 16;
                                    break;
                                case "INVALID_PLACEHOLDER_TYPE_FOR_NON_SYSTEM_GENERATED_FEED":
                                case 17:
                                    m.feed_mapping_error = 17;
                                    break;
                                case "INVALID_PLACEHOLDER_TYPE_FOR_SYSTEM_GENERATED_FEED_TYPE":
                                case 18:
                                    m.feed_mapping_error = 18;
                                    break;
                                case "ATTRIBUTE_FIELD_MAPPING_MISSING_FIELD":
                                case 19:
                                    m.feed_mapping_error = 19;
                                    break;
                                }
                                switch (d.customer_feed_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.customer_feed_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.customer_feed_error = 1;
                                    break;
                                case "FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE":
                                case 2:
                                    m.customer_feed_error = 2;
                                    break;
                                case "CANNOT_CREATE_FOR_REMOVED_FEED":
                                case 3:
                                    m.customer_feed_error = 3;
                                    break;
                                case "CANNOT_CREATE_ALREADY_EXISTING_CUSTOMER_FEED":
                                case 4:
                                    m.customer_feed_error = 4;
                                    break;
                                case "CANNOT_MODIFY_REMOVED_CUSTOMER_FEED":
                                case 5:
                                    m.customer_feed_error = 5;
                                    break;
                                case "INVALID_PLACEHOLDER_TYPE":
                                case 6:
                                    m.customer_feed_error = 6;
                                    break;
                                case "MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE":
                                case 7:
                                    m.customer_feed_error = 7;
                                    break;
                                case "PLACEHOLDER_TYPE_NOT_ALLOWED_ON_CUSTOMER_FEED":
                                case 8:
                                    m.customer_feed_error = 8;
                                    break;
                                }
                                switch (d.ad_group_feed_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_group_feed_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_group_feed_error = 1;
                                    break;
                                case "FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE":
                                case 2:
                                    m.ad_group_feed_error = 2;
                                    break;
                                case "CANNOT_CREATE_FOR_REMOVED_FEED":
                                case 3:
                                    m.ad_group_feed_error = 3;
                                    break;
                                case "ADGROUP_FEED_ALREADY_EXISTS":
                                case 4:
                                    m.ad_group_feed_error = 4;
                                    break;
                                case "CANNOT_OPERATE_ON_REMOVED_ADGROUP_FEED":
                                case 5:
                                    m.ad_group_feed_error = 5;
                                    break;
                                case "INVALID_PLACEHOLDER_TYPE":
                                case 6:
                                    m.ad_group_feed_error = 6;
                                    break;
                                case "MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE":
                                case 7:
                                    m.ad_group_feed_error = 7;
                                    break;
                                case "NO_EXISTING_LOCATION_CUSTOMER_FEED":
                                case 8:
                                    m.ad_group_feed_error = 8;
                                    break;
                                }
                                switch (d.campaign_feed_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_feed_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_feed_error = 1;
                                    break;
                                case "FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE":
                                case 2:
                                    m.campaign_feed_error = 2;
                                    break;
                                case "CANNOT_CREATE_FOR_REMOVED_FEED":
                                case 4:
                                    m.campaign_feed_error = 4;
                                    break;
                                case "CANNOT_CREATE_ALREADY_EXISTING_CAMPAIGN_FEED":
                                case 5:
                                    m.campaign_feed_error = 5;
                                    break;
                                case "CANNOT_MODIFY_REMOVED_CAMPAIGN_FEED":
                                case 6:
                                    m.campaign_feed_error = 6;
                                    break;
                                case "INVALID_PLACEHOLDER_TYPE":
                                case 7:
                                    m.campaign_feed_error = 7;
                                    break;
                                case "MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE":
                                case 8:
                                    m.campaign_feed_error = 8;
                                    break;
                                case "NO_EXISTING_LOCATION_CUSTOMER_FEED":
                                case 9:
                                    m.campaign_feed_error = 9;
                                    break;
                                }
                                switch (d.custom_interest_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.custom_interest_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.custom_interest_error = 1;
                                    break;
                                case "NAME_ALREADY_USED":
                                case 2:
                                    m.custom_interest_error = 2;
                                    break;
                                case "CUSTOM_INTEREST_MEMBER_ID_AND_TYPE_PARAMETER_NOT_PRESENT_IN_REMOVE":
                                case 3:
                                    m.custom_interest_error = 3;
                                    break;
                                case "TYPE_AND_PARAMETER_NOT_FOUND":
                                case 4:
                                    m.custom_interest_error = 4;
                                    break;
                                case "TYPE_AND_PARAMETER_ALREADY_EXISTED":
                                case 5:
                                    m.custom_interest_error = 5;
                                    break;
                                case "INVALID_CUSTOM_INTEREST_MEMBER_TYPE":
                                case 6:
                                    m.custom_interest_error = 6;
                                    break;
                                case "CANNOT_REMOVE_WHILE_IN_USE":
                                case 7:
                                    m.custom_interest_error = 7;
                                    break;
                                case "CANNOT_CHANGE_TYPE":
                                case 8:
                                    m.custom_interest_error = 8;
                                    break;
                                }
                                switch (d.campaign_experiment_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.campaign_experiment_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.campaign_experiment_error = 1;
                                    break;
                                case "DUPLICATE_NAME":
                                case 2:
                                    m.campaign_experiment_error = 2;
                                    break;
                                case "INVALID_TRANSITION":
                                case 3:
                                    m.campaign_experiment_error = 3;
                                    break;
                                case "CANNOT_CREATE_EXPERIMENT_WITH_SHARED_BUDGET":
                                case 4:
                                    m.campaign_experiment_error = 4;
                                    break;
                                case "CANNOT_CREATE_EXPERIMENT_FOR_REMOVED_BASE_CAMPAIGN":
                                case 5:
                                    m.campaign_experiment_error = 5;
                                    break;
                                case "CANNOT_CREATE_EXPERIMENT_FOR_NON_PROPOSED_DRAFT":
                                case 6:
                                    m.campaign_experiment_error = 6;
                                    break;
                                case "CUSTOMER_CANNOT_CREATE_EXPERIMENT":
                                case 7:
                                    m.campaign_experiment_error = 7;
                                    break;
                                case "CAMPAIGN_CANNOT_CREATE_EXPERIMENT":
                                case 8:
                                    m.campaign_experiment_error = 8;
                                    break;
                                case "EXPERIMENT_DURATIONS_MUST_NOT_OVERLAP":
                                case 9:
                                    m.campaign_experiment_error = 9;
                                    break;
                                case "EXPERIMENT_DURATION_MUST_BE_WITHIN_CAMPAIGN_DURATION":
                                case 10:
                                    m.campaign_experiment_error = 10;
                                    break;
                                case "CANNOT_MUTATE_EXPERIMENT_DUE_TO_STATUS":
                                case 11:
                                    m.campaign_experiment_error = 11;
                                    break;
                                }
                                switch (d.extension_feed_item_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.extension_feed_item_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.extension_feed_item_error = 1;
                                    break;
                                case "VALUE_OUT_OF_RANGE":
                                case 2:
                                    m.extension_feed_item_error = 2;
                                    break;
                                case "URL_LIST_TOO_LONG":
                                case 3:
                                    m.extension_feed_item_error = 3;
                                    break;
                                case "CANNOT_HAVE_RESTRICTION_ON_EMPTY_GEO_TARGETING":
                                case 4:
                                    m.extension_feed_item_error = 4;
                                    break;
                                case "CANNOT_SET_WITH_FINAL_URLS":
                                case 5:
                                    m.extension_feed_item_error = 5;
                                    break;
                                case "CANNOT_SET_WITHOUT_FINAL_URLS":
                                case 6:
                                    m.extension_feed_item_error = 6;
                                    break;
                                case "INVALID_PHONE_NUMBER":
                                case 7:
                                    m.extension_feed_item_error = 7;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY":
                                case 8:
                                    m.extension_feed_item_error = 8;
                                    break;
                                case "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED":
                                case 9:
                                    m.extension_feed_item_error = 9;
                                    break;
                                case "PREMIUM_RATE_NUMBER_NOT_ALLOWED":
                                case 10:
                                    m.extension_feed_item_error = 10;
                                    break;
                                case "DISALLOWED_NUMBER_TYPE":
                                case 11:
                                    m.extension_feed_item_error = 11;
                                    break;
                                case "INVALID_DOMESTIC_PHONE_NUMBER_FORMAT":
                                case 12:
                                    m.extension_feed_item_error = 12;
                                    break;
                                case "VANITY_PHONE_NUMBER_NOT_ALLOWED":
                                case 13:
                                    m.extension_feed_item_error = 13;
                                    break;
                                case "INVALID_CALL_CONVERSION_ACTION":
                                case 14:
                                    m.extension_feed_item_error = 14;
                                    break;
                                case "CUSTOMER_NOT_ON_ALLOWLIST_FOR_CALLTRACKING":
                                case 47:
                                    m.extension_feed_item_error = 47;
                                    break;
                                case "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY":
                                case 16:
                                    m.extension_feed_item_error = 16;
                                    break;
                                case "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED":
                                case 17:
                                    m.extension_feed_item_error = 17;
                                    break;
                                case "INVALID_APP_ID":
                                case 18:
                                    m.extension_feed_item_error = 18;
                                    break;
                                case "QUOTES_IN_REVIEW_EXTENSION_SNIPPET":
                                case 19:
                                    m.extension_feed_item_error = 19;
                                    break;
                                case "HYPHENS_IN_REVIEW_EXTENSION_SNIPPET":
                                case 20:
                                    m.extension_feed_item_error = 20;
                                    break;
                                case "REVIEW_EXTENSION_SOURCE_INELIGIBLE":
                                case 21:
                                    m.extension_feed_item_error = 21;
                                    break;
                                case "SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT":
                                case 22:
                                    m.extension_feed_item_error = 22;
                                    break;
                                case "INCONSISTENT_CURRENCY_CODES":
                                case 23:
                                    m.extension_feed_item_error = 23;
                                    break;
                                case "PRICE_EXTENSION_HAS_DUPLICATED_HEADERS":
                                case 24:
                                    m.extension_feed_item_error = 24;
                                    break;
                                case "PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION":
                                case 25:
                                    m.extension_feed_item_error = 25;
                                    break;
                                case "PRICE_EXTENSION_HAS_TOO_FEW_ITEMS":
                                case 26:
                                    m.extension_feed_item_error = 26;
                                    break;
                                case "PRICE_EXTENSION_HAS_TOO_MANY_ITEMS":
                                case 27:
                                    m.extension_feed_item_error = 27;
                                    break;
                                case "UNSUPPORTED_VALUE":
                                case 28:
                                    m.extension_feed_item_error = 28;
                                    break;
                                case "UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE":
                                case 29:
                                    m.extension_feed_item_error = 29;
                                    break;
                                case "INVALID_DEVICE_PREFERENCE":
                                case 30:
                                    m.extension_feed_item_error = 30;
                                    break;
                                case "INVALID_SCHEDULE_END":
                                case 31:
                                    m.extension_feed_item_error = 31;
                                    break;
                                case "DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE":
                                case 32:
                                    m.extension_feed_item_error = 32;
                                    break;
                                case "INVALID_SNIPPETS_HEADER":
                                case 33:
                                    m.extension_feed_item_error = 33;
                                    break;
                                case "CANNOT_OPERATE_ON_REMOVED_FEED_ITEM":
                                case 34:
                                    m.extension_feed_item_error = 34;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY":
                                case 35:
                                    m.extension_feed_item_error = 35;
                                    break;
                                case "CONFLICTING_CALL_CONVERSION_SETTINGS":
                                case 36:
                                    m.extension_feed_item_error = 36;
                                    break;
                                case "EXTENSION_TYPE_MISMATCH":
                                case 37:
                                    m.extension_feed_item_error = 37;
                                    break;
                                case "EXTENSION_SUBTYPE_REQUIRED":
                                case 38:
                                    m.extension_feed_item_error = 38;
                                    break;
                                case "EXTENSION_TYPE_UNSUPPORTED":
                                case 39:
                                    m.extension_feed_item_error = 39;
                                    break;
                                case "CANNOT_OPERATE_ON_FEED_WITH_MULTIPLE_MAPPINGS":
                                case 40:
                                    m.extension_feed_item_error = 40;
                                    break;
                                case "CANNOT_OPERATE_ON_FEED_WITH_KEY_ATTRIBUTES":
                                case 41:
                                    m.extension_feed_item_error = 41;
                                    break;
                                case "INVALID_PRICE_FORMAT":
                                case 42:
                                    m.extension_feed_item_error = 42;
                                    break;
                                case "PROMOTION_INVALID_TIME":
                                case 43:
                                    m.extension_feed_item_error = 43;
                                    break;
                                case "TOO_MANY_DECIMAL_PLACES_SPECIFIED":
                                case 44:
                                    m.extension_feed_item_error = 44;
                                    break;
                                case "CONCRETE_EXTENSION_TYPE_REQUIRED":
                                case 45:
                                    m.extension_feed_item_error = 45;
                                    break;
                                case "SCHEDULE_END_NOT_AFTER_START":
                                case 46:
                                    m.extension_feed_item_error = 46;
                                    break;
                                }
                                switch (d.ad_parameter_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_parameter_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_parameter_error = 1;
                                    break;
                                case "AD_GROUP_CRITERION_MUST_BE_KEYWORD":
                                case 2:
                                    m.ad_parameter_error = 2;
                                    break;
                                case "INVALID_INSERTION_TEXT_FORMAT":
                                case 3:
                                    m.ad_parameter_error = 3;
                                    break;
                                }
                                switch (d.feed_item_validation_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_item_validation_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_item_validation_error = 1;
                                    break;
                                case "STRING_TOO_SHORT":
                                case 2:
                                    m.feed_item_validation_error = 2;
                                    break;
                                case "STRING_TOO_LONG":
                                case 3:
                                    m.feed_item_validation_error = 3;
                                    break;
                                case "VALUE_NOT_SPECIFIED":
                                case 4:
                                    m.feed_item_validation_error = 4;
                                    break;
                                case "INVALID_DOMESTIC_PHONE_NUMBER_FORMAT":
                                case 5:
                                    m.feed_item_validation_error = 5;
                                    break;
                                case "INVALID_PHONE_NUMBER":
                                case 6:
                                    m.feed_item_validation_error = 6;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY":
                                case 7:
                                    m.feed_item_validation_error = 7;
                                    break;
                                case "PREMIUM_RATE_NUMBER_NOT_ALLOWED":
                                case 8:
                                    m.feed_item_validation_error = 8;
                                    break;
                                case "DISALLOWED_NUMBER_TYPE":
                                case 9:
                                    m.feed_item_validation_error = 9;
                                    break;
                                case "VALUE_OUT_OF_RANGE":
                                case 10:
                                    m.feed_item_validation_error = 10;
                                    break;
                                case "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY":
                                case 11:
                                    m.feed_item_validation_error = 11;
                                    break;
                                case "CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING":
                                case 99:
                                    m.feed_item_validation_error = 99;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 13:
                                    m.feed_item_validation_error = 13;
                                    break;
                                case "INVALID_APP_ID":
                                case 14:
                                    m.feed_item_validation_error = 14;
                                    break;
                                case "MISSING_ATTRIBUTES_FOR_FIELDS":
                                case 15:
                                    m.feed_item_validation_error = 15;
                                    break;
                                case "INVALID_TYPE_ID":
                                case 16:
                                    m.feed_item_validation_error = 16;
                                    break;
                                case "INVALID_EMAIL_ADDRESS":
                                case 17:
                                    m.feed_item_validation_error = 17;
                                    break;
                                case "INVALID_HTTPS_URL":
                                case 18:
                                    m.feed_item_validation_error = 18;
                                    break;
                                case "MISSING_DELIVERY_ADDRESS":
                                case 19:
                                    m.feed_item_validation_error = 19;
                                    break;
                                case "START_DATE_AFTER_END_DATE":
                                case 20:
                                    m.feed_item_validation_error = 20;
                                    break;
                                case "MISSING_FEED_ITEM_START_TIME":
                                case 21:
                                    m.feed_item_validation_error = 21;
                                    break;
                                case "MISSING_FEED_ITEM_END_TIME":
                                case 22:
                                    m.feed_item_validation_error = 22;
                                    break;
                                case "MISSING_FEED_ITEM_ID":
                                case 23:
                                    m.feed_item_validation_error = 23;
                                    break;
                                case "VANITY_PHONE_NUMBER_NOT_ALLOWED":
                                case 24:
                                    m.feed_item_validation_error = 24;
                                    break;
                                case "INVALID_REVIEW_EXTENSION_SNIPPET":
                                case 25:
                                    m.feed_item_validation_error = 25;
                                    break;
                                case "INVALID_NUMBER_FORMAT":
                                case 26:
                                    m.feed_item_validation_error = 26;
                                    break;
                                case "INVALID_DATE_FORMAT":
                                case 27:
                                    m.feed_item_validation_error = 27;
                                    break;
                                case "INVALID_PRICE_FORMAT":
                                case 28:
                                    m.feed_item_validation_error = 28;
                                    break;
                                case "UNKNOWN_PLACEHOLDER_FIELD":
                                case 29:
                                    m.feed_item_validation_error = 29;
                                    break;
                                case "MISSING_ENHANCED_SITELINK_DESCRIPTION_LINE":
                                case 30:
                                    m.feed_item_validation_error = 30;
                                    break;
                                case "REVIEW_EXTENSION_SOURCE_INELIGIBLE":
                                case 31:
                                    m.feed_item_validation_error = 31;
                                    break;
                                case "HYPHENS_IN_REVIEW_EXTENSION_SNIPPET":
                                case 32:
                                    m.feed_item_validation_error = 32;
                                    break;
                                case "DOUBLE_QUOTES_IN_REVIEW_EXTENSION_SNIPPET":
                                case 33:
                                    m.feed_item_validation_error = 33;
                                    break;
                                case "QUOTES_IN_REVIEW_EXTENSION_SNIPPET":
                                case 34:
                                    m.feed_item_validation_error = 34;
                                    break;
                                case "INVALID_FORM_ENCODED_PARAMS":
                                case 35:
                                    m.feed_item_validation_error = 35;
                                    break;
                                case "INVALID_URL_PARAMETER_NAME":
                                case 36:
                                    m.feed_item_validation_error = 36;
                                    break;
                                case "NO_GEOCODING_RESULT":
                                case 37:
                                    m.feed_item_validation_error = 37;
                                    break;
                                case "SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT":
                                case 38:
                                    m.feed_item_validation_error = 38;
                                    break;
                                case "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED":
                                case 39:
                                    m.feed_item_validation_error = 39;
                                    break;
                                case "INVALID_PLACEHOLDER_FIELD_ID":
                                case 40:
                                    m.feed_item_validation_error = 40;
                                    break;
                                case "INVALID_URL_TAG":
                                case 41:
                                    m.feed_item_validation_error = 41;
                                    break;
                                case "LIST_TOO_LONG":
                                case 42:
                                    m.feed_item_validation_error = 42;
                                    break;
                                case "INVALID_ATTRIBUTES_COMBINATION":
                                case 43:
                                    m.feed_item_validation_error = 43;
                                    break;
                                case "DUPLICATE_VALUES":
                                case 44:
                                    m.feed_item_validation_error = 44;
                                    break;
                                case "INVALID_CALL_CONVERSION_ACTION_ID":
                                case 45:
                                    m.feed_item_validation_error = 45;
                                    break;
                                case "CANNOT_SET_WITHOUT_FINAL_URLS":
                                case 46:
                                    m.feed_item_validation_error = 46;
                                    break;
                                case "APP_ID_DOESNT_EXIST_IN_APP_STORE":
                                case 47:
                                    m.feed_item_validation_error = 47;
                                    break;
                                case "INVALID_FINAL_URL":
                                case 48:
                                    m.feed_item_validation_error = 48;
                                    break;
                                case "INVALID_TRACKING_URL":
                                case 49:
                                    m.feed_item_validation_error = 49;
                                    break;
                                case "INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL":
                                case 50:
                                    m.feed_item_validation_error = 50;
                                    break;
                                case "LIST_TOO_SHORT":
                                case 51:
                                    m.feed_item_validation_error = 51;
                                    break;
                                case "INVALID_USER_ACTION":
                                case 52:
                                    m.feed_item_validation_error = 52;
                                    break;
                                case "INVALID_TYPE_NAME":
                                case 53:
                                    m.feed_item_validation_error = 53;
                                    break;
                                case "INVALID_EVENT_CHANGE_STATUS":
                                case 54:
                                    m.feed_item_validation_error = 54;
                                    break;
                                case "INVALID_SNIPPETS_HEADER":
                                case 55:
                                    m.feed_item_validation_error = 55;
                                    break;
                                case "INVALID_ANDROID_APP_LINK":
                                case 56:
                                    m.feed_item_validation_error = 56;
                                    break;
                                case "NUMBER_TYPE_WITH_CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY":
                                case 57:
                                    m.feed_item_validation_error = 57;
                                    break;
                                case "RESERVED_KEYWORD_OTHER":
                                case 58:
                                    m.feed_item_validation_error = 58;
                                    break;
                                case "DUPLICATE_OPTION_LABELS":
                                case 59:
                                    m.feed_item_validation_error = 59;
                                    break;
                                case "DUPLICATE_OPTION_PREFILLS":
                                case 60:
                                    m.feed_item_validation_error = 60;
                                    break;
                                case "UNEQUAL_LIST_LENGTHS":
                                case 61:
                                    m.feed_item_validation_error = 61;
                                    break;
                                case "INCONSISTENT_CURRENCY_CODES":
                                case 62:
                                    m.feed_item_validation_error = 62;
                                    break;
                                case "PRICE_EXTENSION_HAS_DUPLICATED_HEADERS":
                                case 63:
                                    m.feed_item_validation_error = 63;
                                    break;
                                case "ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION":
                                case 64:
                                    m.feed_item_validation_error = 64;
                                    break;
                                case "PRICE_EXTENSION_HAS_TOO_FEW_ITEMS":
                                case 65:
                                    m.feed_item_validation_error = 65;
                                    break;
                                case "UNSUPPORTED_VALUE":
                                case 66:
                                    m.feed_item_validation_error = 66;
                                    break;
                                case "INVALID_FINAL_MOBILE_URL":
                                case 67:
                                    m.feed_item_validation_error = 67;
                                    break;
                                case "INVALID_KEYWORDLESS_AD_RULE_LABEL":
                                case 68:
                                    m.feed_item_validation_error = 68;
                                    break;
                                case "VALUE_TRACK_PARAMETER_NOT_SUPPORTED":
                                case 69:
                                    m.feed_item_validation_error = 69;
                                    break;
                                case "UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE":
                                case 70:
                                    m.feed_item_validation_error = 70;
                                    break;
                                case "INVALID_IOS_APP_LINK":
                                case 71:
                                    m.feed_item_validation_error = 71;
                                    break;
                                case "MISSING_IOS_APP_LINK_OR_IOS_APP_STORE_ID":
                                case 72:
                                    m.feed_item_validation_error = 72;
                                    break;
                                case "PROMOTION_INVALID_TIME":
                                case 73:
                                    m.feed_item_validation_error = 73;
                                    break;
                                case "PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF":
                                case 74:
                                    m.feed_item_validation_error = 74;
                                    break;
                                case "PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT":
                                case 75:
                                    m.feed_item_validation_error = 75;
                                    break;
                                case "TOO_MANY_DECIMAL_PLACES_SPECIFIED":
                                case 76:
                                    m.feed_item_validation_error = 76;
                                    break;
                                case "AD_CUSTOMIZERS_NOT_ALLOWED":
                                case 77:
                                    m.feed_item_validation_error = 77;
                                    break;
                                case "INVALID_LANGUAGE_CODE":
                                case 78:
                                    m.feed_item_validation_error = 78;
                                    break;
                                case "UNSUPPORTED_LANGUAGE":
                                case 79:
                                    m.feed_item_validation_error = 79;
                                    break;
                                case "IF_FUNCTION_NOT_ALLOWED":
                                case 80:
                                    m.feed_item_validation_error = 80;
                                    break;
                                case "INVALID_FINAL_URL_SUFFIX":
                                case 81:
                                    m.feed_item_validation_error = 81;
                                    break;
                                case "INVALID_TAG_IN_FINAL_URL_SUFFIX":
                                case 82:
                                    m.feed_item_validation_error = 82;
                                    break;
                                case "INVALID_FINAL_URL_SUFFIX_FORMAT":
                                case 83:
                                    m.feed_item_validation_error = 83;
                                    break;
                                case "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED":
                                case 84:
                                    m.feed_item_validation_error = 84;
                                    break;
                                case "ONLY_ONE_DELIVERY_OPTION_IS_ALLOWED":
                                case 85:
                                    m.feed_item_validation_error = 85;
                                    break;
                                case "NO_DELIVERY_OPTION_IS_SET":
                                case 86:
                                    m.feed_item_validation_error = 86;
                                    break;
                                case "INVALID_CONVERSION_REPORTING_STATE":
                                case 87:
                                    m.feed_item_validation_error = 87;
                                    break;
                                case "IMAGE_SIZE_WRONG":
                                case 88:
                                    m.feed_item_validation_error = 88;
                                    break;
                                case "EMAIL_DELIVERY_NOT_AVAILABLE_IN_COUNTRY":
                                case 89:
                                    m.feed_item_validation_error = 89;
                                    break;
                                case "AUTO_REPLY_NOT_AVAILABLE_IN_COUNTRY":
                                case 90:
                                    m.feed_item_validation_error = 90;
                                    break;
                                case "INVALID_LATITUDE_VALUE":
                                case 91:
                                    m.feed_item_validation_error = 91;
                                    break;
                                case "INVALID_LONGITUDE_VALUE":
                                case 92:
                                    m.feed_item_validation_error = 92;
                                    break;
                                case "TOO_MANY_LABELS":
                                case 93:
                                    m.feed_item_validation_error = 93;
                                    break;
                                case "INVALID_IMAGE_URL":
                                case 94:
                                    m.feed_item_validation_error = 94;
                                    break;
                                case "MISSING_LATITUDE_VALUE":
                                case 95:
                                    m.feed_item_validation_error = 95;
                                    break;
                                case "MISSING_LONGITUDE_VALUE":
                                case 96:
                                    m.feed_item_validation_error = 96;
                                    break;
                                case "ADDRESS_NOT_FOUND":
                                case 97:
                                    m.feed_item_validation_error = 97;
                                    break;
                                case "ADDRESS_NOT_TARGETABLE":
                                case 98:
                                    m.feed_item_validation_error = 98;
                                    break;
                                case "INVALID_ASSET_ID":
                                case 100:
                                    m.feed_item_validation_error = 100;
                                    break;
                                case "INCOMPATIBLE_ASSET_TYPE":
                                case 101:
                                    m.feed_item_validation_error = 101;
                                    break;
                                case "IMAGE_ERROR_UNEXPECTED_SIZE":
                                case 102:
                                    m.feed_item_validation_error = 102;
                                    break;
                                case "IMAGE_ERROR_ASPECT_RATIO_NOT_ALLOWED":
                                case 103:
                                    m.feed_item_validation_error = 103;
                                    break;
                                case "IMAGE_ERROR_FILE_TOO_LARGE":
                                case 104:
                                    m.feed_item_validation_error = 104;
                                    break;
                                case "IMAGE_ERROR_FORMAT_NOT_ALLOWED":
                                case 105:
                                    m.feed_item_validation_error = 105;
                                    break;
                                case "IMAGE_ERROR_CONSTRAINTS_VIOLATED":
                                case 106:
                                    m.feed_item_validation_error = 106;
                                    break;
                                case "IMAGE_ERROR_SERVER_ERROR":
                                case 107:
                                    m.feed_item_validation_error = 107;
                                    break;
                                }
                                switch (d.extension_setting_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.extension_setting_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.extension_setting_error = 1;
                                    break;
                                case "EXTENSIONS_REQUIRED":
                                case 2:
                                    m.extension_setting_error = 2;
                                    break;
                                case "FEED_TYPE_EXTENSION_TYPE_MISMATCH":
                                case 3:
                                    m.extension_setting_error = 3;
                                    break;
                                case "INVALID_FEED_TYPE":
                                case 4:
                                    m.extension_setting_error = 4;
                                    break;
                                case "INVALID_FEED_TYPE_FOR_CUSTOMER_EXTENSION_SETTING":
                                case 5:
                                    m.extension_setting_error = 5;
                                    break;
                                case "CANNOT_CHANGE_FEED_ITEM_ON_CREATE":
                                case 6:
                                    m.extension_setting_error = 6;
                                    break;
                                case "CANNOT_UPDATE_NEWLY_CREATED_EXTENSION":
                                case 7:
                                    m.extension_setting_error = 7;
                                    break;
                                case "NO_EXISTING_AD_GROUP_EXTENSION_SETTING_FOR_TYPE":
                                case 8:
                                    m.extension_setting_error = 8;
                                    break;
                                case "NO_EXISTING_CAMPAIGN_EXTENSION_SETTING_FOR_TYPE":
                                case 9:
                                    m.extension_setting_error = 9;
                                    break;
                                case "NO_EXISTING_CUSTOMER_EXTENSION_SETTING_FOR_TYPE":
                                case 10:
                                    m.extension_setting_error = 10;
                                    break;
                                case "AD_GROUP_EXTENSION_SETTING_ALREADY_EXISTS":
                                case 11:
                                    m.extension_setting_error = 11;
                                    break;
                                case "CAMPAIGN_EXTENSION_SETTING_ALREADY_EXISTS":
                                case 12:
                                    m.extension_setting_error = 12;
                                    break;
                                case "CUSTOMER_EXTENSION_SETTING_ALREADY_EXISTS":
                                case 13:
                                    m.extension_setting_error = 13;
                                    break;
                                case "AD_GROUP_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE":
                                case 14:
                                    m.extension_setting_error = 14;
                                    break;
                                case "CAMPAIGN_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE":
                                case 15:
                                    m.extension_setting_error = 15;
                                    break;
                                case "CUSTOMER_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE":
                                case 16:
                                    m.extension_setting_error = 16;
                                    break;
                                case "VALUE_OUT_OF_RANGE":
                                case 17:
                                    m.extension_setting_error = 17;
                                    break;
                                case "CANNOT_SET_FIELD_WITH_FINAL_URLS":
                                case 18:
                                    m.extension_setting_error = 18;
                                    break;
                                case "FINAL_URLS_NOT_SET":
                                case 19:
                                    m.extension_setting_error = 19;
                                    break;
                                case "INVALID_PHONE_NUMBER":
                                case 20:
                                    m.extension_setting_error = 20;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY":
                                case 21:
                                    m.extension_setting_error = 21;
                                    break;
                                case "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED":
                                case 22:
                                    m.extension_setting_error = 22;
                                    break;
                                case "PREMIUM_RATE_NUMBER_NOT_ALLOWED":
                                case 23:
                                    m.extension_setting_error = 23;
                                    break;
                                case "DISALLOWED_NUMBER_TYPE":
                                case 24:
                                    m.extension_setting_error = 24;
                                    break;
                                case "INVALID_DOMESTIC_PHONE_NUMBER_FORMAT":
                                case 25:
                                    m.extension_setting_error = 25;
                                    break;
                                case "VANITY_PHONE_NUMBER_NOT_ALLOWED":
                                case 26:
                                    m.extension_setting_error = 26;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 27:
                                    m.extension_setting_error = 27;
                                    break;
                                case "INVALID_CALL_CONVERSION_TYPE_ID":
                                case 28:
                                    m.extension_setting_error = 28;
                                    break;
                                case "CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING":
                                case 69:
                                    m.extension_setting_error = 69;
                                    break;
                                case "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY":
                                case 30:
                                    m.extension_setting_error = 30;
                                    break;
                                case "INVALID_APP_ID":
                                case 31:
                                    m.extension_setting_error = 31;
                                    break;
                                case "QUOTES_IN_REVIEW_EXTENSION_SNIPPET":
                                case 32:
                                    m.extension_setting_error = 32;
                                    break;
                                case "HYPHENS_IN_REVIEW_EXTENSION_SNIPPET":
                                case 33:
                                    m.extension_setting_error = 33;
                                    break;
                                case "REVIEW_EXTENSION_SOURCE_NOT_ELIGIBLE":
                                case 34:
                                    m.extension_setting_error = 34;
                                    break;
                                case "SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT":
                                case 35:
                                    m.extension_setting_error = 35;
                                    break;
                                case "MISSING_FIELD":
                                case 36:
                                    m.extension_setting_error = 36;
                                    break;
                                case "INCONSISTENT_CURRENCY_CODES":
                                case 37:
                                    m.extension_setting_error = 37;
                                    break;
                                case "PRICE_EXTENSION_HAS_DUPLICATED_HEADERS":
                                case 38:
                                    m.extension_setting_error = 38;
                                    break;
                                case "PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION":
                                case 39:
                                    m.extension_setting_error = 39;
                                    break;
                                case "PRICE_EXTENSION_HAS_TOO_FEW_ITEMS":
                                case 40:
                                    m.extension_setting_error = 40;
                                    break;
                                case "PRICE_EXTENSION_HAS_TOO_MANY_ITEMS":
                                case 41:
                                    m.extension_setting_error = 41;
                                    break;
                                case "UNSUPPORTED_VALUE":
                                case 42:
                                    m.extension_setting_error = 42;
                                    break;
                                case "INVALID_DEVICE_PREFERENCE":
                                case 43:
                                    m.extension_setting_error = 43;
                                    break;
                                case "INVALID_SCHEDULE_END":
                                case 45:
                                    m.extension_setting_error = 45;
                                    break;
                                case "DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE":
                                case 47:
                                    m.extension_setting_error = 47;
                                    break;
                                case "OVERLAPPING_SCHEDULES_NOT_ALLOWED":
                                case 48:
                                    m.extension_setting_error = 48;
                                    break;
                                case "SCHEDULE_END_NOT_AFTER_START":
                                case 49:
                                    m.extension_setting_error = 49;
                                    break;
                                case "TOO_MANY_SCHEDULES_PER_DAY":
                                case 50:
                                    m.extension_setting_error = 50;
                                    break;
                                case "DUPLICATE_EXTENSION_FEED_ITEM_EDIT":
                                case 51:
                                    m.extension_setting_error = 51;
                                    break;
                                case "INVALID_SNIPPETS_HEADER":
                                case 52:
                                    m.extension_setting_error = 52;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY":
                                case 53:
                                    m.extension_setting_error = 53;
                                    break;
                                case "CAMPAIGN_TARGETING_MISMATCH":
                                case 54:
                                    m.extension_setting_error = 54;
                                    break;
                                case "CANNOT_OPERATE_ON_REMOVED_FEED":
                                case 55:
                                    m.extension_setting_error = 55;
                                    break;
                                case "EXTENSION_TYPE_REQUIRED":
                                case 56:
                                    m.extension_setting_error = 56;
                                    break;
                                case "INCOMPATIBLE_UNDERLYING_MATCHING_FUNCTION":
                                case 57:
                                    m.extension_setting_error = 57;
                                    break;
                                case "START_DATE_AFTER_END_DATE":
                                case 58:
                                    m.extension_setting_error = 58;
                                    break;
                                case "INVALID_PRICE_FORMAT":
                                case 59:
                                    m.extension_setting_error = 59;
                                    break;
                                case "PROMOTION_INVALID_TIME":
                                case 60:
                                    m.extension_setting_error = 60;
                                    break;
                                case "PROMOTION_CANNOT_SET_PERCENT_DISCOUNT_AND_MONEY_DISCOUNT":
                                case 61:
                                    m.extension_setting_error = 61;
                                    break;
                                case "PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT":
                                case 62:
                                    m.extension_setting_error = 62;
                                    break;
                                case "TOO_MANY_DECIMAL_PLACES_SPECIFIED":
                                case 63:
                                    m.extension_setting_error = 63;
                                    break;
                                case "INVALID_LANGUAGE_CODE":
                                case 64:
                                    m.extension_setting_error = 64;
                                    break;
                                case "UNSUPPORTED_LANGUAGE":
                                case 65:
                                    m.extension_setting_error = 65;
                                    break;
                                case "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED":
                                case 66:
                                    m.extension_setting_error = 66;
                                    break;
                                case "EXTENSION_SETTING_UPDATE_IS_A_NOOP":
                                case 67:
                                    m.extension_setting_error = 67;
                                    break;
                                case "DISALLOWED_TEXT":
                                case 68:
                                    m.extension_setting_error = 68;
                                    break;
                                }
                                switch (d.feed_item_set_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_item_set_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_item_set_error = 1;
                                    break;
                                case "FEED_ITEM_SET_REMOVED":
                                case 2:
                                    m.feed_item_set_error = 2;
                                    break;
                                case "CANNOT_CLEAR_DYNAMIC_FILTER":
                                case 3:
                                    m.feed_item_set_error = 3;
                                    break;
                                case "CANNOT_CREATE_DYNAMIC_FILTER":
                                case 4:
                                    m.feed_item_set_error = 4;
                                    break;
                                case "INVALID_FEED_TYPE":
                                case 5:
                                    m.feed_item_set_error = 5;
                                    break;
                                case "DUPLICATE_NAME":
                                case 6:
                                    m.feed_item_set_error = 6;
                                    break;
                                case "WRONG_DYNAMIC_FILTER_FOR_FEED_TYPE":
                                case 7:
                                    m.feed_item_set_error = 7;
                                    break;
                                case "DYNAMIC_FILTER_INVALID_CHAIN_IDS":
                                case 8:
                                    m.feed_item_set_error = 8;
                                    break;
                                }
                                switch (d.feed_item_set_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_item_set_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_item_set_link_error = 1;
                                    break;
                                case "FEED_ID_MISMATCH":
                                case 2:
                                    m.feed_item_set_link_error = 2;
                                    break;
                                case "NO_MUTATE_ALLOWED_FOR_DYNAMIC_SET":
                                case 3:
                                    m.feed_item_set_link_error = 3;
                                    break;
                                }
                                switch (d.feed_item_target_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_item_target_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_item_target_error = 1;
                                    break;
                                case "MUST_SET_TARGET_ONEOF_ON_CREATE":
                                case 2:
                                    m.feed_item_target_error = 2;
                                    break;
                                case "FEED_ITEM_TARGET_ALREADY_EXISTS":
                                case 3:
                                    m.feed_item_target_error = 3;
                                    break;
                                case "FEED_ITEM_SCHEDULES_CANNOT_OVERLAP":
                                case 4:
                                    m.feed_item_target_error = 4;
                                    break;
                                case "TARGET_LIMIT_EXCEEDED_FOR_GIVEN_TYPE":
                                case 5:
                                    m.feed_item_target_error = 5;
                                    break;
                                case "TOO_MANY_SCHEDULES_PER_DAY":
                                case 6:
                                    m.feed_item_target_error = 6;
                                    break;
                                case "CANNOT_HAVE_ENABLED_CAMPAIGN_AND_ENABLED_AD_GROUP_TARGETS":
                                case 7:
                                    m.feed_item_target_error = 7;
                                    break;
                                case "DUPLICATE_AD_SCHEDULE":
                                case 8:
                                    m.feed_item_target_error = 8;
                                    break;
                                case "DUPLICATE_KEYWORD":
                                case 9:
                                    m.feed_item_target_error = 9;
                                    break;
                                }
                                switch (d.policy_violation_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.policy_violation_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.policy_violation_error = 1;
                                    break;
                                case "POLICY_ERROR":
                                case 2:
                                    m.policy_violation_error = 2;
                                    break;
                                }
                                switch (d.partial_failure_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.partial_failure_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.partial_failure_error = 1;
                                    break;
                                case "PARTIAL_FAILURE_MODE_REQUIRED":
                                case 2:
                                    m.partial_failure_error = 2;
                                    break;
                                }
                                switch (d.policy_validation_parameter_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.policy_validation_parameter_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.policy_validation_parameter_error = 1;
                                    break;
                                case "UNSUPPORTED_AD_TYPE_FOR_IGNORABLE_POLICY_TOPICS":
                                case 2:
                                    m.policy_validation_parameter_error = 2;
                                    break;
                                case "UNSUPPORTED_AD_TYPE_FOR_EXEMPT_POLICY_VIOLATION_KEYS":
                                case 3:
                                    m.policy_validation_parameter_error = 3;
                                    break;
                                case "CANNOT_SET_BOTH_IGNORABLE_POLICY_TOPICS_AND_EXEMPT_POLICY_VIOLATION_KEYS":
                                case 4:
                                    m.policy_validation_parameter_error = 4;
                                    break;
                                }
                                switch (d.size_limit_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.size_limit_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.size_limit_error = 1;
                                    break;
                                case "REQUEST_SIZE_LIMIT_EXCEEDED":
                                case 2:
                                    m.size_limit_error = 2;
                                    break;
                                case "RESPONSE_SIZE_LIMIT_EXCEEDED":
                                case 3:
                                    m.size_limit_error = 3;
                                    break;
                                }
                                switch (d.offline_user_data_job_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.offline_user_data_job_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.offline_user_data_job_error = 1;
                                    break;
                                case "INVALID_USER_LIST_ID":
                                case 3:
                                    m.offline_user_data_job_error = 3;
                                    break;
                                case "INVALID_USER_LIST_TYPE":
                                case 4:
                                    m.offline_user_data_job_error = 4;
                                    break;
                                case "NOT_ON_ALLOWLIST_FOR_USER_ID":
                                case 33:
                                    m.offline_user_data_job_error = 33;
                                    break;
                                case "INCOMPATIBLE_UPLOAD_KEY_TYPE":
                                case 6:
                                    m.offline_user_data_job_error = 6;
                                    break;
                                case "MISSING_USER_IDENTIFIER":
                                case 7:
                                    m.offline_user_data_job_error = 7;
                                    break;
                                case "INVALID_MOBILE_ID_FORMAT":
                                case 8:
                                    m.offline_user_data_job_error = 8;
                                    break;
                                case "TOO_MANY_USER_IDENTIFIERS":
                                case 9:
                                    m.offline_user_data_job_error = 9;
                                    break;
                                case "NOT_ON_ALLOWLIST_FOR_STORE_SALES_DIRECT":
                                case 31:
                                    m.offline_user_data_job_error = 31;
                                    break;
                                case "NOT_ON_ALLOWLIST_FOR_UNIFIED_STORE_SALES":
                                case 32:
                                    m.offline_user_data_job_error = 32;
                                    break;
                                case "INVALID_PARTNER_ID":
                                case 11:
                                    m.offline_user_data_job_error = 11;
                                    break;
                                case "INVALID_ENCODING":
                                case 12:
                                    m.offline_user_data_job_error = 12;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 13:
                                    m.offline_user_data_job_error = 13;
                                    break;
                                case "INCOMPATIBLE_USER_IDENTIFIER":
                                case 14:
                                    m.offline_user_data_job_error = 14;
                                    break;
                                case "FUTURE_TRANSACTION_TIME":
                                case 15:
                                    m.offline_user_data_job_error = 15;
                                    break;
                                case "INVALID_CONVERSION_ACTION":
                                case 16:
                                    m.offline_user_data_job_error = 16;
                                    break;
                                case "MOBILE_ID_NOT_SUPPORTED":
                                case 17:
                                    m.offline_user_data_job_error = 17;
                                    break;
                                case "INVALID_OPERATION_ORDER":
                                case 18:
                                    m.offline_user_data_job_error = 18;
                                    break;
                                case "CONFLICTING_OPERATION":
                                case 19:
                                    m.offline_user_data_job_error = 19;
                                    break;
                                case "EXTERNAL_UPDATE_ID_ALREADY_EXISTS":
                                case 21:
                                    m.offline_user_data_job_error = 21;
                                    break;
                                case "JOB_ALREADY_STARTED":
                                case 22:
                                    m.offline_user_data_job_error = 22;
                                    break;
                                case "REMOVE_NOT_SUPPORTED":
                                case 23:
                                    m.offline_user_data_job_error = 23;
                                    break;
                                case "REMOVE_ALL_NOT_SUPPORTED":
                                case 24:
                                    m.offline_user_data_job_error = 24;
                                    break;
                                case "INVALID_SHA256_FORMAT":
                                case 25:
                                    m.offline_user_data_job_error = 25;
                                    break;
                                case "CUSTOM_KEY_DISABLED":
                                case 26:
                                    m.offline_user_data_job_error = 26;
                                    break;
                                case "CUSTOM_KEY_NOT_PREDEFINED":
                                case 27:
                                    m.offline_user_data_job_error = 27;
                                    break;
                                case "CUSTOM_KEY_NOT_SET":
                                case 29:
                                    m.offline_user_data_job_error = 29;
                                    break;
                                case "CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS":
                                case 30:
                                    m.offline_user_data_job_error = 30;
                                    break;
                                case "ATTRIBUTES_NOT_APPLICABLE_FOR_CUSTOMER_MATCH_USER_LIST":
                                case 34:
                                    m.offline_user_data_job_error = 34;
                                    break;
                                case "LIFETIME_VALUE_BUCKET_NOT_IN_RANGE":
                                case 35:
                                    m.offline_user_data_job_error = 35;
                                    break;
                                case "INCOMPATIBLE_USER_IDENTIFIER_FOR_ATTRIBUTES":
                                case 36:
                                    m.offline_user_data_job_error = 36;
                                    break;
                                case "FUTURE_TIME_NOT_ALLOWED":
                                case 37:
                                    m.offline_user_data_job_error = 37;
                                    break;
                                case "LAST_PURCHASE_TIME_LESS_THAN_ACQUISITION_TIME":
                                case 38:
                                    m.offline_user_data_job_error = 38;
                                    break;
                                case "CUSTOMER_IDENTIFIER_NOT_ALLOWED":
                                case 39:
                                    m.offline_user_data_job_error = 39;
                                    break;
                                case "INVALID_ITEM_ID":
                                case 40:
                                    m.offline_user_data_job_error = 40;
                                    break;
                                }
                                switch (d.not_allowlisted_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.not_allowlisted_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.not_allowlisted_error = 1;
                                    break;
                                case "CUSTOMER_NOT_ALLOWLISTED_FOR_THIS_FEATURE":
                                case 2:
                                    m.not_allowlisted_error = 2;
                                    break;
                                }
                                switch (d.manager_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.manager_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.manager_link_error = 1;
                                    break;
                                case "ACCOUNTS_NOT_COMPATIBLE_FOR_LINKING":
                                case 2:
                                    m.manager_link_error = 2;
                                    break;
                                case "TOO_MANY_MANAGERS":
                                case 3:
                                    m.manager_link_error = 3;
                                    break;
                                case "TOO_MANY_INVITES":
                                case 4:
                                    m.manager_link_error = 4;
                                    break;
                                case "ALREADY_INVITED_BY_THIS_MANAGER":
                                case 5:
                                    m.manager_link_error = 5;
                                    break;
                                case "ALREADY_MANAGED_BY_THIS_MANAGER":
                                case 6:
                                    m.manager_link_error = 6;
                                    break;
                                case "ALREADY_MANAGED_IN_HIERARCHY":
                                case 7:
                                    m.manager_link_error = 7;
                                    break;
                                case "DUPLICATE_CHILD_FOUND":
                                case 8:
                                    m.manager_link_error = 8;
                                    break;
                                case "CLIENT_HAS_NO_ADMIN_USER":
                                case 9:
                                    m.manager_link_error = 9;
                                    break;
                                case "MAX_DEPTH_EXCEEDED":
                                case 10:
                                    m.manager_link_error = 10;
                                    break;
                                case "CYCLE_NOT_ALLOWED":
                                case 11:
                                    m.manager_link_error = 11;
                                    break;
                                case "TOO_MANY_ACCOUNTS":
                                case 12:
                                    m.manager_link_error = 12;
                                    break;
                                case "TOO_MANY_ACCOUNTS_AT_MANAGER":
                                case 13:
                                    m.manager_link_error = 13;
                                    break;
                                case "NON_OWNER_USER_CANNOT_MODIFY_LINK":
                                case 14:
                                    m.manager_link_error = 14;
                                    break;
                                case "SUSPENDED_ACCOUNT_CANNOT_ADD_CLIENTS":
                                case 15:
                                    m.manager_link_error = 15;
                                    break;
                                case "CLIENT_OUTSIDE_TREE":
                                case 16:
                                    m.manager_link_error = 16;
                                    break;
                                case "INVALID_STATUS_CHANGE":
                                case 17:
                                    m.manager_link_error = 17;
                                    break;
                                case "INVALID_CHANGE":
                                case 18:
                                    m.manager_link_error = 18;
                                    break;
                                case "CUSTOMER_CANNOT_MANAGE_SELF":
                                case 19:
                                    m.manager_link_error = 19;
                                    break;
                                case "CREATING_ENABLED_LINK_NOT_ALLOWED":
                                case 20:
                                    m.manager_link_error = 20;
                                    break;
                                }
                                switch (d.currency_code_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.currency_code_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.currency_code_error = 1;
                                    break;
                                case "UNSUPPORTED":
                                case 2:
                                    m.currency_code_error = 2;
                                    break;
                                }
                                switch (d.experiment_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.experiment_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.experiment_error = 1;
                                    break;
                                case "CANNOT_SET_START_DATE_IN_PAST":
                                case 2:
                                    m.experiment_error = 2;
                                    break;
                                case "END_DATE_BEFORE_START_DATE":
                                case 3:
                                    m.experiment_error = 3;
                                    break;
                                case "START_DATE_TOO_FAR_IN_FUTURE":
                                case 4:
                                    m.experiment_error = 4;
                                    break;
                                case "DUPLICATE_EXPERIMENT_NAME":
                                case 5:
                                    m.experiment_error = 5;
                                    break;
                                case "CANNOT_MODIFY_REMOVED_EXPERIMENT":
                                case 6:
                                    m.experiment_error = 6;
                                    break;
                                case "START_DATE_ALREADY_PASSED":
                                case 7:
                                    m.experiment_error = 7;
                                    break;
                                case "CANNOT_SET_END_DATE_IN_PAST":
                                case 8:
                                    m.experiment_error = 8;
                                    break;
                                case "CANNOT_SET_STATUS_TO_REMOVED":
                                case 9:
                                    m.experiment_error = 9;
                                    break;
                                case "CANNOT_MODIFY_PAST_END_DATE":
                                case 10:
                                    m.experiment_error = 10;
                                    break;
                                case "INVALID_STATUS":
                                case 11:
                                    m.experiment_error = 11;
                                    break;
                                case "INVALID_CAMPAIGN_CHANNEL_TYPE":
                                case 12:
                                    m.experiment_error = 12;
                                    break;
                                case "OVERLAPPING_MEMBERS_AND_DATE_RANGE":
                                case 13:
                                    m.experiment_error = 13;
                                    break;
                                case "INVALID_TRIAL_ARM_TRAFFIC_SPLIT":
                                case 14:
                                    m.experiment_error = 14;
                                    break;
                                case "TRAFFIC_SPLIT_OVERLAPPING":
                                case 15:
                                    m.experiment_error = 15;
                                    break;
                                case "SUM_TRIAL_ARM_TRAFFIC_UNEQUALS_TO_TRIAL_TRAFFIC_SPLIT_DENOMINATOR":
                                case 16:
                                    m.experiment_error = 16;
                                    break;
                                case "CANNOT_MODIFY_TRAFFIC_SPLIT_AFTER_START":
                                case 17:
                                    m.experiment_error = 17;
                                    break;
                                case "EXPERIMENT_NOT_FOUND":
                                case 18:
                                    m.experiment_error = 18;
                                    break;
                                case "EXPERIMENT_NOT_YET_STARTED":
                                case 19:
                                    m.experiment_error = 19;
                                    break;
                                case "CANNOT_HAVE_MULTIPLE_CONTROL_ARMS":
                                case 20:
                                    m.experiment_error = 20;
                                    break;
                                case "IN_DESIGN_CAMPAIGNS_NOT_SET":
                                case 21:
                                    m.experiment_error = 21;
                                    break;
                                case "CANNOT_SET_STATUS_TO_GRADUATED":
                                case 22:
                                    m.experiment_error = 22;
                                    break;
                                case "CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_SHARED_BUDGET":
                                case 23:
                                    m.experiment_error = 23;
                                    break;
                                case "CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_CUSTOM_BUDGET":
                                case 24:
                                    m.experiment_error = 24;
                                    break;
                                case "STATUS_TRANSITION_INVALID":
                                case 25:
                                    m.experiment_error = 25;
                                    break;
                                }
                                switch (d.access_invitation_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.access_invitation_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.access_invitation_error = 1;
                                    break;
                                case "INVALID_EMAIL_ADDRESS":
                                case 2:
                                    m.access_invitation_error = 2;
                                    break;
                                case "EMAIL_ADDRESS_ALREADY_HAS_ACCESS":
                                case 3:
                                    m.access_invitation_error = 3;
                                    break;
                                case "INVALID_INVITATION_STATUS":
                                case 4:
                                    m.access_invitation_error = 4;
                                    break;
                                case "GOOGLE_CONSUMER_ACCOUNT_NOT_ALLOWED":
                                case 5:
                                    m.access_invitation_error = 5;
                                    break;
                                case "INVALID_INVITATION_ID":
                                case 6:
                                    m.access_invitation_error = 6;
                                    break;
                                case "EMAIL_ADDRESS_ALREADY_HAS_PENDING_INVITATION":
                                case 7:
                                    m.access_invitation_error = 7;
                                    break;
                                case "PENDING_INVITATIONS_LIMIT_EXCEEDED":
                                case 8:
                                    m.access_invitation_error = 8;
                                    break;
                                case "EMAIL_DOMAIN_POLICY_VIOLATED":
                                case 9:
                                    m.access_invitation_error = 9;
                                    break;
                                }
                                switch (d.reach_plan_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.reach_plan_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.reach_plan_error = 1;
                                    break;
                                case "NOT_FORECASTABLE_MISSING_RATE":
                                case 2:
                                    m.reach_plan_error = 2;
                                    break;
                                }
                                switch (d.invoice_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.invoice_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.invoice_error = 1;
                                    break;
                                case "YEAR_MONTH_TOO_OLD":
                                case 2:
                                    m.invoice_error = 2;
                                    break;
                                case "NOT_INVOICED_CUSTOMER":
                                case 3:
                                    m.invoice_error = 3;
                                    break;
                                case "BILLING_SETUP_NOT_APPROVED":
                                case 4:
                                    m.invoice_error = 4;
                                    break;
                                case "BILLING_SETUP_NOT_ON_MONTHLY_INVOICING":
                                case 5:
                                    m.invoice_error = 5;
                                    break;
                                case "NON_SERVING_CUSTOMER":
                                case 6:
                                    m.invoice_error = 6;
                                    break;
                                }
                                switch (d.payments_account_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.payments_account_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.payments_account_error = 1;
                                    break;
                                case "NOT_SUPPORTED_FOR_MANAGER_CUSTOMER":
                                case 2:
                                    m.payments_account_error = 2;
                                    break;
                                }
                                switch (d.time_zone_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.time_zone_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.time_zone_error = 1;
                                    break;
                                case "INVALID_TIME_ZONE":
                                case 5:
                                    m.time_zone_error = 5;
                                    break;
                                }
                                switch (d.asset_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.asset_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.asset_link_error = 1;
                                    break;
                                case "PINNING_UNSUPPORTED":
                                case 2:
                                    m.asset_link_error = 2;
                                    break;
                                case "UNSUPPORTED_FIELD_TYPE":
                                case 3:
                                    m.asset_link_error = 3;
                                    break;
                                case "FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE":
                                case 4:
                                    m.asset_link_error = 4;
                                    break;
                                case "FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE":
                                case 5:
                                    m.asset_link_error = 5;
                                    break;
                                case "INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE":
                                case 6:
                                    m.asset_link_error = 6;
                                    break;
                                case "IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE":
                                case 7:
                                    m.asset_link_error = 7;
                                    break;
                                case "INVALID_PINNED_FIELD":
                                case 8:
                                    m.asset_link_error = 8;
                                    break;
                                case "MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE":
                                case 9:
                                    m.asset_link_error = 9;
                                    break;
                                case "NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION":
                                case 10:
                                    m.asset_link_error = 10;
                                    break;
                                case "NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK":
                                case 11:
                                    m.asset_link_error = 11;
                                    break;
                                case "NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION":
                                case 12:
                                    m.asset_link_error = 12;
                                    break;
                                case "YOUTUBE_VIDEO_REMOVED":
                                case 13:
                                    m.asset_link_error = 13;
                                    break;
                                case "YOUTUBE_VIDEO_TOO_LONG":
                                case 14:
                                    m.asset_link_error = 14;
                                    break;
                                case "YOUTUBE_VIDEO_TOO_SHORT":
                                case 15:
                                    m.asset_link_error = 15;
                                    break;
                                case "INVALID_STATUS":
                                case 17:
                                    m.asset_link_error = 17;
                                    break;
                                case "YOUTUBE_VIDEO_DURATION_NOT_DEFINED":
                                case 18:
                                    m.asset_link_error = 18;
                                    break;
                                case "CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS":
                                case 19:
                                    m.asset_link_error = 19;
                                    break;
                                case "CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET":
                                case 20:
                                    m.asset_link_error = 20;
                                    break;
                                case "CANNOT_MODIFY_ASSET_LINK_SOURCE":
                                case 21:
                                    m.asset_link_error = 21;
                                    break;
                                }
                                switch (d.user_data_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.user_data_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.user_data_error = 1;
                                    break;
                                case "OPERATIONS_FOR_CUSTOMER_MATCH_NOT_ALLOWED":
                                case 2:
                                    m.user_data_error = 2;
                                    break;
                                case "TOO_MANY_USER_IDENTIFIERS":
                                case 3:
                                    m.user_data_error = 3;
                                    break;
                                case "USER_LIST_NOT_APPLICABLE":
                                case 4:
                                    m.user_data_error = 4;
                                    break;
                                }
                                switch (d.batch_job_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.batch_job_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.batch_job_error = 1;
                                    break;
                                case "CANNOT_MODIFY_JOB_AFTER_JOB_STARTS_RUNNING":
                                case 2:
                                    m.batch_job_error = 2;
                                    break;
                                case "EMPTY_OPERATIONS":
                                case 3:
                                    m.batch_job_error = 3;
                                    break;
                                case "INVALID_SEQUENCE_TOKEN":
                                case 4:
                                    m.batch_job_error = 4;
                                    break;
                                case "RESULTS_NOT_READY":
                                case 5:
                                    m.batch_job_error = 5;
                                    break;
                                case "INVALID_PAGE_SIZE":
                                case 6:
                                    m.batch_job_error = 6;
                                    break;
                                case "CAN_ONLY_REMOVE_PENDING_JOB":
                                case 7:
                                    m.batch_job_error = 7;
                                    break;
                                }
                                switch (d.account_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.account_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.account_link_error = 1;
                                    break;
                                case "INVALID_STATUS":
                                case 2:
                                    m.account_link_error = 2;
                                    break;
                                }
                                switch (d.third_party_app_analytics_link_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.third_party_app_analytics_link_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.third_party_app_analytics_link_error = 1;
                                    break;
                                case "INVALID_ANALYTICS_PROVIDER_ID":
                                case 2:
                                    m.third_party_app_analytics_link_error = 2;
                                    break;
                                case "INVALID_MOBILE_APP_ID":
                                case 3:
                                    m.third_party_app_analytics_link_error = 3;
                                    break;
                                case "MOBILE_APP_IS_NOT_ENABLED":
                                case 4:
                                    m.third_party_app_analytics_link_error = 4;
                                    break;
                                case "CANNOT_REGENERATE_SHAREABLE_LINK_ID_FOR_REMOVED_LINK":
                                case 5:
                                    m.third_party_app_analytics_link_error = 5;
                                    break;
                                }
                                switch (d.customer_user_access_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.customer_user_access_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.customer_user_access_error = 1;
                                    break;
                                case "INVALID_USER_ID":
                                case 2:
                                    m.customer_user_access_error = 2;
                                    break;
                                case "REMOVAL_DISALLOWED":
                                case 3:
                                    m.customer_user_access_error = 3;
                                    break;
                                case "DISALLOWED_ACCESS_ROLE":
                                case 4:
                                    m.customer_user_access_error = 4;
                                    break;
                                case "LAST_ADMIN_USER_OF_SERVING_CUSTOMER":
                                case 5:
                                    m.customer_user_access_error = 5;
                                    break;
                                case "LAST_ADMIN_USER_OF_MANAGER":
                                case 6:
                                    m.customer_user_access_error = 6;
                                    break;
                                }
                                switch (d.custom_audience_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.custom_audience_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.custom_audience_error = 1;
                                    break;
                                case "NAME_ALREADY_USED":
                                case 2:
                                    m.custom_audience_error = 2;
                                    break;
                                case "CANNOT_REMOVE_WHILE_IN_USE":
                                case 3:
                                    m.custom_audience_error = 3;
                                    break;
                                case "RESOURCE_ALREADY_REMOVED":
                                case 4:
                                    m.custom_audience_error = 4;
                                    break;
                                case "MEMBER_TYPE_AND_PARAMETER_ALREADY_EXISTED":
                                case 5:
                                    m.custom_audience_error = 5;
                                    break;
                                case "INVALID_MEMBER_TYPE":
                                case 6:
                                    m.custom_audience_error = 6;
                                    break;
                                case "MEMBER_TYPE_AND_VALUE_DOES_NOT_MATCH":
                                case 7:
                                    m.custom_audience_error = 7;
                                    break;
                                case "POLICY_VIOLATION":
                                case 8:
                                    m.custom_audience_error = 8;
                                    break;
                                case "INVALID_TYPE_CHANGE":
                                case 9:
                                    m.custom_audience_error = 9;
                                    break;
                                }
                                switch (d.audience_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.audience_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.audience_error = 1;
                                    break;
                                case "NAME_ALREADY_IN_USE":
                                case 2:
                                    m.audience_error = 2;
                                    break;
                                case "DIMENSION_INVALID":
                                case 3:
                                    m.audience_error = 3;
                                    break;
                                case "AUDIENCE_SEGMENT_NOT_FOUND":
                                case 4:
                                    m.audience_error = 4;
                                    break;
                                case "AUDIENCE_SEGMENT_TYPE_NOT_SUPPORTED":
                                case 5:
                                    m.audience_error = 5;
                                    break;
                                case "DUPLICATE_AUDIENCE_SEGMENT":
                                case 6:
                                    m.audience_error = 6;
                                    break;
                                case "TOO_MANY_SEGMENTS":
                                case 7:
                                    m.audience_error = 7;
                                    break;
                                case "TOO_MANY_DIMENSIONS_OF_SAME_TYPE":
                                case 8:
                                    m.audience_error = 8;
                                    break;
                                case "IN_USE":
                                case 9:
                                    m.audience_error = 9;
                                    break;
                                }
                                switch (d.experiment_arm_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.experiment_arm_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.experiment_arm_error = 1;
                                    break;
                                case "EXPERIMENT_ARM_COUNT_LIMIT_EXCEEDED":
                                case 2:
                                    m.experiment_arm_error = 2;
                                    break;
                                case "INVALID_CAMPAIGN_STATUS":
                                case 3:
                                    m.experiment_arm_error = 3;
                                    break;
                                case "DUPLICATE_EXPERIMENT_ARM_NAME":
                                case 4:
                                    m.experiment_arm_error = 4;
                                    break;
                                case "CANNOT_SET_TREATMENT_ARM_CAMPAIGN":
                                case 5:
                                    m.experiment_arm_error = 5;
                                    break;
                                case "CANNOT_MODIFY_CAMPAIGN_IDS":
                                case 6:
                                    m.experiment_arm_error = 6;
                                    break;
                                case "CANNOT_MODIFY_CAMPAIGN_WITHOUT_SUFFIX_SET":
                                case 7:
                                    m.experiment_arm_error = 7;
                                    break;
                                case "CANNOT_MUTATE_TRAFFIC_SPLIT_AFTER_START":
                                case 8:
                                    m.experiment_arm_error = 8;
                                    break;
                                case "CANNOT_ADD_CAMPAIGN_WITH_SHARED_BUDGET":
                                case 9:
                                    m.experiment_arm_error = 9;
                                    break;
                                case "CANNOT_ADD_CAMPAIGN_WITH_CUSTOM_BUDGET":
                                case 10:
                                    m.experiment_arm_error = 10;
                                    break;
                                case "CANNOT_ADD_CAMPAIGNS_WITH_DYNAMIC_ASSETS_ENABLED":
                                case 11:
                                    m.experiment_arm_error = 11;
                                    break;
                                case "UNSUPPORTED_CAMPAIGN_ADVERTISING_CHANNEL_SUB_TYPE":
                                case 12:
                                    m.experiment_arm_error = 12;
                                    break;
                                case "CANNOT_ADD_BASE_CAMPAIGN_WITH_DATE_RANGE":
                                case 13:
                                    m.experiment_arm_error = 13;
                                    break;
                                case "BIDDING_STRATEGY_NOT_SUPPORTED_IN_EXPERIMENTS":
                                case 14:
                                    m.experiment_arm_error = 14;
                                    break;
                                case "TRAFFIC_SPLIT_NOT_SUPPORTED_FOR_CHANNEL_TYPE":
                                case 15:
                                    m.experiment_arm_error = 15;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ErrorCode message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @static
                             * @param {google.ads.googleads.v11.errors.ErrorCode} m ErrorCode
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ErrorCode.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.request_error != null && m.hasOwnProperty("request_error")) {
                                    d.request_error = o.enums === String ? $root.google.ads.googleads.v11.errors.RequestErrorEnum.RequestError[m.request_error] : m.request_error;
                                    if (o.oneofs)
                                        d.error_code = "request_error";
                                }
                                if (m.bidding_strategy_error != null && m.hasOwnProperty("bidding_strategy_error")) {
                                    d.bidding_strategy_error = o.enums === String ? $root.google.ads.googleads.v11.errors.BiddingStrategyErrorEnum.BiddingStrategyError[m.bidding_strategy_error] : m.bidding_strategy_error;
                                    if (o.oneofs)
                                        d.error_code = "bidding_strategy_error";
                                }
                                if (m.url_field_error != null && m.hasOwnProperty("url_field_error")) {
                                    d.url_field_error = o.enums === String ? $root.google.ads.googleads.v11.errors.UrlFieldErrorEnum.UrlFieldError[m.url_field_error] : m.url_field_error;
                                    if (o.oneofs)
                                        d.error_code = "url_field_error";
                                }
                                if (m.list_operation_error != null && m.hasOwnProperty("list_operation_error")) {
                                    d.list_operation_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ListOperationErrorEnum.ListOperationError[m.list_operation_error] : m.list_operation_error;
                                    if (o.oneofs)
                                        d.error_code = "list_operation_error";
                                }
                                if (m.query_error != null && m.hasOwnProperty("query_error")) {
                                    d.query_error = o.enums === String ? $root.google.ads.googleads.v11.errors.QueryErrorEnum.QueryError[m.query_error] : m.query_error;
                                    if (o.oneofs)
                                        d.error_code = "query_error";
                                }
                                if (m.mutate_error != null && m.hasOwnProperty("mutate_error")) {
                                    d.mutate_error = o.enums === String ? $root.google.ads.googleads.v11.errors.MutateErrorEnum.MutateError[m.mutate_error] : m.mutate_error;
                                    if (o.oneofs)
                                        d.error_code = "mutate_error";
                                }
                                if (m.field_mask_error != null && m.hasOwnProperty("field_mask_error")) {
                                    d.field_mask_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FieldMaskErrorEnum.FieldMaskError[m.field_mask_error] : m.field_mask_error;
                                    if (o.oneofs)
                                        d.error_code = "field_mask_error";
                                }
                                if (m.authorization_error != null && m.hasOwnProperty("authorization_error")) {
                                    d.authorization_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AuthorizationErrorEnum.AuthorizationError[m.authorization_error] : m.authorization_error;
                                    if (o.oneofs)
                                        d.error_code = "authorization_error";
                                }
                                if (m.internal_error != null && m.hasOwnProperty("internal_error")) {
                                    d.internal_error = o.enums === String ? $root.google.ads.googleads.v11.errors.InternalErrorEnum.InternalError[m.internal_error] : m.internal_error;
                                    if (o.oneofs)
                                        d.error_code = "internal_error";
                                }
                                if (m.quota_error != null && m.hasOwnProperty("quota_error")) {
                                    d.quota_error = o.enums === String ? $root.google.ads.googleads.v11.errors.QuotaErrorEnum.QuotaError[m.quota_error] : m.quota_error;
                                    if (o.oneofs)
                                        d.error_code = "quota_error";
                                }
                                if (m.ad_error != null && m.hasOwnProperty("ad_error")) {
                                    d.ad_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdErrorEnum.AdError[m.ad_error] : m.ad_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_error";
                                }
                                if (m.ad_group_error != null && m.hasOwnProperty("ad_group_error")) {
                                    d.ad_group_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdGroupErrorEnum.AdGroupError[m.ad_group_error] : m.ad_group_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_group_error";
                                }
                                if (m.campaign_budget_error != null && m.hasOwnProperty("campaign_budget_error")) {
                                    d.campaign_budget_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignBudgetErrorEnum.CampaignBudgetError[m.campaign_budget_error] : m.campaign_budget_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_budget_error";
                                }
                                if (m.campaign_error != null && m.hasOwnProperty("campaign_error")) {
                                    d.campaign_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignErrorEnum.CampaignError[m.campaign_error] : m.campaign_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_error";
                                }
                                if (m.authentication_error != null && m.hasOwnProperty("authentication_error")) {
                                    d.authentication_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AuthenticationErrorEnum.AuthenticationError[m.authentication_error] : m.authentication_error;
                                    if (o.oneofs)
                                        d.error_code = "authentication_error";
                                }
                                if (m.ad_group_criterion_error != null && m.hasOwnProperty("ad_group_criterion_error")) {
                                    d.ad_group_criterion_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdGroupCriterionErrorEnum.AdGroupCriterionError[m.ad_group_criterion_error] : m.ad_group_criterion_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_group_criterion_error";
                                }
                                if (m.ad_customizer_error != null && m.hasOwnProperty("ad_customizer_error")) {
                                    d.ad_customizer_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdCustomizerErrorEnum.AdCustomizerError[m.ad_customizer_error] : m.ad_customizer_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_customizer_error";
                                }
                                if (m.ad_group_ad_error != null && m.hasOwnProperty("ad_group_ad_error")) {
                                    d.ad_group_ad_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdGroupAdErrorEnum.AdGroupAdError[m.ad_group_ad_error] : m.ad_group_ad_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_group_ad_error";
                                }
                                if (m.ad_sharing_error != null && m.hasOwnProperty("ad_sharing_error")) {
                                    d.ad_sharing_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdSharingErrorEnum.AdSharingError[m.ad_sharing_error] : m.ad_sharing_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_sharing_error";
                                }
                                if (m.adx_error != null && m.hasOwnProperty("adx_error")) {
                                    d.adx_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdxErrorEnum.AdxError[m.adx_error] : m.adx_error;
                                    if (o.oneofs)
                                        d.error_code = "adx_error";
                                }
                                if (m.bidding_error != null && m.hasOwnProperty("bidding_error")) {
                                    d.bidding_error = o.enums === String ? $root.google.ads.googleads.v11.errors.BiddingErrorEnum.BiddingError[m.bidding_error] : m.bidding_error;
                                    if (o.oneofs)
                                        d.error_code = "bidding_error";
                                }
                                if (m.campaign_criterion_error != null && m.hasOwnProperty("campaign_criterion_error")) {
                                    d.campaign_criterion_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignCriterionErrorEnum.CampaignCriterionError[m.campaign_criterion_error] : m.campaign_criterion_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_criterion_error";
                                }
                                if (m.collection_size_error != null && m.hasOwnProperty("collection_size_error")) {
                                    d.collection_size_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CollectionSizeErrorEnum.CollectionSizeError[m.collection_size_error] : m.collection_size_error;
                                    if (o.oneofs)
                                        d.error_code = "collection_size_error";
                                }
                                if (m.criterion_error != null && m.hasOwnProperty("criterion_error")) {
                                    d.criterion_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CriterionErrorEnum.CriterionError[m.criterion_error] : m.criterion_error;
                                    if (o.oneofs)
                                        d.error_code = "criterion_error";
                                }
                                if (m.date_error != null && m.hasOwnProperty("date_error")) {
                                    d.date_error = o.enums === String ? $root.google.ads.googleads.v11.errors.DateErrorEnum.DateError[m.date_error] : m.date_error;
                                    if (o.oneofs)
                                        d.error_code = "date_error";
                                }
                                if (m.date_range_error != null && m.hasOwnProperty("date_range_error")) {
                                    d.date_range_error = o.enums === String ? $root.google.ads.googleads.v11.errors.DateRangeErrorEnum.DateRangeError[m.date_range_error] : m.date_range_error;
                                    if (o.oneofs)
                                        d.error_code = "date_range_error";
                                }
                                if (m.distinct_error != null && m.hasOwnProperty("distinct_error")) {
                                    d.distinct_error = o.enums === String ? $root.google.ads.googleads.v11.errors.DistinctErrorEnum.DistinctError[m.distinct_error] : m.distinct_error;
                                    if (o.oneofs)
                                        d.error_code = "distinct_error";
                                }
                                if (m.feed_attribute_reference_error != null && m.hasOwnProperty("feed_attribute_reference_error")) {
                                    d.feed_attribute_reference_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError[m.feed_attribute_reference_error] : m.feed_attribute_reference_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_attribute_reference_error";
                                }
                                if (m.function_error != null && m.hasOwnProperty("function_error")) {
                                    d.function_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FunctionErrorEnum.FunctionError[m.function_error] : m.function_error;
                                    if (o.oneofs)
                                        d.error_code = "function_error";
                                }
                                if (m.function_parsing_error != null && m.hasOwnProperty("function_parsing_error")) {
                                    d.function_parsing_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FunctionParsingErrorEnum.FunctionParsingError[m.function_parsing_error] : m.function_parsing_error;
                                    if (o.oneofs)
                                        d.error_code = "function_parsing_error";
                                }
                                if (m.id_error != null && m.hasOwnProperty("id_error")) {
                                    d.id_error = o.enums === String ? $root.google.ads.googleads.v11.errors.IdErrorEnum.IdError[m.id_error] : m.id_error;
                                    if (o.oneofs)
                                        d.error_code = "id_error";
                                }
                                if (m.image_error != null && m.hasOwnProperty("image_error")) {
                                    d.image_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ImageErrorEnum.ImageError[m.image_error] : m.image_error;
                                    if (o.oneofs)
                                        d.error_code = "image_error";
                                }
                                if (m.media_bundle_error != null && m.hasOwnProperty("media_bundle_error")) {
                                    d.media_bundle_error = o.enums === String ? $root.google.ads.googleads.v11.errors.MediaBundleErrorEnum.MediaBundleError[m.media_bundle_error] : m.media_bundle_error;
                                    if (o.oneofs)
                                        d.error_code = "media_bundle_error";
                                }
                                if (m.multiplier_error != null && m.hasOwnProperty("multiplier_error")) {
                                    d.multiplier_error = o.enums === String ? $root.google.ads.googleads.v11.errors.MultiplierErrorEnum.MultiplierError[m.multiplier_error] : m.multiplier_error;
                                    if (o.oneofs)
                                        d.error_code = "multiplier_error";
                                }
                                if (m.new_resource_creation_error != null && m.hasOwnProperty("new_resource_creation_error")) {
                                    d.new_resource_creation_error = o.enums === String ? $root.google.ads.googleads.v11.errors.NewResourceCreationErrorEnum.NewResourceCreationError[m.new_resource_creation_error] : m.new_resource_creation_error;
                                    if (o.oneofs)
                                        d.error_code = "new_resource_creation_error";
                                }
                                if (m.not_empty_error != null && m.hasOwnProperty("not_empty_error")) {
                                    d.not_empty_error = o.enums === String ? $root.google.ads.googleads.v11.errors.NotEmptyErrorEnum.NotEmptyError[m.not_empty_error] : m.not_empty_error;
                                    if (o.oneofs)
                                        d.error_code = "not_empty_error";
                                }
                                if (m.null_error != null && m.hasOwnProperty("null_error")) {
                                    d.null_error = o.enums === String ? $root.google.ads.googleads.v11.errors.NullErrorEnum.NullError[m.null_error] : m.null_error;
                                    if (o.oneofs)
                                        d.error_code = "null_error";
                                }
                                if (m.operator_error != null && m.hasOwnProperty("operator_error")) {
                                    d.operator_error = o.enums === String ? $root.google.ads.googleads.v11.errors.OperatorErrorEnum.OperatorError[m.operator_error] : m.operator_error;
                                    if (o.oneofs)
                                        d.error_code = "operator_error";
                                }
                                if (m.range_error != null && m.hasOwnProperty("range_error")) {
                                    d.range_error = o.enums === String ? $root.google.ads.googleads.v11.errors.RangeErrorEnum.RangeError[m.range_error] : m.range_error;
                                    if (o.oneofs)
                                        d.error_code = "range_error";
                                }
                                if (m.region_code_error != null && m.hasOwnProperty("region_code_error")) {
                                    d.region_code_error = o.enums === String ? $root.google.ads.googleads.v11.errors.RegionCodeErrorEnum.RegionCodeError[m.region_code_error] : m.region_code_error;
                                    if (o.oneofs)
                                        d.error_code = "region_code_error";
                                }
                                if (m.setting_error != null && m.hasOwnProperty("setting_error")) {
                                    d.setting_error = o.enums === String ? $root.google.ads.googleads.v11.errors.SettingErrorEnum.SettingError[m.setting_error] : m.setting_error;
                                    if (o.oneofs)
                                        d.error_code = "setting_error";
                                }
                                if (m.string_format_error != null && m.hasOwnProperty("string_format_error")) {
                                    d.string_format_error = o.enums === String ? $root.google.ads.googleads.v11.errors.StringFormatErrorEnum.StringFormatError[m.string_format_error] : m.string_format_error;
                                    if (o.oneofs)
                                        d.error_code = "string_format_error";
                                }
                                if (m.string_length_error != null && m.hasOwnProperty("string_length_error")) {
                                    d.string_length_error = o.enums === String ? $root.google.ads.googleads.v11.errors.StringLengthErrorEnum.StringLengthError[m.string_length_error] : m.string_length_error;
                                    if (o.oneofs)
                                        d.error_code = "string_length_error";
                                }
                                if (m.operation_access_denied_error != null && m.hasOwnProperty("operation_access_denied_error")) {
                                    d.operation_access_denied_error = o.enums === String ? $root.google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum.OperationAccessDeniedError[m.operation_access_denied_error] : m.operation_access_denied_error;
                                    if (o.oneofs)
                                        d.error_code = "operation_access_denied_error";
                                }
                                if (m.resource_access_denied_error != null && m.hasOwnProperty("resource_access_denied_error")) {
                                    d.resource_access_denied_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError[m.resource_access_denied_error] : m.resource_access_denied_error;
                                    if (o.oneofs)
                                        d.error_code = "resource_access_denied_error";
                                }
                                if (m.resource_count_limit_exceeded_error != null && m.hasOwnProperty("resource_count_limit_exceeded_error")) {
                                    d.resource_count_limit_exceeded_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError[m.resource_count_limit_exceeded_error] : m.resource_count_limit_exceeded_error;
                                    if (o.oneofs)
                                        d.error_code = "resource_count_limit_exceeded_error";
                                }
                                if (m.recommendation_error != null && m.hasOwnProperty("recommendation_error")) {
                                    d.recommendation_error = o.enums === String ? $root.google.ads.googleads.v11.errors.RecommendationErrorEnum.RecommendationError[m.recommendation_error] : m.recommendation_error;
                                    if (o.oneofs)
                                        d.error_code = "recommendation_error";
                                }
                                if (m.ad_group_bid_modifier_error != null && m.hasOwnProperty("ad_group_bid_modifier_error")) {
                                    d.ad_group_bid_modifier_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdGroupBidModifierErrorEnum.AdGroupBidModifierError[m.ad_group_bid_modifier_error] : m.ad_group_bid_modifier_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_group_bid_modifier_error";
                                }
                                if (m.context_error != null && m.hasOwnProperty("context_error")) {
                                    d.context_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ContextErrorEnum.ContextError[m.context_error] : m.context_error;
                                    if (o.oneofs)
                                        d.error_code = "context_error";
                                }
                                if (m.field_error != null && m.hasOwnProperty("field_error")) {
                                    d.field_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FieldErrorEnum.FieldError[m.field_error] : m.field_error;
                                    if (o.oneofs)
                                        d.error_code = "field_error";
                                }
                                if (m.shared_set_error != null && m.hasOwnProperty("shared_set_error")) {
                                    d.shared_set_error = o.enums === String ? $root.google.ads.googleads.v11.errors.SharedSetErrorEnum.SharedSetError[m.shared_set_error] : m.shared_set_error;
                                    if (o.oneofs)
                                        d.error_code = "shared_set_error";
                                }
                                if (m.shared_criterion_error != null && m.hasOwnProperty("shared_criterion_error")) {
                                    d.shared_criterion_error = o.enums === String ? $root.google.ads.googleads.v11.errors.SharedCriterionErrorEnum.SharedCriterionError[m.shared_criterion_error] : m.shared_criterion_error;
                                    if (o.oneofs)
                                        d.error_code = "shared_criterion_error";
                                }
                                if (m.campaign_shared_set_error != null && m.hasOwnProperty("campaign_shared_set_error")) {
                                    d.campaign_shared_set_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignSharedSetErrorEnum.CampaignSharedSetError[m.campaign_shared_set_error] : m.campaign_shared_set_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_shared_set_error";
                                }
                                if (m.conversion_action_error != null && m.hasOwnProperty("conversion_action_error")) {
                                    d.conversion_action_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ConversionActionErrorEnum.ConversionActionError[m.conversion_action_error] : m.conversion_action_error;
                                    if (o.oneofs)
                                        d.error_code = "conversion_action_error";
                                }
                                if (m.header_error != null && m.hasOwnProperty("header_error")) {
                                    d.header_error = o.enums === String ? $root.google.ads.googleads.v11.errors.HeaderErrorEnum.HeaderError[m.header_error] : m.header_error;
                                    if (o.oneofs)
                                        d.error_code = "header_error";
                                }
                                if (m.database_error != null && m.hasOwnProperty("database_error")) {
                                    d.database_error = o.enums === String ? $root.google.ads.googleads.v11.errors.DatabaseErrorEnum.DatabaseError[m.database_error] : m.database_error;
                                    if (o.oneofs)
                                        d.error_code = "database_error";
                                }
                                if (m.policy_finding_error != null && m.hasOwnProperty("policy_finding_error")) {
                                    d.policy_finding_error = o.enums === String ? $root.google.ads.googleads.v11.errors.PolicyFindingErrorEnum.PolicyFindingError[m.policy_finding_error] : m.policy_finding_error;
                                    if (o.oneofs)
                                        d.error_code = "policy_finding_error";
                                }
                                if (m.enum_error != null && m.hasOwnProperty("enum_error")) {
                                    d.enum_error = o.enums === String ? $root.google.ads.googleads.v11.errors.EnumErrorEnum.EnumError[m.enum_error] : m.enum_error;
                                    if (o.oneofs)
                                        d.error_code = "enum_error";
                                }
                                if (m.keyword_plan_error != null && m.hasOwnProperty("keyword_plan_error")) {
                                    d.keyword_plan_error = o.enums === String ? $root.google.ads.googleads.v11.errors.KeywordPlanErrorEnum.KeywordPlanError[m.keyword_plan_error] : m.keyword_plan_error;
                                    if (o.oneofs)
                                        d.error_code = "keyword_plan_error";
                                }
                                if (m.keyword_plan_campaign_error != null && m.hasOwnProperty("keyword_plan_campaign_error")) {
                                    d.keyword_plan_campaign_error = o.enums === String ? $root.google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError[m.keyword_plan_campaign_error] : m.keyword_plan_campaign_error;
                                    if (o.oneofs)
                                        d.error_code = "keyword_plan_campaign_error";
                                }
                                if (m.keyword_plan_ad_group_error != null && m.hasOwnProperty("keyword_plan_ad_group_error")) {
                                    d.keyword_plan_ad_group_error = o.enums === String ? $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError[m.keyword_plan_ad_group_error] : m.keyword_plan_ad_group_error;
                                    if (o.oneofs)
                                        d.error_code = "keyword_plan_ad_group_error";
                                }
                                if (m.keyword_plan_idea_error != null && m.hasOwnProperty("keyword_plan_idea_error")) {
                                    d.keyword_plan_idea_error = o.enums === String ? $root.google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError[m.keyword_plan_idea_error] : m.keyword_plan_idea_error;
                                    if (o.oneofs)
                                        d.error_code = "keyword_plan_idea_error";
                                }
                                if (m.account_budget_proposal_error != null && m.hasOwnProperty("account_budget_proposal_error")) {
                                    d.account_budget_proposal_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AccountBudgetProposalErrorEnum.AccountBudgetProposalError[m.account_budget_proposal_error] : m.account_budget_proposal_error;
                                    if (o.oneofs)
                                        d.error_code = "account_budget_proposal_error";
                                }
                                if (m.user_list_error != null && m.hasOwnProperty("user_list_error")) {
                                    d.user_list_error = o.enums === String ? $root.google.ads.googleads.v11.errors.UserListErrorEnum.UserListError[m.user_list_error] : m.user_list_error;
                                    if (o.oneofs)
                                        d.error_code = "user_list_error";
                                }
                                if (m.change_status_error != null && m.hasOwnProperty("change_status_error")) {
                                    d.change_status_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ChangeStatusErrorEnum.ChangeStatusError[m.change_status_error] : m.change_status_error;
                                    if (o.oneofs)
                                        d.error_code = "change_status_error";
                                }
                                if (m.feed_error != null && m.hasOwnProperty("feed_error")) {
                                    d.feed_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedErrorEnum.FeedError[m.feed_error] : m.feed_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_error";
                                }
                                if (m.geo_target_constant_suggestion_error != null && m.hasOwnProperty("geo_target_constant_suggestion_error")) {
                                    d.geo_target_constant_suggestion_error = o.enums === String ? $root.google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError[m.geo_target_constant_suggestion_error] : m.geo_target_constant_suggestion_error;
                                    if (o.oneofs)
                                        d.error_code = "geo_target_constant_suggestion_error";
                                }
                                if (m.campaign_draft_error != null && m.hasOwnProperty("campaign_draft_error")) {
                                    d.campaign_draft_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignDraftErrorEnum.CampaignDraftError[m.campaign_draft_error] : m.campaign_draft_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_draft_error";
                                }
                                if (m.feed_item_error != null && m.hasOwnProperty("feed_item_error")) {
                                    d.feed_item_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedItemErrorEnum.FeedItemError[m.feed_item_error] : m.feed_item_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_item_error";
                                }
                                if (m.label_error != null && m.hasOwnProperty("label_error")) {
                                    d.label_error = o.enums === String ? $root.google.ads.googleads.v11.errors.LabelErrorEnum.LabelError[m.label_error] : m.label_error;
                                    if (o.oneofs)
                                        d.error_code = "label_error";
                                }
                                if (m.media_file_error != null && m.hasOwnProperty("media_file_error")) {
                                    d.media_file_error = o.enums === String ? $root.google.ads.googleads.v11.errors.MediaFileErrorEnum.MediaFileError[m.media_file_error] : m.media_file_error;
                                    if (o.oneofs)
                                        d.error_code = "media_file_error";
                                }
                                if (m.billing_setup_error != null && m.hasOwnProperty("billing_setup_error")) {
                                    d.billing_setup_error = o.enums === String ? $root.google.ads.googleads.v11.errors.BillingSetupErrorEnum.BillingSetupError[m.billing_setup_error] : m.billing_setup_error;
                                    if (o.oneofs)
                                        d.error_code = "billing_setup_error";
                                }
                                if (m.customer_client_link_error != null && m.hasOwnProperty("customer_client_link_error")) {
                                    d.customer_client_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomerClientLinkErrorEnum.CustomerClientLinkError[m.customer_client_link_error] : m.customer_client_link_error;
                                    if (o.oneofs)
                                        d.error_code = "customer_client_link_error";
                                }
                                if (m.customer_error != null && m.hasOwnProperty("customer_error")) {
                                    d.customer_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomerErrorEnum.CustomerError[m.customer_error] : m.customer_error;
                                    if (o.oneofs)
                                        d.error_code = "customer_error";
                                }
                                if (m.customer_manager_link_error != null && m.hasOwnProperty("customer_manager_link_error")) {
                                    d.customer_manager_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomerManagerLinkErrorEnum.CustomerManagerLinkError[m.customer_manager_link_error] : m.customer_manager_link_error;
                                    if (o.oneofs)
                                        d.error_code = "customer_manager_link_error";
                                }
                                if (m.feed_mapping_error != null && m.hasOwnProperty("feed_mapping_error")) {
                                    d.feed_mapping_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedMappingErrorEnum.FeedMappingError[m.feed_mapping_error] : m.feed_mapping_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_mapping_error";
                                }
                                if (m.customer_feed_error != null && m.hasOwnProperty("customer_feed_error")) {
                                    d.customer_feed_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomerFeedErrorEnum.CustomerFeedError[m.customer_feed_error] : m.customer_feed_error;
                                    if (o.oneofs)
                                        d.error_code = "customer_feed_error";
                                }
                                if (m.ad_group_feed_error != null && m.hasOwnProperty("ad_group_feed_error")) {
                                    d.ad_group_feed_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdGroupFeedErrorEnum.AdGroupFeedError[m.ad_group_feed_error] : m.ad_group_feed_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_group_feed_error";
                                }
                                if (m.campaign_feed_error != null && m.hasOwnProperty("campaign_feed_error")) {
                                    d.campaign_feed_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignFeedErrorEnum.CampaignFeedError[m.campaign_feed_error] : m.campaign_feed_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_feed_error";
                                }
                                if (m.custom_interest_error != null && m.hasOwnProperty("custom_interest_error")) {
                                    d.custom_interest_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomInterestErrorEnum.CustomInterestError[m.custom_interest_error] : m.custom_interest_error;
                                    if (o.oneofs)
                                        d.error_code = "custom_interest_error";
                                }
                                if (m.campaign_experiment_error != null && m.hasOwnProperty("campaign_experiment_error")) {
                                    d.campaign_experiment_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignExperimentErrorEnum.CampaignExperimentError[m.campaign_experiment_error] : m.campaign_experiment_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_experiment_error";
                                }
                                if (m.extension_feed_item_error != null && m.hasOwnProperty("extension_feed_item_error")) {
                                    d.extension_feed_item_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum.ExtensionFeedItemError[m.extension_feed_item_error] : m.extension_feed_item_error;
                                    if (o.oneofs)
                                        d.error_code = "extension_feed_item_error";
                                }
                                if (m.ad_parameter_error != null && m.hasOwnProperty("ad_parameter_error")) {
                                    d.ad_parameter_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdParameterErrorEnum.AdParameterError[m.ad_parameter_error] : m.ad_parameter_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_parameter_error";
                                }
                                if (m.feed_item_validation_error != null && m.hasOwnProperty("feed_item_validation_error")) {
                                    d.feed_item_validation_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.FeedItemValidationError[m.feed_item_validation_error] : m.feed_item_validation_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_item_validation_error";
                                }
                                if (m.extension_setting_error != null && m.hasOwnProperty("extension_setting_error")) {
                                    d.extension_setting_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ExtensionSettingErrorEnum.ExtensionSettingError[m.extension_setting_error] : m.extension_setting_error;
                                    if (o.oneofs)
                                        d.error_code = "extension_setting_error";
                                }
                                if (m.feed_item_target_error != null && m.hasOwnProperty("feed_item_target_error")) {
                                    d.feed_item_target_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedItemTargetErrorEnum.FeedItemTargetError[m.feed_item_target_error] : m.feed_item_target_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_item_target_error";
                                }
                                if (m.policy_violation_error != null && m.hasOwnProperty("policy_violation_error")) {
                                    d.policy_violation_error = o.enums === String ? $root.google.ads.googleads.v11.errors.PolicyViolationErrorEnum.PolicyViolationError[m.policy_violation_error] : m.policy_violation_error;
                                    if (o.oneofs)
                                        d.error_code = "policy_violation_error";
                                }
                                if (m.asset_error != null && m.hasOwnProperty("asset_error")) {
                                    d.asset_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetErrorEnum.AssetError[m.asset_error] : m.asset_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_error";
                                }
                                if (m.country_code_error != null && m.hasOwnProperty("country_code_error")) {
                                    d.country_code_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CountryCodeErrorEnum.CountryCodeError[m.country_code_error] : m.country_code_error;
                                    if (o.oneofs)
                                        d.error_code = "country_code_error";
                                }
                                if (m.language_code_error != null && m.hasOwnProperty("language_code_error")) {
                                    d.language_code_error = o.enums === String ? $root.google.ads.googleads.v11.errors.LanguageCodeErrorEnum.LanguageCodeError[m.language_code_error] : m.language_code_error;
                                    if (o.oneofs)
                                        d.error_code = "language_code_error";
                                }
                                if (m.conversion_upload_error != null && m.hasOwnProperty("conversion_upload_error")) {
                                    d.conversion_upload_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ConversionUploadErrorEnum.ConversionUploadError[m.conversion_upload_error] : m.conversion_upload_error;
                                    if (o.oneofs)
                                        d.error_code = "conversion_upload_error";
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = o.enums === String ? $root.google.ads.googleads.v11.errors.PartialFailureErrorEnum.PartialFailureError[m.partial_failure_error] : m.partial_failure_error;
                                    if (o.oneofs)
                                        d.error_code = "partial_failure_error";
                                }
                                if (m.policy_validation_parameter_error != null && m.hasOwnProperty("policy_validation_parameter_error")) {
                                    d.policy_validation_parameter_error = o.enums === String ? $root.google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum.PolicyValidationParameterError[m.policy_validation_parameter_error] : m.policy_validation_parameter_error;
                                    if (o.oneofs)
                                        d.error_code = "policy_validation_parameter_error";
                                }
                                if (m.conversion_adjustment_upload_error != null && m.hasOwnProperty("conversion_adjustment_upload_error")) {
                                    d.conversion_adjustment_upload_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError[m.conversion_adjustment_upload_error] : m.conversion_adjustment_upload_error;
                                    if (o.oneofs)
                                        d.error_code = "conversion_adjustment_upload_error";
                                }
                                if (m.media_upload_error != null && m.hasOwnProperty("media_upload_error")) {
                                    d.media_upload_error = o.enums === String ? $root.google.ads.googleads.v11.errors.MediaUploadErrorEnum.MediaUploadError[m.media_upload_error] : m.media_upload_error;
                                    if (o.oneofs)
                                        d.error_code = "media_upload_error";
                                }
                                if (m.youtube_video_registration_error != null && m.hasOwnProperty("youtube_video_registration_error")) {
                                    d.youtube_video_registration_error = o.enums === String ? $root.google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError[m.youtube_video_registration_error] : m.youtube_video_registration_error;
                                    if (o.oneofs)
                                        d.error_code = "youtube_video_registration_error";
                                }
                                if (m.size_limit_error != null && m.hasOwnProperty("size_limit_error")) {
                                    d.size_limit_error = o.enums === String ? $root.google.ads.googleads.v11.errors.SizeLimitErrorEnum.SizeLimitError[m.size_limit_error] : m.size_limit_error;
                                    if (o.oneofs)
                                        d.error_code = "size_limit_error";
                                }
                                if (m.offline_user_data_job_error != null && m.hasOwnProperty("offline_user_data_job_error")) {
                                    d.offline_user_data_job_error = o.enums === String ? $root.google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum.OfflineUserDataJobError[m.offline_user_data_job_error] : m.offline_user_data_job_error;
                                    if (o.oneofs)
                                        d.error_code = "offline_user_data_job_error";
                                }
                                if (m.manager_link_error != null && m.hasOwnProperty("manager_link_error")) {
                                    d.manager_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ManagerLinkErrorEnum.ManagerLinkError[m.manager_link_error] : m.manager_link_error;
                                    if (o.oneofs)
                                        d.error_code = "manager_link_error";
                                }
                                if (m.currency_code_error != null && m.hasOwnProperty("currency_code_error")) {
                                    d.currency_code_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CurrencyCodeErrorEnum.CurrencyCodeError[m.currency_code_error] : m.currency_code_error;
                                    if (o.oneofs)
                                        d.error_code = "currency_code_error";
                                }
                                if (m.experiment_error != null && m.hasOwnProperty("experiment_error")) {
                                    d.experiment_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ExperimentErrorEnum.ExperimentError[m.experiment_error] : m.experiment_error;
                                    if (o.oneofs)
                                        d.error_code = "experiment_error";
                                }
                                if (m.access_invitation_error != null && m.hasOwnProperty("access_invitation_error")) {
                                    d.access_invitation_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AccessInvitationErrorEnum.AccessInvitationError[m.access_invitation_error] : m.access_invitation_error;
                                    if (o.oneofs)
                                        d.error_code = "access_invitation_error";
                                }
                                if (m.reach_plan_error != null && m.hasOwnProperty("reach_plan_error")) {
                                    d.reach_plan_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ReachPlanErrorEnum.ReachPlanError[m.reach_plan_error] : m.reach_plan_error;
                                    if (o.oneofs)
                                        d.error_code = "reach_plan_error";
                                }
                                if (m.invoice_error != null && m.hasOwnProperty("invoice_error")) {
                                    d.invoice_error = o.enums === String ? $root.google.ads.googleads.v11.errors.InvoiceErrorEnum.InvoiceError[m.invoice_error] : m.invoice_error;
                                    if (o.oneofs)
                                        d.error_code = "invoice_error";
                                }
                                if (m.payments_account_error != null && m.hasOwnProperty("payments_account_error")) {
                                    d.payments_account_error = o.enums === String ? $root.google.ads.googleads.v11.errors.PaymentsAccountErrorEnum.PaymentsAccountError[m.payments_account_error] : m.payments_account_error;
                                    if (o.oneofs)
                                        d.error_code = "payments_account_error";
                                }
                                if (m.time_zone_error != null && m.hasOwnProperty("time_zone_error")) {
                                    d.time_zone_error = o.enums === String ? $root.google.ads.googleads.v11.errors.TimeZoneErrorEnum.TimeZoneError[m.time_zone_error] : m.time_zone_error;
                                    if (o.oneofs)
                                        d.error_code = "time_zone_error";
                                }
                                if (m.asset_link_error != null && m.hasOwnProperty("asset_link_error")) {
                                    d.asset_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetLinkErrorEnum.AssetLinkError[m.asset_link_error] : m.asset_link_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_link_error";
                                }
                                if (m.user_data_error != null && m.hasOwnProperty("user_data_error")) {
                                    d.user_data_error = o.enums === String ? $root.google.ads.googleads.v11.errors.UserDataErrorEnum.UserDataError[m.user_data_error] : m.user_data_error;
                                    if (o.oneofs)
                                        d.error_code = "user_data_error";
                                }
                                if (m.batch_job_error != null && m.hasOwnProperty("batch_job_error")) {
                                    d.batch_job_error = o.enums === String ? $root.google.ads.googleads.v11.errors.BatchJobErrorEnum.BatchJobError[m.batch_job_error] : m.batch_job_error;
                                    if (o.oneofs)
                                        d.error_code = "batch_job_error";
                                }
                                if (m.keyword_plan_campaign_keyword_error != null && m.hasOwnProperty("keyword_plan_campaign_keyword_error")) {
                                    d.keyword_plan_campaign_keyword_error = o.enums === String ? $root.google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError[m.keyword_plan_campaign_keyword_error] : m.keyword_plan_campaign_keyword_error;
                                    if (o.oneofs)
                                        d.error_code = "keyword_plan_campaign_keyword_error";
                                }
                                if (m.keyword_plan_ad_group_keyword_error != null && m.hasOwnProperty("keyword_plan_ad_group_keyword_error")) {
                                    d.keyword_plan_ad_group_keyword_error = o.enums === String ? $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError[m.keyword_plan_ad_group_keyword_error] : m.keyword_plan_ad_group_keyword_error;
                                    if (o.oneofs)
                                        d.error_code = "keyword_plan_ad_group_keyword_error";
                                }
                                if (m.account_link_error != null && m.hasOwnProperty("account_link_error")) {
                                    d.account_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AccountLinkErrorEnum.AccountLinkError[m.account_link_error] : m.account_link_error;
                                    if (o.oneofs)
                                        d.error_code = "account_link_error";
                                }
                                if (m.third_party_app_analytics_link_error != null && m.hasOwnProperty("third_party_app_analytics_link_error")) {
                                    d.third_party_app_analytics_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError[m.third_party_app_analytics_link_error] : m.third_party_app_analytics_link_error;
                                    if (o.oneofs)
                                        d.error_code = "third_party_app_analytics_link_error";
                                }
                                if (m.change_event_error != null && m.hasOwnProperty("change_event_error")) {
                                    d.change_event_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ChangeEventErrorEnum.ChangeEventError[m.change_event_error] : m.change_event_error;
                                    if (o.oneofs)
                                        d.error_code = "change_event_error";
                                }
                                if (m.not_allowlisted_error != null && m.hasOwnProperty("not_allowlisted_error")) {
                                    d.not_allowlisted_error = o.enums === String ? $root.google.ads.googleads.v11.errors.NotAllowlistedErrorEnum.NotAllowlistedError[m.not_allowlisted_error] : m.not_allowlisted_error;
                                    if (o.oneofs)
                                        d.error_code = "not_allowlisted_error";
                                }
                                if (m.customer_user_access_error != null && m.hasOwnProperty("customer_user_access_error")) {
                                    d.customer_user_access_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomerUserAccessErrorEnum.CustomerUserAccessError[m.customer_user_access_error] : m.customer_user_access_error;
                                    if (o.oneofs)
                                        d.error_code = "customer_user_access_error";
                                }
                                if (m.custom_audience_error != null && m.hasOwnProperty("custom_audience_error")) {
                                    d.custom_audience_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomAudienceErrorEnum.CustomAudienceError[m.custom_audience_error] : m.custom_audience_error;
                                    if (o.oneofs)
                                        d.error_code = "custom_audience_error";
                                }
                                if (m.feed_item_set_error != null && m.hasOwnProperty("feed_item_set_error")) {
                                    d.feed_item_set_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedItemSetErrorEnum.FeedItemSetError[m.feed_item_set_error] : m.feed_item_set_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_item_set_error";
                                }
                                if (m.feed_item_set_link_error != null && m.hasOwnProperty("feed_item_set_link_error")) {
                                    d.feed_item_set_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum.FeedItemSetLinkError[m.feed_item_set_link_error] : m.feed_item_set_link_error;
                                    if (o.oneofs)
                                        d.error_code = "feed_item_set_link_error";
                                }
                                if (m.conversion_custom_variable_error != null && m.hasOwnProperty("conversion_custom_variable_error")) {
                                    d.conversion_custom_variable_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ConversionCustomVariableErrorEnum.ConversionCustomVariableError[m.conversion_custom_variable_error] : m.conversion_custom_variable_error;
                                    if (o.oneofs)
                                        d.error_code = "conversion_custom_variable_error";
                                }
                                if (m.conversion_value_rule_error != null && m.hasOwnProperty("conversion_value_rule_error")) {
                                    d.conversion_value_rule_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ConversionValueRuleErrorEnum.ConversionValueRuleError[m.conversion_value_rule_error] : m.conversion_value_rule_error;
                                    if (o.oneofs)
                                        d.error_code = "conversion_value_rule_error";
                                }
                                if (m.conversion_value_rule_set_error != null && m.hasOwnProperty("conversion_value_rule_set_error")) {
                                    d.conversion_value_rule_set_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError[m.conversion_value_rule_set_error] : m.conversion_value_rule_set_error;
                                    if (o.oneofs)
                                        d.error_code = "conversion_value_rule_set_error";
                                }
                                if (m.asset_group_error != null && m.hasOwnProperty("asset_group_error")) {
                                    d.asset_group_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetGroupErrorEnum.AssetGroupError[m.asset_group_error] : m.asset_group_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_group_error";
                                }
                                if (m.asset_group_asset_error != null && m.hasOwnProperty("asset_group_asset_error")) {
                                    d.asset_group_asset_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetGroupAssetErrorEnum.AssetGroupAssetError[m.asset_group_asset_error] : m.asset_group_asset_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_group_asset_error";
                                }
                                if (m.custom_conversion_goal_error != null && m.hasOwnProperty("custom_conversion_goal_error")) {
                                    d.custom_conversion_goal_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomConversionGoalErrorEnum.CustomConversionGoalError[m.custom_conversion_goal_error] : m.custom_conversion_goal_error;
                                    if (o.oneofs)
                                        d.error_code = "custom_conversion_goal_error";
                                }
                                if (m.customizer_attribute_error != null && m.hasOwnProperty("customizer_attribute_error")) {
                                    d.customizer_attribute_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomizerAttributeErrorEnum.CustomizerAttributeError[m.customizer_attribute_error] : m.customizer_attribute_error;
                                    if (o.oneofs)
                                        d.error_code = "customizer_attribute_error";
                                }
                                if (m.asset_set_error != null && m.hasOwnProperty("asset_set_error")) {
                                    d.asset_set_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetSetErrorEnum.AssetSetError[m.asset_set_error] : m.asset_set_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_set_error";
                                }
                                if (m.asset_set_asset_error != null && m.hasOwnProperty("asset_set_asset_error")) {
                                    d.asset_set_asset_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetSetAssetErrorEnum.AssetSetAssetError[m.asset_set_asset_error] : m.asset_set_asset_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_set_asset_error";
                                }
                                if (m.asset_set_link_error != null && m.hasOwnProperty("asset_set_link_error")) {
                                    d.asset_set_link_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetSetLinkErrorEnum.AssetSetLinkError[m.asset_set_link_error] : m.asset_set_link_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_set_link_error";
                                }
                                if (m.asset_group_listing_group_filter_error != null && m.hasOwnProperty("asset_group_listing_group_filter_error")) {
                                    d.asset_group_listing_group_filter_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AssetGroupListingGroupFilterErrorEnum.AssetGroupListingGroupFilterError[m.asset_group_listing_group_filter_error] : m.asset_group_listing_group_filter_error;
                                    if (o.oneofs)
                                        d.error_code = "asset_group_listing_group_filter_error";
                                }
                                if (m.experiment_arm_error != null && m.hasOwnProperty("experiment_arm_error")) {
                                    d.experiment_arm_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ExperimentArmErrorEnum.ExperimentArmError[m.experiment_arm_error] : m.experiment_arm_error;
                                    if (o.oneofs)
                                        d.error_code = "experiment_arm_error";
                                }
                                if (m.customer_customizer_error != null && m.hasOwnProperty("customer_customizer_error")) {
                                    d.customer_customizer_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CustomerCustomizerErrorEnum.CustomerCustomizerError[m.customer_customizer_error] : m.customer_customizer_error;
                                    if (o.oneofs)
                                        d.error_code = "customer_customizer_error";
                                }
                                if (m.ad_group_customizer_error != null && m.hasOwnProperty("ad_group_customizer_error")) {
                                    d.ad_group_customizer_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdGroupCustomizerErrorEnum.AdGroupCustomizerError[m.ad_group_customizer_error] : m.ad_group_customizer_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_group_customizer_error";
                                }
                                if (m.campaign_customizer_error != null && m.hasOwnProperty("campaign_customizer_error")) {
                                    d.campaign_customizer_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignCustomizerErrorEnum.CampaignCustomizerError[m.campaign_customizer_error] : m.campaign_customizer_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_customizer_error";
                                }
                                if (m.ad_group_criterion_customizer_error != null && m.hasOwnProperty("ad_group_criterion_customizer_error")) {
                                    d.ad_group_criterion_customizer_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AdGroupCriterionCustomizerErrorEnum.AdGroupCriterionCustomizerError[m.ad_group_criterion_customizer_error] : m.ad_group_criterion_customizer_error;
                                    if (o.oneofs)
                                        d.error_code = "ad_group_criterion_customizer_error";
                                }
                                if (m.merchant_center_error != null && m.hasOwnProperty("merchant_center_error")) {
                                    d.merchant_center_error = o.enums === String ? $root.google.ads.googleads.v11.errors.MerchantCenterErrorEnum.MerchantCenterError[m.merchant_center_error] : m.merchant_center_error;
                                    if (o.oneofs)
                                        d.error_code = "merchant_center_error";
                                }
                                if (m.audience_error != null && m.hasOwnProperty("audience_error")) {
                                    d.audience_error = o.enums === String ? $root.google.ads.googleads.v11.errors.AudienceErrorEnum.AudienceError[m.audience_error] : m.audience_error;
                                    if (o.oneofs)
                                        d.error_code = "audience_error";
                                }
                                if (m.conversion_goal_campaign_config_error != null && m.hasOwnProperty("conversion_goal_campaign_config_error")) {
                                    d.conversion_goal_campaign_config_error = o.enums === String ? $root.google.ads.googleads.v11.errors.ConversionGoalCampaignConfigErrorEnum.ConversionGoalCampaignConfigError[m.conversion_goal_campaign_config_error] : m.conversion_goal_campaign_config_error;
                                    if (o.oneofs)
                                        d.error_code = "conversion_goal_campaign_config_error";
                                }
                                if (m.campaign_conversion_goal_error != null && m.hasOwnProperty("campaign_conversion_goal_error")) {
                                    d.campaign_conversion_goal_error = o.enums === String ? $root.google.ads.googleads.v11.errors.CampaignConversionGoalErrorEnum.CampaignConversionGoalError[m.campaign_conversion_goal_error] : m.campaign_conversion_goal_error;
                                    if (o.oneofs)
                                        d.error_code = "campaign_conversion_goal_error";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ErrorCode to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ErrorCode
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ErrorCode.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ErrorCode;
                        })();
    
                        errors.ErrorLocation = (function() {
    
                            /**
                             * Properties of an ErrorLocation.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IErrorLocation
                             * @property {Array.<google.ads.googleads.v11.errors.ErrorLocation.IFieldPathElement>|null} [field_path_elements] ErrorLocation field_path_elements
                             */
    
                            /**
                             * Constructs a new ErrorLocation.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ErrorLocation.
                             * @implements IErrorLocation
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IErrorLocation=} [p] Properties to set
                             */
                            function ErrorLocation(p) {
                                this.field_path_elements = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ErrorLocation field_path_elements.
                             * @member {Array.<google.ads.googleads.v11.errors.ErrorLocation.IFieldPathElement>} field_path_elements
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @instance
                             */
                            ErrorLocation.prototype.field_path_elements = $util.emptyArray;
    
                            /**
                             * Creates a new ErrorLocation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorLocation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ErrorLocation} ErrorLocation instance
                             */
                            ErrorLocation.create = function create(properties) {
                                return new ErrorLocation(properties);
                            };
    
                            /**
                             * Encodes the specified ErrorLocation message. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorLocation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorLocation} m ErrorLocation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ErrorLocation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.field_path_elements != null && m.field_path_elements.length) {
                                    for (var i = 0; i < m.field_path_elements.length; ++i)
                                        $root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement.encode(m.field_path_elements[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ErrorLocation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorLocation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorLocation} message ErrorLocation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ErrorLocation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ErrorLocation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ErrorLocation} ErrorLocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ErrorLocation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ErrorLocation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        if (!(m.field_path_elements && m.field_path_elements.length))
                                            m.field_path_elements = [];
                                        m.field_path_elements.push($root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ErrorLocation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ErrorLocation} ErrorLocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ErrorLocation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ErrorLocation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ErrorLocation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.field_path_elements != null && m.hasOwnProperty("field_path_elements")) {
                                    if (!Array.isArray(m.field_path_elements))
                                        return "field_path_elements: array expected";
                                    for (var i = 0; i < m.field_path_elements.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement.verify(m.field_path_elements[i]);
                                            if (e)
                                                return "field_path_elements." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ErrorLocation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ErrorLocation} ErrorLocation
                             */
                            ErrorLocation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ErrorLocation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.ErrorLocation();
                                if (d.field_path_elements) {
                                    if (!Array.isArray(d.field_path_elements))
                                        throw TypeError(".google.ads.googleads.v11.errors.ErrorLocation.field_path_elements: array expected");
                                    m.field_path_elements = [];
                                    for (var i = 0; i < d.field_path_elements.length; ++i) {
                                        if (typeof d.field_path_elements[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.errors.ErrorLocation.field_path_elements: object expected");
                                        m.field_path_elements[i] = $root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement.fromObject(d.field_path_elements[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ErrorLocation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @static
                             * @param {google.ads.googleads.v11.errors.ErrorLocation} m ErrorLocation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ErrorLocation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.field_path_elements = [];
                                }
                                if (m.field_path_elements && m.field_path_elements.length) {
                                    d.field_path_elements = [];
                                    for (var j = 0; j < m.field_path_elements.length; ++j) {
                                        d.field_path_elements[j] = $root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement.toObject(m.field_path_elements[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ErrorLocation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ErrorLocation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ErrorLocation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            ErrorLocation.FieldPathElement = (function() {
    
                                /**
                                 * Properties of a FieldPathElement.
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation
                                 * @interface IFieldPathElement
                                 * @property {string|null} [field_name] FieldPathElement field_name
                                 * @property {number|null} [index] FieldPathElement index
                                 */
    
                                /**
                                 * Constructs a new FieldPathElement.
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation
                                 * @classdesc Represents a FieldPathElement.
                                 * @implements IFieldPathElement
                                 * @constructor
                                 * @param {google.ads.googleads.v11.errors.ErrorLocation.IFieldPathElement=} [p] Properties to set
                                 */
                                function FieldPathElement(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * FieldPathElement field_name.
                                 * @member {string} field_name
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @instance
                                 */
                                FieldPathElement.prototype.field_name = "";
    
                                /**
                                 * FieldPathElement index.
                                 * @member {number|null|undefined} index
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @instance
                                 */
                                FieldPathElement.prototype.index = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * FieldPathElement _index.
                                 * @member {"index"|undefined} _index
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @instance
                                 */
                                Object.defineProperty(FieldPathElement.prototype, "_index", {
                                    get: $util.oneOfGetter($oneOfFields = ["index"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new FieldPathElement instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {google.ads.googleads.v11.errors.ErrorLocation.IFieldPathElement=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement} FieldPathElement instance
                                 */
                                FieldPathElement.create = function create(properties) {
                                    return new FieldPathElement(properties);
                                };
    
                                /**
                                 * Encodes the specified FieldPathElement message. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {google.ads.googleads.v11.errors.ErrorLocation.IFieldPathElement} m FieldPathElement message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FieldPathElement.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.field_name != null && Object.hasOwnProperty.call(m, "field_name"))
                                        w.uint32(10).string(m.field_name);
                                    if (m.index != null && Object.hasOwnProperty.call(m, "index"))
                                        w.uint32(24).int32(m.index);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified FieldPathElement message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {google.ads.googleads.v11.errors.ErrorLocation.IFieldPathElement} message FieldPathElement message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FieldPathElement.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a FieldPathElement message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement} FieldPathElement
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FieldPathElement.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.field_name = r.string();
                                            break;
                                        case 3:
                                            m.index = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a FieldPathElement message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement} FieldPathElement
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FieldPathElement.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a FieldPathElement message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                FieldPathElement.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.field_name != null && m.hasOwnProperty("field_name")) {
                                        if (!$util.isString(m.field_name))
                                            return "field_name: string expected";
                                    }
                                    if (m.index != null && m.hasOwnProperty("index")) {
                                        p._index = 1;
                                        if (!$util.isInteger(m.index))
                                            return "index: integer expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a FieldPathElement message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement} FieldPathElement
                                 */
                                FieldPathElement.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement();
                                    if (d.field_name != null) {
                                        m.field_name = String(d.field_name);
                                    }
                                    if (d.index != null) {
                                        m.index = d.index | 0;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a FieldPathElement message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @static
                                 * @param {google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement} m FieldPathElement
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                FieldPathElement.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.field_name = "";
                                    }
                                    if (m.field_name != null && m.hasOwnProperty("field_name")) {
                                        d.field_name = m.field_name;
                                    }
                                    if (m.index != null && m.hasOwnProperty("index")) {
                                        d.index = m.index;
                                        if (o.oneofs)
                                            d._index = "index";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this FieldPathElement to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.errors.ErrorLocation.FieldPathElement
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                FieldPathElement.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return FieldPathElement;
                            })();
    
                            return ErrorLocation;
                        })();
    
                        errors.ErrorDetails = (function() {
    
                            /**
                             * Properties of an ErrorDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IErrorDetails
                             * @property {string|null} [unpublished_error_code] ErrorDetails unpublished_error_code
                             * @property {google.ads.googleads.v11.errors.IPolicyViolationDetails|null} [policy_violation_details] ErrorDetails policy_violation_details
                             * @property {google.ads.googleads.v11.errors.IPolicyFindingDetails|null} [policy_finding_details] ErrorDetails policy_finding_details
                             * @property {google.ads.googleads.v11.errors.IQuotaErrorDetails|null} [quota_error_details] ErrorDetails quota_error_details
                             * @property {google.ads.googleads.v11.errors.IResourceCountDetails|null} [resource_count_details] ErrorDetails resource_count_details
                             */
    
                            /**
                             * Constructs a new ErrorDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ErrorDetails.
                             * @implements IErrorDetails
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IErrorDetails=} [p] Properties to set
                             */
                            function ErrorDetails(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ErrorDetails unpublished_error_code.
                             * @member {string} unpublished_error_code
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @instance
                             */
                            ErrorDetails.prototype.unpublished_error_code = "";
    
                            /**
                             * ErrorDetails policy_violation_details.
                             * @member {google.ads.googleads.v11.errors.IPolicyViolationDetails|null|undefined} policy_violation_details
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @instance
                             */
                            ErrorDetails.prototype.policy_violation_details = null;
    
                            /**
                             * ErrorDetails policy_finding_details.
                             * @member {google.ads.googleads.v11.errors.IPolicyFindingDetails|null|undefined} policy_finding_details
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @instance
                             */
                            ErrorDetails.prototype.policy_finding_details = null;
    
                            /**
                             * ErrorDetails quota_error_details.
                             * @member {google.ads.googleads.v11.errors.IQuotaErrorDetails|null|undefined} quota_error_details
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @instance
                             */
                            ErrorDetails.prototype.quota_error_details = null;
    
                            /**
                             * ErrorDetails resource_count_details.
                             * @member {google.ads.googleads.v11.errors.IResourceCountDetails|null|undefined} resource_count_details
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @instance
                             */
                            ErrorDetails.prototype.resource_count_details = null;
    
                            /**
                             * Creates a new ErrorDetails instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorDetails=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ErrorDetails} ErrorDetails instance
                             */
                            ErrorDetails.create = function create(properties) {
                                return new ErrorDetails(properties);
                            };
    
                            /**
                             * Encodes the specified ErrorDetails message. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorDetails.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorDetails} m ErrorDetails message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ErrorDetails.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.unpublished_error_code != null && Object.hasOwnProperty.call(m, "unpublished_error_code"))
                                    w.uint32(10).string(m.unpublished_error_code);
                                if (m.policy_violation_details != null && Object.hasOwnProperty.call(m, "policy_violation_details"))
                                    $root.google.ads.googleads.v11.errors.PolicyViolationDetails.encode(m.policy_violation_details, w.uint32(18).fork()).ldelim();
                                if (m.policy_finding_details != null && Object.hasOwnProperty.call(m, "policy_finding_details"))
                                    $root.google.ads.googleads.v11.errors.PolicyFindingDetails.encode(m.policy_finding_details, w.uint32(26).fork()).ldelim();
                                if (m.quota_error_details != null && Object.hasOwnProperty.call(m, "quota_error_details"))
                                    $root.google.ads.googleads.v11.errors.QuotaErrorDetails.encode(m.quota_error_details, w.uint32(34).fork()).ldelim();
                                if (m.resource_count_details != null && Object.hasOwnProperty.call(m, "resource_count_details"))
                                    $root.google.ads.googleads.v11.errors.ResourceCountDetails.encode(m.resource_count_details, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ErrorDetails message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ErrorDetails.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IErrorDetails} message ErrorDetails message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ErrorDetails.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ErrorDetails message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ErrorDetails} ErrorDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ErrorDetails.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ErrorDetails();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.unpublished_error_code = r.string();
                                        break;
                                    case 2:
                                        m.policy_violation_details = $root.google.ads.googleads.v11.errors.PolicyViolationDetails.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.policy_finding_details = $root.google.ads.googleads.v11.errors.PolicyFindingDetails.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.quota_error_details = $root.google.ads.googleads.v11.errors.QuotaErrorDetails.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.resource_count_details = $root.google.ads.googleads.v11.errors.ResourceCountDetails.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ErrorDetails message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ErrorDetails} ErrorDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ErrorDetails.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ErrorDetails message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ErrorDetails.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.unpublished_error_code != null && m.hasOwnProperty("unpublished_error_code")) {
                                    if (!$util.isString(m.unpublished_error_code))
                                        return "unpublished_error_code: string expected";
                                }
                                if (m.policy_violation_details != null && m.hasOwnProperty("policy_violation_details")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.errors.PolicyViolationDetails.verify(m.policy_violation_details);
                                        if (e)
                                            return "policy_violation_details." + e;
                                    }
                                }
                                if (m.policy_finding_details != null && m.hasOwnProperty("policy_finding_details")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.errors.PolicyFindingDetails.verify(m.policy_finding_details);
                                        if (e)
                                            return "policy_finding_details." + e;
                                    }
                                }
                                if (m.quota_error_details != null && m.hasOwnProperty("quota_error_details")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.errors.QuotaErrorDetails.verify(m.quota_error_details);
                                        if (e)
                                            return "quota_error_details." + e;
                                    }
                                }
                                if (m.resource_count_details != null && m.hasOwnProperty("resource_count_details")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.errors.ResourceCountDetails.verify(m.resource_count_details);
                                        if (e)
                                            return "resource_count_details." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ErrorDetails message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ErrorDetails} ErrorDetails
                             */
                            ErrorDetails.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ErrorDetails)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.ErrorDetails();
                                if (d.unpublished_error_code != null) {
                                    m.unpublished_error_code = String(d.unpublished_error_code);
                                }
                                if (d.policy_violation_details != null) {
                                    if (typeof d.policy_violation_details !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.ErrorDetails.policy_violation_details: object expected");
                                    m.policy_violation_details = $root.google.ads.googleads.v11.errors.PolicyViolationDetails.fromObject(d.policy_violation_details);
                                }
                                if (d.policy_finding_details != null) {
                                    if (typeof d.policy_finding_details !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.ErrorDetails.policy_finding_details: object expected");
                                    m.policy_finding_details = $root.google.ads.googleads.v11.errors.PolicyFindingDetails.fromObject(d.policy_finding_details);
                                }
                                if (d.quota_error_details != null) {
                                    if (typeof d.quota_error_details !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.ErrorDetails.quota_error_details: object expected");
                                    m.quota_error_details = $root.google.ads.googleads.v11.errors.QuotaErrorDetails.fromObject(d.quota_error_details);
                                }
                                if (d.resource_count_details != null) {
                                    if (typeof d.resource_count_details !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.ErrorDetails.resource_count_details: object expected");
                                    m.resource_count_details = $root.google.ads.googleads.v11.errors.ResourceCountDetails.fromObject(d.resource_count_details);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ErrorDetails message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.ErrorDetails} m ErrorDetails
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ErrorDetails.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.unpublished_error_code = "";
                                    d.policy_violation_details = null;
                                    d.policy_finding_details = null;
                                    d.quota_error_details = null;
                                    d.resource_count_details = null;
                                }
                                if (m.unpublished_error_code != null && m.hasOwnProperty("unpublished_error_code")) {
                                    d.unpublished_error_code = m.unpublished_error_code;
                                }
                                if (m.policy_violation_details != null && m.hasOwnProperty("policy_violation_details")) {
                                    d.policy_violation_details = $root.google.ads.googleads.v11.errors.PolicyViolationDetails.toObject(m.policy_violation_details, o);
                                }
                                if (m.policy_finding_details != null && m.hasOwnProperty("policy_finding_details")) {
                                    d.policy_finding_details = $root.google.ads.googleads.v11.errors.PolicyFindingDetails.toObject(m.policy_finding_details, o);
                                }
                                if (m.quota_error_details != null && m.hasOwnProperty("quota_error_details")) {
                                    d.quota_error_details = $root.google.ads.googleads.v11.errors.QuotaErrorDetails.toObject(m.quota_error_details, o);
                                }
                                if (m.resource_count_details != null && m.hasOwnProperty("resource_count_details")) {
                                    d.resource_count_details = $root.google.ads.googleads.v11.errors.ResourceCountDetails.toObject(m.resource_count_details, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ErrorDetails to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ErrorDetails
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ErrorDetails.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ErrorDetails;
                        })();
    
                        errors.PolicyViolationDetails = (function() {
    
                            /**
                             * Properties of a PolicyViolationDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IPolicyViolationDetails
                             * @property {string|null} [external_policy_description] PolicyViolationDetails external_policy_description
                             * @property {google.ads.googleads.v11.common.IPolicyViolationKey|null} [key] PolicyViolationDetails key
                             * @property {string|null} [external_policy_name] PolicyViolationDetails external_policy_name
                             * @property {boolean|null} [is_exemptible] PolicyViolationDetails is_exemptible
                             */
    
                            /**
                             * Constructs a new PolicyViolationDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a PolicyViolationDetails.
                             * @implements IPolicyViolationDetails
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationDetails=} [p] Properties to set
                             */
                            function PolicyViolationDetails(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicyViolationDetails external_policy_description.
                             * @member {string} external_policy_description
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @instance
                             */
                            PolicyViolationDetails.prototype.external_policy_description = "";
    
                            /**
                             * PolicyViolationDetails key.
                             * @member {google.ads.googleads.v11.common.IPolicyViolationKey|null|undefined} key
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @instance
                             */
                            PolicyViolationDetails.prototype.key = null;
    
                            /**
                             * PolicyViolationDetails external_policy_name.
                             * @member {string} external_policy_name
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @instance
                             */
                            PolicyViolationDetails.prototype.external_policy_name = "";
    
                            /**
                             * PolicyViolationDetails is_exemptible.
                             * @member {boolean} is_exemptible
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @instance
                             */
                            PolicyViolationDetails.prototype.is_exemptible = false;
    
                            /**
                             * Creates a new PolicyViolationDetails instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationDetails=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationDetails} PolicyViolationDetails instance
                             */
                            PolicyViolationDetails.create = function create(properties) {
                                return new PolicyViolationDetails(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyViolationDetails message. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyViolationDetails.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationDetails} m PolicyViolationDetails message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyViolationDetails.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.external_policy_description != null && Object.hasOwnProperty.call(m, "external_policy_description"))
                                    w.uint32(18).string(m.external_policy_description);
                                if (m.key != null && Object.hasOwnProperty.call(m, "key"))
                                    $root.google.ads.googleads.v11.common.PolicyViolationKey.encode(m.key, w.uint32(34).fork()).ldelim();
                                if (m.external_policy_name != null && Object.hasOwnProperty.call(m, "external_policy_name"))
                                    w.uint32(42).string(m.external_policy_name);
                                if (m.is_exemptible != null && Object.hasOwnProperty.call(m, "is_exemptible"))
                                    w.uint32(48).bool(m.is_exemptible);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyViolationDetails message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyViolationDetails.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationDetails} message PolicyViolationDetails message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyViolationDetails.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyViolationDetails message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationDetails} PolicyViolationDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyViolationDetails.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.PolicyViolationDetails();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.external_policy_description = r.string();
                                        break;
                                    case 4:
                                        m.key = $root.google.ads.googleads.v11.common.PolicyViolationKey.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.external_policy_name = r.string();
                                        break;
                                    case 6:
                                        m.is_exemptible = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyViolationDetails message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationDetails} PolicyViolationDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyViolationDetails.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyViolationDetails message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyViolationDetails.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.external_policy_description != null && m.hasOwnProperty("external_policy_description")) {
                                    if (!$util.isString(m.external_policy_description))
                                        return "external_policy_description: string expected";
                                }
                                if (m.key != null && m.hasOwnProperty("key")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyViolationKey.verify(m.key);
                                        if (e)
                                            return "key." + e;
                                    }
                                }
                                if (m.external_policy_name != null && m.hasOwnProperty("external_policy_name")) {
                                    if (!$util.isString(m.external_policy_name))
                                        return "external_policy_name: string expected";
                                }
                                if (m.is_exemptible != null && m.hasOwnProperty("is_exemptible")) {
                                    if (typeof m.is_exemptible !== "boolean")
                                        return "is_exemptible: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicyViolationDetails message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationDetails} PolicyViolationDetails
                             */
                            PolicyViolationDetails.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.PolicyViolationDetails)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.PolicyViolationDetails();
                                if (d.external_policy_description != null) {
                                    m.external_policy_description = String(d.external_policy_description);
                                }
                                if (d.key != null) {
                                    if (typeof d.key !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.PolicyViolationDetails.key: object expected");
                                    m.key = $root.google.ads.googleads.v11.common.PolicyViolationKey.fromObject(d.key);
                                }
                                if (d.external_policy_name != null) {
                                    m.external_policy_name = String(d.external_policy_name);
                                }
                                if (d.is_exemptible != null) {
                                    m.is_exemptible = Boolean(d.is_exemptible);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicyViolationDetails message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.PolicyViolationDetails} m PolicyViolationDetails
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyViolationDetails.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.external_policy_description = "";
                                    d.key = null;
                                    d.external_policy_name = "";
                                    d.is_exemptible = false;
                                }
                                if (m.external_policy_description != null && m.hasOwnProperty("external_policy_description")) {
                                    d.external_policy_description = m.external_policy_description;
                                }
                                if (m.key != null && m.hasOwnProperty("key")) {
                                    d.key = $root.google.ads.googleads.v11.common.PolicyViolationKey.toObject(m.key, o);
                                }
                                if (m.external_policy_name != null && m.hasOwnProperty("external_policy_name")) {
                                    d.external_policy_name = m.external_policy_name;
                                }
                                if (m.is_exemptible != null && m.hasOwnProperty("is_exemptible")) {
                                    d.is_exemptible = m.is_exemptible;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicyViolationDetails to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationDetails
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyViolationDetails.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PolicyViolationDetails;
                        })();
    
                        errors.PolicyFindingDetails = (function() {
    
                            /**
                             * Properties of a PolicyFindingDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IPolicyFindingDetails
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>|null} [policy_topic_entries] PolicyFindingDetails policy_topic_entries
                             */
    
                            /**
                             * Constructs a new PolicyFindingDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a PolicyFindingDetails.
                             * @implements IPolicyFindingDetails
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingDetails=} [p] Properties to set
                             */
                            function PolicyFindingDetails(p) {
                                this.policy_topic_entries = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PolicyFindingDetails policy_topic_entries.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>} policy_topic_entries
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @instance
                             */
                            PolicyFindingDetails.prototype.policy_topic_entries = $util.emptyArray;
    
                            /**
                             * Creates a new PolicyFindingDetails instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingDetails=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingDetails} PolicyFindingDetails instance
                             */
                            PolicyFindingDetails.create = function create(properties) {
                                return new PolicyFindingDetails(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyFindingDetails message. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyFindingDetails.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingDetails} m PolicyFindingDetails message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyFindingDetails.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.policy_topic_entries != null && m.policy_topic_entries.length) {
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEntry.encode(m.policy_topic_entries[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyFindingDetails message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyFindingDetails.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingDetails} message PolicyFindingDetails message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyFindingDetails.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyFindingDetails message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingDetails} PolicyFindingDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyFindingDetails.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.PolicyFindingDetails();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.policy_topic_entries && m.policy_topic_entries.length))
                                            m.policy_topic_entries = [];
                                        m.policy_topic_entries.push($root.google.ads.googleads.v11.common.PolicyTopicEntry.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyFindingDetails message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingDetails} PolicyFindingDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyFindingDetails.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyFindingDetails message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyFindingDetails.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.policy_topic_entries != null && m.hasOwnProperty("policy_topic_entries")) {
                                    if (!Array.isArray(m.policy_topic_entries))
                                        return "policy_topic_entries: array expected";
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEntry.verify(m.policy_topic_entries[i]);
                                            if (e)
                                                return "policy_topic_entries." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PolicyFindingDetails message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingDetails} PolicyFindingDetails
                             */
                            PolicyFindingDetails.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.PolicyFindingDetails)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.PolicyFindingDetails();
                                if (d.policy_topic_entries) {
                                    if (!Array.isArray(d.policy_topic_entries))
                                        throw TypeError(".google.ads.googleads.v11.errors.PolicyFindingDetails.policy_topic_entries: array expected");
                                    m.policy_topic_entries = [];
                                    for (var i = 0; i < d.policy_topic_entries.length; ++i) {
                                        if (typeof d.policy_topic_entries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.errors.PolicyFindingDetails.policy_topic_entries: object expected");
                                        m.policy_topic_entries[i] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.fromObject(d.policy_topic_entries[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PolicyFindingDetails message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.PolicyFindingDetails} m PolicyFindingDetails
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyFindingDetails.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.policy_topic_entries = [];
                                }
                                if (m.policy_topic_entries && m.policy_topic_entries.length) {
                                    d.policy_topic_entries = [];
                                    for (var j = 0; j < m.policy_topic_entries.length; ++j) {
                                        d.policy_topic_entries[j] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.toObject(m.policy_topic_entries[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PolicyFindingDetails to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingDetails
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyFindingDetails.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PolicyFindingDetails;
                        })();
    
                        errors.QuotaErrorDetails = (function() {
    
                            /**
                             * Properties of a QuotaErrorDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IQuotaErrorDetails
                             * @property {google.ads.googleads.v11.errors.QuotaErrorDetails.QuotaRateScope|null} [rate_scope] QuotaErrorDetails rate_scope
                             * @property {string|null} [rate_name] QuotaErrorDetails rate_name
                             * @property {google.protobuf.IDuration|null} [retry_delay] QuotaErrorDetails retry_delay
                             */
    
                            /**
                             * Constructs a new QuotaErrorDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a QuotaErrorDetails.
                             * @implements IQuotaErrorDetails
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorDetails=} [p] Properties to set
                             */
                            function QuotaErrorDetails(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * QuotaErrorDetails rate_scope.
                             * @member {google.ads.googleads.v11.errors.QuotaErrorDetails.QuotaRateScope} rate_scope
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @instance
                             */
                            QuotaErrorDetails.prototype.rate_scope = 0;
    
                            /**
                             * QuotaErrorDetails rate_name.
                             * @member {string} rate_name
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @instance
                             */
                            QuotaErrorDetails.prototype.rate_name = "";
    
                            /**
                             * QuotaErrorDetails retry_delay.
                             * @member {google.protobuf.IDuration|null|undefined} retry_delay
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @instance
                             */
                            QuotaErrorDetails.prototype.retry_delay = null;
    
                            /**
                             * Creates a new QuotaErrorDetails instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorDetails=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorDetails} QuotaErrorDetails instance
                             */
                            QuotaErrorDetails.create = function create(properties) {
                                return new QuotaErrorDetails(properties);
                            };
    
                            /**
                             * Encodes the specified QuotaErrorDetails message. Does not implicitly {@link google.ads.googleads.v11.errors.QuotaErrorDetails.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorDetails} m QuotaErrorDetails message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QuotaErrorDetails.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.rate_scope != null && Object.hasOwnProperty.call(m, "rate_scope"))
                                    w.uint32(8).int32(m.rate_scope);
                                if (m.rate_name != null && Object.hasOwnProperty.call(m, "rate_name"))
                                    w.uint32(18).string(m.rate_name);
                                if (m.retry_delay != null && Object.hasOwnProperty.call(m, "retry_delay"))
                                    $root.google.protobuf.Duration.encode(m.retry_delay, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified QuotaErrorDetails message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.QuotaErrorDetails.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorDetails} message QuotaErrorDetails message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QuotaErrorDetails.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a QuotaErrorDetails message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorDetails} QuotaErrorDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QuotaErrorDetails.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.QuotaErrorDetails();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.rate_scope = r.int32();
                                        break;
                                    case 2:
                                        m.rate_name = r.string();
                                        break;
                                    case 3:
                                        m.retry_delay = $root.google.protobuf.Duration.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a QuotaErrorDetails message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorDetails} QuotaErrorDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QuotaErrorDetails.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a QuotaErrorDetails message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            QuotaErrorDetails.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.rate_scope != null && m.hasOwnProperty("rate_scope")) {
                                    switch (m.rate_scope) {
                                    default:
                                        return "rate_scope: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.rate_name != null && m.hasOwnProperty("rate_name")) {
                                    if (!$util.isString(m.rate_name))
                                        return "rate_name: string expected";
                                }
                                if (m.retry_delay != null && m.hasOwnProperty("retry_delay")) {
                                    {
                                        var e = $root.google.protobuf.Duration.verify(m.retry_delay);
                                        if (e)
                                            return "retry_delay." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a QuotaErrorDetails message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorDetails} QuotaErrorDetails
                             */
                            QuotaErrorDetails.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.QuotaErrorDetails)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.QuotaErrorDetails();
                                switch (d.rate_scope) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.rate_scope = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.rate_scope = 1;
                                    break;
                                case "ACCOUNT":
                                case 2:
                                    m.rate_scope = 2;
                                    break;
                                case "DEVELOPER":
                                case 3:
                                    m.rate_scope = 3;
                                    break;
                                }
                                if (d.rate_name != null) {
                                    m.rate_name = String(d.rate_name);
                                }
                                if (d.retry_delay != null) {
                                    if (typeof d.retry_delay !== "object")
                                        throw TypeError(".google.ads.googleads.v11.errors.QuotaErrorDetails.retry_delay: object expected");
                                    m.retry_delay = $root.google.protobuf.Duration.fromObject(d.retry_delay);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a QuotaErrorDetails message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.QuotaErrorDetails} m QuotaErrorDetails
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            QuotaErrorDetails.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.rate_scope = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.rate_name = "";
                                    d.retry_delay = null;
                                }
                                if (m.rate_scope != null && m.hasOwnProperty("rate_scope")) {
                                    d.rate_scope = o.enums === String ? $root.google.ads.googleads.v11.errors.QuotaErrorDetails.QuotaRateScope[m.rate_scope] : m.rate_scope;
                                }
                                if (m.rate_name != null && m.hasOwnProperty("rate_name")) {
                                    d.rate_name = m.rate_name;
                                }
                                if (m.retry_delay != null && m.hasOwnProperty("retry_delay")) {
                                    d.retry_delay = $root.google.protobuf.Duration.toObject(m.retry_delay, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this QuotaErrorDetails to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorDetails
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            QuotaErrorDetails.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * QuotaRateScope enum.
                             * @name google.ads.googleads.v11.errors.QuotaErrorDetails.QuotaRateScope
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ACCOUNT=2 ACCOUNT value
                             * @property {number} DEVELOPER=3 DEVELOPER value
                             */
                            QuotaErrorDetails.QuotaRateScope = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ACCOUNT"] = 2;
                                values[valuesById[3] = "DEVELOPER"] = 3;
                                return values;
                            })();
    
                            return QuotaErrorDetails;
                        })();
    
                        errors.ResourceCountDetails = (function() {
    
                            /**
                             * Properties of a ResourceCountDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IResourceCountDetails
                             * @property {string|null} [enclosing_id] ResourceCountDetails enclosing_id
                             * @property {string|null} [enclosing_resource] ResourceCountDetails enclosing_resource
                             * @property {number|null} [limit] ResourceCountDetails limit
                             * @property {google.ads.googleads.v11.enums.ResourceLimitTypeEnum.ResourceLimitType|null} [limit_type] ResourceCountDetails limit_type
                             * @property {number|null} [existing_count] ResourceCountDetails existing_count
                             */
    
                            /**
                             * Constructs a new ResourceCountDetails.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ResourceCountDetails.
                             * @implements IResourceCountDetails
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IResourceCountDetails=} [p] Properties to set
                             */
                            function ResourceCountDetails(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ResourceCountDetails enclosing_id.
                             * @member {string} enclosing_id
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @instance
                             */
                            ResourceCountDetails.prototype.enclosing_id = "";
    
                            /**
                             * ResourceCountDetails enclosing_resource.
                             * @member {string} enclosing_resource
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @instance
                             */
                            ResourceCountDetails.prototype.enclosing_resource = "";
    
                            /**
                             * ResourceCountDetails limit.
                             * @member {number} limit
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @instance
                             */
                            ResourceCountDetails.prototype.limit = 0;
    
                            /**
                             * ResourceCountDetails limit_type.
                             * @member {google.ads.googleads.v11.enums.ResourceLimitTypeEnum.ResourceLimitType} limit_type
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @instance
                             */
                            ResourceCountDetails.prototype.limit_type = 0;
    
                            /**
                             * ResourceCountDetails existing_count.
                             * @member {number} existing_count
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @instance
                             */
                            ResourceCountDetails.prototype.existing_count = 0;
    
                            /**
                             * Creates a new ResourceCountDetails instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceCountDetails=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ResourceCountDetails} ResourceCountDetails instance
                             */
                            ResourceCountDetails.create = function create(properties) {
                                return new ResourceCountDetails(properties);
                            };
    
                            /**
                             * Encodes the specified ResourceCountDetails message. Does not implicitly {@link google.ads.googleads.v11.errors.ResourceCountDetails.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceCountDetails} m ResourceCountDetails message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceCountDetails.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.enclosing_id != null && Object.hasOwnProperty.call(m, "enclosing_id"))
                                    w.uint32(10).string(m.enclosing_id);
                                if (m.limit != null && Object.hasOwnProperty.call(m, "limit"))
                                    w.uint32(16).int32(m.limit);
                                if (m.limit_type != null && Object.hasOwnProperty.call(m, "limit_type"))
                                    w.uint32(24).int32(m.limit_type);
                                if (m.existing_count != null && Object.hasOwnProperty.call(m, "existing_count"))
                                    w.uint32(32).int32(m.existing_count);
                                if (m.enclosing_resource != null && Object.hasOwnProperty.call(m, "enclosing_resource"))
                                    w.uint32(42).string(m.enclosing_resource);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResourceCountDetails message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ResourceCountDetails.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceCountDetails} message ResourceCountDetails message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceCountDetails.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResourceCountDetails message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ResourceCountDetails} ResourceCountDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceCountDetails.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ResourceCountDetails();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.enclosing_id = r.string();
                                        break;
                                    case 5:
                                        m.enclosing_resource = r.string();
                                        break;
                                    case 2:
                                        m.limit = r.int32();
                                        break;
                                    case 3:
                                        m.limit_type = r.int32();
                                        break;
                                    case 4:
                                        m.existing_count = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResourceCountDetails message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ResourceCountDetails} ResourceCountDetails
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceCountDetails.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResourceCountDetails message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResourceCountDetails.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.enclosing_id != null && m.hasOwnProperty("enclosing_id")) {
                                    if (!$util.isString(m.enclosing_id))
                                        return "enclosing_id: string expected";
                                }
                                if (m.enclosing_resource != null && m.hasOwnProperty("enclosing_resource")) {
                                    if (!$util.isString(m.enclosing_resource))
                                        return "enclosing_resource: string expected";
                                }
                                if (m.limit != null && m.hasOwnProperty("limit")) {
                                    if (!$util.isInteger(m.limit))
                                        return "limit: integer expected";
                                }
                                if (m.limit_type != null && m.hasOwnProperty("limit_type")) {
                                    switch (m.limit_type) {
                                    default:
                                        return "limit_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 105:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 52:
                                    case 12:
                                    case 13:
                                    case 107:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 108:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 40:
                                    case 41:
                                    case 44:
                                    case 45:
                                    case 47:
                                    case 48:
                                    case 153:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                    case 58:
                                    case 59:
                                    case 60:
                                    case 61:
                                    case 62:
                                    case 63:
                                    case 143:
                                    case 79:
                                    case 80:
                                    case 81:
                                    case 134:
                                    case 135:
                                    case 136:
                                    case 137:
                                    case 138:
                                    case 139:
                                    case 140:
                                    case 141:
                                    case 142:
                                    case 144:
                                    case 145:
                                    case 146:
                                    case 147:
                                    case 148:
                                    case 149:
                                    case 150:
                                    case 151:
                                    case 152:
                                    case 154:
                                    case 155:
                                    case 156:
                                    case 157:
                                    case 158:
                                    case 159:
                                    case 160:
                                    case 161:
                                    case 162:
                                    case 163:
                                    case 164:
                                    case 165:
                                    case 166:
                                    case 167:
                                    case 168:
                                    case 169:
                                    case 170:
                                    case 171:
                                    case 172:
                                    case 173:
                                    case 174:
                                    case 82:
                                    case 90:
                                    case 91:
                                    case 92:
                                    case 94:
                                    case 95:
                                    case 96:
                                    case 109:
                                    case 97:
                                    case 98:
                                    case 110:
                                    case 99:
                                    case 100:
                                    case 101:
                                    case 112:
                                    case 113:
                                    case 115:
                                    case 117:
                                    case 118:
                                    case 119:
                                    case 120:
                                    case 121:
                                    case 122:
                                    case 123:
                                    case 124:
                                    case 125:
                                    case 126:
                                    case 128:
                                    case 130:
                                    case 131:
                                    case 132:
                                        break;
                                    }
                                }
                                if (m.existing_count != null && m.hasOwnProperty("existing_count")) {
                                    if (!$util.isInteger(m.existing_count))
                                        return "existing_count: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ResourceCountDetails message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ResourceCountDetails} ResourceCountDetails
                             */
                            ResourceCountDetails.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ResourceCountDetails)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.errors.ResourceCountDetails();
                                if (d.enclosing_id != null) {
                                    m.enclosing_id = String(d.enclosing_id);
                                }
                                if (d.enclosing_resource != null) {
                                    m.enclosing_resource = String(d.enclosing_resource);
                                }
                                if (d.limit != null) {
                                    m.limit = d.limit | 0;
                                }
                                switch (d.limit_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.limit_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.limit_type = 1;
                                    break;
                                case "CAMPAIGNS_PER_CUSTOMER":
                                case 2:
                                    m.limit_type = 2;
                                    break;
                                case "BASE_CAMPAIGNS_PER_CUSTOMER":
                                case 3:
                                    m.limit_type = 3;
                                    break;
                                case "EXPERIMENT_CAMPAIGNS_PER_CUSTOMER":
                                case 105:
                                    m.limit_type = 105;
                                    break;
                                case "HOTEL_CAMPAIGNS_PER_CUSTOMER":
                                case 4:
                                    m.limit_type = 4;
                                    break;
                                case "SMART_SHOPPING_CAMPAIGNS_PER_CUSTOMER":
                                case 5:
                                    m.limit_type = 5;
                                    break;
                                case "AD_GROUPS_PER_CAMPAIGN":
                                case 6:
                                    m.limit_type = 6;
                                    break;
                                case "AD_GROUPS_PER_SHOPPING_CAMPAIGN":
                                case 8:
                                    m.limit_type = 8;
                                    break;
                                case "AD_GROUPS_PER_HOTEL_CAMPAIGN":
                                case 9:
                                    m.limit_type = 9;
                                    break;
                                case "REPORTING_AD_GROUPS_PER_LOCAL_CAMPAIGN":
                                case 10:
                                    m.limit_type = 10;
                                    break;
                                case "REPORTING_AD_GROUPS_PER_APP_CAMPAIGN":
                                case 11:
                                    m.limit_type = 11;
                                    break;
                                case "MANAGED_AD_GROUPS_PER_SMART_CAMPAIGN":
                                case 52:
                                    m.limit_type = 52;
                                    break;
                                case "AD_GROUP_CRITERIA_PER_CUSTOMER":
                                case 12:
                                    m.limit_type = 12;
                                    break;
                                case "BASE_AD_GROUP_CRITERIA_PER_CUSTOMER":
                                case 13:
                                    m.limit_type = 13;
                                    break;
                                case "EXPERIMENT_AD_GROUP_CRITERIA_PER_CUSTOMER":
                                case 107:
                                    m.limit_type = 107;
                                    break;
                                case "AD_GROUP_CRITERIA_PER_CAMPAIGN":
                                case 14:
                                    m.limit_type = 14;
                                    break;
                                case "CAMPAIGN_CRITERIA_PER_CUSTOMER":
                                case 15:
                                    m.limit_type = 15;
                                    break;
                                case "BASE_CAMPAIGN_CRITERIA_PER_CUSTOMER":
                                case 16:
                                    m.limit_type = 16;
                                    break;
                                case "EXPERIMENT_CAMPAIGN_CRITERIA_PER_CUSTOMER":
                                case 108:
                                    m.limit_type = 108;
                                    break;
                                case "WEBPAGE_CRITERIA_PER_CUSTOMER":
                                case 17:
                                    m.limit_type = 17;
                                    break;
                                case "BASE_WEBPAGE_CRITERIA_PER_CUSTOMER":
                                case 18:
                                    m.limit_type = 18;
                                    break;
                                case "EXPERIMENT_WEBPAGE_CRITERIA_PER_CUSTOMER":
                                case 19:
                                    m.limit_type = 19;
                                    break;
                                case "COMBINED_AUDIENCE_CRITERIA_PER_AD_GROUP":
                                case 20:
                                    m.limit_type = 20;
                                    break;
                                case "CUSTOMER_NEGATIVE_PLACEMENT_CRITERIA_PER_CUSTOMER":
                                case 21:
                                    m.limit_type = 21;
                                    break;
                                case "CUSTOMER_NEGATIVE_YOUTUBE_CHANNEL_CRITERIA_PER_CUSTOMER":
                                case 22:
                                    m.limit_type = 22;
                                    break;
                                case "CRITERIA_PER_AD_GROUP":
                                case 23:
                                    m.limit_type = 23;
                                    break;
                                case "LISTING_GROUPS_PER_AD_GROUP":
                                case 24:
                                    m.limit_type = 24;
                                    break;
                                case "EXPLICITLY_SHARED_BUDGETS_PER_CUSTOMER":
                                case 25:
                                    m.limit_type = 25;
                                    break;
                                case "IMPLICITLY_SHARED_BUDGETS_PER_CUSTOMER":
                                case 26:
                                    m.limit_type = 26;
                                    break;
                                case "COMBINED_AUDIENCE_CRITERIA_PER_CAMPAIGN":
                                case 27:
                                    m.limit_type = 27;
                                    break;
                                case "NEGATIVE_KEYWORDS_PER_CAMPAIGN":
                                case 28:
                                    m.limit_type = 28;
                                    break;
                                case "NEGATIVE_PLACEMENTS_PER_CAMPAIGN":
                                case 29:
                                    m.limit_type = 29;
                                    break;
                                case "GEO_TARGETS_PER_CAMPAIGN":
                                case 30:
                                    m.limit_type = 30;
                                    break;
                                case "NEGATIVE_IP_BLOCKS_PER_CAMPAIGN":
                                case 32:
                                    m.limit_type = 32;
                                    break;
                                case "PROXIMITIES_PER_CAMPAIGN":
                                case 33:
                                    m.limit_type = 33;
                                    break;
                                case "LISTING_SCOPES_PER_SHOPPING_CAMPAIGN":
                                case 34:
                                    m.limit_type = 34;
                                    break;
                                case "LISTING_SCOPES_PER_NON_SHOPPING_CAMPAIGN":
                                case 35:
                                    m.limit_type = 35;
                                    break;
                                case "NEGATIVE_KEYWORDS_PER_SHARED_SET":
                                case 36:
                                    m.limit_type = 36;
                                    break;
                                case "NEGATIVE_PLACEMENTS_PER_SHARED_SET":
                                case 37:
                                    m.limit_type = 37;
                                    break;
                                case "SHARED_SETS_PER_CUSTOMER_FOR_TYPE_DEFAULT":
                                case 40:
                                    m.limit_type = 40;
                                    break;
                                case "SHARED_SETS_PER_CUSTOMER_FOR_NEGATIVE_PLACEMENT_LIST_LOWER":
                                case 41:
                                    m.limit_type = 41;
                                    break;
                                case "HOTEL_ADVANCE_BOOKING_WINDOW_BID_MODIFIERS_PER_AD_GROUP":
                                case 44:
                                    m.limit_type = 44;
                                    break;
                                case "BIDDING_STRATEGIES_PER_CUSTOMER":
                                case 45:
                                    m.limit_type = 45;
                                    break;
                                case "BASIC_USER_LISTS_PER_CUSTOMER":
                                case 47:
                                    m.limit_type = 47;
                                    break;
                                case "LOGICAL_USER_LISTS_PER_CUSTOMER":
                                case 48:
                                    m.limit_type = 48;
                                    break;
                                case "RULE_BASED_USER_LISTS_PER_CUSTOMER":
                                case 153:
                                    m.limit_type = 153;
                                    break;
                                case "BASE_AD_GROUP_ADS_PER_CUSTOMER":
                                case 53:
                                    m.limit_type = 53;
                                    break;
                                case "EXPERIMENT_AD_GROUP_ADS_PER_CUSTOMER":
                                case 54:
                                    m.limit_type = 54;
                                    break;
                                case "AD_GROUP_ADS_PER_CAMPAIGN":
                                case 55:
                                    m.limit_type = 55;
                                    break;
                                case "TEXT_AND_OTHER_ADS_PER_AD_GROUP":
                                case 56:
                                    m.limit_type = 56;
                                    break;
                                case "IMAGE_ADS_PER_AD_GROUP":
                                case 57:
                                    m.limit_type = 57;
                                    break;
                                case "SHOPPING_SMART_ADS_PER_AD_GROUP":
                                case 58:
                                    m.limit_type = 58;
                                    break;
                                case "RESPONSIVE_SEARCH_ADS_PER_AD_GROUP":
                                case 59:
                                    m.limit_type = 59;
                                    break;
                                case "APP_ADS_PER_AD_GROUP":
                                case 60:
                                    m.limit_type = 60;
                                    break;
                                case "APP_ENGAGEMENT_ADS_PER_AD_GROUP":
                                case 61:
                                    m.limit_type = 61;
                                    break;
                                case "LOCAL_ADS_PER_AD_GROUP":
                                case 62:
                                    m.limit_type = 62;
                                    break;
                                case "VIDEO_ADS_PER_AD_GROUP":
                                case 63:
                                    m.limit_type = 63;
                                    break;
                                case "LEAD_FORM_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 143:
                                    m.limit_type = 143;
                                    break;
                                case "PROMOTION_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 79:
                                    m.limit_type = 79;
                                    break;
                                case "PROMOTION_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 80:
                                    m.limit_type = 80;
                                    break;
                                case "PROMOTION_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 81:
                                    m.limit_type = 81;
                                    break;
                                case "CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 134:
                                    m.limit_type = 134;
                                    break;
                                case "CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 135:
                                    m.limit_type = 135;
                                    break;
                                case "CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 136:
                                    m.limit_type = 136;
                                    break;
                                case "SITELINK_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 137:
                                    m.limit_type = 137;
                                    break;
                                case "SITELINK_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 138:
                                    m.limit_type = 138;
                                    break;
                                case "SITELINK_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 139:
                                    m.limit_type = 139;
                                    break;
                                case "STRUCTURED_SNIPPET_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 140:
                                    m.limit_type = 140;
                                    break;
                                case "STRUCTURED_SNIPPET_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 141:
                                    m.limit_type = 141;
                                    break;
                                case "STRUCTURED_SNIPPET_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 142:
                                    m.limit_type = 142;
                                    break;
                                case "MOBILE_APP_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 144:
                                    m.limit_type = 144;
                                    break;
                                case "MOBILE_APP_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 145:
                                    m.limit_type = 145;
                                    break;
                                case "MOBILE_APP_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 146:
                                    m.limit_type = 146;
                                    break;
                                case "HOTEL_CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 147:
                                    m.limit_type = 147;
                                    break;
                                case "HOTEL_CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 148:
                                    m.limit_type = 148;
                                    break;
                                case "HOTEL_CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 149:
                                    m.limit_type = 149;
                                    break;
                                case "CALL_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 150:
                                    m.limit_type = 150;
                                    break;
                                case "CALL_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 151:
                                    m.limit_type = 151;
                                    break;
                                case "CALL_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 152:
                                    m.limit_type = 152;
                                    break;
                                case "PRICE_CUSTOMER_ASSETS_PER_CUSTOMER":
                                case 154:
                                    m.limit_type = 154;
                                    break;
                                case "PRICE_CAMPAIGN_ASSETS_PER_CAMPAIGN":
                                case 155:
                                    m.limit_type = 155;
                                    break;
                                case "PRICE_AD_GROUP_ASSETS_PER_AD_GROUP":
                                case 156:
                                    m.limit_type = 156;
                                    break;
                                case "PAGE_FEED_ASSET_SETS_PER_CUSTOMER":
                                case 157:
                                    m.limit_type = 157;
                                    break;
                                case "DYNAMIC_EDUCATION_FEED_ASSET_SETS_PER_CUSTOMER":
                                case 158:
                                    m.limit_type = 158;
                                    break;
                                case "ASSETS_PER_PAGE_FEED_ASSET_SET":
                                case 159:
                                    m.limit_type = 159;
                                    break;
                                case "ASSETS_PER_DYNAMIC_EDUCATION_FEED_ASSET_SET":
                                case 160:
                                    m.limit_type = 160;
                                    break;
                                case "DYNAMIC_REAL_ESTATE_ASSET_SETS_PER_CUSTOMER":
                                case 161:
                                    m.limit_type = 161;
                                    break;
                                case "ASSETS_PER_DYNAMIC_REAL_ESTATE_ASSET_SET":
                                case 162:
                                    m.limit_type = 162;
                                    break;
                                case "DYNAMIC_CUSTOM_ASSET_SETS_PER_CUSTOMER":
                                case 163:
                                    m.limit_type = 163;
                                    break;
                                case "ASSETS_PER_DYNAMIC_CUSTOM_ASSET_SET":
                                case 164:
                                    m.limit_type = 164;
                                    break;
                                case "DYNAMIC_HOTELS_AND_RENTALS_ASSET_SETS_PER_CUSTOMER":
                                case 165:
                                    m.limit_type = 165;
                                    break;
                                case "ASSETS_PER_DYNAMIC_HOTELS_AND_RENTALS_ASSET_SET":
                                case 166:
                                    m.limit_type = 166;
                                    break;
                                case "DYNAMIC_LOCAL_ASSET_SETS_PER_CUSTOMER":
                                case 167:
                                    m.limit_type = 167;
                                    break;
                                case "ASSETS_PER_DYNAMIC_LOCAL_ASSET_SET":
                                case 168:
                                    m.limit_type = 168;
                                    break;
                                case "DYNAMIC_FLIGHTS_ASSET_SETS_PER_CUSTOMER":
                                case 169:
                                    m.limit_type = 169;
                                    break;
                                case "ASSETS_PER_DYNAMIC_FLIGHTS_ASSET_SET":
                                case 170:
                                    m.limit_type = 170;
                                    break;
                                case "DYNAMIC_TRAVEL_ASSET_SETS_PER_CUSTOMER":
                                case 171:
                                    m.limit_type = 171;
                                    break;
                                case "ASSETS_PER_DYNAMIC_TRAVEL_ASSET_SET":
                                case 172:
                                    m.limit_type = 172;
                                    break;
                                case "DYNAMIC_JOBS_ASSET_SETS_PER_CUSTOMER":
                                case 173:
                                    m.limit_type = 173;
                                    break;
                                case "ASSETS_PER_DYNAMIC_JOBS_ASSET_SET":
                                case 174:
                                    m.limit_type = 174;
                                    break;
                                case "VERSIONS_PER_AD":
                                case 82:
                                    m.limit_type = 82;
                                    break;
                                case "USER_FEEDS_PER_CUSTOMER":
                                case 90:
                                    m.limit_type = 90;
                                    break;
                                case "SYSTEM_FEEDS_PER_CUSTOMER":
                                case 91:
                                    m.limit_type = 91;
                                    break;
                                case "FEED_ATTRIBUTES_PER_FEED":
                                case 92:
                                    m.limit_type = 92;
                                    break;
                                case "FEED_ITEMS_PER_CUSTOMER":
                                case 94:
                                    m.limit_type = 94;
                                    break;
                                case "CAMPAIGN_FEEDS_PER_CUSTOMER":
                                case 95:
                                    m.limit_type = 95;
                                    break;
                                case "BASE_CAMPAIGN_FEEDS_PER_CUSTOMER":
                                case 96:
                                    m.limit_type = 96;
                                    break;
                                case "EXPERIMENT_CAMPAIGN_FEEDS_PER_CUSTOMER":
                                case 109:
                                    m.limit_type = 109;
                                    break;
                                case "AD_GROUP_FEEDS_PER_CUSTOMER":
                                case 97:
                                    m.limit_type = 97;
                                    break;
                                case "BASE_AD_GROUP_FEEDS_PER_CUSTOMER":
                                case 98:
                                    m.limit_type = 98;
                                    break;
                                case "EXPERIMENT_AD_GROUP_FEEDS_PER_CUSTOMER":
                                case 110:
                                    m.limit_type = 110;
                                    break;
                                case "AD_GROUP_FEEDS_PER_CAMPAIGN":
                                case 99:
                                    m.limit_type = 99;
                                    break;
                                case "FEED_ITEM_SETS_PER_CUSTOMER":
                                case 100:
                                    m.limit_type = 100;
                                    break;
                                case "FEED_ITEMS_PER_FEED_ITEM_SET":
                                case 101:
                                    m.limit_type = 101;
                                    break;
                                case "CAMPAIGN_EXPERIMENTS_PER_CUSTOMER":
                                case 112:
                                    m.limit_type = 112;
                                    break;
                                case "EXPERIMENT_ARMS_PER_VIDEO_EXPERIMENT":
                                case 113:
                                    m.limit_type = 113;
                                    break;
                                case "OWNED_LABELS_PER_CUSTOMER":
                                case 115:
                                    m.limit_type = 115;
                                    break;
                                case "LABELS_PER_CAMPAIGN":
                                case 117:
                                    m.limit_type = 117;
                                    break;
                                case "LABELS_PER_AD_GROUP":
                                case 118:
                                    m.limit_type = 118;
                                    break;
                                case "LABELS_PER_AD_GROUP_AD":
                                case 119:
                                    m.limit_type = 119;
                                    break;
                                case "LABELS_PER_AD_GROUP_CRITERION":
                                case 120:
                                    m.limit_type = 120;
                                    break;
                                case "TARGET_CUSTOMERS_PER_LABEL":
                                case 121:
                                    m.limit_type = 121;
                                    break;
                                case "KEYWORD_PLANS_PER_USER_PER_CUSTOMER":
                                case 122:
                                    m.limit_type = 122;
                                    break;
                                case "KEYWORD_PLAN_AD_GROUP_KEYWORDS_PER_KEYWORD_PLAN":
                                case 123:
                                    m.limit_type = 123;
                                    break;
                                case "KEYWORD_PLAN_AD_GROUPS_PER_KEYWORD_PLAN":
                                case 124:
                                    m.limit_type = 124;
                                    break;
                                case "KEYWORD_PLAN_NEGATIVE_KEYWORDS_PER_KEYWORD_PLAN":
                                case 125:
                                    m.limit_type = 125;
                                    break;
                                case "KEYWORD_PLAN_CAMPAIGNS_PER_KEYWORD_PLAN":
                                case 126:
                                    m.limit_type = 126;
                                    break;
                                case "CONVERSION_ACTIONS_PER_CUSTOMER":
                                case 128:
                                    m.limit_type = 128;
                                    break;
                                case "BATCH_JOB_OPERATIONS_PER_JOB":
                                case 130:
                                    m.limit_type = 130;
                                    break;
                                case "BATCH_JOBS_PER_CUSTOMER":
                                case 131:
                                    m.limit_type = 131;
                                    break;
                                case "HOTEL_CHECK_IN_DATE_RANGE_BID_MODIFIERS_PER_AD_GROUP":
                                case 132:
                                    m.limit_type = 132;
                                    break;
                                }
                                if (d.existing_count != null) {
                                    m.existing_count = d.existing_count | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ResourceCountDetails message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @static
                             * @param {google.ads.googleads.v11.errors.ResourceCountDetails} m ResourceCountDetails
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResourceCountDetails.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.enclosing_id = "";
                                    d.limit = 0;
                                    d.limit_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.existing_count = 0;
                                    d.enclosing_resource = "";
                                }
                                if (m.enclosing_id != null && m.hasOwnProperty("enclosing_id")) {
                                    d.enclosing_id = m.enclosing_id;
                                }
                                if (m.limit != null && m.hasOwnProperty("limit")) {
                                    d.limit = m.limit;
                                }
                                if (m.limit_type != null && m.hasOwnProperty("limit_type")) {
                                    d.limit_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResourceLimitTypeEnum.ResourceLimitType[m.limit_type] : m.limit_type;
                                }
                                if (m.existing_count != null && m.hasOwnProperty("existing_count")) {
                                    d.existing_count = m.existing_count;
                                }
                                if (m.enclosing_resource != null && m.hasOwnProperty("enclosing_resource")) {
                                    d.enclosing_resource = m.enclosing_resource;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ResourceCountDetails to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ResourceCountDetails
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResourceCountDetails.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ResourceCountDetails;
                        })();
    
                        errors.ExperimentArmErrorEnum = (function() {
    
                            /**
                             * Properties of an ExperimentArmErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IExperimentArmErrorEnum
                             */
    
                            /**
                             * Constructs a new ExperimentArmErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ExperimentArmErrorEnum.
                             * @implements IExperimentArmErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IExperimentArmErrorEnum=} [p] Properties to set
                             */
                            function ExperimentArmErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExperimentArmErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExperimentArmErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ExperimentArmErrorEnum} ExperimentArmErrorEnum instance
                             */
                            ExperimentArmErrorEnum.create = function create(properties) {
                                return new ExperimentArmErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentArmErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ExperimentArmErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExperimentArmErrorEnum} m ExperimentArmErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentArmErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentArmErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ExperimentArmErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExperimentArmErrorEnum} message ExperimentArmErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentArmErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentArmErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ExperimentArmErrorEnum} ExperimentArmErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentArmErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ExperimentArmErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentArmErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ExperimentArmErrorEnum} ExperimentArmErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentArmErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentArmErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentArmErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentArmErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ExperimentArmErrorEnum} ExperimentArmErrorEnum
                             */
                            ExperimentArmErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ExperimentArmErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ExperimentArmErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExperimentArmErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ExperimentArmErrorEnum} m ExperimentArmErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentArmErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExperimentArmErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ExperimentArmErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentArmErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExperimentArmError enum.
                             * @name google.ads.googleads.v11.errors.ExperimentArmErrorEnum.ExperimentArmError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EXPERIMENT_ARM_COUNT_LIMIT_EXCEEDED=2 EXPERIMENT_ARM_COUNT_LIMIT_EXCEEDED value
                             * @property {number} INVALID_CAMPAIGN_STATUS=3 INVALID_CAMPAIGN_STATUS value
                             * @property {number} DUPLICATE_EXPERIMENT_ARM_NAME=4 DUPLICATE_EXPERIMENT_ARM_NAME value
                             * @property {number} CANNOT_SET_TREATMENT_ARM_CAMPAIGN=5 CANNOT_SET_TREATMENT_ARM_CAMPAIGN value
                             * @property {number} CANNOT_MODIFY_CAMPAIGN_IDS=6 CANNOT_MODIFY_CAMPAIGN_IDS value
                             * @property {number} CANNOT_MODIFY_CAMPAIGN_WITHOUT_SUFFIX_SET=7 CANNOT_MODIFY_CAMPAIGN_WITHOUT_SUFFIX_SET value
                             * @property {number} CANNOT_MUTATE_TRAFFIC_SPLIT_AFTER_START=8 CANNOT_MUTATE_TRAFFIC_SPLIT_AFTER_START value
                             * @property {number} CANNOT_ADD_CAMPAIGN_WITH_SHARED_BUDGET=9 CANNOT_ADD_CAMPAIGN_WITH_SHARED_BUDGET value
                             * @property {number} CANNOT_ADD_CAMPAIGN_WITH_CUSTOM_BUDGET=10 CANNOT_ADD_CAMPAIGN_WITH_CUSTOM_BUDGET value
                             * @property {number} CANNOT_ADD_CAMPAIGNS_WITH_DYNAMIC_ASSETS_ENABLED=11 CANNOT_ADD_CAMPAIGNS_WITH_DYNAMIC_ASSETS_ENABLED value
                             * @property {number} UNSUPPORTED_CAMPAIGN_ADVERTISING_CHANNEL_SUB_TYPE=12 UNSUPPORTED_CAMPAIGN_ADVERTISING_CHANNEL_SUB_TYPE value
                             * @property {number} CANNOT_ADD_BASE_CAMPAIGN_WITH_DATE_RANGE=13 CANNOT_ADD_BASE_CAMPAIGN_WITH_DATE_RANGE value
                             * @property {number} BIDDING_STRATEGY_NOT_SUPPORTED_IN_EXPERIMENTS=14 BIDDING_STRATEGY_NOT_SUPPORTED_IN_EXPERIMENTS value
                             * @property {number} TRAFFIC_SPLIT_NOT_SUPPORTED_FOR_CHANNEL_TYPE=15 TRAFFIC_SPLIT_NOT_SUPPORTED_FOR_CHANNEL_TYPE value
                             */
                            ExperimentArmErrorEnum.ExperimentArmError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EXPERIMENT_ARM_COUNT_LIMIT_EXCEEDED"] = 2;
                                values[valuesById[3] = "INVALID_CAMPAIGN_STATUS"] = 3;
                                values[valuesById[4] = "DUPLICATE_EXPERIMENT_ARM_NAME"] = 4;
                                values[valuesById[5] = "CANNOT_SET_TREATMENT_ARM_CAMPAIGN"] = 5;
                                values[valuesById[6] = "CANNOT_MODIFY_CAMPAIGN_IDS"] = 6;
                                values[valuesById[7] = "CANNOT_MODIFY_CAMPAIGN_WITHOUT_SUFFIX_SET"] = 7;
                                values[valuesById[8] = "CANNOT_MUTATE_TRAFFIC_SPLIT_AFTER_START"] = 8;
                                values[valuesById[9] = "CANNOT_ADD_CAMPAIGN_WITH_SHARED_BUDGET"] = 9;
                                values[valuesById[10] = "CANNOT_ADD_CAMPAIGN_WITH_CUSTOM_BUDGET"] = 10;
                                values[valuesById[11] = "CANNOT_ADD_CAMPAIGNS_WITH_DYNAMIC_ASSETS_ENABLED"] = 11;
                                values[valuesById[12] = "UNSUPPORTED_CAMPAIGN_ADVERTISING_CHANNEL_SUB_TYPE"] = 12;
                                values[valuesById[13] = "CANNOT_ADD_BASE_CAMPAIGN_WITH_DATE_RANGE"] = 13;
                                values[valuesById[14] = "BIDDING_STRATEGY_NOT_SUPPORTED_IN_EXPERIMENTS"] = 14;
                                values[valuesById[15] = "TRAFFIC_SPLIT_NOT_SUPPORTED_FOR_CHANNEL_TYPE"] = 15;
                                return values;
                            })();
    
                            return ExperimentArmErrorEnum;
                        })();
    
                        errors.ExperimentErrorEnum = (function() {
    
                            /**
                             * Properties of an ExperimentErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IExperimentErrorEnum
                             */
    
                            /**
                             * Constructs a new ExperimentErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ExperimentErrorEnum.
                             * @implements IExperimentErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IExperimentErrorEnum=} [p] Properties to set
                             */
                            function ExperimentErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExperimentErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExperimentErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ExperimentErrorEnum} ExperimentErrorEnum instance
                             */
                            ExperimentErrorEnum.create = function create(properties) {
                                return new ExperimentErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ExperimentErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExperimentErrorEnum} m ExperimentErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ExperimentErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExperimentErrorEnum} message ExperimentErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ExperimentErrorEnum} ExperimentErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ExperimentErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ExperimentErrorEnum} ExperimentErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ExperimentErrorEnum} ExperimentErrorEnum
                             */
                            ExperimentErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ExperimentErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ExperimentErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExperimentErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ExperimentErrorEnum} m ExperimentErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExperimentErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ExperimentErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExperimentError enum.
                             * @name google.ads.googleads.v11.errors.ExperimentErrorEnum.ExperimentError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_SET_START_DATE_IN_PAST=2 CANNOT_SET_START_DATE_IN_PAST value
                             * @property {number} END_DATE_BEFORE_START_DATE=3 END_DATE_BEFORE_START_DATE value
                             * @property {number} START_DATE_TOO_FAR_IN_FUTURE=4 START_DATE_TOO_FAR_IN_FUTURE value
                             * @property {number} DUPLICATE_EXPERIMENT_NAME=5 DUPLICATE_EXPERIMENT_NAME value
                             * @property {number} CANNOT_MODIFY_REMOVED_EXPERIMENT=6 CANNOT_MODIFY_REMOVED_EXPERIMENT value
                             * @property {number} START_DATE_ALREADY_PASSED=7 START_DATE_ALREADY_PASSED value
                             * @property {number} CANNOT_SET_END_DATE_IN_PAST=8 CANNOT_SET_END_DATE_IN_PAST value
                             * @property {number} CANNOT_SET_STATUS_TO_REMOVED=9 CANNOT_SET_STATUS_TO_REMOVED value
                             * @property {number} CANNOT_MODIFY_PAST_END_DATE=10 CANNOT_MODIFY_PAST_END_DATE value
                             * @property {number} INVALID_STATUS=11 INVALID_STATUS value
                             * @property {number} INVALID_CAMPAIGN_CHANNEL_TYPE=12 INVALID_CAMPAIGN_CHANNEL_TYPE value
                             * @property {number} OVERLAPPING_MEMBERS_AND_DATE_RANGE=13 OVERLAPPING_MEMBERS_AND_DATE_RANGE value
                             * @property {number} INVALID_TRIAL_ARM_TRAFFIC_SPLIT=14 INVALID_TRIAL_ARM_TRAFFIC_SPLIT value
                             * @property {number} TRAFFIC_SPLIT_OVERLAPPING=15 TRAFFIC_SPLIT_OVERLAPPING value
                             * @property {number} SUM_TRIAL_ARM_TRAFFIC_UNEQUALS_TO_TRIAL_TRAFFIC_SPLIT_DENOMINATOR=16 SUM_TRIAL_ARM_TRAFFIC_UNEQUALS_TO_TRIAL_TRAFFIC_SPLIT_DENOMINATOR value
                             * @property {number} CANNOT_MODIFY_TRAFFIC_SPLIT_AFTER_START=17 CANNOT_MODIFY_TRAFFIC_SPLIT_AFTER_START value
                             * @property {number} EXPERIMENT_NOT_FOUND=18 EXPERIMENT_NOT_FOUND value
                             * @property {number} EXPERIMENT_NOT_YET_STARTED=19 EXPERIMENT_NOT_YET_STARTED value
                             * @property {number} CANNOT_HAVE_MULTIPLE_CONTROL_ARMS=20 CANNOT_HAVE_MULTIPLE_CONTROL_ARMS value
                             * @property {number} IN_DESIGN_CAMPAIGNS_NOT_SET=21 IN_DESIGN_CAMPAIGNS_NOT_SET value
                             * @property {number} CANNOT_SET_STATUS_TO_GRADUATED=22 CANNOT_SET_STATUS_TO_GRADUATED value
                             * @property {number} CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_SHARED_BUDGET=23 CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_SHARED_BUDGET value
                             * @property {number} CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_CUSTOM_BUDGET=24 CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_CUSTOM_BUDGET value
                             * @property {number} STATUS_TRANSITION_INVALID=25 STATUS_TRANSITION_INVALID value
                             */
                            ExperimentErrorEnum.ExperimentError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_SET_START_DATE_IN_PAST"] = 2;
                                values[valuesById[3] = "END_DATE_BEFORE_START_DATE"] = 3;
                                values[valuesById[4] = "START_DATE_TOO_FAR_IN_FUTURE"] = 4;
                                values[valuesById[5] = "DUPLICATE_EXPERIMENT_NAME"] = 5;
                                values[valuesById[6] = "CANNOT_MODIFY_REMOVED_EXPERIMENT"] = 6;
                                values[valuesById[7] = "START_DATE_ALREADY_PASSED"] = 7;
                                values[valuesById[8] = "CANNOT_SET_END_DATE_IN_PAST"] = 8;
                                values[valuesById[9] = "CANNOT_SET_STATUS_TO_REMOVED"] = 9;
                                values[valuesById[10] = "CANNOT_MODIFY_PAST_END_DATE"] = 10;
                                values[valuesById[11] = "INVALID_STATUS"] = 11;
                                values[valuesById[12] = "INVALID_CAMPAIGN_CHANNEL_TYPE"] = 12;
                                values[valuesById[13] = "OVERLAPPING_MEMBERS_AND_DATE_RANGE"] = 13;
                                values[valuesById[14] = "INVALID_TRIAL_ARM_TRAFFIC_SPLIT"] = 14;
                                values[valuesById[15] = "TRAFFIC_SPLIT_OVERLAPPING"] = 15;
                                values[valuesById[16] = "SUM_TRIAL_ARM_TRAFFIC_UNEQUALS_TO_TRIAL_TRAFFIC_SPLIT_DENOMINATOR"] = 16;
                                values[valuesById[17] = "CANNOT_MODIFY_TRAFFIC_SPLIT_AFTER_START"] = 17;
                                values[valuesById[18] = "EXPERIMENT_NOT_FOUND"] = 18;
                                values[valuesById[19] = "EXPERIMENT_NOT_YET_STARTED"] = 19;
                                values[valuesById[20] = "CANNOT_HAVE_MULTIPLE_CONTROL_ARMS"] = 20;
                                values[valuesById[21] = "IN_DESIGN_CAMPAIGNS_NOT_SET"] = 21;
                                values[valuesById[22] = "CANNOT_SET_STATUS_TO_GRADUATED"] = 22;
                                values[valuesById[23] = "CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_SHARED_BUDGET"] = 23;
                                values[valuesById[24] = "CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_CUSTOM_BUDGET"] = 24;
                                values[valuesById[25] = "STATUS_TRANSITION_INVALID"] = 25;
                                return values;
                            })();
    
                            return ExperimentErrorEnum;
                        })();
    
                        errors.ExtensionFeedItemErrorEnum = (function() {
    
                            /**
                             * Properties of an ExtensionFeedItemErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IExtensionFeedItemErrorEnum
                             */
    
                            /**
                             * Constructs a new ExtensionFeedItemErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ExtensionFeedItemErrorEnum.
                             * @implements IExtensionFeedItemErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IExtensionFeedItemErrorEnum=} [p] Properties to set
                             */
                            function ExtensionFeedItemErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExtensionFeedItemErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExtensionFeedItemErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum} ExtensionFeedItemErrorEnum instance
                             */
                            ExtensionFeedItemErrorEnum.create = function create(properties) {
                                return new ExtensionFeedItemErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExtensionFeedItemErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExtensionFeedItemErrorEnum} m ExtensionFeedItemErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionFeedItemErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExtensionFeedItemErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExtensionFeedItemErrorEnum} message ExtensionFeedItemErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionFeedItemErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExtensionFeedItemErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum} ExtensionFeedItemErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionFeedItemErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExtensionFeedItemErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum} ExtensionFeedItemErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionFeedItemErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExtensionFeedItemErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExtensionFeedItemErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExtensionFeedItemErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum} ExtensionFeedItemErrorEnum
                             */
                            ExtensionFeedItemErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExtensionFeedItemErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum} m ExtensionFeedItemErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExtensionFeedItemErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExtensionFeedItemErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExtensionFeedItemErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExtensionFeedItemError enum.
                             * @name google.ads.googleads.v11.errors.ExtensionFeedItemErrorEnum.ExtensionFeedItemError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} VALUE_OUT_OF_RANGE=2 VALUE_OUT_OF_RANGE value
                             * @property {number} URL_LIST_TOO_LONG=3 URL_LIST_TOO_LONG value
                             * @property {number} CANNOT_HAVE_RESTRICTION_ON_EMPTY_GEO_TARGETING=4 CANNOT_HAVE_RESTRICTION_ON_EMPTY_GEO_TARGETING value
                             * @property {number} CANNOT_SET_WITH_FINAL_URLS=5 CANNOT_SET_WITH_FINAL_URLS value
                             * @property {number} CANNOT_SET_WITHOUT_FINAL_URLS=6 CANNOT_SET_WITHOUT_FINAL_URLS value
                             * @property {number} INVALID_PHONE_NUMBER=7 INVALID_PHONE_NUMBER value
                             * @property {number} PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY=8 PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED=9 CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED value
                             * @property {number} PREMIUM_RATE_NUMBER_NOT_ALLOWED=10 PREMIUM_RATE_NUMBER_NOT_ALLOWED value
                             * @property {number} DISALLOWED_NUMBER_TYPE=11 DISALLOWED_NUMBER_TYPE value
                             * @property {number} INVALID_DOMESTIC_PHONE_NUMBER_FORMAT=12 INVALID_DOMESTIC_PHONE_NUMBER_FORMAT value
                             * @property {number} VANITY_PHONE_NUMBER_NOT_ALLOWED=13 VANITY_PHONE_NUMBER_NOT_ALLOWED value
                             * @property {number} INVALID_CALL_CONVERSION_ACTION=14 INVALID_CALL_CONVERSION_ACTION value
                             * @property {number} CUSTOMER_NOT_ON_ALLOWLIST_FOR_CALLTRACKING=47 CUSTOMER_NOT_ON_ALLOWLIST_FOR_CALLTRACKING value
                             * @property {number} CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY=16 CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED=17 CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED value
                             * @property {number} INVALID_APP_ID=18 INVALID_APP_ID value
                             * @property {number} QUOTES_IN_REVIEW_EXTENSION_SNIPPET=19 QUOTES_IN_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} HYPHENS_IN_REVIEW_EXTENSION_SNIPPET=20 HYPHENS_IN_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} REVIEW_EXTENSION_SOURCE_INELIGIBLE=21 REVIEW_EXTENSION_SOURCE_INELIGIBLE value
                             * @property {number} SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT=22 SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT value
                             * @property {number} INCONSISTENT_CURRENCY_CODES=23 INCONSISTENT_CURRENCY_CODES value
                             * @property {number} PRICE_EXTENSION_HAS_DUPLICATED_HEADERS=24 PRICE_EXTENSION_HAS_DUPLICATED_HEADERS value
                             * @property {number} PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION=25 PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION value
                             * @property {number} PRICE_EXTENSION_HAS_TOO_FEW_ITEMS=26 PRICE_EXTENSION_HAS_TOO_FEW_ITEMS value
                             * @property {number} PRICE_EXTENSION_HAS_TOO_MANY_ITEMS=27 PRICE_EXTENSION_HAS_TOO_MANY_ITEMS value
                             * @property {number} UNSUPPORTED_VALUE=28 UNSUPPORTED_VALUE value
                             * @property {number} UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE=29 UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE value
                             * @property {number} INVALID_DEVICE_PREFERENCE=30 INVALID_DEVICE_PREFERENCE value
                             * @property {number} INVALID_SCHEDULE_END=31 INVALID_SCHEDULE_END value
                             * @property {number} DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE=32 DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE value
                             * @property {number} INVALID_SNIPPETS_HEADER=33 INVALID_SNIPPETS_HEADER value
                             * @property {number} CANNOT_OPERATE_ON_REMOVED_FEED_ITEM=34 CANNOT_OPERATE_ON_REMOVED_FEED_ITEM value
                             * @property {number} PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY=35 PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY value
                             * @property {number} CONFLICTING_CALL_CONVERSION_SETTINGS=36 CONFLICTING_CALL_CONVERSION_SETTINGS value
                             * @property {number} EXTENSION_TYPE_MISMATCH=37 EXTENSION_TYPE_MISMATCH value
                             * @property {number} EXTENSION_SUBTYPE_REQUIRED=38 EXTENSION_SUBTYPE_REQUIRED value
                             * @property {number} EXTENSION_TYPE_UNSUPPORTED=39 EXTENSION_TYPE_UNSUPPORTED value
                             * @property {number} CANNOT_OPERATE_ON_FEED_WITH_MULTIPLE_MAPPINGS=40 CANNOT_OPERATE_ON_FEED_WITH_MULTIPLE_MAPPINGS value
                             * @property {number} CANNOT_OPERATE_ON_FEED_WITH_KEY_ATTRIBUTES=41 CANNOT_OPERATE_ON_FEED_WITH_KEY_ATTRIBUTES value
                             * @property {number} INVALID_PRICE_FORMAT=42 INVALID_PRICE_FORMAT value
                             * @property {number} PROMOTION_INVALID_TIME=43 PROMOTION_INVALID_TIME value
                             * @property {number} TOO_MANY_DECIMAL_PLACES_SPECIFIED=44 TOO_MANY_DECIMAL_PLACES_SPECIFIED value
                             * @property {number} CONCRETE_EXTENSION_TYPE_REQUIRED=45 CONCRETE_EXTENSION_TYPE_REQUIRED value
                             * @property {number} SCHEDULE_END_NOT_AFTER_START=46 SCHEDULE_END_NOT_AFTER_START value
                             */
                            ExtensionFeedItemErrorEnum.ExtensionFeedItemError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "VALUE_OUT_OF_RANGE"] = 2;
                                values[valuesById[3] = "URL_LIST_TOO_LONG"] = 3;
                                values[valuesById[4] = "CANNOT_HAVE_RESTRICTION_ON_EMPTY_GEO_TARGETING"] = 4;
                                values[valuesById[5] = "CANNOT_SET_WITH_FINAL_URLS"] = 5;
                                values[valuesById[6] = "CANNOT_SET_WITHOUT_FINAL_URLS"] = 6;
                                values[valuesById[7] = "INVALID_PHONE_NUMBER"] = 7;
                                values[valuesById[8] = "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY"] = 8;
                                values[valuesById[9] = "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED"] = 9;
                                values[valuesById[10] = "PREMIUM_RATE_NUMBER_NOT_ALLOWED"] = 10;
                                values[valuesById[11] = "DISALLOWED_NUMBER_TYPE"] = 11;
                                values[valuesById[12] = "INVALID_DOMESTIC_PHONE_NUMBER_FORMAT"] = 12;
                                values[valuesById[13] = "VANITY_PHONE_NUMBER_NOT_ALLOWED"] = 13;
                                values[valuesById[14] = "INVALID_CALL_CONVERSION_ACTION"] = 14;
                                values[valuesById[47] = "CUSTOMER_NOT_ON_ALLOWLIST_FOR_CALLTRACKING"] = 47;
                                values[valuesById[16] = "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY"] = 16;
                                values[valuesById[17] = "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED"] = 17;
                                values[valuesById[18] = "INVALID_APP_ID"] = 18;
                                values[valuesById[19] = "QUOTES_IN_REVIEW_EXTENSION_SNIPPET"] = 19;
                                values[valuesById[20] = "HYPHENS_IN_REVIEW_EXTENSION_SNIPPET"] = 20;
                                values[valuesById[21] = "REVIEW_EXTENSION_SOURCE_INELIGIBLE"] = 21;
                                values[valuesById[22] = "SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT"] = 22;
                                values[valuesById[23] = "INCONSISTENT_CURRENCY_CODES"] = 23;
                                values[valuesById[24] = "PRICE_EXTENSION_HAS_DUPLICATED_HEADERS"] = 24;
                                values[valuesById[25] = "PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION"] = 25;
                                values[valuesById[26] = "PRICE_EXTENSION_HAS_TOO_FEW_ITEMS"] = 26;
                                values[valuesById[27] = "PRICE_EXTENSION_HAS_TOO_MANY_ITEMS"] = 27;
                                values[valuesById[28] = "UNSUPPORTED_VALUE"] = 28;
                                values[valuesById[29] = "UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE"] = 29;
                                values[valuesById[30] = "INVALID_DEVICE_PREFERENCE"] = 30;
                                values[valuesById[31] = "INVALID_SCHEDULE_END"] = 31;
                                values[valuesById[32] = "DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE"] = 32;
                                values[valuesById[33] = "INVALID_SNIPPETS_HEADER"] = 33;
                                values[valuesById[34] = "CANNOT_OPERATE_ON_REMOVED_FEED_ITEM"] = 34;
                                values[valuesById[35] = "PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY"] = 35;
                                values[valuesById[36] = "CONFLICTING_CALL_CONVERSION_SETTINGS"] = 36;
                                values[valuesById[37] = "EXTENSION_TYPE_MISMATCH"] = 37;
                                values[valuesById[38] = "EXTENSION_SUBTYPE_REQUIRED"] = 38;
                                values[valuesById[39] = "EXTENSION_TYPE_UNSUPPORTED"] = 39;
                                values[valuesById[40] = "CANNOT_OPERATE_ON_FEED_WITH_MULTIPLE_MAPPINGS"] = 40;
                                values[valuesById[41] = "CANNOT_OPERATE_ON_FEED_WITH_KEY_ATTRIBUTES"] = 41;
                                values[valuesById[42] = "INVALID_PRICE_FORMAT"] = 42;
                                values[valuesById[43] = "PROMOTION_INVALID_TIME"] = 43;
                                values[valuesById[44] = "TOO_MANY_DECIMAL_PLACES_SPECIFIED"] = 44;
                                values[valuesById[45] = "CONCRETE_EXTENSION_TYPE_REQUIRED"] = 45;
                                values[valuesById[46] = "SCHEDULE_END_NOT_AFTER_START"] = 46;
                                return values;
                            })();
    
                            return ExtensionFeedItemErrorEnum;
                        })();
    
                        errors.ExtensionSettingErrorEnum = (function() {
    
                            /**
                             * Properties of an ExtensionSettingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IExtensionSettingErrorEnum
                             */
    
                            /**
                             * Constructs a new ExtensionSettingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ExtensionSettingErrorEnum.
                             * @implements IExtensionSettingErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IExtensionSettingErrorEnum=} [p] Properties to set
                             */
                            function ExtensionSettingErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ExtensionSettingErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExtensionSettingErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ExtensionSettingErrorEnum} ExtensionSettingErrorEnum instance
                             */
                            ExtensionSettingErrorEnum.create = function create(properties) {
                                return new ExtensionSettingErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ExtensionSettingErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ExtensionSettingErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExtensionSettingErrorEnum} m ExtensionSettingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionSettingErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExtensionSettingErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ExtensionSettingErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IExtensionSettingErrorEnum} message ExtensionSettingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionSettingErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExtensionSettingErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ExtensionSettingErrorEnum} ExtensionSettingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionSettingErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ExtensionSettingErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExtensionSettingErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ExtensionSettingErrorEnum} ExtensionSettingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionSettingErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExtensionSettingErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExtensionSettingErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ExtensionSettingErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ExtensionSettingErrorEnum} ExtensionSettingErrorEnum
                             */
                            ExtensionSettingErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ExtensionSettingErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ExtensionSettingErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an ExtensionSettingErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ExtensionSettingErrorEnum} m ExtensionSettingErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExtensionSettingErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ExtensionSettingErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ExtensionSettingErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExtensionSettingErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ExtensionSettingError enum.
                             * @name google.ads.googleads.v11.errors.ExtensionSettingErrorEnum.ExtensionSettingError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EXTENSIONS_REQUIRED=2 EXTENSIONS_REQUIRED value
                             * @property {number} FEED_TYPE_EXTENSION_TYPE_MISMATCH=3 FEED_TYPE_EXTENSION_TYPE_MISMATCH value
                             * @property {number} INVALID_FEED_TYPE=4 INVALID_FEED_TYPE value
                             * @property {number} INVALID_FEED_TYPE_FOR_CUSTOMER_EXTENSION_SETTING=5 INVALID_FEED_TYPE_FOR_CUSTOMER_EXTENSION_SETTING value
                             * @property {number} CANNOT_CHANGE_FEED_ITEM_ON_CREATE=6 CANNOT_CHANGE_FEED_ITEM_ON_CREATE value
                             * @property {number} CANNOT_UPDATE_NEWLY_CREATED_EXTENSION=7 CANNOT_UPDATE_NEWLY_CREATED_EXTENSION value
                             * @property {number} NO_EXISTING_AD_GROUP_EXTENSION_SETTING_FOR_TYPE=8 NO_EXISTING_AD_GROUP_EXTENSION_SETTING_FOR_TYPE value
                             * @property {number} NO_EXISTING_CAMPAIGN_EXTENSION_SETTING_FOR_TYPE=9 NO_EXISTING_CAMPAIGN_EXTENSION_SETTING_FOR_TYPE value
                             * @property {number} NO_EXISTING_CUSTOMER_EXTENSION_SETTING_FOR_TYPE=10 NO_EXISTING_CUSTOMER_EXTENSION_SETTING_FOR_TYPE value
                             * @property {number} AD_GROUP_EXTENSION_SETTING_ALREADY_EXISTS=11 AD_GROUP_EXTENSION_SETTING_ALREADY_EXISTS value
                             * @property {number} CAMPAIGN_EXTENSION_SETTING_ALREADY_EXISTS=12 CAMPAIGN_EXTENSION_SETTING_ALREADY_EXISTS value
                             * @property {number} CUSTOMER_EXTENSION_SETTING_ALREADY_EXISTS=13 CUSTOMER_EXTENSION_SETTING_ALREADY_EXISTS value
                             * @property {number} AD_GROUP_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE=14 AD_GROUP_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE value
                             * @property {number} CAMPAIGN_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE=15 CAMPAIGN_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE value
                             * @property {number} CUSTOMER_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE=16 CUSTOMER_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE value
                             * @property {number} VALUE_OUT_OF_RANGE=17 VALUE_OUT_OF_RANGE value
                             * @property {number} CANNOT_SET_FIELD_WITH_FINAL_URLS=18 CANNOT_SET_FIELD_WITH_FINAL_URLS value
                             * @property {number} FINAL_URLS_NOT_SET=19 FINAL_URLS_NOT_SET value
                             * @property {number} INVALID_PHONE_NUMBER=20 INVALID_PHONE_NUMBER value
                             * @property {number} PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY=21 PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED=22 CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED value
                             * @property {number} PREMIUM_RATE_NUMBER_NOT_ALLOWED=23 PREMIUM_RATE_NUMBER_NOT_ALLOWED value
                             * @property {number} DISALLOWED_NUMBER_TYPE=24 DISALLOWED_NUMBER_TYPE value
                             * @property {number} INVALID_DOMESTIC_PHONE_NUMBER_FORMAT=25 INVALID_DOMESTIC_PHONE_NUMBER_FORMAT value
                             * @property {number} VANITY_PHONE_NUMBER_NOT_ALLOWED=26 VANITY_PHONE_NUMBER_NOT_ALLOWED value
                             * @property {number} INVALID_COUNTRY_CODE=27 INVALID_COUNTRY_CODE value
                             * @property {number} INVALID_CALL_CONVERSION_TYPE_ID=28 INVALID_CALL_CONVERSION_TYPE_ID value
                             * @property {number} CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING=69 CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING value
                             * @property {number} CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY=30 CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} INVALID_APP_ID=31 INVALID_APP_ID value
                             * @property {number} QUOTES_IN_REVIEW_EXTENSION_SNIPPET=32 QUOTES_IN_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} HYPHENS_IN_REVIEW_EXTENSION_SNIPPET=33 HYPHENS_IN_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} REVIEW_EXTENSION_SOURCE_NOT_ELIGIBLE=34 REVIEW_EXTENSION_SOURCE_NOT_ELIGIBLE value
                             * @property {number} SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT=35 SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT value
                             * @property {number} MISSING_FIELD=36 MISSING_FIELD value
                             * @property {number} INCONSISTENT_CURRENCY_CODES=37 INCONSISTENT_CURRENCY_CODES value
                             * @property {number} PRICE_EXTENSION_HAS_DUPLICATED_HEADERS=38 PRICE_EXTENSION_HAS_DUPLICATED_HEADERS value
                             * @property {number} PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION=39 PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION value
                             * @property {number} PRICE_EXTENSION_HAS_TOO_FEW_ITEMS=40 PRICE_EXTENSION_HAS_TOO_FEW_ITEMS value
                             * @property {number} PRICE_EXTENSION_HAS_TOO_MANY_ITEMS=41 PRICE_EXTENSION_HAS_TOO_MANY_ITEMS value
                             * @property {number} UNSUPPORTED_VALUE=42 UNSUPPORTED_VALUE value
                             * @property {number} INVALID_DEVICE_PREFERENCE=43 INVALID_DEVICE_PREFERENCE value
                             * @property {number} INVALID_SCHEDULE_END=45 INVALID_SCHEDULE_END value
                             * @property {number} DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE=47 DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE value
                             * @property {number} OVERLAPPING_SCHEDULES_NOT_ALLOWED=48 OVERLAPPING_SCHEDULES_NOT_ALLOWED value
                             * @property {number} SCHEDULE_END_NOT_AFTER_START=49 SCHEDULE_END_NOT_AFTER_START value
                             * @property {number} TOO_MANY_SCHEDULES_PER_DAY=50 TOO_MANY_SCHEDULES_PER_DAY value
                             * @property {number} DUPLICATE_EXTENSION_FEED_ITEM_EDIT=51 DUPLICATE_EXTENSION_FEED_ITEM_EDIT value
                             * @property {number} INVALID_SNIPPETS_HEADER=52 INVALID_SNIPPETS_HEADER value
                             * @property {number} PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY=53 PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY value
                             * @property {number} CAMPAIGN_TARGETING_MISMATCH=54 CAMPAIGN_TARGETING_MISMATCH value
                             * @property {number} CANNOT_OPERATE_ON_REMOVED_FEED=55 CANNOT_OPERATE_ON_REMOVED_FEED value
                             * @property {number} EXTENSION_TYPE_REQUIRED=56 EXTENSION_TYPE_REQUIRED value
                             * @property {number} INCOMPATIBLE_UNDERLYING_MATCHING_FUNCTION=57 INCOMPATIBLE_UNDERLYING_MATCHING_FUNCTION value
                             * @property {number} START_DATE_AFTER_END_DATE=58 START_DATE_AFTER_END_DATE value
                             * @property {number} INVALID_PRICE_FORMAT=59 INVALID_PRICE_FORMAT value
                             * @property {number} PROMOTION_INVALID_TIME=60 PROMOTION_INVALID_TIME value
                             * @property {number} PROMOTION_CANNOT_SET_PERCENT_DISCOUNT_AND_MONEY_DISCOUNT=61 PROMOTION_CANNOT_SET_PERCENT_DISCOUNT_AND_MONEY_DISCOUNT value
                             * @property {number} PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT=62 PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT value
                             * @property {number} TOO_MANY_DECIMAL_PLACES_SPECIFIED=63 TOO_MANY_DECIMAL_PLACES_SPECIFIED value
                             * @property {number} INVALID_LANGUAGE_CODE=64 INVALID_LANGUAGE_CODE value
                             * @property {number} UNSUPPORTED_LANGUAGE=65 UNSUPPORTED_LANGUAGE value
                             * @property {number} CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED=66 CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED value
                             * @property {number} EXTENSION_SETTING_UPDATE_IS_A_NOOP=67 EXTENSION_SETTING_UPDATE_IS_A_NOOP value
                             * @property {number} DISALLOWED_TEXT=68 DISALLOWED_TEXT value
                             */
                            ExtensionSettingErrorEnum.ExtensionSettingError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EXTENSIONS_REQUIRED"] = 2;
                                values[valuesById[3] = "FEED_TYPE_EXTENSION_TYPE_MISMATCH"] = 3;
                                values[valuesById[4] = "INVALID_FEED_TYPE"] = 4;
                                values[valuesById[5] = "INVALID_FEED_TYPE_FOR_CUSTOMER_EXTENSION_SETTING"] = 5;
                                values[valuesById[6] = "CANNOT_CHANGE_FEED_ITEM_ON_CREATE"] = 6;
                                values[valuesById[7] = "CANNOT_UPDATE_NEWLY_CREATED_EXTENSION"] = 7;
                                values[valuesById[8] = "NO_EXISTING_AD_GROUP_EXTENSION_SETTING_FOR_TYPE"] = 8;
                                values[valuesById[9] = "NO_EXISTING_CAMPAIGN_EXTENSION_SETTING_FOR_TYPE"] = 9;
                                values[valuesById[10] = "NO_EXISTING_CUSTOMER_EXTENSION_SETTING_FOR_TYPE"] = 10;
                                values[valuesById[11] = "AD_GROUP_EXTENSION_SETTING_ALREADY_EXISTS"] = 11;
                                values[valuesById[12] = "CAMPAIGN_EXTENSION_SETTING_ALREADY_EXISTS"] = 12;
                                values[valuesById[13] = "CUSTOMER_EXTENSION_SETTING_ALREADY_EXISTS"] = 13;
                                values[valuesById[14] = "AD_GROUP_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE"] = 14;
                                values[valuesById[15] = "CAMPAIGN_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE"] = 15;
                                values[valuesById[16] = "CUSTOMER_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE"] = 16;
                                values[valuesById[17] = "VALUE_OUT_OF_RANGE"] = 17;
                                values[valuesById[18] = "CANNOT_SET_FIELD_WITH_FINAL_URLS"] = 18;
                                values[valuesById[19] = "FINAL_URLS_NOT_SET"] = 19;
                                values[valuesById[20] = "INVALID_PHONE_NUMBER"] = 20;
                                values[valuesById[21] = "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY"] = 21;
                                values[valuesById[22] = "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED"] = 22;
                                values[valuesById[23] = "PREMIUM_RATE_NUMBER_NOT_ALLOWED"] = 23;
                                values[valuesById[24] = "DISALLOWED_NUMBER_TYPE"] = 24;
                                values[valuesById[25] = "INVALID_DOMESTIC_PHONE_NUMBER_FORMAT"] = 25;
                                values[valuesById[26] = "VANITY_PHONE_NUMBER_NOT_ALLOWED"] = 26;
                                values[valuesById[27] = "INVALID_COUNTRY_CODE"] = 27;
                                values[valuesById[28] = "INVALID_CALL_CONVERSION_TYPE_ID"] = 28;
                                values[valuesById[69] = "CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING"] = 69;
                                values[valuesById[30] = "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY"] = 30;
                                values[valuesById[31] = "INVALID_APP_ID"] = 31;
                                values[valuesById[32] = "QUOTES_IN_REVIEW_EXTENSION_SNIPPET"] = 32;
                                values[valuesById[33] = "HYPHENS_IN_REVIEW_EXTENSION_SNIPPET"] = 33;
                                values[valuesById[34] = "REVIEW_EXTENSION_SOURCE_NOT_ELIGIBLE"] = 34;
                                values[valuesById[35] = "SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT"] = 35;
                                values[valuesById[36] = "MISSING_FIELD"] = 36;
                                values[valuesById[37] = "INCONSISTENT_CURRENCY_CODES"] = 37;
                                values[valuesById[38] = "PRICE_EXTENSION_HAS_DUPLICATED_HEADERS"] = 38;
                                values[valuesById[39] = "PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION"] = 39;
                                values[valuesById[40] = "PRICE_EXTENSION_HAS_TOO_FEW_ITEMS"] = 40;
                                values[valuesById[41] = "PRICE_EXTENSION_HAS_TOO_MANY_ITEMS"] = 41;
                                values[valuesById[42] = "UNSUPPORTED_VALUE"] = 42;
                                values[valuesById[43] = "INVALID_DEVICE_PREFERENCE"] = 43;
                                values[valuesById[45] = "INVALID_SCHEDULE_END"] = 45;
                                values[valuesById[47] = "DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE"] = 47;
                                values[valuesById[48] = "OVERLAPPING_SCHEDULES_NOT_ALLOWED"] = 48;
                                values[valuesById[49] = "SCHEDULE_END_NOT_AFTER_START"] = 49;
                                values[valuesById[50] = "TOO_MANY_SCHEDULES_PER_DAY"] = 50;
                                values[valuesById[51] = "DUPLICATE_EXTENSION_FEED_ITEM_EDIT"] = 51;
                                values[valuesById[52] = "INVALID_SNIPPETS_HEADER"] = 52;
                                values[valuesById[53] = "PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY"] = 53;
                                values[valuesById[54] = "CAMPAIGN_TARGETING_MISMATCH"] = 54;
                                values[valuesById[55] = "CANNOT_OPERATE_ON_REMOVED_FEED"] = 55;
                                values[valuesById[56] = "EXTENSION_TYPE_REQUIRED"] = 56;
                                values[valuesById[57] = "INCOMPATIBLE_UNDERLYING_MATCHING_FUNCTION"] = 57;
                                values[valuesById[58] = "START_DATE_AFTER_END_DATE"] = 58;
                                values[valuesById[59] = "INVALID_PRICE_FORMAT"] = 59;
                                values[valuesById[60] = "PROMOTION_INVALID_TIME"] = 60;
                                values[valuesById[61] = "PROMOTION_CANNOT_SET_PERCENT_DISCOUNT_AND_MONEY_DISCOUNT"] = 61;
                                values[valuesById[62] = "PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT"] = 62;
                                values[valuesById[63] = "TOO_MANY_DECIMAL_PLACES_SPECIFIED"] = 63;
                                values[valuesById[64] = "INVALID_LANGUAGE_CODE"] = 64;
                                values[valuesById[65] = "UNSUPPORTED_LANGUAGE"] = 65;
                                values[valuesById[66] = "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED"] = 66;
                                values[valuesById[67] = "EXTENSION_SETTING_UPDATE_IS_A_NOOP"] = 67;
                                values[valuesById[68] = "DISALLOWED_TEXT"] = 68;
                                return values;
                            })();
    
                            return ExtensionSettingErrorEnum;
                        })();
    
                        errors.FeedAttributeReferenceErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedAttributeReferenceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedAttributeReferenceErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedAttributeReferenceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedAttributeReferenceErrorEnum.
                             * @implements IFeedAttributeReferenceErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedAttributeReferenceErrorEnum=} [p] Properties to set
                             */
                            function FeedAttributeReferenceErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedAttributeReferenceErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedAttributeReferenceErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum} FeedAttributeReferenceErrorEnum instance
                             */
                            FeedAttributeReferenceErrorEnum.create = function create(properties) {
                                return new FeedAttributeReferenceErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedAttributeReferenceErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedAttributeReferenceErrorEnum} m FeedAttributeReferenceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttributeReferenceErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedAttributeReferenceErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedAttributeReferenceErrorEnum} message FeedAttributeReferenceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttributeReferenceErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedAttributeReferenceErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum} FeedAttributeReferenceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttributeReferenceErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedAttributeReferenceErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum} FeedAttributeReferenceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttributeReferenceErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedAttributeReferenceErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedAttributeReferenceErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedAttributeReferenceErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum} FeedAttributeReferenceErrorEnum
                             */
                            FeedAttributeReferenceErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedAttributeReferenceErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum} m FeedAttributeReferenceErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedAttributeReferenceErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedAttributeReferenceErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedAttributeReferenceErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedAttributeReferenceError enum.
                             * @name google.ads.googleads.v11.errors.FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_REFERENCE_REMOVED_FEED=2 CANNOT_REFERENCE_REMOVED_FEED value
                             * @property {number} INVALID_FEED_NAME=3 INVALID_FEED_NAME value
                             * @property {number} INVALID_FEED_ATTRIBUTE_NAME=4 INVALID_FEED_ATTRIBUTE_NAME value
                             */
                            FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_REFERENCE_REMOVED_FEED"] = 2;
                                values[valuesById[3] = "INVALID_FEED_NAME"] = 3;
                                values[valuesById[4] = "INVALID_FEED_ATTRIBUTE_NAME"] = 4;
                                return values;
                            })();
    
                            return FeedAttributeReferenceErrorEnum;
                        })();
    
                        errors.FeedErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedErrorEnum.
                             * @implements IFeedErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedErrorEnum=} [p] Properties to set
                             */
                            function FeedErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedErrorEnum} FeedErrorEnum instance
                             */
                            FeedErrorEnum.create = function create(properties) {
                                return new FeedErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedErrorEnum} m FeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedErrorEnum} message FeedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedErrorEnum} FeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedErrorEnum} FeedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedErrorEnum} FeedErrorEnum
                             */
                            FeedErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedErrorEnum} m FeedErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedError enum.
                             * @name google.ads.googleads.v11.errors.FeedErrorEnum.FeedError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ATTRIBUTE_NAMES_NOT_UNIQUE=2 ATTRIBUTE_NAMES_NOT_UNIQUE value
                             * @property {number} ATTRIBUTES_DO_NOT_MATCH_EXISTING_ATTRIBUTES=3 ATTRIBUTES_DO_NOT_MATCH_EXISTING_ATTRIBUTES value
                             * @property {number} CANNOT_SPECIFY_USER_ORIGIN_FOR_SYSTEM_FEED=4 CANNOT_SPECIFY_USER_ORIGIN_FOR_SYSTEM_FEED value
                             * @property {number} CANNOT_SPECIFY_GOOGLE_ORIGIN_FOR_NON_SYSTEM_FEED=5 CANNOT_SPECIFY_GOOGLE_ORIGIN_FOR_NON_SYSTEM_FEED value
                             * @property {number} CANNOT_SPECIFY_FEED_ATTRIBUTES_FOR_SYSTEM_FEED=6 CANNOT_SPECIFY_FEED_ATTRIBUTES_FOR_SYSTEM_FEED value
                             * @property {number} CANNOT_UPDATE_FEED_ATTRIBUTES_WITH_ORIGIN_GOOGLE=7 CANNOT_UPDATE_FEED_ATTRIBUTES_WITH_ORIGIN_GOOGLE value
                             * @property {number} FEED_REMOVED=8 FEED_REMOVED value
                             * @property {number} INVALID_ORIGIN_VALUE=9 INVALID_ORIGIN_VALUE value
                             * @property {number} FEED_ORIGIN_IS_NOT_USER=10 FEED_ORIGIN_IS_NOT_USER value
                             * @property {number} INVALID_AUTH_TOKEN_FOR_EMAIL=11 INVALID_AUTH_TOKEN_FOR_EMAIL value
                             * @property {number} INVALID_EMAIL=12 INVALID_EMAIL value
                             * @property {number} DUPLICATE_FEED_NAME=13 DUPLICATE_FEED_NAME value
                             * @property {number} INVALID_FEED_NAME=14 INVALID_FEED_NAME value
                             * @property {number} MISSING_OAUTH_INFO=15 MISSING_OAUTH_INFO value
                             * @property {number} NEW_ATTRIBUTE_CANNOT_BE_PART_OF_UNIQUE_KEY=16 NEW_ATTRIBUTE_CANNOT_BE_PART_OF_UNIQUE_KEY value
                             * @property {number} TOO_MANY_ATTRIBUTES=17 TOO_MANY_ATTRIBUTES value
                             * @property {number} INVALID_BUSINESS_ACCOUNT=18 INVALID_BUSINESS_ACCOUNT value
                             * @property {number} BUSINESS_ACCOUNT_CANNOT_ACCESS_LOCATION_ACCOUNT=19 BUSINESS_ACCOUNT_CANNOT_ACCESS_LOCATION_ACCOUNT value
                             * @property {number} INVALID_AFFILIATE_CHAIN_ID=20 INVALID_AFFILIATE_CHAIN_ID value
                             * @property {number} DUPLICATE_SYSTEM_FEED=21 DUPLICATE_SYSTEM_FEED value
                             * @property {number} GMB_ACCESS_ERROR=22 GMB_ACCESS_ERROR value
                             * @property {number} CANNOT_HAVE_LOCATION_AND_AFFILIATE_LOCATION_FEEDS=23 CANNOT_HAVE_LOCATION_AND_AFFILIATE_LOCATION_FEEDS value
                             * @property {number} LEGACY_EXTENSION_TYPE_READ_ONLY=24 LEGACY_EXTENSION_TYPE_READ_ONLY value
                             */
                            FeedErrorEnum.FeedError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ATTRIBUTE_NAMES_NOT_UNIQUE"] = 2;
                                values[valuesById[3] = "ATTRIBUTES_DO_NOT_MATCH_EXISTING_ATTRIBUTES"] = 3;
                                values[valuesById[4] = "CANNOT_SPECIFY_USER_ORIGIN_FOR_SYSTEM_FEED"] = 4;
                                values[valuesById[5] = "CANNOT_SPECIFY_GOOGLE_ORIGIN_FOR_NON_SYSTEM_FEED"] = 5;
                                values[valuesById[6] = "CANNOT_SPECIFY_FEED_ATTRIBUTES_FOR_SYSTEM_FEED"] = 6;
                                values[valuesById[7] = "CANNOT_UPDATE_FEED_ATTRIBUTES_WITH_ORIGIN_GOOGLE"] = 7;
                                values[valuesById[8] = "FEED_REMOVED"] = 8;
                                values[valuesById[9] = "INVALID_ORIGIN_VALUE"] = 9;
                                values[valuesById[10] = "FEED_ORIGIN_IS_NOT_USER"] = 10;
                                values[valuesById[11] = "INVALID_AUTH_TOKEN_FOR_EMAIL"] = 11;
                                values[valuesById[12] = "INVALID_EMAIL"] = 12;
                                values[valuesById[13] = "DUPLICATE_FEED_NAME"] = 13;
                                values[valuesById[14] = "INVALID_FEED_NAME"] = 14;
                                values[valuesById[15] = "MISSING_OAUTH_INFO"] = 15;
                                values[valuesById[16] = "NEW_ATTRIBUTE_CANNOT_BE_PART_OF_UNIQUE_KEY"] = 16;
                                values[valuesById[17] = "TOO_MANY_ATTRIBUTES"] = 17;
                                values[valuesById[18] = "INVALID_BUSINESS_ACCOUNT"] = 18;
                                values[valuesById[19] = "BUSINESS_ACCOUNT_CANNOT_ACCESS_LOCATION_ACCOUNT"] = 19;
                                values[valuesById[20] = "INVALID_AFFILIATE_CHAIN_ID"] = 20;
                                values[valuesById[21] = "DUPLICATE_SYSTEM_FEED"] = 21;
                                values[valuesById[22] = "GMB_ACCESS_ERROR"] = 22;
                                values[valuesById[23] = "CANNOT_HAVE_LOCATION_AND_AFFILIATE_LOCATION_FEEDS"] = 23;
                                values[valuesById[24] = "LEGACY_EXTENSION_TYPE_READ_ONLY"] = 24;
                                return values;
                            })();
    
                            return FeedErrorEnum;
                        })();
    
                        errors.FeedItemErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedItemErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedItemErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedItemErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedItemErrorEnum.
                             * @implements IFeedItemErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedItemErrorEnum=} [p] Properties to set
                             */
                            function FeedItemErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedItemErrorEnum} FeedItemErrorEnum instance
                             */
                            FeedItemErrorEnum.create = function create(properties) {
                                return new FeedItemErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemErrorEnum} m FeedItemErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemErrorEnum} message FeedItemErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedItemErrorEnum} FeedItemErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedItemErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedItemErrorEnum} FeedItemErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedItemErrorEnum} FeedItemErrorEnum
                             */
                            FeedItemErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedItemErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedItemErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedItemErrorEnum} m FeedItemErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedItemErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemError enum.
                             * @name google.ads.googleads.v11.errors.FeedItemErrorEnum.FeedItemError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_CONVERT_ATTRIBUTE_VALUE_FROM_STRING=2 CANNOT_CONVERT_ATTRIBUTE_VALUE_FROM_STRING value
                             * @property {number} CANNOT_OPERATE_ON_REMOVED_FEED_ITEM=3 CANNOT_OPERATE_ON_REMOVED_FEED_ITEM value
                             * @property {number} DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE=4 DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE value
                             * @property {number} KEY_ATTRIBUTES_NOT_FOUND=5 KEY_ATTRIBUTES_NOT_FOUND value
                             * @property {number} INVALID_URL=6 INVALID_URL value
                             * @property {number} MISSING_KEY_ATTRIBUTES=7 MISSING_KEY_ATTRIBUTES value
                             * @property {number} KEY_ATTRIBUTES_NOT_UNIQUE=8 KEY_ATTRIBUTES_NOT_UNIQUE value
                             * @property {number} CANNOT_MODIFY_KEY_ATTRIBUTE_VALUE=9 CANNOT_MODIFY_KEY_ATTRIBUTE_VALUE value
                             * @property {number} SIZE_TOO_LARGE_FOR_MULTI_VALUE_ATTRIBUTE=10 SIZE_TOO_LARGE_FOR_MULTI_VALUE_ATTRIBUTE value
                             */
                            FeedItemErrorEnum.FeedItemError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_CONVERT_ATTRIBUTE_VALUE_FROM_STRING"] = 2;
                                values[valuesById[3] = "CANNOT_OPERATE_ON_REMOVED_FEED_ITEM"] = 3;
                                values[valuesById[4] = "DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE"] = 4;
                                values[valuesById[5] = "KEY_ATTRIBUTES_NOT_FOUND"] = 5;
                                values[valuesById[6] = "INVALID_URL"] = 6;
                                values[valuesById[7] = "MISSING_KEY_ATTRIBUTES"] = 7;
                                values[valuesById[8] = "KEY_ATTRIBUTES_NOT_UNIQUE"] = 8;
                                values[valuesById[9] = "CANNOT_MODIFY_KEY_ATTRIBUTE_VALUE"] = 9;
                                values[valuesById[10] = "SIZE_TOO_LARGE_FOR_MULTI_VALUE_ATTRIBUTE"] = 10;
                                return values;
                            })();
    
                            return FeedItemErrorEnum;
                        })();
    
                        errors.FeedItemSetErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedItemSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedItemSetErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedItemSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedItemSetErrorEnum.
                             * @implements IFeedItemSetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetErrorEnum=} [p] Properties to set
                             */
                            function FeedItemSetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemSetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetErrorEnum} FeedItemSetErrorEnum instance
                             */
                            FeedItemSetErrorEnum.create = function create(properties) {
                                return new FeedItemSetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemSetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetErrorEnum} m FeedItemSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemSetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetErrorEnum} message FeedItemSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetErrorEnum} FeedItemSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedItemSetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetErrorEnum} FeedItemSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetErrorEnum} FeedItemSetErrorEnum
                             */
                            FeedItemSetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedItemSetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedItemSetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedItemSetErrorEnum} m FeedItemSetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemSetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemSetError enum.
                             * @name google.ads.googleads.v11.errors.FeedItemSetErrorEnum.FeedItemSetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FEED_ITEM_SET_REMOVED=2 FEED_ITEM_SET_REMOVED value
                             * @property {number} CANNOT_CLEAR_DYNAMIC_FILTER=3 CANNOT_CLEAR_DYNAMIC_FILTER value
                             * @property {number} CANNOT_CREATE_DYNAMIC_FILTER=4 CANNOT_CREATE_DYNAMIC_FILTER value
                             * @property {number} INVALID_FEED_TYPE=5 INVALID_FEED_TYPE value
                             * @property {number} DUPLICATE_NAME=6 DUPLICATE_NAME value
                             * @property {number} WRONG_DYNAMIC_FILTER_FOR_FEED_TYPE=7 WRONG_DYNAMIC_FILTER_FOR_FEED_TYPE value
                             * @property {number} DYNAMIC_FILTER_INVALID_CHAIN_IDS=8 DYNAMIC_FILTER_INVALID_CHAIN_IDS value
                             */
                            FeedItemSetErrorEnum.FeedItemSetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FEED_ITEM_SET_REMOVED"] = 2;
                                values[valuesById[3] = "CANNOT_CLEAR_DYNAMIC_FILTER"] = 3;
                                values[valuesById[4] = "CANNOT_CREATE_DYNAMIC_FILTER"] = 4;
                                values[valuesById[5] = "INVALID_FEED_TYPE"] = 5;
                                values[valuesById[6] = "DUPLICATE_NAME"] = 6;
                                values[valuesById[7] = "WRONG_DYNAMIC_FILTER_FOR_FEED_TYPE"] = 7;
                                values[valuesById[8] = "DYNAMIC_FILTER_INVALID_CHAIN_IDS"] = 8;
                                return values;
                            })();
    
                            return FeedItemSetErrorEnum;
                        })();
    
                        errors.FeedItemSetLinkErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedItemSetLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedItemSetLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedItemSetLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedItemSetLinkErrorEnum.
                             * @implements IFeedItemSetLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetLinkErrorEnum=} [p] Properties to set
                             */
                            function FeedItemSetLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemSetLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum} FeedItemSetLinkErrorEnum instance
                             */
                            FeedItemSetLinkErrorEnum.create = function create(properties) {
                                return new FeedItemSetLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSetLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetLinkErrorEnum} m FeedItemSetLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSetLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemSetLinkErrorEnum} message FeedItemSetLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSetLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum} FeedItemSetLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSetLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum} FeedItemSetLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSetLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSetLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSetLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum} FeedItemSetLinkErrorEnum
                             */
                            FeedItemSetLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSetLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum} m FeedItemSetLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSetLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemSetLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSetLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemSetLinkError enum.
                             * @name google.ads.googleads.v11.errors.FeedItemSetLinkErrorEnum.FeedItemSetLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FEED_ID_MISMATCH=2 FEED_ID_MISMATCH value
                             * @property {number} NO_MUTATE_ALLOWED_FOR_DYNAMIC_SET=3 NO_MUTATE_ALLOWED_FOR_DYNAMIC_SET value
                             */
                            FeedItemSetLinkErrorEnum.FeedItemSetLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FEED_ID_MISMATCH"] = 2;
                                values[valuesById[3] = "NO_MUTATE_ALLOWED_FOR_DYNAMIC_SET"] = 3;
                                return values;
                            })();
    
                            return FeedItemSetLinkErrorEnum;
                        })();
    
                        errors.FeedItemTargetErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedItemTargetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedItemTargetErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedItemTargetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedItemTargetErrorEnum.
                             * @implements IFeedItemTargetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedItemTargetErrorEnum=} [p] Properties to set
                             */
                            function FeedItemTargetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemTargetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemTargetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedItemTargetErrorEnum} FeedItemTargetErrorEnum instance
                             */
                            FeedItemTargetErrorEnum.create = function create(properties) {
                                return new FeedItemTargetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemTargetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemTargetErrorEnum} m FeedItemTargetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemTargetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemTargetErrorEnum} message FeedItemTargetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemTargetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedItemTargetErrorEnum} FeedItemTargetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedItemTargetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemTargetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedItemTargetErrorEnum} FeedItemTargetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemTargetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemTargetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemTargetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedItemTargetErrorEnum} FeedItemTargetErrorEnum
                             */
                            FeedItemTargetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedItemTargetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedItemTargetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemTargetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedItemTargetErrorEnum} m FeedItemTargetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemTargetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemTargetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedItemTargetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemTargetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemTargetError enum.
                             * @name google.ads.googleads.v11.errors.FeedItemTargetErrorEnum.FeedItemTargetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MUST_SET_TARGET_ONEOF_ON_CREATE=2 MUST_SET_TARGET_ONEOF_ON_CREATE value
                             * @property {number} FEED_ITEM_TARGET_ALREADY_EXISTS=3 FEED_ITEM_TARGET_ALREADY_EXISTS value
                             * @property {number} FEED_ITEM_SCHEDULES_CANNOT_OVERLAP=4 FEED_ITEM_SCHEDULES_CANNOT_OVERLAP value
                             * @property {number} TARGET_LIMIT_EXCEEDED_FOR_GIVEN_TYPE=5 TARGET_LIMIT_EXCEEDED_FOR_GIVEN_TYPE value
                             * @property {number} TOO_MANY_SCHEDULES_PER_DAY=6 TOO_MANY_SCHEDULES_PER_DAY value
                             * @property {number} CANNOT_HAVE_ENABLED_CAMPAIGN_AND_ENABLED_AD_GROUP_TARGETS=7 CANNOT_HAVE_ENABLED_CAMPAIGN_AND_ENABLED_AD_GROUP_TARGETS value
                             * @property {number} DUPLICATE_AD_SCHEDULE=8 DUPLICATE_AD_SCHEDULE value
                             * @property {number} DUPLICATE_KEYWORD=9 DUPLICATE_KEYWORD value
                             */
                            FeedItemTargetErrorEnum.FeedItemTargetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MUST_SET_TARGET_ONEOF_ON_CREATE"] = 2;
                                values[valuesById[3] = "FEED_ITEM_TARGET_ALREADY_EXISTS"] = 3;
                                values[valuesById[4] = "FEED_ITEM_SCHEDULES_CANNOT_OVERLAP"] = 4;
                                values[valuesById[5] = "TARGET_LIMIT_EXCEEDED_FOR_GIVEN_TYPE"] = 5;
                                values[valuesById[6] = "TOO_MANY_SCHEDULES_PER_DAY"] = 6;
                                values[valuesById[7] = "CANNOT_HAVE_ENABLED_CAMPAIGN_AND_ENABLED_AD_GROUP_TARGETS"] = 7;
                                values[valuesById[8] = "DUPLICATE_AD_SCHEDULE"] = 8;
                                values[valuesById[9] = "DUPLICATE_KEYWORD"] = 9;
                                return values;
                            })();
    
                            return FeedItemTargetErrorEnum;
                        })();
    
                        errors.FeedItemValidationErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedItemValidationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedItemValidationErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedItemValidationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedItemValidationErrorEnum.
                             * @implements IFeedItemValidationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedItemValidationErrorEnum=} [p] Properties to set
                             */
                            function FeedItemValidationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedItemValidationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemValidationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum} FeedItemValidationErrorEnum instance
                             */
                            FeedItemValidationErrorEnum.create = function create(properties) {
                                return new FeedItemValidationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemValidationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemValidationErrorEnum} m FeedItemValidationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemValidationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemValidationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedItemValidationErrorEnum} message FeedItemValidationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemValidationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemValidationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum} FeedItemValidationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemValidationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedItemValidationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemValidationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum} FeedItemValidationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemValidationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemValidationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemValidationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemValidationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum} FeedItemValidationErrorEnum
                             */
                            FeedItemValidationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedItemValidationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedItemValidationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedItemValidationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum} m FeedItemValidationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemValidationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedItemValidationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedItemValidationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemValidationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedItemValidationError enum.
                             * @name google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.FeedItemValidationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} STRING_TOO_SHORT=2 STRING_TOO_SHORT value
                             * @property {number} STRING_TOO_LONG=3 STRING_TOO_LONG value
                             * @property {number} VALUE_NOT_SPECIFIED=4 VALUE_NOT_SPECIFIED value
                             * @property {number} INVALID_DOMESTIC_PHONE_NUMBER_FORMAT=5 INVALID_DOMESTIC_PHONE_NUMBER_FORMAT value
                             * @property {number} INVALID_PHONE_NUMBER=6 INVALID_PHONE_NUMBER value
                             * @property {number} PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY=7 PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} PREMIUM_RATE_NUMBER_NOT_ALLOWED=8 PREMIUM_RATE_NUMBER_NOT_ALLOWED value
                             * @property {number} DISALLOWED_NUMBER_TYPE=9 DISALLOWED_NUMBER_TYPE value
                             * @property {number} VALUE_OUT_OF_RANGE=10 VALUE_OUT_OF_RANGE value
                             * @property {number} CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY=11 CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING=99 CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING value
                             * @property {number} INVALID_COUNTRY_CODE=13 INVALID_COUNTRY_CODE value
                             * @property {number} INVALID_APP_ID=14 INVALID_APP_ID value
                             * @property {number} MISSING_ATTRIBUTES_FOR_FIELDS=15 MISSING_ATTRIBUTES_FOR_FIELDS value
                             * @property {number} INVALID_TYPE_ID=16 INVALID_TYPE_ID value
                             * @property {number} INVALID_EMAIL_ADDRESS=17 INVALID_EMAIL_ADDRESS value
                             * @property {number} INVALID_HTTPS_URL=18 INVALID_HTTPS_URL value
                             * @property {number} MISSING_DELIVERY_ADDRESS=19 MISSING_DELIVERY_ADDRESS value
                             * @property {number} START_DATE_AFTER_END_DATE=20 START_DATE_AFTER_END_DATE value
                             * @property {number} MISSING_FEED_ITEM_START_TIME=21 MISSING_FEED_ITEM_START_TIME value
                             * @property {number} MISSING_FEED_ITEM_END_TIME=22 MISSING_FEED_ITEM_END_TIME value
                             * @property {number} MISSING_FEED_ITEM_ID=23 MISSING_FEED_ITEM_ID value
                             * @property {number} VANITY_PHONE_NUMBER_NOT_ALLOWED=24 VANITY_PHONE_NUMBER_NOT_ALLOWED value
                             * @property {number} INVALID_REVIEW_EXTENSION_SNIPPET=25 INVALID_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} INVALID_NUMBER_FORMAT=26 INVALID_NUMBER_FORMAT value
                             * @property {number} INVALID_DATE_FORMAT=27 INVALID_DATE_FORMAT value
                             * @property {number} INVALID_PRICE_FORMAT=28 INVALID_PRICE_FORMAT value
                             * @property {number} UNKNOWN_PLACEHOLDER_FIELD=29 UNKNOWN_PLACEHOLDER_FIELD value
                             * @property {number} MISSING_ENHANCED_SITELINK_DESCRIPTION_LINE=30 MISSING_ENHANCED_SITELINK_DESCRIPTION_LINE value
                             * @property {number} REVIEW_EXTENSION_SOURCE_INELIGIBLE=31 REVIEW_EXTENSION_SOURCE_INELIGIBLE value
                             * @property {number} HYPHENS_IN_REVIEW_EXTENSION_SNIPPET=32 HYPHENS_IN_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} DOUBLE_QUOTES_IN_REVIEW_EXTENSION_SNIPPET=33 DOUBLE_QUOTES_IN_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} QUOTES_IN_REVIEW_EXTENSION_SNIPPET=34 QUOTES_IN_REVIEW_EXTENSION_SNIPPET value
                             * @property {number} INVALID_FORM_ENCODED_PARAMS=35 INVALID_FORM_ENCODED_PARAMS value
                             * @property {number} INVALID_URL_PARAMETER_NAME=36 INVALID_URL_PARAMETER_NAME value
                             * @property {number} NO_GEOCODING_RESULT=37 NO_GEOCODING_RESULT value
                             * @property {number} SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT=38 SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT value
                             * @property {number} CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED=39 CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED value
                             * @property {number} INVALID_PLACEHOLDER_FIELD_ID=40 INVALID_PLACEHOLDER_FIELD_ID value
                             * @property {number} INVALID_URL_TAG=41 INVALID_URL_TAG value
                             * @property {number} LIST_TOO_LONG=42 LIST_TOO_LONG value
                             * @property {number} INVALID_ATTRIBUTES_COMBINATION=43 INVALID_ATTRIBUTES_COMBINATION value
                             * @property {number} DUPLICATE_VALUES=44 DUPLICATE_VALUES value
                             * @property {number} INVALID_CALL_CONVERSION_ACTION_ID=45 INVALID_CALL_CONVERSION_ACTION_ID value
                             * @property {number} CANNOT_SET_WITHOUT_FINAL_URLS=46 CANNOT_SET_WITHOUT_FINAL_URLS value
                             * @property {number} APP_ID_DOESNT_EXIST_IN_APP_STORE=47 APP_ID_DOESNT_EXIST_IN_APP_STORE value
                             * @property {number} INVALID_FINAL_URL=48 INVALID_FINAL_URL value
                             * @property {number} INVALID_TRACKING_URL=49 INVALID_TRACKING_URL value
                             * @property {number} INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL=50 INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL value
                             * @property {number} LIST_TOO_SHORT=51 LIST_TOO_SHORT value
                             * @property {number} INVALID_USER_ACTION=52 INVALID_USER_ACTION value
                             * @property {number} INVALID_TYPE_NAME=53 INVALID_TYPE_NAME value
                             * @property {number} INVALID_EVENT_CHANGE_STATUS=54 INVALID_EVENT_CHANGE_STATUS value
                             * @property {number} INVALID_SNIPPETS_HEADER=55 INVALID_SNIPPETS_HEADER value
                             * @property {number} INVALID_ANDROID_APP_LINK=56 INVALID_ANDROID_APP_LINK value
                             * @property {number} NUMBER_TYPE_WITH_CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY=57 NUMBER_TYPE_WITH_CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY value
                             * @property {number} RESERVED_KEYWORD_OTHER=58 RESERVED_KEYWORD_OTHER value
                             * @property {number} DUPLICATE_OPTION_LABELS=59 DUPLICATE_OPTION_LABELS value
                             * @property {number} DUPLICATE_OPTION_PREFILLS=60 DUPLICATE_OPTION_PREFILLS value
                             * @property {number} UNEQUAL_LIST_LENGTHS=61 UNEQUAL_LIST_LENGTHS value
                             * @property {number} INCONSISTENT_CURRENCY_CODES=62 INCONSISTENT_CURRENCY_CODES value
                             * @property {number} PRICE_EXTENSION_HAS_DUPLICATED_HEADERS=63 PRICE_EXTENSION_HAS_DUPLICATED_HEADERS value
                             * @property {number} ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION=64 ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION value
                             * @property {number} PRICE_EXTENSION_HAS_TOO_FEW_ITEMS=65 PRICE_EXTENSION_HAS_TOO_FEW_ITEMS value
                             * @property {number} UNSUPPORTED_VALUE=66 UNSUPPORTED_VALUE value
                             * @property {number} INVALID_FINAL_MOBILE_URL=67 INVALID_FINAL_MOBILE_URL value
                             * @property {number} INVALID_KEYWORDLESS_AD_RULE_LABEL=68 INVALID_KEYWORDLESS_AD_RULE_LABEL value
                             * @property {number} VALUE_TRACK_PARAMETER_NOT_SUPPORTED=69 VALUE_TRACK_PARAMETER_NOT_SUPPORTED value
                             * @property {number} UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE=70 UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE value
                             * @property {number} INVALID_IOS_APP_LINK=71 INVALID_IOS_APP_LINK value
                             * @property {number} MISSING_IOS_APP_LINK_OR_IOS_APP_STORE_ID=72 MISSING_IOS_APP_LINK_OR_IOS_APP_STORE_ID value
                             * @property {number} PROMOTION_INVALID_TIME=73 PROMOTION_INVALID_TIME value
                             * @property {number} PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF=74 PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF value
                             * @property {number} PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT=75 PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT value
                             * @property {number} TOO_MANY_DECIMAL_PLACES_SPECIFIED=76 TOO_MANY_DECIMAL_PLACES_SPECIFIED value
                             * @property {number} AD_CUSTOMIZERS_NOT_ALLOWED=77 AD_CUSTOMIZERS_NOT_ALLOWED value
                             * @property {number} INVALID_LANGUAGE_CODE=78 INVALID_LANGUAGE_CODE value
                             * @property {number} UNSUPPORTED_LANGUAGE=79 UNSUPPORTED_LANGUAGE value
                             * @property {number} IF_FUNCTION_NOT_ALLOWED=80 IF_FUNCTION_NOT_ALLOWED value
                             * @property {number} INVALID_FINAL_URL_SUFFIX=81 INVALID_FINAL_URL_SUFFIX value
                             * @property {number} INVALID_TAG_IN_FINAL_URL_SUFFIX=82 INVALID_TAG_IN_FINAL_URL_SUFFIX value
                             * @property {number} INVALID_FINAL_URL_SUFFIX_FORMAT=83 INVALID_FINAL_URL_SUFFIX_FORMAT value
                             * @property {number} CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED=84 CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED value
                             * @property {number} ONLY_ONE_DELIVERY_OPTION_IS_ALLOWED=85 ONLY_ONE_DELIVERY_OPTION_IS_ALLOWED value
                             * @property {number} NO_DELIVERY_OPTION_IS_SET=86 NO_DELIVERY_OPTION_IS_SET value
                             * @property {number} INVALID_CONVERSION_REPORTING_STATE=87 INVALID_CONVERSION_REPORTING_STATE value
                             * @property {number} IMAGE_SIZE_WRONG=88 IMAGE_SIZE_WRONG value
                             * @property {number} EMAIL_DELIVERY_NOT_AVAILABLE_IN_COUNTRY=89 EMAIL_DELIVERY_NOT_AVAILABLE_IN_COUNTRY value
                             * @property {number} AUTO_REPLY_NOT_AVAILABLE_IN_COUNTRY=90 AUTO_REPLY_NOT_AVAILABLE_IN_COUNTRY value
                             * @property {number} INVALID_LATITUDE_VALUE=91 INVALID_LATITUDE_VALUE value
                             * @property {number} INVALID_LONGITUDE_VALUE=92 INVALID_LONGITUDE_VALUE value
                             * @property {number} TOO_MANY_LABELS=93 TOO_MANY_LABELS value
                             * @property {number} INVALID_IMAGE_URL=94 INVALID_IMAGE_URL value
                             * @property {number} MISSING_LATITUDE_VALUE=95 MISSING_LATITUDE_VALUE value
                             * @property {number} MISSING_LONGITUDE_VALUE=96 MISSING_LONGITUDE_VALUE value
                             * @property {number} ADDRESS_NOT_FOUND=97 ADDRESS_NOT_FOUND value
                             * @property {number} ADDRESS_NOT_TARGETABLE=98 ADDRESS_NOT_TARGETABLE value
                             * @property {number} INVALID_ASSET_ID=100 INVALID_ASSET_ID value
                             * @property {number} INCOMPATIBLE_ASSET_TYPE=101 INCOMPATIBLE_ASSET_TYPE value
                             * @property {number} IMAGE_ERROR_UNEXPECTED_SIZE=102 IMAGE_ERROR_UNEXPECTED_SIZE value
                             * @property {number} IMAGE_ERROR_ASPECT_RATIO_NOT_ALLOWED=103 IMAGE_ERROR_ASPECT_RATIO_NOT_ALLOWED value
                             * @property {number} IMAGE_ERROR_FILE_TOO_LARGE=104 IMAGE_ERROR_FILE_TOO_LARGE value
                             * @property {number} IMAGE_ERROR_FORMAT_NOT_ALLOWED=105 IMAGE_ERROR_FORMAT_NOT_ALLOWED value
                             * @property {number} IMAGE_ERROR_CONSTRAINTS_VIOLATED=106 IMAGE_ERROR_CONSTRAINTS_VIOLATED value
                             * @property {number} IMAGE_ERROR_SERVER_ERROR=107 IMAGE_ERROR_SERVER_ERROR value
                             */
                            FeedItemValidationErrorEnum.FeedItemValidationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "STRING_TOO_SHORT"] = 2;
                                values[valuesById[3] = "STRING_TOO_LONG"] = 3;
                                values[valuesById[4] = "VALUE_NOT_SPECIFIED"] = 4;
                                values[valuesById[5] = "INVALID_DOMESTIC_PHONE_NUMBER_FORMAT"] = 5;
                                values[valuesById[6] = "INVALID_PHONE_NUMBER"] = 6;
                                values[valuesById[7] = "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY"] = 7;
                                values[valuesById[8] = "PREMIUM_RATE_NUMBER_NOT_ALLOWED"] = 8;
                                values[valuesById[9] = "DISALLOWED_NUMBER_TYPE"] = 9;
                                values[valuesById[10] = "VALUE_OUT_OF_RANGE"] = 10;
                                values[valuesById[11] = "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY"] = 11;
                                values[valuesById[99] = "CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING"] = 99;
                                values[valuesById[13] = "INVALID_COUNTRY_CODE"] = 13;
                                values[valuesById[14] = "INVALID_APP_ID"] = 14;
                                values[valuesById[15] = "MISSING_ATTRIBUTES_FOR_FIELDS"] = 15;
                                values[valuesById[16] = "INVALID_TYPE_ID"] = 16;
                                values[valuesById[17] = "INVALID_EMAIL_ADDRESS"] = 17;
                                values[valuesById[18] = "INVALID_HTTPS_URL"] = 18;
                                values[valuesById[19] = "MISSING_DELIVERY_ADDRESS"] = 19;
                                values[valuesById[20] = "START_DATE_AFTER_END_DATE"] = 20;
                                values[valuesById[21] = "MISSING_FEED_ITEM_START_TIME"] = 21;
                                values[valuesById[22] = "MISSING_FEED_ITEM_END_TIME"] = 22;
                                values[valuesById[23] = "MISSING_FEED_ITEM_ID"] = 23;
                                values[valuesById[24] = "VANITY_PHONE_NUMBER_NOT_ALLOWED"] = 24;
                                values[valuesById[25] = "INVALID_REVIEW_EXTENSION_SNIPPET"] = 25;
                                values[valuesById[26] = "INVALID_NUMBER_FORMAT"] = 26;
                                values[valuesById[27] = "INVALID_DATE_FORMAT"] = 27;
                                values[valuesById[28] = "INVALID_PRICE_FORMAT"] = 28;
                                values[valuesById[29] = "UNKNOWN_PLACEHOLDER_FIELD"] = 29;
                                values[valuesById[30] = "MISSING_ENHANCED_SITELINK_DESCRIPTION_LINE"] = 30;
                                values[valuesById[31] = "REVIEW_EXTENSION_SOURCE_INELIGIBLE"] = 31;
                                values[valuesById[32] = "HYPHENS_IN_REVIEW_EXTENSION_SNIPPET"] = 32;
                                values[valuesById[33] = "DOUBLE_QUOTES_IN_REVIEW_EXTENSION_SNIPPET"] = 33;
                                values[valuesById[34] = "QUOTES_IN_REVIEW_EXTENSION_SNIPPET"] = 34;
                                values[valuesById[35] = "INVALID_FORM_ENCODED_PARAMS"] = 35;
                                values[valuesById[36] = "INVALID_URL_PARAMETER_NAME"] = 36;
                                values[valuesById[37] = "NO_GEOCODING_RESULT"] = 37;
                                values[valuesById[38] = "SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT"] = 38;
                                values[valuesById[39] = "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED"] = 39;
                                values[valuesById[40] = "INVALID_PLACEHOLDER_FIELD_ID"] = 40;
                                values[valuesById[41] = "INVALID_URL_TAG"] = 41;
                                values[valuesById[42] = "LIST_TOO_LONG"] = 42;
                                values[valuesById[43] = "INVALID_ATTRIBUTES_COMBINATION"] = 43;
                                values[valuesById[44] = "DUPLICATE_VALUES"] = 44;
                                values[valuesById[45] = "INVALID_CALL_CONVERSION_ACTION_ID"] = 45;
                                values[valuesById[46] = "CANNOT_SET_WITHOUT_FINAL_URLS"] = 46;
                                values[valuesById[47] = "APP_ID_DOESNT_EXIST_IN_APP_STORE"] = 47;
                                values[valuesById[48] = "INVALID_FINAL_URL"] = 48;
                                values[valuesById[49] = "INVALID_TRACKING_URL"] = 49;
                                values[valuesById[50] = "INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL"] = 50;
                                values[valuesById[51] = "LIST_TOO_SHORT"] = 51;
                                values[valuesById[52] = "INVALID_USER_ACTION"] = 52;
                                values[valuesById[53] = "INVALID_TYPE_NAME"] = 53;
                                values[valuesById[54] = "INVALID_EVENT_CHANGE_STATUS"] = 54;
                                values[valuesById[55] = "INVALID_SNIPPETS_HEADER"] = 55;
                                values[valuesById[56] = "INVALID_ANDROID_APP_LINK"] = 56;
                                values[valuesById[57] = "NUMBER_TYPE_WITH_CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY"] = 57;
                                values[valuesById[58] = "RESERVED_KEYWORD_OTHER"] = 58;
                                values[valuesById[59] = "DUPLICATE_OPTION_LABELS"] = 59;
                                values[valuesById[60] = "DUPLICATE_OPTION_PREFILLS"] = 60;
                                values[valuesById[61] = "UNEQUAL_LIST_LENGTHS"] = 61;
                                values[valuesById[62] = "INCONSISTENT_CURRENCY_CODES"] = 62;
                                values[valuesById[63] = "PRICE_EXTENSION_HAS_DUPLICATED_HEADERS"] = 63;
                                values[valuesById[64] = "ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION"] = 64;
                                values[valuesById[65] = "PRICE_EXTENSION_HAS_TOO_FEW_ITEMS"] = 65;
                                values[valuesById[66] = "UNSUPPORTED_VALUE"] = 66;
                                values[valuesById[67] = "INVALID_FINAL_MOBILE_URL"] = 67;
                                values[valuesById[68] = "INVALID_KEYWORDLESS_AD_RULE_LABEL"] = 68;
                                values[valuesById[69] = "VALUE_TRACK_PARAMETER_NOT_SUPPORTED"] = 69;
                                values[valuesById[70] = "UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE"] = 70;
                                values[valuesById[71] = "INVALID_IOS_APP_LINK"] = 71;
                                values[valuesById[72] = "MISSING_IOS_APP_LINK_OR_IOS_APP_STORE_ID"] = 72;
                                values[valuesById[73] = "PROMOTION_INVALID_TIME"] = 73;
                                values[valuesById[74] = "PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF"] = 74;
                                values[valuesById[75] = "PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT"] = 75;
                                values[valuesById[76] = "TOO_MANY_DECIMAL_PLACES_SPECIFIED"] = 76;
                                values[valuesById[77] = "AD_CUSTOMIZERS_NOT_ALLOWED"] = 77;
                                values[valuesById[78] = "INVALID_LANGUAGE_CODE"] = 78;
                                values[valuesById[79] = "UNSUPPORTED_LANGUAGE"] = 79;
                                values[valuesById[80] = "IF_FUNCTION_NOT_ALLOWED"] = 80;
                                values[valuesById[81] = "INVALID_FINAL_URL_SUFFIX"] = 81;
                                values[valuesById[82] = "INVALID_TAG_IN_FINAL_URL_SUFFIX"] = 82;
                                values[valuesById[83] = "INVALID_FINAL_URL_SUFFIX_FORMAT"] = 83;
                                values[valuesById[84] = "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED"] = 84;
                                values[valuesById[85] = "ONLY_ONE_DELIVERY_OPTION_IS_ALLOWED"] = 85;
                                values[valuesById[86] = "NO_DELIVERY_OPTION_IS_SET"] = 86;
                                values[valuesById[87] = "INVALID_CONVERSION_REPORTING_STATE"] = 87;
                                values[valuesById[88] = "IMAGE_SIZE_WRONG"] = 88;
                                values[valuesById[89] = "EMAIL_DELIVERY_NOT_AVAILABLE_IN_COUNTRY"] = 89;
                                values[valuesById[90] = "AUTO_REPLY_NOT_AVAILABLE_IN_COUNTRY"] = 90;
                                values[valuesById[91] = "INVALID_LATITUDE_VALUE"] = 91;
                                values[valuesById[92] = "INVALID_LONGITUDE_VALUE"] = 92;
                                values[valuesById[93] = "TOO_MANY_LABELS"] = 93;
                                values[valuesById[94] = "INVALID_IMAGE_URL"] = 94;
                                values[valuesById[95] = "MISSING_LATITUDE_VALUE"] = 95;
                                values[valuesById[96] = "MISSING_LONGITUDE_VALUE"] = 96;
                                values[valuesById[97] = "ADDRESS_NOT_FOUND"] = 97;
                                values[valuesById[98] = "ADDRESS_NOT_TARGETABLE"] = 98;
                                values[valuesById[100] = "INVALID_ASSET_ID"] = 100;
                                values[valuesById[101] = "INCOMPATIBLE_ASSET_TYPE"] = 101;
                                values[valuesById[102] = "IMAGE_ERROR_UNEXPECTED_SIZE"] = 102;
                                values[valuesById[103] = "IMAGE_ERROR_ASPECT_RATIO_NOT_ALLOWED"] = 103;
                                values[valuesById[104] = "IMAGE_ERROR_FILE_TOO_LARGE"] = 104;
                                values[valuesById[105] = "IMAGE_ERROR_FORMAT_NOT_ALLOWED"] = 105;
                                values[valuesById[106] = "IMAGE_ERROR_CONSTRAINTS_VIOLATED"] = 106;
                                values[valuesById[107] = "IMAGE_ERROR_SERVER_ERROR"] = 107;
                                return values;
                            })();
    
                            return FeedItemValidationErrorEnum;
                        })();
    
                        errors.FeedMappingErrorEnum = (function() {
    
                            /**
                             * Properties of a FeedMappingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFeedMappingErrorEnum
                             */
    
                            /**
                             * Constructs a new FeedMappingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FeedMappingErrorEnum.
                             * @implements IFeedMappingErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFeedMappingErrorEnum=} [p] Properties to set
                             */
                            function FeedMappingErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FeedMappingErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedMappingErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FeedMappingErrorEnum} FeedMappingErrorEnum instance
                             */
                            FeedMappingErrorEnum.create = function create(properties) {
                                return new FeedMappingErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FeedMappingErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FeedMappingErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedMappingErrorEnum} m FeedMappingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedMappingErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FeedMappingErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFeedMappingErrorEnum} message FeedMappingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedMappingErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FeedMappingErrorEnum} FeedMappingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FeedMappingErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedMappingErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FeedMappingErrorEnum} FeedMappingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedMappingErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedMappingErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FeedMappingErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FeedMappingErrorEnum} FeedMappingErrorEnum
                             */
                            FeedMappingErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FeedMappingErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FeedMappingErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FeedMappingErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FeedMappingErrorEnum} m FeedMappingErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedMappingErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FeedMappingErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FeedMappingErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedMappingErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FeedMappingError enum.
                             * @name google.ads.googleads.v11.errors.FeedMappingErrorEnum.FeedMappingError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_PLACEHOLDER_FIELD=2 INVALID_PLACEHOLDER_FIELD value
                             * @property {number} INVALID_CRITERION_FIELD=3 INVALID_CRITERION_FIELD value
                             * @property {number} INVALID_PLACEHOLDER_TYPE=4 INVALID_PLACEHOLDER_TYPE value
                             * @property {number} INVALID_CRITERION_TYPE=5 INVALID_CRITERION_TYPE value
                             * @property {number} NO_ATTRIBUTE_FIELD_MAPPINGS=7 NO_ATTRIBUTE_FIELD_MAPPINGS value
                             * @property {number} FEED_ATTRIBUTE_TYPE_MISMATCH=8 FEED_ATTRIBUTE_TYPE_MISMATCH value
                             * @property {number} CANNOT_OPERATE_ON_MAPPINGS_FOR_SYSTEM_GENERATED_FEED=9 CANNOT_OPERATE_ON_MAPPINGS_FOR_SYSTEM_GENERATED_FEED value
                             * @property {number} MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_TYPE=10 MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_TYPE value
                             * @property {number} MULTIPLE_MAPPINGS_FOR_CRITERION_TYPE=11 MULTIPLE_MAPPINGS_FOR_CRITERION_TYPE value
                             * @property {number} MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_FIELD=12 MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_FIELD value
                             * @property {number} MULTIPLE_MAPPINGS_FOR_CRITERION_FIELD=13 MULTIPLE_MAPPINGS_FOR_CRITERION_FIELD value
                             * @property {number} UNEXPECTED_ATTRIBUTE_FIELD_MAPPINGS=14 UNEXPECTED_ATTRIBUTE_FIELD_MAPPINGS value
                             * @property {number} LOCATION_PLACEHOLDER_ONLY_FOR_PLACES_FEEDS=15 LOCATION_PLACEHOLDER_ONLY_FOR_PLACES_FEEDS value
                             * @property {number} CANNOT_MODIFY_MAPPINGS_FOR_TYPED_FEED=16 CANNOT_MODIFY_MAPPINGS_FOR_TYPED_FEED value
                             * @property {number} INVALID_PLACEHOLDER_TYPE_FOR_NON_SYSTEM_GENERATED_FEED=17 INVALID_PLACEHOLDER_TYPE_FOR_NON_SYSTEM_GENERATED_FEED value
                             * @property {number} INVALID_PLACEHOLDER_TYPE_FOR_SYSTEM_GENERATED_FEED_TYPE=18 INVALID_PLACEHOLDER_TYPE_FOR_SYSTEM_GENERATED_FEED_TYPE value
                             * @property {number} ATTRIBUTE_FIELD_MAPPING_MISSING_FIELD=19 ATTRIBUTE_FIELD_MAPPING_MISSING_FIELD value
                             */
                            FeedMappingErrorEnum.FeedMappingError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_PLACEHOLDER_FIELD"] = 2;
                                values[valuesById[3] = "INVALID_CRITERION_FIELD"] = 3;
                                values[valuesById[4] = "INVALID_PLACEHOLDER_TYPE"] = 4;
                                values[valuesById[5] = "INVALID_CRITERION_TYPE"] = 5;
                                values[valuesById[7] = "NO_ATTRIBUTE_FIELD_MAPPINGS"] = 7;
                                values[valuesById[8] = "FEED_ATTRIBUTE_TYPE_MISMATCH"] = 8;
                                values[valuesById[9] = "CANNOT_OPERATE_ON_MAPPINGS_FOR_SYSTEM_GENERATED_FEED"] = 9;
                                values[valuesById[10] = "MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_TYPE"] = 10;
                                values[valuesById[11] = "MULTIPLE_MAPPINGS_FOR_CRITERION_TYPE"] = 11;
                                values[valuesById[12] = "MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_FIELD"] = 12;
                                values[valuesById[13] = "MULTIPLE_MAPPINGS_FOR_CRITERION_FIELD"] = 13;
                                values[valuesById[14] = "UNEXPECTED_ATTRIBUTE_FIELD_MAPPINGS"] = 14;
                                values[valuesById[15] = "LOCATION_PLACEHOLDER_ONLY_FOR_PLACES_FEEDS"] = 15;
                                values[valuesById[16] = "CANNOT_MODIFY_MAPPINGS_FOR_TYPED_FEED"] = 16;
                                values[valuesById[17] = "INVALID_PLACEHOLDER_TYPE_FOR_NON_SYSTEM_GENERATED_FEED"] = 17;
                                values[valuesById[18] = "INVALID_PLACEHOLDER_TYPE_FOR_SYSTEM_GENERATED_FEED_TYPE"] = 18;
                                values[valuesById[19] = "ATTRIBUTE_FIELD_MAPPING_MISSING_FIELD"] = 19;
                                return values;
                            })();
    
                            return FeedMappingErrorEnum;
                        })();
    
                        errors.FieldErrorEnum = (function() {
    
                            /**
                             * Properties of a FieldErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFieldErrorEnum
                             */
    
                            /**
                             * Constructs a new FieldErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FieldErrorEnum.
                             * @implements IFieldErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFieldErrorEnum=} [p] Properties to set
                             */
                            function FieldErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FieldErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFieldErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FieldErrorEnum} FieldErrorEnum instance
                             */
                            FieldErrorEnum.create = function create(properties) {
                                return new FieldErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FieldErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FieldErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFieldErrorEnum} m FieldErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FieldErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FieldErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FieldErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFieldErrorEnum} message FieldErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FieldErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FieldErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FieldErrorEnum} FieldErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FieldErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FieldErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FieldErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FieldErrorEnum} FieldErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FieldErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FieldErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FieldErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FieldErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FieldErrorEnum} FieldErrorEnum
                             */
                            FieldErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FieldErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FieldErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FieldErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FieldErrorEnum} m FieldErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FieldErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FieldErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FieldErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FieldErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FieldError enum.
                             * @name google.ads.googleads.v11.errors.FieldErrorEnum.FieldError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} REQUIRED=2 REQUIRED value
                             * @property {number} IMMUTABLE_FIELD=3 IMMUTABLE_FIELD value
                             * @property {number} INVALID_VALUE=4 INVALID_VALUE value
                             * @property {number} VALUE_MUST_BE_UNSET=5 VALUE_MUST_BE_UNSET value
                             * @property {number} REQUIRED_NONEMPTY_LIST=6 REQUIRED_NONEMPTY_LIST value
                             * @property {number} FIELD_CANNOT_BE_CLEARED=7 FIELD_CANNOT_BE_CLEARED value
                             * @property {number} BLOCKED_VALUE=9 BLOCKED_VALUE value
                             * @property {number} FIELD_CAN_ONLY_BE_CLEARED=10 FIELD_CAN_ONLY_BE_CLEARED value
                             */
                            FieldErrorEnum.FieldError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "REQUIRED"] = 2;
                                values[valuesById[3] = "IMMUTABLE_FIELD"] = 3;
                                values[valuesById[4] = "INVALID_VALUE"] = 4;
                                values[valuesById[5] = "VALUE_MUST_BE_UNSET"] = 5;
                                values[valuesById[6] = "REQUIRED_NONEMPTY_LIST"] = 6;
                                values[valuesById[7] = "FIELD_CANNOT_BE_CLEARED"] = 7;
                                values[valuesById[9] = "BLOCKED_VALUE"] = 9;
                                values[valuesById[10] = "FIELD_CAN_ONLY_BE_CLEARED"] = 10;
                                return values;
                            })();
    
                            return FieldErrorEnum;
                        })();
    
                        errors.FieldMaskErrorEnum = (function() {
    
                            /**
                             * Properties of a FieldMaskErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFieldMaskErrorEnum
                             */
    
                            /**
                             * Constructs a new FieldMaskErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FieldMaskErrorEnum.
                             * @implements IFieldMaskErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFieldMaskErrorEnum=} [p] Properties to set
                             */
                            function FieldMaskErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FieldMaskErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFieldMaskErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FieldMaskErrorEnum} FieldMaskErrorEnum instance
                             */
                            FieldMaskErrorEnum.create = function create(properties) {
                                return new FieldMaskErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FieldMaskErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FieldMaskErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFieldMaskErrorEnum} m FieldMaskErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FieldMaskErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FieldMaskErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FieldMaskErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFieldMaskErrorEnum} message FieldMaskErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FieldMaskErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FieldMaskErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FieldMaskErrorEnum} FieldMaskErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FieldMaskErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FieldMaskErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FieldMaskErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FieldMaskErrorEnum} FieldMaskErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FieldMaskErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FieldMaskErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FieldMaskErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FieldMaskErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FieldMaskErrorEnum} FieldMaskErrorEnum
                             */
                            FieldMaskErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FieldMaskErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FieldMaskErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FieldMaskErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FieldMaskErrorEnum} m FieldMaskErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FieldMaskErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FieldMaskErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FieldMaskErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FieldMaskErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FieldMaskError enum.
                             * @name google.ads.googleads.v11.errors.FieldMaskErrorEnum.FieldMaskError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FIELD_MASK_MISSING=5 FIELD_MASK_MISSING value
                             * @property {number} FIELD_MASK_NOT_ALLOWED=4 FIELD_MASK_NOT_ALLOWED value
                             * @property {number} FIELD_NOT_FOUND=2 FIELD_NOT_FOUND value
                             * @property {number} FIELD_HAS_SUBFIELDS=3 FIELD_HAS_SUBFIELDS value
                             */
                            FieldMaskErrorEnum.FieldMaskError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[5] = "FIELD_MASK_MISSING"] = 5;
                                values[valuesById[4] = "FIELD_MASK_NOT_ALLOWED"] = 4;
                                values[valuesById[2] = "FIELD_NOT_FOUND"] = 2;
                                values[valuesById[3] = "FIELD_HAS_SUBFIELDS"] = 3;
                                return values;
                            })();
    
                            return FieldMaskErrorEnum;
                        })();
    
                        errors.FunctionErrorEnum = (function() {
    
                            /**
                             * Properties of a FunctionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFunctionErrorEnum
                             */
    
                            /**
                             * Constructs a new FunctionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FunctionErrorEnum.
                             * @implements IFunctionErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFunctionErrorEnum=} [p] Properties to set
                             */
                            function FunctionErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FunctionErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFunctionErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FunctionErrorEnum} FunctionErrorEnum instance
                             */
                            FunctionErrorEnum.create = function create(properties) {
                                return new FunctionErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FunctionErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FunctionErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFunctionErrorEnum} m FunctionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FunctionErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FunctionErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FunctionErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFunctionErrorEnum} message FunctionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FunctionErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FunctionErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FunctionErrorEnum} FunctionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FunctionErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FunctionErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FunctionErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FunctionErrorEnum} FunctionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FunctionErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FunctionErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FunctionErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FunctionErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FunctionErrorEnum} FunctionErrorEnum
                             */
                            FunctionErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FunctionErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FunctionErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FunctionErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FunctionErrorEnum} m FunctionErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FunctionErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FunctionErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FunctionErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FunctionErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FunctionError enum.
                             * @name google.ads.googleads.v11.errors.FunctionErrorEnum.FunctionError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_FUNCTION_FORMAT=2 INVALID_FUNCTION_FORMAT value
                             * @property {number} DATA_TYPE_MISMATCH=3 DATA_TYPE_MISMATCH value
                             * @property {number} INVALID_CONJUNCTION_OPERANDS=4 INVALID_CONJUNCTION_OPERANDS value
                             * @property {number} INVALID_NUMBER_OF_OPERANDS=5 INVALID_NUMBER_OF_OPERANDS value
                             * @property {number} INVALID_OPERAND_TYPE=6 INVALID_OPERAND_TYPE value
                             * @property {number} INVALID_OPERATOR=7 INVALID_OPERATOR value
                             * @property {number} INVALID_REQUEST_CONTEXT_TYPE=8 INVALID_REQUEST_CONTEXT_TYPE value
                             * @property {number} INVALID_FUNCTION_FOR_CALL_PLACEHOLDER=9 INVALID_FUNCTION_FOR_CALL_PLACEHOLDER value
                             * @property {number} INVALID_FUNCTION_FOR_PLACEHOLDER=10 INVALID_FUNCTION_FOR_PLACEHOLDER value
                             * @property {number} INVALID_OPERAND=11 INVALID_OPERAND value
                             * @property {number} MISSING_CONSTANT_OPERAND_VALUE=12 MISSING_CONSTANT_OPERAND_VALUE value
                             * @property {number} INVALID_CONSTANT_OPERAND_VALUE=13 INVALID_CONSTANT_OPERAND_VALUE value
                             * @property {number} INVALID_NESTING=14 INVALID_NESTING value
                             * @property {number} MULTIPLE_FEED_IDS_NOT_SUPPORTED=15 MULTIPLE_FEED_IDS_NOT_SUPPORTED value
                             * @property {number} INVALID_FUNCTION_FOR_FEED_WITH_FIXED_SCHEMA=16 INVALID_FUNCTION_FOR_FEED_WITH_FIXED_SCHEMA value
                             * @property {number} INVALID_ATTRIBUTE_NAME=17 INVALID_ATTRIBUTE_NAME value
                             */
                            FunctionErrorEnum.FunctionError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_FUNCTION_FORMAT"] = 2;
                                values[valuesById[3] = "DATA_TYPE_MISMATCH"] = 3;
                                values[valuesById[4] = "INVALID_CONJUNCTION_OPERANDS"] = 4;
                                values[valuesById[5] = "INVALID_NUMBER_OF_OPERANDS"] = 5;
                                values[valuesById[6] = "INVALID_OPERAND_TYPE"] = 6;
                                values[valuesById[7] = "INVALID_OPERATOR"] = 7;
                                values[valuesById[8] = "INVALID_REQUEST_CONTEXT_TYPE"] = 8;
                                values[valuesById[9] = "INVALID_FUNCTION_FOR_CALL_PLACEHOLDER"] = 9;
                                values[valuesById[10] = "INVALID_FUNCTION_FOR_PLACEHOLDER"] = 10;
                                values[valuesById[11] = "INVALID_OPERAND"] = 11;
                                values[valuesById[12] = "MISSING_CONSTANT_OPERAND_VALUE"] = 12;
                                values[valuesById[13] = "INVALID_CONSTANT_OPERAND_VALUE"] = 13;
                                values[valuesById[14] = "INVALID_NESTING"] = 14;
                                values[valuesById[15] = "MULTIPLE_FEED_IDS_NOT_SUPPORTED"] = 15;
                                values[valuesById[16] = "INVALID_FUNCTION_FOR_FEED_WITH_FIXED_SCHEMA"] = 16;
                                values[valuesById[17] = "INVALID_ATTRIBUTE_NAME"] = 17;
                                return values;
                            })();
    
                            return FunctionErrorEnum;
                        })();
    
                        errors.FunctionParsingErrorEnum = (function() {
    
                            /**
                             * Properties of a FunctionParsingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IFunctionParsingErrorEnum
                             */
    
                            /**
                             * Constructs a new FunctionParsingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a FunctionParsingErrorEnum.
                             * @implements IFunctionParsingErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IFunctionParsingErrorEnum=} [p] Properties to set
                             */
                            function FunctionParsingErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new FunctionParsingErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFunctionParsingErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.FunctionParsingErrorEnum} FunctionParsingErrorEnum instance
                             */
                            FunctionParsingErrorEnum.create = function create(properties) {
                                return new FunctionParsingErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified FunctionParsingErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.FunctionParsingErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFunctionParsingErrorEnum} m FunctionParsingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FunctionParsingErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FunctionParsingErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.FunctionParsingErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IFunctionParsingErrorEnum} message FunctionParsingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FunctionParsingErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FunctionParsingErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.FunctionParsingErrorEnum} FunctionParsingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FunctionParsingErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.FunctionParsingErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FunctionParsingErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.FunctionParsingErrorEnum} FunctionParsingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FunctionParsingErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FunctionParsingErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FunctionParsingErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a FunctionParsingErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.FunctionParsingErrorEnum} FunctionParsingErrorEnum
                             */
                            FunctionParsingErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.FunctionParsingErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.FunctionParsingErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a FunctionParsingErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.FunctionParsingErrorEnum} m FunctionParsingErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FunctionParsingErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this FunctionParsingErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.FunctionParsingErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FunctionParsingErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * FunctionParsingError enum.
                             * @name google.ads.googleads.v11.errors.FunctionParsingErrorEnum.FunctionParsingError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NO_MORE_INPUT=2 NO_MORE_INPUT value
                             * @property {number} EXPECTED_CHARACTER=3 EXPECTED_CHARACTER value
                             * @property {number} UNEXPECTED_SEPARATOR=4 UNEXPECTED_SEPARATOR value
                             * @property {number} UNMATCHED_LEFT_BRACKET=5 UNMATCHED_LEFT_BRACKET value
                             * @property {number} UNMATCHED_RIGHT_BRACKET=6 UNMATCHED_RIGHT_BRACKET value
                             * @property {number} TOO_MANY_NESTED_FUNCTIONS=7 TOO_MANY_NESTED_FUNCTIONS value
                             * @property {number} MISSING_RIGHT_HAND_OPERAND=8 MISSING_RIGHT_HAND_OPERAND value
                             * @property {number} INVALID_OPERATOR_NAME=9 INVALID_OPERATOR_NAME value
                             * @property {number} FEED_ATTRIBUTE_OPERAND_ARGUMENT_NOT_INTEGER=10 FEED_ATTRIBUTE_OPERAND_ARGUMENT_NOT_INTEGER value
                             * @property {number} NO_OPERANDS=11 NO_OPERANDS value
                             * @property {number} TOO_MANY_OPERANDS=12 TOO_MANY_OPERANDS value
                             */
                            FunctionParsingErrorEnum.FunctionParsingError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NO_MORE_INPUT"] = 2;
                                values[valuesById[3] = "EXPECTED_CHARACTER"] = 3;
                                values[valuesById[4] = "UNEXPECTED_SEPARATOR"] = 4;
                                values[valuesById[5] = "UNMATCHED_LEFT_BRACKET"] = 5;
                                values[valuesById[6] = "UNMATCHED_RIGHT_BRACKET"] = 6;
                                values[valuesById[7] = "TOO_MANY_NESTED_FUNCTIONS"] = 7;
                                values[valuesById[8] = "MISSING_RIGHT_HAND_OPERAND"] = 8;
                                values[valuesById[9] = "INVALID_OPERATOR_NAME"] = 9;
                                values[valuesById[10] = "FEED_ATTRIBUTE_OPERAND_ARGUMENT_NOT_INTEGER"] = 10;
                                values[valuesById[11] = "NO_OPERANDS"] = 11;
                                values[valuesById[12] = "TOO_MANY_OPERANDS"] = 12;
                                return values;
                            })();
    
                            return FunctionParsingErrorEnum;
                        })();
    
                        errors.GeoTargetConstantSuggestionErrorEnum = (function() {
    
                            /**
                             * Properties of a GeoTargetConstantSuggestionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IGeoTargetConstantSuggestionErrorEnum
                             */
    
                            /**
                             * Constructs a new GeoTargetConstantSuggestionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a GeoTargetConstantSuggestionErrorEnum.
                             * @implements IGeoTargetConstantSuggestionErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IGeoTargetConstantSuggestionErrorEnum=} [p] Properties to set
                             */
                            function GeoTargetConstantSuggestionErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new GeoTargetConstantSuggestionErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGeoTargetConstantSuggestionErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum} GeoTargetConstantSuggestionErrorEnum instance
                             */
                            GeoTargetConstantSuggestionErrorEnum.create = function create(properties) {
                                return new GeoTargetConstantSuggestionErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstantSuggestionErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGeoTargetConstantSuggestionErrorEnum} m GeoTargetConstantSuggestionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstantSuggestionErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstantSuggestionErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IGeoTargetConstantSuggestionErrorEnum} message GeoTargetConstantSuggestionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstantSuggestionErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeoTargetConstantSuggestionErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum} GeoTargetConstantSuggestionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstantSuggestionErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeoTargetConstantSuggestionErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum} GeoTargetConstantSuggestionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstantSuggestionErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeoTargetConstantSuggestionErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeoTargetConstantSuggestionErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a GeoTargetConstantSuggestionErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum} GeoTargetConstantSuggestionErrorEnum
                             */
                            GeoTargetConstantSuggestionErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a GeoTargetConstantSuggestionErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum} m GeoTargetConstantSuggestionErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeoTargetConstantSuggestionErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this GeoTargetConstantSuggestionErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeoTargetConstantSuggestionErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * GeoTargetConstantSuggestionError enum.
                             * @name google.ads.googleads.v11.errors.GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LOCATION_NAME_SIZE_LIMIT=2 LOCATION_NAME_SIZE_LIMIT value
                             * @property {number} LOCATION_NAME_LIMIT=3 LOCATION_NAME_LIMIT value
                             * @property {number} INVALID_COUNTRY_CODE=4 INVALID_COUNTRY_CODE value
                             * @property {number} REQUEST_PARAMETERS_UNSET=5 REQUEST_PARAMETERS_UNSET value
                             */
                            GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LOCATION_NAME_SIZE_LIMIT"] = 2;
                                values[valuesById[3] = "LOCATION_NAME_LIMIT"] = 3;
                                values[valuesById[4] = "INVALID_COUNTRY_CODE"] = 4;
                                values[valuesById[5] = "REQUEST_PARAMETERS_UNSET"] = 5;
                                return values;
                            })();
    
                            return GeoTargetConstantSuggestionErrorEnum;
                        })();
    
                        errors.HeaderErrorEnum = (function() {
    
                            /**
                             * Properties of a HeaderErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IHeaderErrorEnum
                             */
    
                            /**
                             * Constructs a new HeaderErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a HeaderErrorEnum.
                             * @implements IHeaderErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IHeaderErrorEnum=} [p] Properties to set
                             */
                            function HeaderErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new HeaderErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IHeaderErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.HeaderErrorEnum} HeaderErrorEnum instance
                             */
                            HeaderErrorEnum.create = function create(properties) {
                                return new HeaderErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified HeaderErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.HeaderErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IHeaderErrorEnum} m HeaderErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HeaderErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified HeaderErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.HeaderErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IHeaderErrorEnum} message HeaderErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HeaderErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HeaderErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.HeaderErrorEnum} HeaderErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HeaderErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.HeaderErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HeaderErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.HeaderErrorEnum} HeaderErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HeaderErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HeaderErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HeaderErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a HeaderErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.HeaderErrorEnum} HeaderErrorEnum
                             */
                            HeaderErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.HeaderErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.HeaderErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a HeaderErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.HeaderErrorEnum} m HeaderErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HeaderErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this HeaderErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.HeaderErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HeaderErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * HeaderError enum.
                             * @name google.ads.googleads.v11.errors.HeaderErrorEnum.HeaderError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_LOGIN_CUSTOMER_ID=3 INVALID_LOGIN_CUSTOMER_ID value
                             * @property {number} INVALID_LINKED_CUSTOMER_ID=7 INVALID_LINKED_CUSTOMER_ID value
                             */
                            HeaderErrorEnum.HeaderError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "INVALID_LOGIN_CUSTOMER_ID"] = 3;
                                values[valuesById[7] = "INVALID_LINKED_CUSTOMER_ID"] = 7;
                                return values;
                            })();
    
                            return HeaderErrorEnum;
                        })();
    
                        errors.IdErrorEnum = (function() {
    
                            /**
                             * Properties of an IdErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IIdErrorEnum
                             */
    
                            /**
                             * Constructs a new IdErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an IdErrorEnum.
                             * @implements IIdErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IIdErrorEnum=} [p] Properties to set
                             */
                            function IdErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new IdErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IIdErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.IdErrorEnum} IdErrorEnum instance
                             */
                            IdErrorEnum.create = function create(properties) {
                                return new IdErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified IdErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.IdErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IIdErrorEnum} m IdErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IdErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified IdErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.IdErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IIdErrorEnum} message IdErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IdErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an IdErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.IdErrorEnum} IdErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IdErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.IdErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an IdErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.IdErrorEnum} IdErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IdErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an IdErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            IdErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an IdErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.IdErrorEnum} IdErrorEnum
                             */
                            IdErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.IdErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.IdErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an IdErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IdErrorEnum} m IdErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            IdErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this IdErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.IdErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            IdErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * IdError enum.
                             * @name google.ads.googleads.v11.errors.IdErrorEnum.IdError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOT_FOUND=2 NOT_FOUND value
                             */
                            IdErrorEnum.IdError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOT_FOUND"] = 2;
                                return values;
                            })();
    
                            return IdErrorEnum;
                        })();
    
                        errors.ImageErrorEnum = (function() {
    
                            /**
                             * Properties of an ImageErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IImageErrorEnum
                             */
    
                            /**
                             * Constructs a new ImageErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an ImageErrorEnum.
                             * @implements IImageErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IImageErrorEnum=} [p] Properties to set
                             */
                            function ImageErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ImageErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IImageErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ImageErrorEnum} ImageErrorEnum instance
                             */
                            ImageErrorEnum.create = function create(properties) {
                                return new ImageErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ImageErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ImageErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IImageErrorEnum} m ImageErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ImageErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ImageErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IImageErrorEnum} message ImageErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ImageErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ImageErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ImageErrorEnum} ImageErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ImageErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ImageErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ImageErrorEnum} ImageErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ImageErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ImageErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ImageErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an ImageErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ImageErrorEnum} ImageErrorEnum
                             */
                            ImageErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ImageErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ImageErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an ImageErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ImageErrorEnum} m ImageErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ImageErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ImageErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ImageErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ImageErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ImageError enum.
                             * @name google.ads.googleads.v11.errors.ImageErrorEnum.ImageError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_IMAGE=2 INVALID_IMAGE value
                             * @property {number} STORAGE_ERROR=3 STORAGE_ERROR value
                             * @property {number} BAD_REQUEST=4 BAD_REQUEST value
                             * @property {number} UNEXPECTED_SIZE=5 UNEXPECTED_SIZE value
                             * @property {number} ANIMATED_NOT_ALLOWED=6 ANIMATED_NOT_ALLOWED value
                             * @property {number} ANIMATION_TOO_LONG=7 ANIMATION_TOO_LONG value
                             * @property {number} SERVER_ERROR=8 SERVER_ERROR value
                             * @property {number} CMYK_JPEG_NOT_ALLOWED=9 CMYK_JPEG_NOT_ALLOWED value
                             * @property {number} FLASH_NOT_ALLOWED=10 FLASH_NOT_ALLOWED value
                             * @property {number} FLASH_WITHOUT_CLICKTAG=11 FLASH_WITHOUT_CLICKTAG value
                             * @property {number} FLASH_ERROR_AFTER_FIXING_CLICK_TAG=12 FLASH_ERROR_AFTER_FIXING_CLICK_TAG value
                             * @property {number} ANIMATED_VISUAL_EFFECT=13 ANIMATED_VISUAL_EFFECT value
                             * @property {number} FLASH_ERROR=14 FLASH_ERROR value
                             * @property {number} LAYOUT_PROBLEM=15 LAYOUT_PROBLEM value
                             * @property {number} PROBLEM_READING_IMAGE_FILE=16 PROBLEM_READING_IMAGE_FILE value
                             * @property {number} ERROR_STORING_IMAGE=17 ERROR_STORING_IMAGE value
                             * @property {number} ASPECT_RATIO_NOT_ALLOWED=18 ASPECT_RATIO_NOT_ALLOWED value
                             * @property {number} FLASH_HAS_NETWORK_OBJECTS=19 FLASH_HAS_NETWORK_OBJECTS value
                             * @property {number} FLASH_HAS_NETWORK_METHODS=20 FLASH_HAS_NETWORK_METHODS value
                             * @property {number} FLASH_HAS_URL=21 FLASH_HAS_URL value
                             * @property {number} FLASH_HAS_MOUSE_TRACKING=22 FLASH_HAS_MOUSE_TRACKING value
                             * @property {number} FLASH_HAS_RANDOM_NUM=23 FLASH_HAS_RANDOM_NUM value
                             * @property {number} FLASH_SELF_TARGETS=24 FLASH_SELF_TARGETS value
                             * @property {number} FLASH_BAD_GETURL_TARGET=25 FLASH_BAD_GETURL_TARGET value
                             * @property {number} FLASH_VERSION_NOT_SUPPORTED=26 FLASH_VERSION_NOT_SUPPORTED value
                             * @property {number} FLASH_WITHOUT_HARD_CODED_CLICK_URL=27 FLASH_WITHOUT_HARD_CODED_CLICK_URL value
                             * @property {number} INVALID_FLASH_FILE=28 INVALID_FLASH_FILE value
                             * @property {number} FAILED_TO_FIX_CLICK_TAG_IN_FLASH=29 FAILED_TO_FIX_CLICK_TAG_IN_FLASH value
                             * @property {number} FLASH_ACCESSES_NETWORK_RESOURCES=30 FLASH_ACCESSES_NETWORK_RESOURCES value
                             * @property {number} FLASH_EXTERNAL_JS_CALL=31 FLASH_EXTERNAL_JS_CALL value
                             * @property {number} FLASH_EXTERNAL_FS_CALL=32 FLASH_EXTERNAL_FS_CALL value
                             * @property {number} FILE_TOO_LARGE=33 FILE_TOO_LARGE value
                             * @property {number} IMAGE_DATA_TOO_LARGE=34 IMAGE_DATA_TOO_LARGE value
                             * @property {number} IMAGE_PROCESSING_ERROR=35 IMAGE_PROCESSING_ERROR value
                             * @property {number} IMAGE_TOO_SMALL=36 IMAGE_TOO_SMALL value
                             * @property {number} INVALID_INPUT=37 INVALID_INPUT value
                             * @property {number} PROBLEM_READING_FILE=38 PROBLEM_READING_FILE value
                             * @property {number} IMAGE_CONSTRAINTS_VIOLATED=39 IMAGE_CONSTRAINTS_VIOLATED value
                             * @property {number} FORMAT_NOT_ALLOWED=40 FORMAT_NOT_ALLOWED value
                             */
                            ImageErrorEnum.ImageError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_IMAGE"] = 2;
                                values[valuesById[3] = "STORAGE_ERROR"] = 3;
                                values[valuesById[4] = "BAD_REQUEST"] = 4;
                                values[valuesById[5] = "UNEXPECTED_SIZE"] = 5;
                                values[valuesById[6] = "ANIMATED_NOT_ALLOWED"] = 6;
                                values[valuesById[7] = "ANIMATION_TOO_LONG"] = 7;
                                values[valuesById[8] = "SERVER_ERROR"] = 8;
                                values[valuesById[9] = "CMYK_JPEG_NOT_ALLOWED"] = 9;
                                values[valuesById[10] = "FLASH_NOT_ALLOWED"] = 10;
                                values[valuesById[11] = "FLASH_WITHOUT_CLICKTAG"] = 11;
                                values[valuesById[12] = "FLASH_ERROR_AFTER_FIXING_CLICK_TAG"] = 12;
                                values[valuesById[13] = "ANIMATED_VISUAL_EFFECT"] = 13;
                                values[valuesById[14] = "FLASH_ERROR"] = 14;
                                values[valuesById[15] = "LAYOUT_PROBLEM"] = 15;
                                values[valuesById[16] = "PROBLEM_READING_IMAGE_FILE"] = 16;
                                values[valuesById[17] = "ERROR_STORING_IMAGE"] = 17;
                                values[valuesById[18] = "ASPECT_RATIO_NOT_ALLOWED"] = 18;
                                values[valuesById[19] = "FLASH_HAS_NETWORK_OBJECTS"] = 19;
                                values[valuesById[20] = "FLASH_HAS_NETWORK_METHODS"] = 20;
                                values[valuesById[21] = "FLASH_HAS_URL"] = 21;
                                values[valuesById[22] = "FLASH_HAS_MOUSE_TRACKING"] = 22;
                                values[valuesById[23] = "FLASH_HAS_RANDOM_NUM"] = 23;
                                values[valuesById[24] = "FLASH_SELF_TARGETS"] = 24;
                                values[valuesById[25] = "FLASH_BAD_GETURL_TARGET"] = 25;
                                values[valuesById[26] = "FLASH_VERSION_NOT_SUPPORTED"] = 26;
                                values[valuesById[27] = "FLASH_WITHOUT_HARD_CODED_CLICK_URL"] = 27;
                                values[valuesById[28] = "INVALID_FLASH_FILE"] = 28;
                                values[valuesById[29] = "FAILED_TO_FIX_CLICK_TAG_IN_FLASH"] = 29;
                                values[valuesById[30] = "FLASH_ACCESSES_NETWORK_RESOURCES"] = 30;
                                values[valuesById[31] = "FLASH_EXTERNAL_JS_CALL"] = 31;
                                values[valuesById[32] = "FLASH_EXTERNAL_FS_CALL"] = 32;
                                values[valuesById[33] = "FILE_TOO_LARGE"] = 33;
                                values[valuesById[34] = "IMAGE_DATA_TOO_LARGE"] = 34;
                                values[valuesById[35] = "IMAGE_PROCESSING_ERROR"] = 35;
                                values[valuesById[36] = "IMAGE_TOO_SMALL"] = 36;
                                values[valuesById[37] = "INVALID_INPUT"] = 37;
                                values[valuesById[38] = "PROBLEM_READING_FILE"] = 38;
                                values[valuesById[39] = "IMAGE_CONSTRAINTS_VIOLATED"] = 39;
                                values[valuesById[40] = "FORMAT_NOT_ALLOWED"] = 40;
                                return values;
                            })();
    
                            return ImageErrorEnum;
                        })();
    
                        errors.InternalErrorEnum = (function() {
    
                            /**
                             * Properties of an InternalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IInternalErrorEnum
                             */
    
                            /**
                             * Constructs a new InternalErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an InternalErrorEnum.
                             * @implements IInternalErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IInternalErrorEnum=} [p] Properties to set
                             */
                            function InternalErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new InternalErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IInternalErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.InternalErrorEnum} InternalErrorEnum instance
                             */
                            InternalErrorEnum.create = function create(properties) {
                                return new InternalErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified InternalErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.InternalErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IInternalErrorEnum} m InternalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InternalErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified InternalErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.InternalErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IInternalErrorEnum} message InternalErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InternalErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an InternalErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.InternalErrorEnum} InternalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InternalErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.InternalErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an InternalErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.InternalErrorEnum} InternalErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InternalErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an InternalErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            InternalErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an InternalErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.InternalErrorEnum} InternalErrorEnum
                             */
                            InternalErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.InternalErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.InternalErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an InternalErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.InternalErrorEnum} m InternalErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            InternalErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this InternalErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.InternalErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            InternalErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * InternalError enum.
                             * @name google.ads.googleads.v11.errors.InternalErrorEnum.InternalError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INTERNAL_ERROR=2 INTERNAL_ERROR value
                             * @property {number} ERROR_CODE_NOT_PUBLISHED=3 ERROR_CODE_NOT_PUBLISHED value
                             * @property {number} TRANSIENT_ERROR=4 TRANSIENT_ERROR value
                             * @property {number} DEADLINE_EXCEEDED=5 DEADLINE_EXCEEDED value
                             */
                            InternalErrorEnum.InternalError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INTERNAL_ERROR"] = 2;
                                values[valuesById[3] = "ERROR_CODE_NOT_PUBLISHED"] = 3;
                                values[valuesById[4] = "TRANSIENT_ERROR"] = 4;
                                values[valuesById[5] = "DEADLINE_EXCEEDED"] = 5;
                                return values;
                            })();
    
                            return InternalErrorEnum;
                        })();
    
                        errors.InvoiceErrorEnum = (function() {
    
                            /**
                             * Properties of an InvoiceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IInvoiceErrorEnum
                             */
    
                            /**
                             * Constructs a new InvoiceErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an InvoiceErrorEnum.
                             * @implements IInvoiceErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IInvoiceErrorEnum=} [p] Properties to set
                             */
                            function InvoiceErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new InvoiceErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IInvoiceErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.InvoiceErrorEnum} InvoiceErrorEnum instance
                             */
                            InvoiceErrorEnum.create = function create(properties) {
                                return new InvoiceErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified InvoiceErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.InvoiceErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IInvoiceErrorEnum} m InvoiceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InvoiceErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified InvoiceErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.InvoiceErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IInvoiceErrorEnum} message InvoiceErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            InvoiceErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an InvoiceErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.InvoiceErrorEnum} InvoiceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InvoiceErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.InvoiceErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an InvoiceErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.InvoiceErrorEnum} InvoiceErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            InvoiceErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an InvoiceErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            InvoiceErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an InvoiceErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.InvoiceErrorEnum} InvoiceErrorEnum
                             */
                            InvoiceErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.InvoiceErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.InvoiceErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an InvoiceErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.InvoiceErrorEnum} m InvoiceErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            InvoiceErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this InvoiceErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.InvoiceErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            InvoiceErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * InvoiceError enum.
                             * @name google.ads.googleads.v11.errors.InvoiceErrorEnum.InvoiceError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} YEAR_MONTH_TOO_OLD=2 YEAR_MONTH_TOO_OLD value
                             * @property {number} NOT_INVOICED_CUSTOMER=3 NOT_INVOICED_CUSTOMER value
                             * @property {number} BILLING_SETUP_NOT_APPROVED=4 BILLING_SETUP_NOT_APPROVED value
                             * @property {number} BILLING_SETUP_NOT_ON_MONTHLY_INVOICING=5 BILLING_SETUP_NOT_ON_MONTHLY_INVOICING value
                             * @property {number} NON_SERVING_CUSTOMER=6 NON_SERVING_CUSTOMER value
                             */
                            InvoiceErrorEnum.InvoiceError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "YEAR_MONTH_TOO_OLD"] = 2;
                                values[valuesById[3] = "NOT_INVOICED_CUSTOMER"] = 3;
                                values[valuesById[4] = "BILLING_SETUP_NOT_APPROVED"] = 4;
                                values[valuesById[5] = "BILLING_SETUP_NOT_ON_MONTHLY_INVOICING"] = 5;
                                values[valuesById[6] = "NON_SERVING_CUSTOMER"] = 6;
                                return values;
                            })();
    
                            return InvoiceErrorEnum;
                        })();
    
                        errors.KeywordPlanAdGroupErrorEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanAdGroupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IKeywordPlanAdGroupErrorEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanAdGroupErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a KeywordPlanAdGroupErrorEnum.
                             * @implements IKeywordPlanAdGroupErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupErrorEnum=} [p] Properties to set
                             */
                            function KeywordPlanAdGroupErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanAdGroupErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum} KeywordPlanAdGroupErrorEnum instance
                             */
                            KeywordPlanAdGroupErrorEnum.create = function create(properties) {
                                return new KeywordPlanAdGroupErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupErrorEnum} m KeywordPlanAdGroupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupErrorEnum} message KeywordPlanAdGroupErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum} KeywordPlanAdGroupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum} KeywordPlanAdGroupErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAdGroupErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAdGroupErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAdGroupErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum} KeywordPlanAdGroupErrorEnum
                             */
                            KeywordPlanAdGroupErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAdGroupErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum} m KeywordPlanAdGroupErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAdGroupErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanAdGroupErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAdGroupErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanAdGroupError enum.
                             * @name google.ads.googleads.v11.errors.KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_NAME=2 INVALID_NAME value
                             * @property {number} DUPLICATE_NAME=3 DUPLICATE_NAME value
                             */
                            KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_NAME"] = 2;
                                values[valuesById[3] = "DUPLICATE_NAME"] = 3;
                                return values;
                            })();
    
                            return KeywordPlanAdGroupErrorEnum;
                        })();
    
                        errors.KeywordPlanAdGroupKeywordErrorEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanAdGroupKeywordErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IKeywordPlanAdGroupKeywordErrorEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanAdGroupKeywordErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a KeywordPlanAdGroupKeywordErrorEnum.
                             * @implements IKeywordPlanAdGroupKeywordErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupKeywordErrorEnum=} [p] Properties to set
                             */
                            function KeywordPlanAdGroupKeywordErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanAdGroupKeywordErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupKeywordErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum} KeywordPlanAdGroupKeywordErrorEnum instance
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.create = function create(properties) {
                                return new KeywordPlanAdGroupKeywordErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupKeywordErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupKeywordErrorEnum} m KeywordPlanAdGroupKeywordErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupKeywordErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanAdGroupKeywordErrorEnum} message KeywordPlanAdGroupKeywordErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupKeywordErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum} KeywordPlanAdGroupKeywordErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupKeywordErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum} KeywordPlanAdGroupKeywordErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAdGroupKeywordErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAdGroupKeywordErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum} KeywordPlanAdGroupKeywordErrorEnum
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAdGroupKeywordErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum} m KeywordPlanAdGroupKeywordErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanAdGroupKeywordErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanAdGroupKeywordError enum.
                             * @name google.ads.googleads.v11.errors.KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_KEYWORD_MATCH_TYPE=2 INVALID_KEYWORD_MATCH_TYPE value
                             * @property {number} DUPLICATE_KEYWORD=3 DUPLICATE_KEYWORD value
                             * @property {number} KEYWORD_TEXT_TOO_LONG=4 KEYWORD_TEXT_TOO_LONG value
                             * @property {number} KEYWORD_HAS_INVALID_CHARS=5 KEYWORD_HAS_INVALID_CHARS value
                             * @property {number} KEYWORD_HAS_TOO_MANY_WORDS=6 KEYWORD_HAS_TOO_MANY_WORDS value
                             * @property {number} INVALID_KEYWORD_TEXT=7 INVALID_KEYWORD_TEXT value
                             * @property {number} NEGATIVE_KEYWORD_HAS_CPC_BID=8 NEGATIVE_KEYWORD_HAS_CPC_BID value
                             * @property {number} NEW_BMM_KEYWORDS_NOT_ALLOWED=9 NEW_BMM_KEYWORDS_NOT_ALLOWED value
                             */
                            KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_KEYWORD_MATCH_TYPE"] = 2;
                                values[valuesById[3] = "DUPLICATE_KEYWORD"] = 3;
                                values[valuesById[4] = "KEYWORD_TEXT_TOO_LONG"] = 4;
                                values[valuesById[5] = "KEYWORD_HAS_INVALID_CHARS"] = 5;
                                values[valuesById[6] = "KEYWORD_HAS_TOO_MANY_WORDS"] = 6;
                                values[valuesById[7] = "INVALID_KEYWORD_TEXT"] = 7;
                                values[valuesById[8] = "NEGATIVE_KEYWORD_HAS_CPC_BID"] = 8;
                                values[valuesById[9] = "NEW_BMM_KEYWORDS_NOT_ALLOWED"] = 9;
                                return values;
                            })();
    
                            return KeywordPlanAdGroupKeywordErrorEnum;
                        })();
    
                        errors.KeywordPlanCampaignErrorEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaignErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IKeywordPlanCampaignErrorEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaignErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a KeywordPlanCampaignErrorEnum.
                             * @implements IKeywordPlanCampaignErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignErrorEnum=} [p] Properties to set
                             */
                            function KeywordPlanCampaignErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanCampaignErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum} KeywordPlanCampaignErrorEnum instance
                             */
                            KeywordPlanCampaignErrorEnum.create = function create(properties) {
                                return new KeywordPlanCampaignErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignErrorEnum} m KeywordPlanCampaignErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignErrorEnum} message KeywordPlanCampaignErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum} KeywordPlanCampaignErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum} KeywordPlanCampaignErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaignErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaignErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaignErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum} KeywordPlanCampaignErrorEnum
                             */
                            KeywordPlanCampaignErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaignErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum} m KeywordPlanCampaignErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaignErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanCampaignErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaignErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanCampaignError enum.
                             * @name google.ads.googleads.v11.errors.KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_NAME=2 INVALID_NAME value
                             * @property {number} INVALID_LANGUAGES=3 INVALID_LANGUAGES value
                             * @property {number} INVALID_GEOS=4 INVALID_GEOS value
                             * @property {number} DUPLICATE_NAME=5 DUPLICATE_NAME value
                             * @property {number} MAX_GEOS_EXCEEDED=6 MAX_GEOS_EXCEEDED value
                             * @property {number} MAX_LANGUAGES_EXCEEDED=7 MAX_LANGUAGES_EXCEEDED value
                             */
                            KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_NAME"] = 2;
                                values[valuesById[3] = "INVALID_LANGUAGES"] = 3;
                                values[valuesById[4] = "INVALID_GEOS"] = 4;
                                values[valuesById[5] = "DUPLICATE_NAME"] = 5;
                                values[valuesById[6] = "MAX_GEOS_EXCEEDED"] = 6;
                                values[valuesById[7] = "MAX_LANGUAGES_EXCEEDED"] = 7;
                                return values;
                            })();
    
                            return KeywordPlanCampaignErrorEnum;
                        })();
    
                        errors.KeywordPlanCampaignKeywordErrorEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaignKeywordErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IKeywordPlanCampaignKeywordErrorEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaignKeywordErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a KeywordPlanCampaignKeywordErrorEnum.
                             * @implements IKeywordPlanCampaignKeywordErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignKeywordErrorEnum=} [p] Properties to set
                             */
                            function KeywordPlanCampaignKeywordErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanCampaignKeywordErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignKeywordErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum} KeywordPlanCampaignKeywordErrorEnum instance
                             */
                            KeywordPlanCampaignKeywordErrorEnum.create = function create(properties) {
                                return new KeywordPlanCampaignKeywordErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignKeywordErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignKeywordErrorEnum} m KeywordPlanCampaignKeywordErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignKeywordErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignKeywordErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanCampaignKeywordErrorEnum} message KeywordPlanCampaignKeywordErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignKeywordErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignKeywordErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum} KeywordPlanCampaignKeywordErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignKeywordErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignKeywordErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum} KeywordPlanCampaignKeywordErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignKeywordErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaignKeywordErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaignKeywordErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaignKeywordErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum} KeywordPlanCampaignKeywordErrorEnum
                             */
                            KeywordPlanCampaignKeywordErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaignKeywordErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum} m KeywordPlanCampaignKeywordErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaignKeywordErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanCampaignKeywordErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaignKeywordErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanCampaignKeywordError enum.
                             * @name google.ads.googleads.v11.errors.KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CAMPAIGN_KEYWORD_IS_POSITIVE=8 CAMPAIGN_KEYWORD_IS_POSITIVE value
                             */
                            KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[8] = "CAMPAIGN_KEYWORD_IS_POSITIVE"] = 8;
                                return values;
                            })();
    
                            return KeywordPlanCampaignKeywordErrorEnum;
                        })();
    
                        errors.KeywordPlanErrorEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IKeywordPlanErrorEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a KeywordPlanErrorEnum.
                             * @implements IKeywordPlanErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanErrorEnum=} [p] Properties to set
                             */
                            function KeywordPlanErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanErrorEnum} KeywordPlanErrorEnum instance
                             */
                            KeywordPlanErrorEnum.create = function create(properties) {
                                return new KeywordPlanErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanErrorEnum} m KeywordPlanErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanErrorEnum} message KeywordPlanErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanErrorEnum} KeywordPlanErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.KeywordPlanErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanErrorEnum} KeywordPlanErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanErrorEnum} KeywordPlanErrorEnum
                             */
                            KeywordPlanErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.KeywordPlanErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.KeywordPlanErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.KeywordPlanErrorEnum} m KeywordPlanErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanError enum.
                             * @name google.ads.googleads.v11.errors.KeywordPlanErrorEnum.KeywordPlanError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BID_MULTIPLIER_OUT_OF_RANGE=2 BID_MULTIPLIER_OUT_OF_RANGE value
                             * @property {number} BID_TOO_HIGH=3 BID_TOO_HIGH value
                             * @property {number} BID_TOO_LOW=4 BID_TOO_LOW value
                             * @property {number} BID_TOO_MANY_FRACTIONAL_DIGITS=5 BID_TOO_MANY_FRACTIONAL_DIGITS value
                             * @property {number} DAILY_BUDGET_TOO_LOW=6 DAILY_BUDGET_TOO_LOW value
                             * @property {number} DAILY_BUDGET_TOO_MANY_FRACTIONAL_DIGITS=7 DAILY_BUDGET_TOO_MANY_FRACTIONAL_DIGITS value
                             * @property {number} INVALID_VALUE=8 INVALID_VALUE value
                             * @property {number} KEYWORD_PLAN_HAS_NO_KEYWORDS=9 KEYWORD_PLAN_HAS_NO_KEYWORDS value
                             * @property {number} KEYWORD_PLAN_NOT_ENABLED=10 KEYWORD_PLAN_NOT_ENABLED value
                             * @property {number} KEYWORD_PLAN_NOT_FOUND=11 KEYWORD_PLAN_NOT_FOUND value
                             * @property {number} MISSING_BID=13 MISSING_BID value
                             * @property {number} MISSING_FORECAST_PERIOD=14 MISSING_FORECAST_PERIOD value
                             * @property {number} INVALID_FORECAST_DATE_RANGE=15 INVALID_FORECAST_DATE_RANGE value
                             * @property {number} INVALID_NAME=16 INVALID_NAME value
                             */
                            KeywordPlanErrorEnum.KeywordPlanError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BID_MULTIPLIER_OUT_OF_RANGE"] = 2;
                                values[valuesById[3] = "BID_TOO_HIGH"] = 3;
                                values[valuesById[4] = "BID_TOO_LOW"] = 4;
                                values[valuesById[5] = "BID_TOO_MANY_FRACTIONAL_DIGITS"] = 5;
                                values[valuesById[6] = "DAILY_BUDGET_TOO_LOW"] = 6;
                                values[valuesById[7] = "DAILY_BUDGET_TOO_MANY_FRACTIONAL_DIGITS"] = 7;
                                values[valuesById[8] = "INVALID_VALUE"] = 8;
                                values[valuesById[9] = "KEYWORD_PLAN_HAS_NO_KEYWORDS"] = 9;
                                values[valuesById[10] = "KEYWORD_PLAN_NOT_ENABLED"] = 10;
                                values[valuesById[11] = "KEYWORD_PLAN_NOT_FOUND"] = 11;
                                values[valuesById[13] = "MISSING_BID"] = 13;
                                values[valuesById[14] = "MISSING_FORECAST_PERIOD"] = 14;
                                values[valuesById[15] = "INVALID_FORECAST_DATE_RANGE"] = 15;
                                values[valuesById[16] = "INVALID_NAME"] = 16;
                                return values;
                            })();
    
                            return KeywordPlanErrorEnum;
                        })();
    
                        errors.KeywordPlanIdeaErrorEnum = (function() {
    
                            /**
                             * Properties of a KeywordPlanIdeaErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IKeywordPlanIdeaErrorEnum
                             */
    
                            /**
                             * Constructs a new KeywordPlanIdeaErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a KeywordPlanIdeaErrorEnum.
                             * @implements IKeywordPlanIdeaErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanIdeaErrorEnum=} [p] Properties to set
                             */
                            function KeywordPlanIdeaErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new KeywordPlanIdeaErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanIdeaErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum} KeywordPlanIdeaErrorEnum instance
                             */
                            KeywordPlanIdeaErrorEnum.create = function create(properties) {
                                return new KeywordPlanIdeaErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanIdeaErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanIdeaErrorEnum} m KeywordPlanIdeaErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanIdeaErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanIdeaErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IKeywordPlanIdeaErrorEnum} message KeywordPlanIdeaErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanIdeaErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanIdeaErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum} KeywordPlanIdeaErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanIdeaErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanIdeaErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum} KeywordPlanIdeaErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanIdeaErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanIdeaErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanIdeaErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanIdeaErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum} KeywordPlanIdeaErrorEnum
                             */
                            KeywordPlanIdeaErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanIdeaErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum} m KeywordPlanIdeaErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanIdeaErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this KeywordPlanIdeaErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanIdeaErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * KeywordPlanIdeaError enum.
                             * @name google.ads.googleads.v11.errors.KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} URL_CRAWL_ERROR=2 URL_CRAWL_ERROR value
                             * @property {number} INVALID_VALUE=3 INVALID_VALUE value
                             */
                            KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "URL_CRAWL_ERROR"] = 2;
                                values[valuesById[3] = "INVALID_VALUE"] = 3;
                                return values;
                            })();
    
                            return KeywordPlanIdeaErrorEnum;
                        })();
    
                        errors.LabelErrorEnum = (function() {
    
                            /**
                             * Properties of a LabelErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ILabelErrorEnum
                             */
    
                            /**
                             * Constructs a new LabelErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a LabelErrorEnum.
                             * @implements ILabelErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ILabelErrorEnum=} [p] Properties to set
                             */
                            function LabelErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LabelErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ILabelErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.LabelErrorEnum} LabelErrorEnum instance
                             */
                            LabelErrorEnum.create = function create(properties) {
                                return new LabelErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LabelErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.LabelErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ILabelErrorEnum} m LabelErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LabelErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LabelErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.LabelErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ILabelErrorEnum} message LabelErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LabelErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LabelErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.LabelErrorEnum} LabelErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LabelErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.LabelErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LabelErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.LabelErrorEnum} LabelErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LabelErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LabelErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LabelErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LabelErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.LabelErrorEnum} LabelErrorEnum
                             */
                            LabelErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.LabelErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.LabelErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a LabelErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.LabelErrorEnum} m LabelErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LabelErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LabelErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.LabelErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LabelErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LabelError enum.
                             * @name google.ads.googleads.v11.errors.LabelErrorEnum.LabelError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_APPLY_INACTIVE_LABEL=2 CANNOT_APPLY_INACTIVE_LABEL value
                             * @property {number} CANNOT_APPLY_LABEL_TO_DISABLED_AD_GROUP_CRITERION=3 CANNOT_APPLY_LABEL_TO_DISABLED_AD_GROUP_CRITERION value
                             * @property {number} CANNOT_APPLY_LABEL_TO_NEGATIVE_AD_GROUP_CRITERION=4 CANNOT_APPLY_LABEL_TO_NEGATIVE_AD_GROUP_CRITERION value
                             * @property {number} EXCEEDED_LABEL_LIMIT_PER_TYPE=5 EXCEEDED_LABEL_LIMIT_PER_TYPE value
                             * @property {number} INVALID_RESOURCE_FOR_MANAGER_LABEL=6 INVALID_RESOURCE_FOR_MANAGER_LABEL value
                             * @property {number} DUPLICATE_NAME=7 DUPLICATE_NAME value
                             * @property {number} INVALID_LABEL_NAME=8 INVALID_LABEL_NAME value
                             * @property {number} CANNOT_ATTACH_LABEL_TO_DRAFT=9 CANNOT_ATTACH_LABEL_TO_DRAFT value
                             * @property {number} CANNOT_ATTACH_NON_MANAGER_LABEL_TO_CUSTOMER=10 CANNOT_ATTACH_NON_MANAGER_LABEL_TO_CUSTOMER value
                             */
                            LabelErrorEnum.LabelError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_APPLY_INACTIVE_LABEL"] = 2;
                                values[valuesById[3] = "CANNOT_APPLY_LABEL_TO_DISABLED_AD_GROUP_CRITERION"] = 3;
                                values[valuesById[4] = "CANNOT_APPLY_LABEL_TO_NEGATIVE_AD_GROUP_CRITERION"] = 4;
                                values[valuesById[5] = "EXCEEDED_LABEL_LIMIT_PER_TYPE"] = 5;
                                values[valuesById[6] = "INVALID_RESOURCE_FOR_MANAGER_LABEL"] = 6;
                                values[valuesById[7] = "DUPLICATE_NAME"] = 7;
                                values[valuesById[8] = "INVALID_LABEL_NAME"] = 8;
                                values[valuesById[9] = "CANNOT_ATTACH_LABEL_TO_DRAFT"] = 9;
                                values[valuesById[10] = "CANNOT_ATTACH_NON_MANAGER_LABEL_TO_CUSTOMER"] = 10;
                                return values;
                            })();
    
                            return LabelErrorEnum;
                        })();
    
                        errors.LanguageCodeErrorEnum = (function() {
    
                            /**
                             * Properties of a LanguageCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ILanguageCodeErrorEnum
                             */
    
                            /**
                             * Constructs a new LanguageCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a LanguageCodeErrorEnum.
                             * @implements ILanguageCodeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ILanguageCodeErrorEnum=} [p] Properties to set
                             */
                            function LanguageCodeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new LanguageCodeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ILanguageCodeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.LanguageCodeErrorEnum} LanguageCodeErrorEnum instance
                             */
                            LanguageCodeErrorEnum.create = function create(properties) {
                                return new LanguageCodeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified LanguageCodeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.LanguageCodeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ILanguageCodeErrorEnum} m LanguageCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LanguageCodeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LanguageCodeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.LanguageCodeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ILanguageCodeErrorEnum} message LanguageCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LanguageCodeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LanguageCodeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.LanguageCodeErrorEnum} LanguageCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LanguageCodeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.LanguageCodeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LanguageCodeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.LanguageCodeErrorEnum} LanguageCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LanguageCodeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LanguageCodeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LanguageCodeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a LanguageCodeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.LanguageCodeErrorEnum} LanguageCodeErrorEnum
                             */
                            LanguageCodeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.LanguageCodeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.LanguageCodeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a LanguageCodeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.LanguageCodeErrorEnum} m LanguageCodeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LanguageCodeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this LanguageCodeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.LanguageCodeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LanguageCodeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * LanguageCodeError enum.
                             * @name google.ads.googleads.v11.errors.LanguageCodeErrorEnum.LanguageCodeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} LANGUAGE_CODE_NOT_FOUND=2 LANGUAGE_CODE_NOT_FOUND value
                             * @property {number} INVALID_LANGUAGE_CODE=3 INVALID_LANGUAGE_CODE value
                             */
                            LanguageCodeErrorEnum.LanguageCodeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "LANGUAGE_CODE_NOT_FOUND"] = 2;
                                values[valuesById[3] = "INVALID_LANGUAGE_CODE"] = 3;
                                return values;
                            })();
    
                            return LanguageCodeErrorEnum;
                        })();
    
                        errors.ListOperationErrorEnum = (function() {
    
                            /**
                             * Properties of a ListOperationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IListOperationErrorEnum
                             */
    
                            /**
                             * Constructs a new ListOperationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ListOperationErrorEnum.
                             * @implements IListOperationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IListOperationErrorEnum=} [p] Properties to set
                             */
                            function ListOperationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListOperationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IListOperationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ListOperationErrorEnum} ListOperationErrorEnum instance
                             */
                            ListOperationErrorEnum.create = function create(properties) {
                                return new ListOperationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ListOperationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ListOperationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IListOperationErrorEnum} m ListOperationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListOperationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListOperationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ListOperationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IListOperationErrorEnum} message ListOperationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListOperationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListOperationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ListOperationErrorEnum} ListOperationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListOperationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ListOperationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListOperationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ListOperationErrorEnum} ListOperationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListOperationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListOperationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListOperationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListOperationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ListOperationErrorEnum} ListOperationErrorEnum
                             */
                            ListOperationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ListOperationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ListOperationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ListOperationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ListOperationErrorEnum} m ListOperationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListOperationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListOperationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ListOperationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListOperationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ListOperationError enum.
                             * @name google.ads.googleads.v11.errors.ListOperationErrorEnum.ListOperationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} REQUIRED_FIELD_MISSING=7 REQUIRED_FIELD_MISSING value
                             * @property {number} DUPLICATE_VALUES=8 DUPLICATE_VALUES value
                             */
                            ListOperationErrorEnum.ListOperationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[7] = "REQUIRED_FIELD_MISSING"] = 7;
                                values[valuesById[8] = "DUPLICATE_VALUES"] = 8;
                                return values;
                            })();
    
                            return ListOperationErrorEnum;
                        })();
    
                        errors.ManagerLinkErrorEnum = (function() {
    
                            /**
                             * Properties of a ManagerLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IManagerLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new ManagerLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ManagerLinkErrorEnum.
                             * @implements IManagerLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IManagerLinkErrorEnum=} [p] Properties to set
                             */
                            function ManagerLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ManagerLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IManagerLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ManagerLinkErrorEnum} ManagerLinkErrorEnum instance
                             */
                            ManagerLinkErrorEnum.create = function create(properties) {
                                return new ManagerLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ManagerLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ManagerLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IManagerLinkErrorEnum} m ManagerLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManagerLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ManagerLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ManagerLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IManagerLinkErrorEnum} message ManagerLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManagerLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ManagerLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ManagerLinkErrorEnum} ManagerLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManagerLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ManagerLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ManagerLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ManagerLinkErrorEnum} ManagerLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManagerLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ManagerLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ManagerLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ManagerLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ManagerLinkErrorEnum} ManagerLinkErrorEnum
                             */
                            ManagerLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ManagerLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ManagerLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ManagerLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ManagerLinkErrorEnum} m ManagerLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ManagerLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ManagerLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ManagerLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ManagerLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ManagerLinkError enum.
                             * @name google.ads.googleads.v11.errors.ManagerLinkErrorEnum.ManagerLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ACCOUNTS_NOT_COMPATIBLE_FOR_LINKING=2 ACCOUNTS_NOT_COMPATIBLE_FOR_LINKING value
                             * @property {number} TOO_MANY_MANAGERS=3 TOO_MANY_MANAGERS value
                             * @property {number} TOO_MANY_INVITES=4 TOO_MANY_INVITES value
                             * @property {number} ALREADY_INVITED_BY_THIS_MANAGER=5 ALREADY_INVITED_BY_THIS_MANAGER value
                             * @property {number} ALREADY_MANAGED_BY_THIS_MANAGER=6 ALREADY_MANAGED_BY_THIS_MANAGER value
                             * @property {number} ALREADY_MANAGED_IN_HIERARCHY=7 ALREADY_MANAGED_IN_HIERARCHY value
                             * @property {number} DUPLICATE_CHILD_FOUND=8 DUPLICATE_CHILD_FOUND value
                             * @property {number} CLIENT_HAS_NO_ADMIN_USER=9 CLIENT_HAS_NO_ADMIN_USER value
                             * @property {number} MAX_DEPTH_EXCEEDED=10 MAX_DEPTH_EXCEEDED value
                             * @property {number} CYCLE_NOT_ALLOWED=11 CYCLE_NOT_ALLOWED value
                             * @property {number} TOO_MANY_ACCOUNTS=12 TOO_MANY_ACCOUNTS value
                             * @property {number} TOO_MANY_ACCOUNTS_AT_MANAGER=13 TOO_MANY_ACCOUNTS_AT_MANAGER value
                             * @property {number} NON_OWNER_USER_CANNOT_MODIFY_LINK=14 NON_OWNER_USER_CANNOT_MODIFY_LINK value
                             * @property {number} SUSPENDED_ACCOUNT_CANNOT_ADD_CLIENTS=15 SUSPENDED_ACCOUNT_CANNOT_ADD_CLIENTS value
                             * @property {number} CLIENT_OUTSIDE_TREE=16 CLIENT_OUTSIDE_TREE value
                             * @property {number} INVALID_STATUS_CHANGE=17 INVALID_STATUS_CHANGE value
                             * @property {number} INVALID_CHANGE=18 INVALID_CHANGE value
                             * @property {number} CUSTOMER_CANNOT_MANAGE_SELF=19 CUSTOMER_CANNOT_MANAGE_SELF value
                             * @property {number} CREATING_ENABLED_LINK_NOT_ALLOWED=20 CREATING_ENABLED_LINK_NOT_ALLOWED value
                             */
                            ManagerLinkErrorEnum.ManagerLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ACCOUNTS_NOT_COMPATIBLE_FOR_LINKING"] = 2;
                                values[valuesById[3] = "TOO_MANY_MANAGERS"] = 3;
                                values[valuesById[4] = "TOO_MANY_INVITES"] = 4;
                                values[valuesById[5] = "ALREADY_INVITED_BY_THIS_MANAGER"] = 5;
                                values[valuesById[6] = "ALREADY_MANAGED_BY_THIS_MANAGER"] = 6;
                                values[valuesById[7] = "ALREADY_MANAGED_IN_HIERARCHY"] = 7;
                                values[valuesById[8] = "DUPLICATE_CHILD_FOUND"] = 8;
                                values[valuesById[9] = "CLIENT_HAS_NO_ADMIN_USER"] = 9;
                                values[valuesById[10] = "MAX_DEPTH_EXCEEDED"] = 10;
                                values[valuesById[11] = "CYCLE_NOT_ALLOWED"] = 11;
                                values[valuesById[12] = "TOO_MANY_ACCOUNTS"] = 12;
                                values[valuesById[13] = "TOO_MANY_ACCOUNTS_AT_MANAGER"] = 13;
                                values[valuesById[14] = "NON_OWNER_USER_CANNOT_MODIFY_LINK"] = 14;
                                values[valuesById[15] = "SUSPENDED_ACCOUNT_CANNOT_ADD_CLIENTS"] = 15;
                                values[valuesById[16] = "CLIENT_OUTSIDE_TREE"] = 16;
                                values[valuesById[17] = "INVALID_STATUS_CHANGE"] = 17;
                                values[valuesById[18] = "INVALID_CHANGE"] = 18;
                                values[valuesById[19] = "CUSTOMER_CANNOT_MANAGE_SELF"] = 19;
                                values[valuesById[20] = "CREATING_ENABLED_LINK_NOT_ALLOWED"] = 20;
                                return values;
                            })();
    
                            return ManagerLinkErrorEnum;
                        })();
    
                        errors.MediaBundleErrorEnum = (function() {
    
                            /**
                             * Properties of a MediaBundleErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IMediaBundleErrorEnum
                             */
    
                            /**
                             * Constructs a new MediaBundleErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a MediaBundleErrorEnum.
                             * @implements IMediaBundleErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IMediaBundleErrorEnum=} [p] Properties to set
                             */
                            function MediaBundleErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MediaBundleErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaBundleErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.MediaBundleErrorEnum} MediaBundleErrorEnum instance
                             */
                            MediaBundleErrorEnum.create = function create(properties) {
                                return new MediaBundleErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MediaBundleErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.MediaBundleErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaBundleErrorEnum} m MediaBundleErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaBundleErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaBundleErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.MediaBundleErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaBundleErrorEnum} message MediaBundleErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaBundleErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaBundleErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.MediaBundleErrorEnum} MediaBundleErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaBundleErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.MediaBundleErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaBundleErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.MediaBundleErrorEnum} MediaBundleErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaBundleErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaBundleErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaBundleErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MediaBundleErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.MediaBundleErrorEnum} MediaBundleErrorEnum
                             */
                            MediaBundleErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.MediaBundleErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.MediaBundleErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MediaBundleErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.MediaBundleErrorEnum} m MediaBundleErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaBundleErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MediaBundleErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.MediaBundleErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaBundleErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MediaBundleError enum.
                             * @name google.ads.googleads.v11.errors.MediaBundleErrorEnum.MediaBundleError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BAD_REQUEST=3 BAD_REQUEST value
                             * @property {number} DOUBLECLICK_BUNDLE_NOT_ALLOWED=4 DOUBLECLICK_BUNDLE_NOT_ALLOWED value
                             * @property {number} EXTERNAL_URL_NOT_ALLOWED=5 EXTERNAL_URL_NOT_ALLOWED value
                             * @property {number} FILE_TOO_LARGE=6 FILE_TOO_LARGE value
                             * @property {number} GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED=7 GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED value
                             * @property {number} INVALID_INPUT=8 INVALID_INPUT value
                             * @property {number} INVALID_MEDIA_BUNDLE=9 INVALID_MEDIA_BUNDLE value
                             * @property {number} INVALID_MEDIA_BUNDLE_ENTRY=10 INVALID_MEDIA_BUNDLE_ENTRY value
                             * @property {number} INVALID_MIME_TYPE=11 INVALID_MIME_TYPE value
                             * @property {number} INVALID_PATH=12 INVALID_PATH value
                             * @property {number} INVALID_URL_REFERENCE=13 INVALID_URL_REFERENCE value
                             * @property {number} MEDIA_DATA_TOO_LARGE=14 MEDIA_DATA_TOO_LARGE value
                             * @property {number} MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY=15 MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY value
                             * @property {number} SERVER_ERROR=16 SERVER_ERROR value
                             * @property {number} STORAGE_ERROR=17 STORAGE_ERROR value
                             * @property {number} SWIFFY_BUNDLE_NOT_ALLOWED=18 SWIFFY_BUNDLE_NOT_ALLOWED value
                             * @property {number} TOO_MANY_FILES=19 TOO_MANY_FILES value
                             * @property {number} UNEXPECTED_SIZE=20 UNEXPECTED_SIZE value
                             * @property {number} UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT=21 UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT value
                             * @property {number} UNSUPPORTED_HTML5_FEATURE=22 UNSUPPORTED_HTML5_FEATURE value
                             * @property {number} URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT=23 URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT value
                             * @property {number} CUSTOM_EXIT_NOT_ALLOWED=24 CUSTOM_EXIT_NOT_ALLOWED value
                             */
                            MediaBundleErrorEnum.MediaBundleError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "BAD_REQUEST"] = 3;
                                values[valuesById[4] = "DOUBLECLICK_BUNDLE_NOT_ALLOWED"] = 4;
                                values[valuesById[5] = "EXTERNAL_URL_NOT_ALLOWED"] = 5;
                                values[valuesById[6] = "FILE_TOO_LARGE"] = 6;
                                values[valuesById[7] = "GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED"] = 7;
                                values[valuesById[8] = "INVALID_INPUT"] = 8;
                                values[valuesById[9] = "INVALID_MEDIA_BUNDLE"] = 9;
                                values[valuesById[10] = "INVALID_MEDIA_BUNDLE_ENTRY"] = 10;
                                values[valuesById[11] = "INVALID_MIME_TYPE"] = 11;
                                values[valuesById[12] = "INVALID_PATH"] = 12;
                                values[valuesById[13] = "INVALID_URL_REFERENCE"] = 13;
                                values[valuesById[14] = "MEDIA_DATA_TOO_LARGE"] = 14;
                                values[valuesById[15] = "MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY"] = 15;
                                values[valuesById[16] = "SERVER_ERROR"] = 16;
                                values[valuesById[17] = "STORAGE_ERROR"] = 17;
                                values[valuesById[18] = "SWIFFY_BUNDLE_NOT_ALLOWED"] = 18;
                                values[valuesById[19] = "TOO_MANY_FILES"] = 19;
                                values[valuesById[20] = "UNEXPECTED_SIZE"] = 20;
                                values[valuesById[21] = "UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT"] = 21;
                                values[valuesById[22] = "UNSUPPORTED_HTML5_FEATURE"] = 22;
                                values[valuesById[23] = "URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT"] = 23;
                                values[valuesById[24] = "CUSTOM_EXIT_NOT_ALLOWED"] = 24;
                                return values;
                            })();
    
                            return MediaBundleErrorEnum;
                        })();
    
                        errors.MediaFileErrorEnum = (function() {
    
                            /**
                             * Properties of a MediaFileErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IMediaFileErrorEnum
                             */
    
                            /**
                             * Constructs a new MediaFileErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a MediaFileErrorEnum.
                             * @implements IMediaFileErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IMediaFileErrorEnum=} [p] Properties to set
                             */
                            function MediaFileErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MediaFileErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaFileErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.MediaFileErrorEnum} MediaFileErrorEnum instance
                             */
                            MediaFileErrorEnum.create = function create(properties) {
                                return new MediaFileErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MediaFileErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.MediaFileErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaFileErrorEnum} m MediaFileErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaFileErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaFileErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.MediaFileErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaFileErrorEnum} message MediaFileErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaFileErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaFileErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.MediaFileErrorEnum} MediaFileErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaFileErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.MediaFileErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaFileErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.MediaFileErrorEnum} MediaFileErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaFileErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaFileErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaFileErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MediaFileErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.MediaFileErrorEnum} MediaFileErrorEnum
                             */
                            MediaFileErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.MediaFileErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.MediaFileErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MediaFileErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.MediaFileErrorEnum} m MediaFileErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaFileErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MediaFileErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.MediaFileErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaFileErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MediaFileError enum.
                             * @name google.ads.googleads.v11.errors.MediaFileErrorEnum.MediaFileError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_CREATE_STANDARD_ICON=2 CANNOT_CREATE_STANDARD_ICON value
                             * @property {number} CANNOT_SELECT_STANDARD_ICON_WITH_OTHER_TYPES=3 CANNOT_SELECT_STANDARD_ICON_WITH_OTHER_TYPES value
                             * @property {number} CANNOT_SPECIFY_MEDIA_FILE_ID_AND_DATA=4 CANNOT_SPECIFY_MEDIA_FILE_ID_AND_DATA value
                             * @property {number} DUPLICATE_MEDIA=5 DUPLICATE_MEDIA value
                             * @property {number} EMPTY_FIELD=6 EMPTY_FIELD value
                             * @property {number} RESOURCE_REFERENCED_IN_MULTIPLE_OPS=7 RESOURCE_REFERENCED_IN_MULTIPLE_OPS value
                             * @property {number} FIELD_NOT_SUPPORTED_FOR_MEDIA_SUB_TYPE=8 FIELD_NOT_SUPPORTED_FOR_MEDIA_SUB_TYPE value
                             * @property {number} INVALID_MEDIA_FILE_ID=9 INVALID_MEDIA_FILE_ID value
                             * @property {number} INVALID_MEDIA_SUB_TYPE=10 INVALID_MEDIA_SUB_TYPE value
                             * @property {number} INVALID_MEDIA_FILE_TYPE=11 INVALID_MEDIA_FILE_TYPE value
                             * @property {number} INVALID_MIME_TYPE=12 INVALID_MIME_TYPE value
                             * @property {number} INVALID_REFERENCE_ID=13 INVALID_REFERENCE_ID value
                             * @property {number} INVALID_YOU_TUBE_ID=14 INVALID_YOU_TUBE_ID value
                             * @property {number} MEDIA_FILE_FAILED_TRANSCODING=15 MEDIA_FILE_FAILED_TRANSCODING value
                             * @property {number} MEDIA_NOT_TRANSCODED=16 MEDIA_NOT_TRANSCODED value
                             * @property {number} MEDIA_TYPE_DOES_NOT_MATCH_MEDIA_FILE_TYPE=17 MEDIA_TYPE_DOES_NOT_MATCH_MEDIA_FILE_TYPE value
                             * @property {number} NO_FIELDS_SPECIFIED=18 NO_FIELDS_SPECIFIED value
                             * @property {number} NULL_REFERENCE_ID_AND_MEDIA_ID=19 NULL_REFERENCE_ID_AND_MEDIA_ID value
                             * @property {number} TOO_LONG=20 TOO_LONG value
                             * @property {number} UNSUPPORTED_TYPE=21 UNSUPPORTED_TYPE value
                             * @property {number} YOU_TUBE_SERVICE_UNAVAILABLE=22 YOU_TUBE_SERVICE_UNAVAILABLE value
                             * @property {number} YOU_TUBE_VIDEO_HAS_NON_POSITIVE_DURATION=23 YOU_TUBE_VIDEO_HAS_NON_POSITIVE_DURATION value
                             * @property {number} YOU_TUBE_VIDEO_NOT_FOUND=24 YOU_TUBE_VIDEO_NOT_FOUND value
                             */
                            MediaFileErrorEnum.MediaFileError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_CREATE_STANDARD_ICON"] = 2;
                                values[valuesById[3] = "CANNOT_SELECT_STANDARD_ICON_WITH_OTHER_TYPES"] = 3;
                                values[valuesById[4] = "CANNOT_SPECIFY_MEDIA_FILE_ID_AND_DATA"] = 4;
                                values[valuesById[5] = "DUPLICATE_MEDIA"] = 5;
                                values[valuesById[6] = "EMPTY_FIELD"] = 6;
                                values[valuesById[7] = "RESOURCE_REFERENCED_IN_MULTIPLE_OPS"] = 7;
                                values[valuesById[8] = "FIELD_NOT_SUPPORTED_FOR_MEDIA_SUB_TYPE"] = 8;
                                values[valuesById[9] = "INVALID_MEDIA_FILE_ID"] = 9;
                                values[valuesById[10] = "INVALID_MEDIA_SUB_TYPE"] = 10;
                                values[valuesById[11] = "INVALID_MEDIA_FILE_TYPE"] = 11;
                                values[valuesById[12] = "INVALID_MIME_TYPE"] = 12;
                                values[valuesById[13] = "INVALID_REFERENCE_ID"] = 13;
                                values[valuesById[14] = "INVALID_YOU_TUBE_ID"] = 14;
                                values[valuesById[15] = "MEDIA_FILE_FAILED_TRANSCODING"] = 15;
                                values[valuesById[16] = "MEDIA_NOT_TRANSCODED"] = 16;
                                values[valuesById[17] = "MEDIA_TYPE_DOES_NOT_MATCH_MEDIA_FILE_TYPE"] = 17;
                                values[valuesById[18] = "NO_FIELDS_SPECIFIED"] = 18;
                                values[valuesById[19] = "NULL_REFERENCE_ID_AND_MEDIA_ID"] = 19;
                                values[valuesById[20] = "TOO_LONG"] = 20;
                                values[valuesById[21] = "UNSUPPORTED_TYPE"] = 21;
                                values[valuesById[22] = "YOU_TUBE_SERVICE_UNAVAILABLE"] = 22;
                                values[valuesById[23] = "YOU_TUBE_VIDEO_HAS_NON_POSITIVE_DURATION"] = 23;
                                values[valuesById[24] = "YOU_TUBE_VIDEO_NOT_FOUND"] = 24;
                                return values;
                            })();
    
                            return MediaFileErrorEnum;
                        })();
    
                        errors.MediaUploadErrorEnum = (function() {
    
                            /**
                             * Properties of a MediaUploadErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IMediaUploadErrorEnum
                             */
    
                            /**
                             * Constructs a new MediaUploadErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a MediaUploadErrorEnum.
                             * @implements IMediaUploadErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IMediaUploadErrorEnum=} [p] Properties to set
                             */
                            function MediaUploadErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MediaUploadErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaUploadErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.MediaUploadErrorEnum} MediaUploadErrorEnum instance
                             */
                            MediaUploadErrorEnum.create = function create(properties) {
                                return new MediaUploadErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MediaUploadErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.MediaUploadErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaUploadErrorEnum} m MediaUploadErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaUploadErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaUploadErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.MediaUploadErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMediaUploadErrorEnum} message MediaUploadErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaUploadErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaUploadErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.MediaUploadErrorEnum} MediaUploadErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaUploadErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.MediaUploadErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaUploadErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.MediaUploadErrorEnum} MediaUploadErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaUploadErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaUploadErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaUploadErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MediaUploadErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.MediaUploadErrorEnum} MediaUploadErrorEnum
                             */
                            MediaUploadErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.MediaUploadErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.MediaUploadErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MediaUploadErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.MediaUploadErrorEnum} m MediaUploadErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaUploadErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MediaUploadErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.MediaUploadErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaUploadErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MediaUploadError enum.
                             * @name google.ads.googleads.v11.errors.MediaUploadErrorEnum.MediaUploadError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} FILE_TOO_BIG=2 FILE_TOO_BIG value
                             * @property {number} UNPARSEABLE_IMAGE=3 UNPARSEABLE_IMAGE value
                             * @property {number} ANIMATED_IMAGE_NOT_ALLOWED=4 ANIMATED_IMAGE_NOT_ALLOWED value
                             * @property {number} FORMAT_NOT_ALLOWED=5 FORMAT_NOT_ALLOWED value
                             * @property {number} EXTERNAL_URL_NOT_ALLOWED=6 EXTERNAL_URL_NOT_ALLOWED value
                             * @property {number} INVALID_URL_REFERENCE=7 INVALID_URL_REFERENCE value
                             * @property {number} MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY=8 MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY value
                             * @property {number} ANIMATED_VISUAL_EFFECT=9 ANIMATED_VISUAL_EFFECT value
                             * @property {number} ANIMATION_TOO_LONG=10 ANIMATION_TOO_LONG value
                             * @property {number} ASPECT_RATIO_NOT_ALLOWED=11 ASPECT_RATIO_NOT_ALLOWED value
                             * @property {number} AUDIO_NOT_ALLOWED_IN_MEDIA_BUNDLE=12 AUDIO_NOT_ALLOWED_IN_MEDIA_BUNDLE value
                             * @property {number} CMYK_JPEG_NOT_ALLOWED=13 CMYK_JPEG_NOT_ALLOWED value
                             * @property {number} FLASH_NOT_ALLOWED=14 FLASH_NOT_ALLOWED value
                             * @property {number} FRAME_RATE_TOO_HIGH=15 FRAME_RATE_TOO_HIGH value
                             * @property {number} GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED=16 GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED value
                             * @property {number} IMAGE_CONSTRAINTS_VIOLATED=17 IMAGE_CONSTRAINTS_VIOLATED value
                             * @property {number} INVALID_MEDIA_BUNDLE=18 INVALID_MEDIA_BUNDLE value
                             * @property {number} INVALID_MEDIA_BUNDLE_ENTRY=19 INVALID_MEDIA_BUNDLE_ENTRY value
                             * @property {number} INVALID_MIME_TYPE=20 INVALID_MIME_TYPE value
                             * @property {number} INVALID_PATH=21 INVALID_PATH value
                             * @property {number} LAYOUT_PROBLEM=22 LAYOUT_PROBLEM value
                             * @property {number} MALFORMED_URL=23 MALFORMED_URL value
                             * @property {number} MEDIA_BUNDLE_NOT_ALLOWED=24 MEDIA_BUNDLE_NOT_ALLOWED value
                             * @property {number} MEDIA_BUNDLE_NOT_COMPATIBLE_TO_PRODUCT_TYPE=25 MEDIA_BUNDLE_NOT_COMPATIBLE_TO_PRODUCT_TYPE value
                             * @property {number} MEDIA_BUNDLE_REJECTED_BY_MULTIPLE_ASSET_SPECS=26 MEDIA_BUNDLE_REJECTED_BY_MULTIPLE_ASSET_SPECS value
                             * @property {number} TOO_MANY_FILES_IN_MEDIA_BUNDLE=27 TOO_MANY_FILES_IN_MEDIA_BUNDLE value
                             * @property {number} UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT=28 UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT value
                             * @property {number} UNSUPPORTED_HTML5_FEATURE=29 UNSUPPORTED_HTML5_FEATURE value
                             * @property {number} URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT=30 URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT value
                             * @property {number} VIDEO_FILE_NAME_TOO_LONG=31 VIDEO_FILE_NAME_TOO_LONG value
                             * @property {number} VIDEO_MULTIPLE_FILES_WITH_SAME_NAME=32 VIDEO_MULTIPLE_FILES_WITH_SAME_NAME value
                             * @property {number} VIDEO_NOT_ALLOWED_IN_MEDIA_BUNDLE=33 VIDEO_NOT_ALLOWED_IN_MEDIA_BUNDLE value
                             * @property {number} CANNOT_UPLOAD_MEDIA_TYPE_THROUGH_API=34 CANNOT_UPLOAD_MEDIA_TYPE_THROUGH_API value
                             * @property {number} DIMENSIONS_NOT_ALLOWED=35 DIMENSIONS_NOT_ALLOWED value
                             */
                            MediaUploadErrorEnum.MediaUploadError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "FILE_TOO_BIG"] = 2;
                                values[valuesById[3] = "UNPARSEABLE_IMAGE"] = 3;
                                values[valuesById[4] = "ANIMATED_IMAGE_NOT_ALLOWED"] = 4;
                                values[valuesById[5] = "FORMAT_NOT_ALLOWED"] = 5;
                                values[valuesById[6] = "EXTERNAL_URL_NOT_ALLOWED"] = 6;
                                values[valuesById[7] = "INVALID_URL_REFERENCE"] = 7;
                                values[valuesById[8] = "MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY"] = 8;
                                values[valuesById[9] = "ANIMATED_VISUAL_EFFECT"] = 9;
                                values[valuesById[10] = "ANIMATION_TOO_LONG"] = 10;
                                values[valuesById[11] = "ASPECT_RATIO_NOT_ALLOWED"] = 11;
                                values[valuesById[12] = "AUDIO_NOT_ALLOWED_IN_MEDIA_BUNDLE"] = 12;
                                values[valuesById[13] = "CMYK_JPEG_NOT_ALLOWED"] = 13;
                                values[valuesById[14] = "FLASH_NOT_ALLOWED"] = 14;
                                values[valuesById[15] = "FRAME_RATE_TOO_HIGH"] = 15;
                                values[valuesById[16] = "GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED"] = 16;
                                values[valuesById[17] = "IMAGE_CONSTRAINTS_VIOLATED"] = 17;
                                values[valuesById[18] = "INVALID_MEDIA_BUNDLE"] = 18;
                                values[valuesById[19] = "INVALID_MEDIA_BUNDLE_ENTRY"] = 19;
                                values[valuesById[20] = "INVALID_MIME_TYPE"] = 20;
                                values[valuesById[21] = "INVALID_PATH"] = 21;
                                values[valuesById[22] = "LAYOUT_PROBLEM"] = 22;
                                values[valuesById[23] = "MALFORMED_URL"] = 23;
                                values[valuesById[24] = "MEDIA_BUNDLE_NOT_ALLOWED"] = 24;
                                values[valuesById[25] = "MEDIA_BUNDLE_NOT_COMPATIBLE_TO_PRODUCT_TYPE"] = 25;
                                values[valuesById[26] = "MEDIA_BUNDLE_REJECTED_BY_MULTIPLE_ASSET_SPECS"] = 26;
                                values[valuesById[27] = "TOO_MANY_FILES_IN_MEDIA_BUNDLE"] = 27;
                                values[valuesById[28] = "UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT"] = 28;
                                values[valuesById[29] = "UNSUPPORTED_HTML5_FEATURE"] = 29;
                                values[valuesById[30] = "URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT"] = 30;
                                values[valuesById[31] = "VIDEO_FILE_NAME_TOO_LONG"] = 31;
                                values[valuesById[32] = "VIDEO_MULTIPLE_FILES_WITH_SAME_NAME"] = 32;
                                values[valuesById[33] = "VIDEO_NOT_ALLOWED_IN_MEDIA_BUNDLE"] = 33;
                                values[valuesById[34] = "CANNOT_UPLOAD_MEDIA_TYPE_THROUGH_API"] = 34;
                                values[valuesById[35] = "DIMENSIONS_NOT_ALLOWED"] = 35;
                                return values;
                            })();
    
                            return MediaUploadErrorEnum;
                        })();
    
                        errors.MerchantCenterErrorEnum = (function() {
    
                            /**
                             * Properties of a MerchantCenterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IMerchantCenterErrorEnum
                             */
    
                            /**
                             * Constructs a new MerchantCenterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a MerchantCenterErrorEnum.
                             * @implements IMerchantCenterErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IMerchantCenterErrorEnum=} [p] Properties to set
                             */
                            function MerchantCenterErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MerchantCenterErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMerchantCenterErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.MerchantCenterErrorEnum} MerchantCenterErrorEnum instance
                             */
                            MerchantCenterErrorEnum.create = function create(properties) {
                                return new MerchantCenterErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MerchantCenterErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.MerchantCenterErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMerchantCenterErrorEnum} m MerchantCenterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MerchantCenterErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.MerchantCenterErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMerchantCenterErrorEnum} message MerchantCenterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MerchantCenterErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.MerchantCenterErrorEnum} MerchantCenterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.MerchantCenterErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MerchantCenterErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.MerchantCenterErrorEnum} MerchantCenterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MerchantCenterErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MerchantCenterErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MerchantCenterErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.MerchantCenterErrorEnum} MerchantCenterErrorEnum
                             */
                            MerchantCenterErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.MerchantCenterErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.MerchantCenterErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MerchantCenterErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.MerchantCenterErrorEnum} m MerchantCenterErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MerchantCenterErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MerchantCenterErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.MerchantCenterErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MerchantCenterErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MerchantCenterError enum.
                             * @name google.ads.googleads.v11.errors.MerchantCenterErrorEnum.MerchantCenterError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MERCHANT_ID_CANNOT_BE_ACCESSED=2 MERCHANT_ID_CANNOT_BE_ACCESSED value
                             * @property {number} CUSTOMER_NOT_ALLOWED_FOR_SHOPPING_PERFORMANCE_MAX=3 CUSTOMER_NOT_ALLOWED_FOR_SHOPPING_PERFORMANCE_MAX value
                             */
                            MerchantCenterErrorEnum.MerchantCenterError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MERCHANT_ID_CANNOT_BE_ACCESSED"] = 2;
                                values[valuesById[3] = "CUSTOMER_NOT_ALLOWED_FOR_SHOPPING_PERFORMANCE_MAX"] = 3;
                                return values;
                            })();
    
                            return MerchantCenterErrorEnum;
                        })();
    
                        errors.MultiplierErrorEnum = (function() {
    
                            /**
                             * Properties of a MultiplierErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IMultiplierErrorEnum
                             */
    
                            /**
                             * Constructs a new MultiplierErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a MultiplierErrorEnum.
                             * @implements IMultiplierErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IMultiplierErrorEnum=} [p] Properties to set
                             */
                            function MultiplierErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MultiplierErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMultiplierErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.MultiplierErrorEnum} MultiplierErrorEnum instance
                             */
                            MultiplierErrorEnum.create = function create(properties) {
                                return new MultiplierErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MultiplierErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.MultiplierErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMultiplierErrorEnum} m MultiplierErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MultiplierErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MultiplierErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.MultiplierErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMultiplierErrorEnum} message MultiplierErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MultiplierErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MultiplierErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.MultiplierErrorEnum} MultiplierErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MultiplierErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.MultiplierErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MultiplierErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.MultiplierErrorEnum} MultiplierErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MultiplierErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MultiplierErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MultiplierErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MultiplierErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.MultiplierErrorEnum} MultiplierErrorEnum
                             */
                            MultiplierErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.MultiplierErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.MultiplierErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MultiplierErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.MultiplierErrorEnum} m MultiplierErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MultiplierErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MultiplierErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.MultiplierErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MultiplierErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MultiplierError enum.
                             * @name google.ads.googleads.v11.errors.MultiplierErrorEnum.MultiplierError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} MULTIPLIER_TOO_HIGH=2 MULTIPLIER_TOO_HIGH value
                             * @property {number} MULTIPLIER_TOO_LOW=3 MULTIPLIER_TOO_LOW value
                             * @property {number} TOO_MANY_FRACTIONAL_DIGITS=4 TOO_MANY_FRACTIONAL_DIGITS value
                             * @property {number} MULTIPLIER_NOT_ALLOWED_FOR_BIDDING_STRATEGY=5 MULTIPLIER_NOT_ALLOWED_FOR_BIDDING_STRATEGY value
                             * @property {number} MULTIPLIER_NOT_ALLOWED_WHEN_BASE_BID_IS_MISSING=6 MULTIPLIER_NOT_ALLOWED_WHEN_BASE_BID_IS_MISSING value
                             * @property {number} NO_MULTIPLIER_SPECIFIED=7 NO_MULTIPLIER_SPECIFIED value
                             * @property {number} MULTIPLIER_CAUSES_BID_TO_EXCEED_DAILY_BUDGET=8 MULTIPLIER_CAUSES_BID_TO_EXCEED_DAILY_BUDGET value
                             * @property {number} MULTIPLIER_CAUSES_BID_TO_EXCEED_MONTHLY_BUDGET=9 MULTIPLIER_CAUSES_BID_TO_EXCEED_MONTHLY_BUDGET value
                             * @property {number} MULTIPLIER_CAUSES_BID_TO_EXCEED_CUSTOM_BUDGET=10 MULTIPLIER_CAUSES_BID_TO_EXCEED_CUSTOM_BUDGET value
                             * @property {number} MULTIPLIER_CAUSES_BID_TO_EXCEED_MAX_ALLOWED_BID=11 MULTIPLIER_CAUSES_BID_TO_EXCEED_MAX_ALLOWED_BID value
                             * @property {number} BID_LESS_THAN_MIN_ALLOWED_BID_WITH_MULTIPLIER=12 BID_LESS_THAN_MIN_ALLOWED_BID_WITH_MULTIPLIER value
                             * @property {number} MULTIPLIER_AND_BIDDING_STRATEGY_TYPE_MISMATCH=13 MULTIPLIER_AND_BIDDING_STRATEGY_TYPE_MISMATCH value
                             */
                            MultiplierErrorEnum.MultiplierError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "MULTIPLIER_TOO_HIGH"] = 2;
                                values[valuesById[3] = "MULTIPLIER_TOO_LOW"] = 3;
                                values[valuesById[4] = "TOO_MANY_FRACTIONAL_DIGITS"] = 4;
                                values[valuesById[5] = "MULTIPLIER_NOT_ALLOWED_FOR_BIDDING_STRATEGY"] = 5;
                                values[valuesById[6] = "MULTIPLIER_NOT_ALLOWED_WHEN_BASE_BID_IS_MISSING"] = 6;
                                values[valuesById[7] = "NO_MULTIPLIER_SPECIFIED"] = 7;
                                values[valuesById[8] = "MULTIPLIER_CAUSES_BID_TO_EXCEED_DAILY_BUDGET"] = 8;
                                values[valuesById[9] = "MULTIPLIER_CAUSES_BID_TO_EXCEED_MONTHLY_BUDGET"] = 9;
                                values[valuesById[10] = "MULTIPLIER_CAUSES_BID_TO_EXCEED_CUSTOM_BUDGET"] = 10;
                                values[valuesById[11] = "MULTIPLIER_CAUSES_BID_TO_EXCEED_MAX_ALLOWED_BID"] = 11;
                                values[valuesById[12] = "BID_LESS_THAN_MIN_ALLOWED_BID_WITH_MULTIPLIER"] = 12;
                                values[valuesById[13] = "MULTIPLIER_AND_BIDDING_STRATEGY_TYPE_MISMATCH"] = 13;
                                return values;
                            })();
    
                            return MultiplierErrorEnum;
                        })();
    
                        errors.MutateErrorEnum = (function() {
    
                            /**
                             * Properties of a MutateErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IMutateErrorEnum
                             */
    
                            /**
                             * Constructs a new MutateErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a MutateErrorEnum.
                             * @implements IMutateErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IMutateErrorEnum=} [p] Properties to set
                             */
                            function MutateErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new MutateErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMutateErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.MutateErrorEnum} MutateErrorEnum instance
                             */
                            MutateErrorEnum.create = function create(properties) {
                                return new MutateErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified MutateErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.MutateErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMutateErrorEnum} m MutateErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.MutateErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IMutateErrorEnum} message MutateErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.MutateErrorEnum} MutateErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.MutateErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.MutateErrorEnum} MutateErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a MutateErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.MutateErrorEnum} MutateErrorEnum
                             */
                            MutateErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.MutateErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.MutateErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a MutateErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.MutateErrorEnum} m MutateErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this MutateErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.MutateErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * MutateError enum.
                             * @name google.ads.googleads.v11.errors.MutateErrorEnum.MutateError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} RESOURCE_NOT_FOUND=3 RESOURCE_NOT_FOUND value
                             * @property {number} ID_EXISTS_IN_MULTIPLE_MUTATES=7 ID_EXISTS_IN_MULTIPLE_MUTATES value
                             * @property {number} INCONSISTENT_FIELD_VALUES=8 INCONSISTENT_FIELD_VALUES value
                             * @property {number} MUTATE_NOT_ALLOWED=9 MUTATE_NOT_ALLOWED value
                             * @property {number} RESOURCE_NOT_IN_GOOGLE_ADS=10 RESOURCE_NOT_IN_GOOGLE_ADS value
                             * @property {number} RESOURCE_ALREADY_EXISTS=11 RESOURCE_ALREADY_EXISTS value
                             * @property {number} RESOURCE_DOES_NOT_SUPPORT_VALIDATE_ONLY=12 RESOURCE_DOES_NOT_SUPPORT_VALIDATE_ONLY value
                             * @property {number} OPERATION_DOES_NOT_SUPPORT_PARTIAL_FAILURE=16 OPERATION_DOES_NOT_SUPPORT_PARTIAL_FAILURE value
                             * @property {number} RESOURCE_READ_ONLY=13 RESOURCE_READ_ONLY value
                             */
                            MutateErrorEnum.MutateError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "RESOURCE_NOT_FOUND"] = 3;
                                values[valuesById[7] = "ID_EXISTS_IN_MULTIPLE_MUTATES"] = 7;
                                values[valuesById[8] = "INCONSISTENT_FIELD_VALUES"] = 8;
                                values[valuesById[9] = "MUTATE_NOT_ALLOWED"] = 9;
                                values[valuesById[10] = "RESOURCE_NOT_IN_GOOGLE_ADS"] = 10;
                                values[valuesById[11] = "RESOURCE_ALREADY_EXISTS"] = 11;
                                values[valuesById[12] = "RESOURCE_DOES_NOT_SUPPORT_VALIDATE_ONLY"] = 12;
                                values[valuesById[16] = "OPERATION_DOES_NOT_SUPPORT_PARTIAL_FAILURE"] = 16;
                                values[valuesById[13] = "RESOURCE_READ_ONLY"] = 13;
                                return values;
                            })();
    
                            return MutateErrorEnum;
                        })();
    
                        errors.NewResourceCreationErrorEnum = (function() {
    
                            /**
                             * Properties of a NewResourceCreationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface INewResourceCreationErrorEnum
                             */
    
                            /**
                             * Constructs a new NewResourceCreationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a NewResourceCreationErrorEnum.
                             * @implements INewResourceCreationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.INewResourceCreationErrorEnum=} [p] Properties to set
                             */
                            function NewResourceCreationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new NewResourceCreationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INewResourceCreationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.NewResourceCreationErrorEnum} NewResourceCreationErrorEnum instance
                             */
                            NewResourceCreationErrorEnum.create = function create(properties) {
                                return new NewResourceCreationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified NewResourceCreationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.NewResourceCreationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INewResourceCreationErrorEnum} m NewResourceCreationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NewResourceCreationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified NewResourceCreationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.NewResourceCreationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INewResourceCreationErrorEnum} message NewResourceCreationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NewResourceCreationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a NewResourceCreationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.NewResourceCreationErrorEnum} NewResourceCreationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NewResourceCreationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.NewResourceCreationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a NewResourceCreationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.NewResourceCreationErrorEnum} NewResourceCreationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NewResourceCreationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a NewResourceCreationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            NewResourceCreationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a NewResourceCreationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.NewResourceCreationErrorEnum} NewResourceCreationErrorEnum
                             */
                            NewResourceCreationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.NewResourceCreationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.NewResourceCreationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a NewResourceCreationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.NewResourceCreationErrorEnum} m NewResourceCreationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            NewResourceCreationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this NewResourceCreationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.NewResourceCreationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            NewResourceCreationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * NewResourceCreationError enum.
                             * @name google.ads.googleads.v11.errors.NewResourceCreationErrorEnum.NewResourceCreationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CANNOT_SET_ID_FOR_CREATE=2 CANNOT_SET_ID_FOR_CREATE value
                             * @property {number} DUPLICATE_TEMP_IDS=3 DUPLICATE_TEMP_IDS value
                             * @property {number} TEMP_ID_RESOURCE_HAD_ERRORS=4 TEMP_ID_RESOURCE_HAD_ERRORS value
                             */
                            NewResourceCreationErrorEnum.NewResourceCreationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CANNOT_SET_ID_FOR_CREATE"] = 2;
                                values[valuesById[3] = "DUPLICATE_TEMP_IDS"] = 3;
                                values[valuesById[4] = "TEMP_ID_RESOURCE_HAD_ERRORS"] = 4;
                                return values;
                            })();
    
                            return NewResourceCreationErrorEnum;
                        })();
    
                        errors.NotAllowlistedErrorEnum = (function() {
    
                            /**
                             * Properties of a NotAllowlistedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface INotAllowlistedErrorEnum
                             */
    
                            /**
                             * Constructs a new NotAllowlistedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a NotAllowlistedErrorEnum.
                             * @implements INotAllowlistedErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.INotAllowlistedErrorEnum=} [p] Properties to set
                             */
                            function NotAllowlistedErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new NotAllowlistedErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INotAllowlistedErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.NotAllowlistedErrorEnum} NotAllowlistedErrorEnum instance
                             */
                            NotAllowlistedErrorEnum.create = function create(properties) {
                                return new NotAllowlistedErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified NotAllowlistedErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.NotAllowlistedErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INotAllowlistedErrorEnum} m NotAllowlistedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NotAllowlistedErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified NotAllowlistedErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.NotAllowlistedErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INotAllowlistedErrorEnum} message NotAllowlistedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NotAllowlistedErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a NotAllowlistedErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.NotAllowlistedErrorEnum} NotAllowlistedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NotAllowlistedErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.NotAllowlistedErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a NotAllowlistedErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.NotAllowlistedErrorEnum} NotAllowlistedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NotAllowlistedErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a NotAllowlistedErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            NotAllowlistedErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a NotAllowlistedErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.NotAllowlistedErrorEnum} NotAllowlistedErrorEnum
                             */
                            NotAllowlistedErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.NotAllowlistedErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.NotAllowlistedErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a NotAllowlistedErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.NotAllowlistedErrorEnum} m NotAllowlistedErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            NotAllowlistedErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this NotAllowlistedErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.NotAllowlistedErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            NotAllowlistedErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * NotAllowlistedError enum.
                             * @name google.ads.googleads.v11.errors.NotAllowlistedErrorEnum.NotAllowlistedError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CUSTOMER_NOT_ALLOWLISTED_FOR_THIS_FEATURE=2 CUSTOMER_NOT_ALLOWLISTED_FOR_THIS_FEATURE value
                             */
                            NotAllowlistedErrorEnum.NotAllowlistedError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CUSTOMER_NOT_ALLOWLISTED_FOR_THIS_FEATURE"] = 2;
                                return values;
                            })();
    
                            return NotAllowlistedErrorEnum;
                        })();
    
                        errors.NotEmptyErrorEnum = (function() {
    
                            /**
                             * Properties of a NotEmptyErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface INotEmptyErrorEnum
                             */
    
                            /**
                             * Constructs a new NotEmptyErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a NotEmptyErrorEnum.
                             * @implements INotEmptyErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.INotEmptyErrorEnum=} [p] Properties to set
                             */
                            function NotEmptyErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new NotEmptyErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INotEmptyErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.NotEmptyErrorEnum} NotEmptyErrorEnum instance
                             */
                            NotEmptyErrorEnum.create = function create(properties) {
                                return new NotEmptyErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified NotEmptyErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.NotEmptyErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INotEmptyErrorEnum} m NotEmptyErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NotEmptyErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified NotEmptyErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.NotEmptyErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INotEmptyErrorEnum} message NotEmptyErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NotEmptyErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a NotEmptyErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.NotEmptyErrorEnum} NotEmptyErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NotEmptyErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.NotEmptyErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a NotEmptyErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.NotEmptyErrorEnum} NotEmptyErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NotEmptyErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a NotEmptyErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            NotEmptyErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a NotEmptyErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.NotEmptyErrorEnum} NotEmptyErrorEnum
                             */
                            NotEmptyErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.NotEmptyErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.NotEmptyErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a NotEmptyErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.NotEmptyErrorEnum} m NotEmptyErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            NotEmptyErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this NotEmptyErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.NotEmptyErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            NotEmptyErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * NotEmptyError enum.
                             * @name google.ads.googleads.v11.errors.NotEmptyErrorEnum.NotEmptyError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EMPTY_LIST=2 EMPTY_LIST value
                             */
                            NotEmptyErrorEnum.NotEmptyError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EMPTY_LIST"] = 2;
                                return values;
                            })();
    
                            return NotEmptyErrorEnum;
                        })();
    
                        errors.NullErrorEnum = (function() {
    
                            /**
                             * Properties of a NullErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface INullErrorEnum
                             */
    
                            /**
                             * Constructs a new NullErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a NullErrorEnum.
                             * @implements INullErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.INullErrorEnum=} [p] Properties to set
                             */
                            function NullErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new NullErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INullErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.NullErrorEnum} NullErrorEnum instance
                             */
                            NullErrorEnum.create = function create(properties) {
                                return new NullErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified NullErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.NullErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INullErrorEnum} m NullErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NullErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified NullErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.NullErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.INullErrorEnum} message NullErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            NullErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a NullErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.NullErrorEnum} NullErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NullErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.NullErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a NullErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.NullErrorEnum} NullErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            NullErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a NullErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            NullErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a NullErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.NullErrorEnum} NullErrorEnum
                             */
                            NullErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.NullErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.NullErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a NullErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.NullErrorEnum} m NullErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            NullErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this NullErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.NullErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            NullErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * NullError enum.
                             * @name google.ads.googleads.v11.errors.NullErrorEnum.NullError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NULL_CONTENT=2 NULL_CONTENT value
                             */
                            NullErrorEnum.NullError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NULL_CONTENT"] = 2;
                                return values;
                            })();
    
                            return NullErrorEnum;
                        })();
    
                        errors.OfflineUserDataJobErrorEnum = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJobErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IOfflineUserDataJobErrorEnum
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJobErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an OfflineUserDataJobErrorEnum.
                             * @implements IOfflineUserDataJobErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IOfflineUserDataJobErrorEnum=} [p] Properties to set
                             */
                            function OfflineUserDataJobErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OfflineUserDataJobErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOfflineUserDataJobErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum} OfflineUserDataJobErrorEnum instance
                             */
                            OfflineUserDataJobErrorEnum.create = function create(properties) {
                                return new OfflineUserDataJobErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOfflineUserDataJobErrorEnum} m OfflineUserDataJobErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOfflineUserDataJobErrorEnum} message OfflineUserDataJobErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum} OfflineUserDataJobErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum} OfflineUserDataJobErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJobErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJobErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJobErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum} OfflineUserDataJobErrorEnum
                             */
                            OfflineUserDataJobErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJobErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum} m OfflineUserDataJobErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJobErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OfflineUserDataJobErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJobErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OfflineUserDataJobError enum.
                             * @name google.ads.googleads.v11.errors.OfflineUserDataJobErrorEnum.OfflineUserDataJobError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_USER_LIST_ID=3 INVALID_USER_LIST_ID value
                             * @property {number} INVALID_USER_LIST_TYPE=4 INVALID_USER_LIST_TYPE value
                             * @property {number} NOT_ON_ALLOWLIST_FOR_USER_ID=33 NOT_ON_ALLOWLIST_FOR_USER_ID value
                             * @property {number} INCOMPATIBLE_UPLOAD_KEY_TYPE=6 INCOMPATIBLE_UPLOAD_KEY_TYPE value
                             * @property {number} MISSING_USER_IDENTIFIER=7 MISSING_USER_IDENTIFIER value
                             * @property {number} INVALID_MOBILE_ID_FORMAT=8 INVALID_MOBILE_ID_FORMAT value
                             * @property {number} TOO_MANY_USER_IDENTIFIERS=9 TOO_MANY_USER_IDENTIFIERS value
                             * @property {number} NOT_ON_ALLOWLIST_FOR_STORE_SALES_DIRECT=31 NOT_ON_ALLOWLIST_FOR_STORE_SALES_DIRECT value
                             * @property {number} NOT_ON_ALLOWLIST_FOR_UNIFIED_STORE_SALES=32 NOT_ON_ALLOWLIST_FOR_UNIFIED_STORE_SALES value
                             * @property {number} INVALID_PARTNER_ID=11 INVALID_PARTNER_ID value
                             * @property {number} INVALID_ENCODING=12 INVALID_ENCODING value
                             * @property {number} INVALID_COUNTRY_CODE=13 INVALID_COUNTRY_CODE value
                             * @property {number} INCOMPATIBLE_USER_IDENTIFIER=14 INCOMPATIBLE_USER_IDENTIFIER value
                             * @property {number} FUTURE_TRANSACTION_TIME=15 FUTURE_TRANSACTION_TIME value
                             * @property {number} INVALID_CONVERSION_ACTION=16 INVALID_CONVERSION_ACTION value
                             * @property {number} MOBILE_ID_NOT_SUPPORTED=17 MOBILE_ID_NOT_SUPPORTED value
                             * @property {number} INVALID_OPERATION_ORDER=18 INVALID_OPERATION_ORDER value
                             * @property {number} CONFLICTING_OPERATION=19 CONFLICTING_OPERATION value
                             * @property {number} EXTERNAL_UPDATE_ID_ALREADY_EXISTS=21 EXTERNAL_UPDATE_ID_ALREADY_EXISTS value
                             * @property {number} JOB_ALREADY_STARTED=22 JOB_ALREADY_STARTED value
                             * @property {number} REMOVE_NOT_SUPPORTED=23 REMOVE_NOT_SUPPORTED value
                             * @property {number} REMOVE_ALL_NOT_SUPPORTED=24 REMOVE_ALL_NOT_SUPPORTED value
                             * @property {number} INVALID_SHA256_FORMAT=25 INVALID_SHA256_FORMAT value
                             * @property {number} CUSTOM_KEY_DISABLED=26 CUSTOM_KEY_DISABLED value
                             * @property {number} CUSTOM_KEY_NOT_PREDEFINED=27 CUSTOM_KEY_NOT_PREDEFINED value
                             * @property {number} CUSTOM_KEY_NOT_SET=29 CUSTOM_KEY_NOT_SET value
                             * @property {number} CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS=30 CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS value
                             * @property {number} ATTRIBUTES_NOT_APPLICABLE_FOR_CUSTOMER_MATCH_USER_LIST=34 ATTRIBUTES_NOT_APPLICABLE_FOR_CUSTOMER_MATCH_USER_LIST value
                             * @property {number} LIFETIME_VALUE_BUCKET_NOT_IN_RANGE=35 LIFETIME_VALUE_BUCKET_NOT_IN_RANGE value
                             * @property {number} INCOMPATIBLE_USER_IDENTIFIER_FOR_ATTRIBUTES=36 INCOMPATIBLE_USER_IDENTIFIER_FOR_ATTRIBUTES value
                             * @property {number} FUTURE_TIME_NOT_ALLOWED=37 FUTURE_TIME_NOT_ALLOWED value
                             * @property {number} LAST_PURCHASE_TIME_LESS_THAN_ACQUISITION_TIME=38 LAST_PURCHASE_TIME_LESS_THAN_ACQUISITION_TIME value
                             * @property {number} CUSTOMER_IDENTIFIER_NOT_ALLOWED=39 CUSTOMER_IDENTIFIER_NOT_ALLOWED value
                             * @property {number} INVALID_ITEM_ID=40 INVALID_ITEM_ID value
                             */
                            OfflineUserDataJobErrorEnum.OfflineUserDataJobError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "INVALID_USER_LIST_ID"] = 3;
                                values[valuesById[4] = "INVALID_USER_LIST_TYPE"] = 4;
                                values[valuesById[33] = "NOT_ON_ALLOWLIST_FOR_USER_ID"] = 33;
                                values[valuesById[6] = "INCOMPATIBLE_UPLOAD_KEY_TYPE"] = 6;
                                values[valuesById[7] = "MISSING_USER_IDENTIFIER"] = 7;
                                values[valuesById[8] = "INVALID_MOBILE_ID_FORMAT"] = 8;
                                values[valuesById[9] = "TOO_MANY_USER_IDENTIFIERS"] = 9;
                                values[valuesById[31] = "NOT_ON_ALLOWLIST_FOR_STORE_SALES_DIRECT"] = 31;
                                values[valuesById[32] = "NOT_ON_ALLOWLIST_FOR_UNIFIED_STORE_SALES"] = 32;
                                values[valuesById[11] = "INVALID_PARTNER_ID"] = 11;
                                values[valuesById[12] = "INVALID_ENCODING"] = 12;
                                values[valuesById[13] = "INVALID_COUNTRY_CODE"] = 13;
                                values[valuesById[14] = "INCOMPATIBLE_USER_IDENTIFIER"] = 14;
                                values[valuesById[15] = "FUTURE_TRANSACTION_TIME"] = 15;
                                values[valuesById[16] = "INVALID_CONVERSION_ACTION"] = 16;
                                values[valuesById[17] = "MOBILE_ID_NOT_SUPPORTED"] = 17;
                                values[valuesById[18] = "INVALID_OPERATION_ORDER"] = 18;
                                values[valuesById[19] = "CONFLICTING_OPERATION"] = 19;
                                values[valuesById[21] = "EXTERNAL_UPDATE_ID_ALREADY_EXISTS"] = 21;
                                values[valuesById[22] = "JOB_ALREADY_STARTED"] = 22;
                                values[valuesById[23] = "REMOVE_NOT_SUPPORTED"] = 23;
                                values[valuesById[24] = "REMOVE_ALL_NOT_SUPPORTED"] = 24;
                                values[valuesById[25] = "INVALID_SHA256_FORMAT"] = 25;
                                values[valuesById[26] = "CUSTOM_KEY_DISABLED"] = 26;
                                values[valuesById[27] = "CUSTOM_KEY_NOT_PREDEFINED"] = 27;
                                values[valuesById[29] = "CUSTOM_KEY_NOT_SET"] = 29;
                                values[valuesById[30] = "CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS"] = 30;
                                values[valuesById[34] = "ATTRIBUTES_NOT_APPLICABLE_FOR_CUSTOMER_MATCH_USER_LIST"] = 34;
                                values[valuesById[35] = "LIFETIME_VALUE_BUCKET_NOT_IN_RANGE"] = 35;
                                values[valuesById[36] = "INCOMPATIBLE_USER_IDENTIFIER_FOR_ATTRIBUTES"] = 36;
                                values[valuesById[37] = "FUTURE_TIME_NOT_ALLOWED"] = 37;
                                values[valuesById[38] = "LAST_PURCHASE_TIME_LESS_THAN_ACQUISITION_TIME"] = 38;
                                values[valuesById[39] = "CUSTOMER_IDENTIFIER_NOT_ALLOWED"] = 39;
                                values[valuesById[40] = "INVALID_ITEM_ID"] = 40;
                                return values;
                            })();
    
                            return OfflineUserDataJobErrorEnum;
                        })();
    
                        errors.OperationAccessDeniedErrorEnum = (function() {
    
                            /**
                             * Properties of an OperationAccessDeniedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IOperationAccessDeniedErrorEnum
                             */
    
                            /**
                             * Constructs a new OperationAccessDeniedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an OperationAccessDeniedErrorEnum.
                             * @implements IOperationAccessDeniedErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IOperationAccessDeniedErrorEnum=} [p] Properties to set
                             */
                            function OperationAccessDeniedErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OperationAccessDeniedErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOperationAccessDeniedErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum} OperationAccessDeniedErrorEnum instance
                             */
                            OperationAccessDeniedErrorEnum.create = function create(properties) {
                                return new OperationAccessDeniedErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OperationAccessDeniedErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOperationAccessDeniedErrorEnum} m OperationAccessDeniedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperationAccessDeniedErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OperationAccessDeniedErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOperationAccessDeniedErrorEnum} message OperationAccessDeniedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperationAccessDeniedErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OperationAccessDeniedErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum} OperationAccessDeniedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperationAccessDeniedErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OperationAccessDeniedErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum} OperationAccessDeniedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperationAccessDeniedErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OperationAccessDeniedErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OperationAccessDeniedErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OperationAccessDeniedErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum} OperationAccessDeniedErrorEnum
                             */
                            OperationAccessDeniedErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an OperationAccessDeniedErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum} m OperationAccessDeniedErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OperationAccessDeniedErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OperationAccessDeniedErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OperationAccessDeniedErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OperationAccessDeniedError enum.
                             * @name google.ads.googleads.v11.errors.OperationAccessDeniedErrorEnum.OperationAccessDeniedError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ACTION_NOT_PERMITTED=2 ACTION_NOT_PERMITTED value
                             * @property {number} CREATE_OPERATION_NOT_PERMITTED=3 CREATE_OPERATION_NOT_PERMITTED value
                             * @property {number} REMOVE_OPERATION_NOT_PERMITTED=4 REMOVE_OPERATION_NOT_PERMITTED value
                             * @property {number} UPDATE_OPERATION_NOT_PERMITTED=5 UPDATE_OPERATION_NOT_PERMITTED value
                             * @property {number} MUTATE_ACTION_NOT_PERMITTED_FOR_CLIENT=6 MUTATE_ACTION_NOT_PERMITTED_FOR_CLIENT value
                             * @property {number} OPERATION_NOT_PERMITTED_FOR_CAMPAIGN_TYPE=7 OPERATION_NOT_PERMITTED_FOR_CAMPAIGN_TYPE value
                             * @property {number} CREATE_AS_REMOVED_NOT_PERMITTED=8 CREATE_AS_REMOVED_NOT_PERMITTED value
                             * @property {number} OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE=9 OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE value
                             * @property {number} OPERATION_NOT_PERMITTED_FOR_AD_GROUP_TYPE=10 OPERATION_NOT_PERMITTED_FOR_AD_GROUP_TYPE value
                             * @property {number} MUTATE_NOT_PERMITTED_FOR_CUSTOMER=11 MUTATE_NOT_PERMITTED_FOR_CUSTOMER value
                             */
                            OperationAccessDeniedErrorEnum.OperationAccessDeniedError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ACTION_NOT_PERMITTED"] = 2;
                                values[valuesById[3] = "CREATE_OPERATION_NOT_PERMITTED"] = 3;
                                values[valuesById[4] = "REMOVE_OPERATION_NOT_PERMITTED"] = 4;
                                values[valuesById[5] = "UPDATE_OPERATION_NOT_PERMITTED"] = 5;
                                values[valuesById[6] = "MUTATE_ACTION_NOT_PERMITTED_FOR_CLIENT"] = 6;
                                values[valuesById[7] = "OPERATION_NOT_PERMITTED_FOR_CAMPAIGN_TYPE"] = 7;
                                values[valuesById[8] = "CREATE_AS_REMOVED_NOT_PERMITTED"] = 8;
                                values[valuesById[9] = "OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE"] = 9;
                                values[valuesById[10] = "OPERATION_NOT_PERMITTED_FOR_AD_GROUP_TYPE"] = 10;
                                values[valuesById[11] = "MUTATE_NOT_PERMITTED_FOR_CUSTOMER"] = 11;
                                return values;
                            })();
    
                            return OperationAccessDeniedErrorEnum;
                        })();
    
                        errors.OperatorErrorEnum = (function() {
    
                            /**
                             * Properties of an OperatorErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IOperatorErrorEnum
                             */
    
                            /**
                             * Constructs a new OperatorErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an OperatorErrorEnum.
                             * @implements IOperatorErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IOperatorErrorEnum=} [p] Properties to set
                             */
                            function OperatorErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new OperatorErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOperatorErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.OperatorErrorEnum} OperatorErrorEnum instance
                             */
                            OperatorErrorEnum.create = function create(properties) {
                                return new OperatorErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified OperatorErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.OperatorErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOperatorErrorEnum} m OperatorErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatorErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OperatorErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.OperatorErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IOperatorErrorEnum} message OperatorErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatorErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OperatorErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.OperatorErrorEnum} OperatorErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatorErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.OperatorErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OperatorErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.OperatorErrorEnum} OperatorErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatorErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OperatorErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OperatorErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an OperatorErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.OperatorErrorEnum} OperatorErrorEnum
                             */
                            OperatorErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.OperatorErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.OperatorErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an OperatorErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.OperatorErrorEnum} m OperatorErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OperatorErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this OperatorErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.OperatorErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OperatorErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * OperatorError enum.
                             * @name google.ads.googleads.v11.errors.OperatorErrorEnum.OperatorError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OPERATOR_NOT_SUPPORTED=2 OPERATOR_NOT_SUPPORTED value
                             */
                            OperatorErrorEnum.OperatorError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OPERATOR_NOT_SUPPORTED"] = 2;
                                return values;
                            })();
    
                            return OperatorErrorEnum;
                        })();
    
                        errors.PartialFailureErrorEnum = (function() {
    
                            /**
                             * Properties of a PartialFailureErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IPartialFailureErrorEnum
                             */
    
                            /**
                             * Constructs a new PartialFailureErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a PartialFailureErrorEnum.
                             * @implements IPartialFailureErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IPartialFailureErrorEnum=} [p] Properties to set
                             */
                            function PartialFailureErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PartialFailureErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPartialFailureErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.PartialFailureErrorEnum} PartialFailureErrorEnum instance
                             */
                            PartialFailureErrorEnum.create = function create(properties) {
                                return new PartialFailureErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PartialFailureErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.PartialFailureErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPartialFailureErrorEnum} m PartialFailureErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PartialFailureErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PartialFailureErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.PartialFailureErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPartialFailureErrorEnum} message PartialFailureErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PartialFailureErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PartialFailureErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.PartialFailureErrorEnum} PartialFailureErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PartialFailureErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.PartialFailureErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PartialFailureErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.PartialFailureErrorEnum} PartialFailureErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PartialFailureErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PartialFailureErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PartialFailureErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PartialFailureErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.PartialFailureErrorEnum} PartialFailureErrorEnum
                             */
                            PartialFailureErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.PartialFailureErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.PartialFailureErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a PartialFailureErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.PartialFailureErrorEnum} m PartialFailureErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PartialFailureErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PartialFailureErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.PartialFailureErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PartialFailureErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PartialFailureError enum.
                             * @name google.ads.googleads.v11.errors.PartialFailureErrorEnum.PartialFailureError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} PARTIAL_FAILURE_MODE_REQUIRED=2 PARTIAL_FAILURE_MODE_REQUIRED value
                             */
                            PartialFailureErrorEnum.PartialFailureError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "PARTIAL_FAILURE_MODE_REQUIRED"] = 2;
                                return values;
                            })();
    
                            return PartialFailureErrorEnum;
                        })();
    
                        errors.PaymentsAccountErrorEnum = (function() {
    
                            /**
                             * Properties of a PaymentsAccountErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IPaymentsAccountErrorEnum
                             */
    
                            /**
                             * Constructs a new PaymentsAccountErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a PaymentsAccountErrorEnum.
                             * @implements IPaymentsAccountErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IPaymentsAccountErrorEnum=} [p] Properties to set
                             */
                            function PaymentsAccountErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PaymentsAccountErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPaymentsAccountErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.PaymentsAccountErrorEnum} PaymentsAccountErrorEnum instance
                             */
                            PaymentsAccountErrorEnum.create = function create(properties) {
                                return new PaymentsAccountErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PaymentsAccountErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.PaymentsAccountErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPaymentsAccountErrorEnum} m PaymentsAccountErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaymentsAccountErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PaymentsAccountErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.PaymentsAccountErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPaymentsAccountErrorEnum} message PaymentsAccountErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaymentsAccountErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PaymentsAccountErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.PaymentsAccountErrorEnum} PaymentsAccountErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaymentsAccountErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.PaymentsAccountErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PaymentsAccountErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.PaymentsAccountErrorEnum} PaymentsAccountErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaymentsAccountErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PaymentsAccountErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PaymentsAccountErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PaymentsAccountErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.PaymentsAccountErrorEnum} PaymentsAccountErrorEnum
                             */
                            PaymentsAccountErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.PaymentsAccountErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.PaymentsAccountErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a PaymentsAccountErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.PaymentsAccountErrorEnum} m PaymentsAccountErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PaymentsAccountErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PaymentsAccountErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.PaymentsAccountErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PaymentsAccountErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PaymentsAccountError enum.
                             * @name google.ads.googleads.v11.errors.PaymentsAccountErrorEnum.PaymentsAccountError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOT_SUPPORTED_FOR_MANAGER_CUSTOMER=2 NOT_SUPPORTED_FOR_MANAGER_CUSTOMER value
                             */
                            PaymentsAccountErrorEnum.PaymentsAccountError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOT_SUPPORTED_FOR_MANAGER_CUSTOMER"] = 2;
                                return values;
                            })();
    
                            return PaymentsAccountErrorEnum;
                        })();
    
                        errors.PolicyFindingErrorEnum = (function() {
    
                            /**
                             * Properties of a PolicyFindingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IPolicyFindingErrorEnum
                             */
    
                            /**
                             * Constructs a new PolicyFindingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a PolicyFindingErrorEnum.
                             * @implements IPolicyFindingErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingErrorEnum=} [p] Properties to set
                             */
                            function PolicyFindingErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyFindingErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingErrorEnum} PolicyFindingErrorEnum instance
                             */
                            PolicyFindingErrorEnum.create = function create(properties) {
                                return new PolicyFindingErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyFindingErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyFindingErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingErrorEnum} m PolicyFindingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyFindingErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyFindingErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyFindingErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyFindingErrorEnum} message PolicyFindingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyFindingErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyFindingErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingErrorEnum} PolicyFindingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyFindingErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.PolicyFindingErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyFindingErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingErrorEnum} PolicyFindingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyFindingErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyFindingErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyFindingErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyFindingErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.PolicyFindingErrorEnum} PolicyFindingErrorEnum
                             */
                            PolicyFindingErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.PolicyFindingErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.PolicyFindingErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyFindingErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.PolicyFindingErrorEnum} m PolicyFindingErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyFindingErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyFindingErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.PolicyFindingErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyFindingErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyFindingError enum.
                             * @name google.ads.googleads.v11.errors.PolicyFindingErrorEnum.PolicyFindingError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} POLICY_FINDING=2 POLICY_FINDING value
                             * @property {number} POLICY_TOPIC_NOT_FOUND=3 POLICY_TOPIC_NOT_FOUND value
                             */
                            PolicyFindingErrorEnum.PolicyFindingError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "POLICY_FINDING"] = 2;
                                values[valuesById[3] = "POLICY_TOPIC_NOT_FOUND"] = 3;
                                return values;
                            })();
    
                            return PolicyFindingErrorEnum;
                        })();
    
                        errors.PolicyValidationParameterErrorEnum = (function() {
    
                            /**
                             * Properties of a PolicyValidationParameterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IPolicyValidationParameterErrorEnum
                             */
    
                            /**
                             * Constructs a new PolicyValidationParameterErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a PolicyValidationParameterErrorEnum.
                             * @implements IPolicyValidationParameterErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IPolicyValidationParameterErrorEnum=} [p] Properties to set
                             */
                            function PolicyValidationParameterErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyValidationParameterErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyValidationParameterErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum} PolicyValidationParameterErrorEnum instance
                             */
                            PolicyValidationParameterErrorEnum.create = function create(properties) {
                                return new PolicyValidationParameterErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyValidationParameterErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyValidationParameterErrorEnum} m PolicyValidationParameterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyValidationParameterErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyValidationParameterErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyValidationParameterErrorEnum} message PolicyValidationParameterErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyValidationParameterErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyValidationParameterErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum} PolicyValidationParameterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyValidationParameterErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyValidationParameterErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum} PolicyValidationParameterErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyValidationParameterErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyValidationParameterErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyValidationParameterErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyValidationParameterErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum} PolicyValidationParameterErrorEnum
                             */
                            PolicyValidationParameterErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyValidationParameterErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum} m PolicyValidationParameterErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyValidationParameterErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyValidationParameterErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyValidationParameterErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyValidationParameterError enum.
                             * @name google.ads.googleads.v11.errors.PolicyValidationParameterErrorEnum.PolicyValidationParameterError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} UNSUPPORTED_AD_TYPE_FOR_IGNORABLE_POLICY_TOPICS=2 UNSUPPORTED_AD_TYPE_FOR_IGNORABLE_POLICY_TOPICS value
                             * @property {number} UNSUPPORTED_AD_TYPE_FOR_EXEMPT_POLICY_VIOLATION_KEYS=3 UNSUPPORTED_AD_TYPE_FOR_EXEMPT_POLICY_VIOLATION_KEYS value
                             * @property {number} CANNOT_SET_BOTH_IGNORABLE_POLICY_TOPICS_AND_EXEMPT_POLICY_VIOLATION_KEYS=4 CANNOT_SET_BOTH_IGNORABLE_POLICY_TOPICS_AND_EXEMPT_POLICY_VIOLATION_KEYS value
                             */
                            PolicyValidationParameterErrorEnum.PolicyValidationParameterError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "UNSUPPORTED_AD_TYPE_FOR_IGNORABLE_POLICY_TOPICS"] = 2;
                                values[valuesById[3] = "UNSUPPORTED_AD_TYPE_FOR_EXEMPT_POLICY_VIOLATION_KEYS"] = 3;
                                values[valuesById[4] = "CANNOT_SET_BOTH_IGNORABLE_POLICY_TOPICS_AND_EXEMPT_POLICY_VIOLATION_KEYS"] = 4;
                                return values;
                            })();
    
                            return PolicyValidationParameterErrorEnum;
                        })();
    
                        errors.PolicyViolationErrorEnum = (function() {
    
                            /**
                             * Properties of a PolicyViolationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IPolicyViolationErrorEnum
                             */
    
                            /**
                             * Constructs a new PolicyViolationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a PolicyViolationErrorEnum.
                             * @implements IPolicyViolationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationErrorEnum=} [p] Properties to set
                             */
                            function PolicyViolationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new PolicyViolationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationErrorEnum} PolicyViolationErrorEnum instance
                             */
                            PolicyViolationErrorEnum.create = function create(properties) {
                                return new PolicyViolationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified PolicyViolationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyViolationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationErrorEnum} m PolicyViolationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyViolationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PolicyViolationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.PolicyViolationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IPolicyViolationErrorEnum} message PolicyViolationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PolicyViolationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PolicyViolationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationErrorEnum} PolicyViolationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyViolationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.PolicyViolationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PolicyViolationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationErrorEnum} PolicyViolationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PolicyViolationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PolicyViolationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PolicyViolationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a PolicyViolationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.PolicyViolationErrorEnum} PolicyViolationErrorEnum
                             */
                            PolicyViolationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.PolicyViolationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.PolicyViolationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a PolicyViolationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.PolicyViolationErrorEnum} m PolicyViolationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PolicyViolationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this PolicyViolationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.PolicyViolationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PolicyViolationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * PolicyViolationError enum.
                             * @name google.ads.googleads.v11.errors.PolicyViolationErrorEnum.PolicyViolationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} POLICY_ERROR=2 POLICY_ERROR value
                             */
                            PolicyViolationErrorEnum.PolicyViolationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "POLICY_ERROR"] = 2;
                                return values;
                            })();
    
                            return PolicyViolationErrorEnum;
                        })();
    
                        errors.QueryErrorEnum = (function() {
    
                            /**
                             * Properties of a QueryErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IQueryErrorEnum
                             */
    
                            /**
                             * Constructs a new QueryErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a QueryErrorEnum.
                             * @implements IQueryErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IQueryErrorEnum=} [p] Properties to set
                             */
                            function QueryErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new QueryErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQueryErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.QueryErrorEnum} QueryErrorEnum instance
                             */
                            QueryErrorEnum.create = function create(properties) {
                                return new QueryErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified QueryErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.QueryErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQueryErrorEnum} m QueryErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QueryErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified QueryErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.QueryErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQueryErrorEnum} message QueryErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QueryErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a QueryErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.QueryErrorEnum} QueryErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QueryErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.QueryErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a QueryErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.QueryErrorEnum} QueryErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QueryErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a QueryErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            QueryErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a QueryErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.QueryErrorEnum} QueryErrorEnum
                             */
                            QueryErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.QueryErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.QueryErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a QueryErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.QueryErrorEnum} m QueryErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            QueryErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this QueryErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.QueryErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            QueryErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * QueryError enum.
                             * @name google.ads.googleads.v11.errors.QueryErrorEnum.QueryError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} QUERY_ERROR=50 QUERY_ERROR value
                             * @property {number} BAD_ENUM_CONSTANT=18 BAD_ENUM_CONSTANT value
                             * @property {number} BAD_ESCAPE_SEQUENCE=7 BAD_ESCAPE_SEQUENCE value
                             * @property {number} BAD_FIELD_NAME=12 BAD_FIELD_NAME value
                             * @property {number} BAD_LIMIT_VALUE=15 BAD_LIMIT_VALUE value
                             * @property {number} BAD_NUMBER=5 BAD_NUMBER value
                             * @property {number} BAD_OPERATOR=3 BAD_OPERATOR value
                             * @property {number} BAD_PARAMETER_NAME=61 BAD_PARAMETER_NAME value
                             * @property {number} BAD_PARAMETER_VALUE=62 BAD_PARAMETER_VALUE value
                             * @property {number} BAD_RESOURCE_TYPE_IN_FROM_CLAUSE=45 BAD_RESOURCE_TYPE_IN_FROM_CLAUSE value
                             * @property {number} BAD_SYMBOL=2 BAD_SYMBOL value
                             * @property {number} BAD_VALUE=4 BAD_VALUE value
                             * @property {number} DATE_RANGE_TOO_WIDE=36 DATE_RANGE_TOO_WIDE value
                             * @property {number} DATE_RANGE_TOO_NARROW=60 DATE_RANGE_TOO_NARROW value
                             * @property {number} EXPECTED_AND=30 EXPECTED_AND value
                             * @property {number} EXPECTED_BY=14 EXPECTED_BY value
                             * @property {number} EXPECTED_DIMENSION_FIELD_IN_SELECT_CLAUSE=37 EXPECTED_DIMENSION_FIELD_IN_SELECT_CLAUSE value
                             * @property {number} EXPECTED_FILTERS_ON_DATE_RANGE=55 EXPECTED_FILTERS_ON_DATE_RANGE value
                             * @property {number} EXPECTED_FROM=44 EXPECTED_FROM value
                             * @property {number} EXPECTED_LIST=41 EXPECTED_LIST value
                             * @property {number} EXPECTED_REFERENCED_FIELD_IN_SELECT_CLAUSE=16 EXPECTED_REFERENCED_FIELD_IN_SELECT_CLAUSE value
                             * @property {number} EXPECTED_SELECT=13 EXPECTED_SELECT value
                             * @property {number} EXPECTED_SINGLE_VALUE=42 EXPECTED_SINGLE_VALUE value
                             * @property {number} EXPECTED_VALUE_WITH_BETWEEN_OPERATOR=29 EXPECTED_VALUE_WITH_BETWEEN_OPERATOR value
                             * @property {number} INVALID_DATE_FORMAT=38 INVALID_DATE_FORMAT value
                             * @property {number} MISALIGNED_DATE_FOR_FILTER=64 MISALIGNED_DATE_FOR_FILTER value
                             * @property {number} INVALID_STRING_VALUE=57 INVALID_STRING_VALUE value
                             * @property {number} INVALID_VALUE_WITH_BETWEEN_OPERATOR=26 INVALID_VALUE_WITH_BETWEEN_OPERATOR value
                             * @property {number} INVALID_VALUE_WITH_DURING_OPERATOR=22 INVALID_VALUE_WITH_DURING_OPERATOR value
                             * @property {number} INVALID_VALUE_WITH_LIKE_OPERATOR=56 INVALID_VALUE_WITH_LIKE_OPERATOR value
                             * @property {number} OPERATOR_FIELD_MISMATCH=35 OPERATOR_FIELD_MISMATCH value
                             * @property {number} PROHIBITED_EMPTY_LIST_IN_CONDITION=28 PROHIBITED_EMPTY_LIST_IN_CONDITION value
                             * @property {number} PROHIBITED_ENUM_CONSTANT=54 PROHIBITED_ENUM_CONSTANT value
                             * @property {number} PROHIBITED_FIELD_COMBINATION_IN_SELECT_CLAUSE=31 PROHIBITED_FIELD_COMBINATION_IN_SELECT_CLAUSE value
                             * @property {number} PROHIBITED_FIELD_IN_ORDER_BY_CLAUSE=40 PROHIBITED_FIELD_IN_ORDER_BY_CLAUSE value
                             * @property {number} PROHIBITED_FIELD_IN_SELECT_CLAUSE=23 PROHIBITED_FIELD_IN_SELECT_CLAUSE value
                             * @property {number} PROHIBITED_FIELD_IN_WHERE_CLAUSE=24 PROHIBITED_FIELD_IN_WHERE_CLAUSE value
                             * @property {number} PROHIBITED_RESOURCE_TYPE_IN_FROM_CLAUSE=43 PROHIBITED_RESOURCE_TYPE_IN_FROM_CLAUSE value
                             * @property {number} PROHIBITED_RESOURCE_TYPE_IN_SELECT_CLAUSE=48 PROHIBITED_RESOURCE_TYPE_IN_SELECT_CLAUSE value
                             * @property {number} PROHIBITED_RESOURCE_TYPE_IN_WHERE_CLAUSE=58 PROHIBITED_RESOURCE_TYPE_IN_WHERE_CLAUSE value
                             * @property {number} PROHIBITED_METRIC_IN_SELECT_OR_WHERE_CLAUSE=49 PROHIBITED_METRIC_IN_SELECT_OR_WHERE_CLAUSE value
                             * @property {number} PROHIBITED_SEGMENT_IN_SELECT_OR_WHERE_CLAUSE=51 PROHIBITED_SEGMENT_IN_SELECT_OR_WHERE_CLAUSE value
                             * @property {number} PROHIBITED_SEGMENT_WITH_METRIC_IN_SELECT_OR_WHERE_CLAUSE=53 PROHIBITED_SEGMENT_WITH_METRIC_IN_SELECT_OR_WHERE_CLAUSE value
                             * @property {number} LIMIT_VALUE_TOO_LOW=25 LIMIT_VALUE_TOO_LOW value
                             * @property {number} PROHIBITED_NEWLINE_IN_STRING=8 PROHIBITED_NEWLINE_IN_STRING value
                             * @property {number} PROHIBITED_VALUE_COMBINATION_IN_LIST=10 PROHIBITED_VALUE_COMBINATION_IN_LIST value
                             * @property {number} PROHIBITED_VALUE_COMBINATION_WITH_BETWEEN_OPERATOR=21 PROHIBITED_VALUE_COMBINATION_WITH_BETWEEN_OPERATOR value
                             * @property {number} STRING_NOT_TERMINATED=6 STRING_NOT_TERMINATED value
                             * @property {number} TOO_MANY_SEGMENTS=34 TOO_MANY_SEGMENTS value
                             * @property {number} UNEXPECTED_END_OF_QUERY=9 UNEXPECTED_END_OF_QUERY value
                             * @property {number} UNEXPECTED_FROM_CLAUSE=47 UNEXPECTED_FROM_CLAUSE value
                             * @property {number} UNRECOGNIZED_FIELD=32 UNRECOGNIZED_FIELD value
                             * @property {number} UNEXPECTED_INPUT=11 UNEXPECTED_INPUT value
                             * @property {number} REQUESTED_METRICS_FOR_MANAGER=59 REQUESTED_METRICS_FOR_MANAGER value
                             * @property {number} FILTER_HAS_TOO_MANY_VALUES=63 FILTER_HAS_TOO_MANY_VALUES value
                             */
                            QueryErrorEnum.QueryError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[50] = "QUERY_ERROR"] = 50;
                                values[valuesById[18] = "BAD_ENUM_CONSTANT"] = 18;
                                values[valuesById[7] = "BAD_ESCAPE_SEQUENCE"] = 7;
                                values[valuesById[12] = "BAD_FIELD_NAME"] = 12;
                                values[valuesById[15] = "BAD_LIMIT_VALUE"] = 15;
                                values[valuesById[5] = "BAD_NUMBER"] = 5;
                                values[valuesById[3] = "BAD_OPERATOR"] = 3;
                                values[valuesById[61] = "BAD_PARAMETER_NAME"] = 61;
                                values[valuesById[62] = "BAD_PARAMETER_VALUE"] = 62;
                                values[valuesById[45] = "BAD_RESOURCE_TYPE_IN_FROM_CLAUSE"] = 45;
                                values[valuesById[2] = "BAD_SYMBOL"] = 2;
                                values[valuesById[4] = "BAD_VALUE"] = 4;
                                values[valuesById[36] = "DATE_RANGE_TOO_WIDE"] = 36;
                                values[valuesById[60] = "DATE_RANGE_TOO_NARROW"] = 60;
                                values[valuesById[30] = "EXPECTED_AND"] = 30;
                                values[valuesById[14] = "EXPECTED_BY"] = 14;
                                values[valuesById[37] = "EXPECTED_DIMENSION_FIELD_IN_SELECT_CLAUSE"] = 37;
                                values[valuesById[55] = "EXPECTED_FILTERS_ON_DATE_RANGE"] = 55;
                                values[valuesById[44] = "EXPECTED_FROM"] = 44;
                                values[valuesById[41] = "EXPECTED_LIST"] = 41;
                                values[valuesById[16] = "EXPECTED_REFERENCED_FIELD_IN_SELECT_CLAUSE"] = 16;
                                values[valuesById[13] = "EXPECTED_SELECT"] = 13;
                                values[valuesById[42] = "EXPECTED_SINGLE_VALUE"] = 42;
                                values[valuesById[29] = "EXPECTED_VALUE_WITH_BETWEEN_OPERATOR"] = 29;
                                values[valuesById[38] = "INVALID_DATE_FORMAT"] = 38;
                                values[valuesById[64] = "MISALIGNED_DATE_FOR_FILTER"] = 64;
                                values[valuesById[57] = "INVALID_STRING_VALUE"] = 57;
                                values[valuesById[26] = "INVALID_VALUE_WITH_BETWEEN_OPERATOR"] = 26;
                                values[valuesById[22] = "INVALID_VALUE_WITH_DURING_OPERATOR"] = 22;
                                values[valuesById[56] = "INVALID_VALUE_WITH_LIKE_OPERATOR"] = 56;
                                values[valuesById[35] = "OPERATOR_FIELD_MISMATCH"] = 35;
                                values[valuesById[28] = "PROHIBITED_EMPTY_LIST_IN_CONDITION"] = 28;
                                values[valuesById[54] = "PROHIBITED_ENUM_CONSTANT"] = 54;
                                values[valuesById[31] = "PROHIBITED_FIELD_COMBINATION_IN_SELECT_CLAUSE"] = 31;
                                values[valuesById[40] = "PROHIBITED_FIELD_IN_ORDER_BY_CLAUSE"] = 40;
                                values[valuesById[23] = "PROHIBITED_FIELD_IN_SELECT_CLAUSE"] = 23;
                                values[valuesById[24] = "PROHIBITED_FIELD_IN_WHERE_CLAUSE"] = 24;
                                values[valuesById[43] = "PROHIBITED_RESOURCE_TYPE_IN_FROM_CLAUSE"] = 43;
                                values[valuesById[48] = "PROHIBITED_RESOURCE_TYPE_IN_SELECT_CLAUSE"] = 48;
                                values[valuesById[58] = "PROHIBITED_RESOURCE_TYPE_IN_WHERE_CLAUSE"] = 58;
                                values[valuesById[49] = "PROHIBITED_METRIC_IN_SELECT_OR_WHERE_CLAUSE"] = 49;
                                values[valuesById[51] = "PROHIBITED_SEGMENT_IN_SELECT_OR_WHERE_CLAUSE"] = 51;
                                values[valuesById[53] = "PROHIBITED_SEGMENT_WITH_METRIC_IN_SELECT_OR_WHERE_CLAUSE"] = 53;
                                values[valuesById[25] = "LIMIT_VALUE_TOO_LOW"] = 25;
                                values[valuesById[8] = "PROHIBITED_NEWLINE_IN_STRING"] = 8;
                                values[valuesById[10] = "PROHIBITED_VALUE_COMBINATION_IN_LIST"] = 10;
                                values[valuesById[21] = "PROHIBITED_VALUE_COMBINATION_WITH_BETWEEN_OPERATOR"] = 21;
                                values[valuesById[6] = "STRING_NOT_TERMINATED"] = 6;
                                values[valuesById[34] = "TOO_MANY_SEGMENTS"] = 34;
                                values[valuesById[9] = "UNEXPECTED_END_OF_QUERY"] = 9;
                                values[valuesById[47] = "UNEXPECTED_FROM_CLAUSE"] = 47;
                                values[valuesById[32] = "UNRECOGNIZED_FIELD"] = 32;
                                values[valuesById[11] = "UNEXPECTED_INPUT"] = 11;
                                values[valuesById[59] = "REQUESTED_METRICS_FOR_MANAGER"] = 59;
                                values[valuesById[63] = "FILTER_HAS_TOO_MANY_VALUES"] = 63;
                                return values;
                            })();
    
                            return QueryErrorEnum;
                        })();
    
                        errors.QuotaErrorEnum = (function() {
    
                            /**
                             * Properties of a QuotaErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IQuotaErrorEnum
                             */
    
                            /**
                             * Constructs a new QuotaErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a QuotaErrorEnum.
                             * @implements IQuotaErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorEnum=} [p] Properties to set
                             */
                            function QuotaErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new QuotaErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorEnum} QuotaErrorEnum instance
                             */
                            QuotaErrorEnum.create = function create(properties) {
                                return new QuotaErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified QuotaErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.QuotaErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorEnum} m QuotaErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QuotaErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified QuotaErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.QuotaErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IQuotaErrorEnum} message QuotaErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            QuotaErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a QuotaErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorEnum} QuotaErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QuotaErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.QuotaErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a QuotaErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorEnum} QuotaErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            QuotaErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a QuotaErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            QuotaErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a QuotaErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.QuotaErrorEnum} QuotaErrorEnum
                             */
                            QuotaErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.QuotaErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.QuotaErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a QuotaErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.QuotaErrorEnum} m QuotaErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            QuotaErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this QuotaErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.QuotaErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            QuotaErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * QuotaError enum.
                             * @name google.ads.googleads.v11.errors.QuotaErrorEnum.QuotaError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} RESOURCE_EXHAUSTED=2 RESOURCE_EXHAUSTED value
                             * @property {number} ACCESS_PROHIBITED=3 ACCESS_PROHIBITED value
                             * @property {number} RESOURCE_TEMPORARILY_EXHAUSTED=4 RESOURCE_TEMPORARILY_EXHAUSTED value
                             */
                            QuotaErrorEnum.QuotaError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "RESOURCE_EXHAUSTED"] = 2;
                                values[valuesById[3] = "ACCESS_PROHIBITED"] = 3;
                                values[valuesById[4] = "RESOURCE_TEMPORARILY_EXHAUSTED"] = 4;
                                return values;
                            })();
    
                            return QuotaErrorEnum;
                        })();
    
                        errors.RangeErrorEnum = (function() {
    
                            /**
                             * Properties of a RangeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IRangeErrorEnum
                             */
    
                            /**
                             * Constructs a new RangeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a RangeErrorEnum.
                             * @implements IRangeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IRangeErrorEnum=} [p] Properties to set
                             */
                            function RangeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new RangeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRangeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.RangeErrorEnum} RangeErrorEnum instance
                             */
                            RangeErrorEnum.create = function create(properties) {
                                return new RangeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified RangeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.RangeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRangeErrorEnum} m RangeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RangeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RangeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.RangeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRangeErrorEnum} message RangeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RangeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RangeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.RangeErrorEnum} RangeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RangeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.RangeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RangeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.RangeErrorEnum} RangeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RangeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RangeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RangeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a RangeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.RangeErrorEnum} RangeErrorEnum
                             */
                            RangeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.RangeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.RangeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a RangeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.RangeErrorEnum} m RangeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RangeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this RangeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.RangeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RangeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * RangeError enum.
                             * @name google.ads.googleads.v11.errors.RangeErrorEnum.RangeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} TOO_LOW=2 TOO_LOW value
                             * @property {number} TOO_HIGH=3 TOO_HIGH value
                             */
                            RangeErrorEnum.RangeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "TOO_LOW"] = 2;
                                values[valuesById[3] = "TOO_HIGH"] = 3;
                                return values;
                            })();
    
                            return RangeErrorEnum;
                        })();
    
                        errors.ReachPlanErrorEnum = (function() {
    
                            /**
                             * Properties of a ReachPlanErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IReachPlanErrorEnum
                             */
    
                            /**
                             * Constructs a new ReachPlanErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ReachPlanErrorEnum.
                             * @implements IReachPlanErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IReachPlanErrorEnum=} [p] Properties to set
                             */
                            function ReachPlanErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ReachPlanErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IReachPlanErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ReachPlanErrorEnum} ReachPlanErrorEnum instance
                             */
                            ReachPlanErrorEnum.create = function create(properties) {
                                return new ReachPlanErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ReachPlanErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ReachPlanErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IReachPlanErrorEnum} m ReachPlanErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ReachPlanErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ReachPlanErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IReachPlanErrorEnum} message ReachPlanErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachPlanErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ReachPlanErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ReachPlanErrorEnum} ReachPlanErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ReachPlanErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ReachPlanErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ReachPlanErrorEnum} ReachPlanErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachPlanErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ReachPlanErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ReachPlanErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ReachPlanErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ReachPlanErrorEnum} ReachPlanErrorEnum
                             */
                            ReachPlanErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ReachPlanErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ReachPlanErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ReachPlanErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ReachPlanErrorEnum} m ReachPlanErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ReachPlanErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ReachPlanErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ReachPlanErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ReachPlanErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ReachPlanError enum.
                             * @name google.ads.googleads.v11.errors.ReachPlanErrorEnum.ReachPlanError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} NOT_FORECASTABLE_MISSING_RATE=2 NOT_FORECASTABLE_MISSING_RATE value
                             */
                            ReachPlanErrorEnum.ReachPlanError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "NOT_FORECASTABLE_MISSING_RATE"] = 2;
                                return values;
                            })();
    
                            return ReachPlanErrorEnum;
                        })();
    
                        errors.RecommendationErrorEnum = (function() {
    
                            /**
                             * Properties of a RecommendationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IRecommendationErrorEnum
                             */
    
                            /**
                             * Constructs a new RecommendationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a RecommendationErrorEnum.
                             * @implements IRecommendationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IRecommendationErrorEnum=} [p] Properties to set
                             */
                            function RecommendationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new RecommendationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRecommendationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.RecommendationErrorEnum} RecommendationErrorEnum instance
                             */
                            RecommendationErrorEnum.create = function create(properties) {
                                return new RecommendationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified RecommendationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.RecommendationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRecommendationErrorEnum} m RecommendationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RecommendationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RecommendationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.RecommendationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRecommendationErrorEnum} message RecommendationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RecommendationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RecommendationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.RecommendationErrorEnum} RecommendationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RecommendationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.RecommendationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RecommendationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.RecommendationErrorEnum} RecommendationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RecommendationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RecommendationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RecommendationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a RecommendationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.RecommendationErrorEnum} RecommendationErrorEnum
                             */
                            RecommendationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.RecommendationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.RecommendationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a RecommendationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.RecommendationErrorEnum} m RecommendationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RecommendationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this RecommendationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.RecommendationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RecommendationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * RecommendationError enum.
                             * @name google.ads.googleads.v11.errors.RecommendationErrorEnum.RecommendationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} BUDGET_AMOUNT_TOO_SMALL=2 BUDGET_AMOUNT_TOO_SMALL value
                             * @property {number} BUDGET_AMOUNT_TOO_LARGE=3 BUDGET_AMOUNT_TOO_LARGE value
                             * @property {number} INVALID_BUDGET_AMOUNT=4 INVALID_BUDGET_AMOUNT value
                             * @property {number} POLICY_ERROR=5 POLICY_ERROR value
                             * @property {number} INVALID_BID_AMOUNT=6 INVALID_BID_AMOUNT value
                             * @property {number} ADGROUP_KEYWORD_LIMIT=7 ADGROUP_KEYWORD_LIMIT value
                             * @property {number} RECOMMENDATION_ALREADY_APPLIED=8 RECOMMENDATION_ALREADY_APPLIED value
                             * @property {number} RECOMMENDATION_INVALIDATED=9 RECOMMENDATION_INVALIDATED value
                             * @property {number} TOO_MANY_OPERATIONS=10 TOO_MANY_OPERATIONS value
                             * @property {number} NO_OPERATIONS=11 NO_OPERATIONS value
                             * @property {number} DIFFERENT_TYPES_NOT_SUPPORTED=12 DIFFERENT_TYPES_NOT_SUPPORTED value
                             * @property {number} DUPLICATE_RESOURCE_NAME=13 DUPLICATE_RESOURCE_NAME value
                             * @property {number} RECOMMENDATION_ALREADY_DISMISSED=14 RECOMMENDATION_ALREADY_DISMISSED value
                             * @property {number} INVALID_APPLY_REQUEST=15 INVALID_APPLY_REQUEST value
                             */
                            RecommendationErrorEnum.RecommendationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "BUDGET_AMOUNT_TOO_SMALL"] = 2;
                                values[valuesById[3] = "BUDGET_AMOUNT_TOO_LARGE"] = 3;
                                values[valuesById[4] = "INVALID_BUDGET_AMOUNT"] = 4;
                                values[valuesById[5] = "POLICY_ERROR"] = 5;
                                values[valuesById[6] = "INVALID_BID_AMOUNT"] = 6;
                                values[valuesById[7] = "ADGROUP_KEYWORD_LIMIT"] = 7;
                                values[valuesById[8] = "RECOMMENDATION_ALREADY_APPLIED"] = 8;
                                values[valuesById[9] = "RECOMMENDATION_INVALIDATED"] = 9;
                                values[valuesById[10] = "TOO_MANY_OPERATIONS"] = 10;
                                values[valuesById[11] = "NO_OPERATIONS"] = 11;
                                values[valuesById[12] = "DIFFERENT_TYPES_NOT_SUPPORTED"] = 12;
                                values[valuesById[13] = "DUPLICATE_RESOURCE_NAME"] = 13;
                                values[valuesById[14] = "RECOMMENDATION_ALREADY_DISMISSED"] = 14;
                                values[valuesById[15] = "INVALID_APPLY_REQUEST"] = 15;
                                return values;
                            })();
    
                            return RecommendationErrorEnum;
                        })();
    
                        errors.RegionCodeErrorEnum = (function() {
    
                            /**
                             * Properties of a RegionCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IRegionCodeErrorEnum
                             */
    
                            /**
                             * Constructs a new RegionCodeErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a RegionCodeErrorEnum.
                             * @implements IRegionCodeErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IRegionCodeErrorEnum=} [p] Properties to set
                             */
                            function RegionCodeErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new RegionCodeErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRegionCodeErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.RegionCodeErrorEnum} RegionCodeErrorEnum instance
                             */
                            RegionCodeErrorEnum.create = function create(properties) {
                                return new RegionCodeErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified RegionCodeErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.RegionCodeErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRegionCodeErrorEnum} m RegionCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RegionCodeErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RegionCodeErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.RegionCodeErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRegionCodeErrorEnum} message RegionCodeErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RegionCodeErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RegionCodeErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.RegionCodeErrorEnum} RegionCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RegionCodeErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.RegionCodeErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RegionCodeErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.RegionCodeErrorEnum} RegionCodeErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RegionCodeErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RegionCodeErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RegionCodeErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a RegionCodeErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.RegionCodeErrorEnum} RegionCodeErrorEnum
                             */
                            RegionCodeErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.RegionCodeErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.RegionCodeErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a RegionCodeErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.RegionCodeErrorEnum} m RegionCodeErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RegionCodeErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this RegionCodeErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.RegionCodeErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RegionCodeErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * RegionCodeError enum.
                             * @name google.ads.googleads.v11.errors.RegionCodeErrorEnum.RegionCodeError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_REGION_CODE=2 INVALID_REGION_CODE value
                             */
                            RegionCodeErrorEnum.RegionCodeError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_REGION_CODE"] = 2;
                                return values;
                            })();
    
                            return RegionCodeErrorEnum;
                        })();
    
                        errors.RequestErrorEnum = (function() {
    
                            /**
                             * Properties of a RequestErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IRequestErrorEnum
                             */
    
                            /**
                             * Constructs a new RequestErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a RequestErrorEnum.
                             * @implements IRequestErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IRequestErrorEnum=} [p] Properties to set
                             */
                            function RequestErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new RequestErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRequestErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.RequestErrorEnum} RequestErrorEnum instance
                             */
                            RequestErrorEnum.create = function create(properties) {
                                return new RequestErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified RequestErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.RequestErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRequestErrorEnum} m RequestErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RequestErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RequestErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.RequestErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IRequestErrorEnum} message RequestErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RequestErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RequestErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.RequestErrorEnum} RequestErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RequestErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.RequestErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RequestErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.RequestErrorEnum} RequestErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RequestErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RequestErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RequestErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a RequestErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.RequestErrorEnum} RequestErrorEnum
                             */
                            RequestErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.RequestErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.RequestErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a RequestErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.RequestErrorEnum} m RequestErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RequestErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this RequestErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.RequestErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RequestErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * RequestError enum.
                             * @name google.ads.googleads.v11.errors.RequestErrorEnum.RequestError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} RESOURCE_NAME_MISSING=3 RESOURCE_NAME_MISSING value
                             * @property {number} RESOURCE_NAME_MALFORMED=4 RESOURCE_NAME_MALFORMED value
                             * @property {number} BAD_RESOURCE_ID=17 BAD_RESOURCE_ID value
                             * @property {number} INVALID_CUSTOMER_ID=16 INVALID_CUSTOMER_ID value
                             * @property {number} OPERATION_REQUIRED=5 OPERATION_REQUIRED value
                             * @property {number} RESOURCE_NOT_FOUND=6 RESOURCE_NOT_FOUND value
                             * @property {number} INVALID_PAGE_TOKEN=7 INVALID_PAGE_TOKEN value
                             * @property {number} EXPIRED_PAGE_TOKEN=8 EXPIRED_PAGE_TOKEN value
                             * @property {number} INVALID_PAGE_SIZE=22 INVALID_PAGE_SIZE value
                             * @property {number} REQUIRED_FIELD_MISSING=9 REQUIRED_FIELD_MISSING value
                             * @property {number} IMMUTABLE_FIELD=11 IMMUTABLE_FIELD value
                             * @property {number} TOO_MANY_MUTATE_OPERATIONS=13 TOO_MANY_MUTATE_OPERATIONS value
                             * @property {number} CANNOT_BE_EXECUTED_BY_MANAGER_ACCOUNT=14 CANNOT_BE_EXECUTED_BY_MANAGER_ACCOUNT value
                             * @property {number} CANNOT_MODIFY_FOREIGN_FIELD=15 CANNOT_MODIFY_FOREIGN_FIELD value
                             * @property {number} INVALID_ENUM_VALUE=18 INVALID_ENUM_VALUE value
                             * @property {number} DEVELOPER_TOKEN_PARAMETER_MISSING=19 DEVELOPER_TOKEN_PARAMETER_MISSING value
                             * @property {number} LOGIN_CUSTOMER_ID_PARAMETER_MISSING=20 LOGIN_CUSTOMER_ID_PARAMETER_MISSING value
                             * @property {number} VALIDATE_ONLY_REQUEST_HAS_PAGE_TOKEN=21 VALIDATE_ONLY_REQUEST_HAS_PAGE_TOKEN value
                             * @property {number} CANNOT_RETURN_SUMMARY_ROW_FOR_REQUEST_WITHOUT_METRICS=29 CANNOT_RETURN_SUMMARY_ROW_FOR_REQUEST_WITHOUT_METRICS value
                             * @property {number} CANNOT_RETURN_SUMMARY_ROW_FOR_VALIDATE_ONLY_REQUESTS=30 CANNOT_RETURN_SUMMARY_ROW_FOR_VALIDATE_ONLY_REQUESTS value
                             * @property {number} INCONSISTENT_RETURN_SUMMARY_ROW_VALUE=31 INCONSISTENT_RETURN_SUMMARY_ROW_VALUE value
                             * @property {number} TOTAL_RESULTS_COUNT_NOT_ORIGINALLY_REQUESTED=32 TOTAL_RESULTS_COUNT_NOT_ORIGINALLY_REQUESTED value
                             * @property {number} RPC_DEADLINE_TOO_SHORT=33 RPC_DEADLINE_TOO_SHORT value
                             */
                            RequestErrorEnum.RequestError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "RESOURCE_NAME_MISSING"] = 3;
                                values[valuesById[4] = "RESOURCE_NAME_MALFORMED"] = 4;
                                values[valuesById[17] = "BAD_RESOURCE_ID"] = 17;
                                values[valuesById[16] = "INVALID_CUSTOMER_ID"] = 16;
                                values[valuesById[5] = "OPERATION_REQUIRED"] = 5;
                                values[valuesById[6] = "RESOURCE_NOT_FOUND"] = 6;
                                values[valuesById[7] = "INVALID_PAGE_TOKEN"] = 7;
                                values[valuesById[8] = "EXPIRED_PAGE_TOKEN"] = 8;
                                values[valuesById[22] = "INVALID_PAGE_SIZE"] = 22;
                                values[valuesById[9] = "REQUIRED_FIELD_MISSING"] = 9;
                                values[valuesById[11] = "IMMUTABLE_FIELD"] = 11;
                                values[valuesById[13] = "TOO_MANY_MUTATE_OPERATIONS"] = 13;
                                values[valuesById[14] = "CANNOT_BE_EXECUTED_BY_MANAGER_ACCOUNT"] = 14;
                                values[valuesById[15] = "CANNOT_MODIFY_FOREIGN_FIELD"] = 15;
                                values[valuesById[18] = "INVALID_ENUM_VALUE"] = 18;
                                values[valuesById[19] = "DEVELOPER_TOKEN_PARAMETER_MISSING"] = 19;
                                values[valuesById[20] = "LOGIN_CUSTOMER_ID_PARAMETER_MISSING"] = 20;
                                values[valuesById[21] = "VALIDATE_ONLY_REQUEST_HAS_PAGE_TOKEN"] = 21;
                                values[valuesById[29] = "CANNOT_RETURN_SUMMARY_ROW_FOR_REQUEST_WITHOUT_METRICS"] = 29;
                                values[valuesById[30] = "CANNOT_RETURN_SUMMARY_ROW_FOR_VALIDATE_ONLY_REQUESTS"] = 30;
                                values[valuesById[31] = "INCONSISTENT_RETURN_SUMMARY_ROW_VALUE"] = 31;
                                values[valuesById[32] = "TOTAL_RESULTS_COUNT_NOT_ORIGINALLY_REQUESTED"] = 32;
                                values[valuesById[33] = "RPC_DEADLINE_TOO_SHORT"] = 33;
                                return values;
                            })();
    
                            return RequestErrorEnum;
                        })();
    
                        errors.ResourceAccessDeniedErrorEnum = (function() {
    
                            /**
                             * Properties of a ResourceAccessDeniedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IResourceAccessDeniedErrorEnum
                             */
    
                            /**
                             * Constructs a new ResourceAccessDeniedErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ResourceAccessDeniedErrorEnum.
                             * @implements IResourceAccessDeniedErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IResourceAccessDeniedErrorEnum=} [p] Properties to set
                             */
                            function ResourceAccessDeniedErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ResourceAccessDeniedErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceAccessDeniedErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum} ResourceAccessDeniedErrorEnum instance
                             */
                            ResourceAccessDeniedErrorEnum.create = function create(properties) {
                                return new ResourceAccessDeniedErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ResourceAccessDeniedErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceAccessDeniedErrorEnum} m ResourceAccessDeniedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceAccessDeniedErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResourceAccessDeniedErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceAccessDeniedErrorEnum} message ResourceAccessDeniedErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceAccessDeniedErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResourceAccessDeniedErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum} ResourceAccessDeniedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceAccessDeniedErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResourceAccessDeniedErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum} ResourceAccessDeniedErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceAccessDeniedErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResourceAccessDeniedErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResourceAccessDeniedErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ResourceAccessDeniedErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum} ResourceAccessDeniedErrorEnum
                             */
                            ResourceAccessDeniedErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ResourceAccessDeniedErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum} m ResourceAccessDeniedErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResourceAccessDeniedErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ResourceAccessDeniedErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResourceAccessDeniedErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ResourceAccessDeniedError enum.
                             * @name google.ads.googleads.v11.errors.ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} WRITE_ACCESS_DENIED=3 WRITE_ACCESS_DENIED value
                             */
                            ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "WRITE_ACCESS_DENIED"] = 3;
                                return values;
                            })();
    
                            return ResourceAccessDeniedErrorEnum;
                        })();
    
                        errors.ResourceCountLimitExceededErrorEnum = (function() {
    
                            /**
                             * Properties of a ResourceCountLimitExceededErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IResourceCountLimitExceededErrorEnum
                             */
    
                            /**
                             * Constructs a new ResourceCountLimitExceededErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ResourceCountLimitExceededErrorEnum.
                             * @implements IResourceCountLimitExceededErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IResourceCountLimitExceededErrorEnum=} [p] Properties to set
                             */
                            function ResourceCountLimitExceededErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ResourceCountLimitExceededErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceCountLimitExceededErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum} ResourceCountLimitExceededErrorEnum instance
                             */
                            ResourceCountLimitExceededErrorEnum.create = function create(properties) {
                                return new ResourceCountLimitExceededErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ResourceCountLimitExceededErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceCountLimitExceededErrorEnum} m ResourceCountLimitExceededErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceCountLimitExceededErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ResourceCountLimitExceededErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IResourceCountLimitExceededErrorEnum} message ResourceCountLimitExceededErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ResourceCountLimitExceededErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ResourceCountLimitExceededErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum} ResourceCountLimitExceededErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceCountLimitExceededErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ResourceCountLimitExceededErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum} ResourceCountLimitExceededErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ResourceCountLimitExceededErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ResourceCountLimitExceededErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ResourceCountLimitExceededErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ResourceCountLimitExceededErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum} ResourceCountLimitExceededErrorEnum
                             */
                            ResourceCountLimitExceededErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ResourceCountLimitExceededErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum} m ResourceCountLimitExceededErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ResourceCountLimitExceededErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ResourceCountLimitExceededErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ResourceCountLimitExceededErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ResourceCountLimitExceededError enum.
                             * @name google.ads.googleads.v11.errors.ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ACCOUNT_LIMIT=2 ACCOUNT_LIMIT value
                             * @property {number} CAMPAIGN_LIMIT=3 CAMPAIGN_LIMIT value
                             * @property {number} ADGROUP_LIMIT=4 ADGROUP_LIMIT value
                             * @property {number} AD_GROUP_AD_LIMIT=5 AD_GROUP_AD_LIMIT value
                             * @property {number} AD_GROUP_CRITERION_LIMIT=6 AD_GROUP_CRITERION_LIMIT value
                             * @property {number} SHARED_SET_LIMIT=7 SHARED_SET_LIMIT value
                             * @property {number} MATCHING_FUNCTION_LIMIT=8 MATCHING_FUNCTION_LIMIT value
                             * @property {number} RESPONSE_ROW_LIMIT_EXCEEDED=9 RESPONSE_ROW_LIMIT_EXCEEDED value
                             * @property {number} RESOURCE_LIMIT=10 RESOURCE_LIMIT value
                             */
                            ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ACCOUNT_LIMIT"] = 2;
                                values[valuesById[3] = "CAMPAIGN_LIMIT"] = 3;
                                values[valuesById[4] = "ADGROUP_LIMIT"] = 4;
                                values[valuesById[5] = "AD_GROUP_AD_LIMIT"] = 5;
                                values[valuesById[6] = "AD_GROUP_CRITERION_LIMIT"] = 6;
                                values[valuesById[7] = "SHARED_SET_LIMIT"] = 7;
                                values[valuesById[8] = "MATCHING_FUNCTION_LIMIT"] = 8;
                                values[valuesById[9] = "RESPONSE_ROW_LIMIT_EXCEEDED"] = 9;
                                values[valuesById[10] = "RESOURCE_LIMIT"] = 10;
                                return values;
                            })();
    
                            return ResourceCountLimitExceededErrorEnum;
                        })();
    
                        errors.SettingErrorEnum = (function() {
    
                            /**
                             * Properties of a SettingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ISettingErrorEnum
                             */
    
                            /**
                             * Constructs a new SettingErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a SettingErrorEnum.
                             * @implements ISettingErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ISettingErrorEnum=} [p] Properties to set
                             */
                            function SettingErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SettingErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISettingErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.SettingErrorEnum} SettingErrorEnum instance
                             */
                            SettingErrorEnum.create = function create(properties) {
                                return new SettingErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SettingErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.SettingErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISettingErrorEnum} m SettingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SettingErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SettingErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.SettingErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISettingErrorEnum} message SettingErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SettingErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SettingErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.SettingErrorEnum} SettingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SettingErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.SettingErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SettingErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.SettingErrorEnum} SettingErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SettingErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SettingErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SettingErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SettingErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.SettingErrorEnum} SettingErrorEnum
                             */
                            SettingErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.SettingErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.SettingErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a SettingErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.SettingErrorEnum} m SettingErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SettingErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SettingErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.SettingErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SettingErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SettingError enum.
                             * @name google.ads.googleads.v11.errors.SettingErrorEnum.SettingError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} SETTING_TYPE_IS_NOT_AVAILABLE=3 SETTING_TYPE_IS_NOT_AVAILABLE value
                             * @property {number} SETTING_TYPE_IS_NOT_COMPATIBLE_WITH_CAMPAIGN=4 SETTING_TYPE_IS_NOT_COMPATIBLE_WITH_CAMPAIGN value
                             * @property {number} TARGETING_SETTING_CONTAINS_INVALID_CRITERION_TYPE_GROUP=5 TARGETING_SETTING_CONTAINS_INVALID_CRITERION_TYPE_GROUP value
                             * @property {number} TARGETING_SETTING_DEMOGRAPHIC_CRITERION_TYPE_GROUPS_MUST_BE_SET_TO_TARGET_ALL=6 TARGETING_SETTING_DEMOGRAPHIC_CRITERION_TYPE_GROUPS_MUST_BE_SET_TO_TARGET_ALL value
                             * @property {number} TARGETING_SETTING_CANNOT_CHANGE_TARGET_ALL_TO_FALSE_FOR_DEMOGRAPHIC_CRITERION_TYPE_GROUP=7 TARGETING_SETTING_CANNOT_CHANGE_TARGET_ALL_TO_FALSE_FOR_DEMOGRAPHIC_CRITERION_TYPE_GROUP value
                             * @property {number} DYNAMIC_SEARCH_ADS_SETTING_AT_LEAST_ONE_FEED_ID_MUST_BE_PRESENT=8 DYNAMIC_SEARCH_ADS_SETTING_AT_LEAST_ONE_FEED_ID_MUST_BE_PRESENT value
                             * @property {number} DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_DOMAIN_NAME=9 DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_DOMAIN_NAME value
                             * @property {number} DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_SUBDOMAIN_NAME=10 DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_SUBDOMAIN_NAME value
                             * @property {number} DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_LANGUAGE_CODE=11 DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_LANGUAGE_CODE value
                             * @property {number} TARGET_ALL_IS_NOT_ALLOWED_FOR_PLACEMENT_IN_SEARCH_CAMPAIGN=12 TARGET_ALL_IS_NOT_ALLOWED_FOR_PLACEMENT_IN_SEARCH_CAMPAIGN value
                             * @property {number} SETTING_VALUE_NOT_COMPATIBLE_WITH_CAMPAIGN=20 SETTING_VALUE_NOT_COMPATIBLE_WITH_CAMPAIGN value
                             * @property {number} BID_ONLY_IS_NOT_ALLOWED_TO_BE_MODIFIED_WITH_CUSTOMER_MATCH_TARGETING=21 BID_ONLY_IS_NOT_ALLOWED_TO_BE_MODIFIED_WITH_CUSTOMER_MATCH_TARGETING value
                             */
                            SettingErrorEnum.SettingError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[3] = "SETTING_TYPE_IS_NOT_AVAILABLE"] = 3;
                                values[valuesById[4] = "SETTING_TYPE_IS_NOT_COMPATIBLE_WITH_CAMPAIGN"] = 4;
                                values[valuesById[5] = "TARGETING_SETTING_CONTAINS_INVALID_CRITERION_TYPE_GROUP"] = 5;
                                values[valuesById[6] = "TARGETING_SETTING_DEMOGRAPHIC_CRITERION_TYPE_GROUPS_MUST_BE_SET_TO_TARGET_ALL"] = 6;
                                values[valuesById[7] = "TARGETING_SETTING_CANNOT_CHANGE_TARGET_ALL_TO_FALSE_FOR_DEMOGRAPHIC_CRITERION_TYPE_GROUP"] = 7;
                                values[valuesById[8] = "DYNAMIC_SEARCH_ADS_SETTING_AT_LEAST_ONE_FEED_ID_MUST_BE_PRESENT"] = 8;
                                values[valuesById[9] = "DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_DOMAIN_NAME"] = 9;
                                values[valuesById[10] = "DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_SUBDOMAIN_NAME"] = 10;
                                values[valuesById[11] = "DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_LANGUAGE_CODE"] = 11;
                                values[valuesById[12] = "TARGET_ALL_IS_NOT_ALLOWED_FOR_PLACEMENT_IN_SEARCH_CAMPAIGN"] = 12;
                                values[valuesById[20] = "SETTING_VALUE_NOT_COMPATIBLE_WITH_CAMPAIGN"] = 20;
                                values[valuesById[21] = "BID_ONLY_IS_NOT_ALLOWED_TO_BE_MODIFIED_WITH_CUSTOMER_MATCH_TARGETING"] = 21;
                                return values;
                            })();
    
                            return SettingErrorEnum;
                        })();
    
                        errors.SharedCriterionErrorEnum = (function() {
    
                            /**
                             * Properties of a SharedCriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ISharedCriterionErrorEnum
                             */
    
                            /**
                             * Constructs a new SharedCriterionErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a SharedCriterionErrorEnum.
                             * @implements ISharedCriterionErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ISharedCriterionErrorEnum=} [p] Properties to set
                             */
                            function SharedCriterionErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SharedCriterionErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISharedCriterionErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.SharedCriterionErrorEnum} SharedCriterionErrorEnum instance
                             */
                            SharedCriterionErrorEnum.create = function create(properties) {
                                return new SharedCriterionErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SharedCriterionErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.SharedCriterionErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISharedCriterionErrorEnum} m SharedCriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedCriterionErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedCriterionErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.SharedCriterionErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISharedCriterionErrorEnum} message SharedCriterionErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedCriterionErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedCriterionErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.SharedCriterionErrorEnum} SharedCriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedCriterionErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.SharedCriterionErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedCriterionErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.SharedCriterionErrorEnum} SharedCriterionErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedCriterionErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedCriterionErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedCriterionErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SharedCriterionErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.SharedCriterionErrorEnum} SharedCriterionErrorEnum
                             */
                            SharedCriterionErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.SharedCriterionErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.SharedCriterionErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a SharedCriterionErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.SharedCriterionErrorEnum} m SharedCriterionErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedCriterionErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SharedCriterionErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.SharedCriterionErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedCriterionErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SharedCriterionError enum.
                             * @name google.ads.googleads.v11.errors.SharedCriterionErrorEnum.SharedCriterionError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CRITERION_TYPE_NOT_ALLOWED_FOR_SHARED_SET_TYPE=2 CRITERION_TYPE_NOT_ALLOWED_FOR_SHARED_SET_TYPE value
                             */
                            SharedCriterionErrorEnum.SharedCriterionError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CRITERION_TYPE_NOT_ALLOWED_FOR_SHARED_SET_TYPE"] = 2;
                                return values;
                            })();
    
                            return SharedCriterionErrorEnum;
                        })();
    
                        errors.SharedSetErrorEnum = (function() {
    
                            /**
                             * Properties of a SharedSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ISharedSetErrorEnum
                             */
    
                            /**
                             * Constructs a new SharedSetErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a SharedSetErrorEnum.
                             * @implements ISharedSetErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ISharedSetErrorEnum=} [p] Properties to set
                             */
                            function SharedSetErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SharedSetErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISharedSetErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.SharedSetErrorEnum} SharedSetErrorEnum instance
                             */
                            SharedSetErrorEnum.create = function create(properties) {
                                return new SharedSetErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SharedSetErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.SharedSetErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISharedSetErrorEnum} m SharedSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedSetErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.SharedSetErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISharedSetErrorEnum} message SharedSetErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedSetErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.SharedSetErrorEnum} SharedSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.SharedSetErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedSetErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.SharedSetErrorEnum} SharedSetErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedSetErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedSetErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SharedSetErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.SharedSetErrorEnum} SharedSetErrorEnum
                             */
                            SharedSetErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.SharedSetErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.SharedSetErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a SharedSetErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.SharedSetErrorEnum} m SharedSetErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedSetErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SharedSetErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.SharedSetErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedSetErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SharedSetError enum.
                             * @name google.ads.googleads.v11.errors.SharedSetErrorEnum.SharedSetError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} CUSTOMER_CANNOT_CREATE_SHARED_SET_OF_THIS_TYPE=2 CUSTOMER_CANNOT_CREATE_SHARED_SET_OF_THIS_TYPE value
                             * @property {number} DUPLICATE_NAME=3 DUPLICATE_NAME value
                             * @property {number} SHARED_SET_REMOVED=4 SHARED_SET_REMOVED value
                             * @property {number} SHARED_SET_IN_USE=5 SHARED_SET_IN_USE value
                             */
                            SharedSetErrorEnum.SharedSetError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "CUSTOMER_CANNOT_CREATE_SHARED_SET_OF_THIS_TYPE"] = 2;
                                values[valuesById[3] = "DUPLICATE_NAME"] = 3;
                                values[valuesById[4] = "SHARED_SET_REMOVED"] = 4;
                                values[valuesById[5] = "SHARED_SET_IN_USE"] = 5;
                                return values;
                            })();
    
                            return SharedSetErrorEnum;
                        })();
    
                        errors.SizeLimitErrorEnum = (function() {
    
                            /**
                             * Properties of a SizeLimitErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ISizeLimitErrorEnum
                             */
    
                            /**
                             * Constructs a new SizeLimitErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a SizeLimitErrorEnum.
                             * @implements ISizeLimitErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ISizeLimitErrorEnum=} [p] Properties to set
                             */
                            function SizeLimitErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new SizeLimitErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISizeLimitErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.SizeLimitErrorEnum} SizeLimitErrorEnum instance
                             */
                            SizeLimitErrorEnum.create = function create(properties) {
                                return new SizeLimitErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified SizeLimitErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.SizeLimitErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISizeLimitErrorEnum} m SizeLimitErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SizeLimitErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SizeLimitErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.SizeLimitErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ISizeLimitErrorEnum} message SizeLimitErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SizeLimitErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SizeLimitErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.SizeLimitErrorEnum} SizeLimitErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SizeLimitErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.SizeLimitErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SizeLimitErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.SizeLimitErrorEnum} SizeLimitErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SizeLimitErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SizeLimitErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SizeLimitErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a SizeLimitErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.SizeLimitErrorEnum} SizeLimitErrorEnum
                             */
                            SizeLimitErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.SizeLimitErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.SizeLimitErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a SizeLimitErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.SizeLimitErrorEnum} m SizeLimitErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SizeLimitErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this SizeLimitErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.SizeLimitErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SizeLimitErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * SizeLimitError enum.
                             * @name google.ads.googleads.v11.errors.SizeLimitErrorEnum.SizeLimitError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} REQUEST_SIZE_LIMIT_EXCEEDED=2 REQUEST_SIZE_LIMIT_EXCEEDED value
                             * @property {number} RESPONSE_SIZE_LIMIT_EXCEEDED=3 RESPONSE_SIZE_LIMIT_EXCEEDED value
                             */
                            SizeLimitErrorEnum.SizeLimitError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "REQUEST_SIZE_LIMIT_EXCEEDED"] = 2;
                                values[valuesById[3] = "RESPONSE_SIZE_LIMIT_EXCEEDED"] = 3;
                                return values;
                            })();
    
                            return SizeLimitErrorEnum;
                        })();
    
                        errors.StringFormatErrorEnum = (function() {
    
                            /**
                             * Properties of a StringFormatErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IStringFormatErrorEnum
                             */
    
                            /**
                             * Constructs a new StringFormatErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a StringFormatErrorEnum.
                             * @implements IStringFormatErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IStringFormatErrorEnum=} [p] Properties to set
                             */
                            function StringFormatErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new StringFormatErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IStringFormatErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.StringFormatErrorEnum} StringFormatErrorEnum instance
                             */
                            StringFormatErrorEnum.create = function create(properties) {
                                return new StringFormatErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified StringFormatErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.StringFormatErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IStringFormatErrorEnum} m StringFormatErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StringFormatErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified StringFormatErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.StringFormatErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IStringFormatErrorEnum} message StringFormatErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StringFormatErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StringFormatErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.StringFormatErrorEnum} StringFormatErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StringFormatErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.StringFormatErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StringFormatErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.StringFormatErrorEnum} StringFormatErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StringFormatErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StringFormatErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StringFormatErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a StringFormatErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.StringFormatErrorEnum} StringFormatErrorEnum
                             */
                            StringFormatErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.StringFormatErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.StringFormatErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a StringFormatErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.StringFormatErrorEnum} m StringFormatErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StringFormatErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this StringFormatErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.StringFormatErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StringFormatErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * StringFormatError enum.
                             * @name google.ads.googleads.v11.errors.StringFormatErrorEnum.StringFormatError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ILLEGAL_CHARS=2 ILLEGAL_CHARS value
                             * @property {number} INVALID_FORMAT=3 INVALID_FORMAT value
                             */
                            StringFormatErrorEnum.StringFormatError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ILLEGAL_CHARS"] = 2;
                                values[valuesById[3] = "INVALID_FORMAT"] = 3;
                                return values;
                            })();
    
                            return StringFormatErrorEnum;
                        })();
    
                        errors.StringLengthErrorEnum = (function() {
    
                            /**
                             * Properties of a StringLengthErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IStringLengthErrorEnum
                             */
    
                            /**
                             * Constructs a new StringLengthErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a StringLengthErrorEnum.
                             * @implements IStringLengthErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IStringLengthErrorEnum=} [p] Properties to set
                             */
                            function StringLengthErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new StringLengthErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IStringLengthErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.StringLengthErrorEnum} StringLengthErrorEnum instance
                             */
                            StringLengthErrorEnum.create = function create(properties) {
                                return new StringLengthErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified StringLengthErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.StringLengthErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IStringLengthErrorEnum} m StringLengthErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StringLengthErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified StringLengthErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.StringLengthErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IStringLengthErrorEnum} message StringLengthErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StringLengthErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StringLengthErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.StringLengthErrorEnum} StringLengthErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StringLengthErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.StringLengthErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a StringLengthErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.StringLengthErrorEnum} StringLengthErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StringLengthErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StringLengthErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StringLengthErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a StringLengthErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.StringLengthErrorEnum} StringLengthErrorEnum
                             */
                            StringLengthErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.StringLengthErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.StringLengthErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a StringLengthErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.StringLengthErrorEnum} m StringLengthErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StringLengthErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this StringLengthErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.StringLengthErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StringLengthErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * StringLengthError enum.
                             * @name google.ads.googleads.v11.errors.StringLengthErrorEnum.StringLengthError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EMPTY=4 EMPTY value
                             * @property {number} TOO_SHORT=2 TOO_SHORT value
                             * @property {number} TOO_LONG=3 TOO_LONG value
                             */
                            StringLengthErrorEnum.StringLengthError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[4] = "EMPTY"] = 4;
                                values[valuesById[2] = "TOO_SHORT"] = 2;
                                values[valuesById[3] = "TOO_LONG"] = 3;
                                return values;
                            })();
    
                            return StringLengthErrorEnum;
                        })();
    
                        errors.ThirdPartyAppAnalyticsLinkErrorEnum = (function() {
    
                            /**
                             * Properties of a ThirdPartyAppAnalyticsLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IThirdPartyAppAnalyticsLinkErrorEnum
                             */
    
                            /**
                             * Constructs a new ThirdPartyAppAnalyticsLinkErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a ThirdPartyAppAnalyticsLinkErrorEnum.
                             * @implements IThirdPartyAppAnalyticsLinkErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IThirdPartyAppAnalyticsLinkErrorEnum=} [p] Properties to set
                             */
                            function ThirdPartyAppAnalyticsLinkErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ThirdPartyAppAnalyticsLinkErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IThirdPartyAppAnalyticsLinkErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum} ThirdPartyAppAnalyticsLinkErrorEnum instance
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.create = function create(properties) {
                                return new ThirdPartyAppAnalyticsLinkErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified ThirdPartyAppAnalyticsLinkErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IThirdPartyAppAnalyticsLinkErrorEnum} m ThirdPartyAppAnalyticsLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ThirdPartyAppAnalyticsLinkErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IThirdPartyAppAnalyticsLinkErrorEnum} message ThirdPartyAppAnalyticsLinkErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ThirdPartyAppAnalyticsLinkErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum} ThirdPartyAppAnalyticsLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ThirdPartyAppAnalyticsLinkErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum} ThirdPartyAppAnalyticsLinkErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ThirdPartyAppAnalyticsLinkErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ThirdPartyAppAnalyticsLinkErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum} ThirdPartyAppAnalyticsLinkErrorEnum
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a ThirdPartyAppAnalyticsLinkErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum} m ThirdPartyAppAnalyticsLinkErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ThirdPartyAppAnalyticsLinkErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * ThirdPartyAppAnalyticsLinkError enum.
                             * @name google.ads.googleads.v11.errors.ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_ANALYTICS_PROVIDER_ID=2 INVALID_ANALYTICS_PROVIDER_ID value
                             * @property {number} INVALID_MOBILE_APP_ID=3 INVALID_MOBILE_APP_ID value
                             * @property {number} MOBILE_APP_IS_NOT_ENABLED=4 MOBILE_APP_IS_NOT_ENABLED value
                             * @property {number} CANNOT_REGENERATE_SHAREABLE_LINK_ID_FOR_REMOVED_LINK=5 CANNOT_REGENERATE_SHAREABLE_LINK_ID_FOR_REMOVED_LINK value
                             */
                            ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_ANALYTICS_PROVIDER_ID"] = 2;
                                values[valuesById[3] = "INVALID_MOBILE_APP_ID"] = 3;
                                values[valuesById[4] = "MOBILE_APP_IS_NOT_ENABLED"] = 4;
                                values[valuesById[5] = "CANNOT_REGENERATE_SHAREABLE_LINK_ID_FOR_REMOVED_LINK"] = 5;
                                return values;
                            })();
    
                            return ThirdPartyAppAnalyticsLinkErrorEnum;
                        })();
    
                        errors.TimeZoneErrorEnum = (function() {
    
                            /**
                             * Properties of a TimeZoneErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface ITimeZoneErrorEnum
                             */
    
                            /**
                             * Constructs a new TimeZoneErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a TimeZoneErrorEnum.
                             * @implements ITimeZoneErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.ITimeZoneErrorEnum=} [p] Properties to set
                             */
                            function TimeZoneErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new TimeZoneErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ITimeZoneErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.TimeZoneErrorEnum} TimeZoneErrorEnum instance
                             */
                            TimeZoneErrorEnum.create = function create(properties) {
                                return new TimeZoneErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified TimeZoneErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.TimeZoneErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ITimeZoneErrorEnum} m TimeZoneErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TimeZoneErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified TimeZoneErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.TimeZoneErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.ITimeZoneErrorEnum} message TimeZoneErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TimeZoneErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TimeZoneErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.TimeZoneErrorEnum} TimeZoneErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TimeZoneErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.TimeZoneErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TimeZoneErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.TimeZoneErrorEnum} TimeZoneErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TimeZoneErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TimeZoneErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TimeZoneErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a TimeZoneErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.TimeZoneErrorEnum} TimeZoneErrorEnum
                             */
                            TimeZoneErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.TimeZoneErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.TimeZoneErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a TimeZoneErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.TimeZoneErrorEnum} m TimeZoneErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TimeZoneErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this TimeZoneErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.TimeZoneErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TimeZoneErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * TimeZoneError enum.
                             * @name google.ads.googleads.v11.errors.TimeZoneErrorEnum.TimeZoneError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_TIME_ZONE=5 INVALID_TIME_ZONE value
                             */
                            TimeZoneErrorEnum.TimeZoneError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[5] = "INVALID_TIME_ZONE"] = 5;
                                return values;
                            })();
    
                            return TimeZoneErrorEnum;
                        })();
    
                        errors.UrlFieldErrorEnum = (function() {
    
                            /**
                             * Properties of an UrlFieldErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IUrlFieldErrorEnum
                             */
    
                            /**
                             * Constructs a new UrlFieldErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents an UrlFieldErrorEnum.
                             * @implements IUrlFieldErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IUrlFieldErrorEnum=} [p] Properties to set
                             */
                            function UrlFieldErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UrlFieldErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUrlFieldErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.UrlFieldErrorEnum} UrlFieldErrorEnum instance
                             */
                            UrlFieldErrorEnum.create = function create(properties) {
                                return new UrlFieldErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UrlFieldErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.UrlFieldErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUrlFieldErrorEnum} m UrlFieldErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UrlFieldErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UrlFieldErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.UrlFieldErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUrlFieldErrorEnum} message UrlFieldErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UrlFieldErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UrlFieldErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.UrlFieldErrorEnum} UrlFieldErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UrlFieldErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.UrlFieldErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UrlFieldErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.UrlFieldErrorEnum} UrlFieldErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UrlFieldErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UrlFieldErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UrlFieldErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates an UrlFieldErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.UrlFieldErrorEnum} UrlFieldErrorEnum
                             */
                            UrlFieldErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.UrlFieldErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.UrlFieldErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from an UrlFieldErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.UrlFieldErrorEnum} m UrlFieldErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UrlFieldErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UrlFieldErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.UrlFieldErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UrlFieldErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UrlFieldError enum.
                             * @name google.ads.googleads.v11.errors.UrlFieldErrorEnum.UrlFieldError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} INVALID_TRACKING_URL_TEMPLATE=2 INVALID_TRACKING_URL_TEMPLATE value
                             * @property {number} INVALID_TAG_IN_TRACKING_URL_TEMPLATE=3 INVALID_TAG_IN_TRACKING_URL_TEMPLATE value
                             * @property {number} MISSING_TRACKING_URL_TEMPLATE_TAG=4 MISSING_TRACKING_URL_TEMPLATE_TAG value
                             * @property {number} MISSING_PROTOCOL_IN_TRACKING_URL_TEMPLATE=5 MISSING_PROTOCOL_IN_TRACKING_URL_TEMPLATE value
                             * @property {number} INVALID_PROTOCOL_IN_TRACKING_URL_TEMPLATE=6 INVALID_PROTOCOL_IN_TRACKING_URL_TEMPLATE value
                             * @property {number} MALFORMED_TRACKING_URL_TEMPLATE=7 MALFORMED_TRACKING_URL_TEMPLATE value
                             * @property {number} MISSING_HOST_IN_TRACKING_URL_TEMPLATE=8 MISSING_HOST_IN_TRACKING_URL_TEMPLATE value
                             * @property {number} INVALID_TLD_IN_TRACKING_URL_TEMPLATE=9 INVALID_TLD_IN_TRACKING_URL_TEMPLATE value
                             * @property {number} REDUNDANT_NESTED_TRACKING_URL_TEMPLATE_TAG=10 REDUNDANT_NESTED_TRACKING_URL_TEMPLATE_TAG value
                             * @property {number} INVALID_FINAL_URL=11 INVALID_FINAL_URL value
                             * @property {number} INVALID_TAG_IN_FINAL_URL=12 INVALID_TAG_IN_FINAL_URL value
                             * @property {number} REDUNDANT_NESTED_FINAL_URL_TAG=13 REDUNDANT_NESTED_FINAL_URL_TAG value
                             * @property {number} MISSING_PROTOCOL_IN_FINAL_URL=14 MISSING_PROTOCOL_IN_FINAL_URL value
                             * @property {number} INVALID_PROTOCOL_IN_FINAL_URL=15 INVALID_PROTOCOL_IN_FINAL_URL value
                             * @property {number} MALFORMED_FINAL_URL=16 MALFORMED_FINAL_URL value
                             * @property {number} MISSING_HOST_IN_FINAL_URL=17 MISSING_HOST_IN_FINAL_URL value
                             * @property {number} INVALID_TLD_IN_FINAL_URL=18 INVALID_TLD_IN_FINAL_URL value
                             * @property {number} INVALID_FINAL_MOBILE_URL=19 INVALID_FINAL_MOBILE_URL value
                             * @property {number} INVALID_TAG_IN_FINAL_MOBILE_URL=20 INVALID_TAG_IN_FINAL_MOBILE_URL value
                             * @property {number} REDUNDANT_NESTED_FINAL_MOBILE_URL_TAG=21 REDUNDANT_NESTED_FINAL_MOBILE_URL_TAG value
                             * @property {number} MISSING_PROTOCOL_IN_FINAL_MOBILE_URL=22 MISSING_PROTOCOL_IN_FINAL_MOBILE_URL value
                             * @property {number} INVALID_PROTOCOL_IN_FINAL_MOBILE_URL=23 INVALID_PROTOCOL_IN_FINAL_MOBILE_URL value
                             * @property {number} MALFORMED_FINAL_MOBILE_URL=24 MALFORMED_FINAL_MOBILE_URL value
                             * @property {number} MISSING_HOST_IN_FINAL_MOBILE_URL=25 MISSING_HOST_IN_FINAL_MOBILE_URL value
                             * @property {number} INVALID_TLD_IN_FINAL_MOBILE_URL=26 INVALID_TLD_IN_FINAL_MOBILE_URL value
                             * @property {number} INVALID_FINAL_APP_URL=27 INVALID_FINAL_APP_URL value
                             * @property {number} INVALID_TAG_IN_FINAL_APP_URL=28 INVALID_TAG_IN_FINAL_APP_URL value
                             * @property {number} REDUNDANT_NESTED_FINAL_APP_URL_TAG=29 REDUNDANT_NESTED_FINAL_APP_URL_TAG value
                             * @property {number} MULTIPLE_APP_URLS_FOR_OSTYPE=30 MULTIPLE_APP_URLS_FOR_OSTYPE value
                             * @property {number} INVALID_OSTYPE=31 INVALID_OSTYPE value
                             * @property {number} INVALID_PROTOCOL_FOR_APP_URL=32 INVALID_PROTOCOL_FOR_APP_URL value
                             * @property {number} INVALID_PACKAGE_ID_FOR_APP_URL=33 INVALID_PACKAGE_ID_FOR_APP_URL value
                             * @property {number} URL_CUSTOM_PARAMETERS_COUNT_EXCEEDS_LIMIT=34 URL_CUSTOM_PARAMETERS_COUNT_EXCEEDS_LIMIT value
                             * @property {number} INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_KEY=39 INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_KEY value
                             * @property {number} INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_VALUE=40 INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_VALUE value
                             * @property {number} INVALID_TAG_IN_URL_CUSTOM_PARAMETER_VALUE=41 INVALID_TAG_IN_URL_CUSTOM_PARAMETER_VALUE value
                             * @property {number} REDUNDANT_NESTED_URL_CUSTOM_PARAMETER_TAG=42 REDUNDANT_NESTED_URL_CUSTOM_PARAMETER_TAG value
                             * @property {number} MISSING_PROTOCOL=43 MISSING_PROTOCOL value
                             * @property {number} INVALID_PROTOCOL=52 INVALID_PROTOCOL value
                             * @property {number} INVALID_URL=44 INVALID_URL value
                             * @property {number} DESTINATION_URL_DEPRECATED=45 DESTINATION_URL_DEPRECATED value
                             * @property {number} INVALID_TAG_IN_URL=46 INVALID_TAG_IN_URL value
                             * @property {number} MISSING_URL_TAG=47 MISSING_URL_TAG value
                             * @property {number} DUPLICATE_URL_ID=48 DUPLICATE_URL_ID value
                             * @property {number} INVALID_URL_ID=49 INVALID_URL_ID value
                             * @property {number} FINAL_URL_SUFFIX_MALFORMED=50 FINAL_URL_SUFFIX_MALFORMED value
                             * @property {number} INVALID_TAG_IN_FINAL_URL_SUFFIX=51 INVALID_TAG_IN_FINAL_URL_SUFFIX value
                             * @property {number} INVALID_TOP_LEVEL_DOMAIN=53 INVALID_TOP_LEVEL_DOMAIN value
                             * @property {number} MALFORMED_TOP_LEVEL_DOMAIN=54 MALFORMED_TOP_LEVEL_DOMAIN value
                             * @property {number} MALFORMED_URL=55 MALFORMED_URL value
                             * @property {number} MISSING_HOST=56 MISSING_HOST value
                             * @property {number} NULL_CUSTOM_PARAMETER_VALUE=57 NULL_CUSTOM_PARAMETER_VALUE value
                             * @property {number} VALUE_TRACK_PARAMETER_NOT_SUPPORTED=58 VALUE_TRACK_PARAMETER_NOT_SUPPORTED value
                             */
                            UrlFieldErrorEnum.UrlFieldError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "INVALID_TRACKING_URL_TEMPLATE"] = 2;
                                values[valuesById[3] = "INVALID_TAG_IN_TRACKING_URL_TEMPLATE"] = 3;
                                values[valuesById[4] = "MISSING_TRACKING_URL_TEMPLATE_TAG"] = 4;
                                values[valuesById[5] = "MISSING_PROTOCOL_IN_TRACKING_URL_TEMPLATE"] = 5;
                                values[valuesById[6] = "INVALID_PROTOCOL_IN_TRACKING_URL_TEMPLATE"] = 6;
                                values[valuesById[7] = "MALFORMED_TRACKING_URL_TEMPLATE"] = 7;
                                values[valuesById[8] = "MISSING_HOST_IN_TRACKING_URL_TEMPLATE"] = 8;
                                values[valuesById[9] = "INVALID_TLD_IN_TRACKING_URL_TEMPLATE"] = 9;
                                values[valuesById[10] = "REDUNDANT_NESTED_TRACKING_URL_TEMPLATE_TAG"] = 10;
                                values[valuesById[11] = "INVALID_FINAL_URL"] = 11;
                                values[valuesById[12] = "INVALID_TAG_IN_FINAL_URL"] = 12;
                                values[valuesById[13] = "REDUNDANT_NESTED_FINAL_URL_TAG"] = 13;
                                values[valuesById[14] = "MISSING_PROTOCOL_IN_FINAL_URL"] = 14;
                                values[valuesById[15] = "INVALID_PROTOCOL_IN_FINAL_URL"] = 15;
                                values[valuesById[16] = "MALFORMED_FINAL_URL"] = 16;
                                values[valuesById[17] = "MISSING_HOST_IN_FINAL_URL"] = 17;
                                values[valuesById[18] = "INVALID_TLD_IN_FINAL_URL"] = 18;
                                values[valuesById[19] = "INVALID_FINAL_MOBILE_URL"] = 19;
                                values[valuesById[20] = "INVALID_TAG_IN_FINAL_MOBILE_URL"] = 20;
                                values[valuesById[21] = "REDUNDANT_NESTED_FINAL_MOBILE_URL_TAG"] = 21;
                                values[valuesById[22] = "MISSING_PROTOCOL_IN_FINAL_MOBILE_URL"] = 22;
                                values[valuesById[23] = "INVALID_PROTOCOL_IN_FINAL_MOBILE_URL"] = 23;
                                values[valuesById[24] = "MALFORMED_FINAL_MOBILE_URL"] = 24;
                                values[valuesById[25] = "MISSING_HOST_IN_FINAL_MOBILE_URL"] = 25;
                                values[valuesById[26] = "INVALID_TLD_IN_FINAL_MOBILE_URL"] = 26;
                                values[valuesById[27] = "INVALID_FINAL_APP_URL"] = 27;
                                values[valuesById[28] = "INVALID_TAG_IN_FINAL_APP_URL"] = 28;
                                values[valuesById[29] = "REDUNDANT_NESTED_FINAL_APP_URL_TAG"] = 29;
                                values[valuesById[30] = "MULTIPLE_APP_URLS_FOR_OSTYPE"] = 30;
                                values[valuesById[31] = "INVALID_OSTYPE"] = 31;
                                values[valuesById[32] = "INVALID_PROTOCOL_FOR_APP_URL"] = 32;
                                values[valuesById[33] = "INVALID_PACKAGE_ID_FOR_APP_URL"] = 33;
                                values[valuesById[34] = "URL_CUSTOM_PARAMETERS_COUNT_EXCEEDS_LIMIT"] = 34;
                                values[valuesById[39] = "INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_KEY"] = 39;
                                values[valuesById[40] = "INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_VALUE"] = 40;
                                values[valuesById[41] = "INVALID_TAG_IN_URL_CUSTOM_PARAMETER_VALUE"] = 41;
                                values[valuesById[42] = "REDUNDANT_NESTED_URL_CUSTOM_PARAMETER_TAG"] = 42;
                                values[valuesById[43] = "MISSING_PROTOCOL"] = 43;
                                values[valuesById[52] = "INVALID_PROTOCOL"] = 52;
                                values[valuesById[44] = "INVALID_URL"] = 44;
                                values[valuesById[45] = "DESTINATION_URL_DEPRECATED"] = 45;
                                values[valuesById[46] = "INVALID_TAG_IN_URL"] = 46;
                                values[valuesById[47] = "MISSING_URL_TAG"] = 47;
                                values[valuesById[48] = "DUPLICATE_URL_ID"] = 48;
                                values[valuesById[49] = "INVALID_URL_ID"] = 49;
                                values[valuesById[50] = "FINAL_URL_SUFFIX_MALFORMED"] = 50;
                                values[valuesById[51] = "INVALID_TAG_IN_FINAL_URL_SUFFIX"] = 51;
                                values[valuesById[53] = "INVALID_TOP_LEVEL_DOMAIN"] = 53;
                                values[valuesById[54] = "MALFORMED_TOP_LEVEL_DOMAIN"] = 54;
                                values[valuesById[55] = "MALFORMED_URL"] = 55;
                                values[valuesById[56] = "MISSING_HOST"] = 56;
                                values[valuesById[57] = "NULL_CUSTOM_PARAMETER_VALUE"] = 57;
                                values[valuesById[58] = "VALUE_TRACK_PARAMETER_NOT_SUPPORTED"] = 58;
                                return values;
                            })();
    
                            return UrlFieldErrorEnum;
                        })();
    
                        errors.UserDataErrorEnum = (function() {
    
                            /**
                             * Properties of a UserDataErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IUserDataErrorEnum
                             */
    
                            /**
                             * Constructs a new UserDataErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a UserDataErrorEnum.
                             * @implements IUserDataErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IUserDataErrorEnum=} [p] Properties to set
                             */
                            function UserDataErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserDataErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUserDataErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.UserDataErrorEnum} UserDataErrorEnum instance
                             */
                            UserDataErrorEnum.create = function create(properties) {
                                return new UserDataErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserDataErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.UserDataErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUserDataErrorEnum} m UserDataErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserDataErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserDataErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.UserDataErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUserDataErrorEnum} message UserDataErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserDataErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserDataErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.UserDataErrorEnum} UserDataErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserDataErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.UserDataErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserDataErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.UserDataErrorEnum} UserDataErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserDataErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserDataErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserDataErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserDataErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.UserDataErrorEnum} UserDataErrorEnum
                             */
                            UserDataErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.UserDataErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.UserDataErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserDataErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.UserDataErrorEnum} m UserDataErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserDataErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserDataErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.UserDataErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserDataErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserDataError enum.
                             * @name google.ads.googleads.v11.errors.UserDataErrorEnum.UserDataError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} OPERATIONS_FOR_CUSTOMER_MATCH_NOT_ALLOWED=2 OPERATIONS_FOR_CUSTOMER_MATCH_NOT_ALLOWED value
                             * @property {number} TOO_MANY_USER_IDENTIFIERS=3 TOO_MANY_USER_IDENTIFIERS value
                             * @property {number} USER_LIST_NOT_APPLICABLE=4 USER_LIST_NOT_APPLICABLE value
                             */
                            UserDataErrorEnum.UserDataError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "OPERATIONS_FOR_CUSTOMER_MATCH_NOT_ALLOWED"] = 2;
                                values[valuesById[3] = "TOO_MANY_USER_IDENTIFIERS"] = 3;
                                values[valuesById[4] = "USER_LIST_NOT_APPLICABLE"] = 4;
                                return values;
                            })();
    
                            return UserDataErrorEnum;
                        })();
    
                        errors.UserListErrorEnum = (function() {
    
                            /**
                             * Properties of a UserListErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IUserListErrorEnum
                             */
    
                            /**
                             * Constructs a new UserListErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a UserListErrorEnum.
                             * @implements IUserListErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IUserListErrorEnum=} [p] Properties to set
                             */
                            function UserListErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new UserListErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUserListErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.UserListErrorEnum} UserListErrorEnum instance
                             */
                            UserListErrorEnum.create = function create(properties) {
                                return new UserListErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified UserListErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.UserListErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUserListErrorEnum} m UserListErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.UserListErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IUserListErrorEnum} message UserListErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.UserListErrorEnum} UserListErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.UserListErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.UserListErrorEnum} UserListErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a UserListErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.UserListErrorEnum} UserListErrorEnum
                             */
                            UserListErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.UserListErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.UserListErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a UserListErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.UserListErrorEnum} m UserListErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this UserListErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.UserListErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * UserListError enum.
                             * @name google.ads.googleads.v11.errors.UserListErrorEnum.UserListError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} EXTERNAL_REMARKETING_USER_LIST_MUTATE_NOT_SUPPORTED=2 EXTERNAL_REMARKETING_USER_LIST_MUTATE_NOT_SUPPORTED value
                             * @property {number} CONCRETE_TYPE_REQUIRED=3 CONCRETE_TYPE_REQUIRED value
                             * @property {number} CONVERSION_TYPE_ID_REQUIRED=4 CONVERSION_TYPE_ID_REQUIRED value
                             * @property {number} DUPLICATE_CONVERSION_TYPES=5 DUPLICATE_CONVERSION_TYPES value
                             * @property {number} INVALID_CONVERSION_TYPE=6 INVALID_CONVERSION_TYPE value
                             * @property {number} INVALID_DESCRIPTION=7 INVALID_DESCRIPTION value
                             * @property {number} INVALID_NAME=8 INVALID_NAME value
                             * @property {number} INVALID_TYPE=9 INVALID_TYPE value
                             * @property {number} CAN_NOT_ADD_LOGICAL_LIST_AS_LOGICAL_LIST_OPERAND=10 CAN_NOT_ADD_LOGICAL_LIST_AS_LOGICAL_LIST_OPERAND value
                             * @property {number} INVALID_USER_LIST_LOGICAL_RULE_OPERAND=11 INVALID_USER_LIST_LOGICAL_RULE_OPERAND value
                             * @property {number} NAME_ALREADY_USED=12 NAME_ALREADY_USED value
                             * @property {number} NEW_CONVERSION_TYPE_NAME_REQUIRED=13 NEW_CONVERSION_TYPE_NAME_REQUIRED value
                             * @property {number} CONVERSION_TYPE_NAME_ALREADY_USED=14 CONVERSION_TYPE_NAME_ALREADY_USED value
                             * @property {number} OWNERSHIP_REQUIRED_FOR_SET=15 OWNERSHIP_REQUIRED_FOR_SET value
                             * @property {number} USER_LIST_MUTATE_NOT_SUPPORTED=16 USER_LIST_MUTATE_NOT_SUPPORTED value
                             * @property {number} INVALID_RULE=17 INVALID_RULE value
                             * @property {number} INVALID_DATE_RANGE=27 INVALID_DATE_RANGE value
                             * @property {number} CAN_NOT_MUTATE_SENSITIVE_USERLIST=28 CAN_NOT_MUTATE_SENSITIVE_USERLIST value
                             * @property {number} MAX_NUM_RULEBASED_USERLISTS=29 MAX_NUM_RULEBASED_USERLISTS value
                             * @property {number} CANNOT_MODIFY_BILLABLE_RECORD_COUNT=30 CANNOT_MODIFY_BILLABLE_RECORD_COUNT value
                             * @property {number} APP_ID_NOT_SET=31 APP_ID_NOT_SET value
                             * @property {number} USERLIST_NAME_IS_RESERVED_FOR_SYSTEM_LIST=32 USERLIST_NAME_IS_RESERVED_FOR_SYSTEM_LIST value
                             * @property {number} ADVERTISER_NOT_ON_ALLOWLIST_FOR_USING_UPLOADED_DATA=37 ADVERTISER_NOT_ON_ALLOWLIST_FOR_USING_UPLOADED_DATA value
                             * @property {number} RULE_TYPE_IS_NOT_SUPPORTED=34 RULE_TYPE_IS_NOT_SUPPORTED value
                             * @property {number} CAN_NOT_ADD_A_SIMILAR_USERLIST_AS_LOGICAL_LIST_OPERAND=35 CAN_NOT_ADD_A_SIMILAR_USERLIST_AS_LOGICAL_LIST_OPERAND value
                             * @property {number} CAN_NOT_MIX_CRM_BASED_IN_LOGICAL_LIST_WITH_OTHER_LISTS=36 CAN_NOT_MIX_CRM_BASED_IN_LOGICAL_LIST_WITH_OTHER_LISTS value
                             */
                            UserListErrorEnum.UserListError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "EXTERNAL_REMARKETING_USER_LIST_MUTATE_NOT_SUPPORTED"] = 2;
                                values[valuesById[3] = "CONCRETE_TYPE_REQUIRED"] = 3;
                                values[valuesById[4] = "CONVERSION_TYPE_ID_REQUIRED"] = 4;
                                values[valuesById[5] = "DUPLICATE_CONVERSION_TYPES"] = 5;
                                values[valuesById[6] = "INVALID_CONVERSION_TYPE"] = 6;
                                values[valuesById[7] = "INVALID_DESCRIPTION"] = 7;
                                values[valuesById[8] = "INVALID_NAME"] = 8;
                                values[valuesById[9] = "INVALID_TYPE"] = 9;
                                values[valuesById[10] = "CAN_NOT_ADD_LOGICAL_LIST_AS_LOGICAL_LIST_OPERAND"] = 10;
                                values[valuesById[11] = "INVALID_USER_LIST_LOGICAL_RULE_OPERAND"] = 11;
                                values[valuesById[12] = "NAME_ALREADY_USED"] = 12;
                                values[valuesById[13] = "NEW_CONVERSION_TYPE_NAME_REQUIRED"] = 13;
                                values[valuesById[14] = "CONVERSION_TYPE_NAME_ALREADY_USED"] = 14;
                                values[valuesById[15] = "OWNERSHIP_REQUIRED_FOR_SET"] = 15;
                                values[valuesById[16] = "USER_LIST_MUTATE_NOT_SUPPORTED"] = 16;
                                values[valuesById[17] = "INVALID_RULE"] = 17;
                                values[valuesById[27] = "INVALID_DATE_RANGE"] = 27;
                                values[valuesById[28] = "CAN_NOT_MUTATE_SENSITIVE_USERLIST"] = 28;
                                values[valuesById[29] = "MAX_NUM_RULEBASED_USERLISTS"] = 29;
                                values[valuesById[30] = "CANNOT_MODIFY_BILLABLE_RECORD_COUNT"] = 30;
                                values[valuesById[31] = "APP_ID_NOT_SET"] = 31;
                                values[valuesById[32] = "USERLIST_NAME_IS_RESERVED_FOR_SYSTEM_LIST"] = 32;
                                values[valuesById[37] = "ADVERTISER_NOT_ON_ALLOWLIST_FOR_USING_UPLOADED_DATA"] = 37;
                                values[valuesById[34] = "RULE_TYPE_IS_NOT_SUPPORTED"] = 34;
                                values[valuesById[35] = "CAN_NOT_ADD_A_SIMILAR_USERLIST_AS_LOGICAL_LIST_OPERAND"] = 35;
                                values[valuesById[36] = "CAN_NOT_MIX_CRM_BASED_IN_LOGICAL_LIST_WITH_OTHER_LISTS"] = 36;
                                return values;
                            })();
    
                            return UserListErrorEnum;
                        })();
    
                        errors.YoutubeVideoRegistrationErrorEnum = (function() {
    
                            /**
                             * Properties of a YoutubeVideoRegistrationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @interface IYoutubeVideoRegistrationErrorEnum
                             */
    
                            /**
                             * Constructs a new YoutubeVideoRegistrationErrorEnum.
                             * @memberof google.ads.googleads.v11.errors
                             * @classdesc Represents a YoutubeVideoRegistrationErrorEnum.
                             * @implements IYoutubeVideoRegistrationErrorEnum
                             * @constructor
                             * @param {google.ads.googleads.v11.errors.IYoutubeVideoRegistrationErrorEnum=} [p] Properties to set
                             */
                            function YoutubeVideoRegistrationErrorEnum(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new YoutubeVideoRegistrationErrorEnum instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IYoutubeVideoRegistrationErrorEnum=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum} YoutubeVideoRegistrationErrorEnum instance
                             */
                            YoutubeVideoRegistrationErrorEnum.create = function create(properties) {
                                return new YoutubeVideoRegistrationErrorEnum(properties);
                            };
    
                            /**
                             * Encodes the specified YoutubeVideoRegistrationErrorEnum message. Does not implicitly {@link google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IYoutubeVideoRegistrationErrorEnum} m YoutubeVideoRegistrationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YoutubeVideoRegistrationErrorEnum.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified YoutubeVideoRegistrationErrorEnum message, length delimited. Does not implicitly {@link google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.IYoutubeVideoRegistrationErrorEnum} message YoutubeVideoRegistrationErrorEnum message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            YoutubeVideoRegistrationErrorEnum.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a YoutubeVideoRegistrationErrorEnum message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum} YoutubeVideoRegistrationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YoutubeVideoRegistrationErrorEnum.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a YoutubeVideoRegistrationErrorEnum message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum} YoutubeVideoRegistrationErrorEnum
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            YoutubeVideoRegistrationErrorEnum.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a YoutubeVideoRegistrationErrorEnum message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            YoutubeVideoRegistrationErrorEnum.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a YoutubeVideoRegistrationErrorEnum message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum} YoutubeVideoRegistrationErrorEnum
                             */
                            YoutubeVideoRegistrationErrorEnum.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum)
                                    return d;
                                return new $root.google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum();
                            };
    
                            /**
                             * Creates a plain object from a YoutubeVideoRegistrationErrorEnum message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @static
                             * @param {google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum} m YoutubeVideoRegistrationErrorEnum
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            YoutubeVideoRegistrationErrorEnum.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this YoutubeVideoRegistrationErrorEnum to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            YoutubeVideoRegistrationErrorEnum.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * YoutubeVideoRegistrationError enum.
                             * @name google.ads.googleads.v11.errors.YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} VIDEO_NOT_FOUND=2 VIDEO_NOT_FOUND value
                             * @property {number} VIDEO_NOT_ACCESSIBLE=3 VIDEO_NOT_ACCESSIBLE value
                             * @property {number} VIDEO_NOT_ELIGIBLE=4 VIDEO_NOT_ELIGIBLE value
                             */
                            YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "VIDEO_NOT_FOUND"] = 2;
                                values[valuesById[3] = "VIDEO_NOT_ACCESSIBLE"] = 3;
                                values[valuesById[4] = "VIDEO_NOT_ELIGIBLE"] = 4;
                                return values;
                            })();
    
                            return YoutubeVideoRegistrationErrorEnum;
                        })();
    
                        return errors;
                    })();
    
                    v11.resources = (function() {
    
                        /**
                         * Namespace resources.
                         * @memberof google.ads.googleads.v11
                         * @namespace
                         */
                        var resources = {};
    
                        resources.AccessibleBiddingStrategy = (function() {
    
                            /**
                             * Properties of an AccessibleBiddingStrategy.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAccessibleBiddingStrategy
                             * @property {string|null} [resource_name] AccessibleBiddingStrategy resource_name
                             * @property {number|null} [id] AccessibleBiddingStrategy id
                             * @property {string|null} [name] AccessibleBiddingStrategy name
                             * @property {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType|null} [type] AccessibleBiddingStrategy type
                             * @property {number|null} [owner_customer_id] AccessibleBiddingStrategy owner_customer_id
                             * @property {string|null} [owner_descriptive_name] AccessibleBiddingStrategy owner_descriptive_name
                             * @property {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversionValue|null} [maximize_conversion_value] AccessibleBiddingStrategy maximize_conversion_value
                             * @property {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversions|null} [maximize_conversions] AccessibleBiddingStrategy maximize_conversions
                             * @property {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetCpa|null} [target_cpa] AccessibleBiddingStrategy target_cpa
                             * @property {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetImpressionShare|null} [target_impression_share] AccessibleBiddingStrategy target_impression_share
                             * @property {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetRoas|null} [target_roas] AccessibleBiddingStrategy target_roas
                             * @property {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetSpend|null} [target_spend] AccessibleBiddingStrategy target_spend
                             */
    
                            /**
                             * Constructs a new AccessibleBiddingStrategy.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AccessibleBiddingStrategy.
                             * @implements IAccessibleBiddingStrategy
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAccessibleBiddingStrategy=} [p] Properties to set
                             */
                            function AccessibleBiddingStrategy(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AccessibleBiddingStrategy resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.resource_name = "";
    
                            /**
                             * AccessibleBiddingStrategy id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AccessibleBiddingStrategy name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.name = "";
    
                            /**
                             * AccessibleBiddingStrategy type.
                             * @member {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType} type
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.type = 0;
    
                            /**
                             * AccessibleBiddingStrategy owner_customer_id.
                             * @member {number} owner_customer_id
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.owner_customer_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AccessibleBiddingStrategy owner_descriptive_name.
                             * @member {string} owner_descriptive_name
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.owner_descriptive_name = "";
    
                            /**
                             * AccessibleBiddingStrategy maximize_conversion_value.
                             * @member {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversionValue|null|undefined} maximize_conversion_value
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.maximize_conversion_value = null;
    
                            /**
                             * AccessibleBiddingStrategy maximize_conversions.
                             * @member {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversions|null|undefined} maximize_conversions
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.maximize_conversions = null;
    
                            /**
                             * AccessibleBiddingStrategy target_cpa.
                             * @member {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetCpa|null|undefined} target_cpa
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.target_cpa = null;
    
                            /**
                             * AccessibleBiddingStrategy target_impression_share.
                             * @member {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetImpressionShare|null|undefined} target_impression_share
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.target_impression_share = null;
    
                            /**
                             * AccessibleBiddingStrategy target_roas.
                             * @member {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetRoas|null|undefined} target_roas
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.target_roas = null;
    
                            /**
                             * AccessibleBiddingStrategy target_spend.
                             * @member {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetSpend|null|undefined} target_spend
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            AccessibleBiddingStrategy.prototype.target_spend = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AccessibleBiddingStrategy scheme.
                             * @member {"maximize_conversion_value"|"maximize_conversions"|"target_cpa"|"target_impression_share"|"target_roas"|"target_spend"|undefined} scheme
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             */
                            Object.defineProperty(AccessibleBiddingStrategy.prototype, "scheme", {
                                get: $util.oneOfGetter($oneOfFields = ["maximize_conversion_value", "maximize_conversions", "target_cpa", "target_impression_share", "target_roas", "target_spend"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AccessibleBiddingStrategy instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccessibleBiddingStrategy=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy} AccessibleBiddingStrategy instance
                             */
                            AccessibleBiddingStrategy.create = function create(properties) {
                                return new AccessibleBiddingStrategy(properties);
                            };
    
                            /**
                             * Encodes the specified AccessibleBiddingStrategy message. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccessibleBiddingStrategy} m AccessibleBiddingStrategy message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessibleBiddingStrategy.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.owner_customer_id != null && Object.hasOwnProperty.call(m, "owner_customer_id"))
                                    w.uint32(40).int64(m.owner_customer_id);
                                if (m.owner_descriptive_name != null && Object.hasOwnProperty.call(m, "owner_descriptive_name"))
                                    w.uint32(50).string(m.owner_descriptive_name);
                                if (m.maximize_conversion_value != null && Object.hasOwnProperty.call(m, "maximize_conversion_value"))
                                    $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue.encode(m.maximize_conversion_value, w.uint32(58).fork()).ldelim();
                                if (m.maximize_conversions != null && Object.hasOwnProperty.call(m, "maximize_conversions"))
                                    $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions.encode(m.maximize_conversions, w.uint32(66).fork()).ldelim();
                                if (m.target_cpa != null && Object.hasOwnProperty.call(m, "target_cpa"))
                                    $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa.encode(m.target_cpa, w.uint32(74).fork()).ldelim();
                                if (m.target_impression_share != null && Object.hasOwnProperty.call(m, "target_impression_share"))
                                    $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare.encode(m.target_impression_share, w.uint32(82).fork()).ldelim();
                                if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                    $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas.encode(m.target_roas, w.uint32(90).fork()).ldelim();
                                if (m.target_spend != null && Object.hasOwnProperty.call(m, "target_spend"))
                                    $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend.encode(m.target_spend, w.uint32(98).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccessibleBiddingStrategy message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccessibleBiddingStrategy} message AccessibleBiddingStrategy message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccessibleBiddingStrategy.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccessibleBiddingStrategy message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy} AccessibleBiddingStrategy
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessibleBiddingStrategy.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 5:
                                        m.owner_customer_id = r.int64();
                                        break;
                                    case 6:
                                        m.owner_descriptive_name = r.string();
                                        break;
                                    case 7:
                                        m.maximize_conversion_value = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.maximize_conversions = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.target_cpa = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.target_impression_share = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.target_roas = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.target_spend = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccessibleBiddingStrategy message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy} AccessibleBiddingStrategy
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccessibleBiddingStrategy.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccessibleBiddingStrategy message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccessibleBiddingStrategy.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 16:
                                    case 2:
                                    case 17:
                                    case 18:
                                    case 3:
                                    case 4:
                                    case 13:
                                    case 10:
                                    case 11:
                                    case 5:
                                    case 12:
                                    case 6:
                                    case 14:
                                    case 15:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.owner_customer_id != null && m.hasOwnProperty("owner_customer_id")) {
                                    if (!$util.isInteger(m.owner_customer_id) && !(m.owner_customer_id && $util.isInteger(m.owner_customer_id.low) && $util.isInteger(m.owner_customer_id.high)))
                                        return "owner_customer_id: integer|Long expected";
                                }
                                if (m.owner_descriptive_name != null && m.hasOwnProperty("owner_descriptive_name")) {
                                    if (!$util.isString(m.owner_descriptive_name))
                                        return "owner_descriptive_name: string expected";
                                }
                                if (m.maximize_conversion_value != null && m.hasOwnProperty("maximize_conversion_value")) {
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue.verify(m.maximize_conversion_value);
                                        if (e)
                                            return "maximize_conversion_value." + e;
                                    }
                                }
                                if (m.maximize_conversions != null && m.hasOwnProperty("maximize_conversions")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions.verify(m.maximize_conversions);
                                        if (e)
                                            return "maximize_conversions." + e;
                                    }
                                }
                                if (m.target_cpa != null && m.hasOwnProperty("target_cpa")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa.verify(m.target_cpa);
                                        if (e)
                                            return "target_cpa." + e;
                                    }
                                }
                                if (m.target_impression_share != null && m.hasOwnProperty("target_impression_share")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare.verify(m.target_impression_share);
                                        if (e)
                                            return "target_impression_share." + e;
                                    }
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas.verify(m.target_roas);
                                        if (e)
                                            return "target_roas." + e;
                                    }
                                }
                                if (m.target_spend != null && m.hasOwnProperty("target_spend")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend.verify(m.target_spend);
                                        if (e)
                                            return "target_spend." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AccessibleBiddingStrategy message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy} AccessibleBiddingStrategy
                             */
                            AccessibleBiddingStrategy.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "COMMISSION":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "ENHANCED_CPC":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "INVALID":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "MANUAL_CPA":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "MANUAL_CPC":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MANUAL_CPM":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "MANUAL_CPV":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "MAXIMIZE_CONVERSIONS":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "MAXIMIZE_CONVERSION_VALUE":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "PAGE_ONE_PROMOTED":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "PERCENT_CPC":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "TARGET_CPA":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "TARGET_CPM":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "TARGET_OUTRANK_SHARE":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TARGET_ROAS":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "TARGET_SPEND":
                                case 9:
                                    m.type = 9;
                                    break;
                                }
                                if (d.owner_customer_id != null) {
                                    if ($util.Long)
                                        (m.owner_customer_id = $util.Long.fromValue(d.owner_customer_id)).unsigned = false;
                                    else if (typeof d.owner_customer_id === "string")
                                        m.owner_customer_id = parseInt(d.owner_customer_id, 10);
                                    else if (typeof d.owner_customer_id === "number")
                                        m.owner_customer_id = d.owner_customer_id;
                                    else if (typeof d.owner_customer_id === "object")
                                        m.owner_customer_id = new $util.LongBits(d.owner_customer_id.low >>> 0, d.owner_customer_id.high >>> 0).toNumber();
                                }
                                if (d.owner_descriptive_name != null) {
                                    m.owner_descriptive_name = String(d.owner_descriptive_name);
                                }
                                if (d.maximize_conversion_value != null) {
                                    if (typeof d.maximize_conversion_value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccessibleBiddingStrategy.maximize_conversion_value: object expected");
                                    m.maximize_conversion_value = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue.fromObject(d.maximize_conversion_value);
                                }
                                if (d.maximize_conversions != null) {
                                    if (typeof d.maximize_conversions !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccessibleBiddingStrategy.maximize_conversions: object expected");
                                    m.maximize_conversions = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions.fromObject(d.maximize_conversions);
                                }
                                if (d.target_cpa != null) {
                                    if (typeof d.target_cpa !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccessibleBiddingStrategy.target_cpa: object expected");
                                    m.target_cpa = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa.fromObject(d.target_cpa);
                                }
                                if (d.target_impression_share != null) {
                                    if (typeof d.target_impression_share !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccessibleBiddingStrategy.target_impression_share: object expected");
                                    m.target_impression_share = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare.fromObject(d.target_impression_share);
                                }
                                if (d.target_roas != null) {
                                    if (typeof d.target_roas !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccessibleBiddingStrategy.target_roas: object expected");
                                    m.target_roas = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas.fromObject(d.target_roas);
                                }
                                if (d.target_spend != null) {
                                    if (typeof d.target_spend !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccessibleBiddingStrategy.target_spend: object expected");
                                    m.target_spend = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend.fromObject(d.target_spend);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AccessibleBiddingStrategy message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy} m AccessibleBiddingStrategy
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccessibleBiddingStrategy.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.owner_customer_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.owner_customer_id = o.longs === String ? "0" : 0;
                                    d.owner_descriptive_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType[m.type] : m.type;
                                }
                                if (m.owner_customer_id != null && m.hasOwnProperty("owner_customer_id")) {
                                    if (typeof m.owner_customer_id === "number")
                                        d.owner_customer_id = o.longs === String ? String(m.owner_customer_id) : m.owner_customer_id;
                                    else
                                        d.owner_customer_id = o.longs === String ? $util.Long.prototype.toString.call(m.owner_customer_id) : o.longs === Number ? new $util.LongBits(m.owner_customer_id.low >>> 0, m.owner_customer_id.high >>> 0).toNumber() : m.owner_customer_id;
                                }
                                if (m.owner_descriptive_name != null && m.hasOwnProperty("owner_descriptive_name")) {
                                    d.owner_descriptive_name = m.owner_descriptive_name;
                                }
                                if (m.maximize_conversion_value != null && m.hasOwnProperty("maximize_conversion_value")) {
                                    d.maximize_conversion_value = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue.toObject(m.maximize_conversion_value, o);
                                    if (o.oneofs)
                                        d.scheme = "maximize_conversion_value";
                                }
                                if (m.maximize_conversions != null && m.hasOwnProperty("maximize_conversions")) {
                                    d.maximize_conversions = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions.toObject(m.maximize_conversions, o);
                                    if (o.oneofs)
                                        d.scheme = "maximize_conversions";
                                }
                                if (m.target_cpa != null && m.hasOwnProperty("target_cpa")) {
                                    d.target_cpa = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa.toObject(m.target_cpa, o);
                                    if (o.oneofs)
                                        d.scheme = "target_cpa";
                                }
                                if (m.target_impression_share != null && m.hasOwnProperty("target_impression_share")) {
                                    d.target_impression_share = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare.toObject(m.target_impression_share, o);
                                    if (o.oneofs)
                                        d.scheme = "target_impression_share";
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    d.target_roas = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas.toObject(m.target_roas, o);
                                    if (o.oneofs)
                                        d.scheme = "target_roas";
                                }
                                if (m.target_spend != null && m.hasOwnProperty("target_spend")) {
                                    d.target_spend = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend.toObject(m.target_spend, o);
                                    if (o.oneofs)
                                        d.scheme = "target_spend";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AccessibleBiddingStrategy to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccessibleBiddingStrategy.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            AccessibleBiddingStrategy.MaximizeConversionValue = (function() {
    
                                /**
                                 * Properties of a MaximizeConversionValue.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @interface IMaximizeConversionValue
                                 * @property {number|null} [target_roas] MaximizeConversionValue target_roas
                                 */
    
                                /**
                                 * Constructs a new MaximizeConversionValue.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @classdesc Represents a MaximizeConversionValue.
                                 * @implements IMaximizeConversionValue
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversionValue=} [p] Properties to set
                                 */
                                function MaximizeConversionValue(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * MaximizeConversionValue target_roas.
                                 * @member {number} target_roas
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @instance
                                 */
                                MaximizeConversionValue.prototype.target_roas = 0;
    
                                /**
                                 * Creates a new MaximizeConversionValue instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversionValue=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue} MaximizeConversionValue instance
                                 */
                                MaximizeConversionValue.create = function create(properties) {
                                    return new MaximizeConversionValue(properties);
                                };
    
                                /**
                                 * Encodes the specified MaximizeConversionValue message. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversionValue} m MaximizeConversionValue message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeConversionValue.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                        w.uint32(9).double(m.target_roas);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified MaximizeConversionValue message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversionValue} message MaximizeConversionValue message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeConversionValue.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a MaximizeConversionValue message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue} MaximizeConversionValue
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeConversionValue.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.target_roas = r.double();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a MaximizeConversionValue message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue} MaximizeConversionValue
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeConversionValue.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a MaximizeConversionValue message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                MaximizeConversionValue.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                        if (typeof m.target_roas !== "number")
                                            return "target_roas: number expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a MaximizeConversionValue message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue} MaximizeConversionValue
                                 */
                                MaximizeConversionValue.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue();
                                    if (d.target_roas != null) {
                                        m.target_roas = Number(d.target_roas);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a MaximizeConversionValue message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue} m MaximizeConversionValue
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                MaximizeConversionValue.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.target_roas = 0;
                                    }
                                    if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                        d.target_roas = o.json && !isFinite(m.target_roas) ? String(m.target_roas) : m.target_roas;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this MaximizeConversionValue to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversionValue
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                MaximizeConversionValue.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return MaximizeConversionValue;
                            })();
    
                            AccessibleBiddingStrategy.MaximizeConversions = (function() {
    
                                /**
                                 * Properties of a MaximizeConversions.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @interface IMaximizeConversions
                                 * @property {number|null} [target_cpa_micros] MaximizeConversions target_cpa_micros
                                 */
    
                                /**
                                 * Constructs a new MaximizeConversions.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @classdesc Represents a MaximizeConversions.
                                 * @implements IMaximizeConversions
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversions=} [p] Properties to set
                                 */
                                function MaximizeConversions(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * MaximizeConversions target_cpa_micros.
                                 * @member {number} target_cpa_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @instance
                                 */
                                MaximizeConversions.prototype.target_cpa_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * Creates a new MaximizeConversions instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversions=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions} MaximizeConversions instance
                                 */
                                MaximizeConversions.create = function create(properties) {
                                    return new MaximizeConversions(properties);
                                };
    
                                /**
                                 * Encodes the specified MaximizeConversions message. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversions} m MaximizeConversions message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeConversions.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                        w.uint32(16).int64(m.target_cpa_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified MaximizeConversions message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.IMaximizeConversions} message MaximizeConversions message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeConversions.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a MaximizeConversions message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions} MaximizeConversions
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeConversions.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.target_cpa_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a MaximizeConversions message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions} MaximizeConversions
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeConversions.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a MaximizeConversions message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                MaximizeConversions.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                        if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                            return "target_cpa_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a MaximizeConversions message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions} MaximizeConversions
                                 */
                                MaximizeConversions.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions();
                                    if (d.target_cpa_micros != null) {
                                        if ($util.Long)
                                            (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                        else if (typeof d.target_cpa_micros === "string")
                                            m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                        else if (typeof d.target_cpa_micros === "number")
                                            m.target_cpa_micros = d.target_cpa_micros;
                                        else if (typeof d.target_cpa_micros === "object")
                                            m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a MaximizeConversions message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions} m MaximizeConversions
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                MaximizeConversions.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.target_cpa_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.target_cpa_micros = o.longs === String ? "0" : 0;
                                    }
                                    if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                        if (typeof m.target_cpa_micros === "number")
                                            d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                        else
                                            d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this MaximizeConversions to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.MaximizeConversions
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                MaximizeConversions.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return MaximizeConversions;
                            })();
    
                            AccessibleBiddingStrategy.TargetCpa = (function() {
    
                                /**
                                 * Properties of a TargetCpa.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @interface ITargetCpa
                                 * @property {number|null} [target_cpa_micros] TargetCpa target_cpa_micros
                                 */
    
                                /**
                                 * Constructs a new TargetCpa.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @classdesc Represents a TargetCpa.
                                 * @implements ITargetCpa
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetCpa=} [p] Properties to set
                                 */
                                function TargetCpa(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetCpa target_cpa_micros.
                                 * @member {number|null|undefined} target_cpa_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @instance
                                 */
                                TargetCpa.prototype.target_cpa_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetCpa _target_cpa_micros.
                                 * @member {"target_cpa_micros"|undefined} _target_cpa_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @instance
                                 */
                                Object.defineProperty(TargetCpa.prototype, "_target_cpa_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_cpa_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetCpa instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetCpa=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa} TargetCpa instance
                                 */
                                TargetCpa.create = function create(properties) {
                                    return new TargetCpa(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetCpa message. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetCpa} m TargetCpa message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetCpa.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                        w.uint32(8).int64(m.target_cpa_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetCpa message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetCpa} message TargetCpa message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetCpa.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetCpa message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa} TargetCpa
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetCpa.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.target_cpa_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetCpa message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa} TargetCpa
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetCpa.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetCpa message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetCpa.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                        p._target_cpa_micros = 1;
                                        if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                            return "target_cpa_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetCpa message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa} TargetCpa
                                 */
                                TargetCpa.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa();
                                    if (d.target_cpa_micros != null) {
                                        if ($util.Long)
                                            (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                        else if (typeof d.target_cpa_micros === "string")
                                            m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                        else if (typeof d.target_cpa_micros === "number")
                                            m.target_cpa_micros = d.target_cpa_micros;
                                        else if (typeof d.target_cpa_micros === "object")
                                            m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetCpa message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa} m TargetCpa
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetCpa.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                        if (typeof m.target_cpa_micros === "number")
                                            d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                        else
                                            d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                        if (o.oneofs)
                                            d._target_cpa_micros = "target_cpa_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetCpa to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetCpa
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetCpa.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TargetCpa;
                            })();
    
                            AccessibleBiddingStrategy.TargetImpressionShare = (function() {
    
                                /**
                                 * Properties of a TargetImpressionShare.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @interface ITargetImpressionShare
                                 * @property {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.TargetImpressionShareLocation|null} [location] TargetImpressionShare location
                                 * @property {number|null} [location_fraction_micros] TargetImpressionShare location_fraction_micros
                                 * @property {number|null} [cpc_bid_ceiling_micros] TargetImpressionShare cpc_bid_ceiling_micros
                                 */
    
                                /**
                                 * Constructs a new TargetImpressionShare.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @classdesc Represents a TargetImpressionShare.
                                 * @implements ITargetImpressionShare
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetImpressionShare=} [p] Properties to set
                                 */
                                function TargetImpressionShare(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetImpressionShare location.
                                 * @member {google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.TargetImpressionShareLocation} location
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @instance
                                 */
                                TargetImpressionShare.prototype.location = 0;
    
                                /**
                                 * TargetImpressionShare location_fraction_micros.
                                 * @member {number|null|undefined} location_fraction_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @instance
                                 */
                                TargetImpressionShare.prototype.location_fraction_micros = null;
    
                                /**
                                 * TargetImpressionShare cpc_bid_ceiling_micros.
                                 * @member {number|null|undefined} cpc_bid_ceiling_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @instance
                                 */
                                TargetImpressionShare.prototype.cpc_bid_ceiling_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetImpressionShare _location_fraction_micros.
                                 * @member {"location_fraction_micros"|undefined} _location_fraction_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @instance
                                 */
                                Object.defineProperty(TargetImpressionShare.prototype, "_location_fraction_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["location_fraction_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * TargetImpressionShare _cpc_bid_ceiling_micros.
                                 * @member {"cpc_bid_ceiling_micros"|undefined} _cpc_bid_ceiling_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @instance
                                 */
                                Object.defineProperty(TargetImpressionShare.prototype, "_cpc_bid_ceiling_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["cpc_bid_ceiling_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetImpressionShare instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetImpressionShare=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare} TargetImpressionShare instance
                                 */
                                TargetImpressionShare.create = function create(properties) {
                                    return new TargetImpressionShare(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetImpressionShare message. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetImpressionShare} m TargetImpressionShare message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetImpressionShare.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.location != null && Object.hasOwnProperty.call(m, "location"))
                                        w.uint32(8).int32(m.location);
                                    if (m.location_fraction_micros != null && Object.hasOwnProperty.call(m, "location_fraction_micros"))
                                        w.uint32(16).int64(m.location_fraction_micros);
                                    if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                        w.uint32(24).int64(m.cpc_bid_ceiling_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetImpressionShare message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetImpressionShare} message TargetImpressionShare message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetImpressionShare.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetImpressionShare message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare} TargetImpressionShare
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetImpressionShare.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.location = r.int32();
                                            break;
                                        case 2:
                                            m.location_fraction_micros = r.int64();
                                            break;
                                        case 3:
                                            m.cpc_bid_ceiling_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetImpressionShare message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare} TargetImpressionShare
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetImpressionShare.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetImpressionShare message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetImpressionShare.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.location != null && m.hasOwnProperty("location")) {
                                        switch (m.location) {
                                        default:
                                            return "location: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    if (m.location_fraction_micros != null && m.hasOwnProperty("location_fraction_micros")) {
                                        p._location_fraction_micros = 1;
                                        if (!$util.isInteger(m.location_fraction_micros) && !(m.location_fraction_micros && $util.isInteger(m.location_fraction_micros.low) && $util.isInteger(m.location_fraction_micros.high)))
                                            return "location_fraction_micros: integer|Long expected";
                                    }
                                    if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                        p._cpc_bid_ceiling_micros = 1;
                                        if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                            return "cpc_bid_ceiling_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetImpressionShare message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare} TargetImpressionShare
                                 */
                                TargetImpressionShare.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare();
                                    switch (d.location) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.location = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.location = 1;
                                        break;
                                    case "ANYWHERE_ON_PAGE":
                                    case 2:
                                        m.location = 2;
                                        break;
                                    case "TOP_OF_PAGE":
                                    case 3:
                                        m.location = 3;
                                        break;
                                    case "ABSOLUTE_TOP_OF_PAGE":
                                    case 4:
                                        m.location = 4;
                                        break;
                                    }
                                    if (d.location_fraction_micros != null) {
                                        if ($util.Long)
                                            (m.location_fraction_micros = $util.Long.fromValue(d.location_fraction_micros)).unsigned = false;
                                        else if (typeof d.location_fraction_micros === "string")
                                            m.location_fraction_micros = parseInt(d.location_fraction_micros, 10);
                                        else if (typeof d.location_fraction_micros === "number")
                                            m.location_fraction_micros = d.location_fraction_micros;
                                        else if (typeof d.location_fraction_micros === "object")
                                            m.location_fraction_micros = new $util.LongBits(d.location_fraction_micros.low >>> 0, d.location_fraction_micros.high >>> 0).toNumber();
                                    }
                                    if (d.cpc_bid_ceiling_micros != null) {
                                        if ($util.Long)
                                            (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                        else if (typeof d.cpc_bid_ceiling_micros === "string")
                                            m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                        else if (typeof d.cpc_bid_ceiling_micros === "number")
                                            m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                        else if (typeof d.cpc_bid_ceiling_micros === "object")
                                            m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetImpressionShare message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare} m TargetImpressionShare
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetImpressionShare.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.location = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.location != null && m.hasOwnProperty("location")) {
                                        d.location = o.enums === String ? $root.google.ads.googleads.v11.enums.TargetImpressionShareLocationEnum.TargetImpressionShareLocation[m.location] : m.location;
                                    }
                                    if (m.location_fraction_micros != null && m.hasOwnProperty("location_fraction_micros")) {
                                        if (typeof m.location_fraction_micros === "number")
                                            d.location_fraction_micros = o.longs === String ? String(m.location_fraction_micros) : m.location_fraction_micros;
                                        else
                                            d.location_fraction_micros = o.longs === String ? $util.Long.prototype.toString.call(m.location_fraction_micros) : o.longs === Number ? new $util.LongBits(m.location_fraction_micros.low >>> 0, m.location_fraction_micros.high >>> 0).toNumber() : m.location_fraction_micros;
                                        if (o.oneofs)
                                            d._location_fraction_micros = "location_fraction_micros";
                                    }
                                    if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                        if (typeof m.cpc_bid_ceiling_micros === "number")
                                            d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                        else
                                            d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                        if (o.oneofs)
                                            d._cpc_bid_ceiling_micros = "cpc_bid_ceiling_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetImpressionShare to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetImpressionShare
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetImpressionShare.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TargetImpressionShare;
                            })();
    
                            AccessibleBiddingStrategy.TargetRoas = (function() {
    
                                /**
                                 * Properties of a TargetRoas.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @interface ITargetRoas
                                 * @property {number|null} [target_roas] TargetRoas target_roas
                                 */
    
                                /**
                                 * Constructs a new TargetRoas.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @classdesc Represents a TargetRoas.
                                 * @implements ITargetRoas
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetRoas=} [p] Properties to set
                                 */
                                function TargetRoas(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetRoas target_roas.
                                 * @member {number|null|undefined} target_roas
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @instance
                                 */
                                TargetRoas.prototype.target_roas = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetRoas _target_roas.
                                 * @member {"target_roas"|undefined} _target_roas
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @instance
                                 */
                                Object.defineProperty(TargetRoas.prototype, "_target_roas", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_roas"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetRoas instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetRoas=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas} TargetRoas instance
                                 */
                                TargetRoas.create = function create(properties) {
                                    return new TargetRoas(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetRoas message. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetRoas} m TargetRoas message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetRoas.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                        w.uint32(9).double(m.target_roas);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetRoas message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetRoas} message TargetRoas message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetRoas.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetRoas message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas} TargetRoas
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetRoas.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.target_roas = r.double();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetRoas message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas} TargetRoas
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetRoas.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetRoas message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetRoas.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                        p._target_roas = 1;
                                        if (typeof m.target_roas !== "number")
                                            return "target_roas: number expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetRoas message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas} TargetRoas
                                 */
                                TargetRoas.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas();
                                    if (d.target_roas != null) {
                                        m.target_roas = Number(d.target_roas);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetRoas message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas} m TargetRoas
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetRoas.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                        d.target_roas = o.json && !isFinite(m.target_roas) ? String(m.target_roas) : m.target_roas;
                                        if (o.oneofs)
                                            d._target_roas = "target_roas";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetRoas to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetRoas
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetRoas.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TargetRoas;
                            })();
    
                            AccessibleBiddingStrategy.TargetSpend = (function() {
    
                                /**
                                 * Properties of a TargetSpend.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @interface ITargetSpend
                                 * @property {number|null} [target_spend_micros] TargetSpend target_spend_micros
                                 * @property {number|null} [cpc_bid_ceiling_micros] TargetSpend cpc_bid_ceiling_micros
                                 */
    
                                /**
                                 * Constructs a new TargetSpend.
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy
                                 * @classdesc Represents a TargetSpend.
                                 * @implements ITargetSpend
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetSpend=} [p] Properties to set
                                 */
                                function TargetSpend(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetSpend target_spend_micros.
                                 * @member {number|null|undefined} target_spend_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @instance
                                 */
                                TargetSpend.prototype.target_spend_micros = null;
    
                                /**
                                 * TargetSpend cpc_bid_ceiling_micros.
                                 * @member {number|null|undefined} cpc_bid_ceiling_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @instance
                                 */
                                TargetSpend.prototype.cpc_bid_ceiling_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetSpend _target_spend_micros.
                                 * @member {"target_spend_micros"|undefined} _target_spend_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @instance
                                 */
                                Object.defineProperty(TargetSpend.prototype, "_target_spend_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_spend_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * TargetSpend _cpc_bid_ceiling_micros.
                                 * @member {"cpc_bid_ceiling_micros"|undefined} _cpc_bid_ceiling_micros
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @instance
                                 */
                                Object.defineProperty(TargetSpend.prototype, "_cpc_bid_ceiling_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["cpc_bid_ceiling_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetSpend instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetSpend=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend} TargetSpend instance
                                 */
                                TargetSpend.create = function create(properties) {
                                    return new TargetSpend(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetSpend message. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetSpend} m TargetSpend message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetSpend.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_spend_micros != null && Object.hasOwnProperty.call(m, "target_spend_micros"))
                                        w.uint32(8).int64(m.target_spend_micros);
                                    if (m.cpc_bid_ceiling_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_ceiling_micros"))
                                        w.uint32(16).int64(m.cpc_bid_ceiling_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetSpend message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.ITargetSpend} message TargetSpend message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetSpend.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetSpend message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend} TargetSpend
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetSpend.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.target_spend_micros = r.int64();
                                            break;
                                        case 2:
                                            m.cpc_bid_ceiling_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetSpend message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend} TargetSpend
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetSpend.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetSpend message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetSpend.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.target_spend_micros != null && m.hasOwnProperty("target_spend_micros")) {
                                        p._target_spend_micros = 1;
                                        if (!$util.isInteger(m.target_spend_micros) && !(m.target_spend_micros && $util.isInteger(m.target_spend_micros.low) && $util.isInteger(m.target_spend_micros.high)))
                                            return "target_spend_micros: integer|Long expected";
                                    }
                                    if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                        p._cpc_bid_ceiling_micros = 1;
                                        if (!$util.isInteger(m.cpc_bid_ceiling_micros) && !(m.cpc_bid_ceiling_micros && $util.isInteger(m.cpc_bid_ceiling_micros.low) && $util.isInteger(m.cpc_bid_ceiling_micros.high)))
                                            return "cpc_bid_ceiling_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetSpend message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend} TargetSpend
                                 */
                                TargetSpend.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend();
                                    if (d.target_spend_micros != null) {
                                        if ($util.Long)
                                            (m.target_spend_micros = $util.Long.fromValue(d.target_spend_micros)).unsigned = false;
                                        else if (typeof d.target_spend_micros === "string")
                                            m.target_spend_micros = parseInt(d.target_spend_micros, 10);
                                        else if (typeof d.target_spend_micros === "number")
                                            m.target_spend_micros = d.target_spend_micros;
                                        else if (typeof d.target_spend_micros === "object")
                                            m.target_spend_micros = new $util.LongBits(d.target_spend_micros.low >>> 0, d.target_spend_micros.high >>> 0).toNumber();
                                    }
                                    if (d.cpc_bid_ceiling_micros != null) {
                                        if ($util.Long)
                                            (m.cpc_bid_ceiling_micros = $util.Long.fromValue(d.cpc_bid_ceiling_micros)).unsigned = false;
                                        else if (typeof d.cpc_bid_ceiling_micros === "string")
                                            m.cpc_bid_ceiling_micros = parseInt(d.cpc_bid_ceiling_micros, 10);
                                        else if (typeof d.cpc_bid_ceiling_micros === "number")
                                            m.cpc_bid_ceiling_micros = d.cpc_bid_ceiling_micros;
                                        else if (typeof d.cpc_bid_ceiling_micros === "object")
                                            m.cpc_bid_ceiling_micros = new $util.LongBits(d.cpc_bid_ceiling_micros.low >>> 0, d.cpc_bid_ceiling_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetSpend message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend} m TargetSpend
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetSpend.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.target_spend_micros != null && m.hasOwnProperty("target_spend_micros")) {
                                        if (typeof m.target_spend_micros === "number")
                                            d.target_spend_micros = o.longs === String ? String(m.target_spend_micros) : m.target_spend_micros;
                                        else
                                            d.target_spend_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_spend_micros) : o.longs === Number ? new $util.LongBits(m.target_spend_micros.low >>> 0, m.target_spend_micros.high >>> 0).toNumber() : m.target_spend_micros;
                                        if (o.oneofs)
                                            d._target_spend_micros = "target_spend_micros";
                                    }
                                    if (m.cpc_bid_ceiling_micros != null && m.hasOwnProperty("cpc_bid_ceiling_micros")) {
                                        if (typeof m.cpc_bid_ceiling_micros === "number")
                                            d.cpc_bid_ceiling_micros = o.longs === String ? String(m.cpc_bid_ceiling_micros) : m.cpc_bid_ceiling_micros;
                                        else
                                            d.cpc_bid_ceiling_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_ceiling_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_ceiling_micros.low >>> 0, m.cpc_bid_ceiling_micros.high >>> 0).toNumber() : m.cpc_bid_ceiling_micros;
                                        if (o.oneofs)
                                            d._cpc_bid_ceiling_micros = "cpc_bid_ceiling_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetSpend to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AccessibleBiddingStrategy.TargetSpend
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetSpend.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TargetSpend;
                            })();
    
                            return AccessibleBiddingStrategy;
                        })();
    
                        resources.AccountBudget = (function() {
    
                            /**
                             * Properties of an AccountBudget.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAccountBudget
                             * @property {string|null} [resource_name] AccountBudget resource_name
                             * @property {number|null} [id] AccountBudget id
                             * @property {string|null} [billing_setup] AccountBudget billing_setup
                             * @property {google.ads.googleads.v11.enums.AccountBudgetStatusEnum.AccountBudgetStatus|null} [status] AccountBudget status
                             * @property {string|null} [name] AccountBudget name
                             * @property {string|null} [proposed_start_date_time] AccountBudget proposed_start_date_time
                             * @property {string|null} [approved_start_date_time] AccountBudget approved_start_date_time
                             * @property {number|null} [total_adjustments_micros] AccountBudget total_adjustments_micros
                             * @property {number|null} [amount_served_micros] AccountBudget amount_served_micros
                             * @property {string|null} [purchase_order_number] AccountBudget purchase_order_number
                             * @property {string|null} [notes] AccountBudget notes
                             * @property {google.ads.googleads.v11.resources.AccountBudget.IPendingAccountBudgetProposal|null} [pending_proposal] AccountBudget pending_proposal
                             * @property {string|null} [proposed_end_date_time] AccountBudget proposed_end_date_time
                             * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [proposed_end_time_type] AccountBudget proposed_end_time_type
                             * @property {string|null} [approved_end_date_time] AccountBudget approved_end_date_time
                             * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [approved_end_time_type] AccountBudget approved_end_time_type
                             * @property {number|null} [proposed_spending_limit_micros] AccountBudget proposed_spending_limit_micros
                             * @property {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null} [proposed_spending_limit_type] AccountBudget proposed_spending_limit_type
                             * @property {number|null} [approved_spending_limit_micros] AccountBudget approved_spending_limit_micros
                             * @property {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null} [approved_spending_limit_type] AccountBudget approved_spending_limit_type
                             * @property {number|null} [adjusted_spending_limit_micros] AccountBudget adjusted_spending_limit_micros
                             * @property {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null} [adjusted_spending_limit_type] AccountBudget adjusted_spending_limit_type
                             */
    
                            /**
                             * Constructs a new AccountBudget.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AccountBudget.
                             * @implements IAccountBudget
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAccountBudget=} [p] Properties to set
                             */
                            function AccountBudget(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AccountBudget resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.resource_name = "";
    
                            /**
                             * AccountBudget id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.id = null;
    
                            /**
                             * AccountBudget billing_setup.
                             * @member {string|null|undefined} billing_setup
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.billing_setup = null;
    
                            /**
                             * AccountBudget status.
                             * @member {google.ads.googleads.v11.enums.AccountBudgetStatusEnum.AccountBudgetStatus} status
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.status = 0;
    
                            /**
                             * AccountBudget name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.name = null;
    
                            /**
                             * AccountBudget proposed_start_date_time.
                             * @member {string|null|undefined} proposed_start_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.proposed_start_date_time = null;
    
                            /**
                             * AccountBudget approved_start_date_time.
                             * @member {string|null|undefined} approved_start_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.approved_start_date_time = null;
    
                            /**
                             * AccountBudget total_adjustments_micros.
                             * @member {number} total_adjustments_micros
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.total_adjustments_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AccountBudget amount_served_micros.
                             * @member {number} amount_served_micros
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.amount_served_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AccountBudget purchase_order_number.
                             * @member {string|null|undefined} purchase_order_number
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.purchase_order_number = null;
    
                            /**
                             * AccountBudget notes.
                             * @member {string|null|undefined} notes
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.notes = null;
    
                            /**
                             * AccountBudget pending_proposal.
                             * @member {google.ads.googleads.v11.resources.AccountBudget.IPendingAccountBudgetProposal|null|undefined} pending_proposal
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.pending_proposal = null;
    
                            /**
                             * AccountBudget proposed_end_date_time.
                             * @member {string|null|undefined} proposed_end_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.proposed_end_date_time = null;
    
                            /**
                             * AccountBudget proposed_end_time_type.
                             * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} proposed_end_time_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.proposed_end_time_type = null;
    
                            /**
                             * AccountBudget approved_end_date_time.
                             * @member {string|null|undefined} approved_end_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.approved_end_date_time = null;
    
                            /**
                             * AccountBudget approved_end_time_type.
                             * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} approved_end_time_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.approved_end_time_type = null;
    
                            /**
                             * AccountBudget proposed_spending_limit_micros.
                             * @member {number|null|undefined} proposed_spending_limit_micros
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.proposed_spending_limit_micros = null;
    
                            /**
                             * AccountBudget proposed_spending_limit_type.
                             * @member {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null|undefined} proposed_spending_limit_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.proposed_spending_limit_type = null;
    
                            /**
                             * AccountBudget approved_spending_limit_micros.
                             * @member {number|null|undefined} approved_spending_limit_micros
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.approved_spending_limit_micros = null;
    
                            /**
                             * AccountBudget approved_spending_limit_type.
                             * @member {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null|undefined} approved_spending_limit_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.approved_spending_limit_type = null;
    
                            /**
                             * AccountBudget adjusted_spending_limit_micros.
                             * @member {number|null|undefined} adjusted_spending_limit_micros
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.adjusted_spending_limit_micros = null;
    
                            /**
                             * AccountBudget adjusted_spending_limit_type.
                             * @member {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null|undefined} adjusted_spending_limit_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            AccountBudget.prototype.adjusted_spending_limit_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AccountBudget _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget _billing_setup.
                             * @member {"billing_setup"|undefined} _billing_setup
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "_billing_setup", {
                                get: $util.oneOfGetter($oneOfFields = ["billing_setup"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget _proposed_start_date_time.
                             * @member {"proposed_start_date_time"|undefined} _proposed_start_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "_proposed_start_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_start_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget _approved_start_date_time.
                             * @member {"approved_start_date_time"|undefined} _approved_start_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "_approved_start_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["approved_start_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget _purchase_order_number.
                             * @member {"purchase_order_number"|undefined} _purchase_order_number
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "_purchase_order_number", {
                                get: $util.oneOfGetter($oneOfFields = ["purchase_order_number"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget _notes.
                             * @member {"notes"|undefined} _notes
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "_notes", {
                                get: $util.oneOfGetter($oneOfFields = ["notes"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget proposed_end_time.
                             * @member {"proposed_end_date_time"|"proposed_end_time_type"|undefined} proposed_end_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "proposed_end_time", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_end_date_time", "proposed_end_time_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget approved_end_time.
                             * @member {"approved_end_date_time"|"approved_end_time_type"|undefined} approved_end_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "approved_end_time", {
                                get: $util.oneOfGetter($oneOfFields = ["approved_end_date_time", "approved_end_time_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget proposed_spending_limit.
                             * @member {"proposed_spending_limit_micros"|"proposed_spending_limit_type"|undefined} proposed_spending_limit
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "proposed_spending_limit", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_spending_limit_micros", "proposed_spending_limit_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget approved_spending_limit.
                             * @member {"approved_spending_limit_micros"|"approved_spending_limit_type"|undefined} approved_spending_limit
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "approved_spending_limit", {
                                get: $util.oneOfGetter($oneOfFields = ["approved_spending_limit_micros", "approved_spending_limit_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudget adjusted_spending_limit.
                             * @member {"adjusted_spending_limit_micros"|"adjusted_spending_limit_type"|undefined} adjusted_spending_limit
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             */
                            Object.defineProperty(AccountBudget.prototype, "adjusted_spending_limit", {
                                get: $util.oneOfGetter($oneOfFields = ["adjusted_spending_limit_micros", "adjusted_spending_limit_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AccountBudget instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountBudget=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AccountBudget} AccountBudget instance
                             */
                            AccountBudget.create = function create(properties) {
                                return new AccountBudget(properties);
                            };
    
                            /**
                             * Encodes the specified AccountBudget message. Does not implicitly {@link google.ads.googleads.v11.resources.AccountBudget.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountBudget} m AccountBudget message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudget.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.proposed_end_time_type != null && Object.hasOwnProperty.call(m, "proposed_end_time_type"))
                                    w.uint32(72).int32(m.proposed_end_time_type);
                                if (m.approved_end_time_type != null && Object.hasOwnProperty.call(m, "approved_end_time_type"))
                                    w.uint32(88).int32(m.approved_end_time_type);
                                if (m.proposed_spending_limit_type != null && Object.hasOwnProperty.call(m, "proposed_spending_limit_type"))
                                    w.uint32(104).int32(m.proposed_spending_limit_type);
                                if (m.approved_spending_limit_type != null && Object.hasOwnProperty.call(m, "approved_spending_limit_type"))
                                    w.uint32(120).int32(m.approved_spending_limit_type);
                                if (m.adjusted_spending_limit_type != null && Object.hasOwnProperty.call(m, "adjusted_spending_limit_type"))
                                    w.uint32(136).int32(m.adjusted_spending_limit_type);
                                if (m.pending_proposal != null && Object.hasOwnProperty.call(m, "pending_proposal"))
                                    $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal.encode(m.pending_proposal, w.uint32(178).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(184).int64(m.id);
                                if (m.billing_setup != null && Object.hasOwnProperty.call(m, "billing_setup"))
                                    w.uint32(194).string(m.billing_setup);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(202).string(m.name);
                                if (m.proposed_start_date_time != null && Object.hasOwnProperty.call(m, "proposed_start_date_time"))
                                    w.uint32(210).string(m.proposed_start_date_time);
                                if (m.approved_start_date_time != null && Object.hasOwnProperty.call(m, "approved_start_date_time"))
                                    w.uint32(218).string(m.approved_start_date_time);
                                if (m.proposed_end_date_time != null && Object.hasOwnProperty.call(m, "proposed_end_date_time"))
                                    w.uint32(226).string(m.proposed_end_date_time);
                                if (m.approved_end_date_time != null && Object.hasOwnProperty.call(m, "approved_end_date_time"))
                                    w.uint32(234).string(m.approved_end_date_time);
                                if (m.proposed_spending_limit_micros != null && Object.hasOwnProperty.call(m, "proposed_spending_limit_micros"))
                                    w.uint32(240).int64(m.proposed_spending_limit_micros);
                                if (m.approved_spending_limit_micros != null && Object.hasOwnProperty.call(m, "approved_spending_limit_micros"))
                                    w.uint32(248).int64(m.approved_spending_limit_micros);
                                if (m.adjusted_spending_limit_micros != null && Object.hasOwnProperty.call(m, "adjusted_spending_limit_micros"))
                                    w.uint32(256).int64(m.adjusted_spending_limit_micros);
                                if (m.total_adjustments_micros != null && Object.hasOwnProperty.call(m, "total_adjustments_micros"))
                                    w.uint32(264).int64(m.total_adjustments_micros);
                                if (m.amount_served_micros != null && Object.hasOwnProperty.call(m, "amount_served_micros"))
                                    w.uint32(272).int64(m.amount_served_micros);
                                if (m.purchase_order_number != null && Object.hasOwnProperty.call(m, "purchase_order_number"))
                                    w.uint32(282).string(m.purchase_order_number);
                                if (m.notes != null && Object.hasOwnProperty.call(m, "notes"))
                                    w.uint32(290).string(m.notes);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountBudget message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccountBudget.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountBudget} message AccountBudget message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudget.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountBudget message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AccountBudget} AccountBudget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudget.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccountBudget();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 23:
                                        m.id = r.int64();
                                        break;
                                    case 24:
                                        m.billing_setup = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 25:
                                        m.name = r.string();
                                        break;
                                    case 26:
                                        m.proposed_start_date_time = r.string();
                                        break;
                                    case 27:
                                        m.approved_start_date_time = r.string();
                                        break;
                                    case 33:
                                        m.total_adjustments_micros = r.int64();
                                        break;
                                    case 34:
                                        m.amount_served_micros = r.int64();
                                        break;
                                    case 35:
                                        m.purchase_order_number = r.string();
                                        break;
                                    case 36:
                                        m.notes = r.string();
                                        break;
                                    case 22:
                                        m.pending_proposal = $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.proposed_end_date_time = r.string();
                                        break;
                                    case 9:
                                        m.proposed_end_time_type = r.int32();
                                        break;
                                    case 29:
                                        m.approved_end_date_time = r.string();
                                        break;
                                    case 11:
                                        m.approved_end_time_type = r.int32();
                                        break;
                                    case 30:
                                        m.proposed_spending_limit_micros = r.int64();
                                        break;
                                    case 13:
                                        m.proposed_spending_limit_type = r.int32();
                                        break;
                                    case 31:
                                        m.approved_spending_limit_micros = r.int64();
                                        break;
                                    case 15:
                                        m.approved_spending_limit_type = r.int32();
                                        break;
                                    case 32:
                                        m.adjusted_spending_limit_micros = r.int64();
                                        break;
                                    case 17:
                                        m.adjusted_spending_limit_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountBudget message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AccountBudget} AccountBudget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudget.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountBudget message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountBudget.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    p._billing_setup = 1;
                                    if (!$util.isString(m.billing_setup))
                                        return "billing_setup: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.proposed_start_date_time != null && m.hasOwnProperty("proposed_start_date_time")) {
                                    p._proposed_start_date_time = 1;
                                    if (!$util.isString(m.proposed_start_date_time))
                                        return "proposed_start_date_time: string expected";
                                }
                                if (m.approved_start_date_time != null && m.hasOwnProperty("approved_start_date_time")) {
                                    p._approved_start_date_time = 1;
                                    if (!$util.isString(m.approved_start_date_time))
                                        return "approved_start_date_time: string expected";
                                }
                                if (m.total_adjustments_micros != null && m.hasOwnProperty("total_adjustments_micros")) {
                                    if (!$util.isInteger(m.total_adjustments_micros) && !(m.total_adjustments_micros && $util.isInteger(m.total_adjustments_micros.low) && $util.isInteger(m.total_adjustments_micros.high)))
                                        return "total_adjustments_micros: integer|Long expected";
                                }
                                if (m.amount_served_micros != null && m.hasOwnProperty("amount_served_micros")) {
                                    if (!$util.isInteger(m.amount_served_micros) && !(m.amount_served_micros && $util.isInteger(m.amount_served_micros.low) && $util.isInteger(m.amount_served_micros.high)))
                                        return "amount_served_micros: integer|Long expected";
                                }
                                if (m.purchase_order_number != null && m.hasOwnProperty("purchase_order_number")) {
                                    p._purchase_order_number = 1;
                                    if (!$util.isString(m.purchase_order_number))
                                        return "purchase_order_number: string expected";
                                }
                                if (m.notes != null && m.hasOwnProperty("notes")) {
                                    p._notes = 1;
                                    if (!$util.isString(m.notes))
                                        return "notes: string expected";
                                }
                                if (m.pending_proposal != null && m.hasOwnProperty("pending_proposal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal.verify(m.pending_proposal);
                                        if (e)
                                            return "pending_proposal." + e;
                                    }
                                }
                                if (m.proposed_end_date_time != null && m.hasOwnProperty("proposed_end_date_time")) {
                                    p.proposed_end_time = 1;
                                    if (!$util.isString(m.proposed_end_date_time))
                                        return "proposed_end_date_time: string expected";
                                }
                                if (m.proposed_end_time_type != null && m.hasOwnProperty("proposed_end_time_type")) {
                                    if (p.proposed_end_time === 1)
                                        return "proposed_end_time: multiple values";
                                    p.proposed_end_time = 1;
                                    switch (m.proposed_end_time_type) {
                                    default:
                                        return "proposed_end_time_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.approved_end_date_time != null && m.hasOwnProperty("approved_end_date_time")) {
                                    p.approved_end_time = 1;
                                    if (!$util.isString(m.approved_end_date_time))
                                        return "approved_end_date_time: string expected";
                                }
                                if (m.approved_end_time_type != null && m.hasOwnProperty("approved_end_time_type")) {
                                    if (p.approved_end_time === 1)
                                        return "approved_end_time: multiple values";
                                    p.approved_end_time = 1;
                                    switch (m.approved_end_time_type) {
                                    default:
                                        return "approved_end_time_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.proposed_spending_limit_micros != null && m.hasOwnProperty("proposed_spending_limit_micros")) {
                                    p.proposed_spending_limit = 1;
                                    if (!$util.isInteger(m.proposed_spending_limit_micros) && !(m.proposed_spending_limit_micros && $util.isInteger(m.proposed_spending_limit_micros.low) && $util.isInteger(m.proposed_spending_limit_micros.high)))
                                        return "proposed_spending_limit_micros: integer|Long expected";
                                }
                                if (m.proposed_spending_limit_type != null && m.hasOwnProperty("proposed_spending_limit_type")) {
                                    if (p.proposed_spending_limit === 1)
                                        return "proposed_spending_limit: multiple values";
                                    p.proposed_spending_limit = 1;
                                    switch (m.proposed_spending_limit_type) {
                                    default:
                                        return "proposed_spending_limit_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.approved_spending_limit_micros != null && m.hasOwnProperty("approved_spending_limit_micros")) {
                                    p.approved_spending_limit = 1;
                                    if (!$util.isInteger(m.approved_spending_limit_micros) && !(m.approved_spending_limit_micros && $util.isInteger(m.approved_spending_limit_micros.low) && $util.isInteger(m.approved_spending_limit_micros.high)))
                                        return "approved_spending_limit_micros: integer|Long expected";
                                }
                                if (m.approved_spending_limit_type != null && m.hasOwnProperty("approved_spending_limit_type")) {
                                    if (p.approved_spending_limit === 1)
                                        return "approved_spending_limit: multiple values";
                                    p.approved_spending_limit = 1;
                                    switch (m.approved_spending_limit_type) {
                                    default:
                                        return "approved_spending_limit_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.adjusted_spending_limit_micros != null && m.hasOwnProperty("adjusted_spending_limit_micros")) {
                                    p.adjusted_spending_limit = 1;
                                    if (!$util.isInteger(m.adjusted_spending_limit_micros) && !(m.adjusted_spending_limit_micros && $util.isInteger(m.adjusted_spending_limit_micros.low) && $util.isInteger(m.adjusted_spending_limit_micros.high)))
                                        return "adjusted_spending_limit_micros: integer|Long expected";
                                }
                                if (m.adjusted_spending_limit_type != null && m.hasOwnProperty("adjusted_spending_limit_type")) {
                                    if (p.adjusted_spending_limit === 1)
                                        return "adjusted_spending_limit: multiple values";
                                    p.adjusted_spending_limit = 1;
                                    switch (m.adjusted_spending_limit_type) {
                                    default:
                                        return "adjusted_spending_limit_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AccountBudget message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AccountBudget} AccountBudget
                             */
                            AccountBudget.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AccountBudget)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AccountBudget();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.billing_setup != null) {
                                    m.billing_setup = String(d.billing_setup);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "APPROVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "CANCELLED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.proposed_start_date_time != null) {
                                    m.proposed_start_date_time = String(d.proposed_start_date_time);
                                }
                                if (d.approved_start_date_time != null) {
                                    m.approved_start_date_time = String(d.approved_start_date_time);
                                }
                                if (d.total_adjustments_micros != null) {
                                    if ($util.Long)
                                        (m.total_adjustments_micros = $util.Long.fromValue(d.total_adjustments_micros)).unsigned = false;
                                    else if (typeof d.total_adjustments_micros === "string")
                                        m.total_adjustments_micros = parseInt(d.total_adjustments_micros, 10);
                                    else if (typeof d.total_adjustments_micros === "number")
                                        m.total_adjustments_micros = d.total_adjustments_micros;
                                    else if (typeof d.total_adjustments_micros === "object")
                                        m.total_adjustments_micros = new $util.LongBits(d.total_adjustments_micros.low >>> 0, d.total_adjustments_micros.high >>> 0).toNumber();
                                }
                                if (d.amount_served_micros != null) {
                                    if ($util.Long)
                                        (m.amount_served_micros = $util.Long.fromValue(d.amount_served_micros)).unsigned = false;
                                    else if (typeof d.amount_served_micros === "string")
                                        m.amount_served_micros = parseInt(d.amount_served_micros, 10);
                                    else if (typeof d.amount_served_micros === "number")
                                        m.amount_served_micros = d.amount_served_micros;
                                    else if (typeof d.amount_served_micros === "object")
                                        m.amount_served_micros = new $util.LongBits(d.amount_served_micros.low >>> 0, d.amount_served_micros.high >>> 0).toNumber();
                                }
                                if (d.purchase_order_number != null) {
                                    m.purchase_order_number = String(d.purchase_order_number);
                                }
                                if (d.notes != null) {
                                    m.notes = String(d.notes);
                                }
                                if (d.pending_proposal != null) {
                                    if (typeof d.pending_proposal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccountBudget.pending_proposal: object expected");
                                    m.pending_proposal = $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal.fromObject(d.pending_proposal);
                                }
                                if (d.proposed_end_date_time != null) {
                                    m.proposed_end_date_time = String(d.proposed_end_date_time);
                                }
                                switch (d.proposed_end_time_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.proposed_end_time_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.proposed_end_time_type = 1;
                                    break;
                                case "NOW":
                                case 2:
                                    m.proposed_end_time_type = 2;
                                    break;
                                case "FOREVER":
                                case 3:
                                    m.proposed_end_time_type = 3;
                                    break;
                                }
                                if (d.approved_end_date_time != null) {
                                    m.approved_end_date_time = String(d.approved_end_date_time);
                                }
                                switch (d.approved_end_time_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approved_end_time_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approved_end_time_type = 1;
                                    break;
                                case "NOW":
                                case 2:
                                    m.approved_end_time_type = 2;
                                    break;
                                case "FOREVER":
                                case 3:
                                    m.approved_end_time_type = 3;
                                    break;
                                }
                                if (d.proposed_spending_limit_micros != null) {
                                    if ($util.Long)
                                        (m.proposed_spending_limit_micros = $util.Long.fromValue(d.proposed_spending_limit_micros)).unsigned = false;
                                    else if (typeof d.proposed_spending_limit_micros === "string")
                                        m.proposed_spending_limit_micros = parseInt(d.proposed_spending_limit_micros, 10);
                                    else if (typeof d.proposed_spending_limit_micros === "number")
                                        m.proposed_spending_limit_micros = d.proposed_spending_limit_micros;
                                    else if (typeof d.proposed_spending_limit_micros === "object")
                                        m.proposed_spending_limit_micros = new $util.LongBits(d.proposed_spending_limit_micros.low >>> 0, d.proposed_spending_limit_micros.high >>> 0).toNumber();
                                }
                                switch (d.proposed_spending_limit_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.proposed_spending_limit_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.proposed_spending_limit_type = 1;
                                    break;
                                case "INFINITE":
                                case 2:
                                    m.proposed_spending_limit_type = 2;
                                    break;
                                }
                                if (d.approved_spending_limit_micros != null) {
                                    if ($util.Long)
                                        (m.approved_spending_limit_micros = $util.Long.fromValue(d.approved_spending_limit_micros)).unsigned = false;
                                    else if (typeof d.approved_spending_limit_micros === "string")
                                        m.approved_spending_limit_micros = parseInt(d.approved_spending_limit_micros, 10);
                                    else if (typeof d.approved_spending_limit_micros === "number")
                                        m.approved_spending_limit_micros = d.approved_spending_limit_micros;
                                    else if (typeof d.approved_spending_limit_micros === "object")
                                        m.approved_spending_limit_micros = new $util.LongBits(d.approved_spending_limit_micros.low >>> 0, d.approved_spending_limit_micros.high >>> 0).toNumber();
                                }
                                switch (d.approved_spending_limit_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approved_spending_limit_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approved_spending_limit_type = 1;
                                    break;
                                case "INFINITE":
                                case 2:
                                    m.approved_spending_limit_type = 2;
                                    break;
                                }
                                if (d.adjusted_spending_limit_micros != null) {
                                    if ($util.Long)
                                        (m.adjusted_spending_limit_micros = $util.Long.fromValue(d.adjusted_spending_limit_micros)).unsigned = false;
                                    else if (typeof d.adjusted_spending_limit_micros === "string")
                                        m.adjusted_spending_limit_micros = parseInt(d.adjusted_spending_limit_micros, 10);
                                    else if (typeof d.adjusted_spending_limit_micros === "number")
                                        m.adjusted_spending_limit_micros = d.adjusted_spending_limit_micros;
                                    else if (typeof d.adjusted_spending_limit_micros === "object")
                                        m.adjusted_spending_limit_micros = new $util.LongBits(d.adjusted_spending_limit_micros.low >>> 0, d.adjusted_spending_limit_micros.high >>> 0).toNumber();
                                }
                                switch (d.adjusted_spending_limit_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.adjusted_spending_limit_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.adjusted_spending_limit_type = 1;
                                    break;
                                case "INFINITE":
                                case 2:
                                    m.adjusted_spending_limit_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AccountBudget message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.AccountBudget} m AccountBudget
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountBudget.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.pending_proposal = null;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_adjustments_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_adjustments_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.amount_served_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.amount_served_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AccountBudgetStatusEnum.AccountBudgetStatus[m.status] : m.status;
                                }
                                if (m.proposed_end_time_type != null && m.hasOwnProperty("proposed_end_time_type")) {
                                    d.proposed_end_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.proposed_end_time_type] : m.proposed_end_time_type;
                                    if (o.oneofs)
                                        d.proposed_end_time = "proposed_end_time_type";
                                }
                                if (m.approved_end_time_type != null && m.hasOwnProperty("approved_end_time_type")) {
                                    d.approved_end_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.approved_end_time_type] : m.approved_end_time_type;
                                    if (o.oneofs)
                                        d.approved_end_time = "approved_end_time_type";
                                }
                                if (m.proposed_spending_limit_type != null && m.hasOwnProperty("proposed_spending_limit_type")) {
                                    d.proposed_spending_limit_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType[m.proposed_spending_limit_type] : m.proposed_spending_limit_type;
                                    if (o.oneofs)
                                        d.proposed_spending_limit = "proposed_spending_limit_type";
                                }
                                if (m.approved_spending_limit_type != null && m.hasOwnProperty("approved_spending_limit_type")) {
                                    d.approved_spending_limit_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType[m.approved_spending_limit_type] : m.approved_spending_limit_type;
                                    if (o.oneofs)
                                        d.approved_spending_limit = "approved_spending_limit_type";
                                }
                                if (m.adjusted_spending_limit_type != null && m.hasOwnProperty("adjusted_spending_limit_type")) {
                                    d.adjusted_spending_limit_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType[m.adjusted_spending_limit_type] : m.adjusted_spending_limit_type;
                                    if (o.oneofs)
                                        d.adjusted_spending_limit = "adjusted_spending_limit_type";
                                }
                                if (m.pending_proposal != null && m.hasOwnProperty("pending_proposal")) {
                                    d.pending_proposal = $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal.toObject(m.pending_proposal, o);
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    d.billing_setup = m.billing_setup;
                                    if (o.oneofs)
                                        d._billing_setup = "billing_setup";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.proposed_start_date_time != null && m.hasOwnProperty("proposed_start_date_time")) {
                                    d.proposed_start_date_time = m.proposed_start_date_time;
                                    if (o.oneofs)
                                        d._proposed_start_date_time = "proposed_start_date_time";
                                }
                                if (m.approved_start_date_time != null && m.hasOwnProperty("approved_start_date_time")) {
                                    d.approved_start_date_time = m.approved_start_date_time;
                                    if (o.oneofs)
                                        d._approved_start_date_time = "approved_start_date_time";
                                }
                                if (m.proposed_end_date_time != null && m.hasOwnProperty("proposed_end_date_time")) {
                                    d.proposed_end_date_time = m.proposed_end_date_time;
                                    if (o.oneofs)
                                        d.proposed_end_time = "proposed_end_date_time";
                                }
                                if (m.approved_end_date_time != null && m.hasOwnProperty("approved_end_date_time")) {
                                    d.approved_end_date_time = m.approved_end_date_time;
                                    if (o.oneofs)
                                        d.approved_end_time = "approved_end_date_time";
                                }
                                if (m.proposed_spending_limit_micros != null && m.hasOwnProperty("proposed_spending_limit_micros")) {
                                    if (typeof m.proposed_spending_limit_micros === "number")
                                        d.proposed_spending_limit_micros = o.longs === String ? String(m.proposed_spending_limit_micros) : m.proposed_spending_limit_micros;
                                    else
                                        d.proposed_spending_limit_micros = o.longs === String ? $util.Long.prototype.toString.call(m.proposed_spending_limit_micros) : o.longs === Number ? new $util.LongBits(m.proposed_spending_limit_micros.low >>> 0, m.proposed_spending_limit_micros.high >>> 0).toNumber() : m.proposed_spending_limit_micros;
                                    if (o.oneofs)
                                        d.proposed_spending_limit = "proposed_spending_limit_micros";
                                }
                                if (m.approved_spending_limit_micros != null && m.hasOwnProperty("approved_spending_limit_micros")) {
                                    if (typeof m.approved_spending_limit_micros === "number")
                                        d.approved_spending_limit_micros = o.longs === String ? String(m.approved_spending_limit_micros) : m.approved_spending_limit_micros;
                                    else
                                        d.approved_spending_limit_micros = o.longs === String ? $util.Long.prototype.toString.call(m.approved_spending_limit_micros) : o.longs === Number ? new $util.LongBits(m.approved_spending_limit_micros.low >>> 0, m.approved_spending_limit_micros.high >>> 0).toNumber() : m.approved_spending_limit_micros;
                                    if (o.oneofs)
                                        d.approved_spending_limit = "approved_spending_limit_micros";
                                }
                                if (m.adjusted_spending_limit_micros != null && m.hasOwnProperty("adjusted_spending_limit_micros")) {
                                    if (typeof m.adjusted_spending_limit_micros === "number")
                                        d.adjusted_spending_limit_micros = o.longs === String ? String(m.adjusted_spending_limit_micros) : m.adjusted_spending_limit_micros;
                                    else
                                        d.adjusted_spending_limit_micros = o.longs === String ? $util.Long.prototype.toString.call(m.adjusted_spending_limit_micros) : o.longs === Number ? new $util.LongBits(m.adjusted_spending_limit_micros.low >>> 0, m.adjusted_spending_limit_micros.high >>> 0).toNumber() : m.adjusted_spending_limit_micros;
                                    if (o.oneofs)
                                        d.adjusted_spending_limit = "adjusted_spending_limit_micros";
                                }
                                if (m.total_adjustments_micros != null && m.hasOwnProperty("total_adjustments_micros")) {
                                    if (typeof m.total_adjustments_micros === "number")
                                        d.total_adjustments_micros = o.longs === String ? String(m.total_adjustments_micros) : m.total_adjustments_micros;
                                    else
                                        d.total_adjustments_micros = o.longs === String ? $util.Long.prototype.toString.call(m.total_adjustments_micros) : o.longs === Number ? new $util.LongBits(m.total_adjustments_micros.low >>> 0, m.total_adjustments_micros.high >>> 0).toNumber() : m.total_adjustments_micros;
                                }
                                if (m.amount_served_micros != null && m.hasOwnProperty("amount_served_micros")) {
                                    if (typeof m.amount_served_micros === "number")
                                        d.amount_served_micros = o.longs === String ? String(m.amount_served_micros) : m.amount_served_micros;
                                    else
                                        d.amount_served_micros = o.longs === String ? $util.Long.prototype.toString.call(m.amount_served_micros) : o.longs === Number ? new $util.LongBits(m.amount_served_micros.low >>> 0, m.amount_served_micros.high >>> 0).toNumber() : m.amount_served_micros;
                                }
                                if (m.purchase_order_number != null && m.hasOwnProperty("purchase_order_number")) {
                                    d.purchase_order_number = m.purchase_order_number;
                                    if (o.oneofs)
                                        d._purchase_order_number = "purchase_order_number";
                                }
                                if (m.notes != null && m.hasOwnProperty("notes")) {
                                    d.notes = m.notes;
                                    if (o.oneofs)
                                        d._notes = "notes";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AccountBudget to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AccountBudget
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountBudget.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            AccountBudget.PendingAccountBudgetProposal = (function() {
    
                                /**
                                 * Properties of a PendingAccountBudgetProposal.
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget
                                 * @interface IPendingAccountBudgetProposal
                                 * @property {string|null} [account_budget_proposal] PendingAccountBudgetProposal account_budget_proposal
                                 * @property {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.AccountBudgetProposalType|null} [proposal_type] PendingAccountBudgetProposal proposal_type
                                 * @property {string|null} [name] PendingAccountBudgetProposal name
                                 * @property {string|null} [start_date_time] PendingAccountBudgetProposal start_date_time
                                 * @property {string|null} [purchase_order_number] PendingAccountBudgetProposal purchase_order_number
                                 * @property {string|null} [notes] PendingAccountBudgetProposal notes
                                 * @property {string|null} [creation_date_time] PendingAccountBudgetProposal creation_date_time
                                 * @property {string|null} [end_date_time] PendingAccountBudgetProposal end_date_time
                                 * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [end_time_type] PendingAccountBudgetProposal end_time_type
                                 * @property {number|null} [spending_limit_micros] PendingAccountBudgetProposal spending_limit_micros
                                 * @property {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null} [spending_limit_type] PendingAccountBudgetProposal spending_limit_type
                                 */
    
                                /**
                                 * Constructs a new PendingAccountBudgetProposal.
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget
                                 * @classdesc Represents a PendingAccountBudgetProposal.
                                 * @implements IPendingAccountBudgetProposal
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AccountBudget.IPendingAccountBudgetProposal=} [p] Properties to set
                                 */
                                function PendingAccountBudgetProposal(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * PendingAccountBudgetProposal account_budget_proposal.
                                 * @member {string|null|undefined} account_budget_proposal
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.account_budget_proposal = null;
    
                                /**
                                 * PendingAccountBudgetProposal proposal_type.
                                 * @member {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.AccountBudgetProposalType} proposal_type
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.proposal_type = 0;
    
                                /**
                                 * PendingAccountBudgetProposal name.
                                 * @member {string|null|undefined} name
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.name = null;
    
                                /**
                                 * PendingAccountBudgetProposal start_date_time.
                                 * @member {string|null|undefined} start_date_time
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.start_date_time = null;
    
                                /**
                                 * PendingAccountBudgetProposal purchase_order_number.
                                 * @member {string|null|undefined} purchase_order_number
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.purchase_order_number = null;
    
                                /**
                                 * PendingAccountBudgetProposal notes.
                                 * @member {string|null|undefined} notes
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.notes = null;
    
                                /**
                                 * PendingAccountBudgetProposal creation_date_time.
                                 * @member {string|null|undefined} creation_date_time
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.creation_date_time = null;
    
                                /**
                                 * PendingAccountBudgetProposal end_date_time.
                                 * @member {string|null|undefined} end_date_time
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.end_date_time = null;
    
                                /**
                                 * PendingAccountBudgetProposal end_time_type.
                                 * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} end_time_type
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.end_time_type = null;
    
                                /**
                                 * PendingAccountBudgetProposal spending_limit_micros.
                                 * @member {number|null|undefined} spending_limit_micros
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.spending_limit_micros = null;
    
                                /**
                                 * PendingAccountBudgetProposal spending_limit_type.
                                 * @member {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null|undefined} spending_limit_type
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                PendingAccountBudgetProposal.prototype.spending_limit_type = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * PendingAccountBudgetProposal _account_budget_proposal.
                                 * @member {"account_budget_proposal"|undefined} _account_budget_proposal
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "_account_budget_proposal", {
                                    get: $util.oneOfGetter($oneOfFields = ["account_budget_proposal"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PendingAccountBudgetProposal _name.
                                 * @member {"name"|undefined} _name
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "_name", {
                                    get: $util.oneOfGetter($oneOfFields = ["name"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PendingAccountBudgetProposal _start_date_time.
                                 * @member {"start_date_time"|undefined} _start_date_time
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "_start_date_time", {
                                    get: $util.oneOfGetter($oneOfFields = ["start_date_time"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PendingAccountBudgetProposal _purchase_order_number.
                                 * @member {"purchase_order_number"|undefined} _purchase_order_number
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "_purchase_order_number", {
                                    get: $util.oneOfGetter($oneOfFields = ["purchase_order_number"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PendingAccountBudgetProposal _notes.
                                 * @member {"notes"|undefined} _notes
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "_notes", {
                                    get: $util.oneOfGetter($oneOfFields = ["notes"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PendingAccountBudgetProposal _creation_date_time.
                                 * @member {"creation_date_time"|undefined} _creation_date_time
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "_creation_date_time", {
                                    get: $util.oneOfGetter($oneOfFields = ["creation_date_time"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PendingAccountBudgetProposal end_time.
                                 * @member {"end_date_time"|"end_time_type"|undefined} end_time
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "end_time", {
                                    get: $util.oneOfGetter($oneOfFields = ["end_date_time", "end_time_type"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PendingAccountBudgetProposal spending_limit.
                                 * @member {"spending_limit_micros"|"spending_limit_type"|undefined} spending_limit
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 */
                                Object.defineProperty(PendingAccountBudgetProposal.prototype, "spending_limit", {
                                    get: $util.oneOfGetter($oneOfFields = ["spending_limit_micros", "spending_limit_type"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new PendingAccountBudgetProposal instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccountBudget.IPendingAccountBudgetProposal=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal} PendingAccountBudgetProposal instance
                                 */
                                PendingAccountBudgetProposal.create = function create(properties) {
                                    return new PendingAccountBudgetProposal(properties);
                                };
    
                                /**
                                 * Encodes the specified PendingAccountBudgetProposal message. Does not implicitly {@link google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccountBudget.IPendingAccountBudgetProposal} m PendingAccountBudgetProposal message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PendingAccountBudgetProposal.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.proposal_type != null && Object.hasOwnProperty.call(m, "proposal_type"))
                                        w.uint32(16).int32(m.proposal_type);
                                    if (m.end_time_type != null && Object.hasOwnProperty.call(m, "end_time_type"))
                                        w.uint32(48).int32(m.end_time_type);
                                    if (m.spending_limit_type != null && Object.hasOwnProperty.call(m, "spending_limit_type"))
                                        w.uint32(64).int32(m.spending_limit_type);
                                    if (m.account_budget_proposal != null && Object.hasOwnProperty.call(m, "account_budget_proposal"))
                                        w.uint32(98).string(m.account_budget_proposal);
                                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                        w.uint32(106).string(m.name);
                                    if (m.start_date_time != null && Object.hasOwnProperty.call(m, "start_date_time"))
                                        w.uint32(114).string(m.start_date_time);
                                    if (m.end_date_time != null && Object.hasOwnProperty.call(m, "end_date_time"))
                                        w.uint32(122).string(m.end_date_time);
                                    if (m.spending_limit_micros != null && Object.hasOwnProperty.call(m, "spending_limit_micros"))
                                        w.uint32(128).int64(m.spending_limit_micros);
                                    if (m.purchase_order_number != null && Object.hasOwnProperty.call(m, "purchase_order_number"))
                                        w.uint32(138).string(m.purchase_order_number);
                                    if (m.notes != null && Object.hasOwnProperty.call(m, "notes"))
                                        w.uint32(146).string(m.notes);
                                    if (m.creation_date_time != null && Object.hasOwnProperty.call(m, "creation_date_time"))
                                        w.uint32(154).string(m.creation_date_time);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified PendingAccountBudgetProposal message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccountBudget.IPendingAccountBudgetProposal} message PendingAccountBudgetProposal message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PendingAccountBudgetProposal.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a PendingAccountBudgetProposal message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal} PendingAccountBudgetProposal
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PendingAccountBudgetProposal.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 12:
                                            m.account_budget_proposal = r.string();
                                            break;
                                        case 2:
                                            m.proposal_type = r.int32();
                                            break;
                                        case 13:
                                            m.name = r.string();
                                            break;
                                        case 14:
                                            m.start_date_time = r.string();
                                            break;
                                        case 17:
                                            m.purchase_order_number = r.string();
                                            break;
                                        case 18:
                                            m.notes = r.string();
                                            break;
                                        case 19:
                                            m.creation_date_time = r.string();
                                            break;
                                        case 15:
                                            m.end_date_time = r.string();
                                            break;
                                        case 6:
                                            m.end_time_type = r.int32();
                                            break;
                                        case 16:
                                            m.spending_limit_micros = r.int64();
                                            break;
                                        case 8:
                                            m.spending_limit_type = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a PendingAccountBudgetProposal message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal} PendingAccountBudgetProposal
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PendingAccountBudgetProposal.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a PendingAccountBudgetProposal message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                PendingAccountBudgetProposal.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.account_budget_proposal != null && m.hasOwnProperty("account_budget_proposal")) {
                                        p._account_budget_proposal = 1;
                                        if (!$util.isString(m.account_budget_proposal))
                                            return "account_budget_proposal: string expected";
                                    }
                                    if (m.proposal_type != null && m.hasOwnProperty("proposal_type")) {
                                        switch (m.proposal_type) {
                                        default:
                                            return "proposal_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                            break;
                                        }
                                    }
                                    if (m.name != null && m.hasOwnProperty("name")) {
                                        p._name = 1;
                                        if (!$util.isString(m.name))
                                            return "name: string expected";
                                    }
                                    if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                        p._start_date_time = 1;
                                        if (!$util.isString(m.start_date_time))
                                            return "start_date_time: string expected";
                                    }
                                    if (m.purchase_order_number != null && m.hasOwnProperty("purchase_order_number")) {
                                        p._purchase_order_number = 1;
                                        if (!$util.isString(m.purchase_order_number))
                                            return "purchase_order_number: string expected";
                                    }
                                    if (m.notes != null && m.hasOwnProperty("notes")) {
                                        p._notes = 1;
                                        if (!$util.isString(m.notes))
                                            return "notes: string expected";
                                    }
                                    if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                        p._creation_date_time = 1;
                                        if (!$util.isString(m.creation_date_time))
                                            return "creation_date_time: string expected";
                                    }
                                    if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                        p.end_time = 1;
                                        if (!$util.isString(m.end_date_time))
                                            return "end_date_time: string expected";
                                    }
                                    if (m.end_time_type != null && m.hasOwnProperty("end_time_type")) {
                                        if (p.end_time === 1)
                                            return "end_time: multiple values";
                                        p.end_time = 1;
                                        switch (m.end_time_type) {
                                        default:
                                            return "end_time_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    if (m.spending_limit_micros != null && m.hasOwnProperty("spending_limit_micros")) {
                                        p.spending_limit = 1;
                                        if (!$util.isInteger(m.spending_limit_micros) && !(m.spending_limit_micros && $util.isInteger(m.spending_limit_micros.low) && $util.isInteger(m.spending_limit_micros.high)))
                                            return "spending_limit_micros: integer|Long expected";
                                    }
                                    if (m.spending_limit_type != null && m.hasOwnProperty("spending_limit_type")) {
                                        if (p.spending_limit === 1)
                                            return "spending_limit: multiple values";
                                        p.spending_limit = 1;
                                        switch (m.spending_limit_type) {
                                        default:
                                            return "spending_limit_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a PendingAccountBudgetProposal message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal} PendingAccountBudgetProposal
                                 */
                                PendingAccountBudgetProposal.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal();
                                    if (d.account_budget_proposal != null) {
                                        m.account_budget_proposal = String(d.account_budget_proposal);
                                    }
                                    switch (d.proposal_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.proposal_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.proposal_type = 1;
                                        break;
                                    case "CREATE":
                                    case 2:
                                        m.proposal_type = 2;
                                        break;
                                    case "UPDATE":
                                    case 3:
                                        m.proposal_type = 3;
                                        break;
                                    case "END":
                                    case 4:
                                        m.proposal_type = 4;
                                        break;
                                    case "REMOVE":
                                    case 5:
                                        m.proposal_type = 5;
                                        break;
                                    }
                                    if (d.name != null) {
                                        m.name = String(d.name);
                                    }
                                    if (d.start_date_time != null) {
                                        m.start_date_time = String(d.start_date_time);
                                    }
                                    if (d.purchase_order_number != null) {
                                        m.purchase_order_number = String(d.purchase_order_number);
                                    }
                                    if (d.notes != null) {
                                        m.notes = String(d.notes);
                                    }
                                    if (d.creation_date_time != null) {
                                        m.creation_date_time = String(d.creation_date_time);
                                    }
                                    if (d.end_date_time != null) {
                                        m.end_date_time = String(d.end_date_time);
                                    }
                                    switch (d.end_time_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.end_time_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.end_time_type = 1;
                                        break;
                                    case "NOW":
                                    case 2:
                                        m.end_time_type = 2;
                                        break;
                                    case "FOREVER":
                                    case 3:
                                        m.end_time_type = 3;
                                        break;
                                    }
                                    if (d.spending_limit_micros != null) {
                                        if ($util.Long)
                                            (m.spending_limit_micros = $util.Long.fromValue(d.spending_limit_micros)).unsigned = false;
                                        else if (typeof d.spending_limit_micros === "string")
                                            m.spending_limit_micros = parseInt(d.spending_limit_micros, 10);
                                        else if (typeof d.spending_limit_micros === "number")
                                            m.spending_limit_micros = d.spending_limit_micros;
                                        else if (typeof d.spending_limit_micros === "object")
                                            m.spending_limit_micros = new $util.LongBits(d.spending_limit_micros.low >>> 0, d.spending_limit_micros.high >>> 0).toNumber();
                                    }
                                    switch (d.spending_limit_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.spending_limit_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.spending_limit_type = 1;
                                        break;
                                    case "INFINITE":
                                    case 2:
                                        m.spending_limit_type = 2;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a PendingAccountBudgetProposal message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal} m PendingAccountBudgetProposal
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                PendingAccountBudgetProposal.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.proposal_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.proposal_type != null && m.hasOwnProperty("proposal_type")) {
                                        d.proposal_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.AccountBudgetProposalType[m.proposal_type] : m.proposal_type;
                                    }
                                    if (m.end_time_type != null && m.hasOwnProperty("end_time_type")) {
                                        d.end_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.end_time_type] : m.end_time_type;
                                        if (o.oneofs)
                                            d.end_time = "end_time_type";
                                    }
                                    if (m.spending_limit_type != null && m.hasOwnProperty("spending_limit_type")) {
                                        d.spending_limit_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType[m.spending_limit_type] : m.spending_limit_type;
                                        if (o.oneofs)
                                            d.spending_limit = "spending_limit_type";
                                    }
                                    if (m.account_budget_proposal != null && m.hasOwnProperty("account_budget_proposal")) {
                                        d.account_budget_proposal = m.account_budget_proposal;
                                        if (o.oneofs)
                                            d._account_budget_proposal = "account_budget_proposal";
                                    }
                                    if (m.name != null && m.hasOwnProperty("name")) {
                                        d.name = m.name;
                                        if (o.oneofs)
                                            d._name = "name";
                                    }
                                    if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                        d.start_date_time = m.start_date_time;
                                        if (o.oneofs)
                                            d._start_date_time = "start_date_time";
                                    }
                                    if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                        d.end_date_time = m.end_date_time;
                                        if (o.oneofs)
                                            d.end_time = "end_date_time";
                                    }
                                    if (m.spending_limit_micros != null && m.hasOwnProperty("spending_limit_micros")) {
                                        if (typeof m.spending_limit_micros === "number")
                                            d.spending_limit_micros = o.longs === String ? String(m.spending_limit_micros) : m.spending_limit_micros;
                                        else
                                            d.spending_limit_micros = o.longs === String ? $util.Long.prototype.toString.call(m.spending_limit_micros) : o.longs === Number ? new $util.LongBits(m.spending_limit_micros.low >>> 0, m.spending_limit_micros.high >>> 0).toNumber() : m.spending_limit_micros;
                                        if (o.oneofs)
                                            d.spending_limit = "spending_limit_micros";
                                    }
                                    if (m.purchase_order_number != null && m.hasOwnProperty("purchase_order_number")) {
                                        d.purchase_order_number = m.purchase_order_number;
                                        if (o.oneofs)
                                            d._purchase_order_number = "purchase_order_number";
                                    }
                                    if (m.notes != null && m.hasOwnProperty("notes")) {
                                        d.notes = m.notes;
                                        if (o.oneofs)
                                            d._notes = "notes";
                                    }
                                    if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                        d.creation_date_time = m.creation_date_time;
                                        if (o.oneofs)
                                            d._creation_date_time = "creation_date_time";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this PendingAccountBudgetProposal to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AccountBudget.PendingAccountBudgetProposal
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                PendingAccountBudgetProposal.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return PendingAccountBudgetProposal;
                            })();
    
                            return AccountBudget;
                        })();
    
                        resources.AccountBudgetProposal = (function() {
    
                            /**
                             * Properties of an AccountBudgetProposal.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAccountBudgetProposal
                             * @property {string|null} [resource_name] AccountBudgetProposal resource_name
                             * @property {number|null} [id] AccountBudgetProposal id
                             * @property {string|null} [billing_setup] AccountBudgetProposal billing_setup
                             * @property {string|null} [account_budget] AccountBudgetProposal account_budget
                             * @property {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.AccountBudgetProposalType|null} [proposal_type] AccountBudgetProposal proposal_type
                             * @property {google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus|null} [status] AccountBudgetProposal status
                             * @property {string|null} [proposed_name] AccountBudgetProposal proposed_name
                             * @property {string|null} [approved_start_date_time] AccountBudgetProposal approved_start_date_time
                             * @property {string|null} [proposed_purchase_order_number] AccountBudgetProposal proposed_purchase_order_number
                             * @property {string|null} [proposed_notes] AccountBudgetProposal proposed_notes
                             * @property {string|null} [creation_date_time] AccountBudgetProposal creation_date_time
                             * @property {string|null} [approval_date_time] AccountBudgetProposal approval_date_time
                             * @property {string|null} [proposed_start_date_time] AccountBudgetProposal proposed_start_date_time
                             * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [proposed_start_time_type] AccountBudgetProposal proposed_start_time_type
                             * @property {string|null} [proposed_end_date_time] AccountBudgetProposal proposed_end_date_time
                             * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [proposed_end_time_type] AccountBudgetProposal proposed_end_time_type
                             * @property {string|null} [approved_end_date_time] AccountBudgetProposal approved_end_date_time
                             * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [approved_end_time_type] AccountBudgetProposal approved_end_time_type
                             * @property {number|null} [proposed_spending_limit_micros] AccountBudgetProposal proposed_spending_limit_micros
                             * @property {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null} [proposed_spending_limit_type] AccountBudgetProposal proposed_spending_limit_type
                             * @property {number|null} [approved_spending_limit_micros] AccountBudgetProposal approved_spending_limit_micros
                             * @property {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null} [approved_spending_limit_type] AccountBudgetProposal approved_spending_limit_type
                             */
    
                            /**
                             * Constructs a new AccountBudgetProposal.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AccountBudgetProposal.
                             * @implements IAccountBudgetProposal
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAccountBudgetProposal=} [p] Properties to set
                             */
                            function AccountBudgetProposal(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AccountBudgetProposal resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.resource_name = "";
    
                            /**
                             * AccountBudgetProposal id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.id = null;
    
                            /**
                             * AccountBudgetProposal billing_setup.
                             * @member {string|null|undefined} billing_setup
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.billing_setup = null;
    
                            /**
                             * AccountBudgetProposal account_budget.
                             * @member {string|null|undefined} account_budget
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.account_budget = null;
    
                            /**
                             * AccountBudgetProposal proposal_type.
                             * @member {google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.AccountBudgetProposalType} proposal_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposal_type = 0;
    
                            /**
                             * AccountBudgetProposal status.
                             * @member {google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus} status
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.status = 0;
    
                            /**
                             * AccountBudgetProposal proposed_name.
                             * @member {string|null|undefined} proposed_name
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_name = null;
    
                            /**
                             * AccountBudgetProposal approved_start_date_time.
                             * @member {string|null|undefined} approved_start_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.approved_start_date_time = null;
    
                            /**
                             * AccountBudgetProposal proposed_purchase_order_number.
                             * @member {string|null|undefined} proposed_purchase_order_number
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_purchase_order_number = null;
    
                            /**
                             * AccountBudgetProposal proposed_notes.
                             * @member {string|null|undefined} proposed_notes
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_notes = null;
    
                            /**
                             * AccountBudgetProposal creation_date_time.
                             * @member {string|null|undefined} creation_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.creation_date_time = null;
    
                            /**
                             * AccountBudgetProposal approval_date_time.
                             * @member {string|null|undefined} approval_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.approval_date_time = null;
    
                            /**
                             * AccountBudgetProposal proposed_start_date_time.
                             * @member {string|null|undefined} proposed_start_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_start_date_time = null;
    
                            /**
                             * AccountBudgetProposal proposed_start_time_type.
                             * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} proposed_start_time_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_start_time_type = null;
    
                            /**
                             * AccountBudgetProposal proposed_end_date_time.
                             * @member {string|null|undefined} proposed_end_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_end_date_time = null;
    
                            /**
                             * AccountBudgetProposal proposed_end_time_type.
                             * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} proposed_end_time_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_end_time_type = null;
    
                            /**
                             * AccountBudgetProposal approved_end_date_time.
                             * @member {string|null|undefined} approved_end_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.approved_end_date_time = null;
    
                            /**
                             * AccountBudgetProposal approved_end_time_type.
                             * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} approved_end_time_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.approved_end_time_type = null;
    
                            /**
                             * AccountBudgetProposal proposed_spending_limit_micros.
                             * @member {number|null|undefined} proposed_spending_limit_micros
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_spending_limit_micros = null;
    
                            /**
                             * AccountBudgetProposal proposed_spending_limit_type.
                             * @member {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null|undefined} proposed_spending_limit_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.proposed_spending_limit_type = null;
    
                            /**
                             * AccountBudgetProposal approved_spending_limit_micros.
                             * @member {number|null|undefined} approved_spending_limit_micros
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.approved_spending_limit_micros = null;
    
                            /**
                             * AccountBudgetProposal approved_spending_limit_type.
                             * @member {google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType|null|undefined} approved_spending_limit_type
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            AccountBudgetProposal.prototype.approved_spending_limit_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AccountBudgetProposal _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _billing_setup.
                             * @member {"billing_setup"|undefined} _billing_setup
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_billing_setup", {
                                get: $util.oneOfGetter($oneOfFields = ["billing_setup"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _account_budget.
                             * @member {"account_budget"|undefined} _account_budget
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_account_budget", {
                                get: $util.oneOfGetter($oneOfFields = ["account_budget"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _proposed_name.
                             * @member {"proposed_name"|undefined} _proposed_name
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_proposed_name", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _approved_start_date_time.
                             * @member {"approved_start_date_time"|undefined} _approved_start_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_approved_start_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["approved_start_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _proposed_purchase_order_number.
                             * @member {"proposed_purchase_order_number"|undefined} _proposed_purchase_order_number
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_proposed_purchase_order_number", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_purchase_order_number"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _proposed_notes.
                             * @member {"proposed_notes"|undefined} _proposed_notes
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_proposed_notes", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_notes"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _creation_date_time.
                             * @member {"creation_date_time"|undefined} _creation_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_creation_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["creation_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal _approval_date_time.
                             * @member {"approval_date_time"|undefined} _approval_date_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "_approval_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["approval_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal proposed_start_time.
                             * @member {"proposed_start_date_time"|"proposed_start_time_type"|undefined} proposed_start_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "proposed_start_time", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_start_date_time", "proposed_start_time_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal proposed_end_time.
                             * @member {"proposed_end_date_time"|"proposed_end_time_type"|undefined} proposed_end_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "proposed_end_time", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_end_date_time", "proposed_end_time_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal approved_end_time.
                             * @member {"approved_end_date_time"|"approved_end_time_type"|undefined} approved_end_time
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "approved_end_time", {
                                get: $util.oneOfGetter($oneOfFields = ["approved_end_date_time", "approved_end_time_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal proposed_spending_limit.
                             * @member {"proposed_spending_limit_micros"|"proposed_spending_limit_type"|undefined} proposed_spending_limit
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "proposed_spending_limit", {
                                get: $util.oneOfGetter($oneOfFields = ["proposed_spending_limit_micros", "proposed_spending_limit_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountBudgetProposal approved_spending_limit.
                             * @member {"approved_spending_limit_micros"|"approved_spending_limit_type"|undefined} approved_spending_limit
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposal.prototype, "approved_spending_limit", {
                                get: $util.oneOfGetter($oneOfFields = ["approved_spending_limit_micros", "approved_spending_limit_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AccountBudgetProposal instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountBudgetProposal=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AccountBudgetProposal} AccountBudgetProposal instance
                             */
                            AccountBudgetProposal.create = function create(properties) {
                                return new AccountBudgetProposal(properties);
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposal message. Does not implicitly {@link google.ads.googleads.v11.resources.AccountBudgetProposal.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountBudgetProposal} m AccountBudgetProposal message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposal.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.proposal_type != null && Object.hasOwnProperty.call(m, "proposal_type"))
                                    w.uint32(32).int32(m.proposal_type);
                                if (m.proposed_start_time_type != null && Object.hasOwnProperty.call(m, "proposed_start_time_type"))
                                    w.uint32(56).int32(m.proposed_start_time_type);
                                if (m.proposed_end_time_type != null && Object.hasOwnProperty.call(m, "proposed_end_time_type"))
                                    w.uint32(72).int32(m.proposed_end_time_type);
                                if (m.proposed_spending_limit_type != null && Object.hasOwnProperty.call(m, "proposed_spending_limit_type"))
                                    w.uint32(88).int32(m.proposed_spending_limit_type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(120).int32(m.status);
                                if (m.approved_end_time_type != null && Object.hasOwnProperty.call(m, "approved_end_time_type"))
                                    w.uint32(176).int32(m.approved_end_time_type);
                                if (m.approved_spending_limit_type != null && Object.hasOwnProperty.call(m, "approved_spending_limit_type"))
                                    w.uint32(192).int32(m.approved_spending_limit_type);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(200).int64(m.id);
                                if (m.billing_setup != null && Object.hasOwnProperty.call(m, "billing_setup"))
                                    w.uint32(210).string(m.billing_setup);
                                if (m.account_budget != null && Object.hasOwnProperty.call(m, "account_budget"))
                                    w.uint32(218).string(m.account_budget);
                                if (m.proposed_name != null && Object.hasOwnProperty.call(m, "proposed_name"))
                                    w.uint32(226).string(m.proposed_name);
                                if (m.proposed_start_date_time != null && Object.hasOwnProperty.call(m, "proposed_start_date_time"))
                                    w.uint32(234).string(m.proposed_start_date_time);
                                if (m.approved_start_date_time != null && Object.hasOwnProperty.call(m, "approved_start_date_time"))
                                    w.uint32(242).string(m.approved_start_date_time);
                                if (m.proposed_end_date_time != null && Object.hasOwnProperty.call(m, "proposed_end_date_time"))
                                    w.uint32(250).string(m.proposed_end_date_time);
                                if (m.approved_end_date_time != null && Object.hasOwnProperty.call(m, "approved_end_date_time"))
                                    w.uint32(258).string(m.approved_end_date_time);
                                if (m.proposed_spending_limit_micros != null && Object.hasOwnProperty.call(m, "proposed_spending_limit_micros"))
                                    w.uint32(264).int64(m.proposed_spending_limit_micros);
                                if (m.approved_spending_limit_micros != null && Object.hasOwnProperty.call(m, "approved_spending_limit_micros"))
                                    w.uint32(272).int64(m.approved_spending_limit_micros);
                                if (m.proposed_purchase_order_number != null && Object.hasOwnProperty.call(m, "proposed_purchase_order_number"))
                                    w.uint32(282).string(m.proposed_purchase_order_number);
                                if (m.proposed_notes != null && Object.hasOwnProperty.call(m, "proposed_notes"))
                                    w.uint32(290).string(m.proposed_notes);
                                if (m.creation_date_time != null && Object.hasOwnProperty.call(m, "creation_date_time"))
                                    w.uint32(298).string(m.creation_date_time);
                                if (m.approval_date_time != null && Object.hasOwnProperty.call(m, "approval_date_time"))
                                    w.uint32(306).string(m.approval_date_time);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposal message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccountBudgetProposal.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountBudgetProposal} message AccountBudgetProposal message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposal.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountBudgetProposal message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AccountBudgetProposal} AccountBudgetProposal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposal.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccountBudgetProposal();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 25:
                                        m.id = r.int64();
                                        break;
                                    case 26:
                                        m.billing_setup = r.string();
                                        break;
                                    case 27:
                                        m.account_budget = r.string();
                                        break;
                                    case 4:
                                        m.proposal_type = r.int32();
                                        break;
                                    case 15:
                                        m.status = r.int32();
                                        break;
                                    case 28:
                                        m.proposed_name = r.string();
                                        break;
                                    case 30:
                                        m.approved_start_date_time = r.string();
                                        break;
                                    case 35:
                                        m.proposed_purchase_order_number = r.string();
                                        break;
                                    case 36:
                                        m.proposed_notes = r.string();
                                        break;
                                    case 37:
                                        m.creation_date_time = r.string();
                                        break;
                                    case 38:
                                        m.approval_date_time = r.string();
                                        break;
                                    case 29:
                                        m.proposed_start_date_time = r.string();
                                        break;
                                    case 7:
                                        m.proposed_start_time_type = r.int32();
                                        break;
                                    case 31:
                                        m.proposed_end_date_time = r.string();
                                        break;
                                    case 9:
                                        m.proposed_end_time_type = r.int32();
                                        break;
                                    case 32:
                                        m.approved_end_date_time = r.string();
                                        break;
                                    case 22:
                                        m.approved_end_time_type = r.int32();
                                        break;
                                    case 33:
                                        m.proposed_spending_limit_micros = r.int64();
                                        break;
                                    case 11:
                                        m.proposed_spending_limit_type = r.int32();
                                        break;
                                    case 34:
                                        m.approved_spending_limit_micros = r.int64();
                                        break;
                                    case 24:
                                        m.approved_spending_limit_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountBudgetProposal message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AccountBudgetProposal} AccountBudgetProposal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposal.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountBudgetProposal message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountBudgetProposal.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    p._billing_setup = 1;
                                    if (!$util.isString(m.billing_setup))
                                        return "billing_setup: string expected";
                                }
                                if (m.account_budget != null && m.hasOwnProperty("account_budget")) {
                                    p._account_budget = 1;
                                    if (!$util.isString(m.account_budget))
                                        return "account_budget: string expected";
                                }
                                if (m.proposal_type != null && m.hasOwnProperty("proposal_type")) {
                                    switch (m.proposal_type) {
                                    default:
                                        return "proposal_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.proposed_name != null && m.hasOwnProperty("proposed_name")) {
                                    p._proposed_name = 1;
                                    if (!$util.isString(m.proposed_name))
                                        return "proposed_name: string expected";
                                }
                                if (m.approved_start_date_time != null && m.hasOwnProperty("approved_start_date_time")) {
                                    p._approved_start_date_time = 1;
                                    if (!$util.isString(m.approved_start_date_time))
                                        return "approved_start_date_time: string expected";
                                }
                                if (m.proposed_purchase_order_number != null && m.hasOwnProperty("proposed_purchase_order_number")) {
                                    p._proposed_purchase_order_number = 1;
                                    if (!$util.isString(m.proposed_purchase_order_number))
                                        return "proposed_purchase_order_number: string expected";
                                }
                                if (m.proposed_notes != null && m.hasOwnProperty("proposed_notes")) {
                                    p._proposed_notes = 1;
                                    if (!$util.isString(m.proposed_notes))
                                        return "proposed_notes: string expected";
                                }
                                if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                    p._creation_date_time = 1;
                                    if (!$util.isString(m.creation_date_time))
                                        return "creation_date_time: string expected";
                                }
                                if (m.approval_date_time != null && m.hasOwnProperty("approval_date_time")) {
                                    p._approval_date_time = 1;
                                    if (!$util.isString(m.approval_date_time))
                                        return "approval_date_time: string expected";
                                }
                                if (m.proposed_start_date_time != null && m.hasOwnProperty("proposed_start_date_time")) {
                                    p.proposed_start_time = 1;
                                    if (!$util.isString(m.proposed_start_date_time))
                                        return "proposed_start_date_time: string expected";
                                }
                                if (m.proposed_start_time_type != null && m.hasOwnProperty("proposed_start_time_type")) {
                                    if (p.proposed_start_time === 1)
                                        return "proposed_start_time: multiple values";
                                    p.proposed_start_time = 1;
                                    switch (m.proposed_start_time_type) {
                                    default:
                                        return "proposed_start_time_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.proposed_end_date_time != null && m.hasOwnProperty("proposed_end_date_time")) {
                                    p.proposed_end_time = 1;
                                    if (!$util.isString(m.proposed_end_date_time))
                                        return "proposed_end_date_time: string expected";
                                }
                                if (m.proposed_end_time_type != null && m.hasOwnProperty("proposed_end_time_type")) {
                                    if (p.proposed_end_time === 1)
                                        return "proposed_end_time: multiple values";
                                    p.proposed_end_time = 1;
                                    switch (m.proposed_end_time_type) {
                                    default:
                                        return "proposed_end_time_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.approved_end_date_time != null && m.hasOwnProperty("approved_end_date_time")) {
                                    p.approved_end_time = 1;
                                    if (!$util.isString(m.approved_end_date_time))
                                        return "approved_end_date_time: string expected";
                                }
                                if (m.approved_end_time_type != null && m.hasOwnProperty("approved_end_time_type")) {
                                    if (p.approved_end_time === 1)
                                        return "approved_end_time: multiple values";
                                    p.approved_end_time = 1;
                                    switch (m.approved_end_time_type) {
                                    default:
                                        return "approved_end_time_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.proposed_spending_limit_micros != null && m.hasOwnProperty("proposed_spending_limit_micros")) {
                                    p.proposed_spending_limit = 1;
                                    if (!$util.isInteger(m.proposed_spending_limit_micros) && !(m.proposed_spending_limit_micros && $util.isInteger(m.proposed_spending_limit_micros.low) && $util.isInteger(m.proposed_spending_limit_micros.high)))
                                        return "proposed_spending_limit_micros: integer|Long expected";
                                }
                                if (m.proposed_spending_limit_type != null && m.hasOwnProperty("proposed_spending_limit_type")) {
                                    if (p.proposed_spending_limit === 1)
                                        return "proposed_spending_limit: multiple values";
                                    p.proposed_spending_limit = 1;
                                    switch (m.proposed_spending_limit_type) {
                                    default:
                                        return "proposed_spending_limit_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.approved_spending_limit_micros != null && m.hasOwnProperty("approved_spending_limit_micros")) {
                                    p.approved_spending_limit = 1;
                                    if (!$util.isInteger(m.approved_spending_limit_micros) && !(m.approved_spending_limit_micros && $util.isInteger(m.approved_spending_limit_micros.low) && $util.isInteger(m.approved_spending_limit_micros.high)))
                                        return "approved_spending_limit_micros: integer|Long expected";
                                }
                                if (m.approved_spending_limit_type != null && m.hasOwnProperty("approved_spending_limit_type")) {
                                    if (p.approved_spending_limit === 1)
                                        return "approved_spending_limit: multiple values";
                                    p.approved_spending_limit = 1;
                                    switch (m.approved_spending_limit_type) {
                                    default:
                                        return "approved_spending_limit_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AccountBudgetProposal message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AccountBudgetProposal} AccountBudgetProposal
                             */
                            AccountBudgetProposal.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AccountBudgetProposal)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AccountBudgetProposal();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.billing_setup != null) {
                                    m.billing_setup = String(d.billing_setup);
                                }
                                if (d.account_budget != null) {
                                    m.account_budget = String(d.account_budget);
                                }
                                switch (d.proposal_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.proposal_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.proposal_type = 1;
                                    break;
                                case "CREATE":
                                case 2:
                                    m.proposal_type = 2;
                                    break;
                                case "UPDATE":
                                case 3:
                                    m.proposal_type = 3;
                                    break;
                                case "END":
                                case 4:
                                    m.proposal_type = 4;
                                    break;
                                case "REMOVE":
                                case 5:
                                    m.proposal_type = 5;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "APPROVED_HELD":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "CANCELLED":
                                case 5:
                                    m.status = 5;
                                    break;
                                case "REJECTED":
                                case 6:
                                    m.status = 6;
                                    break;
                                }
                                if (d.proposed_name != null) {
                                    m.proposed_name = String(d.proposed_name);
                                }
                                if (d.approved_start_date_time != null) {
                                    m.approved_start_date_time = String(d.approved_start_date_time);
                                }
                                if (d.proposed_purchase_order_number != null) {
                                    m.proposed_purchase_order_number = String(d.proposed_purchase_order_number);
                                }
                                if (d.proposed_notes != null) {
                                    m.proposed_notes = String(d.proposed_notes);
                                }
                                if (d.creation_date_time != null) {
                                    m.creation_date_time = String(d.creation_date_time);
                                }
                                if (d.approval_date_time != null) {
                                    m.approval_date_time = String(d.approval_date_time);
                                }
                                if (d.proposed_start_date_time != null) {
                                    m.proposed_start_date_time = String(d.proposed_start_date_time);
                                }
                                switch (d.proposed_start_time_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.proposed_start_time_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.proposed_start_time_type = 1;
                                    break;
                                case "NOW":
                                case 2:
                                    m.proposed_start_time_type = 2;
                                    break;
                                case "FOREVER":
                                case 3:
                                    m.proposed_start_time_type = 3;
                                    break;
                                }
                                if (d.proposed_end_date_time != null) {
                                    m.proposed_end_date_time = String(d.proposed_end_date_time);
                                }
                                switch (d.proposed_end_time_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.proposed_end_time_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.proposed_end_time_type = 1;
                                    break;
                                case "NOW":
                                case 2:
                                    m.proposed_end_time_type = 2;
                                    break;
                                case "FOREVER":
                                case 3:
                                    m.proposed_end_time_type = 3;
                                    break;
                                }
                                if (d.approved_end_date_time != null) {
                                    m.approved_end_date_time = String(d.approved_end_date_time);
                                }
                                switch (d.approved_end_time_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approved_end_time_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approved_end_time_type = 1;
                                    break;
                                case "NOW":
                                case 2:
                                    m.approved_end_time_type = 2;
                                    break;
                                case "FOREVER":
                                case 3:
                                    m.approved_end_time_type = 3;
                                    break;
                                }
                                if (d.proposed_spending_limit_micros != null) {
                                    if ($util.Long)
                                        (m.proposed_spending_limit_micros = $util.Long.fromValue(d.proposed_spending_limit_micros)).unsigned = false;
                                    else if (typeof d.proposed_spending_limit_micros === "string")
                                        m.proposed_spending_limit_micros = parseInt(d.proposed_spending_limit_micros, 10);
                                    else if (typeof d.proposed_spending_limit_micros === "number")
                                        m.proposed_spending_limit_micros = d.proposed_spending_limit_micros;
                                    else if (typeof d.proposed_spending_limit_micros === "object")
                                        m.proposed_spending_limit_micros = new $util.LongBits(d.proposed_spending_limit_micros.low >>> 0, d.proposed_spending_limit_micros.high >>> 0).toNumber();
                                }
                                switch (d.proposed_spending_limit_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.proposed_spending_limit_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.proposed_spending_limit_type = 1;
                                    break;
                                case "INFINITE":
                                case 2:
                                    m.proposed_spending_limit_type = 2;
                                    break;
                                }
                                if (d.approved_spending_limit_micros != null) {
                                    if ($util.Long)
                                        (m.approved_spending_limit_micros = $util.Long.fromValue(d.approved_spending_limit_micros)).unsigned = false;
                                    else if (typeof d.approved_spending_limit_micros === "string")
                                        m.approved_spending_limit_micros = parseInt(d.approved_spending_limit_micros, 10);
                                    else if (typeof d.approved_spending_limit_micros === "number")
                                        m.approved_spending_limit_micros = d.approved_spending_limit_micros;
                                    else if (typeof d.approved_spending_limit_micros === "object")
                                        m.approved_spending_limit_micros = new $util.LongBits(d.approved_spending_limit_micros.low >>> 0, d.approved_spending_limit_micros.high >>> 0).toNumber();
                                }
                                switch (d.approved_spending_limit_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approved_spending_limit_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approved_spending_limit_type = 1;
                                    break;
                                case "INFINITE":
                                case 2:
                                    m.approved_spending_limit_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AccountBudgetProposal message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @static
                             * @param {google.ads.googleads.v11.resources.AccountBudgetProposal} m AccountBudgetProposal
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountBudgetProposal.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.proposal_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.proposal_type != null && m.hasOwnProperty("proposal_type")) {
                                    d.proposal_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AccountBudgetProposalTypeEnum.AccountBudgetProposalType[m.proposal_type] : m.proposal_type;
                                }
                                if (m.proposed_start_time_type != null && m.hasOwnProperty("proposed_start_time_type")) {
                                    d.proposed_start_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.proposed_start_time_type] : m.proposed_start_time_type;
                                    if (o.oneofs)
                                        d.proposed_start_time = "proposed_start_time_type";
                                }
                                if (m.proposed_end_time_type != null && m.hasOwnProperty("proposed_end_time_type")) {
                                    d.proposed_end_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.proposed_end_time_type] : m.proposed_end_time_type;
                                    if (o.oneofs)
                                        d.proposed_end_time = "proposed_end_time_type";
                                }
                                if (m.proposed_spending_limit_type != null && m.hasOwnProperty("proposed_spending_limit_type")) {
                                    d.proposed_spending_limit_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType[m.proposed_spending_limit_type] : m.proposed_spending_limit_type;
                                    if (o.oneofs)
                                        d.proposed_spending_limit = "proposed_spending_limit_type";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus[m.status] : m.status;
                                }
                                if (m.approved_end_time_type != null && m.hasOwnProperty("approved_end_time_type")) {
                                    d.approved_end_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.approved_end_time_type] : m.approved_end_time_type;
                                    if (o.oneofs)
                                        d.approved_end_time = "approved_end_time_type";
                                }
                                if (m.approved_spending_limit_type != null && m.hasOwnProperty("approved_spending_limit_type")) {
                                    d.approved_spending_limit_type = o.enums === String ? $root.google.ads.googleads.v11.enums.SpendingLimitTypeEnum.SpendingLimitType[m.approved_spending_limit_type] : m.approved_spending_limit_type;
                                    if (o.oneofs)
                                        d.approved_spending_limit = "approved_spending_limit_type";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    d.billing_setup = m.billing_setup;
                                    if (o.oneofs)
                                        d._billing_setup = "billing_setup";
                                }
                                if (m.account_budget != null && m.hasOwnProperty("account_budget")) {
                                    d.account_budget = m.account_budget;
                                    if (o.oneofs)
                                        d._account_budget = "account_budget";
                                }
                                if (m.proposed_name != null && m.hasOwnProperty("proposed_name")) {
                                    d.proposed_name = m.proposed_name;
                                    if (o.oneofs)
                                        d._proposed_name = "proposed_name";
                                }
                                if (m.proposed_start_date_time != null && m.hasOwnProperty("proposed_start_date_time")) {
                                    d.proposed_start_date_time = m.proposed_start_date_time;
                                    if (o.oneofs)
                                        d.proposed_start_time = "proposed_start_date_time";
                                }
                                if (m.approved_start_date_time != null && m.hasOwnProperty("approved_start_date_time")) {
                                    d.approved_start_date_time = m.approved_start_date_time;
                                    if (o.oneofs)
                                        d._approved_start_date_time = "approved_start_date_time";
                                }
                                if (m.proposed_end_date_time != null && m.hasOwnProperty("proposed_end_date_time")) {
                                    d.proposed_end_date_time = m.proposed_end_date_time;
                                    if (o.oneofs)
                                        d.proposed_end_time = "proposed_end_date_time";
                                }
                                if (m.approved_end_date_time != null && m.hasOwnProperty("approved_end_date_time")) {
                                    d.approved_end_date_time = m.approved_end_date_time;
                                    if (o.oneofs)
                                        d.approved_end_time = "approved_end_date_time";
                                }
                                if (m.proposed_spending_limit_micros != null && m.hasOwnProperty("proposed_spending_limit_micros")) {
                                    if (typeof m.proposed_spending_limit_micros === "number")
                                        d.proposed_spending_limit_micros = o.longs === String ? String(m.proposed_spending_limit_micros) : m.proposed_spending_limit_micros;
                                    else
                                        d.proposed_spending_limit_micros = o.longs === String ? $util.Long.prototype.toString.call(m.proposed_spending_limit_micros) : o.longs === Number ? new $util.LongBits(m.proposed_spending_limit_micros.low >>> 0, m.proposed_spending_limit_micros.high >>> 0).toNumber() : m.proposed_spending_limit_micros;
                                    if (o.oneofs)
                                        d.proposed_spending_limit = "proposed_spending_limit_micros";
                                }
                                if (m.approved_spending_limit_micros != null && m.hasOwnProperty("approved_spending_limit_micros")) {
                                    if (typeof m.approved_spending_limit_micros === "number")
                                        d.approved_spending_limit_micros = o.longs === String ? String(m.approved_spending_limit_micros) : m.approved_spending_limit_micros;
                                    else
                                        d.approved_spending_limit_micros = o.longs === String ? $util.Long.prototype.toString.call(m.approved_spending_limit_micros) : o.longs === Number ? new $util.LongBits(m.approved_spending_limit_micros.low >>> 0, m.approved_spending_limit_micros.high >>> 0).toNumber() : m.approved_spending_limit_micros;
                                    if (o.oneofs)
                                        d.approved_spending_limit = "approved_spending_limit_micros";
                                }
                                if (m.proposed_purchase_order_number != null && m.hasOwnProperty("proposed_purchase_order_number")) {
                                    d.proposed_purchase_order_number = m.proposed_purchase_order_number;
                                    if (o.oneofs)
                                        d._proposed_purchase_order_number = "proposed_purchase_order_number";
                                }
                                if (m.proposed_notes != null && m.hasOwnProperty("proposed_notes")) {
                                    d.proposed_notes = m.proposed_notes;
                                    if (o.oneofs)
                                        d._proposed_notes = "proposed_notes";
                                }
                                if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                    d.creation_date_time = m.creation_date_time;
                                    if (o.oneofs)
                                        d._creation_date_time = "creation_date_time";
                                }
                                if (m.approval_date_time != null && m.hasOwnProperty("approval_date_time")) {
                                    d.approval_date_time = m.approval_date_time;
                                    if (o.oneofs)
                                        d._approval_date_time = "approval_date_time";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AccountBudgetProposal to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AccountBudgetProposal
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountBudgetProposal.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AccountBudgetProposal;
                        })();
    
                        resources.AccountLink = (function() {
    
                            /**
                             * Properties of an AccountLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAccountLink
                             * @property {string|null} [resource_name] AccountLink resource_name
                             * @property {number|null} [account_link_id] AccountLink account_link_id
                             * @property {google.ads.googleads.v11.enums.AccountLinkStatusEnum.AccountLinkStatus|null} [status] AccountLink status
                             * @property {google.ads.googleads.v11.enums.LinkedAccountTypeEnum.LinkedAccountType|null} [type] AccountLink type
                             * @property {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLinkIdentifier|null} [third_party_app_analytics] AccountLink third_party_app_analytics
                             * @property {google.ads.googleads.v11.resources.IDataPartnerLinkIdentifier|null} [data_partner] AccountLink data_partner
                             * @property {google.ads.googleads.v11.resources.IGoogleAdsLinkIdentifier|null} [google_ads] AccountLink google_ads
                             * @property {google.ads.googleads.v11.resources.IHotelCenterLinkIdentifier|null} [hotel_center] AccountLink hotel_center
                             */
    
                            /**
                             * Constructs a new AccountLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AccountLink.
                             * @implements IAccountLink
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAccountLink=} [p] Properties to set
                             */
                            function AccountLink(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AccountLink resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.resource_name = "";
    
                            /**
                             * AccountLink account_link_id.
                             * @member {number|null|undefined} account_link_id
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.account_link_id = null;
    
                            /**
                             * AccountLink status.
                             * @member {google.ads.googleads.v11.enums.AccountLinkStatusEnum.AccountLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.status = 0;
    
                            /**
                             * AccountLink type.
                             * @member {google.ads.googleads.v11.enums.LinkedAccountTypeEnum.LinkedAccountType} type
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.type = 0;
    
                            /**
                             * AccountLink third_party_app_analytics.
                             * @member {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLinkIdentifier|null|undefined} third_party_app_analytics
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.third_party_app_analytics = null;
    
                            /**
                             * AccountLink data_partner.
                             * @member {google.ads.googleads.v11.resources.IDataPartnerLinkIdentifier|null|undefined} data_partner
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.data_partner = null;
    
                            /**
                             * AccountLink google_ads.
                             * @member {google.ads.googleads.v11.resources.IGoogleAdsLinkIdentifier|null|undefined} google_ads
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.google_ads = null;
    
                            /**
                             * AccountLink hotel_center.
                             * @member {google.ads.googleads.v11.resources.IHotelCenterLinkIdentifier|null|undefined} hotel_center
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            AccountLink.prototype.hotel_center = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AccountLink _account_link_id.
                             * @member {"account_link_id"|undefined} _account_link_id
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            Object.defineProperty(AccountLink.prototype, "_account_link_id", {
                                get: $util.oneOfGetter($oneOfFields = ["account_link_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AccountLink linked_account.
                             * @member {"third_party_app_analytics"|"data_partner"|"google_ads"|"hotel_center"|undefined} linked_account
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             */
                            Object.defineProperty(AccountLink.prototype, "linked_account", {
                                get: $util.oneOfGetter($oneOfFields = ["third_party_app_analytics", "data_partner", "google_ads", "hotel_center"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AccountLink instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountLink=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AccountLink} AccountLink instance
                             */
                            AccountLink.create = function create(properties) {
                                return new AccountLink(properties);
                            };
    
                            /**
                             * Encodes the specified AccountLink message. Does not implicitly {@link google.ads.googleads.v11.resources.AccountLink.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountLink} m AccountLink message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLink.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.third_party_app_analytics != null && Object.hasOwnProperty.call(m, "third_party_app_analytics"))
                                    $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier.encode(m.third_party_app_analytics, w.uint32(42).fork()).ldelim();
                                if (m.data_partner != null && Object.hasOwnProperty.call(m, "data_partner"))
                                    $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier.encode(m.data_partner, w.uint32(50).fork()).ldelim();
                                if (m.google_ads != null && Object.hasOwnProperty.call(m, "google_ads"))
                                    $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier.encode(m.google_ads, w.uint32(58).fork()).ldelim();
                                if (m.account_link_id != null && Object.hasOwnProperty.call(m, "account_link_id"))
                                    w.uint32(64).int64(m.account_link_id);
                                if (m.hotel_center != null && Object.hasOwnProperty.call(m, "hotel_center"))
                                    $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier.encode(m.hotel_center, w.uint32(74).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountLink message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AccountLink.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAccountLink} message AccountLink message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLink.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountLink message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AccountLink} AccountLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLink.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AccountLink();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 8:
                                        m.account_link_id = r.int64();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 5:
                                        m.third_party_app_analytics = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.data_partner = $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.google_ads = $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.hotel_center = $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountLink message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AccountLink} AccountLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLink.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountLink message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountLink.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.account_link_id != null && m.hasOwnProperty("account_link_id")) {
                                    p._account_link_id = 1;
                                    if (!$util.isInteger(m.account_link_id) && !(m.account_link_id && $util.isInteger(m.account_link_id.low) && $util.isInteger(m.account_link_id.high)))
                                        return "account_link_id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.third_party_app_analytics != null && m.hasOwnProperty("third_party_app_analytics")) {
                                    p.linked_account = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier.verify(m.third_party_app_analytics);
                                        if (e)
                                            return "third_party_app_analytics." + e;
                                    }
                                }
                                if (m.data_partner != null && m.hasOwnProperty("data_partner")) {
                                    if (p.linked_account === 1)
                                        return "linked_account: multiple values";
                                    p.linked_account = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier.verify(m.data_partner);
                                        if (e)
                                            return "data_partner." + e;
                                    }
                                }
                                if (m.google_ads != null && m.hasOwnProperty("google_ads")) {
                                    if (p.linked_account === 1)
                                        return "linked_account: multiple values";
                                    p.linked_account = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier.verify(m.google_ads);
                                        if (e)
                                            return "google_ads." + e;
                                    }
                                }
                                if (m.hotel_center != null && m.hasOwnProperty("hotel_center")) {
                                    if (p.linked_account === 1)
                                        return "linked_account: multiple values";
                                    p.linked_account = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier.verify(m.hotel_center);
                                        if (e)
                                            return "hotel_center." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AccountLink message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AccountLink} AccountLink
                             */
                            AccountLink.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AccountLink)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AccountLink();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.account_link_id != null) {
                                    if ($util.Long)
                                        (m.account_link_id = $util.Long.fromValue(d.account_link_id)).unsigned = false;
                                    else if (typeof d.account_link_id === "string")
                                        m.account_link_id = parseInt(d.account_link_id, 10);
                                    else if (typeof d.account_link_id === "number")
                                        m.account_link_id = d.account_link_id;
                                    else if (typeof d.account_link_id === "object")
                                        m.account_link_id = new $util.LongBits(d.account_link_id.low >>> 0, d.account_link_id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "REQUESTED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "PENDING_APPROVAL":
                                case 5:
                                    m.status = 5;
                                    break;
                                case "REJECTED":
                                case 6:
                                    m.status = 6;
                                    break;
                                case "REVOKED":
                                case 7:
                                    m.status = 7;
                                    break;
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "DATA_PARTNER":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "GOOGLE_ADS":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "HOTEL_CENTER":
                                case 5:
                                    m.type = 5;
                                    break;
                                }
                                if (d.third_party_app_analytics != null) {
                                    if (typeof d.third_party_app_analytics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccountLink.third_party_app_analytics: object expected");
                                    m.third_party_app_analytics = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier.fromObject(d.third_party_app_analytics);
                                }
                                if (d.data_partner != null) {
                                    if (typeof d.data_partner !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccountLink.data_partner: object expected");
                                    m.data_partner = $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier.fromObject(d.data_partner);
                                }
                                if (d.google_ads != null) {
                                    if (typeof d.google_ads !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccountLink.google_ads: object expected");
                                    m.google_ads = $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier.fromObject(d.google_ads);
                                }
                                if (d.hotel_center != null) {
                                    if (typeof d.hotel_center !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AccountLink.hotel_center: object expected");
                                    m.hotel_center = $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier.fromObject(d.hotel_center);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AccountLink message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.AccountLink} m AccountLink
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountLink.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AccountLinkStatusEnum.AccountLinkStatus[m.status] : m.status;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.LinkedAccountTypeEnum.LinkedAccountType[m.type] : m.type;
                                }
                                if (m.third_party_app_analytics != null && m.hasOwnProperty("third_party_app_analytics")) {
                                    d.third_party_app_analytics = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier.toObject(m.third_party_app_analytics, o);
                                    if (o.oneofs)
                                        d.linked_account = "third_party_app_analytics";
                                }
                                if (m.data_partner != null && m.hasOwnProperty("data_partner")) {
                                    d.data_partner = $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier.toObject(m.data_partner, o);
                                    if (o.oneofs)
                                        d.linked_account = "data_partner";
                                }
                                if (m.google_ads != null && m.hasOwnProperty("google_ads")) {
                                    d.google_ads = $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier.toObject(m.google_ads, o);
                                    if (o.oneofs)
                                        d.linked_account = "google_ads";
                                }
                                if (m.account_link_id != null && m.hasOwnProperty("account_link_id")) {
                                    if (typeof m.account_link_id === "number")
                                        d.account_link_id = o.longs === String ? String(m.account_link_id) : m.account_link_id;
                                    else
                                        d.account_link_id = o.longs === String ? $util.Long.prototype.toString.call(m.account_link_id) : o.longs === Number ? new $util.LongBits(m.account_link_id.low >>> 0, m.account_link_id.high >>> 0).toNumber() : m.account_link_id;
                                    if (o.oneofs)
                                        d._account_link_id = "account_link_id";
                                }
                                if (m.hotel_center != null && m.hasOwnProperty("hotel_center")) {
                                    d.hotel_center = $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier.toObject(m.hotel_center, o);
                                    if (o.oneofs)
                                        d.linked_account = "hotel_center";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AccountLink to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AccountLink
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountLink.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AccountLink;
                        })();
    
                        resources.ThirdPartyAppAnalyticsLinkIdentifier = (function() {
    
                            /**
                             * Properties of a ThirdPartyAppAnalyticsLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IThirdPartyAppAnalyticsLinkIdentifier
                             * @property {number|null} [app_analytics_provider_id] ThirdPartyAppAnalyticsLinkIdentifier app_analytics_provider_id
                             * @property {string|null} [app_id] ThirdPartyAppAnalyticsLinkIdentifier app_id
                             * @property {google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor|null} [app_vendor] ThirdPartyAppAnalyticsLinkIdentifier app_vendor
                             */
    
                            /**
                             * Constructs a new ThirdPartyAppAnalyticsLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ThirdPartyAppAnalyticsLinkIdentifier.
                             * @implements IThirdPartyAppAnalyticsLinkIdentifier
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLinkIdentifier=} [p] Properties to set
                             */
                            function ThirdPartyAppAnalyticsLinkIdentifier(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ThirdPartyAppAnalyticsLinkIdentifier app_analytics_provider_id.
                             * @member {number|null|undefined} app_analytics_provider_id
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @instance
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.prototype.app_analytics_provider_id = null;
    
                            /**
                             * ThirdPartyAppAnalyticsLinkIdentifier app_id.
                             * @member {string|null|undefined} app_id
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @instance
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.prototype.app_id = null;
    
                            /**
                             * ThirdPartyAppAnalyticsLinkIdentifier app_vendor.
                             * @member {google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor} app_vendor
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @instance
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.prototype.app_vendor = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ThirdPartyAppAnalyticsLinkIdentifier _app_analytics_provider_id.
                             * @member {"app_analytics_provider_id"|undefined} _app_analytics_provider_id
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @instance
                             */
                            Object.defineProperty(ThirdPartyAppAnalyticsLinkIdentifier.prototype, "_app_analytics_provider_id", {
                                get: $util.oneOfGetter($oneOfFields = ["app_analytics_provider_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ThirdPartyAppAnalyticsLinkIdentifier _app_id.
                             * @member {"app_id"|undefined} _app_id
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @instance
                             */
                            Object.defineProperty(ThirdPartyAppAnalyticsLinkIdentifier.prototype, "_app_id", {
                                get: $util.oneOfGetter($oneOfFields = ["app_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ThirdPartyAppAnalyticsLinkIdentifier instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLinkIdentifier=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier} ThirdPartyAppAnalyticsLinkIdentifier instance
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.create = function create(properties) {
                                return new ThirdPartyAppAnalyticsLinkIdentifier(properties);
                            };
    
                            /**
                             * Encodes the specified ThirdPartyAppAnalyticsLinkIdentifier message. Does not implicitly {@link google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLinkIdentifier} m ThirdPartyAppAnalyticsLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.app_vendor != null && Object.hasOwnProperty.call(m, "app_vendor"))
                                    w.uint32(24).int32(m.app_vendor);
                                if (m.app_analytics_provider_id != null && Object.hasOwnProperty.call(m, "app_analytics_provider_id"))
                                    w.uint32(32).int64(m.app_analytics_provider_id);
                                if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                    w.uint32(42).string(m.app_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ThirdPartyAppAnalyticsLinkIdentifier message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLinkIdentifier} message ThirdPartyAppAnalyticsLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ThirdPartyAppAnalyticsLinkIdentifier message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier} ThirdPartyAppAnalyticsLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.app_analytics_provider_id = r.int64();
                                        break;
                                    case 5:
                                        m.app_id = r.string();
                                        break;
                                    case 3:
                                        m.app_vendor = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ThirdPartyAppAnalyticsLinkIdentifier message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier} ThirdPartyAppAnalyticsLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ThirdPartyAppAnalyticsLinkIdentifier message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.app_analytics_provider_id != null && m.hasOwnProperty("app_analytics_provider_id")) {
                                    p._app_analytics_provider_id = 1;
                                    if (!$util.isInteger(m.app_analytics_provider_id) && !(m.app_analytics_provider_id && $util.isInteger(m.app_analytics_provider_id.low) && $util.isInteger(m.app_analytics_provider_id.high)))
                                        return "app_analytics_provider_id: integer|Long expected";
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    p._app_id = 1;
                                    if (!$util.isString(m.app_id))
                                        return "app_id: string expected";
                                }
                                if (m.app_vendor != null && m.hasOwnProperty("app_vendor")) {
                                    switch (m.app_vendor) {
                                    default:
                                        return "app_vendor: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ThirdPartyAppAnalyticsLinkIdentifier message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier} ThirdPartyAppAnalyticsLinkIdentifier
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier();
                                if (d.app_analytics_provider_id != null) {
                                    if ($util.Long)
                                        (m.app_analytics_provider_id = $util.Long.fromValue(d.app_analytics_provider_id)).unsigned = false;
                                    else if (typeof d.app_analytics_provider_id === "string")
                                        m.app_analytics_provider_id = parseInt(d.app_analytics_provider_id, 10);
                                    else if (typeof d.app_analytics_provider_id === "number")
                                        m.app_analytics_provider_id = d.app_analytics_provider_id;
                                    else if (typeof d.app_analytics_provider_id === "object")
                                        m.app_analytics_provider_id = new $util.LongBits(d.app_analytics_provider_id.low >>> 0, d.app_analytics_provider_id.high >>> 0).toNumber();
                                }
                                if (d.app_id != null) {
                                    m.app_id = String(d.app_id);
                                }
                                switch (d.app_vendor) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.app_vendor = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.app_vendor = 1;
                                    break;
                                case "APPLE_APP_STORE":
                                case 2:
                                    m.app_vendor = 2;
                                    break;
                                case "GOOGLE_APP_STORE":
                                case 3:
                                    m.app_vendor = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ThirdPartyAppAnalyticsLinkIdentifier message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier} m ThirdPartyAppAnalyticsLinkIdentifier
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.app_vendor = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.app_vendor != null && m.hasOwnProperty("app_vendor")) {
                                    d.app_vendor = o.enums === String ? $root.google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor[m.app_vendor] : m.app_vendor;
                                }
                                if (m.app_analytics_provider_id != null && m.hasOwnProperty("app_analytics_provider_id")) {
                                    if (typeof m.app_analytics_provider_id === "number")
                                        d.app_analytics_provider_id = o.longs === String ? String(m.app_analytics_provider_id) : m.app_analytics_provider_id;
                                    else
                                        d.app_analytics_provider_id = o.longs === String ? $util.Long.prototype.toString.call(m.app_analytics_provider_id) : o.longs === Number ? new $util.LongBits(m.app_analytics_provider_id.low >>> 0, m.app_analytics_provider_id.high >>> 0).toNumber() : m.app_analytics_provider_id;
                                    if (o.oneofs)
                                        d._app_analytics_provider_id = "app_analytics_provider_id";
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    d.app_id = m.app_id;
                                    if (o.oneofs)
                                        d._app_id = "app_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ThirdPartyAppAnalyticsLinkIdentifier to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLinkIdentifier
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ThirdPartyAppAnalyticsLinkIdentifier.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ThirdPartyAppAnalyticsLinkIdentifier;
                        })();
    
                        resources.DataPartnerLinkIdentifier = (function() {
    
                            /**
                             * Properties of a DataPartnerLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IDataPartnerLinkIdentifier
                             * @property {number|null} [data_partner_id] DataPartnerLinkIdentifier data_partner_id
                             */
    
                            /**
                             * Constructs a new DataPartnerLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a DataPartnerLinkIdentifier.
                             * @implements IDataPartnerLinkIdentifier
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IDataPartnerLinkIdentifier=} [p] Properties to set
                             */
                            function DataPartnerLinkIdentifier(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DataPartnerLinkIdentifier data_partner_id.
                             * @member {number|null|undefined} data_partner_id
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @instance
                             */
                            DataPartnerLinkIdentifier.prototype.data_partner_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DataPartnerLinkIdentifier _data_partner_id.
                             * @member {"data_partner_id"|undefined} _data_partner_id
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @instance
                             */
                            Object.defineProperty(DataPartnerLinkIdentifier.prototype, "_data_partner_id", {
                                get: $util.oneOfGetter($oneOfFields = ["data_partner_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DataPartnerLinkIdentifier instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDataPartnerLinkIdentifier=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.DataPartnerLinkIdentifier} DataPartnerLinkIdentifier instance
                             */
                            DataPartnerLinkIdentifier.create = function create(properties) {
                                return new DataPartnerLinkIdentifier(properties);
                            };
    
                            /**
                             * Encodes the specified DataPartnerLinkIdentifier message. Does not implicitly {@link google.ads.googleads.v11.resources.DataPartnerLinkIdentifier.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDataPartnerLinkIdentifier} m DataPartnerLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DataPartnerLinkIdentifier.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.data_partner_id != null && Object.hasOwnProperty.call(m, "data_partner_id"))
                                    w.uint32(8).int64(m.data_partner_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DataPartnerLinkIdentifier message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.DataPartnerLinkIdentifier.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDataPartnerLinkIdentifier} message DataPartnerLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DataPartnerLinkIdentifier.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DataPartnerLinkIdentifier message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.DataPartnerLinkIdentifier} DataPartnerLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DataPartnerLinkIdentifier.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.data_partner_id = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DataPartnerLinkIdentifier message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.DataPartnerLinkIdentifier} DataPartnerLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DataPartnerLinkIdentifier.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DataPartnerLinkIdentifier message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DataPartnerLinkIdentifier.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.data_partner_id != null && m.hasOwnProperty("data_partner_id")) {
                                    p._data_partner_id = 1;
                                    if (!$util.isInteger(m.data_partner_id) && !(m.data_partner_id && $util.isInteger(m.data_partner_id.low) && $util.isInteger(m.data_partner_id.high)))
                                        return "data_partner_id: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DataPartnerLinkIdentifier message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.DataPartnerLinkIdentifier} DataPartnerLinkIdentifier
                             */
                            DataPartnerLinkIdentifier.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.DataPartnerLinkIdentifier();
                                if (d.data_partner_id != null) {
                                    if ($util.Long)
                                        (m.data_partner_id = $util.Long.fromValue(d.data_partner_id)).unsigned = false;
                                    else if (typeof d.data_partner_id === "string")
                                        m.data_partner_id = parseInt(d.data_partner_id, 10);
                                    else if (typeof d.data_partner_id === "number")
                                        m.data_partner_id = d.data_partner_id;
                                    else if (typeof d.data_partner_id === "object")
                                        m.data_partner_id = new $util.LongBits(d.data_partner_id.low >>> 0, d.data_partner_id.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DataPartnerLinkIdentifier message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.DataPartnerLinkIdentifier} m DataPartnerLinkIdentifier
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DataPartnerLinkIdentifier.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.data_partner_id != null && m.hasOwnProperty("data_partner_id")) {
                                    if (typeof m.data_partner_id === "number")
                                        d.data_partner_id = o.longs === String ? String(m.data_partner_id) : m.data_partner_id;
                                    else
                                        d.data_partner_id = o.longs === String ? $util.Long.prototype.toString.call(m.data_partner_id) : o.longs === Number ? new $util.LongBits(m.data_partner_id.low >>> 0, m.data_partner_id.high >>> 0).toNumber() : m.data_partner_id;
                                    if (o.oneofs)
                                        d._data_partner_id = "data_partner_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DataPartnerLinkIdentifier to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.DataPartnerLinkIdentifier
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DataPartnerLinkIdentifier.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DataPartnerLinkIdentifier;
                        })();
    
                        resources.HotelCenterLinkIdentifier = (function() {
    
                            /**
                             * Properties of a HotelCenterLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IHotelCenterLinkIdentifier
                             * @property {number|null} [hotel_center_id] HotelCenterLinkIdentifier hotel_center_id
                             */
    
                            /**
                             * Constructs a new HotelCenterLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a HotelCenterLinkIdentifier.
                             * @implements IHotelCenterLinkIdentifier
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IHotelCenterLinkIdentifier=} [p] Properties to set
                             */
                            function HotelCenterLinkIdentifier(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelCenterLinkIdentifier hotel_center_id.
                             * @member {number} hotel_center_id
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @instance
                             */
                            HotelCenterLinkIdentifier.prototype.hotel_center_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new HotelCenterLinkIdentifier instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelCenterLinkIdentifier=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.HotelCenterLinkIdentifier} HotelCenterLinkIdentifier instance
                             */
                            HotelCenterLinkIdentifier.create = function create(properties) {
                                return new HotelCenterLinkIdentifier(properties);
                            };
    
                            /**
                             * Encodes the specified HotelCenterLinkIdentifier message. Does not implicitly {@link google.ads.googleads.v11.resources.HotelCenterLinkIdentifier.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelCenterLinkIdentifier} m HotelCenterLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCenterLinkIdentifier.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.hotel_center_id != null && Object.hasOwnProperty.call(m, "hotel_center_id"))
                                    w.uint32(8).int64(m.hotel_center_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelCenterLinkIdentifier message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.HotelCenterLinkIdentifier.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelCenterLinkIdentifier} message HotelCenterLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelCenterLinkIdentifier.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelCenterLinkIdentifier message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.HotelCenterLinkIdentifier} HotelCenterLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCenterLinkIdentifier.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.hotel_center_id = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelCenterLinkIdentifier message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.HotelCenterLinkIdentifier} HotelCenterLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelCenterLinkIdentifier.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelCenterLinkIdentifier message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelCenterLinkIdentifier.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                    if (!$util.isInteger(m.hotel_center_id) && !(m.hotel_center_id && $util.isInteger(m.hotel_center_id.low) && $util.isInteger(m.hotel_center_id.high)))
                                        return "hotel_center_id: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelCenterLinkIdentifier message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.HotelCenterLinkIdentifier} HotelCenterLinkIdentifier
                             */
                            HotelCenterLinkIdentifier.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.HotelCenterLinkIdentifier();
                                if (d.hotel_center_id != null) {
                                    if ($util.Long)
                                        (m.hotel_center_id = $util.Long.fromValue(d.hotel_center_id)).unsigned = false;
                                    else if (typeof d.hotel_center_id === "string")
                                        m.hotel_center_id = parseInt(d.hotel_center_id, 10);
                                    else if (typeof d.hotel_center_id === "number")
                                        m.hotel_center_id = d.hotel_center_id;
                                    else if (typeof d.hotel_center_id === "object")
                                        m.hotel_center_id = new $util.LongBits(d.hotel_center_id.low >>> 0, d.hotel_center_id.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelCenterLinkIdentifier message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.HotelCenterLinkIdentifier} m HotelCenterLinkIdentifier
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelCenterLinkIdentifier.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.hotel_center_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.hotel_center_id = o.longs === String ? "0" : 0;
                                }
                                if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                    if (typeof m.hotel_center_id === "number")
                                        d.hotel_center_id = o.longs === String ? String(m.hotel_center_id) : m.hotel_center_id;
                                    else
                                        d.hotel_center_id = o.longs === String ? $util.Long.prototype.toString.call(m.hotel_center_id) : o.longs === Number ? new $util.LongBits(m.hotel_center_id.low >>> 0, m.hotel_center_id.high >>> 0).toNumber() : m.hotel_center_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelCenterLinkIdentifier to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.HotelCenterLinkIdentifier
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelCenterLinkIdentifier.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelCenterLinkIdentifier;
                        })();
    
                        resources.GoogleAdsLinkIdentifier = (function() {
    
                            /**
                             * Properties of a GoogleAdsLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IGoogleAdsLinkIdentifier
                             * @property {string|null} [customer] GoogleAdsLinkIdentifier customer
                             */
    
                            /**
                             * Constructs a new GoogleAdsLinkIdentifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a GoogleAdsLinkIdentifier.
                             * @implements IGoogleAdsLinkIdentifier
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsLinkIdentifier=} [p] Properties to set
                             */
                            function GoogleAdsLinkIdentifier(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GoogleAdsLinkIdentifier customer.
                             * @member {string|null|undefined} customer
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @instance
                             */
                            GoogleAdsLinkIdentifier.prototype.customer = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GoogleAdsLinkIdentifier _customer.
                             * @member {"customer"|undefined} _customer
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @instance
                             */
                            Object.defineProperty(GoogleAdsLinkIdentifier.prototype, "_customer", {
                                get: $util.oneOfGetter($oneOfFields = ["customer"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GoogleAdsLinkIdentifier instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsLinkIdentifier=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier} GoogleAdsLinkIdentifier instance
                             */
                            GoogleAdsLinkIdentifier.create = function create(properties) {
                                return new GoogleAdsLinkIdentifier(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleAdsLinkIdentifier message. Does not implicitly {@link google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsLinkIdentifier} m GoogleAdsLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsLinkIdentifier.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer != null && Object.hasOwnProperty.call(m, "customer"))
                                    w.uint32(26).string(m.customer);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleAdsLinkIdentifier message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsLinkIdentifier} message GoogleAdsLinkIdentifier message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsLinkIdentifier.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleAdsLinkIdentifier message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier} GoogleAdsLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsLinkIdentifier.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.customer = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleAdsLinkIdentifier message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier} GoogleAdsLinkIdentifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsLinkIdentifier.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleAdsLinkIdentifier message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleAdsLinkIdentifier.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    p._customer = 1;
                                    if (!$util.isString(m.customer))
                                        return "customer: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GoogleAdsLinkIdentifier message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier} GoogleAdsLinkIdentifier
                             */
                            GoogleAdsLinkIdentifier.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier();
                                if (d.customer != null) {
                                    m.customer = String(d.customer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GoogleAdsLinkIdentifier message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier} m GoogleAdsLinkIdentifier
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleAdsLinkIdentifier.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    d.customer = m.customer;
                                    if (o.oneofs)
                                        d._customer = "customer";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GoogleAdsLinkIdentifier to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsLinkIdentifier
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleAdsLinkIdentifier.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GoogleAdsLinkIdentifier;
                        })();
    
                        resources.Ad = (function() {
    
                            /**
                             * Properties of an Ad.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAd
                             * @property {string|null} [resource_name] Ad resource_name
                             * @property {number|null} [id] Ad id
                             * @property {Array.<string>|null} [final_urls] Ad final_urls
                             * @property {Array.<google.ads.googleads.v11.common.IFinalAppUrl>|null} [final_app_urls] Ad final_app_urls
                             * @property {Array.<string>|null} [final_mobile_urls] Ad final_mobile_urls
                             * @property {string|null} [tracking_url_template] Ad tracking_url_template
                             * @property {string|null} [final_url_suffix] Ad final_url_suffix
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] Ad url_custom_parameters
                             * @property {string|null} [display_url] Ad display_url
                             * @property {google.ads.googleads.v11.enums.AdTypeEnum.AdType|null} [type] Ad type
                             * @property {boolean|null} [added_by_google_ads] Ad added_by_google_ads
                             * @property {google.ads.googleads.v11.enums.DeviceEnum.Device|null} [device_preference] Ad device_preference
                             * @property {Array.<google.ads.googleads.v11.common.IUrlCollection>|null} [url_collections] Ad url_collections
                             * @property {string|null} [name] Ad name
                             * @property {google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum.SystemManagedResourceSource|null} [system_managed_resource_source] Ad system_managed_resource_source
                             * @property {google.ads.googleads.v11.common.ITextAdInfo|null} [text_ad] Ad text_ad
                             * @property {google.ads.googleads.v11.common.IExpandedTextAdInfo|null} [expanded_text_ad] Ad expanded_text_ad
                             * @property {google.ads.googleads.v11.common.ICallAdInfo|null} [call_ad] Ad call_ad
                             * @property {google.ads.googleads.v11.common.IExpandedDynamicSearchAdInfo|null} [expanded_dynamic_search_ad] Ad expanded_dynamic_search_ad
                             * @property {google.ads.googleads.v11.common.IHotelAdInfo|null} [hotel_ad] Ad hotel_ad
                             * @property {google.ads.googleads.v11.common.IShoppingSmartAdInfo|null} [shopping_smart_ad] Ad shopping_smart_ad
                             * @property {google.ads.googleads.v11.common.IShoppingProductAdInfo|null} [shopping_product_ad] Ad shopping_product_ad
                             * @property {google.ads.googleads.v11.common.IGmailAdInfo|null} [gmail_ad] Ad gmail_ad
                             * @property {google.ads.googleads.v11.common.IImageAdInfo|null} [image_ad] Ad image_ad
                             * @property {google.ads.googleads.v11.common.IVideoAdInfo|null} [video_ad] Ad video_ad
                             * @property {google.ads.googleads.v11.common.IVideoResponsiveAdInfo|null} [video_responsive_ad] Ad video_responsive_ad
                             * @property {google.ads.googleads.v11.common.IResponsiveSearchAdInfo|null} [responsive_search_ad] Ad responsive_search_ad
                             * @property {google.ads.googleads.v11.common.ILegacyResponsiveDisplayAdInfo|null} [legacy_responsive_display_ad] Ad legacy_responsive_display_ad
                             * @property {google.ads.googleads.v11.common.IAppAdInfo|null} [app_ad] Ad app_ad
                             * @property {google.ads.googleads.v11.common.ILegacyAppInstallAdInfo|null} [legacy_app_install_ad] Ad legacy_app_install_ad
                             * @property {google.ads.googleads.v11.common.IResponsiveDisplayAdInfo|null} [responsive_display_ad] Ad responsive_display_ad
                             * @property {google.ads.googleads.v11.common.ILocalAdInfo|null} [local_ad] Ad local_ad
                             * @property {google.ads.googleads.v11.common.IDisplayUploadAdInfo|null} [display_upload_ad] Ad display_upload_ad
                             * @property {google.ads.googleads.v11.common.IAppEngagementAdInfo|null} [app_engagement_ad] Ad app_engagement_ad
                             * @property {google.ads.googleads.v11.common.IShoppingComparisonListingAdInfo|null} [shopping_comparison_listing_ad] Ad shopping_comparison_listing_ad
                             * @property {google.ads.googleads.v11.common.ISmartCampaignAdInfo|null} [smart_campaign_ad] Ad smart_campaign_ad
                             * @property {google.ads.googleads.v11.common.IAppPreRegistrationAdInfo|null} [app_pre_registration_ad] Ad app_pre_registration_ad
                             * @property {google.ads.googleads.v11.common.IDiscoveryMultiAssetAdInfo|null} [discovery_multi_asset_ad] Ad discovery_multi_asset_ad
                             * @property {google.ads.googleads.v11.common.IDiscoveryCarouselAdInfo|null} [discovery_carousel_ad] Ad discovery_carousel_ad
                             */
    
                            /**
                             * Constructs a new Ad.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an Ad.
                             * @implements IAd
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAd=} [p] Properties to set
                             */
                            function Ad(p) {
                                this.final_urls = [];
                                this.final_app_urls = [];
                                this.final_mobile_urls = [];
                                this.url_custom_parameters = [];
                                this.url_collections = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Ad resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.resource_name = "";
    
                            /**
                             * Ad id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.id = null;
    
                            /**
                             * Ad final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * Ad final_app_urls.
                             * @member {Array.<google.ads.googleads.v11.common.IFinalAppUrl>} final_app_urls
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.final_app_urls = $util.emptyArray;
    
                            /**
                             * Ad final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * Ad tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.tracking_url_template = null;
    
                            /**
                             * Ad final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.final_url_suffix = null;
    
                            /**
                             * Ad url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * Ad display_url.
                             * @member {string|null|undefined} display_url
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.display_url = null;
    
                            /**
                             * Ad type.
                             * @member {google.ads.googleads.v11.enums.AdTypeEnum.AdType} type
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.type = 0;
    
                            /**
                             * Ad added_by_google_ads.
                             * @member {boolean|null|undefined} added_by_google_ads
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.added_by_google_ads = null;
    
                            /**
                             * Ad device_preference.
                             * @member {google.ads.googleads.v11.enums.DeviceEnum.Device} device_preference
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.device_preference = 0;
    
                            /**
                             * Ad url_collections.
                             * @member {Array.<google.ads.googleads.v11.common.IUrlCollection>} url_collections
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.url_collections = $util.emptyArray;
    
                            /**
                             * Ad name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.name = null;
    
                            /**
                             * Ad system_managed_resource_source.
                             * @member {google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum.SystemManagedResourceSource} system_managed_resource_source
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.system_managed_resource_source = 0;
    
                            /**
                             * Ad text_ad.
                             * @member {google.ads.googleads.v11.common.ITextAdInfo|null|undefined} text_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.text_ad = null;
    
                            /**
                             * Ad expanded_text_ad.
                             * @member {google.ads.googleads.v11.common.IExpandedTextAdInfo|null|undefined} expanded_text_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.expanded_text_ad = null;
    
                            /**
                             * Ad call_ad.
                             * @member {google.ads.googleads.v11.common.ICallAdInfo|null|undefined} call_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.call_ad = null;
    
                            /**
                             * Ad expanded_dynamic_search_ad.
                             * @member {google.ads.googleads.v11.common.IExpandedDynamicSearchAdInfo|null|undefined} expanded_dynamic_search_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.expanded_dynamic_search_ad = null;
    
                            /**
                             * Ad hotel_ad.
                             * @member {google.ads.googleads.v11.common.IHotelAdInfo|null|undefined} hotel_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.hotel_ad = null;
    
                            /**
                             * Ad shopping_smart_ad.
                             * @member {google.ads.googleads.v11.common.IShoppingSmartAdInfo|null|undefined} shopping_smart_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.shopping_smart_ad = null;
    
                            /**
                             * Ad shopping_product_ad.
                             * @member {google.ads.googleads.v11.common.IShoppingProductAdInfo|null|undefined} shopping_product_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.shopping_product_ad = null;
    
                            /**
                             * Ad gmail_ad.
                             * @member {google.ads.googleads.v11.common.IGmailAdInfo|null|undefined} gmail_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.gmail_ad = null;
    
                            /**
                             * Ad image_ad.
                             * @member {google.ads.googleads.v11.common.IImageAdInfo|null|undefined} image_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.image_ad = null;
    
                            /**
                             * Ad video_ad.
                             * @member {google.ads.googleads.v11.common.IVideoAdInfo|null|undefined} video_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.video_ad = null;
    
                            /**
                             * Ad video_responsive_ad.
                             * @member {google.ads.googleads.v11.common.IVideoResponsiveAdInfo|null|undefined} video_responsive_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.video_responsive_ad = null;
    
                            /**
                             * Ad responsive_search_ad.
                             * @member {google.ads.googleads.v11.common.IResponsiveSearchAdInfo|null|undefined} responsive_search_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.responsive_search_ad = null;
    
                            /**
                             * Ad legacy_responsive_display_ad.
                             * @member {google.ads.googleads.v11.common.ILegacyResponsiveDisplayAdInfo|null|undefined} legacy_responsive_display_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.legacy_responsive_display_ad = null;
    
                            /**
                             * Ad app_ad.
                             * @member {google.ads.googleads.v11.common.IAppAdInfo|null|undefined} app_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.app_ad = null;
    
                            /**
                             * Ad legacy_app_install_ad.
                             * @member {google.ads.googleads.v11.common.ILegacyAppInstallAdInfo|null|undefined} legacy_app_install_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.legacy_app_install_ad = null;
    
                            /**
                             * Ad responsive_display_ad.
                             * @member {google.ads.googleads.v11.common.IResponsiveDisplayAdInfo|null|undefined} responsive_display_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.responsive_display_ad = null;
    
                            /**
                             * Ad local_ad.
                             * @member {google.ads.googleads.v11.common.ILocalAdInfo|null|undefined} local_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.local_ad = null;
    
                            /**
                             * Ad display_upload_ad.
                             * @member {google.ads.googleads.v11.common.IDisplayUploadAdInfo|null|undefined} display_upload_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.display_upload_ad = null;
    
                            /**
                             * Ad app_engagement_ad.
                             * @member {google.ads.googleads.v11.common.IAppEngagementAdInfo|null|undefined} app_engagement_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.app_engagement_ad = null;
    
                            /**
                             * Ad shopping_comparison_listing_ad.
                             * @member {google.ads.googleads.v11.common.IShoppingComparisonListingAdInfo|null|undefined} shopping_comparison_listing_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.shopping_comparison_listing_ad = null;
    
                            /**
                             * Ad smart_campaign_ad.
                             * @member {google.ads.googleads.v11.common.ISmartCampaignAdInfo|null|undefined} smart_campaign_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.smart_campaign_ad = null;
    
                            /**
                             * Ad app_pre_registration_ad.
                             * @member {google.ads.googleads.v11.common.IAppPreRegistrationAdInfo|null|undefined} app_pre_registration_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.app_pre_registration_ad = null;
    
                            /**
                             * Ad discovery_multi_asset_ad.
                             * @member {google.ads.googleads.v11.common.IDiscoveryMultiAssetAdInfo|null|undefined} discovery_multi_asset_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.discovery_multi_asset_ad = null;
    
                            /**
                             * Ad discovery_carousel_ad.
                             * @member {google.ads.googleads.v11.common.IDiscoveryCarouselAdInfo|null|undefined} discovery_carousel_ad
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Ad.prototype.discovery_carousel_ad = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Ad _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Object.defineProperty(Ad.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Ad _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Object.defineProperty(Ad.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Ad _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Object.defineProperty(Ad.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Ad _display_url.
                             * @member {"display_url"|undefined} _display_url
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Object.defineProperty(Ad.prototype, "_display_url", {
                                get: $util.oneOfGetter($oneOfFields = ["display_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Ad _added_by_google_ads.
                             * @member {"added_by_google_ads"|undefined} _added_by_google_ads
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Object.defineProperty(Ad.prototype, "_added_by_google_ads", {
                                get: $util.oneOfGetter($oneOfFields = ["added_by_google_ads"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Ad _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Object.defineProperty(Ad.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Ad ad_data.
                             * @member {"text_ad"|"expanded_text_ad"|"call_ad"|"expanded_dynamic_search_ad"|"hotel_ad"|"shopping_smart_ad"|"shopping_product_ad"|"gmail_ad"|"image_ad"|"video_ad"|"video_responsive_ad"|"responsive_search_ad"|"legacy_responsive_display_ad"|"app_ad"|"legacy_app_install_ad"|"responsive_display_ad"|"local_ad"|"display_upload_ad"|"app_engagement_ad"|"shopping_comparison_listing_ad"|"smart_campaign_ad"|"app_pre_registration_ad"|"discovery_multi_asset_ad"|"discovery_carousel_ad"|undefined} ad_data
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             */
                            Object.defineProperty(Ad.prototype, "ad_data", {
                                get: $util.oneOfGetter($oneOfFields = ["text_ad", "expanded_text_ad", "call_ad", "expanded_dynamic_search_ad", "hotel_ad", "shopping_smart_ad", "shopping_product_ad", "gmail_ad", "image_ad", "video_ad", "video_responsive_ad", "responsive_search_ad", "legacy_responsive_display_ad", "app_ad", "legacy_app_install_ad", "responsive_display_ad", "local_ad", "display_upload_ad", "app_engagement_ad", "shopping_comparison_listing_ad", "smart_campaign_ad", "app_pre_registration_ad", "discovery_multi_asset_ad", "discovery_carousel_ad"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Ad instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAd=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Ad} Ad instance
                             */
                            Ad.create = function create(properties) {
                                return new Ad(properties);
                            };
    
                            /**
                             * Encodes the specified Ad message. Does not implicitly {@link google.ads.googleads.v11.resources.Ad.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAd} m Ad message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Ad.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(40).int32(m.type);
                                if (m.text_ad != null && Object.hasOwnProperty.call(m, "text_ad"))
                                    $root.google.ads.googleads.v11.common.TextAdInfo.encode(m.text_ad, w.uint32(50).fork()).ldelim();
                                if (m.expanded_text_ad != null && Object.hasOwnProperty.call(m, "expanded_text_ad"))
                                    $root.google.ads.googleads.v11.common.ExpandedTextAdInfo.encode(m.expanded_text_ad, w.uint32(58).fork()).ldelim();
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(82).fork()).ldelim();
                                }
                                if (m.expanded_dynamic_search_ad != null && Object.hasOwnProperty.call(m, "expanded_dynamic_search_ad"))
                                    $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo.encode(m.expanded_dynamic_search_ad, w.uint32(114).fork()).ldelim();
                                if (m.hotel_ad != null && Object.hasOwnProperty.call(m, "hotel_ad"))
                                    $root.google.ads.googleads.v11.common.HotelAdInfo.encode(m.hotel_ad, w.uint32(122).fork()).ldelim();
                                if (m.shopping_smart_ad != null && Object.hasOwnProperty.call(m, "shopping_smart_ad"))
                                    $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo.encode(m.shopping_smart_ad, w.uint32(138).fork()).ldelim();
                                if (m.shopping_product_ad != null && Object.hasOwnProperty.call(m, "shopping_product_ad"))
                                    $root.google.ads.googleads.v11.common.ShoppingProductAdInfo.encode(m.shopping_product_ad, w.uint32(146).fork()).ldelim();
                                if (m.device_preference != null && Object.hasOwnProperty.call(m, "device_preference"))
                                    w.uint32(160).int32(m.device_preference);
                                if (m.gmail_ad != null && Object.hasOwnProperty.call(m, "gmail_ad"))
                                    $root.google.ads.googleads.v11.common.GmailAdInfo.encode(m.gmail_ad, w.uint32(170).fork()).ldelim();
                                if (m.image_ad != null && Object.hasOwnProperty.call(m, "image_ad"))
                                    $root.google.ads.googleads.v11.common.ImageAdInfo.encode(m.image_ad, w.uint32(178).fork()).ldelim();
                                if (m.video_ad != null && Object.hasOwnProperty.call(m, "video_ad"))
                                    $root.google.ads.googleads.v11.common.VideoAdInfo.encode(m.video_ad, w.uint32(194).fork()).ldelim();
                                if (m.responsive_search_ad != null && Object.hasOwnProperty.call(m, "responsive_search_ad"))
                                    $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo.encode(m.responsive_search_ad, w.uint32(202).fork()).ldelim();
                                if (m.url_collections != null && m.url_collections.length) {
                                    for (var i = 0; i < m.url_collections.length; ++i)
                                        $root.google.ads.googleads.v11.common.UrlCollection.encode(m.url_collections[i], w.uint32(210).fork()).ldelim();
                                }
                                if (m.system_managed_resource_source != null && Object.hasOwnProperty.call(m, "system_managed_resource_source"))
                                    w.uint32(216).int32(m.system_managed_resource_source);
                                if (m.legacy_responsive_display_ad != null && Object.hasOwnProperty.call(m, "legacy_responsive_display_ad"))
                                    $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo.encode(m.legacy_responsive_display_ad, w.uint32(226).fork()).ldelim();
                                if (m.app_ad != null && Object.hasOwnProperty.call(m, "app_ad"))
                                    $root.google.ads.googleads.v11.common.AppAdInfo.encode(m.app_ad, w.uint32(234).fork()).ldelim();
                                if (m.legacy_app_install_ad != null && Object.hasOwnProperty.call(m, "legacy_app_install_ad"))
                                    $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo.encode(m.legacy_app_install_ad, w.uint32(242).fork()).ldelim();
                                if (m.responsive_display_ad != null && Object.hasOwnProperty.call(m, "responsive_display_ad"))
                                    $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.encode(m.responsive_display_ad, w.uint32(250).fork()).ldelim();
                                if (m.local_ad != null && Object.hasOwnProperty.call(m, "local_ad"))
                                    $root.google.ads.googleads.v11.common.LocalAdInfo.encode(m.local_ad, w.uint32(258).fork()).ldelim();
                                if (m.display_upload_ad != null && Object.hasOwnProperty.call(m, "display_upload_ad"))
                                    $root.google.ads.googleads.v11.common.DisplayUploadAdInfo.encode(m.display_upload_ad, w.uint32(266).fork()).ldelim();
                                if (m.app_engagement_ad != null && Object.hasOwnProperty.call(m, "app_engagement_ad"))
                                    $root.google.ads.googleads.v11.common.AppEngagementAdInfo.encode(m.app_engagement_ad, w.uint32(274).fork()).ldelim();
                                if (m.final_app_urls != null && m.final_app_urls.length) {
                                    for (var i = 0; i < m.final_app_urls.length; ++i)
                                        $root.google.ads.googleads.v11.common.FinalAppUrl.encode(m.final_app_urls[i], w.uint32(282).fork()).ldelim();
                                }
                                if (m.shopping_comparison_listing_ad != null && Object.hasOwnProperty.call(m, "shopping_comparison_listing_ad"))
                                    $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo.encode(m.shopping_comparison_listing_ad, w.uint32(290).fork()).ldelim();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(298).string(m.resource_name);
                                if (m.video_responsive_ad != null && Object.hasOwnProperty.call(m, "video_responsive_ad"))
                                    $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo.encode(m.video_responsive_ad, w.uint32(314).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(320).int64(m.id);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(330).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(338).string(m.final_mobile_urls[i]);
                                }
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(346).string(m.tracking_url_template);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(354).string(m.final_url_suffix);
                                if (m.display_url != null && Object.hasOwnProperty.call(m, "display_url"))
                                    w.uint32(362).string(m.display_url);
                                if (m.added_by_google_ads != null && Object.hasOwnProperty.call(m, "added_by_google_ads"))
                                    w.uint32(368).bool(m.added_by_google_ads);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(378).string(m.name);
                                if (m.smart_campaign_ad != null && Object.hasOwnProperty.call(m, "smart_campaign_ad"))
                                    $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.encode(m.smart_campaign_ad, w.uint32(386).fork()).ldelim();
                                if (m.call_ad != null && Object.hasOwnProperty.call(m, "call_ad"))
                                    $root.google.ads.googleads.v11.common.CallAdInfo.encode(m.call_ad, w.uint32(394).fork()).ldelim();
                                if (m.app_pre_registration_ad != null && Object.hasOwnProperty.call(m, "app_pre_registration_ad"))
                                    $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo.encode(m.app_pre_registration_ad, w.uint32(402).fork()).ldelim();
                                if (m.discovery_multi_asset_ad != null && Object.hasOwnProperty.call(m, "discovery_multi_asset_ad"))
                                    $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.encode(m.discovery_multi_asset_ad, w.uint32(410).fork()).ldelim();
                                if (m.discovery_carousel_ad != null && Object.hasOwnProperty.call(m, "discovery_carousel_ad"))
                                    $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.encode(m.discovery_carousel_ad, w.uint32(418).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified Ad message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Ad.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAd} message Ad message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Ad.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Ad message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Ad} Ad
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Ad.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Ad();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 37:
                                        m.resource_name = r.string();
                                        break;
                                    case 40:
                                        m.id = r.int64();
                                        break;
                                    case 41:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 35:
                                        if (!(m.final_app_urls && m.final_app_urls.length))
                                            m.final_app_urls = [];
                                        m.final_app_urls.push($root.google.ads.googleads.v11.common.FinalAppUrl.decode(r, r.uint32()));
                                        break;
                                    case 42:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 43:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 44:
                                        m.final_url_suffix = r.string();
                                        break;
                                    case 10:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 45:
                                        m.display_url = r.string();
                                        break;
                                    case 5:
                                        m.type = r.int32();
                                        break;
                                    case 46:
                                        m.added_by_google_ads = r.bool();
                                        break;
                                    case 20:
                                        m.device_preference = r.int32();
                                        break;
                                    case 26:
                                        if (!(m.url_collections && m.url_collections.length))
                                            m.url_collections = [];
                                        m.url_collections.push($root.google.ads.googleads.v11.common.UrlCollection.decode(r, r.uint32()));
                                        break;
                                    case 47:
                                        m.name = r.string();
                                        break;
                                    case 27:
                                        m.system_managed_resource_source = r.int32();
                                        break;
                                    case 6:
                                        m.text_ad = $root.google.ads.googleads.v11.common.TextAdInfo.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.expanded_text_ad = $root.google.ads.googleads.v11.common.ExpandedTextAdInfo.decode(r, r.uint32());
                                        break;
                                    case 49:
                                        m.call_ad = $root.google.ads.googleads.v11.common.CallAdInfo.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.expanded_dynamic_search_ad = $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.hotel_ad = $root.google.ads.googleads.v11.common.HotelAdInfo.decode(r, r.uint32());
                                        break;
                                    case 17:
                                        m.shopping_smart_ad = $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.shopping_product_ad = $root.google.ads.googleads.v11.common.ShoppingProductAdInfo.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.gmail_ad = $root.google.ads.googleads.v11.common.GmailAdInfo.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.image_ad = $root.google.ads.googleads.v11.common.ImageAdInfo.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.video_ad = $root.google.ads.googleads.v11.common.VideoAdInfo.decode(r, r.uint32());
                                        break;
                                    case 39:
                                        m.video_responsive_ad = $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo.decode(r, r.uint32());
                                        break;
                                    case 25:
                                        m.responsive_search_ad = $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.legacy_responsive_display_ad = $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo.decode(r, r.uint32());
                                        break;
                                    case 29:
                                        m.app_ad = $root.google.ads.googleads.v11.common.AppAdInfo.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.legacy_app_install_ad = $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.responsive_display_ad = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.local_ad = $root.google.ads.googleads.v11.common.LocalAdInfo.decode(r, r.uint32());
                                        break;
                                    case 33:
                                        m.display_upload_ad = $root.google.ads.googleads.v11.common.DisplayUploadAdInfo.decode(r, r.uint32());
                                        break;
                                    case 34:
                                        m.app_engagement_ad = $root.google.ads.googleads.v11.common.AppEngagementAdInfo.decode(r, r.uint32());
                                        break;
                                    case 36:
                                        m.shopping_comparison_listing_ad = $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo.decode(r, r.uint32());
                                        break;
                                    case 48:
                                        m.smart_campaign_ad = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.decode(r, r.uint32());
                                        break;
                                    case 50:
                                        m.app_pre_registration_ad = $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo.decode(r, r.uint32());
                                        break;
                                    case 51:
                                        m.discovery_multi_asset_ad = $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.decode(r, r.uint32());
                                        break;
                                    case 52:
                                        m.discovery_carousel_ad = $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an Ad message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Ad} Ad
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Ad.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Ad message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Ad.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_app_urls != null && m.hasOwnProperty("final_app_urls")) {
                                    if (!Array.isArray(m.final_app_urls))
                                        return "final_app_urls: array expected";
                                    for (var i = 0; i < m.final_app_urls.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.FinalAppUrl.verify(m.final_app_urls[i]);
                                            if (e)
                                                return "final_app_urls." + e;
                                        }
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.display_url != null && m.hasOwnProperty("display_url")) {
                                    p._display_url = 1;
                                    if (!$util.isString(m.display_url))
                                        return "display_url: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                        break;
                                    }
                                }
                                if (m.added_by_google_ads != null && m.hasOwnProperty("added_by_google_ads")) {
                                    p._added_by_google_ads = 1;
                                    if (typeof m.added_by_google_ads !== "boolean")
                                        return "added_by_google_ads: boolean expected";
                                }
                                if (m.device_preference != null && m.hasOwnProperty("device_preference")) {
                                    switch (m.device_preference) {
                                    default:
                                        return "device_preference: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.url_collections != null && m.hasOwnProperty("url_collections")) {
                                    if (!Array.isArray(m.url_collections))
                                        return "url_collections: array expected";
                                    for (var i = 0; i < m.url_collections.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UrlCollection.verify(m.url_collections[i]);
                                            if (e)
                                                return "url_collections." + e;
                                        }
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.system_managed_resource_source != null && m.hasOwnProperty("system_managed_resource_source")) {
                                    switch (m.system_managed_resource_source) {
                                    default:
                                        return "system_managed_resource_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.text_ad != null && m.hasOwnProperty("text_ad")) {
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TextAdInfo.verify(m.text_ad);
                                        if (e)
                                            return "text_ad." + e;
                                    }
                                }
                                if (m.expanded_text_ad != null && m.hasOwnProperty("expanded_text_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ExpandedTextAdInfo.verify(m.expanded_text_ad);
                                        if (e)
                                            return "expanded_text_ad." + e;
                                    }
                                }
                                if (m.call_ad != null && m.hasOwnProperty("call_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CallAdInfo.verify(m.call_ad);
                                        if (e)
                                            return "call_ad." + e;
                                    }
                                }
                                if (m.expanded_dynamic_search_ad != null && m.hasOwnProperty("expanded_dynamic_search_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo.verify(m.expanded_dynamic_search_ad);
                                        if (e)
                                            return "expanded_dynamic_search_ad." + e;
                                    }
                                }
                                if (m.hotel_ad != null && m.hasOwnProperty("hotel_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelAdInfo.verify(m.hotel_ad);
                                        if (e)
                                            return "hotel_ad." + e;
                                    }
                                }
                                if (m.shopping_smart_ad != null && m.hasOwnProperty("shopping_smart_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo.verify(m.shopping_smart_ad);
                                        if (e)
                                            return "shopping_smart_ad." + e;
                                    }
                                }
                                if (m.shopping_product_ad != null && m.hasOwnProperty("shopping_product_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ShoppingProductAdInfo.verify(m.shopping_product_ad);
                                        if (e)
                                            return "shopping_product_ad." + e;
                                    }
                                }
                                if (m.gmail_ad != null && m.hasOwnProperty("gmail_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GmailAdInfo.verify(m.gmail_ad);
                                        if (e)
                                            return "gmail_ad." + e;
                                    }
                                }
                                if (m.image_ad != null && m.hasOwnProperty("image_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ImageAdInfo.verify(m.image_ad);
                                        if (e)
                                            return "image_ad." + e;
                                    }
                                }
                                if (m.video_ad != null && m.hasOwnProperty("video_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.VideoAdInfo.verify(m.video_ad);
                                        if (e)
                                            return "video_ad." + e;
                                    }
                                }
                                if (m.video_responsive_ad != null && m.hasOwnProperty("video_responsive_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo.verify(m.video_responsive_ad);
                                        if (e)
                                            return "video_responsive_ad." + e;
                                    }
                                }
                                if (m.responsive_search_ad != null && m.hasOwnProperty("responsive_search_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo.verify(m.responsive_search_ad);
                                        if (e)
                                            return "responsive_search_ad." + e;
                                    }
                                }
                                if (m.legacy_responsive_display_ad != null && m.hasOwnProperty("legacy_responsive_display_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo.verify(m.legacy_responsive_display_ad);
                                        if (e)
                                            return "legacy_responsive_display_ad." + e;
                                    }
                                }
                                if (m.app_ad != null && m.hasOwnProperty("app_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AppAdInfo.verify(m.app_ad);
                                        if (e)
                                            return "app_ad." + e;
                                    }
                                }
                                if (m.legacy_app_install_ad != null && m.hasOwnProperty("legacy_app_install_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo.verify(m.legacy_app_install_ad);
                                        if (e)
                                            return "legacy_app_install_ad." + e;
                                    }
                                }
                                if (m.responsive_display_ad != null && m.hasOwnProperty("responsive_display_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.verify(m.responsive_display_ad);
                                        if (e)
                                            return "responsive_display_ad." + e;
                                    }
                                }
                                if (m.local_ad != null && m.hasOwnProperty("local_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LocalAdInfo.verify(m.local_ad);
                                        if (e)
                                            return "local_ad." + e;
                                    }
                                }
                                if (m.display_upload_ad != null && m.hasOwnProperty("display_upload_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DisplayUploadAdInfo.verify(m.display_upload_ad);
                                        if (e)
                                            return "display_upload_ad." + e;
                                    }
                                }
                                if (m.app_engagement_ad != null && m.hasOwnProperty("app_engagement_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AppEngagementAdInfo.verify(m.app_engagement_ad);
                                        if (e)
                                            return "app_engagement_ad." + e;
                                    }
                                }
                                if (m.shopping_comparison_listing_ad != null && m.hasOwnProperty("shopping_comparison_listing_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo.verify(m.shopping_comparison_listing_ad);
                                        if (e)
                                            return "shopping_comparison_listing_ad." + e;
                                    }
                                }
                                if (m.smart_campaign_ad != null && m.hasOwnProperty("smart_campaign_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.verify(m.smart_campaign_ad);
                                        if (e)
                                            return "smart_campaign_ad." + e;
                                    }
                                }
                                if (m.app_pre_registration_ad != null && m.hasOwnProperty("app_pre_registration_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo.verify(m.app_pre_registration_ad);
                                        if (e)
                                            return "app_pre_registration_ad." + e;
                                    }
                                }
                                if (m.discovery_multi_asset_ad != null && m.hasOwnProperty("discovery_multi_asset_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.verify(m.discovery_multi_asset_ad);
                                        if (e)
                                            return "discovery_multi_asset_ad." + e;
                                    }
                                }
                                if (m.discovery_carousel_ad != null && m.hasOwnProperty("discovery_carousel_ad")) {
                                    if (p.ad_data === 1)
                                        return "ad_data: multiple values";
                                    p.ad_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.verify(m.discovery_carousel_ad);
                                        if (e)
                                            return "discovery_carousel_ad." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Ad message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Ad} Ad
                             */
                            Ad.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Ad)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Ad();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_app_urls) {
                                    if (!Array.isArray(d.final_app_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.final_app_urls: array expected");
                                    m.final_app_urls = [];
                                    for (var i = 0; i < d.final_app_urls.length; ++i) {
                                        if (typeof d.final_app_urls[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Ad.final_app_urls: object expected");
                                        m.final_app_urls[i] = $root.google.ads.googleads.v11.common.FinalAppUrl.fromObject(d.final_app_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Ad.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.display_url != null) {
                                    m.display_url = String(d.display_url);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "TEXT_AD":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "EXPANDED_TEXT_AD":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "EXPANDED_DYNAMIC_SEARCH_AD":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "HOTEL_AD":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "SHOPPING_SMART_AD":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "SHOPPING_PRODUCT_AD":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "VIDEO_AD":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "GMAIL_AD":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "IMAGE_AD":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "RESPONSIVE_SEARCH_AD":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "LEGACY_RESPONSIVE_DISPLAY_AD":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "APP_AD":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "LEGACY_APP_INSTALL_AD":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "RESPONSIVE_DISPLAY_AD":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "LOCAL_AD":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "HTML5_UPLOAD_AD":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "DYNAMIC_HTML5_AD":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "APP_ENGAGEMENT_AD":
                                case 23:
                                    m.type = 23;
                                    break;
                                case "SHOPPING_COMPARISON_LISTING_AD":
                                case 24:
                                    m.type = 24;
                                    break;
                                case "VIDEO_BUMPER_AD":
                                case 25:
                                    m.type = 25;
                                    break;
                                case "VIDEO_NON_SKIPPABLE_IN_STREAM_AD":
                                case 26:
                                    m.type = 26;
                                    break;
                                case "VIDEO_OUTSTREAM_AD":
                                case 27:
                                    m.type = 27;
                                    break;
                                case "VIDEO_TRUEVIEW_IN_STREAM_AD":
                                case 29:
                                    m.type = 29;
                                    break;
                                case "VIDEO_RESPONSIVE_AD":
                                case 30:
                                    m.type = 30;
                                    break;
                                case "SMART_CAMPAIGN_AD":
                                case 31:
                                    m.type = 31;
                                    break;
                                case "CALL_AD":
                                case 32:
                                    m.type = 32;
                                    break;
                                case "APP_PRE_REGISTRATION_AD":
                                case 33:
                                    m.type = 33;
                                    break;
                                case "IN_FEED_VIDEO_AD":
                                case 34:
                                    m.type = 34;
                                    break;
                                case "DISCOVERY_MULTI_ASSET_AD":
                                case 35:
                                    m.type = 35;
                                    break;
                                case "DISCOVERY_CAROUSEL_AD":
                                case 36:
                                    m.type = 36;
                                    break;
                                }
                                if (d.added_by_google_ads != null) {
                                    m.added_by_google_ads = Boolean(d.added_by_google_ads);
                                }
                                switch (d.device_preference) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device_preference = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device_preference = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device_preference = 2;
                                    break;
                                case "TABLET":
                                case 3:
                                    m.device_preference = 3;
                                    break;
                                case "DESKTOP":
                                case 4:
                                    m.device_preference = 4;
                                    break;
                                case "CONNECTED_TV":
                                case 6:
                                    m.device_preference = 6;
                                    break;
                                case "OTHER":
                                case 5:
                                    m.device_preference = 5;
                                    break;
                                }
                                if (d.url_collections) {
                                    if (!Array.isArray(d.url_collections))
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.url_collections: array expected");
                                    m.url_collections = [];
                                    for (var i = 0; i < d.url_collections.length; ++i) {
                                        if (typeof d.url_collections[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Ad.url_collections: object expected");
                                        m.url_collections[i] = $root.google.ads.googleads.v11.common.UrlCollection.fromObject(d.url_collections[i]);
                                    }
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.system_managed_resource_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.system_managed_resource_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.system_managed_resource_source = 1;
                                    break;
                                case "AD_VARIATIONS":
                                case 2:
                                    m.system_managed_resource_source = 2;
                                    break;
                                }
                                if (d.text_ad != null) {
                                    if (typeof d.text_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.text_ad: object expected");
                                    m.text_ad = $root.google.ads.googleads.v11.common.TextAdInfo.fromObject(d.text_ad);
                                }
                                if (d.expanded_text_ad != null) {
                                    if (typeof d.expanded_text_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.expanded_text_ad: object expected");
                                    m.expanded_text_ad = $root.google.ads.googleads.v11.common.ExpandedTextAdInfo.fromObject(d.expanded_text_ad);
                                }
                                if (d.call_ad != null) {
                                    if (typeof d.call_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.call_ad: object expected");
                                    m.call_ad = $root.google.ads.googleads.v11.common.CallAdInfo.fromObject(d.call_ad);
                                }
                                if (d.expanded_dynamic_search_ad != null) {
                                    if (typeof d.expanded_dynamic_search_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.expanded_dynamic_search_ad: object expected");
                                    m.expanded_dynamic_search_ad = $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo.fromObject(d.expanded_dynamic_search_ad);
                                }
                                if (d.hotel_ad != null) {
                                    if (typeof d.hotel_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.hotel_ad: object expected");
                                    m.hotel_ad = $root.google.ads.googleads.v11.common.HotelAdInfo.fromObject(d.hotel_ad);
                                }
                                if (d.shopping_smart_ad != null) {
                                    if (typeof d.shopping_smart_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.shopping_smart_ad: object expected");
                                    m.shopping_smart_ad = $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo.fromObject(d.shopping_smart_ad);
                                }
                                if (d.shopping_product_ad != null) {
                                    if (typeof d.shopping_product_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.shopping_product_ad: object expected");
                                    m.shopping_product_ad = $root.google.ads.googleads.v11.common.ShoppingProductAdInfo.fromObject(d.shopping_product_ad);
                                }
                                if (d.gmail_ad != null) {
                                    if (typeof d.gmail_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.gmail_ad: object expected");
                                    m.gmail_ad = $root.google.ads.googleads.v11.common.GmailAdInfo.fromObject(d.gmail_ad);
                                }
                                if (d.image_ad != null) {
                                    if (typeof d.image_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.image_ad: object expected");
                                    m.image_ad = $root.google.ads.googleads.v11.common.ImageAdInfo.fromObject(d.image_ad);
                                }
                                if (d.video_ad != null) {
                                    if (typeof d.video_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.video_ad: object expected");
                                    m.video_ad = $root.google.ads.googleads.v11.common.VideoAdInfo.fromObject(d.video_ad);
                                }
                                if (d.video_responsive_ad != null) {
                                    if (typeof d.video_responsive_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.video_responsive_ad: object expected");
                                    m.video_responsive_ad = $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo.fromObject(d.video_responsive_ad);
                                }
                                if (d.responsive_search_ad != null) {
                                    if (typeof d.responsive_search_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.responsive_search_ad: object expected");
                                    m.responsive_search_ad = $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo.fromObject(d.responsive_search_ad);
                                }
                                if (d.legacy_responsive_display_ad != null) {
                                    if (typeof d.legacy_responsive_display_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.legacy_responsive_display_ad: object expected");
                                    m.legacy_responsive_display_ad = $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo.fromObject(d.legacy_responsive_display_ad);
                                }
                                if (d.app_ad != null) {
                                    if (typeof d.app_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.app_ad: object expected");
                                    m.app_ad = $root.google.ads.googleads.v11.common.AppAdInfo.fromObject(d.app_ad);
                                }
                                if (d.legacy_app_install_ad != null) {
                                    if (typeof d.legacy_app_install_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.legacy_app_install_ad: object expected");
                                    m.legacy_app_install_ad = $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo.fromObject(d.legacy_app_install_ad);
                                }
                                if (d.responsive_display_ad != null) {
                                    if (typeof d.responsive_display_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.responsive_display_ad: object expected");
                                    m.responsive_display_ad = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.fromObject(d.responsive_display_ad);
                                }
                                if (d.local_ad != null) {
                                    if (typeof d.local_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.local_ad: object expected");
                                    m.local_ad = $root.google.ads.googleads.v11.common.LocalAdInfo.fromObject(d.local_ad);
                                }
                                if (d.display_upload_ad != null) {
                                    if (typeof d.display_upload_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.display_upload_ad: object expected");
                                    m.display_upload_ad = $root.google.ads.googleads.v11.common.DisplayUploadAdInfo.fromObject(d.display_upload_ad);
                                }
                                if (d.app_engagement_ad != null) {
                                    if (typeof d.app_engagement_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.app_engagement_ad: object expected");
                                    m.app_engagement_ad = $root.google.ads.googleads.v11.common.AppEngagementAdInfo.fromObject(d.app_engagement_ad);
                                }
                                if (d.shopping_comparison_listing_ad != null) {
                                    if (typeof d.shopping_comparison_listing_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.shopping_comparison_listing_ad: object expected");
                                    m.shopping_comparison_listing_ad = $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo.fromObject(d.shopping_comparison_listing_ad);
                                }
                                if (d.smart_campaign_ad != null) {
                                    if (typeof d.smart_campaign_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.smart_campaign_ad: object expected");
                                    m.smart_campaign_ad = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.fromObject(d.smart_campaign_ad);
                                }
                                if (d.app_pre_registration_ad != null) {
                                    if (typeof d.app_pre_registration_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.app_pre_registration_ad: object expected");
                                    m.app_pre_registration_ad = $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo.fromObject(d.app_pre_registration_ad);
                                }
                                if (d.discovery_multi_asset_ad != null) {
                                    if (typeof d.discovery_multi_asset_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.discovery_multi_asset_ad: object expected");
                                    m.discovery_multi_asset_ad = $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.fromObject(d.discovery_multi_asset_ad);
                                }
                                if (d.discovery_carousel_ad != null) {
                                    if (typeof d.discovery_carousel_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Ad.discovery_carousel_ad: object expected");
                                    m.discovery_carousel_ad = $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.fromObject(d.discovery_carousel_ad);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an Ad message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @static
                             * @param {google.ads.googleads.v11.resources.Ad} m Ad
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Ad.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.url_custom_parameters = [];
                                    d.url_collections = [];
                                    d.final_app_urls = [];
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.device_preference = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.system_managed_resource_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.resource_name = "";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.AdTypeEnum.AdType[m.type] : m.type;
                                }
                                if (m.text_ad != null && m.hasOwnProperty("text_ad")) {
                                    d.text_ad = $root.google.ads.googleads.v11.common.TextAdInfo.toObject(m.text_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "text_ad";
                                }
                                if (m.expanded_text_ad != null && m.hasOwnProperty("expanded_text_ad")) {
                                    d.expanded_text_ad = $root.google.ads.googleads.v11.common.ExpandedTextAdInfo.toObject(m.expanded_text_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "expanded_text_ad";
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.expanded_dynamic_search_ad != null && m.hasOwnProperty("expanded_dynamic_search_ad")) {
                                    d.expanded_dynamic_search_ad = $root.google.ads.googleads.v11.common.ExpandedDynamicSearchAdInfo.toObject(m.expanded_dynamic_search_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "expanded_dynamic_search_ad";
                                }
                                if (m.hotel_ad != null && m.hasOwnProperty("hotel_ad")) {
                                    d.hotel_ad = $root.google.ads.googleads.v11.common.HotelAdInfo.toObject(m.hotel_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "hotel_ad";
                                }
                                if (m.shopping_smart_ad != null && m.hasOwnProperty("shopping_smart_ad")) {
                                    d.shopping_smart_ad = $root.google.ads.googleads.v11.common.ShoppingSmartAdInfo.toObject(m.shopping_smart_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "shopping_smart_ad";
                                }
                                if (m.shopping_product_ad != null && m.hasOwnProperty("shopping_product_ad")) {
                                    d.shopping_product_ad = $root.google.ads.googleads.v11.common.ShoppingProductAdInfo.toObject(m.shopping_product_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "shopping_product_ad";
                                }
                                if (m.device_preference != null && m.hasOwnProperty("device_preference")) {
                                    d.device_preference = o.enums === String ? $root.google.ads.googleads.v11.enums.DeviceEnum.Device[m.device_preference] : m.device_preference;
                                }
                                if (m.gmail_ad != null && m.hasOwnProperty("gmail_ad")) {
                                    d.gmail_ad = $root.google.ads.googleads.v11.common.GmailAdInfo.toObject(m.gmail_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "gmail_ad";
                                }
                                if (m.image_ad != null && m.hasOwnProperty("image_ad")) {
                                    d.image_ad = $root.google.ads.googleads.v11.common.ImageAdInfo.toObject(m.image_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "image_ad";
                                }
                                if (m.video_ad != null && m.hasOwnProperty("video_ad")) {
                                    d.video_ad = $root.google.ads.googleads.v11.common.VideoAdInfo.toObject(m.video_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "video_ad";
                                }
                                if (m.responsive_search_ad != null && m.hasOwnProperty("responsive_search_ad")) {
                                    d.responsive_search_ad = $root.google.ads.googleads.v11.common.ResponsiveSearchAdInfo.toObject(m.responsive_search_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "responsive_search_ad";
                                }
                                if (m.url_collections && m.url_collections.length) {
                                    d.url_collections = [];
                                    for (var j = 0; j < m.url_collections.length; ++j) {
                                        d.url_collections[j] = $root.google.ads.googleads.v11.common.UrlCollection.toObject(m.url_collections[j], o);
                                    }
                                }
                                if (m.system_managed_resource_source != null && m.hasOwnProperty("system_managed_resource_source")) {
                                    d.system_managed_resource_source = o.enums === String ? $root.google.ads.googleads.v11.enums.SystemManagedResourceSourceEnum.SystemManagedResourceSource[m.system_managed_resource_source] : m.system_managed_resource_source;
                                }
                                if (m.legacy_responsive_display_ad != null && m.hasOwnProperty("legacy_responsive_display_ad")) {
                                    d.legacy_responsive_display_ad = $root.google.ads.googleads.v11.common.LegacyResponsiveDisplayAdInfo.toObject(m.legacy_responsive_display_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "legacy_responsive_display_ad";
                                }
                                if (m.app_ad != null && m.hasOwnProperty("app_ad")) {
                                    d.app_ad = $root.google.ads.googleads.v11.common.AppAdInfo.toObject(m.app_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "app_ad";
                                }
                                if (m.legacy_app_install_ad != null && m.hasOwnProperty("legacy_app_install_ad")) {
                                    d.legacy_app_install_ad = $root.google.ads.googleads.v11.common.LegacyAppInstallAdInfo.toObject(m.legacy_app_install_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "legacy_app_install_ad";
                                }
                                if (m.responsive_display_ad != null && m.hasOwnProperty("responsive_display_ad")) {
                                    d.responsive_display_ad = $root.google.ads.googleads.v11.common.ResponsiveDisplayAdInfo.toObject(m.responsive_display_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "responsive_display_ad";
                                }
                                if (m.local_ad != null && m.hasOwnProperty("local_ad")) {
                                    d.local_ad = $root.google.ads.googleads.v11.common.LocalAdInfo.toObject(m.local_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "local_ad";
                                }
                                if (m.display_upload_ad != null && m.hasOwnProperty("display_upload_ad")) {
                                    d.display_upload_ad = $root.google.ads.googleads.v11.common.DisplayUploadAdInfo.toObject(m.display_upload_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "display_upload_ad";
                                }
                                if (m.app_engagement_ad != null && m.hasOwnProperty("app_engagement_ad")) {
                                    d.app_engagement_ad = $root.google.ads.googleads.v11.common.AppEngagementAdInfo.toObject(m.app_engagement_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "app_engagement_ad";
                                }
                                if (m.final_app_urls && m.final_app_urls.length) {
                                    d.final_app_urls = [];
                                    for (var j = 0; j < m.final_app_urls.length; ++j) {
                                        d.final_app_urls[j] = $root.google.ads.googleads.v11.common.FinalAppUrl.toObject(m.final_app_urls[j], o);
                                    }
                                }
                                if (m.shopping_comparison_listing_ad != null && m.hasOwnProperty("shopping_comparison_listing_ad")) {
                                    d.shopping_comparison_listing_ad = $root.google.ads.googleads.v11.common.ShoppingComparisonListingAdInfo.toObject(m.shopping_comparison_listing_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "shopping_comparison_listing_ad";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.video_responsive_ad != null && m.hasOwnProperty("video_responsive_ad")) {
                                    d.video_responsive_ad = $root.google.ads.googleads.v11.common.VideoResponsiveAdInfo.toObject(m.video_responsive_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "video_responsive_ad";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                if (m.display_url != null && m.hasOwnProperty("display_url")) {
                                    d.display_url = m.display_url;
                                    if (o.oneofs)
                                        d._display_url = "display_url";
                                }
                                if (m.added_by_google_ads != null && m.hasOwnProperty("added_by_google_ads")) {
                                    d.added_by_google_ads = m.added_by_google_ads;
                                    if (o.oneofs)
                                        d._added_by_google_ads = "added_by_google_ads";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.smart_campaign_ad != null && m.hasOwnProperty("smart_campaign_ad")) {
                                    d.smart_campaign_ad = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.toObject(m.smart_campaign_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "smart_campaign_ad";
                                }
                                if (m.call_ad != null && m.hasOwnProperty("call_ad")) {
                                    d.call_ad = $root.google.ads.googleads.v11.common.CallAdInfo.toObject(m.call_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "call_ad";
                                }
                                if (m.app_pre_registration_ad != null && m.hasOwnProperty("app_pre_registration_ad")) {
                                    d.app_pre_registration_ad = $root.google.ads.googleads.v11.common.AppPreRegistrationAdInfo.toObject(m.app_pre_registration_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "app_pre_registration_ad";
                                }
                                if (m.discovery_multi_asset_ad != null && m.hasOwnProperty("discovery_multi_asset_ad")) {
                                    d.discovery_multi_asset_ad = $root.google.ads.googleads.v11.common.DiscoveryMultiAssetAdInfo.toObject(m.discovery_multi_asset_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "discovery_multi_asset_ad";
                                }
                                if (m.discovery_carousel_ad != null && m.hasOwnProperty("discovery_carousel_ad")) {
                                    d.discovery_carousel_ad = $root.google.ads.googleads.v11.common.DiscoveryCarouselAdInfo.toObject(m.discovery_carousel_ad, o);
                                    if (o.oneofs)
                                        d.ad_data = "discovery_carousel_ad";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Ad to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Ad
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Ad.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Ad;
                        })();
    
                        resources.AdGroup = (function() {
    
                            /**
                             * Properties of an AdGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroup
                             * @property {string|null} [resource_name] AdGroup resource_name
                             * @property {number|null} [id] AdGroup id
                             * @property {string|null} [name] AdGroup name
                             * @property {google.ads.googleads.v11.enums.AdGroupStatusEnum.AdGroupStatus|null} [status] AdGroup status
                             * @property {google.ads.googleads.v11.enums.AdGroupTypeEnum.AdGroupType|null} [type] AdGroup type
                             * @property {google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum.AdGroupAdRotationMode|null} [ad_rotation_mode] AdGroup ad_rotation_mode
                             * @property {string|null} [base_ad_group] AdGroup base_ad_group
                             * @property {string|null} [tracking_url_template] AdGroup tracking_url_template
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] AdGroup url_custom_parameters
                             * @property {string|null} [campaign] AdGroup campaign
                             * @property {number|null} [cpc_bid_micros] AdGroup cpc_bid_micros
                             * @property {number|null} [effective_cpc_bid_micros] AdGroup effective_cpc_bid_micros
                             * @property {number|null} [cpm_bid_micros] AdGroup cpm_bid_micros
                             * @property {number|null} [target_cpa_micros] AdGroup target_cpa_micros
                             * @property {number|null} [cpv_bid_micros] AdGroup cpv_bid_micros
                             * @property {number|null} [target_cpm_micros] AdGroup target_cpm_micros
                             * @property {number|null} [target_roas] AdGroup target_roas
                             * @property {number|null} [percent_cpc_bid_micros] AdGroup percent_cpc_bid_micros
                             * @property {google.ads.googleads.v11.common.IExplorerAutoOptimizerSetting|null} [explorer_auto_optimizer_setting] AdGroup explorer_auto_optimizer_setting
                             * @property {google.ads.googleads.v11.enums.TargetingDimensionEnum.TargetingDimension|null} [display_custom_bid_dimension] AdGroup display_custom_bid_dimension
                             * @property {string|null} [final_url_suffix] AdGroup final_url_suffix
                             * @property {google.ads.googleads.v11.common.ITargetingSetting|null} [targeting_setting] AdGroup targeting_setting
                             * @property {google.ads.googleads.v11.resources.AdGroup.IAudienceSetting|null} [audience_setting] AdGroup audience_setting
                             * @property {number|null} [effective_target_cpa_micros] AdGroup effective_target_cpa_micros
                             * @property {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource|null} [effective_target_cpa_source] AdGroup effective_target_cpa_source
                             * @property {number|null} [effective_target_roas] AdGroup effective_target_roas
                             * @property {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource|null} [effective_target_roas_source] AdGroup effective_target_roas_source
                             * @property {Array.<string>|null} [labels] AdGroup labels
                             * @property {Array.<google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType>|null} [excluded_parent_asset_field_types] AdGroup excluded_parent_asset_field_types
                             */
    
                            /**
                             * Constructs a new AdGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroup.
                             * @implements IAdGroup
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroup=} [p] Properties to set
                             */
                            function AdGroup(p) {
                                this.url_custom_parameters = [];
                                this.labels = [];
                                this.excluded_parent_asset_field_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroup resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.resource_name = "";
    
                            /**
                             * AdGroup id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.id = null;
    
                            /**
                             * AdGroup name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.name = null;
    
                            /**
                             * AdGroup status.
                             * @member {google.ads.googleads.v11.enums.AdGroupStatusEnum.AdGroupStatus} status
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.status = 0;
    
                            /**
                             * AdGroup type.
                             * @member {google.ads.googleads.v11.enums.AdGroupTypeEnum.AdGroupType} type
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.type = 0;
    
                            /**
                             * AdGroup ad_rotation_mode.
                             * @member {google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum.AdGroupAdRotationMode} ad_rotation_mode
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.ad_rotation_mode = 0;
    
                            /**
                             * AdGroup base_ad_group.
                             * @member {string|null|undefined} base_ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.base_ad_group = null;
    
                            /**
                             * AdGroup tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.tracking_url_template = null;
    
                            /**
                             * AdGroup url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * AdGroup campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.campaign = null;
    
                            /**
                             * AdGroup cpc_bid_micros.
                             * @member {number|null|undefined} cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.cpc_bid_micros = null;
    
                            /**
                             * AdGroup effective_cpc_bid_micros.
                             * @member {number|null|undefined} effective_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.effective_cpc_bid_micros = null;
    
                            /**
                             * AdGroup cpm_bid_micros.
                             * @member {number|null|undefined} cpm_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.cpm_bid_micros = null;
    
                            /**
                             * AdGroup target_cpa_micros.
                             * @member {number|null|undefined} target_cpa_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.target_cpa_micros = null;
    
                            /**
                             * AdGroup cpv_bid_micros.
                             * @member {number|null|undefined} cpv_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.cpv_bid_micros = null;
    
                            /**
                             * AdGroup target_cpm_micros.
                             * @member {number|null|undefined} target_cpm_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.target_cpm_micros = null;
    
                            /**
                             * AdGroup target_roas.
                             * @member {number|null|undefined} target_roas
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.target_roas = null;
    
                            /**
                             * AdGroup percent_cpc_bid_micros.
                             * @member {number|null|undefined} percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.percent_cpc_bid_micros = null;
    
                            /**
                             * AdGroup explorer_auto_optimizer_setting.
                             * @member {google.ads.googleads.v11.common.IExplorerAutoOptimizerSetting|null|undefined} explorer_auto_optimizer_setting
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.explorer_auto_optimizer_setting = null;
    
                            /**
                             * AdGroup display_custom_bid_dimension.
                             * @member {google.ads.googleads.v11.enums.TargetingDimensionEnum.TargetingDimension} display_custom_bid_dimension
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.display_custom_bid_dimension = 0;
    
                            /**
                             * AdGroup final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.final_url_suffix = null;
    
                            /**
                             * AdGroup targeting_setting.
                             * @member {google.ads.googleads.v11.common.ITargetingSetting|null|undefined} targeting_setting
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.targeting_setting = null;
    
                            /**
                             * AdGroup audience_setting.
                             * @member {google.ads.googleads.v11.resources.AdGroup.IAudienceSetting|null|undefined} audience_setting
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.audience_setting = null;
    
                            /**
                             * AdGroup effective_target_cpa_micros.
                             * @member {number|null|undefined} effective_target_cpa_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.effective_target_cpa_micros = null;
    
                            /**
                             * AdGroup effective_target_cpa_source.
                             * @member {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource} effective_target_cpa_source
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.effective_target_cpa_source = 0;
    
                            /**
                             * AdGroup effective_target_roas.
                             * @member {number|null|undefined} effective_target_roas
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.effective_target_roas = null;
    
                            /**
                             * AdGroup effective_target_roas_source.
                             * @member {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource} effective_target_roas_source
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.effective_target_roas_source = 0;
    
                            /**
                             * AdGroup labels.
                             * @member {Array.<string>} labels
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.labels = $util.emptyArray;
    
                            /**
                             * AdGroup excluded_parent_asset_field_types.
                             * @member {Array.<google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType>} excluded_parent_asset_field_types
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            AdGroup.prototype.excluded_parent_asset_field_types = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroup _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _base_ad_group.
                             * @member {"base_ad_group"|undefined} _base_ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_base_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["base_ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _cpc_bid_micros.
                             * @member {"cpc_bid_micros"|undefined} _cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _effective_cpc_bid_micros.
                             * @member {"effective_cpc_bid_micros"|undefined} _effective_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_effective_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _cpm_bid_micros.
                             * @member {"cpm_bid_micros"|undefined} _cpm_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_cpm_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpm_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _target_cpa_micros.
                             * @member {"target_cpa_micros"|undefined} _target_cpa_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_target_cpa_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["target_cpa_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _cpv_bid_micros.
                             * @member {"cpv_bid_micros"|undefined} _cpv_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_cpv_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpv_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _target_cpm_micros.
                             * @member {"target_cpm_micros"|undefined} _target_cpm_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_target_cpm_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["target_cpm_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _target_roas.
                             * @member {"target_roas"|undefined} _target_roas
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_target_roas", {
                                get: $util.oneOfGetter($oneOfFields = ["target_roas"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _percent_cpc_bid_micros.
                             * @member {"percent_cpc_bid_micros"|undefined} _percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_percent_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["percent_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _effective_target_cpa_micros.
                             * @member {"effective_target_cpa_micros"|undefined} _effective_target_cpa_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_effective_target_cpa_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_target_cpa_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroup _effective_target_roas.
                             * @member {"effective_target_roas"|undefined} _effective_target_roas
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             */
                            Object.defineProperty(AdGroup.prototype, "_effective_target_roas", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_target_roas"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroup instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroup=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroup} AdGroup instance
                             */
                            AdGroup.create = function create(properties) {
                                return new AdGroup(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroup message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroup.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroup} m AdGroup message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroup.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(96).int32(m.type);
                                if (m.explorer_auto_optimizer_setting != null && Object.hasOwnProperty.call(m, "explorer_auto_optimizer_setting"))
                                    $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting.encode(m.explorer_auto_optimizer_setting, w.uint32(170).fork()).ldelim();
                                if (m.ad_rotation_mode != null && Object.hasOwnProperty.call(m, "ad_rotation_mode"))
                                    w.uint32(176).int32(m.ad_rotation_mode);
                                if (m.display_custom_bid_dimension != null && Object.hasOwnProperty.call(m, "display_custom_bid_dimension"))
                                    w.uint32(184).int32(m.display_custom_bid_dimension);
                                if (m.targeting_setting != null && Object.hasOwnProperty.call(m, "targeting_setting"))
                                    $root.google.ads.googleads.v11.common.TargetingSetting.encode(m.targeting_setting, w.uint32(202).fork()).ldelim();
                                if (m.effective_target_cpa_source != null && Object.hasOwnProperty.call(m, "effective_target_cpa_source"))
                                    w.uint32(232).int32(m.effective_target_cpa_source);
                                if (m.effective_target_roas_source != null && Object.hasOwnProperty.call(m, "effective_target_roas_source"))
                                    w.uint32(256).int32(m.effective_target_roas_source);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(272).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(282).string(m.name);
                                if (m.base_ad_group != null && Object.hasOwnProperty.call(m, "base_ad_group"))
                                    w.uint32(290).string(m.base_ad_group);
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(298).string(m.tracking_url_template);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(306).string(m.campaign);
                                if (m.cpc_bid_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_micros"))
                                    w.uint32(312).int64(m.cpc_bid_micros);
                                if (m.cpm_bid_micros != null && Object.hasOwnProperty.call(m, "cpm_bid_micros"))
                                    w.uint32(320).int64(m.cpm_bid_micros);
                                if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                    w.uint32(328).int64(m.target_cpa_micros);
                                if (m.cpv_bid_micros != null && Object.hasOwnProperty.call(m, "cpv_bid_micros"))
                                    w.uint32(336).int64(m.cpv_bid_micros);
                                if (m.target_cpm_micros != null && Object.hasOwnProperty.call(m, "target_cpm_micros"))
                                    w.uint32(344).int64(m.target_cpm_micros);
                                if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                    w.uint32(353).double(m.target_roas);
                                if (m.percent_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "percent_cpc_bid_micros"))
                                    w.uint32(360).int64(m.percent_cpc_bid_micros);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(370).string(m.final_url_suffix);
                                if (m.effective_target_cpa_micros != null && Object.hasOwnProperty.call(m, "effective_target_cpa_micros"))
                                    w.uint32(376).int64(m.effective_target_cpa_micros);
                                if (m.effective_target_roas != null && Object.hasOwnProperty.call(m, "effective_target_roas"))
                                    w.uint32(385).double(m.effective_target_roas);
                                if (m.labels != null && m.labels.length) {
                                    for (var i = 0; i < m.labels.length; ++i)
                                        w.uint32(394).string(m.labels[i]);
                                }
                                if (m.excluded_parent_asset_field_types != null && m.excluded_parent_asset_field_types.length) {
                                    w.uint32(434).fork();
                                    for (var i = 0; i < m.excluded_parent_asset_field_types.length; ++i)
                                        w.int32(m.excluded_parent_asset_field_types[i]);
                                    w.ldelim();
                                }
                                if (m.audience_setting != null && Object.hasOwnProperty.call(m, "audience_setting"))
                                    $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting.encode(m.audience_setting, w.uint32(450).fork()).ldelim();
                                if (m.effective_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "effective_cpc_bid_micros"))
                                    w.uint32(456).int64(m.effective_cpc_bid_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroup message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroup.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroup} message AdGroup message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroup.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroup message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroup} AdGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroup.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroup();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 34:
                                        m.id = r.int64();
                                        break;
                                    case 35:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 12:
                                        m.type = r.int32();
                                        break;
                                    case 22:
                                        m.ad_rotation_mode = r.int32();
                                        break;
                                    case 36:
                                        m.base_ad_group = r.string();
                                        break;
                                    case 37:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 6:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 38:
                                        m.campaign = r.string();
                                        break;
                                    case 39:
                                        m.cpc_bid_micros = r.int64();
                                        break;
                                    case 57:
                                        m.effective_cpc_bid_micros = r.int64();
                                        break;
                                    case 40:
                                        m.cpm_bid_micros = r.int64();
                                        break;
                                    case 41:
                                        m.target_cpa_micros = r.int64();
                                        break;
                                    case 42:
                                        m.cpv_bid_micros = r.int64();
                                        break;
                                    case 43:
                                        m.target_cpm_micros = r.int64();
                                        break;
                                    case 44:
                                        m.target_roas = r.double();
                                        break;
                                    case 45:
                                        m.percent_cpc_bid_micros = r.int64();
                                        break;
                                    case 21:
                                        m.explorer_auto_optimizer_setting = $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.display_custom_bid_dimension = r.int32();
                                        break;
                                    case 46:
                                        m.final_url_suffix = r.string();
                                        break;
                                    case 25:
                                        m.targeting_setting = $root.google.ads.googleads.v11.common.TargetingSetting.decode(r, r.uint32());
                                        break;
                                    case 56:
                                        m.audience_setting = $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting.decode(r, r.uint32());
                                        break;
                                    case 47:
                                        m.effective_target_cpa_micros = r.int64();
                                        break;
                                    case 29:
                                        m.effective_target_cpa_source = r.int32();
                                        break;
                                    case 48:
                                        m.effective_target_roas = r.double();
                                        break;
                                    case 32:
                                        m.effective_target_roas_source = r.int32();
                                        break;
                                    case 49:
                                        if (!(m.labels && m.labels.length))
                                            m.labels = [];
                                        m.labels.push(r.string());
                                        break;
                                    case 54:
                                        if (!(m.excluded_parent_asset_field_types && m.excluded_parent_asset_field_types.length))
                                            m.excluded_parent_asset_field_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.excluded_parent_asset_field_types.push(r.int32());
                                        } else
                                            m.excluded_parent_asset_field_types.push(r.int32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroup message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroup} AdGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroup.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroup message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroup.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                        break;
                                    }
                                }
                                if (m.ad_rotation_mode != null && m.hasOwnProperty("ad_rotation_mode")) {
                                    switch (m.ad_rotation_mode) {
                                    default:
                                        return "ad_rotation_mode: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.base_ad_group != null && m.hasOwnProperty("base_ad_group")) {
                                    p._base_ad_group = 1;
                                    if (!$util.isString(m.base_ad_group))
                                        return "base_ad_group: string expected";
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    p._cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_micros) && !(m.cpc_bid_micros && $util.isInteger(m.cpc_bid_micros.low) && $util.isInteger(m.cpc_bid_micros.high)))
                                        return "cpc_bid_micros: integer|Long expected";
                                }
                                if (m.effective_cpc_bid_micros != null && m.hasOwnProperty("effective_cpc_bid_micros")) {
                                    p._effective_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.effective_cpc_bid_micros) && !(m.effective_cpc_bid_micros && $util.isInteger(m.effective_cpc_bid_micros.low) && $util.isInteger(m.effective_cpc_bid_micros.high)))
                                        return "effective_cpc_bid_micros: integer|Long expected";
                                }
                                if (m.cpm_bid_micros != null && m.hasOwnProperty("cpm_bid_micros")) {
                                    p._cpm_bid_micros = 1;
                                    if (!$util.isInteger(m.cpm_bid_micros) && !(m.cpm_bid_micros && $util.isInteger(m.cpm_bid_micros.low) && $util.isInteger(m.cpm_bid_micros.high)))
                                        return "cpm_bid_micros: integer|Long expected";
                                }
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    p._target_cpa_micros = 1;
                                    if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                        return "target_cpa_micros: integer|Long expected";
                                }
                                if (m.cpv_bid_micros != null && m.hasOwnProperty("cpv_bid_micros")) {
                                    p._cpv_bid_micros = 1;
                                    if (!$util.isInteger(m.cpv_bid_micros) && !(m.cpv_bid_micros && $util.isInteger(m.cpv_bid_micros.low) && $util.isInteger(m.cpv_bid_micros.high)))
                                        return "cpv_bid_micros: integer|Long expected";
                                }
                                if (m.target_cpm_micros != null && m.hasOwnProperty("target_cpm_micros")) {
                                    p._target_cpm_micros = 1;
                                    if (!$util.isInteger(m.target_cpm_micros) && !(m.target_cpm_micros && $util.isInteger(m.target_cpm_micros.low) && $util.isInteger(m.target_cpm_micros.high)))
                                        return "target_cpm_micros: integer|Long expected";
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    p._target_roas = 1;
                                    if (typeof m.target_roas !== "number")
                                        return "target_roas: number expected";
                                }
                                if (m.percent_cpc_bid_micros != null && m.hasOwnProperty("percent_cpc_bid_micros")) {
                                    p._percent_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.percent_cpc_bid_micros) && !(m.percent_cpc_bid_micros && $util.isInteger(m.percent_cpc_bid_micros.low) && $util.isInteger(m.percent_cpc_bid_micros.high)))
                                        return "percent_cpc_bid_micros: integer|Long expected";
                                }
                                if (m.explorer_auto_optimizer_setting != null && m.hasOwnProperty("explorer_auto_optimizer_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting.verify(m.explorer_auto_optimizer_setting);
                                        if (e)
                                            return "explorer_auto_optimizer_setting." + e;
                                    }
                                }
                                if (m.display_custom_bid_dimension != null && m.hasOwnProperty("display_custom_bid_dimension")) {
                                    switch (m.display_custom_bid_dimension) {
                                    default:
                                        return "display_custom_bid_dimension: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                if (m.targeting_setting != null && m.hasOwnProperty("targeting_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetingSetting.verify(m.targeting_setting);
                                        if (e)
                                            return "targeting_setting." + e;
                                    }
                                }
                                if (m.audience_setting != null && m.hasOwnProperty("audience_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting.verify(m.audience_setting);
                                        if (e)
                                            return "audience_setting." + e;
                                    }
                                }
                                if (m.effective_target_cpa_micros != null && m.hasOwnProperty("effective_target_cpa_micros")) {
                                    p._effective_target_cpa_micros = 1;
                                    if (!$util.isInteger(m.effective_target_cpa_micros) && !(m.effective_target_cpa_micros && $util.isInteger(m.effective_target_cpa_micros.low) && $util.isInteger(m.effective_target_cpa_micros.high)))
                                        return "effective_target_cpa_micros: integer|Long expected";
                                }
                                if (m.effective_target_cpa_source != null && m.hasOwnProperty("effective_target_cpa_source")) {
                                    switch (m.effective_target_cpa_source) {
                                    default:
                                        return "effective_target_cpa_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.effective_target_roas != null && m.hasOwnProperty("effective_target_roas")) {
                                    p._effective_target_roas = 1;
                                    if (typeof m.effective_target_roas !== "number")
                                        return "effective_target_roas: number expected";
                                }
                                if (m.effective_target_roas_source != null && m.hasOwnProperty("effective_target_roas_source")) {
                                    switch (m.effective_target_roas_source) {
                                    default:
                                        return "effective_target_roas_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.labels != null && m.hasOwnProperty("labels")) {
                                    if (!Array.isArray(m.labels))
                                        return "labels: array expected";
                                    for (var i = 0; i < m.labels.length; ++i) {
                                        if (!$util.isString(m.labels[i]))
                                            return "labels: string[] expected";
                                    }
                                }
                                if (m.excluded_parent_asset_field_types != null && m.hasOwnProperty("excluded_parent_asset_field_types")) {
                                    if (!Array.isArray(m.excluded_parent_asset_field_types))
                                        return "excluded_parent_asset_field_types: array expected";
                                    for (var i = 0; i < m.excluded_parent_asset_field_types.length; ++i) {
                                        switch (m.excluded_parent_asset_field_types[i]) {
                                        default:
                                            return "excluded_parent_asset_field_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 24:
                                        case 17:
                                        case 18:
                                        case 19:
                                        case 20:
                                        case 21:
                                        case 22:
                                        case 23:
                                        case 25:
                                            break;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroup message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroup} AdGroup
                             */
                            AdGroup.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroup)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroup();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "PAUSED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "SEARCH_STANDARD":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "DISPLAY_STANDARD":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "SHOPPING_PRODUCT_ADS":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "HOTEL_ADS":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "SHOPPING_SMART_ADS":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "VIDEO_BUMPER":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "VIDEO_TRUE_VIEW_IN_STREAM":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "VIDEO_TRUE_VIEW_IN_DISPLAY":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "VIDEO_NON_SKIPPABLE_IN_STREAM":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "VIDEO_OUTSTREAM":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "SEARCH_DYNAMIC_ADS":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "SHOPPING_COMPARISON_LISTING_ADS":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "PROMOTED_HOTEL_ADS":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "VIDEO_RESPONSIVE":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "VIDEO_EFFICIENT_REACH":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "SMART_CAMPAIGN_ADS":
                                case 18:
                                    m.type = 18;
                                    break;
                                }
                                switch (d.ad_rotation_mode) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_rotation_mode = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_rotation_mode = 1;
                                    break;
                                case "OPTIMIZE":
                                case 2:
                                    m.ad_rotation_mode = 2;
                                    break;
                                case "ROTATE_FOREVER":
                                case 3:
                                    m.ad_rotation_mode = 3;
                                    break;
                                }
                                if (d.base_ad_group != null) {
                                    m.base_ad_group = String(d.base_ad_group);
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroup.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.AdGroup.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_micros = $util.Long.fromValue(d.cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_micros === "string")
                                        m.cpc_bid_micros = parseInt(d.cpc_bid_micros, 10);
                                    else if (typeof d.cpc_bid_micros === "number")
                                        m.cpc_bid_micros = d.cpc_bid_micros;
                                    else if (typeof d.cpc_bid_micros === "object")
                                        m.cpc_bid_micros = new $util.LongBits(d.cpc_bid_micros.low >>> 0, d.cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.effective_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.effective_cpc_bid_micros = $util.Long.fromValue(d.effective_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.effective_cpc_bid_micros === "string")
                                        m.effective_cpc_bid_micros = parseInt(d.effective_cpc_bid_micros, 10);
                                    else if (typeof d.effective_cpc_bid_micros === "number")
                                        m.effective_cpc_bid_micros = d.effective_cpc_bid_micros;
                                    else if (typeof d.effective_cpc_bid_micros === "object")
                                        m.effective_cpc_bid_micros = new $util.LongBits(d.effective_cpc_bid_micros.low >>> 0, d.effective_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.cpm_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpm_bid_micros = $util.Long.fromValue(d.cpm_bid_micros)).unsigned = false;
                                    else if (typeof d.cpm_bid_micros === "string")
                                        m.cpm_bid_micros = parseInt(d.cpm_bid_micros, 10);
                                    else if (typeof d.cpm_bid_micros === "number")
                                        m.cpm_bid_micros = d.cpm_bid_micros;
                                    else if (typeof d.cpm_bid_micros === "object")
                                        m.cpm_bid_micros = new $util.LongBits(d.cpm_bid_micros.low >>> 0, d.cpm_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.target_cpa_micros != null) {
                                    if ($util.Long)
                                        (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                    else if (typeof d.target_cpa_micros === "string")
                                        m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                    else if (typeof d.target_cpa_micros === "number")
                                        m.target_cpa_micros = d.target_cpa_micros;
                                    else if (typeof d.target_cpa_micros === "object")
                                        m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                }
                                if (d.cpv_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpv_bid_micros = $util.Long.fromValue(d.cpv_bid_micros)).unsigned = false;
                                    else if (typeof d.cpv_bid_micros === "string")
                                        m.cpv_bid_micros = parseInt(d.cpv_bid_micros, 10);
                                    else if (typeof d.cpv_bid_micros === "number")
                                        m.cpv_bid_micros = d.cpv_bid_micros;
                                    else if (typeof d.cpv_bid_micros === "object")
                                        m.cpv_bid_micros = new $util.LongBits(d.cpv_bid_micros.low >>> 0, d.cpv_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.target_cpm_micros != null) {
                                    if ($util.Long)
                                        (m.target_cpm_micros = $util.Long.fromValue(d.target_cpm_micros)).unsigned = false;
                                    else if (typeof d.target_cpm_micros === "string")
                                        m.target_cpm_micros = parseInt(d.target_cpm_micros, 10);
                                    else if (typeof d.target_cpm_micros === "number")
                                        m.target_cpm_micros = d.target_cpm_micros;
                                    else if (typeof d.target_cpm_micros === "object")
                                        m.target_cpm_micros = new $util.LongBits(d.target_cpm_micros.low >>> 0, d.target_cpm_micros.high >>> 0).toNumber();
                                }
                                if (d.target_roas != null) {
                                    m.target_roas = Number(d.target_roas);
                                }
                                if (d.percent_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.percent_cpc_bid_micros = $util.Long.fromValue(d.percent_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.percent_cpc_bid_micros === "string")
                                        m.percent_cpc_bid_micros = parseInt(d.percent_cpc_bid_micros, 10);
                                    else if (typeof d.percent_cpc_bid_micros === "number")
                                        m.percent_cpc_bid_micros = d.percent_cpc_bid_micros;
                                    else if (typeof d.percent_cpc_bid_micros === "object")
                                        m.percent_cpc_bid_micros = new $util.LongBits(d.percent_cpc_bid_micros.low >>> 0, d.percent_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.explorer_auto_optimizer_setting != null) {
                                    if (typeof d.explorer_auto_optimizer_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroup.explorer_auto_optimizer_setting: object expected");
                                    m.explorer_auto_optimizer_setting = $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting.fromObject(d.explorer_auto_optimizer_setting);
                                }
                                switch (d.display_custom_bid_dimension) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.display_custom_bid_dimension = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.display_custom_bid_dimension = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.display_custom_bid_dimension = 2;
                                    break;
                                case "AUDIENCE":
                                case 3:
                                    m.display_custom_bid_dimension = 3;
                                    break;
                                case "TOPIC":
                                case 4:
                                    m.display_custom_bid_dimension = 4;
                                    break;
                                case "GENDER":
                                case 5:
                                    m.display_custom_bid_dimension = 5;
                                    break;
                                case "AGE_RANGE":
                                case 6:
                                    m.display_custom_bid_dimension = 6;
                                    break;
                                case "PLACEMENT":
                                case 7:
                                    m.display_custom_bid_dimension = 7;
                                    break;
                                case "PARENTAL_STATUS":
                                case 8:
                                    m.display_custom_bid_dimension = 8;
                                    break;
                                case "INCOME_RANGE":
                                case 9:
                                    m.display_custom_bid_dimension = 9;
                                    break;
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                if (d.targeting_setting != null) {
                                    if (typeof d.targeting_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroup.targeting_setting: object expected");
                                    m.targeting_setting = $root.google.ads.googleads.v11.common.TargetingSetting.fromObject(d.targeting_setting);
                                }
                                if (d.audience_setting != null) {
                                    if (typeof d.audience_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroup.audience_setting: object expected");
                                    m.audience_setting = $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting.fromObject(d.audience_setting);
                                }
                                if (d.effective_target_cpa_micros != null) {
                                    if ($util.Long)
                                        (m.effective_target_cpa_micros = $util.Long.fromValue(d.effective_target_cpa_micros)).unsigned = false;
                                    else if (typeof d.effective_target_cpa_micros === "string")
                                        m.effective_target_cpa_micros = parseInt(d.effective_target_cpa_micros, 10);
                                    else if (typeof d.effective_target_cpa_micros === "number")
                                        m.effective_target_cpa_micros = d.effective_target_cpa_micros;
                                    else if (typeof d.effective_target_cpa_micros === "object")
                                        m.effective_target_cpa_micros = new $util.LongBits(d.effective_target_cpa_micros.low >>> 0, d.effective_target_cpa_micros.high >>> 0).toNumber();
                                }
                                switch (d.effective_target_cpa_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.effective_target_cpa_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.effective_target_cpa_source = 1;
                                    break;
                                case "CAMPAIGN_BIDDING_STRATEGY":
                                case 5:
                                    m.effective_target_cpa_source = 5;
                                    break;
                                case "AD_GROUP":
                                case 6:
                                    m.effective_target_cpa_source = 6;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 7:
                                    m.effective_target_cpa_source = 7;
                                    break;
                                }
                                if (d.effective_target_roas != null) {
                                    m.effective_target_roas = Number(d.effective_target_roas);
                                }
                                switch (d.effective_target_roas_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.effective_target_roas_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.effective_target_roas_source = 1;
                                    break;
                                case "CAMPAIGN_BIDDING_STRATEGY":
                                case 5:
                                    m.effective_target_roas_source = 5;
                                    break;
                                case "AD_GROUP":
                                case 6:
                                    m.effective_target_roas_source = 6;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 7:
                                    m.effective_target_roas_source = 7;
                                    break;
                                }
                                if (d.labels) {
                                    if (!Array.isArray(d.labels))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroup.labels: array expected");
                                    m.labels = [];
                                    for (var i = 0; i < d.labels.length; ++i) {
                                        m.labels[i] = String(d.labels[i]);
                                    }
                                }
                                if (d.excluded_parent_asset_field_types) {
                                    if (!Array.isArray(d.excluded_parent_asset_field_types))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroup.excluded_parent_asset_field_types: array expected");
                                    m.excluded_parent_asset_field_types = [];
                                    for (var i = 0; i < d.excluded_parent_asset_field_types.length; ++i) {
                                        switch (d.excluded_parent_asset_field_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.excluded_parent_asset_field_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.excluded_parent_asset_field_types[i] = 1;
                                            break;
                                        case "HEADLINE":
                                        case 2:
                                            m.excluded_parent_asset_field_types[i] = 2;
                                            break;
                                        case "DESCRIPTION":
                                        case 3:
                                            m.excluded_parent_asset_field_types[i] = 3;
                                            break;
                                        case "MANDATORY_AD_TEXT":
                                        case 4:
                                            m.excluded_parent_asset_field_types[i] = 4;
                                            break;
                                        case "MARKETING_IMAGE":
                                        case 5:
                                            m.excluded_parent_asset_field_types[i] = 5;
                                            break;
                                        case "MEDIA_BUNDLE":
                                        case 6:
                                            m.excluded_parent_asset_field_types[i] = 6;
                                            break;
                                        case "YOUTUBE_VIDEO":
                                        case 7:
                                            m.excluded_parent_asset_field_types[i] = 7;
                                            break;
                                        case "BOOK_ON_GOOGLE":
                                        case 8:
                                            m.excluded_parent_asset_field_types[i] = 8;
                                            break;
                                        case "LEAD_FORM":
                                        case 9:
                                            m.excluded_parent_asset_field_types[i] = 9;
                                            break;
                                        case "PROMOTION":
                                        case 10:
                                            m.excluded_parent_asset_field_types[i] = 10;
                                            break;
                                        case "CALLOUT":
                                        case 11:
                                            m.excluded_parent_asset_field_types[i] = 11;
                                            break;
                                        case "STRUCTURED_SNIPPET":
                                        case 12:
                                            m.excluded_parent_asset_field_types[i] = 12;
                                            break;
                                        case "SITELINK":
                                        case 13:
                                            m.excluded_parent_asset_field_types[i] = 13;
                                            break;
                                        case "MOBILE_APP":
                                        case 14:
                                            m.excluded_parent_asset_field_types[i] = 14;
                                            break;
                                        case "HOTEL_CALLOUT":
                                        case 15:
                                            m.excluded_parent_asset_field_types[i] = 15;
                                            break;
                                        case "CALL":
                                        case 16:
                                            m.excluded_parent_asset_field_types[i] = 16;
                                            break;
                                        case "PRICE":
                                        case 24:
                                            m.excluded_parent_asset_field_types[i] = 24;
                                            break;
                                        case "LONG_HEADLINE":
                                        case 17:
                                            m.excluded_parent_asset_field_types[i] = 17;
                                            break;
                                        case "BUSINESS_NAME":
                                        case 18:
                                            m.excluded_parent_asset_field_types[i] = 18;
                                            break;
                                        case "SQUARE_MARKETING_IMAGE":
                                        case 19:
                                            m.excluded_parent_asset_field_types[i] = 19;
                                            break;
                                        case "PORTRAIT_MARKETING_IMAGE":
                                        case 20:
                                            m.excluded_parent_asset_field_types[i] = 20;
                                            break;
                                        case "LOGO":
                                        case 21:
                                            m.excluded_parent_asset_field_types[i] = 21;
                                            break;
                                        case "LANDSCAPE_LOGO":
                                        case 22:
                                            m.excluded_parent_asset_field_types[i] = 22;
                                            break;
                                        case "VIDEO":
                                        case 23:
                                            m.excluded_parent_asset_field_types[i] = 23;
                                            break;
                                        case "CALL_TO_ACTION_SELECTION":
                                        case 25:
                                            m.excluded_parent_asset_field_types[i] = 25;
                                            break;
                                        }
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroup message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroup} m AdGroup
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroup.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.url_custom_parameters = [];
                                    d.labels = [];
                                    d.excluded_parent_asset_field_types = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.explorer_auto_optimizer_setting = null;
                                    d.ad_rotation_mode = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.display_custom_bid_dimension = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.targeting_setting = null;
                                    d.effective_target_cpa_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.effective_target_roas_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.audience_setting = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AdGroupStatusEnum.AdGroupStatus[m.status] : m.status;
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.AdGroupTypeEnum.AdGroupType[m.type] : m.type;
                                }
                                if (m.explorer_auto_optimizer_setting != null && m.hasOwnProperty("explorer_auto_optimizer_setting")) {
                                    d.explorer_auto_optimizer_setting = $root.google.ads.googleads.v11.common.ExplorerAutoOptimizerSetting.toObject(m.explorer_auto_optimizer_setting, o);
                                }
                                if (m.ad_rotation_mode != null && m.hasOwnProperty("ad_rotation_mode")) {
                                    d.ad_rotation_mode = o.enums === String ? $root.google.ads.googleads.v11.enums.AdGroupAdRotationModeEnum.AdGroupAdRotationMode[m.ad_rotation_mode] : m.ad_rotation_mode;
                                }
                                if (m.display_custom_bid_dimension != null && m.hasOwnProperty("display_custom_bid_dimension")) {
                                    d.display_custom_bid_dimension = o.enums === String ? $root.google.ads.googleads.v11.enums.TargetingDimensionEnum.TargetingDimension[m.display_custom_bid_dimension] : m.display_custom_bid_dimension;
                                }
                                if (m.targeting_setting != null && m.hasOwnProperty("targeting_setting")) {
                                    d.targeting_setting = $root.google.ads.googleads.v11.common.TargetingSetting.toObject(m.targeting_setting, o);
                                }
                                if (m.effective_target_cpa_source != null && m.hasOwnProperty("effective_target_cpa_source")) {
                                    d.effective_target_cpa_source = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource[m.effective_target_cpa_source] : m.effective_target_cpa_source;
                                }
                                if (m.effective_target_roas_source != null && m.hasOwnProperty("effective_target_roas_source")) {
                                    d.effective_target_roas_source = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource[m.effective_target_roas_source] : m.effective_target_roas_source;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.base_ad_group != null && m.hasOwnProperty("base_ad_group")) {
                                    d.base_ad_group = m.base_ad_group;
                                    if (o.oneofs)
                                        d._base_ad_group = "base_ad_group";
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    if (typeof m.cpc_bid_micros === "number")
                                        d.cpc_bid_micros = o.longs === String ? String(m.cpc_bid_micros) : m.cpc_bid_micros;
                                    else
                                        d.cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_micros.low >>> 0, m.cpc_bid_micros.high >>> 0).toNumber() : m.cpc_bid_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_micros = "cpc_bid_micros";
                                }
                                if (m.cpm_bid_micros != null && m.hasOwnProperty("cpm_bid_micros")) {
                                    if (typeof m.cpm_bid_micros === "number")
                                        d.cpm_bid_micros = o.longs === String ? String(m.cpm_bid_micros) : m.cpm_bid_micros;
                                    else
                                        d.cpm_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpm_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpm_bid_micros.low >>> 0, m.cpm_bid_micros.high >>> 0).toNumber() : m.cpm_bid_micros;
                                    if (o.oneofs)
                                        d._cpm_bid_micros = "cpm_bid_micros";
                                }
                                if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                    if (typeof m.target_cpa_micros === "number")
                                        d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                    else
                                        d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                    if (o.oneofs)
                                        d._target_cpa_micros = "target_cpa_micros";
                                }
                                if (m.cpv_bid_micros != null && m.hasOwnProperty("cpv_bid_micros")) {
                                    if (typeof m.cpv_bid_micros === "number")
                                        d.cpv_bid_micros = o.longs === String ? String(m.cpv_bid_micros) : m.cpv_bid_micros;
                                    else
                                        d.cpv_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpv_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpv_bid_micros.low >>> 0, m.cpv_bid_micros.high >>> 0).toNumber() : m.cpv_bid_micros;
                                    if (o.oneofs)
                                        d._cpv_bid_micros = "cpv_bid_micros";
                                }
                                if (m.target_cpm_micros != null && m.hasOwnProperty("target_cpm_micros")) {
                                    if (typeof m.target_cpm_micros === "number")
                                        d.target_cpm_micros = o.longs === String ? String(m.target_cpm_micros) : m.target_cpm_micros;
                                    else
                                        d.target_cpm_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpm_micros) : o.longs === Number ? new $util.LongBits(m.target_cpm_micros.low >>> 0, m.target_cpm_micros.high >>> 0).toNumber() : m.target_cpm_micros;
                                    if (o.oneofs)
                                        d._target_cpm_micros = "target_cpm_micros";
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    d.target_roas = o.json && !isFinite(m.target_roas) ? String(m.target_roas) : m.target_roas;
                                    if (o.oneofs)
                                        d._target_roas = "target_roas";
                                }
                                if (m.percent_cpc_bid_micros != null && m.hasOwnProperty("percent_cpc_bid_micros")) {
                                    if (typeof m.percent_cpc_bid_micros === "number")
                                        d.percent_cpc_bid_micros = o.longs === String ? String(m.percent_cpc_bid_micros) : m.percent_cpc_bid_micros;
                                    else
                                        d.percent_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.percent_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.percent_cpc_bid_micros.low >>> 0, m.percent_cpc_bid_micros.high >>> 0).toNumber() : m.percent_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._percent_cpc_bid_micros = "percent_cpc_bid_micros";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                if (m.effective_target_cpa_micros != null && m.hasOwnProperty("effective_target_cpa_micros")) {
                                    if (typeof m.effective_target_cpa_micros === "number")
                                        d.effective_target_cpa_micros = o.longs === String ? String(m.effective_target_cpa_micros) : m.effective_target_cpa_micros;
                                    else
                                        d.effective_target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.effective_target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.effective_target_cpa_micros.low >>> 0, m.effective_target_cpa_micros.high >>> 0).toNumber() : m.effective_target_cpa_micros;
                                    if (o.oneofs)
                                        d._effective_target_cpa_micros = "effective_target_cpa_micros";
                                }
                                if (m.effective_target_roas != null && m.hasOwnProperty("effective_target_roas")) {
                                    d.effective_target_roas = o.json && !isFinite(m.effective_target_roas) ? String(m.effective_target_roas) : m.effective_target_roas;
                                    if (o.oneofs)
                                        d._effective_target_roas = "effective_target_roas";
                                }
                                if (m.labels && m.labels.length) {
                                    d.labels = [];
                                    for (var j = 0; j < m.labels.length; ++j) {
                                        d.labels[j] = m.labels[j];
                                    }
                                }
                                if (m.excluded_parent_asset_field_types && m.excluded_parent_asset_field_types.length) {
                                    d.excluded_parent_asset_field_types = [];
                                    for (var j = 0; j < m.excluded_parent_asset_field_types.length; ++j) {
                                        d.excluded_parent_asset_field_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.excluded_parent_asset_field_types[j]] : m.excluded_parent_asset_field_types[j];
                                    }
                                }
                                if (m.audience_setting != null && m.hasOwnProperty("audience_setting")) {
                                    d.audience_setting = $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting.toObject(m.audience_setting, o);
                                }
                                if (m.effective_cpc_bid_micros != null && m.hasOwnProperty("effective_cpc_bid_micros")) {
                                    if (typeof m.effective_cpc_bid_micros === "number")
                                        d.effective_cpc_bid_micros = o.longs === String ? String(m.effective_cpc_bid_micros) : m.effective_cpc_bid_micros;
                                    else
                                        d.effective_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.effective_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.effective_cpc_bid_micros.low >>> 0, m.effective_cpc_bid_micros.high >>> 0).toNumber() : m.effective_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._effective_cpc_bid_micros = "effective_cpc_bid_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroup to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroup
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroup.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            AdGroup.AudienceSetting = (function() {
    
                                /**
                                 * Properties of an AudienceSetting.
                                 * @memberof google.ads.googleads.v11.resources.AdGroup
                                 * @interface IAudienceSetting
                                 * @property {boolean|null} [use_audience_grouped] AudienceSetting use_audience_grouped
                                 */
    
                                /**
                                 * Constructs a new AudienceSetting.
                                 * @memberof google.ads.googleads.v11.resources.AdGroup
                                 * @classdesc Represents an AudienceSetting.
                                 * @implements IAudienceSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AdGroup.IAudienceSetting=} [p] Properties to set
                                 */
                                function AudienceSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * AudienceSetting use_audience_grouped.
                                 * @member {boolean} use_audience_grouped
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @instance
                                 */
                                AudienceSetting.prototype.use_audience_grouped = false;
    
                                /**
                                 * Creates a new AudienceSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroup.IAudienceSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AdGroup.AudienceSetting} AudienceSetting instance
                                 */
                                AudienceSetting.create = function create(properties) {
                                    return new AudienceSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified AudienceSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroup.AudienceSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroup.IAudienceSetting} m AudienceSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AudienceSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.use_audience_grouped != null && Object.hasOwnProperty.call(m, "use_audience_grouped"))
                                        w.uint32(8).bool(m.use_audience_grouped);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified AudienceSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroup.AudienceSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroup.IAudienceSetting} message AudienceSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AudienceSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an AudienceSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AdGroup.AudienceSetting} AudienceSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AudienceSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.use_audience_grouped = r.bool();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an AudienceSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AdGroup.AudienceSetting} AudienceSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AudienceSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an AudienceSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                AudienceSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.use_audience_grouped != null && m.hasOwnProperty("use_audience_grouped")) {
                                        if (typeof m.use_audience_grouped !== "boolean")
                                            return "use_audience_grouped: boolean expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an AudienceSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AdGroup.AudienceSetting} AudienceSetting
                                 */
                                AudienceSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AdGroup.AudienceSetting();
                                    if (d.use_audience_grouped != null) {
                                        m.use_audience_grouped = Boolean(d.use_audience_grouped);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an AudienceSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroup.AudienceSetting} m AudienceSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                AudienceSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.use_audience_grouped = false;
                                    }
                                    if (m.use_audience_grouped != null && m.hasOwnProperty("use_audience_grouped")) {
                                        d.use_audience_grouped = m.use_audience_grouped;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this AudienceSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AdGroup.AudienceSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                AudienceSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return AudienceSetting;
                            })();
    
                            return AdGroup;
                        })();
    
                        resources.AdGroupAd = (function() {
    
                            /**
                             * Properties of an AdGroupAd.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAd
                             * @property {string|null} [resource_name] AdGroupAd resource_name
                             * @property {google.ads.googleads.v11.enums.AdGroupAdStatusEnum.AdGroupAdStatus|null} [status] AdGroupAd status
                             * @property {string|null} [ad_group] AdGroupAd ad_group
                             * @property {google.ads.googleads.v11.resources.IAd|null} [ad] AdGroupAd ad
                             * @property {google.ads.googleads.v11.resources.IAdGroupAdPolicySummary|null} [policy_summary] AdGroupAd policy_summary
                             * @property {google.ads.googleads.v11.enums.AdStrengthEnum.AdStrength|null} [ad_strength] AdGroupAd ad_strength
                             * @property {Array.<string>|null} [action_items] AdGroupAd action_items
                             * @property {Array.<string>|null} [labels] AdGroupAd labels
                             */
    
                            /**
                             * Constructs a new AdGroupAd.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAd.
                             * @implements IAdGroupAd
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAd=} [p] Properties to set
                             */
                            function AdGroupAd(p) {
                                this.action_items = [];
                                this.labels = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAd resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.resource_name = "";
    
                            /**
                             * AdGroupAd status.
                             * @member {google.ads.googleads.v11.enums.AdGroupAdStatusEnum.AdGroupAdStatus} status
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.status = 0;
    
                            /**
                             * AdGroupAd ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.ad_group = null;
    
                            /**
                             * AdGroupAd ad.
                             * @member {google.ads.googleads.v11.resources.IAd|null|undefined} ad
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.ad = null;
    
                            /**
                             * AdGroupAd policy_summary.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAdPolicySummary|null|undefined} policy_summary
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.policy_summary = null;
    
                            /**
                             * AdGroupAd ad_strength.
                             * @member {google.ads.googleads.v11.enums.AdStrengthEnum.AdStrength} ad_strength
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.ad_strength = 0;
    
                            /**
                             * AdGroupAd action_items.
                             * @member {Array.<string>} action_items
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.action_items = $util.emptyArray;
    
                            /**
                             * AdGroupAd labels.
                             * @member {Array.<string>} labels
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            AdGroupAd.prototype.labels = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupAd _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             */
                            Object.defineProperty(AdGroupAd.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupAd instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAd=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAd} AdGroupAd instance
                             */
                            AdGroupAd.create = function create(properties) {
                                return new AdGroupAd(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAd message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAd.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAd} m AdGroupAd message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAd.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.ad != null && Object.hasOwnProperty.call(m, "ad"))
                                    $root.google.ads.googleads.v11.resources.Ad.encode(m.ad, w.uint32(42).fork()).ldelim();
                                if (m.policy_summary != null && Object.hasOwnProperty.call(m, "policy_summary"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary.encode(m.policy_summary, w.uint32(50).fork()).ldelim();
                                if (m.ad_strength != null && Object.hasOwnProperty.call(m, "ad_strength"))
                                    w.uint32(56).int32(m.ad_strength);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(74).string(m.ad_group);
                                if (m.labels != null && m.labels.length) {
                                    for (var i = 0; i < m.labels.length; ++i)
                                        w.uint32(82).string(m.labels[i]);
                                }
                                if (m.action_items != null && m.action_items.length) {
                                    for (var i = 0; i < m.action_items.length; ++i)
                                        w.uint32(106).string(m.action_items[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAd message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAd.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAd} message AdGroupAd message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAd.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAd message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAd} AdGroupAd
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAd.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAd();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 9:
                                        m.ad_group = r.string();
                                        break;
                                    case 5:
                                        m.ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.policy_summary = $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.ad_strength = r.int32();
                                        break;
                                    case 13:
                                        if (!(m.action_items && m.action_items.length))
                                            m.action_items = [];
                                        m.action_items.push(r.string());
                                        break;
                                    case 10:
                                        if (!(m.labels && m.labels.length))
                                            m.labels = [];
                                        m.labels.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAd message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAd} AdGroupAd
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAd.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAd message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAd.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.ad != null && m.hasOwnProperty("ad")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.ad);
                                        if (e)
                                            return "ad." + e;
                                    }
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary.verify(m.policy_summary);
                                        if (e)
                                            return "policy_summary." + e;
                                    }
                                }
                                if (m.ad_strength != null && m.hasOwnProperty("ad_strength")) {
                                    switch (m.ad_strength) {
                                    default:
                                        return "ad_strength: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.action_items != null && m.hasOwnProperty("action_items")) {
                                    if (!Array.isArray(m.action_items))
                                        return "action_items: array expected";
                                    for (var i = 0; i < m.action_items.length; ++i) {
                                        if (!$util.isString(m.action_items[i]))
                                            return "action_items: string[] expected";
                                    }
                                }
                                if (m.labels != null && m.hasOwnProperty("labels")) {
                                    if (!Array.isArray(m.labels))
                                        return "labels: array expected";
                                    for (var i = 0; i < m.labels.length; ++i) {
                                        if (!$util.isString(m.labels[i]))
                                            return "labels: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAd message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAd} AdGroupAd
                             */
                            AdGroupAd.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAd)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAd();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "PAUSED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.ad != null) {
                                    if (typeof d.ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAd.ad: object expected");
                                    m.ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.ad);
                                }
                                if (d.policy_summary != null) {
                                    if (typeof d.policy_summary !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAd.policy_summary: object expected");
                                    m.policy_summary = $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary.fromObject(d.policy_summary);
                                }
                                switch (d.ad_strength) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_strength = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_strength = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.ad_strength = 2;
                                    break;
                                case "NO_ADS":
                                case 3:
                                    m.ad_strength = 3;
                                    break;
                                case "POOR":
                                case 4:
                                    m.ad_strength = 4;
                                    break;
                                case "AVERAGE":
                                case 5:
                                    m.ad_strength = 5;
                                    break;
                                case "GOOD":
                                case 6:
                                    m.ad_strength = 6;
                                    break;
                                case "EXCELLENT":
                                case 7:
                                    m.ad_strength = 7;
                                    break;
                                }
                                if (d.action_items) {
                                    if (!Array.isArray(d.action_items))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAd.action_items: array expected");
                                    m.action_items = [];
                                    for (var i = 0; i < d.action_items.length; ++i) {
                                        m.action_items[i] = String(d.action_items[i]);
                                    }
                                }
                                if (d.labels) {
                                    if (!Array.isArray(d.labels))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAd.labels: array expected");
                                    m.labels = [];
                                    for (var i = 0; i < d.labels.length; ++i) {
                                        m.labels[i] = String(d.labels[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAd message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAd} m AdGroupAd
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAd.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.labels = [];
                                    d.action_items = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.ad = null;
                                    d.policy_summary = null;
                                    d.ad_strength = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AdGroupAdStatusEnum.AdGroupAdStatus[m.status] : m.status;
                                }
                                if (m.ad != null && m.hasOwnProperty("ad")) {
                                    d.ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.ad, o);
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    d.policy_summary = $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary.toObject(m.policy_summary, o);
                                }
                                if (m.ad_strength != null && m.hasOwnProperty("ad_strength")) {
                                    d.ad_strength = o.enums === String ? $root.google.ads.googleads.v11.enums.AdStrengthEnum.AdStrength[m.ad_strength] : m.ad_strength;
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                if (m.labels && m.labels.length) {
                                    d.labels = [];
                                    for (var j = 0; j < m.labels.length; ++j) {
                                        d.labels[j] = m.labels[j];
                                    }
                                }
                                if (m.action_items && m.action_items.length) {
                                    d.action_items = [];
                                    for (var j = 0; j < m.action_items.length; ++j) {
                                        d.action_items[j] = m.action_items[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAd to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAd
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAd.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAd;
                        })();
    
                        resources.AdGroupAdPolicySummary = (function() {
    
                            /**
                             * Properties of an AdGroupAdPolicySummary.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAdPolicySummary
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>|null} [policy_topic_entries] AdGroupAdPolicySummary policy_topic_entries
                             * @property {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus|null} [review_status] AdGroupAdPolicySummary review_status
                             * @property {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus|null} [approval_status] AdGroupAdPolicySummary approval_status
                             */
    
                            /**
                             * Constructs a new AdGroupAdPolicySummary.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAdPolicySummary.
                             * @implements IAdGroupAdPolicySummary
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdPolicySummary=} [p] Properties to set
                             */
                            function AdGroupAdPolicySummary(p) {
                                this.policy_topic_entries = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAdPolicySummary policy_topic_entries.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>} policy_topic_entries
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @instance
                             */
                            AdGroupAdPolicySummary.prototype.policy_topic_entries = $util.emptyArray;
    
                            /**
                             * AdGroupAdPolicySummary review_status.
                             * @member {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus} review_status
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @instance
                             */
                            AdGroupAdPolicySummary.prototype.review_status = 0;
    
                            /**
                             * AdGroupAdPolicySummary approval_status.
                             * @member {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus} approval_status
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @instance
                             */
                            AdGroupAdPolicySummary.prototype.approval_status = 0;
    
                            /**
                             * Creates a new AdGroupAdPolicySummary instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdPolicySummary=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdPolicySummary} AdGroupAdPolicySummary instance
                             */
                            AdGroupAdPolicySummary.create = function create(properties) {
                                return new AdGroupAdPolicySummary(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdPolicySummary message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdPolicySummary.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdPolicySummary} m AdGroupAdPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdPolicySummary.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.policy_topic_entries != null && m.policy_topic_entries.length) {
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEntry.encode(m.policy_topic_entries[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.review_status != null && Object.hasOwnProperty.call(m, "review_status"))
                                    w.uint32(16).int32(m.review_status);
                                if (m.approval_status != null && Object.hasOwnProperty.call(m, "approval_status"))
                                    w.uint32(24).int32(m.approval_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdPolicySummary message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdPolicySummary.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdPolicySummary} message AdGroupAdPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdPolicySummary.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdPolicySummary message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdPolicySummary} AdGroupAdPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdPolicySummary.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.policy_topic_entries && m.policy_topic_entries.length))
                                            m.policy_topic_entries = [];
                                        m.policy_topic_entries.push($root.google.ads.googleads.v11.common.PolicyTopicEntry.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.review_status = r.int32();
                                        break;
                                    case 3:
                                        m.approval_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdPolicySummary message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdPolicySummary} AdGroupAdPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdPolicySummary.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdPolicySummary message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdPolicySummary.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.policy_topic_entries != null && m.hasOwnProperty("policy_topic_entries")) {
                                    if (!Array.isArray(m.policy_topic_entries))
                                        return "policy_topic_entries: array expected";
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEntry.verify(m.policy_topic_entries[i]);
                                            if (e)
                                                return "policy_topic_entries." + e;
                                        }
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    switch (m.review_status) {
                                    default:
                                        return "review_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    switch (m.approval_status) {
                                    default:
                                        return "approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdPolicySummary message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdPolicySummary} AdGroupAdPolicySummary
                             */
                            AdGroupAdPolicySummary.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAdPolicySummary();
                                if (d.policy_topic_entries) {
                                    if (!Array.isArray(d.policy_topic_entries))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAdPolicySummary.policy_topic_entries: array expected");
                                    m.policy_topic_entries = [];
                                    for (var i = 0; i < d.policy_topic_entries.length; ++i) {
                                        if (typeof d.policy_topic_entries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.AdGroupAdPolicySummary.policy_topic_entries: object expected");
                                        m.policy_topic_entries[i] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.fromObject(d.policy_topic_entries[i]);
                                    }
                                }
                                switch (d.review_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.review_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.review_status = 1;
                                    break;
                                case "REVIEW_IN_PROGRESS":
                                case 2:
                                    m.review_status = 2;
                                    break;
                                case "REVIEWED":
                                case 3:
                                    m.review_status = 3;
                                    break;
                                case "UNDER_APPEAL":
                                case 4:
                                    m.review_status = 4;
                                    break;
                                case "ELIGIBLE_MAY_SERVE":
                                case 5:
                                    m.review_status = 5;
                                    break;
                                }
                                switch (d.approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approval_status = 1;
                                    break;
                                case "DISAPPROVED":
                                case 2:
                                    m.approval_status = 2;
                                    break;
                                case "APPROVED_LIMITED":
                                case 3:
                                    m.approval_status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.approval_status = 4;
                                    break;
                                case "AREA_OF_INTEREST_ONLY":
                                case 5:
                                    m.approval_status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdPolicySummary message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAdPolicySummary} m AdGroupAdPolicySummary
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdPolicySummary.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.policy_topic_entries = [];
                                }
                                if (o.defaults) {
                                    d.review_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.policy_topic_entries && m.policy_topic_entries.length) {
                                    d.policy_topic_entries = [];
                                    for (var j = 0; j < m.policy_topic_entries.length; ++j) {
                                        d.policy_topic_entries[j] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.toObject(m.policy_topic_entries[j], o);
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    d.review_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus[m.review_status] : m.review_status;
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    d.approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus[m.approval_status] : m.approval_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAdPolicySummary to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdPolicySummary
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdPolicySummary.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAdPolicySummary;
                        })();
    
                        resources.AdGroupAdAssetCombinationView = (function() {
    
                            /**
                             * Properties of an AdGroupAdAssetCombinationView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAdAssetCombinationView
                             * @property {string|null} [resource_name] AdGroupAdAssetCombinationView resource_name
                             * @property {Array.<google.ads.googleads.v11.common.IAssetUsage>|null} [served_assets] AdGroupAdAssetCombinationView served_assets
                             * @property {boolean|null} [enabled] AdGroupAdAssetCombinationView enabled
                             */
    
                            /**
                             * Constructs a new AdGroupAdAssetCombinationView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAdAssetCombinationView.
                             * @implements IAdGroupAdAssetCombinationView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetCombinationView=} [p] Properties to set
                             */
                            function AdGroupAdAssetCombinationView(p) {
                                this.served_assets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAdAssetCombinationView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @instance
                             */
                            AdGroupAdAssetCombinationView.prototype.resource_name = "";
    
                            /**
                             * AdGroupAdAssetCombinationView served_assets.
                             * @member {Array.<google.ads.googleads.v11.common.IAssetUsage>} served_assets
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @instance
                             */
                            AdGroupAdAssetCombinationView.prototype.served_assets = $util.emptyArray;
    
                            /**
                             * AdGroupAdAssetCombinationView enabled.
                             * @member {boolean|null|undefined} enabled
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @instance
                             */
                            AdGroupAdAssetCombinationView.prototype.enabled = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupAdAssetCombinationView _enabled.
                             * @member {"enabled"|undefined} _enabled
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdAssetCombinationView.prototype, "_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupAdAssetCombinationView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetCombinationView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView} AdGroupAdAssetCombinationView instance
                             */
                            AdGroupAdAssetCombinationView.create = function create(properties) {
                                return new AdGroupAdAssetCombinationView(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdAssetCombinationView message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetCombinationView} m AdGroupAdAssetCombinationView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdAssetCombinationView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.served_assets != null && m.served_assets.length) {
                                    for (var i = 0; i < m.served_assets.length; ++i)
                                        $root.google.ads.googleads.v11.common.AssetUsage.encode(m.served_assets[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.enabled != null && Object.hasOwnProperty.call(m, "enabled"))
                                    w.uint32(24).bool(m.enabled);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdAssetCombinationView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetCombinationView} message AdGroupAdAssetCombinationView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdAssetCombinationView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdAssetCombinationView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView} AdGroupAdAssetCombinationView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdAssetCombinationView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        if (!(m.served_assets && m.served_assets.length))
                                            m.served_assets = [];
                                        m.served_assets.push($root.google.ads.googleads.v11.common.AssetUsage.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.enabled = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdAssetCombinationView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView} AdGroupAdAssetCombinationView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdAssetCombinationView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdAssetCombinationView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdAssetCombinationView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.served_assets != null && m.hasOwnProperty("served_assets")) {
                                    if (!Array.isArray(m.served_assets))
                                        return "served_assets: array expected";
                                    for (var i = 0; i < m.served_assets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AssetUsage.verify(m.served_assets[i]);
                                            if (e)
                                                return "served_assets." + e;
                                        }
                                    }
                                }
                                if (m.enabled != null && m.hasOwnProperty("enabled")) {
                                    p._enabled = 1;
                                    if (typeof m.enabled !== "boolean")
                                        return "enabled: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdAssetCombinationView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView} AdGroupAdAssetCombinationView
                             */
                            AdGroupAdAssetCombinationView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.served_assets) {
                                    if (!Array.isArray(d.served_assets))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.served_assets: array expected");
                                    m.served_assets = [];
                                    for (var i = 0; i < d.served_assets.length; ++i) {
                                        if (typeof d.served_assets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.served_assets: object expected");
                                        m.served_assets[i] = $root.google.ads.googleads.v11.common.AssetUsage.fromObject(d.served_assets[i]);
                                    }
                                }
                                if (d.enabled != null) {
                                    m.enabled = Boolean(d.enabled);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdAssetCombinationView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView} m AdGroupAdAssetCombinationView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdAssetCombinationView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.served_assets = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.served_assets && m.served_assets.length) {
                                    d.served_assets = [];
                                    for (var j = 0; j < m.served_assets.length; ++j) {
                                        d.served_assets[j] = $root.google.ads.googleads.v11.common.AssetUsage.toObject(m.served_assets[j], o);
                                    }
                                }
                                if (m.enabled != null && m.hasOwnProperty("enabled")) {
                                    d.enabled = m.enabled;
                                    if (o.oneofs)
                                        d._enabled = "enabled";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAdAssetCombinationView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdAssetCombinationView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAdAssetCombinationView;
                        })();
    
                        resources.AdGroupAdAssetView = (function() {
    
                            /**
                             * Properties of an AdGroupAdAssetView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAdAssetView
                             * @property {string|null} [resource_name] AdGroupAdAssetView resource_name
                             * @property {string|null} [ad_group_ad] AdGroupAdAssetView ad_group_ad
                             * @property {string|null} [asset] AdGroupAdAssetView asset
                             * @property {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType|null} [field_type] AdGroupAdAssetView field_type
                             * @property {boolean|null} [enabled] AdGroupAdAssetView enabled
                             * @property {google.ads.googleads.v11.resources.IAdGroupAdAssetPolicySummary|null} [policy_summary] AdGroupAdAssetView policy_summary
                             * @property {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel|null} [performance_label] AdGroupAdAssetView performance_label
                             */
    
                            /**
                             * Constructs a new AdGroupAdAssetView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAdAssetView.
                             * @implements IAdGroupAdAssetView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetView=} [p] Properties to set
                             */
                            function AdGroupAdAssetView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAdAssetView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            AdGroupAdAssetView.prototype.resource_name = "";
    
                            /**
                             * AdGroupAdAssetView ad_group_ad.
                             * @member {string|null|undefined} ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            AdGroupAdAssetView.prototype.ad_group_ad = null;
    
                            /**
                             * AdGroupAdAssetView asset.
                             * @member {string|null|undefined} asset
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            AdGroupAdAssetView.prototype.asset = null;
    
                            /**
                             * AdGroupAdAssetView field_type.
                             * @member {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType} field_type
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            AdGroupAdAssetView.prototype.field_type = 0;
    
                            /**
                             * AdGroupAdAssetView enabled.
                             * @member {boolean|null|undefined} enabled
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            AdGroupAdAssetView.prototype.enabled = null;
    
                            /**
                             * AdGroupAdAssetView policy_summary.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAdAssetPolicySummary|null|undefined} policy_summary
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            AdGroupAdAssetView.prototype.policy_summary = null;
    
                            /**
                             * AdGroupAdAssetView performance_label.
                             * @member {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel} performance_label
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            AdGroupAdAssetView.prototype.performance_label = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupAdAssetView _ad_group_ad.
                             * @member {"ad_group_ad"|undefined} _ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdAssetView.prototype, "_ad_group_ad", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_ad"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupAdAssetView _asset.
                             * @member {"asset"|undefined} _asset
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdAssetView.prototype, "_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupAdAssetView _enabled.
                             * @member {"enabled"|undefined} _enabled
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdAssetView.prototype, "_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupAdAssetView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetView} AdGroupAdAssetView instance
                             */
                            AdGroupAdAssetView.create = function create(properties) {
                                return new AdGroupAdAssetView(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdAssetView message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdAssetView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetView} m AdGroupAdAssetView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdAssetView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.field_type != null && Object.hasOwnProperty.call(m, "field_type"))
                                    w.uint32(16).int32(m.field_type);
                                if (m.policy_summary != null && Object.hasOwnProperty.call(m, "policy_summary"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.encode(m.policy_summary, w.uint32(26).fork()).ldelim();
                                if (m.performance_label != null && Object.hasOwnProperty.call(m, "performance_label"))
                                    w.uint32(32).int32(m.performance_label);
                                if (m.enabled != null && Object.hasOwnProperty.call(m, "enabled"))
                                    w.uint32(64).bool(m.enabled);
                                if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                    w.uint32(74).string(m.ad_group_ad);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(82).string(m.asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdAssetView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdAssetView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetView} message AdGroupAdAssetView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdAssetView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdAssetView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetView} AdGroupAdAssetView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdAssetView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAdAssetView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.ad_group_ad = r.string();
                                        break;
                                    case 10:
                                        m.asset = r.string();
                                        break;
                                    case 2:
                                        m.field_type = r.int32();
                                        break;
                                    case 8:
                                        m.enabled = r.bool();
                                        break;
                                    case 3:
                                        m.policy_summary = $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.performance_label = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdAssetView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetView} AdGroupAdAssetView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdAssetView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdAssetView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdAssetView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    p._ad_group_ad = 1;
                                    if (!$util.isString(m.ad_group_ad))
                                        return "ad_group_ad: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    p._asset = 1;
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    switch (m.field_type) {
                                    default:
                                        return "field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 24:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.enabled != null && m.hasOwnProperty("enabled")) {
                                    p._enabled = 1;
                                    if (typeof m.enabled !== "boolean")
                                        return "enabled: boolean expected";
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.verify(m.policy_summary);
                                        if (e)
                                            return "policy_summary." + e;
                                    }
                                }
                                if (m.performance_label != null && m.hasOwnProperty("performance_label")) {
                                    switch (m.performance_label) {
                                    default:
                                        return "performance_label: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdAssetView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetView} AdGroupAdAssetView
                             */
                            AdGroupAdAssetView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAdAssetView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAdAssetView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_ad != null) {
                                    m.ad_group_ad = String(d.ad_group_ad);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                switch (d.field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_type = 1;
                                    break;
                                case "HEADLINE":
                                case 2:
                                    m.field_type = 2;
                                    break;
                                case "DESCRIPTION":
                                case 3:
                                    m.field_type = 3;
                                    break;
                                case "MANDATORY_AD_TEXT":
                                case 4:
                                    m.field_type = 4;
                                    break;
                                case "MARKETING_IMAGE":
                                case 5:
                                    m.field_type = 5;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 6:
                                    m.field_type = 6;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 7:
                                    m.field_type = 7;
                                    break;
                                case "BOOK_ON_GOOGLE":
                                case 8:
                                    m.field_type = 8;
                                    break;
                                case "LEAD_FORM":
                                case 9:
                                    m.field_type = 9;
                                    break;
                                case "PROMOTION":
                                case 10:
                                    m.field_type = 10;
                                    break;
                                case "CALLOUT":
                                case 11:
                                    m.field_type = 11;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 12:
                                    m.field_type = 12;
                                    break;
                                case "SITELINK":
                                case 13:
                                    m.field_type = 13;
                                    break;
                                case "MOBILE_APP":
                                case 14:
                                    m.field_type = 14;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.field_type = 15;
                                    break;
                                case "CALL":
                                case 16:
                                    m.field_type = 16;
                                    break;
                                case "PRICE":
                                case 24:
                                    m.field_type = 24;
                                    break;
                                case "LONG_HEADLINE":
                                case 17:
                                    m.field_type = 17;
                                    break;
                                case "BUSINESS_NAME":
                                case 18:
                                    m.field_type = 18;
                                    break;
                                case "SQUARE_MARKETING_IMAGE":
                                case 19:
                                    m.field_type = 19;
                                    break;
                                case "PORTRAIT_MARKETING_IMAGE":
                                case 20:
                                    m.field_type = 20;
                                    break;
                                case "LOGO":
                                case 21:
                                    m.field_type = 21;
                                    break;
                                case "LANDSCAPE_LOGO":
                                case 22:
                                    m.field_type = 22;
                                    break;
                                case "VIDEO":
                                case 23:
                                    m.field_type = 23;
                                    break;
                                case "CALL_TO_ACTION_SELECTION":
                                case 25:
                                    m.field_type = 25;
                                    break;
                                }
                                if (d.enabled != null) {
                                    m.enabled = Boolean(d.enabled);
                                }
                                if (d.policy_summary != null) {
                                    if (typeof d.policy_summary !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAdAssetView.policy_summary: object expected");
                                    m.policy_summary = $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.fromObject(d.policy_summary);
                                }
                                switch (d.performance_label) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.performance_label = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.performance_label = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.performance_label = 2;
                                    break;
                                case "LEARNING":
                                case 3:
                                    m.performance_label = 3;
                                    break;
                                case "LOW":
                                case 4:
                                    m.performance_label = 4;
                                    break;
                                case "GOOD":
                                case 5:
                                    m.performance_label = 5;
                                    break;
                                case "BEST":
                                case 6:
                                    m.performance_label = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdAssetView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAdAssetView} m AdGroupAdAssetView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdAssetView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.policy_summary = null;
                                    d.performance_label = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    d.field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.field_type] : m.field_type;
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    d.policy_summary = $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.toObject(m.policy_summary, o);
                                }
                                if (m.performance_label != null && m.hasOwnProperty("performance_label")) {
                                    d.performance_label = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel[m.performance_label] : m.performance_label;
                                }
                                if (m.enabled != null && m.hasOwnProperty("enabled")) {
                                    d.enabled = m.enabled;
                                    if (o.oneofs)
                                        d._enabled = "enabled";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    d.ad_group_ad = m.ad_group_ad;
                                    if (o.oneofs)
                                        d._ad_group_ad = "ad_group_ad";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                    if (o.oneofs)
                                        d._asset = "asset";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAdAssetView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdAssetView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAdAssetView;
                        })();
    
                        resources.AdGroupAdAssetPolicySummary = (function() {
    
                            /**
                             * Properties of an AdGroupAdAssetPolicySummary.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAdAssetPolicySummary
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>|null} [policy_topic_entries] AdGroupAdAssetPolicySummary policy_topic_entries
                             * @property {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus|null} [review_status] AdGroupAdAssetPolicySummary review_status
                             * @property {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus|null} [approval_status] AdGroupAdAssetPolicySummary approval_status
                             */
    
                            /**
                             * Constructs a new AdGroupAdAssetPolicySummary.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAdAssetPolicySummary.
                             * @implements IAdGroupAdAssetPolicySummary
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetPolicySummary=} [p] Properties to set
                             */
                            function AdGroupAdAssetPolicySummary(p) {
                                this.policy_topic_entries = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAdAssetPolicySummary policy_topic_entries.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>} policy_topic_entries
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @instance
                             */
                            AdGroupAdAssetPolicySummary.prototype.policy_topic_entries = $util.emptyArray;
    
                            /**
                             * AdGroupAdAssetPolicySummary review_status.
                             * @member {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus} review_status
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @instance
                             */
                            AdGroupAdAssetPolicySummary.prototype.review_status = 0;
    
                            /**
                             * AdGroupAdAssetPolicySummary approval_status.
                             * @member {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus} approval_status
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @instance
                             */
                            AdGroupAdAssetPolicySummary.prototype.approval_status = 0;
    
                            /**
                             * Creates a new AdGroupAdAssetPolicySummary instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetPolicySummary=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary} AdGroupAdAssetPolicySummary instance
                             */
                            AdGroupAdAssetPolicySummary.create = function create(properties) {
                                return new AdGroupAdAssetPolicySummary(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdAssetPolicySummary message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetPolicySummary} m AdGroupAdAssetPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdAssetPolicySummary.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.policy_topic_entries != null && m.policy_topic_entries.length) {
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEntry.encode(m.policy_topic_entries[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.review_status != null && Object.hasOwnProperty.call(m, "review_status"))
                                    w.uint32(16).int32(m.review_status);
                                if (m.approval_status != null && Object.hasOwnProperty.call(m, "approval_status"))
                                    w.uint32(24).int32(m.approval_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdAssetPolicySummary message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdAssetPolicySummary} message AdGroupAdAssetPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdAssetPolicySummary.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdAssetPolicySummary message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary} AdGroupAdAssetPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdAssetPolicySummary.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.policy_topic_entries && m.policy_topic_entries.length))
                                            m.policy_topic_entries = [];
                                        m.policy_topic_entries.push($root.google.ads.googleads.v11.common.PolicyTopicEntry.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.review_status = r.int32();
                                        break;
                                    case 3:
                                        m.approval_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdAssetPolicySummary message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary} AdGroupAdAssetPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdAssetPolicySummary.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdAssetPolicySummary message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdAssetPolicySummary.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.policy_topic_entries != null && m.hasOwnProperty("policy_topic_entries")) {
                                    if (!Array.isArray(m.policy_topic_entries))
                                        return "policy_topic_entries: array expected";
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEntry.verify(m.policy_topic_entries[i]);
                                            if (e)
                                                return "policy_topic_entries." + e;
                                        }
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    switch (m.review_status) {
                                    default:
                                        return "review_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    switch (m.approval_status) {
                                    default:
                                        return "approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdAssetPolicySummary message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary} AdGroupAdAssetPolicySummary
                             */
                            AdGroupAdAssetPolicySummary.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary();
                                if (d.policy_topic_entries) {
                                    if (!Array.isArray(d.policy_topic_entries))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.policy_topic_entries: array expected");
                                    m.policy_topic_entries = [];
                                    for (var i = 0; i < d.policy_topic_entries.length; ++i) {
                                        if (typeof d.policy_topic_entries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary.policy_topic_entries: object expected");
                                        m.policy_topic_entries[i] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.fromObject(d.policy_topic_entries[i]);
                                    }
                                }
                                switch (d.review_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.review_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.review_status = 1;
                                    break;
                                case "REVIEW_IN_PROGRESS":
                                case 2:
                                    m.review_status = 2;
                                    break;
                                case "REVIEWED":
                                case 3:
                                    m.review_status = 3;
                                    break;
                                case "UNDER_APPEAL":
                                case 4:
                                    m.review_status = 4;
                                    break;
                                case "ELIGIBLE_MAY_SERVE":
                                case 5:
                                    m.review_status = 5;
                                    break;
                                }
                                switch (d.approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approval_status = 1;
                                    break;
                                case "DISAPPROVED":
                                case 2:
                                    m.approval_status = 2;
                                    break;
                                case "APPROVED_LIMITED":
                                case 3:
                                    m.approval_status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.approval_status = 4;
                                    break;
                                case "AREA_OF_INTEREST_ONLY":
                                case 5:
                                    m.approval_status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdAssetPolicySummary message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary} m AdGroupAdAssetPolicySummary
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdAssetPolicySummary.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.policy_topic_entries = [];
                                }
                                if (o.defaults) {
                                    d.review_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.policy_topic_entries && m.policy_topic_entries.length) {
                                    d.policy_topic_entries = [];
                                    for (var j = 0; j < m.policy_topic_entries.length; ++j) {
                                        d.policy_topic_entries[j] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.toObject(m.policy_topic_entries[j], o);
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    d.review_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus[m.review_status] : m.review_status;
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    d.approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus[m.approval_status] : m.approval_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAdAssetPolicySummary to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdAssetPolicySummary
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdAssetPolicySummary.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAdAssetPolicySummary;
                        })();
    
                        resources.AdGroupAdLabel = (function() {
    
                            /**
                             * Properties of an AdGroupAdLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAdLabel
                             * @property {string|null} [resource_name] AdGroupAdLabel resource_name
                             * @property {string|null} [ad_group_ad] AdGroupAdLabel ad_group_ad
                             * @property {string|null} [label] AdGroupAdLabel label
                             */
    
                            /**
                             * Constructs a new AdGroupAdLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAdLabel.
                             * @implements IAdGroupAdLabel
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdLabel=} [p] Properties to set
                             */
                            function AdGroupAdLabel(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAdLabel resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @instance
                             */
                            AdGroupAdLabel.prototype.resource_name = "";
    
                            /**
                             * AdGroupAdLabel ad_group_ad.
                             * @member {string|null|undefined} ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @instance
                             */
                            AdGroupAdLabel.prototype.ad_group_ad = null;
    
                            /**
                             * AdGroupAdLabel label.
                             * @member {string|null|undefined} label
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @instance
                             */
                            AdGroupAdLabel.prototype.label = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupAdLabel _ad_group_ad.
                             * @member {"ad_group_ad"|undefined} _ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdLabel.prototype, "_ad_group_ad", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_ad"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupAdLabel _label.
                             * @member {"label"|undefined} _label
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdLabel.prototype, "_label", {
                                get: $util.oneOfGetter($oneOfFields = ["label"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupAdLabel instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdLabel=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdLabel} AdGroupAdLabel instance
                             */
                            AdGroupAdLabel.create = function create(properties) {
                                return new AdGroupAdLabel(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdLabel message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdLabel.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdLabel} m AdGroupAdLabel message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdLabel.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                    w.uint32(34).string(m.ad_group_ad);
                                if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                                    w.uint32(42).string(m.label);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdLabel message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAdLabel.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAdLabel} message AdGroupAdLabel message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdLabel.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdLabel message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdLabel} AdGroupAdLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdLabel.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAdLabel();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.ad_group_ad = r.string();
                                        break;
                                    case 5:
                                        m.label = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdLabel message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdLabel} AdGroupAdLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdLabel.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdLabel message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdLabel.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    p._ad_group_ad = 1;
                                    if (!$util.isString(m.ad_group_ad))
                                        return "ad_group_ad: string expected";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    p._label = 1;
                                    if (!$util.isString(m.label))
                                        return "label: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdLabel message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAdLabel} AdGroupAdLabel
                             */
                            AdGroupAdLabel.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAdLabel)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAdLabel();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_ad != null) {
                                    m.ad_group_ad = String(d.ad_group_ad);
                                }
                                if (d.label != null) {
                                    m.label = String(d.label);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdLabel message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAdLabel} m AdGroupAdLabel
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdLabel.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    d.ad_group_ad = m.ad_group_ad;
                                    if (o.oneofs)
                                        d._ad_group_ad = "ad_group_ad";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    d.label = m.label;
                                    if (o.oneofs)
                                        d._label = "label";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAdLabel to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAdLabel
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdLabel.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAdLabel;
                        })();
    
                        resources.AdGroupAsset = (function() {
    
                            /**
                             * Properties of an AdGroupAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAsset
                             * @property {string|null} [resource_name] AdGroupAsset resource_name
                             * @property {string|null} [ad_group] AdGroupAsset ad_group
                             * @property {string|null} [asset] AdGroupAsset asset
                             * @property {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType|null} [field_type] AdGroupAsset field_type
                             * @property {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource|null} [source] AdGroupAsset source
                             * @property {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus|null} [status] AdGroupAsset status
                             */
    
                            /**
                             * Constructs a new AdGroupAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAsset.
                             * @implements IAdGroupAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAsset=} [p] Properties to set
                             */
                            function AdGroupAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAsset resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @instance
                             */
                            AdGroupAsset.prototype.resource_name = "";
    
                            /**
                             * AdGroupAsset ad_group.
                             * @member {string} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @instance
                             */
                            AdGroupAsset.prototype.ad_group = "";
    
                            /**
                             * AdGroupAsset asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @instance
                             */
                            AdGroupAsset.prototype.asset = "";
    
                            /**
                             * AdGroupAsset field_type.
                             * @member {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType} field_type
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @instance
                             */
                            AdGroupAsset.prototype.field_type = 0;
    
                            /**
                             * AdGroupAsset source.
                             * @member {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource} source
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @instance
                             */
                            AdGroupAsset.prototype.source = 0;
    
                            /**
                             * AdGroupAsset status.
                             * @member {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @instance
                             */
                            AdGroupAsset.prototype.status = 0;
    
                            /**
                             * Creates a new AdGroupAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAsset} AdGroupAsset instance
                             */
                            AdGroupAsset.create = function create(properties) {
                                return new AdGroupAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAsset message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAsset} m AdGroupAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(18).string(m.ad_group);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(26).string(m.asset);
                                if (m.field_type != null && Object.hasOwnProperty.call(m, "field_type"))
                                    w.uint32(32).int32(m.field_type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.source != null && Object.hasOwnProperty.call(m, "source"))
                                    w.uint32(48).int32(m.source);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAsset} message AdGroupAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAsset} AdGroupAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group = r.string();
                                        break;
                                    case 3:
                                        m.asset = r.string();
                                        break;
                                    case 4:
                                        m.field_type = r.int32();
                                        break;
                                    case 6:
                                        m.source = r.int32();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAsset} AdGroupAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    switch (m.field_type) {
                                    default:
                                        return "field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 24:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    switch (m.source) {
                                    default:
                                        return "source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAsset} AdGroupAsset
                             */
                            AdGroupAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAsset();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                switch (d.field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_type = 1;
                                    break;
                                case "HEADLINE":
                                case 2:
                                    m.field_type = 2;
                                    break;
                                case "DESCRIPTION":
                                case 3:
                                    m.field_type = 3;
                                    break;
                                case "MANDATORY_AD_TEXT":
                                case 4:
                                    m.field_type = 4;
                                    break;
                                case "MARKETING_IMAGE":
                                case 5:
                                    m.field_type = 5;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 6:
                                    m.field_type = 6;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 7:
                                    m.field_type = 7;
                                    break;
                                case "BOOK_ON_GOOGLE":
                                case 8:
                                    m.field_type = 8;
                                    break;
                                case "LEAD_FORM":
                                case 9:
                                    m.field_type = 9;
                                    break;
                                case "PROMOTION":
                                case 10:
                                    m.field_type = 10;
                                    break;
                                case "CALLOUT":
                                case 11:
                                    m.field_type = 11;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 12:
                                    m.field_type = 12;
                                    break;
                                case "SITELINK":
                                case 13:
                                    m.field_type = 13;
                                    break;
                                case "MOBILE_APP":
                                case 14:
                                    m.field_type = 14;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.field_type = 15;
                                    break;
                                case "CALL":
                                case 16:
                                    m.field_type = 16;
                                    break;
                                case "PRICE":
                                case 24:
                                    m.field_type = 24;
                                    break;
                                case "LONG_HEADLINE":
                                case 17:
                                    m.field_type = 17;
                                    break;
                                case "BUSINESS_NAME":
                                case 18:
                                    m.field_type = 18;
                                    break;
                                case "SQUARE_MARKETING_IMAGE":
                                case 19:
                                    m.field_type = 19;
                                    break;
                                case "PORTRAIT_MARKETING_IMAGE":
                                case 20:
                                    m.field_type = 20;
                                    break;
                                case "LOGO":
                                case 21:
                                    m.field_type = 21;
                                    break;
                                case "LANDSCAPE_LOGO":
                                case 22:
                                    m.field_type = 22;
                                    break;
                                case "VIDEO":
                                case 23:
                                    m.field_type = 23;
                                    break;
                                case "CALL_TO_ACTION_SELECTION":
                                case 25:
                                    m.field_type = 25;
                                    break;
                                }
                                switch (d.source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.source = 1;
                                    break;
                                case "ADVERTISER":
                                case 2:
                                    m.source = 2;
                                    break;
                                case "AUTOMATICALLY_CREATED":
                                case 3:
                                    m.source = 3;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PAUSED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAsset} m AdGroupAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group = "";
                                    d.asset = "";
                                    d.field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.source = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    d.field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.field_type] : m.field_type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus[m.status] : m.status;
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    d.source = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource[m.source] : m.source;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAsset;
                        })();
    
                        resources.AdGroupAudienceView = (function() {
    
                            /**
                             * Properties of an AdGroupAudienceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupAudienceView
                             * @property {string|null} [resource_name] AdGroupAudienceView resource_name
                             */
    
                            /**
                             * Constructs a new AdGroupAudienceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupAudienceView.
                             * @implements IAdGroupAudienceView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupAudienceView=} [p] Properties to set
                             */
                            function AdGroupAudienceView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAudienceView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @instance
                             */
                            AdGroupAudienceView.prototype.resource_name = "";
    
                            /**
                             * Creates a new AdGroupAudienceView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAudienceView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupAudienceView} AdGroupAudienceView instance
                             */
                            AdGroupAudienceView.create = function create(properties) {
                                return new AdGroupAudienceView(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAudienceView message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAudienceView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAudienceView} m AdGroupAudienceView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAudienceView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAudienceView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupAudienceView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupAudienceView} message AdGroupAudienceView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAudienceView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAudienceView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupAudienceView} AdGroupAudienceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAudienceView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupAudienceView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAudienceView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupAudienceView} AdGroupAudienceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAudienceView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAudienceView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAudienceView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAudienceView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupAudienceView} AdGroupAudienceView
                             */
                            AdGroupAudienceView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupAudienceView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupAudienceView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAudienceView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupAudienceView} m AdGroupAudienceView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAudienceView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAudienceView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupAudienceView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAudienceView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAudienceView;
                        })();
    
                        resources.AdGroupBidModifier = (function() {
    
                            /**
                             * Properties of an AdGroupBidModifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupBidModifier
                             * @property {string|null} [resource_name] AdGroupBidModifier resource_name
                             * @property {string|null} [ad_group] AdGroupBidModifier ad_group
                             * @property {number|null} [criterion_id] AdGroupBidModifier criterion_id
                             * @property {number|null} [bid_modifier] AdGroupBidModifier bid_modifier
                             * @property {string|null} [base_ad_group] AdGroupBidModifier base_ad_group
                             * @property {google.ads.googleads.v11.enums.BidModifierSourceEnum.BidModifierSource|null} [bid_modifier_source] AdGroupBidModifier bid_modifier_source
                             * @property {google.ads.googleads.v11.common.IHotelDateSelectionTypeInfo|null} [hotel_date_selection_type] AdGroupBidModifier hotel_date_selection_type
                             * @property {google.ads.googleads.v11.common.IHotelAdvanceBookingWindowInfo|null} [hotel_advance_booking_window] AdGroupBidModifier hotel_advance_booking_window
                             * @property {google.ads.googleads.v11.common.IHotelLengthOfStayInfo|null} [hotel_length_of_stay] AdGroupBidModifier hotel_length_of_stay
                             * @property {google.ads.googleads.v11.common.IHotelCheckInDayInfo|null} [hotel_check_in_day] AdGroupBidModifier hotel_check_in_day
                             * @property {google.ads.googleads.v11.common.IDeviceInfo|null} [device] AdGroupBidModifier device
                             * @property {google.ads.googleads.v11.common.IPreferredContentInfo|null} [preferred_content] AdGroupBidModifier preferred_content
                             * @property {google.ads.googleads.v11.common.IHotelCheckInDateRangeInfo|null} [hotel_check_in_date_range] AdGroupBidModifier hotel_check_in_date_range
                             */
    
                            /**
                             * Constructs a new AdGroupBidModifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupBidModifier.
                             * @implements IAdGroupBidModifier
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupBidModifier=} [p] Properties to set
                             */
                            function AdGroupBidModifier(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupBidModifier resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.resource_name = "";
    
                            /**
                             * AdGroupBidModifier ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.ad_group = null;
    
                            /**
                             * AdGroupBidModifier criterion_id.
                             * @member {number|null|undefined} criterion_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.criterion_id = null;
    
                            /**
                             * AdGroupBidModifier bid_modifier.
                             * @member {number|null|undefined} bid_modifier
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.bid_modifier = null;
    
                            /**
                             * AdGroupBidModifier base_ad_group.
                             * @member {string|null|undefined} base_ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.base_ad_group = null;
    
                            /**
                             * AdGroupBidModifier bid_modifier_source.
                             * @member {google.ads.googleads.v11.enums.BidModifierSourceEnum.BidModifierSource} bid_modifier_source
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.bid_modifier_source = 0;
    
                            /**
                             * AdGroupBidModifier hotel_date_selection_type.
                             * @member {google.ads.googleads.v11.common.IHotelDateSelectionTypeInfo|null|undefined} hotel_date_selection_type
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.hotel_date_selection_type = null;
    
                            /**
                             * AdGroupBidModifier hotel_advance_booking_window.
                             * @member {google.ads.googleads.v11.common.IHotelAdvanceBookingWindowInfo|null|undefined} hotel_advance_booking_window
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.hotel_advance_booking_window = null;
    
                            /**
                             * AdGroupBidModifier hotel_length_of_stay.
                             * @member {google.ads.googleads.v11.common.IHotelLengthOfStayInfo|null|undefined} hotel_length_of_stay
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.hotel_length_of_stay = null;
    
                            /**
                             * AdGroupBidModifier hotel_check_in_day.
                             * @member {google.ads.googleads.v11.common.IHotelCheckInDayInfo|null|undefined} hotel_check_in_day
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.hotel_check_in_day = null;
    
                            /**
                             * AdGroupBidModifier device.
                             * @member {google.ads.googleads.v11.common.IDeviceInfo|null|undefined} device
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.device = null;
    
                            /**
                             * AdGroupBidModifier preferred_content.
                             * @member {google.ads.googleads.v11.common.IPreferredContentInfo|null|undefined} preferred_content
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.preferred_content = null;
    
                            /**
                             * AdGroupBidModifier hotel_check_in_date_range.
                             * @member {google.ads.googleads.v11.common.IHotelCheckInDateRangeInfo|null|undefined} hotel_check_in_date_range
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            AdGroupBidModifier.prototype.hotel_check_in_date_range = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupBidModifier _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            Object.defineProperty(AdGroupBidModifier.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupBidModifier _criterion_id.
                             * @member {"criterion_id"|undefined} _criterion_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            Object.defineProperty(AdGroupBidModifier.prototype, "_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupBidModifier _bid_modifier.
                             * @member {"bid_modifier"|undefined} _bid_modifier
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            Object.defineProperty(AdGroupBidModifier.prototype, "_bid_modifier", {
                                get: $util.oneOfGetter($oneOfFields = ["bid_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupBidModifier _base_ad_group.
                             * @member {"base_ad_group"|undefined} _base_ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            Object.defineProperty(AdGroupBidModifier.prototype, "_base_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["base_ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupBidModifier criterion.
                             * @member {"hotel_date_selection_type"|"hotel_advance_booking_window"|"hotel_length_of_stay"|"hotel_check_in_day"|"device"|"preferred_content"|"hotel_check_in_date_range"|undefined} criterion
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             */
                            Object.defineProperty(AdGroupBidModifier.prototype, "criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["hotel_date_selection_type", "hotel_advance_booking_window", "hotel_length_of_stay", "hotel_check_in_day", "device", "preferred_content", "hotel_check_in_date_range"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupBidModifier instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupBidModifier=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupBidModifier} AdGroupBidModifier instance
                             */
                            AdGroupBidModifier.create = function create(properties) {
                                return new AdGroupBidModifier(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupBidModifier message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupBidModifier.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupBidModifier} m AdGroupBidModifier message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupBidModifier.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.hotel_date_selection_type != null && Object.hasOwnProperty.call(m, "hotel_date_selection_type"))
                                    $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo.encode(m.hotel_date_selection_type, w.uint32(42).fork()).ldelim();
                                if (m.hotel_advance_booking_window != null && Object.hasOwnProperty.call(m, "hotel_advance_booking_window"))
                                    $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo.encode(m.hotel_advance_booking_window, w.uint32(50).fork()).ldelim();
                                if (m.hotel_length_of_stay != null && Object.hasOwnProperty.call(m, "hotel_length_of_stay"))
                                    $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo.encode(m.hotel_length_of_stay, w.uint32(58).fork()).ldelim();
                                if (m.hotel_check_in_day != null && Object.hasOwnProperty.call(m, "hotel_check_in_day"))
                                    $root.google.ads.googleads.v11.common.HotelCheckInDayInfo.encode(m.hotel_check_in_day, w.uint32(66).fork()).ldelim();
                                if (m.bid_modifier_source != null && Object.hasOwnProperty.call(m, "bid_modifier_source"))
                                    w.uint32(80).int32(m.bid_modifier_source);
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    $root.google.ads.googleads.v11.common.DeviceInfo.encode(m.device, w.uint32(90).fork()).ldelim();
                                if (m.preferred_content != null && Object.hasOwnProperty.call(m, "preferred_content"))
                                    $root.google.ads.googleads.v11.common.PreferredContentInfo.encode(m.preferred_content, w.uint32(98).fork()).ldelim();
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(106).string(m.ad_group);
                                if (m.criterion_id != null && Object.hasOwnProperty.call(m, "criterion_id"))
                                    w.uint32(112).int64(m.criterion_id);
                                if (m.bid_modifier != null && Object.hasOwnProperty.call(m, "bid_modifier"))
                                    w.uint32(121).double(m.bid_modifier);
                                if (m.base_ad_group != null && Object.hasOwnProperty.call(m, "base_ad_group"))
                                    w.uint32(130).string(m.base_ad_group);
                                if (m.hotel_check_in_date_range != null && Object.hasOwnProperty.call(m, "hotel_check_in_date_range"))
                                    $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo.encode(m.hotel_check_in_date_range, w.uint32(138).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupBidModifier message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupBidModifier.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupBidModifier} message AdGroupBidModifier message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupBidModifier.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupBidModifier message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupBidModifier} AdGroupBidModifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupBidModifier.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupBidModifier();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 13:
                                        m.ad_group = r.string();
                                        break;
                                    case 14:
                                        m.criterion_id = r.int64();
                                        break;
                                    case 15:
                                        m.bid_modifier = r.double();
                                        break;
                                    case 16:
                                        m.base_ad_group = r.string();
                                        break;
                                    case 10:
                                        m.bid_modifier_source = r.int32();
                                        break;
                                    case 5:
                                        m.hotel_date_selection_type = $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.hotel_advance_booking_window = $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.hotel_length_of_stay = $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.hotel_check_in_day = $root.google.ads.googleads.v11.common.HotelCheckInDayInfo.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.device = $root.google.ads.googleads.v11.common.DeviceInfo.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.preferred_content = $root.google.ads.googleads.v11.common.PreferredContentInfo.decode(r, r.uint32());
                                        break;
                                    case 17:
                                        m.hotel_check_in_date_range = $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupBidModifier message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupBidModifier} AdGroupBidModifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupBidModifier.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupBidModifier message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupBidModifier.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    p._criterion_id = 1;
                                    if (!$util.isInteger(m.criterion_id) && !(m.criterion_id && $util.isInteger(m.criterion_id.low) && $util.isInteger(m.criterion_id.high)))
                                        return "criterion_id: integer|Long expected";
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    p._bid_modifier = 1;
                                    if (typeof m.bid_modifier !== "number")
                                        return "bid_modifier: number expected";
                                }
                                if (m.base_ad_group != null && m.hasOwnProperty("base_ad_group")) {
                                    p._base_ad_group = 1;
                                    if (!$util.isString(m.base_ad_group))
                                        return "base_ad_group: string expected";
                                }
                                if (m.bid_modifier_source != null && m.hasOwnProperty("bid_modifier_source")) {
                                    switch (m.bid_modifier_source) {
                                    default:
                                        return "bid_modifier_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.hotel_date_selection_type != null && m.hasOwnProperty("hotel_date_selection_type")) {
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo.verify(m.hotel_date_selection_type);
                                        if (e)
                                            return "hotel_date_selection_type." + e;
                                    }
                                }
                                if (m.hotel_advance_booking_window != null && m.hasOwnProperty("hotel_advance_booking_window")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo.verify(m.hotel_advance_booking_window);
                                        if (e)
                                            return "hotel_advance_booking_window." + e;
                                    }
                                }
                                if (m.hotel_length_of_stay != null && m.hasOwnProperty("hotel_length_of_stay")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo.verify(m.hotel_length_of_stay);
                                        if (e)
                                            return "hotel_length_of_stay." + e;
                                    }
                                }
                                if (m.hotel_check_in_day != null && m.hasOwnProperty("hotel_check_in_day")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelCheckInDayInfo.verify(m.hotel_check_in_day);
                                        if (e)
                                            return "hotel_check_in_day." + e;
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DeviceInfo.verify(m.device);
                                        if (e)
                                            return "device." + e;
                                    }
                                }
                                if (m.preferred_content != null && m.hasOwnProperty("preferred_content")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PreferredContentInfo.verify(m.preferred_content);
                                        if (e)
                                            return "preferred_content." + e;
                                    }
                                }
                                if (m.hotel_check_in_date_range != null && m.hasOwnProperty("hotel_check_in_date_range")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo.verify(m.hotel_check_in_date_range);
                                        if (e)
                                            return "hotel_check_in_date_range." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupBidModifier message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupBidModifier} AdGroupBidModifier
                             */
                            AdGroupBidModifier.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupBidModifier)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupBidModifier();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.criterion_id != null) {
                                    if ($util.Long)
                                        (m.criterion_id = $util.Long.fromValue(d.criterion_id)).unsigned = false;
                                    else if (typeof d.criterion_id === "string")
                                        m.criterion_id = parseInt(d.criterion_id, 10);
                                    else if (typeof d.criterion_id === "number")
                                        m.criterion_id = d.criterion_id;
                                    else if (typeof d.criterion_id === "object")
                                        m.criterion_id = new $util.LongBits(d.criterion_id.low >>> 0, d.criterion_id.high >>> 0).toNumber();
                                }
                                if (d.bid_modifier != null) {
                                    m.bid_modifier = Number(d.bid_modifier);
                                }
                                if (d.base_ad_group != null) {
                                    m.base_ad_group = String(d.base_ad_group);
                                }
                                switch (d.bid_modifier_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.bid_modifier_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.bid_modifier_source = 1;
                                    break;
                                case "CAMPAIGN":
                                case 2:
                                    m.bid_modifier_source = 2;
                                    break;
                                case "AD_GROUP":
                                case 3:
                                    m.bid_modifier_source = 3;
                                    break;
                                }
                                if (d.hotel_date_selection_type != null) {
                                    if (typeof d.hotel_date_selection_type !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupBidModifier.hotel_date_selection_type: object expected");
                                    m.hotel_date_selection_type = $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo.fromObject(d.hotel_date_selection_type);
                                }
                                if (d.hotel_advance_booking_window != null) {
                                    if (typeof d.hotel_advance_booking_window !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupBidModifier.hotel_advance_booking_window: object expected");
                                    m.hotel_advance_booking_window = $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo.fromObject(d.hotel_advance_booking_window);
                                }
                                if (d.hotel_length_of_stay != null) {
                                    if (typeof d.hotel_length_of_stay !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupBidModifier.hotel_length_of_stay: object expected");
                                    m.hotel_length_of_stay = $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo.fromObject(d.hotel_length_of_stay);
                                }
                                if (d.hotel_check_in_day != null) {
                                    if (typeof d.hotel_check_in_day !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupBidModifier.hotel_check_in_day: object expected");
                                    m.hotel_check_in_day = $root.google.ads.googleads.v11.common.HotelCheckInDayInfo.fromObject(d.hotel_check_in_day);
                                }
                                if (d.device != null) {
                                    if (typeof d.device !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupBidModifier.device: object expected");
                                    m.device = $root.google.ads.googleads.v11.common.DeviceInfo.fromObject(d.device);
                                }
                                if (d.preferred_content != null) {
                                    if (typeof d.preferred_content !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupBidModifier.preferred_content: object expected");
                                    m.preferred_content = $root.google.ads.googleads.v11.common.PreferredContentInfo.fromObject(d.preferred_content);
                                }
                                if (d.hotel_check_in_date_range != null) {
                                    if (typeof d.hotel_check_in_date_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupBidModifier.hotel_check_in_date_range: object expected");
                                    m.hotel_check_in_date_range = $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo.fromObject(d.hotel_check_in_date_range);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupBidModifier message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupBidModifier} m AdGroupBidModifier
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupBidModifier.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.bid_modifier_source = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.hotel_date_selection_type != null && m.hasOwnProperty("hotel_date_selection_type")) {
                                    d.hotel_date_selection_type = $root.google.ads.googleads.v11.common.HotelDateSelectionTypeInfo.toObject(m.hotel_date_selection_type, o);
                                    if (o.oneofs)
                                        d.criterion = "hotel_date_selection_type";
                                }
                                if (m.hotel_advance_booking_window != null && m.hasOwnProperty("hotel_advance_booking_window")) {
                                    d.hotel_advance_booking_window = $root.google.ads.googleads.v11.common.HotelAdvanceBookingWindowInfo.toObject(m.hotel_advance_booking_window, o);
                                    if (o.oneofs)
                                        d.criterion = "hotel_advance_booking_window";
                                }
                                if (m.hotel_length_of_stay != null && m.hasOwnProperty("hotel_length_of_stay")) {
                                    d.hotel_length_of_stay = $root.google.ads.googleads.v11.common.HotelLengthOfStayInfo.toObject(m.hotel_length_of_stay, o);
                                    if (o.oneofs)
                                        d.criterion = "hotel_length_of_stay";
                                }
                                if (m.hotel_check_in_day != null && m.hasOwnProperty("hotel_check_in_day")) {
                                    d.hotel_check_in_day = $root.google.ads.googleads.v11.common.HotelCheckInDayInfo.toObject(m.hotel_check_in_day, o);
                                    if (o.oneofs)
                                        d.criterion = "hotel_check_in_day";
                                }
                                if (m.bid_modifier_source != null && m.hasOwnProperty("bid_modifier_source")) {
                                    d.bid_modifier_source = o.enums === String ? $root.google.ads.googleads.v11.enums.BidModifierSourceEnum.BidModifierSource[m.bid_modifier_source] : m.bid_modifier_source;
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = $root.google.ads.googleads.v11.common.DeviceInfo.toObject(m.device, o);
                                    if (o.oneofs)
                                        d.criterion = "device";
                                }
                                if (m.preferred_content != null && m.hasOwnProperty("preferred_content")) {
                                    d.preferred_content = $root.google.ads.googleads.v11.common.PreferredContentInfo.toObject(m.preferred_content, o);
                                    if (o.oneofs)
                                        d.criterion = "preferred_content";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    if (typeof m.criterion_id === "number")
                                        d.criterion_id = o.longs === String ? String(m.criterion_id) : m.criterion_id;
                                    else
                                        d.criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.criterion_id) : o.longs === Number ? new $util.LongBits(m.criterion_id.low >>> 0, m.criterion_id.high >>> 0).toNumber() : m.criterion_id;
                                    if (o.oneofs)
                                        d._criterion_id = "criterion_id";
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    d.bid_modifier = o.json && !isFinite(m.bid_modifier) ? String(m.bid_modifier) : m.bid_modifier;
                                    if (o.oneofs)
                                        d._bid_modifier = "bid_modifier";
                                }
                                if (m.base_ad_group != null && m.hasOwnProperty("base_ad_group")) {
                                    d.base_ad_group = m.base_ad_group;
                                    if (o.oneofs)
                                        d._base_ad_group = "base_ad_group";
                                }
                                if (m.hotel_check_in_date_range != null && m.hasOwnProperty("hotel_check_in_date_range")) {
                                    d.hotel_check_in_date_range = $root.google.ads.googleads.v11.common.HotelCheckInDateRangeInfo.toObject(m.hotel_check_in_date_range, o);
                                    if (o.oneofs)
                                        d.criterion = "hotel_check_in_date_range";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupBidModifier to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupBidModifier
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupBidModifier.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupBidModifier;
                        })();
    
                        resources.AdGroupCriterion = (function() {
    
                            /**
                             * Properties of an AdGroupCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupCriterion
                             * @property {string|null} [resource_name] AdGroupCriterion resource_name
                             * @property {number|null} [criterion_id] AdGroupCriterion criterion_id
                             * @property {string|null} [display_name] AdGroupCriterion display_name
                             * @property {google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum.AdGroupCriterionStatus|null} [status] AdGroupCriterion status
                             * @property {google.ads.googleads.v11.resources.AdGroupCriterion.IQualityInfo|null} [quality_info] AdGroupCriterion quality_info
                             * @property {string|null} [ad_group] AdGroupCriterion ad_group
                             * @property {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType|null} [type] AdGroupCriterion type
                             * @property {boolean|null} [negative] AdGroupCriterion negative
                             * @property {google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum.CriterionSystemServingStatus|null} [system_serving_status] AdGroupCriterion system_serving_status
                             * @property {google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum.AdGroupCriterionApprovalStatus|null} [approval_status] AdGroupCriterion approval_status
                             * @property {Array.<string>|null} [disapproval_reasons] AdGroupCriterion disapproval_reasons
                             * @property {Array.<string>|null} [labels] AdGroupCriterion labels
                             * @property {number|null} [bid_modifier] AdGroupCriterion bid_modifier
                             * @property {number|null} [cpc_bid_micros] AdGroupCriterion cpc_bid_micros
                             * @property {number|null} [cpm_bid_micros] AdGroupCriterion cpm_bid_micros
                             * @property {number|null} [cpv_bid_micros] AdGroupCriterion cpv_bid_micros
                             * @property {number|null} [percent_cpc_bid_micros] AdGroupCriterion percent_cpc_bid_micros
                             * @property {number|null} [effective_cpc_bid_micros] AdGroupCriterion effective_cpc_bid_micros
                             * @property {number|null} [effective_cpm_bid_micros] AdGroupCriterion effective_cpm_bid_micros
                             * @property {number|null} [effective_cpv_bid_micros] AdGroupCriterion effective_cpv_bid_micros
                             * @property {number|null} [effective_percent_cpc_bid_micros] AdGroupCriterion effective_percent_cpc_bid_micros
                             * @property {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource|null} [effective_cpc_bid_source] AdGroupCriterion effective_cpc_bid_source
                             * @property {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource|null} [effective_cpm_bid_source] AdGroupCriterion effective_cpm_bid_source
                             * @property {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource|null} [effective_cpv_bid_source] AdGroupCriterion effective_cpv_bid_source
                             * @property {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource|null} [effective_percent_cpc_bid_source] AdGroupCriterion effective_percent_cpc_bid_source
                             * @property {google.ads.googleads.v11.resources.AdGroupCriterion.IPositionEstimates|null} [position_estimates] AdGroupCriterion position_estimates
                             * @property {Array.<string>|null} [final_urls] AdGroupCriterion final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] AdGroupCriterion final_mobile_urls
                             * @property {string|null} [final_url_suffix] AdGroupCriterion final_url_suffix
                             * @property {string|null} [tracking_url_template] AdGroupCriterion tracking_url_template
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] AdGroupCriterion url_custom_parameters
                             * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [keyword] AdGroupCriterion keyword
                             * @property {google.ads.googleads.v11.common.IPlacementInfo|null} [placement] AdGroupCriterion placement
                             * @property {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null} [mobile_app_category] AdGroupCriterion mobile_app_category
                             * @property {google.ads.googleads.v11.common.IMobileApplicationInfo|null} [mobile_application] AdGroupCriterion mobile_application
                             * @property {google.ads.googleads.v11.common.IListingGroupInfo|null} [listing_group] AdGroupCriterion listing_group
                             * @property {google.ads.googleads.v11.common.IAgeRangeInfo|null} [age_range] AdGroupCriterion age_range
                             * @property {google.ads.googleads.v11.common.IGenderInfo|null} [gender] AdGroupCriterion gender
                             * @property {google.ads.googleads.v11.common.IIncomeRangeInfo|null} [income_range] AdGroupCriterion income_range
                             * @property {google.ads.googleads.v11.common.IParentalStatusInfo|null} [parental_status] AdGroupCriterion parental_status
                             * @property {google.ads.googleads.v11.common.IUserListInfo|null} [user_list] AdGroupCriterion user_list
                             * @property {google.ads.googleads.v11.common.IYouTubeVideoInfo|null} [youtube_video] AdGroupCriterion youtube_video
                             * @property {google.ads.googleads.v11.common.IYouTubeChannelInfo|null} [youtube_channel] AdGroupCriterion youtube_channel
                             * @property {google.ads.googleads.v11.common.ITopicInfo|null} [topic] AdGroupCriterion topic
                             * @property {google.ads.googleads.v11.common.IUserInterestInfo|null} [user_interest] AdGroupCriterion user_interest
                             * @property {google.ads.googleads.v11.common.IWebpageInfo|null} [webpage] AdGroupCriterion webpage
                             * @property {google.ads.googleads.v11.common.IAppPaymentModelInfo|null} [app_payment_model] AdGroupCriterion app_payment_model
                             * @property {google.ads.googleads.v11.common.ICustomAffinityInfo|null} [custom_affinity] AdGroupCriterion custom_affinity
                             * @property {google.ads.googleads.v11.common.ICustomIntentInfo|null} [custom_intent] AdGroupCriterion custom_intent
                             * @property {google.ads.googleads.v11.common.ICustomAudienceInfo|null} [custom_audience] AdGroupCriterion custom_audience
                             * @property {google.ads.googleads.v11.common.ICombinedAudienceInfo|null} [combined_audience] AdGroupCriterion combined_audience
                             * @property {google.ads.googleads.v11.common.IAudienceInfo|null} [audience] AdGroupCriterion audience
                             */
    
                            /**
                             * Constructs a new AdGroupCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupCriterion.
                             * @implements IAdGroupCriterion
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterion=} [p] Properties to set
                             */
                            function AdGroupCriterion(p) {
                                this.disapproval_reasons = [];
                                this.labels = [];
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                this.url_custom_parameters = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCriterion resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.resource_name = "";
    
                            /**
                             * AdGroupCriterion criterion_id.
                             * @member {number|null|undefined} criterion_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.criterion_id = null;
    
                            /**
                             * AdGroupCriterion display_name.
                             * @member {string} display_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.display_name = "";
    
                            /**
                             * AdGroupCriterion status.
                             * @member {google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum.AdGroupCriterionStatus} status
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.status = 0;
    
                            /**
                             * AdGroupCriterion quality_info.
                             * @member {google.ads.googleads.v11.resources.AdGroupCriterion.IQualityInfo|null|undefined} quality_info
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.quality_info = null;
    
                            /**
                             * AdGroupCriterion ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.ad_group = null;
    
                            /**
                             * AdGroupCriterion type.
                             * @member {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType} type
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.type = 0;
    
                            /**
                             * AdGroupCriterion negative.
                             * @member {boolean|null|undefined} negative
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.negative = null;
    
                            /**
                             * AdGroupCriterion system_serving_status.
                             * @member {google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum.CriterionSystemServingStatus} system_serving_status
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.system_serving_status = 0;
    
                            /**
                             * AdGroupCriterion approval_status.
                             * @member {google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum.AdGroupCriterionApprovalStatus} approval_status
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.approval_status = 0;
    
                            /**
                             * AdGroupCriterion disapproval_reasons.
                             * @member {Array.<string>} disapproval_reasons
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.disapproval_reasons = $util.emptyArray;
    
                            /**
                             * AdGroupCriterion labels.
                             * @member {Array.<string>} labels
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.labels = $util.emptyArray;
    
                            /**
                             * AdGroupCriterion bid_modifier.
                             * @member {number|null|undefined} bid_modifier
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.bid_modifier = null;
    
                            /**
                             * AdGroupCriterion cpc_bid_micros.
                             * @member {number|null|undefined} cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.cpc_bid_micros = null;
    
                            /**
                             * AdGroupCriterion cpm_bid_micros.
                             * @member {number|null|undefined} cpm_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.cpm_bid_micros = null;
    
                            /**
                             * AdGroupCriterion cpv_bid_micros.
                             * @member {number|null|undefined} cpv_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.cpv_bid_micros = null;
    
                            /**
                             * AdGroupCriterion percent_cpc_bid_micros.
                             * @member {number|null|undefined} percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.percent_cpc_bid_micros = null;
    
                            /**
                             * AdGroupCriterion effective_cpc_bid_micros.
                             * @member {number|null|undefined} effective_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_cpc_bid_micros = null;
    
                            /**
                             * AdGroupCriterion effective_cpm_bid_micros.
                             * @member {number|null|undefined} effective_cpm_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_cpm_bid_micros = null;
    
                            /**
                             * AdGroupCriterion effective_cpv_bid_micros.
                             * @member {number|null|undefined} effective_cpv_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_cpv_bid_micros = null;
    
                            /**
                             * AdGroupCriterion effective_percent_cpc_bid_micros.
                             * @member {number|null|undefined} effective_percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_percent_cpc_bid_micros = null;
    
                            /**
                             * AdGroupCriterion effective_cpc_bid_source.
                             * @member {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource} effective_cpc_bid_source
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_cpc_bid_source = 0;
    
                            /**
                             * AdGroupCriterion effective_cpm_bid_source.
                             * @member {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource} effective_cpm_bid_source
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_cpm_bid_source = 0;
    
                            /**
                             * AdGroupCriterion effective_cpv_bid_source.
                             * @member {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource} effective_cpv_bid_source
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_cpv_bid_source = 0;
    
                            /**
                             * AdGroupCriterion effective_percent_cpc_bid_source.
                             * @member {google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource} effective_percent_cpc_bid_source
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.effective_percent_cpc_bid_source = 0;
    
                            /**
                             * AdGroupCriterion position_estimates.
                             * @member {google.ads.googleads.v11.resources.AdGroupCriterion.IPositionEstimates|null|undefined} position_estimates
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.position_estimates = null;
    
                            /**
                             * AdGroupCriterion final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * AdGroupCriterion final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * AdGroupCriterion final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.final_url_suffix = null;
    
                            /**
                             * AdGroupCriterion tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.tracking_url_template = null;
    
                            /**
                             * AdGroupCriterion url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * AdGroupCriterion keyword.
                             * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} keyword
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.keyword = null;
    
                            /**
                             * AdGroupCriterion placement.
                             * @member {google.ads.googleads.v11.common.IPlacementInfo|null|undefined} placement
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.placement = null;
    
                            /**
                             * AdGroupCriterion mobile_app_category.
                             * @member {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null|undefined} mobile_app_category
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.mobile_app_category = null;
    
                            /**
                             * AdGroupCriterion mobile_application.
                             * @member {google.ads.googleads.v11.common.IMobileApplicationInfo|null|undefined} mobile_application
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.mobile_application = null;
    
                            /**
                             * AdGroupCriterion listing_group.
                             * @member {google.ads.googleads.v11.common.IListingGroupInfo|null|undefined} listing_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.listing_group = null;
    
                            /**
                             * AdGroupCriterion age_range.
                             * @member {google.ads.googleads.v11.common.IAgeRangeInfo|null|undefined} age_range
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.age_range = null;
    
                            /**
                             * AdGroupCriterion gender.
                             * @member {google.ads.googleads.v11.common.IGenderInfo|null|undefined} gender
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.gender = null;
    
                            /**
                             * AdGroupCriterion income_range.
                             * @member {google.ads.googleads.v11.common.IIncomeRangeInfo|null|undefined} income_range
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.income_range = null;
    
                            /**
                             * AdGroupCriterion parental_status.
                             * @member {google.ads.googleads.v11.common.IParentalStatusInfo|null|undefined} parental_status
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.parental_status = null;
    
                            /**
                             * AdGroupCriterion user_list.
                             * @member {google.ads.googleads.v11.common.IUserListInfo|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.user_list = null;
    
                            /**
                             * AdGroupCriterion youtube_video.
                             * @member {google.ads.googleads.v11.common.IYouTubeVideoInfo|null|undefined} youtube_video
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.youtube_video = null;
    
                            /**
                             * AdGroupCriterion youtube_channel.
                             * @member {google.ads.googleads.v11.common.IYouTubeChannelInfo|null|undefined} youtube_channel
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.youtube_channel = null;
    
                            /**
                             * AdGroupCriterion topic.
                             * @member {google.ads.googleads.v11.common.ITopicInfo|null|undefined} topic
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.topic = null;
    
                            /**
                             * AdGroupCriterion user_interest.
                             * @member {google.ads.googleads.v11.common.IUserInterestInfo|null|undefined} user_interest
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.user_interest = null;
    
                            /**
                             * AdGroupCriterion webpage.
                             * @member {google.ads.googleads.v11.common.IWebpageInfo|null|undefined} webpage
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.webpage = null;
    
                            /**
                             * AdGroupCriterion app_payment_model.
                             * @member {google.ads.googleads.v11.common.IAppPaymentModelInfo|null|undefined} app_payment_model
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.app_payment_model = null;
    
                            /**
                             * AdGroupCriterion custom_affinity.
                             * @member {google.ads.googleads.v11.common.ICustomAffinityInfo|null|undefined} custom_affinity
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.custom_affinity = null;
    
                            /**
                             * AdGroupCriterion custom_intent.
                             * @member {google.ads.googleads.v11.common.ICustomIntentInfo|null|undefined} custom_intent
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.custom_intent = null;
    
                            /**
                             * AdGroupCriterion custom_audience.
                             * @member {google.ads.googleads.v11.common.ICustomAudienceInfo|null|undefined} custom_audience
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.custom_audience = null;
    
                            /**
                             * AdGroupCriterion combined_audience.
                             * @member {google.ads.googleads.v11.common.ICombinedAudienceInfo|null|undefined} combined_audience
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.combined_audience = null;
    
                            /**
                             * AdGroupCriterion audience.
                             * @member {google.ads.googleads.v11.common.IAudienceInfo|null|undefined} audience
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            AdGroupCriterion.prototype.audience = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCriterion _criterion_id.
                             * @member {"criterion_id"|undefined} _criterion_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _negative.
                             * @member {"negative"|undefined} _negative
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_negative", {
                                get: $util.oneOfGetter($oneOfFields = ["negative"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _bid_modifier.
                             * @member {"bid_modifier"|undefined} _bid_modifier
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_bid_modifier", {
                                get: $util.oneOfGetter($oneOfFields = ["bid_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _cpc_bid_micros.
                             * @member {"cpc_bid_micros"|undefined} _cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _cpm_bid_micros.
                             * @member {"cpm_bid_micros"|undefined} _cpm_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_cpm_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpm_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _cpv_bid_micros.
                             * @member {"cpv_bid_micros"|undefined} _cpv_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_cpv_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpv_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _percent_cpc_bid_micros.
                             * @member {"percent_cpc_bid_micros"|undefined} _percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_percent_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["percent_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _effective_cpc_bid_micros.
                             * @member {"effective_cpc_bid_micros"|undefined} _effective_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_effective_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _effective_cpm_bid_micros.
                             * @member {"effective_cpm_bid_micros"|undefined} _effective_cpm_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_effective_cpm_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_cpm_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _effective_cpv_bid_micros.
                             * @member {"effective_cpv_bid_micros"|undefined} _effective_cpv_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_effective_cpv_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_cpv_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _effective_percent_cpc_bid_micros.
                             * @member {"effective_percent_cpc_bid_micros"|undefined} _effective_percent_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_effective_percent_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_percent_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterion criterion.
                             * @member {"keyword"|"placement"|"mobile_app_category"|"mobile_application"|"listing_group"|"age_range"|"gender"|"income_range"|"parental_status"|"user_list"|"youtube_video"|"youtube_channel"|"topic"|"user_interest"|"webpage"|"app_payment_model"|"custom_affinity"|"custom_intent"|"custom_audience"|"combined_audience"|"audience"|undefined} criterion
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterion.prototype, "criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword", "placement", "mobile_app_category", "mobile_application", "listing_group", "age_range", "gender", "income_range", "parental_status", "user_list", "youtube_video", "youtube_channel", "topic", "user_interest", "webpage", "app_payment_model", "custom_affinity", "custom_intent", "custom_audience", "combined_audience", "audience"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCriterion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterion} AdGroupCriterion instance
                             */
                            AdGroupCriterion.create = function create(properties) {
                                return new AdGroupCriterion(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterion message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterion} m AdGroupCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.quality_info != null && Object.hasOwnProperty.call(m, "quality_info"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo.encode(m.quality_info, w.uint32(34).fork()).ldelim();
                                if (m.position_estimates != null && Object.hasOwnProperty.call(m, "position_estimates"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates.encode(m.position_estimates, w.uint32(82).fork()).ldelim();
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(114).fork()).ldelim();
                                }
                                if (m.effective_cpc_bid_source != null && Object.hasOwnProperty.call(m, "effective_cpc_bid_source"))
                                    w.uint32(168).int32(m.effective_cpc_bid_source);
                                if (m.effective_cpm_bid_source != null && Object.hasOwnProperty.call(m, "effective_cpm_bid_source"))
                                    w.uint32(176).int32(m.effective_cpm_bid_source);
                                if (m.effective_cpv_bid_source != null && Object.hasOwnProperty.call(m, "effective_cpv_bid_source"))
                                    w.uint32(184).int32(m.effective_cpv_bid_source);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(200).int32(m.type);
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword, w.uint32(218).fork()).ldelim();
                                if (m.placement != null && Object.hasOwnProperty.call(m, "placement"))
                                    $root.google.ads.googleads.v11.common.PlacementInfo.encode(m.placement, w.uint32(226).fork()).ldelim();
                                if (m.mobile_app_category != null && Object.hasOwnProperty.call(m, "mobile_app_category"))
                                    $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.encode(m.mobile_app_category, w.uint32(234).fork()).ldelim();
                                if (m.mobile_application != null && Object.hasOwnProperty.call(m, "mobile_application"))
                                    $root.google.ads.googleads.v11.common.MobileApplicationInfo.encode(m.mobile_application, w.uint32(242).fork()).ldelim();
                                if (m.listing_group != null && Object.hasOwnProperty.call(m, "listing_group"))
                                    $root.google.ads.googleads.v11.common.ListingGroupInfo.encode(m.listing_group, w.uint32(258).fork()).ldelim();
                                if (m.effective_percent_cpc_bid_source != null && Object.hasOwnProperty.call(m, "effective_percent_cpc_bid_source"))
                                    w.uint32(280).int32(m.effective_percent_cpc_bid_source);
                                if (m.age_range != null && Object.hasOwnProperty.call(m, "age_range"))
                                    $root.google.ads.googleads.v11.common.AgeRangeInfo.encode(m.age_range, w.uint32(290).fork()).ldelim();
                                if (m.gender != null && Object.hasOwnProperty.call(m, "gender"))
                                    $root.google.ads.googleads.v11.common.GenderInfo.encode(m.gender, w.uint32(298).fork()).ldelim();
                                if (m.income_range != null && Object.hasOwnProperty.call(m, "income_range"))
                                    $root.google.ads.googleads.v11.common.IncomeRangeInfo.encode(m.income_range, w.uint32(306).fork()).ldelim();
                                if (m.parental_status != null && Object.hasOwnProperty.call(m, "parental_status"))
                                    $root.google.ads.googleads.v11.common.ParentalStatusInfo.encode(m.parental_status, w.uint32(314).fork()).ldelim();
                                if (m.youtube_video != null && Object.hasOwnProperty.call(m, "youtube_video"))
                                    $root.google.ads.googleads.v11.common.YouTubeVideoInfo.encode(m.youtube_video, w.uint32(322).fork()).ldelim();
                                if (m.youtube_channel != null && Object.hasOwnProperty.call(m, "youtube_channel"))
                                    $root.google.ads.googleads.v11.common.YouTubeChannelInfo.encode(m.youtube_channel, w.uint32(330).fork()).ldelim();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    $root.google.ads.googleads.v11.common.UserListInfo.encode(m.user_list, w.uint32(338).fork()).ldelim();
                                if (m.topic != null && Object.hasOwnProperty.call(m, "topic"))
                                    $root.google.ads.googleads.v11.common.TopicInfo.encode(m.topic, w.uint32(346).fork()).ldelim();
                                if (m.user_interest != null && Object.hasOwnProperty.call(m, "user_interest"))
                                    $root.google.ads.googleads.v11.common.UserInterestInfo.encode(m.user_interest, w.uint32(362).fork()).ldelim();
                                if (m.webpage != null && Object.hasOwnProperty.call(m, "webpage"))
                                    $root.google.ads.googleads.v11.common.WebpageInfo.encode(m.webpage, w.uint32(370).fork()).ldelim();
                                if (m.app_payment_model != null && Object.hasOwnProperty.call(m, "app_payment_model"))
                                    $root.google.ads.googleads.v11.common.AppPaymentModelInfo.encode(m.app_payment_model, w.uint32(378).fork()).ldelim();
                                if (m.custom_affinity != null && Object.hasOwnProperty.call(m, "custom_affinity"))
                                    $root.google.ads.googleads.v11.common.CustomAffinityInfo.encode(m.custom_affinity, w.uint32(386).fork()).ldelim();
                                if (m.custom_intent != null && Object.hasOwnProperty.call(m, "custom_intent"))
                                    $root.google.ads.googleads.v11.common.CustomIntentInfo.encode(m.custom_intent, w.uint32(394).fork()).ldelim();
                                if (m.system_serving_status != null && Object.hasOwnProperty.call(m, "system_serving_status"))
                                    w.uint32(416).int32(m.system_serving_status);
                                if (m.approval_status != null && Object.hasOwnProperty.call(m, "approval_status"))
                                    w.uint32(424).int32(m.approval_status);
                                if (m.criterion_id != null && Object.hasOwnProperty.call(m, "criterion_id"))
                                    w.uint32(448).int64(m.criterion_id);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(458).string(m.ad_group);
                                if (m.negative != null && Object.hasOwnProperty.call(m, "negative"))
                                    w.uint32(464).bool(m.negative);
                                if (m.disapproval_reasons != null && m.disapproval_reasons.length) {
                                    for (var i = 0; i < m.disapproval_reasons.length; ++i)
                                        w.uint32(474).string(m.disapproval_reasons[i]);
                                }
                                if (m.labels != null && m.labels.length) {
                                    for (var i = 0; i < m.labels.length; ++i)
                                        w.uint32(482).string(m.labels[i]);
                                }
                                if (m.bid_modifier != null && Object.hasOwnProperty.call(m, "bid_modifier"))
                                    w.uint32(489).double(m.bid_modifier);
                                if (m.cpc_bid_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_micros"))
                                    w.uint32(496).int64(m.cpc_bid_micros);
                                if (m.cpm_bid_micros != null && Object.hasOwnProperty.call(m, "cpm_bid_micros"))
                                    w.uint32(504).int64(m.cpm_bid_micros);
                                if (m.cpv_bid_micros != null && Object.hasOwnProperty.call(m, "cpv_bid_micros"))
                                    w.uint32(512).int64(m.cpv_bid_micros);
                                if (m.percent_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "percent_cpc_bid_micros"))
                                    w.uint32(520).int64(m.percent_cpc_bid_micros);
                                if (m.effective_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "effective_cpc_bid_micros"))
                                    w.uint32(528).int64(m.effective_cpc_bid_micros);
                                if (m.effective_cpm_bid_micros != null && Object.hasOwnProperty.call(m, "effective_cpm_bid_micros"))
                                    w.uint32(536).int64(m.effective_cpm_bid_micros);
                                if (m.effective_cpv_bid_micros != null && Object.hasOwnProperty.call(m, "effective_cpv_bid_micros"))
                                    w.uint32(544).int64(m.effective_cpv_bid_micros);
                                if (m.effective_percent_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "effective_percent_cpc_bid_micros"))
                                    w.uint32(552).int64(m.effective_percent_cpc_bid_micros);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(562).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(570).string(m.final_mobile_urls[i]);
                                }
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(578).string(m.final_url_suffix);
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(586).string(m.tracking_url_template);
                                if (m.custom_audience != null && Object.hasOwnProperty.call(m, "custom_audience"))
                                    $root.google.ads.googleads.v11.common.CustomAudienceInfo.encode(m.custom_audience, w.uint32(594).fork()).ldelim();
                                if (m.combined_audience != null && Object.hasOwnProperty.call(m, "combined_audience"))
                                    $root.google.ads.googleads.v11.common.CombinedAudienceInfo.encode(m.combined_audience, w.uint32(602).fork()).ldelim();
                                if (m.display_name != null && Object.hasOwnProperty.call(m, "display_name"))
                                    w.uint32(618).string(m.display_name);
                                if (m.audience != null && Object.hasOwnProperty.call(m, "audience"))
                                    $root.google.ads.googleads.v11.common.AudienceInfo.encode(m.audience, w.uint32(634).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterion} message AdGroupCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterion} AdGroupCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupCriterion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 56:
                                        m.criterion_id = r.int64();
                                        break;
                                    case 77:
                                        m.display_name = r.string();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 4:
                                        m.quality_info = $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo.decode(r, r.uint32());
                                        break;
                                    case 57:
                                        m.ad_group = r.string();
                                        break;
                                    case 25:
                                        m.type = r.int32();
                                        break;
                                    case 58:
                                        m.negative = r.bool();
                                        break;
                                    case 52:
                                        m.system_serving_status = r.int32();
                                        break;
                                    case 53:
                                        m.approval_status = r.int32();
                                        break;
                                    case 59:
                                        if (!(m.disapproval_reasons && m.disapproval_reasons.length))
                                            m.disapproval_reasons = [];
                                        m.disapproval_reasons.push(r.string());
                                        break;
                                    case 60:
                                        if (!(m.labels && m.labels.length))
                                            m.labels = [];
                                        m.labels.push(r.string());
                                        break;
                                    case 61:
                                        m.bid_modifier = r.double();
                                        break;
                                    case 62:
                                        m.cpc_bid_micros = r.int64();
                                        break;
                                    case 63:
                                        m.cpm_bid_micros = r.int64();
                                        break;
                                    case 64:
                                        m.cpv_bid_micros = r.int64();
                                        break;
                                    case 65:
                                        m.percent_cpc_bid_micros = r.int64();
                                        break;
                                    case 66:
                                        m.effective_cpc_bid_micros = r.int64();
                                        break;
                                    case 67:
                                        m.effective_cpm_bid_micros = r.int64();
                                        break;
                                    case 68:
                                        m.effective_cpv_bid_micros = r.int64();
                                        break;
                                    case 69:
                                        m.effective_percent_cpc_bid_micros = r.int64();
                                        break;
                                    case 21:
                                        m.effective_cpc_bid_source = r.int32();
                                        break;
                                    case 22:
                                        m.effective_cpm_bid_source = r.int32();
                                        break;
                                    case 23:
                                        m.effective_cpv_bid_source = r.int32();
                                        break;
                                    case 35:
                                        m.effective_percent_cpc_bid_source = r.int32();
                                        break;
                                    case 10:
                                        m.position_estimates = $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates.decode(r, r.uint32());
                                        break;
                                    case 70:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 71:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 72:
                                        m.final_url_suffix = r.string();
                                        break;
                                    case 73:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 14:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 27:
                                        m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.decode(r, r.uint32());
                                        break;
                                    case 29:
                                        m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.listing_group = $root.google.ads.googleads.v11.common.ListingGroupInfo.decode(r, r.uint32());
                                        break;
                                    case 36:
                                        m.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.decode(r, r.uint32());
                                        break;
                                    case 37:
                                        m.gender = $root.google.ads.googleads.v11.common.GenderInfo.decode(r, r.uint32());
                                        break;
                                    case 38:
                                        m.income_range = $root.google.ads.googleads.v11.common.IncomeRangeInfo.decode(r, r.uint32());
                                        break;
                                    case 39:
                                        m.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusInfo.decode(r, r.uint32());
                                        break;
                                    case 42:
                                        m.user_list = $root.google.ads.googleads.v11.common.UserListInfo.decode(r, r.uint32());
                                        break;
                                    case 40:
                                        m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.decode(r, r.uint32());
                                        break;
                                    case 41:
                                        m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.decode(r, r.uint32());
                                        break;
                                    case 43:
                                        m.topic = $root.google.ads.googleads.v11.common.TopicInfo.decode(r, r.uint32());
                                        break;
                                    case 45:
                                        m.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.decode(r, r.uint32());
                                        break;
                                    case 46:
                                        m.webpage = $root.google.ads.googleads.v11.common.WebpageInfo.decode(r, r.uint32());
                                        break;
                                    case 47:
                                        m.app_payment_model = $root.google.ads.googleads.v11.common.AppPaymentModelInfo.decode(r, r.uint32());
                                        break;
                                    case 48:
                                        m.custom_affinity = $root.google.ads.googleads.v11.common.CustomAffinityInfo.decode(r, r.uint32());
                                        break;
                                    case 49:
                                        m.custom_intent = $root.google.ads.googleads.v11.common.CustomIntentInfo.decode(r, r.uint32());
                                        break;
                                    case 74:
                                        m.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceInfo.decode(r, r.uint32());
                                        break;
                                    case 75:
                                        m.combined_audience = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.decode(r, r.uint32());
                                        break;
                                    case 79:
                                        m.audience = $root.google.ads.googleads.v11.common.AudienceInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterion} AdGroupCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    p._criterion_id = 1;
                                    if (!$util.isInteger(m.criterion_id) && !(m.criterion_id && $util.isInteger(m.criterion_id.low) && $util.isInteger(m.criterion_id.high)))
                                        return "criterion_id: integer|Long expected";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    if (!$util.isString(m.display_name))
                                        return "display_name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.quality_info != null && m.hasOwnProperty("quality_info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo.verify(m.quality_info);
                                        if (e)
                                            return "quality_info." + e;
                                    }
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                        break;
                                    }
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    p._negative = 1;
                                    if (typeof m.negative !== "boolean")
                                        return "negative: boolean expected";
                                }
                                if (m.system_serving_status != null && m.hasOwnProperty("system_serving_status")) {
                                    switch (m.system_serving_status) {
                                    default:
                                        return "system_serving_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    switch (m.approval_status) {
                                    default:
                                        return "approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.disapproval_reasons != null && m.hasOwnProperty("disapproval_reasons")) {
                                    if (!Array.isArray(m.disapproval_reasons))
                                        return "disapproval_reasons: array expected";
                                    for (var i = 0; i < m.disapproval_reasons.length; ++i) {
                                        if (!$util.isString(m.disapproval_reasons[i]))
                                            return "disapproval_reasons: string[] expected";
                                    }
                                }
                                if (m.labels != null && m.hasOwnProperty("labels")) {
                                    if (!Array.isArray(m.labels))
                                        return "labels: array expected";
                                    for (var i = 0; i < m.labels.length; ++i) {
                                        if (!$util.isString(m.labels[i]))
                                            return "labels: string[] expected";
                                    }
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    p._bid_modifier = 1;
                                    if (typeof m.bid_modifier !== "number")
                                        return "bid_modifier: number expected";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    p._cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_micros) && !(m.cpc_bid_micros && $util.isInteger(m.cpc_bid_micros.low) && $util.isInteger(m.cpc_bid_micros.high)))
                                        return "cpc_bid_micros: integer|Long expected";
                                }
                                if (m.cpm_bid_micros != null && m.hasOwnProperty("cpm_bid_micros")) {
                                    p._cpm_bid_micros = 1;
                                    if (!$util.isInteger(m.cpm_bid_micros) && !(m.cpm_bid_micros && $util.isInteger(m.cpm_bid_micros.low) && $util.isInteger(m.cpm_bid_micros.high)))
                                        return "cpm_bid_micros: integer|Long expected";
                                }
                                if (m.cpv_bid_micros != null && m.hasOwnProperty("cpv_bid_micros")) {
                                    p._cpv_bid_micros = 1;
                                    if (!$util.isInteger(m.cpv_bid_micros) && !(m.cpv_bid_micros && $util.isInteger(m.cpv_bid_micros.low) && $util.isInteger(m.cpv_bid_micros.high)))
                                        return "cpv_bid_micros: integer|Long expected";
                                }
                                if (m.percent_cpc_bid_micros != null && m.hasOwnProperty("percent_cpc_bid_micros")) {
                                    p._percent_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.percent_cpc_bid_micros) && !(m.percent_cpc_bid_micros && $util.isInteger(m.percent_cpc_bid_micros.low) && $util.isInteger(m.percent_cpc_bid_micros.high)))
                                        return "percent_cpc_bid_micros: integer|Long expected";
                                }
                                if (m.effective_cpc_bid_micros != null && m.hasOwnProperty("effective_cpc_bid_micros")) {
                                    p._effective_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.effective_cpc_bid_micros) && !(m.effective_cpc_bid_micros && $util.isInteger(m.effective_cpc_bid_micros.low) && $util.isInteger(m.effective_cpc_bid_micros.high)))
                                        return "effective_cpc_bid_micros: integer|Long expected";
                                }
                                if (m.effective_cpm_bid_micros != null && m.hasOwnProperty("effective_cpm_bid_micros")) {
                                    p._effective_cpm_bid_micros = 1;
                                    if (!$util.isInteger(m.effective_cpm_bid_micros) && !(m.effective_cpm_bid_micros && $util.isInteger(m.effective_cpm_bid_micros.low) && $util.isInteger(m.effective_cpm_bid_micros.high)))
                                        return "effective_cpm_bid_micros: integer|Long expected";
                                }
                                if (m.effective_cpv_bid_micros != null && m.hasOwnProperty("effective_cpv_bid_micros")) {
                                    p._effective_cpv_bid_micros = 1;
                                    if (!$util.isInteger(m.effective_cpv_bid_micros) && !(m.effective_cpv_bid_micros && $util.isInteger(m.effective_cpv_bid_micros.low) && $util.isInteger(m.effective_cpv_bid_micros.high)))
                                        return "effective_cpv_bid_micros: integer|Long expected";
                                }
                                if (m.effective_percent_cpc_bid_micros != null && m.hasOwnProperty("effective_percent_cpc_bid_micros")) {
                                    p._effective_percent_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.effective_percent_cpc_bid_micros) && !(m.effective_percent_cpc_bid_micros && $util.isInteger(m.effective_percent_cpc_bid_micros.low) && $util.isInteger(m.effective_percent_cpc_bid_micros.high)))
                                        return "effective_percent_cpc_bid_micros: integer|Long expected";
                                }
                                if (m.effective_cpc_bid_source != null && m.hasOwnProperty("effective_cpc_bid_source")) {
                                    switch (m.effective_cpc_bid_source) {
                                    default:
                                        return "effective_cpc_bid_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.effective_cpm_bid_source != null && m.hasOwnProperty("effective_cpm_bid_source")) {
                                    switch (m.effective_cpm_bid_source) {
                                    default:
                                        return "effective_cpm_bid_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.effective_cpv_bid_source != null && m.hasOwnProperty("effective_cpv_bid_source")) {
                                    switch (m.effective_cpv_bid_source) {
                                    default:
                                        return "effective_cpv_bid_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.effective_percent_cpc_bid_source != null && m.hasOwnProperty("effective_percent_cpc_bid_source")) {
                                    switch (m.effective_percent_cpc_bid_source) {
                                    default:
                                        return "effective_percent_cpc_bid_source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.position_estimates != null && m.hasOwnProperty("position_estimates")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates.verify(m.position_estimates);
                                        if (e)
                                            return "position_estimates." + e;
                                    }
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword);
                                        if (e)
                                            return "keyword." + e;
                                    }
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PlacementInfo.verify(m.placement);
                                        if (e)
                                            return "placement." + e;
                                    }
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.verify(m.mobile_app_category);
                                        if (e)
                                            return "mobile_app_category." + e;
                                    }
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileApplicationInfo.verify(m.mobile_application);
                                        if (e)
                                            return "mobile_application." + e;
                                    }
                                }
                                if (m.listing_group != null && m.hasOwnProperty("listing_group")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ListingGroupInfo.verify(m.listing_group);
                                        if (e)
                                            return "listing_group." + e;
                                    }
                                }
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AgeRangeInfo.verify(m.age_range);
                                        if (e)
                                            return "age_range." + e;
                                    }
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GenderInfo.verify(m.gender);
                                        if (e)
                                            return "gender." + e;
                                    }
                                }
                                if (m.income_range != null && m.hasOwnProperty("income_range")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.IncomeRangeInfo.verify(m.income_range);
                                        if (e)
                                            return "income_range." + e;
                                    }
                                }
                                if (m.parental_status != null && m.hasOwnProperty("parental_status")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ParentalStatusInfo.verify(m.parental_status);
                                        if (e)
                                            return "parental_status." + e;
                                    }
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListInfo.verify(m.user_list);
                                        if (e)
                                            return "user_list." + e;
                                    }
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.verify(m.youtube_video);
                                        if (e)
                                            return "youtube_video." + e;
                                    }
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.verify(m.youtube_channel);
                                        if (e)
                                            return "youtube_channel." + e;
                                    }
                                }
                                if (m.topic != null && m.hasOwnProperty("topic")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TopicInfo.verify(m.topic);
                                        if (e)
                                            return "topic." + e;
                                    }
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserInterestInfo.verify(m.user_interest);
                                        if (e)
                                            return "user_interest." + e;
                                    }
                                }
                                if (m.webpage != null && m.hasOwnProperty("webpage")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.WebpageInfo.verify(m.webpage);
                                        if (e)
                                            return "webpage." + e;
                                    }
                                }
                                if (m.app_payment_model != null && m.hasOwnProperty("app_payment_model")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AppPaymentModelInfo.verify(m.app_payment_model);
                                        if (e)
                                            return "app_payment_model." + e;
                                    }
                                }
                                if (m.custom_affinity != null && m.hasOwnProperty("custom_affinity")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomAffinityInfo.verify(m.custom_affinity);
                                        if (e)
                                            return "custom_affinity." + e;
                                    }
                                }
                                if (m.custom_intent != null && m.hasOwnProperty("custom_intent")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomIntentInfo.verify(m.custom_intent);
                                        if (e)
                                            return "custom_intent." + e;
                                    }
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomAudienceInfo.verify(m.custom_audience);
                                        if (e)
                                            return "custom_audience." + e;
                                    }
                                }
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.verify(m.combined_audience);
                                        if (e)
                                            return "combined_audience." + e;
                                    }
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AudienceInfo.verify(m.audience);
                                        if (e)
                                            return "audience." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterion} AdGroupCriterion
                             */
                            AdGroupCriterion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupCriterion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupCriterion();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.criterion_id != null) {
                                    if ($util.Long)
                                        (m.criterion_id = $util.Long.fromValue(d.criterion_id)).unsigned = false;
                                    else if (typeof d.criterion_id === "string")
                                        m.criterion_id = parseInt(d.criterion_id, 10);
                                    else if (typeof d.criterion_id === "number")
                                        m.criterion_id = d.criterion_id;
                                    else if (typeof d.criterion_id === "object")
                                        m.criterion_id = new $util.LongBits(d.criterion_id.low >>> 0, d.criterion_id.high >>> 0).toNumber();
                                }
                                if (d.display_name != null) {
                                    m.display_name = String(d.display_name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "PAUSED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.quality_info != null) {
                                    if (typeof d.quality_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.quality_info: object expected");
                                    m.quality_info = $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo.fromObject(d.quality_info);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "PLACEMENT":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MOBILE_APP_CATEGORY":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "MOBILE_APPLICATION":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "DEVICE":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "LOCATION":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "LISTING_GROUP":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "AD_SCHEDULE":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "AGE_RANGE":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "GENDER":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "INCOME_RANGE":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "PARENTAL_STATUS":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "YOUTUBE_CHANNEL":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "USER_LIST":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "PROXIMITY":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "TOPIC":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "LISTING_SCOPE":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "LANGUAGE":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "IP_BLOCK":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "CONTENT_LABEL":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "CARRIER":
                                case 23:
                                    m.type = 23;
                                    break;
                                case "USER_INTEREST":
                                case 24:
                                    m.type = 24;
                                    break;
                                case "WEBPAGE":
                                case 25:
                                    m.type = 25;
                                    break;
                                case "OPERATING_SYSTEM_VERSION":
                                case 26:
                                    m.type = 26;
                                    break;
                                case "APP_PAYMENT_MODEL":
                                case 27:
                                    m.type = 27;
                                    break;
                                case "MOBILE_DEVICE":
                                case 28:
                                    m.type = 28;
                                    break;
                                case "CUSTOM_AFFINITY":
                                case 29:
                                    m.type = 29;
                                    break;
                                case "CUSTOM_INTENT":
                                case 30:
                                    m.type = 30;
                                    break;
                                case "LOCATION_GROUP":
                                case 31:
                                    m.type = 31;
                                    break;
                                case "CUSTOM_AUDIENCE":
                                case 32:
                                    m.type = 32;
                                    break;
                                case "COMBINED_AUDIENCE":
                                case 33:
                                    m.type = 33;
                                    break;
                                case "KEYWORD_THEME":
                                case 34:
                                    m.type = 34;
                                    break;
                                case "AUDIENCE":
                                case 35:
                                    m.type = 35;
                                    break;
                                }
                                if (d.negative != null) {
                                    m.negative = Boolean(d.negative);
                                }
                                switch (d.system_serving_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.system_serving_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.system_serving_status = 1;
                                    break;
                                case "ELIGIBLE":
                                case 2:
                                    m.system_serving_status = 2;
                                    break;
                                case "RARELY_SERVED":
                                case 3:
                                    m.system_serving_status = 3;
                                    break;
                                }
                                switch (d.approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approval_status = 1;
                                    break;
                                case "APPROVED":
                                case 2:
                                    m.approval_status = 2;
                                    break;
                                case "DISAPPROVED":
                                case 3:
                                    m.approval_status = 3;
                                    break;
                                case "PENDING_REVIEW":
                                case 4:
                                    m.approval_status = 4;
                                    break;
                                case "UNDER_REVIEW":
                                case 5:
                                    m.approval_status = 5;
                                    break;
                                }
                                if (d.disapproval_reasons) {
                                    if (!Array.isArray(d.disapproval_reasons))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.disapproval_reasons: array expected");
                                    m.disapproval_reasons = [];
                                    for (var i = 0; i < d.disapproval_reasons.length; ++i) {
                                        m.disapproval_reasons[i] = String(d.disapproval_reasons[i]);
                                    }
                                }
                                if (d.labels) {
                                    if (!Array.isArray(d.labels))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.labels: array expected");
                                    m.labels = [];
                                    for (var i = 0; i < d.labels.length; ++i) {
                                        m.labels[i] = String(d.labels[i]);
                                    }
                                }
                                if (d.bid_modifier != null) {
                                    m.bid_modifier = Number(d.bid_modifier);
                                }
                                if (d.cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_micros = $util.Long.fromValue(d.cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_micros === "string")
                                        m.cpc_bid_micros = parseInt(d.cpc_bid_micros, 10);
                                    else if (typeof d.cpc_bid_micros === "number")
                                        m.cpc_bid_micros = d.cpc_bid_micros;
                                    else if (typeof d.cpc_bid_micros === "object")
                                        m.cpc_bid_micros = new $util.LongBits(d.cpc_bid_micros.low >>> 0, d.cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.cpm_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpm_bid_micros = $util.Long.fromValue(d.cpm_bid_micros)).unsigned = false;
                                    else if (typeof d.cpm_bid_micros === "string")
                                        m.cpm_bid_micros = parseInt(d.cpm_bid_micros, 10);
                                    else if (typeof d.cpm_bid_micros === "number")
                                        m.cpm_bid_micros = d.cpm_bid_micros;
                                    else if (typeof d.cpm_bid_micros === "object")
                                        m.cpm_bid_micros = new $util.LongBits(d.cpm_bid_micros.low >>> 0, d.cpm_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.cpv_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpv_bid_micros = $util.Long.fromValue(d.cpv_bid_micros)).unsigned = false;
                                    else if (typeof d.cpv_bid_micros === "string")
                                        m.cpv_bid_micros = parseInt(d.cpv_bid_micros, 10);
                                    else if (typeof d.cpv_bid_micros === "number")
                                        m.cpv_bid_micros = d.cpv_bid_micros;
                                    else if (typeof d.cpv_bid_micros === "object")
                                        m.cpv_bid_micros = new $util.LongBits(d.cpv_bid_micros.low >>> 0, d.cpv_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.percent_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.percent_cpc_bid_micros = $util.Long.fromValue(d.percent_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.percent_cpc_bid_micros === "string")
                                        m.percent_cpc_bid_micros = parseInt(d.percent_cpc_bid_micros, 10);
                                    else if (typeof d.percent_cpc_bid_micros === "number")
                                        m.percent_cpc_bid_micros = d.percent_cpc_bid_micros;
                                    else if (typeof d.percent_cpc_bid_micros === "object")
                                        m.percent_cpc_bid_micros = new $util.LongBits(d.percent_cpc_bid_micros.low >>> 0, d.percent_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.effective_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.effective_cpc_bid_micros = $util.Long.fromValue(d.effective_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.effective_cpc_bid_micros === "string")
                                        m.effective_cpc_bid_micros = parseInt(d.effective_cpc_bid_micros, 10);
                                    else if (typeof d.effective_cpc_bid_micros === "number")
                                        m.effective_cpc_bid_micros = d.effective_cpc_bid_micros;
                                    else if (typeof d.effective_cpc_bid_micros === "object")
                                        m.effective_cpc_bid_micros = new $util.LongBits(d.effective_cpc_bid_micros.low >>> 0, d.effective_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.effective_cpm_bid_micros != null) {
                                    if ($util.Long)
                                        (m.effective_cpm_bid_micros = $util.Long.fromValue(d.effective_cpm_bid_micros)).unsigned = false;
                                    else if (typeof d.effective_cpm_bid_micros === "string")
                                        m.effective_cpm_bid_micros = parseInt(d.effective_cpm_bid_micros, 10);
                                    else if (typeof d.effective_cpm_bid_micros === "number")
                                        m.effective_cpm_bid_micros = d.effective_cpm_bid_micros;
                                    else if (typeof d.effective_cpm_bid_micros === "object")
                                        m.effective_cpm_bid_micros = new $util.LongBits(d.effective_cpm_bid_micros.low >>> 0, d.effective_cpm_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.effective_cpv_bid_micros != null) {
                                    if ($util.Long)
                                        (m.effective_cpv_bid_micros = $util.Long.fromValue(d.effective_cpv_bid_micros)).unsigned = false;
                                    else if (typeof d.effective_cpv_bid_micros === "string")
                                        m.effective_cpv_bid_micros = parseInt(d.effective_cpv_bid_micros, 10);
                                    else if (typeof d.effective_cpv_bid_micros === "number")
                                        m.effective_cpv_bid_micros = d.effective_cpv_bid_micros;
                                    else if (typeof d.effective_cpv_bid_micros === "object")
                                        m.effective_cpv_bid_micros = new $util.LongBits(d.effective_cpv_bid_micros.low >>> 0, d.effective_cpv_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.effective_percent_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.effective_percent_cpc_bid_micros = $util.Long.fromValue(d.effective_percent_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.effective_percent_cpc_bid_micros === "string")
                                        m.effective_percent_cpc_bid_micros = parseInt(d.effective_percent_cpc_bid_micros, 10);
                                    else if (typeof d.effective_percent_cpc_bid_micros === "number")
                                        m.effective_percent_cpc_bid_micros = d.effective_percent_cpc_bid_micros;
                                    else if (typeof d.effective_percent_cpc_bid_micros === "object")
                                        m.effective_percent_cpc_bid_micros = new $util.LongBits(d.effective_percent_cpc_bid_micros.low >>> 0, d.effective_percent_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                switch (d.effective_cpc_bid_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.effective_cpc_bid_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.effective_cpc_bid_source = 1;
                                    break;
                                case "CAMPAIGN_BIDDING_STRATEGY":
                                case 5:
                                    m.effective_cpc_bid_source = 5;
                                    break;
                                case "AD_GROUP":
                                case 6:
                                    m.effective_cpc_bid_source = 6;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 7:
                                    m.effective_cpc_bid_source = 7;
                                    break;
                                }
                                switch (d.effective_cpm_bid_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.effective_cpm_bid_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.effective_cpm_bid_source = 1;
                                    break;
                                case "CAMPAIGN_BIDDING_STRATEGY":
                                case 5:
                                    m.effective_cpm_bid_source = 5;
                                    break;
                                case "AD_GROUP":
                                case 6:
                                    m.effective_cpm_bid_source = 6;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 7:
                                    m.effective_cpm_bid_source = 7;
                                    break;
                                }
                                switch (d.effective_cpv_bid_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.effective_cpv_bid_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.effective_cpv_bid_source = 1;
                                    break;
                                case "CAMPAIGN_BIDDING_STRATEGY":
                                case 5:
                                    m.effective_cpv_bid_source = 5;
                                    break;
                                case "AD_GROUP":
                                case 6:
                                    m.effective_cpv_bid_source = 6;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 7:
                                    m.effective_cpv_bid_source = 7;
                                    break;
                                }
                                switch (d.effective_percent_cpc_bid_source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.effective_percent_cpc_bid_source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.effective_percent_cpc_bid_source = 1;
                                    break;
                                case "CAMPAIGN_BIDDING_STRATEGY":
                                case 5:
                                    m.effective_percent_cpc_bid_source = 5;
                                    break;
                                case "AD_GROUP":
                                case 6:
                                    m.effective_percent_cpc_bid_source = 6;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 7:
                                    m.effective_percent_cpc_bid_source = 7;
                                    break;
                                }
                                if (d.position_estimates != null) {
                                    if (typeof d.position_estimates !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.position_estimates: object expected");
                                    m.position_estimates = $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates.fromObject(d.position_estimates);
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.keyword != null) {
                                    if (typeof d.keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.keyword: object expected");
                                    m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword);
                                }
                                if (d.placement != null) {
                                    if (typeof d.placement !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.placement: object expected");
                                    m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.fromObject(d.placement);
                                }
                                if (d.mobile_app_category != null) {
                                    if (typeof d.mobile_app_category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.mobile_app_category: object expected");
                                    m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.fromObject(d.mobile_app_category);
                                }
                                if (d.mobile_application != null) {
                                    if (typeof d.mobile_application !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.mobile_application: object expected");
                                    m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.fromObject(d.mobile_application);
                                }
                                if (d.listing_group != null) {
                                    if (typeof d.listing_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.listing_group: object expected");
                                    m.listing_group = $root.google.ads.googleads.v11.common.ListingGroupInfo.fromObject(d.listing_group);
                                }
                                if (d.age_range != null) {
                                    if (typeof d.age_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.age_range: object expected");
                                    m.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.fromObject(d.age_range);
                                }
                                if (d.gender != null) {
                                    if (typeof d.gender !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.gender: object expected");
                                    m.gender = $root.google.ads.googleads.v11.common.GenderInfo.fromObject(d.gender);
                                }
                                if (d.income_range != null) {
                                    if (typeof d.income_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.income_range: object expected");
                                    m.income_range = $root.google.ads.googleads.v11.common.IncomeRangeInfo.fromObject(d.income_range);
                                }
                                if (d.parental_status != null) {
                                    if (typeof d.parental_status !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.parental_status: object expected");
                                    m.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusInfo.fromObject(d.parental_status);
                                }
                                if (d.user_list != null) {
                                    if (typeof d.user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.user_list: object expected");
                                    m.user_list = $root.google.ads.googleads.v11.common.UserListInfo.fromObject(d.user_list);
                                }
                                if (d.youtube_video != null) {
                                    if (typeof d.youtube_video !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.youtube_video: object expected");
                                    m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.fromObject(d.youtube_video);
                                }
                                if (d.youtube_channel != null) {
                                    if (typeof d.youtube_channel !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.youtube_channel: object expected");
                                    m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.fromObject(d.youtube_channel);
                                }
                                if (d.topic != null) {
                                    if (typeof d.topic !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.topic: object expected");
                                    m.topic = $root.google.ads.googleads.v11.common.TopicInfo.fromObject(d.topic);
                                }
                                if (d.user_interest != null) {
                                    if (typeof d.user_interest !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.user_interest: object expected");
                                    m.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.fromObject(d.user_interest);
                                }
                                if (d.webpage != null) {
                                    if (typeof d.webpage !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.webpage: object expected");
                                    m.webpage = $root.google.ads.googleads.v11.common.WebpageInfo.fromObject(d.webpage);
                                }
                                if (d.app_payment_model != null) {
                                    if (typeof d.app_payment_model !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.app_payment_model: object expected");
                                    m.app_payment_model = $root.google.ads.googleads.v11.common.AppPaymentModelInfo.fromObject(d.app_payment_model);
                                }
                                if (d.custom_affinity != null) {
                                    if (typeof d.custom_affinity !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.custom_affinity: object expected");
                                    m.custom_affinity = $root.google.ads.googleads.v11.common.CustomAffinityInfo.fromObject(d.custom_affinity);
                                }
                                if (d.custom_intent != null) {
                                    if (typeof d.custom_intent !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.custom_intent: object expected");
                                    m.custom_intent = $root.google.ads.googleads.v11.common.CustomIntentInfo.fromObject(d.custom_intent);
                                }
                                if (d.custom_audience != null) {
                                    if (typeof d.custom_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.custom_audience: object expected");
                                    m.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceInfo.fromObject(d.custom_audience);
                                }
                                if (d.combined_audience != null) {
                                    if (typeof d.combined_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.combined_audience: object expected");
                                    m.combined_audience = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.fromObject(d.combined_audience);
                                }
                                if (d.audience != null) {
                                    if (typeof d.audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterion.audience: object expected");
                                    m.audience = $root.google.ads.googleads.v11.common.AudienceInfo.fromObject(d.audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupCriterion} m AdGroupCriterion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.url_custom_parameters = [];
                                    d.disapproval_reasons = [];
                                    d.labels = [];
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.quality_info = null;
                                    d.position_estimates = null;
                                    d.effective_cpc_bid_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.effective_cpm_bid_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.effective_cpv_bid_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.effective_percent_cpc_bid_source = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.system_serving_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.display_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AdGroupCriterionStatusEnum.AdGroupCriterionStatus[m.status] : m.status;
                                }
                                if (m.quality_info != null && m.hasOwnProperty("quality_info")) {
                                    d.quality_info = $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo.toObject(m.quality_info, o);
                                }
                                if (m.position_estimates != null && m.hasOwnProperty("position_estimates")) {
                                    d.position_estimates = $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates.toObject(m.position_estimates, o);
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.effective_cpc_bid_source != null && m.hasOwnProperty("effective_cpc_bid_source")) {
                                    d.effective_cpc_bid_source = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource[m.effective_cpc_bid_source] : m.effective_cpc_bid_source;
                                }
                                if (m.effective_cpm_bid_source != null && m.hasOwnProperty("effective_cpm_bid_source")) {
                                    d.effective_cpm_bid_source = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource[m.effective_cpm_bid_source] : m.effective_cpm_bid_source;
                                }
                                if (m.effective_cpv_bid_source != null && m.hasOwnProperty("effective_cpv_bid_source")) {
                                    d.effective_cpv_bid_source = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource[m.effective_cpv_bid_source] : m.effective_cpv_bid_source;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType[m.type] : m.type;
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword, o);
                                    if (o.oneofs)
                                        d.criterion = "keyword";
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    d.placement = $root.google.ads.googleads.v11.common.PlacementInfo.toObject(m.placement, o);
                                    if (o.oneofs)
                                        d.criterion = "placement";
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    d.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.toObject(m.mobile_app_category, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_app_category";
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    d.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.toObject(m.mobile_application, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_application";
                                }
                                if (m.listing_group != null && m.hasOwnProperty("listing_group")) {
                                    d.listing_group = $root.google.ads.googleads.v11.common.ListingGroupInfo.toObject(m.listing_group, o);
                                    if (o.oneofs)
                                        d.criterion = "listing_group";
                                }
                                if (m.effective_percent_cpc_bid_source != null && m.hasOwnProperty("effective_percent_cpc_bid_source")) {
                                    d.effective_percent_cpc_bid_source = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingSourceEnum.BiddingSource[m.effective_percent_cpc_bid_source] : m.effective_percent_cpc_bid_source;
                                }
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    d.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.toObject(m.age_range, o);
                                    if (o.oneofs)
                                        d.criterion = "age_range";
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    d.gender = $root.google.ads.googleads.v11.common.GenderInfo.toObject(m.gender, o);
                                    if (o.oneofs)
                                        d.criterion = "gender";
                                }
                                if (m.income_range != null && m.hasOwnProperty("income_range")) {
                                    d.income_range = $root.google.ads.googleads.v11.common.IncomeRangeInfo.toObject(m.income_range, o);
                                    if (o.oneofs)
                                        d.criterion = "income_range";
                                }
                                if (m.parental_status != null && m.hasOwnProperty("parental_status")) {
                                    d.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusInfo.toObject(m.parental_status, o);
                                    if (o.oneofs)
                                        d.criterion = "parental_status";
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    d.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.toObject(m.youtube_video, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_video";
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    d.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.toObject(m.youtube_channel, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_channel";
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = $root.google.ads.googleads.v11.common.UserListInfo.toObject(m.user_list, o);
                                    if (o.oneofs)
                                        d.criterion = "user_list";
                                }
                                if (m.topic != null && m.hasOwnProperty("topic")) {
                                    d.topic = $root.google.ads.googleads.v11.common.TopicInfo.toObject(m.topic, o);
                                    if (o.oneofs)
                                        d.criterion = "topic";
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    d.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.toObject(m.user_interest, o);
                                    if (o.oneofs)
                                        d.criterion = "user_interest";
                                }
                                if (m.webpage != null && m.hasOwnProperty("webpage")) {
                                    d.webpage = $root.google.ads.googleads.v11.common.WebpageInfo.toObject(m.webpage, o);
                                    if (o.oneofs)
                                        d.criterion = "webpage";
                                }
                                if (m.app_payment_model != null && m.hasOwnProperty("app_payment_model")) {
                                    d.app_payment_model = $root.google.ads.googleads.v11.common.AppPaymentModelInfo.toObject(m.app_payment_model, o);
                                    if (o.oneofs)
                                        d.criterion = "app_payment_model";
                                }
                                if (m.custom_affinity != null && m.hasOwnProperty("custom_affinity")) {
                                    d.custom_affinity = $root.google.ads.googleads.v11.common.CustomAffinityInfo.toObject(m.custom_affinity, o);
                                    if (o.oneofs)
                                        d.criterion = "custom_affinity";
                                }
                                if (m.custom_intent != null && m.hasOwnProperty("custom_intent")) {
                                    d.custom_intent = $root.google.ads.googleads.v11.common.CustomIntentInfo.toObject(m.custom_intent, o);
                                    if (o.oneofs)
                                        d.criterion = "custom_intent";
                                }
                                if (m.system_serving_status != null && m.hasOwnProperty("system_serving_status")) {
                                    d.system_serving_status = o.enums === String ? $root.google.ads.googleads.v11.enums.CriterionSystemServingStatusEnum.CriterionSystemServingStatus[m.system_serving_status] : m.system_serving_status;
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    d.approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.AdGroupCriterionApprovalStatusEnum.AdGroupCriterionApprovalStatus[m.approval_status] : m.approval_status;
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    if (typeof m.criterion_id === "number")
                                        d.criterion_id = o.longs === String ? String(m.criterion_id) : m.criterion_id;
                                    else
                                        d.criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.criterion_id) : o.longs === Number ? new $util.LongBits(m.criterion_id.low >>> 0, m.criterion_id.high >>> 0).toNumber() : m.criterion_id;
                                    if (o.oneofs)
                                        d._criterion_id = "criterion_id";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    d.negative = m.negative;
                                    if (o.oneofs)
                                        d._negative = "negative";
                                }
                                if (m.disapproval_reasons && m.disapproval_reasons.length) {
                                    d.disapproval_reasons = [];
                                    for (var j = 0; j < m.disapproval_reasons.length; ++j) {
                                        d.disapproval_reasons[j] = m.disapproval_reasons[j];
                                    }
                                }
                                if (m.labels && m.labels.length) {
                                    d.labels = [];
                                    for (var j = 0; j < m.labels.length; ++j) {
                                        d.labels[j] = m.labels[j];
                                    }
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    d.bid_modifier = o.json && !isFinite(m.bid_modifier) ? String(m.bid_modifier) : m.bid_modifier;
                                    if (o.oneofs)
                                        d._bid_modifier = "bid_modifier";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    if (typeof m.cpc_bid_micros === "number")
                                        d.cpc_bid_micros = o.longs === String ? String(m.cpc_bid_micros) : m.cpc_bid_micros;
                                    else
                                        d.cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_micros.low >>> 0, m.cpc_bid_micros.high >>> 0).toNumber() : m.cpc_bid_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_micros = "cpc_bid_micros";
                                }
                                if (m.cpm_bid_micros != null && m.hasOwnProperty("cpm_bid_micros")) {
                                    if (typeof m.cpm_bid_micros === "number")
                                        d.cpm_bid_micros = o.longs === String ? String(m.cpm_bid_micros) : m.cpm_bid_micros;
                                    else
                                        d.cpm_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpm_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpm_bid_micros.low >>> 0, m.cpm_bid_micros.high >>> 0).toNumber() : m.cpm_bid_micros;
                                    if (o.oneofs)
                                        d._cpm_bid_micros = "cpm_bid_micros";
                                }
                                if (m.cpv_bid_micros != null && m.hasOwnProperty("cpv_bid_micros")) {
                                    if (typeof m.cpv_bid_micros === "number")
                                        d.cpv_bid_micros = o.longs === String ? String(m.cpv_bid_micros) : m.cpv_bid_micros;
                                    else
                                        d.cpv_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpv_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpv_bid_micros.low >>> 0, m.cpv_bid_micros.high >>> 0).toNumber() : m.cpv_bid_micros;
                                    if (o.oneofs)
                                        d._cpv_bid_micros = "cpv_bid_micros";
                                }
                                if (m.percent_cpc_bid_micros != null && m.hasOwnProperty("percent_cpc_bid_micros")) {
                                    if (typeof m.percent_cpc_bid_micros === "number")
                                        d.percent_cpc_bid_micros = o.longs === String ? String(m.percent_cpc_bid_micros) : m.percent_cpc_bid_micros;
                                    else
                                        d.percent_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.percent_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.percent_cpc_bid_micros.low >>> 0, m.percent_cpc_bid_micros.high >>> 0).toNumber() : m.percent_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._percent_cpc_bid_micros = "percent_cpc_bid_micros";
                                }
                                if (m.effective_cpc_bid_micros != null && m.hasOwnProperty("effective_cpc_bid_micros")) {
                                    if (typeof m.effective_cpc_bid_micros === "number")
                                        d.effective_cpc_bid_micros = o.longs === String ? String(m.effective_cpc_bid_micros) : m.effective_cpc_bid_micros;
                                    else
                                        d.effective_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.effective_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.effective_cpc_bid_micros.low >>> 0, m.effective_cpc_bid_micros.high >>> 0).toNumber() : m.effective_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._effective_cpc_bid_micros = "effective_cpc_bid_micros";
                                }
                                if (m.effective_cpm_bid_micros != null && m.hasOwnProperty("effective_cpm_bid_micros")) {
                                    if (typeof m.effective_cpm_bid_micros === "number")
                                        d.effective_cpm_bid_micros = o.longs === String ? String(m.effective_cpm_bid_micros) : m.effective_cpm_bid_micros;
                                    else
                                        d.effective_cpm_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.effective_cpm_bid_micros) : o.longs === Number ? new $util.LongBits(m.effective_cpm_bid_micros.low >>> 0, m.effective_cpm_bid_micros.high >>> 0).toNumber() : m.effective_cpm_bid_micros;
                                    if (o.oneofs)
                                        d._effective_cpm_bid_micros = "effective_cpm_bid_micros";
                                }
                                if (m.effective_cpv_bid_micros != null && m.hasOwnProperty("effective_cpv_bid_micros")) {
                                    if (typeof m.effective_cpv_bid_micros === "number")
                                        d.effective_cpv_bid_micros = o.longs === String ? String(m.effective_cpv_bid_micros) : m.effective_cpv_bid_micros;
                                    else
                                        d.effective_cpv_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.effective_cpv_bid_micros) : o.longs === Number ? new $util.LongBits(m.effective_cpv_bid_micros.low >>> 0, m.effective_cpv_bid_micros.high >>> 0).toNumber() : m.effective_cpv_bid_micros;
                                    if (o.oneofs)
                                        d._effective_cpv_bid_micros = "effective_cpv_bid_micros";
                                }
                                if (m.effective_percent_cpc_bid_micros != null && m.hasOwnProperty("effective_percent_cpc_bid_micros")) {
                                    if (typeof m.effective_percent_cpc_bid_micros === "number")
                                        d.effective_percent_cpc_bid_micros = o.longs === String ? String(m.effective_percent_cpc_bid_micros) : m.effective_percent_cpc_bid_micros;
                                    else
                                        d.effective_percent_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.effective_percent_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.effective_percent_cpc_bid_micros.low >>> 0, m.effective_percent_cpc_bid_micros.high >>> 0).toNumber() : m.effective_percent_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._effective_percent_cpc_bid_micros = "effective_percent_cpc_bid_micros";
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    d.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceInfo.toObject(m.custom_audience, o);
                                    if (o.oneofs)
                                        d.criterion = "custom_audience";
                                }
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    d.combined_audience = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.toObject(m.combined_audience, o);
                                    if (o.oneofs)
                                        d.criterion = "combined_audience";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    d.display_name = m.display_name;
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    d.audience = $root.google.ads.googleads.v11.common.AudienceInfo.toObject(m.audience, o);
                                    if (o.oneofs)
                                        d.criterion = "audience";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCriterion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            AdGroupCriterion.QualityInfo = (function() {
    
                                /**
                                 * Properties of a QualityInfo.
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                                 * @interface IQualityInfo
                                 * @property {number|null} [quality_score] QualityInfo quality_score
                                 * @property {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket|null} [creative_quality_score] QualityInfo creative_quality_score
                                 * @property {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket|null} [post_click_quality_score] QualityInfo post_click_quality_score
                                 * @property {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket|null} [search_predicted_ctr] QualityInfo search_predicted_ctr
                                 */
    
                                /**
                                 * Constructs a new QualityInfo.
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                                 * @classdesc Represents a QualityInfo.
                                 * @implements IQualityInfo
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IQualityInfo=} [p] Properties to set
                                 */
                                function QualityInfo(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * QualityInfo quality_score.
                                 * @member {number|null|undefined} quality_score
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @instance
                                 */
                                QualityInfo.prototype.quality_score = null;
    
                                /**
                                 * QualityInfo creative_quality_score.
                                 * @member {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket} creative_quality_score
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @instance
                                 */
                                QualityInfo.prototype.creative_quality_score = 0;
    
                                /**
                                 * QualityInfo post_click_quality_score.
                                 * @member {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket} post_click_quality_score
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @instance
                                 */
                                QualityInfo.prototype.post_click_quality_score = 0;
    
                                /**
                                 * QualityInfo search_predicted_ctr.
                                 * @member {google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket} search_predicted_ctr
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @instance
                                 */
                                QualityInfo.prototype.search_predicted_ctr = 0;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * QualityInfo _quality_score.
                                 * @member {"quality_score"|undefined} _quality_score
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @instance
                                 */
                                Object.defineProperty(QualityInfo.prototype, "_quality_score", {
                                    get: $util.oneOfGetter($oneOfFields = ["quality_score"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new QualityInfo instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IQualityInfo=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo} QualityInfo instance
                                 */
                                QualityInfo.create = function create(properties) {
                                    return new QualityInfo(properties);
                                };
    
                                /**
                                 * Encodes the specified QualityInfo message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IQualityInfo} m QualityInfo message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                QualityInfo.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.creative_quality_score != null && Object.hasOwnProperty.call(m, "creative_quality_score"))
                                        w.uint32(16).int32(m.creative_quality_score);
                                    if (m.post_click_quality_score != null && Object.hasOwnProperty.call(m, "post_click_quality_score"))
                                        w.uint32(24).int32(m.post_click_quality_score);
                                    if (m.search_predicted_ctr != null && Object.hasOwnProperty.call(m, "search_predicted_ctr"))
                                        w.uint32(32).int32(m.search_predicted_ctr);
                                    if (m.quality_score != null && Object.hasOwnProperty.call(m, "quality_score"))
                                        w.uint32(40).int32(m.quality_score);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified QualityInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IQualityInfo} message QualityInfo message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                QualityInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a QualityInfo message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo} QualityInfo
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                QualityInfo.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 5:
                                            m.quality_score = r.int32();
                                            break;
                                        case 2:
                                            m.creative_quality_score = r.int32();
                                            break;
                                        case 3:
                                            m.post_click_quality_score = r.int32();
                                            break;
                                        case 4:
                                            m.search_predicted_ctr = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a QualityInfo message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo} QualityInfo
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                QualityInfo.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a QualityInfo message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                QualityInfo.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.quality_score != null && m.hasOwnProperty("quality_score")) {
                                        p._quality_score = 1;
                                        if (!$util.isInteger(m.quality_score))
                                            return "quality_score: integer expected";
                                    }
                                    if (m.creative_quality_score != null && m.hasOwnProperty("creative_quality_score")) {
                                        switch (m.creative_quality_score) {
                                        default:
                                            return "creative_quality_score: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    if (m.post_click_quality_score != null && m.hasOwnProperty("post_click_quality_score")) {
                                        switch (m.post_click_quality_score) {
                                        default:
                                            return "post_click_quality_score: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    if (m.search_predicted_ctr != null && m.hasOwnProperty("search_predicted_ctr")) {
                                        switch (m.search_predicted_ctr) {
                                        default:
                                            return "search_predicted_ctr: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a QualityInfo message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo} QualityInfo
                                 */
                                QualityInfo.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo();
                                    if (d.quality_score != null) {
                                        m.quality_score = d.quality_score | 0;
                                    }
                                    switch (d.creative_quality_score) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.creative_quality_score = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.creative_quality_score = 1;
                                        break;
                                    case "BELOW_AVERAGE":
                                    case 2:
                                        m.creative_quality_score = 2;
                                        break;
                                    case "AVERAGE":
                                    case 3:
                                        m.creative_quality_score = 3;
                                        break;
                                    case "ABOVE_AVERAGE":
                                    case 4:
                                        m.creative_quality_score = 4;
                                        break;
                                    }
                                    switch (d.post_click_quality_score) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.post_click_quality_score = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.post_click_quality_score = 1;
                                        break;
                                    case "BELOW_AVERAGE":
                                    case 2:
                                        m.post_click_quality_score = 2;
                                        break;
                                    case "AVERAGE":
                                    case 3:
                                        m.post_click_quality_score = 3;
                                        break;
                                    case "ABOVE_AVERAGE":
                                    case 4:
                                        m.post_click_quality_score = 4;
                                        break;
                                    }
                                    switch (d.search_predicted_ctr) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.search_predicted_ctr = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.search_predicted_ctr = 1;
                                        break;
                                    case "BELOW_AVERAGE":
                                    case 2:
                                        m.search_predicted_ctr = 2;
                                        break;
                                    case "AVERAGE":
                                    case 3:
                                        m.search_predicted_ctr = 3;
                                        break;
                                    case "ABOVE_AVERAGE":
                                    case 4:
                                        m.search_predicted_ctr = 4;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a QualityInfo message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo} m QualityInfo
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                QualityInfo.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.creative_quality_score = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.post_click_quality_score = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.search_predicted_ctr = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.creative_quality_score != null && m.hasOwnProperty("creative_quality_score")) {
                                        d.creative_quality_score = o.enums === String ? $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket[m.creative_quality_score] : m.creative_quality_score;
                                    }
                                    if (m.post_click_quality_score != null && m.hasOwnProperty("post_click_quality_score")) {
                                        d.post_click_quality_score = o.enums === String ? $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket[m.post_click_quality_score] : m.post_click_quality_score;
                                    }
                                    if (m.search_predicted_ctr != null && m.hasOwnProperty("search_predicted_ctr")) {
                                        d.search_predicted_ctr = o.enums === String ? $root.google.ads.googleads.v11.enums.QualityScoreBucketEnum.QualityScoreBucket[m.search_predicted_ctr] : m.search_predicted_ctr;
                                    }
                                    if (m.quality_score != null && m.hasOwnProperty("quality_score")) {
                                        d.quality_score = m.quality_score;
                                        if (o.oneofs)
                                            d._quality_score = "quality_score";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this QualityInfo to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.QualityInfo
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                QualityInfo.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return QualityInfo;
                            })();
    
                            AdGroupCriterion.PositionEstimates = (function() {
    
                                /**
                                 * Properties of a PositionEstimates.
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                                 * @interface IPositionEstimates
                                 * @property {number|null} [first_page_cpc_micros] PositionEstimates first_page_cpc_micros
                                 * @property {number|null} [first_position_cpc_micros] PositionEstimates first_position_cpc_micros
                                 * @property {number|null} [top_of_page_cpc_micros] PositionEstimates top_of_page_cpc_micros
                                 * @property {number|null} [estimated_add_clicks_at_first_position_cpc] PositionEstimates estimated_add_clicks_at_first_position_cpc
                                 * @property {number|null} [estimated_add_cost_at_first_position_cpc] PositionEstimates estimated_add_cost_at_first_position_cpc
                                 */
    
                                /**
                                 * Constructs a new PositionEstimates.
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion
                                 * @classdesc Represents a PositionEstimates.
                                 * @implements IPositionEstimates
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IPositionEstimates=} [p] Properties to set
                                 */
                                function PositionEstimates(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * PositionEstimates first_page_cpc_micros.
                                 * @member {number|null|undefined} first_page_cpc_micros
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                PositionEstimates.prototype.first_page_cpc_micros = null;
    
                                /**
                                 * PositionEstimates first_position_cpc_micros.
                                 * @member {number|null|undefined} first_position_cpc_micros
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                PositionEstimates.prototype.first_position_cpc_micros = null;
    
                                /**
                                 * PositionEstimates top_of_page_cpc_micros.
                                 * @member {number|null|undefined} top_of_page_cpc_micros
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                PositionEstimates.prototype.top_of_page_cpc_micros = null;
    
                                /**
                                 * PositionEstimates estimated_add_clicks_at_first_position_cpc.
                                 * @member {number|null|undefined} estimated_add_clicks_at_first_position_cpc
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                PositionEstimates.prototype.estimated_add_clicks_at_first_position_cpc = null;
    
                                /**
                                 * PositionEstimates estimated_add_cost_at_first_position_cpc.
                                 * @member {number|null|undefined} estimated_add_cost_at_first_position_cpc
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                PositionEstimates.prototype.estimated_add_cost_at_first_position_cpc = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * PositionEstimates _first_page_cpc_micros.
                                 * @member {"first_page_cpc_micros"|undefined} _first_page_cpc_micros
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                Object.defineProperty(PositionEstimates.prototype, "_first_page_cpc_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["first_page_cpc_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PositionEstimates _first_position_cpc_micros.
                                 * @member {"first_position_cpc_micros"|undefined} _first_position_cpc_micros
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                Object.defineProperty(PositionEstimates.prototype, "_first_position_cpc_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["first_position_cpc_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PositionEstimates _top_of_page_cpc_micros.
                                 * @member {"top_of_page_cpc_micros"|undefined} _top_of_page_cpc_micros
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                Object.defineProperty(PositionEstimates.prototype, "_top_of_page_cpc_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["top_of_page_cpc_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PositionEstimates _estimated_add_clicks_at_first_position_cpc.
                                 * @member {"estimated_add_clicks_at_first_position_cpc"|undefined} _estimated_add_clicks_at_first_position_cpc
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                Object.defineProperty(PositionEstimates.prototype, "_estimated_add_clicks_at_first_position_cpc", {
                                    get: $util.oneOfGetter($oneOfFields = ["estimated_add_clicks_at_first_position_cpc"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PositionEstimates _estimated_add_cost_at_first_position_cpc.
                                 * @member {"estimated_add_cost_at_first_position_cpc"|undefined} _estimated_add_cost_at_first_position_cpc
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 */
                                Object.defineProperty(PositionEstimates.prototype, "_estimated_add_cost_at_first_position_cpc", {
                                    get: $util.oneOfGetter($oneOfFields = ["estimated_add_cost_at_first_position_cpc"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new PositionEstimates instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IPositionEstimates=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates} PositionEstimates instance
                                 */
                                PositionEstimates.create = function create(properties) {
                                    return new PositionEstimates(properties);
                                };
    
                                /**
                                 * Encodes the specified PositionEstimates message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IPositionEstimates} m PositionEstimates message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PositionEstimates.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.first_page_cpc_micros != null && Object.hasOwnProperty.call(m, "first_page_cpc_micros"))
                                        w.uint32(48).int64(m.first_page_cpc_micros);
                                    if (m.first_position_cpc_micros != null && Object.hasOwnProperty.call(m, "first_position_cpc_micros"))
                                        w.uint32(56).int64(m.first_position_cpc_micros);
                                    if (m.top_of_page_cpc_micros != null && Object.hasOwnProperty.call(m, "top_of_page_cpc_micros"))
                                        w.uint32(64).int64(m.top_of_page_cpc_micros);
                                    if (m.estimated_add_clicks_at_first_position_cpc != null && Object.hasOwnProperty.call(m, "estimated_add_clicks_at_first_position_cpc"))
                                        w.uint32(72).int64(m.estimated_add_clicks_at_first_position_cpc);
                                    if (m.estimated_add_cost_at_first_position_cpc != null && Object.hasOwnProperty.call(m, "estimated_add_cost_at_first_position_cpc"))
                                        w.uint32(80).int64(m.estimated_add_cost_at_first_position_cpc);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified PositionEstimates message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.IPositionEstimates} message PositionEstimates message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PositionEstimates.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a PositionEstimates message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates} PositionEstimates
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PositionEstimates.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 6:
                                            m.first_page_cpc_micros = r.int64();
                                            break;
                                        case 7:
                                            m.first_position_cpc_micros = r.int64();
                                            break;
                                        case 8:
                                            m.top_of_page_cpc_micros = r.int64();
                                            break;
                                        case 9:
                                            m.estimated_add_clicks_at_first_position_cpc = r.int64();
                                            break;
                                        case 10:
                                            m.estimated_add_cost_at_first_position_cpc = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a PositionEstimates message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates} PositionEstimates
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PositionEstimates.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a PositionEstimates message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                PositionEstimates.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.first_page_cpc_micros != null && m.hasOwnProperty("first_page_cpc_micros")) {
                                        p._first_page_cpc_micros = 1;
                                        if (!$util.isInteger(m.first_page_cpc_micros) && !(m.first_page_cpc_micros && $util.isInteger(m.first_page_cpc_micros.low) && $util.isInteger(m.first_page_cpc_micros.high)))
                                            return "first_page_cpc_micros: integer|Long expected";
                                    }
                                    if (m.first_position_cpc_micros != null && m.hasOwnProperty("first_position_cpc_micros")) {
                                        p._first_position_cpc_micros = 1;
                                        if (!$util.isInteger(m.first_position_cpc_micros) && !(m.first_position_cpc_micros && $util.isInteger(m.first_position_cpc_micros.low) && $util.isInteger(m.first_position_cpc_micros.high)))
                                            return "first_position_cpc_micros: integer|Long expected";
                                    }
                                    if (m.top_of_page_cpc_micros != null && m.hasOwnProperty("top_of_page_cpc_micros")) {
                                        p._top_of_page_cpc_micros = 1;
                                        if (!$util.isInteger(m.top_of_page_cpc_micros) && !(m.top_of_page_cpc_micros && $util.isInteger(m.top_of_page_cpc_micros.low) && $util.isInteger(m.top_of_page_cpc_micros.high)))
                                            return "top_of_page_cpc_micros: integer|Long expected";
                                    }
                                    if (m.estimated_add_clicks_at_first_position_cpc != null && m.hasOwnProperty("estimated_add_clicks_at_first_position_cpc")) {
                                        p._estimated_add_clicks_at_first_position_cpc = 1;
                                        if (!$util.isInteger(m.estimated_add_clicks_at_first_position_cpc) && !(m.estimated_add_clicks_at_first_position_cpc && $util.isInteger(m.estimated_add_clicks_at_first_position_cpc.low) && $util.isInteger(m.estimated_add_clicks_at_first_position_cpc.high)))
                                            return "estimated_add_clicks_at_first_position_cpc: integer|Long expected";
                                    }
                                    if (m.estimated_add_cost_at_first_position_cpc != null && m.hasOwnProperty("estimated_add_cost_at_first_position_cpc")) {
                                        p._estimated_add_cost_at_first_position_cpc = 1;
                                        if (!$util.isInteger(m.estimated_add_cost_at_first_position_cpc) && !(m.estimated_add_cost_at_first_position_cpc && $util.isInteger(m.estimated_add_cost_at_first_position_cpc.low) && $util.isInteger(m.estimated_add_cost_at_first_position_cpc.high)))
                                            return "estimated_add_cost_at_first_position_cpc: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a PositionEstimates message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates} PositionEstimates
                                 */
                                PositionEstimates.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates();
                                    if (d.first_page_cpc_micros != null) {
                                        if ($util.Long)
                                            (m.first_page_cpc_micros = $util.Long.fromValue(d.first_page_cpc_micros)).unsigned = false;
                                        else if (typeof d.first_page_cpc_micros === "string")
                                            m.first_page_cpc_micros = parseInt(d.first_page_cpc_micros, 10);
                                        else if (typeof d.first_page_cpc_micros === "number")
                                            m.first_page_cpc_micros = d.first_page_cpc_micros;
                                        else if (typeof d.first_page_cpc_micros === "object")
                                            m.first_page_cpc_micros = new $util.LongBits(d.first_page_cpc_micros.low >>> 0, d.first_page_cpc_micros.high >>> 0).toNumber();
                                    }
                                    if (d.first_position_cpc_micros != null) {
                                        if ($util.Long)
                                            (m.first_position_cpc_micros = $util.Long.fromValue(d.first_position_cpc_micros)).unsigned = false;
                                        else if (typeof d.first_position_cpc_micros === "string")
                                            m.first_position_cpc_micros = parseInt(d.first_position_cpc_micros, 10);
                                        else if (typeof d.first_position_cpc_micros === "number")
                                            m.first_position_cpc_micros = d.first_position_cpc_micros;
                                        else if (typeof d.first_position_cpc_micros === "object")
                                            m.first_position_cpc_micros = new $util.LongBits(d.first_position_cpc_micros.low >>> 0, d.first_position_cpc_micros.high >>> 0).toNumber();
                                    }
                                    if (d.top_of_page_cpc_micros != null) {
                                        if ($util.Long)
                                            (m.top_of_page_cpc_micros = $util.Long.fromValue(d.top_of_page_cpc_micros)).unsigned = false;
                                        else if (typeof d.top_of_page_cpc_micros === "string")
                                            m.top_of_page_cpc_micros = parseInt(d.top_of_page_cpc_micros, 10);
                                        else if (typeof d.top_of_page_cpc_micros === "number")
                                            m.top_of_page_cpc_micros = d.top_of_page_cpc_micros;
                                        else if (typeof d.top_of_page_cpc_micros === "object")
                                            m.top_of_page_cpc_micros = new $util.LongBits(d.top_of_page_cpc_micros.low >>> 0, d.top_of_page_cpc_micros.high >>> 0).toNumber();
                                    }
                                    if (d.estimated_add_clicks_at_first_position_cpc != null) {
                                        if ($util.Long)
                                            (m.estimated_add_clicks_at_first_position_cpc = $util.Long.fromValue(d.estimated_add_clicks_at_first_position_cpc)).unsigned = false;
                                        else if (typeof d.estimated_add_clicks_at_first_position_cpc === "string")
                                            m.estimated_add_clicks_at_first_position_cpc = parseInt(d.estimated_add_clicks_at_first_position_cpc, 10);
                                        else if (typeof d.estimated_add_clicks_at_first_position_cpc === "number")
                                            m.estimated_add_clicks_at_first_position_cpc = d.estimated_add_clicks_at_first_position_cpc;
                                        else if (typeof d.estimated_add_clicks_at_first_position_cpc === "object")
                                            m.estimated_add_clicks_at_first_position_cpc = new $util.LongBits(d.estimated_add_clicks_at_first_position_cpc.low >>> 0, d.estimated_add_clicks_at_first_position_cpc.high >>> 0).toNumber();
                                    }
                                    if (d.estimated_add_cost_at_first_position_cpc != null) {
                                        if ($util.Long)
                                            (m.estimated_add_cost_at_first_position_cpc = $util.Long.fromValue(d.estimated_add_cost_at_first_position_cpc)).unsigned = false;
                                        else if (typeof d.estimated_add_cost_at_first_position_cpc === "string")
                                            m.estimated_add_cost_at_first_position_cpc = parseInt(d.estimated_add_cost_at_first_position_cpc, 10);
                                        else if (typeof d.estimated_add_cost_at_first_position_cpc === "number")
                                            m.estimated_add_cost_at_first_position_cpc = d.estimated_add_cost_at_first_position_cpc;
                                        else if (typeof d.estimated_add_cost_at_first_position_cpc === "object")
                                            m.estimated_add_cost_at_first_position_cpc = new $util.LongBits(d.estimated_add_cost_at_first_position_cpc.low >>> 0, d.estimated_add_cost_at_first_position_cpc.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a PositionEstimates message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates} m PositionEstimates
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                PositionEstimates.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.first_page_cpc_micros != null && m.hasOwnProperty("first_page_cpc_micros")) {
                                        if (typeof m.first_page_cpc_micros === "number")
                                            d.first_page_cpc_micros = o.longs === String ? String(m.first_page_cpc_micros) : m.first_page_cpc_micros;
                                        else
                                            d.first_page_cpc_micros = o.longs === String ? $util.Long.prototype.toString.call(m.first_page_cpc_micros) : o.longs === Number ? new $util.LongBits(m.first_page_cpc_micros.low >>> 0, m.first_page_cpc_micros.high >>> 0).toNumber() : m.first_page_cpc_micros;
                                        if (o.oneofs)
                                            d._first_page_cpc_micros = "first_page_cpc_micros";
                                    }
                                    if (m.first_position_cpc_micros != null && m.hasOwnProperty("first_position_cpc_micros")) {
                                        if (typeof m.first_position_cpc_micros === "number")
                                            d.first_position_cpc_micros = o.longs === String ? String(m.first_position_cpc_micros) : m.first_position_cpc_micros;
                                        else
                                            d.first_position_cpc_micros = o.longs === String ? $util.Long.prototype.toString.call(m.first_position_cpc_micros) : o.longs === Number ? new $util.LongBits(m.first_position_cpc_micros.low >>> 0, m.first_position_cpc_micros.high >>> 0).toNumber() : m.first_position_cpc_micros;
                                        if (o.oneofs)
                                            d._first_position_cpc_micros = "first_position_cpc_micros";
                                    }
                                    if (m.top_of_page_cpc_micros != null && m.hasOwnProperty("top_of_page_cpc_micros")) {
                                        if (typeof m.top_of_page_cpc_micros === "number")
                                            d.top_of_page_cpc_micros = o.longs === String ? String(m.top_of_page_cpc_micros) : m.top_of_page_cpc_micros;
                                        else
                                            d.top_of_page_cpc_micros = o.longs === String ? $util.Long.prototype.toString.call(m.top_of_page_cpc_micros) : o.longs === Number ? new $util.LongBits(m.top_of_page_cpc_micros.low >>> 0, m.top_of_page_cpc_micros.high >>> 0).toNumber() : m.top_of_page_cpc_micros;
                                        if (o.oneofs)
                                            d._top_of_page_cpc_micros = "top_of_page_cpc_micros";
                                    }
                                    if (m.estimated_add_clicks_at_first_position_cpc != null && m.hasOwnProperty("estimated_add_clicks_at_first_position_cpc")) {
                                        if (typeof m.estimated_add_clicks_at_first_position_cpc === "number")
                                            d.estimated_add_clicks_at_first_position_cpc = o.longs === String ? String(m.estimated_add_clicks_at_first_position_cpc) : m.estimated_add_clicks_at_first_position_cpc;
                                        else
                                            d.estimated_add_clicks_at_first_position_cpc = o.longs === String ? $util.Long.prototype.toString.call(m.estimated_add_clicks_at_first_position_cpc) : o.longs === Number ? new $util.LongBits(m.estimated_add_clicks_at_first_position_cpc.low >>> 0, m.estimated_add_clicks_at_first_position_cpc.high >>> 0).toNumber() : m.estimated_add_clicks_at_first_position_cpc;
                                        if (o.oneofs)
                                            d._estimated_add_clicks_at_first_position_cpc = "estimated_add_clicks_at_first_position_cpc";
                                    }
                                    if (m.estimated_add_cost_at_first_position_cpc != null && m.hasOwnProperty("estimated_add_cost_at_first_position_cpc")) {
                                        if (typeof m.estimated_add_cost_at_first_position_cpc === "number")
                                            d.estimated_add_cost_at_first_position_cpc = o.longs === String ? String(m.estimated_add_cost_at_first_position_cpc) : m.estimated_add_cost_at_first_position_cpc;
                                        else
                                            d.estimated_add_cost_at_first_position_cpc = o.longs === String ? $util.Long.prototype.toString.call(m.estimated_add_cost_at_first_position_cpc) : o.longs === Number ? new $util.LongBits(m.estimated_add_cost_at_first_position_cpc.low >>> 0, m.estimated_add_cost_at_first_position_cpc.high >>> 0).toNumber() : m.estimated_add_cost_at_first_position_cpc;
                                        if (o.oneofs)
                                            d._estimated_add_cost_at_first_position_cpc = "estimated_add_cost_at_first_position_cpc";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this PositionEstimates to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AdGroupCriterion.PositionEstimates
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                PositionEstimates.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return PositionEstimates;
                            })();
    
                            return AdGroupCriterion;
                        })();
    
                        resources.AdGroupCriterionCustomizer = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupCriterionCustomizer
                             * @property {string|null} [resource_name] AdGroupCriterionCustomizer resource_name
                             * @property {string|null} [ad_group_criterion] AdGroupCriterionCustomizer ad_group_criterion
                             * @property {string|null} [customizer_attribute] AdGroupCriterionCustomizer customizer_attribute
                             * @property {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus|null} [status] AdGroupCriterionCustomizer status
                             * @property {google.ads.googleads.v11.common.ICustomizerValue|null} [value] AdGroupCriterionCustomizer value
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupCriterionCustomizer.
                             * @implements IAdGroupCriterionCustomizer
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer=} [p] Properties to set
                             */
                            function AdGroupCriterionCustomizer(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCriterionCustomizer resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @instance
                             */
                            AdGroupCriterionCustomizer.prototype.resource_name = "";
    
                            /**
                             * AdGroupCriterionCustomizer ad_group_criterion.
                             * @member {string|null|undefined} ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @instance
                             */
                            AdGroupCriterionCustomizer.prototype.ad_group_criterion = null;
    
                            /**
                             * AdGroupCriterionCustomizer customizer_attribute.
                             * @member {string} customizer_attribute
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @instance
                             */
                            AdGroupCriterionCustomizer.prototype.customizer_attribute = "";
    
                            /**
                             * AdGroupCriterionCustomizer status.
                             * @member {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus} status
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @instance
                             */
                            AdGroupCriterionCustomizer.prototype.status = 0;
    
                            /**
                             * AdGroupCriterionCustomizer value.
                             * @member {google.ads.googleads.v11.common.ICustomizerValue|null|undefined} value
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @instance
                             */
                            AdGroupCriterionCustomizer.prototype.value = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCriterionCustomizer _ad_group_criterion.
                             * @member {"ad_group_criterion"|undefined} _ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionCustomizer.prototype, "_ad_group_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCriterionCustomizer instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionCustomizer} AdGroupCriterionCustomizer instance
                             */
                            AdGroupCriterionCustomizer.create = function create(properties) {
                                return new AdGroupCriterionCustomizer(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionCustomizer message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer} m AdGroupCriterionCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionCustomizer.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                    w.uint32(18).string(m.ad_group_criterion);
                                if (m.customizer_attribute != null && Object.hasOwnProperty.call(m, "customizer_attribute"))
                                    w.uint32(26).string(m.customizer_attribute);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    $root.google.ads.googleads.v11.common.CustomizerValue.encode(m.value, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionCustomizer message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer} message AdGroupCriterionCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionCustomizer.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionCustomizer message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionCustomizer} AdGroupCriterionCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionCustomizer.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_criterion = r.string();
                                        break;
                                    case 3:
                                        m.customizer_attribute = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.value = $root.google.ads.googleads.v11.common.CustomizerValue.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionCustomizer message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionCustomizer} AdGroupCriterionCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionCustomizer.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionCustomizer message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionCustomizer.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    p._ad_group_criterion = 1;
                                    if (!$util.isString(m.ad_group_criterion))
                                        return "ad_group_criterion: string expected";
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    if (!$util.isString(m.customizer_attribute))
                                        return "customizer_attribute: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomizerValue.verify(m.value);
                                        if (e)
                                            return "value." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionCustomizer message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionCustomizer} AdGroupCriterionCustomizer
                             */
                            AdGroupCriterionCustomizer.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_criterion != null) {
                                    m.ad_group_criterion = String(d.ad_group_criterion);
                                }
                                if (d.customizer_attribute != null) {
                                    m.customizer_attribute = String(d.customizer_attribute);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.value != null) {
                                    if (typeof d.value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.value: object expected");
                                    m.value = $root.google.ads.googleads.v11.common.CustomizerValue.fromObject(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionCustomizer message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupCriterionCustomizer} m AdGroupCriterionCustomizer
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionCustomizer.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customizer_attribute = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.value = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    d.ad_group_criterion = m.ad_group_criterion;
                                    if (o.oneofs)
                                        d._ad_group_criterion = "ad_group_criterion";
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    d.customizer_attribute = m.customizer_attribute;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus[m.status] : m.status;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = $root.google.ads.googleads.v11.common.CustomizerValue.toObject(m.value, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCriterionCustomizer to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionCustomizer
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionCustomizer.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCriterionCustomizer;
                        })();
    
                        resources.AdGroupCriterionLabel = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupCriterionLabel
                             * @property {string|null} [resource_name] AdGroupCriterionLabel resource_name
                             * @property {string|null} [ad_group_criterion] AdGroupCriterionLabel ad_group_criterion
                             * @property {string|null} [label] AdGroupCriterionLabel label
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupCriterionLabel.
                             * @implements IAdGroupCriterionLabel
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionLabel=} [p] Properties to set
                             */
                            function AdGroupCriterionLabel(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCriterionLabel resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @instance
                             */
                            AdGroupCriterionLabel.prototype.resource_name = "";
    
                            /**
                             * AdGroupCriterionLabel ad_group_criterion.
                             * @member {string|null|undefined} ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @instance
                             */
                            AdGroupCriterionLabel.prototype.ad_group_criterion = null;
    
                            /**
                             * AdGroupCriterionLabel label.
                             * @member {string|null|undefined} label
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @instance
                             */
                            AdGroupCriterionLabel.prototype.label = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCriterionLabel _ad_group_criterion.
                             * @member {"ad_group_criterion"|undefined} _ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionLabel.prototype, "_ad_group_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterionLabel _label.
                             * @member {"label"|undefined} _label
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionLabel.prototype, "_label", {
                                get: $util.oneOfGetter($oneOfFields = ["label"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCriterionLabel instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionLabel=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionLabel} AdGroupCriterionLabel instance
                             */
                            AdGroupCriterionLabel.create = function create(properties) {
                                return new AdGroupCriterionLabel(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionLabel message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterionLabel.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionLabel} m AdGroupCriterionLabel message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionLabel.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                    w.uint32(34).string(m.ad_group_criterion);
                                if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                                    w.uint32(42).string(m.label);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionLabel message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterionLabel.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionLabel} message AdGroupCriterionLabel message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionLabel.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionLabel message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionLabel} AdGroupCriterionLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionLabel.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.ad_group_criterion = r.string();
                                        break;
                                    case 5:
                                        m.label = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionLabel message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionLabel} AdGroupCriterionLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionLabel.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionLabel message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionLabel.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    p._ad_group_criterion = 1;
                                    if (!$util.isString(m.ad_group_criterion))
                                        return "ad_group_criterion: string expected";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    p._label = 1;
                                    if (!$util.isString(m.label))
                                        return "label: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionLabel message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionLabel} AdGroupCriterionLabel
                             */
                            AdGroupCriterionLabel.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_criterion != null) {
                                    m.ad_group_criterion = String(d.ad_group_criterion);
                                }
                                if (d.label != null) {
                                    m.label = String(d.label);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionLabel message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupCriterionLabel} m AdGroupCriterionLabel
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionLabel.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    d.ad_group_criterion = m.ad_group_criterion;
                                    if (o.oneofs)
                                        d._ad_group_criterion = "ad_group_criterion";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    d.label = m.label;
                                    if (o.oneofs)
                                        d._label = "label";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCriterionLabel to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionLabel
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionLabel.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCriterionLabel;
                        })();
    
                        resources.AdGroupCriterionSimulation = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupCriterionSimulation
                             * @property {string|null} [resource_name] AdGroupCriterionSimulation resource_name
                             * @property {number|null} [ad_group_id] AdGroupCriterionSimulation ad_group_id
                             * @property {number|null} [criterion_id] AdGroupCriterionSimulation criterion_id
                             * @property {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType|null} [type] AdGroupCriterionSimulation type
                             * @property {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod|null} [modification_method] AdGroupCriterionSimulation modification_method
                             * @property {string|null} [start_date] AdGroupCriterionSimulation start_date
                             * @property {string|null} [end_date] AdGroupCriterionSimulation end_date
                             * @property {google.ads.googleads.v11.common.ICpcBidSimulationPointList|null} [cpc_bid_point_list] AdGroupCriterionSimulation cpc_bid_point_list
                             * @property {google.ads.googleads.v11.common.IPercentCpcBidSimulationPointList|null} [percent_cpc_bid_point_list] AdGroupCriterionSimulation percent_cpc_bid_point_list
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupCriterionSimulation.
                             * @implements IAdGroupCriterionSimulation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionSimulation=} [p] Properties to set
                             */
                            function AdGroupCriterionSimulation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCriterionSimulation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.resource_name = "";
    
                            /**
                             * AdGroupCriterionSimulation ad_group_id.
                             * @member {number|null|undefined} ad_group_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.ad_group_id = null;
    
                            /**
                             * AdGroupCriterionSimulation criterion_id.
                             * @member {number|null|undefined} criterion_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.criterion_id = null;
    
                            /**
                             * AdGroupCriterionSimulation type.
                             * @member {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType} type
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.type = 0;
    
                            /**
                             * AdGroupCriterionSimulation modification_method.
                             * @member {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod} modification_method
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.modification_method = 0;
    
                            /**
                             * AdGroupCriterionSimulation start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.start_date = null;
    
                            /**
                             * AdGroupCriterionSimulation end_date.
                             * @member {string|null|undefined} end_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.end_date = null;
    
                            /**
                             * AdGroupCriterionSimulation cpc_bid_point_list.
                             * @member {google.ads.googleads.v11.common.ICpcBidSimulationPointList|null|undefined} cpc_bid_point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.cpc_bid_point_list = null;
    
                            /**
                             * AdGroupCriterionSimulation percent_cpc_bid_point_list.
                             * @member {google.ads.googleads.v11.common.IPercentCpcBidSimulationPointList|null|undefined} percent_cpc_bid_point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            AdGroupCriterionSimulation.prototype.percent_cpc_bid_point_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCriterionSimulation _ad_group_id.
                             * @member {"ad_group_id"|undefined} _ad_group_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionSimulation.prototype, "_ad_group_id", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterionSimulation _criterion_id.
                             * @member {"criterion_id"|undefined} _criterion_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionSimulation.prototype, "_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterionSimulation _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionSimulation.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterionSimulation _end_date.
                             * @member {"end_date"|undefined} _end_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionSimulation.prototype, "_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupCriterionSimulation point_list.
                             * @member {"cpc_bid_point_list"|"percent_cpc_bid_point_list"|undefined} point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionSimulation.prototype, "point_list", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_point_list", "percent_cpc_bid_point_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCriterionSimulation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionSimulation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionSimulation} AdGroupCriterionSimulation instance
                             */
                            AdGroupCriterionSimulation.create = function create(properties) {
                                return new AdGroupCriterionSimulation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionSimulation message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterionSimulation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionSimulation} m AdGroupCriterionSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionSimulation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.modification_method != null && Object.hasOwnProperty.call(m, "modification_method"))
                                    w.uint32(40).int32(m.modification_method);
                                if (m.cpc_bid_point_list != null && Object.hasOwnProperty.call(m, "cpc_bid_point_list"))
                                    $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.encode(m.cpc_bid_point_list, w.uint32(66).fork()).ldelim();
                                if (m.ad_group_id != null && Object.hasOwnProperty.call(m, "ad_group_id"))
                                    w.uint32(72).int64(m.ad_group_id);
                                if (m.criterion_id != null && Object.hasOwnProperty.call(m, "criterion_id"))
                                    w.uint32(80).int64(m.criterion_id);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(90).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(98).string(m.end_date);
                                if (m.percent_cpc_bid_point_list != null && Object.hasOwnProperty.call(m, "percent_cpc_bid_point_list"))
                                    $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.encode(m.percent_cpc_bid_point_list, w.uint32(106).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionSimulation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCriterionSimulation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCriterionSimulation} message AdGroupCriterionSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionSimulation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionSimulation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionSimulation} AdGroupCriterionSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionSimulation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.ad_group_id = r.int64();
                                        break;
                                    case 10:
                                        m.criterion_id = r.int64();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 5:
                                        m.modification_method = r.int32();
                                        break;
                                    case 11:
                                        m.start_date = r.string();
                                        break;
                                    case 12:
                                        m.end_date = r.string();
                                        break;
                                    case 8:
                                        m.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.percent_cpc_bid_point_list = $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionSimulation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionSimulation} AdGroupCriterionSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionSimulation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionSimulation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionSimulation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_id != null && m.hasOwnProperty("ad_group_id")) {
                                    p._ad_group_id = 1;
                                    if (!$util.isInteger(m.ad_group_id) && !(m.ad_group_id && $util.isInteger(m.ad_group_id.low) && $util.isInteger(m.ad_group_id.high)))
                                        return "ad_group_id: integer|Long expected";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    p._criterion_id = 1;
                                    if (!$util.isInteger(m.criterion_id) && !(m.criterion_id && $util.isInteger(m.criterion_id.low) && $util.isInteger(m.criterion_id.high)))
                                        return "criterion_id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    switch (m.modification_method) {
                                    default:
                                        return "modification_method: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    p._end_date = 1;
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.cpc_bid_point_list != null && m.hasOwnProperty("cpc_bid_point_list")) {
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.verify(m.cpc_bid_point_list);
                                        if (e)
                                            return "cpc_bid_point_list." + e;
                                    }
                                }
                                if (m.percent_cpc_bid_point_list != null && m.hasOwnProperty("percent_cpc_bid_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.verify(m.percent_cpc_bid_point_list);
                                        if (e)
                                            return "percent_cpc_bid_point_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionSimulation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupCriterionSimulation} AdGroupCriterionSimulation
                             */
                            AdGroupCriterionSimulation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_id != null) {
                                    if ($util.Long)
                                        (m.ad_group_id = $util.Long.fromValue(d.ad_group_id)).unsigned = false;
                                    else if (typeof d.ad_group_id === "string")
                                        m.ad_group_id = parseInt(d.ad_group_id, 10);
                                    else if (typeof d.ad_group_id === "number")
                                        m.ad_group_id = d.ad_group_id;
                                    else if (typeof d.ad_group_id === "object")
                                        m.ad_group_id = new $util.LongBits(d.ad_group_id.low >>> 0, d.ad_group_id.high >>> 0).toNumber();
                                }
                                if (d.criterion_id != null) {
                                    if ($util.Long)
                                        (m.criterion_id = $util.Long.fromValue(d.criterion_id)).unsigned = false;
                                    else if (typeof d.criterion_id === "string")
                                        m.criterion_id = parseInt(d.criterion_id, 10);
                                    else if (typeof d.criterion_id === "number")
                                        m.criterion_id = d.criterion_id;
                                    else if (typeof d.criterion_id === "object")
                                        m.criterion_id = new $util.LongBits(d.criterion_id.low >>> 0, d.criterion_id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CPC_BID":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "CPV_BID":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "TARGET_CPA":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "BID_MODIFIER":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "TARGET_ROAS":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PERCENT_CPC_BID":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "BUDGET":
                                case 9:
                                    m.type = 9;
                                    break;
                                }
                                switch (d.modification_method) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.modification_method = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.modification_method = 1;
                                    break;
                                case "UNIFORM":
                                case 2:
                                    m.modification_method = 2;
                                    break;
                                case "DEFAULT":
                                case 3:
                                    m.modification_method = 3;
                                    break;
                                case "SCALING":
                                case 4:
                                    m.modification_method = 4;
                                    break;
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.cpc_bid_point_list != null) {
                                    if (typeof d.cpc_bid_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterionSimulation.cpc_bid_point_list: object expected");
                                    m.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.fromObject(d.cpc_bid_point_list);
                                }
                                if (d.percent_cpc_bid_point_list != null) {
                                    if (typeof d.percent_cpc_bid_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCriterionSimulation.percent_cpc_bid_point_list: object expected");
                                    m.percent_cpc_bid_point_list = $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.fromObject(d.percent_cpc_bid_point_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionSimulation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupCriterionSimulation} m AdGroupCriterionSimulation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionSimulation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.modification_method = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType[m.type] : m.type;
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    d.modification_method = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod[m.modification_method] : m.modification_method;
                                }
                                if (m.cpc_bid_point_list != null && m.hasOwnProperty("cpc_bid_point_list")) {
                                    d.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.toObject(m.cpc_bid_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "cpc_bid_point_list";
                                }
                                if (m.ad_group_id != null && m.hasOwnProperty("ad_group_id")) {
                                    if (typeof m.ad_group_id === "number")
                                        d.ad_group_id = o.longs === String ? String(m.ad_group_id) : m.ad_group_id;
                                    else
                                        d.ad_group_id = o.longs === String ? $util.Long.prototype.toString.call(m.ad_group_id) : o.longs === Number ? new $util.LongBits(m.ad_group_id.low >>> 0, m.ad_group_id.high >>> 0).toNumber() : m.ad_group_id;
                                    if (o.oneofs)
                                        d._ad_group_id = "ad_group_id";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    if (typeof m.criterion_id === "number")
                                        d.criterion_id = o.longs === String ? String(m.criterion_id) : m.criterion_id;
                                    else
                                        d.criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.criterion_id) : o.longs === Number ? new $util.LongBits(m.criterion_id.low >>> 0, m.criterion_id.high >>> 0).toNumber() : m.criterion_id;
                                    if (o.oneofs)
                                        d._criterion_id = "criterion_id";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                    if (o.oneofs)
                                        d._end_date = "end_date";
                                }
                                if (m.percent_cpc_bid_point_list != null && m.hasOwnProperty("percent_cpc_bid_point_list")) {
                                    d.percent_cpc_bid_point_list = $root.google.ads.googleads.v11.common.PercentCpcBidSimulationPointList.toObject(m.percent_cpc_bid_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "percent_cpc_bid_point_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCriterionSimulation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupCriterionSimulation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionSimulation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCriterionSimulation;
                        })();
    
                        resources.AdGroupCustomizer = (function() {
    
                            /**
                             * Properties of an AdGroupCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupCustomizer
                             * @property {string|null} [resource_name] AdGroupCustomizer resource_name
                             * @property {string|null} [ad_group] AdGroupCustomizer ad_group
                             * @property {string|null} [customizer_attribute] AdGroupCustomizer customizer_attribute
                             * @property {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus|null} [status] AdGroupCustomizer status
                             * @property {google.ads.googleads.v11.common.ICustomizerValue|null} [value] AdGroupCustomizer value
                             */
    
                            /**
                             * Constructs a new AdGroupCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupCustomizer.
                             * @implements IAdGroupCustomizer
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupCustomizer=} [p] Properties to set
                             */
                            function AdGroupCustomizer(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCustomizer resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @instance
                             */
                            AdGroupCustomizer.prototype.resource_name = "";
    
                            /**
                             * AdGroupCustomizer ad_group.
                             * @member {string} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @instance
                             */
                            AdGroupCustomizer.prototype.ad_group = "";
    
                            /**
                             * AdGroupCustomizer customizer_attribute.
                             * @member {string} customizer_attribute
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @instance
                             */
                            AdGroupCustomizer.prototype.customizer_attribute = "";
    
                            /**
                             * AdGroupCustomizer status.
                             * @member {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus} status
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @instance
                             */
                            AdGroupCustomizer.prototype.status = 0;
    
                            /**
                             * AdGroupCustomizer value.
                             * @member {google.ads.googleads.v11.common.ICustomizerValue|null|undefined} value
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @instance
                             */
                            AdGroupCustomizer.prototype.value = null;
    
                            /**
                             * Creates a new AdGroupCustomizer instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCustomizer=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupCustomizer} AdGroupCustomizer instance
                             */
                            AdGroupCustomizer.create = function create(properties) {
                                return new AdGroupCustomizer(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCustomizer message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCustomizer.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCustomizer} m AdGroupCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCustomizer.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(18).string(m.ad_group);
                                if (m.customizer_attribute != null && Object.hasOwnProperty.call(m, "customizer_attribute"))
                                    w.uint32(26).string(m.customizer_attribute);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    $root.google.ads.googleads.v11.common.CustomizerValue.encode(m.value, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCustomizer message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupCustomizer.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupCustomizer} message AdGroupCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCustomizer.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCustomizer message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupCustomizer} AdGroupCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCustomizer.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupCustomizer();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group = r.string();
                                        break;
                                    case 3:
                                        m.customizer_attribute = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.value = $root.google.ads.googleads.v11.common.CustomizerValue.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCustomizer message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupCustomizer} AdGroupCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCustomizer.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCustomizer message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCustomizer.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    if (!$util.isString(m.customizer_attribute))
                                        return "customizer_attribute: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomizerValue.verify(m.value);
                                        if (e)
                                            return "value." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCustomizer message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupCustomizer} AdGroupCustomizer
                             */
                            AdGroupCustomizer.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupCustomizer)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupCustomizer();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.customizer_attribute != null) {
                                    m.customizer_attribute = String(d.customizer_attribute);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.value != null) {
                                    if (typeof d.value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupCustomizer.value: object expected");
                                    m.value = $root.google.ads.googleads.v11.common.CustomizerValue.fromObject(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCustomizer message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupCustomizer} m AdGroupCustomizer
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCustomizer.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group = "";
                                    d.customizer_attribute = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.value = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    d.customizer_attribute = m.customizer_attribute;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus[m.status] : m.status;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = $root.google.ads.googleads.v11.common.CustomizerValue.toObject(m.value, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCustomizer to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupCustomizer
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCustomizer.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCustomizer;
                        })();
    
                        resources.AdGroupExtensionSetting = (function() {
    
                            /**
                             * Properties of an AdGroupExtensionSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupExtensionSetting
                             * @property {string|null} [resource_name] AdGroupExtensionSetting resource_name
                             * @property {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType|null} [extension_type] AdGroupExtensionSetting extension_type
                             * @property {string|null} [ad_group] AdGroupExtensionSetting ad_group
                             * @property {Array.<string>|null} [extension_feed_items] AdGroupExtensionSetting extension_feed_items
                             * @property {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice|null} [device] AdGroupExtensionSetting device
                             */
    
                            /**
                             * Constructs a new AdGroupExtensionSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupExtensionSetting.
                             * @implements IAdGroupExtensionSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupExtensionSetting=} [p] Properties to set
                             */
                            function AdGroupExtensionSetting(p) {
                                this.extension_feed_items = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupExtensionSetting resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @instance
                             */
                            AdGroupExtensionSetting.prototype.resource_name = "";
    
                            /**
                             * AdGroupExtensionSetting extension_type.
                             * @member {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType} extension_type
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @instance
                             */
                            AdGroupExtensionSetting.prototype.extension_type = 0;
    
                            /**
                             * AdGroupExtensionSetting ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @instance
                             */
                            AdGroupExtensionSetting.prototype.ad_group = null;
    
                            /**
                             * AdGroupExtensionSetting extension_feed_items.
                             * @member {Array.<string>} extension_feed_items
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @instance
                             */
                            AdGroupExtensionSetting.prototype.extension_feed_items = $util.emptyArray;
    
                            /**
                             * AdGroupExtensionSetting device.
                             * @member {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice} device
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @instance
                             */
                            AdGroupExtensionSetting.prototype.device = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupExtensionSetting _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @instance
                             */
                            Object.defineProperty(AdGroupExtensionSetting.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupExtensionSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupExtensionSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupExtensionSetting} AdGroupExtensionSetting instance
                             */
                            AdGroupExtensionSetting.create = function create(properties) {
                                return new AdGroupExtensionSetting(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupExtensionSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupExtensionSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupExtensionSetting} m AdGroupExtensionSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupExtensionSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.extension_type != null && Object.hasOwnProperty.call(m, "extension_type"))
                                    w.uint32(16).int32(m.extension_type);
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    w.uint32(40).int32(m.device);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(50).string(m.ad_group);
                                if (m.extension_feed_items != null && m.extension_feed_items.length) {
                                    for (var i = 0; i < m.extension_feed_items.length; ++i)
                                        w.uint32(58).string(m.extension_feed_items[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupExtensionSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupExtensionSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupExtensionSetting} message AdGroupExtensionSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupExtensionSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupExtensionSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupExtensionSetting} AdGroupExtensionSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupExtensionSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.extension_type = r.int32();
                                        break;
                                    case 6:
                                        m.ad_group = r.string();
                                        break;
                                    case 7:
                                        if (!(m.extension_feed_items && m.extension_feed_items.length))
                                            m.extension_feed_items = [];
                                        m.extension_feed_items.push(r.string());
                                        break;
                                    case 5:
                                        m.device = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupExtensionSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupExtensionSetting} AdGroupExtensionSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupExtensionSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupExtensionSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupExtensionSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    switch (m.extension_type) {
                                    default:
                                        return "extension_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 15:
                                    case 16:
                                        break;
                                    }
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.extension_feed_items != null && m.hasOwnProperty("extension_feed_items")) {
                                    if (!Array.isArray(m.extension_feed_items))
                                        return "extension_feed_items: array expected";
                                    for (var i = 0; i < m.extension_feed_items.length; ++i) {
                                        if (!$util.isString(m.extension_feed_items[i]))
                                            return "extension_feed_items: string[] expected";
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    switch (m.device) {
                                    default:
                                        return "device: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupExtensionSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupExtensionSetting} AdGroupExtensionSetting
                             */
                            AdGroupExtensionSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.extension_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.extension_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.extension_type = 1;
                                    break;
                                case "NONE":
                                case 2:
                                    m.extension_type = 2;
                                    break;
                                case "APP":
                                case 3:
                                    m.extension_type = 3;
                                    break;
                                case "CALL":
                                case 4:
                                    m.extension_type = 4;
                                    break;
                                case "CALLOUT":
                                case 5:
                                    m.extension_type = 5;
                                    break;
                                case "MESSAGE":
                                case 6:
                                    m.extension_type = 6;
                                    break;
                                case "PRICE":
                                case 7:
                                    m.extension_type = 7;
                                    break;
                                case "PROMOTION":
                                case 8:
                                    m.extension_type = 8;
                                    break;
                                case "SITELINK":
                                case 10:
                                    m.extension_type = 10;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 11:
                                    m.extension_type = 11;
                                    break;
                                case "LOCATION":
                                case 12:
                                    m.extension_type = 12;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 13:
                                    m.extension_type = 13;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.extension_type = 15;
                                    break;
                                case "IMAGE":
                                case 16:
                                    m.extension_type = 16;
                                    break;
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.extension_feed_items) {
                                    if (!Array.isArray(d.extension_feed_items))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupExtensionSetting.extension_feed_items: array expected");
                                    m.extension_feed_items = [];
                                    for (var i = 0; i < d.extension_feed_items.length; ++i) {
                                        m.extension_feed_items[i] = String(d.extension_feed_items[i]);
                                    }
                                }
                                switch (d.device) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device = 2;
                                    break;
                                case "DESKTOP":
                                case 3:
                                    m.device = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupExtensionSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupExtensionSetting} m AdGroupExtensionSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupExtensionSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.extension_feed_items = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.extension_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.device = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    d.extension_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType[m.extension_type] : m.extension_type;
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice[m.device] : m.device;
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                if (m.extension_feed_items && m.extension_feed_items.length) {
                                    d.extension_feed_items = [];
                                    for (var j = 0; j < m.extension_feed_items.length; ++j) {
                                        d.extension_feed_items[j] = m.extension_feed_items[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupExtensionSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupExtensionSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupExtensionSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupExtensionSetting;
                        })();
    
                        resources.AdGroupFeed = (function() {
    
                            /**
                             * Properties of an AdGroupFeed.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupFeed
                             * @property {string|null} [resource_name] AdGroupFeed resource_name
                             * @property {string|null} [feed] AdGroupFeed feed
                             * @property {string|null} [ad_group] AdGroupFeed ad_group
                             * @property {Array.<google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType>|null} [placeholder_types] AdGroupFeed placeholder_types
                             * @property {google.ads.googleads.v11.common.IMatchingFunction|null} [matching_function] AdGroupFeed matching_function
                             * @property {google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus|null} [status] AdGroupFeed status
                             */
    
                            /**
                             * Constructs a new AdGroupFeed.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupFeed.
                             * @implements IAdGroupFeed
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupFeed=} [p] Properties to set
                             */
                            function AdGroupFeed(p) {
                                this.placeholder_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupFeed resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            AdGroupFeed.prototype.resource_name = "";
    
                            /**
                             * AdGroupFeed feed.
                             * @member {string|null|undefined} feed
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            AdGroupFeed.prototype.feed = null;
    
                            /**
                             * AdGroupFeed ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            AdGroupFeed.prototype.ad_group = null;
    
                            /**
                             * AdGroupFeed placeholder_types.
                             * @member {Array.<google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType>} placeholder_types
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            AdGroupFeed.prototype.placeholder_types = $util.emptyArray;
    
                            /**
                             * AdGroupFeed matching_function.
                             * @member {google.ads.googleads.v11.common.IMatchingFunction|null|undefined} matching_function
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            AdGroupFeed.prototype.matching_function = null;
    
                            /**
                             * AdGroupFeed status.
                             * @member {google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            AdGroupFeed.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupFeed _feed.
                             * @member {"feed"|undefined} _feed
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            Object.defineProperty(AdGroupFeed.prototype, "_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupFeed _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             */
                            Object.defineProperty(AdGroupFeed.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupFeed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupFeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupFeed} AdGroupFeed instance
                             */
                            AdGroupFeed.create = function create(properties) {
                                return new AdGroupFeed(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupFeed message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupFeed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupFeed} m AdGroupFeed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupFeed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.placeholder_types != null && m.placeholder_types.length) {
                                    w.uint32(34).fork();
                                    for (var i = 0; i < m.placeholder_types.length; ++i)
                                        w.int32(m.placeholder_types[i]);
                                    w.ldelim();
                                }
                                if (m.matching_function != null && Object.hasOwnProperty.call(m, "matching_function"))
                                    $root.google.ads.googleads.v11.common.MatchingFunction.encode(m.matching_function, w.uint32(42).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(48).int32(m.status);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(58).string(m.feed);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(66).string(m.ad_group);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupFeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupFeed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupFeed} message AdGroupFeed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupFeed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupFeed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupFeed} AdGroupFeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupFeed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupFeed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.feed = r.string();
                                        break;
                                    case 8:
                                        m.ad_group = r.string();
                                        break;
                                    case 4:
                                        if (!(m.placeholder_types && m.placeholder_types.length))
                                            m.placeholder_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.placeholder_types.push(r.int32());
                                        } else
                                            m.placeholder_types.push(r.int32());
                                        break;
                                    case 5:
                                        m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupFeed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupFeed} AdGroupFeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupFeed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupFeed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupFeed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    p._feed = 1;
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.placeholder_types != null && m.hasOwnProperty("placeholder_types")) {
                                    if (!Array.isArray(m.placeholder_types))
                                        return "placeholder_types: array expected";
                                    for (var i = 0; i < m.placeholder_types.length; ++i) {
                                        switch (m.placeholder_types[i]) {
                                        default:
                                            return "placeholder_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 17:
                                        case 18:
                                        case 19:
                                        case 20:
                                        case 21:
                                            break;
                                        }
                                    }
                                }
                                if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MatchingFunction.verify(m.matching_function);
                                        if (e)
                                            return "matching_function." + e;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupFeed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupFeed} AdGroupFeed
                             */
                            AdGroupFeed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupFeed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupFeed();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.placeholder_types) {
                                    if (!Array.isArray(d.placeholder_types))
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupFeed.placeholder_types: array expected");
                                    m.placeholder_types = [];
                                    for (var i = 0; i < d.placeholder_types.length; ++i) {
                                        switch (d.placeholder_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.placeholder_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.placeholder_types[i] = 1;
                                            break;
                                        case "SITELINK":
                                        case 2:
                                            m.placeholder_types[i] = 2;
                                            break;
                                        case "CALL":
                                        case 3:
                                            m.placeholder_types[i] = 3;
                                            break;
                                        case "APP":
                                        case 4:
                                            m.placeholder_types[i] = 4;
                                            break;
                                        case "LOCATION":
                                        case 5:
                                            m.placeholder_types[i] = 5;
                                            break;
                                        case "AFFILIATE_LOCATION":
                                        case 6:
                                            m.placeholder_types[i] = 6;
                                            break;
                                        case "CALLOUT":
                                        case 7:
                                            m.placeholder_types[i] = 7;
                                            break;
                                        case "STRUCTURED_SNIPPET":
                                        case 8:
                                            m.placeholder_types[i] = 8;
                                            break;
                                        case "MESSAGE":
                                        case 9:
                                            m.placeholder_types[i] = 9;
                                            break;
                                        case "PRICE":
                                        case 10:
                                            m.placeholder_types[i] = 10;
                                            break;
                                        case "PROMOTION":
                                        case 11:
                                            m.placeholder_types[i] = 11;
                                            break;
                                        case "AD_CUSTOMIZER":
                                        case 12:
                                            m.placeholder_types[i] = 12;
                                            break;
                                        case "DYNAMIC_EDUCATION":
                                        case 13:
                                            m.placeholder_types[i] = 13;
                                            break;
                                        case "DYNAMIC_FLIGHT":
                                        case 14:
                                            m.placeholder_types[i] = 14;
                                            break;
                                        case "DYNAMIC_CUSTOM":
                                        case 15:
                                            m.placeholder_types[i] = 15;
                                            break;
                                        case "DYNAMIC_HOTEL":
                                        case 16:
                                            m.placeholder_types[i] = 16;
                                            break;
                                        case "DYNAMIC_REAL_ESTATE":
                                        case 17:
                                            m.placeholder_types[i] = 17;
                                            break;
                                        case "DYNAMIC_TRAVEL":
                                        case 18:
                                            m.placeholder_types[i] = 18;
                                            break;
                                        case "DYNAMIC_LOCAL":
                                        case 19:
                                            m.placeholder_types[i] = 19;
                                            break;
                                        case "DYNAMIC_JOB":
                                        case 20:
                                            m.placeholder_types[i] = 20;
                                            break;
                                        case "IMAGE":
                                        case 21:
                                            m.placeholder_types[i] = 21;
                                            break;
                                        }
                                    }
                                }
                                if (d.matching_function != null) {
                                    if (typeof d.matching_function !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupFeed.matching_function: object expected");
                                    m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.fromObject(d.matching_function);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupFeed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupFeed} m AdGroupFeed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupFeed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.placeholder_types = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.matching_function = null;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.placeholder_types && m.placeholder_types.length) {
                                    d.placeholder_types = [];
                                    for (var j = 0; j < m.placeholder_types.length; ++j) {
                                        d.placeholder_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType[m.placeholder_types[j]] : m.placeholder_types[j];
                                    }
                                }
                                if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                    d.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.toObject(m.matching_function, o);
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus[m.status] : m.status;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                    if (o.oneofs)
                                        d._feed = "feed";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupFeed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupFeed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupFeed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupFeed;
                        })();
    
                        resources.AdGroupLabel = (function() {
    
                            /**
                             * Properties of an AdGroupLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupLabel
                             * @property {string|null} [resource_name] AdGroupLabel resource_name
                             * @property {string|null} [ad_group] AdGroupLabel ad_group
                             * @property {string|null} [label] AdGroupLabel label
                             */
    
                            /**
                             * Constructs a new AdGroupLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupLabel.
                             * @implements IAdGroupLabel
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupLabel=} [p] Properties to set
                             */
                            function AdGroupLabel(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupLabel resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @instance
                             */
                            AdGroupLabel.prototype.resource_name = "";
    
                            /**
                             * AdGroupLabel ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @instance
                             */
                            AdGroupLabel.prototype.ad_group = null;
    
                            /**
                             * AdGroupLabel label.
                             * @member {string|null|undefined} label
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @instance
                             */
                            AdGroupLabel.prototype.label = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupLabel _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @instance
                             */
                            Object.defineProperty(AdGroupLabel.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupLabel _label.
                             * @member {"label"|undefined} _label
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @instance
                             */
                            Object.defineProperty(AdGroupLabel.prototype, "_label", {
                                get: $util.oneOfGetter($oneOfFields = ["label"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupLabel instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupLabel=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupLabel} AdGroupLabel instance
                             */
                            AdGroupLabel.create = function create(properties) {
                                return new AdGroupLabel(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupLabel message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupLabel.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupLabel} m AdGroupLabel message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupLabel.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(34).string(m.ad_group);
                                if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                                    w.uint32(42).string(m.label);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupLabel message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupLabel.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupLabel} message AdGroupLabel message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupLabel.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupLabel message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupLabel} AdGroupLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupLabel.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupLabel();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.ad_group = r.string();
                                        break;
                                    case 5:
                                        m.label = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupLabel message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupLabel} AdGroupLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupLabel.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupLabel message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupLabel.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    p._label = 1;
                                    if (!$util.isString(m.label))
                                        return "label: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupLabel message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupLabel} AdGroupLabel
                             */
                            AdGroupLabel.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupLabel)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupLabel();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.label != null) {
                                    m.label = String(d.label);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupLabel message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupLabel} m AdGroupLabel
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupLabel.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    d.label = m.label;
                                    if (o.oneofs)
                                        d._label = "label";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupLabel to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupLabel
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupLabel.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupLabel;
                        })();
    
                        resources.AdGroupSimulation = (function() {
    
                            /**
                             * Properties of an AdGroupSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdGroupSimulation
                             * @property {string|null} [resource_name] AdGroupSimulation resource_name
                             * @property {number|null} [ad_group_id] AdGroupSimulation ad_group_id
                             * @property {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType|null} [type] AdGroupSimulation type
                             * @property {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod|null} [modification_method] AdGroupSimulation modification_method
                             * @property {string|null} [start_date] AdGroupSimulation start_date
                             * @property {string|null} [end_date] AdGroupSimulation end_date
                             * @property {google.ads.googleads.v11.common.ICpcBidSimulationPointList|null} [cpc_bid_point_list] AdGroupSimulation cpc_bid_point_list
                             * @property {google.ads.googleads.v11.common.ICpvBidSimulationPointList|null} [cpv_bid_point_list] AdGroupSimulation cpv_bid_point_list
                             * @property {google.ads.googleads.v11.common.ITargetCpaSimulationPointList|null} [target_cpa_point_list] AdGroupSimulation target_cpa_point_list
                             * @property {google.ads.googleads.v11.common.ITargetRoasSimulationPointList|null} [target_roas_point_list] AdGroupSimulation target_roas_point_list
                             */
    
                            /**
                             * Constructs a new AdGroupSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdGroupSimulation.
                             * @implements IAdGroupSimulation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdGroupSimulation=} [p] Properties to set
                             */
                            function AdGroupSimulation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupSimulation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.resource_name = "";
    
                            /**
                             * AdGroupSimulation ad_group_id.
                             * @member {number|null|undefined} ad_group_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.ad_group_id = null;
    
                            /**
                             * AdGroupSimulation type.
                             * @member {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType} type
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.type = 0;
    
                            /**
                             * AdGroupSimulation modification_method.
                             * @member {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod} modification_method
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.modification_method = 0;
    
                            /**
                             * AdGroupSimulation start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.start_date = null;
    
                            /**
                             * AdGroupSimulation end_date.
                             * @member {string|null|undefined} end_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.end_date = null;
    
                            /**
                             * AdGroupSimulation cpc_bid_point_list.
                             * @member {google.ads.googleads.v11.common.ICpcBidSimulationPointList|null|undefined} cpc_bid_point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.cpc_bid_point_list = null;
    
                            /**
                             * AdGroupSimulation cpv_bid_point_list.
                             * @member {google.ads.googleads.v11.common.ICpvBidSimulationPointList|null|undefined} cpv_bid_point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.cpv_bid_point_list = null;
    
                            /**
                             * AdGroupSimulation target_cpa_point_list.
                             * @member {google.ads.googleads.v11.common.ITargetCpaSimulationPointList|null|undefined} target_cpa_point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.target_cpa_point_list = null;
    
                            /**
                             * AdGroupSimulation target_roas_point_list.
                             * @member {google.ads.googleads.v11.common.ITargetRoasSimulationPointList|null|undefined} target_roas_point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            AdGroupSimulation.prototype.target_roas_point_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupSimulation _ad_group_id.
                             * @member {"ad_group_id"|undefined} _ad_group_id
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupSimulation.prototype, "_ad_group_id", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupSimulation _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupSimulation.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupSimulation _end_date.
                             * @member {"end_date"|undefined} _end_date
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupSimulation.prototype, "_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdGroupSimulation point_list.
                             * @member {"cpc_bid_point_list"|"cpv_bid_point_list"|"target_cpa_point_list"|"target_roas_point_list"|undefined} point_list
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             */
                            Object.defineProperty(AdGroupSimulation.prototype, "point_list", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_point_list", "cpv_bid_point_list", "target_cpa_point_list", "target_roas_point_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupSimulation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupSimulation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdGroupSimulation} AdGroupSimulation instance
                             */
                            AdGroupSimulation.create = function create(properties) {
                                return new AdGroupSimulation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupSimulation message. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupSimulation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupSimulation} m AdGroupSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupSimulation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.modification_method != null && Object.hasOwnProperty.call(m, "modification_method"))
                                    w.uint32(32).int32(m.modification_method);
                                if (m.cpc_bid_point_list != null && Object.hasOwnProperty.call(m, "cpc_bid_point_list"))
                                    $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.encode(m.cpc_bid_point_list, w.uint32(66).fork()).ldelim();
                                if (m.target_cpa_point_list != null && Object.hasOwnProperty.call(m, "target_cpa_point_list"))
                                    $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.encode(m.target_cpa_point_list, w.uint32(74).fork()).ldelim();
                                if (m.cpv_bid_point_list != null && Object.hasOwnProperty.call(m, "cpv_bid_point_list"))
                                    $root.google.ads.googleads.v11.common.CpvBidSimulationPointList.encode(m.cpv_bid_point_list, w.uint32(82).fork()).ldelim();
                                if (m.target_roas_point_list != null && Object.hasOwnProperty.call(m, "target_roas_point_list"))
                                    $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.encode(m.target_roas_point_list, w.uint32(90).fork()).ldelim();
                                if (m.ad_group_id != null && Object.hasOwnProperty.call(m, "ad_group_id"))
                                    w.uint32(96).int64(m.ad_group_id);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(106).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(114).string(m.end_date);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupSimulation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdGroupSimulation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdGroupSimulation} message AdGroupSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupSimulation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupSimulation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdGroupSimulation} AdGroupSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupSimulation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdGroupSimulation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 12:
                                        m.ad_group_id = r.int64();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 4:
                                        m.modification_method = r.int32();
                                        break;
                                    case 13:
                                        m.start_date = r.string();
                                        break;
                                    case 14:
                                        m.end_date = r.string();
                                        break;
                                    case 8:
                                        m.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.cpv_bid_point_list = $root.google.ads.googleads.v11.common.CpvBidSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupSimulation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdGroupSimulation} AdGroupSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupSimulation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupSimulation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupSimulation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_id != null && m.hasOwnProperty("ad_group_id")) {
                                    p._ad_group_id = 1;
                                    if (!$util.isInteger(m.ad_group_id) && !(m.ad_group_id && $util.isInteger(m.ad_group_id.low) && $util.isInteger(m.ad_group_id.high)))
                                        return "ad_group_id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    switch (m.modification_method) {
                                    default:
                                        return "modification_method: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    p._end_date = 1;
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.cpc_bid_point_list != null && m.hasOwnProperty("cpc_bid_point_list")) {
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.verify(m.cpc_bid_point_list);
                                        if (e)
                                            return "cpc_bid_point_list." + e;
                                    }
                                }
                                if (m.cpv_bid_point_list != null && m.hasOwnProperty("cpv_bid_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CpvBidSimulationPointList.verify(m.cpv_bid_point_list);
                                        if (e)
                                            return "cpv_bid_point_list." + e;
                                    }
                                }
                                if (m.target_cpa_point_list != null && m.hasOwnProperty("target_cpa_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.verify(m.target_cpa_point_list);
                                        if (e)
                                            return "target_cpa_point_list." + e;
                                    }
                                }
                                if (m.target_roas_point_list != null && m.hasOwnProperty("target_roas_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.verify(m.target_roas_point_list);
                                        if (e)
                                            return "target_roas_point_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupSimulation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdGroupSimulation} AdGroupSimulation
                             */
                            AdGroupSimulation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdGroupSimulation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdGroupSimulation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_id != null) {
                                    if ($util.Long)
                                        (m.ad_group_id = $util.Long.fromValue(d.ad_group_id)).unsigned = false;
                                    else if (typeof d.ad_group_id === "string")
                                        m.ad_group_id = parseInt(d.ad_group_id, 10);
                                    else if (typeof d.ad_group_id === "number")
                                        m.ad_group_id = d.ad_group_id;
                                    else if (typeof d.ad_group_id === "object")
                                        m.ad_group_id = new $util.LongBits(d.ad_group_id.low >>> 0, d.ad_group_id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CPC_BID":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "CPV_BID":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "TARGET_CPA":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "BID_MODIFIER":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "TARGET_ROAS":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PERCENT_CPC_BID":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "BUDGET":
                                case 9:
                                    m.type = 9;
                                    break;
                                }
                                switch (d.modification_method) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.modification_method = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.modification_method = 1;
                                    break;
                                case "UNIFORM":
                                case 2:
                                    m.modification_method = 2;
                                    break;
                                case "DEFAULT":
                                case 3:
                                    m.modification_method = 3;
                                    break;
                                case "SCALING":
                                case 4:
                                    m.modification_method = 4;
                                    break;
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.cpc_bid_point_list != null) {
                                    if (typeof d.cpc_bid_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupSimulation.cpc_bid_point_list: object expected");
                                    m.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.fromObject(d.cpc_bid_point_list);
                                }
                                if (d.cpv_bid_point_list != null) {
                                    if (typeof d.cpv_bid_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupSimulation.cpv_bid_point_list: object expected");
                                    m.cpv_bid_point_list = $root.google.ads.googleads.v11.common.CpvBidSimulationPointList.fromObject(d.cpv_bid_point_list);
                                }
                                if (d.target_cpa_point_list != null) {
                                    if (typeof d.target_cpa_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupSimulation.target_cpa_point_list: object expected");
                                    m.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.fromObject(d.target_cpa_point_list);
                                }
                                if (d.target_roas_point_list != null) {
                                    if (typeof d.target_roas_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AdGroupSimulation.target_roas_point_list: object expected");
                                    m.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.fromObject(d.target_roas_point_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupSimulation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdGroupSimulation} m AdGroupSimulation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupSimulation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.modification_method = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType[m.type] : m.type;
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    d.modification_method = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod[m.modification_method] : m.modification_method;
                                }
                                if (m.cpc_bid_point_list != null && m.hasOwnProperty("cpc_bid_point_list")) {
                                    d.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.toObject(m.cpc_bid_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "cpc_bid_point_list";
                                }
                                if (m.target_cpa_point_list != null && m.hasOwnProperty("target_cpa_point_list")) {
                                    d.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.toObject(m.target_cpa_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "target_cpa_point_list";
                                }
                                if (m.cpv_bid_point_list != null && m.hasOwnProperty("cpv_bid_point_list")) {
                                    d.cpv_bid_point_list = $root.google.ads.googleads.v11.common.CpvBidSimulationPointList.toObject(m.cpv_bid_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "cpv_bid_point_list";
                                }
                                if (m.target_roas_point_list != null && m.hasOwnProperty("target_roas_point_list")) {
                                    d.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.toObject(m.target_roas_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "target_roas_point_list";
                                }
                                if (m.ad_group_id != null && m.hasOwnProperty("ad_group_id")) {
                                    if (typeof m.ad_group_id === "number")
                                        d.ad_group_id = o.longs === String ? String(m.ad_group_id) : m.ad_group_id;
                                    else
                                        d.ad_group_id = o.longs === String ? $util.Long.prototype.toString.call(m.ad_group_id) : o.longs === Number ? new $util.LongBits(m.ad_group_id.low >>> 0, m.ad_group_id.high >>> 0).toNumber() : m.ad_group_id;
                                    if (o.oneofs)
                                        d._ad_group_id = "ad_group_id";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                    if (o.oneofs)
                                        d._end_date = "end_date";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupSimulation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdGroupSimulation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupSimulation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupSimulation;
                        })();
    
                        resources.AdParameter = (function() {
    
                            /**
                             * Properties of an AdParameter.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdParameter
                             * @property {string|null} [resource_name] AdParameter resource_name
                             * @property {string|null} [ad_group_criterion] AdParameter ad_group_criterion
                             * @property {number|null} [parameter_index] AdParameter parameter_index
                             * @property {string|null} [insertion_text] AdParameter insertion_text
                             */
    
                            /**
                             * Constructs a new AdParameter.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdParameter.
                             * @implements IAdParameter
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdParameter=} [p] Properties to set
                             */
                            function AdParameter(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdParameter resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             */
                            AdParameter.prototype.resource_name = "";
    
                            /**
                             * AdParameter ad_group_criterion.
                             * @member {string|null|undefined} ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             */
                            AdParameter.prototype.ad_group_criterion = null;
    
                            /**
                             * AdParameter parameter_index.
                             * @member {number|null|undefined} parameter_index
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             */
                            AdParameter.prototype.parameter_index = null;
    
                            /**
                             * AdParameter insertion_text.
                             * @member {string|null|undefined} insertion_text
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             */
                            AdParameter.prototype.insertion_text = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdParameter _ad_group_criterion.
                             * @member {"ad_group_criterion"|undefined} _ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             */
                            Object.defineProperty(AdParameter.prototype, "_ad_group_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdParameter _parameter_index.
                             * @member {"parameter_index"|undefined} _parameter_index
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             */
                            Object.defineProperty(AdParameter.prototype, "_parameter_index", {
                                get: $util.oneOfGetter($oneOfFields = ["parameter_index"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AdParameter _insertion_text.
                             * @member {"insertion_text"|undefined} _insertion_text
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             */
                            Object.defineProperty(AdParameter.prototype, "_insertion_text", {
                                get: $util.oneOfGetter($oneOfFields = ["insertion_text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdParameter instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdParameter=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdParameter} AdParameter instance
                             */
                            AdParameter.create = function create(properties) {
                                return new AdParameter(properties);
                            };
    
                            /**
                             * Encodes the specified AdParameter message. Does not implicitly {@link google.ads.googleads.v11.resources.AdParameter.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdParameter} m AdParameter message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdParameter.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                    w.uint32(42).string(m.ad_group_criterion);
                                if (m.parameter_index != null && Object.hasOwnProperty.call(m, "parameter_index"))
                                    w.uint32(48).int64(m.parameter_index);
                                if (m.insertion_text != null && Object.hasOwnProperty.call(m, "insertion_text"))
                                    w.uint32(58).string(m.insertion_text);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdParameter message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdParameter.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdParameter} message AdParameter message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdParameter.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdParameter message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdParameter} AdParameter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdParameter.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdParameter();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 5:
                                        m.ad_group_criterion = r.string();
                                        break;
                                    case 6:
                                        m.parameter_index = r.int64();
                                        break;
                                    case 7:
                                        m.insertion_text = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdParameter message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdParameter} AdParameter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdParameter.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdParameter message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdParameter.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    p._ad_group_criterion = 1;
                                    if (!$util.isString(m.ad_group_criterion))
                                        return "ad_group_criterion: string expected";
                                }
                                if (m.parameter_index != null && m.hasOwnProperty("parameter_index")) {
                                    p._parameter_index = 1;
                                    if (!$util.isInteger(m.parameter_index) && !(m.parameter_index && $util.isInteger(m.parameter_index.low) && $util.isInteger(m.parameter_index.high)))
                                        return "parameter_index: integer|Long expected";
                                }
                                if (m.insertion_text != null && m.hasOwnProperty("insertion_text")) {
                                    p._insertion_text = 1;
                                    if (!$util.isString(m.insertion_text))
                                        return "insertion_text: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdParameter message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdParameter} AdParameter
                             */
                            AdParameter.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdParameter)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdParameter();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_criterion != null) {
                                    m.ad_group_criterion = String(d.ad_group_criterion);
                                }
                                if (d.parameter_index != null) {
                                    if ($util.Long)
                                        (m.parameter_index = $util.Long.fromValue(d.parameter_index)).unsigned = false;
                                    else if (typeof d.parameter_index === "string")
                                        m.parameter_index = parseInt(d.parameter_index, 10);
                                    else if (typeof d.parameter_index === "number")
                                        m.parameter_index = d.parameter_index;
                                    else if (typeof d.parameter_index === "object")
                                        m.parameter_index = new $util.LongBits(d.parameter_index.low >>> 0, d.parameter_index.high >>> 0).toNumber();
                                }
                                if (d.insertion_text != null) {
                                    m.insertion_text = String(d.insertion_text);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdParameter message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdParameter} m AdParameter
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdParameter.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    d.ad_group_criterion = m.ad_group_criterion;
                                    if (o.oneofs)
                                        d._ad_group_criterion = "ad_group_criterion";
                                }
                                if (m.parameter_index != null && m.hasOwnProperty("parameter_index")) {
                                    if (typeof m.parameter_index === "number")
                                        d.parameter_index = o.longs === String ? String(m.parameter_index) : m.parameter_index;
                                    else
                                        d.parameter_index = o.longs === String ? $util.Long.prototype.toString.call(m.parameter_index) : o.longs === Number ? new $util.LongBits(m.parameter_index.low >>> 0, m.parameter_index.high >>> 0).toNumber() : m.parameter_index;
                                    if (o.oneofs)
                                        d._parameter_index = "parameter_index";
                                }
                                if (m.insertion_text != null && m.hasOwnProperty("insertion_text")) {
                                    d.insertion_text = m.insertion_text;
                                    if (o.oneofs)
                                        d._insertion_text = "insertion_text";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdParameter to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdParameter
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdParameter.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdParameter;
                        })();
    
                        resources.AdScheduleView = (function() {
    
                            /**
                             * Properties of an AdScheduleView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAdScheduleView
                             * @property {string|null} [resource_name] AdScheduleView resource_name
                             */
    
                            /**
                             * Constructs a new AdScheduleView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AdScheduleView.
                             * @implements IAdScheduleView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAdScheduleView=} [p] Properties to set
                             */
                            function AdScheduleView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdScheduleView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @instance
                             */
                            AdScheduleView.prototype.resource_name = "";
    
                            /**
                             * Creates a new AdScheduleView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdScheduleView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AdScheduleView} AdScheduleView instance
                             */
                            AdScheduleView.create = function create(properties) {
                                return new AdScheduleView(properties);
                            };
    
                            /**
                             * Encodes the specified AdScheduleView message. Does not implicitly {@link google.ads.googleads.v11.resources.AdScheduleView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdScheduleView} m AdScheduleView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdScheduleView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdScheduleView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AdScheduleView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAdScheduleView} message AdScheduleView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdScheduleView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdScheduleView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AdScheduleView} AdScheduleView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdScheduleView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AdScheduleView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdScheduleView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AdScheduleView} AdScheduleView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdScheduleView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdScheduleView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdScheduleView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdScheduleView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AdScheduleView} AdScheduleView
                             */
                            AdScheduleView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AdScheduleView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AdScheduleView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdScheduleView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @static
                             * @param {google.ads.googleads.v11.resources.AdScheduleView} m AdScheduleView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdScheduleView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdScheduleView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AdScheduleView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdScheduleView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdScheduleView;
                        })();
    
                        resources.AgeRangeView = (function() {
    
                            /**
                             * Properties of an AgeRangeView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAgeRangeView
                             * @property {string|null} [resource_name] AgeRangeView resource_name
                             */
    
                            /**
                             * Constructs a new AgeRangeView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AgeRangeView.
                             * @implements IAgeRangeView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAgeRangeView=} [p] Properties to set
                             */
                            function AgeRangeView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AgeRangeView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @instance
                             */
                            AgeRangeView.prototype.resource_name = "";
    
                            /**
                             * Creates a new AgeRangeView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAgeRangeView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AgeRangeView} AgeRangeView instance
                             */
                            AgeRangeView.create = function create(properties) {
                                return new AgeRangeView(properties);
                            };
    
                            /**
                             * Encodes the specified AgeRangeView message. Does not implicitly {@link google.ads.googleads.v11.resources.AgeRangeView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAgeRangeView} m AgeRangeView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeRangeView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AgeRangeView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AgeRangeView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAgeRangeView} message AgeRangeView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AgeRangeView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AgeRangeView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AgeRangeView} AgeRangeView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeRangeView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AgeRangeView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AgeRangeView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AgeRangeView} AgeRangeView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AgeRangeView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AgeRangeView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AgeRangeView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AgeRangeView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AgeRangeView} AgeRangeView
                             */
                            AgeRangeView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AgeRangeView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AgeRangeView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AgeRangeView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.AgeRangeView} m AgeRangeView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AgeRangeView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AgeRangeView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AgeRangeView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AgeRangeView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AgeRangeView;
                        })();
    
                        resources.Asset = (function() {
    
                            /**
                             * Properties of an Asset.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAsset
                             * @property {string|null} [resource_name] Asset resource_name
                             * @property {number|null} [id] Asset id
                             * @property {string|null} [name] Asset name
                             * @property {google.ads.googleads.v11.enums.AssetTypeEnum.AssetType|null} [type] Asset type
                             * @property {Array.<string>|null} [final_urls] Asset final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] Asset final_mobile_urls
                             * @property {string|null} [tracking_url_template] Asset tracking_url_template
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] Asset url_custom_parameters
                             * @property {string|null} [final_url_suffix] Asset final_url_suffix
                             * @property {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource|null} [source] Asset source
                             * @property {google.ads.googleads.v11.resources.IAssetPolicySummary|null} [policy_summary] Asset policy_summary
                             * @property {google.ads.googleads.v11.common.IYoutubeVideoAsset|null} [youtube_video_asset] Asset youtube_video_asset
                             * @property {google.ads.googleads.v11.common.IMediaBundleAsset|null} [media_bundle_asset] Asset media_bundle_asset
                             * @property {google.ads.googleads.v11.common.IImageAsset|null} [image_asset] Asset image_asset
                             * @property {google.ads.googleads.v11.common.ITextAsset|null} [text_asset] Asset text_asset
                             * @property {google.ads.googleads.v11.common.ILeadFormAsset|null} [lead_form_asset] Asset lead_form_asset
                             * @property {google.ads.googleads.v11.common.IBookOnGoogleAsset|null} [book_on_google_asset] Asset book_on_google_asset
                             * @property {google.ads.googleads.v11.common.IPromotionAsset|null} [promotion_asset] Asset promotion_asset
                             * @property {google.ads.googleads.v11.common.ICalloutAsset|null} [callout_asset] Asset callout_asset
                             * @property {google.ads.googleads.v11.common.IStructuredSnippetAsset|null} [structured_snippet_asset] Asset structured_snippet_asset
                             * @property {google.ads.googleads.v11.common.ISitelinkAsset|null} [sitelink_asset] Asset sitelink_asset
                             * @property {google.ads.googleads.v11.common.IPageFeedAsset|null} [page_feed_asset] Asset page_feed_asset
                             * @property {google.ads.googleads.v11.common.IDynamicEducationAsset|null} [dynamic_education_asset] Asset dynamic_education_asset
                             * @property {google.ads.googleads.v11.common.IMobileAppAsset|null} [mobile_app_asset] Asset mobile_app_asset
                             * @property {google.ads.googleads.v11.common.IHotelCalloutAsset|null} [hotel_callout_asset] Asset hotel_callout_asset
                             * @property {google.ads.googleads.v11.common.ICallAsset|null} [call_asset] Asset call_asset
                             * @property {google.ads.googleads.v11.common.IPriceAsset|null} [price_asset] Asset price_asset
                             * @property {google.ads.googleads.v11.common.ICallToActionAsset|null} [call_to_action_asset] Asset call_to_action_asset
                             * @property {google.ads.googleads.v11.common.IDynamicRealEstateAsset|null} [dynamic_real_estate_asset] Asset dynamic_real_estate_asset
                             * @property {google.ads.googleads.v11.common.IDynamicCustomAsset|null} [dynamic_custom_asset] Asset dynamic_custom_asset
                             * @property {google.ads.googleads.v11.common.IDynamicHotelsAndRentalsAsset|null} [dynamic_hotels_and_rentals_asset] Asset dynamic_hotels_and_rentals_asset
                             * @property {google.ads.googleads.v11.common.IDynamicFlightsAsset|null} [dynamic_flights_asset] Asset dynamic_flights_asset
                             * @property {google.ads.googleads.v11.common.IDiscoveryCarouselCardAsset|null} [discovery_carousel_card_asset] Asset discovery_carousel_card_asset
                             * @property {google.ads.googleads.v11.common.IDynamicTravelAsset|null} [dynamic_travel_asset] Asset dynamic_travel_asset
                             * @property {google.ads.googleads.v11.common.IDynamicLocalAsset|null} [dynamic_local_asset] Asset dynamic_local_asset
                             * @property {google.ads.googleads.v11.common.IDynamicJobsAsset|null} [dynamic_jobs_asset] Asset dynamic_jobs_asset
                             */
    
                            /**
                             * Constructs a new Asset.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an Asset.
                             * @implements IAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAsset=} [p] Properties to set
                             */
                            function Asset(p) {
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                this.url_custom_parameters = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Asset resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.resource_name = "";
    
                            /**
                             * Asset id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.id = null;
    
                            /**
                             * Asset name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.name = null;
    
                            /**
                             * Asset type.
                             * @member {google.ads.googleads.v11.enums.AssetTypeEnum.AssetType} type
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.type = 0;
    
                            /**
                             * Asset final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * Asset final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * Asset tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.tracking_url_template = null;
    
                            /**
                             * Asset url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * Asset final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.final_url_suffix = null;
    
                            /**
                             * Asset source.
                             * @member {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource} source
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.source = 0;
    
                            /**
                             * Asset policy_summary.
                             * @member {google.ads.googleads.v11.resources.IAssetPolicySummary|null|undefined} policy_summary
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.policy_summary = null;
    
                            /**
                             * Asset youtube_video_asset.
                             * @member {google.ads.googleads.v11.common.IYoutubeVideoAsset|null|undefined} youtube_video_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.youtube_video_asset = null;
    
                            /**
                             * Asset media_bundle_asset.
                             * @member {google.ads.googleads.v11.common.IMediaBundleAsset|null|undefined} media_bundle_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.media_bundle_asset = null;
    
                            /**
                             * Asset image_asset.
                             * @member {google.ads.googleads.v11.common.IImageAsset|null|undefined} image_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.image_asset = null;
    
                            /**
                             * Asset text_asset.
                             * @member {google.ads.googleads.v11.common.ITextAsset|null|undefined} text_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.text_asset = null;
    
                            /**
                             * Asset lead_form_asset.
                             * @member {google.ads.googleads.v11.common.ILeadFormAsset|null|undefined} lead_form_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.lead_form_asset = null;
    
                            /**
                             * Asset book_on_google_asset.
                             * @member {google.ads.googleads.v11.common.IBookOnGoogleAsset|null|undefined} book_on_google_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.book_on_google_asset = null;
    
                            /**
                             * Asset promotion_asset.
                             * @member {google.ads.googleads.v11.common.IPromotionAsset|null|undefined} promotion_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.promotion_asset = null;
    
                            /**
                             * Asset callout_asset.
                             * @member {google.ads.googleads.v11.common.ICalloutAsset|null|undefined} callout_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.callout_asset = null;
    
                            /**
                             * Asset structured_snippet_asset.
                             * @member {google.ads.googleads.v11.common.IStructuredSnippetAsset|null|undefined} structured_snippet_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.structured_snippet_asset = null;
    
                            /**
                             * Asset sitelink_asset.
                             * @member {google.ads.googleads.v11.common.ISitelinkAsset|null|undefined} sitelink_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.sitelink_asset = null;
    
                            /**
                             * Asset page_feed_asset.
                             * @member {google.ads.googleads.v11.common.IPageFeedAsset|null|undefined} page_feed_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.page_feed_asset = null;
    
                            /**
                             * Asset dynamic_education_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicEducationAsset|null|undefined} dynamic_education_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_education_asset = null;
    
                            /**
                             * Asset mobile_app_asset.
                             * @member {google.ads.googleads.v11.common.IMobileAppAsset|null|undefined} mobile_app_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.mobile_app_asset = null;
    
                            /**
                             * Asset hotel_callout_asset.
                             * @member {google.ads.googleads.v11.common.IHotelCalloutAsset|null|undefined} hotel_callout_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.hotel_callout_asset = null;
    
                            /**
                             * Asset call_asset.
                             * @member {google.ads.googleads.v11.common.ICallAsset|null|undefined} call_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.call_asset = null;
    
                            /**
                             * Asset price_asset.
                             * @member {google.ads.googleads.v11.common.IPriceAsset|null|undefined} price_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.price_asset = null;
    
                            /**
                             * Asset call_to_action_asset.
                             * @member {google.ads.googleads.v11.common.ICallToActionAsset|null|undefined} call_to_action_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.call_to_action_asset = null;
    
                            /**
                             * Asset dynamic_real_estate_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicRealEstateAsset|null|undefined} dynamic_real_estate_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_real_estate_asset = null;
    
                            /**
                             * Asset dynamic_custom_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicCustomAsset|null|undefined} dynamic_custom_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_custom_asset = null;
    
                            /**
                             * Asset dynamic_hotels_and_rentals_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicHotelsAndRentalsAsset|null|undefined} dynamic_hotels_and_rentals_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_hotels_and_rentals_asset = null;
    
                            /**
                             * Asset dynamic_flights_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicFlightsAsset|null|undefined} dynamic_flights_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_flights_asset = null;
    
                            /**
                             * Asset discovery_carousel_card_asset.
                             * @member {google.ads.googleads.v11.common.IDiscoveryCarouselCardAsset|null|undefined} discovery_carousel_card_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.discovery_carousel_card_asset = null;
    
                            /**
                             * Asset dynamic_travel_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicTravelAsset|null|undefined} dynamic_travel_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_travel_asset = null;
    
                            /**
                             * Asset dynamic_local_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicLocalAsset|null|undefined} dynamic_local_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_local_asset = null;
    
                            /**
                             * Asset dynamic_jobs_asset.
                             * @member {google.ads.googleads.v11.common.IDynamicJobsAsset|null|undefined} dynamic_jobs_asset
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Asset.prototype.dynamic_jobs_asset = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Asset _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Object.defineProperty(Asset.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Asset _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Object.defineProperty(Asset.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Asset _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Object.defineProperty(Asset.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Asset _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Object.defineProperty(Asset.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Asset asset_data.
                             * @member {"youtube_video_asset"|"media_bundle_asset"|"image_asset"|"text_asset"|"lead_form_asset"|"book_on_google_asset"|"promotion_asset"|"callout_asset"|"structured_snippet_asset"|"sitelink_asset"|"page_feed_asset"|"dynamic_education_asset"|"mobile_app_asset"|"hotel_callout_asset"|"call_asset"|"price_asset"|"call_to_action_asset"|"dynamic_real_estate_asset"|"dynamic_custom_asset"|"dynamic_hotels_and_rentals_asset"|"dynamic_flights_asset"|"discovery_carousel_card_asset"|"dynamic_travel_asset"|"dynamic_local_asset"|"dynamic_jobs_asset"|undefined} asset_data
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             */
                            Object.defineProperty(Asset.prototype, "asset_data", {
                                get: $util.oneOfGetter($oneOfFields = ["youtube_video_asset", "media_bundle_asset", "image_asset", "text_asset", "lead_form_asset", "book_on_google_asset", "promotion_asset", "callout_asset", "structured_snippet_asset", "sitelink_asset", "page_feed_asset", "dynamic_education_asset", "mobile_app_asset", "hotel_callout_asset", "call_asset", "price_asset", "call_to_action_asset", "dynamic_real_estate_asset", "dynamic_custom_asset", "dynamic_hotels_and_rentals_asset", "dynamic_flights_asset", "discovery_carousel_card_asset", "dynamic_travel_asset", "dynamic_local_asset", "dynamic_jobs_asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Asset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Asset} Asset instance
                             */
                            Asset.create = function create(properties) {
                                return new Asset(properties);
                            };
    
                            /**
                             * Encodes the specified Asset message. Does not implicitly {@link google.ads.googleads.v11.resources.Asset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAsset} m Asset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Asset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.youtube_video_asset != null && Object.hasOwnProperty.call(m, "youtube_video_asset"))
                                    $root.google.ads.googleads.v11.common.YoutubeVideoAsset.encode(m.youtube_video_asset, w.uint32(42).fork()).ldelim();
                                if (m.media_bundle_asset != null && Object.hasOwnProperty.call(m, "media_bundle_asset"))
                                    $root.google.ads.googleads.v11.common.MediaBundleAsset.encode(m.media_bundle_asset, w.uint32(50).fork()).ldelim();
                                if (m.image_asset != null && Object.hasOwnProperty.call(m, "image_asset"))
                                    $root.google.ads.googleads.v11.common.ImageAsset.encode(m.image_asset, w.uint32(58).fork()).ldelim();
                                if (m.text_asset != null && Object.hasOwnProperty.call(m, "text_asset"))
                                    $root.google.ads.googleads.v11.common.TextAsset.encode(m.text_asset, w.uint32(66).fork()).ldelim();
                                if (m.lead_form_asset != null && Object.hasOwnProperty.call(m, "lead_form_asset"))
                                    $root.google.ads.googleads.v11.common.LeadFormAsset.encode(m.lead_form_asset, w.uint32(74).fork()).ldelim();
                                if (m.book_on_google_asset != null && Object.hasOwnProperty.call(m, "book_on_google_asset"))
                                    $root.google.ads.googleads.v11.common.BookOnGoogleAsset.encode(m.book_on_google_asset, w.uint32(82).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(88).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(98).string(m.name);
                                if (m.policy_summary != null && Object.hasOwnProperty.call(m, "policy_summary"))
                                    $root.google.ads.googleads.v11.resources.AssetPolicySummary.encode(m.policy_summary, w.uint32(106).fork()).ldelim();
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(114).string(m.final_urls[i]);
                                }
                                if (m.promotion_asset != null && Object.hasOwnProperty.call(m, "promotion_asset"))
                                    $root.google.ads.googleads.v11.common.PromotionAsset.encode(m.promotion_asset, w.uint32(122).fork()).ldelim();
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(130).string(m.final_mobile_urls[i]);
                                }
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(138).string(m.tracking_url_template);
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(146).fork()).ldelim();
                                }
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(154).string(m.final_url_suffix);
                                if (m.callout_asset != null && Object.hasOwnProperty.call(m, "callout_asset"))
                                    $root.google.ads.googleads.v11.common.CalloutAsset.encode(m.callout_asset, w.uint32(162).fork()).ldelim();
                                if (m.structured_snippet_asset != null && Object.hasOwnProperty.call(m, "structured_snippet_asset"))
                                    $root.google.ads.googleads.v11.common.StructuredSnippetAsset.encode(m.structured_snippet_asset, w.uint32(170).fork()).ldelim();
                                if (m.sitelink_asset != null && Object.hasOwnProperty.call(m, "sitelink_asset"))
                                    $root.google.ads.googleads.v11.common.SitelinkAsset.encode(m.sitelink_asset, w.uint32(178).fork()).ldelim();
                                if (m.page_feed_asset != null && Object.hasOwnProperty.call(m, "page_feed_asset"))
                                    $root.google.ads.googleads.v11.common.PageFeedAsset.encode(m.page_feed_asset, w.uint32(186).fork()).ldelim();
                                if (m.dynamic_education_asset != null && Object.hasOwnProperty.call(m, "dynamic_education_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicEducationAsset.encode(m.dynamic_education_asset, w.uint32(194).fork()).ldelim();
                                if (m.mobile_app_asset != null && Object.hasOwnProperty.call(m, "mobile_app_asset"))
                                    $root.google.ads.googleads.v11.common.MobileAppAsset.encode(m.mobile_app_asset, w.uint32(202).fork()).ldelim();
                                if (m.hotel_callout_asset != null && Object.hasOwnProperty.call(m, "hotel_callout_asset"))
                                    $root.google.ads.googleads.v11.common.HotelCalloutAsset.encode(m.hotel_callout_asset, w.uint32(210).fork()).ldelim();
                                if (m.call_asset != null && Object.hasOwnProperty.call(m, "call_asset"))
                                    $root.google.ads.googleads.v11.common.CallAsset.encode(m.call_asset, w.uint32(218).fork()).ldelim();
                                if (m.price_asset != null && Object.hasOwnProperty.call(m, "price_asset"))
                                    $root.google.ads.googleads.v11.common.PriceAsset.encode(m.price_asset, w.uint32(226).fork()).ldelim();
                                if (m.call_to_action_asset != null && Object.hasOwnProperty.call(m, "call_to_action_asset"))
                                    $root.google.ads.googleads.v11.common.CallToActionAsset.encode(m.call_to_action_asset, w.uint32(234).fork()).ldelim();
                                if (m.dynamic_real_estate_asset != null && Object.hasOwnProperty.call(m, "dynamic_real_estate_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicRealEstateAsset.encode(m.dynamic_real_estate_asset, w.uint32(242).fork()).ldelim();
                                if (m.dynamic_custom_asset != null && Object.hasOwnProperty.call(m, "dynamic_custom_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicCustomAsset.encode(m.dynamic_custom_asset, w.uint32(250).fork()).ldelim();
                                if (m.dynamic_hotels_and_rentals_asset != null && Object.hasOwnProperty.call(m, "dynamic_hotels_and_rentals_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.encode(m.dynamic_hotels_and_rentals_asset, w.uint32(258).fork()).ldelim();
                                if (m.dynamic_flights_asset != null && Object.hasOwnProperty.call(m, "dynamic_flights_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicFlightsAsset.encode(m.dynamic_flights_asset, w.uint32(266).fork()).ldelim();
                                if (m.discovery_carousel_card_asset != null && Object.hasOwnProperty.call(m, "discovery_carousel_card_asset"))
                                    $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset.encode(m.discovery_carousel_card_asset, w.uint32(274).fork()).ldelim();
                                if (m.dynamic_travel_asset != null && Object.hasOwnProperty.call(m, "dynamic_travel_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicTravelAsset.encode(m.dynamic_travel_asset, w.uint32(282).fork()).ldelim();
                                if (m.dynamic_local_asset != null && Object.hasOwnProperty.call(m, "dynamic_local_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicLocalAsset.encode(m.dynamic_local_asset, w.uint32(290).fork()).ldelim();
                                if (m.dynamic_jobs_asset != null && Object.hasOwnProperty.call(m, "dynamic_jobs_asset"))
                                    $root.google.ads.googleads.v11.common.DynamicJobsAsset.encode(m.dynamic_jobs_asset, w.uint32(298).fork()).ldelim();
                                if (m.source != null && Object.hasOwnProperty.call(m, "source"))
                                    w.uint32(304).int32(m.source);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Asset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Asset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAsset} message Asset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Asset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Asset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Asset} Asset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Asset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Asset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 11:
                                        m.id = r.int64();
                                        break;
                                    case 12:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 14:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 16:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 17:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 18:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 19:
                                        m.final_url_suffix = r.string();
                                        break;
                                    case 38:
                                        m.source = r.int32();
                                        break;
                                    case 13:
                                        m.policy_summary = $root.google.ads.googleads.v11.resources.AssetPolicySummary.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.youtube_video_asset = $root.google.ads.googleads.v11.common.YoutubeVideoAsset.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.media_bundle_asset = $root.google.ads.googleads.v11.common.MediaBundleAsset.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.image_asset = $root.google.ads.googleads.v11.common.ImageAsset.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.text_asset = $root.google.ads.googleads.v11.common.TextAsset.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.lead_form_asset = $root.google.ads.googleads.v11.common.LeadFormAsset.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.book_on_google_asset = $root.google.ads.googleads.v11.common.BookOnGoogleAsset.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.promotion_asset = $root.google.ads.googleads.v11.common.PromotionAsset.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.callout_asset = $root.google.ads.googleads.v11.common.CalloutAsset.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.structured_snippet_asset = $root.google.ads.googleads.v11.common.StructuredSnippetAsset.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.sitelink_asset = $root.google.ads.googleads.v11.common.SitelinkAsset.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.page_feed_asset = $root.google.ads.googleads.v11.common.PageFeedAsset.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.dynamic_education_asset = $root.google.ads.googleads.v11.common.DynamicEducationAsset.decode(r, r.uint32());
                                        break;
                                    case 25:
                                        m.mobile_app_asset = $root.google.ads.googleads.v11.common.MobileAppAsset.decode(r, r.uint32());
                                        break;
                                    case 26:
                                        m.hotel_callout_asset = $root.google.ads.googleads.v11.common.HotelCalloutAsset.decode(r, r.uint32());
                                        break;
                                    case 27:
                                        m.call_asset = $root.google.ads.googleads.v11.common.CallAsset.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.price_asset = $root.google.ads.googleads.v11.common.PriceAsset.decode(r, r.uint32());
                                        break;
                                    case 29:
                                        m.call_to_action_asset = $root.google.ads.googleads.v11.common.CallToActionAsset.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.dynamic_real_estate_asset = $root.google.ads.googleads.v11.common.DynamicRealEstateAsset.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.dynamic_custom_asset = $root.google.ads.googleads.v11.common.DynamicCustomAsset.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.dynamic_hotels_and_rentals_asset = $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.decode(r, r.uint32());
                                        break;
                                    case 33:
                                        m.dynamic_flights_asset = $root.google.ads.googleads.v11.common.DynamicFlightsAsset.decode(r, r.uint32());
                                        break;
                                    case 34:
                                        m.discovery_carousel_card_asset = $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset.decode(r, r.uint32());
                                        break;
                                    case 35:
                                        m.dynamic_travel_asset = $root.google.ads.googleads.v11.common.DynamicTravelAsset.decode(r, r.uint32());
                                        break;
                                    case 36:
                                        m.dynamic_local_asset = $root.google.ads.googleads.v11.common.DynamicLocalAsset.decode(r, r.uint32());
                                        break;
                                    case 37:
                                        m.dynamic_jobs_asset = $root.google.ads.googleads.v11.common.DynamicJobsAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an Asset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Asset} Asset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Asset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Asset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Asset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                        break;
                                    }
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    switch (m.source) {
                                    default:
                                        return "source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetPolicySummary.verify(m.policy_summary);
                                        if (e)
                                            return "policy_summary." + e;
                                    }
                                }
                                if (m.youtube_video_asset != null && m.hasOwnProperty("youtube_video_asset")) {
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YoutubeVideoAsset.verify(m.youtube_video_asset);
                                        if (e)
                                            return "youtube_video_asset." + e;
                                    }
                                }
                                if (m.media_bundle_asset != null && m.hasOwnProperty("media_bundle_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MediaBundleAsset.verify(m.media_bundle_asset);
                                        if (e)
                                            return "media_bundle_asset." + e;
                                    }
                                }
                                if (m.image_asset != null && m.hasOwnProperty("image_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ImageAsset.verify(m.image_asset);
                                        if (e)
                                            return "image_asset." + e;
                                    }
                                }
                                if (m.text_asset != null && m.hasOwnProperty("text_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TextAsset.verify(m.text_asset);
                                        if (e)
                                            return "text_asset." + e;
                                    }
                                }
                                if (m.lead_form_asset != null && m.hasOwnProperty("lead_form_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LeadFormAsset.verify(m.lead_form_asset);
                                        if (e)
                                            return "lead_form_asset." + e;
                                    }
                                }
                                if (m.book_on_google_asset != null && m.hasOwnProperty("book_on_google_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.BookOnGoogleAsset.verify(m.book_on_google_asset);
                                        if (e)
                                            return "book_on_google_asset." + e;
                                    }
                                }
                                if (m.promotion_asset != null && m.hasOwnProperty("promotion_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PromotionAsset.verify(m.promotion_asset);
                                        if (e)
                                            return "promotion_asset." + e;
                                    }
                                }
                                if (m.callout_asset != null && m.hasOwnProperty("callout_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CalloutAsset.verify(m.callout_asset);
                                        if (e)
                                            return "callout_asset." + e;
                                    }
                                }
                                if (m.structured_snippet_asset != null && m.hasOwnProperty("structured_snippet_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.StructuredSnippetAsset.verify(m.structured_snippet_asset);
                                        if (e)
                                            return "structured_snippet_asset." + e;
                                    }
                                }
                                if (m.sitelink_asset != null && m.hasOwnProperty("sitelink_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.SitelinkAsset.verify(m.sitelink_asset);
                                        if (e)
                                            return "sitelink_asset." + e;
                                    }
                                }
                                if (m.page_feed_asset != null && m.hasOwnProperty("page_feed_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PageFeedAsset.verify(m.page_feed_asset);
                                        if (e)
                                            return "page_feed_asset." + e;
                                    }
                                }
                                if (m.dynamic_education_asset != null && m.hasOwnProperty("dynamic_education_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicEducationAsset.verify(m.dynamic_education_asset);
                                        if (e)
                                            return "dynamic_education_asset." + e;
                                    }
                                }
                                if (m.mobile_app_asset != null && m.hasOwnProperty("mobile_app_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileAppAsset.verify(m.mobile_app_asset);
                                        if (e)
                                            return "mobile_app_asset." + e;
                                    }
                                }
                                if (m.hotel_callout_asset != null && m.hasOwnProperty("hotel_callout_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelCalloutAsset.verify(m.hotel_callout_asset);
                                        if (e)
                                            return "hotel_callout_asset." + e;
                                    }
                                }
                                if (m.call_asset != null && m.hasOwnProperty("call_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CallAsset.verify(m.call_asset);
                                        if (e)
                                            return "call_asset." + e;
                                    }
                                }
                                if (m.price_asset != null && m.hasOwnProperty("price_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PriceAsset.verify(m.price_asset);
                                        if (e)
                                            return "price_asset." + e;
                                    }
                                }
                                if (m.call_to_action_asset != null && m.hasOwnProperty("call_to_action_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CallToActionAsset.verify(m.call_to_action_asset);
                                        if (e)
                                            return "call_to_action_asset." + e;
                                    }
                                }
                                if (m.dynamic_real_estate_asset != null && m.hasOwnProperty("dynamic_real_estate_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicRealEstateAsset.verify(m.dynamic_real_estate_asset);
                                        if (e)
                                            return "dynamic_real_estate_asset." + e;
                                    }
                                }
                                if (m.dynamic_custom_asset != null && m.hasOwnProperty("dynamic_custom_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicCustomAsset.verify(m.dynamic_custom_asset);
                                        if (e)
                                            return "dynamic_custom_asset." + e;
                                    }
                                }
                                if (m.dynamic_hotels_and_rentals_asset != null && m.hasOwnProperty("dynamic_hotels_and_rentals_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.verify(m.dynamic_hotels_and_rentals_asset);
                                        if (e)
                                            return "dynamic_hotels_and_rentals_asset." + e;
                                    }
                                }
                                if (m.dynamic_flights_asset != null && m.hasOwnProperty("dynamic_flights_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicFlightsAsset.verify(m.dynamic_flights_asset);
                                        if (e)
                                            return "dynamic_flights_asset." + e;
                                    }
                                }
                                if (m.discovery_carousel_card_asset != null && m.hasOwnProperty("discovery_carousel_card_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset.verify(m.discovery_carousel_card_asset);
                                        if (e)
                                            return "discovery_carousel_card_asset." + e;
                                    }
                                }
                                if (m.dynamic_travel_asset != null && m.hasOwnProperty("dynamic_travel_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicTravelAsset.verify(m.dynamic_travel_asset);
                                        if (e)
                                            return "dynamic_travel_asset." + e;
                                    }
                                }
                                if (m.dynamic_local_asset != null && m.hasOwnProperty("dynamic_local_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicLocalAsset.verify(m.dynamic_local_asset);
                                        if (e)
                                            return "dynamic_local_asset." + e;
                                    }
                                }
                                if (m.dynamic_jobs_asset != null && m.hasOwnProperty("dynamic_jobs_asset")) {
                                    if (p.asset_data === 1)
                                        return "asset_data: multiple values";
                                    p.asset_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicJobsAsset.verify(m.dynamic_jobs_asset);
                                        if (e)
                                            return "dynamic_jobs_asset." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Asset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Asset} Asset
                             */
                            Asset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Asset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Asset();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "IMAGE":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "TEXT":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "LEAD_FORM":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "BOOK_ON_GOOGLE":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "PROMOTION":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "CALLOUT":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "SITELINK":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "PAGE_FEED":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "DYNAMIC_EDUCATION":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "MOBILE_APP":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "CALL":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "PRICE":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "CALL_TO_ACTION":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "DYNAMIC_REAL_ESTATE":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "DYNAMIC_CUSTOM":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "DYNAMIC_HOTELS_AND_RENTALS":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "DYNAMIC_FLIGHTS":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "DISCOVERY_CAROUSEL_CARD":
                                case 23:
                                    m.type = 23;
                                    break;
                                case "DYNAMIC_TRAVEL":
                                case 24:
                                    m.type = 24;
                                    break;
                                case "DYNAMIC_LOCAL":
                                case 25:
                                    m.type = 25;
                                    break;
                                case "DYNAMIC_JOBS":
                                case 26:
                                    m.type = 26;
                                    break;
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Asset.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                switch (d.source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.source = 1;
                                    break;
                                case "ADVERTISER":
                                case 2:
                                    m.source = 2;
                                    break;
                                case "AUTOMATICALLY_CREATED":
                                case 3:
                                    m.source = 3;
                                    break;
                                }
                                if (d.policy_summary != null) {
                                    if (typeof d.policy_summary !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.policy_summary: object expected");
                                    m.policy_summary = $root.google.ads.googleads.v11.resources.AssetPolicySummary.fromObject(d.policy_summary);
                                }
                                if (d.youtube_video_asset != null) {
                                    if (typeof d.youtube_video_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.youtube_video_asset: object expected");
                                    m.youtube_video_asset = $root.google.ads.googleads.v11.common.YoutubeVideoAsset.fromObject(d.youtube_video_asset);
                                }
                                if (d.media_bundle_asset != null) {
                                    if (typeof d.media_bundle_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.media_bundle_asset: object expected");
                                    m.media_bundle_asset = $root.google.ads.googleads.v11.common.MediaBundleAsset.fromObject(d.media_bundle_asset);
                                }
                                if (d.image_asset != null) {
                                    if (typeof d.image_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.image_asset: object expected");
                                    m.image_asset = $root.google.ads.googleads.v11.common.ImageAsset.fromObject(d.image_asset);
                                }
                                if (d.text_asset != null) {
                                    if (typeof d.text_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.text_asset: object expected");
                                    m.text_asset = $root.google.ads.googleads.v11.common.TextAsset.fromObject(d.text_asset);
                                }
                                if (d.lead_form_asset != null) {
                                    if (typeof d.lead_form_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.lead_form_asset: object expected");
                                    m.lead_form_asset = $root.google.ads.googleads.v11.common.LeadFormAsset.fromObject(d.lead_form_asset);
                                }
                                if (d.book_on_google_asset != null) {
                                    if (typeof d.book_on_google_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.book_on_google_asset: object expected");
                                    m.book_on_google_asset = $root.google.ads.googleads.v11.common.BookOnGoogleAsset.fromObject(d.book_on_google_asset);
                                }
                                if (d.promotion_asset != null) {
                                    if (typeof d.promotion_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.promotion_asset: object expected");
                                    m.promotion_asset = $root.google.ads.googleads.v11.common.PromotionAsset.fromObject(d.promotion_asset);
                                }
                                if (d.callout_asset != null) {
                                    if (typeof d.callout_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.callout_asset: object expected");
                                    m.callout_asset = $root.google.ads.googleads.v11.common.CalloutAsset.fromObject(d.callout_asset);
                                }
                                if (d.structured_snippet_asset != null) {
                                    if (typeof d.structured_snippet_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.structured_snippet_asset: object expected");
                                    m.structured_snippet_asset = $root.google.ads.googleads.v11.common.StructuredSnippetAsset.fromObject(d.structured_snippet_asset);
                                }
                                if (d.sitelink_asset != null) {
                                    if (typeof d.sitelink_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.sitelink_asset: object expected");
                                    m.sitelink_asset = $root.google.ads.googleads.v11.common.SitelinkAsset.fromObject(d.sitelink_asset);
                                }
                                if (d.page_feed_asset != null) {
                                    if (typeof d.page_feed_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.page_feed_asset: object expected");
                                    m.page_feed_asset = $root.google.ads.googleads.v11.common.PageFeedAsset.fromObject(d.page_feed_asset);
                                }
                                if (d.dynamic_education_asset != null) {
                                    if (typeof d.dynamic_education_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_education_asset: object expected");
                                    m.dynamic_education_asset = $root.google.ads.googleads.v11.common.DynamicEducationAsset.fromObject(d.dynamic_education_asset);
                                }
                                if (d.mobile_app_asset != null) {
                                    if (typeof d.mobile_app_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.mobile_app_asset: object expected");
                                    m.mobile_app_asset = $root.google.ads.googleads.v11.common.MobileAppAsset.fromObject(d.mobile_app_asset);
                                }
                                if (d.hotel_callout_asset != null) {
                                    if (typeof d.hotel_callout_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.hotel_callout_asset: object expected");
                                    m.hotel_callout_asset = $root.google.ads.googleads.v11.common.HotelCalloutAsset.fromObject(d.hotel_callout_asset);
                                }
                                if (d.call_asset != null) {
                                    if (typeof d.call_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.call_asset: object expected");
                                    m.call_asset = $root.google.ads.googleads.v11.common.CallAsset.fromObject(d.call_asset);
                                }
                                if (d.price_asset != null) {
                                    if (typeof d.price_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.price_asset: object expected");
                                    m.price_asset = $root.google.ads.googleads.v11.common.PriceAsset.fromObject(d.price_asset);
                                }
                                if (d.call_to_action_asset != null) {
                                    if (typeof d.call_to_action_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.call_to_action_asset: object expected");
                                    m.call_to_action_asset = $root.google.ads.googleads.v11.common.CallToActionAsset.fromObject(d.call_to_action_asset);
                                }
                                if (d.dynamic_real_estate_asset != null) {
                                    if (typeof d.dynamic_real_estate_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_real_estate_asset: object expected");
                                    m.dynamic_real_estate_asset = $root.google.ads.googleads.v11.common.DynamicRealEstateAsset.fromObject(d.dynamic_real_estate_asset);
                                }
                                if (d.dynamic_custom_asset != null) {
                                    if (typeof d.dynamic_custom_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_custom_asset: object expected");
                                    m.dynamic_custom_asset = $root.google.ads.googleads.v11.common.DynamicCustomAsset.fromObject(d.dynamic_custom_asset);
                                }
                                if (d.dynamic_hotels_and_rentals_asset != null) {
                                    if (typeof d.dynamic_hotels_and_rentals_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_hotels_and_rentals_asset: object expected");
                                    m.dynamic_hotels_and_rentals_asset = $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.fromObject(d.dynamic_hotels_and_rentals_asset);
                                }
                                if (d.dynamic_flights_asset != null) {
                                    if (typeof d.dynamic_flights_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_flights_asset: object expected");
                                    m.dynamic_flights_asset = $root.google.ads.googleads.v11.common.DynamicFlightsAsset.fromObject(d.dynamic_flights_asset);
                                }
                                if (d.discovery_carousel_card_asset != null) {
                                    if (typeof d.discovery_carousel_card_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.discovery_carousel_card_asset: object expected");
                                    m.discovery_carousel_card_asset = $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset.fromObject(d.discovery_carousel_card_asset);
                                }
                                if (d.dynamic_travel_asset != null) {
                                    if (typeof d.dynamic_travel_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_travel_asset: object expected");
                                    m.dynamic_travel_asset = $root.google.ads.googleads.v11.common.DynamicTravelAsset.fromObject(d.dynamic_travel_asset);
                                }
                                if (d.dynamic_local_asset != null) {
                                    if (typeof d.dynamic_local_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_local_asset: object expected");
                                    m.dynamic_local_asset = $root.google.ads.googleads.v11.common.DynamicLocalAsset.fromObject(d.dynamic_local_asset);
                                }
                                if (d.dynamic_jobs_asset != null) {
                                    if (typeof d.dynamic_jobs_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Asset.dynamic_jobs_asset: object expected");
                                    m.dynamic_jobs_asset = $root.google.ads.googleads.v11.common.DynamicJobsAsset.fromObject(d.dynamic_jobs_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an Asset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @static
                             * @param {google.ads.googleads.v11.resources.Asset} m Asset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Asset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                    d.url_custom_parameters = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.policy_summary = null;
                                    d.source = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetTypeEnum.AssetType[m.type] : m.type;
                                }
                                if (m.youtube_video_asset != null && m.hasOwnProperty("youtube_video_asset")) {
                                    d.youtube_video_asset = $root.google.ads.googleads.v11.common.YoutubeVideoAsset.toObject(m.youtube_video_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "youtube_video_asset";
                                }
                                if (m.media_bundle_asset != null && m.hasOwnProperty("media_bundle_asset")) {
                                    d.media_bundle_asset = $root.google.ads.googleads.v11.common.MediaBundleAsset.toObject(m.media_bundle_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "media_bundle_asset";
                                }
                                if (m.image_asset != null && m.hasOwnProperty("image_asset")) {
                                    d.image_asset = $root.google.ads.googleads.v11.common.ImageAsset.toObject(m.image_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "image_asset";
                                }
                                if (m.text_asset != null && m.hasOwnProperty("text_asset")) {
                                    d.text_asset = $root.google.ads.googleads.v11.common.TextAsset.toObject(m.text_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "text_asset";
                                }
                                if (m.lead_form_asset != null && m.hasOwnProperty("lead_form_asset")) {
                                    d.lead_form_asset = $root.google.ads.googleads.v11.common.LeadFormAsset.toObject(m.lead_form_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "lead_form_asset";
                                }
                                if (m.book_on_google_asset != null && m.hasOwnProperty("book_on_google_asset")) {
                                    d.book_on_google_asset = $root.google.ads.googleads.v11.common.BookOnGoogleAsset.toObject(m.book_on_google_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "book_on_google_asset";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    d.policy_summary = $root.google.ads.googleads.v11.resources.AssetPolicySummary.toObject(m.policy_summary, o);
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.promotion_asset != null && m.hasOwnProperty("promotion_asset")) {
                                    d.promotion_asset = $root.google.ads.googleads.v11.common.PromotionAsset.toObject(m.promotion_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "promotion_asset";
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                if (m.callout_asset != null && m.hasOwnProperty("callout_asset")) {
                                    d.callout_asset = $root.google.ads.googleads.v11.common.CalloutAsset.toObject(m.callout_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "callout_asset";
                                }
                                if (m.structured_snippet_asset != null && m.hasOwnProperty("structured_snippet_asset")) {
                                    d.structured_snippet_asset = $root.google.ads.googleads.v11.common.StructuredSnippetAsset.toObject(m.structured_snippet_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "structured_snippet_asset";
                                }
                                if (m.sitelink_asset != null && m.hasOwnProperty("sitelink_asset")) {
                                    d.sitelink_asset = $root.google.ads.googleads.v11.common.SitelinkAsset.toObject(m.sitelink_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "sitelink_asset";
                                }
                                if (m.page_feed_asset != null && m.hasOwnProperty("page_feed_asset")) {
                                    d.page_feed_asset = $root.google.ads.googleads.v11.common.PageFeedAsset.toObject(m.page_feed_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "page_feed_asset";
                                }
                                if (m.dynamic_education_asset != null && m.hasOwnProperty("dynamic_education_asset")) {
                                    d.dynamic_education_asset = $root.google.ads.googleads.v11.common.DynamicEducationAsset.toObject(m.dynamic_education_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_education_asset";
                                }
                                if (m.mobile_app_asset != null && m.hasOwnProperty("mobile_app_asset")) {
                                    d.mobile_app_asset = $root.google.ads.googleads.v11.common.MobileAppAsset.toObject(m.mobile_app_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "mobile_app_asset";
                                }
                                if (m.hotel_callout_asset != null && m.hasOwnProperty("hotel_callout_asset")) {
                                    d.hotel_callout_asset = $root.google.ads.googleads.v11.common.HotelCalloutAsset.toObject(m.hotel_callout_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "hotel_callout_asset";
                                }
                                if (m.call_asset != null && m.hasOwnProperty("call_asset")) {
                                    d.call_asset = $root.google.ads.googleads.v11.common.CallAsset.toObject(m.call_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "call_asset";
                                }
                                if (m.price_asset != null && m.hasOwnProperty("price_asset")) {
                                    d.price_asset = $root.google.ads.googleads.v11.common.PriceAsset.toObject(m.price_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "price_asset";
                                }
                                if (m.call_to_action_asset != null && m.hasOwnProperty("call_to_action_asset")) {
                                    d.call_to_action_asset = $root.google.ads.googleads.v11.common.CallToActionAsset.toObject(m.call_to_action_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "call_to_action_asset";
                                }
                                if (m.dynamic_real_estate_asset != null && m.hasOwnProperty("dynamic_real_estate_asset")) {
                                    d.dynamic_real_estate_asset = $root.google.ads.googleads.v11.common.DynamicRealEstateAsset.toObject(m.dynamic_real_estate_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_real_estate_asset";
                                }
                                if (m.dynamic_custom_asset != null && m.hasOwnProperty("dynamic_custom_asset")) {
                                    d.dynamic_custom_asset = $root.google.ads.googleads.v11.common.DynamicCustomAsset.toObject(m.dynamic_custom_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_custom_asset";
                                }
                                if (m.dynamic_hotels_and_rentals_asset != null && m.hasOwnProperty("dynamic_hotels_and_rentals_asset")) {
                                    d.dynamic_hotels_and_rentals_asset = $root.google.ads.googleads.v11.common.DynamicHotelsAndRentalsAsset.toObject(m.dynamic_hotels_and_rentals_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_hotels_and_rentals_asset";
                                }
                                if (m.dynamic_flights_asset != null && m.hasOwnProperty("dynamic_flights_asset")) {
                                    d.dynamic_flights_asset = $root.google.ads.googleads.v11.common.DynamicFlightsAsset.toObject(m.dynamic_flights_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_flights_asset";
                                }
                                if (m.discovery_carousel_card_asset != null && m.hasOwnProperty("discovery_carousel_card_asset")) {
                                    d.discovery_carousel_card_asset = $root.google.ads.googleads.v11.common.DiscoveryCarouselCardAsset.toObject(m.discovery_carousel_card_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "discovery_carousel_card_asset";
                                }
                                if (m.dynamic_travel_asset != null && m.hasOwnProperty("dynamic_travel_asset")) {
                                    d.dynamic_travel_asset = $root.google.ads.googleads.v11.common.DynamicTravelAsset.toObject(m.dynamic_travel_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_travel_asset";
                                }
                                if (m.dynamic_local_asset != null && m.hasOwnProperty("dynamic_local_asset")) {
                                    d.dynamic_local_asset = $root.google.ads.googleads.v11.common.DynamicLocalAsset.toObject(m.dynamic_local_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_local_asset";
                                }
                                if (m.dynamic_jobs_asset != null && m.hasOwnProperty("dynamic_jobs_asset")) {
                                    d.dynamic_jobs_asset = $root.google.ads.googleads.v11.common.DynamicJobsAsset.toObject(m.dynamic_jobs_asset, o);
                                    if (o.oneofs)
                                        d.asset_data = "dynamic_jobs_asset";
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    d.source = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource[m.source] : m.source;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Asset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Asset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Asset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Asset;
                        })();
    
                        resources.AssetPolicySummary = (function() {
    
                            /**
                             * Properties of an AssetPolicySummary.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetPolicySummary
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>|null} [policy_topic_entries] AssetPolicySummary policy_topic_entries
                             * @property {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus|null} [review_status] AssetPolicySummary review_status
                             * @property {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus|null} [approval_status] AssetPolicySummary approval_status
                             */
    
                            /**
                             * Constructs a new AssetPolicySummary.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetPolicySummary.
                             * @implements IAssetPolicySummary
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetPolicySummary=} [p] Properties to set
                             */
                            function AssetPolicySummary(p) {
                                this.policy_topic_entries = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetPolicySummary policy_topic_entries.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>} policy_topic_entries
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @instance
                             */
                            AssetPolicySummary.prototype.policy_topic_entries = $util.emptyArray;
    
                            /**
                             * AssetPolicySummary review_status.
                             * @member {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus} review_status
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @instance
                             */
                            AssetPolicySummary.prototype.review_status = 0;
    
                            /**
                             * AssetPolicySummary approval_status.
                             * @member {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus} approval_status
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @instance
                             */
                            AssetPolicySummary.prototype.approval_status = 0;
    
                            /**
                             * Creates a new AssetPolicySummary instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetPolicySummary=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetPolicySummary} AssetPolicySummary instance
                             */
                            AssetPolicySummary.create = function create(properties) {
                                return new AssetPolicySummary(properties);
                            };
    
                            /**
                             * Encodes the specified AssetPolicySummary message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetPolicySummary.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetPolicySummary} m AssetPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetPolicySummary.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.policy_topic_entries != null && m.policy_topic_entries.length) {
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEntry.encode(m.policy_topic_entries[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.review_status != null && Object.hasOwnProperty.call(m, "review_status"))
                                    w.uint32(16).int32(m.review_status);
                                if (m.approval_status != null && Object.hasOwnProperty.call(m, "approval_status"))
                                    w.uint32(24).int32(m.approval_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetPolicySummary message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetPolicySummary.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetPolicySummary} message AssetPolicySummary message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetPolicySummary.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetPolicySummary message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetPolicySummary} AssetPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetPolicySummary.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetPolicySummary();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.policy_topic_entries && m.policy_topic_entries.length))
                                            m.policy_topic_entries = [];
                                        m.policy_topic_entries.push($root.google.ads.googleads.v11.common.PolicyTopicEntry.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.review_status = r.int32();
                                        break;
                                    case 3:
                                        m.approval_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetPolicySummary message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetPolicySummary} AssetPolicySummary
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetPolicySummary.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetPolicySummary message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetPolicySummary.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.policy_topic_entries != null && m.hasOwnProperty("policy_topic_entries")) {
                                    if (!Array.isArray(m.policy_topic_entries))
                                        return "policy_topic_entries: array expected";
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEntry.verify(m.policy_topic_entries[i]);
                                            if (e)
                                                return "policy_topic_entries." + e;
                                        }
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    switch (m.review_status) {
                                    default:
                                        return "review_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    switch (m.approval_status) {
                                    default:
                                        return "approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetPolicySummary message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetPolicySummary} AssetPolicySummary
                             */
                            AssetPolicySummary.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetPolicySummary)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetPolicySummary();
                                if (d.policy_topic_entries) {
                                    if (!Array.isArray(d.policy_topic_entries))
                                        throw TypeError(".google.ads.googleads.v11.resources.AssetPolicySummary.policy_topic_entries: array expected");
                                    m.policy_topic_entries = [];
                                    for (var i = 0; i < d.policy_topic_entries.length; ++i) {
                                        if (typeof d.policy_topic_entries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.AssetPolicySummary.policy_topic_entries: object expected");
                                        m.policy_topic_entries[i] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.fromObject(d.policy_topic_entries[i]);
                                    }
                                }
                                switch (d.review_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.review_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.review_status = 1;
                                    break;
                                case "REVIEW_IN_PROGRESS":
                                case 2:
                                    m.review_status = 2;
                                    break;
                                case "REVIEWED":
                                case 3:
                                    m.review_status = 3;
                                    break;
                                case "UNDER_APPEAL":
                                case 4:
                                    m.review_status = 4;
                                    break;
                                case "ELIGIBLE_MAY_SERVE":
                                case 5:
                                    m.review_status = 5;
                                    break;
                                }
                                switch (d.approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approval_status = 1;
                                    break;
                                case "DISAPPROVED":
                                case 2:
                                    m.approval_status = 2;
                                    break;
                                case "APPROVED_LIMITED":
                                case 3:
                                    m.approval_status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.approval_status = 4;
                                    break;
                                case "AREA_OF_INTEREST_ONLY":
                                case 5:
                                    m.approval_status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetPolicySummary message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetPolicySummary} m AssetPolicySummary
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetPolicySummary.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.policy_topic_entries = [];
                                }
                                if (o.defaults) {
                                    d.review_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.policy_topic_entries && m.policy_topic_entries.length) {
                                    d.policy_topic_entries = [];
                                    for (var j = 0; j < m.policy_topic_entries.length; ++j) {
                                        d.policy_topic_entries[j] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.toObject(m.policy_topic_entries[j], o);
                                    }
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    d.review_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus[m.review_status] : m.review_status;
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    d.approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus[m.approval_status] : m.approval_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetPolicySummary to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetPolicySummary
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetPolicySummary.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetPolicySummary;
                        })();
    
                        resources.AssetFieldTypeView = (function() {
    
                            /**
                             * Properties of an AssetFieldTypeView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetFieldTypeView
                             * @property {string|null} [resource_name] AssetFieldTypeView resource_name
                             * @property {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType|null} [field_type] AssetFieldTypeView field_type
                             */
    
                            /**
                             * Constructs a new AssetFieldTypeView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetFieldTypeView.
                             * @implements IAssetFieldTypeView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetFieldTypeView=} [p] Properties to set
                             */
                            function AssetFieldTypeView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetFieldTypeView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @instance
                             */
                            AssetFieldTypeView.prototype.resource_name = "";
    
                            /**
                             * AssetFieldTypeView field_type.
                             * @member {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType} field_type
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @instance
                             */
                            AssetFieldTypeView.prototype.field_type = 0;
    
                            /**
                             * Creates a new AssetFieldTypeView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetFieldTypeView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetFieldTypeView} AssetFieldTypeView instance
                             */
                            AssetFieldTypeView.create = function create(properties) {
                                return new AssetFieldTypeView(properties);
                            };
    
                            /**
                             * Encodes the specified AssetFieldTypeView message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetFieldTypeView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetFieldTypeView} m AssetFieldTypeView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetFieldTypeView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.field_type != null && Object.hasOwnProperty.call(m, "field_type"))
                                    w.uint32(24).int32(m.field_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetFieldTypeView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetFieldTypeView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetFieldTypeView} message AssetFieldTypeView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetFieldTypeView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetFieldTypeView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetFieldTypeView} AssetFieldTypeView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetFieldTypeView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetFieldTypeView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.field_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetFieldTypeView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetFieldTypeView} AssetFieldTypeView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetFieldTypeView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetFieldTypeView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetFieldTypeView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    switch (m.field_type) {
                                    default:
                                        return "field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 24:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 25:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetFieldTypeView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetFieldTypeView} AssetFieldTypeView
                             */
                            AssetFieldTypeView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetFieldTypeView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetFieldTypeView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_type = 1;
                                    break;
                                case "HEADLINE":
                                case 2:
                                    m.field_type = 2;
                                    break;
                                case "DESCRIPTION":
                                case 3:
                                    m.field_type = 3;
                                    break;
                                case "MANDATORY_AD_TEXT":
                                case 4:
                                    m.field_type = 4;
                                    break;
                                case "MARKETING_IMAGE":
                                case 5:
                                    m.field_type = 5;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 6:
                                    m.field_type = 6;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 7:
                                    m.field_type = 7;
                                    break;
                                case "BOOK_ON_GOOGLE":
                                case 8:
                                    m.field_type = 8;
                                    break;
                                case "LEAD_FORM":
                                case 9:
                                    m.field_type = 9;
                                    break;
                                case "PROMOTION":
                                case 10:
                                    m.field_type = 10;
                                    break;
                                case "CALLOUT":
                                case 11:
                                    m.field_type = 11;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 12:
                                    m.field_type = 12;
                                    break;
                                case "SITELINK":
                                case 13:
                                    m.field_type = 13;
                                    break;
                                case "MOBILE_APP":
                                case 14:
                                    m.field_type = 14;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.field_type = 15;
                                    break;
                                case "CALL":
                                case 16:
                                    m.field_type = 16;
                                    break;
                                case "PRICE":
                                case 24:
                                    m.field_type = 24;
                                    break;
                                case "LONG_HEADLINE":
                                case 17:
                                    m.field_type = 17;
                                    break;
                                case "BUSINESS_NAME":
                                case 18:
                                    m.field_type = 18;
                                    break;
                                case "SQUARE_MARKETING_IMAGE":
                                case 19:
                                    m.field_type = 19;
                                    break;
                                case "PORTRAIT_MARKETING_IMAGE":
                                case 20:
                                    m.field_type = 20;
                                    break;
                                case "LOGO":
                                case 21:
                                    m.field_type = 21;
                                    break;
                                case "LANDSCAPE_LOGO":
                                case 22:
                                    m.field_type = 22;
                                    break;
                                case "VIDEO":
                                case 23:
                                    m.field_type = 23;
                                    break;
                                case "CALL_TO_ACTION_SELECTION":
                                case 25:
                                    m.field_type = 25;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetFieldTypeView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetFieldTypeView} m AssetFieldTypeView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetFieldTypeView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    d.field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.field_type] : m.field_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetFieldTypeView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetFieldTypeView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetFieldTypeView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetFieldTypeView;
                        })();
    
                        resources.AssetGroup = (function() {
    
                            /**
                             * Properties of an AssetGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetGroup
                             * @property {string|null} [resource_name] AssetGroup resource_name
                             * @property {number|null} [id] AssetGroup id
                             * @property {string|null} [campaign] AssetGroup campaign
                             * @property {string|null} [name] AssetGroup name
                             * @property {Array.<string>|null} [final_urls] AssetGroup final_urls
                             * @property {Array.<string>|null} [final_mobile_urls] AssetGroup final_mobile_urls
                             * @property {google.ads.googleads.v11.enums.AssetGroupStatusEnum.AssetGroupStatus|null} [status] AssetGroup status
                             * @property {string|null} [path1] AssetGroup path1
                             * @property {string|null} [path2] AssetGroup path2
                             */
    
                            /**
                             * Constructs a new AssetGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetGroup.
                             * @implements IAssetGroup
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetGroup=} [p] Properties to set
                             */
                            function AssetGroup(p) {
                                this.final_urls = [];
                                this.final_mobile_urls = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroup resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.resource_name = "";
    
                            /**
                             * AssetGroup id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AssetGroup campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.campaign = "";
    
                            /**
                             * AssetGroup name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.name = "";
    
                            /**
                             * AssetGroup final_urls.
                             * @member {Array.<string>} final_urls
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.final_urls = $util.emptyArray;
    
                            /**
                             * AssetGroup final_mobile_urls.
                             * @member {Array.<string>} final_mobile_urls
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.final_mobile_urls = $util.emptyArray;
    
                            /**
                             * AssetGroup status.
                             * @member {google.ads.googleads.v11.enums.AssetGroupStatusEnum.AssetGroupStatus} status
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.status = 0;
    
                            /**
                             * AssetGroup path1.
                             * @member {string} path1
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.path1 = "";
    
                            /**
                             * AssetGroup path2.
                             * @member {string} path2
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             */
                            AssetGroup.prototype.path2 = "";
    
                            /**
                             * Creates a new AssetGroup instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroup=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetGroup} AssetGroup instance
                             */
                            AssetGroup.create = function create(properties) {
                                return new AssetGroup(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroup message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroup.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroup} m AssetGroup message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroup.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.final_urls != null && m.final_urls.length) {
                                    for (var i = 0; i < m.final_urls.length; ++i)
                                        w.uint32(34).string(m.final_urls[i]);
                                }
                                if (m.final_mobile_urls != null && m.final_mobile_urls.length) {
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i)
                                        w.uint32(42).string(m.final_mobile_urls[i]);
                                }
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(48).int32(m.status);
                                if (m.path1 != null && Object.hasOwnProperty.call(m, "path1"))
                                    w.uint32(58).string(m.path1);
                                if (m.path2 != null && Object.hasOwnProperty.call(m, "path2"))
                                    w.uint32(66).string(m.path2);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(72).int64(m.id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroup message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroup.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroup} message AssetGroup message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroup.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroup message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetGroup} AssetGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroup.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetGroup();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.id = r.int64();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        if (!(m.final_urls && m.final_urls.length))
                                            m.final_urls = [];
                                        m.final_urls.push(r.string());
                                        break;
                                    case 5:
                                        if (!(m.final_mobile_urls && m.final_mobile_urls.length))
                                            m.final_mobile_urls = [];
                                        m.final_mobile_urls.push(r.string());
                                        break;
                                    case 6:
                                        m.status = r.int32();
                                        break;
                                    case 7:
                                        m.path1 = r.string();
                                        break;
                                    case 8:
                                        m.path2 = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroup message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetGroup} AssetGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroup.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroup message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroup.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.final_urls != null && m.hasOwnProperty("final_urls")) {
                                    if (!Array.isArray(m.final_urls))
                                        return "final_urls: array expected";
                                    for (var i = 0; i < m.final_urls.length; ++i) {
                                        if (!$util.isString(m.final_urls[i]))
                                            return "final_urls: string[] expected";
                                    }
                                }
                                if (m.final_mobile_urls != null && m.hasOwnProperty("final_mobile_urls")) {
                                    if (!Array.isArray(m.final_mobile_urls))
                                        return "final_mobile_urls: array expected";
                                    for (var i = 0; i < m.final_mobile_urls.length; ++i) {
                                        if (!$util.isString(m.final_mobile_urls[i]))
                                            return "final_mobile_urls: string[] expected";
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    if (!$util.isString(m.path1))
                                        return "path1: string expected";
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    if (!$util.isString(m.path2))
                                        return "path2: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroup message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetGroup} AssetGroup
                             */
                            AssetGroup.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetGroup)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetGroup();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.final_urls) {
                                    if (!Array.isArray(d.final_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.AssetGroup.final_urls: array expected");
                                    m.final_urls = [];
                                    for (var i = 0; i < d.final_urls.length; ++i) {
                                        m.final_urls[i] = String(d.final_urls[i]);
                                    }
                                }
                                if (d.final_mobile_urls) {
                                    if (!Array.isArray(d.final_mobile_urls))
                                        throw TypeError(".google.ads.googleads.v11.resources.AssetGroup.final_mobile_urls: array expected");
                                    m.final_mobile_urls = [];
                                    for (var i = 0; i < d.final_mobile_urls.length; ++i) {
                                        m.final_mobile_urls[i] = String(d.final_mobile_urls[i]);
                                    }
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "PAUSED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.path1 != null) {
                                    m.path1 = String(d.path1);
                                }
                                if (d.path2 != null) {
                                    m.path2 = String(d.path2);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroup message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetGroup} m AssetGroup
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroup.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.final_urls = [];
                                    d.final_mobile_urls = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign = "";
                                    d.name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.path1 = "";
                                    d.path2 = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.final_urls && m.final_urls.length) {
                                    d.final_urls = [];
                                    for (var j = 0; j < m.final_urls.length; ++j) {
                                        d.final_urls[j] = m.final_urls[j];
                                    }
                                }
                                if (m.final_mobile_urls && m.final_mobile_urls.length) {
                                    d.final_mobile_urls = [];
                                    for (var j = 0; j < m.final_mobile_urls.length; ++j) {
                                        d.final_mobile_urls[j] = m.final_mobile_urls[j];
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetGroupStatusEnum.AssetGroupStatus[m.status] : m.status;
                                }
                                if (m.path1 != null && m.hasOwnProperty("path1")) {
                                    d.path1 = m.path1;
                                }
                                if (m.path2 != null && m.hasOwnProperty("path2")) {
                                    d.path2 = m.path2;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroup to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetGroup
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroup.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroup;
                        })();
    
                        resources.AssetGroupAsset = (function() {
    
                            /**
                             * Properties of an AssetGroupAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetGroupAsset
                             * @property {string|null} [resource_name] AssetGroupAsset resource_name
                             * @property {string|null} [asset_group] AssetGroupAsset asset_group
                             * @property {string|null} [asset] AssetGroupAsset asset
                             * @property {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType|null} [field_type] AssetGroupAsset field_type
                             * @property {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus|null} [status] AssetGroupAsset status
                             * @property {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel|null} [performance_label] AssetGroupAsset performance_label
                             * @property {google.ads.googleads.v11.common.IPolicySummary|null} [policy_summary] AssetGroupAsset policy_summary
                             */
    
                            /**
                             * Constructs a new AssetGroupAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetGroupAsset.
                             * @implements IAssetGroupAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetGroupAsset=} [p] Properties to set
                             */
                            function AssetGroupAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupAsset resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             */
                            AssetGroupAsset.prototype.resource_name = "";
    
                            /**
                             * AssetGroupAsset asset_group.
                             * @member {string} asset_group
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             */
                            AssetGroupAsset.prototype.asset_group = "";
    
                            /**
                             * AssetGroupAsset asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             */
                            AssetGroupAsset.prototype.asset = "";
    
                            /**
                             * AssetGroupAsset field_type.
                             * @member {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType} field_type
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             */
                            AssetGroupAsset.prototype.field_type = 0;
    
                            /**
                             * AssetGroupAsset status.
                             * @member {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             */
                            AssetGroupAsset.prototype.status = 0;
    
                            /**
                             * AssetGroupAsset performance_label.
                             * @member {google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel} performance_label
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             */
                            AssetGroupAsset.prototype.performance_label = 0;
    
                            /**
                             * AssetGroupAsset policy_summary.
                             * @member {google.ads.googleads.v11.common.IPolicySummary|null|undefined} policy_summary
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             */
                            AssetGroupAsset.prototype.policy_summary = null;
    
                            /**
                             * Creates a new AssetGroupAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetGroupAsset} AssetGroupAsset instance
                             */
                            AssetGroupAsset.create = function create(properties) {
                                return new AssetGroupAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupAsset message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupAsset} m AssetGroupAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_group != null && Object.hasOwnProperty.call(m, "asset_group"))
                                    w.uint32(18).string(m.asset_group);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(26).string(m.asset);
                                if (m.field_type != null && Object.hasOwnProperty.call(m, "field_type"))
                                    w.uint32(32).int32(m.field_type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.performance_label != null && Object.hasOwnProperty.call(m, "performance_label"))
                                    w.uint32(48).int32(m.performance_label);
                                if (m.policy_summary != null && Object.hasOwnProperty.call(m, "policy_summary"))
                                    $root.google.ads.googleads.v11.common.PolicySummary.encode(m.policy_summary, w.uint32(58).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupAsset} message AssetGroupAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetGroupAsset} AssetGroupAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetGroupAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_group = r.string();
                                        break;
                                    case 3:
                                        m.asset = r.string();
                                        break;
                                    case 4:
                                        m.field_type = r.int32();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 6:
                                        m.performance_label = r.int32();
                                        break;
                                    case 7:
                                        m.policy_summary = $root.google.ads.googleads.v11.common.PolicySummary.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetGroupAsset} AssetGroupAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    if (!$util.isString(m.asset_group))
                                        return "asset_group: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    switch (m.field_type) {
                                    default:
                                        return "field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 24:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.performance_label != null && m.hasOwnProperty("performance_label")) {
                                    switch (m.performance_label) {
                                    default:
                                        return "performance_label: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicySummary.verify(m.policy_summary);
                                        if (e)
                                            return "policy_summary." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetGroupAsset} AssetGroupAsset
                             */
                            AssetGroupAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetGroupAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetGroupAsset();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_group != null) {
                                    m.asset_group = String(d.asset_group);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                switch (d.field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_type = 1;
                                    break;
                                case "HEADLINE":
                                case 2:
                                    m.field_type = 2;
                                    break;
                                case "DESCRIPTION":
                                case 3:
                                    m.field_type = 3;
                                    break;
                                case "MANDATORY_AD_TEXT":
                                case 4:
                                    m.field_type = 4;
                                    break;
                                case "MARKETING_IMAGE":
                                case 5:
                                    m.field_type = 5;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 6:
                                    m.field_type = 6;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 7:
                                    m.field_type = 7;
                                    break;
                                case "BOOK_ON_GOOGLE":
                                case 8:
                                    m.field_type = 8;
                                    break;
                                case "LEAD_FORM":
                                case 9:
                                    m.field_type = 9;
                                    break;
                                case "PROMOTION":
                                case 10:
                                    m.field_type = 10;
                                    break;
                                case "CALLOUT":
                                case 11:
                                    m.field_type = 11;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 12:
                                    m.field_type = 12;
                                    break;
                                case "SITELINK":
                                case 13:
                                    m.field_type = 13;
                                    break;
                                case "MOBILE_APP":
                                case 14:
                                    m.field_type = 14;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.field_type = 15;
                                    break;
                                case "CALL":
                                case 16:
                                    m.field_type = 16;
                                    break;
                                case "PRICE":
                                case 24:
                                    m.field_type = 24;
                                    break;
                                case "LONG_HEADLINE":
                                case 17:
                                    m.field_type = 17;
                                    break;
                                case "BUSINESS_NAME":
                                case 18:
                                    m.field_type = 18;
                                    break;
                                case "SQUARE_MARKETING_IMAGE":
                                case 19:
                                    m.field_type = 19;
                                    break;
                                case "PORTRAIT_MARKETING_IMAGE":
                                case 20:
                                    m.field_type = 20;
                                    break;
                                case "LOGO":
                                case 21:
                                    m.field_type = 21;
                                    break;
                                case "LANDSCAPE_LOGO":
                                case 22:
                                    m.field_type = 22;
                                    break;
                                case "VIDEO":
                                case 23:
                                    m.field_type = 23;
                                    break;
                                case "CALL_TO_ACTION_SELECTION":
                                case 25:
                                    m.field_type = 25;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PAUSED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                switch (d.performance_label) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.performance_label = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.performance_label = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.performance_label = 2;
                                    break;
                                case "LEARNING":
                                case 3:
                                    m.performance_label = 3;
                                    break;
                                case "LOW":
                                case 4:
                                    m.performance_label = 4;
                                    break;
                                case "GOOD":
                                case 5:
                                    m.performance_label = 5;
                                    break;
                                case "BEST":
                                case 6:
                                    m.performance_label = 6;
                                    break;
                                }
                                if (d.policy_summary != null) {
                                    if (typeof d.policy_summary !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AssetGroupAsset.policy_summary: object expected");
                                    m.policy_summary = $root.google.ads.googleads.v11.common.PolicySummary.fromObject(d.policy_summary);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetGroupAsset} m AssetGroupAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_group = "";
                                    d.asset = "";
                                    d.field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.performance_label = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.policy_summary = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    d.asset_group = m.asset_group;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    d.field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.field_type] : m.field_type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus[m.status] : m.status;
                                }
                                if (m.performance_label != null && m.hasOwnProperty("performance_label")) {
                                    d.performance_label = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetPerformanceLabelEnum.AssetPerformanceLabel[m.performance_label] : m.performance_label;
                                }
                                if (m.policy_summary != null && m.hasOwnProperty("policy_summary")) {
                                    d.policy_summary = $root.google.ads.googleads.v11.common.PolicySummary.toObject(m.policy_summary, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetGroupAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupAsset;
                        })();
    
                        resources.AssetGroupListingGroupFilter = (function() {
    
                            /**
                             * Properties of an AssetGroupListingGroupFilter.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetGroupListingGroupFilter
                             * @property {string|null} [resource_name] AssetGroupListingGroupFilter resource_name
                             * @property {string|null} [asset_group] AssetGroupListingGroupFilter asset_group
                             * @property {number|null} [id] AssetGroupListingGroupFilter id
                             * @property {google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum.ListingGroupFilterType|null} [type] AssetGroupListingGroupFilter type
                             * @property {google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum.ListingGroupFilterVertical|null} [vertical] AssetGroupListingGroupFilter vertical
                             * @property {google.ads.googleads.v11.resources.IListingGroupFilterDimension|null} [case_value] AssetGroupListingGroupFilter case_value
                             * @property {string|null} [parent_listing_group_filter] AssetGroupListingGroupFilter parent_listing_group_filter
                             */
    
                            /**
                             * Constructs a new AssetGroupListingGroupFilter.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetGroupListingGroupFilter.
                             * @implements IAssetGroupListingGroupFilter
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter=} [p] Properties to set
                             */
                            function AssetGroupListingGroupFilter(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupListingGroupFilter resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             */
                            AssetGroupListingGroupFilter.prototype.resource_name = "";
    
                            /**
                             * AssetGroupListingGroupFilter asset_group.
                             * @member {string} asset_group
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             */
                            AssetGroupListingGroupFilter.prototype.asset_group = "";
    
                            /**
                             * AssetGroupListingGroupFilter id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             */
                            AssetGroupListingGroupFilter.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AssetGroupListingGroupFilter type.
                             * @member {google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum.ListingGroupFilterType} type
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             */
                            AssetGroupListingGroupFilter.prototype.type = 0;
    
                            /**
                             * AssetGroupListingGroupFilter vertical.
                             * @member {google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum.ListingGroupFilterVertical} vertical
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             */
                            AssetGroupListingGroupFilter.prototype.vertical = 0;
    
                            /**
                             * AssetGroupListingGroupFilter case_value.
                             * @member {google.ads.googleads.v11.resources.IListingGroupFilterDimension|null|undefined} case_value
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             */
                            AssetGroupListingGroupFilter.prototype.case_value = null;
    
                            /**
                             * AssetGroupListingGroupFilter parent_listing_group_filter.
                             * @member {string} parent_listing_group_filter
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             */
                            AssetGroupListingGroupFilter.prototype.parent_listing_group_filter = "";
    
                            /**
                             * Creates a new AssetGroupListingGroupFilter instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetGroupListingGroupFilter} AssetGroupListingGroupFilter instance
                             */
                            AssetGroupListingGroupFilter.create = function create(properties) {
                                return new AssetGroupListingGroupFilter(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupListingGroupFilter message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter} m AssetGroupListingGroupFilter message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupListingGroupFilter.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_group != null && Object.hasOwnProperty.call(m, "asset_group"))
                                    w.uint32(18).string(m.asset_group);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(24).int64(m.id);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.vertical != null && Object.hasOwnProperty.call(m, "vertical"))
                                    w.uint32(40).int32(m.vertical);
                                if (m.case_value != null && Object.hasOwnProperty.call(m, "case_value"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.encode(m.case_value, w.uint32(50).fork()).ldelim();
                                if (m.parent_listing_group_filter != null && Object.hasOwnProperty.call(m, "parent_listing_group_filter"))
                                    w.uint32(58).string(m.parent_listing_group_filter);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupListingGroupFilter message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter} message AssetGroupListingGroupFilter message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupListingGroupFilter.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupListingGroupFilter message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetGroupListingGroupFilter} AssetGroupListingGroupFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupListingGroupFilter.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_group = r.string();
                                        break;
                                    case 3:
                                        m.id = r.int64();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 5:
                                        m.vertical = r.int32();
                                        break;
                                    case 6:
                                        m.case_value = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.parent_listing_group_filter = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupListingGroupFilter message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetGroupListingGroupFilter} AssetGroupListingGroupFilter
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupListingGroupFilter.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupListingGroupFilter message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupListingGroupFilter.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    if (!$util.isString(m.asset_group))
                                        return "asset_group: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.vertical != null && m.hasOwnProperty("vertical")) {
                                    switch (m.vertical) {
                                    default:
                                        return "vertical: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.case_value != null && m.hasOwnProperty("case_value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.verify(m.case_value);
                                        if (e)
                                            return "case_value." + e;
                                    }
                                }
                                if (m.parent_listing_group_filter != null && m.hasOwnProperty("parent_listing_group_filter")) {
                                    if (!$util.isString(m.parent_listing_group_filter))
                                        return "parent_listing_group_filter: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupListingGroupFilter message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetGroupListingGroupFilter} AssetGroupListingGroupFilter
                             */
                            AssetGroupListingGroupFilter.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_group != null) {
                                    m.asset_group = String(d.asset_group);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "SUBDIVISION":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "UNIT_INCLUDED":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "UNIT_EXCLUDED":
                                case 4:
                                    m.type = 4;
                                    break;
                                }
                                switch (d.vertical) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.vertical = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.vertical = 1;
                                    break;
                                case "SHOPPING":
                                case 2:
                                    m.vertical = 2;
                                    break;
                                }
                                if (d.case_value != null) {
                                    if (typeof d.case_value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.case_value: object expected");
                                    m.case_value = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.fromObject(d.case_value);
                                }
                                if (d.parent_listing_group_filter != null) {
                                    m.parent_listing_group_filter = String(d.parent_listing_group_filter);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupListingGroupFilter message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetGroupListingGroupFilter} m AssetGroupListingGroupFilter
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupListingGroupFilter.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_group = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.vertical = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.case_value = null;
                                    d.parent_listing_group_filter = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    d.asset_group = m.asset_group;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupFilterTypeEnum.ListingGroupFilterType[m.type] : m.type;
                                }
                                if (m.vertical != null && m.hasOwnProperty("vertical")) {
                                    d.vertical = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupFilterVerticalEnum.ListingGroupFilterVertical[m.vertical] : m.vertical;
                                }
                                if (m.case_value != null && m.hasOwnProperty("case_value")) {
                                    d.case_value = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.toObject(m.case_value, o);
                                }
                                if (m.parent_listing_group_filter != null && m.hasOwnProperty("parent_listing_group_filter")) {
                                    d.parent_listing_group_filter = m.parent_listing_group_filter;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupListingGroupFilter to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetGroupListingGroupFilter
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupListingGroupFilter.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupListingGroupFilter;
                        })();
    
                        resources.ListingGroupFilterDimension = (function() {
    
                            /**
                             * Properties of a ListingGroupFilterDimension.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IListingGroupFilterDimension
                             * @property {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBiddingCategory|null} [product_bidding_category] ListingGroupFilterDimension product_bidding_category
                             * @property {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBrand|null} [product_brand] ListingGroupFilterDimension product_brand
                             * @property {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductChannel|null} [product_channel] ListingGroupFilterDimension product_channel
                             * @property {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCondition|null} [product_condition] ListingGroupFilterDimension product_condition
                             * @property {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCustomAttribute|null} [product_custom_attribute] ListingGroupFilterDimension product_custom_attribute
                             * @property {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductItemId|null} [product_item_id] ListingGroupFilterDimension product_item_id
                             * @property {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductType|null} [product_type] ListingGroupFilterDimension product_type
                             */
    
                            /**
                             * Constructs a new ListingGroupFilterDimension.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ListingGroupFilterDimension.
                             * @implements IListingGroupFilterDimension
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IListingGroupFilterDimension=} [p] Properties to set
                             */
                            function ListingGroupFilterDimension(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListingGroupFilterDimension product_bidding_category.
                             * @member {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBiddingCategory|null|undefined} product_bidding_category
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            ListingGroupFilterDimension.prototype.product_bidding_category = null;
    
                            /**
                             * ListingGroupFilterDimension product_brand.
                             * @member {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBrand|null|undefined} product_brand
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            ListingGroupFilterDimension.prototype.product_brand = null;
    
                            /**
                             * ListingGroupFilterDimension product_channel.
                             * @member {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductChannel|null|undefined} product_channel
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            ListingGroupFilterDimension.prototype.product_channel = null;
    
                            /**
                             * ListingGroupFilterDimension product_condition.
                             * @member {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCondition|null|undefined} product_condition
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            ListingGroupFilterDimension.prototype.product_condition = null;
    
                            /**
                             * ListingGroupFilterDimension product_custom_attribute.
                             * @member {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCustomAttribute|null|undefined} product_custom_attribute
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            ListingGroupFilterDimension.prototype.product_custom_attribute = null;
    
                            /**
                             * ListingGroupFilterDimension product_item_id.
                             * @member {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductItemId|null|undefined} product_item_id
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            ListingGroupFilterDimension.prototype.product_item_id = null;
    
                            /**
                             * ListingGroupFilterDimension product_type.
                             * @member {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductType|null|undefined} product_type
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            ListingGroupFilterDimension.prototype.product_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ListingGroupFilterDimension dimension.
                             * @member {"product_bidding_category"|"product_brand"|"product_channel"|"product_condition"|"product_custom_attribute"|"product_item_id"|"product_type"|undefined} dimension
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             */
                            Object.defineProperty(ListingGroupFilterDimension.prototype, "dimension", {
                                get: $util.oneOfGetter($oneOfFields = ["product_bidding_category", "product_brand", "product_channel", "product_condition", "product_custom_attribute", "product_item_id", "product_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ListingGroupFilterDimension instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {google.ads.googleads.v11.resources.IListingGroupFilterDimension=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension} ListingGroupFilterDimension instance
                             */
                            ListingGroupFilterDimension.create = function create(properties) {
                                return new ListingGroupFilterDimension(properties);
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterDimension message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {google.ads.googleads.v11.resources.IListingGroupFilterDimension} m ListingGroupFilterDimension message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterDimension.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.product_bidding_category != null && Object.hasOwnProperty.call(m, "product_bidding_category"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory.encode(m.product_bidding_category, w.uint32(10).fork()).ldelim();
                                if (m.product_brand != null && Object.hasOwnProperty.call(m, "product_brand"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand.encode(m.product_brand, w.uint32(18).fork()).ldelim();
                                if (m.product_channel != null && Object.hasOwnProperty.call(m, "product_channel"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel.encode(m.product_channel, w.uint32(26).fork()).ldelim();
                                if (m.product_condition != null && Object.hasOwnProperty.call(m, "product_condition"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition.encode(m.product_condition, w.uint32(34).fork()).ldelim();
                                if (m.product_custom_attribute != null && Object.hasOwnProperty.call(m, "product_custom_attribute"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute.encode(m.product_custom_attribute, w.uint32(42).fork()).ldelim();
                                if (m.product_item_id != null && Object.hasOwnProperty.call(m, "product_item_id"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId.encode(m.product_item_id, w.uint32(50).fork()).ldelim();
                                if (m.product_type != null && Object.hasOwnProperty.call(m, "product_type"))
                                    $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType.encode(m.product_type, w.uint32(58).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListingGroupFilterDimension message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {google.ads.googleads.v11.resources.IListingGroupFilterDimension} message ListingGroupFilterDimension message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListingGroupFilterDimension.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListingGroupFilterDimension message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension} ListingGroupFilterDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterDimension.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.product_bidding_category = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.product_brand = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.product_channel = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.product_condition = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.product_custom_attribute = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.product_item_id = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.product_type = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListingGroupFilterDimension message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension} ListingGroupFilterDimension
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListingGroupFilterDimension.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListingGroupFilterDimension message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListingGroupFilterDimension.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.product_bidding_category != null && m.hasOwnProperty("product_bidding_category")) {
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory.verify(m.product_bidding_category);
                                        if (e)
                                            return "product_bidding_category." + e;
                                    }
                                }
                                if (m.product_brand != null && m.hasOwnProperty("product_brand")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand.verify(m.product_brand);
                                        if (e)
                                            return "product_brand." + e;
                                    }
                                }
                                if (m.product_channel != null && m.hasOwnProperty("product_channel")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel.verify(m.product_channel);
                                        if (e)
                                            return "product_channel." + e;
                                    }
                                }
                                if (m.product_condition != null && m.hasOwnProperty("product_condition")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition.verify(m.product_condition);
                                        if (e)
                                            return "product_condition." + e;
                                    }
                                }
                                if (m.product_custom_attribute != null && m.hasOwnProperty("product_custom_attribute")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute.verify(m.product_custom_attribute);
                                        if (e)
                                            return "product_custom_attribute." + e;
                                    }
                                }
                                if (m.product_item_id != null && m.hasOwnProperty("product_item_id")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId.verify(m.product_item_id);
                                        if (e)
                                            return "product_item_id." + e;
                                    }
                                }
                                if (m.product_type != null && m.hasOwnProperty("product_type")) {
                                    if (p.dimension === 1)
                                        return "dimension: multiple values";
                                    p.dimension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType.verify(m.product_type);
                                        if (e)
                                            return "product_type." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListingGroupFilterDimension message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension} ListingGroupFilterDimension
                             */
                            ListingGroupFilterDimension.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension();
                                if (d.product_bidding_category != null) {
                                    if (typeof d.product_bidding_category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ListingGroupFilterDimension.product_bidding_category: object expected");
                                    m.product_bidding_category = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory.fromObject(d.product_bidding_category);
                                }
                                if (d.product_brand != null) {
                                    if (typeof d.product_brand !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ListingGroupFilterDimension.product_brand: object expected");
                                    m.product_brand = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand.fromObject(d.product_brand);
                                }
                                if (d.product_channel != null) {
                                    if (typeof d.product_channel !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ListingGroupFilterDimension.product_channel: object expected");
                                    m.product_channel = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel.fromObject(d.product_channel);
                                }
                                if (d.product_condition != null) {
                                    if (typeof d.product_condition !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ListingGroupFilterDimension.product_condition: object expected");
                                    m.product_condition = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition.fromObject(d.product_condition);
                                }
                                if (d.product_custom_attribute != null) {
                                    if (typeof d.product_custom_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ListingGroupFilterDimension.product_custom_attribute: object expected");
                                    m.product_custom_attribute = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute.fromObject(d.product_custom_attribute);
                                }
                                if (d.product_item_id != null) {
                                    if (typeof d.product_item_id !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ListingGroupFilterDimension.product_item_id: object expected");
                                    m.product_item_id = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId.fromObject(d.product_item_id);
                                }
                                if (d.product_type != null) {
                                    if (typeof d.product_type !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ListingGroupFilterDimension.product_type: object expected");
                                    m.product_type = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType.fromObject(d.product_type);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListingGroupFilterDimension message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @static
                             * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension} m ListingGroupFilterDimension
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListingGroupFilterDimension.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.product_bidding_category != null && m.hasOwnProperty("product_bidding_category")) {
                                    d.product_bidding_category = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory.toObject(m.product_bidding_category, o);
                                    if (o.oneofs)
                                        d.dimension = "product_bidding_category";
                                }
                                if (m.product_brand != null && m.hasOwnProperty("product_brand")) {
                                    d.product_brand = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand.toObject(m.product_brand, o);
                                    if (o.oneofs)
                                        d.dimension = "product_brand";
                                }
                                if (m.product_channel != null && m.hasOwnProperty("product_channel")) {
                                    d.product_channel = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel.toObject(m.product_channel, o);
                                    if (o.oneofs)
                                        d.dimension = "product_channel";
                                }
                                if (m.product_condition != null && m.hasOwnProperty("product_condition")) {
                                    d.product_condition = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition.toObject(m.product_condition, o);
                                    if (o.oneofs)
                                        d.dimension = "product_condition";
                                }
                                if (m.product_custom_attribute != null && m.hasOwnProperty("product_custom_attribute")) {
                                    d.product_custom_attribute = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute.toObject(m.product_custom_attribute, o);
                                    if (o.oneofs)
                                        d.dimension = "product_custom_attribute";
                                }
                                if (m.product_item_id != null && m.hasOwnProperty("product_item_id")) {
                                    d.product_item_id = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId.toObject(m.product_item_id, o);
                                    if (o.oneofs)
                                        d.dimension = "product_item_id";
                                }
                                if (m.product_type != null && m.hasOwnProperty("product_type")) {
                                    d.product_type = $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType.toObject(m.product_type, o);
                                    if (o.oneofs)
                                        d.dimension = "product_type";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListingGroupFilterDimension to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListingGroupFilterDimension.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            ListingGroupFilterDimension.ProductBiddingCategory = (function() {
    
                                /**
                                 * Properties of a ProductBiddingCategory.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @interface IProductBiddingCategory
                                 * @property {number|null} [id] ProductBiddingCategory id
                                 * @property {google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum.ListingGroupFilterBiddingCategoryLevel|null} [level] ProductBiddingCategory level
                                 */
    
                                /**
                                 * Constructs a new ProductBiddingCategory.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @classdesc Represents a ProductBiddingCategory.
                                 * @implements IProductBiddingCategory
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBiddingCategory=} [p] Properties to set
                                 */
                                function ProductBiddingCategory(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ProductBiddingCategory id.
                                 * @member {number|null|undefined} id
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @instance
                                 */
                                ProductBiddingCategory.prototype.id = null;
    
                                /**
                                 * ProductBiddingCategory level.
                                 * @member {google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum.ListingGroupFilterBiddingCategoryLevel} level
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @instance
                                 */
                                ProductBiddingCategory.prototype.level = 0;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ProductBiddingCategory _id.
                                 * @member {"id"|undefined} _id
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @instance
                                 */
                                Object.defineProperty(ProductBiddingCategory.prototype, "_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ProductBiddingCategory instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBiddingCategory=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory} ProductBiddingCategory instance
                                 */
                                ProductBiddingCategory.create = function create(properties) {
                                    return new ProductBiddingCategory(properties);
                                };
    
                                /**
                                 * Encodes the specified ProductBiddingCategory message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBiddingCategory} m ProductBiddingCategory message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductBiddingCategory.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                        w.uint32(8).int64(m.id);
                                    if (m.level != null && Object.hasOwnProperty.call(m, "level"))
                                        w.uint32(16).int32(m.level);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ProductBiddingCategory message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBiddingCategory} message ProductBiddingCategory message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductBiddingCategory.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ProductBiddingCategory message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory} ProductBiddingCategory
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductBiddingCategory.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.id = r.int64();
                                            break;
                                        case 2:
                                            m.level = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ProductBiddingCategory message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory} ProductBiddingCategory
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductBiddingCategory.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ProductBiddingCategory message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ProductBiddingCategory.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.id != null && m.hasOwnProperty("id")) {
                                        p._id = 1;
                                        if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                            return "id: integer|Long expected";
                                    }
                                    if (m.level != null && m.hasOwnProperty("level")) {
                                        switch (m.level) {
                                        default:
                                            return "level: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ProductBiddingCategory message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory} ProductBiddingCategory
                                 */
                                ProductBiddingCategory.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory();
                                    if (d.id != null) {
                                        if ($util.Long)
                                            (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                        else if (typeof d.id === "string")
                                            m.id = parseInt(d.id, 10);
                                        else if (typeof d.id === "number")
                                            m.id = d.id;
                                        else if (typeof d.id === "object")
                                            m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                    }
                                    switch (d.level) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.level = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.level = 1;
                                        break;
                                    case "LEVEL1":
                                    case 2:
                                        m.level = 2;
                                        break;
                                    case "LEVEL2":
                                    case 3:
                                        m.level = 3;
                                        break;
                                    case "LEVEL3":
                                    case 4:
                                        m.level = 4;
                                        break;
                                    case "LEVEL4":
                                    case 5:
                                        m.level = 5;
                                        break;
                                    case "LEVEL5":
                                    case 6:
                                        m.level = 6;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ProductBiddingCategory message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory} m ProductBiddingCategory
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ProductBiddingCategory.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.level = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.id != null && m.hasOwnProperty("id")) {
                                        if (typeof m.id === "number")
                                            d.id = o.longs === String ? String(m.id) : m.id;
                                        else
                                            d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                        if (o.oneofs)
                                            d._id = "id";
                                    }
                                    if (m.level != null && m.hasOwnProperty("level")) {
                                        d.level = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupFilterBiddingCategoryLevelEnum.ListingGroupFilterBiddingCategoryLevel[m.level] : m.level;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ProductBiddingCategory to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBiddingCategory
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ProductBiddingCategory.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ProductBiddingCategory;
                            })();
    
                            ListingGroupFilterDimension.ProductBrand = (function() {
    
                                /**
                                 * Properties of a ProductBrand.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @interface IProductBrand
                                 * @property {string|null} [value] ProductBrand value
                                 */
    
                                /**
                                 * Constructs a new ProductBrand.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @classdesc Represents a ProductBrand.
                                 * @implements IProductBrand
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBrand=} [p] Properties to set
                                 */
                                function ProductBrand(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ProductBrand value.
                                 * @member {string|null|undefined} value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @instance
                                 */
                                ProductBrand.prototype.value = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ProductBrand _value.
                                 * @member {"value"|undefined} _value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @instance
                                 */
                                Object.defineProperty(ProductBrand.prototype, "_value", {
                                    get: $util.oneOfGetter($oneOfFields = ["value"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ProductBrand instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBrand=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand} ProductBrand instance
                                 */
                                ProductBrand.create = function create(properties) {
                                    return new ProductBrand(properties);
                                };
    
                                /**
                                 * Encodes the specified ProductBrand message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBrand} m ProductBrand message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductBrand.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                        w.uint32(10).string(m.value);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ProductBrand message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductBrand} message ProductBrand message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductBrand.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ProductBrand message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand} ProductBrand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductBrand.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.value = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ProductBrand message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand} ProductBrand
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductBrand.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ProductBrand message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ProductBrand.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        p._value = 1;
                                        if (!$util.isString(m.value))
                                            return "value: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ProductBrand message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand} ProductBrand
                                 */
                                ProductBrand.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand();
                                    if (d.value != null) {
                                        m.value = String(d.value);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ProductBrand message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand} m ProductBrand
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ProductBrand.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        d.value = m.value;
                                        if (o.oneofs)
                                            d._value = "value";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ProductBrand to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductBrand
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ProductBrand.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ProductBrand;
                            })();
    
                            ListingGroupFilterDimension.ProductChannel = (function() {
    
                                /**
                                 * Properties of a ProductChannel.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @interface IProductChannel
                                 * @property {google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel|null} [channel] ProductChannel channel
                                 */
    
                                /**
                                 * Constructs a new ProductChannel.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @classdesc Represents a ProductChannel.
                                 * @implements IProductChannel
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductChannel=} [p] Properties to set
                                 */
                                function ProductChannel(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ProductChannel channel.
                                 * @member {google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel} channel
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @instance
                                 */
                                ProductChannel.prototype.channel = 0;
    
                                /**
                                 * Creates a new ProductChannel instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductChannel=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel} ProductChannel instance
                                 */
                                ProductChannel.create = function create(properties) {
                                    return new ProductChannel(properties);
                                };
    
                                /**
                                 * Encodes the specified ProductChannel message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductChannel} m ProductChannel message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductChannel.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.channel != null && Object.hasOwnProperty.call(m, "channel"))
                                        w.uint32(8).int32(m.channel);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ProductChannel message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductChannel} message ProductChannel message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductChannel.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ProductChannel message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel} ProductChannel
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductChannel.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.channel = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ProductChannel message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel} ProductChannel
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductChannel.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ProductChannel message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ProductChannel.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.channel != null && m.hasOwnProperty("channel")) {
                                        switch (m.channel) {
                                        default:
                                            return "channel: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ProductChannel message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel} ProductChannel
                                 */
                                ProductChannel.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel();
                                    switch (d.channel) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.channel = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.channel = 1;
                                        break;
                                    case "ONLINE":
                                    case 2:
                                        m.channel = 2;
                                        break;
                                    case "LOCAL":
                                    case 3:
                                        m.channel = 3;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ProductChannel message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel} m ProductChannel
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ProductChannel.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.channel = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.channel != null && m.hasOwnProperty("channel")) {
                                        d.channel = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel[m.channel] : m.channel;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ProductChannel to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductChannel
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ProductChannel.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ProductChannel;
                            })();
    
                            ListingGroupFilterDimension.ProductCondition = (function() {
    
                                /**
                                 * Properties of a ProductCondition.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @interface IProductCondition
                                 * @property {google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition|null} [condition] ProductCondition condition
                                 */
    
                                /**
                                 * Constructs a new ProductCondition.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @classdesc Represents a ProductCondition.
                                 * @implements IProductCondition
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCondition=} [p] Properties to set
                                 */
                                function ProductCondition(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ProductCondition condition.
                                 * @member {google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition} condition
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @instance
                                 */
                                ProductCondition.prototype.condition = 0;
    
                                /**
                                 * Creates a new ProductCondition instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCondition=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition} ProductCondition instance
                                 */
                                ProductCondition.create = function create(properties) {
                                    return new ProductCondition(properties);
                                };
    
                                /**
                                 * Encodes the specified ProductCondition message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCondition} m ProductCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductCondition.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.condition != null && Object.hasOwnProperty.call(m, "condition"))
                                        w.uint32(8).int32(m.condition);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ProductCondition message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCondition} message ProductCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductCondition.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ProductCondition message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition} ProductCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductCondition.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.condition = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ProductCondition message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition} ProductCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductCondition.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ProductCondition message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ProductCondition.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.condition != null && m.hasOwnProperty("condition")) {
                                        switch (m.condition) {
                                        default:
                                            return "condition: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ProductCondition message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition} ProductCondition
                                 */
                                ProductCondition.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition();
                                    switch (d.condition) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.condition = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.condition = 1;
                                        break;
                                    case "NEW":
                                    case 2:
                                        m.condition = 2;
                                        break;
                                    case "REFURBISHED":
                                    case 3:
                                        m.condition = 3;
                                        break;
                                    case "USED":
                                    case 4:
                                        m.condition = 4;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ProductCondition message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition} m ProductCondition
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ProductCondition.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.condition = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.condition != null && m.hasOwnProperty("condition")) {
                                        d.condition = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition[m.condition] : m.condition;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ProductCondition to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCondition
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ProductCondition.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ProductCondition;
                            })();
    
                            ListingGroupFilterDimension.ProductCustomAttribute = (function() {
    
                                /**
                                 * Properties of a ProductCustomAttribute.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @interface IProductCustomAttribute
                                 * @property {string|null} [value] ProductCustomAttribute value
                                 * @property {google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex|null} [index] ProductCustomAttribute index
                                 */
    
                                /**
                                 * Constructs a new ProductCustomAttribute.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @classdesc Represents a ProductCustomAttribute.
                                 * @implements IProductCustomAttribute
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCustomAttribute=} [p] Properties to set
                                 */
                                function ProductCustomAttribute(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ProductCustomAttribute value.
                                 * @member {string|null|undefined} value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @instance
                                 */
                                ProductCustomAttribute.prototype.value = null;
    
                                /**
                                 * ProductCustomAttribute index.
                                 * @member {google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex} index
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @instance
                                 */
                                ProductCustomAttribute.prototype.index = 0;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ProductCustomAttribute _value.
                                 * @member {"value"|undefined} _value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @instance
                                 */
                                Object.defineProperty(ProductCustomAttribute.prototype, "_value", {
                                    get: $util.oneOfGetter($oneOfFields = ["value"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ProductCustomAttribute instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCustomAttribute=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute} ProductCustomAttribute instance
                                 */
                                ProductCustomAttribute.create = function create(properties) {
                                    return new ProductCustomAttribute(properties);
                                };
    
                                /**
                                 * Encodes the specified ProductCustomAttribute message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCustomAttribute} m ProductCustomAttribute message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductCustomAttribute.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                        w.uint32(10).string(m.value);
                                    if (m.index != null && Object.hasOwnProperty.call(m, "index"))
                                        w.uint32(16).int32(m.index);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ProductCustomAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductCustomAttribute} message ProductCustomAttribute message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductCustomAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ProductCustomAttribute message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute} ProductCustomAttribute
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductCustomAttribute.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.value = r.string();
                                            break;
                                        case 2:
                                            m.index = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ProductCustomAttribute message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute} ProductCustomAttribute
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductCustomAttribute.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ProductCustomAttribute message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ProductCustomAttribute.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        p._value = 1;
                                        if (!$util.isString(m.value))
                                            return "value: string expected";
                                    }
                                    if (m.index != null && m.hasOwnProperty("index")) {
                                        switch (m.index) {
                                        default:
                                            return "index: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ProductCustomAttribute message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute} ProductCustomAttribute
                                 */
                                ProductCustomAttribute.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute();
                                    if (d.value != null) {
                                        m.value = String(d.value);
                                    }
                                    switch (d.index) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.index = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.index = 1;
                                        break;
                                    case "INDEX0":
                                    case 2:
                                        m.index = 2;
                                        break;
                                    case "INDEX1":
                                    case 3:
                                        m.index = 3;
                                        break;
                                    case "INDEX2":
                                    case 4:
                                        m.index = 4;
                                        break;
                                    case "INDEX3":
                                    case 5:
                                        m.index = 5;
                                        break;
                                    case "INDEX4":
                                    case 6:
                                        m.index = 6;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ProductCustomAttribute message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute} m ProductCustomAttribute
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ProductCustomAttribute.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.index = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        d.value = m.value;
                                        if (o.oneofs)
                                            d._value = "value";
                                    }
                                    if (m.index != null && m.hasOwnProperty("index")) {
                                        d.index = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex[m.index] : m.index;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ProductCustomAttribute to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductCustomAttribute
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ProductCustomAttribute.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ProductCustomAttribute;
                            })();
    
                            ListingGroupFilterDimension.ProductItemId = (function() {
    
                                /**
                                 * Properties of a ProductItemId.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @interface IProductItemId
                                 * @property {string|null} [value] ProductItemId value
                                 */
    
                                /**
                                 * Constructs a new ProductItemId.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @classdesc Represents a ProductItemId.
                                 * @implements IProductItemId
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductItemId=} [p] Properties to set
                                 */
                                function ProductItemId(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ProductItemId value.
                                 * @member {string|null|undefined} value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @instance
                                 */
                                ProductItemId.prototype.value = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ProductItemId _value.
                                 * @member {"value"|undefined} _value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @instance
                                 */
                                Object.defineProperty(ProductItemId.prototype, "_value", {
                                    get: $util.oneOfGetter($oneOfFields = ["value"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ProductItemId instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductItemId=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId} ProductItemId instance
                                 */
                                ProductItemId.create = function create(properties) {
                                    return new ProductItemId(properties);
                                };
    
                                /**
                                 * Encodes the specified ProductItemId message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductItemId} m ProductItemId message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductItemId.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                        w.uint32(10).string(m.value);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ProductItemId message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductItemId} message ProductItemId message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductItemId.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ProductItemId message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId} ProductItemId
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductItemId.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.value = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ProductItemId message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId} ProductItemId
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductItemId.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ProductItemId message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ProductItemId.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        p._value = 1;
                                        if (!$util.isString(m.value))
                                            return "value: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ProductItemId message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId} ProductItemId
                                 */
                                ProductItemId.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId();
                                    if (d.value != null) {
                                        m.value = String(d.value);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ProductItemId message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId} m ProductItemId
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ProductItemId.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        d.value = m.value;
                                        if (o.oneofs)
                                            d._value = "value";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ProductItemId to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductItemId
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ProductItemId.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ProductItemId;
                            })();
    
                            ListingGroupFilterDimension.ProductType = (function() {
    
                                /**
                                 * Properties of a ProductType.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @interface IProductType
                                 * @property {string|null} [value] ProductType value
                                 * @property {google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel|null} [level] ProductType level
                                 */
    
                                /**
                                 * Constructs a new ProductType.
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension
                                 * @classdesc Represents a ProductType.
                                 * @implements IProductType
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductType=} [p] Properties to set
                                 */
                                function ProductType(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ProductType value.
                                 * @member {string|null|undefined} value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @instance
                                 */
                                ProductType.prototype.value = null;
    
                                /**
                                 * ProductType level.
                                 * @member {google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel} level
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @instance
                                 */
                                ProductType.prototype.level = 0;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ProductType _value.
                                 * @member {"value"|undefined} _value
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @instance
                                 */
                                Object.defineProperty(ProductType.prototype, "_value", {
                                    get: $util.oneOfGetter($oneOfFields = ["value"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ProductType instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductType=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType} ProductType instance
                                 */
                                ProductType.create = function create(properties) {
                                    return new ProductType(properties);
                                };
    
                                /**
                                 * Encodes the specified ProductType message. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductType} m ProductType message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductType.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                        w.uint32(10).string(m.value);
                                    if (m.level != null && Object.hasOwnProperty.call(m, "level"))
                                        w.uint32(16).int32(m.level);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ProductType message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.IProductType} message ProductType message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ProductType.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ProductType message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType} ProductType
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductType.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.value = r.string();
                                            break;
                                        case 2:
                                            m.level = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ProductType message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType} ProductType
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ProductType.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ProductType message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ProductType.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        p._value = 1;
                                        if (!$util.isString(m.value))
                                            return "value: string expected";
                                    }
                                    if (m.level != null && m.hasOwnProperty("level")) {
                                        switch (m.level) {
                                        default:
                                            return "level: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ProductType message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType} ProductType
                                 */
                                ProductType.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType();
                                    if (d.value != null) {
                                        m.value = String(d.value);
                                    }
                                    switch (d.level) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.level = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.level = 1;
                                        break;
                                    case "LEVEL1":
                                    case 2:
                                        m.level = 2;
                                        break;
                                    case "LEVEL2":
                                    case 3:
                                        m.level = 3;
                                        break;
                                    case "LEVEL3":
                                    case 4:
                                        m.level = 4;
                                        break;
                                    case "LEVEL4":
                                    case 5:
                                        m.level = 5;
                                        break;
                                    case "LEVEL5":
                                    case 6:
                                        m.level = 6;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ProductType message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType} m ProductType
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ProductType.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.level = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        d.value = m.value;
                                        if (o.oneofs)
                                            d._value = "value";
                                    }
                                    if (m.level != null && m.hasOwnProperty("level")) {
                                        d.level = o.enums === String ? $root.google.ads.googleads.v11.enums.ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel[m.level] : m.level;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ProductType to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ListingGroupFilterDimension.ProductType
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ProductType.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ProductType;
                            })();
    
                            return ListingGroupFilterDimension;
                        })();
    
                        resources.AssetGroupProductGroupView = (function() {
    
                            /**
                             * Properties of an AssetGroupProductGroupView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetGroupProductGroupView
                             * @property {string|null} [resource_name] AssetGroupProductGroupView resource_name
                             * @property {string|null} [asset_group] AssetGroupProductGroupView asset_group
                             * @property {string|null} [asset_group_listing_group_filter] AssetGroupProductGroupView asset_group_listing_group_filter
                             */
    
                            /**
                             * Constructs a new AssetGroupProductGroupView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetGroupProductGroupView.
                             * @implements IAssetGroupProductGroupView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetGroupProductGroupView=} [p] Properties to set
                             */
                            function AssetGroupProductGroupView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupProductGroupView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @instance
                             */
                            AssetGroupProductGroupView.prototype.resource_name = "";
    
                            /**
                             * AssetGroupProductGroupView asset_group.
                             * @member {string} asset_group
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @instance
                             */
                            AssetGroupProductGroupView.prototype.asset_group = "";
    
                            /**
                             * AssetGroupProductGroupView asset_group_listing_group_filter.
                             * @member {string} asset_group_listing_group_filter
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @instance
                             */
                            AssetGroupProductGroupView.prototype.asset_group_listing_group_filter = "";
    
                            /**
                             * Creates a new AssetGroupProductGroupView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupProductGroupView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetGroupProductGroupView} AssetGroupProductGroupView instance
                             */
                            AssetGroupProductGroupView.create = function create(properties) {
                                return new AssetGroupProductGroupView(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupProductGroupView message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupProductGroupView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupProductGroupView} m AssetGroupProductGroupView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupProductGroupView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_group != null && Object.hasOwnProperty.call(m, "asset_group"))
                                    w.uint32(18).string(m.asset_group);
                                if (m.asset_group_listing_group_filter != null && Object.hasOwnProperty.call(m, "asset_group_listing_group_filter"))
                                    w.uint32(34).string(m.asset_group_listing_group_filter);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupProductGroupView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupProductGroupView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupProductGroupView} message AssetGroupProductGroupView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupProductGroupView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupProductGroupView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetGroupProductGroupView} AssetGroupProductGroupView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupProductGroupView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_group = r.string();
                                        break;
                                    case 4:
                                        m.asset_group_listing_group_filter = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupProductGroupView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetGroupProductGroupView} AssetGroupProductGroupView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupProductGroupView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupProductGroupView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupProductGroupView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    if (!$util.isString(m.asset_group))
                                        return "asset_group: string expected";
                                }
                                if (m.asset_group_listing_group_filter != null && m.hasOwnProperty("asset_group_listing_group_filter")) {
                                    if (!$util.isString(m.asset_group_listing_group_filter))
                                        return "asset_group_listing_group_filter: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupProductGroupView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetGroupProductGroupView} AssetGroupProductGroupView
                             */
                            AssetGroupProductGroupView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_group != null) {
                                    m.asset_group = String(d.asset_group);
                                }
                                if (d.asset_group_listing_group_filter != null) {
                                    m.asset_group_listing_group_filter = String(d.asset_group_listing_group_filter);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupProductGroupView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetGroupProductGroupView} m AssetGroupProductGroupView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupProductGroupView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_group = "";
                                    d.asset_group_listing_group_filter = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    d.asset_group = m.asset_group;
                                }
                                if (m.asset_group_listing_group_filter != null && m.hasOwnProperty("asset_group_listing_group_filter")) {
                                    d.asset_group_listing_group_filter = m.asset_group_listing_group_filter;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupProductGroupView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetGroupProductGroupView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupProductGroupView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupProductGroupView;
                        })();
    
                        resources.AssetGroupSignal = (function() {
    
                            /**
                             * Properties of an AssetGroupSignal.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetGroupSignal
                             * @property {string|null} [resource_name] AssetGroupSignal resource_name
                             * @property {string|null} [asset_group] AssetGroupSignal asset_group
                             * @property {google.ads.googleads.v11.common.IAudienceInfo|null} [audience] AssetGroupSignal audience
                             */
    
                            /**
                             * Constructs a new AssetGroupSignal.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetGroupSignal.
                             * @implements IAssetGroupSignal
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetGroupSignal=} [p] Properties to set
                             */
                            function AssetGroupSignal(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupSignal resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @instance
                             */
                            AssetGroupSignal.prototype.resource_name = "";
    
                            /**
                             * AssetGroupSignal asset_group.
                             * @member {string} asset_group
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @instance
                             */
                            AssetGroupSignal.prototype.asset_group = "";
    
                            /**
                             * AssetGroupSignal audience.
                             * @member {google.ads.googleads.v11.common.IAudienceInfo|null|undefined} audience
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @instance
                             */
                            AssetGroupSignal.prototype.audience = null;
    
                            /**
                             * Creates a new AssetGroupSignal instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupSignal=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetGroupSignal} AssetGroupSignal instance
                             */
                            AssetGroupSignal.create = function create(properties) {
                                return new AssetGroupSignal(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupSignal message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupSignal.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupSignal} m AssetGroupSignal message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupSignal.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_group != null && Object.hasOwnProperty.call(m, "asset_group"))
                                    w.uint32(18).string(m.asset_group);
                                if (m.audience != null && Object.hasOwnProperty.call(m, "audience"))
                                    $root.google.ads.googleads.v11.common.AudienceInfo.encode(m.audience, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupSignal message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetGroupSignal.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetGroupSignal} message AssetGroupSignal message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupSignal.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupSignal message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetGroupSignal} AssetGroupSignal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupSignal.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetGroupSignal();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_group = r.string();
                                        break;
                                    case 3:
                                        m.audience = $root.google.ads.googleads.v11.common.AudienceInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupSignal message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetGroupSignal} AssetGroupSignal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupSignal.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupSignal message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupSignal.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    if (!$util.isString(m.asset_group))
                                        return "asset_group: string expected";
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AudienceInfo.verify(m.audience);
                                        if (e)
                                            return "audience." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupSignal message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetGroupSignal} AssetGroupSignal
                             */
                            AssetGroupSignal.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetGroupSignal)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetGroupSignal();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_group != null) {
                                    m.asset_group = String(d.asset_group);
                                }
                                if (d.audience != null) {
                                    if (typeof d.audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AssetGroupSignal.audience: object expected");
                                    m.audience = $root.google.ads.googleads.v11.common.AudienceInfo.fromObject(d.audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupSignal message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetGroupSignal} m AssetGroupSignal
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupSignal.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_group = "";
                                    d.audience = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    d.asset_group = m.asset_group;
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    d.audience = $root.google.ads.googleads.v11.common.AudienceInfo.toObject(m.audience, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupSignal to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetGroupSignal
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupSignal.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupSignal;
                        })();
    
                        resources.AssetSet = (function() {
    
                            /**
                             * Properties of an AssetSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetSet
                             * @property {number|null} [id] AssetSet id
                             * @property {string|null} [resource_name] AssetSet resource_name
                             * @property {string|null} [name] AssetSet name
                             * @property {google.ads.googleads.v11.enums.AssetSetTypeEnum.AssetSetType|null} [type] AssetSet type
                             * @property {google.ads.googleads.v11.enums.AssetSetStatusEnum.AssetSetStatus|null} [status] AssetSet status
                             * @property {google.ads.googleads.v11.resources.AssetSet.IMerchantCenterFeed|null} [merchant_center_feed] AssetSet merchant_center_feed
                             */
    
                            /**
                             * Constructs a new AssetSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetSet.
                             * @implements IAssetSet
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetSet=} [p] Properties to set
                             */
                            function AssetSet(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetSet id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @instance
                             */
                            AssetSet.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AssetSet resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @instance
                             */
                            AssetSet.prototype.resource_name = "";
    
                            /**
                             * AssetSet name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @instance
                             */
                            AssetSet.prototype.name = "";
    
                            /**
                             * AssetSet type.
                             * @member {google.ads.googleads.v11.enums.AssetSetTypeEnum.AssetSetType} type
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @instance
                             */
                            AssetSet.prototype.type = 0;
    
                            /**
                             * AssetSet status.
                             * @member {google.ads.googleads.v11.enums.AssetSetStatusEnum.AssetSetStatus} status
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @instance
                             */
                            AssetSet.prototype.status = 0;
    
                            /**
                             * AssetSet merchant_center_feed.
                             * @member {google.ads.googleads.v11.resources.AssetSet.IMerchantCenterFeed|null|undefined} merchant_center_feed
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @instance
                             */
                            AssetSet.prototype.merchant_center_feed = null;
    
                            /**
                             * Creates a new AssetSet instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetSet=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetSet} AssetSet instance
                             */
                            AssetSet.create = function create(properties) {
                                return new AssetSet(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSet message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetSet.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetSet} m AssetSet message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSet.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(18).string(m.name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.merchant_center_feed != null && Object.hasOwnProperty.call(m, "merchant_center_feed"))
                                    $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed.encode(m.merchant_center_feed, w.uint32(42).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(48).int64(m.id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSet message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetSet.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetSet} message AssetSet message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSet.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSet message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetSet} AssetSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSet.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetSet();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.id = r.int64();
                                        break;
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.name = r.string();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.merchant_center_feed = $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSet message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetSet} AssetSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSet.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSet message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSet.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.merchant_center_feed != null && m.hasOwnProperty("merchant_center_feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed.verify(m.merchant_center_feed);
                                        if (e)
                                            return "merchant_center_feed." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetSet message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetSet} AssetSet
                             */
                            AssetSet.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetSet)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetSet();
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "PAGE_FEED":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "DYNAMIC_EDUCATION":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MERCHANT_CENTER_FEED":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "DYNAMIC_REAL_ESTATE":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "DYNAMIC_CUSTOM":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "DYNAMIC_HOTELS_AND_RENTALS":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "DYNAMIC_FLIGHTS":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "DYNAMIC_TRAVEL":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "DYNAMIC_LOCAL":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "DYNAMIC_JOBS":
                                case 11:
                                    m.type = 11;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.merchant_center_feed != null) {
                                    if (typeof d.merchant_center_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.AssetSet.merchant_center_feed: object expected");
                                    m.merchant_center_feed = $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed.fromObject(d.merchant_center_feed);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetSet message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetSet} m AssetSet
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSet.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.merchant_center_feed = null;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSetTypeEnum.AssetSetType[m.type] : m.type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSetStatusEnum.AssetSetStatus[m.status] : m.status;
                                }
                                if (m.merchant_center_feed != null && m.hasOwnProperty("merchant_center_feed")) {
                                    d.merchant_center_feed = $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed.toObject(m.merchant_center_feed, o);
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetSet to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetSet
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSet.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            AssetSet.MerchantCenterFeed = (function() {
    
                                /**
                                 * Properties of a MerchantCenterFeed.
                                 * @memberof google.ads.googleads.v11.resources.AssetSet
                                 * @interface IMerchantCenterFeed
                                 * @property {number|null} [merchant_id] MerchantCenterFeed merchant_id
                                 * @property {string|null} [feed_label] MerchantCenterFeed feed_label
                                 */
    
                                /**
                                 * Constructs a new MerchantCenterFeed.
                                 * @memberof google.ads.googleads.v11.resources.AssetSet
                                 * @classdesc Represents a MerchantCenterFeed.
                                 * @implements IMerchantCenterFeed
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.AssetSet.IMerchantCenterFeed=} [p] Properties to set
                                 */
                                function MerchantCenterFeed(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * MerchantCenterFeed merchant_id.
                                 * @member {number} merchant_id
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @instance
                                 */
                                MerchantCenterFeed.prototype.merchant_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * MerchantCenterFeed feed_label.
                                 * @member {string|null|undefined} feed_label
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @instance
                                 */
                                MerchantCenterFeed.prototype.feed_label = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * MerchantCenterFeed _feed_label.
                                 * @member {"feed_label"|undefined} _feed_label
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @instance
                                 */
                                Object.defineProperty(MerchantCenterFeed.prototype, "_feed_label", {
                                    get: $util.oneOfGetter($oneOfFields = ["feed_label"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new MerchantCenterFeed instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AssetSet.IMerchantCenterFeed=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed} MerchantCenterFeed instance
                                 */
                                MerchantCenterFeed.create = function create(properties) {
                                    return new MerchantCenterFeed(properties);
                                };
    
                                /**
                                 * Encodes the specified MerchantCenterFeed message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AssetSet.IMerchantCenterFeed} m MerchantCenterFeed message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MerchantCenterFeed.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.merchant_id != null && Object.hasOwnProperty.call(m, "merchant_id"))
                                        w.uint32(8).int64(m.merchant_id);
                                    if (m.feed_label != null && Object.hasOwnProperty.call(m, "feed_label"))
                                        w.uint32(18).string(m.feed_label);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified MerchantCenterFeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AssetSet.IMerchantCenterFeed} message MerchantCenterFeed message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MerchantCenterFeed.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a MerchantCenterFeed message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed} MerchantCenterFeed
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MerchantCenterFeed.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.merchant_id = r.int64();
                                            break;
                                        case 2:
                                            m.feed_label = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a MerchantCenterFeed message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed} MerchantCenterFeed
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MerchantCenterFeed.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a MerchantCenterFeed message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                MerchantCenterFeed.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                        if (!$util.isInteger(m.merchant_id) && !(m.merchant_id && $util.isInteger(m.merchant_id.low) && $util.isInteger(m.merchant_id.high)))
                                            return "merchant_id: integer|Long expected";
                                    }
                                    if (m.feed_label != null && m.hasOwnProperty("feed_label")) {
                                        p._feed_label = 1;
                                        if (!$util.isString(m.feed_label))
                                            return "feed_label: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a MerchantCenterFeed message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed} MerchantCenterFeed
                                 */
                                MerchantCenterFeed.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed();
                                    if (d.merchant_id != null) {
                                        if ($util.Long)
                                            (m.merchant_id = $util.Long.fromValue(d.merchant_id)).unsigned = false;
                                        else if (typeof d.merchant_id === "string")
                                            m.merchant_id = parseInt(d.merchant_id, 10);
                                        else if (typeof d.merchant_id === "number")
                                            m.merchant_id = d.merchant_id;
                                        else if (typeof d.merchant_id === "object")
                                            m.merchant_id = new $util.LongBits(d.merchant_id.low >>> 0, d.merchant_id.high >>> 0).toNumber();
                                    }
                                    if (d.feed_label != null) {
                                        m.feed_label = String(d.feed_label);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a MerchantCenterFeed message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed} m MerchantCenterFeed
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                MerchantCenterFeed.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.merchant_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.merchant_id = o.longs === String ? "0" : 0;
                                    }
                                    if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                        if (typeof m.merchant_id === "number")
                                            d.merchant_id = o.longs === String ? String(m.merchant_id) : m.merchant_id;
                                        else
                                            d.merchant_id = o.longs === String ? $util.Long.prototype.toString.call(m.merchant_id) : o.longs === Number ? new $util.LongBits(m.merchant_id.low >>> 0, m.merchant_id.high >>> 0).toNumber() : m.merchant_id;
                                    }
                                    if (m.feed_label != null && m.hasOwnProperty("feed_label")) {
                                        d.feed_label = m.feed_label;
                                        if (o.oneofs)
                                            d._feed_label = "feed_label";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this MerchantCenterFeed to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.AssetSet.MerchantCenterFeed
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                MerchantCenterFeed.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return MerchantCenterFeed;
                            })();
    
                            return AssetSet;
                        })();
    
                        resources.AssetSetAsset = (function() {
    
                            /**
                             * Properties of an AssetSetAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAssetSetAsset
                             * @property {string|null} [resource_name] AssetSetAsset resource_name
                             * @property {string|null} [asset_set] AssetSetAsset asset_set
                             * @property {string|null} [asset] AssetSetAsset asset
                             * @property {google.ads.googleads.v11.enums.AssetSetAssetStatusEnum.AssetSetAssetStatus|null} [status] AssetSetAsset status
                             */
    
                            /**
                             * Constructs a new AssetSetAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AssetSetAsset.
                             * @implements IAssetSetAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAssetSetAsset=} [p] Properties to set
                             */
                            function AssetSetAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetSetAsset resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @instance
                             */
                            AssetSetAsset.prototype.resource_name = "";
    
                            /**
                             * AssetSetAsset asset_set.
                             * @member {string} asset_set
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @instance
                             */
                            AssetSetAsset.prototype.asset_set = "";
    
                            /**
                             * AssetSetAsset asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @instance
                             */
                            AssetSetAsset.prototype.asset = "";
    
                            /**
                             * AssetSetAsset status.
                             * @member {google.ads.googleads.v11.enums.AssetSetAssetStatusEnum.AssetSetAssetStatus} status
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @instance
                             */
                            AssetSetAsset.prototype.status = 0;
    
                            /**
                             * Creates a new AssetSetAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetSetAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AssetSetAsset} AssetSetAsset instance
                             */
                            AssetSetAsset.create = function create(properties) {
                                return new AssetSetAsset(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetAsset message. Does not implicitly {@link google.ads.googleads.v11.resources.AssetSetAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetSetAsset} m AssetSetAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_set != null && Object.hasOwnProperty.call(m, "asset_set"))
                                    w.uint32(18).string(m.asset_set);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(26).string(m.asset);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AssetSetAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAssetSetAsset} message AssetSetAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AssetSetAsset} AssetSetAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AssetSetAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_set = r.string();
                                        break;
                                    case 3:
                                        m.asset = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AssetSetAsset} AssetSetAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    if (!$util.isString(m.asset_set))
                                        return "asset_set: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AssetSetAsset} AssetSetAsset
                             */
                            AssetSetAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AssetSetAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AssetSetAsset();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_set != null) {
                                    m.asset_set = String(d.asset_set);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetSetAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.AssetSetAsset} m AssetSetAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_set = "";
                                    d.asset = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    d.asset_set = m.asset_set;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSetAssetStatusEnum.AssetSetAssetStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetSetAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AssetSetAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetSetAsset;
                        })();
    
                        resources.Audience = (function() {
    
                            /**
                             * Properties of an Audience.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAudience
                             * @property {string|null} [resource_name] Audience resource_name
                             * @property {number|null} [id] Audience id
                             * @property {google.ads.googleads.v11.enums.AudienceStatusEnum.AudienceStatus|null} [status] Audience status
                             * @property {string|null} [name] Audience name
                             * @property {string|null} [description] Audience description
                             * @property {Array.<google.ads.googleads.v11.common.IAudienceDimension>|null} [dimensions] Audience dimensions
                             * @property {google.ads.googleads.v11.common.IAudienceExclusionDimension|null} [exclusion_dimension] Audience exclusion_dimension
                             */
    
                            /**
                             * Constructs a new Audience.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an Audience.
                             * @implements IAudience
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAudience=} [p] Properties to set
                             */
                            function Audience(p) {
                                this.dimensions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Audience resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             */
                            Audience.prototype.resource_name = "";
    
                            /**
                             * Audience id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             */
                            Audience.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Audience status.
                             * @member {google.ads.googleads.v11.enums.AudienceStatusEnum.AudienceStatus} status
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             */
                            Audience.prototype.status = 0;
    
                            /**
                             * Audience name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             */
                            Audience.prototype.name = "";
    
                            /**
                             * Audience description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             */
                            Audience.prototype.description = "";
    
                            /**
                             * Audience dimensions.
                             * @member {Array.<google.ads.googleads.v11.common.IAudienceDimension>} dimensions
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             */
                            Audience.prototype.dimensions = $util.emptyArray;
    
                            /**
                             * Audience exclusion_dimension.
                             * @member {google.ads.googleads.v11.common.IAudienceExclusionDimension|null|undefined} exclusion_dimension
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             */
                            Audience.prototype.exclusion_dimension = null;
    
                            /**
                             * Creates a new Audience instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAudience=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Audience} Audience instance
                             */
                            Audience.create = function create(properties) {
                                return new Audience(properties);
                            };
    
                            /**
                             * Encodes the specified Audience message. Does not implicitly {@link google.ads.googleads.v11.resources.Audience.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAudience} m Audience message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Audience.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(34).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(42).string(m.description);
                                if (m.dimensions != null && m.dimensions.length) {
                                    for (var i = 0; i < m.dimensions.length; ++i)
                                        $root.google.ads.googleads.v11.common.AudienceDimension.encode(m.dimensions[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.exclusion_dimension != null && Object.hasOwnProperty.call(m, "exclusion_dimension"))
                                    $root.google.ads.googleads.v11.common.AudienceExclusionDimension.encode(m.exclusion_dimension, w.uint32(58).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified Audience message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Audience.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAudience} message Audience message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Audience.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Audience message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Audience} Audience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Audience.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Audience();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 4:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.description = r.string();
                                        break;
                                    case 6:
                                        if (!(m.dimensions && m.dimensions.length))
                                            m.dimensions = [];
                                        m.dimensions.push($root.google.ads.googleads.v11.common.AudienceDimension.decode(r, r.uint32()));
                                        break;
                                    case 7:
                                        m.exclusion_dimension = $root.google.ads.googleads.v11.common.AudienceExclusionDimension.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an Audience message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Audience} Audience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Audience.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Audience message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Audience.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.dimensions != null && m.hasOwnProperty("dimensions")) {
                                    if (!Array.isArray(m.dimensions))
                                        return "dimensions: array expected";
                                    for (var i = 0; i < m.dimensions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AudienceDimension.verify(m.dimensions[i]);
                                            if (e)
                                                return "dimensions." + e;
                                        }
                                    }
                                }
                                if (m.exclusion_dimension != null && m.hasOwnProperty("exclusion_dimension")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AudienceExclusionDimension.verify(m.exclusion_dimension);
                                        if (e)
                                            return "exclusion_dimension." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Audience message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Audience} Audience
                             */
                            Audience.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Audience)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Audience();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.dimensions) {
                                    if (!Array.isArray(d.dimensions))
                                        throw TypeError(".google.ads.googleads.v11.resources.Audience.dimensions: array expected");
                                    m.dimensions = [];
                                    for (var i = 0; i < d.dimensions.length; ++i) {
                                        if (typeof d.dimensions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Audience.dimensions: object expected");
                                        m.dimensions[i] = $root.google.ads.googleads.v11.common.AudienceDimension.fromObject(d.dimensions[i]);
                                    }
                                }
                                if (d.exclusion_dimension != null) {
                                    if (typeof d.exclusion_dimension !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Audience.exclusion_dimension: object expected");
                                    m.exclusion_dimension = $root.google.ads.googleads.v11.common.AudienceExclusionDimension.fromObject(d.exclusion_dimension);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an Audience message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @static
                             * @param {google.ads.googleads.v11.resources.Audience} m Audience
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Audience.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.dimensions = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.name = "";
                                    d.description = "";
                                    d.exclusion_dimension = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AudienceStatusEnum.AudienceStatus[m.status] : m.status;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.dimensions && m.dimensions.length) {
                                    d.dimensions = [];
                                    for (var j = 0; j < m.dimensions.length; ++j) {
                                        d.dimensions[j] = $root.google.ads.googleads.v11.common.AudienceDimension.toObject(m.dimensions[j], o);
                                    }
                                }
                                if (m.exclusion_dimension != null && m.hasOwnProperty("exclusion_dimension")) {
                                    d.exclusion_dimension = $root.google.ads.googleads.v11.common.AudienceExclusionDimension.toObject(m.exclusion_dimension, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Audience to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Audience
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Audience.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Audience;
                        })();
    
                        resources.BatchJob = (function() {
    
                            /**
                             * Properties of a BatchJob.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IBatchJob
                             * @property {string|null} [resource_name] BatchJob resource_name
                             * @property {number|null} [id] BatchJob id
                             * @property {string|null} [next_add_sequence_token] BatchJob next_add_sequence_token
                             * @property {google.ads.googleads.v11.resources.BatchJob.IBatchJobMetadata|null} [metadata] BatchJob metadata
                             * @property {google.ads.googleads.v11.enums.BatchJobStatusEnum.BatchJobStatus|null} [status] BatchJob status
                             * @property {string|null} [long_running_operation] BatchJob long_running_operation
                             */
    
                            /**
                             * Constructs a new BatchJob.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a BatchJob.
                             * @implements IBatchJob
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IBatchJob=} [p] Properties to set
                             */
                            function BatchJob(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BatchJob resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            BatchJob.prototype.resource_name = "";
    
                            /**
                             * BatchJob id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            BatchJob.prototype.id = null;
    
                            /**
                             * BatchJob next_add_sequence_token.
                             * @member {string|null|undefined} next_add_sequence_token
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            BatchJob.prototype.next_add_sequence_token = null;
    
                            /**
                             * BatchJob metadata.
                             * @member {google.ads.googleads.v11.resources.BatchJob.IBatchJobMetadata|null|undefined} metadata
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            BatchJob.prototype.metadata = null;
    
                            /**
                             * BatchJob status.
                             * @member {google.ads.googleads.v11.enums.BatchJobStatusEnum.BatchJobStatus} status
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            BatchJob.prototype.status = 0;
    
                            /**
                             * BatchJob long_running_operation.
                             * @member {string|null|undefined} long_running_operation
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            BatchJob.prototype.long_running_operation = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BatchJob _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            Object.defineProperty(BatchJob.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BatchJob _next_add_sequence_token.
                             * @member {"next_add_sequence_token"|undefined} _next_add_sequence_token
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            Object.defineProperty(BatchJob.prototype, "_next_add_sequence_token", {
                                get: $util.oneOfGetter($oneOfFields = ["next_add_sequence_token"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BatchJob _long_running_operation.
                             * @member {"long_running_operation"|undefined} _long_running_operation
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             */
                            Object.defineProperty(BatchJob.prototype, "_long_running_operation", {
                                get: $util.oneOfGetter($oneOfFields = ["long_running_operation"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BatchJob instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBatchJob=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.BatchJob} BatchJob instance
                             */
                            BatchJob.create = function create(properties) {
                                return new BatchJob(properties);
                            };
    
                            /**
                             * Encodes the specified BatchJob message. Does not implicitly {@link google.ads.googleads.v11.resources.BatchJob.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBatchJob} m BatchJob message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJob.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.metadata != null && Object.hasOwnProperty.call(m, "metadata"))
                                    $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata.encode(m.metadata, w.uint32(34).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(56).int64(m.id);
                                if (m.next_add_sequence_token != null && Object.hasOwnProperty.call(m, "next_add_sequence_token"))
                                    w.uint32(66).string(m.next_add_sequence_token);
                                if (m.long_running_operation != null && Object.hasOwnProperty.call(m, "long_running_operation"))
                                    w.uint32(74).string(m.long_running_operation);
                                return w;
                            };
    
                            /**
                             * Encodes the specified BatchJob message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BatchJob.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBatchJob} message BatchJob message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJob.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BatchJob message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.BatchJob} BatchJob
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJob.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BatchJob();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.id = r.int64();
                                        break;
                                    case 8:
                                        m.next_add_sequence_token = r.string();
                                        break;
                                    case 4:
                                        m.metadata = $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 9:
                                        m.long_running_operation = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BatchJob message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.BatchJob} BatchJob
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJob.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BatchJob message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BatchJob.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.next_add_sequence_token != null && m.hasOwnProperty("next_add_sequence_token")) {
                                    p._next_add_sequence_token = 1;
                                    if (!$util.isString(m.next_add_sequence_token))
                                        return "next_add_sequence_token: string expected";
                                }
                                if (m.metadata != null && m.hasOwnProperty("metadata")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata.verify(m.metadata);
                                        if (e)
                                            return "metadata." + e;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    p._long_running_operation = 1;
                                    if (!$util.isString(m.long_running_operation))
                                        return "long_running_operation: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BatchJob message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.BatchJob} BatchJob
                             */
                            BatchJob.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.BatchJob)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.BatchJob();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.next_add_sequence_token != null) {
                                    m.next_add_sequence_token = String(d.next_add_sequence_token);
                                }
                                if (d.metadata != null) {
                                    if (typeof d.metadata !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BatchJob.metadata: object expected");
                                    m.metadata = $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata.fromObject(d.metadata);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "RUNNING":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "DONE":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.long_running_operation != null) {
                                    m.long_running_operation = String(d.long_running_operation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BatchJob message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.BatchJob} m BatchJob
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BatchJob.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.metadata = null;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.metadata != null && m.hasOwnProperty("metadata")) {
                                    d.metadata = $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata.toObject(m.metadata, o);
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.BatchJobStatusEnum.BatchJobStatus[m.status] : m.status;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.next_add_sequence_token != null && m.hasOwnProperty("next_add_sequence_token")) {
                                    d.next_add_sequence_token = m.next_add_sequence_token;
                                    if (o.oneofs)
                                        d._next_add_sequence_token = "next_add_sequence_token";
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    d.long_running_operation = m.long_running_operation;
                                    if (o.oneofs)
                                        d._long_running_operation = "long_running_operation";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BatchJob to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.BatchJob
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BatchJob.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            BatchJob.BatchJobMetadata = (function() {
    
                                /**
                                 * Properties of a BatchJobMetadata.
                                 * @memberof google.ads.googleads.v11.resources.BatchJob
                                 * @interface IBatchJobMetadata
                                 * @property {string|null} [creation_date_time] BatchJobMetadata creation_date_time
                                 * @property {string|null} [start_date_time] BatchJobMetadata start_date_time
                                 * @property {string|null} [completion_date_time] BatchJobMetadata completion_date_time
                                 * @property {number|null} [estimated_completion_ratio] BatchJobMetadata estimated_completion_ratio
                                 * @property {number|null} [operation_count] BatchJobMetadata operation_count
                                 * @property {number|null} [executed_operation_count] BatchJobMetadata executed_operation_count
                                 */
    
                                /**
                                 * Constructs a new BatchJobMetadata.
                                 * @memberof google.ads.googleads.v11.resources.BatchJob
                                 * @classdesc Represents a BatchJobMetadata.
                                 * @implements IBatchJobMetadata
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.BatchJob.IBatchJobMetadata=} [p] Properties to set
                                 */
                                function BatchJobMetadata(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * BatchJobMetadata creation_date_time.
                                 * @member {string|null|undefined} creation_date_time
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                BatchJobMetadata.prototype.creation_date_time = null;
    
                                /**
                                 * BatchJobMetadata start_date_time.
                                 * @member {string|null|undefined} start_date_time
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                BatchJobMetadata.prototype.start_date_time = null;
    
                                /**
                                 * BatchJobMetadata completion_date_time.
                                 * @member {string|null|undefined} completion_date_time
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                BatchJobMetadata.prototype.completion_date_time = null;
    
                                /**
                                 * BatchJobMetadata estimated_completion_ratio.
                                 * @member {number|null|undefined} estimated_completion_ratio
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                BatchJobMetadata.prototype.estimated_completion_ratio = null;
    
                                /**
                                 * BatchJobMetadata operation_count.
                                 * @member {number|null|undefined} operation_count
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                BatchJobMetadata.prototype.operation_count = null;
    
                                /**
                                 * BatchJobMetadata executed_operation_count.
                                 * @member {number|null|undefined} executed_operation_count
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                BatchJobMetadata.prototype.executed_operation_count = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * BatchJobMetadata _creation_date_time.
                                 * @member {"creation_date_time"|undefined} _creation_date_time
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                Object.defineProperty(BatchJobMetadata.prototype, "_creation_date_time", {
                                    get: $util.oneOfGetter($oneOfFields = ["creation_date_time"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * BatchJobMetadata _start_date_time.
                                 * @member {"start_date_time"|undefined} _start_date_time
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                Object.defineProperty(BatchJobMetadata.prototype, "_start_date_time", {
                                    get: $util.oneOfGetter($oneOfFields = ["start_date_time"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * BatchJobMetadata _completion_date_time.
                                 * @member {"completion_date_time"|undefined} _completion_date_time
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                Object.defineProperty(BatchJobMetadata.prototype, "_completion_date_time", {
                                    get: $util.oneOfGetter($oneOfFields = ["completion_date_time"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * BatchJobMetadata _estimated_completion_ratio.
                                 * @member {"estimated_completion_ratio"|undefined} _estimated_completion_ratio
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                Object.defineProperty(BatchJobMetadata.prototype, "_estimated_completion_ratio", {
                                    get: $util.oneOfGetter($oneOfFields = ["estimated_completion_ratio"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * BatchJobMetadata _operation_count.
                                 * @member {"operation_count"|undefined} _operation_count
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                Object.defineProperty(BatchJobMetadata.prototype, "_operation_count", {
                                    get: $util.oneOfGetter($oneOfFields = ["operation_count"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * BatchJobMetadata _executed_operation_count.
                                 * @member {"executed_operation_count"|undefined} _executed_operation_count
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 */
                                Object.defineProperty(BatchJobMetadata.prototype, "_executed_operation_count", {
                                    get: $util.oneOfGetter($oneOfFields = ["executed_operation_count"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new BatchJobMetadata instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BatchJob.IBatchJobMetadata=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata} BatchJobMetadata instance
                                 */
                                BatchJobMetadata.create = function create(properties) {
                                    return new BatchJobMetadata(properties);
                                };
    
                                /**
                                 * Encodes the specified BatchJobMetadata message. Does not implicitly {@link google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BatchJob.IBatchJobMetadata} m BatchJobMetadata message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                BatchJobMetadata.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.start_date_time != null && Object.hasOwnProperty.call(m, "start_date_time"))
                                        w.uint32(58).string(m.start_date_time);
                                    if (m.creation_date_time != null && Object.hasOwnProperty.call(m, "creation_date_time"))
                                        w.uint32(66).string(m.creation_date_time);
                                    if (m.completion_date_time != null && Object.hasOwnProperty.call(m, "completion_date_time"))
                                        w.uint32(74).string(m.completion_date_time);
                                    if (m.estimated_completion_ratio != null && Object.hasOwnProperty.call(m, "estimated_completion_ratio"))
                                        w.uint32(81).double(m.estimated_completion_ratio);
                                    if (m.operation_count != null && Object.hasOwnProperty.call(m, "operation_count"))
                                        w.uint32(88).int64(m.operation_count);
                                    if (m.executed_operation_count != null && Object.hasOwnProperty.call(m, "executed_operation_count"))
                                        w.uint32(96).int64(m.executed_operation_count);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified BatchJobMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BatchJob.IBatchJobMetadata} message BatchJobMetadata message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                BatchJobMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a BatchJobMetadata message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata} BatchJobMetadata
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                BatchJobMetadata.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 8:
                                            m.creation_date_time = r.string();
                                            break;
                                        case 7:
                                            m.start_date_time = r.string();
                                            break;
                                        case 9:
                                            m.completion_date_time = r.string();
                                            break;
                                        case 10:
                                            m.estimated_completion_ratio = r.double();
                                            break;
                                        case 11:
                                            m.operation_count = r.int64();
                                            break;
                                        case 12:
                                            m.executed_operation_count = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a BatchJobMetadata message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata} BatchJobMetadata
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                BatchJobMetadata.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a BatchJobMetadata message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                BatchJobMetadata.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                        p._creation_date_time = 1;
                                        if (!$util.isString(m.creation_date_time))
                                            return "creation_date_time: string expected";
                                    }
                                    if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                        p._start_date_time = 1;
                                        if (!$util.isString(m.start_date_time))
                                            return "start_date_time: string expected";
                                    }
                                    if (m.completion_date_time != null && m.hasOwnProperty("completion_date_time")) {
                                        p._completion_date_time = 1;
                                        if (!$util.isString(m.completion_date_time))
                                            return "completion_date_time: string expected";
                                    }
                                    if (m.estimated_completion_ratio != null && m.hasOwnProperty("estimated_completion_ratio")) {
                                        p._estimated_completion_ratio = 1;
                                        if (typeof m.estimated_completion_ratio !== "number")
                                            return "estimated_completion_ratio: number expected";
                                    }
                                    if (m.operation_count != null && m.hasOwnProperty("operation_count")) {
                                        p._operation_count = 1;
                                        if (!$util.isInteger(m.operation_count) && !(m.operation_count && $util.isInteger(m.operation_count.low) && $util.isInteger(m.operation_count.high)))
                                            return "operation_count: integer|Long expected";
                                    }
                                    if (m.executed_operation_count != null && m.hasOwnProperty("executed_operation_count")) {
                                        p._executed_operation_count = 1;
                                        if (!$util.isInteger(m.executed_operation_count) && !(m.executed_operation_count && $util.isInteger(m.executed_operation_count.low) && $util.isInteger(m.executed_operation_count.high)))
                                            return "executed_operation_count: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a BatchJobMetadata message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata} BatchJobMetadata
                                 */
                                BatchJobMetadata.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata();
                                    if (d.creation_date_time != null) {
                                        m.creation_date_time = String(d.creation_date_time);
                                    }
                                    if (d.start_date_time != null) {
                                        m.start_date_time = String(d.start_date_time);
                                    }
                                    if (d.completion_date_time != null) {
                                        m.completion_date_time = String(d.completion_date_time);
                                    }
                                    if (d.estimated_completion_ratio != null) {
                                        m.estimated_completion_ratio = Number(d.estimated_completion_ratio);
                                    }
                                    if (d.operation_count != null) {
                                        if ($util.Long)
                                            (m.operation_count = $util.Long.fromValue(d.operation_count)).unsigned = false;
                                        else if (typeof d.operation_count === "string")
                                            m.operation_count = parseInt(d.operation_count, 10);
                                        else if (typeof d.operation_count === "number")
                                            m.operation_count = d.operation_count;
                                        else if (typeof d.operation_count === "object")
                                            m.operation_count = new $util.LongBits(d.operation_count.low >>> 0, d.operation_count.high >>> 0).toNumber();
                                    }
                                    if (d.executed_operation_count != null) {
                                        if ($util.Long)
                                            (m.executed_operation_count = $util.Long.fromValue(d.executed_operation_count)).unsigned = false;
                                        else if (typeof d.executed_operation_count === "string")
                                            m.executed_operation_count = parseInt(d.executed_operation_count, 10);
                                        else if (typeof d.executed_operation_count === "number")
                                            m.executed_operation_count = d.executed_operation_count;
                                        else if (typeof d.executed_operation_count === "object")
                                            m.executed_operation_count = new $util.LongBits(d.executed_operation_count.low >>> 0, d.executed_operation_count.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a BatchJobMetadata message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata} m BatchJobMetadata
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                BatchJobMetadata.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                        d.start_date_time = m.start_date_time;
                                        if (o.oneofs)
                                            d._start_date_time = "start_date_time";
                                    }
                                    if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                        d.creation_date_time = m.creation_date_time;
                                        if (o.oneofs)
                                            d._creation_date_time = "creation_date_time";
                                    }
                                    if (m.completion_date_time != null && m.hasOwnProperty("completion_date_time")) {
                                        d.completion_date_time = m.completion_date_time;
                                        if (o.oneofs)
                                            d._completion_date_time = "completion_date_time";
                                    }
                                    if (m.estimated_completion_ratio != null && m.hasOwnProperty("estimated_completion_ratio")) {
                                        d.estimated_completion_ratio = o.json && !isFinite(m.estimated_completion_ratio) ? String(m.estimated_completion_ratio) : m.estimated_completion_ratio;
                                        if (o.oneofs)
                                            d._estimated_completion_ratio = "estimated_completion_ratio";
                                    }
                                    if (m.operation_count != null && m.hasOwnProperty("operation_count")) {
                                        if (typeof m.operation_count === "number")
                                            d.operation_count = o.longs === String ? String(m.operation_count) : m.operation_count;
                                        else
                                            d.operation_count = o.longs === String ? $util.Long.prototype.toString.call(m.operation_count) : o.longs === Number ? new $util.LongBits(m.operation_count.low >>> 0, m.operation_count.high >>> 0).toNumber() : m.operation_count;
                                        if (o.oneofs)
                                            d._operation_count = "operation_count";
                                    }
                                    if (m.executed_operation_count != null && m.hasOwnProperty("executed_operation_count")) {
                                        if (typeof m.executed_operation_count === "number")
                                            d.executed_operation_count = o.longs === String ? String(m.executed_operation_count) : m.executed_operation_count;
                                        else
                                            d.executed_operation_count = o.longs === String ? $util.Long.prototype.toString.call(m.executed_operation_count) : o.longs === Number ? new $util.LongBits(m.executed_operation_count.low >>> 0, m.executed_operation_count.high >>> 0).toNumber() : m.executed_operation_count;
                                        if (o.oneofs)
                                            d._executed_operation_count = "executed_operation_count";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this BatchJobMetadata to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.BatchJob.BatchJobMetadata
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                BatchJobMetadata.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return BatchJobMetadata;
                            })();
    
                            return BatchJob;
                        })();
    
                        resources.BiddingDataExclusion = (function() {
    
                            /**
                             * Properties of a BiddingDataExclusion.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IBiddingDataExclusion
                             * @property {string|null} [resource_name] BiddingDataExclusion resource_name
                             * @property {number|null} [data_exclusion_id] BiddingDataExclusion data_exclusion_id
                             * @property {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.SeasonalityEventScope|null} [scope] BiddingDataExclusion scope
                             * @property {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.SeasonalityEventStatus|null} [status] BiddingDataExclusion status
                             * @property {string|null} [start_date_time] BiddingDataExclusion start_date_time
                             * @property {string|null} [end_date_time] BiddingDataExclusion end_date_time
                             * @property {string|null} [name] BiddingDataExclusion name
                             * @property {string|null} [description] BiddingDataExclusion description
                             * @property {Array.<google.ads.googleads.v11.enums.DeviceEnum.Device>|null} [devices] BiddingDataExclusion devices
                             * @property {Array.<string>|null} [campaigns] BiddingDataExclusion campaigns
                             * @property {Array.<google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType>|null} [advertising_channel_types] BiddingDataExclusion advertising_channel_types
                             */
    
                            /**
                             * Constructs a new BiddingDataExclusion.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a BiddingDataExclusion.
                             * @implements IBiddingDataExclusion
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IBiddingDataExclusion=} [p] Properties to set
                             */
                            function BiddingDataExclusion(p) {
                                this.devices = [];
                                this.campaigns = [];
                                this.advertising_channel_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BiddingDataExclusion resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.resource_name = "";
    
                            /**
                             * BiddingDataExclusion data_exclusion_id.
                             * @member {number} data_exclusion_id
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.data_exclusion_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BiddingDataExclusion scope.
                             * @member {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.SeasonalityEventScope} scope
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.scope = 0;
    
                            /**
                             * BiddingDataExclusion status.
                             * @member {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.SeasonalityEventStatus} status
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.status = 0;
    
                            /**
                             * BiddingDataExclusion start_date_time.
                             * @member {string} start_date_time
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.start_date_time = "";
    
                            /**
                             * BiddingDataExclusion end_date_time.
                             * @member {string} end_date_time
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.end_date_time = "";
    
                            /**
                             * BiddingDataExclusion name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.name = "";
    
                            /**
                             * BiddingDataExclusion description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.description = "";
    
                            /**
                             * BiddingDataExclusion devices.
                             * @member {Array.<google.ads.googleads.v11.enums.DeviceEnum.Device>} devices
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.devices = $util.emptyArray;
    
                            /**
                             * BiddingDataExclusion campaigns.
                             * @member {Array.<string>} campaigns
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.campaigns = $util.emptyArray;
    
                            /**
                             * BiddingDataExclusion advertising_channel_types.
                             * @member {Array.<google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType>} advertising_channel_types
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             */
                            BiddingDataExclusion.prototype.advertising_channel_types = $util.emptyArray;
    
                            /**
                             * Creates a new BiddingDataExclusion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingDataExclusion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.BiddingDataExclusion} BiddingDataExclusion instance
                             */
                            BiddingDataExclusion.create = function create(properties) {
                                return new BiddingDataExclusion(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingDataExclusion message. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingDataExclusion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingDataExclusion} m BiddingDataExclusion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingDataExclusion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.data_exclusion_id != null && Object.hasOwnProperty.call(m, "data_exclusion_id"))
                                    w.uint32(16).int64(m.data_exclusion_id);
                                if (m.scope != null && Object.hasOwnProperty.call(m, "scope"))
                                    w.uint32(24).int32(m.scope);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.start_date_time != null && Object.hasOwnProperty.call(m, "start_date_time"))
                                    w.uint32(42).string(m.start_date_time);
                                if (m.end_date_time != null && Object.hasOwnProperty.call(m, "end_date_time"))
                                    w.uint32(50).string(m.end_date_time);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(58).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(66).string(m.description);
                                if (m.devices != null && m.devices.length) {
                                    w.uint32(74).fork();
                                    for (var i = 0; i < m.devices.length; ++i)
                                        w.int32(m.devices[i]);
                                    w.ldelim();
                                }
                                if (m.campaigns != null && m.campaigns.length) {
                                    for (var i = 0; i < m.campaigns.length; ++i)
                                        w.uint32(82).string(m.campaigns[i]);
                                }
                                if (m.advertising_channel_types != null && m.advertising_channel_types.length) {
                                    w.uint32(90).fork();
                                    for (var i = 0; i < m.advertising_channel_types.length; ++i)
                                        w.int32(m.advertising_channel_types[i]);
                                    w.ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingDataExclusion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingDataExclusion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingDataExclusion} message BiddingDataExclusion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingDataExclusion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingDataExclusion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.BiddingDataExclusion} BiddingDataExclusion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingDataExclusion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BiddingDataExclusion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.data_exclusion_id = r.int64();
                                        break;
                                    case 3:
                                        m.scope = r.int32();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.start_date_time = r.string();
                                        break;
                                    case 6:
                                        m.end_date_time = r.string();
                                        break;
                                    case 7:
                                        m.name = r.string();
                                        break;
                                    case 8:
                                        m.description = r.string();
                                        break;
                                    case 9:
                                        if (!(m.devices && m.devices.length))
                                            m.devices = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.devices.push(r.int32());
                                        } else
                                            m.devices.push(r.int32());
                                        break;
                                    case 10:
                                        if (!(m.campaigns && m.campaigns.length))
                                            m.campaigns = [];
                                        m.campaigns.push(r.string());
                                        break;
                                    case 11:
                                        if (!(m.advertising_channel_types && m.advertising_channel_types.length))
                                            m.advertising_channel_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.advertising_channel_types.push(r.int32());
                                        } else
                                            m.advertising_channel_types.push(r.int32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingDataExclusion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.BiddingDataExclusion} BiddingDataExclusion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingDataExclusion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingDataExclusion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingDataExclusion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.data_exclusion_id != null && m.hasOwnProperty("data_exclusion_id")) {
                                    if (!$util.isInteger(m.data_exclusion_id) && !(m.data_exclusion_id && $util.isInteger(m.data_exclusion_id.low) && $util.isInteger(m.data_exclusion_id.high)))
                                        return "data_exclusion_id: integer|Long expected";
                                }
                                if (m.scope != null && m.hasOwnProperty("scope")) {
                                    switch (m.scope) {
                                    default:
                                        return "scope: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    if (!$util.isString(m.start_date_time))
                                        return "start_date_time: string expected";
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    if (!$util.isString(m.end_date_time))
                                        return "end_date_time: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.devices != null && m.hasOwnProperty("devices")) {
                                    if (!Array.isArray(m.devices))
                                        return "devices: array expected";
                                    for (var i = 0; i < m.devices.length; ++i) {
                                        switch (m.devices[i]) {
                                        default:
                                            return "devices: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 6:
                                        case 5:
                                            break;
                                        }
                                    }
                                }
                                if (m.campaigns != null && m.hasOwnProperty("campaigns")) {
                                    if (!Array.isArray(m.campaigns))
                                        return "campaigns: array expected";
                                    for (var i = 0; i < m.campaigns.length; ++i) {
                                        if (!$util.isString(m.campaigns[i]))
                                            return "campaigns: string[] expected";
                                    }
                                }
                                if (m.advertising_channel_types != null && m.hasOwnProperty("advertising_channel_types")) {
                                    if (!Array.isArray(m.advertising_channel_types))
                                        return "advertising_channel_types: array expected";
                                    for (var i = 0; i < m.advertising_channel_types.length; ++i) {
                                        switch (m.advertising_channel_types[i]) {
                                        default:
                                            return "advertising_channel_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                            break;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BiddingDataExclusion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.BiddingDataExclusion} BiddingDataExclusion
                             */
                            BiddingDataExclusion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.BiddingDataExclusion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.BiddingDataExclusion();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.data_exclusion_id != null) {
                                    if ($util.Long)
                                        (m.data_exclusion_id = $util.Long.fromValue(d.data_exclusion_id)).unsigned = false;
                                    else if (typeof d.data_exclusion_id === "string")
                                        m.data_exclusion_id = parseInt(d.data_exclusion_id, 10);
                                    else if (typeof d.data_exclusion_id === "number")
                                        m.data_exclusion_id = d.data_exclusion_id;
                                    else if (typeof d.data_exclusion_id === "object")
                                        m.data_exclusion_id = new $util.LongBits(d.data_exclusion_id.low >>> 0, d.data_exclusion_id.high >>> 0).toNumber();
                                }
                                switch (d.scope) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.scope = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.scope = 1;
                                    break;
                                case "CUSTOMER":
                                case 2:
                                    m.scope = 2;
                                    break;
                                case "CAMPAIGN":
                                case 4:
                                    m.scope = 4;
                                    break;
                                case "CHANNEL":
                                case 5:
                                    m.scope = 5;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.start_date_time != null) {
                                    m.start_date_time = String(d.start_date_time);
                                }
                                if (d.end_date_time != null) {
                                    m.end_date_time = String(d.end_date_time);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.devices) {
                                    if (!Array.isArray(d.devices))
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingDataExclusion.devices: array expected");
                                    m.devices = [];
                                    for (var i = 0; i < d.devices.length; ++i) {
                                        switch (d.devices[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.devices[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.devices[i] = 1;
                                            break;
                                        case "MOBILE":
                                        case 2:
                                            m.devices[i] = 2;
                                            break;
                                        case "TABLET":
                                        case 3:
                                            m.devices[i] = 3;
                                            break;
                                        case "DESKTOP":
                                        case 4:
                                            m.devices[i] = 4;
                                            break;
                                        case "CONNECTED_TV":
                                        case 6:
                                            m.devices[i] = 6;
                                            break;
                                        case "OTHER":
                                        case 5:
                                            m.devices[i] = 5;
                                            break;
                                        }
                                    }
                                }
                                if (d.campaigns) {
                                    if (!Array.isArray(d.campaigns))
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingDataExclusion.campaigns: array expected");
                                    m.campaigns = [];
                                    for (var i = 0; i < d.campaigns.length; ++i) {
                                        m.campaigns[i] = String(d.campaigns[i]);
                                    }
                                }
                                if (d.advertising_channel_types) {
                                    if (!Array.isArray(d.advertising_channel_types))
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingDataExclusion.advertising_channel_types: array expected");
                                    m.advertising_channel_types = [];
                                    for (var i = 0; i < d.advertising_channel_types.length; ++i) {
                                        switch (d.advertising_channel_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.advertising_channel_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.advertising_channel_types[i] = 1;
                                            break;
                                        case "SEARCH":
                                        case 2:
                                            m.advertising_channel_types[i] = 2;
                                            break;
                                        case "DISPLAY":
                                        case 3:
                                            m.advertising_channel_types[i] = 3;
                                            break;
                                        case "SHOPPING":
                                        case 4:
                                            m.advertising_channel_types[i] = 4;
                                            break;
                                        case "HOTEL":
                                        case 5:
                                            m.advertising_channel_types[i] = 5;
                                            break;
                                        case "VIDEO":
                                        case 6:
                                            m.advertising_channel_types[i] = 6;
                                            break;
                                        case "MULTI_CHANNEL":
                                        case 7:
                                            m.advertising_channel_types[i] = 7;
                                            break;
                                        case "LOCAL":
                                        case 8:
                                            m.advertising_channel_types[i] = 8;
                                            break;
                                        case "SMART":
                                        case 9:
                                            m.advertising_channel_types[i] = 9;
                                            break;
                                        case "PERFORMANCE_MAX":
                                        case 10:
                                            m.advertising_channel_types[i] = 10;
                                            break;
                                        case "LOCAL_SERVICES":
                                        case 11:
                                            m.advertising_channel_types[i] = 11;
                                            break;
                                        case "DISCOVERY":
                                        case 12:
                                            m.advertising_channel_types[i] = 12;
                                            break;
                                        }
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BiddingDataExclusion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @static
                             * @param {google.ads.googleads.v11.resources.BiddingDataExclusion} m BiddingDataExclusion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingDataExclusion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.devices = [];
                                    d.campaigns = [];
                                    d.advertising_channel_types = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.data_exclusion_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.data_exclusion_id = o.longs === String ? "0" : 0;
                                    d.scope = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.start_date_time = "";
                                    d.end_date_time = "";
                                    d.name = "";
                                    d.description = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.data_exclusion_id != null && m.hasOwnProperty("data_exclusion_id")) {
                                    if (typeof m.data_exclusion_id === "number")
                                        d.data_exclusion_id = o.longs === String ? String(m.data_exclusion_id) : m.data_exclusion_id;
                                    else
                                        d.data_exclusion_id = o.longs === String ? $util.Long.prototype.toString.call(m.data_exclusion_id) : o.longs === Number ? new $util.LongBits(m.data_exclusion_id.low >>> 0, m.data_exclusion_id.high >>> 0).toNumber() : m.data_exclusion_id;
                                }
                                if (m.scope != null && m.hasOwnProperty("scope")) {
                                    d.scope = o.enums === String ? $root.google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.SeasonalityEventScope[m.scope] : m.scope;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.SeasonalityEventStatus[m.status] : m.status;
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    d.start_date_time = m.start_date_time;
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    d.end_date_time = m.end_date_time;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.devices && m.devices.length) {
                                    d.devices = [];
                                    for (var j = 0; j < m.devices.length; ++j) {
                                        d.devices[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.DeviceEnum.Device[m.devices[j]] : m.devices[j];
                                    }
                                }
                                if (m.campaigns && m.campaigns.length) {
                                    d.campaigns = [];
                                    for (var j = 0; j < m.campaigns.length; ++j) {
                                        d.campaigns[j] = m.campaigns[j];
                                    }
                                }
                                if (m.advertising_channel_types && m.advertising_channel_types.length) {
                                    d.advertising_channel_types = [];
                                    for (var j = 0; j < m.advertising_channel_types.length; ++j) {
                                        d.advertising_channel_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType[m.advertising_channel_types[j]] : m.advertising_channel_types[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BiddingDataExclusion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.BiddingDataExclusion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingDataExclusion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BiddingDataExclusion;
                        })();
    
                        resources.BiddingSeasonalityAdjustment = (function() {
    
                            /**
                             * Properties of a BiddingSeasonalityAdjustment.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IBiddingSeasonalityAdjustment
                             * @property {string|null} [resource_name] BiddingSeasonalityAdjustment resource_name
                             * @property {number|null} [seasonality_adjustment_id] BiddingSeasonalityAdjustment seasonality_adjustment_id
                             * @property {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.SeasonalityEventScope|null} [scope] BiddingSeasonalityAdjustment scope
                             * @property {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.SeasonalityEventStatus|null} [status] BiddingSeasonalityAdjustment status
                             * @property {string|null} [start_date_time] BiddingSeasonalityAdjustment start_date_time
                             * @property {string|null} [end_date_time] BiddingSeasonalityAdjustment end_date_time
                             * @property {string|null} [name] BiddingSeasonalityAdjustment name
                             * @property {string|null} [description] BiddingSeasonalityAdjustment description
                             * @property {Array.<google.ads.googleads.v11.enums.DeviceEnum.Device>|null} [devices] BiddingSeasonalityAdjustment devices
                             * @property {number|null} [conversion_rate_modifier] BiddingSeasonalityAdjustment conversion_rate_modifier
                             * @property {Array.<string>|null} [campaigns] BiddingSeasonalityAdjustment campaigns
                             * @property {Array.<google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType>|null} [advertising_channel_types] BiddingSeasonalityAdjustment advertising_channel_types
                             */
    
                            /**
                             * Constructs a new BiddingSeasonalityAdjustment.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a BiddingSeasonalityAdjustment.
                             * @implements IBiddingSeasonalityAdjustment
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment=} [p] Properties to set
                             */
                            function BiddingSeasonalityAdjustment(p) {
                                this.devices = [];
                                this.campaigns = [];
                                this.advertising_channel_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BiddingSeasonalityAdjustment resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.resource_name = "";
    
                            /**
                             * BiddingSeasonalityAdjustment seasonality_adjustment_id.
                             * @member {number} seasonality_adjustment_id
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.seasonality_adjustment_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BiddingSeasonalityAdjustment scope.
                             * @member {google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.SeasonalityEventScope} scope
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.scope = 0;
    
                            /**
                             * BiddingSeasonalityAdjustment status.
                             * @member {google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.SeasonalityEventStatus} status
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.status = 0;
    
                            /**
                             * BiddingSeasonalityAdjustment start_date_time.
                             * @member {string} start_date_time
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.start_date_time = "";
    
                            /**
                             * BiddingSeasonalityAdjustment end_date_time.
                             * @member {string} end_date_time
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.end_date_time = "";
    
                            /**
                             * BiddingSeasonalityAdjustment name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.name = "";
    
                            /**
                             * BiddingSeasonalityAdjustment description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.description = "";
    
                            /**
                             * BiddingSeasonalityAdjustment devices.
                             * @member {Array.<google.ads.googleads.v11.enums.DeviceEnum.Device>} devices
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.devices = $util.emptyArray;
    
                            /**
                             * BiddingSeasonalityAdjustment conversion_rate_modifier.
                             * @member {number} conversion_rate_modifier
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.conversion_rate_modifier = 0;
    
                            /**
                             * BiddingSeasonalityAdjustment campaigns.
                             * @member {Array.<string>} campaigns
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.campaigns = $util.emptyArray;
    
                            /**
                             * BiddingSeasonalityAdjustment advertising_channel_types.
                             * @member {Array.<google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType>} advertising_channel_types
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             */
                            BiddingSeasonalityAdjustment.prototype.advertising_channel_types = $util.emptyArray;
    
                            /**
                             * Creates a new BiddingSeasonalityAdjustment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment} BiddingSeasonalityAdjustment instance
                             */
                            BiddingSeasonalityAdjustment.create = function create(properties) {
                                return new BiddingSeasonalityAdjustment(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingSeasonalityAdjustment message. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment} m BiddingSeasonalityAdjustment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingSeasonalityAdjustment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.seasonality_adjustment_id != null && Object.hasOwnProperty.call(m, "seasonality_adjustment_id"))
                                    w.uint32(16).int64(m.seasonality_adjustment_id);
                                if (m.scope != null && Object.hasOwnProperty.call(m, "scope"))
                                    w.uint32(24).int32(m.scope);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.start_date_time != null && Object.hasOwnProperty.call(m, "start_date_time"))
                                    w.uint32(42).string(m.start_date_time);
                                if (m.end_date_time != null && Object.hasOwnProperty.call(m, "end_date_time"))
                                    w.uint32(50).string(m.end_date_time);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(58).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(66).string(m.description);
                                if (m.devices != null && m.devices.length) {
                                    w.uint32(74).fork();
                                    for (var i = 0; i < m.devices.length; ++i)
                                        w.int32(m.devices[i]);
                                    w.ldelim();
                                }
                                if (m.conversion_rate_modifier != null && Object.hasOwnProperty.call(m, "conversion_rate_modifier"))
                                    w.uint32(81).double(m.conversion_rate_modifier);
                                if (m.campaigns != null && m.campaigns.length) {
                                    for (var i = 0; i < m.campaigns.length; ++i)
                                        w.uint32(90).string(m.campaigns[i]);
                                }
                                if (m.advertising_channel_types != null && m.advertising_channel_types.length) {
                                    w.uint32(98).fork();
                                    for (var i = 0; i < m.advertising_channel_types.length; ++i)
                                        w.int32(m.advertising_channel_types[i]);
                                    w.ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingSeasonalityAdjustment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment} message BiddingSeasonalityAdjustment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingSeasonalityAdjustment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingSeasonalityAdjustment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment} BiddingSeasonalityAdjustment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingSeasonalityAdjustment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.seasonality_adjustment_id = r.int64();
                                        break;
                                    case 3:
                                        m.scope = r.int32();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.start_date_time = r.string();
                                        break;
                                    case 6:
                                        m.end_date_time = r.string();
                                        break;
                                    case 7:
                                        m.name = r.string();
                                        break;
                                    case 8:
                                        m.description = r.string();
                                        break;
                                    case 9:
                                        if (!(m.devices && m.devices.length))
                                            m.devices = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.devices.push(r.int32());
                                        } else
                                            m.devices.push(r.int32());
                                        break;
                                    case 10:
                                        m.conversion_rate_modifier = r.double();
                                        break;
                                    case 11:
                                        if (!(m.campaigns && m.campaigns.length))
                                            m.campaigns = [];
                                        m.campaigns.push(r.string());
                                        break;
                                    case 12:
                                        if (!(m.advertising_channel_types && m.advertising_channel_types.length))
                                            m.advertising_channel_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.advertising_channel_types.push(r.int32());
                                        } else
                                            m.advertising_channel_types.push(r.int32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingSeasonalityAdjustment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment} BiddingSeasonalityAdjustment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingSeasonalityAdjustment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingSeasonalityAdjustment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingSeasonalityAdjustment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.seasonality_adjustment_id != null && m.hasOwnProperty("seasonality_adjustment_id")) {
                                    if (!$util.isInteger(m.seasonality_adjustment_id) && !(m.seasonality_adjustment_id && $util.isInteger(m.seasonality_adjustment_id.low) && $util.isInteger(m.seasonality_adjustment_id.high)))
                                        return "seasonality_adjustment_id: integer|Long expected";
                                }
                                if (m.scope != null && m.hasOwnProperty("scope")) {
                                    switch (m.scope) {
                                    default:
                                        return "scope: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    if (!$util.isString(m.start_date_time))
                                        return "start_date_time: string expected";
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    if (!$util.isString(m.end_date_time))
                                        return "end_date_time: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.devices != null && m.hasOwnProperty("devices")) {
                                    if (!Array.isArray(m.devices))
                                        return "devices: array expected";
                                    for (var i = 0; i < m.devices.length; ++i) {
                                        switch (m.devices[i]) {
                                        default:
                                            return "devices: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 6:
                                        case 5:
                                            break;
                                        }
                                    }
                                }
                                if (m.conversion_rate_modifier != null && m.hasOwnProperty("conversion_rate_modifier")) {
                                    if (typeof m.conversion_rate_modifier !== "number")
                                        return "conversion_rate_modifier: number expected";
                                }
                                if (m.campaigns != null && m.hasOwnProperty("campaigns")) {
                                    if (!Array.isArray(m.campaigns))
                                        return "campaigns: array expected";
                                    for (var i = 0; i < m.campaigns.length; ++i) {
                                        if (!$util.isString(m.campaigns[i]))
                                            return "campaigns: string[] expected";
                                    }
                                }
                                if (m.advertising_channel_types != null && m.hasOwnProperty("advertising_channel_types")) {
                                    if (!Array.isArray(m.advertising_channel_types))
                                        return "advertising_channel_types: array expected";
                                    for (var i = 0; i < m.advertising_channel_types.length; ++i) {
                                        switch (m.advertising_channel_types[i]) {
                                        default:
                                            return "advertising_channel_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                            break;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BiddingSeasonalityAdjustment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment} BiddingSeasonalityAdjustment
                             */
                            BiddingSeasonalityAdjustment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.seasonality_adjustment_id != null) {
                                    if ($util.Long)
                                        (m.seasonality_adjustment_id = $util.Long.fromValue(d.seasonality_adjustment_id)).unsigned = false;
                                    else if (typeof d.seasonality_adjustment_id === "string")
                                        m.seasonality_adjustment_id = parseInt(d.seasonality_adjustment_id, 10);
                                    else if (typeof d.seasonality_adjustment_id === "number")
                                        m.seasonality_adjustment_id = d.seasonality_adjustment_id;
                                    else if (typeof d.seasonality_adjustment_id === "object")
                                        m.seasonality_adjustment_id = new $util.LongBits(d.seasonality_adjustment_id.low >>> 0, d.seasonality_adjustment_id.high >>> 0).toNumber();
                                }
                                switch (d.scope) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.scope = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.scope = 1;
                                    break;
                                case "CUSTOMER":
                                case 2:
                                    m.scope = 2;
                                    break;
                                case "CAMPAIGN":
                                case 4:
                                    m.scope = 4;
                                    break;
                                case "CHANNEL":
                                case 5:
                                    m.scope = 5;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.start_date_time != null) {
                                    m.start_date_time = String(d.start_date_time);
                                }
                                if (d.end_date_time != null) {
                                    m.end_date_time = String(d.end_date_time);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.devices) {
                                    if (!Array.isArray(d.devices))
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.devices: array expected");
                                    m.devices = [];
                                    for (var i = 0; i < d.devices.length; ++i) {
                                        switch (d.devices[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.devices[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.devices[i] = 1;
                                            break;
                                        case "MOBILE":
                                        case 2:
                                            m.devices[i] = 2;
                                            break;
                                        case "TABLET":
                                        case 3:
                                            m.devices[i] = 3;
                                            break;
                                        case "DESKTOP":
                                        case 4:
                                            m.devices[i] = 4;
                                            break;
                                        case "CONNECTED_TV":
                                        case 6:
                                            m.devices[i] = 6;
                                            break;
                                        case "OTHER":
                                        case 5:
                                            m.devices[i] = 5;
                                            break;
                                        }
                                    }
                                }
                                if (d.conversion_rate_modifier != null) {
                                    m.conversion_rate_modifier = Number(d.conversion_rate_modifier);
                                }
                                if (d.campaigns) {
                                    if (!Array.isArray(d.campaigns))
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.campaigns: array expected");
                                    m.campaigns = [];
                                    for (var i = 0; i < d.campaigns.length; ++i) {
                                        m.campaigns[i] = String(d.campaigns[i]);
                                    }
                                }
                                if (d.advertising_channel_types) {
                                    if (!Array.isArray(d.advertising_channel_types))
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.advertising_channel_types: array expected");
                                    m.advertising_channel_types = [];
                                    for (var i = 0; i < d.advertising_channel_types.length; ++i) {
                                        switch (d.advertising_channel_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.advertising_channel_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.advertising_channel_types[i] = 1;
                                            break;
                                        case "SEARCH":
                                        case 2:
                                            m.advertising_channel_types[i] = 2;
                                            break;
                                        case "DISPLAY":
                                        case 3:
                                            m.advertising_channel_types[i] = 3;
                                            break;
                                        case "SHOPPING":
                                        case 4:
                                            m.advertising_channel_types[i] = 4;
                                            break;
                                        case "HOTEL":
                                        case 5:
                                            m.advertising_channel_types[i] = 5;
                                            break;
                                        case "VIDEO":
                                        case 6:
                                            m.advertising_channel_types[i] = 6;
                                            break;
                                        case "MULTI_CHANNEL":
                                        case 7:
                                            m.advertising_channel_types[i] = 7;
                                            break;
                                        case "LOCAL":
                                        case 8:
                                            m.advertising_channel_types[i] = 8;
                                            break;
                                        case "SMART":
                                        case 9:
                                            m.advertising_channel_types[i] = 9;
                                            break;
                                        case "PERFORMANCE_MAX":
                                        case 10:
                                            m.advertising_channel_types[i] = 10;
                                            break;
                                        case "LOCAL_SERVICES":
                                        case 11:
                                            m.advertising_channel_types[i] = 11;
                                            break;
                                        case "DISCOVERY":
                                        case 12:
                                            m.advertising_channel_types[i] = 12;
                                            break;
                                        }
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BiddingSeasonalityAdjustment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment} m BiddingSeasonalityAdjustment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingSeasonalityAdjustment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.devices = [];
                                    d.campaigns = [];
                                    d.advertising_channel_types = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.seasonality_adjustment_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.seasonality_adjustment_id = o.longs === String ? "0" : 0;
                                    d.scope = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.start_date_time = "";
                                    d.end_date_time = "";
                                    d.name = "";
                                    d.description = "";
                                    d.conversion_rate_modifier = 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.seasonality_adjustment_id != null && m.hasOwnProperty("seasonality_adjustment_id")) {
                                    if (typeof m.seasonality_adjustment_id === "number")
                                        d.seasonality_adjustment_id = o.longs === String ? String(m.seasonality_adjustment_id) : m.seasonality_adjustment_id;
                                    else
                                        d.seasonality_adjustment_id = o.longs === String ? $util.Long.prototype.toString.call(m.seasonality_adjustment_id) : o.longs === Number ? new $util.LongBits(m.seasonality_adjustment_id.low >>> 0, m.seasonality_adjustment_id.high >>> 0).toNumber() : m.seasonality_adjustment_id;
                                }
                                if (m.scope != null && m.hasOwnProperty("scope")) {
                                    d.scope = o.enums === String ? $root.google.ads.googleads.v11.enums.SeasonalityEventScopeEnum.SeasonalityEventScope[m.scope] : m.scope;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.SeasonalityEventStatusEnum.SeasonalityEventStatus[m.status] : m.status;
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    d.start_date_time = m.start_date_time;
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    d.end_date_time = m.end_date_time;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.devices && m.devices.length) {
                                    d.devices = [];
                                    for (var j = 0; j < m.devices.length; ++j) {
                                        d.devices[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.DeviceEnum.Device[m.devices[j]] : m.devices[j];
                                    }
                                }
                                if (m.conversion_rate_modifier != null && m.hasOwnProperty("conversion_rate_modifier")) {
                                    d.conversion_rate_modifier = o.json && !isFinite(m.conversion_rate_modifier) ? String(m.conversion_rate_modifier) : m.conversion_rate_modifier;
                                }
                                if (m.campaigns && m.campaigns.length) {
                                    d.campaigns = [];
                                    for (var j = 0; j < m.campaigns.length; ++j) {
                                        d.campaigns[j] = m.campaigns[j];
                                    }
                                }
                                if (m.advertising_channel_types && m.advertising_channel_types.length) {
                                    d.advertising_channel_types = [];
                                    for (var j = 0; j < m.advertising_channel_types.length; ++j) {
                                        d.advertising_channel_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType[m.advertising_channel_types[j]] : m.advertising_channel_types[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BiddingSeasonalityAdjustment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingSeasonalityAdjustment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BiddingSeasonalityAdjustment;
                        })();
    
                        resources.BiddingStrategy = (function() {
    
                            /**
                             * Properties of a BiddingStrategy.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IBiddingStrategy
                             * @property {string|null} [resource_name] BiddingStrategy resource_name
                             * @property {number|null} [id] BiddingStrategy id
                             * @property {string|null} [name] BiddingStrategy name
                             * @property {google.ads.googleads.v11.enums.BiddingStrategyStatusEnum.BiddingStrategyStatus|null} [status] BiddingStrategy status
                             * @property {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType|null} [type] BiddingStrategy type
                             * @property {string|null} [currency_code] BiddingStrategy currency_code
                             * @property {string|null} [effective_currency_code] BiddingStrategy effective_currency_code
                             * @property {number|null} [campaign_count] BiddingStrategy campaign_count
                             * @property {number|null} [non_removed_campaign_count] BiddingStrategy non_removed_campaign_count
                             * @property {google.ads.googleads.v11.common.IEnhancedCpc|null} [enhanced_cpc] BiddingStrategy enhanced_cpc
                             * @property {google.ads.googleads.v11.common.IMaximizeConversionValue|null} [maximize_conversion_value] BiddingStrategy maximize_conversion_value
                             * @property {google.ads.googleads.v11.common.IMaximizeConversions|null} [maximize_conversions] BiddingStrategy maximize_conversions
                             * @property {google.ads.googleads.v11.common.ITargetCpa|null} [target_cpa] BiddingStrategy target_cpa
                             * @property {google.ads.googleads.v11.common.ITargetImpressionShare|null} [target_impression_share] BiddingStrategy target_impression_share
                             * @property {google.ads.googleads.v11.common.ITargetRoas|null} [target_roas] BiddingStrategy target_roas
                             * @property {google.ads.googleads.v11.common.ITargetSpend|null} [target_spend] BiddingStrategy target_spend
                             */
    
                            /**
                             * Constructs a new BiddingStrategy.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a BiddingStrategy.
                             * @implements IBiddingStrategy
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategy=} [p] Properties to set
                             */
                            function BiddingStrategy(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BiddingStrategy resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.resource_name = "";
    
                            /**
                             * BiddingStrategy id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.id = null;
    
                            /**
                             * BiddingStrategy name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.name = null;
    
                            /**
                             * BiddingStrategy status.
                             * @member {google.ads.googleads.v11.enums.BiddingStrategyStatusEnum.BiddingStrategyStatus} status
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.status = 0;
    
                            /**
                             * BiddingStrategy type.
                             * @member {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType} type
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.type = 0;
    
                            /**
                             * BiddingStrategy currency_code.
                             * @member {string} currency_code
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.currency_code = "";
    
                            /**
                             * BiddingStrategy effective_currency_code.
                             * @member {string|null|undefined} effective_currency_code
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.effective_currency_code = null;
    
                            /**
                             * BiddingStrategy campaign_count.
                             * @member {number|null|undefined} campaign_count
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.campaign_count = null;
    
                            /**
                             * BiddingStrategy non_removed_campaign_count.
                             * @member {number|null|undefined} non_removed_campaign_count
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.non_removed_campaign_count = null;
    
                            /**
                             * BiddingStrategy enhanced_cpc.
                             * @member {google.ads.googleads.v11.common.IEnhancedCpc|null|undefined} enhanced_cpc
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.enhanced_cpc = null;
    
                            /**
                             * BiddingStrategy maximize_conversion_value.
                             * @member {google.ads.googleads.v11.common.IMaximizeConversionValue|null|undefined} maximize_conversion_value
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.maximize_conversion_value = null;
    
                            /**
                             * BiddingStrategy maximize_conversions.
                             * @member {google.ads.googleads.v11.common.IMaximizeConversions|null|undefined} maximize_conversions
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.maximize_conversions = null;
    
                            /**
                             * BiddingStrategy target_cpa.
                             * @member {google.ads.googleads.v11.common.ITargetCpa|null|undefined} target_cpa
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.target_cpa = null;
    
                            /**
                             * BiddingStrategy target_impression_share.
                             * @member {google.ads.googleads.v11.common.ITargetImpressionShare|null|undefined} target_impression_share
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.target_impression_share = null;
    
                            /**
                             * BiddingStrategy target_roas.
                             * @member {google.ads.googleads.v11.common.ITargetRoas|null|undefined} target_roas
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.target_roas = null;
    
                            /**
                             * BiddingStrategy target_spend.
                             * @member {google.ads.googleads.v11.common.ITargetSpend|null|undefined} target_spend
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            BiddingStrategy.prototype.target_spend = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BiddingStrategy _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategy.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BiddingStrategy _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategy.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BiddingStrategy _effective_currency_code.
                             * @member {"effective_currency_code"|undefined} _effective_currency_code
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategy.prototype, "_effective_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BiddingStrategy _campaign_count.
                             * @member {"campaign_count"|undefined} _campaign_count
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategy.prototype, "_campaign_count", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_count"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BiddingStrategy _non_removed_campaign_count.
                             * @member {"non_removed_campaign_count"|undefined} _non_removed_campaign_count
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategy.prototype, "_non_removed_campaign_count", {
                                get: $util.oneOfGetter($oneOfFields = ["non_removed_campaign_count"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BiddingStrategy scheme.
                             * @member {"enhanced_cpc"|"maximize_conversion_value"|"maximize_conversions"|"target_cpa"|"target_impression_share"|"target_roas"|"target_spend"|undefined} scheme
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategy.prototype, "scheme", {
                                get: $util.oneOfGetter($oneOfFields = ["enhanced_cpc", "maximize_conversion_value", "maximize_conversions", "target_cpa", "target_impression_share", "target_roas", "target_spend"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BiddingStrategy instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategy=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategy} BiddingStrategy instance
                             */
                            BiddingStrategy.create = function create(properties) {
                                return new BiddingStrategy(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingStrategy message. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingStrategy.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategy} m BiddingStrategy message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategy.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(40).int32(m.type);
                                if (m.enhanced_cpc != null && Object.hasOwnProperty.call(m, "enhanced_cpc"))
                                    $root.google.ads.googleads.v11.common.EnhancedCpc.encode(m.enhanced_cpc, w.uint32(58).fork()).ldelim();
                                if (m.target_cpa != null && Object.hasOwnProperty.call(m, "target_cpa"))
                                    $root.google.ads.googleads.v11.common.TargetCpa.encode(m.target_cpa, w.uint32(74).fork()).ldelim();
                                if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                    $root.google.ads.googleads.v11.common.TargetRoas.encode(m.target_roas, w.uint32(90).fork()).ldelim();
                                if (m.target_spend != null && Object.hasOwnProperty.call(m, "target_spend"))
                                    $root.google.ads.googleads.v11.common.TargetSpend.encode(m.target_spend, w.uint32(98).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(120).int32(m.status);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(128).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(138).string(m.name);
                                if (m.campaign_count != null && Object.hasOwnProperty.call(m, "campaign_count"))
                                    w.uint32(144).int64(m.campaign_count);
                                if (m.non_removed_campaign_count != null && Object.hasOwnProperty.call(m, "non_removed_campaign_count"))
                                    w.uint32(152).int64(m.non_removed_campaign_count);
                                if (m.effective_currency_code != null && Object.hasOwnProperty.call(m, "effective_currency_code"))
                                    w.uint32(162).string(m.effective_currency_code);
                                if (m.maximize_conversion_value != null && Object.hasOwnProperty.call(m, "maximize_conversion_value"))
                                    $root.google.ads.googleads.v11.common.MaximizeConversionValue.encode(m.maximize_conversion_value, w.uint32(170).fork()).ldelim();
                                if (m.maximize_conversions != null && Object.hasOwnProperty.call(m, "maximize_conversions"))
                                    $root.google.ads.googleads.v11.common.MaximizeConversions.encode(m.maximize_conversions, w.uint32(178).fork()).ldelim();
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(186).string(m.currency_code);
                                if (m.target_impression_share != null && Object.hasOwnProperty.call(m, "target_impression_share"))
                                    $root.google.ads.googleads.v11.common.TargetImpressionShare.encode(m.target_impression_share, w.uint32(386).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingStrategy message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingStrategy.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategy} message BiddingStrategy message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategy.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingStrategy message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategy} BiddingStrategy
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategy.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BiddingStrategy();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 16:
                                        m.id = r.int64();
                                        break;
                                    case 17:
                                        m.name = r.string();
                                        break;
                                    case 15:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.type = r.int32();
                                        break;
                                    case 23:
                                        m.currency_code = r.string();
                                        break;
                                    case 20:
                                        m.effective_currency_code = r.string();
                                        break;
                                    case 18:
                                        m.campaign_count = r.int64();
                                        break;
                                    case 19:
                                        m.non_removed_campaign_count = r.int64();
                                        break;
                                    case 7:
                                        m.enhanced_cpc = $root.google.ads.googleads.v11.common.EnhancedCpc.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.maximize_conversion_value = $root.google.ads.googleads.v11.common.MaximizeConversionValue.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.maximize_conversions = $root.google.ads.googleads.v11.common.MaximizeConversions.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.target_cpa = $root.google.ads.googleads.v11.common.TargetCpa.decode(r, r.uint32());
                                        break;
                                    case 48:
                                        m.target_impression_share = $root.google.ads.googleads.v11.common.TargetImpressionShare.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.target_roas = $root.google.ads.googleads.v11.common.TargetRoas.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.target_spend = $root.google.ads.googleads.v11.common.TargetSpend.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingStrategy message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategy} BiddingStrategy
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategy.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingStrategy message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingStrategy.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 16:
                                    case 2:
                                    case 17:
                                    case 18:
                                    case 3:
                                    case 4:
                                    case 13:
                                    case 10:
                                    case 11:
                                    case 5:
                                    case 12:
                                    case 6:
                                    case 14:
                                    case 15:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.effective_currency_code != null && m.hasOwnProperty("effective_currency_code")) {
                                    p._effective_currency_code = 1;
                                    if (!$util.isString(m.effective_currency_code))
                                        return "effective_currency_code: string expected";
                                }
                                if (m.campaign_count != null && m.hasOwnProperty("campaign_count")) {
                                    p._campaign_count = 1;
                                    if (!$util.isInteger(m.campaign_count) && !(m.campaign_count && $util.isInteger(m.campaign_count.low) && $util.isInteger(m.campaign_count.high)))
                                        return "campaign_count: integer|Long expected";
                                }
                                if (m.non_removed_campaign_count != null && m.hasOwnProperty("non_removed_campaign_count")) {
                                    p._non_removed_campaign_count = 1;
                                    if (!$util.isInteger(m.non_removed_campaign_count) && !(m.non_removed_campaign_count && $util.isInteger(m.non_removed_campaign_count.low) && $util.isInteger(m.non_removed_campaign_count.high)))
                                        return "non_removed_campaign_count: integer|Long expected";
                                }
                                if (m.enhanced_cpc != null && m.hasOwnProperty("enhanced_cpc")) {
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.EnhancedCpc.verify(m.enhanced_cpc);
                                        if (e)
                                            return "enhanced_cpc." + e;
                                    }
                                }
                                if (m.maximize_conversion_value != null && m.hasOwnProperty("maximize_conversion_value")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MaximizeConversionValue.verify(m.maximize_conversion_value);
                                        if (e)
                                            return "maximize_conversion_value." + e;
                                    }
                                }
                                if (m.maximize_conversions != null && m.hasOwnProperty("maximize_conversions")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MaximizeConversions.verify(m.maximize_conversions);
                                        if (e)
                                            return "maximize_conversions." + e;
                                    }
                                }
                                if (m.target_cpa != null && m.hasOwnProperty("target_cpa")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetCpa.verify(m.target_cpa);
                                        if (e)
                                            return "target_cpa." + e;
                                    }
                                }
                                if (m.target_impression_share != null && m.hasOwnProperty("target_impression_share")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetImpressionShare.verify(m.target_impression_share);
                                        if (e)
                                            return "target_impression_share." + e;
                                    }
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetRoas.verify(m.target_roas);
                                        if (e)
                                            return "target_roas." + e;
                                    }
                                }
                                if (m.target_spend != null && m.hasOwnProperty("target_spend")) {
                                    if (p.scheme === 1)
                                        return "scheme: multiple values";
                                    p.scheme = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetSpend.verify(m.target_spend);
                                        if (e)
                                            return "target_spend." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BiddingStrategy message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategy} BiddingStrategy
                             */
                            BiddingStrategy.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.BiddingStrategy)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.BiddingStrategy();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "COMMISSION":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "ENHANCED_CPC":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "INVALID":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "MANUAL_CPA":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "MANUAL_CPC":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MANUAL_CPM":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "MANUAL_CPV":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "MAXIMIZE_CONVERSIONS":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "MAXIMIZE_CONVERSION_VALUE":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "PAGE_ONE_PROMOTED":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "PERCENT_CPC":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "TARGET_CPA":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "TARGET_CPM":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "TARGET_OUTRANK_SHARE":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TARGET_ROAS":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "TARGET_SPEND":
                                case 9:
                                    m.type = 9;
                                    break;
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.effective_currency_code != null) {
                                    m.effective_currency_code = String(d.effective_currency_code);
                                }
                                if (d.campaign_count != null) {
                                    if ($util.Long)
                                        (m.campaign_count = $util.Long.fromValue(d.campaign_count)).unsigned = false;
                                    else if (typeof d.campaign_count === "string")
                                        m.campaign_count = parseInt(d.campaign_count, 10);
                                    else if (typeof d.campaign_count === "number")
                                        m.campaign_count = d.campaign_count;
                                    else if (typeof d.campaign_count === "object")
                                        m.campaign_count = new $util.LongBits(d.campaign_count.low >>> 0, d.campaign_count.high >>> 0).toNumber();
                                }
                                if (d.non_removed_campaign_count != null) {
                                    if ($util.Long)
                                        (m.non_removed_campaign_count = $util.Long.fromValue(d.non_removed_campaign_count)).unsigned = false;
                                    else if (typeof d.non_removed_campaign_count === "string")
                                        m.non_removed_campaign_count = parseInt(d.non_removed_campaign_count, 10);
                                    else if (typeof d.non_removed_campaign_count === "number")
                                        m.non_removed_campaign_count = d.non_removed_campaign_count;
                                    else if (typeof d.non_removed_campaign_count === "object")
                                        m.non_removed_campaign_count = new $util.LongBits(d.non_removed_campaign_count.low >>> 0, d.non_removed_campaign_count.high >>> 0).toNumber();
                                }
                                if (d.enhanced_cpc != null) {
                                    if (typeof d.enhanced_cpc !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategy.enhanced_cpc: object expected");
                                    m.enhanced_cpc = $root.google.ads.googleads.v11.common.EnhancedCpc.fromObject(d.enhanced_cpc);
                                }
                                if (d.maximize_conversion_value != null) {
                                    if (typeof d.maximize_conversion_value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategy.maximize_conversion_value: object expected");
                                    m.maximize_conversion_value = $root.google.ads.googleads.v11.common.MaximizeConversionValue.fromObject(d.maximize_conversion_value);
                                }
                                if (d.maximize_conversions != null) {
                                    if (typeof d.maximize_conversions !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategy.maximize_conversions: object expected");
                                    m.maximize_conversions = $root.google.ads.googleads.v11.common.MaximizeConversions.fromObject(d.maximize_conversions);
                                }
                                if (d.target_cpa != null) {
                                    if (typeof d.target_cpa !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategy.target_cpa: object expected");
                                    m.target_cpa = $root.google.ads.googleads.v11.common.TargetCpa.fromObject(d.target_cpa);
                                }
                                if (d.target_impression_share != null) {
                                    if (typeof d.target_impression_share !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategy.target_impression_share: object expected");
                                    m.target_impression_share = $root.google.ads.googleads.v11.common.TargetImpressionShare.fromObject(d.target_impression_share);
                                }
                                if (d.target_roas != null) {
                                    if (typeof d.target_roas !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategy.target_roas: object expected");
                                    m.target_roas = $root.google.ads.googleads.v11.common.TargetRoas.fromObject(d.target_roas);
                                }
                                if (d.target_spend != null) {
                                    if (typeof d.target_spend !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategy.target_spend: object expected");
                                    m.target_spend = $root.google.ads.googleads.v11.common.TargetSpend.fromObject(d.target_spend);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BiddingStrategy message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @static
                             * @param {google.ads.googleads.v11.resources.BiddingStrategy} m BiddingStrategy
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingStrategy.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.currency_code = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType[m.type] : m.type;
                                }
                                if (m.enhanced_cpc != null && m.hasOwnProperty("enhanced_cpc")) {
                                    d.enhanced_cpc = $root.google.ads.googleads.v11.common.EnhancedCpc.toObject(m.enhanced_cpc, o);
                                    if (o.oneofs)
                                        d.scheme = "enhanced_cpc";
                                }
                                if (m.target_cpa != null && m.hasOwnProperty("target_cpa")) {
                                    d.target_cpa = $root.google.ads.googleads.v11.common.TargetCpa.toObject(m.target_cpa, o);
                                    if (o.oneofs)
                                        d.scheme = "target_cpa";
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    d.target_roas = $root.google.ads.googleads.v11.common.TargetRoas.toObject(m.target_roas, o);
                                    if (o.oneofs)
                                        d.scheme = "target_roas";
                                }
                                if (m.target_spend != null && m.hasOwnProperty("target_spend")) {
                                    d.target_spend = $root.google.ads.googleads.v11.common.TargetSpend.toObject(m.target_spend, o);
                                    if (o.oneofs)
                                        d.scheme = "target_spend";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingStrategyStatusEnum.BiddingStrategyStatus[m.status] : m.status;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.campaign_count != null && m.hasOwnProperty("campaign_count")) {
                                    if (typeof m.campaign_count === "number")
                                        d.campaign_count = o.longs === String ? String(m.campaign_count) : m.campaign_count;
                                    else
                                        d.campaign_count = o.longs === String ? $util.Long.prototype.toString.call(m.campaign_count) : o.longs === Number ? new $util.LongBits(m.campaign_count.low >>> 0, m.campaign_count.high >>> 0).toNumber() : m.campaign_count;
                                    if (o.oneofs)
                                        d._campaign_count = "campaign_count";
                                }
                                if (m.non_removed_campaign_count != null && m.hasOwnProperty("non_removed_campaign_count")) {
                                    if (typeof m.non_removed_campaign_count === "number")
                                        d.non_removed_campaign_count = o.longs === String ? String(m.non_removed_campaign_count) : m.non_removed_campaign_count;
                                    else
                                        d.non_removed_campaign_count = o.longs === String ? $util.Long.prototype.toString.call(m.non_removed_campaign_count) : o.longs === Number ? new $util.LongBits(m.non_removed_campaign_count.low >>> 0, m.non_removed_campaign_count.high >>> 0).toNumber() : m.non_removed_campaign_count;
                                    if (o.oneofs)
                                        d._non_removed_campaign_count = "non_removed_campaign_count";
                                }
                                if (m.effective_currency_code != null && m.hasOwnProperty("effective_currency_code")) {
                                    d.effective_currency_code = m.effective_currency_code;
                                    if (o.oneofs)
                                        d._effective_currency_code = "effective_currency_code";
                                }
                                if (m.maximize_conversion_value != null && m.hasOwnProperty("maximize_conversion_value")) {
                                    d.maximize_conversion_value = $root.google.ads.googleads.v11.common.MaximizeConversionValue.toObject(m.maximize_conversion_value, o);
                                    if (o.oneofs)
                                        d.scheme = "maximize_conversion_value";
                                }
                                if (m.maximize_conversions != null && m.hasOwnProperty("maximize_conversions")) {
                                    d.maximize_conversions = $root.google.ads.googleads.v11.common.MaximizeConversions.toObject(m.maximize_conversions, o);
                                    if (o.oneofs)
                                        d.scheme = "maximize_conversions";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                }
                                if (m.target_impression_share != null && m.hasOwnProperty("target_impression_share")) {
                                    d.target_impression_share = $root.google.ads.googleads.v11.common.TargetImpressionShare.toObject(m.target_impression_share, o);
                                    if (o.oneofs)
                                        d.scheme = "target_impression_share";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BiddingStrategy to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategy
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingStrategy.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BiddingStrategy;
                        })();
    
                        resources.BiddingStrategySimulation = (function() {
    
                            /**
                             * Properties of a BiddingStrategySimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IBiddingStrategySimulation
                             * @property {string|null} [resource_name] BiddingStrategySimulation resource_name
                             * @property {number|null} [bidding_strategy_id] BiddingStrategySimulation bidding_strategy_id
                             * @property {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType|null} [type] BiddingStrategySimulation type
                             * @property {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod|null} [modification_method] BiddingStrategySimulation modification_method
                             * @property {string|null} [start_date] BiddingStrategySimulation start_date
                             * @property {string|null} [end_date] BiddingStrategySimulation end_date
                             * @property {google.ads.googleads.v11.common.ITargetCpaSimulationPointList|null} [target_cpa_point_list] BiddingStrategySimulation target_cpa_point_list
                             * @property {google.ads.googleads.v11.common.ITargetRoasSimulationPointList|null} [target_roas_point_list] BiddingStrategySimulation target_roas_point_list
                             */
    
                            /**
                             * Constructs a new BiddingStrategySimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a BiddingStrategySimulation.
                             * @implements IBiddingStrategySimulation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategySimulation=} [p] Properties to set
                             */
                            function BiddingStrategySimulation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BiddingStrategySimulation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.resource_name = "";
    
                            /**
                             * BiddingStrategySimulation bidding_strategy_id.
                             * @member {number} bidding_strategy_id
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.bidding_strategy_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BiddingStrategySimulation type.
                             * @member {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType} type
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.type = 0;
    
                            /**
                             * BiddingStrategySimulation modification_method.
                             * @member {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod} modification_method
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.modification_method = 0;
    
                            /**
                             * BiddingStrategySimulation start_date.
                             * @member {string} start_date
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.start_date = "";
    
                            /**
                             * BiddingStrategySimulation end_date.
                             * @member {string} end_date
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.end_date = "";
    
                            /**
                             * BiddingStrategySimulation target_cpa_point_list.
                             * @member {google.ads.googleads.v11.common.ITargetCpaSimulationPointList|null|undefined} target_cpa_point_list
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.target_cpa_point_list = null;
    
                            /**
                             * BiddingStrategySimulation target_roas_point_list.
                             * @member {google.ads.googleads.v11.common.ITargetRoasSimulationPointList|null|undefined} target_roas_point_list
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            BiddingStrategySimulation.prototype.target_roas_point_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BiddingStrategySimulation point_list.
                             * @member {"target_cpa_point_list"|"target_roas_point_list"|undefined} point_list
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategySimulation.prototype, "point_list", {
                                get: $util.oneOfGetter($oneOfFields = ["target_cpa_point_list", "target_roas_point_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BiddingStrategySimulation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategySimulation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategySimulation} BiddingStrategySimulation instance
                             */
                            BiddingStrategySimulation.create = function create(properties) {
                                return new BiddingStrategySimulation(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingStrategySimulation message. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingStrategySimulation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategySimulation} m BiddingStrategySimulation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategySimulation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.bidding_strategy_id != null && Object.hasOwnProperty.call(m, "bidding_strategy_id"))
                                    w.uint32(16).int64(m.bidding_strategy_id);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.modification_method != null && Object.hasOwnProperty.call(m, "modification_method"))
                                    w.uint32(32).int32(m.modification_method);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(42).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(50).string(m.end_date);
                                if (m.target_cpa_point_list != null && Object.hasOwnProperty.call(m, "target_cpa_point_list"))
                                    $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.encode(m.target_cpa_point_list, w.uint32(58).fork()).ldelim();
                                if (m.target_roas_point_list != null && Object.hasOwnProperty.call(m, "target_roas_point_list"))
                                    $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.encode(m.target_roas_point_list, w.uint32(66).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingStrategySimulation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BiddingStrategySimulation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBiddingStrategySimulation} message BiddingStrategySimulation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategySimulation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingStrategySimulation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategySimulation} BiddingStrategySimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategySimulation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BiddingStrategySimulation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.bidding_strategy_id = r.int64();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 4:
                                        m.modification_method = r.int32();
                                        break;
                                    case 5:
                                        m.start_date = r.string();
                                        break;
                                    case 6:
                                        m.end_date = r.string();
                                        break;
                                    case 7:
                                        m.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingStrategySimulation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategySimulation} BiddingStrategySimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategySimulation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingStrategySimulation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingStrategySimulation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.bidding_strategy_id != null && m.hasOwnProperty("bidding_strategy_id")) {
                                    if (!$util.isInteger(m.bidding_strategy_id) && !(m.bidding_strategy_id && $util.isInteger(m.bidding_strategy_id.low) && $util.isInteger(m.bidding_strategy_id.high)))
                                        return "bidding_strategy_id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    switch (m.modification_method) {
                                    default:
                                        return "modification_method: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.target_cpa_point_list != null && m.hasOwnProperty("target_cpa_point_list")) {
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.verify(m.target_cpa_point_list);
                                        if (e)
                                            return "target_cpa_point_list." + e;
                                    }
                                }
                                if (m.target_roas_point_list != null && m.hasOwnProperty("target_roas_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.verify(m.target_roas_point_list);
                                        if (e)
                                            return "target_roas_point_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BiddingStrategySimulation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.BiddingStrategySimulation} BiddingStrategySimulation
                             */
                            BiddingStrategySimulation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.BiddingStrategySimulation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.BiddingStrategySimulation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.bidding_strategy_id != null) {
                                    if ($util.Long)
                                        (m.bidding_strategy_id = $util.Long.fromValue(d.bidding_strategy_id)).unsigned = false;
                                    else if (typeof d.bidding_strategy_id === "string")
                                        m.bidding_strategy_id = parseInt(d.bidding_strategy_id, 10);
                                    else if (typeof d.bidding_strategy_id === "number")
                                        m.bidding_strategy_id = d.bidding_strategy_id;
                                    else if (typeof d.bidding_strategy_id === "object")
                                        m.bidding_strategy_id = new $util.LongBits(d.bidding_strategy_id.low >>> 0, d.bidding_strategy_id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CPC_BID":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "CPV_BID":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "TARGET_CPA":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "BID_MODIFIER":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "TARGET_ROAS":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PERCENT_CPC_BID":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "BUDGET":
                                case 9:
                                    m.type = 9;
                                    break;
                                }
                                switch (d.modification_method) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.modification_method = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.modification_method = 1;
                                    break;
                                case "UNIFORM":
                                case 2:
                                    m.modification_method = 2;
                                    break;
                                case "DEFAULT":
                                case 3:
                                    m.modification_method = 3;
                                    break;
                                case "SCALING":
                                case 4:
                                    m.modification_method = 4;
                                    break;
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.target_cpa_point_list != null) {
                                    if (typeof d.target_cpa_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategySimulation.target_cpa_point_list: object expected");
                                    m.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.fromObject(d.target_cpa_point_list);
                                }
                                if (d.target_roas_point_list != null) {
                                    if (typeof d.target_roas_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BiddingStrategySimulation.target_roas_point_list: object expected");
                                    m.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.fromObject(d.target_roas_point_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BiddingStrategySimulation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.BiddingStrategySimulation} m BiddingStrategySimulation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingStrategySimulation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.bidding_strategy_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.bidding_strategy_id = o.longs === String ? "0" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.modification_method = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.start_date = "";
                                    d.end_date = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.bidding_strategy_id != null && m.hasOwnProperty("bidding_strategy_id")) {
                                    if (typeof m.bidding_strategy_id === "number")
                                        d.bidding_strategy_id = o.longs === String ? String(m.bidding_strategy_id) : m.bidding_strategy_id;
                                    else
                                        d.bidding_strategy_id = o.longs === String ? $util.Long.prototype.toString.call(m.bidding_strategy_id) : o.longs === Number ? new $util.LongBits(m.bidding_strategy_id.low >>> 0, m.bidding_strategy_id.high >>> 0).toNumber() : m.bidding_strategy_id;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType[m.type] : m.type;
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    d.modification_method = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod[m.modification_method] : m.modification_method;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                }
                                if (m.target_cpa_point_list != null && m.hasOwnProperty("target_cpa_point_list")) {
                                    d.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.toObject(m.target_cpa_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "target_cpa_point_list";
                                }
                                if (m.target_roas_point_list != null && m.hasOwnProperty("target_roas_point_list")) {
                                    d.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.toObject(m.target_roas_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "target_roas_point_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BiddingStrategySimulation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.BiddingStrategySimulation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingStrategySimulation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BiddingStrategySimulation;
                        })();
    
                        resources.BillingSetup = (function() {
    
                            /**
                             * Properties of a BillingSetup.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IBillingSetup
                             * @property {string|null} [resource_name] BillingSetup resource_name
                             * @property {number|null} [id] BillingSetup id
                             * @property {google.ads.googleads.v11.enums.BillingSetupStatusEnum.BillingSetupStatus|null} [status] BillingSetup status
                             * @property {string|null} [payments_account] BillingSetup payments_account
                             * @property {google.ads.googleads.v11.resources.BillingSetup.IPaymentsAccountInfo|null} [payments_account_info] BillingSetup payments_account_info
                             * @property {string|null} [start_date_time] BillingSetup start_date_time
                             * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [start_time_type] BillingSetup start_time_type
                             * @property {string|null} [end_date_time] BillingSetup end_date_time
                             * @property {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null} [end_time_type] BillingSetup end_time_type
                             */
    
                            /**
                             * Constructs a new BillingSetup.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a BillingSetup.
                             * @implements IBillingSetup
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IBillingSetup=} [p] Properties to set
                             */
                            function BillingSetup(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BillingSetup resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.resource_name = "";
    
                            /**
                             * BillingSetup id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.id = null;
    
                            /**
                             * BillingSetup status.
                             * @member {google.ads.googleads.v11.enums.BillingSetupStatusEnum.BillingSetupStatus} status
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.status = 0;
    
                            /**
                             * BillingSetup payments_account.
                             * @member {string|null|undefined} payments_account
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.payments_account = null;
    
                            /**
                             * BillingSetup payments_account_info.
                             * @member {google.ads.googleads.v11.resources.BillingSetup.IPaymentsAccountInfo|null|undefined} payments_account_info
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.payments_account_info = null;
    
                            /**
                             * BillingSetup start_date_time.
                             * @member {string|null|undefined} start_date_time
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.start_date_time = null;
    
                            /**
                             * BillingSetup start_time_type.
                             * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} start_time_type
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.start_time_type = null;
    
                            /**
                             * BillingSetup end_date_time.
                             * @member {string|null|undefined} end_date_time
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.end_date_time = null;
    
                            /**
                             * BillingSetup end_time_type.
                             * @member {google.ads.googleads.v11.enums.TimeTypeEnum.TimeType|null|undefined} end_time_type
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            BillingSetup.prototype.end_time_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BillingSetup _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            Object.defineProperty(BillingSetup.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BillingSetup _payments_account.
                             * @member {"payments_account"|undefined} _payments_account
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            Object.defineProperty(BillingSetup.prototype, "_payments_account", {
                                get: $util.oneOfGetter($oneOfFields = ["payments_account"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BillingSetup start_time.
                             * @member {"start_date_time"|"start_time_type"|undefined} start_time
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            Object.defineProperty(BillingSetup.prototype, "start_time", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date_time", "start_time_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * BillingSetup end_time.
                             * @member {"end_date_time"|"end_time_type"|undefined} end_time
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             */
                            Object.defineProperty(BillingSetup.prototype, "end_time", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date_time", "end_time_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BillingSetup instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBillingSetup=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.BillingSetup} BillingSetup instance
                             */
                            BillingSetup.create = function create(properties) {
                                return new BillingSetup(properties);
                            };
    
                            /**
                             * Encodes the specified BillingSetup message. Does not implicitly {@link google.ads.googleads.v11.resources.BillingSetup.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBillingSetup} m BillingSetup message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetup.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.start_time_type != null && Object.hasOwnProperty.call(m, "start_time_type"))
                                    w.uint32(80).int32(m.start_time_type);
                                if (m.payments_account_info != null && Object.hasOwnProperty.call(m, "payments_account_info"))
                                    $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo.encode(m.payments_account_info, w.uint32(98).fork()).ldelim();
                                if (m.end_time_type != null && Object.hasOwnProperty.call(m, "end_time_type"))
                                    w.uint32(112).int32(m.end_time_type);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(120).int64(m.id);
                                if (m.start_date_time != null && Object.hasOwnProperty.call(m, "start_date_time"))
                                    w.uint32(130).string(m.start_date_time);
                                if (m.end_date_time != null && Object.hasOwnProperty.call(m, "end_date_time"))
                                    w.uint32(138).string(m.end_date_time);
                                if (m.payments_account != null && Object.hasOwnProperty.call(m, "payments_account"))
                                    w.uint32(146).string(m.payments_account);
                                return w;
                            };
    
                            /**
                             * Encodes the specified BillingSetup message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BillingSetup.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IBillingSetup} message BillingSetup message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetup.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BillingSetup message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.BillingSetup} BillingSetup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetup.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BillingSetup();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 15:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 18:
                                        m.payments_account = r.string();
                                        break;
                                    case 12:
                                        m.payments_account_info = $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.start_date_time = r.string();
                                        break;
                                    case 10:
                                        m.start_time_type = r.int32();
                                        break;
                                    case 17:
                                        m.end_date_time = r.string();
                                        break;
                                    case 14:
                                        m.end_time_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BillingSetup message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.BillingSetup} BillingSetup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetup.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BillingSetup message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BillingSetup.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.payments_account != null && m.hasOwnProperty("payments_account")) {
                                    p._payments_account = 1;
                                    if (!$util.isString(m.payments_account))
                                        return "payments_account: string expected";
                                }
                                if (m.payments_account_info != null && m.hasOwnProperty("payments_account_info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo.verify(m.payments_account_info);
                                        if (e)
                                            return "payments_account_info." + e;
                                    }
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    p.start_time = 1;
                                    if (!$util.isString(m.start_date_time))
                                        return "start_date_time: string expected";
                                }
                                if (m.start_time_type != null && m.hasOwnProperty("start_time_type")) {
                                    if (p.start_time === 1)
                                        return "start_time: multiple values";
                                    p.start_time = 1;
                                    switch (m.start_time_type) {
                                    default:
                                        return "start_time_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    p.end_time = 1;
                                    if (!$util.isString(m.end_date_time))
                                        return "end_date_time: string expected";
                                }
                                if (m.end_time_type != null && m.hasOwnProperty("end_time_type")) {
                                    if (p.end_time === 1)
                                        return "end_time: multiple values";
                                    p.end_time = 1;
                                    switch (m.end_time_type) {
                                    default:
                                        return "end_time_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BillingSetup message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.BillingSetup} BillingSetup
                             */
                            BillingSetup.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.BillingSetup)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.BillingSetup();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "APPROVED_HELD":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "CANCELLED":
                                case 5:
                                    m.status = 5;
                                    break;
                                }
                                if (d.payments_account != null) {
                                    m.payments_account = String(d.payments_account);
                                }
                                if (d.payments_account_info != null) {
                                    if (typeof d.payments_account_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.BillingSetup.payments_account_info: object expected");
                                    m.payments_account_info = $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo.fromObject(d.payments_account_info);
                                }
                                if (d.start_date_time != null) {
                                    m.start_date_time = String(d.start_date_time);
                                }
                                switch (d.start_time_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.start_time_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.start_time_type = 1;
                                    break;
                                case "NOW":
                                case 2:
                                    m.start_time_type = 2;
                                    break;
                                case "FOREVER":
                                case 3:
                                    m.start_time_type = 3;
                                    break;
                                }
                                if (d.end_date_time != null) {
                                    m.end_date_time = String(d.end_date_time);
                                }
                                switch (d.end_time_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.end_time_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.end_time_type = 1;
                                    break;
                                case "NOW":
                                case 2:
                                    m.end_time_type = 2;
                                    break;
                                case "FOREVER":
                                case 3:
                                    m.end_time_type = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BillingSetup message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @static
                             * @param {google.ads.googleads.v11.resources.BillingSetup} m BillingSetup
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BillingSetup.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.payments_account_info = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.BillingSetupStatusEnum.BillingSetupStatus[m.status] : m.status;
                                }
                                if (m.start_time_type != null && m.hasOwnProperty("start_time_type")) {
                                    d.start_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.start_time_type] : m.start_time_type;
                                    if (o.oneofs)
                                        d.start_time = "start_time_type";
                                }
                                if (m.payments_account_info != null && m.hasOwnProperty("payments_account_info")) {
                                    d.payments_account_info = $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo.toObject(m.payments_account_info, o);
                                }
                                if (m.end_time_type != null && m.hasOwnProperty("end_time_type")) {
                                    d.end_time_type = o.enums === String ? $root.google.ads.googleads.v11.enums.TimeTypeEnum.TimeType[m.end_time_type] : m.end_time_type;
                                    if (o.oneofs)
                                        d.end_time = "end_time_type";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    d.start_date_time = m.start_date_time;
                                    if (o.oneofs)
                                        d.start_time = "start_date_time";
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    d.end_date_time = m.end_date_time;
                                    if (o.oneofs)
                                        d.end_time = "end_date_time";
                                }
                                if (m.payments_account != null && m.hasOwnProperty("payments_account")) {
                                    d.payments_account = m.payments_account;
                                    if (o.oneofs)
                                        d._payments_account = "payments_account";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BillingSetup to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.BillingSetup
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BillingSetup.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            BillingSetup.PaymentsAccountInfo = (function() {
    
                                /**
                                 * Properties of a PaymentsAccountInfo.
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup
                                 * @interface IPaymentsAccountInfo
                                 * @property {string|null} [payments_account_id] PaymentsAccountInfo payments_account_id
                                 * @property {string|null} [payments_account_name] PaymentsAccountInfo payments_account_name
                                 * @property {string|null} [payments_profile_id] PaymentsAccountInfo payments_profile_id
                                 * @property {string|null} [payments_profile_name] PaymentsAccountInfo payments_profile_name
                                 * @property {string|null} [secondary_payments_profile_id] PaymentsAccountInfo secondary_payments_profile_id
                                 */
    
                                /**
                                 * Constructs a new PaymentsAccountInfo.
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup
                                 * @classdesc Represents a PaymentsAccountInfo.
                                 * @implements IPaymentsAccountInfo
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.BillingSetup.IPaymentsAccountInfo=} [p] Properties to set
                                 */
                                function PaymentsAccountInfo(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * PaymentsAccountInfo payments_account_id.
                                 * @member {string|null|undefined} payments_account_id
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                PaymentsAccountInfo.prototype.payments_account_id = null;
    
                                /**
                                 * PaymentsAccountInfo payments_account_name.
                                 * @member {string|null|undefined} payments_account_name
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                PaymentsAccountInfo.prototype.payments_account_name = null;
    
                                /**
                                 * PaymentsAccountInfo payments_profile_id.
                                 * @member {string|null|undefined} payments_profile_id
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                PaymentsAccountInfo.prototype.payments_profile_id = null;
    
                                /**
                                 * PaymentsAccountInfo payments_profile_name.
                                 * @member {string|null|undefined} payments_profile_name
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                PaymentsAccountInfo.prototype.payments_profile_name = null;
    
                                /**
                                 * PaymentsAccountInfo secondary_payments_profile_id.
                                 * @member {string|null|undefined} secondary_payments_profile_id
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                PaymentsAccountInfo.prototype.secondary_payments_profile_id = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * PaymentsAccountInfo _payments_account_id.
                                 * @member {"payments_account_id"|undefined} _payments_account_id
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                Object.defineProperty(PaymentsAccountInfo.prototype, "_payments_account_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["payments_account_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PaymentsAccountInfo _payments_account_name.
                                 * @member {"payments_account_name"|undefined} _payments_account_name
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                Object.defineProperty(PaymentsAccountInfo.prototype, "_payments_account_name", {
                                    get: $util.oneOfGetter($oneOfFields = ["payments_account_name"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PaymentsAccountInfo _payments_profile_id.
                                 * @member {"payments_profile_id"|undefined} _payments_profile_id
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                Object.defineProperty(PaymentsAccountInfo.prototype, "_payments_profile_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["payments_profile_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PaymentsAccountInfo _payments_profile_name.
                                 * @member {"payments_profile_name"|undefined} _payments_profile_name
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                Object.defineProperty(PaymentsAccountInfo.prototype, "_payments_profile_name", {
                                    get: $util.oneOfGetter($oneOfFields = ["payments_profile_name"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PaymentsAccountInfo _secondary_payments_profile_id.
                                 * @member {"secondary_payments_profile_id"|undefined} _secondary_payments_profile_id
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 */
                                Object.defineProperty(PaymentsAccountInfo.prototype, "_secondary_payments_profile_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["secondary_payments_profile_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new PaymentsAccountInfo instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BillingSetup.IPaymentsAccountInfo=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo} PaymentsAccountInfo instance
                                 */
                                PaymentsAccountInfo.create = function create(properties) {
                                    return new PaymentsAccountInfo(properties);
                                };
    
                                /**
                                 * Encodes the specified PaymentsAccountInfo message. Does not implicitly {@link google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BillingSetup.IPaymentsAccountInfo} m PaymentsAccountInfo message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PaymentsAccountInfo.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.payments_account_id != null && Object.hasOwnProperty.call(m, "payments_account_id"))
                                        w.uint32(50).string(m.payments_account_id);
                                    if (m.payments_account_name != null && Object.hasOwnProperty.call(m, "payments_account_name"))
                                        w.uint32(58).string(m.payments_account_name);
                                    if (m.payments_profile_id != null && Object.hasOwnProperty.call(m, "payments_profile_id"))
                                        w.uint32(66).string(m.payments_profile_id);
                                    if (m.payments_profile_name != null && Object.hasOwnProperty.call(m, "payments_profile_name"))
                                        w.uint32(74).string(m.payments_profile_name);
                                    if (m.secondary_payments_profile_id != null && Object.hasOwnProperty.call(m, "secondary_payments_profile_id"))
                                        w.uint32(82).string(m.secondary_payments_profile_id);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified PaymentsAccountInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BillingSetup.IPaymentsAccountInfo} message PaymentsAccountInfo message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PaymentsAccountInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a PaymentsAccountInfo message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo} PaymentsAccountInfo
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PaymentsAccountInfo.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 6:
                                            m.payments_account_id = r.string();
                                            break;
                                        case 7:
                                            m.payments_account_name = r.string();
                                            break;
                                        case 8:
                                            m.payments_profile_id = r.string();
                                            break;
                                        case 9:
                                            m.payments_profile_name = r.string();
                                            break;
                                        case 10:
                                            m.secondary_payments_profile_id = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a PaymentsAccountInfo message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo} PaymentsAccountInfo
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PaymentsAccountInfo.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a PaymentsAccountInfo message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                PaymentsAccountInfo.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.payments_account_id != null && m.hasOwnProperty("payments_account_id")) {
                                        p._payments_account_id = 1;
                                        if (!$util.isString(m.payments_account_id))
                                            return "payments_account_id: string expected";
                                    }
                                    if (m.payments_account_name != null && m.hasOwnProperty("payments_account_name")) {
                                        p._payments_account_name = 1;
                                        if (!$util.isString(m.payments_account_name))
                                            return "payments_account_name: string expected";
                                    }
                                    if (m.payments_profile_id != null && m.hasOwnProperty("payments_profile_id")) {
                                        p._payments_profile_id = 1;
                                        if (!$util.isString(m.payments_profile_id))
                                            return "payments_profile_id: string expected";
                                    }
                                    if (m.payments_profile_name != null && m.hasOwnProperty("payments_profile_name")) {
                                        p._payments_profile_name = 1;
                                        if (!$util.isString(m.payments_profile_name))
                                            return "payments_profile_name: string expected";
                                    }
                                    if (m.secondary_payments_profile_id != null && m.hasOwnProperty("secondary_payments_profile_id")) {
                                        p._secondary_payments_profile_id = 1;
                                        if (!$util.isString(m.secondary_payments_profile_id))
                                            return "secondary_payments_profile_id: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a PaymentsAccountInfo message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo} PaymentsAccountInfo
                                 */
                                PaymentsAccountInfo.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo();
                                    if (d.payments_account_id != null) {
                                        m.payments_account_id = String(d.payments_account_id);
                                    }
                                    if (d.payments_account_name != null) {
                                        m.payments_account_name = String(d.payments_account_name);
                                    }
                                    if (d.payments_profile_id != null) {
                                        m.payments_profile_id = String(d.payments_profile_id);
                                    }
                                    if (d.payments_profile_name != null) {
                                        m.payments_profile_name = String(d.payments_profile_name);
                                    }
                                    if (d.secondary_payments_profile_id != null) {
                                        m.secondary_payments_profile_id = String(d.secondary_payments_profile_id);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a PaymentsAccountInfo message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo} m PaymentsAccountInfo
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                PaymentsAccountInfo.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.payments_account_id != null && m.hasOwnProperty("payments_account_id")) {
                                        d.payments_account_id = m.payments_account_id;
                                        if (o.oneofs)
                                            d._payments_account_id = "payments_account_id";
                                    }
                                    if (m.payments_account_name != null && m.hasOwnProperty("payments_account_name")) {
                                        d.payments_account_name = m.payments_account_name;
                                        if (o.oneofs)
                                            d._payments_account_name = "payments_account_name";
                                    }
                                    if (m.payments_profile_id != null && m.hasOwnProperty("payments_profile_id")) {
                                        d.payments_profile_id = m.payments_profile_id;
                                        if (o.oneofs)
                                            d._payments_profile_id = "payments_profile_id";
                                    }
                                    if (m.payments_profile_name != null && m.hasOwnProperty("payments_profile_name")) {
                                        d.payments_profile_name = m.payments_profile_name;
                                        if (o.oneofs)
                                            d._payments_profile_name = "payments_profile_name";
                                    }
                                    if (m.secondary_payments_profile_id != null && m.hasOwnProperty("secondary_payments_profile_id")) {
                                        d.secondary_payments_profile_id = m.secondary_payments_profile_id;
                                        if (o.oneofs)
                                            d._secondary_payments_profile_id = "secondary_payments_profile_id";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this PaymentsAccountInfo to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.BillingSetup.PaymentsAccountInfo
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                PaymentsAccountInfo.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return PaymentsAccountInfo;
                            })();
    
                            return BillingSetup;
                        })();
    
                        resources.CallView = (function() {
    
                            /**
                             * Properties of a CallView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICallView
                             * @property {string|null} [resource_name] CallView resource_name
                             * @property {string|null} [caller_country_code] CallView caller_country_code
                             * @property {string|null} [caller_area_code] CallView caller_area_code
                             * @property {number|null} [call_duration_seconds] CallView call_duration_seconds
                             * @property {string|null} [start_call_date_time] CallView start_call_date_time
                             * @property {string|null} [end_call_date_time] CallView end_call_date_time
                             * @property {google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation|null} [call_tracking_display_location] CallView call_tracking_display_location
                             * @property {google.ads.googleads.v11.enums.CallTypeEnum.CallType|null} [type] CallView type
                             * @property {google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus|null} [call_status] CallView call_status
                             */
    
                            /**
                             * Constructs a new CallView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CallView.
                             * @implements ICallView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICallView=} [p] Properties to set
                             */
                            function CallView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.resource_name = "";
    
                            /**
                             * CallView caller_country_code.
                             * @member {string} caller_country_code
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.caller_country_code = "";
    
                            /**
                             * CallView caller_area_code.
                             * @member {string} caller_area_code
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.caller_area_code = "";
    
                            /**
                             * CallView call_duration_seconds.
                             * @member {number} call_duration_seconds
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.call_duration_seconds = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CallView start_call_date_time.
                             * @member {string} start_call_date_time
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.start_call_date_time = "";
    
                            /**
                             * CallView end_call_date_time.
                             * @member {string} end_call_date_time
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.end_call_date_time = "";
    
                            /**
                             * CallView call_tracking_display_location.
                             * @member {google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation} call_tracking_display_location
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.call_tracking_display_location = 0;
    
                            /**
                             * CallView type.
                             * @member {google.ads.googleads.v11.enums.CallTypeEnum.CallType} type
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.type = 0;
    
                            /**
                             * CallView call_status.
                             * @member {google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus} call_status
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             */
                            CallView.prototype.call_status = 0;
    
                            /**
                             * Creates a new CallView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICallView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CallView} CallView instance
                             */
                            CallView.create = function create(properties) {
                                return new CallView(properties);
                            };
    
                            /**
                             * Encodes the specified CallView message. Does not implicitly {@link google.ads.googleads.v11.resources.CallView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICallView} m CallView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.caller_country_code != null && Object.hasOwnProperty.call(m, "caller_country_code"))
                                    w.uint32(18).string(m.caller_country_code);
                                if (m.caller_area_code != null && Object.hasOwnProperty.call(m, "caller_area_code"))
                                    w.uint32(26).string(m.caller_area_code);
                                if (m.call_duration_seconds != null && Object.hasOwnProperty.call(m, "call_duration_seconds"))
                                    w.uint32(32).int64(m.call_duration_seconds);
                                if (m.start_call_date_time != null && Object.hasOwnProperty.call(m, "start_call_date_time"))
                                    w.uint32(42).string(m.start_call_date_time);
                                if (m.end_call_date_time != null && Object.hasOwnProperty.call(m, "end_call_date_time"))
                                    w.uint32(50).string(m.end_call_date_time);
                                if (m.call_tracking_display_location != null && Object.hasOwnProperty.call(m, "call_tracking_display_location"))
                                    w.uint32(56).int32(m.call_tracking_display_location);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(64).int32(m.type);
                                if (m.call_status != null && Object.hasOwnProperty.call(m, "call_status"))
                                    w.uint32(72).int32(m.call_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CallView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICallView} message CallView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CallView} CallView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CallView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.caller_country_code = r.string();
                                        break;
                                    case 3:
                                        m.caller_area_code = r.string();
                                        break;
                                    case 4:
                                        m.call_duration_seconds = r.int64();
                                        break;
                                    case 5:
                                        m.start_call_date_time = r.string();
                                        break;
                                    case 6:
                                        m.end_call_date_time = r.string();
                                        break;
                                    case 7:
                                        m.call_tracking_display_location = r.int32();
                                        break;
                                    case 8:
                                        m.type = r.int32();
                                        break;
                                    case 9:
                                        m.call_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CallView} CallView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.caller_country_code != null && m.hasOwnProperty("caller_country_code")) {
                                    if (!$util.isString(m.caller_country_code))
                                        return "caller_country_code: string expected";
                                }
                                if (m.caller_area_code != null && m.hasOwnProperty("caller_area_code")) {
                                    if (!$util.isString(m.caller_area_code))
                                        return "caller_area_code: string expected";
                                }
                                if (m.call_duration_seconds != null && m.hasOwnProperty("call_duration_seconds")) {
                                    if (!$util.isInteger(m.call_duration_seconds) && !(m.call_duration_seconds && $util.isInteger(m.call_duration_seconds.low) && $util.isInteger(m.call_duration_seconds.high)))
                                        return "call_duration_seconds: integer|Long expected";
                                }
                                if (m.start_call_date_time != null && m.hasOwnProperty("start_call_date_time")) {
                                    if (!$util.isString(m.start_call_date_time))
                                        return "start_call_date_time: string expected";
                                }
                                if (m.end_call_date_time != null && m.hasOwnProperty("end_call_date_time")) {
                                    if (!$util.isString(m.end_call_date_time))
                                        return "end_call_date_time: string expected";
                                }
                                if (m.call_tracking_display_location != null && m.hasOwnProperty("call_tracking_display_location")) {
                                    switch (m.call_tracking_display_location) {
                                    default:
                                        return "call_tracking_display_location: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.call_status != null && m.hasOwnProperty("call_status")) {
                                    switch (m.call_status) {
                                    default:
                                        return "call_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CallView} CallView
                             */
                            CallView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CallView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CallView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.caller_country_code != null) {
                                    m.caller_country_code = String(d.caller_country_code);
                                }
                                if (d.caller_area_code != null) {
                                    m.caller_area_code = String(d.caller_area_code);
                                }
                                if (d.call_duration_seconds != null) {
                                    if ($util.Long)
                                        (m.call_duration_seconds = $util.Long.fromValue(d.call_duration_seconds)).unsigned = false;
                                    else if (typeof d.call_duration_seconds === "string")
                                        m.call_duration_seconds = parseInt(d.call_duration_seconds, 10);
                                    else if (typeof d.call_duration_seconds === "number")
                                        m.call_duration_seconds = d.call_duration_seconds;
                                    else if (typeof d.call_duration_seconds === "object")
                                        m.call_duration_seconds = new $util.LongBits(d.call_duration_seconds.low >>> 0, d.call_duration_seconds.high >>> 0).toNumber();
                                }
                                if (d.start_call_date_time != null) {
                                    m.start_call_date_time = String(d.start_call_date_time);
                                }
                                if (d.end_call_date_time != null) {
                                    m.end_call_date_time = String(d.end_call_date_time);
                                }
                                switch (d.call_tracking_display_location) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.call_tracking_display_location = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.call_tracking_display_location = 1;
                                    break;
                                case "AD":
                                case 2:
                                    m.call_tracking_display_location = 2;
                                    break;
                                case "LANDING_PAGE":
                                case 3:
                                    m.call_tracking_display_location = 3;
                                    break;
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "MANUALLY_DIALED":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "HIGH_END_MOBILE_SEARCH":
                                case 3:
                                    m.type = 3;
                                    break;
                                }
                                switch (d.call_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.call_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.call_status = 1;
                                    break;
                                case "MISSED":
                                case 2:
                                    m.call_status = 2;
                                    break;
                                case "RECEIVED":
                                case 3:
                                    m.call_status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @static
                             * @param {google.ads.googleads.v11.resources.CallView} m CallView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.caller_country_code = "";
                                    d.caller_area_code = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.call_duration_seconds = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.call_duration_seconds = o.longs === String ? "0" : 0;
                                    d.start_call_date_time = "";
                                    d.end_call_date_time = "";
                                    d.call_tracking_display_location = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.call_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.caller_country_code != null && m.hasOwnProperty("caller_country_code")) {
                                    d.caller_country_code = m.caller_country_code;
                                }
                                if (m.caller_area_code != null && m.hasOwnProperty("caller_area_code")) {
                                    d.caller_area_code = m.caller_area_code;
                                }
                                if (m.call_duration_seconds != null && m.hasOwnProperty("call_duration_seconds")) {
                                    if (typeof m.call_duration_seconds === "number")
                                        d.call_duration_seconds = o.longs === String ? String(m.call_duration_seconds) : m.call_duration_seconds;
                                    else
                                        d.call_duration_seconds = o.longs === String ? $util.Long.prototype.toString.call(m.call_duration_seconds) : o.longs === Number ? new $util.LongBits(m.call_duration_seconds.low >>> 0, m.call_duration_seconds.high >>> 0).toNumber() : m.call_duration_seconds;
                                }
                                if (m.start_call_date_time != null && m.hasOwnProperty("start_call_date_time")) {
                                    d.start_call_date_time = m.start_call_date_time;
                                }
                                if (m.end_call_date_time != null && m.hasOwnProperty("end_call_date_time")) {
                                    d.end_call_date_time = m.end_call_date_time;
                                }
                                if (m.call_tracking_display_location != null && m.hasOwnProperty("call_tracking_display_location")) {
                                    d.call_tracking_display_location = o.enums === String ? $root.google.ads.googleads.v11.enums.CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation[m.call_tracking_display_location] : m.call_tracking_display_location;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CallTypeEnum.CallType[m.type] : m.type;
                                }
                                if (m.call_status != null && m.hasOwnProperty("call_status")) {
                                    d.call_status = o.enums === String ? $root.google.ads.googleads.v11.enums.GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus[m.call_status] : m.call_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CallView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallView;
                        })();
    
                        resources.Campaign = (function() {
    
                            /**
                             * Properties of a Campaign.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaign
                             * @property {string|null} [resource_name] Campaign resource_name
                             * @property {number|null} [id] Campaign id
                             * @property {string|null} [name] Campaign name
                             * @property {google.ads.googleads.v11.enums.CampaignStatusEnum.CampaignStatus|null} [status] Campaign status
                             * @property {google.ads.googleads.v11.enums.CampaignServingStatusEnum.CampaignServingStatus|null} [serving_status] Campaign serving_status
                             * @property {google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum.AdServingOptimizationStatus|null} [ad_serving_optimization_status] Campaign ad_serving_optimization_status
                             * @property {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType|null} [advertising_channel_type] Campaign advertising_channel_type
                             * @property {google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType|null} [advertising_channel_sub_type] Campaign advertising_channel_sub_type
                             * @property {string|null} [tracking_url_template] Campaign tracking_url_template
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] Campaign url_custom_parameters
                             * @property {google.ads.googleads.v11.resources.Campaign.ILocalServicesCampaignSettings|null} [local_services_campaign_settings] Campaign local_services_campaign_settings
                             * @property {google.ads.googleads.v11.common.IRealTimeBiddingSetting|null} [real_time_bidding_setting] Campaign real_time_bidding_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.INetworkSettings|null} [network_settings] Campaign network_settings
                             * @property {google.ads.googleads.v11.resources.Campaign.IHotelSettingInfo|null} [hotel_setting] Campaign hotel_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.IDynamicSearchAdsSetting|null} [dynamic_search_ads_setting] Campaign dynamic_search_ads_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.IShoppingSetting|null} [shopping_setting] Campaign shopping_setting
                             * @property {google.ads.googleads.v11.common.ITargetingSetting|null} [targeting_setting] Campaign targeting_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.IAudienceSetting|null} [audience_setting] Campaign audience_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.IGeoTargetTypeSetting|null} [geo_target_type_setting] Campaign geo_target_type_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.ILocalCampaignSetting|null} [local_campaign_setting] Campaign local_campaign_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.IAppCampaignSetting|null} [app_campaign_setting] Campaign app_campaign_setting
                             * @property {Array.<string>|null} [labels] Campaign labels
                             * @property {google.ads.googleads.v11.enums.CampaignExperimentTypeEnum.CampaignExperimentType|null} [experiment_type] Campaign experiment_type
                             * @property {string|null} [base_campaign] Campaign base_campaign
                             * @property {string|null} [campaign_budget] Campaign campaign_budget
                             * @property {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType|null} [bidding_strategy_type] Campaign bidding_strategy_type
                             * @property {string|null} [accessible_bidding_strategy] Campaign accessible_bidding_strategy
                             * @property {string|null} [start_date] Campaign start_date
                             * @property {string|null} [campaign_group] Campaign campaign_group
                             * @property {string|null} [end_date] Campaign end_date
                             * @property {string|null} [final_url_suffix] Campaign final_url_suffix
                             * @property {Array.<google.ads.googleads.v11.common.IFrequencyCapEntry>|null} [frequency_caps] Campaign frequency_caps
                             * @property {google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum.BrandSafetySuitability|null} [video_brand_safety_suitability] Campaign video_brand_safety_suitability
                             * @property {google.ads.googleads.v11.resources.Campaign.IVanityPharma|null} [vanity_pharma] Campaign vanity_pharma
                             * @property {google.ads.googleads.v11.resources.Campaign.ISelectiveOptimization|null} [selective_optimization] Campaign selective_optimization
                             * @property {google.ads.googleads.v11.resources.Campaign.IOptimizationGoalSetting|null} [optimization_goal_setting] Campaign optimization_goal_setting
                             * @property {google.ads.googleads.v11.resources.Campaign.ITrackingSetting|null} [tracking_setting] Campaign tracking_setting
                             * @property {google.ads.googleads.v11.enums.PaymentModeEnum.PaymentMode|null} [payment_mode] Campaign payment_mode
                             * @property {number|null} [optimization_score] Campaign optimization_score
                             * @property {Array.<google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType>|null} [excluded_parent_asset_field_types] Campaign excluded_parent_asset_field_types
                             * @property {boolean|null} [url_expansion_opt_out] Campaign url_expansion_opt_out
                             * @property {google.ads.googleads.v11.resources.Campaign.IPerformanceMaxUpgrade|null} [performance_max_upgrade] Campaign performance_max_upgrade
                             * @property {string|null} [bidding_strategy] Campaign bidding_strategy
                             * @property {google.ads.googleads.v11.common.ICommission|null} [commission] Campaign commission
                             * @property {google.ads.googleads.v11.common.IManualCpa|null} [manual_cpa] Campaign manual_cpa
                             * @property {google.ads.googleads.v11.common.IManualCpc|null} [manual_cpc] Campaign manual_cpc
                             * @property {google.ads.googleads.v11.common.IManualCpm|null} [manual_cpm] Campaign manual_cpm
                             * @property {google.ads.googleads.v11.common.IManualCpv|null} [manual_cpv] Campaign manual_cpv
                             * @property {google.ads.googleads.v11.common.IMaximizeConversions|null} [maximize_conversions] Campaign maximize_conversions
                             * @property {google.ads.googleads.v11.common.IMaximizeConversionValue|null} [maximize_conversion_value] Campaign maximize_conversion_value
                             * @property {google.ads.googleads.v11.common.ITargetCpa|null} [target_cpa] Campaign target_cpa
                             * @property {google.ads.googleads.v11.common.ITargetImpressionShare|null} [target_impression_share] Campaign target_impression_share
                             * @property {google.ads.googleads.v11.common.ITargetRoas|null} [target_roas] Campaign target_roas
                             * @property {google.ads.googleads.v11.common.ITargetSpend|null} [target_spend] Campaign target_spend
                             * @property {google.ads.googleads.v11.common.IPercentCpc|null} [percent_cpc] Campaign percent_cpc
                             * @property {google.ads.googleads.v11.common.ITargetCpm|null} [target_cpm] Campaign target_cpm
                             */
    
                            /**
                             * Constructs a new Campaign.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a Campaign.
                             * @implements ICampaign
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaign=} [p] Properties to set
                             */
                            function Campaign(p) {
                                this.url_custom_parameters = [];
                                this.labels = [];
                                this.frequency_caps = [];
                                this.excluded_parent_asset_field_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Campaign resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.resource_name = "";
    
                            /**
                             * Campaign id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.id = null;
    
                            /**
                             * Campaign name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.name = null;
    
                            /**
                             * Campaign status.
                             * @member {google.ads.googleads.v11.enums.CampaignStatusEnum.CampaignStatus} status
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.status = 0;
    
                            /**
                             * Campaign serving_status.
                             * @member {google.ads.googleads.v11.enums.CampaignServingStatusEnum.CampaignServingStatus} serving_status
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.serving_status = 0;
    
                            /**
                             * Campaign ad_serving_optimization_status.
                             * @member {google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum.AdServingOptimizationStatus} ad_serving_optimization_status
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.ad_serving_optimization_status = 0;
    
                            /**
                             * Campaign advertising_channel_type.
                             * @member {google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType} advertising_channel_type
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.advertising_channel_type = 0;
    
                            /**
                             * Campaign advertising_channel_sub_type.
                             * @member {google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType} advertising_channel_sub_type
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.advertising_channel_sub_type = 0;
    
                            /**
                             * Campaign tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.tracking_url_template = null;
    
                            /**
                             * Campaign url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * Campaign local_services_campaign_settings.
                             * @member {google.ads.googleads.v11.resources.Campaign.ILocalServicesCampaignSettings|null|undefined} local_services_campaign_settings
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.local_services_campaign_settings = null;
    
                            /**
                             * Campaign real_time_bidding_setting.
                             * @member {google.ads.googleads.v11.common.IRealTimeBiddingSetting|null|undefined} real_time_bidding_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.real_time_bidding_setting = null;
    
                            /**
                             * Campaign network_settings.
                             * @member {google.ads.googleads.v11.resources.Campaign.INetworkSettings|null|undefined} network_settings
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.network_settings = null;
    
                            /**
                             * Campaign hotel_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.IHotelSettingInfo|null|undefined} hotel_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.hotel_setting = null;
    
                            /**
                             * Campaign dynamic_search_ads_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.IDynamicSearchAdsSetting|null|undefined} dynamic_search_ads_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.dynamic_search_ads_setting = null;
    
                            /**
                             * Campaign shopping_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.IShoppingSetting|null|undefined} shopping_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.shopping_setting = null;
    
                            /**
                             * Campaign targeting_setting.
                             * @member {google.ads.googleads.v11.common.ITargetingSetting|null|undefined} targeting_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.targeting_setting = null;
    
                            /**
                             * Campaign audience_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.IAudienceSetting|null|undefined} audience_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.audience_setting = null;
    
                            /**
                             * Campaign geo_target_type_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.IGeoTargetTypeSetting|null|undefined} geo_target_type_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.geo_target_type_setting = null;
    
                            /**
                             * Campaign local_campaign_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.ILocalCampaignSetting|null|undefined} local_campaign_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.local_campaign_setting = null;
    
                            /**
                             * Campaign app_campaign_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.IAppCampaignSetting|null|undefined} app_campaign_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.app_campaign_setting = null;
    
                            /**
                             * Campaign labels.
                             * @member {Array.<string>} labels
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.labels = $util.emptyArray;
    
                            /**
                             * Campaign experiment_type.
                             * @member {google.ads.googleads.v11.enums.CampaignExperimentTypeEnum.CampaignExperimentType} experiment_type
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.experiment_type = 0;
    
                            /**
                             * Campaign base_campaign.
                             * @member {string|null|undefined} base_campaign
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.base_campaign = null;
    
                            /**
                             * Campaign campaign_budget.
                             * @member {string|null|undefined} campaign_budget
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.campaign_budget = null;
    
                            /**
                             * Campaign bidding_strategy_type.
                             * @member {google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType} bidding_strategy_type
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.bidding_strategy_type = 0;
    
                            /**
                             * Campaign accessible_bidding_strategy.
                             * @member {string} accessible_bidding_strategy
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.accessible_bidding_strategy = "";
    
                            /**
                             * Campaign start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.start_date = null;
    
                            /**
                             * Campaign campaign_group.
                             * @member {string|null|undefined} campaign_group
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.campaign_group = null;
    
                            /**
                             * Campaign end_date.
                             * @member {string|null|undefined} end_date
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.end_date = null;
    
                            /**
                             * Campaign final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.final_url_suffix = null;
    
                            /**
                             * Campaign frequency_caps.
                             * @member {Array.<google.ads.googleads.v11.common.IFrequencyCapEntry>} frequency_caps
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.frequency_caps = $util.emptyArray;
    
                            /**
                             * Campaign video_brand_safety_suitability.
                             * @member {google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum.BrandSafetySuitability} video_brand_safety_suitability
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.video_brand_safety_suitability = 0;
    
                            /**
                             * Campaign vanity_pharma.
                             * @member {google.ads.googleads.v11.resources.Campaign.IVanityPharma|null|undefined} vanity_pharma
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.vanity_pharma = null;
    
                            /**
                             * Campaign selective_optimization.
                             * @member {google.ads.googleads.v11.resources.Campaign.ISelectiveOptimization|null|undefined} selective_optimization
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.selective_optimization = null;
    
                            /**
                             * Campaign optimization_goal_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.IOptimizationGoalSetting|null|undefined} optimization_goal_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.optimization_goal_setting = null;
    
                            /**
                             * Campaign tracking_setting.
                             * @member {google.ads.googleads.v11.resources.Campaign.ITrackingSetting|null|undefined} tracking_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.tracking_setting = null;
    
                            /**
                             * Campaign payment_mode.
                             * @member {google.ads.googleads.v11.enums.PaymentModeEnum.PaymentMode} payment_mode
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.payment_mode = 0;
    
                            /**
                             * Campaign optimization_score.
                             * @member {number|null|undefined} optimization_score
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.optimization_score = null;
    
                            /**
                             * Campaign excluded_parent_asset_field_types.
                             * @member {Array.<google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType>} excluded_parent_asset_field_types
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.excluded_parent_asset_field_types = $util.emptyArray;
    
                            /**
                             * Campaign url_expansion_opt_out.
                             * @member {boolean|null|undefined} url_expansion_opt_out
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.url_expansion_opt_out = null;
    
                            /**
                             * Campaign performance_max_upgrade.
                             * @member {google.ads.googleads.v11.resources.Campaign.IPerformanceMaxUpgrade|null|undefined} performance_max_upgrade
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.performance_max_upgrade = null;
    
                            /**
                             * Campaign bidding_strategy.
                             * @member {string|null|undefined} bidding_strategy
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.bidding_strategy = null;
    
                            /**
                             * Campaign commission.
                             * @member {google.ads.googleads.v11.common.ICommission|null|undefined} commission
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.commission = null;
    
                            /**
                             * Campaign manual_cpa.
                             * @member {google.ads.googleads.v11.common.IManualCpa|null|undefined} manual_cpa
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.manual_cpa = null;
    
                            /**
                             * Campaign manual_cpc.
                             * @member {google.ads.googleads.v11.common.IManualCpc|null|undefined} manual_cpc
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.manual_cpc = null;
    
                            /**
                             * Campaign manual_cpm.
                             * @member {google.ads.googleads.v11.common.IManualCpm|null|undefined} manual_cpm
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.manual_cpm = null;
    
                            /**
                             * Campaign manual_cpv.
                             * @member {google.ads.googleads.v11.common.IManualCpv|null|undefined} manual_cpv
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.manual_cpv = null;
    
                            /**
                             * Campaign maximize_conversions.
                             * @member {google.ads.googleads.v11.common.IMaximizeConversions|null|undefined} maximize_conversions
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.maximize_conversions = null;
    
                            /**
                             * Campaign maximize_conversion_value.
                             * @member {google.ads.googleads.v11.common.IMaximizeConversionValue|null|undefined} maximize_conversion_value
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.maximize_conversion_value = null;
    
                            /**
                             * Campaign target_cpa.
                             * @member {google.ads.googleads.v11.common.ITargetCpa|null|undefined} target_cpa
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.target_cpa = null;
    
                            /**
                             * Campaign target_impression_share.
                             * @member {google.ads.googleads.v11.common.ITargetImpressionShare|null|undefined} target_impression_share
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.target_impression_share = null;
    
                            /**
                             * Campaign target_roas.
                             * @member {google.ads.googleads.v11.common.ITargetRoas|null|undefined} target_roas
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.target_roas = null;
    
                            /**
                             * Campaign target_spend.
                             * @member {google.ads.googleads.v11.common.ITargetSpend|null|undefined} target_spend
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.target_spend = null;
    
                            /**
                             * Campaign percent_cpc.
                             * @member {google.ads.googleads.v11.common.IPercentCpc|null|undefined} percent_cpc
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.percent_cpc = null;
    
                            /**
                             * Campaign target_cpm.
                             * @member {google.ads.googleads.v11.common.ITargetCpm|null|undefined} target_cpm
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Campaign.prototype.target_cpm = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Campaign _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _audience_setting.
                             * @member {"audience_setting"|undefined} _audience_setting
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_audience_setting", {
                                get: $util.oneOfGetter($oneOfFields = ["audience_setting"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _base_campaign.
                             * @member {"base_campaign"|undefined} _base_campaign
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_base_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["base_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _campaign_budget.
                             * @member {"campaign_budget"|undefined} _campaign_budget
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_campaign_budget", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_budget"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _campaign_group.
                             * @member {"campaign_group"|undefined} _campaign_group
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_campaign_group", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _end_date.
                             * @member {"end_date"|undefined} _end_date
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _optimization_score.
                             * @member {"optimization_score"|undefined} _optimization_score
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_optimization_score", {
                                get: $util.oneOfGetter($oneOfFields = ["optimization_score"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign _url_expansion_opt_out.
                             * @member {"url_expansion_opt_out"|undefined} _url_expansion_opt_out
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "_url_expansion_opt_out", {
                                get: $util.oneOfGetter($oneOfFields = ["url_expansion_opt_out"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Campaign campaign_bidding_strategy.
                             * @member {"bidding_strategy"|"commission"|"manual_cpa"|"manual_cpc"|"manual_cpm"|"manual_cpv"|"maximize_conversions"|"maximize_conversion_value"|"target_cpa"|"target_impression_share"|"target_roas"|"target_spend"|"percent_cpc"|"target_cpm"|undefined} campaign_bidding_strategy
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             */
                            Object.defineProperty(Campaign.prototype, "campaign_bidding_strategy", {
                                get: $util.oneOfGetter($oneOfFields = ["bidding_strategy", "commission", "manual_cpa", "manual_cpc", "manual_cpm", "manual_cpv", "maximize_conversions", "maximize_conversion_value", "target_cpa", "target_impression_share", "target_roas", "target_spend", "percent_cpc", "target_cpm"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Campaign instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaign=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Campaign} Campaign instance
                             */
                            Campaign.create = function create(properties) {
                                return new Campaign(properties);
                            };
    
                            /**
                             * Encodes the specified Campaign message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaign} m Campaign message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Campaign.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.ad_serving_optimization_status != null && Object.hasOwnProperty.call(m, "ad_serving_optimization_status"))
                                    w.uint32(64).int32(m.ad_serving_optimization_status);
                                if (m.advertising_channel_type != null && Object.hasOwnProperty.call(m, "advertising_channel_type"))
                                    w.uint32(72).int32(m.advertising_channel_type);
                                if (m.advertising_channel_sub_type != null && Object.hasOwnProperty.call(m, "advertising_channel_sub_type"))
                                    w.uint32(80).int32(m.advertising_channel_sub_type);
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(98).fork()).ldelim();
                                }
                                if (m.network_settings != null && Object.hasOwnProperty.call(m, "network_settings"))
                                    $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings.encode(m.network_settings, w.uint32(114).fork()).ldelim();
                                if (m.experiment_type != null && Object.hasOwnProperty.call(m, "experiment_type"))
                                    w.uint32(136).int32(m.experiment_type);
                                if (m.serving_status != null && Object.hasOwnProperty.call(m, "serving_status"))
                                    w.uint32(168).int32(m.serving_status);
                                if (m.bidding_strategy_type != null && Object.hasOwnProperty.call(m, "bidding_strategy_type"))
                                    w.uint32(176).int32(m.bidding_strategy_type);
                                if (m.manual_cpc != null && Object.hasOwnProperty.call(m, "manual_cpc"))
                                    $root.google.ads.googleads.v11.common.ManualCpc.encode(m.manual_cpc, w.uint32(194).fork()).ldelim();
                                if (m.manual_cpm != null && Object.hasOwnProperty.call(m, "manual_cpm"))
                                    $root.google.ads.googleads.v11.common.ManualCpm.encode(m.manual_cpm, w.uint32(202).fork()).ldelim();
                                if (m.target_cpa != null && Object.hasOwnProperty.call(m, "target_cpa"))
                                    $root.google.ads.googleads.v11.common.TargetCpa.encode(m.target_cpa, w.uint32(210).fork()).ldelim();
                                if (m.target_spend != null && Object.hasOwnProperty.call(m, "target_spend"))
                                    $root.google.ads.googleads.v11.common.TargetSpend.encode(m.target_spend, w.uint32(218).fork()).ldelim();
                                if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                    $root.google.ads.googleads.v11.common.TargetRoas.encode(m.target_roas, w.uint32(234).fork()).ldelim();
                                if (m.maximize_conversions != null && Object.hasOwnProperty.call(m, "maximize_conversions"))
                                    $root.google.ads.googleads.v11.common.MaximizeConversions.encode(m.maximize_conversions, w.uint32(242).fork()).ldelim();
                                if (m.maximize_conversion_value != null && Object.hasOwnProperty.call(m, "maximize_conversion_value"))
                                    $root.google.ads.googleads.v11.common.MaximizeConversionValue.encode(m.maximize_conversion_value, w.uint32(250).fork()).ldelim();
                                if (m.hotel_setting != null && Object.hasOwnProperty.call(m, "hotel_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo.encode(m.hotel_setting, w.uint32(258).fork()).ldelim();
                                if (m.dynamic_search_ads_setting != null && Object.hasOwnProperty.call(m, "dynamic_search_ads_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.encode(m.dynamic_search_ads_setting, w.uint32(266).fork()).ldelim();
                                if (m.percent_cpc != null && Object.hasOwnProperty.call(m, "percent_cpc"))
                                    $root.google.ads.googleads.v11.common.PercentCpc.encode(m.percent_cpc, w.uint32(274).fork()).ldelim();
                                if (m.shopping_setting != null && Object.hasOwnProperty.call(m, "shopping_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting.encode(m.shopping_setting, w.uint32(290).fork()).ldelim();
                                if (m.manual_cpv != null && Object.hasOwnProperty.call(m, "manual_cpv"))
                                    $root.google.ads.googleads.v11.common.ManualCpv.encode(m.manual_cpv, w.uint32(298).fork()).ldelim();
                                if (m.real_time_bidding_setting != null && Object.hasOwnProperty.call(m, "real_time_bidding_setting"))
                                    $root.google.ads.googleads.v11.common.RealTimeBiddingSetting.encode(m.real_time_bidding_setting, w.uint32(314).fork()).ldelim();
                                if (m.frequency_caps != null && m.frequency_caps.length) {
                                    for (var i = 0; i < m.frequency_caps.length; ++i)
                                        $root.google.ads.googleads.v11.common.FrequencyCapEntry.encode(m.frequency_caps[i], w.uint32(322).fork()).ldelim();
                                }
                                if (m.target_cpm != null && Object.hasOwnProperty.call(m, "target_cpm"))
                                    $root.google.ads.googleads.v11.common.TargetCpm.encode(m.target_cpm, w.uint32(330).fork()).ldelim();
                                if (m.video_brand_safety_suitability != null && Object.hasOwnProperty.call(m, "video_brand_safety_suitability"))
                                    w.uint32(336).int32(m.video_brand_safety_suitability);
                                if (m.targeting_setting != null && Object.hasOwnProperty.call(m, "targeting_setting"))
                                    $root.google.ads.googleads.v11.common.TargetingSetting.encode(m.targeting_setting, w.uint32(346).fork()).ldelim();
                                if (m.vanity_pharma != null && Object.hasOwnProperty.call(m, "vanity_pharma"))
                                    $root.google.ads.googleads.v11.resources.Campaign.VanityPharma.encode(m.vanity_pharma, w.uint32(354).fork()).ldelim();
                                if (m.selective_optimization != null && Object.hasOwnProperty.call(m, "selective_optimization"))
                                    $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.encode(m.selective_optimization, w.uint32(362).fork()).ldelim();
                                if (m.tracking_setting != null && Object.hasOwnProperty.call(m, "tracking_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting.encode(m.tracking_setting, w.uint32(370).fork()).ldelim();
                                if (m.geo_target_type_setting != null && Object.hasOwnProperty.call(m, "geo_target_type_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting.encode(m.geo_target_type_setting, w.uint32(378).fork()).ldelim();
                                if (m.target_impression_share != null && Object.hasOwnProperty.call(m, "target_impression_share"))
                                    $root.google.ads.googleads.v11.common.TargetImpressionShare.encode(m.target_impression_share, w.uint32(386).fork()).ldelim();
                                if (m.commission != null && Object.hasOwnProperty.call(m, "commission"))
                                    $root.google.ads.googleads.v11.common.Commission.encode(m.commission, w.uint32(394).fork()).ldelim();
                                if (m.local_campaign_setting != null && Object.hasOwnProperty.call(m, "local_campaign_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting.encode(m.local_campaign_setting, w.uint32(402).fork()).ldelim();
                                if (m.app_campaign_setting != null && Object.hasOwnProperty.call(m, "app_campaign_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting.encode(m.app_campaign_setting, w.uint32(410).fork()).ldelim();
                                if (m.payment_mode != null && Object.hasOwnProperty.call(m, "payment_mode"))
                                    w.uint32(416).int32(m.payment_mode);
                                if (m.optimization_goal_setting != null && Object.hasOwnProperty.call(m, "optimization_goal_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.encode(m.optimization_goal_setting, w.uint32(434).fork()).ldelim();
                                if (m.base_campaign != null && Object.hasOwnProperty.call(m, "base_campaign"))
                                    w.uint32(450).string(m.base_campaign);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(466).string(m.name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(472).int64(m.id);
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(482).string(m.tracking_url_template);
                                if (m.labels != null && m.labels.length) {
                                    for (var i = 0; i < m.labels.length; ++i)
                                        w.uint32(490).string(m.labels[i]);
                                }
                                if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                    w.uint32(498).string(m.campaign_budget);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(506).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(514).string(m.end_date);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(522).string(m.final_url_suffix);
                                if (m.optimization_score != null && Object.hasOwnProperty.call(m, "optimization_score"))
                                    w.uint32(529).double(m.optimization_score);
                                if (m.bidding_strategy != null && Object.hasOwnProperty.call(m, "bidding_strategy"))
                                    w.uint32(538).string(m.bidding_strategy);
                                if (m.excluded_parent_asset_field_types != null && m.excluded_parent_asset_field_types.length) {
                                    w.uint32(554).fork();
                                    for (var i = 0; i < m.excluded_parent_asset_field_types.length; ++i)
                                        w.int32(m.excluded_parent_asset_field_types[i]);
                                    w.ldelim();
                                }
                                if (m.accessible_bidding_strategy != null && Object.hasOwnProperty.call(m, "accessible_bidding_strategy"))
                                    w.uint32(570).string(m.accessible_bidding_strategy);
                                if (m.url_expansion_opt_out != null && Object.hasOwnProperty.call(m, "url_expansion_opt_out"))
                                    w.uint32(576).bool(m.url_expansion_opt_out);
                                if (m.audience_setting != null && Object.hasOwnProperty.call(m, "audience_setting"))
                                    $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting.encode(m.audience_setting, w.uint32(586).fork()).ldelim();
                                if (m.manual_cpa != null && Object.hasOwnProperty.call(m, "manual_cpa"))
                                    $root.google.ads.googleads.v11.common.ManualCpa.encode(m.manual_cpa, w.uint32(594).fork()).ldelim();
                                if (m.local_services_campaign_settings != null && Object.hasOwnProperty.call(m, "local_services_campaign_settings"))
                                    $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.encode(m.local_services_campaign_settings, w.uint32(602).fork()).ldelim();
                                if (m.campaign_group != null && Object.hasOwnProperty.call(m, "campaign_group"))
                                    w.uint32(610).string(m.campaign_group);
                                if (m.performance_max_upgrade != null && Object.hasOwnProperty.call(m, "performance_max_upgrade"))
                                    $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade.encode(m.performance_max_upgrade, w.uint32(618).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified Campaign message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaign} message Campaign message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Campaign.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Campaign message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Campaign} Campaign
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Campaign.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 59:
                                        m.id = r.int64();
                                        break;
                                    case 58:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 21:
                                        m.serving_status = r.int32();
                                        break;
                                    case 8:
                                        m.ad_serving_optimization_status = r.int32();
                                        break;
                                    case 9:
                                        m.advertising_channel_type = r.int32();
                                        break;
                                    case 10:
                                        m.advertising_channel_sub_type = r.int32();
                                        break;
                                    case 60:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 12:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 75:
                                        m.local_services_campaign_settings = $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.decode(r, r.uint32());
                                        break;
                                    case 39:
                                        m.real_time_bidding_setting = $root.google.ads.googleads.v11.common.RealTimeBiddingSetting.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.network_settings = $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.hotel_setting = $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo.decode(r, r.uint32());
                                        break;
                                    case 33:
                                        m.dynamic_search_ads_setting = $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.decode(r, r.uint32());
                                        break;
                                    case 36:
                                        m.shopping_setting = $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting.decode(r, r.uint32());
                                        break;
                                    case 43:
                                        m.targeting_setting = $root.google.ads.googleads.v11.common.TargetingSetting.decode(r, r.uint32());
                                        break;
                                    case 73:
                                        m.audience_setting = $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting.decode(r, r.uint32());
                                        break;
                                    case 47:
                                        m.geo_target_type_setting = $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting.decode(r, r.uint32());
                                        break;
                                    case 50:
                                        m.local_campaign_setting = $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting.decode(r, r.uint32());
                                        break;
                                    case 51:
                                        m.app_campaign_setting = $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting.decode(r, r.uint32());
                                        break;
                                    case 61:
                                        if (!(m.labels && m.labels.length))
                                            m.labels = [];
                                        m.labels.push(r.string());
                                        break;
                                    case 17:
                                        m.experiment_type = r.int32();
                                        break;
                                    case 56:
                                        m.base_campaign = r.string();
                                        break;
                                    case 62:
                                        m.campaign_budget = r.string();
                                        break;
                                    case 22:
                                        m.bidding_strategy_type = r.int32();
                                        break;
                                    case 71:
                                        m.accessible_bidding_strategy = r.string();
                                        break;
                                    case 63:
                                        m.start_date = r.string();
                                        break;
                                    case 76:
                                        m.campaign_group = r.string();
                                        break;
                                    case 64:
                                        m.end_date = r.string();
                                        break;
                                    case 65:
                                        m.final_url_suffix = r.string();
                                        break;
                                    case 40:
                                        if (!(m.frequency_caps && m.frequency_caps.length))
                                            m.frequency_caps = [];
                                        m.frequency_caps.push($root.google.ads.googleads.v11.common.FrequencyCapEntry.decode(r, r.uint32()));
                                        break;
                                    case 42:
                                        m.video_brand_safety_suitability = r.int32();
                                        break;
                                    case 44:
                                        m.vanity_pharma = $root.google.ads.googleads.v11.resources.Campaign.VanityPharma.decode(r, r.uint32());
                                        break;
                                    case 45:
                                        m.selective_optimization = $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.decode(r, r.uint32());
                                        break;
                                    case 54:
                                        m.optimization_goal_setting = $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.decode(r, r.uint32());
                                        break;
                                    case 46:
                                        m.tracking_setting = $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting.decode(r, r.uint32());
                                        break;
                                    case 52:
                                        m.payment_mode = r.int32();
                                        break;
                                    case 66:
                                        m.optimization_score = r.double();
                                        break;
                                    case 69:
                                        if (!(m.excluded_parent_asset_field_types && m.excluded_parent_asset_field_types.length))
                                            m.excluded_parent_asset_field_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.excluded_parent_asset_field_types.push(r.int32());
                                        } else
                                            m.excluded_parent_asset_field_types.push(r.int32());
                                        break;
                                    case 72:
                                        m.url_expansion_opt_out = r.bool();
                                        break;
                                    case 77:
                                        m.performance_max_upgrade = $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade.decode(r, r.uint32());
                                        break;
                                    case 67:
                                        m.bidding_strategy = r.string();
                                        break;
                                    case 49:
                                        m.commission = $root.google.ads.googleads.v11.common.Commission.decode(r, r.uint32());
                                        break;
                                    case 74:
                                        m.manual_cpa = $root.google.ads.googleads.v11.common.ManualCpa.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.manual_cpc = $root.google.ads.googleads.v11.common.ManualCpc.decode(r, r.uint32());
                                        break;
                                    case 25:
                                        m.manual_cpm = $root.google.ads.googleads.v11.common.ManualCpm.decode(r, r.uint32());
                                        break;
                                    case 37:
                                        m.manual_cpv = $root.google.ads.googleads.v11.common.ManualCpv.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.maximize_conversions = $root.google.ads.googleads.v11.common.MaximizeConversions.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.maximize_conversion_value = $root.google.ads.googleads.v11.common.MaximizeConversionValue.decode(r, r.uint32());
                                        break;
                                    case 26:
                                        m.target_cpa = $root.google.ads.googleads.v11.common.TargetCpa.decode(r, r.uint32());
                                        break;
                                    case 48:
                                        m.target_impression_share = $root.google.ads.googleads.v11.common.TargetImpressionShare.decode(r, r.uint32());
                                        break;
                                    case 29:
                                        m.target_roas = $root.google.ads.googleads.v11.common.TargetRoas.decode(r, r.uint32());
                                        break;
                                    case 27:
                                        m.target_spend = $root.google.ads.googleads.v11.common.TargetSpend.decode(r, r.uint32());
                                        break;
                                    case 34:
                                        m.percent_cpc = $root.google.ads.googleads.v11.common.PercentCpc.decode(r, r.uint32());
                                        break;
                                    case 41:
                                        m.target_cpm = $root.google.ads.googleads.v11.common.TargetCpm.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Campaign message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Campaign} Campaign
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Campaign.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Campaign message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Campaign.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.serving_status != null && m.hasOwnProperty("serving_status")) {
                                    switch (m.serving_status) {
                                    default:
                                        return "serving_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.ad_serving_optimization_status != null && m.hasOwnProperty("ad_serving_optimization_status")) {
                                    switch (m.ad_serving_optimization_status) {
                                    default:
                                        return "ad_serving_optimization_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.advertising_channel_type != null && m.hasOwnProperty("advertising_channel_type")) {
                                    switch (m.advertising_channel_type) {
                                    default:
                                        return "advertising_channel_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.advertising_channel_sub_type != null && m.hasOwnProperty("advertising_channel_sub_type")) {
                                    switch (m.advertising_channel_sub_type) {
                                    default:
                                        return "advertising_channel_sub_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                        break;
                                    }
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.local_services_campaign_settings != null && m.hasOwnProperty("local_services_campaign_settings")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.verify(m.local_services_campaign_settings);
                                        if (e)
                                            return "local_services_campaign_settings." + e;
                                    }
                                }
                                if (m.real_time_bidding_setting != null && m.hasOwnProperty("real_time_bidding_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.RealTimeBiddingSetting.verify(m.real_time_bidding_setting);
                                        if (e)
                                            return "real_time_bidding_setting." + e;
                                    }
                                }
                                if (m.network_settings != null && m.hasOwnProperty("network_settings")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings.verify(m.network_settings);
                                        if (e)
                                            return "network_settings." + e;
                                    }
                                }
                                if (m.hotel_setting != null && m.hasOwnProperty("hotel_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo.verify(m.hotel_setting);
                                        if (e)
                                            return "hotel_setting." + e;
                                    }
                                }
                                if (m.dynamic_search_ads_setting != null && m.hasOwnProperty("dynamic_search_ads_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.verify(m.dynamic_search_ads_setting);
                                        if (e)
                                            return "dynamic_search_ads_setting." + e;
                                    }
                                }
                                if (m.shopping_setting != null && m.hasOwnProperty("shopping_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting.verify(m.shopping_setting);
                                        if (e)
                                            return "shopping_setting." + e;
                                    }
                                }
                                if (m.targeting_setting != null && m.hasOwnProperty("targeting_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetingSetting.verify(m.targeting_setting);
                                        if (e)
                                            return "targeting_setting." + e;
                                    }
                                }
                                if (m.audience_setting != null && m.hasOwnProperty("audience_setting")) {
                                    p._audience_setting = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting.verify(m.audience_setting);
                                        if (e)
                                            return "audience_setting." + e;
                                    }
                                }
                                if (m.geo_target_type_setting != null && m.hasOwnProperty("geo_target_type_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting.verify(m.geo_target_type_setting);
                                        if (e)
                                            return "geo_target_type_setting." + e;
                                    }
                                }
                                if (m.local_campaign_setting != null && m.hasOwnProperty("local_campaign_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting.verify(m.local_campaign_setting);
                                        if (e)
                                            return "local_campaign_setting." + e;
                                    }
                                }
                                if (m.app_campaign_setting != null && m.hasOwnProperty("app_campaign_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting.verify(m.app_campaign_setting);
                                        if (e)
                                            return "app_campaign_setting." + e;
                                    }
                                }
                                if (m.labels != null && m.hasOwnProperty("labels")) {
                                    if (!Array.isArray(m.labels))
                                        return "labels: array expected";
                                    for (var i = 0; i < m.labels.length; ++i) {
                                        if (!$util.isString(m.labels[i]))
                                            return "labels: string[] expected";
                                    }
                                }
                                if (m.experiment_type != null && m.hasOwnProperty("experiment_type")) {
                                    switch (m.experiment_type) {
                                    default:
                                        return "experiment_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.base_campaign != null && m.hasOwnProperty("base_campaign")) {
                                    p._base_campaign = 1;
                                    if (!$util.isString(m.base_campaign))
                                        return "base_campaign: string expected";
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    p._campaign_budget = 1;
                                    if (!$util.isString(m.campaign_budget))
                                        return "campaign_budget: string expected";
                                }
                                if (m.bidding_strategy_type != null && m.hasOwnProperty("bidding_strategy_type")) {
                                    switch (m.bidding_strategy_type) {
                                    default:
                                        return "bidding_strategy_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 16:
                                    case 2:
                                    case 17:
                                    case 18:
                                    case 3:
                                    case 4:
                                    case 13:
                                    case 10:
                                    case 11:
                                    case 5:
                                    case 12:
                                    case 6:
                                    case 14:
                                    case 15:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.accessible_bidding_strategy != null && m.hasOwnProperty("accessible_bidding_strategy")) {
                                    if (!$util.isString(m.accessible_bidding_strategy))
                                        return "accessible_bidding_strategy: string expected";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.campaign_group != null && m.hasOwnProperty("campaign_group")) {
                                    p._campaign_group = 1;
                                    if (!$util.isString(m.campaign_group))
                                        return "campaign_group: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    p._end_date = 1;
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                if (m.frequency_caps != null && m.hasOwnProperty("frequency_caps")) {
                                    if (!Array.isArray(m.frequency_caps))
                                        return "frequency_caps: array expected";
                                    for (var i = 0; i < m.frequency_caps.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.FrequencyCapEntry.verify(m.frequency_caps[i]);
                                            if (e)
                                                return "frequency_caps." + e;
                                        }
                                    }
                                }
                                if (m.video_brand_safety_suitability != null && m.hasOwnProperty("video_brand_safety_suitability")) {
                                    switch (m.video_brand_safety_suitability) {
                                    default:
                                        return "video_brand_safety_suitability: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.vanity_pharma != null && m.hasOwnProperty("vanity_pharma")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.VanityPharma.verify(m.vanity_pharma);
                                        if (e)
                                            return "vanity_pharma." + e;
                                    }
                                }
                                if (m.selective_optimization != null && m.hasOwnProperty("selective_optimization")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.verify(m.selective_optimization);
                                        if (e)
                                            return "selective_optimization." + e;
                                    }
                                }
                                if (m.optimization_goal_setting != null && m.hasOwnProperty("optimization_goal_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.verify(m.optimization_goal_setting);
                                        if (e)
                                            return "optimization_goal_setting." + e;
                                    }
                                }
                                if (m.tracking_setting != null && m.hasOwnProperty("tracking_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting.verify(m.tracking_setting);
                                        if (e)
                                            return "tracking_setting." + e;
                                    }
                                }
                                if (m.payment_mode != null && m.hasOwnProperty("payment_mode")) {
                                    switch (m.payment_mode) {
                                    default:
                                        return "payment_mode: enum value expected";
                                    case 0:
                                    case 1:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.optimization_score != null && m.hasOwnProperty("optimization_score")) {
                                    p._optimization_score = 1;
                                    if (typeof m.optimization_score !== "number")
                                        return "optimization_score: number expected";
                                }
                                if (m.excluded_parent_asset_field_types != null && m.hasOwnProperty("excluded_parent_asset_field_types")) {
                                    if (!Array.isArray(m.excluded_parent_asset_field_types))
                                        return "excluded_parent_asset_field_types: array expected";
                                    for (var i = 0; i < m.excluded_parent_asset_field_types.length; ++i) {
                                        switch (m.excluded_parent_asset_field_types[i]) {
                                        default:
                                            return "excluded_parent_asset_field_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 24:
                                        case 17:
                                        case 18:
                                        case 19:
                                        case 20:
                                        case 21:
                                        case 22:
                                        case 23:
                                        case 25:
                                            break;
                                        }
                                    }
                                }
                                if (m.url_expansion_opt_out != null && m.hasOwnProperty("url_expansion_opt_out")) {
                                    p._url_expansion_opt_out = 1;
                                    if (typeof m.url_expansion_opt_out !== "boolean")
                                        return "url_expansion_opt_out: boolean expected";
                                }
                                if (m.performance_max_upgrade != null && m.hasOwnProperty("performance_max_upgrade")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade.verify(m.performance_max_upgrade);
                                        if (e)
                                            return "performance_max_upgrade." + e;
                                    }
                                }
                                if (m.bidding_strategy != null && m.hasOwnProperty("bidding_strategy")) {
                                    p.campaign_bidding_strategy = 1;
                                    if (!$util.isString(m.bidding_strategy))
                                        return "bidding_strategy: string expected";
                                }
                                if (m.commission != null && m.hasOwnProperty("commission")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Commission.verify(m.commission);
                                        if (e)
                                            return "commission." + e;
                                    }
                                }
                                if (m.manual_cpa != null && m.hasOwnProperty("manual_cpa")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ManualCpa.verify(m.manual_cpa);
                                        if (e)
                                            return "manual_cpa." + e;
                                    }
                                }
                                if (m.manual_cpc != null && m.hasOwnProperty("manual_cpc")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ManualCpc.verify(m.manual_cpc);
                                        if (e)
                                            return "manual_cpc." + e;
                                    }
                                }
                                if (m.manual_cpm != null && m.hasOwnProperty("manual_cpm")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ManualCpm.verify(m.manual_cpm);
                                        if (e)
                                            return "manual_cpm." + e;
                                    }
                                }
                                if (m.manual_cpv != null && m.hasOwnProperty("manual_cpv")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ManualCpv.verify(m.manual_cpv);
                                        if (e)
                                            return "manual_cpv." + e;
                                    }
                                }
                                if (m.maximize_conversions != null && m.hasOwnProperty("maximize_conversions")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MaximizeConversions.verify(m.maximize_conversions);
                                        if (e)
                                            return "maximize_conversions." + e;
                                    }
                                }
                                if (m.maximize_conversion_value != null && m.hasOwnProperty("maximize_conversion_value")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MaximizeConversionValue.verify(m.maximize_conversion_value);
                                        if (e)
                                            return "maximize_conversion_value." + e;
                                    }
                                }
                                if (m.target_cpa != null && m.hasOwnProperty("target_cpa")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetCpa.verify(m.target_cpa);
                                        if (e)
                                            return "target_cpa." + e;
                                    }
                                }
                                if (m.target_impression_share != null && m.hasOwnProperty("target_impression_share")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetImpressionShare.verify(m.target_impression_share);
                                        if (e)
                                            return "target_impression_share." + e;
                                    }
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetRoas.verify(m.target_roas);
                                        if (e)
                                            return "target_roas." + e;
                                    }
                                }
                                if (m.target_spend != null && m.hasOwnProperty("target_spend")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetSpend.verify(m.target_spend);
                                        if (e)
                                            return "target_spend." + e;
                                    }
                                }
                                if (m.percent_cpc != null && m.hasOwnProperty("percent_cpc")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PercentCpc.verify(m.percent_cpc);
                                        if (e)
                                            return "percent_cpc." + e;
                                    }
                                }
                                if (m.target_cpm != null && m.hasOwnProperty("target_cpm")) {
                                    if (p.campaign_bidding_strategy === 1)
                                        return "campaign_bidding_strategy: multiple values";
                                    p.campaign_bidding_strategy = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetCpm.verify(m.target_cpm);
                                        if (e)
                                            return "target_cpm." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Campaign message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Campaign} Campaign
                             */
                            Campaign.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Campaign)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Campaign();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "PAUSED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                switch (d.serving_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.serving_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.serving_status = 1;
                                    break;
                                case "SERVING":
                                case 2:
                                    m.serving_status = 2;
                                    break;
                                case "NONE":
                                case 3:
                                    m.serving_status = 3;
                                    break;
                                case "ENDED":
                                case 4:
                                    m.serving_status = 4;
                                    break;
                                case "PENDING":
                                case 5:
                                    m.serving_status = 5;
                                    break;
                                case "SUSPENDED":
                                case 6:
                                    m.serving_status = 6;
                                    break;
                                }
                                switch (d.ad_serving_optimization_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_serving_optimization_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_serving_optimization_status = 1;
                                    break;
                                case "OPTIMIZE":
                                case 2:
                                    m.ad_serving_optimization_status = 2;
                                    break;
                                case "CONVERSION_OPTIMIZE":
                                case 3:
                                    m.ad_serving_optimization_status = 3;
                                    break;
                                case "ROTATE":
                                case 4:
                                    m.ad_serving_optimization_status = 4;
                                    break;
                                case "ROTATE_INDEFINITELY":
                                case 5:
                                    m.ad_serving_optimization_status = 5;
                                    break;
                                case "UNAVAILABLE":
                                case 6:
                                    m.ad_serving_optimization_status = 6;
                                    break;
                                }
                                switch (d.advertising_channel_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.advertising_channel_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.advertising_channel_type = 1;
                                    break;
                                case "SEARCH":
                                case 2:
                                    m.advertising_channel_type = 2;
                                    break;
                                case "DISPLAY":
                                case 3:
                                    m.advertising_channel_type = 3;
                                    break;
                                case "SHOPPING":
                                case 4:
                                    m.advertising_channel_type = 4;
                                    break;
                                case "HOTEL":
                                case 5:
                                    m.advertising_channel_type = 5;
                                    break;
                                case "VIDEO":
                                case 6:
                                    m.advertising_channel_type = 6;
                                    break;
                                case "MULTI_CHANNEL":
                                case 7:
                                    m.advertising_channel_type = 7;
                                    break;
                                case "LOCAL":
                                case 8:
                                    m.advertising_channel_type = 8;
                                    break;
                                case "SMART":
                                case 9:
                                    m.advertising_channel_type = 9;
                                    break;
                                case "PERFORMANCE_MAX":
                                case 10:
                                    m.advertising_channel_type = 10;
                                    break;
                                case "LOCAL_SERVICES":
                                case 11:
                                    m.advertising_channel_type = 11;
                                    break;
                                case "DISCOVERY":
                                case 12:
                                    m.advertising_channel_type = 12;
                                    break;
                                }
                                switch (d.advertising_channel_sub_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.advertising_channel_sub_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.advertising_channel_sub_type = 1;
                                    break;
                                case "SEARCH_MOBILE_APP":
                                case 2:
                                    m.advertising_channel_sub_type = 2;
                                    break;
                                case "DISPLAY_MOBILE_APP":
                                case 3:
                                    m.advertising_channel_sub_type = 3;
                                    break;
                                case "SEARCH_EXPRESS":
                                case 4:
                                    m.advertising_channel_sub_type = 4;
                                    break;
                                case "DISPLAY_EXPRESS":
                                case 5:
                                    m.advertising_channel_sub_type = 5;
                                    break;
                                case "SHOPPING_SMART_ADS":
                                case 6:
                                    m.advertising_channel_sub_type = 6;
                                    break;
                                case "DISPLAY_GMAIL_AD":
                                case 7:
                                    m.advertising_channel_sub_type = 7;
                                    break;
                                case "DISPLAY_SMART_CAMPAIGN":
                                case 8:
                                    m.advertising_channel_sub_type = 8;
                                    break;
                                case "VIDEO_OUTSTREAM":
                                case 9:
                                    m.advertising_channel_sub_type = 9;
                                    break;
                                case "VIDEO_ACTION":
                                case 10:
                                    m.advertising_channel_sub_type = 10;
                                    break;
                                case "VIDEO_NON_SKIPPABLE":
                                case 11:
                                    m.advertising_channel_sub_type = 11;
                                    break;
                                case "APP_CAMPAIGN":
                                case 12:
                                    m.advertising_channel_sub_type = 12;
                                    break;
                                case "APP_CAMPAIGN_FOR_ENGAGEMENT":
                                case 13:
                                    m.advertising_channel_sub_type = 13;
                                    break;
                                case "LOCAL_CAMPAIGN":
                                case 14:
                                    m.advertising_channel_sub_type = 14;
                                    break;
                                case "SHOPPING_COMPARISON_LISTING_ADS":
                                case 15:
                                    m.advertising_channel_sub_type = 15;
                                    break;
                                case "SMART_CAMPAIGN":
                                case 16:
                                    m.advertising_channel_sub_type = 16;
                                    break;
                                case "VIDEO_SEQUENCE":
                                case 17:
                                    m.advertising_channel_sub_type = 17;
                                    break;
                                case "APP_CAMPAIGN_FOR_PRE_REGISTRATION":
                                case 18:
                                    m.advertising_channel_sub_type = 18;
                                    break;
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Campaign.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                if (d.local_services_campaign_settings != null) {
                                    if (typeof d.local_services_campaign_settings !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.local_services_campaign_settings: object expected");
                                    m.local_services_campaign_settings = $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.fromObject(d.local_services_campaign_settings);
                                }
                                if (d.real_time_bidding_setting != null) {
                                    if (typeof d.real_time_bidding_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.real_time_bidding_setting: object expected");
                                    m.real_time_bidding_setting = $root.google.ads.googleads.v11.common.RealTimeBiddingSetting.fromObject(d.real_time_bidding_setting);
                                }
                                if (d.network_settings != null) {
                                    if (typeof d.network_settings !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.network_settings: object expected");
                                    m.network_settings = $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings.fromObject(d.network_settings);
                                }
                                if (d.hotel_setting != null) {
                                    if (typeof d.hotel_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.hotel_setting: object expected");
                                    m.hotel_setting = $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo.fromObject(d.hotel_setting);
                                }
                                if (d.dynamic_search_ads_setting != null) {
                                    if (typeof d.dynamic_search_ads_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.dynamic_search_ads_setting: object expected");
                                    m.dynamic_search_ads_setting = $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.fromObject(d.dynamic_search_ads_setting);
                                }
                                if (d.shopping_setting != null) {
                                    if (typeof d.shopping_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.shopping_setting: object expected");
                                    m.shopping_setting = $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting.fromObject(d.shopping_setting);
                                }
                                if (d.targeting_setting != null) {
                                    if (typeof d.targeting_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.targeting_setting: object expected");
                                    m.targeting_setting = $root.google.ads.googleads.v11.common.TargetingSetting.fromObject(d.targeting_setting);
                                }
                                if (d.audience_setting != null) {
                                    if (typeof d.audience_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.audience_setting: object expected");
                                    m.audience_setting = $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting.fromObject(d.audience_setting);
                                }
                                if (d.geo_target_type_setting != null) {
                                    if (typeof d.geo_target_type_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.geo_target_type_setting: object expected");
                                    m.geo_target_type_setting = $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting.fromObject(d.geo_target_type_setting);
                                }
                                if (d.local_campaign_setting != null) {
                                    if (typeof d.local_campaign_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.local_campaign_setting: object expected");
                                    m.local_campaign_setting = $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting.fromObject(d.local_campaign_setting);
                                }
                                if (d.app_campaign_setting != null) {
                                    if (typeof d.app_campaign_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.app_campaign_setting: object expected");
                                    m.app_campaign_setting = $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting.fromObject(d.app_campaign_setting);
                                }
                                if (d.labels) {
                                    if (!Array.isArray(d.labels))
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.labels: array expected");
                                    m.labels = [];
                                    for (var i = 0; i < d.labels.length; ++i) {
                                        m.labels[i] = String(d.labels[i]);
                                    }
                                }
                                switch (d.experiment_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.experiment_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.experiment_type = 1;
                                    break;
                                case "BASE":
                                case 2:
                                    m.experiment_type = 2;
                                    break;
                                case "DRAFT":
                                case 3:
                                    m.experiment_type = 3;
                                    break;
                                case "EXPERIMENT":
                                case 4:
                                    m.experiment_type = 4;
                                    break;
                                }
                                if (d.base_campaign != null) {
                                    m.base_campaign = String(d.base_campaign);
                                }
                                if (d.campaign_budget != null) {
                                    m.campaign_budget = String(d.campaign_budget);
                                }
                                switch (d.bidding_strategy_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.bidding_strategy_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.bidding_strategy_type = 1;
                                    break;
                                case "COMMISSION":
                                case 16:
                                    m.bidding_strategy_type = 16;
                                    break;
                                case "ENHANCED_CPC":
                                case 2:
                                    m.bidding_strategy_type = 2;
                                    break;
                                case "INVALID":
                                case 17:
                                    m.bidding_strategy_type = 17;
                                    break;
                                case "MANUAL_CPA":
                                case 18:
                                    m.bidding_strategy_type = 18;
                                    break;
                                case "MANUAL_CPC":
                                case 3:
                                    m.bidding_strategy_type = 3;
                                    break;
                                case "MANUAL_CPM":
                                case 4:
                                    m.bidding_strategy_type = 4;
                                    break;
                                case "MANUAL_CPV":
                                case 13:
                                    m.bidding_strategy_type = 13;
                                    break;
                                case "MAXIMIZE_CONVERSIONS":
                                case 10:
                                    m.bidding_strategy_type = 10;
                                    break;
                                case "MAXIMIZE_CONVERSION_VALUE":
                                case 11:
                                    m.bidding_strategy_type = 11;
                                    break;
                                case "PAGE_ONE_PROMOTED":
                                case 5:
                                    m.bidding_strategy_type = 5;
                                    break;
                                case "PERCENT_CPC":
                                case 12:
                                    m.bidding_strategy_type = 12;
                                    break;
                                case "TARGET_CPA":
                                case 6:
                                    m.bidding_strategy_type = 6;
                                    break;
                                case "TARGET_CPM":
                                case 14:
                                    m.bidding_strategy_type = 14;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 15:
                                    m.bidding_strategy_type = 15;
                                    break;
                                case "TARGET_OUTRANK_SHARE":
                                case 7:
                                    m.bidding_strategy_type = 7;
                                    break;
                                case "TARGET_ROAS":
                                case 8:
                                    m.bidding_strategy_type = 8;
                                    break;
                                case "TARGET_SPEND":
                                case 9:
                                    m.bidding_strategy_type = 9;
                                    break;
                                }
                                if (d.accessible_bidding_strategy != null) {
                                    m.accessible_bidding_strategy = String(d.accessible_bidding_strategy);
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.campaign_group != null) {
                                    m.campaign_group = String(d.campaign_group);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                if (d.frequency_caps) {
                                    if (!Array.isArray(d.frequency_caps))
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.frequency_caps: array expected");
                                    m.frequency_caps = [];
                                    for (var i = 0; i < d.frequency_caps.length; ++i) {
                                        if (typeof d.frequency_caps[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Campaign.frequency_caps: object expected");
                                        m.frequency_caps[i] = $root.google.ads.googleads.v11.common.FrequencyCapEntry.fromObject(d.frequency_caps[i]);
                                    }
                                }
                                switch (d.video_brand_safety_suitability) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.video_brand_safety_suitability = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.video_brand_safety_suitability = 1;
                                    break;
                                case "EXPANDED_INVENTORY":
                                case 2:
                                    m.video_brand_safety_suitability = 2;
                                    break;
                                case "STANDARD_INVENTORY":
                                case 3:
                                    m.video_brand_safety_suitability = 3;
                                    break;
                                case "LIMITED_INVENTORY":
                                case 4:
                                    m.video_brand_safety_suitability = 4;
                                    break;
                                }
                                if (d.vanity_pharma != null) {
                                    if (typeof d.vanity_pharma !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.vanity_pharma: object expected");
                                    m.vanity_pharma = $root.google.ads.googleads.v11.resources.Campaign.VanityPharma.fromObject(d.vanity_pharma);
                                }
                                if (d.selective_optimization != null) {
                                    if (typeof d.selective_optimization !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.selective_optimization: object expected");
                                    m.selective_optimization = $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.fromObject(d.selective_optimization);
                                }
                                if (d.optimization_goal_setting != null) {
                                    if (typeof d.optimization_goal_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.optimization_goal_setting: object expected");
                                    m.optimization_goal_setting = $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.fromObject(d.optimization_goal_setting);
                                }
                                if (d.tracking_setting != null) {
                                    if (typeof d.tracking_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.tracking_setting: object expected");
                                    m.tracking_setting = $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting.fromObject(d.tracking_setting);
                                }
                                switch (d.payment_mode) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.payment_mode = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.payment_mode = 1;
                                    break;
                                case "CLICKS":
                                case 4:
                                    m.payment_mode = 4;
                                    break;
                                case "CONVERSION_VALUE":
                                case 5:
                                    m.payment_mode = 5;
                                    break;
                                case "CONVERSIONS":
                                case 6:
                                    m.payment_mode = 6;
                                    break;
                                case "GUEST_STAY":
                                case 7:
                                    m.payment_mode = 7;
                                    break;
                                }
                                if (d.optimization_score != null) {
                                    m.optimization_score = Number(d.optimization_score);
                                }
                                if (d.excluded_parent_asset_field_types) {
                                    if (!Array.isArray(d.excluded_parent_asset_field_types))
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.excluded_parent_asset_field_types: array expected");
                                    m.excluded_parent_asset_field_types = [];
                                    for (var i = 0; i < d.excluded_parent_asset_field_types.length; ++i) {
                                        switch (d.excluded_parent_asset_field_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.excluded_parent_asset_field_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.excluded_parent_asset_field_types[i] = 1;
                                            break;
                                        case "HEADLINE":
                                        case 2:
                                            m.excluded_parent_asset_field_types[i] = 2;
                                            break;
                                        case "DESCRIPTION":
                                        case 3:
                                            m.excluded_parent_asset_field_types[i] = 3;
                                            break;
                                        case "MANDATORY_AD_TEXT":
                                        case 4:
                                            m.excluded_parent_asset_field_types[i] = 4;
                                            break;
                                        case "MARKETING_IMAGE":
                                        case 5:
                                            m.excluded_parent_asset_field_types[i] = 5;
                                            break;
                                        case "MEDIA_BUNDLE":
                                        case 6:
                                            m.excluded_parent_asset_field_types[i] = 6;
                                            break;
                                        case "YOUTUBE_VIDEO":
                                        case 7:
                                            m.excluded_parent_asset_field_types[i] = 7;
                                            break;
                                        case "BOOK_ON_GOOGLE":
                                        case 8:
                                            m.excluded_parent_asset_field_types[i] = 8;
                                            break;
                                        case "LEAD_FORM":
                                        case 9:
                                            m.excluded_parent_asset_field_types[i] = 9;
                                            break;
                                        case "PROMOTION":
                                        case 10:
                                            m.excluded_parent_asset_field_types[i] = 10;
                                            break;
                                        case "CALLOUT":
                                        case 11:
                                            m.excluded_parent_asset_field_types[i] = 11;
                                            break;
                                        case "STRUCTURED_SNIPPET":
                                        case 12:
                                            m.excluded_parent_asset_field_types[i] = 12;
                                            break;
                                        case "SITELINK":
                                        case 13:
                                            m.excluded_parent_asset_field_types[i] = 13;
                                            break;
                                        case "MOBILE_APP":
                                        case 14:
                                            m.excluded_parent_asset_field_types[i] = 14;
                                            break;
                                        case "HOTEL_CALLOUT":
                                        case 15:
                                            m.excluded_parent_asset_field_types[i] = 15;
                                            break;
                                        case "CALL":
                                        case 16:
                                            m.excluded_parent_asset_field_types[i] = 16;
                                            break;
                                        case "PRICE":
                                        case 24:
                                            m.excluded_parent_asset_field_types[i] = 24;
                                            break;
                                        case "LONG_HEADLINE":
                                        case 17:
                                            m.excluded_parent_asset_field_types[i] = 17;
                                            break;
                                        case "BUSINESS_NAME":
                                        case 18:
                                            m.excluded_parent_asset_field_types[i] = 18;
                                            break;
                                        case "SQUARE_MARKETING_IMAGE":
                                        case 19:
                                            m.excluded_parent_asset_field_types[i] = 19;
                                            break;
                                        case "PORTRAIT_MARKETING_IMAGE":
                                        case 20:
                                            m.excluded_parent_asset_field_types[i] = 20;
                                            break;
                                        case "LOGO":
                                        case 21:
                                            m.excluded_parent_asset_field_types[i] = 21;
                                            break;
                                        case "LANDSCAPE_LOGO":
                                        case 22:
                                            m.excluded_parent_asset_field_types[i] = 22;
                                            break;
                                        case "VIDEO":
                                        case 23:
                                            m.excluded_parent_asset_field_types[i] = 23;
                                            break;
                                        case "CALL_TO_ACTION_SELECTION":
                                        case 25:
                                            m.excluded_parent_asset_field_types[i] = 25;
                                            break;
                                        }
                                    }
                                }
                                if (d.url_expansion_opt_out != null) {
                                    m.url_expansion_opt_out = Boolean(d.url_expansion_opt_out);
                                }
                                if (d.performance_max_upgrade != null) {
                                    if (typeof d.performance_max_upgrade !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.performance_max_upgrade: object expected");
                                    m.performance_max_upgrade = $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade.fromObject(d.performance_max_upgrade);
                                }
                                if (d.bidding_strategy != null) {
                                    m.bidding_strategy = String(d.bidding_strategy);
                                }
                                if (d.commission != null) {
                                    if (typeof d.commission !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.commission: object expected");
                                    m.commission = $root.google.ads.googleads.v11.common.Commission.fromObject(d.commission);
                                }
                                if (d.manual_cpa != null) {
                                    if (typeof d.manual_cpa !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.manual_cpa: object expected");
                                    m.manual_cpa = $root.google.ads.googleads.v11.common.ManualCpa.fromObject(d.manual_cpa);
                                }
                                if (d.manual_cpc != null) {
                                    if (typeof d.manual_cpc !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.manual_cpc: object expected");
                                    m.manual_cpc = $root.google.ads.googleads.v11.common.ManualCpc.fromObject(d.manual_cpc);
                                }
                                if (d.manual_cpm != null) {
                                    if (typeof d.manual_cpm !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.manual_cpm: object expected");
                                    m.manual_cpm = $root.google.ads.googleads.v11.common.ManualCpm.fromObject(d.manual_cpm);
                                }
                                if (d.manual_cpv != null) {
                                    if (typeof d.manual_cpv !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.manual_cpv: object expected");
                                    m.manual_cpv = $root.google.ads.googleads.v11.common.ManualCpv.fromObject(d.manual_cpv);
                                }
                                if (d.maximize_conversions != null) {
                                    if (typeof d.maximize_conversions !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.maximize_conversions: object expected");
                                    m.maximize_conversions = $root.google.ads.googleads.v11.common.MaximizeConversions.fromObject(d.maximize_conversions);
                                }
                                if (d.maximize_conversion_value != null) {
                                    if (typeof d.maximize_conversion_value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.maximize_conversion_value: object expected");
                                    m.maximize_conversion_value = $root.google.ads.googleads.v11.common.MaximizeConversionValue.fromObject(d.maximize_conversion_value);
                                }
                                if (d.target_cpa != null) {
                                    if (typeof d.target_cpa !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.target_cpa: object expected");
                                    m.target_cpa = $root.google.ads.googleads.v11.common.TargetCpa.fromObject(d.target_cpa);
                                }
                                if (d.target_impression_share != null) {
                                    if (typeof d.target_impression_share !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.target_impression_share: object expected");
                                    m.target_impression_share = $root.google.ads.googleads.v11.common.TargetImpressionShare.fromObject(d.target_impression_share);
                                }
                                if (d.target_roas != null) {
                                    if (typeof d.target_roas !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.target_roas: object expected");
                                    m.target_roas = $root.google.ads.googleads.v11.common.TargetRoas.fromObject(d.target_roas);
                                }
                                if (d.target_spend != null) {
                                    if (typeof d.target_spend !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.target_spend: object expected");
                                    m.target_spend = $root.google.ads.googleads.v11.common.TargetSpend.fromObject(d.target_spend);
                                }
                                if (d.percent_cpc != null) {
                                    if (typeof d.percent_cpc !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.percent_cpc: object expected");
                                    m.percent_cpc = $root.google.ads.googleads.v11.common.PercentCpc.fromObject(d.percent_cpc);
                                }
                                if (d.target_cpm != null) {
                                    if (typeof d.target_cpm !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Campaign.target_cpm: object expected");
                                    m.target_cpm = $root.google.ads.googleads.v11.common.TargetCpm.fromObject(d.target_cpm);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Campaign message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.Campaign} m Campaign
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Campaign.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.url_custom_parameters = [];
                                    d.frequency_caps = [];
                                    d.labels = [];
                                    d.excluded_parent_asset_field_types = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.ad_serving_optimization_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.advertising_channel_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.advertising_channel_sub_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.network_settings = null;
                                    d.experiment_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.serving_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.bidding_strategy_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.hotel_setting = null;
                                    d.dynamic_search_ads_setting = null;
                                    d.shopping_setting = null;
                                    d.real_time_bidding_setting = null;
                                    d.video_brand_safety_suitability = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.targeting_setting = null;
                                    d.vanity_pharma = null;
                                    d.selective_optimization = null;
                                    d.tracking_setting = null;
                                    d.geo_target_type_setting = null;
                                    d.local_campaign_setting = null;
                                    d.app_campaign_setting = null;
                                    d.payment_mode = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.optimization_goal_setting = null;
                                    d.accessible_bidding_strategy = "";
                                    d.local_services_campaign_settings = null;
                                    d.performance_max_upgrade = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignStatusEnum.CampaignStatus[m.status] : m.status;
                                }
                                if (m.ad_serving_optimization_status != null && m.hasOwnProperty("ad_serving_optimization_status")) {
                                    d.ad_serving_optimization_status = o.enums === String ? $root.google.ads.googleads.v11.enums.AdServingOptimizationStatusEnum.AdServingOptimizationStatus[m.ad_serving_optimization_status] : m.ad_serving_optimization_status;
                                }
                                if (m.advertising_channel_type != null && m.hasOwnProperty("advertising_channel_type")) {
                                    d.advertising_channel_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AdvertisingChannelTypeEnum.AdvertisingChannelType[m.advertising_channel_type] : m.advertising_channel_type;
                                }
                                if (m.advertising_channel_sub_type != null && m.hasOwnProperty("advertising_channel_sub_type")) {
                                    d.advertising_channel_sub_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType[m.advertising_channel_sub_type] : m.advertising_channel_sub_type;
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.network_settings != null && m.hasOwnProperty("network_settings")) {
                                    d.network_settings = $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings.toObject(m.network_settings, o);
                                }
                                if (m.experiment_type != null && m.hasOwnProperty("experiment_type")) {
                                    d.experiment_type = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignExperimentTypeEnum.CampaignExperimentType[m.experiment_type] : m.experiment_type;
                                }
                                if (m.serving_status != null && m.hasOwnProperty("serving_status")) {
                                    d.serving_status = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignServingStatusEnum.CampaignServingStatus[m.serving_status] : m.serving_status;
                                }
                                if (m.bidding_strategy_type != null && m.hasOwnProperty("bidding_strategy_type")) {
                                    d.bidding_strategy_type = o.enums === String ? $root.google.ads.googleads.v11.enums.BiddingStrategyTypeEnum.BiddingStrategyType[m.bidding_strategy_type] : m.bidding_strategy_type;
                                }
                                if (m.manual_cpc != null && m.hasOwnProperty("manual_cpc")) {
                                    d.manual_cpc = $root.google.ads.googleads.v11.common.ManualCpc.toObject(m.manual_cpc, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "manual_cpc";
                                }
                                if (m.manual_cpm != null && m.hasOwnProperty("manual_cpm")) {
                                    d.manual_cpm = $root.google.ads.googleads.v11.common.ManualCpm.toObject(m.manual_cpm, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "manual_cpm";
                                }
                                if (m.target_cpa != null && m.hasOwnProperty("target_cpa")) {
                                    d.target_cpa = $root.google.ads.googleads.v11.common.TargetCpa.toObject(m.target_cpa, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "target_cpa";
                                }
                                if (m.target_spend != null && m.hasOwnProperty("target_spend")) {
                                    d.target_spend = $root.google.ads.googleads.v11.common.TargetSpend.toObject(m.target_spend, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "target_spend";
                                }
                                if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                    d.target_roas = $root.google.ads.googleads.v11.common.TargetRoas.toObject(m.target_roas, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "target_roas";
                                }
                                if (m.maximize_conversions != null && m.hasOwnProperty("maximize_conversions")) {
                                    d.maximize_conversions = $root.google.ads.googleads.v11.common.MaximizeConversions.toObject(m.maximize_conversions, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "maximize_conversions";
                                }
                                if (m.maximize_conversion_value != null && m.hasOwnProperty("maximize_conversion_value")) {
                                    d.maximize_conversion_value = $root.google.ads.googleads.v11.common.MaximizeConversionValue.toObject(m.maximize_conversion_value, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "maximize_conversion_value";
                                }
                                if (m.hotel_setting != null && m.hasOwnProperty("hotel_setting")) {
                                    d.hotel_setting = $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo.toObject(m.hotel_setting, o);
                                }
                                if (m.dynamic_search_ads_setting != null && m.hasOwnProperty("dynamic_search_ads_setting")) {
                                    d.dynamic_search_ads_setting = $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.toObject(m.dynamic_search_ads_setting, o);
                                }
                                if (m.percent_cpc != null && m.hasOwnProperty("percent_cpc")) {
                                    d.percent_cpc = $root.google.ads.googleads.v11.common.PercentCpc.toObject(m.percent_cpc, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "percent_cpc";
                                }
                                if (m.shopping_setting != null && m.hasOwnProperty("shopping_setting")) {
                                    d.shopping_setting = $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting.toObject(m.shopping_setting, o);
                                }
                                if (m.manual_cpv != null && m.hasOwnProperty("manual_cpv")) {
                                    d.manual_cpv = $root.google.ads.googleads.v11.common.ManualCpv.toObject(m.manual_cpv, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "manual_cpv";
                                }
                                if (m.real_time_bidding_setting != null && m.hasOwnProperty("real_time_bidding_setting")) {
                                    d.real_time_bidding_setting = $root.google.ads.googleads.v11.common.RealTimeBiddingSetting.toObject(m.real_time_bidding_setting, o);
                                }
                                if (m.frequency_caps && m.frequency_caps.length) {
                                    d.frequency_caps = [];
                                    for (var j = 0; j < m.frequency_caps.length; ++j) {
                                        d.frequency_caps[j] = $root.google.ads.googleads.v11.common.FrequencyCapEntry.toObject(m.frequency_caps[j], o);
                                    }
                                }
                                if (m.target_cpm != null && m.hasOwnProperty("target_cpm")) {
                                    d.target_cpm = $root.google.ads.googleads.v11.common.TargetCpm.toObject(m.target_cpm, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "target_cpm";
                                }
                                if (m.video_brand_safety_suitability != null && m.hasOwnProperty("video_brand_safety_suitability")) {
                                    d.video_brand_safety_suitability = o.enums === String ? $root.google.ads.googleads.v11.enums.BrandSafetySuitabilityEnum.BrandSafetySuitability[m.video_brand_safety_suitability] : m.video_brand_safety_suitability;
                                }
                                if (m.targeting_setting != null && m.hasOwnProperty("targeting_setting")) {
                                    d.targeting_setting = $root.google.ads.googleads.v11.common.TargetingSetting.toObject(m.targeting_setting, o);
                                }
                                if (m.vanity_pharma != null && m.hasOwnProperty("vanity_pharma")) {
                                    d.vanity_pharma = $root.google.ads.googleads.v11.resources.Campaign.VanityPharma.toObject(m.vanity_pharma, o);
                                }
                                if (m.selective_optimization != null && m.hasOwnProperty("selective_optimization")) {
                                    d.selective_optimization = $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.toObject(m.selective_optimization, o);
                                }
                                if (m.tracking_setting != null && m.hasOwnProperty("tracking_setting")) {
                                    d.tracking_setting = $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting.toObject(m.tracking_setting, o);
                                }
                                if (m.geo_target_type_setting != null && m.hasOwnProperty("geo_target_type_setting")) {
                                    d.geo_target_type_setting = $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting.toObject(m.geo_target_type_setting, o);
                                }
                                if (m.target_impression_share != null && m.hasOwnProperty("target_impression_share")) {
                                    d.target_impression_share = $root.google.ads.googleads.v11.common.TargetImpressionShare.toObject(m.target_impression_share, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "target_impression_share";
                                }
                                if (m.commission != null && m.hasOwnProperty("commission")) {
                                    d.commission = $root.google.ads.googleads.v11.common.Commission.toObject(m.commission, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "commission";
                                }
                                if (m.local_campaign_setting != null && m.hasOwnProperty("local_campaign_setting")) {
                                    d.local_campaign_setting = $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting.toObject(m.local_campaign_setting, o);
                                }
                                if (m.app_campaign_setting != null && m.hasOwnProperty("app_campaign_setting")) {
                                    d.app_campaign_setting = $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting.toObject(m.app_campaign_setting, o);
                                }
                                if (m.payment_mode != null && m.hasOwnProperty("payment_mode")) {
                                    d.payment_mode = o.enums === String ? $root.google.ads.googleads.v11.enums.PaymentModeEnum.PaymentMode[m.payment_mode] : m.payment_mode;
                                }
                                if (m.optimization_goal_setting != null && m.hasOwnProperty("optimization_goal_setting")) {
                                    d.optimization_goal_setting = $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.toObject(m.optimization_goal_setting, o);
                                }
                                if (m.base_campaign != null && m.hasOwnProperty("base_campaign")) {
                                    d.base_campaign = m.base_campaign;
                                    if (o.oneofs)
                                        d._base_campaign = "base_campaign";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.labels && m.labels.length) {
                                    d.labels = [];
                                    for (var j = 0; j < m.labels.length; ++j) {
                                        d.labels[j] = m.labels[j];
                                    }
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    d.campaign_budget = m.campaign_budget;
                                    if (o.oneofs)
                                        d._campaign_budget = "campaign_budget";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                    if (o.oneofs)
                                        d._end_date = "end_date";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                if (m.optimization_score != null && m.hasOwnProperty("optimization_score")) {
                                    d.optimization_score = o.json && !isFinite(m.optimization_score) ? String(m.optimization_score) : m.optimization_score;
                                    if (o.oneofs)
                                        d._optimization_score = "optimization_score";
                                }
                                if (m.bidding_strategy != null && m.hasOwnProperty("bidding_strategy")) {
                                    d.bidding_strategy = m.bidding_strategy;
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "bidding_strategy";
                                }
                                if (m.excluded_parent_asset_field_types && m.excluded_parent_asset_field_types.length) {
                                    d.excluded_parent_asset_field_types = [];
                                    for (var j = 0; j < m.excluded_parent_asset_field_types.length; ++j) {
                                        d.excluded_parent_asset_field_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.excluded_parent_asset_field_types[j]] : m.excluded_parent_asset_field_types[j];
                                    }
                                }
                                if (m.accessible_bidding_strategy != null && m.hasOwnProperty("accessible_bidding_strategy")) {
                                    d.accessible_bidding_strategy = m.accessible_bidding_strategy;
                                }
                                if (m.url_expansion_opt_out != null && m.hasOwnProperty("url_expansion_opt_out")) {
                                    d.url_expansion_opt_out = m.url_expansion_opt_out;
                                    if (o.oneofs)
                                        d._url_expansion_opt_out = "url_expansion_opt_out";
                                }
                                if (m.audience_setting != null && m.hasOwnProperty("audience_setting")) {
                                    d.audience_setting = $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting.toObject(m.audience_setting, o);
                                    if (o.oneofs)
                                        d._audience_setting = "audience_setting";
                                }
                                if (m.manual_cpa != null && m.hasOwnProperty("manual_cpa")) {
                                    d.manual_cpa = $root.google.ads.googleads.v11.common.ManualCpa.toObject(m.manual_cpa, o);
                                    if (o.oneofs)
                                        d.campaign_bidding_strategy = "manual_cpa";
                                }
                                if (m.local_services_campaign_settings != null && m.hasOwnProperty("local_services_campaign_settings")) {
                                    d.local_services_campaign_settings = $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.toObject(m.local_services_campaign_settings, o);
                                }
                                if (m.campaign_group != null && m.hasOwnProperty("campaign_group")) {
                                    d.campaign_group = m.campaign_group;
                                    if (o.oneofs)
                                        d._campaign_group = "campaign_group";
                                }
                                if (m.performance_max_upgrade != null && m.hasOwnProperty("performance_max_upgrade")) {
                                    d.performance_max_upgrade = $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade.toObject(m.performance_max_upgrade, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Campaign to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Campaign
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Campaign.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            Campaign.PerformanceMaxUpgrade = (function() {
    
                                /**
                                 * Properties of a PerformanceMaxUpgrade.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IPerformanceMaxUpgrade
                                 * @property {string|null} [performance_max_campaign] PerformanceMaxUpgrade performance_max_campaign
                                 * @property {string|null} [pre_upgrade_campaign] PerformanceMaxUpgrade pre_upgrade_campaign
                                 * @property {google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus|null} [status] PerformanceMaxUpgrade status
                                 */
    
                                /**
                                 * Constructs a new PerformanceMaxUpgrade.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a PerformanceMaxUpgrade.
                                 * @implements IPerformanceMaxUpgrade
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IPerformanceMaxUpgrade=} [p] Properties to set
                                 */
                                function PerformanceMaxUpgrade(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * PerformanceMaxUpgrade performance_max_campaign.
                                 * @member {string} performance_max_campaign
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @instance
                                 */
                                PerformanceMaxUpgrade.prototype.performance_max_campaign = "";
    
                                /**
                                 * PerformanceMaxUpgrade pre_upgrade_campaign.
                                 * @member {string} pre_upgrade_campaign
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @instance
                                 */
                                PerformanceMaxUpgrade.prototype.pre_upgrade_campaign = "";
    
                                /**
                                 * PerformanceMaxUpgrade status.
                                 * @member {google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus} status
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @instance
                                 */
                                PerformanceMaxUpgrade.prototype.status = 0;
    
                                /**
                                 * Creates a new PerformanceMaxUpgrade instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IPerformanceMaxUpgrade=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade} PerformanceMaxUpgrade instance
                                 */
                                PerformanceMaxUpgrade.create = function create(properties) {
                                    return new PerformanceMaxUpgrade(properties);
                                };
    
                                /**
                                 * Encodes the specified PerformanceMaxUpgrade message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IPerformanceMaxUpgrade} m PerformanceMaxUpgrade message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PerformanceMaxUpgrade.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.performance_max_campaign != null && Object.hasOwnProperty.call(m, "performance_max_campaign"))
                                        w.uint32(10).string(m.performance_max_campaign);
                                    if (m.pre_upgrade_campaign != null && Object.hasOwnProperty.call(m, "pre_upgrade_campaign"))
                                        w.uint32(18).string(m.pre_upgrade_campaign);
                                    if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                        w.uint32(24).int32(m.status);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified PerformanceMaxUpgrade message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IPerformanceMaxUpgrade} message PerformanceMaxUpgrade message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PerformanceMaxUpgrade.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a PerformanceMaxUpgrade message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade} PerformanceMaxUpgrade
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PerformanceMaxUpgrade.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.performance_max_campaign = r.string();
                                            break;
                                        case 2:
                                            m.pre_upgrade_campaign = r.string();
                                            break;
                                        case 3:
                                            m.status = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a PerformanceMaxUpgrade message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade} PerformanceMaxUpgrade
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PerformanceMaxUpgrade.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a PerformanceMaxUpgrade message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                PerformanceMaxUpgrade.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.performance_max_campaign != null && m.hasOwnProperty("performance_max_campaign")) {
                                        if (!$util.isString(m.performance_max_campaign))
                                            return "performance_max_campaign: string expected";
                                    }
                                    if (m.pre_upgrade_campaign != null && m.hasOwnProperty("pre_upgrade_campaign")) {
                                        if (!$util.isString(m.pre_upgrade_campaign))
                                            return "pre_upgrade_campaign: string expected";
                                    }
                                    if (m.status != null && m.hasOwnProperty("status")) {
                                        switch (m.status) {
                                        default:
                                            return "status: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a PerformanceMaxUpgrade message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade} PerformanceMaxUpgrade
                                 */
                                PerformanceMaxUpgrade.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade();
                                    if (d.performance_max_campaign != null) {
                                        m.performance_max_campaign = String(d.performance_max_campaign);
                                    }
                                    if (d.pre_upgrade_campaign != null) {
                                        m.pre_upgrade_campaign = String(d.pre_upgrade_campaign);
                                    }
                                    switch (d.status) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.status = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.status = 1;
                                        break;
                                    case "UPGRADE_ELIBIGLE":
                                    case 2:
                                        m.status = 2;
                                        break;
                                    case "UPGRADE_IN_PROGRESS":
                                    case 3:
                                        m.status = 3;
                                        break;
                                    case "UPGRADE_COMPLETE":
                                    case 4:
                                        m.status = 4;
                                        break;
                                    case "UPGRADE_FAILED":
                                    case 5:
                                        m.status = 5;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a PerformanceMaxUpgrade message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade} m PerformanceMaxUpgrade
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                PerformanceMaxUpgrade.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.performance_max_campaign = "";
                                        d.pre_upgrade_campaign = "";
                                        d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.performance_max_campaign != null && m.hasOwnProperty("performance_max_campaign")) {
                                        d.performance_max_campaign = m.performance_max_campaign;
                                    }
                                    if (m.pre_upgrade_campaign != null && m.hasOwnProperty("pre_upgrade_campaign")) {
                                        d.pre_upgrade_campaign = m.pre_upgrade_campaign;
                                    }
                                    if (m.status != null && m.hasOwnProperty("status")) {
                                        d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus[m.status] : m.status;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this PerformanceMaxUpgrade to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.PerformanceMaxUpgrade
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                PerformanceMaxUpgrade.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return PerformanceMaxUpgrade;
                            })();
    
                            Campaign.NetworkSettings = (function() {
    
                                /**
                                 * Properties of a NetworkSettings.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface INetworkSettings
                                 * @property {boolean|null} [target_google_search] NetworkSettings target_google_search
                                 * @property {boolean|null} [target_search_network] NetworkSettings target_search_network
                                 * @property {boolean|null} [target_content_network] NetworkSettings target_content_network
                                 * @property {boolean|null} [target_partner_search_network] NetworkSettings target_partner_search_network
                                 */
    
                                /**
                                 * Constructs a new NetworkSettings.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a NetworkSettings.
                                 * @implements INetworkSettings
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.INetworkSettings=} [p] Properties to set
                                 */
                                function NetworkSettings(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * NetworkSettings target_google_search.
                                 * @member {boolean|null|undefined} target_google_search
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                NetworkSettings.prototype.target_google_search = null;
    
                                /**
                                 * NetworkSettings target_search_network.
                                 * @member {boolean|null|undefined} target_search_network
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                NetworkSettings.prototype.target_search_network = null;
    
                                /**
                                 * NetworkSettings target_content_network.
                                 * @member {boolean|null|undefined} target_content_network
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                NetworkSettings.prototype.target_content_network = null;
    
                                /**
                                 * NetworkSettings target_partner_search_network.
                                 * @member {boolean|null|undefined} target_partner_search_network
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                NetworkSettings.prototype.target_partner_search_network = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * NetworkSettings _target_google_search.
                                 * @member {"target_google_search"|undefined} _target_google_search
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                Object.defineProperty(NetworkSettings.prototype, "_target_google_search", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_google_search"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * NetworkSettings _target_search_network.
                                 * @member {"target_search_network"|undefined} _target_search_network
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                Object.defineProperty(NetworkSettings.prototype, "_target_search_network", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_search_network"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * NetworkSettings _target_content_network.
                                 * @member {"target_content_network"|undefined} _target_content_network
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                Object.defineProperty(NetworkSettings.prototype, "_target_content_network", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_content_network"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * NetworkSettings _target_partner_search_network.
                                 * @member {"target_partner_search_network"|undefined} _target_partner_search_network
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 */
                                Object.defineProperty(NetworkSettings.prototype, "_target_partner_search_network", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_partner_search_network"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new NetworkSettings instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.INetworkSettings=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.NetworkSettings} NetworkSettings instance
                                 */
                                NetworkSettings.create = function create(properties) {
                                    return new NetworkSettings(properties);
                                };
    
                                /**
                                 * Encodes the specified NetworkSettings message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.NetworkSettings.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.INetworkSettings} m NetworkSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                NetworkSettings.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_google_search != null && Object.hasOwnProperty.call(m, "target_google_search"))
                                        w.uint32(40).bool(m.target_google_search);
                                    if (m.target_search_network != null && Object.hasOwnProperty.call(m, "target_search_network"))
                                        w.uint32(48).bool(m.target_search_network);
                                    if (m.target_content_network != null && Object.hasOwnProperty.call(m, "target_content_network"))
                                        w.uint32(56).bool(m.target_content_network);
                                    if (m.target_partner_search_network != null && Object.hasOwnProperty.call(m, "target_partner_search_network"))
                                        w.uint32(64).bool(m.target_partner_search_network);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified NetworkSettings message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.NetworkSettings.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.INetworkSettings} message NetworkSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                NetworkSettings.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a NetworkSettings message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.NetworkSettings} NetworkSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                NetworkSettings.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 5:
                                            m.target_google_search = r.bool();
                                            break;
                                        case 6:
                                            m.target_search_network = r.bool();
                                            break;
                                        case 7:
                                            m.target_content_network = r.bool();
                                            break;
                                        case 8:
                                            m.target_partner_search_network = r.bool();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a NetworkSettings message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.NetworkSettings} NetworkSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                NetworkSettings.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a NetworkSettings message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                NetworkSettings.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.target_google_search != null && m.hasOwnProperty("target_google_search")) {
                                        p._target_google_search = 1;
                                        if (typeof m.target_google_search !== "boolean")
                                            return "target_google_search: boolean expected";
                                    }
                                    if (m.target_search_network != null && m.hasOwnProperty("target_search_network")) {
                                        p._target_search_network = 1;
                                        if (typeof m.target_search_network !== "boolean")
                                            return "target_search_network: boolean expected";
                                    }
                                    if (m.target_content_network != null && m.hasOwnProperty("target_content_network")) {
                                        p._target_content_network = 1;
                                        if (typeof m.target_content_network !== "boolean")
                                            return "target_content_network: boolean expected";
                                    }
                                    if (m.target_partner_search_network != null && m.hasOwnProperty("target_partner_search_network")) {
                                        p._target_partner_search_network = 1;
                                        if (typeof m.target_partner_search_network !== "boolean")
                                            return "target_partner_search_network: boolean expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a NetworkSettings message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.NetworkSettings} NetworkSettings
                                 */
                                NetworkSettings.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.NetworkSettings();
                                    if (d.target_google_search != null) {
                                        m.target_google_search = Boolean(d.target_google_search);
                                    }
                                    if (d.target_search_network != null) {
                                        m.target_search_network = Boolean(d.target_search_network);
                                    }
                                    if (d.target_content_network != null) {
                                        m.target_content_network = Boolean(d.target_content_network);
                                    }
                                    if (d.target_partner_search_network != null) {
                                        m.target_partner_search_network = Boolean(d.target_partner_search_network);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a NetworkSettings message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.NetworkSettings} m NetworkSettings
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                NetworkSettings.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.target_google_search != null && m.hasOwnProperty("target_google_search")) {
                                        d.target_google_search = m.target_google_search;
                                        if (o.oneofs)
                                            d._target_google_search = "target_google_search";
                                    }
                                    if (m.target_search_network != null && m.hasOwnProperty("target_search_network")) {
                                        d.target_search_network = m.target_search_network;
                                        if (o.oneofs)
                                            d._target_search_network = "target_search_network";
                                    }
                                    if (m.target_content_network != null && m.hasOwnProperty("target_content_network")) {
                                        d.target_content_network = m.target_content_network;
                                        if (o.oneofs)
                                            d._target_content_network = "target_content_network";
                                    }
                                    if (m.target_partner_search_network != null && m.hasOwnProperty("target_partner_search_network")) {
                                        d.target_partner_search_network = m.target_partner_search_network;
                                        if (o.oneofs)
                                            d._target_partner_search_network = "target_partner_search_network";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this NetworkSettings to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.NetworkSettings
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                NetworkSettings.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return NetworkSettings;
                            })();
    
                            Campaign.HotelSettingInfo = (function() {
    
                                /**
                                 * Properties of a HotelSettingInfo.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IHotelSettingInfo
                                 * @property {number|null} [hotel_center_id] HotelSettingInfo hotel_center_id
                                 */
    
                                /**
                                 * Constructs a new HotelSettingInfo.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a HotelSettingInfo.
                                 * @implements IHotelSettingInfo
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IHotelSettingInfo=} [p] Properties to set
                                 */
                                function HotelSettingInfo(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * HotelSettingInfo hotel_center_id.
                                 * @member {number|null|undefined} hotel_center_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @instance
                                 */
                                HotelSettingInfo.prototype.hotel_center_id = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * HotelSettingInfo _hotel_center_id.
                                 * @member {"hotel_center_id"|undefined} _hotel_center_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @instance
                                 */
                                Object.defineProperty(HotelSettingInfo.prototype, "_hotel_center_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["hotel_center_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new HotelSettingInfo instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IHotelSettingInfo=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.HotelSettingInfo} HotelSettingInfo instance
                                 */
                                HotelSettingInfo.create = function create(properties) {
                                    return new HotelSettingInfo(properties);
                                };
    
                                /**
                                 * Encodes the specified HotelSettingInfo message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.HotelSettingInfo.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IHotelSettingInfo} m HotelSettingInfo message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                HotelSettingInfo.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.hotel_center_id != null && Object.hasOwnProperty.call(m, "hotel_center_id"))
                                        w.uint32(16).int64(m.hotel_center_id);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified HotelSettingInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.HotelSettingInfo.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IHotelSettingInfo} message HotelSettingInfo message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                HotelSettingInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a HotelSettingInfo message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.HotelSettingInfo} HotelSettingInfo
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                HotelSettingInfo.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.hotel_center_id = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a HotelSettingInfo message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.HotelSettingInfo} HotelSettingInfo
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                HotelSettingInfo.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a HotelSettingInfo message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                HotelSettingInfo.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                        p._hotel_center_id = 1;
                                        if (!$util.isInteger(m.hotel_center_id) && !(m.hotel_center_id && $util.isInteger(m.hotel_center_id.low) && $util.isInteger(m.hotel_center_id.high)))
                                            return "hotel_center_id: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a HotelSettingInfo message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.HotelSettingInfo} HotelSettingInfo
                                 */
                                HotelSettingInfo.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.HotelSettingInfo();
                                    if (d.hotel_center_id != null) {
                                        if ($util.Long)
                                            (m.hotel_center_id = $util.Long.fromValue(d.hotel_center_id)).unsigned = false;
                                        else if (typeof d.hotel_center_id === "string")
                                            m.hotel_center_id = parseInt(d.hotel_center_id, 10);
                                        else if (typeof d.hotel_center_id === "number")
                                            m.hotel_center_id = d.hotel_center_id;
                                        else if (typeof d.hotel_center_id === "object")
                                            m.hotel_center_id = new $util.LongBits(d.hotel_center_id.low >>> 0, d.hotel_center_id.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a HotelSettingInfo message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.HotelSettingInfo} m HotelSettingInfo
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                HotelSettingInfo.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                        if (typeof m.hotel_center_id === "number")
                                            d.hotel_center_id = o.longs === String ? String(m.hotel_center_id) : m.hotel_center_id;
                                        else
                                            d.hotel_center_id = o.longs === String ? $util.Long.prototype.toString.call(m.hotel_center_id) : o.longs === Number ? new $util.LongBits(m.hotel_center_id.low >>> 0, m.hotel_center_id.high >>> 0).toNumber() : m.hotel_center_id;
                                        if (o.oneofs)
                                            d._hotel_center_id = "hotel_center_id";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this HotelSettingInfo to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.HotelSettingInfo
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                HotelSettingInfo.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return HotelSettingInfo;
                            })();
    
                            Campaign.DynamicSearchAdsSetting = (function() {
    
                                /**
                                 * Properties of a DynamicSearchAdsSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IDynamicSearchAdsSetting
                                 * @property {string|null} [domain_name] DynamicSearchAdsSetting domain_name
                                 * @property {string|null} [language_code] DynamicSearchAdsSetting language_code
                                 * @property {boolean|null} [use_supplied_urls_only] DynamicSearchAdsSetting use_supplied_urls_only
                                 * @property {Array.<string>|null} [feeds] DynamicSearchAdsSetting feeds
                                 */
    
                                /**
                                 * Constructs a new DynamicSearchAdsSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a DynamicSearchAdsSetting.
                                 * @implements IDynamicSearchAdsSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IDynamicSearchAdsSetting=} [p] Properties to set
                                 */
                                function DynamicSearchAdsSetting(p) {
                                    this.feeds = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * DynamicSearchAdsSetting domain_name.
                                 * @member {string} domain_name
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @instance
                                 */
                                DynamicSearchAdsSetting.prototype.domain_name = "";
    
                                /**
                                 * DynamicSearchAdsSetting language_code.
                                 * @member {string} language_code
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @instance
                                 */
                                DynamicSearchAdsSetting.prototype.language_code = "";
    
                                /**
                                 * DynamicSearchAdsSetting use_supplied_urls_only.
                                 * @member {boolean|null|undefined} use_supplied_urls_only
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @instance
                                 */
                                DynamicSearchAdsSetting.prototype.use_supplied_urls_only = null;
    
                                /**
                                 * DynamicSearchAdsSetting feeds.
                                 * @member {Array.<string>} feeds
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @instance
                                 */
                                DynamicSearchAdsSetting.prototype.feeds = $util.emptyArray;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * DynamicSearchAdsSetting _use_supplied_urls_only.
                                 * @member {"use_supplied_urls_only"|undefined} _use_supplied_urls_only
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @instance
                                 */
                                Object.defineProperty(DynamicSearchAdsSetting.prototype, "_use_supplied_urls_only", {
                                    get: $util.oneOfGetter($oneOfFields = ["use_supplied_urls_only"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new DynamicSearchAdsSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IDynamicSearchAdsSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting} DynamicSearchAdsSetting instance
                                 */
                                DynamicSearchAdsSetting.create = function create(properties) {
                                    return new DynamicSearchAdsSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified DynamicSearchAdsSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IDynamicSearchAdsSetting} m DynamicSearchAdsSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DynamicSearchAdsSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.domain_name != null && Object.hasOwnProperty.call(m, "domain_name"))
                                        w.uint32(50).string(m.domain_name);
                                    if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                        w.uint32(58).string(m.language_code);
                                    if (m.use_supplied_urls_only != null && Object.hasOwnProperty.call(m, "use_supplied_urls_only"))
                                        w.uint32(64).bool(m.use_supplied_urls_only);
                                    if (m.feeds != null && m.feeds.length) {
                                        for (var i = 0; i < m.feeds.length; ++i)
                                            w.uint32(74).string(m.feeds[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified DynamicSearchAdsSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IDynamicSearchAdsSetting} message DynamicSearchAdsSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DynamicSearchAdsSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a DynamicSearchAdsSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting} DynamicSearchAdsSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DynamicSearchAdsSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 6:
                                            m.domain_name = r.string();
                                            break;
                                        case 7:
                                            m.language_code = r.string();
                                            break;
                                        case 8:
                                            m.use_supplied_urls_only = r.bool();
                                            break;
                                        case 9:
                                            if (!(m.feeds && m.feeds.length))
                                                m.feeds = [];
                                            m.feeds.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a DynamicSearchAdsSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting} DynamicSearchAdsSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DynamicSearchAdsSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a DynamicSearchAdsSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                DynamicSearchAdsSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.domain_name != null && m.hasOwnProperty("domain_name")) {
                                        if (!$util.isString(m.domain_name))
                                            return "domain_name: string expected";
                                    }
                                    if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                        if (!$util.isString(m.language_code))
                                            return "language_code: string expected";
                                    }
                                    if (m.use_supplied_urls_only != null && m.hasOwnProperty("use_supplied_urls_only")) {
                                        p._use_supplied_urls_only = 1;
                                        if (typeof m.use_supplied_urls_only !== "boolean")
                                            return "use_supplied_urls_only: boolean expected";
                                    }
                                    if (m.feeds != null && m.hasOwnProperty("feeds")) {
                                        if (!Array.isArray(m.feeds))
                                            return "feeds: array expected";
                                        for (var i = 0; i < m.feeds.length; ++i) {
                                            if (!$util.isString(m.feeds[i]))
                                                return "feeds: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a DynamicSearchAdsSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting} DynamicSearchAdsSetting
                                 */
                                DynamicSearchAdsSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting();
                                    if (d.domain_name != null) {
                                        m.domain_name = String(d.domain_name);
                                    }
                                    if (d.language_code != null) {
                                        m.language_code = String(d.language_code);
                                    }
                                    if (d.use_supplied_urls_only != null) {
                                        m.use_supplied_urls_only = Boolean(d.use_supplied_urls_only);
                                    }
                                    if (d.feeds) {
                                        if (!Array.isArray(d.feeds))
                                            throw TypeError(".google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting.feeds: array expected");
                                        m.feeds = [];
                                        for (var i = 0; i < d.feeds.length; ++i) {
                                            m.feeds[i] = String(d.feeds[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a DynamicSearchAdsSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting} m DynamicSearchAdsSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                DynamicSearchAdsSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.feeds = [];
                                    }
                                    if (o.defaults) {
                                        d.domain_name = "";
                                        d.language_code = "";
                                    }
                                    if (m.domain_name != null && m.hasOwnProperty("domain_name")) {
                                        d.domain_name = m.domain_name;
                                    }
                                    if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                        d.language_code = m.language_code;
                                    }
                                    if (m.use_supplied_urls_only != null && m.hasOwnProperty("use_supplied_urls_only")) {
                                        d.use_supplied_urls_only = m.use_supplied_urls_only;
                                        if (o.oneofs)
                                            d._use_supplied_urls_only = "use_supplied_urls_only";
                                    }
                                    if (m.feeds && m.feeds.length) {
                                        d.feeds = [];
                                        for (var j = 0; j < m.feeds.length; ++j) {
                                            d.feeds[j] = m.feeds[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this DynamicSearchAdsSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.DynamicSearchAdsSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                DynamicSearchAdsSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return DynamicSearchAdsSetting;
                            })();
    
                            Campaign.ShoppingSetting = (function() {
    
                                /**
                                 * Properties of a ShoppingSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IShoppingSetting
                                 * @property {number|null} [merchant_id] ShoppingSetting merchant_id
                                 * @property {string|null} [sales_country] ShoppingSetting sales_country
                                 * @property {string|null} [feed_label] ShoppingSetting feed_label
                                 * @property {number|null} [campaign_priority] ShoppingSetting campaign_priority
                                 * @property {boolean|null} [enable_local] ShoppingSetting enable_local
                                 * @property {boolean|null} [use_vehicle_inventory] ShoppingSetting use_vehicle_inventory
                                 */
    
                                /**
                                 * Constructs a new ShoppingSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a ShoppingSetting.
                                 * @implements IShoppingSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IShoppingSetting=} [p] Properties to set
                                 */
                                function ShoppingSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ShoppingSetting merchant_id.
                                 * @member {number|null|undefined} merchant_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                ShoppingSetting.prototype.merchant_id = null;
    
                                /**
                                 * ShoppingSetting sales_country.
                                 * @member {string|null|undefined} sales_country
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                ShoppingSetting.prototype.sales_country = null;
    
                                /**
                                 * ShoppingSetting feed_label.
                                 * @member {string} feed_label
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                ShoppingSetting.prototype.feed_label = "";
    
                                /**
                                 * ShoppingSetting campaign_priority.
                                 * @member {number|null|undefined} campaign_priority
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                ShoppingSetting.prototype.campaign_priority = null;
    
                                /**
                                 * ShoppingSetting enable_local.
                                 * @member {boolean|null|undefined} enable_local
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                ShoppingSetting.prototype.enable_local = null;
    
                                /**
                                 * ShoppingSetting use_vehicle_inventory.
                                 * @member {boolean} use_vehicle_inventory
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                ShoppingSetting.prototype.use_vehicle_inventory = false;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ShoppingSetting _merchant_id.
                                 * @member {"merchant_id"|undefined} _merchant_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                Object.defineProperty(ShoppingSetting.prototype, "_merchant_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["merchant_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * ShoppingSetting _sales_country.
                                 * @member {"sales_country"|undefined} _sales_country
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                Object.defineProperty(ShoppingSetting.prototype, "_sales_country", {
                                    get: $util.oneOfGetter($oneOfFields = ["sales_country"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * ShoppingSetting _campaign_priority.
                                 * @member {"campaign_priority"|undefined} _campaign_priority
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                Object.defineProperty(ShoppingSetting.prototype, "_campaign_priority", {
                                    get: $util.oneOfGetter($oneOfFields = ["campaign_priority"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * ShoppingSetting _enable_local.
                                 * @member {"enable_local"|undefined} _enable_local
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 */
                                Object.defineProperty(ShoppingSetting.prototype, "_enable_local", {
                                    get: $util.oneOfGetter($oneOfFields = ["enable_local"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ShoppingSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IShoppingSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.ShoppingSetting} ShoppingSetting instance
                                 */
                                ShoppingSetting.create = function create(properties) {
                                    return new ShoppingSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified ShoppingSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.ShoppingSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IShoppingSetting} m ShoppingSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ShoppingSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.merchant_id != null && Object.hasOwnProperty.call(m, "merchant_id"))
                                        w.uint32(40).int64(m.merchant_id);
                                    if (m.sales_country != null && Object.hasOwnProperty.call(m, "sales_country"))
                                        w.uint32(50).string(m.sales_country);
                                    if (m.campaign_priority != null && Object.hasOwnProperty.call(m, "campaign_priority"))
                                        w.uint32(56).int32(m.campaign_priority);
                                    if (m.enable_local != null && Object.hasOwnProperty.call(m, "enable_local"))
                                        w.uint32(64).bool(m.enable_local);
                                    if (m.use_vehicle_inventory != null && Object.hasOwnProperty.call(m, "use_vehicle_inventory"))
                                        w.uint32(72).bool(m.use_vehicle_inventory);
                                    if (m.feed_label != null && Object.hasOwnProperty.call(m, "feed_label"))
                                        w.uint32(82).string(m.feed_label);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ShoppingSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.ShoppingSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IShoppingSetting} message ShoppingSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ShoppingSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ShoppingSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.ShoppingSetting} ShoppingSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ShoppingSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 5:
                                            m.merchant_id = r.int64();
                                            break;
                                        case 6:
                                            m.sales_country = r.string();
                                            break;
                                        case 10:
                                            m.feed_label = r.string();
                                            break;
                                        case 7:
                                            m.campaign_priority = r.int32();
                                            break;
                                        case 8:
                                            m.enable_local = r.bool();
                                            break;
                                        case 9:
                                            m.use_vehicle_inventory = r.bool();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ShoppingSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.ShoppingSetting} ShoppingSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ShoppingSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ShoppingSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ShoppingSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                        p._merchant_id = 1;
                                        if (!$util.isInteger(m.merchant_id) && !(m.merchant_id && $util.isInteger(m.merchant_id.low) && $util.isInteger(m.merchant_id.high)))
                                            return "merchant_id: integer|Long expected";
                                    }
                                    if (m.sales_country != null && m.hasOwnProperty("sales_country")) {
                                        p._sales_country = 1;
                                        if (!$util.isString(m.sales_country))
                                            return "sales_country: string expected";
                                    }
                                    if (m.feed_label != null && m.hasOwnProperty("feed_label")) {
                                        if (!$util.isString(m.feed_label))
                                            return "feed_label: string expected";
                                    }
                                    if (m.campaign_priority != null && m.hasOwnProperty("campaign_priority")) {
                                        p._campaign_priority = 1;
                                        if (!$util.isInteger(m.campaign_priority))
                                            return "campaign_priority: integer expected";
                                    }
                                    if (m.enable_local != null && m.hasOwnProperty("enable_local")) {
                                        p._enable_local = 1;
                                        if (typeof m.enable_local !== "boolean")
                                            return "enable_local: boolean expected";
                                    }
                                    if (m.use_vehicle_inventory != null && m.hasOwnProperty("use_vehicle_inventory")) {
                                        if (typeof m.use_vehicle_inventory !== "boolean")
                                            return "use_vehicle_inventory: boolean expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ShoppingSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.ShoppingSetting} ShoppingSetting
                                 */
                                ShoppingSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.ShoppingSetting();
                                    if (d.merchant_id != null) {
                                        if ($util.Long)
                                            (m.merchant_id = $util.Long.fromValue(d.merchant_id)).unsigned = false;
                                        else if (typeof d.merchant_id === "string")
                                            m.merchant_id = parseInt(d.merchant_id, 10);
                                        else if (typeof d.merchant_id === "number")
                                            m.merchant_id = d.merchant_id;
                                        else if (typeof d.merchant_id === "object")
                                            m.merchant_id = new $util.LongBits(d.merchant_id.low >>> 0, d.merchant_id.high >>> 0).toNumber();
                                    }
                                    if (d.sales_country != null) {
                                        m.sales_country = String(d.sales_country);
                                    }
                                    if (d.feed_label != null) {
                                        m.feed_label = String(d.feed_label);
                                    }
                                    if (d.campaign_priority != null) {
                                        m.campaign_priority = d.campaign_priority | 0;
                                    }
                                    if (d.enable_local != null) {
                                        m.enable_local = Boolean(d.enable_local);
                                    }
                                    if (d.use_vehicle_inventory != null) {
                                        m.use_vehicle_inventory = Boolean(d.use_vehicle_inventory);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ShoppingSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ShoppingSetting} m ShoppingSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ShoppingSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.use_vehicle_inventory = false;
                                        d.feed_label = "";
                                    }
                                    if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                        if (typeof m.merchant_id === "number")
                                            d.merchant_id = o.longs === String ? String(m.merchant_id) : m.merchant_id;
                                        else
                                            d.merchant_id = o.longs === String ? $util.Long.prototype.toString.call(m.merchant_id) : o.longs === Number ? new $util.LongBits(m.merchant_id.low >>> 0, m.merchant_id.high >>> 0).toNumber() : m.merchant_id;
                                        if (o.oneofs)
                                            d._merchant_id = "merchant_id";
                                    }
                                    if (m.sales_country != null && m.hasOwnProperty("sales_country")) {
                                        d.sales_country = m.sales_country;
                                        if (o.oneofs)
                                            d._sales_country = "sales_country";
                                    }
                                    if (m.campaign_priority != null && m.hasOwnProperty("campaign_priority")) {
                                        d.campaign_priority = m.campaign_priority;
                                        if (o.oneofs)
                                            d._campaign_priority = "campaign_priority";
                                    }
                                    if (m.enable_local != null && m.hasOwnProperty("enable_local")) {
                                        d.enable_local = m.enable_local;
                                        if (o.oneofs)
                                            d._enable_local = "enable_local";
                                    }
                                    if (m.use_vehicle_inventory != null && m.hasOwnProperty("use_vehicle_inventory")) {
                                        d.use_vehicle_inventory = m.use_vehicle_inventory;
                                    }
                                    if (m.feed_label != null && m.hasOwnProperty("feed_label")) {
                                        d.feed_label = m.feed_label;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ShoppingSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.ShoppingSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ShoppingSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ShoppingSetting;
                            })();
    
                            Campaign.TrackingSetting = (function() {
    
                                /**
                                 * Properties of a TrackingSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface ITrackingSetting
                                 * @property {string|null} [tracking_url] TrackingSetting tracking_url
                                 */
    
                                /**
                                 * Constructs a new TrackingSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a TrackingSetting.
                                 * @implements ITrackingSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.ITrackingSetting=} [p] Properties to set
                                 */
                                function TrackingSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TrackingSetting tracking_url.
                                 * @member {string|null|undefined} tracking_url
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @instance
                                 */
                                TrackingSetting.prototype.tracking_url = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TrackingSetting _tracking_url.
                                 * @member {"tracking_url"|undefined} _tracking_url
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @instance
                                 */
                                Object.defineProperty(TrackingSetting.prototype, "_tracking_url", {
                                    get: $util.oneOfGetter($oneOfFields = ["tracking_url"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TrackingSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ITrackingSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.TrackingSetting} TrackingSetting instance
                                 */
                                TrackingSetting.create = function create(properties) {
                                    return new TrackingSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified TrackingSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.TrackingSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ITrackingSetting} m TrackingSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TrackingSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.tracking_url != null && Object.hasOwnProperty.call(m, "tracking_url"))
                                        w.uint32(18).string(m.tracking_url);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TrackingSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.TrackingSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ITrackingSetting} message TrackingSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TrackingSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TrackingSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.TrackingSetting} TrackingSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TrackingSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.tracking_url = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TrackingSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.TrackingSetting} TrackingSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TrackingSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TrackingSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TrackingSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.tracking_url != null && m.hasOwnProperty("tracking_url")) {
                                        p._tracking_url = 1;
                                        if (!$util.isString(m.tracking_url))
                                            return "tracking_url: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TrackingSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.TrackingSetting} TrackingSetting
                                 */
                                TrackingSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.TrackingSetting();
                                    if (d.tracking_url != null) {
                                        m.tracking_url = String(d.tracking_url);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TrackingSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.TrackingSetting} m TrackingSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TrackingSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.tracking_url != null && m.hasOwnProperty("tracking_url")) {
                                        d.tracking_url = m.tracking_url;
                                        if (o.oneofs)
                                            d._tracking_url = "tracking_url";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TrackingSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.TrackingSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TrackingSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TrackingSetting;
                            })();
    
                            Campaign.GeoTargetTypeSetting = (function() {
    
                                /**
                                 * Properties of a GeoTargetTypeSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IGeoTargetTypeSetting
                                 * @property {google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum.PositiveGeoTargetType|null} [positive_geo_target_type] GeoTargetTypeSetting positive_geo_target_type
                                 * @property {google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum.NegativeGeoTargetType|null} [negative_geo_target_type] GeoTargetTypeSetting negative_geo_target_type
                                 */
    
                                /**
                                 * Constructs a new GeoTargetTypeSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a GeoTargetTypeSetting.
                                 * @implements IGeoTargetTypeSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IGeoTargetTypeSetting=} [p] Properties to set
                                 */
                                function GeoTargetTypeSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * GeoTargetTypeSetting positive_geo_target_type.
                                 * @member {google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum.PositiveGeoTargetType} positive_geo_target_type
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @instance
                                 */
                                GeoTargetTypeSetting.prototype.positive_geo_target_type = 0;
    
                                /**
                                 * GeoTargetTypeSetting negative_geo_target_type.
                                 * @member {google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum.NegativeGeoTargetType} negative_geo_target_type
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @instance
                                 */
                                GeoTargetTypeSetting.prototype.negative_geo_target_type = 0;
    
                                /**
                                 * Creates a new GeoTargetTypeSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IGeoTargetTypeSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting} GeoTargetTypeSetting instance
                                 */
                                GeoTargetTypeSetting.create = function create(properties) {
                                    return new GeoTargetTypeSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified GeoTargetTypeSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IGeoTargetTypeSetting} m GeoTargetTypeSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                GeoTargetTypeSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.positive_geo_target_type != null && Object.hasOwnProperty.call(m, "positive_geo_target_type"))
                                        w.uint32(8).int32(m.positive_geo_target_type);
                                    if (m.negative_geo_target_type != null && Object.hasOwnProperty.call(m, "negative_geo_target_type"))
                                        w.uint32(16).int32(m.negative_geo_target_type);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified GeoTargetTypeSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IGeoTargetTypeSetting} message GeoTargetTypeSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                GeoTargetTypeSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a GeoTargetTypeSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting} GeoTargetTypeSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                GeoTargetTypeSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.positive_geo_target_type = r.int32();
                                            break;
                                        case 2:
                                            m.negative_geo_target_type = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a GeoTargetTypeSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting} GeoTargetTypeSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                GeoTargetTypeSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a GeoTargetTypeSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                GeoTargetTypeSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.positive_geo_target_type != null && m.hasOwnProperty("positive_geo_target_type")) {
                                        switch (m.positive_geo_target_type) {
                                        default:
                                            return "positive_geo_target_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 5:
                                        case 6:
                                        case 7:
                                            break;
                                        }
                                    }
                                    if (m.negative_geo_target_type != null && m.hasOwnProperty("negative_geo_target_type")) {
                                        switch (m.negative_geo_target_type) {
                                        default:
                                            return "negative_geo_target_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 4:
                                        case 5:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a GeoTargetTypeSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting} GeoTargetTypeSetting
                                 */
                                GeoTargetTypeSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting();
                                    switch (d.positive_geo_target_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.positive_geo_target_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.positive_geo_target_type = 1;
                                        break;
                                    case "PRESENCE_OR_INTEREST":
                                    case 5:
                                        m.positive_geo_target_type = 5;
                                        break;
                                    case "SEARCH_INTEREST":
                                    case 6:
                                        m.positive_geo_target_type = 6;
                                        break;
                                    case "PRESENCE":
                                    case 7:
                                        m.positive_geo_target_type = 7;
                                        break;
                                    }
                                    switch (d.negative_geo_target_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.negative_geo_target_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.negative_geo_target_type = 1;
                                        break;
                                    case "PRESENCE_OR_INTEREST":
                                    case 4:
                                        m.negative_geo_target_type = 4;
                                        break;
                                    case "PRESENCE":
                                    case 5:
                                        m.negative_geo_target_type = 5;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a GeoTargetTypeSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting} m GeoTargetTypeSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                GeoTargetTypeSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.positive_geo_target_type = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.negative_geo_target_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.positive_geo_target_type != null && m.hasOwnProperty("positive_geo_target_type")) {
                                        d.positive_geo_target_type = o.enums === String ? $root.google.ads.googleads.v11.enums.PositiveGeoTargetTypeEnum.PositiveGeoTargetType[m.positive_geo_target_type] : m.positive_geo_target_type;
                                    }
                                    if (m.negative_geo_target_type != null && m.hasOwnProperty("negative_geo_target_type")) {
                                        d.negative_geo_target_type = o.enums === String ? $root.google.ads.googleads.v11.enums.NegativeGeoTargetTypeEnum.NegativeGeoTargetType[m.negative_geo_target_type] : m.negative_geo_target_type;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this GeoTargetTypeSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.GeoTargetTypeSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                GeoTargetTypeSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return GeoTargetTypeSetting;
                            })();
    
                            Campaign.LocalCampaignSetting = (function() {
    
                                /**
                                 * Properties of a LocalCampaignSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface ILocalCampaignSetting
                                 * @property {google.ads.googleads.v11.enums.LocationSourceTypeEnum.LocationSourceType|null} [location_source_type] LocalCampaignSetting location_source_type
                                 */
    
                                /**
                                 * Constructs a new LocalCampaignSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a LocalCampaignSetting.
                                 * @implements ILocalCampaignSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalCampaignSetting=} [p] Properties to set
                                 */
                                function LocalCampaignSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * LocalCampaignSetting location_source_type.
                                 * @member {google.ads.googleads.v11.enums.LocationSourceTypeEnum.LocationSourceType} location_source_type
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @instance
                                 */
                                LocalCampaignSetting.prototype.location_source_type = 0;
    
                                /**
                                 * Creates a new LocalCampaignSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalCampaignSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting} LocalCampaignSetting instance
                                 */
                                LocalCampaignSetting.create = function create(properties) {
                                    return new LocalCampaignSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified LocalCampaignSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalCampaignSetting} m LocalCampaignSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocalCampaignSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.location_source_type != null && Object.hasOwnProperty.call(m, "location_source_type"))
                                        w.uint32(8).int32(m.location_source_type);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified LocalCampaignSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalCampaignSetting} message LocalCampaignSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocalCampaignSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a LocalCampaignSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting} LocalCampaignSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocalCampaignSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.location_source_type = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a LocalCampaignSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting} LocalCampaignSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocalCampaignSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a LocalCampaignSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                LocalCampaignSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.location_source_type != null && m.hasOwnProperty("location_source_type")) {
                                        switch (m.location_source_type) {
                                        default:
                                            return "location_source_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a LocalCampaignSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting} LocalCampaignSetting
                                 */
                                LocalCampaignSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting();
                                    switch (d.location_source_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.location_source_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.location_source_type = 1;
                                        break;
                                    case "GOOGLE_MY_BUSINESS":
                                    case 2:
                                        m.location_source_type = 2;
                                        break;
                                    case "AFFILIATE":
                                    case 3:
                                        m.location_source_type = 3;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a LocalCampaignSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting} m LocalCampaignSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                LocalCampaignSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.location_source_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.location_source_type != null && m.hasOwnProperty("location_source_type")) {
                                        d.location_source_type = o.enums === String ? $root.google.ads.googleads.v11.enums.LocationSourceTypeEnum.LocationSourceType[m.location_source_type] : m.location_source_type;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this LocalCampaignSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalCampaignSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                LocalCampaignSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return LocalCampaignSetting;
                            })();
    
                            Campaign.AppCampaignSetting = (function() {
    
                                /**
                                 * Properties of an AppCampaignSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IAppCampaignSetting
                                 * @property {google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType|null} [bidding_strategy_goal_type] AppCampaignSetting bidding_strategy_goal_type
                                 * @property {string|null} [app_id] AppCampaignSetting app_id
                                 * @property {google.ads.googleads.v11.enums.AppCampaignAppStoreEnum.AppCampaignAppStore|null} [app_store] AppCampaignSetting app_store
                                 */
    
                                /**
                                 * Constructs a new AppCampaignSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents an AppCampaignSetting.
                                 * @implements IAppCampaignSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAppCampaignSetting=} [p] Properties to set
                                 */
                                function AppCampaignSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * AppCampaignSetting bidding_strategy_goal_type.
                                 * @member {google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType} bidding_strategy_goal_type
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @instance
                                 */
                                AppCampaignSetting.prototype.bidding_strategy_goal_type = 0;
    
                                /**
                                 * AppCampaignSetting app_id.
                                 * @member {string|null|undefined} app_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @instance
                                 */
                                AppCampaignSetting.prototype.app_id = null;
    
                                /**
                                 * AppCampaignSetting app_store.
                                 * @member {google.ads.googleads.v11.enums.AppCampaignAppStoreEnum.AppCampaignAppStore} app_store
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @instance
                                 */
                                AppCampaignSetting.prototype.app_store = 0;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * AppCampaignSetting _app_id.
                                 * @member {"app_id"|undefined} _app_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @instance
                                 */
                                Object.defineProperty(AppCampaignSetting.prototype, "_app_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["app_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new AppCampaignSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAppCampaignSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AppCampaignSetting} AppCampaignSetting instance
                                 */
                                AppCampaignSetting.create = function create(properties) {
                                    return new AppCampaignSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified AppCampaignSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.AppCampaignSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAppCampaignSetting} m AppCampaignSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AppCampaignSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.bidding_strategy_goal_type != null && Object.hasOwnProperty.call(m, "bidding_strategy_goal_type"))
                                        w.uint32(8).int32(m.bidding_strategy_goal_type);
                                    if (m.app_store != null && Object.hasOwnProperty.call(m, "app_store"))
                                        w.uint32(24).int32(m.app_store);
                                    if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                        w.uint32(34).string(m.app_id);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified AppCampaignSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.AppCampaignSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAppCampaignSetting} message AppCampaignSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AppCampaignSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an AppCampaignSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AppCampaignSetting} AppCampaignSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AppCampaignSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.bidding_strategy_goal_type = r.int32();
                                            break;
                                        case 4:
                                            m.app_id = r.string();
                                            break;
                                        case 3:
                                            m.app_store = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an AppCampaignSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AppCampaignSetting} AppCampaignSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AppCampaignSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an AppCampaignSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                AppCampaignSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.bidding_strategy_goal_type != null && m.hasOwnProperty("bidding_strategy_goal_type")) {
                                        switch (m.bidding_strategy_goal_type) {
                                        default:
                                            return "bidding_strategy_goal_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                            break;
                                        }
                                    }
                                    if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                        p._app_id = 1;
                                        if (!$util.isString(m.app_id))
                                            return "app_id: string expected";
                                    }
                                    if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                        switch (m.app_store) {
                                        default:
                                            return "app_store: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an AppCampaignSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AppCampaignSetting} AppCampaignSetting
                                 */
                                AppCampaignSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.AppCampaignSetting();
                                    switch (d.bidding_strategy_goal_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.bidding_strategy_goal_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.bidding_strategy_goal_type = 1;
                                        break;
                                    case "OPTIMIZE_INSTALLS_TARGET_INSTALL_COST":
                                    case 2:
                                        m.bidding_strategy_goal_type = 2;
                                        break;
                                    case "OPTIMIZE_IN_APP_CONVERSIONS_TARGET_INSTALL_COST":
                                    case 3:
                                        m.bidding_strategy_goal_type = 3;
                                        break;
                                    case "OPTIMIZE_IN_APP_CONVERSIONS_TARGET_CONVERSION_COST":
                                    case 4:
                                        m.bidding_strategy_goal_type = 4;
                                        break;
                                    case "OPTIMIZE_RETURN_ON_ADVERTISING_SPEND":
                                    case 5:
                                        m.bidding_strategy_goal_type = 5;
                                        break;
                                    case "OPTIMIZE_PRE_REGISTRATION_CONVERSION_VOLUME":
                                    case 6:
                                        m.bidding_strategy_goal_type = 6;
                                        break;
                                    case "OPTIMIZE_INSTALLS_WITHOUT_TARGET_INSTALL_COST":
                                    case 7:
                                        m.bidding_strategy_goal_type = 7;
                                        break;
                                    }
                                    if (d.app_id != null) {
                                        m.app_id = String(d.app_id);
                                    }
                                    switch (d.app_store) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.app_store = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.app_store = 1;
                                        break;
                                    case "APPLE_APP_STORE":
                                    case 2:
                                        m.app_store = 2;
                                        break;
                                    case "GOOGLE_APP_STORE":
                                    case 3:
                                        m.app_store = 3;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an AppCampaignSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.AppCampaignSetting} m AppCampaignSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                AppCampaignSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.bidding_strategy_goal_type = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.app_store = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.bidding_strategy_goal_type != null && m.hasOwnProperty("bidding_strategy_goal_type")) {
                                        d.bidding_strategy_goal_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType[m.bidding_strategy_goal_type] : m.bidding_strategy_goal_type;
                                    }
                                    if (m.app_store != null && m.hasOwnProperty("app_store")) {
                                        d.app_store = o.enums === String ? $root.google.ads.googleads.v11.enums.AppCampaignAppStoreEnum.AppCampaignAppStore[m.app_store] : m.app_store;
                                    }
                                    if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                        d.app_id = m.app_id;
                                        if (o.oneofs)
                                            d._app_id = "app_id";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this AppCampaignSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AppCampaignSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                AppCampaignSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return AppCampaignSetting;
                            })();
    
                            Campaign.VanityPharma = (function() {
    
                                /**
                                 * Properties of a VanityPharma.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IVanityPharma
                                 * @property {google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode|null} [vanity_pharma_display_url_mode] VanityPharma vanity_pharma_display_url_mode
                                 * @property {google.ads.googleads.v11.enums.VanityPharmaTextEnum.VanityPharmaText|null} [vanity_pharma_text] VanityPharma vanity_pharma_text
                                 */
    
                                /**
                                 * Constructs a new VanityPharma.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a VanityPharma.
                                 * @implements IVanityPharma
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IVanityPharma=} [p] Properties to set
                                 */
                                function VanityPharma(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * VanityPharma vanity_pharma_display_url_mode.
                                 * @member {google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode} vanity_pharma_display_url_mode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @instance
                                 */
                                VanityPharma.prototype.vanity_pharma_display_url_mode = 0;
    
                                /**
                                 * VanityPharma vanity_pharma_text.
                                 * @member {google.ads.googleads.v11.enums.VanityPharmaTextEnum.VanityPharmaText} vanity_pharma_text
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @instance
                                 */
                                VanityPharma.prototype.vanity_pharma_text = 0;
    
                                /**
                                 * Creates a new VanityPharma instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IVanityPharma=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.VanityPharma} VanityPharma instance
                                 */
                                VanityPharma.create = function create(properties) {
                                    return new VanityPharma(properties);
                                };
    
                                /**
                                 * Encodes the specified VanityPharma message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.VanityPharma.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IVanityPharma} m VanityPharma message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                VanityPharma.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.vanity_pharma_display_url_mode != null && Object.hasOwnProperty.call(m, "vanity_pharma_display_url_mode"))
                                        w.uint32(8).int32(m.vanity_pharma_display_url_mode);
                                    if (m.vanity_pharma_text != null && Object.hasOwnProperty.call(m, "vanity_pharma_text"))
                                        w.uint32(16).int32(m.vanity_pharma_text);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified VanityPharma message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.VanityPharma.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IVanityPharma} message VanityPharma message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                VanityPharma.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a VanityPharma message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.VanityPharma} VanityPharma
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                VanityPharma.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.VanityPharma();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.vanity_pharma_display_url_mode = r.int32();
                                            break;
                                        case 2:
                                            m.vanity_pharma_text = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a VanityPharma message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.VanityPharma} VanityPharma
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                VanityPharma.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a VanityPharma message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                VanityPharma.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.vanity_pharma_display_url_mode != null && m.hasOwnProperty("vanity_pharma_display_url_mode")) {
                                        switch (m.vanity_pharma_display_url_mode) {
                                        default:
                                            return "vanity_pharma_display_url_mode: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    if (m.vanity_pharma_text != null && m.hasOwnProperty("vanity_pharma_text")) {
                                        switch (m.vanity_pharma_text) {
                                        default:
                                            return "vanity_pharma_text: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a VanityPharma message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.VanityPharma} VanityPharma
                                 */
                                VanityPharma.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.VanityPharma)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.VanityPharma();
                                    switch (d.vanity_pharma_display_url_mode) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.vanity_pharma_display_url_mode = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.vanity_pharma_display_url_mode = 1;
                                        break;
                                    case "MANUFACTURER_WEBSITE_URL":
                                    case 2:
                                        m.vanity_pharma_display_url_mode = 2;
                                        break;
                                    case "WEBSITE_DESCRIPTION":
                                    case 3:
                                        m.vanity_pharma_display_url_mode = 3;
                                        break;
                                    }
                                    switch (d.vanity_pharma_text) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.vanity_pharma_text = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.vanity_pharma_text = 1;
                                        break;
                                    case "PRESCRIPTION_TREATMENT_WEBSITE_EN":
                                    case 2:
                                        m.vanity_pharma_text = 2;
                                        break;
                                    case "PRESCRIPTION_TREATMENT_WEBSITE_ES":
                                    case 3:
                                        m.vanity_pharma_text = 3;
                                        break;
                                    case "PRESCRIPTION_DEVICE_WEBSITE_EN":
                                    case 4:
                                        m.vanity_pharma_text = 4;
                                        break;
                                    case "PRESCRIPTION_DEVICE_WEBSITE_ES":
                                    case 5:
                                        m.vanity_pharma_text = 5;
                                        break;
                                    case "MEDICAL_DEVICE_WEBSITE_EN":
                                    case 6:
                                        m.vanity_pharma_text = 6;
                                        break;
                                    case "MEDICAL_DEVICE_WEBSITE_ES":
                                    case 7:
                                        m.vanity_pharma_text = 7;
                                        break;
                                    case "PREVENTATIVE_TREATMENT_WEBSITE_EN":
                                    case 8:
                                        m.vanity_pharma_text = 8;
                                        break;
                                    case "PREVENTATIVE_TREATMENT_WEBSITE_ES":
                                    case 9:
                                        m.vanity_pharma_text = 9;
                                        break;
                                    case "PRESCRIPTION_CONTRACEPTION_WEBSITE_EN":
                                    case 10:
                                        m.vanity_pharma_text = 10;
                                        break;
                                    case "PRESCRIPTION_CONTRACEPTION_WEBSITE_ES":
                                    case 11:
                                        m.vanity_pharma_text = 11;
                                        break;
                                    case "PRESCRIPTION_VACCINE_WEBSITE_EN":
                                    case 12:
                                        m.vanity_pharma_text = 12;
                                        break;
                                    case "PRESCRIPTION_VACCINE_WEBSITE_ES":
                                    case 13:
                                        m.vanity_pharma_text = 13;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a VanityPharma message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.VanityPharma} m VanityPharma
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                VanityPharma.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.vanity_pharma_display_url_mode = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.vanity_pharma_text = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.vanity_pharma_display_url_mode != null && m.hasOwnProperty("vanity_pharma_display_url_mode")) {
                                        d.vanity_pharma_display_url_mode = o.enums === String ? $root.google.ads.googleads.v11.enums.VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode[m.vanity_pharma_display_url_mode] : m.vanity_pharma_display_url_mode;
                                    }
                                    if (m.vanity_pharma_text != null && m.hasOwnProperty("vanity_pharma_text")) {
                                        d.vanity_pharma_text = o.enums === String ? $root.google.ads.googleads.v11.enums.VanityPharmaTextEnum.VanityPharmaText[m.vanity_pharma_text] : m.vanity_pharma_text;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this VanityPharma to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.VanityPharma
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                VanityPharma.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return VanityPharma;
                            })();
    
                            Campaign.SelectiveOptimization = (function() {
    
                                /**
                                 * Properties of a SelectiveOptimization.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface ISelectiveOptimization
                                 * @property {Array.<string>|null} [conversion_actions] SelectiveOptimization conversion_actions
                                 */
    
                                /**
                                 * Constructs a new SelectiveOptimization.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a SelectiveOptimization.
                                 * @implements ISelectiveOptimization
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.ISelectiveOptimization=} [p] Properties to set
                                 */
                                function SelectiveOptimization(p) {
                                    this.conversion_actions = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * SelectiveOptimization conversion_actions.
                                 * @member {Array.<string>} conversion_actions
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @instance
                                 */
                                SelectiveOptimization.prototype.conversion_actions = $util.emptyArray;
    
                                /**
                                 * Creates a new SelectiveOptimization instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ISelectiveOptimization=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.SelectiveOptimization} SelectiveOptimization instance
                                 */
                                SelectiveOptimization.create = function create(properties) {
                                    return new SelectiveOptimization(properties);
                                };
    
                                /**
                                 * Encodes the specified SelectiveOptimization message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ISelectiveOptimization} m SelectiveOptimization message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SelectiveOptimization.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.conversion_actions != null && m.conversion_actions.length) {
                                        for (var i = 0; i < m.conversion_actions.length; ++i)
                                            w.uint32(18).string(m.conversion_actions[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified SelectiveOptimization message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ISelectiveOptimization} message SelectiveOptimization message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SelectiveOptimization.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a SelectiveOptimization message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.SelectiveOptimization} SelectiveOptimization
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SelectiveOptimization.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            if (!(m.conversion_actions && m.conversion_actions.length))
                                                m.conversion_actions = [];
                                            m.conversion_actions.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a SelectiveOptimization message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.SelectiveOptimization} SelectiveOptimization
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SelectiveOptimization.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a SelectiveOptimization message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                SelectiveOptimization.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.conversion_actions != null && m.hasOwnProperty("conversion_actions")) {
                                        if (!Array.isArray(m.conversion_actions))
                                            return "conversion_actions: array expected";
                                        for (var i = 0; i < m.conversion_actions.length; ++i) {
                                            if (!$util.isString(m.conversion_actions[i]))
                                                return "conversion_actions: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a SelectiveOptimization message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.SelectiveOptimization} SelectiveOptimization
                                 */
                                SelectiveOptimization.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.SelectiveOptimization();
                                    if (d.conversion_actions) {
                                        if (!Array.isArray(d.conversion_actions))
                                            throw TypeError(".google.ads.googleads.v11.resources.Campaign.SelectiveOptimization.conversion_actions: array expected");
                                        m.conversion_actions = [];
                                        for (var i = 0; i < d.conversion_actions.length; ++i) {
                                            m.conversion_actions[i] = String(d.conversion_actions[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a SelectiveOptimization message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.SelectiveOptimization} m SelectiveOptimization
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                SelectiveOptimization.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.conversion_actions = [];
                                    }
                                    if (m.conversion_actions && m.conversion_actions.length) {
                                        d.conversion_actions = [];
                                        for (var j = 0; j < m.conversion_actions.length; ++j) {
                                            d.conversion_actions[j] = m.conversion_actions[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this SelectiveOptimization to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.SelectiveOptimization
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                SelectiveOptimization.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return SelectiveOptimization;
                            })();
    
                            Campaign.OptimizationGoalSetting = (function() {
    
                                /**
                                 * Properties of an OptimizationGoalSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IOptimizationGoalSetting
                                 * @property {Array.<google.ads.googleads.v11.enums.OptimizationGoalTypeEnum.OptimizationGoalType>|null} [optimization_goal_types] OptimizationGoalSetting optimization_goal_types
                                 */
    
                                /**
                                 * Constructs a new OptimizationGoalSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents an OptimizationGoalSetting.
                                 * @implements IOptimizationGoalSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IOptimizationGoalSetting=} [p] Properties to set
                                 */
                                function OptimizationGoalSetting(p) {
                                    this.optimization_goal_types = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * OptimizationGoalSetting optimization_goal_types.
                                 * @member {Array.<google.ads.googleads.v11.enums.OptimizationGoalTypeEnum.OptimizationGoalType>} optimization_goal_types
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @instance
                                 */
                                OptimizationGoalSetting.prototype.optimization_goal_types = $util.emptyArray;
    
                                /**
                                 * Creates a new OptimizationGoalSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IOptimizationGoalSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting} OptimizationGoalSetting instance
                                 */
                                OptimizationGoalSetting.create = function create(properties) {
                                    return new OptimizationGoalSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified OptimizationGoalSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IOptimizationGoalSetting} m OptimizationGoalSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                OptimizationGoalSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.optimization_goal_types != null && m.optimization_goal_types.length) {
                                        w.uint32(10).fork();
                                        for (var i = 0; i < m.optimization_goal_types.length; ++i)
                                            w.int32(m.optimization_goal_types[i]);
                                        w.ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified OptimizationGoalSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IOptimizationGoalSetting} message OptimizationGoalSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                OptimizationGoalSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an OptimizationGoalSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting} OptimizationGoalSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                OptimizationGoalSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.optimization_goal_types && m.optimization_goal_types.length))
                                                m.optimization_goal_types = [];
                                            if ((t & 7) === 2) {
                                                var c2 = r.uint32() + r.pos;
                                                while (r.pos < c2)
                                                    m.optimization_goal_types.push(r.int32());
                                            } else
                                                m.optimization_goal_types.push(r.int32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an OptimizationGoalSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting} OptimizationGoalSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                OptimizationGoalSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an OptimizationGoalSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                OptimizationGoalSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.optimization_goal_types != null && m.hasOwnProperty("optimization_goal_types")) {
                                        if (!Array.isArray(m.optimization_goal_types))
                                            return "optimization_goal_types: array expected";
                                        for (var i = 0; i < m.optimization_goal_types.length; ++i) {
                                            switch (m.optimization_goal_types[i]) {
                                            default:
                                                return "optimization_goal_types: enum value[] expected";
                                            case 0:
                                            case 1:
                                            case 2:
                                            case 3:
                                            case 4:
                                                break;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an OptimizationGoalSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting} OptimizationGoalSetting
                                 */
                                OptimizationGoalSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting();
                                    if (d.optimization_goal_types) {
                                        if (!Array.isArray(d.optimization_goal_types))
                                            throw TypeError(".google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting.optimization_goal_types: array expected");
                                        m.optimization_goal_types = [];
                                        for (var i = 0; i < d.optimization_goal_types.length; ++i) {
                                            switch (d.optimization_goal_types[i]) {
                                            default:
                                            case "UNSPECIFIED":
                                            case 0:
                                                m.optimization_goal_types[i] = 0;
                                                break;
                                            case "UNKNOWN":
                                            case 1:
                                                m.optimization_goal_types[i] = 1;
                                                break;
                                            case "CALL_CLICKS":
                                            case 2:
                                                m.optimization_goal_types[i] = 2;
                                                break;
                                            case "DRIVING_DIRECTIONS":
                                            case 3:
                                                m.optimization_goal_types[i] = 3;
                                                break;
                                            case "APP_PRE_REGISTRATION":
                                            case 4:
                                                m.optimization_goal_types[i] = 4;
                                                break;
                                            }
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an OptimizationGoalSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting} m OptimizationGoalSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                OptimizationGoalSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.optimization_goal_types = [];
                                    }
                                    if (m.optimization_goal_types && m.optimization_goal_types.length) {
                                        d.optimization_goal_types = [];
                                        for (var j = 0; j < m.optimization_goal_types.length; ++j) {
                                            d.optimization_goal_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.OptimizationGoalTypeEnum.OptimizationGoalType[m.optimization_goal_types[j]] : m.optimization_goal_types[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this OptimizationGoalSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.OptimizationGoalSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                OptimizationGoalSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return OptimizationGoalSetting;
                            })();
    
                            Campaign.AudienceSetting = (function() {
    
                                /**
                                 * Properties of an AudienceSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface IAudienceSetting
                                 * @property {boolean|null} [use_audience_grouped] AudienceSetting use_audience_grouped
                                 */
    
                                /**
                                 * Constructs a new AudienceSetting.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents an AudienceSetting.
                                 * @implements IAudienceSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAudienceSetting=} [p] Properties to set
                                 */
                                function AudienceSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * AudienceSetting use_audience_grouped.
                                 * @member {boolean|null|undefined} use_audience_grouped
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @instance
                                 */
                                AudienceSetting.prototype.use_audience_grouped = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * AudienceSetting _use_audience_grouped.
                                 * @member {"use_audience_grouped"|undefined} _use_audience_grouped
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @instance
                                 */
                                Object.defineProperty(AudienceSetting.prototype, "_use_audience_grouped", {
                                    get: $util.oneOfGetter($oneOfFields = ["use_audience_grouped"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new AudienceSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAudienceSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AudienceSetting} AudienceSetting instance
                                 */
                                AudienceSetting.create = function create(properties) {
                                    return new AudienceSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified AudienceSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.AudienceSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAudienceSetting} m AudienceSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AudienceSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.use_audience_grouped != null && Object.hasOwnProperty.call(m, "use_audience_grouped"))
                                        w.uint32(8).bool(m.use_audience_grouped);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified AudienceSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.AudienceSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.IAudienceSetting} message AudienceSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AudienceSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an AudienceSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AudienceSetting} AudienceSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AudienceSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.use_audience_grouped = r.bool();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an AudienceSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AudienceSetting} AudienceSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AudienceSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an AudienceSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                AudienceSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.use_audience_grouped != null && m.hasOwnProperty("use_audience_grouped")) {
                                        p._use_audience_grouped = 1;
                                        if (typeof m.use_audience_grouped !== "boolean")
                                            return "use_audience_grouped: boolean expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an AudienceSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.AudienceSetting} AudienceSetting
                                 */
                                AudienceSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.AudienceSetting();
                                    if (d.use_audience_grouped != null) {
                                        m.use_audience_grouped = Boolean(d.use_audience_grouped);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an AudienceSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.AudienceSetting} m AudienceSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                AudienceSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.use_audience_grouped != null && m.hasOwnProperty("use_audience_grouped")) {
                                        d.use_audience_grouped = m.use_audience_grouped;
                                        if (o.oneofs)
                                            d._use_audience_grouped = "use_audience_grouped";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this AudienceSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.AudienceSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                AudienceSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return AudienceSetting;
                            })();
    
                            Campaign.LocalServicesCampaignSettings = (function() {
    
                                /**
                                 * Properties of a LocalServicesCampaignSettings.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface ILocalServicesCampaignSettings
                                 * @property {Array.<google.ads.googleads.v11.resources.Campaign.ICategoryBid>|null} [category_bids] LocalServicesCampaignSettings category_bids
                                 */
    
                                /**
                                 * Constructs a new LocalServicesCampaignSettings.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a LocalServicesCampaignSettings.
                                 * @implements ILocalServicesCampaignSettings
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalServicesCampaignSettings=} [p] Properties to set
                                 */
                                function LocalServicesCampaignSettings(p) {
                                    this.category_bids = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * LocalServicesCampaignSettings category_bids.
                                 * @member {Array.<google.ads.googleads.v11.resources.Campaign.ICategoryBid>} category_bids
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @instance
                                 */
                                LocalServicesCampaignSettings.prototype.category_bids = $util.emptyArray;
    
                                /**
                                 * Creates a new LocalServicesCampaignSettings instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalServicesCampaignSettings=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings} LocalServicesCampaignSettings instance
                                 */
                                LocalServicesCampaignSettings.create = function create(properties) {
                                    return new LocalServicesCampaignSettings(properties);
                                };
    
                                /**
                                 * Encodes the specified LocalServicesCampaignSettings message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalServicesCampaignSettings} m LocalServicesCampaignSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocalServicesCampaignSettings.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.category_bids != null && m.category_bids.length) {
                                        for (var i = 0; i < m.category_bids.length; ++i)
                                            $root.google.ads.googleads.v11.resources.Campaign.CategoryBid.encode(m.category_bids[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified LocalServicesCampaignSettings message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ILocalServicesCampaignSettings} message LocalServicesCampaignSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocalServicesCampaignSettings.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a LocalServicesCampaignSettings message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings} LocalServicesCampaignSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocalServicesCampaignSettings.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.category_bids && m.category_bids.length))
                                                m.category_bids = [];
                                            m.category_bids.push($root.google.ads.googleads.v11.resources.Campaign.CategoryBid.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a LocalServicesCampaignSettings message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings} LocalServicesCampaignSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocalServicesCampaignSettings.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a LocalServicesCampaignSettings message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                LocalServicesCampaignSettings.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.category_bids != null && m.hasOwnProperty("category_bids")) {
                                        if (!Array.isArray(m.category_bids))
                                            return "category_bids: array expected";
                                        for (var i = 0; i < m.category_bids.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.resources.Campaign.CategoryBid.verify(m.category_bids[i]);
                                                if (e)
                                                    return "category_bids." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a LocalServicesCampaignSettings message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings} LocalServicesCampaignSettings
                                 */
                                LocalServicesCampaignSettings.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings();
                                    if (d.category_bids) {
                                        if (!Array.isArray(d.category_bids))
                                            throw TypeError(".google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.category_bids: array expected");
                                        m.category_bids = [];
                                        for (var i = 0; i < d.category_bids.length; ++i) {
                                            if (typeof d.category_bids[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings.category_bids: object expected");
                                            m.category_bids[i] = $root.google.ads.googleads.v11.resources.Campaign.CategoryBid.fromObject(d.category_bids[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a LocalServicesCampaignSettings message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings} m LocalServicesCampaignSettings
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                LocalServicesCampaignSettings.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.category_bids = [];
                                    }
                                    if (m.category_bids && m.category_bids.length) {
                                        d.category_bids = [];
                                        for (var j = 0; j < m.category_bids.length; ++j) {
                                            d.category_bids[j] = $root.google.ads.googleads.v11.resources.Campaign.CategoryBid.toObject(m.category_bids[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this LocalServicesCampaignSettings to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.LocalServicesCampaignSettings
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                LocalServicesCampaignSettings.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return LocalServicesCampaignSettings;
                            })();
    
                            Campaign.CategoryBid = (function() {
    
                                /**
                                 * Properties of a CategoryBid.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @interface ICategoryBid
                                 * @property {string|null} [category_id] CategoryBid category_id
                                 * @property {number|null} [manual_cpa_bid_micros] CategoryBid manual_cpa_bid_micros
                                 */
    
                                /**
                                 * Constructs a new CategoryBid.
                                 * @memberof google.ads.googleads.v11.resources.Campaign
                                 * @classdesc Represents a CategoryBid.
                                 * @implements ICategoryBid
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Campaign.ICategoryBid=} [p] Properties to set
                                 */
                                function CategoryBid(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CategoryBid category_id.
                                 * @member {string|null|undefined} category_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @instance
                                 */
                                CategoryBid.prototype.category_id = null;
    
                                /**
                                 * CategoryBid manual_cpa_bid_micros.
                                 * @member {number|null|undefined} manual_cpa_bid_micros
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @instance
                                 */
                                CategoryBid.prototype.manual_cpa_bid_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * CategoryBid _category_id.
                                 * @member {"category_id"|undefined} _category_id
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @instance
                                 */
                                Object.defineProperty(CategoryBid.prototype, "_category_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["category_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * CategoryBid _manual_cpa_bid_micros.
                                 * @member {"manual_cpa_bid_micros"|undefined} _manual_cpa_bid_micros
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @instance
                                 */
                                Object.defineProperty(CategoryBid.prototype, "_manual_cpa_bid_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["manual_cpa_bid_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new CategoryBid instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ICategoryBid=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Campaign.CategoryBid} CategoryBid instance
                                 */
                                CategoryBid.create = function create(properties) {
                                    return new CategoryBid(properties);
                                };
    
                                /**
                                 * Encodes the specified CategoryBid message. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.CategoryBid.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ICategoryBid} m CategoryBid message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CategoryBid.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.category_id != null && Object.hasOwnProperty.call(m, "category_id"))
                                        w.uint32(10).string(m.category_id);
                                    if (m.manual_cpa_bid_micros != null && Object.hasOwnProperty.call(m, "manual_cpa_bid_micros"))
                                        w.uint32(16).int64(m.manual_cpa_bid_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CategoryBid message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Campaign.CategoryBid.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.ICategoryBid} message CategoryBid message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CategoryBid.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CategoryBid message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Campaign.CategoryBid} CategoryBid
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CategoryBid.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Campaign.CategoryBid();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.category_id = r.string();
                                            break;
                                        case 2:
                                            m.manual_cpa_bid_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CategoryBid message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Campaign.CategoryBid} CategoryBid
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CategoryBid.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CategoryBid message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CategoryBid.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.category_id != null && m.hasOwnProperty("category_id")) {
                                        p._category_id = 1;
                                        if (!$util.isString(m.category_id))
                                            return "category_id: string expected";
                                    }
                                    if (m.manual_cpa_bid_micros != null && m.hasOwnProperty("manual_cpa_bid_micros")) {
                                        p._manual_cpa_bid_micros = 1;
                                        if (!$util.isInteger(m.manual_cpa_bid_micros) && !(m.manual_cpa_bid_micros && $util.isInteger(m.manual_cpa_bid_micros.low) && $util.isInteger(m.manual_cpa_bid_micros.high)))
                                            return "manual_cpa_bid_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CategoryBid message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Campaign.CategoryBid} CategoryBid
                                 */
                                CategoryBid.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Campaign.CategoryBid)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Campaign.CategoryBid();
                                    if (d.category_id != null) {
                                        m.category_id = String(d.category_id);
                                    }
                                    if (d.manual_cpa_bid_micros != null) {
                                        if ($util.Long)
                                            (m.manual_cpa_bid_micros = $util.Long.fromValue(d.manual_cpa_bid_micros)).unsigned = false;
                                        else if (typeof d.manual_cpa_bid_micros === "string")
                                            m.manual_cpa_bid_micros = parseInt(d.manual_cpa_bid_micros, 10);
                                        else if (typeof d.manual_cpa_bid_micros === "number")
                                            m.manual_cpa_bid_micros = d.manual_cpa_bid_micros;
                                        else if (typeof d.manual_cpa_bid_micros === "object")
                                            m.manual_cpa_bid_micros = new $util.LongBits(d.manual_cpa_bid_micros.low >>> 0, d.manual_cpa_bid_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CategoryBid message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Campaign.CategoryBid} m CategoryBid
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CategoryBid.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.category_id != null && m.hasOwnProperty("category_id")) {
                                        d.category_id = m.category_id;
                                        if (o.oneofs)
                                            d._category_id = "category_id";
                                    }
                                    if (m.manual_cpa_bid_micros != null && m.hasOwnProperty("manual_cpa_bid_micros")) {
                                        if (typeof m.manual_cpa_bid_micros === "number")
                                            d.manual_cpa_bid_micros = o.longs === String ? String(m.manual_cpa_bid_micros) : m.manual_cpa_bid_micros;
                                        else
                                            d.manual_cpa_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.manual_cpa_bid_micros) : o.longs === Number ? new $util.LongBits(m.manual_cpa_bid_micros.low >>> 0, m.manual_cpa_bid_micros.high >>> 0).toNumber() : m.manual_cpa_bid_micros;
                                        if (o.oneofs)
                                            d._manual_cpa_bid_micros = "manual_cpa_bid_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CategoryBid to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Campaign.CategoryBid
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CategoryBid.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CategoryBid;
                            })();
    
                            return Campaign;
                        })();
    
                        resources.CampaignAsset = (function() {
    
                            /**
                             * Properties of a CampaignAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignAsset
                             * @property {string|null} [resource_name] CampaignAsset resource_name
                             * @property {string|null} [campaign] CampaignAsset campaign
                             * @property {string|null} [asset] CampaignAsset asset
                             * @property {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType|null} [field_type] CampaignAsset field_type
                             * @property {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource|null} [source] CampaignAsset source
                             * @property {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus|null} [status] CampaignAsset status
                             */
    
                            /**
                             * Constructs a new CampaignAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignAsset.
                             * @implements ICampaignAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignAsset=} [p] Properties to set
                             */
                            function CampaignAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignAsset resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            CampaignAsset.prototype.resource_name = "";
    
                            /**
                             * CampaignAsset campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            CampaignAsset.prototype.campaign = null;
    
                            /**
                             * CampaignAsset asset.
                             * @member {string|null|undefined} asset
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            CampaignAsset.prototype.asset = null;
    
                            /**
                             * CampaignAsset field_type.
                             * @member {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType} field_type
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            CampaignAsset.prototype.field_type = 0;
    
                            /**
                             * CampaignAsset source.
                             * @member {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource} source
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            CampaignAsset.prototype.source = 0;
    
                            /**
                             * CampaignAsset status.
                             * @member {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            CampaignAsset.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignAsset _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            Object.defineProperty(CampaignAsset.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignAsset _asset.
                             * @member {"asset"|undefined} _asset
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             */
                            Object.defineProperty(CampaignAsset.prototype, "_asset", {
                                get: $util.oneOfGetter($oneOfFields = ["asset"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignAsset} CampaignAsset instance
                             */
                            CampaignAsset.create = function create(properties) {
                                return new CampaignAsset(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignAsset message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAsset} m CampaignAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.field_type != null && Object.hasOwnProperty.call(m, "field_type"))
                                    w.uint32(32).int32(m.field_type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(50).string(m.campaign);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(58).string(m.asset);
                                if (m.source != null && Object.hasOwnProperty.call(m, "source"))
                                    w.uint32(64).int32(m.source);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAsset} message CampaignAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignAsset} CampaignAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.campaign = r.string();
                                        break;
                                    case 7:
                                        m.asset = r.string();
                                        break;
                                    case 4:
                                        m.field_type = r.int32();
                                        break;
                                    case 8:
                                        m.source = r.int32();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignAsset} CampaignAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    p._asset = 1;
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    switch (m.field_type) {
                                    default:
                                        return "field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 24:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    switch (m.source) {
                                    default:
                                        return "source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignAsset} CampaignAsset
                             */
                            CampaignAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignAsset();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                switch (d.field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_type = 1;
                                    break;
                                case "HEADLINE":
                                case 2:
                                    m.field_type = 2;
                                    break;
                                case "DESCRIPTION":
                                case 3:
                                    m.field_type = 3;
                                    break;
                                case "MANDATORY_AD_TEXT":
                                case 4:
                                    m.field_type = 4;
                                    break;
                                case "MARKETING_IMAGE":
                                case 5:
                                    m.field_type = 5;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 6:
                                    m.field_type = 6;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 7:
                                    m.field_type = 7;
                                    break;
                                case "BOOK_ON_GOOGLE":
                                case 8:
                                    m.field_type = 8;
                                    break;
                                case "LEAD_FORM":
                                case 9:
                                    m.field_type = 9;
                                    break;
                                case "PROMOTION":
                                case 10:
                                    m.field_type = 10;
                                    break;
                                case "CALLOUT":
                                case 11:
                                    m.field_type = 11;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 12:
                                    m.field_type = 12;
                                    break;
                                case "SITELINK":
                                case 13:
                                    m.field_type = 13;
                                    break;
                                case "MOBILE_APP":
                                case 14:
                                    m.field_type = 14;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.field_type = 15;
                                    break;
                                case "CALL":
                                case 16:
                                    m.field_type = 16;
                                    break;
                                case "PRICE":
                                case 24:
                                    m.field_type = 24;
                                    break;
                                case "LONG_HEADLINE":
                                case 17:
                                    m.field_type = 17;
                                    break;
                                case "BUSINESS_NAME":
                                case 18:
                                    m.field_type = 18;
                                    break;
                                case "SQUARE_MARKETING_IMAGE":
                                case 19:
                                    m.field_type = 19;
                                    break;
                                case "PORTRAIT_MARKETING_IMAGE":
                                case 20:
                                    m.field_type = 20;
                                    break;
                                case "LOGO":
                                case 21:
                                    m.field_type = 21;
                                    break;
                                case "LANDSCAPE_LOGO":
                                case 22:
                                    m.field_type = 22;
                                    break;
                                case "VIDEO":
                                case 23:
                                    m.field_type = 23;
                                    break;
                                case "CALL_TO_ACTION_SELECTION":
                                case 25:
                                    m.field_type = 25;
                                    break;
                                }
                                switch (d.source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.source = 1;
                                    break;
                                case "ADVERTISER":
                                case 2:
                                    m.source = 2;
                                    break;
                                case "AUTOMATICALLY_CREATED":
                                case 3:
                                    m.source = 3;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PAUSED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignAsset} m CampaignAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.source = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    d.field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.field_type] : m.field_type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus[m.status] : m.status;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                    if (o.oneofs)
                                        d._asset = "asset";
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    d.source = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource[m.source] : m.source;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignAsset;
                        })();
    
                        resources.CampaignAssetSet = (function() {
    
                            /**
                             * Properties of a CampaignAssetSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignAssetSet
                             * @property {string|null} [resource_name] CampaignAssetSet resource_name
                             * @property {string|null} [campaign] CampaignAssetSet campaign
                             * @property {string|null} [asset_set] CampaignAssetSet asset_set
                             * @property {google.ads.googleads.v11.enums.AssetSetLinkStatusEnum.AssetSetLinkStatus|null} [status] CampaignAssetSet status
                             */
    
                            /**
                             * Constructs a new CampaignAssetSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignAssetSet.
                             * @implements ICampaignAssetSet
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignAssetSet=} [p] Properties to set
                             */
                            function CampaignAssetSet(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignAssetSet resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @instance
                             */
                            CampaignAssetSet.prototype.resource_name = "";
    
                            /**
                             * CampaignAssetSet campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @instance
                             */
                            CampaignAssetSet.prototype.campaign = "";
    
                            /**
                             * CampaignAssetSet asset_set.
                             * @member {string} asset_set
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @instance
                             */
                            CampaignAssetSet.prototype.asset_set = "";
    
                            /**
                             * CampaignAssetSet status.
                             * @member {google.ads.googleads.v11.enums.AssetSetLinkStatusEnum.AssetSetLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @instance
                             */
                            CampaignAssetSet.prototype.status = 0;
    
                            /**
                             * Creates a new CampaignAssetSet instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAssetSet=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignAssetSet} CampaignAssetSet instance
                             */
                            CampaignAssetSet.create = function create(properties) {
                                return new CampaignAssetSet(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignAssetSet message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignAssetSet.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAssetSet} m CampaignAssetSet message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAssetSet.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                if (m.asset_set != null && Object.hasOwnProperty.call(m, "asset_set"))
                                    w.uint32(26).string(m.asset_set);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignAssetSet message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignAssetSet.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAssetSet} message CampaignAssetSet message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAssetSet.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignAssetSet message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignAssetSet} CampaignAssetSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAssetSet.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignAssetSet();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    case 3:
                                        m.asset_set = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignAssetSet message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignAssetSet} CampaignAssetSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAssetSet.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignAssetSet message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignAssetSet.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    if (!$util.isString(m.asset_set))
                                        return "asset_set: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignAssetSet message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignAssetSet} CampaignAssetSet
                             */
                            CampaignAssetSet.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignAssetSet)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignAssetSet();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.asset_set != null) {
                                    m.asset_set = String(d.asset_set);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignAssetSet message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignAssetSet} m CampaignAssetSet
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignAssetSet.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign = "";
                                    d.asset_set = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    d.asset_set = m.asset_set;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSetLinkStatusEnum.AssetSetLinkStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignAssetSet to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignAssetSet
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignAssetSet.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignAssetSet;
                        })();
    
                        resources.CampaignAudienceView = (function() {
    
                            /**
                             * Properties of a CampaignAudienceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignAudienceView
                             * @property {string|null} [resource_name] CampaignAudienceView resource_name
                             */
    
                            /**
                             * Constructs a new CampaignAudienceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignAudienceView.
                             * @implements ICampaignAudienceView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignAudienceView=} [p] Properties to set
                             */
                            function CampaignAudienceView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignAudienceView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @instance
                             */
                            CampaignAudienceView.prototype.resource_name = "";
    
                            /**
                             * Creates a new CampaignAudienceView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAudienceView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignAudienceView} CampaignAudienceView instance
                             */
                            CampaignAudienceView.create = function create(properties) {
                                return new CampaignAudienceView(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignAudienceView message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignAudienceView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAudienceView} m CampaignAudienceView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAudienceView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignAudienceView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignAudienceView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignAudienceView} message CampaignAudienceView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAudienceView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignAudienceView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignAudienceView} CampaignAudienceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAudienceView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignAudienceView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignAudienceView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignAudienceView} CampaignAudienceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAudienceView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignAudienceView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignAudienceView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignAudienceView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignAudienceView} CampaignAudienceView
                             */
                            CampaignAudienceView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignAudienceView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignAudienceView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignAudienceView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignAudienceView} m CampaignAudienceView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignAudienceView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignAudienceView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignAudienceView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignAudienceView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignAudienceView;
                        })();
    
                        resources.CampaignBidModifier = (function() {
    
                            /**
                             * Properties of a CampaignBidModifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignBidModifier
                             * @property {string|null} [resource_name] CampaignBidModifier resource_name
                             * @property {string|null} [campaign] CampaignBidModifier campaign
                             * @property {number|null} [criterion_id] CampaignBidModifier criterion_id
                             * @property {number|null} [bid_modifier] CampaignBidModifier bid_modifier
                             * @property {google.ads.googleads.v11.common.IInteractionTypeInfo|null} [interaction_type] CampaignBidModifier interaction_type
                             */
    
                            /**
                             * Constructs a new CampaignBidModifier.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignBidModifier.
                             * @implements ICampaignBidModifier
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignBidModifier=} [p] Properties to set
                             */
                            function CampaignBidModifier(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignBidModifier resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            CampaignBidModifier.prototype.resource_name = "";
    
                            /**
                             * CampaignBidModifier campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            CampaignBidModifier.prototype.campaign = null;
    
                            /**
                             * CampaignBidModifier criterion_id.
                             * @member {number|null|undefined} criterion_id
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            CampaignBidModifier.prototype.criterion_id = null;
    
                            /**
                             * CampaignBidModifier bid_modifier.
                             * @member {number|null|undefined} bid_modifier
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            CampaignBidModifier.prototype.bid_modifier = null;
    
                            /**
                             * CampaignBidModifier interaction_type.
                             * @member {google.ads.googleads.v11.common.IInteractionTypeInfo|null|undefined} interaction_type
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            CampaignBidModifier.prototype.interaction_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignBidModifier _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            Object.defineProperty(CampaignBidModifier.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBidModifier _criterion_id.
                             * @member {"criterion_id"|undefined} _criterion_id
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            Object.defineProperty(CampaignBidModifier.prototype, "_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBidModifier _bid_modifier.
                             * @member {"bid_modifier"|undefined} _bid_modifier
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            Object.defineProperty(CampaignBidModifier.prototype, "_bid_modifier", {
                                get: $util.oneOfGetter($oneOfFields = ["bid_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBidModifier criterion.
                             * @member {"interaction_type"|undefined} criterion
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             */
                            Object.defineProperty(CampaignBidModifier.prototype, "criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["interaction_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignBidModifier instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignBidModifier=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignBidModifier} CampaignBidModifier instance
                             */
                            CampaignBidModifier.create = function create(properties) {
                                return new CampaignBidModifier(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignBidModifier message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignBidModifier.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignBidModifier} m CampaignBidModifier message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBidModifier.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.interaction_type != null && Object.hasOwnProperty.call(m, "interaction_type"))
                                    $root.google.ads.googleads.v11.common.InteractionTypeInfo.encode(m.interaction_type, w.uint32(42).fork()).ldelim();
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(50).string(m.campaign);
                                if (m.criterion_id != null && Object.hasOwnProperty.call(m, "criterion_id"))
                                    w.uint32(56).int64(m.criterion_id);
                                if (m.bid_modifier != null && Object.hasOwnProperty.call(m, "bid_modifier"))
                                    w.uint32(65).double(m.bid_modifier);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignBidModifier message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignBidModifier.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignBidModifier} message CampaignBidModifier message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBidModifier.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignBidModifier message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignBidModifier} CampaignBidModifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBidModifier.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignBidModifier();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.campaign = r.string();
                                        break;
                                    case 7:
                                        m.criterion_id = r.int64();
                                        break;
                                    case 8:
                                        m.bid_modifier = r.double();
                                        break;
                                    case 5:
                                        m.interaction_type = $root.google.ads.googleads.v11.common.InteractionTypeInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignBidModifier message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignBidModifier} CampaignBidModifier
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBidModifier.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignBidModifier message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignBidModifier.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    p._criterion_id = 1;
                                    if (!$util.isInteger(m.criterion_id) && !(m.criterion_id && $util.isInteger(m.criterion_id.low) && $util.isInteger(m.criterion_id.high)))
                                        return "criterion_id: integer|Long expected";
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    p._bid_modifier = 1;
                                    if (typeof m.bid_modifier !== "number")
                                        return "bid_modifier: number expected";
                                }
                                if (m.interaction_type != null && m.hasOwnProperty("interaction_type")) {
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.InteractionTypeInfo.verify(m.interaction_type);
                                        if (e)
                                            return "interaction_type." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignBidModifier message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignBidModifier} CampaignBidModifier
                             */
                            CampaignBidModifier.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignBidModifier)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignBidModifier();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.criterion_id != null) {
                                    if ($util.Long)
                                        (m.criterion_id = $util.Long.fromValue(d.criterion_id)).unsigned = false;
                                    else if (typeof d.criterion_id === "string")
                                        m.criterion_id = parseInt(d.criterion_id, 10);
                                    else if (typeof d.criterion_id === "number")
                                        m.criterion_id = d.criterion_id;
                                    else if (typeof d.criterion_id === "object")
                                        m.criterion_id = new $util.LongBits(d.criterion_id.low >>> 0, d.criterion_id.high >>> 0).toNumber();
                                }
                                if (d.bid_modifier != null) {
                                    m.bid_modifier = Number(d.bid_modifier);
                                }
                                if (d.interaction_type != null) {
                                    if (typeof d.interaction_type !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignBidModifier.interaction_type: object expected");
                                    m.interaction_type = $root.google.ads.googleads.v11.common.InteractionTypeInfo.fromObject(d.interaction_type);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignBidModifier message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignBidModifier} m CampaignBidModifier
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignBidModifier.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.interaction_type != null && m.hasOwnProperty("interaction_type")) {
                                    d.interaction_type = $root.google.ads.googleads.v11.common.InteractionTypeInfo.toObject(m.interaction_type, o);
                                    if (o.oneofs)
                                        d.criterion = "interaction_type";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    if (typeof m.criterion_id === "number")
                                        d.criterion_id = o.longs === String ? String(m.criterion_id) : m.criterion_id;
                                    else
                                        d.criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.criterion_id) : o.longs === Number ? new $util.LongBits(m.criterion_id.low >>> 0, m.criterion_id.high >>> 0).toNumber() : m.criterion_id;
                                    if (o.oneofs)
                                        d._criterion_id = "criterion_id";
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    d.bid_modifier = o.json && !isFinite(m.bid_modifier) ? String(m.bid_modifier) : m.bid_modifier;
                                    if (o.oneofs)
                                        d._bid_modifier = "bid_modifier";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignBidModifier to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignBidModifier
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignBidModifier.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignBidModifier;
                        })();
    
                        resources.CampaignBudget = (function() {
    
                            /**
                             * Properties of a CampaignBudget.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignBudget
                             * @property {string|null} [resource_name] CampaignBudget resource_name
                             * @property {number|null} [id] CampaignBudget id
                             * @property {string|null} [name] CampaignBudget name
                             * @property {number|null} [amount_micros] CampaignBudget amount_micros
                             * @property {number|null} [total_amount_micros] CampaignBudget total_amount_micros
                             * @property {google.ads.googleads.v11.enums.BudgetStatusEnum.BudgetStatus|null} [status] CampaignBudget status
                             * @property {google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum.BudgetDeliveryMethod|null} [delivery_method] CampaignBudget delivery_method
                             * @property {boolean|null} [explicitly_shared] CampaignBudget explicitly_shared
                             * @property {number|null} [reference_count] CampaignBudget reference_count
                             * @property {boolean|null} [has_recommended_budget] CampaignBudget has_recommended_budget
                             * @property {number|null} [recommended_budget_amount_micros] CampaignBudget recommended_budget_amount_micros
                             * @property {google.ads.googleads.v11.enums.BudgetPeriodEnum.BudgetPeriod|null} [period] CampaignBudget period
                             * @property {number|null} [recommended_budget_estimated_change_weekly_clicks] CampaignBudget recommended_budget_estimated_change_weekly_clicks
                             * @property {number|null} [recommended_budget_estimated_change_weekly_cost_micros] CampaignBudget recommended_budget_estimated_change_weekly_cost_micros
                             * @property {number|null} [recommended_budget_estimated_change_weekly_interactions] CampaignBudget recommended_budget_estimated_change_weekly_interactions
                             * @property {number|null} [recommended_budget_estimated_change_weekly_views] CampaignBudget recommended_budget_estimated_change_weekly_views
                             * @property {google.ads.googleads.v11.enums.BudgetTypeEnum.BudgetType|null} [type] CampaignBudget type
                             */
    
                            /**
                             * Constructs a new CampaignBudget.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignBudget.
                             * @implements ICampaignBudget
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignBudget=} [p] Properties to set
                             */
                            function CampaignBudget(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignBudget resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.resource_name = "";
    
                            /**
                             * CampaignBudget id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.id = null;
    
                            /**
                             * CampaignBudget name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.name = null;
    
                            /**
                             * CampaignBudget amount_micros.
                             * @member {number|null|undefined} amount_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.amount_micros = null;
    
                            /**
                             * CampaignBudget total_amount_micros.
                             * @member {number|null|undefined} total_amount_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.total_amount_micros = null;
    
                            /**
                             * CampaignBudget status.
                             * @member {google.ads.googleads.v11.enums.BudgetStatusEnum.BudgetStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.status = 0;
    
                            /**
                             * CampaignBudget delivery_method.
                             * @member {google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum.BudgetDeliveryMethod} delivery_method
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.delivery_method = 0;
    
                            /**
                             * CampaignBudget explicitly_shared.
                             * @member {boolean|null|undefined} explicitly_shared
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.explicitly_shared = null;
    
                            /**
                             * CampaignBudget reference_count.
                             * @member {number|null|undefined} reference_count
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.reference_count = null;
    
                            /**
                             * CampaignBudget has_recommended_budget.
                             * @member {boolean|null|undefined} has_recommended_budget
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.has_recommended_budget = null;
    
                            /**
                             * CampaignBudget recommended_budget_amount_micros.
                             * @member {number|null|undefined} recommended_budget_amount_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.recommended_budget_amount_micros = null;
    
                            /**
                             * CampaignBudget period.
                             * @member {google.ads.googleads.v11.enums.BudgetPeriodEnum.BudgetPeriod} period
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.period = 0;
    
                            /**
                             * CampaignBudget recommended_budget_estimated_change_weekly_clicks.
                             * @member {number|null|undefined} recommended_budget_estimated_change_weekly_clicks
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.recommended_budget_estimated_change_weekly_clicks = null;
    
                            /**
                             * CampaignBudget recommended_budget_estimated_change_weekly_cost_micros.
                             * @member {number|null|undefined} recommended_budget_estimated_change_weekly_cost_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.recommended_budget_estimated_change_weekly_cost_micros = null;
    
                            /**
                             * CampaignBudget recommended_budget_estimated_change_weekly_interactions.
                             * @member {number|null|undefined} recommended_budget_estimated_change_weekly_interactions
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.recommended_budget_estimated_change_weekly_interactions = null;
    
                            /**
                             * CampaignBudget recommended_budget_estimated_change_weekly_views.
                             * @member {number|null|undefined} recommended_budget_estimated_change_weekly_views
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.recommended_budget_estimated_change_weekly_views = null;
    
                            /**
                             * CampaignBudget type.
                             * @member {google.ads.googleads.v11.enums.BudgetTypeEnum.BudgetType} type
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            CampaignBudget.prototype.type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignBudget _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _amount_micros.
                             * @member {"amount_micros"|undefined} _amount_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _total_amount_micros.
                             * @member {"total_amount_micros"|undefined} _total_amount_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_total_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["total_amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _explicitly_shared.
                             * @member {"explicitly_shared"|undefined} _explicitly_shared
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_explicitly_shared", {
                                get: $util.oneOfGetter($oneOfFields = ["explicitly_shared"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _reference_count.
                             * @member {"reference_count"|undefined} _reference_count
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_reference_count", {
                                get: $util.oneOfGetter($oneOfFields = ["reference_count"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _has_recommended_budget.
                             * @member {"has_recommended_budget"|undefined} _has_recommended_budget
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_has_recommended_budget", {
                                get: $util.oneOfGetter($oneOfFields = ["has_recommended_budget"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _recommended_budget_amount_micros.
                             * @member {"recommended_budget_amount_micros"|undefined} _recommended_budget_amount_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_recommended_budget_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["recommended_budget_amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _recommended_budget_estimated_change_weekly_clicks.
                             * @member {"recommended_budget_estimated_change_weekly_clicks"|undefined} _recommended_budget_estimated_change_weekly_clicks
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_recommended_budget_estimated_change_weekly_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["recommended_budget_estimated_change_weekly_clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _recommended_budget_estimated_change_weekly_cost_micros.
                             * @member {"recommended_budget_estimated_change_weekly_cost_micros"|undefined} _recommended_budget_estimated_change_weekly_cost_micros
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_recommended_budget_estimated_change_weekly_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["recommended_budget_estimated_change_weekly_cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _recommended_budget_estimated_change_weekly_interactions.
                             * @member {"recommended_budget_estimated_change_weekly_interactions"|undefined} _recommended_budget_estimated_change_weekly_interactions
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_recommended_budget_estimated_change_weekly_interactions", {
                                get: $util.oneOfGetter($oneOfFields = ["recommended_budget_estimated_change_weekly_interactions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignBudget _recommended_budget_estimated_change_weekly_views.
                             * @member {"recommended_budget_estimated_change_weekly_views"|undefined} _recommended_budget_estimated_change_weekly_views
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             */
                            Object.defineProperty(CampaignBudget.prototype, "_recommended_budget_estimated_change_weekly_views", {
                                get: $util.oneOfGetter($oneOfFields = ["recommended_budget_estimated_change_weekly_views"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignBudget instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignBudget=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignBudget} CampaignBudget instance
                             */
                            CampaignBudget.create = function create(properties) {
                                return new CampaignBudget(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignBudget message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignBudget.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignBudget} m CampaignBudget message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudget.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(48).int32(m.status);
                                if (m.delivery_method != null && Object.hasOwnProperty.call(m, "delivery_method"))
                                    w.uint32(56).int32(m.delivery_method);
                                if (m.period != null && Object.hasOwnProperty.call(m, "period"))
                                    w.uint32(104).int32(m.period);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(144).int32(m.type);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(152).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(162).string(m.name);
                                if (m.amount_micros != null && Object.hasOwnProperty.call(m, "amount_micros"))
                                    w.uint32(168).int64(m.amount_micros);
                                if (m.total_amount_micros != null && Object.hasOwnProperty.call(m, "total_amount_micros"))
                                    w.uint32(176).int64(m.total_amount_micros);
                                if (m.explicitly_shared != null && Object.hasOwnProperty.call(m, "explicitly_shared"))
                                    w.uint32(184).bool(m.explicitly_shared);
                                if (m.reference_count != null && Object.hasOwnProperty.call(m, "reference_count"))
                                    w.uint32(192).int64(m.reference_count);
                                if (m.has_recommended_budget != null && Object.hasOwnProperty.call(m, "has_recommended_budget"))
                                    w.uint32(200).bool(m.has_recommended_budget);
                                if (m.recommended_budget_amount_micros != null && Object.hasOwnProperty.call(m, "recommended_budget_amount_micros"))
                                    w.uint32(208).int64(m.recommended_budget_amount_micros);
                                if (m.recommended_budget_estimated_change_weekly_clicks != null && Object.hasOwnProperty.call(m, "recommended_budget_estimated_change_weekly_clicks"))
                                    w.uint32(216).int64(m.recommended_budget_estimated_change_weekly_clicks);
                                if (m.recommended_budget_estimated_change_weekly_cost_micros != null && Object.hasOwnProperty.call(m, "recommended_budget_estimated_change_weekly_cost_micros"))
                                    w.uint32(224).int64(m.recommended_budget_estimated_change_weekly_cost_micros);
                                if (m.recommended_budget_estimated_change_weekly_interactions != null && Object.hasOwnProperty.call(m, "recommended_budget_estimated_change_weekly_interactions"))
                                    w.uint32(232).int64(m.recommended_budget_estimated_change_weekly_interactions);
                                if (m.recommended_budget_estimated_change_weekly_views != null && Object.hasOwnProperty.call(m, "recommended_budget_estimated_change_weekly_views"))
                                    w.uint32(240).int64(m.recommended_budget_estimated_change_weekly_views);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignBudget message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignBudget.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignBudget} message CampaignBudget message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudget.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignBudget message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignBudget} CampaignBudget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudget.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignBudget();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 19:
                                        m.id = r.int64();
                                        break;
                                    case 20:
                                        m.name = r.string();
                                        break;
                                    case 21:
                                        m.amount_micros = r.int64();
                                        break;
                                    case 22:
                                        m.total_amount_micros = r.int64();
                                        break;
                                    case 6:
                                        m.status = r.int32();
                                        break;
                                    case 7:
                                        m.delivery_method = r.int32();
                                        break;
                                    case 23:
                                        m.explicitly_shared = r.bool();
                                        break;
                                    case 24:
                                        m.reference_count = r.int64();
                                        break;
                                    case 25:
                                        m.has_recommended_budget = r.bool();
                                        break;
                                    case 26:
                                        m.recommended_budget_amount_micros = r.int64();
                                        break;
                                    case 13:
                                        m.period = r.int32();
                                        break;
                                    case 27:
                                        m.recommended_budget_estimated_change_weekly_clicks = r.int64();
                                        break;
                                    case 28:
                                        m.recommended_budget_estimated_change_weekly_cost_micros = r.int64();
                                        break;
                                    case 29:
                                        m.recommended_budget_estimated_change_weekly_interactions = r.int64();
                                        break;
                                    case 30:
                                        m.recommended_budget_estimated_change_weekly_views = r.int64();
                                        break;
                                    case 18:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignBudget message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignBudget} CampaignBudget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudget.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignBudget message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignBudget.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.amount_micros != null && m.hasOwnProperty("amount_micros")) {
                                    p._amount_micros = 1;
                                    if (!$util.isInteger(m.amount_micros) && !(m.amount_micros && $util.isInteger(m.amount_micros.low) && $util.isInteger(m.amount_micros.high)))
                                        return "amount_micros: integer|Long expected";
                                }
                                if (m.total_amount_micros != null && m.hasOwnProperty("total_amount_micros")) {
                                    p._total_amount_micros = 1;
                                    if (!$util.isInteger(m.total_amount_micros) && !(m.total_amount_micros && $util.isInteger(m.total_amount_micros.low) && $util.isInteger(m.total_amount_micros.high)))
                                        return "total_amount_micros: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.delivery_method != null && m.hasOwnProperty("delivery_method")) {
                                    switch (m.delivery_method) {
                                    default:
                                        return "delivery_method: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.explicitly_shared != null && m.hasOwnProperty("explicitly_shared")) {
                                    p._explicitly_shared = 1;
                                    if (typeof m.explicitly_shared !== "boolean")
                                        return "explicitly_shared: boolean expected";
                                }
                                if (m.reference_count != null && m.hasOwnProperty("reference_count")) {
                                    p._reference_count = 1;
                                    if (!$util.isInteger(m.reference_count) && !(m.reference_count && $util.isInteger(m.reference_count.low) && $util.isInteger(m.reference_count.high)))
                                        return "reference_count: integer|Long expected";
                                }
                                if (m.has_recommended_budget != null && m.hasOwnProperty("has_recommended_budget")) {
                                    p._has_recommended_budget = 1;
                                    if (typeof m.has_recommended_budget !== "boolean")
                                        return "has_recommended_budget: boolean expected";
                                }
                                if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                    p._recommended_budget_amount_micros = 1;
                                    if (!$util.isInteger(m.recommended_budget_amount_micros) && !(m.recommended_budget_amount_micros && $util.isInteger(m.recommended_budget_amount_micros.low) && $util.isInteger(m.recommended_budget_amount_micros.high)))
                                        return "recommended_budget_amount_micros: integer|Long expected";
                                }
                                if (m.period != null && m.hasOwnProperty("period")) {
                                    switch (m.period) {
                                    default:
                                        return "period: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.recommended_budget_estimated_change_weekly_clicks != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_clicks")) {
                                    p._recommended_budget_estimated_change_weekly_clicks = 1;
                                    if (!$util.isInteger(m.recommended_budget_estimated_change_weekly_clicks) && !(m.recommended_budget_estimated_change_weekly_clicks && $util.isInteger(m.recommended_budget_estimated_change_weekly_clicks.low) && $util.isInteger(m.recommended_budget_estimated_change_weekly_clicks.high)))
                                        return "recommended_budget_estimated_change_weekly_clicks: integer|Long expected";
                                }
                                if (m.recommended_budget_estimated_change_weekly_cost_micros != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_cost_micros")) {
                                    p._recommended_budget_estimated_change_weekly_cost_micros = 1;
                                    if (!$util.isInteger(m.recommended_budget_estimated_change_weekly_cost_micros) && !(m.recommended_budget_estimated_change_weekly_cost_micros && $util.isInteger(m.recommended_budget_estimated_change_weekly_cost_micros.low) && $util.isInteger(m.recommended_budget_estimated_change_weekly_cost_micros.high)))
                                        return "recommended_budget_estimated_change_weekly_cost_micros: integer|Long expected";
                                }
                                if (m.recommended_budget_estimated_change_weekly_interactions != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_interactions")) {
                                    p._recommended_budget_estimated_change_weekly_interactions = 1;
                                    if (!$util.isInteger(m.recommended_budget_estimated_change_weekly_interactions) && !(m.recommended_budget_estimated_change_weekly_interactions && $util.isInteger(m.recommended_budget_estimated_change_weekly_interactions.low) && $util.isInteger(m.recommended_budget_estimated_change_weekly_interactions.high)))
                                        return "recommended_budget_estimated_change_weekly_interactions: integer|Long expected";
                                }
                                if (m.recommended_budget_estimated_change_weekly_views != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_views")) {
                                    p._recommended_budget_estimated_change_weekly_views = 1;
                                    if (!$util.isInteger(m.recommended_budget_estimated_change_weekly_views) && !(m.recommended_budget_estimated_change_weekly_views && $util.isInteger(m.recommended_budget_estimated_change_weekly_views.low) && $util.isInteger(m.recommended_budget_estimated_change_weekly_views.high)))
                                        return "recommended_budget_estimated_change_weekly_views: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignBudget message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignBudget} CampaignBudget
                             */
                            CampaignBudget.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignBudget)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignBudget();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.amount_micros != null) {
                                    if ($util.Long)
                                        (m.amount_micros = $util.Long.fromValue(d.amount_micros)).unsigned = false;
                                    else if (typeof d.amount_micros === "string")
                                        m.amount_micros = parseInt(d.amount_micros, 10);
                                    else if (typeof d.amount_micros === "number")
                                        m.amount_micros = d.amount_micros;
                                    else if (typeof d.amount_micros === "object")
                                        m.amount_micros = new $util.LongBits(d.amount_micros.low >>> 0, d.amount_micros.high >>> 0).toNumber();
                                }
                                if (d.total_amount_micros != null) {
                                    if ($util.Long)
                                        (m.total_amount_micros = $util.Long.fromValue(d.total_amount_micros)).unsigned = false;
                                    else if (typeof d.total_amount_micros === "string")
                                        m.total_amount_micros = parseInt(d.total_amount_micros, 10);
                                    else if (typeof d.total_amount_micros === "number")
                                        m.total_amount_micros = d.total_amount_micros;
                                    else if (typeof d.total_amount_micros === "object")
                                        m.total_amount_micros = new $util.LongBits(d.total_amount_micros.low >>> 0, d.total_amount_micros.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                switch (d.delivery_method) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.delivery_method = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.delivery_method = 1;
                                    break;
                                case "STANDARD":
                                case 2:
                                    m.delivery_method = 2;
                                    break;
                                case "ACCELERATED":
                                case 3:
                                    m.delivery_method = 3;
                                    break;
                                }
                                if (d.explicitly_shared != null) {
                                    m.explicitly_shared = Boolean(d.explicitly_shared);
                                }
                                if (d.reference_count != null) {
                                    if ($util.Long)
                                        (m.reference_count = $util.Long.fromValue(d.reference_count)).unsigned = false;
                                    else if (typeof d.reference_count === "string")
                                        m.reference_count = parseInt(d.reference_count, 10);
                                    else if (typeof d.reference_count === "number")
                                        m.reference_count = d.reference_count;
                                    else if (typeof d.reference_count === "object")
                                        m.reference_count = new $util.LongBits(d.reference_count.low >>> 0, d.reference_count.high >>> 0).toNumber();
                                }
                                if (d.has_recommended_budget != null) {
                                    m.has_recommended_budget = Boolean(d.has_recommended_budget);
                                }
                                if (d.recommended_budget_amount_micros != null) {
                                    if ($util.Long)
                                        (m.recommended_budget_amount_micros = $util.Long.fromValue(d.recommended_budget_amount_micros)).unsigned = false;
                                    else if (typeof d.recommended_budget_amount_micros === "string")
                                        m.recommended_budget_amount_micros = parseInt(d.recommended_budget_amount_micros, 10);
                                    else if (typeof d.recommended_budget_amount_micros === "number")
                                        m.recommended_budget_amount_micros = d.recommended_budget_amount_micros;
                                    else if (typeof d.recommended_budget_amount_micros === "object")
                                        m.recommended_budget_amount_micros = new $util.LongBits(d.recommended_budget_amount_micros.low >>> 0, d.recommended_budget_amount_micros.high >>> 0).toNumber();
                                }
                                switch (d.period) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.period = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.period = 1;
                                    break;
                                case "DAILY":
                                case 2:
                                    m.period = 2;
                                    break;
                                case "CUSTOM_PERIOD":
                                case 5:
                                    m.period = 5;
                                    break;
                                }
                                if (d.recommended_budget_estimated_change_weekly_clicks != null) {
                                    if ($util.Long)
                                        (m.recommended_budget_estimated_change_weekly_clicks = $util.Long.fromValue(d.recommended_budget_estimated_change_weekly_clicks)).unsigned = false;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_clicks === "string")
                                        m.recommended_budget_estimated_change_weekly_clicks = parseInt(d.recommended_budget_estimated_change_weekly_clicks, 10);
                                    else if (typeof d.recommended_budget_estimated_change_weekly_clicks === "number")
                                        m.recommended_budget_estimated_change_weekly_clicks = d.recommended_budget_estimated_change_weekly_clicks;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_clicks === "object")
                                        m.recommended_budget_estimated_change_weekly_clicks = new $util.LongBits(d.recommended_budget_estimated_change_weekly_clicks.low >>> 0, d.recommended_budget_estimated_change_weekly_clicks.high >>> 0).toNumber();
                                }
                                if (d.recommended_budget_estimated_change_weekly_cost_micros != null) {
                                    if ($util.Long)
                                        (m.recommended_budget_estimated_change_weekly_cost_micros = $util.Long.fromValue(d.recommended_budget_estimated_change_weekly_cost_micros)).unsigned = false;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_cost_micros === "string")
                                        m.recommended_budget_estimated_change_weekly_cost_micros = parseInt(d.recommended_budget_estimated_change_weekly_cost_micros, 10);
                                    else if (typeof d.recommended_budget_estimated_change_weekly_cost_micros === "number")
                                        m.recommended_budget_estimated_change_weekly_cost_micros = d.recommended_budget_estimated_change_weekly_cost_micros;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_cost_micros === "object")
                                        m.recommended_budget_estimated_change_weekly_cost_micros = new $util.LongBits(d.recommended_budget_estimated_change_weekly_cost_micros.low >>> 0, d.recommended_budget_estimated_change_weekly_cost_micros.high >>> 0).toNumber();
                                }
                                if (d.recommended_budget_estimated_change_weekly_interactions != null) {
                                    if ($util.Long)
                                        (m.recommended_budget_estimated_change_weekly_interactions = $util.Long.fromValue(d.recommended_budget_estimated_change_weekly_interactions)).unsigned = false;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_interactions === "string")
                                        m.recommended_budget_estimated_change_weekly_interactions = parseInt(d.recommended_budget_estimated_change_weekly_interactions, 10);
                                    else if (typeof d.recommended_budget_estimated_change_weekly_interactions === "number")
                                        m.recommended_budget_estimated_change_weekly_interactions = d.recommended_budget_estimated_change_weekly_interactions;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_interactions === "object")
                                        m.recommended_budget_estimated_change_weekly_interactions = new $util.LongBits(d.recommended_budget_estimated_change_weekly_interactions.low >>> 0, d.recommended_budget_estimated_change_weekly_interactions.high >>> 0).toNumber();
                                }
                                if (d.recommended_budget_estimated_change_weekly_views != null) {
                                    if ($util.Long)
                                        (m.recommended_budget_estimated_change_weekly_views = $util.Long.fromValue(d.recommended_budget_estimated_change_weekly_views)).unsigned = false;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_views === "string")
                                        m.recommended_budget_estimated_change_weekly_views = parseInt(d.recommended_budget_estimated_change_weekly_views, 10);
                                    else if (typeof d.recommended_budget_estimated_change_weekly_views === "number")
                                        m.recommended_budget_estimated_change_weekly_views = d.recommended_budget_estimated_change_weekly_views;
                                    else if (typeof d.recommended_budget_estimated_change_weekly_views === "object")
                                        m.recommended_budget_estimated_change_weekly_views = new $util.LongBits(d.recommended_budget_estimated_change_weekly_views.low >>> 0, d.recommended_budget_estimated_change_weekly_views.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "STANDARD":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "FIXED_CPA":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "SMART_CAMPAIGN":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "LOCAL_SERVICES":
                                case 6:
                                    m.type = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignBudget message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignBudget} m CampaignBudget
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignBudget.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.delivery_method = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.period = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.BudgetStatusEnum.BudgetStatus[m.status] : m.status;
                                }
                                if (m.delivery_method != null && m.hasOwnProperty("delivery_method")) {
                                    d.delivery_method = o.enums === String ? $root.google.ads.googleads.v11.enums.BudgetDeliveryMethodEnum.BudgetDeliveryMethod[m.delivery_method] : m.delivery_method;
                                }
                                if (m.period != null && m.hasOwnProperty("period")) {
                                    d.period = o.enums === String ? $root.google.ads.googleads.v11.enums.BudgetPeriodEnum.BudgetPeriod[m.period] : m.period;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.BudgetTypeEnum.BudgetType[m.type] : m.type;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.amount_micros != null && m.hasOwnProperty("amount_micros")) {
                                    if (typeof m.amount_micros === "number")
                                        d.amount_micros = o.longs === String ? String(m.amount_micros) : m.amount_micros;
                                    else
                                        d.amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.amount_micros) : o.longs === Number ? new $util.LongBits(m.amount_micros.low >>> 0, m.amount_micros.high >>> 0).toNumber() : m.amount_micros;
                                    if (o.oneofs)
                                        d._amount_micros = "amount_micros";
                                }
                                if (m.total_amount_micros != null && m.hasOwnProperty("total_amount_micros")) {
                                    if (typeof m.total_amount_micros === "number")
                                        d.total_amount_micros = o.longs === String ? String(m.total_amount_micros) : m.total_amount_micros;
                                    else
                                        d.total_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.total_amount_micros) : o.longs === Number ? new $util.LongBits(m.total_amount_micros.low >>> 0, m.total_amount_micros.high >>> 0).toNumber() : m.total_amount_micros;
                                    if (o.oneofs)
                                        d._total_amount_micros = "total_amount_micros";
                                }
                                if (m.explicitly_shared != null && m.hasOwnProperty("explicitly_shared")) {
                                    d.explicitly_shared = m.explicitly_shared;
                                    if (o.oneofs)
                                        d._explicitly_shared = "explicitly_shared";
                                }
                                if (m.reference_count != null && m.hasOwnProperty("reference_count")) {
                                    if (typeof m.reference_count === "number")
                                        d.reference_count = o.longs === String ? String(m.reference_count) : m.reference_count;
                                    else
                                        d.reference_count = o.longs === String ? $util.Long.prototype.toString.call(m.reference_count) : o.longs === Number ? new $util.LongBits(m.reference_count.low >>> 0, m.reference_count.high >>> 0).toNumber() : m.reference_count;
                                    if (o.oneofs)
                                        d._reference_count = "reference_count";
                                }
                                if (m.has_recommended_budget != null && m.hasOwnProperty("has_recommended_budget")) {
                                    d.has_recommended_budget = m.has_recommended_budget;
                                    if (o.oneofs)
                                        d._has_recommended_budget = "has_recommended_budget";
                                }
                                if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                    if (typeof m.recommended_budget_amount_micros === "number")
                                        d.recommended_budget_amount_micros = o.longs === String ? String(m.recommended_budget_amount_micros) : m.recommended_budget_amount_micros;
                                    else
                                        d.recommended_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.recommended_budget_amount_micros.low >>> 0, m.recommended_budget_amount_micros.high >>> 0).toNumber() : m.recommended_budget_amount_micros;
                                    if (o.oneofs)
                                        d._recommended_budget_amount_micros = "recommended_budget_amount_micros";
                                }
                                if (m.recommended_budget_estimated_change_weekly_clicks != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_clicks")) {
                                    if (typeof m.recommended_budget_estimated_change_weekly_clicks === "number")
                                        d.recommended_budget_estimated_change_weekly_clicks = o.longs === String ? String(m.recommended_budget_estimated_change_weekly_clicks) : m.recommended_budget_estimated_change_weekly_clicks;
                                    else
                                        d.recommended_budget_estimated_change_weekly_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_estimated_change_weekly_clicks) : o.longs === Number ? new $util.LongBits(m.recommended_budget_estimated_change_weekly_clicks.low >>> 0, m.recommended_budget_estimated_change_weekly_clicks.high >>> 0).toNumber() : m.recommended_budget_estimated_change_weekly_clicks;
                                    if (o.oneofs)
                                        d._recommended_budget_estimated_change_weekly_clicks = "recommended_budget_estimated_change_weekly_clicks";
                                }
                                if (m.recommended_budget_estimated_change_weekly_cost_micros != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_cost_micros")) {
                                    if (typeof m.recommended_budget_estimated_change_weekly_cost_micros === "number")
                                        d.recommended_budget_estimated_change_weekly_cost_micros = o.longs === String ? String(m.recommended_budget_estimated_change_weekly_cost_micros) : m.recommended_budget_estimated_change_weekly_cost_micros;
                                    else
                                        d.recommended_budget_estimated_change_weekly_cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_estimated_change_weekly_cost_micros) : o.longs === Number ? new $util.LongBits(m.recommended_budget_estimated_change_weekly_cost_micros.low >>> 0, m.recommended_budget_estimated_change_weekly_cost_micros.high >>> 0).toNumber() : m.recommended_budget_estimated_change_weekly_cost_micros;
                                    if (o.oneofs)
                                        d._recommended_budget_estimated_change_weekly_cost_micros = "recommended_budget_estimated_change_weekly_cost_micros";
                                }
                                if (m.recommended_budget_estimated_change_weekly_interactions != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_interactions")) {
                                    if (typeof m.recommended_budget_estimated_change_weekly_interactions === "number")
                                        d.recommended_budget_estimated_change_weekly_interactions = o.longs === String ? String(m.recommended_budget_estimated_change_weekly_interactions) : m.recommended_budget_estimated_change_weekly_interactions;
                                    else
                                        d.recommended_budget_estimated_change_weekly_interactions = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_estimated_change_weekly_interactions) : o.longs === Number ? new $util.LongBits(m.recommended_budget_estimated_change_weekly_interactions.low >>> 0, m.recommended_budget_estimated_change_weekly_interactions.high >>> 0).toNumber() : m.recommended_budget_estimated_change_weekly_interactions;
                                    if (o.oneofs)
                                        d._recommended_budget_estimated_change_weekly_interactions = "recommended_budget_estimated_change_weekly_interactions";
                                }
                                if (m.recommended_budget_estimated_change_weekly_views != null && m.hasOwnProperty("recommended_budget_estimated_change_weekly_views")) {
                                    if (typeof m.recommended_budget_estimated_change_weekly_views === "number")
                                        d.recommended_budget_estimated_change_weekly_views = o.longs === String ? String(m.recommended_budget_estimated_change_weekly_views) : m.recommended_budget_estimated_change_weekly_views;
                                    else
                                        d.recommended_budget_estimated_change_weekly_views = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_estimated_change_weekly_views) : o.longs === Number ? new $util.LongBits(m.recommended_budget_estimated_change_weekly_views.low >>> 0, m.recommended_budget_estimated_change_weekly_views.high >>> 0).toNumber() : m.recommended_budget_estimated_change_weekly_views;
                                    if (o.oneofs)
                                        d._recommended_budget_estimated_change_weekly_views = "recommended_budget_estimated_change_weekly_views";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignBudget to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignBudget
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignBudget.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignBudget;
                        })();
    
                        resources.CampaignConversionGoal = (function() {
    
                            /**
                             * Properties of a CampaignConversionGoal.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignConversionGoal
                             * @property {string|null} [resource_name] CampaignConversionGoal resource_name
                             * @property {string|null} [campaign] CampaignConversionGoal campaign
                             * @property {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory|null} [category] CampaignConversionGoal category
                             * @property {google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin|null} [origin] CampaignConversionGoal origin
                             * @property {boolean|null} [biddable] CampaignConversionGoal biddable
                             */
    
                            /**
                             * Constructs a new CampaignConversionGoal.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignConversionGoal.
                             * @implements ICampaignConversionGoal
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignConversionGoal=} [p] Properties to set
                             */
                            function CampaignConversionGoal(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignConversionGoal resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @instance
                             */
                            CampaignConversionGoal.prototype.resource_name = "";
    
                            /**
                             * CampaignConversionGoal campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @instance
                             */
                            CampaignConversionGoal.prototype.campaign = "";
    
                            /**
                             * CampaignConversionGoal category.
                             * @member {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory} category
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @instance
                             */
                            CampaignConversionGoal.prototype.category = 0;
    
                            /**
                             * CampaignConversionGoal origin.
                             * @member {google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin} origin
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @instance
                             */
                            CampaignConversionGoal.prototype.origin = 0;
    
                            /**
                             * CampaignConversionGoal biddable.
                             * @member {boolean} biddable
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @instance
                             */
                            CampaignConversionGoal.prototype.biddable = false;
    
                            /**
                             * Creates a new CampaignConversionGoal instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignConversionGoal=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignConversionGoal} CampaignConversionGoal instance
                             */
                            CampaignConversionGoal.create = function create(properties) {
                                return new CampaignConversionGoal(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignConversionGoal message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignConversionGoal.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignConversionGoal} m CampaignConversionGoal message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignConversionGoal.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(24).int32(m.category);
                                if (m.origin != null && Object.hasOwnProperty.call(m, "origin"))
                                    w.uint32(32).int32(m.origin);
                                if (m.biddable != null && Object.hasOwnProperty.call(m, "biddable"))
                                    w.uint32(40).bool(m.biddable);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignConversionGoal message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignConversionGoal.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignConversionGoal} message CampaignConversionGoal message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignConversionGoal.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignConversionGoal message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignConversionGoal} CampaignConversionGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignConversionGoal.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignConversionGoal();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    case 3:
                                        m.category = r.int32();
                                        break;
                                    case 4:
                                        m.origin = r.int32();
                                        break;
                                    case 5:
                                        m.biddable = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignConversionGoal message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignConversionGoal} CampaignConversionGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignConversionGoal.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignConversionGoal message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignConversionGoal.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    switch (m.category) {
                                    default:
                                        return "category: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                        break;
                                    }
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    switch (m.origin) {
                                    default:
                                        return "origin: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.biddable != null && m.hasOwnProperty("biddable")) {
                                    if (typeof m.biddable !== "boolean")
                                        return "biddable: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignConversionGoal message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignConversionGoal} CampaignConversionGoal
                             */
                            CampaignConversionGoal.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignConversionGoal)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignConversionGoal();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                switch (d.category) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.category = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.category = 1;
                                    break;
                                case "DEFAULT":
                                case 2:
                                    m.category = 2;
                                    break;
                                case "PAGE_VIEW":
                                case 3:
                                    m.category = 3;
                                    break;
                                case "PURCHASE":
                                case 4:
                                    m.category = 4;
                                    break;
                                case "SIGNUP":
                                case 5:
                                    m.category = 5;
                                    break;
                                case "DOWNLOAD":
                                case 7:
                                    m.category = 7;
                                    break;
                                case "ADD_TO_CART":
                                case 8:
                                    m.category = 8;
                                    break;
                                case "BEGIN_CHECKOUT":
                                case 9:
                                    m.category = 9;
                                    break;
                                case "SUBSCRIBE_PAID":
                                case 10:
                                    m.category = 10;
                                    break;
                                case "PHONE_CALL_LEAD":
                                case 11:
                                    m.category = 11;
                                    break;
                                case "IMPORTED_LEAD":
                                case 12:
                                    m.category = 12;
                                    break;
                                case "SUBMIT_LEAD_FORM":
                                case 13:
                                    m.category = 13;
                                    break;
                                case "BOOK_APPOINTMENT":
                                case 14:
                                    m.category = 14;
                                    break;
                                case "REQUEST_QUOTE":
                                case 15:
                                    m.category = 15;
                                    break;
                                case "GET_DIRECTIONS":
                                case 16:
                                    m.category = 16;
                                    break;
                                case "OUTBOUND_CLICK":
                                case 17:
                                    m.category = 17;
                                    break;
                                case "CONTACT":
                                case 18:
                                    m.category = 18;
                                    break;
                                case "ENGAGEMENT":
                                case 19:
                                    m.category = 19;
                                    break;
                                case "STORE_VISIT":
                                case 20:
                                    m.category = 20;
                                    break;
                                case "STORE_SALE":
                                case 21:
                                    m.category = 21;
                                    break;
                                case "QUALIFIED_LEAD":
                                case 22:
                                    m.category = 22;
                                    break;
                                case "CONVERTED_LEAD":
                                case 23:
                                    m.category = 23;
                                    break;
                                }
                                switch (d.origin) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.origin = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.origin = 1;
                                    break;
                                case "WEBSITE":
                                case 2:
                                    m.origin = 2;
                                    break;
                                case "GOOGLE_HOSTED":
                                case 3:
                                    m.origin = 3;
                                    break;
                                case "APP":
                                case 4:
                                    m.origin = 4;
                                    break;
                                case "CALL_FROM_ADS":
                                case 5:
                                    m.origin = 5;
                                    break;
                                case "STORE":
                                case 6:
                                    m.origin = 6;
                                    break;
                                case "YOUTUBE_HOSTED":
                                case 7:
                                    m.origin = 7;
                                    break;
                                }
                                if (d.biddable != null) {
                                    m.biddable = Boolean(d.biddable);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignConversionGoal message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignConversionGoal} m CampaignConversionGoal
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignConversionGoal.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign = "";
                                    d.category = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.origin = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.biddable = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory[m.category] : m.category;
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    d.origin = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin[m.origin] : m.origin;
                                }
                                if (m.biddable != null && m.hasOwnProperty("biddable")) {
                                    d.biddable = m.biddable;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignConversionGoal to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignConversionGoal
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignConversionGoal.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignConversionGoal;
                        })();
    
                        resources.CampaignCriterion = (function() {
    
                            /**
                             * Properties of a CampaignCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignCriterion
                             * @property {string|null} [resource_name] CampaignCriterion resource_name
                             * @property {string|null} [campaign] CampaignCriterion campaign
                             * @property {number|null} [criterion_id] CampaignCriterion criterion_id
                             * @property {string|null} [display_name] CampaignCriterion display_name
                             * @property {number|null} [bid_modifier] CampaignCriterion bid_modifier
                             * @property {boolean|null} [negative] CampaignCriterion negative
                             * @property {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType|null} [type] CampaignCriterion type
                             * @property {google.ads.googleads.v11.enums.CampaignCriterionStatusEnum.CampaignCriterionStatus|null} [status] CampaignCriterion status
                             * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [keyword] CampaignCriterion keyword
                             * @property {google.ads.googleads.v11.common.IPlacementInfo|null} [placement] CampaignCriterion placement
                             * @property {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null} [mobile_app_category] CampaignCriterion mobile_app_category
                             * @property {google.ads.googleads.v11.common.IMobileApplicationInfo|null} [mobile_application] CampaignCriterion mobile_application
                             * @property {google.ads.googleads.v11.common.ILocationInfo|null} [location] CampaignCriterion location
                             * @property {google.ads.googleads.v11.common.IDeviceInfo|null} [device] CampaignCriterion device
                             * @property {google.ads.googleads.v11.common.IAdScheduleInfo|null} [ad_schedule] CampaignCriterion ad_schedule
                             * @property {google.ads.googleads.v11.common.IAgeRangeInfo|null} [age_range] CampaignCriterion age_range
                             * @property {google.ads.googleads.v11.common.IGenderInfo|null} [gender] CampaignCriterion gender
                             * @property {google.ads.googleads.v11.common.IIncomeRangeInfo|null} [income_range] CampaignCriterion income_range
                             * @property {google.ads.googleads.v11.common.IParentalStatusInfo|null} [parental_status] CampaignCriterion parental_status
                             * @property {google.ads.googleads.v11.common.IUserListInfo|null} [user_list] CampaignCriterion user_list
                             * @property {google.ads.googleads.v11.common.IYouTubeVideoInfo|null} [youtube_video] CampaignCriterion youtube_video
                             * @property {google.ads.googleads.v11.common.IYouTubeChannelInfo|null} [youtube_channel] CampaignCriterion youtube_channel
                             * @property {google.ads.googleads.v11.common.IProximityInfo|null} [proximity] CampaignCriterion proximity
                             * @property {google.ads.googleads.v11.common.ITopicInfo|null} [topic] CampaignCriterion topic
                             * @property {google.ads.googleads.v11.common.IListingScopeInfo|null} [listing_scope] CampaignCriterion listing_scope
                             * @property {google.ads.googleads.v11.common.ILanguageInfo|null} [language] CampaignCriterion language
                             * @property {google.ads.googleads.v11.common.IIpBlockInfo|null} [ip_block] CampaignCriterion ip_block
                             * @property {google.ads.googleads.v11.common.IContentLabelInfo|null} [content_label] CampaignCriterion content_label
                             * @property {google.ads.googleads.v11.common.ICarrierInfo|null} [carrier] CampaignCriterion carrier
                             * @property {google.ads.googleads.v11.common.IUserInterestInfo|null} [user_interest] CampaignCriterion user_interest
                             * @property {google.ads.googleads.v11.common.IWebpageInfo|null} [webpage] CampaignCriterion webpage
                             * @property {google.ads.googleads.v11.common.IOperatingSystemVersionInfo|null} [operating_system_version] CampaignCriterion operating_system_version
                             * @property {google.ads.googleads.v11.common.IMobileDeviceInfo|null} [mobile_device] CampaignCriterion mobile_device
                             * @property {google.ads.googleads.v11.common.ILocationGroupInfo|null} [location_group] CampaignCriterion location_group
                             * @property {google.ads.googleads.v11.common.ICustomAffinityInfo|null} [custom_affinity] CampaignCriterion custom_affinity
                             * @property {google.ads.googleads.v11.common.ICustomAudienceInfo|null} [custom_audience] CampaignCriterion custom_audience
                             * @property {google.ads.googleads.v11.common.ICombinedAudienceInfo|null} [combined_audience] CampaignCriterion combined_audience
                             * @property {google.ads.googleads.v11.common.IKeywordThemeInfo|null} [keyword_theme] CampaignCriterion keyword_theme
                             */
    
                            /**
                             * Constructs a new CampaignCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignCriterion.
                             * @implements ICampaignCriterion
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterion=} [p] Properties to set
                             */
                            function CampaignCriterion(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignCriterion resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.resource_name = "";
    
                            /**
                             * CampaignCriterion campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.campaign = null;
    
                            /**
                             * CampaignCriterion criterion_id.
                             * @member {number|null|undefined} criterion_id
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.criterion_id = null;
    
                            /**
                             * CampaignCriterion display_name.
                             * @member {string} display_name
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.display_name = "";
    
                            /**
                             * CampaignCriterion bid_modifier.
                             * @member {number|null|undefined} bid_modifier
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.bid_modifier = null;
    
                            /**
                             * CampaignCriterion negative.
                             * @member {boolean|null|undefined} negative
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.negative = null;
    
                            /**
                             * CampaignCriterion type.
                             * @member {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType} type
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.type = 0;
    
                            /**
                             * CampaignCriterion status.
                             * @member {google.ads.googleads.v11.enums.CampaignCriterionStatusEnum.CampaignCriterionStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.status = 0;
    
                            /**
                             * CampaignCriterion keyword.
                             * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} keyword
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.keyword = null;
    
                            /**
                             * CampaignCriterion placement.
                             * @member {google.ads.googleads.v11.common.IPlacementInfo|null|undefined} placement
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.placement = null;
    
                            /**
                             * CampaignCriterion mobile_app_category.
                             * @member {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null|undefined} mobile_app_category
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.mobile_app_category = null;
    
                            /**
                             * CampaignCriterion mobile_application.
                             * @member {google.ads.googleads.v11.common.IMobileApplicationInfo|null|undefined} mobile_application
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.mobile_application = null;
    
                            /**
                             * CampaignCriterion location.
                             * @member {google.ads.googleads.v11.common.ILocationInfo|null|undefined} location
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.location = null;
    
                            /**
                             * CampaignCriterion device.
                             * @member {google.ads.googleads.v11.common.IDeviceInfo|null|undefined} device
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.device = null;
    
                            /**
                             * CampaignCriterion ad_schedule.
                             * @member {google.ads.googleads.v11.common.IAdScheduleInfo|null|undefined} ad_schedule
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.ad_schedule = null;
    
                            /**
                             * CampaignCriterion age_range.
                             * @member {google.ads.googleads.v11.common.IAgeRangeInfo|null|undefined} age_range
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.age_range = null;
    
                            /**
                             * CampaignCriterion gender.
                             * @member {google.ads.googleads.v11.common.IGenderInfo|null|undefined} gender
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.gender = null;
    
                            /**
                             * CampaignCriterion income_range.
                             * @member {google.ads.googleads.v11.common.IIncomeRangeInfo|null|undefined} income_range
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.income_range = null;
    
                            /**
                             * CampaignCriterion parental_status.
                             * @member {google.ads.googleads.v11.common.IParentalStatusInfo|null|undefined} parental_status
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.parental_status = null;
    
                            /**
                             * CampaignCriterion user_list.
                             * @member {google.ads.googleads.v11.common.IUserListInfo|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.user_list = null;
    
                            /**
                             * CampaignCriterion youtube_video.
                             * @member {google.ads.googleads.v11.common.IYouTubeVideoInfo|null|undefined} youtube_video
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.youtube_video = null;
    
                            /**
                             * CampaignCriterion youtube_channel.
                             * @member {google.ads.googleads.v11.common.IYouTubeChannelInfo|null|undefined} youtube_channel
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.youtube_channel = null;
    
                            /**
                             * CampaignCriterion proximity.
                             * @member {google.ads.googleads.v11.common.IProximityInfo|null|undefined} proximity
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.proximity = null;
    
                            /**
                             * CampaignCriterion topic.
                             * @member {google.ads.googleads.v11.common.ITopicInfo|null|undefined} topic
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.topic = null;
    
                            /**
                             * CampaignCriterion listing_scope.
                             * @member {google.ads.googleads.v11.common.IListingScopeInfo|null|undefined} listing_scope
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.listing_scope = null;
    
                            /**
                             * CampaignCriterion language.
                             * @member {google.ads.googleads.v11.common.ILanguageInfo|null|undefined} language
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.language = null;
    
                            /**
                             * CampaignCriterion ip_block.
                             * @member {google.ads.googleads.v11.common.IIpBlockInfo|null|undefined} ip_block
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.ip_block = null;
    
                            /**
                             * CampaignCriterion content_label.
                             * @member {google.ads.googleads.v11.common.IContentLabelInfo|null|undefined} content_label
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.content_label = null;
    
                            /**
                             * CampaignCriterion carrier.
                             * @member {google.ads.googleads.v11.common.ICarrierInfo|null|undefined} carrier
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.carrier = null;
    
                            /**
                             * CampaignCriterion user_interest.
                             * @member {google.ads.googleads.v11.common.IUserInterestInfo|null|undefined} user_interest
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.user_interest = null;
    
                            /**
                             * CampaignCriterion webpage.
                             * @member {google.ads.googleads.v11.common.IWebpageInfo|null|undefined} webpage
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.webpage = null;
    
                            /**
                             * CampaignCriterion operating_system_version.
                             * @member {google.ads.googleads.v11.common.IOperatingSystemVersionInfo|null|undefined} operating_system_version
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.operating_system_version = null;
    
                            /**
                             * CampaignCriterion mobile_device.
                             * @member {google.ads.googleads.v11.common.IMobileDeviceInfo|null|undefined} mobile_device
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.mobile_device = null;
    
                            /**
                             * CampaignCriterion location_group.
                             * @member {google.ads.googleads.v11.common.ILocationGroupInfo|null|undefined} location_group
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.location_group = null;
    
                            /**
                             * CampaignCriterion custom_affinity.
                             * @member {google.ads.googleads.v11.common.ICustomAffinityInfo|null|undefined} custom_affinity
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.custom_affinity = null;
    
                            /**
                             * CampaignCriterion custom_audience.
                             * @member {google.ads.googleads.v11.common.ICustomAudienceInfo|null|undefined} custom_audience
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.custom_audience = null;
    
                            /**
                             * CampaignCriterion combined_audience.
                             * @member {google.ads.googleads.v11.common.ICombinedAudienceInfo|null|undefined} combined_audience
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.combined_audience = null;
    
                            /**
                             * CampaignCriterion keyword_theme.
                             * @member {google.ads.googleads.v11.common.IKeywordThemeInfo|null|undefined} keyword_theme
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            CampaignCriterion.prototype.keyword_theme = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignCriterion _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterion.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterion _criterion_id.
                             * @member {"criterion_id"|undefined} _criterion_id
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterion.prototype, "_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterion _bid_modifier.
                             * @member {"bid_modifier"|undefined} _bid_modifier
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterion.prototype, "_bid_modifier", {
                                get: $util.oneOfGetter($oneOfFields = ["bid_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterion _negative.
                             * @member {"negative"|undefined} _negative
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterion.prototype, "_negative", {
                                get: $util.oneOfGetter($oneOfFields = ["negative"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterion criterion.
                             * @member {"keyword"|"placement"|"mobile_app_category"|"mobile_application"|"location"|"device"|"ad_schedule"|"age_range"|"gender"|"income_range"|"parental_status"|"user_list"|"youtube_video"|"youtube_channel"|"proximity"|"topic"|"listing_scope"|"language"|"ip_block"|"content_label"|"carrier"|"user_interest"|"webpage"|"operating_system_version"|"mobile_device"|"location_group"|"custom_affinity"|"custom_audience"|"combined_audience"|"keyword_theme"|undefined} criterion
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterion.prototype, "criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword", "placement", "mobile_app_category", "mobile_application", "location", "device", "ad_schedule", "age_range", "gender", "income_range", "parental_status", "user_list", "youtube_video", "youtube_channel", "proximity", "topic", "listing_scope", "language", "ip_block", "content_label", "carrier", "user_interest", "webpage", "operating_system_version", "mobile_device", "location_group", "custom_affinity", "custom_audience", "combined_audience", "keyword_theme"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignCriterion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterion} CampaignCriterion instance
                             */
                            CampaignCriterion.create = function create(properties) {
                                return new CampaignCriterion(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCriterion message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignCriterion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterion} m CampaignCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(48).int32(m.type);
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword, w.uint32(66).fork()).ldelim();
                                if (m.placement != null && Object.hasOwnProperty.call(m, "placement"))
                                    $root.google.ads.googleads.v11.common.PlacementInfo.encode(m.placement, w.uint32(74).fork()).ldelim();
                                if (m.mobile_app_category != null && Object.hasOwnProperty.call(m, "mobile_app_category"))
                                    $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.encode(m.mobile_app_category, w.uint32(82).fork()).ldelim();
                                if (m.mobile_application != null && Object.hasOwnProperty.call(m, "mobile_application"))
                                    $root.google.ads.googleads.v11.common.MobileApplicationInfo.encode(m.mobile_application, w.uint32(90).fork()).ldelim();
                                if (m.location != null && Object.hasOwnProperty.call(m, "location"))
                                    $root.google.ads.googleads.v11.common.LocationInfo.encode(m.location, w.uint32(98).fork()).ldelim();
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    $root.google.ads.googleads.v11.common.DeviceInfo.encode(m.device, w.uint32(106).fork()).ldelim();
                                if (m.ad_schedule != null && Object.hasOwnProperty.call(m, "ad_schedule"))
                                    $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedule, w.uint32(122).fork()).ldelim();
                                if (m.age_range != null && Object.hasOwnProperty.call(m, "age_range"))
                                    $root.google.ads.googleads.v11.common.AgeRangeInfo.encode(m.age_range, w.uint32(130).fork()).ldelim();
                                if (m.gender != null && Object.hasOwnProperty.call(m, "gender"))
                                    $root.google.ads.googleads.v11.common.GenderInfo.encode(m.gender, w.uint32(138).fork()).ldelim();
                                if (m.income_range != null && Object.hasOwnProperty.call(m, "income_range"))
                                    $root.google.ads.googleads.v11.common.IncomeRangeInfo.encode(m.income_range, w.uint32(146).fork()).ldelim();
                                if (m.parental_status != null && Object.hasOwnProperty.call(m, "parental_status"))
                                    $root.google.ads.googleads.v11.common.ParentalStatusInfo.encode(m.parental_status, w.uint32(154).fork()).ldelim();
                                if (m.youtube_video != null && Object.hasOwnProperty.call(m, "youtube_video"))
                                    $root.google.ads.googleads.v11.common.YouTubeVideoInfo.encode(m.youtube_video, w.uint32(162).fork()).ldelim();
                                if (m.youtube_channel != null && Object.hasOwnProperty.call(m, "youtube_channel"))
                                    $root.google.ads.googleads.v11.common.YouTubeChannelInfo.encode(m.youtube_channel, w.uint32(170).fork()).ldelim();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    $root.google.ads.googleads.v11.common.UserListInfo.encode(m.user_list, w.uint32(178).fork()).ldelim();
                                if (m.proximity != null && Object.hasOwnProperty.call(m, "proximity"))
                                    $root.google.ads.googleads.v11.common.ProximityInfo.encode(m.proximity, w.uint32(186).fork()).ldelim();
                                if (m.topic != null && Object.hasOwnProperty.call(m, "topic"))
                                    $root.google.ads.googleads.v11.common.TopicInfo.encode(m.topic, w.uint32(194).fork()).ldelim();
                                if (m.listing_scope != null && Object.hasOwnProperty.call(m, "listing_scope"))
                                    $root.google.ads.googleads.v11.common.ListingScopeInfo.encode(m.listing_scope, w.uint32(202).fork()).ldelim();
                                if (m.language != null && Object.hasOwnProperty.call(m, "language"))
                                    $root.google.ads.googleads.v11.common.LanguageInfo.encode(m.language, w.uint32(210).fork()).ldelim();
                                if (m.ip_block != null && Object.hasOwnProperty.call(m, "ip_block"))
                                    $root.google.ads.googleads.v11.common.IpBlockInfo.encode(m.ip_block, w.uint32(218).fork()).ldelim();
                                if (m.content_label != null && Object.hasOwnProperty.call(m, "content_label"))
                                    $root.google.ads.googleads.v11.common.ContentLabelInfo.encode(m.content_label, w.uint32(226).fork()).ldelim();
                                if (m.carrier != null && Object.hasOwnProperty.call(m, "carrier"))
                                    $root.google.ads.googleads.v11.common.CarrierInfo.encode(m.carrier, w.uint32(234).fork()).ldelim();
                                if (m.user_interest != null && Object.hasOwnProperty.call(m, "user_interest"))
                                    $root.google.ads.googleads.v11.common.UserInterestInfo.encode(m.user_interest, w.uint32(242).fork()).ldelim();
                                if (m.webpage != null && Object.hasOwnProperty.call(m, "webpage"))
                                    $root.google.ads.googleads.v11.common.WebpageInfo.encode(m.webpage, w.uint32(250).fork()).ldelim();
                                if (m.operating_system_version != null && Object.hasOwnProperty.call(m, "operating_system_version"))
                                    $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo.encode(m.operating_system_version, w.uint32(258).fork()).ldelim();
                                if (m.mobile_device != null && Object.hasOwnProperty.call(m, "mobile_device"))
                                    $root.google.ads.googleads.v11.common.MobileDeviceInfo.encode(m.mobile_device, w.uint32(266).fork()).ldelim();
                                if (m.location_group != null && Object.hasOwnProperty.call(m, "location_group"))
                                    $root.google.ads.googleads.v11.common.LocationGroupInfo.encode(m.location_group, w.uint32(274).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(280).int32(m.status);
                                if (m.custom_affinity != null && Object.hasOwnProperty.call(m, "custom_affinity"))
                                    $root.google.ads.googleads.v11.common.CustomAffinityInfo.encode(m.custom_affinity, w.uint32(290).fork()).ldelim();
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(298).string(m.campaign);
                                if (m.criterion_id != null && Object.hasOwnProperty.call(m, "criterion_id"))
                                    w.uint32(304).int64(m.criterion_id);
                                if (m.bid_modifier != null && Object.hasOwnProperty.call(m, "bid_modifier"))
                                    w.uint32(317).float(m.bid_modifier);
                                if (m.negative != null && Object.hasOwnProperty.call(m, "negative"))
                                    w.uint32(320).bool(m.negative);
                                if (m.custom_audience != null && Object.hasOwnProperty.call(m, "custom_audience"))
                                    $root.google.ads.googleads.v11.common.CustomAudienceInfo.encode(m.custom_audience, w.uint32(330).fork()).ldelim();
                                if (m.combined_audience != null && Object.hasOwnProperty.call(m, "combined_audience"))
                                    $root.google.ads.googleads.v11.common.CombinedAudienceInfo.encode(m.combined_audience, w.uint32(338).fork()).ldelim();
                                if (m.display_name != null && Object.hasOwnProperty.call(m, "display_name"))
                                    w.uint32(346).string(m.display_name);
                                if (m.keyword_theme != null && Object.hasOwnProperty.call(m, "keyword_theme"))
                                    $root.google.ads.googleads.v11.common.KeywordThemeInfo.encode(m.keyword_theme, w.uint32(362).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCriterion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignCriterion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterion} message CampaignCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCriterion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterion} CampaignCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignCriterion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 37:
                                        m.campaign = r.string();
                                        break;
                                    case 38:
                                        m.criterion_id = r.int64();
                                        break;
                                    case 43:
                                        m.display_name = r.string();
                                        break;
                                    case 39:
                                        m.bid_modifier = r.float();
                                        break;
                                    case 40:
                                        m.negative = r.bool();
                                        break;
                                    case 6:
                                        m.type = r.int32();
                                        break;
                                    case 35:
                                        m.status = r.int32();
                                        break;
                                    case 8:
                                        m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.location = $root.google.ads.googleads.v11.common.LocationInfo.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.device = $root.google.ads.googleads.v11.common.DeviceInfo.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.ad_schedule = $root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.decode(r, r.uint32());
                                        break;
                                    case 17:
                                        m.gender = $root.google.ads.googleads.v11.common.GenderInfo.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.income_range = $root.google.ads.googleads.v11.common.IncomeRangeInfo.decode(r, r.uint32());
                                        break;
                                    case 19:
                                        m.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusInfo.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.user_list = $root.google.ads.googleads.v11.common.UserListInfo.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.proximity = $root.google.ads.googleads.v11.common.ProximityInfo.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.topic = $root.google.ads.googleads.v11.common.TopicInfo.decode(r, r.uint32());
                                        break;
                                    case 25:
                                        m.listing_scope = $root.google.ads.googleads.v11.common.ListingScopeInfo.decode(r, r.uint32());
                                        break;
                                    case 26:
                                        m.language = $root.google.ads.googleads.v11.common.LanguageInfo.decode(r, r.uint32());
                                        break;
                                    case 27:
                                        m.ip_block = $root.google.ads.googleads.v11.common.IpBlockInfo.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.content_label = $root.google.ads.googleads.v11.common.ContentLabelInfo.decode(r, r.uint32());
                                        break;
                                    case 29:
                                        m.carrier = $root.google.ads.googleads.v11.common.CarrierInfo.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.webpage = $root.google.ads.googleads.v11.common.WebpageInfo.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.operating_system_version = $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo.decode(r, r.uint32());
                                        break;
                                    case 33:
                                        m.mobile_device = $root.google.ads.googleads.v11.common.MobileDeviceInfo.decode(r, r.uint32());
                                        break;
                                    case 34:
                                        m.location_group = $root.google.ads.googleads.v11.common.LocationGroupInfo.decode(r, r.uint32());
                                        break;
                                    case 36:
                                        m.custom_affinity = $root.google.ads.googleads.v11.common.CustomAffinityInfo.decode(r, r.uint32());
                                        break;
                                    case 41:
                                        m.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceInfo.decode(r, r.uint32());
                                        break;
                                    case 42:
                                        m.combined_audience = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.decode(r, r.uint32());
                                        break;
                                    case 45:
                                        m.keyword_theme = $root.google.ads.googleads.v11.common.KeywordThemeInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCriterion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterion} CampaignCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCriterion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCriterion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    p._criterion_id = 1;
                                    if (!$util.isInteger(m.criterion_id) && !(m.criterion_id && $util.isInteger(m.criterion_id.low) && $util.isInteger(m.criterion_id.high)))
                                        return "criterion_id: integer|Long expected";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    if (!$util.isString(m.display_name))
                                        return "display_name: string expected";
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    p._bid_modifier = 1;
                                    if (typeof m.bid_modifier !== "number")
                                        return "bid_modifier: number expected";
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    p._negative = 1;
                                    if (typeof m.negative !== "boolean")
                                        return "negative: boolean expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword);
                                        if (e)
                                            return "keyword." + e;
                                    }
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PlacementInfo.verify(m.placement);
                                        if (e)
                                            return "placement." + e;
                                    }
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.verify(m.mobile_app_category);
                                        if (e)
                                            return "mobile_app_category." + e;
                                    }
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileApplicationInfo.verify(m.mobile_application);
                                        if (e)
                                            return "mobile_application." + e;
                                    }
                                }
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LocationInfo.verify(m.location);
                                        if (e)
                                            return "location." + e;
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DeviceInfo.verify(m.device);
                                        if (e)
                                            return "device." + e;
                                    }
                                }
                                if (m.ad_schedule != null && m.hasOwnProperty("ad_schedule")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedule);
                                        if (e)
                                            return "ad_schedule." + e;
                                    }
                                }
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AgeRangeInfo.verify(m.age_range);
                                        if (e)
                                            return "age_range." + e;
                                    }
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GenderInfo.verify(m.gender);
                                        if (e)
                                            return "gender." + e;
                                    }
                                }
                                if (m.income_range != null && m.hasOwnProperty("income_range")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.IncomeRangeInfo.verify(m.income_range);
                                        if (e)
                                            return "income_range." + e;
                                    }
                                }
                                if (m.parental_status != null && m.hasOwnProperty("parental_status")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ParentalStatusInfo.verify(m.parental_status);
                                        if (e)
                                            return "parental_status." + e;
                                    }
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserListInfo.verify(m.user_list);
                                        if (e)
                                            return "user_list." + e;
                                    }
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.verify(m.youtube_video);
                                        if (e)
                                            return "youtube_video." + e;
                                    }
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.verify(m.youtube_channel);
                                        if (e)
                                            return "youtube_channel." + e;
                                    }
                                }
                                if (m.proximity != null && m.hasOwnProperty("proximity")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProximityInfo.verify(m.proximity);
                                        if (e)
                                            return "proximity." + e;
                                    }
                                }
                                if (m.topic != null && m.hasOwnProperty("topic")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TopicInfo.verify(m.topic);
                                        if (e)
                                            return "topic." + e;
                                    }
                                }
                                if (m.listing_scope != null && m.hasOwnProperty("listing_scope")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ListingScopeInfo.verify(m.listing_scope);
                                        if (e)
                                            return "listing_scope." + e;
                                    }
                                }
                                if (m.language != null && m.hasOwnProperty("language")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LanguageInfo.verify(m.language);
                                        if (e)
                                            return "language." + e;
                                    }
                                }
                                if (m.ip_block != null && m.hasOwnProperty("ip_block")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.IpBlockInfo.verify(m.ip_block);
                                        if (e)
                                            return "ip_block." + e;
                                    }
                                }
                                if (m.content_label != null && m.hasOwnProperty("content_label")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ContentLabelInfo.verify(m.content_label);
                                        if (e)
                                            return "content_label." + e;
                                    }
                                }
                                if (m.carrier != null && m.hasOwnProperty("carrier")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CarrierInfo.verify(m.carrier);
                                        if (e)
                                            return "carrier." + e;
                                    }
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserInterestInfo.verify(m.user_interest);
                                        if (e)
                                            return "user_interest." + e;
                                    }
                                }
                                if (m.webpage != null && m.hasOwnProperty("webpage")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.WebpageInfo.verify(m.webpage);
                                        if (e)
                                            return "webpage." + e;
                                    }
                                }
                                if (m.operating_system_version != null && m.hasOwnProperty("operating_system_version")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo.verify(m.operating_system_version);
                                        if (e)
                                            return "operating_system_version." + e;
                                    }
                                }
                                if (m.mobile_device != null && m.hasOwnProperty("mobile_device")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileDeviceInfo.verify(m.mobile_device);
                                        if (e)
                                            return "mobile_device." + e;
                                    }
                                }
                                if (m.location_group != null && m.hasOwnProperty("location_group")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LocationGroupInfo.verify(m.location_group);
                                        if (e)
                                            return "location_group." + e;
                                    }
                                }
                                if (m.custom_affinity != null && m.hasOwnProperty("custom_affinity")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomAffinityInfo.verify(m.custom_affinity);
                                        if (e)
                                            return "custom_affinity." + e;
                                    }
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomAudienceInfo.verify(m.custom_audience);
                                        if (e)
                                            return "custom_audience." + e;
                                    }
                                }
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.verify(m.combined_audience);
                                        if (e)
                                            return "combined_audience." + e;
                                    }
                                }
                                if (m.keyword_theme != null && m.hasOwnProperty("keyword_theme")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordThemeInfo.verify(m.keyword_theme);
                                        if (e)
                                            return "keyword_theme." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCriterion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterion} CampaignCriterion
                             */
                            CampaignCriterion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignCriterion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignCriterion();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.criterion_id != null) {
                                    if ($util.Long)
                                        (m.criterion_id = $util.Long.fromValue(d.criterion_id)).unsigned = false;
                                    else if (typeof d.criterion_id === "string")
                                        m.criterion_id = parseInt(d.criterion_id, 10);
                                    else if (typeof d.criterion_id === "number")
                                        m.criterion_id = d.criterion_id;
                                    else if (typeof d.criterion_id === "object")
                                        m.criterion_id = new $util.LongBits(d.criterion_id.low >>> 0, d.criterion_id.high >>> 0).toNumber();
                                }
                                if (d.display_name != null) {
                                    m.display_name = String(d.display_name);
                                }
                                if (d.bid_modifier != null) {
                                    m.bid_modifier = Number(d.bid_modifier);
                                }
                                if (d.negative != null) {
                                    m.negative = Boolean(d.negative);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "PLACEMENT":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MOBILE_APP_CATEGORY":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "MOBILE_APPLICATION":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "DEVICE":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "LOCATION":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "LISTING_GROUP":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "AD_SCHEDULE":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "AGE_RANGE":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "GENDER":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "INCOME_RANGE":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "PARENTAL_STATUS":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "YOUTUBE_CHANNEL":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "USER_LIST":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "PROXIMITY":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "TOPIC":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "LISTING_SCOPE":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "LANGUAGE":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "IP_BLOCK":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "CONTENT_LABEL":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "CARRIER":
                                case 23:
                                    m.type = 23;
                                    break;
                                case "USER_INTEREST":
                                case 24:
                                    m.type = 24;
                                    break;
                                case "WEBPAGE":
                                case 25:
                                    m.type = 25;
                                    break;
                                case "OPERATING_SYSTEM_VERSION":
                                case 26:
                                    m.type = 26;
                                    break;
                                case "APP_PAYMENT_MODEL":
                                case 27:
                                    m.type = 27;
                                    break;
                                case "MOBILE_DEVICE":
                                case 28:
                                    m.type = 28;
                                    break;
                                case "CUSTOM_AFFINITY":
                                case 29:
                                    m.type = 29;
                                    break;
                                case "CUSTOM_INTENT":
                                case 30:
                                    m.type = 30;
                                    break;
                                case "LOCATION_GROUP":
                                case 31:
                                    m.type = 31;
                                    break;
                                case "CUSTOM_AUDIENCE":
                                case 32:
                                    m.type = 32;
                                    break;
                                case "COMBINED_AUDIENCE":
                                case 33:
                                    m.type = 33;
                                    break;
                                case "KEYWORD_THEME":
                                case 34:
                                    m.type = 34;
                                    break;
                                case "AUDIENCE":
                                case 35:
                                    m.type = 35;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "PAUSED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.keyword != null) {
                                    if (typeof d.keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.keyword: object expected");
                                    m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword);
                                }
                                if (d.placement != null) {
                                    if (typeof d.placement !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.placement: object expected");
                                    m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.fromObject(d.placement);
                                }
                                if (d.mobile_app_category != null) {
                                    if (typeof d.mobile_app_category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.mobile_app_category: object expected");
                                    m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.fromObject(d.mobile_app_category);
                                }
                                if (d.mobile_application != null) {
                                    if (typeof d.mobile_application !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.mobile_application: object expected");
                                    m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.fromObject(d.mobile_application);
                                }
                                if (d.location != null) {
                                    if (typeof d.location !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.location: object expected");
                                    m.location = $root.google.ads.googleads.v11.common.LocationInfo.fromObject(d.location);
                                }
                                if (d.device != null) {
                                    if (typeof d.device !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.device: object expected");
                                    m.device = $root.google.ads.googleads.v11.common.DeviceInfo.fromObject(d.device);
                                }
                                if (d.ad_schedule != null) {
                                    if (typeof d.ad_schedule !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.ad_schedule: object expected");
                                    m.ad_schedule = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedule);
                                }
                                if (d.age_range != null) {
                                    if (typeof d.age_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.age_range: object expected");
                                    m.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.fromObject(d.age_range);
                                }
                                if (d.gender != null) {
                                    if (typeof d.gender !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.gender: object expected");
                                    m.gender = $root.google.ads.googleads.v11.common.GenderInfo.fromObject(d.gender);
                                }
                                if (d.income_range != null) {
                                    if (typeof d.income_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.income_range: object expected");
                                    m.income_range = $root.google.ads.googleads.v11.common.IncomeRangeInfo.fromObject(d.income_range);
                                }
                                if (d.parental_status != null) {
                                    if (typeof d.parental_status !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.parental_status: object expected");
                                    m.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusInfo.fromObject(d.parental_status);
                                }
                                if (d.user_list != null) {
                                    if (typeof d.user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.user_list: object expected");
                                    m.user_list = $root.google.ads.googleads.v11.common.UserListInfo.fromObject(d.user_list);
                                }
                                if (d.youtube_video != null) {
                                    if (typeof d.youtube_video !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.youtube_video: object expected");
                                    m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.fromObject(d.youtube_video);
                                }
                                if (d.youtube_channel != null) {
                                    if (typeof d.youtube_channel !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.youtube_channel: object expected");
                                    m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.fromObject(d.youtube_channel);
                                }
                                if (d.proximity != null) {
                                    if (typeof d.proximity !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.proximity: object expected");
                                    m.proximity = $root.google.ads.googleads.v11.common.ProximityInfo.fromObject(d.proximity);
                                }
                                if (d.topic != null) {
                                    if (typeof d.topic !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.topic: object expected");
                                    m.topic = $root.google.ads.googleads.v11.common.TopicInfo.fromObject(d.topic);
                                }
                                if (d.listing_scope != null) {
                                    if (typeof d.listing_scope !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.listing_scope: object expected");
                                    m.listing_scope = $root.google.ads.googleads.v11.common.ListingScopeInfo.fromObject(d.listing_scope);
                                }
                                if (d.language != null) {
                                    if (typeof d.language !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.language: object expected");
                                    m.language = $root.google.ads.googleads.v11.common.LanguageInfo.fromObject(d.language);
                                }
                                if (d.ip_block != null) {
                                    if (typeof d.ip_block !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.ip_block: object expected");
                                    m.ip_block = $root.google.ads.googleads.v11.common.IpBlockInfo.fromObject(d.ip_block);
                                }
                                if (d.content_label != null) {
                                    if (typeof d.content_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.content_label: object expected");
                                    m.content_label = $root.google.ads.googleads.v11.common.ContentLabelInfo.fromObject(d.content_label);
                                }
                                if (d.carrier != null) {
                                    if (typeof d.carrier !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.carrier: object expected");
                                    m.carrier = $root.google.ads.googleads.v11.common.CarrierInfo.fromObject(d.carrier);
                                }
                                if (d.user_interest != null) {
                                    if (typeof d.user_interest !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.user_interest: object expected");
                                    m.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.fromObject(d.user_interest);
                                }
                                if (d.webpage != null) {
                                    if (typeof d.webpage !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.webpage: object expected");
                                    m.webpage = $root.google.ads.googleads.v11.common.WebpageInfo.fromObject(d.webpage);
                                }
                                if (d.operating_system_version != null) {
                                    if (typeof d.operating_system_version !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.operating_system_version: object expected");
                                    m.operating_system_version = $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo.fromObject(d.operating_system_version);
                                }
                                if (d.mobile_device != null) {
                                    if (typeof d.mobile_device !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.mobile_device: object expected");
                                    m.mobile_device = $root.google.ads.googleads.v11.common.MobileDeviceInfo.fromObject(d.mobile_device);
                                }
                                if (d.location_group != null) {
                                    if (typeof d.location_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.location_group: object expected");
                                    m.location_group = $root.google.ads.googleads.v11.common.LocationGroupInfo.fromObject(d.location_group);
                                }
                                if (d.custom_affinity != null) {
                                    if (typeof d.custom_affinity !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.custom_affinity: object expected");
                                    m.custom_affinity = $root.google.ads.googleads.v11.common.CustomAffinityInfo.fromObject(d.custom_affinity);
                                }
                                if (d.custom_audience != null) {
                                    if (typeof d.custom_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.custom_audience: object expected");
                                    m.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceInfo.fromObject(d.custom_audience);
                                }
                                if (d.combined_audience != null) {
                                    if (typeof d.combined_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.combined_audience: object expected");
                                    m.combined_audience = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.fromObject(d.combined_audience);
                                }
                                if (d.keyword_theme != null) {
                                    if (typeof d.keyword_theme !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterion.keyword_theme: object expected");
                                    m.keyword_theme = $root.google.ads.googleads.v11.common.KeywordThemeInfo.fromObject(d.keyword_theme);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignCriterion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignCriterion} m CampaignCriterion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCriterion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.display_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType[m.type] : m.type;
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword, o);
                                    if (o.oneofs)
                                        d.criterion = "keyword";
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    d.placement = $root.google.ads.googleads.v11.common.PlacementInfo.toObject(m.placement, o);
                                    if (o.oneofs)
                                        d.criterion = "placement";
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    d.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.toObject(m.mobile_app_category, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_app_category";
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    d.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.toObject(m.mobile_application, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_application";
                                }
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    d.location = $root.google.ads.googleads.v11.common.LocationInfo.toObject(m.location, o);
                                    if (o.oneofs)
                                        d.criterion = "location";
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = $root.google.ads.googleads.v11.common.DeviceInfo.toObject(m.device, o);
                                    if (o.oneofs)
                                        d.criterion = "device";
                                }
                                if (m.ad_schedule != null && m.hasOwnProperty("ad_schedule")) {
                                    d.ad_schedule = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedule, o);
                                    if (o.oneofs)
                                        d.criterion = "ad_schedule";
                                }
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    d.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.toObject(m.age_range, o);
                                    if (o.oneofs)
                                        d.criterion = "age_range";
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    d.gender = $root.google.ads.googleads.v11.common.GenderInfo.toObject(m.gender, o);
                                    if (o.oneofs)
                                        d.criterion = "gender";
                                }
                                if (m.income_range != null && m.hasOwnProperty("income_range")) {
                                    d.income_range = $root.google.ads.googleads.v11.common.IncomeRangeInfo.toObject(m.income_range, o);
                                    if (o.oneofs)
                                        d.criterion = "income_range";
                                }
                                if (m.parental_status != null && m.hasOwnProperty("parental_status")) {
                                    d.parental_status = $root.google.ads.googleads.v11.common.ParentalStatusInfo.toObject(m.parental_status, o);
                                    if (o.oneofs)
                                        d.criterion = "parental_status";
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    d.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.toObject(m.youtube_video, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_video";
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    d.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.toObject(m.youtube_channel, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_channel";
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = $root.google.ads.googleads.v11.common.UserListInfo.toObject(m.user_list, o);
                                    if (o.oneofs)
                                        d.criterion = "user_list";
                                }
                                if (m.proximity != null && m.hasOwnProperty("proximity")) {
                                    d.proximity = $root.google.ads.googleads.v11.common.ProximityInfo.toObject(m.proximity, o);
                                    if (o.oneofs)
                                        d.criterion = "proximity";
                                }
                                if (m.topic != null && m.hasOwnProperty("topic")) {
                                    d.topic = $root.google.ads.googleads.v11.common.TopicInfo.toObject(m.topic, o);
                                    if (o.oneofs)
                                        d.criterion = "topic";
                                }
                                if (m.listing_scope != null && m.hasOwnProperty("listing_scope")) {
                                    d.listing_scope = $root.google.ads.googleads.v11.common.ListingScopeInfo.toObject(m.listing_scope, o);
                                    if (o.oneofs)
                                        d.criterion = "listing_scope";
                                }
                                if (m.language != null && m.hasOwnProperty("language")) {
                                    d.language = $root.google.ads.googleads.v11.common.LanguageInfo.toObject(m.language, o);
                                    if (o.oneofs)
                                        d.criterion = "language";
                                }
                                if (m.ip_block != null && m.hasOwnProperty("ip_block")) {
                                    d.ip_block = $root.google.ads.googleads.v11.common.IpBlockInfo.toObject(m.ip_block, o);
                                    if (o.oneofs)
                                        d.criterion = "ip_block";
                                }
                                if (m.content_label != null && m.hasOwnProperty("content_label")) {
                                    d.content_label = $root.google.ads.googleads.v11.common.ContentLabelInfo.toObject(m.content_label, o);
                                    if (o.oneofs)
                                        d.criterion = "content_label";
                                }
                                if (m.carrier != null && m.hasOwnProperty("carrier")) {
                                    d.carrier = $root.google.ads.googleads.v11.common.CarrierInfo.toObject(m.carrier, o);
                                    if (o.oneofs)
                                        d.criterion = "carrier";
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    d.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.toObject(m.user_interest, o);
                                    if (o.oneofs)
                                        d.criterion = "user_interest";
                                }
                                if (m.webpage != null && m.hasOwnProperty("webpage")) {
                                    d.webpage = $root.google.ads.googleads.v11.common.WebpageInfo.toObject(m.webpage, o);
                                    if (o.oneofs)
                                        d.criterion = "webpage";
                                }
                                if (m.operating_system_version != null && m.hasOwnProperty("operating_system_version")) {
                                    d.operating_system_version = $root.google.ads.googleads.v11.common.OperatingSystemVersionInfo.toObject(m.operating_system_version, o);
                                    if (o.oneofs)
                                        d.criterion = "operating_system_version";
                                }
                                if (m.mobile_device != null && m.hasOwnProperty("mobile_device")) {
                                    d.mobile_device = $root.google.ads.googleads.v11.common.MobileDeviceInfo.toObject(m.mobile_device, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_device";
                                }
                                if (m.location_group != null && m.hasOwnProperty("location_group")) {
                                    d.location_group = $root.google.ads.googleads.v11.common.LocationGroupInfo.toObject(m.location_group, o);
                                    if (o.oneofs)
                                        d.criterion = "location_group";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignCriterionStatusEnum.CampaignCriterionStatus[m.status] : m.status;
                                }
                                if (m.custom_affinity != null && m.hasOwnProperty("custom_affinity")) {
                                    d.custom_affinity = $root.google.ads.googleads.v11.common.CustomAffinityInfo.toObject(m.custom_affinity, o);
                                    if (o.oneofs)
                                        d.criterion = "custom_affinity";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    if (typeof m.criterion_id === "number")
                                        d.criterion_id = o.longs === String ? String(m.criterion_id) : m.criterion_id;
                                    else
                                        d.criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.criterion_id) : o.longs === Number ? new $util.LongBits(m.criterion_id.low >>> 0, m.criterion_id.high >>> 0).toNumber() : m.criterion_id;
                                    if (o.oneofs)
                                        d._criterion_id = "criterion_id";
                                }
                                if (m.bid_modifier != null && m.hasOwnProperty("bid_modifier")) {
                                    d.bid_modifier = o.json && !isFinite(m.bid_modifier) ? String(m.bid_modifier) : m.bid_modifier;
                                    if (o.oneofs)
                                        d._bid_modifier = "bid_modifier";
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    d.negative = m.negative;
                                    if (o.oneofs)
                                        d._negative = "negative";
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    d.custom_audience = $root.google.ads.googleads.v11.common.CustomAudienceInfo.toObject(m.custom_audience, o);
                                    if (o.oneofs)
                                        d.criterion = "custom_audience";
                                }
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    d.combined_audience = $root.google.ads.googleads.v11.common.CombinedAudienceInfo.toObject(m.combined_audience, o);
                                    if (o.oneofs)
                                        d.criterion = "combined_audience";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    d.display_name = m.display_name;
                                }
                                if (m.keyword_theme != null && m.hasOwnProperty("keyword_theme")) {
                                    d.keyword_theme = $root.google.ads.googleads.v11.common.KeywordThemeInfo.toObject(m.keyword_theme, o);
                                    if (o.oneofs)
                                        d.criterion = "keyword_theme";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignCriterion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCriterion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignCriterion;
                        })();
    
                        resources.CampaignCriterionSimulation = (function() {
    
                            /**
                             * Properties of a CampaignCriterionSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignCriterionSimulation
                             * @property {string|null} [resource_name] CampaignCriterionSimulation resource_name
                             * @property {number|null} [campaign_id] CampaignCriterionSimulation campaign_id
                             * @property {number|null} [criterion_id] CampaignCriterionSimulation criterion_id
                             * @property {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType|null} [type] CampaignCriterionSimulation type
                             * @property {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod|null} [modification_method] CampaignCriterionSimulation modification_method
                             * @property {string|null} [start_date] CampaignCriterionSimulation start_date
                             * @property {string|null} [end_date] CampaignCriterionSimulation end_date
                             * @property {google.ads.googleads.v11.common.IBidModifierSimulationPointList|null} [bid_modifier_point_list] CampaignCriterionSimulation bid_modifier_point_list
                             */
    
                            /**
                             * Constructs a new CampaignCriterionSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignCriterionSimulation.
                             * @implements ICampaignCriterionSimulation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterionSimulation=} [p] Properties to set
                             */
                            function CampaignCriterionSimulation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignCriterionSimulation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.resource_name = "";
    
                            /**
                             * CampaignCriterionSimulation campaign_id.
                             * @member {number|null|undefined} campaign_id
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.campaign_id = null;
    
                            /**
                             * CampaignCriterionSimulation criterion_id.
                             * @member {number|null|undefined} criterion_id
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.criterion_id = null;
    
                            /**
                             * CampaignCriterionSimulation type.
                             * @member {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType} type
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.type = 0;
    
                            /**
                             * CampaignCriterionSimulation modification_method.
                             * @member {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod} modification_method
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.modification_method = 0;
    
                            /**
                             * CampaignCriterionSimulation start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.start_date = null;
    
                            /**
                             * CampaignCriterionSimulation end_date.
                             * @member {string|null|undefined} end_date
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.end_date = null;
    
                            /**
                             * CampaignCriterionSimulation bid_modifier_point_list.
                             * @member {google.ads.googleads.v11.common.IBidModifierSimulationPointList|null|undefined} bid_modifier_point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            CampaignCriterionSimulation.prototype.bid_modifier_point_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignCriterionSimulation _campaign_id.
                             * @member {"campaign_id"|undefined} _campaign_id
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterionSimulation.prototype, "_campaign_id", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterionSimulation _criterion_id.
                             * @member {"criterion_id"|undefined} _criterion_id
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterionSimulation.prototype, "_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterionSimulation _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterionSimulation.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterionSimulation _end_date.
                             * @member {"end_date"|undefined} _end_date
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterionSimulation.prototype, "_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignCriterionSimulation point_list.
                             * @member {"bid_modifier_point_list"|undefined} point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterionSimulation.prototype, "point_list", {
                                get: $util.oneOfGetter($oneOfFields = ["bid_modifier_point_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignCriterionSimulation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterionSimulation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterionSimulation} CampaignCriterionSimulation instance
                             */
                            CampaignCriterionSimulation.create = function create(properties) {
                                return new CampaignCriterionSimulation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionSimulation message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignCriterionSimulation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterionSimulation} m CampaignCriterionSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionSimulation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.modification_method != null && Object.hasOwnProperty.call(m, "modification_method"))
                                    w.uint32(40).int32(m.modification_method);
                                if (m.bid_modifier_point_list != null && Object.hasOwnProperty.call(m, "bid_modifier_point_list"))
                                    $root.google.ads.googleads.v11.common.BidModifierSimulationPointList.encode(m.bid_modifier_point_list, w.uint32(66).fork()).ldelim();
                                if (m.campaign_id != null && Object.hasOwnProperty.call(m, "campaign_id"))
                                    w.uint32(72).int64(m.campaign_id);
                                if (m.criterion_id != null && Object.hasOwnProperty.call(m, "criterion_id"))
                                    w.uint32(80).int64(m.criterion_id);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(90).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(98).string(m.end_date);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionSimulation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignCriterionSimulation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCriterionSimulation} message CampaignCriterionSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionSimulation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCriterionSimulation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterionSimulation} CampaignCriterionSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionSimulation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.campaign_id = r.int64();
                                        break;
                                    case 10:
                                        m.criterion_id = r.int64();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 5:
                                        m.modification_method = r.int32();
                                        break;
                                    case 11:
                                        m.start_date = r.string();
                                        break;
                                    case 12:
                                        m.end_date = r.string();
                                        break;
                                    case 8:
                                        m.bid_modifier_point_list = $root.google.ads.googleads.v11.common.BidModifierSimulationPointList.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCriterionSimulation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterionSimulation} CampaignCriterionSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionSimulation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCriterionSimulation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCriterionSimulation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_id != null && m.hasOwnProperty("campaign_id")) {
                                    p._campaign_id = 1;
                                    if (!$util.isInteger(m.campaign_id) && !(m.campaign_id && $util.isInteger(m.campaign_id.low) && $util.isInteger(m.campaign_id.high)))
                                        return "campaign_id: integer|Long expected";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    p._criterion_id = 1;
                                    if (!$util.isInteger(m.criterion_id) && !(m.criterion_id && $util.isInteger(m.criterion_id.low) && $util.isInteger(m.criterion_id.high)))
                                        return "criterion_id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    switch (m.modification_method) {
                                    default:
                                        return "modification_method: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    p._end_date = 1;
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.bid_modifier_point_list != null && m.hasOwnProperty("bid_modifier_point_list")) {
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.BidModifierSimulationPointList.verify(m.bid_modifier_point_list);
                                        if (e)
                                            return "bid_modifier_point_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCriterionSimulation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignCriterionSimulation} CampaignCriterionSimulation
                             */
                            CampaignCriterionSimulation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_id != null) {
                                    if ($util.Long)
                                        (m.campaign_id = $util.Long.fromValue(d.campaign_id)).unsigned = false;
                                    else if (typeof d.campaign_id === "string")
                                        m.campaign_id = parseInt(d.campaign_id, 10);
                                    else if (typeof d.campaign_id === "number")
                                        m.campaign_id = d.campaign_id;
                                    else if (typeof d.campaign_id === "object")
                                        m.campaign_id = new $util.LongBits(d.campaign_id.low >>> 0, d.campaign_id.high >>> 0).toNumber();
                                }
                                if (d.criterion_id != null) {
                                    if ($util.Long)
                                        (m.criterion_id = $util.Long.fromValue(d.criterion_id)).unsigned = false;
                                    else if (typeof d.criterion_id === "string")
                                        m.criterion_id = parseInt(d.criterion_id, 10);
                                    else if (typeof d.criterion_id === "number")
                                        m.criterion_id = d.criterion_id;
                                    else if (typeof d.criterion_id === "object")
                                        m.criterion_id = new $util.LongBits(d.criterion_id.low >>> 0, d.criterion_id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CPC_BID":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "CPV_BID":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "TARGET_CPA":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "BID_MODIFIER":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "TARGET_ROAS":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PERCENT_CPC_BID":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "BUDGET":
                                case 9:
                                    m.type = 9;
                                    break;
                                }
                                switch (d.modification_method) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.modification_method = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.modification_method = 1;
                                    break;
                                case "UNIFORM":
                                case 2:
                                    m.modification_method = 2;
                                    break;
                                case "DEFAULT":
                                case 3:
                                    m.modification_method = 3;
                                    break;
                                case "SCALING":
                                case 4:
                                    m.modification_method = 4;
                                    break;
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.bid_modifier_point_list != null) {
                                    if (typeof d.bid_modifier_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCriterionSimulation.bid_modifier_point_list: object expected");
                                    m.bid_modifier_point_list = $root.google.ads.googleads.v11.common.BidModifierSimulationPointList.fromObject(d.bid_modifier_point_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignCriterionSimulation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignCriterionSimulation} m CampaignCriterionSimulation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCriterionSimulation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.modification_method = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType[m.type] : m.type;
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    d.modification_method = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod[m.modification_method] : m.modification_method;
                                }
                                if (m.bid_modifier_point_list != null && m.hasOwnProperty("bid_modifier_point_list")) {
                                    d.bid_modifier_point_list = $root.google.ads.googleads.v11.common.BidModifierSimulationPointList.toObject(m.bid_modifier_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "bid_modifier_point_list";
                                }
                                if (m.campaign_id != null && m.hasOwnProperty("campaign_id")) {
                                    if (typeof m.campaign_id === "number")
                                        d.campaign_id = o.longs === String ? String(m.campaign_id) : m.campaign_id;
                                    else
                                        d.campaign_id = o.longs === String ? $util.Long.prototype.toString.call(m.campaign_id) : o.longs === Number ? new $util.LongBits(m.campaign_id.low >>> 0, m.campaign_id.high >>> 0).toNumber() : m.campaign_id;
                                    if (o.oneofs)
                                        d._campaign_id = "campaign_id";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    if (typeof m.criterion_id === "number")
                                        d.criterion_id = o.longs === String ? String(m.criterion_id) : m.criterion_id;
                                    else
                                        d.criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.criterion_id) : o.longs === Number ? new $util.LongBits(m.criterion_id.low >>> 0, m.criterion_id.high >>> 0).toNumber() : m.criterion_id;
                                    if (o.oneofs)
                                        d._criterion_id = "criterion_id";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                    if (o.oneofs)
                                        d._end_date = "end_date";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignCriterionSimulation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignCriterionSimulation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCriterionSimulation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignCriterionSimulation;
                        })();
    
                        resources.CampaignCustomizer = (function() {
    
                            /**
                             * Properties of a CampaignCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignCustomizer
                             * @property {string|null} [resource_name] CampaignCustomizer resource_name
                             * @property {string|null} [campaign] CampaignCustomizer campaign
                             * @property {string|null} [customizer_attribute] CampaignCustomizer customizer_attribute
                             * @property {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus|null} [status] CampaignCustomizer status
                             * @property {google.ads.googleads.v11.common.ICustomizerValue|null} [value] CampaignCustomizer value
                             */
    
                            /**
                             * Constructs a new CampaignCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignCustomizer.
                             * @implements ICampaignCustomizer
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignCustomizer=} [p] Properties to set
                             */
                            function CampaignCustomizer(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignCustomizer resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @instance
                             */
                            CampaignCustomizer.prototype.resource_name = "";
    
                            /**
                             * CampaignCustomizer campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @instance
                             */
                            CampaignCustomizer.prototype.campaign = "";
    
                            /**
                             * CampaignCustomizer customizer_attribute.
                             * @member {string} customizer_attribute
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @instance
                             */
                            CampaignCustomizer.prototype.customizer_attribute = "";
    
                            /**
                             * CampaignCustomizer status.
                             * @member {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @instance
                             */
                            CampaignCustomizer.prototype.status = 0;
    
                            /**
                             * CampaignCustomizer value.
                             * @member {google.ads.googleads.v11.common.ICustomizerValue|null|undefined} value
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @instance
                             */
                            CampaignCustomizer.prototype.value = null;
    
                            /**
                             * Creates a new CampaignCustomizer instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCustomizer=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignCustomizer} CampaignCustomizer instance
                             */
                            CampaignCustomizer.create = function create(properties) {
                                return new CampaignCustomizer(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCustomizer message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignCustomizer.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCustomizer} m CampaignCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCustomizer.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                if (m.customizer_attribute != null && Object.hasOwnProperty.call(m, "customizer_attribute"))
                                    w.uint32(26).string(m.customizer_attribute);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    $root.google.ads.googleads.v11.common.CustomizerValue.encode(m.value, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCustomizer message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignCustomizer.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignCustomizer} message CampaignCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCustomizer.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCustomizer message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignCustomizer} CampaignCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCustomizer.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignCustomizer();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    case 3:
                                        m.customizer_attribute = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.value = $root.google.ads.googleads.v11.common.CustomizerValue.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCustomizer message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignCustomizer} CampaignCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCustomizer.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCustomizer message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCustomizer.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    if (!$util.isString(m.customizer_attribute))
                                        return "customizer_attribute: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomizerValue.verify(m.value);
                                        if (e)
                                            return "value." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCustomizer message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignCustomizer} CampaignCustomizer
                             */
                            CampaignCustomizer.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignCustomizer)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignCustomizer();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.customizer_attribute != null) {
                                    m.customizer_attribute = String(d.customizer_attribute);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.value != null) {
                                    if (typeof d.value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignCustomizer.value: object expected");
                                    m.value = $root.google.ads.googleads.v11.common.CustomizerValue.fromObject(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignCustomizer message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignCustomizer} m CampaignCustomizer
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCustomizer.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign = "";
                                    d.customizer_attribute = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.value = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    d.customizer_attribute = m.customizer_attribute;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus[m.status] : m.status;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = $root.google.ads.googleads.v11.common.CustomizerValue.toObject(m.value, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignCustomizer to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignCustomizer
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCustomizer.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignCustomizer;
                        })();
    
                        resources.CampaignDraft = (function() {
    
                            /**
                             * Properties of a CampaignDraft.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignDraft
                             * @property {string|null} [resource_name] CampaignDraft resource_name
                             * @property {number|null} [draft_id] CampaignDraft draft_id
                             * @property {string|null} [base_campaign] CampaignDraft base_campaign
                             * @property {string|null} [name] CampaignDraft name
                             * @property {string|null} [draft_campaign] CampaignDraft draft_campaign
                             * @property {google.ads.googleads.v11.enums.CampaignDraftStatusEnum.CampaignDraftStatus|null} [status] CampaignDraft status
                             * @property {boolean|null} [has_experiment_running] CampaignDraft has_experiment_running
                             * @property {string|null} [long_running_operation] CampaignDraft long_running_operation
                             */
    
                            /**
                             * Constructs a new CampaignDraft.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignDraft.
                             * @implements ICampaignDraft
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignDraft=} [p] Properties to set
                             */
                            function CampaignDraft(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignDraft resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.resource_name = "";
    
                            /**
                             * CampaignDraft draft_id.
                             * @member {number|null|undefined} draft_id
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.draft_id = null;
    
                            /**
                             * CampaignDraft base_campaign.
                             * @member {string|null|undefined} base_campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.base_campaign = null;
    
                            /**
                             * CampaignDraft name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.name = null;
    
                            /**
                             * CampaignDraft draft_campaign.
                             * @member {string|null|undefined} draft_campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.draft_campaign = null;
    
                            /**
                             * CampaignDraft status.
                             * @member {google.ads.googleads.v11.enums.CampaignDraftStatusEnum.CampaignDraftStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.status = 0;
    
                            /**
                             * CampaignDraft has_experiment_running.
                             * @member {boolean|null|undefined} has_experiment_running
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.has_experiment_running = null;
    
                            /**
                             * CampaignDraft long_running_operation.
                             * @member {string|null|undefined} long_running_operation
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            CampaignDraft.prototype.long_running_operation = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignDraft _draft_id.
                             * @member {"draft_id"|undefined} _draft_id
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            Object.defineProperty(CampaignDraft.prototype, "_draft_id", {
                                get: $util.oneOfGetter($oneOfFields = ["draft_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignDraft _base_campaign.
                             * @member {"base_campaign"|undefined} _base_campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            Object.defineProperty(CampaignDraft.prototype, "_base_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["base_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignDraft _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            Object.defineProperty(CampaignDraft.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignDraft _draft_campaign.
                             * @member {"draft_campaign"|undefined} _draft_campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            Object.defineProperty(CampaignDraft.prototype, "_draft_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["draft_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignDraft _has_experiment_running.
                             * @member {"has_experiment_running"|undefined} _has_experiment_running
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            Object.defineProperty(CampaignDraft.prototype, "_has_experiment_running", {
                                get: $util.oneOfGetter($oneOfFields = ["has_experiment_running"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignDraft _long_running_operation.
                             * @member {"long_running_operation"|undefined} _long_running_operation
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             */
                            Object.defineProperty(CampaignDraft.prototype, "_long_running_operation", {
                                get: $util.oneOfGetter($oneOfFields = ["long_running_operation"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignDraft instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignDraft=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignDraft} CampaignDraft instance
                             */
                            CampaignDraft.create = function create(properties) {
                                return new CampaignDraft(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignDraft message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignDraft.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignDraft} m CampaignDraft message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraft.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(48).int32(m.status);
                                if (m.draft_id != null && Object.hasOwnProperty.call(m, "draft_id"))
                                    w.uint32(72).int64(m.draft_id);
                                if (m.base_campaign != null && Object.hasOwnProperty.call(m, "base_campaign"))
                                    w.uint32(82).string(m.base_campaign);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(90).string(m.name);
                                if (m.draft_campaign != null && Object.hasOwnProperty.call(m, "draft_campaign"))
                                    w.uint32(98).string(m.draft_campaign);
                                if (m.has_experiment_running != null && Object.hasOwnProperty.call(m, "has_experiment_running"))
                                    w.uint32(104).bool(m.has_experiment_running);
                                if (m.long_running_operation != null && Object.hasOwnProperty.call(m, "long_running_operation"))
                                    w.uint32(114).string(m.long_running_operation);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignDraft message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignDraft.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignDraft} message CampaignDraft message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraft.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignDraft message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignDraft} CampaignDraft
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraft.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignDraft();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.draft_id = r.int64();
                                        break;
                                    case 10:
                                        m.base_campaign = r.string();
                                        break;
                                    case 11:
                                        m.name = r.string();
                                        break;
                                    case 12:
                                        m.draft_campaign = r.string();
                                        break;
                                    case 6:
                                        m.status = r.int32();
                                        break;
                                    case 13:
                                        m.has_experiment_running = r.bool();
                                        break;
                                    case 14:
                                        m.long_running_operation = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignDraft message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignDraft} CampaignDraft
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraft.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignDraft message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignDraft.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.draft_id != null && m.hasOwnProperty("draft_id")) {
                                    p._draft_id = 1;
                                    if (!$util.isInteger(m.draft_id) && !(m.draft_id && $util.isInteger(m.draft_id.low) && $util.isInteger(m.draft_id.high)))
                                        return "draft_id: integer|Long expected";
                                }
                                if (m.base_campaign != null && m.hasOwnProperty("base_campaign")) {
                                    p._base_campaign = 1;
                                    if (!$util.isString(m.base_campaign))
                                        return "base_campaign: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.draft_campaign != null && m.hasOwnProperty("draft_campaign")) {
                                    p._draft_campaign = 1;
                                    if (!$util.isString(m.draft_campaign))
                                        return "draft_campaign: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 5:
                                    case 4:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.has_experiment_running != null && m.hasOwnProperty("has_experiment_running")) {
                                    p._has_experiment_running = 1;
                                    if (typeof m.has_experiment_running !== "boolean")
                                        return "has_experiment_running: boolean expected";
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    p._long_running_operation = 1;
                                    if (!$util.isString(m.long_running_operation))
                                        return "long_running_operation: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignDraft message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignDraft} CampaignDraft
                             */
                            CampaignDraft.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignDraft)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignDraft();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.draft_id != null) {
                                    if ($util.Long)
                                        (m.draft_id = $util.Long.fromValue(d.draft_id)).unsigned = false;
                                    else if (typeof d.draft_id === "string")
                                        m.draft_id = parseInt(d.draft_id, 10);
                                    else if (typeof d.draft_id === "number")
                                        m.draft_id = d.draft_id;
                                    else if (typeof d.draft_id === "object")
                                        m.draft_id = new $util.LongBits(d.draft_id.low >>> 0, d.draft_id.high >>> 0).toNumber();
                                }
                                if (d.base_campaign != null) {
                                    m.base_campaign = String(d.base_campaign);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.draft_campaign != null) {
                                    m.draft_campaign = String(d.draft_campaign);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "PROPOSED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PROMOTING":
                                case 5:
                                    m.status = 5;
                                    break;
                                case "PROMOTED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "PROMOTE_FAILED":
                                case 6:
                                    m.status = 6;
                                    break;
                                }
                                if (d.has_experiment_running != null) {
                                    m.has_experiment_running = Boolean(d.has_experiment_running);
                                }
                                if (d.long_running_operation != null) {
                                    m.long_running_operation = String(d.long_running_operation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignDraft message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignDraft} m CampaignDraft
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignDraft.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignDraftStatusEnum.CampaignDraftStatus[m.status] : m.status;
                                }
                                if (m.draft_id != null && m.hasOwnProperty("draft_id")) {
                                    if (typeof m.draft_id === "number")
                                        d.draft_id = o.longs === String ? String(m.draft_id) : m.draft_id;
                                    else
                                        d.draft_id = o.longs === String ? $util.Long.prototype.toString.call(m.draft_id) : o.longs === Number ? new $util.LongBits(m.draft_id.low >>> 0, m.draft_id.high >>> 0).toNumber() : m.draft_id;
                                    if (o.oneofs)
                                        d._draft_id = "draft_id";
                                }
                                if (m.base_campaign != null && m.hasOwnProperty("base_campaign")) {
                                    d.base_campaign = m.base_campaign;
                                    if (o.oneofs)
                                        d._base_campaign = "base_campaign";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.draft_campaign != null && m.hasOwnProperty("draft_campaign")) {
                                    d.draft_campaign = m.draft_campaign;
                                    if (o.oneofs)
                                        d._draft_campaign = "draft_campaign";
                                }
                                if (m.has_experiment_running != null && m.hasOwnProperty("has_experiment_running")) {
                                    d.has_experiment_running = m.has_experiment_running;
                                    if (o.oneofs)
                                        d._has_experiment_running = "has_experiment_running";
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    d.long_running_operation = m.long_running_operation;
                                    if (o.oneofs)
                                        d._long_running_operation = "long_running_operation";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignDraft to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignDraft
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignDraft.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignDraft;
                        })();
    
                        resources.CampaignExperiment = (function() {
    
                            /**
                             * Properties of a CampaignExperiment.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignExperiment
                             * @property {string|null} [resource_name] CampaignExperiment resource_name
                             * @property {number|null} [id] CampaignExperiment id
                             * @property {string|null} [campaign_draft] CampaignExperiment campaign_draft
                             * @property {string|null} [name] CampaignExperiment name
                             * @property {string|null} [description] CampaignExperiment description
                             * @property {number|null} [traffic_split_percent] CampaignExperiment traffic_split_percent
                             * @property {google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum.CampaignExperimentTrafficSplitType|null} [traffic_split_type] CampaignExperiment traffic_split_type
                             * @property {string|null} [experiment_campaign] CampaignExperiment experiment_campaign
                             * @property {google.ads.googleads.v11.enums.CampaignExperimentStatusEnum.CampaignExperimentStatus|null} [status] CampaignExperiment status
                             * @property {string|null} [long_running_operation] CampaignExperiment long_running_operation
                             * @property {string|null} [start_date] CampaignExperiment start_date
                             * @property {string|null} [end_date] CampaignExperiment end_date
                             */
    
                            /**
                             * Constructs a new CampaignExperiment.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignExperiment.
                             * @implements ICampaignExperiment
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignExperiment=} [p] Properties to set
                             */
                            function CampaignExperiment(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignExperiment resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.resource_name = "";
    
                            /**
                             * CampaignExperiment id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.id = null;
    
                            /**
                             * CampaignExperiment campaign_draft.
                             * @member {string|null|undefined} campaign_draft
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.campaign_draft = null;
    
                            /**
                             * CampaignExperiment name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.name = null;
    
                            /**
                             * CampaignExperiment description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.description = null;
    
                            /**
                             * CampaignExperiment traffic_split_percent.
                             * @member {number|null|undefined} traffic_split_percent
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.traffic_split_percent = null;
    
                            /**
                             * CampaignExperiment traffic_split_type.
                             * @member {google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum.CampaignExperimentTrafficSplitType} traffic_split_type
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.traffic_split_type = 0;
    
                            /**
                             * CampaignExperiment experiment_campaign.
                             * @member {string|null|undefined} experiment_campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.experiment_campaign = null;
    
                            /**
                             * CampaignExperiment status.
                             * @member {google.ads.googleads.v11.enums.CampaignExperimentStatusEnum.CampaignExperimentStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.status = 0;
    
                            /**
                             * CampaignExperiment long_running_operation.
                             * @member {string|null|undefined} long_running_operation
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.long_running_operation = null;
    
                            /**
                             * CampaignExperiment start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.start_date = null;
    
                            /**
                             * CampaignExperiment end_date.
                             * @member {string|null|undefined} end_date
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            CampaignExperiment.prototype.end_date = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignExperiment _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _campaign_draft.
                             * @member {"campaign_draft"|undefined} _campaign_draft
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_campaign_draft", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_draft"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _traffic_split_percent.
                             * @member {"traffic_split_percent"|undefined} _traffic_split_percent
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_traffic_split_percent", {
                                get: $util.oneOfGetter($oneOfFields = ["traffic_split_percent"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _experiment_campaign.
                             * @member {"experiment_campaign"|undefined} _experiment_campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_experiment_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["experiment_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _long_running_operation.
                             * @member {"long_running_operation"|undefined} _long_running_operation
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_long_running_operation", {
                                get: $util.oneOfGetter($oneOfFields = ["long_running_operation"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignExperiment _end_date.
                             * @member {"end_date"|undefined} _end_date
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             */
                            Object.defineProperty(CampaignExperiment.prototype, "_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignExperiment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignExperiment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignExperiment} CampaignExperiment instance
                             */
                            CampaignExperiment.create = function create(properties) {
                                return new CampaignExperiment(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExperiment message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignExperiment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignExperiment} m CampaignExperiment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperiment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.traffic_split_type != null && Object.hasOwnProperty.call(m, "traffic_split_type"))
                                    w.uint32(56).int32(m.traffic_split_type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(72).int32(m.status);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(104).int64(m.id);
                                if (m.campaign_draft != null && Object.hasOwnProperty.call(m, "campaign_draft"))
                                    w.uint32(114).string(m.campaign_draft);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(122).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(130).string(m.description);
                                if (m.traffic_split_percent != null && Object.hasOwnProperty.call(m, "traffic_split_percent"))
                                    w.uint32(136).int64(m.traffic_split_percent);
                                if (m.experiment_campaign != null && Object.hasOwnProperty.call(m, "experiment_campaign"))
                                    w.uint32(146).string(m.experiment_campaign);
                                if (m.long_running_operation != null && Object.hasOwnProperty.call(m, "long_running_operation"))
                                    w.uint32(154).string(m.long_running_operation);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(162).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(170).string(m.end_date);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExperiment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignExperiment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignExperiment} message CampaignExperiment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperiment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExperiment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignExperiment} CampaignExperiment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperiment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignExperiment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 13:
                                        m.id = r.int64();
                                        break;
                                    case 14:
                                        m.campaign_draft = r.string();
                                        break;
                                    case 15:
                                        m.name = r.string();
                                        break;
                                    case 16:
                                        m.description = r.string();
                                        break;
                                    case 17:
                                        m.traffic_split_percent = r.int64();
                                        break;
                                    case 7:
                                        m.traffic_split_type = r.int32();
                                        break;
                                    case 18:
                                        m.experiment_campaign = r.string();
                                        break;
                                    case 9:
                                        m.status = r.int32();
                                        break;
                                    case 19:
                                        m.long_running_operation = r.string();
                                        break;
                                    case 20:
                                        m.start_date = r.string();
                                        break;
                                    case 21:
                                        m.end_date = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExperiment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignExperiment} CampaignExperiment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperiment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExperiment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExperiment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    p._campaign_draft = 1;
                                    if (!$util.isString(m.campaign_draft))
                                        return "campaign_draft: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.traffic_split_percent != null && m.hasOwnProperty("traffic_split_percent")) {
                                    p._traffic_split_percent = 1;
                                    if (!$util.isInteger(m.traffic_split_percent) && !(m.traffic_split_percent && $util.isInteger(m.traffic_split_percent.low) && $util.isInteger(m.traffic_split_percent.high)))
                                        return "traffic_split_percent: integer|Long expected";
                                }
                                if (m.traffic_split_type != null && m.hasOwnProperty("traffic_split_type")) {
                                    switch (m.traffic_split_type) {
                                    default:
                                        return "traffic_split_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.experiment_campaign != null && m.hasOwnProperty("experiment_campaign")) {
                                    p._experiment_campaign = 1;
                                    if (!$util.isString(m.experiment_campaign))
                                        return "experiment_campaign: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 8:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 9:
                                    case 7:
                                    case 10:
                                        break;
                                    }
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    p._long_running_operation = 1;
                                    if (!$util.isString(m.long_running_operation))
                                        return "long_running_operation: string expected";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    p._end_date = 1;
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExperiment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignExperiment} CampaignExperiment
                             */
                            CampaignExperiment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignExperiment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignExperiment();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.campaign_draft != null) {
                                    m.campaign_draft = String(d.campaign_draft);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.traffic_split_percent != null) {
                                    if ($util.Long)
                                        (m.traffic_split_percent = $util.Long.fromValue(d.traffic_split_percent)).unsigned = false;
                                    else if (typeof d.traffic_split_percent === "string")
                                        m.traffic_split_percent = parseInt(d.traffic_split_percent, 10);
                                    else if (typeof d.traffic_split_percent === "number")
                                        m.traffic_split_percent = d.traffic_split_percent;
                                    else if (typeof d.traffic_split_percent === "object")
                                        m.traffic_split_percent = new $util.LongBits(d.traffic_split_percent.low >>> 0, d.traffic_split_percent.high >>> 0).toNumber();
                                }
                                switch (d.traffic_split_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.traffic_split_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.traffic_split_type = 1;
                                    break;
                                case "RANDOM_QUERY":
                                case 2:
                                    m.traffic_split_type = 2;
                                    break;
                                case "COOKIE":
                                case 3:
                                    m.traffic_split_type = 3;
                                    break;
                                }
                                if (d.experiment_campaign != null) {
                                    m.experiment_campaign = String(d.experiment_campaign);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "INITIALIZING":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "INITIALIZATION_FAILED":
                                case 8:
                                    m.status = 8;
                                    break;
                                case "ENABLED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "GRADUATED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "REMOVED":
                                case 5:
                                    m.status = 5;
                                    break;
                                case "PROMOTING":
                                case 6:
                                    m.status = 6;
                                    break;
                                case "PROMOTION_FAILED":
                                case 9:
                                    m.status = 9;
                                    break;
                                case "PROMOTED":
                                case 7:
                                    m.status = 7;
                                    break;
                                case "ENDED_MANUALLY":
                                case 10:
                                    m.status = 10;
                                    break;
                                }
                                if (d.long_running_operation != null) {
                                    m.long_running_operation = String(d.long_running_operation);
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignExperiment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignExperiment} m CampaignExperiment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExperiment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.traffic_split_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.traffic_split_type != null && m.hasOwnProperty("traffic_split_type")) {
                                    d.traffic_split_type = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignExperimentTrafficSplitTypeEnum.CampaignExperimentTrafficSplitType[m.traffic_split_type] : m.traffic_split_type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignExperimentStatusEnum.CampaignExperimentStatus[m.status] : m.status;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    d.campaign_draft = m.campaign_draft;
                                    if (o.oneofs)
                                        d._campaign_draft = "campaign_draft";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.traffic_split_percent != null && m.hasOwnProperty("traffic_split_percent")) {
                                    if (typeof m.traffic_split_percent === "number")
                                        d.traffic_split_percent = o.longs === String ? String(m.traffic_split_percent) : m.traffic_split_percent;
                                    else
                                        d.traffic_split_percent = o.longs === String ? $util.Long.prototype.toString.call(m.traffic_split_percent) : o.longs === Number ? new $util.LongBits(m.traffic_split_percent.low >>> 0, m.traffic_split_percent.high >>> 0).toNumber() : m.traffic_split_percent;
                                    if (o.oneofs)
                                        d._traffic_split_percent = "traffic_split_percent";
                                }
                                if (m.experiment_campaign != null && m.hasOwnProperty("experiment_campaign")) {
                                    d.experiment_campaign = m.experiment_campaign;
                                    if (o.oneofs)
                                        d._experiment_campaign = "experiment_campaign";
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    d.long_running_operation = m.long_running_operation;
                                    if (o.oneofs)
                                        d._long_running_operation = "long_running_operation";
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                    if (o.oneofs)
                                        d._end_date = "end_date";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignExperiment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignExperiment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExperiment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignExperiment;
                        })();
    
                        resources.CampaignExtensionSetting = (function() {
    
                            /**
                             * Properties of a CampaignExtensionSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignExtensionSetting
                             * @property {string|null} [resource_name] CampaignExtensionSetting resource_name
                             * @property {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType|null} [extension_type] CampaignExtensionSetting extension_type
                             * @property {string|null} [campaign] CampaignExtensionSetting campaign
                             * @property {Array.<string>|null} [extension_feed_items] CampaignExtensionSetting extension_feed_items
                             * @property {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice|null} [device] CampaignExtensionSetting device
                             */
    
                            /**
                             * Constructs a new CampaignExtensionSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignExtensionSetting.
                             * @implements ICampaignExtensionSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignExtensionSetting=} [p] Properties to set
                             */
                            function CampaignExtensionSetting(p) {
                                this.extension_feed_items = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignExtensionSetting resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @instance
                             */
                            CampaignExtensionSetting.prototype.resource_name = "";
    
                            /**
                             * CampaignExtensionSetting extension_type.
                             * @member {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType} extension_type
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @instance
                             */
                            CampaignExtensionSetting.prototype.extension_type = 0;
    
                            /**
                             * CampaignExtensionSetting campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @instance
                             */
                            CampaignExtensionSetting.prototype.campaign = null;
    
                            /**
                             * CampaignExtensionSetting extension_feed_items.
                             * @member {Array.<string>} extension_feed_items
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @instance
                             */
                            CampaignExtensionSetting.prototype.extension_feed_items = $util.emptyArray;
    
                            /**
                             * CampaignExtensionSetting device.
                             * @member {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice} device
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @instance
                             */
                            CampaignExtensionSetting.prototype.device = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignExtensionSetting _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @instance
                             */
                            Object.defineProperty(CampaignExtensionSetting.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignExtensionSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignExtensionSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignExtensionSetting} CampaignExtensionSetting instance
                             */
                            CampaignExtensionSetting.create = function create(properties) {
                                return new CampaignExtensionSetting(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExtensionSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignExtensionSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignExtensionSetting} m CampaignExtensionSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExtensionSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.extension_type != null && Object.hasOwnProperty.call(m, "extension_type"))
                                    w.uint32(16).int32(m.extension_type);
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    w.uint32(40).int32(m.device);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(50).string(m.campaign);
                                if (m.extension_feed_items != null && m.extension_feed_items.length) {
                                    for (var i = 0; i < m.extension_feed_items.length; ++i)
                                        w.uint32(58).string(m.extension_feed_items[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExtensionSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignExtensionSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignExtensionSetting} message CampaignExtensionSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExtensionSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExtensionSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignExtensionSetting} CampaignExtensionSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExtensionSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignExtensionSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.extension_type = r.int32();
                                        break;
                                    case 6:
                                        m.campaign = r.string();
                                        break;
                                    case 7:
                                        if (!(m.extension_feed_items && m.extension_feed_items.length))
                                            m.extension_feed_items = [];
                                        m.extension_feed_items.push(r.string());
                                        break;
                                    case 5:
                                        m.device = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExtensionSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignExtensionSetting} CampaignExtensionSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExtensionSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExtensionSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExtensionSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    switch (m.extension_type) {
                                    default:
                                        return "extension_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 15:
                                    case 16:
                                        break;
                                    }
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.extension_feed_items != null && m.hasOwnProperty("extension_feed_items")) {
                                    if (!Array.isArray(m.extension_feed_items))
                                        return "extension_feed_items: array expected";
                                    for (var i = 0; i < m.extension_feed_items.length; ++i) {
                                        if (!$util.isString(m.extension_feed_items[i]))
                                            return "extension_feed_items: string[] expected";
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    switch (m.device) {
                                    default:
                                        return "device: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExtensionSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignExtensionSetting} CampaignExtensionSetting
                             */
                            CampaignExtensionSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignExtensionSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignExtensionSetting();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.extension_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.extension_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.extension_type = 1;
                                    break;
                                case "NONE":
                                case 2:
                                    m.extension_type = 2;
                                    break;
                                case "APP":
                                case 3:
                                    m.extension_type = 3;
                                    break;
                                case "CALL":
                                case 4:
                                    m.extension_type = 4;
                                    break;
                                case "CALLOUT":
                                case 5:
                                    m.extension_type = 5;
                                    break;
                                case "MESSAGE":
                                case 6:
                                    m.extension_type = 6;
                                    break;
                                case "PRICE":
                                case 7:
                                    m.extension_type = 7;
                                    break;
                                case "PROMOTION":
                                case 8:
                                    m.extension_type = 8;
                                    break;
                                case "SITELINK":
                                case 10:
                                    m.extension_type = 10;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 11:
                                    m.extension_type = 11;
                                    break;
                                case "LOCATION":
                                case 12:
                                    m.extension_type = 12;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 13:
                                    m.extension_type = 13;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.extension_type = 15;
                                    break;
                                case "IMAGE":
                                case 16:
                                    m.extension_type = 16;
                                    break;
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.extension_feed_items) {
                                    if (!Array.isArray(d.extension_feed_items))
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignExtensionSetting.extension_feed_items: array expected");
                                    m.extension_feed_items = [];
                                    for (var i = 0; i < d.extension_feed_items.length; ++i) {
                                        m.extension_feed_items[i] = String(d.extension_feed_items[i]);
                                    }
                                }
                                switch (d.device) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device = 2;
                                    break;
                                case "DESKTOP":
                                case 3:
                                    m.device = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignExtensionSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignExtensionSetting} m CampaignExtensionSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExtensionSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.extension_feed_items = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.extension_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.device = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    d.extension_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType[m.extension_type] : m.extension_type;
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice[m.device] : m.device;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.extension_feed_items && m.extension_feed_items.length) {
                                    d.extension_feed_items = [];
                                    for (var j = 0; j < m.extension_feed_items.length; ++j) {
                                        d.extension_feed_items[j] = m.extension_feed_items[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignExtensionSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignExtensionSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExtensionSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignExtensionSetting;
                        })();
    
                        resources.CampaignFeed = (function() {
    
                            /**
                             * Properties of a CampaignFeed.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignFeed
                             * @property {string|null} [resource_name] CampaignFeed resource_name
                             * @property {string|null} [feed] CampaignFeed feed
                             * @property {string|null} [campaign] CampaignFeed campaign
                             * @property {Array.<google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType>|null} [placeholder_types] CampaignFeed placeholder_types
                             * @property {google.ads.googleads.v11.common.IMatchingFunction|null} [matching_function] CampaignFeed matching_function
                             * @property {google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus|null} [status] CampaignFeed status
                             */
    
                            /**
                             * Constructs a new CampaignFeed.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignFeed.
                             * @implements ICampaignFeed
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignFeed=} [p] Properties to set
                             */
                            function CampaignFeed(p) {
                                this.placeholder_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignFeed resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            CampaignFeed.prototype.resource_name = "";
    
                            /**
                             * CampaignFeed feed.
                             * @member {string|null|undefined} feed
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            CampaignFeed.prototype.feed = null;
    
                            /**
                             * CampaignFeed campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            CampaignFeed.prototype.campaign = null;
    
                            /**
                             * CampaignFeed placeholder_types.
                             * @member {Array.<google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType>} placeholder_types
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            CampaignFeed.prototype.placeholder_types = $util.emptyArray;
    
                            /**
                             * CampaignFeed matching_function.
                             * @member {google.ads.googleads.v11.common.IMatchingFunction|null|undefined} matching_function
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            CampaignFeed.prototype.matching_function = null;
    
                            /**
                             * CampaignFeed status.
                             * @member {google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            CampaignFeed.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignFeed _feed.
                             * @member {"feed"|undefined} _feed
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            Object.defineProperty(CampaignFeed.prototype, "_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignFeed _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             */
                            Object.defineProperty(CampaignFeed.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignFeed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignFeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignFeed} CampaignFeed instance
                             */
                            CampaignFeed.create = function create(properties) {
                                return new CampaignFeed(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignFeed message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignFeed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignFeed} m CampaignFeed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignFeed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.placeholder_types != null && m.placeholder_types.length) {
                                    w.uint32(34).fork();
                                    for (var i = 0; i < m.placeholder_types.length; ++i)
                                        w.int32(m.placeholder_types[i]);
                                    w.ldelim();
                                }
                                if (m.matching_function != null && Object.hasOwnProperty.call(m, "matching_function"))
                                    $root.google.ads.googleads.v11.common.MatchingFunction.encode(m.matching_function, w.uint32(42).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(48).int32(m.status);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(58).string(m.feed);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(66).string(m.campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignFeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignFeed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignFeed} message CampaignFeed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignFeed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignFeed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignFeed} CampaignFeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignFeed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignFeed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.feed = r.string();
                                        break;
                                    case 8:
                                        m.campaign = r.string();
                                        break;
                                    case 4:
                                        if (!(m.placeholder_types && m.placeholder_types.length))
                                            m.placeholder_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.placeholder_types.push(r.int32());
                                        } else
                                            m.placeholder_types.push(r.int32());
                                        break;
                                    case 5:
                                        m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignFeed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignFeed} CampaignFeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignFeed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignFeed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignFeed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    p._feed = 1;
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.placeholder_types != null && m.hasOwnProperty("placeholder_types")) {
                                    if (!Array.isArray(m.placeholder_types))
                                        return "placeholder_types: array expected";
                                    for (var i = 0; i < m.placeholder_types.length; ++i) {
                                        switch (m.placeholder_types[i]) {
                                        default:
                                            return "placeholder_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 17:
                                        case 18:
                                        case 19:
                                        case 20:
                                        case 21:
                                            break;
                                        }
                                    }
                                }
                                if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MatchingFunction.verify(m.matching_function);
                                        if (e)
                                            return "matching_function." + e;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignFeed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignFeed} CampaignFeed
                             */
                            CampaignFeed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignFeed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignFeed();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.placeholder_types) {
                                    if (!Array.isArray(d.placeholder_types))
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignFeed.placeholder_types: array expected");
                                    m.placeholder_types = [];
                                    for (var i = 0; i < d.placeholder_types.length; ++i) {
                                        switch (d.placeholder_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.placeholder_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.placeholder_types[i] = 1;
                                            break;
                                        case "SITELINK":
                                        case 2:
                                            m.placeholder_types[i] = 2;
                                            break;
                                        case "CALL":
                                        case 3:
                                            m.placeholder_types[i] = 3;
                                            break;
                                        case "APP":
                                        case 4:
                                            m.placeholder_types[i] = 4;
                                            break;
                                        case "LOCATION":
                                        case 5:
                                            m.placeholder_types[i] = 5;
                                            break;
                                        case "AFFILIATE_LOCATION":
                                        case 6:
                                            m.placeholder_types[i] = 6;
                                            break;
                                        case "CALLOUT":
                                        case 7:
                                            m.placeholder_types[i] = 7;
                                            break;
                                        case "STRUCTURED_SNIPPET":
                                        case 8:
                                            m.placeholder_types[i] = 8;
                                            break;
                                        case "MESSAGE":
                                        case 9:
                                            m.placeholder_types[i] = 9;
                                            break;
                                        case "PRICE":
                                        case 10:
                                            m.placeholder_types[i] = 10;
                                            break;
                                        case "PROMOTION":
                                        case 11:
                                            m.placeholder_types[i] = 11;
                                            break;
                                        case "AD_CUSTOMIZER":
                                        case 12:
                                            m.placeholder_types[i] = 12;
                                            break;
                                        case "DYNAMIC_EDUCATION":
                                        case 13:
                                            m.placeholder_types[i] = 13;
                                            break;
                                        case "DYNAMIC_FLIGHT":
                                        case 14:
                                            m.placeholder_types[i] = 14;
                                            break;
                                        case "DYNAMIC_CUSTOM":
                                        case 15:
                                            m.placeholder_types[i] = 15;
                                            break;
                                        case "DYNAMIC_HOTEL":
                                        case 16:
                                            m.placeholder_types[i] = 16;
                                            break;
                                        case "DYNAMIC_REAL_ESTATE":
                                        case 17:
                                            m.placeholder_types[i] = 17;
                                            break;
                                        case "DYNAMIC_TRAVEL":
                                        case 18:
                                            m.placeholder_types[i] = 18;
                                            break;
                                        case "DYNAMIC_LOCAL":
                                        case 19:
                                            m.placeholder_types[i] = 19;
                                            break;
                                        case "DYNAMIC_JOB":
                                        case 20:
                                            m.placeholder_types[i] = 20;
                                            break;
                                        case "IMAGE":
                                        case 21:
                                            m.placeholder_types[i] = 21;
                                            break;
                                        }
                                    }
                                }
                                if (d.matching_function != null) {
                                    if (typeof d.matching_function !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignFeed.matching_function: object expected");
                                    m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.fromObject(d.matching_function);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignFeed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignFeed} m CampaignFeed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignFeed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.placeholder_types = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.matching_function = null;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.placeholder_types && m.placeholder_types.length) {
                                    d.placeholder_types = [];
                                    for (var j = 0; j < m.placeholder_types.length; ++j) {
                                        d.placeholder_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType[m.placeholder_types[j]] : m.placeholder_types[j];
                                    }
                                }
                                if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                    d.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.toObject(m.matching_function, o);
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus[m.status] : m.status;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                    if (o.oneofs)
                                        d._feed = "feed";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignFeed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignFeed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignFeed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignFeed;
                        })();
    
                        resources.CampaignGroup = (function() {
    
                            /**
                             * Properties of a CampaignGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignGroup
                             * @property {string|null} [resource_name] CampaignGroup resource_name
                             * @property {number|null} [id] CampaignGroup id
                             * @property {string|null} [name] CampaignGroup name
                             * @property {google.ads.googleads.v11.enums.CampaignGroupStatusEnum.CampaignGroupStatus|null} [status] CampaignGroup status
                             */
    
                            /**
                             * Constructs a new CampaignGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignGroup.
                             * @implements ICampaignGroup
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignGroup=} [p] Properties to set
                             */
                            function CampaignGroup(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignGroup resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @instance
                             */
                            CampaignGroup.prototype.resource_name = "";
    
                            /**
                             * CampaignGroup id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @instance
                             */
                            CampaignGroup.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CampaignGroup name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @instance
                             */
                            CampaignGroup.prototype.name = "";
    
                            /**
                             * CampaignGroup status.
                             * @member {google.ads.googleads.v11.enums.CampaignGroupStatusEnum.CampaignGroupStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @instance
                             */
                            CampaignGroup.prototype.status = 0;
    
                            /**
                             * Creates a new CampaignGroup instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignGroup=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignGroup} CampaignGroup instance
                             */
                            CampaignGroup.create = function create(properties) {
                                return new CampaignGroup(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignGroup message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignGroup.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignGroup} m CampaignGroup message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignGroup.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(24).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(34).string(m.name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignGroup message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignGroup.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignGroup} message CampaignGroup message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignGroup.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignGroup message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignGroup} CampaignGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignGroup.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignGroup();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.id = r.int64();
                                        break;
                                    case 4:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignGroup message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignGroup} CampaignGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignGroup.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignGroup message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignGroup.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignGroup message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignGroup} CampaignGroup
                             */
                            CampaignGroup.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignGroup)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignGroup();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignGroup message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignGroup} m CampaignGroup
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignGroup.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignGroupStatusEnum.CampaignGroupStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignGroup to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignGroup
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignGroup.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignGroup;
                        })();
    
                        resources.CampaignLabel = (function() {
    
                            /**
                             * Properties of a CampaignLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignLabel
                             * @property {string|null} [resource_name] CampaignLabel resource_name
                             * @property {string|null} [campaign] CampaignLabel campaign
                             * @property {string|null} [label] CampaignLabel label
                             */
    
                            /**
                             * Constructs a new CampaignLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignLabel.
                             * @implements ICampaignLabel
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignLabel=} [p] Properties to set
                             */
                            function CampaignLabel(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignLabel resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @instance
                             */
                            CampaignLabel.prototype.resource_name = "";
    
                            /**
                             * CampaignLabel campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @instance
                             */
                            CampaignLabel.prototype.campaign = null;
    
                            /**
                             * CampaignLabel label.
                             * @member {string|null|undefined} label
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @instance
                             */
                            CampaignLabel.prototype.label = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignLabel _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @instance
                             */
                            Object.defineProperty(CampaignLabel.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignLabel _label.
                             * @member {"label"|undefined} _label
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @instance
                             */
                            Object.defineProperty(CampaignLabel.prototype, "_label", {
                                get: $util.oneOfGetter($oneOfFields = ["label"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignLabel instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignLabel=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignLabel} CampaignLabel instance
                             */
                            CampaignLabel.create = function create(properties) {
                                return new CampaignLabel(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignLabel message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignLabel.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignLabel} m CampaignLabel message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignLabel.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(34).string(m.campaign);
                                if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                                    w.uint32(42).string(m.label);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignLabel message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignLabel.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignLabel} message CampaignLabel message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignLabel.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignLabel message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignLabel} CampaignLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignLabel.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignLabel();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.campaign = r.string();
                                        break;
                                    case 5:
                                        m.label = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignLabel message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignLabel} CampaignLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignLabel.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignLabel message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignLabel.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    p._label = 1;
                                    if (!$util.isString(m.label))
                                        return "label: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignLabel message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignLabel} CampaignLabel
                             */
                            CampaignLabel.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignLabel)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignLabel();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.label != null) {
                                    m.label = String(d.label);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignLabel message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignLabel} m CampaignLabel
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignLabel.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    d.label = m.label;
                                    if (o.oneofs)
                                        d._label = "label";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignLabel to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignLabel
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignLabel.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignLabel;
                        })();
    
                        resources.CampaignSharedSet = (function() {
    
                            /**
                             * Properties of a CampaignSharedSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignSharedSet
                             * @property {string|null} [resource_name] CampaignSharedSet resource_name
                             * @property {string|null} [campaign] CampaignSharedSet campaign
                             * @property {string|null} [shared_set] CampaignSharedSet shared_set
                             * @property {google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum.CampaignSharedSetStatus|null} [status] CampaignSharedSet status
                             */
    
                            /**
                             * Constructs a new CampaignSharedSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignSharedSet.
                             * @implements ICampaignSharedSet
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignSharedSet=} [p] Properties to set
                             */
                            function CampaignSharedSet(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignSharedSet resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @instance
                             */
                            CampaignSharedSet.prototype.resource_name = "";
    
                            /**
                             * CampaignSharedSet campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @instance
                             */
                            CampaignSharedSet.prototype.campaign = null;
    
                            /**
                             * CampaignSharedSet shared_set.
                             * @member {string|null|undefined} shared_set
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @instance
                             */
                            CampaignSharedSet.prototype.shared_set = null;
    
                            /**
                             * CampaignSharedSet status.
                             * @member {google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum.CampaignSharedSetStatus} status
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @instance
                             */
                            CampaignSharedSet.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignSharedSet _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @instance
                             */
                            Object.defineProperty(CampaignSharedSet.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CampaignSharedSet _shared_set.
                             * @member {"shared_set"|undefined} _shared_set
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @instance
                             */
                            Object.defineProperty(CampaignSharedSet.prototype, "_shared_set", {
                                get: $util.oneOfGetter($oneOfFields = ["shared_set"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignSharedSet instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignSharedSet=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignSharedSet} CampaignSharedSet instance
                             */
                            CampaignSharedSet.create = function create(properties) {
                                return new CampaignSharedSet(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSet message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignSharedSet.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignSharedSet} m CampaignSharedSet message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSet.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(16).int32(m.status);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(42).string(m.campaign);
                                if (m.shared_set != null && Object.hasOwnProperty.call(m, "shared_set"))
                                    w.uint32(50).string(m.shared_set);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSet message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignSharedSet.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignSharedSet} message CampaignSharedSet message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSet.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignSharedSet message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignSharedSet} CampaignSharedSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSet.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignSharedSet();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 5:
                                        m.campaign = r.string();
                                        break;
                                    case 6:
                                        m.shared_set = r.string();
                                        break;
                                    case 2:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignSharedSet message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignSharedSet} CampaignSharedSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSet.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignSharedSet message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignSharedSet.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    p._shared_set = 1;
                                    if (!$util.isString(m.shared_set))
                                        return "shared_set: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignSharedSet message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignSharedSet} CampaignSharedSet
                             */
                            CampaignSharedSet.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignSharedSet)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignSharedSet();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.shared_set != null) {
                                    m.shared_set = String(d.shared_set);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignSharedSet message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignSharedSet} m CampaignSharedSet
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignSharedSet.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CampaignSharedSetStatusEnum.CampaignSharedSetStatus[m.status] : m.status;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    d.shared_set = m.shared_set;
                                    if (o.oneofs)
                                        d._shared_set = "shared_set";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignSharedSet to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignSharedSet
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignSharedSet.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignSharedSet;
                        })();
    
                        resources.CampaignSimulation = (function() {
    
                            /**
                             * Properties of a CampaignSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICampaignSimulation
                             * @property {string|null} [resource_name] CampaignSimulation resource_name
                             * @property {number|null} [campaign_id] CampaignSimulation campaign_id
                             * @property {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType|null} [type] CampaignSimulation type
                             * @property {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod|null} [modification_method] CampaignSimulation modification_method
                             * @property {string|null} [start_date] CampaignSimulation start_date
                             * @property {string|null} [end_date] CampaignSimulation end_date
                             * @property {google.ads.googleads.v11.common.ICpcBidSimulationPointList|null} [cpc_bid_point_list] CampaignSimulation cpc_bid_point_list
                             * @property {google.ads.googleads.v11.common.ITargetCpaSimulationPointList|null} [target_cpa_point_list] CampaignSimulation target_cpa_point_list
                             * @property {google.ads.googleads.v11.common.ITargetRoasSimulationPointList|null} [target_roas_point_list] CampaignSimulation target_roas_point_list
                             * @property {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPointList|null} [target_impression_share_point_list] CampaignSimulation target_impression_share_point_list
                             * @property {google.ads.googleads.v11.common.IBudgetSimulationPointList|null} [budget_point_list] CampaignSimulation budget_point_list
                             */
    
                            /**
                             * Constructs a new CampaignSimulation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CampaignSimulation.
                             * @implements ICampaignSimulation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICampaignSimulation=} [p] Properties to set
                             */
                            function CampaignSimulation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignSimulation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.resource_name = "";
    
                            /**
                             * CampaignSimulation campaign_id.
                             * @member {number} campaign_id
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.campaign_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CampaignSimulation type.
                             * @member {google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType} type
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.type = 0;
    
                            /**
                             * CampaignSimulation modification_method.
                             * @member {google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod} modification_method
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.modification_method = 0;
    
                            /**
                             * CampaignSimulation start_date.
                             * @member {string} start_date
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.start_date = "";
    
                            /**
                             * CampaignSimulation end_date.
                             * @member {string} end_date
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.end_date = "";
    
                            /**
                             * CampaignSimulation cpc_bid_point_list.
                             * @member {google.ads.googleads.v11.common.ICpcBidSimulationPointList|null|undefined} cpc_bid_point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.cpc_bid_point_list = null;
    
                            /**
                             * CampaignSimulation target_cpa_point_list.
                             * @member {google.ads.googleads.v11.common.ITargetCpaSimulationPointList|null|undefined} target_cpa_point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.target_cpa_point_list = null;
    
                            /**
                             * CampaignSimulation target_roas_point_list.
                             * @member {google.ads.googleads.v11.common.ITargetRoasSimulationPointList|null|undefined} target_roas_point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.target_roas_point_list = null;
    
                            /**
                             * CampaignSimulation target_impression_share_point_list.
                             * @member {google.ads.googleads.v11.common.ITargetImpressionShareSimulationPointList|null|undefined} target_impression_share_point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.target_impression_share_point_list = null;
    
                            /**
                             * CampaignSimulation budget_point_list.
                             * @member {google.ads.googleads.v11.common.IBudgetSimulationPointList|null|undefined} budget_point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            CampaignSimulation.prototype.budget_point_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignSimulation point_list.
                             * @member {"cpc_bid_point_list"|"target_cpa_point_list"|"target_roas_point_list"|"target_impression_share_point_list"|"budget_point_list"|undefined} point_list
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             */
                            Object.defineProperty(CampaignSimulation.prototype, "point_list", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_point_list", "target_cpa_point_list", "target_roas_point_list", "target_impression_share_point_list", "budget_point_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignSimulation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignSimulation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CampaignSimulation} CampaignSimulation instance
                             */
                            CampaignSimulation.create = function create(properties) {
                                return new CampaignSimulation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignSimulation message. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignSimulation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignSimulation} m CampaignSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSimulation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_id != null && Object.hasOwnProperty.call(m, "campaign_id"))
                                    w.uint32(16).int64(m.campaign_id);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.modification_method != null && Object.hasOwnProperty.call(m, "modification_method"))
                                    w.uint32(32).int32(m.modification_method);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(42).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(50).string(m.end_date);
                                if (m.cpc_bid_point_list != null && Object.hasOwnProperty.call(m, "cpc_bid_point_list"))
                                    $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.encode(m.cpc_bid_point_list, w.uint32(58).fork()).ldelim();
                                if (m.target_cpa_point_list != null && Object.hasOwnProperty.call(m, "target_cpa_point_list"))
                                    $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.encode(m.target_cpa_point_list, w.uint32(66).fork()).ldelim();
                                if (m.target_roas_point_list != null && Object.hasOwnProperty.call(m, "target_roas_point_list"))
                                    $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.encode(m.target_roas_point_list, w.uint32(74).fork()).ldelim();
                                if (m.target_impression_share_point_list != null && Object.hasOwnProperty.call(m, "target_impression_share_point_list"))
                                    $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.encode(m.target_impression_share_point_list, w.uint32(82).fork()).ldelim();
                                if (m.budget_point_list != null && Object.hasOwnProperty.call(m, "budget_point_list"))
                                    $root.google.ads.googleads.v11.common.BudgetSimulationPointList.encode(m.budget_point_list, w.uint32(90).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignSimulation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CampaignSimulation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICampaignSimulation} message CampaignSimulation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSimulation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignSimulation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CampaignSimulation} CampaignSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSimulation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CampaignSimulation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_id = r.int64();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 4:
                                        m.modification_method = r.int32();
                                        break;
                                    case 5:
                                        m.start_date = r.string();
                                        break;
                                    case 6:
                                        m.end_date = r.string();
                                        break;
                                    case 7:
                                        m.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.target_impression_share_point_list = $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.budget_point_list = $root.google.ads.googleads.v11.common.BudgetSimulationPointList.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignSimulation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CampaignSimulation} CampaignSimulation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSimulation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignSimulation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignSimulation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_id != null && m.hasOwnProperty("campaign_id")) {
                                    if (!$util.isInteger(m.campaign_id) && !(m.campaign_id && $util.isInteger(m.campaign_id.low) && $util.isInteger(m.campaign_id.high)))
                                        return "campaign_id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    switch (m.modification_method) {
                                    default:
                                        return "modification_method: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.cpc_bid_point_list != null && m.hasOwnProperty("cpc_bid_point_list")) {
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.verify(m.cpc_bid_point_list);
                                        if (e)
                                            return "cpc_bid_point_list." + e;
                                    }
                                }
                                if (m.target_cpa_point_list != null && m.hasOwnProperty("target_cpa_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.verify(m.target_cpa_point_list);
                                        if (e)
                                            return "target_cpa_point_list." + e;
                                    }
                                }
                                if (m.target_roas_point_list != null && m.hasOwnProperty("target_roas_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.verify(m.target_roas_point_list);
                                        if (e)
                                            return "target_roas_point_list." + e;
                                    }
                                }
                                if (m.target_impression_share_point_list != null && m.hasOwnProperty("target_impression_share_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.verify(m.target_impression_share_point_list);
                                        if (e)
                                            return "target_impression_share_point_list." + e;
                                    }
                                }
                                if (m.budget_point_list != null && m.hasOwnProperty("budget_point_list")) {
                                    if (p.point_list === 1)
                                        return "point_list: multiple values";
                                    p.point_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.BudgetSimulationPointList.verify(m.budget_point_list);
                                        if (e)
                                            return "budget_point_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignSimulation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CampaignSimulation} CampaignSimulation
                             */
                            CampaignSimulation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CampaignSimulation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CampaignSimulation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_id != null) {
                                    if ($util.Long)
                                        (m.campaign_id = $util.Long.fromValue(d.campaign_id)).unsigned = false;
                                    else if (typeof d.campaign_id === "string")
                                        m.campaign_id = parseInt(d.campaign_id, 10);
                                    else if (typeof d.campaign_id === "number")
                                        m.campaign_id = d.campaign_id;
                                    else if (typeof d.campaign_id === "object")
                                        m.campaign_id = new $util.LongBits(d.campaign_id.low >>> 0, d.campaign_id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CPC_BID":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "CPV_BID":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "TARGET_CPA":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "BID_MODIFIER":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "TARGET_ROAS":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "PERCENT_CPC_BID":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "TARGET_IMPRESSION_SHARE":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "BUDGET":
                                case 9:
                                    m.type = 9;
                                    break;
                                }
                                switch (d.modification_method) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.modification_method = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.modification_method = 1;
                                    break;
                                case "UNIFORM":
                                case 2:
                                    m.modification_method = 2;
                                    break;
                                case "DEFAULT":
                                case 3:
                                    m.modification_method = 3;
                                    break;
                                case "SCALING":
                                case 4:
                                    m.modification_method = 4;
                                    break;
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.cpc_bid_point_list != null) {
                                    if (typeof d.cpc_bid_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignSimulation.cpc_bid_point_list: object expected");
                                    m.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.fromObject(d.cpc_bid_point_list);
                                }
                                if (d.target_cpa_point_list != null) {
                                    if (typeof d.target_cpa_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignSimulation.target_cpa_point_list: object expected");
                                    m.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.fromObject(d.target_cpa_point_list);
                                }
                                if (d.target_roas_point_list != null) {
                                    if (typeof d.target_roas_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignSimulation.target_roas_point_list: object expected");
                                    m.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.fromObject(d.target_roas_point_list);
                                }
                                if (d.target_impression_share_point_list != null) {
                                    if (typeof d.target_impression_share_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignSimulation.target_impression_share_point_list: object expected");
                                    m.target_impression_share_point_list = $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.fromObject(d.target_impression_share_point_list);
                                }
                                if (d.budget_point_list != null) {
                                    if (typeof d.budget_point_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CampaignSimulation.budget_point_list: object expected");
                                    m.budget_point_list = $root.google.ads.googleads.v11.common.BudgetSimulationPointList.fromObject(d.budget_point_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignSimulation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @static
                             * @param {google.ads.googleads.v11.resources.CampaignSimulation} m CampaignSimulation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignSimulation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.campaign_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.campaign_id = o.longs === String ? "0" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.modification_method = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.start_date = "";
                                    d.end_date = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_id != null && m.hasOwnProperty("campaign_id")) {
                                    if (typeof m.campaign_id === "number")
                                        d.campaign_id = o.longs === String ? String(m.campaign_id) : m.campaign_id;
                                    else
                                        d.campaign_id = o.longs === String ? $util.Long.prototype.toString.call(m.campaign_id) : o.longs === Number ? new $util.LongBits(m.campaign_id.low >>> 0, m.campaign_id.high >>> 0).toNumber() : m.campaign_id;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationTypeEnum.SimulationType[m.type] : m.type;
                                }
                                if (m.modification_method != null && m.hasOwnProperty("modification_method")) {
                                    d.modification_method = o.enums === String ? $root.google.ads.googleads.v11.enums.SimulationModificationMethodEnum.SimulationModificationMethod[m.modification_method] : m.modification_method;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                }
                                if (m.cpc_bid_point_list != null && m.hasOwnProperty("cpc_bid_point_list")) {
                                    d.cpc_bid_point_list = $root.google.ads.googleads.v11.common.CpcBidSimulationPointList.toObject(m.cpc_bid_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "cpc_bid_point_list";
                                }
                                if (m.target_cpa_point_list != null && m.hasOwnProperty("target_cpa_point_list")) {
                                    d.target_cpa_point_list = $root.google.ads.googleads.v11.common.TargetCpaSimulationPointList.toObject(m.target_cpa_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "target_cpa_point_list";
                                }
                                if (m.target_roas_point_list != null && m.hasOwnProperty("target_roas_point_list")) {
                                    d.target_roas_point_list = $root.google.ads.googleads.v11.common.TargetRoasSimulationPointList.toObject(m.target_roas_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "target_roas_point_list";
                                }
                                if (m.target_impression_share_point_list != null && m.hasOwnProperty("target_impression_share_point_list")) {
                                    d.target_impression_share_point_list = $root.google.ads.googleads.v11.common.TargetImpressionShareSimulationPointList.toObject(m.target_impression_share_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "target_impression_share_point_list";
                                }
                                if (m.budget_point_list != null && m.hasOwnProperty("budget_point_list")) {
                                    d.budget_point_list = $root.google.ads.googleads.v11.common.BudgetSimulationPointList.toObject(m.budget_point_list, o);
                                    if (o.oneofs)
                                        d.point_list = "budget_point_list";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignSimulation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CampaignSimulation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignSimulation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignSimulation;
                        })();
    
                        resources.CarrierConstant = (function() {
    
                            /**
                             * Properties of a CarrierConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICarrierConstant
                             * @property {string|null} [resource_name] CarrierConstant resource_name
                             * @property {number|null} [id] CarrierConstant id
                             * @property {string|null} [name] CarrierConstant name
                             * @property {string|null} [country_code] CarrierConstant country_code
                             */
    
                            /**
                             * Constructs a new CarrierConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CarrierConstant.
                             * @implements ICarrierConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICarrierConstant=} [p] Properties to set
                             */
                            function CarrierConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CarrierConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             */
                            CarrierConstant.prototype.resource_name = "";
    
                            /**
                             * CarrierConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             */
                            CarrierConstant.prototype.id = null;
    
                            /**
                             * CarrierConstant name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             */
                            CarrierConstant.prototype.name = null;
    
                            /**
                             * CarrierConstant country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             */
                            CarrierConstant.prototype.country_code = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CarrierConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             */
                            Object.defineProperty(CarrierConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CarrierConstant _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             */
                            Object.defineProperty(CarrierConstant.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CarrierConstant _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             */
                            Object.defineProperty(CarrierConstant.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CarrierConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICarrierConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CarrierConstant} CarrierConstant instance
                             */
                            CarrierConstant.create = function create(properties) {
                                return new CarrierConstant(properties);
                            };
    
                            /**
                             * Encodes the specified CarrierConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.CarrierConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICarrierConstant} m CarrierConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CarrierConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(40).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(50).string(m.name);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(58).string(m.country_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CarrierConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CarrierConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICarrierConstant} message CarrierConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CarrierConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CarrierConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CarrierConstant} CarrierConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CarrierConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CarrierConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 5:
                                        m.id = r.int64();
                                        break;
                                    case 6:
                                        m.name = r.string();
                                        break;
                                    case 7:
                                        m.country_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CarrierConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CarrierConstant} CarrierConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CarrierConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CarrierConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CarrierConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CarrierConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CarrierConstant} CarrierConstant
                             */
                            CarrierConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CarrierConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CarrierConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CarrierConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.CarrierConstant} m CarrierConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CarrierConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CarrierConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CarrierConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CarrierConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CarrierConstant;
                        })();
    
                        resources.ChangeEvent = (function() {
    
                            /**
                             * Properties of a ChangeEvent.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IChangeEvent
                             * @property {string|null} [resource_name] ChangeEvent resource_name
                             * @property {string|null} [change_date_time] ChangeEvent change_date_time
                             * @property {google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum.ChangeEventResourceType|null} [change_resource_type] ChangeEvent change_resource_type
                             * @property {string|null} [change_resource_name] ChangeEvent change_resource_name
                             * @property {google.ads.googleads.v11.enums.ChangeClientTypeEnum.ChangeClientType|null} [client_type] ChangeEvent client_type
                             * @property {string|null} [user_email] ChangeEvent user_email
                             * @property {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource|null} [old_resource] ChangeEvent old_resource
                             * @property {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource|null} [new_resource] ChangeEvent new_resource
                             * @property {google.ads.googleads.v11.enums.ResourceChangeOperationEnum.ResourceChangeOperation|null} [resource_change_operation] ChangeEvent resource_change_operation
                             * @property {google.protobuf.IFieldMask|null} [changed_fields] ChangeEvent changed_fields
                             * @property {string|null} [campaign] ChangeEvent campaign
                             * @property {string|null} [ad_group] ChangeEvent ad_group
                             * @property {string|null} [feed] ChangeEvent feed
                             * @property {string|null} [feed_item] ChangeEvent feed_item
                             * @property {string|null} [asset] ChangeEvent asset
                             */
    
                            /**
                             * Constructs a new ChangeEvent.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ChangeEvent.
                             * @implements IChangeEvent
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IChangeEvent=} [p] Properties to set
                             */
                            function ChangeEvent(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ChangeEvent resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.resource_name = "";
    
                            /**
                             * ChangeEvent change_date_time.
                             * @member {string} change_date_time
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.change_date_time = "";
    
                            /**
                             * ChangeEvent change_resource_type.
                             * @member {google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum.ChangeEventResourceType} change_resource_type
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.change_resource_type = 0;
    
                            /**
                             * ChangeEvent change_resource_name.
                             * @member {string} change_resource_name
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.change_resource_name = "";
    
                            /**
                             * ChangeEvent client_type.
                             * @member {google.ads.googleads.v11.enums.ChangeClientTypeEnum.ChangeClientType} client_type
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.client_type = 0;
    
                            /**
                             * ChangeEvent user_email.
                             * @member {string} user_email
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.user_email = "";
    
                            /**
                             * ChangeEvent old_resource.
                             * @member {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource|null|undefined} old_resource
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.old_resource = null;
    
                            /**
                             * ChangeEvent new_resource.
                             * @member {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource|null|undefined} new_resource
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.new_resource = null;
    
                            /**
                             * ChangeEvent resource_change_operation.
                             * @member {google.ads.googleads.v11.enums.ResourceChangeOperationEnum.ResourceChangeOperation} resource_change_operation
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.resource_change_operation = 0;
    
                            /**
                             * ChangeEvent changed_fields.
                             * @member {google.protobuf.IFieldMask|null|undefined} changed_fields
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.changed_fields = null;
    
                            /**
                             * ChangeEvent campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.campaign = "";
    
                            /**
                             * ChangeEvent ad_group.
                             * @member {string} ad_group
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.ad_group = "";
    
                            /**
                             * ChangeEvent feed.
                             * @member {string} feed
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.feed = "";
    
                            /**
                             * ChangeEvent feed_item.
                             * @member {string} feed_item
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.feed_item = "";
    
                            /**
                             * ChangeEvent asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             */
                            ChangeEvent.prototype.asset = "";
    
                            /**
                             * Creates a new ChangeEvent instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.IChangeEvent=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ChangeEvent} ChangeEvent instance
                             */
                            ChangeEvent.create = function create(properties) {
                                return new ChangeEvent(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeEvent message. Does not implicitly {@link google.ads.googleads.v11.resources.ChangeEvent.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.IChangeEvent} m ChangeEvent message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeEvent.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.change_date_time != null && Object.hasOwnProperty.call(m, "change_date_time"))
                                    w.uint32(18).string(m.change_date_time);
                                if (m.change_resource_type != null && Object.hasOwnProperty.call(m, "change_resource_type"))
                                    w.uint32(24).int32(m.change_resource_type);
                                if (m.change_resource_name != null && Object.hasOwnProperty.call(m, "change_resource_name"))
                                    w.uint32(34).string(m.change_resource_name);
                                if (m.client_type != null && Object.hasOwnProperty.call(m, "client_type"))
                                    w.uint32(40).int32(m.client_type);
                                if (m.user_email != null && Object.hasOwnProperty.call(m, "user_email"))
                                    w.uint32(50).string(m.user_email);
                                if (m.old_resource != null && Object.hasOwnProperty.call(m, "old_resource"))
                                    $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.encode(m.old_resource, w.uint32(58).fork()).ldelim();
                                if (m.new_resource != null && Object.hasOwnProperty.call(m, "new_resource"))
                                    $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.encode(m.new_resource, w.uint32(66).fork()).ldelim();
                                if (m.resource_change_operation != null && Object.hasOwnProperty.call(m, "resource_change_operation"))
                                    w.uint32(72).int32(m.resource_change_operation);
                                if (m.changed_fields != null && Object.hasOwnProperty.call(m, "changed_fields"))
                                    $root.google.protobuf.FieldMask.encode(m.changed_fields, w.uint32(82).fork()).ldelim();
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(90).string(m.campaign);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(98).string(m.ad_group);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(106).string(m.feed);
                                if (m.feed_item != null && Object.hasOwnProperty.call(m, "feed_item"))
                                    w.uint32(114).string(m.feed_item);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(162).string(m.asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeEvent message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ChangeEvent.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.IChangeEvent} message ChangeEvent message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeEvent.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeEvent message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ChangeEvent} ChangeEvent
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeEvent.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ChangeEvent();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.change_date_time = r.string();
                                        break;
                                    case 3:
                                        m.change_resource_type = r.int32();
                                        break;
                                    case 4:
                                        m.change_resource_name = r.string();
                                        break;
                                    case 5:
                                        m.client_type = r.int32();
                                        break;
                                    case 6:
                                        m.user_email = r.string();
                                        break;
                                    case 7:
                                        m.old_resource = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.new_resource = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.resource_change_operation = r.int32();
                                        break;
                                    case 10:
                                        m.changed_fields = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.campaign = r.string();
                                        break;
                                    case 12:
                                        m.ad_group = r.string();
                                        break;
                                    case 13:
                                        m.feed = r.string();
                                        break;
                                    case 14:
                                        m.feed_item = r.string();
                                        break;
                                    case 20:
                                        m.asset = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeEvent message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ChangeEvent} ChangeEvent
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeEvent.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeEvent message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeEvent.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.change_date_time != null && m.hasOwnProperty("change_date_time")) {
                                    if (!$util.isString(m.change_date_time))
                                        return "change_date_time: string expected";
                                }
                                if (m.change_resource_type != null && m.hasOwnProperty("change_resource_type")) {
                                    switch (m.change_resource_type) {
                                    default:
                                        return "change_resource_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                        break;
                                    }
                                }
                                if (m.change_resource_name != null && m.hasOwnProperty("change_resource_name")) {
                                    if (!$util.isString(m.change_resource_name))
                                        return "change_resource_name: string expected";
                                }
                                if (m.client_type != null && m.hasOwnProperty("client_type")) {
                                    switch (m.client_type) {
                                    default:
                                        return "client_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.user_email != null && m.hasOwnProperty("user_email")) {
                                    if (!$util.isString(m.user_email))
                                        return "user_email: string expected";
                                }
                                if (m.old_resource != null && m.hasOwnProperty("old_resource")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.verify(m.old_resource);
                                        if (e)
                                            return "old_resource." + e;
                                    }
                                }
                                if (m.new_resource != null && m.hasOwnProperty("new_resource")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.verify(m.new_resource);
                                        if (e)
                                            return "new_resource." + e;
                                    }
                                }
                                if (m.resource_change_operation != null && m.hasOwnProperty("resource_change_operation")) {
                                    switch (m.resource_change_operation) {
                                    default:
                                        return "resource_change_operation: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.changed_fields != null && m.hasOwnProperty("changed_fields")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.changed_fields);
                                        if (e)
                                            return "changed_fields." + e;
                                    }
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    if (!$util.isString(m.feed_item))
                                        return "feed_item: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ChangeEvent message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ChangeEvent} ChangeEvent
                             */
                            ChangeEvent.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ChangeEvent)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ChangeEvent();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.change_date_time != null) {
                                    m.change_date_time = String(d.change_date_time);
                                }
                                switch (d.change_resource_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.change_resource_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.change_resource_type = 1;
                                    break;
                                case "AD":
                                case 2:
                                    m.change_resource_type = 2;
                                    break;
                                case "AD_GROUP":
                                case 3:
                                    m.change_resource_type = 3;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 4:
                                    m.change_resource_type = 4;
                                    break;
                                case "CAMPAIGN":
                                case 5:
                                    m.change_resource_type = 5;
                                    break;
                                case "CAMPAIGN_BUDGET":
                                case 6:
                                    m.change_resource_type = 6;
                                    break;
                                case "AD_GROUP_BID_MODIFIER":
                                case 7:
                                    m.change_resource_type = 7;
                                    break;
                                case "CAMPAIGN_CRITERION":
                                case 8:
                                    m.change_resource_type = 8;
                                    break;
                                case "FEED":
                                case 9:
                                    m.change_resource_type = 9;
                                    break;
                                case "FEED_ITEM":
                                case 10:
                                    m.change_resource_type = 10;
                                    break;
                                case "CAMPAIGN_FEED":
                                case 11:
                                    m.change_resource_type = 11;
                                    break;
                                case "AD_GROUP_FEED":
                                case 12:
                                    m.change_resource_type = 12;
                                    break;
                                case "AD_GROUP_AD":
                                case 13:
                                    m.change_resource_type = 13;
                                    break;
                                case "ASSET":
                                case 14:
                                    m.change_resource_type = 14;
                                    break;
                                case "CUSTOMER_ASSET":
                                case 15:
                                    m.change_resource_type = 15;
                                    break;
                                case "CAMPAIGN_ASSET":
                                case 16:
                                    m.change_resource_type = 16;
                                    break;
                                case "AD_GROUP_ASSET":
                                case 17:
                                    m.change_resource_type = 17;
                                    break;
                                case "ASSET_SET":
                                case 18:
                                    m.change_resource_type = 18;
                                    break;
                                case "ASSET_SET_ASSET":
                                case 19:
                                    m.change_resource_type = 19;
                                    break;
                                case "CAMPAIGN_ASSET_SET":
                                case 20:
                                    m.change_resource_type = 20;
                                    break;
                                }
                                if (d.change_resource_name != null) {
                                    m.change_resource_name = String(d.change_resource_name);
                                }
                                switch (d.client_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.client_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.client_type = 1;
                                    break;
                                case "GOOGLE_ADS_WEB_CLIENT":
                                case 2:
                                    m.client_type = 2;
                                    break;
                                case "GOOGLE_ADS_AUTOMATED_RULE":
                                case 3:
                                    m.client_type = 3;
                                    break;
                                case "GOOGLE_ADS_SCRIPTS":
                                case 4:
                                    m.client_type = 4;
                                    break;
                                case "GOOGLE_ADS_BULK_UPLOAD":
                                case 5:
                                    m.client_type = 5;
                                    break;
                                case "GOOGLE_ADS_API":
                                case 6:
                                    m.client_type = 6;
                                    break;
                                case "GOOGLE_ADS_EDITOR":
                                case 7:
                                    m.client_type = 7;
                                    break;
                                case "GOOGLE_ADS_MOBILE_APP":
                                case 8:
                                    m.client_type = 8;
                                    break;
                                case "GOOGLE_ADS_RECOMMENDATIONS":
                                case 9:
                                    m.client_type = 9;
                                    break;
                                case "SEARCH_ADS_360_SYNC":
                                case 10:
                                    m.client_type = 10;
                                    break;
                                case "SEARCH_ADS_360_POST":
                                case 11:
                                    m.client_type = 11;
                                    break;
                                case "INTERNAL_TOOL":
                                case 12:
                                    m.client_type = 12;
                                    break;
                                case "OTHER":
                                case 13:
                                    m.client_type = 13;
                                    break;
                                }
                                if (d.user_email != null) {
                                    m.user_email = String(d.user_email);
                                }
                                if (d.old_resource != null) {
                                    if (typeof d.old_resource !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.old_resource: object expected");
                                    m.old_resource = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.fromObject(d.old_resource);
                                }
                                if (d.new_resource != null) {
                                    if (typeof d.new_resource !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.new_resource: object expected");
                                    m.new_resource = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.fromObject(d.new_resource);
                                }
                                switch (d.resource_change_operation) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.resource_change_operation = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.resource_change_operation = 1;
                                    break;
                                case "CREATE":
                                case 2:
                                    m.resource_change_operation = 2;
                                    break;
                                case "UPDATE":
                                case 3:
                                    m.resource_change_operation = 3;
                                    break;
                                case "REMOVE":
                                case 4:
                                    m.resource_change_operation = 4;
                                    break;
                                }
                                if (d.changed_fields != null) {
                                    if (typeof d.changed_fields !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.changed_fields: object expected");
                                    m.changed_fields = $root.google.protobuf.FieldMask.fromObject(d.changed_fields);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.feed_item != null) {
                                    m.feed_item = String(d.feed_item);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ChangeEvent message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.ChangeEvent} m ChangeEvent
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeEvent.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.change_date_time = "";
                                    d.change_resource_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.change_resource_name = "";
                                    d.client_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.user_email = "";
                                    d.old_resource = null;
                                    d.new_resource = null;
                                    d.resource_change_operation = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.changed_fields = null;
                                    d.campaign = "";
                                    d.ad_group = "";
                                    d.feed = "";
                                    d.feed_item = "";
                                    d.asset = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.change_date_time != null && m.hasOwnProperty("change_date_time")) {
                                    d.change_date_time = m.change_date_time;
                                }
                                if (m.change_resource_type != null && m.hasOwnProperty("change_resource_type")) {
                                    d.change_resource_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ChangeEventResourceTypeEnum.ChangeEventResourceType[m.change_resource_type] : m.change_resource_type;
                                }
                                if (m.change_resource_name != null && m.hasOwnProperty("change_resource_name")) {
                                    d.change_resource_name = m.change_resource_name;
                                }
                                if (m.client_type != null && m.hasOwnProperty("client_type")) {
                                    d.client_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ChangeClientTypeEnum.ChangeClientType[m.client_type] : m.client_type;
                                }
                                if (m.user_email != null && m.hasOwnProperty("user_email")) {
                                    d.user_email = m.user_email;
                                }
                                if (m.old_resource != null && m.hasOwnProperty("old_resource")) {
                                    d.old_resource = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.toObject(m.old_resource, o);
                                }
                                if (m.new_resource != null && m.hasOwnProperty("new_resource")) {
                                    d.new_resource = $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.toObject(m.new_resource, o);
                                }
                                if (m.resource_change_operation != null && m.hasOwnProperty("resource_change_operation")) {
                                    d.resource_change_operation = o.enums === String ? $root.google.ads.googleads.v11.enums.ResourceChangeOperationEnum.ResourceChangeOperation[m.resource_change_operation] : m.resource_change_operation;
                                }
                                if (m.changed_fields != null && m.hasOwnProperty("changed_fields")) {
                                    d.changed_fields = $root.google.protobuf.FieldMask.toObject(m.changed_fields, o);
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    d.feed_item = m.feed_item;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ChangeEvent to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ChangeEvent
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeEvent.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            ChangeEvent.ChangedResource = (function() {
    
                                /**
                                 * Properties of a ChangedResource.
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent
                                 * @interface IChangedResource
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [ad] ChangedResource ad
                                 * @property {google.ads.googleads.v11.resources.IAdGroup|null} [ad_group] ChangedResource ad_group
                                 * @property {google.ads.googleads.v11.resources.IAdGroupCriterion|null} [ad_group_criterion] ChangedResource ad_group_criterion
                                 * @property {google.ads.googleads.v11.resources.ICampaign|null} [campaign] ChangedResource campaign
                                 * @property {google.ads.googleads.v11.resources.ICampaignBudget|null} [campaign_budget] ChangedResource campaign_budget
                                 * @property {google.ads.googleads.v11.resources.IAdGroupBidModifier|null} [ad_group_bid_modifier] ChangedResource ad_group_bid_modifier
                                 * @property {google.ads.googleads.v11.resources.ICampaignCriterion|null} [campaign_criterion] ChangedResource campaign_criterion
                                 * @property {google.ads.googleads.v11.resources.IFeed|null} [feed] ChangedResource feed
                                 * @property {google.ads.googleads.v11.resources.IFeedItem|null} [feed_item] ChangedResource feed_item
                                 * @property {google.ads.googleads.v11.resources.ICampaignFeed|null} [campaign_feed] ChangedResource campaign_feed
                                 * @property {google.ads.googleads.v11.resources.IAdGroupFeed|null} [ad_group_feed] ChangedResource ad_group_feed
                                 * @property {google.ads.googleads.v11.resources.IAdGroupAd|null} [ad_group_ad] ChangedResource ad_group_ad
                                 * @property {google.ads.googleads.v11.resources.IAsset|null} [asset] ChangedResource asset
                                 * @property {google.ads.googleads.v11.resources.ICustomerAsset|null} [customer_asset] ChangedResource customer_asset
                                 * @property {google.ads.googleads.v11.resources.ICampaignAsset|null} [campaign_asset] ChangedResource campaign_asset
                                 * @property {google.ads.googleads.v11.resources.IAdGroupAsset|null} [ad_group_asset] ChangedResource ad_group_asset
                                 * @property {google.ads.googleads.v11.resources.IAssetSet|null} [asset_set] ChangedResource asset_set
                                 * @property {google.ads.googleads.v11.resources.IAssetSetAsset|null} [asset_set_asset] ChangedResource asset_set_asset
                                 * @property {google.ads.googleads.v11.resources.ICampaignAssetSet|null} [campaign_asset_set] ChangedResource campaign_asset_set
                                 */
    
                                /**
                                 * Constructs a new ChangedResource.
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent
                                 * @classdesc Represents a ChangedResource.
                                 * @implements IChangedResource
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource=} [p] Properties to set
                                 */
                                function ChangedResource(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ChangedResource ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} ad
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.ad = null;
    
                                /**
                                 * ChangedResource ad_group.
                                 * @member {google.ads.googleads.v11.resources.IAdGroup|null|undefined} ad_group
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.ad_group = null;
    
                                /**
                                 * ChangedResource ad_group_criterion.
                                 * @member {google.ads.googleads.v11.resources.IAdGroupCriterion|null|undefined} ad_group_criterion
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.ad_group_criterion = null;
    
                                /**
                                 * ChangedResource campaign.
                                 * @member {google.ads.googleads.v11.resources.ICampaign|null|undefined} campaign
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.campaign = null;
    
                                /**
                                 * ChangedResource campaign_budget.
                                 * @member {google.ads.googleads.v11.resources.ICampaignBudget|null|undefined} campaign_budget
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.campaign_budget = null;
    
                                /**
                                 * ChangedResource ad_group_bid_modifier.
                                 * @member {google.ads.googleads.v11.resources.IAdGroupBidModifier|null|undefined} ad_group_bid_modifier
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.ad_group_bid_modifier = null;
    
                                /**
                                 * ChangedResource campaign_criterion.
                                 * @member {google.ads.googleads.v11.resources.ICampaignCriterion|null|undefined} campaign_criterion
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.campaign_criterion = null;
    
                                /**
                                 * ChangedResource feed.
                                 * @member {google.ads.googleads.v11.resources.IFeed|null|undefined} feed
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.feed = null;
    
                                /**
                                 * ChangedResource feed_item.
                                 * @member {google.ads.googleads.v11.resources.IFeedItem|null|undefined} feed_item
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.feed_item = null;
    
                                /**
                                 * ChangedResource campaign_feed.
                                 * @member {google.ads.googleads.v11.resources.ICampaignFeed|null|undefined} campaign_feed
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.campaign_feed = null;
    
                                /**
                                 * ChangedResource ad_group_feed.
                                 * @member {google.ads.googleads.v11.resources.IAdGroupFeed|null|undefined} ad_group_feed
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.ad_group_feed = null;
    
                                /**
                                 * ChangedResource ad_group_ad.
                                 * @member {google.ads.googleads.v11.resources.IAdGroupAd|null|undefined} ad_group_ad
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.ad_group_ad = null;
    
                                /**
                                 * ChangedResource asset.
                                 * @member {google.ads.googleads.v11.resources.IAsset|null|undefined} asset
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.asset = null;
    
                                /**
                                 * ChangedResource customer_asset.
                                 * @member {google.ads.googleads.v11.resources.ICustomerAsset|null|undefined} customer_asset
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.customer_asset = null;
    
                                /**
                                 * ChangedResource campaign_asset.
                                 * @member {google.ads.googleads.v11.resources.ICampaignAsset|null|undefined} campaign_asset
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.campaign_asset = null;
    
                                /**
                                 * ChangedResource ad_group_asset.
                                 * @member {google.ads.googleads.v11.resources.IAdGroupAsset|null|undefined} ad_group_asset
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.ad_group_asset = null;
    
                                /**
                                 * ChangedResource asset_set.
                                 * @member {google.ads.googleads.v11.resources.IAssetSet|null|undefined} asset_set
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.asset_set = null;
    
                                /**
                                 * ChangedResource asset_set_asset.
                                 * @member {google.ads.googleads.v11.resources.IAssetSetAsset|null|undefined} asset_set_asset
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.asset_set_asset = null;
    
                                /**
                                 * ChangedResource campaign_asset_set.
                                 * @member {google.ads.googleads.v11.resources.ICampaignAssetSet|null|undefined} campaign_asset_set
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 */
                                ChangedResource.prototype.campaign_asset_set = null;
    
                                /**
                                 * Creates a new ChangedResource instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ChangeEvent.ChangedResource} ChangedResource instance
                                 */
                                ChangedResource.create = function create(properties) {
                                    return new ChangedResource(properties);
                                };
    
                                /**
                                 * Encodes the specified ChangedResource message. Does not implicitly {@link google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource} m ChangedResource message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ChangedResource.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.ad != null && Object.hasOwnProperty.call(m, "ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.ad, w.uint32(10).fork()).ldelim();
                                    if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                        $root.google.ads.googleads.v11.resources.AdGroup.encode(m.ad_group, w.uint32(18).fork()).ldelim();
                                    if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                        $root.google.ads.googleads.v11.resources.AdGroupCriterion.encode(m.ad_group_criterion, w.uint32(26).fork()).ldelim();
                                    if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                        $root.google.ads.googleads.v11.resources.Campaign.encode(m.campaign, w.uint32(34).fork()).ldelim();
                                    if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                        $root.google.ads.googleads.v11.resources.CampaignBudget.encode(m.campaign_budget, w.uint32(42).fork()).ldelim();
                                    if (m.ad_group_bid_modifier != null && Object.hasOwnProperty.call(m, "ad_group_bid_modifier"))
                                        $root.google.ads.googleads.v11.resources.AdGroupBidModifier.encode(m.ad_group_bid_modifier, w.uint32(50).fork()).ldelim();
                                    if (m.campaign_criterion != null && Object.hasOwnProperty.call(m, "campaign_criterion"))
                                        $root.google.ads.googleads.v11.resources.CampaignCriterion.encode(m.campaign_criterion, w.uint32(58).fork()).ldelim();
                                    if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                        $root.google.ads.googleads.v11.resources.Feed.encode(m.feed, w.uint32(66).fork()).ldelim();
                                    if (m.feed_item != null && Object.hasOwnProperty.call(m, "feed_item"))
                                        $root.google.ads.googleads.v11.resources.FeedItem.encode(m.feed_item, w.uint32(74).fork()).ldelim();
                                    if (m.campaign_feed != null && Object.hasOwnProperty.call(m, "campaign_feed"))
                                        $root.google.ads.googleads.v11.resources.CampaignFeed.encode(m.campaign_feed, w.uint32(82).fork()).ldelim();
                                    if (m.ad_group_feed != null && Object.hasOwnProperty.call(m, "ad_group_feed"))
                                        $root.google.ads.googleads.v11.resources.AdGroupFeed.encode(m.ad_group_feed, w.uint32(90).fork()).ldelim();
                                    if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                        $root.google.ads.googleads.v11.resources.AdGroupAd.encode(m.ad_group_ad, w.uint32(98).fork()).ldelim();
                                    if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                        $root.google.ads.googleads.v11.resources.Asset.encode(m.asset, w.uint32(106).fork()).ldelim();
                                    if (m.customer_asset != null && Object.hasOwnProperty.call(m, "customer_asset"))
                                        $root.google.ads.googleads.v11.resources.CustomerAsset.encode(m.customer_asset, w.uint32(114).fork()).ldelim();
                                    if (m.campaign_asset != null && Object.hasOwnProperty.call(m, "campaign_asset"))
                                        $root.google.ads.googleads.v11.resources.CampaignAsset.encode(m.campaign_asset, w.uint32(122).fork()).ldelim();
                                    if (m.ad_group_asset != null && Object.hasOwnProperty.call(m, "ad_group_asset"))
                                        $root.google.ads.googleads.v11.resources.AdGroupAsset.encode(m.ad_group_asset, w.uint32(130).fork()).ldelim();
                                    if (m.asset_set != null && Object.hasOwnProperty.call(m, "asset_set"))
                                        $root.google.ads.googleads.v11.resources.AssetSet.encode(m.asset_set, w.uint32(138).fork()).ldelim();
                                    if (m.asset_set_asset != null && Object.hasOwnProperty.call(m, "asset_set_asset"))
                                        $root.google.ads.googleads.v11.resources.AssetSetAsset.encode(m.asset_set_asset, w.uint32(146).fork()).ldelim();
                                    if (m.campaign_asset_set != null && Object.hasOwnProperty.call(m, "campaign_asset_set"))
                                        $root.google.ads.googleads.v11.resources.CampaignAssetSet.encode(m.campaign_asset_set, w.uint32(154).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ChangedResource message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ChangeEvent.IChangedResource} message ChangedResource message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ChangedResource.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ChangedResource message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ChangeEvent.ChangedResource} ChangedResource
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ChangedResource.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        case 2:
                                            m.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.decode(r, r.uint32());
                                            break;
                                        case 3:
                                            m.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.decode(r, r.uint32());
                                            break;
                                        case 4:
                                            m.campaign = $root.google.ads.googleads.v11.resources.Campaign.decode(r, r.uint32());
                                            break;
                                        case 5:
                                            m.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.decode(r, r.uint32());
                                            break;
                                        case 6:
                                            m.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.decode(r, r.uint32());
                                            break;
                                        case 7:
                                            m.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.decode(r, r.uint32());
                                            break;
                                        case 8:
                                            m.feed = $root.google.ads.googleads.v11.resources.Feed.decode(r, r.uint32());
                                            break;
                                        case 9:
                                            m.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.decode(r, r.uint32());
                                            break;
                                        case 10:
                                            m.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.decode(r, r.uint32());
                                            break;
                                        case 11:
                                            m.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.decode(r, r.uint32());
                                            break;
                                        case 12:
                                            m.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.decode(r, r.uint32());
                                            break;
                                        case 13:
                                            m.asset = $root.google.ads.googleads.v11.resources.Asset.decode(r, r.uint32());
                                            break;
                                        case 14:
                                            m.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.decode(r, r.uint32());
                                            break;
                                        case 15:
                                            m.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.decode(r, r.uint32());
                                            break;
                                        case 16:
                                            m.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.decode(r, r.uint32());
                                            break;
                                        case 17:
                                            m.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.decode(r, r.uint32());
                                            break;
                                        case 18:
                                            m.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.decode(r, r.uint32());
                                            break;
                                        case 19:
                                            m.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ChangedResource message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ChangeEvent.ChangedResource} ChangedResource
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ChangedResource.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ChangedResource message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ChangedResource.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.ad);
                                            if (e)
                                                return "ad." + e;
                                        }
                                    }
                                    if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AdGroup.verify(m.ad_group);
                                            if (e)
                                                return "ad_group." + e;
                                        }
                                    }
                                    if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AdGroupCriterion.verify(m.ad_group_criterion);
                                            if (e)
                                                return "ad_group_criterion." + e;
                                        }
                                    }
                                    if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Campaign.verify(m.campaign);
                                            if (e)
                                                return "campaign." + e;
                                        }
                                    }
                                    if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CampaignBudget.verify(m.campaign_budget);
                                            if (e)
                                                return "campaign_budget." + e;
                                        }
                                    }
                                    if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.verify(m.ad_group_bid_modifier);
                                            if (e)
                                                return "ad_group_bid_modifier." + e;
                                        }
                                    }
                                    if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CampaignCriterion.verify(m.campaign_criterion);
                                            if (e)
                                                return "campaign_criterion." + e;
                                        }
                                    }
                                    if (m.feed != null && m.hasOwnProperty("feed")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Feed.verify(m.feed);
                                            if (e)
                                                return "feed." + e;
                                        }
                                    }
                                    if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.FeedItem.verify(m.feed_item);
                                            if (e)
                                                return "feed_item." + e;
                                        }
                                    }
                                    if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CampaignFeed.verify(m.campaign_feed);
                                            if (e)
                                                return "campaign_feed." + e;
                                        }
                                    }
                                    if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AdGroupFeed.verify(m.ad_group_feed);
                                            if (e)
                                                return "ad_group_feed." + e;
                                        }
                                    }
                                    if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AdGroupAd.verify(m.ad_group_ad);
                                            if (e)
                                                return "ad_group_ad." + e;
                                        }
                                    }
                                    if (m.asset != null && m.hasOwnProperty("asset")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Asset.verify(m.asset);
                                            if (e)
                                                return "asset." + e;
                                        }
                                    }
                                    if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CustomerAsset.verify(m.customer_asset);
                                            if (e)
                                                return "customer_asset." + e;
                                        }
                                    }
                                    if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CampaignAsset.verify(m.campaign_asset);
                                            if (e)
                                                return "campaign_asset." + e;
                                        }
                                    }
                                    if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AdGroupAsset.verify(m.ad_group_asset);
                                            if (e)
                                                return "ad_group_asset." + e;
                                        }
                                    }
                                    if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AssetSet.verify(m.asset_set);
                                            if (e)
                                                return "asset_set." + e;
                                        }
                                    }
                                    if (m.asset_set_asset != null && m.hasOwnProperty("asset_set_asset")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AssetSetAsset.verify(m.asset_set_asset);
                                            if (e)
                                                return "asset_set_asset." + e;
                                        }
                                    }
                                    if (m.campaign_asset_set != null && m.hasOwnProperty("campaign_asset_set")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CampaignAssetSet.verify(m.campaign_asset_set);
                                            if (e)
                                                return "campaign_asset_set." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ChangedResource message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ChangeEvent.ChangedResource} ChangedResource
                                 */
                                ChangedResource.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ChangeEvent.ChangedResource();
                                    if (d.ad != null) {
                                        if (typeof d.ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.ad: object expected");
                                        m.ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.ad);
                                    }
                                    if (d.ad_group != null) {
                                        if (typeof d.ad_group !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.ad_group: object expected");
                                        m.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.fromObject(d.ad_group);
                                    }
                                    if (d.ad_group_criterion != null) {
                                        if (typeof d.ad_group_criterion !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.ad_group_criterion: object expected");
                                        m.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.fromObject(d.ad_group_criterion);
                                    }
                                    if (d.campaign != null) {
                                        if (typeof d.campaign !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.campaign: object expected");
                                        m.campaign = $root.google.ads.googleads.v11.resources.Campaign.fromObject(d.campaign);
                                    }
                                    if (d.campaign_budget != null) {
                                        if (typeof d.campaign_budget !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.campaign_budget: object expected");
                                        m.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.fromObject(d.campaign_budget);
                                    }
                                    if (d.ad_group_bid_modifier != null) {
                                        if (typeof d.ad_group_bid_modifier !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.ad_group_bid_modifier: object expected");
                                        m.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.fromObject(d.ad_group_bid_modifier);
                                    }
                                    if (d.campaign_criterion != null) {
                                        if (typeof d.campaign_criterion !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.campaign_criterion: object expected");
                                        m.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.fromObject(d.campaign_criterion);
                                    }
                                    if (d.feed != null) {
                                        if (typeof d.feed !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.feed: object expected");
                                        m.feed = $root.google.ads.googleads.v11.resources.Feed.fromObject(d.feed);
                                    }
                                    if (d.feed_item != null) {
                                        if (typeof d.feed_item !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.feed_item: object expected");
                                        m.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.fromObject(d.feed_item);
                                    }
                                    if (d.campaign_feed != null) {
                                        if (typeof d.campaign_feed !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.campaign_feed: object expected");
                                        m.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.fromObject(d.campaign_feed);
                                    }
                                    if (d.ad_group_feed != null) {
                                        if (typeof d.ad_group_feed !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.ad_group_feed: object expected");
                                        m.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.fromObject(d.ad_group_feed);
                                    }
                                    if (d.ad_group_ad != null) {
                                        if (typeof d.ad_group_ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.ad_group_ad: object expected");
                                        m.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.fromObject(d.ad_group_ad);
                                    }
                                    if (d.asset != null) {
                                        if (typeof d.asset !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.asset: object expected");
                                        m.asset = $root.google.ads.googleads.v11.resources.Asset.fromObject(d.asset);
                                    }
                                    if (d.customer_asset != null) {
                                        if (typeof d.customer_asset !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.customer_asset: object expected");
                                        m.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.fromObject(d.customer_asset);
                                    }
                                    if (d.campaign_asset != null) {
                                        if (typeof d.campaign_asset !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.campaign_asset: object expected");
                                        m.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.fromObject(d.campaign_asset);
                                    }
                                    if (d.ad_group_asset != null) {
                                        if (typeof d.ad_group_asset !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.ad_group_asset: object expected");
                                        m.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.fromObject(d.ad_group_asset);
                                    }
                                    if (d.asset_set != null) {
                                        if (typeof d.asset_set !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.asset_set: object expected");
                                        m.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.fromObject(d.asset_set);
                                    }
                                    if (d.asset_set_asset != null) {
                                        if (typeof d.asset_set_asset !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.asset_set_asset: object expected");
                                        m.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.fromObject(d.asset_set_asset);
                                    }
                                    if (d.campaign_asset_set != null) {
                                        if (typeof d.campaign_asset_set !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ChangeEvent.ChangedResource.campaign_asset_set: object expected");
                                        m.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.fromObject(d.campaign_asset_set);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ChangedResource message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ChangeEvent.ChangedResource} m ChangedResource
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ChangedResource.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.ad = null;
                                        d.ad_group = null;
                                        d.ad_group_criterion = null;
                                        d.campaign = null;
                                        d.campaign_budget = null;
                                        d.ad_group_bid_modifier = null;
                                        d.campaign_criterion = null;
                                        d.feed = null;
                                        d.feed_item = null;
                                        d.campaign_feed = null;
                                        d.ad_group_feed = null;
                                        d.ad_group_ad = null;
                                        d.asset = null;
                                        d.customer_asset = null;
                                        d.campaign_asset = null;
                                        d.ad_group_asset = null;
                                        d.asset_set = null;
                                        d.asset_set_asset = null;
                                        d.campaign_asset_set = null;
                                    }
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        d.ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.ad, o);
                                    }
                                    if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                        d.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.toObject(m.ad_group, o);
                                    }
                                    if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                        d.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.toObject(m.ad_group_criterion, o);
                                    }
                                    if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                        d.campaign = $root.google.ads.googleads.v11.resources.Campaign.toObject(m.campaign, o);
                                    }
                                    if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                        d.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.toObject(m.campaign_budget, o);
                                    }
                                    if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                        d.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.toObject(m.ad_group_bid_modifier, o);
                                    }
                                    if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                        d.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.toObject(m.campaign_criterion, o);
                                    }
                                    if (m.feed != null && m.hasOwnProperty("feed")) {
                                        d.feed = $root.google.ads.googleads.v11.resources.Feed.toObject(m.feed, o);
                                    }
                                    if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                        d.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.toObject(m.feed_item, o);
                                    }
                                    if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                        d.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.toObject(m.campaign_feed, o);
                                    }
                                    if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                        d.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.toObject(m.ad_group_feed, o);
                                    }
                                    if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                        d.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.toObject(m.ad_group_ad, o);
                                    }
                                    if (m.asset != null && m.hasOwnProperty("asset")) {
                                        d.asset = $root.google.ads.googleads.v11.resources.Asset.toObject(m.asset, o);
                                    }
                                    if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                        d.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.toObject(m.customer_asset, o);
                                    }
                                    if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                        d.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.toObject(m.campaign_asset, o);
                                    }
                                    if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                        d.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.toObject(m.ad_group_asset, o);
                                    }
                                    if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                        d.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.toObject(m.asset_set, o);
                                    }
                                    if (m.asset_set_asset != null && m.hasOwnProperty("asset_set_asset")) {
                                        d.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.toObject(m.asset_set_asset, o);
                                    }
                                    if (m.campaign_asset_set != null && m.hasOwnProperty("campaign_asset_set")) {
                                        d.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.toObject(m.campaign_asset_set, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ChangedResource to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ChangeEvent.ChangedResource
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ChangedResource.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ChangedResource;
                            })();
    
                            return ChangeEvent;
                        })();
    
                        resources.CustomerAsset = (function() {
    
                            /**
                             * Properties of a CustomerAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerAsset
                             * @property {string|null} [resource_name] CustomerAsset resource_name
                             * @property {string|null} [asset] CustomerAsset asset
                             * @property {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType|null} [field_type] CustomerAsset field_type
                             * @property {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource|null} [source] CustomerAsset source
                             * @property {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus|null} [status] CustomerAsset status
                             */
    
                            /**
                             * Constructs a new CustomerAsset.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerAsset.
                             * @implements ICustomerAsset
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerAsset=} [p] Properties to set
                             */
                            function CustomerAsset(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerAsset resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @instance
                             */
                            CustomerAsset.prototype.resource_name = "";
    
                            /**
                             * CustomerAsset asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @instance
                             */
                            CustomerAsset.prototype.asset = "";
    
                            /**
                             * CustomerAsset field_type.
                             * @member {google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType} field_type
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @instance
                             */
                            CustomerAsset.prototype.field_type = 0;
    
                            /**
                             * CustomerAsset source.
                             * @member {google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource} source
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @instance
                             */
                            CustomerAsset.prototype.source = 0;
    
                            /**
                             * CustomerAsset status.
                             * @member {google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @instance
                             */
                            CustomerAsset.prototype.status = 0;
    
                            /**
                             * Creates a new CustomerAsset instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerAsset=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerAsset} CustomerAsset instance
                             */
                            CustomerAsset.create = function create(properties) {
                                return new CustomerAsset(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerAsset message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerAsset.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerAsset} m CustomerAsset message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerAsset.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(18).string(m.asset);
                                if (m.field_type != null && Object.hasOwnProperty.call(m, "field_type"))
                                    w.uint32(24).int32(m.field_type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.source != null && Object.hasOwnProperty.call(m, "source"))
                                    w.uint32(40).int32(m.source);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerAsset message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerAsset.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerAsset} message CustomerAsset message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerAsset.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerAsset message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerAsset} CustomerAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerAsset.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerAsset();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset = r.string();
                                        break;
                                    case 3:
                                        m.field_type = r.int32();
                                        break;
                                    case 5:
                                        m.source = r.int32();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerAsset message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerAsset} CustomerAsset
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerAsset.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerAsset message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerAsset.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    switch (m.field_type) {
                                    default:
                                        return "field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 24:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 25:
                                        break;
                                    }
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    switch (m.source) {
                                    default:
                                        return "source: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerAsset message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerAsset} CustomerAsset
                             */
                            CustomerAsset.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerAsset)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerAsset();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                switch (d.field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_type = 1;
                                    break;
                                case "HEADLINE":
                                case 2:
                                    m.field_type = 2;
                                    break;
                                case "DESCRIPTION":
                                case 3:
                                    m.field_type = 3;
                                    break;
                                case "MANDATORY_AD_TEXT":
                                case 4:
                                    m.field_type = 4;
                                    break;
                                case "MARKETING_IMAGE":
                                case 5:
                                    m.field_type = 5;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 6:
                                    m.field_type = 6;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 7:
                                    m.field_type = 7;
                                    break;
                                case "BOOK_ON_GOOGLE":
                                case 8:
                                    m.field_type = 8;
                                    break;
                                case "LEAD_FORM":
                                case 9:
                                    m.field_type = 9;
                                    break;
                                case "PROMOTION":
                                case 10:
                                    m.field_type = 10;
                                    break;
                                case "CALLOUT":
                                case 11:
                                    m.field_type = 11;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 12:
                                    m.field_type = 12;
                                    break;
                                case "SITELINK":
                                case 13:
                                    m.field_type = 13;
                                    break;
                                case "MOBILE_APP":
                                case 14:
                                    m.field_type = 14;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.field_type = 15;
                                    break;
                                case "CALL":
                                case 16:
                                    m.field_type = 16;
                                    break;
                                case "PRICE":
                                case 24:
                                    m.field_type = 24;
                                    break;
                                case "LONG_HEADLINE":
                                case 17:
                                    m.field_type = 17;
                                    break;
                                case "BUSINESS_NAME":
                                case 18:
                                    m.field_type = 18;
                                    break;
                                case "SQUARE_MARKETING_IMAGE":
                                case 19:
                                    m.field_type = 19;
                                    break;
                                case "PORTRAIT_MARKETING_IMAGE":
                                case 20:
                                    m.field_type = 20;
                                    break;
                                case "LOGO":
                                case 21:
                                    m.field_type = 21;
                                    break;
                                case "LANDSCAPE_LOGO":
                                case 22:
                                    m.field_type = 22;
                                    break;
                                case "VIDEO":
                                case 23:
                                    m.field_type = 23;
                                    break;
                                case "CALL_TO_ACTION_SELECTION":
                                case 25:
                                    m.field_type = 25;
                                    break;
                                }
                                switch (d.source) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.source = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.source = 1;
                                    break;
                                case "ADVERTISER":
                                case 2:
                                    m.source = 2;
                                    break;
                                case "AUTOMATICALLY_CREATED":
                                case 3:
                                    m.source = 3;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PAUSED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerAsset message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerAsset} m CustomerAsset
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerAsset.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset = "";
                                    d.field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.source = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    d.field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetFieldTypeEnum.AssetFieldType[m.field_type] : m.field_type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetLinkStatusEnum.AssetLinkStatus[m.status] : m.status;
                                }
                                if (m.source != null && m.hasOwnProperty("source")) {
                                    d.source = o.enums === String ? $root.google.ads.googleads.v11.enums.AssetSourceEnum.AssetSource[m.source] : m.source;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerAsset to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerAsset
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerAsset.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerAsset;
                        })();
    
                        resources.Feed = (function() {
    
                            /**
                             * Properties of a Feed.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeed
                             * @property {string|null} [resource_name] Feed resource_name
                             * @property {number|null} [id] Feed id
                             * @property {string|null} [name] Feed name
                             * @property {Array.<google.ads.googleads.v11.resources.IFeedAttribute>|null} [attributes] Feed attributes
                             * @property {Array.<google.ads.googleads.v11.resources.IFeedAttributeOperation>|null} [attribute_operations] Feed attribute_operations
                             * @property {google.ads.googleads.v11.enums.FeedOriginEnum.FeedOrigin|null} [origin] Feed origin
                             * @property {google.ads.googleads.v11.enums.FeedStatusEnum.FeedStatus|null} [status] Feed status
                             * @property {google.ads.googleads.v11.resources.Feed.IPlacesLocationFeedData|null} [places_location_feed_data] Feed places_location_feed_data
                             * @property {google.ads.googleads.v11.resources.Feed.IAffiliateLocationFeedData|null} [affiliate_location_feed_data] Feed affiliate_location_feed_data
                             */
    
                            /**
                             * Constructs a new Feed.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a Feed.
                             * @implements IFeed
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeed=} [p] Properties to set
                             */
                            function Feed(p) {
                                this.attributes = [];
                                this.attribute_operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Feed resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.resource_name = "";
    
                            /**
                             * Feed id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.id = null;
    
                            /**
                             * Feed name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.name = null;
    
                            /**
                             * Feed attributes.
                             * @member {Array.<google.ads.googleads.v11.resources.IFeedAttribute>} attributes
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.attributes = $util.emptyArray;
    
                            /**
                             * Feed attribute_operations.
                             * @member {Array.<google.ads.googleads.v11.resources.IFeedAttributeOperation>} attribute_operations
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.attribute_operations = $util.emptyArray;
    
                            /**
                             * Feed origin.
                             * @member {google.ads.googleads.v11.enums.FeedOriginEnum.FeedOrigin} origin
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.origin = 0;
    
                            /**
                             * Feed status.
                             * @member {google.ads.googleads.v11.enums.FeedStatusEnum.FeedStatus} status
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.status = 0;
    
                            /**
                             * Feed places_location_feed_data.
                             * @member {google.ads.googleads.v11.resources.Feed.IPlacesLocationFeedData|null|undefined} places_location_feed_data
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.places_location_feed_data = null;
    
                            /**
                             * Feed affiliate_location_feed_data.
                             * @member {google.ads.googleads.v11.resources.Feed.IAffiliateLocationFeedData|null|undefined} affiliate_location_feed_data
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Feed.prototype.affiliate_location_feed_data = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Feed _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Object.defineProperty(Feed.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Feed _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Object.defineProperty(Feed.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Feed system_feed_generation_data.
                             * @member {"places_location_feed_data"|"affiliate_location_feed_data"|undefined} system_feed_generation_data
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             */
                            Object.defineProperty(Feed.prototype, "system_feed_generation_data", {
                                get: $util.oneOfGetter($oneOfFields = ["places_location_feed_data", "affiliate_location_feed_data"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Feed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Feed} Feed instance
                             */
                            Feed.create = function create(properties) {
                                return new Feed(properties);
                            };
    
                            /**
                             * Encodes the specified Feed message. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeed} m Feed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Feed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.attributes != null && m.attributes.length) {
                                    for (var i = 0; i < m.attributes.length; ++i)
                                        $root.google.ads.googleads.v11.resources.FeedAttribute.encode(m.attributes[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.origin != null && Object.hasOwnProperty.call(m, "origin"))
                                    w.uint32(40).int32(m.origin);
                                if (m.places_location_feed_data != null && Object.hasOwnProperty.call(m, "places_location_feed_data"))
                                    $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.encode(m.places_location_feed_data, w.uint32(50).fork()).ldelim();
                                if (m.affiliate_location_feed_data != null && Object.hasOwnProperty.call(m, "affiliate_location_feed_data"))
                                    $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.encode(m.affiliate_location_feed_data, w.uint32(58).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(64).int32(m.status);
                                if (m.attribute_operations != null && m.attribute_operations.length) {
                                    for (var i = 0; i < m.attribute_operations.length; ++i)
                                        $root.google.ads.googleads.v11.resources.FeedAttributeOperation.encode(m.attribute_operations[i], w.uint32(74).fork()).ldelim();
                                }
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(88).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(98).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Feed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeed} message Feed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Feed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Feed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Feed} Feed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Feed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Feed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 11:
                                        m.id = r.int64();
                                        break;
                                    case 12:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        if (!(m.attributes && m.attributes.length))
                                            m.attributes = [];
                                        m.attributes.push($root.google.ads.googleads.v11.resources.FeedAttribute.decode(r, r.uint32()));
                                        break;
                                    case 9:
                                        if (!(m.attribute_operations && m.attribute_operations.length))
                                            m.attribute_operations = [];
                                        m.attribute_operations.push($root.google.ads.googleads.v11.resources.FeedAttributeOperation.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        m.origin = r.int32();
                                        break;
                                    case 8:
                                        m.status = r.int32();
                                        break;
                                    case 6:
                                        m.places_location_feed_data = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.affiliate_location_feed_data = $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Feed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Feed} Feed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Feed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Feed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Feed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.attributes != null && m.hasOwnProperty("attributes")) {
                                    if (!Array.isArray(m.attributes))
                                        return "attributes: array expected";
                                    for (var i = 0; i < m.attributes.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.FeedAttribute.verify(m.attributes[i]);
                                            if (e)
                                                return "attributes." + e;
                                        }
                                    }
                                }
                                if (m.attribute_operations != null && m.hasOwnProperty("attribute_operations")) {
                                    if (!Array.isArray(m.attribute_operations))
                                        return "attribute_operations: array expected";
                                    for (var i = 0; i < m.attribute_operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.FeedAttributeOperation.verify(m.attribute_operations[i]);
                                            if (e)
                                                return "attribute_operations." + e;
                                        }
                                    }
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    switch (m.origin) {
                                    default:
                                        return "origin: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.places_location_feed_data != null && m.hasOwnProperty("places_location_feed_data")) {
                                    p.system_feed_generation_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.verify(m.places_location_feed_data);
                                        if (e)
                                            return "places_location_feed_data." + e;
                                    }
                                }
                                if (m.affiliate_location_feed_data != null && m.hasOwnProperty("affiliate_location_feed_data")) {
                                    if (p.system_feed_generation_data === 1)
                                        return "system_feed_generation_data: multiple values";
                                    p.system_feed_generation_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.verify(m.affiliate_location_feed_data);
                                        if (e)
                                            return "affiliate_location_feed_data." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Feed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Feed} Feed
                             */
                            Feed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Feed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Feed();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.attributes) {
                                    if (!Array.isArray(d.attributes))
                                        throw TypeError(".google.ads.googleads.v11.resources.Feed.attributes: array expected");
                                    m.attributes = [];
                                    for (var i = 0; i < d.attributes.length; ++i) {
                                        if (typeof d.attributes[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Feed.attributes: object expected");
                                        m.attributes[i] = $root.google.ads.googleads.v11.resources.FeedAttribute.fromObject(d.attributes[i]);
                                    }
                                }
                                if (d.attribute_operations) {
                                    if (!Array.isArray(d.attribute_operations))
                                        throw TypeError(".google.ads.googleads.v11.resources.Feed.attribute_operations: array expected");
                                    m.attribute_operations = [];
                                    for (var i = 0; i < d.attribute_operations.length; ++i) {
                                        if (typeof d.attribute_operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Feed.attribute_operations: object expected");
                                        m.attribute_operations[i] = $root.google.ads.googleads.v11.resources.FeedAttributeOperation.fromObject(d.attribute_operations[i]);
                                    }
                                }
                                switch (d.origin) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.origin = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.origin = 1;
                                    break;
                                case "USER":
                                case 2:
                                    m.origin = 2;
                                    break;
                                case "GOOGLE":
                                case 3:
                                    m.origin = 3;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.places_location_feed_data != null) {
                                    if (typeof d.places_location_feed_data !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Feed.places_location_feed_data: object expected");
                                    m.places_location_feed_data = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.fromObject(d.places_location_feed_data);
                                }
                                if (d.affiliate_location_feed_data != null) {
                                    if (typeof d.affiliate_location_feed_data !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Feed.affiliate_location_feed_data: object expected");
                                    m.affiliate_location_feed_data = $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.fromObject(d.affiliate_location_feed_data);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Feed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @static
                             * @param {google.ads.googleads.v11.resources.Feed} m Feed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Feed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.attributes = [];
                                    d.attribute_operations = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.origin = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.attributes && m.attributes.length) {
                                    d.attributes = [];
                                    for (var j = 0; j < m.attributes.length; ++j) {
                                        d.attributes[j] = $root.google.ads.googleads.v11.resources.FeedAttribute.toObject(m.attributes[j], o);
                                    }
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    d.origin = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedOriginEnum.FeedOrigin[m.origin] : m.origin;
                                }
                                if (m.places_location_feed_data != null && m.hasOwnProperty("places_location_feed_data")) {
                                    d.places_location_feed_data = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.toObject(m.places_location_feed_data, o);
                                    if (o.oneofs)
                                        d.system_feed_generation_data = "places_location_feed_data";
                                }
                                if (m.affiliate_location_feed_data != null && m.hasOwnProperty("affiliate_location_feed_data")) {
                                    d.affiliate_location_feed_data = $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.toObject(m.affiliate_location_feed_data, o);
                                    if (o.oneofs)
                                        d.system_feed_generation_data = "affiliate_location_feed_data";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedStatusEnum.FeedStatus[m.status] : m.status;
                                }
                                if (m.attribute_operations && m.attribute_operations.length) {
                                    d.attribute_operations = [];
                                    for (var j = 0; j < m.attribute_operations.length; ++j) {
                                        d.attribute_operations[j] = $root.google.ads.googleads.v11.resources.FeedAttributeOperation.toObject(m.attribute_operations[j], o);
                                    }
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Feed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Feed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Feed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            Feed.PlacesLocationFeedData = (function() {
    
                                /**
                                 * Properties of a PlacesLocationFeedData.
                                 * @memberof google.ads.googleads.v11.resources.Feed
                                 * @interface IPlacesLocationFeedData
                                 * @property {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.IOAuthInfo|null} [oauth_info] PlacesLocationFeedData oauth_info
                                 * @property {string|null} [email_address] PlacesLocationFeedData email_address
                                 * @property {string|null} [business_account_id] PlacesLocationFeedData business_account_id
                                 * @property {string|null} [business_name_filter] PlacesLocationFeedData business_name_filter
                                 * @property {Array.<string>|null} [category_filters] PlacesLocationFeedData category_filters
                                 * @property {Array.<string>|null} [label_filters] PlacesLocationFeedData label_filters
                                 */
    
                                /**
                                 * Constructs a new PlacesLocationFeedData.
                                 * @memberof google.ads.googleads.v11.resources.Feed
                                 * @classdesc Represents a PlacesLocationFeedData.
                                 * @implements IPlacesLocationFeedData
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Feed.IPlacesLocationFeedData=} [p] Properties to set
                                 */
                                function PlacesLocationFeedData(p) {
                                    this.category_filters = [];
                                    this.label_filters = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * PlacesLocationFeedData oauth_info.
                                 * @member {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.IOAuthInfo|null|undefined} oauth_info
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                PlacesLocationFeedData.prototype.oauth_info = null;
    
                                /**
                                 * PlacesLocationFeedData email_address.
                                 * @member {string|null|undefined} email_address
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                PlacesLocationFeedData.prototype.email_address = null;
    
                                /**
                                 * PlacesLocationFeedData business_account_id.
                                 * @member {string} business_account_id
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                PlacesLocationFeedData.prototype.business_account_id = "";
    
                                /**
                                 * PlacesLocationFeedData business_name_filter.
                                 * @member {string|null|undefined} business_name_filter
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                PlacesLocationFeedData.prototype.business_name_filter = null;
    
                                /**
                                 * PlacesLocationFeedData category_filters.
                                 * @member {Array.<string>} category_filters
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                PlacesLocationFeedData.prototype.category_filters = $util.emptyArray;
    
                                /**
                                 * PlacesLocationFeedData label_filters.
                                 * @member {Array.<string>} label_filters
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                PlacesLocationFeedData.prototype.label_filters = $util.emptyArray;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * PlacesLocationFeedData _email_address.
                                 * @member {"email_address"|undefined} _email_address
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                Object.defineProperty(PlacesLocationFeedData.prototype, "_email_address", {
                                    get: $util.oneOfGetter($oneOfFields = ["email_address"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PlacesLocationFeedData _business_name_filter.
                                 * @member {"business_name_filter"|undefined} _business_name_filter
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 */
                                Object.defineProperty(PlacesLocationFeedData.prototype, "_business_name_filter", {
                                    get: $util.oneOfGetter($oneOfFields = ["business_name_filter"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new PlacesLocationFeedData instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.IPlacesLocationFeedData=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData} PlacesLocationFeedData instance
                                 */
                                PlacesLocationFeedData.create = function create(properties) {
                                    return new PlacesLocationFeedData(properties);
                                };
    
                                /**
                                 * Encodes the specified PlacesLocationFeedData message. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.IPlacesLocationFeedData} m PlacesLocationFeedData message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PlacesLocationFeedData.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.oauth_info != null && Object.hasOwnProperty.call(m, "oauth_info"))
                                        $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo.encode(m.oauth_info, w.uint32(10).fork()).ldelim();
                                    if (m.email_address != null && Object.hasOwnProperty.call(m, "email_address"))
                                        w.uint32(58).string(m.email_address);
                                    if (m.business_account_id != null && Object.hasOwnProperty.call(m, "business_account_id"))
                                        w.uint32(66).string(m.business_account_id);
                                    if (m.business_name_filter != null && Object.hasOwnProperty.call(m, "business_name_filter"))
                                        w.uint32(74).string(m.business_name_filter);
                                    if (m.category_filters != null && m.category_filters.length) {
                                        for (var i = 0; i < m.category_filters.length; ++i)
                                            w.uint32(90).string(m.category_filters[i]);
                                    }
                                    if (m.label_filters != null && m.label_filters.length) {
                                        for (var i = 0; i < m.label_filters.length; ++i)
                                            w.uint32(98).string(m.label_filters[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified PlacesLocationFeedData message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.IPlacesLocationFeedData} message PlacesLocationFeedData message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PlacesLocationFeedData.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a PlacesLocationFeedData message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData} PlacesLocationFeedData
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PlacesLocationFeedData.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.oauth_info = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo.decode(r, r.uint32());
                                            break;
                                        case 7:
                                            m.email_address = r.string();
                                            break;
                                        case 8:
                                            m.business_account_id = r.string();
                                            break;
                                        case 9:
                                            m.business_name_filter = r.string();
                                            break;
                                        case 11:
                                            if (!(m.category_filters && m.category_filters.length))
                                                m.category_filters = [];
                                            m.category_filters.push(r.string());
                                            break;
                                        case 12:
                                            if (!(m.label_filters && m.label_filters.length))
                                                m.label_filters = [];
                                            m.label_filters.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a PlacesLocationFeedData message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData} PlacesLocationFeedData
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PlacesLocationFeedData.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a PlacesLocationFeedData message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                PlacesLocationFeedData.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.oauth_info != null && m.hasOwnProperty("oauth_info")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo.verify(m.oauth_info);
                                            if (e)
                                                return "oauth_info." + e;
                                        }
                                    }
                                    if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                        p._email_address = 1;
                                        if (!$util.isString(m.email_address))
                                            return "email_address: string expected";
                                    }
                                    if (m.business_account_id != null && m.hasOwnProperty("business_account_id")) {
                                        if (!$util.isString(m.business_account_id))
                                            return "business_account_id: string expected";
                                    }
                                    if (m.business_name_filter != null && m.hasOwnProperty("business_name_filter")) {
                                        p._business_name_filter = 1;
                                        if (!$util.isString(m.business_name_filter))
                                            return "business_name_filter: string expected";
                                    }
                                    if (m.category_filters != null && m.hasOwnProperty("category_filters")) {
                                        if (!Array.isArray(m.category_filters))
                                            return "category_filters: array expected";
                                        for (var i = 0; i < m.category_filters.length; ++i) {
                                            if (!$util.isString(m.category_filters[i]))
                                                return "category_filters: string[] expected";
                                        }
                                    }
                                    if (m.label_filters != null && m.hasOwnProperty("label_filters")) {
                                        if (!Array.isArray(m.label_filters))
                                            return "label_filters: array expected";
                                        for (var i = 0; i < m.label_filters.length; ++i) {
                                            if (!$util.isString(m.label_filters[i]))
                                                return "label_filters: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a PlacesLocationFeedData message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData} PlacesLocationFeedData
                                 */
                                PlacesLocationFeedData.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData();
                                    if (d.oauth_info != null) {
                                        if (typeof d.oauth_info !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.oauth_info: object expected");
                                        m.oauth_info = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo.fromObject(d.oauth_info);
                                    }
                                    if (d.email_address != null) {
                                        m.email_address = String(d.email_address);
                                    }
                                    if (d.business_account_id != null) {
                                        m.business_account_id = String(d.business_account_id);
                                    }
                                    if (d.business_name_filter != null) {
                                        m.business_name_filter = String(d.business_name_filter);
                                    }
                                    if (d.category_filters) {
                                        if (!Array.isArray(d.category_filters))
                                            throw TypeError(".google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.category_filters: array expected");
                                        m.category_filters = [];
                                        for (var i = 0; i < d.category_filters.length; ++i) {
                                            m.category_filters[i] = String(d.category_filters[i]);
                                        }
                                    }
                                    if (d.label_filters) {
                                        if (!Array.isArray(d.label_filters))
                                            throw TypeError(".google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.label_filters: array expected");
                                        m.label_filters = [];
                                        for (var i = 0; i < d.label_filters.length; ++i) {
                                            m.label_filters[i] = String(d.label_filters[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a PlacesLocationFeedData message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData} m PlacesLocationFeedData
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                PlacesLocationFeedData.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.category_filters = [];
                                        d.label_filters = [];
                                    }
                                    if (o.defaults) {
                                        d.oauth_info = null;
                                        d.business_account_id = "";
                                    }
                                    if (m.oauth_info != null && m.hasOwnProperty("oauth_info")) {
                                        d.oauth_info = $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo.toObject(m.oauth_info, o);
                                    }
                                    if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                        d.email_address = m.email_address;
                                        if (o.oneofs)
                                            d._email_address = "email_address";
                                    }
                                    if (m.business_account_id != null && m.hasOwnProperty("business_account_id")) {
                                        d.business_account_id = m.business_account_id;
                                    }
                                    if (m.business_name_filter != null && m.hasOwnProperty("business_name_filter")) {
                                        d.business_name_filter = m.business_name_filter;
                                        if (o.oneofs)
                                            d._business_name_filter = "business_name_filter";
                                    }
                                    if (m.category_filters && m.category_filters.length) {
                                        d.category_filters = [];
                                        for (var j = 0; j < m.category_filters.length; ++j) {
                                            d.category_filters[j] = m.category_filters[j];
                                        }
                                    }
                                    if (m.label_filters && m.label_filters.length) {
                                        d.label_filters = [];
                                        for (var j = 0; j < m.label_filters.length; ++j) {
                                            d.label_filters[j] = m.label_filters[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this PlacesLocationFeedData to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                PlacesLocationFeedData.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                PlacesLocationFeedData.OAuthInfo = (function() {
    
                                    /**
                                     * Properties of a OAuthInfo.
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                     * @interface IOAuthInfo
                                     * @property {string|null} [http_method] OAuthInfo http_method
                                     * @property {string|null} [http_request_url] OAuthInfo http_request_url
                                     * @property {string|null} [http_authorization_header] OAuthInfo http_authorization_header
                                     */
    
                                    /**
                                     * Constructs a new OAuthInfo.
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData
                                     * @classdesc Represents a OAuthInfo.
                                     * @implements IOAuthInfo
                                     * @constructor
                                     * @param {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.IOAuthInfo=} [p] Properties to set
                                     */
                                    function OAuthInfo(p) {
                                        if (p)
                                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                                if (p[ks[i]] != null)
                                                    this[ks[i]] = p[ks[i]];
                                    }
    
                                    /**
                                     * OAuthInfo http_method.
                                     * @member {string|null|undefined} http_method
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @instance
                                     */
                                    OAuthInfo.prototype.http_method = null;
    
                                    /**
                                     * OAuthInfo http_request_url.
                                     * @member {string|null|undefined} http_request_url
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @instance
                                     */
                                    OAuthInfo.prototype.http_request_url = null;
    
                                    /**
                                     * OAuthInfo http_authorization_header.
                                     * @member {string|null|undefined} http_authorization_header
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @instance
                                     */
                                    OAuthInfo.prototype.http_authorization_header = null;
    
                                    // OneOf field names bound to virtual getters and setters
                                    var $oneOfFields;
    
                                    /**
                                     * OAuthInfo _http_method.
                                     * @member {"http_method"|undefined} _http_method
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @instance
                                     */
                                    Object.defineProperty(OAuthInfo.prototype, "_http_method", {
                                        get: $util.oneOfGetter($oneOfFields = ["http_method"]),
                                        set: $util.oneOfSetter($oneOfFields)
                                    });
    
                                    /**
                                     * OAuthInfo _http_request_url.
                                     * @member {"http_request_url"|undefined} _http_request_url
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @instance
                                     */
                                    Object.defineProperty(OAuthInfo.prototype, "_http_request_url", {
                                        get: $util.oneOfGetter($oneOfFields = ["http_request_url"]),
                                        set: $util.oneOfSetter($oneOfFields)
                                    });
    
                                    /**
                                     * OAuthInfo _http_authorization_header.
                                     * @member {"http_authorization_header"|undefined} _http_authorization_header
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @instance
                                     */
                                    Object.defineProperty(OAuthInfo.prototype, "_http_authorization_header", {
                                        get: $util.oneOfGetter($oneOfFields = ["http_authorization_header"]),
                                        set: $util.oneOfSetter($oneOfFields)
                                    });
    
                                    /**
                                     * Creates a new OAuthInfo instance using the specified properties.
                                     * @function create
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.IOAuthInfo=} [properties] Properties to set
                                     * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo} OAuthInfo instance
                                     */
                                    OAuthInfo.create = function create(properties) {
                                        return new OAuthInfo(properties);
                                    };
    
                                    /**
                                     * Encodes the specified OAuthInfo message. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo.verify|verify} messages.
                                     * @function encode
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.IOAuthInfo} m OAuthInfo message or plain object to encode
                                     * @param {$protobuf.Writer} [w] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    OAuthInfo.encode = function encode(m, w) {
                                        if (!w)
                                            w = $Writer.create();
                                        if (m.http_method != null && Object.hasOwnProperty.call(m, "http_method"))
                                            w.uint32(34).string(m.http_method);
                                        if (m.http_request_url != null && Object.hasOwnProperty.call(m, "http_request_url"))
                                            w.uint32(42).string(m.http_request_url);
                                        if (m.http_authorization_header != null && Object.hasOwnProperty.call(m, "http_authorization_header"))
                                            w.uint32(50).string(m.http_authorization_header);
                                        return w;
                                    };
    
                                    /**
                                     * Encodes the specified OAuthInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo.verify|verify} messages.
                                     * @function encodeDelimited
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.IOAuthInfo} message OAuthInfo message or plain object to encode
                                     * @param {$protobuf.Writer} [writer] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    OAuthInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                        return this.encode(message, writer).ldelim();
                                    };
    
                                    /**
                                     * Decodes a OAuthInfo message from the specified reader or buffer.
                                     * @function decode
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                     * @param {number} [l] Message length if known beforehand
                                     * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo} OAuthInfo
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    OAuthInfo.decode = function decode(r, l) {
                                        if (!(r instanceof $Reader))
                                            r = $Reader.create(r);
                                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo();
                                        while (r.pos < c) {
                                            var t = r.uint32();
                                            switch (t >>> 3) {
                                            case 4:
                                                m.http_method = r.string();
                                                break;
                                            case 5:
                                                m.http_request_url = r.string();
                                                break;
                                            case 6:
                                                m.http_authorization_header = r.string();
                                                break;
                                            default:
                                                r.skipType(t & 7);
                                                break;
                                            }
                                        }
                                        return m;
                                    };
    
                                    /**
                                     * Decodes a OAuthInfo message from the specified reader or buffer, length delimited.
                                     * @function decodeDelimited
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                     * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo} OAuthInfo
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    OAuthInfo.decodeDelimited = function decodeDelimited(reader) {
                                        if (!(reader instanceof $Reader))
                                            reader = new $Reader(reader);
                                        return this.decode(reader, reader.uint32());
                                    };
    
                                    /**
                                     * Verifies a OAuthInfo message.
                                     * @function verify
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {Object.<string,*>} m Plain object to verify
                                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                     */
                                    OAuthInfo.verify = function verify(m) {
                                        if (typeof m !== "object" || m === null)
                                            return "object expected";
                                        var p = {};
                                        if (m.http_method != null && m.hasOwnProperty("http_method")) {
                                            p._http_method = 1;
                                            if (!$util.isString(m.http_method))
                                                return "http_method: string expected";
                                        }
                                        if (m.http_request_url != null && m.hasOwnProperty("http_request_url")) {
                                            p._http_request_url = 1;
                                            if (!$util.isString(m.http_request_url))
                                                return "http_request_url: string expected";
                                        }
                                        if (m.http_authorization_header != null && m.hasOwnProperty("http_authorization_header")) {
                                            p._http_authorization_header = 1;
                                            if (!$util.isString(m.http_authorization_header))
                                                return "http_authorization_header: string expected";
                                        }
                                        return null;
                                    };
    
                                    /**
                                     * Creates a OAuthInfo message from a plain object. Also converts values to their respective internal types.
                                     * @function fromObject
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {Object.<string,*>} d Plain object
                                     * @returns {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo} OAuthInfo
                                     */
                                    OAuthInfo.fromObject = function fromObject(d) {
                                        if (d instanceof $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo)
                                            return d;
                                        var m = new $root.google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo();
                                        if (d.http_method != null) {
                                            m.http_method = String(d.http_method);
                                        }
                                        if (d.http_request_url != null) {
                                            m.http_request_url = String(d.http_request_url);
                                        }
                                        if (d.http_authorization_header != null) {
                                            m.http_authorization_header = String(d.http_authorization_header);
                                        }
                                        return m;
                                    };
    
                                    /**
                                     * Creates a plain object from a OAuthInfo message. Also converts values to other types if specified.
                                     * @function toObject
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo} m OAuthInfo
                                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                                     * @returns {Object.<string,*>} Plain object
                                     */
                                    OAuthInfo.toObject = function toObject(m, o) {
                                        if (!o)
                                            o = {};
                                        var d = {};
                                        if (m.http_method != null && m.hasOwnProperty("http_method")) {
                                            d.http_method = m.http_method;
                                            if (o.oneofs)
                                                d._http_method = "http_method";
                                        }
                                        if (m.http_request_url != null && m.hasOwnProperty("http_request_url")) {
                                            d.http_request_url = m.http_request_url;
                                            if (o.oneofs)
                                                d._http_request_url = "http_request_url";
                                        }
                                        if (m.http_authorization_header != null && m.hasOwnProperty("http_authorization_header")) {
                                            d.http_authorization_header = m.http_authorization_header;
                                            if (o.oneofs)
                                                d._http_authorization_header = "http_authorization_header";
                                        }
                                        return d;
                                    };
    
                                    /**
                                     * Converts this OAuthInfo to JSON.
                                     * @function toJSON
                                     * @memberof google.ads.googleads.v11.resources.Feed.PlacesLocationFeedData.OAuthInfo
                                     * @instance
                                     * @returns {Object.<string,*>} JSON object
                                     */
                                    OAuthInfo.prototype.toJSON = function toJSON() {
                                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                    };
    
                                    return OAuthInfo;
                                })();
    
                                return PlacesLocationFeedData;
                            })();
    
                            Feed.AffiliateLocationFeedData = (function() {
    
                                /**
                                 * Properties of an AffiliateLocationFeedData.
                                 * @memberof google.ads.googleads.v11.resources.Feed
                                 * @interface IAffiliateLocationFeedData
                                 * @property {Array.<number>|null} [chain_ids] AffiliateLocationFeedData chain_ids
                                 * @property {google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum.AffiliateLocationFeedRelationshipType|null} [relationship_type] AffiliateLocationFeedData relationship_type
                                 */
    
                                /**
                                 * Constructs a new AffiliateLocationFeedData.
                                 * @memberof google.ads.googleads.v11.resources.Feed
                                 * @classdesc Represents an AffiliateLocationFeedData.
                                 * @implements IAffiliateLocationFeedData
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Feed.IAffiliateLocationFeedData=} [p] Properties to set
                                 */
                                function AffiliateLocationFeedData(p) {
                                    this.chain_ids = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * AffiliateLocationFeedData chain_ids.
                                 * @member {Array.<number>} chain_ids
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @instance
                                 */
                                AffiliateLocationFeedData.prototype.chain_ids = $util.emptyArray;
    
                                /**
                                 * AffiliateLocationFeedData relationship_type.
                                 * @member {google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum.AffiliateLocationFeedRelationshipType} relationship_type
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @instance
                                 */
                                AffiliateLocationFeedData.prototype.relationship_type = 0;
    
                                /**
                                 * Creates a new AffiliateLocationFeedData instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.IAffiliateLocationFeedData=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData} AffiliateLocationFeedData instance
                                 */
                                AffiliateLocationFeedData.create = function create(properties) {
                                    return new AffiliateLocationFeedData(properties);
                                };
    
                                /**
                                 * Encodes the specified AffiliateLocationFeedData message. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.IAffiliateLocationFeedData} m AffiliateLocationFeedData message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AffiliateLocationFeedData.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.relationship_type != null && Object.hasOwnProperty.call(m, "relationship_type"))
                                        w.uint32(16).int32(m.relationship_type);
                                    if (m.chain_ids != null && m.chain_ids.length) {
                                        w.uint32(26).fork();
                                        for (var i = 0; i < m.chain_ids.length; ++i)
                                            w.int64(m.chain_ids[i]);
                                        w.ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified AffiliateLocationFeedData message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.IAffiliateLocationFeedData} message AffiliateLocationFeedData message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AffiliateLocationFeedData.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an AffiliateLocationFeedData message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData} AffiliateLocationFeedData
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AffiliateLocationFeedData.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 3:
                                            if (!(m.chain_ids && m.chain_ids.length))
                                                m.chain_ids = [];
                                            if ((t & 7) === 2) {
                                                var c2 = r.uint32() + r.pos;
                                                while (r.pos < c2)
                                                    m.chain_ids.push(r.int64());
                                            } else
                                                m.chain_ids.push(r.int64());
                                            break;
                                        case 2:
                                            m.relationship_type = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an AffiliateLocationFeedData message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData} AffiliateLocationFeedData
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AffiliateLocationFeedData.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an AffiliateLocationFeedData message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                AffiliateLocationFeedData.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.chain_ids != null && m.hasOwnProperty("chain_ids")) {
                                        if (!Array.isArray(m.chain_ids))
                                            return "chain_ids: array expected";
                                        for (var i = 0; i < m.chain_ids.length; ++i) {
                                            if (!$util.isInteger(m.chain_ids[i]) && !(m.chain_ids[i] && $util.isInteger(m.chain_ids[i].low) && $util.isInteger(m.chain_ids[i].high)))
                                                return "chain_ids: integer|Long[] expected";
                                        }
                                    }
                                    if (m.relationship_type != null && m.hasOwnProperty("relationship_type")) {
                                        switch (m.relationship_type) {
                                        default:
                                            return "relationship_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an AffiliateLocationFeedData message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData} AffiliateLocationFeedData
                                 */
                                AffiliateLocationFeedData.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData();
                                    if (d.chain_ids) {
                                        if (!Array.isArray(d.chain_ids))
                                            throw TypeError(".google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData.chain_ids: array expected");
                                        m.chain_ids = [];
                                        for (var i = 0; i < d.chain_ids.length; ++i) {
                                            if ($util.Long)
                                                (m.chain_ids[i] = $util.Long.fromValue(d.chain_ids[i])).unsigned = false;
                                            else if (typeof d.chain_ids[i] === "string")
                                                m.chain_ids[i] = parseInt(d.chain_ids[i], 10);
                                            else if (typeof d.chain_ids[i] === "number")
                                                m.chain_ids[i] = d.chain_ids[i];
                                            else if (typeof d.chain_ids[i] === "object")
                                                m.chain_ids[i] = new $util.LongBits(d.chain_ids[i].low >>> 0, d.chain_ids[i].high >>> 0).toNumber();
                                        }
                                    }
                                    switch (d.relationship_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.relationship_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.relationship_type = 1;
                                        break;
                                    case "GENERAL_RETAILER":
                                    case 2:
                                        m.relationship_type = 2;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an AffiliateLocationFeedData message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData} m AffiliateLocationFeedData
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                AffiliateLocationFeedData.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.chain_ids = [];
                                    }
                                    if (o.defaults) {
                                        d.relationship_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.relationship_type != null && m.hasOwnProperty("relationship_type")) {
                                        d.relationship_type = o.enums === String ? $root.google.ads.googleads.v11.enums.AffiliateLocationFeedRelationshipTypeEnum.AffiliateLocationFeedRelationshipType[m.relationship_type] : m.relationship_type;
                                    }
                                    if (m.chain_ids && m.chain_ids.length) {
                                        d.chain_ids = [];
                                        for (var j = 0; j < m.chain_ids.length; ++j) {
                                            if (typeof m.chain_ids[j] === "number")
                                                d.chain_ids[j] = o.longs === String ? String(m.chain_ids[j]) : m.chain_ids[j];
                                            else
                                                d.chain_ids[j] = o.longs === String ? $util.Long.prototype.toString.call(m.chain_ids[j]) : o.longs === Number ? new $util.LongBits(m.chain_ids[j].low >>> 0, m.chain_ids[j].high >>> 0).toNumber() : m.chain_ids[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this AffiliateLocationFeedData to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Feed.AffiliateLocationFeedData
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                AffiliateLocationFeedData.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return AffiliateLocationFeedData;
                            })();
    
                            return Feed;
                        })();
    
                        resources.FeedAttribute = (function() {
    
                            /**
                             * Properties of a FeedAttribute.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedAttribute
                             * @property {number|null} [id] FeedAttribute id
                             * @property {string|null} [name] FeedAttribute name
                             * @property {google.ads.googleads.v11.enums.FeedAttributeTypeEnum.FeedAttributeType|null} [type] FeedAttribute type
                             * @property {boolean|null} [is_part_of_key] FeedAttribute is_part_of_key
                             */
    
                            /**
                             * Constructs a new FeedAttribute.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedAttribute.
                             * @implements IFeedAttribute
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedAttribute=} [p] Properties to set
                             */
                            function FeedAttribute(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedAttribute id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             */
                            FeedAttribute.prototype.id = null;
    
                            /**
                             * FeedAttribute name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             */
                            FeedAttribute.prototype.name = null;
    
                            /**
                             * FeedAttribute type.
                             * @member {google.ads.googleads.v11.enums.FeedAttributeTypeEnum.FeedAttributeType} type
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             */
                            FeedAttribute.prototype.type = 0;
    
                            /**
                             * FeedAttribute is_part_of_key.
                             * @member {boolean|null|undefined} is_part_of_key
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             */
                            FeedAttribute.prototype.is_part_of_key = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedAttribute _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             */
                            Object.defineProperty(FeedAttribute.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedAttribute _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             */
                            Object.defineProperty(FeedAttribute.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedAttribute _is_part_of_key.
                             * @member {"is_part_of_key"|undefined} _is_part_of_key
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             */
                            Object.defineProperty(FeedAttribute.prototype, "_is_part_of_key", {
                                get: $util.oneOfGetter($oneOfFields = ["is_part_of_key"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedAttribute instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedAttribute=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedAttribute} FeedAttribute instance
                             */
                            FeedAttribute.create = function create(properties) {
                                return new FeedAttribute(properties);
                            };
    
                            /**
                             * Encodes the specified FeedAttribute message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedAttribute.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedAttribute} m FeedAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttribute.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(40).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(50).string(m.name);
                                if (m.is_part_of_key != null && Object.hasOwnProperty.call(m, "is_part_of_key"))
                                    w.uint32(56).bool(m.is_part_of_key);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedAttribute.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedAttribute} message FeedAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedAttribute message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedAttribute} FeedAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttribute.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedAttribute();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.id = r.int64();
                                        break;
                                    case 6:
                                        m.name = r.string();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 7:
                                        m.is_part_of_key = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedAttribute message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedAttribute} FeedAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttribute.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedAttribute message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedAttribute.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                        break;
                                    }
                                }
                                if (m.is_part_of_key != null && m.hasOwnProperty("is_part_of_key")) {
                                    p._is_part_of_key = 1;
                                    if (typeof m.is_part_of_key !== "boolean")
                                        return "is_part_of_key: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedAttribute message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedAttribute} FeedAttribute
                             */
                            FeedAttribute.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedAttribute)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedAttribute();
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "INT64":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "DOUBLE":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "STRING":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "BOOLEAN":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "URL":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "DATE_TIME":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "INT64_LIST":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "DOUBLE_LIST":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "STRING_LIST":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "BOOLEAN_LIST":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "URL_LIST":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "DATE_TIME_LIST":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "PRICE":
                                case 14:
                                    m.type = 14;
                                    break;
                                }
                                if (d.is_part_of_key != null) {
                                    m.is_part_of_key = Boolean(d.is_part_of_key);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedAttribute message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedAttribute} m FeedAttribute
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedAttribute.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedAttributeTypeEnum.FeedAttributeType[m.type] : m.type;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.is_part_of_key != null && m.hasOwnProperty("is_part_of_key")) {
                                    d.is_part_of_key = m.is_part_of_key;
                                    if (o.oneofs)
                                        d._is_part_of_key = "is_part_of_key";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedAttribute to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedAttribute
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedAttribute.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedAttribute;
                        })();
    
                        resources.FeedAttributeOperation = (function() {
    
                            /**
                             * Properties of a FeedAttributeOperation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedAttributeOperation
                             * @property {google.ads.googleads.v11.resources.FeedAttributeOperation.Operator|null} [operator] FeedAttributeOperation operator
                             * @property {google.ads.googleads.v11.resources.IFeedAttribute|null} [value] FeedAttributeOperation value
                             */
    
                            /**
                             * Constructs a new FeedAttributeOperation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedAttributeOperation.
                             * @implements IFeedAttributeOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedAttributeOperation=} [p] Properties to set
                             */
                            function FeedAttributeOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedAttributeOperation operator.
                             * @member {google.ads.googleads.v11.resources.FeedAttributeOperation.Operator} operator
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @instance
                             */
                            FeedAttributeOperation.prototype.operator = 0;
    
                            /**
                             * FeedAttributeOperation value.
                             * @member {google.ads.googleads.v11.resources.IFeedAttribute|null|undefined} value
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @instance
                             */
                            FeedAttributeOperation.prototype.value = null;
    
                            /**
                             * Creates a new FeedAttributeOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedAttributeOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedAttributeOperation} FeedAttributeOperation instance
                             */
                            FeedAttributeOperation.create = function create(properties) {
                                return new FeedAttributeOperation(properties);
                            };
    
                            /**
                             * Encodes the specified FeedAttributeOperation message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedAttributeOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedAttributeOperation} m FeedAttributeOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttributeOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operator != null && Object.hasOwnProperty.call(m, "operator"))
                                    w.uint32(8).int32(m.operator);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    $root.google.ads.googleads.v11.resources.FeedAttribute.encode(m.value, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedAttributeOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedAttributeOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedAttributeOperation} message FeedAttributeOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedAttributeOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedAttributeOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedAttributeOperation} FeedAttributeOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttributeOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedAttributeOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operator = r.int32();
                                        break;
                                    case 2:
                                        m.value = $root.google.ads.googleads.v11.resources.FeedAttribute.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedAttributeOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedAttributeOperation} FeedAttributeOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedAttributeOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedAttributeOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedAttributeOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    switch (m.operator) {
                                    default:
                                        return "operator: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedAttribute.verify(m.value);
                                        if (e)
                                            return "value." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedAttributeOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedAttributeOperation} FeedAttributeOperation
                             */
                            FeedAttributeOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedAttributeOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedAttributeOperation();
                                switch (d.operator) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator = 1;
                                    break;
                                case "ADD":
                                case 2:
                                    m.operator = 2;
                                    break;
                                }
                                if (d.value != null) {
                                    if (typeof d.value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedAttributeOperation.value: object expected");
                                    m.value = $root.google.ads.googleads.v11.resources.FeedAttribute.fromObject(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedAttributeOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedAttributeOperation} m FeedAttributeOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedAttributeOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.operator = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.value = null;
                                }
                                if (m.operator != null && m.hasOwnProperty("operator")) {
                                    d.operator = o.enums === String ? $root.google.ads.googleads.v11.resources.FeedAttributeOperation.Operator[m.operator] : m.operator;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = $root.google.ads.googleads.v11.resources.FeedAttribute.toObject(m.value, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedAttributeOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedAttributeOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedAttributeOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Operator enum.
                             * @name google.ads.googleads.v11.resources.FeedAttributeOperation.Operator
                             * @enum {number}
                             * @property {number} UNSPECIFIED=0 UNSPECIFIED value
                             * @property {number} UNKNOWN=1 UNKNOWN value
                             * @property {number} ADD=2 ADD value
                             */
                            FeedAttributeOperation.Operator = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "UNSPECIFIED"] = 0;
                                values[valuesById[1] = "UNKNOWN"] = 1;
                                values[valuesById[2] = "ADD"] = 2;
                                return values;
                            })();
    
                            return FeedAttributeOperation;
                        })();
    
                        resources.FeedItem = (function() {
    
                            /**
                             * Properties of a FeedItem.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedItem
                             * @property {string|null} [resource_name] FeedItem resource_name
                             * @property {string|null} [feed] FeedItem feed
                             * @property {number|null} [id] FeedItem id
                             * @property {string|null} [start_date_time] FeedItem start_date_time
                             * @property {string|null} [end_date_time] FeedItem end_date_time
                             * @property {Array.<google.ads.googleads.v11.resources.IFeedItemAttributeValue>|null} [attribute_values] FeedItem attribute_values
                             * @property {google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum.GeoTargetingRestriction|null} [geo_targeting_restriction] FeedItem geo_targeting_restriction
                             * @property {Array.<google.ads.googleads.v11.common.ICustomParameter>|null} [url_custom_parameters] FeedItem url_custom_parameters
                             * @property {google.ads.googleads.v11.enums.FeedItemStatusEnum.FeedItemStatus|null} [status] FeedItem status
                             * @property {Array.<google.ads.googleads.v11.resources.IFeedItemPlaceholderPolicyInfo>|null} [policy_infos] FeedItem policy_infos
                             */
    
                            /**
                             * Constructs a new FeedItem.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedItem.
                             * @implements IFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedItem=} [p] Properties to set
                             */
                            function FeedItem(p) {
                                this.attribute_values = [];
                                this.url_custom_parameters = [];
                                this.policy_infos = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItem resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.resource_name = "";
    
                            /**
                             * FeedItem feed.
                             * @member {string|null|undefined} feed
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.feed = null;
    
                            /**
                             * FeedItem id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.id = null;
    
                            /**
                             * FeedItem start_date_time.
                             * @member {string|null|undefined} start_date_time
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.start_date_time = null;
    
                            /**
                             * FeedItem end_date_time.
                             * @member {string|null|undefined} end_date_time
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.end_date_time = null;
    
                            /**
                             * FeedItem attribute_values.
                             * @member {Array.<google.ads.googleads.v11.resources.IFeedItemAttributeValue>} attribute_values
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.attribute_values = $util.emptyArray;
    
                            /**
                             * FeedItem geo_targeting_restriction.
                             * @member {google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum.GeoTargetingRestriction} geo_targeting_restriction
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.geo_targeting_restriction = 0;
    
                            /**
                             * FeedItem url_custom_parameters.
                             * @member {Array.<google.ads.googleads.v11.common.ICustomParameter>} url_custom_parameters
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.url_custom_parameters = $util.emptyArray;
    
                            /**
                             * FeedItem status.
                             * @member {google.ads.googleads.v11.enums.FeedItemStatusEnum.FeedItemStatus} status
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.status = 0;
    
                            /**
                             * FeedItem policy_infos.
                             * @member {Array.<google.ads.googleads.v11.resources.IFeedItemPlaceholderPolicyInfo>} policy_infos
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            FeedItem.prototype.policy_infos = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItem _feed.
                             * @member {"feed"|undefined} _feed
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            Object.defineProperty(FeedItem.prototype, "_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItem _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            Object.defineProperty(FeedItem.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItem _start_date_time.
                             * @member {"start_date_time"|undefined} _start_date_time
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            Object.defineProperty(FeedItem.prototype, "_start_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItem _end_date_time.
                             * @member {"end_date_time"|undefined} _end_date_time
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             */
                            Object.defineProperty(FeedItem.prototype, "_end_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedItem} FeedItem instance
                             */
                            FeedItem.create = function create(properties) {
                                return new FeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItem message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItem} m FeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.attribute_values != null && m.attribute_values.length) {
                                    for (var i = 0; i < m.attribute_values.length; ++i)
                                        $root.google.ads.googleads.v11.resources.FeedItemAttributeValue.encode(m.attribute_values[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.geo_targeting_restriction != null && Object.hasOwnProperty.call(m, "geo_targeting_restriction"))
                                    w.uint32(56).int32(m.geo_targeting_restriction);
                                if (m.url_custom_parameters != null && m.url_custom_parameters.length) {
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i)
                                        $root.google.ads.googleads.v11.common.CustomParameter.encode(m.url_custom_parameters[i], w.uint32(66).fork()).ldelim();
                                }
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(72).int32(m.status);
                                if (m.policy_infos != null && m.policy_infos.length) {
                                    for (var i = 0; i < m.policy_infos.length; ++i)
                                        $root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.encode(m.policy_infos[i], w.uint32(82).fork()).ldelim();
                                }
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(90).string(m.feed);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(96).int64(m.id);
                                if (m.start_date_time != null && Object.hasOwnProperty.call(m, "start_date_time"))
                                    w.uint32(106).string(m.start_date_time);
                                if (m.end_date_time != null && Object.hasOwnProperty.call(m, "end_date_time"))
                                    w.uint32(114).string(m.end_date_time);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItem} message FeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedItem} FeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 11:
                                        m.feed = r.string();
                                        break;
                                    case 12:
                                        m.id = r.int64();
                                        break;
                                    case 13:
                                        m.start_date_time = r.string();
                                        break;
                                    case 14:
                                        m.end_date_time = r.string();
                                        break;
                                    case 6:
                                        if (!(m.attribute_values && m.attribute_values.length))
                                            m.attribute_values = [];
                                        m.attribute_values.push($root.google.ads.googleads.v11.resources.FeedItemAttributeValue.decode(r, r.uint32()));
                                        break;
                                    case 7:
                                        m.geo_targeting_restriction = r.int32();
                                        break;
                                    case 8:
                                        if (!(m.url_custom_parameters && m.url_custom_parameters.length))
                                            m.url_custom_parameters = [];
                                        m.url_custom_parameters.push($root.google.ads.googleads.v11.common.CustomParameter.decode(r, r.uint32()));
                                        break;
                                    case 9:
                                        m.status = r.int32();
                                        break;
                                    case 10:
                                        if (!(m.policy_infos && m.policy_infos.length))
                                            m.policy_infos = [];
                                        m.policy_infos.push($root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedItem} FeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    p._feed = 1;
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    p._start_date_time = 1;
                                    if (!$util.isString(m.start_date_time))
                                        return "start_date_time: string expected";
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    p._end_date_time = 1;
                                    if (!$util.isString(m.end_date_time))
                                        return "end_date_time: string expected";
                                }
                                if (m.attribute_values != null && m.hasOwnProperty("attribute_values")) {
                                    if (!Array.isArray(m.attribute_values))
                                        return "attribute_values: array expected";
                                    for (var i = 0; i < m.attribute_values.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.FeedItemAttributeValue.verify(m.attribute_values[i]);
                                            if (e)
                                                return "attribute_values." + e;
                                        }
                                    }
                                }
                                if (m.geo_targeting_restriction != null && m.hasOwnProperty("geo_targeting_restriction")) {
                                    switch (m.geo_targeting_restriction) {
                                    default:
                                        return "geo_targeting_restriction: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.url_custom_parameters != null && m.hasOwnProperty("url_custom_parameters")) {
                                    if (!Array.isArray(m.url_custom_parameters))
                                        return "url_custom_parameters: array expected";
                                    for (var i = 0; i < m.url_custom_parameters.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CustomParameter.verify(m.url_custom_parameters[i]);
                                            if (e)
                                                return "url_custom_parameters." + e;
                                        }
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.policy_infos != null && m.hasOwnProperty("policy_infos")) {
                                    if (!Array.isArray(m.policy_infos))
                                        return "policy_infos: array expected";
                                    for (var i = 0; i < m.policy_infos.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.verify(m.policy_infos[i]);
                                            if (e)
                                                return "policy_infos." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedItem} FeedItem
                             */
                            FeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedItem();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.start_date_time != null) {
                                    m.start_date_time = String(d.start_date_time);
                                }
                                if (d.end_date_time != null) {
                                    m.end_date_time = String(d.end_date_time);
                                }
                                if (d.attribute_values) {
                                    if (!Array.isArray(d.attribute_values))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItem.attribute_values: array expected");
                                    m.attribute_values = [];
                                    for (var i = 0; i < d.attribute_values.length; ++i) {
                                        if (typeof d.attribute_values[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.FeedItem.attribute_values: object expected");
                                        m.attribute_values[i] = $root.google.ads.googleads.v11.resources.FeedItemAttributeValue.fromObject(d.attribute_values[i]);
                                    }
                                }
                                switch (d.geo_targeting_restriction) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.geo_targeting_restriction = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.geo_targeting_restriction = 1;
                                    break;
                                case "LOCATION_OF_PRESENCE":
                                case 2:
                                    m.geo_targeting_restriction = 2;
                                    break;
                                }
                                if (d.url_custom_parameters) {
                                    if (!Array.isArray(d.url_custom_parameters))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItem.url_custom_parameters: array expected");
                                    m.url_custom_parameters = [];
                                    for (var i = 0; i < d.url_custom_parameters.length; ++i) {
                                        if (typeof d.url_custom_parameters[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.FeedItem.url_custom_parameters: object expected");
                                        m.url_custom_parameters[i] = $root.google.ads.googleads.v11.common.CustomParameter.fromObject(d.url_custom_parameters[i]);
                                    }
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.policy_infos) {
                                    if (!Array.isArray(d.policy_infos))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItem.policy_infos: array expected");
                                    m.policy_infos = [];
                                    for (var i = 0; i < d.policy_infos.length; ++i) {
                                        if (typeof d.policy_infos[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.FeedItem.policy_infos: object expected");
                                        m.policy_infos[i] = $root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.fromObject(d.policy_infos[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedItem} m FeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.attribute_values = [];
                                    d.url_custom_parameters = [];
                                    d.policy_infos = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.geo_targeting_restriction = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.attribute_values && m.attribute_values.length) {
                                    d.attribute_values = [];
                                    for (var j = 0; j < m.attribute_values.length; ++j) {
                                        d.attribute_values[j] = $root.google.ads.googleads.v11.resources.FeedItemAttributeValue.toObject(m.attribute_values[j], o);
                                    }
                                }
                                if (m.geo_targeting_restriction != null && m.hasOwnProperty("geo_targeting_restriction")) {
                                    d.geo_targeting_restriction = o.enums === String ? $root.google.ads.googleads.v11.enums.GeoTargetingRestrictionEnum.GeoTargetingRestriction[m.geo_targeting_restriction] : m.geo_targeting_restriction;
                                }
                                if (m.url_custom_parameters && m.url_custom_parameters.length) {
                                    d.url_custom_parameters = [];
                                    for (var j = 0; j < m.url_custom_parameters.length; ++j) {
                                        d.url_custom_parameters[j] = $root.google.ads.googleads.v11.common.CustomParameter.toObject(m.url_custom_parameters[j], o);
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemStatusEnum.FeedItemStatus[m.status] : m.status;
                                }
                                if (m.policy_infos && m.policy_infos.length) {
                                    d.policy_infos = [];
                                    for (var j = 0; j < m.policy_infos.length; ++j) {
                                        d.policy_infos[j] = $root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.toObject(m.policy_infos[j], o);
                                    }
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                    if (o.oneofs)
                                        d._feed = "feed";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    d.start_date_time = m.start_date_time;
                                    if (o.oneofs)
                                        d._start_date_time = "start_date_time";
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    d.end_date_time = m.end_date_time;
                                    if (o.oneofs)
                                        d._end_date_time = "end_date_time";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItem;
                        })();
    
                        resources.FeedItemAttributeValue = (function() {
    
                            /**
                             * Properties of a FeedItemAttributeValue.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedItemAttributeValue
                             * @property {number|null} [feed_attribute_id] FeedItemAttributeValue feed_attribute_id
                             * @property {number|null} [integer_value] FeedItemAttributeValue integer_value
                             * @property {boolean|null} [boolean_value] FeedItemAttributeValue boolean_value
                             * @property {string|null} [string_value] FeedItemAttributeValue string_value
                             * @property {number|null} [double_value] FeedItemAttributeValue double_value
                             * @property {google.ads.googleads.v11.common.IMoney|null} [price_value] FeedItemAttributeValue price_value
                             * @property {Array.<number>|null} [integer_values] FeedItemAttributeValue integer_values
                             * @property {Array.<boolean>|null} [boolean_values] FeedItemAttributeValue boolean_values
                             * @property {Array.<string>|null} [string_values] FeedItemAttributeValue string_values
                             * @property {Array.<number>|null} [double_values] FeedItemAttributeValue double_values
                             */
    
                            /**
                             * Constructs a new FeedItemAttributeValue.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedItemAttributeValue.
                             * @implements IFeedItemAttributeValue
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedItemAttributeValue=} [p] Properties to set
                             */
                            function FeedItemAttributeValue(p) {
                                this.integer_values = [];
                                this.boolean_values = [];
                                this.string_values = [];
                                this.double_values = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemAttributeValue feed_attribute_id.
                             * @member {number|null|undefined} feed_attribute_id
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.feed_attribute_id = null;
    
                            /**
                             * FeedItemAttributeValue integer_value.
                             * @member {number|null|undefined} integer_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.integer_value = null;
    
                            /**
                             * FeedItemAttributeValue boolean_value.
                             * @member {boolean|null|undefined} boolean_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.boolean_value = null;
    
                            /**
                             * FeedItemAttributeValue string_value.
                             * @member {string|null|undefined} string_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.string_value = null;
    
                            /**
                             * FeedItemAttributeValue double_value.
                             * @member {number|null|undefined} double_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.double_value = null;
    
                            /**
                             * FeedItemAttributeValue price_value.
                             * @member {google.ads.googleads.v11.common.IMoney|null|undefined} price_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.price_value = null;
    
                            /**
                             * FeedItemAttributeValue integer_values.
                             * @member {Array.<number>} integer_values
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.integer_values = $util.emptyArray;
    
                            /**
                             * FeedItemAttributeValue boolean_values.
                             * @member {Array.<boolean>} boolean_values
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.boolean_values = $util.emptyArray;
    
                            /**
                             * FeedItemAttributeValue string_values.
                             * @member {Array.<string>} string_values
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.string_values = $util.emptyArray;
    
                            /**
                             * FeedItemAttributeValue double_values.
                             * @member {Array.<number>} double_values
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            FeedItemAttributeValue.prototype.double_values = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemAttributeValue _feed_attribute_id.
                             * @member {"feed_attribute_id"|undefined} _feed_attribute_id
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            Object.defineProperty(FeedItemAttributeValue.prototype, "_feed_attribute_id", {
                                get: $util.oneOfGetter($oneOfFields = ["feed_attribute_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItemAttributeValue _integer_value.
                             * @member {"integer_value"|undefined} _integer_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            Object.defineProperty(FeedItemAttributeValue.prototype, "_integer_value", {
                                get: $util.oneOfGetter($oneOfFields = ["integer_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItemAttributeValue _boolean_value.
                             * @member {"boolean_value"|undefined} _boolean_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            Object.defineProperty(FeedItemAttributeValue.prototype, "_boolean_value", {
                                get: $util.oneOfGetter($oneOfFields = ["boolean_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItemAttributeValue _string_value.
                             * @member {"string_value"|undefined} _string_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            Object.defineProperty(FeedItemAttributeValue.prototype, "_string_value", {
                                get: $util.oneOfGetter($oneOfFields = ["string_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItemAttributeValue _double_value.
                             * @member {"double_value"|undefined} _double_value
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             */
                            Object.defineProperty(FeedItemAttributeValue.prototype, "_double_value", {
                                get: $util.oneOfGetter($oneOfFields = ["double_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemAttributeValue instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemAttributeValue=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedItemAttributeValue} FeedItemAttributeValue instance
                             */
                            FeedItemAttributeValue.create = function create(properties) {
                                return new FeedItemAttributeValue(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemAttributeValue message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemAttributeValue.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemAttributeValue} m FeedItemAttributeValue message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemAttributeValue.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.price_value != null && Object.hasOwnProperty.call(m, "price_value"))
                                    $root.google.ads.googleads.v11.common.Money.encode(m.price_value, w.uint32(50).fork()).ldelim();
                                if (m.feed_attribute_id != null && Object.hasOwnProperty.call(m, "feed_attribute_id"))
                                    w.uint32(88).int64(m.feed_attribute_id);
                                if (m.integer_value != null && Object.hasOwnProperty.call(m, "integer_value"))
                                    w.uint32(96).int64(m.integer_value);
                                if (m.boolean_value != null && Object.hasOwnProperty.call(m, "boolean_value"))
                                    w.uint32(104).bool(m.boolean_value);
                                if (m.string_value != null && Object.hasOwnProperty.call(m, "string_value"))
                                    w.uint32(114).string(m.string_value);
                                if (m.double_value != null && Object.hasOwnProperty.call(m, "double_value"))
                                    w.uint32(121).double(m.double_value);
                                if (m.integer_values != null && m.integer_values.length) {
                                    w.uint32(130).fork();
                                    for (var i = 0; i < m.integer_values.length; ++i)
                                        w.int64(m.integer_values[i]);
                                    w.ldelim();
                                }
                                if (m.boolean_values != null && m.boolean_values.length) {
                                    w.uint32(138).fork();
                                    for (var i = 0; i < m.boolean_values.length; ++i)
                                        w.bool(m.boolean_values[i]);
                                    w.ldelim();
                                }
                                if (m.string_values != null && m.string_values.length) {
                                    for (var i = 0; i < m.string_values.length; ++i)
                                        w.uint32(146).string(m.string_values[i]);
                                }
                                if (m.double_values != null && m.double_values.length) {
                                    w.uint32(154).fork();
                                    for (var i = 0; i < m.double_values.length; ++i)
                                        w.double(m.double_values[i]);
                                    w.ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemAttributeValue message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemAttributeValue.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemAttributeValue} message FeedItemAttributeValue message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemAttributeValue.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemAttributeValue message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedItemAttributeValue} FeedItemAttributeValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemAttributeValue.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedItemAttributeValue();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 11:
                                        m.feed_attribute_id = r.int64();
                                        break;
                                    case 12:
                                        m.integer_value = r.int64();
                                        break;
                                    case 13:
                                        m.boolean_value = r.bool();
                                        break;
                                    case 14:
                                        m.string_value = r.string();
                                        break;
                                    case 15:
                                        m.double_value = r.double();
                                        break;
                                    case 6:
                                        m.price_value = $root.google.ads.googleads.v11.common.Money.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        if (!(m.integer_values && m.integer_values.length))
                                            m.integer_values = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.integer_values.push(r.int64());
                                        } else
                                            m.integer_values.push(r.int64());
                                        break;
                                    case 17:
                                        if (!(m.boolean_values && m.boolean_values.length))
                                            m.boolean_values = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.boolean_values.push(r.bool());
                                        } else
                                            m.boolean_values.push(r.bool());
                                        break;
                                    case 18:
                                        if (!(m.string_values && m.string_values.length))
                                            m.string_values = [];
                                        m.string_values.push(r.string());
                                        break;
                                    case 19:
                                        if (!(m.double_values && m.double_values.length))
                                            m.double_values = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.double_values.push(r.double());
                                        } else
                                            m.double_values.push(r.double());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemAttributeValue message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedItemAttributeValue} FeedItemAttributeValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemAttributeValue.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemAttributeValue message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemAttributeValue.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.feed_attribute_id != null && m.hasOwnProperty("feed_attribute_id")) {
                                    p._feed_attribute_id = 1;
                                    if (!$util.isInteger(m.feed_attribute_id) && !(m.feed_attribute_id && $util.isInteger(m.feed_attribute_id.low) && $util.isInteger(m.feed_attribute_id.high)))
                                        return "feed_attribute_id: integer|Long expected";
                                }
                                if (m.integer_value != null && m.hasOwnProperty("integer_value")) {
                                    p._integer_value = 1;
                                    if (!$util.isInteger(m.integer_value) && !(m.integer_value && $util.isInteger(m.integer_value.low) && $util.isInteger(m.integer_value.high)))
                                        return "integer_value: integer|Long expected";
                                }
                                if (m.boolean_value != null && m.hasOwnProperty("boolean_value")) {
                                    p._boolean_value = 1;
                                    if (typeof m.boolean_value !== "boolean")
                                        return "boolean_value: boolean expected";
                                }
                                if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                    p._string_value = 1;
                                    if (!$util.isString(m.string_value))
                                        return "string_value: string expected";
                                }
                                if (m.double_value != null && m.hasOwnProperty("double_value")) {
                                    p._double_value = 1;
                                    if (typeof m.double_value !== "number")
                                        return "double_value: number expected";
                                }
                                if (m.price_value != null && m.hasOwnProperty("price_value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Money.verify(m.price_value);
                                        if (e)
                                            return "price_value." + e;
                                    }
                                }
                                if (m.integer_values != null && m.hasOwnProperty("integer_values")) {
                                    if (!Array.isArray(m.integer_values))
                                        return "integer_values: array expected";
                                    for (var i = 0; i < m.integer_values.length; ++i) {
                                        if (!$util.isInteger(m.integer_values[i]) && !(m.integer_values[i] && $util.isInteger(m.integer_values[i].low) && $util.isInteger(m.integer_values[i].high)))
                                            return "integer_values: integer|Long[] expected";
                                    }
                                }
                                if (m.boolean_values != null && m.hasOwnProperty("boolean_values")) {
                                    if (!Array.isArray(m.boolean_values))
                                        return "boolean_values: array expected";
                                    for (var i = 0; i < m.boolean_values.length; ++i) {
                                        if (typeof m.boolean_values[i] !== "boolean")
                                            return "boolean_values: boolean[] expected";
                                    }
                                }
                                if (m.string_values != null && m.hasOwnProperty("string_values")) {
                                    if (!Array.isArray(m.string_values))
                                        return "string_values: array expected";
                                    for (var i = 0; i < m.string_values.length; ++i) {
                                        if (!$util.isString(m.string_values[i]))
                                            return "string_values: string[] expected";
                                    }
                                }
                                if (m.double_values != null && m.hasOwnProperty("double_values")) {
                                    if (!Array.isArray(m.double_values))
                                        return "double_values: array expected";
                                    for (var i = 0; i < m.double_values.length; ++i) {
                                        if (typeof m.double_values[i] !== "number")
                                            return "double_values: number[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemAttributeValue message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedItemAttributeValue} FeedItemAttributeValue
                             */
                            FeedItemAttributeValue.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedItemAttributeValue)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedItemAttributeValue();
                                if (d.feed_attribute_id != null) {
                                    if ($util.Long)
                                        (m.feed_attribute_id = $util.Long.fromValue(d.feed_attribute_id)).unsigned = false;
                                    else if (typeof d.feed_attribute_id === "string")
                                        m.feed_attribute_id = parseInt(d.feed_attribute_id, 10);
                                    else if (typeof d.feed_attribute_id === "number")
                                        m.feed_attribute_id = d.feed_attribute_id;
                                    else if (typeof d.feed_attribute_id === "object")
                                        m.feed_attribute_id = new $util.LongBits(d.feed_attribute_id.low >>> 0, d.feed_attribute_id.high >>> 0).toNumber();
                                }
                                if (d.integer_value != null) {
                                    if ($util.Long)
                                        (m.integer_value = $util.Long.fromValue(d.integer_value)).unsigned = false;
                                    else if (typeof d.integer_value === "string")
                                        m.integer_value = parseInt(d.integer_value, 10);
                                    else if (typeof d.integer_value === "number")
                                        m.integer_value = d.integer_value;
                                    else if (typeof d.integer_value === "object")
                                        m.integer_value = new $util.LongBits(d.integer_value.low >>> 0, d.integer_value.high >>> 0).toNumber();
                                }
                                if (d.boolean_value != null) {
                                    m.boolean_value = Boolean(d.boolean_value);
                                }
                                if (d.string_value != null) {
                                    m.string_value = String(d.string_value);
                                }
                                if (d.double_value != null) {
                                    m.double_value = Number(d.double_value);
                                }
                                if (d.price_value != null) {
                                    if (typeof d.price_value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemAttributeValue.price_value: object expected");
                                    m.price_value = $root.google.ads.googleads.v11.common.Money.fromObject(d.price_value);
                                }
                                if (d.integer_values) {
                                    if (!Array.isArray(d.integer_values))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemAttributeValue.integer_values: array expected");
                                    m.integer_values = [];
                                    for (var i = 0; i < d.integer_values.length; ++i) {
                                        if ($util.Long)
                                            (m.integer_values[i] = $util.Long.fromValue(d.integer_values[i])).unsigned = false;
                                        else if (typeof d.integer_values[i] === "string")
                                            m.integer_values[i] = parseInt(d.integer_values[i], 10);
                                        else if (typeof d.integer_values[i] === "number")
                                            m.integer_values[i] = d.integer_values[i];
                                        else if (typeof d.integer_values[i] === "object")
                                            m.integer_values[i] = new $util.LongBits(d.integer_values[i].low >>> 0, d.integer_values[i].high >>> 0).toNumber();
                                    }
                                }
                                if (d.boolean_values) {
                                    if (!Array.isArray(d.boolean_values))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemAttributeValue.boolean_values: array expected");
                                    m.boolean_values = [];
                                    for (var i = 0; i < d.boolean_values.length; ++i) {
                                        m.boolean_values[i] = Boolean(d.boolean_values[i]);
                                    }
                                }
                                if (d.string_values) {
                                    if (!Array.isArray(d.string_values))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemAttributeValue.string_values: array expected");
                                    m.string_values = [];
                                    for (var i = 0; i < d.string_values.length; ++i) {
                                        m.string_values[i] = String(d.string_values[i]);
                                    }
                                }
                                if (d.double_values) {
                                    if (!Array.isArray(d.double_values))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemAttributeValue.double_values: array expected");
                                    m.double_values = [];
                                    for (var i = 0; i < d.double_values.length; ++i) {
                                        m.double_values[i] = Number(d.double_values[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemAttributeValue message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedItemAttributeValue} m FeedItemAttributeValue
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemAttributeValue.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.integer_values = [];
                                    d.boolean_values = [];
                                    d.string_values = [];
                                    d.double_values = [];
                                }
                                if (o.defaults) {
                                    d.price_value = null;
                                }
                                if (m.price_value != null && m.hasOwnProperty("price_value")) {
                                    d.price_value = $root.google.ads.googleads.v11.common.Money.toObject(m.price_value, o);
                                }
                                if (m.feed_attribute_id != null && m.hasOwnProperty("feed_attribute_id")) {
                                    if (typeof m.feed_attribute_id === "number")
                                        d.feed_attribute_id = o.longs === String ? String(m.feed_attribute_id) : m.feed_attribute_id;
                                    else
                                        d.feed_attribute_id = o.longs === String ? $util.Long.prototype.toString.call(m.feed_attribute_id) : o.longs === Number ? new $util.LongBits(m.feed_attribute_id.low >>> 0, m.feed_attribute_id.high >>> 0).toNumber() : m.feed_attribute_id;
                                    if (o.oneofs)
                                        d._feed_attribute_id = "feed_attribute_id";
                                }
                                if (m.integer_value != null && m.hasOwnProperty("integer_value")) {
                                    if (typeof m.integer_value === "number")
                                        d.integer_value = o.longs === String ? String(m.integer_value) : m.integer_value;
                                    else
                                        d.integer_value = o.longs === String ? $util.Long.prototype.toString.call(m.integer_value) : o.longs === Number ? new $util.LongBits(m.integer_value.low >>> 0, m.integer_value.high >>> 0).toNumber() : m.integer_value;
                                    if (o.oneofs)
                                        d._integer_value = "integer_value";
                                }
                                if (m.boolean_value != null && m.hasOwnProperty("boolean_value")) {
                                    d.boolean_value = m.boolean_value;
                                    if (o.oneofs)
                                        d._boolean_value = "boolean_value";
                                }
                                if (m.string_value != null && m.hasOwnProperty("string_value")) {
                                    d.string_value = m.string_value;
                                    if (o.oneofs)
                                        d._string_value = "string_value";
                                }
                                if (m.double_value != null && m.hasOwnProperty("double_value")) {
                                    d.double_value = o.json && !isFinite(m.double_value) ? String(m.double_value) : m.double_value;
                                    if (o.oneofs)
                                        d._double_value = "double_value";
                                }
                                if (m.integer_values && m.integer_values.length) {
                                    d.integer_values = [];
                                    for (var j = 0; j < m.integer_values.length; ++j) {
                                        if (typeof m.integer_values[j] === "number")
                                            d.integer_values[j] = o.longs === String ? String(m.integer_values[j]) : m.integer_values[j];
                                        else
                                            d.integer_values[j] = o.longs === String ? $util.Long.prototype.toString.call(m.integer_values[j]) : o.longs === Number ? new $util.LongBits(m.integer_values[j].low >>> 0, m.integer_values[j].high >>> 0).toNumber() : m.integer_values[j];
                                    }
                                }
                                if (m.boolean_values && m.boolean_values.length) {
                                    d.boolean_values = [];
                                    for (var j = 0; j < m.boolean_values.length; ++j) {
                                        d.boolean_values[j] = m.boolean_values[j];
                                    }
                                }
                                if (m.string_values && m.string_values.length) {
                                    d.string_values = [];
                                    for (var j = 0; j < m.string_values.length; ++j) {
                                        d.string_values[j] = m.string_values[j];
                                    }
                                }
                                if (m.double_values && m.double_values.length) {
                                    d.double_values = [];
                                    for (var j = 0; j < m.double_values.length; ++j) {
                                        d.double_values[j] = o.json && !isFinite(m.double_values[j]) ? String(m.double_values[j]) : m.double_values[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemAttributeValue to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedItemAttributeValue
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemAttributeValue.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemAttributeValue;
                        })();
    
                        resources.FeedItemPlaceholderPolicyInfo = (function() {
    
                            /**
                             * Properties of a FeedItemPlaceholderPolicyInfo.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedItemPlaceholderPolicyInfo
                             * @property {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType|null} [placeholder_type_enum] FeedItemPlaceholderPolicyInfo placeholder_type_enum
                             * @property {string|null} [feed_mapping_resource_name] FeedItemPlaceholderPolicyInfo feed_mapping_resource_name
                             * @property {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus|null} [review_status] FeedItemPlaceholderPolicyInfo review_status
                             * @property {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus|null} [approval_status] FeedItemPlaceholderPolicyInfo approval_status
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>|null} [policy_topic_entries] FeedItemPlaceholderPolicyInfo policy_topic_entries
                             * @property {google.ads.googleads.v11.enums.FeedItemValidationStatusEnum.FeedItemValidationStatus|null} [validation_status] FeedItemPlaceholderPolicyInfo validation_status
                             * @property {Array.<google.ads.googleads.v11.resources.IFeedItemValidationError>|null} [validation_errors] FeedItemPlaceholderPolicyInfo validation_errors
                             * @property {google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum.FeedItemQualityApprovalStatus|null} [quality_approval_status] FeedItemPlaceholderPolicyInfo quality_approval_status
                             * @property {Array.<google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum.FeedItemQualityDisapprovalReason>|null} [quality_disapproval_reasons] FeedItemPlaceholderPolicyInfo quality_disapproval_reasons
                             */
    
                            /**
                             * Constructs a new FeedItemPlaceholderPolicyInfo.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedItemPlaceholderPolicyInfo.
                             * @implements IFeedItemPlaceholderPolicyInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedItemPlaceholderPolicyInfo=} [p] Properties to set
                             */
                            function FeedItemPlaceholderPolicyInfo(p) {
                                this.policy_topic_entries = [];
                                this.validation_errors = [];
                                this.quality_disapproval_reasons = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemPlaceholderPolicyInfo placeholder_type_enum.
                             * @member {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType} placeholder_type_enum
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.placeholder_type_enum = 0;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo feed_mapping_resource_name.
                             * @member {string|null|undefined} feed_mapping_resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.feed_mapping_resource_name = null;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo review_status.
                             * @member {google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus} review_status
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.review_status = 0;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo approval_status.
                             * @member {google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus} approval_status
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.approval_status = 0;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo policy_topic_entries.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyTopicEntry>} policy_topic_entries
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.policy_topic_entries = $util.emptyArray;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo validation_status.
                             * @member {google.ads.googleads.v11.enums.FeedItemValidationStatusEnum.FeedItemValidationStatus} validation_status
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.validation_status = 0;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo validation_errors.
                             * @member {Array.<google.ads.googleads.v11.resources.IFeedItemValidationError>} validation_errors
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.validation_errors = $util.emptyArray;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo quality_approval_status.
                             * @member {google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum.FeedItemQualityApprovalStatus} quality_approval_status
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.quality_approval_status = 0;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo quality_disapproval_reasons.
                             * @member {Array.<google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum.FeedItemQualityDisapprovalReason>} quality_disapproval_reasons
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.quality_disapproval_reasons = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemPlaceholderPolicyInfo _feed_mapping_resource_name.
                             * @member {"feed_mapping_resource_name"|undefined} _feed_mapping_resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             */
                            Object.defineProperty(FeedItemPlaceholderPolicyInfo.prototype, "_feed_mapping_resource_name", {
                                get: $util.oneOfGetter($oneOfFields = ["feed_mapping_resource_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemPlaceholderPolicyInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemPlaceholderPolicyInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo} FeedItemPlaceholderPolicyInfo instance
                             */
                            FeedItemPlaceholderPolicyInfo.create = function create(properties) {
                                return new FeedItemPlaceholderPolicyInfo(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemPlaceholderPolicyInfo message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemPlaceholderPolicyInfo} m FeedItemPlaceholderPolicyInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemPlaceholderPolicyInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.review_status != null && Object.hasOwnProperty.call(m, "review_status"))
                                    w.uint32(24).int32(m.review_status);
                                if (m.approval_status != null && Object.hasOwnProperty.call(m, "approval_status"))
                                    w.uint32(32).int32(m.approval_status);
                                if (m.policy_topic_entries != null && m.policy_topic_entries.length) {
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyTopicEntry.encode(m.policy_topic_entries[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.validation_status != null && Object.hasOwnProperty.call(m, "validation_status"))
                                    w.uint32(48).int32(m.validation_status);
                                if (m.validation_errors != null && m.validation_errors.length) {
                                    for (var i = 0; i < m.validation_errors.length; ++i)
                                        $root.google.ads.googleads.v11.resources.FeedItemValidationError.encode(m.validation_errors[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.quality_approval_status != null && Object.hasOwnProperty.call(m, "quality_approval_status"))
                                    w.uint32(64).int32(m.quality_approval_status);
                                if (m.quality_disapproval_reasons != null && m.quality_disapproval_reasons.length) {
                                    w.uint32(74).fork();
                                    for (var i = 0; i < m.quality_disapproval_reasons.length; ++i)
                                        w.int32(m.quality_disapproval_reasons[i]);
                                    w.ldelim();
                                }
                                if (m.placeholder_type_enum != null && Object.hasOwnProperty.call(m, "placeholder_type_enum"))
                                    w.uint32(80).int32(m.placeholder_type_enum);
                                if (m.feed_mapping_resource_name != null && Object.hasOwnProperty.call(m, "feed_mapping_resource_name"))
                                    w.uint32(90).string(m.feed_mapping_resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemPlaceholderPolicyInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemPlaceholderPolicyInfo} message FeedItemPlaceholderPolicyInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemPlaceholderPolicyInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemPlaceholderPolicyInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo} FeedItemPlaceholderPolicyInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemPlaceholderPolicyInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 10:
                                        m.placeholder_type_enum = r.int32();
                                        break;
                                    case 11:
                                        m.feed_mapping_resource_name = r.string();
                                        break;
                                    case 3:
                                        m.review_status = r.int32();
                                        break;
                                    case 4:
                                        m.approval_status = r.int32();
                                        break;
                                    case 5:
                                        if (!(m.policy_topic_entries && m.policy_topic_entries.length))
                                            m.policy_topic_entries = [];
                                        m.policy_topic_entries.push($root.google.ads.googleads.v11.common.PolicyTopicEntry.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        m.validation_status = r.int32();
                                        break;
                                    case 7:
                                        if (!(m.validation_errors && m.validation_errors.length))
                                            m.validation_errors = [];
                                        m.validation_errors.push($root.google.ads.googleads.v11.resources.FeedItemValidationError.decode(r, r.uint32()));
                                        break;
                                    case 8:
                                        m.quality_approval_status = r.int32();
                                        break;
                                    case 9:
                                        if (!(m.quality_disapproval_reasons && m.quality_disapproval_reasons.length))
                                            m.quality_disapproval_reasons = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.quality_disapproval_reasons.push(r.int32());
                                        } else
                                            m.quality_disapproval_reasons.push(r.int32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemPlaceholderPolicyInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo} FeedItemPlaceholderPolicyInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemPlaceholderPolicyInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemPlaceholderPolicyInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemPlaceholderPolicyInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.placeholder_type_enum != null && m.hasOwnProperty("placeholder_type_enum")) {
                                    switch (m.placeholder_type_enum) {
                                    default:
                                        return "placeholder_type_enum: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.feed_mapping_resource_name != null && m.hasOwnProperty("feed_mapping_resource_name")) {
                                    p._feed_mapping_resource_name = 1;
                                    if (!$util.isString(m.feed_mapping_resource_name))
                                        return "feed_mapping_resource_name: string expected";
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    switch (m.review_status) {
                                    default:
                                        return "review_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    switch (m.approval_status) {
                                    default:
                                        return "approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.policy_topic_entries != null && m.hasOwnProperty("policy_topic_entries")) {
                                    if (!Array.isArray(m.policy_topic_entries))
                                        return "policy_topic_entries: array expected";
                                    for (var i = 0; i < m.policy_topic_entries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyTopicEntry.verify(m.policy_topic_entries[i]);
                                            if (e)
                                                return "policy_topic_entries." + e;
                                        }
                                    }
                                }
                                if (m.validation_status != null && m.hasOwnProperty("validation_status")) {
                                    switch (m.validation_status) {
                                    default:
                                        return "validation_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.validation_errors != null && m.hasOwnProperty("validation_errors")) {
                                    if (!Array.isArray(m.validation_errors))
                                        return "validation_errors: array expected";
                                    for (var i = 0; i < m.validation_errors.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.FeedItemValidationError.verify(m.validation_errors[i]);
                                            if (e)
                                                return "validation_errors." + e;
                                        }
                                    }
                                }
                                if (m.quality_approval_status != null && m.hasOwnProperty("quality_approval_status")) {
                                    switch (m.quality_approval_status) {
                                    default:
                                        return "quality_approval_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.quality_disapproval_reasons != null && m.hasOwnProperty("quality_disapproval_reasons")) {
                                    if (!Array.isArray(m.quality_disapproval_reasons))
                                        return "quality_disapproval_reasons: array expected";
                                    for (var i = 0; i < m.quality_disapproval_reasons.length; ++i) {
                                        switch (m.quality_disapproval_reasons[i]) {
                                        default:
                                            return "quality_disapproval_reasons: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 17:
                                        case 18:
                                        case 19:
                                            break;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemPlaceholderPolicyInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo} FeedItemPlaceholderPolicyInfo
                             */
                            FeedItemPlaceholderPolicyInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo();
                                switch (d.placeholder_type_enum) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.placeholder_type_enum = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.placeholder_type_enum = 1;
                                    break;
                                case "SITELINK":
                                case 2:
                                    m.placeholder_type_enum = 2;
                                    break;
                                case "CALL":
                                case 3:
                                    m.placeholder_type_enum = 3;
                                    break;
                                case "APP":
                                case 4:
                                    m.placeholder_type_enum = 4;
                                    break;
                                case "LOCATION":
                                case 5:
                                    m.placeholder_type_enum = 5;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 6:
                                    m.placeholder_type_enum = 6;
                                    break;
                                case "CALLOUT":
                                case 7:
                                    m.placeholder_type_enum = 7;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 8:
                                    m.placeholder_type_enum = 8;
                                    break;
                                case "MESSAGE":
                                case 9:
                                    m.placeholder_type_enum = 9;
                                    break;
                                case "PRICE":
                                case 10:
                                    m.placeholder_type_enum = 10;
                                    break;
                                case "PROMOTION":
                                case 11:
                                    m.placeholder_type_enum = 11;
                                    break;
                                case "AD_CUSTOMIZER":
                                case 12:
                                    m.placeholder_type_enum = 12;
                                    break;
                                case "DYNAMIC_EDUCATION":
                                case 13:
                                    m.placeholder_type_enum = 13;
                                    break;
                                case "DYNAMIC_FLIGHT":
                                case 14:
                                    m.placeholder_type_enum = 14;
                                    break;
                                case "DYNAMIC_CUSTOM":
                                case 15:
                                    m.placeholder_type_enum = 15;
                                    break;
                                case "DYNAMIC_HOTEL":
                                case 16:
                                    m.placeholder_type_enum = 16;
                                    break;
                                case "DYNAMIC_REAL_ESTATE":
                                case 17:
                                    m.placeholder_type_enum = 17;
                                    break;
                                case "DYNAMIC_TRAVEL":
                                case 18:
                                    m.placeholder_type_enum = 18;
                                    break;
                                case "DYNAMIC_LOCAL":
                                case 19:
                                    m.placeholder_type_enum = 19;
                                    break;
                                case "DYNAMIC_JOB":
                                case 20:
                                    m.placeholder_type_enum = 20;
                                    break;
                                case "IMAGE":
                                case 21:
                                    m.placeholder_type_enum = 21;
                                    break;
                                }
                                if (d.feed_mapping_resource_name != null) {
                                    m.feed_mapping_resource_name = String(d.feed_mapping_resource_name);
                                }
                                switch (d.review_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.review_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.review_status = 1;
                                    break;
                                case "REVIEW_IN_PROGRESS":
                                case 2:
                                    m.review_status = 2;
                                    break;
                                case "REVIEWED":
                                case 3:
                                    m.review_status = 3;
                                    break;
                                case "UNDER_APPEAL":
                                case 4:
                                    m.review_status = 4;
                                    break;
                                case "ELIGIBLE_MAY_SERVE":
                                case 5:
                                    m.review_status = 5;
                                    break;
                                }
                                switch (d.approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.approval_status = 1;
                                    break;
                                case "DISAPPROVED":
                                case 2:
                                    m.approval_status = 2;
                                    break;
                                case "APPROVED_LIMITED":
                                case 3:
                                    m.approval_status = 3;
                                    break;
                                case "APPROVED":
                                case 4:
                                    m.approval_status = 4;
                                    break;
                                case "AREA_OF_INTEREST_ONLY":
                                case 5:
                                    m.approval_status = 5;
                                    break;
                                }
                                if (d.policy_topic_entries) {
                                    if (!Array.isArray(d.policy_topic_entries))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.policy_topic_entries: array expected");
                                    m.policy_topic_entries = [];
                                    for (var i = 0; i < d.policy_topic_entries.length; ++i) {
                                        if (typeof d.policy_topic_entries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.policy_topic_entries: object expected");
                                        m.policy_topic_entries[i] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.fromObject(d.policy_topic_entries[i]);
                                    }
                                }
                                switch (d.validation_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.validation_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.validation_status = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.validation_status = 2;
                                    break;
                                case "INVALID":
                                case 3:
                                    m.validation_status = 3;
                                    break;
                                case "VALID":
                                case 4:
                                    m.validation_status = 4;
                                    break;
                                }
                                if (d.validation_errors) {
                                    if (!Array.isArray(d.validation_errors))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.validation_errors: array expected");
                                    m.validation_errors = [];
                                    for (var i = 0; i < d.validation_errors.length; ++i) {
                                        if (typeof d.validation_errors[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.validation_errors: object expected");
                                        m.validation_errors[i] = $root.google.ads.googleads.v11.resources.FeedItemValidationError.fromObject(d.validation_errors[i]);
                                    }
                                }
                                switch (d.quality_approval_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.quality_approval_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.quality_approval_status = 1;
                                    break;
                                case "APPROVED":
                                case 2:
                                    m.quality_approval_status = 2;
                                    break;
                                case "DISAPPROVED":
                                case 3:
                                    m.quality_approval_status = 3;
                                    break;
                                }
                                if (d.quality_disapproval_reasons) {
                                    if (!Array.isArray(d.quality_disapproval_reasons))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo.quality_disapproval_reasons: array expected");
                                    m.quality_disapproval_reasons = [];
                                    for (var i = 0; i < d.quality_disapproval_reasons.length; ++i) {
                                        switch (d.quality_disapproval_reasons[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.quality_disapproval_reasons[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.quality_disapproval_reasons[i] = 1;
                                            break;
                                        case "PRICE_TABLE_REPETITIVE_HEADERS":
                                        case 2:
                                            m.quality_disapproval_reasons[i] = 2;
                                            break;
                                        case "PRICE_TABLE_REPETITIVE_DESCRIPTION":
                                        case 3:
                                            m.quality_disapproval_reasons[i] = 3;
                                            break;
                                        case "PRICE_TABLE_INCONSISTENT_ROWS":
                                        case 4:
                                            m.quality_disapproval_reasons[i] = 4;
                                            break;
                                        case "PRICE_DESCRIPTION_HAS_PRICE_QUALIFIERS":
                                        case 5:
                                            m.quality_disapproval_reasons[i] = 5;
                                            break;
                                        case "PRICE_UNSUPPORTED_LANGUAGE":
                                        case 6:
                                            m.quality_disapproval_reasons[i] = 6;
                                            break;
                                        case "PRICE_TABLE_ROW_HEADER_TABLE_TYPE_MISMATCH":
                                        case 7:
                                            m.quality_disapproval_reasons[i] = 7;
                                            break;
                                        case "PRICE_TABLE_ROW_HEADER_HAS_PROMOTIONAL_TEXT":
                                        case 8:
                                            m.quality_disapproval_reasons[i] = 8;
                                            break;
                                        case "PRICE_TABLE_ROW_DESCRIPTION_NOT_RELEVANT":
                                        case 9:
                                            m.quality_disapproval_reasons[i] = 9;
                                            break;
                                        case "PRICE_TABLE_ROW_DESCRIPTION_HAS_PROMOTIONAL_TEXT":
                                        case 10:
                                            m.quality_disapproval_reasons[i] = 10;
                                            break;
                                        case "PRICE_TABLE_ROW_HEADER_DESCRIPTION_REPETITIVE":
                                        case 11:
                                            m.quality_disapproval_reasons[i] = 11;
                                            break;
                                        case "PRICE_TABLE_ROW_UNRATEABLE":
                                        case 12:
                                            m.quality_disapproval_reasons[i] = 12;
                                            break;
                                        case "PRICE_TABLE_ROW_PRICE_INVALID":
                                        case 13:
                                            m.quality_disapproval_reasons[i] = 13;
                                            break;
                                        case "PRICE_TABLE_ROW_URL_INVALID":
                                        case 14:
                                            m.quality_disapproval_reasons[i] = 14;
                                            break;
                                        case "PRICE_HEADER_OR_DESCRIPTION_HAS_PRICE":
                                        case 15:
                                            m.quality_disapproval_reasons[i] = 15;
                                            break;
                                        case "STRUCTURED_SNIPPETS_HEADER_POLICY_VIOLATED":
                                        case 16:
                                            m.quality_disapproval_reasons[i] = 16;
                                            break;
                                        case "STRUCTURED_SNIPPETS_REPEATED_VALUES":
                                        case 17:
                                            m.quality_disapproval_reasons[i] = 17;
                                            break;
                                        case "STRUCTURED_SNIPPETS_EDITORIAL_GUIDELINES":
                                        case 18:
                                            m.quality_disapproval_reasons[i] = 18;
                                            break;
                                        case "STRUCTURED_SNIPPETS_HAS_PROMOTIONAL_TEXT":
                                        case 19:
                                            m.quality_disapproval_reasons[i] = 19;
                                            break;
                                        }
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemPlaceholderPolicyInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo} m FeedItemPlaceholderPolicyInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemPlaceholderPolicyInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.policy_topic_entries = [];
                                    d.validation_errors = [];
                                    d.quality_disapproval_reasons = [];
                                }
                                if (o.defaults) {
                                    d.review_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.validation_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.quality_approval_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.placeholder_type_enum = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.review_status != null && m.hasOwnProperty("review_status")) {
                                    d.review_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyReviewStatusEnum.PolicyReviewStatus[m.review_status] : m.review_status;
                                }
                                if (m.approval_status != null && m.hasOwnProperty("approval_status")) {
                                    d.approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.PolicyApprovalStatusEnum.PolicyApprovalStatus[m.approval_status] : m.approval_status;
                                }
                                if (m.policy_topic_entries && m.policy_topic_entries.length) {
                                    d.policy_topic_entries = [];
                                    for (var j = 0; j < m.policy_topic_entries.length; ++j) {
                                        d.policy_topic_entries[j] = $root.google.ads.googleads.v11.common.PolicyTopicEntry.toObject(m.policy_topic_entries[j], o);
                                    }
                                }
                                if (m.validation_status != null && m.hasOwnProperty("validation_status")) {
                                    d.validation_status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemValidationStatusEnum.FeedItemValidationStatus[m.validation_status] : m.validation_status;
                                }
                                if (m.validation_errors && m.validation_errors.length) {
                                    d.validation_errors = [];
                                    for (var j = 0; j < m.validation_errors.length; ++j) {
                                        d.validation_errors[j] = $root.google.ads.googleads.v11.resources.FeedItemValidationError.toObject(m.validation_errors[j], o);
                                    }
                                }
                                if (m.quality_approval_status != null && m.hasOwnProperty("quality_approval_status")) {
                                    d.quality_approval_status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemQualityApprovalStatusEnum.FeedItemQualityApprovalStatus[m.quality_approval_status] : m.quality_approval_status;
                                }
                                if (m.quality_disapproval_reasons && m.quality_disapproval_reasons.length) {
                                    d.quality_disapproval_reasons = [];
                                    for (var j = 0; j < m.quality_disapproval_reasons.length; ++j) {
                                        d.quality_disapproval_reasons[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemQualityDisapprovalReasonEnum.FeedItemQualityDisapprovalReason[m.quality_disapproval_reasons[j]] : m.quality_disapproval_reasons[j];
                                    }
                                }
                                if (m.placeholder_type_enum != null && m.hasOwnProperty("placeholder_type_enum")) {
                                    d.placeholder_type_enum = o.enums === String ? $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType[m.placeholder_type_enum] : m.placeholder_type_enum;
                                }
                                if (m.feed_mapping_resource_name != null && m.hasOwnProperty("feed_mapping_resource_name")) {
                                    d.feed_mapping_resource_name = m.feed_mapping_resource_name;
                                    if (o.oneofs)
                                        d._feed_mapping_resource_name = "feed_mapping_resource_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemPlaceholderPolicyInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedItemPlaceholderPolicyInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemPlaceholderPolicyInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemPlaceholderPolicyInfo;
                        })();
    
                        resources.FeedItemValidationError = (function() {
    
                            /**
                             * Properties of a FeedItemValidationError.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedItemValidationError
                             * @property {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.FeedItemValidationError|null} [validation_error] FeedItemValidationError validation_error
                             * @property {string|null} [description] FeedItemValidationError description
                             * @property {Array.<number>|null} [feed_attribute_ids] FeedItemValidationError feed_attribute_ids
                             * @property {string|null} [extra_info] FeedItemValidationError extra_info
                             */
    
                            /**
                             * Constructs a new FeedItemValidationError.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedItemValidationError.
                             * @implements IFeedItemValidationError
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedItemValidationError=} [p] Properties to set
                             */
                            function FeedItemValidationError(p) {
                                this.feed_attribute_ids = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemValidationError validation_error.
                             * @member {google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.FeedItemValidationError} validation_error
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @instance
                             */
                            FeedItemValidationError.prototype.validation_error = 0;
    
                            /**
                             * FeedItemValidationError description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @instance
                             */
                            FeedItemValidationError.prototype.description = null;
    
                            /**
                             * FeedItemValidationError feed_attribute_ids.
                             * @member {Array.<number>} feed_attribute_ids
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @instance
                             */
                            FeedItemValidationError.prototype.feed_attribute_ids = $util.emptyArray;
    
                            /**
                             * FeedItemValidationError extra_info.
                             * @member {string|null|undefined} extra_info
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @instance
                             */
                            FeedItemValidationError.prototype.extra_info = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemValidationError _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @instance
                             */
                            Object.defineProperty(FeedItemValidationError.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItemValidationError _extra_info.
                             * @member {"extra_info"|undefined} _extra_info
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @instance
                             */
                            Object.defineProperty(FeedItemValidationError.prototype, "_extra_info", {
                                get: $util.oneOfGetter($oneOfFields = ["extra_info"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemValidationError instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemValidationError=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedItemValidationError} FeedItemValidationError instance
                             */
                            FeedItemValidationError.create = function create(properties) {
                                return new FeedItemValidationError(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemValidationError message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemValidationError.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemValidationError} m FeedItemValidationError message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemValidationError.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.validation_error != null && Object.hasOwnProperty.call(m, "validation_error"))
                                    w.uint32(8).int32(m.validation_error);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(50).string(m.description);
                                if (m.feed_attribute_ids != null && m.feed_attribute_ids.length) {
                                    w.uint32(58).fork();
                                    for (var i = 0; i < m.feed_attribute_ids.length; ++i)
                                        w.int64(m.feed_attribute_ids[i]);
                                    w.ldelim();
                                }
                                if (m.extra_info != null && Object.hasOwnProperty.call(m, "extra_info"))
                                    w.uint32(66).string(m.extra_info);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemValidationError message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemValidationError.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemValidationError} message FeedItemValidationError message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemValidationError.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemValidationError message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedItemValidationError} FeedItemValidationError
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemValidationError.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedItemValidationError();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.validation_error = r.int32();
                                        break;
                                    case 6:
                                        m.description = r.string();
                                        break;
                                    case 7:
                                        if (!(m.feed_attribute_ids && m.feed_attribute_ids.length))
                                            m.feed_attribute_ids = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.feed_attribute_ids.push(r.int64());
                                        } else
                                            m.feed_attribute_ids.push(r.int64());
                                        break;
                                    case 8:
                                        m.extra_info = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemValidationError message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedItemValidationError} FeedItemValidationError
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemValidationError.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemValidationError message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemValidationError.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.validation_error != null && m.hasOwnProperty("validation_error")) {
                                    switch (m.validation_error) {
                                    default:
                                        return "validation_error: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 99:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                    case 37:
                                    case 38:
                                    case 39:
                                    case 40:
                                    case 41:
                                    case 42:
                                    case 43:
                                    case 44:
                                    case 45:
                                    case 46:
                                    case 47:
                                    case 48:
                                    case 49:
                                    case 50:
                                    case 51:
                                    case 52:
                                    case 53:
                                    case 54:
                                    case 55:
                                    case 56:
                                    case 57:
                                    case 58:
                                    case 59:
                                    case 60:
                                    case 61:
                                    case 62:
                                    case 63:
                                    case 64:
                                    case 65:
                                    case 66:
                                    case 67:
                                    case 68:
                                    case 69:
                                    case 70:
                                    case 71:
                                    case 72:
                                    case 73:
                                    case 74:
                                    case 75:
                                    case 76:
                                    case 77:
                                    case 78:
                                    case 79:
                                    case 80:
                                    case 81:
                                    case 82:
                                    case 83:
                                    case 84:
                                    case 85:
                                    case 86:
                                    case 87:
                                    case 88:
                                    case 89:
                                    case 90:
                                    case 91:
                                    case 92:
                                    case 93:
                                    case 94:
                                    case 95:
                                    case 96:
                                    case 97:
                                    case 98:
                                    case 100:
                                    case 101:
                                    case 102:
                                    case 103:
                                    case 104:
                                    case 105:
                                    case 106:
                                    case 107:
                                        break;
                                    }
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.feed_attribute_ids != null && m.hasOwnProperty("feed_attribute_ids")) {
                                    if (!Array.isArray(m.feed_attribute_ids))
                                        return "feed_attribute_ids: array expected";
                                    for (var i = 0; i < m.feed_attribute_ids.length; ++i) {
                                        if (!$util.isInteger(m.feed_attribute_ids[i]) && !(m.feed_attribute_ids[i] && $util.isInteger(m.feed_attribute_ids[i].low) && $util.isInteger(m.feed_attribute_ids[i].high)))
                                            return "feed_attribute_ids: integer|Long[] expected";
                                    }
                                }
                                if (m.extra_info != null && m.hasOwnProperty("extra_info")) {
                                    p._extra_info = 1;
                                    if (!$util.isString(m.extra_info))
                                        return "extra_info: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemValidationError message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedItemValidationError} FeedItemValidationError
                             */
                            FeedItemValidationError.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedItemValidationError)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedItemValidationError();
                                switch (d.validation_error) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.validation_error = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.validation_error = 1;
                                    break;
                                case "STRING_TOO_SHORT":
                                case 2:
                                    m.validation_error = 2;
                                    break;
                                case "STRING_TOO_LONG":
                                case 3:
                                    m.validation_error = 3;
                                    break;
                                case "VALUE_NOT_SPECIFIED":
                                case 4:
                                    m.validation_error = 4;
                                    break;
                                case "INVALID_DOMESTIC_PHONE_NUMBER_FORMAT":
                                case 5:
                                    m.validation_error = 5;
                                    break;
                                case "INVALID_PHONE_NUMBER":
                                case 6:
                                    m.validation_error = 6;
                                    break;
                                case "PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY":
                                case 7:
                                    m.validation_error = 7;
                                    break;
                                case "PREMIUM_RATE_NUMBER_NOT_ALLOWED":
                                case 8:
                                    m.validation_error = 8;
                                    break;
                                case "DISALLOWED_NUMBER_TYPE":
                                case 9:
                                    m.validation_error = 9;
                                    break;
                                case "VALUE_OUT_OF_RANGE":
                                case 10:
                                    m.validation_error = 10;
                                    break;
                                case "CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY":
                                case 11:
                                    m.validation_error = 11;
                                    break;
                                case "CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING":
                                case 99:
                                    m.validation_error = 99;
                                    break;
                                case "INVALID_COUNTRY_CODE":
                                case 13:
                                    m.validation_error = 13;
                                    break;
                                case "INVALID_APP_ID":
                                case 14:
                                    m.validation_error = 14;
                                    break;
                                case "MISSING_ATTRIBUTES_FOR_FIELDS":
                                case 15:
                                    m.validation_error = 15;
                                    break;
                                case "INVALID_TYPE_ID":
                                case 16:
                                    m.validation_error = 16;
                                    break;
                                case "INVALID_EMAIL_ADDRESS":
                                case 17:
                                    m.validation_error = 17;
                                    break;
                                case "INVALID_HTTPS_URL":
                                case 18:
                                    m.validation_error = 18;
                                    break;
                                case "MISSING_DELIVERY_ADDRESS":
                                case 19:
                                    m.validation_error = 19;
                                    break;
                                case "START_DATE_AFTER_END_DATE":
                                case 20:
                                    m.validation_error = 20;
                                    break;
                                case "MISSING_FEED_ITEM_START_TIME":
                                case 21:
                                    m.validation_error = 21;
                                    break;
                                case "MISSING_FEED_ITEM_END_TIME":
                                case 22:
                                    m.validation_error = 22;
                                    break;
                                case "MISSING_FEED_ITEM_ID":
                                case 23:
                                    m.validation_error = 23;
                                    break;
                                case "VANITY_PHONE_NUMBER_NOT_ALLOWED":
                                case 24:
                                    m.validation_error = 24;
                                    break;
                                case "INVALID_REVIEW_EXTENSION_SNIPPET":
                                case 25:
                                    m.validation_error = 25;
                                    break;
                                case "INVALID_NUMBER_FORMAT":
                                case 26:
                                    m.validation_error = 26;
                                    break;
                                case "INVALID_DATE_FORMAT":
                                case 27:
                                    m.validation_error = 27;
                                    break;
                                case "INVALID_PRICE_FORMAT":
                                case 28:
                                    m.validation_error = 28;
                                    break;
                                case "UNKNOWN_PLACEHOLDER_FIELD":
                                case 29:
                                    m.validation_error = 29;
                                    break;
                                case "MISSING_ENHANCED_SITELINK_DESCRIPTION_LINE":
                                case 30:
                                    m.validation_error = 30;
                                    break;
                                case "REVIEW_EXTENSION_SOURCE_INELIGIBLE":
                                case 31:
                                    m.validation_error = 31;
                                    break;
                                case "HYPHENS_IN_REVIEW_EXTENSION_SNIPPET":
                                case 32:
                                    m.validation_error = 32;
                                    break;
                                case "DOUBLE_QUOTES_IN_REVIEW_EXTENSION_SNIPPET":
                                case 33:
                                    m.validation_error = 33;
                                    break;
                                case "QUOTES_IN_REVIEW_EXTENSION_SNIPPET":
                                case 34:
                                    m.validation_error = 34;
                                    break;
                                case "INVALID_FORM_ENCODED_PARAMS":
                                case 35:
                                    m.validation_error = 35;
                                    break;
                                case "INVALID_URL_PARAMETER_NAME":
                                case 36:
                                    m.validation_error = 36;
                                    break;
                                case "NO_GEOCODING_RESULT":
                                case 37:
                                    m.validation_error = 37;
                                    break;
                                case "SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT":
                                case 38:
                                    m.validation_error = 38;
                                    break;
                                case "CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED":
                                case 39:
                                    m.validation_error = 39;
                                    break;
                                case "INVALID_PLACEHOLDER_FIELD_ID":
                                case 40:
                                    m.validation_error = 40;
                                    break;
                                case "INVALID_URL_TAG":
                                case 41:
                                    m.validation_error = 41;
                                    break;
                                case "LIST_TOO_LONG":
                                case 42:
                                    m.validation_error = 42;
                                    break;
                                case "INVALID_ATTRIBUTES_COMBINATION":
                                case 43:
                                    m.validation_error = 43;
                                    break;
                                case "DUPLICATE_VALUES":
                                case 44:
                                    m.validation_error = 44;
                                    break;
                                case "INVALID_CALL_CONVERSION_ACTION_ID":
                                case 45:
                                    m.validation_error = 45;
                                    break;
                                case "CANNOT_SET_WITHOUT_FINAL_URLS":
                                case 46:
                                    m.validation_error = 46;
                                    break;
                                case "APP_ID_DOESNT_EXIST_IN_APP_STORE":
                                case 47:
                                    m.validation_error = 47;
                                    break;
                                case "INVALID_FINAL_URL":
                                case 48:
                                    m.validation_error = 48;
                                    break;
                                case "INVALID_TRACKING_URL":
                                case 49:
                                    m.validation_error = 49;
                                    break;
                                case "INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL":
                                case 50:
                                    m.validation_error = 50;
                                    break;
                                case "LIST_TOO_SHORT":
                                case 51:
                                    m.validation_error = 51;
                                    break;
                                case "INVALID_USER_ACTION":
                                case 52:
                                    m.validation_error = 52;
                                    break;
                                case "INVALID_TYPE_NAME":
                                case 53:
                                    m.validation_error = 53;
                                    break;
                                case "INVALID_EVENT_CHANGE_STATUS":
                                case 54:
                                    m.validation_error = 54;
                                    break;
                                case "INVALID_SNIPPETS_HEADER":
                                case 55:
                                    m.validation_error = 55;
                                    break;
                                case "INVALID_ANDROID_APP_LINK":
                                case 56:
                                    m.validation_error = 56;
                                    break;
                                case "NUMBER_TYPE_WITH_CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY":
                                case 57:
                                    m.validation_error = 57;
                                    break;
                                case "RESERVED_KEYWORD_OTHER":
                                case 58:
                                    m.validation_error = 58;
                                    break;
                                case "DUPLICATE_OPTION_LABELS":
                                case 59:
                                    m.validation_error = 59;
                                    break;
                                case "DUPLICATE_OPTION_PREFILLS":
                                case 60:
                                    m.validation_error = 60;
                                    break;
                                case "UNEQUAL_LIST_LENGTHS":
                                case 61:
                                    m.validation_error = 61;
                                    break;
                                case "INCONSISTENT_CURRENCY_CODES":
                                case 62:
                                    m.validation_error = 62;
                                    break;
                                case "PRICE_EXTENSION_HAS_DUPLICATED_HEADERS":
                                case 63:
                                    m.validation_error = 63;
                                    break;
                                case "ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION":
                                case 64:
                                    m.validation_error = 64;
                                    break;
                                case "PRICE_EXTENSION_HAS_TOO_FEW_ITEMS":
                                case 65:
                                    m.validation_error = 65;
                                    break;
                                case "UNSUPPORTED_VALUE":
                                case 66:
                                    m.validation_error = 66;
                                    break;
                                case "INVALID_FINAL_MOBILE_URL":
                                case 67:
                                    m.validation_error = 67;
                                    break;
                                case "INVALID_KEYWORDLESS_AD_RULE_LABEL":
                                case 68:
                                    m.validation_error = 68;
                                    break;
                                case "VALUE_TRACK_PARAMETER_NOT_SUPPORTED":
                                case 69:
                                    m.validation_error = 69;
                                    break;
                                case "UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE":
                                case 70:
                                    m.validation_error = 70;
                                    break;
                                case "INVALID_IOS_APP_LINK":
                                case 71:
                                    m.validation_error = 71;
                                    break;
                                case "MISSING_IOS_APP_LINK_OR_IOS_APP_STORE_ID":
                                case 72:
                                    m.validation_error = 72;
                                    break;
                                case "PROMOTION_INVALID_TIME":
                                case 73:
                                    m.validation_error = 73;
                                    break;
                                case "PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF":
                                case 74:
                                    m.validation_error = 74;
                                    break;
                                case "PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT":
                                case 75:
                                    m.validation_error = 75;
                                    break;
                                case "TOO_MANY_DECIMAL_PLACES_SPECIFIED":
                                case 76:
                                    m.validation_error = 76;
                                    break;
                                case "AD_CUSTOMIZERS_NOT_ALLOWED":
                                case 77:
                                    m.validation_error = 77;
                                    break;
                                case "INVALID_LANGUAGE_CODE":
                                case 78:
                                    m.validation_error = 78;
                                    break;
                                case "UNSUPPORTED_LANGUAGE":
                                case 79:
                                    m.validation_error = 79;
                                    break;
                                case "IF_FUNCTION_NOT_ALLOWED":
                                case 80:
                                    m.validation_error = 80;
                                    break;
                                case "INVALID_FINAL_URL_SUFFIX":
                                case 81:
                                    m.validation_error = 81;
                                    break;
                                case "INVALID_TAG_IN_FINAL_URL_SUFFIX":
                                case 82:
                                    m.validation_error = 82;
                                    break;
                                case "INVALID_FINAL_URL_SUFFIX_FORMAT":
                                case 83:
                                    m.validation_error = 83;
                                    break;
                                case "CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED":
                                case 84:
                                    m.validation_error = 84;
                                    break;
                                case "ONLY_ONE_DELIVERY_OPTION_IS_ALLOWED":
                                case 85:
                                    m.validation_error = 85;
                                    break;
                                case "NO_DELIVERY_OPTION_IS_SET":
                                case 86:
                                    m.validation_error = 86;
                                    break;
                                case "INVALID_CONVERSION_REPORTING_STATE":
                                case 87:
                                    m.validation_error = 87;
                                    break;
                                case "IMAGE_SIZE_WRONG":
                                case 88:
                                    m.validation_error = 88;
                                    break;
                                case "EMAIL_DELIVERY_NOT_AVAILABLE_IN_COUNTRY":
                                case 89:
                                    m.validation_error = 89;
                                    break;
                                case "AUTO_REPLY_NOT_AVAILABLE_IN_COUNTRY":
                                case 90:
                                    m.validation_error = 90;
                                    break;
                                case "INVALID_LATITUDE_VALUE":
                                case 91:
                                    m.validation_error = 91;
                                    break;
                                case "INVALID_LONGITUDE_VALUE":
                                case 92:
                                    m.validation_error = 92;
                                    break;
                                case "TOO_MANY_LABELS":
                                case 93:
                                    m.validation_error = 93;
                                    break;
                                case "INVALID_IMAGE_URL":
                                case 94:
                                    m.validation_error = 94;
                                    break;
                                case "MISSING_LATITUDE_VALUE":
                                case 95:
                                    m.validation_error = 95;
                                    break;
                                case "MISSING_LONGITUDE_VALUE":
                                case 96:
                                    m.validation_error = 96;
                                    break;
                                case "ADDRESS_NOT_FOUND":
                                case 97:
                                    m.validation_error = 97;
                                    break;
                                case "ADDRESS_NOT_TARGETABLE":
                                case 98:
                                    m.validation_error = 98;
                                    break;
                                case "INVALID_ASSET_ID":
                                case 100:
                                    m.validation_error = 100;
                                    break;
                                case "INCOMPATIBLE_ASSET_TYPE":
                                case 101:
                                    m.validation_error = 101;
                                    break;
                                case "IMAGE_ERROR_UNEXPECTED_SIZE":
                                case 102:
                                    m.validation_error = 102;
                                    break;
                                case "IMAGE_ERROR_ASPECT_RATIO_NOT_ALLOWED":
                                case 103:
                                    m.validation_error = 103;
                                    break;
                                case "IMAGE_ERROR_FILE_TOO_LARGE":
                                case 104:
                                    m.validation_error = 104;
                                    break;
                                case "IMAGE_ERROR_FORMAT_NOT_ALLOWED":
                                case 105:
                                    m.validation_error = 105;
                                    break;
                                case "IMAGE_ERROR_CONSTRAINTS_VIOLATED":
                                case 106:
                                    m.validation_error = 106;
                                    break;
                                case "IMAGE_ERROR_SERVER_ERROR":
                                case 107:
                                    m.validation_error = 107;
                                    break;
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.feed_attribute_ids) {
                                    if (!Array.isArray(d.feed_attribute_ids))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemValidationError.feed_attribute_ids: array expected");
                                    m.feed_attribute_ids = [];
                                    for (var i = 0; i < d.feed_attribute_ids.length; ++i) {
                                        if ($util.Long)
                                            (m.feed_attribute_ids[i] = $util.Long.fromValue(d.feed_attribute_ids[i])).unsigned = false;
                                        else if (typeof d.feed_attribute_ids[i] === "string")
                                            m.feed_attribute_ids[i] = parseInt(d.feed_attribute_ids[i], 10);
                                        else if (typeof d.feed_attribute_ids[i] === "number")
                                            m.feed_attribute_ids[i] = d.feed_attribute_ids[i];
                                        else if (typeof d.feed_attribute_ids[i] === "object")
                                            m.feed_attribute_ids[i] = new $util.LongBits(d.feed_attribute_ids[i].low >>> 0, d.feed_attribute_ids[i].high >>> 0).toNumber();
                                    }
                                }
                                if (d.extra_info != null) {
                                    m.extra_info = String(d.extra_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemValidationError message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedItemValidationError} m FeedItemValidationError
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemValidationError.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.feed_attribute_ids = [];
                                }
                                if (o.defaults) {
                                    d.validation_error = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.validation_error != null && m.hasOwnProperty("validation_error")) {
                                    d.validation_error = o.enums === String ? $root.google.ads.googleads.v11.errors.FeedItemValidationErrorEnum.FeedItemValidationError[m.validation_error] : m.validation_error;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.feed_attribute_ids && m.feed_attribute_ids.length) {
                                    d.feed_attribute_ids = [];
                                    for (var j = 0; j < m.feed_attribute_ids.length; ++j) {
                                        if (typeof m.feed_attribute_ids[j] === "number")
                                            d.feed_attribute_ids[j] = o.longs === String ? String(m.feed_attribute_ids[j]) : m.feed_attribute_ids[j];
                                        else
                                            d.feed_attribute_ids[j] = o.longs === String ? $util.Long.prototype.toString.call(m.feed_attribute_ids[j]) : o.longs === Number ? new $util.LongBits(m.feed_attribute_ids[j].low >>> 0, m.feed_attribute_ids[j].high >>> 0).toNumber() : m.feed_attribute_ids[j];
                                    }
                                }
                                if (m.extra_info != null && m.hasOwnProperty("extra_info")) {
                                    d.extra_info = m.extra_info;
                                    if (o.oneofs)
                                        d._extra_info = "extra_info";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemValidationError to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedItemValidationError
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemValidationError.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemValidationError;
                        })();
    
                        resources.ChangeStatus = (function() {
    
                            /**
                             * Properties of a ChangeStatus.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IChangeStatus
                             * @property {string|null} [resource_name] ChangeStatus resource_name
                             * @property {string|null} [last_change_date_time] ChangeStatus last_change_date_time
                             * @property {google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum.ChangeStatusResourceType|null} [resource_type] ChangeStatus resource_type
                             * @property {string|null} [campaign] ChangeStatus campaign
                             * @property {string|null} [ad_group] ChangeStatus ad_group
                             * @property {google.ads.googleads.v11.enums.ChangeStatusOperationEnum.ChangeStatusOperation|null} [resource_status] ChangeStatus resource_status
                             * @property {string|null} [ad_group_ad] ChangeStatus ad_group_ad
                             * @property {string|null} [ad_group_criterion] ChangeStatus ad_group_criterion
                             * @property {string|null} [campaign_criterion] ChangeStatus campaign_criterion
                             * @property {string|null} [feed] ChangeStatus feed
                             * @property {string|null} [feed_item] ChangeStatus feed_item
                             * @property {string|null} [ad_group_feed] ChangeStatus ad_group_feed
                             * @property {string|null} [campaign_feed] ChangeStatus campaign_feed
                             * @property {string|null} [ad_group_bid_modifier] ChangeStatus ad_group_bid_modifier
                             * @property {string|null} [shared_set] ChangeStatus shared_set
                             * @property {string|null} [campaign_shared_set] ChangeStatus campaign_shared_set
                             * @property {string|null} [asset] ChangeStatus asset
                             * @property {string|null} [customer_asset] ChangeStatus customer_asset
                             * @property {string|null} [campaign_asset] ChangeStatus campaign_asset
                             * @property {string|null} [ad_group_asset] ChangeStatus ad_group_asset
                             */
    
                            /**
                             * Constructs a new ChangeStatus.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ChangeStatus.
                             * @implements IChangeStatus
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IChangeStatus=} [p] Properties to set
                             */
                            function ChangeStatus(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ChangeStatus resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.resource_name = "";
    
                            /**
                             * ChangeStatus last_change_date_time.
                             * @member {string|null|undefined} last_change_date_time
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.last_change_date_time = null;
    
                            /**
                             * ChangeStatus resource_type.
                             * @member {google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum.ChangeStatusResourceType} resource_type
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.resource_type = 0;
    
                            /**
                             * ChangeStatus campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.campaign = null;
    
                            /**
                             * ChangeStatus ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.ad_group = null;
    
                            /**
                             * ChangeStatus resource_status.
                             * @member {google.ads.googleads.v11.enums.ChangeStatusOperationEnum.ChangeStatusOperation} resource_status
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.resource_status = 0;
    
                            /**
                             * ChangeStatus ad_group_ad.
                             * @member {string|null|undefined} ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.ad_group_ad = null;
    
                            /**
                             * ChangeStatus ad_group_criterion.
                             * @member {string|null|undefined} ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.ad_group_criterion = null;
    
                            /**
                             * ChangeStatus campaign_criterion.
                             * @member {string|null|undefined} campaign_criterion
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.campaign_criterion = null;
    
                            /**
                             * ChangeStatus feed.
                             * @member {string|null|undefined} feed
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.feed = null;
    
                            /**
                             * ChangeStatus feed_item.
                             * @member {string|null|undefined} feed_item
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.feed_item = null;
    
                            /**
                             * ChangeStatus ad_group_feed.
                             * @member {string|null|undefined} ad_group_feed
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.ad_group_feed = null;
    
                            /**
                             * ChangeStatus campaign_feed.
                             * @member {string|null|undefined} campaign_feed
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.campaign_feed = null;
    
                            /**
                             * ChangeStatus ad_group_bid_modifier.
                             * @member {string|null|undefined} ad_group_bid_modifier
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.ad_group_bid_modifier = null;
    
                            /**
                             * ChangeStatus shared_set.
                             * @member {string} shared_set
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.shared_set = "";
    
                            /**
                             * ChangeStatus campaign_shared_set.
                             * @member {string} campaign_shared_set
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.campaign_shared_set = "";
    
                            /**
                             * ChangeStatus asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.asset = "";
    
                            /**
                             * ChangeStatus customer_asset.
                             * @member {string} customer_asset
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.customer_asset = "";
    
                            /**
                             * ChangeStatus campaign_asset.
                             * @member {string} campaign_asset
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.campaign_asset = "";
    
                            /**
                             * ChangeStatus ad_group_asset.
                             * @member {string} ad_group_asset
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            ChangeStatus.prototype.ad_group_asset = "";
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ChangeStatus _last_change_date_time.
                             * @member {"last_change_date_time"|undefined} _last_change_date_time
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_last_change_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["last_change_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _ad_group_ad.
                             * @member {"ad_group_ad"|undefined} _ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_ad_group_ad", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_ad"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _ad_group_criterion.
                             * @member {"ad_group_criterion"|undefined} _ad_group_criterion
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_ad_group_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _campaign_criterion.
                             * @member {"campaign_criterion"|undefined} _campaign_criterion
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_campaign_criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_criterion"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _feed.
                             * @member {"feed"|undefined} _feed
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _feed_item.
                             * @member {"feed_item"|undefined} _feed_item
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_feed_item", {
                                get: $util.oneOfGetter($oneOfFields = ["feed_item"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _ad_group_feed.
                             * @member {"ad_group_feed"|undefined} _ad_group_feed
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_ad_group_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _campaign_feed.
                             * @member {"campaign_feed"|undefined} _campaign_feed
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_campaign_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ChangeStatus _ad_group_bid_modifier.
                             * @member {"ad_group_bid_modifier"|undefined} _ad_group_bid_modifier
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             */
                            Object.defineProperty(ChangeStatus.prototype, "_ad_group_bid_modifier", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_bid_modifier"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ChangeStatus instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {google.ads.googleads.v11.resources.IChangeStatus=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ChangeStatus} ChangeStatus instance
                             */
                            ChangeStatus.create = function create(properties) {
                                return new ChangeStatus(properties);
                            };
    
                            /**
                             * Encodes the specified ChangeStatus message. Does not implicitly {@link google.ads.googleads.v11.resources.ChangeStatus.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {google.ads.googleads.v11.resources.IChangeStatus} m ChangeStatus message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatus.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.resource_type != null && Object.hasOwnProperty.call(m, "resource_type"))
                                    w.uint32(32).int32(m.resource_type);
                                if (m.resource_status != null && Object.hasOwnProperty.call(m, "resource_status"))
                                    w.uint32(64).int32(m.resource_status);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(138).string(m.campaign);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(146).string(m.ad_group);
                                if (m.last_change_date_time != null && Object.hasOwnProperty.call(m, "last_change_date_time"))
                                    w.uint32(194).string(m.last_change_date_time);
                                if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                    w.uint32(202).string(m.ad_group_ad);
                                if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                    w.uint32(210).string(m.ad_group_criterion);
                                if (m.campaign_criterion != null && Object.hasOwnProperty.call(m, "campaign_criterion"))
                                    w.uint32(218).string(m.campaign_criterion);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(226).string(m.feed);
                                if (m.feed_item != null && Object.hasOwnProperty.call(m, "feed_item"))
                                    w.uint32(234).string(m.feed_item);
                                if (m.ad_group_feed != null && Object.hasOwnProperty.call(m, "ad_group_feed"))
                                    w.uint32(242).string(m.ad_group_feed);
                                if (m.campaign_feed != null && Object.hasOwnProperty.call(m, "campaign_feed"))
                                    w.uint32(250).string(m.campaign_feed);
                                if (m.ad_group_bid_modifier != null && Object.hasOwnProperty.call(m, "ad_group_bid_modifier"))
                                    w.uint32(258).string(m.ad_group_bid_modifier);
                                if (m.shared_set != null && Object.hasOwnProperty.call(m, "shared_set"))
                                    w.uint32(266).string(m.shared_set);
                                if (m.campaign_shared_set != null && Object.hasOwnProperty.call(m, "campaign_shared_set"))
                                    w.uint32(274).string(m.campaign_shared_set);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(282).string(m.asset);
                                if (m.customer_asset != null && Object.hasOwnProperty.call(m, "customer_asset"))
                                    w.uint32(290).string(m.customer_asset);
                                if (m.campaign_asset != null && Object.hasOwnProperty.call(m, "campaign_asset"))
                                    w.uint32(298).string(m.campaign_asset);
                                if (m.ad_group_asset != null && Object.hasOwnProperty.call(m, "ad_group_asset"))
                                    w.uint32(306).string(m.ad_group_asset);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ChangeStatus message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ChangeStatus.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {google.ads.googleads.v11.resources.IChangeStatus} message ChangeStatus message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChangeStatus.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChangeStatus message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ChangeStatus} ChangeStatus
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatus.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ChangeStatus();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 24:
                                        m.last_change_date_time = r.string();
                                        break;
                                    case 4:
                                        m.resource_type = r.int32();
                                        break;
                                    case 17:
                                        m.campaign = r.string();
                                        break;
                                    case 18:
                                        m.ad_group = r.string();
                                        break;
                                    case 8:
                                        m.resource_status = r.int32();
                                        break;
                                    case 25:
                                        m.ad_group_ad = r.string();
                                        break;
                                    case 26:
                                        m.ad_group_criterion = r.string();
                                        break;
                                    case 27:
                                        m.campaign_criterion = r.string();
                                        break;
                                    case 28:
                                        m.feed = r.string();
                                        break;
                                    case 29:
                                        m.feed_item = r.string();
                                        break;
                                    case 30:
                                        m.ad_group_feed = r.string();
                                        break;
                                    case 31:
                                        m.campaign_feed = r.string();
                                        break;
                                    case 32:
                                        m.ad_group_bid_modifier = r.string();
                                        break;
                                    case 33:
                                        m.shared_set = r.string();
                                        break;
                                    case 34:
                                        m.campaign_shared_set = r.string();
                                        break;
                                    case 35:
                                        m.asset = r.string();
                                        break;
                                    case 36:
                                        m.customer_asset = r.string();
                                        break;
                                    case 37:
                                        m.campaign_asset = r.string();
                                        break;
                                    case 38:
                                        m.ad_group_asset = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ChangeStatus message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ChangeStatus} ChangeStatus
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChangeStatus.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChangeStatus message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChangeStatus.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.last_change_date_time != null && m.hasOwnProperty("last_change_date_time")) {
                                    p._last_change_date_time = 1;
                                    if (!$util.isString(m.last_change_date_time))
                                        return "last_change_date_time: string expected";
                                }
                                if (m.resource_type != null && m.hasOwnProperty("resource_type")) {
                                    switch (m.resource_type) {
                                    default:
                                        return "resource_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                        break;
                                    }
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.resource_status != null && m.hasOwnProperty("resource_status")) {
                                    switch (m.resource_status) {
                                    default:
                                        return "resource_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    p._ad_group_ad = 1;
                                    if (!$util.isString(m.ad_group_ad))
                                        return "ad_group_ad: string expected";
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    p._ad_group_criterion = 1;
                                    if (!$util.isString(m.ad_group_criterion))
                                        return "ad_group_criterion: string expected";
                                }
                                if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                    p._campaign_criterion = 1;
                                    if (!$util.isString(m.campaign_criterion))
                                        return "campaign_criterion: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    p._feed = 1;
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    p._feed_item = 1;
                                    if (!$util.isString(m.feed_item))
                                        return "feed_item: string expected";
                                }
                                if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                    p._ad_group_feed = 1;
                                    if (!$util.isString(m.ad_group_feed))
                                        return "ad_group_feed: string expected";
                                }
                                if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                    p._campaign_feed = 1;
                                    if (!$util.isString(m.campaign_feed))
                                        return "campaign_feed: string expected";
                                }
                                if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                    p._ad_group_bid_modifier = 1;
                                    if (!$util.isString(m.ad_group_bid_modifier))
                                        return "ad_group_bid_modifier: string expected";
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    if (!$util.isString(m.shared_set))
                                        return "shared_set: string expected";
                                }
                                if (m.campaign_shared_set != null && m.hasOwnProperty("campaign_shared_set")) {
                                    if (!$util.isString(m.campaign_shared_set))
                                        return "campaign_shared_set: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                    if (!$util.isString(m.customer_asset))
                                        return "customer_asset: string expected";
                                }
                                if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                    if (!$util.isString(m.campaign_asset))
                                        return "campaign_asset: string expected";
                                }
                                if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                    if (!$util.isString(m.ad_group_asset))
                                        return "ad_group_asset: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ChangeStatus message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ChangeStatus} ChangeStatus
                             */
                            ChangeStatus.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ChangeStatus)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ChangeStatus();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.last_change_date_time != null) {
                                    m.last_change_date_time = String(d.last_change_date_time);
                                }
                                switch (d.resource_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.resource_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.resource_type = 1;
                                    break;
                                case "AD_GROUP":
                                case 3:
                                    m.resource_type = 3;
                                    break;
                                case "AD_GROUP_AD":
                                case 4:
                                    m.resource_type = 4;
                                    break;
                                case "AD_GROUP_CRITERION":
                                case 5:
                                    m.resource_type = 5;
                                    break;
                                case "CAMPAIGN":
                                case 6:
                                    m.resource_type = 6;
                                    break;
                                case "CAMPAIGN_CRITERION":
                                case 7:
                                    m.resource_type = 7;
                                    break;
                                case "FEED":
                                case 9:
                                    m.resource_type = 9;
                                    break;
                                case "FEED_ITEM":
                                case 10:
                                    m.resource_type = 10;
                                    break;
                                case "AD_GROUP_FEED":
                                case 11:
                                    m.resource_type = 11;
                                    break;
                                case "CAMPAIGN_FEED":
                                case 12:
                                    m.resource_type = 12;
                                    break;
                                case "AD_GROUP_BID_MODIFIER":
                                case 13:
                                    m.resource_type = 13;
                                    break;
                                case "SHARED_SET":
                                case 14:
                                    m.resource_type = 14;
                                    break;
                                case "CAMPAIGN_SHARED_SET":
                                case 15:
                                    m.resource_type = 15;
                                    break;
                                case "ASSET":
                                case 16:
                                    m.resource_type = 16;
                                    break;
                                case "CUSTOMER_ASSET":
                                case 17:
                                    m.resource_type = 17;
                                    break;
                                case "CAMPAIGN_ASSET":
                                case 18:
                                    m.resource_type = 18;
                                    break;
                                case "AD_GROUP_ASSET":
                                case 19:
                                    m.resource_type = 19;
                                    break;
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                switch (d.resource_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.resource_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.resource_status = 1;
                                    break;
                                case "ADDED":
                                case 2:
                                    m.resource_status = 2;
                                    break;
                                case "CHANGED":
                                case 3:
                                    m.resource_status = 3;
                                    break;
                                case "REMOVED":
                                case 4:
                                    m.resource_status = 4;
                                    break;
                                }
                                if (d.ad_group_ad != null) {
                                    m.ad_group_ad = String(d.ad_group_ad);
                                }
                                if (d.ad_group_criterion != null) {
                                    m.ad_group_criterion = String(d.ad_group_criterion);
                                }
                                if (d.campaign_criterion != null) {
                                    m.campaign_criterion = String(d.campaign_criterion);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.feed_item != null) {
                                    m.feed_item = String(d.feed_item);
                                }
                                if (d.ad_group_feed != null) {
                                    m.ad_group_feed = String(d.ad_group_feed);
                                }
                                if (d.campaign_feed != null) {
                                    m.campaign_feed = String(d.campaign_feed);
                                }
                                if (d.ad_group_bid_modifier != null) {
                                    m.ad_group_bid_modifier = String(d.ad_group_bid_modifier);
                                }
                                if (d.shared_set != null) {
                                    m.shared_set = String(d.shared_set);
                                }
                                if (d.campaign_shared_set != null) {
                                    m.campaign_shared_set = String(d.campaign_shared_set);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                if (d.customer_asset != null) {
                                    m.customer_asset = String(d.customer_asset);
                                }
                                if (d.campaign_asset != null) {
                                    m.campaign_asset = String(d.campaign_asset);
                                }
                                if (d.ad_group_asset != null) {
                                    m.ad_group_asset = String(d.ad_group_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ChangeStatus message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @static
                             * @param {google.ads.googleads.v11.resources.ChangeStatus} m ChangeStatus
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChangeStatus.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.resource_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.resource_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.shared_set = "";
                                    d.campaign_shared_set = "";
                                    d.asset = "";
                                    d.customer_asset = "";
                                    d.campaign_asset = "";
                                    d.ad_group_asset = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.resource_type != null && m.hasOwnProperty("resource_type")) {
                                    d.resource_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ChangeStatusResourceTypeEnum.ChangeStatusResourceType[m.resource_type] : m.resource_type;
                                }
                                if (m.resource_status != null && m.hasOwnProperty("resource_status")) {
                                    d.resource_status = o.enums === String ? $root.google.ads.googleads.v11.enums.ChangeStatusOperationEnum.ChangeStatusOperation[m.resource_status] : m.resource_status;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                if (m.last_change_date_time != null && m.hasOwnProperty("last_change_date_time")) {
                                    d.last_change_date_time = m.last_change_date_time;
                                    if (o.oneofs)
                                        d._last_change_date_time = "last_change_date_time";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    d.ad_group_ad = m.ad_group_ad;
                                    if (o.oneofs)
                                        d._ad_group_ad = "ad_group_ad";
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    d.ad_group_criterion = m.ad_group_criterion;
                                    if (o.oneofs)
                                        d._ad_group_criterion = "ad_group_criterion";
                                }
                                if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                    d.campaign_criterion = m.campaign_criterion;
                                    if (o.oneofs)
                                        d._campaign_criterion = "campaign_criterion";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                    if (o.oneofs)
                                        d._feed = "feed";
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    d.feed_item = m.feed_item;
                                    if (o.oneofs)
                                        d._feed_item = "feed_item";
                                }
                                if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                    d.ad_group_feed = m.ad_group_feed;
                                    if (o.oneofs)
                                        d._ad_group_feed = "ad_group_feed";
                                }
                                if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                    d.campaign_feed = m.campaign_feed;
                                    if (o.oneofs)
                                        d._campaign_feed = "campaign_feed";
                                }
                                if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                    d.ad_group_bid_modifier = m.ad_group_bid_modifier;
                                    if (o.oneofs)
                                        d._ad_group_bid_modifier = "ad_group_bid_modifier";
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    d.shared_set = m.shared_set;
                                }
                                if (m.campaign_shared_set != null && m.hasOwnProperty("campaign_shared_set")) {
                                    d.campaign_shared_set = m.campaign_shared_set;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                    d.customer_asset = m.customer_asset;
                                }
                                if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                    d.campaign_asset = m.campaign_asset;
                                }
                                if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                    d.ad_group_asset = m.ad_group_asset;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ChangeStatus to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ChangeStatus
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChangeStatus.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ChangeStatus;
                        })();
    
                        resources.ClickView = (function() {
    
                            /**
                             * Properties of a ClickView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IClickView
                             * @property {string|null} [resource_name] ClickView resource_name
                             * @property {string|null} [gclid] ClickView gclid
                             * @property {google.ads.googleads.v11.common.IClickLocation|null} [area_of_interest] ClickView area_of_interest
                             * @property {google.ads.googleads.v11.common.IClickLocation|null} [location_of_presence] ClickView location_of_presence
                             * @property {number|null} [page_number] ClickView page_number
                             * @property {string|null} [ad_group_ad] ClickView ad_group_ad
                             * @property {string|null} [campaign_location_target] ClickView campaign_location_target
                             * @property {string|null} [user_list] ClickView user_list
                             * @property {string|null} [keyword] ClickView keyword
                             * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [keyword_info] ClickView keyword_info
                             */
    
                            /**
                             * Constructs a new ClickView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ClickView.
                             * @implements IClickView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IClickView=} [p] Properties to set
                             */
                            function ClickView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ClickView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.resource_name = "";
    
                            /**
                             * ClickView gclid.
                             * @member {string|null|undefined} gclid
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.gclid = null;
    
                            /**
                             * ClickView area_of_interest.
                             * @member {google.ads.googleads.v11.common.IClickLocation|null|undefined} area_of_interest
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.area_of_interest = null;
    
                            /**
                             * ClickView location_of_presence.
                             * @member {google.ads.googleads.v11.common.IClickLocation|null|undefined} location_of_presence
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.location_of_presence = null;
    
                            /**
                             * ClickView page_number.
                             * @member {number|null|undefined} page_number
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.page_number = null;
    
                            /**
                             * ClickView ad_group_ad.
                             * @member {string|null|undefined} ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.ad_group_ad = null;
    
                            /**
                             * ClickView campaign_location_target.
                             * @member {string|null|undefined} campaign_location_target
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.campaign_location_target = null;
    
                            /**
                             * ClickView user_list.
                             * @member {string|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.user_list = null;
    
                            /**
                             * ClickView keyword.
                             * @member {string} keyword
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.keyword = "";
    
                            /**
                             * ClickView keyword_info.
                             * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} keyword_info
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            ClickView.prototype.keyword_info = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ClickView _gclid.
                             * @member {"gclid"|undefined} _gclid
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            Object.defineProperty(ClickView.prototype, "_gclid", {
                                get: $util.oneOfGetter($oneOfFields = ["gclid"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickView _page_number.
                             * @member {"page_number"|undefined} _page_number
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            Object.defineProperty(ClickView.prototype, "_page_number", {
                                get: $util.oneOfGetter($oneOfFields = ["page_number"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickView _ad_group_ad.
                             * @member {"ad_group_ad"|undefined} _ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            Object.defineProperty(ClickView.prototype, "_ad_group_ad", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_ad"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickView _campaign_location_target.
                             * @member {"campaign_location_target"|undefined} _campaign_location_target
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            Object.defineProperty(ClickView.prototype, "_campaign_location_target", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_location_target"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickView _user_list.
                             * @member {"user_list"|undefined} _user_list
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             */
                            Object.defineProperty(ClickView.prototype, "_user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ClickView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IClickView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ClickView} ClickView instance
                             */
                            ClickView.create = function create(properties) {
                                return new ClickView(properties);
                            };
    
                            /**
                             * Encodes the specified ClickView message. Does not implicitly {@link google.ads.googleads.v11.resources.ClickView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IClickView} m ClickView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.area_of_interest != null && Object.hasOwnProperty.call(m, "area_of_interest"))
                                    $root.google.ads.googleads.v11.common.ClickLocation.encode(m.area_of_interest, w.uint32(26).fork()).ldelim();
                                if (m.location_of_presence != null && Object.hasOwnProperty.call(m, "location_of_presence"))
                                    $root.google.ads.googleads.v11.common.ClickLocation.encode(m.location_of_presence, w.uint32(34).fork()).ldelim();
                                if (m.gclid != null && Object.hasOwnProperty.call(m, "gclid"))
                                    w.uint32(66).string(m.gclid);
                                if (m.page_number != null && Object.hasOwnProperty.call(m, "page_number"))
                                    w.uint32(72).int64(m.page_number);
                                if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                    w.uint32(82).string(m.ad_group_ad);
                                if (m.campaign_location_target != null && Object.hasOwnProperty.call(m, "campaign_location_target"))
                                    w.uint32(90).string(m.campaign_location_target);
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    w.uint32(98).string(m.user_list);
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    w.uint32(106).string(m.keyword);
                                if (m.keyword_info != null && Object.hasOwnProperty.call(m, "keyword_info"))
                                    $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword_info, w.uint32(114).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ClickView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ClickView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IClickView} message ClickView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClickView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ClickView} ClickView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ClickView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 8:
                                        m.gclid = r.string();
                                        break;
                                    case 3:
                                        m.area_of_interest = $root.google.ads.googleads.v11.common.ClickLocation.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.location_of_presence = $root.google.ads.googleads.v11.common.ClickLocation.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.page_number = r.int64();
                                        break;
                                    case 10:
                                        m.ad_group_ad = r.string();
                                        break;
                                    case 11:
                                        m.campaign_location_target = r.string();
                                        break;
                                    case 12:
                                        m.user_list = r.string();
                                        break;
                                    case 13:
                                        m.keyword = r.string();
                                        break;
                                    case 14:
                                        m.keyword_info = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ClickView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ClickView} ClickView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClickView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClickView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    p._gclid = 1;
                                    if (!$util.isString(m.gclid))
                                        return "gclid: string expected";
                                }
                                if (m.area_of_interest != null && m.hasOwnProperty("area_of_interest")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ClickLocation.verify(m.area_of_interest);
                                        if (e)
                                            return "area_of_interest." + e;
                                    }
                                }
                                if (m.location_of_presence != null && m.hasOwnProperty("location_of_presence")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ClickLocation.verify(m.location_of_presence);
                                        if (e)
                                            return "location_of_presence." + e;
                                    }
                                }
                                if (m.page_number != null && m.hasOwnProperty("page_number")) {
                                    p._page_number = 1;
                                    if (!$util.isInteger(m.page_number) && !(m.page_number && $util.isInteger(m.page_number.low) && $util.isInteger(m.page_number.high)))
                                        return "page_number: integer|Long expected";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    p._ad_group_ad = 1;
                                    if (!$util.isString(m.ad_group_ad))
                                        return "ad_group_ad: string expected";
                                }
                                if (m.campaign_location_target != null && m.hasOwnProperty("campaign_location_target")) {
                                    p._campaign_location_target = 1;
                                    if (!$util.isString(m.campaign_location_target))
                                        return "campaign_location_target: string expected";
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    p._user_list = 1;
                                    if (!$util.isString(m.user_list))
                                        return "user_list: string expected";
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    if (!$util.isString(m.keyword))
                                        return "keyword: string expected";
                                }
                                if (m.keyword_info != null && m.hasOwnProperty("keyword_info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword_info);
                                        if (e)
                                            return "keyword_info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ClickView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ClickView} ClickView
                             */
                            ClickView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ClickView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ClickView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.gclid != null) {
                                    m.gclid = String(d.gclid);
                                }
                                if (d.area_of_interest != null) {
                                    if (typeof d.area_of_interest !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ClickView.area_of_interest: object expected");
                                    m.area_of_interest = $root.google.ads.googleads.v11.common.ClickLocation.fromObject(d.area_of_interest);
                                }
                                if (d.location_of_presence != null) {
                                    if (typeof d.location_of_presence !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ClickView.location_of_presence: object expected");
                                    m.location_of_presence = $root.google.ads.googleads.v11.common.ClickLocation.fromObject(d.location_of_presence);
                                }
                                if (d.page_number != null) {
                                    if ($util.Long)
                                        (m.page_number = $util.Long.fromValue(d.page_number)).unsigned = false;
                                    else if (typeof d.page_number === "string")
                                        m.page_number = parseInt(d.page_number, 10);
                                    else if (typeof d.page_number === "number")
                                        m.page_number = d.page_number;
                                    else if (typeof d.page_number === "object")
                                        m.page_number = new $util.LongBits(d.page_number.low >>> 0, d.page_number.high >>> 0).toNumber();
                                }
                                if (d.ad_group_ad != null) {
                                    m.ad_group_ad = String(d.ad_group_ad);
                                }
                                if (d.campaign_location_target != null) {
                                    m.campaign_location_target = String(d.campaign_location_target);
                                }
                                if (d.user_list != null) {
                                    m.user_list = String(d.user_list);
                                }
                                if (d.keyword != null) {
                                    m.keyword = String(d.keyword);
                                }
                                if (d.keyword_info != null) {
                                    if (typeof d.keyword_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ClickView.keyword_info: object expected");
                                    m.keyword_info = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ClickView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ClickView} m ClickView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClickView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.area_of_interest = null;
                                    d.location_of_presence = null;
                                    d.keyword = "";
                                    d.keyword_info = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.area_of_interest != null && m.hasOwnProperty("area_of_interest")) {
                                    d.area_of_interest = $root.google.ads.googleads.v11.common.ClickLocation.toObject(m.area_of_interest, o);
                                }
                                if (m.location_of_presence != null && m.hasOwnProperty("location_of_presence")) {
                                    d.location_of_presence = $root.google.ads.googleads.v11.common.ClickLocation.toObject(m.location_of_presence, o);
                                }
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    d.gclid = m.gclid;
                                    if (o.oneofs)
                                        d._gclid = "gclid";
                                }
                                if (m.page_number != null && m.hasOwnProperty("page_number")) {
                                    if (typeof m.page_number === "number")
                                        d.page_number = o.longs === String ? String(m.page_number) : m.page_number;
                                    else
                                        d.page_number = o.longs === String ? $util.Long.prototype.toString.call(m.page_number) : o.longs === Number ? new $util.LongBits(m.page_number.low >>> 0, m.page_number.high >>> 0).toNumber() : m.page_number;
                                    if (o.oneofs)
                                        d._page_number = "page_number";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    d.ad_group_ad = m.ad_group_ad;
                                    if (o.oneofs)
                                        d._ad_group_ad = "ad_group_ad";
                                }
                                if (m.campaign_location_target != null && m.hasOwnProperty("campaign_location_target")) {
                                    d.campaign_location_target = m.campaign_location_target;
                                    if (o.oneofs)
                                        d._campaign_location_target = "campaign_location_target";
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = m.user_list;
                                    if (o.oneofs)
                                        d._user_list = "user_list";
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = m.keyword;
                                }
                                if (m.keyword_info != null && m.hasOwnProperty("keyword_info")) {
                                    d.keyword_info = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword_info, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ClickView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ClickView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClickView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ClickView;
                        })();
    
                        resources.CombinedAudience = (function() {
    
                            /**
                             * Properties of a CombinedAudience.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICombinedAudience
                             * @property {string|null} [resource_name] CombinedAudience resource_name
                             * @property {number|null} [id] CombinedAudience id
                             * @property {google.ads.googleads.v11.enums.CombinedAudienceStatusEnum.CombinedAudienceStatus|null} [status] CombinedAudience status
                             * @property {string|null} [name] CombinedAudience name
                             * @property {string|null} [description] CombinedAudience description
                             */
    
                            /**
                             * Constructs a new CombinedAudience.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CombinedAudience.
                             * @implements ICombinedAudience
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICombinedAudience=} [p] Properties to set
                             */
                            function CombinedAudience(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CombinedAudience resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @instance
                             */
                            CombinedAudience.prototype.resource_name = "";
    
                            /**
                             * CombinedAudience id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @instance
                             */
                            CombinedAudience.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CombinedAudience status.
                             * @member {google.ads.googleads.v11.enums.CombinedAudienceStatusEnum.CombinedAudienceStatus} status
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @instance
                             */
                            CombinedAudience.prototype.status = 0;
    
                            /**
                             * CombinedAudience name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @instance
                             */
                            CombinedAudience.prototype.name = "";
    
                            /**
                             * CombinedAudience description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @instance
                             */
                            CombinedAudience.prototype.description = "";
    
                            /**
                             * Creates a new CombinedAudience instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICombinedAudience=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CombinedAudience} CombinedAudience instance
                             */
                            CombinedAudience.create = function create(properties) {
                                return new CombinedAudience(properties);
                            };
    
                            /**
                             * Encodes the specified CombinedAudience message. Does not implicitly {@link google.ads.googleads.v11.resources.CombinedAudience.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICombinedAudience} m CombinedAudience message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedAudience.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(34).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(42).string(m.description);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CombinedAudience message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CombinedAudience.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICombinedAudience} message CombinedAudience message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CombinedAudience.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CombinedAudience message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CombinedAudience} CombinedAudience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedAudience.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CombinedAudience();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 4:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.description = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CombinedAudience message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CombinedAudience} CombinedAudience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CombinedAudience.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CombinedAudience message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CombinedAudience.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CombinedAudience message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CombinedAudience} CombinedAudience
                             */
                            CombinedAudience.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CombinedAudience)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CombinedAudience();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CombinedAudience message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.CombinedAudience} m CombinedAudience
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CombinedAudience.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.name = "";
                                    d.description = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CombinedAudienceStatusEnum.CombinedAudienceStatus[m.status] : m.status;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CombinedAudience to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CombinedAudience
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CombinedAudience.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CombinedAudience;
                        })();
    
                        resources.ConversionAction = (function() {
    
                            /**
                             * Properties of a ConversionAction.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IConversionAction
                             * @property {string|null} [resource_name] ConversionAction resource_name
                             * @property {number|null} [id] ConversionAction id
                             * @property {string|null} [name] ConversionAction name
                             * @property {google.ads.googleads.v11.enums.ConversionActionStatusEnum.ConversionActionStatus|null} [status] ConversionAction status
                             * @property {google.ads.googleads.v11.enums.ConversionActionTypeEnum.ConversionActionType|null} [type] ConversionAction type
                             * @property {google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin|null} [origin] ConversionAction origin
                             * @property {boolean|null} [primary_for_goal] ConversionAction primary_for_goal
                             * @property {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory|null} [category] ConversionAction category
                             * @property {string|null} [owner_customer] ConversionAction owner_customer
                             * @property {boolean|null} [include_in_conversions_metric] ConversionAction include_in_conversions_metric
                             * @property {number|null} [click_through_lookback_window_days] ConversionAction click_through_lookback_window_days
                             * @property {number|null} [view_through_lookback_window_days] ConversionAction view_through_lookback_window_days
                             * @property {google.ads.googleads.v11.resources.ConversionAction.IValueSettings|null} [value_settings] ConversionAction value_settings
                             * @property {google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum.ConversionActionCountingType|null} [counting_type] ConversionAction counting_type
                             * @property {google.ads.googleads.v11.resources.ConversionAction.IAttributionModelSettings|null} [attribution_model_settings] ConversionAction attribution_model_settings
                             * @property {Array.<google.ads.googleads.v11.common.ITagSnippet>|null} [tag_snippets] ConversionAction tag_snippets
                             * @property {number|null} [phone_call_duration_seconds] ConversionAction phone_call_duration_seconds
                             * @property {string|null} [app_id] ConversionAction app_id
                             * @property {google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor|null} [mobile_app_vendor] ConversionAction mobile_app_vendor
                             * @property {google.ads.googleads.v11.resources.ConversionAction.IFirebaseSettings|null} [firebase_settings] ConversionAction firebase_settings
                             * @property {google.ads.googleads.v11.resources.ConversionAction.IThirdPartyAppAnalyticsSettings|null} [third_party_app_analytics_settings] ConversionAction third_party_app_analytics_settings
                             */
    
                            /**
                             * Constructs a new ConversionAction.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ConversionAction.
                             * @implements IConversionAction
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IConversionAction=} [p] Properties to set
                             */
                            function ConversionAction(p) {
                                this.tag_snippets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionAction resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.resource_name = "";
    
                            /**
                             * ConversionAction id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.id = null;
    
                            /**
                             * ConversionAction name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.name = null;
    
                            /**
                             * ConversionAction status.
                             * @member {google.ads.googleads.v11.enums.ConversionActionStatusEnum.ConversionActionStatus} status
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.status = 0;
    
                            /**
                             * ConversionAction type.
                             * @member {google.ads.googleads.v11.enums.ConversionActionTypeEnum.ConversionActionType} type
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.type = 0;
    
                            /**
                             * ConversionAction origin.
                             * @member {google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin} origin
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.origin = 0;
    
                            /**
                             * ConversionAction primary_for_goal.
                             * @member {boolean|null|undefined} primary_for_goal
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.primary_for_goal = null;
    
                            /**
                             * ConversionAction category.
                             * @member {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory} category
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.category = 0;
    
                            /**
                             * ConversionAction owner_customer.
                             * @member {string|null|undefined} owner_customer
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.owner_customer = null;
    
                            /**
                             * ConversionAction include_in_conversions_metric.
                             * @member {boolean|null|undefined} include_in_conversions_metric
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.include_in_conversions_metric = null;
    
                            /**
                             * ConversionAction click_through_lookback_window_days.
                             * @member {number|null|undefined} click_through_lookback_window_days
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.click_through_lookback_window_days = null;
    
                            /**
                             * ConversionAction view_through_lookback_window_days.
                             * @member {number|null|undefined} view_through_lookback_window_days
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.view_through_lookback_window_days = null;
    
                            /**
                             * ConversionAction value_settings.
                             * @member {google.ads.googleads.v11.resources.ConversionAction.IValueSettings|null|undefined} value_settings
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.value_settings = null;
    
                            /**
                             * ConversionAction counting_type.
                             * @member {google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum.ConversionActionCountingType} counting_type
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.counting_type = 0;
    
                            /**
                             * ConversionAction attribution_model_settings.
                             * @member {google.ads.googleads.v11.resources.ConversionAction.IAttributionModelSettings|null|undefined} attribution_model_settings
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.attribution_model_settings = null;
    
                            /**
                             * ConversionAction tag_snippets.
                             * @member {Array.<google.ads.googleads.v11.common.ITagSnippet>} tag_snippets
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.tag_snippets = $util.emptyArray;
    
                            /**
                             * ConversionAction phone_call_duration_seconds.
                             * @member {number|null|undefined} phone_call_duration_seconds
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.phone_call_duration_seconds = null;
    
                            /**
                             * ConversionAction app_id.
                             * @member {string|null|undefined} app_id
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.app_id = null;
    
                            /**
                             * ConversionAction mobile_app_vendor.
                             * @member {google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor} mobile_app_vendor
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.mobile_app_vendor = 0;
    
                            /**
                             * ConversionAction firebase_settings.
                             * @member {google.ads.googleads.v11.resources.ConversionAction.IFirebaseSettings|null|undefined} firebase_settings
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.firebase_settings = null;
    
                            /**
                             * ConversionAction third_party_app_analytics_settings.
                             * @member {google.ads.googleads.v11.resources.ConversionAction.IThirdPartyAppAnalyticsSettings|null|undefined} third_party_app_analytics_settings
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            ConversionAction.prototype.third_party_app_analytics_settings = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionAction _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _primary_for_goal.
                             * @member {"primary_for_goal"|undefined} _primary_for_goal
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_primary_for_goal", {
                                get: $util.oneOfGetter($oneOfFields = ["primary_for_goal"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _owner_customer.
                             * @member {"owner_customer"|undefined} _owner_customer
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_owner_customer", {
                                get: $util.oneOfGetter($oneOfFields = ["owner_customer"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _include_in_conversions_metric.
                             * @member {"include_in_conversions_metric"|undefined} _include_in_conversions_metric
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_include_in_conversions_metric", {
                                get: $util.oneOfGetter($oneOfFields = ["include_in_conversions_metric"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _click_through_lookback_window_days.
                             * @member {"click_through_lookback_window_days"|undefined} _click_through_lookback_window_days
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_click_through_lookback_window_days", {
                                get: $util.oneOfGetter($oneOfFields = ["click_through_lookback_window_days"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _view_through_lookback_window_days.
                             * @member {"view_through_lookback_window_days"|undefined} _view_through_lookback_window_days
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_view_through_lookback_window_days", {
                                get: $util.oneOfGetter($oneOfFields = ["view_through_lookback_window_days"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _phone_call_duration_seconds.
                             * @member {"phone_call_duration_seconds"|undefined} _phone_call_duration_seconds
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_phone_call_duration_seconds", {
                                get: $util.oneOfGetter($oneOfFields = ["phone_call_duration_seconds"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAction _app_id.
                             * @member {"app_id"|undefined} _app_id
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             */
                            Object.defineProperty(ConversionAction.prototype, "_app_id", {
                                get: $util.oneOfGetter($oneOfFields = ["app_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionAction instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionAction=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ConversionAction} ConversionAction instance
                             */
                            ConversionAction.create = function create(properties) {
                                return new ConversionAction(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionAction message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionAction} m ConversionAction message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAction.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(40).int32(m.type);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(48).int32(m.category);
                                if (m.value_settings != null && Object.hasOwnProperty.call(m, "value_settings"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings.encode(m.value_settings, w.uint32(90).fork()).ldelim();
                                if (m.counting_type != null && Object.hasOwnProperty.call(m, "counting_type"))
                                    w.uint32(96).int32(m.counting_type);
                                if (m.attribution_model_settings != null && Object.hasOwnProperty.call(m, "attribution_model_settings"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings.encode(m.attribution_model_settings, w.uint32(106).fork()).ldelim();
                                if (m.tag_snippets != null && m.tag_snippets.length) {
                                    for (var i = 0; i < m.tag_snippets.length; ++i)
                                        $root.google.ads.googleads.v11.common.TagSnippet.encode(m.tag_snippets[i], w.uint32(114).fork()).ldelim();
                                }
                                if (m.mobile_app_vendor != null && Object.hasOwnProperty.call(m, "mobile_app_vendor"))
                                    w.uint32(136).int32(m.mobile_app_vendor);
                                if (m.firebase_settings != null && Object.hasOwnProperty.call(m, "firebase_settings"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings.encode(m.firebase_settings, w.uint32(146).fork()).ldelim();
                                if (m.third_party_app_analytics_settings != null && Object.hasOwnProperty.call(m, "third_party_app_analytics_settings"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings.encode(m.third_party_app_analytics_settings, w.uint32(154).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(168).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(178).string(m.name);
                                if (m.owner_customer != null && Object.hasOwnProperty.call(m, "owner_customer"))
                                    w.uint32(186).string(m.owner_customer);
                                if (m.include_in_conversions_metric != null && Object.hasOwnProperty.call(m, "include_in_conversions_metric"))
                                    w.uint32(192).bool(m.include_in_conversions_metric);
                                if (m.click_through_lookback_window_days != null && Object.hasOwnProperty.call(m, "click_through_lookback_window_days"))
                                    w.uint32(200).int64(m.click_through_lookback_window_days);
                                if (m.view_through_lookback_window_days != null && Object.hasOwnProperty.call(m, "view_through_lookback_window_days"))
                                    w.uint32(208).int64(m.view_through_lookback_window_days);
                                if (m.phone_call_duration_seconds != null && Object.hasOwnProperty.call(m, "phone_call_duration_seconds"))
                                    w.uint32(216).int64(m.phone_call_duration_seconds);
                                if (m.app_id != null && Object.hasOwnProperty.call(m, "app_id"))
                                    w.uint32(226).string(m.app_id);
                                if (m.origin != null && Object.hasOwnProperty.call(m, "origin"))
                                    w.uint32(240).int32(m.origin);
                                if (m.primary_for_goal != null && Object.hasOwnProperty.call(m, "primary_for_goal"))
                                    w.uint32(248).bool(m.primary_for_goal);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionAction message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionAction} message ConversionAction message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAction.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionAction message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ConversionAction} ConversionAction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAction.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionAction();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 21:
                                        m.id = r.int64();
                                        break;
                                    case 22:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.type = r.int32();
                                        break;
                                    case 30:
                                        m.origin = r.int32();
                                        break;
                                    case 31:
                                        m.primary_for_goal = r.bool();
                                        break;
                                    case 6:
                                        m.category = r.int32();
                                        break;
                                    case 23:
                                        m.owner_customer = r.string();
                                        break;
                                    case 24:
                                        m.include_in_conversions_metric = r.bool();
                                        break;
                                    case 25:
                                        m.click_through_lookback_window_days = r.int64();
                                        break;
                                    case 26:
                                        m.view_through_lookback_window_days = r.int64();
                                        break;
                                    case 11:
                                        m.value_settings = $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.counting_type = r.int32();
                                        break;
                                    case 13:
                                        m.attribution_model_settings = $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        if (!(m.tag_snippets && m.tag_snippets.length))
                                            m.tag_snippets = [];
                                        m.tag_snippets.push($root.google.ads.googleads.v11.common.TagSnippet.decode(r, r.uint32()));
                                        break;
                                    case 27:
                                        m.phone_call_duration_seconds = r.int64();
                                        break;
                                    case 28:
                                        m.app_id = r.string();
                                        break;
                                    case 17:
                                        m.mobile_app_vendor = r.int32();
                                        break;
                                    case 18:
                                        m.firebase_settings = $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings.decode(r, r.uint32());
                                        break;
                                    case 19:
                                        m.third_party_app_analytics_settings = $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionAction message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ConversionAction} ConversionAction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAction.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionAction message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionAction.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                    case 36:
                                        break;
                                    }
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    switch (m.origin) {
                                    default:
                                        return "origin: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.primary_for_goal != null && m.hasOwnProperty("primary_for_goal")) {
                                    p._primary_for_goal = 1;
                                    if (typeof m.primary_for_goal !== "boolean")
                                        return "primary_for_goal: boolean expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    switch (m.category) {
                                    default:
                                        return "category: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                        break;
                                    }
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    p._owner_customer = 1;
                                    if (!$util.isString(m.owner_customer))
                                        return "owner_customer: string expected";
                                }
                                if (m.include_in_conversions_metric != null && m.hasOwnProperty("include_in_conversions_metric")) {
                                    p._include_in_conversions_metric = 1;
                                    if (typeof m.include_in_conversions_metric !== "boolean")
                                        return "include_in_conversions_metric: boolean expected";
                                }
                                if (m.click_through_lookback_window_days != null && m.hasOwnProperty("click_through_lookback_window_days")) {
                                    p._click_through_lookback_window_days = 1;
                                    if (!$util.isInteger(m.click_through_lookback_window_days) && !(m.click_through_lookback_window_days && $util.isInteger(m.click_through_lookback_window_days.low) && $util.isInteger(m.click_through_lookback_window_days.high)))
                                        return "click_through_lookback_window_days: integer|Long expected";
                                }
                                if (m.view_through_lookback_window_days != null && m.hasOwnProperty("view_through_lookback_window_days")) {
                                    p._view_through_lookback_window_days = 1;
                                    if (!$util.isInteger(m.view_through_lookback_window_days) && !(m.view_through_lookback_window_days && $util.isInteger(m.view_through_lookback_window_days.low) && $util.isInteger(m.view_through_lookback_window_days.high)))
                                        return "view_through_lookback_window_days: integer|Long expected";
                                }
                                if (m.value_settings != null && m.hasOwnProperty("value_settings")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings.verify(m.value_settings);
                                        if (e)
                                            return "value_settings." + e;
                                    }
                                }
                                if (m.counting_type != null && m.hasOwnProperty("counting_type")) {
                                    switch (m.counting_type) {
                                    default:
                                        return "counting_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.attribution_model_settings != null && m.hasOwnProperty("attribution_model_settings")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings.verify(m.attribution_model_settings);
                                        if (e)
                                            return "attribution_model_settings." + e;
                                    }
                                }
                                if (m.tag_snippets != null && m.hasOwnProperty("tag_snippets")) {
                                    if (!Array.isArray(m.tag_snippets))
                                        return "tag_snippets: array expected";
                                    for (var i = 0; i < m.tag_snippets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.TagSnippet.verify(m.tag_snippets[i]);
                                            if (e)
                                                return "tag_snippets." + e;
                                        }
                                    }
                                }
                                if (m.phone_call_duration_seconds != null && m.hasOwnProperty("phone_call_duration_seconds")) {
                                    p._phone_call_duration_seconds = 1;
                                    if (!$util.isInteger(m.phone_call_duration_seconds) && !(m.phone_call_duration_seconds && $util.isInteger(m.phone_call_duration_seconds.low) && $util.isInteger(m.phone_call_duration_seconds.high)))
                                        return "phone_call_duration_seconds: integer|Long expected";
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    p._app_id = 1;
                                    if (!$util.isString(m.app_id))
                                        return "app_id: string expected";
                                }
                                if (m.mobile_app_vendor != null && m.hasOwnProperty("mobile_app_vendor")) {
                                    switch (m.mobile_app_vendor) {
                                    default:
                                        return "mobile_app_vendor: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.firebase_settings != null && m.hasOwnProperty("firebase_settings")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings.verify(m.firebase_settings);
                                        if (e)
                                            return "firebase_settings." + e;
                                    }
                                }
                                if (m.third_party_app_analytics_settings != null && m.hasOwnProperty("third_party_app_analytics_settings")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings.verify(m.third_party_app_analytics_settings);
                                        if (e)
                                            return "third_party_app_analytics_settings." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionAction message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ConversionAction} ConversionAction
                             */
                            ConversionAction.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ConversionAction)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ConversionAction();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "HIDDEN":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "AD_CALL":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "CLICK_TO_CALL":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "GOOGLE_PLAY_DOWNLOAD":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "GOOGLE_PLAY_IN_APP_PURCHASE":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "UPLOAD_CALLS":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "UPLOAD_CLICKS":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "WEBPAGE":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "WEBSITE_CALL":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "STORE_SALES_DIRECT_UPLOAD":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "STORE_SALES":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "FIREBASE_ANDROID_FIRST_OPEN":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "FIREBASE_ANDROID_IN_APP_PURCHASE":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "FIREBASE_ANDROID_CUSTOM":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "FIREBASE_IOS_FIRST_OPEN":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "FIREBASE_IOS_IN_APP_PURCHASE":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "FIREBASE_IOS_CUSTOM":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS_ANDROID_FIRST_OPEN":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS_ANDROID_IN_APP_PURCHASE":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS_ANDROID_CUSTOM":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS_IOS_FIRST_OPEN":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS_IOS_IN_APP_PURCHASE":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "THIRD_PARTY_APP_ANALYTICS_IOS_CUSTOM":
                                case 23:
                                    m.type = 23;
                                    break;
                                case "ANDROID_APP_PRE_REGISTRATION":
                                case 24:
                                    m.type = 24;
                                    break;
                                case "ANDROID_INSTALLS_ALL_OTHER_APPS":
                                case 25:
                                    m.type = 25;
                                    break;
                                case "FLOODLIGHT_ACTION":
                                case 26:
                                    m.type = 26;
                                    break;
                                case "FLOODLIGHT_TRANSACTION":
                                case 27:
                                    m.type = 27;
                                    break;
                                case "GOOGLE_HOSTED":
                                case 28:
                                    m.type = 28;
                                    break;
                                case "LEAD_FORM_SUBMIT":
                                case 29:
                                    m.type = 29;
                                    break;
                                case "SALESFORCE":
                                case 30:
                                    m.type = 30;
                                    break;
                                case "SEARCH_ADS_360":
                                case 31:
                                    m.type = 31;
                                    break;
                                case "SMART_CAMPAIGN_AD_CLICKS_TO_CALL":
                                case 32:
                                    m.type = 32;
                                    break;
                                case "SMART_CAMPAIGN_MAP_CLICKS_TO_CALL":
                                case 33:
                                    m.type = 33;
                                    break;
                                case "SMART_CAMPAIGN_MAP_DIRECTIONS":
                                case 34:
                                    m.type = 34;
                                    break;
                                case "SMART_CAMPAIGN_TRACKED_CALLS":
                                case 35:
                                    m.type = 35;
                                    break;
                                case "STORE_VISITS":
                                case 36:
                                    m.type = 36;
                                    break;
                                }
                                switch (d.origin) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.origin = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.origin = 1;
                                    break;
                                case "WEBSITE":
                                case 2:
                                    m.origin = 2;
                                    break;
                                case "GOOGLE_HOSTED":
                                case 3:
                                    m.origin = 3;
                                    break;
                                case "APP":
                                case 4:
                                    m.origin = 4;
                                    break;
                                case "CALL_FROM_ADS":
                                case 5:
                                    m.origin = 5;
                                    break;
                                case "STORE":
                                case 6:
                                    m.origin = 6;
                                    break;
                                case "YOUTUBE_HOSTED":
                                case 7:
                                    m.origin = 7;
                                    break;
                                }
                                if (d.primary_for_goal != null) {
                                    m.primary_for_goal = Boolean(d.primary_for_goal);
                                }
                                switch (d.category) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.category = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.category = 1;
                                    break;
                                case "DEFAULT":
                                case 2:
                                    m.category = 2;
                                    break;
                                case "PAGE_VIEW":
                                case 3:
                                    m.category = 3;
                                    break;
                                case "PURCHASE":
                                case 4:
                                    m.category = 4;
                                    break;
                                case "SIGNUP":
                                case 5:
                                    m.category = 5;
                                    break;
                                case "DOWNLOAD":
                                case 7:
                                    m.category = 7;
                                    break;
                                case "ADD_TO_CART":
                                case 8:
                                    m.category = 8;
                                    break;
                                case "BEGIN_CHECKOUT":
                                case 9:
                                    m.category = 9;
                                    break;
                                case "SUBSCRIBE_PAID":
                                case 10:
                                    m.category = 10;
                                    break;
                                case "PHONE_CALL_LEAD":
                                case 11:
                                    m.category = 11;
                                    break;
                                case "IMPORTED_LEAD":
                                case 12:
                                    m.category = 12;
                                    break;
                                case "SUBMIT_LEAD_FORM":
                                case 13:
                                    m.category = 13;
                                    break;
                                case "BOOK_APPOINTMENT":
                                case 14:
                                    m.category = 14;
                                    break;
                                case "REQUEST_QUOTE":
                                case 15:
                                    m.category = 15;
                                    break;
                                case "GET_DIRECTIONS":
                                case 16:
                                    m.category = 16;
                                    break;
                                case "OUTBOUND_CLICK":
                                case 17:
                                    m.category = 17;
                                    break;
                                case "CONTACT":
                                case 18:
                                    m.category = 18;
                                    break;
                                case "ENGAGEMENT":
                                case 19:
                                    m.category = 19;
                                    break;
                                case "STORE_VISIT":
                                case 20:
                                    m.category = 20;
                                    break;
                                case "STORE_SALE":
                                case 21:
                                    m.category = 21;
                                    break;
                                case "QUALIFIED_LEAD":
                                case 22:
                                    m.category = 22;
                                    break;
                                case "CONVERTED_LEAD":
                                case 23:
                                    m.category = 23;
                                    break;
                                }
                                if (d.owner_customer != null) {
                                    m.owner_customer = String(d.owner_customer);
                                }
                                if (d.include_in_conversions_metric != null) {
                                    m.include_in_conversions_metric = Boolean(d.include_in_conversions_metric);
                                }
                                if (d.click_through_lookback_window_days != null) {
                                    if ($util.Long)
                                        (m.click_through_lookback_window_days = $util.Long.fromValue(d.click_through_lookback_window_days)).unsigned = false;
                                    else if (typeof d.click_through_lookback_window_days === "string")
                                        m.click_through_lookback_window_days = parseInt(d.click_through_lookback_window_days, 10);
                                    else if (typeof d.click_through_lookback_window_days === "number")
                                        m.click_through_lookback_window_days = d.click_through_lookback_window_days;
                                    else if (typeof d.click_through_lookback_window_days === "object")
                                        m.click_through_lookback_window_days = new $util.LongBits(d.click_through_lookback_window_days.low >>> 0, d.click_through_lookback_window_days.high >>> 0).toNumber();
                                }
                                if (d.view_through_lookback_window_days != null) {
                                    if ($util.Long)
                                        (m.view_through_lookback_window_days = $util.Long.fromValue(d.view_through_lookback_window_days)).unsigned = false;
                                    else if (typeof d.view_through_lookback_window_days === "string")
                                        m.view_through_lookback_window_days = parseInt(d.view_through_lookback_window_days, 10);
                                    else if (typeof d.view_through_lookback_window_days === "number")
                                        m.view_through_lookback_window_days = d.view_through_lookback_window_days;
                                    else if (typeof d.view_through_lookback_window_days === "object")
                                        m.view_through_lookback_window_days = new $util.LongBits(d.view_through_lookback_window_days.low >>> 0, d.view_through_lookback_window_days.high >>> 0).toNumber();
                                }
                                if (d.value_settings != null) {
                                    if (typeof d.value_settings !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionAction.value_settings: object expected");
                                    m.value_settings = $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings.fromObject(d.value_settings);
                                }
                                switch (d.counting_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.counting_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.counting_type = 1;
                                    break;
                                case "ONE_PER_CLICK":
                                case 2:
                                    m.counting_type = 2;
                                    break;
                                case "MANY_PER_CLICK":
                                case 3:
                                    m.counting_type = 3;
                                    break;
                                }
                                if (d.attribution_model_settings != null) {
                                    if (typeof d.attribution_model_settings !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionAction.attribution_model_settings: object expected");
                                    m.attribution_model_settings = $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings.fromObject(d.attribution_model_settings);
                                }
                                if (d.tag_snippets) {
                                    if (!Array.isArray(d.tag_snippets))
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionAction.tag_snippets: array expected");
                                    m.tag_snippets = [];
                                    for (var i = 0; i < d.tag_snippets.length; ++i) {
                                        if (typeof d.tag_snippets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ConversionAction.tag_snippets: object expected");
                                        m.tag_snippets[i] = $root.google.ads.googleads.v11.common.TagSnippet.fromObject(d.tag_snippets[i]);
                                    }
                                }
                                if (d.phone_call_duration_seconds != null) {
                                    if ($util.Long)
                                        (m.phone_call_duration_seconds = $util.Long.fromValue(d.phone_call_duration_seconds)).unsigned = false;
                                    else if (typeof d.phone_call_duration_seconds === "string")
                                        m.phone_call_duration_seconds = parseInt(d.phone_call_duration_seconds, 10);
                                    else if (typeof d.phone_call_duration_seconds === "number")
                                        m.phone_call_duration_seconds = d.phone_call_duration_seconds;
                                    else if (typeof d.phone_call_duration_seconds === "object")
                                        m.phone_call_duration_seconds = new $util.LongBits(d.phone_call_duration_seconds.low >>> 0, d.phone_call_duration_seconds.high >>> 0).toNumber();
                                }
                                if (d.app_id != null) {
                                    m.app_id = String(d.app_id);
                                }
                                switch (d.mobile_app_vendor) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.mobile_app_vendor = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.mobile_app_vendor = 1;
                                    break;
                                case "APPLE_APP_STORE":
                                case 2:
                                    m.mobile_app_vendor = 2;
                                    break;
                                case "GOOGLE_APP_STORE":
                                case 3:
                                    m.mobile_app_vendor = 3;
                                    break;
                                }
                                if (d.firebase_settings != null) {
                                    if (typeof d.firebase_settings !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionAction.firebase_settings: object expected");
                                    m.firebase_settings = $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings.fromObject(d.firebase_settings);
                                }
                                if (d.third_party_app_analytics_settings != null) {
                                    if (typeof d.third_party_app_analytics_settings !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionAction.third_party_app_analytics_settings: object expected");
                                    m.third_party_app_analytics_settings = $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings.fromObject(d.third_party_app_analytics_settings);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionAction message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.ConversionAction} m ConversionAction
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionAction.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.tag_snippets = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.category = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.value_settings = null;
                                    d.counting_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.attribution_model_settings = null;
                                    d.mobile_app_vendor = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.firebase_settings = null;
                                    d.third_party_app_analytics_settings = null;
                                    d.origin = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionStatusEnum.ConversionActionStatus[m.status] : m.status;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionTypeEnum.ConversionActionType[m.type] : m.type;
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory[m.category] : m.category;
                                }
                                if (m.value_settings != null && m.hasOwnProperty("value_settings")) {
                                    d.value_settings = $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings.toObject(m.value_settings, o);
                                }
                                if (m.counting_type != null && m.hasOwnProperty("counting_type")) {
                                    d.counting_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionCountingTypeEnum.ConversionActionCountingType[m.counting_type] : m.counting_type;
                                }
                                if (m.attribution_model_settings != null && m.hasOwnProperty("attribution_model_settings")) {
                                    d.attribution_model_settings = $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings.toObject(m.attribution_model_settings, o);
                                }
                                if (m.tag_snippets && m.tag_snippets.length) {
                                    d.tag_snippets = [];
                                    for (var j = 0; j < m.tag_snippets.length; ++j) {
                                        d.tag_snippets[j] = $root.google.ads.googleads.v11.common.TagSnippet.toObject(m.tag_snippets[j], o);
                                    }
                                }
                                if (m.mobile_app_vendor != null && m.hasOwnProperty("mobile_app_vendor")) {
                                    d.mobile_app_vendor = o.enums === String ? $root.google.ads.googleads.v11.enums.MobileAppVendorEnum.MobileAppVendor[m.mobile_app_vendor] : m.mobile_app_vendor;
                                }
                                if (m.firebase_settings != null && m.hasOwnProperty("firebase_settings")) {
                                    d.firebase_settings = $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings.toObject(m.firebase_settings, o);
                                }
                                if (m.third_party_app_analytics_settings != null && m.hasOwnProperty("third_party_app_analytics_settings")) {
                                    d.third_party_app_analytics_settings = $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings.toObject(m.third_party_app_analytics_settings, o);
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    d.owner_customer = m.owner_customer;
                                    if (o.oneofs)
                                        d._owner_customer = "owner_customer";
                                }
                                if (m.include_in_conversions_metric != null && m.hasOwnProperty("include_in_conversions_metric")) {
                                    d.include_in_conversions_metric = m.include_in_conversions_metric;
                                    if (o.oneofs)
                                        d._include_in_conversions_metric = "include_in_conversions_metric";
                                }
                                if (m.click_through_lookback_window_days != null && m.hasOwnProperty("click_through_lookback_window_days")) {
                                    if (typeof m.click_through_lookback_window_days === "number")
                                        d.click_through_lookback_window_days = o.longs === String ? String(m.click_through_lookback_window_days) : m.click_through_lookback_window_days;
                                    else
                                        d.click_through_lookback_window_days = o.longs === String ? $util.Long.prototype.toString.call(m.click_through_lookback_window_days) : o.longs === Number ? new $util.LongBits(m.click_through_lookback_window_days.low >>> 0, m.click_through_lookback_window_days.high >>> 0).toNumber() : m.click_through_lookback_window_days;
                                    if (o.oneofs)
                                        d._click_through_lookback_window_days = "click_through_lookback_window_days";
                                }
                                if (m.view_through_lookback_window_days != null && m.hasOwnProperty("view_through_lookback_window_days")) {
                                    if (typeof m.view_through_lookback_window_days === "number")
                                        d.view_through_lookback_window_days = o.longs === String ? String(m.view_through_lookback_window_days) : m.view_through_lookback_window_days;
                                    else
                                        d.view_through_lookback_window_days = o.longs === String ? $util.Long.prototype.toString.call(m.view_through_lookback_window_days) : o.longs === Number ? new $util.LongBits(m.view_through_lookback_window_days.low >>> 0, m.view_through_lookback_window_days.high >>> 0).toNumber() : m.view_through_lookback_window_days;
                                    if (o.oneofs)
                                        d._view_through_lookback_window_days = "view_through_lookback_window_days";
                                }
                                if (m.phone_call_duration_seconds != null && m.hasOwnProperty("phone_call_duration_seconds")) {
                                    if (typeof m.phone_call_duration_seconds === "number")
                                        d.phone_call_duration_seconds = o.longs === String ? String(m.phone_call_duration_seconds) : m.phone_call_duration_seconds;
                                    else
                                        d.phone_call_duration_seconds = o.longs === String ? $util.Long.prototype.toString.call(m.phone_call_duration_seconds) : o.longs === Number ? new $util.LongBits(m.phone_call_duration_seconds.low >>> 0, m.phone_call_duration_seconds.high >>> 0).toNumber() : m.phone_call_duration_seconds;
                                    if (o.oneofs)
                                        d._phone_call_duration_seconds = "phone_call_duration_seconds";
                                }
                                if (m.app_id != null && m.hasOwnProperty("app_id")) {
                                    d.app_id = m.app_id;
                                    if (o.oneofs)
                                        d._app_id = "app_id";
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    d.origin = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin[m.origin] : m.origin;
                                }
                                if (m.primary_for_goal != null && m.hasOwnProperty("primary_for_goal")) {
                                    d.primary_for_goal = m.primary_for_goal;
                                    if (o.oneofs)
                                        d._primary_for_goal = "primary_for_goal";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionAction to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ConversionAction
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionAction.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            ConversionAction.AttributionModelSettings = (function() {
    
                                /**
                                 * Properties of an AttributionModelSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @interface IAttributionModelSettings
                                 * @property {google.ads.googleads.v11.enums.AttributionModelEnum.AttributionModel|null} [attribution_model] AttributionModelSettings attribution_model
                                 * @property {google.ads.googleads.v11.enums.DataDrivenModelStatusEnum.DataDrivenModelStatus|null} [data_driven_model_status] AttributionModelSettings data_driven_model_status
                                 */
    
                                /**
                                 * Constructs a new AttributionModelSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @classdesc Represents an AttributionModelSettings.
                                 * @implements IAttributionModelSettings
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IAttributionModelSettings=} [p] Properties to set
                                 */
                                function AttributionModelSettings(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * AttributionModelSettings attribution_model.
                                 * @member {google.ads.googleads.v11.enums.AttributionModelEnum.AttributionModel} attribution_model
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @instance
                                 */
                                AttributionModelSettings.prototype.attribution_model = 0;
    
                                /**
                                 * AttributionModelSettings data_driven_model_status.
                                 * @member {google.ads.googleads.v11.enums.DataDrivenModelStatusEnum.DataDrivenModelStatus} data_driven_model_status
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @instance
                                 */
                                AttributionModelSettings.prototype.data_driven_model_status = 0;
    
                                /**
                                 * Creates a new AttributionModelSettings instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IAttributionModelSettings=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings} AttributionModelSettings instance
                                 */
                                AttributionModelSettings.create = function create(properties) {
                                    return new AttributionModelSettings(properties);
                                };
    
                                /**
                                 * Encodes the specified AttributionModelSettings message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IAttributionModelSettings} m AttributionModelSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AttributionModelSettings.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.attribution_model != null && Object.hasOwnProperty.call(m, "attribution_model"))
                                        w.uint32(8).int32(m.attribution_model);
                                    if (m.data_driven_model_status != null && Object.hasOwnProperty.call(m, "data_driven_model_status"))
                                        w.uint32(16).int32(m.data_driven_model_status);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified AttributionModelSettings message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IAttributionModelSettings} message AttributionModelSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AttributionModelSettings.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an AttributionModelSettings message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings} AttributionModelSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AttributionModelSettings.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.attribution_model = r.int32();
                                            break;
                                        case 2:
                                            m.data_driven_model_status = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an AttributionModelSettings message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings} AttributionModelSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AttributionModelSettings.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an AttributionModelSettings message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                AttributionModelSettings.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.attribution_model != null && m.hasOwnProperty("attribution_model")) {
                                        switch (m.attribution_model) {
                                        default:
                                            return "attribution_model: enum value expected";
                                        case 0:
                                        case 1:
                                        case 100:
                                        case 101:
                                        case 102:
                                        case 103:
                                        case 104:
                                        case 105:
                                        case 106:
                                            break;
                                        }
                                    }
                                    if (m.data_driven_model_status != null && m.hasOwnProperty("data_driven_model_status")) {
                                        switch (m.data_driven_model_status) {
                                        default:
                                            return "data_driven_model_status: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an AttributionModelSettings message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings} AttributionModelSettings
                                 */
                                AttributionModelSettings.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings();
                                    switch (d.attribution_model) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.attribution_model = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.attribution_model = 1;
                                        break;
                                    case "EXTERNAL":
                                    case 100:
                                        m.attribution_model = 100;
                                        break;
                                    case "GOOGLE_ADS_LAST_CLICK":
                                    case 101:
                                        m.attribution_model = 101;
                                        break;
                                    case "GOOGLE_SEARCH_ATTRIBUTION_FIRST_CLICK":
                                    case 102:
                                        m.attribution_model = 102;
                                        break;
                                    case "GOOGLE_SEARCH_ATTRIBUTION_LINEAR":
                                    case 103:
                                        m.attribution_model = 103;
                                        break;
                                    case "GOOGLE_SEARCH_ATTRIBUTION_TIME_DECAY":
                                    case 104:
                                        m.attribution_model = 104;
                                        break;
                                    case "GOOGLE_SEARCH_ATTRIBUTION_POSITION_BASED":
                                    case 105:
                                        m.attribution_model = 105;
                                        break;
                                    case "GOOGLE_SEARCH_ATTRIBUTION_DATA_DRIVEN":
                                    case 106:
                                        m.attribution_model = 106;
                                        break;
                                    }
                                    switch (d.data_driven_model_status) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.data_driven_model_status = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.data_driven_model_status = 1;
                                        break;
                                    case "AVAILABLE":
                                    case 2:
                                        m.data_driven_model_status = 2;
                                        break;
                                    case "STALE":
                                    case 3:
                                        m.data_driven_model_status = 3;
                                        break;
                                    case "EXPIRED":
                                    case 4:
                                        m.data_driven_model_status = 4;
                                        break;
                                    case "NEVER_GENERATED":
                                    case 5:
                                        m.data_driven_model_status = 5;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an AttributionModelSettings message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings} m AttributionModelSettings
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                AttributionModelSettings.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.attribution_model = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.data_driven_model_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.attribution_model != null && m.hasOwnProperty("attribution_model")) {
                                        d.attribution_model = o.enums === String ? $root.google.ads.googleads.v11.enums.AttributionModelEnum.AttributionModel[m.attribution_model] : m.attribution_model;
                                    }
                                    if (m.data_driven_model_status != null && m.hasOwnProperty("data_driven_model_status")) {
                                        d.data_driven_model_status = o.enums === String ? $root.google.ads.googleads.v11.enums.DataDrivenModelStatusEnum.DataDrivenModelStatus[m.data_driven_model_status] : m.data_driven_model_status;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this AttributionModelSettings to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.AttributionModelSettings
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                AttributionModelSettings.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return AttributionModelSettings;
                            })();
    
                            ConversionAction.ValueSettings = (function() {
    
                                /**
                                 * Properties of a ValueSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @interface IValueSettings
                                 * @property {number|null} [default_value] ValueSettings default_value
                                 * @property {string|null} [default_currency_code] ValueSettings default_currency_code
                                 * @property {boolean|null} [always_use_default_value] ValueSettings always_use_default_value
                                 */
    
                                /**
                                 * Constructs a new ValueSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @classdesc Represents a ValueSettings.
                                 * @implements IValueSettings
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IValueSettings=} [p] Properties to set
                                 */
                                function ValueSettings(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ValueSettings default_value.
                                 * @member {number|null|undefined} default_value
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @instance
                                 */
                                ValueSettings.prototype.default_value = null;
    
                                /**
                                 * ValueSettings default_currency_code.
                                 * @member {string|null|undefined} default_currency_code
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @instance
                                 */
                                ValueSettings.prototype.default_currency_code = null;
    
                                /**
                                 * ValueSettings always_use_default_value.
                                 * @member {boolean|null|undefined} always_use_default_value
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @instance
                                 */
                                ValueSettings.prototype.always_use_default_value = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ValueSettings _default_value.
                                 * @member {"default_value"|undefined} _default_value
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @instance
                                 */
                                Object.defineProperty(ValueSettings.prototype, "_default_value", {
                                    get: $util.oneOfGetter($oneOfFields = ["default_value"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * ValueSettings _default_currency_code.
                                 * @member {"default_currency_code"|undefined} _default_currency_code
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @instance
                                 */
                                Object.defineProperty(ValueSettings.prototype, "_default_currency_code", {
                                    get: $util.oneOfGetter($oneOfFields = ["default_currency_code"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * ValueSettings _always_use_default_value.
                                 * @member {"always_use_default_value"|undefined} _always_use_default_value
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @instance
                                 */
                                Object.defineProperty(ValueSettings.prototype, "_always_use_default_value", {
                                    get: $util.oneOfGetter($oneOfFields = ["always_use_default_value"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ValueSettings instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IValueSettings=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ValueSettings} ValueSettings instance
                                 */
                                ValueSettings.create = function create(properties) {
                                    return new ValueSettings(properties);
                                };
    
                                /**
                                 * Encodes the specified ValueSettings message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.ValueSettings.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IValueSettings} m ValueSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueSettings.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.default_value != null && Object.hasOwnProperty.call(m, "default_value"))
                                        w.uint32(33).double(m.default_value);
                                    if (m.default_currency_code != null && Object.hasOwnProperty.call(m, "default_currency_code"))
                                        w.uint32(42).string(m.default_currency_code);
                                    if (m.always_use_default_value != null && Object.hasOwnProperty.call(m, "always_use_default_value"))
                                        w.uint32(48).bool(m.always_use_default_value);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ValueSettings message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.ValueSettings.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IValueSettings} message ValueSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueSettings.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ValueSettings message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ValueSettings} ValueSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueSettings.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 4:
                                            m.default_value = r.double();
                                            break;
                                        case 5:
                                            m.default_currency_code = r.string();
                                            break;
                                        case 6:
                                            m.always_use_default_value = r.bool();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ValueSettings message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ValueSettings} ValueSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueSettings.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ValueSettings message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ValueSettings.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.default_value != null && m.hasOwnProperty("default_value")) {
                                        p._default_value = 1;
                                        if (typeof m.default_value !== "number")
                                            return "default_value: number expected";
                                    }
                                    if (m.default_currency_code != null && m.hasOwnProperty("default_currency_code")) {
                                        p._default_currency_code = 1;
                                        if (!$util.isString(m.default_currency_code))
                                            return "default_currency_code: string expected";
                                    }
                                    if (m.always_use_default_value != null && m.hasOwnProperty("always_use_default_value")) {
                                        p._always_use_default_value = 1;
                                        if (typeof m.always_use_default_value !== "boolean")
                                            return "always_use_default_value: boolean expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ValueSettings message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ValueSettings} ValueSettings
                                 */
                                ValueSettings.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionAction.ValueSettings();
                                    if (d.default_value != null) {
                                        m.default_value = Number(d.default_value);
                                    }
                                    if (d.default_currency_code != null) {
                                        m.default_currency_code = String(d.default_currency_code);
                                    }
                                    if (d.always_use_default_value != null) {
                                        m.always_use_default_value = Boolean(d.always_use_default_value);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ValueSettings message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.ValueSettings} m ValueSettings
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ValueSettings.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.default_value != null && m.hasOwnProperty("default_value")) {
                                        d.default_value = o.json && !isFinite(m.default_value) ? String(m.default_value) : m.default_value;
                                        if (o.oneofs)
                                            d._default_value = "default_value";
                                    }
                                    if (m.default_currency_code != null && m.hasOwnProperty("default_currency_code")) {
                                        d.default_currency_code = m.default_currency_code;
                                        if (o.oneofs)
                                            d._default_currency_code = "default_currency_code";
                                    }
                                    if (m.always_use_default_value != null && m.hasOwnProperty("always_use_default_value")) {
                                        d.always_use_default_value = m.always_use_default_value;
                                        if (o.oneofs)
                                            d._always_use_default_value = "always_use_default_value";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ValueSettings to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ValueSettings
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ValueSettings.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ValueSettings;
                            })();
    
                            ConversionAction.ThirdPartyAppAnalyticsSettings = (function() {
    
                                /**
                                 * Properties of a ThirdPartyAppAnalyticsSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @interface IThirdPartyAppAnalyticsSettings
                                 * @property {string|null} [event_name] ThirdPartyAppAnalyticsSettings event_name
                                 * @property {string|null} [provider_name] ThirdPartyAppAnalyticsSettings provider_name
                                 */
    
                                /**
                                 * Constructs a new ThirdPartyAppAnalyticsSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @classdesc Represents a ThirdPartyAppAnalyticsSettings.
                                 * @implements IThirdPartyAppAnalyticsSettings
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IThirdPartyAppAnalyticsSettings=} [p] Properties to set
                                 */
                                function ThirdPartyAppAnalyticsSettings(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ThirdPartyAppAnalyticsSettings event_name.
                                 * @member {string|null|undefined} event_name
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @instance
                                 */
                                ThirdPartyAppAnalyticsSettings.prototype.event_name = null;
    
                                /**
                                 * ThirdPartyAppAnalyticsSettings provider_name.
                                 * @member {string} provider_name
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @instance
                                 */
                                ThirdPartyAppAnalyticsSettings.prototype.provider_name = "";
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * ThirdPartyAppAnalyticsSettings _event_name.
                                 * @member {"event_name"|undefined} _event_name
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @instance
                                 */
                                Object.defineProperty(ThirdPartyAppAnalyticsSettings.prototype, "_event_name", {
                                    get: $util.oneOfGetter($oneOfFields = ["event_name"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new ThirdPartyAppAnalyticsSettings instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IThirdPartyAppAnalyticsSettings=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings} ThirdPartyAppAnalyticsSettings instance
                                 */
                                ThirdPartyAppAnalyticsSettings.create = function create(properties) {
                                    return new ThirdPartyAppAnalyticsSettings(properties);
                                };
    
                                /**
                                 * Encodes the specified ThirdPartyAppAnalyticsSettings message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IThirdPartyAppAnalyticsSettings} m ThirdPartyAppAnalyticsSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ThirdPartyAppAnalyticsSettings.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.event_name != null && Object.hasOwnProperty.call(m, "event_name"))
                                        w.uint32(18).string(m.event_name);
                                    if (m.provider_name != null && Object.hasOwnProperty.call(m, "provider_name"))
                                        w.uint32(26).string(m.provider_name);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ThirdPartyAppAnalyticsSettings message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IThirdPartyAppAnalyticsSettings} message ThirdPartyAppAnalyticsSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ThirdPartyAppAnalyticsSettings.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ThirdPartyAppAnalyticsSettings message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings} ThirdPartyAppAnalyticsSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ThirdPartyAppAnalyticsSettings.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.event_name = r.string();
                                            break;
                                        case 3:
                                            m.provider_name = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ThirdPartyAppAnalyticsSettings message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings} ThirdPartyAppAnalyticsSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ThirdPartyAppAnalyticsSettings.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ThirdPartyAppAnalyticsSettings message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ThirdPartyAppAnalyticsSettings.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.event_name != null && m.hasOwnProperty("event_name")) {
                                        p._event_name = 1;
                                        if (!$util.isString(m.event_name))
                                            return "event_name: string expected";
                                    }
                                    if (m.provider_name != null && m.hasOwnProperty("provider_name")) {
                                        if (!$util.isString(m.provider_name))
                                            return "provider_name: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ThirdPartyAppAnalyticsSettings message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings} ThirdPartyAppAnalyticsSettings
                                 */
                                ThirdPartyAppAnalyticsSettings.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings();
                                    if (d.event_name != null) {
                                        m.event_name = String(d.event_name);
                                    }
                                    if (d.provider_name != null) {
                                        m.provider_name = String(d.provider_name);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ThirdPartyAppAnalyticsSettings message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings} m ThirdPartyAppAnalyticsSettings
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ThirdPartyAppAnalyticsSettings.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.provider_name = "";
                                    }
                                    if (m.event_name != null && m.hasOwnProperty("event_name")) {
                                        d.event_name = m.event_name;
                                        if (o.oneofs)
                                            d._event_name = "event_name";
                                    }
                                    if (m.provider_name != null && m.hasOwnProperty("provider_name")) {
                                        d.provider_name = m.provider_name;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ThirdPartyAppAnalyticsSettings to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.ThirdPartyAppAnalyticsSettings
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ThirdPartyAppAnalyticsSettings.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ThirdPartyAppAnalyticsSettings;
                            })();
    
                            ConversionAction.FirebaseSettings = (function() {
    
                                /**
                                 * Properties of a FirebaseSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @interface IFirebaseSettings
                                 * @property {string|null} [event_name] FirebaseSettings event_name
                                 * @property {string|null} [project_id] FirebaseSettings project_id
                                 * @property {number|null} [property_id] FirebaseSettings property_id
                                 * @property {string|null} [property_name] FirebaseSettings property_name
                                 */
    
                                /**
                                 * Constructs a new FirebaseSettings.
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction
                                 * @classdesc Represents a FirebaseSettings.
                                 * @implements IFirebaseSettings
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IFirebaseSettings=} [p] Properties to set
                                 */
                                function FirebaseSettings(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * FirebaseSettings event_name.
                                 * @member {string|null|undefined} event_name
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @instance
                                 */
                                FirebaseSettings.prototype.event_name = null;
    
                                /**
                                 * FirebaseSettings project_id.
                                 * @member {string|null|undefined} project_id
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @instance
                                 */
                                FirebaseSettings.prototype.project_id = null;
    
                                /**
                                 * FirebaseSettings property_id.
                                 * @member {number} property_id
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @instance
                                 */
                                FirebaseSettings.prototype.property_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * FirebaseSettings property_name.
                                 * @member {string} property_name
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @instance
                                 */
                                FirebaseSettings.prototype.property_name = "";
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * FirebaseSettings _event_name.
                                 * @member {"event_name"|undefined} _event_name
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @instance
                                 */
                                Object.defineProperty(FirebaseSettings.prototype, "_event_name", {
                                    get: $util.oneOfGetter($oneOfFields = ["event_name"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * FirebaseSettings _project_id.
                                 * @member {"project_id"|undefined} _project_id
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @instance
                                 */
                                Object.defineProperty(FirebaseSettings.prototype, "_project_id", {
                                    get: $util.oneOfGetter($oneOfFields = ["project_id"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new FirebaseSettings instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IFirebaseSettings=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings} FirebaseSettings instance
                                 */
                                FirebaseSettings.create = function create(properties) {
                                    return new FirebaseSettings(properties);
                                };
    
                                /**
                                 * Encodes the specified FirebaseSettings message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IFirebaseSettings} m FirebaseSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FirebaseSettings.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.event_name != null && Object.hasOwnProperty.call(m, "event_name"))
                                        w.uint32(26).string(m.event_name);
                                    if (m.project_id != null && Object.hasOwnProperty.call(m, "project_id"))
                                        w.uint32(34).string(m.project_id);
                                    if (m.property_id != null && Object.hasOwnProperty.call(m, "property_id"))
                                        w.uint32(40).int64(m.property_id);
                                    if (m.property_name != null && Object.hasOwnProperty.call(m, "property_name"))
                                        w.uint32(50).string(m.property_name);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified FirebaseSettings message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.IFirebaseSettings} message FirebaseSettings message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                FirebaseSettings.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a FirebaseSettings message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings} FirebaseSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FirebaseSettings.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 3:
                                            m.event_name = r.string();
                                            break;
                                        case 4:
                                            m.project_id = r.string();
                                            break;
                                        case 5:
                                            m.property_id = r.int64();
                                            break;
                                        case 6:
                                            m.property_name = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a FirebaseSettings message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings} FirebaseSettings
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                FirebaseSettings.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a FirebaseSettings message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                FirebaseSettings.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.event_name != null && m.hasOwnProperty("event_name")) {
                                        p._event_name = 1;
                                        if (!$util.isString(m.event_name))
                                            return "event_name: string expected";
                                    }
                                    if (m.project_id != null && m.hasOwnProperty("project_id")) {
                                        p._project_id = 1;
                                        if (!$util.isString(m.project_id))
                                            return "project_id: string expected";
                                    }
                                    if (m.property_id != null && m.hasOwnProperty("property_id")) {
                                        if (!$util.isInteger(m.property_id) && !(m.property_id && $util.isInteger(m.property_id.low) && $util.isInteger(m.property_id.high)))
                                            return "property_id: integer|Long expected";
                                    }
                                    if (m.property_name != null && m.hasOwnProperty("property_name")) {
                                        if (!$util.isString(m.property_name))
                                            return "property_name: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a FirebaseSettings message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings} FirebaseSettings
                                 */
                                FirebaseSettings.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings();
                                    if (d.event_name != null) {
                                        m.event_name = String(d.event_name);
                                    }
                                    if (d.project_id != null) {
                                        m.project_id = String(d.project_id);
                                    }
                                    if (d.property_id != null) {
                                        if ($util.Long)
                                            (m.property_id = $util.Long.fromValue(d.property_id)).unsigned = false;
                                        else if (typeof d.property_id === "string")
                                            m.property_id = parseInt(d.property_id, 10);
                                        else if (typeof d.property_id === "number")
                                            m.property_id = d.property_id;
                                        else if (typeof d.property_id === "object")
                                            m.property_id = new $util.LongBits(d.property_id.low >>> 0, d.property_id.high >>> 0).toNumber();
                                    }
                                    if (d.property_name != null) {
                                        m.property_name = String(d.property_name);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a FirebaseSettings message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings} m FirebaseSettings
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                FirebaseSettings.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.property_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.property_id = o.longs === String ? "0" : 0;
                                        d.property_name = "";
                                    }
                                    if (m.event_name != null && m.hasOwnProperty("event_name")) {
                                        d.event_name = m.event_name;
                                        if (o.oneofs)
                                            d._event_name = "event_name";
                                    }
                                    if (m.project_id != null && m.hasOwnProperty("project_id")) {
                                        d.project_id = m.project_id;
                                        if (o.oneofs)
                                            d._project_id = "project_id";
                                    }
                                    if (m.property_id != null && m.hasOwnProperty("property_id")) {
                                        if (typeof m.property_id === "number")
                                            d.property_id = o.longs === String ? String(m.property_id) : m.property_id;
                                        else
                                            d.property_id = o.longs === String ? $util.Long.prototype.toString.call(m.property_id) : o.longs === Number ? new $util.LongBits(m.property_id.low >>> 0, m.property_id.high >>> 0).toNumber() : m.property_id;
                                    }
                                    if (m.property_name != null && m.hasOwnProperty("property_name")) {
                                        d.property_name = m.property_name;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this FirebaseSettings to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionAction.FirebaseSettings
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                FirebaseSettings.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return FirebaseSettings;
                            })();
    
                            return ConversionAction;
                        })();
    
                        resources.ConversionCustomVariable = (function() {
    
                            /**
                             * Properties of a ConversionCustomVariable.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IConversionCustomVariable
                             * @property {string|null} [resource_name] ConversionCustomVariable resource_name
                             * @property {number|null} [id] ConversionCustomVariable id
                             * @property {string|null} [name] ConversionCustomVariable name
                             * @property {string|null} [tag] ConversionCustomVariable tag
                             * @property {google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus|null} [status] ConversionCustomVariable status
                             * @property {string|null} [owner_customer] ConversionCustomVariable owner_customer
                             */
    
                            /**
                             * Constructs a new ConversionCustomVariable.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ConversionCustomVariable.
                             * @implements IConversionCustomVariable
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IConversionCustomVariable=} [p] Properties to set
                             */
                            function ConversionCustomVariable(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionCustomVariable resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @instance
                             */
                            ConversionCustomVariable.prototype.resource_name = "";
    
                            /**
                             * ConversionCustomVariable id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @instance
                             */
                            ConversionCustomVariable.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * ConversionCustomVariable name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @instance
                             */
                            ConversionCustomVariable.prototype.name = "";
    
                            /**
                             * ConversionCustomVariable tag.
                             * @member {string} tag
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @instance
                             */
                            ConversionCustomVariable.prototype.tag = "";
    
                            /**
                             * ConversionCustomVariable status.
                             * @member {google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus} status
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @instance
                             */
                            ConversionCustomVariable.prototype.status = 0;
    
                            /**
                             * ConversionCustomVariable owner_customer.
                             * @member {string} owner_customer
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @instance
                             */
                            ConversionCustomVariable.prototype.owner_customer = "";
    
                            /**
                             * Creates a new ConversionCustomVariable instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionCustomVariable=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ConversionCustomVariable} ConversionCustomVariable instance
                             */
                            ConversionCustomVariable.create = function create(properties) {
                                return new ConversionCustomVariable(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariable message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionCustomVariable.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionCustomVariable} m ConversionCustomVariable message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariable.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.tag != null && Object.hasOwnProperty.call(m, "tag"))
                                    w.uint32(34).string(m.tag);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.owner_customer != null && Object.hasOwnProperty.call(m, "owner_customer"))
                                    w.uint32(50).string(m.owner_customer);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariable message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionCustomVariable.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionCustomVariable} message ConversionCustomVariable message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariable.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionCustomVariable message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ConversionCustomVariable} ConversionCustomVariable
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariable.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionCustomVariable();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.tag = r.string();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 6:
                                        m.owner_customer = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionCustomVariable message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ConversionCustomVariable} ConversionCustomVariable
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariable.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionCustomVariable message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionCustomVariable.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.tag != null && m.hasOwnProperty("tag")) {
                                    if (!$util.isString(m.tag))
                                        return "tag: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    if (!$util.isString(m.owner_customer))
                                        return "owner_customer: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionCustomVariable message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ConversionCustomVariable} ConversionCustomVariable
                             */
                            ConversionCustomVariable.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ConversionCustomVariable)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ConversionCustomVariable();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.tag != null) {
                                    m.tag = String(d.tag);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ACTIVATION_NEEDED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "ENABLED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PAUSED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.owner_customer != null) {
                                    m.owner_customer = String(d.owner_customer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionCustomVariable message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.resources.ConversionCustomVariable} m ConversionCustomVariable
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionCustomVariable.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.name = "";
                                    d.tag = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.owner_customer = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.tag != null && m.hasOwnProperty("tag")) {
                                    d.tag = m.tag;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus[m.status] : m.status;
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    d.owner_customer = m.owner_customer;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionCustomVariable to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ConversionCustomVariable
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionCustomVariable.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionCustomVariable;
                        })();
    
                        resources.ConversionGoalCampaignConfig = (function() {
    
                            /**
                             * Properties of a ConversionGoalCampaignConfig.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IConversionGoalCampaignConfig
                             * @property {string|null} [resource_name] ConversionGoalCampaignConfig resource_name
                             * @property {string|null} [campaign] ConversionGoalCampaignConfig campaign
                             * @property {google.ads.googleads.v11.enums.GoalConfigLevelEnum.GoalConfigLevel|null} [goal_config_level] ConversionGoalCampaignConfig goal_config_level
                             * @property {string|null} [custom_conversion_goal] ConversionGoalCampaignConfig custom_conversion_goal
                             */
    
                            /**
                             * Constructs a new ConversionGoalCampaignConfig.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ConversionGoalCampaignConfig.
                             * @implements IConversionGoalCampaignConfig
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig=} [p] Properties to set
                             */
                            function ConversionGoalCampaignConfig(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionGoalCampaignConfig resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @instance
                             */
                            ConversionGoalCampaignConfig.prototype.resource_name = "";
    
                            /**
                             * ConversionGoalCampaignConfig campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @instance
                             */
                            ConversionGoalCampaignConfig.prototype.campaign = "";
    
                            /**
                             * ConversionGoalCampaignConfig goal_config_level.
                             * @member {google.ads.googleads.v11.enums.GoalConfigLevelEnum.GoalConfigLevel} goal_config_level
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @instance
                             */
                            ConversionGoalCampaignConfig.prototype.goal_config_level = 0;
    
                            /**
                             * ConversionGoalCampaignConfig custom_conversion_goal.
                             * @member {string} custom_conversion_goal
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @instance
                             */
                            ConversionGoalCampaignConfig.prototype.custom_conversion_goal = "";
    
                            /**
                             * Creates a new ConversionGoalCampaignConfig instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ConversionGoalCampaignConfig} ConversionGoalCampaignConfig instance
                             */
                            ConversionGoalCampaignConfig.create = function create(properties) {
                                return new ConversionGoalCampaignConfig(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionGoalCampaignConfig message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig} m ConversionGoalCampaignConfig message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionGoalCampaignConfig.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                if (m.goal_config_level != null && Object.hasOwnProperty.call(m, "goal_config_level"))
                                    w.uint32(24).int32(m.goal_config_level);
                                if (m.custom_conversion_goal != null && Object.hasOwnProperty.call(m, "custom_conversion_goal"))
                                    w.uint32(34).string(m.custom_conversion_goal);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionGoalCampaignConfig message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig} message ConversionGoalCampaignConfig message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionGoalCampaignConfig.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionGoalCampaignConfig message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ConversionGoalCampaignConfig} ConversionGoalCampaignConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionGoalCampaignConfig.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    case 3:
                                        m.goal_config_level = r.int32();
                                        break;
                                    case 4:
                                        m.custom_conversion_goal = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionGoalCampaignConfig message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ConversionGoalCampaignConfig} ConversionGoalCampaignConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionGoalCampaignConfig.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionGoalCampaignConfig message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionGoalCampaignConfig.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.goal_config_level != null && m.hasOwnProperty("goal_config_level")) {
                                    switch (m.goal_config_level) {
                                    default:
                                        return "goal_config_level: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.custom_conversion_goal != null && m.hasOwnProperty("custom_conversion_goal")) {
                                    if (!$util.isString(m.custom_conversion_goal))
                                        return "custom_conversion_goal: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionGoalCampaignConfig message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ConversionGoalCampaignConfig} ConversionGoalCampaignConfig
                             */
                            ConversionGoalCampaignConfig.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                switch (d.goal_config_level) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.goal_config_level = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.goal_config_level = 1;
                                    break;
                                case "CUSTOMER":
                                case 2:
                                    m.goal_config_level = 2;
                                    break;
                                case "CAMPAIGN":
                                case 3:
                                    m.goal_config_level = 3;
                                    break;
                                }
                                if (d.custom_conversion_goal != null) {
                                    m.custom_conversion_goal = String(d.custom_conversion_goal);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionGoalCampaignConfig message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @static
                             * @param {google.ads.googleads.v11.resources.ConversionGoalCampaignConfig} m ConversionGoalCampaignConfig
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionGoalCampaignConfig.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign = "";
                                    d.goal_config_level = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.custom_conversion_goal = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.goal_config_level != null && m.hasOwnProperty("goal_config_level")) {
                                    d.goal_config_level = o.enums === String ? $root.google.ads.googleads.v11.enums.GoalConfigLevelEnum.GoalConfigLevel[m.goal_config_level] : m.goal_config_level;
                                }
                                if (m.custom_conversion_goal != null && m.hasOwnProperty("custom_conversion_goal")) {
                                    d.custom_conversion_goal = m.custom_conversion_goal;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionGoalCampaignConfig to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ConversionGoalCampaignConfig
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionGoalCampaignConfig.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionGoalCampaignConfig;
                        })();
    
                        resources.ConversionValueRule = (function() {
    
                            /**
                             * Properties of a ConversionValueRule.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IConversionValueRule
                             * @property {string|null} [resource_name] ConversionValueRule resource_name
                             * @property {number|null} [id] ConversionValueRule id
                             * @property {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAction|null} [action] ConversionValueRule action
                             * @property {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleGeoLocationCondition|null} [geo_location_condition] ConversionValueRule geo_location_condition
                             * @property {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleDeviceCondition|null} [device_condition] ConversionValueRule device_condition
                             * @property {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAudienceCondition|null} [audience_condition] ConversionValueRule audience_condition
                             * @property {string|null} [owner_customer] ConversionValueRule owner_customer
                             * @property {google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum.ConversionValueRuleStatus|null} [status] ConversionValueRule status
                             */
    
                            /**
                             * Constructs a new ConversionValueRule.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ConversionValueRule.
                             * @implements IConversionValueRule
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IConversionValueRule=} [p] Properties to set
                             */
                            function ConversionValueRule(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionValueRule resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.resource_name = "";
    
                            /**
                             * ConversionValueRule id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * ConversionValueRule action.
                             * @member {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAction|null|undefined} action
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.action = null;
    
                            /**
                             * ConversionValueRule geo_location_condition.
                             * @member {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleGeoLocationCondition|null|undefined} geo_location_condition
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.geo_location_condition = null;
    
                            /**
                             * ConversionValueRule device_condition.
                             * @member {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleDeviceCondition|null|undefined} device_condition
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.device_condition = null;
    
                            /**
                             * ConversionValueRule audience_condition.
                             * @member {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAudienceCondition|null|undefined} audience_condition
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.audience_condition = null;
    
                            /**
                             * ConversionValueRule owner_customer.
                             * @member {string} owner_customer
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.owner_customer = "";
    
                            /**
                             * ConversionValueRule status.
                             * @member {google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum.ConversionValueRuleStatus} status
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             */
                            ConversionValueRule.prototype.status = 0;
    
                            /**
                             * Creates a new ConversionValueRule instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionValueRule=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRule} ConversionValueRule instance
                             */
                            ConversionValueRule.create = function create(properties) {
                                return new ConversionValueRule(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRule message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionValueRule} m ConversionValueRule message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRule.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.action != null && Object.hasOwnProperty.call(m, "action"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction.encode(m.action, w.uint32(26).fork()).ldelim();
                                if (m.geo_location_condition != null && Object.hasOwnProperty.call(m, "geo_location_condition"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.encode(m.geo_location_condition, w.uint32(34).fork()).ldelim();
                                if (m.device_condition != null && Object.hasOwnProperty.call(m, "device_condition"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.encode(m.device_condition, w.uint32(42).fork()).ldelim();
                                if (m.audience_condition != null && Object.hasOwnProperty.call(m, "audience_condition"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.encode(m.audience_condition, w.uint32(50).fork()).ldelim();
                                if (m.owner_customer != null && Object.hasOwnProperty.call(m, "owner_customer"))
                                    w.uint32(58).string(m.owner_customer);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(64).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRule message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionValueRule} message ConversionValueRule message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRule.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRule message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRule} ConversionValueRule
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRule.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionValueRule();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.action = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.geo_location_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.device_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.audience_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.owner_customer = r.string();
                                        break;
                                    case 8:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRule message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRule} ConversionValueRule
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRule.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRule message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRule.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.action != null && m.hasOwnProperty("action")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction.verify(m.action);
                                        if (e)
                                            return "action." + e;
                                    }
                                }
                                if (m.geo_location_condition != null && m.hasOwnProperty("geo_location_condition")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.verify(m.geo_location_condition);
                                        if (e)
                                            return "geo_location_condition." + e;
                                    }
                                }
                                if (m.device_condition != null && m.hasOwnProperty("device_condition")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.verify(m.device_condition);
                                        if (e)
                                            return "device_condition." + e;
                                    }
                                }
                                if (m.audience_condition != null && m.hasOwnProperty("audience_condition")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.verify(m.audience_condition);
                                        if (e)
                                            return "audience_condition." + e;
                                    }
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    if (!$util.isString(m.owner_customer))
                                        return "owner_customer: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRule message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRule} ConversionValueRule
                             */
                            ConversionValueRule.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ConversionValueRule)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ConversionValueRule();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.action != null) {
                                    if (typeof d.action !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.action: object expected");
                                    m.action = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction.fromObject(d.action);
                                }
                                if (d.geo_location_condition != null) {
                                    if (typeof d.geo_location_condition !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.geo_location_condition: object expected");
                                    m.geo_location_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.fromObject(d.geo_location_condition);
                                }
                                if (d.device_condition != null) {
                                    if (typeof d.device_condition !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.device_condition: object expected");
                                    m.device_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.fromObject(d.device_condition);
                                }
                                if (d.audience_condition != null) {
                                    if (typeof d.audience_condition !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.audience_condition: object expected");
                                    m.audience_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.fromObject(d.audience_condition);
                                }
                                if (d.owner_customer != null) {
                                    m.owner_customer = String(d.owner_customer);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PAUSED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRule message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @static
                             * @param {google.ads.googleads.v11.resources.ConversionValueRule} m ConversionValueRule
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRule.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.action = null;
                                    d.geo_location_condition = null;
                                    d.device_condition = null;
                                    d.audience_condition = null;
                                    d.owner_customer = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.action != null && m.hasOwnProperty("action")) {
                                    d.action = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction.toObject(m.action, o);
                                }
                                if (m.geo_location_condition != null && m.hasOwnProperty("geo_location_condition")) {
                                    d.geo_location_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.toObject(m.geo_location_condition, o);
                                }
                                if (m.device_condition != null && m.hasOwnProperty("device_condition")) {
                                    d.device_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.toObject(m.device_condition, o);
                                }
                                if (m.audience_condition != null && m.hasOwnProperty("audience_condition")) {
                                    d.audience_condition = $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.toObject(m.audience_condition, o);
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    d.owner_customer = m.owner_customer;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionValueRuleStatusEnum.ConversionValueRuleStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionValueRule to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRule.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            ConversionValueRule.ValueRuleAction = (function() {
    
                                /**
                                 * Properties of a ValueRuleAction.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @interface IValueRuleAction
                                 * @property {google.ads.googleads.v11.enums.ValueRuleOperationEnum.ValueRuleOperation|null} [operation] ValueRuleAction operation
                                 * @property {number|null} [value] ValueRuleAction value
                                 */
    
                                /**
                                 * Constructs a new ValueRuleAction.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @classdesc Represents a ValueRuleAction.
                                 * @implements IValueRuleAction
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAction=} [p] Properties to set
                                 */
                                function ValueRuleAction(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ValueRuleAction operation.
                                 * @member {google.ads.googleads.v11.enums.ValueRuleOperationEnum.ValueRuleOperation} operation
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @instance
                                 */
                                ValueRuleAction.prototype.operation = 0;
    
                                /**
                                 * ValueRuleAction value.
                                 * @member {number} value
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @instance
                                 */
                                ValueRuleAction.prototype.value = 0;
    
                                /**
                                 * Creates a new ValueRuleAction instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAction=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction} ValueRuleAction instance
                                 */
                                ValueRuleAction.create = function create(properties) {
                                    return new ValueRuleAction(properties);
                                };
    
                                /**
                                 * Encodes the specified ValueRuleAction message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAction} m ValueRuleAction message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleAction.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                        w.uint32(8).int32(m.operation);
                                    if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                        w.uint32(17).double(m.value);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ValueRuleAction message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAction} message ValueRuleAction message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleAction.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ValueRuleAction message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction} ValueRuleAction
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleAction.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.operation = r.int32();
                                            break;
                                        case 2:
                                            m.value = r.double();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ValueRuleAction message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction} ValueRuleAction
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleAction.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ValueRuleAction message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ValueRuleAction.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.operation != null && m.hasOwnProperty("operation")) {
                                        switch (m.operation) {
                                        default:
                                            return "operation: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        if (typeof m.value !== "number")
                                            return "value: number expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ValueRuleAction message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction} ValueRuleAction
                                 */
                                ValueRuleAction.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction();
                                    switch (d.operation) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.operation = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.operation = 1;
                                        break;
                                    case "ADD":
                                    case 2:
                                        m.operation = 2;
                                        break;
                                    case "MULTIPLY":
                                    case 3:
                                        m.operation = 3;
                                        break;
                                    case "SET":
                                    case 4:
                                        m.operation = 4;
                                        break;
                                    }
                                    if (d.value != null) {
                                        m.value = Number(d.value);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ValueRuleAction message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction} m ValueRuleAction
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ValueRuleAction.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.operation = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.value = 0;
                                    }
                                    if (m.operation != null && m.hasOwnProperty("operation")) {
                                        d.operation = o.enums === String ? $root.google.ads.googleads.v11.enums.ValueRuleOperationEnum.ValueRuleOperation[m.operation] : m.operation;
                                    }
                                    if (m.value != null && m.hasOwnProperty("value")) {
                                        d.value = o.json && !isFinite(m.value) ? String(m.value) : m.value;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ValueRuleAction to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAction
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ValueRuleAction.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ValueRuleAction;
                            })();
    
                            ConversionValueRule.ValueRuleGeoLocationCondition = (function() {
    
                                /**
                                 * Properties of a ValueRuleGeoLocationCondition.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @interface IValueRuleGeoLocationCondition
                                 * @property {Array.<string>|null} [excluded_geo_target_constants] ValueRuleGeoLocationCondition excluded_geo_target_constants
                                 * @property {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType|null} [excluded_geo_match_type] ValueRuleGeoLocationCondition excluded_geo_match_type
                                 * @property {Array.<string>|null} [geo_target_constants] ValueRuleGeoLocationCondition geo_target_constants
                                 * @property {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType|null} [geo_match_type] ValueRuleGeoLocationCondition geo_match_type
                                 */
    
                                /**
                                 * Constructs a new ValueRuleGeoLocationCondition.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @classdesc Represents a ValueRuleGeoLocationCondition.
                                 * @implements IValueRuleGeoLocationCondition
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleGeoLocationCondition=} [p] Properties to set
                                 */
                                function ValueRuleGeoLocationCondition(p) {
                                    this.excluded_geo_target_constants = [];
                                    this.geo_target_constants = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ValueRuleGeoLocationCondition excluded_geo_target_constants.
                                 * @member {Array.<string>} excluded_geo_target_constants
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @instance
                                 */
                                ValueRuleGeoLocationCondition.prototype.excluded_geo_target_constants = $util.emptyArray;
    
                                /**
                                 * ValueRuleGeoLocationCondition excluded_geo_match_type.
                                 * @member {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType} excluded_geo_match_type
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @instance
                                 */
                                ValueRuleGeoLocationCondition.prototype.excluded_geo_match_type = 0;
    
                                /**
                                 * ValueRuleGeoLocationCondition geo_target_constants.
                                 * @member {Array.<string>} geo_target_constants
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @instance
                                 */
                                ValueRuleGeoLocationCondition.prototype.geo_target_constants = $util.emptyArray;
    
                                /**
                                 * ValueRuleGeoLocationCondition geo_match_type.
                                 * @member {google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType} geo_match_type
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @instance
                                 */
                                ValueRuleGeoLocationCondition.prototype.geo_match_type = 0;
    
                                /**
                                 * Creates a new ValueRuleGeoLocationCondition instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleGeoLocationCondition=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition} ValueRuleGeoLocationCondition instance
                                 */
                                ValueRuleGeoLocationCondition.create = function create(properties) {
                                    return new ValueRuleGeoLocationCondition(properties);
                                };
    
                                /**
                                 * Encodes the specified ValueRuleGeoLocationCondition message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleGeoLocationCondition} m ValueRuleGeoLocationCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleGeoLocationCondition.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.excluded_geo_target_constants != null && m.excluded_geo_target_constants.length) {
                                        for (var i = 0; i < m.excluded_geo_target_constants.length; ++i)
                                            w.uint32(10).string(m.excluded_geo_target_constants[i]);
                                    }
                                    if (m.excluded_geo_match_type != null && Object.hasOwnProperty.call(m, "excluded_geo_match_type"))
                                        w.uint32(16).int32(m.excluded_geo_match_type);
                                    if (m.geo_target_constants != null && m.geo_target_constants.length) {
                                        for (var i = 0; i < m.geo_target_constants.length; ++i)
                                            w.uint32(26).string(m.geo_target_constants[i]);
                                    }
                                    if (m.geo_match_type != null && Object.hasOwnProperty.call(m, "geo_match_type"))
                                        w.uint32(32).int32(m.geo_match_type);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ValueRuleGeoLocationCondition message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleGeoLocationCondition} message ValueRuleGeoLocationCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleGeoLocationCondition.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ValueRuleGeoLocationCondition message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition} ValueRuleGeoLocationCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleGeoLocationCondition.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.excluded_geo_target_constants && m.excluded_geo_target_constants.length))
                                                m.excluded_geo_target_constants = [];
                                            m.excluded_geo_target_constants.push(r.string());
                                            break;
                                        case 2:
                                            m.excluded_geo_match_type = r.int32();
                                            break;
                                        case 3:
                                            if (!(m.geo_target_constants && m.geo_target_constants.length))
                                                m.geo_target_constants = [];
                                            m.geo_target_constants.push(r.string());
                                            break;
                                        case 4:
                                            m.geo_match_type = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ValueRuleGeoLocationCondition message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition} ValueRuleGeoLocationCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleGeoLocationCondition.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ValueRuleGeoLocationCondition message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ValueRuleGeoLocationCondition.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.excluded_geo_target_constants != null && m.hasOwnProperty("excluded_geo_target_constants")) {
                                        if (!Array.isArray(m.excluded_geo_target_constants))
                                            return "excluded_geo_target_constants: array expected";
                                        for (var i = 0; i < m.excluded_geo_target_constants.length; ++i) {
                                            if (!$util.isString(m.excluded_geo_target_constants[i]))
                                                return "excluded_geo_target_constants: string[] expected";
                                        }
                                    }
                                    if (m.excluded_geo_match_type != null && m.hasOwnProperty("excluded_geo_match_type")) {
                                        switch (m.excluded_geo_match_type) {
                                        default:
                                            return "excluded_geo_match_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    if (m.geo_target_constants != null && m.hasOwnProperty("geo_target_constants")) {
                                        if (!Array.isArray(m.geo_target_constants))
                                            return "geo_target_constants: array expected";
                                        for (var i = 0; i < m.geo_target_constants.length; ++i) {
                                            if (!$util.isString(m.geo_target_constants[i]))
                                                return "geo_target_constants: string[] expected";
                                        }
                                    }
                                    if (m.geo_match_type != null && m.hasOwnProperty("geo_match_type")) {
                                        switch (m.geo_match_type) {
                                        default:
                                            return "geo_match_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ValueRuleGeoLocationCondition message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition} ValueRuleGeoLocationCondition
                                 */
                                ValueRuleGeoLocationCondition.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition();
                                    if (d.excluded_geo_target_constants) {
                                        if (!Array.isArray(d.excluded_geo_target_constants))
                                            throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.excluded_geo_target_constants: array expected");
                                        m.excluded_geo_target_constants = [];
                                        for (var i = 0; i < d.excluded_geo_target_constants.length; ++i) {
                                            m.excluded_geo_target_constants[i] = String(d.excluded_geo_target_constants[i]);
                                        }
                                    }
                                    switch (d.excluded_geo_match_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.excluded_geo_match_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.excluded_geo_match_type = 1;
                                        break;
                                    case "ANY":
                                    case 2:
                                        m.excluded_geo_match_type = 2;
                                        break;
                                    case "LOCATION_OF_PRESENCE":
                                    case 3:
                                        m.excluded_geo_match_type = 3;
                                        break;
                                    }
                                    if (d.geo_target_constants) {
                                        if (!Array.isArray(d.geo_target_constants))
                                            throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition.geo_target_constants: array expected");
                                        m.geo_target_constants = [];
                                        for (var i = 0; i < d.geo_target_constants.length; ++i) {
                                            m.geo_target_constants[i] = String(d.geo_target_constants[i]);
                                        }
                                    }
                                    switch (d.geo_match_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.geo_match_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.geo_match_type = 1;
                                        break;
                                    case "ANY":
                                    case 2:
                                        m.geo_match_type = 2;
                                        break;
                                    case "LOCATION_OF_PRESENCE":
                                    case 3:
                                        m.geo_match_type = 3;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ValueRuleGeoLocationCondition message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition} m ValueRuleGeoLocationCondition
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ValueRuleGeoLocationCondition.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.excluded_geo_target_constants = [];
                                        d.geo_target_constants = [];
                                    }
                                    if (o.defaults) {
                                        d.excluded_geo_match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                        d.geo_match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.excluded_geo_target_constants && m.excluded_geo_target_constants.length) {
                                        d.excluded_geo_target_constants = [];
                                        for (var j = 0; j < m.excluded_geo_target_constants.length; ++j) {
                                            d.excluded_geo_target_constants[j] = m.excluded_geo_target_constants[j];
                                        }
                                    }
                                    if (m.excluded_geo_match_type != null && m.hasOwnProperty("excluded_geo_match_type")) {
                                        d.excluded_geo_match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType[m.excluded_geo_match_type] : m.excluded_geo_match_type;
                                    }
                                    if (m.geo_target_constants && m.geo_target_constants.length) {
                                        d.geo_target_constants = [];
                                        for (var j = 0; j < m.geo_target_constants.length; ++j) {
                                            d.geo_target_constants[j] = m.geo_target_constants[j];
                                        }
                                    }
                                    if (m.geo_match_type != null && m.hasOwnProperty("geo_match_type")) {
                                        d.geo_match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType[m.geo_match_type] : m.geo_match_type;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ValueRuleGeoLocationCondition to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleGeoLocationCondition
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ValueRuleGeoLocationCondition.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ValueRuleGeoLocationCondition;
                            })();
    
                            ConversionValueRule.ValueRuleDeviceCondition = (function() {
    
                                /**
                                 * Properties of a ValueRuleDeviceCondition.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @interface IValueRuleDeviceCondition
                                 * @property {Array.<google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum.ValueRuleDeviceType>|null} [device_types] ValueRuleDeviceCondition device_types
                                 */
    
                                /**
                                 * Constructs a new ValueRuleDeviceCondition.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @classdesc Represents a ValueRuleDeviceCondition.
                                 * @implements IValueRuleDeviceCondition
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleDeviceCondition=} [p] Properties to set
                                 */
                                function ValueRuleDeviceCondition(p) {
                                    this.device_types = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ValueRuleDeviceCondition device_types.
                                 * @member {Array.<google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum.ValueRuleDeviceType>} device_types
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @instance
                                 */
                                ValueRuleDeviceCondition.prototype.device_types = $util.emptyArray;
    
                                /**
                                 * Creates a new ValueRuleDeviceCondition instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleDeviceCondition=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition} ValueRuleDeviceCondition instance
                                 */
                                ValueRuleDeviceCondition.create = function create(properties) {
                                    return new ValueRuleDeviceCondition(properties);
                                };
    
                                /**
                                 * Encodes the specified ValueRuleDeviceCondition message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleDeviceCondition} m ValueRuleDeviceCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleDeviceCondition.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.device_types != null && m.device_types.length) {
                                        w.uint32(10).fork();
                                        for (var i = 0; i < m.device_types.length; ++i)
                                            w.int32(m.device_types[i]);
                                        w.ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ValueRuleDeviceCondition message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleDeviceCondition} message ValueRuleDeviceCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleDeviceCondition.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ValueRuleDeviceCondition message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition} ValueRuleDeviceCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleDeviceCondition.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.device_types && m.device_types.length))
                                                m.device_types = [];
                                            if ((t & 7) === 2) {
                                                var c2 = r.uint32() + r.pos;
                                                while (r.pos < c2)
                                                    m.device_types.push(r.int32());
                                            } else
                                                m.device_types.push(r.int32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ValueRuleDeviceCondition message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition} ValueRuleDeviceCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleDeviceCondition.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ValueRuleDeviceCondition message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ValueRuleDeviceCondition.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.device_types != null && m.hasOwnProperty("device_types")) {
                                        if (!Array.isArray(m.device_types))
                                            return "device_types: array expected";
                                        for (var i = 0; i < m.device_types.length; ++i) {
                                            switch (m.device_types[i]) {
                                            default:
                                                return "device_types: enum value[] expected";
                                            case 0:
                                            case 1:
                                            case 2:
                                            case 3:
                                            case 4:
                                                break;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ValueRuleDeviceCondition message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition} ValueRuleDeviceCondition
                                 */
                                ValueRuleDeviceCondition.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition();
                                    if (d.device_types) {
                                        if (!Array.isArray(d.device_types))
                                            throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition.device_types: array expected");
                                        m.device_types = [];
                                        for (var i = 0; i < d.device_types.length; ++i) {
                                            switch (d.device_types[i]) {
                                            default:
                                            case "UNSPECIFIED":
                                            case 0:
                                                m.device_types[i] = 0;
                                                break;
                                            case "UNKNOWN":
                                            case 1:
                                                m.device_types[i] = 1;
                                                break;
                                            case "MOBILE":
                                            case 2:
                                                m.device_types[i] = 2;
                                                break;
                                            case "DESKTOP":
                                            case 3:
                                                m.device_types[i] = 3;
                                                break;
                                            case "TABLET":
                                            case 4:
                                                m.device_types[i] = 4;
                                                break;
                                            }
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ValueRuleDeviceCondition message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition} m ValueRuleDeviceCondition
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ValueRuleDeviceCondition.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.device_types = [];
                                    }
                                    if (m.device_types && m.device_types.length) {
                                        d.device_types = [];
                                        for (var j = 0; j < m.device_types.length; ++j) {
                                            d.device_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.ValueRuleDeviceTypeEnum.ValueRuleDeviceType[m.device_types[j]] : m.device_types[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ValueRuleDeviceCondition to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleDeviceCondition
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ValueRuleDeviceCondition.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ValueRuleDeviceCondition;
                            })();
    
                            ConversionValueRule.ValueRuleAudienceCondition = (function() {
    
                                /**
                                 * Properties of a ValueRuleAudienceCondition.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @interface IValueRuleAudienceCondition
                                 * @property {Array.<string>|null} [user_lists] ValueRuleAudienceCondition user_lists
                                 * @property {Array.<string>|null} [user_interests] ValueRuleAudienceCondition user_interests
                                 */
    
                                /**
                                 * Constructs a new ValueRuleAudienceCondition.
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule
                                 * @classdesc Represents a ValueRuleAudienceCondition.
                                 * @implements IValueRuleAudienceCondition
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAudienceCondition=} [p] Properties to set
                                 */
                                function ValueRuleAudienceCondition(p) {
                                    this.user_lists = [];
                                    this.user_interests = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ValueRuleAudienceCondition user_lists.
                                 * @member {Array.<string>} user_lists
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @instance
                                 */
                                ValueRuleAudienceCondition.prototype.user_lists = $util.emptyArray;
    
                                /**
                                 * ValueRuleAudienceCondition user_interests.
                                 * @member {Array.<string>} user_interests
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @instance
                                 */
                                ValueRuleAudienceCondition.prototype.user_interests = $util.emptyArray;
    
                                /**
                                 * Creates a new ValueRuleAudienceCondition instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAudienceCondition=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition} ValueRuleAudienceCondition instance
                                 */
                                ValueRuleAudienceCondition.create = function create(properties) {
                                    return new ValueRuleAudienceCondition(properties);
                                };
    
                                /**
                                 * Encodes the specified ValueRuleAudienceCondition message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAudienceCondition} m ValueRuleAudienceCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleAudienceCondition.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.user_lists != null && m.user_lists.length) {
                                        for (var i = 0; i < m.user_lists.length; ++i)
                                            w.uint32(10).string(m.user_lists[i]);
                                    }
                                    if (m.user_interests != null && m.user_interests.length) {
                                        for (var i = 0; i < m.user_interests.length; ++i)
                                            w.uint32(18).string(m.user_interests[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ValueRuleAudienceCondition message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.IValueRuleAudienceCondition} message ValueRuleAudienceCondition message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ValueRuleAudienceCondition.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ValueRuleAudienceCondition message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition} ValueRuleAudienceCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleAudienceCondition.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.user_lists && m.user_lists.length))
                                                m.user_lists = [];
                                            m.user_lists.push(r.string());
                                            break;
                                        case 2:
                                            if (!(m.user_interests && m.user_interests.length))
                                                m.user_interests = [];
                                            m.user_interests.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ValueRuleAudienceCondition message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition} ValueRuleAudienceCondition
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ValueRuleAudienceCondition.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ValueRuleAudienceCondition message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ValueRuleAudienceCondition.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.user_lists != null && m.hasOwnProperty("user_lists")) {
                                        if (!Array.isArray(m.user_lists))
                                            return "user_lists: array expected";
                                        for (var i = 0; i < m.user_lists.length; ++i) {
                                            if (!$util.isString(m.user_lists[i]))
                                                return "user_lists: string[] expected";
                                        }
                                    }
                                    if (m.user_interests != null && m.hasOwnProperty("user_interests")) {
                                        if (!Array.isArray(m.user_interests))
                                            return "user_interests: array expected";
                                        for (var i = 0; i < m.user_interests.length; ++i) {
                                            if (!$util.isString(m.user_interests[i]))
                                                return "user_interests: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ValueRuleAudienceCondition message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition} ValueRuleAudienceCondition
                                 */
                                ValueRuleAudienceCondition.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition();
                                    if (d.user_lists) {
                                        if (!Array.isArray(d.user_lists))
                                            throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.user_lists: array expected");
                                        m.user_lists = [];
                                        for (var i = 0; i < d.user_lists.length; ++i) {
                                            m.user_lists[i] = String(d.user_lists[i]);
                                        }
                                    }
                                    if (d.user_interests) {
                                        if (!Array.isArray(d.user_interests))
                                            throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition.user_interests: array expected");
                                        m.user_interests = [];
                                        for (var i = 0; i < d.user_interests.length; ++i) {
                                            m.user_interests[i] = String(d.user_interests[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ValueRuleAudienceCondition message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition} m ValueRuleAudienceCondition
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ValueRuleAudienceCondition.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.user_lists = [];
                                        d.user_interests = [];
                                    }
                                    if (m.user_lists && m.user_lists.length) {
                                        d.user_lists = [];
                                        for (var j = 0; j < m.user_lists.length; ++j) {
                                            d.user_lists[j] = m.user_lists[j];
                                        }
                                    }
                                    if (m.user_interests && m.user_interests.length) {
                                        d.user_interests = [];
                                        for (var j = 0; j < m.user_interests.length; ++j) {
                                            d.user_interests[j] = m.user_interests[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ValueRuleAudienceCondition to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.ConversionValueRule.ValueRuleAudienceCondition
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ValueRuleAudienceCondition.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ValueRuleAudienceCondition;
                            })();
    
                            return ConversionValueRule;
                        })();
    
                        resources.ConversionValueRuleSet = (function() {
    
                            /**
                             * Properties of a ConversionValueRuleSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IConversionValueRuleSet
                             * @property {string|null} [resource_name] ConversionValueRuleSet resource_name
                             * @property {number|null} [id] ConversionValueRuleSet id
                             * @property {Array.<string>|null} [conversion_value_rules] ConversionValueRuleSet conversion_value_rules
                             * @property {Array.<google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum.ValueRuleSetDimension>|null} [dimensions] ConversionValueRuleSet dimensions
                             * @property {string|null} [owner_customer] ConversionValueRuleSet owner_customer
                             * @property {google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType|null} [attachment_type] ConversionValueRuleSet attachment_type
                             * @property {string|null} [campaign] ConversionValueRuleSet campaign
                             * @property {google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus|null} [status] ConversionValueRuleSet status
                             * @property {Array.<google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory>|null} [conversion_action_categories] ConversionValueRuleSet conversion_action_categories
                             */
    
                            /**
                             * Constructs a new ConversionValueRuleSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ConversionValueRuleSet.
                             * @implements IConversionValueRuleSet
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IConversionValueRuleSet=} [p] Properties to set
                             */
                            function ConversionValueRuleSet(p) {
                                this.conversion_value_rules = [];
                                this.dimensions = [];
                                this.conversion_action_categories = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionValueRuleSet resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.resource_name = "";
    
                            /**
                             * ConversionValueRuleSet id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * ConversionValueRuleSet conversion_value_rules.
                             * @member {Array.<string>} conversion_value_rules
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.conversion_value_rules = $util.emptyArray;
    
                            /**
                             * ConversionValueRuleSet dimensions.
                             * @member {Array.<google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum.ValueRuleSetDimension>} dimensions
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.dimensions = $util.emptyArray;
    
                            /**
                             * ConversionValueRuleSet owner_customer.
                             * @member {string} owner_customer
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.owner_customer = "";
    
                            /**
                             * ConversionValueRuleSet attachment_type.
                             * @member {google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType} attachment_type
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.attachment_type = 0;
    
                            /**
                             * ConversionValueRuleSet campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.campaign = "";
    
                            /**
                             * ConversionValueRuleSet status.
                             * @member {google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus} status
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.status = 0;
    
                            /**
                             * ConversionValueRuleSet conversion_action_categories.
                             * @member {Array.<google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory>} conversion_action_categories
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             */
                            ConversionValueRuleSet.prototype.conversion_action_categories = $util.emptyArray;
    
                            /**
                             * Creates a new ConversionValueRuleSet instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionValueRuleSet=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRuleSet} ConversionValueRuleSet instance
                             */
                            ConversionValueRuleSet.create = function create(properties) {
                                return new ConversionValueRuleSet(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSet message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRuleSet.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionValueRuleSet} m ConversionValueRuleSet message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSet.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.conversion_value_rules != null && m.conversion_value_rules.length) {
                                    for (var i = 0; i < m.conversion_value_rules.length; ++i)
                                        w.uint32(26).string(m.conversion_value_rules[i]);
                                }
                                if (m.dimensions != null && m.dimensions.length) {
                                    w.uint32(34).fork();
                                    for (var i = 0; i < m.dimensions.length; ++i)
                                        w.int32(m.dimensions[i]);
                                    w.ldelim();
                                }
                                if (m.owner_customer != null && Object.hasOwnProperty.call(m, "owner_customer"))
                                    w.uint32(42).string(m.owner_customer);
                                if (m.attachment_type != null && Object.hasOwnProperty.call(m, "attachment_type"))
                                    w.uint32(48).int32(m.attachment_type);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(58).string(m.campaign);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(64).int32(m.status);
                                if (m.conversion_action_categories != null && m.conversion_action_categories.length) {
                                    w.uint32(74).fork();
                                    for (var i = 0; i < m.conversion_action_categories.length; ++i)
                                        w.int32(m.conversion_action_categories[i]);
                                    w.ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSet message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionValueRuleSet.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionValueRuleSet} message ConversionValueRuleSet message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSet.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSet message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRuleSet} ConversionValueRuleSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSet.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionValueRuleSet();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        if (!(m.conversion_value_rules && m.conversion_value_rules.length))
                                            m.conversion_value_rules = [];
                                        m.conversion_value_rules.push(r.string());
                                        break;
                                    case 4:
                                        if (!(m.dimensions && m.dimensions.length))
                                            m.dimensions = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.dimensions.push(r.int32());
                                        } else
                                            m.dimensions.push(r.int32());
                                        break;
                                    case 5:
                                        m.owner_customer = r.string();
                                        break;
                                    case 6:
                                        m.attachment_type = r.int32();
                                        break;
                                    case 7:
                                        m.campaign = r.string();
                                        break;
                                    case 8:
                                        m.status = r.int32();
                                        break;
                                    case 9:
                                        if (!(m.conversion_action_categories && m.conversion_action_categories.length))
                                            m.conversion_action_categories = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.conversion_action_categories.push(r.int32());
                                        } else
                                            m.conversion_action_categories.push(r.int32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSet message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRuleSet} ConversionValueRuleSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSet.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRuleSet message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRuleSet.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.conversion_value_rules != null && m.hasOwnProperty("conversion_value_rules")) {
                                    if (!Array.isArray(m.conversion_value_rules))
                                        return "conversion_value_rules: array expected";
                                    for (var i = 0; i < m.conversion_value_rules.length; ++i) {
                                        if (!$util.isString(m.conversion_value_rules[i]))
                                            return "conversion_value_rules: string[] expected";
                                    }
                                }
                                if (m.dimensions != null && m.hasOwnProperty("dimensions")) {
                                    if (!Array.isArray(m.dimensions))
                                        return "dimensions: array expected";
                                    for (var i = 0; i < m.dimensions.length; ++i) {
                                        switch (m.dimensions[i]) {
                                        default:
                                            return "dimensions: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                            break;
                                        }
                                    }
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    if (!$util.isString(m.owner_customer))
                                        return "owner_customer: string expected";
                                }
                                if (m.attachment_type != null && m.hasOwnProperty("attachment_type")) {
                                    switch (m.attachment_type) {
                                    default:
                                        return "attachment_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.conversion_action_categories != null && m.hasOwnProperty("conversion_action_categories")) {
                                    if (!Array.isArray(m.conversion_action_categories))
                                        return "conversion_action_categories: array expected";
                                    for (var i = 0; i < m.conversion_action_categories.length; ++i) {
                                        switch (m.conversion_action_categories[i]) {
                                        default:
                                            return "conversion_action_categories: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 17:
                                        case 18:
                                        case 19:
                                        case 20:
                                        case 21:
                                        case 22:
                                        case 23:
                                            break;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRuleSet message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ConversionValueRuleSet} ConversionValueRuleSet
                             */
                            ConversionValueRuleSet.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ConversionValueRuleSet)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ConversionValueRuleSet();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.conversion_value_rules) {
                                    if (!Array.isArray(d.conversion_value_rules))
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRuleSet.conversion_value_rules: array expected");
                                    m.conversion_value_rules = [];
                                    for (var i = 0; i < d.conversion_value_rules.length; ++i) {
                                        m.conversion_value_rules[i] = String(d.conversion_value_rules[i]);
                                    }
                                }
                                if (d.dimensions) {
                                    if (!Array.isArray(d.dimensions))
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRuleSet.dimensions: array expected");
                                    m.dimensions = [];
                                    for (var i = 0; i < d.dimensions.length; ++i) {
                                        switch (d.dimensions[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.dimensions[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.dimensions[i] = 1;
                                            break;
                                        case "GEO_LOCATION":
                                        case 2:
                                            m.dimensions[i] = 2;
                                            break;
                                        case "DEVICE":
                                        case 3:
                                            m.dimensions[i] = 3;
                                            break;
                                        case "AUDIENCE":
                                        case 4:
                                            m.dimensions[i] = 4;
                                            break;
                                        case "NO_CONDITION":
                                        case 5:
                                            m.dimensions[i] = 5;
                                            break;
                                        }
                                    }
                                }
                                if (d.owner_customer != null) {
                                    m.owner_customer = String(d.owner_customer);
                                }
                                switch (d.attachment_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.attachment_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.attachment_type = 1;
                                    break;
                                case "CUSTOMER":
                                case 2:
                                    m.attachment_type = 2;
                                    break;
                                case "CAMPAIGN":
                                case 3:
                                    m.attachment_type = 3;
                                    break;
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PAUSED":
                                case 4:
                                    m.status = 4;
                                    break;
                                }
                                if (d.conversion_action_categories) {
                                    if (!Array.isArray(d.conversion_action_categories))
                                        throw TypeError(".google.ads.googleads.v11.resources.ConversionValueRuleSet.conversion_action_categories: array expected");
                                    m.conversion_action_categories = [];
                                    for (var i = 0; i < d.conversion_action_categories.length; ++i) {
                                        switch (d.conversion_action_categories[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.conversion_action_categories[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.conversion_action_categories[i] = 1;
                                            break;
                                        case "DEFAULT":
                                        case 2:
                                            m.conversion_action_categories[i] = 2;
                                            break;
                                        case "PAGE_VIEW":
                                        case 3:
                                            m.conversion_action_categories[i] = 3;
                                            break;
                                        case "PURCHASE":
                                        case 4:
                                            m.conversion_action_categories[i] = 4;
                                            break;
                                        case "SIGNUP":
                                        case 5:
                                            m.conversion_action_categories[i] = 5;
                                            break;
                                        case "DOWNLOAD":
                                        case 7:
                                            m.conversion_action_categories[i] = 7;
                                            break;
                                        case "ADD_TO_CART":
                                        case 8:
                                            m.conversion_action_categories[i] = 8;
                                            break;
                                        case "BEGIN_CHECKOUT":
                                        case 9:
                                            m.conversion_action_categories[i] = 9;
                                            break;
                                        case "SUBSCRIBE_PAID":
                                        case 10:
                                            m.conversion_action_categories[i] = 10;
                                            break;
                                        case "PHONE_CALL_LEAD":
                                        case 11:
                                            m.conversion_action_categories[i] = 11;
                                            break;
                                        case "IMPORTED_LEAD":
                                        case 12:
                                            m.conversion_action_categories[i] = 12;
                                            break;
                                        case "SUBMIT_LEAD_FORM":
                                        case 13:
                                            m.conversion_action_categories[i] = 13;
                                            break;
                                        case "BOOK_APPOINTMENT":
                                        case 14:
                                            m.conversion_action_categories[i] = 14;
                                            break;
                                        case "REQUEST_QUOTE":
                                        case 15:
                                            m.conversion_action_categories[i] = 15;
                                            break;
                                        case "GET_DIRECTIONS":
                                        case 16:
                                            m.conversion_action_categories[i] = 16;
                                            break;
                                        case "OUTBOUND_CLICK":
                                        case 17:
                                            m.conversion_action_categories[i] = 17;
                                            break;
                                        case "CONTACT":
                                        case 18:
                                            m.conversion_action_categories[i] = 18;
                                            break;
                                        case "ENGAGEMENT":
                                        case 19:
                                            m.conversion_action_categories[i] = 19;
                                            break;
                                        case "STORE_VISIT":
                                        case 20:
                                            m.conversion_action_categories[i] = 20;
                                            break;
                                        case "STORE_SALE":
                                        case 21:
                                            m.conversion_action_categories[i] = 21;
                                            break;
                                        case "QUALIFIED_LEAD":
                                        case 22:
                                            m.conversion_action_categories[i] = 22;
                                            break;
                                        case "CONVERTED_LEAD":
                                        case 23:
                                            m.conversion_action_categories[i] = 23;
                                            break;
                                        }
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRuleSet message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ConversionValueRuleSet} m ConversionValueRuleSet
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRuleSet.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.conversion_value_rules = [];
                                    d.dimensions = [];
                                    d.conversion_action_categories = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.owner_customer = "";
                                    d.attachment_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.campaign = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.conversion_value_rules && m.conversion_value_rules.length) {
                                    d.conversion_value_rules = [];
                                    for (var j = 0; j < m.conversion_value_rules.length; ++j) {
                                        d.conversion_value_rules[j] = m.conversion_value_rules[j];
                                    }
                                }
                                if (m.dimensions && m.dimensions.length) {
                                    d.dimensions = [];
                                    for (var j = 0; j < m.dimensions.length; ++j) {
                                        d.dimensions[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.ValueRuleSetDimensionEnum.ValueRuleSetDimension[m.dimensions[j]] : m.dimensions[j];
                                    }
                                }
                                if (m.owner_customer != null && m.hasOwnProperty("owner_customer")) {
                                    d.owner_customer = m.owner_customer;
                                }
                                if (m.attachment_type != null && m.hasOwnProperty("attachment_type")) {
                                    d.attachment_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType[m.attachment_type] : m.attachment_type;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus[m.status] : m.status;
                                }
                                if (m.conversion_action_categories && m.conversion_action_categories.length) {
                                    d.conversion_action_categories = [];
                                    for (var j = 0; j < m.conversion_action_categories.length; ++j) {
                                        d.conversion_action_categories[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory[m.conversion_action_categories[j]] : m.conversion_action_categories[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionValueRuleSet to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ConversionValueRuleSet
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRuleSet.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionValueRuleSet;
                        })();
    
                        resources.CurrencyConstant = (function() {
    
                            /**
                             * Properties of a CurrencyConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICurrencyConstant
                             * @property {string|null} [resource_name] CurrencyConstant resource_name
                             * @property {string|null} [code] CurrencyConstant code
                             * @property {string|null} [name] CurrencyConstant name
                             * @property {string|null} [symbol] CurrencyConstant symbol
                             * @property {number|null} [billable_unit_micros] CurrencyConstant billable_unit_micros
                             */
    
                            /**
                             * Constructs a new CurrencyConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CurrencyConstant.
                             * @implements ICurrencyConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICurrencyConstant=} [p] Properties to set
                             */
                            function CurrencyConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CurrencyConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            CurrencyConstant.prototype.resource_name = "";
    
                            /**
                             * CurrencyConstant code.
                             * @member {string|null|undefined} code
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            CurrencyConstant.prototype.code = null;
    
                            /**
                             * CurrencyConstant name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            CurrencyConstant.prototype.name = null;
    
                            /**
                             * CurrencyConstant symbol.
                             * @member {string|null|undefined} symbol
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            CurrencyConstant.prototype.symbol = null;
    
                            /**
                             * CurrencyConstant billable_unit_micros.
                             * @member {number|null|undefined} billable_unit_micros
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            CurrencyConstant.prototype.billable_unit_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CurrencyConstant _code.
                             * @member {"code"|undefined} _code
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            Object.defineProperty(CurrencyConstant.prototype, "_code", {
                                get: $util.oneOfGetter($oneOfFields = ["code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CurrencyConstant _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            Object.defineProperty(CurrencyConstant.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CurrencyConstant _symbol.
                             * @member {"symbol"|undefined} _symbol
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            Object.defineProperty(CurrencyConstant.prototype, "_symbol", {
                                get: $util.oneOfGetter($oneOfFields = ["symbol"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CurrencyConstant _billable_unit_micros.
                             * @member {"billable_unit_micros"|undefined} _billable_unit_micros
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             */
                            Object.defineProperty(CurrencyConstant.prototype, "_billable_unit_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["billable_unit_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CurrencyConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICurrencyConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CurrencyConstant} CurrencyConstant instance
                             */
                            CurrencyConstant.create = function create(properties) {
                                return new CurrencyConstant(properties);
                            };
    
                            /**
                             * Encodes the specified CurrencyConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.CurrencyConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICurrencyConstant} m CurrencyConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CurrencyConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.code != null && Object.hasOwnProperty.call(m, "code"))
                                    w.uint32(50).string(m.code);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(58).string(m.name);
                                if (m.symbol != null && Object.hasOwnProperty.call(m, "symbol"))
                                    w.uint32(66).string(m.symbol);
                                if (m.billable_unit_micros != null && Object.hasOwnProperty.call(m, "billable_unit_micros"))
                                    w.uint32(72).int64(m.billable_unit_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CurrencyConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CurrencyConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICurrencyConstant} message CurrencyConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CurrencyConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CurrencyConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CurrencyConstant} CurrencyConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CurrencyConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CurrencyConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.code = r.string();
                                        break;
                                    case 7:
                                        m.name = r.string();
                                        break;
                                    case 8:
                                        m.symbol = r.string();
                                        break;
                                    case 9:
                                        m.billable_unit_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CurrencyConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CurrencyConstant} CurrencyConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CurrencyConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CurrencyConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CurrencyConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.code != null && m.hasOwnProperty("code")) {
                                    p._code = 1;
                                    if (!$util.isString(m.code))
                                        return "code: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.symbol != null && m.hasOwnProperty("symbol")) {
                                    p._symbol = 1;
                                    if (!$util.isString(m.symbol))
                                        return "symbol: string expected";
                                }
                                if (m.billable_unit_micros != null && m.hasOwnProperty("billable_unit_micros")) {
                                    p._billable_unit_micros = 1;
                                    if (!$util.isInteger(m.billable_unit_micros) && !(m.billable_unit_micros && $util.isInteger(m.billable_unit_micros.low) && $util.isInteger(m.billable_unit_micros.high)))
                                        return "billable_unit_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CurrencyConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CurrencyConstant} CurrencyConstant
                             */
                            CurrencyConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CurrencyConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CurrencyConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.code != null) {
                                    m.code = String(d.code);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.symbol != null) {
                                    m.symbol = String(d.symbol);
                                }
                                if (d.billable_unit_micros != null) {
                                    if ($util.Long)
                                        (m.billable_unit_micros = $util.Long.fromValue(d.billable_unit_micros)).unsigned = false;
                                    else if (typeof d.billable_unit_micros === "string")
                                        m.billable_unit_micros = parseInt(d.billable_unit_micros, 10);
                                    else if (typeof d.billable_unit_micros === "number")
                                        m.billable_unit_micros = d.billable_unit_micros;
                                    else if (typeof d.billable_unit_micros === "object")
                                        m.billable_unit_micros = new $util.LongBits(d.billable_unit_micros.low >>> 0, d.billable_unit_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CurrencyConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.CurrencyConstant} m CurrencyConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CurrencyConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.code != null && m.hasOwnProperty("code")) {
                                    d.code = m.code;
                                    if (o.oneofs)
                                        d._code = "code";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.symbol != null && m.hasOwnProperty("symbol")) {
                                    d.symbol = m.symbol;
                                    if (o.oneofs)
                                        d._symbol = "symbol";
                                }
                                if (m.billable_unit_micros != null && m.hasOwnProperty("billable_unit_micros")) {
                                    if (typeof m.billable_unit_micros === "number")
                                        d.billable_unit_micros = o.longs === String ? String(m.billable_unit_micros) : m.billable_unit_micros;
                                    else
                                        d.billable_unit_micros = o.longs === String ? $util.Long.prototype.toString.call(m.billable_unit_micros) : o.longs === Number ? new $util.LongBits(m.billable_unit_micros.low >>> 0, m.billable_unit_micros.high >>> 0).toNumber() : m.billable_unit_micros;
                                    if (o.oneofs)
                                        d._billable_unit_micros = "billable_unit_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CurrencyConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CurrencyConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CurrencyConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CurrencyConstant;
                        })();
    
                        resources.CustomAudience = (function() {
    
                            /**
                             * Properties of a CustomAudience.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomAudience
                             * @property {string|null} [resource_name] CustomAudience resource_name
                             * @property {number|null} [id] CustomAudience id
                             * @property {google.ads.googleads.v11.enums.CustomAudienceStatusEnum.CustomAudienceStatus|null} [status] CustomAudience status
                             * @property {string|null} [name] CustomAudience name
                             * @property {google.ads.googleads.v11.enums.CustomAudienceTypeEnum.CustomAudienceType|null} [type] CustomAudience type
                             * @property {string|null} [description] CustomAudience description
                             * @property {Array.<google.ads.googleads.v11.resources.ICustomAudienceMember>|null} [members] CustomAudience members
                             */
    
                            /**
                             * Constructs a new CustomAudience.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomAudience.
                             * @implements ICustomAudience
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomAudience=} [p] Properties to set
                             */
                            function CustomAudience(p) {
                                this.members = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomAudience resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             */
                            CustomAudience.prototype.resource_name = "";
    
                            /**
                             * CustomAudience id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             */
                            CustomAudience.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CustomAudience status.
                             * @member {google.ads.googleads.v11.enums.CustomAudienceStatusEnum.CustomAudienceStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             */
                            CustomAudience.prototype.status = 0;
    
                            /**
                             * CustomAudience name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             */
                            CustomAudience.prototype.name = "";
    
                            /**
                             * CustomAudience type.
                             * @member {google.ads.googleads.v11.enums.CustomAudienceTypeEnum.CustomAudienceType} type
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             */
                            CustomAudience.prototype.type = 0;
    
                            /**
                             * CustomAudience description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             */
                            CustomAudience.prototype.description = "";
    
                            /**
                             * CustomAudience members.
                             * @member {Array.<google.ads.googleads.v11.resources.ICustomAudienceMember>} members
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             */
                            CustomAudience.prototype.members = $util.emptyArray;
    
                            /**
                             * Creates a new CustomAudience instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomAudience=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomAudience} CustomAudience instance
                             */
                            CustomAudience.create = function create(properties) {
                                return new CustomAudience(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudience message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomAudience.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomAudience} m CustomAudience message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudience.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(34).string(m.name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(40).int32(m.type);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(50).string(m.description);
                                if (m.members != null && m.members.length) {
                                    for (var i = 0; i < m.members.length; ++i)
                                        $root.google.ads.googleads.v11.resources.CustomAudienceMember.encode(m.members[i], w.uint32(58).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudience message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomAudience.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomAudience} message CustomAudience message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudience.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudience message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomAudience} CustomAudience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudience.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomAudience();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 4:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.type = r.int32();
                                        break;
                                    case 6:
                                        m.description = r.string();
                                        break;
                                    case 7:
                                        if (!(m.members && m.members.length))
                                            m.members = [];
                                        m.members.push($root.google.ads.googleads.v11.resources.CustomAudienceMember.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudience message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomAudience} CustomAudience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudience.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudience message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudience.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.members != null && m.hasOwnProperty("members")) {
                                    if (!Array.isArray(m.members))
                                        return "members: array expected";
                                    for (var i = 0; i < m.members.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CustomAudienceMember.verify(m.members[i]);
                                            if (e)
                                                return "members." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudience message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomAudience} CustomAudience
                             */
                            CustomAudience.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomAudience)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomAudience();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "AUTO":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "INTEREST":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "PURCHASE_INTENT":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "SEARCH":
                                case 5:
                                    m.type = 5;
                                    break;
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.members) {
                                    if (!Array.isArray(d.members))
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomAudience.members: array expected");
                                    m.members = [];
                                    for (var i = 0; i < d.members.length; ++i) {
                                        if (typeof d.members[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.CustomAudience.members: object expected");
                                        m.members[i] = $root.google.ads.googleads.v11.resources.CustomAudienceMember.fromObject(d.members[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomAudience message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomAudience} m CustomAudience
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudience.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.members = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.description = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomAudienceStatusEnum.CustomAudienceStatus[m.status] : m.status;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomAudienceTypeEnum.CustomAudienceType[m.type] : m.type;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.members && m.members.length) {
                                    d.members = [];
                                    for (var j = 0; j < m.members.length; ++j) {
                                        d.members[j] = $root.google.ads.googleads.v11.resources.CustomAudienceMember.toObject(m.members[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomAudience to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomAudience
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudience.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomAudience;
                        })();
    
                        resources.CustomAudienceMember = (function() {
    
                            /**
                             * Properties of a CustomAudienceMember.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomAudienceMember
                             * @property {google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum.CustomAudienceMemberType|null} [member_type] CustomAudienceMember member_type
                             * @property {string|null} [keyword] CustomAudienceMember keyword
                             * @property {string|null} [url] CustomAudienceMember url
                             * @property {number|null} [place_category] CustomAudienceMember place_category
                             * @property {string|null} [app] CustomAudienceMember app
                             */
    
                            /**
                             * Constructs a new CustomAudienceMember.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomAudienceMember.
                             * @implements ICustomAudienceMember
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomAudienceMember=} [p] Properties to set
                             */
                            function CustomAudienceMember(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomAudienceMember member_type.
                             * @member {google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum.CustomAudienceMemberType} member_type
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @instance
                             */
                            CustomAudienceMember.prototype.member_type = 0;
    
                            /**
                             * CustomAudienceMember keyword.
                             * @member {string|null|undefined} keyword
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @instance
                             */
                            CustomAudienceMember.prototype.keyword = null;
    
                            /**
                             * CustomAudienceMember url.
                             * @member {string|null|undefined} url
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @instance
                             */
                            CustomAudienceMember.prototype.url = null;
    
                            /**
                             * CustomAudienceMember place_category.
                             * @member {number|null|undefined} place_category
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @instance
                             */
                            CustomAudienceMember.prototype.place_category = null;
    
                            /**
                             * CustomAudienceMember app.
                             * @member {string|null|undefined} app
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @instance
                             */
                            CustomAudienceMember.prototype.app = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomAudienceMember value.
                             * @member {"keyword"|"url"|"place_category"|"app"|undefined} value
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @instance
                             */
                            Object.defineProperty(CustomAudienceMember.prototype, "value", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword", "url", "place_category", "app"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomAudienceMember instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomAudienceMember=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomAudienceMember} CustomAudienceMember instance
                             */
                            CustomAudienceMember.create = function create(properties) {
                                return new CustomAudienceMember(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceMember message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomAudienceMember.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomAudienceMember} m CustomAudienceMember message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceMember.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.member_type != null && Object.hasOwnProperty.call(m, "member_type"))
                                    w.uint32(8).int32(m.member_type);
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    w.uint32(18).string(m.keyword);
                                if (m.url != null && Object.hasOwnProperty.call(m, "url"))
                                    w.uint32(26).string(m.url);
                                if (m.place_category != null && Object.hasOwnProperty.call(m, "place_category"))
                                    w.uint32(32).int64(m.place_category);
                                if (m.app != null && Object.hasOwnProperty.call(m, "app"))
                                    w.uint32(42).string(m.app);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceMember message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomAudienceMember.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomAudienceMember} message CustomAudienceMember message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceMember.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceMember message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomAudienceMember} CustomAudienceMember
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceMember.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomAudienceMember();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.member_type = r.int32();
                                        break;
                                    case 2:
                                        m.keyword = r.string();
                                        break;
                                    case 3:
                                        m.url = r.string();
                                        break;
                                    case 4:
                                        m.place_category = r.int64();
                                        break;
                                    case 5:
                                        m.app = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceMember message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomAudienceMember} CustomAudienceMember
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceMember.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceMember message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceMember.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.member_type != null && m.hasOwnProperty("member_type")) {
                                    switch (m.member_type) {
                                    default:
                                        return "member_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    p.value = 1;
                                    if (!$util.isString(m.keyword))
                                        return "keyword: string expected";
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (!$util.isString(m.url))
                                        return "url: string expected";
                                }
                                if (m.place_category != null && m.hasOwnProperty("place_category")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (!$util.isInteger(m.place_category) && !(m.place_category && $util.isInteger(m.place_category.low) && $util.isInteger(m.place_category.high)))
                                        return "place_category: integer|Long expected";
                                }
                                if (m.app != null && m.hasOwnProperty("app")) {
                                    if (p.value === 1)
                                        return "value: multiple values";
                                    p.value = 1;
                                    if (!$util.isString(m.app))
                                        return "app: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceMember message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomAudienceMember} CustomAudienceMember
                             */
                            CustomAudienceMember.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomAudienceMember)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomAudienceMember();
                                switch (d.member_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.member_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.member_type = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.member_type = 2;
                                    break;
                                case "URL":
                                case 3:
                                    m.member_type = 3;
                                    break;
                                case "PLACE_CATEGORY":
                                case 4:
                                    m.member_type = 4;
                                    break;
                                case "APP":
                                case 5:
                                    m.member_type = 5;
                                    break;
                                }
                                if (d.keyword != null) {
                                    m.keyword = String(d.keyword);
                                }
                                if (d.url != null) {
                                    m.url = String(d.url);
                                }
                                if (d.place_category != null) {
                                    if ($util.Long)
                                        (m.place_category = $util.Long.fromValue(d.place_category)).unsigned = false;
                                    else if (typeof d.place_category === "string")
                                        m.place_category = parseInt(d.place_category, 10);
                                    else if (typeof d.place_category === "number")
                                        m.place_category = d.place_category;
                                    else if (typeof d.place_category === "object")
                                        m.place_category = new $util.LongBits(d.place_category.low >>> 0, d.place_category.high >>> 0).toNumber();
                                }
                                if (d.app != null) {
                                    m.app = String(d.app);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceMember message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomAudienceMember} m CustomAudienceMember
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceMember.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.member_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.member_type != null && m.hasOwnProperty("member_type")) {
                                    d.member_type = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomAudienceMemberTypeEnum.CustomAudienceMemberType[m.member_type] : m.member_type;
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = m.keyword;
                                    if (o.oneofs)
                                        d.value = "keyword";
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    d.url = m.url;
                                    if (o.oneofs)
                                        d.value = "url";
                                }
                                if (m.place_category != null && m.hasOwnProperty("place_category")) {
                                    if (typeof m.place_category === "number")
                                        d.place_category = o.longs === String ? String(m.place_category) : m.place_category;
                                    else
                                        d.place_category = o.longs === String ? $util.Long.prototype.toString.call(m.place_category) : o.longs === Number ? new $util.LongBits(m.place_category.low >>> 0, m.place_category.high >>> 0).toNumber() : m.place_category;
                                    if (o.oneofs)
                                        d.value = "place_category";
                                }
                                if (m.app != null && m.hasOwnProperty("app")) {
                                    d.app = m.app;
                                    if (o.oneofs)
                                        d.value = "app";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomAudienceMember to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomAudienceMember
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceMember.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomAudienceMember;
                        })();
    
                        resources.CustomConversionGoal = (function() {
    
                            /**
                             * Properties of a CustomConversionGoal.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomConversionGoal
                             * @property {string|null} [resource_name] CustomConversionGoal resource_name
                             * @property {number|null} [id] CustomConversionGoal id
                             * @property {string|null} [name] CustomConversionGoal name
                             * @property {Array.<string>|null} [conversion_actions] CustomConversionGoal conversion_actions
                             * @property {google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum.CustomConversionGoalStatus|null} [status] CustomConversionGoal status
                             */
    
                            /**
                             * Constructs a new CustomConversionGoal.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomConversionGoal.
                             * @implements ICustomConversionGoal
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomConversionGoal=} [p] Properties to set
                             */
                            function CustomConversionGoal(p) {
                                this.conversion_actions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomConversionGoal resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @instance
                             */
                            CustomConversionGoal.prototype.resource_name = "";
    
                            /**
                             * CustomConversionGoal id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @instance
                             */
                            CustomConversionGoal.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CustomConversionGoal name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @instance
                             */
                            CustomConversionGoal.prototype.name = "";
    
                            /**
                             * CustomConversionGoal conversion_actions.
                             * @member {Array.<string>} conversion_actions
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @instance
                             */
                            CustomConversionGoal.prototype.conversion_actions = $util.emptyArray;
    
                            /**
                             * CustomConversionGoal status.
                             * @member {google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum.CustomConversionGoalStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @instance
                             */
                            CustomConversionGoal.prototype.status = 0;
    
                            /**
                             * Creates a new CustomConversionGoal instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomConversionGoal=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomConversionGoal} CustomConversionGoal instance
                             */
                            CustomConversionGoal.create = function create(properties) {
                                return new CustomConversionGoal(properties);
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoal message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomConversionGoal.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomConversionGoal} m CustomConversionGoal message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoal.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.conversion_actions != null && m.conversion_actions.length) {
                                    for (var i = 0; i < m.conversion_actions.length; ++i)
                                        w.uint32(34).string(m.conversion_actions[i]);
                                }
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoal message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomConversionGoal.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomConversionGoal} message CustomConversionGoal message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoal.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomConversionGoal message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomConversionGoal} CustomConversionGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoal.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomConversionGoal();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        if (!(m.conversion_actions && m.conversion_actions.length))
                                            m.conversion_actions = [];
                                        m.conversion_actions.push(r.string());
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomConversionGoal message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomConversionGoal} CustomConversionGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoal.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomConversionGoal message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomConversionGoal.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.conversion_actions != null && m.hasOwnProperty("conversion_actions")) {
                                    if (!Array.isArray(m.conversion_actions))
                                        return "conversion_actions: array expected";
                                    for (var i = 0; i < m.conversion_actions.length; ++i) {
                                        if (!$util.isString(m.conversion_actions[i]))
                                            return "conversion_actions: string[] expected";
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomConversionGoal message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomConversionGoal} CustomConversionGoal
                             */
                            CustomConversionGoal.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomConversionGoal)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomConversionGoal();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.conversion_actions) {
                                    if (!Array.isArray(d.conversion_actions))
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomConversionGoal.conversion_actions: array expected");
                                    m.conversion_actions = [];
                                    for (var i = 0; i < d.conversion_actions.length; ++i) {
                                        m.conversion_actions[i] = String(d.conversion_actions[i]);
                                    }
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomConversionGoal message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomConversionGoal} m CustomConversionGoal
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomConversionGoal.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.conversion_actions = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.conversion_actions && m.conversion_actions.length) {
                                    d.conversion_actions = [];
                                    for (var j = 0; j < m.conversion_actions.length; ++j) {
                                        d.conversion_actions[j] = m.conversion_actions[j];
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomConversionGoalStatusEnum.CustomConversionGoalStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomConversionGoal to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomConversionGoal
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomConversionGoal.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomConversionGoal;
                        })();
    
                        resources.CustomInterest = (function() {
    
                            /**
                             * Properties of a CustomInterest.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomInterest
                             * @property {string|null} [resource_name] CustomInterest resource_name
                             * @property {number|null} [id] CustomInterest id
                             * @property {google.ads.googleads.v11.enums.CustomInterestStatusEnum.CustomInterestStatus|null} [status] CustomInterest status
                             * @property {string|null} [name] CustomInterest name
                             * @property {google.ads.googleads.v11.enums.CustomInterestTypeEnum.CustomInterestType|null} [type] CustomInterest type
                             * @property {string|null} [description] CustomInterest description
                             * @property {Array.<google.ads.googleads.v11.resources.ICustomInterestMember>|null} [members] CustomInterest members
                             */
    
                            /**
                             * Constructs a new CustomInterest.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomInterest.
                             * @implements ICustomInterest
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomInterest=} [p] Properties to set
                             */
                            function CustomInterest(p) {
                                this.members = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomInterest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            CustomInterest.prototype.resource_name = "";
    
                            /**
                             * CustomInterest id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            CustomInterest.prototype.id = null;
    
                            /**
                             * CustomInterest status.
                             * @member {google.ads.googleads.v11.enums.CustomInterestStatusEnum.CustomInterestStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            CustomInterest.prototype.status = 0;
    
                            /**
                             * CustomInterest name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            CustomInterest.prototype.name = null;
    
                            /**
                             * CustomInterest type.
                             * @member {google.ads.googleads.v11.enums.CustomInterestTypeEnum.CustomInterestType} type
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            CustomInterest.prototype.type = 0;
    
                            /**
                             * CustomInterest description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            CustomInterest.prototype.description = null;
    
                            /**
                             * CustomInterest members.
                             * @member {Array.<google.ads.googleads.v11.resources.ICustomInterestMember>} members
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            CustomInterest.prototype.members = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomInterest _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            Object.defineProperty(CustomInterest.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomInterest _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            Object.defineProperty(CustomInterest.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomInterest _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             */
                            Object.defineProperty(CustomInterest.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomInterest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomInterest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomInterest} CustomInterest instance
                             */
                            CustomInterest.create = function create(properties) {
                                return new CustomInterest(properties);
                            };
    
                            /**
                             * Encodes the specified CustomInterest message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomInterest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomInterest} m CustomInterest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(40).int32(m.type);
                                if (m.members != null && m.members.length) {
                                    for (var i = 0; i < m.members.length; ++i)
                                        $root.google.ads.googleads.v11.resources.CustomInterestMember.encode(m.members[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(64).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(74).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(82).string(m.description);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomInterest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomInterest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomInterest} message CustomInterest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomInterest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomInterest} CustomInterest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomInterest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 8:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 9:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.type = r.int32();
                                        break;
                                    case 10:
                                        m.description = r.string();
                                        break;
                                    case 7:
                                        if (!(m.members && m.members.length))
                                            m.members = [];
                                        m.members.push($root.google.ads.googleads.v11.resources.CustomInterestMember.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomInterest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomInterest} CustomInterest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomInterest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomInterest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.members != null && m.hasOwnProperty("members")) {
                                    if (!Array.isArray(m.members))
                                        return "members: array expected";
                                    for (var i = 0; i < m.members.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.CustomInterestMember.verify(m.members[i]);
                                            if (e)
                                                return "members." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomInterest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomInterest} CustomInterest
                             */
                            CustomInterest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomInterest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomInterest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CUSTOM_AFFINITY":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "CUSTOM_INTENT":
                                case 3:
                                    m.type = 3;
                                    break;
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.members) {
                                    if (!Array.isArray(d.members))
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomInterest.members: array expected");
                                    m.members = [];
                                    for (var i = 0; i < d.members.length; ++i) {
                                        if (typeof d.members[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.CustomInterest.members: object expected");
                                        m.members[i] = $root.google.ads.googleads.v11.resources.CustomInterestMember.fromObject(d.members[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomInterest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomInterest} m CustomInterest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomInterest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.members = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomInterestStatusEnum.CustomInterestStatus[m.status] : m.status;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomInterestTypeEnum.CustomInterestType[m.type] : m.type;
                                }
                                if (m.members && m.members.length) {
                                    d.members = [];
                                    for (var j = 0; j < m.members.length; ++j) {
                                        d.members[j] = $root.google.ads.googleads.v11.resources.CustomInterestMember.toObject(m.members[j], o);
                                    }
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomInterest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomInterest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomInterest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomInterest;
                        })();
    
                        resources.CustomInterestMember = (function() {
    
                            /**
                             * Properties of a CustomInterestMember.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomInterestMember
                             * @property {google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum.CustomInterestMemberType|null} [member_type] CustomInterestMember member_type
                             * @property {string|null} [parameter] CustomInterestMember parameter
                             */
    
                            /**
                             * Constructs a new CustomInterestMember.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomInterestMember.
                             * @implements ICustomInterestMember
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomInterestMember=} [p] Properties to set
                             */
                            function CustomInterestMember(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomInterestMember member_type.
                             * @member {google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum.CustomInterestMemberType} member_type
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @instance
                             */
                            CustomInterestMember.prototype.member_type = 0;
    
                            /**
                             * CustomInterestMember parameter.
                             * @member {string|null|undefined} parameter
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @instance
                             */
                            CustomInterestMember.prototype.parameter = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomInterestMember _parameter.
                             * @member {"parameter"|undefined} _parameter
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @instance
                             */
                            Object.defineProperty(CustomInterestMember.prototype, "_parameter", {
                                get: $util.oneOfGetter($oneOfFields = ["parameter"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomInterestMember instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomInterestMember=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomInterestMember} CustomInterestMember instance
                             */
                            CustomInterestMember.create = function create(properties) {
                                return new CustomInterestMember(properties);
                            };
    
                            /**
                             * Encodes the specified CustomInterestMember message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomInterestMember.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomInterestMember} m CustomInterestMember message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestMember.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.member_type != null && Object.hasOwnProperty.call(m, "member_type"))
                                    w.uint32(8).int32(m.member_type);
                                if (m.parameter != null && Object.hasOwnProperty.call(m, "parameter"))
                                    w.uint32(26).string(m.parameter);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomInterestMember message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomInterestMember.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomInterestMember} message CustomInterestMember message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestMember.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomInterestMember message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomInterestMember} CustomInterestMember
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestMember.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomInterestMember();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.member_type = r.int32();
                                        break;
                                    case 3:
                                        m.parameter = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomInterestMember message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomInterestMember} CustomInterestMember
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestMember.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomInterestMember message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomInterestMember.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.member_type != null && m.hasOwnProperty("member_type")) {
                                    switch (m.member_type) {
                                    default:
                                        return "member_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.parameter != null && m.hasOwnProperty("parameter")) {
                                    p._parameter = 1;
                                    if (!$util.isString(m.parameter))
                                        return "parameter: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomInterestMember message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomInterestMember} CustomInterestMember
                             */
                            CustomInterestMember.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomInterestMember)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomInterestMember();
                                switch (d.member_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.member_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.member_type = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.member_type = 2;
                                    break;
                                case "URL":
                                case 3:
                                    m.member_type = 3;
                                    break;
                                }
                                if (d.parameter != null) {
                                    m.parameter = String(d.parameter);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomInterestMember message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomInterestMember} m CustomInterestMember
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomInterestMember.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.member_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.member_type != null && m.hasOwnProperty("member_type")) {
                                    d.member_type = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomInterestMemberTypeEnum.CustomInterestMemberType[m.member_type] : m.member_type;
                                }
                                if (m.parameter != null && m.hasOwnProperty("parameter")) {
                                    d.parameter = m.parameter;
                                    if (o.oneofs)
                                        d._parameter = "parameter";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomInterestMember to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomInterestMember
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomInterestMember.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomInterestMember;
                        })();
    
                        resources.Customer = (function() {
    
                            /**
                             * Properties of a Customer.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomer
                             * @property {string|null} [resource_name] Customer resource_name
                             * @property {number|null} [id] Customer id
                             * @property {string|null} [descriptive_name] Customer descriptive_name
                             * @property {string|null} [currency_code] Customer currency_code
                             * @property {string|null} [time_zone] Customer time_zone
                             * @property {string|null} [tracking_url_template] Customer tracking_url_template
                             * @property {string|null} [final_url_suffix] Customer final_url_suffix
                             * @property {boolean|null} [auto_tagging_enabled] Customer auto_tagging_enabled
                             * @property {boolean|null} [has_partners_badge] Customer has_partners_badge
                             * @property {boolean|null} [manager] Customer manager
                             * @property {boolean|null} [test_account] Customer test_account
                             * @property {google.ads.googleads.v11.resources.ICallReportingSetting|null} [call_reporting_setting] Customer call_reporting_setting
                             * @property {google.ads.googleads.v11.resources.IConversionTrackingSetting|null} [conversion_tracking_setting] Customer conversion_tracking_setting
                             * @property {google.ads.googleads.v11.resources.IRemarketingSetting|null} [remarketing_setting] Customer remarketing_setting
                             * @property {Array.<google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason>|null} [pay_per_conversion_eligibility_failure_reasons] Customer pay_per_conversion_eligibility_failure_reasons
                             * @property {number|null} [optimization_score] Customer optimization_score
                             * @property {number|null} [optimization_score_weight] Customer optimization_score_weight
                             * @property {google.ads.googleads.v11.enums.CustomerStatusEnum.CustomerStatus|null} [status] Customer status
                             */
    
                            /**
                             * Constructs a new Customer.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a Customer.
                             * @implements ICustomer
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomer=} [p] Properties to set
                             */
                            function Customer(p) {
                                this.pay_per_conversion_eligibility_failure_reasons = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Customer resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.resource_name = "";
    
                            /**
                             * Customer id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.id = null;
    
                            /**
                             * Customer descriptive_name.
                             * @member {string|null|undefined} descriptive_name
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.descriptive_name = null;
    
                            /**
                             * Customer currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.currency_code = null;
    
                            /**
                             * Customer time_zone.
                             * @member {string|null|undefined} time_zone
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.time_zone = null;
    
                            /**
                             * Customer tracking_url_template.
                             * @member {string|null|undefined} tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.tracking_url_template = null;
    
                            /**
                             * Customer final_url_suffix.
                             * @member {string|null|undefined} final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.final_url_suffix = null;
    
                            /**
                             * Customer auto_tagging_enabled.
                             * @member {boolean|null|undefined} auto_tagging_enabled
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.auto_tagging_enabled = null;
    
                            /**
                             * Customer has_partners_badge.
                             * @member {boolean|null|undefined} has_partners_badge
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.has_partners_badge = null;
    
                            /**
                             * Customer manager.
                             * @member {boolean|null|undefined} manager
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.manager = null;
    
                            /**
                             * Customer test_account.
                             * @member {boolean|null|undefined} test_account
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.test_account = null;
    
                            /**
                             * Customer call_reporting_setting.
                             * @member {google.ads.googleads.v11.resources.ICallReportingSetting|null|undefined} call_reporting_setting
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.call_reporting_setting = null;
    
                            /**
                             * Customer conversion_tracking_setting.
                             * @member {google.ads.googleads.v11.resources.IConversionTrackingSetting|null|undefined} conversion_tracking_setting
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.conversion_tracking_setting = null;
    
                            /**
                             * Customer remarketing_setting.
                             * @member {google.ads.googleads.v11.resources.IRemarketingSetting|null|undefined} remarketing_setting
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.remarketing_setting = null;
    
                            /**
                             * Customer pay_per_conversion_eligibility_failure_reasons.
                             * @member {Array.<google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason>} pay_per_conversion_eligibility_failure_reasons
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.pay_per_conversion_eligibility_failure_reasons = $util.emptyArray;
    
                            /**
                             * Customer optimization_score.
                             * @member {number|null|undefined} optimization_score
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.optimization_score = null;
    
                            /**
                             * Customer optimization_score_weight.
                             * @member {number} optimization_score_weight
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.optimization_score_weight = 0;
    
                            /**
                             * Customer status.
                             * @member {google.ads.googleads.v11.enums.CustomerStatusEnum.CustomerStatus} status
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Customer.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Customer _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _descriptive_name.
                             * @member {"descriptive_name"|undefined} _descriptive_name
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_descriptive_name", {
                                get: $util.oneOfGetter($oneOfFields = ["descriptive_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _time_zone.
                             * @member {"time_zone"|undefined} _time_zone
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_time_zone", {
                                get: $util.oneOfGetter($oneOfFields = ["time_zone"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _tracking_url_template.
                             * @member {"tracking_url_template"|undefined} _tracking_url_template
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_tracking_url_template", {
                                get: $util.oneOfGetter($oneOfFields = ["tracking_url_template"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _final_url_suffix.
                             * @member {"final_url_suffix"|undefined} _final_url_suffix
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_final_url_suffix", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url_suffix"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _auto_tagging_enabled.
                             * @member {"auto_tagging_enabled"|undefined} _auto_tagging_enabled
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_auto_tagging_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["auto_tagging_enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _has_partners_badge.
                             * @member {"has_partners_badge"|undefined} _has_partners_badge
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_has_partners_badge", {
                                get: $util.oneOfGetter($oneOfFields = ["has_partners_badge"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _manager.
                             * @member {"manager"|undefined} _manager
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_manager", {
                                get: $util.oneOfGetter($oneOfFields = ["manager"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _test_account.
                             * @member {"test_account"|undefined} _test_account
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_test_account", {
                                get: $util.oneOfGetter($oneOfFields = ["test_account"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Customer _optimization_score.
                             * @member {"optimization_score"|undefined} _optimization_score
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             */
                            Object.defineProperty(Customer.prototype, "_optimization_score", {
                                get: $util.oneOfGetter($oneOfFields = ["optimization_score"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Customer instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomer=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Customer} Customer instance
                             */
                            Customer.create = function create(properties) {
                                return new Customer(properties);
                            };
    
                            /**
                             * Encodes the specified Customer message. Does not implicitly {@link google.ads.googleads.v11.resources.Customer.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomer} m Customer message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Customer.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.call_reporting_setting != null && Object.hasOwnProperty.call(m, "call_reporting_setting"))
                                    $root.google.ads.googleads.v11.resources.CallReportingSetting.encode(m.call_reporting_setting, w.uint32(82).fork()).ldelim();
                                if (m.conversion_tracking_setting != null && Object.hasOwnProperty.call(m, "conversion_tracking_setting"))
                                    $root.google.ads.googleads.v11.resources.ConversionTrackingSetting.encode(m.conversion_tracking_setting, w.uint32(114).fork()).ldelim();
                                if (m.remarketing_setting != null && Object.hasOwnProperty.call(m, "remarketing_setting"))
                                    $root.google.ads.googleads.v11.resources.RemarketingSetting.encode(m.remarketing_setting, w.uint32(122).fork()).ldelim();
                                if (m.pay_per_conversion_eligibility_failure_reasons != null && m.pay_per_conversion_eligibility_failure_reasons.length) {
                                    w.uint32(130).fork();
                                    for (var i = 0; i < m.pay_per_conversion_eligibility_failure_reasons.length; ++i)
                                        w.int32(m.pay_per_conversion_eligibility_failure_reasons[i]);
                                    w.ldelim();
                                }
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(152).int64(m.id);
                                if (m.descriptive_name != null && Object.hasOwnProperty.call(m, "descriptive_name"))
                                    w.uint32(162).string(m.descriptive_name);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(170).string(m.currency_code);
                                if (m.time_zone != null && Object.hasOwnProperty.call(m, "time_zone"))
                                    w.uint32(178).string(m.time_zone);
                                if (m.tracking_url_template != null && Object.hasOwnProperty.call(m, "tracking_url_template"))
                                    w.uint32(186).string(m.tracking_url_template);
                                if (m.final_url_suffix != null && Object.hasOwnProperty.call(m, "final_url_suffix"))
                                    w.uint32(194).string(m.final_url_suffix);
                                if (m.auto_tagging_enabled != null && Object.hasOwnProperty.call(m, "auto_tagging_enabled"))
                                    w.uint32(200).bool(m.auto_tagging_enabled);
                                if (m.has_partners_badge != null && Object.hasOwnProperty.call(m, "has_partners_badge"))
                                    w.uint32(208).bool(m.has_partners_badge);
                                if (m.manager != null && Object.hasOwnProperty.call(m, "manager"))
                                    w.uint32(216).bool(m.manager);
                                if (m.test_account != null && Object.hasOwnProperty.call(m, "test_account"))
                                    w.uint32(224).bool(m.test_account);
                                if (m.optimization_score != null && Object.hasOwnProperty.call(m, "optimization_score"))
                                    w.uint32(233).double(m.optimization_score);
                                if (m.optimization_score_weight != null && Object.hasOwnProperty.call(m, "optimization_score_weight"))
                                    w.uint32(241).double(m.optimization_score_weight);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(288).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Customer message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Customer.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomer} message Customer message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Customer.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Customer message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Customer} Customer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Customer.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Customer();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 19:
                                        m.id = r.int64();
                                        break;
                                    case 20:
                                        m.descriptive_name = r.string();
                                        break;
                                    case 21:
                                        m.currency_code = r.string();
                                        break;
                                    case 22:
                                        m.time_zone = r.string();
                                        break;
                                    case 23:
                                        m.tracking_url_template = r.string();
                                        break;
                                    case 24:
                                        m.final_url_suffix = r.string();
                                        break;
                                    case 25:
                                        m.auto_tagging_enabled = r.bool();
                                        break;
                                    case 26:
                                        m.has_partners_badge = r.bool();
                                        break;
                                    case 27:
                                        m.manager = r.bool();
                                        break;
                                    case 28:
                                        m.test_account = r.bool();
                                        break;
                                    case 10:
                                        m.call_reporting_setting = $root.google.ads.googleads.v11.resources.CallReportingSetting.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.conversion_tracking_setting = $root.google.ads.googleads.v11.resources.ConversionTrackingSetting.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.remarketing_setting = $root.google.ads.googleads.v11.resources.RemarketingSetting.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        if (!(m.pay_per_conversion_eligibility_failure_reasons && m.pay_per_conversion_eligibility_failure_reasons.length))
                                            m.pay_per_conversion_eligibility_failure_reasons = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.pay_per_conversion_eligibility_failure_reasons.push(r.int32());
                                        } else
                                            m.pay_per_conversion_eligibility_failure_reasons.push(r.int32());
                                        break;
                                    case 29:
                                        m.optimization_score = r.double();
                                        break;
                                    case 30:
                                        m.optimization_score_weight = r.double();
                                        break;
                                    case 36:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Customer message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Customer} Customer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Customer.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Customer message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Customer.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.descriptive_name != null && m.hasOwnProperty("descriptive_name")) {
                                    p._descriptive_name = 1;
                                    if (!$util.isString(m.descriptive_name))
                                        return "descriptive_name: string expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.time_zone != null && m.hasOwnProperty("time_zone")) {
                                    p._time_zone = 1;
                                    if (!$util.isString(m.time_zone))
                                        return "time_zone: string expected";
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    p._tracking_url_template = 1;
                                    if (!$util.isString(m.tracking_url_template))
                                        return "tracking_url_template: string expected";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    p._final_url_suffix = 1;
                                    if (!$util.isString(m.final_url_suffix))
                                        return "final_url_suffix: string expected";
                                }
                                if (m.auto_tagging_enabled != null && m.hasOwnProperty("auto_tagging_enabled")) {
                                    p._auto_tagging_enabled = 1;
                                    if (typeof m.auto_tagging_enabled !== "boolean")
                                        return "auto_tagging_enabled: boolean expected";
                                }
                                if (m.has_partners_badge != null && m.hasOwnProperty("has_partners_badge")) {
                                    p._has_partners_badge = 1;
                                    if (typeof m.has_partners_badge !== "boolean")
                                        return "has_partners_badge: boolean expected";
                                }
                                if (m.manager != null && m.hasOwnProperty("manager")) {
                                    p._manager = 1;
                                    if (typeof m.manager !== "boolean")
                                        return "manager: boolean expected";
                                }
                                if (m.test_account != null && m.hasOwnProperty("test_account")) {
                                    p._test_account = 1;
                                    if (typeof m.test_account !== "boolean")
                                        return "test_account: boolean expected";
                                }
                                if (m.call_reporting_setting != null && m.hasOwnProperty("call_reporting_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CallReportingSetting.verify(m.call_reporting_setting);
                                        if (e)
                                            return "call_reporting_setting." + e;
                                    }
                                }
                                if (m.conversion_tracking_setting != null && m.hasOwnProperty("conversion_tracking_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionTrackingSetting.verify(m.conversion_tracking_setting);
                                        if (e)
                                            return "conversion_tracking_setting." + e;
                                    }
                                }
                                if (m.remarketing_setting != null && m.hasOwnProperty("remarketing_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.RemarketingSetting.verify(m.remarketing_setting);
                                        if (e)
                                            return "remarketing_setting." + e;
                                    }
                                }
                                if (m.pay_per_conversion_eligibility_failure_reasons != null && m.hasOwnProperty("pay_per_conversion_eligibility_failure_reasons")) {
                                    if (!Array.isArray(m.pay_per_conversion_eligibility_failure_reasons))
                                        return "pay_per_conversion_eligibility_failure_reasons: array expected";
                                    for (var i = 0; i < m.pay_per_conversion_eligibility_failure_reasons.length; ++i) {
                                        switch (m.pay_per_conversion_eligibility_failure_reasons[i]) {
                                        default:
                                            return "pay_per_conversion_eligibility_failure_reasons: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                            break;
                                        }
                                    }
                                }
                                if (m.optimization_score != null && m.hasOwnProperty("optimization_score")) {
                                    p._optimization_score = 1;
                                    if (typeof m.optimization_score !== "number")
                                        return "optimization_score: number expected";
                                }
                                if (m.optimization_score_weight != null && m.hasOwnProperty("optimization_score_weight")) {
                                    if (typeof m.optimization_score_weight !== "number")
                                        return "optimization_score_weight: number expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Customer message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Customer} Customer
                             */
                            Customer.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Customer)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Customer();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.descriptive_name != null) {
                                    m.descriptive_name = String(d.descriptive_name);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.time_zone != null) {
                                    m.time_zone = String(d.time_zone);
                                }
                                if (d.tracking_url_template != null) {
                                    m.tracking_url_template = String(d.tracking_url_template);
                                }
                                if (d.final_url_suffix != null) {
                                    m.final_url_suffix = String(d.final_url_suffix);
                                }
                                if (d.auto_tagging_enabled != null) {
                                    m.auto_tagging_enabled = Boolean(d.auto_tagging_enabled);
                                }
                                if (d.has_partners_badge != null) {
                                    m.has_partners_badge = Boolean(d.has_partners_badge);
                                }
                                if (d.manager != null) {
                                    m.manager = Boolean(d.manager);
                                }
                                if (d.test_account != null) {
                                    m.test_account = Boolean(d.test_account);
                                }
                                if (d.call_reporting_setting != null) {
                                    if (typeof d.call_reporting_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Customer.call_reporting_setting: object expected");
                                    m.call_reporting_setting = $root.google.ads.googleads.v11.resources.CallReportingSetting.fromObject(d.call_reporting_setting);
                                }
                                if (d.conversion_tracking_setting != null) {
                                    if (typeof d.conversion_tracking_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Customer.conversion_tracking_setting: object expected");
                                    m.conversion_tracking_setting = $root.google.ads.googleads.v11.resources.ConversionTrackingSetting.fromObject(d.conversion_tracking_setting);
                                }
                                if (d.remarketing_setting != null) {
                                    if (typeof d.remarketing_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Customer.remarketing_setting: object expected");
                                    m.remarketing_setting = $root.google.ads.googleads.v11.resources.RemarketingSetting.fromObject(d.remarketing_setting);
                                }
                                if (d.pay_per_conversion_eligibility_failure_reasons) {
                                    if (!Array.isArray(d.pay_per_conversion_eligibility_failure_reasons))
                                        throw TypeError(".google.ads.googleads.v11.resources.Customer.pay_per_conversion_eligibility_failure_reasons: array expected");
                                    m.pay_per_conversion_eligibility_failure_reasons = [];
                                    for (var i = 0; i < d.pay_per_conversion_eligibility_failure_reasons.length; ++i) {
                                        switch (d.pay_per_conversion_eligibility_failure_reasons[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 1;
                                            break;
                                        case "NOT_ENOUGH_CONVERSIONS":
                                        case 2:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 2;
                                            break;
                                        case "CONVERSION_LAG_TOO_HIGH":
                                        case 3:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 3;
                                            break;
                                        case "HAS_CAMPAIGN_WITH_SHARED_BUDGET":
                                        case 4:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 4;
                                            break;
                                        case "HAS_UPLOAD_CLICKS_CONVERSION":
                                        case 5:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 5;
                                            break;
                                        case "AVERAGE_DAILY_SPEND_TOO_HIGH":
                                        case 6:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 6;
                                            break;
                                        case "ANALYSIS_NOT_COMPLETE":
                                        case 7:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 7;
                                            break;
                                        case "OTHER":
                                        case 8:
                                            m.pay_per_conversion_eligibility_failure_reasons[i] = 8;
                                            break;
                                        }
                                    }
                                }
                                if (d.optimization_score != null) {
                                    m.optimization_score = Number(d.optimization_score);
                                }
                                if (d.optimization_score_weight != null) {
                                    m.optimization_score_weight = Number(d.optimization_score_weight);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "CANCELED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "SUSPENDED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "CLOSED":
                                case 5:
                                    m.status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Customer message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @static
                             * @param {google.ads.googleads.v11.resources.Customer} m Customer
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Customer.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.pay_per_conversion_eligibility_failure_reasons = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.call_reporting_setting = null;
                                    d.conversion_tracking_setting = null;
                                    d.remarketing_setting = null;
                                    d.optimization_score_weight = 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.call_reporting_setting != null && m.hasOwnProperty("call_reporting_setting")) {
                                    d.call_reporting_setting = $root.google.ads.googleads.v11.resources.CallReportingSetting.toObject(m.call_reporting_setting, o);
                                }
                                if (m.conversion_tracking_setting != null && m.hasOwnProperty("conversion_tracking_setting")) {
                                    d.conversion_tracking_setting = $root.google.ads.googleads.v11.resources.ConversionTrackingSetting.toObject(m.conversion_tracking_setting, o);
                                }
                                if (m.remarketing_setting != null && m.hasOwnProperty("remarketing_setting")) {
                                    d.remarketing_setting = $root.google.ads.googleads.v11.resources.RemarketingSetting.toObject(m.remarketing_setting, o);
                                }
                                if (m.pay_per_conversion_eligibility_failure_reasons && m.pay_per_conversion_eligibility_failure_reasons.length) {
                                    d.pay_per_conversion_eligibility_failure_reasons = [];
                                    for (var j = 0; j < m.pay_per_conversion_eligibility_failure_reasons.length; ++j) {
                                        d.pay_per_conversion_eligibility_failure_reasons[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason[m.pay_per_conversion_eligibility_failure_reasons[j]] : m.pay_per_conversion_eligibility_failure_reasons[j];
                                    }
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.descriptive_name != null && m.hasOwnProperty("descriptive_name")) {
                                    d.descriptive_name = m.descriptive_name;
                                    if (o.oneofs)
                                        d._descriptive_name = "descriptive_name";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.time_zone != null && m.hasOwnProperty("time_zone")) {
                                    d.time_zone = m.time_zone;
                                    if (o.oneofs)
                                        d._time_zone = "time_zone";
                                }
                                if (m.tracking_url_template != null && m.hasOwnProperty("tracking_url_template")) {
                                    d.tracking_url_template = m.tracking_url_template;
                                    if (o.oneofs)
                                        d._tracking_url_template = "tracking_url_template";
                                }
                                if (m.final_url_suffix != null && m.hasOwnProperty("final_url_suffix")) {
                                    d.final_url_suffix = m.final_url_suffix;
                                    if (o.oneofs)
                                        d._final_url_suffix = "final_url_suffix";
                                }
                                if (m.auto_tagging_enabled != null && m.hasOwnProperty("auto_tagging_enabled")) {
                                    d.auto_tagging_enabled = m.auto_tagging_enabled;
                                    if (o.oneofs)
                                        d._auto_tagging_enabled = "auto_tagging_enabled";
                                }
                                if (m.has_partners_badge != null && m.hasOwnProperty("has_partners_badge")) {
                                    d.has_partners_badge = m.has_partners_badge;
                                    if (o.oneofs)
                                        d._has_partners_badge = "has_partners_badge";
                                }
                                if (m.manager != null && m.hasOwnProperty("manager")) {
                                    d.manager = m.manager;
                                    if (o.oneofs)
                                        d._manager = "manager";
                                }
                                if (m.test_account != null && m.hasOwnProperty("test_account")) {
                                    d.test_account = m.test_account;
                                    if (o.oneofs)
                                        d._test_account = "test_account";
                                }
                                if (m.optimization_score != null && m.hasOwnProperty("optimization_score")) {
                                    d.optimization_score = o.json && !isFinite(m.optimization_score) ? String(m.optimization_score) : m.optimization_score;
                                    if (o.oneofs)
                                        d._optimization_score = "optimization_score";
                                }
                                if (m.optimization_score_weight != null && m.hasOwnProperty("optimization_score_weight")) {
                                    d.optimization_score_weight = o.json && !isFinite(m.optimization_score_weight) ? String(m.optimization_score_weight) : m.optimization_score_weight;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomerStatusEnum.CustomerStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Customer to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Customer
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Customer.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Customer;
                        })();
    
                        resources.CallReportingSetting = (function() {
    
                            /**
                             * Properties of a CallReportingSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICallReportingSetting
                             * @property {boolean|null} [call_reporting_enabled] CallReportingSetting call_reporting_enabled
                             * @property {boolean|null} [call_conversion_reporting_enabled] CallReportingSetting call_conversion_reporting_enabled
                             * @property {string|null} [call_conversion_action] CallReportingSetting call_conversion_action
                             */
    
                            /**
                             * Constructs a new CallReportingSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CallReportingSetting.
                             * @implements ICallReportingSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICallReportingSetting=} [p] Properties to set
                             */
                            function CallReportingSetting(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallReportingSetting call_reporting_enabled.
                             * @member {boolean|null|undefined} call_reporting_enabled
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @instance
                             */
                            CallReportingSetting.prototype.call_reporting_enabled = null;
    
                            /**
                             * CallReportingSetting call_conversion_reporting_enabled.
                             * @member {boolean|null|undefined} call_conversion_reporting_enabled
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @instance
                             */
                            CallReportingSetting.prototype.call_conversion_reporting_enabled = null;
    
                            /**
                             * CallReportingSetting call_conversion_action.
                             * @member {string|null|undefined} call_conversion_action
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @instance
                             */
                            CallReportingSetting.prototype.call_conversion_action = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CallReportingSetting _call_reporting_enabled.
                             * @member {"call_reporting_enabled"|undefined} _call_reporting_enabled
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @instance
                             */
                            Object.defineProperty(CallReportingSetting.prototype, "_call_reporting_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["call_reporting_enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallReportingSetting _call_conversion_reporting_enabled.
                             * @member {"call_conversion_reporting_enabled"|undefined} _call_conversion_reporting_enabled
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @instance
                             */
                            Object.defineProperty(CallReportingSetting.prototype, "_call_conversion_reporting_enabled", {
                                get: $util.oneOfGetter($oneOfFields = ["call_conversion_reporting_enabled"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallReportingSetting _call_conversion_action.
                             * @member {"call_conversion_action"|undefined} _call_conversion_action
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @instance
                             */
                            Object.defineProperty(CallReportingSetting.prototype, "_call_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["call_conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CallReportingSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICallReportingSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CallReportingSetting} CallReportingSetting instance
                             */
                            CallReportingSetting.create = function create(properties) {
                                return new CallReportingSetting(properties);
                            };
    
                            /**
                             * Encodes the specified CallReportingSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.CallReportingSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICallReportingSetting} m CallReportingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallReportingSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.call_reporting_enabled != null && Object.hasOwnProperty.call(m, "call_reporting_enabled"))
                                    w.uint32(80).bool(m.call_reporting_enabled);
                                if (m.call_conversion_reporting_enabled != null && Object.hasOwnProperty.call(m, "call_conversion_reporting_enabled"))
                                    w.uint32(88).bool(m.call_conversion_reporting_enabled);
                                if (m.call_conversion_action != null && Object.hasOwnProperty.call(m, "call_conversion_action"))
                                    w.uint32(98).string(m.call_conversion_action);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallReportingSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CallReportingSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICallReportingSetting} message CallReportingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallReportingSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallReportingSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CallReportingSetting} CallReportingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallReportingSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CallReportingSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 10:
                                        m.call_reporting_enabled = r.bool();
                                        break;
                                    case 11:
                                        m.call_conversion_reporting_enabled = r.bool();
                                        break;
                                    case 12:
                                        m.call_conversion_action = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallReportingSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CallReportingSetting} CallReportingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallReportingSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallReportingSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallReportingSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.call_reporting_enabled != null && m.hasOwnProperty("call_reporting_enabled")) {
                                    p._call_reporting_enabled = 1;
                                    if (typeof m.call_reporting_enabled !== "boolean")
                                        return "call_reporting_enabled: boolean expected";
                                }
                                if (m.call_conversion_reporting_enabled != null && m.hasOwnProperty("call_conversion_reporting_enabled")) {
                                    p._call_conversion_reporting_enabled = 1;
                                    if (typeof m.call_conversion_reporting_enabled !== "boolean")
                                        return "call_conversion_reporting_enabled: boolean expected";
                                }
                                if (m.call_conversion_action != null && m.hasOwnProperty("call_conversion_action")) {
                                    p._call_conversion_action = 1;
                                    if (!$util.isString(m.call_conversion_action))
                                        return "call_conversion_action: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallReportingSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CallReportingSetting} CallReportingSetting
                             */
                            CallReportingSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CallReportingSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CallReportingSetting();
                                if (d.call_reporting_enabled != null) {
                                    m.call_reporting_enabled = Boolean(d.call_reporting_enabled);
                                }
                                if (d.call_conversion_reporting_enabled != null) {
                                    m.call_conversion_reporting_enabled = Boolean(d.call_conversion_reporting_enabled);
                                }
                                if (d.call_conversion_action != null) {
                                    m.call_conversion_action = String(d.call_conversion_action);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallReportingSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.CallReportingSetting} m CallReportingSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallReportingSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.call_reporting_enabled != null && m.hasOwnProperty("call_reporting_enabled")) {
                                    d.call_reporting_enabled = m.call_reporting_enabled;
                                    if (o.oneofs)
                                        d._call_reporting_enabled = "call_reporting_enabled";
                                }
                                if (m.call_conversion_reporting_enabled != null && m.hasOwnProperty("call_conversion_reporting_enabled")) {
                                    d.call_conversion_reporting_enabled = m.call_conversion_reporting_enabled;
                                    if (o.oneofs)
                                        d._call_conversion_reporting_enabled = "call_conversion_reporting_enabled";
                                }
                                if (m.call_conversion_action != null && m.hasOwnProperty("call_conversion_action")) {
                                    d.call_conversion_action = m.call_conversion_action;
                                    if (o.oneofs)
                                        d._call_conversion_action = "call_conversion_action";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallReportingSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CallReportingSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallReportingSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallReportingSetting;
                        })();
    
                        resources.ConversionTrackingSetting = (function() {
    
                            /**
                             * Properties of a ConversionTrackingSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IConversionTrackingSetting
                             * @property {number|null} [conversion_tracking_id] ConversionTrackingSetting conversion_tracking_id
                             * @property {number|null} [cross_account_conversion_tracking_id] ConversionTrackingSetting cross_account_conversion_tracking_id
                             * @property {boolean|null} [accepted_customer_data_terms] ConversionTrackingSetting accepted_customer_data_terms
                             * @property {google.ads.googleads.v11.enums.ConversionTrackingStatusEnum.ConversionTrackingStatus|null} [conversion_tracking_status] ConversionTrackingSetting conversion_tracking_status
                             * @property {boolean|null} [enhanced_conversions_for_leads_enabled] ConversionTrackingSetting enhanced_conversions_for_leads_enabled
                             * @property {string|null} [google_ads_conversion_customer] ConversionTrackingSetting google_ads_conversion_customer
                             */
    
                            /**
                             * Constructs a new ConversionTrackingSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ConversionTrackingSetting.
                             * @implements IConversionTrackingSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IConversionTrackingSetting=} [p] Properties to set
                             */
                            function ConversionTrackingSetting(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionTrackingSetting conversion_tracking_id.
                             * @member {number|null|undefined} conversion_tracking_id
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            ConversionTrackingSetting.prototype.conversion_tracking_id = null;
    
                            /**
                             * ConversionTrackingSetting cross_account_conversion_tracking_id.
                             * @member {number|null|undefined} cross_account_conversion_tracking_id
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            ConversionTrackingSetting.prototype.cross_account_conversion_tracking_id = null;
    
                            /**
                             * ConversionTrackingSetting accepted_customer_data_terms.
                             * @member {boolean} accepted_customer_data_terms
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            ConversionTrackingSetting.prototype.accepted_customer_data_terms = false;
    
                            /**
                             * ConversionTrackingSetting conversion_tracking_status.
                             * @member {google.ads.googleads.v11.enums.ConversionTrackingStatusEnum.ConversionTrackingStatus} conversion_tracking_status
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            ConversionTrackingSetting.prototype.conversion_tracking_status = 0;
    
                            /**
                             * ConversionTrackingSetting enhanced_conversions_for_leads_enabled.
                             * @member {boolean} enhanced_conversions_for_leads_enabled
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            ConversionTrackingSetting.prototype.enhanced_conversions_for_leads_enabled = false;
    
                            /**
                             * ConversionTrackingSetting google_ads_conversion_customer.
                             * @member {string} google_ads_conversion_customer
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            ConversionTrackingSetting.prototype.google_ads_conversion_customer = "";
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionTrackingSetting _conversion_tracking_id.
                             * @member {"conversion_tracking_id"|undefined} _conversion_tracking_id
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            Object.defineProperty(ConversionTrackingSetting.prototype, "_conversion_tracking_id", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_tracking_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionTrackingSetting _cross_account_conversion_tracking_id.
                             * @member {"cross_account_conversion_tracking_id"|undefined} _cross_account_conversion_tracking_id
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             */
                            Object.defineProperty(ConversionTrackingSetting.prototype, "_cross_account_conversion_tracking_id", {
                                get: $util.oneOfGetter($oneOfFields = ["cross_account_conversion_tracking_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionTrackingSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionTrackingSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ConversionTrackingSetting} ConversionTrackingSetting instance
                             */
                            ConversionTrackingSetting.create = function create(properties) {
                                return new ConversionTrackingSetting(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionTrackingSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionTrackingSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionTrackingSetting} m ConversionTrackingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionTrackingSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.conversion_tracking_id != null && Object.hasOwnProperty.call(m, "conversion_tracking_id"))
                                    w.uint32(24).int64(m.conversion_tracking_id);
                                if (m.cross_account_conversion_tracking_id != null && Object.hasOwnProperty.call(m, "cross_account_conversion_tracking_id"))
                                    w.uint32(32).int64(m.cross_account_conversion_tracking_id);
                                if (m.accepted_customer_data_terms != null && Object.hasOwnProperty.call(m, "accepted_customer_data_terms"))
                                    w.uint32(40).bool(m.accepted_customer_data_terms);
                                if (m.conversion_tracking_status != null && Object.hasOwnProperty.call(m, "conversion_tracking_status"))
                                    w.uint32(48).int32(m.conversion_tracking_status);
                                if (m.enhanced_conversions_for_leads_enabled != null && Object.hasOwnProperty.call(m, "enhanced_conversions_for_leads_enabled"))
                                    w.uint32(56).bool(m.enhanced_conversions_for_leads_enabled);
                                if (m.google_ads_conversion_customer != null && Object.hasOwnProperty.call(m, "google_ads_conversion_customer"))
                                    w.uint32(66).string(m.google_ads_conversion_customer);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionTrackingSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ConversionTrackingSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IConversionTrackingSetting} message ConversionTrackingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionTrackingSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionTrackingSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ConversionTrackingSetting} ConversionTrackingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionTrackingSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ConversionTrackingSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.conversion_tracking_id = r.int64();
                                        break;
                                    case 4:
                                        m.cross_account_conversion_tracking_id = r.int64();
                                        break;
                                    case 5:
                                        m.accepted_customer_data_terms = r.bool();
                                        break;
                                    case 6:
                                        m.conversion_tracking_status = r.int32();
                                        break;
                                    case 7:
                                        m.enhanced_conversions_for_leads_enabled = r.bool();
                                        break;
                                    case 8:
                                        m.google_ads_conversion_customer = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionTrackingSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ConversionTrackingSetting} ConversionTrackingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionTrackingSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionTrackingSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionTrackingSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.conversion_tracking_id != null && m.hasOwnProperty("conversion_tracking_id")) {
                                    p._conversion_tracking_id = 1;
                                    if (!$util.isInteger(m.conversion_tracking_id) && !(m.conversion_tracking_id && $util.isInteger(m.conversion_tracking_id.low) && $util.isInteger(m.conversion_tracking_id.high)))
                                        return "conversion_tracking_id: integer|Long expected";
                                }
                                if (m.cross_account_conversion_tracking_id != null && m.hasOwnProperty("cross_account_conversion_tracking_id")) {
                                    p._cross_account_conversion_tracking_id = 1;
                                    if (!$util.isInteger(m.cross_account_conversion_tracking_id) && !(m.cross_account_conversion_tracking_id && $util.isInteger(m.cross_account_conversion_tracking_id.low) && $util.isInteger(m.cross_account_conversion_tracking_id.high)))
                                        return "cross_account_conversion_tracking_id: integer|Long expected";
                                }
                                if (m.accepted_customer_data_terms != null && m.hasOwnProperty("accepted_customer_data_terms")) {
                                    if (typeof m.accepted_customer_data_terms !== "boolean")
                                        return "accepted_customer_data_terms: boolean expected";
                                }
                                if (m.conversion_tracking_status != null && m.hasOwnProperty("conversion_tracking_status")) {
                                    switch (m.conversion_tracking_status) {
                                    default:
                                        return "conversion_tracking_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.enhanced_conversions_for_leads_enabled != null && m.hasOwnProperty("enhanced_conversions_for_leads_enabled")) {
                                    if (typeof m.enhanced_conversions_for_leads_enabled !== "boolean")
                                        return "enhanced_conversions_for_leads_enabled: boolean expected";
                                }
                                if (m.google_ads_conversion_customer != null && m.hasOwnProperty("google_ads_conversion_customer")) {
                                    if (!$util.isString(m.google_ads_conversion_customer))
                                        return "google_ads_conversion_customer: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionTrackingSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ConversionTrackingSetting} ConversionTrackingSetting
                             */
                            ConversionTrackingSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ConversionTrackingSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ConversionTrackingSetting();
                                if (d.conversion_tracking_id != null) {
                                    if ($util.Long)
                                        (m.conversion_tracking_id = $util.Long.fromValue(d.conversion_tracking_id)).unsigned = false;
                                    else if (typeof d.conversion_tracking_id === "string")
                                        m.conversion_tracking_id = parseInt(d.conversion_tracking_id, 10);
                                    else if (typeof d.conversion_tracking_id === "number")
                                        m.conversion_tracking_id = d.conversion_tracking_id;
                                    else if (typeof d.conversion_tracking_id === "object")
                                        m.conversion_tracking_id = new $util.LongBits(d.conversion_tracking_id.low >>> 0, d.conversion_tracking_id.high >>> 0).toNumber();
                                }
                                if (d.cross_account_conversion_tracking_id != null) {
                                    if ($util.Long)
                                        (m.cross_account_conversion_tracking_id = $util.Long.fromValue(d.cross_account_conversion_tracking_id)).unsigned = false;
                                    else if (typeof d.cross_account_conversion_tracking_id === "string")
                                        m.cross_account_conversion_tracking_id = parseInt(d.cross_account_conversion_tracking_id, 10);
                                    else if (typeof d.cross_account_conversion_tracking_id === "number")
                                        m.cross_account_conversion_tracking_id = d.cross_account_conversion_tracking_id;
                                    else if (typeof d.cross_account_conversion_tracking_id === "object")
                                        m.cross_account_conversion_tracking_id = new $util.LongBits(d.cross_account_conversion_tracking_id.low >>> 0, d.cross_account_conversion_tracking_id.high >>> 0).toNumber();
                                }
                                if (d.accepted_customer_data_terms != null) {
                                    m.accepted_customer_data_terms = Boolean(d.accepted_customer_data_terms);
                                }
                                switch (d.conversion_tracking_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_tracking_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_tracking_status = 1;
                                    break;
                                case "NOT_CONVERSION_TRACKED":
                                case 2:
                                    m.conversion_tracking_status = 2;
                                    break;
                                case "CONVERSION_TRACKING_MANAGED_BY_SELF":
                                case 3:
                                    m.conversion_tracking_status = 3;
                                    break;
                                case "CONVERSION_TRACKING_MANAGED_BY_THIS_MANAGER":
                                case 4:
                                    m.conversion_tracking_status = 4;
                                    break;
                                case "CONVERSION_TRACKING_MANAGED_BY_ANOTHER_MANAGER":
                                case 5:
                                    m.conversion_tracking_status = 5;
                                    break;
                                }
                                if (d.enhanced_conversions_for_leads_enabled != null) {
                                    m.enhanced_conversions_for_leads_enabled = Boolean(d.enhanced_conversions_for_leads_enabled);
                                }
                                if (d.google_ads_conversion_customer != null) {
                                    m.google_ads_conversion_customer = String(d.google_ads_conversion_customer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionTrackingSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ConversionTrackingSetting} m ConversionTrackingSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionTrackingSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.accepted_customer_data_terms = false;
                                    d.conversion_tracking_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.enhanced_conversions_for_leads_enabled = false;
                                    d.google_ads_conversion_customer = "";
                                }
                                if (m.conversion_tracking_id != null && m.hasOwnProperty("conversion_tracking_id")) {
                                    if (typeof m.conversion_tracking_id === "number")
                                        d.conversion_tracking_id = o.longs === String ? String(m.conversion_tracking_id) : m.conversion_tracking_id;
                                    else
                                        d.conversion_tracking_id = o.longs === String ? $util.Long.prototype.toString.call(m.conversion_tracking_id) : o.longs === Number ? new $util.LongBits(m.conversion_tracking_id.low >>> 0, m.conversion_tracking_id.high >>> 0).toNumber() : m.conversion_tracking_id;
                                    if (o.oneofs)
                                        d._conversion_tracking_id = "conversion_tracking_id";
                                }
                                if (m.cross_account_conversion_tracking_id != null && m.hasOwnProperty("cross_account_conversion_tracking_id")) {
                                    if (typeof m.cross_account_conversion_tracking_id === "number")
                                        d.cross_account_conversion_tracking_id = o.longs === String ? String(m.cross_account_conversion_tracking_id) : m.cross_account_conversion_tracking_id;
                                    else
                                        d.cross_account_conversion_tracking_id = o.longs === String ? $util.Long.prototype.toString.call(m.cross_account_conversion_tracking_id) : o.longs === Number ? new $util.LongBits(m.cross_account_conversion_tracking_id.low >>> 0, m.cross_account_conversion_tracking_id.high >>> 0).toNumber() : m.cross_account_conversion_tracking_id;
                                    if (o.oneofs)
                                        d._cross_account_conversion_tracking_id = "cross_account_conversion_tracking_id";
                                }
                                if (m.accepted_customer_data_terms != null && m.hasOwnProperty("accepted_customer_data_terms")) {
                                    d.accepted_customer_data_terms = m.accepted_customer_data_terms;
                                }
                                if (m.conversion_tracking_status != null && m.hasOwnProperty("conversion_tracking_status")) {
                                    d.conversion_tracking_status = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionTrackingStatusEnum.ConversionTrackingStatus[m.conversion_tracking_status] : m.conversion_tracking_status;
                                }
                                if (m.enhanced_conversions_for_leads_enabled != null && m.hasOwnProperty("enhanced_conversions_for_leads_enabled")) {
                                    d.enhanced_conversions_for_leads_enabled = m.enhanced_conversions_for_leads_enabled;
                                }
                                if (m.google_ads_conversion_customer != null && m.hasOwnProperty("google_ads_conversion_customer")) {
                                    d.google_ads_conversion_customer = m.google_ads_conversion_customer;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionTrackingSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ConversionTrackingSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionTrackingSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionTrackingSetting;
                        })();
    
                        resources.RemarketingSetting = (function() {
    
                            /**
                             * Properties of a RemarketingSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IRemarketingSetting
                             * @property {string|null} [google_global_site_tag] RemarketingSetting google_global_site_tag
                             */
    
                            /**
                             * Constructs a new RemarketingSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a RemarketingSetting.
                             * @implements IRemarketingSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IRemarketingSetting=} [p] Properties to set
                             */
                            function RemarketingSetting(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RemarketingSetting google_global_site_tag.
                             * @member {string|null|undefined} google_global_site_tag
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @instance
                             */
                            RemarketingSetting.prototype.google_global_site_tag = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * RemarketingSetting _google_global_site_tag.
                             * @member {"google_global_site_tag"|undefined} _google_global_site_tag
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @instance
                             */
                            Object.defineProperty(RemarketingSetting.prototype, "_google_global_site_tag", {
                                get: $util.oneOfGetter($oneOfFields = ["google_global_site_tag"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new RemarketingSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRemarketingSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.RemarketingSetting} RemarketingSetting instance
                             */
                            RemarketingSetting.create = function create(properties) {
                                return new RemarketingSetting(properties);
                            };
    
                            /**
                             * Encodes the specified RemarketingSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.RemarketingSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRemarketingSetting} m RemarketingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RemarketingSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.google_global_site_tag != null && Object.hasOwnProperty.call(m, "google_global_site_tag"))
                                    w.uint32(18).string(m.google_global_site_tag);
                                return w;
                            };
    
                            /**
                             * Encodes the specified RemarketingSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.RemarketingSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRemarketingSetting} message RemarketingSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RemarketingSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RemarketingSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.RemarketingSetting} RemarketingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RemarketingSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.RemarketingSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.google_global_site_tag = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RemarketingSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.RemarketingSetting} RemarketingSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RemarketingSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RemarketingSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RemarketingSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.google_global_site_tag != null && m.hasOwnProperty("google_global_site_tag")) {
                                    p._google_global_site_tag = 1;
                                    if (!$util.isString(m.google_global_site_tag))
                                        return "google_global_site_tag: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RemarketingSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.RemarketingSetting} RemarketingSetting
                             */
                            RemarketingSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.RemarketingSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.RemarketingSetting();
                                if (d.google_global_site_tag != null) {
                                    m.google_global_site_tag = String(d.google_global_site_tag);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RemarketingSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.RemarketingSetting} m RemarketingSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RemarketingSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.google_global_site_tag != null && m.hasOwnProperty("google_global_site_tag")) {
                                    d.google_global_site_tag = m.google_global_site_tag;
                                    if (o.oneofs)
                                        d._google_global_site_tag = "google_global_site_tag";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RemarketingSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.RemarketingSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RemarketingSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RemarketingSetting;
                        })();
    
                        resources.CustomerClient = (function() {
    
                            /**
                             * Properties of a CustomerClient.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerClient
                             * @property {string|null} [resource_name] CustomerClient resource_name
                             * @property {string|null} [client_customer] CustomerClient client_customer
                             * @property {boolean|null} [hidden] CustomerClient hidden
                             * @property {number|null} [level] CustomerClient level
                             * @property {string|null} [time_zone] CustomerClient time_zone
                             * @property {boolean|null} [test_account] CustomerClient test_account
                             * @property {boolean|null} [manager] CustomerClient manager
                             * @property {string|null} [descriptive_name] CustomerClient descriptive_name
                             * @property {string|null} [currency_code] CustomerClient currency_code
                             * @property {number|null} [id] CustomerClient id
                             * @property {Array.<string>|null} [applied_labels] CustomerClient applied_labels
                             * @property {google.ads.googleads.v11.enums.CustomerStatusEnum.CustomerStatus|null} [status] CustomerClient status
                             */
    
                            /**
                             * Constructs a new CustomerClient.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerClient.
                             * @implements ICustomerClient
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerClient=} [p] Properties to set
                             */
                            function CustomerClient(p) {
                                this.applied_labels = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerClient resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.resource_name = "";
    
                            /**
                             * CustomerClient client_customer.
                             * @member {string|null|undefined} client_customer
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.client_customer = null;
    
                            /**
                             * CustomerClient hidden.
                             * @member {boolean|null|undefined} hidden
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.hidden = null;
    
                            /**
                             * CustomerClient level.
                             * @member {number|null|undefined} level
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.level = null;
    
                            /**
                             * CustomerClient time_zone.
                             * @member {string|null|undefined} time_zone
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.time_zone = null;
    
                            /**
                             * CustomerClient test_account.
                             * @member {boolean|null|undefined} test_account
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.test_account = null;
    
                            /**
                             * CustomerClient manager.
                             * @member {boolean|null|undefined} manager
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.manager = null;
    
                            /**
                             * CustomerClient descriptive_name.
                             * @member {string|null|undefined} descriptive_name
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.descriptive_name = null;
    
                            /**
                             * CustomerClient currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.currency_code = null;
    
                            /**
                             * CustomerClient id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.id = null;
    
                            /**
                             * CustomerClient applied_labels.
                             * @member {Array.<string>} applied_labels
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.applied_labels = $util.emptyArray;
    
                            /**
                             * CustomerClient status.
                             * @member {google.ads.googleads.v11.enums.CustomerStatusEnum.CustomerStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            CustomerClient.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerClient _client_customer.
                             * @member {"client_customer"|undefined} _client_customer
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_client_customer", {
                                get: $util.oneOfGetter($oneOfFields = ["client_customer"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _hidden.
                             * @member {"hidden"|undefined} _hidden
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_hidden", {
                                get: $util.oneOfGetter($oneOfFields = ["hidden"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _level.
                             * @member {"level"|undefined} _level
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_level", {
                                get: $util.oneOfGetter($oneOfFields = ["level"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _time_zone.
                             * @member {"time_zone"|undefined} _time_zone
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_time_zone", {
                                get: $util.oneOfGetter($oneOfFields = ["time_zone"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _test_account.
                             * @member {"test_account"|undefined} _test_account
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_test_account", {
                                get: $util.oneOfGetter($oneOfFields = ["test_account"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _manager.
                             * @member {"manager"|undefined} _manager
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_manager", {
                                get: $util.oneOfGetter($oneOfFields = ["manager"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _descriptive_name.
                             * @member {"descriptive_name"|undefined} _descriptive_name
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_descriptive_name", {
                                get: $util.oneOfGetter($oneOfFields = ["descriptive_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClient _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             */
                            Object.defineProperty(CustomerClient.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerClient instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerClient=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerClient} CustomerClient instance
                             */
                            CustomerClient.create = function create(properties) {
                                return new CustomerClient(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerClient message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerClient.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerClient} m CustomerClient message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClient.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.client_customer != null && Object.hasOwnProperty.call(m, "client_customer"))
                                    w.uint32(98).string(m.client_customer);
                                if (m.hidden != null && Object.hasOwnProperty.call(m, "hidden"))
                                    w.uint32(104).bool(m.hidden);
                                if (m.level != null && Object.hasOwnProperty.call(m, "level"))
                                    w.uint32(112).int64(m.level);
                                if (m.time_zone != null && Object.hasOwnProperty.call(m, "time_zone"))
                                    w.uint32(122).string(m.time_zone);
                                if (m.test_account != null && Object.hasOwnProperty.call(m, "test_account"))
                                    w.uint32(128).bool(m.test_account);
                                if (m.manager != null && Object.hasOwnProperty.call(m, "manager"))
                                    w.uint32(136).bool(m.manager);
                                if (m.descriptive_name != null && Object.hasOwnProperty.call(m, "descriptive_name"))
                                    w.uint32(146).string(m.descriptive_name);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(154).string(m.currency_code);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(160).int64(m.id);
                                if (m.applied_labels != null && m.applied_labels.length) {
                                    for (var i = 0; i < m.applied_labels.length; ++i)
                                        w.uint32(170).string(m.applied_labels[i]);
                                }
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(176).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerClient message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerClient.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerClient} message CustomerClient message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClient.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerClient message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerClient} CustomerClient
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClient.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerClient();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 12:
                                        m.client_customer = r.string();
                                        break;
                                    case 13:
                                        m.hidden = r.bool();
                                        break;
                                    case 14:
                                        m.level = r.int64();
                                        break;
                                    case 15:
                                        m.time_zone = r.string();
                                        break;
                                    case 16:
                                        m.test_account = r.bool();
                                        break;
                                    case 17:
                                        m.manager = r.bool();
                                        break;
                                    case 18:
                                        m.descriptive_name = r.string();
                                        break;
                                    case 19:
                                        m.currency_code = r.string();
                                        break;
                                    case 20:
                                        m.id = r.int64();
                                        break;
                                    case 21:
                                        if (!(m.applied_labels && m.applied_labels.length))
                                            m.applied_labels = [];
                                        m.applied_labels.push(r.string());
                                        break;
                                    case 22:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerClient message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerClient} CustomerClient
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClient.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerClient message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerClient.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.client_customer != null && m.hasOwnProperty("client_customer")) {
                                    p._client_customer = 1;
                                    if (!$util.isString(m.client_customer))
                                        return "client_customer: string expected";
                                }
                                if (m.hidden != null && m.hasOwnProperty("hidden")) {
                                    p._hidden = 1;
                                    if (typeof m.hidden !== "boolean")
                                        return "hidden: boolean expected";
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    p._level = 1;
                                    if (!$util.isInteger(m.level) && !(m.level && $util.isInteger(m.level.low) && $util.isInteger(m.level.high)))
                                        return "level: integer|Long expected";
                                }
                                if (m.time_zone != null && m.hasOwnProperty("time_zone")) {
                                    p._time_zone = 1;
                                    if (!$util.isString(m.time_zone))
                                        return "time_zone: string expected";
                                }
                                if (m.test_account != null && m.hasOwnProperty("test_account")) {
                                    p._test_account = 1;
                                    if (typeof m.test_account !== "boolean")
                                        return "test_account: boolean expected";
                                }
                                if (m.manager != null && m.hasOwnProperty("manager")) {
                                    p._manager = 1;
                                    if (typeof m.manager !== "boolean")
                                        return "manager: boolean expected";
                                }
                                if (m.descriptive_name != null && m.hasOwnProperty("descriptive_name")) {
                                    p._descriptive_name = 1;
                                    if (!$util.isString(m.descriptive_name))
                                        return "descriptive_name: string expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.applied_labels != null && m.hasOwnProperty("applied_labels")) {
                                    if (!Array.isArray(m.applied_labels))
                                        return "applied_labels: array expected";
                                    for (var i = 0; i < m.applied_labels.length; ++i) {
                                        if (!$util.isString(m.applied_labels[i]))
                                            return "applied_labels: string[] expected";
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerClient message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerClient} CustomerClient
                             */
                            CustomerClient.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerClient)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerClient();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.client_customer != null) {
                                    m.client_customer = String(d.client_customer);
                                }
                                if (d.hidden != null) {
                                    m.hidden = Boolean(d.hidden);
                                }
                                if (d.level != null) {
                                    if ($util.Long)
                                        (m.level = $util.Long.fromValue(d.level)).unsigned = false;
                                    else if (typeof d.level === "string")
                                        m.level = parseInt(d.level, 10);
                                    else if (typeof d.level === "number")
                                        m.level = d.level;
                                    else if (typeof d.level === "object")
                                        m.level = new $util.LongBits(d.level.low >>> 0, d.level.high >>> 0).toNumber();
                                }
                                if (d.time_zone != null) {
                                    m.time_zone = String(d.time_zone);
                                }
                                if (d.test_account != null) {
                                    m.test_account = Boolean(d.test_account);
                                }
                                if (d.manager != null) {
                                    m.manager = Boolean(d.manager);
                                }
                                if (d.descriptive_name != null) {
                                    m.descriptive_name = String(d.descriptive_name);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.applied_labels) {
                                    if (!Array.isArray(d.applied_labels))
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerClient.applied_labels: array expected");
                                    m.applied_labels = [];
                                    for (var i = 0; i < d.applied_labels.length; ++i) {
                                        m.applied_labels[i] = String(d.applied_labels[i]);
                                    }
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "CANCELED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "SUSPENDED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "CLOSED":
                                case 5:
                                    m.status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerClient message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerClient} m CustomerClient
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerClient.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.applied_labels = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.client_customer != null && m.hasOwnProperty("client_customer")) {
                                    d.client_customer = m.client_customer;
                                    if (o.oneofs)
                                        d._client_customer = "client_customer";
                                }
                                if (m.hidden != null && m.hasOwnProperty("hidden")) {
                                    d.hidden = m.hidden;
                                    if (o.oneofs)
                                        d._hidden = "hidden";
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    if (typeof m.level === "number")
                                        d.level = o.longs === String ? String(m.level) : m.level;
                                    else
                                        d.level = o.longs === String ? $util.Long.prototype.toString.call(m.level) : o.longs === Number ? new $util.LongBits(m.level.low >>> 0, m.level.high >>> 0).toNumber() : m.level;
                                    if (o.oneofs)
                                        d._level = "level";
                                }
                                if (m.time_zone != null && m.hasOwnProperty("time_zone")) {
                                    d.time_zone = m.time_zone;
                                    if (o.oneofs)
                                        d._time_zone = "time_zone";
                                }
                                if (m.test_account != null && m.hasOwnProperty("test_account")) {
                                    d.test_account = m.test_account;
                                    if (o.oneofs)
                                        d._test_account = "test_account";
                                }
                                if (m.manager != null && m.hasOwnProperty("manager")) {
                                    d.manager = m.manager;
                                    if (o.oneofs)
                                        d._manager = "manager";
                                }
                                if (m.descriptive_name != null && m.hasOwnProperty("descriptive_name")) {
                                    d.descriptive_name = m.descriptive_name;
                                    if (o.oneofs)
                                        d._descriptive_name = "descriptive_name";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.applied_labels && m.applied_labels.length) {
                                    d.applied_labels = [];
                                    for (var j = 0; j < m.applied_labels.length; ++j) {
                                        d.applied_labels[j] = m.applied_labels[j];
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomerStatusEnum.CustomerStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerClient to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerClient
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerClient.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerClient;
                        })();
    
                        resources.CustomerClientLink = (function() {
    
                            /**
                             * Properties of a CustomerClientLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerClientLink
                             * @property {string|null} [resource_name] CustomerClientLink resource_name
                             * @property {string|null} [client_customer] CustomerClientLink client_customer
                             * @property {number|null} [manager_link_id] CustomerClientLink manager_link_id
                             * @property {google.ads.googleads.v11.enums.ManagerLinkStatusEnum.ManagerLinkStatus|null} [status] CustomerClientLink status
                             * @property {boolean|null} [hidden] CustomerClientLink hidden
                             */
    
                            /**
                             * Constructs a new CustomerClientLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerClientLink.
                             * @implements ICustomerClientLink
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerClientLink=} [p] Properties to set
                             */
                            function CustomerClientLink(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerClientLink resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            CustomerClientLink.prototype.resource_name = "";
    
                            /**
                             * CustomerClientLink client_customer.
                             * @member {string|null|undefined} client_customer
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            CustomerClientLink.prototype.client_customer = null;
    
                            /**
                             * CustomerClientLink manager_link_id.
                             * @member {number|null|undefined} manager_link_id
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            CustomerClientLink.prototype.manager_link_id = null;
    
                            /**
                             * CustomerClientLink status.
                             * @member {google.ads.googleads.v11.enums.ManagerLinkStatusEnum.ManagerLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            CustomerClientLink.prototype.status = 0;
    
                            /**
                             * CustomerClientLink hidden.
                             * @member {boolean|null|undefined} hidden
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            CustomerClientLink.prototype.hidden = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerClientLink _client_customer.
                             * @member {"client_customer"|undefined} _client_customer
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            Object.defineProperty(CustomerClientLink.prototype, "_client_customer", {
                                get: $util.oneOfGetter($oneOfFields = ["client_customer"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClientLink _manager_link_id.
                             * @member {"manager_link_id"|undefined} _manager_link_id
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            Object.defineProperty(CustomerClientLink.prototype, "_manager_link_id", {
                                get: $util.oneOfGetter($oneOfFields = ["manager_link_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerClientLink _hidden.
                             * @member {"hidden"|undefined} _hidden
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             */
                            Object.defineProperty(CustomerClientLink.prototype, "_hidden", {
                                get: $util.oneOfGetter($oneOfFields = ["hidden"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerClientLink instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerClientLink=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerClientLink} CustomerClientLink instance
                             */
                            CustomerClientLink.create = function create(properties) {
                                return new CustomerClientLink(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerClientLink message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerClientLink.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerClientLink} m CustomerClientLink message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClientLink.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.client_customer != null && Object.hasOwnProperty.call(m, "client_customer"))
                                    w.uint32(58).string(m.client_customer);
                                if (m.manager_link_id != null && Object.hasOwnProperty.call(m, "manager_link_id"))
                                    w.uint32(64).int64(m.manager_link_id);
                                if (m.hidden != null && Object.hasOwnProperty.call(m, "hidden"))
                                    w.uint32(72).bool(m.hidden);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerClientLink message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerClientLink.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerClientLink} message CustomerClientLink message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClientLink.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerClientLink message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerClientLink} CustomerClientLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClientLink.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerClientLink();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.client_customer = r.string();
                                        break;
                                    case 8:
                                        m.manager_link_id = r.int64();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 9:
                                        m.hidden = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerClientLink message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerClientLink} CustomerClientLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClientLink.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerClientLink message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerClientLink.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.client_customer != null && m.hasOwnProperty("client_customer")) {
                                    p._client_customer = 1;
                                    if (!$util.isString(m.client_customer))
                                        return "client_customer: string expected";
                                }
                                if (m.manager_link_id != null && m.hasOwnProperty("manager_link_id")) {
                                    p._manager_link_id = 1;
                                    if (!$util.isInteger(m.manager_link_id) && !(m.manager_link_id && $util.isInteger(m.manager_link_id.low) && $util.isInteger(m.manager_link_id.high)))
                                        return "manager_link_id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.hidden != null && m.hasOwnProperty("hidden")) {
                                    p._hidden = 1;
                                    if (typeof m.hidden !== "boolean")
                                        return "hidden: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerClientLink message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerClientLink} CustomerClientLink
                             */
                            CustomerClientLink.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerClientLink)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerClientLink();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.client_customer != null) {
                                    m.client_customer = String(d.client_customer);
                                }
                                if (d.manager_link_id != null) {
                                    if ($util.Long)
                                        (m.manager_link_id = $util.Long.fromValue(d.manager_link_id)).unsigned = false;
                                    else if (typeof d.manager_link_id === "string")
                                        m.manager_link_id = parseInt(d.manager_link_id, 10);
                                    else if (typeof d.manager_link_id === "number")
                                        m.manager_link_id = d.manager_link_id;
                                    else if (typeof d.manager_link_id === "object")
                                        m.manager_link_id = new $util.LongBits(d.manager_link_id.low >>> 0, d.manager_link_id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ACTIVE":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "INACTIVE":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PENDING":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "REFUSED":
                                case 5:
                                    m.status = 5;
                                    break;
                                case "CANCELED":
                                case 6:
                                    m.status = 6;
                                    break;
                                }
                                if (d.hidden != null) {
                                    m.hidden = Boolean(d.hidden);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerClientLink message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerClientLink} m CustomerClientLink
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerClientLink.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ManagerLinkStatusEnum.ManagerLinkStatus[m.status] : m.status;
                                }
                                if (m.client_customer != null && m.hasOwnProperty("client_customer")) {
                                    d.client_customer = m.client_customer;
                                    if (o.oneofs)
                                        d._client_customer = "client_customer";
                                }
                                if (m.manager_link_id != null && m.hasOwnProperty("manager_link_id")) {
                                    if (typeof m.manager_link_id === "number")
                                        d.manager_link_id = o.longs === String ? String(m.manager_link_id) : m.manager_link_id;
                                    else
                                        d.manager_link_id = o.longs === String ? $util.Long.prototype.toString.call(m.manager_link_id) : o.longs === Number ? new $util.LongBits(m.manager_link_id.low >>> 0, m.manager_link_id.high >>> 0).toNumber() : m.manager_link_id;
                                    if (o.oneofs)
                                        d._manager_link_id = "manager_link_id";
                                }
                                if (m.hidden != null && m.hasOwnProperty("hidden")) {
                                    d.hidden = m.hidden;
                                    if (o.oneofs)
                                        d._hidden = "hidden";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerClientLink to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerClientLink
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerClientLink.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerClientLink;
                        })();
    
                        resources.CustomerConversionGoal = (function() {
    
                            /**
                             * Properties of a CustomerConversionGoal.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerConversionGoal
                             * @property {string|null} [resource_name] CustomerConversionGoal resource_name
                             * @property {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory|null} [category] CustomerConversionGoal category
                             * @property {google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin|null} [origin] CustomerConversionGoal origin
                             * @property {boolean|null} [biddable] CustomerConversionGoal biddable
                             */
    
                            /**
                             * Constructs a new CustomerConversionGoal.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerConversionGoal.
                             * @implements ICustomerConversionGoal
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerConversionGoal=} [p] Properties to set
                             */
                            function CustomerConversionGoal(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerConversionGoal resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @instance
                             */
                            CustomerConversionGoal.prototype.resource_name = "";
    
                            /**
                             * CustomerConversionGoal category.
                             * @member {google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory} category
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @instance
                             */
                            CustomerConversionGoal.prototype.category = 0;
    
                            /**
                             * CustomerConversionGoal origin.
                             * @member {google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin} origin
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @instance
                             */
                            CustomerConversionGoal.prototype.origin = 0;
    
                            /**
                             * CustomerConversionGoal biddable.
                             * @member {boolean} biddable
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @instance
                             */
                            CustomerConversionGoal.prototype.biddable = false;
    
                            /**
                             * Creates a new CustomerConversionGoal instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerConversionGoal=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerConversionGoal} CustomerConversionGoal instance
                             */
                            CustomerConversionGoal.create = function create(properties) {
                                return new CustomerConversionGoal(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerConversionGoal message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerConversionGoal.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerConversionGoal} m CustomerConversionGoal message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerConversionGoal.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(16).int32(m.category);
                                if (m.origin != null && Object.hasOwnProperty.call(m, "origin"))
                                    w.uint32(24).int32(m.origin);
                                if (m.biddable != null && Object.hasOwnProperty.call(m, "biddable"))
                                    w.uint32(32).bool(m.biddable);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerConversionGoal message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerConversionGoal.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerConversionGoal} message CustomerConversionGoal message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerConversionGoal.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerConversionGoal message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerConversionGoal} CustomerConversionGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerConversionGoal.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerConversionGoal();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.category = r.int32();
                                        break;
                                    case 3:
                                        m.origin = r.int32();
                                        break;
                                    case 4:
                                        m.biddable = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerConversionGoal message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerConversionGoal} CustomerConversionGoal
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerConversionGoal.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerConversionGoal message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerConversionGoal.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    switch (m.category) {
                                    default:
                                        return "category: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                        break;
                                    }
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    switch (m.origin) {
                                    default:
                                        return "origin: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.biddable != null && m.hasOwnProperty("biddable")) {
                                    if (typeof m.biddable !== "boolean")
                                        return "biddable: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerConversionGoal message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerConversionGoal} CustomerConversionGoal
                             */
                            CustomerConversionGoal.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerConversionGoal)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerConversionGoal();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.category) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.category = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.category = 1;
                                    break;
                                case "DEFAULT":
                                case 2:
                                    m.category = 2;
                                    break;
                                case "PAGE_VIEW":
                                case 3:
                                    m.category = 3;
                                    break;
                                case "PURCHASE":
                                case 4:
                                    m.category = 4;
                                    break;
                                case "SIGNUP":
                                case 5:
                                    m.category = 5;
                                    break;
                                case "DOWNLOAD":
                                case 7:
                                    m.category = 7;
                                    break;
                                case "ADD_TO_CART":
                                case 8:
                                    m.category = 8;
                                    break;
                                case "BEGIN_CHECKOUT":
                                case 9:
                                    m.category = 9;
                                    break;
                                case "SUBSCRIBE_PAID":
                                case 10:
                                    m.category = 10;
                                    break;
                                case "PHONE_CALL_LEAD":
                                case 11:
                                    m.category = 11;
                                    break;
                                case "IMPORTED_LEAD":
                                case 12:
                                    m.category = 12;
                                    break;
                                case "SUBMIT_LEAD_FORM":
                                case 13:
                                    m.category = 13;
                                    break;
                                case "BOOK_APPOINTMENT":
                                case 14:
                                    m.category = 14;
                                    break;
                                case "REQUEST_QUOTE":
                                case 15:
                                    m.category = 15;
                                    break;
                                case "GET_DIRECTIONS":
                                case 16:
                                    m.category = 16;
                                    break;
                                case "OUTBOUND_CLICK":
                                case 17:
                                    m.category = 17;
                                    break;
                                case "CONTACT":
                                case 18:
                                    m.category = 18;
                                    break;
                                case "ENGAGEMENT":
                                case 19:
                                    m.category = 19;
                                    break;
                                case "STORE_VISIT":
                                case 20:
                                    m.category = 20;
                                    break;
                                case "STORE_SALE":
                                case 21:
                                    m.category = 21;
                                    break;
                                case "QUALIFIED_LEAD":
                                case 22:
                                    m.category = 22;
                                    break;
                                case "CONVERTED_LEAD":
                                case 23:
                                    m.category = 23;
                                    break;
                                }
                                switch (d.origin) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.origin = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.origin = 1;
                                    break;
                                case "WEBSITE":
                                case 2:
                                    m.origin = 2;
                                    break;
                                case "GOOGLE_HOSTED":
                                case 3:
                                    m.origin = 3;
                                    break;
                                case "APP":
                                case 4:
                                    m.origin = 4;
                                    break;
                                case "CALL_FROM_ADS":
                                case 5:
                                    m.origin = 5;
                                    break;
                                case "STORE":
                                case 6:
                                    m.origin = 6;
                                    break;
                                case "YOUTUBE_HOSTED":
                                case 7:
                                    m.origin = 7;
                                    break;
                                }
                                if (d.biddable != null) {
                                    m.biddable = Boolean(d.biddable);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerConversionGoal message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerConversionGoal} m CustomerConversionGoal
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerConversionGoal.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.category = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.origin = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.biddable = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionActionCategoryEnum.ConversionActionCategory[m.category] : m.category;
                                }
                                if (m.origin != null && m.hasOwnProperty("origin")) {
                                    d.origin = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionOriginEnum.ConversionOrigin[m.origin] : m.origin;
                                }
                                if (m.biddable != null && m.hasOwnProperty("biddable")) {
                                    d.biddable = m.biddable;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerConversionGoal to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerConversionGoal
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerConversionGoal.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerConversionGoal;
                        })();
    
                        resources.CustomerCustomizer = (function() {
    
                            /**
                             * Properties of a CustomerCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerCustomizer
                             * @property {string|null} [resource_name] CustomerCustomizer resource_name
                             * @property {string|null} [customizer_attribute] CustomerCustomizer customizer_attribute
                             * @property {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus|null} [status] CustomerCustomizer status
                             * @property {google.ads.googleads.v11.common.ICustomizerValue|null} [value] CustomerCustomizer value
                             */
    
                            /**
                             * Constructs a new CustomerCustomizer.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerCustomizer.
                             * @implements ICustomerCustomizer
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerCustomizer=} [p] Properties to set
                             */
                            function CustomerCustomizer(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerCustomizer resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @instance
                             */
                            CustomerCustomizer.prototype.resource_name = "";
    
                            /**
                             * CustomerCustomizer customizer_attribute.
                             * @member {string} customizer_attribute
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @instance
                             */
                            CustomerCustomizer.prototype.customizer_attribute = "";
    
                            /**
                             * CustomerCustomizer status.
                             * @member {google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @instance
                             */
                            CustomerCustomizer.prototype.status = 0;
    
                            /**
                             * CustomerCustomizer value.
                             * @member {google.ads.googleads.v11.common.ICustomizerValue|null|undefined} value
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @instance
                             */
                            CustomerCustomizer.prototype.value = null;
    
                            /**
                             * Creates a new CustomerCustomizer instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerCustomizer=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerCustomizer} CustomerCustomizer instance
                             */
                            CustomerCustomizer.create = function create(properties) {
                                return new CustomerCustomizer(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerCustomizer message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerCustomizer.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerCustomizer} m CustomerCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerCustomizer.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customizer_attribute != null && Object.hasOwnProperty.call(m, "customizer_attribute"))
                                    w.uint32(18).string(m.customizer_attribute);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(24).int32(m.status);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    $root.google.ads.googleads.v11.common.CustomizerValue.encode(m.value, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerCustomizer message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerCustomizer.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerCustomizer} message CustomerCustomizer message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerCustomizer.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerCustomizer message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerCustomizer} CustomerCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerCustomizer.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerCustomizer();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customizer_attribute = r.string();
                                        break;
                                    case 3:
                                        m.status = r.int32();
                                        break;
                                    case 4:
                                        m.value = $root.google.ads.googleads.v11.common.CustomizerValue.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerCustomizer message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerCustomizer} CustomerCustomizer
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerCustomizer.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerCustomizer message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerCustomizer.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    if (!$util.isString(m.customizer_attribute))
                                        return "customizer_attribute: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomizerValue.verify(m.value);
                                        if (e)
                                            return "value." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerCustomizer message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerCustomizer} CustomerCustomizer
                             */
                            CustomerCustomizer.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerCustomizer)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerCustomizer();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customizer_attribute != null) {
                                    m.customizer_attribute = String(d.customizer_attribute);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.value != null) {
                                    if (typeof d.value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerCustomizer.value: object expected");
                                    m.value = $root.google.ads.googleads.v11.common.CustomizerValue.fromObject(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerCustomizer message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerCustomizer} m CustomerCustomizer
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerCustomizer.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customizer_attribute = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.value = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    d.customizer_attribute = m.customizer_attribute;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomizerValueStatusEnum.CustomizerValueStatus[m.status] : m.status;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = $root.google.ads.googleads.v11.common.CustomizerValue.toObject(m.value, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerCustomizer to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerCustomizer
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerCustomizer.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerCustomizer;
                        })();
    
                        resources.CustomerExtensionSetting = (function() {
    
                            /**
                             * Properties of a CustomerExtensionSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerExtensionSetting
                             * @property {string|null} [resource_name] CustomerExtensionSetting resource_name
                             * @property {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType|null} [extension_type] CustomerExtensionSetting extension_type
                             * @property {Array.<string>|null} [extension_feed_items] CustomerExtensionSetting extension_feed_items
                             * @property {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice|null} [device] CustomerExtensionSetting device
                             */
    
                            /**
                             * Constructs a new CustomerExtensionSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerExtensionSetting.
                             * @implements ICustomerExtensionSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerExtensionSetting=} [p] Properties to set
                             */
                            function CustomerExtensionSetting(p) {
                                this.extension_feed_items = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerExtensionSetting resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @instance
                             */
                            CustomerExtensionSetting.prototype.resource_name = "";
    
                            /**
                             * CustomerExtensionSetting extension_type.
                             * @member {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType} extension_type
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @instance
                             */
                            CustomerExtensionSetting.prototype.extension_type = 0;
    
                            /**
                             * CustomerExtensionSetting extension_feed_items.
                             * @member {Array.<string>} extension_feed_items
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @instance
                             */
                            CustomerExtensionSetting.prototype.extension_feed_items = $util.emptyArray;
    
                            /**
                             * CustomerExtensionSetting device.
                             * @member {google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice} device
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @instance
                             */
                            CustomerExtensionSetting.prototype.device = 0;
    
                            /**
                             * Creates a new CustomerExtensionSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerExtensionSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerExtensionSetting} CustomerExtensionSetting instance
                             */
                            CustomerExtensionSetting.create = function create(properties) {
                                return new CustomerExtensionSetting(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerExtensionSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerExtensionSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerExtensionSetting} m CustomerExtensionSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerExtensionSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.extension_type != null && Object.hasOwnProperty.call(m, "extension_type"))
                                    w.uint32(16).int32(m.extension_type);
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    w.uint32(32).int32(m.device);
                                if (m.extension_feed_items != null && m.extension_feed_items.length) {
                                    for (var i = 0; i < m.extension_feed_items.length; ++i)
                                        w.uint32(42).string(m.extension_feed_items[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerExtensionSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerExtensionSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerExtensionSetting} message CustomerExtensionSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerExtensionSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerExtensionSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerExtensionSetting} CustomerExtensionSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerExtensionSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerExtensionSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.extension_type = r.int32();
                                        break;
                                    case 5:
                                        if (!(m.extension_feed_items && m.extension_feed_items.length))
                                            m.extension_feed_items = [];
                                        m.extension_feed_items.push(r.string());
                                        break;
                                    case 4:
                                        m.device = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerExtensionSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerExtensionSetting} CustomerExtensionSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerExtensionSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerExtensionSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerExtensionSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    switch (m.extension_type) {
                                    default:
                                        return "extension_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 15:
                                    case 16:
                                        break;
                                    }
                                }
                                if (m.extension_feed_items != null && m.hasOwnProperty("extension_feed_items")) {
                                    if (!Array.isArray(m.extension_feed_items))
                                        return "extension_feed_items: array expected";
                                    for (var i = 0; i < m.extension_feed_items.length; ++i) {
                                        if (!$util.isString(m.extension_feed_items[i]))
                                            return "extension_feed_items: string[] expected";
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    switch (m.device) {
                                    default:
                                        return "device: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerExtensionSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerExtensionSetting} CustomerExtensionSetting
                             */
                            CustomerExtensionSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerExtensionSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerExtensionSetting();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.extension_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.extension_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.extension_type = 1;
                                    break;
                                case "NONE":
                                case 2:
                                    m.extension_type = 2;
                                    break;
                                case "APP":
                                case 3:
                                    m.extension_type = 3;
                                    break;
                                case "CALL":
                                case 4:
                                    m.extension_type = 4;
                                    break;
                                case "CALLOUT":
                                case 5:
                                    m.extension_type = 5;
                                    break;
                                case "MESSAGE":
                                case 6:
                                    m.extension_type = 6;
                                    break;
                                case "PRICE":
                                case 7:
                                    m.extension_type = 7;
                                    break;
                                case "PROMOTION":
                                case 8:
                                    m.extension_type = 8;
                                    break;
                                case "SITELINK":
                                case 10:
                                    m.extension_type = 10;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 11:
                                    m.extension_type = 11;
                                    break;
                                case "LOCATION":
                                case 12:
                                    m.extension_type = 12;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 13:
                                    m.extension_type = 13;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.extension_type = 15;
                                    break;
                                case "IMAGE":
                                case 16:
                                    m.extension_type = 16;
                                    break;
                                }
                                if (d.extension_feed_items) {
                                    if (!Array.isArray(d.extension_feed_items))
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerExtensionSetting.extension_feed_items: array expected");
                                    m.extension_feed_items = [];
                                    for (var i = 0; i < d.extension_feed_items.length; ++i) {
                                        m.extension_feed_items[i] = String(d.extension_feed_items[i]);
                                    }
                                }
                                switch (d.device) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device = 2;
                                    break;
                                case "DESKTOP":
                                case 3:
                                    m.device = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerExtensionSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerExtensionSetting} m CustomerExtensionSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerExtensionSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.extension_feed_items = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.extension_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.device = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    d.extension_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType[m.extension_type] : m.extension_type;
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.ExtensionSettingDeviceEnum.ExtensionSettingDevice[m.device] : m.device;
                                }
                                if (m.extension_feed_items && m.extension_feed_items.length) {
                                    d.extension_feed_items = [];
                                    for (var j = 0; j < m.extension_feed_items.length; ++j) {
                                        d.extension_feed_items[j] = m.extension_feed_items[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerExtensionSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerExtensionSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerExtensionSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerExtensionSetting;
                        })();
    
                        resources.CustomerFeed = (function() {
    
                            /**
                             * Properties of a CustomerFeed.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerFeed
                             * @property {string|null} [resource_name] CustomerFeed resource_name
                             * @property {string|null} [feed] CustomerFeed feed
                             * @property {Array.<google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType>|null} [placeholder_types] CustomerFeed placeholder_types
                             * @property {google.ads.googleads.v11.common.IMatchingFunction|null} [matching_function] CustomerFeed matching_function
                             * @property {google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus|null} [status] CustomerFeed status
                             */
    
                            /**
                             * Constructs a new CustomerFeed.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerFeed.
                             * @implements ICustomerFeed
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerFeed=} [p] Properties to set
                             */
                            function CustomerFeed(p) {
                                this.placeholder_types = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerFeed resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @instance
                             */
                            CustomerFeed.prototype.resource_name = "";
    
                            /**
                             * CustomerFeed feed.
                             * @member {string|null|undefined} feed
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @instance
                             */
                            CustomerFeed.prototype.feed = null;
    
                            /**
                             * CustomerFeed placeholder_types.
                             * @member {Array.<google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType>} placeholder_types
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @instance
                             */
                            CustomerFeed.prototype.placeholder_types = $util.emptyArray;
    
                            /**
                             * CustomerFeed matching_function.
                             * @member {google.ads.googleads.v11.common.IMatchingFunction|null|undefined} matching_function
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @instance
                             */
                            CustomerFeed.prototype.matching_function = null;
    
                            /**
                             * CustomerFeed status.
                             * @member {google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @instance
                             */
                            CustomerFeed.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerFeed _feed.
                             * @member {"feed"|undefined} _feed
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @instance
                             */
                            Object.defineProperty(CustomerFeed.prototype, "_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerFeed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerFeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerFeed} CustomerFeed instance
                             */
                            CustomerFeed.create = function create(properties) {
                                return new CustomerFeed(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerFeed message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerFeed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerFeed} m CustomerFeed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerFeed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.placeholder_types != null && m.placeholder_types.length) {
                                    w.uint32(26).fork();
                                    for (var i = 0; i < m.placeholder_types.length; ++i)
                                        w.int32(m.placeholder_types[i]);
                                    w.ldelim();
                                }
                                if (m.matching_function != null && Object.hasOwnProperty.call(m, "matching_function"))
                                    $root.google.ads.googleads.v11.common.MatchingFunction.encode(m.matching_function, w.uint32(34).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(50).string(m.feed);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerFeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerFeed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerFeed} message CustomerFeed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerFeed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerFeed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerFeed} CustomerFeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerFeed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerFeed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.feed = r.string();
                                        break;
                                    case 3:
                                        if (!(m.placeholder_types && m.placeholder_types.length))
                                            m.placeholder_types = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.placeholder_types.push(r.int32());
                                        } else
                                            m.placeholder_types.push(r.int32());
                                        break;
                                    case 4:
                                        m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerFeed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerFeed} CustomerFeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerFeed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerFeed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerFeed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    p._feed = 1;
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.placeholder_types != null && m.hasOwnProperty("placeholder_types")) {
                                    if (!Array.isArray(m.placeholder_types))
                                        return "placeholder_types: array expected";
                                    for (var i = 0; i < m.placeholder_types.length; ++i) {
                                        switch (m.placeholder_types[i]) {
                                        default:
                                            return "placeholder_types: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 17:
                                        case 18:
                                        case 19:
                                        case 20:
                                        case 21:
                                            break;
                                        }
                                    }
                                }
                                if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MatchingFunction.verify(m.matching_function);
                                        if (e)
                                            return "matching_function." + e;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerFeed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerFeed} CustomerFeed
                             */
                            CustomerFeed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerFeed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerFeed();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.placeholder_types) {
                                    if (!Array.isArray(d.placeholder_types))
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerFeed.placeholder_types: array expected");
                                    m.placeholder_types = [];
                                    for (var i = 0; i < d.placeholder_types.length; ++i) {
                                        switch (d.placeholder_types[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.placeholder_types[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.placeholder_types[i] = 1;
                                            break;
                                        case "SITELINK":
                                        case 2:
                                            m.placeholder_types[i] = 2;
                                            break;
                                        case "CALL":
                                        case 3:
                                            m.placeholder_types[i] = 3;
                                            break;
                                        case "APP":
                                        case 4:
                                            m.placeholder_types[i] = 4;
                                            break;
                                        case "LOCATION":
                                        case 5:
                                            m.placeholder_types[i] = 5;
                                            break;
                                        case "AFFILIATE_LOCATION":
                                        case 6:
                                            m.placeholder_types[i] = 6;
                                            break;
                                        case "CALLOUT":
                                        case 7:
                                            m.placeholder_types[i] = 7;
                                            break;
                                        case "STRUCTURED_SNIPPET":
                                        case 8:
                                            m.placeholder_types[i] = 8;
                                            break;
                                        case "MESSAGE":
                                        case 9:
                                            m.placeholder_types[i] = 9;
                                            break;
                                        case "PRICE":
                                        case 10:
                                            m.placeholder_types[i] = 10;
                                            break;
                                        case "PROMOTION":
                                        case 11:
                                            m.placeholder_types[i] = 11;
                                            break;
                                        case "AD_CUSTOMIZER":
                                        case 12:
                                            m.placeholder_types[i] = 12;
                                            break;
                                        case "DYNAMIC_EDUCATION":
                                        case 13:
                                            m.placeholder_types[i] = 13;
                                            break;
                                        case "DYNAMIC_FLIGHT":
                                        case 14:
                                            m.placeholder_types[i] = 14;
                                            break;
                                        case "DYNAMIC_CUSTOM":
                                        case 15:
                                            m.placeholder_types[i] = 15;
                                            break;
                                        case "DYNAMIC_HOTEL":
                                        case 16:
                                            m.placeholder_types[i] = 16;
                                            break;
                                        case "DYNAMIC_REAL_ESTATE":
                                        case 17:
                                            m.placeholder_types[i] = 17;
                                            break;
                                        case "DYNAMIC_TRAVEL":
                                        case 18:
                                            m.placeholder_types[i] = 18;
                                            break;
                                        case "DYNAMIC_LOCAL":
                                        case 19:
                                            m.placeholder_types[i] = 19;
                                            break;
                                        case "DYNAMIC_JOB":
                                        case 20:
                                            m.placeholder_types[i] = 20;
                                            break;
                                        case "IMAGE":
                                        case 21:
                                            m.placeholder_types[i] = 21;
                                            break;
                                        }
                                    }
                                }
                                if (d.matching_function != null) {
                                    if (typeof d.matching_function !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerFeed.matching_function: object expected");
                                    m.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.fromObject(d.matching_function);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerFeed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerFeed} m CustomerFeed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerFeed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.placeholder_types = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.matching_function = null;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.placeholder_types && m.placeholder_types.length) {
                                    d.placeholder_types = [];
                                    for (var j = 0; j < m.placeholder_types.length; ++j) {
                                        d.placeholder_types[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType[m.placeholder_types[j]] : m.placeholder_types[j];
                                    }
                                }
                                if (m.matching_function != null && m.hasOwnProperty("matching_function")) {
                                    d.matching_function = $root.google.ads.googleads.v11.common.MatchingFunction.toObject(m.matching_function, o);
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedLinkStatusEnum.FeedLinkStatus[m.status] : m.status;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                    if (o.oneofs)
                                        d._feed = "feed";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerFeed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerFeed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerFeed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerFeed;
                        })();
    
                        resources.CustomerLabel = (function() {
    
                            /**
                             * Properties of a CustomerLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerLabel
                             * @property {string|null} [resource_name] CustomerLabel resource_name
                             * @property {string|null} [customer] CustomerLabel customer
                             * @property {string|null} [label] CustomerLabel label
                             */
    
                            /**
                             * Constructs a new CustomerLabel.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerLabel.
                             * @implements ICustomerLabel
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerLabel=} [p] Properties to set
                             */
                            function CustomerLabel(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerLabel resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @instance
                             */
                            CustomerLabel.prototype.resource_name = "";
    
                            /**
                             * CustomerLabel customer.
                             * @member {string|null|undefined} customer
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @instance
                             */
                            CustomerLabel.prototype.customer = null;
    
                            /**
                             * CustomerLabel label.
                             * @member {string|null|undefined} label
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @instance
                             */
                            CustomerLabel.prototype.label = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerLabel _customer.
                             * @member {"customer"|undefined} _customer
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @instance
                             */
                            Object.defineProperty(CustomerLabel.prototype, "_customer", {
                                get: $util.oneOfGetter($oneOfFields = ["customer"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerLabel _label.
                             * @member {"label"|undefined} _label
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @instance
                             */
                            Object.defineProperty(CustomerLabel.prototype, "_label", {
                                get: $util.oneOfGetter($oneOfFields = ["label"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerLabel instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerLabel=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerLabel} CustomerLabel instance
                             */
                            CustomerLabel.create = function create(properties) {
                                return new CustomerLabel(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerLabel message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerLabel.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerLabel} m CustomerLabel message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerLabel.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customer != null && Object.hasOwnProperty.call(m, "customer"))
                                    w.uint32(34).string(m.customer);
                                if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                                    w.uint32(42).string(m.label);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerLabel message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerLabel.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerLabel} message CustomerLabel message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerLabel.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerLabel message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerLabel} CustomerLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerLabel.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerLabel();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.customer = r.string();
                                        break;
                                    case 5:
                                        m.label = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerLabel message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerLabel} CustomerLabel
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerLabel.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerLabel message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerLabel.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    p._customer = 1;
                                    if (!$util.isString(m.customer))
                                        return "customer: string expected";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    p._label = 1;
                                    if (!$util.isString(m.label))
                                        return "label: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerLabel message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerLabel} CustomerLabel
                             */
                            CustomerLabel.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerLabel)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerLabel();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customer != null) {
                                    m.customer = String(d.customer);
                                }
                                if (d.label != null) {
                                    m.label = String(d.label);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerLabel message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerLabel} m CustomerLabel
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerLabel.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    d.customer = m.customer;
                                    if (o.oneofs)
                                        d._customer = "customer";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    d.label = m.label;
                                    if (o.oneofs)
                                        d._label = "label";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerLabel to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerLabel
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerLabel.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerLabel;
                        })();
    
                        resources.CustomerManagerLink = (function() {
    
                            /**
                             * Properties of a CustomerManagerLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerManagerLink
                             * @property {string|null} [resource_name] CustomerManagerLink resource_name
                             * @property {string|null} [manager_customer] CustomerManagerLink manager_customer
                             * @property {number|null} [manager_link_id] CustomerManagerLink manager_link_id
                             * @property {google.ads.googleads.v11.enums.ManagerLinkStatusEnum.ManagerLinkStatus|null} [status] CustomerManagerLink status
                             */
    
                            /**
                             * Constructs a new CustomerManagerLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerManagerLink.
                             * @implements ICustomerManagerLink
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerManagerLink=} [p] Properties to set
                             */
                            function CustomerManagerLink(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerManagerLink resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @instance
                             */
                            CustomerManagerLink.prototype.resource_name = "";
    
                            /**
                             * CustomerManagerLink manager_customer.
                             * @member {string|null|undefined} manager_customer
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @instance
                             */
                            CustomerManagerLink.prototype.manager_customer = null;
    
                            /**
                             * CustomerManagerLink manager_link_id.
                             * @member {number|null|undefined} manager_link_id
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @instance
                             */
                            CustomerManagerLink.prototype.manager_link_id = null;
    
                            /**
                             * CustomerManagerLink status.
                             * @member {google.ads.googleads.v11.enums.ManagerLinkStatusEnum.ManagerLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @instance
                             */
                            CustomerManagerLink.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerManagerLink _manager_customer.
                             * @member {"manager_customer"|undefined} _manager_customer
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @instance
                             */
                            Object.defineProperty(CustomerManagerLink.prototype, "_manager_customer", {
                                get: $util.oneOfGetter($oneOfFields = ["manager_customer"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerManagerLink _manager_link_id.
                             * @member {"manager_link_id"|undefined} _manager_link_id
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @instance
                             */
                            Object.defineProperty(CustomerManagerLink.prototype, "_manager_link_id", {
                                get: $util.oneOfGetter($oneOfFields = ["manager_link_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerManagerLink instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerManagerLink=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerManagerLink} CustomerManagerLink instance
                             */
                            CustomerManagerLink.create = function create(properties) {
                                return new CustomerManagerLink(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerManagerLink message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerManagerLink.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerManagerLink} m CustomerManagerLink message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerManagerLink.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.manager_customer != null && Object.hasOwnProperty.call(m, "manager_customer"))
                                    w.uint32(50).string(m.manager_customer);
                                if (m.manager_link_id != null && Object.hasOwnProperty.call(m, "manager_link_id"))
                                    w.uint32(56).int64(m.manager_link_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerManagerLink message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerManagerLink.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerManagerLink} message CustomerManagerLink message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerManagerLink.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerManagerLink message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerManagerLink} CustomerManagerLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerManagerLink.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerManagerLink();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.manager_customer = r.string();
                                        break;
                                    case 7:
                                        m.manager_link_id = r.int64();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerManagerLink message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerManagerLink} CustomerManagerLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerManagerLink.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerManagerLink message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerManagerLink.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.manager_customer != null && m.hasOwnProperty("manager_customer")) {
                                    p._manager_customer = 1;
                                    if (!$util.isString(m.manager_customer))
                                        return "manager_customer: string expected";
                                }
                                if (m.manager_link_id != null && m.hasOwnProperty("manager_link_id")) {
                                    p._manager_link_id = 1;
                                    if (!$util.isInteger(m.manager_link_id) && !(m.manager_link_id && $util.isInteger(m.manager_link_id.low) && $util.isInteger(m.manager_link_id.high)))
                                        return "manager_link_id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerManagerLink message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerManagerLink} CustomerManagerLink
                             */
                            CustomerManagerLink.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerManagerLink)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerManagerLink();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.manager_customer != null) {
                                    m.manager_customer = String(d.manager_customer);
                                }
                                if (d.manager_link_id != null) {
                                    if ($util.Long)
                                        (m.manager_link_id = $util.Long.fromValue(d.manager_link_id)).unsigned = false;
                                    else if (typeof d.manager_link_id === "string")
                                        m.manager_link_id = parseInt(d.manager_link_id, 10);
                                    else if (typeof d.manager_link_id === "number")
                                        m.manager_link_id = d.manager_link_id;
                                    else if (typeof d.manager_link_id === "object")
                                        m.manager_link_id = new $util.LongBits(d.manager_link_id.low >>> 0, d.manager_link_id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ACTIVE":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "INACTIVE":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "PENDING":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "REFUSED":
                                case 5:
                                    m.status = 5;
                                    break;
                                case "CANCELED":
                                case 6:
                                    m.status = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerManagerLink message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerManagerLink} m CustomerManagerLink
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerManagerLink.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ManagerLinkStatusEnum.ManagerLinkStatus[m.status] : m.status;
                                }
                                if (m.manager_customer != null && m.hasOwnProperty("manager_customer")) {
                                    d.manager_customer = m.manager_customer;
                                    if (o.oneofs)
                                        d._manager_customer = "manager_customer";
                                }
                                if (m.manager_link_id != null && m.hasOwnProperty("manager_link_id")) {
                                    if (typeof m.manager_link_id === "number")
                                        d.manager_link_id = o.longs === String ? String(m.manager_link_id) : m.manager_link_id;
                                    else
                                        d.manager_link_id = o.longs === String ? $util.Long.prototype.toString.call(m.manager_link_id) : o.longs === Number ? new $util.LongBits(m.manager_link_id.low >>> 0, m.manager_link_id.high >>> 0).toNumber() : m.manager_link_id;
                                    if (o.oneofs)
                                        d._manager_link_id = "manager_link_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerManagerLink to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerManagerLink
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerManagerLink.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerManagerLink;
                        })();
    
                        resources.CustomerNegativeCriterion = (function() {
    
                            /**
                             * Properties of a CustomerNegativeCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerNegativeCriterion
                             * @property {string|null} [resource_name] CustomerNegativeCriterion resource_name
                             * @property {number|null} [id] CustomerNegativeCriterion id
                             * @property {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType|null} [type] CustomerNegativeCriterion type
                             * @property {google.ads.googleads.v11.common.IContentLabelInfo|null} [content_label] CustomerNegativeCriterion content_label
                             * @property {google.ads.googleads.v11.common.IMobileApplicationInfo|null} [mobile_application] CustomerNegativeCriterion mobile_application
                             * @property {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null} [mobile_app_category] CustomerNegativeCriterion mobile_app_category
                             * @property {google.ads.googleads.v11.common.IPlacementInfo|null} [placement] CustomerNegativeCriterion placement
                             * @property {google.ads.googleads.v11.common.IYouTubeVideoInfo|null} [youtube_video] CustomerNegativeCriterion youtube_video
                             * @property {google.ads.googleads.v11.common.IYouTubeChannelInfo|null} [youtube_channel] CustomerNegativeCriterion youtube_channel
                             */
    
                            /**
                             * Constructs a new CustomerNegativeCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerNegativeCriterion.
                             * @implements ICustomerNegativeCriterion
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerNegativeCriterion=} [p] Properties to set
                             */
                            function CustomerNegativeCriterion(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerNegativeCriterion resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.resource_name = "";
    
                            /**
                             * CustomerNegativeCriterion id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.id = null;
    
                            /**
                             * CustomerNegativeCriterion type.
                             * @member {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType} type
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.type = 0;
    
                            /**
                             * CustomerNegativeCriterion content_label.
                             * @member {google.ads.googleads.v11.common.IContentLabelInfo|null|undefined} content_label
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.content_label = null;
    
                            /**
                             * CustomerNegativeCriterion mobile_application.
                             * @member {google.ads.googleads.v11.common.IMobileApplicationInfo|null|undefined} mobile_application
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.mobile_application = null;
    
                            /**
                             * CustomerNegativeCriterion mobile_app_category.
                             * @member {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null|undefined} mobile_app_category
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.mobile_app_category = null;
    
                            /**
                             * CustomerNegativeCriterion placement.
                             * @member {google.ads.googleads.v11.common.IPlacementInfo|null|undefined} placement
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.placement = null;
    
                            /**
                             * CustomerNegativeCriterion youtube_video.
                             * @member {google.ads.googleads.v11.common.IYouTubeVideoInfo|null|undefined} youtube_video
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.youtube_video = null;
    
                            /**
                             * CustomerNegativeCriterion youtube_channel.
                             * @member {google.ads.googleads.v11.common.IYouTubeChannelInfo|null|undefined} youtube_channel
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            CustomerNegativeCriterion.prototype.youtube_channel = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerNegativeCriterion _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            Object.defineProperty(CustomerNegativeCriterion.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerNegativeCriterion criterion.
                             * @member {"content_label"|"mobile_application"|"mobile_app_category"|"placement"|"youtube_video"|"youtube_channel"|undefined} criterion
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             */
                            Object.defineProperty(CustomerNegativeCriterion.prototype, "criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["content_label", "mobile_application", "mobile_app_category", "placement", "youtube_video", "youtube_channel"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerNegativeCriterion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerNegativeCriterion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerNegativeCriterion} CustomerNegativeCriterion instance
                             */
                            CustomerNegativeCriterion.create = function create(properties) {
                                return new CustomerNegativeCriterion(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerNegativeCriterion message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerNegativeCriterion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerNegativeCriterion} m CustomerNegativeCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerNegativeCriterion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.content_label != null && Object.hasOwnProperty.call(m, "content_label"))
                                    $root.google.ads.googleads.v11.common.ContentLabelInfo.encode(m.content_label, w.uint32(34).fork()).ldelim();
                                if (m.mobile_application != null && Object.hasOwnProperty.call(m, "mobile_application"))
                                    $root.google.ads.googleads.v11.common.MobileApplicationInfo.encode(m.mobile_application, w.uint32(42).fork()).ldelim();
                                if (m.mobile_app_category != null && Object.hasOwnProperty.call(m, "mobile_app_category"))
                                    $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.encode(m.mobile_app_category, w.uint32(50).fork()).ldelim();
                                if (m.placement != null && Object.hasOwnProperty.call(m, "placement"))
                                    $root.google.ads.googleads.v11.common.PlacementInfo.encode(m.placement, w.uint32(58).fork()).ldelim();
                                if (m.youtube_video != null && Object.hasOwnProperty.call(m, "youtube_video"))
                                    $root.google.ads.googleads.v11.common.YouTubeVideoInfo.encode(m.youtube_video, w.uint32(66).fork()).ldelim();
                                if (m.youtube_channel != null && Object.hasOwnProperty.call(m, "youtube_channel"))
                                    $root.google.ads.googleads.v11.common.YouTubeChannelInfo.encode(m.youtube_channel, w.uint32(74).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(80).int64(m.id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerNegativeCriterion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerNegativeCriterion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerNegativeCriterion} message CustomerNegativeCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerNegativeCriterion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerNegativeCriterion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerNegativeCriterion} CustomerNegativeCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerNegativeCriterion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 10:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 4:
                                        m.content_label = $root.google.ads.googleads.v11.common.ContentLabelInfo.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerNegativeCriterion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerNegativeCriterion} CustomerNegativeCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerNegativeCriterion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerNegativeCriterion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerNegativeCriterion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                        break;
                                    }
                                }
                                if (m.content_label != null && m.hasOwnProperty("content_label")) {
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ContentLabelInfo.verify(m.content_label);
                                        if (e)
                                            return "content_label." + e;
                                    }
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileApplicationInfo.verify(m.mobile_application);
                                        if (e)
                                            return "mobile_application." + e;
                                    }
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.verify(m.mobile_app_category);
                                        if (e)
                                            return "mobile_app_category." + e;
                                    }
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PlacementInfo.verify(m.placement);
                                        if (e)
                                            return "placement." + e;
                                    }
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.verify(m.youtube_video);
                                        if (e)
                                            return "youtube_video." + e;
                                    }
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.verify(m.youtube_channel);
                                        if (e)
                                            return "youtube_channel." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerNegativeCriterion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerNegativeCriterion} CustomerNegativeCriterion
                             */
                            CustomerNegativeCriterion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "PLACEMENT":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MOBILE_APP_CATEGORY":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "MOBILE_APPLICATION":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "DEVICE":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "LOCATION":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "LISTING_GROUP":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "AD_SCHEDULE":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "AGE_RANGE":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "GENDER":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "INCOME_RANGE":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "PARENTAL_STATUS":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "YOUTUBE_CHANNEL":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "USER_LIST":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "PROXIMITY":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "TOPIC":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "LISTING_SCOPE":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "LANGUAGE":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "IP_BLOCK":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "CONTENT_LABEL":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "CARRIER":
                                case 23:
                                    m.type = 23;
                                    break;
                                case "USER_INTEREST":
                                case 24:
                                    m.type = 24;
                                    break;
                                case "WEBPAGE":
                                case 25:
                                    m.type = 25;
                                    break;
                                case "OPERATING_SYSTEM_VERSION":
                                case 26:
                                    m.type = 26;
                                    break;
                                case "APP_PAYMENT_MODEL":
                                case 27:
                                    m.type = 27;
                                    break;
                                case "MOBILE_DEVICE":
                                case 28:
                                    m.type = 28;
                                    break;
                                case "CUSTOM_AFFINITY":
                                case 29:
                                    m.type = 29;
                                    break;
                                case "CUSTOM_INTENT":
                                case 30:
                                    m.type = 30;
                                    break;
                                case "LOCATION_GROUP":
                                case 31:
                                    m.type = 31;
                                    break;
                                case "CUSTOM_AUDIENCE":
                                case 32:
                                    m.type = 32;
                                    break;
                                case "COMBINED_AUDIENCE":
                                case 33:
                                    m.type = 33;
                                    break;
                                case "KEYWORD_THEME":
                                case 34:
                                    m.type = 34;
                                    break;
                                case "AUDIENCE":
                                case 35:
                                    m.type = 35;
                                    break;
                                }
                                if (d.content_label != null) {
                                    if (typeof d.content_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerNegativeCriterion.content_label: object expected");
                                    m.content_label = $root.google.ads.googleads.v11.common.ContentLabelInfo.fromObject(d.content_label);
                                }
                                if (d.mobile_application != null) {
                                    if (typeof d.mobile_application !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerNegativeCriterion.mobile_application: object expected");
                                    m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.fromObject(d.mobile_application);
                                }
                                if (d.mobile_app_category != null) {
                                    if (typeof d.mobile_app_category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerNegativeCriterion.mobile_app_category: object expected");
                                    m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.fromObject(d.mobile_app_category);
                                }
                                if (d.placement != null) {
                                    if (typeof d.placement !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerNegativeCriterion.placement: object expected");
                                    m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.fromObject(d.placement);
                                }
                                if (d.youtube_video != null) {
                                    if (typeof d.youtube_video !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerNegativeCriterion.youtube_video: object expected");
                                    m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.fromObject(d.youtube_video);
                                }
                                if (d.youtube_channel != null) {
                                    if (typeof d.youtube_channel !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.CustomerNegativeCriterion.youtube_channel: object expected");
                                    m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.fromObject(d.youtube_channel);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerNegativeCriterion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerNegativeCriterion} m CustomerNegativeCriterion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerNegativeCriterion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType[m.type] : m.type;
                                }
                                if (m.content_label != null && m.hasOwnProperty("content_label")) {
                                    d.content_label = $root.google.ads.googleads.v11.common.ContentLabelInfo.toObject(m.content_label, o);
                                    if (o.oneofs)
                                        d.criterion = "content_label";
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    d.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.toObject(m.mobile_application, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_application";
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    d.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.toObject(m.mobile_app_category, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_app_category";
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    d.placement = $root.google.ads.googleads.v11.common.PlacementInfo.toObject(m.placement, o);
                                    if (o.oneofs)
                                        d.criterion = "placement";
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    d.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.toObject(m.youtube_video, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_video";
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    d.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.toObject(m.youtube_channel, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_channel";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerNegativeCriterion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerNegativeCriterion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerNegativeCriterion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerNegativeCriterion;
                        })();
    
                        resources.CustomerUserAccess = (function() {
    
                            /**
                             * Properties of a CustomerUserAccess.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerUserAccess
                             * @property {string|null} [resource_name] CustomerUserAccess resource_name
                             * @property {number|null} [user_id] CustomerUserAccess user_id
                             * @property {string|null} [email_address] CustomerUserAccess email_address
                             * @property {google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole|null} [access_role] CustomerUserAccess access_role
                             * @property {string|null} [access_creation_date_time] CustomerUserAccess access_creation_date_time
                             * @property {string|null} [inviter_user_email_address] CustomerUserAccess inviter_user_email_address
                             */
    
                            /**
                             * Constructs a new CustomerUserAccess.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerUserAccess.
                             * @implements ICustomerUserAccess
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccess=} [p] Properties to set
                             */
                            function CustomerUserAccess(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerUserAccess resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            CustomerUserAccess.prototype.resource_name = "";
    
                            /**
                             * CustomerUserAccess user_id.
                             * @member {number} user_id
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            CustomerUserAccess.prototype.user_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CustomerUserAccess email_address.
                             * @member {string|null|undefined} email_address
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            CustomerUserAccess.prototype.email_address = null;
    
                            /**
                             * CustomerUserAccess access_role.
                             * @member {google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole} access_role
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            CustomerUserAccess.prototype.access_role = 0;
    
                            /**
                             * CustomerUserAccess access_creation_date_time.
                             * @member {string|null|undefined} access_creation_date_time
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            CustomerUserAccess.prototype.access_creation_date_time = null;
    
                            /**
                             * CustomerUserAccess inviter_user_email_address.
                             * @member {string|null|undefined} inviter_user_email_address
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            CustomerUserAccess.prototype.inviter_user_email_address = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerUserAccess _email_address.
                             * @member {"email_address"|undefined} _email_address
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            Object.defineProperty(CustomerUserAccess.prototype, "_email_address", {
                                get: $util.oneOfGetter($oneOfFields = ["email_address"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerUserAccess _access_creation_date_time.
                             * @member {"access_creation_date_time"|undefined} _access_creation_date_time
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            Object.defineProperty(CustomerUserAccess.prototype, "_access_creation_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["access_creation_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CustomerUserAccess _inviter_user_email_address.
                             * @member {"inviter_user_email_address"|undefined} _inviter_user_email_address
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             */
                            Object.defineProperty(CustomerUserAccess.prototype, "_inviter_user_email_address", {
                                get: $util.oneOfGetter($oneOfFields = ["inviter_user_email_address"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerUserAccess instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccess=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccess} CustomerUserAccess instance
                             */
                            CustomerUserAccess.create = function create(properties) {
                                return new CustomerUserAccess(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccess message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerUserAccess.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccess} m CustomerUserAccess message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccess.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.user_id != null && Object.hasOwnProperty.call(m, "user_id"))
                                    w.uint32(16).int64(m.user_id);
                                if (m.email_address != null && Object.hasOwnProperty.call(m, "email_address"))
                                    w.uint32(26).string(m.email_address);
                                if (m.access_role != null && Object.hasOwnProperty.call(m, "access_role"))
                                    w.uint32(32).int32(m.access_role);
                                if (m.access_creation_date_time != null && Object.hasOwnProperty.call(m, "access_creation_date_time"))
                                    w.uint32(50).string(m.access_creation_date_time);
                                if (m.inviter_user_email_address != null && Object.hasOwnProperty.call(m, "inviter_user_email_address"))
                                    w.uint32(58).string(m.inviter_user_email_address);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccess message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerUserAccess.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccess} message CustomerUserAccess message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccess.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerUserAccess message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccess} CustomerUserAccess
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccess.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerUserAccess();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.user_id = r.int64();
                                        break;
                                    case 3:
                                        m.email_address = r.string();
                                        break;
                                    case 4:
                                        m.access_role = r.int32();
                                        break;
                                    case 6:
                                        m.access_creation_date_time = r.string();
                                        break;
                                    case 7:
                                        m.inviter_user_email_address = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerUserAccess message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccess} CustomerUserAccess
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccess.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerUserAccess message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerUserAccess.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.user_id != null && m.hasOwnProperty("user_id")) {
                                    if (!$util.isInteger(m.user_id) && !(m.user_id && $util.isInteger(m.user_id.low) && $util.isInteger(m.user_id.high)))
                                        return "user_id: integer|Long expected";
                                }
                                if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                    p._email_address = 1;
                                    if (!$util.isString(m.email_address))
                                        return "email_address: string expected";
                                }
                                if (m.access_role != null && m.hasOwnProperty("access_role")) {
                                    switch (m.access_role) {
                                    default:
                                        return "access_role: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.access_creation_date_time != null && m.hasOwnProperty("access_creation_date_time")) {
                                    p._access_creation_date_time = 1;
                                    if (!$util.isString(m.access_creation_date_time))
                                        return "access_creation_date_time: string expected";
                                }
                                if (m.inviter_user_email_address != null && m.hasOwnProperty("inviter_user_email_address")) {
                                    p._inviter_user_email_address = 1;
                                    if (!$util.isString(m.inviter_user_email_address))
                                        return "inviter_user_email_address: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerUserAccess message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccess} CustomerUserAccess
                             */
                            CustomerUserAccess.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerUserAccess)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerUserAccess();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.user_id != null) {
                                    if ($util.Long)
                                        (m.user_id = $util.Long.fromValue(d.user_id)).unsigned = false;
                                    else if (typeof d.user_id === "string")
                                        m.user_id = parseInt(d.user_id, 10);
                                    else if (typeof d.user_id === "number")
                                        m.user_id = d.user_id;
                                    else if (typeof d.user_id === "object")
                                        m.user_id = new $util.LongBits(d.user_id.low >>> 0, d.user_id.high >>> 0).toNumber();
                                }
                                if (d.email_address != null) {
                                    m.email_address = String(d.email_address);
                                }
                                switch (d.access_role) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.access_role = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.access_role = 1;
                                    break;
                                case "ADMIN":
                                case 2:
                                    m.access_role = 2;
                                    break;
                                case "STANDARD":
                                case 3:
                                    m.access_role = 3;
                                    break;
                                case "READ_ONLY":
                                case 4:
                                    m.access_role = 4;
                                    break;
                                case "EMAIL_ONLY":
                                case 5:
                                    m.access_role = 5;
                                    break;
                                }
                                if (d.access_creation_date_time != null) {
                                    m.access_creation_date_time = String(d.access_creation_date_time);
                                }
                                if (d.inviter_user_email_address != null) {
                                    m.inviter_user_email_address = String(d.inviter_user_email_address);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerUserAccess message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerUserAccess} m CustomerUserAccess
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerUserAccess.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.user_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.user_id = o.longs === String ? "0" : 0;
                                    d.access_role = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.user_id != null && m.hasOwnProperty("user_id")) {
                                    if (typeof m.user_id === "number")
                                        d.user_id = o.longs === String ? String(m.user_id) : m.user_id;
                                    else
                                        d.user_id = o.longs === String ? $util.Long.prototype.toString.call(m.user_id) : o.longs === Number ? new $util.LongBits(m.user_id.low >>> 0, m.user_id.high >>> 0).toNumber() : m.user_id;
                                }
                                if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                    d.email_address = m.email_address;
                                    if (o.oneofs)
                                        d._email_address = "email_address";
                                }
                                if (m.access_role != null && m.hasOwnProperty("access_role")) {
                                    d.access_role = o.enums === String ? $root.google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole[m.access_role] : m.access_role;
                                }
                                if (m.access_creation_date_time != null && m.hasOwnProperty("access_creation_date_time")) {
                                    d.access_creation_date_time = m.access_creation_date_time;
                                    if (o.oneofs)
                                        d._access_creation_date_time = "access_creation_date_time";
                                }
                                if (m.inviter_user_email_address != null && m.hasOwnProperty("inviter_user_email_address")) {
                                    d.inviter_user_email_address = m.inviter_user_email_address;
                                    if (o.oneofs)
                                        d._inviter_user_email_address = "inviter_user_email_address";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerUserAccess to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccess
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerUserAccess.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerUserAccess;
                        })();
    
                        resources.CustomerUserAccessInvitation = (function() {
    
                            /**
                             * Properties of a CustomerUserAccessInvitation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomerUserAccessInvitation
                             * @property {string|null} [resource_name] CustomerUserAccessInvitation resource_name
                             * @property {number|null} [invitation_id] CustomerUserAccessInvitation invitation_id
                             * @property {google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole|null} [access_role] CustomerUserAccessInvitation access_role
                             * @property {string|null} [email_address] CustomerUserAccessInvitation email_address
                             * @property {string|null} [creation_date_time] CustomerUserAccessInvitation creation_date_time
                             * @property {google.ads.googleads.v11.enums.AccessInvitationStatusEnum.AccessInvitationStatus|null} [invitation_status] CustomerUserAccessInvitation invitation_status
                             */
    
                            /**
                             * Constructs a new CustomerUserAccessInvitation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomerUserAccessInvitation.
                             * @implements ICustomerUserAccessInvitation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation=} [p] Properties to set
                             */
                            function CustomerUserAccessInvitation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerUserAccessInvitation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @instance
                             */
                            CustomerUserAccessInvitation.prototype.resource_name = "";
    
                            /**
                             * CustomerUserAccessInvitation invitation_id.
                             * @member {number} invitation_id
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @instance
                             */
                            CustomerUserAccessInvitation.prototype.invitation_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CustomerUserAccessInvitation access_role.
                             * @member {google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole} access_role
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @instance
                             */
                            CustomerUserAccessInvitation.prototype.access_role = 0;
    
                            /**
                             * CustomerUserAccessInvitation email_address.
                             * @member {string} email_address
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @instance
                             */
                            CustomerUserAccessInvitation.prototype.email_address = "";
    
                            /**
                             * CustomerUserAccessInvitation creation_date_time.
                             * @member {string} creation_date_time
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @instance
                             */
                            CustomerUserAccessInvitation.prototype.creation_date_time = "";
    
                            /**
                             * CustomerUserAccessInvitation invitation_status.
                             * @member {google.ads.googleads.v11.enums.AccessInvitationStatusEnum.AccessInvitationStatus} invitation_status
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @instance
                             */
                            CustomerUserAccessInvitation.prototype.invitation_status = 0;
    
                            /**
                             * Creates a new CustomerUserAccessInvitation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccessInvitation} CustomerUserAccessInvitation instance
                             */
                            CustomerUserAccessInvitation.create = function create(properties) {
                                return new CustomerUserAccessInvitation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessInvitation message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerUserAccessInvitation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation} m CustomerUserAccessInvitation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessInvitation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.invitation_id != null && Object.hasOwnProperty.call(m, "invitation_id"))
                                    w.uint32(16).int64(m.invitation_id);
                                if (m.access_role != null && Object.hasOwnProperty.call(m, "access_role"))
                                    w.uint32(24).int32(m.access_role);
                                if (m.email_address != null && Object.hasOwnProperty.call(m, "email_address"))
                                    w.uint32(34).string(m.email_address);
                                if (m.creation_date_time != null && Object.hasOwnProperty.call(m, "creation_date_time"))
                                    w.uint32(42).string(m.creation_date_time);
                                if (m.invitation_status != null && Object.hasOwnProperty.call(m, "invitation_status"))
                                    w.uint32(48).int32(m.invitation_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessInvitation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomerUserAccessInvitation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation} message CustomerUserAccessInvitation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessInvitation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerUserAccessInvitation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccessInvitation} CustomerUserAccessInvitation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessInvitation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.invitation_id = r.int64();
                                        break;
                                    case 3:
                                        m.access_role = r.int32();
                                        break;
                                    case 4:
                                        m.email_address = r.string();
                                        break;
                                    case 5:
                                        m.creation_date_time = r.string();
                                        break;
                                    case 6:
                                        m.invitation_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerUserAccessInvitation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccessInvitation} CustomerUserAccessInvitation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessInvitation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerUserAccessInvitation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerUserAccessInvitation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.invitation_id != null && m.hasOwnProperty("invitation_id")) {
                                    if (!$util.isInteger(m.invitation_id) && !(m.invitation_id && $util.isInteger(m.invitation_id.low) && $util.isInteger(m.invitation_id.high)))
                                        return "invitation_id: integer|Long expected";
                                }
                                if (m.access_role != null && m.hasOwnProperty("access_role")) {
                                    switch (m.access_role) {
                                    default:
                                        return "access_role: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                    if (!$util.isString(m.email_address))
                                        return "email_address: string expected";
                                }
                                if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                    if (!$util.isString(m.creation_date_time))
                                        return "creation_date_time: string expected";
                                }
                                if (m.invitation_status != null && m.hasOwnProperty("invitation_status")) {
                                    switch (m.invitation_status) {
                                    default:
                                        return "invitation_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerUserAccessInvitation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomerUserAccessInvitation} CustomerUserAccessInvitation
                             */
                            CustomerUserAccessInvitation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.invitation_id != null) {
                                    if ($util.Long)
                                        (m.invitation_id = $util.Long.fromValue(d.invitation_id)).unsigned = false;
                                    else if (typeof d.invitation_id === "string")
                                        m.invitation_id = parseInt(d.invitation_id, 10);
                                    else if (typeof d.invitation_id === "number")
                                        m.invitation_id = d.invitation_id;
                                    else if (typeof d.invitation_id === "object")
                                        m.invitation_id = new $util.LongBits(d.invitation_id.low >>> 0, d.invitation_id.high >>> 0).toNumber();
                                }
                                switch (d.access_role) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.access_role = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.access_role = 1;
                                    break;
                                case "ADMIN":
                                case 2:
                                    m.access_role = 2;
                                    break;
                                case "STANDARD":
                                case 3:
                                    m.access_role = 3;
                                    break;
                                case "READ_ONLY":
                                case 4:
                                    m.access_role = 4;
                                    break;
                                case "EMAIL_ONLY":
                                case 5:
                                    m.access_role = 5;
                                    break;
                                }
                                if (d.email_address != null) {
                                    m.email_address = String(d.email_address);
                                }
                                if (d.creation_date_time != null) {
                                    m.creation_date_time = String(d.creation_date_time);
                                }
                                switch (d.invitation_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.invitation_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.invitation_status = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.invitation_status = 2;
                                    break;
                                case "DECLINED":
                                case 3:
                                    m.invitation_status = 3;
                                    break;
                                case "EXPIRED":
                                case 4:
                                    m.invitation_status = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerUserAccessInvitation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomerUserAccessInvitation} m CustomerUserAccessInvitation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerUserAccessInvitation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.invitation_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.invitation_id = o.longs === String ? "0" : 0;
                                    d.access_role = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.email_address = "";
                                    d.creation_date_time = "";
                                    d.invitation_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.invitation_id != null && m.hasOwnProperty("invitation_id")) {
                                    if (typeof m.invitation_id === "number")
                                        d.invitation_id = o.longs === String ? String(m.invitation_id) : m.invitation_id;
                                    else
                                        d.invitation_id = o.longs === String ? $util.Long.prototype.toString.call(m.invitation_id) : o.longs === Number ? new $util.LongBits(m.invitation_id.low >>> 0, m.invitation_id.high >>> 0).toNumber() : m.invitation_id;
                                }
                                if (m.access_role != null && m.hasOwnProperty("access_role")) {
                                    d.access_role = o.enums === String ? $root.google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole[m.access_role] : m.access_role;
                                }
                                if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                    d.email_address = m.email_address;
                                }
                                if (m.creation_date_time != null && m.hasOwnProperty("creation_date_time")) {
                                    d.creation_date_time = m.creation_date_time;
                                }
                                if (m.invitation_status != null && m.hasOwnProperty("invitation_status")) {
                                    d.invitation_status = o.enums === String ? $root.google.ads.googleads.v11.enums.AccessInvitationStatusEnum.AccessInvitationStatus[m.invitation_status] : m.invitation_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerUserAccessInvitation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomerUserAccessInvitation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerUserAccessInvitation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerUserAccessInvitation;
                        })();
    
                        resources.CustomizerAttribute = (function() {
    
                            /**
                             * Properties of a CustomizerAttribute.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ICustomizerAttribute
                             * @property {string|null} [resource_name] CustomizerAttribute resource_name
                             * @property {number|null} [id] CustomizerAttribute id
                             * @property {string|null} [name] CustomizerAttribute name
                             * @property {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.CustomizerAttributeType|null} [type] CustomizerAttribute type
                             * @property {google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum.CustomizerAttributeStatus|null} [status] CustomizerAttribute status
                             */
    
                            /**
                             * Constructs a new CustomizerAttribute.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a CustomizerAttribute.
                             * @implements ICustomizerAttribute
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ICustomizerAttribute=} [p] Properties to set
                             */
                            function CustomizerAttribute(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomizerAttribute resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @instance
                             */
                            CustomizerAttribute.prototype.resource_name = "";
    
                            /**
                             * CustomizerAttribute id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @instance
                             */
                            CustomizerAttribute.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CustomizerAttribute name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @instance
                             */
                            CustomizerAttribute.prototype.name = "";
    
                            /**
                             * CustomizerAttribute type.
                             * @member {google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.CustomizerAttributeType} type
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @instance
                             */
                            CustomizerAttribute.prototype.type = 0;
    
                            /**
                             * CustomizerAttribute status.
                             * @member {google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum.CustomizerAttributeStatus} status
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @instance
                             */
                            CustomizerAttribute.prototype.status = 0;
    
                            /**
                             * Creates a new CustomizerAttribute instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomizerAttribute=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.CustomizerAttribute} CustomizerAttribute instance
                             */
                            CustomizerAttribute.create = function create(properties) {
                                return new CustomizerAttribute(properties);
                            };
    
                            /**
                             * Encodes the specified CustomizerAttribute message. Does not implicitly {@link google.ads.googleads.v11.resources.CustomizerAttribute.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomizerAttribute} m CustomizerAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttribute.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomizerAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.CustomizerAttribute.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.ICustomizerAttribute} message CustomizerAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomizerAttribute message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.CustomizerAttribute} CustomizerAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttribute.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.CustomizerAttribute();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomizerAttribute message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.CustomizerAttribute} CustomizerAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttribute.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomizerAttribute message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomizerAttribute.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomizerAttribute message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.CustomizerAttribute} CustomizerAttribute
                             */
                            CustomizerAttribute.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.CustomizerAttribute)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.CustomizerAttribute();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "TEXT":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "NUMBER":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "PRICE":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "PERCENT":
                                case 5:
                                    m.type = 5;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomizerAttribute message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @static
                             * @param {google.ads.googleads.v11.resources.CustomizerAttribute} m CustomizerAttribute
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomizerAttribute.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomizerAttributeTypeEnum.CustomizerAttributeType[m.type] : m.type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomizerAttributeStatusEnum.CustomizerAttributeStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomizerAttribute to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.CustomizerAttribute
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomizerAttribute.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomizerAttribute;
                        })();
    
                        resources.DetailPlacementView = (function() {
    
                            /**
                             * Properties of a DetailPlacementView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IDetailPlacementView
                             * @property {string|null} [resource_name] DetailPlacementView resource_name
                             * @property {string|null} [placement] DetailPlacementView placement
                             * @property {string|null} [display_name] DetailPlacementView display_name
                             * @property {string|null} [group_placement_target_url] DetailPlacementView group_placement_target_url
                             * @property {string|null} [target_url] DetailPlacementView target_url
                             * @property {google.ads.googleads.v11.enums.PlacementTypeEnum.PlacementType|null} [placement_type] DetailPlacementView placement_type
                             */
    
                            /**
                             * Constructs a new DetailPlacementView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a DetailPlacementView.
                             * @implements IDetailPlacementView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IDetailPlacementView=} [p] Properties to set
                             */
                            function DetailPlacementView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DetailPlacementView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            DetailPlacementView.prototype.resource_name = "";
    
                            /**
                             * DetailPlacementView placement.
                             * @member {string|null|undefined} placement
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            DetailPlacementView.prototype.placement = null;
    
                            /**
                             * DetailPlacementView display_name.
                             * @member {string|null|undefined} display_name
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            DetailPlacementView.prototype.display_name = null;
    
                            /**
                             * DetailPlacementView group_placement_target_url.
                             * @member {string|null|undefined} group_placement_target_url
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            DetailPlacementView.prototype.group_placement_target_url = null;
    
                            /**
                             * DetailPlacementView target_url.
                             * @member {string|null|undefined} target_url
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            DetailPlacementView.prototype.target_url = null;
    
                            /**
                             * DetailPlacementView placement_type.
                             * @member {google.ads.googleads.v11.enums.PlacementTypeEnum.PlacementType} placement_type
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            DetailPlacementView.prototype.placement_type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DetailPlacementView _placement.
                             * @member {"placement"|undefined} _placement
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            Object.defineProperty(DetailPlacementView.prototype, "_placement", {
                                get: $util.oneOfGetter($oneOfFields = ["placement"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DetailPlacementView _display_name.
                             * @member {"display_name"|undefined} _display_name
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            Object.defineProperty(DetailPlacementView.prototype, "_display_name", {
                                get: $util.oneOfGetter($oneOfFields = ["display_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DetailPlacementView _group_placement_target_url.
                             * @member {"group_placement_target_url"|undefined} _group_placement_target_url
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            Object.defineProperty(DetailPlacementView.prototype, "_group_placement_target_url", {
                                get: $util.oneOfGetter($oneOfFields = ["group_placement_target_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DetailPlacementView _target_url.
                             * @member {"target_url"|undefined} _target_url
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             */
                            Object.defineProperty(DetailPlacementView.prototype, "_target_url", {
                                get: $util.oneOfGetter($oneOfFields = ["target_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DetailPlacementView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDetailPlacementView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.DetailPlacementView} DetailPlacementView instance
                             */
                            DetailPlacementView.create = function create(properties) {
                                return new DetailPlacementView(properties);
                            };
    
                            /**
                             * Encodes the specified DetailPlacementView message. Does not implicitly {@link google.ads.googleads.v11.resources.DetailPlacementView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDetailPlacementView} m DetailPlacementView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DetailPlacementView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.placement_type != null && Object.hasOwnProperty.call(m, "placement_type"))
                                    w.uint32(48).int32(m.placement_type);
                                if (m.placement != null && Object.hasOwnProperty.call(m, "placement"))
                                    w.uint32(58).string(m.placement);
                                if (m.display_name != null && Object.hasOwnProperty.call(m, "display_name"))
                                    w.uint32(66).string(m.display_name);
                                if (m.group_placement_target_url != null && Object.hasOwnProperty.call(m, "group_placement_target_url"))
                                    w.uint32(74).string(m.group_placement_target_url);
                                if (m.target_url != null && Object.hasOwnProperty.call(m, "target_url"))
                                    w.uint32(82).string(m.target_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DetailPlacementView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.DetailPlacementView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDetailPlacementView} message DetailPlacementView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DetailPlacementView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DetailPlacementView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.DetailPlacementView} DetailPlacementView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DetailPlacementView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.DetailPlacementView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.placement = r.string();
                                        break;
                                    case 8:
                                        m.display_name = r.string();
                                        break;
                                    case 9:
                                        m.group_placement_target_url = r.string();
                                        break;
                                    case 10:
                                        m.target_url = r.string();
                                        break;
                                    case 6:
                                        m.placement_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DetailPlacementView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.DetailPlacementView} DetailPlacementView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DetailPlacementView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DetailPlacementView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DetailPlacementView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    p._placement = 1;
                                    if (!$util.isString(m.placement))
                                        return "placement: string expected";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    p._display_name = 1;
                                    if (!$util.isString(m.display_name))
                                        return "display_name: string expected";
                                }
                                if (m.group_placement_target_url != null && m.hasOwnProperty("group_placement_target_url")) {
                                    p._group_placement_target_url = 1;
                                    if (!$util.isString(m.group_placement_target_url))
                                        return "group_placement_target_url: string expected";
                                }
                                if (m.target_url != null && m.hasOwnProperty("target_url")) {
                                    p._target_url = 1;
                                    if (!$util.isString(m.target_url))
                                        return "target_url: string expected";
                                }
                                if (m.placement_type != null && m.hasOwnProperty("placement_type")) {
                                    switch (m.placement_type) {
                                    default:
                                        return "placement_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DetailPlacementView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.DetailPlacementView} DetailPlacementView
                             */
                            DetailPlacementView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.DetailPlacementView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.DetailPlacementView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.placement != null) {
                                    m.placement = String(d.placement);
                                }
                                if (d.display_name != null) {
                                    m.display_name = String(d.display_name);
                                }
                                if (d.group_placement_target_url != null) {
                                    m.group_placement_target_url = String(d.group_placement_target_url);
                                }
                                if (d.target_url != null) {
                                    m.target_url = String(d.target_url);
                                }
                                switch (d.placement_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.placement_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.placement_type = 1;
                                    break;
                                case "WEBSITE":
                                case 2:
                                    m.placement_type = 2;
                                    break;
                                case "MOBILE_APP_CATEGORY":
                                case 3:
                                    m.placement_type = 3;
                                    break;
                                case "MOBILE_APPLICATION":
                                case 4:
                                    m.placement_type = 4;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 5:
                                    m.placement_type = 5;
                                    break;
                                case "YOUTUBE_CHANNEL":
                                case 6:
                                    m.placement_type = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DetailPlacementView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.DetailPlacementView} m DetailPlacementView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DetailPlacementView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.placement_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.placement_type != null && m.hasOwnProperty("placement_type")) {
                                    d.placement_type = o.enums === String ? $root.google.ads.googleads.v11.enums.PlacementTypeEnum.PlacementType[m.placement_type] : m.placement_type;
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    d.placement = m.placement;
                                    if (o.oneofs)
                                        d._placement = "placement";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    d.display_name = m.display_name;
                                    if (o.oneofs)
                                        d._display_name = "display_name";
                                }
                                if (m.group_placement_target_url != null && m.hasOwnProperty("group_placement_target_url")) {
                                    d.group_placement_target_url = m.group_placement_target_url;
                                    if (o.oneofs)
                                        d._group_placement_target_url = "group_placement_target_url";
                                }
                                if (m.target_url != null && m.hasOwnProperty("target_url")) {
                                    d.target_url = m.target_url;
                                    if (o.oneofs)
                                        d._target_url = "target_url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DetailPlacementView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.DetailPlacementView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DetailPlacementView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DetailPlacementView;
                        })();
    
                        resources.DetailedDemographic = (function() {
    
                            /**
                             * Properties of a DetailedDemographic.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IDetailedDemographic
                             * @property {string|null} [resource_name] DetailedDemographic resource_name
                             * @property {number|null} [id] DetailedDemographic id
                             * @property {string|null} [name] DetailedDemographic name
                             * @property {string|null} [parent] DetailedDemographic parent
                             * @property {boolean|null} [launched_to_all] DetailedDemographic launched_to_all
                             * @property {Array.<google.ads.googleads.v11.common.ICriterionCategoryAvailability>|null} [availabilities] DetailedDemographic availabilities
                             */
    
                            /**
                             * Constructs a new DetailedDemographic.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a DetailedDemographic.
                             * @implements IDetailedDemographic
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IDetailedDemographic=} [p] Properties to set
                             */
                            function DetailedDemographic(p) {
                                this.availabilities = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DetailedDemographic resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @instance
                             */
                            DetailedDemographic.prototype.resource_name = "";
    
                            /**
                             * DetailedDemographic id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @instance
                             */
                            DetailedDemographic.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * DetailedDemographic name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @instance
                             */
                            DetailedDemographic.prototype.name = "";
    
                            /**
                             * DetailedDemographic parent.
                             * @member {string} parent
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @instance
                             */
                            DetailedDemographic.prototype.parent = "";
    
                            /**
                             * DetailedDemographic launched_to_all.
                             * @member {boolean} launched_to_all
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @instance
                             */
                            DetailedDemographic.prototype.launched_to_all = false;
    
                            /**
                             * DetailedDemographic availabilities.
                             * @member {Array.<google.ads.googleads.v11.common.ICriterionCategoryAvailability>} availabilities
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @instance
                             */
                            DetailedDemographic.prototype.availabilities = $util.emptyArray;
    
                            /**
                             * Creates a new DetailedDemographic instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDetailedDemographic=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.DetailedDemographic} DetailedDemographic instance
                             */
                            DetailedDemographic.create = function create(properties) {
                                return new DetailedDemographic(properties);
                            };
    
                            /**
                             * Encodes the specified DetailedDemographic message. Does not implicitly {@link google.ads.googleads.v11.resources.DetailedDemographic.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDetailedDemographic} m DetailedDemographic message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DetailedDemographic.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.parent != null && Object.hasOwnProperty.call(m, "parent"))
                                    w.uint32(34).string(m.parent);
                                if (m.launched_to_all != null && Object.hasOwnProperty.call(m, "launched_to_all"))
                                    w.uint32(40).bool(m.launched_to_all);
                                if (m.availabilities != null && m.availabilities.length) {
                                    for (var i = 0; i < m.availabilities.length; ++i)
                                        $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.encode(m.availabilities[i], w.uint32(50).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified DetailedDemographic message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.DetailedDemographic.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDetailedDemographic} message DetailedDemographic message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DetailedDemographic.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DetailedDemographic message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.DetailedDemographic} DetailedDemographic
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DetailedDemographic.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.DetailedDemographic();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.parent = r.string();
                                        break;
                                    case 5:
                                        m.launched_to_all = r.bool();
                                        break;
                                    case 6:
                                        if (!(m.availabilities && m.availabilities.length))
                                            m.availabilities = [];
                                        m.availabilities.push($root.google.ads.googleads.v11.common.CriterionCategoryAvailability.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DetailedDemographic message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.DetailedDemographic} DetailedDemographic
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DetailedDemographic.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DetailedDemographic message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DetailedDemographic.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.parent != null && m.hasOwnProperty("parent")) {
                                    if (!$util.isString(m.parent))
                                        return "parent: string expected";
                                }
                                if (m.launched_to_all != null && m.hasOwnProperty("launched_to_all")) {
                                    if (typeof m.launched_to_all !== "boolean")
                                        return "launched_to_all: boolean expected";
                                }
                                if (m.availabilities != null && m.hasOwnProperty("availabilities")) {
                                    if (!Array.isArray(m.availabilities))
                                        return "availabilities: array expected";
                                    for (var i = 0; i < m.availabilities.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.verify(m.availabilities[i]);
                                            if (e)
                                                return "availabilities." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DetailedDemographic message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.DetailedDemographic} DetailedDemographic
                             */
                            DetailedDemographic.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.DetailedDemographic)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.DetailedDemographic();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.parent != null) {
                                    m.parent = String(d.parent);
                                }
                                if (d.launched_to_all != null) {
                                    m.launched_to_all = Boolean(d.launched_to_all);
                                }
                                if (d.availabilities) {
                                    if (!Array.isArray(d.availabilities))
                                        throw TypeError(".google.ads.googleads.v11.resources.DetailedDemographic.availabilities: array expected");
                                    m.availabilities = [];
                                    for (var i = 0; i < d.availabilities.length; ++i) {
                                        if (typeof d.availabilities[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.DetailedDemographic.availabilities: object expected");
                                        m.availabilities[i] = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.fromObject(d.availabilities[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DetailedDemographic message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @static
                             * @param {google.ads.googleads.v11.resources.DetailedDemographic} m DetailedDemographic
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DetailedDemographic.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.availabilities = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.name = "";
                                    d.parent = "";
                                    d.launched_to_all = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.parent != null && m.hasOwnProperty("parent")) {
                                    d.parent = m.parent;
                                }
                                if (m.launched_to_all != null && m.hasOwnProperty("launched_to_all")) {
                                    d.launched_to_all = m.launched_to_all;
                                }
                                if (m.availabilities && m.availabilities.length) {
                                    d.availabilities = [];
                                    for (var j = 0; j < m.availabilities.length; ++j) {
                                        d.availabilities[j] = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.toObject(m.availabilities[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DetailedDemographic to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.DetailedDemographic
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DetailedDemographic.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DetailedDemographic;
                        })();
    
                        resources.DisplayKeywordView = (function() {
    
                            /**
                             * Properties of a DisplayKeywordView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IDisplayKeywordView
                             * @property {string|null} [resource_name] DisplayKeywordView resource_name
                             */
    
                            /**
                             * Constructs a new DisplayKeywordView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a DisplayKeywordView.
                             * @implements IDisplayKeywordView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IDisplayKeywordView=} [p] Properties to set
                             */
                            function DisplayKeywordView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DisplayKeywordView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @instance
                             */
                            DisplayKeywordView.prototype.resource_name = "";
    
                            /**
                             * Creates a new DisplayKeywordView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDisplayKeywordView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.DisplayKeywordView} DisplayKeywordView instance
                             */
                            DisplayKeywordView.create = function create(properties) {
                                return new DisplayKeywordView(properties);
                            };
    
                            /**
                             * Encodes the specified DisplayKeywordView message. Does not implicitly {@link google.ads.googleads.v11.resources.DisplayKeywordView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDisplayKeywordView} m DisplayKeywordView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayKeywordView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DisplayKeywordView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.DisplayKeywordView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDisplayKeywordView} message DisplayKeywordView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DisplayKeywordView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DisplayKeywordView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.DisplayKeywordView} DisplayKeywordView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayKeywordView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.DisplayKeywordView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DisplayKeywordView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.DisplayKeywordView} DisplayKeywordView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DisplayKeywordView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DisplayKeywordView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DisplayKeywordView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DisplayKeywordView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.DisplayKeywordView} DisplayKeywordView
                             */
                            DisplayKeywordView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.DisplayKeywordView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.DisplayKeywordView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DisplayKeywordView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.DisplayKeywordView} m DisplayKeywordView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DisplayKeywordView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DisplayKeywordView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.DisplayKeywordView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DisplayKeywordView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DisplayKeywordView;
                        })();
    
                        resources.DistanceView = (function() {
    
                            /**
                             * Properties of a DistanceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IDistanceView
                             * @property {string|null} [resource_name] DistanceView resource_name
                             * @property {google.ads.googleads.v11.enums.DistanceBucketEnum.DistanceBucket|null} [distance_bucket] DistanceView distance_bucket
                             * @property {boolean|null} [metric_system] DistanceView metric_system
                             */
    
                            /**
                             * Constructs a new DistanceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a DistanceView.
                             * @implements IDistanceView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IDistanceView=} [p] Properties to set
                             */
                            function DistanceView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DistanceView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @instance
                             */
                            DistanceView.prototype.resource_name = "";
    
                            /**
                             * DistanceView distance_bucket.
                             * @member {google.ads.googleads.v11.enums.DistanceBucketEnum.DistanceBucket} distance_bucket
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @instance
                             */
                            DistanceView.prototype.distance_bucket = 0;
    
                            /**
                             * DistanceView metric_system.
                             * @member {boolean|null|undefined} metric_system
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @instance
                             */
                            DistanceView.prototype.metric_system = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DistanceView _metric_system.
                             * @member {"metric_system"|undefined} _metric_system
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @instance
                             */
                            Object.defineProperty(DistanceView.prototype, "_metric_system", {
                                get: $util.oneOfGetter($oneOfFields = ["metric_system"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DistanceView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDistanceView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.DistanceView} DistanceView instance
                             */
                            DistanceView.create = function create(properties) {
                                return new DistanceView(properties);
                            };
    
                            /**
                             * Encodes the specified DistanceView message. Does not implicitly {@link google.ads.googleads.v11.resources.DistanceView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDistanceView} m DistanceView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DistanceView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.distance_bucket != null && Object.hasOwnProperty.call(m, "distance_bucket"))
                                    w.uint32(16).int32(m.distance_bucket);
                                if (m.metric_system != null && Object.hasOwnProperty.call(m, "metric_system"))
                                    w.uint32(32).bool(m.metric_system);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DistanceView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.DistanceView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDistanceView} message DistanceView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DistanceView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DistanceView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.DistanceView} DistanceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DistanceView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.DistanceView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.distance_bucket = r.int32();
                                        break;
                                    case 4:
                                        m.metric_system = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DistanceView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.DistanceView} DistanceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DistanceView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DistanceView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DistanceView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.distance_bucket != null && m.hasOwnProperty("distance_bucket")) {
                                    switch (m.distance_bucket) {
                                    default:
                                        return "distance_bucket: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                        break;
                                    }
                                }
                                if (m.metric_system != null && m.hasOwnProperty("metric_system")) {
                                    p._metric_system = 1;
                                    if (typeof m.metric_system !== "boolean")
                                        return "metric_system: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DistanceView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.DistanceView} DistanceView
                             */
                            DistanceView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.DistanceView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.DistanceView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.distance_bucket) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.distance_bucket = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.distance_bucket = 1;
                                    break;
                                case "WITHIN_700M":
                                case 2:
                                    m.distance_bucket = 2;
                                    break;
                                case "WITHIN_1KM":
                                case 3:
                                    m.distance_bucket = 3;
                                    break;
                                case "WITHIN_5KM":
                                case 4:
                                    m.distance_bucket = 4;
                                    break;
                                case "WITHIN_10KM":
                                case 5:
                                    m.distance_bucket = 5;
                                    break;
                                case "WITHIN_15KM":
                                case 6:
                                    m.distance_bucket = 6;
                                    break;
                                case "WITHIN_20KM":
                                case 7:
                                    m.distance_bucket = 7;
                                    break;
                                case "WITHIN_25KM":
                                case 8:
                                    m.distance_bucket = 8;
                                    break;
                                case "WITHIN_30KM":
                                case 9:
                                    m.distance_bucket = 9;
                                    break;
                                case "WITHIN_35KM":
                                case 10:
                                    m.distance_bucket = 10;
                                    break;
                                case "WITHIN_40KM":
                                case 11:
                                    m.distance_bucket = 11;
                                    break;
                                case "WITHIN_45KM":
                                case 12:
                                    m.distance_bucket = 12;
                                    break;
                                case "WITHIN_50KM":
                                case 13:
                                    m.distance_bucket = 13;
                                    break;
                                case "WITHIN_55KM":
                                case 14:
                                    m.distance_bucket = 14;
                                    break;
                                case "WITHIN_60KM":
                                case 15:
                                    m.distance_bucket = 15;
                                    break;
                                case "WITHIN_65KM":
                                case 16:
                                    m.distance_bucket = 16;
                                    break;
                                case "BEYOND_65KM":
                                case 17:
                                    m.distance_bucket = 17;
                                    break;
                                case "WITHIN_0_7MILES":
                                case 18:
                                    m.distance_bucket = 18;
                                    break;
                                case "WITHIN_1MILE":
                                case 19:
                                    m.distance_bucket = 19;
                                    break;
                                case "WITHIN_5MILES":
                                case 20:
                                    m.distance_bucket = 20;
                                    break;
                                case "WITHIN_10MILES":
                                case 21:
                                    m.distance_bucket = 21;
                                    break;
                                case "WITHIN_15MILES":
                                case 22:
                                    m.distance_bucket = 22;
                                    break;
                                case "WITHIN_20MILES":
                                case 23:
                                    m.distance_bucket = 23;
                                    break;
                                case "WITHIN_25MILES":
                                case 24:
                                    m.distance_bucket = 24;
                                    break;
                                case "WITHIN_30MILES":
                                case 25:
                                    m.distance_bucket = 25;
                                    break;
                                case "WITHIN_35MILES":
                                case 26:
                                    m.distance_bucket = 26;
                                    break;
                                case "WITHIN_40MILES":
                                case 27:
                                    m.distance_bucket = 27;
                                    break;
                                case "BEYOND_40MILES":
                                case 28:
                                    m.distance_bucket = 28;
                                    break;
                                }
                                if (d.metric_system != null) {
                                    m.metric_system = Boolean(d.metric_system);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DistanceView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.DistanceView} m DistanceView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DistanceView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.distance_bucket = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.distance_bucket != null && m.hasOwnProperty("distance_bucket")) {
                                    d.distance_bucket = o.enums === String ? $root.google.ads.googleads.v11.enums.DistanceBucketEnum.DistanceBucket[m.distance_bucket] : m.distance_bucket;
                                }
                                if (m.metric_system != null && m.hasOwnProperty("metric_system")) {
                                    d.metric_system = m.metric_system;
                                    if (o.oneofs)
                                        d._metric_system = "metric_system";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DistanceView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.DistanceView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DistanceView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DistanceView;
                        })();
    
                        resources.DomainCategory = (function() {
    
                            /**
                             * Properties of a DomainCategory.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IDomainCategory
                             * @property {string|null} [resource_name] DomainCategory resource_name
                             * @property {string|null} [campaign] DomainCategory campaign
                             * @property {string|null} [category] DomainCategory category
                             * @property {string|null} [language_code] DomainCategory language_code
                             * @property {string|null} [domain] DomainCategory domain
                             * @property {number|null} [coverage_fraction] DomainCategory coverage_fraction
                             * @property {number|null} [category_rank] DomainCategory category_rank
                             * @property {boolean|null} [has_children] DomainCategory has_children
                             * @property {number|null} [recommended_cpc_bid_micros] DomainCategory recommended_cpc_bid_micros
                             */
    
                            /**
                             * Constructs a new DomainCategory.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a DomainCategory.
                             * @implements IDomainCategory
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IDomainCategory=} [p] Properties to set
                             */
                            function DomainCategory(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DomainCategory resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.resource_name = "";
    
                            /**
                             * DomainCategory campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.campaign = null;
    
                            /**
                             * DomainCategory category.
                             * @member {string|null|undefined} category
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.category = null;
    
                            /**
                             * DomainCategory language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.language_code = null;
    
                            /**
                             * DomainCategory domain.
                             * @member {string|null|undefined} domain
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.domain = null;
    
                            /**
                             * DomainCategory coverage_fraction.
                             * @member {number|null|undefined} coverage_fraction
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.coverage_fraction = null;
    
                            /**
                             * DomainCategory category_rank.
                             * @member {number|null|undefined} category_rank
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.category_rank = null;
    
                            /**
                             * DomainCategory has_children.
                             * @member {boolean|null|undefined} has_children
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.has_children = null;
    
                            /**
                             * DomainCategory recommended_cpc_bid_micros.
                             * @member {number|null|undefined} recommended_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            DomainCategory.prototype.recommended_cpc_bid_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DomainCategory _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DomainCategory _category.
                             * @member {"category"|undefined} _category
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_category", {
                                get: $util.oneOfGetter($oneOfFields = ["category"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DomainCategory _language_code.
                             * @member {"language_code"|undefined} _language_code
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_language_code", {
                                get: $util.oneOfGetter($oneOfFields = ["language_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DomainCategory _domain.
                             * @member {"domain"|undefined} _domain
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_domain", {
                                get: $util.oneOfGetter($oneOfFields = ["domain"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DomainCategory _coverage_fraction.
                             * @member {"coverage_fraction"|undefined} _coverage_fraction
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_coverage_fraction", {
                                get: $util.oneOfGetter($oneOfFields = ["coverage_fraction"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DomainCategory _category_rank.
                             * @member {"category_rank"|undefined} _category_rank
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_category_rank", {
                                get: $util.oneOfGetter($oneOfFields = ["category_rank"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DomainCategory _has_children.
                             * @member {"has_children"|undefined} _has_children
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_has_children", {
                                get: $util.oneOfGetter($oneOfFields = ["has_children"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DomainCategory _recommended_cpc_bid_micros.
                             * @member {"recommended_cpc_bid_micros"|undefined} _recommended_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             */
                            Object.defineProperty(DomainCategory.prototype, "_recommended_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["recommended_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DomainCategory instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDomainCategory=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.DomainCategory} DomainCategory instance
                             */
                            DomainCategory.create = function create(properties) {
                                return new DomainCategory(properties);
                            };
    
                            /**
                             * Encodes the specified DomainCategory message. Does not implicitly {@link google.ads.googleads.v11.resources.DomainCategory.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDomainCategory} m DomainCategory message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DomainCategory.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(82).string(m.campaign);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(90).string(m.category);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(98).string(m.language_code);
                                if (m.domain != null && Object.hasOwnProperty.call(m, "domain"))
                                    w.uint32(106).string(m.domain);
                                if (m.coverage_fraction != null && Object.hasOwnProperty.call(m, "coverage_fraction"))
                                    w.uint32(113).double(m.coverage_fraction);
                                if (m.category_rank != null && Object.hasOwnProperty.call(m, "category_rank"))
                                    w.uint32(120).int64(m.category_rank);
                                if (m.has_children != null && Object.hasOwnProperty.call(m, "has_children"))
                                    w.uint32(128).bool(m.has_children);
                                if (m.recommended_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "recommended_cpc_bid_micros"))
                                    w.uint32(136).int64(m.recommended_cpc_bid_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DomainCategory message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.DomainCategory.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDomainCategory} message DomainCategory message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DomainCategory.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DomainCategory message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.DomainCategory} DomainCategory
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DomainCategory.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.DomainCategory();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 10:
                                        m.campaign = r.string();
                                        break;
                                    case 11:
                                        m.category = r.string();
                                        break;
                                    case 12:
                                        m.language_code = r.string();
                                        break;
                                    case 13:
                                        m.domain = r.string();
                                        break;
                                    case 14:
                                        m.coverage_fraction = r.double();
                                        break;
                                    case 15:
                                        m.category_rank = r.int64();
                                        break;
                                    case 16:
                                        m.has_children = r.bool();
                                        break;
                                    case 17:
                                        m.recommended_cpc_bid_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DomainCategory message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.DomainCategory} DomainCategory
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DomainCategory.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DomainCategory message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DomainCategory.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    p._category = 1;
                                    if (!$util.isString(m.category))
                                        return "category: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    p._language_code = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.domain != null && m.hasOwnProperty("domain")) {
                                    p._domain = 1;
                                    if (!$util.isString(m.domain))
                                        return "domain: string expected";
                                }
                                if (m.coverage_fraction != null && m.hasOwnProperty("coverage_fraction")) {
                                    p._coverage_fraction = 1;
                                    if (typeof m.coverage_fraction !== "number")
                                        return "coverage_fraction: number expected";
                                }
                                if (m.category_rank != null && m.hasOwnProperty("category_rank")) {
                                    p._category_rank = 1;
                                    if (!$util.isInteger(m.category_rank) && !(m.category_rank && $util.isInteger(m.category_rank.low) && $util.isInteger(m.category_rank.high)))
                                        return "category_rank: integer|Long expected";
                                }
                                if (m.has_children != null && m.hasOwnProperty("has_children")) {
                                    p._has_children = 1;
                                    if (typeof m.has_children !== "boolean")
                                        return "has_children: boolean expected";
                                }
                                if (m.recommended_cpc_bid_micros != null && m.hasOwnProperty("recommended_cpc_bid_micros")) {
                                    p._recommended_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.recommended_cpc_bid_micros) && !(m.recommended_cpc_bid_micros && $util.isInteger(m.recommended_cpc_bid_micros.low) && $util.isInteger(m.recommended_cpc_bid_micros.high)))
                                        return "recommended_cpc_bid_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DomainCategory message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.DomainCategory} DomainCategory
                             */
                            DomainCategory.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.DomainCategory)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.DomainCategory();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.category != null) {
                                    m.category = String(d.category);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.domain != null) {
                                    m.domain = String(d.domain);
                                }
                                if (d.coverage_fraction != null) {
                                    m.coverage_fraction = Number(d.coverage_fraction);
                                }
                                if (d.category_rank != null) {
                                    if ($util.Long)
                                        (m.category_rank = $util.Long.fromValue(d.category_rank)).unsigned = false;
                                    else if (typeof d.category_rank === "string")
                                        m.category_rank = parseInt(d.category_rank, 10);
                                    else if (typeof d.category_rank === "number")
                                        m.category_rank = d.category_rank;
                                    else if (typeof d.category_rank === "object")
                                        m.category_rank = new $util.LongBits(d.category_rank.low >>> 0, d.category_rank.high >>> 0).toNumber();
                                }
                                if (d.has_children != null) {
                                    m.has_children = Boolean(d.has_children);
                                }
                                if (d.recommended_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.recommended_cpc_bid_micros = $util.Long.fromValue(d.recommended_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.recommended_cpc_bid_micros === "string")
                                        m.recommended_cpc_bid_micros = parseInt(d.recommended_cpc_bid_micros, 10);
                                    else if (typeof d.recommended_cpc_bid_micros === "number")
                                        m.recommended_cpc_bid_micros = d.recommended_cpc_bid_micros;
                                    else if (typeof d.recommended_cpc_bid_micros === "object")
                                        m.recommended_cpc_bid_micros = new $util.LongBits(d.recommended_cpc_bid_micros.low >>> 0, d.recommended_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DomainCategory message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @static
                             * @param {google.ads.googleads.v11.resources.DomainCategory} m DomainCategory
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DomainCategory.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = m.category;
                                    if (o.oneofs)
                                        d._category = "category";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d._language_code = "language_code";
                                }
                                if (m.domain != null && m.hasOwnProperty("domain")) {
                                    d.domain = m.domain;
                                    if (o.oneofs)
                                        d._domain = "domain";
                                }
                                if (m.coverage_fraction != null && m.hasOwnProperty("coverage_fraction")) {
                                    d.coverage_fraction = o.json && !isFinite(m.coverage_fraction) ? String(m.coverage_fraction) : m.coverage_fraction;
                                    if (o.oneofs)
                                        d._coverage_fraction = "coverage_fraction";
                                }
                                if (m.category_rank != null && m.hasOwnProperty("category_rank")) {
                                    if (typeof m.category_rank === "number")
                                        d.category_rank = o.longs === String ? String(m.category_rank) : m.category_rank;
                                    else
                                        d.category_rank = o.longs === String ? $util.Long.prototype.toString.call(m.category_rank) : o.longs === Number ? new $util.LongBits(m.category_rank.low >>> 0, m.category_rank.high >>> 0).toNumber() : m.category_rank;
                                    if (o.oneofs)
                                        d._category_rank = "category_rank";
                                }
                                if (m.has_children != null && m.hasOwnProperty("has_children")) {
                                    d.has_children = m.has_children;
                                    if (o.oneofs)
                                        d._has_children = "has_children";
                                }
                                if (m.recommended_cpc_bid_micros != null && m.hasOwnProperty("recommended_cpc_bid_micros")) {
                                    if (typeof m.recommended_cpc_bid_micros === "number")
                                        d.recommended_cpc_bid_micros = o.longs === String ? String(m.recommended_cpc_bid_micros) : m.recommended_cpc_bid_micros;
                                    else
                                        d.recommended_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.recommended_cpc_bid_micros.low >>> 0, m.recommended_cpc_bid_micros.high >>> 0).toNumber() : m.recommended_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._recommended_cpc_bid_micros = "recommended_cpc_bid_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DomainCategory to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.DomainCategory
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DomainCategory.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DomainCategory;
                        })();
    
                        resources.DynamicSearchAdsSearchTermView = (function() {
    
                            /**
                             * Properties of a DynamicSearchAdsSearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IDynamicSearchAdsSearchTermView
                             * @property {string|null} [resource_name] DynamicSearchAdsSearchTermView resource_name
                             * @property {string|null} [search_term] DynamicSearchAdsSearchTermView search_term
                             * @property {string|null} [headline] DynamicSearchAdsSearchTermView headline
                             * @property {string|null} [landing_page] DynamicSearchAdsSearchTermView landing_page
                             * @property {string|null} [page_url] DynamicSearchAdsSearchTermView page_url
                             * @property {boolean|null} [has_negative_keyword] DynamicSearchAdsSearchTermView has_negative_keyword
                             * @property {boolean|null} [has_matching_keyword] DynamicSearchAdsSearchTermView has_matching_keyword
                             * @property {boolean|null} [has_negative_url] DynamicSearchAdsSearchTermView has_negative_url
                             */
    
                            /**
                             * Constructs a new DynamicSearchAdsSearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a DynamicSearchAdsSearchTermView.
                             * @implements IDynamicSearchAdsSearchTermView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IDynamicSearchAdsSearchTermView=} [p] Properties to set
                             */
                            function DynamicSearchAdsSearchTermView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DynamicSearchAdsSearchTermView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.resource_name = "";
    
                            /**
                             * DynamicSearchAdsSearchTermView search_term.
                             * @member {string|null|undefined} search_term
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.search_term = null;
    
                            /**
                             * DynamicSearchAdsSearchTermView headline.
                             * @member {string|null|undefined} headline
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.headline = null;
    
                            /**
                             * DynamicSearchAdsSearchTermView landing_page.
                             * @member {string|null|undefined} landing_page
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.landing_page = null;
    
                            /**
                             * DynamicSearchAdsSearchTermView page_url.
                             * @member {string|null|undefined} page_url
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.page_url = null;
    
                            /**
                             * DynamicSearchAdsSearchTermView has_negative_keyword.
                             * @member {boolean|null|undefined} has_negative_keyword
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.has_negative_keyword = null;
    
                            /**
                             * DynamicSearchAdsSearchTermView has_matching_keyword.
                             * @member {boolean|null|undefined} has_matching_keyword
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.has_matching_keyword = null;
    
                            /**
                             * DynamicSearchAdsSearchTermView has_negative_url.
                             * @member {boolean|null|undefined} has_negative_url
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            DynamicSearchAdsSearchTermView.prototype.has_negative_url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DynamicSearchAdsSearchTermView _search_term.
                             * @member {"search_term"|undefined} _search_term
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            Object.defineProperty(DynamicSearchAdsSearchTermView.prototype, "_search_term", {
                                get: $util.oneOfGetter($oneOfFields = ["search_term"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DynamicSearchAdsSearchTermView _headline.
                             * @member {"headline"|undefined} _headline
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            Object.defineProperty(DynamicSearchAdsSearchTermView.prototype, "_headline", {
                                get: $util.oneOfGetter($oneOfFields = ["headline"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DynamicSearchAdsSearchTermView _landing_page.
                             * @member {"landing_page"|undefined} _landing_page
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            Object.defineProperty(DynamicSearchAdsSearchTermView.prototype, "_landing_page", {
                                get: $util.oneOfGetter($oneOfFields = ["landing_page"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DynamicSearchAdsSearchTermView _page_url.
                             * @member {"page_url"|undefined} _page_url
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            Object.defineProperty(DynamicSearchAdsSearchTermView.prototype, "_page_url", {
                                get: $util.oneOfGetter($oneOfFields = ["page_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DynamicSearchAdsSearchTermView _has_negative_keyword.
                             * @member {"has_negative_keyword"|undefined} _has_negative_keyword
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            Object.defineProperty(DynamicSearchAdsSearchTermView.prototype, "_has_negative_keyword", {
                                get: $util.oneOfGetter($oneOfFields = ["has_negative_keyword"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DynamicSearchAdsSearchTermView _has_matching_keyword.
                             * @member {"has_matching_keyword"|undefined} _has_matching_keyword
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            Object.defineProperty(DynamicSearchAdsSearchTermView.prototype, "_has_matching_keyword", {
                                get: $util.oneOfGetter($oneOfFields = ["has_matching_keyword"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * DynamicSearchAdsSearchTermView _has_negative_url.
                             * @member {"has_negative_url"|undefined} _has_negative_url
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             */
                            Object.defineProperty(DynamicSearchAdsSearchTermView.prototype, "_has_negative_url", {
                                get: $util.oneOfGetter($oneOfFields = ["has_negative_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DynamicSearchAdsSearchTermView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDynamicSearchAdsSearchTermView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView} DynamicSearchAdsSearchTermView instance
                             */
                            DynamicSearchAdsSearchTermView.create = function create(properties) {
                                return new DynamicSearchAdsSearchTermView(properties);
                            };
    
                            /**
                             * Encodes the specified DynamicSearchAdsSearchTermView message. Does not implicitly {@link google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDynamicSearchAdsSearchTermView} m DynamicSearchAdsSearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicSearchAdsSearchTermView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.search_term != null && Object.hasOwnProperty.call(m, "search_term"))
                                    w.uint32(74).string(m.search_term);
                                if (m.headline != null && Object.hasOwnProperty.call(m, "headline"))
                                    w.uint32(82).string(m.headline);
                                if (m.landing_page != null && Object.hasOwnProperty.call(m, "landing_page"))
                                    w.uint32(90).string(m.landing_page);
                                if (m.page_url != null && Object.hasOwnProperty.call(m, "page_url"))
                                    w.uint32(98).string(m.page_url);
                                if (m.has_negative_keyword != null && Object.hasOwnProperty.call(m, "has_negative_keyword"))
                                    w.uint32(104).bool(m.has_negative_keyword);
                                if (m.has_matching_keyword != null && Object.hasOwnProperty.call(m, "has_matching_keyword"))
                                    w.uint32(112).bool(m.has_matching_keyword);
                                if (m.has_negative_url != null && Object.hasOwnProperty.call(m, "has_negative_url"))
                                    w.uint32(120).bool(m.has_negative_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified DynamicSearchAdsSearchTermView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IDynamicSearchAdsSearchTermView} message DynamicSearchAdsSearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DynamicSearchAdsSearchTermView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DynamicSearchAdsSearchTermView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView} DynamicSearchAdsSearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicSearchAdsSearchTermView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.search_term = r.string();
                                        break;
                                    case 10:
                                        m.headline = r.string();
                                        break;
                                    case 11:
                                        m.landing_page = r.string();
                                        break;
                                    case 12:
                                        m.page_url = r.string();
                                        break;
                                    case 13:
                                        m.has_negative_keyword = r.bool();
                                        break;
                                    case 14:
                                        m.has_matching_keyword = r.bool();
                                        break;
                                    case 15:
                                        m.has_negative_url = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DynamicSearchAdsSearchTermView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView} DynamicSearchAdsSearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DynamicSearchAdsSearchTermView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DynamicSearchAdsSearchTermView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DynamicSearchAdsSearchTermView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    p._search_term = 1;
                                    if (!$util.isString(m.search_term))
                                        return "search_term: string expected";
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    p._headline = 1;
                                    if (!$util.isString(m.headline))
                                        return "headline: string expected";
                                }
                                if (m.landing_page != null && m.hasOwnProperty("landing_page")) {
                                    p._landing_page = 1;
                                    if (!$util.isString(m.landing_page))
                                        return "landing_page: string expected";
                                }
                                if (m.page_url != null && m.hasOwnProperty("page_url")) {
                                    p._page_url = 1;
                                    if (!$util.isString(m.page_url))
                                        return "page_url: string expected";
                                }
                                if (m.has_negative_keyword != null && m.hasOwnProperty("has_negative_keyword")) {
                                    p._has_negative_keyword = 1;
                                    if (typeof m.has_negative_keyword !== "boolean")
                                        return "has_negative_keyword: boolean expected";
                                }
                                if (m.has_matching_keyword != null && m.hasOwnProperty("has_matching_keyword")) {
                                    p._has_matching_keyword = 1;
                                    if (typeof m.has_matching_keyword !== "boolean")
                                        return "has_matching_keyword: boolean expected";
                                }
                                if (m.has_negative_url != null && m.hasOwnProperty("has_negative_url")) {
                                    p._has_negative_url = 1;
                                    if (typeof m.has_negative_url !== "boolean")
                                        return "has_negative_url: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DynamicSearchAdsSearchTermView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView} DynamicSearchAdsSearchTermView
                             */
                            DynamicSearchAdsSearchTermView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.search_term != null) {
                                    m.search_term = String(d.search_term);
                                }
                                if (d.headline != null) {
                                    m.headline = String(d.headline);
                                }
                                if (d.landing_page != null) {
                                    m.landing_page = String(d.landing_page);
                                }
                                if (d.page_url != null) {
                                    m.page_url = String(d.page_url);
                                }
                                if (d.has_negative_keyword != null) {
                                    m.has_negative_keyword = Boolean(d.has_negative_keyword);
                                }
                                if (d.has_matching_keyword != null) {
                                    m.has_matching_keyword = Boolean(d.has_matching_keyword);
                                }
                                if (d.has_negative_url != null) {
                                    m.has_negative_url = Boolean(d.has_negative_url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DynamicSearchAdsSearchTermView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView} m DynamicSearchAdsSearchTermView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DynamicSearchAdsSearchTermView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    d.search_term = m.search_term;
                                    if (o.oneofs)
                                        d._search_term = "search_term";
                                }
                                if (m.headline != null && m.hasOwnProperty("headline")) {
                                    d.headline = m.headline;
                                    if (o.oneofs)
                                        d._headline = "headline";
                                }
                                if (m.landing_page != null && m.hasOwnProperty("landing_page")) {
                                    d.landing_page = m.landing_page;
                                    if (o.oneofs)
                                        d._landing_page = "landing_page";
                                }
                                if (m.page_url != null && m.hasOwnProperty("page_url")) {
                                    d.page_url = m.page_url;
                                    if (o.oneofs)
                                        d._page_url = "page_url";
                                }
                                if (m.has_negative_keyword != null && m.hasOwnProperty("has_negative_keyword")) {
                                    d.has_negative_keyword = m.has_negative_keyword;
                                    if (o.oneofs)
                                        d._has_negative_keyword = "has_negative_keyword";
                                }
                                if (m.has_matching_keyword != null && m.hasOwnProperty("has_matching_keyword")) {
                                    d.has_matching_keyword = m.has_matching_keyword;
                                    if (o.oneofs)
                                        d._has_matching_keyword = "has_matching_keyword";
                                }
                                if (m.has_negative_url != null && m.hasOwnProperty("has_negative_url")) {
                                    d.has_negative_url = m.has_negative_url;
                                    if (o.oneofs)
                                        d._has_negative_url = "has_negative_url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DynamicSearchAdsSearchTermView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DynamicSearchAdsSearchTermView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return DynamicSearchAdsSearchTermView;
                        })();
    
                        resources.ExpandedLandingPageView = (function() {
    
                            /**
                             * Properties of an ExpandedLandingPageView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IExpandedLandingPageView
                             * @property {string|null} [resource_name] ExpandedLandingPageView resource_name
                             * @property {string|null} [expanded_final_url] ExpandedLandingPageView expanded_final_url
                             */
    
                            /**
                             * Constructs a new ExpandedLandingPageView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an ExpandedLandingPageView.
                             * @implements IExpandedLandingPageView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IExpandedLandingPageView=} [p] Properties to set
                             */
                            function ExpandedLandingPageView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExpandedLandingPageView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @instance
                             */
                            ExpandedLandingPageView.prototype.resource_name = "";
    
                            /**
                             * ExpandedLandingPageView expanded_final_url.
                             * @member {string|null|undefined} expanded_final_url
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @instance
                             */
                            ExpandedLandingPageView.prototype.expanded_final_url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExpandedLandingPageView _expanded_final_url.
                             * @member {"expanded_final_url"|undefined} _expanded_final_url
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @instance
                             */
                            Object.defineProperty(ExpandedLandingPageView.prototype, "_expanded_final_url", {
                                get: $util.oneOfGetter($oneOfFields = ["expanded_final_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExpandedLandingPageView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExpandedLandingPageView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ExpandedLandingPageView} ExpandedLandingPageView instance
                             */
                            ExpandedLandingPageView.create = function create(properties) {
                                return new ExpandedLandingPageView(properties);
                            };
    
                            /**
                             * Encodes the specified ExpandedLandingPageView message. Does not implicitly {@link google.ads.googleads.v11.resources.ExpandedLandingPageView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExpandedLandingPageView} m ExpandedLandingPageView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpandedLandingPageView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.expanded_final_url != null && Object.hasOwnProperty.call(m, "expanded_final_url"))
                                    w.uint32(26).string(m.expanded_final_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExpandedLandingPageView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ExpandedLandingPageView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExpandedLandingPageView} message ExpandedLandingPageView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExpandedLandingPageView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExpandedLandingPageView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ExpandedLandingPageView} ExpandedLandingPageView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpandedLandingPageView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ExpandedLandingPageView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.expanded_final_url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExpandedLandingPageView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ExpandedLandingPageView} ExpandedLandingPageView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExpandedLandingPageView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExpandedLandingPageView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExpandedLandingPageView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.expanded_final_url != null && m.hasOwnProperty("expanded_final_url")) {
                                    p._expanded_final_url = 1;
                                    if (!$util.isString(m.expanded_final_url))
                                        return "expanded_final_url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExpandedLandingPageView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ExpandedLandingPageView} ExpandedLandingPageView
                             */
                            ExpandedLandingPageView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ExpandedLandingPageView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ExpandedLandingPageView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.expanded_final_url != null) {
                                    m.expanded_final_url = String(d.expanded_final_url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExpandedLandingPageView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ExpandedLandingPageView} m ExpandedLandingPageView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExpandedLandingPageView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.expanded_final_url != null && m.hasOwnProperty("expanded_final_url")) {
                                    d.expanded_final_url = m.expanded_final_url;
                                    if (o.oneofs)
                                        d._expanded_final_url = "expanded_final_url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExpandedLandingPageView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ExpandedLandingPageView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExpandedLandingPageView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExpandedLandingPageView;
                        })();
    
                        resources.Experiment = (function() {
    
                            /**
                             * Properties of an Experiment.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IExperiment
                             * @property {string|null} [resource_name] Experiment resource_name
                             * @property {number|null} [experiment_id] Experiment experiment_id
                             * @property {string|null} [name] Experiment name
                             * @property {string|null} [description] Experiment description
                             * @property {string|null} [suffix] Experiment suffix
                             * @property {google.ads.googleads.v11.enums.ExperimentTypeEnum.ExperimentType|null} [type] Experiment type
                             * @property {google.ads.googleads.v11.enums.ExperimentStatusEnum.ExperimentStatus|null} [status] Experiment status
                             * @property {string|null} [start_date] Experiment start_date
                             * @property {string|null} [end_date] Experiment end_date
                             * @property {Array.<google.ads.googleads.v11.common.IMetricGoal>|null} [goals] Experiment goals
                             * @property {string|null} [long_running_operation] Experiment long_running_operation
                             * @property {google.ads.googleads.v11.enums.AsyncActionStatusEnum.AsyncActionStatus|null} [promote_status] Experiment promote_status
                             */
    
                            /**
                             * Constructs a new Experiment.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an Experiment.
                             * @implements IExperiment
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IExperiment=} [p] Properties to set
                             */
                            function Experiment(p) {
                                this.goals = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Experiment resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.resource_name = "";
    
                            /**
                             * Experiment experiment_id.
                             * @member {number|null|undefined} experiment_id
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.experiment_id = null;
    
                            /**
                             * Experiment name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.name = "";
    
                            /**
                             * Experiment description.
                             * @member {string} description
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.description = "";
    
                            /**
                             * Experiment suffix.
                             * @member {string} suffix
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.suffix = "";
    
                            /**
                             * Experiment type.
                             * @member {google.ads.googleads.v11.enums.ExperimentTypeEnum.ExperimentType} type
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.type = 0;
    
                            /**
                             * Experiment status.
                             * @member {google.ads.googleads.v11.enums.ExperimentStatusEnum.ExperimentStatus} status
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.status = 0;
    
                            /**
                             * Experiment start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.start_date = null;
    
                            /**
                             * Experiment end_date.
                             * @member {string|null|undefined} end_date
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.end_date = null;
    
                            /**
                             * Experiment goals.
                             * @member {Array.<google.ads.googleads.v11.common.IMetricGoal>} goals
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.goals = $util.emptyArray;
    
                            /**
                             * Experiment long_running_operation.
                             * @member {string|null|undefined} long_running_operation
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.long_running_operation = null;
    
                            /**
                             * Experiment promote_status.
                             * @member {google.ads.googleads.v11.enums.AsyncActionStatusEnum.AsyncActionStatus} promote_status
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Experiment.prototype.promote_status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Experiment _experiment_id.
                             * @member {"experiment_id"|undefined} _experiment_id
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Object.defineProperty(Experiment.prototype, "_experiment_id", {
                                get: $util.oneOfGetter($oneOfFields = ["experiment_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Experiment _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Object.defineProperty(Experiment.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Experiment _end_date.
                             * @member {"end_date"|undefined} _end_date
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Object.defineProperty(Experiment.prototype, "_end_date", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Experiment _long_running_operation.
                             * @member {"long_running_operation"|undefined} _long_running_operation
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             */
                            Object.defineProperty(Experiment.prototype, "_long_running_operation", {
                                get: $util.oneOfGetter($oneOfFields = ["long_running_operation"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Experiment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExperiment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Experiment} Experiment instance
                             */
                            Experiment.create = function create(properties) {
                                return new Experiment(properties);
                            };
    
                            /**
                             * Encodes the specified Experiment message. Does not implicitly {@link google.ads.googleads.v11.resources.Experiment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExperiment} m Experiment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Experiment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.experiment_id != null && Object.hasOwnProperty.call(m, "experiment_id"))
                                    w.uint32(72).int64(m.experiment_id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(82).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(90).string(m.description);
                                if (m.suffix != null && Object.hasOwnProperty.call(m, "suffix"))
                                    w.uint32(98).string(m.suffix);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(104).int32(m.type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(112).int32(m.status);
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(122).string(m.start_date);
                                if (m.end_date != null && Object.hasOwnProperty.call(m, "end_date"))
                                    w.uint32(130).string(m.end_date);
                                if (m.goals != null && m.goals.length) {
                                    for (var i = 0; i < m.goals.length; ++i)
                                        $root.google.ads.googleads.v11.common.MetricGoal.encode(m.goals[i], w.uint32(138).fork()).ldelim();
                                }
                                if (m.long_running_operation != null && Object.hasOwnProperty.call(m, "long_running_operation"))
                                    w.uint32(146).string(m.long_running_operation);
                                if (m.promote_status != null && Object.hasOwnProperty.call(m, "promote_status"))
                                    w.uint32(152).int32(m.promote_status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Experiment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Experiment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExperiment} message Experiment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Experiment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Experiment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Experiment} Experiment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Experiment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Experiment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.experiment_id = r.int64();
                                        break;
                                    case 10:
                                        m.name = r.string();
                                        break;
                                    case 11:
                                        m.description = r.string();
                                        break;
                                    case 12:
                                        m.suffix = r.string();
                                        break;
                                    case 13:
                                        m.type = r.int32();
                                        break;
                                    case 14:
                                        m.status = r.int32();
                                        break;
                                    case 15:
                                        m.start_date = r.string();
                                        break;
                                    case 16:
                                        m.end_date = r.string();
                                        break;
                                    case 17:
                                        if (!(m.goals && m.goals.length))
                                            m.goals = [];
                                        m.goals.push($root.google.ads.googleads.v11.common.MetricGoal.decode(r, r.uint32()));
                                        break;
                                    case 18:
                                        m.long_running_operation = r.string();
                                        break;
                                    case 19:
                                        m.promote_status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an Experiment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Experiment} Experiment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Experiment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Experiment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Experiment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.experiment_id != null && m.hasOwnProperty("experiment_id")) {
                                    p._experiment_id = 1;
                                    if (!$util.isInteger(m.experiment_id) && !(m.experiment_id && $util.isInteger(m.experiment_id.low) && $util.isInteger(m.experiment_id.high)))
                                        return "experiment_id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.suffix != null && m.hasOwnProperty("suffix")) {
                                    if (!$util.isString(m.suffix))
                                        return "suffix: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    p._end_date = 1;
                                    if (!$util.isString(m.end_date))
                                        return "end_date: string expected";
                                }
                                if (m.goals != null && m.hasOwnProperty("goals")) {
                                    if (!Array.isArray(m.goals))
                                        return "goals: array expected";
                                    for (var i = 0; i < m.goals.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.MetricGoal.verify(m.goals[i]);
                                            if (e)
                                                return "goals." + e;
                                        }
                                    }
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    p._long_running_operation = 1;
                                    if (!$util.isString(m.long_running_operation))
                                        return "long_running_operation: string expected";
                                }
                                if (m.promote_status != null && m.hasOwnProperty("promote_status")) {
                                    switch (m.promote_status) {
                                    default:
                                        return "promote_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Experiment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Experiment} Experiment
                             */
                            Experiment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Experiment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Experiment();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.experiment_id != null) {
                                    if ($util.Long)
                                        (m.experiment_id = $util.Long.fromValue(d.experiment_id)).unsigned = false;
                                    else if (typeof d.experiment_id === "string")
                                        m.experiment_id = parseInt(d.experiment_id, 10);
                                    else if (typeof d.experiment_id === "number")
                                        m.experiment_id = d.experiment_id;
                                    else if (typeof d.experiment_id === "object")
                                        m.experiment_id = new $util.LongBits(d.experiment_id.low >>> 0, d.experiment_id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                if (d.suffix != null) {
                                    m.suffix = String(d.suffix);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "DISPLAY_AND_VIDEO_360":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "AD_VARIATION":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "YOUTUBE_CUSTOM":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "DISPLAY_CUSTOM":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "SEARCH_CUSTOM":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "DISPLAY_AUTOMATED_BIDDING_STRATEGY":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "SEARCH_AUTOMATED_BIDDING_STRATEGY":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "SHOPPING_AUTOMATED_BIDDING_STRATEGY":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "SMART_MATCHING":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "HOTEL_CUSTOM":
                                case 12:
                                    m.type = 12;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "HALTED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "PROMOTED":
                                case 5:
                                    m.status = 5;
                                    break;
                                case "SETUP":
                                case 6:
                                    m.status = 6;
                                    break;
                                case "INITIATED":
                                case 7:
                                    m.status = 7;
                                    break;
                                case "GRADUATED":
                                case 8:
                                    m.status = 8;
                                    break;
                                }
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.end_date != null) {
                                    m.end_date = String(d.end_date);
                                }
                                if (d.goals) {
                                    if (!Array.isArray(d.goals))
                                        throw TypeError(".google.ads.googleads.v11.resources.Experiment.goals: array expected");
                                    m.goals = [];
                                    for (var i = 0; i < d.goals.length; ++i) {
                                        if (typeof d.goals[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Experiment.goals: object expected");
                                        m.goals[i] = $root.google.ads.googleads.v11.common.MetricGoal.fromObject(d.goals[i]);
                                    }
                                }
                                if (d.long_running_operation != null) {
                                    m.long_running_operation = String(d.long_running_operation);
                                }
                                switch (d.promote_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.promote_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.promote_status = 1;
                                    break;
                                case "NOT_STARTED":
                                case 2:
                                    m.promote_status = 2;
                                    break;
                                case "IN_PROGRESS":
                                case 3:
                                    m.promote_status = 3;
                                    break;
                                case "COMPLETED":
                                case 4:
                                    m.promote_status = 4;
                                    break;
                                case "FAILED":
                                case 5:
                                    m.promote_status = 5;
                                    break;
                                case "COMPLETED_WITH_WARNING":
                                case 6:
                                    m.promote_status = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an Experiment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @static
                             * @param {google.ads.googleads.v11.resources.Experiment} m Experiment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Experiment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.goals = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.name = "";
                                    d.description = "";
                                    d.suffix = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.promote_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.experiment_id != null && m.hasOwnProperty("experiment_id")) {
                                    if (typeof m.experiment_id === "number")
                                        d.experiment_id = o.longs === String ? String(m.experiment_id) : m.experiment_id;
                                    else
                                        d.experiment_id = o.longs === String ? $util.Long.prototype.toString.call(m.experiment_id) : o.longs === Number ? new $util.LongBits(m.experiment_id.low >>> 0, m.experiment_id.high >>> 0).toNumber() : m.experiment_id;
                                    if (o.oneofs)
                                        d._experiment_id = "experiment_id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                }
                                if (m.suffix != null && m.hasOwnProperty("suffix")) {
                                    d.suffix = m.suffix;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.ExperimentTypeEnum.ExperimentType[m.type] : m.type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ExperimentStatusEnum.ExperimentStatus[m.status] : m.status;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.end_date != null && m.hasOwnProperty("end_date")) {
                                    d.end_date = m.end_date;
                                    if (o.oneofs)
                                        d._end_date = "end_date";
                                }
                                if (m.goals && m.goals.length) {
                                    d.goals = [];
                                    for (var j = 0; j < m.goals.length; ++j) {
                                        d.goals[j] = $root.google.ads.googleads.v11.common.MetricGoal.toObject(m.goals[j], o);
                                    }
                                }
                                if (m.long_running_operation != null && m.hasOwnProperty("long_running_operation")) {
                                    d.long_running_operation = m.long_running_operation;
                                    if (o.oneofs)
                                        d._long_running_operation = "long_running_operation";
                                }
                                if (m.promote_status != null && m.hasOwnProperty("promote_status")) {
                                    d.promote_status = o.enums === String ? $root.google.ads.googleads.v11.enums.AsyncActionStatusEnum.AsyncActionStatus[m.promote_status] : m.promote_status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Experiment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Experiment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Experiment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Experiment;
                        })();
    
                        resources.ExperimentArm = (function() {
    
                            /**
                             * Properties of an ExperimentArm.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IExperimentArm
                             * @property {string|null} [resource_name] ExperimentArm resource_name
                             * @property {string|null} [trial] ExperimentArm trial
                             * @property {string|null} [name] ExperimentArm name
                             * @property {boolean|null} [control] ExperimentArm control
                             * @property {number|null} [traffic_split] ExperimentArm traffic_split
                             * @property {Array.<string>|null} [campaigns] ExperimentArm campaigns
                             * @property {Array.<string>|null} [in_design_campaigns] ExperimentArm in_design_campaigns
                             */
    
                            /**
                             * Constructs a new ExperimentArm.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an ExperimentArm.
                             * @implements IExperimentArm
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IExperimentArm=} [p] Properties to set
                             */
                            function ExperimentArm(p) {
                                this.campaigns = [];
                                this.in_design_campaigns = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExperimentArm resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             */
                            ExperimentArm.prototype.resource_name = "";
    
                            /**
                             * ExperimentArm trial.
                             * @member {string} trial
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             */
                            ExperimentArm.prototype.trial = "";
    
                            /**
                             * ExperimentArm name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             */
                            ExperimentArm.prototype.name = "";
    
                            /**
                             * ExperimentArm control.
                             * @member {boolean} control
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             */
                            ExperimentArm.prototype.control = false;
    
                            /**
                             * ExperimentArm traffic_split.
                             * @member {number} traffic_split
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             */
                            ExperimentArm.prototype.traffic_split = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * ExperimentArm campaigns.
                             * @member {Array.<string>} campaigns
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             */
                            ExperimentArm.prototype.campaigns = $util.emptyArray;
    
                            /**
                             * ExperimentArm in_design_campaigns.
                             * @member {Array.<string>} in_design_campaigns
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             */
                            ExperimentArm.prototype.in_design_campaigns = $util.emptyArray;
    
                            /**
                             * Creates a new ExperimentArm instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExperimentArm=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ExperimentArm} ExperimentArm instance
                             */
                            ExperimentArm.create = function create(properties) {
                                return new ExperimentArm(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentArm message. Does not implicitly {@link google.ads.googleads.v11.resources.ExperimentArm.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExperimentArm} m ExperimentArm message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentArm.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.trial != null && Object.hasOwnProperty.call(m, "trial"))
                                    w.uint32(18).string(m.trial);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.control != null && Object.hasOwnProperty.call(m, "control"))
                                    w.uint32(32).bool(m.control);
                                if (m.traffic_split != null && Object.hasOwnProperty.call(m, "traffic_split"))
                                    w.uint32(40).int64(m.traffic_split);
                                if (m.campaigns != null && m.campaigns.length) {
                                    for (var i = 0; i < m.campaigns.length; ++i)
                                        w.uint32(50).string(m.campaigns[i]);
                                }
                                if (m.in_design_campaigns != null && m.in_design_campaigns.length) {
                                    for (var i = 0; i < m.in_design_campaigns.length; ++i)
                                        w.uint32(58).string(m.in_design_campaigns[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentArm message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ExperimentArm.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExperimentArm} message ExperimentArm message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentArm.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentArm message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ExperimentArm} ExperimentArm
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentArm.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ExperimentArm();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.trial = r.string();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.control = r.bool();
                                        break;
                                    case 5:
                                        m.traffic_split = r.int64();
                                        break;
                                    case 6:
                                        if (!(m.campaigns && m.campaigns.length))
                                            m.campaigns = [];
                                        m.campaigns.push(r.string());
                                        break;
                                    case 7:
                                        if (!(m.in_design_campaigns && m.in_design_campaigns.length))
                                            m.in_design_campaigns = [];
                                        m.in_design_campaigns.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentArm message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ExperimentArm} ExperimentArm
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentArm.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentArm message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentArm.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.trial != null && m.hasOwnProperty("trial")) {
                                    if (!$util.isString(m.trial))
                                        return "trial: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.control != null && m.hasOwnProperty("control")) {
                                    if (typeof m.control !== "boolean")
                                        return "control: boolean expected";
                                }
                                if (m.traffic_split != null && m.hasOwnProperty("traffic_split")) {
                                    if (!$util.isInteger(m.traffic_split) && !(m.traffic_split && $util.isInteger(m.traffic_split.low) && $util.isInteger(m.traffic_split.high)))
                                        return "traffic_split: integer|Long expected";
                                }
                                if (m.campaigns != null && m.hasOwnProperty("campaigns")) {
                                    if (!Array.isArray(m.campaigns))
                                        return "campaigns: array expected";
                                    for (var i = 0; i < m.campaigns.length; ++i) {
                                        if (!$util.isString(m.campaigns[i]))
                                            return "campaigns: string[] expected";
                                    }
                                }
                                if (m.in_design_campaigns != null && m.hasOwnProperty("in_design_campaigns")) {
                                    if (!Array.isArray(m.in_design_campaigns))
                                        return "in_design_campaigns: array expected";
                                    for (var i = 0; i < m.in_design_campaigns.length; ++i) {
                                        if (!$util.isString(m.in_design_campaigns[i]))
                                            return "in_design_campaigns: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentArm message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ExperimentArm} ExperimentArm
                             */
                            ExperimentArm.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ExperimentArm)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ExperimentArm();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.trial != null) {
                                    m.trial = String(d.trial);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.control != null) {
                                    m.control = Boolean(d.control);
                                }
                                if (d.traffic_split != null) {
                                    if ($util.Long)
                                        (m.traffic_split = $util.Long.fromValue(d.traffic_split)).unsigned = false;
                                    else if (typeof d.traffic_split === "string")
                                        m.traffic_split = parseInt(d.traffic_split, 10);
                                    else if (typeof d.traffic_split === "number")
                                        m.traffic_split = d.traffic_split;
                                    else if (typeof d.traffic_split === "object")
                                        m.traffic_split = new $util.LongBits(d.traffic_split.low >>> 0, d.traffic_split.high >>> 0).toNumber();
                                }
                                if (d.campaigns) {
                                    if (!Array.isArray(d.campaigns))
                                        throw TypeError(".google.ads.googleads.v11.resources.ExperimentArm.campaigns: array expected");
                                    m.campaigns = [];
                                    for (var i = 0; i < d.campaigns.length; ++i) {
                                        m.campaigns[i] = String(d.campaigns[i]);
                                    }
                                }
                                if (d.in_design_campaigns) {
                                    if (!Array.isArray(d.in_design_campaigns))
                                        throw TypeError(".google.ads.googleads.v11.resources.ExperimentArm.in_design_campaigns: array expected");
                                    m.in_design_campaigns = [];
                                    for (var i = 0; i < d.in_design_campaigns.length; ++i) {
                                        m.in_design_campaigns[i] = String(d.in_design_campaigns[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExperimentArm message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @static
                             * @param {google.ads.googleads.v11.resources.ExperimentArm} m ExperimentArm
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentArm.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.campaigns = [];
                                    d.in_design_campaigns = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.trial = "";
                                    d.name = "";
                                    d.control = false;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.traffic_split = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.traffic_split = o.longs === String ? "0" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.trial != null && m.hasOwnProperty("trial")) {
                                    d.trial = m.trial;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.control != null && m.hasOwnProperty("control")) {
                                    d.control = m.control;
                                }
                                if (m.traffic_split != null && m.hasOwnProperty("traffic_split")) {
                                    if (typeof m.traffic_split === "number")
                                        d.traffic_split = o.longs === String ? String(m.traffic_split) : m.traffic_split;
                                    else
                                        d.traffic_split = o.longs === String ? $util.Long.prototype.toString.call(m.traffic_split) : o.longs === Number ? new $util.LongBits(m.traffic_split.low >>> 0, m.traffic_split.high >>> 0).toNumber() : m.traffic_split;
                                }
                                if (m.campaigns && m.campaigns.length) {
                                    d.campaigns = [];
                                    for (var j = 0; j < m.campaigns.length; ++j) {
                                        d.campaigns[j] = m.campaigns[j];
                                    }
                                }
                                if (m.in_design_campaigns && m.in_design_campaigns.length) {
                                    d.in_design_campaigns = [];
                                    for (var j = 0; j < m.in_design_campaigns.length; ++j) {
                                        d.in_design_campaigns[j] = m.in_design_campaigns[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExperimentArm to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ExperimentArm
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentArm.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExperimentArm;
                        })();
    
                        resources.ExtensionFeedItem = (function() {
    
                            /**
                             * Properties of an ExtensionFeedItem.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IExtensionFeedItem
                             * @property {string|null} [resource_name] ExtensionFeedItem resource_name
                             * @property {number|null} [id] ExtensionFeedItem id
                             * @property {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType|null} [extension_type] ExtensionFeedItem extension_type
                             * @property {string|null} [start_date_time] ExtensionFeedItem start_date_time
                             * @property {string|null} [end_date_time] ExtensionFeedItem end_date_time
                             * @property {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>|null} [ad_schedules] ExtensionFeedItem ad_schedules
                             * @property {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.FeedItemTargetDevice|null} [device] ExtensionFeedItem device
                             * @property {string|null} [targeted_geo_target_constant] ExtensionFeedItem targeted_geo_target_constant
                             * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [targeted_keyword] ExtensionFeedItem targeted_keyword
                             * @property {google.ads.googleads.v11.enums.FeedItemStatusEnum.FeedItemStatus|null} [status] ExtensionFeedItem status
                             * @property {google.ads.googleads.v11.common.ISitelinkFeedItem|null} [sitelink_feed_item] ExtensionFeedItem sitelink_feed_item
                             * @property {google.ads.googleads.v11.common.IStructuredSnippetFeedItem|null} [structured_snippet_feed_item] ExtensionFeedItem structured_snippet_feed_item
                             * @property {google.ads.googleads.v11.common.IAppFeedItem|null} [app_feed_item] ExtensionFeedItem app_feed_item
                             * @property {google.ads.googleads.v11.common.ICallFeedItem|null} [call_feed_item] ExtensionFeedItem call_feed_item
                             * @property {google.ads.googleads.v11.common.ICalloutFeedItem|null} [callout_feed_item] ExtensionFeedItem callout_feed_item
                             * @property {google.ads.googleads.v11.common.ITextMessageFeedItem|null} [text_message_feed_item] ExtensionFeedItem text_message_feed_item
                             * @property {google.ads.googleads.v11.common.IPriceFeedItem|null} [price_feed_item] ExtensionFeedItem price_feed_item
                             * @property {google.ads.googleads.v11.common.IPromotionFeedItem|null} [promotion_feed_item] ExtensionFeedItem promotion_feed_item
                             * @property {google.ads.googleads.v11.common.ILocationFeedItem|null} [location_feed_item] ExtensionFeedItem location_feed_item
                             * @property {google.ads.googleads.v11.common.IAffiliateLocationFeedItem|null} [affiliate_location_feed_item] ExtensionFeedItem affiliate_location_feed_item
                             * @property {google.ads.googleads.v11.common.IHotelCalloutFeedItem|null} [hotel_callout_feed_item] ExtensionFeedItem hotel_callout_feed_item
                             * @property {google.ads.googleads.v11.common.IImageFeedItem|null} [image_feed_item] ExtensionFeedItem image_feed_item
                             * @property {string|null} [targeted_campaign] ExtensionFeedItem targeted_campaign
                             * @property {string|null} [targeted_ad_group] ExtensionFeedItem targeted_ad_group
                             */
    
                            /**
                             * Constructs a new ExtensionFeedItem.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an ExtensionFeedItem.
                             * @implements IExtensionFeedItem
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IExtensionFeedItem=} [p] Properties to set
                             */
                            function ExtensionFeedItem(p) {
                                this.ad_schedules = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExtensionFeedItem resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.resource_name = "";
    
                            /**
                             * ExtensionFeedItem id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.id = null;
    
                            /**
                             * ExtensionFeedItem extension_type.
                             * @member {google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType} extension_type
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.extension_type = 0;
    
                            /**
                             * ExtensionFeedItem start_date_time.
                             * @member {string|null|undefined} start_date_time
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.start_date_time = null;
    
                            /**
                             * ExtensionFeedItem end_date_time.
                             * @member {string|null|undefined} end_date_time
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.end_date_time = null;
    
                            /**
                             * ExtensionFeedItem ad_schedules.
                             * @member {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>} ad_schedules
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.ad_schedules = $util.emptyArray;
    
                            /**
                             * ExtensionFeedItem device.
                             * @member {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.FeedItemTargetDevice} device
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.device = 0;
    
                            /**
                             * ExtensionFeedItem targeted_geo_target_constant.
                             * @member {string|null|undefined} targeted_geo_target_constant
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.targeted_geo_target_constant = null;
    
                            /**
                             * ExtensionFeedItem targeted_keyword.
                             * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} targeted_keyword
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.targeted_keyword = null;
    
                            /**
                             * ExtensionFeedItem status.
                             * @member {google.ads.googleads.v11.enums.FeedItemStatusEnum.FeedItemStatus} status
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.status = 0;
    
                            /**
                             * ExtensionFeedItem sitelink_feed_item.
                             * @member {google.ads.googleads.v11.common.ISitelinkFeedItem|null|undefined} sitelink_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.sitelink_feed_item = null;
    
                            /**
                             * ExtensionFeedItem structured_snippet_feed_item.
                             * @member {google.ads.googleads.v11.common.IStructuredSnippetFeedItem|null|undefined} structured_snippet_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.structured_snippet_feed_item = null;
    
                            /**
                             * ExtensionFeedItem app_feed_item.
                             * @member {google.ads.googleads.v11.common.IAppFeedItem|null|undefined} app_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.app_feed_item = null;
    
                            /**
                             * ExtensionFeedItem call_feed_item.
                             * @member {google.ads.googleads.v11.common.ICallFeedItem|null|undefined} call_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.call_feed_item = null;
    
                            /**
                             * ExtensionFeedItem callout_feed_item.
                             * @member {google.ads.googleads.v11.common.ICalloutFeedItem|null|undefined} callout_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.callout_feed_item = null;
    
                            /**
                             * ExtensionFeedItem text_message_feed_item.
                             * @member {google.ads.googleads.v11.common.ITextMessageFeedItem|null|undefined} text_message_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.text_message_feed_item = null;
    
                            /**
                             * ExtensionFeedItem price_feed_item.
                             * @member {google.ads.googleads.v11.common.IPriceFeedItem|null|undefined} price_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.price_feed_item = null;
    
                            /**
                             * ExtensionFeedItem promotion_feed_item.
                             * @member {google.ads.googleads.v11.common.IPromotionFeedItem|null|undefined} promotion_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.promotion_feed_item = null;
    
                            /**
                             * ExtensionFeedItem location_feed_item.
                             * @member {google.ads.googleads.v11.common.ILocationFeedItem|null|undefined} location_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.location_feed_item = null;
    
                            /**
                             * ExtensionFeedItem affiliate_location_feed_item.
                             * @member {google.ads.googleads.v11.common.IAffiliateLocationFeedItem|null|undefined} affiliate_location_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.affiliate_location_feed_item = null;
    
                            /**
                             * ExtensionFeedItem hotel_callout_feed_item.
                             * @member {google.ads.googleads.v11.common.IHotelCalloutFeedItem|null|undefined} hotel_callout_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.hotel_callout_feed_item = null;
    
                            /**
                             * ExtensionFeedItem image_feed_item.
                             * @member {google.ads.googleads.v11.common.IImageFeedItem|null|undefined} image_feed_item
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.image_feed_item = null;
    
                            /**
                             * ExtensionFeedItem targeted_campaign.
                             * @member {string|null|undefined} targeted_campaign
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.targeted_campaign = null;
    
                            /**
                             * ExtensionFeedItem targeted_ad_group.
                             * @member {string|null|undefined} targeted_ad_group
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            ExtensionFeedItem.prototype.targeted_ad_group = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExtensionFeedItem _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            Object.defineProperty(ExtensionFeedItem.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExtensionFeedItem _start_date_time.
                             * @member {"start_date_time"|undefined} _start_date_time
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            Object.defineProperty(ExtensionFeedItem.prototype, "_start_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExtensionFeedItem _end_date_time.
                             * @member {"end_date_time"|undefined} _end_date_time
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            Object.defineProperty(ExtensionFeedItem.prototype, "_end_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["end_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExtensionFeedItem _targeted_geo_target_constant.
                             * @member {"targeted_geo_target_constant"|undefined} _targeted_geo_target_constant
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            Object.defineProperty(ExtensionFeedItem.prototype, "_targeted_geo_target_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["targeted_geo_target_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExtensionFeedItem extension.
                             * @member {"sitelink_feed_item"|"structured_snippet_feed_item"|"app_feed_item"|"call_feed_item"|"callout_feed_item"|"text_message_feed_item"|"price_feed_item"|"promotion_feed_item"|"location_feed_item"|"affiliate_location_feed_item"|"hotel_callout_feed_item"|"image_feed_item"|undefined} extension
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            Object.defineProperty(ExtensionFeedItem.prototype, "extension", {
                                get: $util.oneOfGetter($oneOfFields = ["sitelink_feed_item", "structured_snippet_feed_item", "app_feed_item", "call_feed_item", "callout_feed_item", "text_message_feed_item", "price_feed_item", "promotion_feed_item", "location_feed_item", "affiliate_location_feed_item", "hotel_callout_feed_item", "image_feed_item"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExtensionFeedItem serving_resource_targeting.
                             * @member {"targeted_campaign"|"targeted_ad_group"|undefined} serving_resource_targeting
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             */
                            Object.defineProperty(ExtensionFeedItem.prototype, "serving_resource_targeting", {
                                get: $util.oneOfGetter($oneOfFields = ["targeted_campaign", "targeted_ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExtensionFeedItem instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExtensionFeedItem=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ExtensionFeedItem} ExtensionFeedItem instance
                             */
                            ExtensionFeedItem.create = function create(properties) {
                                return new ExtensionFeedItem(properties);
                            };
    
                            /**
                             * Encodes the specified ExtensionFeedItem message. Does not implicitly {@link google.ads.googleads.v11.resources.ExtensionFeedItem.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExtensionFeedItem} m ExtensionFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionFeedItem.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.sitelink_feed_item != null && Object.hasOwnProperty.call(m, "sitelink_feed_item"))
                                    $root.google.ads.googleads.v11.common.SitelinkFeedItem.encode(m.sitelink_feed_item, w.uint32(18).fork()).ldelim();
                                if (m.structured_snippet_feed_item != null && Object.hasOwnProperty.call(m, "structured_snippet_feed_item"))
                                    $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem.encode(m.structured_snippet_feed_item, w.uint32(26).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.app_feed_item != null && Object.hasOwnProperty.call(m, "app_feed_item"))
                                    $root.google.ads.googleads.v11.common.AppFeedItem.encode(m.app_feed_item, w.uint32(58).fork()).ldelim();
                                if (m.call_feed_item != null && Object.hasOwnProperty.call(m, "call_feed_item"))
                                    $root.google.ads.googleads.v11.common.CallFeedItem.encode(m.call_feed_item, w.uint32(66).fork()).ldelim();
                                if (m.callout_feed_item != null && Object.hasOwnProperty.call(m, "callout_feed_item"))
                                    $root.google.ads.googleads.v11.common.CalloutFeedItem.encode(m.callout_feed_item, w.uint32(74).fork()).ldelim();
                                if (m.text_message_feed_item != null && Object.hasOwnProperty.call(m, "text_message_feed_item"))
                                    $root.google.ads.googleads.v11.common.TextMessageFeedItem.encode(m.text_message_feed_item, w.uint32(82).fork()).ldelim();
                                if (m.price_feed_item != null && Object.hasOwnProperty.call(m, "price_feed_item"))
                                    $root.google.ads.googleads.v11.common.PriceFeedItem.encode(m.price_feed_item, w.uint32(90).fork()).ldelim();
                                if (m.promotion_feed_item != null && Object.hasOwnProperty.call(m, "promotion_feed_item"))
                                    $root.google.ads.googleads.v11.common.PromotionFeedItem.encode(m.promotion_feed_item, w.uint32(98).fork()).ldelim();
                                if (m.extension_type != null && Object.hasOwnProperty.call(m, "extension_type"))
                                    w.uint32(104).int32(m.extension_type);
                                if (m.location_feed_item != null && Object.hasOwnProperty.call(m, "location_feed_item"))
                                    $root.google.ads.googleads.v11.common.LocationFeedItem.encode(m.location_feed_item, w.uint32(114).fork()).ldelim();
                                if (m.affiliate_location_feed_item != null && Object.hasOwnProperty.call(m, "affiliate_location_feed_item"))
                                    $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem.encode(m.affiliate_location_feed_item, w.uint32(122).fork()).ldelim();
                                if (m.ad_schedules != null && m.ad_schedules.length) {
                                    for (var i = 0; i < m.ad_schedules.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedules[i], w.uint32(130).fork()).ldelim();
                                }
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    w.uint32(136).int32(m.device);
                                if (m.targeted_keyword != null && Object.hasOwnProperty.call(m, "targeted_keyword"))
                                    $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.targeted_keyword, w.uint32(178).fork()).ldelim();
                                if (m.hotel_callout_feed_item != null && Object.hasOwnProperty.call(m, "hotel_callout_feed_item"))
                                    $root.google.ads.googleads.v11.common.HotelCalloutFeedItem.encode(m.hotel_callout_feed_item, w.uint32(186).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(200).int64(m.id);
                                if (m.start_date_time != null && Object.hasOwnProperty.call(m, "start_date_time"))
                                    w.uint32(210).string(m.start_date_time);
                                if (m.end_date_time != null && Object.hasOwnProperty.call(m, "end_date_time"))
                                    w.uint32(218).string(m.end_date_time);
                                if (m.targeted_campaign != null && Object.hasOwnProperty.call(m, "targeted_campaign"))
                                    w.uint32(226).string(m.targeted_campaign);
                                if (m.targeted_ad_group != null && Object.hasOwnProperty.call(m, "targeted_ad_group"))
                                    w.uint32(234).string(m.targeted_ad_group);
                                if (m.targeted_geo_target_constant != null && Object.hasOwnProperty.call(m, "targeted_geo_target_constant"))
                                    w.uint32(242).string(m.targeted_geo_target_constant);
                                if (m.image_feed_item != null && Object.hasOwnProperty.call(m, "image_feed_item"))
                                    $root.google.ads.googleads.v11.common.ImageFeedItem.encode(m.image_feed_item, w.uint32(250).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExtensionFeedItem message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ExtensionFeedItem.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.IExtensionFeedItem} message ExtensionFeedItem message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionFeedItem.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExtensionFeedItem message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ExtensionFeedItem} ExtensionFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionFeedItem.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ExtensionFeedItem();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 25:
                                        m.id = r.int64();
                                        break;
                                    case 13:
                                        m.extension_type = r.int32();
                                        break;
                                    case 26:
                                        m.start_date_time = r.string();
                                        break;
                                    case 27:
                                        m.end_date_time = r.string();
                                        break;
                                    case 16:
                                        if (!(m.ad_schedules && m.ad_schedules.length))
                                            m.ad_schedules = [];
                                        m.ad_schedules.push($root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32()));
                                        break;
                                    case 17:
                                        m.device = r.int32();
                                        break;
                                    case 30:
                                        m.targeted_geo_target_constant = r.string();
                                        break;
                                    case 22:
                                        m.targeted_keyword = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 2:
                                        m.sitelink_feed_item = $root.google.ads.googleads.v11.common.SitelinkFeedItem.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.structured_snippet_feed_item = $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.app_feed_item = $root.google.ads.googleads.v11.common.AppFeedItem.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.call_feed_item = $root.google.ads.googleads.v11.common.CallFeedItem.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.callout_feed_item = $root.google.ads.googleads.v11.common.CalloutFeedItem.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.text_message_feed_item = $root.google.ads.googleads.v11.common.TextMessageFeedItem.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.price_feed_item = $root.google.ads.googleads.v11.common.PriceFeedItem.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.promotion_feed_item = $root.google.ads.googleads.v11.common.PromotionFeedItem.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.location_feed_item = $root.google.ads.googleads.v11.common.LocationFeedItem.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.affiliate_location_feed_item = $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.hotel_callout_feed_item = $root.google.ads.googleads.v11.common.HotelCalloutFeedItem.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.image_feed_item = $root.google.ads.googleads.v11.common.ImageFeedItem.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.targeted_campaign = r.string();
                                        break;
                                    case 29:
                                        m.targeted_ad_group = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExtensionFeedItem message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ExtensionFeedItem} ExtensionFeedItem
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionFeedItem.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExtensionFeedItem message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExtensionFeedItem.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    switch (m.extension_type) {
                                    default:
                                        return "extension_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 15:
                                    case 16:
                                        break;
                                    }
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    p._start_date_time = 1;
                                    if (!$util.isString(m.start_date_time))
                                        return "start_date_time: string expected";
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    p._end_date_time = 1;
                                    if (!$util.isString(m.end_date_time))
                                        return "end_date_time: string expected";
                                }
                                if (m.ad_schedules != null && m.hasOwnProperty("ad_schedules")) {
                                    if (!Array.isArray(m.ad_schedules))
                                        return "ad_schedules: array expected";
                                    for (var i = 0; i < m.ad_schedules.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedules[i]);
                                            if (e)
                                                return "ad_schedules." + e;
                                        }
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    switch (m.device) {
                                    default:
                                        return "device: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.targeted_geo_target_constant != null && m.hasOwnProperty("targeted_geo_target_constant")) {
                                    p._targeted_geo_target_constant = 1;
                                    if (!$util.isString(m.targeted_geo_target_constant))
                                        return "targeted_geo_target_constant: string expected";
                                }
                                if (m.targeted_keyword != null && m.hasOwnProperty("targeted_keyword")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.targeted_keyword);
                                        if (e)
                                            return "targeted_keyword." + e;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.sitelink_feed_item != null && m.hasOwnProperty("sitelink_feed_item")) {
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.SitelinkFeedItem.verify(m.sitelink_feed_item);
                                        if (e)
                                            return "sitelink_feed_item." + e;
                                    }
                                }
                                if (m.structured_snippet_feed_item != null && m.hasOwnProperty("structured_snippet_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem.verify(m.structured_snippet_feed_item);
                                        if (e)
                                            return "structured_snippet_feed_item." + e;
                                    }
                                }
                                if (m.app_feed_item != null && m.hasOwnProperty("app_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AppFeedItem.verify(m.app_feed_item);
                                        if (e)
                                            return "app_feed_item." + e;
                                    }
                                }
                                if (m.call_feed_item != null && m.hasOwnProperty("call_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CallFeedItem.verify(m.call_feed_item);
                                        if (e)
                                            return "call_feed_item." + e;
                                    }
                                }
                                if (m.callout_feed_item != null && m.hasOwnProperty("callout_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CalloutFeedItem.verify(m.callout_feed_item);
                                        if (e)
                                            return "callout_feed_item." + e;
                                    }
                                }
                                if (m.text_message_feed_item != null && m.hasOwnProperty("text_message_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TextMessageFeedItem.verify(m.text_message_feed_item);
                                        if (e)
                                            return "text_message_feed_item." + e;
                                    }
                                }
                                if (m.price_feed_item != null && m.hasOwnProperty("price_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PriceFeedItem.verify(m.price_feed_item);
                                        if (e)
                                            return "price_feed_item." + e;
                                    }
                                }
                                if (m.promotion_feed_item != null && m.hasOwnProperty("promotion_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PromotionFeedItem.verify(m.promotion_feed_item);
                                        if (e)
                                            return "promotion_feed_item." + e;
                                    }
                                }
                                if (m.location_feed_item != null && m.hasOwnProperty("location_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LocationFeedItem.verify(m.location_feed_item);
                                        if (e)
                                            return "location_feed_item." + e;
                                    }
                                }
                                if (m.affiliate_location_feed_item != null && m.hasOwnProperty("affiliate_location_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem.verify(m.affiliate_location_feed_item);
                                        if (e)
                                            return "affiliate_location_feed_item." + e;
                                    }
                                }
                                if (m.hotel_callout_feed_item != null && m.hasOwnProperty("hotel_callout_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HotelCalloutFeedItem.verify(m.hotel_callout_feed_item);
                                        if (e)
                                            return "hotel_callout_feed_item." + e;
                                    }
                                }
                                if (m.image_feed_item != null && m.hasOwnProperty("image_feed_item")) {
                                    if (p.extension === 1)
                                        return "extension: multiple values";
                                    p.extension = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ImageFeedItem.verify(m.image_feed_item);
                                        if (e)
                                            return "image_feed_item." + e;
                                    }
                                }
                                if (m.targeted_campaign != null && m.hasOwnProperty("targeted_campaign")) {
                                    p.serving_resource_targeting = 1;
                                    if (!$util.isString(m.targeted_campaign))
                                        return "targeted_campaign: string expected";
                                }
                                if (m.targeted_ad_group != null && m.hasOwnProperty("targeted_ad_group")) {
                                    if (p.serving_resource_targeting === 1)
                                        return "serving_resource_targeting: multiple values";
                                    p.serving_resource_targeting = 1;
                                    if (!$util.isString(m.targeted_ad_group))
                                        return "targeted_ad_group: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExtensionFeedItem message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ExtensionFeedItem} ExtensionFeedItem
                             */
                            ExtensionFeedItem.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ExtensionFeedItem)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ExtensionFeedItem();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.extension_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.extension_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.extension_type = 1;
                                    break;
                                case "NONE":
                                case 2:
                                    m.extension_type = 2;
                                    break;
                                case "APP":
                                case 3:
                                    m.extension_type = 3;
                                    break;
                                case "CALL":
                                case 4:
                                    m.extension_type = 4;
                                    break;
                                case "CALLOUT":
                                case 5:
                                    m.extension_type = 5;
                                    break;
                                case "MESSAGE":
                                case 6:
                                    m.extension_type = 6;
                                    break;
                                case "PRICE":
                                case 7:
                                    m.extension_type = 7;
                                    break;
                                case "PROMOTION":
                                case 8:
                                    m.extension_type = 8;
                                    break;
                                case "SITELINK":
                                case 10:
                                    m.extension_type = 10;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 11:
                                    m.extension_type = 11;
                                    break;
                                case "LOCATION":
                                case 12:
                                    m.extension_type = 12;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 13:
                                    m.extension_type = 13;
                                    break;
                                case "HOTEL_CALLOUT":
                                case 15:
                                    m.extension_type = 15;
                                    break;
                                case "IMAGE":
                                case 16:
                                    m.extension_type = 16;
                                    break;
                                }
                                if (d.start_date_time != null) {
                                    m.start_date_time = String(d.start_date_time);
                                }
                                if (d.end_date_time != null) {
                                    m.end_date_time = String(d.end_date_time);
                                }
                                if (d.ad_schedules) {
                                    if (!Array.isArray(d.ad_schedules))
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.ad_schedules: array expected");
                                    m.ad_schedules = [];
                                    for (var i = 0; i < d.ad_schedules.length; ++i) {
                                        if (typeof d.ad_schedules[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.ad_schedules: object expected");
                                        m.ad_schedules[i] = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedules[i]);
                                    }
                                }
                                switch (d.device) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device = 2;
                                    break;
                                }
                                if (d.targeted_geo_target_constant != null) {
                                    m.targeted_geo_target_constant = String(d.targeted_geo_target_constant);
                                }
                                if (d.targeted_keyword != null) {
                                    if (typeof d.targeted_keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.targeted_keyword: object expected");
                                    m.targeted_keyword = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.targeted_keyword);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.sitelink_feed_item != null) {
                                    if (typeof d.sitelink_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.sitelink_feed_item: object expected");
                                    m.sitelink_feed_item = $root.google.ads.googleads.v11.common.SitelinkFeedItem.fromObject(d.sitelink_feed_item);
                                }
                                if (d.structured_snippet_feed_item != null) {
                                    if (typeof d.structured_snippet_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.structured_snippet_feed_item: object expected");
                                    m.structured_snippet_feed_item = $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem.fromObject(d.structured_snippet_feed_item);
                                }
                                if (d.app_feed_item != null) {
                                    if (typeof d.app_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.app_feed_item: object expected");
                                    m.app_feed_item = $root.google.ads.googleads.v11.common.AppFeedItem.fromObject(d.app_feed_item);
                                }
                                if (d.call_feed_item != null) {
                                    if (typeof d.call_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.call_feed_item: object expected");
                                    m.call_feed_item = $root.google.ads.googleads.v11.common.CallFeedItem.fromObject(d.call_feed_item);
                                }
                                if (d.callout_feed_item != null) {
                                    if (typeof d.callout_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.callout_feed_item: object expected");
                                    m.callout_feed_item = $root.google.ads.googleads.v11.common.CalloutFeedItem.fromObject(d.callout_feed_item);
                                }
                                if (d.text_message_feed_item != null) {
                                    if (typeof d.text_message_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.text_message_feed_item: object expected");
                                    m.text_message_feed_item = $root.google.ads.googleads.v11.common.TextMessageFeedItem.fromObject(d.text_message_feed_item);
                                }
                                if (d.price_feed_item != null) {
                                    if (typeof d.price_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.price_feed_item: object expected");
                                    m.price_feed_item = $root.google.ads.googleads.v11.common.PriceFeedItem.fromObject(d.price_feed_item);
                                }
                                if (d.promotion_feed_item != null) {
                                    if (typeof d.promotion_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.promotion_feed_item: object expected");
                                    m.promotion_feed_item = $root.google.ads.googleads.v11.common.PromotionFeedItem.fromObject(d.promotion_feed_item);
                                }
                                if (d.location_feed_item != null) {
                                    if (typeof d.location_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.location_feed_item: object expected");
                                    m.location_feed_item = $root.google.ads.googleads.v11.common.LocationFeedItem.fromObject(d.location_feed_item);
                                }
                                if (d.affiliate_location_feed_item != null) {
                                    if (typeof d.affiliate_location_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.affiliate_location_feed_item: object expected");
                                    m.affiliate_location_feed_item = $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem.fromObject(d.affiliate_location_feed_item);
                                }
                                if (d.hotel_callout_feed_item != null) {
                                    if (typeof d.hotel_callout_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.hotel_callout_feed_item: object expected");
                                    m.hotel_callout_feed_item = $root.google.ads.googleads.v11.common.HotelCalloutFeedItem.fromObject(d.hotel_callout_feed_item);
                                }
                                if (d.image_feed_item != null) {
                                    if (typeof d.image_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.ExtensionFeedItem.image_feed_item: object expected");
                                    m.image_feed_item = $root.google.ads.googleads.v11.common.ImageFeedItem.fromObject(d.image_feed_item);
                                }
                                if (d.targeted_campaign != null) {
                                    m.targeted_campaign = String(d.targeted_campaign);
                                }
                                if (d.targeted_ad_group != null) {
                                    m.targeted_ad_group = String(d.targeted_ad_group);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExtensionFeedItem message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @static
                             * @param {google.ads.googleads.v11.resources.ExtensionFeedItem} m ExtensionFeedItem
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExtensionFeedItem.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.ad_schedules = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.extension_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.device = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.targeted_keyword = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.sitelink_feed_item != null && m.hasOwnProperty("sitelink_feed_item")) {
                                    d.sitelink_feed_item = $root.google.ads.googleads.v11.common.SitelinkFeedItem.toObject(m.sitelink_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "sitelink_feed_item";
                                }
                                if (m.structured_snippet_feed_item != null && m.hasOwnProperty("structured_snippet_feed_item")) {
                                    d.structured_snippet_feed_item = $root.google.ads.googleads.v11.common.StructuredSnippetFeedItem.toObject(m.structured_snippet_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "structured_snippet_feed_item";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemStatusEnum.FeedItemStatus[m.status] : m.status;
                                }
                                if (m.app_feed_item != null && m.hasOwnProperty("app_feed_item")) {
                                    d.app_feed_item = $root.google.ads.googleads.v11.common.AppFeedItem.toObject(m.app_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "app_feed_item";
                                }
                                if (m.call_feed_item != null && m.hasOwnProperty("call_feed_item")) {
                                    d.call_feed_item = $root.google.ads.googleads.v11.common.CallFeedItem.toObject(m.call_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "call_feed_item";
                                }
                                if (m.callout_feed_item != null && m.hasOwnProperty("callout_feed_item")) {
                                    d.callout_feed_item = $root.google.ads.googleads.v11.common.CalloutFeedItem.toObject(m.callout_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "callout_feed_item";
                                }
                                if (m.text_message_feed_item != null && m.hasOwnProperty("text_message_feed_item")) {
                                    d.text_message_feed_item = $root.google.ads.googleads.v11.common.TextMessageFeedItem.toObject(m.text_message_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "text_message_feed_item";
                                }
                                if (m.price_feed_item != null && m.hasOwnProperty("price_feed_item")) {
                                    d.price_feed_item = $root.google.ads.googleads.v11.common.PriceFeedItem.toObject(m.price_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "price_feed_item";
                                }
                                if (m.promotion_feed_item != null && m.hasOwnProperty("promotion_feed_item")) {
                                    d.promotion_feed_item = $root.google.ads.googleads.v11.common.PromotionFeedItem.toObject(m.promotion_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "promotion_feed_item";
                                }
                                if (m.extension_type != null && m.hasOwnProperty("extension_type")) {
                                    d.extension_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ExtensionTypeEnum.ExtensionType[m.extension_type] : m.extension_type;
                                }
                                if (m.location_feed_item != null && m.hasOwnProperty("location_feed_item")) {
                                    d.location_feed_item = $root.google.ads.googleads.v11.common.LocationFeedItem.toObject(m.location_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "location_feed_item";
                                }
                                if (m.affiliate_location_feed_item != null && m.hasOwnProperty("affiliate_location_feed_item")) {
                                    d.affiliate_location_feed_item = $root.google.ads.googleads.v11.common.AffiliateLocationFeedItem.toObject(m.affiliate_location_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "affiliate_location_feed_item";
                                }
                                if (m.ad_schedules && m.ad_schedules.length) {
                                    d.ad_schedules = [];
                                    for (var j = 0; j < m.ad_schedules.length; ++j) {
                                        d.ad_schedules[j] = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedules[j], o);
                                    }
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.FeedItemTargetDevice[m.device] : m.device;
                                }
                                if (m.targeted_keyword != null && m.hasOwnProperty("targeted_keyword")) {
                                    d.targeted_keyword = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.targeted_keyword, o);
                                }
                                if (m.hotel_callout_feed_item != null && m.hasOwnProperty("hotel_callout_feed_item")) {
                                    d.hotel_callout_feed_item = $root.google.ads.googleads.v11.common.HotelCalloutFeedItem.toObject(m.hotel_callout_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "hotel_callout_feed_item";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.start_date_time != null && m.hasOwnProperty("start_date_time")) {
                                    d.start_date_time = m.start_date_time;
                                    if (o.oneofs)
                                        d._start_date_time = "start_date_time";
                                }
                                if (m.end_date_time != null && m.hasOwnProperty("end_date_time")) {
                                    d.end_date_time = m.end_date_time;
                                    if (o.oneofs)
                                        d._end_date_time = "end_date_time";
                                }
                                if (m.targeted_campaign != null && m.hasOwnProperty("targeted_campaign")) {
                                    d.targeted_campaign = m.targeted_campaign;
                                    if (o.oneofs)
                                        d.serving_resource_targeting = "targeted_campaign";
                                }
                                if (m.targeted_ad_group != null && m.hasOwnProperty("targeted_ad_group")) {
                                    d.targeted_ad_group = m.targeted_ad_group;
                                    if (o.oneofs)
                                        d.serving_resource_targeting = "targeted_ad_group";
                                }
                                if (m.targeted_geo_target_constant != null && m.hasOwnProperty("targeted_geo_target_constant")) {
                                    d.targeted_geo_target_constant = m.targeted_geo_target_constant;
                                    if (o.oneofs)
                                        d._targeted_geo_target_constant = "targeted_geo_target_constant";
                                }
                                if (m.image_feed_item != null && m.hasOwnProperty("image_feed_item")) {
                                    d.image_feed_item = $root.google.ads.googleads.v11.common.ImageFeedItem.toObject(m.image_feed_item, o);
                                    if (o.oneofs)
                                        d.extension = "image_feed_item";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExtensionFeedItem to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ExtensionFeedItem
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExtensionFeedItem.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExtensionFeedItem;
                        })();
    
                        resources.FeedItemSet = (function() {
    
                            /**
                             * Properties of a FeedItemSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedItemSet
                             * @property {string|null} [resource_name] FeedItemSet resource_name
                             * @property {string|null} [feed] FeedItemSet feed
                             * @property {number|null} [feed_item_set_id] FeedItemSet feed_item_set_id
                             * @property {string|null} [display_name] FeedItemSet display_name
                             * @property {google.ads.googleads.v11.enums.FeedItemSetStatusEnum.FeedItemSetStatus|null} [status] FeedItemSet status
                             * @property {google.ads.googleads.v11.common.IDynamicLocationSetFilter|null} [dynamic_location_set_filter] FeedItemSet dynamic_location_set_filter
                             * @property {google.ads.googleads.v11.common.IDynamicAffiliateLocationSetFilter|null} [dynamic_affiliate_location_set_filter] FeedItemSet dynamic_affiliate_location_set_filter
                             */
    
                            /**
                             * Constructs a new FeedItemSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedItemSet.
                             * @implements IFeedItemSet
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedItemSet=} [p] Properties to set
                             */
                            function FeedItemSet(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemSet resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            FeedItemSet.prototype.resource_name = "";
    
                            /**
                             * FeedItemSet feed.
                             * @member {string} feed
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            FeedItemSet.prototype.feed = "";
    
                            /**
                             * FeedItemSet feed_item_set_id.
                             * @member {number} feed_item_set_id
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            FeedItemSet.prototype.feed_item_set_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * FeedItemSet display_name.
                             * @member {string} display_name
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            FeedItemSet.prototype.display_name = "";
    
                            /**
                             * FeedItemSet status.
                             * @member {google.ads.googleads.v11.enums.FeedItemSetStatusEnum.FeedItemSetStatus} status
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            FeedItemSet.prototype.status = 0;
    
                            /**
                             * FeedItemSet dynamic_location_set_filter.
                             * @member {google.ads.googleads.v11.common.IDynamicLocationSetFilter|null|undefined} dynamic_location_set_filter
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            FeedItemSet.prototype.dynamic_location_set_filter = null;
    
                            /**
                             * FeedItemSet dynamic_affiliate_location_set_filter.
                             * @member {google.ads.googleads.v11.common.IDynamicAffiliateLocationSetFilter|null|undefined} dynamic_affiliate_location_set_filter
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            FeedItemSet.prototype.dynamic_affiliate_location_set_filter = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemSet dynamic_set_filter.
                             * @member {"dynamic_location_set_filter"|"dynamic_affiliate_location_set_filter"|undefined} dynamic_set_filter
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             */
                            Object.defineProperty(FeedItemSet.prototype, "dynamic_set_filter", {
                                get: $util.oneOfGetter($oneOfFields = ["dynamic_location_set_filter", "dynamic_affiliate_location_set_filter"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemSet instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemSet=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedItemSet} FeedItemSet instance
                             */
                            FeedItemSet.create = function create(properties) {
                                return new FeedItemSet(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSet message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemSet.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemSet} m FeedItemSet message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSet.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(18).string(m.feed);
                                if (m.feed_item_set_id != null && Object.hasOwnProperty.call(m, "feed_item_set_id"))
                                    w.uint32(24).int64(m.feed_item_set_id);
                                if (m.display_name != null && Object.hasOwnProperty.call(m, "display_name"))
                                    w.uint32(34).string(m.display_name);
                                if (m.dynamic_location_set_filter != null && Object.hasOwnProperty.call(m, "dynamic_location_set_filter"))
                                    $root.google.ads.googleads.v11.common.DynamicLocationSetFilter.encode(m.dynamic_location_set_filter, w.uint32(42).fork()).ldelim();
                                if (m.dynamic_affiliate_location_set_filter != null && Object.hasOwnProperty.call(m, "dynamic_affiliate_location_set_filter"))
                                    $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.encode(m.dynamic_affiliate_location_set_filter, w.uint32(50).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(64).int32(m.status);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSet message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemSet.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemSet} message FeedItemSet message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSet.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSet message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedItemSet} FeedItemSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSet.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedItemSet();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.feed = r.string();
                                        break;
                                    case 3:
                                        m.feed_item_set_id = r.int64();
                                        break;
                                    case 4:
                                        m.display_name = r.string();
                                        break;
                                    case 8:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.dynamic_location_set_filter = $root.google.ads.googleads.v11.common.DynamicLocationSetFilter.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.dynamic_affiliate_location_set_filter = $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSet message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedItemSet} FeedItemSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSet.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSet message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSet.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.feed_item_set_id != null && m.hasOwnProperty("feed_item_set_id")) {
                                    if (!$util.isInteger(m.feed_item_set_id) && !(m.feed_item_set_id && $util.isInteger(m.feed_item_set_id.low) && $util.isInteger(m.feed_item_set_id.high)))
                                        return "feed_item_set_id: integer|Long expected";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    if (!$util.isString(m.display_name))
                                        return "display_name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.dynamic_location_set_filter != null && m.hasOwnProperty("dynamic_location_set_filter")) {
                                    p.dynamic_set_filter = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicLocationSetFilter.verify(m.dynamic_location_set_filter);
                                        if (e)
                                            return "dynamic_location_set_filter." + e;
                                    }
                                }
                                if (m.dynamic_affiliate_location_set_filter != null && m.hasOwnProperty("dynamic_affiliate_location_set_filter")) {
                                    if (p.dynamic_set_filter === 1)
                                        return "dynamic_set_filter: multiple values";
                                    p.dynamic_set_filter = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.verify(m.dynamic_affiliate_location_set_filter);
                                        if (e)
                                            return "dynamic_affiliate_location_set_filter." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSet message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedItemSet} FeedItemSet
                             */
                            FeedItemSet.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedItemSet)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedItemSet();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.feed_item_set_id != null) {
                                    if ($util.Long)
                                        (m.feed_item_set_id = $util.Long.fromValue(d.feed_item_set_id)).unsigned = false;
                                    else if (typeof d.feed_item_set_id === "string")
                                        m.feed_item_set_id = parseInt(d.feed_item_set_id, 10);
                                    else if (typeof d.feed_item_set_id === "number")
                                        m.feed_item_set_id = d.feed_item_set_id;
                                    else if (typeof d.feed_item_set_id === "object")
                                        m.feed_item_set_id = new $util.LongBits(d.feed_item_set_id.low >>> 0, d.feed_item_set_id.high >>> 0).toNumber();
                                }
                                if (d.display_name != null) {
                                    m.display_name = String(d.display_name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.dynamic_location_set_filter != null) {
                                    if (typeof d.dynamic_location_set_filter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemSet.dynamic_location_set_filter: object expected");
                                    m.dynamic_location_set_filter = $root.google.ads.googleads.v11.common.DynamicLocationSetFilter.fromObject(d.dynamic_location_set_filter);
                                }
                                if (d.dynamic_affiliate_location_set_filter != null) {
                                    if (typeof d.dynamic_affiliate_location_set_filter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemSet.dynamic_affiliate_location_set_filter: object expected");
                                    m.dynamic_affiliate_location_set_filter = $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.fromObject(d.dynamic_affiliate_location_set_filter);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSet message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedItemSet} m FeedItemSet
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSet.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.feed = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.feed_item_set_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.feed_item_set_id = o.longs === String ? "0" : 0;
                                    d.display_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                }
                                if (m.feed_item_set_id != null && m.hasOwnProperty("feed_item_set_id")) {
                                    if (typeof m.feed_item_set_id === "number")
                                        d.feed_item_set_id = o.longs === String ? String(m.feed_item_set_id) : m.feed_item_set_id;
                                    else
                                        d.feed_item_set_id = o.longs === String ? $util.Long.prototype.toString.call(m.feed_item_set_id) : o.longs === Number ? new $util.LongBits(m.feed_item_set_id.low >>> 0, m.feed_item_set_id.high >>> 0).toNumber() : m.feed_item_set_id;
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    d.display_name = m.display_name;
                                }
                                if (m.dynamic_location_set_filter != null && m.hasOwnProperty("dynamic_location_set_filter")) {
                                    d.dynamic_location_set_filter = $root.google.ads.googleads.v11.common.DynamicLocationSetFilter.toObject(m.dynamic_location_set_filter, o);
                                    if (o.oneofs)
                                        d.dynamic_set_filter = "dynamic_location_set_filter";
                                }
                                if (m.dynamic_affiliate_location_set_filter != null && m.hasOwnProperty("dynamic_affiliate_location_set_filter")) {
                                    d.dynamic_affiliate_location_set_filter = $root.google.ads.googleads.v11.common.DynamicAffiliateLocationSetFilter.toObject(m.dynamic_affiliate_location_set_filter, o);
                                    if (o.oneofs)
                                        d.dynamic_set_filter = "dynamic_affiliate_location_set_filter";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemSetStatusEnum.FeedItemSetStatus[m.status] : m.status;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemSet to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedItemSet
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSet.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemSet;
                        })();
    
                        resources.FeedItemSetLink = (function() {
    
                            /**
                             * Properties of a FeedItemSetLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedItemSetLink
                             * @property {string|null} [resource_name] FeedItemSetLink resource_name
                             * @property {string|null} [feed_item] FeedItemSetLink feed_item
                             * @property {string|null} [feed_item_set] FeedItemSetLink feed_item_set
                             */
    
                            /**
                             * Constructs a new FeedItemSetLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedItemSetLink.
                             * @implements IFeedItemSetLink
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedItemSetLink=} [p] Properties to set
                             */
                            function FeedItemSetLink(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemSetLink resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @instance
                             */
                            FeedItemSetLink.prototype.resource_name = "";
    
                            /**
                             * FeedItemSetLink feed_item.
                             * @member {string} feed_item
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @instance
                             */
                            FeedItemSetLink.prototype.feed_item = "";
    
                            /**
                             * FeedItemSetLink feed_item_set.
                             * @member {string} feed_item_set
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @instance
                             */
                            FeedItemSetLink.prototype.feed_item_set = "";
    
                            /**
                             * Creates a new FeedItemSetLink instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemSetLink=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedItemSetLink} FeedItemSetLink instance
                             */
                            FeedItemSetLink.create = function create(properties) {
                                return new FeedItemSetLink(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSetLink message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemSetLink.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemSetLink} m FeedItemSetLink message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetLink.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.feed_item != null && Object.hasOwnProperty.call(m, "feed_item"))
                                    w.uint32(18).string(m.feed_item);
                                if (m.feed_item_set != null && Object.hasOwnProperty.call(m, "feed_item_set"))
                                    w.uint32(26).string(m.feed_item_set);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSetLink message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemSetLink.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemSetLink} message FeedItemSetLink message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetLink.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSetLink message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedItemSetLink} FeedItemSetLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetLink.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedItemSetLink();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.feed_item = r.string();
                                        break;
                                    case 3:
                                        m.feed_item_set = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSetLink message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedItemSetLink} FeedItemSetLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetLink.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSetLink message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSetLink.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    if (!$util.isString(m.feed_item))
                                        return "feed_item: string expected";
                                }
                                if (m.feed_item_set != null && m.hasOwnProperty("feed_item_set")) {
                                    if (!$util.isString(m.feed_item_set))
                                        return "feed_item_set: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSetLink message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedItemSetLink} FeedItemSetLink
                             */
                            FeedItemSetLink.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedItemSetLink)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedItemSetLink();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed_item != null) {
                                    m.feed_item = String(d.feed_item);
                                }
                                if (d.feed_item_set != null) {
                                    m.feed_item_set = String(d.feed_item_set);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSetLink message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedItemSetLink} m FeedItemSetLink
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSetLink.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.feed_item = "";
                                    d.feed_item_set = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    d.feed_item = m.feed_item;
                                }
                                if (m.feed_item_set != null && m.hasOwnProperty("feed_item_set")) {
                                    d.feed_item_set = m.feed_item_set;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemSetLink to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedItemSetLink
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSetLink.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemSetLink;
                        })();
    
                        resources.FeedItemTarget = (function() {
    
                            /**
                             * Properties of a FeedItemTarget.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedItemTarget
                             * @property {string|null} [resource_name] FeedItemTarget resource_name
                             * @property {string|null} [feed_item] FeedItemTarget feed_item
                             * @property {google.ads.googleads.v11.enums.FeedItemTargetTypeEnum.FeedItemTargetType|null} [feed_item_target_type] FeedItemTarget feed_item_target_type
                             * @property {number|null} [feed_item_target_id] FeedItemTarget feed_item_target_id
                             * @property {google.ads.googleads.v11.enums.FeedItemTargetStatusEnum.FeedItemTargetStatus|null} [status] FeedItemTarget status
                             * @property {string|null} [campaign] FeedItemTarget campaign
                             * @property {string|null} [ad_group] FeedItemTarget ad_group
                             * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [keyword] FeedItemTarget keyword
                             * @property {string|null} [geo_target_constant] FeedItemTarget geo_target_constant
                             * @property {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.FeedItemTargetDevice|null} [device] FeedItemTarget device
                             * @property {google.ads.googleads.v11.common.IAdScheduleInfo|null} [ad_schedule] FeedItemTarget ad_schedule
                             */
    
                            /**
                             * Constructs a new FeedItemTarget.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedItemTarget.
                             * @implements IFeedItemTarget
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedItemTarget=} [p] Properties to set
                             */
                            function FeedItemTarget(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemTarget resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.resource_name = "";
    
                            /**
                             * FeedItemTarget feed_item.
                             * @member {string|null|undefined} feed_item
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.feed_item = null;
    
                            /**
                             * FeedItemTarget feed_item_target_type.
                             * @member {google.ads.googleads.v11.enums.FeedItemTargetTypeEnum.FeedItemTargetType} feed_item_target_type
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.feed_item_target_type = 0;
    
                            /**
                             * FeedItemTarget feed_item_target_id.
                             * @member {number|null|undefined} feed_item_target_id
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.feed_item_target_id = null;
    
                            /**
                             * FeedItemTarget status.
                             * @member {google.ads.googleads.v11.enums.FeedItemTargetStatusEnum.FeedItemTargetStatus} status
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.status = 0;
    
                            /**
                             * FeedItemTarget campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.campaign = null;
    
                            /**
                             * FeedItemTarget ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.ad_group = null;
    
                            /**
                             * FeedItemTarget keyword.
                             * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} keyword
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.keyword = null;
    
                            /**
                             * FeedItemTarget geo_target_constant.
                             * @member {string|null|undefined} geo_target_constant
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.geo_target_constant = null;
    
                            /**
                             * FeedItemTarget device.
                             * @member {google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.FeedItemTargetDevice|null|undefined} device
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.device = null;
    
                            /**
                             * FeedItemTarget ad_schedule.
                             * @member {google.ads.googleads.v11.common.IAdScheduleInfo|null|undefined} ad_schedule
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            FeedItemTarget.prototype.ad_schedule = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemTarget _feed_item.
                             * @member {"feed_item"|undefined} _feed_item
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            Object.defineProperty(FeedItemTarget.prototype, "_feed_item", {
                                get: $util.oneOfGetter($oneOfFields = ["feed_item"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItemTarget _feed_item_target_id.
                             * @member {"feed_item_target_id"|undefined} _feed_item_target_id
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            Object.defineProperty(FeedItemTarget.prototype, "_feed_item_target_id", {
                                get: $util.oneOfGetter($oneOfFields = ["feed_item_target_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedItemTarget target.
                             * @member {"campaign"|"ad_group"|"keyword"|"geo_target_constant"|"device"|"ad_schedule"|undefined} target
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             */
                            Object.defineProperty(FeedItemTarget.prototype, "target", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign", "ad_group", "keyword", "geo_target_constant", "device", "ad_schedule"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemTarget instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemTarget=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedItemTarget} FeedItemTarget instance
                             */
                            FeedItemTarget.create = function create(properties) {
                                return new FeedItemTarget(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemTarget message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemTarget.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemTarget} m FeedItemTarget message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTarget.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.feed_item_target_type != null && Object.hasOwnProperty.call(m, "feed_item_target_type"))
                                    w.uint32(24).int32(m.feed_item_target_type);
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword, w.uint32(58).fork()).ldelim();
                                if (m.device != null && Object.hasOwnProperty.call(m, "device"))
                                    w.uint32(72).int32(m.device);
                                if (m.ad_schedule != null && Object.hasOwnProperty.call(m, "ad_schedule"))
                                    $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedule, w.uint32(82).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(88).int32(m.status);
                                if (m.feed_item != null && Object.hasOwnProperty.call(m, "feed_item"))
                                    w.uint32(98).string(m.feed_item);
                                if (m.feed_item_target_id != null && Object.hasOwnProperty.call(m, "feed_item_target_id"))
                                    w.uint32(104).int64(m.feed_item_target_id);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(114).string(m.campaign);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(122).string(m.ad_group);
                                if (m.geo_target_constant != null && Object.hasOwnProperty.call(m, "geo_target_constant"))
                                    w.uint32(130).string(m.geo_target_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemTarget message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedItemTarget.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedItemTarget} message FeedItemTarget message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTarget.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemTarget message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedItemTarget} FeedItemTarget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTarget.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedItemTarget();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 12:
                                        m.feed_item = r.string();
                                        break;
                                    case 3:
                                        m.feed_item_target_type = r.int32();
                                        break;
                                    case 13:
                                        m.feed_item_target_id = r.int64();
                                        break;
                                    case 11:
                                        m.status = r.int32();
                                        break;
                                    case 14:
                                        m.campaign = r.string();
                                        break;
                                    case 15:
                                        m.ad_group = r.string();
                                        break;
                                    case 7:
                                        m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.geo_target_constant = r.string();
                                        break;
                                    case 9:
                                        m.device = r.int32();
                                        break;
                                    case 10:
                                        m.ad_schedule = $root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemTarget message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedItemTarget} FeedItemTarget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTarget.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemTarget message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemTarget.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    p._feed_item = 1;
                                    if (!$util.isString(m.feed_item))
                                        return "feed_item: string expected";
                                }
                                if (m.feed_item_target_type != null && m.hasOwnProperty("feed_item_target_type")) {
                                    switch (m.feed_item_target_type) {
                                    default:
                                        return "feed_item_target_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.feed_item_target_id != null && m.hasOwnProperty("feed_item_target_id")) {
                                    p._feed_item_target_id = 1;
                                    if (!$util.isInteger(m.feed_item_target_id) && !(m.feed_item_target_id && $util.isInteger(m.feed_item_target_id.low) && $util.isInteger(m.feed_item_target_id.high)))
                                        return "feed_item_target_id: integer|Long expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p.target = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    if (p.target === 1)
                                        return "target: multiple values";
                                    p.target = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    if (p.target === 1)
                                        return "target: multiple values";
                                    p.target = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword);
                                        if (e)
                                            return "keyword." + e;
                                    }
                                }
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    if (p.target === 1)
                                        return "target: multiple values";
                                    p.target = 1;
                                    if (!$util.isString(m.geo_target_constant))
                                        return "geo_target_constant: string expected";
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    if (p.target === 1)
                                        return "target: multiple values";
                                    p.target = 1;
                                    switch (m.device) {
                                    default:
                                        return "device: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.ad_schedule != null && m.hasOwnProperty("ad_schedule")) {
                                    if (p.target === 1)
                                        return "target: multiple values";
                                    p.target = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedule);
                                        if (e)
                                            return "ad_schedule." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemTarget message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedItemTarget} FeedItemTarget
                             */
                            FeedItemTarget.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedItemTarget)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedItemTarget();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed_item != null) {
                                    m.feed_item = String(d.feed_item);
                                }
                                switch (d.feed_item_target_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.feed_item_target_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.feed_item_target_type = 1;
                                    break;
                                case "CAMPAIGN":
                                case 2:
                                    m.feed_item_target_type = 2;
                                    break;
                                case "AD_GROUP":
                                case 3:
                                    m.feed_item_target_type = 3;
                                    break;
                                case "CRITERION":
                                case 4:
                                    m.feed_item_target_type = 4;
                                    break;
                                }
                                if (d.feed_item_target_id != null) {
                                    if ($util.Long)
                                        (m.feed_item_target_id = $util.Long.fromValue(d.feed_item_target_id)).unsigned = false;
                                    else if (typeof d.feed_item_target_id === "string")
                                        m.feed_item_target_id = parseInt(d.feed_item_target_id, 10);
                                    else if (typeof d.feed_item_target_id === "number")
                                        m.feed_item_target_id = d.feed_item_target_id;
                                    else if (typeof d.feed_item_target_id === "object")
                                        m.feed_item_target_id = new $util.LongBits(d.feed_item_target_id.low >>> 0, d.feed_item_target_id.high >>> 0).toNumber();
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.keyword != null) {
                                    if (typeof d.keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemTarget.keyword: object expected");
                                    m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword);
                                }
                                if (d.geo_target_constant != null) {
                                    m.geo_target_constant = String(d.geo_target_constant);
                                }
                                switch (d.device) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.device = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.device = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.device = 2;
                                    break;
                                }
                                if (d.ad_schedule != null) {
                                    if (typeof d.ad_schedule !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedItemTarget.ad_schedule: object expected");
                                    m.ad_schedule = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedule);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemTarget message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedItemTarget} m FeedItemTarget
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemTarget.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.feed_item_target_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.feed_item_target_type != null && m.hasOwnProperty("feed_item_target_type")) {
                                    d.feed_item_target_type = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemTargetTypeEnum.FeedItemTargetType[m.feed_item_target_type] : m.feed_item_target_type;
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword, o);
                                    if (o.oneofs)
                                        d.target = "keyword";
                                }
                                if (m.device != null && m.hasOwnProperty("device")) {
                                    d.device = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemTargetDeviceEnum.FeedItemTargetDevice[m.device] : m.device;
                                    if (o.oneofs)
                                        d.target = "device";
                                }
                                if (m.ad_schedule != null && m.hasOwnProperty("ad_schedule")) {
                                    d.ad_schedule = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedule, o);
                                    if (o.oneofs)
                                        d.target = "ad_schedule";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedItemTargetStatusEnum.FeedItemTargetStatus[m.status] : m.status;
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    d.feed_item = m.feed_item;
                                    if (o.oneofs)
                                        d._feed_item = "feed_item";
                                }
                                if (m.feed_item_target_id != null && m.hasOwnProperty("feed_item_target_id")) {
                                    if (typeof m.feed_item_target_id === "number")
                                        d.feed_item_target_id = o.longs === String ? String(m.feed_item_target_id) : m.feed_item_target_id;
                                    else
                                        d.feed_item_target_id = o.longs === String ? $util.Long.prototype.toString.call(m.feed_item_target_id) : o.longs === Number ? new $util.LongBits(m.feed_item_target_id.low >>> 0, m.feed_item_target_id.high >>> 0).toNumber() : m.feed_item_target_id;
                                    if (o.oneofs)
                                        d._feed_item_target_id = "feed_item_target_id";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d.target = "campaign";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d.target = "ad_group";
                                }
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    d.geo_target_constant = m.geo_target_constant;
                                    if (o.oneofs)
                                        d.target = "geo_target_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemTarget to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedItemTarget
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemTarget.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemTarget;
                        })();
    
                        resources.FeedMapping = (function() {
    
                            /**
                             * Properties of a FeedMapping.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedMapping
                             * @property {string|null} [resource_name] FeedMapping resource_name
                             * @property {string|null} [feed] FeedMapping feed
                             * @property {Array.<google.ads.googleads.v11.resources.IAttributeFieldMapping>|null} [attribute_field_mappings] FeedMapping attribute_field_mappings
                             * @property {google.ads.googleads.v11.enums.FeedMappingStatusEnum.FeedMappingStatus|null} [status] FeedMapping status
                             * @property {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType|null} [placeholder_type] FeedMapping placeholder_type
                             * @property {google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum.FeedMappingCriterionType|null} [criterion_type] FeedMapping criterion_type
                             */
    
                            /**
                             * Constructs a new FeedMapping.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedMapping.
                             * @implements IFeedMapping
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedMapping=} [p] Properties to set
                             */
                            function FeedMapping(p) {
                                this.attribute_field_mappings = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedMapping resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            FeedMapping.prototype.resource_name = "";
    
                            /**
                             * FeedMapping feed.
                             * @member {string|null|undefined} feed
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            FeedMapping.prototype.feed = null;
    
                            /**
                             * FeedMapping attribute_field_mappings.
                             * @member {Array.<google.ads.googleads.v11.resources.IAttributeFieldMapping>} attribute_field_mappings
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            FeedMapping.prototype.attribute_field_mappings = $util.emptyArray;
    
                            /**
                             * FeedMapping status.
                             * @member {google.ads.googleads.v11.enums.FeedMappingStatusEnum.FeedMappingStatus} status
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            FeedMapping.prototype.status = 0;
    
                            /**
                             * FeedMapping placeholder_type.
                             * @member {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType|null|undefined} placeholder_type
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            FeedMapping.prototype.placeholder_type = null;
    
                            /**
                             * FeedMapping criterion_type.
                             * @member {google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum.FeedMappingCriterionType|null|undefined} criterion_type
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            FeedMapping.prototype.criterion_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedMapping _feed.
                             * @member {"feed"|undefined} _feed
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            Object.defineProperty(FeedMapping.prototype, "_feed", {
                                get: $util.oneOfGetter($oneOfFields = ["feed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * FeedMapping target.
                             * @member {"placeholder_type"|"criterion_type"|undefined} target
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             */
                            Object.defineProperty(FeedMapping.prototype, "target", {
                                get: $util.oneOfGetter($oneOfFields = ["placeholder_type", "criterion_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedMapping instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedMapping=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedMapping} FeedMapping instance
                             */
                            FeedMapping.create = function create(properties) {
                                return new FeedMapping(properties);
                            };
    
                            /**
                             * Encodes the specified FeedMapping message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedMapping.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedMapping} m FeedMapping message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMapping.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.placeholder_type != null && Object.hasOwnProperty.call(m, "placeholder_type"))
                                    w.uint32(24).int32(m.placeholder_type);
                                if (m.criterion_type != null && Object.hasOwnProperty.call(m, "criterion_type"))
                                    w.uint32(32).int32(m.criterion_type);
                                if (m.attribute_field_mappings != null && m.attribute_field_mappings.length) {
                                    for (var i = 0; i < m.attribute_field_mappings.length; ++i)
                                        $root.google.ads.googleads.v11.resources.AttributeFieldMapping.encode(m.attribute_field_mappings[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(48).int32(m.status);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    w.uint32(58).string(m.feed);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedMapping message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedMapping.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedMapping} message FeedMapping message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMapping.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedMapping message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedMapping} FeedMapping
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMapping.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedMapping();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.feed = r.string();
                                        break;
                                    case 5:
                                        if (!(m.attribute_field_mappings && m.attribute_field_mappings.length))
                                            m.attribute_field_mappings = [];
                                        m.attribute_field_mappings.push($root.google.ads.googleads.v11.resources.AttributeFieldMapping.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        m.status = r.int32();
                                        break;
                                    case 3:
                                        m.placeholder_type = r.int32();
                                        break;
                                    case 4:
                                        m.criterion_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedMapping message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedMapping} FeedMapping
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMapping.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedMapping message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedMapping.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    p._feed = 1;
                                    if (!$util.isString(m.feed))
                                        return "feed: string expected";
                                }
                                if (m.attribute_field_mappings != null && m.hasOwnProperty("attribute_field_mappings")) {
                                    if (!Array.isArray(m.attribute_field_mappings))
                                        return "attribute_field_mappings: array expected";
                                    for (var i = 0; i < m.attribute_field_mappings.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.AttributeFieldMapping.verify(m.attribute_field_mappings[i]);
                                            if (e)
                                                return "attribute_field_mappings." + e;
                                        }
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.placeholder_type != null && m.hasOwnProperty("placeholder_type")) {
                                    p.target = 1;
                                    switch (m.placeholder_type) {
                                    default:
                                        return "placeholder_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.criterion_type != null && m.hasOwnProperty("criterion_type")) {
                                    if (p.target === 1)
                                        return "target: multiple values";
                                    p.target = 1;
                                    switch (m.criterion_type) {
                                    default:
                                        return "criterion_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 4:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedMapping message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedMapping} FeedMapping
                             */
                            FeedMapping.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedMapping)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedMapping();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed != null) {
                                    m.feed = String(d.feed);
                                }
                                if (d.attribute_field_mappings) {
                                    if (!Array.isArray(d.attribute_field_mappings))
                                        throw TypeError(".google.ads.googleads.v11.resources.FeedMapping.attribute_field_mappings: array expected");
                                    m.attribute_field_mappings = [];
                                    for (var i = 0; i < d.attribute_field_mappings.length; ++i) {
                                        if (typeof d.attribute_field_mappings[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.FeedMapping.attribute_field_mappings: object expected");
                                        m.attribute_field_mappings[i] = $root.google.ads.googleads.v11.resources.AttributeFieldMapping.fromObject(d.attribute_field_mappings[i]);
                                    }
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                switch (d.placeholder_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.placeholder_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.placeholder_type = 1;
                                    break;
                                case "SITELINK":
                                case 2:
                                    m.placeholder_type = 2;
                                    break;
                                case "CALL":
                                case 3:
                                    m.placeholder_type = 3;
                                    break;
                                case "APP":
                                case 4:
                                    m.placeholder_type = 4;
                                    break;
                                case "LOCATION":
                                case 5:
                                    m.placeholder_type = 5;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 6:
                                    m.placeholder_type = 6;
                                    break;
                                case "CALLOUT":
                                case 7:
                                    m.placeholder_type = 7;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 8:
                                    m.placeholder_type = 8;
                                    break;
                                case "MESSAGE":
                                case 9:
                                    m.placeholder_type = 9;
                                    break;
                                case "PRICE":
                                case 10:
                                    m.placeholder_type = 10;
                                    break;
                                case "PROMOTION":
                                case 11:
                                    m.placeholder_type = 11;
                                    break;
                                case "AD_CUSTOMIZER":
                                case 12:
                                    m.placeholder_type = 12;
                                    break;
                                case "DYNAMIC_EDUCATION":
                                case 13:
                                    m.placeholder_type = 13;
                                    break;
                                case "DYNAMIC_FLIGHT":
                                case 14:
                                    m.placeholder_type = 14;
                                    break;
                                case "DYNAMIC_CUSTOM":
                                case 15:
                                    m.placeholder_type = 15;
                                    break;
                                case "DYNAMIC_HOTEL":
                                case 16:
                                    m.placeholder_type = 16;
                                    break;
                                case "DYNAMIC_REAL_ESTATE":
                                case 17:
                                    m.placeholder_type = 17;
                                    break;
                                case "DYNAMIC_TRAVEL":
                                case 18:
                                    m.placeholder_type = 18;
                                    break;
                                case "DYNAMIC_LOCAL":
                                case 19:
                                    m.placeholder_type = 19;
                                    break;
                                case "DYNAMIC_JOB":
                                case 20:
                                    m.placeholder_type = 20;
                                    break;
                                case "IMAGE":
                                case 21:
                                    m.placeholder_type = 21;
                                    break;
                                }
                                switch (d.criterion_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.criterion_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.criterion_type = 1;
                                    break;
                                case "LOCATION_EXTENSION_TARGETING":
                                case 4:
                                    m.criterion_type = 4;
                                    break;
                                case "DSA_PAGE_FEED":
                                case 3:
                                    m.criterion_type = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedMapping message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedMapping} m FeedMapping
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedMapping.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.attribute_field_mappings = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.placeholder_type != null && m.hasOwnProperty("placeholder_type")) {
                                    d.placeholder_type = o.enums === String ? $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType[m.placeholder_type] : m.placeholder_type;
                                    if (o.oneofs)
                                        d.target = "placeholder_type";
                                }
                                if (m.criterion_type != null && m.hasOwnProperty("criterion_type")) {
                                    d.criterion_type = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedMappingCriterionTypeEnum.FeedMappingCriterionType[m.criterion_type] : m.criterion_type;
                                    if (o.oneofs)
                                        d.target = "criterion_type";
                                }
                                if (m.attribute_field_mappings && m.attribute_field_mappings.length) {
                                    d.attribute_field_mappings = [];
                                    for (var j = 0; j < m.attribute_field_mappings.length; ++j) {
                                        d.attribute_field_mappings[j] = $root.google.ads.googleads.v11.resources.AttributeFieldMapping.toObject(m.attribute_field_mappings[j], o);
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.FeedMappingStatusEnum.FeedMappingStatus[m.status] : m.status;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = m.feed;
                                    if (o.oneofs)
                                        d._feed = "feed";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedMapping to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedMapping
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedMapping.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedMapping;
                        })();
    
                        resources.AttributeFieldMapping = (function() {
    
                            /**
                             * Properties of an AttributeFieldMapping.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IAttributeFieldMapping
                             * @property {number|null} [feed_attribute_id] AttributeFieldMapping feed_attribute_id
                             * @property {number|null} [field_id] AttributeFieldMapping field_id
                             * @property {google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField|null} [sitelink_field] AttributeFieldMapping sitelink_field
                             * @property {google.ads.googleads.v11.enums.CallPlaceholderFieldEnum.CallPlaceholderField|null} [call_field] AttributeFieldMapping call_field
                             * @property {google.ads.googleads.v11.enums.AppPlaceholderFieldEnum.AppPlaceholderField|null} [app_field] AttributeFieldMapping app_field
                             * @property {google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum.LocationPlaceholderField|null} [location_field] AttributeFieldMapping location_field
                             * @property {google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField|null} [affiliate_location_field] AttributeFieldMapping affiliate_location_field
                             * @property {google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum.CalloutPlaceholderField|null} [callout_field] AttributeFieldMapping callout_field
                             * @property {google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField|null} [structured_snippet_field] AttributeFieldMapping structured_snippet_field
                             * @property {google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum.MessagePlaceholderField|null} [message_field] AttributeFieldMapping message_field
                             * @property {google.ads.googleads.v11.enums.PricePlaceholderFieldEnum.PricePlaceholderField|null} [price_field] AttributeFieldMapping price_field
                             * @property {google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum.PromotionPlaceholderField|null} [promotion_field] AttributeFieldMapping promotion_field
                             * @property {google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField|null} [ad_customizer_field] AttributeFieldMapping ad_customizer_field
                             * @property {google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField|null} [dsa_page_feed_field] AttributeFieldMapping dsa_page_feed_field
                             * @property {google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField|null} [location_extension_targeting_field] AttributeFieldMapping location_extension_targeting_field
                             * @property {google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum.EducationPlaceholderField|null} [education_field] AttributeFieldMapping education_field
                             * @property {google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum.FlightPlaceholderField|null} [flight_field] AttributeFieldMapping flight_field
                             * @property {google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum.CustomPlaceholderField|null} [custom_field] AttributeFieldMapping custom_field
                             * @property {google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum.HotelPlaceholderField|null} [hotel_field] AttributeFieldMapping hotel_field
                             * @property {google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField|null} [real_estate_field] AttributeFieldMapping real_estate_field
                             * @property {google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum.TravelPlaceholderField|null} [travel_field] AttributeFieldMapping travel_field
                             * @property {google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum.LocalPlaceholderField|null} [local_field] AttributeFieldMapping local_field
                             * @property {google.ads.googleads.v11.enums.JobPlaceholderFieldEnum.JobPlaceholderField|null} [job_field] AttributeFieldMapping job_field
                             * @property {google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum.ImagePlaceholderField|null} [image_field] AttributeFieldMapping image_field
                             */
    
                            /**
                             * Constructs a new AttributeFieldMapping.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an AttributeFieldMapping.
                             * @implements IAttributeFieldMapping
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IAttributeFieldMapping=} [p] Properties to set
                             */
                            function AttributeFieldMapping(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AttributeFieldMapping feed_attribute_id.
                             * @member {number|null|undefined} feed_attribute_id
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.feed_attribute_id = null;
    
                            /**
                             * AttributeFieldMapping field_id.
                             * @member {number|null|undefined} field_id
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.field_id = null;
    
                            /**
                             * AttributeFieldMapping sitelink_field.
                             * @member {google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField|null|undefined} sitelink_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.sitelink_field = null;
    
                            /**
                             * AttributeFieldMapping call_field.
                             * @member {google.ads.googleads.v11.enums.CallPlaceholderFieldEnum.CallPlaceholderField|null|undefined} call_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.call_field = null;
    
                            /**
                             * AttributeFieldMapping app_field.
                             * @member {google.ads.googleads.v11.enums.AppPlaceholderFieldEnum.AppPlaceholderField|null|undefined} app_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.app_field = null;
    
                            /**
                             * AttributeFieldMapping location_field.
                             * @member {google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum.LocationPlaceholderField|null|undefined} location_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.location_field = null;
    
                            /**
                             * AttributeFieldMapping affiliate_location_field.
                             * @member {google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField|null|undefined} affiliate_location_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.affiliate_location_field = null;
    
                            /**
                             * AttributeFieldMapping callout_field.
                             * @member {google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum.CalloutPlaceholderField|null|undefined} callout_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.callout_field = null;
    
                            /**
                             * AttributeFieldMapping structured_snippet_field.
                             * @member {google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField|null|undefined} structured_snippet_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.structured_snippet_field = null;
    
                            /**
                             * AttributeFieldMapping message_field.
                             * @member {google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum.MessagePlaceholderField|null|undefined} message_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.message_field = null;
    
                            /**
                             * AttributeFieldMapping price_field.
                             * @member {google.ads.googleads.v11.enums.PricePlaceholderFieldEnum.PricePlaceholderField|null|undefined} price_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.price_field = null;
    
                            /**
                             * AttributeFieldMapping promotion_field.
                             * @member {google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum.PromotionPlaceholderField|null|undefined} promotion_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.promotion_field = null;
    
                            /**
                             * AttributeFieldMapping ad_customizer_field.
                             * @member {google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField|null|undefined} ad_customizer_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.ad_customizer_field = null;
    
                            /**
                             * AttributeFieldMapping dsa_page_feed_field.
                             * @member {google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField|null|undefined} dsa_page_feed_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.dsa_page_feed_field = null;
    
                            /**
                             * AttributeFieldMapping location_extension_targeting_field.
                             * @member {google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField|null|undefined} location_extension_targeting_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.location_extension_targeting_field = null;
    
                            /**
                             * AttributeFieldMapping education_field.
                             * @member {google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum.EducationPlaceholderField|null|undefined} education_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.education_field = null;
    
                            /**
                             * AttributeFieldMapping flight_field.
                             * @member {google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum.FlightPlaceholderField|null|undefined} flight_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.flight_field = null;
    
                            /**
                             * AttributeFieldMapping custom_field.
                             * @member {google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum.CustomPlaceholderField|null|undefined} custom_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.custom_field = null;
    
                            /**
                             * AttributeFieldMapping hotel_field.
                             * @member {google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum.HotelPlaceholderField|null|undefined} hotel_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.hotel_field = null;
    
                            /**
                             * AttributeFieldMapping real_estate_field.
                             * @member {google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField|null|undefined} real_estate_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.real_estate_field = null;
    
                            /**
                             * AttributeFieldMapping travel_field.
                             * @member {google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum.TravelPlaceholderField|null|undefined} travel_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.travel_field = null;
    
                            /**
                             * AttributeFieldMapping local_field.
                             * @member {google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum.LocalPlaceholderField|null|undefined} local_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.local_field = null;
    
                            /**
                             * AttributeFieldMapping job_field.
                             * @member {google.ads.googleads.v11.enums.JobPlaceholderFieldEnum.JobPlaceholderField|null|undefined} job_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.job_field = null;
    
                            /**
                             * AttributeFieldMapping image_field.
                             * @member {google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum.ImagePlaceholderField|null|undefined} image_field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            AttributeFieldMapping.prototype.image_field = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AttributeFieldMapping _feed_attribute_id.
                             * @member {"feed_attribute_id"|undefined} _feed_attribute_id
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            Object.defineProperty(AttributeFieldMapping.prototype, "_feed_attribute_id", {
                                get: $util.oneOfGetter($oneOfFields = ["feed_attribute_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AttributeFieldMapping _field_id.
                             * @member {"field_id"|undefined} _field_id
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            Object.defineProperty(AttributeFieldMapping.prototype, "_field_id", {
                                get: $util.oneOfGetter($oneOfFields = ["field_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AttributeFieldMapping field.
                             * @member {"sitelink_field"|"call_field"|"app_field"|"location_field"|"affiliate_location_field"|"callout_field"|"structured_snippet_field"|"message_field"|"price_field"|"promotion_field"|"ad_customizer_field"|"dsa_page_feed_field"|"location_extension_targeting_field"|"education_field"|"flight_field"|"custom_field"|"hotel_field"|"real_estate_field"|"travel_field"|"local_field"|"job_field"|"image_field"|undefined} field
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             */
                            Object.defineProperty(AttributeFieldMapping.prototype, "field", {
                                get: $util.oneOfGetter($oneOfFields = ["sitelink_field", "call_field", "app_field", "location_field", "affiliate_location_field", "callout_field", "structured_snippet_field", "message_field", "price_field", "promotion_field", "ad_customizer_field", "dsa_page_feed_field", "location_extension_targeting_field", "education_field", "flight_field", "custom_field", "hotel_field", "real_estate_field", "travel_field", "local_field", "job_field", "image_field"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AttributeFieldMapping instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAttributeFieldMapping=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.AttributeFieldMapping} AttributeFieldMapping instance
                             */
                            AttributeFieldMapping.create = function create(properties) {
                                return new AttributeFieldMapping(properties);
                            };
    
                            /**
                             * Encodes the specified AttributeFieldMapping message. Does not implicitly {@link google.ads.googleads.v11.resources.AttributeFieldMapping.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAttributeFieldMapping} m AttributeFieldMapping message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AttributeFieldMapping.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.sitelink_field != null && Object.hasOwnProperty.call(m, "sitelink_field"))
                                    w.uint32(24).int32(m.sitelink_field);
                                if (m.call_field != null && Object.hasOwnProperty.call(m, "call_field"))
                                    w.uint32(32).int32(m.call_field);
                                if (m.app_field != null && Object.hasOwnProperty.call(m, "app_field"))
                                    w.uint32(40).int32(m.app_field);
                                if (m.location_field != null && Object.hasOwnProperty.call(m, "location_field"))
                                    w.uint32(48).int32(m.location_field);
                                if (m.affiliate_location_field != null && Object.hasOwnProperty.call(m, "affiliate_location_field"))
                                    w.uint32(56).int32(m.affiliate_location_field);
                                if (m.callout_field != null && Object.hasOwnProperty.call(m, "callout_field"))
                                    w.uint32(64).int32(m.callout_field);
                                if (m.structured_snippet_field != null && Object.hasOwnProperty.call(m, "structured_snippet_field"))
                                    w.uint32(72).int32(m.structured_snippet_field);
                                if (m.message_field != null && Object.hasOwnProperty.call(m, "message_field"))
                                    w.uint32(80).int32(m.message_field);
                                if (m.price_field != null && Object.hasOwnProperty.call(m, "price_field"))
                                    w.uint32(88).int32(m.price_field);
                                if (m.promotion_field != null && Object.hasOwnProperty.call(m, "promotion_field"))
                                    w.uint32(96).int32(m.promotion_field);
                                if (m.ad_customizer_field != null && Object.hasOwnProperty.call(m, "ad_customizer_field"))
                                    w.uint32(104).int32(m.ad_customizer_field);
                                if (m.dsa_page_feed_field != null && Object.hasOwnProperty.call(m, "dsa_page_feed_field"))
                                    w.uint32(112).int32(m.dsa_page_feed_field);
                                if (m.location_extension_targeting_field != null && Object.hasOwnProperty.call(m, "location_extension_targeting_field"))
                                    w.uint32(120).int32(m.location_extension_targeting_field);
                                if (m.education_field != null && Object.hasOwnProperty.call(m, "education_field"))
                                    w.uint32(128).int32(m.education_field);
                                if (m.flight_field != null && Object.hasOwnProperty.call(m, "flight_field"))
                                    w.uint32(136).int32(m.flight_field);
                                if (m.custom_field != null && Object.hasOwnProperty.call(m, "custom_field"))
                                    w.uint32(144).int32(m.custom_field);
                                if (m.hotel_field != null && Object.hasOwnProperty.call(m, "hotel_field"))
                                    w.uint32(152).int32(m.hotel_field);
                                if (m.real_estate_field != null && Object.hasOwnProperty.call(m, "real_estate_field"))
                                    w.uint32(160).int32(m.real_estate_field);
                                if (m.travel_field != null && Object.hasOwnProperty.call(m, "travel_field"))
                                    w.uint32(168).int32(m.travel_field);
                                if (m.local_field != null && Object.hasOwnProperty.call(m, "local_field"))
                                    w.uint32(176).int32(m.local_field);
                                if (m.job_field != null && Object.hasOwnProperty.call(m, "job_field"))
                                    w.uint32(184).int32(m.job_field);
                                if (m.feed_attribute_id != null && Object.hasOwnProperty.call(m, "feed_attribute_id"))
                                    w.uint32(192).int64(m.feed_attribute_id);
                                if (m.field_id != null && Object.hasOwnProperty.call(m, "field_id"))
                                    w.uint32(200).int64(m.field_id);
                                if (m.image_field != null && Object.hasOwnProperty.call(m, "image_field"))
                                    w.uint32(208).int32(m.image_field);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AttributeFieldMapping message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.AttributeFieldMapping.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.IAttributeFieldMapping} message AttributeFieldMapping message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AttributeFieldMapping.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AttributeFieldMapping message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.AttributeFieldMapping} AttributeFieldMapping
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AttributeFieldMapping.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.AttributeFieldMapping();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 24:
                                        m.feed_attribute_id = r.int64();
                                        break;
                                    case 25:
                                        m.field_id = r.int64();
                                        break;
                                    case 3:
                                        m.sitelink_field = r.int32();
                                        break;
                                    case 4:
                                        m.call_field = r.int32();
                                        break;
                                    case 5:
                                        m.app_field = r.int32();
                                        break;
                                    case 6:
                                        m.location_field = r.int32();
                                        break;
                                    case 7:
                                        m.affiliate_location_field = r.int32();
                                        break;
                                    case 8:
                                        m.callout_field = r.int32();
                                        break;
                                    case 9:
                                        m.structured_snippet_field = r.int32();
                                        break;
                                    case 10:
                                        m.message_field = r.int32();
                                        break;
                                    case 11:
                                        m.price_field = r.int32();
                                        break;
                                    case 12:
                                        m.promotion_field = r.int32();
                                        break;
                                    case 13:
                                        m.ad_customizer_field = r.int32();
                                        break;
                                    case 14:
                                        m.dsa_page_feed_field = r.int32();
                                        break;
                                    case 15:
                                        m.location_extension_targeting_field = r.int32();
                                        break;
                                    case 16:
                                        m.education_field = r.int32();
                                        break;
                                    case 17:
                                        m.flight_field = r.int32();
                                        break;
                                    case 18:
                                        m.custom_field = r.int32();
                                        break;
                                    case 19:
                                        m.hotel_field = r.int32();
                                        break;
                                    case 20:
                                        m.real_estate_field = r.int32();
                                        break;
                                    case 21:
                                        m.travel_field = r.int32();
                                        break;
                                    case 22:
                                        m.local_field = r.int32();
                                        break;
                                    case 23:
                                        m.job_field = r.int32();
                                        break;
                                    case 26:
                                        m.image_field = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AttributeFieldMapping message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.AttributeFieldMapping} AttributeFieldMapping
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AttributeFieldMapping.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AttributeFieldMapping message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AttributeFieldMapping.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.feed_attribute_id != null && m.hasOwnProperty("feed_attribute_id")) {
                                    p._feed_attribute_id = 1;
                                    if (!$util.isInteger(m.feed_attribute_id) && !(m.feed_attribute_id && $util.isInteger(m.feed_attribute_id.low) && $util.isInteger(m.feed_attribute_id.high)))
                                        return "feed_attribute_id: integer|Long expected";
                                }
                                if (m.field_id != null && m.hasOwnProperty("field_id")) {
                                    p._field_id = 1;
                                    if (!$util.isInteger(m.field_id) && !(m.field_id && $util.isInteger(m.field_id.low) && $util.isInteger(m.field_id.high)))
                                        return "field_id: integer|Long expected";
                                }
                                if (m.sitelink_field != null && m.hasOwnProperty("sitelink_field")) {
                                    p.field = 1;
                                    switch (m.sitelink_field) {
                                    default:
                                        return "sitelink_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                        break;
                                    }
                                }
                                if (m.call_field != null && m.hasOwnProperty("call_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.call_field) {
                                    default:
                                        return "call_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.app_field != null && m.hasOwnProperty("app_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.app_field) {
                                    default:
                                        return "app_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.location_field != null && m.hasOwnProperty("location_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.location_field) {
                                    default:
                                        return "location_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                        break;
                                    }
                                }
                                if (m.affiliate_location_field != null && m.hasOwnProperty("affiliate_location_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.affiliate_location_field) {
                                    default:
                                        return "affiliate_location_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.callout_field != null && m.hasOwnProperty("callout_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.callout_field) {
                                    default:
                                        return "callout_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.structured_snippet_field != null && m.hasOwnProperty("structured_snippet_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.structured_snippet_field) {
                                    default:
                                        return "structured_snippet_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.message_field != null && m.hasOwnProperty("message_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.message_field) {
                                    default:
                                        return "message_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.price_field != null && m.hasOwnProperty("price_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.price_field) {
                                    default:
                                        return "price_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 100:
                                    case 101:
                                    case 102:
                                    case 103:
                                    case 104:
                                    case 105:
                                    case 200:
                                    case 201:
                                    case 202:
                                    case 203:
                                    case 204:
                                    case 205:
                                    case 300:
                                    case 301:
                                    case 302:
                                    case 303:
                                    case 304:
                                    case 305:
                                    case 400:
                                    case 401:
                                    case 402:
                                    case 403:
                                    case 404:
                                    case 405:
                                    case 500:
                                    case 501:
                                    case 502:
                                    case 503:
                                    case 504:
                                    case 505:
                                    case 600:
                                    case 601:
                                    case 602:
                                    case 603:
                                    case 604:
                                    case 605:
                                    case 700:
                                    case 701:
                                    case 702:
                                    case 703:
                                    case 704:
                                    case 705:
                                    case 800:
                                    case 801:
                                    case 802:
                                    case 803:
                                    case 804:
                                    case 805:
                                        break;
                                    }
                                }
                                if (m.promotion_field != null && m.hasOwnProperty("promotion_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.promotion_field) {
                                    default:
                                        return "promotion_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                        break;
                                    }
                                }
                                if (m.ad_customizer_field != null && m.hasOwnProperty("ad_customizer_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.ad_customizer_field) {
                                    default:
                                        return "ad_customizer_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.dsa_page_feed_field != null && m.hasOwnProperty("dsa_page_feed_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.dsa_page_feed_field) {
                                    default:
                                        return "dsa_page_feed_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.location_extension_targeting_field != null && m.hasOwnProperty("location_extension_targeting_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.location_extension_targeting_field) {
                                    default:
                                        return "location_extension_targeting_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.education_field != null && m.hasOwnProperty("education_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.education_field) {
                                    default:
                                        return "education_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                        break;
                                    }
                                }
                                if (m.flight_field != null && m.hasOwnProperty("flight_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.flight_field) {
                                    default:
                                        return "flight_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                        break;
                                    }
                                }
                                if (m.custom_field != null && m.hasOwnProperty("custom_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.custom_field) {
                                    default:
                                        return "custom_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.hotel_field != null && m.hasOwnProperty("hotel_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.hotel_field) {
                                    default:
                                        return "hotel_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.real_estate_field != null && m.hasOwnProperty("real_estate_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.real_estate_field) {
                                    default:
                                        return "real_estate_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                        break;
                                    }
                                }
                                if (m.travel_field != null && m.hasOwnProperty("travel_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.travel_field) {
                                    default:
                                        return "travel_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                if (m.local_field != null && m.hasOwnProperty("local_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.local_field) {
                                    default:
                                        return "local_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                        break;
                                    }
                                }
                                if (m.job_field != null && m.hasOwnProperty("job_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.job_field) {
                                    default:
                                        return "job_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                        break;
                                    }
                                }
                                if (m.image_field != null && m.hasOwnProperty("image_field")) {
                                    if (p.field === 1)
                                        return "field: multiple values";
                                    p.field = 1;
                                    switch (m.image_field) {
                                    default:
                                        return "image_field: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AttributeFieldMapping message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.AttributeFieldMapping} AttributeFieldMapping
                             */
                            AttributeFieldMapping.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.AttributeFieldMapping)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.AttributeFieldMapping();
                                if (d.feed_attribute_id != null) {
                                    if ($util.Long)
                                        (m.feed_attribute_id = $util.Long.fromValue(d.feed_attribute_id)).unsigned = false;
                                    else if (typeof d.feed_attribute_id === "string")
                                        m.feed_attribute_id = parseInt(d.feed_attribute_id, 10);
                                    else if (typeof d.feed_attribute_id === "number")
                                        m.feed_attribute_id = d.feed_attribute_id;
                                    else if (typeof d.feed_attribute_id === "object")
                                        m.feed_attribute_id = new $util.LongBits(d.feed_attribute_id.low >>> 0, d.feed_attribute_id.high >>> 0).toNumber();
                                }
                                if (d.field_id != null) {
                                    if ($util.Long)
                                        (m.field_id = $util.Long.fromValue(d.field_id)).unsigned = false;
                                    else if (typeof d.field_id === "string")
                                        m.field_id = parseInt(d.field_id, 10);
                                    else if (typeof d.field_id === "number")
                                        m.field_id = d.field_id;
                                    else if (typeof d.field_id === "object")
                                        m.field_id = new $util.LongBits(d.field_id.low >>> 0, d.field_id.high >>> 0).toNumber();
                                }
                                switch (d.sitelink_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.sitelink_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.sitelink_field = 1;
                                    break;
                                case "TEXT":
                                case 2:
                                    m.sitelink_field = 2;
                                    break;
                                case "LINE_1":
                                case 3:
                                    m.sitelink_field = 3;
                                    break;
                                case "LINE_2":
                                case 4:
                                    m.sitelink_field = 4;
                                    break;
                                case "FINAL_URLS":
                                case 5:
                                    m.sitelink_field = 5;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 6:
                                    m.sitelink_field = 6;
                                    break;
                                case "TRACKING_URL":
                                case 7:
                                    m.sitelink_field = 7;
                                    break;
                                case "FINAL_URL_SUFFIX":
                                case 8:
                                    m.sitelink_field = 8;
                                    break;
                                }
                                switch (d.call_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.call_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.call_field = 1;
                                    break;
                                case "PHONE_NUMBER":
                                case 2:
                                    m.call_field = 2;
                                    break;
                                case "COUNTRY_CODE":
                                case 3:
                                    m.call_field = 3;
                                    break;
                                case "TRACKED":
                                case 4:
                                    m.call_field = 4;
                                    break;
                                case "CONVERSION_TYPE_ID":
                                case 5:
                                    m.call_field = 5;
                                    break;
                                case "CONVERSION_REPORTING_STATE":
                                case 6:
                                    m.call_field = 6;
                                    break;
                                }
                                switch (d.app_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.app_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.app_field = 1;
                                    break;
                                case "STORE":
                                case 2:
                                    m.app_field = 2;
                                    break;
                                case "ID":
                                case 3:
                                    m.app_field = 3;
                                    break;
                                case "LINK_TEXT":
                                case 4:
                                    m.app_field = 4;
                                    break;
                                case "URL":
                                case 5:
                                    m.app_field = 5;
                                    break;
                                case "FINAL_URLS":
                                case 6:
                                    m.app_field = 6;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 7:
                                    m.app_field = 7;
                                    break;
                                case "TRACKING_URL":
                                case 8:
                                    m.app_field = 8;
                                    break;
                                case "FINAL_URL_SUFFIX":
                                case 9:
                                    m.app_field = 9;
                                    break;
                                }
                                switch (d.location_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.location_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.location_field = 1;
                                    break;
                                case "BUSINESS_NAME":
                                case 2:
                                    m.location_field = 2;
                                    break;
                                case "ADDRESS_LINE_1":
                                case 3:
                                    m.location_field = 3;
                                    break;
                                case "ADDRESS_LINE_2":
                                case 4:
                                    m.location_field = 4;
                                    break;
                                case "CITY":
                                case 5:
                                    m.location_field = 5;
                                    break;
                                case "PROVINCE":
                                case 6:
                                    m.location_field = 6;
                                    break;
                                case "POSTAL_CODE":
                                case 7:
                                    m.location_field = 7;
                                    break;
                                case "COUNTRY_CODE":
                                case 8:
                                    m.location_field = 8;
                                    break;
                                case "PHONE_NUMBER":
                                case 9:
                                    m.location_field = 9;
                                    break;
                                }
                                switch (d.affiliate_location_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.affiliate_location_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.affiliate_location_field = 1;
                                    break;
                                case "BUSINESS_NAME":
                                case 2:
                                    m.affiliate_location_field = 2;
                                    break;
                                case "ADDRESS_LINE_1":
                                case 3:
                                    m.affiliate_location_field = 3;
                                    break;
                                case "ADDRESS_LINE_2":
                                case 4:
                                    m.affiliate_location_field = 4;
                                    break;
                                case "CITY":
                                case 5:
                                    m.affiliate_location_field = 5;
                                    break;
                                case "PROVINCE":
                                case 6:
                                    m.affiliate_location_field = 6;
                                    break;
                                case "POSTAL_CODE":
                                case 7:
                                    m.affiliate_location_field = 7;
                                    break;
                                case "COUNTRY_CODE":
                                case 8:
                                    m.affiliate_location_field = 8;
                                    break;
                                case "PHONE_NUMBER":
                                case 9:
                                    m.affiliate_location_field = 9;
                                    break;
                                case "LANGUAGE_CODE":
                                case 10:
                                    m.affiliate_location_field = 10;
                                    break;
                                case "CHAIN_ID":
                                case 11:
                                    m.affiliate_location_field = 11;
                                    break;
                                case "CHAIN_NAME":
                                case 12:
                                    m.affiliate_location_field = 12;
                                    break;
                                }
                                switch (d.callout_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.callout_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.callout_field = 1;
                                    break;
                                case "CALLOUT_TEXT":
                                case 2:
                                    m.callout_field = 2;
                                    break;
                                }
                                switch (d.structured_snippet_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.structured_snippet_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.structured_snippet_field = 1;
                                    break;
                                case "HEADER":
                                case 2:
                                    m.structured_snippet_field = 2;
                                    break;
                                case "SNIPPETS":
                                case 3:
                                    m.structured_snippet_field = 3;
                                    break;
                                }
                                switch (d.message_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.message_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.message_field = 1;
                                    break;
                                case "BUSINESS_NAME":
                                case 2:
                                    m.message_field = 2;
                                    break;
                                case "COUNTRY_CODE":
                                case 3:
                                    m.message_field = 3;
                                    break;
                                case "PHONE_NUMBER":
                                case 4:
                                    m.message_field = 4;
                                    break;
                                case "MESSAGE_EXTENSION_TEXT":
                                case 5:
                                    m.message_field = 5;
                                    break;
                                case "MESSAGE_TEXT":
                                case 6:
                                    m.message_field = 6;
                                    break;
                                }
                                switch (d.price_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.price_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.price_field = 1;
                                    break;
                                case "TYPE":
                                case 2:
                                    m.price_field = 2;
                                    break;
                                case "PRICE_QUALIFIER":
                                case 3:
                                    m.price_field = 3;
                                    break;
                                case "TRACKING_TEMPLATE":
                                case 4:
                                    m.price_field = 4;
                                    break;
                                case "LANGUAGE":
                                case 5:
                                    m.price_field = 5;
                                    break;
                                case "FINAL_URL_SUFFIX":
                                case 6:
                                    m.price_field = 6;
                                    break;
                                case "ITEM_1_HEADER":
                                case 100:
                                    m.price_field = 100;
                                    break;
                                case "ITEM_1_DESCRIPTION":
                                case 101:
                                    m.price_field = 101;
                                    break;
                                case "ITEM_1_PRICE":
                                case 102:
                                    m.price_field = 102;
                                    break;
                                case "ITEM_1_UNIT":
                                case 103:
                                    m.price_field = 103;
                                    break;
                                case "ITEM_1_FINAL_URLS":
                                case 104:
                                    m.price_field = 104;
                                    break;
                                case "ITEM_1_FINAL_MOBILE_URLS":
                                case 105:
                                    m.price_field = 105;
                                    break;
                                case "ITEM_2_HEADER":
                                case 200:
                                    m.price_field = 200;
                                    break;
                                case "ITEM_2_DESCRIPTION":
                                case 201:
                                    m.price_field = 201;
                                    break;
                                case "ITEM_2_PRICE":
                                case 202:
                                    m.price_field = 202;
                                    break;
                                case "ITEM_2_UNIT":
                                case 203:
                                    m.price_field = 203;
                                    break;
                                case "ITEM_2_FINAL_URLS":
                                case 204:
                                    m.price_field = 204;
                                    break;
                                case "ITEM_2_FINAL_MOBILE_URLS":
                                case 205:
                                    m.price_field = 205;
                                    break;
                                case "ITEM_3_HEADER":
                                case 300:
                                    m.price_field = 300;
                                    break;
                                case "ITEM_3_DESCRIPTION":
                                case 301:
                                    m.price_field = 301;
                                    break;
                                case "ITEM_3_PRICE":
                                case 302:
                                    m.price_field = 302;
                                    break;
                                case "ITEM_3_UNIT":
                                case 303:
                                    m.price_field = 303;
                                    break;
                                case "ITEM_3_FINAL_URLS":
                                case 304:
                                    m.price_field = 304;
                                    break;
                                case "ITEM_3_FINAL_MOBILE_URLS":
                                case 305:
                                    m.price_field = 305;
                                    break;
                                case "ITEM_4_HEADER":
                                case 400:
                                    m.price_field = 400;
                                    break;
                                case "ITEM_4_DESCRIPTION":
                                case 401:
                                    m.price_field = 401;
                                    break;
                                case "ITEM_4_PRICE":
                                case 402:
                                    m.price_field = 402;
                                    break;
                                case "ITEM_4_UNIT":
                                case 403:
                                    m.price_field = 403;
                                    break;
                                case "ITEM_4_FINAL_URLS":
                                case 404:
                                    m.price_field = 404;
                                    break;
                                case "ITEM_4_FINAL_MOBILE_URLS":
                                case 405:
                                    m.price_field = 405;
                                    break;
                                case "ITEM_5_HEADER":
                                case 500:
                                    m.price_field = 500;
                                    break;
                                case "ITEM_5_DESCRIPTION":
                                case 501:
                                    m.price_field = 501;
                                    break;
                                case "ITEM_5_PRICE":
                                case 502:
                                    m.price_field = 502;
                                    break;
                                case "ITEM_5_UNIT":
                                case 503:
                                    m.price_field = 503;
                                    break;
                                case "ITEM_5_FINAL_URLS":
                                case 504:
                                    m.price_field = 504;
                                    break;
                                case "ITEM_5_FINAL_MOBILE_URLS":
                                case 505:
                                    m.price_field = 505;
                                    break;
                                case "ITEM_6_HEADER":
                                case 600:
                                    m.price_field = 600;
                                    break;
                                case "ITEM_6_DESCRIPTION":
                                case 601:
                                    m.price_field = 601;
                                    break;
                                case "ITEM_6_PRICE":
                                case 602:
                                    m.price_field = 602;
                                    break;
                                case "ITEM_6_UNIT":
                                case 603:
                                    m.price_field = 603;
                                    break;
                                case "ITEM_6_FINAL_URLS":
                                case 604:
                                    m.price_field = 604;
                                    break;
                                case "ITEM_6_FINAL_MOBILE_URLS":
                                case 605:
                                    m.price_field = 605;
                                    break;
                                case "ITEM_7_HEADER":
                                case 700:
                                    m.price_field = 700;
                                    break;
                                case "ITEM_7_DESCRIPTION":
                                case 701:
                                    m.price_field = 701;
                                    break;
                                case "ITEM_7_PRICE":
                                case 702:
                                    m.price_field = 702;
                                    break;
                                case "ITEM_7_UNIT":
                                case 703:
                                    m.price_field = 703;
                                    break;
                                case "ITEM_7_FINAL_URLS":
                                case 704:
                                    m.price_field = 704;
                                    break;
                                case "ITEM_7_FINAL_MOBILE_URLS":
                                case 705:
                                    m.price_field = 705;
                                    break;
                                case "ITEM_8_HEADER":
                                case 800:
                                    m.price_field = 800;
                                    break;
                                case "ITEM_8_DESCRIPTION":
                                case 801:
                                    m.price_field = 801;
                                    break;
                                case "ITEM_8_PRICE":
                                case 802:
                                    m.price_field = 802;
                                    break;
                                case "ITEM_8_UNIT":
                                case 803:
                                    m.price_field = 803;
                                    break;
                                case "ITEM_8_FINAL_URLS":
                                case 804:
                                    m.price_field = 804;
                                    break;
                                case "ITEM_8_FINAL_MOBILE_URLS":
                                case 805:
                                    m.price_field = 805;
                                    break;
                                }
                                switch (d.promotion_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.promotion_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.promotion_field = 1;
                                    break;
                                case "PROMOTION_TARGET":
                                case 2:
                                    m.promotion_field = 2;
                                    break;
                                case "DISCOUNT_MODIFIER":
                                case 3:
                                    m.promotion_field = 3;
                                    break;
                                case "PERCENT_OFF":
                                case 4:
                                    m.promotion_field = 4;
                                    break;
                                case "MONEY_AMOUNT_OFF":
                                case 5:
                                    m.promotion_field = 5;
                                    break;
                                case "PROMOTION_CODE":
                                case 6:
                                    m.promotion_field = 6;
                                    break;
                                case "ORDERS_OVER_AMOUNT":
                                case 7:
                                    m.promotion_field = 7;
                                    break;
                                case "PROMOTION_START":
                                case 8:
                                    m.promotion_field = 8;
                                    break;
                                case "PROMOTION_END":
                                case 9:
                                    m.promotion_field = 9;
                                    break;
                                case "OCCASION":
                                case 10:
                                    m.promotion_field = 10;
                                    break;
                                case "FINAL_URLS":
                                case 11:
                                    m.promotion_field = 11;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 12:
                                    m.promotion_field = 12;
                                    break;
                                case "TRACKING_URL":
                                case 13:
                                    m.promotion_field = 13;
                                    break;
                                case "LANGUAGE":
                                case 14:
                                    m.promotion_field = 14;
                                    break;
                                case "FINAL_URL_SUFFIX":
                                case 15:
                                    m.promotion_field = 15;
                                    break;
                                }
                                switch (d.ad_customizer_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_customizer_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_customizer_field = 1;
                                    break;
                                case "INTEGER":
                                case 2:
                                    m.ad_customizer_field = 2;
                                    break;
                                case "PRICE":
                                case 3:
                                    m.ad_customizer_field = 3;
                                    break;
                                case "DATE":
                                case 4:
                                    m.ad_customizer_field = 4;
                                    break;
                                case "STRING":
                                case 5:
                                    m.ad_customizer_field = 5;
                                    break;
                                }
                                switch (d.dsa_page_feed_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.dsa_page_feed_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.dsa_page_feed_field = 1;
                                    break;
                                case "PAGE_URL":
                                case 2:
                                    m.dsa_page_feed_field = 2;
                                    break;
                                case "LABEL":
                                case 3:
                                    m.dsa_page_feed_field = 3;
                                    break;
                                }
                                switch (d.location_extension_targeting_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.location_extension_targeting_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.location_extension_targeting_field = 1;
                                    break;
                                case "ADDRESS_LINE_1":
                                case 2:
                                    m.location_extension_targeting_field = 2;
                                    break;
                                case "ADDRESS_LINE_2":
                                case 3:
                                    m.location_extension_targeting_field = 3;
                                    break;
                                case "CITY":
                                case 4:
                                    m.location_extension_targeting_field = 4;
                                    break;
                                case "PROVINCE":
                                case 5:
                                    m.location_extension_targeting_field = 5;
                                    break;
                                case "POSTAL_CODE":
                                case 6:
                                    m.location_extension_targeting_field = 6;
                                    break;
                                case "COUNTRY_CODE":
                                case 7:
                                    m.location_extension_targeting_field = 7;
                                    break;
                                }
                                switch (d.education_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.education_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.education_field = 1;
                                    break;
                                case "PROGRAM_ID":
                                case 2:
                                    m.education_field = 2;
                                    break;
                                case "LOCATION_ID":
                                case 3:
                                    m.education_field = 3;
                                    break;
                                case "PROGRAM_NAME":
                                case 4:
                                    m.education_field = 4;
                                    break;
                                case "AREA_OF_STUDY":
                                case 5:
                                    m.education_field = 5;
                                    break;
                                case "PROGRAM_DESCRIPTION":
                                case 6:
                                    m.education_field = 6;
                                    break;
                                case "SCHOOL_NAME":
                                case 7:
                                    m.education_field = 7;
                                    break;
                                case "ADDRESS":
                                case 8:
                                    m.education_field = 8;
                                    break;
                                case "THUMBNAIL_IMAGE_URL":
                                case 9:
                                    m.education_field = 9;
                                    break;
                                case "ALTERNATIVE_THUMBNAIL_IMAGE_URL":
                                case 10:
                                    m.education_field = 10;
                                    break;
                                case "FINAL_URLS":
                                case 11:
                                    m.education_field = 11;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 12:
                                    m.education_field = 12;
                                    break;
                                case "TRACKING_URL":
                                case 13:
                                    m.education_field = 13;
                                    break;
                                case "CONTEXTUAL_KEYWORDS":
                                case 14:
                                    m.education_field = 14;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 15:
                                    m.education_field = 15;
                                    break;
                                case "SIMILAR_PROGRAM_IDS":
                                case 16:
                                    m.education_field = 16;
                                    break;
                                case "IOS_APP_LINK":
                                case 17:
                                    m.education_field = 17;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 18:
                                    m.education_field = 18;
                                    break;
                                }
                                switch (d.flight_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.flight_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.flight_field = 1;
                                    break;
                                case "DESTINATION_ID":
                                case 2:
                                    m.flight_field = 2;
                                    break;
                                case "ORIGIN_ID":
                                case 3:
                                    m.flight_field = 3;
                                    break;
                                case "FLIGHT_DESCRIPTION":
                                case 4:
                                    m.flight_field = 4;
                                    break;
                                case "ORIGIN_NAME":
                                case 5:
                                    m.flight_field = 5;
                                    break;
                                case "DESTINATION_NAME":
                                case 6:
                                    m.flight_field = 6;
                                    break;
                                case "FLIGHT_PRICE":
                                case 7:
                                    m.flight_field = 7;
                                    break;
                                case "FORMATTED_PRICE":
                                case 8:
                                    m.flight_field = 8;
                                    break;
                                case "FLIGHT_SALE_PRICE":
                                case 9:
                                    m.flight_field = 9;
                                    break;
                                case "FORMATTED_SALE_PRICE":
                                case 10:
                                    m.flight_field = 10;
                                    break;
                                case "IMAGE_URL":
                                case 11:
                                    m.flight_field = 11;
                                    break;
                                case "FINAL_URLS":
                                case 12:
                                    m.flight_field = 12;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 13:
                                    m.flight_field = 13;
                                    break;
                                case "TRACKING_URL":
                                case 14:
                                    m.flight_field = 14;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 15:
                                    m.flight_field = 15;
                                    break;
                                case "SIMILAR_DESTINATION_IDS":
                                case 16:
                                    m.flight_field = 16;
                                    break;
                                case "IOS_APP_LINK":
                                case 17:
                                    m.flight_field = 17;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 18:
                                    m.flight_field = 18;
                                    break;
                                }
                                switch (d.custom_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.custom_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.custom_field = 1;
                                    break;
                                case "ID":
                                case 2:
                                    m.custom_field = 2;
                                    break;
                                case "ID2":
                                case 3:
                                    m.custom_field = 3;
                                    break;
                                case "ITEM_TITLE":
                                case 4:
                                    m.custom_field = 4;
                                    break;
                                case "ITEM_SUBTITLE":
                                case 5:
                                    m.custom_field = 5;
                                    break;
                                case "ITEM_DESCRIPTION":
                                case 6:
                                    m.custom_field = 6;
                                    break;
                                case "ITEM_ADDRESS":
                                case 7:
                                    m.custom_field = 7;
                                    break;
                                case "PRICE":
                                case 8:
                                    m.custom_field = 8;
                                    break;
                                case "FORMATTED_PRICE":
                                case 9:
                                    m.custom_field = 9;
                                    break;
                                case "SALE_PRICE":
                                case 10:
                                    m.custom_field = 10;
                                    break;
                                case "FORMATTED_SALE_PRICE":
                                case 11:
                                    m.custom_field = 11;
                                    break;
                                case "IMAGE_URL":
                                case 12:
                                    m.custom_field = 12;
                                    break;
                                case "ITEM_CATEGORY":
                                case 13:
                                    m.custom_field = 13;
                                    break;
                                case "FINAL_URLS":
                                case 14:
                                    m.custom_field = 14;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 15:
                                    m.custom_field = 15;
                                    break;
                                case "TRACKING_URL":
                                case 16:
                                    m.custom_field = 16;
                                    break;
                                case "CONTEXTUAL_KEYWORDS":
                                case 17:
                                    m.custom_field = 17;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 18:
                                    m.custom_field = 18;
                                    break;
                                case "SIMILAR_IDS":
                                case 19:
                                    m.custom_field = 19;
                                    break;
                                case "IOS_APP_LINK":
                                case 20:
                                    m.custom_field = 20;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 21:
                                    m.custom_field = 21;
                                    break;
                                }
                                switch (d.hotel_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.hotel_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.hotel_field = 1;
                                    break;
                                case "PROPERTY_ID":
                                case 2:
                                    m.hotel_field = 2;
                                    break;
                                case "PROPERTY_NAME":
                                case 3:
                                    m.hotel_field = 3;
                                    break;
                                case "DESTINATION_NAME":
                                case 4:
                                    m.hotel_field = 4;
                                    break;
                                case "DESCRIPTION":
                                case 5:
                                    m.hotel_field = 5;
                                    break;
                                case "ADDRESS":
                                case 6:
                                    m.hotel_field = 6;
                                    break;
                                case "PRICE":
                                case 7:
                                    m.hotel_field = 7;
                                    break;
                                case "FORMATTED_PRICE":
                                case 8:
                                    m.hotel_field = 8;
                                    break;
                                case "SALE_PRICE":
                                case 9:
                                    m.hotel_field = 9;
                                    break;
                                case "FORMATTED_SALE_PRICE":
                                case 10:
                                    m.hotel_field = 10;
                                    break;
                                case "IMAGE_URL":
                                case 11:
                                    m.hotel_field = 11;
                                    break;
                                case "CATEGORY":
                                case 12:
                                    m.hotel_field = 12;
                                    break;
                                case "STAR_RATING":
                                case 13:
                                    m.hotel_field = 13;
                                    break;
                                case "CONTEXTUAL_KEYWORDS":
                                case 14:
                                    m.hotel_field = 14;
                                    break;
                                case "FINAL_URLS":
                                case 15:
                                    m.hotel_field = 15;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 16:
                                    m.hotel_field = 16;
                                    break;
                                case "TRACKING_URL":
                                case 17:
                                    m.hotel_field = 17;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 18:
                                    m.hotel_field = 18;
                                    break;
                                case "SIMILAR_PROPERTY_IDS":
                                case 19:
                                    m.hotel_field = 19;
                                    break;
                                case "IOS_APP_LINK":
                                case 20:
                                    m.hotel_field = 20;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 21:
                                    m.hotel_field = 21;
                                    break;
                                }
                                switch (d.real_estate_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.real_estate_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.real_estate_field = 1;
                                    break;
                                case "LISTING_ID":
                                case 2:
                                    m.real_estate_field = 2;
                                    break;
                                case "LISTING_NAME":
                                case 3:
                                    m.real_estate_field = 3;
                                    break;
                                case "CITY_NAME":
                                case 4:
                                    m.real_estate_field = 4;
                                    break;
                                case "DESCRIPTION":
                                case 5:
                                    m.real_estate_field = 5;
                                    break;
                                case "ADDRESS":
                                case 6:
                                    m.real_estate_field = 6;
                                    break;
                                case "PRICE":
                                case 7:
                                    m.real_estate_field = 7;
                                    break;
                                case "FORMATTED_PRICE":
                                case 8:
                                    m.real_estate_field = 8;
                                    break;
                                case "IMAGE_URL":
                                case 9:
                                    m.real_estate_field = 9;
                                    break;
                                case "PROPERTY_TYPE":
                                case 10:
                                    m.real_estate_field = 10;
                                    break;
                                case "LISTING_TYPE":
                                case 11:
                                    m.real_estate_field = 11;
                                    break;
                                case "CONTEXTUAL_KEYWORDS":
                                case 12:
                                    m.real_estate_field = 12;
                                    break;
                                case "FINAL_URLS":
                                case 13:
                                    m.real_estate_field = 13;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 14:
                                    m.real_estate_field = 14;
                                    break;
                                case "TRACKING_URL":
                                case 15:
                                    m.real_estate_field = 15;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 16:
                                    m.real_estate_field = 16;
                                    break;
                                case "SIMILAR_LISTING_IDS":
                                case 17:
                                    m.real_estate_field = 17;
                                    break;
                                case "IOS_APP_LINK":
                                case 18:
                                    m.real_estate_field = 18;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 19:
                                    m.real_estate_field = 19;
                                    break;
                                }
                                switch (d.travel_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.travel_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.travel_field = 1;
                                    break;
                                case "DESTINATION_ID":
                                case 2:
                                    m.travel_field = 2;
                                    break;
                                case "ORIGIN_ID":
                                case 3:
                                    m.travel_field = 3;
                                    break;
                                case "TITLE":
                                case 4:
                                    m.travel_field = 4;
                                    break;
                                case "DESTINATION_NAME":
                                case 5:
                                    m.travel_field = 5;
                                    break;
                                case "ORIGIN_NAME":
                                case 6:
                                    m.travel_field = 6;
                                    break;
                                case "PRICE":
                                case 7:
                                    m.travel_field = 7;
                                    break;
                                case "FORMATTED_PRICE":
                                case 8:
                                    m.travel_field = 8;
                                    break;
                                case "SALE_PRICE":
                                case 9:
                                    m.travel_field = 9;
                                    break;
                                case "FORMATTED_SALE_PRICE":
                                case 10:
                                    m.travel_field = 10;
                                    break;
                                case "IMAGE_URL":
                                case 11:
                                    m.travel_field = 11;
                                    break;
                                case "CATEGORY":
                                case 12:
                                    m.travel_field = 12;
                                    break;
                                case "CONTEXTUAL_KEYWORDS":
                                case 13:
                                    m.travel_field = 13;
                                    break;
                                case "DESTINATION_ADDRESS":
                                case 14:
                                    m.travel_field = 14;
                                    break;
                                case "FINAL_URL":
                                case 15:
                                    m.travel_field = 15;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 16:
                                    m.travel_field = 16;
                                    break;
                                case "TRACKING_URL":
                                case 17:
                                    m.travel_field = 17;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 18:
                                    m.travel_field = 18;
                                    break;
                                case "SIMILAR_DESTINATION_IDS":
                                case 19:
                                    m.travel_field = 19;
                                    break;
                                case "IOS_APP_LINK":
                                case 20:
                                    m.travel_field = 20;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 21:
                                    m.travel_field = 21;
                                    break;
                                }
                                switch (d.local_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.local_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.local_field = 1;
                                    break;
                                case "DEAL_ID":
                                case 2:
                                    m.local_field = 2;
                                    break;
                                case "DEAL_NAME":
                                case 3:
                                    m.local_field = 3;
                                    break;
                                case "SUBTITLE":
                                case 4:
                                    m.local_field = 4;
                                    break;
                                case "DESCRIPTION":
                                case 5:
                                    m.local_field = 5;
                                    break;
                                case "PRICE":
                                case 6:
                                    m.local_field = 6;
                                    break;
                                case "FORMATTED_PRICE":
                                case 7:
                                    m.local_field = 7;
                                    break;
                                case "SALE_PRICE":
                                case 8:
                                    m.local_field = 8;
                                    break;
                                case "FORMATTED_SALE_PRICE":
                                case 9:
                                    m.local_field = 9;
                                    break;
                                case "IMAGE_URL":
                                case 10:
                                    m.local_field = 10;
                                    break;
                                case "ADDRESS":
                                case 11:
                                    m.local_field = 11;
                                    break;
                                case "CATEGORY":
                                case 12:
                                    m.local_field = 12;
                                    break;
                                case "CONTEXTUAL_KEYWORDS":
                                case 13:
                                    m.local_field = 13;
                                    break;
                                case "FINAL_URLS":
                                case 14:
                                    m.local_field = 14;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 15:
                                    m.local_field = 15;
                                    break;
                                case "TRACKING_URL":
                                case 16:
                                    m.local_field = 16;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 17:
                                    m.local_field = 17;
                                    break;
                                case "SIMILAR_DEAL_IDS":
                                case 18:
                                    m.local_field = 18;
                                    break;
                                case "IOS_APP_LINK":
                                case 19:
                                    m.local_field = 19;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 20:
                                    m.local_field = 20;
                                    break;
                                }
                                switch (d.job_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.job_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.job_field = 1;
                                    break;
                                case "JOB_ID":
                                case 2:
                                    m.job_field = 2;
                                    break;
                                case "LOCATION_ID":
                                case 3:
                                    m.job_field = 3;
                                    break;
                                case "TITLE":
                                case 4:
                                    m.job_field = 4;
                                    break;
                                case "SUBTITLE":
                                case 5:
                                    m.job_field = 5;
                                    break;
                                case "DESCRIPTION":
                                case 6:
                                    m.job_field = 6;
                                    break;
                                case "IMAGE_URL":
                                case 7:
                                    m.job_field = 7;
                                    break;
                                case "CATEGORY":
                                case 8:
                                    m.job_field = 8;
                                    break;
                                case "CONTEXTUAL_KEYWORDS":
                                case 9:
                                    m.job_field = 9;
                                    break;
                                case "ADDRESS":
                                case 10:
                                    m.job_field = 10;
                                    break;
                                case "SALARY":
                                case 11:
                                    m.job_field = 11;
                                    break;
                                case "FINAL_URLS":
                                case 12:
                                    m.job_field = 12;
                                    break;
                                case "FINAL_MOBILE_URLS":
                                case 14:
                                    m.job_field = 14;
                                    break;
                                case "TRACKING_URL":
                                case 15:
                                    m.job_field = 15;
                                    break;
                                case "ANDROID_APP_LINK":
                                case 16:
                                    m.job_field = 16;
                                    break;
                                case "SIMILAR_JOB_IDS":
                                case 17:
                                    m.job_field = 17;
                                    break;
                                case "IOS_APP_LINK":
                                case 18:
                                    m.job_field = 18;
                                    break;
                                case "IOS_APP_STORE_ID":
                                case 19:
                                    m.job_field = 19;
                                    break;
                                }
                                switch (d.image_field) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.image_field = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.image_field = 1;
                                    break;
                                case "ASSET_ID":
                                case 2:
                                    m.image_field = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AttributeFieldMapping message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @static
                             * @param {google.ads.googleads.v11.resources.AttributeFieldMapping} m AttributeFieldMapping
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AttributeFieldMapping.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.sitelink_field != null && m.hasOwnProperty("sitelink_field")) {
                                    d.sitelink_field = o.enums === String ? $root.google.ads.googleads.v11.enums.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField[m.sitelink_field] : m.sitelink_field;
                                    if (o.oneofs)
                                        d.field = "sitelink_field";
                                }
                                if (m.call_field != null && m.hasOwnProperty("call_field")) {
                                    d.call_field = o.enums === String ? $root.google.ads.googleads.v11.enums.CallPlaceholderFieldEnum.CallPlaceholderField[m.call_field] : m.call_field;
                                    if (o.oneofs)
                                        d.field = "call_field";
                                }
                                if (m.app_field != null && m.hasOwnProperty("app_field")) {
                                    d.app_field = o.enums === String ? $root.google.ads.googleads.v11.enums.AppPlaceholderFieldEnum.AppPlaceholderField[m.app_field] : m.app_field;
                                    if (o.oneofs)
                                        d.field = "app_field";
                                }
                                if (m.location_field != null && m.hasOwnProperty("location_field")) {
                                    d.location_field = o.enums === String ? $root.google.ads.googleads.v11.enums.LocationPlaceholderFieldEnum.LocationPlaceholderField[m.location_field] : m.location_field;
                                    if (o.oneofs)
                                        d.field = "location_field";
                                }
                                if (m.affiliate_location_field != null && m.hasOwnProperty("affiliate_location_field")) {
                                    d.affiliate_location_field = o.enums === String ? $root.google.ads.googleads.v11.enums.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField[m.affiliate_location_field] : m.affiliate_location_field;
                                    if (o.oneofs)
                                        d.field = "affiliate_location_field";
                                }
                                if (m.callout_field != null && m.hasOwnProperty("callout_field")) {
                                    d.callout_field = o.enums === String ? $root.google.ads.googleads.v11.enums.CalloutPlaceholderFieldEnum.CalloutPlaceholderField[m.callout_field] : m.callout_field;
                                    if (o.oneofs)
                                        d.field = "callout_field";
                                }
                                if (m.structured_snippet_field != null && m.hasOwnProperty("structured_snippet_field")) {
                                    d.structured_snippet_field = o.enums === String ? $root.google.ads.googleads.v11.enums.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField[m.structured_snippet_field] : m.structured_snippet_field;
                                    if (o.oneofs)
                                        d.field = "structured_snippet_field";
                                }
                                if (m.message_field != null && m.hasOwnProperty("message_field")) {
                                    d.message_field = o.enums === String ? $root.google.ads.googleads.v11.enums.MessagePlaceholderFieldEnum.MessagePlaceholderField[m.message_field] : m.message_field;
                                    if (o.oneofs)
                                        d.field = "message_field";
                                }
                                if (m.price_field != null && m.hasOwnProperty("price_field")) {
                                    d.price_field = o.enums === String ? $root.google.ads.googleads.v11.enums.PricePlaceholderFieldEnum.PricePlaceholderField[m.price_field] : m.price_field;
                                    if (o.oneofs)
                                        d.field = "price_field";
                                }
                                if (m.promotion_field != null && m.hasOwnProperty("promotion_field")) {
                                    d.promotion_field = o.enums === String ? $root.google.ads.googleads.v11.enums.PromotionPlaceholderFieldEnum.PromotionPlaceholderField[m.promotion_field] : m.promotion_field;
                                    if (o.oneofs)
                                        d.field = "promotion_field";
                                }
                                if (m.ad_customizer_field != null && m.hasOwnProperty("ad_customizer_field")) {
                                    d.ad_customizer_field = o.enums === String ? $root.google.ads.googleads.v11.enums.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField[m.ad_customizer_field] : m.ad_customizer_field;
                                    if (o.oneofs)
                                        d.field = "ad_customizer_field";
                                }
                                if (m.dsa_page_feed_field != null && m.hasOwnProperty("dsa_page_feed_field")) {
                                    d.dsa_page_feed_field = o.enums === String ? $root.google.ads.googleads.v11.enums.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField[m.dsa_page_feed_field] : m.dsa_page_feed_field;
                                    if (o.oneofs)
                                        d.field = "dsa_page_feed_field";
                                }
                                if (m.location_extension_targeting_field != null && m.hasOwnProperty("location_extension_targeting_field")) {
                                    d.location_extension_targeting_field = o.enums === String ? $root.google.ads.googleads.v11.enums.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField[m.location_extension_targeting_field] : m.location_extension_targeting_field;
                                    if (o.oneofs)
                                        d.field = "location_extension_targeting_field";
                                }
                                if (m.education_field != null && m.hasOwnProperty("education_field")) {
                                    d.education_field = o.enums === String ? $root.google.ads.googleads.v11.enums.EducationPlaceholderFieldEnum.EducationPlaceholderField[m.education_field] : m.education_field;
                                    if (o.oneofs)
                                        d.field = "education_field";
                                }
                                if (m.flight_field != null && m.hasOwnProperty("flight_field")) {
                                    d.flight_field = o.enums === String ? $root.google.ads.googleads.v11.enums.FlightPlaceholderFieldEnum.FlightPlaceholderField[m.flight_field] : m.flight_field;
                                    if (o.oneofs)
                                        d.field = "flight_field";
                                }
                                if (m.custom_field != null && m.hasOwnProperty("custom_field")) {
                                    d.custom_field = o.enums === String ? $root.google.ads.googleads.v11.enums.CustomPlaceholderFieldEnum.CustomPlaceholderField[m.custom_field] : m.custom_field;
                                    if (o.oneofs)
                                        d.field = "custom_field";
                                }
                                if (m.hotel_field != null && m.hasOwnProperty("hotel_field")) {
                                    d.hotel_field = o.enums === String ? $root.google.ads.googleads.v11.enums.HotelPlaceholderFieldEnum.HotelPlaceholderField[m.hotel_field] : m.hotel_field;
                                    if (o.oneofs)
                                        d.field = "hotel_field";
                                }
                                if (m.real_estate_field != null && m.hasOwnProperty("real_estate_field")) {
                                    d.real_estate_field = o.enums === String ? $root.google.ads.googleads.v11.enums.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField[m.real_estate_field] : m.real_estate_field;
                                    if (o.oneofs)
                                        d.field = "real_estate_field";
                                }
                                if (m.travel_field != null && m.hasOwnProperty("travel_field")) {
                                    d.travel_field = o.enums === String ? $root.google.ads.googleads.v11.enums.TravelPlaceholderFieldEnum.TravelPlaceholderField[m.travel_field] : m.travel_field;
                                    if (o.oneofs)
                                        d.field = "travel_field";
                                }
                                if (m.local_field != null && m.hasOwnProperty("local_field")) {
                                    d.local_field = o.enums === String ? $root.google.ads.googleads.v11.enums.LocalPlaceholderFieldEnum.LocalPlaceholderField[m.local_field] : m.local_field;
                                    if (o.oneofs)
                                        d.field = "local_field";
                                }
                                if (m.job_field != null && m.hasOwnProperty("job_field")) {
                                    d.job_field = o.enums === String ? $root.google.ads.googleads.v11.enums.JobPlaceholderFieldEnum.JobPlaceholderField[m.job_field] : m.job_field;
                                    if (o.oneofs)
                                        d.field = "job_field";
                                }
                                if (m.feed_attribute_id != null && m.hasOwnProperty("feed_attribute_id")) {
                                    if (typeof m.feed_attribute_id === "number")
                                        d.feed_attribute_id = o.longs === String ? String(m.feed_attribute_id) : m.feed_attribute_id;
                                    else
                                        d.feed_attribute_id = o.longs === String ? $util.Long.prototype.toString.call(m.feed_attribute_id) : o.longs === Number ? new $util.LongBits(m.feed_attribute_id.low >>> 0, m.feed_attribute_id.high >>> 0).toNumber() : m.feed_attribute_id;
                                    if (o.oneofs)
                                        d._feed_attribute_id = "feed_attribute_id";
                                }
                                if (m.field_id != null && m.hasOwnProperty("field_id")) {
                                    if (typeof m.field_id === "number")
                                        d.field_id = o.longs === String ? String(m.field_id) : m.field_id;
                                    else
                                        d.field_id = o.longs === String ? $util.Long.prototype.toString.call(m.field_id) : o.longs === Number ? new $util.LongBits(m.field_id.low >>> 0, m.field_id.high >>> 0).toNumber() : m.field_id;
                                    if (o.oneofs)
                                        d._field_id = "field_id";
                                }
                                if (m.image_field != null && m.hasOwnProperty("image_field")) {
                                    d.image_field = o.enums === String ? $root.google.ads.googleads.v11.enums.ImagePlaceholderFieldEnum.ImagePlaceholderField[m.image_field] : m.image_field;
                                    if (o.oneofs)
                                        d.field = "image_field";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AttributeFieldMapping to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.AttributeFieldMapping
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AttributeFieldMapping.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AttributeFieldMapping;
                        })();
    
                        resources.FeedPlaceholderView = (function() {
    
                            /**
                             * Properties of a FeedPlaceholderView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IFeedPlaceholderView
                             * @property {string|null} [resource_name] FeedPlaceholderView resource_name
                             * @property {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType|null} [placeholder_type] FeedPlaceholderView placeholder_type
                             */
    
                            /**
                             * Constructs a new FeedPlaceholderView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a FeedPlaceholderView.
                             * @implements IFeedPlaceholderView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IFeedPlaceholderView=} [p] Properties to set
                             */
                            function FeedPlaceholderView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedPlaceholderView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @instance
                             */
                            FeedPlaceholderView.prototype.resource_name = "";
    
                            /**
                             * FeedPlaceholderView placeholder_type.
                             * @member {google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType} placeholder_type
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @instance
                             */
                            FeedPlaceholderView.prototype.placeholder_type = 0;
    
                            /**
                             * Creates a new FeedPlaceholderView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedPlaceholderView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.FeedPlaceholderView} FeedPlaceholderView instance
                             */
                            FeedPlaceholderView.create = function create(properties) {
                                return new FeedPlaceholderView(properties);
                            };
    
                            /**
                             * Encodes the specified FeedPlaceholderView message. Does not implicitly {@link google.ads.googleads.v11.resources.FeedPlaceholderView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedPlaceholderView} m FeedPlaceholderView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedPlaceholderView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.placeholder_type != null && Object.hasOwnProperty.call(m, "placeholder_type"))
                                    w.uint32(16).int32(m.placeholder_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedPlaceholderView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.FeedPlaceholderView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IFeedPlaceholderView} message FeedPlaceholderView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedPlaceholderView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedPlaceholderView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.FeedPlaceholderView} FeedPlaceholderView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedPlaceholderView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.FeedPlaceholderView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.placeholder_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedPlaceholderView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.FeedPlaceholderView} FeedPlaceholderView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedPlaceholderView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedPlaceholderView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedPlaceholderView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.placeholder_type != null && m.hasOwnProperty("placeholder_type")) {
                                    switch (m.placeholder_type) {
                                    default:
                                        return "placeholder_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedPlaceholderView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.FeedPlaceholderView} FeedPlaceholderView
                             */
                            FeedPlaceholderView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.FeedPlaceholderView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.FeedPlaceholderView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.placeholder_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.placeholder_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.placeholder_type = 1;
                                    break;
                                case "SITELINK":
                                case 2:
                                    m.placeholder_type = 2;
                                    break;
                                case "CALL":
                                case 3:
                                    m.placeholder_type = 3;
                                    break;
                                case "APP":
                                case 4:
                                    m.placeholder_type = 4;
                                    break;
                                case "LOCATION":
                                case 5:
                                    m.placeholder_type = 5;
                                    break;
                                case "AFFILIATE_LOCATION":
                                case 6:
                                    m.placeholder_type = 6;
                                    break;
                                case "CALLOUT":
                                case 7:
                                    m.placeholder_type = 7;
                                    break;
                                case "STRUCTURED_SNIPPET":
                                case 8:
                                    m.placeholder_type = 8;
                                    break;
                                case "MESSAGE":
                                case 9:
                                    m.placeholder_type = 9;
                                    break;
                                case "PRICE":
                                case 10:
                                    m.placeholder_type = 10;
                                    break;
                                case "PROMOTION":
                                case 11:
                                    m.placeholder_type = 11;
                                    break;
                                case "AD_CUSTOMIZER":
                                case 12:
                                    m.placeholder_type = 12;
                                    break;
                                case "DYNAMIC_EDUCATION":
                                case 13:
                                    m.placeholder_type = 13;
                                    break;
                                case "DYNAMIC_FLIGHT":
                                case 14:
                                    m.placeholder_type = 14;
                                    break;
                                case "DYNAMIC_CUSTOM":
                                case 15:
                                    m.placeholder_type = 15;
                                    break;
                                case "DYNAMIC_HOTEL":
                                case 16:
                                    m.placeholder_type = 16;
                                    break;
                                case "DYNAMIC_REAL_ESTATE":
                                case 17:
                                    m.placeholder_type = 17;
                                    break;
                                case "DYNAMIC_TRAVEL":
                                case 18:
                                    m.placeholder_type = 18;
                                    break;
                                case "DYNAMIC_LOCAL":
                                case 19:
                                    m.placeholder_type = 19;
                                    break;
                                case "DYNAMIC_JOB":
                                case 20:
                                    m.placeholder_type = 20;
                                    break;
                                case "IMAGE":
                                case 21:
                                    m.placeholder_type = 21;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedPlaceholderView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.FeedPlaceholderView} m FeedPlaceholderView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedPlaceholderView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.placeholder_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.placeholder_type != null && m.hasOwnProperty("placeholder_type")) {
                                    d.placeholder_type = o.enums === String ? $root.google.ads.googleads.v11.enums.PlaceholderTypeEnum.PlaceholderType[m.placeholder_type] : m.placeholder_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedPlaceholderView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.FeedPlaceholderView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedPlaceholderView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedPlaceholderView;
                        })();
    
                        resources.GenderView = (function() {
    
                            /**
                             * Properties of a GenderView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IGenderView
                             * @property {string|null} [resource_name] GenderView resource_name
                             */
    
                            /**
                             * Constructs a new GenderView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a GenderView.
                             * @implements IGenderView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IGenderView=} [p] Properties to set
                             */
                            function GenderView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenderView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @instance
                             */
                            GenderView.prototype.resource_name = "";
    
                            /**
                             * Creates a new GenderView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGenderView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.GenderView} GenderView instance
                             */
                            GenderView.create = function create(properties) {
                                return new GenderView(properties);
                            };
    
                            /**
                             * Encodes the specified GenderView message. Does not implicitly {@link google.ads.googleads.v11.resources.GenderView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGenderView} m GenderView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenderView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.GenderView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGenderView} message GenderView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenderView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenderView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.GenderView} GenderView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.GenderView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenderView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.GenderView} GenderView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenderView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenderView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenderView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenderView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.GenderView} GenderView
                             */
                            GenderView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.GenderView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.GenderView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenderView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @static
                             * @param {google.ads.googleads.v11.resources.GenderView} m GenderView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenderView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenderView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.GenderView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenderView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenderView;
                        })();
    
                        resources.GeoTargetConstant = (function() {
    
                            /**
                             * Properties of a GeoTargetConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IGeoTargetConstant
                             * @property {string|null} [resource_name] GeoTargetConstant resource_name
                             * @property {number|null} [id] GeoTargetConstant id
                             * @property {string|null} [name] GeoTargetConstant name
                             * @property {string|null} [country_code] GeoTargetConstant country_code
                             * @property {string|null} [target_type] GeoTargetConstant target_type
                             * @property {google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum.GeoTargetConstantStatus|null} [status] GeoTargetConstant status
                             * @property {string|null} [canonical_name] GeoTargetConstant canonical_name
                             * @property {string|null} [parent_geo_target] GeoTargetConstant parent_geo_target
                             */
    
                            /**
                             * Constructs a new GeoTargetConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a GeoTargetConstant.
                             * @implements IGeoTargetConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IGeoTargetConstant=} [p] Properties to set
                             */
                            function GeoTargetConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GeoTargetConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.resource_name = "";
    
                            /**
                             * GeoTargetConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.id = null;
    
                            /**
                             * GeoTargetConstant name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.name = null;
    
                            /**
                             * GeoTargetConstant country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.country_code = null;
    
                            /**
                             * GeoTargetConstant target_type.
                             * @member {string|null|undefined} target_type
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.target_type = null;
    
                            /**
                             * GeoTargetConstant status.
                             * @member {google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum.GeoTargetConstantStatus} status
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.status = 0;
    
                            /**
                             * GeoTargetConstant canonical_name.
                             * @member {string|null|undefined} canonical_name
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.canonical_name = null;
    
                            /**
                             * GeoTargetConstant parent_geo_target.
                             * @member {string|null|undefined} parent_geo_target
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            GeoTargetConstant.prototype.parent_geo_target = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GeoTargetConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoTargetConstant _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstant.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoTargetConstant _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstant.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoTargetConstant _target_type.
                             * @member {"target_type"|undefined} _target_type
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstant.prototype, "_target_type", {
                                get: $util.oneOfGetter($oneOfFields = ["target_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoTargetConstant _canonical_name.
                             * @member {"canonical_name"|undefined} _canonical_name
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstant.prototype, "_canonical_name", {
                                get: $util.oneOfGetter($oneOfFields = ["canonical_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoTargetConstant _parent_geo_target.
                             * @member {"parent_geo_target"|undefined} _parent_geo_target
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstant.prototype, "_parent_geo_target", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_geo_target"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GeoTargetConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGeoTargetConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.GeoTargetConstant} GeoTargetConstant instance
                             */
                            GeoTargetConstant.create = function create(properties) {
                                return new GeoTargetConstant(properties);
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.GeoTargetConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGeoTargetConstant} m GeoTargetConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(56).int32(m.status);
                                if (m.parent_geo_target != null && Object.hasOwnProperty.call(m, "parent_geo_target"))
                                    w.uint32(74).string(m.parent_geo_target);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(80).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(90).string(m.name);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(98).string(m.country_code);
                                if (m.target_type != null && Object.hasOwnProperty.call(m, "target_type"))
                                    w.uint32(106).string(m.target_type);
                                if (m.canonical_name != null && Object.hasOwnProperty.call(m, "canonical_name"))
                                    w.uint32(114).string(m.canonical_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.GeoTargetConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGeoTargetConstant} message GeoTargetConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeoTargetConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.GeoTargetConstant} GeoTargetConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.GeoTargetConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 10:
                                        m.id = r.int64();
                                        break;
                                    case 11:
                                        m.name = r.string();
                                        break;
                                    case 12:
                                        m.country_code = r.string();
                                        break;
                                    case 13:
                                        m.target_type = r.string();
                                        break;
                                    case 7:
                                        m.status = r.int32();
                                        break;
                                    case 14:
                                        m.canonical_name = r.string();
                                        break;
                                    case 9:
                                        m.parent_geo_target = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeoTargetConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.GeoTargetConstant} GeoTargetConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeoTargetConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeoTargetConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.target_type != null && m.hasOwnProperty("target_type")) {
                                    p._target_type = 1;
                                    if (!$util.isString(m.target_type))
                                        return "target_type: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.canonical_name != null && m.hasOwnProperty("canonical_name")) {
                                    p._canonical_name = 1;
                                    if (!$util.isString(m.canonical_name))
                                        return "canonical_name: string expected";
                                }
                                if (m.parent_geo_target != null && m.hasOwnProperty("parent_geo_target")) {
                                    p._parent_geo_target = 1;
                                    if (!$util.isString(m.parent_geo_target))
                                        return "parent_geo_target: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GeoTargetConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.GeoTargetConstant} GeoTargetConstant
                             */
                            GeoTargetConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.GeoTargetConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.GeoTargetConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.target_type != null) {
                                    m.target_type = String(d.target_type);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVAL_PLANNED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.canonical_name != null) {
                                    m.canonical_name = String(d.canonical_name);
                                }
                                if (d.parent_geo_target != null) {
                                    m.parent_geo_target = String(d.parent_geo_target);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GeoTargetConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.GeoTargetConstant} m GeoTargetConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeoTargetConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.GeoTargetConstantStatusEnum.GeoTargetConstantStatus[m.status] : m.status;
                                }
                                if (m.parent_geo_target != null && m.hasOwnProperty("parent_geo_target")) {
                                    d.parent_geo_target = m.parent_geo_target;
                                    if (o.oneofs)
                                        d._parent_geo_target = "parent_geo_target";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.target_type != null && m.hasOwnProperty("target_type")) {
                                    d.target_type = m.target_type;
                                    if (o.oneofs)
                                        d._target_type = "target_type";
                                }
                                if (m.canonical_name != null && m.hasOwnProperty("canonical_name")) {
                                    d.canonical_name = m.canonical_name;
                                    if (o.oneofs)
                                        d._canonical_name = "canonical_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GeoTargetConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.GeoTargetConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeoTargetConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GeoTargetConstant;
                        })();
    
                        resources.GeographicView = (function() {
    
                            /**
                             * Properties of a GeographicView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IGeographicView
                             * @property {string|null} [resource_name] GeographicView resource_name
                             * @property {google.ads.googleads.v11.enums.GeoTargetingTypeEnum.GeoTargetingType|null} [location_type] GeographicView location_type
                             * @property {number|null} [country_criterion_id] GeographicView country_criterion_id
                             */
    
                            /**
                             * Constructs a new GeographicView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a GeographicView.
                             * @implements IGeographicView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IGeographicView=} [p] Properties to set
                             */
                            function GeographicView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GeographicView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @instance
                             */
                            GeographicView.prototype.resource_name = "";
    
                            /**
                             * GeographicView location_type.
                             * @member {google.ads.googleads.v11.enums.GeoTargetingTypeEnum.GeoTargetingType} location_type
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @instance
                             */
                            GeographicView.prototype.location_type = 0;
    
                            /**
                             * GeographicView country_criterion_id.
                             * @member {number|null|undefined} country_criterion_id
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @instance
                             */
                            GeographicView.prototype.country_criterion_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GeographicView _country_criterion_id.
                             * @member {"country_criterion_id"|undefined} _country_criterion_id
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @instance
                             */
                            Object.defineProperty(GeographicView.prototype, "_country_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["country_criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GeographicView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGeographicView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.GeographicView} GeographicView instance
                             */
                            GeographicView.create = function create(properties) {
                                return new GeographicView(properties);
                            };
    
                            /**
                             * Encodes the specified GeographicView message. Does not implicitly {@link google.ads.googleads.v11.resources.GeographicView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGeographicView} m GeographicView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeographicView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.location_type != null && Object.hasOwnProperty.call(m, "location_type"))
                                    w.uint32(24).int32(m.location_type);
                                if (m.country_criterion_id != null && Object.hasOwnProperty.call(m, "country_criterion_id"))
                                    w.uint32(40).int64(m.country_criterion_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeographicView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.GeographicView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGeographicView} message GeographicView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeographicView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeographicView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.GeographicView} GeographicView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeographicView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.GeographicView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.location_type = r.int32();
                                        break;
                                    case 5:
                                        m.country_criterion_id = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeographicView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.GeographicView} GeographicView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeographicView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeographicView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeographicView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.location_type != null && m.hasOwnProperty("location_type")) {
                                    switch (m.location_type) {
                                    default:
                                        return "location_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.country_criterion_id != null && m.hasOwnProperty("country_criterion_id")) {
                                    p._country_criterion_id = 1;
                                    if (!$util.isInteger(m.country_criterion_id) && !(m.country_criterion_id && $util.isInteger(m.country_criterion_id.low) && $util.isInteger(m.country_criterion_id.high)))
                                        return "country_criterion_id: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GeographicView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.GeographicView} GeographicView
                             */
                            GeographicView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.GeographicView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.GeographicView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.location_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.location_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.location_type = 1;
                                    break;
                                case "AREA_OF_INTEREST":
                                case 2:
                                    m.location_type = 2;
                                    break;
                                case "LOCATION_OF_PRESENCE":
                                case 3:
                                    m.location_type = 3;
                                    break;
                                }
                                if (d.country_criterion_id != null) {
                                    if ($util.Long)
                                        (m.country_criterion_id = $util.Long.fromValue(d.country_criterion_id)).unsigned = false;
                                    else if (typeof d.country_criterion_id === "string")
                                        m.country_criterion_id = parseInt(d.country_criterion_id, 10);
                                    else if (typeof d.country_criterion_id === "number")
                                        m.country_criterion_id = d.country_criterion_id;
                                    else if (typeof d.country_criterion_id === "object")
                                        m.country_criterion_id = new $util.LongBits(d.country_criterion_id.low >>> 0, d.country_criterion_id.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GeographicView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.GeographicView} m GeographicView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeographicView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.location_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.location_type != null && m.hasOwnProperty("location_type")) {
                                    d.location_type = o.enums === String ? $root.google.ads.googleads.v11.enums.GeoTargetingTypeEnum.GeoTargetingType[m.location_type] : m.location_type;
                                }
                                if (m.country_criterion_id != null && m.hasOwnProperty("country_criterion_id")) {
                                    if (typeof m.country_criterion_id === "number")
                                        d.country_criterion_id = o.longs === String ? String(m.country_criterion_id) : m.country_criterion_id;
                                    else
                                        d.country_criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.country_criterion_id) : o.longs === Number ? new $util.LongBits(m.country_criterion_id.low >>> 0, m.country_criterion_id.high >>> 0).toNumber() : m.country_criterion_id;
                                    if (o.oneofs)
                                        d._country_criterion_id = "country_criterion_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GeographicView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.GeographicView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeographicView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GeographicView;
                        })();
    
                        resources.GoogleAdsField = (function() {
    
                            /**
                             * Properties of a GoogleAdsField.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IGoogleAdsField
                             * @property {string|null} [resource_name] GoogleAdsField resource_name
                             * @property {string|null} [name] GoogleAdsField name
                             * @property {google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory|null} [category] GoogleAdsField category
                             * @property {boolean|null} [selectable] GoogleAdsField selectable
                             * @property {boolean|null} [filterable] GoogleAdsField filterable
                             * @property {boolean|null} [sortable] GoogleAdsField sortable
                             * @property {Array.<string>|null} [selectable_with] GoogleAdsField selectable_with
                             * @property {Array.<string>|null} [attribute_resources] GoogleAdsField attribute_resources
                             * @property {Array.<string>|null} [metrics] GoogleAdsField metrics
                             * @property {Array.<string>|null} [segments] GoogleAdsField segments
                             * @property {Array.<string>|null} [enum_values] GoogleAdsField enum_values
                             * @property {google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType|null} [data_type] GoogleAdsField data_type
                             * @property {string|null} [type_url] GoogleAdsField type_url
                             * @property {boolean|null} [is_repeated] GoogleAdsField is_repeated
                             */
    
                            /**
                             * Constructs a new GoogleAdsField.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a GoogleAdsField.
                             * @implements IGoogleAdsField
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsField=} [p] Properties to set
                             */
                            function GoogleAdsField(p) {
                                this.selectable_with = [];
                                this.attribute_resources = [];
                                this.metrics = [];
                                this.segments = [];
                                this.enum_values = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GoogleAdsField resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.resource_name = "";
    
                            /**
                             * GoogleAdsField name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.name = null;
    
                            /**
                             * GoogleAdsField category.
                             * @member {google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory} category
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.category = 0;
    
                            /**
                             * GoogleAdsField selectable.
                             * @member {boolean|null|undefined} selectable
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.selectable = null;
    
                            /**
                             * GoogleAdsField filterable.
                             * @member {boolean|null|undefined} filterable
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.filterable = null;
    
                            /**
                             * GoogleAdsField sortable.
                             * @member {boolean|null|undefined} sortable
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.sortable = null;
    
                            /**
                             * GoogleAdsField selectable_with.
                             * @member {Array.<string>} selectable_with
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.selectable_with = $util.emptyArray;
    
                            /**
                             * GoogleAdsField attribute_resources.
                             * @member {Array.<string>} attribute_resources
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.attribute_resources = $util.emptyArray;
    
                            /**
                             * GoogleAdsField metrics.
                             * @member {Array.<string>} metrics
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.metrics = $util.emptyArray;
    
                            /**
                             * GoogleAdsField segments.
                             * @member {Array.<string>} segments
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.segments = $util.emptyArray;
    
                            /**
                             * GoogleAdsField enum_values.
                             * @member {Array.<string>} enum_values
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.enum_values = $util.emptyArray;
    
                            /**
                             * GoogleAdsField data_type.
                             * @member {google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType} data_type
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.data_type = 0;
    
                            /**
                             * GoogleAdsField type_url.
                             * @member {string|null|undefined} type_url
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.type_url = null;
    
                            /**
                             * GoogleAdsField is_repeated.
                             * @member {boolean|null|undefined} is_repeated
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            GoogleAdsField.prototype.is_repeated = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GoogleAdsField _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            Object.defineProperty(GoogleAdsField.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GoogleAdsField _selectable.
                             * @member {"selectable"|undefined} _selectable
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            Object.defineProperty(GoogleAdsField.prototype, "_selectable", {
                                get: $util.oneOfGetter($oneOfFields = ["selectable"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GoogleAdsField _filterable.
                             * @member {"filterable"|undefined} _filterable
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            Object.defineProperty(GoogleAdsField.prototype, "_filterable", {
                                get: $util.oneOfGetter($oneOfFields = ["filterable"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GoogleAdsField _sortable.
                             * @member {"sortable"|undefined} _sortable
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            Object.defineProperty(GoogleAdsField.prototype, "_sortable", {
                                get: $util.oneOfGetter($oneOfFields = ["sortable"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GoogleAdsField _type_url.
                             * @member {"type_url"|undefined} _type_url
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            Object.defineProperty(GoogleAdsField.prototype, "_type_url", {
                                get: $util.oneOfGetter($oneOfFields = ["type_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GoogleAdsField _is_repeated.
                             * @member {"is_repeated"|undefined} _is_repeated
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             */
                            Object.defineProperty(GoogleAdsField.prototype, "_is_repeated", {
                                get: $util.oneOfGetter($oneOfFields = ["is_repeated"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GoogleAdsField instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsField=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsField} GoogleAdsField instance
                             */
                            GoogleAdsField.create = function create(properties) {
                                return new GoogleAdsField(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleAdsField message. Does not implicitly {@link google.ads.googleads.v11.resources.GoogleAdsField.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsField} m GoogleAdsField message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsField.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    w.uint32(24).int32(m.category);
                                if (m.data_type != null && Object.hasOwnProperty.call(m, "data_type"))
                                    w.uint32(96).int32(m.data_type);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(170).string(m.name);
                                if (m.selectable != null && Object.hasOwnProperty.call(m, "selectable"))
                                    w.uint32(176).bool(m.selectable);
                                if (m.filterable != null && Object.hasOwnProperty.call(m, "filterable"))
                                    w.uint32(184).bool(m.filterable);
                                if (m.sortable != null && Object.hasOwnProperty.call(m, "sortable"))
                                    w.uint32(192).bool(m.sortable);
                                if (m.selectable_with != null && m.selectable_with.length) {
                                    for (var i = 0; i < m.selectable_with.length; ++i)
                                        w.uint32(202).string(m.selectable_with[i]);
                                }
                                if (m.attribute_resources != null && m.attribute_resources.length) {
                                    for (var i = 0; i < m.attribute_resources.length; ++i)
                                        w.uint32(210).string(m.attribute_resources[i]);
                                }
                                if (m.metrics != null && m.metrics.length) {
                                    for (var i = 0; i < m.metrics.length; ++i)
                                        w.uint32(218).string(m.metrics[i]);
                                }
                                if (m.segments != null && m.segments.length) {
                                    for (var i = 0; i < m.segments.length; ++i)
                                        w.uint32(226).string(m.segments[i]);
                                }
                                if (m.enum_values != null && m.enum_values.length) {
                                    for (var i = 0; i < m.enum_values.length; ++i)
                                        w.uint32(234).string(m.enum_values[i]);
                                }
                                if (m.type_url != null && Object.hasOwnProperty.call(m, "type_url"))
                                    w.uint32(242).string(m.type_url);
                                if (m.is_repeated != null && Object.hasOwnProperty.call(m, "is_repeated"))
                                    w.uint32(248).bool(m.is_repeated);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleAdsField message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.GoogleAdsField.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGoogleAdsField} message GoogleAdsField message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsField.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleAdsField message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsField} GoogleAdsField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsField.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.GoogleAdsField();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 21:
                                        m.name = r.string();
                                        break;
                                    case 3:
                                        m.category = r.int32();
                                        break;
                                    case 22:
                                        m.selectable = r.bool();
                                        break;
                                    case 23:
                                        m.filterable = r.bool();
                                        break;
                                    case 24:
                                        m.sortable = r.bool();
                                        break;
                                    case 25:
                                        if (!(m.selectable_with && m.selectable_with.length))
                                            m.selectable_with = [];
                                        m.selectable_with.push(r.string());
                                        break;
                                    case 26:
                                        if (!(m.attribute_resources && m.attribute_resources.length))
                                            m.attribute_resources = [];
                                        m.attribute_resources.push(r.string());
                                        break;
                                    case 27:
                                        if (!(m.metrics && m.metrics.length))
                                            m.metrics = [];
                                        m.metrics.push(r.string());
                                        break;
                                    case 28:
                                        if (!(m.segments && m.segments.length))
                                            m.segments = [];
                                        m.segments.push(r.string());
                                        break;
                                    case 29:
                                        if (!(m.enum_values && m.enum_values.length))
                                            m.enum_values = [];
                                        m.enum_values.push(r.string());
                                        break;
                                    case 12:
                                        m.data_type = r.int32();
                                        break;
                                    case 30:
                                        m.type_url = r.string();
                                        break;
                                    case 31:
                                        m.is_repeated = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleAdsField message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsField} GoogleAdsField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsField.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleAdsField message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleAdsField.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    switch (m.category) {
                                    default:
                                        return "category: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.selectable != null && m.hasOwnProperty("selectable")) {
                                    p._selectable = 1;
                                    if (typeof m.selectable !== "boolean")
                                        return "selectable: boolean expected";
                                }
                                if (m.filterable != null && m.hasOwnProperty("filterable")) {
                                    p._filterable = 1;
                                    if (typeof m.filterable !== "boolean")
                                        return "filterable: boolean expected";
                                }
                                if (m.sortable != null && m.hasOwnProperty("sortable")) {
                                    p._sortable = 1;
                                    if (typeof m.sortable !== "boolean")
                                        return "sortable: boolean expected";
                                }
                                if (m.selectable_with != null && m.hasOwnProperty("selectable_with")) {
                                    if (!Array.isArray(m.selectable_with))
                                        return "selectable_with: array expected";
                                    for (var i = 0; i < m.selectable_with.length; ++i) {
                                        if (!$util.isString(m.selectable_with[i]))
                                            return "selectable_with: string[] expected";
                                    }
                                }
                                if (m.attribute_resources != null && m.hasOwnProperty("attribute_resources")) {
                                    if (!Array.isArray(m.attribute_resources))
                                        return "attribute_resources: array expected";
                                    for (var i = 0; i < m.attribute_resources.length; ++i) {
                                        if (!$util.isString(m.attribute_resources[i]))
                                            return "attribute_resources: string[] expected";
                                    }
                                }
                                if (m.metrics != null && m.hasOwnProperty("metrics")) {
                                    if (!Array.isArray(m.metrics))
                                        return "metrics: array expected";
                                    for (var i = 0; i < m.metrics.length; ++i) {
                                        if (!$util.isString(m.metrics[i]))
                                            return "metrics: string[] expected";
                                    }
                                }
                                if (m.segments != null && m.hasOwnProperty("segments")) {
                                    if (!Array.isArray(m.segments))
                                        return "segments: array expected";
                                    for (var i = 0; i < m.segments.length; ++i) {
                                        if (!$util.isString(m.segments[i]))
                                            return "segments: string[] expected";
                                    }
                                }
                                if (m.enum_values != null && m.hasOwnProperty("enum_values")) {
                                    if (!Array.isArray(m.enum_values))
                                        return "enum_values: array expected";
                                    for (var i = 0; i < m.enum_values.length; ++i) {
                                        if (!$util.isString(m.enum_values[i]))
                                            return "enum_values: string[] expected";
                                    }
                                }
                                if (m.data_type != null && m.hasOwnProperty("data_type")) {
                                    switch (m.data_type) {
                                    default:
                                        return "data_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                        break;
                                    }
                                }
                                if (m.type_url != null && m.hasOwnProperty("type_url")) {
                                    p._type_url = 1;
                                    if (!$util.isString(m.type_url))
                                        return "type_url: string expected";
                                }
                                if (m.is_repeated != null && m.hasOwnProperty("is_repeated")) {
                                    p._is_repeated = 1;
                                    if (typeof m.is_repeated !== "boolean")
                                        return "is_repeated: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GoogleAdsField message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.GoogleAdsField} GoogleAdsField
                             */
                            GoogleAdsField.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.GoogleAdsField)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.GoogleAdsField();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.category) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.category = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.category = 1;
                                    break;
                                case "RESOURCE":
                                case 2:
                                    m.category = 2;
                                    break;
                                case "ATTRIBUTE":
                                case 3:
                                    m.category = 3;
                                    break;
                                case "SEGMENT":
                                case 5:
                                    m.category = 5;
                                    break;
                                case "METRIC":
                                case 6:
                                    m.category = 6;
                                    break;
                                }
                                if (d.selectable != null) {
                                    m.selectable = Boolean(d.selectable);
                                }
                                if (d.filterable != null) {
                                    m.filterable = Boolean(d.filterable);
                                }
                                if (d.sortable != null) {
                                    m.sortable = Boolean(d.sortable);
                                }
                                if (d.selectable_with) {
                                    if (!Array.isArray(d.selectable_with))
                                        throw TypeError(".google.ads.googleads.v11.resources.GoogleAdsField.selectable_with: array expected");
                                    m.selectable_with = [];
                                    for (var i = 0; i < d.selectable_with.length; ++i) {
                                        m.selectable_with[i] = String(d.selectable_with[i]);
                                    }
                                }
                                if (d.attribute_resources) {
                                    if (!Array.isArray(d.attribute_resources))
                                        throw TypeError(".google.ads.googleads.v11.resources.GoogleAdsField.attribute_resources: array expected");
                                    m.attribute_resources = [];
                                    for (var i = 0; i < d.attribute_resources.length; ++i) {
                                        m.attribute_resources[i] = String(d.attribute_resources[i]);
                                    }
                                }
                                if (d.metrics) {
                                    if (!Array.isArray(d.metrics))
                                        throw TypeError(".google.ads.googleads.v11.resources.GoogleAdsField.metrics: array expected");
                                    m.metrics = [];
                                    for (var i = 0; i < d.metrics.length; ++i) {
                                        m.metrics[i] = String(d.metrics[i]);
                                    }
                                }
                                if (d.segments) {
                                    if (!Array.isArray(d.segments))
                                        throw TypeError(".google.ads.googleads.v11.resources.GoogleAdsField.segments: array expected");
                                    m.segments = [];
                                    for (var i = 0; i < d.segments.length; ++i) {
                                        m.segments[i] = String(d.segments[i]);
                                    }
                                }
                                if (d.enum_values) {
                                    if (!Array.isArray(d.enum_values))
                                        throw TypeError(".google.ads.googleads.v11.resources.GoogleAdsField.enum_values: array expected");
                                    m.enum_values = [];
                                    for (var i = 0; i < d.enum_values.length; ++i) {
                                        m.enum_values[i] = String(d.enum_values[i]);
                                    }
                                }
                                switch (d.data_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.data_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.data_type = 1;
                                    break;
                                case "BOOLEAN":
                                case 2:
                                    m.data_type = 2;
                                    break;
                                case "DATE":
                                case 3:
                                    m.data_type = 3;
                                    break;
                                case "DOUBLE":
                                case 4:
                                    m.data_type = 4;
                                    break;
                                case "ENUM":
                                case 5:
                                    m.data_type = 5;
                                    break;
                                case "FLOAT":
                                case 6:
                                    m.data_type = 6;
                                    break;
                                case "INT32":
                                case 7:
                                    m.data_type = 7;
                                    break;
                                case "INT64":
                                case 8:
                                    m.data_type = 8;
                                    break;
                                case "MESSAGE":
                                case 9:
                                    m.data_type = 9;
                                    break;
                                case "RESOURCE_NAME":
                                case 10:
                                    m.data_type = 10;
                                    break;
                                case "STRING":
                                case 11:
                                    m.data_type = 11;
                                    break;
                                case "UINT64":
                                case 12:
                                    m.data_type = 12;
                                    break;
                                }
                                if (d.type_url != null) {
                                    m.type_url = String(d.type_url);
                                }
                                if (d.is_repeated != null) {
                                    m.is_repeated = Boolean(d.is_repeated);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GoogleAdsField message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @static
                             * @param {google.ads.googleads.v11.resources.GoogleAdsField} m GoogleAdsField
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleAdsField.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.selectable_with = [];
                                    d.attribute_resources = [];
                                    d.metrics = [];
                                    d.segments = [];
                                    d.enum_values = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.category = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.data_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = o.enums === String ? $root.google.ads.googleads.v11.enums.GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory[m.category] : m.category;
                                }
                                if (m.data_type != null && m.hasOwnProperty("data_type")) {
                                    d.data_type = o.enums === String ? $root.google.ads.googleads.v11.enums.GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType[m.data_type] : m.data_type;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.selectable != null && m.hasOwnProperty("selectable")) {
                                    d.selectable = m.selectable;
                                    if (o.oneofs)
                                        d._selectable = "selectable";
                                }
                                if (m.filterable != null && m.hasOwnProperty("filterable")) {
                                    d.filterable = m.filterable;
                                    if (o.oneofs)
                                        d._filterable = "filterable";
                                }
                                if (m.sortable != null && m.hasOwnProperty("sortable")) {
                                    d.sortable = m.sortable;
                                    if (o.oneofs)
                                        d._sortable = "sortable";
                                }
                                if (m.selectable_with && m.selectable_with.length) {
                                    d.selectable_with = [];
                                    for (var j = 0; j < m.selectable_with.length; ++j) {
                                        d.selectable_with[j] = m.selectable_with[j];
                                    }
                                }
                                if (m.attribute_resources && m.attribute_resources.length) {
                                    d.attribute_resources = [];
                                    for (var j = 0; j < m.attribute_resources.length; ++j) {
                                        d.attribute_resources[j] = m.attribute_resources[j];
                                    }
                                }
                                if (m.metrics && m.metrics.length) {
                                    d.metrics = [];
                                    for (var j = 0; j < m.metrics.length; ++j) {
                                        d.metrics[j] = m.metrics[j];
                                    }
                                }
                                if (m.segments && m.segments.length) {
                                    d.segments = [];
                                    for (var j = 0; j < m.segments.length; ++j) {
                                        d.segments[j] = m.segments[j];
                                    }
                                }
                                if (m.enum_values && m.enum_values.length) {
                                    d.enum_values = [];
                                    for (var j = 0; j < m.enum_values.length; ++j) {
                                        d.enum_values[j] = m.enum_values[j];
                                    }
                                }
                                if (m.type_url != null && m.hasOwnProperty("type_url")) {
                                    d.type_url = m.type_url;
                                    if (o.oneofs)
                                        d._type_url = "type_url";
                                }
                                if (m.is_repeated != null && m.hasOwnProperty("is_repeated")) {
                                    d.is_repeated = m.is_repeated;
                                    if (o.oneofs)
                                        d._is_repeated = "is_repeated";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GoogleAdsField to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.GoogleAdsField
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleAdsField.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GoogleAdsField;
                        })();
    
                        resources.GroupPlacementView = (function() {
    
                            /**
                             * Properties of a GroupPlacementView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IGroupPlacementView
                             * @property {string|null} [resource_name] GroupPlacementView resource_name
                             * @property {string|null} [placement] GroupPlacementView placement
                             * @property {string|null} [display_name] GroupPlacementView display_name
                             * @property {string|null} [target_url] GroupPlacementView target_url
                             * @property {google.ads.googleads.v11.enums.PlacementTypeEnum.PlacementType|null} [placement_type] GroupPlacementView placement_type
                             */
    
                            /**
                             * Constructs a new GroupPlacementView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a GroupPlacementView.
                             * @implements IGroupPlacementView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IGroupPlacementView=} [p] Properties to set
                             */
                            function GroupPlacementView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GroupPlacementView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            GroupPlacementView.prototype.resource_name = "";
    
                            /**
                             * GroupPlacementView placement.
                             * @member {string|null|undefined} placement
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            GroupPlacementView.prototype.placement = null;
    
                            /**
                             * GroupPlacementView display_name.
                             * @member {string|null|undefined} display_name
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            GroupPlacementView.prototype.display_name = null;
    
                            /**
                             * GroupPlacementView target_url.
                             * @member {string|null|undefined} target_url
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            GroupPlacementView.prototype.target_url = null;
    
                            /**
                             * GroupPlacementView placement_type.
                             * @member {google.ads.googleads.v11.enums.PlacementTypeEnum.PlacementType} placement_type
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            GroupPlacementView.prototype.placement_type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GroupPlacementView _placement.
                             * @member {"placement"|undefined} _placement
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            Object.defineProperty(GroupPlacementView.prototype, "_placement", {
                                get: $util.oneOfGetter($oneOfFields = ["placement"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GroupPlacementView _display_name.
                             * @member {"display_name"|undefined} _display_name
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            Object.defineProperty(GroupPlacementView.prototype, "_display_name", {
                                get: $util.oneOfGetter($oneOfFields = ["display_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GroupPlacementView _target_url.
                             * @member {"target_url"|undefined} _target_url
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             */
                            Object.defineProperty(GroupPlacementView.prototype, "_target_url", {
                                get: $util.oneOfGetter($oneOfFields = ["target_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GroupPlacementView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGroupPlacementView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.GroupPlacementView} GroupPlacementView instance
                             */
                            GroupPlacementView.create = function create(properties) {
                                return new GroupPlacementView(properties);
                            };
    
                            /**
                             * Encodes the specified GroupPlacementView message. Does not implicitly {@link google.ads.googleads.v11.resources.GroupPlacementView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGroupPlacementView} m GroupPlacementView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GroupPlacementView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.placement_type != null && Object.hasOwnProperty.call(m, "placement_type"))
                                    w.uint32(40).int32(m.placement_type);
                                if (m.placement != null && Object.hasOwnProperty.call(m, "placement"))
                                    w.uint32(50).string(m.placement);
                                if (m.display_name != null && Object.hasOwnProperty.call(m, "display_name"))
                                    w.uint32(58).string(m.display_name);
                                if (m.target_url != null && Object.hasOwnProperty.call(m, "target_url"))
                                    w.uint32(66).string(m.target_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GroupPlacementView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.GroupPlacementView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IGroupPlacementView} message GroupPlacementView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GroupPlacementView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GroupPlacementView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.GroupPlacementView} GroupPlacementView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GroupPlacementView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.GroupPlacementView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.placement = r.string();
                                        break;
                                    case 7:
                                        m.display_name = r.string();
                                        break;
                                    case 8:
                                        m.target_url = r.string();
                                        break;
                                    case 5:
                                        m.placement_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GroupPlacementView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.GroupPlacementView} GroupPlacementView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GroupPlacementView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GroupPlacementView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GroupPlacementView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    p._placement = 1;
                                    if (!$util.isString(m.placement))
                                        return "placement: string expected";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    p._display_name = 1;
                                    if (!$util.isString(m.display_name))
                                        return "display_name: string expected";
                                }
                                if (m.target_url != null && m.hasOwnProperty("target_url")) {
                                    p._target_url = 1;
                                    if (!$util.isString(m.target_url))
                                        return "target_url: string expected";
                                }
                                if (m.placement_type != null && m.hasOwnProperty("placement_type")) {
                                    switch (m.placement_type) {
                                    default:
                                        return "placement_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GroupPlacementView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.GroupPlacementView} GroupPlacementView
                             */
                            GroupPlacementView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.GroupPlacementView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.GroupPlacementView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.placement != null) {
                                    m.placement = String(d.placement);
                                }
                                if (d.display_name != null) {
                                    m.display_name = String(d.display_name);
                                }
                                if (d.target_url != null) {
                                    m.target_url = String(d.target_url);
                                }
                                switch (d.placement_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.placement_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.placement_type = 1;
                                    break;
                                case "WEBSITE":
                                case 2:
                                    m.placement_type = 2;
                                    break;
                                case "MOBILE_APP_CATEGORY":
                                case 3:
                                    m.placement_type = 3;
                                    break;
                                case "MOBILE_APPLICATION":
                                case 4:
                                    m.placement_type = 4;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 5:
                                    m.placement_type = 5;
                                    break;
                                case "YOUTUBE_CHANNEL":
                                case 6:
                                    m.placement_type = 6;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GroupPlacementView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.GroupPlacementView} m GroupPlacementView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GroupPlacementView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.placement_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.placement_type != null && m.hasOwnProperty("placement_type")) {
                                    d.placement_type = o.enums === String ? $root.google.ads.googleads.v11.enums.PlacementTypeEnum.PlacementType[m.placement_type] : m.placement_type;
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    d.placement = m.placement;
                                    if (o.oneofs)
                                        d._placement = "placement";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    d.display_name = m.display_name;
                                    if (o.oneofs)
                                        d._display_name = "display_name";
                                }
                                if (m.target_url != null && m.hasOwnProperty("target_url")) {
                                    d.target_url = m.target_url;
                                    if (o.oneofs)
                                        d._target_url = "target_url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GroupPlacementView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.GroupPlacementView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GroupPlacementView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GroupPlacementView;
                        })();
    
                        resources.HotelGroupView = (function() {
    
                            /**
                             * Properties of a HotelGroupView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IHotelGroupView
                             * @property {string|null} [resource_name] HotelGroupView resource_name
                             */
    
                            /**
                             * Constructs a new HotelGroupView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a HotelGroupView.
                             * @implements IHotelGroupView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IHotelGroupView=} [p] Properties to set
                             */
                            function HotelGroupView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelGroupView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @instance
                             */
                            HotelGroupView.prototype.resource_name = "";
    
                            /**
                             * Creates a new HotelGroupView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelGroupView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.HotelGroupView} HotelGroupView instance
                             */
                            HotelGroupView.create = function create(properties) {
                                return new HotelGroupView(properties);
                            };
    
                            /**
                             * Encodes the specified HotelGroupView message. Does not implicitly {@link google.ads.googleads.v11.resources.HotelGroupView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelGroupView} m HotelGroupView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelGroupView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelGroupView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.HotelGroupView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelGroupView} message HotelGroupView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelGroupView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelGroupView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.HotelGroupView} HotelGroupView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelGroupView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.HotelGroupView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelGroupView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.HotelGroupView} HotelGroupView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelGroupView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelGroupView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelGroupView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelGroupView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.HotelGroupView} HotelGroupView
                             */
                            HotelGroupView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.HotelGroupView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.HotelGroupView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelGroupView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.HotelGroupView} m HotelGroupView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelGroupView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelGroupView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.HotelGroupView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelGroupView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelGroupView;
                        })();
    
                        resources.HotelPerformanceView = (function() {
    
                            /**
                             * Properties of a HotelPerformanceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IHotelPerformanceView
                             * @property {string|null} [resource_name] HotelPerformanceView resource_name
                             */
    
                            /**
                             * Constructs a new HotelPerformanceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a HotelPerformanceView.
                             * @implements IHotelPerformanceView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IHotelPerformanceView=} [p] Properties to set
                             */
                            function HotelPerformanceView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelPerformanceView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @instance
                             */
                            HotelPerformanceView.prototype.resource_name = "";
    
                            /**
                             * Creates a new HotelPerformanceView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelPerformanceView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.HotelPerformanceView} HotelPerformanceView instance
                             */
                            HotelPerformanceView.create = function create(properties) {
                                return new HotelPerformanceView(properties);
                            };
    
                            /**
                             * Encodes the specified HotelPerformanceView message. Does not implicitly {@link google.ads.googleads.v11.resources.HotelPerformanceView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelPerformanceView} m HotelPerformanceView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelPerformanceView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelPerformanceView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.HotelPerformanceView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelPerformanceView} message HotelPerformanceView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelPerformanceView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelPerformanceView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.HotelPerformanceView} HotelPerformanceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelPerformanceView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.HotelPerformanceView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelPerformanceView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.HotelPerformanceView} HotelPerformanceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelPerformanceView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelPerformanceView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelPerformanceView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelPerformanceView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.HotelPerformanceView} HotelPerformanceView
                             */
                            HotelPerformanceView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.HotelPerformanceView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.HotelPerformanceView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelPerformanceView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.HotelPerformanceView} m HotelPerformanceView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelPerformanceView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelPerformanceView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.HotelPerformanceView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelPerformanceView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelPerformanceView;
                        })();
    
                        resources.HotelReconciliation = (function() {
    
                            /**
                             * Properties of a HotelReconciliation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IHotelReconciliation
                             * @property {string|null} [resource_name] HotelReconciliation resource_name
                             * @property {string|null} [commission_id] HotelReconciliation commission_id
                             * @property {string|null} [order_id] HotelReconciliation order_id
                             * @property {string|null} [campaign] HotelReconciliation campaign
                             * @property {number|null} [hotel_center_id] HotelReconciliation hotel_center_id
                             * @property {string|null} [hotel_id] HotelReconciliation hotel_id
                             * @property {string|null} [check_in_date] HotelReconciliation check_in_date
                             * @property {string|null} [check_out_date] HotelReconciliation check_out_date
                             * @property {number|null} [reconciled_value_micros] HotelReconciliation reconciled_value_micros
                             * @property {boolean|null} [billed] HotelReconciliation billed
                             * @property {google.ads.googleads.v11.enums.HotelReconciliationStatusEnum.HotelReconciliationStatus|null} [status] HotelReconciliation status
                             */
    
                            /**
                             * Constructs a new HotelReconciliation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a HotelReconciliation.
                             * @implements IHotelReconciliation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IHotelReconciliation=} [p] Properties to set
                             */
                            function HotelReconciliation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * HotelReconciliation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.resource_name = "";
    
                            /**
                             * HotelReconciliation commission_id.
                             * @member {string} commission_id
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.commission_id = "";
    
                            /**
                             * HotelReconciliation order_id.
                             * @member {string} order_id
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.order_id = "";
    
                            /**
                             * HotelReconciliation campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.campaign = "";
    
                            /**
                             * HotelReconciliation hotel_center_id.
                             * @member {number} hotel_center_id
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.hotel_center_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * HotelReconciliation hotel_id.
                             * @member {string} hotel_id
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.hotel_id = "";
    
                            /**
                             * HotelReconciliation check_in_date.
                             * @member {string} check_in_date
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.check_in_date = "";
    
                            /**
                             * HotelReconciliation check_out_date.
                             * @member {string} check_out_date
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.check_out_date = "";
    
                            /**
                             * HotelReconciliation reconciled_value_micros.
                             * @member {number} reconciled_value_micros
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.reconciled_value_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * HotelReconciliation billed.
                             * @member {boolean} billed
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.billed = false;
    
                            /**
                             * HotelReconciliation status.
                             * @member {google.ads.googleads.v11.enums.HotelReconciliationStatusEnum.HotelReconciliationStatus} status
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             */
                            HotelReconciliation.prototype.status = 0;
    
                            /**
                             * Creates a new HotelReconciliation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelReconciliation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.HotelReconciliation} HotelReconciliation instance
                             */
                            HotelReconciliation.create = function create(properties) {
                                return new HotelReconciliation(properties);
                            };
    
                            /**
                             * Encodes the specified HotelReconciliation message. Does not implicitly {@link google.ads.googleads.v11.resources.HotelReconciliation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelReconciliation} m HotelReconciliation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelReconciliation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.commission_id != null && Object.hasOwnProperty.call(m, "commission_id"))
                                    w.uint32(18).string(m.commission_id);
                                if (m.order_id != null && Object.hasOwnProperty.call(m, "order_id"))
                                    w.uint32(26).string(m.order_id);
                                if (m.hotel_center_id != null && Object.hasOwnProperty.call(m, "hotel_center_id"))
                                    w.uint32(32).int64(m.hotel_center_id);
                                if (m.hotel_id != null && Object.hasOwnProperty.call(m, "hotel_id"))
                                    w.uint32(42).string(m.hotel_id);
                                if (m.check_in_date != null && Object.hasOwnProperty.call(m, "check_in_date"))
                                    w.uint32(50).string(m.check_in_date);
                                if (m.check_out_date != null && Object.hasOwnProperty.call(m, "check_out_date"))
                                    w.uint32(58).string(m.check_out_date);
                                if (m.reconciled_value_micros != null && Object.hasOwnProperty.call(m, "reconciled_value_micros"))
                                    w.uint32(64).int64(m.reconciled_value_micros);
                                if (m.billed != null && Object.hasOwnProperty.call(m, "billed"))
                                    w.uint32(72).bool(m.billed);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(80).int32(m.status);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(90).string(m.campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified HotelReconciliation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.HotelReconciliation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IHotelReconciliation} message HotelReconciliation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            HotelReconciliation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a HotelReconciliation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.HotelReconciliation} HotelReconciliation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelReconciliation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.HotelReconciliation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.commission_id = r.string();
                                        break;
                                    case 3:
                                        m.order_id = r.string();
                                        break;
                                    case 11:
                                        m.campaign = r.string();
                                        break;
                                    case 4:
                                        m.hotel_center_id = r.int64();
                                        break;
                                    case 5:
                                        m.hotel_id = r.string();
                                        break;
                                    case 6:
                                        m.check_in_date = r.string();
                                        break;
                                    case 7:
                                        m.check_out_date = r.string();
                                        break;
                                    case 8:
                                        m.reconciled_value_micros = r.int64();
                                        break;
                                    case 9:
                                        m.billed = r.bool();
                                        break;
                                    case 10:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a HotelReconciliation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.HotelReconciliation} HotelReconciliation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            HotelReconciliation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a HotelReconciliation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            HotelReconciliation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.commission_id != null && m.hasOwnProperty("commission_id")) {
                                    if (!$util.isString(m.commission_id))
                                        return "commission_id: string expected";
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    if (!$util.isString(m.order_id))
                                        return "order_id: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                    if (!$util.isInteger(m.hotel_center_id) && !(m.hotel_center_id && $util.isInteger(m.hotel_center_id.low) && $util.isInteger(m.hotel_center_id.high)))
                                        return "hotel_center_id: integer|Long expected";
                                }
                                if (m.hotel_id != null && m.hasOwnProperty("hotel_id")) {
                                    if (!$util.isString(m.hotel_id))
                                        return "hotel_id: string expected";
                                }
                                if (m.check_in_date != null && m.hasOwnProperty("check_in_date")) {
                                    if (!$util.isString(m.check_in_date))
                                        return "check_in_date: string expected";
                                }
                                if (m.check_out_date != null && m.hasOwnProperty("check_out_date")) {
                                    if (!$util.isString(m.check_out_date))
                                        return "check_out_date: string expected";
                                }
                                if (m.reconciled_value_micros != null && m.hasOwnProperty("reconciled_value_micros")) {
                                    if (!$util.isInteger(m.reconciled_value_micros) && !(m.reconciled_value_micros && $util.isInteger(m.reconciled_value_micros.low) && $util.isInteger(m.reconciled_value_micros.high)))
                                        return "reconciled_value_micros: integer|Long expected";
                                }
                                if (m.billed != null && m.hasOwnProperty("billed")) {
                                    if (typeof m.billed !== "boolean")
                                        return "billed: boolean expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a HotelReconciliation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.HotelReconciliation} HotelReconciliation
                             */
                            HotelReconciliation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.HotelReconciliation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.HotelReconciliation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.commission_id != null) {
                                    m.commission_id = String(d.commission_id);
                                }
                                if (d.order_id != null) {
                                    m.order_id = String(d.order_id);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.hotel_center_id != null) {
                                    if ($util.Long)
                                        (m.hotel_center_id = $util.Long.fromValue(d.hotel_center_id)).unsigned = false;
                                    else if (typeof d.hotel_center_id === "string")
                                        m.hotel_center_id = parseInt(d.hotel_center_id, 10);
                                    else if (typeof d.hotel_center_id === "number")
                                        m.hotel_center_id = d.hotel_center_id;
                                    else if (typeof d.hotel_center_id === "object")
                                        m.hotel_center_id = new $util.LongBits(d.hotel_center_id.low >>> 0, d.hotel_center_id.high >>> 0).toNumber();
                                }
                                if (d.hotel_id != null) {
                                    m.hotel_id = String(d.hotel_id);
                                }
                                if (d.check_in_date != null) {
                                    m.check_in_date = String(d.check_in_date);
                                }
                                if (d.check_out_date != null) {
                                    m.check_out_date = String(d.check_out_date);
                                }
                                if (d.reconciled_value_micros != null) {
                                    if ($util.Long)
                                        (m.reconciled_value_micros = $util.Long.fromValue(d.reconciled_value_micros)).unsigned = false;
                                    else if (typeof d.reconciled_value_micros === "string")
                                        m.reconciled_value_micros = parseInt(d.reconciled_value_micros, 10);
                                    else if (typeof d.reconciled_value_micros === "number")
                                        m.reconciled_value_micros = d.reconciled_value_micros;
                                    else if (typeof d.reconciled_value_micros === "object")
                                        m.reconciled_value_micros = new $util.LongBits(d.reconciled_value_micros.low >>> 0, d.reconciled_value_micros.high >>> 0).toNumber();
                                }
                                if (d.billed != null) {
                                    m.billed = Boolean(d.billed);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "RESERVATION_ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "RECONCILIATION_NEEDED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "RECONCILED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "CANCELED":
                                case 5:
                                    m.status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a HotelReconciliation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @static
                             * @param {google.ads.googleads.v11.resources.HotelReconciliation} m HotelReconciliation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            HotelReconciliation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.commission_id = "";
                                    d.order_id = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.hotel_center_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.hotel_center_id = o.longs === String ? "0" : 0;
                                    d.hotel_id = "";
                                    d.check_in_date = "";
                                    d.check_out_date = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.reconciled_value_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.reconciled_value_micros = o.longs === String ? "0" : 0;
                                    d.billed = false;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.campaign = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.commission_id != null && m.hasOwnProperty("commission_id")) {
                                    d.commission_id = m.commission_id;
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    d.order_id = m.order_id;
                                }
                                if (m.hotel_center_id != null && m.hasOwnProperty("hotel_center_id")) {
                                    if (typeof m.hotel_center_id === "number")
                                        d.hotel_center_id = o.longs === String ? String(m.hotel_center_id) : m.hotel_center_id;
                                    else
                                        d.hotel_center_id = o.longs === String ? $util.Long.prototype.toString.call(m.hotel_center_id) : o.longs === Number ? new $util.LongBits(m.hotel_center_id.low >>> 0, m.hotel_center_id.high >>> 0).toNumber() : m.hotel_center_id;
                                }
                                if (m.hotel_id != null && m.hasOwnProperty("hotel_id")) {
                                    d.hotel_id = m.hotel_id;
                                }
                                if (m.check_in_date != null && m.hasOwnProperty("check_in_date")) {
                                    d.check_in_date = m.check_in_date;
                                }
                                if (m.check_out_date != null && m.hasOwnProperty("check_out_date")) {
                                    d.check_out_date = m.check_out_date;
                                }
                                if (m.reconciled_value_micros != null && m.hasOwnProperty("reconciled_value_micros")) {
                                    if (typeof m.reconciled_value_micros === "number")
                                        d.reconciled_value_micros = o.longs === String ? String(m.reconciled_value_micros) : m.reconciled_value_micros;
                                    else
                                        d.reconciled_value_micros = o.longs === String ? $util.Long.prototype.toString.call(m.reconciled_value_micros) : o.longs === Number ? new $util.LongBits(m.reconciled_value_micros.low >>> 0, m.reconciled_value_micros.high >>> 0).toNumber() : m.reconciled_value_micros;
                                }
                                if (m.billed != null && m.hasOwnProperty("billed")) {
                                    d.billed = m.billed;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.HotelReconciliationStatusEnum.HotelReconciliationStatus[m.status] : m.status;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this HotelReconciliation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.HotelReconciliation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            HotelReconciliation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return HotelReconciliation;
                        })();
    
                        resources.IncomeRangeView = (function() {
    
                            /**
                             * Properties of an IncomeRangeView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IIncomeRangeView
                             * @property {string|null} [resource_name] IncomeRangeView resource_name
                             */
    
                            /**
                             * Constructs a new IncomeRangeView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an IncomeRangeView.
                             * @implements IIncomeRangeView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IIncomeRangeView=} [p] Properties to set
                             */
                            function IncomeRangeView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * IncomeRangeView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @instance
                             */
                            IncomeRangeView.prototype.resource_name = "";
    
                            /**
                             * Creates a new IncomeRangeView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IIncomeRangeView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.IncomeRangeView} IncomeRangeView instance
                             */
                            IncomeRangeView.create = function create(properties) {
                                return new IncomeRangeView(properties);
                            };
    
                            /**
                             * Encodes the specified IncomeRangeView message. Does not implicitly {@link google.ads.googleads.v11.resources.IncomeRangeView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IIncomeRangeView} m IncomeRangeView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IncomeRangeView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified IncomeRangeView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.IncomeRangeView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IIncomeRangeView} message IncomeRangeView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            IncomeRangeView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an IncomeRangeView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.IncomeRangeView} IncomeRangeView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IncomeRangeView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.IncomeRangeView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an IncomeRangeView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.IncomeRangeView} IncomeRangeView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            IncomeRangeView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an IncomeRangeView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            IncomeRangeView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an IncomeRangeView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.IncomeRangeView} IncomeRangeView
                             */
                            IncomeRangeView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.IncomeRangeView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.IncomeRangeView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an IncomeRangeView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IncomeRangeView} m IncomeRangeView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            IncomeRangeView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this IncomeRangeView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.IncomeRangeView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            IncomeRangeView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return IncomeRangeView;
                        })();
    
                        resources.Invoice = (function() {
    
                            /**
                             * Properties of an Invoice.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IInvoice
                             * @property {string|null} [resource_name] Invoice resource_name
                             * @property {string|null} [id] Invoice id
                             * @property {google.ads.googleads.v11.enums.InvoiceTypeEnum.InvoiceType|null} [type] Invoice type
                             * @property {string|null} [billing_setup] Invoice billing_setup
                             * @property {string|null} [payments_account_id] Invoice payments_account_id
                             * @property {string|null} [payments_profile_id] Invoice payments_profile_id
                             * @property {string|null} [issue_date] Invoice issue_date
                             * @property {string|null} [due_date] Invoice due_date
                             * @property {google.ads.googleads.v11.common.IDateRange|null} [service_date_range] Invoice service_date_range
                             * @property {string|null} [currency_code] Invoice currency_code
                             * @property {number|null} [adjustments_subtotal_amount_micros] Invoice adjustments_subtotal_amount_micros
                             * @property {number|null} [adjustments_tax_amount_micros] Invoice adjustments_tax_amount_micros
                             * @property {number|null} [adjustments_total_amount_micros] Invoice adjustments_total_amount_micros
                             * @property {number|null} [regulatory_costs_subtotal_amount_micros] Invoice regulatory_costs_subtotal_amount_micros
                             * @property {number|null} [regulatory_costs_tax_amount_micros] Invoice regulatory_costs_tax_amount_micros
                             * @property {number|null} [regulatory_costs_total_amount_micros] Invoice regulatory_costs_total_amount_micros
                             * @property {number|null} [subtotal_amount_micros] Invoice subtotal_amount_micros
                             * @property {number|null} [tax_amount_micros] Invoice tax_amount_micros
                             * @property {number|null} [total_amount_micros] Invoice total_amount_micros
                             * @property {string|null} [corrected_invoice] Invoice corrected_invoice
                             * @property {Array.<string>|null} [replaced_invoices] Invoice replaced_invoices
                             * @property {string|null} [pdf_url] Invoice pdf_url
                             * @property {Array.<google.ads.googleads.v11.resources.Invoice.IAccountBudgetSummary>|null} [account_budget_summaries] Invoice account_budget_summaries
                             */
    
                            /**
                             * Constructs a new Invoice.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an Invoice.
                             * @implements IInvoice
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IInvoice=} [p] Properties to set
                             */
                            function Invoice(p) {
                                this.replaced_invoices = [];
                                this.account_budget_summaries = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Invoice resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.resource_name = "";
    
                            /**
                             * Invoice id.
                             * @member {string|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.id = null;
    
                            /**
                             * Invoice type.
                             * @member {google.ads.googleads.v11.enums.InvoiceTypeEnum.InvoiceType} type
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.type = 0;
    
                            /**
                             * Invoice billing_setup.
                             * @member {string|null|undefined} billing_setup
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.billing_setup = null;
    
                            /**
                             * Invoice payments_account_id.
                             * @member {string|null|undefined} payments_account_id
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.payments_account_id = null;
    
                            /**
                             * Invoice payments_profile_id.
                             * @member {string|null|undefined} payments_profile_id
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.payments_profile_id = null;
    
                            /**
                             * Invoice issue_date.
                             * @member {string|null|undefined} issue_date
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.issue_date = null;
    
                            /**
                             * Invoice due_date.
                             * @member {string|null|undefined} due_date
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.due_date = null;
    
                            /**
                             * Invoice service_date_range.
                             * @member {google.ads.googleads.v11.common.IDateRange|null|undefined} service_date_range
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.service_date_range = null;
    
                            /**
                             * Invoice currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.currency_code = null;
    
                            /**
                             * Invoice adjustments_subtotal_amount_micros.
                             * @member {number} adjustments_subtotal_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.adjustments_subtotal_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Invoice adjustments_tax_amount_micros.
                             * @member {number} adjustments_tax_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.adjustments_tax_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Invoice adjustments_total_amount_micros.
                             * @member {number} adjustments_total_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.adjustments_total_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Invoice regulatory_costs_subtotal_amount_micros.
                             * @member {number} regulatory_costs_subtotal_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.regulatory_costs_subtotal_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Invoice regulatory_costs_tax_amount_micros.
                             * @member {number} regulatory_costs_tax_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.regulatory_costs_tax_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Invoice regulatory_costs_total_amount_micros.
                             * @member {number} regulatory_costs_total_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.regulatory_costs_total_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Invoice subtotal_amount_micros.
                             * @member {number|null|undefined} subtotal_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.subtotal_amount_micros = null;
    
                            /**
                             * Invoice tax_amount_micros.
                             * @member {number|null|undefined} tax_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.tax_amount_micros = null;
    
                            /**
                             * Invoice total_amount_micros.
                             * @member {number|null|undefined} total_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.total_amount_micros = null;
    
                            /**
                             * Invoice corrected_invoice.
                             * @member {string|null|undefined} corrected_invoice
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.corrected_invoice = null;
    
                            /**
                             * Invoice replaced_invoices.
                             * @member {Array.<string>} replaced_invoices
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.replaced_invoices = $util.emptyArray;
    
                            /**
                             * Invoice pdf_url.
                             * @member {string|null|undefined} pdf_url
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.pdf_url = null;
    
                            /**
                             * Invoice account_budget_summaries.
                             * @member {Array.<google.ads.googleads.v11.resources.Invoice.IAccountBudgetSummary>} account_budget_summaries
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Invoice.prototype.account_budget_summaries = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Invoice _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _billing_setup.
                             * @member {"billing_setup"|undefined} _billing_setup
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_billing_setup", {
                                get: $util.oneOfGetter($oneOfFields = ["billing_setup"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _payments_account_id.
                             * @member {"payments_account_id"|undefined} _payments_account_id
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_payments_account_id", {
                                get: $util.oneOfGetter($oneOfFields = ["payments_account_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _payments_profile_id.
                             * @member {"payments_profile_id"|undefined} _payments_profile_id
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_payments_profile_id", {
                                get: $util.oneOfGetter($oneOfFields = ["payments_profile_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _issue_date.
                             * @member {"issue_date"|undefined} _issue_date
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_issue_date", {
                                get: $util.oneOfGetter($oneOfFields = ["issue_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _due_date.
                             * @member {"due_date"|undefined} _due_date
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_due_date", {
                                get: $util.oneOfGetter($oneOfFields = ["due_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _subtotal_amount_micros.
                             * @member {"subtotal_amount_micros"|undefined} _subtotal_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_subtotal_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["subtotal_amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _tax_amount_micros.
                             * @member {"tax_amount_micros"|undefined} _tax_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_tax_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["tax_amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _total_amount_micros.
                             * @member {"total_amount_micros"|undefined} _total_amount_micros
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_total_amount_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["total_amount_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _corrected_invoice.
                             * @member {"corrected_invoice"|undefined} _corrected_invoice
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_corrected_invoice", {
                                get: $util.oneOfGetter($oneOfFields = ["corrected_invoice"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Invoice _pdf_url.
                             * @member {"pdf_url"|undefined} _pdf_url
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             */
                            Object.defineProperty(Invoice.prototype, "_pdf_url", {
                                get: $util.oneOfGetter($oneOfFields = ["pdf_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Invoice instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {google.ads.googleads.v11.resources.IInvoice=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Invoice} Invoice instance
                             */
                            Invoice.create = function create(properties) {
                                return new Invoice(properties);
                            };
    
                            /**
                             * Encodes the specified Invoice message. Does not implicitly {@link google.ads.googleads.v11.resources.Invoice.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {google.ads.googleads.v11.resources.IInvoice} m Invoice message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Invoice.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.service_date_range != null && Object.hasOwnProperty.call(m, "service_date_range"))
                                    $root.google.ads.googleads.v11.common.DateRange.encode(m.service_date_range, w.uint32(74).fork()).ldelim();
                                if (m.account_budget_summaries != null && m.account_budget_summaries.length) {
                                    for (var i = 0; i < m.account_budget_summaries.length; ++i)
                                        $root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.encode(m.account_budget_summaries[i], w.uint32(146).fork()).ldelim();
                                }
                                if (m.adjustments_subtotal_amount_micros != null && Object.hasOwnProperty.call(m, "adjustments_subtotal_amount_micros"))
                                    w.uint32(152).int64(m.adjustments_subtotal_amount_micros);
                                if (m.adjustments_tax_amount_micros != null && Object.hasOwnProperty.call(m, "adjustments_tax_amount_micros"))
                                    w.uint32(160).int64(m.adjustments_tax_amount_micros);
                                if (m.adjustments_total_amount_micros != null && Object.hasOwnProperty.call(m, "adjustments_total_amount_micros"))
                                    w.uint32(168).int64(m.adjustments_total_amount_micros);
                                if (m.regulatory_costs_subtotal_amount_micros != null && Object.hasOwnProperty.call(m, "regulatory_costs_subtotal_amount_micros"))
                                    w.uint32(176).int64(m.regulatory_costs_subtotal_amount_micros);
                                if (m.regulatory_costs_tax_amount_micros != null && Object.hasOwnProperty.call(m, "regulatory_costs_tax_amount_micros"))
                                    w.uint32(184).int64(m.regulatory_costs_tax_amount_micros);
                                if (m.regulatory_costs_total_amount_micros != null && Object.hasOwnProperty.call(m, "regulatory_costs_total_amount_micros"))
                                    w.uint32(192).int64(m.regulatory_costs_total_amount_micros);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(202).string(m.id);
                                if (m.billing_setup != null && Object.hasOwnProperty.call(m, "billing_setup"))
                                    w.uint32(210).string(m.billing_setup);
                                if (m.payments_account_id != null && Object.hasOwnProperty.call(m, "payments_account_id"))
                                    w.uint32(218).string(m.payments_account_id);
                                if (m.payments_profile_id != null && Object.hasOwnProperty.call(m, "payments_profile_id"))
                                    w.uint32(226).string(m.payments_profile_id);
                                if (m.issue_date != null && Object.hasOwnProperty.call(m, "issue_date"))
                                    w.uint32(234).string(m.issue_date);
                                if (m.due_date != null && Object.hasOwnProperty.call(m, "due_date"))
                                    w.uint32(242).string(m.due_date);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(250).string(m.currency_code);
                                if (m.subtotal_amount_micros != null && Object.hasOwnProperty.call(m, "subtotal_amount_micros"))
                                    w.uint32(264).int64(m.subtotal_amount_micros);
                                if (m.tax_amount_micros != null && Object.hasOwnProperty.call(m, "tax_amount_micros"))
                                    w.uint32(272).int64(m.tax_amount_micros);
                                if (m.total_amount_micros != null && Object.hasOwnProperty.call(m, "total_amount_micros"))
                                    w.uint32(280).int64(m.total_amount_micros);
                                if (m.corrected_invoice != null && Object.hasOwnProperty.call(m, "corrected_invoice"))
                                    w.uint32(290).string(m.corrected_invoice);
                                if (m.replaced_invoices != null && m.replaced_invoices.length) {
                                    for (var i = 0; i < m.replaced_invoices.length; ++i)
                                        w.uint32(298).string(m.replaced_invoices[i]);
                                }
                                if (m.pdf_url != null && Object.hasOwnProperty.call(m, "pdf_url"))
                                    w.uint32(306).string(m.pdf_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Invoice message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Invoice.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {google.ads.googleads.v11.resources.IInvoice} message Invoice message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Invoice.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Invoice message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Invoice} Invoice
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Invoice.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Invoice();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 25:
                                        m.id = r.string();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 26:
                                        m.billing_setup = r.string();
                                        break;
                                    case 27:
                                        m.payments_account_id = r.string();
                                        break;
                                    case 28:
                                        m.payments_profile_id = r.string();
                                        break;
                                    case 29:
                                        m.issue_date = r.string();
                                        break;
                                    case 30:
                                        m.due_date = r.string();
                                        break;
                                    case 9:
                                        m.service_date_range = $root.google.ads.googleads.v11.common.DateRange.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.currency_code = r.string();
                                        break;
                                    case 19:
                                        m.adjustments_subtotal_amount_micros = r.int64();
                                        break;
                                    case 20:
                                        m.adjustments_tax_amount_micros = r.int64();
                                        break;
                                    case 21:
                                        m.adjustments_total_amount_micros = r.int64();
                                        break;
                                    case 22:
                                        m.regulatory_costs_subtotal_amount_micros = r.int64();
                                        break;
                                    case 23:
                                        m.regulatory_costs_tax_amount_micros = r.int64();
                                        break;
                                    case 24:
                                        m.regulatory_costs_total_amount_micros = r.int64();
                                        break;
                                    case 33:
                                        m.subtotal_amount_micros = r.int64();
                                        break;
                                    case 34:
                                        m.tax_amount_micros = r.int64();
                                        break;
                                    case 35:
                                        m.total_amount_micros = r.int64();
                                        break;
                                    case 36:
                                        m.corrected_invoice = r.string();
                                        break;
                                    case 37:
                                        if (!(m.replaced_invoices && m.replaced_invoices.length))
                                            m.replaced_invoices = [];
                                        m.replaced_invoices.push(r.string());
                                        break;
                                    case 38:
                                        m.pdf_url = r.string();
                                        break;
                                    case 18:
                                        if (!(m.account_budget_summaries && m.account_budget_summaries.length))
                                            m.account_budget_summaries = [];
                                        m.account_budget_summaries.push($root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an Invoice message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Invoice} Invoice
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Invoice.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Invoice message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Invoice.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isString(m.id))
                                        return "id: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    p._billing_setup = 1;
                                    if (!$util.isString(m.billing_setup))
                                        return "billing_setup: string expected";
                                }
                                if (m.payments_account_id != null && m.hasOwnProperty("payments_account_id")) {
                                    p._payments_account_id = 1;
                                    if (!$util.isString(m.payments_account_id))
                                        return "payments_account_id: string expected";
                                }
                                if (m.payments_profile_id != null && m.hasOwnProperty("payments_profile_id")) {
                                    p._payments_profile_id = 1;
                                    if (!$util.isString(m.payments_profile_id))
                                        return "payments_profile_id: string expected";
                                }
                                if (m.issue_date != null && m.hasOwnProperty("issue_date")) {
                                    p._issue_date = 1;
                                    if (!$util.isString(m.issue_date))
                                        return "issue_date: string expected";
                                }
                                if (m.due_date != null && m.hasOwnProperty("due_date")) {
                                    p._due_date = 1;
                                    if (!$util.isString(m.due_date))
                                        return "due_date: string expected";
                                }
                                if (m.service_date_range != null && m.hasOwnProperty("service_date_range")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DateRange.verify(m.service_date_range);
                                        if (e)
                                            return "service_date_range." + e;
                                    }
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.adjustments_subtotal_amount_micros != null && m.hasOwnProperty("adjustments_subtotal_amount_micros")) {
                                    if (!$util.isInteger(m.adjustments_subtotal_amount_micros) && !(m.adjustments_subtotal_amount_micros && $util.isInteger(m.adjustments_subtotal_amount_micros.low) && $util.isInteger(m.adjustments_subtotal_amount_micros.high)))
                                        return "adjustments_subtotal_amount_micros: integer|Long expected";
                                }
                                if (m.adjustments_tax_amount_micros != null && m.hasOwnProperty("adjustments_tax_amount_micros")) {
                                    if (!$util.isInteger(m.adjustments_tax_amount_micros) && !(m.adjustments_tax_amount_micros && $util.isInteger(m.adjustments_tax_amount_micros.low) && $util.isInteger(m.adjustments_tax_amount_micros.high)))
                                        return "adjustments_tax_amount_micros: integer|Long expected";
                                }
                                if (m.adjustments_total_amount_micros != null && m.hasOwnProperty("adjustments_total_amount_micros")) {
                                    if (!$util.isInteger(m.adjustments_total_amount_micros) && !(m.adjustments_total_amount_micros && $util.isInteger(m.adjustments_total_amount_micros.low) && $util.isInteger(m.adjustments_total_amount_micros.high)))
                                        return "adjustments_total_amount_micros: integer|Long expected";
                                }
                                if (m.regulatory_costs_subtotal_amount_micros != null && m.hasOwnProperty("regulatory_costs_subtotal_amount_micros")) {
                                    if (!$util.isInteger(m.regulatory_costs_subtotal_amount_micros) && !(m.regulatory_costs_subtotal_amount_micros && $util.isInteger(m.regulatory_costs_subtotal_amount_micros.low) && $util.isInteger(m.regulatory_costs_subtotal_amount_micros.high)))
                                        return "regulatory_costs_subtotal_amount_micros: integer|Long expected";
                                }
                                if (m.regulatory_costs_tax_amount_micros != null && m.hasOwnProperty("regulatory_costs_tax_amount_micros")) {
                                    if (!$util.isInteger(m.regulatory_costs_tax_amount_micros) && !(m.regulatory_costs_tax_amount_micros && $util.isInteger(m.regulatory_costs_tax_amount_micros.low) && $util.isInteger(m.regulatory_costs_tax_amount_micros.high)))
                                        return "regulatory_costs_tax_amount_micros: integer|Long expected";
                                }
                                if (m.regulatory_costs_total_amount_micros != null && m.hasOwnProperty("regulatory_costs_total_amount_micros")) {
                                    if (!$util.isInteger(m.regulatory_costs_total_amount_micros) && !(m.regulatory_costs_total_amount_micros && $util.isInteger(m.regulatory_costs_total_amount_micros.low) && $util.isInteger(m.regulatory_costs_total_amount_micros.high)))
                                        return "regulatory_costs_total_amount_micros: integer|Long expected";
                                }
                                if (m.subtotal_amount_micros != null && m.hasOwnProperty("subtotal_amount_micros")) {
                                    p._subtotal_amount_micros = 1;
                                    if (!$util.isInteger(m.subtotal_amount_micros) && !(m.subtotal_amount_micros && $util.isInteger(m.subtotal_amount_micros.low) && $util.isInteger(m.subtotal_amount_micros.high)))
                                        return "subtotal_amount_micros: integer|Long expected";
                                }
                                if (m.tax_amount_micros != null && m.hasOwnProperty("tax_amount_micros")) {
                                    p._tax_amount_micros = 1;
                                    if (!$util.isInteger(m.tax_amount_micros) && !(m.tax_amount_micros && $util.isInteger(m.tax_amount_micros.low) && $util.isInteger(m.tax_amount_micros.high)))
                                        return "tax_amount_micros: integer|Long expected";
                                }
                                if (m.total_amount_micros != null && m.hasOwnProperty("total_amount_micros")) {
                                    p._total_amount_micros = 1;
                                    if (!$util.isInteger(m.total_amount_micros) && !(m.total_amount_micros && $util.isInteger(m.total_amount_micros.low) && $util.isInteger(m.total_amount_micros.high)))
                                        return "total_amount_micros: integer|Long expected";
                                }
                                if (m.corrected_invoice != null && m.hasOwnProperty("corrected_invoice")) {
                                    p._corrected_invoice = 1;
                                    if (!$util.isString(m.corrected_invoice))
                                        return "corrected_invoice: string expected";
                                }
                                if (m.replaced_invoices != null && m.hasOwnProperty("replaced_invoices")) {
                                    if (!Array.isArray(m.replaced_invoices))
                                        return "replaced_invoices: array expected";
                                    for (var i = 0; i < m.replaced_invoices.length; ++i) {
                                        if (!$util.isString(m.replaced_invoices[i]))
                                            return "replaced_invoices: string[] expected";
                                    }
                                }
                                if (m.pdf_url != null && m.hasOwnProperty("pdf_url")) {
                                    p._pdf_url = 1;
                                    if (!$util.isString(m.pdf_url))
                                        return "pdf_url: string expected";
                                }
                                if (m.account_budget_summaries != null && m.hasOwnProperty("account_budget_summaries")) {
                                    if (!Array.isArray(m.account_budget_summaries))
                                        return "account_budget_summaries: array expected";
                                    for (var i = 0; i < m.account_budget_summaries.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.verify(m.account_budget_summaries[i]);
                                            if (e)
                                                return "account_budget_summaries." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Invoice message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Invoice} Invoice
                             */
                            Invoice.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Invoice)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Invoice();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    m.id = String(d.id);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CREDIT_MEMO":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "INVOICE":
                                case 3:
                                    m.type = 3;
                                    break;
                                }
                                if (d.billing_setup != null) {
                                    m.billing_setup = String(d.billing_setup);
                                }
                                if (d.payments_account_id != null) {
                                    m.payments_account_id = String(d.payments_account_id);
                                }
                                if (d.payments_profile_id != null) {
                                    m.payments_profile_id = String(d.payments_profile_id);
                                }
                                if (d.issue_date != null) {
                                    m.issue_date = String(d.issue_date);
                                }
                                if (d.due_date != null) {
                                    m.due_date = String(d.due_date);
                                }
                                if (d.service_date_range != null) {
                                    if (typeof d.service_date_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Invoice.service_date_range: object expected");
                                    m.service_date_range = $root.google.ads.googleads.v11.common.DateRange.fromObject(d.service_date_range);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.adjustments_subtotal_amount_micros != null) {
                                    if ($util.Long)
                                        (m.adjustments_subtotal_amount_micros = $util.Long.fromValue(d.adjustments_subtotal_amount_micros)).unsigned = false;
                                    else if (typeof d.adjustments_subtotal_amount_micros === "string")
                                        m.adjustments_subtotal_amount_micros = parseInt(d.adjustments_subtotal_amount_micros, 10);
                                    else if (typeof d.adjustments_subtotal_amount_micros === "number")
                                        m.adjustments_subtotal_amount_micros = d.adjustments_subtotal_amount_micros;
                                    else if (typeof d.adjustments_subtotal_amount_micros === "object")
                                        m.adjustments_subtotal_amount_micros = new $util.LongBits(d.adjustments_subtotal_amount_micros.low >>> 0, d.adjustments_subtotal_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.adjustments_tax_amount_micros != null) {
                                    if ($util.Long)
                                        (m.adjustments_tax_amount_micros = $util.Long.fromValue(d.adjustments_tax_amount_micros)).unsigned = false;
                                    else if (typeof d.adjustments_tax_amount_micros === "string")
                                        m.adjustments_tax_amount_micros = parseInt(d.adjustments_tax_amount_micros, 10);
                                    else if (typeof d.adjustments_tax_amount_micros === "number")
                                        m.adjustments_tax_amount_micros = d.adjustments_tax_amount_micros;
                                    else if (typeof d.adjustments_tax_amount_micros === "object")
                                        m.adjustments_tax_amount_micros = new $util.LongBits(d.adjustments_tax_amount_micros.low >>> 0, d.adjustments_tax_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.adjustments_total_amount_micros != null) {
                                    if ($util.Long)
                                        (m.adjustments_total_amount_micros = $util.Long.fromValue(d.adjustments_total_amount_micros)).unsigned = false;
                                    else if (typeof d.adjustments_total_amount_micros === "string")
                                        m.adjustments_total_amount_micros = parseInt(d.adjustments_total_amount_micros, 10);
                                    else if (typeof d.adjustments_total_amount_micros === "number")
                                        m.adjustments_total_amount_micros = d.adjustments_total_amount_micros;
                                    else if (typeof d.adjustments_total_amount_micros === "object")
                                        m.adjustments_total_amount_micros = new $util.LongBits(d.adjustments_total_amount_micros.low >>> 0, d.adjustments_total_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.regulatory_costs_subtotal_amount_micros != null) {
                                    if ($util.Long)
                                        (m.regulatory_costs_subtotal_amount_micros = $util.Long.fromValue(d.regulatory_costs_subtotal_amount_micros)).unsigned = false;
                                    else if (typeof d.regulatory_costs_subtotal_amount_micros === "string")
                                        m.regulatory_costs_subtotal_amount_micros = parseInt(d.regulatory_costs_subtotal_amount_micros, 10);
                                    else if (typeof d.regulatory_costs_subtotal_amount_micros === "number")
                                        m.regulatory_costs_subtotal_amount_micros = d.regulatory_costs_subtotal_amount_micros;
                                    else if (typeof d.regulatory_costs_subtotal_amount_micros === "object")
                                        m.regulatory_costs_subtotal_amount_micros = new $util.LongBits(d.regulatory_costs_subtotal_amount_micros.low >>> 0, d.regulatory_costs_subtotal_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.regulatory_costs_tax_amount_micros != null) {
                                    if ($util.Long)
                                        (m.regulatory_costs_tax_amount_micros = $util.Long.fromValue(d.regulatory_costs_tax_amount_micros)).unsigned = false;
                                    else if (typeof d.regulatory_costs_tax_amount_micros === "string")
                                        m.regulatory_costs_tax_amount_micros = parseInt(d.regulatory_costs_tax_amount_micros, 10);
                                    else if (typeof d.regulatory_costs_tax_amount_micros === "number")
                                        m.regulatory_costs_tax_amount_micros = d.regulatory_costs_tax_amount_micros;
                                    else if (typeof d.regulatory_costs_tax_amount_micros === "object")
                                        m.regulatory_costs_tax_amount_micros = new $util.LongBits(d.regulatory_costs_tax_amount_micros.low >>> 0, d.regulatory_costs_tax_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.regulatory_costs_total_amount_micros != null) {
                                    if ($util.Long)
                                        (m.regulatory_costs_total_amount_micros = $util.Long.fromValue(d.regulatory_costs_total_amount_micros)).unsigned = false;
                                    else if (typeof d.regulatory_costs_total_amount_micros === "string")
                                        m.regulatory_costs_total_amount_micros = parseInt(d.regulatory_costs_total_amount_micros, 10);
                                    else if (typeof d.regulatory_costs_total_amount_micros === "number")
                                        m.regulatory_costs_total_amount_micros = d.regulatory_costs_total_amount_micros;
                                    else if (typeof d.regulatory_costs_total_amount_micros === "object")
                                        m.regulatory_costs_total_amount_micros = new $util.LongBits(d.regulatory_costs_total_amount_micros.low >>> 0, d.regulatory_costs_total_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.subtotal_amount_micros != null) {
                                    if ($util.Long)
                                        (m.subtotal_amount_micros = $util.Long.fromValue(d.subtotal_amount_micros)).unsigned = false;
                                    else if (typeof d.subtotal_amount_micros === "string")
                                        m.subtotal_amount_micros = parseInt(d.subtotal_amount_micros, 10);
                                    else if (typeof d.subtotal_amount_micros === "number")
                                        m.subtotal_amount_micros = d.subtotal_amount_micros;
                                    else if (typeof d.subtotal_amount_micros === "object")
                                        m.subtotal_amount_micros = new $util.LongBits(d.subtotal_amount_micros.low >>> 0, d.subtotal_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.tax_amount_micros != null) {
                                    if ($util.Long)
                                        (m.tax_amount_micros = $util.Long.fromValue(d.tax_amount_micros)).unsigned = false;
                                    else if (typeof d.tax_amount_micros === "string")
                                        m.tax_amount_micros = parseInt(d.tax_amount_micros, 10);
                                    else if (typeof d.tax_amount_micros === "number")
                                        m.tax_amount_micros = d.tax_amount_micros;
                                    else if (typeof d.tax_amount_micros === "object")
                                        m.tax_amount_micros = new $util.LongBits(d.tax_amount_micros.low >>> 0, d.tax_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.total_amount_micros != null) {
                                    if ($util.Long)
                                        (m.total_amount_micros = $util.Long.fromValue(d.total_amount_micros)).unsigned = false;
                                    else if (typeof d.total_amount_micros === "string")
                                        m.total_amount_micros = parseInt(d.total_amount_micros, 10);
                                    else if (typeof d.total_amount_micros === "number")
                                        m.total_amount_micros = d.total_amount_micros;
                                    else if (typeof d.total_amount_micros === "object")
                                        m.total_amount_micros = new $util.LongBits(d.total_amount_micros.low >>> 0, d.total_amount_micros.high >>> 0).toNumber();
                                }
                                if (d.corrected_invoice != null) {
                                    m.corrected_invoice = String(d.corrected_invoice);
                                }
                                if (d.replaced_invoices) {
                                    if (!Array.isArray(d.replaced_invoices))
                                        throw TypeError(".google.ads.googleads.v11.resources.Invoice.replaced_invoices: array expected");
                                    m.replaced_invoices = [];
                                    for (var i = 0; i < d.replaced_invoices.length; ++i) {
                                        m.replaced_invoices[i] = String(d.replaced_invoices[i]);
                                    }
                                }
                                if (d.pdf_url != null) {
                                    m.pdf_url = String(d.pdf_url);
                                }
                                if (d.account_budget_summaries) {
                                    if (!Array.isArray(d.account_budget_summaries))
                                        throw TypeError(".google.ads.googleads.v11.resources.Invoice.account_budget_summaries: array expected");
                                    m.account_budget_summaries = [];
                                    for (var i = 0; i < d.account_budget_summaries.length; ++i) {
                                        if (typeof d.account_budget_summaries[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Invoice.account_budget_summaries: object expected");
                                        m.account_budget_summaries[i] = $root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.fromObject(d.account_budget_summaries[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an Invoice message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @static
                             * @param {google.ads.googleads.v11.resources.Invoice} m Invoice
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Invoice.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.account_budget_summaries = [];
                                    d.replaced_invoices = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.service_date_range = null;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.adjustments_subtotal_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.adjustments_subtotal_amount_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.adjustments_tax_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.adjustments_tax_amount_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.adjustments_total_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.adjustments_total_amount_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.regulatory_costs_subtotal_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.regulatory_costs_subtotal_amount_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.regulatory_costs_tax_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.regulatory_costs_tax_amount_micros = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.regulatory_costs_total_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.regulatory_costs_total_amount_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.InvoiceTypeEnum.InvoiceType[m.type] : m.type;
                                }
                                if (m.service_date_range != null && m.hasOwnProperty("service_date_range")) {
                                    d.service_date_range = $root.google.ads.googleads.v11.common.DateRange.toObject(m.service_date_range, o);
                                }
                                if (m.account_budget_summaries && m.account_budget_summaries.length) {
                                    d.account_budget_summaries = [];
                                    for (var j = 0; j < m.account_budget_summaries.length; ++j) {
                                        d.account_budget_summaries[j] = $root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.toObject(m.account_budget_summaries[j], o);
                                    }
                                }
                                if (m.adjustments_subtotal_amount_micros != null && m.hasOwnProperty("adjustments_subtotal_amount_micros")) {
                                    if (typeof m.adjustments_subtotal_amount_micros === "number")
                                        d.adjustments_subtotal_amount_micros = o.longs === String ? String(m.adjustments_subtotal_amount_micros) : m.adjustments_subtotal_amount_micros;
                                    else
                                        d.adjustments_subtotal_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.adjustments_subtotal_amount_micros) : o.longs === Number ? new $util.LongBits(m.adjustments_subtotal_amount_micros.low >>> 0, m.adjustments_subtotal_amount_micros.high >>> 0).toNumber() : m.adjustments_subtotal_amount_micros;
                                }
                                if (m.adjustments_tax_amount_micros != null && m.hasOwnProperty("adjustments_tax_amount_micros")) {
                                    if (typeof m.adjustments_tax_amount_micros === "number")
                                        d.adjustments_tax_amount_micros = o.longs === String ? String(m.adjustments_tax_amount_micros) : m.adjustments_tax_amount_micros;
                                    else
                                        d.adjustments_tax_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.adjustments_tax_amount_micros) : o.longs === Number ? new $util.LongBits(m.adjustments_tax_amount_micros.low >>> 0, m.adjustments_tax_amount_micros.high >>> 0).toNumber() : m.adjustments_tax_amount_micros;
                                }
                                if (m.adjustments_total_amount_micros != null && m.hasOwnProperty("adjustments_total_amount_micros")) {
                                    if (typeof m.adjustments_total_amount_micros === "number")
                                        d.adjustments_total_amount_micros = o.longs === String ? String(m.adjustments_total_amount_micros) : m.adjustments_total_amount_micros;
                                    else
                                        d.adjustments_total_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.adjustments_total_amount_micros) : o.longs === Number ? new $util.LongBits(m.adjustments_total_amount_micros.low >>> 0, m.adjustments_total_amount_micros.high >>> 0).toNumber() : m.adjustments_total_amount_micros;
                                }
                                if (m.regulatory_costs_subtotal_amount_micros != null && m.hasOwnProperty("regulatory_costs_subtotal_amount_micros")) {
                                    if (typeof m.regulatory_costs_subtotal_amount_micros === "number")
                                        d.regulatory_costs_subtotal_amount_micros = o.longs === String ? String(m.regulatory_costs_subtotal_amount_micros) : m.regulatory_costs_subtotal_amount_micros;
                                    else
                                        d.regulatory_costs_subtotal_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.regulatory_costs_subtotal_amount_micros) : o.longs === Number ? new $util.LongBits(m.regulatory_costs_subtotal_amount_micros.low >>> 0, m.regulatory_costs_subtotal_amount_micros.high >>> 0).toNumber() : m.regulatory_costs_subtotal_amount_micros;
                                }
                                if (m.regulatory_costs_tax_amount_micros != null && m.hasOwnProperty("regulatory_costs_tax_amount_micros")) {
                                    if (typeof m.regulatory_costs_tax_amount_micros === "number")
                                        d.regulatory_costs_tax_amount_micros = o.longs === String ? String(m.regulatory_costs_tax_amount_micros) : m.regulatory_costs_tax_amount_micros;
                                    else
                                        d.regulatory_costs_tax_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.regulatory_costs_tax_amount_micros) : o.longs === Number ? new $util.LongBits(m.regulatory_costs_tax_amount_micros.low >>> 0, m.regulatory_costs_tax_amount_micros.high >>> 0).toNumber() : m.regulatory_costs_tax_amount_micros;
                                }
                                if (m.regulatory_costs_total_amount_micros != null && m.hasOwnProperty("regulatory_costs_total_amount_micros")) {
                                    if (typeof m.regulatory_costs_total_amount_micros === "number")
                                        d.regulatory_costs_total_amount_micros = o.longs === String ? String(m.regulatory_costs_total_amount_micros) : m.regulatory_costs_total_amount_micros;
                                    else
                                        d.regulatory_costs_total_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.regulatory_costs_total_amount_micros) : o.longs === Number ? new $util.LongBits(m.regulatory_costs_total_amount_micros.low >>> 0, m.regulatory_costs_total_amount_micros.high >>> 0).toNumber() : m.regulatory_costs_total_amount_micros;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    d.id = m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    d.billing_setup = m.billing_setup;
                                    if (o.oneofs)
                                        d._billing_setup = "billing_setup";
                                }
                                if (m.payments_account_id != null && m.hasOwnProperty("payments_account_id")) {
                                    d.payments_account_id = m.payments_account_id;
                                    if (o.oneofs)
                                        d._payments_account_id = "payments_account_id";
                                }
                                if (m.payments_profile_id != null && m.hasOwnProperty("payments_profile_id")) {
                                    d.payments_profile_id = m.payments_profile_id;
                                    if (o.oneofs)
                                        d._payments_profile_id = "payments_profile_id";
                                }
                                if (m.issue_date != null && m.hasOwnProperty("issue_date")) {
                                    d.issue_date = m.issue_date;
                                    if (o.oneofs)
                                        d._issue_date = "issue_date";
                                }
                                if (m.due_date != null && m.hasOwnProperty("due_date")) {
                                    d.due_date = m.due_date;
                                    if (o.oneofs)
                                        d._due_date = "due_date";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.subtotal_amount_micros != null && m.hasOwnProperty("subtotal_amount_micros")) {
                                    if (typeof m.subtotal_amount_micros === "number")
                                        d.subtotal_amount_micros = o.longs === String ? String(m.subtotal_amount_micros) : m.subtotal_amount_micros;
                                    else
                                        d.subtotal_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.subtotal_amount_micros) : o.longs === Number ? new $util.LongBits(m.subtotal_amount_micros.low >>> 0, m.subtotal_amount_micros.high >>> 0).toNumber() : m.subtotal_amount_micros;
                                    if (o.oneofs)
                                        d._subtotal_amount_micros = "subtotal_amount_micros";
                                }
                                if (m.tax_amount_micros != null && m.hasOwnProperty("tax_amount_micros")) {
                                    if (typeof m.tax_amount_micros === "number")
                                        d.tax_amount_micros = o.longs === String ? String(m.tax_amount_micros) : m.tax_amount_micros;
                                    else
                                        d.tax_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.tax_amount_micros) : o.longs === Number ? new $util.LongBits(m.tax_amount_micros.low >>> 0, m.tax_amount_micros.high >>> 0).toNumber() : m.tax_amount_micros;
                                    if (o.oneofs)
                                        d._tax_amount_micros = "tax_amount_micros";
                                }
                                if (m.total_amount_micros != null && m.hasOwnProperty("total_amount_micros")) {
                                    if (typeof m.total_amount_micros === "number")
                                        d.total_amount_micros = o.longs === String ? String(m.total_amount_micros) : m.total_amount_micros;
                                    else
                                        d.total_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.total_amount_micros) : o.longs === Number ? new $util.LongBits(m.total_amount_micros.low >>> 0, m.total_amount_micros.high >>> 0).toNumber() : m.total_amount_micros;
                                    if (o.oneofs)
                                        d._total_amount_micros = "total_amount_micros";
                                }
                                if (m.corrected_invoice != null && m.hasOwnProperty("corrected_invoice")) {
                                    d.corrected_invoice = m.corrected_invoice;
                                    if (o.oneofs)
                                        d._corrected_invoice = "corrected_invoice";
                                }
                                if (m.replaced_invoices && m.replaced_invoices.length) {
                                    d.replaced_invoices = [];
                                    for (var j = 0; j < m.replaced_invoices.length; ++j) {
                                        d.replaced_invoices[j] = m.replaced_invoices[j];
                                    }
                                }
                                if (m.pdf_url != null && m.hasOwnProperty("pdf_url")) {
                                    d.pdf_url = m.pdf_url;
                                    if (o.oneofs)
                                        d._pdf_url = "pdf_url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Invoice to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Invoice
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Invoice.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            Invoice.AccountBudgetSummary = (function() {
    
                                /**
                                 * Properties of an AccountBudgetSummary.
                                 * @memberof google.ads.googleads.v11.resources.Invoice
                                 * @interface IAccountBudgetSummary
                                 * @property {string|null} [customer] AccountBudgetSummary customer
                                 * @property {string|null} [customer_descriptive_name] AccountBudgetSummary customer_descriptive_name
                                 * @property {string|null} [account_budget] AccountBudgetSummary account_budget
                                 * @property {string|null} [account_budget_name] AccountBudgetSummary account_budget_name
                                 * @property {string|null} [purchase_order_number] AccountBudgetSummary purchase_order_number
                                 * @property {number|null} [subtotal_amount_micros] AccountBudgetSummary subtotal_amount_micros
                                 * @property {number|null} [tax_amount_micros] AccountBudgetSummary tax_amount_micros
                                 * @property {number|null} [total_amount_micros] AccountBudgetSummary total_amount_micros
                                 * @property {google.ads.googleads.v11.common.IDateRange|null} [billable_activity_date_range] AccountBudgetSummary billable_activity_date_range
                                 */
    
                                /**
                                 * Constructs a new AccountBudgetSummary.
                                 * @memberof google.ads.googleads.v11.resources.Invoice
                                 * @classdesc Represents an AccountBudgetSummary.
                                 * @implements IAccountBudgetSummary
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Invoice.IAccountBudgetSummary=} [p] Properties to set
                                 */
                                function AccountBudgetSummary(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * AccountBudgetSummary customer.
                                 * @member {string|null|undefined} customer
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.customer = null;
    
                                /**
                                 * AccountBudgetSummary customer_descriptive_name.
                                 * @member {string|null|undefined} customer_descriptive_name
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.customer_descriptive_name = null;
    
                                /**
                                 * AccountBudgetSummary account_budget.
                                 * @member {string|null|undefined} account_budget
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.account_budget = null;
    
                                /**
                                 * AccountBudgetSummary account_budget_name.
                                 * @member {string|null|undefined} account_budget_name
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.account_budget_name = null;
    
                                /**
                                 * AccountBudgetSummary purchase_order_number.
                                 * @member {string|null|undefined} purchase_order_number
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.purchase_order_number = null;
    
                                /**
                                 * AccountBudgetSummary subtotal_amount_micros.
                                 * @member {number|null|undefined} subtotal_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.subtotal_amount_micros = null;
    
                                /**
                                 * AccountBudgetSummary tax_amount_micros.
                                 * @member {number|null|undefined} tax_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.tax_amount_micros = null;
    
                                /**
                                 * AccountBudgetSummary total_amount_micros.
                                 * @member {number|null|undefined} total_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.total_amount_micros = null;
    
                                /**
                                 * AccountBudgetSummary billable_activity_date_range.
                                 * @member {google.ads.googleads.v11.common.IDateRange|null|undefined} billable_activity_date_range
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                AccountBudgetSummary.prototype.billable_activity_date_range = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * AccountBudgetSummary _customer.
                                 * @member {"customer"|undefined} _customer
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_customer", {
                                    get: $util.oneOfGetter($oneOfFields = ["customer"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * AccountBudgetSummary _customer_descriptive_name.
                                 * @member {"customer_descriptive_name"|undefined} _customer_descriptive_name
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_customer_descriptive_name", {
                                    get: $util.oneOfGetter($oneOfFields = ["customer_descriptive_name"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * AccountBudgetSummary _account_budget.
                                 * @member {"account_budget"|undefined} _account_budget
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_account_budget", {
                                    get: $util.oneOfGetter($oneOfFields = ["account_budget"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * AccountBudgetSummary _account_budget_name.
                                 * @member {"account_budget_name"|undefined} _account_budget_name
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_account_budget_name", {
                                    get: $util.oneOfGetter($oneOfFields = ["account_budget_name"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * AccountBudgetSummary _purchase_order_number.
                                 * @member {"purchase_order_number"|undefined} _purchase_order_number
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_purchase_order_number", {
                                    get: $util.oneOfGetter($oneOfFields = ["purchase_order_number"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * AccountBudgetSummary _subtotal_amount_micros.
                                 * @member {"subtotal_amount_micros"|undefined} _subtotal_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_subtotal_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["subtotal_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * AccountBudgetSummary _tax_amount_micros.
                                 * @member {"tax_amount_micros"|undefined} _tax_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_tax_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["tax_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * AccountBudgetSummary _total_amount_micros.
                                 * @member {"total_amount_micros"|undefined} _total_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 */
                                Object.defineProperty(AccountBudgetSummary.prototype, "_total_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["total_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new AccountBudgetSummary instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Invoice.IAccountBudgetSummary=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary} AccountBudgetSummary instance
                                 */
                                AccountBudgetSummary.create = function create(properties) {
                                    return new AccountBudgetSummary(properties);
                                };
    
                                /**
                                 * Encodes the specified AccountBudgetSummary message. Does not implicitly {@link google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Invoice.IAccountBudgetSummary} m AccountBudgetSummary message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AccountBudgetSummary.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.billable_activity_date_range != null && Object.hasOwnProperty.call(m, "billable_activity_date_range"))
                                        $root.google.ads.googleads.v11.common.DateRange.encode(m.billable_activity_date_range, w.uint32(74).fork()).ldelim();
                                    if (m.customer != null && Object.hasOwnProperty.call(m, "customer"))
                                        w.uint32(82).string(m.customer);
                                    if (m.customer_descriptive_name != null && Object.hasOwnProperty.call(m, "customer_descriptive_name"))
                                        w.uint32(90).string(m.customer_descriptive_name);
                                    if (m.account_budget != null && Object.hasOwnProperty.call(m, "account_budget"))
                                        w.uint32(98).string(m.account_budget);
                                    if (m.account_budget_name != null && Object.hasOwnProperty.call(m, "account_budget_name"))
                                        w.uint32(106).string(m.account_budget_name);
                                    if (m.purchase_order_number != null && Object.hasOwnProperty.call(m, "purchase_order_number"))
                                        w.uint32(114).string(m.purchase_order_number);
                                    if (m.subtotal_amount_micros != null && Object.hasOwnProperty.call(m, "subtotal_amount_micros"))
                                        w.uint32(120).int64(m.subtotal_amount_micros);
                                    if (m.tax_amount_micros != null && Object.hasOwnProperty.call(m, "tax_amount_micros"))
                                        w.uint32(128).int64(m.tax_amount_micros);
                                    if (m.total_amount_micros != null && Object.hasOwnProperty.call(m, "total_amount_micros"))
                                        w.uint32(136).int64(m.total_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified AccountBudgetSummary message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Invoice.IAccountBudgetSummary} message AccountBudgetSummary message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AccountBudgetSummary.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an AccountBudgetSummary message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary} AccountBudgetSummary
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AccountBudgetSummary.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 10:
                                            m.customer = r.string();
                                            break;
                                        case 11:
                                            m.customer_descriptive_name = r.string();
                                            break;
                                        case 12:
                                            m.account_budget = r.string();
                                            break;
                                        case 13:
                                            m.account_budget_name = r.string();
                                            break;
                                        case 14:
                                            m.purchase_order_number = r.string();
                                            break;
                                        case 15:
                                            m.subtotal_amount_micros = r.int64();
                                            break;
                                        case 16:
                                            m.tax_amount_micros = r.int64();
                                            break;
                                        case 17:
                                            m.total_amount_micros = r.int64();
                                            break;
                                        case 9:
                                            m.billable_activity_date_range = $root.google.ads.googleads.v11.common.DateRange.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an AccountBudgetSummary message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary} AccountBudgetSummary
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AccountBudgetSummary.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an AccountBudgetSummary message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                AccountBudgetSummary.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.customer != null && m.hasOwnProperty("customer")) {
                                        p._customer = 1;
                                        if (!$util.isString(m.customer))
                                            return "customer: string expected";
                                    }
                                    if (m.customer_descriptive_name != null && m.hasOwnProperty("customer_descriptive_name")) {
                                        p._customer_descriptive_name = 1;
                                        if (!$util.isString(m.customer_descriptive_name))
                                            return "customer_descriptive_name: string expected";
                                    }
                                    if (m.account_budget != null && m.hasOwnProperty("account_budget")) {
                                        p._account_budget = 1;
                                        if (!$util.isString(m.account_budget))
                                            return "account_budget: string expected";
                                    }
                                    if (m.account_budget_name != null && m.hasOwnProperty("account_budget_name")) {
                                        p._account_budget_name = 1;
                                        if (!$util.isString(m.account_budget_name))
                                            return "account_budget_name: string expected";
                                    }
                                    if (m.purchase_order_number != null && m.hasOwnProperty("purchase_order_number")) {
                                        p._purchase_order_number = 1;
                                        if (!$util.isString(m.purchase_order_number))
                                            return "purchase_order_number: string expected";
                                    }
                                    if (m.subtotal_amount_micros != null && m.hasOwnProperty("subtotal_amount_micros")) {
                                        p._subtotal_amount_micros = 1;
                                        if (!$util.isInteger(m.subtotal_amount_micros) && !(m.subtotal_amount_micros && $util.isInteger(m.subtotal_amount_micros.low) && $util.isInteger(m.subtotal_amount_micros.high)))
                                            return "subtotal_amount_micros: integer|Long expected";
                                    }
                                    if (m.tax_amount_micros != null && m.hasOwnProperty("tax_amount_micros")) {
                                        p._tax_amount_micros = 1;
                                        if (!$util.isInteger(m.tax_amount_micros) && !(m.tax_amount_micros && $util.isInteger(m.tax_amount_micros.low) && $util.isInteger(m.tax_amount_micros.high)))
                                            return "tax_amount_micros: integer|Long expected";
                                    }
                                    if (m.total_amount_micros != null && m.hasOwnProperty("total_amount_micros")) {
                                        p._total_amount_micros = 1;
                                        if (!$util.isInteger(m.total_amount_micros) && !(m.total_amount_micros && $util.isInteger(m.total_amount_micros.low) && $util.isInteger(m.total_amount_micros.high)))
                                            return "total_amount_micros: integer|Long expected";
                                    }
                                    if (m.billable_activity_date_range != null && m.hasOwnProperty("billable_activity_date_range")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.DateRange.verify(m.billable_activity_date_range);
                                            if (e)
                                                return "billable_activity_date_range." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an AccountBudgetSummary message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary} AccountBudgetSummary
                                 */
                                AccountBudgetSummary.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary();
                                    if (d.customer != null) {
                                        m.customer = String(d.customer);
                                    }
                                    if (d.customer_descriptive_name != null) {
                                        m.customer_descriptive_name = String(d.customer_descriptive_name);
                                    }
                                    if (d.account_budget != null) {
                                        m.account_budget = String(d.account_budget);
                                    }
                                    if (d.account_budget_name != null) {
                                        m.account_budget_name = String(d.account_budget_name);
                                    }
                                    if (d.purchase_order_number != null) {
                                        m.purchase_order_number = String(d.purchase_order_number);
                                    }
                                    if (d.subtotal_amount_micros != null) {
                                        if ($util.Long)
                                            (m.subtotal_amount_micros = $util.Long.fromValue(d.subtotal_amount_micros)).unsigned = false;
                                        else if (typeof d.subtotal_amount_micros === "string")
                                            m.subtotal_amount_micros = parseInt(d.subtotal_amount_micros, 10);
                                        else if (typeof d.subtotal_amount_micros === "number")
                                            m.subtotal_amount_micros = d.subtotal_amount_micros;
                                        else if (typeof d.subtotal_amount_micros === "object")
                                            m.subtotal_amount_micros = new $util.LongBits(d.subtotal_amount_micros.low >>> 0, d.subtotal_amount_micros.high >>> 0).toNumber();
                                    }
                                    if (d.tax_amount_micros != null) {
                                        if ($util.Long)
                                            (m.tax_amount_micros = $util.Long.fromValue(d.tax_amount_micros)).unsigned = false;
                                        else if (typeof d.tax_amount_micros === "string")
                                            m.tax_amount_micros = parseInt(d.tax_amount_micros, 10);
                                        else if (typeof d.tax_amount_micros === "number")
                                            m.tax_amount_micros = d.tax_amount_micros;
                                        else if (typeof d.tax_amount_micros === "object")
                                            m.tax_amount_micros = new $util.LongBits(d.tax_amount_micros.low >>> 0, d.tax_amount_micros.high >>> 0).toNumber();
                                    }
                                    if (d.total_amount_micros != null) {
                                        if ($util.Long)
                                            (m.total_amount_micros = $util.Long.fromValue(d.total_amount_micros)).unsigned = false;
                                        else if (typeof d.total_amount_micros === "string")
                                            m.total_amount_micros = parseInt(d.total_amount_micros, 10);
                                        else if (typeof d.total_amount_micros === "number")
                                            m.total_amount_micros = d.total_amount_micros;
                                        else if (typeof d.total_amount_micros === "object")
                                            m.total_amount_micros = new $util.LongBits(d.total_amount_micros.low >>> 0, d.total_amount_micros.high >>> 0).toNumber();
                                    }
                                    if (d.billable_activity_date_range != null) {
                                        if (typeof d.billable_activity_date_range !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary.billable_activity_date_range: object expected");
                                        m.billable_activity_date_range = $root.google.ads.googleads.v11.common.DateRange.fromObject(d.billable_activity_date_range);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an AccountBudgetSummary message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary} m AccountBudgetSummary
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                AccountBudgetSummary.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.billable_activity_date_range = null;
                                    }
                                    if (m.billable_activity_date_range != null && m.hasOwnProperty("billable_activity_date_range")) {
                                        d.billable_activity_date_range = $root.google.ads.googleads.v11.common.DateRange.toObject(m.billable_activity_date_range, o);
                                    }
                                    if (m.customer != null && m.hasOwnProperty("customer")) {
                                        d.customer = m.customer;
                                        if (o.oneofs)
                                            d._customer = "customer";
                                    }
                                    if (m.customer_descriptive_name != null && m.hasOwnProperty("customer_descriptive_name")) {
                                        d.customer_descriptive_name = m.customer_descriptive_name;
                                        if (o.oneofs)
                                            d._customer_descriptive_name = "customer_descriptive_name";
                                    }
                                    if (m.account_budget != null && m.hasOwnProperty("account_budget")) {
                                        d.account_budget = m.account_budget;
                                        if (o.oneofs)
                                            d._account_budget = "account_budget";
                                    }
                                    if (m.account_budget_name != null && m.hasOwnProperty("account_budget_name")) {
                                        d.account_budget_name = m.account_budget_name;
                                        if (o.oneofs)
                                            d._account_budget_name = "account_budget_name";
                                    }
                                    if (m.purchase_order_number != null && m.hasOwnProperty("purchase_order_number")) {
                                        d.purchase_order_number = m.purchase_order_number;
                                        if (o.oneofs)
                                            d._purchase_order_number = "purchase_order_number";
                                    }
                                    if (m.subtotal_amount_micros != null && m.hasOwnProperty("subtotal_amount_micros")) {
                                        if (typeof m.subtotal_amount_micros === "number")
                                            d.subtotal_amount_micros = o.longs === String ? String(m.subtotal_amount_micros) : m.subtotal_amount_micros;
                                        else
                                            d.subtotal_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.subtotal_amount_micros) : o.longs === Number ? new $util.LongBits(m.subtotal_amount_micros.low >>> 0, m.subtotal_amount_micros.high >>> 0).toNumber() : m.subtotal_amount_micros;
                                        if (o.oneofs)
                                            d._subtotal_amount_micros = "subtotal_amount_micros";
                                    }
                                    if (m.tax_amount_micros != null && m.hasOwnProperty("tax_amount_micros")) {
                                        if (typeof m.tax_amount_micros === "number")
                                            d.tax_amount_micros = o.longs === String ? String(m.tax_amount_micros) : m.tax_amount_micros;
                                        else
                                            d.tax_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.tax_amount_micros) : o.longs === Number ? new $util.LongBits(m.tax_amount_micros.low >>> 0, m.tax_amount_micros.high >>> 0).toNumber() : m.tax_amount_micros;
                                        if (o.oneofs)
                                            d._tax_amount_micros = "tax_amount_micros";
                                    }
                                    if (m.total_amount_micros != null && m.hasOwnProperty("total_amount_micros")) {
                                        if (typeof m.total_amount_micros === "number")
                                            d.total_amount_micros = o.longs === String ? String(m.total_amount_micros) : m.total_amount_micros;
                                        else
                                            d.total_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.total_amount_micros) : o.longs === Number ? new $util.LongBits(m.total_amount_micros.low >>> 0, m.total_amount_micros.high >>> 0).toNumber() : m.total_amount_micros;
                                        if (o.oneofs)
                                            d._total_amount_micros = "total_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this AccountBudgetSummary to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Invoice.AccountBudgetSummary
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                AccountBudgetSummary.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return AccountBudgetSummary;
                            })();
    
                            return Invoice;
                        })();
    
                        resources.KeywordPlan = (function() {
    
                            /**
                             * Properties of a KeywordPlan.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordPlan
                             * @property {string|null} [resource_name] KeywordPlan resource_name
                             * @property {number|null} [id] KeywordPlan id
                             * @property {string|null} [name] KeywordPlan name
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanForecastPeriod|null} [forecast_period] KeywordPlan forecast_period
                             */
    
                            /**
                             * Constructs a new KeywordPlan.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordPlan.
                             * @implements IKeywordPlan
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordPlan=} [p] Properties to set
                             */
                            function KeywordPlan(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlan resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @instance
                             */
                            KeywordPlan.prototype.resource_name = "";
    
                            /**
                             * KeywordPlan id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @instance
                             */
                            KeywordPlan.prototype.id = null;
    
                            /**
                             * KeywordPlan name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @instance
                             */
                            KeywordPlan.prototype.name = null;
    
                            /**
                             * KeywordPlan forecast_period.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanForecastPeriod|null|undefined} forecast_period
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @instance
                             */
                            KeywordPlan.prototype.forecast_period = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlan _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @instance
                             */
                            Object.defineProperty(KeywordPlan.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlan _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @instance
                             */
                            Object.defineProperty(KeywordPlan.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlan instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlan=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordPlan} KeywordPlan instance
                             */
                            KeywordPlan.create = function create(properties) {
                                return new KeywordPlan(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlan message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlan.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlan} m KeywordPlan message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlan.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.forecast_period != null && Object.hasOwnProperty.call(m, "forecast_period"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.encode(m.forecast_period, w.uint32(34).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(40).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(50).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlan message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlan.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlan} message KeywordPlan message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlan.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlan message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordPlan} KeywordPlan
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlan.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordPlan();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 5:
                                        m.id = r.int64();
                                        break;
                                    case 6:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.forecast_period = $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlan message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordPlan} KeywordPlan
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlan.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlan message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlan.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.forecast_period != null && m.hasOwnProperty("forecast_period")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.verify(m.forecast_period);
                                        if (e)
                                            return "forecast_period." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlan message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordPlan} KeywordPlan
                             */
                            KeywordPlan.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordPlan)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordPlan();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.forecast_period != null) {
                                    if (typeof d.forecast_period !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.KeywordPlan.forecast_period: object expected");
                                    m.forecast_period = $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.fromObject(d.forecast_period);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlan message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordPlan} m KeywordPlan
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlan.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.forecast_period = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.forecast_period != null && m.hasOwnProperty("forecast_period")) {
                                    d.forecast_period = $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.toObject(m.forecast_period, o);
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlan to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordPlan
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlan.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlan;
                        })();
    
                        resources.KeywordPlanForecastPeriod = (function() {
    
                            /**
                             * Properties of a KeywordPlanForecastPeriod.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordPlanForecastPeriod
                             * @property {google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval|null} [date_interval] KeywordPlanForecastPeriod date_interval
                             * @property {google.ads.googleads.v11.common.IDateRange|null} [date_range] KeywordPlanForecastPeriod date_range
                             */
    
                            /**
                             * Constructs a new KeywordPlanForecastPeriod.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordPlanForecastPeriod.
                             * @implements IKeywordPlanForecastPeriod
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanForecastPeriod=} [p] Properties to set
                             */
                            function KeywordPlanForecastPeriod(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanForecastPeriod date_interval.
                             * @member {google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval|null|undefined} date_interval
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @instance
                             */
                            KeywordPlanForecastPeriod.prototype.date_interval = null;
    
                            /**
                             * KeywordPlanForecastPeriod date_range.
                             * @member {google.ads.googleads.v11.common.IDateRange|null|undefined} date_range
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @instance
                             */
                            KeywordPlanForecastPeriod.prototype.date_range = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanForecastPeriod interval.
                             * @member {"date_interval"|"date_range"|undefined} interval
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanForecastPeriod.prototype, "interval", {
                                get: $util.oneOfGetter($oneOfFields = ["date_interval", "date_range"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanForecastPeriod instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanForecastPeriod=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanForecastPeriod} KeywordPlanForecastPeriod instance
                             */
                            KeywordPlanForecastPeriod.create = function create(properties) {
                                return new KeywordPlanForecastPeriod(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanForecastPeriod message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanForecastPeriod} m KeywordPlanForecastPeriod message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanForecastPeriod.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.date_interval != null && Object.hasOwnProperty.call(m, "date_interval"))
                                    w.uint32(8).int32(m.date_interval);
                                if (m.date_range != null && Object.hasOwnProperty.call(m, "date_range"))
                                    $root.google.ads.googleads.v11.common.DateRange.encode(m.date_range, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanForecastPeriod message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanForecastPeriod} message KeywordPlanForecastPeriod message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanForecastPeriod.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanForecastPeriod message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanForecastPeriod} KeywordPlanForecastPeriod
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanForecastPeriod.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.date_interval = r.int32();
                                        break;
                                    case 2:
                                        m.date_range = $root.google.ads.googleads.v11.common.DateRange.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanForecastPeriod message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanForecastPeriod} KeywordPlanForecastPeriod
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanForecastPeriod.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanForecastPeriod message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanForecastPeriod.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.date_interval != null && m.hasOwnProperty("date_interval")) {
                                    p.interval = 1;
                                    switch (m.date_interval) {
                                    default:
                                        return "date_interval: enum value expected";
                                    case 0:
                                    case 1:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.date_range != null && m.hasOwnProperty("date_range")) {
                                    if (p.interval === 1)
                                        return "interval: multiple values";
                                    p.interval = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DateRange.verify(m.date_range);
                                        if (e)
                                            return "date_range." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanForecastPeriod message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanForecastPeriod} KeywordPlanForecastPeriod
                             */
                            KeywordPlanForecastPeriod.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordPlanForecastPeriod();
                                switch (d.date_interval) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.date_interval = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.date_interval = 1;
                                    break;
                                case "NEXT_WEEK":
                                case 3:
                                    m.date_interval = 3;
                                    break;
                                case "NEXT_MONTH":
                                case 4:
                                    m.date_interval = 4;
                                    break;
                                case "NEXT_QUARTER":
                                case 5:
                                    m.date_interval = 5;
                                    break;
                                }
                                if (d.date_range != null) {
                                    if (typeof d.date_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.KeywordPlanForecastPeriod.date_range: object expected");
                                    m.date_range = $root.google.ads.googleads.v11.common.DateRange.fromObject(d.date_range);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanForecastPeriod message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordPlanForecastPeriod} m KeywordPlanForecastPeriod
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanForecastPeriod.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.date_interval != null && m.hasOwnProperty("date_interval")) {
                                    d.date_interval = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval[m.date_interval] : m.date_interval;
                                    if (o.oneofs)
                                        d.interval = "date_interval";
                                }
                                if (m.date_range != null && m.hasOwnProperty("date_range")) {
                                    d.date_range = $root.google.ads.googleads.v11.common.DateRange.toObject(m.date_range, o);
                                    if (o.oneofs)
                                        d.interval = "date_range";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanForecastPeriod to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanForecastPeriod
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanForecastPeriod.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanForecastPeriod;
                        })();
    
                        resources.KeywordPlanAdGroup = (function() {
    
                            /**
                             * Properties of a KeywordPlanAdGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordPlanAdGroup
                             * @property {string|null} [resource_name] KeywordPlanAdGroup resource_name
                             * @property {string|null} [keyword_plan_campaign] KeywordPlanAdGroup keyword_plan_campaign
                             * @property {number|null} [id] KeywordPlanAdGroup id
                             * @property {string|null} [name] KeywordPlanAdGroup name
                             * @property {number|null} [cpc_bid_micros] KeywordPlanAdGroup cpc_bid_micros
                             */
    
                            /**
                             * Constructs a new KeywordPlanAdGroup.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordPlanAdGroup.
                             * @implements IKeywordPlanAdGroup
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroup=} [p] Properties to set
                             */
                            function KeywordPlanAdGroup(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanAdGroup resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            KeywordPlanAdGroup.prototype.resource_name = "";
    
                            /**
                             * KeywordPlanAdGroup keyword_plan_campaign.
                             * @member {string|null|undefined} keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            KeywordPlanAdGroup.prototype.keyword_plan_campaign = null;
    
                            /**
                             * KeywordPlanAdGroup id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            KeywordPlanAdGroup.prototype.id = null;
    
                            /**
                             * KeywordPlanAdGroup name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            KeywordPlanAdGroup.prototype.name = null;
    
                            /**
                             * KeywordPlanAdGroup cpc_bid_micros.
                             * @member {number|null|undefined} cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            KeywordPlanAdGroup.prototype.cpc_bid_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanAdGroup _keyword_plan_campaign.
                             * @member {"keyword_plan_campaign"|undefined} _keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroup.prototype, "_keyword_plan_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanAdGroup _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroup.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanAdGroup _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroup.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanAdGroup _cpc_bid_micros.
                             * @member {"cpc_bid_micros"|undefined} _cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroup.prototype, "_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanAdGroup instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroup=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroup} KeywordPlanAdGroup instance
                             */
                            KeywordPlanAdGroup.create = function create(properties) {
                                return new KeywordPlanAdGroup(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroup message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanAdGroup.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroup} m KeywordPlanAdGroup message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroup.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.keyword_plan_campaign != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign"))
                                    w.uint32(50).string(m.keyword_plan_campaign);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(56).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(66).string(m.name);
                                if (m.cpc_bid_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_micros"))
                                    w.uint32(72).int64(m.cpc_bid_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroup message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanAdGroup.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroup} message KeywordPlanAdGroup message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroup.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroup message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroup} KeywordPlanAdGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroup.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.keyword_plan_campaign = r.string();
                                        break;
                                    case 7:
                                        m.id = r.int64();
                                        break;
                                    case 8:
                                        m.name = r.string();
                                        break;
                                    case 9:
                                        m.cpc_bid_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroup message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroup} KeywordPlanAdGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroup.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAdGroup message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAdGroup.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    p._keyword_plan_campaign = 1;
                                    if (!$util.isString(m.keyword_plan_campaign))
                                        return "keyword_plan_campaign: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    p._cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_micros) && !(m.cpc_bid_micros && $util.isInteger(m.cpc_bid_micros.low) && $util.isInteger(m.cpc_bid_micros.high)))
                                        return "cpc_bid_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAdGroup message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroup} KeywordPlanAdGroup
                             */
                            KeywordPlanAdGroup.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.keyword_plan_campaign != null) {
                                    m.keyword_plan_campaign = String(d.keyword_plan_campaign);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_micros = $util.Long.fromValue(d.cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_micros === "string")
                                        m.cpc_bid_micros = parseInt(d.cpc_bid_micros, 10);
                                    else if (typeof d.cpc_bid_micros === "number")
                                        m.cpc_bid_micros = d.cpc_bid_micros;
                                    else if (typeof d.cpc_bid_micros === "object")
                                        m.cpc_bid_micros = new $util.LongBits(d.cpc_bid_micros.low >>> 0, d.cpc_bid_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAdGroup message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordPlanAdGroup} m KeywordPlanAdGroup
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAdGroup.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    d.keyword_plan_campaign = m.keyword_plan_campaign;
                                    if (o.oneofs)
                                        d._keyword_plan_campaign = "keyword_plan_campaign";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    if (typeof m.cpc_bid_micros === "number")
                                        d.cpc_bid_micros = o.longs === String ? String(m.cpc_bid_micros) : m.cpc_bid_micros;
                                    else
                                        d.cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_micros.low >>> 0, m.cpc_bid_micros.high >>> 0).toNumber() : m.cpc_bid_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_micros = "cpc_bid_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanAdGroup to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroup
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAdGroup.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanAdGroup;
                        })();
    
                        resources.KeywordPlanAdGroupKeyword = (function() {
    
                            /**
                             * Properties of a KeywordPlanAdGroupKeyword.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordPlanAdGroupKeyword
                             * @property {string|null} [resource_name] KeywordPlanAdGroupKeyword resource_name
                             * @property {string|null} [keyword_plan_ad_group] KeywordPlanAdGroupKeyword keyword_plan_ad_group
                             * @property {number|null} [id] KeywordPlanAdGroupKeyword id
                             * @property {string|null} [text] KeywordPlanAdGroupKeyword text
                             * @property {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType|null} [match_type] KeywordPlanAdGroupKeyword match_type
                             * @property {number|null} [cpc_bid_micros] KeywordPlanAdGroupKeyword cpc_bid_micros
                             * @property {boolean|null} [negative] KeywordPlanAdGroupKeyword negative
                             */
    
                            /**
                             * Constructs a new KeywordPlanAdGroupKeyword.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordPlanAdGroupKeyword.
                             * @implements IKeywordPlanAdGroupKeyword
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword=} [p] Properties to set
                             */
                            function KeywordPlanAdGroupKeyword(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanAdGroupKeyword resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            KeywordPlanAdGroupKeyword.prototype.resource_name = "";
    
                            /**
                             * KeywordPlanAdGroupKeyword keyword_plan_ad_group.
                             * @member {string|null|undefined} keyword_plan_ad_group
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            KeywordPlanAdGroupKeyword.prototype.keyword_plan_ad_group = null;
    
                            /**
                             * KeywordPlanAdGroupKeyword id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            KeywordPlanAdGroupKeyword.prototype.id = null;
    
                            /**
                             * KeywordPlanAdGroupKeyword text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            KeywordPlanAdGroupKeyword.prototype.text = null;
    
                            /**
                             * KeywordPlanAdGroupKeyword match_type.
                             * @member {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType} match_type
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            KeywordPlanAdGroupKeyword.prototype.match_type = 0;
    
                            /**
                             * KeywordPlanAdGroupKeyword cpc_bid_micros.
                             * @member {number|null|undefined} cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            KeywordPlanAdGroupKeyword.prototype.cpc_bid_micros = null;
    
                            /**
                             * KeywordPlanAdGroupKeyword negative.
                             * @member {boolean|null|undefined} negative
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            KeywordPlanAdGroupKeyword.prototype.negative = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanAdGroupKeyword _keyword_plan_ad_group.
                             * @member {"keyword_plan_ad_group"|undefined} _keyword_plan_ad_group
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupKeyword.prototype, "_keyword_plan_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanAdGroupKeyword _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupKeyword.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanAdGroupKeyword _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupKeyword.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanAdGroupKeyword _cpc_bid_micros.
                             * @member {"cpc_bid_micros"|undefined} _cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupKeyword.prototype, "_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanAdGroupKeyword _negative.
                             * @member {"negative"|undefined} _negative
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupKeyword.prototype, "_negative", {
                                get: $util.oneOfGetter($oneOfFields = ["negative"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanAdGroupKeyword instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword} KeywordPlanAdGroupKeyword instance
                             */
                            KeywordPlanAdGroupKeyword.create = function create(properties) {
                                return new KeywordPlanAdGroupKeyword(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupKeyword message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword} m KeywordPlanAdGroupKeyword message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupKeyword.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.match_type != null && Object.hasOwnProperty.call(m, "match_type"))
                                    w.uint32(40).int32(m.match_type);
                                if (m.keyword_plan_ad_group != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group"))
                                    w.uint32(66).string(m.keyword_plan_ad_group);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(72).int64(m.id);
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(82).string(m.text);
                                if (m.cpc_bid_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_micros"))
                                    w.uint32(88).int64(m.cpc_bid_micros);
                                if (m.negative != null && Object.hasOwnProperty.call(m, "negative"))
                                    w.uint32(96).bool(m.negative);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupKeyword message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword} message KeywordPlanAdGroupKeyword message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupKeyword.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupKeyword message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword} KeywordPlanAdGroupKeyword
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupKeyword.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 8:
                                        m.keyword_plan_ad_group = r.string();
                                        break;
                                    case 9:
                                        m.id = r.int64();
                                        break;
                                    case 10:
                                        m.text = r.string();
                                        break;
                                    case 5:
                                        m.match_type = r.int32();
                                        break;
                                    case 11:
                                        m.cpc_bid_micros = r.int64();
                                        break;
                                    case 12:
                                        m.negative = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupKeyword message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword} KeywordPlanAdGroupKeyword
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupKeyword.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAdGroupKeyword message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAdGroupKeyword.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.keyword_plan_ad_group != null && m.hasOwnProperty("keyword_plan_ad_group")) {
                                    p._keyword_plan_ad_group = 1;
                                    if (!$util.isString(m.keyword_plan_ad_group))
                                        return "keyword_plan_ad_group: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                    switch (m.match_type) {
                                    default:
                                        return "match_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    p._cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_micros) && !(m.cpc_bid_micros && $util.isInteger(m.cpc_bid_micros.low) && $util.isInteger(m.cpc_bid_micros.high)))
                                        return "cpc_bid_micros: integer|Long expected";
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    p._negative = 1;
                                    if (typeof m.negative !== "boolean")
                                        return "negative: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAdGroupKeyword message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword} KeywordPlanAdGroupKeyword
                             */
                            KeywordPlanAdGroupKeyword.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.keyword_plan_ad_group != null) {
                                    m.keyword_plan_ad_group = String(d.keyword_plan_ad_group);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                switch (d.match_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.match_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.match_type = 1;
                                    break;
                                case "EXACT":
                                case 2:
                                    m.match_type = 2;
                                    break;
                                case "PHRASE":
                                case 3:
                                    m.match_type = 3;
                                    break;
                                case "BROAD":
                                case 4:
                                    m.match_type = 4;
                                    break;
                                }
                                if (d.cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_micros = $util.Long.fromValue(d.cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_micros === "string")
                                        m.cpc_bid_micros = parseInt(d.cpc_bid_micros, 10);
                                    else if (typeof d.cpc_bid_micros === "number")
                                        m.cpc_bid_micros = d.cpc_bid_micros;
                                    else if (typeof d.cpc_bid_micros === "object")
                                        m.cpc_bid_micros = new $util.LongBits(d.cpc_bid_micros.low >>> 0, d.cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.negative != null) {
                                    m.negative = Boolean(d.negative);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAdGroupKeyword message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword} m KeywordPlanAdGroupKeyword
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAdGroupKeyword.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                    d.match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType[m.match_type] : m.match_type;
                                }
                                if (m.keyword_plan_ad_group != null && m.hasOwnProperty("keyword_plan_ad_group")) {
                                    d.keyword_plan_ad_group = m.keyword_plan_ad_group;
                                    if (o.oneofs)
                                        d._keyword_plan_ad_group = "keyword_plan_ad_group";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    if (typeof m.cpc_bid_micros === "number")
                                        d.cpc_bid_micros = o.longs === String ? String(m.cpc_bid_micros) : m.cpc_bid_micros;
                                    else
                                        d.cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_micros.low >>> 0, m.cpc_bid_micros.high >>> 0).toNumber() : m.cpc_bid_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_micros = "cpc_bid_micros";
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    d.negative = m.negative;
                                    if (o.oneofs)
                                        d._negative = "negative";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanAdGroupKeyword to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAdGroupKeyword.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanAdGroupKeyword;
                        })();
    
                        resources.KeywordPlanCampaign = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaign.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordPlanCampaign
                             * @property {string|null} [resource_name] KeywordPlanCampaign resource_name
                             * @property {string|null} [keyword_plan] KeywordPlanCampaign keyword_plan
                             * @property {number|null} [id] KeywordPlanCampaign id
                             * @property {string|null} [name] KeywordPlanCampaign name
                             * @property {Array.<string>|null} [language_constants] KeywordPlanCampaign language_constants
                             * @property {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork|null} [keyword_plan_network] KeywordPlanCampaign keyword_plan_network
                             * @property {number|null} [cpc_bid_micros] KeywordPlanCampaign cpc_bid_micros
                             * @property {Array.<google.ads.googleads.v11.resources.IKeywordPlanGeoTarget>|null} [geo_targets] KeywordPlanCampaign geo_targets
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaign.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordPlanCampaign.
                             * @implements IKeywordPlanCampaign
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaign=} [p] Properties to set
                             */
                            function KeywordPlanCampaign(p) {
                                this.language_constants = [];
                                this.geo_targets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanCampaign resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.resource_name = "";
    
                            /**
                             * KeywordPlanCampaign keyword_plan.
                             * @member {string|null|undefined} keyword_plan
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.keyword_plan = null;
    
                            /**
                             * KeywordPlanCampaign id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.id = null;
    
                            /**
                             * KeywordPlanCampaign name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.name = null;
    
                            /**
                             * KeywordPlanCampaign language_constants.
                             * @member {Array.<string>} language_constants
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.language_constants = $util.emptyArray;
    
                            /**
                             * KeywordPlanCampaign keyword_plan_network.
                             * @member {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork} keyword_plan_network
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.keyword_plan_network = 0;
    
                            /**
                             * KeywordPlanCampaign cpc_bid_micros.
                             * @member {number|null|undefined} cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.cpc_bid_micros = null;
    
                            /**
                             * KeywordPlanCampaign geo_targets.
                             * @member {Array.<google.ads.googleads.v11.resources.IKeywordPlanGeoTarget>} geo_targets
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            KeywordPlanCampaign.prototype.geo_targets = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanCampaign _keyword_plan.
                             * @member {"keyword_plan"|undefined} _keyword_plan
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaign.prototype, "_keyword_plan", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanCampaign _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaign.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanCampaign _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaign.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanCampaign _cpc_bid_micros.
                             * @member {"cpc_bid_micros"|undefined} _cpc_bid_micros
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaign.prototype, "_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanCampaign instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaign=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaign} KeywordPlanCampaign instance
                             */
                            KeywordPlanCampaign.create = function create(properties) {
                                return new KeywordPlanCampaign(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaign message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanCampaign.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaign} m KeywordPlanCampaign message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaign.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.keyword_plan_network != null && Object.hasOwnProperty.call(m, "keyword_plan_network"))
                                    w.uint32(48).int32(m.keyword_plan_network);
                                if (m.geo_targets != null && m.geo_targets.length) {
                                    for (var i = 0; i < m.geo_targets.length; ++i)
                                        $root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget.encode(m.geo_targets[i], w.uint32(66).fork()).ldelim();
                                }
                                if (m.keyword_plan != null && Object.hasOwnProperty.call(m, "keyword_plan"))
                                    w.uint32(74).string(m.keyword_plan);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(80).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(90).string(m.name);
                                if (m.language_constants != null && m.language_constants.length) {
                                    for (var i = 0; i < m.language_constants.length; ++i)
                                        w.uint32(98).string(m.language_constants[i]);
                                }
                                if (m.cpc_bid_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_micros"))
                                    w.uint32(104).int64(m.cpc_bid_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaign message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanCampaign.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaign} message KeywordPlanCampaign message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaign.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaign message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaign} KeywordPlanCampaign
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaign.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordPlanCampaign();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.keyword_plan = r.string();
                                        break;
                                    case 10:
                                        m.id = r.int64();
                                        break;
                                    case 11:
                                        m.name = r.string();
                                        break;
                                    case 12:
                                        if (!(m.language_constants && m.language_constants.length))
                                            m.language_constants = [];
                                        m.language_constants.push(r.string());
                                        break;
                                    case 6:
                                        m.keyword_plan_network = r.int32();
                                        break;
                                    case 13:
                                        m.cpc_bid_micros = r.int64();
                                        break;
                                    case 8:
                                        if (!(m.geo_targets && m.geo_targets.length))
                                            m.geo_targets = [];
                                        m.geo_targets.push($root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaign message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaign} KeywordPlanCampaign
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaign.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaign message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaign.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    p._keyword_plan = 1;
                                    if (!$util.isString(m.keyword_plan))
                                        return "keyword_plan: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.language_constants != null && m.hasOwnProperty("language_constants")) {
                                    if (!Array.isArray(m.language_constants))
                                        return "language_constants: array expected";
                                    for (var i = 0; i < m.language_constants.length; ++i) {
                                        if (!$util.isString(m.language_constants[i]))
                                            return "language_constants: string[] expected";
                                    }
                                }
                                if (m.keyword_plan_network != null && m.hasOwnProperty("keyword_plan_network")) {
                                    switch (m.keyword_plan_network) {
                                    default:
                                        return "keyword_plan_network: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    p._cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.cpc_bid_micros) && !(m.cpc_bid_micros && $util.isInteger(m.cpc_bid_micros.low) && $util.isInteger(m.cpc_bid_micros.high)))
                                        return "cpc_bid_micros: integer|Long expected";
                                }
                                if (m.geo_targets != null && m.hasOwnProperty("geo_targets")) {
                                    if (!Array.isArray(m.geo_targets))
                                        return "geo_targets: array expected";
                                    for (var i = 0; i < m.geo_targets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget.verify(m.geo_targets[i]);
                                            if (e)
                                                return "geo_targets." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaign message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaign} KeywordPlanCampaign
                             */
                            KeywordPlanCampaign.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordPlanCampaign)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordPlanCampaign();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.keyword_plan != null) {
                                    m.keyword_plan = String(d.keyword_plan);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.language_constants) {
                                    if (!Array.isArray(d.language_constants))
                                        throw TypeError(".google.ads.googleads.v11.resources.KeywordPlanCampaign.language_constants: array expected");
                                    m.language_constants = [];
                                    for (var i = 0; i < d.language_constants.length; ++i) {
                                        m.language_constants[i] = String(d.language_constants[i]);
                                    }
                                }
                                switch (d.keyword_plan_network) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_network = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_network = 1;
                                    break;
                                case "GOOGLE_SEARCH":
                                case 2:
                                    m.keyword_plan_network = 2;
                                    break;
                                case "GOOGLE_SEARCH_AND_PARTNERS":
                                case 3:
                                    m.keyword_plan_network = 3;
                                    break;
                                }
                                if (d.cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.cpc_bid_micros = $util.Long.fromValue(d.cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.cpc_bid_micros === "string")
                                        m.cpc_bid_micros = parseInt(d.cpc_bid_micros, 10);
                                    else if (typeof d.cpc_bid_micros === "number")
                                        m.cpc_bid_micros = d.cpc_bid_micros;
                                    else if (typeof d.cpc_bid_micros === "object")
                                        m.cpc_bid_micros = new $util.LongBits(d.cpc_bid_micros.low >>> 0, d.cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.geo_targets) {
                                    if (!Array.isArray(d.geo_targets))
                                        throw TypeError(".google.ads.googleads.v11.resources.KeywordPlanCampaign.geo_targets: array expected");
                                    m.geo_targets = [];
                                    for (var i = 0; i < d.geo_targets.length; ++i) {
                                        if (typeof d.geo_targets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.KeywordPlanCampaign.geo_targets: object expected");
                                        m.geo_targets[i] = $root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget.fromObject(d.geo_targets[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaign message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordPlanCampaign} m KeywordPlanCampaign
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaign.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.geo_targets = [];
                                    d.language_constants = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.keyword_plan_network = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.keyword_plan_network != null && m.hasOwnProperty("keyword_plan_network")) {
                                    d.keyword_plan_network = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork[m.keyword_plan_network] : m.keyword_plan_network;
                                }
                                if (m.geo_targets && m.geo_targets.length) {
                                    d.geo_targets = [];
                                    for (var j = 0; j < m.geo_targets.length; ++j) {
                                        d.geo_targets[j] = $root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget.toObject(m.geo_targets[j], o);
                                    }
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    d.keyword_plan = m.keyword_plan;
                                    if (o.oneofs)
                                        d._keyword_plan = "keyword_plan";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.language_constants && m.language_constants.length) {
                                    d.language_constants = [];
                                    for (var j = 0; j < m.language_constants.length; ++j) {
                                        d.language_constants[j] = m.language_constants[j];
                                    }
                                }
                                if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                    if (typeof m.cpc_bid_micros === "number")
                                        d.cpc_bid_micros = o.longs === String ? String(m.cpc_bid_micros) : m.cpc_bid_micros;
                                    else
                                        d.cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_micros.low >>> 0, m.cpc_bid_micros.high >>> 0).toNumber() : m.cpc_bid_micros;
                                    if (o.oneofs)
                                        d._cpc_bid_micros = "cpc_bid_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanCampaign to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaign
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaign.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanCampaign;
                        })();
    
                        resources.KeywordPlanGeoTarget = (function() {
    
                            /**
                             * Properties of a KeywordPlanGeoTarget.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordPlanGeoTarget
                             * @property {string|null} [geo_target_constant] KeywordPlanGeoTarget geo_target_constant
                             */
    
                            /**
                             * Constructs a new KeywordPlanGeoTarget.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordPlanGeoTarget.
                             * @implements IKeywordPlanGeoTarget
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanGeoTarget=} [p] Properties to set
                             */
                            function KeywordPlanGeoTarget(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanGeoTarget geo_target_constant.
                             * @member {string|null|undefined} geo_target_constant
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @instance
                             */
                            KeywordPlanGeoTarget.prototype.geo_target_constant = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanGeoTarget _geo_target_constant.
                             * @member {"geo_target_constant"|undefined} _geo_target_constant
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanGeoTarget.prototype, "_geo_target_constant", {
                                get: $util.oneOfGetter($oneOfFields = ["geo_target_constant"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanGeoTarget instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanGeoTarget=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanGeoTarget} KeywordPlanGeoTarget instance
                             */
                            KeywordPlanGeoTarget.create = function create(properties) {
                                return new KeywordPlanGeoTarget(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanGeoTarget message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanGeoTarget.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanGeoTarget} m KeywordPlanGeoTarget message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanGeoTarget.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.geo_target_constant != null && Object.hasOwnProperty.call(m, "geo_target_constant"))
                                    w.uint32(18).string(m.geo_target_constant);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanGeoTarget message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanGeoTarget.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanGeoTarget} message KeywordPlanGeoTarget message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanGeoTarget.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanGeoTarget message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanGeoTarget} KeywordPlanGeoTarget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanGeoTarget.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.geo_target_constant = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanGeoTarget message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanGeoTarget} KeywordPlanGeoTarget
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanGeoTarget.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanGeoTarget message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanGeoTarget.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    p._geo_target_constant = 1;
                                    if (!$util.isString(m.geo_target_constant))
                                        return "geo_target_constant: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanGeoTarget message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanGeoTarget} KeywordPlanGeoTarget
                             */
                            KeywordPlanGeoTarget.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordPlanGeoTarget();
                                if (d.geo_target_constant != null) {
                                    m.geo_target_constant = String(d.geo_target_constant);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanGeoTarget message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordPlanGeoTarget} m KeywordPlanGeoTarget
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanGeoTarget.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    d.geo_target_constant = m.geo_target_constant;
                                    if (o.oneofs)
                                        d._geo_target_constant = "geo_target_constant";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanGeoTarget to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanGeoTarget
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanGeoTarget.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanGeoTarget;
                        })();
    
                        resources.KeywordPlanCampaignKeyword = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaignKeyword.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordPlanCampaignKeyword
                             * @property {string|null} [resource_name] KeywordPlanCampaignKeyword resource_name
                             * @property {string|null} [keyword_plan_campaign] KeywordPlanCampaignKeyword keyword_plan_campaign
                             * @property {number|null} [id] KeywordPlanCampaignKeyword id
                             * @property {string|null} [text] KeywordPlanCampaignKeyword text
                             * @property {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType|null} [match_type] KeywordPlanCampaignKeyword match_type
                             * @property {boolean|null} [negative] KeywordPlanCampaignKeyword negative
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaignKeyword.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordPlanCampaignKeyword.
                             * @implements IKeywordPlanCampaignKeyword
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword=} [p] Properties to set
                             */
                            function KeywordPlanCampaignKeyword(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanCampaignKeyword resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            KeywordPlanCampaignKeyword.prototype.resource_name = "";
    
                            /**
                             * KeywordPlanCampaignKeyword keyword_plan_campaign.
                             * @member {string|null|undefined} keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            KeywordPlanCampaignKeyword.prototype.keyword_plan_campaign = null;
    
                            /**
                             * KeywordPlanCampaignKeyword id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            KeywordPlanCampaignKeyword.prototype.id = null;
    
                            /**
                             * KeywordPlanCampaignKeyword text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            KeywordPlanCampaignKeyword.prototype.text = null;
    
                            /**
                             * KeywordPlanCampaignKeyword match_type.
                             * @member {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType} match_type
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            KeywordPlanCampaignKeyword.prototype.match_type = 0;
    
                            /**
                             * KeywordPlanCampaignKeyword negative.
                             * @member {boolean|null|undefined} negative
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            KeywordPlanCampaignKeyword.prototype.negative = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanCampaignKeyword _keyword_plan_campaign.
                             * @member {"keyword_plan_campaign"|undefined} _keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignKeyword.prototype, "_keyword_plan_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanCampaignKeyword _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignKeyword.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanCampaignKeyword _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignKeyword.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordPlanCampaignKeyword _negative.
                             * @member {"negative"|undefined} _negative
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignKeyword.prototype, "_negative", {
                                get: $util.oneOfGetter($oneOfFields = ["negative"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanCampaignKeyword instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword} KeywordPlanCampaignKeyword instance
                             */
                            KeywordPlanCampaignKeyword.create = function create(properties) {
                                return new KeywordPlanCampaignKeyword(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignKeyword message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword} m KeywordPlanCampaignKeyword message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignKeyword.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.match_type != null && Object.hasOwnProperty.call(m, "match_type"))
                                    w.uint32(40).int32(m.match_type);
                                if (m.keyword_plan_campaign != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign"))
                                    w.uint32(66).string(m.keyword_plan_campaign);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(72).int64(m.id);
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(82).string(m.text);
                                if (m.negative != null && Object.hasOwnProperty.call(m, "negative"))
                                    w.uint32(88).bool(m.negative);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignKeyword message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword} message KeywordPlanCampaignKeyword message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignKeyword.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignKeyword message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword} KeywordPlanCampaignKeyword
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignKeyword.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 8:
                                        m.keyword_plan_campaign = r.string();
                                        break;
                                    case 9:
                                        m.id = r.int64();
                                        break;
                                    case 10:
                                        m.text = r.string();
                                        break;
                                    case 5:
                                        m.match_type = r.int32();
                                        break;
                                    case 11:
                                        m.negative = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignKeyword message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword} KeywordPlanCampaignKeyword
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignKeyword.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaignKeyword message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaignKeyword.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    p._keyword_plan_campaign = 1;
                                    if (!$util.isString(m.keyword_plan_campaign))
                                        return "keyword_plan_campaign: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                    switch (m.match_type) {
                                    default:
                                        return "match_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    p._negative = 1;
                                    if (typeof m.negative !== "boolean")
                                        return "negative: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaignKeyword message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword} KeywordPlanCampaignKeyword
                             */
                            KeywordPlanCampaignKeyword.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.keyword_plan_campaign != null) {
                                    m.keyword_plan_campaign = String(d.keyword_plan_campaign);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                switch (d.match_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.match_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.match_type = 1;
                                    break;
                                case "EXACT":
                                case 2:
                                    m.match_type = 2;
                                    break;
                                case "PHRASE":
                                case 3:
                                    m.match_type = 3;
                                    break;
                                case "BROAD":
                                case 4:
                                    m.match_type = 4;
                                    break;
                                }
                                if (d.negative != null) {
                                    m.negative = Boolean(d.negative);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaignKeyword message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword} m KeywordPlanCampaignKeyword
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaignKeyword.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                    d.match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType[m.match_type] : m.match_type;
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    d.keyword_plan_campaign = m.keyword_plan_campaign;
                                    if (o.oneofs)
                                        d._keyword_plan_campaign = "keyword_plan_campaign";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.negative != null && m.hasOwnProperty("negative")) {
                                    d.negative = m.negative;
                                    if (o.oneofs)
                                        d._negative = "negative";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanCampaignKeyword to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaignKeyword.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanCampaignKeyword;
                        })();
    
                        resources.KeywordThemeConstant = (function() {
    
                            /**
                             * Properties of a KeywordThemeConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordThemeConstant
                             * @property {string|null} [resource_name] KeywordThemeConstant resource_name
                             * @property {string|null} [country_code] KeywordThemeConstant country_code
                             * @property {string|null} [language_code] KeywordThemeConstant language_code
                             * @property {string|null} [display_name] KeywordThemeConstant display_name
                             */
    
                            /**
                             * Constructs a new KeywordThemeConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordThemeConstant.
                             * @implements IKeywordThemeConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordThemeConstant=} [p] Properties to set
                             */
                            function KeywordThemeConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordThemeConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             */
                            KeywordThemeConstant.prototype.resource_name = "";
    
                            /**
                             * KeywordThemeConstant country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             */
                            KeywordThemeConstant.prototype.country_code = null;
    
                            /**
                             * KeywordThemeConstant language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             */
                            KeywordThemeConstant.prototype.language_code = null;
    
                            /**
                             * KeywordThemeConstant display_name.
                             * @member {string|null|undefined} display_name
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             */
                            KeywordThemeConstant.prototype.display_name = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordThemeConstant _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             */
                            Object.defineProperty(KeywordThemeConstant.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordThemeConstant _language_code.
                             * @member {"language_code"|undefined} _language_code
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             */
                            Object.defineProperty(KeywordThemeConstant.prototype, "_language_code", {
                                get: $util.oneOfGetter($oneOfFields = ["language_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * KeywordThemeConstant _display_name.
                             * @member {"display_name"|undefined} _display_name
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             */
                            Object.defineProperty(KeywordThemeConstant.prototype, "_display_name", {
                                get: $util.oneOfGetter($oneOfFields = ["display_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordThemeConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordThemeConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordThemeConstant} KeywordThemeConstant instance
                             */
                            KeywordThemeConstant.create = function create(properties) {
                                return new KeywordThemeConstant(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordThemeConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordThemeConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordThemeConstant} m KeywordThemeConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordThemeConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(18).string(m.country_code);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(26).string(m.language_code);
                                if (m.display_name != null && Object.hasOwnProperty.call(m, "display_name"))
                                    w.uint32(34).string(m.display_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordThemeConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordThemeConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordThemeConstant} message KeywordThemeConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordThemeConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordThemeConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordThemeConstant} KeywordThemeConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordThemeConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordThemeConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.country_code = r.string();
                                        break;
                                    case 3:
                                        m.language_code = r.string();
                                        break;
                                    case 4:
                                        m.display_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordThemeConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordThemeConstant} KeywordThemeConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordThemeConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordThemeConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordThemeConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    p._language_code = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    p._display_name = 1;
                                    if (!$util.isString(m.display_name))
                                        return "display_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordThemeConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordThemeConstant} KeywordThemeConstant
                             */
                            KeywordThemeConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordThemeConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordThemeConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.display_name != null) {
                                    m.display_name = String(d.display_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordThemeConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordThemeConstant} m KeywordThemeConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordThemeConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d._language_code = "language_code";
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    d.display_name = m.display_name;
                                    if (o.oneofs)
                                        d._display_name = "display_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordThemeConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordThemeConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordThemeConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordThemeConstant;
                        })();
    
                        resources.KeywordView = (function() {
    
                            /**
                             * Properties of a KeywordView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IKeywordView
                             * @property {string|null} [resource_name] KeywordView resource_name
                             */
    
                            /**
                             * Constructs a new KeywordView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a KeywordView.
                             * @implements IKeywordView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IKeywordView=} [p] Properties to set
                             */
                            function KeywordView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @instance
                             */
                            KeywordView.prototype.resource_name = "";
    
                            /**
                             * Creates a new KeywordView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.KeywordView} KeywordView instance
                             */
                            KeywordView.create = function create(properties) {
                                return new KeywordView(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordView message. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordView} m KeywordView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.KeywordView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IKeywordView} message KeywordView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.KeywordView} KeywordView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.KeywordView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.KeywordView} KeywordView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.KeywordView} KeywordView
                             */
                            KeywordView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.KeywordView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.KeywordView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @static
                             * @param {google.ads.googleads.v11.resources.KeywordView} m KeywordView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.KeywordView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordView;
                        })();
    
                        resources.Label = (function() {
    
                            /**
                             * Properties of a Label.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ILabel
                             * @property {string|null} [resource_name] Label resource_name
                             * @property {number|null} [id] Label id
                             * @property {string|null} [name] Label name
                             * @property {google.ads.googleads.v11.enums.LabelStatusEnum.LabelStatus|null} [status] Label status
                             * @property {google.ads.googleads.v11.common.ITextLabel|null} [text_label] Label text_label
                             */
    
                            /**
                             * Constructs a new Label.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a Label.
                             * @implements ILabel
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ILabel=} [p] Properties to set
                             */
                            function Label(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Label resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             */
                            Label.prototype.resource_name = "";
    
                            /**
                             * Label id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             */
                            Label.prototype.id = null;
    
                            /**
                             * Label name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             */
                            Label.prototype.name = null;
    
                            /**
                             * Label status.
                             * @member {google.ads.googleads.v11.enums.LabelStatusEnum.LabelStatus} status
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             */
                            Label.prototype.status = 0;
    
                            /**
                             * Label text_label.
                             * @member {google.ads.googleads.v11.common.ITextLabel|null|undefined} text_label
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             */
                            Label.prototype.text_label = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Label _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             */
                            Object.defineProperty(Label.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Label _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             */
                            Object.defineProperty(Label.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Label instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILabel=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Label} Label instance
                             */
                            Label.create = function create(properties) {
                                return new Label(properties);
                            };
    
                            /**
                             * Encodes the specified Label message. Does not implicitly {@link google.ads.googleads.v11.resources.Label.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILabel} m Label message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Label.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.text_label != null && Object.hasOwnProperty.call(m, "text_label"))
                                    $root.google.ads.googleads.v11.common.TextLabel.encode(m.text_label, w.uint32(42).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(48).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(58).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Label message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Label.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILabel} message Label message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Label.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Label message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Label} Label
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Label.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Label();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.id = r.int64();
                                        break;
                                    case 7:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    case 5:
                                        m.text_label = $root.google.ads.googleads.v11.common.TextLabel.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Label message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Label} Label
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Label.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Label message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Label.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.text_label != null && m.hasOwnProperty("text_label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.TextLabel.verify(m.text_label);
                                        if (e)
                                            return "text_label." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Label message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Label} Label
                             */
                            Label.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Label)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Label();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.text_label != null) {
                                    if (typeof d.text_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Label.text_label: object expected");
                                    m.text_label = $root.google.ads.googleads.v11.common.TextLabel.fromObject(d.text_label);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Label message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @static
                             * @param {google.ads.googleads.v11.resources.Label} m Label
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Label.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.text_label = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.LabelStatusEnum.LabelStatus[m.status] : m.status;
                                }
                                if (m.text_label != null && m.hasOwnProperty("text_label")) {
                                    d.text_label = $root.google.ads.googleads.v11.common.TextLabel.toObject(m.text_label, o);
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Label to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Label
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Label.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Label;
                        })();
    
                        resources.LandingPageView = (function() {
    
                            /**
                             * Properties of a LandingPageView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ILandingPageView
                             * @property {string|null} [resource_name] LandingPageView resource_name
                             * @property {string|null} [unexpanded_final_url] LandingPageView unexpanded_final_url
                             */
    
                            /**
                             * Constructs a new LandingPageView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a LandingPageView.
                             * @implements ILandingPageView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ILandingPageView=} [p] Properties to set
                             */
                            function LandingPageView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LandingPageView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @instance
                             */
                            LandingPageView.prototype.resource_name = "";
    
                            /**
                             * LandingPageView unexpanded_final_url.
                             * @member {string|null|undefined} unexpanded_final_url
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @instance
                             */
                            LandingPageView.prototype.unexpanded_final_url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LandingPageView _unexpanded_final_url.
                             * @member {"unexpanded_final_url"|undefined} _unexpanded_final_url
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @instance
                             */
                            Object.defineProperty(LandingPageView.prototype, "_unexpanded_final_url", {
                                get: $util.oneOfGetter($oneOfFields = ["unexpanded_final_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LandingPageView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILandingPageView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.LandingPageView} LandingPageView instance
                             */
                            LandingPageView.create = function create(properties) {
                                return new LandingPageView(properties);
                            };
    
                            /**
                             * Encodes the specified LandingPageView message. Does not implicitly {@link google.ads.googleads.v11.resources.LandingPageView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILandingPageView} m LandingPageView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LandingPageView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.unexpanded_final_url != null && Object.hasOwnProperty.call(m, "unexpanded_final_url"))
                                    w.uint32(26).string(m.unexpanded_final_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LandingPageView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.LandingPageView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILandingPageView} message LandingPageView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LandingPageView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LandingPageView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.LandingPageView} LandingPageView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LandingPageView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.LandingPageView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.unexpanded_final_url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LandingPageView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.LandingPageView} LandingPageView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LandingPageView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LandingPageView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LandingPageView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.unexpanded_final_url != null && m.hasOwnProperty("unexpanded_final_url")) {
                                    p._unexpanded_final_url = 1;
                                    if (!$util.isString(m.unexpanded_final_url))
                                        return "unexpanded_final_url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LandingPageView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.LandingPageView} LandingPageView
                             */
                            LandingPageView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.LandingPageView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.LandingPageView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.unexpanded_final_url != null) {
                                    m.unexpanded_final_url = String(d.unexpanded_final_url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LandingPageView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.LandingPageView} m LandingPageView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LandingPageView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.unexpanded_final_url != null && m.hasOwnProperty("unexpanded_final_url")) {
                                    d.unexpanded_final_url = m.unexpanded_final_url;
                                    if (o.oneofs)
                                        d._unexpanded_final_url = "unexpanded_final_url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LandingPageView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.LandingPageView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LandingPageView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LandingPageView;
                        })();
    
                        resources.LanguageConstant = (function() {
    
                            /**
                             * Properties of a LanguageConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ILanguageConstant
                             * @property {string|null} [resource_name] LanguageConstant resource_name
                             * @property {number|null} [id] LanguageConstant id
                             * @property {string|null} [code] LanguageConstant code
                             * @property {string|null} [name] LanguageConstant name
                             * @property {boolean|null} [targetable] LanguageConstant targetable
                             */
    
                            /**
                             * Constructs a new LanguageConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a LanguageConstant.
                             * @implements ILanguageConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ILanguageConstant=} [p] Properties to set
                             */
                            function LanguageConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LanguageConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            LanguageConstant.prototype.resource_name = "";
    
                            /**
                             * LanguageConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            LanguageConstant.prototype.id = null;
    
                            /**
                             * LanguageConstant code.
                             * @member {string|null|undefined} code
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            LanguageConstant.prototype.code = null;
    
                            /**
                             * LanguageConstant name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            LanguageConstant.prototype.name = null;
    
                            /**
                             * LanguageConstant targetable.
                             * @member {boolean|null|undefined} targetable
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            LanguageConstant.prototype.targetable = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LanguageConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            Object.defineProperty(LanguageConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LanguageConstant _code.
                             * @member {"code"|undefined} _code
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            Object.defineProperty(LanguageConstant.prototype, "_code", {
                                get: $util.oneOfGetter($oneOfFields = ["code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LanguageConstant _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            Object.defineProperty(LanguageConstant.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * LanguageConstant _targetable.
                             * @member {"targetable"|undefined} _targetable
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             */
                            Object.defineProperty(LanguageConstant.prototype, "_targetable", {
                                get: $util.oneOfGetter($oneOfFields = ["targetable"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LanguageConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILanguageConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.LanguageConstant} LanguageConstant instance
                             */
                            LanguageConstant.create = function create(properties) {
                                return new LanguageConstant(properties);
                            };
    
                            /**
                             * Encodes the specified LanguageConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.LanguageConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILanguageConstant} m LanguageConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LanguageConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(48).int64(m.id);
                                if (m.code != null && Object.hasOwnProperty.call(m, "code"))
                                    w.uint32(58).string(m.code);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(66).string(m.name);
                                if (m.targetable != null && Object.hasOwnProperty.call(m, "targetable"))
                                    w.uint32(72).bool(m.targetable);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LanguageConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.LanguageConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILanguageConstant} message LanguageConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LanguageConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LanguageConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.LanguageConstant} LanguageConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LanguageConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.LanguageConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.id = r.int64();
                                        break;
                                    case 7:
                                        m.code = r.string();
                                        break;
                                    case 8:
                                        m.name = r.string();
                                        break;
                                    case 9:
                                        m.targetable = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LanguageConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.LanguageConstant} LanguageConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LanguageConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LanguageConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LanguageConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.code != null && m.hasOwnProperty("code")) {
                                    p._code = 1;
                                    if (!$util.isString(m.code))
                                        return "code: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.targetable != null && m.hasOwnProperty("targetable")) {
                                    p._targetable = 1;
                                    if (typeof m.targetable !== "boolean")
                                        return "targetable: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LanguageConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.LanguageConstant} LanguageConstant
                             */
                            LanguageConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.LanguageConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.LanguageConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.code != null) {
                                    m.code = String(d.code);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.targetable != null) {
                                    m.targetable = Boolean(d.targetable);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LanguageConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.LanguageConstant} m LanguageConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LanguageConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.code != null && m.hasOwnProperty("code")) {
                                    d.code = m.code;
                                    if (o.oneofs)
                                        d._code = "code";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.targetable != null && m.hasOwnProperty("targetable")) {
                                    d.targetable = m.targetable;
                                    if (o.oneofs)
                                        d._targetable = "targetable";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LanguageConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.LanguageConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LanguageConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LanguageConstant;
                        })();
    
                        resources.LeadFormSubmissionData = (function() {
    
                            /**
                             * Properties of a LeadFormSubmissionData.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ILeadFormSubmissionData
                             * @property {string|null} [resource_name] LeadFormSubmissionData resource_name
                             * @property {string|null} [id] LeadFormSubmissionData id
                             * @property {string|null} [asset] LeadFormSubmissionData asset
                             * @property {string|null} [campaign] LeadFormSubmissionData campaign
                             * @property {Array.<google.ads.googleads.v11.resources.ILeadFormSubmissionField>|null} [lead_form_submission_fields] LeadFormSubmissionData lead_form_submission_fields
                             * @property {string|null} [ad_group] LeadFormSubmissionData ad_group
                             * @property {string|null} [ad_group_ad] LeadFormSubmissionData ad_group_ad
                             * @property {string|null} [gclid] LeadFormSubmissionData gclid
                             * @property {string|null} [submission_date_time] LeadFormSubmissionData submission_date_time
                             */
    
                            /**
                             * Constructs a new LeadFormSubmissionData.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a LeadFormSubmissionData.
                             * @implements ILeadFormSubmissionData
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionData=} [p] Properties to set
                             */
                            function LeadFormSubmissionData(p) {
                                this.lead_form_submission_fields = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LeadFormSubmissionData resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.resource_name = "";
    
                            /**
                             * LeadFormSubmissionData id.
                             * @member {string} id
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.id = "";
    
                            /**
                             * LeadFormSubmissionData asset.
                             * @member {string} asset
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.asset = "";
    
                            /**
                             * LeadFormSubmissionData campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.campaign = "";
    
                            /**
                             * LeadFormSubmissionData lead_form_submission_fields.
                             * @member {Array.<google.ads.googleads.v11.resources.ILeadFormSubmissionField>} lead_form_submission_fields
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.lead_form_submission_fields = $util.emptyArray;
    
                            /**
                             * LeadFormSubmissionData ad_group.
                             * @member {string} ad_group
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.ad_group = "";
    
                            /**
                             * LeadFormSubmissionData ad_group_ad.
                             * @member {string} ad_group_ad
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.ad_group_ad = "";
    
                            /**
                             * LeadFormSubmissionData gclid.
                             * @member {string} gclid
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.gclid = "";
    
                            /**
                             * LeadFormSubmissionData submission_date_time.
                             * @member {string} submission_date_time
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             */
                            LeadFormSubmissionData.prototype.submission_date_time = "";
    
                            /**
                             * Creates a new LeadFormSubmissionData instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionData=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionData} LeadFormSubmissionData instance
                             */
                            LeadFormSubmissionData.create = function create(properties) {
                                return new LeadFormSubmissionData(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormSubmissionData message. Does not implicitly {@link google.ads.googleads.v11.resources.LeadFormSubmissionData.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionData} m LeadFormSubmissionData message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormSubmissionData.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(18).string(m.id);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    w.uint32(26).string(m.asset);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(34).string(m.campaign);
                                if (m.lead_form_submission_fields != null && m.lead_form_submission_fields.length) {
                                    for (var i = 0; i < m.lead_form_submission_fields.length; ++i)
                                        $root.google.ads.googleads.v11.resources.LeadFormSubmissionField.encode(m.lead_form_submission_fields[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(50).string(m.ad_group);
                                if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                    w.uint32(58).string(m.ad_group_ad);
                                if (m.gclid != null && Object.hasOwnProperty.call(m, "gclid"))
                                    w.uint32(66).string(m.gclid);
                                if (m.submission_date_time != null && Object.hasOwnProperty.call(m, "submission_date_time"))
                                    w.uint32(74).string(m.submission_date_time);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormSubmissionData message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.LeadFormSubmissionData.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionData} message LeadFormSubmissionData message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormSubmissionData.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormSubmissionData message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionData} LeadFormSubmissionData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormSubmissionData.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.LeadFormSubmissionData();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.string();
                                        break;
                                    case 3:
                                        m.asset = r.string();
                                        break;
                                    case 4:
                                        m.campaign = r.string();
                                        break;
                                    case 5:
                                        if (!(m.lead_form_submission_fields && m.lead_form_submission_fields.length))
                                            m.lead_form_submission_fields = [];
                                        m.lead_form_submission_fields.push($root.google.ads.googleads.v11.resources.LeadFormSubmissionField.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        m.ad_group = r.string();
                                        break;
                                    case 7:
                                        m.ad_group_ad = r.string();
                                        break;
                                    case 8:
                                        m.gclid = r.string();
                                        break;
                                    case 9:
                                        m.submission_date_time = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormSubmissionData message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionData} LeadFormSubmissionData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormSubmissionData.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormSubmissionData message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormSubmissionData.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isString(m.id))
                                        return "id: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    if (!$util.isString(m.asset))
                                        return "asset: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.lead_form_submission_fields != null && m.hasOwnProperty("lead_form_submission_fields")) {
                                    if (!Array.isArray(m.lead_form_submission_fields))
                                        return "lead_form_submission_fields: array expected";
                                    for (var i = 0; i < m.lead_form_submission_fields.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.LeadFormSubmissionField.verify(m.lead_form_submission_fields[i]);
                                            if (e)
                                                return "lead_form_submission_fields." + e;
                                        }
                                    }
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    if (!$util.isString(m.ad_group_ad))
                                        return "ad_group_ad: string expected";
                                }
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    if (!$util.isString(m.gclid))
                                        return "gclid: string expected";
                                }
                                if (m.submission_date_time != null && m.hasOwnProperty("submission_date_time")) {
                                    if (!$util.isString(m.submission_date_time))
                                        return "submission_date_time: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormSubmissionData message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionData} LeadFormSubmissionData
                             */
                            LeadFormSubmissionData.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.LeadFormSubmissionData)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.LeadFormSubmissionData();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    m.id = String(d.id);
                                }
                                if (d.asset != null) {
                                    m.asset = String(d.asset);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.lead_form_submission_fields) {
                                    if (!Array.isArray(d.lead_form_submission_fields))
                                        throw TypeError(".google.ads.googleads.v11.resources.LeadFormSubmissionData.lead_form_submission_fields: array expected");
                                    m.lead_form_submission_fields = [];
                                    for (var i = 0; i < d.lead_form_submission_fields.length; ++i) {
                                        if (typeof d.lead_form_submission_fields[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.LeadFormSubmissionData.lead_form_submission_fields: object expected");
                                        m.lead_form_submission_fields[i] = $root.google.ads.googleads.v11.resources.LeadFormSubmissionField.fromObject(d.lead_form_submission_fields[i]);
                                    }
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.ad_group_ad != null) {
                                    m.ad_group_ad = String(d.ad_group_ad);
                                }
                                if (d.gclid != null) {
                                    m.gclid = String(d.gclid);
                                }
                                if (d.submission_date_time != null) {
                                    m.submission_date_time = String(d.submission_date_time);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LeadFormSubmissionData message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @static
                             * @param {google.ads.googleads.v11.resources.LeadFormSubmissionData} m LeadFormSubmissionData
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormSubmissionData.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.lead_form_submission_fields = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.id = "";
                                    d.asset = "";
                                    d.campaign = "";
                                    d.ad_group = "";
                                    d.ad_group_ad = "";
                                    d.gclid = "";
                                    d.submission_date_time = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    d.id = m.id;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = m.asset;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.lead_form_submission_fields && m.lead_form_submission_fields.length) {
                                    d.lead_form_submission_fields = [];
                                    for (var j = 0; j < m.lead_form_submission_fields.length; ++j) {
                                        d.lead_form_submission_fields[j] = $root.google.ads.googleads.v11.resources.LeadFormSubmissionField.toObject(m.lead_form_submission_fields[j], o);
                                    }
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    d.ad_group_ad = m.ad_group_ad;
                                }
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    d.gclid = m.gclid;
                                }
                                if (m.submission_date_time != null && m.hasOwnProperty("submission_date_time")) {
                                    d.submission_date_time = m.submission_date_time;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LeadFormSubmissionData to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionData
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormSubmissionData.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LeadFormSubmissionData;
                        })();
    
                        resources.LeadFormSubmissionField = (function() {
    
                            /**
                             * Properties of a LeadFormSubmissionField.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ILeadFormSubmissionField
                             * @property {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType|null} [field_type] LeadFormSubmissionField field_type
                             * @property {string|null} [field_value] LeadFormSubmissionField field_value
                             */
    
                            /**
                             * Constructs a new LeadFormSubmissionField.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a LeadFormSubmissionField.
                             * @implements ILeadFormSubmissionField
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionField=} [p] Properties to set
                             */
                            function LeadFormSubmissionField(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LeadFormSubmissionField field_type.
                             * @member {google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType} field_type
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @instance
                             */
                            LeadFormSubmissionField.prototype.field_type = 0;
    
                            /**
                             * LeadFormSubmissionField field_value.
                             * @member {string} field_value
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @instance
                             */
                            LeadFormSubmissionField.prototype.field_value = "";
    
                            /**
                             * Creates a new LeadFormSubmissionField instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionField=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionField} LeadFormSubmissionField instance
                             */
                            LeadFormSubmissionField.create = function create(properties) {
                                return new LeadFormSubmissionField(properties);
                            };
    
                            /**
                             * Encodes the specified LeadFormSubmissionField message. Does not implicitly {@link google.ads.googleads.v11.resources.LeadFormSubmissionField.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionField} m LeadFormSubmissionField message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormSubmissionField.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.field_type != null && Object.hasOwnProperty.call(m, "field_type"))
                                    w.uint32(8).int32(m.field_type);
                                if (m.field_value != null && Object.hasOwnProperty.call(m, "field_value"))
                                    w.uint32(18).string(m.field_value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LeadFormSubmissionField message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.LeadFormSubmissionField.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILeadFormSubmissionField} message LeadFormSubmissionField message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LeadFormSubmissionField.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LeadFormSubmissionField message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionField} LeadFormSubmissionField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormSubmissionField.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.LeadFormSubmissionField();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.field_type = r.int32();
                                        break;
                                    case 2:
                                        m.field_value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LeadFormSubmissionField message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionField} LeadFormSubmissionField
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LeadFormSubmissionField.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LeadFormSubmissionField message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LeadFormSubmissionField.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    switch (m.field_type) {
                                    default:
                                        return "field_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 1001:
                                    case 1002:
                                    case 1003:
                                    case 1004:
                                    case 1005:
                                    case 1009:
                                    case 1010:
                                    case 1006:
                                    case 1007:
                                    case 1008:
                                    case 1011:
                                    case 1012:
                                    case 1013:
                                    case 1014:
                                    case 1016:
                                    case 1017:
                                    case 1018:
                                    case 1019:
                                    case 1020:
                                    case 1021:
                                    case 1022:
                                    case 1023:
                                    case 1048:
                                    case 1049:
                                    case 1050:
                                    case 1024:
                                    case 1025:
                                    case 1026:
                                    case 1027:
                                    case 1028:
                                    case 1029:
                                    case 1030:
                                    case 1031:
                                    case 1033:
                                    case 1034:
                                    case 1035:
                                    case 1036:
                                    case 1038:
                                    case 1039:
                                    case 1040:
                                    case 1041:
                                    case 1042:
                                    case 1043:
                                    case 1044:
                                    case 1045:
                                    case 1046:
                                    case 1047:
                                        break;
                                    }
                                }
                                if (m.field_value != null && m.hasOwnProperty("field_value")) {
                                    if (!$util.isString(m.field_value))
                                        return "field_value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LeadFormSubmissionField message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.LeadFormSubmissionField} LeadFormSubmissionField
                             */
                            LeadFormSubmissionField.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.LeadFormSubmissionField)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.LeadFormSubmissionField();
                                switch (d.field_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.field_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.field_type = 1;
                                    break;
                                case "FULL_NAME":
                                case 2:
                                    m.field_type = 2;
                                    break;
                                case "EMAIL":
                                case 3:
                                    m.field_type = 3;
                                    break;
                                case "PHONE_NUMBER":
                                case 4:
                                    m.field_type = 4;
                                    break;
                                case "POSTAL_CODE":
                                case 5:
                                    m.field_type = 5;
                                    break;
                                case "CITY":
                                case 9:
                                    m.field_type = 9;
                                    break;
                                case "REGION":
                                case 10:
                                    m.field_type = 10;
                                    break;
                                case "COUNTRY":
                                case 11:
                                    m.field_type = 11;
                                    break;
                                case "WORK_EMAIL":
                                case 12:
                                    m.field_type = 12;
                                    break;
                                case "COMPANY_NAME":
                                case 13:
                                    m.field_type = 13;
                                    break;
                                case "WORK_PHONE":
                                case 14:
                                    m.field_type = 14;
                                    break;
                                case "JOB_TITLE":
                                case 15:
                                    m.field_type = 15;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_CPF_BR":
                                case 16:
                                    m.field_type = 16;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_DNI_AR":
                                case 17:
                                    m.field_type = 17;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_DNI_PE":
                                case 18:
                                    m.field_type = 18;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_RUT_CL":
                                case 19:
                                    m.field_type = 19;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_CC_CO":
                                case 20:
                                    m.field_type = 20;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_CI_EC":
                                case 21:
                                    m.field_type = 21;
                                    break;
                                case "GOVERNMENT_ISSUED_ID_RFC_MX":
                                case 22:
                                    m.field_type = 22;
                                    break;
                                case "FIRST_NAME":
                                case 23:
                                    m.field_type = 23;
                                    break;
                                case "LAST_NAME":
                                case 24:
                                    m.field_type = 24;
                                    break;
                                case "VEHICLE_MODEL":
                                case 1001:
                                    m.field_type = 1001;
                                    break;
                                case "VEHICLE_TYPE":
                                case 1002:
                                    m.field_type = 1002;
                                    break;
                                case "PREFERRED_DEALERSHIP":
                                case 1003:
                                    m.field_type = 1003;
                                    break;
                                case "VEHICLE_PURCHASE_TIMELINE":
                                case 1004:
                                    m.field_type = 1004;
                                    break;
                                case "VEHICLE_OWNERSHIP":
                                case 1005:
                                    m.field_type = 1005;
                                    break;
                                case "VEHICLE_PAYMENT_TYPE":
                                case 1009:
                                    m.field_type = 1009;
                                    break;
                                case "VEHICLE_CONDITION":
                                case 1010:
                                    m.field_type = 1010;
                                    break;
                                case "COMPANY_SIZE":
                                case 1006:
                                    m.field_type = 1006;
                                    break;
                                case "ANNUAL_SALES":
                                case 1007:
                                    m.field_type = 1007;
                                    break;
                                case "YEARS_IN_BUSINESS":
                                case 1008:
                                    m.field_type = 1008;
                                    break;
                                case "JOB_DEPARTMENT":
                                case 1011:
                                    m.field_type = 1011;
                                    break;
                                case "JOB_ROLE":
                                case 1012:
                                    m.field_type = 1012;
                                    break;
                                case "EDUCATION_PROGRAM":
                                case 1013:
                                    m.field_type = 1013;
                                    break;
                                case "EDUCATION_COURSE":
                                case 1014:
                                    m.field_type = 1014;
                                    break;
                                case "PRODUCT":
                                case 1016:
                                    m.field_type = 1016;
                                    break;
                                case "SERVICE":
                                case 1017:
                                    m.field_type = 1017;
                                    break;
                                case "OFFER":
                                case 1018:
                                    m.field_type = 1018;
                                    break;
                                case "CATEGORY":
                                case 1019:
                                    m.field_type = 1019;
                                    break;
                                case "PREFERRED_CONTACT_METHOD":
                                case 1020:
                                    m.field_type = 1020;
                                    break;
                                case "PREFERRED_LOCATION":
                                case 1021:
                                    m.field_type = 1021;
                                    break;
                                case "PREFERRED_CONTACT_TIME":
                                case 1022:
                                    m.field_type = 1022;
                                    break;
                                case "PURCHASE_TIMELINE":
                                case 1023:
                                    m.field_type = 1023;
                                    break;
                                case "YEARS_OF_EXPERIENCE":
                                case 1048:
                                    m.field_type = 1048;
                                    break;
                                case "JOB_INDUSTRY":
                                case 1049:
                                    m.field_type = 1049;
                                    break;
                                case "LEVEL_OF_EDUCATION":
                                case 1050:
                                    m.field_type = 1050;
                                    break;
                                case "PROPERTY_TYPE":
                                case 1024:
                                    m.field_type = 1024;
                                    break;
                                case "REALTOR_HELP_GOAL":
                                case 1025:
                                    m.field_type = 1025;
                                    break;
                                case "PROPERTY_COMMUNITY":
                                case 1026:
                                    m.field_type = 1026;
                                    break;
                                case "PRICE_RANGE":
                                case 1027:
                                    m.field_type = 1027;
                                    break;
                                case "NUMBER_OF_BEDROOMS":
                                case 1028:
                                    m.field_type = 1028;
                                    break;
                                case "FURNISHED_PROPERTY":
                                case 1029:
                                    m.field_type = 1029;
                                    break;
                                case "PETS_ALLOWED_PROPERTY":
                                case 1030:
                                    m.field_type = 1030;
                                    break;
                                case "NEXT_PLANNED_PURCHASE":
                                case 1031:
                                    m.field_type = 1031;
                                    break;
                                case "EVENT_SIGNUP_INTEREST":
                                case 1033:
                                    m.field_type = 1033;
                                    break;
                                case "PREFERRED_SHOPPING_PLACES":
                                case 1034:
                                    m.field_type = 1034;
                                    break;
                                case "FAVORITE_BRAND":
                                case 1035:
                                    m.field_type = 1035;
                                    break;
                                case "TRANSPORTATION_COMMERCIAL_LICENSE_TYPE":
                                case 1036:
                                    m.field_type = 1036;
                                    break;
                                case "EVENT_BOOKING_INTEREST":
                                case 1038:
                                    m.field_type = 1038;
                                    break;
                                case "DESTINATION_COUNTRY":
                                case 1039:
                                    m.field_type = 1039;
                                    break;
                                case "DESTINATION_CITY":
                                case 1040:
                                    m.field_type = 1040;
                                    break;
                                case "DEPARTURE_COUNTRY":
                                case 1041:
                                    m.field_type = 1041;
                                    break;
                                case "DEPARTURE_CITY":
                                case 1042:
                                    m.field_type = 1042;
                                    break;
                                case "DEPARTURE_DATE":
                                case 1043:
                                    m.field_type = 1043;
                                    break;
                                case "RETURN_DATE":
                                case 1044:
                                    m.field_type = 1044;
                                    break;
                                case "NUMBER_OF_TRAVELERS":
                                case 1045:
                                    m.field_type = 1045;
                                    break;
                                case "TRAVEL_BUDGET":
                                case 1046:
                                    m.field_type = 1046;
                                    break;
                                case "TRAVEL_ACCOMMODATION":
                                case 1047:
                                    m.field_type = 1047;
                                    break;
                                }
                                if (d.field_value != null) {
                                    m.field_value = String(d.field_value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LeadFormSubmissionField message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @static
                             * @param {google.ads.googleads.v11.resources.LeadFormSubmissionField} m LeadFormSubmissionField
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LeadFormSubmissionField.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.field_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.field_value = "";
                                }
                                if (m.field_type != null && m.hasOwnProperty("field_type")) {
                                    d.field_type = o.enums === String ? $root.google.ads.googleads.v11.enums.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType[m.field_type] : m.field_type;
                                }
                                if (m.field_value != null && m.hasOwnProperty("field_value")) {
                                    d.field_value = m.field_value;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LeadFormSubmissionField to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.LeadFormSubmissionField
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LeadFormSubmissionField.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LeadFormSubmissionField;
                        })();
    
                        resources.LifeEvent = (function() {
    
                            /**
                             * Properties of a LifeEvent.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ILifeEvent
                             * @property {string|null} [resource_name] LifeEvent resource_name
                             * @property {number|null} [id] LifeEvent id
                             * @property {string|null} [name] LifeEvent name
                             * @property {string|null} [parent] LifeEvent parent
                             * @property {boolean|null} [launched_to_all] LifeEvent launched_to_all
                             * @property {Array.<google.ads.googleads.v11.common.ICriterionCategoryAvailability>|null} [availabilities] LifeEvent availabilities
                             */
    
                            /**
                             * Constructs a new LifeEvent.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a LifeEvent.
                             * @implements ILifeEvent
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ILifeEvent=} [p] Properties to set
                             */
                            function LifeEvent(p) {
                                this.availabilities = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LifeEvent resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @instance
                             */
                            LifeEvent.prototype.resource_name = "";
    
                            /**
                             * LifeEvent id.
                             * @member {number} id
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @instance
                             */
                            LifeEvent.prototype.id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * LifeEvent name.
                             * @member {string} name
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @instance
                             */
                            LifeEvent.prototype.name = "";
    
                            /**
                             * LifeEvent parent.
                             * @member {string} parent
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @instance
                             */
                            LifeEvent.prototype.parent = "";
    
                            /**
                             * LifeEvent launched_to_all.
                             * @member {boolean} launched_to_all
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @instance
                             */
                            LifeEvent.prototype.launched_to_all = false;
    
                            /**
                             * LifeEvent availabilities.
                             * @member {Array.<google.ads.googleads.v11.common.ICriterionCategoryAvailability>} availabilities
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @instance
                             */
                            LifeEvent.prototype.availabilities = $util.emptyArray;
    
                            /**
                             * Creates a new LifeEvent instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILifeEvent=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.LifeEvent} LifeEvent instance
                             */
                            LifeEvent.create = function create(properties) {
                                return new LifeEvent(properties);
                            };
    
                            /**
                             * Encodes the specified LifeEvent message. Does not implicitly {@link google.ads.googleads.v11.resources.LifeEvent.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILifeEvent} m LifeEvent message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LifeEvent.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(16).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(26).string(m.name);
                                if (m.parent != null && Object.hasOwnProperty.call(m, "parent"))
                                    w.uint32(34).string(m.parent);
                                if (m.launched_to_all != null && Object.hasOwnProperty.call(m, "launched_to_all"))
                                    w.uint32(40).bool(m.launched_to_all);
                                if (m.availabilities != null && m.availabilities.length) {
                                    for (var i = 0; i < m.availabilities.length; ++i)
                                        $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.encode(m.availabilities[i], w.uint32(50).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified LifeEvent message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.LifeEvent.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILifeEvent} message LifeEvent message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LifeEvent.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LifeEvent message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.LifeEvent} LifeEvent
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LifeEvent.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.LifeEvent();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        m.parent = r.string();
                                        break;
                                    case 5:
                                        m.launched_to_all = r.bool();
                                        break;
                                    case 6:
                                        if (!(m.availabilities && m.availabilities.length))
                                            m.availabilities = [];
                                        m.availabilities.push($root.google.ads.googleads.v11.common.CriterionCategoryAvailability.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LifeEvent message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.LifeEvent} LifeEvent
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LifeEvent.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LifeEvent message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LifeEvent.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.parent != null && m.hasOwnProperty("parent")) {
                                    if (!$util.isString(m.parent))
                                        return "parent: string expected";
                                }
                                if (m.launched_to_all != null && m.hasOwnProperty("launched_to_all")) {
                                    if (typeof m.launched_to_all !== "boolean")
                                        return "launched_to_all: boolean expected";
                                }
                                if (m.availabilities != null && m.hasOwnProperty("availabilities")) {
                                    if (!Array.isArray(m.availabilities))
                                        return "availabilities: array expected";
                                    for (var i = 0; i < m.availabilities.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.verify(m.availabilities[i]);
                                            if (e)
                                                return "availabilities." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LifeEvent message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.LifeEvent} LifeEvent
                             */
                            LifeEvent.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.LifeEvent)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.LifeEvent();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.parent != null) {
                                    m.parent = String(d.parent);
                                }
                                if (d.launched_to_all != null) {
                                    m.launched_to_all = Boolean(d.launched_to_all);
                                }
                                if (d.availabilities) {
                                    if (!Array.isArray(d.availabilities))
                                        throw TypeError(".google.ads.googleads.v11.resources.LifeEvent.availabilities: array expected");
                                    m.availabilities = [];
                                    for (var i = 0; i < d.availabilities.length; ++i) {
                                        if (typeof d.availabilities[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.LifeEvent.availabilities: object expected");
                                        m.availabilities[i] = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.fromObject(d.availabilities[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LifeEvent message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @static
                             * @param {google.ads.googleads.v11.resources.LifeEvent} m LifeEvent
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LifeEvent.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.availabilities = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.id = o.longs === String ? "0" : 0;
                                    d.name = "";
                                    d.parent = "";
                                    d.launched_to_all = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                }
                                if (m.parent != null && m.hasOwnProperty("parent")) {
                                    d.parent = m.parent;
                                }
                                if (m.launched_to_all != null && m.hasOwnProperty("launched_to_all")) {
                                    d.launched_to_all = m.launched_to_all;
                                }
                                if (m.availabilities && m.availabilities.length) {
                                    d.availabilities = [];
                                    for (var j = 0; j < m.availabilities.length; ++j) {
                                        d.availabilities[j] = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.toObject(m.availabilities[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LifeEvent to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.LifeEvent
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LifeEvent.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LifeEvent;
                        })();
    
                        resources.LocationView = (function() {
    
                            /**
                             * Properties of a LocationView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ILocationView
                             * @property {string|null} [resource_name] LocationView resource_name
                             */
    
                            /**
                             * Constructs a new LocationView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a LocationView.
                             * @implements ILocationView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ILocationView=} [p] Properties to set
                             */
                            function LocationView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LocationView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @instance
                             */
                            LocationView.prototype.resource_name = "";
    
                            /**
                             * Creates a new LocationView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILocationView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.LocationView} LocationView instance
                             */
                            LocationView.create = function create(properties) {
                                return new LocationView(properties);
                            };
    
                            /**
                             * Encodes the specified LocationView message. Does not implicitly {@link google.ads.googleads.v11.resources.LocationView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILocationView} m LocationView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified LocationView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.LocationView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ILocationView} message LocationView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LocationView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LocationView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.LocationView} LocationView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.LocationView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LocationView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.LocationView} LocationView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LocationView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LocationView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LocationView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LocationView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.LocationView} LocationView
                             */
                            LocationView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.LocationView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.LocationView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LocationView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.LocationView} m LocationView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LocationView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LocationView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.LocationView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LocationView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LocationView;
                        })();
    
                        resources.ManagedPlacementView = (function() {
    
                            /**
                             * Properties of a ManagedPlacementView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IManagedPlacementView
                             * @property {string|null} [resource_name] ManagedPlacementView resource_name
                             */
    
                            /**
                             * Constructs a new ManagedPlacementView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ManagedPlacementView.
                             * @implements IManagedPlacementView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IManagedPlacementView=} [p] Properties to set
                             */
                            function ManagedPlacementView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ManagedPlacementView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @instance
                             */
                            ManagedPlacementView.prototype.resource_name = "";
    
                            /**
                             * Creates a new ManagedPlacementView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IManagedPlacementView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ManagedPlacementView} ManagedPlacementView instance
                             */
                            ManagedPlacementView.create = function create(properties) {
                                return new ManagedPlacementView(properties);
                            };
    
                            /**
                             * Encodes the specified ManagedPlacementView message. Does not implicitly {@link google.ads.googleads.v11.resources.ManagedPlacementView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IManagedPlacementView} m ManagedPlacementView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManagedPlacementView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ManagedPlacementView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ManagedPlacementView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IManagedPlacementView} message ManagedPlacementView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ManagedPlacementView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ManagedPlacementView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ManagedPlacementView} ManagedPlacementView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManagedPlacementView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ManagedPlacementView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ManagedPlacementView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ManagedPlacementView} ManagedPlacementView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ManagedPlacementView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ManagedPlacementView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ManagedPlacementView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ManagedPlacementView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ManagedPlacementView} ManagedPlacementView
                             */
                            ManagedPlacementView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ManagedPlacementView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ManagedPlacementView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ManagedPlacementView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ManagedPlacementView} m ManagedPlacementView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ManagedPlacementView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ManagedPlacementView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ManagedPlacementView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ManagedPlacementView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ManagedPlacementView;
                        })();
    
                        resources.MediaFile = (function() {
    
                            /**
                             * Properties of a MediaFile.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMediaFile
                             * @property {string|null} [resource_name] MediaFile resource_name
                             * @property {number|null} [id] MediaFile id
                             * @property {google.ads.googleads.v11.enums.MediaTypeEnum.MediaType|null} [type] MediaFile type
                             * @property {google.ads.googleads.v11.enums.MimeTypeEnum.MimeType|null} [mime_type] MediaFile mime_type
                             * @property {string|null} [source_url] MediaFile source_url
                             * @property {string|null} [name] MediaFile name
                             * @property {number|null} [file_size] MediaFile file_size
                             * @property {google.ads.googleads.v11.resources.IMediaImage|null} [image] MediaFile image
                             * @property {google.ads.googleads.v11.resources.IMediaBundle|null} [media_bundle] MediaFile media_bundle
                             * @property {google.ads.googleads.v11.resources.IMediaAudio|null} [audio] MediaFile audio
                             * @property {google.ads.googleads.v11.resources.IMediaVideo|null} [video] MediaFile video
                             */
    
                            /**
                             * Constructs a new MediaFile.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MediaFile.
                             * @implements IMediaFile
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMediaFile=} [p] Properties to set
                             */
                            function MediaFile(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MediaFile resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.resource_name = "";
    
                            /**
                             * MediaFile id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.id = null;
    
                            /**
                             * MediaFile type.
                             * @member {google.ads.googleads.v11.enums.MediaTypeEnum.MediaType} type
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.type = 0;
    
                            /**
                             * MediaFile mime_type.
                             * @member {google.ads.googleads.v11.enums.MimeTypeEnum.MimeType} mime_type
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.mime_type = 0;
    
                            /**
                             * MediaFile source_url.
                             * @member {string|null|undefined} source_url
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.source_url = null;
    
                            /**
                             * MediaFile name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.name = null;
    
                            /**
                             * MediaFile file_size.
                             * @member {number|null|undefined} file_size
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.file_size = null;
    
                            /**
                             * MediaFile image.
                             * @member {google.ads.googleads.v11.resources.IMediaImage|null|undefined} image
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.image = null;
    
                            /**
                             * MediaFile media_bundle.
                             * @member {google.ads.googleads.v11.resources.IMediaBundle|null|undefined} media_bundle
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.media_bundle = null;
    
                            /**
                             * MediaFile audio.
                             * @member {google.ads.googleads.v11.resources.IMediaAudio|null|undefined} audio
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.audio = null;
    
                            /**
                             * MediaFile video.
                             * @member {google.ads.googleads.v11.resources.IMediaVideo|null|undefined} video
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            MediaFile.prototype.video = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MediaFile _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            Object.defineProperty(MediaFile.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaFile _source_url.
                             * @member {"source_url"|undefined} _source_url
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            Object.defineProperty(MediaFile.prototype, "_source_url", {
                                get: $util.oneOfGetter($oneOfFields = ["source_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaFile _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            Object.defineProperty(MediaFile.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaFile _file_size.
                             * @member {"file_size"|undefined} _file_size
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            Object.defineProperty(MediaFile.prototype, "_file_size", {
                                get: $util.oneOfGetter($oneOfFields = ["file_size"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaFile mediatype.
                             * @member {"image"|"media_bundle"|"audio"|"video"|undefined} mediatype
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             */
                            Object.defineProperty(MediaFile.prototype, "mediatype", {
                                get: $util.oneOfGetter($oneOfFields = ["image", "media_bundle", "audio", "video"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MediaFile instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaFile=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MediaFile} MediaFile instance
                             */
                            MediaFile.create = function create(properties) {
                                return new MediaFile(properties);
                            };
    
                            /**
                             * Encodes the specified MediaFile message. Does not implicitly {@link google.ads.googleads.v11.resources.MediaFile.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaFile} m MediaFile message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaFile.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.image != null && Object.hasOwnProperty.call(m, "image"))
                                    $root.google.ads.googleads.v11.resources.MediaImage.encode(m.image, w.uint32(26).fork()).ldelim();
                                if (m.media_bundle != null && Object.hasOwnProperty.call(m, "media_bundle"))
                                    $root.google.ads.googleads.v11.resources.MediaBundle.encode(m.media_bundle, w.uint32(34).fork()).ldelim();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(40).int32(m.type);
                                if (m.mime_type != null && Object.hasOwnProperty.call(m, "mime_type"))
                                    w.uint32(48).int32(m.mime_type);
                                if (m.audio != null && Object.hasOwnProperty.call(m, "audio"))
                                    $root.google.ads.googleads.v11.resources.MediaAudio.encode(m.audio, w.uint32(82).fork()).ldelim();
                                if (m.video != null && Object.hasOwnProperty.call(m, "video"))
                                    $root.google.ads.googleads.v11.resources.MediaVideo.encode(m.video, w.uint32(90).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(96).int64(m.id);
                                if (m.source_url != null && Object.hasOwnProperty.call(m, "source_url"))
                                    w.uint32(106).string(m.source_url);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(114).string(m.name);
                                if (m.file_size != null && Object.hasOwnProperty.call(m, "file_size"))
                                    w.uint32(120).int64(m.file_size);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaFile message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MediaFile.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaFile} message MediaFile message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaFile.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaFile message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MediaFile} MediaFile
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaFile.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MediaFile();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 12:
                                        m.id = r.int64();
                                        break;
                                    case 5:
                                        m.type = r.int32();
                                        break;
                                    case 6:
                                        m.mime_type = r.int32();
                                        break;
                                    case 13:
                                        m.source_url = r.string();
                                        break;
                                    case 14:
                                        m.name = r.string();
                                        break;
                                    case 15:
                                        m.file_size = r.int64();
                                        break;
                                    case 3:
                                        m.image = $root.google.ads.googleads.v11.resources.MediaImage.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.media_bundle = $root.google.ads.googleads.v11.resources.MediaBundle.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.audio = $root.google.ads.googleads.v11.resources.MediaAudio.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.video = $root.google.ads.googleads.v11.resources.MediaVideo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaFile message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MediaFile} MediaFile
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaFile.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaFile message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaFile.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.mime_type != null && m.hasOwnProperty("mime_type")) {
                                    switch (m.mime_type) {
                                    default:
                                        return "mime_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                if (m.source_url != null && m.hasOwnProperty("source_url")) {
                                    p._source_url = 1;
                                    if (!$util.isString(m.source_url))
                                        return "source_url: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.file_size != null && m.hasOwnProperty("file_size")) {
                                    p._file_size = 1;
                                    if (!$util.isInteger(m.file_size) && !(m.file_size && $util.isInteger(m.file_size.low) && $util.isInteger(m.file_size.high)))
                                        return "file_size: integer|Long expected";
                                }
                                if (m.image != null && m.hasOwnProperty("image")) {
                                    p.mediatype = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MediaImage.verify(m.image);
                                        if (e)
                                            return "image." + e;
                                    }
                                }
                                if (m.media_bundle != null && m.hasOwnProperty("media_bundle")) {
                                    if (p.mediatype === 1)
                                        return "mediatype: multiple values";
                                    p.mediatype = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MediaBundle.verify(m.media_bundle);
                                        if (e)
                                            return "media_bundle." + e;
                                    }
                                }
                                if (m.audio != null && m.hasOwnProperty("audio")) {
                                    if (p.mediatype === 1)
                                        return "mediatype: multiple values";
                                    p.mediatype = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MediaAudio.verify(m.audio);
                                        if (e)
                                            return "audio." + e;
                                    }
                                }
                                if (m.video != null && m.hasOwnProperty("video")) {
                                    if (p.mediatype === 1)
                                        return "mediatype: multiple values";
                                    p.mediatype = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MediaVideo.verify(m.video);
                                        if (e)
                                            return "video." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MediaFile message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MediaFile} MediaFile
                             */
                            MediaFile.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MediaFile)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MediaFile();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "IMAGE":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "ICON":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MEDIA_BUNDLE":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "AUDIO":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "VIDEO":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "DYNAMIC_IMAGE":
                                case 7:
                                    m.type = 7;
                                    break;
                                }
                                switch (d.mime_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.mime_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.mime_type = 1;
                                    break;
                                case "IMAGE_JPEG":
                                case 2:
                                    m.mime_type = 2;
                                    break;
                                case "IMAGE_GIF":
                                case 3:
                                    m.mime_type = 3;
                                    break;
                                case "IMAGE_PNG":
                                case 4:
                                    m.mime_type = 4;
                                    break;
                                case "FLASH":
                                case 5:
                                    m.mime_type = 5;
                                    break;
                                case "TEXT_HTML":
                                case 6:
                                    m.mime_type = 6;
                                    break;
                                case "PDF":
                                case 7:
                                    m.mime_type = 7;
                                    break;
                                case "MSWORD":
                                case 8:
                                    m.mime_type = 8;
                                    break;
                                case "MSEXCEL":
                                case 9:
                                    m.mime_type = 9;
                                    break;
                                case "RTF":
                                case 10:
                                    m.mime_type = 10;
                                    break;
                                case "AUDIO_WAV":
                                case 11:
                                    m.mime_type = 11;
                                    break;
                                case "AUDIO_MP3":
                                case 12:
                                    m.mime_type = 12;
                                    break;
                                case "HTML5_AD_ZIP":
                                case 13:
                                    m.mime_type = 13;
                                    break;
                                }
                                if (d.source_url != null) {
                                    m.source_url = String(d.source_url);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.file_size != null) {
                                    if ($util.Long)
                                        (m.file_size = $util.Long.fromValue(d.file_size)).unsigned = false;
                                    else if (typeof d.file_size === "string")
                                        m.file_size = parseInt(d.file_size, 10);
                                    else if (typeof d.file_size === "number")
                                        m.file_size = d.file_size;
                                    else if (typeof d.file_size === "object")
                                        m.file_size = new $util.LongBits(d.file_size.low >>> 0, d.file_size.high >>> 0).toNumber();
                                }
                                if (d.image != null) {
                                    if (typeof d.image !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.MediaFile.image: object expected");
                                    m.image = $root.google.ads.googleads.v11.resources.MediaImage.fromObject(d.image);
                                }
                                if (d.media_bundle != null) {
                                    if (typeof d.media_bundle !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.MediaFile.media_bundle: object expected");
                                    m.media_bundle = $root.google.ads.googleads.v11.resources.MediaBundle.fromObject(d.media_bundle);
                                }
                                if (d.audio != null) {
                                    if (typeof d.audio !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.MediaFile.audio: object expected");
                                    m.audio = $root.google.ads.googleads.v11.resources.MediaAudio.fromObject(d.audio);
                                }
                                if (d.video != null) {
                                    if (typeof d.video !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.MediaFile.video: object expected");
                                    m.video = $root.google.ads.googleads.v11.resources.MediaVideo.fromObject(d.video);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MediaFile message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @static
                             * @param {google.ads.googleads.v11.resources.MediaFile} m MediaFile
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaFile.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.mime_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.image != null && m.hasOwnProperty("image")) {
                                    d.image = $root.google.ads.googleads.v11.resources.MediaImage.toObject(m.image, o);
                                    if (o.oneofs)
                                        d.mediatype = "image";
                                }
                                if (m.media_bundle != null && m.hasOwnProperty("media_bundle")) {
                                    d.media_bundle = $root.google.ads.googleads.v11.resources.MediaBundle.toObject(m.media_bundle, o);
                                    if (o.oneofs)
                                        d.mediatype = "media_bundle";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.MediaTypeEnum.MediaType[m.type] : m.type;
                                }
                                if (m.mime_type != null && m.hasOwnProperty("mime_type")) {
                                    d.mime_type = o.enums === String ? $root.google.ads.googleads.v11.enums.MimeTypeEnum.MimeType[m.mime_type] : m.mime_type;
                                }
                                if (m.audio != null && m.hasOwnProperty("audio")) {
                                    d.audio = $root.google.ads.googleads.v11.resources.MediaAudio.toObject(m.audio, o);
                                    if (o.oneofs)
                                        d.mediatype = "audio";
                                }
                                if (m.video != null && m.hasOwnProperty("video")) {
                                    d.video = $root.google.ads.googleads.v11.resources.MediaVideo.toObject(m.video, o);
                                    if (o.oneofs)
                                        d.mediatype = "video";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.source_url != null && m.hasOwnProperty("source_url")) {
                                    d.source_url = m.source_url;
                                    if (o.oneofs)
                                        d._source_url = "source_url";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.file_size != null && m.hasOwnProperty("file_size")) {
                                    if (typeof m.file_size === "number")
                                        d.file_size = o.longs === String ? String(m.file_size) : m.file_size;
                                    else
                                        d.file_size = o.longs === String ? $util.Long.prototype.toString.call(m.file_size) : o.longs === Number ? new $util.LongBits(m.file_size.low >>> 0, m.file_size.high >>> 0).toNumber() : m.file_size;
                                    if (o.oneofs)
                                        d._file_size = "file_size";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MediaFile to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MediaFile
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaFile.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MediaFile;
                        })();
    
                        resources.MediaImage = (function() {
    
                            /**
                             * Properties of a MediaImage.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMediaImage
                             * @property {Uint8Array|null} [data] MediaImage data
                             * @property {string|null} [full_size_image_url] MediaImage full_size_image_url
                             * @property {string|null} [preview_size_image_url] MediaImage preview_size_image_url
                             */
    
                            /**
                             * Constructs a new MediaImage.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MediaImage.
                             * @implements IMediaImage
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMediaImage=} [p] Properties to set
                             */
                            function MediaImage(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MediaImage data.
                             * @member {Uint8Array|null|undefined} data
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @instance
                             */
                            MediaImage.prototype.data = null;
    
                            /**
                             * MediaImage full_size_image_url.
                             * @member {string|null|undefined} full_size_image_url
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @instance
                             */
                            MediaImage.prototype.full_size_image_url = null;
    
                            /**
                             * MediaImage preview_size_image_url.
                             * @member {string|null|undefined} preview_size_image_url
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @instance
                             */
                            MediaImage.prototype.preview_size_image_url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MediaImage _data.
                             * @member {"data"|undefined} _data
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @instance
                             */
                            Object.defineProperty(MediaImage.prototype, "_data", {
                                get: $util.oneOfGetter($oneOfFields = ["data"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaImage _full_size_image_url.
                             * @member {"full_size_image_url"|undefined} _full_size_image_url
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @instance
                             */
                            Object.defineProperty(MediaImage.prototype, "_full_size_image_url", {
                                get: $util.oneOfGetter($oneOfFields = ["full_size_image_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaImage _preview_size_image_url.
                             * @member {"preview_size_image_url"|undefined} _preview_size_image_url
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @instance
                             */
                            Object.defineProperty(MediaImage.prototype, "_preview_size_image_url", {
                                get: $util.oneOfGetter($oneOfFields = ["preview_size_image_url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MediaImage instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaImage=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MediaImage} MediaImage instance
                             */
                            MediaImage.create = function create(properties) {
                                return new MediaImage(properties);
                            };
    
                            /**
                             * Encodes the specified MediaImage message. Does not implicitly {@link google.ads.googleads.v11.resources.MediaImage.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaImage} m MediaImage message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaImage.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.full_size_image_url != null && Object.hasOwnProperty.call(m, "full_size_image_url"))
                                    w.uint32(18).string(m.full_size_image_url);
                                if (m.preview_size_image_url != null && Object.hasOwnProperty.call(m, "preview_size_image_url"))
                                    w.uint32(26).string(m.preview_size_image_url);
                                if (m.data != null && Object.hasOwnProperty.call(m, "data"))
                                    w.uint32(34).bytes(m.data);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaImage message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MediaImage.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaImage} message MediaImage message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaImage.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaImage message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MediaImage} MediaImage
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaImage.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MediaImage();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.data = r.bytes();
                                        break;
                                    case 2:
                                        m.full_size_image_url = r.string();
                                        break;
                                    case 3:
                                        m.preview_size_image_url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaImage message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MediaImage} MediaImage
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaImage.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaImage message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaImage.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    p._data = 1;
                                    if (!(m.data && typeof m.data.length === "number" || $util.isString(m.data)))
                                        return "data: buffer expected";
                                }
                                if (m.full_size_image_url != null && m.hasOwnProperty("full_size_image_url")) {
                                    p._full_size_image_url = 1;
                                    if (!$util.isString(m.full_size_image_url))
                                        return "full_size_image_url: string expected";
                                }
                                if (m.preview_size_image_url != null && m.hasOwnProperty("preview_size_image_url")) {
                                    p._preview_size_image_url = 1;
                                    if (!$util.isString(m.preview_size_image_url))
                                        return "preview_size_image_url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MediaImage message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MediaImage} MediaImage
                             */
                            MediaImage.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MediaImage)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MediaImage();
                                if (d.data != null) {
                                    if (typeof d.data === "string")
                                        $util.base64.decode(d.data, m.data = $util.newBuffer($util.base64.length(d.data)), 0);
                                    else if (d.data.length)
                                        m.data = d.data;
                                }
                                if (d.full_size_image_url != null) {
                                    m.full_size_image_url = String(d.full_size_image_url);
                                }
                                if (d.preview_size_image_url != null) {
                                    m.preview_size_image_url = String(d.preview_size_image_url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MediaImage message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @static
                             * @param {google.ads.googleads.v11.resources.MediaImage} m MediaImage
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaImage.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.full_size_image_url != null && m.hasOwnProperty("full_size_image_url")) {
                                    d.full_size_image_url = m.full_size_image_url;
                                    if (o.oneofs)
                                        d._full_size_image_url = "full_size_image_url";
                                }
                                if (m.preview_size_image_url != null && m.hasOwnProperty("preview_size_image_url")) {
                                    d.preview_size_image_url = m.preview_size_image_url;
                                    if (o.oneofs)
                                        d._preview_size_image_url = "preview_size_image_url";
                                }
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    d.data = o.bytes === String ? $util.base64.encode(m.data, 0, m.data.length) : o.bytes === Array ? Array.prototype.slice.call(m.data) : m.data;
                                    if (o.oneofs)
                                        d._data = "data";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MediaImage to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MediaImage
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaImage.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MediaImage;
                        })();
    
                        resources.MediaBundle = (function() {
    
                            /**
                             * Properties of a MediaBundle.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMediaBundle
                             * @property {Uint8Array|null} [data] MediaBundle data
                             * @property {string|null} [url] MediaBundle url
                             */
    
                            /**
                             * Constructs a new MediaBundle.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MediaBundle.
                             * @implements IMediaBundle
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMediaBundle=} [p] Properties to set
                             */
                            function MediaBundle(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MediaBundle data.
                             * @member {Uint8Array|null|undefined} data
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @instance
                             */
                            MediaBundle.prototype.data = null;
    
                            /**
                             * MediaBundle url.
                             * @member {string|null|undefined} url
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @instance
                             */
                            MediaBundle.prototype.url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MediaBundle _data.
                             * @member {"data"|undefined} _data
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @instance
                             */
                            Object.defineProperty(MediaBundle.prototype, "_data", {
                                get: $util.oneOfGetter($oneOfFields = ["data"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaBundle _url.
                             * @member {"url"|undefined} _url
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @instance
                             */
                            Object.defineProperty(MediaBundle.prototype, "_url", {
                                get: $util.oneOfGetter($oneOfFields = ["url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MediaBundle instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaBundle=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MediaBundle} MediaBundle instance
                             */
                            MediaBundle.create = function create(properties) {
                                return new MediaBundle(properties);
                            };
    
                            /**
                             * Encodes the specified MediaBundle message. Does not implicitly {@link google.ads.googleads.v11.resources.MediaBundle.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaBundle} m MediaBundle message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaBundle.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.url != null && Object.hasOwnProperty.call(m, "url"))
                                    w.uint32(18).string(m.url);
                                if (m.data != null && Object.hasOwnProperty.call(m, "data"))
                                    w.uint32(26).bytes(m.data);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaBundle message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MediaBundle.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaBundle} message MediaBundle message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaBundle.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaBundle message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MediaBundle} MediaBundle
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaBundle.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MediaBundle();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.data = r.bytes();
                                        break;
                                    case 2:
                                        m.url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaBundle message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MediaBundle} MediaBundle
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaBundle.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaBundle message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaBundle.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    p._data = 1;
                                    if (!(m.data && typeof m.data.length === "number" || $util.isString(m.data)))
                                        return "data: buffer expected";
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    p._url = 1;
                                    if (!$util.isString(m.url))
                                        return "url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MediaBundle message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MediaBundle} MediaBundle
                             */
                            MediaBundle.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MediaBundle)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MediaBundle();
                                if (d.data != null) {
                                    if (typeof d.data === "string")
                                        $util.base64.decode(d.data, m.data = $util.newBuffer($util.base64.length(d.data)), 0);
                                    else if (d.data.length)
                                        m.data = d.data;
                                }
                                if (d.url != null) {
                                    m.url = String(d.url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MediaBundle message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @static
                             * @param {google.ads.googleads.v11.resources.MediaBundle} m MediaBundle
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaBundle.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    d.url = m.url;
                                    if (o.oneofs)
                                        d._url = "url";
                                }
                                if (m.data != null && m.hasOwnProperty("data")) {
                                    d.data = o.bytes === String ? $util.base64.encode(m.data, 0, m.data.length) : o.bytes === Array ? Array.prototype.slice.call(m.data) : m.data;
                                    if (o.oneofs)
                                        d._data = "data";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MediaBundle to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MediaBundle
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaBundle.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MediaBundle;
                        })();
    
                        resources.MediaAudio = (function() {
    
                            /**
                             * Properties of a MediaAudio.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMediaAudio
                             * @property {number|null} [ad_duration_millis] MediaAudio ad_duration_millis
                             */
    
                            /**
                             * Constructs a new MediaAudio.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MediaAudio.
                             * @implements IMediaAudio
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMediaAudio=} [p] Properties to set
                             */
                            function MediaAudio(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MediaAudio ad_duration_millis.
                             * @member {number|null|undefined} ad_duration_millis
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @instance
                             */
                            MediaAudio.prototype.ad_duration_millis = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MediaAudio _ad_duration_millis.
                             * @member {"ad_duration_millis"|undefined} _ad_duration_millis
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @instance
                             */
                            Object.defineProperty(MediaAudio.prototype, "_ad_duration_millis", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_duration_millis"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MediaAudio instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaAudio=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MediaAudio} MediaAudio instance
                             */
                            MediaAudio.create = function create(properties) {
                                return new MediaAudio(properties);
                            };
    
                            /**
                             * Encodes the specified MediaAudio message. Does not implicitly {@link google.ads.googleads.v11.resources.MediaAudio.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaAudio} m MediaAudio message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaAudio.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_duration_millis != null && Object.hasOwnProperty.call(m, "ad_duration_millis"))
                                    w.uint32(16).int64(m.ad_duration_millis);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaAudio message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MediaAudio.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaAudio} message MediaAudio message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaAudio.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaAudio message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MediaAudio} MediaAudio
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaAudio.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MediaAudio();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.ad_duration_millis = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaAudio message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MediaAudio} MediaAudio
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaAudio.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaAudio message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaAudio.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.ad_duration_millis != null && m.hasOwnProperty("ad_duration_millis")) {
                                    p._ad_duration_millis = 1;
                                    if (!$util.isInteger(m.ad_duration_millis) && !(m.ad_duration_millis && $util.isInteger(m.ad_duration_millis.low) && $util.isInteger(m.ad_duration_millis.high)))
                                        return "ad_duration_millis: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MediaAudio message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MediaAudio} MediaAudio
                             */
                            MediaAudio.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MediaAudio)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MediaAudio();
                                if (d.ad_duration_millis != null) {
                                    if ($util.Long)
                                        (m.ad_duration_millis = $util.Long.fromValue(d.ad_duration_millis)).unsigned = false;
                                    else if (typeof d.ad_duration_millis === "string")
                                        m.ad_duration_millis = parseInt(d.ad_duration_millis, 10);
                                    else if (typeof d.ad_duration_millis === "number")
                                        m.ad_duration_millis = d.ad_duration_millis;
                                    else if (typeof d.ad_duration_millis === "object")
                                        m.ad_duration_millis = new $util.LongBits(d.ad_duration_millis.low >>> 0, d.ad_duration_millis.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MediaAudio message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @static
                             * @param {google.ads.googleads.v11.resources.MediaAudio} m MediaAudio
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaAudio.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.ad_duration_millis != null && m.hasOwnProperty("ad_duration_millis")) {
                                    if (typeof m.ad_duration_millis === "number")
                                        d.ad_duration_millis = o.longs === String ? String(m.ad_duration_millis) : m.ad_duration_millis;
                                    else
                                        d.ad_duration_millis = o.longs === String ? $util.Long.prototype.toString.call(m.ad_duration_millis) : o.longs === Number ? new $util.LongBits(m.ad_duration_millis.low >>> 0, m.ad_duration_millis.high >>> 0).toNumber() : m.ad_duration_millis;
                                    if (o.oneofs)
                                        d._ad_duration_millis = "ad_duration_millis";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MediaAudio to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MediaAudio
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaAudio.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MediaAudio;
                        })();
    
                        resources.MediaVideo = (function() {
    
                            /**
                             * Properties of a MediaVideo.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMediaVideo
                             * @property {number|null} [ad_duration_millis] MediaVideo ad_duration_millis
                             * @property {string|null} [youtube_video_id] MediaVideo youtube_video_id
                             * @property {string|null} [advertising_id_code] MediaVideo advertising_id_code
                             * @property {string|null} [isci_code] MediaVideo isci_code
                             */
    
                            /**
                             * Constructs a new MediaVideo.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MediaVideo.
                             * @implements IMediaVideo
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMediaVideo=} [p] Properties to set
                             */
                            function MediaVideo(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MediaVideo ad_duration_millis.
                             * @member {number|null|undefined} ad_duration_millis
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            MediaVideo.prototype.ad_duration_millis = null;
    
                            /**
                             * MediaVideo youtube_video_id.
                             * @member {string|null|undefined} youtube_video_id
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            MediaVideo.prototype.youtube_video_id = null;
    
                            /**
                             * MediaVideo advertising_id_code.
                             * @member {string|null|undefined} advertising_id_code
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            MediaVideo.prototype.advertising_id_code = null;
    
                            /**
                             * MediaVideo isci_code.
                             * @member {string|null|undefined} isci_code
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            MediaVideo.prototype.isci_code = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MediaVideo _ad_duration_millis.
                             * @member {"ad_duration_millis"|undefined} _ad_duration_millis
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            Object.defineProperty(MediaVideo.prototype, "_ad_duration_millis", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_duration_millis"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaVideo _youtube_video_id.
                             * @member {"youtube_video_id"|undefined} _youtube_video_id
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            Object.defineProperty(MediaVideo.prototype, "_youtube_video_id", {
                                get: $util.oneOfGetter($oneOfFields = ["youtube_video_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaVideo _advertising_id_code.
                             * @member {"advertising_id_code"|undefined} _advertising_id_code
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            Object.defineProperty(MediaVideo.prototype, "_advertising_id_code", {
                                get: $util.oneOfGetter($oneOfFields = ["advertising_id_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MediaVideo _isci_code.
                             * @member {"isci_code"|undefined} _isci_code
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             */
                            Object.defineProperty(MediaVideo.prototype, "_isci_code", {
                                get: $util.oneOfGetter($oneOfFields = ["isci_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MediaVideo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaVideo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MediaVideo} MediaVideo instance
                             */
                            MediaVideo.create = function create(properties) {
                                return new MediaVideo(properties);
                            };
    
                            /**
                             * Encodes the specified MediaVideo message. Does not implicitly {@link google.ads.googleads.v11.resources.MediaVideo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaVideo} m MediaVideo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaVideo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_duration_millis != null && Object.hasOwnProperty.call(m, "ad_duration_millis"))
                                    w.uint32(40).int64(m.ad_duration_millis);
                                if (m.youtube_video_id != null && Object.hasOwnProperty.call(m, "youtube_video_id"))
                                    w.uint32(50).string(m.youtube_video_id);
                                if (m.advertising_id_code != null && Object.hasOwnProperty.call(m, "advertising_id_code"))
                                    w.uint32(58).string(m.advertising_id_code);
                                if (m.isci_code != null && Object.hasOwnProperty.call(m, "isci_code"))
                                    w.uint32(66).string(m.isci_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaVideo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MediaVideo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMediaVideo} message MediaVideo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaVideo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaVideo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MediaVideo} MediaVideo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaVideo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MediaVideo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.ad_duration_millis = r.int64();
                                        break;
                                    case 6:
                                        m.youtube_video_id = r.string();
                                        break;
                                    case 7:
                                        m.advertising_id_code = r.string();
                                        break;
                                    case 8:
                                        m.isci_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaVideo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MediaVideo} MediaVideo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaVideo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaVideo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaVideo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.ad_duration_millis != null && m.hasOwnProperty("ad_duration_millis")) {
                                    p._ad_duration_millis = 1;
                                    if (!$util.isInteger(m.ad_duration_millis) && !(m.ad_duration_millis && $util.isInteger(m.ad_duration_millis.low) && $util.isInteger(m.ad_duration_millis.high)))
                                        return "ad_duration_millis: integer|Long expected";
                                }
                                if (m.youtube_video_id != null && m.hasOwnProperty("youtube_video_id")) {
                                    p._youtube_video_id = 1;
                                    if (!$util.isString(m.youtube_video_id))
                                        return "youtube_video_id: string expected";
                                }
                                if (m.advertising_id_code != null && m.hasOwnProperty("advertising_id_code")) {
                                    p._advertising_id_code = 1;
                                    if (!$util.isString(m.advertising_id_code))
                                        return "advertising_id_code: string expected";
                                }
                                if (m.isci_code != null && m.hasOwnProperty("isci_code")) {
                                    p._isci_code = 1;
                                    if (!$util.isString(m.isci_code))
                                        return "isci_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MediaVideo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MediaVideo} MediaVideo
                             */
                            MediaVideo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MediaVideo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MediaVideo();
                                if (d.ad_duration_millis != null) {
                                    if ($util.Long)
                                        (m.ad_duration_millis = $util.Long.fromValue(d.ad_duration_millis)).unsigned = false;
                                    else if (typeof d.ad_duration_millis === "string")
                                        m.ad_duration_millis = parseInt(d.ad_duration_millis, 10);
                                    else if (typeof d.ad_duration_millis === "number")
                                        m.ad_duration_millis = d.ad_duration_millis;
                                    else if (typeof d.ad_duration_millis === "object")
                                        m.ad_duration_millis = new $util.LongBits(d.ad_duration_millis.low >>> 0, d.ad_duration_millis.high >>> 0).toNumber();
                                }
                                if (d.youtube_video_id != null) {
                                    m.youtube_video_id = String(d.youtube_video_id);
                                }
                                if (d.advertising_id_code != null) {
                                    m.advertising_id_code = String(d.advertising_id_code);
                                }
                                if (d.isci_code != null) {
                                    m.isci_code = String(d.isci_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MediaVideo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @static
                             * @param {google.ads.googleads.v11.resources.MediaVideo} m MediaVideo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaVideo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.ad_duration_millis != null && m.hasOwnProperty("ad_duration_millis")) {
                                    if (typeof m.ad_duration_millis === "number")
                                        d.ad_duration_millis = o.longs === String ? String(m.ad_duration_millis) : m.ad_duration_millis;
                                    else
                                        d.ad_duration_millis = o.longs === String ? $util.Long.prototype.toString.call(m.ad_duration_millis) : o.longs === Number ? new $util.LongBits(m.ad_duration_millis.low >>> 0, m.ad_duration_millis.high >>> 0).toNumber() : m.ad_duration_millis;
                                    if (o.oneofs)
                                        d._ad_duration_millis = "ad_duration_millis";
                                }
                                if (m.youtube_video_id != null && m.hasOwnProperty("youtube_video_id")) {
                                    d.youtube_video_id = m.youtube_video_id;
                                    if (o.oneofs)
                                        d._youtube_video_id = "youtube_video_id";
                                }
                                if (m.advertising_id_code != null && m.hasOwnProperty("advertising_id_code")) {
                                    d.advertising_id_code = m.advertising_id_code;
                                    if (o.oneofs)
                                        d._advertising_id_code = "advertising_id_code";
                                }
                                if (m.isci_code != null && m.hasOwnProperty("isci_code")) {
                                    d.isci_code = m.isci_code;
                                    if (o.oneofs)
                                        d._isci_code = "isci_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MediaVideo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MediaVideo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaVideo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MediaVideo;
                        })();
    
                        resources.MerchantCenterLink = (function() {
    
                            /**
                             * Properties of a MerchantCenterLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMerchantCenterLink
                             * @property {string|null} [resource_name] MerchantCenterLink resource_name
                             * @property {number|null} [id] MerchantCenterLink id
                             * @property {string|null} [merchant_center_account_name] MerchantCenterLink merchant_center_account_name
                             * @property {google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum.MerchantCenterLinkStatus|null} [status] MerchantCenterLink status
                             */
    
                            /**
                             * Constructs a new MerchantCenterLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MerchantCenterLink.
                             * @implements IMerchantCenterLink
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMerchantCenterLink=} [p] Properties to set
                             */
                            function MerchantCenterLink(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MerchantCenterLink resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @instance
                             */
                            MerchantCenterLink.prototype.resource_name = "";
    
                            /**
                             * MerchantCenterLink id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @instance
                             */
                            MerchantCenterLink.prototype.id = null;
    
                            /**
                             * MerchantCenterLink merchant_center_account_name.
                             * @member {string|null|undefined} merchant_center_account_name
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @instance
                             */
                            MerchantCenterLink.prototype.merchant_center_account_name = null;
    
                            /**
                             * MerchantCenterLink status.
                             * @member {google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum.MerchantCenterLinkStatus} status
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @instance
                             */
                            MerchantCenterLink.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MerchantCenterLink _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @instance
                             */
                            Object.defineProperty(MerchantCenterLink.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MerchantCenterLink _merchant_center_account_name.
                             * @member {"merchant_center_account_name"|undefined} _merchant_center_account_name
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @instance
                             */
                            Object.defineProperty(MerchantCenterLink.prototype, "_merchant_center_account_name", {
                                get: $util.oneOfGetter($oneOfFields = ["merchant_center_account_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MerchantCenterLink instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMerchantCenterLink=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MerchantCenterLink} MerchantCenterLink instance
                             */
                            MerchantCenterLink.create = function create(properties) {
                                return new MerchantCenterLink(properties);
                            };
    
                            /**
                             * Encodes the specified MerchantCenterLink message. Does not implicitly {@link google.ads.googleads.v11.resources.MerchantCenterLink.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMerchantCenterLink} m MerchantCenterLink message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterLink.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(48).int64(m.id);
                                if (m.merchant_center_account_name != null && Object.hasOwnProperty.call(m, "merchant_center_account_name"))
                                    w.uint32(58).string(m.merchant_center_account_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MerchantCenterLink message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MerchantCenterLink.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMerchantCenterLink} message MerchantCenterLink message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterLink.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MerchantCenterLink message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MerchantCenterLink} MerchantCenterLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterLink.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MerchantCenterLink();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.id = r.int64();
                                        break;
                                    case 7:
                                        m.merchant_center_account_name = r.string();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MerchantCenterLink message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MerchantCenterLink} MerchantCenterLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterLink.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MerchantCenterLink message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MerchantCenterLink.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.merchant_center_account_name != null && m.hasOwnProperty("merchant_center_account_name")) {
                                    p._merchant_center_account_name = 1;
                                    if (!$util.isString(m.merchant_center_account_name))
                                        return "merchant_center_account_name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MerchantCenterLink message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MerchantCenterLink} MerchantCenterLink
                             */
                            MerchantCenterLink.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MerchantCenterLink)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MerchantCenterLink();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.merchant_center_account_name != null) {
                                    m.merchant_center_account_name = String(d.merchant_center_account_name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "PENDING":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MerchantCenterLink message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.MerchantCenterLink} m MerchantCenterLink
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MerchantCenterLink.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.MerchantCenterLinkStatusEnum.MerchantCenterLinkStatus[m.status] : m.status;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.merchant_center_account_name != null && m.hasOwnProperty("merchant_center_account_name")) {
                                    d.merchant_center_account_name = m.merchant_center_account_name;
                                    if (o.oneofs)
                                        d._merchant_center_account_name = "merchant_center_account_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MerchantCenterLink to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MerchantCenterLink
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MerchantCenterLink.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MerchantCenterLink;
                        })();
    
                        resources.MobileAppCategoryConstant = (function() {
    
                            /**
                             * Properties of a MobileAppCategoryConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMobileAppCategoryConstant
                             * @property {string|null} [resource_name] MobileAppCategoryConstant resource_name
                             * @property {number|null} [id] MobileAppCategoryConstant id
                             * @property {string|null} [name] MobileAppCategoryConstant name
                             */
    
                            /**
                             * Constructs a new MobileAppCategoryConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MobileAppCategoryConstant.
                             * @implements IMobileAppCategoryConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMobileAppCategoryConstant=} [p] Properties to set
                             */
                            function MobileAppCategoryConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MobileAppCategoryConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @instance
                             */
                            MobileAppCategoryConstant.prototype.resource_name = "";
    
                            /**
                             * MobileAppCategoryConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @instance
                             */
                            MobileAppCategoryConstant.prototype.id = null;
    
                            /**
                             * MobileAppCategoryConstant name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @instance
                             */
                            MobileAppCategoryConstant.prototype.name = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MobileAppCategoryConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @instance
                             */
                            Object.defineProperty(MobileAppCategoryConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MobileAppCategoryConstant _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @instance
                             */
                            Object.defineProperty(MobileAppCategoryConstant.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MobileAppCategoryConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMobileAppCategoryConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MobileAppCategoryConstant} MobileAppCategoryConstant instance
                             */
                            MobileAppCategoryConstant.create = function create(properties) {
                                return new MobileAppCategoryConstant(properties);
                            };
    
                            /**
                             * Encodes the specified MobileAppCategoryConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.MobileAppCategoryConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMobileAppCategoryConstant} m MobileAppCategoryConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppCategoryConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(32).int32(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(42).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileAppCategoryConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MobileAppCategoryConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMobileAppCategoryConstant} message MobileAppCategoryConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileAppCategoryConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileAppCategoryConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MobileAppCategoryConstant} MobileAppCategoryConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppCategoryConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.id = r.int32();
                                        break;
                                    case 5:
                                        m.name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileAppCategoryConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MobileAppCategoryConstant} MobileAppCategoryConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileAppCategoryConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileAppCategoryConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileAppCategoryConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id))
                                        return "id: integer expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MobileAppCategoryConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MobileAppCategoryConstant} MobileAppCategoryConstant
                             */
                            MobileAppCategoryConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    m.id = d.id | 0;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MobileAppCategoryConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.MobileAppCategoryConstant} m MobileAppCategoryConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileAppCategoryConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    d.id = m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MobileAppCategoryConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MobileAppCategoryConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileAppCategoryConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MobileAppCategoryConstant;
                        })();
    
                        resources.MobileDeviceConstant = (function() {
    
                            /**
                             * Properties of a MobileDeviceConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IMobileDeviceConstant
                             * @property {string|null} [resource_name] MobileDeviceConstant resource_name
                             * @property {number|null} [id] MobileDeviceConstant id
                             * @property {string|null} [name] MobileDeviceConstant name
                             * @property {string|null} [manufacturer_name] MobileDeviceConstant manufacturer_name
                             * @property {string|null} [operating_system_name] MobileDeviceConstant operating_system_name
                             * @property {google.ads.googleads.v11.enums.MobileDeviceTypeEnum.MobileDeviceType|null} [type] MobileDeviceConstant type
                             */
    
                            /**
                             * Constructs a new MobileDeviceConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a MobileDeviceConstant.
                             * @implements IMobileDeviceConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IMobileDeviceConstant=} [p] Properties to set
                             */
                            function MobileDeviceConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MobileDeviceConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            MobileDeviceConstant.prototype.resource_name = "";
    
                            /**
                             * MobileDeviceConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            MobileDeviceConstant.prototype.id = null;
    
                            /**
                             * MobileDeviceConstant name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            MobileDeviceConstant.prototype.name = null;
    
                            /**
                             * MobileDeviceConstant manufacturer_name.
                             * @member {string|null|undefined} manufacturer_name
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            MobileDeviceConstant.prototype.manufacturer_name = null;
    
                            /**
                             * MobileDeviceConstant operating_system_name.
                             * @member {string|null|undefined} operating_system_name
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            MobileDeviceConstant.prototype.operating_system_name = null;
    
                            /**
                             * MobileDeviceConstant type.
                             * @member {google.ads.googleads.v11.enums.MobileDeviceTypeEnum.MobileDeviceType} type
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            MobileDeviceConstant.prototype.type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MobileDeviceConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            Object.defineProperty(MobileDeviceConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MobileDeviceConstant _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            Object.defineProperty(MobileDeviceConstant.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MobileDeviceConstant _manufacturer_name.
                             * @member {"manufacturer_name"|undefined} _manufacturer_name
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            Object.defineProperty(MobileDeviceConstant.prototype, "_manufacturer_name", {
                                get: $util.oneOfGetter($oneOfFields = ["manufacturer_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * MobileDeviceConstant _operating_system_name.
                             * @member {"operating_system_name"|undefined} _operating_system_name
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             */
                            Object.defineProperty(MobileDeviceConstant.prototype, "_operating_system_name", {
                                get: $util.oneOfGetter($oneOfFields = ["operating_system_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MobileDeviceConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMobileDeviceConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.MobileDeviceConstant} MobileDeviceConstant instance
                             */
                            MobileDeviceConstant.create = function create(properties) {
                                return new MobileDeviceConstant(properties);
                            };
    
                            /**
                             * Encodes the specified MobileDeviceConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.MobileDeviceConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMobileDeviceConstant} m MobileDeviceConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileDeviceConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(48).int32(m.type);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(56).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(66).string(m.name);
                                if (m.manufacturer_name != null && Object.hasOwnProperty.call(m, "manufacturer_name"))
                                    w.uint32(74).string(m.manufacturer_name);
                                if (m.operating_system_name != null && Object.hasOwnProperty.call(m, "operating_system_name"))
                                    w.uint32(82).string(m.operating_system_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MobileDeviceConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.MobileDeviceConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IMobileDeviceConstant} message MobileDeviceConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MobileDeviceConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MobileDeviceConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.MobileDeviceConstant} MobileDeviceConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileDeviceConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.MobileDeviceConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.id = r.int64();
                                        break;
                                    case 8:
                                        m.name = r.string();
                                        break;
                                    case 9:
                                        m.manufacturer_name = r.string();
                                        break;
                                    case 10:
                                        m.operating_system_name = r.string();
                                        break;
                                    case 6:
                                        m.type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MobileDeviceConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.MobileDeviceConstant} MobileDeviceConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MobileDeviceConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MobileDeviceConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MobileDeviceConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.manufacturer_name != null && m.hasOwnProperty("manufacturer_name")) {
                                    p._manufacturer_name = 1;
                                    if (!$util.isString(m.manufacturer_name))
                                        return "manufacturer_name: string expected";
                                }
                                if (m.operating_system_name != null && m.hasOwnProperty("operating_system_name")) {
                                    p._operating_system_name = 1;
                                    if (!$util.isString(m.operating_system_name))
                                        return "operating_system_name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MobileDeviceConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.MobileDeviceConstant} MobileDeviceConstant
                             */
                            MobileDeviceConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.MobileDeviceConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.MobileDeviceConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.manufacturer_name != null) {
                                    m.manufacturer_name = String(d.manufacturer_name);
                                }
                                if (d.operating_system_name != null) {
                                    m.operating_system_name = String(d.operating_system_name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "MOBILE":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "TABLET":
                                case 3:
                                    m.type = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MobileDeviceConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.MobileDeviceConstant} m MobileDeviceConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MobileDeviceConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.MobileDeviceTypeEnum.MobileDeviceType[m.type] : m.type;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.manufacturer_name != null && m.hasOwnProperty("manufacturer_name")) {
                                    d.manufacturer_name = m.manufacturer_name;
                                    if (o.oneofs)
                                        d._manufacturer_name = "manufacturer_name";
                                }
                                if (m.operating_system_name != null && m.hasOwnProperty("operating_system_name")) {
                                    d.operating_system_name = m.operating_system_name;
                                    if (o.oneofs)
                                        d._operating_system_name = "operating_system_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MobileDeviceConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.MobileDeviceConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MobileDeviceConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MobileDeviceConstant;
                        })();
    
                        resources.OfflineUserDataJob = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJob.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IOfflineUserDataJob
                             * @property {string|null} [resource_name] OfflineUserDataJob resource_name
                             * @property {number|null} [id] OfflineUserDataJob id
                             * @property {number|null} [external_id] OfflineUserDataJob external_id
                             * @property {google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum.OfflineUserDataJobType|null} [type] OfflineUserDataJob type
                             * @property {google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus|null} [status] OfflineUserDataJob status
                             * @property {google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason|null} [failure_reason] OfflineUserDataJob failure_reason
                             * @property {google.ads.googleads.v11.resources.IOfflineUserDataJobMetadata|null} [operation_metadata] OfflineUserDataJob operation_metadata
                             * @property {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata|null} [customer_match_user_list_metadata] OfflineUserDataJob customer_match_user_list_metadata
                             * @property {google.ads.googleads.v11.common.IStoreSalesMetadata|null} [store_sales_metadata] OfflineUserDataJob store_sales_metadata
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJob.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an OfflineUserDataJob.
                             * @implements IOfflineUserDataJob
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJob=} [p] Properties to set
                             */
                            function OfflineUserDataJob(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * OfflineUserDataJob resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.resource_name = "";
    
                            /**
                             * OfflineUserDataJob id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.id = null;
    
                            /**
                             * OfflineUserDataJob external_id.
                             * @member {number|null|undefined} external_id
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.external_id = null;
    
                            /**
                             * OfflineUserDataJob type.
                             * @member {google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum.OfflineUserDataJobType} type
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.type = 0;
    
                            /**
                             * OfflineUserDataJob status.
                             * @member {google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus} status
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.status = 0;
    
                            /**
                             * OfflineUserDataJob failure_reason.
                             * @member {google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason} failure_reason
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.failure_reason = 0;
    
                            /**
                             * OfflineUserDataJob operation_metadata.
                             * @member {google.ads.googleads.v11.resources.IOfflineUserDataJobMetadata|null|undefined} operation_metadata
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.operation_metadata = null;
    
                            /**
                             * OfflineUserDataJob customer_match_user_list_metadata.
                             * @member {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata|null|undefined} customer_match_user_list_metadata
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.customer_match_user_list_metadata = null;
    
                            /**
                             * OfflineUserDataJob store_sales_metadata.
                             * @member {google.ads.googleads.v11.common.IStoreSalesMetadata|null|undefined} store_sales_metadata
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            OfflineUserDataJob.prototype.store_sales_metadata = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * OfflineUserDataJob _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            Object.defineProperty(OfflineUserDataJob.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserDataJob _external_id.
                             * @member {"external_id"|undefined} _external_id
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            Object.defineProperty(OfflineUserDataJob.prototype, "_external_id", {
                                get: $util.oneOfGetter($oneOfFields = ["external_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OfflineUserDataJob metadata.
                             * @member {"customer_match_user_list_metadata"|"store_sales_metadata"|undefined} metadata
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             */
                            Object.defineProperty(OfflineUserDataJob.prototype, "metadata", {
                                get: $util.oneOfGetter($oneOfFields = ["customer_match_user_list_metadata", "store_sales_metadata"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new OfflineUserDataJob instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJob=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJob} OfflineUserDataJob instance
                             */
                            OfflineUserDataJob.create = function create(properties) {
                                return new OfflineUserDataJob(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJob message. Does not implicitly {@link google.ads.googleads.v11.resources.OfflineUserDataJob.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJob} m OfflineUserDataJob message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJob.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.failure_reason != null && Object.hasOwnProperty.call(m, "failure_reason"))
                                    w.uint32(48).int32(m.failure_reason);
                                if (m.customer_match_user_list_metadata != null && Object.hasOwnProperty.call(m, "customer_match_user_list_metadata"))
                                    $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.encode(m.customer_match_user_list_metadata, w.uint32(58).fork()).ldelim();
                                if (m.store_sales_metadata != null && Object.hasOwnProperty.call(m, "store_sales_metadata"))
                                    $root.google.ads.googleads.v11.common.StoreSalesMetadata.encode(m.store_sales_metadata, w.uint32(66).fork()).ldelim();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(72).int64(m.id);
                                if (m.external_id != null && Object.hasOwnProperty.call(m, "external_id"))
                                    w.uint32(80).int64(m.external_id);
                                if (m.operation_metadata != null && Object.hasOwnProperty.call(m, "operation_metadata"))
                                    $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata.encode(m.operation_metadata, w.uint32(90).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJob message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.OfflineUserDataJob.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJob} message OfflineUserDataJob message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJob.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJob message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJob} OfflineUserDataJob
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJob.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.OfflineUserDataJob();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 9:
                                        m.id = r.int64();
                                        break;
                                    case 10:
                                        m.external_id = r.int64();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 6:
                                        m.failure_reason = r.int32();
                                        break;
                                    case 11:
                                        m.operation_metadata = $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.customer_match_user_list_metadata = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.store_sales_metadata = $root.google.ads.googleads.v11.common.StoreSalesMetadata.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJob message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJob} OfflineUserDataJob
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJob.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJob message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJob.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.external_id != null && m.hasOwnProperty("external_id")) {
                                    p._external_id = 1;
                                    if (!$util.isInteger(m.external_id) && !(m.external_id && $util.isInteger(m.external_id.low) && $util.isInteger(m.external_id.high)))
                                        return "external_id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.failure_reason != null && m.hasOwnProperty("failure_reason")) {
                                    switch (m.failure_reason) {
                                    default:
                                        return "failure_reason: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.operation_metadata != null && m.hasOwnProperty("operation_metadata")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata.verify(m.operation_metadata);
                                        if (e)
                                            return "operation_metadata." + e;
                                    }
                                }
                                if (m.customer_match_user_list_metadata != null && m.hasOwnProperty("customer_match_user_list_metadata")) {
                                    p.metadata = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.verify(m.customer_match_user_list_metadata);
                                        if (e)
                                            return "customer_match_user_list_metadata." + e;
                                    }
                                }
                                if (m.store_sales_metadata != null && m.hasOwnProperty("store_sales_metadata")) {
                                    if (p.metadata === 1)
                                        return "metadata: multiple values";
                                    p.metadata = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.StoreSalesMetadata.verify(m.store_sales_metadata);
                                        if (e)
                                            return "store_sales_metadata." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJob message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJob} OfflineUserDataJob
                             */
                            OfflineUserDataJob.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.OfflineUserDataJob)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.OfflineUserDataJob();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.external_id != null) {
                                    if ($util.Long)
                                        (m.external_id = $util.Long.fromValue(d.external_id)).unsigned = false;
                                    else if (typeof d.external_id === "string")
                                        m.external_id = parseInt(d.external_id, 10);
                                    else if (typeof d.external_id === "number")
                                        m.external_id = d.external_id;
                                    else if (typeof d.external_id === "object")
                                        m.external_id = new $util.LongBits(d.external_id.low >>> 0, d.external_id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "STORE_SALES_UPLOAD_FIRST_PARTY":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "STORE_SALES_UPLOAD_THIRD_PARTY":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "CUSTOMER_MATCH_USER_LIST":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "CUSTOMER_MATCH_WITH_ATTRIBUTES":
                                case 5:
                                    m.type = 5;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "PENDING":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "RUNNING":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "SUCCESS":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "FAILED":
                                case 5:
                                    m.status = 5;
                                    break;
                                }
                                switch (d.failure_reason) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.failure_reason = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.failure_reason = 1;
                                    break;
                                case "INSUFFICIENT_MATCHED_TRANSACTIONS":
                                case 2:
                                    m.failure_reason = 2;
                                    break;
                                case "INSUFFICIENT_TRANSACTIONS":
                                case 3:
                                    m.failure_reason = 3;
                                    break;
                                }
                                if (d.operation_metadata != null) {
                                    if (typeof d.operation_metadata !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.OfflineUserDataJob.operation_metadata: object expected");
                                    m.operation_metadata = $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata.fromObject(d.operation_metadata);
                                }
                                if (d.customer_match_user_list_metadata != null) {
                                    if (typeof d.customer_match_user_list_metadata !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.OfflineUserDataJob.customer_match_user_list_metadata: object expected");
                                    m.customer_match_user_list_metadata = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.fromObject(d.customer_match_user_list_metadata);
                                }
                                if (d.store_sales_metadata != null) {
                                    if (typeof d.store_sales_metadata !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.OfflineUserDataJob.store_sales_metadata: object expected");
                                    m.store_sales_metadata = $root.google.ads.googleads.v11.common.StoreSalesMetadata.fromObject(d.store_sales_metadata);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJob message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @static
                             * @param {google.ads.googleads.v11.resources.OfflineUserDataJob} m OfflineUserDataJob
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJob.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.failure_reason = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.operation_metadata = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.OfflineUserDataJobTypeEnum.OfflineUserDataJobType[m.type] : m.type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus[m.status] : m.status;
                                }
                                if (m.failure_reason != null && m.hasOwnProperty("failure_reason")) {
                                    d.failure_reason = o.enums === String ? $root.google.ads.googleads.v11.enums.OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason[m.failure_reason] : m.failure_reason;
                                }
                                if (m.customer_match_user_list_metadata != null && m.hasOwnProperty("customer_match_user_list_metadata")) {
                                    d.customer_match_user_list_metadata = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.toObject(m.customer_match_user_list_metadata, o);
                                    if (o.oneofs)
                                        d.metadata = "customer_match_user_list_metadata";
                                }
                                if (m.store_sales_metadata != null && m.hasOwnProperty("store_sales_metadata")) {
                                    d.store_sales_metadata = $root.google.ads.googleads.v11.common.StoreSalesMetadata.toObject(m.store_sales_metadata, o);
                                    if (o.oneofs)
                                        d.metadata = "store_sales_metadata";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.external_id != null && m.hasOwnProperty("external_id")) {
                                    if (typeof m.external_id === "number")
                                        d.external_id = o.longs === String ? String(m.external_id) : m.external_id;
                                    else
                                        d.external_id = o.longs === String ? $util.Long.prototype.toString.call(m.external_id) : o.longs === Number ? new $util.LongBits(m.external_id.low >>> 0, m.external_id.high >>> 0).toNumber() : m.external_id;
                                    if (o.oneofs)
                                        d._external_id = "external_id";
                                }
                                if (m.operation_metadata != null && m.hasOwnProperty("operation_metadata")) {
                                    d.operation_metadata = $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata.toObject(m.operation_metadata, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this OfflineUserDataJob to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJob
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJob.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return OfflineUserDataJob;
                        })();
    
                        resources.OfflineUserDataJobMetadata = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJobMetadata.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IOfflineUserDataJobMetadata
                             * @property {google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange|null} [match_rate_range] OfflineUserDataJobMetadata match_rate_range
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJobMetadata.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an OfflineUserDataJobMetadata.
                             * @implements IOfflineUserDataJobMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJobMetadata=} [p] Properties to set
                             */
                            function OfflineUserDataJobMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * OfflineUserDataJobMetadata match_rate_range.
                             * @member {google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange} match_rate_range
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @instance
                             */
                            OfflineUserDataJobMetadata.prototype.match_rate_range = 0;
    
                            /**
                             * Creates a new OfflineUserDataJobMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJobMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJobMetadata} OfflineUserDataJobMetadata instance
                             */
                            OfflineUserDataJobMetadata.create = function create(properties) {
                                return new OfflineUserDataJobMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobMetadata message. Does not implicitly {@link google.ads.googleads.v11.resources.OfflineUserDataJobMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJobMetadata} m OfflineUserDataJobMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.match_rate_range != null && Object.hasOwnProperty.call(m, "match_rate_range"))
                                    w.uint32(8).int32(m.match_rate_range);
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.OfflineUserDataJobMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOfflineUserDataJobMetadata} message OfflineUserDataJobMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJobMetadata} OfflineUserDataJobMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.match_rate_range = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJobMetadata} OfflineUserDataJobMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJobMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJobMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.match_rate_range != null && m.hasOwnProperty("match_rate_range")) {
                                    switch (m.match_rate_range) {
                                    default:
                                        return "match_rate_range: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJobMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.OfflineUserDataJobMetadata} OfflineUserDataJobMetadata
                             */
                            OfflineUserDataJobMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.OfflineUserDataJobMetadata();
                                switch (d.match_rate_range) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.match_rate_range = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.match_rate_range = 1;
                                    break;
                                case "MATCH_RANGE_LESS_THAN_20":
                                case 2:
                                    m.match_rate_range = 2;
                                    break;
                                case "MATCH_RANGE_20_TO_30":
                                case 3:
                                    m.match_rate_range = 3;
                                    break;
                                case "MATCH_RANGE_31_TO_40":
                                case 4:
                                    m.match_rate_range = 4;
                                    break;
                                case "MATCH_RANGE_41_TO_50":
                                case 5:
                                    m.match_rate_range = 5;
                                    break;
                                case "MATCH_RANGE_51_TO_60":
                                case 6:
                                    m.match_rate_range = 6;
                                    break;
                                case "MATCH_RANGE_61_TO_70":
                                case 7:
                                    m.match_rate_range = 7;
                                    break;
                                case "MATCH_RANGE_71_TO_80":
                                case 8:
                                    m.match_rate_range = 8;
                                    break;
                                case "MATCH_RANGE_81_TO_90":
                                case 9:
                                    m.match_rate_range = 9;
                                    break;
                                case "MATCH_RANGE_91_TO_100":
                                case 10:
                                    m.match_rate_range = 10;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJobMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @static
                             * @param {google.ads.googleads.v11.resources.OfflineUserDataJobMetadata} m OfflineUserDataJobMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJobMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.match_rate_range = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.match_rate_range != null && m.hasOwnProperty("match_rate_range")) {
                                    d.match_rate_range = o.enums === String ? $root.google.ads.googleads.v11.enums.OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange[m.match_rate_range] : m.match_rate_range;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this OfflineUserDataJobMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.OfflineUserDataJobMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJobMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return OfflineUserDataJobMetadata;
                        })();
    
                        resources.OperatingSystemVersionConstant = (function() {
    
                            /**
                             * Properties of an OperatingSystemVersionConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IOperatingSystemVersionConstant
                             * @property {string|null} [resource_name] OperatingSystemVersionConstant resource_name
                             * @property {number|null} [id] OperatingSystemVersionConstant id
                             * @property {string|null} [name] OperatingSystemVersionConstant name
                             * @property {number|null} [os_major_version] OperatingSystemVersionConstant os_major_version
                             * @property {number|null} [os_minor_version] OperatingSystemVersionConstant os_minor_version
                             * @property {google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType|null} [operator_type] OperatingSystemVersionConstant operator_type
                             */
    
                            /**
                             * Constructs a new OperatingSystemVersionConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents an OperatingSystemVersionConstant.
                             * @implements IOperatingSystemVersionConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IOperatingSystemVersionConstant=} [p] Properties to set
                             */
                            function OperatingSystemVersionConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * OperatingSystemVersionConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            OperatingSystemVersionConstant.prototype.resource_name = "";
    
                            /**
                             * OperatingSystemVersionConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            OperatingSystemVersionConstant.prototype.id = null;
    
                            /**
                             * OperatingSystemVersionConstant name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            OperatingSystemVersionConstant.prototype.name = null;
    
                            /**
                             * OperatingSystemVersionConstant os_major_version.
                             * @member {number|null|undefined} os_major_version
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            OperatingSystemVersionConstant.prototype.os_major_version = null;
    
                            /**
                             * OperatingSystemVersionConstant os_minor_version.
                             * @member {number|null|undefined} os_minor_version
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            OperatingSystemVersionConstant.prototype.os_minor_version = null;
    
                            /**
                             * OperatingSystemVersionConstant operator_type.
                             * @member {google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType} operator_type
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            OperatingSystemVersionConstant.prototype.operator_type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * OperatingSystemVersionConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            Object.defineProperty(OperatingSystemVersionConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OperatingSystemVersionConstant _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            Object.defineProperty(OperatingSystemVersionConstant.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OperatingSystemVersionConstant _os_major_version.
                             * @member {"os_major_version"|undefined} _os_major_version
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            Object.defineProperty(OperatingSystemVersionConstant.prototype, "_os_major_version", {
                                get: $util.oneOfGetter($oneOfFields = ["os_major_version"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OperatingSystemVersionConstant _os_minor_version.
                             * @member {"os_minor_version"|undefined} _os_minor_version
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             */
                            Object.defineProperty(OperatingSystemVersionConstant.prototype, "_os_minor_version", {
                                get: $util.oneOfGetter($oneOfFields = ["os_minor_version"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new OperatingSystemVersionConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOperatingSystemVersionConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.OperatingSystemVersionConstant} OperatingSystemVersionConstant instance
                             */
                            OperatingSystemVersionConstant.create = function create(properties) {
                                return new OperatingSystemVersionConstant(properties);
                            };
    
                            /**
                             * Encodes the specified OperatingSystemVersionConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.OperatingSystemVersionConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOperatingSystemVersionConstant} m OperatingSystemVersionConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatingSystemVersionConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.operator_type != null && Object.hasOwnProperty.call(m, "operator_type"))
                                    w.uint32(48).int32(m.operator_type);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(56).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(66).string(m.name);
                                if (m.os_major_version != null && Object.hasOwnProperty.call(m, "os_major_version"))
                                    w.uint32(72).int32(m.os_major_version);
                                if (m.os_minor_version != null && Object.hasOwnProperty.call(m, "os_minor_version"))
                                    w.uint32(80).int32(m.os_minor_version);
                                return w;
                            };
    
                            /**
                             * Encodes the specified OperatingSystemVersionConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.OperatingSystemVersionConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IOperatingSystemVersionConstant} message OperatingSystemVersionConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OperatingSystemVersionConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OperatingSystemVersionConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.OperatingSystemVersionConstant} OperatingSystemVersionConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatingSystemVersionConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 7:
                                        m.id = r.int64();
                                        break;
                                    case 8:
                                        m.name = r.string();
                                        break;
                                    case 9:
                                        m.os_major_version = r.int32();
                                        break;
                                    case 10:
                                        m.os_minor_version = r.int32();
                                        break;
                                    case 6:
                                        m.operator_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OperatingSystemVersionConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.OperatingSystemVersionConstant} OperatingSystemVersionConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OperatingSystemVersionConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OperatingSystemVersionConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OperatingSystemVersionConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.os_major_version != null && m.hasOwnProperty("os_major_version")) {
                                    p._os_major_version = 1;
                                    if (!$util.isInteger(m.os_major_version))
                                        return "os_major_version: integer expected";
                                }
                                if (m.os_minor_version != null && m.hasOwnProperty("os_minor_version")) {
                                    p._os_minor_version = 1;
                                    if (!$util.isInteger(m.os_minor_version))
                                        return "os_minor_version: integer expected";
                                }
                                if (m.operator_type != null && m.hasOwnProperty("operator_type")) {
                                    switch (m.operator_type) {
                                    default:
                                        return "operator_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an OperatingSystemVersionConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.OperatingSystemVersionConstant} OperatingSystemVersionConstant
                             */
                            OperatingSystemVersionConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.os_major_version != null) {
                                    m.os_major_version = d.os_major_version | 0;
                                }
                                if (d.os_minor_version != null) {
                                    m.os_minor_version = d.os_minor_version | 0;
                                }
                                switch (d.operator_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.operator_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.operator_type = 1;
                                    break;
                                case "EQUALS_TO":
                                case 2:
                                    m.operator_type = 2;
                                    break;
                                case "GREATER_THAN_EQUALS_TO":
                                case 4:
                                    m.operator_type = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an OperatingSystemVersionConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.OperatingSystemVersionConstant} m OperatingSystemVersionConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OperatingSystemVersionConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.operator_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.operator_type != null && m.hasOwnProperty("operator_type")) {
                                    d.operator_type = o.enums === String ? $root.google.ads.googleads.v11.enums.OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType[m.operator_type] : m.operator_type;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.os_major_version != null && m.hasOwnProperty("os_major_version")) {
                                    d.os_major_version = m.os_major_version;
                                    if (o.oneofs)
                                        d._os_major_version = "os_major_version";
                                }
                                if (m.os_minor_version != null && m.hasOwnProperty("os_minor_version")) {
                                    d.os_minor_version = m.os_minor_version;
                                    if (o.oneofs)
                                        d._os_minor_version = "os_minor_version";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this OperatingSystemVersionConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.OperatingSystemVersionConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OperatingSystemVersionConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return OperatingSystemVersionConstant;
                        })();
    
                        resources.PaidOrganicSearchTermView = (function() {
    
                            /**
                             * Properties of a PaidOrganicSearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IPaidOrganicSearchTermView
                             * @property {string|null} [resource_name] PaidOrganicSearchTermView resource_name
                             * @property {string|null} [search_term] PaidOrganicSearchTermView search_term
                             */
    
                            /**
                             * Constructs a new PaidOrganicSearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a PaidOrganicSearchTermView.
                             * @implements IPaidOrganicSearchTermView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IPaidOrganicSearchTermView=} [p] Properties to set
                             */
                            function PaidOrganicSearchTermView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PaidOrganicSearchTermView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @instance
                             */
                            PaidOrganicSearchTermView.prototype.resource_name = "";
    
                            /**
                             * PaidOrganicSearchTermView search_term.
                             * @member {string|null|undefined} search_term
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @instance
                             */
                            PaidOrganicSearchTermView.prototype.search_term = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PaidOrganicSearchTermView _search_term.
                             * @member {"search_term"|undefined} _search_term
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @instance
                             */
                            Object.defineProperty(PaidOrganicSearchTermView.prototype, "_search_term", {
                                get: $util.oneOfGetter($oneOfFields = ["search_term"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PaidOrganicSearchTermView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IPaidOrganicSearchTermView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.PaidOrganicSearchTermView} PaidOrganicSearchTermView instance
                             */
                            PaidOrganicSearchTermView.create = function create(properties) {
                                return new PaidOrganicSearchTermView(properties);
                            };
    
                            /**
                             * Encodes the specified PaidOrganicSearchTermView message. Does not implicitly {@link google.ads.googleads.v11.resources.PaidOrganicSearchTermView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IPaidOrganicSearchTermView} m PaidOrganicSearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaidOrganicSearchTermView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.search_term != null && Object.hasOwnProperty.call(m, "search_term"))
                                    w.uint32(26).string(m.search_term);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PaidOrganicSearchTermView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.PaidOrganicSearchTermView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IPaidOrganicSearchTermView} message PaidOrganicSearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaidOrganicSearchTermView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PaidOrganicSearchTermView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.PaidOrganicSearchTermView} PaidOrganicSearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaidOrganicSearchTermView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.search_term = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PaidOrganicSearchTermView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.PaidOrganicSearchTermView} PaidOrganicSearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaidOrganicSearchTermView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PaidOrganicSearchTermView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PaidOrganicSearchTermView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    p._search_term = 1;
                                    if (!$util.isString(m.search_term))
                                        return "search_term: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PaidOrganicSearchTermView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.PaidOrganicSearchTermView} PaidOrganicSearchTermView
                             */
                            PaidOrganicSearchTermView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.search_term != null) {
                                    m.search_term = String(d.search_term);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PaidOrganicSearchTermView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.PaidOrganicSearchTermView} m PaidOrganicSearchTermView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PaidOrganicSearchTermView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    d.search_term = m.search_term;
                                    if (o.oneofs)
                                        d._search_term = "search_term";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PaidOrganicSearchTermView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.PaidOrganicSearchTermView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PaidOrganicSearchTermView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PaidOrganicSearchTermView;
                        })();
    
                        resources.ParentalStatusView = (function() {
    
                            /**
                             * Properties of a ParentalStatusView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IParentalStatusView
                             * @property {string|null} [resource_name] ParentalStatusView resource_name
                             */
    
                            /**
                             * Constructs a new ParentalStatusView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ParentalStatusView.
                             * @implements IParentalStatusView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IParentalStatusView=} [p] Properties to set
                             */
                            function ParentalStatusView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ParentalStatusView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @instance
                             */
                            ParentalStatusView.prototype.resource_name = "";
    
                            /**
                             * Creates a new ParentalStatusView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IParentalStatusView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ParentalStatusView} ParentalStatusView instance
                             */
                            ParentalStatusView.create = function create(properties) {
                                return new ParentalStatusView(properties);
                            };
    
                            /**
                             * Encodes the specified ParentalStatusView message. Does not implicitly {@link google.ads.googleads.v11.resources.ParentalStatusView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IParentalStatusView} m ParentalStatusView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ParentalStatusView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ParentalStatusView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IParentalStatusView} message ParentalStatusView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ParentalStatusView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ParentalStatusView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ParentalStatusView} ParentalStatusView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ParentalStatusView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ParentalStatusView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ParentalStatusView} ParentalStatusView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ParentalStatusView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ParentalStatusView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ParentalStatusView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ParentalStatusView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ParentalStatusView} ParentalStatusView
                             */
                            ParentalStatusView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ParentalStatusView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ParentalStatusView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ParentalStatusView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ParentalStatusView} m ParentalStatusView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ParentalStatusView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ParentalStatusView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ParentalStatusView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ParentalStatusView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ParentalStatusView;
                        })();
    
                        resources.PaymentsAccount = (function() {
    
                            /**
                             * Properties of a PaymentsAccount.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IPaymentsAccount
                             * @property {string|null} [resource_name] PaymentsAccount resource_name
                             * @property {string|null} [payments_account_id] PaymentsAccount payments_account_id
                             * @property {string|null} [name] PaymentsAccount name
                             * @property {string|null} [currency_code] PaymentsAccount currency_code
                             * @property {string|null} [payments_profile_id] PaymentsAccount payments_profile_id
                             * @property {string|null} [secondary_payments_profile_id] PaymentsAccount secondary_payments_profile_id
                             * @property {string|null} [paying_manager_customer] PaymentsAccount paying_manager_customer
                             */
    
                            /**
                             * Constructs a new PaymentsAccount.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a PaymentsAccount.
                             * @implements IPaymentsAccount
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IPaymentsAccount=} [p] Properties to set
                             */
                            function PaymentsAccount(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PaymentsAccount resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            PaymentsAccount.prototype.resource_name = "";
    
                            /**
                             * PaymentsAccount payments_account_id.
                             * @member {string|null|undefined} payments_account_id
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            PaymentsAccount.prototype.payments_account_id = null;
    
                            /**
                             * PaymentsAccount name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            PaymentsAccount.prototype.name = null;
    
                            /**
                             * PaymentsAccount currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            PaymentsAccount.prototype.currency_code = null;
    
                            /**
                             * PaymentsAccount payments_profile_id.
                             * @member {string|null|undefined} payments_profile_id
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            PaymentsAccount.prototype.payments_profile_id = null;
    
                            /**
                             * PaymentsAccount secondary_payments_profile_id.
                             * @member {string|null|undefined} secondary_payments_profile_id
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            PaymentsAccount.prototype.secondary_payments_profile_id = null;
    
                            /**
                             * PaymentsAccount paying_manager_customer.
                             * @member {string|null|undefined} paying_manager_customer
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            PaymentsAccount.prototype.paying_manager_customer = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PaymentsAccount _payments_account_id.
                             * @member {"payments_account_id"|undefined} _payments_account_id
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            Object.defineProperty(PaymentsAccount.prototype, "_payments_account_id", {
                                get: $util.oneOfGetter($oneOfFields = ["payments_account_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PaymentsAccount _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            Object.defineProperty(PaymentsAccount.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PaymentsAccount _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            Object.defineProperty(PaymentsAccount.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PaymentsAccount _payments_profile_id.
                             * @member {"payments_profile_id"|undefined} _payments_profile_id
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            Object.defineProperty(PaymentsAccount.prototype, "_payments_profile_id", {
                                get: $util.oneOfGetter($oneOfFields = ["payments_profile_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PaymentsAccount _secondary_payments_profile_id.
                             * @member {"secondary_payments_profile_id"|undefined} _secondary_payments_profile_id
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            Object.defineProperty(PaymentsAccount.prototype, "_secondary_payments_profile_id", {
                                get: $util.oneOfGetter($oneOfFields = ["secondary_payments_profile_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PaymentsAccount _paying_manager_customer.
                             * @member {"paying_manager_customer"|undefined} _paying_manager_customer
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             */
                            Object.defineProperty(PaymentsAccount.prototype, "_paying_manager_customer", {
                                get: $util.oneOfGetter($oneOfFields = ["paying_manager_customer"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PaymentsAccount instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {google.ads.googleads.v11.resources.IPaymentsAccount=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.PaymentsAccount} PaymentsAccount instance
                             */
                            PaymentsAccount.create = function create(properties) {
                                return new PaymentsAccount(properties);
                            };
    
                            /**
                             * Encodes the specified PaymentsAccount message. Does not implicitly {@link google.ads.googleads.v11.resources.PaymentsAccount.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {google.ads.googleads.v11.resources.IPaymentsAccount} m PaymentsAccount message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaymentsAccount.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.payments_account_id != null && Object.hasOwnProperty.call(m, "payments_account_id"))
                                    w.uint32(66).string(m.payments_account_id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(74).string(m.name);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(82).string(m.currency_code);
                                if (m.payments_profile_id != null && Object.hasOwnProperty.call(m, "payments_profile_id"))
                                    w.uint32(90).string(m.payments_profile_id);
                                if (m.secondary_payments_profile_id != null && Object.hasOwnProperty.call(m, "secondary_payments_profile_id"))
                                    w.uint32(98).string(m.secondary_payments_profile_id);
                                if (m.paying_manager_customer != null && Object.hasOwnProperty.call(m, "paying_manager_customer"))
                                    w.uint32(106).string(m.paying_manager_customer);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PaymentsAccount message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.PaymentsAccount.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {google.ads.googleads.v11.resources.IPaymentsAccount} message PaymentsAccount message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PaymentsAccount.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PaymentsAccount message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.PaymentsAccount} PaymentsAccount
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaymentsAccount.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.PaymentsAccount();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 8:
                                        m.payments_account_id = r.string();
                                        break;
                                    case 9:
                                        m.name = r.string();
                                        break;
                                    case 10:
                                        m.currency_code = r.string();
                                        break;
                                    case 11:
                                        m.payments_profile_id = r.string();
                                        break;
                                    case 12:
                                        m.secondary_payments_profile_id = r.string();
                                        break;
                                    case 13:
                                        m.paying_manager_customer = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PaymentsAccount message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.PaymentsAccount} PaymentsAccount
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PaymentsAccount.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PaymentsAccount message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PaymentsAccount.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.payments_account_id != null && m.hasOwnProperty("payments_account_id")) {
                                    p._payments_account_id = 1;
                                    if (!$util.isString(m.payments_account_id))
                                        return "payments_account_id: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.payments_profile_id != null && m.hasOwnProperty("payments_profile_id")) {
                                    p._payments_profile_id = 1;
                                    if (!$util.isString(m.payments_profile_id))
                                        return "payments_profile_id: string expected";
                                }
                                if (m.secondary_payments_profile_id != null && m.hasOwnProperty("secondary_payments_profile_id")) {
                                    p._secondary_payments_profile_id = 1;
                                    if (!$util.isString(m.secondary_payments_profile_id))
                                        return "secondary_payments_profile_id: string expected";
                                }
                                if (m.paying_manager_customer != null && m.hasOwnProperty("paying_manager_customer")) {
                                    p._paying_manager_customer = 1;
                                    if (!$util.isString(m.paying_manager_customer))
                                        return "paying_manager_customer: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PaymentsAccount message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.PaymentsAccount} PaymentsAccount
                             */
                            PaymentsAccount.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.PaymentsAccount)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.PaymentsAccount();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.payments_account_id != null) {
                                    m.payments_account_id = String(d.payments_account_id);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.payments_profile_id != null) {
                                    m.payments_profile_id = String(d.payments_profile_id);
                                }
                                if (d.secondary_payments_profile_id != null) {
                                    m.secondary_payments_profile_id = String(d.secondary_payments_profile_id);
                                }
                                if (d.paying_manager_customer != null) {
                                    m.paying_manager_customer = String(d.paying_manager_customer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PaymentsAccount message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @static
                             * @param {google.ads.googleads.v11.resources.PaymentsAccount} m PaymentsAccount
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PaymentsAccount.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.payments_account_id != null && m.hasOwnProperty("payments_account_id")) {
                                    d.payments_account_id = m.payments_account_id;
                                    if (o.oneofs)
                                        d._payments_account_id = "payments_account_id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.payments_profile_id != null && m.hasOwnProperty("payments_profile_id")) {
                                    d.payments_profile_id = m.payments_profile_id;
                                    if (o.oneofs)
                                        d._payments_profile_id = "payments_profile_id";
                                }
                                if (m.secondary_payments_profile_id != null && m.hasOwnProperty("secondary_payments_profile_id")) {
                                    d.secondary_payments_profile_id = m.secondary_payments_profile_id;
                                    if (o.oneofs)
                                        d._secondary_payments_profile_id = "secondary_payments_profile_id";
                                }
                                if (m.paying_manager_customer != null && m.hasOwnProperty("paying_manager_customer")) {
                                    d.paying_manager_customer = m.paying_manager_customer;
                                    if (o.oneofs)
                                        d._paying_manager_customer = "paying_manager_customer";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PaymentsAccount to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.PaymentsAccount
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PaymentsAccount.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PaymentsAccount;
                        })();
    
                        resources.ProductBiddingCategoryConstant = (function() {
    
                            /**
                             * Properties of a ProductBiddingCategoryConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IProductBiddingCategoryConstant
                             * @property {string|null} [resource_name] ProductBiddingCategoryConstant resource_name
                             * @property {number|null} [id] ProductBiddingCategoryConstant id
                             * @property {string|null} [country_code] ProductBiddingCategoryConstant country_code
                             * @property {string|null} [product_bidding_category_constant_parent] ProductBiddingCategoryConstant product_bidding_category_constant_parent
                             * @property {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel|null} [level] ProductBiddingCategoryConstant level
                             * @property {google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum.ProductBiddingCategoryStatus|null} [status] ProductBiddingCategoryConstant status
                             * @property {string|null} [language_code] ProductBiddingCategoryConstant language_code
                             * @property {string|null} [localized_name] ProductBiddingCategoryConstant localized_name
                             */
    
                            /**
                             * Constructs a new ProductBiddingCategoryConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ProductBiddingCategoryConstant.
                             * @implements IProductBiddingCategoryConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IProductBiddingCategoryConstant=} [p] Properties to set
                             */
                            function ProductBiddingCategoryConstant(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductBiddingCategoryConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.resource_name = "";
    
                            /**
                             * ProductBiddingCategoryConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.id = null;
    
                            /**
                             * ProductBiddingCategoryConstant country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.country_code = null;
    
                            /**
                             * ProductBiddingCategoryConstant product_bidding_category_constant_parent.
                             * @member {string|null|undefined} product_bidding_category_constant_parent
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.product_bidding_category_constant_parent = null;
    
                            /**
                             * ProductBiddingCategoryConstant level.
                             * @member {google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel} level
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.level = 0;
    
                            /**
                             * ProductBiddingCategoryConstant status.
                             * @member {google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum.ProductBiddingCategoryStatus} status
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.status = 0;
    
                            /**
                             * ProductBiddingCategoryConstant language_code.
                             * @member {string|null|undefined} language_code
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.language_code = null;
    
                            /**
                             * ProductBiddingCategoryConstant localized_name.
                             * @member {string|null|undefined} localized_name
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            ProductBiddingCategoryConstant.prototype.localized_name = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductBiddingCategoryConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            Object.defineProperty(ProductBiddingCategoryConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ProductBiddingCategoryConstant _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            Object.defineProperty(ProductBiddingCategoryConstant.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ProductBiddingCategoryConstant _product_bidding_category_constant_parent.
                             * @member {"product_bidding_category_constant_parent"|undefined} _product_bidding_category_constant_parent
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            Object.defineProperty(ProductBiddingCategoryConstant.prototype, "_product_bidding_category_constant_parent", {
                                get: $util.oneOfGetter($oneOfFields = ["product_bidding_category_constant_parent"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ProductBiddingCategoryConstant _language_code.
                             * @member {"language_code"|undefined} _language_code
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            Object.defineProperty(ProductBiddingCategoryConstant.prototype, "_language_code", {
                                get: $util.oneOfGetter($oneOfFields = ["language_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ProductBiddingCategoryConstant _localized_name.
                             * @member {"localized_name"|undefined} _localized_name
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             */
                            Object.defineProperty(ProductBiddingCategoryConstant.prototype, "_localized_name", {
                                get: $util.oneOfGetter($oneOfFields = ["localized_name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductBiddingCategoryConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IProductBiddingCategoryConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ProductBiddingCategoryConstant} ProductBiddingCategoryConstant instance
                             */
                            ProductBiddingCategoryConstant.create = function create(properties) {
                                return new ProductBiddingCategoryConstant(properties);
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.ProductBiddingCategoryConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IProductBiddingCategoryConstant} m ProductBiddingCategoryConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.level != null && Object.hasOwnProperty.call(m, "level"))
                                    w.uint32(40).int32(m.level);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(48).int32(m.status);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(80).int64(m.id);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(90).string(m.country_code);
                                if (m.product_bidding_category_constant_parent != null && Object.hasOwnProperty.call(m, "product_bidding_category_constant_parent"))
                                    w.uint32(98).string(m.product_bidding_category_constant_parent);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(106).string(m.language_code);
                                if (m.localized_name != null && Object.hasOwnProperty.call(m, "localized_name"))
                                    w.uint32(114).string(m.localized_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductBiddingCategoryConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ProductBiddingCategoryConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.IProductBiddingCategoryConstant} message ProductBiddingCategoryConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductBiddingCategoryConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ProductBiddingCategoryConstant} ProductBiddingCategoryConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 10:
                                        m.id = r.int64();
                                        break;
                                    case 11:
                                        m.country_code = r.string();
                                        break;
                                    case 12:
                                        m.product_bidding_category_constant_parent = r.string();
                                        break;
                                    case 5:
                                        m.level = r.int32();
                                        break;
                                    case 6:
                                        m.status = r.int32();
                                        break;
                                    case 13:
                                        m.language_code = r.string();
                                        break;
                                    case 14:
                                        m.localized_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductBiddingCategoryConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ProductBiddingCategoryConstant} ProductBiddingCategoryConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductBiddingCategoryConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductBiddingCategoryConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductBiddingCategoryConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.product_bidding_category_constant_parent != null && m.hasOwnProperty("product_bidding_category_constant_parent")) {
                                    p._product_bidding_category_constant_parent = 1;
                                    if (!$util.isString(m.product_bidding_category_constant_parent))
                                        return "product_bidding_category_constant_parent: string expected";
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    switch (m.level) {
                                    default:
                                        return "level: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    p._language_code = 1;
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.localized_name != null && m.hasOwnProperty("localized_name")) {
                                    p._localized_name = 1;
                                    if (!$util.isString(m.localized_name))
                                        return "localized_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductBiddingCategoryConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ProductBiddingCategoryConstant} ProductBiddingCategoryConstant
                             */
                            ProductBiddingCategoryConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.product_bidding_category_constant_parent != null) {
                                    m.product_bidding_category_constant_parent = String(d.product_bidding_category_constant_parent);
                                }
                                switch (d.level) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.level = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.level = 1;
                                    break;
                                case "LEVEL1":
                                case 2:
                                    m.level = 2;
                                    break;
                                case "LEVEL2":
                                case 3:
                                    m.level = 3;
                                    break;
                                case "LEVEL3":
                                case 4:
                                    m.level = 4;
                                    break;
                                case "LEVEL4":
                                case 5:
                                    m.level = 5;
                                    break;
                                case "LEVEL5":
                                case 6:
                                    m.level = 6;
                                    break;
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ACTIVE":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "OBSOLETE":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.localized_name != null) {
                                    m.localized_name = String(d.localized_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductBiddingCategoryConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ProductBiddingCategoryConstant} m ProductBiddingCategoryConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductBiddingCategoryConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.level = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.level != null && m.hasOwnProperty("level")) {
                                    d.level = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductBiddingCategoryLevelEnum.ProductBiddingCategoryLevel[m.level] : m.level;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.ProductBiddingCategoryStatusEnum.ProductBiddingCategoryStatus[m.status] : m.status;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.product_bidding_category_constant_parent != null && m.hasOwnProperty("product_bidding_category_constant_parent")) {
                                    d.product_bidding_category_constant_parent = m.product_bidding_category_constant_parent;
                                    if (o.oneofs)
                                        d._product_bidding_category_constant_parent = "product_bidding_category_constant_parent";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                    if (o.oneofs)
                                        d._language_code = "language_code";
                                }
                                if (m.localized_name != null && m.hasOwnProperty("localized_name")) {
                                    d.localized_name = m.localized_name;
                                    if (o.oneofs)
                                        d._localized_name = "localized_name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductBiddingCategoryConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ProductBiddingCategoryConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductBiddingCategoryConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductBiddingCategoryConstant;
                        })();
    
                        resources.ProductGroupView = (function() {
    
                            /**
                             * Properties of a ProductGroupView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IProductGroupView
                             * @property {string|null} [resource_name] ProductGroupView resource_name
                             */
    
                            /**
                             * Constructs a new ProductGroupView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ProductGroupView.
                             * @implements IProductGroupView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IProductGroupView=} [p] Properties to set
                             */
                            function ProductGroupView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductGroupView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @instance
                             */
                            ProductGroupView.prototype.resource_name = "";
    
                            /**
                             * Creates a new ProductGroupView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IProductGroupView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ProductGroupView} ProductGroupView instance
                             */
                            ProductGroupView.create = function create(properties) {
                                return new ProductGroupView(properties);
                            };
    
                            /**
                             * Encodes the specified ProductGroupView message. Does not implicitly {@link google.ads.googleads.v11.resources.ProductGroupView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IProductGroupView} m ProductGroupView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductGroupView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductGroupView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ProductGroupView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IProductGroupView} message ProductGroupView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductGroupView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductGroupView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ProductGroupView} ProductGroupView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductGroupView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ProductGroupView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductGroupView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ProductGroupView} ProductGroupView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductGroupView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductGroupView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductGroupView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductGroupView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ProductGroupView} ProductGroupView
                             */
                            ProductGroupView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ProductGroupView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ProductGroupView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductGroupView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ProductGroupView} m ProductGroupView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductGroupView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductGroupView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ProductGroupView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductGroupView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductGroupView;
                        })();
    
                        resources.Recommendation = (function() {
    
                            /**
                             * Properties of a Recommendation.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IRecommendation
                             * @property {string|null} [resource_name] Recommendation resource_name
                             * @property {google.ads.googleads.v11.enums.RecommendationTypeEnum.RecommendationType|null} [type] Recommendation type
                             * @property {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact|null} [impact] Recommendation impact
                             * @property {string|null} [campaign_budget] Recommendation campaign_budget
                             * @property {string|null} [campaign] Recommendation campaign
                             * @property {string|null} [ad_group] Recommendation ad_group
                             * @property {boolean|null} [dismissed] Recommendation dismissed
                             * @property {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null} [campaign_budget_recommendation] Recommendation campaign_budget_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null} [forecasting_campaign_budget_recommendation] Recommendation forecasting_campaign_budget_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IKeywordRecommendation|null} [keyword_recommendation] Recommendation keyword_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ITextAdRecommendation|null} [text_ad_recommendation] Recommendation text_ad_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ITargetCpaOptInRecommendation|null} [target_cpa_opt_in_recommendation] Recommendation target_cpa_opt_in_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IMaximizeConversionsOptInRecommendation|null} [maximize_conversions_opt_in_recommendation] Recommendation maximize_conversions_opt_in_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IEnhancedCpcOptInRecommendation|null} [enhanced_cpc_opt_in_recommendation] Recommendation enhanced_cpc_opt_in_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ISearchPartnersOptInRecommendation|null} [search_partners_opt_in_recommendation] Recommendation search_partners_opt_in_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IMaximizeClicksOptInRecommendation|null} [maximize_clicks_opt_in_recommendation] Recommendation maximize_clicks_opt_in_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IOptimizeAdRotationRecommendation|null} [optimize_ad_rotation_recommendation] Recommendation optimize_ad_rotation_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ICalloutExtensionRecommendation|null} [callout_extension_recommendation] Recommendation callout_extension_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ISitelinkExtensionRecommendation|null} [sitelink_extension_recommendation] Recommendation sitelink_extension_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ICallExtensionRecommendation|null} [call_extension_recommendation] Recommendation call_extension_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IKeywordMatchTypeRecommendation|null} [keyword_match_type_recommendation] Recommendation keyword_match_type_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IMoveUnusedBudgetRecommendation|null} [move_unused_budget_recommendation] Recommendation move_unused_budget_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ITargetRoasOptInRecommendation|null} [target_roas_opt_in_recommendation] Recommendation target_roas_opt_in_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdRecommendation|null} [responsive_search_ad_recommendation] Recommendation responsive_search_ad_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null} [marginal_roi_campaign_budget_recommendation] Recommendation marginal_roi_campaign_budget_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IUseBroadMatchKeywordRecommendation|null} [use_broad_match_keyword_recommendation] Recommendation use_broad_match_keyword_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdAssetRecommendation|null} [responsive_search_ad_asset_recommendation] Recommendation responsive_search_ad_asset_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation|null} [upgrade_smart_shopping_campaign_to_performance_max_recommendation] Recommendation upgrade_smart_shopping_campaign_to_performance_max_recommendation
                             * @property {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdImproveAdStrengthRecommendation|null} [responsive_search_ad_improve_ad_strength_recommendation] Recommendation responsive_search_ad_improve_ad_strength_recommendation
                             */
    
                            /**
                             * Constructs a new Recommendation.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a Recommendation.
                             * @implements IRecommendation
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IRecommendation=} [p] Properties to set
                             */
                            function Recommendation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Recommendation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.resource_name = "";
    
                            /**
                             * Recommendation type.
                             * @member {google.ads.googleads.v11.enums.RecommendationTypeEnum.RecommendationType} type
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.type = 0;
    
                            /**
                             * Recommendation impact.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact|null|undefined} impact
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.impact = null;
    
                            /**
                             * Recommendation campaign_budget.
                             * @member {string|null|undefined} campaign_budget
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.campaign_budget = null;
    
                            /**
                             * Recommendation campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.campaign = null;
    
                            /**
                             * Recommendation ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.ad_group = null;
    
                            /**
                             * Recommendation dismissed.
                             * @member {boolean|null|undefined} dismissed
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.dismissed = null;
    
                            /**
                             * Recommendation campaign_budget_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null|undefined} campaign_budget_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.campaign_budget_recommendation = null;
    
                            /**
                             * Recommendation forecasting_campaign_budget_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null|undefined} forecasting_campaign_budget_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.forecasting_campaign_budget_recommendation = null;
    
                            /**
                             * Recommendation keyword_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IKeywordRecommendation|null|undefined} keyword_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.keyword_recommendation = null;
    
                            /**
                             * Recommendation text_ad_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ITextAdRecommendation|null|undefined} text_ad_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.text_ad_recommendation = null;
    
                            /**
                             * Recommendation target_cpa_opt_in_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ITargetCpaOptInRecommendation|null|undefined} target_cpa_opt_in_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.target_cpa_opt_in_recommendation = null;
    
                            /**
                             * Recommendation maximize_conversions_opt_in_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IMaximizeConversionsOptInRecommendation|null|undefined} maximize_conversions_opt_in_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.maximize_conversions_opt_in_recommendation = null;
    
                            /**
                             * Recommendation enhanced_cpc_opt_in_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IEnhancedCpcOptInRecommendation|null|undefined} enhanced_cpc_opt_in_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.enhanced_cpc_opt_in_recommendation = null;
    
                            /**
                             * Recommendation search_partners_opt_in_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ISearchPartnersOptInRecommendation|null|undefined} search_partners_opt_in_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.search_partners_opt_in_recommendation = null;
    
                            /**
                             * Recommendation maximize_clicks_opt_in_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IMaximizeClicksOptInRecommendation|null|undefined} maximize_clicks_opt_in_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.maximize_clicks_opt_in_recommendation = null;
    
                            /**
                             * Recommendation optimize_ad_rotation_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IOptimizeAdRotationRecommendation|null|undefined} optimize_ad_rotation_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.optimize_ad_rotation_recommendation = null;
    
                            /**
                             * Recommendation callout_extension_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ICalloutExtensionRecommendation|null|undefined} callout_extension_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.callout_extension_recommendation = null;
    
                            /**
                             * Recommendation sitelink_extension_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ISitelinkExtensionRecommendation|null|undefined} sitelink_extension_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.sitelink_extension_recommendation = null;
    
                            /**
                             * Recommendation call_extension_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ICallExtensionRecommendation|null|undefined} call_extension_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.call_extension_recommendation = null;
    
                            /**
                             * Recommendation keyword_match_type_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IKeywordMatchTypeRecommendation|null|undefined} keyword_match_type_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.keyword_match_type_recommendation = null;
    
                            /**
                             * Recommendation move_unused_budget_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IMoveUnusedBudgetRecommendation|null|undefined} move_unused_budget_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.move_unused_budget_recommendation = null;
    
                            /**
                             * Recommendation target_roas_opt_in_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ITargetRoasOptInRecommendation|null|undefined} target_roas_opt_in_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.target_roas_opt_in_recommendation = null;
    
                            /**
                             * Recommendation responsive_search_ad_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdRecommendation|null|undefined} responsive_search_ad_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.responsive_search_ad_recommendation = null;
    
                            /**
                             * Recommendation marginal_roi_campaign_budget_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null|undefined} marginal_roi_campaign_budget_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.marginal_roi_campaign_budget_recommendation = null;
    
                            /**
                             * Recommendation use_broad_match_keyword_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IUseBroadMatchKeywordRecommendation|null|undefined} use_broad_match_keyword_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.use_broad_match_keyword_recommendation = null;
    
                            /**
                             * Recommendation responsive_search_ad_asset_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdAssetRecommendation|null|undefined} responsive_search_ad_asset_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.responsive_search_ad_asset_recommendation = null;
    
                            /**
                             * Recommendation upgrade_smart_shopping_campaign_to_performance_max_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation|null|undefined} upgrade_smart_shopping_campaign_to_performance_max_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.upgrade_smart_shopping_campaign_to_performance_max_recommendation = null;
    
                            /**
                             * Recommendation responsive_search_ad_improve_ad_strength_recommendation.
                             * @member {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdImproveAdStrengthRecommendation|null|undefined} responsive_search_ad_improve_ad_strength_recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Recommendation.prototype.responsive_search_ad_improve_ad_strength_recommendation = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Recommendation _campaign_budget.
                             * @member {"campaign_budget"|undefined} _campaign_budget
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Object.defineProperty(Recommendation.prototype, "_campaign_budget", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_budget"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Recommendation _campaign.
                             * @member {"campaign"|undefined} _campaign
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Object.defineProperty(Recommendation.prototype, "_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Recommendation _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Object.defineProperty(Recommendation.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Recommendation _dismissed.
                             * @member {"dismissed"|undefined} _dismissed
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Object.defineProperty(Recommendation.prototype, "_dismissed", {
                                get: $util.oneOfGetter($oneOfFields = ["dismissed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Recommendation recommendation.
                             * @member {"campaign_budget_recommendation"|"forecasting_campaign_budget_recommendation"|"keyword_recommendation"|"text_ad_recommendation"|"target_cpa_opt_in_recommendation"|"maximize_conversions_opt_in_recommendation"|"enhanced_cpc_opt_in_recommendation"|"search_partners_opt_in_recommendation"|"maximize_clicks_opt_in_recommendation"|"optimize_ad_rotation_recommendation"|"callout_extension_recommendation"|"sitelink_extension_recommendation"|"call_extension_recommendation"|"keyword_match_type_recommendation"|"move_unused_budget_recommendation"|"target_roas_opt_in_recommendation"|"responsive_search_ad_recommendation"|"marginal_roi_campaign_budget_recommendation"|"use_broad_match_keyword_recommendation"|"responsive_search_ad_asset_recommendation"|"upgrade_smart_shopping_campaign_to_performance_max_recommendation"|"responsive_search_ad_improve_ad_strength_recommendation"|undefined} recommendation
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             */
                            Object.defineProperty(Recommendation.prototype, "recommendation", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_budget_recommendation", "forecasting_campaign_budget_recommendation", "keyword_recommendation", "text_ad_recommendation", "target_cpa_opt_in_recommendation", "maximize_conversions_opt_in_recommendation", "enhanced_cpc_opt_in_recommendation", "search_partners_opt_in_recommendation", "maximize_clicks_opt_in_recommendation", "optimize_ad_rotation_recommendation", "callout_extension_recommendation", "sitelink_extension_recommendation", "call_extension_recommendation", "keyword_match_type_recommendation", "move_unused_budget_recommendation", "target_roas_opt_in_recommendation", "responsive_search_ad_recommendation", "marginal_roi_campaign_budget_recommendation", "use_broad_match_keyword_recommendation", "responsive_search_ad_asset_recommendation", "upgrade_smart_shopping_campaign_to_performance_max_recommendation", "responsive_search_ad_improve_ad_strength_recommendation"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Recommendation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRecommendation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Recommendation} Recommendation instance
                             */
                            Recommendation.create = function create(properties) {
                                return new Recommendation(properties);
                            };
    
                            /**
                             * Encodes the specified Recommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRecommendation} m Recommendation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Recommendation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(16).int32(m.type);
                                if (m.impact != null && Object.hasOwnProperty.call(m, "impact"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.encode(m.impact, w.uint32(26).fork()).ldelim();
                                if (m.campaign_budget_recommendation != null && Object.hasOwnProperty.call(m, "campaign_budget_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.encode(m.campaign_budget_recommendation, w.uint32(34).fork()).ldelim();
                                if (m.keyword_recommendation != null && Object.hasOwnProperty.call(m, "keyword_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.encode(m.keyword_recommendation, w.uint32(66).fork()).ldelim();
                                if (m.text_ad_recommendation != null && Object.hasOwnProperty.call(m, "text_ad_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.encode(m.text_ad_recommendation, w.uint32(74).fork()).ldelim();
                                if (m.target_cpa_opt_in_recommendation != null && Object.hasOwnProperty.call(m, "target_cpa_opt_in_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.encode(m.target_cpa_opt_in_recommendation, w.uint32(82).fork()).ldelim();
                                if (m.maximize_conversions_opt_in_recommendation != null && Object.hasOwnProperty.call(m, "maximize_conversions_opt_in_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation.encode(m.maximize_conversions_opt_in_recommendation, w.uint32(90).fork()).ldelim();
                                if (m.enhanced_cpc_opt_in_recommendation != null && Object.hasOwnProperty.call(m, "enhanced_cpc_opt_in_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation.encode(m.enhanced_cpc_opt_in_recommendation, w.uint32(98).fork()).ldelim();
                                if (m.search_partners_opt_in_recommendation != null && Object.hasOwnProperty.call(m, "search_partners_opt_in_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation.encode(m.search_partners_opt_in_recommendation, w.uint32(114).fork()).ldelim();
                                if (m.maximize_clicks_opt_in_recommendation != null && Object.hasOwnProperty.call(m, "maximize_clicks_opt_in_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation.encode(m.maximize_clicks_opt_in_recommendation, w.uint32(122).fork()).ldelim();
                                if (m.optimize_ad_rotation_recommendation != null && Object.hasOwnProperty.call(m, "optimize_ad_rotation_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation.encode(m.optimize_ad_rotation_recommendation, w.uint32(130).fork()).ldelim();
                                if (m.callout_extension_recommendation != null && Object.hasOwnProperty.call(m, "callout_extension_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.encode(m.callout_extension_recommendation, w.uint32(138).fork()).ldelim();
                                if (m.sitelink_extension_recommendation != null && Object.hasOwnProperty.call(m, "sitelink_extension_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.encode(m.sitelink_extension_recommendation, w.uint32(146).fork()).ldelim();
                                if (m.call_extension_recommendation != null && Object.hasOwnProperty.call(m, "call_extension_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.encode(m.call_extension_recommendation, w.uint32(154).fork()).ldelim();
                                if (m.keyword_match_type_recommendation != null && Object.hasOwnProperty.call(m, "keyword_match_type_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.encode(m.keyword_match_type_recommendation, w.uint32(162).fork()).ldelim();
                                if (m.move_unused_budget_recommendation != null && Object.hasOwnProperty.call(m, "move_unused_budget_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.encode(m.move_unused_budget_recommendation, w.uint32(170).fork()).ldelim();
                                if (m.forecasting_campaign_budget_recommendation != null && Object.hasOwnProperty.call(m, "forecasting_campaign_budget_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.encode(m.forecasting_campaign_budget_recommendation, w.uint32(178).fork()).ldelim();
                                if (m.target_roas_opt_in_recommendation != null && Object.hasOwnProperty.call(m, "target_roas_opt_in_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation.encode(m.target_roas_opt_in_recommendation, w.uint32(186).fork()).ldelim();
                                if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                    w.uint32(194).string(m.campaign_budget);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(202).string(m.campaign);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(210).string(m.ad_group);
                                if (m.dismissed != null && Object.hasOwnProperty.call(m, "dismissed"))
                                    w.uint32(216).bool(m.dismissed);
                                if (m.responsive_search_ad_recommendation != null && Object.hasOwnProperty.call(m, "responsive_search_ad_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.encode(m.responsive_search_ad_recommendation, w.uint32(226).fork()).ldelim();
                                if (m.marginal_roi_campaign_budget_recommendation != null && Object.hasOwnProperty.call(m, "marginal_roi_campaign_budget_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.encode(m.marginal_roi_campaign_budget_recommendation, w.uint32(234).fork()).ldelim();
                                if (m.use_broad_match_keyword_recommendation != null && Object.hasOwnProperty.call(m, "use_broad_match_keyword_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.encode(m.use_broad_match_keyword_recommendation, w.uint32(242).fork()).ldelim();
                                if (m.responsive_search_ad_asset_recommendation != null && Object.hasOwnProperty.call(m, "responsive_search_ad_asset_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.encode(m.responsive_search_ad_asset_recommendation, w.uint32(250).fork()).ldelim();
                                if (m.upgrade_smart_shopping_campaign_to_performance_max_recommendation != null && Object.hasOwnProperty.call(m, "upgrade_smart_shopping_campaign_to_performance_max_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.encode(m.upgrade_smart_shopping_campaign_to_performance_max_recommendation, w.uint32(258).fork()).ldelim();
                                if (m.responsive_search_ad_improve_ad_strength_recommendation != null && Object.hasOwnProperty.call(m, "responsive_search_ad_improve_ad_strength_recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.encode(m.responsive_search_ad_improve_ad_strength_recommendation, w.uint32(266).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified Recommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRecommendation} message Recommendation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Recommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Recommendation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Recommendation} Recommendation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Recommendation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.type = r.int32();
                                        break;
                                    case 3:
                                        m.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.campaign_budget = r.string();
                                        break;
                                    case 25:
                                        m.campaign = r.string();
                                        break;
                                    case 26:
                                        m.ad_group = r.string();
                                        break;
                                    case 27:
                                        m.dismissed = r.bool();
                                        break;
                                    case 4:
                                        m.campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.forecasting_campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.keyword_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.text_ad_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.target_cpa_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.maximize_conversions_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.enhanced_cpc_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.search_partners_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.maximize_clicks_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.optimize_ad_rotation_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation.decode(r, r.uint32());
                                        break;
                                    case 17:
                                        m.callout_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.sitelink_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.decode(r, r.uint32());
                                        break;
                                    case 19:
                                        m.call_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.keyword_match_type_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.move_unused_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.target_roas_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.responsive_search_ad_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.decode(r, r.uint32());
                                        break;
                                    case 29:
                                        m.marginal_roi_campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.use_broad_match_keyword_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.responsive_search_ad_asset_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.upgrade_smart_shopping_campaign_to_performance_max_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.decode(r, r.uint32());
                                        break;
                                    case 33:
                                        m.responsive_search_ad_improve_ad_strength_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Recommendation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Recommendation} Recommendation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Recommendation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Recommendation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Recommendation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                        break;
                                    }
                                }
                                if (m.impact != null && m.hasOwnProperty("impact")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.verify(m.impact);
                                        if (e)
                                            return "impact." + e;
                                    }
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    p._campaign_budget = 1;
                                    if (!$util.isString(m.campaign_budget))
                                        return "campaign_budget: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p._campaign = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.dismissed != null && m.hasOwnProperty("dismissed")) {
                                    p._dismissed = 1;
                                    if (typeof m.dismissed !== "boolean")
                                        return "dismissed: boolean expected";
                                }
                                if (m.campaign_budget_recommendation != null && m.hasOwnProperty("campaign_budget_recommendation")) {
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.verify(m.campaign_budget_recommendation);
                                        if (e)
                                            return "campaign_budget_recommendation." + e;
                                    }
                                }
                                if (m.forecasting_campaign_budget_recommendation != null && m.hasOwnProperty("forecasting_campaign_budget_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.verify(m.forecasting_campaign_budget_recommendation);
                                        if (e)
                                            return "forecasting_campaign_budget_recommendation." + e;
                                    }
                                }
                                if (m.keyword_recommendation != null && m.hasOwnProperty("keyword_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.verify(m.keyword_recommendation);
                                        if (e)
                                            return "keyword_recommendation." + e;
                                    }
                                }
                                if (m.text_ad_recommendation != null && m.hasOwnProperty("text_ad_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.verify(m.text_ad_recommendation);
                                        if (e)
                                            return "text_ad_recommendation." + e;
                                    }
                                }
                                if (m.target_cpa_opt_in_recommendation != null && m.hasOwnProperty("target_cpa_opt_in_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.verify(m.target_cpa_opt_in_recommendation);
                                        if (e)
                                            return "target_cpa_opt_in_recommendation." + e;
                                    }
                                }
                                if (m.maximize_conversions_opt_in_recommendation != null && m.hasOwnProperty("maximize_conversions_opt_in_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation.verify(m.maximize_conversions_opt_in_recommendation);
                                        if (e)
                                            return "maximize_conversions_opt_in_recommendation." + e;
                                    }
                                }
                                if (m.enhanced_cpc_opt_in_recommendation != null && m.hasOwnProperty("enhanced_cpc_opt_in_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation.verify(m.enhanced_cpc_opt_in_recommendation);
                                        if (e)
                                            return "enhanced_cpc_opt_in_recommendation." + e;
                                    }
                                }
                                if (m.search_partners_opt_in_recommendation != null && m.hasOwnProperty("search_partners_opt_in_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation.verify(m.search_partners_opt_in_recommendation);
                                        if (e)
                                            return "search_partners_opt_in_recommendation." + e;
                                    }
                                }
                                if (m.maximize_clicks_opt_in_recommendation != null && m.hasOwnProperty("maximize_clicks_opt_in_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation.verify(m.maximize_clicks_opt_in_recommendation);
                                        if (e)
                                            return "maximize_clicks_opt_in_recommendation." + e;
                                    }
                                }
                                if (m.optimize_ad_rotation_recommendation != null && m.hasOwnProperty("optimize_ad_rotation_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation.verify(m.optimize_ad_rotation_recommendation);
                                        if (e)
                                            return "optimize_ad_rotation_recommendation." + e;
                                    }
                                }
                                if (m.callout_extension_recommendation != null && m.hasOwnProperty("callout_extension_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.verify(m.callout_extension_recommendation);
                                        if (e)
                                            return "callout_extension_recommendation." + e;
                                    }
                                }
                                if (m.sitelink_extension_recommendation != null && m.hasOwnProperty("sitelink_extension_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.verify(m.sitelink_extension_recommendation);
                                        if (e)
                                            return "sitelink_extension_recommendation." + e;
                                    }
                                }
                                if (m.call_extension_recommendation != null && m.hasOwnProperty("call_extension_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.verify(m.call_extension_recommendation);
                                        if (e)
                                            return "call_extension_recommendation." + e;
                                    }
                                }
                                if (m.keyword_match_type_recommendation != null && m.hasOwnProperty("keyword_match_type_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.verify(m.keyword_match_type_recommendation);
                                        if (e)
                                            return "keyword_match_type_recommendation." + e;
                                    }
                                }
                                if (m.move_unused_budget_recommendation != null && m.hasOwnProperty("move_unused_budget_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.verify(m.move_unused_budget_recommendation);
                                        if (e)
                                            return "move_unused_budget_recommendation." + e;
                                    }
                                }
                                if (m.target_roas_opt_in_recommendation != null && m.hasOwnProperty("target_roas_opt_in_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation.verify(m.target_roas_opt_in_recommendation);
                                        if (e)
                                            return "target_roas_opt_in_recommendation." + e;
                                    }
                                }
                                if (m.responsive_search_ad_recommendation != null && m.hasOwnProperty("responsive_search_ad_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.verify(m.responsive_search_ad_recommendation);
                                        if (e)
                                            return "responsive_search_ad_recommendation." + e;
                                    }
                                }
                                if (m.marginal_roi_campaign_budget_recommendation != null && m.hasOwnProperty("marginal_roi_campaign_budget_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.verify(m.marginal_roi_campaign_budget_recommendation);
                                        if (e)
                                            return "marginal_roi_campaign_budget_recommendation." + e;
                                    }
                                }
                                if (m.use_broad_match_keyword_recommendation != null && m.hasOwnProperty("use_broad_match_keyword_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.verify(m.use_broad_match_keyword_recommendation);
                                        if (e)
                                            return "use_broad_match_keyword_recommendation." + e;
                                    }
                                }
                                if (m.responsive_search_ad_asset_recommendation != null && m.hasOwnProperty("responsive_search_ad_asset_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.verify(m.responsive_search_ad_asset_recommendation);
                                        if (e)
                                            return "responsive_search_ad_asset_recommendation." + e;
                                    }
                                }
                                if (m.upgrade_smart_shopping_campaign_to_performance_max_recommendation != null && m.hasOwnProperty("upgrade_smart_shopping_campaign_to_performance_max_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.verify(m.upgrade_smart_shopping_campaign_to_performance_max_recommendation);
                                        if (e)
                                            return "upgrade_smart_shopping_campaign_to_performance_max_recommendation." + e;
                                    }
                                }
                                if (m.responsive_search_ad_improve_ad_strength_recommendation != null && m.hasOwnProperty("responsive_search_ad_improve_ad_strength_recommendation")) {
                                    if (p.recommendation === 1)
                                        return "recommendation: multiple values";
                                    p.recommendation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.verify(m.responsive_search_ad_improve_ad_strength_recommendation);
                                        if (e)
                                            return "responsive_search_ad_improve_ad_strength_recommendation." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Recommendation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Recommendation} Recommendation
                             */
                            Recommendation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Recommendation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "CAMPAIGN_BUDGET":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "KEYWORD":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "TEXT_AD":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "TARGET_CPA_OPT_IN":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "MAXIMIZE_CONVERSIONS_OPT_IN":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "ENHANCED_CPC_OPT_IN":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "SEARCH_PARTNERS_OPT_IN":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "MAXIMIZE_CLICKS_OPT_IN":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "OPTIMIZE_AD_ROTATION":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "CALLOUT_EXTENSION":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "SITELINK_EXTENSION":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "CALL_EXTENSION":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "KEYWORD_MATCH_TYPE":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "MOVE_UNUSED_BUDGET":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "FORECASTING_CAMPAIGN_BUDGET":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "TARGET_ROAS_OPT_IN":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "RESPONSIVE_SEARCH_AD":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "MARGINAL_ROI_CAMPAIGN_BUDGET":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "USE_BROAD_MATCH_KEYWORD":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "RESPONSIVE_SEARCH_AD_ASSET":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH":
                                case 23:
                                    m.type = 23;
                                    break;
                                }
                                if (d.impact != null) {
                                    if (typeof d.impact !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.impact: object expected");
                                    m.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.fromObject(d.impact);
                                }
                                if (d.campaign_budget != null) {
                                    m.campaign_budget = String(d.campaign_budget);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.dismissed != null) {
                                    m.dismissed = Boolean(d.dismissed);
                                }
                                if (d.campaign_budget_recommendation != null) {
                                    if (typeof d.campaign_budget_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.campaign_budget_recommendation: object expected");
                                    m.campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.fromObject(d.campaign_budget_recommendation);
                                }
                                if (d.forecasting_campaign_budget_recommendation != null) {
                                    if (typeof d.forecasting_campaign_budget_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.forecasting_campaign_budget_recommendation: object expected");
                                    m.forecasting_campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.fromObject(d.forecasting_campaign_budget_recommendation);
                                }
                                if (d.keyword_recommendation != null) {
                                    if (typeof d.keyword_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.keyword_recommendation: object expected");
                                    m.keyword_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.fromObject(d.keyword_recommendation);
                                }
                                if (d.text_ad_recommendation != null) {
                                    if (typeof d.text_ad_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.text_ad_recommendation: object expected");
                                    m.text_ad_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.fromObject(d.text_ad_recommendation);
                                }
                                if (d.target_cpa_opt_in_recommendation != null) {
                                    if (typeof d.target_cpa_opt_in_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.target_cpa_opt_in_recommendation: object expected");
                                    m.target_cpa_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.fromObject(d.target_cpa_opt_in_recommendation);
                                }
                                if (d.maximize_conversions_opt_in_recommendation != null) {
                                    if (typeof d.maximize_conversions_opt_in_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.maximize_conversions_opt_in_recommendation: object expected");
                                    m.maximize_conversions_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation.fromObject(d.maximize_conversions_opt_in_recommendation);
                                }
                                if (d.enhanced_cpc_opt_in_recommendation != null) {
                                    if (typeof d.enhanced_cpc_opt_in_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.enhanced_cpc_opt_in_recommendation: object expected");
                                    m.enhanced_cpc_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation.fromObject(d.enhanced_cpc_opt_in_recommendation);
                                }
                                if (d.search_partners_opt_in_recommendation != null) {
                                    if (typeof d.search_partners_opt_in_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.search_partners_opt_in_recommendation: object expected");
                                    m.search_partners_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation.fromObject(d.search_partners_opt_in_recommendation);
                                }
                                if (d.maximize_clicks_opt_in_recommendation != null) {
                                    if (typeof d.maximize_clicks_opt_in_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.maximize_clicks_opt_in_recommendation: object expected");
                                    m.maximize_clicks_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation.fromObject(d.maximize_clicks_opt_in_recommendation);
                                }
                                if (d.optimize_ad_rotation_recommendation != null) {
                                    if (typeof d.optimize_ad_rotation_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.optimize_ad_rotation_recommendation: object expected");
                                    m.optimize_ad_rotation_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation.fromObject(d.optimize_ad_rotation_recommendation);
                                }
                                if (d.callout_extension_recommendation != null) {
                                    if (typeof d.callout_extension_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.callout_extension_recommendation: object expected");
                                    m.callout_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.fromObject(d.callout_extension_recommendation);
                                }
                                if (d.sitelink_extension_recommendation != null) {
                                    if (typeof d.sitelink_extension_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.sitelink_extension_recommendation: object expected");
                                    m.sitelink_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.fromObject(d.sitelink_extension_recommendation);
                                }
                                if (d.call_extension_recommendation != null) {
                                    if (typeof d.call_extension_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.call_extension_recommendation: object expected");
                                    m.call_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.fromObject(d.call_extension_recommendation);
                                }
                                if (d.keyword_match_type_recommendation != null) {
                                    if (typeof d.keyword_match_type_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.keyword_match_type_recommendation: object expected");
                                    m.keyword_match_type_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.fromObject(d.keyword_match_type_recommendation);
                                }
                                if (d.move_unused_budget_recommendation != null) {
                                    if (typeof d.move_unused_budget_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.move_unused_budget_recommendation: object expected");
                                    m.move_unused_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.fromObject(d.move_unused_budget_recommendation);
                                }
                                if (d.target_roas_opt_in_recommendation != null) {
                                    if (typeof d.target_roas_opt_in_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.target_roas_opt_in_recommendation: object expected");
                                    m.target_roas_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation.fromObject(d.target_roas_opt_in_recommendation);
                                }
                                if (d.responsive_search_ad_recommendation != null) {
                                    if (typeof d.responsive_search_ad_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.responsive_search_ad_recommendation: object expected");
                                    m.responsive_search_ad_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.fromObject(d.responsive_search_ad_recommendation);
                                }
                                if (d.marginal_roi_campaign_budget_recommendation != null) {
                                    if (typeof d.marginal_roi_campaign_budget_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.marginal_roi_campaign_budget_recommendation: object expected");
                                    m.marginal_roi_campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.fromObject(d.marginal_roi_campaign_budget_recommendation);
                                }
                                if (d.use_broad_match_keyword_recommendation != null) {
                                    if (typeof d.use_broad_match_keyword_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.use_broad_match_keyword_recommendation: object expected");
                                    m.use_broad_match_keyword_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.fromObject(d.use_broad_match_keyword_recommendation);
                                }
                                if (d.responsive_search_ad_asset_recommendation != null) {
                                    if (typeof d.responsive_search_ad_asset_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.responsive_search_ad_asset_recommendation: object expected");
                                    m.responsive_search_ad_asset_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.fromObject(d.responsive_search_ad_asset_recommendation);
                                }
                                if (d.upgrade_smart_shopping_campaign_to_performance_max_recommendation != null) {
                                    if (typeof d.upgrade_smart_shopping_campaign_to_performance_max_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.upgrade_smart_shopping_campaign_to_performance_max_recommendation: object expected");
                                    m.upgrade_smart_shopping_campaign_to_performance_max_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.fromObject(d.upgrade_smart_shopping_campaign_to_performance_max_recommendation);
                                }
                                if (d.responsive_search_ad_improve_ad_strength_recommendation != null) {
                                    if (typeof d.responsive_search_ad_improve_ad_strength_recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.Recommendation.responsive_search_ad_improve_ad_strength_recommendation: object expected");
                                    m.responsive_search_ad_improve_ad_strength_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.fromObject(d.responsive_search_ad_improve_ad_strength_recommendation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Recommendation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @static
                             * @param {google.ads.googleads.v11.resources.Recommendation} m Recommendation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Recommendation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.impact = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.RecommendationTypeEnum.RecommendationType[m.type] : m.type;
                                }
                                if (m.impact != null && m.hasOwnProperty("impact")) {
                                    d.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.toObject(m.impact, o);
                                }
                                if (m.campaign_budget_recommendation != null && m.hasOwnProperty("campaign_budget_recommendation")) {
                                    d.campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.toObject(m.campaign_budget_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "campaign_budget_recommendation";
                                }
                                if (m.keyword_recommendation != null && m.hasOwnProperty("keyword_recommendation")) {
                                    d.keyword_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.toObject(m.keyword_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "keyword_recommendation";
                                }
                                if (m.text_ad_recommendation != null && m.hasOwnProperty("text_ad_recommendation")) {
                                    d.text_ad_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.toObject(m.text_ad_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "text_ad_recommendation";
                                }
                                if (m.target_cpa_opt_in_recommendation != null && m.hasOwnProperty("target_cpa_opt_in_recommendation")) {
                                    d.target_cpa_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.toObject(m.target_cpa_opt_in_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "target_cpa_opt_in_recommendation";
                                }
                                if (m.maximize_conversions_opt_in_recommendation != null && m.hasOwnProperty("maximize_conversions_opt_in_recommendation")) {
                                    d.maximize_conversions_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation.toObject(m.maximize_conversions_opt_in_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "maximize_conversions_opt_in_recommendation";
                                }
                                if (m.enhanced_cpc_opt_in_recommendation != null && m.hasOwnProperty("enhanced_cpc_opt_in_recommendation")) {
                                    d.enhanced_cpc_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation.toObject(m.enhanced_cpc_opt_in_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "enhanced_cpc_opt_in_recommendation";
                                }
                                if (m.search_partners_opt_in_recommendation != null && m.hasOwnProperty("search_partners_opt_in_recommendation")) {
                                    d.search_partners_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation.toObject(m.search_partners_opt_in_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "search_partners_opt_in_recommendation";
                                }
                                if (m.maximize_clicks_opt_in_recommendation != null && m.hasOwnProperty("maximize_clicks_opt_in_recommendation")) {
                                    d.maximize_clicks_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation.toObject(m.maximize_clicks_opt_in_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "maximize_clicks_opt_in_recommendation";
                                }
                                if (m.optimize_ad_rotation_recommendation != null && m.hasOwnProperty("optimize_ad_rotation_recommendation")) {
                                    d.optimize_ad_rotation_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation.toObject(m.optimize_ad_rotation_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "optimize_ad_rotation_recommendation";
                                }
                                if (m.callout_extension_recommendation != null && m.hasOwnProperty("callout_extension_recommendation")) {
                                    d.callout_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.toObject(m.callout_extension_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "callout_extension_recommendation";
                                }
                                if (m.sitelink_extension_recommendation != null && m.hasOwnProperty("sitelink_extension_recommendation")) {
                                    d.sitelink_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.toObject(m.sitelink_extension_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "sitelink_extension_recommendation";
                                }
                                if (m.call_extension_recommendation != null && m.hasOwnProperty("call_extension_recommendation")) {
                                    d.call_extension_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.toObject(m.call_extension_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "call_extension_recommendation";
                                }
                                if (m.keyword_match_type_recommendation != null && m.hasOwnProperty("keyword_match_type_recommendation")) {
                                    d.keyword_match_type_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.toObject(m.keyword_match_type_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "keyword_match_type_recommendation";
                                }
                                if (m.move_unused_budget_recommendation != null && m.hasOwnProperty("move_unused_budget_recommendation")) {
                                    d.move_unused_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.toObject(m.move_unused_budget_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "move_unused_budget_recommendation";
                                }
                                if (m.forecasting_campaign_budget_recommendation != null && m.hasOwnProperty("forecasting_campaign_budget_recommendation")) {
                                    d.forecasting_campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.toObject(m.forecasting_campaign_budget_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "forecasting_campaign_budget_recommendation";
                                }
                                if (m.target_roas_opt_in_recommendation != null && m.hasOwnProperty("target_roas_opt_in_recommendation")) {
                                    d.target_roas_opt_in_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation.toObject(m.target_roas_opt_in_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "target_roas_opt_in_recommendation";
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    d.campaign_budget = m.campaign_budget;
                                    if (o.oneofs)
                                        d._campaign_budget = "campaign_budget";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d._campaign = "campaign";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                if (m.dismissed != null && m.hasOwnProperty("dismissed")) {
                                    d.dismissed = m.dismissed;
                                    if (o.oneofs)
                                        d._dismissed = "dismissed";
                                }
                                if (m.responsive_search_ad_recommendation != null && m.hasOwnProperty("responsive_search_ad_recommendation")) {
                                    d.responsive_search_ad_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.toObject(m.responsive_search_ad_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "responsive_search_ad_recommendation";
                                }
                                if (m.marginal_roi_campaign_budget_recommendation != null && m.hasOwnProperty("marginal_roi_campaign_budget_recommendation")) {
                                    d.marginal_roi_campaign_budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.toObject(m.marginal_roi_campaign_budget_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "marginal_roi_campaign_budget_recommendation";
                                }
                                if (m.use_broad_match_keyword_recommendation != null && m.hasOwnProperty("use_broad_match_keyword_recommendation")) {
                                    d.use_broad_match_keyword_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.toObject(m.use_broad_match_keyword_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "use_broad_match_keyword_recommendation";
                                }
                                if (m.responsive_search_ad_asset_recommendation != null && m.hasOwnProperty("responsive_search_ad_asset_recommendation")) {
                                    d.responsive_search_ad_asset_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.toObject(m.responsive_search_ad_asset_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "responsive_search_ad_asset_recommendation";
                                }
                                if (m.upgrade_smart_shopping_campaign_to_performance_max_recommendation != null && m.hasOwnProperty("upgrade_smart_shopping_campaign_to_performance_max_recommendation")) {
                                    d.upgrade_smart_shopping_campaign_to_performance_max_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.toObject(m.upgrade_smart_shopping_campaign_to_performance_max_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "upgrade_smart_shopping_campaign_to_performance_max_recommendation";
                                }
                                if (m.responsive_search_ad_improve_ad_strength_recommendation != null && m.hasOwnProperty("responsive_search_ad_improve_ad_strength_recommendation")) {
                                    d.responsive_search_ad_improve_ad_strength_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.toObject(m.responsive_search_ad_improve_ad_strength_recommendation, o);
                                    if (o.oneofs)
                                        d.recommendation = "responsive_search_ad_improve_ad_strength_recommendation";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Recommendation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Recommendation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Recommendation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            Recommendation.RecommendationImpact = (function() {
    
                                /**
                                 * Properties of a RecommendationImpact.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IRecommendationImpact
                                 * @property {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics|null} [base_metrics] RecommendationImpact base_metrics
                                 * @property {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics|null} [potential_metrics] RecommendationImpact potential_metrics
                                 */
    
                                /**
                                 * Constructs a new RecommendationImpact.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a RecommendationImpact.
                                 * @implements IRecommendationImpact
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact=} [p] Properties to set
                                 */
                                function RecommendationImpact(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * RecommendationImpact base_metrics.
                                 * @member {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics|null|undefined} base_metrics
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @instance
                                 */
                                RecommendationImpact.prototype.base_metrics = null;
    
                                /**
                                 * RecommendationImpact potential_metrics.
                                 * @member {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics|null|undefined} potential_metrics
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @instance
                                 */
                                RecommendationImpact.prototype.potential_metrics = null;
    
                                /**
                                 * Creates a new RecommendationImpact instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationImpact} RecommendationImpact instance
                                 */
                                RecommendationImpact.create = function create(properties) {
                                    return new RecommendationImpact(properties);
                                };
    
                                /**
                                 * Encodes the specified RecommendationImpact message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact} m RecommendationImpact message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                RecommendationImpact.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.base_metrics != null && Object.hasOwnProperty.call(m, "base_metrics"))
                                        $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.encode(m.base_metrics, w.uint32(10).fork()).ldelim();
                                    if (m.potential_metrics != null && Object.hasOwnProperty.call(m, "potential_metrics"))
                                        $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.encode(m.potential_metrics, w.uint32(18).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified RecommendationImpact message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact} message RecommendationImpact message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                RecommendationImpact.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a RecommendationImpact message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationImpact} RecommendationImpact
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                RecommendationImpact.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.base_metrics = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.decode(r, r.uint32());
                                            break;
                                        case 2:
                                            m.potential_metrics = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a RecommendationImpact message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationImpact} RecommendationImpact
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                RecommendationImpact.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a RecommendationImpact message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                RecommendationImpact.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.base_metrics != null && m.hasOwnProperty("base_metrics")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.verify(m.base_metrics);
                                            if (e)
                                                return "base_metrics." + e;
                                        }
                                    }
                                    if (m.potential_metrics != null && m.hasOwnProperty("potential_metrics")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.verify(m.potential_metrics);
                                            if (e)
                                                return "potential_metrics." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a RecommendationImpact message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationImpact} RecommendationImpact
                                 */
                                RecommendationImpact.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact();
                                    if (d.base_metrics != null) {
                                        if (typeof d.base_metrics !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.base_metrics: object expected");
                                        m.base_metrics = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.fromObject(d.base_metrics);
                                    }
                                    if (d.potential_metrics != null) {
                                        if (typeof d.potential_metrics !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.potential_metrics: object expected");
                                        m.potential_metrics = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.fromObject(d.potential_metrics);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a RecommendationImpact message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.RecommendationImpact} m RecommendationImpact
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                RecommendationImpact.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.base_metrics = null;
                                        d.potential_metrics = null;
                                    }
                                    if (m.base_metrics != null && m.hasOwnProperty("base_metrics")) {
                                        d.base_metrics = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.toObject(m.base_metrics, o);
                                    }
                                    if (m.potential_metrics != null && m.hasOwnProperty("potential_metrics")) {
                                        d.potential_metrics = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.toObject(m.potential_metrics, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this RecommendationImpact to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationImpact
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                RecommendationImpact.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return RecommendationImpact;
                            })();
    
                            Recommendation.RecommendationMetrics = (function() {
    
                                /**
                                 * Properties of a RecommendationMetrics.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IRecommendationMetrics
                                 * @property {number|null} [impressions] RecommendationMetrics impressions
                                 * @property {number|null} [clicks] RecommendationMetrics clicks
                                 * @property {number|null} [cost_micros] RecommendationMetrics cost_micros
                                 * @property {number|null} [conversions] RecommendationMetrics conversions
                                 * @property {number|null} [video_views] RecommendationMetrics video_views
                                 */
    
                                /**
                                 * Constructs a new RecommendationMetrics.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a RecommendationMetrics.
                                 * @implements IRecommendationMetrics
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics=} [p] Properties to set
                                 */
                                function RecommendationMetrics(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * RecommendationMetrics impressions.
                                 * @member {number|null|undefined} impressions
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                RecommendationMetrics.prototype.impressions = null;
    
                                /**
                                 * RecommendationMetrics clicks.
                                 * @member {number|null|undefined} clicks
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                RecommendationMetrics.prototype.clicks = null;
    
                                /**
                                 * RecommendationMetrics cost_micros.
                                 * @member {number|null|undefined} cost_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                RecommendationMetrics.prototype.cost_micros = null;
    
                                /**
                                 * RecommendationMetrics conversions.
                                 * @member {number|null|undefined} conversions
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                RecommendationMetrics.prototype.conversions = null;
    
                                /**
                                 * RecommendationMetrics video_views.
                                 * @member {number|null|undefined} video_views
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                RecommendationMetrics.prototype.video_views = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * RecommendationMetrics _impressions.
                                 * @member {"impressions"|undefined} _impressions
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                Object.defineProperty(RecommendationMetrics.prototype, "_impressions", {
                                    get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * RecommendationMetrics _clicks.
                                 * @member {"clicks"|undefined} _clicks
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                Object.defineProperty(RecommendationMetrics.prototype, "_clicks", {
                                    get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * RecommendationMetrics _cost_micros.
                                 * @member {"cost_micros"|undefined} _cost_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                Object.defineProperty(RecommendationMetrics.prototype, "_cost_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * RecommendationMetrics _conversions.
                                 * @member {"conversions"|undefined} _conversions
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                Object.defineProperty(RecommendationMetrics.prototype, "_conversions", {
                                    get: $util.oneOfGetter($oneOfFields = ["conversions"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * RecommendationMetrics _video_views.
                                 * @member {"video_views"|undefined} _video_views
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 */
                                Object.defineProperty(RecommendationMetrics.prototype, "_video_views", {
                                    get: $util.oneOfGetter($oneOfFields = ["video_views"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new RecommendationMetrics instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics} RecommendationMetrics instance
                                 */
                                RecommendationMetrics.create = function create(properties) {
                                    return new RecommendationMetrics(properties);
                                };
    
                                /**
                                 * Encodes the specified RecommendationMetrics message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics} m RecommendationMetrics message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                RecommendationMetrics.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                        w.uint32(49).double(m.impressions);
                                    if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                        w.uint32(57).double(m.clicks);
                                    if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                        w.uint32(64).int64(m.cost_micros);
                                    if (m.conversions != null && Object.hasOwnProperty.call(m, "conversions"))
                                        w.uint32(73).double(m.conversions);
                                    if (m.video_views != null && Object.hasOwnProperty.call(m, "video_views"))
                                        w.uint32(81).double(m.video_views);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified RecommendationMetrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IRecommendationMetrics} message RecommendationMetrics message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                RecommendationMetrics.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a RecommendationMetrics message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics} RecommendationMetrics
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                RecommendationMetrics.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 6:
                                            m.impressions = r.double();
                                            break;
                                        case 7:
                                            m.clicks = r.double();
                                            break;
                                        case 8:
                                            m.cost_micros = r.int64();
                                            break;
                                        case 9:
                                            m.conversions = r.double();
                                            break;
                                        case 10:
                                            m.video_views = r.double();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a RecommendationMetrics message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics} RecommendationMetrics
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                RecommendationMetrics.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a RecommendationMetrics message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                RecommendationMetrics.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                        p._impressions = 1;
                                        if (typeof m.impressions !== "number")
                                            return "impressions: number expected";
                                    }
                                    if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                        p._clicks = 1;
                                        if (typeof m.clicks !== "number")
                                            return "clicks: number expected";
                                    }
                                    if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                        p._cost_micros = 1;
                                        if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                            return "cost_micros: integer|Long expected";
                                    }
                                    if (m.conversions != null && m.hasOwnProperty("conversions")) {
                                        p._conversions = 1;
                                        if (typeof m.conversions !== "number")
                                            return "conversions: number expected";
                                    }
                                    if (m.video_views != null && m.hasOwnProperty("video_views")) {
                                        p._video_views = 1;
                                        if (typeof m.video_views !== "number")
                                            return "video_views: number expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a RecommendationMetrics message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics} RecommendationMetrics
                                 */
                                RecommendationMetrics.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics();
                                    if (d.impressions != null) {
                                        m.impressions = Number(d.impressions);
                                    }
                                    if (d.clicks != null) {
                                        m.clicks = Number(d.clicks);
                                    }
                                    if (d.cost_micros != null) {
                                        if ($util.Long)
                                            (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                        else if (typeof d.cost_micros === "string")
                                            m.cost_micros = parseInt(d.cost_micros, 10);
                                        else if (typeof d.cost_micros === "number")
                                            m.cost_micros = d.cost_micros;
                                        else if (typeof d.cost_micros === "object")
                                            m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                    }
                                    if (d.conversions != null) {
                                        m.conversions = Number(d.conversions);
                                    }
                                    if (d.video_views != null) {
                                        m.video_views = Number(d.video_views);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a RecommendationMetrics message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics} m RecommendationMetrics
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                RecommendationMetrics.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                        d.impressions = o.json && !isFinite(m.impressions) ? String(m.impressions) : m.impressions;
                                        if (o.oneofs)
                                            d._impressions = "impressions";
                                    }
                                    if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                        d.clicks = o.json && !isFinite(m.clicks) ? String(m.clicks) : m.clicks;
                                        if (o.oneofs)
                                            d._clicks = "clicks";
                                    }
                                    if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                        if (typeof m.cost_micros === "number")
                                            d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                        else
                                            d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                        if (o.oneofs)
                                            d._cost_micros = "cost_micros";
                                    }
                                    if (m.conversions != null && m.hasOwnProperty("conversions")) {
                                        d.conversions = o.json && !isFinite(m.conversions) ? String(m.conversions) : m.conversions;
                                        if (o.oneofs)
                                            d._conversions = "conversions";
                                    }
                                    if (m.video_views != null && m.hasOwnProperty("video_views")) {
                                        d.video_views = o.json && !isFinite(m.video_views) ? String(m.video_views) : m.video_views;
                                        if (o.oneofs)
                                            d._video_views = "video_views";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this RecommendationMetrics to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.RecommendationMetrics
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                RecommendationMetrics.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return RecommendationMetrics;
                            })();
    
                            Recommendation.CampaignBudgetRecommendation = (function() {
    
                                /**
                                 * Properties of a CampaignBudgetRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ICampaignBudgetRecommendation
                                 * @property {number|null} [current_budget_amount_micros] CampaignBudgetRecommendation current_budget_amount_micros
                                 * @property {number|null} [recommended_budget_amount_micros] CampaignBudgetRecommendation recommended_budget_amount_micros
                                 * @property {Array.<google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.ICampaignBudgetRecommendationOption>|null} [budget_options] CampaignBudgetRecommendation budget_options
                                 */
    
                                /**
                                 * Constructs a new CampaignBudgetRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a CampaignBudgetRecommendation.
                                 * @implements ICampaignBudgetRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation=} [p] Properties to set
                                 */
                                function CampaignBudgetRecommendation(p) {
                                    this.budget_options = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CampaignBudgetRecommendation current_budget_amount_micros.
                                 * @member {number|null|undefined} current_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @instance
                                 */
                                CampaignBudgetRecommendation.prototype.current_budget_amount_micros = null;
    
                                /**
                                 * CampaignBudgetRecommendation recommended_budget_amount_micros.
                                 * @member {number|null|undefined} recommended_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @instance
                                 */
                                CampaignBudgetRecommendation.prototype.recommended_budget_amount_micros = null;
    
                                /**
                                 * CampaignBudgetRecommendation budget_options.
                                 * @member {Array.<google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.ICampaignBudgetRecommendationOption>} budget_options
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @instance
                                 */
                                CampaignBudgetRecommendation.prototype.budget_options = $util.emptyArray;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * CampaignBudgetRecommendation _current_budget_amount_micros.
                                 * @member {"current_budget_amount_micros"|undefined} _current_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(CampaignBudgetRecommendation.prototype, "_current_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["current_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * CampaignBudgetRecommendation _recommended_budget_amount_micros.
                                 * @member {"recommended_budget_amount_micros"|undefined} _recommended_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(CampaignBudgetRecommendation.prototype, "_recommended_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["recommended_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new CampaignBudgetRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation} CampaignBudgetRecommendation instance
                                 */
                                CampaignBudgetRecommendation.create = function create(properties) {
                                    return new CampaignBudgetRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified CampaignBudgetRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation} m CampaignBudgetRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CampaignBudgetRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.budget_options != null && m.budget_options.length) {
                                        for (var i = 0; i < m.budget_options.length; ++i)
                                            $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.encode(m.budget_options[i], w.uint32(26).fork()).ldelim();
                                    }
                                    if (m.current_budget_amount_micros != null && Object.hasOwnProperty.call(m, "current_budget_amount_micros"))
                                        w.uint32(56).int64(m.current_budget_amount_micros);
                                    if (m.recommended_budget_amount_micros != null && Object.hasOwnProperty.call(m, "recommended_budget_amount_micros"))
                                        w.uint32(64).int64(m.recommended_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CampaignBudgetRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation} message CampaignBudgetRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CampaignBudgetRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CampaignBudgetRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation} CampaignBudgetRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CampaignBudgetRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 7:
                                            m.current_budget_amount_micros = r.int64();
                                            break;
                                        case 8:
                                            m.recommended_budget_amount_micros = r.int64();
                                            break;
                                        case 3:
                                            if (!(m.budget_options && m.budget_options.length))
                                                m.budget_options = [];
                                            m.budget_options.push($root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CampaignBudgetRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation} CampaignBudgetRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CampaignBudgetRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CampaignBudgetRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CampaignBudgetRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.current_budget_amount_micros != null && m.hasOwnProperty("current_budget_amount_micros")) {
                                        p._current_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.current_budget_amount_micros) && !(m.current_budget_amount_micros && $util.isInteger(m.current_budget_amount_micros.low) && $util.isInteger(m.current_budget_amount_micros.high)))
                                            return "current_budget_amount_micros: integer|Long expected";
                                    }
                                    if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                        p._recommended_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.recommended_budget_amount_micros) && !(m.recommended_budget_amount_micros && $util.isInteger(m.recommended_budget_amount_micros.low) && $util.isInteger(m.recommended_budget_amount_micros.high)))
                                            return "recommended_budget_amount_micros: integer|Long expected";
                                    }
                                    if (m.budget_options != null && m.hasOwnProperty("budget_options")) {
                                        if (!Array.isArray(m.budget_options))
                                            return "budget_options: array expected";
                                        for (var i = 0; i < m.budget_options.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.verify(m.budget_options[i]);
                                                if (e)
                                                    return "budget_options." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CampaignBudgetRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation} CampaignBudgetRecommendation
                                 */
                                CampaignBudgetRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation();
                                    if (d.current_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.current_budget_amount_micros = $util.Long.fromValue(d.current_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.current_budget_amount_micros === "string")
                                            m.current_budget_amount_micros = parseInt(d.current_budget_amount_micros, 10);
                                        else if (typeof d.current_budget_amount_micros === "number")
                                            m.current_budget_amount_micros = d.current_budget_amount_micros;
                                        else if (typeof d.current_budget_amount_micros === "object")
                                            m.current_budget_amount_micros = new $util.LongBits(d.current_budget_amount_micros.low >>> 0, d.current_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    if (d.recommended_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.recommended_budget_amount_micros = $util.Long.fromValue(d.recommended_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.recommended_budget_amount_micros === "string")
                                            m.recommended_budget_amount_micros = parseInt(d.recommended_budget_amount_micros, 10);
                                        else if (typeof d.recommended_budget_amount_micros === "number")
                                            m.recommended_budget_amount_micros = d.recommended_budget_amount_micros;
                                        else if (typeof d.recommended_budget_amount_micros === "object")
                                            m.recommended_budget_amount_micros = new $util.LongBits(d.recommended_budget_amount_micros.low >>> 0, d.recommended_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    if (d.budget_options) {
                                        if (!Array.isArray(d.budget_options))
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.budget_options: array expected");
                                        m.budget_options = [];
                                        for (var i = 0; i < d.budget_options.length; ++i) {
                                            if (typeof d.budget_options[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.budget_options: object expected");
                                            m.budget_options[i] = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.fromObject(d.budget_options[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CampaignBudgetRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation} m CampaignBudgetRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CampaignBudgetRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.budget_options = [];
                                    }
                                    if (m.budget_options && m.budget_options.length) {
                                        d.budget_options = [];
                                        for (var j = 0; j < m.budget_options.length; ++j) {
                                            d.budget_options[j] = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.toObject(m.budget_options[j], o);
                                        }
                                    }
                                    if (m.current_budget_amount_micros != null && m.hasOwnProperty("current_budget_amount_micros")) {
                                        if (typeof m.current_budget_amount_micros === "number")
                                            d.current_budget_amount_micros = o.longs === String ? String(m.current_budget_amount_micros) : m.current_budget_amount_micros;
                                        else
                                            d.current_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.current_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.current_budget_amount_micros.low >>> 0, m.current_budget_amount_micros.high >>> 0).toNumber() : m.current_budget_amount_micros;
                                        if (o.oneofs)
                                            d._current_budget_amount_micros = "current_budget_amount_micros";
                                    }
                                    if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                        if (typeof m.recommended_budget_amount_micros === "number")
                                            d.recommended_budget_amount_micros = o.longs === String ? String(m.recommended_budget_amount_micros) : m.recommended_budget_amount_micros;
                                        else
                                            d.recommended_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.recommended_budget_amount_micros.low >>> 0, m.recommended_budget_amount_micros.high >>> 0).toNumber() : m.recommended_budget_amount_micros;
                                        if (o.oneofs)
                                            d._recommended_budget_amount_micros = "recommended_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CampaignBudgetRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CampaignBudgetRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                CampaignBudgetRecommendation.CampaignBudgetRecommendationOption = (function() {
    
                                    /**
                                     * Properties of a CampaignBudgetRecommendationOption.
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                     * @interface ICampaignBudgetRecommendationOption
                                     * @property {number|null} [budget_amount_micros] CampaignBudgetRecommendationOption budget_amount_micros
                                     * @property {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact|null} [impact] CampaignBudgetRecommendationOption impact
                                     */
    
                                    /**
                                     * Constructs a new CampaignBudgetRecommendationOption.
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation
                                     * @classdesc Represents a CampaignBudgetRecommendationOption.
                                     * @implements ICampaignBudgetRecommendationOption
                                     * @constructor
                                     * @param {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.ICampaignBudgetRecommendationOption=} [p] Properties to set
                                     */
                                    function CampaignBudgetRecommendationOption(p) {
                                        if (p)
                                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                                if (p[ks[i]] != null)
                                                    this[ks[i]] = p[ks[i]];
                                    }
    
                                    /**
                                     * CampaignBudgetRecommendationOption budget_amount_micros.
                                     * @member {number|null|undefined} budget_amount_micros
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @instance
                                     */
                                    CampaignBudgetRecommendationOption.prototype.budget_amount_micros = null;
    
                                    /**
                                     * CampaignBudgetRecommendationOption impact.
                                     * @member {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact|null|undefined} impact
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @instance
                                     */
                                    CampaignBudgetRecommendationOption.prototype.impact = null;
    
                                    // OneOf field names bound to virtual getters and setters
                                    var $oneOfFields;
    
                                    /**
                                     * CampaignBudgetRecommendationOption _budget_amount_micros.
                                     * @member {"budget_amount_micros"|undefined} _budget_amount_micros
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @instance
                                     */
                                    Object.defineProperty(CampaignBudgetRecommendationOption.prototype, "_budget_amount_micros", {
                                        get: $util.oneOfGetter($oneOfFields = ["budget_amount_micros"]),
                                        set: $util.oneOfSetter($oneOfFields)
                                    });
    
                                    /**
                                     * Creates a new CampaignBudgetRecommendationOption instance using the specified properties.
                                     * @function create
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.ICampaignBudgetRecommendationOption=} [properties] Properties to set
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption} CampaignBudgetRecommendationOption instance
                                     */
                                    CampaignBudgetRecommendationOption.create = function create(properties) {
                                        return new CampaignBudgetRecommendationOption(properties);
                                    };
    
                                    /**
                                     * Encodes the specified CampaignBudgetRecommendationOption message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.verify|verify} messages.
                                     * @function encode
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.ICampaignBudgetRecommendationOption} m CampaignBudgetRecommendationOption message or plain object to encode
                                     * @param {$protobuf.Writer} [w] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    CampaignBudgetRecommendationOption.encode = function encode(m, w) {
                                        if (!w)
                                            w = $Writer.create();
                                        if (m.impact != null && Object.hasOwnProperty.call(m, "impact"))
                                            $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.encode(m.impact, w.uint32(18).fork()).ldelim();
                                        if (m.budget_amount_micros != null && Object.hasOwnProperty.call(m, "budget_amount_micros"))
                                            w.uint32(24).int64(m.budget_amount_micros);
                                        return w;
                                    };
    
                                    /**
                                     * Encodes the specified CampaignBudgetRecommendationOption message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.verify|verify} messages.
                                     * @function encodeDelimited
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.ICampaignBudgetRecommendationOption} message CampaignBudgetRecommendationOption message or plain object to encode
                                     * @param {$protobuf.Writer} [writer] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    CampaignBudgetRecommendationOption.encodeDelimited = function encodeDelimited(message, writer) {
                                        return this.encode(message, writer).ldelim();
                                    };
    
                                    /**
                                     * Decodes a CampaignBudgetRecommendationOption message from the specified reader or buffer.
                                     * @function decode
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                     * @param {number} [l] Message length if known beforehand
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption} CampaignBudgetRecommendationOption
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    CampaignBudgetRecommendationOption.decode = function decode(r, l) {
                                        if (!(r instanceof $Reader))
                                            r = $Reader.create(r);
                                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption();
                                        while (r.pos < c) {
                                            var t = r.uint32();
                                            switch (t >>> 3) {
                                            case 3:
                                                m.budget_amount_micros = r.int64();
                                                break;
                                            case 2:
                                                m.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.decode(r, r.uint32());
                                                break;
                                            default:
                                                r.skipType(t & 7);
                                                break;
                                            }
                                        }
                                        return m;
                                    };
    
                                    /**
                                     * Decodes a CampaignBudgetRecommendationOption message from the specified reader or buffer, length delimited.
                                     * @function decodeDelimited
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption} CampaignBudgetRecommendationOption
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    CampaignBudgetRecommendationOption.decodeDelimited = function decodeDelimited(reader) {
                                        if (!(reader instanceof $Reader))
                                            reader = new $Reader(reader);
                                        return this.decode(reader, reader.uint32());
                                    };
    
                                    /**
                                     * Verifies a CampaignBudgetRecommendationOption message.
                                     * @function verify
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {Object.<string,*>} m Plain object to verify
                                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                     */
                                    CampaignBudgetRecommendationOption.verify = function verify(m) {
                                        if (typeof m !== "object" || m === null)
                                            return "object expected";
                                        var p = {};
                                        if (m.budget_amount_micros != null && m.hasOwnProperty("budget_amount_micros")) {
                                            p._budget_amount_micros = 1;
                                            if (!$util.isInteger(m.budget_amount_micros) && !(m.budget_amount_micros && $util.isInteger(m.budget_amount_micros.low) && $util.isInteger(m.budget_amount_micros.high)))
                                                return "budget_amount_micros: integer|Long expected";
                                        }
                                        if (m.impact != null && m.hasOwnProperty("impact")) {
                                            {
                                                var e = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.verify(m.impact);
                                                if (e)
                                                    return "impact." + e;
                                            }
                                        }
                                        return null;
                                    };
    
                                    /**
                                     * Creates a CampaignBudgetRecommendationOption message from a plain object. Also converts values to their respective internal types.
                                     * @function fromObject
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {Object.<string,*>} d Plain object
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption} CampaignBudgetRecommendationOption
                                     */
                                    CampaignBudgetRecommendationOption.fromObject = function fromObject(d) {
                                        if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption)
                                            return d;
                                        var m = new $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption();
                                        if (d.budget_amount_micros != null) {
                                            if ($util.Long)
                                                (m.budget_amount_micros = $util.Long.fromValue(d.budget_amount_micros)).unsigned = false;
                                            else if (typeof d.budget_amount_micros === "string")
                                                m.budget_amount_micros = parseInt(d.budget_amount_micros, 10);
                                            else if (typeof d.budget_amount_micros === "number")
                                                m.budget_amount_micros = d.budget_amount_micros;
                                            else if (typeof d.budget_amount_micros === "object")
                                                m.budget_amount_micros = new $util.LongBits(d.budget_amount_micros.low >>> 0, d.budget_amount_micros.high >>> 0).toNumber();
                                        }
                                        if (d.impact != null) {
                                            if (typeof d.impact !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption.impact: object expected");
                                            m.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.fromObject(d.impact);
                                        }
                                        return m;
                                    };
    
                                    /**
                                     * Creates a plain object from a CampaignBudgetRecommendationOption message. Also converts values to other types if specified.
                                     * @function toObject
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption} m CampaignBudgetRecommendationOption
                                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                                     * @returns {Object.<string,*>} Plain object
                                     */
                                    CampaignBudgetRecommendationOption.toObject = function toObject(m, o) {
                                        if (!o)
                                            o = {};
                                        var d = {};
                                        if (o.defaults) {
                                            d.impact = null;
                                        }
                                        if (m.impact != null && m.hasOwnProperty("impact")) {
                                            d.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.toObject(m.impact, o);
                                        }
                                        if (m.budget_amount_micros != null && m.hasOwnProperty("budget_amount_micros")) {
                                            if (typeof m.budget_amount_micros === "number")
                                                d.budget_amount_micros = o.longs === String ? String(m.budget_amount_micros) : m.budget_amount_micros;
                                            else
                                                d.budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.budget_amount_micros.low >>> 0, m.budget_amount_micros.high >>> 0).toNumber() : m.budget_amount_micros;
                                            if (o.oneofs)
                                                d._budget_amount_micros = "budget_amount_micros";
                                        }
                                        return d;
                                    };
    
                                    /**
                                     * Converts this CampaignBudgetRecommendationOption to JSON.
                                     * @function toJSON
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
                                     * @instance
                                     * @returns {Object.<string,*>} JSON object
                                     */
                                    CampaignBudgetRecommendationOption.prototype.toJSON = function toJSON() {
                                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                    };
    
                                    return CampaignBudgetRecommendationOption;
                                })();
    
                                return CampaignBudgetRecommendation;
                            })();
    
                            Recommendation.KeywordRecommendation = (function() {
    
                                /**
                                 * Properties of a KeywordRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IKeywordRecommendation
                                 * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [keyword] KeywordRecommendation keyword
                                 * @property {number|null} [recommended_cpc_bid_micros] KeywordRecommendation recommended_cpc_bid_micros
                                 */
    
                                /**
                                 * Constructs a new KeywordRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a KeywordRecommendation.
                                 * @implements IKeywordRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordRecommendation=} [p] Properties to set
                                 */
                                function KeywordRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * KeywordRecommendation keyword.
                                 * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} keyword
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @instance
                                 */
                                KeywordRecommendation.prototype.keyword = null;
    
                                /**
                                 * KeywordRecommendation recommended_cpc_bid_micros.
                                 * @member {number|null|undefined} recommended_cpc_bid_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @instance
                                 */
                                KeywordRecommendation.prototype.recommended_cpc_bid_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * KeywordRecommendation _recommended_cpc_bid_micros.
                                 * @member {"recommended_cpc_bid_micros"|undefined} _recommended_cpc_bid_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(KeywordRecommendation.prototype, "_recommended_cpc_bid_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["recommended_cpc_bid_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new KeywordRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation} KeywordRecommendation instance
                                 */
                                KeywordRecommendation.create = function create(properties) {
                                    return new KeywordRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified KeywordRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordRecommendation} m KeywordRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                KeywordRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                        $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword, w.uint32(10).fork()).ldelim();
                                    if (m.recommended_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "recommended_cpc_bid_micros"))
                                        w.uint32(24).int64(m.recommended_cpc_bid_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified KeywordRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordRecommendation} message KeywordRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                KeywordRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a KeywordRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation} KeywordRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                KeywordRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                            break;
                                        case 3:
                                            m.recommended_cpc_bid_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a KeywordRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation} KeywordRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                KeywordRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a KeywordRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                KeywordRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword);
                                            if (e)
                                                return "keyword." + e;
                                        }
                                    }
                                    if (m.recommended_cpc_bid_micros != null && m.hasOwnProperty("recommended_cpc_bid_micros")) {
                                        p._recommended_cpc_bid_micros = 1;
                                        if (!$util.isInteger(m.recommended_cpc_bid_micros) && !(m.recommended_cpc_bid_micros && $util.isInteger(m.recommended_cpc_bid_micros.low) && $util.isInteger(m.recommended_cpc_bid_micros.high)))
                                            return "recommended_cpc_bid_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a KeywordRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation} KeywordRecommendation
                                 */
                                KeywordRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation();
                                    if (d.keyword != null) {
                                        if (typeof d.keyword !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation.keyword: object expected");
                                        m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword);
                                    }
                                    if (d.recommended_cpc_bid_micros != null) {
                                        if ($util.Long)
                                            (m.recommended_cpc_bid_micros = $util.Long.fromValue(d.recommended_cpc_bid_micros)).unsigned = false;
                                        else if (typeof d.recommended_cpc_bid_micros === "string")
                                            m.recommended_cpc_bid_micros = parseInt(d.recommended_cpc_bid_micros, 10);
                                        else if (typeof d.recommended_cpc_bid_micros === "number")
                                            m.recommended_cpc_bid_micros = d.recommended_cpc_bid_micros;
                                        else if (typeof d.recommended_cpc_bid_micros === "object")
                                            m.recommended_cpc_bid_micros = new $util.LongBits(d.recommended_cpc_bid_micros.low >>> 0, d.recommended_cpc_bid_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a KeywordRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation} m KeywordRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                KeywordRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.keyword = null;
                                    }
                                    if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                        d.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword, o);
                                    }
                                    if (m.recommended_cpc_bid_micros != null && m.hasOwnProperty("recommended_cpc_bid_micros")) {
                                        if (typeof m.recommended_cpc_bid_micros === "number")
                                            d.recommended_cpc_bid_micros = o.longs === String ? String(m.recommended_cpc_bid_micros) : m.recommended_cpc_bid_micros;
                                        else
                                            d.recommended_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.recommended_cpc_bid_micros.low >>> 0, m.recommended_cpc_bid_micros.high >>> 0).toNumber() : m.recommended_cpc_bid_micros;
                                        if (o.oneofs)
                                            d._recommended_cpc_bid_micros = "recommended_cpc_bid_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this KeywordRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                KeywordRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return KeywordRecommendation;
                            })();
    
                            Recommendation.TextAdRecommendation = (function() {
    
                                /**
                                 * Properties of a TextAdRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ITextAdRecommendation
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [ad] TextAdRecommendation ad
                                 * @property {string|null} [creation_date] TextAdRecommendation creation_date
                                 * @property {string|null} [auto_apply_date] TextAdRecommendation auto_apply_date
                                 */
    
                                /**
                                 * Constructs a new TextAdRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a TextAdRecommendation.
                                 * @implements ITextAdRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITextAdRecommendation=} [p] Properties to set
                                 */
                                function TextAdRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TextAdRecommendation ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} ad
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @instance
                                 */
                                TextAdRecommendation.prototype.ad = null;
    
                                /**
                                 * TextAdRecommendation creation_date.
                                 * @member {string|null|undefined} creation_date
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @instance
                                 */
                                TextAdRecommendation.prototype.creation_date = null;
    
                                /**
                                 * TextAdRecommendation auto_apply_date.
                                 * @member {string|null|undefined} auto_apply_date
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @instance
                                 */
                                TextAdRecommendation.prototype.auto_apply_date = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TextAdRecommendation _creation_date.
                                 * @member {"creation_date"|undefined} _creation_date
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(TextAdRecommendation.prototype, "_creation_date", {
                                    get: $util.oneOfGetter($oneOfFields = ["creation_date"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * TextAdRecommendation _auto_apply_date.
                                 * @member {"auto_apply_date"|undefined} _auto_apply_date
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(TextAdRecommendation.prototype, "_auto_apply_date", {
                                    get: $util.oneOfGetter($oneOfFields = ["auto_apply_date"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TextAdRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITextAdRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation} TextAdRecommendation instance
                                 */
                                TextAdRecommendation.create = function create(properties) {
                                    return new TextAdRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified TextAdRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITextAdRecommendation} m TextAdRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TextAdRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.ad != null && Object.hasOwnProperty.call(m, "ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.ad, w.uint32(10).fork()).ldelim();
                                    if (m.creation_date != null && Object.hasOwnProperty.call(m, "creation_date"))
                                        w.uint32(34).string(m.creation_date);
                                    if (m.auto_apply_date != null && Object.hasOwnProperty.call(m, "auto_apply_date"))
                                        w.uint32(42).string(m.auto_apply_date);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TextAdRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITextAdRecommendation} message TextAdRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TextAdRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TextAdRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation} TextAdRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TextAdRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        case 4:
                                            m.creation_date = r.string();
                                            break;
                                        case 5:
                                            m.auto_apply_date = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TextAdRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation} TextAdRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TextAdRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TextAdRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TextAdRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.ad);
                                            if (e)
                                                return "ad." + e;
                                        }
                                    }
                                    if (m.creation_date != null && m.hasOwnProperty("creation_date")) {
                                        p._creation_date = 1;
                                        if (!$util.isString(m.creation_date))
                                            return "creation_date: string expected";
                                    }
                                    if (m.auto_apply_date != null && m.hasOwnProperty("auto_apply_date")) {
                                        p._auto_apply_date = 1;
                                        if (!$util.isString(m.auto_apply_date))
                                            return "auto_apply_date: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TextAdRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation} TextAdRecommendation
                                 */
                                TextAdRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation();
                                    if (d.ad != null) {
                                        if (typeof d.ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation.ad: object expected");
                                        m.ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.ad);
                                    }
                                    if (d.creation_date != null) {
                                        m.creation_date = String(d.creation_date);
                                    }
                                    if (d.auto_apply_date != null) {
                                        m.auto_apply_date = String(d.auto_apply_date);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TextAdRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation} m TextAdRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TextAdRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.ad = null;
                                    }
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        d.ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.ad, o);
                                    }
                                    if (m.creation_date != null && m.hasOwnProperty("creation_date")) {
                                        d.creation_date = m.creation_date;
                                        if (o.oneofs)
                                            d._creation_date = "creation_date";
                                    }
                                    if (m.auto_apply_date != null && m.hasOwnProperty("auto_apply_date")) {
                                        d.auto_apply_date = m.auto_apply_date;
                                        if (o.oneofs)
                                            d._auto_apply_date = "auto_apply_date";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TextAdRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TextAdRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TextAdRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TextAdRecommendation;
                            })();
    
                            Recommendation.TargetCpaOptInRecommendation = (function() {
    
                                /**
                                 * Properties of a TargetCpaOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ITargetCpaOptInRecommendation
                                 * @property {Array.<google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.ITargetCpaOptInRecommendationOption>|null} [options] TargetCpaOptInRecommendation options
                                 * @property {number|null} [recommended_target_cpa_micros] TargetCpaOptInRecommendation recommended_target_cpa_micros
                                 */
    
                                /**
                                 * Constructs a new TargetCpaOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a TargetCpaOptInRecommendation.
                                 * @implements ITargetCpaOptInRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetCpaOptInRecommendation=} [p] Properties to set
                                 */
                                function TargetCpaOptInRecommendation(p) {
                                    this.options = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetCpaOptInRecommendation options.
                                 * @member {Array.<google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.ITargetCpaOptInRecommendationOption>} options
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @instance
                                 */
                                TargetCpaOptInRecommendation.prototype.options = $util.emptyArray;
    
                                /**
                                 * TargetCpaOptInRecommendation recommended_target_cpa_micros.
                                 * @member {number|null|undefined} recommended_target_cpa_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @instance
                                 */
                                TargetCpaOptInRecommendation.prototype.recommended_target_cpa_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetCpaOptInRecommendation _recommended_target_cpa_micros.
                                 * @member {"recommended_target_cpa_micros"|undefined} _recommended_target_cpa_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(TargetCpaOptInRecommendation.prototype, "_recommended_target_cpa_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["recommended_target_cpa_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetCpaOptInRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetCpaOptInRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation} TargetCpaOptInRecommendation instance
                                 */
                                TargetCpaOptInRecommendation.create = function create(properties) {
                                    return new TargetCpaOptInRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetCpaOptInRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetCpaOptInRecommendation} m TargetCpaOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetCpaOptInRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.options != null && m.options.length) {
                                        for (var i = 0; i < m.options.length; ++i)
                                            $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.encode(m.options[i], w.uint32(10).fork()).ldelim();
                                    }
                                    if (m.recommended_target_cpa_micros != null && Object.hasOwnProperty.call(m, "recommended_target_cpa_micros"))
                                        w.uint32(24).int64(m.recommended_target_cpa_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetCpaOptInRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetCpaOptInRecommendation} message TargetCpaOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetCpaOptInRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetCpaOptInRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation} TargetCpaOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetCpaOptInRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.options && m.options.length))
                                                m.options = [];
                                            m.options.push($root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.decode(r, r.uint32()));
                                            break;
                                        case 3:
                                            m.recommended_target_cpa_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetCpaOptInRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation} TargetCpaOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetCpaOptInRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetCpaOptInRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetCpaOptInRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.options != null && m.hasOwnProperty("options")) {
                                        if (!Array.isArray(m.options))
                                            return "options: array expected";
                                        for (var i = 0; i < m.options.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.verify(m.options[i]);
                                                if (e)
                                                    return "options." + e;
                                            }
                                        }
                                    }
                                    if (m.recommended_target_cpa_micros != null && m.hasOwnProperty("recommended_target_cpa_micros")) {
                                        p._recommended_target_cpa_micros = 1;
                                        if (!$util.isInteger(m.recommended_target_cpa_micros) && !(m.recommended_target_cpa_micros && $util.isInteger(m.recommended_target_cpa_micros.low) && $util.isInteger(m.recommended_target_cpa_micros.high)))
                                            return "recommended_target_cpa_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetCpaOptInRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation} TargetCpaOptInRecommendation
                                 */
                                TargetCpaOptInRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation();
                                    if (d.options) {
                                        if (!Array.isArray(d.options))
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.options: array expected");
                                        m.options = [];
                                        for (var i = 0; i < d.options.length; ++i) {
                                            if (typeof d.options[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.options: object expected");
                                            m.options[i] = $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.fromObject(d.options[i]);
                                        }
                                    }
                                    if (d.recommended_target_cpa_micros != null) {
                                        if ($util.Long)
                                            (m.recommended_target_cpa_micros = $util.Long.fromValue(d.recommended_target_cpa_micros)).unsigned = false;
                                        else if (typeof d.recommended_target_cpa_micros === "string")
                                            m.recommended_target_cpa_micros = parseInt(d.recommended_target_cpa_micros, 10);
                                        else if (typeof d.recommended_target_cpa_micros === "number")
                                            m.recommended_target_cpa_micros = d.recommended_target_cpa_micros;
                                        else if (typeof d.recommended_target_cpa_micros === "object")
                                            m.recommended_target_cpa_micros = new $util.LongBits(d.recommended_target_cpa_micros.low >>> 0, d.recommended_target_cpa_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetCpaOptInRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation} m TargetCpaOptInRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetCpaOptInRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.options = [];
                                    }
                                    if (m.options && m.options.length) {
                                        d.options = [];
                                        for (var j = 0; j < m.options.length; ++j) {
                                            d.options[j] = $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.toObject(m.options[j], o);
                                        }
                                    }
                                    if (m.recommended_target_cpa_micros != null && m.hasOwnProperty("recommended_target_cpa_micros")) {
                                        if (typeof m.recommended_target_cpa_micros === "number")
                                            d.recommended_target_cpa_micros = o.longs === String ? String(m.recommended_target_cpa_micros) : m.recommended_target_cpa_micros;
                                        else
                                            d.recommended_target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.recommended_target_cpa_micros.low >>> 0, m.recommended_target_cpa_micros.high >>> 0).toNumber() : m.recommended_target_cpa_micros;
                                        if (o.oneofs)
                                            d._recommended_target_cpa_micros = "recommended_target_cpa_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetCpaOptInRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetCpaOptInRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption = (function() {
    
                                    /**
                                     * Properties of a TargetCpaOptInRecommendationOption.
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                     * @interface ITargetCpaOptInRecommendationOption
                                     * @property {google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal|null} [goal] TargetCpaOptInRecommendationOption goal
                                     * @property {number|null} [target_cpa_micros] TargetCpaOptInRecommendationOption target_cpa_micros
                                     * @property {number|null} [required_campaign_budget_amount_micros] TargetCpaOptInRecommendationOption required_campaign_budget_amount_micros
                                     * @property {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact|null} [impact] TargetCpaOptInRecommendationOption impact
                                     */
    
                                    /**
                                     * Constructs a new TargetCpaOptInRecommendationOption.
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation
                                     * @classdesc Represents a TargetCpaOptInRecommendationOption.
                                     * @implements ITargetCpaOptInRecommendationOption
                                     * @constructor
                                     * @param {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.ITargetCpaOptInRecommendationOption=} [p] Properties to set
                                     */
                                    function TargetCpaOptInRecommendationOption(p) {
                                        if (p)
                                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                                if (p[ks[i]] != null)
                                                    this[ks[i]] = p[ks[i]];
                                    }
    
                                    /**
                                     * TargetCpaOptInRecommendationOption goal.
                                     * @member {google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal} goal
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @instance
                                     */
                                    TargetCpaOptInRecommendationOption.prototype.goal = 0;
    
                                    /**
                                     * TargetCpaOptInRecommendationOption target_cpa_micros.
                                     * @member {number|null|undefined} target_cpa_micros
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @instance
                                     */
                                    TargetCpaOptInRecommendationOption.prototype.target_cpa_micros = null;
    
                                    /**
                                     * TargetCpaOptInRecommendationOption required_campaign_budget_amount_micros.
                                     * @member {number|null|undefined} required_campaign_budget_amount_micros
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @instance
                                     */
                                    TargetCpaOptInRecommendationOption.prototype.required_campaign_budget_amount_micros = null;
    
                                    /**
                                     * TargetCpaOptInRecommendationOption impact.
                                     * @member {google.ads.googleads.v11.resources.Recommendation.IRecommendationImpact|null|undefined} impact
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @instance
                                     */
                                    TargetCpaOptInRecommendationOption.prototype.impact = null;
    
                                    // OneOf field names bound to virtual getters and setters
                                    var $oneOfFields;
    
                                    /**
                                     * TargetCpaOptInRecommendationOption _target_cpa_micros.
                                     * @member {"target_cpa_micros"|undefined} _target_cpa_micros
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @instance
                                     */
                                    Object.defineProperty(TargetCpaOptInRecommendationOption.prototype, "_target_cpa_micros", {
                                        get: $util.oneOfGetter($oneOfFields = ["target_cpa_micros"]),
                                        set: $util.oneOfSetter($oneOfFields)
                                    });
    
                                    /**
                                     * TargetCpaOptInRecommendationOption _required_campaign_budget_amount_micros.
                                     * @member {"required_campaign_budget_amount_micros"|undefined} _required_campaign_budget_amount_micros
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @instance
                                     */
                                    Object.defineProperty(TargetCpaOptInRecommendationOption.prototype, "_required_campaign_budget_amount_micros", {
                                        get: $util.oneOfGetter($oneOfFields = ["required_campaign_budget_amount_micros"]),
                                        set: $util.oneOfSetter($oneOfFields)
                                    });
    
                                    /**
                                     * Creates a new TargetCpaOptInRecommendationOption instance using the specified properties.
                                     * @function create
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.ITargetCpaOptInRecommendationOption=} [properties] Properties to set
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption} TargetCpaOptInRecommendationOption instance
                                     */
                                    TargetCpaOptInRecommendationOption.create = function create(properties) {
                                        return new TargetCpaOptInRecommendationOption(properties);
                                    };
    
                                    /**
                                     * Encodes the specified TargetCpaOptInRecommendationOption message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.verify|verify} messages.
                                     * @function encode
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.ITargetCpaOptInRecommendationOption} m TargetCpaOptInRecommendationOption message or plain object to encode
                                     * @param {$protobuf.Writer} [w] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    TargetCpaOptInRecommendationOption.encode = function encode(m, w) {
                                        if (!w)
                                            w = $Writer.create();
                                        if (m.goal != null && Object.hasOwnProperty.call(m, "goal"))
                                            w.uint32(8).int32(m.goal);
                                        if (m.impact != null && Object.hasOwnProperty.call(m, "impact"))
                                            $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.encode(m.impact, w.uint32(34).fork()).ldelim();
                                        if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                            w.uint32(40).int64(m.target_cpa_micros);
                                        if (m.required_campaign_budget_amount_micros != null && Object.hasOwnProperty.call(m, "required_campaign_budget_amount_micros"))
                                            w.uint32(48).int64(m.required_campaign_budget_amount_micros);
                                        return w;
                                    };
    
                                    /**
                                     * Encodes the specified TargetCpaOptInRecommendationOption message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.verify|verify} messages.
                                     * @function encodeDelimited
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.ITargetCpaOptInRecommendationOption} message TargetCpaOptInRecommendationOption message or plain object to encode
                                     * @param {$protobuf.Writer} [writer] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    TargetCpaOptInRecommendationOption.encodeDelimited = function encodeDelimited(message, writer) {
                                        return this.encode(message, writer).ldelim();
                                    };
    
                                    /**
                                     * Decodes a TargetCpaOptInRecommendationOption message from the specified reader or buffer.
                                     * @function decode
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                     * @param {number} [l] Message length if known beforehand
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption} TargetCpaOptInRecommendationOption
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    TargetCpaOptInRecommendationOption.decode = function decode(r, l) {
                                        if (!(r instanceof $Reader))
                                            r = $Reader.create(r);
                                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption();
                                        while (r.pos < c) {
                                            var t = r.uint32();
                                            switch (t >>> 3) {
                                            case 1:
                                                m.goal = r.int32();
                                                break;
                                            case 5:
                                                m.target_cpa_micros = r.int64();
                                                break;
                                            case 6:
                                                m.required_campaign_budget_amount_micros = r.int64();
                                                break;
                                            case 4:
                                                m.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.decode(r, r.uint32());
                                                break;
                                            default:
                                                r.skipType(t & 7);
                                                break;
                                            }
                                        }
                                        return m;
                                    };
    
                                    /**
                                     * Decodes a TargetCpaOptInRecommendationOption message from the specified reader or buffer, length delimited.
                                     * @function decodeDelimited
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption} TargetCpaOptInRecommendationOption
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    TargetCpaOptInRecommendationOption.decodeDelimited = function decodeDelimited(reader) {
                                        if (!(reader instanceof $Reader))
                                            reader = new $Reader(reader);
                                        return this.decode(reader, reader.uint32());
                                    };
    
                                    /**
                                     * Verifies a TargetCpaOptInRecommendationOption message.
                                     * @function verify
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {Object.<string,*>} m Plain object to verify
                                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                     */
                                    TargetCpaOptInRecommendationOption.verify = function verify(m) {
                                        if (typeof m !== "object" || m === null)
                                            return "object expected";
                                        var p = {};
                                        if (m.goal != null && m.hasOwnProperty("goal")) {
                                            switch (m.goal) {
                                            default:
                                                return "goal: enum value expected";
                                            case 0:
                                            case 1:
                                            case 2:
                                            case 3:
                                            case 4:
                                            case 5:
                                                break;
                                            }
                                        }
                                        if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                            p._target_cpa_micros = 1;
                                            if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                                return "target_cpa_micros: integer|Long expected";
                                        }
                                        if (m.required_campaign_budget_amount_micros != null && m.hasOwnProperty("required_campaign_budget_amount_micros")) {
                                            p._required_campaign_budget_amount_micros = 1;
                                            if (!$util.isInteger(m.required_campaign_budget_amount_micros) && !(m.required_campaign_budget_amount_micros && $util.isInteger(m.required_campaign_budget_amount_micros.low) && $util.isInteger(m.required_campaign_budget_amount_micros.high)))
                                                return "required_campaign_budget_amount_micros: integer|Long expected";
                                        }
                                        if (m.impact != null && m.hasOwnProperty("impact")) {
                                            {
                                                var e = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.verify(m.impact);
                                                if (e)
                                                    return "impact." + e;
                                            }
                                        }
                                        return null;
                                    };
    
                                    /**
                                     * Creates a TargetCpaOptInRecommendationOption message from a plain object. Also converts values to their respective internal types.
                                     * @function fromObject
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {Object.<string,*>} d Plain object
                                     * @returns {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption} TargetCpaOptInRecommendationOption
                                     */
                                    TargetCpaOptInRecommendationOption.fromObject = function fromObject(d) {
                                        if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption)
                                            return d;
                                        var m = new $root.google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption();
                                        switch (d.goal) {
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.goal = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.goal = 1;
                                            break;
                                        case "SAME_COST":
                                        case 2:
                                            m.goal = 2;
                                            break;
                                        case "SAME_CONVERSIONS":
                                        case 3:
                                            m.goal = 3;
                                            break;
                                        case "SAME_CPA":
                                        case 4:
                                            m.goal = 4;
                                            break;
                                        case "CLOSEST_CPA":
                                        case 5:
                                            m.goal = 5;
                                            break;
                                        }
                                        if (d.target_cpa_micros != null) {
                                            if ($util.Long)
                                                (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                            else if (typeof d.target_cpa_micros === "string")
                                                m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                            else if (typeof d.target_cpa_micros === "number")
                                                m.target_cpa_micros = d.target_cpa_micros;
                                            else if (typeof d.target_cpa_micros === "object")
                                                m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                        }
                                        if (d.required_campaign_budget_amount_micros != null) {
                                            if ($util.Long)
                                                (m.required_campaign_budget_amount_micros = $util.Long.fromValue(d.required_campaign_budget_amount_micros)).unsigned = false;
                                            else if (typeof d.required_campaign_budget_amount_micros === "string")
                                                m.required_campaign_budget_amount_micros = parseInt(d.required_campaign_budget_amount_micros, 10);
                                            else if (typeof d.required_campaign_budget_amount_micros === "number")
                                                m.required_campaign_budget_amount_micros = d.required_campaign_budget_amount_micros;
                                            else if (typeof d.required_campaign_budget_amount_micros === "object")
                                                m.required_campaign_budget_amount_micros = new $util.LongBits(d.required_campaign_budget_amount_micros.low >>> 0, d.required_campaign_budget_amount_micros.high >>> 0).toNumber();
                                        }
                                        if (d.impact != null) {
                                            if (typeof d.impact !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption.impact: object expected");
                                            m.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.fromObject(d.impact);
                                        }
                                        return m;
                                    };
    
                                    /**
                                     * Creates a plain object from a TargetCpaOptInRecommendationOption message. Also converts values to other types if specified.
                                     * @function toObject
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @static
                                     * @param {google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption} m TargetCpaOptInRecommendationOption
                                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                                     * @returns {Object.<string,*>} Plain object
                                     */
                                    TargetCpaOptInRecommendationOption.toObject = function toObject(m, o) {
                                        if (!o)
                                            o = {};
                                        var d = {};
                                        if (o.defaults) {
                                            d.goal = o.enums === String ? "UNSPECIFIED" : 0;
                                            d.impact = null;
                                        }
                                        if (m.goal != null && m.hasOwnProperty("goal")) {
                                            d.goal = o.enums === String ? $root.google.ads.googleads.v11.enums.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal[m.goal] : m.goal;
                                        }
                                        if (m.impact != null && m.hasOwnProperty("impact")) {
                                            d.impact = $root.google.ads.googleads.v11.resources.Recommendation.RecommendationImpact.toObject(m.impact, o);
                                        }
                                        if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                            if (typeof m.target_cpa_micros === "number")
                                                d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                            else
                                                d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                            if (o.oneofs)
                                                d._target_cpa_micros = "target_cpa_micros";
                                        }
                                        if (m.required_campaign_budget_amount_micros != null && m.hasOwnProperty("required_campaign_budget_amount_micros")) {
                                            if (typeof m.required_campaign_budget_amount_micros === "number")
                                                d.required_campaign_budget_amount_micros = o.longs === String ? String(m.required_campaign_budget_amount_micros) : m.required_campaign_budget_amount_micros;
                                            else
                                                d.required_campaign_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_campaign_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.required_campaign_budget_amount_micros.low >>> 0, m.required_campaign_budget_amount_micros.high >>> 0).toNumber() : m.required_campaign_budget_amount_micros;
                                            if (o.oneofs)
                                                d._required_campaign_budget_amount_micros = "required_campaign_budget_amount_micros";
                                        }
                                        return d;
                                    };
    
                                    /**
                                     * Converts this TargetCpaOptInRecommendationOption to JSON.
                                     * @function toJSON
                                     * @memberof google.ads.googleads.v11.resources.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
                                     * @instance
                                     * @returns {Object.<string,*>} JSON object
                                     */
                                    TargetCpaOptInRecommendationOption.prototype.toJSON = function toJSON() {
                                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                    };
    
                                    return TargetCpaOptInRecommendationOption;
                                })();
    
                                return TargetCpaOptInRecommendation;
                            })();
    
                            Recommendation.MaximizeConversionsOptInRecommendation = (function() {
    
                                /**
                                 * Properties of a MaximizeConversionsOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IMaximizeConversionsOptInRecommendation
                                 * @property {number|null} [recommended_budget_amount_micros] MaximizeConversionsOptInRecommendation recommended_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new MaximizeConversionsOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a MaximizeConversionsOptInRecommendation.
                                 * @implements IMaximizeConversionsOptInRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeConversionsOptInRecommendation=} [p] Properties to set
                                 */
                                function MaximizeConversionsOptInRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * MaximizeConversionsOptInRecommendation recommended_budget_amount_micros.
                                 * @member {number|null|undefined} recommended_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @instance
                                 */
                                MaximizeConversionsOptInRecommendation.prototype.recommended_budget_amount_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * MaximizeConversionsOptInRecommendation _recommended_budget_amount_micros.
                                 * @member {"recommended_budget_amount_micros"|undefined} _recommended_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(MaximizeConversionsOptInRecommendation.prototype, "_recommended_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["recommended_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new MaximizeConversionsOptInRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeConversionsOptInRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation} MaximizeConversionsOptInRecommendation instance
                                 */
                                MaximizeConversionsOptInRecommendation.create = function create(properties) {
                                    return new MaximizeConversionsOptInRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified MaximizeConversionsOptInRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeConversionsOptInRecommendation} m MaximizeConversionsOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeConversionsOptInRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.recommended_budget_amount_micros != null && Object.hasOwnProperty.call(m, "recommended_budget_amount_micros"))
                                        w.uint32(16).int64(m.recommended_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified MaximizeConversionsOptInRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeConversionsOptInRecommendation} message MaximizeConversionsOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeConversionsOptInRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a MaximizeConversionsOptInRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation} MaximizeConversionsOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeConversionsOptInRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.recommended_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a MaximizeConversionsOptInRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation} MaximizeConversionsOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeConversionsOptInRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a MaximizeConversionsOptInRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                MaximizeConversionsOptInRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                        p._recommended_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.recommended_budget_amount_micros) && !(m.recommended_budget_amount_micros && $util.isInteger(m.recommended_budget_amount_micros.low) && $util.isInteger(m.recommended_budget_amount_micros.high)))
                                            return "recommended_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a MaximizeConversionsOptInRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation} MaximizeConversionsOptInRecommendation
                                 */
                                MaximizeConversionsOptInRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation();
                                    if (d.recommended_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.recommended_budget_amount_micros = $util.Long.fromValue(d.recommended_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.recommended_budget_amount_micros === "string")
                                            m.recommended_budget_amount_micros = parseInt(d.recommended_budget_amount_micros, 10);
                                        else if (typeof d.recommended_budget_amount_micros === "number")
                                            m.recommended_budget_amount_micros = d.recommended_budget_amount_micros;
                                        else if (typeof d.recommended_budget_amount_micros === "object")
                                            m.recommended_budget_amount_micros = new $util.LongBits(d.recommended_budget_amount_micros.low >>> 0, d.recommended_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a MaximizeConversionsOptInRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation} m MaximizeConversionsOptInRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                MaximizeConversionsOptInRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                        if (typeof m.recommended_budget_amount_micros === "number")
                                            d.recommended_budget_amount_micros = o.longs === String ? String(m.recommended_budget_amount_micros) : m.recommended_budget_amount_micros;
                                        else
                                            d.recommended_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.recommended_budget_amount_micros.low >>> 0, m.recommended_budget_amount_micros.high >>> 0).toNumber() : m.recommended_budget_amount_micros;
                                        if (o.oneofs)
                                            d._recommended_budget_amount_micros = "recommended_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this MaximizeConversionsOptInRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeConversionsOptInRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                MaximizeConversionsOptInRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return MaximizeConversionsOptInRecommendation;
                            })();
    
                            Recommendation.EnhancedCpcOptInRecommendation = (function() {
    
                                /**
                                 * Properties of an EnhancedCpcOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IEnhancedCpcOptInRecommendation
                                 */
    
                                /**
                                 * Constructs a new EnhancedCpcOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents an EnhancedCpcOptInRecommendation.
                                 * @implements IEnhancedCpcOptInRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IEnhancedCpcOptInRecommendation=} [p] Properties to set
                                 */
                                function EnhancedCpcOptInRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * Creates a new EnhancedCpcOptInRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IEnhancedCpcOptInRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation} EnhancedCpcOptInRecommendation instance
                                 */
                                EnhancedCpcOptInRecommendation.create = function create(properties) {
                                    return new EnhancedCpcOptInRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified EnhancedCpcOptInRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IEnhancedCpcOptInRecommendation} m EnhancedCpcOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                EnhancedCpcOptInRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified EnhancedCpcOptInRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IEnhancedCpcOptInRecommendation} message EnhancedCpcOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                EnhancedCpcOptInRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an EnhancedCpcOptInRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation} EnhancedCpcOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                EnhancedCpcOptInRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an EnhancedCpcOptInRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation} EnhancedCpcOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                EnhancedCpcOptInRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an EnhancedCpcOptInRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                EnhancedCpcOptInRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    return null;
                                };
    
                                /**
                                 * Creates an EnhancedCpcOptInRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation} EnhancedCpcOptInRecommendation
                                 */
                                EnhancedCpcOptInRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation)
                                        return d;
                                    return new $root.google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation();
                                };
    
                                /**
                                 * Creates a plain object from an EnhancedCpcOptInRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation} m EnhancedCpcOptInRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                EnhancedCpcOptInRecommendation.toObject = function toObject() {
                                    return {};
                                };
    
                                /**
                                 * Converts this EnhancedCpcOptInRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.EnhancedCpcOptInRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                EnhancedCpcOptInRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return EnhancedCpcOptInRecommendation;
                            })();
    
                            Recommendation.SearchPartnersOptInRecommendation = (function() {
    
                                /**
                                 * Properties of a SearchPartnersOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ISearchPartnersOptInRecommendation
                                 */
    
                                /**
                                 * Constructs a new SearchPartnersOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a SearchPartnersOptInRecommendation.
                                 * @implements ISearchPartnersOptInRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISearchPartnersOptInRecommendation=} [p] Properties to set
                                 */
                                function SearchPartnersOptInRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * Creates a new SearchPartnersOptInRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISearchPartnersOptInRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation} SearchPartnersOptInRecommendation instance
                                 */
                                SearchPartnersOptInRecommendation.create = function create(properties) {
                                    return new SearchPartnersOptInRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified SearchPartnersOptInRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISearchPartnersOptInRecommendation} m SearchPartnersOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SearchPartnersOptInRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified SearchPartnersOptInRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISearchPartnersOptInRecommendation} message SearchPartnersOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SearchPartnersOptInRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a SearchPartnersOptInRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation} SearchPartnersOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SearchPartnersOptInRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a SearchPartnersOptInRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation} SearchPartnersOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SearchPartnersOptInRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a SearchPartnersOptInRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                SearchPartnersOptInRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    return null;
                                };
    
                                /**
                                 * Creates a SearchPartnersOptInRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation} SearchPartnersOptInRecommendation
                                 */
                                SearchPartnersOptInRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation)
                                        return d;
                                    return new $root.google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation();
                                };
    
                                /**
                                 * Creates a plain object from a SearchPartnersOptInRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation} m SearchPartnersOptInRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                SearchPartnersOptInRecommendation.toObject = function toObject() {
                                    return {};
                                };
    
                                /**
                                 * Converts this SearchPartnersOptInRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SearchPartnersOptInRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                SearchPartnersOptInRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return SearchPartnersOptInRecommendation;
                            })();
    
                            Recommendation.MaximizeClicksOptInRecommendation = (function() {
    
                                /**
                                 * Properties of a MaximizeClicksOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IMaximizeClicksOptInRecommendation
                                 * @property {number|null} [recommended_budget_amount_micros] MaximizeClicksOptInRecommendation recommended_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new MaximizeClicksOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a MaximizeClicksOptInRecommendation.
                                 * @implements IMaximizeClicksOptInRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeClicksOptInRecommendation=} [p] Properties to set
                                 */
                                function MaximizeClicksOptInRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * MaximizeClicksOptInRecommendation recommended_budget_amount_micros.
                                 * @member {number|null|undefined} recommended_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @instance
                                 */
                                MaximizeClicksOptInRecommendation.prototype.recommended_budget_amount_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * MaximizeClicksOptInRecommendation _recommended_budget_amount_micros.
                                 * @member {"recommended_budget_amount_micros"|undefined} _recommended_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(MaximizeClicksOptInRecommendation.prototype, "_recommended_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["recommended_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new MaximizeClicksOptInRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeClicksOptInRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation} MaximizeClicksOptInRecommendation instance
                                 */
                                MaximizeClicksOptInRecommendation.create = function create(properties) {
                                    return new MaximizeClicksOptInRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified MaximizeClicksOptInRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeClicksOptInRecommendation} m MaximizeClicksOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeClicksOptInRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.recommended_budget_amount_micros != null && Object.hasOwnProperty.call(m, "recommended_budget_amount_micros"))
                                        w.uint32(16).int64(m.recommended_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified MaximizeClicksOptInRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMaximizeClicksOptInRecommendation} message MaximizeClicksOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MaximizeClicksOptInRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a MaximizeClicksOptInRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation} MaximizeClicksOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeClicksOptInRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.recommended_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a MaximizeClicksOptInRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation} MaximizeClicksOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MaximizeClicksOptInRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a MaximizeClicksOptInRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                MaximizeClicksOptInRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                        p._recommended_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.recommended_budget_amount_micros) && !(m.recommended_budget_amount_micros && $util.isInteger(m.recommended_budget_amount_micros.low) && $util.isInteger(m.recommended_budget_amount_micros.high)))
                                            return "recommended_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a MaximizeClicksOptInRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation} MaximizeClicksOptInRecommendation
                                 */
                                MaximizeClicksOptInRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation();
                                    if (d.recommended_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.recommended_budget_amount_micros = $util.Long.fromValue(d.recommended_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.recommended_budget_amount_micros === "string")
                                            m.recommended_budget_amount_micros = parseInt(d.recommended_budget_amount_micros, 10);
                                        else if (typeof d.recommended_budget_amount_micros === "number")
                                            m.recommended_budget_amount_micros = d.recommended_budget_amount_micros;
                                        else if (typeof d.recommended_budget_amount_micros === "object")
                                            m.recommended_budget_amount_micros = new $util.LongBits(d.recommended_budget_amount_micros.low >>> 0, d.recommended_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a MaximizeClicksOptInRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation} m MaximizeClicksOptInRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                MaximizeClicksOptInRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.recommended_budget_amount_micros != null && m.hasOwnProperty("recommended_budget_amount_micros")) {
                                        if (typeof m.recommended_budget_amount_micros === "number")
                                            d.recommended_budget_amount_micros = o.longs === String ? String(m.recommended_budget_amount_micros) : m.recommended_budget_amount_micros;
                                        else
                                            d.recommended_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.recommended_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.recommended_budget_amount_micros.low >>> 0, m.recommended_budget_amount_micros.high >>> 0).toNumber() : m.recommended_budget_amount_micros;
                                        if (o.oneofs)
                                            d._recommended_budget_amount_micros = "recommended_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this MaximizeClicksOptInRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MaximizeClicksOptInRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                MaximizeClicksOptInRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return MaximizeClicksOptInRecommendation;
                            })();
    
                            Recommendation.OptimizeAdRotationRecommendation = (function() {
    
                                /**
                                 * Properties of an OptimizeAdRotationRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IOptimizeAdRotationRecommendation
                                 */
    
                                /**
                                 * Constructs a new OptimizeAdRotationRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents an OptimizeAdRotationRecommendation.
                                 * @implements IOptimizeAdRotationRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IOptimizeAdRotationRecommendation=} [p] Properties to set
                                 */
                                function OptimizeAdRotationRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * Creates a new OptimizeAdRotationRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IOptimizeAdRotationRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation} OptimizeAdRotationRecommendation instance
                                 */
                                OptimizeAdRotationRecommendation.create = function create(properties) {
                                    return new OptimizeAdRotationRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified OptimizeAdRotationRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IOptimizeAdRotationRecommendation} m OptimizeAdRotationRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                OptimizeAdRotationRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified OptimizeAdRotationRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IOptimizeAdRotationRecommendation} message OptimizeAdRotationRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                OptimizeAdRotationRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an OptimizeAdRotationRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation} OptimizeAdRotationRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                OptimizeAdRotationRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an OptimizeAdRotationRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation} OptimizeAdRotationRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                OptimizeAdRotationRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an OptimizeAdRotationRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                OptimizeAdRotationRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    return null;
                                };
    
                                /**
                                 * Creates an OptimizeAdRotationRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation} OptimizeAdRotationRecommendation
                                 */
                                OptimizeAdRotationRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation)
                                        return d;
                                    return new $root.google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation();
                                };
    
                                /**
                                 * Creates a plain object from an OptimizeAdRotationRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation} m OptimizeAdRotationRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                OptimizeAdRotationRecommendation.toObject = function toObject() {
                                    return {};
                                };
    
                                /**
                                 * Converts this OptimizeAdRotationRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.OptimizeAdRotationRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                OptimizeAdRotationRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return OptimizeAdRotationRecommendation;
                            })();
    
                            Recommendation.CalloutExtensionRecommendation = (function() {
    
                                /**
                                 * Properties of a CalloutExtensionRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ICalloutExtensionRecommendation
                                 * @property {Array.<google.ads.googleads.v11.common.ICalloutFeedItem>|null} [recommended_extensions] CalloutExtensionRecommendation recommended_extensions
                                 */
    
                                /**
                                 * Constructs a new CalloutExtensionRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a CalloutExtensionRecommendation.
                                 * @implements ICalloutExtensionRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICalloutExtensionRecommendation=} [p] Properties to set
                                 */
                                function CalloutExtensionRecommendation(p) {
                                    this.recommended_extensions = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CalloutExtensionRecommendation recommended_extensions.
                                 * @member {Array.<google.ads.googleads.v11.common.ICalloutFeedItem>} recommended_extensions
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @instance
                                 */
                                CalloutExtensionRecommendation.prototype.recommended_extensions = $util.emptyArray;
    
                                /**
                                 * Creates a new CalloutExtensionRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICalloutExtensionRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation} CalloutExtensionRecommendation instance
                                 */
                                CalloutExtensionRecommendation.create = function create(properties) {
                                    return new CalloutExtensionRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified CalloutExtensionRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICalloutExtensionRecommendation} m CalloutExtensionRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CalloutExtensionRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.recommended_extensions != null && m.recommended_extensions.length) {
                                        for (var i = 0; i < m.recommended_extensions.length; ++i)
                                            $root.google.ads.googleads.v11.common.CalloutFeedItem.encode(m.recommended_extensions[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CalloutExtensionRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICalloutExtensionRecommendation} message CalloutExtensionRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CalloutExtensionRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CalloutExtensionRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation} CalloutExtensionRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CalloutExtensionRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.recommended_extensions && m.recommended_extensions.length))
                                                m.recommended_extensions = [];
                                            m.recommended_extensions.push($root.google.ads.googleads.v11.common.CalloutFeedItem.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CalloutExtensionRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation} CalloutExtensionRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CalloutExtensionRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CalloutExtensionRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CalloutExtensionRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.recommended_extensions != null && m.hasOwnProperty("recommended_extensions")) {
                                        if (!Array.isArray(m.recommended_extensions))
                                            return "recommended_extensions: array expected";
                                        for (var i = 0; i < m.recommended_extensions.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.CalloutFeedItem.verify(m.recommended_extensions[i]);
                                                if (e)
                                                    return "recommended_extensions." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CalloutExtensionRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation} CalloutExtensionRecommendation
                                 */
                                CalloutExtensionRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation();
                                    if (d.recommended_extensions) {
                                        if (!Array.isArray(d.recommended_extensions))
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.recommended_extensions: array expected");
                                        m.recommended_extensions = [];
                                        for (var i = 0; i < d.recommended_extensions.length; ++i) {
                                            if (typeof d.recommended_extensions[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation.recommended_extensions: object expected");
                                            m.recommended_extensions[i] = $root.google.ads.googleads.v11.common.CalloutFeedItem.fromObject(d.recommended_extensions[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CalloutExtensionRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation} m CalloutExtensionRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CalloutExtensionRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.recommended_extensions = [];
                                    }
                                    if (m.recommended_extensions && m.recommended_extensions.length) {
                                        d.recommended_extensions = [];
                                        for (var j = 0; j < m.recommended_extensions.length; ++j) {
                                            d.recommended_extensions[j] = $root.google.ads.googleads.v11.common.CalloutFeedItem.toObject(m.recommended_extensions[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CalloutExtensionRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CalloutExtensionRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CalloutExtensionRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CalloutExtensionRecommendation;
                            })();
    
                            Recommendation.SitelinkExtensionRecommendation = (function() {
    
                                /**
                                 * Properties of a SitelinkExtensionRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ISitelinkExtensionRecommendation
                                 * @property {Array.<google.ads.googleads.v11.common.ISitelinkFeedItem>|null} [recommended_extensions] SitelinkExtensionRecommendation recommended_extensions
                                 */
    
                                /**
                                 * Constructs a new SitelinkExtensionRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a SitelinkExtensionRecommendation.
                                 * @implements ISitelinkExtensionRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISitelinkExtensionRecommendation=} [p] Properties to set
                                 */
                                function SitelinkExtensionRecommendation(p) {
                                    this.recommended_extensions = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * SitelinkExtensionRecommendation recommended_extensions.
                                 * @member {Array.<google.ads.googleads.v11.common.ISitelinkFeedItem>} recommended_extensions
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @instance
                                 */
                                SitelinkExtensionRecommendation.prototype.recommended_extensions = $util.emptyArray;
    
                                /**
                                 * Creates a new SitelinkExtensionRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISitelinkExtensionRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation} SitelinkExtensionRecommendation instance
                                 */
                                SitelinkExtensionRecommendation.create = function create(properties) {
                                    return new SitelinkExtensionRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified SitelinkExtensionRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISitelinkExtensionRecommendation} m SitelinkExtensionRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SitelinkExtensionRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.recommended_extensions != null && m.recommended_extensions.length) {
                                        for (var i = 0; i < m.recommended_extensions.length; ++i)
                                            $root.google.ads.googleads.v11.common.SitelinkFeedItem.encode(m.recommended_extensions[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified SitelinkExtensionRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ISitelinkExtensionRecommendation} message SitelinkExtensionRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SitelinkExtensionRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a SitelinkExtensionRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation} SitelinkExtensionRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SitelinkExtensionRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.recommended_extensions && m.recommended_extensions.length))
                                                m.recommended_extensions = [];
                                            m.recommended_extensions.push($root.google.ads.googleads.v11.common.SitelinkFeedItem.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a SitelinkExtensionRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation} SitelinkExtensionRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SitelinkExtensionRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a SitelinkExtensionRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                SitelinkExtensionRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.recommended_extensions != null && m.hasOwnProperty("recommended_extensions")) {
                                        if (!Array.isArray(m.recommended_extensions))
                                            return "recommended_extensions: array expected";
                                        for (var i = 0; i < m.recommended_extensions.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.SitelinkFeedItem.verify(m.recommended_extensions[i]);
                                                if (e)
                                                    return "recommended_extensions." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a SitelinkExtensionRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation} SitelinkExtensionRecommendation
                                 */
                                SitelinkExtensionRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation();
                                    if (d.recommended_extensions) {
                                        if (!Array.isArray(d.recommended_extensions))
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.recommended_extensions: array expected");
                                        m.recommended_extensions = [];
                                        for (var i = 0; i < d.recommended_extensions.length; ++i) {
                                            if (typeof d.recommended_extensions[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation.recommended_extensions: object expected");
                                            m.recommended_extensions[i] = $root.google.ads.googleads.v11.common.SitelinkFeedItem.fromObject(d.recommended_extensions[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a SitelinkExtensionRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation} m SitelinkExtensionRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                SitelinkExtensionRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.recommended_extensions = [];
                                    }
                                    if (m.recommended_extensions && m.recommended_extensions.length) {
                                        d.recommended_extensions = [];
                                        for (var j = 0; j < m.recommended_extensions.length; ++j) {
                                            d.recommended_extensions[j] = $root.google.ads.googleads.v11.common.SitelinkFeedItem.toObject(m.recommended_extensions[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this SitelinkExtensionRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.SitelinkExtensionRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                SitelinkExtensionRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return SitelinkExtensionRecommendation;
                            })();
    
                            Recommendation.CallExtensionRecommendation = (function() {
    
                                /**
                                 * Properties of a CallExtensionRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ICallExtensionRecommendation
                                 * @property {Array.<google.ads.googleads.v11.common.ICallFeedItem>|null} [recommended_extensions] CallExtensionRecommendation recommended_extensions
                                 */
    
                                /**
                                 * Constructs a new CallExtensionRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a CallExtensionRecommendation.
                                 * @implements ICallExtensionRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICallExtensionRecommendation=} [p] Properties to set
                                 */
                                function CallExtensionRecommendation(p) {
                                    this.recommended_extensions = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CallExtensionRecommendation recommended_extensions.
                                 * @member {Array.<google.ads.googleads.v11.common.ICallFeedItem>} recommended_extensions
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @instance
                                 */
                                CallExtensionRecommendation.prototype.recommended_extensions = $util.emptyArray;
    
                                /**
                                 * Creates a new CallExtensionRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICallExtensionRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation} CallExtensionRecommendation instance
                                 */
                                CallExtensionRecommendation.create = function create(properties) {
                                    return new CallExtensionRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified CallExtensionRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICallExtensionRecommendation} m CallExtensionRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CallExtensionRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.recommended_extensions != null && m.recommended_extensions.length) {
                                        for (var i = 0; i < m.recommended_extensions.length; ++i)
                                            $root.google.ads.googleads.v11.common.CallFeedItem.encode(m.recommended_extensions[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CallExtensionRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ICallExtensionRecommendation} message CallExtensionRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CallExtensionRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CallExtensionRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation} CallExtensionRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CallExtensionRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.recommended_extensions && m.recommended_extensions.length))
                                                m.recommended_extensions = [];
                                            m.recommended_extensions.push($root.google.ads.googleads.v11.common.CallFeedItem.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CallExtensionRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation} CallExtensionRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CallExtensionRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CallExtensionRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CallExtensionRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.recommended_extensions != null && m.hasOwnProperty("recommended_extensions")) {
                                        if (!Array.isArray(m.recommended_extensions))
                                            return "recommended_extensions: array expected";
                                        for (var i = 0; i < m.recommended_extensions.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.CallFeedItem.verify(m.recommended_extensions[i]);
                                                if (e)
                                                    return "recommended_extensions." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CallExtensionRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation} CallExtensionRecommendation
                                 */
                                CallExtensionRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation();
                                    if (d.recommended_extensions) {
                                        if (!Array.isArray(d.recommended_extensions))
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.recommended_extensions: array expected");
                                        m.recommended_extensions = [];
                                        for (var i = 0; i < d.recommended_extensions.length; ++i) {
                                            if (typeof d.recommended_extensions[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation.recommended_extensions: object expected");
                                            m.recommended_extensions[i] = $root.google.ads.googleads.v11.common.CallFeedItem.fromObject(d.recommended_extensions[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CallExtensionRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation} m CallExtensionRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CallExtensionRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.recommended_extensions = [];
                                    }
                                    if (m.recommended_extensions && m.recommended_extensions.length) {
                                        d.recommended_extensions = [];
                                        for (var j = 0; j < m.recommended_extensions.length; ++j) {
                                            d.recommended_extensions[j] = $root.google.ads.googleads.v11.common.CallFeedItem.toObject(m.recommended_extensions[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CallExtensionRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.CallExtensionRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CallExtensionRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CallExtensionRecommendation;
                            })();
    
                            Recommendation.KeywordMatchTypeRecommendation = (function() {
    
                                /**
                                 * Properties of a KeywordMatchTypeRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IKeywordMatchTypeRecommendation
                                 * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [keyword] KeywordMatchTypeRecommendation keyword
                                 * @property {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType|null} [recommended_match_type] KeywordMatchTypeRecommendation recommended_match_type
                                 */
    
                                /**
                                 * Constructs a new KeywordMatchTypeRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a KeywordMatchTypeRecommendation.
                                 * @implements IKeywordMatchTypeRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordMatchTypeRecommendation=} [p] Properties to set
                                 */
                                function KeywordMatchTypeRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * KeywordMatchTypeRecommendation keyword.
                                 * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} keyword
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @instance
                                 */
                                KeywordMatchTypeRecommendation.prototype.keyword = null;
    
                                /**
                                 * KeywordMatchTypeRecommendation recommended_match_type.
                                 * @member {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType} recommended_match_type
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @instance
                                 */
                                KeywordMatchTypeRecommendation.prototype.recommended_match_type = 0;
    
                                /**
                                 * Creates a new KeywordMatchTypeRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordMatchTypeRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation} KeywordMatchTypeRecommendation instance
                                 */
                                KeywordMatchTypeRecommendation.create = function create(properties) {
                                    return new KeywordMatchTypeRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified KeywordMatchTypeRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordMatchTypeRecommendation} m KeywordMatchTypeRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                KeywordMatchTypeRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                        $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword, w.uint32(10).fork()).ldelim();
                                    if (m.recommended_match_type != null && Object.hasOwnProperty.call(m, "recommended_match_type"))
                                        w.uint32(16).int32(m.recommended_match_type);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified KeywordMatchTypeRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IKeywordMatchTypeRecommendation} message KeywordMatchTypeRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                KeywordMatchTypeRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a KeywordMatchTypeRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation} KeywordMatchTypeRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                KeywordMatchTypeRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                            break;
                                        case 2:
                                            m.recommended_match_type = r.int32();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a KeywordMatchTypeRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation} KeywordMatchTypeRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                KeywordMatchTypeRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a KeywordMatchTypeRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                KeywordMatchTypeRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword);
                                            if (e)
                                                return "keyword." + e;
                                        }
                                    }
                                    if (m.recommended_match_type != null && m.hasOwnProperty("recommended_match_type")) {
                                        switch (m.recommended_match_type) {
                                        default:
                                            return "recommended_match_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a KeywordMatchTypeRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation} KeywordMatchTypeRecommendation
                                 */
                                KeywordMatchTypeRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation();
                                    if (d.keyword != null) {
                                        if (typeof d.keyword !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation.keyword: object expected");
                                        m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword);
                                    }
                                    switch (d.recommended_match_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.recommended_match_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.recommended_match_type = 1;
                                        break;
                                    case "EXACT":
                                    case 2:
                                        m.recommended_match_type = 2;
                                        break;
                                    case "PHRASE":
                                    case 3:
                                        m.recommended_match_type = 3;
                                        break;
                                    case "BROAD":
                                    case 4:
                                        m.recommended_match_type = 4;
                                        break;
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a KeywordMatchTypeRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation} m KeywordMatchTypeRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                KeywordMatchTypeRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.keyword = null;
                                        d.recommended_match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                        d.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword, o);
                                    }
                                    if (m.recommended_match_type != null && m.hasOwnProperty("recommended_match_type")) {
                                        d.recommended_match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType[m.recommended_match_type] : m.recommended_match_type;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this KeywordMatchTypeRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.KeywordMatchTypeRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                KeywordMatchTypeRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return KeywordMatchTypeRecommendation;
                            })();
    
                            Recommendation.MoveUnusedBudgetRecommendation = (function() {
    
                                /**
                                 * Properties of a MoveUnusedBudgetRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IMoveUnusedBudgetRecommendation
                                 * @property {string|null} [excess_campaign_budget] MoveUnusedBudgetRecommendation excess_campaign_budget
                                 * @property {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null} [budget_recommendation] MoveUnusedBudgetRecommendation budget_recommendation
                                 */
    
                                /**
                                 * Constructs a new MoveUnusedBudgetRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a MoveUnusedBudgetRecommendation.
                                 * @implements IMoveUnusedBudgetRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMoveUnusedBudgetRecommendation=} [p] Properties to set
                                 */
                                function MoveUnusedBudgetRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * MoveUnusedBudgetRecommendation excess_campaign_budget.
                                 * @member {string|null|undefined} excess_campaign_budget
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @instance
                                 */
                                MoveUnusedBudgetRecommendation.prototype.excess_campaign_budget = null;
    
                                /**
                                 * MoveUnusedBudgetRecommendation budget_recommendation.
                                 * @member {google.ads.googleads.v11.resources.Recommendation.ICampaignBudgetRecommendation|null|undefined} budget_recommendation
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @instance
                                 */
                                MoveUnusedBudgetRecommendation.prototype.budget_recommendation = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * MoveUnusedBudgetRecommendation _excess_campaign_budget.
                                 * @member {"excess_campaign_budget"|undefined} _excess_campaign_budget
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(MoveUnusedBudgetRecommendation.prototype, "_excess_campaign_budget", {
                                    get: $util.oneOfGetter($oneOfFields = ["excess_campaign_budget"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new MoveUnusedBudgetRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMoveUnusedBudgetRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation} MoveUnusedBudgetRecommendation instance
                                 */
                                MoveUnusedBudgetRecommendation.create = function create(properties) {
                                    return new MoveUnusedBudgetRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified MoveUnusedBudgetRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMoveUnusedBudgetRecommendation} m MoveUnusedBudgetRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MoveUnusedBudgetRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.budget_recommendation != null && Object.hasOwnProperty.call(m, "budget_recommendation"))
                                        $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.encode(m.budget_recommendation, w.uint32(18).fork()).ldelim();
                                    if (m.excess_campaign_budget != null && Object.hasOwnProperty.call(m, "excess_campaign_budget"))
                                        w.uint32(26).string(m.excess_campaign_budget);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified MoveUnusedBudgetRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IMoveUnusedBudgetRecommendation} message MoveUnusedBudgetRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MoveUnusedBudgetRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a MoveUnusedBudgetRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation} MoveUnusedBudgetRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MoveUnusedBudgetRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 3:
                                            m.excess_campaign_budget = r.string();
                                            break;
                                        case 2:
                                            m.budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a MoveUnusedBudgetRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation} MoveUnusedBudgetRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MoveUnusedBudgetRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a MoveUnusedBudgetRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                MoveUnusedBudgetRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.excess_campaign_budget != null && m.hasOwnProperty("excess_campaign_budget")) {
                                        p._excess_campaign_budget = 1;
                                        if (!$util.isString(m.excess_campaign_budget))
                                            return "excess_campaign_budget: string expected";
                                    }
                                    if (m.budget_recommendation != null && m.hasOwnProperty("budget_recommendation")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.verify(m.budget_recommendation);
                                            if (e)
                                                return "budget_recommendation." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a MoveUnusedBudgetRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation} MoveUnusedBudgetRecommendation
                                 */
                                MoveUnusedBudgetRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation();
                                    if (d.excess_campaign_budget != null) {
                                        m.excess_campaign_budget = String(d.excess_campaign_budget);
                                    }
                                    if (d.budget_recommendation != null) {
                                        if (typeof d.budget_recommendation !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation.budget_recommendation: object expected");
                                        m.budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.fromObject(d.budget_recommendation);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a MoveUnusedBudgetRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation} m MoveUnusedBudgetRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                MoveUnusedBudgetRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.budget_recommendation = null;
                                    }
                                    if (m.budget_recommendation != null && m.hasOwnProperty("budget_recommendation")) {
                                        d.budget_recommendation = $root.google.ads.googleads.v11.resources.Recommendation.CampaignBudgetRecommendation.toObject(m.budget_recommendation, o);
                                    }
                                    if (m.excess_campaign_budget != null && m.hasOwnProperty("excess_campaign_budget")) {
                                        d.excess_campaign_budget = m.excess_campaign_budget;
                                        if (o.oneofs)
                                            d._excess_campaign_budget = "excess_campaign_budget";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this MoveUnusedBudgetRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.MoveUnusedBudgetRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                MoveUnusedBudgetRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return MoveUnusedBudgetRecommendation;
                            })();
    
                            Recommendation.TargetRoasOptInRecommendation = (function() {
    
                                /**
                                 * Properties of a TargetRoasOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface ITargetRoasOptInRecommendation
                                 * @property {number|null} [recommended_target_roas] TargetRoasOptInRecommendation recommended_target_roas
                                 * @property {number|null} [required_campaign_budget_amount_micros] TargetRoasOptInRecommendation required_campaign_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new TargetRoasOptInRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a TargetRoasOptInRecommendation.
                                 * @implements ITargetRoasOptInRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetRoasOptInRecommendation=} [p] Properties to set
                                 */
                                function TargetRoasOptInRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetRoasOptInRecommendation recommended_target_roas.
                                 * @member {number|null|undefined} recommended_target_roas
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @instance
                                 */
                                TargetRoasOptInRecommendation.prototype.recommended_target_roas = null;
    
                                /**
                                 * TargetRoasOptInRecommendation required_campaign_budget_amount_micros.
                                 * @member {number|null|undefined} required_campaign_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @instance
                                 */
                                TargetRoasOptInRecommendation.prototype.required_campaign_budget_amount_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetRoasOptInRecommendation _recommended_target_roas.
                                 * @member {"recommended_target_roas"|undefined} _recommended_target_roas
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(TargetRoasOptInRecommendation.prototype, "_recommended_target_roas", {
                                    get: $util.oneOfGetter($oneOfFields = ["recommended_target_roas"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * TargetRoasOptInRecommendation _required_campaign_budget_amount_micros.
                                 * @member {"required_campaign_budget_amount_micros"|undefined} _required_campaign_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @instance
                                 */
                                Object.defineProperty(TargetRoasOptInRecommendation.prototype, "_required_campaign_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["required_campaign_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetRoasOptInRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetRoasOptInRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation} TargetRoasOptInRecommendation instance
                                 */
                                TargetRoasOptInRecommendation.create = function create(properties) {
                                    return new TargetRoasOptInRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetRoasOptInRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetRoasOptInRecommendation} m TargetRoasOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetRoasOptInRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.recommended_target_roas != null && Object.hasOwnProperty.call(m, "recommended_target_roas"))
                                        w.uint32(9).double(m.recommended_target_roas);
                                    if (m.required_campaign_budget_amount_micros != null && Object.hasOwnProperty.call(m, "required_campaign_budget_amount_micros"))
                                        w.uint32(16).int64(m.required_campaign_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetRoasOptInRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ITargetRoasOptInRecommendation} message TargetRoasOptInRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetRoasOptInRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetRoasOptInRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation} TargetRoasOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetRoasOptInRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.recommended_target_roas = r.double();
                                            break;
                                        case 2:
                                            m.required_campaign_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetRoasOptInRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation} TargetRoasOptInRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetRoasOptInRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetRoasOptInRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetRoasOptInRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.recommended_target_roas != null && m.hasOwnProperty("recommended_target_roas")) {
                                        p._recommended_target_roas = 1;
                                        if (typeof m.recommended_target_roas !== "number")
                                            return "recommended_target_roas: number expected";
                                    }
                                    if (m.required_campaign_budget_amount_micros != null && m.hasOwnProperty("required_campaign_budget_amount_micros")) {
                                        p._required_campaign_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.required_campaign_budget_amount_micros) && !(m.required_campaign_budget_amount_micros && $util.isInteger(m.required_campaign_budget_amount_micros.low) && $util.isInteger(m.required_campaign_budget_amount_micros.high)))
                                            return "required_campaign_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetRoasOptInRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation} TargetRoasOptInRecommendation
                                 */
                                TargetRoasOptInRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation();
                                    if (d.recommended_target_roas != null) {
                                        m.recommended_target_roas = Number(d.recommended_target_roas);
                                    }
                                    if (d.required_campaign_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.required_campaign_budget_amount_micros = $util.Long.fromValue(d.required_campaign_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.required_campaign_budget_amount_micros === "string")
                                            m.required_campaign_budget_amount_micros = parseInt(d.required_campaign_budget_amount_micros, 10);
                                        else if (typeof d.required_campaign_budget_amount_micros === "number")
                                            m.required_campaign_budget_amount_micros = d.required_campaign_budget_amount_micros;
                                        else if (typeof d.required_campaign_budget_amount_micros === "object")
                                            m.required_campaign_budget_amount_micros = new $util.LongBits(d.required_campaign_budget_amount_micros.low >>> 0, d.required_campaign_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetRoasOptInRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation} m TargetRoasOptInRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetRoasOptInRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.recommended_target_roas != null && m.hasOwnProperty("recommended_target_roas")) {
                                        d.recommended_target_roas = o.json && !isFinite(m.recommended_target_roas) ? String(m.recommended_target_roas) : m.recommended_target_roas;
                                        if (o.oneofs)
                                            d._recommended_target_roas = "recommended_target_roas";
                                    }
                                    if (m.required_campaign_budget_amount_micros != null && m.hasOwnProperty("required_campaign_budget_amount_micros")) {
                                        if (typeof m.required_campaign_budget_amount_micros === "number")
                                            d.required_campaign_budget_amount_micros = o.longs === String ? String(m.required_campaign_budget_amount_micros) : m.required_campaign_budget_amount_micros;
                                        else
                                            d.required_campaign_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_campaign_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.required_campaign_budget_amount_micros.low >>> 0, m.required_campaign_budget_amount_micros.high >>> 0).toNumber() : m.required_campaign_budget_amount_micros;
                                        if (o.oneofs)
                                            d._required_campaign_budget_amount_micros = "required_campaign_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetRoasOptInRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.TargetRoasOptInRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetRoasOptInRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TargetRoasOptInRecommendation;
                            })();
    
                            Recommendation.ResponsiveSearchAdAssetRecommendation = (function() {
    
                                /**
                                 * Properties of a ResponsiveSearchAdAssetRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IResponsiveSearchAdAssetRecommendation
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [current_ad] ResponsiveSearchAdAssetRecommendation current_ad
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [recommended_assets] ResponsiveSearchAdAssetRecommendation recommended_assets
                                 */
    
                                /**
                                 * Constructs a new ResponsiveSearchAdAssetRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a ResponsiveSearchAdAssetRecommendation.
                                 * @implements IResponsiveSearchAdAssetRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdAssetRecommendation=} [p] Properties to set
                                 */
                                function ResponsiveSearchAdAssetRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ResponsiveSearchAdAssetRecommendation current_ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} current_ad
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @instance
                                 */
                                ResponsiveSearchAdAssetRecommendation.prototype.current_ad = null;
    
                                /**
                                 * ResponsiveSearchAdAssetRecommendation recommended_assets.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} recommended_assets
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @instance
                                 */
                                ResponsiveSearchAdAssetRecommendation.prototype.recommended_assets = null;
    
                                /**
                                 * Creates a new ResponsiveSearchAdAssetRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdAssetRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation} ResponsiveSearchAdAssetRecommendation instance
                                 */
                                ResponsiveSearchAdAssetRecommendation.create = function create(properties) {
                                    return new ResponsiveSearchAdAssetRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdAssetRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdAssetRecommendation} m ResponsiveSearchAdAssetRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdAssetRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.current_ad != null && Object.hasOwnProperty.call(m, "current_ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.current_ad, w.uint32(10).fork()).ldelim();
                                    if (m.recommended_assets != null && Object.hasOwnProperty.call(m, "recommended_assets"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.recommended_assets, w.uint32(18).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdAssetRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdAssetRecommendation} message ResponsiveSearchAdAssetRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdAssetRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdAssetRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation} ResponsiveSearchAdAssetRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdAssetRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.current_ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        case 2:
                                            m.recommended_assets = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdAssetRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation} ResponsiveSearchAdAssetRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdAssetRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ResponsiveSearchAdAssetRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ResponsiveSearchAdAssetRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.current_ad != null && m.hasOwnProperty("current_ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.current_ad);
                                            if (e)
                                                return "current_ad." + e;
                                        }
                                    }
                                    if (m.recommended_assets != null && m.hasOwnProperty("recommended_assets")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.recommended_assets);
                                            if (e)
                                                return "recommended_assets." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ResponsiveSearchAdAssetRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation} ResponsiveSearchAdAssetRecommendation
                                 */
                                ResponsiveSearchAdAssetRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation();
                                    if (d.current_ad != null) {
                                        if (typeof d.current_ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.current_ad: object expected");
                                        m.current_ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.current_ad);
                                    }
                                    if (d.recommended_assets != null) {
                                        if (typeof d.recommended_assets !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation.recommended_assets: object expected");
                                        m.recommended_assets = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.recommended_assets);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ResponsiveSearchAdAssetRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation} m ResponsiveSearchAdAssetRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ResponsiveSearchAdAssetRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.current_ad = null;
                                        d.recommended_assets = null;
                                    }
                                    if (m.current_ad != null && m.hasOwnProperty("current_ad")) {
                                        d.current_ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.current_ad, o);
                                    }
                                    if (m.recommended_assets != null && m.hasOwnProperty("recommended_assets")) {
                                        d.recommended_assets = $root.google.ads.googleads.v11.resources.Ad.toObject(m.recommended_assets, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ResponsiveSearchAdAssetRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdAssetRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ResponsiveSearchAdAssetRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ResponsiveSearchAdAssetRecommendation;
                            })();
    
                            Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation = (function() {
    
                                /**
                                 * Properties of a ResponsiveSearchAdImproveAdStrengthRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [current_ad] ResponsiveSearchAdImproveAdStrengthRecommendation current_ad
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [recommended_ad] ResponsiveSearchAdImproveAdStrengthRecommendation recommended_ad
                                 */
    
                                /**
                                 * Constructs a new ResponsiveSearchAdImproveAdStrengthRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a ResponsiveSearchAdImproveAdStrengthRecommendation.
                                 * @implements IResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdImproveAdStrengthRecommendation=} [p] Properties to set
                                 */
                                function ResponsiveSearchAdImproveAdStrengthRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ResponsiveSearchAdImproveAdStrengthRecommendation current_ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} current_ad
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @instance
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.prototype.current_ad = null;
    
                                /**
                                 * ResponsiveSearchAdImproveAdStrengthRecommendation recommended_ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} recommended_ad
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @instance
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.prototype.recommended_ad = null;
    
                                /**
                                 * Creates a new ResponsiveSearchAdImproveAdStrengthRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdImproveAdStrengthRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation} ResponsiveSearchAdImproveAdStrengthRecommendation instance
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.create = function create(properties) {
                                    return new ResponsiveSearchAdImproveAdStrengthRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdImproveAdStrengthRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdImproveAdStrengthRecommendation} m ResponsiveSearchAdImproveAdStrengthRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.current_ad != null && Object.hasOwnProperty.call(m, "current_ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.current_ad, w.uint32(10).fork()).ldelim();
                                    if (m.recommended_ad != null && Object.hasOwnProperty.call(m, "recommended_ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.recommended_ad, w.uint32(18).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdImproveAdStrengthRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdImproveAdStrengthRecommendation} message ResponsiveSearchAdImproveAdStrengthRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdImproveAdStrengthRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation} ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.current_ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        case 2:
                                            m.recommended_ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdImproveAdStrengthRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation} ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ResponsiveSearchAdImproveAdStrengthRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.current_ad != null && m.hasOwnProperty("current_ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.current_ad);
                                            if (e)
                                                return "current_ad." + e;
                                        }
                                    }
                                    if (m.recommended_ad != null && m.hasOwnProperty("recommended_ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.recommended_ad);
                                            if (e)
                                                return "recommended_ad." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ResponsiveSearchAdImproveAdStrengthRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation} ResponsiveSearchAdImproveAdStrengthRecommendation
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation();
                                    if (d.current_ad != null) {
                                        if (typeof d.current_ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.current_ad: object expected");
                                        m.current_ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.current_ad);
                                    }
                                    if (d.recommended_ad != null) {
                                        if (typeof d.recommended_ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation.recommended_ad: object expected");
                                        m.recommended_ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.recommended_ad);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ResponsiveSearchAdImproveAdStrengthRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation} m ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.current_ad = null;
                                        d.recommended_ad = null;
                                    }
                                    if (m.current_ad != null && m.hasOwnProperty("current_ad")) {
                                        d.current_ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.current_ad, o);
                                    }
                                    if (m.recommended_ad != null && m.hasOwnProperty("recommended_ad")) {
                                        d.recommended_ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.recommended_ad, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ResponsiveSearchAdImproveAdStrengthRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ResponsiveSearchAdImproveAdStrengthRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ResponsiveSearchAdImproveAdStrengthRecommendation;
                            })();
    
                            Recommendation.ResponsiveSearchAdRecommendation = (function() {
    
                                /**
                                 * Properties of a ResponsiveSearchAdRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IResponsiveSearchAdRecommendation
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [ad] ResponsiveSearchAdRecommendation ad
                                 */
    
                                /**
                                 * Constructs a new ResponsiveSearchAdRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a ResponsiveSearchAdRecommendation.
                                 * @implements IResponsiveSearchAdRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdRecommendation=} [p] Properties to set
                                 */
                                function ResponsiveSearchAdRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ResponsiveSearchAdRecommendation ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} ad
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @instance
                                 */
                                ResponsiveSearchAdRecommendation.prototype.ad = null;
    
                                /**
                                 * Creates a new ResponsiveSearchAdRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation} ResponsiveSearchAdRecommendation instance
                                 */
                                ResponsiveSearchAdRecommendation.create = function create(properties) {
                                    return new ResponsiveSearchAdRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdRecommendation} m ResponsiveSearchAdRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.ad != null && Object.hasOwnProperty.call(m, "ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.ad, w.uint32(10).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IResponsiveSearchAdRecommendation} message ResponsiveSearchAdRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation} ResponsiveSearchAdRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation} ResponsiveSearchAdRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ResponsiveSearchAdRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ResponsiveSearchAdRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.ad);
                                            if (e)
                                                return "ad." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ResponsiveSearchAdRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation} ResponsiveSearchAdRecommendation
                                 */
                                ResponsiveSearchAdRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation();
                                    if (d.ad != null) {
                                        if (typeof d.ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation.ad: object expected");
                                        m.ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.ad);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ResponsiveSearchAdRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation} m ResponsiveSearchAdRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ResponsiveSearchAdRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.ad = null;
                                    }
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        d.ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.ad, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ResponsiveSearchAdRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.ResponsiveSearchAdRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ResponsiveSearchAdRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ResponsiveSearchAdRecommendation;
                            })();
    
                            Recommendation.UseBroadMatchKeywordRecommendation = (function() {
    
                                /**
                                 * Properties of a UseBroadMatchKeywordRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IUseBroadMatchKeywordRecommendation
                                 * @property {Array.<google.ads.googleads.v11.common.IKeywordInfo>|null} [keyword] UseBroadMatchKeywordRecommendation keyword
                                 * @property {number|null} [suggested_keywords_count] UseBroadMatchKeywordRecommendation suggested_keywords_count
                                 * @property {number|null} [campaign_keywords_count] UseBroadMatchKeywordRecommendation campaign_keywords_count
                                 * @property {boolean|null} [campaign_uses_shared_budget] UseBroadMatchKeywordRecommendation campaign_uses_shared_budget
                                 * @property {number|null} [required_campaign_budget_amount_micros] UseBroadMatchKeywordRecommendation required_campaign_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new UseBroadMatchKeywordRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents a UseBroadMatchKeywordRecommendation.
                                 * @implements IUseBroadMatchKeywordRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUseBroadMatchKeywordRecommendation=} [p] Properties to set
                                 */
                                function UseBroadMatchKeywordRecommendation(p) {
                                    this.keyword = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * UseBroadMatchKeywordRecommendation keyword.
                                 * @member {Array.<google.ads.googleads.v11.common.IKeywordInfo>} keyword
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @instance
                                 */
                                UseBroadMatchKeywordRecommendation.prototype.keyword = $util.emptyArray;
    
                                /**
                                 * UseBroadMatchKeywordRecommendation suggested_keywords_count.
                                 * @member {number} suggested_keywords_count
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @instance
                                 */
                                UseBroadMatchKeywordRecommendation.prototype.suggested_keywords_count = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * UseBroadMatchKeywordRecommendation campaign_keywords_count.
                                 * @member {number} campaign_keywords_count
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @instance
                                 */
                                UseBroadMatchKeywordRecommendation.prototype.campaign_keywords_count = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * UseBroadMatchKeywordRecommendation campaign_uses_shared_budget.
                                 * @member {boolean} campaign_uses_shared_budget
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @instance
                                 */
                                UseBroadMatchKeywordRecommendation.prototype.campaign_uses_shared_budget = false;
    
                                /**
                                 * UseBroadMatchKeywordRecommendation required_campaign_budget_amount_micros.
                                 * @member {number} required_campaign_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @instance
                                 */
                                UseBroadMatchKeywordRecommendation.prototype.required_campaign_budget_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * Creates a new UseBroadMatchKeywordRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUseBroadMatchKeywordRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation} UseBroadMatchKeywordRecommendation instance
                                 */
                                UseBroadMatchKeywordRecommendation.create = function create(properties) {
                                    return new UseBroadMatchKeywordRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified UseBroadMatchKeywordRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUseBroadMatchKeywordRecommendation} m UseBroadMatchKeywordRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                UseBroadMatchKeywordRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.keyword != null && m.keyword.length) {
                                        for (var i = 0; i < m.keyword.length; ++i)
                                            $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword[i], w.uint32(10).fork()).ldelim();
                                    }
                                    if (m.suggested_keywords_count != null && Object.hasOwnProperty.call(m, "suggested_keywords_count"))
                                        w.uint32(16).int64(m.suggested_keywords_count);
                                    if (m.campaign_keywords_count != null && Object.hasOwnProperty.call(m, "campaign_keywords_count"))
                                        w.uint32(24).int64(m.campaign_keywords_count);
                                    if (m.campaign_uses_shared_budget != null && Object.hasOwnProperty.call(m, "campaign_uses_shared_budget"))
                                        w.uint32(32).bool(m.campaign_uses_shared_budget);
                                    if (m.required_campaign_budget_amount_micros != null && Object.hasOwnProperty.call(m, "required_campaign_budget_amount_micros"))
                                        w.uint32(40).int64(m.required_campaign_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified UseBroadMatchKeywordRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUseBroadMatchKeywordRecommendation} message UseBroadMatchKeywordRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                UseBroadMatchKeywordRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a UseBroadMatchKeywordRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation} UseBroadMatchKeywordRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                UseBroadMatchKeywordRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.keyword && m.keyword.length))
                                                m.keyword = [];
                                            m.keyword.push($root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32()));
                                            break;
                                        case 2:
                                            m.suggested_keywords_count = r.int64();
                                            break;
                                        case 3:
                                            m.campaign_keywords_count = r.int64();
                                            break;
                                        case 4:
                                            m.campaign_uses_shared_budget = r.bool();
                                            break;
                                        case 5:
                                            m.required_campaign_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a UseBroadMatchKeywordRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation} UseBroadMatchKeywordRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                UseBroadMatchKeywordRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a UseBroadMatchKeywordRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                UseBroadMatchKeywordRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                        if (!Array.isArray(m.keyword))
                                            return "keyword: array expected";
                                        for (var i = 0; i < m.keyword.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword[i]);
                                                if (e)
                                                    return "keyword." + e;
                                            }
                                        }
                                    }
                                    if (m.suggested_keywords_count != null && m.hasOwnProperty("suggested_keywords_count")) {
                                        if (!$util.isInteger(m.suggested_keywords_count) && !(m.suggested_keywords_count && $util.isInteger(m.suggested_keywords_count.low) && $util.isInteger(m.suggested_keywords_count.high)))
                                            return "suggested_keywords_count: integer|Long expected";
                                    }
                                    if (m.campaign_keywords_count != null && m.hasOwnProperty("campaign_keywords_count")) {
                                        if (!$util.isInteger(m.campaign_keywords_count) && !(m.campaign_keywords_count && $util.isInteger(m.campaign_keywords_count.low) && $util.isInteger(m.campaign_keywords_count.high)))
                                            return "campaign_keywords_count: integer|Long expected";
                                    }
                                    if (m.campaign_uses_shared_budget != null && m.hasOwnProperty("campaign_uses_shared_budget")) {
                                        if (typeof m.campaign_uses_shared_budget !== "boolean")
                                            return "campaign_uses_shared_budget: boolean expected";
                                    }
                                    if (m.required_campaign_budget_amount_micros != null && m.hasOwnProperty("required_campaign_budget_amount_micros")) {
                                        if (!$util.isInteger(m.required_campaign_budget_amount_micros) && !(m.required_campaign_budget_amount_micros && $util.isInteger(m.required_campaign_budget_amount_micros.low) && $util.isInteger(m.required_campaign_budget_amount_micros.high)))
                                            return "required_campaign_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a UseBroadMatchKeywordRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation} UseBroadMatchKeywordRecommendation
                                 */
                                UseBroadMatchKeywordRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation();
                                    if (d.keyword) {
                                        if (!Array.isArray(d.keyword))
                                            throw TypeError(".google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.keyword: array expected");
                                        m.keyword = [];
                                        for (var i = 0; i < d.keyword.length; ++i) {
                                            if (typeof d.keyword[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation.keyword: object expected");
                                            m.keyword[i] = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword[i]);
                                        }
                                    }
                                    if (d.suggested_keywords_count != null) {
                                        if ($util.Long)
                                            (m.suggested_keywords_count = $util.Long.fromValue(d.suggested_keywords_count)).unsigned = false;
                                        else if (typeof d.suggested_keywords_count === "string")
                                            m.suggested_keywords_count = parseInt(d.suggested_keywords_count, 10);
                                        else if (typeof d.suggested_keywords_count === "number")
                                            m.suggested_keywords_count = d.suggested_keywords_count;
                                        else if (typeof d.suggested_keywords_count === "object")
                                            m.suggested_keywords_count = new $util.LongBits(d.suggested_keywords_count.low >>> 0, d.suggested_keywords_count.high >>> 0).toNumber();
                                    }
                                    if (d.campaign_keywords_count != null) {
                                        if ($util.Long)
                                            (m.campaign_keywords_count = $util.Long.fromValue(d.campaign_keywords_count)).unsigned = false;
                                        else if (typeof d.campaign_keywords_count === "string")
                                            m.campaign_keywords_count = parseInt(d.campaign_keywords_count, 10);
                                        else if (typeof d.campaign_keywords_count === "number")
                                            m.campaign_keywords_count = d.campaign_keywords_count;
                                        else if (typeof d.campaign_keywords_count === "object")
                                            m.campaign_keywords_count = new $util.LongBits(d.campaign_keywords_count.low >>> 0, d.campaign_keywords_count.high >>> 0).toNumber();
                                    }
                                    if (d.campaign_uses_shared_budget != null) {
                                        m.campaign_uses_shared_budget = Boolean(d.campaign_uses_shared_budget);
                                    }
                                    if (d.required_campaign_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.required_campaign_budget_amount_micros = $util.Long.fromValue(d.required_campaign_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.required_campaign_budget_amount_micros === "string")
                                            m.required_campaign_budget_amount_micros = parseInt(d.required_campaign_budget_amount_micros, 10);
                                        else if (typeof d.required_campaign_budget_amount_micros === "number")
                                            m.required_campaign_budget_amount_micros = d.required_campaign_budget_amount_micros;
                                        else if (typeof d.required_campaign_budget_amount_micros === "object")
                                            m.required_campaign_budget_amount_micros = new $util.LongBits(d.required_campaign_budget_amount_micros.low >>> 0, d.required_campaign_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a UseBroadMatchKeywordRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation} m UseBroadMatchKeywordRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                UseBroadMatchKeywordRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.keyword = [];
                                    }
                                    if (o.defaults) {
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.suggested_keywords_count = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.suggested_keywords_count = o.longs === String ? "0" : 0;
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.campaign_keywords_count = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.campaign_keywords_count = o.longs === String ? "0" : 0;
                                        d.campaign_uses_shared_budget = false;
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.required_campaign_budget_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.required_campaign_budget_amount_micros = o.longs === String ? "0" : 0;
                                    }
                                    if (m.keyword && m.keyword.length) {
                                        d.keyword = [];
                                        for (var j = 0; j < m.keyword.length; ++j) {
                                            d.keyword[j] = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword[j], o);
                                        }
                                    }
                                    if (m.suggested_keywords_count != null && m.hasOwnProperty("suggested_keywords_count")) {
                                        if (typeof m.suggested_keywords_count === "number")
                                            d.suggested_keywords_count = o.longs === String ? String(m.suggested_keywords_count) : m.suggested_keywords_count;
                                        else
                                            d.suggested_keywords_count = o.longs === String ? $util.Long.prototype.toString.call(m.suggested_keywords_count) : o.longs === Number ? new $util.LongBits(m.suggested_keywords_count.low >>> 0, m.suggested_keywords_count.high >>> 0).toNumber() : m.suggested_keywords_count;
                                    }
                                    if (m.campaign_keywords_count != null && m.hasOwnProperty("campaign_keywords_count")) {
                                        if (typeof m.campaign_keywords_count === "number")
                                            d.campaign_keywords_count = o.longs === String ? String(m.campaign_keywords_count) : m.campaign_keywords_count;
                                        else
                                            d.campaign_keywords_count = o.longs === String ? $util.Long.prototype.toString.call(m.campaign_keywords_count) : o.longs === Number ? new $util.LongBits(m.campaign_keywords_count.low >>> 0, m.campaign_keywords_count.high >>> 0).toNumber() : m.campaign_keywords_count;
                                    }
                                    if (m.campaign_uses_shared_budget != null && m.hasOwnProperty("campaign_uses_shared_budget")) {
                                        d.campaign_uses_shared_budget = m.campaign_uses_shared_budget;
                                    }
                                    if (m.required_campaign_budget_amount_micros != null && m.hasOwnProperty("required_campaign_budget_amount_micros")) {
                                        if (typeof m.required_campaign_budget_amount_micros === "number")
                                            d.required_campaign_budget_amount_micros = o.longs === String ? String(m.required_campaign_budget_amount_micros) : m.required_campaign_budget_amount_micros;
                                        else
                                            d.required_campaign_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.required_campaign_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.required_campaign_budget_amount_micros.low >>> 0, m.required_campaign_budget_amount_micros.high >>> 0).toNumber() : m.required_campaign_budget_amount_micros;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this UseBroadMatchKeywordRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UseBroadMatchKeywordRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                UseBroadMatchKeywordRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return UseBroadMatchKeywordRecommendation;
                            })();
    
                            Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation = (function() {
    
                                /**
                                 * Properties of an UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @interface IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @property {number|null} [merchant_id] UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation merchant_id
                                 * @property {string|null} [sales_country_code] UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation sales_country_code
                                 */
    
                                /**
                                 * Constructs a new UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.
                                 * @memberof google.ads.googleads.v11.resources.Recommendation
                                 * @classdesc Represents an UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.
                                 * @implements IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation=} [p] Properties to set
                                 */
                                function UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation merchant_id.
                                 * @member {number} merchant_id
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @instance
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.prototype.merchant_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation sales_country_code.
                                 * @member {string} sales_country_code
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @instance
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.prototype.sales_country_code = "";
    
                                /**
                                 * Creates a new UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation} UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation instance
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.create = function create(properties) {
                                    return new UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation(properties);
                                };
    
                                /**
                                 * Encodes the specified UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation} m UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.merchant_id != null && Object.hasOwnProperty.call(m, "merchant_id"))
                                        w.uint32(8).int64(m.merchant_id);
                                    if (m.sales_country_code != null && Object.hasOwnProperty.call(m, "sales_country_code"))
                                        w.uint32(18).string(m.sales_country_code);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.IUpgradeSmartShoppingCampaignToPerformanceMaxRecommendation} message UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation} UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.merchant_id = r.int64();
                                            break;
                                        case 2:
                                            m.sales_country_code = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation} UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                        if (!$util.isInteger(m.merchant_id) && !(m.merchant_id && $util.isInteger(m.merchant_id.low) && $util.isInteger(m.merchant_id.high)))
                                            return "merchant_id: integer|Long expected";
                                    }
                                    if (m.sales_country_code != null && m.hasOwnProperty("sales_country_code")) {
                                        if (!$util.isString(m.sales_country_code))
                                            return "sales_country_code: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation} UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation();
                                    if (d.merchant_id != null) {
                                        if ($util.Long)
                                            (m.merchant_id = $util.Long.fromValue(d.merchant_id)).unsigned = false;
                                        else if (typeof d.merchant_id === "string")
                                            m.merchant_id = parseInt(d.merchant_id, 10);
                                        else if (typeof d.merchant_id === "number")
                                            m.merchant_id = d.merchant_id;
                                        else if (typeof d.merchant_id === "object")
                                            m.merchant_id = new $util.LongBits(d.merchant_id.low >>> 0, d.merchant_id.high >>> 0).toNumber();
                                    }
                                    if (d.sales_country_code != null) {
                                        m.sales_country_code = String(d.sales_country_code);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation} m UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.merchant_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.merchant_id = o.longs === String ? "0" : 0;
                                        d.sales_country_code = "";
                                    }
                                    if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                        if (typeof m.merchant_id === "number")
                                            d.merchant_id = o.longs === String ? String(m.merchant_id) : m.merchant_id;
                                        else
                                            d.merchant_id = o.longs === String ? $util.Long.prototype.toString.call(m.merchant_id) : o.longs === Number ? new $util.LongBits(m.merchant_id.low >>> 0, m.merchant_id.high >>> 0).toNumber() : m.merchant_id;
                                    }
                                    if (m.sales_country_code != null && m.hasOwnProperty("sales_country_code")) {
                                        d.sales_country_code = m.sales_country_code;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation;
                            })();
    
                            return Recommendation;
                        })();
    
                        resources.RemarketingAction = (function() {
    
                            /**
                             * Properties of a RemarketingAction.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IRemarketingAction
                             * @property {string|null} [resource_name] RemarketingAction resource_name
                             * @property {number|null} [id] RemarketingAction id
                             * @property {string|null} [name] RemarketingAction name
                             * @property {Array.<google.ads.googleads.v11.common.ITagSnippet>|null} [tag_snippets] RemarketingAction tag_snippets
                             */
    
                            /**
                             * Constructs a new RemarketingAction.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a RemarketingAction.
                             * @implements IRemarketingAction
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IRemarketingAction=} [p] Properties to set
                             */
                            function RemarketingAction(p) {
                                this.tag_snippets = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RemarketingAction resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @instance
                             */
                            RemarketingAction.prototype.resource_name = "";
    
                            /**
                             * RemarketingAction id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @instance
                             */
                            RemarketingAction.prototype.id = null;
    
                            /**
                             * RemarketingAction name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @instance
                             */
                            RemarketingAction.prototype.name = null;
    
                            /**
                             * RemarketingAction tag_snippets.
                             * @member {Array.<google.ads.googleads.v11.common.ITagSnippet>} tag_snippets
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @instance
                             */
                            RemarketingAction.prototype.tag_snippets = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * RemarketingAction _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @instance
                             */
                            Object.defineProperty(RemarketingAction.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * RemarketingAction _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @instance
                             */
                            Object.defineProperty(RemarketingAction.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new RemarketingAction instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRemarketingAction=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.RemarketingAction} RemarketingAction instance
                             */
                            RemarketingAction.create = function create(properties) {
                                return new RemarketingAction(properties);
                            };
    
                            /**
                             * Encodes the specified RemarketingAction message. Does not implicitly {@link google.ads.googleads.v11.resources.RemarketingAction.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRemarketingAction} m RemarketingAction message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RemarketingAction.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.tag_snippets != null && m.tag_snippets.length) {
                                    for (var i = 0; i < m.tag_snippets.length; ++i)
                                        $root.google.ads.googleads.v11.common.TagSnippet.encode(m.tag_snippets[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(40).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(50).string(m.name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified RemarketingAction message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.RemarketingAction.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.IRemarketingAction} message RemarketingAction message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RemarketingAction.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RemarketingAction message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.RemarketingAction} RemarketingAction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RemarketingAction.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.RemarketingAction();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 5:
                                        m.id = r.int64();
                                        break;
                                    case 6:
                                        m.name = r.string();
                                        break;
                                    case 4:
                                        if (!(m.tag_snippets && m.tag_snippets.length))
                                            m.tag_snippets = [];
                                        m.tag_snippets.push($root.google.ads.googleads.v11.common.TagSnippet.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RemarketingAction message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.RemarketingAction} RemarketingAction
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RemarketingAction.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RemarketingAction message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RemarketingAction.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.tag_snippets != null && m.hasOwnProperty("tag_snippets")) {
                                    if (!Array.isArray(m.tag_snippets))
                                        return "tag_snippets: array expected";
                                    for (var i = 0; i < m.tag_snippets.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.TagSnippet.verify(m.tag_snippets[i]);
                                            if (e)
                                                return "tag_snippets." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RemarketingAction message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.RemarketingAction} RemarketingAction
                             */
                            RemarketingAction.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.RemarketingAction)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.RemarketingAction();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.tag_snippets) {
                                    if (!Array.isArray(d.tag_snippets))
                                        throw TypeError(".google.ads.googleads.v11.resources.RemarketingAction.tag_snippets: array expected");
                                    m.tag_snippets = [];
                                    for (var i = 0; i < d.tag_snippets.length; ++i) {
                                        if (typeof d.tag_snippets[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.RemarketingAction.tag_snippets: object expected");
                                        m.tag_snippets[i] = $root.google.ads.googleads.v11.common.TagSnippet.fromObject(d.tag_snippets[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RemarketingAction message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @static
                             * @param {google.ads.googleads.v11.resources.RemarketingAction} m RemarketingAction
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RemarketingAction.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.tag_snippets = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.tag_snippets && m.tag_snippets.length) {
                                    d.tag_snippets = [];
                                    for (var j = 0; j < m.tag_snippets.length; ++j) {
                                        d.tag_snippets[j] = $root.google.ads.googleads.v11.common.TagSnippet.toObject(m.tag_snippets[j], o);
                                    }
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RemarketingAction to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.RemarketingAction
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RemarketingAction.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RemarketingAction;
                        })();
    
                        resources.SearchTermView = (function() {
    
                            /**
                             * Properties of a SearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ISearchTermView
                             * @property {string|null} [resource_name] SearchTermView resource_name
                             * @property {string|null} [search_term] SearchTermView search_term
                             * @property {string|null} [ad_group] SearchTermView ad_group
                             * @property {google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum.SearchTermTargetingStatus|null} [status] SearchTermView status
                             */
    
                            /**
                             * Constructs a new SearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a SearchTermView.
                             * @implements ISearchTermView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ISearchTermView=} [p] Properties to set
                             */
                            function SearchTermView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SearchTermView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @instance
                             */
                            SearchTermView.prototype.resource_name = "";
    
                            /**
                             * SearchTermView search_term.
                             * @member {string|null|undefined} search_term
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @instance
                             */
                            SearchTermView.prototype.search_term = null;
    
                            /**
                             * SearchTermView ad_group.
                             * @member {string|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @instance
                             */
                            SearchTermView.prototype.ad_group = null;
    
                            /**
                             * SearchTermView status.
                             * @member {google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum.SearchTermTargetingStatus} status
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @instance
                             */
                            SearchTermView.prototype.status = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SearchTermView _search_term.
                             * @member {"search_term"|undefined} _search_term
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @instance
                             */
                            Object.defineProperty(SearchTermView.prototype, "_search_term", {
                                get: $util.oneOfGetter($oneOfFields = ["search_term"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SearchTermView _ad_group.
                             * @member {"ad_group"|undefined} _ad_group
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @instance
                             */
                            Object.defineProperty(SearchTermView.prototype, "_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SearchTermView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISearchTermView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.SearchTermView} SearchTermView instance
                             */
                            SearchTermView.create = function create(properties) {
                                return new SearchTermView(properties);
                            };
    
                            /**
                             * Encodes the specified SearchTermView message. Does not implicitly {@link google.ads.googleads.v11.resources.SearchTermView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISearchTermView} m SearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchTermView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(32).int32(m.status);
                                if (m.search_term != null && Object.hasOwnProperty.call(m, "search_term"))
                                    w.uint32(42).string(m.search_term);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(50).string(m.ad_group);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchTermView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.SearchTermView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISearchTermView} message SearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchTermView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchTermView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.SearchTermView} SearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchTermView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.SearchTermView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 5:
                                        m.search_term = r.string();
                                        break;
                                    case 6:
                                        m.ad_group = r.string();
                                        break;
                                    case 4:
                                        m.status = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchTermView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.SearchTermView} SearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchTermView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchTermView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchTermView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    p._search_term = 1;
                                    if (!$util.isString(m.search_term))
                                        return "search_term: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    p._ad_group = 1;
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SearchTermView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.SearchTermView} SearchTermView
                             */
                            SearchTermView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.SearchTermView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.SearchTermView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.search_term != null) {
                                    m.search_term = String(d.search_term);
                                }
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ADDED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "EXCLUDED":
                                case 3:
                                    m.status = 3;
                                    break;
                                case "ADDED_EXCLUDED":
                                case 4:
                                    m.status = 4;
                                    break;
                                case "NONE":
                                case 5:
                                    m.status = 5;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SearchTermView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.SearchTermView} m SearchTermView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchTermView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.SearchTermTargetingStatusEnum.SearchTermTargetingStatus[m.status] : m.status;
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    d.search_term = m.search_term;
                                    if (o.oneofs)
                                        d._search_term = "search_term";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                    if (o.oneofs)
                                        d._ad_group = "ad_group";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SearchTermView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.SearchTermView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchTermView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SearchTermView;
                        })();
    
                        resources.SharedCriterion = (function() {
    
                            /**
                             * Properties of a SharedCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ISharedCriterion
                             * @property {string|null} [resource_name] SharedCriterion resource_name
                             * @property {string|null} [shared_set] SharedCriterion shared_set
                             * @property {number|null} [criterion_id] SharedCriterion criterion_id
                             * @property {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType|null} [type] SharedCriterion type
                             * @property {google.ads.googleads.v11.common.IKeywordInfo|null} [keyword] SharedCriterion keyword
                             * @property {google.ads.googleads.v11.common.IYouTubeVideoInfo|null} [youtube_video] SharedCriterion youtube_video
                             * @property {google.ads.googleads.v11.common.IYouTubeChannelInfo|null} [youtube_channel] SharedCriterion youtube_channel
                             * @property {google.ads.googleads.v11.common.IPlacementInfo|null} [placement] SharedCriterion placement
                             * @property {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null} [mobile_app_category] SharedCriterion mobile_app_category
                             * @property {google.ads.googleads.v11.common.IMobileApplicationInfo|null} [mobile_application] SharedCriterion mobile_application
                             */
    
                            /**
                             * Constructs a new SharedCriterion.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a SharedCriterion.
                             * @implements ISharedCriterion
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ISharedCriterion=} [p] Properties to set
                             */
                            function SharedCriterion(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SharedCriterion resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.resource_name = "";
    
                            /**
                             * SharedCriterion shared_set.
                             * @member {string|null|undefined} shared_set
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.shared_set = null;
    
                            /**
                             * SharedCriterion criterion_id.
                             * @member {number|null|undefined} criterion_id
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.criterion_id = null;
    
                            /**
                             * SharedCriterion type.
                             * @member {google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType} type
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.type = 0;
    
                            /**
                             * SharedCriterion keyword.
                             * @member {google.ads.googleads.v11.common.IKeywordInfo|null|undefined} keyword
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.keyword = null;
    
                            /**
                             * SharedCriterion youtube_video.
                             * @member {google.ads.googleads.v11.common.IYouTubeVideoInfo|null|undefined} youtube_video
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.youtube_video = null;
    
                            /**
                             * SharedCriterion youtube_channel.
                             * @member {google.ads.googleads.v11.common.IYouTubeChannelInfo|null|undefined} youtube_channel
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.youtube_channel = null;
    
                            /**
                             * SharedCriterion placement.
                             * @member {google.ads.googleads.v11.common.IPlacementInfo|null|undefined} placement
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.placement = null;
    
                            /**
                             * SharedCriterion mobile_app_category.
                             * @member {google.ads.googleads.v11.common.IMobileAppCategoryInfo|null|undefined} mobile_app_category
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.mobile_app_category = null;
    
                            /**
                             * SharedCriterion mobile_application.
                             * @member {google.ads.googleads.v11.common.IMobileApplicationInfo|null|undefined} mobile_application
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            SharedCriterion.prototype.mobile_application = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SharedCriterion _shared_set.
                             * @member {"shared_set"|undefined} _shared_set
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            Object.defineProperty(SharedCriterion.prototype, "_shared_set", {
                                get: $util.oneOfGetter($oneOfFields = ["shared_set"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SharedCriterion _criterion_id.
                             * @member {"criterion_id"|undefined} _criterion_id
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            Object.defineProperty(SharedCriterion.prototype, "_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SharedCriterion criterion.
                             * @member {"keyword"|"youtube_video"|"youtube_channel"|"placement"|"mobile_app_category"|"mobile_application"|undefined} criterion
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             */
                            Object.defineProperty(SharedCriterion.prototype, "criterion", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword", "youtube_video", "youtube_channel", "placement", "mobile_app_category", "mobile_application"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SharedCriterion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISharedCriterion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.SharedCriterion} SharedCriterion instance
                             */
                            SharedCriterion.create = function create(properties) {
                                return new SharedCriterion(properties);
                            };
    
                            /**
                             * Encodes the specified SharedCriterion message. Does not implicitly {@link google.ads.googleads.v11.resources.SharedCriterion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISharedCriterion} m SharedCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedCriterion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    $root.google.ads.googleads.v11.common.KeywordInfo.encode(m.keyword, w.uint32(26).fork()).ldelim();
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(32).int32(m.type);
                                if (m.youtube_video != null && Object.hasOwnProperty.call(m, "youtube_video"))
                                    $root.google.ads.googleads.v11.common.YouTubeVideoInfo.encode(m.youtube_video, w.uint32(42).fork()).ldelim();
                                if (m.youtube_channel != null && Object.hasOwnProperty.call(m, "youtube_channel"))
                                    $root.google.ads.googleads.v11.common.YouTubeChannelInfo.encode(m.youtube_channel, w.uint32(50).fork()).ldelim();
                                if (m.placement != null && Object.hasOwnProperty.call(m, "placement"))
                                    $root.google.ads.googleads.v11.common.PlacementInfo.encode(m.placement, w.uint32(58).fork()).ldelim();
                                if (m.mobile_app_category != null && Object.hasOwnProperty.call(m, "mobile_app_category"))
                                    $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.encode(m.mobile_app_category, w.uint32(66).fork()).ldelim();
                                if (m.mobile_application != null && Object.hasOwnProperty.call(m, "mobile_application"))
                                    $root.google.ads.googleads.v11.common.MobileApplicationInfo.encode(m.mobile_application, w.uint32(74).fork()).ldelim();
                                if (m.shared_set != null && Object.hasOwnProperty.call(m, "shared_set"))
                                    w.uint32(82).string(m.shared_set);
                                if (m.criterion_id != null && Object.hasOwnProperty.call(m, "criterion_id"))
                                    w.uint32(88).int64(m.criterion_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedCriterion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.SharedCriterion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISharedCriterion} message SharedCriterion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedCriterion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedCriterion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.SharedCriterion} SharedCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedCriterion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.SharedCriterion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 10:
                                        m.shared_set = r.string();
                                        break;
                                    case 11:
                                        m.criterion_id = r.int64();
                                        break;
                                    case 4:
                                        m.type = r.int32();
                                        break;
                                    case 3:
                                        m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedCriterion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.SharedCriterion} SharedCriterion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedCriterion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedCriterion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedCriterion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    p._shared_set = 1;
                                    if (!$util.isString(m.shared_set))
                                        return "shared_set: string expected";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    p._criterion_id = 1;
                                    if (!$util.isInteger(m.criterion_id) && !(m.criterion_id && $util.isInteger(m.criterion_id.low) && $util.isInteger(m.criterion_id.high)))
                                        return "criterion_id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 21:
                                    case 22:
                                    case 23:
                                    case 24:
                                    case 25:
                                    case 26:
                                    case 27:
                                    case 28:
                                    case 29:
                                    case 30:
                                    case 31:
                                    case 32:
                                    case 33:
                                    case 34:
                                    case 35:
                                        break;
                                    }
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordInfo.verify(m.keyword);
                                        if (e)
                                            return "keyword." + e;
                                    }
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.verify(m.youtube_video);
                                        if (e)
                                            return "youtube_video." + e;
                                    }
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.verify(m.youtube_channel);
                                        if (e)
                                            return "youtube_channel." + e;
                                    }
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PlacementInfo.verify(m.placement);
                                        if (e)
                                            return "placement." + e;
                                    }
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.verify(m.mobile_app_category);
                                        if (e)
                                            return "mobile_app_category." + e;
                                    }
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    if (p.criterion === 1)
                                        return "criterion: multiple values";
                                    p.criterion = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.MobileApplicationInfo.verify(m.mobile_application);
                                        if (e)
                                            return "mobile_application." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SharedCriterion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.SharedCriterion} SharedCriterion
                             */
                            SharedCriterion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.SharedCriterion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.SharedCriterion();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.shared_set != null) {
                                    m.shared_set = String(d.shared_set);
                                }
                                if (d.criterion_id != null) {
                                    if ($util.Long)
                                        (m.criterion_id = $util.Long.fromValue(d.criterion_id)).unsigned = false;
                                    else if (typeof d.criterion_id === "string")
                                        m.criterion_id = parseInt(d.criterion_id, 10);
                                    else if (typeof d.criterion_id === "number")
                                        m.criterion_id = d.criterion_id;
                                    else if (typeof d.criterion_id === "object")
                                        m.criterion_id = new $util.LongBits(d.criterion_id.low >>> 0, d.criterion_id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "KEYWORD":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "PLACEMENT":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "MOBILE_APP_CATEGORY":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "MOBILE_APPLICATION":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "DEVICE":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "LOCATION":
                                case 7:
                                    m.type = 7;
                                    break;
                                case "LISTING_GROUP":
                                case 8:
                                    m.type = 8;
                                    break;
                                case "AD_SCHEDULE":
                                case 9:
                                    m.type = 9;
                                    break;
                                case "AGE_RANGE":
                                case 10:
                                    m.type = 10;
                                    break;
                                case "GENDER":
                                case 11:
                                    m.type = 11;
                                    break;
                                case "INCOME_RANGE":
                                case 12:
                                    m.type = 12;
                                    break;
                                case "PARENTAL_STATUS":
                                case 13:
                                    m.type = 13;
                                    break;
                                case "YOUTUBE_VIDEO":
                                case 14:
                                    m.type = 14;
                                    break;
                                case "YOUTUBE_CHANNEL":
                                case 15:
                                    m.type = 15;
                                    break;
                                case "USER_LIST":
                                case 16:
                                    m.type = 16;
                                    break;
                                case "PROXIMITY":
                                case 17:
                                    m.type = 17;
                                    break;
                                case "TOPIC":
                                case 18:
                                    m.type = 18;
                                    break;
                                case "LISTING_SCOPE":
                                case 19:
                                    m.type = 19;
                                    break;
                                case "LANGUAGE":
                                case 20:
                                    m.type = 20;
                                    break;
                                case "IP_BLOCK":
                                case 21:
                                    m.type = 21;
                                    break;
                                case "CONTENT_LABEL":
                                case 22:
                                    m.type = 22;
                                    break;
                                case "CARRIER":
                                case 23:
                                    m.type = 23;
                                    break;
                                case "USER_INTEREST":
                                case 24:
                                    m.type = 24;
                                    break;
                                case "WEBPAGE":
                                case 25:
                                    m.type = 25;
                                    break;
                                case "OPERATING_SYSTEM_VERSION":
                                case 26:
                                    m.type = 26;
                                    break;
                                case "APP_PAYMENT_MODEL":
                                case 27:
                                    m.type = 27;
                                    break;
                                case "MOBILE_DEVICE":
                                case 28:
                                    m.type = 28;
                                    break;
                                case "CUSTOM_AFFINITY":
                                case 29:
                                    m.type = 29;
                                    break;
                                case "CUSTOM_INTENT":
                                case 30:
                                    m.type = 30;
                                    break;
                                case "LOCATION_GROUP":
                                case 31:
                                    m.type = 31;
                                    break;
                                case "CUSTOM_AUDIENCE":
                                case 32:
                                    m.type = 32;
                                    break;
                                case "COMBINED_AUDIENCE":
                                case 33:
                                    m.type = 33;
                                    break;
                                case "KEYWORD_THEME":
                                case 34:
                                    m.type = 34;
                                    break;
                                case "AUDIENCE":
                                case 35:
                                    m.type = 35;
                                    break;
                                }
                                if (d.keyword != null) {
                                    if (typeof d.keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SharedCriterion.keyword: object expected");
                                    m.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.fromObject(d.keyword);
                                }
                                if (d.youtube_video != null) {
                                    if (typeof d.youtube_video !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SharedCriterion.youtube_video: object expected");
                                    m.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.fromObject(d.youtube_video);
                                }
                                if (d.youtube_channel != null) {
                                    if (typeof d.youtube_channel !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SharedCriterion.youtube_channel: object expected");
                                    m.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.fromObject(d.youtube_channel);
                                }
                                if (d.placement != null) {
                                    if (typeof d.placement !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SharedCriterion.placement: object expected");
                                    m.placement = $root.google.ads.googleads.v11.common.PlacementInfo.fromObject(d.placement);
                                }
                                if (d.mobile_app_category != null) {
                                    if (typeof d.mobile_app_category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SharedCriterion.mobile_app_category: object expected");
                                    m.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.fromObject(d.mobile_app_category);
                                }
                                if (d.mobile_application != null) {
                                    if (typeof d.mobile_application !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SharedCriterion.mobile_application: object expected");
                                    m.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.fromObject(d.mobile_application);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SharedCriterion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @static
                             * @param {google.ads.googleads.v11.resources.SharedCriterion} m SharedCriterion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedCriterion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = $root.google.ads.googleads.v11.common.KeywordInfo.toObject(m.keyword, o);
                                    if (o.oneofs)
                                        d.criterion = "keyword";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.CriterionTypeEnum.CriterionType[m.type] : m.type;
                                }
                                if (m.youtube_video != null && m.hasOwnProperty("youtube_video")) {
                                    d.youtube_video = $root.google.ads.googleads.v11.common.YouTubeVideoInfo.toObject(m.youtube_video, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_video";
                                }
                                if (m.youtube_channel != null && m.hasOwnProperty("youtube_channel")) {
                                    d.youtube_channel = $root.google.ads.googleads.v11.common.YouTubeChannelInfo.toObject(m.youtube_channel, o);
                                    if (o.oneofs)
                                        d.criterion = "youtube_channel";
                                }
                                if (m.placement != null && m.hasOwnProperty("placement")) {
                                    d.placement = $root.google.ads.googleads.v11.common.PlacementInfo.toObject(m.placement, o);
                                    if (o.oneofs)
                                        d.criterion = "placement";
                                }
                                if (m.mobile_app_category != null && m.hasOwnProperty("mobile_app_category")) {
                                    d.mobile_app_category = $root.google.ads.googleads.v11.common.MobileAppCategoryInfo.toObject(m.mobile_app_category, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_app_category";
                                }
                                if (m.mobile_application != null && m.hasOwnProperty("mobile_application")) {
                                    d.mobile_application = $root.google.ads.googleads.v11.common.MobileApplicationInfo.toObject(m.mobile_application, o);
                                    if (o.oneofs)
                                        d.criterion = "mobile_application";
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    d.shared_set = m.shared_set;
                                    if (o.oneofs)
                                        d._shared_set = "shared_set";
                                }
                                if (m.criterion_id != null && m.hasOwnProperty("criterion_id")) {
                                    if (typeof m.criterion_id === "number")
                                        d.criterion_id = o.longs === String ? String(m.criterion_id) : m.criterion_id;
                                    else
                                        d.criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.criterion_id) : o.longs === Number ? new $util.LongBits(m.criterion_id.low >>> 0, m.criterion_id.high >>> 0).toNumber() : m.criterion_id;
                                    if (o.oneofs)
                                        d._criterion_id = "criterion_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SharedCriterion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.SharedCriterion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedCriterion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SharedCriterion;
                        })();
    
                        resources.SharedSet = (function() {
    
                            /**
                             * Properties of a SharedSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ISharedSet
                             * @property {string|null} [resource_name] SharedSet resource_name
                             * @property {number|null} [id] SharedSet id
                             * @property {google.ads.googleads.v11.enums.SharedSetTypeEnum.SharedSetType|null} [type] SharedSet type
                             * @property {string|null} [name] SharedSet name
                             * @property {google.ads.googleads.v11.enums.SharedSetStatusEnum.SharedSetStatus|null} [status] SharedSet status
                             * @property {number|null} [member_count] SharedSet member_count
                             * @property {number|null} [reference_count] SharedSet reference_count
                             */
    
                            /**
                             * Constructs a new SharedSet.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a SharedSet.
                             * @implements ISharedSet
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ISharedSet=} [p] Properties to set
                             */
                            function SharedSet(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SharedSet resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            SharedSet.prototype.resource_name = "";
    
                            /**
                             * SharedSet id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            SharedSet.prototype.id = null;
    
                            /**
                             * SharedSet type.
                             * @member {google.ads.googleads.v11.enums.SharedSetTypeEnum.SharedSetType} type
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            SharedSet.prototype.type = 0;
    
                            /**
                             * SharedSet name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            SharedSet.prototype.name = null;
    
                            /**
                             * SharedSet status.
                             * @member {google.ads.googleads.v11.enums.SharedSetStatusEnum.SharedSetStatus} status
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            SharedSet.prototype.status = 0;
    
                            /**
                             * SharedSet member_count.
                             * @member {number|null|undefined} member_count
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            SharedSet.prototype.member_count = null;
    
                            /**
                             * SharedSet reference_count.
                             * @member {number|null|undefined} reference_count
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            SharedSet.prototype.reference_count = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SharedSet _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            Object.defineProperty(SharedSet.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SharedSet _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            Object.defineProperty(SharedSet.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SharedSet _member_count.
                             * @member {"member_count"|undefined} _member_count
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            Object.defineProperty(SharedSet.prototype, "_member_count", {
                                get: $util.oneOfGetter($oneOfFields = ["member_count"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SharedSet _reference_count.
                             * @member {"reference_count"|undefined} _reference_count
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             */
                            Object.defineProperty(SharedSet.prototype, "_reference_count", {
                                get: $util.oneOfGetter($oneOfFields = ["reference_count"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SharedSet instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISharedSet=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.SharedSet} SharedSet instance
                             */
                            SharedSet.create = function create(properties) {
                                return new SharedSet(properties);
                            };
    
                            /**
                             * Encodes the specified SharedSet message. Does not implicitly {@link google.ads.googleads.v11.resources.SharedSet.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISharedSet} m SharedSet message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSet.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(24).int32(m.type);
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    w.uint32(40).int32(m.status);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(64).int64(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(74).string(m.name);
                                if (m.member_count != null && Object.hasOwnProperty.call(m, "member_count"))
                                    w.uint32(80).int64(m.member_count);
                                if (m.reference_count != null && Object.hasOwnProperty.call(m, "reference_count"))
                                    w.uint32(88).int64(m.reference_count);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedSet message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.SharedSet.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISharedSet} message SharedSet message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSet.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedSet message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.SharedSet} SharedSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSet.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.SharedSet();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 8:
                                        m.id = r.int64();
                                        break;
                                    case 3:
                                        m.type = r.int32();
                                        break;
                                    case 9:
                                        m.name = r.string();
                                        break;
                                    case 5:
                                        m.status = r.int32();
                                        break;
                                    case 10:
                                        m.member_count = r.int64();
                                        break;
                                    case 11:
                                        m.reference_count = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedSet message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.SharedSet} SharedSet
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSet.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedSet message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedSet.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    switch (m.status) {
                                    default:
                                        return "status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.member_count != null && m.hasOwnProperty("member_count")) {
                                    p._member_count = 1;
                                    if (!$util.isInteger(m.member_count) && !(m.member_count && $util.isInteger(m.member_count.low) && $util.isInteger(m.member_count.high)))
                                        return "member_count: integer|Long expected";
                                }
                                if (m.reference_count != null && m.hasOwnProperty("reference_count")) {
                                    p._reference_count = 1;
                                    if (!$util.isInteger(m.reference_count) && !(m.reference_count && $util.isInteger(m.reference_count.low) && $util.isInteger(m.reference_count.high)))
                                        return "reference_count: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SharedSet message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.SharedSet} SharedSet
                             */
                            SharedSet.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.SharedSet)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.SharedSet();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "NEGATIVE_KEYWORDS":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "NEGATIVE_PLACEMENTS":
                                case 3:
                                    m.type = 3;
                                    break;
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                switch (d.status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.status = 2;
                                    break;
                                case "REMOVED":
                                case 3:
                                    m.status = 3;
                                    break;
                                }
                                if (d.member_count != null) {
                                    if ($util.Long)
                                        (m.member_count = $util.Long.fromValue(d.member_count)).unsigned = false;
                                    else if (typeof d.member_count === "string")
                                        m.member_count = parseInt(d.member_count, 10);
                                    else if (typeof d.member_count === "number")
                                        m.member_count = d.member_count;
                                    else if (typeof d.member_count === "object")
                                        m.member_count = new $util.LongBits(d.member_count.low >>> 0, d.member_count.high >>> 0).toNumber();
                                }
                                if (d.reference_count != null) {
                                    if ($util.Long)
                                        (m.reference_count = $util.Long.fromValue(d.reference_count)).unsigned = false;
                                    else if (typeof d.reference_count === "string")
                                        m.reference_count = parseInt(d.reference_count, 10);
                                    else if (typeof d.reference_count === "number")
                                        m.reference_count = d.reference_count;
                                    else if (typeof d.reference_count === "object")
                                        m.reference_count = new $util.LongBits(d.reference_count.low >>> 0, d.reference_count.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SharedSet message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @static
                             * @param {google.ads.googleads.v11.resources.SharedSet} m SharedSet
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedSet.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.SharedSetTypeEnum.SharedSetType[m.type] : m.type;
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = o.enums === String ? $root.google.ads.googleads.v11.enums.SharedSetStatusEnum.SharedSetStatus[m.status] : m.status;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.member_count != null && m.hasOwnProperty("member_count")) {
                                    if (typeof m.member_count === "number")
                                        d.member_count = o.longs === String ? String(m.member_count) : m.member_count;
                                    else
                                        d.member_count = o.longs === String ? $util.Long.prototype.toString.call(m.member_count) : o.longs === Number ? new $util.LongBits(m.member_count.low >>> 0, m.member_count.high >>> 0).toNumber() : m.member_count;
                                    if (o.oneofs)
                                        d._member_count = "member_count";
                                }
                                if (m.reference_count != null && m.hasOwnProperty("reference_count")) {
                                    if (typeof m.reference_count === "number")
                                        d.reference_count = o.longs === String ? String(m.reference_count) : m.reference_count;
                                    else
                                        d.reference_count = o.longs === String ? $util.Long.prototype.toString.call(m.reference_count) : o.longs === Number ? new $util.LongBits(m.reference_count.low >>> 0, m.reference_count.high >>> 0).toNumber() : m.reference_count;
                                    if (o.oneofs)
                                        d._reference_count = "reference_count";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SharedSet to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.SharedSet
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedSet.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SharedSet;
                        })();
    
                        resources.ShoppingPerformanceView = (function() {
    
                            /**
                             * Properties of a ShoppingPerformanceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IShoppingPerformanceView
                             * @property {string|null} [resource_name] ShoppingPerformanceView resource_name
                             */
    
                            /**
                             * Constructs a new ShoppingPerformanceView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ShoppingPerformanceView.
                             * @implements IShoppingPerformanceView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IShoppingPerformanceView=} [p] Properties to set
                             */
                            function ShoppingPerformanceView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ShoppingPerformanceView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @instance
                             */
                            ShoppingPerformanceView.prototype.resource_name = "";
    
                            /**
                             * Creates a new ShoppingPerformanceView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IShoppingPerformanceView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ShoppingPerformanceView} ShoppingPerformanceView instance
                             */
                            ShoppingPerformanceView.create = function create(properties) {
                                return new ShoppingPerformanceView(properties);
                            };
    
                            /**
                             * Encodes the specified ShoppingPerformanceView message. Does not implicitly {@link google.ads.googleads.v11.resources.ShoppingPerformanceView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IShoppingPerformanceView} m ShoppingPerformanceView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingPerformanceView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ShoppingPerformanceView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ShoppingPerformanceView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IShoppingPerformanceView} message ShoppingPerformanceView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ShoppingPerformanceView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ShoppingPerformanceView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ShoppingPerformanceView} ShoppingPerformanceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingPerformanceView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ShoppingPerformanceView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ShoppingPerformanceView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ShoppingPerformanceView} ShoppingPerformanceView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ShoppingPerformanceView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ShoppingPerformanceView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ShoppingPerformanceView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ShoppingPerformanceView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ShoppingPerformanceView} ShoppingPerformanceView
                             */
                            ShoppingPerformanceView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ShoppingPerformanceView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ShoppingPerformanceView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ShoppingPerformanceView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ShoppingPerformanceView} m ShoppingPerformanceView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ShoppingPerformanceView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ShoppingPerformanceView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ShoppingPerformanceView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ShoppingPerformanceView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ShoppingPerformanceView;
                        })();
    
                        resources.SmartCampaignSearchTermView = (function() {
    
                            /**
                             * Properties of a SmartCampaignSearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ISmartCampaignSearchTermView
                             * @property {string|null} [resource_name] SmartCampaignSearchTermView resource_name
                             * @property {string|null} [search_term] SmartCampaignSearchTermView search_term
                             * @property {string|null} [campaign] SmartCampaignSearchTermView campaign
                             */
    
                            /**
                             * Constructs a new SmartCampaignSearchTermView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a SmartCampaignSearchTermView.
                             * @implements ISmartCampaignSearchTermView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSearchTermView=} [p] Properties to set
                             */
                            function SmartCampaignSearchTermView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SmartCampaignSearchTermView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @instance
                             */
                            SmartCampaignSearchTermView.prototype.resource_name = "";
    
                            /**
                             * SmartCampaignSearchTermView search_term.
                             * @member {string} search_term
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @instance
                             */
                            SmartCampaignSearchTermView.prototype.search_term = "";
    
                            /**
                             * SmartCampaignSearchTermView campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @instance
                             */
                            SmartCampaignSearchTermView.prototype.campaign = "";
    
                            /**
                             * Creates a new SmartCampaignSearchTermView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSearchTermView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSearchTermView} SmartCampaignSearchTermView instance
                             */
                            SmartCampaignSearchTermView.create = function create(properties) {
                                return new SmartCampaignSearchTermView(properties);
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSearchTermView message. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSearchTermView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSearchTermView} m SmartCampaignSearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSearchTermView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.search_term != null && Object.hasOwnProperty.call(m, "search_term"))
                                    w.uint32(18).string(m.search_term);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(26).string(m.campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSearchTermView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSearchTermView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSearchTermView} message SmartCampaignSearchTermView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSearchTermView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SmartCampaignSearchTermView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSearchTermView} SmartCampaignSearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSearchTermView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.search_term = r.string();
                                        break;
                                    case 3:
                                        m.campaign = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SmartCampaignSearchTermView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSearchTermView} SmartCampaignSearchTermView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSearchTermView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SmartCampaignSearchTermView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SmartCampaignSearchTermView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    if (!$util.isString(m.search_term))
                                        return "search_term: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SmartCampaignSearchTermView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSearchTermView} SmartCampaignSearchTermView
                             */
                            SmartCampaignSearchTermView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.search_term != null) {
                                    m.search_term = String(d.search_term);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SmartCampaignSearchTermView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @static
                             * @param {google.ads.googleads.v11.resources.SmartCampaignSearchTermView} m SmartCampaignSearchTermView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SmartCampaignSearchTermView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.search_term = "";
                                    d.campaign = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    d.search_term = m.search_term;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SmartCampaignSearchTermView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSearchTermView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SmartCampaignSearchTermView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SmartCampaignSearchTermView;
                        })();
    
                        resources.SmartCampaignSetting = (function() {
    
                            /**
                             * Properties of a SmartCampaignSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ISmartCampaignSetting
                             * @property {string|null} [resource_name] SmartCampaignSetting resource_name
                             * @property {string|null} [campaign] SmartCampaignSetting campaign
                             * @property {google.ads.googleads.v11.resources.SmartCampaignSetting.IPhoneNumber|null} [phone_number] SmartCampaignSetting phone_number
                             * @property {string|null} [advertising_language_code] SmartCampaignSetting advertising_language_code
                             * @property {string|null} [final_url] SmartCampaignSetting final_url
                             * @property {google.ads.googleads.v11.resources.SmartCampaignSetting.IAdOptimizedBusinessProfileSetting|null} [ad_optimized_business_profile_setting] SmartCampaignSetting ad_optimized_business_profile_setting
                             * @property {string|null} [business_name] SmartCampaignSetting business_name
                             * @property {string|null} [business_profile_location] SmartCampaignSetting business_profile_location
                             */
    
                            /**
                             * Constructs a new SmartCampaignSetting.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a SmartCampaignSetting.
                             * @implements ISmartCampaignSetting
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSetting=} [p] Properties to set
                             */
                            function SmartCampaignSetting(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SmartCampaignSetting resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.resource_name = "";
    
                            /**
                             * SmartCampaignSetting campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.campaign = "";
    
                            /**
                             * SmartCampaignSetting phone_number.
                             * @member {google.ads.googleads.v11.resources.SmartCampaignSetting.IPhoneNumber|null|undefined} phone_number
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.phone_number = null;
    
                            /**
                             * SmartCampaignSetting advertising_language_code.
                             * @member {string} advertising_language_code
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.advertising_language_code = "";
    
                            /**
                             * SmartCampaignSetting final_url.
                             * @member {string|null|undefined} final_url
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.final_url = null;
    
                            /**
                             * SmartCampaignSetting ad_optimized_business_profile_setting.
                             * @member {google.ads.googleads.v11.resources.SmartCampaignSetting.IAdOptimizedBusinessProfileSetting|null|undefined} ad_optimized_business_profile_setting
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.ad_optimized_business_profile_setting = null;
    
                            /**
                             * SmartCampaignSetting business_name.
                             * @member {string|null|undefined} business_name
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.business_name = null;
    
                            /**
                             * SmartCampaignSetting business_profile_location.
                             * @member {string|null|undefined} business_profile_location
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            SmartCampaignSetting.prototype.business_profile_location = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SmartCampaignSetting landing_page.
                             * @member {"final_url"|"ad_optimized_business_profile_setting"|undefined} landing_page
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            Object.defineProperty(SmartCampaignSetting.prototype, "landing_page", {
                                get: $util.oneOfGetter($oneOfFields = ["final_url", "ad_optimized_business_profile_setting"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SmartCampaignSetting business_setting.
                             * @member {"business_name"|"business_profile_location"|undefined} business_setting
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             */
                            Object.defineProperty(SmartCampaignSetting.prototype, "business_setting", {
                                get: $util.oneOfGetter($oneOfFields = ["business_name", "business_profile_location"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SmartCampaignSetting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSetting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting} SmartCampaignSetting instance
                             */
                            SmartCampaignSetting.create = function create(properties) {
                                return new SmartCampaignSetting(properties);
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSetting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSetting} m SmartCampaignSetting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSetting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                    $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber.encode(m.phone_number, w.uint32(26).fork()).ldelim();
                                if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                    w.uint32(42).string(m.business_name);
                                if (m.advertising_language_code != null && Object.hasOwnProperty.call(m, "advertising_language_code"))
                                    w.uint32(58).string(m.advertising_language_code);
                                if (m.final_url != null && Object.hasOwnProperty.call(m, "final_url"))
                                    w.uint32(66).string(m.final_url);
                                if (m.ad_optimized_business_profile_setting != null && Object.hasOwnProperty.call(m, "ad_optimized_business_profile_setting"))
                                    $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting.encode(m.ad_optimized_business_profile_setting, w.uint32(74).fork()).ldelim();
                                if (m.business_profile_location != null && Object.hasOwnProperty.call(m, "business_profile_location"))
                                    w.uint32(82).string(m.business_profile_location);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSetting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.ISmartCampaignSetting} message SmartCampaignSetting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SmartCampaignSetting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting} SmartCampaignSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSetting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.SmartCampaignSetting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    case 3:
                                        m.phone_number = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.advertising_language_code = r.string();
                                        break;
                                    case 8:
                                        m.final_url = r.string();
                                        break;
                                    case 9:
                                        m.ad_optimized_business_profile_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.business_name = r.string();
                                        break;
                                    case 10:
                                        m.business_profile_location = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SmartCampaignSetting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting} SmartCampaignSetting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSetting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SmartCampaignSetting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SmartCampaignSetting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber.verify(m.phone_number);
                                        if (e)
                                            return "phone_number." + e;
                                    }
                                }
                                if (m.advertising_language_code != null && m.hasOwnProperty("advertising_language_code")) {
                                    if (!$util.isString(m.advertising_language_code))
                                        return "advertising_language_code: string expected";
                                }
                                if (m.final_url != null && m.hasOwnProperty("final_url")) {
                                    p.landing_page = 1;
                                    if (!$util.isString(m.final_url))
                                        return "final_url: string expected";
                                }
                                if (m.ad_optimized_business_profile_setting != null && m.hasOwnProperty("ad_optimized_business_profile_setting")) {
                                    if (p.landing_page === 1)
                                        return "landing_page: multiple values";
                                    p.landing_page = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting.verify(m.ad_optimized_business_profile_setting);
                                        if (e)
                                            return "ad_optimized_business_profile_setting." + e;
                                    }
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    p.business_setting = 1;
                                    if (!$util.isString(m.business_name))
                                        return "business_name: string expected";
                                }
                                if (m.business_profile_location != null && m.hasOwnProperty("business_profile_location")) {
                                    if (p.business_setting === 1)
                                        return "business_setting: multiple values";
                                    p.business_setting = 1;
                                    if (!$util.isString(m.business_profile_location))
                                        return "business_profile_location: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SmartCampaignSetting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting} SmartCampaignSetting
                             */
                            SmartCampaignSetting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.SmartCampaignSetting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.SmartCampaignSetting();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.phone_number != null) {
                                    if (typeof d.phone_number !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SmartCampaignSetting.phone_number: object expected");
                                    m.phone_number = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber.fromObject(d.phone_number);
                                }
                                if (d.advertising_language_code != null) {
                                    m.advertising_language_code = String(d.advertising_language_code);
                                }
                                if (d.final_url != null) {
                                    m.final_url = String(d.final_url);
                                }
                                if (d.ad_optimized_business_profile_setting != null) {
                                    if (typeof d.ad_optimized_business_profile_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.SmartCampaignSetting.ad_optimized_business_profile_setting: object expected");
                                    m.ad_optimized_business_profile_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting.fromObject(d.ad_optimized_business_profile_setting);
                                }
                                if (d.business_name != null) {
                                    m.business_name = String(d.business_name);
                                }
                                if (d.business_profile_location != null) {
                                    m.business_profile_location = String(d.business_profile_location);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SmartCampaignSetting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @static
                             * @param {google.ads.googleads.v11.resources.SmartCampaignSetting} m SmartCampaignSetting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SmartCampaignSetting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign = "";
                                    d.phone_number = null;
                                    d.advertising_language_code = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                    d.phone_number = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber.toObject(m.phone_number, o);
                                }
                                if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                    d.business_name = m.business_name;
                                    if (o.oneofs)
                                        d.business_setting = "business_name";
                                }
                                if (m.advertising_language_code != null && m.hasOwnProperty("advertising_language_code")) {
                                    d.advertising_language_code = m.advertising_language_code;
                                }
                                if (m.final_url != null && m.hasOwnProperty("final_url")) {
                                    d.final_url = m.final_url;
                                    if (o.oneofs)
                                        d.landing_page = "final_url";
                                }
                                if (m.ad_optimized_business_profile_setting != null && m.hasOwnProperty("ad_optimized_business_profile_setting")) {
                                    d.ad_optimized_business_profile_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting.toObject(m.ad_optimized_business_profile_setting, o);
                                    if (o.oneofs)
                                        d.landing_page = "ad_optimized_business_profile_setting";
                                }
                                if (m.business_profile_location != null && m.hasOwnProperty("business_profile_location")) {
                                    d.business_profile_location = m.business_profile_location;
                                    if (o.oneofs)
                                        d.business_setting = "business_profile_location";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SmartCampaignSetting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SmartCampaignSetting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            SmartCampaignSetting.PhoneNumber = (function() {
    
                                /**
                                 * Properties of a PhoneNumber.
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                                 * @interface IPhoneNumber
                                 * @property {string|null} [phone_number] PhoneNumber phone_number
                                 * @property {string|null} [country_code] PhoneNumber country_code
                                 */
    
                                /**
                                 * Constructs a new PhoneNumber.
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                                 * @classdesc Represents a PhoneNumber.
                                 * @implements IPhoneNumber
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IPhoneNumber=} [p] Properties to set
                                 */
                                function PhoneNumber(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * PhoneNumber phone_number.
                                 * @member {string|null|undefined} phone_number
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @instance
                                 */
                                PhoneNumber.prototype.phone_number = null;
    
                                /**
                                 * PhoneNumber country_code.
                                 * @member {string|null|undefined} country_code
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @instance
                                 */
                                PhoneNumber.prototype.country_code = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * PhoneNumber _phone_number.
                                 * @member {"phone_number"|undefined} _phone_number
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @instance
                                 */
                                Object.defineProperty(PhoneNumber.prototype, "_phone_number", {
                                    get: $util.oneOfGetter($oneOfFields = ["phone_number"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * PhoneNumber _country_code.
                                 * @member {"country_code"|undefined} _country_code
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @instance
                                 */
                                Object.defineProperty(PhoneNumber.prototype, "_country_code", {
                                    get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new PhoneNumber instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IPhoneNumber=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber} PhoneNumber instance
                                 */
                                PhoneNumber.create = function create(properties) {
                                    return new PhoneNumber(properties);
                                };
    
                                /**
                                 * Encodes the specified PhoneNumber message. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IPhoneNumber} m PhoneNumber message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PhoneNumber.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.phone_number != null && Object.hasOwnProperty.call(m, "phone_number"))
                                        w.uint32(10).string(m.phone_number);
                                    if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                        w.uint32(18).string(m.country_code);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified PhoneNumber message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IPhoneNumber} message PhoneNumber message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                PhoneNumber.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a PhoneNumber message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber} PhoneNumber
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PhoneNumber.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.phone_number = r.string();
                                            break;
                                        case 2:
                                            m.country_code = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a PhoneNumber message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber} PhoneNumber
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                PhoneNumber.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a PhoneNumber message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                PhoneNumber.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                        p._phone_number = 1;
                                        if (!$util.isString(m.phone_number))
                                            return "phone_number: string expected";
                                    }
                                    if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                        p._country_code = 1;
                                        if (!$util.isString(m.country_code))
                                            return "country_code: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a PhoneNumber message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber} PhoneNumber
                                 */
                                PhoneNumber.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber();
                                    if (d.phone_number != null) {
                                        m.phone_number = String(d.phone_number);
                                    }
                                    if (d.country_code != null) {
                                        m.country_code = String(d.country_code);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a PhoneNumber message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber} m PhoneNumber
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                PhoneNumber.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.phone_number != null && m.hasOwnProperty("phone_number")) {
                                        d.phone_number = m.phone_number;
                                        if (o.oneofs)
                                            d._phone_number = "phone_number";
                                    }
                                    if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                        d.country_code = m.country_code;
                                        if (o.oneofs)
                                            d._country_code = "country_code";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this PhoneNumber to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.PhoneNumber
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                PhoneNumber.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return PhoneNumber;
                            })();
    
                            SmartCampaignSetting.AdOptimizedBusinessProfileSetting = (function() {
    
                                /**
                                 * Properties of an AdOptimizedBusinessProfileSetting.
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                                 * @interface IAdOptimizedBusinessProfileSetting
                                 * @property {boolean|null} [include_lead_form] AdOptimizedBusinessProfileSetting include_lead_form
                                 */
    
                                /**
                                 * Constructs a new AdOptimizedBusinessProfileSetting.
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting
                                 * @classdesc Represents an AdOptimizedBusinessProfileSetting.
                                 * @implements IAdOptimizedBusinessProfileSetting
                                 * @constructor
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IAdOptimizedBusinessProfileSetting=} [p] Properties to set
                                 */
                                function AdOptimizedBusinessProfileSetting(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * AdOptimizedBusinessProfileSetting include_lead_form.
                                 * @member {boolean} include_lead_form
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @instance
                                 */
                                AdOptimizedBusinessProfileSetting.prototype.include_lead_form = false;
    
                                /**
                                 * Creates a new AdOptimizedBusinessProfileSetting instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IAdOptimizedBusinessProfileSetting=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting} AdOptimizedBusinessProfileSetting instance
                                 */
                                AdOptimizedBusinessProfileSetting.create = function create(properties) {
                                    return new AdOptimizedBusinessProfileSetting(properties);
                                };
    
                                /**
                                 * Encodes the specified AdOptimizedBusinessProfileSetting message. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IAdOptimizedBusinessProfileSetting} m AdOptimizedBusinessProfileSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AdOptimizedBusinessProfileSetting.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.include_lead_form != null && Object.hasOwnProperty.call(m, "include_lead_form"))
                                        w.uint32(8).bool(m.include_lead_form);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified AdOptimizedBusinessProfileSetting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.IAdOptimizedBusinessProfileSetting} message AdOptimizedBusinessProfileSetting message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                AdOptimizedBusinessProfileSetting.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an AdOptimizedBusinessProfileSetting message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting} AdOptimizedBusinessProfileSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AdOptimizedBusinessProfileSetting.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.include_lead_form = r.bool();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an AdOptimizedBusinessProfileSetting message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting} AdOptimizedBusinessProfileSetting
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                AdOptimizedBusinessProfileSetting.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an AdOptimizedBusinessProfileSetting message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                AdOptimizedBusinessProfileSetting.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.include_lead_form != null && m.hasOwnProperty("include_lead_form")) {
                                        if (typeof m.include_lead_form !== "boolean")
                                            return "include_lead_form: boolean expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an AdOptimizedBusinessProfileSetting message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting} AdOptimizedBusinessProfileSetting
                                 */
                                AdOptimizedBusinessProfileSetting.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting();
                                    if (d.include_lead_form != null) {
                                        m.include_lead_form = Boolean(d.include_lead_form);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an AdOptimizedBusinessProfileSetting message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @static
                                 * @param {google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting} m AdOptimizedBusinessProfileSetting
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                AdOptimizedBusinessProfileSetting.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.include_lead_form = false;
                                    }
                                    if (m.include_lead_form != null && m.hasOwnProperty("include_lead_form")) {
                                        d.include_lead_form = m.include_lead_form;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this AdOptimizedBusinessProfileSetting to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.resources.SmartCampaignSetting.AdOptimizedBusinessProfileSetting
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                AdOptimizedBusinessProfileSetting.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return AdOptimizedBusinessProfileSetting;
                            })();
    
                            return SmartCampaignSetting;
                        })();
    
                        resources.ThirdPartyAppAnalyticsLink = (function() {
    
                            /**
                             * Properties of a ThirdPartyAppAnalyticsLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IThirdPartyAppAnalyticsLink
                             * @property {string|null} [resource_name] ThirdPartyAppAnalyticsLink resource_name
                             * @property {string|null} [shareable_link_id] ThirdPartyAppAnalyticsLink shareable_link_id
                             */
    
                            /**
                             * Constructs a new ThirdPartyAppAnalyticsLink.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a ThirdPartyAppAnalyticsLink.
                             * @implements IThirdPartyAppAnalyticsLink
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLink=} [p] Properties to set
                             */
                            function ThirdPartyAppAnalyticsLink(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ThirdPartyAppAnalyticsLink resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @instance
                             */
                            ThirdPartyAppAnalyticsLink.prototype.resource_name = "";
    
                            /**
                             * ThirdPartyAppAnalyticsLink shareable_link_id.
                             * @member {string|null|undefined} shareable_link_id
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @instance
                             */
                            ThirdPartyAppAnalyticsLink.prototype.shareable_link_id = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ThirdPartyAppAnalyticsLink _shareable_link_id.
                             * @member {"shareable_link_id"|undefined} _shareable_link_id
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @instance
                             */
                            Object.defineProperty(ThirdPartyAppAnalyticsLink.prototype, "_shareable_link_id", {
                                get: $util.oneOfGetter($oneOfFields = ["shareable_link_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ThirdPartyAppAnalyticsLink instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLink=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink} ThirdPartyAppAnalyticsLink instance
                             */
                            ThirdPartyAppAnalyticsLink.create = function create(properties) {
                                return new ThirdPartyAppAnalyticsLink(properties);
                            };
    
                            /**
                             * Encodes the specified ThirdPartyAppAnalyticsLink message. Does not implicitly {@link google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLink} m ThirdPartyAppAnalyticsLink message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ThirdPartyAppAnalyticsLink.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.shareable_link_id != null && Object.hasOwnProperty.call(m, "shareable_link_id"))
                                    w.uint32(26).string(m.shareable_link_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ThirdPartyAppAnalyticsLink message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLink} message ThirdPartyAppAnalyticsLink message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ThirdPartyAppAnalyticsLink.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ThirdPartyAppAnalyticsLink message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink} ThirdPartyAppAnalyticsLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ThirdPartyAppAnalyticsLink.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.shareable_link_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ThirdPartyAppAnalyticsLink message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink} ThirdPartyAppAnalyticsLink
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ThirdPartyAppAnalyticsLink.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ThirdPartyAppAnalyticsLink message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ThirdPartyAppAnalyticsLink.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.shareable_link_id != null && m.hasOwnProperty("shareable_link_id")) {
                                    p._shareable_link_id = 1;
                                    if (!$util.isString(m.shareable_link_id))
                                        return "shareable_link_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ThirdPartyAppAnalyticsLink message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink} ThirdPartyAppAnalyticsLink
                             */
                            ThirdPartyAppAnalyticsLink.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.shareable_link_id != null) {
                                    m.shareable_link_id = String(d.shareable_link_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ThirdPartyAppAnalyticsLink message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @static
                             * @param {google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink} m ThirdPartyAppAnalyticsLink
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ThirdPartyAppAnalyticsLink.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.shareable_link_id != null && m.hasOwnProperty("shareable_link_id")) {
                                    d.shareable_link_id = m.shareable_link_id;
                                    if (o.oneofs)
                                        d._shareable_link_id = "shareable_link_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ThirdPartyAppAnalyticsLink to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ThirdPartyAppAnalyticsLink.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ThirdPartyAppAnalyticsLink;
                        })();
    
                        resources.TopicConstant = (function() {
    
                            /**
                             * Properties of a TopicConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ITopicConstant
                             * @property {string|null} [resource_name] TopicConstant resource_name
                             * @property {number|null} [id] TopicConstant id
                             * @property {string|null} [topic_constant_parent] TopicConstant topic_constant_parent
                             * @property {Array.<string>|null} [path] TopicConstant path
                             */
    
                            /**
                             * Constructs a new TopicConstant.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a TopicConstant.
                             * @implements ITopicConstant
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ITopicConstant=} [p] Properties to set
                             */
                            function TopicConstant(p) {
                                this.path = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TopicConstant resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @instance
                             */
                            TopicConstant.prototype.resource_name = "";
    
                            /**
                             * TopicConstant id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @instance
                             */
                            TopicConstant.prototype.id = null;
    
                            /**
                             * TopicConstant topic_constant_parent.
                             * @member {string|null|undefined} topic_constant_parent
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @instance
                             */
                            TopicConstant.prototype.topic_constant_parent = null;
    
                            /**
                             * TopicConstant path.
                             * @member {Array.<string>} path
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @instance
                             */
                            TopicConstant.prototype.path = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * TopicConstant _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @instance
                             */
                            Object.defineProperty(TopicConstant.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * TopicConstant _topic_constant_parent.
                             * @member {"topic_constant_parent"|undefined} _topic_constant_parent
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @instance
                             */
                            Object.defineProperty(TopicConstant.prototype, "_topic_constant_parent", {
                                get: $util.oneOfGetter($oneOfFields = ["topic_constant_parent"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new TopicConstant instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ITopicConstant=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.TopicConstant} TopicConstant instance
                             */
                            TopicConstant.create = function create(properties) {
                                return new TopicConstant(properties);
                            };
    
                            /**
                             * Encodes the specified TopicConstant message. Does not implicitly {@link google.ads.googleads.v11.resources.TopicConstant.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ITopicConstant} m TopicConstant message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TopicConstant.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(40).int64(m.id);
                                if (m.topic_constant_parent != null && Object.hasOwnProperty.call(m, "topic_constant_parent"))
                                    w.uint32(50).string(m.topic_constant_parent);
                                if (m.path != null && m.path.length) {
                                    for (var i = 0; i < m.path.length; ++i)
                                        w.uint32(58).string(m.path[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified TopicConstant message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.TopicConstant.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.ITopicConstant} message TopicConstant message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TopicConstant.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TopicConstant message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.TopicConstant} TopicConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TopicConstant.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.TopicConstant();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 5:
                                        m.id = r.int64();
                                        break;
                                    case 6:
                                        m.topic_constant_parent = r.string();
                                        break;
                                    case 7:
                                        if (!(m.path && m.path.length))
                                            m.path = [];
                                        m.path.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TopicConstant message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.TopicConstant} TopicConstant
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TopicConstant.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TopicConstant message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TopicConstant.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.topic_constant_parent != null && m.hasOwnProperty("topic_constant_parent")) {
                                    p._topic_constant_parent = 1;
                                    if (!$util.isString(m.topic_constant_parent))
                                        return "topic_constant_parent: string expected";
                                }
                                if (m.path != null && m.hasOwnProperty("path")) {
                                    if (!Array.isArray(m.path))
                                        return "path: array expected";
                                    for (var i = 0; i < m.path.length; ++i) {
                                        if (!$util.isString(m.path[i]))
                                            return "path: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TopicConstant message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.TopicConstant} TopicConstant
                             */
                            TopicConstant.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.TopicConstant)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.TopicConstant();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.topic_constant_parent != null) {
                                    m.topic_constant_parent = String(d.topic_constant_parent);
                                }
                                if (d.path) {
                                    if (!Array.isArray(d.path))
                                        throw TypeError(".google.ads.googleads.v11.resources.TopicConstant.path: array expected");
                                    m.path = [];
                                    for (var i = 0; i < d.path.length; ++i) {
                                        m.path[i] = String(d.path[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TopicConstant message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @static
                             * @param {google.ads.googleads.v11.resources.TopicConstant} m TopicConstant
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TopicConstant.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.path = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.topic_constant_parent != null && m.hasOwnProperty("topic_constant_parent")) {
                                    d.topic_constant_parent = m.topic_constant_parent;
                                    if (o.oneofs)
                                        d._topic_constant_parent = "topic_constant_parent";
                                }
                                if (m.path && m.path.length) {
                                    d.path = [];
                                    for (var j = 0; j < m.path.length; ++j) {
                                        d.path[j] = m.path[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TopicConstant to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.TopicConstant
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TopicConstant.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TopicConstant;
                        })();
    
                        resources.TopicView = (function() {
    
                            /**
                             * Properties of a TopicView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface ITopicView
                             * @property {string|null} [resource_name] TopicView resource_name
                             */
    
                            /**
                             * Constructs a new TopicView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a TopicView.
                             * @implements ITopicView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.ITopicView=} [p] Properties to set
                             */
                            function TopicView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * TopicView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @instance
                             */
                            TopicView.prototype.resource_name = "";
    
                            /**
                             * Creates a new TopicView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ITopicView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.TopicView} TopicView instance
                             */
                            TopicView.create = function create(properties) {
                                return new TopicView(properties);
                            };
    
                            /**
                             * Encodes the specified TopicView message. Does not implicitly {@link google.ads.googleads.v11.resources.TopicView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ITopicView} m TopicView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TopicView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified TopicView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.TopicView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.ITopicView} message TopicView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TopicView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TopicView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.TopicView} TopicView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TopicView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.TopicView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a TopicView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.TopicView} TopicView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TopicView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TopicView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TopicView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a TopicView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.TopicView} TopicView
                             */
                            TopicView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.TopicView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.TopicView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a TopicView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @static
                             * @param {google.ads.googleads.v11.resources.TopicView} m TopicView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TopicView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this TopicView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.TopicView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TopicView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return TopicView;
                        })();
    
                        resources.UserInterest = (function() {
    
                            /**
                             * Properties of a UserInterest.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IUserInterest
                             * @property {string|null} [resource_name] UserInterest resource_name
                             * @property {google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType|null} [taxonomy_type] UserInterest taxonomy_type
                             * @property {number|null} [user_interest_id] UserInterest user_interest_id
                             * @property {string|null} [name] UserInterest name
                             * @property {string|null} [user_interest_parent] UserInterest user_interest_parent
                             * @property {boolean|null} [launched_to_all] UserInterest launched_to_all
                             * @property {Array.<google.ads.googleads.v11.common.ICriterionCategoryAvailability>|null} [availabilities] UserInterest availabilities
                             */
    
                            /**
                             * Constructs a new UserInterest.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a UserInterest.
                             * @implements IUserInterest
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IUserInterest=} [p] Properties to set
                             */
                            function UserInterest(p) {
                                this.availabilities = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserInterest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            UserInterest.prototype.resource_name = "";
    
                            /**
                             * UserInterest taxonomy_type.
                             * @member {google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType} taxonomy_type
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            UserInterest.prototype.taxonomy_type = 0;
    
                            /**
                             * UserInterest user_interest_id.
                             * @member {number|null|undefined} user_interest_id
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            UserInterest.prototype.user_interest_id = null;
    
                            /**
                             * UserInterest name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            UserInterest.prototype.name = null;
    
                            /**
                             * UserInterest user_interest_parent.
                             * @member {string|null|undefined} user_interest_parent
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            UserInterest.prototype.user_interest_parent = null;
    
                            /**
                             * UserInterest launched_to_all.
                             * @member {boolean|null|undefined} launched_to_all
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            UserInterest.prototype.launched_to_all = null;
    
                            /**
                             * UserInterest availabilities.
                             * @member {Array.<google.ads.googleads.v11.common.ICriterionCategoryAvailability>} availabilities
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            UserInterest.prototype.availabilities = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserInterest _user_interest_id.
                             * @member {"user_interest_id"|undefined} _user_interest_id
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            Object.defineProperty(UserInterest.prototype, "_user_interest_id", {
                                get: $util.oneOfGetter($oneOfFields = ["user_interest_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserInterest _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            Object.defineProperty(UserInterest.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserInterest _user_interest_parent.
                             * @member {"user_interest_parent"|undefined} _user_interest_parent
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            Object.defineProperty(UserInterest.prototype, "_user_interest_parent", {
                                get: $util.oneOfGetter($oneOfFields = ["user_interest_parent"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserInterest _launched_to_all.
                             * @member {"launched_to_all"|undefined} _launched_to_all
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             */
                            Object.defineProperty(UserInterest.prototype, "_launched_to_all", {
                                get: $util.oneOfGetter($oneOfFields = ["launched_to_all"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserInterest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserInterest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.UserInterest} UserInterest instance
                             */
                            UserInterest.create = function create(properties) {
                                return new UserInterest(properties);
                            };
    
                            /**
                             * Encodes the specified UserInterest message. Does not implicitly {@link google.ads.googleads.v11.resources.UserInterest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserInterest} m UserInterest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.taxonomy_type != null && Object.hasOwnProperty.call(m, "taxonomy_type"))
                                    w.uint32(16).int32(m.taxonomy_type);
                                if (m.availabilities != null && m.availabilities.length) {
                                    for (var i = 0; i < m.availabilities.length; ++i)
                                        $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.encode(m.availabilities[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.user_interest_id != null && Object.hasOwnProperty.call(m, "user_interest_id"))
                                    w.uint32(64).int64(m.user_interest_id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(74).string(m.name);
                                if (m.user_interest_parent != null && Object.hasOwnProperty.call(m, "user_interest_parent"))
                                    w.uint32(82).string(m.user_interest_parent);
                                if (m.launched_to_all != null && Object.hasOwnProperty.call(m, "launched_to_all"))
                                    w.uint32(88).bool(m.launched_to_all);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserInterest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.UserInterest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserInterest} message UserInterest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserInterest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserInterest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.UserInterest} UserInterest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.UserInterest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.taxonomy_type = r.int32();
                                        break;
                                    case 8:
                                        m.user_interest_id = r.int64();
                                        break;
                                    case 9:
                                        m.name = r.string();
                                        break;
                                    case 10:
                                        m.user_interest_parent = r.string();
                                        break;
                                    case 11:
                                        m.launched_to_all = r.bool();
                                        break;
                                    case 7:
                                        if (!(m.availabilities && m.availabilities.length))
                                            m.availabilities = [];
                                        m.availabilities.push($root.google.ads.googleads.v11.common.CriterionCategoryAvailability.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserInterest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.UserInterest} UserInterest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserInterest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserInterest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserInterest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.taxonomy_type != null && m.hasOwnProperty("taxonomy_type")) {
                                    switch (m.taxonomy_type) {
                                    default:
                                        return "taxonomy_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.user_interest_id != null && m.hasOwnProperty("user_interest_id")) {
                                    p._user_interest_id = 1;
                                    if (!$util.isInteger(m.user_interest_id) && !(m.user_interest_id && $util.isInteger(m.user_interest_id.low) && $util.isInteger(m.user_interest_id.high)))
                                        return "user_interest_id: integer|Long expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.user_interest_parent != null && m.hasOwnProperty("user_interest_parent")) {
                                    p._user_interest_parent = 1;
                                    if (!$util.isString(m.user_interest_parent))
                                        return "user_interest_parent: string expected";
                                }
                                if (m.launched_to_all != null && m.hasOwnProperty("launched_to_all")) {
                                    p._launched_to_all = 1;
                                    if (typeof m.launched_to_all !== "boolean")
                                        return "launched_to_all: boolean expected";
                                }
                                if (m.availabilities != null && m.hasOwnProperty("availabilities")) {
                                    if (!Array.isArray(m.availabilities))
                                        return "availabilities: array expected";
                                    for (var i = 0; i < m.availabilities.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.verify(m.availabilities[i]);
                                            if (e)
                                                return "availabilities." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserInterest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.UserInterest} UserInterest
                             */
                            UserInterest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.UserInterest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.UserInterest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                switch (d.taxonomy_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.taxonomy_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.taxonomy_type = 1;
                                    break;
                                case "AFFINITY":
                                case 2:
                                    m.taxonomy_type = 2;
                                    break;
                                case "IN_MARKET":
                                case 3:
                                    m.taxonomy_type = 3;
                                    break;
                                case "MOBILE_APP_INSTALL_USER":
                                case 4:
                                    m.taxonomy_type = 4;
                                    break;
                                case "VERTICAL_GEO":
                                case 5:
                                    m.taxonomy_type = 5;
                                    break;
                                case "NEW_SMART_PHONE_USER":
                                case 6:
                                    m.taxonomy_type = 6;
                                    break;
                                }
                                if (d.user_interest_id != null) {
                                    if ($util.Long)
                                        (m.user_interest_id = $util.Long.fromValue(d.user_interest_id)).unsigned = false;
                                    else if (typeof d.user_interest_id === "string")
                                        m.user_interest_id = parseInt(d.user_interest_id, 10);
                                    else if (typeof d.user_interest_id === "number")
                                        m.user_interest_id = d.user_interest_id;
                                    else if (typeof d.user_interest_id === "object")
                                        m.user_interest_id = new $util.LongBits(d.user_interest_id.low >>> 0, d.user_interest_id.high >>> 0).toNumber();
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.user_interest_parent != null) {
                                    m.user_interest_parent = String(d.user_interest_parent);
                                }
                                if (d.launched_to_all != null) {
                                    m.launched_to_all = Boolean(d.launched_to_all);
                                }
                                if (d.availabilities) {
                                    if (!Array.isArray(d.availabilities))
                                        throw TypeError(".google.ads.googleads.v11.resources.UserInterest.availabilities: array expected");
                                    m.availabilities = [];
                                    for (var i = 0; i < d.availabilities.length; ++i) {
                                        if (typeof d.availabilities[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.resources.UserInterest.availabilities: object expected");
                                        m.availabilities[i] = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.fromObject(d.availabilities[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserInterest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @static
                             * @param {google.ads.googleads.v11.resources.UserInterest} m UserInterest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserInterest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.availabilities = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.taxonomy_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.taxonomy_type != null && m.hasOwnProperty("taxonomy_type")) {
                                    d.taxonomy_type = o.enums === String ? $root.google.ads.googleads.v11.enums.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType[m.taxonomy_type] : m.taxonomy_type;
                                }
                                if (m.availabilities && m.availabilities.length) {
                                    d.availabilities = [];
                                    for (var j = 0; j < m.availabilities.length; ++j) {
                                        d.availabilities[j] = $root.google.ads.googleads.v11.common.CriterionCategoryAvailability.toObject(m.availabilities[j], o);
                                    }
                                }
                                if (m.user_interest_id != null && m.hasOwnProperty("user_interest_id")) {
                                    if (typeof m.user_interest_id === "number")
                                        d.user_interest_id = o.longs === String ? String(m.user_interest_id) : m.user_interest_id;
                                    else
                                        d.user_interest_id = o.longs === String ? $util.Long.prototype.toString.call(m.user_interest_id) : o.longs === Number ? new $util.LongBits(m.user_interest_id.low >>> 0, m.user_interest_id.high >>> 0).toNumber() : m.user_interest_id;
                                    if (o.oneofs)
                                        d._user_interest_id = "user_interest_id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.user_interest_parent != null && m.hasOwnProperty("user_interest_parent")) {
                                    d.user_interest_parent = m.user_interest_parent;
                                    if (o.oneofs)
                                        d._user_interest_parent = "user_interest_parent";
                                }
                                if (m.launched_to_all != null && m.hasOwnProperty("launched_to_all")) {
                                    d.launched_to_all = m.launched_to_all;
                                    if (o.oneofs)
                                        d._launched_to_all = "launched_to_all";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserInterest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.UserInterest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserInterest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserInterest;
                        })();
    
                        resources.UserList = (function() {
    
                            /**
                             * Properties of a UserList.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IUserList
                             * @property {string|null} [resource_name] UserList resource_name
                             * @property {number|null} [id] UserList id
                             * @property {boolean|null} [read_only] UserList read_only
                             * @property {string|null} [name] UserList name
                             * @property {string|null} [description] UserList description
                             * @property {google.ads.googleads.v11.enums.UserListMembershipStatusEnum.UserListMembershipStatus|null} [membership_status] UserList membership_status
                             * @property {string|null} [integration_code] UserList integration_code
                             * @property {number|null} [membership_life_span] UserList membership_life_span
                             * @property {number|null} [size_for_display] UserList size_for_display
                             * @property {google.ads.googleads.v11.enums.UserListSizeRangeEnum.UserListSizeRange|null} [size_range_for_display] UserList size_range_for_display
                             * @property {number|null} [size_for_search] UserList size_for_search
                             * @property {google.ads.googleads.v11.enums.UserListSizeRangeEnum.UserListSizeRange|null} [size_range_for_search] UserList size_range_for_search
                             * @property {google.ads.googleads.v11.enums.UserListTypeEnum.UserListType|null} [type] UserList type
                             * @property {google.ads.googleads.v11.enums.UserListClosingReasonEnum.UserListClosingReason|null} [closing_reason] UserList closing_reason
                             * @property {google.ads.googleads.v11.enums.AccessReasonEnum.AccessReason|null} [access_reason] UserList access_reason
                             * @property {google.ads.googleads.v11.enums.UserListAccessStatusEnum.UserListAccessStatus|null} [account_user_list_status] UserList account_user_list_status
                             * @property {boolean|null} [eligible_for_search] UserList eligible_for_search
                             * @property {boolean|null} [eligible_for_display] UserList eligible_for_display
                             * @property {number|null} [match_rate_percentage] UserList match_rate_percentage
                             * @property {google.ads.googleads.v11.common.ICrmBasedUserListInfo|null} [crm_based_user_list] UserList crm_based_user_list
                             * @property {google.ads.googleads.v11.common.ISimilarUserListInfo|null} [similar_user_list] UserList similar_user_list
                             * @property {google.ads.googleads.v11.common.IRuleBasedUserListInfo|null} [rule_based_user_list] UserList rule_based_user_list
                             * @property {google.ads.googleads.v11.common.ILogicalUserListInfo|null} [logical_user_list] UserList logical_user_list
                             * @property {google.ads.googleads.v11.common.IBasicUserListInfo|null} [basic_user_list] UserList basic_user_list
                             */
    
                            /**
                             * Constructs a new UserList.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a UserList.
                             * @implements IUserList
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IUserList=} [p] Properties to set
                             */
                            function UserList(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserList resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.resource_name = "";
    
                            /**
                             * UserList id.
                             * @member {number|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.id = null;
    
                            /**
                             * UserList read_only.
                             * @member {boolean|null|undefined} read_only
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.read_only = null;
    
                            /**
                             * UserList name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.name = null;
    
                            /**
                             * UserList description.
                             * @member {string|null|undefined} description
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.description = null;
    
                            /**
                             * UserList membership_status.
                             * @member {google.ads.googleads.v11.enums.UserListMembershipStatusEnum.UserListMembershipStatus} membership_status
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.membership_status = 0;
    
                            /**
                             * UserList integration_code.
                             * @member {string|null|undefined} integration_code
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.integration_code = null;
    
                            /**
                             * UserList membership_life_span.
                             * @member {number|null|undefined} membership_life_span
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.membership_life_span = null;
    
                            /**
                             * UserList size_for_display.
                             * @member {number|null|undefined} size_for_display
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.size_for_display = null;
    
                            /**
                             * UserList size_range_for_display.
                             * @member {google.ads.googleads.v11.enums.UserListSizeRangeEnum.UserListSizeRange} size_range_for_display
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.size_range_for_display = 0;
    
                            /**
                             * UserList size_for_search.
                             * @member {number|null|undefined} size_for_search
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.size_for_search = null;
    
                            /**
                             * UserList size_range_for_search.
                             * @member {google.ads.googleads.v11.enums.UserListSizeRangeEnum.UserListSizeRange} size_range_for_search
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.size_range_for_search = 0;
    
                            /**
                             * UserList type.
                             * @member {google.ads.googleads.v11.enums.UserListTypeEnum.UserListType} type
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.type = 0;
    
                            /**
                             * UserList closing_reason.
                             * @member {google.ads.googleads.v11.enums.UserListClosingReasonEnum.UserListClosingReason} closing_reason
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.closing_reason = 0;
    
                            /**
                             * UserList access_reason.
                             * @member {google.ads.googleads.v11.enums.AccessReasonEnum.AccessReason} access_reason
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.access_reason = 0;
    
                            /**
                             * UserList account_user_list_status.
                             * @member {google.ads.googleads.v11.enums.UserListAccessStatusEnum.UserListAccessStatus} account_user_list_status
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.account_user_list_status = 0;
    
                            /**
                             * UserList eligible_for_search.
                             * @member {boolean|null|undefined} eligible_for_search
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.eligible_for_search = null;
    
                            /**
                             * UserList eligible_for_display.
                             * @member {boolean|null|undefined} eligible_for_display
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.eligible_for_display = null;
    
                            /**
                             * UserList match_rate_percentage.
                             * @member {number|null|undefined} match_rate_percentage
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.match_rate_percentage = null;
    
                            /**
                             * UserList crm_based_user_list.
                             * @member {google.ads.googleads.v11.common.ICrmBasedUserListInfo|null|undefined} crm_based_user_list
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.crm_based_user_list = null;
    
                            /**
                             * UserList similar_user_list.
                             * @member {google.ads.googleads.v11.common.ISimilarUserListInfo|null|undefined} similar_user_list
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.similar_user_list = null;
    
                            /**
                             * UserList rule_based_user_list.
                             * @member {google.ads.googleads.v11.common.IRuleBasedUserListInfo|null|undefined} rule_based_user_list
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.rule_based_user_list = null;
    
                            /**
                             * UserList logical_user_list.
                             * @member {google.ads.googleads.v11.common.ILogicalUserListInfo|null|undefined} logical_user_list
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.logical_user_list = null;
    
                            /**
                             * UserList basic_user_list.
                             * @member {google.ads.googleads.v11.common.IBasicUserListInfo|null|undefined} basic_user_list
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            UserList.prototype.basic_user_list = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserList _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _read_only.
                             * @member {"read_only"|undefined} _read_only
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_read_only", {
                                get: $util.oneOfGetter($oneOfFields = ["read_only"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _description.
                             * @member {"description"|undefined} _description
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_description", {
                                get: $util.oneOfGetter($oneOfFields = ["description"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _integration_code.
                             * @member {"integration_code"|undefined} _integration_code
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_integration_code", {
                                get: $util.oneOfGetter($oneOfFields = ["integration_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _membership_life_span.
                             * @member {"membership_life_span"|undefined} _membership_life_span
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_membership_life_span", {
                                get: $util.oneOfGetter($oneOfFields = ["membership_life_span"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _size_for_display.
                             * @member {"size_for_display"|undefined} _size_for_display
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_size_for_display", {
                                get: $util.oneOfGetter($oneOfFields = ["size_for_display"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _size_for_search.
                             * @member {"size_for_search"|undefined} _size_for_search
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_size_for_search", {
                                get: $util.oneOfGetter($oneOfFields = ["size_for_search"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _eligible_for_search.
                             * @member {"eligible_for_search"|undefined} _eligible_for_search
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_eligible_for_search", {
                                get: $util.oneOfGetter($oneOfFields = ["eligible_for_search"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _eligible_for_display.
                             * @member {"eligible_for_display"|undefined} _eligible_for_display
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_eligible_for_display", {
                                get: $util.oneOfGetter($oneOfFields = ["eligible_for_display"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList _match_rate_percentage.
                             * @member {"match_rate_percentage"|undefined} _match_rate_percentage
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "_match_rate_percentage", {
                                get: $util.oneOfGetter($oneOfFields = ["match_rate_percentage"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserList user_list.
                             * @member {"crm_based_user_list"|"similar_user_list"|"rule_based_user_list"|"logical_user_list"|"basic_user_list"|undefined} user_list
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             */
                            Object.defineProperty(UserList.prototype, "user_list", {
                                get: $util.oneOfGetter($oneOfFields = ["crm_based_user_list", "similar_user_list", "rule_based_user_list", "logical_user_list", "basic_user_list"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserList instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserList=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.UserList} UserList instance
                             */
                            UserList.create = function create(properties) {
                                return new UserList(properties);
                            };
    
                            /**
                             * Encodes the specified UserList message. Does not implicitly {@link google.ads.googleads.v11.resources.UserList.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserList} m UserList message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserList.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.membership_status != null && Object.hasOwnProperty.call(m, "membership_status"))
                                    w.uint32(48).int32(m.membership_status);
                                if (m.size_range_for_display != null && Object.hasOwnProperty.call(m, "size_range_for_display"))
                                    w.uint32(80).int32(m.size_range_for_display);
                                if (m.size_range_for_search != null && Object.hasOwnProperty.call(m, "size_range_for_search"))
                                    w.uint32(96).int32(m.size_range_for_search);
                                if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                                    w.uint32(104).int32(m.type);
                                if (m.closing_reason != null && Object.hasOwnProperty.call(m, "closing_reason"))
                                    w.uint32(112).int32(m.closing_reason);
                                if (m.access_reason != null && Object.hasOwnProperty.call(m, "access_reason"))
                                    w.uint32(120).int32(m.access_reason);
                                if (m.account_user_list_status != null && Object.hasOwnProperty.call(m, "account_user_list_status"))
                                    w.uint32(128).int32(m.account_user_list_status);
                                if (m.crm_based_user_list != null && Object.hasOwnProperty.call(m, "crm_based_user_list"))
                                    $root.google.ads.googleads.v11.common.CrmBasedUserListInfo.encode(m.crm_based_user_list, w.uint32(154).fork()).ldelim();
                                if (m.similar_user_list != null && Object.hasOwnProperty.call(m, "similar_user_list"))
                                    $root.google.ads.googleads.v11.common.SimilarUserListInfo.encode(m.similar_user_list, w.uint32(162).fork()).ldelim();
                                if (m.rule_based_user_list != null && Object.hasOwnProperty.call(m, "rule_based_user_list"))
                                    $root.google.ads.googleads.v11.common.RuleBasedUserListInfo.encode(m.rule_based_user_list, w.uint32(170).fork()).ldelim();
                                if (m.logical_user_list != null && Object.hasOwnProperty.call(m, "logical_user_list"))
                                    $root.google.ads.googleads.v11.common.LogicalUserListInfo.encode(m.logical_user_list, w.uint32(178).fork()).ldelim();
                                if (m.basic_user_list != null && Object.hasOwnProperty.call(m, "basic_user_list"))
                                    $root.google.ads.googleads.v11.common.BasicUserListInfo.encode(m.basic_user_list, w.uint32(186).fork()).ldelim();
                                if (m.match_rate_percentage != null && Object.hasOwnProperty.call(m, "match_rate_percentage"))
                                    w.uint32(192).int32(m.match_rate_percentage);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(200).int64(m.id);
                                if (m.read_only != null && Object.hasOwnProperty.call(m, "read_only"))
                                    w.uint32(208).bool(m.read_only);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(218).string(m.name);
                                if (m.description != null && Object.hasOwnProperty.call(m, "description"))
                                    w.uint32(226).string(m.description);
                                if (m.integration_code != null && Object.hasOwnProperty.call(m, "integration_code"))
                                    w.uint32(234).string(m.integration_code);
                                if (m.membership_life_span != null && Object.hasOwnProperty.call(m, "membership_life_span"))
                                    w.uint32(240).int64(m.membership_life_span);
                                if (m.size_for_display != null && Object.hasOwnProperty.call(m, "size_for_display"))
                                    w.uint32(248).int64(m.size_for_display);
                                if (m.size_for_search != null && Object.hasOwnProperty.call(m, "size_for_search"))
                                    w.uint32(256).int64(m.size_for_search);
                                if (m.eligible_for_search != null && Object.hasOwnProperty.call(m, "eligible_for_search"))
                                    w.uint32(264).bool(m.eligible_for_search);
                                if (m.eligible_for_display != null && Object.hasOwnProperty.call(m, "eligible_for_display"))
                                    w.uint32(272).bool(m.eligible_for_display);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.UserList.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserList} message UserList message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserList.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserList message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.UserList} UserList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserList.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.UserList();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 25:
                                        m.id = r.int64();
                                        break;
                                    case 26:
                                        m.read_only = r.bool();
                                        break;
                                    case 27:
                                        m.name = r.string();
                                        break;
                                    case 28:
                                        m.description = r.string();
                                        break;
                                    case 6:
                                        m.membership_status = r.int32();
                                        break;
                                    case 29:
                                        m.integration_code = r.string();
                                        break;
                                    case 30:
                                        m.membership_life_span = r.int64();
                                        break;
                                    case 31:
                                        m.size_for_display = r.int64();
                                        break;
                                    case 10:
                                        m.size_range_for_display = r.int32();
                                        break;
                                    case 32:
                                        m.size_for_search = r.int64();
                                        break;
                                    case 12:
                                        m.size_range_for_search = r.int32();
                                        break;
                                    case 13:
                                        m.type = r.int32();
                                        break;
                                    case 14:
                                        m.closing_reason = r.int32();
                                        break;
                                    case 15:
                                        m.access_reason = r.int32();
                                        break;
                                    case 16:
                                        m.account_user_list_status = r.int32();
                                        break;
                                    case 33:
                                        m.eligible_for_search = r.bool();
                                        break;
                                    case 34:
                                        m.eligible_for_display = r.bool();
                                        break;
                                    case 24:
                                        m.match_rate_percentage = r.int32();
                                        break;
                                    case 19:
                                        m.crm_based_user_list = $root.google.ads.googleads.v11.common.CrmBasedUserListInfo.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.similar_user_list = $root.google.ads.googleads.v11.common.SimilarUserListInfo.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.rule_based_user_list = $root.google.ads.googleads.v11.common.RuleBasedUserListInfo.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.logical_user_list = $root.google.ads.googleads.v11.common.LogicalUserListInfo.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.basic_user_list = $root.google.ads.googleads.v11.common.BasicUserListInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserList message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.UserList} UserList
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserList.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserList message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserList.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isInteger(m.id) && !(m.id && $util.isInteger(m.id.low) && $util.isInteger(m.id.high)))
                                        return "id: integer|Long expected";
                                }
                                if (m.read_only != null && m.hasOwnProperty("read_only")) {
                                    p._read_only = 1;
                                    if (typeof m.read_only !== "boolean")
                                        return "read_only: boolean expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    p._description = 1;
                                    if (!$util.isString(m.description))
                                        return "description: string expected";
                                }
                                if (m.membership_status != null && m.hasOwnProperty("membership_status")) {
                                    switch (m.membership_status) {
                                    default:
                                        return "membership_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.integration_code != null && m.hasOwnProperty("integration_code")) {
                                    p._integration_code = 1;
                                    if (!$util.isString(m.integration_code))
                                        return "integration_code: string expected";
                                }
                                if (m.membership_life_span != null && m.hasOwnProperty("membership_life_span")) {
                                    p._membership_life_span = 1;
                                    if (!$util.isInteger(m.membership_life_span) && !(m.membership_life_span && $util.isInteger(m.membership_life_span.low) && $util.isInteger(m.membership_life_span.high)))
                                        return "membership_life_span: integer|Long expected";
                                }
                                if (m.size_for_display != null && m.hasOwnProperty("size_for_display")) {
                                    p._size_for_display = 1;
                                    if (!$util.isInteger(m.size_for_display) && !(m.size_for_display && $util.isInteger(m.size_for_display.low) && $util.isInteger(m.size_for_display.high)))
                                        return "size_for_display: integer|Long expected";
                                }
                                if (m.size_range_for_display != null && m.hasOwnProperty("size_range_for_display")) {
                                    switch (m.size_range_for_display) {
                                    default:
                                        return "size_range_for_display: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                        break;
                                    }
                                }
                                if (m.size_for_search != null && m.hasOwnProperty("size_for_search")) {
                                    p._size_for_search = 1;
                                    if (!$util.isInteger(m.size_for_search) && !(m.size_for_search && $util.isInteger(m.size_for_search.low) && $util.isInteger(m.size_for_search.high)))
                                        return "size_for_search: integer|Long expected";
                                }
                                if (m.size_range_for_search != null && m.hasOwnProperty("size_range_for_search")) {
                                    switch (m.size_range_for_search) {
                                    default:
                                        return "size_range_for_search: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                        break;
                                    }
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    switch (m.type) {
                                    default:
                                        return "type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                        break;
                                    }
                                }
                                if (m.closing_reason != null && m.hasOwnProperty("closing_reason")) {
                                    switch (m.closing_reason) {
                                    default:
                                        return "closing_reason: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.access_reason != null && m.hasOwnProperty("access_reason")) {
                                    switch (m.access_reason) {
                                    default:
                                        return "access_reason: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                        break;
                                    }
                                }
                                if (m.account_user_list_status != null && m.hasOwnProperty("account_user_list_status")) {
                                    switch (m.account_user_list_status) {
                                    default:
                                        return "account_user_list_status: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.eligible_for_search != null && m.hasOwnProperty("eligible_for_search")) {
                                    p._eligible_for_search = 1;
                                    if (typeof m.eligible_for_search !== "boolean")
                                        return "eligible_for_search: boolean expected";
                                }
                                if (m.eligible_for_display != null && m.hasOwnProperty("eligible_for_display")) {
                                    p._eligible_for_display = 1;
                                    if (typeof m.eligible_for_display !== "boolean")
                                        return "eligible_for_display: boolean expected";
                                }
                                if (m.match_rate_percentage != null && m.hasOwnProperty("match_rate_percentage")) {
                                    p._match_rate_percentage = 1;
                                    if (!$util.isInteger(m.match_rate_percentage))
                                        return "match_rate_percentage: integer expected";
                                }
                                if (m.crm_based_user_list != null && m.hasOwnProperty("crm_based_user_list")) {
                                    p.user_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CrmBasedUserListInfo.verify(m.crm_based_user_list);
                                        if (e)
                                            return "crm_based_user_list." + e;
                                    }
                                }
                                if (m.similar_user_list != null && m.hasOwnProperty("similar_user_list")) {
                                    if (p.user_list === 1)
                                        return "user_list: multiple values";
                                    p.user_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.SimilarUserListInfo.verify(m.similar_user_list);
                                        if (e)
                                            return "similar_user_list." + e;
                                    }
                                }
                                if (m.rule_based_user_list != null && m.hasOwnProperty("rule_based_user_list")) {
                                    if (p.user_list === 1)
                                        return "user_list: multiple values";
                                    p.user_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.RuleBasedUserListInfo.verify(m.rule_based_user_list);
                                        if (e)
                                            return "rule_based_user_list." + e;
                                    }
                                }
                                if (m.logical_user_list != null && m.hasOwnProperty("logical_user_list")) {
                                    if (p.user_list === 1)
                                        return "user_list: multiple values";
                                    p.user_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LogicalUserListInfo.verify(m.logical_user_list);
                                        if (e)
                                            return "logical_user_list." + e;
                                    }
                                }
                                if (m.basic_user_list != null && m.hasOwnProperty("basic_user_list")) {
                                    if (p.user_list === 1)
                                        return "user_list: multiple values";
                                    p.user_list = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.BasicUserListInfo.verify(m.basic_user_list);
                                        if (e)
                                            return "basic_user_list." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserList message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.UserList} UserList
                             */
                            UserList.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.UserList)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.UserList();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    if ($util.Long)
                                        (m.id = $util.Long.fromValue(d.id)).unsigned = false;
                                    else if (typeof d.id === "string")
                                        m.id = parseInt(d.id, 10);
                                    else if (typeof d.id === "number")
                                        m.id = d.id;
                                    else if (typeof d.id === "object")
                                        m.id = new $util.LongBits(d.id.low >>> 0, d.id.high >>> 0).toNumber();
                                }
                                if (d.read_only != null) {
                                    m.read_only = Boolean(d.read_only);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.description != null) {
                                    m.description = String(d.description);
                                }
                                switch (d.membership_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.membership_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.membership_status = 1;
                                    break;
                                case "OPEN":
                                case 2:
                                    m.membership_status = 2;
                                    break;
                                case "CLOSED":
                                case 3:
                                    m.membership_status = 3;
                                    break;
                                }
                                if (d.integration_code != null) {
                                    m.integration_code = String(d.integration_code);
                                }
                                if (d.membership_life_span != null) {
                                    if ($util.Long)
                                        (m.membership_life_span = $util.Long.fromValue(d.membership_life_span)).unsigned = false;
                                    else if (typeof d.membership_life_span === "string")
                                        m.membership_life_span = parseInt(d.membership_life_span, 10);
                                    else if (typeof d.membership_life_span === "number")
                                        m.membership_life_span = d.membership_life_span;
                                    else if (typeof d.membership_life_span === "object")
                                        m.membership_life_span = new $util.LongBits(d.membership_life_span.low >>> 0, d.membership_life_span.high >>> 0).toNumber();
                                }
                                if (d.size_for_display != null) {
                                    if ($util.Long)
                                        (m.size_for_display = $util.Long.fromValue(d.size_for_display)).unsigned = false;
                                    else if (typeof d.size_for_display === "string")
                                        m.size_for_display = parseInt(d.size_for_display, 10);
                                    else if (typeof d.size_for_display === "number")
                                        m.size_for_display = d.size_for_display;
                                    else if (typeof d.size_for_display === "object")
                                        m.size_for_display = new $util.LongBits(d.size_for_display.low >>> 0, d.size_for_display.high >>> 0).toNumber();
                                }
                                switch (d.size_range_for_display) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.size_range_for_display = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.size_range_for_display = 1;
                                    break;
                                case "LESS_THAN_FIVE_HUNDRED":
                                case 2:
                                    m.size_range_for_display = 2;
                                    break;
                                case "LESS_THAN_ONE_THOUSAND":
                                case 3:
                                    m.size_range_for_display = 3;
                                    break;
                                case "ONE_THOUSAND_TO_TEN_THOUSAND":
                                case 4:
                                    m.size_range_for_display = 4;
                                    break;
                                case "TEN_THOUSAND_TO_FIFTY_THOUSAND":
                                case 5:
                                    m.size_range_for_display = 5;
                                    break;
                                case "FIFTY_THOUSAND_TO_ONE_HUNDRED_THOUSAND":
                                case 6:
                                    m.size_range_for_display = 6;
                                    break;
                                case "ONE_HUNDRED_THOUSAND_TO_THREE_HUNDRED_THOUSAND":
                                case 7:
                                    m.size_range_for_display = 7;
                                    break;
                                case "THREE_HUNDRED_THOUSAND_TO_FIVE_HUNDRED_THOUSAND":
                                case 8:
                                    m.size_range_for_display = 8;
                                    break;
                                case "FIVE_HUNDRED_THOUSAND_TO_ONE_MILLION":
                                case 9:
                                    m.size_range_for_display = 9;
                                    break;
                                case "ONE_MILLION_TO_TWO_MILLION":
                                case 10:
                                    m.size_range_for_display = 10;
                                    break;
                                case "TWO_MILLION_TO_THREE_MILLION":
                                case 11:
                                    m.size_range_for_display = 11;
                                    break;
                                case "THREE_MILLION_TO_FIVE_MILLION":
                                case 12:
                                    m.size_range_for_display = 12;
                                    break;
                                case "FIVE_MILLION_TO_TEN_MILLION":
                                case 13:
                                    m.size_range_for_display = 13;
                                    break;
                                case "TEN_MILLION_TO_TWENTY_MILLION":
                                case 14:
                                    m.size_range_for_display = 14;
                                    break;
                                case "TWENTY_MILLION_TO_THIRTY_MILLION":
                                case 15:
                                    m.size_range_for_display = 15;
                                    break;
                                case "THIRTY_MILLION_TO_FIFTY_MILLION":
                                case 16:
                                    m.size_range_for_display = 16;
                                    break;
                                case "OVER_FIFTY_MILLION":
                                case 17:
                                    m.size_range_for_display = 17;
                                    break;
                                }
                                if (d.size_for_search != null) {
                                    if ($util.Long)
                                        (m.size_for_search = $util.Long.fromValue(d.size_for_search)).unsigned = false;
                                    else if (typeof d.size_for_search === "string")
                                        m.size_for_search = parseInt(d.size_for_search, 10);
                                    else if (typeof d.size_for_search === "number")
                                        m.size_for_search = d.size_for_search;
                                    else if (typeof d.size_for_search === "object")
                                        m.size_for_search = new $util.LongBits(d.size_for_search.low >>> 0, d.size_for_search.high >>> 0).toNumber();
                                }
                                switch (d.size_range_for_search) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.size_range_for_search = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.size_range_for_search = 1;
                                    break;
                                case "LESS_THAN_FIVE_HUNDRED":
                                case 2:
                                    m.size_range_for_search = 2;
                                    break;
                                case "LESS_THAN_ONE_THOUSAND":
                                case 3:
                                    m.size_range_for_search = 3;
                                    break;
                                case "ONE_THOUSAND_TO_TEN_THOUSAND":
                                case 4:
                                    m.size_range_for_search = 4;
                                    break;
                                case "TEN_THOUSAND_TO_FIFTY_THOUSAND":
                                case 5:
                                    m.size_range_for_search = 5;
                                    break;
                                case "FIFTY_THOUSAND_TO_ONE_HUNDRED_THOUSAND":
                                case 6:
                                    m.size_range_for_search = 6;
                                    break;
                                case "ONE_HUNDRED_THOUSAND_TO_THREE_HUNDRED_THOUSAND":
                                case 7:
                                    m.size_range_for_search = 7;
                                    break;
                                case "THREE_HUNDRED_THOUSAND_TO_FIVE_HUNDRED_THOUSAND":
                                case 8:
                                    m.size_range_for_search = 8;
                                    break;
                                case "FIVE_HUNDRED_THOUSAND_TO_ONE_MILLION":
                                case 9:
                                    m.size_range_for_search = 9;
                                    break;
                                case "ONE_MILLION_TO_TWO_MILLION":
                                case 10:
                                    m.size_range_for_search = 10;
                                    break;
                                case "TWO_MILLION_TO_THREE_MILLION":
                                case 11:
                                    m.size_range_for_search = 11;
                                    break;
                                case "THREE_MILLION_TO_FIVE_MILLION":
                                case 12:
                                    m.size_range_for_search = 12;
                                    break;
                                case "FIVE_MILLION_TO_TEN_MILLION":
                                case 13:
                                    m.size_range_for_search = 13;
                                    break;
                                case "TEN_MILLION_TO_TWENTY_MILLION":
                                case 14:
                                    m.size_range_for_search = 14;
                                    break;
                                case "TWENTY_MILLION_TO_THIRTY_MILLION":
                                case 15:
                                    m.size_range_for_search = 15;
                                    break;
                                case "THIRTY_MILLION_TO_FIFTY_MILLION":
                                case 16:
                                    m.size_range_for_search = 16;
                                    break;
                                case "OVER_FIFTY_MILLION":
                                case 17:
                                    m.size_range_for_search = 17;
                                    break;
                                }
                                switch (d.type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.type = 1;
                                    break;
                                case "REMARKETING":
                                case 2:
                                    m.type = 2;
                                    break;
                                case "LOGICAL":
                                case 3:
                                    m.type = 3;
                                    break;
                                case "EXTERNAL_REMARKETING":
                                case 4:
                                    m.type = 4;
                                    break;
                                case "RULE_BASED":
                                case 5:
                                    m.type = 5;
                                    break;
                                case "SIMILAR":
                                case 6:
                                    m.type = 6;
                                    break;
                                case "CRM_BASED":
                                case 7:
                                    m.type = 7;
                                    break;
                                }
                                switch (d.closing_reason) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.closing_reason = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.closing_reason = 1;
                                    break;
                                case "UNUSED":
                                case 2:
                                    m.closing_reason = 2;
                                    break;
                                }
                                switch (d.access_reason) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.access_reason = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.access_reason = 1;
                                    break;
                                case "OWNED":
                                case 2:
                                    m.access_reason = 2;
                                    break;
                                case "SHARED":
                                case 3:
                                    m.access_reason = 3;
                                    break;
                                case "LICENSED":
                                case 4:
                                    m.access_reason = 4;
                                    break;
                                case "SUBSCRIBED":
                                case 5:
                                    m.access_reason = 5;
                                    break;
                                case "AFFILIATED":
                                case 6:
                                    m.access_reason = 6;
                                    break;
                                }
                                switch (d.account_user_list_status) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.account_user_list_status = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.account_user_list_status = 1;
                                    break;
                                case "ENABLED":
                                case 2:
                                    m.account_user_list_status = 2;
                                    break;
                                case "DISABLED":
                                case 3:
                                    m.account_user_list_status = 3;
                                    break;
                                }
                                if (d.eligible_for_search != null) {
                                    m.eligible_for_search = Boolean(d.eligible_for_search);
                                }
                                if (d.eligible_for_display != null) {
                                    m.eligible_for_display = Boolean(d.eligible_for_display);
                                }
                                if (d.match_rate_percentage != null) {
                                    m.match_rate_percentage = d.match_rate_percentage | 0;
                                }
                                if (d.crm_based_user_list != null) {
                                    if (typeof d.crm_based_user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.UserList.crm_based_user_list: object expected");
                                    m.crm_based_user_list = $root.google.ads.googleads.v11.common.CrmBasedUserListInfo.fromObject(d.crm_based_user_list);
                                }
                                if (d.similar_user_list != null) {
                                    if (typeof d.similar_user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.UserList.similar_user_list: object expected");
                                    m.similar_user_list = $root.google.ads.googleads.v11.common.SimilarUserListInfo.fromObject(d.similar_user_list);
                                }
                                if (d.rule_based_user_list != null) {
                                    if (typeof d.rule_based_user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.UserList.rule_based_user_list: object expected");
                                    m.rule_based_user_list = $root.google.ads.googleads.v11.common.RuleBasedUserListInfo.fromObject(d.rule_based_user_list);
                                }
                                if (d.logical_user_list != null) {
                                    if (typeof d.logical_user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.UserList.logical_user_list: object expected");
                                    m.logical_user_list = $root.google.ads.googleads.v11.common.LogicalUserListInfo.fromObject(d.logical_user_list);
                                }
                                if (d.basic_user_list != null) {
                                    if (typeof d.basic_user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.resources.UserList.basic_user_list: object expected");
                                    m.basic_user_list = $root.google.ads.googleads.v11.common.BasicUserListInfo.fromObject(d.basic_user_list);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserList message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @static
                             * @param {google.ads.googleads.v11.resources.UserList} m UserList
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserList.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.membership_status = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.size_range_for_display = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.size_range_for_search = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.closing_reason = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.access_reason = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.account_user_list_status = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.membership_status != null && m.hasOwnProperty("membership_status")) {
                                    d.membership_status = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListMembershipStatusEnum.UserListMembershipStatus[m.membership_status] : m.membership_status;
                                }
                                if (m.size_range_for_display != null && m.hasOwnProperty("size_range_for_display")) {
                                    d.size_range_for_display = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListSizeRangeEnum.UserListSizeRange[m.size_range_for_display] : m.size_range_for_display;
                                }
                                if (m.size_range_for_search != null && m.hasOwnProperty("size_range_for_search")) {
                                    d.size_range_for_search = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListSizeRangeEnum.UserListSizeRange[m.size_range_for_search] : m.size_range_for_search;
                                }
                                if (m.type != null && m.hasOwnProperty("type")) {
                                    d.type = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListTypeEnum.UserListType[m.type] : m.type;
                                }
                                if (m.closing_reason != null && m.hasOwnProperty("closing_reason")) {
                                    d.closing_reason = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListClosingReasonEnum.UserListClosingReason[m.closing_reason] : m.closing_reason;
                                }
                                if (m.access_reason != null && m.hasOwnProperty("access_reason")) {
                                    d.access_reason = o.enums === String ? $root.google.ads.googleads.v11.enums.AccessReasonEnum.AccessReason[m.access_reason] : m.access_reason;
                                }
                                if (m.account_user_list_status != null && m.hasOwnProperty("account_user_list_status")) {
                                    d.account_user_list_status = o.enums === String ? $root.google.ads.googleads.v11.enums.UserListAccessStatusEnum.UserListAccessStatus[m.account_user_list_status] : m.account_user_list_status;
                                }
                                if (m.crm_based_user_list != null && m.hasOwnProperty("crm_based_user_list")) {
                                    d.crm_based_user_list = $root.google.ads.googleads.v11.common.CrmBasedUserListInfo.toObject(m.crm_based_user_list, o);
                                    if (o.oneofs)
                                        d.user_list = "crm_based_user_list";
                                }
                                if (m.similar_user_list != null && m.hasOwnProperty("similar_user_list")) {
                                    d.similar_user_list = $root.google.ads.googleads.v11.common.SimilarUserListInfo.toObject(m.similar_user_list, o);
                                    if (o.oneofs)
                                        d.user_list = "similar_user_list";
                                }
                                if (m.rule_based_user_list != null && m.hasOwnProperty("rule_based_user_list")) {
                                    d.rule_based_user_list = $root.google.ads.googleads.v11.common.RuleBasedUserListInfo.toObject(m.rule_based_user_list, o);
                                    if (o.oneofs)
                                        d.user_list = "rule_based_user_list";
                                }
                                if (m.logical_user_list != null && m.hasOwnProperty("logical_user_list")) {
                                    d.logical_user_list = $root.google.ads.googleads.v11.common.LogicalUserListInfo.toObject(m.logical_user_list, o);
                                    if (o.oneofs)
                                        d.user_list = "logical_user_list";
                                }
                                if (m.basic_user_list != null && m.hasOwnProperty("basic_user_list")) {
                                    d.basic_user_list = $root.google.ads.googleads.v11.common.BasicUserListInfo.toObject(m.basic_user_list, o);
                                    if (o.oneofs)
                                        d.user_list = "basic_user_list";
                                }
                                if (m.match_rate_percentage != null && m.hasOwnProperty("match_rate_percentage")) {
                                    d.match_rate_percentage = m.match_rate_percentage;
                                    if (o.oneofs)
                                        d._match_rate_percentage = "match_rate_percentage";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    if (typeof m.id === "number")
                                        d.id = o.longs === String ? String(m.id) : m.id;
                                    else
                                        d.id = o.longs === String ? $util.Long.prototype.toString.call(m.id) : o.longs === Number ? new $util.LongBits(m.id.low >>> 0, m.id.high >>> 0).toNumber() : m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.read_only != null && m.hasOwnProperty("read_only")) {
                                    d.read_only = m.read_only;
                                    if (o.oneofs)
                                        d._read_only = "read_only";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.description != null && m.hasOwnProperty("description")) {
                                    d.description = m.description;
                                    if (o.oneofs)
                                        d._description = "description";
                                }
                                if (m.integration_code != null && m.hasOwnProperty("integration_code")) {
                                    d.integration_code = m.integration_code;
                                    if (o.oneofs)
                                        d._integration_code = "integration_code";
                                }
                                if (m.membership_life_span != null && m.hasOwnProperty("membership_life_span")) {
                                    if (typeof m.membership_life_span === "number")
                                        d.membership_life_span = o.longs === String ? String(m.membership_life_span) : m.membership_life_span;
                                    else
                                        d.membership_life_span = o.longs === String ? $util.Long.prototype.toString.call(m.membership_life_span) : o.longs === Number ? new $util.LongBits(m.membership_life_span.low >>> 0, m.membership_life_span.high >>> 0).toNumber() : m.membership_life_span;
                                    if (o.oneofs)
                                        d._membership_life_span = "membership_life_span";
                                }
                                if (m.size_for_display != null && m.hasOwnProperty("size_for_display")) {
                                    if (typeof m.size_for_display === "number")
                                        d.size_for_display = o.longs === String ? String(m.size_for_display) : m.size_for_display;
                                    else
                                        d.size_for_display = o.longs === String ? $util.Long.prototype.toString.call(m.size_for_display) : o.longs === Number ? new $util.LongBits(m.size_for_display.low >>> 0, m.size_for_display.high >>> 0).toNumber() : m.size_for_display;
                                    if (o.oneofs)
                                        d._size_for_display = "size_for_display";
                                }
                                if (m.size_for_search != null && m.hasOwnProperty("size_for_search")) {
                                    if (typeof m.size_for_search === "number")
                                        d.size_for_search = o.longs === String ? String(m.size_for_search) : m.size_for_search;
                                    else
                                        d.size_for_search = o.longs === String ? $util.Long.prototype.toString.call(m.size_for_search) : o.longs === Number ? new $util.LongBits(m.size_for_search.low >>> 0, m.size_for_search.high >>> 0).toNumber() : m.size_for_search;
                                    if (o.oneofs)
                                        d._size_for_search = "size_for_search";
                                }
                                if (m.eligible_for_search != null && m.hasOwnProperty("eligible_for_search")) {
                                    d.eligible_for_search = m.eligible_for_search;
                                    if (o.oneofs)
                                        d._eligible_for_search = "eligible_for_search";
                                }
                                if (m.eligible_for_display != null && m.hasOwnProperty("eligible_for_display")) {
                                    d.eligible_for_display = m.eligible_for_display;
                                    if (o.oneofs)
                                        d._eligible_for_display = "eligible_for_display";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserList to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.UserList
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserList.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserList;
                        })();
    
                        resources.UserLocationView = (function() {
    
                            /**
                             * Properties of a UserLocationView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IUserLocationView
                             * @property {string|null} [resource_name] UserLocationView resource_name
                             * @property {number|null} [country_criterion_id] UserLocationView country_criterion_id
                             * @property {boolean|null} [targeting_location] UserLocationView targeting_location
                             */
    
                            /**
                             * Constructs a new UserLocationView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a UserLocationView.
                             * @implements IUserLocationView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IUserLocationView=} [p] Properties to set
                             */
                            function UserLocationView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserLocationView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @instance
                             */
                            UserLocationView.prototype.resource_name = "";
    
                            /**
                             * UserLocationView country_criterion_id.
                             * @member {number|null|undefined} country_criterion_id
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @instance
                             */
                            UserLocationView.prototype.country_criterion_id = null;
    
                            /**
                             * UserLocationView targeting_location.
                             * @member {boolean|null|undefined} targeting_location
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @instance
                             */
                            UserLocationView.prototype.targeting_location = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserLocationView _country_criterion_id.
                             * @member {"country_criterion_id"|undefined} _country_criterion_id
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @instance
                             */
                            Object.defineProperty(UserLocationView.prototype, "_country_criterion_id", {
                                get: $util.oneOfGetter($oneOfFields = ["country_criterion_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UserLocationView _targeting_location.
                             * @member {"targeting_location"|undefined} _targeting_location
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @instance
                             */
                            Object.defineProperty(UserLocationView.prototype, "_targeting_location", {
                                get: $util.oneOfGetter($oneOfFields = ["targeting_location"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserLocationView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserLocationView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.UserLocationView} UserLocationView instance
                             */
                            UserLocationView.create = function create(properties) {
                                return new UserLocationView(properties);
                            };
    
                            /**
                             * Encodes the specified UserLocationView message. Does not implicitly {@link google.ads.googleads.v11.resources.UserLocationView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserLocationView} m UserLocationView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserLocationView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.country_criterion_id != null && Object.hasOwnProperty.call(m, "country_criterion_id"))
                                    w.uint32(32).int64(m.country_criterion_id);
                                if (m.targeting_location != null && Object.hasOwnProperty.call(m, "targeting_location"))
                                    w.uint32(40).bool(m.targeting_location);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserLocationView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.UserLocationView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IUserLocationView} message UserLocationView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserLocationView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserLocationView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.UserLocationView} UserLocationView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserLocationView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.UserLocationView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.country_criterion_id = r.int64();
                                        break;
                                    case 5:
                                        m.targeting_location = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserLocationView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.UserLocationView} UserLocationView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserLocationView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserLocationView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserLocationView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.country_criterion_id != null && m.hasOwnProperty("country_criterion_id")) {
                                    p._country_criterion_id = 1;
                                    if (!$util.isInteger(m.country_criterion_id) && !(m.country_criterion_id && $util.isInteger(m.country_criterion_id.low) && $util.isInteger(m.country_criterion_id.high)))
                                        return "country_criterion_id: integer|Long expected";
                                }
                                if (m.targeting_location != null && m.hasOwnProperty("targeting_location")) {
                                    p._targeting_location = 1;
                                    if (typeof m.targeting_location !== "boolean")
                                        return "targeting_location: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserLocationView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.UserLocationView} UserLocationView
                             */
                            UserLocationView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.UserLocationView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.UserLocationView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.country_criterion_id != null) {
                                    if ($util.Long)
                                        (m.country_criterion_id = $util.Long.fromValue(d.country_criterion_id)).unsigned = false;
                                    else if (typeof d.country_criterion_id === "string")
                                        m.country_criterion_id = parseInt(d.country_criterion_id, 10);
                                    else if (typeof d.country_criterion_id === "number")
                                        m.country_criterion_id = d.country_criterion_id;
                                    else if (typeof d.country_criterion_id === "object")
                                        m.country_criterion_id = new $util.LongBits(d.country_criterion_id.low >>> 0, d.country_criterion_id.high >>> 0).toNumber();
                                }
                                if (d.targeting_location != null) {
                                    m.targeting_location = Boolean(d.targeting_location);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserLocationView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @static
                             * @param {google.ads.googleads.v11.resources.UserLocationView} m UserLocationView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserLocationView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.country_criterion_id != null && m.hasOwnProperty("country_criterion_id")) {
                                    if (typeof m.country_criterion_id === "number")
                                        d.country_criterion_id = o.longs === String ? String(m.country_criterion_id) : m.country_criterion_id;
                                    else
                                        d.country_criterion_id = o.longs === String ? $util.Long.prototype.toString.call(m.country_criterion_id) : o.longs === Number ? new $util.LongBits(m.country_criterion_id.low >>> 0, m.country_criterion_id.high >>> 0).toNumber() : m.country_criterion_id;
                                    if (o.oneofs)
                                        d._country_criterion_id = "country_criterion_id";
                                }
                                if (m.targeting_location != null && m.hasOwnProperty("targeting_location")) {
                                    d.targeting_location = m.targeting_location;
                                    if (o.oneofs)
                                        d._targeting_location = "targeting_location";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserLocationView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.UserLocationView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserLocationView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserLocationView;
                        })();
    
                        resources.Video = (function() {
    
                            /**
                             * Properties of a Video.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IVideo
                             * @property {string|null} [resource_name] Video resource_name
                             * @property {string|null} [id] Video id
                             * @property {string|null} [channel_id] Video channel_id
                             * @property {number|null} [duration_millis] Video duration_millis
                             * @property {string|null} [title] Video title
                             */
    
                            /**
                             * Constructs a new Video.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a Video.
                             * @implements IVideo
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IVideo=} [p] Properties to set
                             */
                            function Video(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Video resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Video.prototype.resource_name = "";
    
                            /**
                             * Video id.
                             * @member {string|null|undefined} id
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Video.prototype.id = null;
    
                            /**
                             * Video channel_id.
                             * @member {string|null|undefined} channel_id
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Video.prototype.channel_id = null;
    
                            /**
                             * Video duration_millis.
                             * @member {number|null|undefined} duration_millis
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Video.prototype.duration_millis = null;
    
                            /**
                             * Video title.
                             * @member {string|null|undefined} title
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Video.prototype.title = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Video _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Object.defineProperty(Video.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Video _channel_id.
                             * @member {"channel_id"|undefined} _channel_id
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Object.defineProperty(Video.prototype, "_channel_id", {
                                get: $util.oneOfGetter($oneOfFields = ["channel_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Video _duration_millis.
                             * @member {"duration_millis"|undefined} _duration_millis
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Object.defineProperty(Video.prototype, "_duration_millis", {
                                get: $util.oneOfGetter($oneOfFields = ["duration_millis"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Video _title.
                             * @member {"title"|undefined} _title
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             */
                            Object.defineProperty(Video.prototype, "_title", {
                                get: $util.oneOfGetter($oneOfFields = ["title"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Video instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {google.ads.googleads.v11.resources.IVideo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.Video} Video instance
                             */
                            Video.create = function create(properties) {
                                return new Video(properties);
                            };
    
                            /**
                             * Encodes the specified Video message. Does not implicitly {@link google.ads.googleads.v11.resources.Video.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {google.ads.googleads.v11.resources.IVideo} m Video message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Video.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(50).string(m.id);
                                if (m.channel_id != null && Object.hasOwnProperty.call(m, "channel_id"))
                                    w.uint32(58).string(m.channel_id);
                                if (m.duration_millis != null && Object.hasOwnProperty.call(m, "duration_millis"))
                                    w.uint32(64).int64(m.duration_millis);
                                if (m.title != null && Object.hasOwnProperty.call(m, "title"))
                                    w.uint32(74).string(m.title);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Video message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.Video.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {google.ads.googleads.v11.resources.IVideo} message Video message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Video.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Video message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.Video} Video
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Video.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.Video();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 6:
                                        m.id = r.string();
                                        break;
                                    case 7:
                                        m.channel_id = r.string();
                                        break;
                                    case 8:
                                        m.duration_millis = r.int64();
                                        break;
                                    case 9:
                                        m.title = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Video message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.Video} Video
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Video.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Video message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Video.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isString(m.id))
                                        return "id: string expected";
                                }
                                if (m.channel_id != null && m.hasOwnProperty("channel_id")) {
                                    p._channel_id = 1;
                                    if (!$util.isString(m.channel_id))
                                        return "channel_id: string expected";
                                }
                                if (m.duration_millis != null && m.hasOwnProperty("duration_millis")) {
                                    p._duration_millis = 1;
                                    if (!$util.isInteger(m.duration_millis) && !(m.duration_millis && $util.isInteger(m.duration_millis.low) && $util.isInteger(m.duration_millis.high)))
                                        return "duration_millis: integer|Long expected";
                                }
                                if (m.title != null && m.hasOwnProperty("title")) {
                                    p._title = 1;
                                    if (!$util.isString(m.title))
                                        return "title: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Video message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.Video} Video
                             */
                            Video.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.Video)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.Video();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.id != null) {
                                    m.id = String(d.id);
                                }
                                if (d.channel_id != null) {
                                    m.channel_id = String(d.channel_id);
                                }
                                if (d.duration_millis != null) {
                                    if ($util.Long)
                                        (m.duration_millis = $util.Long.fromValue(d.duration_millis)).unsigned = false;
                                    else if (typeof d.duration_millis === "string")
                                        m.duration_millis = parseInt(d.duration_millis, 10);
                                    else if (typeof d.duration_millis === "number")
                                        m.duration_millis = d.duration_millis;
                                    else if (typeof d.duration_millis === "object")
                                        m.duration_millis = new $util.LongBits(d.duration_millis.low >>> 0, d.duration_millis.high >>> 0).toNumber();
                                }
                                if (d.title != null) {
                                    m.title = String(d.title);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Video message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @static
                             * @param {google.ads.googleads.v11.resources.Video} m Video
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Video.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    d.id = m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.channel_id != null && m.hasOwnProperty("channel_id")) {
                                    d.channel_id = m.channel_id;
                                    if (o.oneofs)
                                        d._channel_id = "channel_id";
                                }
                                if (m.duration_millis != null && m.hasOwnProperty("duration_millis")) {
                                    if (typeof m.duration_millis === "number")
                                        d.duration_millis = o.longs === String ? String(m.duration_millis) : m.duration_millis;
                                    else
                                        d.duration_millis = o.longs === String ? $util.Long.prototype.toString.call(m.duration_millis) : o.longs === Number ? new $util.LongBits(m.duration_millis.low >>> 0, m.duration_millis.high >>> 0).toNumber() : m.duration_millis;
                                    if (o.oneofs)
                                        d._duration_millis = "duration_millis";
                                }
                                if (m.title != null && m.hasOwnProperty("title")) {
                                    d.title = m.title;
                                    if (o.oneofs)
                                        d._title = "title";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Video to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.Video
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Video.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Video;
                        })();
    
                        resources.WebpageView = (function() {
    
                            /**
                             * Properties of a WebpageView.
                             * @memberof google.ads.googleads.v11.resources
                             * @interface IWebpageView
                             * @property {string|null} [resource_name] WebpageView resource_name
                             */
    
                            /**
                             * Constructs a new WebpageView.
                             * @memberof google.ads.googleads.v11.resources
                             * @classdesc Represents a WebpageView.
                             * @implements IWebpageView
                             * @constructor
                             * @param {google.ads.googleads.v11.resources.IWebpageView=} [p] Properties to set
                             */
                            function WebpageView(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * WebpageView resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @instance
                             */
                            WebpageView.prototype.resource_name = "";
    
                            /**
                             * Creates a new WebpageView instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IWebpageView=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.resources.WebpageView} WebpageView instance
                             */
                            WebpageView.create = function create(properties) {
                                return new WebpageView(properties);
                            };
    
                            /**
                             * Encodes the specified WebpageView message. Does not implicitly {@link google.ads.googleads.v11.resources.WebpageView.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IWebpageView} m WebpageView message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageView.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified WebpageView message, length delimited. Does not implicitly {@link google.ads.googleads.v11.resources.WebpageView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.IWebpageView} message WebpageView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            WebpageView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a WebpageView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.resources.WebpageView} WebpageView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageView.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.resources.WebpageView();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a WebpageView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.resources.WebpageView} WebpageView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            WebpageView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a WebpageView message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            WebpageView.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a WebpageView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.resources.WebpageView} WebpageView
                             */
                            WebpageView.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.resources.WebpageView)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.resources.WebpageView();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a WebpageView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @static
                             * @param {google.ads.googleads.v11.resources.WebpageView} m WebpageView
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            WebpageView.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this WebpageView to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.resources.WebpageView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            WebpageView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return WebpageView;
                        })();
    
                        return resources;
                    })();
    
                    v11.services = (function() {
    
                        /**
                         * Namespace services.
                         * @memberof google.ads.googleads.v11
                         * @namespace
                         */
                        var services = {};
    
                        services.AccountBudgetProposalService = (function() {
    
                            /**
                             * Constructs a new AccountBudgetProposalService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AccountBudgetProposalService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AccountBudgetProposalService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AccountBudgetProposalService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AccountBudgetProposalService;
    
                            /**
                             * Creates new AccountBudgetProposalService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AccountBudgetProposalService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AccountBudgetProposalService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AccountBudgetProposalService|mutateAccountBudgetProposal}.
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalService
                             * @typedef MutateAccountBudgetProposalCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse} [response] MutateAccountBudgetProposalResponse
                             */
    
                            /**
                             * Calls MutateAccountBudgetProposal.
                             * @function mutateAccountBudgetProposal
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalRequest} request MutateAccountBudgetProposalRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AccountBudgetProposalService.MutateAccountBudgetProposalCallback} callback Node-style callback called with the error, if any, and MutateAccountBudgetProposalResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AccountBudgetProposalService.prototype.mutateAccountBudgetProposal = function mutateAccountBudgetProposal(request, callback) {
                                return this.rpcCall(mutateAccountBudgetProposal, $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest, $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse, request, callback);
                            }, "name", { value: "MutateAccountBudgetProposal" });
    
                            /**
                             * Calls MutateAccountBudgetProposal.
                             * @function mutateAccountBudgetProposal
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalRequest} request MutateAccountBudgetProposalRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse>} Promise
                             * @variation 2
                             */
    
                            return AccountBudgetProposalService;
                        })();
    
                        services.MutateAccountBudgetProposalRequest = (function() {
    
                            /**
                             * Properties of a MutateAccountBudgetProposalRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAccountBudgetProposalRequest
                             * @property {string|null} [customer_id] MutateAccountBudgetProposalRequest customer_id
                             * @property {google.ads.googleads.v11.services.IAccountBudgetProposalOperation|null} [operation] MutateAccountBudgetProposalRequest operation
                             * @property {boolean|null} [validate_only] MutateAccountBudgetProposalRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAccountBudgetProposalRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAccountBudgetProposalRequest.
                             * @implements IMutateAccountBudgetProposalRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalRequest=} [p] Properties to set
                             */
                            function MutateAccountBudgetProposalRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAccountBudgetProposalRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @instance
                             */
                            MutateAccountBudgetProposalRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAccountBudgetProposalRequest operation.
                             * @member {google.ads.googleads.v11.services.IAccountBudgetProposalOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @instance
                             */
                            MutateAccountBudgetProposalRequest.prototype.operation = null;
    
                            /**
                             * MutateAccountBudgetProposalRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @instance
                             */
                            MutateAccountBudgetProposalRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAccountBudgetProposalRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest} MutateAccountBudgetProposalRequest instance
                             */
                            MutateAccountBudgetProposalRequest.create = function create(properties) {
                                return new MutateAccountBudgetProposalRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAccountBudgetProposalRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalRequest} m MutateAccountBudgetProposalRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountBudgetProposalRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAccountBudgetProposalRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalRequest} message MutateAccountBudgetProposalRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountBudgetProposalRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAccountBudgetProposalRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest} MutateAccountBudgetProposalRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountBudgetProposalRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAccountBudgetProposalRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest} MutateAccountBudgetProposalRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountBudgetProposalRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAccountBudgetProposalRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAccountBudgetProposalRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAccountBudgetProposalRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest} MutateAccountBudgetProposalRequest
                             */
                            MutateAccountBudgetProposalRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation.fromObject(d.operation);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAccountBudgetProposalRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest} m MutateAccountBudgetProposalRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAccountBudgetProposalRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation.toObject(m.operation, o);
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAccountBudgetProposalRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAccountBudgetProposalRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAccountBudgetProposalRequest;
                        })();
    
                        services.AccountBudgetProposalOperation = (function() {
    
                            /**
                             * Properties of an AccountBudgetProposalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAccountBudgetProposalOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AccountBudgetProposalOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAccountBudgetProposal|null} [create] AccountBudgetProposalOperation create
                             * @property {string|null} [remove] AccountBudgetProposalOperation remove
                             */
    
                            /**
                             * Constructs a new AccountBudgetProposalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AccountBudgetProposalOperation.
                             * @implements IAccountBudgetProposalOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAccountBudgetProposalOperation=} [p] Properties to set
                             */
                            function AccountBudgetProposalOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AccountBudgetProposalOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @instance
                             */
                            AccountBudgetProposalOperation.prototype.update_mask = null;
    
                            /**
                             * AccountBudgetProposalOperation create.
                             * @member {google.ads.googleads.v11.resources.IAccountBudgetProposal|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @instance
                             */
                            AccountBudgetProposalOperation.prototype.create = null;
    
                            /**
                             * AccountBudgetProposalOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @instance
                             */
                            AccountBudgetProposalOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AccountBudgetProposalOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @instance
                             */
                            Object.defineProperty(AccountBudgetProposalOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AccountBudgetProposalOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAccountBudgetProposalOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AccountBudgetProposalOperation} AccountBudgetProposalOperation instance
                             */
                            AccountBudgetProposalOperation.create = function create(properties) {
                                return new AccountBudgetProposalOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AccountBudgetProposalOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAccountBudgetProposalOperation} m AccountBudgetProposalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(10).string(m.remove);
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AccountBudgetProposal.encode(m.create, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountBudgetProposalOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AccountBudgetProposalOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAccountBudgetProposalOperation} message AccountBudgetProposalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountBudgetProposalOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AccountBudgetProposalOperation} AccountBudgetProposalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.create = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountBudgetProposalOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AccountBudgetProposalOperation} AccountBudgetProposalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountBudgetProposalOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountBudgetProposalOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountBudgetProposalOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AccountBudgetProposalOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AccountBudgetProposalOperation} AccountBudgetProposalOperation
                             */
                            AccountBudgetProposalOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AccountBudgetProposalOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AccountBudgetProposalOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AccountBudgetProposalOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AccountBudgetProposalOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AccountBudgetProposalOperation} m AccountBudgetProposalOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountBudgetProposalOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AccountBudgetProposalOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AccountBudgetProposalOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountBudgetProposalOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AccountBudgetProposalOperation;
                        })();
    
                        services.MutateAccountBudgetProposalResponse = (function() {
    
                            /**
                             * Properties of a MutateAccountBudgetProposalResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAccountBudgetProposalResponse
                             * @property {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResult|null} [result] MutateAccountBudgetProposalResponse result
                             */
    
                            /**
                             * Constructs a new MutateAccountBudgetProposalResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAccountBudgetProposalResponse.
                             * @implements IMutateAccountBudgetProposalResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResponse=} [p] Properties to set
                             */
                            function MutateAccountBudgetProposalResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAccountBudgetProposalResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @instance
                             */
                            MutateAccountBudgetProposalResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateAccountBudgetProposalResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse} MutateAccountBudgetProposalResponse instance
                             */
                            MutateAccountBudgetProposalResponse.create = function create(properties) {
                                return new MutateAccountBudgetProposalResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAccountBudgetProposalResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResponse} m MutateAccountBudgetProposalResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountBudgetProposalResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult.encode(m.result, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAccountBudgetProposalResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResponse} message MutateAccountBudgetProposalResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountBudgetProposalResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAccountBudgetProposalResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse} MutateAccountBudgetProposalResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountBudgetProposalResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.result = $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAccountBudgetProposalResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse} MutateAccountBudgetProposalResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountBudgetProposalResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAccountBudgetProposalResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAccountBudgetProposalResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAccountBudgetProposalResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse} MutateAccountBudgetProposalResponse
                             */
                            MutateAccountBudgetProposalResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAccountBudgetProposalResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse} m MutateAccountBudgetProposalResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAccountBudgetProposalResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAccountBudgetProposalResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAccountBudgetProposalResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAccountBudgetProposalResponse;
                        })();
    
                        services.MutateAccountBudgetProposalResult = (function() {
    
                            /**
                             * Properties of a MutateAccountBudgetProposalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAccountBudgetProposalResult
                             * @property {string|null} [resource_name] MutateAccountBudgetProposalResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateAccountBudgetProposalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAccountBudgetProposalResult.
                             * @implements IMutateAccountBudgetProposalResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResult=} [p] Properties to set
                             */
                            function MutateAccountBudgetProposalResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAccountBudgetProposalResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @instance
                             */
                            MutateAccountBudgetProposalResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateAccountBudgetProposalResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResult} MutateAccountBudgetProposalResult instance
                             */
                            MutateAccountBudgetProposalResult.create = function create(properties) {
                                return new MutateAccountBudgetProposalResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAccountBudgetProposalResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountBudgetProposalResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResult} m MutateAccountBudgetProposalResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountBudgetProposalResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAccountBudgetProposalResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountBudgetProposalResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountBudgetProposalResult} message MutateAccountBudgetProposalResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountBudgetProposalResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAccountBudgetProposalResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResult} MutateAccountBudgetProposalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountBudgetProposalResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAccountBudgetProposalResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResult} MutateAccountBudgetProposalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountBudgetProposalResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAccountBudgetProposalResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAccountBudgetProposalResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAccountBudgetProposalResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAccountBudgetProposalResult} MutateAccountBudgetProposalResult
                             */
                            MutateAccountBudgetProposalResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAccountBudgetProposalResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAccountBudgetProposalResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAccountBudgetProposalResult} m MutateAccountBudgetProposalResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAccountBudgetProposalResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAccountBudgetProposalResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAccountBudgetProposalResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAccountBudgetProposalResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAccountBudgetProposalResult;
                        })();
    
                        services.AccountLinkService = (function() {
    
                            /**
                             * Constructs a new AccountLinkService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AccountLinkService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AccountLinkService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AccountLinkService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AccountLinkService;
    
                            /**
                             * Creates new AccountLinkService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AccountLinkService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AccountLinkService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AccountLinkService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AccountLinkService|createAccountLink}.
                             * @memberof google.ads.googleads.v11.services.AccountLinkService
                             * @typedef CreateAccountLinkCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.CreateAccountLinkResponse} [response] CreateAccountLinkResponse
                             */
    
                            /**
                             * Calls CreateAccountLink.
                             * @function createAccountLink
                             * @memberof google.ads.googleads.v11.services.AccountLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkRequest} request CreateAccountLinkRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AccountLinkService.CreateAccountLinkCallback} callback Node-style callback called with the error, if any, and CreateAccountLinkResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AccountLinkService.prototype.createAccountLink = function createAccountLink(request, callback) {
                                return this.rpcCall(createAccountLink, $root.google.ads.googleads.v11.services.CreateAccountLinkRequest, $root.google.ads.googleads.v11.services.CreateAccountLinkResponse, request, callback);
                            }, "name", { value: "CreateAccountLink" });
    
                            /**
                             * Calls CreateAccountLink.
                             * @function createAccountLink
                             * @memberof google.ads.googleads.v11.services.AccountLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkRequest} request CreateAccountLinkRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.CreateAccountLinkResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AccountLinkService|mutateAccountLink}.
                             * @memberof google.ads.googleads.v11.services.AccountLinkService
                             * @typedef MutateAccountLinkCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAccountLinkResponse} [response] MutateAccountLinkResponse
                             */
    
                            /**
                             * Calls MutateAccountLink.
                             * @function mutateAccountLink
                             * @memberof google.ads.googleads.v11.services.AccountLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkRequest} request MutateAccountLinkRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AccountLinkService.MutateAccountLinkCallback} callback Node-style callback called with the error, if any, and MutateAccountLinkResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AccountLinkService.prototype.mutateAccountLink = function mutateAccountLink(request, callback) {
                                return this.rpcCall(mutateAccountLink, $root.google.ads.googleads.v11.services.MutateAccountLinkRequest, $root.google.ads.googleads.v11.services.MutateAccountLinkResponse, request, callback);
                            }, "name", { value: "MutateAccountLink" });
    
                            /**
                             * Calls MutateAccountLink.
                             * @function mutateAccountLink
                             * @memberof google.ads.googleads.v11.services.AccountLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkRequest} request MutateAccountLinkRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAccountLinkResponse>} Promise
                             * @variation 2
                             */
    
                            return AccountLinkService;
                        })();
    
                        services.CreateAccountLinkRequest = (function() {
    
                            /**
                             * Properties of a CreateAccountLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateAccountLinkRequest
                             * @property {string|null} [customer_id] CreateAccountLinkRequest customer_id
                             * @property {google.ads.googleads.v11.resources.IAccountLink|null} [account_link] CreateAccountLinkRequest account_link
                             */
    
                            /**
                             * Constructs a new CreateAccountLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateAccountLinkRequest.
                             * @implements ICreateAccountLinkRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkRequest=} [p] Properties to set
                             */
                            function CreateAccountLinkRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateAccountLinkRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @instance
                             */
                            CreateAccountLinkRequest.prototype.customer_id = "";
    
                            /**
                             * CreateAccountLinkRequest account_link.
                             * @member {google.ads.googleads.v11.resources.IAccountLink|null|undefined} account_link
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @instance
                             */
                            CreateAccountLinkRequest.prototype.account_link = null;
    
                            /**
                             * Creates a new CreateAccountLinkRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkRequest} CreateAccountLinkRequest instance
                             */
                            CreateAccountLinkRequest.create = function create(properties) {
                                return new CreateAccountLinkRequest(properties);
                            };
    
                            /**
                             * Encodes the specified CreateAccountLinkRequest message. Does not implicitly {@link google.ads.googleads.v11.services.CreateAccountLinkRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkRequest} m CreateAccountLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateAccountLinkRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.account_link != null && Object.hasOwnProperty.call(m, "account_link"))
                                    $root.google.ads.googleads.v11.resources.AccountLink.encode(m.account_link, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateAccountLinkRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateAccountLinkRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkRequest} message CreateAccountLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateAccountLinkRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateAccountLinkRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkRequest} CreateAccountLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateAccountLinkRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateAccountLinkRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.account_link = $root.google.ads.googleads.v11.resources.AccountLink.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateAccountLinkRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkRequest} CreateAccountLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateAccountLinkRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateAccountLinkRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateAccountLinkRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.account_link != null && m.hasOwnProperty("account_link")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccountLink.verify(m.account_link);
                                        if (e)
                                            return "account_link." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateAccountLinkRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkRequest} CreateAccountLinkRequest
                             */
                            CreateAccountLinkRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateAccountLinkRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateAccountLinkRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.account_link != null) {
                                    if (typeof d.account_link !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CreateAccountLinkRequest.account_link: object expected");
                                    m.account_link = $root.google.ads.googleads.v11.resources.AccountLink.fromObject(d.account_link);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateAccountLinkRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateAccountLinkRequest} m CreateAccountLinkRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateAccountLinkRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.account_link = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.account_link != null && m.hasOwnProperty("account_link")) {
                                    d.account_link = $root.google.ads.googleads.v11.resources.AccountLink.toObject(m.account_link, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateAccountLinkRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateAccountLinkRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateAccountLinkRequest;
                        })();
    
                        services.CreateAccountLinkResponse = (function() {
    
                            /**
                             * Properties of a CreateAccountLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateAccountLinkResponse
                             * @property {string|null} [resource_name] CreateAccountLinkResponse resource_name
                             */
    
                            /**
                             * Constructs a new CreateAccountLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateAccountLinkResponse.
                             * @implements ICreateAccountLinkResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkResponse=} [p] Properties to set
                             */
                            function CreateAccountLinkResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateAccountLinkResponse resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @instance
                             */
                            CreateAccountLinkResponse.prototype.resource_name = "";
    
                            /**
                             * Creates a new CreateAccountLinkResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkResponse} CreateAccountLinkResponse instance
                             */
                            CreateAccountLinkResponse.create = function create(properties) {
                                return new CreateAccountLinkResponse(properties);
                            };
    
                            /**
                             * Encodes the specified CreateAccountLinkResponse message. Does not implicitly {@link google.ads.googleads.v11.services.CreateAccountLinkResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkResponse} m CreateAccountLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateAccountLinkResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateAccountLinkResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateAccountLinkResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateAccountLinkResponse} message CreateAccountLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateAccountLinkResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateAccountLinkResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkResponse} CreateAccountLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateAccountLinkResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateAccountLinkResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateAccountLinkResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkResponse} CreateAccountLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateAccountLinkResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateAccountLinkResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateAccountLinkResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateAccountLinkResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateAccountLinkResponse} CreateAccountLinkResponse
                             */
                            CreateAccountLinkResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateAccountLinkResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateAccountLinkResponse();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateAccountLinkResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateAccountLinkResponse} m CreateAccountLinkResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateAccountLinkResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateAccountLinkResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateAccountLinkResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateAccountLinkResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateAccountLinkResponse;
                        })();
    
                        services.MutateAccountLinkRequest = (function() {
    
                            /**
                             * Properties of a MutateAccountLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAccountLinkRequest
                             * @property {string|null} [customer_id] MutateAccountLinkRequest customer_id
                             * @property {google.ads.googleads.v11.services.IAccountLinkOperation|null} [operation] MutateAccountLinkRequest operation
                             * @property {boolean|null} [partial_failure] MutateAccountLinkRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAccountLinkRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAccountLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAccountLinkRequest.
                             * @implements IMutateAccountLinkRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkRequest=} [p] Properties to set
                             */
                            function MutateAccountLinkRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAccountLinkRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @instance
                             */
                            MutateAccountLinkRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAccountLinkRequest operation.
                             * @member {google.ads.googleads.v11.services.IAccountLinkOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @instance
                             */
                            MutateAccountLinkRequest.prototype.operation = null;
    
                            /**
                             * MutateAccountLinkRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @instance
                             */
                            MutateAccountLinkRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAccountLinkRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @instance
                             */
                            MutateAccountLinkRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAccountLinkRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkRequest} MutateAccountLinkRequest instance
                             */
                            MutateAccountLinkRequest.create = function create(properties) {
                                return new MutateAccountLinkRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAccountLinkRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountLinkRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkRequest} m MutateAccountLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountLinkRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.AccountLinkOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAccountLinkRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountLinkRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkRequest} message MutateAccountLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountLinkRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAccountLinkRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkRequest} MutateAccountLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountLinkRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAccountLinkRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.AccountLinkOperation.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAccountLinkRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkRequest} MutateAccountLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountLinkRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAccountLinkRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAccountLinkRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AccountLinkOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAccountLinkRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkRequest} MutateAccountLinkRequest
                             */
                            MutateAccountLinkRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAccountLinkRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAccountLinkRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAccountLinkRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.AccountLinkOperation.fromObject(d.operation);
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAccountLinkRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAccountLinkRequest} m MutateAccountLinkRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAccountLinkRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.AccountLinkOperation.toObject(m.operation, o);
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAccountLinkRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAccountLinkRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAccountLinkRequest;
                        })();
    
                        services.AccountLinkOperation = (function() {
    
                            /**
                             * Properties of an AccountLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAccountLinkOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AccountLinkOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAccountLink|null} [update] AccountLinkOperation update
                             * @property {string|null} [remove] AccountLinkOperation remove
                             */
    
                            /**
                             * Constructs a new AccountLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AccountLinkOperation.
                             * @implements IAccountLinkOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAccountLinkOperation=} [p] Properties to set
                             */
                            function AccountLinkOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AccountLinkOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @instance
                             */
                            AccountLinkOperation.prototype.update_mask = null;
    
                            /**
                             * AccountLinkOperation update.
                             * @member {google.ads.googleads.v11.resources.IAccountLink|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @instance
                             */
                            AccountLinkOperation.prototype.update = null;
    
                            /**
                             * AccountLinkOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @instance
                             */
                            AccountLinkOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AccountLinkOperation operation.
                             * @member {"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @instance
                             */
                            Object.defineProperty(AccountLinkOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AccountLinkOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAccountLinkOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AccountLinkOperation} AccountLinkOperation instance
                             */
                            AccountLinkOperation.create = function create(properties) {
                                return new AccountLinkOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AccountLinkOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AccountLinkOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAccountLinkOperation} m AccountLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLinkOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AccountLink.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AccountLinkOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AccountLinkOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAccountLinkOperation} message AccountLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AccountLinkOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AccountLinkOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AccountLinkOperation} AccountLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLinkOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AccountLinkOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AccountLink.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AccountLinkOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AccountLinkOperation} AccountLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AccountLinkOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AccountLinkOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AccountLinkOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccountLink.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AccountLinkOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AccountLinkOperation} AccountLinkOperation
                             */
                            AccountLinkOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AccountLinkOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AccountLinkOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AccountLinkOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AccountLinkOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AccountLink.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AccountLinkOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AccountLinkOperation} m AccountLinkOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AccountLinkOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AccountLink.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AccountLinkOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AccountLinkOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AccountLinkOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AccountLinkOperation;
                        })();
    
                        services.MutateAccountLinkResponse = (function() {
    
                            /**
                             * Properties of a MutateAccountLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAccountLinkResponse
                             * @property {google.ads.googleads.v11.services.IMutateAccountLinkResult|null} [result] MutateAccountLinkResponse result
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAccountLinkResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAccountLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAccountLinkResponse.
                             * @implements IMutateAccountLinkResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResponse=} [p] Properties to set
                             */
                            function MutateAccountLinkResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAccountLinkResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateAccountLinkResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @instance
                             */
                            MutateAccountLinkResponse.prototype.result = null;
    
                            /**
                             * MutateAccountLinkResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @instance
                             */
                            MutateAccountLinkResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAccountLinkResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResponse} MutateAccountLinkResponse instance
                             */
                            MutateAccountLinkResponse.create = function create(properties) {
                                return new MutateAccountLinkResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAccountLinkResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountLinkResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResponse} m MutateAccountLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountLinkResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateAccountLinkResult.encode(m.result, w.uint32(10).fork()).ldelim();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAccountLinkResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountLinkResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResponse} message MutateAccountLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountLinkResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAccountLinkResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResponse} MutateAccountLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountLinkResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAccountLinkResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.result = $root.google.ads.googleads.v11.services.MutateAccountLinkResult.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAccountLinkResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResponse} MutateAccountLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountLinkResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAccountLinkResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAccountLinkResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAccountLinkResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAccountLinkResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResponse} MutateAccountLinkResponse
                             */
                            MutateAccountLinkResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAccountLinkResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAccountLinkResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAccountLinkResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateAccountLinkResult.fromObject(d.result);
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAccountLinkResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAccountLinkResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAccountLinkResponse} m MutateAccountLinkResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAccountLinkResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                    d.partial_failure_error = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateAccountLinkResult.toObject(m.result, o);
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAccountLinkResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAccountLinkResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAccountLinkResponse;
                        })();
    
                        services.MutateAccountLinkResult = (function() {
    
                            /**
                             * Properties of a MutateAccountLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAccountLinkResult
                             * @property {string|null} [resource_name] MutateAccountLinkResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateAccountLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAccountLinkResult.
                             * @implements IMutateAccountLinkResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResult=} [p] Properties to set
                             */
                            function MutateAccountLinkResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAccountLinkResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @instance
                             */
                            MutateAccountLinkResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateAccountLinkResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResult} MutateAccountLinkResult instance
                             */
                            MutateAccountLinkResult.create = function create(properties) {
                                return new MutateAccountLinkResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAccountLinkResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountLinkResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResult} m MutateAccountLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountLinkResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAccountLinkResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAccountLinkResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAccountLinkResult} message MutateAccountLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAccountLinkResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAccountLinkResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResult} MutateAccountLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountLinkResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAccountLinkResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAccountLinkResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResult} MutateAccountLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAccountLinkResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAccountLinkResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAccountLinkResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAccountLinkResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAccountLinkResult} MutateAccountLinkResult
                             */
                            MutateAccountLinkResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAccountLinkResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAccountLinkResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAccountLinkResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAccountLinkResult} m MutateAccountLinkResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAccountLinkResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAccountLinkResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAccountLinkResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAccountLinkResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAccountLinkResult;
                        })();
    
                        services.AdGroupAdLabelService = (function() {
    
                            /**
                             * Constructs a new AdGroupAdLabelService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupAdLabelService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupAdLabelService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupAdLabelService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupAdLabelService;
    
                            /**
                             * Creates new AdGroupAdLabelService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupAdLabelService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupAdLabelService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupAdLabelService|mutateAdGroupAdLabels}.
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelService
                             * @typedef MutateAdGroupAdLabelsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse} [response] MutateAdGroupAdLabelsResponse
                             */
    
                            /**
                             * Calls MutateAdGroupAdLabels.
                             * @function mutateAdGroupAdLabels
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsRequest} request MutateAdGroupAdLabelsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupAdLabelService.MutateAdGroupAdLabelsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupAdLabelsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupAdLabelService.prototype.mutateAdGroupAdLabels = function mutateAdGroupAdLabels(request, callback) {
                                return this.rpcCall(mutateAdGroupAdLabels, $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse, request, callback);
                            }, "name", { value: "MutateAdGroupAdLabels" });
    
                            /**
                             * Calls MutateAdGroupAdLabels.
                             * @function mutateAdGroupAdLabels
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsRequest} request MutateAdGroupAdLabelsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupAdLabelService;
                        })();
    
                        services.MutateAdGroupAdLabelsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAdLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAdLabelsRequest
                             * @property {string|null} [customer_id] MutateAdGroupAdLabelsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupAdLabelOperation>|null} [operations] MutateAdGroupAdLabelsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupAdLabelsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupAdLabelsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAdLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAdLabelsRequest.
                             * @implements IMutateAdGroupAdLabelsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupAdLabelsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAdLabelsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @instance
                             */
                            MutateAdGroupAdLabelsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupAdLabelsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupAdLabelOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @instance
                             */
                            MutateAdGroupAdLabelsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupAdLabelsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @instance
                             */
                            MutateAdGroupAdLabelsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupAdLabelsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @instance
                             */
                            MutateAdGroupAdLabelsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAdGroupAdLabelsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest} MutateAdGroupAdLabelsRequest instance
                             */
                            MutateAdGroupAdLabelsRequest.create = function create(properties) {
                                return new MutateAdGroupAdLabelsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdLabelsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsRequest} m MutateAdGroupAdLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdLabelsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdLabelsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsRequest} message MutateAdGroupAdLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdLabelsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdLabelsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest} MutateAdGroupAdLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdLabelsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdLabelsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest} MutateAdGroupAdLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdLabelsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAdLabelsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAdLabelsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAdLabelsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest} MutateAdGroupAdLabelsRequest
                             */
                            MutateAdGroupAdLabelsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAdLabelsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest} m MutateAdGroupAdLabelsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAdLabelsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAdLabelsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAdLabelsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAdLabelsRequest;
                        })();
    
                        services.AdGroupAdLabelOperation = (function() {
    
                            /**
                             * Properties of an AdGroupAdLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupAdLabelOperation
                             * @property {google.ads.googleads.v11.resources.IAdGroupAdLabel|null} [create] AdGroupAdLabelOperation create
                             * @property {string|null} [remove] AdGroupAdLabelOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupAdLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupAdLabelOperation.
                             * @implements IAdGroupAdLabelOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupAdLabelOperation=} [p] Properties to set
                             */
                            function AdGroupAdLabelOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAdLabelOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAdLabel|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @instance
                             */
                            AdGroupAdLabelOperation.prototype.create = null;
    
                            /**
                             * AdGroupAdLabelOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @instance
                             */
                            AdGroupAdLabelOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupAdLabelOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdLabelOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupAdLabelOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAdLabelOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupAdLabelOperation} AdGroupAdLabelOperation instance
                             */
                            AdGroupAdLabelOperation.create = function create(properties) {
                                return new AdGroupAdLabelOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdLabelOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupAdLabelOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAdLabelOperation} m AdGroupAdLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdLabelOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAdLabel.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdLabelOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupAdLabelOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAdLabelOperation} message AdGroupAdLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdLabelOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdLabelOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupAdLabelOperation} AdGroupAdLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdLabelOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdLabelOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupAdLabelOperation} AdGroupAdLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdLabelOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdLabelOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdLabelOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdLabelOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupAdLabelOperation} AdGroupAdLabelOperation
                             */
                            AdGroupAdLabelOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAdLabelOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdLabelOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupAdLabelOperation} m AdGroupAdLabelOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdLabelOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAdLabelOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupAdLabelOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdLabelOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAdLabelOperation;
                        })();
    
                        services.MutateAdGroupAdLabelsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAdLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAdLabelsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupAdLabelsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult>|null} [results] MutateAdGroupAdLabelsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAdLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAdLabelsResponse.
                             * @implements IMutateAdGroupAdLabelsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupAdLabelsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAdLabelsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @instance
                             */
                            MutateAdGroupAdLabelsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupAdLabelsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @instance
                             */
                            MutateAdGroupAdLabelsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupAdLabelsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse} MutateAdGroupAdLabelsResponse instance
                             */
                            MutateAdGroupAdLabelsResponse.create = function create(properties) {
                                return new MutateAdGroupAdLabelsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdLabelsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsResponse} m MutateAdGroupAdLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdLabelsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdLabelsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelsResponse} message MutateAdGroupAdLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdLabelsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdLabelsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse} MutateAdGroupAdLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdLabelsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdLabelsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse} MutateAdGroupAdLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdLabelsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAdLabelsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAdLabelsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAdLabelsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse} MutateAdGroupAdLabelsResponse
                             */
                            MutateAdGroupAdLabelsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAdLabelsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse} m MutateAdGroupAdLabelsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAdLabelsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAdLabelsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAdLabelsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAdLabelsResponse;
                        })();
    
                        services.MutateAdGroupAdLabelResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAdLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAdLabelResult
                             * @property {string|null} [resource_name] MutateAdGroupAdLabelResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAdLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAdLabelResult.
                             * @implements IMutateAdGroupAdLabelResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult=} [p] Properties to set
                             */
                            function MutateAdGroupAdLabelResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAdLabelResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @instance
                             */
                            MutateAdGroupAdLabelResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateAdGroupAdLabelResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelResult} MutateAdGroupAdLabelResult instance
                             */
                            MutateAdGroupAdLabelResult.create = function create(properties) {
                                return new MutateAdGroupAdLabelResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdLabelResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult} m MutateAdGroupAdLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdLabelResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdLabelResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult} message MutateAdGroupAdLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdLabelResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdLabelResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelResult} MutateAdGroupAdLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdLabelResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdLabelResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelResult} MutateAdGroupAdLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdLabelResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAdLabelResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAdLabelResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAdLabelResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdLabelResult} MutateAdGroupAdLabelResult
                             */
                            MutateAdGroupAdLabelResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAdLabelResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdLabelResult} m MutateAdGroupAdLabelResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAdLabelResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAdLabelResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdLabelResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAdLabelResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAdLabelResult;
                        })();
    
                        services.AdGroupAdService = (function() {
    
                            /**
                             * Constructs a new AdGroupAdService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupAdService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupAdService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupAdService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupAdService;
    
                            /**
                             * Creates new AdGroupAdService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupAdService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupAdService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupAdService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupAdService|mutateAdGroupAds}.
                             * @memberof google.ads.googleads.v11.services.AdGroupAdService
                             * @typedef MutateAdGroupAdsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdsResponse} [response] MutateAdGroupAdsResponse
                             */
    
                            /**
                             * Calls MutateAdGroupAds.
                             * @function mutateAdGroupAds
                             * @memberof google.ads.googleads.v11.services.AdGroupAdService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsRequest} request MutateAdGroupAdsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupAdService.MutateAdGroupAdsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupAdsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupAdService.prototype.mutateAdGroupAds = function mutateAdGroupAds(request, callback) {
                                return this.rpcCall(mutateAdGroupAds, $root.google.ads.googleads.v11.services.MutateAdGroupAdsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupAdsResponse, request, callback);
                            }, "name", { value: "MutateAdGroupAds" });
    
                            /**
                             * Calls MutateAdGroupAds.
                             * @function mutateAdGroupAds
                             * @memberof google.ads.googleads.v11.services.AdGroupAdService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsRequest} request MutateAdGroupAdsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupAdsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupAdService;
                        })();
    
                        services.MutateAdGroupAdsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAdsRequest
                             * @property {string|null} [customer_id] MutateAdGroupAdsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupAdOperation>|null} [operations] MutateAdGroupAdsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupAdsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupAdsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupAdsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAdsRequest.
                             * @implements IMutateAdGroupAdsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupAdsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAdsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @instance
                             */
                            MutateAdGroupAdsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupAdsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupAdOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @instance
                             */
                            MutateAdGroupAdsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupAdsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @instance
                             */
                            MutateAdGroupAdsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupAdsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @instance
                             */
                            MutateAdGroupAdsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupAdsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @instance
                             */
                            MutateAdGroupAdsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupAdsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsRequest} MutateAdGroupAdsRequest instance
                             */
                            MutateAdGroupAdsRequest.create = function create(properties) {
                                return new MutateAdGroupAdsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsRequest} m MutateAdGroupAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupAdOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsRequest} message MutateAdGroupAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsRequest} MutateAdGroupAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupAdOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsRequest} MutateAdGroupAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAdsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAdsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupAdOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAdsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsRequest} MutateAdGroupAdsRequest
                             */
                            MutateAdGroupAdsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAdsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupAdOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAdsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdsRequest} m MutateAdGroupAdsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAdsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupAdOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAdsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAdsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAdsRequest;
                        })();
    
                        services.AdGroupAdOperation = (function() {
    
                            /**
                             * Properties of an AdGroupAdOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupAdOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdGroupAdOperation update_mask
                             * @property {google.ads.googleads.v11.common.IPolicyValidationParameter|null} [policy_validation_parameter] AdGroupAdOperation policy_validation_parameter
                             * @property {google.ads.googleads.v11.resources.IAdGroupAd|null} [create] AdGroupAdOperation create
                             * @property {google.ads.googleads.v11.resources.IAdGroupAd|null} [update] AdGroupAdOperation update
                             * @property {string|null} [remove] AdGroupAdOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupAdOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupAdOperation.
                             * @implements IAdGroupAdOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupAdOperation=} [p] Properties to set
                             */
                            function AdGroupAdOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAdOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @instance
                             */
                            AdGroupAdOperation.prototype.update_mask = null;
    
                            /**
                             * AdGroupAdOperation policy_validation_parameter.
                             * @member {google.ads.googleads.v11.common.IPolicyValidationParameter|null|undefined} policy_validation_parameter
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @instance
                             */
                            AdGroupAdOperation.prototype.policy_validation_parameter = null;
    
                            /**
                             * AdGroupAdOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAd|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @instance
                             */
                            AdGroupAdOperation.prototype.create = null;
    
                            /**
                             * AdGroupAdOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAd|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @instance
                             */
                            AdGroupAdOperation.prototype.update = null;
    
                            /**
                             * AdGroupAdOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @instance
                             */
                            AdGroupAdOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupAdOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupAdOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupAdOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAdOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupAdOperation} AdGroupAdOperation instance
                             */
                            AdGroupAdOperation.create = function create(properties) {
                                return new AdGroupAdOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAdOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupAdOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAdOperation} m AdGroupAdOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAd.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAd.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                if (m.policy_validation_parameter != null && Object.hasOwnProperty.call(m, "policy_validation_parameter"))
                                    $root.google.ads.googleads.v11.common.PolicyValidationParameter.encode(m.policy_validation_parameter, w.uint32(42).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAdOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupAdOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAdOperation} message AdGroupAdOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAdOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAdOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupAdOperation} AdGroupAdOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupAdOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.policy_validation_parameter = $root.google.ads.googleads.v11.common.PolicyValidationParameter.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupAd.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AdGroupAd.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAdOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupAdOperation} AdGroupAdOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAdOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAdOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAdOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.policy_validation_parameter != null && m.hasOwnProperty("policy_validation_parameter")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyValidationParameter.verify(m.policy_validation_parameter);
                                        if (e)
                                            return "policy_validation_parameter." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAd.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAd.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAdOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupAdOperation} AdGroupAdOperation
                             */
                            AdGroupAdOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupAdOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupAdOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAdOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.policy_validation_parameter != null) {
                                    if (typeof d.policy_validation_parameter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAdOperation.policy_validation_parameter: object expected");
                                    m.policy_validation_parameter = $root.google.ads.googleads.v11.common.PolicyValidationParameter.fromObject(d.policy_validation_parameter);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAdOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupAd.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAdOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdGroupAd.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAdOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupAdOperation} m AdGroupAdOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAdOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                    d.policy_validation_parameter = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupAd.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdGroupAd.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                if (m.policy_validation_parameter != null && m.hasOwnProperty("policy_validation_parameter")) {
                                    d.policy_validation_parameter = $root.google.ads.googleads.v11.common.PolicyValidationParameter.toObject(m.policy_validation_parameter, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAdOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupAdOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAdOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAdOperation;
                        })();
    
                        services.MutateAdGroupAdsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAdsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupAdsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupAdResult>|null} [results] MutateAdGroupAdsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAdsResponse.
                             * @implements IMutateAdGroupAdsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupAdsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAdsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @instance
                             */
                            MutateAdGroupAdsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupAdsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupAdResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @instance
                             */
                            MutateAdGroupAdsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupAdsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsResponse} MutateAdGroupAdsResponse instance
                             */
                            MutateAdGroupAdsResponse.create = function create(properties) {
                                return new MutateAdGroupAdsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsResponse} m MutateAdGroupAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdsResponse} message MutateAdGroupAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsResponse} MutateAdGroupAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupAdResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsResponse} MutateAdGroupAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAdsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAdsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAdsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdsResponse} MutateAdGroupAdsResponse
                             */
                            MutateAdGroupAdsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAdsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAdsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdsResponse} m MutateAdGroupAdsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAdsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAdsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAdsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAdsResponse;
                        })();
    
                        services.MutateAdGroupAdResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAdResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAdResult
                             * @property {string|null} [resource_name] MutateAdGroupAdResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupAd|null} [ad_group_ad] MutateAdGroupAdResult ad_group_ad
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAdResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAdResult.
                             * @implements IMutateAdGroupAdResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdResult=} [p] Properties to set
                             */
                            function MutateAdGroupAdResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAdResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @instance
                             */
                            MutateAdGroupAdResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupAdResult ad_group_ad.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAd|null|undefined} ad_group_ad
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @instance
                             */
                            MutateAdGroupAdResult.prototype.ad_group_ad = null;
    
                            /**
                             * Creates a new MutateAdGroupAdResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdResult} MutateAdGroupAdResult instance
                             */
                            MutateAdGroupAdResult.create = function create(properties) {
                                return new MutateAdGroupAdResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdResult} m MutateAdGroupAdResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAd.encode(m.ad_group_ad, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAdResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAdResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAdResult} message MutateAdGroupAdResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAdResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdResult} MutateAdGroupAdResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAdResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdResult} MutateAdGroupAdResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAdResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAdResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAdResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAd.verify(m.ad_group_ad);
                                        if (e)
                                            return "ad_group_ad." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAdResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAdResult} MutateAdGroupAdResult
                             */
                            MutateAdGroupAdResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAdResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAdResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_ad != null) {
                                    if (typeof d.ad_group_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAdResult.ad_group_ad: object expected");
                                    m.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.fromObject(d.ad_group_ad);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAdResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAdResult} m MutateAdGroupAdResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAdResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_ad = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    d.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.toObject(m.ad_group_ad, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAdResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAdResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAdResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAdResult;
                        })();
    
                        services.AdGroupAssetService = (function() {
    
                            /**
                             * Constructs a new AdGroupAssetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupAssetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupAssetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupAssetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupAssetService;
    
                            /**
                             * Creates new AdGroupAssetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupAssetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupAssetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupAssetService|mutateAdGroupAssets}.
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetService
                             * @typedef MutateAdGroupAssetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAssetsResponse} [response] MutateAdGroupAssetsResponse
                             */
    
                            /**
                             * Calls MutateAdGroupAssets.
                             * @function mutateAdGroupAssets
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsRequest} request MutateAdGroupAssetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupAssetService.MutateAdGroupAssetsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupAssetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupAssetService.prototype.mutateAdGroupAssets = function mutateAdGroupAssets(request, callback) {
                                return this.rpcCall(mutateAdGroupAssets, $root.google.ads.googleads.v11.services.MutateAdGroupAssetsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupAssetsResponse, request, callback);
                            }, "name", { value: "MutateAdGroupAssets" });
    
                            /**
                             * Calls MutateAdGroupAssets.
                             * @function mutateAdGroupAssets
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsRequest} request MutateAdGroupAssetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupAssetsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupAssetService;
                        })();
    
                        services.MutateAdGroupAssetsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAssetsRequest
                             * @property {string|null} [customer_id] MutateAdGroupAssetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupAssetOperation>|null} [operations] MutateAdGroupAssetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupAssetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupAssetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupAssetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAssetsRequest.
                             * @implements IMutateAdGroupAssetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupAssetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAssetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @instance
                             */
                            MutateAdGroupAssetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupAssetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupAssetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @instance
                             */
                            MutateAdGroupAssetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupAssetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @instance
                             */
                            MutateAdGroupAssetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupAssetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @instance
                             */
                            MutateAdGroupAssetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupAssetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @instance
                             */
                            MutateAdGroupAssetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupAssetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsRequest} MutateAdGroupAssetsRequest instance
                             */
                            MutateAdGroupAssetsRequest.create = function create(properties) {
                                return new MutateAdGroupAssetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAssetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAssetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsRequest} m MutateAdGroupAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAssetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupAssetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAssetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAssetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsRequest} message MutateAdGroupAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAssetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAssetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsRequest} MutateAdGroupAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAssetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAssetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupAssetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAssetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsRequest} MutateAdGroupAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAssetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAssetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAssetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupAssetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAssetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsRequest} MutateAdGroupAssetsRequest
                             */
                            MutateAdGroupAssetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAssetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAssetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAssetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAssetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupAssetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAssetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAssetsRequest} m MutateAdGroupAssetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAssetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupAssetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAssetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAssetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAssetsRequest;
                        })();
    
                        services.AdGroupAssetOperation = (function() {
    
                            /**
                             * Properties of an AdGroupAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupAssetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdGroupAssetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAdGroupAsset|null} [create] AdGroupAssetOperation create
                             * @property {google.ads.googleads.v11.resources.IAdGroupAsset|null} [update] AdGroupAssetOperation update
                             * @property {string|null} [remove] AdGroupAssetOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupAssetOperation.
                             * @implements IAdGroupAssetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupAssetOperation=} [p] Properties to set
                             */
                            function AdGroupAssetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupAssetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @instance
                             */
                            AdGroupAssetOperation.prototype.update_mask = null;
    
                            /**
                             * AdGroupAssetOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAsset|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @instance
                             */
                            AdGroupAssetOperation.prototype.create = null;
    
                            /**
                             * AdGroupAssetOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAsset|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @instance
                             */
                            AdGroupAssetOperation.prototype.update = null;
    
                            /**
                             * AdGroupAssetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @instance
                             */
                            AdGroupAssetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupAssetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupAssetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupAssetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAssetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupAssetOperation} AdGroupAssetOperation instance
                             */
                            AdGroupAssetOperation.create = function create(properties) {
                                return new AdGroupAssetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupAssetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupAssetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAssetOperation} m AdGroupAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAssetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAsset.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAsset.encode(m.update, w.uint32(26).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupAssetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupAssetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupAssetOperation} message AdGroupAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupAssetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupAssetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupAssetOperation} AdGroupAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAssetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupAssetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupAsset.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.update = $root.google.ads.googleads.v11.resources.AdGroupAsset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupAssetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupAssetOperation} AdGroupAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupAssetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupAssetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupAssetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAsset.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAsset.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupAssetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupAssetOperation} AdGroupAssetOperation
                             */
                            AdGroupAssetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupAssetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupAssetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAssetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAssetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupAsset.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupAssetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdGroupAsset.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupAssetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupAssetOperation} m AdGroupAssetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupAssetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupAsset.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdGroupAsset.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupAssetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupAssetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupAssetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupAssetOperation;
                        })();
    
                        services.MutateAdGroupAssetsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAssetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupAssetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupAssetResult>|null} [results] MutateAdGroupAssetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAssetsResponse.
                             * @implements IMutateAdGroupAssetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupAssetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAssetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @instance
                             */
                            MutateAdGroupAssetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupAssetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupAssetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @instance
                             */
                            MutateAdGroupAssetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupAssetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsResponse} MutateAdGroupAssetsResponse instance
                             */
                            MutateAdGroupAssetsResponse.create = function create(properties) {
                                return new MutateAdGroupAssetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAssetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAssetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsResponse} m MutateAdGroupAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAssetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAssetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAssetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetsResponse} message MutateAdGroupAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAssetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAssetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsResponse} MutateAdGroupAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAssetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAssetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAssetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsResponse} MutateAdGroupAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAssetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAssetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAssetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAssetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetsResponse} MutateAdGroupAssetsResponse
                             */
                            MutateAdGroupAssetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAssetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAssetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAssetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAssetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAssetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAssetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAssetsResponse} m MutateAdGroupAssetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAssetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAssetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAssetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAssetsResponse;
                        })();
    
                        services.MutateAdGroupAssetResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupAssetResult
                             * @property {string|null} [resource_name] MutateAdGroupAssetResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupAsset|null} [ad_group_asset] MutateAdGroupAssetResult ad_group_asset
                             */
    
                            /**
                             * Constructs a new MutateAdGroupAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupAssetResult.
                             * @implements IMutateAdGroupAssetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetResult=} [p] Properties to set
                             */
                            function MutateAdGroupAssetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupAssetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @instance
                             */
                            MutateAdGroupAssetResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupAssetResult ad_group_asset.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAsset|null|undefined} ad_group_asset
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @instance
                             */
                            MutateAdGroupAssetResult.prototype.ad_group_asset = null;
    
                            /**
                             * Creates a new MutateAdGroupAssetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetResult} MutateAdGroupAssetResult instance
                             */
                            MutateAdGroupAssetResult.create = function create(properties) {
                                return new MutateAdGroupAssetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAssetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAssetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetResult} m MutateAdGroupAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAssetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_asset != null && Object.hasOwnProperty.call(m, "ad_group_asset"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAsset.encode(m.ad_group_asset, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupAssetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupAssetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupAssetResult} message MutateAdGroupAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupAssetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupAssetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetResult} MutateAdGroupAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAssetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupAssetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetResult} MutateAdGroupAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupAssetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupAssetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupAssetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAsset.verify(m.ad_group_asset);
                                        if (e)
                                            return "ad_group_asset." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupAssetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupAssetResult} MutateAdGroupAssetResult
                             */
                            MutateAdGroupAssetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_asset != null) {
                                    if (typeof d.ad_group_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupAssetResult.ad_group_asset: object expected");
                                    m.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.fromObject(d.ad_group_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupAssetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupAssetResult} m MutateAdGroupAssetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupAssetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_asset = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                    d.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.toObject(m.ad_group_asset, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupAssetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupAssetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupAssetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupAssetResult;
                        })();
    
                        services.AdGroupBidModifierService = (function() {
    
                            /**
                             * Constructs a new AdGroupBidModifierService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupBidModifierService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupBidModifierService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupBidModifierService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupBidModifierService;
    
                            /**
                             * Creates new AdGroupBidModifierService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupBidModifierService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupBidModifierService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupBidModifierService|mutateAdGroupBidModifiers}.
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierService
                             * @typedef MutateAdGroupBidModifiersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse} [response] MutateAdGroupBidModifiersResponse
                             */
    
                            /**
                             * Calls MutateAdGroupBidModifiers.
                             * @function mutateAdGroupBidModifiers
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersRequest} request MutateAdGroupBidModifiersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupBidModifierService.MutateAdGroupBidModifiersCallback} callback Node-style callback called with the error, if any, and MutateAdGroupBidModifiersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupBidModifierService.prototype.mutateAdGroupBidModifiers = function mutateAdGroupBidModifiers(request, callback) {
                                return this.rpcCall(mutateAdGroupBidModifiers, $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest, $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse, request, callback);
                            }, "name", { value: "MutateAdGroupBidModifiers" });
    
                            /**
                             * Calls MutateAdGroupBidModifiers.
                             * @function mutateAdGroupBidModifiers
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersRequest} request MutateAdGroupBidModifiersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupBidModifierService;
                        })();
    
                        services.MutateAdGroupBidModifiersRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupBidModifiersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupBidModifiersRequest
                             * @property {string|null} [customer_id] MutateAdGroupBidModifiersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupBidModifierOperation>|null} [operations] MutateAdGroupBidModifiersRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupBidModifiersRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupBidModifiersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupBidModifiersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupBidModifiersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupBidModifiersRequest.
                             * @implements IMutateAdGroupBidModifiersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersRequest=} [p] Properties to set
                             */
                            function MutateAdGroupBidModifiersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupBidModifiersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @instance
                             */
                            MutateAdGroupBidModifiersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupBidModifiersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupBidModifierOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @instance
                             */
                            MutateAdGroupBidModifiersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupBidModifiersRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @instance
                             */
                            MutateAdGroupBidModifiersRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupBidModifiersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @instance
                             */
                            MutateAdGroupBidModifiersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupBidModifiersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @instance
                             */
                            MutateAdGroupBidModifiersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupBidModifiersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest} MutateAdGroupBidModifiersRequest instance
                             */
                            MutateAdGroupBidModifiersRequest.create = function create(properties) {
                                return new MutateAdGroupBidModifiersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupBidModifiersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersRequest} m MutateAdGroupBidModifiersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupBidModifiersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupBidModifiersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersRequest} message MutateAdGroupBidModifiersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupBidModifiersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupBidModifiersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest} MutateAdGroupBidModifiersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupBidModifiersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupBidModifiersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest} MutateAdGroupBidModifiersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupBidModifiersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupBidModifiersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupBidModifiersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupBidModifiersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest} MutateAdGroupBidModifiersRequest
                             */
                            MutateAdGroupBidModifiersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupBidModifiersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest} m MutateAdGroupBidModifiersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupBidModifiersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupBidModifiersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupBidModifiersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupBidModifiersRequest;
                        })();
    
                        services.AdGroupBidModifierOperation = (function() {
    
                            /**
                             * Properties of an AdGroupBidModifierOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupBidModifierOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdGroupBidModifierOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAdGroupBidModifier|null} [create] AdGroupBidModifierOperation create
                             * @property {google.ads.googleads.v11.resources.IAdGroupBidModifier|null} [update] AdGroupBidModifierOperation update
                             * @property {string|null} [remove] AdGroupBidModifierOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupBidModifierOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupBidModifierOperation.
                             * @implements IAdGroupBidModifierOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupBidModifierOperation=} [p] Properties to set
                             */
                            function AdGroupBidModifierOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupBidModifierOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @instance
                             */
                            AdGroupBidModifierOperation.prototype.update_mask = null;
    
                            /**
                             * AdGroupBidModifierOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupBidModifier|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @instance
                             */
                            AdGroupBidModifierOperation.prototype.create = null;
    
                            /**
                             * AdGroupBidModifierOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdGroupBidModifier|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @instance
                             */
                            AdGroupBidModifierOperation.prototype.update = null;
    
                            /**
                             * AdGroupBidModifierOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @instance
                             */
                            AdGroupBidModifierOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupBidModifierOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupBidModifierOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupBidModifierOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupBidModifierOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupBidModifierOperation} AdGroupBidModifierOperation instance
                             */
                            AdGroupBidModifierOperation.create = function create(properties) {
                                return new AdGroupBidModifierOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupBidModifierOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupBidModifierOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupBidModifierOperation} m AdGroupBidModifierOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupBidModifierOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupBidModifier.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdGroupBidModifier.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupBidModifierOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupBidModifierOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupBidModifierOperation} message AdGroupBidModifierOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupBidModifierOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupBidModifierOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupBidModifierOperation} AdGroupBidModifierOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupBidModifierOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupBidModifierOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupBidModifierOperation} AdGroupBidModifierOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupBidModifierOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupBidModifierOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupBidModifierOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupBidModifierOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupBidModifierOperation} AdGroupBidModifierOperation
                             */
                            AdGroupBidModifierOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupBidModifierOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupBidModifierOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupBidModifierOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupBidModifierOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupBidModifierOperation} m AdGroupBidModifierOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupBidModifierOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupBidModifierOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupBidModifierOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupBidModifierOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupBidModifierOperation;
                        })();
    
                        services.MutateAdGroupBidModifiersResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupBidModifiersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupBidModifiersResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupBidModifiersResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult>|null} [results] MutateAdGroupBidModifiersResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupBidModifiersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupBidModifiersResponse.
                             * @implements IMutateAdGroupBidModifiersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersResponse=} [p] Properties to set
                             */
                            function MutateAdGroupBidModifiersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupBidModifiersResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @instance
                             */
                            MutateAdGroupBidModifiersResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupBidModifiersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @instance
                             */
                            MutateAdGroupBidModifiersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupBidModifiersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse} MutateAdGroupBidModifiersResponse instance
                             */
                            MutateAdGroupBidModifiersResponse.create = function create(properties) {
                                return new MutateAdGroupBidModifiersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupBidModifiersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersResponse} m MutateAdGroupBidModifiersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupBidModifiersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupBidModifiersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifiersResponse} message MutateAdGroupBidModifiersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupBidModifiersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupBidModifiersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse} MutateAdGroupBidModifiersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupBidModifiersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupBidModifiersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse} MutateAdGroupBidModifiersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupBidModifiersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupBidModifiersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupBidModifiersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupBidModifiersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse} MutateAdGroupBidModifiersResponse
                             */
                            MutateAdGroupBidModifiersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupBidModifiersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse} m MutateAdGroupBidModifiersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupBidModifiersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupBidModifiersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifiersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupBidModifiersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupBidModifiersResponse;
                        })();
    
                        services.MutateAdGroupBidModifierResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupBidModifierResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupBidModifierResult
                             * @property {string|null} [resource_name] MutateAdGroupBidModifierResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupBidModifier|null} [ad_group_bid_modifier] MutateAdGroupBidModifierResult ad_group_bid_modifier
                             */
    
                            /**
                             * Constructs a new MutateAdGroupBidModifierResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupBidModifierResult.
                             * @implements IMutateAdGroupBidModifierResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult=} [p] Properties to set
                             */
                            function MutateAdGroupBidModifierResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupBidModifierResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @instance
                             */
                            MutateAdGroupBidModifierResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupBidModifierResult ad_group_bid_modifier.
                             * @member {google.ads.googleads.v11.resources.IAdGroupBidModifier|null|undefined} ad_group_bid_modifier
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @instance
                             */
                            MutateAdGroupBidModifierResult.prototype.ad_group_bid_modifier = null;
    
                            /**
                             * Creates a new MutateAdGroupBidModifierResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifierResult} MutateAdGroupBidModifierResult instance
                             */
                            MutateAdGroupBidModifierResult.create = function create(properties) {
                                return new MutateAdGroupBidModifierResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupBidModifierResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult} m MutateAdGroupBidModifierResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupBidModifierResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_bid_modifier != null && Object.hasOwnProperty.call(m, "ad_group_bid_modifier"))
                                    $root.google.ads.googleads.v11.resources.AdGroupBidModifier.encode(m.ad_group_bid_modifier, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupBidModifierResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult} message MutateAdGroupBidModifierResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupBidModifierResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupBidModifierResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifierResult} MutateAdGroupBidModifierResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupBidModifierResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupBidModifierResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifierResult} MutateAdGroupBidModifierResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupBidModifierResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupBidModifierResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupBidModifierResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.verify(m.ad_group_bid_modifier);
                                        if (e)
                                            return "ad_group_bid_modifier." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupBidModifierResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupBidModifierResult} MutateAdGroupBidModifierResult
                             */
                            MutateAdGroupBidModifierResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_bid_modifier != null) {
                                    if (typeof d.ad_group_bid_modifier !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.ad_group_bid_modifier: object expected");
                                    m.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.fromObject(d.ad_group_bid_modifier);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupBidModifierResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupBidModifierResult} m MutateAdGroupBidModifierResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupBidModifierResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_bid_modifier = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                    d.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.toObject(m.ad_group_bid_modifier, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupBidModifierResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupBidModifierResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupBidModifierResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupBidModifierResult;
                        })();
    
                        services.AdGroupCriterionCustomizerService = (function() {
    
                            /**
                             * Constructs a new AdGroupCriterionCustomizerService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCriterionCustomizerService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupCriterionCustomizerService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupCriterionCustomizerService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupCriterionCustomizerService;
    
                            /**
                             * Creates new AdGroupCriterionCustomizerService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupCriterionCustomizerService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupCriterionCustomizerService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupCriterionCustomizerService|mutateAdGroupCriterionCustomizers}.
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerService
                             * @typedef MutateAdGroupCriterionCustomizersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse} [response] MutateAdGroupCriterionCustomizersResponse
                             */
    
                            /**
                             * Calls MutateAdGroupCriterionCustomizers.
                             * @function mutateAdGroupCriterionCustomizers
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersRequest} request MutateAdGroupCriterionCustomizersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupCriterionCustomizerService.MutateAdGroupCriterionCustomizersCallback} callback Node-style callback called with the error, if any, and MutateAdGroupCriterionCustomizersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupCriterionCustomizerService.prototype.mutateAdGroupCriterionCustomizers = function mutateAdGroupCriterionCustomizers(request, callback) {
                                return this.rpcCall(mutateAdGroupCriterionCustomizers, $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest, $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse, request, callback);
                            }, "name", { value: "MutateAdGroupCriterionCustomizers" });
    
                            /**
                             * Calls MutateAdGroupCriterionCustomizers.
                             * @function mutateAdGroupCriterionCustomizers
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersRequest} request MutateAdGroupCriterionCustomizersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupCriterionCustomizerService;
                        })();
    
                        services.MutateAdGroupCriterionCustomizersRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriterionCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriterionCustomizersRequest
                             * @property {string|null} [customer_id] MutateAdGroupCriterionCustomizersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation>|null} [operations] MutateAdGroupCriterionCustomizersRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupCriterionCustomizersRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupCriterionCustomizersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupCriterionCustomizersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriterionCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriterionCustomizersRequest.
                             * @implements IMutateAdGroupCriterionCustomizersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersRequest=} [p] Properties to set
                             */
                            function MutateAdGroupCriterionCustomizersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriterionCustomizersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupCriterionCustomizersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupCriterionCustomizersRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizersRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupCriterionCustomizersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupCriterionCustomizersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupCriterionCustomizersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest} MutateAdGroupCriterionCustomizersRequest instance
                             */
                            MutateAdGroupCriterionCustomizersRequest.create = function create(properties) {
                                return new MutateAdGroupCriterionCustomizersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionCustomizersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersRequest} m MutateAdGroupCriterionCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionCustomizersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionCustomizersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersRequest} message MutateAdGroupCriterionCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionCustomizersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionCustomizersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest} MutateAdGroupCriterionCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionCustomizersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionCustomizersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest} MutateAdGroupCriterionCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionCustomizersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriterionCustomizersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriterionCustomizersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriterionCustomizersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest} MutateAdGroupCriterionCustomizersRequest
                             */
                            MutateAdGroupCriterionCustomizersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriterionCustomizersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest} m MutateAdGroupCriterionCustomizersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriterionCustomizersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriterionCustomizersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriterionCustomizersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriterionCustomizersRequest;
                        })();
    
                        services.AdGroupCriterionCustomizerOperation = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupCriterionCustomizerOperation
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer|null} [create] AdGroupCriterionCustomizerOperation create
                             * @property {string|null} [remove] AdGroupCriterionCustomizerOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCriterionCustomizerOperation.
                             * @implements IAdGroupCriterionCustomizerOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation=} [p] Properties to set
                             */
                            function AdGroupCriterionCustomizerOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCriterionCustomizerOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @instance
                             */
                            AdGroupCriterionCustomizerOperation.prototype.create = null;
    
                            /**
                             * AdGroupCriterionCustomizerOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @instance
                             */
                            AdGroupCriterionCustomizerOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCriterionCustomizerOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionCustomizerOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCriterionCustomizerOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation} AdGroupCriterionCustomizerOperation instance
                             */
                            AdGroupCriterionCustomizerOperation.create = function create(properties) {
                                return new AdGroupCriterionCustomizerOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionCustomizerOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation} m AdGroupCriterionCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionCustomizerOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionCustomizerOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation} message AdGroupCriterionCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionCustomizerOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionCustomizerOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation} AdGroupCriterionCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionCustomizerOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionCustomizerOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation} AdGroupCriterionCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionCustomizerOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionCustomizerOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionCustomizerOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionCustomizerOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation} AdGroupCriterionCustomizerOperation
                             */
                            AdGroupCriterionCustomizerOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionCustomizerOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation} m AdGroupCriterionCustomizerOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionCustomizerOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCriterionCustomizerOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionCustomizerOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCriterionCustomizerOperation;
                        })();
    
                        services.MutateAdGroupCriterionCustomizersResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriterionCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriterionCustomizersResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult>|null} [results] MutateAdGroupCriterionCustomizersResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupCriterionCustomizersResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriterionCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriterionCustomizersResponse.
                             * @implements IMutateAdGroupCriterionCustomizersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersResponse=} [p] Properties to set
                             */
                            function MutateAdGroupCriterionCustomizersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriterionCustomizersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAdGroupCriterionCustomizersResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizersResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAdGroupCriterionCustomizersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse} MutateAdGroupCriterionCustomizersResponse instance
                             */
                            MutateAdGroupCriterionCustomizersResponse.create = function create(properties) {
                                return new MutateAdGroupCriterionCustomizersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionCustomizersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersResponse} m MutateAdGroupCriterionCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionCustomizersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionCustomizersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizersResponse} message MutateAdGroupCriterionCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionCustomizersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionCustomizersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse} MutateAdGroupCriterionCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionCustomizersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionCustomizersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse} MutateAdGroupCriterionCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionCustomizersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriterionCustomizersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriterionCustomizersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriterionCustomizersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse} MutateAdGroupCriterionCustomizersResponse
                             */
                            MutateAdGroupCriterionCustomizersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriterionCustomizersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse} m MutateAdGroupCriterionCustomizersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriterionCustomizersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriterionCustomizersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriterionCustomizersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriterionCustomizersResponse;
                        })();
    
                        services.MutateAdGroupCriterionCustomizerResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriterionCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriterionCustomizerResult
                             * @property {string|null} [resource_name] MutateAdGroupCriterionCustomizerResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer|null} [ad_group_criterion_customizer] MutateAdGroupCriterionCustomizerResult ad_group_criterion_customizer
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriterionCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriterionCustomizerResult.
                             * @implements IMutateAdGroupCriterionCustomizerResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult=} [p] Properties to set
                             */
                            function MutateAdGroupCriterionCustomizerResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriterionCustomizerResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizerResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupCriterionCustomizerResult ad_group_criterion_customizer.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer|null|undefined} ad_group_criterion_customizer
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @instance
                             */
                            MutateAdGroupCriterionCustomizerResult.prototype.ad_group_criterion_customizer = null;
    
                            /**
                             * Creates a new MutateAdGroupCriterionCustomizerResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult} MutateAdGroupCriterionCustomizerResult instance
                             */
                            MutateAdGroupCriterionCustomizerResult.create = function create(properties) {
                                return new MutateAdGroupCriterionCustomizerResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionCustomizerResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult} m MutateAdGroupCriterionCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionCustomizerResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_criterion_customizer != null && Object.hasOwnProperty.call(m, "ad_group_criterion_customizer"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.encode(m.ad_group_criterion_customizer, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionCustomizerResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult} message MutateAdGroupCriterionCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionCustomizerResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionCustomizerResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult} MutateAdGroupCriterionCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionCustomizerResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_criterion_customizer = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionCustomizerResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult} MutateAdGroupCriterionCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionCustomizerResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriterionCustomizerResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriterionCustomizerResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_criterion_customizer != null && m.hasOwnProperty("ad_group_criterion_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.verify(m.ad_group_criterion_customizer);
                                        if (e)
                                            return "ad_group_criterion_customizer." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriterionCustomizerResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult} MutateAdGroupCriterionCustomizerResult
                             */
                            MutateAdGroupCriterionCustomizerResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_criterion_customizer != null) {
                                    if (typeof d.ad_group_criterion_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.ad_group_criterion_customizer: object expected");
                                    m.ad_group_criterion_customizer = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.fromObject(d.ad_group_criterion_customizer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriterionCustomizerResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult} m MutateAdGroupCriterionCustomizerResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriterionCustomizerResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_criterion_customizer = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_criterion_customizer != null && m.hasOwnProperty("ad_group_criterion_customizer")) {
                                    d.ad_group_criterion_customizer = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.toObject(m.ad_group_criterion_customizer, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriterionCustomizerResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriterionCustomizerResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriterionCustomizerResult;
                        })();
    
                        services.AdGroupCriterionLabelService = (function() {
    
                            /**
                             * Constructs a new AdGroupCriterionLabelService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCriterionLabelService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupCriterionLabelService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupCriterionLabelService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupCriterionLabelService;
    
                            /**
                             * Creates new AdGroupCriterionLabelService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupCriterionLabelService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupCriterionLabelService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupCriterionLabelService|mutateAdGroupCriterionLabels}.
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelService
                             * @typedef MutateAdGroupCriterionLabelsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse} [response] MutateAdGroupCriterionLabelsResponse
                             */
    
                            /**
                             * Calls MutateAdGroupCriterionLabels.
                             * @function mutateAdGroupCriterionLabels
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsRequest} request MutateAdGroupCriterionLabelsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupCriterionLabelService.MutateAdGroupCriterionLabelsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupCriterionLabelsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupCriterionLabelService.prototype.mutateAdGroupCriterionLabels = function mutateAdGroupCriterionLabels(request, callback) {
                                return this.rpcCall(mutateAdGroupCriterionLabels, $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse, request, callback);
                            }, "name", { value: "MutateAdGroupCriterionLabels" });
    
                            /**
                             * Calls MutateAdGroupCriterionLabels.
                             * @function mutateAdGroupCriterionLabels
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsRequest} request MutateAdGroupCriterionLabelsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupCriterionLabelService;
                        })();
    
                        services.MutateAdGroupCriterionLabelsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriterionLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriterionLabelsRequest
                             * @property {string|null} [customer_id] MutateAdGroupCriterionLabelsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation>|null} [operations] MutateAdGroupCriterionLabelsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupCriterionLabelsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupCriterionLabelsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriterionLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriterionLabelsRequest.
                             * @implements IMutateAdGroupCriterionLabelsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupCriterionLabelsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriterionLabelsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @instance
                             */
                            MutateAdGroupCriterionLabelsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupCriterionLabelsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @instance
                             */
                            MutateAdGroupCriterionLabelsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupCriterionLabelsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @instance
                             */
                            MutateAdGroupCriterionLabelsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupCriterionLabelsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @instance
                             */
                            MutateAdGroupCriterionLabelsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAdGroupCriterionLabelsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest} MutateAdGroupCriterionLabelsRequest instance
                             */
                            MutateAdGroupCriterionLabelsRequest.create = function create(properties) {
                                return new MutateAdGroupCriterionLabelsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionLabelsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsRequest} m MutateAdGroupCriterionLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionLabelsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionLabelsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsRequest} message MutateAdGroupCriterionLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionLabelsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionLabelsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest} MutateAdGroupCriterionLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionLabelsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionLabelsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest} MutateAdGroupCriterionLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionLabelsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriterionLabelsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriterionLabelsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriterionLabelsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest} MutateAdGroupCriterionLabelsRequest
                             */
                            MutateAdGroupCriterionLabelsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriterionLabelsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest} m MutateAdGroupCriterionLabelsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriterionLabelsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriterionLabelsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriterionLabelsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriterionLabelsRequest;
                        })();
    
                        services.AdGroupCriterionLabelOperation = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupCriterionLabelOperation
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterionLabel|null} [create] AdGroupCriterionLabelOperation create
                             * @property {string|null} [remove] AdGroupCriterionLabelOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCriterionLabelOperation.
                             * @implements IAdGroupCriterionLabelOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation=} [p] Properties to set
                             */
                            function AdGroupCriterionLabelOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCriterionLabelOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterionLabel|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @instance
                             */
                            AdGroupCriterionLabelOperation.prototype.create = null;
    
                            /**
                             * AdGroupCriterionLabelOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @instance
                             */
                            AdGroupCriterionLabelOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCriterionLabelOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionLabelOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCriterionLabelOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionLabelOperation} AdGroupCriterionLabelOperation instance
                             */
                            AdGroupCriterionLabelOperation.create = function create(properties) {
                                return new AdGroupCriterionLabelOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionLabelOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation} m AdGroupCriterionLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionLabelOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionLabelOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation} message AdGroupCriterionLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionLabelOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionLabelOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionLabelOperation} AdGroupCriterionLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionLabelOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionLabelOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionLabelOperation} AdGroupCriterionLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionLabelOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionLabelOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionLabelOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionLabelOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionLabelOperation} AdGroupCriterionLabelOperation
                             */
                            AdGroupCriterionLabelOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionLabelOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupCriterionLabelOperation} m AdGroupCriterionLabelOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionLabelOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCriterionLabelOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionLabelOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionLabelOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCriterionLabelOperation;
                        })();
    
                        services.MutateAdGroupCriterionLabelsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriterionLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriterionLabelsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupCriterionLabelsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult>|null} [results] MutateAdGroupCriterionLabelsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriterionLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriterionLabelsResponse.
                             * @implements IMutateAdGroupCriterionLabelsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupCriterionLabelsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriterionLabelsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @instance
                             */
                            MutateAdGroupCriterionLabelsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupCriterionLabelsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @instance
                             */
                            MutateAdGroupCriterionLabelsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupCriterionLabelsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse} MutateAdGroupCriterionLabelsResponse instance
                             */
                            MutateAdGroupCriterionLabelsResponse.create = function create(properties) {
                                return new MutateAdGroupCriterionLabelsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionLabelsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsResponse} m MutateAdGroupCriterionLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionLabelsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionLabelsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelsResponse} message MutateAdGroupCriterionLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionLabelsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionLabelsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse} MutateAdGroupCriterionLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionLabelsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionLabelsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse} MutateAdGroupCriterionLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionLabelsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriterionLabelsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriterionLabelsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriterionLabelsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse} MutateAdGroupCriterionLabelsResponse
                             */
                            MutateAdGroupCriterionLabelsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriterionLabelsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse} m MutateAdGroupCriterionLabelsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriterionLabelsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriterionLabelsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriterionLabelsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriterionLabelsResponse;
                        })();
    
                        services.MutateAdGroupCriterionLabelResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriterionLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriterionLabelResult
                             * @property {string|null} [resource_name] MutateAdGroupCriterionLabelResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriterionLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriterionLabelResult.
                             * @implements IMutateAdGroupCriterionLabelResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult=} [p] Properties to set
                             */
                            function MutateAdGroupCriterionLabelResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriterionLabelResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @instance
                             */
                            MutateAdGroupCriterionLabelResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateAdGroupCriterionLabelResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult} MutateAdGroupCriterionLabelResult instance
                             */
                            MutateAdGroupCriterionLabelResult.create = function create(properties) {
                                return new MutateAdGroupCriterionLabelResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionLabelResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult} m MutateAdGroupCriterionLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionLabelResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionLabelResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult} message MutateAdGroupCriterionLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionLabelResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionLabelResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult} MutateAdGroupCriterionLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionLabelResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionLabelResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult} MutateAdGroupCriterionLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionLabelResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriterionLabelResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriterionLabelResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriterionLabelResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult} MutateAdGroupCriterionLabelResult
                             */
                            MutateAdGroupCriterionLabelResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriterionLabelResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult} m MutateAdGroupCriterionLabelResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriterionLabelResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriterionLabelResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriterionLabelResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriterionLabelResult;
                        })();
    
                        services.AdGroupCriterionService = (function() {
    
                            /**
                             * Constructs a new AdGroupCriterionService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCriterionService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupCriterionService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupCriterionService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupCriterionService;
    
                            /**
                             * Creates new AdGroupCriterionService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupCriterionService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupCriterionService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupCriterionService|mutateAdGroupCriteria}.
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionService
                             * @typedef MutateAdGroupCriteriaCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse} [response] MutateAdGroupCriteriaResponse
                             */
    
                            /**
                             * Calls MutateAdGroupCriteria.
                             * @function mutateAdGroupCriteria
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaRequest} request MutateAdGroupCriteriaRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupCriterionService.MutateAdGroupCriteriaCallback} callback Node-style callback called with the error, if any, and MutateAdGroupCriteriaResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupCriterionService.prototype.mutateAdGroupCriteria = function mutateAdGroupCriteria(request, callback) {
                                return this.rpcCall(mutateAdGroupCriteria, $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest, $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse, request, callback);
                            }, "name", { value: "MutateAdGroupCriteria" });
    
                            /**
                             * Calls MutateAdGroupCriteria.
                             * @function mutateAdGroupCriteria
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaRequest} request MutateAdGroupCriteriaRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupCriterionService;
                        })();
    
                        services.MutateAdGroupCriteriaRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriteriaRequest
                             * @property {string|null} [customer_id] MutateAdGroupCriteriaRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupCriterionOperation>|null} [operations] MutateAdGroupCriteriaRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupCriteriaRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupCriteriaRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupCriteriaRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriteriaRequest.
                             * @implements IMutateAdGroupCriteriaRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaRequest=} [p] Properties to set
                             */
                            function MutateAdGroupCriteriaRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriteriaRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @instance
                             */
                            MutateAdGroupCriteriaRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupCriteriaRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupCriterionOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @instance
                             */
                            MutateAdGroupCriteriaRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupCriteriaRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @instance
                             */
                            MutateAdGroupCriteriaRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupCriteriaRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @instance
                             */
                            MutateAdGroupCriteriaRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupCriteriaRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @instance
                             */
                            MutateAdGroupCriteriaRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupCriteriaRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest} MutateAdGroupCriteriaRequest instance
                             */
                            MutateAdGroupCriteriaRequest.create = function create(properties) {
                                return new MutateAdGroupCriteriaRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriteriaRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaRequest} m MutateAdGroupCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriteriaRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriteriaRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaRequest} message MutateAdGroupCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriteriaRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriteriaRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest} MutateAdGroupCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriteriaRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupCriterionOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriteriaRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest} MutateAdGroupCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriteriaRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriteriaRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriteriaRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriteriaRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest} MutateAdGroupCriteriaRequest
                             */
                            MutateAdGroupCriteriaRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriteriaRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest} m MutateAdGroupCriteriaRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriteriaRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriteriaRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriteriaRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriteriaRequest;
                        })();
    
                        services.AdGroupCriterionOperation = (function() {
    
                            /**
                             * Properties of an AdGroupCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupCriterionOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdGroupCriterionOperation update_mask
                             * @property {Array.<google.ads.googleads.v11.common.IPolicyViolationKey>|null} [exempt_policy_violation_keys] AdGroupCriterionOperation exempt_policy_violation_keys
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterion|null} [create] AdGroupCriterionOperation create
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterion|null} [update] AdGroupCriterionOperation update
                             * @property {string|null} [remove] AdGroupCriterionOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCriterionOperation.
                             * @implements IAdGroupCriterionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionOperation=} [p] Properties to set
                             */
                            function AdGroupCriterionOperation(p) {
                                this.exempt_policy_violation_keys = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCriterionOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @instance
                             */
                            AdGroupCriterionOperation.prototype.update_mask = null;
    
                            /**
                             * AdGroupCriterionOperation exempt_policy_violation_keys.
                             * @member {Array.<google.ads.googleads.v11.common.IPolicyViolationKey>} exempt_policy_violation_keys
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @instance
                             */
                            AdGroupCriterionOperation.prototype.exempt_policy_violation_keys = $util.emptyArray;
    
                            /**
                             * AdGroupCriterionOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterion|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @instance
                             */
                            AdGroupCriterionOperation.prototype.create = null;
    
                            /**
                             * AdGroupCriterionOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterion|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @instance
                             */
                            AdGroupCriterionOperation.prototype.update = null;
    
                            /**
                             * AdGroupCriterionOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @instance
                             */
                            AdGroupCriterionOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCriterionOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCriterionOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCriterionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionOperation} AdGroupCriterionOperation instance
                             */
                            AdGroupCriterionOperation.create = function create(properties) {
                                return new AdGroupCriterionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCriterionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionOperation} m AdGroupCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterion.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterion.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                if (m.exempt_policy_violation_keys != null && m.exempt_policy_violation_keys.length) {
                                    for (var i = 0; i < m.exempt_policy_violation_keys.length; ++i)
                                        $root.google.ads.googleads.v11.common.PolicyViolationKey.encode(m.exempt_policy_violation_keys[i], w.uint32(42).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCriterionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCriterionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCriterionOperation} message AdGroupCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCriterionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCriterionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionOperation} AdGroupCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupCriterionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        if (!(m.exempt_policy_violation_keys && m.exempt_policy_violation_keys.length))
                                            m.exempt_policy_violation_keys = [];
                                        m.exempt_policy_violation_keys.push($root.google.ads.googleads.v11.common.PolicyViolationKey.decode(r, r.uint32()));
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupCriterion.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AdGroupCriterion.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCriterionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionOperation} AdGroupCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCriterionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCriterionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCriterionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.exempt_policy_violation_keys != null && m.hasOwnProperty("exempt_policy_violation_keys")) {
                                    if (!Array.isArray(m.exempt_policy_violation_keys))
                                        return "exempt_policy_violation_keys: array expected";
                                    for (var i = 0; i < m.exempt_policy_violation_keys.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.PolicyViolationKey.verify(m.exempt_policy_violation_keys[i]);
                                            if (e)
                                                return "exempt_policy_violation_keys." + e;
                                        }
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterion.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterion.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCriterionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupCriterionOperation} AdGroupCriterionOperation
                             */
                            AdGroupCriterionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupCriterionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupCriterionOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupCriterionOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.exempt_policy_violation_keys) {
                                    if (!Array.isArray(d.exempt_policy_violation_keys))
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupCriterionOperation.exempt_policy_violation_keys: array expected");
                                    m.exempt_policy_violation_keys = [];
                                    for (var i = 0; i < d.exempt_policy_violation_keys.length; ++i) {
                                        if (typeof d.exempt_policy_violation_keys[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.AdGroupCriterionOperation.exempt_policy_violation_keys: object expected");
                                        m.exempt_policy_violation_keys[i] = $root.google.ads.googleads.v11.common.PolicyViolationKey.fromObject(d.exempt_policy_violation_keys[i]);
                                    }
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupCriterionOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupCriterion.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupCriterionOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdGroupCriterion.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCriterionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupCriterionOperation} m AdGroupCriterionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCriterionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.exempt_policy_violation_keys = [];
                                }
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupCriterion.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdGroupCriterion.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                if (m.exempt_policy_violation_keys && m.exempt_policy_violation_keys.length) {
                                    d.exempt_policy_violation_keys = [];
                                    for (var j = 0; j < m.exempt_policy_violation_keys.length; ++j) {
                                        d.exempt_policy_violation_keys[j] = $root.google.ads.googleads.v11.common.PolicyViolationKey.toObject(m.exempt_policy_violation_keys[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCriterionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupCriterionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCriterionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCriterionOperation;
                        })();
    
                        services.MutateAdGroupCriteriaResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriteriaResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupCriteriaResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupCriterionResult>|null} [results] MutateAdGroupCriteriaResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriteriaResponse.
                             * @implements IMutateAdGroupCriteriaResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaResponse=} [p] Properties to set
                             */
                            function MutateAdGroupCriteriaResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriteriaResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @instance
                             */
                            MutateAdGroupCriteriaResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupCriteriaResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupCriterionResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @instance
                             */
                            MutateAdGroupCriteriaResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupCriteriaResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse} MutateAdGroupCriteriaResponse instance
                             */
                            MutateAdGroupCriteriaResponse.create = function create(properties) {
                                return new MutateAdGroupCriteriaResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriteriaResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaResponse} m MutateAdGroupCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriteriaResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriteriaResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriteriaResponse} message MutateAdGroupCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriteriaResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriteriaResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse} MutateAdGroupCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriteriaResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriteriaResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse} MutateAdGroupCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriteriaResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriteriaResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriteriaResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriteriaResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse} MutateAdGroupCriteriaResponse
                             */
                            MutateAdGroupCriteriaResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriteriaResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse} m MutateAdGroupCriteriaResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriteriaResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriteriaResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriteriaResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriteriaResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriteriaResponse;
                        })();
    
                        services.MutateAdGroupCriterionResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCriterionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCriterionResult
                             * @property {string|null} [resource_name] MutateAdGroupCriterionResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterion|null} [ad_group_criterion] MutateAdGroupCriterionResult ad_group_criterion
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCriterionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCriterionResult.
                             * @implements IMutateAdGroupCriterionResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionResult=} [p] Properties to set
                             */
                            function MutateAdGroupCriterionResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCriterionResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @instance
                             */
                            MutateAdGroupCriterionResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupCriterionResult ad_group_criterion.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterion|null|undefined} ad_group_criterion
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @instance
                             */
                            MutateAdGroupCriterionResult.prototype.ad_group_criterion = null;
    
                            /**
                             * Creates a new MutateAdGroupCriterionResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionResult} MutateAdGroupCriterionResult instance
                             */
                            MutateAdGroupCriterionResult.create = function create(properties) {
                                return new MutateAdGroupCriterionResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionResult} m MutateAdGroupCriterionResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterion.encode(m.ad_group_criterion, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCriterionResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCriterionResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCriterionResult} message MutateAdGroupCriterionResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCriterionResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionResult} MutateAdGroupCriterionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCriterionResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionResult} MutateAdGroupCriterionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCriterionResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCriterionResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCriterionResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterion.verify(m.ad_group_criterion);
                                        if (e)
                                            return "ad_group_criterion." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCriterionResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCriterionResult} MutateAdGroupCriterionResult
                             */
                            MutateAdGroupCriterionResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_criterion != null) {
                                    if (typeof d.ad_group_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCriterionResult.ad_group_criterion: object expected");
                                    m.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.fromObject(d.ad_group_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCriterionResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCriterionResult} m MutateAdGroupCriterionResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCriterionResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_criterion = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    d.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.toObject(m.ad_group_criterion, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCriterionResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCriterionResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCriterionResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCriterionResult;
                        })();
    
                        services.AdGroupCustomizerService = (function() {
    
                            /**
                             * Constructs a new AdGroupCustomizerService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCustomizerService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupCustomizerService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupCustomizerService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupCustomizerService;
    
                            /**
                             * Creates new AdGroupCustomizerService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupCustomizerService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupCustomizerService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupCustomizerService|mutateAdGroupCustomizers}.
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerService
                             * @typedef MutateAdGroupCustomizersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse} [response] MutateAdGroupCustomizersResponse
                             */
    
                            /**
                             * Calls MutateAdGroupCustomizers.
                             * @function mutateAdGroupCustomizers
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersRequest} request MutateAdGroupCustomizersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupCustomizerService.MutateAdGroupCustomizersCallback} callback Node-style callback called with the error, if any, and MutateAdGroupCustomizersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupCustomizerService.prototype.mutateAdGroupCustomizers = function mutateAdGroupCustomizers(request, callback) {
                                return this.rpcCall(mutateAdGroupCustomizers, $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest, $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse, request, callback);
                            }, "name", { value: "MutateAdGroupCustomizers" });
    
                            /**
                             * Calls MutateAdGroupCustomizers.
                             * @function mutateAdGroupCustomizers
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersRequest} request MutateAdGroupCustomizersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupCustomizerService;
                        })();
    
                        services.MutateAdGroupCustomizersRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCustomizersRequest
                             * @property {string|null} [customer_id] MutateAdGroupCustomizersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupCustomizerOperation>|null} [operations] MutateAdGroupCustomizersRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupCustomizersRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupCustomizersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupCustomizersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCustomizersRequest.
                             * @implements IMutateAdGroupCustomizersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersRequest=} [p] Properties to set
                             */
                            function MutateAdGroupCustomizersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCustomizersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCustomizersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupCustomizersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupCustomizerOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCustomizersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupCustomizersRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCustomizersRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupCustomizersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCustomizersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupCustomizersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @instance
                             */
                            MutateAdGroupCustomizersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupCustomizersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest} MutateAdGroupCustomizersRequest instance
                             */
                            MutateAdGroupCustomizersRequest.create = function create(properties) {
                                return new MutateAdGroupCustomizersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCustomizersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersRequest} m MutateAdGroupCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCustomizersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCustomizersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersRequest} message MutateAdGroupCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCustomizersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCustomizersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest} MutateAdGroupCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCustomizersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCustomizersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest} MutateAdGroupCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCustomizersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCustomizersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCustomizersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCustomizersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest} MutateAdGroupCustomizersRequest
                             */
                            MutateAdGroupCustomizersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCustomizersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest} m MutateAdGroupCustomizersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCustomizersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCustomizersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCustomizersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCustomizersRequest;
                        })();
    
                        services.AdGroupCustomizerOperation = (function() {
    
                            /**
                             * Properties of an AdGroupCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupCustomizerOperation
                             * @property {google.ads.googleads.v11.resources.IAdGroupCustomizer|null} [create] AdGroupCustomizerOperation create
                             * @property {string|null} [remove] AdGroupCustomizerOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupCustomizerOperation.
                             * @implements IAdGroupCustomizerOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupCustomizerOperation=} [p] Properties to set
                             */
                            function AdGroupCustomizerOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupCustomizerOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCustomizer|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @instance
                             */
                            AdGroupCustomizerOperation.prototype.create = null;
    
                            /**
                             * AdGroupCustomizerOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @instance
                             */
                            AdGroupCustomizerOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupCustomizerOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupCustomizerOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupCustomizerOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCustomizerOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupCustomizerOperation} AdGroupCustomizerOperation instance
                             */
                            AdGroupCustomizerOperation.create = function create(properties) {
                                return new AdGroupCustomizerOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupCustomizerOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCustomizerOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCustomizerOperation} m AdGroupCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCustomizerOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCustomizer.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupCustomizerOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupCustomizerOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupCustomizerOperation} message AdGroupCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupCustomizerOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupCustomizerOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupCustomizerOperation} AdGroupCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCustomizerOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupCustomizerOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupCustomizerOperation} AdGroupCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupCustomizerOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupCustomizerOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupCustomizerOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupCustomizerOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupCustomizerOperation} AdGroupCustomizerOperation
                             */
                            AdGroupCustomizerOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupCustomizerOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupCustomizerOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupCustomizerOperation} m AdGroupCustomizerOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupCustomizerOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupCustomizerOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupCustomizerOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupCustomizerOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupCustomizerOperation;
                        })();
    
                        services.MutateAdGroupCustomizersResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCustomizersResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult>|null} [results] MutateAdGroupCustomizersResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupCustomizersResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCustomizersResponse.
                             * @implements IMutateAdGroupCustomizersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersResponse=} [p] Properties to set
                             */
                            function MutateAdGroupCustomizersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCustomizersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @instance
                             */
                            MutateAdGroupCustomizersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAdGroupCustomizersResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @instance
                             */
                            MutateAdGroupCustomizersResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAdGroupCustomizersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse} MutateAdGroupCustomizersResponse instance
                             */
                            MutateAdGroupCustomizersResponse.create = function create(properties) {
                                return new MutateAdGroupCustomizersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCustomizersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersResponse} m MutateAdGroupCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCustomizersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCustomizersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizersResponse} message MutateAdGroupCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCustomizersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCustomizersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse} MutateAdGroupCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCustomizersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCustomizersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse} MutateAdGroupCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCustomizersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCustomizersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCustomizersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCustomizersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse} MutateAdGroupCustomizersResponse
                             */
                            MutateAdGroupCustomizersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCustomizersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse} m MutateAdGroupCustomizersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCustomizersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCustomizersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCustomizersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCustomizersResponse;
                        })();
    
                        services.MutateAdGroupCustomizerResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupCustomizerResult
                             * @property {string|null} [resource_name] MutateAdGroupCustomizerResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupCustomizer|null} [ad_group_customizer] MutateAdGroupCustomizerResult ad_group_customizer
                             */
    
                            /**
                             * Constructs a new MutateAdGroupCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupCustomizerResult.
                             * @implements IMutateAdGroupCustomizerResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult=} [p] Properties to set
                             */
                            function MutateAdGroupCustomizerResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupCustomizerResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @instance
                             */
                            MutateAdGroupCustomizerResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupCustomizerResult ad_group_customizer.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCustomizer|null|undefined} ad_group_customizer
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @instance
                             */
                            MutateAdGroupCustomizerResult.prototype.ad_group_customizer = null;
    
                            /**
                             * Creates a new MutateAdGroupCustomizerResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizerResult} MutateAdGroupCustomizerResult instance
                             */
                            MutateAdGroupCustomizerResult.create = function create(properties) {
                                return new MutateAdGroupCustomizerResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCustomizerResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult} m MutateAdGroupCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCustomizerResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_customizer != null && Object.hasOwnProperty.call(m, "ad_group_customizer"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCustomizer.encode(m.ad_group_customizer, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupCustomizerResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult} message MutateAdGroupCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupCustomizerResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupCustomizerResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizerResult} MutateAdGroupCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCustomizerResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_customizer = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupCustomizerResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizerResult} MutateAdGroupCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupCustomizerResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupCustomizerResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupCustomizerResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_customizer != null && m.hasOwnProperty("ad_group_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.verify(m.ad_group_customizer);
                                        if (e)
                                            return "ad_group_customizer." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupCustomizerResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupCustomizerResult} MutateAdGroupCustomizerResult
                             */
                            MutateAdGroupCustomizerResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_customizer != null) {
                                    if (typeof d.ad_group_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.ad_group_customizer: object expected");
                                    m.ad_group_customizer = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.fromObject(d.ad_group_customizer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupCustomizerResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupCustomizerResult} m MutateAdGroupCustomizerResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupCustomizerResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_customizer = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_customizer != null && m.hasOwnProperty("ad_group_customizer")) {
                                    d.ad_group_customizer = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.toObject(m.ad_group_customizer, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupCustomizerResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupCustomizerResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupCustomizerResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupCustomizerResult;
                        })();
    
                        services.AdGroupExtensionSettingService = (function() {
    
                            /**
                             * Constructs a new AdGroupExtensionSettingService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupExtensionSettingService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupExtensionSettingService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupExtensionSettingService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupExtensionSettingService;
    
                            /**
                             * Creates new AdGroupExtensionSettingService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupExtensionSettingService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupExtensionSettingService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupExtensionSettingService|mutateAdGroupExtensionSettings}.
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingService
                             * @typedef MutateAdGroupExtensionSettingsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse} [response] MutateAdGroupExtensionSettingsResponse
                             */
    
                            /**
                             * Calls MutateAdGroupExtensionSettings.
                             * @function mutateAdGroupExtensionSettings
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsRequest} request MutateAdGroupExtensionSettingsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupExtensionSettingService.MutateAdGroupExtensionSettingsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupExtensionSettingsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupExtensionSettingService.prototype.mutateAdGroupExtensionSettings = function mutateAdGroupExtensionSettings(request, callback) {
                                return this.rpcCall(mutateAdGroupExtensionSettings, $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse, request, callback);
                            }, "name", { value: "MutateAdGroupExtensionSettings" });
    
                            /**
                             * Calls MutateAdGroupExtensionSettings.
                             * @function mutateAdGroupExtensionSettings
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsRequest} request MutateAdGroupExtensionSettingsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupExtensionSettingService;
                        })();
    
                        services.MutateAdGroupExtensionSettingsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupExtensionSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupExtensionSettingsRequest
                             * @property {string|null} [customer_id] MutateAdGroupExtensionSettingsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation>|null} [operations] MutateAdGroupExtensionSettingsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupExtensionSettingsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupExtensionSettingsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAdGroupExtensionSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupExtensionSettingsRequest.
                             * @implements IMutateAdGroupExtensionSettingsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupExtensionSettingsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupExtensionSettingsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @instance
                             */
                            MutateAdGroupExtensionSettingsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupExtensionSettingsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @instance
                             */
                            MutateAdGroupExtensionSettingsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupExtensionSettingsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @instance
                             */
                            MutateAdGroupExtensionSettingsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupExtensionSettingsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @instance
                             */
                            MutateAdGroupExtensionSettingsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAdGroupExtensionSettingsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest} MutateAdGroupExtensionSettingsRequest instance
                             */
                            MutateAdGroupExtensionSettingsRequest.create = function create(properties) {
                                return new MutateAdGroupExtensionSettingsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupExtensionSettingsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsRequest} m MutateAdGroupExtensionSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupExtensionSettingsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupExtensionSettingsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsRequest} message MutateAdGroupExtensionSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupExtensionSettingsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupExtensionSettingsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest} MutateAdGroupExtensionSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupExtensionSettingsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupExtensionSettingsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest} MutateAdGroupExtensionSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupExtensionSettingsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupExtensionSettingsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupExtensionSettingsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupExtensionSettingsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest} MutateAdGroupExtensionSettingsRequest
                             */
                            MutateAdGroupExtensionSettingsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupExtensionSettingsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest} m MutateAdGroupExtensionSettingsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupExtensionSettingsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupExtensionSettingsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupExtensionSettingsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupExtensionSettingsRequest;
                        })();
    
                        services.AdGroupExtensionSettingOperation = (function() {
    
                            /**
                             * Properties of an AdGroupExtensionSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupExtensionSettingOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdGroupExtensionSettingOperation update_mask
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] AdGroupExtensionSettingOperation response_content_type
                             * @property {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null} [create] AdGroupExtensionSettingOperation create
                             * @property {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null} [update] AdGroupExtensionSettingOperation update
                             * @property {string|null} [remove] AdGroupExtensionSettingOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupExtensionSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupExtensionSettingOperation.
                             * @implements IAdGroupExtensionSettingOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation=} [p] Properties to set
                             */
                            function AdGroupExtensionSettingOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupExtensionSettingOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @instance
                             */
                            AdGroupExtensionSettingOperation.prototype.update_mask = null;
    
                            /**
                             * AdGroupExtensionSettingOperation response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @instance
                             */
                            AdGroupExtensionSettingOperation.prototype.response_content_type = 0;
    
                            /**
                             * AdGroupExtensionSettingOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @instance
                             */
                            AdGroupExtensionSettingOperation.prototype.create = null;
    
                            /**
                             * AdGroupExtensionSettingOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @instance
                             */
                            AdGroupExtensionSettingOperation.prototype.update = null;
    
                            /**
                             * AdGroupExtensionSettingOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @instance
                             */
                            AdGroupExtensionSettingOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupExtensionSettingOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupExtensionSettingOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupExtensionSettingOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupExtensionSettingOperation} AdGroupExtensionSettingOperation instance
                             */
                            AdGroupExtensionSettingOperation.create = function create(properties) {
                                return new AdGroupExtensionSettingOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupExtensionSettingOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation} m AdGroupExtensionSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupExtensionSettingOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupExtensionSettingOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation} message AdGroupExtensionSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupExtensionSettingOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupExtensionSettingOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupExtensionSettingOperation} AdGroupExtensionSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupExtensionSettingOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupExtensionSettingOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupExtensionSettingOperation} AdGroupExtensionSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupExtensionSettingOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupExtensionSettingOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupExtensionSettingOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupExtensionSettingOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupExtensionSettingOperation} AdGroupExtensionSettingOperation
                             */
                            AdGroupExtensionSettingOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupExtensionSettingOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupExtensionSettingOperation} m AdGroupExtensionSettingOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupExtensionSettingOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupExtensionSettingOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupExtensionSettingOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupExtensionSettingOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupExtensionSettingOperation;
                        })();
    
                        services.MutateAdGroupExtensionSettingsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupExtensionSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupExtensionSettingsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupExtensionSettingsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult>|null} [results] MutateAdGroupExtensionSettingsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupExtensionSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupExtensionSettingsResponse.
                             * @implements IMutateAdGroupExtensionSettingsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupExtensionSettingsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupExtensionSettingsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @instance
                             */
                            MutateAdGroupExtensionSettingsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupExtensionSettingsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @instance
                             */
                            MutateAdGroupExtensionSettingsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupExtensionSettingsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse} MutateAdGroupExtensionSettingsResponse instance
                             */
                            MutateAdGroupExtensionSettingsResponse.create = function create(properties) {
                                return new MutateAdGroupExtensionSettingsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupExtensionSettingsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsResponse} m MutateAdGroupExtensionSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupExtensionSettingsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupExtensionSettingsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingsResponse} message MutateAdGroupExtensionSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupExtensionSettingsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupExtensionSettingsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse} MutateAdGroupExtensionSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupExtensionSettingsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupExtensionSettingsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse} MutateAdGroupExtensionSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupExtensionSettingsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupExtensionSettingsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupExtensionSettingsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupExtensionSettingsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse} MutateAdGroupExtensionSettingsResponse
                             */
                            MutateAdGroupExtensionSettingsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupExtensionSettingsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse} m MutateAdGroupExtensionSettingsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupExtensionSettingsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupExtensionSettingsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupExtensionSettingsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupExtensionSettingsResponse;
                        })();
    
                        services.MutateAdGroupExtensionSettingResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupExtensionSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupExtensionSettingResult
                             * @property {string|null} [resource_name] MutateAdGroupExtensionSettingResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null} [ad_group_extension_setting] MutateAdGroupExtensionSettingResult ad_group_extension_setting
                             */
    
                            /**
                             * Constructs a new MutateAdGroupExtensionSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupExtensionSettingResult.
                             * @implements IMutateAdGroupExtensionSettingResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult=} [p] Properties to set
                             */
                            function MutateAdGroupExtensionSettingResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupExtensionSettingResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @instance
                             */
                            MutateAdGroupExtensionSettingResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupExtensionSettingResult ad_group_extension_setting.
                             * @member {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null|undefined} ad_group_extension_setting
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @instance
                             */
                            MutateAdGroupExtensionSettingResult.prototype.ad_group_extension_setting = null;
    
                            /**
                             * Creates a new MutateAdGroupExtensionSettingResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult} MutateAdGroupExtensionSettingResult instance
                             */
                            MutateAdGroupExtensionSettingResult.create = function create(properties) {
                                return new MutateAdGroupExtensionSettingResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupExtensionSettingResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult} m MutateAdGroupExtensionSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupExtensionSettingResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_extension_setting != null && Object.hasOwnProperty.call(m, "ad_group_extension_setting"))
                                    $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.encode(m.ad_group_extension_setting, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupExtensionSettingResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult} message MutateAdGroupExtensionSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupExtensionSettingResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupExtensionSettingResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult} MutateAdGroupExtensionSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupExtensionSettingResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_extension_setting = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupExtensionSettingResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult} MutateAdGroupExtensionSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupExtensionSettingResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupExtensionSettingResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupExtensionSettingResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_extension_setting != null && m.hasOwnProperty("ad_group_extension_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.verify(m.ad_group_extension_setting);
                                        if (e)
                                            return "ad_group_extension_setting." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupExtensionSettingResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult} MutateAdGroupExtensionSettingResult
                             */
                            MutateAdGroupExtensionSettingResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_extension_setting != null) {
                                    if (typeof d.ad_group_extension_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.ad_group_extension_setting: object expected");
                                    m.ad_group_extension_setting = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.fromObject(d.ad_group_extension_setting);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupExtensionSettingResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult} m MutateAdGroupExtensionSettingResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupExtensionSettingResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_extension_setting = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_extension_setting != null && m.hasOwnProperty("ad_group_extension_setting")) {
                                    d.ad_group_extension_setting = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.toObject(m.ad_group_extension_setting, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupExtensionSettingResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupExtensionSettingResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupExtensionSettingResult;
                        })();
    
                        services.AdGroupFeedService = (function() {
    
                            /**
                             * Constructs a new AdGroupFeedService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupFeedService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupFeedService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupFeedService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupFeedService;
    
                            /**
                             * Creates new AdGroupFeedService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupFeedService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupFeedService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupFeedService|mutateAdGroupFeeds}.
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedService
                             * @typedef MutateAdGroupFeedsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupFeedsResponse} [response] MutateAdGroupFeedsResponse
                             */
    
                            /**
                             * Calls MutateAdGroupFeeds.
                             * @function mutateAdGroupFeeds
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsRequest} request MutateAdGroupFeedsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupFeedService.MutateAdGroupFeedsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupFeedsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupFeedService.prototype.mutateAdGroupFeeds = function mutateAdGroupFeeds(request, callback) {
                                return this.rpcCall(mutateAdGroupFeeds, $root.google.ads.googleads.v11.services.MutateAdGroupFeedsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupFeedsResponse, request, callback);
                            }, "name", { value: "MutateAdGroupFeeds" });
    
                            /**
                             * Calls MutateAdGroupFeeds.
                             * @function mutateAdGroupFeeds
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsRequest} request MutateAdGroupFeedsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupFeedsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupFeedService;
                        })();
    
                        services.MutateAdGroupFeedsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupFeedsRequest
                             * @property {string|null} [customer_id] MutateAdGroupFeedsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupFeedOperation>|null} [operations] MutateAdGroupFeedsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupFeedsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupFeedsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupFeedsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupFeedsRequest.
                             * @implements IMutateAdGroupFeedsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupFeedsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupFeedsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @instance
                             */
                            MutateAdGroupFeedsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupFeedsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupFeedOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @instance
                             */
                            MutateAdGroupFeedsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupFeedsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @instance
                             */
                            MutateAdGroupFeedsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupFeedsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @instance
                             */
                            MutateAdGroupFeedsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupFeedsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @instance
                             */
                            MutateAdGroupFeedsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupFeedsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsRequest} MutateAdGroupFeedsRequest instance
                             */
                            MutateAdGroupFeedsRequest.create = function create(properties) {
                                return new MutateAdGroupFeedsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupFeedsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupFeedsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsRequest} m MutateAdGroupFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupFeedsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupFeedOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupFeedsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupFeedsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsRequest} message MutateAdGroupFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupFeedsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupFeedsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsRequest} MutateAdGroupFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupFeedsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupFeedsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupFeedOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupFeedsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsRequest} MutateAdGroupFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupFeedsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupFeedsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupFeedsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupFeedOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupFeedsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsRequest} MutateAdGroupFeedsRequest
                             */
                            MutateAdGroupFeedsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupFeedsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupFeedsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupFeedsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupFeedsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupFeedOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupFeedsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupFeedsRequest} m MutateAdGroupFeedsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupFeedsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupFeedOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupFeedsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupFeedsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupFeedsRequest;
                        })();
    
                        services.AdGroupFeedOperation = (function() {
    
                            /**
                             * Properties of an AdGroupFeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupFeedOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdGroupFeedOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAdGroupFeed|null} [create] AdGroupFeedOperation create
                             * @property {google.ads.googleads.v11.resources.IAdGroupFeed|null} [update] AdGroupFeedOperation update
                             * @property {string|null} [remove] AdGroupFeedOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupFeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupFeedOperation.
                             * @implements IAdGroupFeedOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupFeedOperation=} [p] Properties to set
                             */
                            function AdGroupFeedOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupFeedOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @instance
                             */
                            AdGroupFeedOperation.prototype.update_mask = null;
    
                            /**
                             * AdGroupFeedOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupFeed|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @instance
                             */
                            AdGroupFeedOperation.prototype.create = null;
    
                            /**
                             * AdGroupFeedOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdGroupFeed|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @instance
                             */
                            AdGroupFeedOperation.prototype.update = null;
    
                            /**
                             * AdGroupFeedOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @instance
                             */
                            AdGroupFeedOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupFeedOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupFeedOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupFeedOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupFeedOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupFeedOperation} AdGroupFeedOperation instance
                             */
                            AdGroupFeedOperation.create = function create(properties) {
                                return new AdGroupFeedOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupFeedOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupFeedOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupFeedOperation} m AdGroupFeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupFeedOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupFeed.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdGroupFeed.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupFeedOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupFeedOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupFeedOperation} message AdGroupFeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupFeedOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupFeedOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupFeedOperation} AdGroupFeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupFeedOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupFeedOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupFeed.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AdGroupFeed.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupFeedOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupFeedOperation} AdGroupFeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupFeedOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupFeedOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupFeedOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupFeed.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupFeed.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupFeedOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupFeedOperation} AdGroupFeedOperation
                             */
                            AdGroupFeedOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupFeedOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupFeedOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupFeedOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupFeedOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupFeed.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupFeedOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdGroupFeed.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupFeedOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupFeedOperation} m AdGroupFeedOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupFeedOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupFeed.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdGroupFeed.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupFeedOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupFeedOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupFeedOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupFeedOperation;
                        })();
    
                        services.MutateAdGroupFeedsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupFeedsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupFeedsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupFeedResult>|null} [results] MutateAdGroupFeedsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupFeedsResponse.
                             * @implements IMutateAdGroupFeedsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupFeedsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupFeedsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @instance
                             */
                            MutateAdGroupFeedsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupFeedsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupFeedResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @instance
                             */
                            MutateAdGroupFeedsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupFeedsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsResponse} MutateAdGroupFeedsResponse instance
                             */
                            MutateAdGroupFeedsResponse.create = function create(properties) {
                                return new MutateAdGroupFeedsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupFeedsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupFeedsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsResponse} m MutateAdGroupFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupFeedsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupFeedsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupFeedsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedsResponse} message MutateAdGroupFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupFeedsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupFeedsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsResponse} MutateAdGroupFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupFeedsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupFeedsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupFeedsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsResponse} MutateAdGroupFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupFeedsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupFeedsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupFeedsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupFeedsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedsResponse} MutateAdGroupFeedsResponse
                             */
                            MutateAdGroupFeedsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupFeedsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupFeedsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupFeedsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupFeedsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupFeedsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupFeedsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupFeedsResponse} m MutateAdGroupFeedsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupFeedsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupFeedsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupFeedsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupFeedsResponse;
                        })();
    
                        services.MutateAdGroupFeedResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupFeedResult
                             * @property {string|null} [resource_name] MutateAdGroupFeedResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroupFeed|null} [ad_group_feed] MutateAdGroupFeedResult ad_group_feed
                             */
    
                            /**
                             * Constructs a new MutateAdGroupFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupFeedResult.
                             * @implements IMutateAdGroupFeedResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedResult=} [p] Properties to set
                             */
                            function MutateAdGroupFeedResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupFeedResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @instance
                             */
                            MutateAdGroupFeedResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupFeedResult ad_group_feed.
                             * @member {google.ads.googleads.v11.resources.IAdGroupFeed|null|undefined} ad_group_feed
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @instance
                             */
                            MutateAdGroupFeedResult.prototype.ad_group_feed = null;
    
                            /**
                             * Creates a new MutateAdGroupFeedResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedResult} MutateAdGroupFeedResult instance
                             */
                            MutateAdGroupFeedResult.create = function create(properties) {
                                return new MutateAdGroupFeedResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupFeedResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupFeedResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedResult} m MutateAdGroupFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupFeedResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group_feed != null && Object.hasOwnProperty.call(m, "ad_group_feed"))
                                    $root.google.ads.googleads.v11.resources.AdGroupFeed.encode(m.ad_group_feed, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupFeedResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupFeedResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupFeedResult} message MutateAdGroupFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupFeedResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupFeedResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedResult} MutateAdGroupFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupFeedResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupFeedResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedResult} MutateAdGroupFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupFeedResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupFeedResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupFeedResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupFeed.verify(m.ad_group_feed);
                                        if (e)
                                            return "ad_group_feed." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupFeedResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupFeedResult} MutateAdGroupFeedResult
                             */
                            MutateAdGroupFeedResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group_feed != null) {
                                    if (typeof d.ad_group_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupFeedResult.ad_group_feed: object expected");
                                    m.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.fromObject(d.ad_group_feed);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupFeedResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupFeedResult} m MutateAdGroupFeedResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupFeedResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group_feed = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                    d.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.toObject(m.ad_group_feed, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupFeedResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupFeedResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupFeedResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupFeedResult;
                        })();
    
                        services.AdGroupLabelService = (function() {
    
                            /**
                             * Constructs a new AdGroupLabelService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupLabelService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupLabelService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupLabelService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupLabelService;
    
                            /**
                             * Creates new AdGroupLabelService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupLabelService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupLabelService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupLabelService|mutateAdGroupLabels}.
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelService
                             * @typedef MutateAdGroupLabelsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupLabelsResponse} [response] MutateAdGroupLabelsResponse
                             */
    
                            /**
                             * Calls MutateAdGroupLabels.
                             * @function mutateAdGroupLabels
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsRequest} request MutateAdGroupLabelsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupLabelService.MutateAdGroupLabelsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupLabelsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupLabelService.prototype.mutateAdGroupLabels = function mutateAdGroupLabels(request, callback) {
                                return this.rpcCall(mutateAdGroupLabels, $root.google.ads.googleads.v11.services.MutateAdGroupLabelsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupLabelsResponse, request, callback);
                            }, "name", { value: "MutateAdGroupLabels" });
    
                            /**
                             * Calls MutateAdGroupLabels.
                             * @function mutateAdGroupLabels
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsRequest} request MutateAdGroupLabelsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupLabelsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupLabelService;
                        })();
    
                        services.MutateAdGroupLabelsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupLabelsRequest
                             * @property {string|null} [customer_id] MutateAdGroupLabelsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupLabelOperation>|null} [operations] MutateAdGroupLabelsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupLabelsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupLabelsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAdGroupLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupLabelsRequest.
                             * @implements IMutateAdGroupLabelsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupLabelsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupLabelsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @instance
                             */
                            MutateAdGroupLabelsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupLabelsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupLabelOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @instance
                             */
                            MutateAdGroupLabelsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupLabelsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @instance
                             */
                            MutateAdGroupLabelsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupLabelsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @instance
                             */
                            MutateAdGroupLabelsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAdGroupLabelsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsRequest} MutateAdGroupLabelsRequest instance
                             */
                            MutateAdGroupLabelsRequest.create = function create(properties) {
                                return new MutateAdGroupLabelsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupLabelsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupLabelsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsRequest} m MutateAdGroupLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupLabelsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupLabelOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupLabelsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupLabelsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsRequest} message MutateAdGroupLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupLabelsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupLabelsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsRequest} MutateAdGroupLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupLabelsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupLabelsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupLabelOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupLabelsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsRequest} MutateAdGroupLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupLabelsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupLabelsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupLabelsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupLabelOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupLabelsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsRequest} MutateAdGroupLabelsRequest
                             */
                            MutateAdGroupLabelsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupLabelsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupLabelsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupLabelsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupLabelsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupLabelOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupLabelsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupLabelsRequest} m MutateAdGroupLabelsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupLabelsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupLabelOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupLabelsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupLabelsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupLabelsRequest;
                        })();
    
                        services.AdGroupLabelOperation = (function() {
    
                            /**
                             * Properties of an AdGroupLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupLabelOperation
                             * @property {google.ads.googleads.v11.resources.IAdGroupLabel|null} [create] AdGroupLabelOperation create
                             * @property {string|null} [remove] AdGroupLabelOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupLabelOperation.
                             * @implements IAdGroupLabelOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupLabelOperation=} [p] Properties to set
                             */
                            function AdGroupLabelOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupLabelOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroupLabel|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @instance
                             */
                            AdGroupLabelOperation.prototype.create = null;
    
                            /**
                             * AdGroupLabelOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @instance
                             */
                            AdGroupLabelOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupLabelOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupLabelOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupLabelOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupLabelOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupLabelOperation} AdGroupLabelOperation instance
                             */
                            AdGroupLabelOperation.create = function create(properties) {
                                return new AdGroupLabelOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupLabelOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupLabelOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupLabelOperation} m AdGroupLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupLabelOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroupLabel.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupLabelOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupLabelOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupLabelOperation} message AdGroupLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupLabelOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupLabelOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupLabelOperation} AdGroupLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupLabelOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupLabelOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroupLabel.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupLabelOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupLabelOperation} AdGroupLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupLabelOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupLabelOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupLabelOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupLabel.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupLabelOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupLabelOperation} AdGroupLabelOperation
                             */
                            AdGroupLabelOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupLabelOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupLabelOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupLabelOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroupLabel.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupLabelOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupLabelOperation} m AdGroupLabelOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupLabelOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroupLabel.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupLabelOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupLabelOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupLabelOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupLabelOperation;
                        })();
    
                        services.MutateAdGroupLabelsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupLabelsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupLabelsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupLabelResult>|null} [results] MutateAdGroupLabelsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupLabelsResponse.
                             * @implements IMutateAdGroupLabelsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupLabelsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupLabelsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @instance
                             */
                            MutateAdGroupLabelsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupLabelsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupLabelResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @instance
                             */
                            MutateAdGroupLabelsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupLabelsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsResponse} MutateAdGroupLabelsResponse instance
                             */
                            MutateAdGroupLabelsResponse.create = function create(properties) {
                                return new MutateAdGroupLabelsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupLabelsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupLabelsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsResponse} m MutateAdGroupLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupLabelsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupLabelsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupLabelsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelsResponse} message MutateAdGroupLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupLabelsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupLabelsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsResponse} MutateAdGroupLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupLabelsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupLabelsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupLabelsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsResponse} MutateAdGroupLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupLabelsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupLabelsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupLabelsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupLabelsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelsResponse} MutateAdGroupLabelsResponse
                             */
                            MutateAdGroupLabelsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupLabelsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupLabelsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupLabelsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupLabelsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupLabelsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupLabelsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupLabelsResponse} m MutateAdGroupLabelsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupLabelsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupLabelsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupLabelsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupLabelsResponse;
                        })();
    
                        services.MutateAdGroupLabelResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupLabelResult
                             * @property {string|null} [resource_name] MutateAdGroupLabelResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateAdGroupLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupLabelResult.
                             * @implements IMutateAdGroupLabelResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelResult=} [p] Properties to set
                             */
                            function MutateAdGroupLabelResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupLabelResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @instance
                             */
                            MutateAdGroupLabelResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateAdGroupLabelResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelResult} MutateAdGroupLabelResult instance
                             */
                            MutateAdGroupLabelResult.create = function create(properties) {
                                return new MutateAdGroupLabelResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupLabelResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupLabelResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelResult} m MutateAdGroupLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupLabelResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupLabelResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupLabelResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupLabelResult} message MutateAdGroupLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupLabelResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupLabelResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelResult} MutateAdGroupLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupLabelResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupLabelResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelResult} MutateAdGroupLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupLabelResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupLabelResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupLabelResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupLabelResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupLabelResult} MutateAdGroupLabelResult
                             */
                            MutateAdGroupLabelResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupLabelResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupLabelResult} m MutateAdGroupLabelResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupLabelResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupLabelResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupLabelResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupLabelResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupLabelResult;
                        })();
    
                        services.AdGroupService = (function() {
    
                            /**
                             * Constructs a new AdGroupService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdGroupService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdGroupService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdGroupService;
    
                            /**
                             * Creates new AdGroupService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdGroupService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdGroupService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdGroupService|mutateAdGroups}.
                             * @memberof google.ads.googleads.v11.services.AdGroupService
                             * @typedef MutateAdGroupsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdGroupsResponse} [response] MutateAdGroupsResponse
                             */
    
                            /**
                             * Calls MutateAdGroups.
                             * @function mutateAdGroups
                             * @memberof google.ads.googleads.v11.services.AdGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsRequest} request MutateAdGroupsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdGroupService.MutateAdGroupsCallback} callback Node-style callback called with the error, if any, and MutateAdGroupsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdGroupService.prototype.mutateAdGroups = function mutateAdGroups(request, callback) {
                                return this.rpcCall(mutateAdGroups, $root.google.ads.googleads.v11.services.MutateAdGroupsRequest, $root.google.ads.googleads.v11.services.MutateAdGroupsResponse, request, callback);
                            }, "name", { value: "MutateAdGroups" });
    
                            /**
                             * Calls MutateAdGroups.
                             * @function mutateAdGroups
                             * @memberof google.ads.googleads.v11.services.AdGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsRequest} request MutateAdGroupsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdGroupsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdGroupService;
                        })();
    
                        services.MutateAdGroupsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupsRequest
                             * @property {string|null} [customer_id] MutateAdGroupsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupOperation>|null} [operations] MutateAdGroupsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdGroupsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdGroupsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdGroupsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupsRequest.
                             * @implements IMutateAdGroupsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsRequest=} [p] Properties to set
                             */
                            function MutateAdGroupsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @instance
                             */
                            MutateAdGroupsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdGroupsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @instance
                             */
                            MutateAdGroupsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdGroupsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @instance
                             */
                            MutateAdGroupsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdGroupsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @instance
                             */
                            MutateAdGroupsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdGroupsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @instance
                             */
                            MutateAdGroupsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdGroupsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsRequest} MutateAdGroupsRequest instance
                             */
                            MutateAdGroupsRequest.create = function create(properties) {
                                return new MutateAdGroupsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsRequest} m MutateAdGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsRequest} message MutateAdGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsRequest} MutateAdGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdGroupOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsRequest} MutateAdGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsRequest} MutateAdGroupsRequest
                             */
                            MutateAdGroupsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdGroupOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupsRequest} m MutateAdGroupsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdGroupOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupsRequest;
                        })();
    
                        services.AdGroupOperation = (function() {
    
                            /**
                             * Properties of an AdGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdGroupOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAdGroup|null} [create] AdGroupOperation create
                             * @property {google.ads.googleads.v11.resources.IAdGroup|null} [update] AdGroupOperation update
                             * @property {string|null} [remove] AdGroupOperation remove
                             */
    
                            /**
                             * Constructs a new AdGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupOperation.
                             * @implements IAdGroupOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupOperation=} [p] Properties to set
                             */
                            function AdGroupOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @instance
                             */
                            AdGroupOperation.prototype.update_mask = null;
    
                            /**
                             * AdGroupOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdGroup|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @instance
                             */
                            AdGroupOperation.prototype.create = null;
    
                            /**
                             * AdGroupOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdGroup|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @instance
                             */
                            AdGroupOperation.prototype.update = null;
    
                            /**
                             * AdGroupOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @instance
                             */
                            AdGroupOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdGroupOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @instance
                             */
                            Object.defineProperty(AdGroupOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdGroupOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupOperation} AdGroupOperation instance
                             */
                            AdGroupOperation.create = function create(properties) {
                                return new AdGroupOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupOperation} m AdGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdGroup.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdGroup.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupOperation} message AdGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupOperation} AdGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdGroup.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AdGroup.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupOperation} AdGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroup.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroup.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupOperation} AdGroupOperation
                             */
                            AdGroupOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdGroup.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdGroupOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdGroup.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupOperation} m AdGroupOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdGroup.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdGroup.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupOperation;
                        })();
    
                        services.MutateAdGroupsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdGroupsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdGroupResult>|null} [results] MutateAdGroupsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupsResponse.
                             * @implements IMutateAdGroupsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsResponse=} [p] Properties to set
                             */
                            function MutateAdGroupsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @instance
                             */
                            MutateAdGroupsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdGroupsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdGroupResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @instance
                             */
                            MutateAdGroupsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdGroupsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsResponse} MutateAdGroupsResponse instance
                             */
                            MutateAdGroupsResponse.create = function create(properties) {
                                return new MutateAdGroupsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsResponse} m MutateAdGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdGroupResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupsResponse} message MutateAdGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsResponse} MutateAdGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdGroupResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsResponse} MutateAdGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdGroupResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupsResponse} MutateAdGroupsResponse
                             */
                            MutateAdGroupsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdGroupResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupsResponse} m MutateAdGroupsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdGroupResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupsResponse;
                        })();
    
                        services.MutateAdGroupResult = (function() {
    
                            /**
                             * Properties of a MutateAdGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdGroupResult
                             * @property {string|null} [resource_name] MutateAdGroupResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdGroup|null} [ad_group] MutateAdGroupResult ad_group
                             */
    
                            /**
                             * Constructs a new MutateAdGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdGroupResult.
                             * @implements IMutateAdGroupResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupResult=} [p] Properties to set
                             */
                            function MutateAdGroupResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdGroupResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @instance
                             */
                            MutateAdGroupResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdGroupResult ad_group.
                             * @member {google.ads.googleads.v11.resources.IAdGroup|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @instance
                             */
                            MutateAdGroupResult.prototype.ad_group = null;
    
                            /**
                             * Creates a new MutateAdGroupResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupResult} MutateAdGroupResult instance
                             */
                            MutateAdGroupResult.create = function create(properties) {
                                return new MutateAdGroupResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupResult} m MutateAdGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    $root.google.ads.googleads.v11.resources.AdGroup.encode(m.ad_group, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdGroupResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdGroupResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdGroupResult} message MutateAdGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdGroupResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdGroupResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupResult} MutateAdGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdGroupResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdGroupResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupResult} MutateAdGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdGroupResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdGroupResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdGroupResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroup.verify(m.ad_group);
                                        if (e)
                                            return "ad_group." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdGroupResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdGroupResult} MutateAdGroupResult
                             */
                            MutateAdGroupResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdGroupResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdGroupResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_group != null) {
                                    if (typeof d.ad_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdGroupResult.ad_group: object expected");
                                    m.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.fromObject(d.ad_group);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdGroupResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdGroupResult} m MutateAdGroupResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdGroupResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_group = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.toObject(m.ad_group, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdGroupResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdGroupResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdGroupResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdGroupResult;
                        })();
    
                        services.AdParameterService = (function() {
    
                            /**
                             * Constructs a new AdParameterService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdParameterService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdParameterService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdParameterService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdParameterService;
    
                            /**
                             * Creates new AdParameterService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdParameterService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdParameterService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdParameterService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdParameterService|mutateAdParameters}.
                             * @memberof google.ads.googleads.v11.services.AdParameterService
                             * @typedef MutateAdParametersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdParametersResponse} [response] MutateAdParametersResponse
                             */
    
                            /**
                             * Calls MutateAdParameters.
                             * @function mutateAdParameters
                             * @memberof google.ads.googleads.v11.services.AdParameterService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersRequest} request MutateAdParametersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdParameterService.MutateAdParametersCallback} callback Node-style callback called with the error, if any, and MutateAdParametersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdParameterService.prototype.mutateAdParameters = function mutateAdParameters(request, callback) {
                                return this.rpcCall(mutateAdParameters, $root.google.ads.googleads.v11.services.MutateAdParametersRequest, $root.google.ads.googleads.v11.services.MutateAdParametersResponse, request, callback);
                            }, "name", { value: "MutateAdParameters" });
    
                            /**
                             * Calls MutateAdParameters.
                             * @function mutateAdParameters
                             * @memberof google.ads.googleads.v11.services.AdParameterService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersRequest} request MutateAdParametersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdParametersResponse>} Promise
                             * @variation 2
                             */
    
                            return AdParameterService;
                        })();
    
                        services.MutateAdParametersRequest = (function() {
    
                            /**
                             * Properties of a MutateAdParametersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdParametersRequest
                             * @property {string|null} [customer_id] MutateAdParametersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdParameterOperation>|null} [operations] MutateAdParametersRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdParametersRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAdParametersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdParametersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAdParametersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdParametersRequest.
                             * @implements IMutateAdParametersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersRequest=} [p] Properties to set
                             */
                            function MutateAdParametersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdParametersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @instance
                             */
                            MutateAdParametersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdParametersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdParameterOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @instance
                             */
                            MutateAdParametersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdParametersRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @instance
                             */
                            MutateAdParametersRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdParametersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @instance
                             */
                            MutateAdParametersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAdParametersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @instance
                             */
                            MutateAdParametersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAdParametersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersRequest} MutateAdParametersRequest instance
                             */
                            MutateAdParametersRequest.create = function create(properties) {
                                return new MutateAdParametersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdParametersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdParametersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersRequest} m MutateAdParametersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdParametersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdParameterOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdParametersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdParametersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersRequest} message MutateAdParametersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdParametersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdParametersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersRequest} MutateAdParametersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdParametersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdParametersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdParameterOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdParametersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersRequest} MutateAdParametersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdParametersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdParametersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdParametersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdParameterOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdParametersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersRequest} MutateAdParametersRequest
                             */
                            MutateAdParametersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdParametersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdParametersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdParametersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdParametersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdParameterOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdParametersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdParametersRequest} m MutateAdParametersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdParametersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdParameterOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdParametersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdParametersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdParametersRequest;
                        })();
    
                        services.AdParameterOperation = (function() {
    
                            /**
                             * Properties of an AdParameterOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdParameterOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdParameterOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAdParameter|null} [create] AdParameterOperation create
                             * @property {google.ads.googleads.v11.resources.IAdParameter|null} [update] AdParameterOperation update
                             * @property {string|null} [remove] AdParameterOperation remove
                             */
    
                            /**
                             * Constructs a new AdParameterOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdParameterOperation.
                             * @implements IAdParameterOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdParameterOperation=} [p] Properties to set
                             */
                            function AdParameterOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdParameterOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @instance
                             */
                            AdParameterOperation.prototype.update_mask = null;
    
                            /**
                             * AdParameterOperation create.
                             * @member {google.ads.googleads.v11.resources.IAdParameter|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @instance
                             */
                            AdParameterOperation.prototype.create = null;
    
                            /**
                             * AdParameterOperation update.
                             * @member {google.ads.googleads.v11.resources.IAdParameter|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @instance
                             */
                            AdParameterOperation.prototype.update = null;
    
                            /**
                             * AdParameterOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @instance
                             */
                            AdParameterOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdParameterOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @instance
                             */
                            Object.defineProperty(AdParameterOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdParameterOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdParameterOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdParameterOperation} AdParameterOperation instance
                             */
                            AdParameterOperation.create = function create(properties) {
                                return new AdParameterOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdParameterOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdParameterOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdParameterOperation} m AdParameterOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdParameterOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AdParameter.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AdParameter.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdParameterOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdParameterOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdParameterOperation} message AdParameterOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdParameterOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdParameterOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdParameterOperation} AdParameterOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdParameterOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdParameterOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AdParameter.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AdParameter.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdParameterOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdParameterOperation} AdParameterOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdParameterOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdParameterOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdParameterOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdParameter.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdParameter.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdParameterOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdParameterOperation} AdParameterOperation
                             */
                            AdParameterOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdParameterOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdParameterOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdParameterOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdParameterOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AdParameter.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdParameterOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AdParameter.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdParameterOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdParameterOperation} m AdParameterOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdParameterOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AdParameter.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AdParameter.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdParameterOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdParameterOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdParameterOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdParameterOperation;
                        })();
    
                        services.MutateAdParametersResponse = (function() {
    
                            /**
                             * Properties of a MutateAdParametersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdParametersResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdParametersResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdParameterResult>|null} [results] MutateAdParametersResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdParametersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdParametersResponse.
                             * @implements IMutateAdParametersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersResponse=} [p] Properties to set
                             */
                            function MutateAdParametersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdParametersResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @instance
                             */
                            MutateAdParametersResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdParametersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdParameterResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @instance
                             */
                            MutateAdParametersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdParametersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersResponse} MutateAdParametersResponse instance
                             */
                            MutateAdParametersResponse.create = function create(properties) {
                                return new MutateAdParametersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdParametersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdParametersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersResponse} m MutateAdParametersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdParametersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdParameterResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdParametersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdParametersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParametersResponse} message MutateAdParametersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdParametersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdParametersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersResponse} MutateAdParametersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdParametersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdParametersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdParameterResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdParametersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersResponse} MutateAdParametersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdParametersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdParametersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdParametersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdParameterResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdParametersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdParametersResponse} MutateAdParametersResponse
                             */
                            MutateAdParametersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdParametersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdParametersResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdParametersResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdParametersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdParametersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdParameterResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdParametersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdParametersResponse} m MutateAdParametersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdParametersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdParameterResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdParametersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdParametersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdParametersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdParametersResponse;
                        })();
    
                        services.MutateAdParameterResult = (function() {
    
                            /**
                             * Properties of a MutateAdParameterResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdParameterResult
                             * @property {string|null} [resource_name] MutateAdParameterResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAdParameter|null} [ad_parameter] MutateAdParameterResult ad_parameter
                             */
    
                            /**
                             * Constructs a new MutateAdParameterResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdParameterResult.
                             * @implements IMutateAdParameterResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdParameterResult=} [p] Properties to set
                             */
                            function MutateAdParameterResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdParameterResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @instance
                             */
                            MutateAdParameterResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdParameterResult ad_parameter.
                             * @member {google.ads.googleads.v11.resources.IAdParameter|null|undefined} ad_parameter
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @instance
                             */
                            MutateAdParameterResult.prototype.ad_parameter = null;
    
                            /**
                             * Creates a new MutateAdParameterResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParameterResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdParameterResult} MutateAdParameterResult instance
                             */
                            MutateAdParameterResult.create = function create(properties) {
                                return new MutateAdParameterResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdParameterResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdParameterResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParameterResult} m MutateAdParameterResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdParameterResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad_parameter != null && Object.hasOwnProperty.call(m, "ad_parameter"))
                                    $root.google.ads.googleads.v11.resources.AdParameter.encode(m.ad_parameter, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdParameterResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdParameterResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdParameterResult} message MutateAdParameterResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdParameterResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdParameterResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdParameterResult} MutateAdParameterResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdParameterResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdParameterResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad_parameter = $root.google.ads.googleads.v11.resources.AdParameter.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdParameterResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdParameterResult} MutateAdParameterResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdParameterResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdParameterResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdParameterResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad_parameter != null && m.hasOwnProperty("ad_parameter")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdParameter.verify(m.ad_parameter);
                                        if (e)
                                            return "ad_parameter." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdParameterResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdParameterResult} MutateAdParameterResult
                             */
                            MutateAdParameterResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdParameterResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdParameterResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad_parameter != null) {
                                    if (typeof d.ad_parameter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdParameterResult.ad_parameter: object expected");
                                    m.ad_parameter = $root.google.ads.googleads.v11.resources.AdParameter.fromObject(d.ad_parameter);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdParameterResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdParameterResult} m MutateAdParameterResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdParameterResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad_parameter = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad_parameter != null && m.hasOwnProperty("ad_parameter")) {
                                    d.ad_parameter = $root.google.ads.googleads.v11.resources.AdParameter.toObject(m.ad_parameter, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdParameterResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdParameterResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdParameterResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdParameterResult;
                        })();
    
                        services.AdService = (function() {
    
                            /**
                             * Constructs a new AdService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AdService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AdService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AdService;
    
                            /**
                             * Creates new AdService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AdService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AdService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdService|getAd}.
                             * @memberof google.ads.googleads.v11.services.AdService
                             * @typedef GetAdCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.resources.Ad} [response] Ad
                             */
    
                            /**
                             * Calls GetAd.
                             * @function getAd
                             * @memberof google.ads.googleads.v11.services.AdService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGetAdRequest} request GetAdRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdService.GetAdCallback} callback Node-style callback called with the error, if any, and Ad
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdService.prototype.getAd = function getAd(request, callback) {
                                return this.rpcCall(getAd, $root.google.ads.googleads.v11.services.GetAdRequest, $root.google.ads.googleads.v11.resources.Ad, request, callback);
                            }, "name", { value: "GetAd" });
    
                            /**
                             * Calls GetAd.
                             * @function getAd
                             * @memberof google.ads.googleads.v11.services.AdService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGetAdRequest} request GetAdRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.resources.Ad>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AdService|mutateAds}.
                             * @memberof google.ads.googleads.v11.services.AdService
                             * @typedef MutateAdsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAdsResponse} [response] MutateAdsResponse
                             */
    
                            /**
                             * Calls MutateAds.
                             * @function mutateAds
                             * @memberof google.ads.googleads.v11.services.AdService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdsRequest} request MutateAdsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AdService.MutateAdsCallback} callback Node-style callback called with the error, if any, and MutateAdsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AdService.prototype.mutateAds = function mutateAds(request, callback) {
                                return this.rpcCall(mutateAds, $root.google.ads.googleads.v11.services.MutateAdsRequest, $root.google.ads.googleads.v11.services.MutateAdsResponse, request, callback);
                            }, "name", { value: "MutateAds" });
    
                            /**
                             * Calls MutateAds.
                             * @function mutateAds
                             * @memberof google.ads.googleads.v11.services.AdService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAdsRequest} request MutateAdsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAdsResponse>} Promise
                             * @variation 2
                             */
    
                            return AdService;
                        })();
    
                        services.GetAdRequest = (function() {
    
                            /**
                             * Properties of a GetAdRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGetAdRequest
                             * @property {string|null} [resource_name] GetAdRequest resource_name
                             */
    
                            /**
                             * Constructs a new GetAdRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GetAdRequest.
                             * @implements IGetAdRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGetAdRequest=} [p] Properties to set
                             */
                            function GetAdRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GetAdRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @instance
                             */
                            GetAdRequest.prototype.resource_name = "";
    
                            /**
                             * Creates a new GetAdRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetAdRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GetAdRequest} GetAdRequest instance
                             */
                            GetAdRequest.create = function create(properties) {
                                return new GetAdRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GetAdRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GetAdRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetAdRequest} m GetAdRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GetAdRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GetAdRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GetAdRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetAdRequest} message GetAdRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GetAdRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GetAdRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GetAdRequest} GetAdRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GetAdRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GetAdRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GetAdRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GetAdRequest} GetAdRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GetAdRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GetAdRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GetAdRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GetAdRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GetAdRequest} GetAdRequest
                             */
                            GetAdRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GetAdRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GetAdRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GetAdRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GetAdRequest} m GetAdRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GetAdRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GetAdRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GetAdRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GetAdRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GetAdRequest;
                        })();
    
                        services.MutateAdsRequest = (function() {
    
                            /**
                             * Properties of a MutateAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdsRequest
                             * @property {string|null} [customer_id] MutateAdsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAdOperation>|null} [operations] MutateAdsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAdsRequest partial_failure
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAdsRequest response_content_type
                             * @property {boolean|null} [validate_only] MutateAdsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdsRequest.
                             * @implements IMutateAdsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdsRequest=} [p] Properties to set
                             */
                            function MutateAdsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @instance
                             */
                            MutateAdsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAdsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAdOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @instance
                             */
                            MutateAdsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAdsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @instance
                             */
                            MutateAdsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAdsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @instance
                             */
                            MutateAdsRequest.prototype.response_content_type = 0;
    
                            /**
                             * MutateAdsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @instance
                             */
                            MutateAdsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAdsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdsRequest} MutateAdsRequest instance
                             */
                            MutateAdsRequest.create = function create(properties) {
                                return new MutateAdsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdsRequest} m MutateAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(32).bool(m.partial_failure);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdsRequest} message MutateAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdsRequest} MutateAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AdOperation.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdsRequest} MutateAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdsRequest} MutateAdsRequest
                             */
                            MutateAdsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AdOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdsRequest} m MutateAdsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                    d.partial_failure = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AdOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdsRequest;
                        })();
    
                        services.AdOperation = (function() {
    
                            /**
                             * Properties of an AdOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AdOperation update_mask
                             * @property {google.ads.googleads.v11.common.IPolicyValidationParameter|null} [policy_validation_parameter] AdOperation policy_validation_parameter
                             * @property {google.ads.googleads.v11.resources.IAd|null} [update] AdOperation update
                             */
    
                            /**
                             * Constructs a new AdOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdOperation.
                             * @implements IAdOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdOperation=} [p] Properties to set
                             */
                            function AdOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @instance
                             */
                            AdOperation.prototype.update_mask = null;
    
                            /**
                             * AdOperation policy_validation_parameter.
                             * @member {google.ads.googleads.v11.common.IPolicyValidationParameter|null|undefined} policy_validation_parameter
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @instance
                             */
                            AdOperation.prototype.policy_validation_parameter = null;
    
                            /**
                             * AdOperation update.
                             * @member {google.ads.googleads.v11.resources.IAd|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @instance
                             */
                            AdOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AdOperation operation.
                             * @member {"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @instance
                             */
                            Object.defineProperty(AdOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AdOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdOperation} AdOperation instance
                             */
                            AdOperation.create = function create(properties) {
                                return new AdOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AdOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AdOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdOperation} m AdOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Ad.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(18).fork()).ldelim();
                                if (m.policy_validation_parameter != null && Object.hasOwnProperty.call(m, "policy_validation_parameter"))
                                    $root.google.ads.googleads.v11.common.PolicyValidationParameter.encode(m.policy_validation_parameter, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdOperation} message AdOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdOperation} AdOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.policy_validation_parameter = $root.google.ads.googleads.v11.common.PolicyValidationParameter.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdOperation} AdOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.policy_validation_parameter != null && m.hasOwnProperty("policy_validation_parameter")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.PolicyValidationParameter.verify(m.policy_validation_parameter);
                                        if (e)
                                            return "policy_validation_parameter." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdOperation} AdOperation
                             */
                            AdOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.policy_validation_parameter != null) {
                                    if (typeof d.policy_validation_parameter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdOperation.policy_validation_parameter: object expected");
                                    m.policy_validation_parameter = $root.google.ads.googleads.v11.common.PolicyValidationParameter.fromObject(d.policy_validation_parameter);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AdOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AdOperation} m AdOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                    d.policy_validation_parameter = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Ad.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                if (m.policy_validation_parameter != null && m.hasOwnProperty("policy_validation_parameter")) {
                                    d.policy_validation_parameter = $root.google.ads.googleads.v11.common.PolicyValidationParameter.toObject(m.policy_validation_parameter, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdOperation;
                        })();
    
                        services.MutateAdsResponse = (function() {
    
                            /**
                             * Properties of a MutateAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAdsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAdResult>|null} [results] MutateAdsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdsResponse.
                             * @implements IMutateAdsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdsResponse=} [p] Properties to set
                             */
                            function MutateAdsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @instance
                             */
                            MutateAdsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAdsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAdResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @instance
                             */
                            MutateAdsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAdsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdsResponse} MutateAdsResponse instance
                             */
                            MutateAdsResponse.create = function create(properties) {
                                return new MutateAdsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdsResponse} m MutateAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAdResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdsResponse} message MutateAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdsResponse} MutateAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAdResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdsResponse} MutateAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAdResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdsResponse} MutateAdsResponse
                             */
                            MutateAdsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAdsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAdResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdsResponse} m MutateAdsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAdResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdsResponse;
                        })();
    
                        services.MutateAdResult = (function() {
    
                            /**
                             * Properties of a MutateAdResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAdResult
                             * @property {string|null} [resource_name] MutateAdResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAd|null} [ad] MutateAdResult ad
                             */
    
                            /**
                             * Constructs a new MutateAdResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAdResult.
                             * @implements IMutateAdResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAdResult=} [p] Properties to set
                             */
                            function MutateAdResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAdResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @instance
                             */
                            MutateAdResult.prototype.resource_name = "";
    
                            /**
                             * MutateAdResult ad.
                             * @member {google.ads.googleads.v11.resources.IAd|null|undefined} ad
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @instance
                             */
                            MutateAdResult.prototype.ad = null;
    
                            /**
                             * Creates a new MutateAdResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAdResult} MutateAdResult instance
                             */
                            MutateAdResult.create = function create(properties) {
                                return new MutateAdResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAdResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdResult} m MutateAdResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.ad != null && Object.hasOwnProperty.call(m, "ad"))
                                    $root.google.ads.googleads.v11.resources.Ad.encode(m.ad, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAdResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAdResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAdResult} message MutateAdResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAdResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAdResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAdResult} MutateAdResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAdResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAdResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAdResult} MutateAdResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAdResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAdResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAdResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.ad != null && m.hasOwnProperty("ad")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.ad);
                                        if (e)
                                            return "ad." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAdResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAdResult} MutateAdResult
                             */
                            MutateAdResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAdResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAdResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.ad != null) {
                                    if (typeof d.ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAdResult.ad: object expected");
                                    m.ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.ad);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAdResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAdResult} m MutateAdResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAdResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.ad = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.ad != null && m.hasOwnProperty("ad")) {
                                    d.ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.ad, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAdResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAdResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAdResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAdResult;
                        })();
    
                        services.AssetGroupAssetService = (function() {
    
                            /**
                             * Constructs a new AssetGroupAssetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupAssetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AssetGroupAssetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AssetGroupAssetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AssetGroupAssetService;
    
                            /**
                             * Creates new AssetGroupAssetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AssetGroupAssetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AssetGroupAssetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AssetGroupAssetService|mutateAssetGroupAssets}.
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetService
                             * @typedef MutateAssetGroupAssetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse} [response] MutateAssetGroupAssetsResponse
                             */
    
                            /**
                             * Calls MutateAssetGroupAssets.
                             * @function mutateAssetGroupAssets
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsRequest} request MutateAssetGroupAssetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AssetGroupAssetService.MutateAssetGroupAssetsCallback} callback Node-style callback called with the error, if any, and MutateAssetGroupAssetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AssetGroupAssetService.prototype.mutateAssetGroupAssets = function mutateAssetGroupAssets(request, callback) {
                                return this.rpcCall(mutateAssetGroupAssets, $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest, $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse, request, callback);
                            }, "name", { value: "MutateAssetGroupAssets" });
    
                            /**
                             * Calls MutateAssetGroupAssets.
                             * @function mutateAssetGroupAssets
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsRequest} request MutateAssetGroupAssetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse>} Promise
                             * @variation 2
                             */
    
                            return AssetGroupAssetService;
                        })();
    
                        services.MutateAssetGroupAssetsRequest = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupAssetsRequest
                             * @property {string|null} [customer_id] MutateAssetGroupAssetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAssetGroupAssetOperation>|null} [operations] MutateAssetGroupAssetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAssetGroupAssetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAssetGroupAssetsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupAssetsRequest.
                             * @implements IMutateAssetGroupAssetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsRequest=} [p] Properties to set
                             */
                            function MutateAssetGroupAssetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupAssetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @instance
                             */
                            MutateAssetGroupAssetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAssetGroupAssetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAssetGroupAssetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @instance
                             */
                            MutateAssetGroupAssetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAssetGroupAssetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @instance
                             */
                            MutateAssetGroupAssetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAssetGroupAssetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @instance
                             */
                            MutateAssetGroupAssetsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAssetGroupAssetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest} MutateAssetGroupAssetsRequest instance
                             */
                            MutateAssetGroupAssetsRequest.create = function create(properties) {
                                return new MutateAssetGroupAssetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupAssetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsRequest} m MutateAssetGroupAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupAssetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupAssetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsRequest} message MutateAssetGroupAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupAssetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupAssetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest} MutateAssetGroupAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupAssetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AssetGroupAssetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupAssetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest} MutateAssetGroupAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupAssetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupAssetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupAssetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupAssetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest} MutateAssetGroupAssetsRequest
                             */
                            MutateAssetGroupAssetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupAssetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest} m MutateAssetGroupAssetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupAssetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupAssetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupAssetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupAssetsRequest;
                        })();
    
                        services.AssetGroupAssetOperation = (function() {
    
                            /**
                             * Properties of an AssetGroupAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAssetGroupAssetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AssetGroupAssetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAssetGroupAsset|null} [create] AssetGroupAssetOperation create
                             * @property {google.ads.googleads.v11.resources.IAssetGroupAsset|null} [update] AssetGroupAssetOperation update
                             * @property {string|null} [remove] AssetGroupAssetOperation remove
                             */
    
                            /**
                             * Constructs a new AssetGroupAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupAssetOperation.
                             * @implements IAssetGroupAssetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAssetGroupAssetOperation=} [p] Properties to set
                             */
                            function AssetGroupAssetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupAssetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @instance
                             */
                            AssetGroupAssetOperation.prototype.update_mask = null;
    
                            /**
                             * AssetGroupAssetOperation create.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupAsset|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @instance
                             */
                            AssetGroupAssetOperation.prototype.create = null;
    
                            /**
                             * AssetGroupAssetOperation update.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupAsset|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @instance
                             */
                            AssetGroupAssetOperation.prototype.update = null;
    
                            /**
                             * AssetGroupAssetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @instance
                             */
                            AssetGroupAssetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AssetGroupAssetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @instance
                             */
                            Object.defineProperty(AssetGroupAssetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AssetGroupAssetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupAssetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AssetGroupAssetOperation} AssetGroupAssetOperation instance
                             */
                            AssetGroupAssetOperation.create = function create(properties) {
                                return new AssetGroupAssetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupAssetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupAssetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupAssetOperation} m AssetGroupAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupAssetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupAsset.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupAsset.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupAssetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupAssetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupAssetOperation} message AssetGroupAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupAssetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupAssetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AssetGroupAssetOperation} AssetGroupAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupAssetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AssetGroupAssetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AssetGroupAsset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AssetGroupAsset.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupAssetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AssetGroupAssetOperation} AssetGroupAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupAssetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupAssetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupAssetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupAsset.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupAsset.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupAssetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AssetGroupAssetOperation} AssetGroupAssetOperation
                             */
                            AssetGroupAssetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AssetGroupAssetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AssetGroupAssetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupAssetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupAssetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AssetGroupAsset.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupAssetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AssetGroupAsset.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupAssetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AssetGroupAssetOperation} m AssetGroupAssetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupAssetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AssetGroupAsset.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AssetGroupAsset.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupAssetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AssetGroupAssetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupAssetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupAssetOperation;
                        })();
    
                        services.MutateAssetGroupAssetsResponse = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupAssetsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAssetGroupAssetResult>|null} [results] MutateAssetGroupAssetsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAssetGroupAssetsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupAssetsResponse.
                             * @implements IMutateAssetGroupAssetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsResponse=} [p] Properties to set
                             */
                            function MutateAssetGroupAssetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupAssetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAssetGroupAssetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @instance
                             */
                            MutateAssetGroupAssetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAssetGroupAssetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @instance
                             */
                            MutateAssetGroupAssetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAssetGroupAssetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse} MutateAssetGroupAssetsResponse instance
                             */
                            MutateAssetGroupAssetsResponse.create = function create(properties) {
                                return new MutateAssetGroupAssetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupAssetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsResponse} m MutateAssetGroupAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupAssetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupAssetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetsResponse} message MutateAssetGroupAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupAssetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupAssetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse} MutateAssetGroupAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupAssetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupAssetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse} MutateAssetGroupAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupAssetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupAssetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupAssetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupAssetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse} MutateAssetGroupAssetsResponse
                             */
                            MutateAssetGroupAssetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupAssetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse} m MutateAssetGroupAssetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupAssetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupAssetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupAssetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupAssetsResponse;
                        })();
    
                        services.MutateAssetGroupAssetResult = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupAssetResult
                             * @property {string|null} [resource_name] MutateAssetGroupAssetResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupAssetResult.
                             * @implements IMutateAssetGroupAssetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetResult=} [p] Properties to set
                             */
                            function MutateAssetGroupAssetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupAssetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @instance
                             */
                            MutateAssetGroupAssetResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateAssetGroupAssetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetResult} MutateAssetGroupAssetResult instance
                             */
                            MutateAssetGroupAssetResult.create = function create(properties) {
                                return new MutateAssetGroupAssetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupAssetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupAssetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetResult} m MutateAssetGroupAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupAssetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupAssetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupAssetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupAssetResult} message MutateAssetGroupAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupAssetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupAssetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetResult} MutateAssetGroupAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupAssetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupAssetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetResult} MutateAssetGroupAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupAssetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupAssetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupAssetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupAssetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupAssetResult} MutateAssetGroupAssetResult
                             */
                            MutateAssetGroupAssetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupAssetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupAssetResult} m MutateAssetGroupAssetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupAssetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupAssetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupAssetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupAssetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupAssetResult;
                        })();
    
                        services.AssetGroupListingGroupFilterService = (function() {
    
                            /**
                             * Constructs a new AssetGroupListingGroupFilterService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupListingGroupFilterService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AssetGroupListingGroupFilterService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AssetGroupListingGroupFilterService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AssetGroupListingGroupFilterService;
    
                            /**
                             * Creates new AssetGroupListingGroupFilterService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AssetGroupListingGroupFilterService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AssetGroupListingGroupFilterService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AssetGroupListingGroupFilterService|mutateAssetGroupListingGroupFilters}.
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterService
                             * @typedef MutateAssetGroupListingGroupFiltersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse} [response] MutateAssetGroupListingGroupFiltersResponse
                             */
    
                            /**
                             * Calls MutateAssetGroupListingGroupFilters.
                             * @function mutateAssetGroupListingGroupFilters
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersRequest} request MutateAssetGroupListingGroupFiltersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AssetGroupListingGroupFilterService.MutateAssetGroupListingGroupFiltersCallback} callback Node-style callback called with the error, if any, and MutateAssetGroupListingGroupFiltersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AssetGroupListingGroupFilterService.prototype.mutateAssetGroupListingGroupFilters = function mutateAssetGroupListingGroupFilters(request, callback) {
                                return this.rpcCall(mutateAssetGroupListingGroupFilters, $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest, $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse, request, callback);
                            }, "name", { value: "MutateAssetGroupListingGroupFilters" });
    
                            /**
                             * Calls MutateAssetGroupListingGroupFilters.
                             * @function mutateAssetGroupListingGroupFilters
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersRequest} request MutateAssetGroupListingGroupFiltersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse>} Promise
                             * @variation 2
                             */
    
                            return AssetGroupListingGroupFilterService;
                        })();
    
                        services.MutateAssetGroupListingGroupFiltersRequest = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupListingGroupFiltersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupListingGroupFiltersRequest
                             * @property {string|null} [customer_id] MutateAssetGroupListingGroupFiltersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation>|null} [operations] MutateAssetGroupListingGroupFiltersRequest operations
                             * @property {boolean|null} [validate_only] MutateAssetGroupListingGroupFiltersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAssetGroupListingGroupFiltersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupListingGroupFiltersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupListingGroupFiltersRequest.
                             * @implements IMutateAssetGroupListingGroupFiltersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersRequest=} [p] Properties to set
                             */
                            function MutateAssetGroupListingGroupFiltersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupListingGroupFiltersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @instance
                             */
                            MutateAssetGroupListingGroupFiltersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAssetGroupListingGroupFiltersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @instance
                             */
                            MutateAssetGroupListingGroupFiltersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAssetGroupListingGroupFiltersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @instance
                             */
                            MutateAssetGroupListingGroupFiltersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAssetGroupListingGroupFiltersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @instance
                             */
                            MutateAssetGroupListingGroupFiltersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAssetGroupListingGroupFiltersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest} MutateAssetGroupListingGroupFiltersRequest instance
                             */
                            MutateAssetGroupListingGroupFiltersRequest.create = function create(properties) {
                                return new MutateAssetGroupListingGroupFiltersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupListingGroupFiltersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersRequest} m MutateAssetGroupListingGroupFiltersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupListingGroupFiltersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(32).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupListingGroupFiltersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersRequest} message MutateAssetGroupListingGroupFiltersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupListingGroupFiltersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupListingGroupFiltersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest} MutateAssetGroupListingGroupFiltersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupListingGroupFiltersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    case 4:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupListingGroupFiltersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest} MutateAssetGroupListingGroupFiltersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupListingGroupFiltersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupListingGroupFiltersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupListingGroupFiltersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupListingGroupFiltersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest} MutateAssetGroupListingGroupFiltersRequest
                             */
                            MutateAssetGroupListingGroupFiltersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupListingGroupFiltersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest} m MutateAssetGroupListingGroupFiltersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupListingGroupFiltersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupListingGroupFiltersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupListingGroupFiltersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupListingGroupFiltersRequest;
                        })();
    
                        services.AssetGroupListingGroupFilterOperation = (function() {
    
                            /**
                             * Properties of an AssetGroupListingGroupFilterOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAssetGroupListingGroupFilterOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AssetGroupListingGroupFilterOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null} [create] AssetGroupListingGroupFilterOperation create
                             * @property {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null} [update] AssetGroupListingGroupFilterOperation update
                             * @property {string|null} [remove] AssetGroupListingGroupFilterOperation remove
                             */
    
                            /**
                             * Constructs a new AssetGroupListingGroupFilterOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupListingGroupFilterOperation.
                             * @implements IAssetGroupListingGroupFilterOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation=} [p] Properties to set
                             */
                            function AssetGroupListingGroupFilterOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupListingGroupFilterOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @instance
                             */
                            AssetGroupListingGroupFilterOperation.prototype.update_mask = null;
    
                            /**
                             * AssetGroupListingGroupFilterOperation create.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @instance
                             */
                            AssetGroupListingGroupFilterOperation.prototype.create = null;
    
                            /**
                             * AssetGroupListingGroupFilterOperation update.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @instance
                             */
                            AssetGroupListingGroupFilterOperation.prototype.update = null;
    
                            /**
                             * AssetGroupListingGroupFilterOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @instance
                             */
                            AssetGroupListingGroupFilterOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AssetGroupListingGroupFilterOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @instance
                             */
                            Object.defineProperty(AssetGroupListingGroupFilterOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AssetGroupListingGroupFilterOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation} AssetGroupListingGroupFilterOperation instance
                             */
                            AssetGroupListingGroupFilterOperation.create = function create(properties) {
                                return new AssetGroupListingGroupFilterOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupListingGroupFilterOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation} m AssetGroupListingGroupFilterOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupListingGroupFilterOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupListingGroupFilterOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation} message AssetGroupListingGroupFilterOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupListingGroupFilterOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupListingGroupFilterOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation} AssetGroupListingGroupFilterOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupListingGroupFilterOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupListingGroupFilterOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation} AssetGroupListingGroupFilterOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupListingGroupFilterOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupListingGroupFilterOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupListingGroupFilterOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupListingGroupFilterOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation} AssetGroupListingGroupFilterOperation
                             */
                            AssetGroupListingGroupFilterOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupListingGroupFilterOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation} m AssetGroupListingGroupFilterOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupListingGroupFilterOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupListingGroupFilterOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupListingGroupFilterOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupListingGroupFilterOperation;
                        })();
    
                        services.MutateAssetGroupListingGroupFiltersResponse = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupListingGroupFiltersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupListingGroupFiltersResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult>|null} [results] MutateAssetGroupListingGroupFiltersResponse results
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupListingGroupFiltersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupListingGroupFiltersResponse.
                             * @implements IMutateAssetGroupListingGroupFiltersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersResponse=} [p] Properties to set
                             */
                            function MutateAssetGroupListingGroupFiltersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupListingGroupFiltersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @instance
                             */
                            MutateAssetGroupListingGroupFiltersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAssetGroupListingGroupFiltersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse} MutateAssetGroupListingGroupFiltersResponse instance
                             */
                            MutateAssetGroupListingGroupFiltersResponse.create = function create(properties) {
                                return new MutateAssetGroupListingGroupFiltersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupListingGroupFiltersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersResponse} m MutateAssetGroupListingGroupFiltersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupListingGroupFiltersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupListingGroupFiltersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFiltersResponse} message MutateAssetGroupListingGroupFiltersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupListingGroupFiltersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupListingGroupFiltersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse} MutateAssetGroupListingGroupFiltersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupListingGroupFiltersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupListingGroupFiltersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse} MutateAssetGroupListingGroupFiltersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupListingGroupFiltersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupListingGroupFiltersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupListingGroupFiltersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupListingGroupFiltersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse} MutateAssetGroupListingGroupFiltersResponse
                             */
                            MutateAssetGroupListingGroupFiltersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupListingGroupFiltersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse} m MutateAssetGroupListingGroupFiltersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupListingGroupFiltersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupListingGroupFiltersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFiltersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupListingGroupFiltersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupListingGroupFiltersResponse;
                        })();
    
                        services.MutateAssetGroupListingGroupFilterResult = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupListingGroupFilterResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupListingGroupFilterResult
                             * @property {string|null} [resource_name] MutateAssetGroupListingGroupFilterResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null} [asset_group_listing_group_filter] MutateAssetGroupListingGroupFilterResult asset_group_listing_group_filter
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupListingGroupFilterResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupListingGroupFilterResult.
                             * @implements IMutateAssetGroupListingGroupFilterResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult=} [p] Properties to set
                             */
                            function MutateAssetGroupListingGroupFilterResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupListingGroupFilterResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @instance
                             */
                            MutateAssetGroupListingGroupFilterResult.prototype.resource_name = "";
    
                            /**
                             * MutateAssetGroupListingGroupFilterResult asset_group_listing_group_filter.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null|undefined} asset_group_listing_group_filter
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @instance
                             */
                            MutateAssetGroupListingGroupFilterResult.prototype.asset_group_listing_group_filter = null;
    
                            /**
                             * Creates a new MutateAssetGroupListingGroupFilterResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult} MutateAssetGroupListingGroupFilterResult instance
                             */
                            MutateAssetGroupListingGroupFilterResult.create = function create(properties) {
                                return new MutateAssetGroupListingGroupFilterResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupListingGroupFilterResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult} m MutateAssetGroupListingGroupFilterResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupListingGroupFilterResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_group_listing_group_filter != null && Object.hasOwnProperty.call(m, "asset_group_listing_group_filter"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.encode(m.asset_group_listing_group_filter, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupListingGroupFilterResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult} message MutateAssetGroupListingGroupFilterResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupListingGroupFilterResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupListingGroupFilterResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult} MutateAssetGroupListingGroupFilterResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupListingGroupFilterResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_group_listing_group_filter = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupListingGroupFilterResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult} MutateAssetGroupListingGroupFilterResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupListingGroupFilterResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupListingGroupFilterResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupListingGroupFilterResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_group_listing_group_filter != null && m.hasOwnProperty("asset_group_listing_group_filter")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.verify(m.asset_group_listing_group_filter);
                                        if (e)
                                            return "asset_group_listing_group_filter." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupListingGroupFilterResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult} MutateAssetGroupListingGroupFilterResult
                             */
                            MutateAssetGroupListingGroupFilterResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_group_listing_group_filter != null) {
                                    if (typeof d.asset_group_listing_group_filter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.asset_group_listing_group_filter: object expected");
                                    m.asset_group_listing_group_filter = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.fromObject(d.asset_group_listing_group_filter);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupListingGroupFilterResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult} m MutateAssetGroupListingGroupFilterResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupListingGroupFilterResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_group_listing_group_filter = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_group_listing_group_filter != null && m.hasOwnProperty("asset_group_listing_group_filter")) {
                                    d.asset_group_listing_group_filter = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.toObject(m.asset_group_listing_group_filter, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupListingGroupFilterResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupListingGroupFilterResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupListingGroupFilterResult;
                        })();
    
                        services.AssetGroupService = (function() {
    
                            /**
                             * Constructs a new AssetGroupService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AssetGroupService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AssetGroupService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AssetGroupService;
    
                            /**
                             * Creates new AssetGroupService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AssetGroupService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AssetGroupService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AssetGroupService|mutateAssetGroups}.
                             * @memberof google.ads.googleads.v11.services.AssetGroupService
                             * @typedef MutateAssetGroupsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupsResponse} [response] MutateAssetGroupsResponse
                             */
    
                            /**
                             * Calls MutateAssetGroups.
                             * @function mutateAssetGroups
                             * @memberof google.ads.googleads.v11.services.AssetGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsRequest} request MutateAssetGroupsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AssetGroupService.MutateAssetGroupsCallback} callback Node-style callback called with the error, if any, and MutateAssetGroupsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AssetGroupService.prototype.mutateAssetGroups = function mutateAssetGroups(request, callback) {
                                return this.rpcCall(mutateAssetGroups, $root.google.ads.googleads.v11.services.MutateAssetGroupsRequest, $root.google.ads.googleads.v11.services.MutateAssetGroupsResponse, request, callback);
                            }, "name", { value: "MutateAssetGroups" });
    
                            /**
                             * Calls MutateAssetGroups.
                             * @function mutateAssetGroups
                             * @memberof google.ads.googleads.v11.services.AssetGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsRequest} request MutateAssetGroupsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAssetGroupsResponse>} Promise
                             * @variation 2
                             */
    
                            return AssetGroupService;
                        })();
    
                        services.MutateAssetGroupsRequest = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupsRequest
                             * @property {string|null} [customer_id] MutateAssetGroupsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAssetGroupOperation>|null} [operations] MutateAssetGroupsRequest operations
                             * @property {boolean|null} [validate_only] MutateAssetGroupsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupsRequest.
                             * @implements IMutateAssetGroupsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsRequest=} [p] Properties to set
                             */
                            function MutateAssetGroupsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @instance
                             */
                            MutateAssetGroupsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAssetGroupsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAssetGroupOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @instance
                             */
                            MutateAssetGroupsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAssetGroupsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @instance
                             */
                            MutateAssetGroupsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAssetGroupsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsRequest} MutateAssetGroupsRequest instance
                             */
                            MutateAssetGroupsRequest.create = function create(properties) {
                                return new MutateAssetGroupsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsRequest} m MutateAssetGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AssetGroupOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsRequest} message MutateAssetGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsRequest} MutateAssetGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AssetGroupOperation.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsRequest} MutateAssetGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AssetGroupOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsRequest} MutateAssetGroupsRequest
                             */
                            MutateAssetGroupsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AssetGroupOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupsRequest} m MutateAssetGroupsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AssetGroupOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupsRequest;
                        })();
    
                        services.AssetGroupOperation = (function() {
    
                            /**
                             * Properties of an AssetGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAssetGroupOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AssetGroupOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAssetGroup|null} [create] AssetGroupOperation create
                             * @property {google.ads.googleads.v11.resources.IAssetGroup|null} [update] AssetGroupOperation update
                             * @property {string|null} [remove] AssetGroupOperation remove
                             */
    
                            /**
                             * Constructs a new AssetGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupOperation.
                             * @implements IAssetGroupOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAssetGroupOperation=} [p] Properties to set
                             */
                            function AssetGroupOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @instance
                             */
                            AssetGroupOperation.prototype.update_mask = null;
    
                            /**
                             * AssetGroupOperation create.
                             * @member {google.ads.googleads.v11.resources.IAssetGroup|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @instance
                             */
                            AssetGroupOperation.prototype.create = null;
    
                            /**
                             * AssetGroupOperation update.
                             * @member {google.ads.googleads.v11.resources.IAssetGroup|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @instance
                             */
                            AssetGroupOperation.prototype.update = null;
    
                            /**
                             * AssetGroupOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @instance
                             */
                            AssetGroupOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AssetGroupOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @instance
                             */
                            Object.defineProperty(AssetGroupOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AssetGroupOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AssetGroupOperation} AssetGroupOperation instance
                             */
                            AssetGroupOperation.create = function create(properties) {
                                return new AssetGroupOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupOperation} m AssetGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AssetGroup.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AssetGroup.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupOperation} message AssetGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AssetGroupOperation} AssetGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AssetGroupOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AssetGroup.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AssetGroup.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AssetGroupOperation} AssetGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroup.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroup.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AssetGroupOperation} AssetGroupOperation
                             */
                            AssetGroupOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AssetGroupOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AssetGroupOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AssetGroup.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AssetGroup.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AssetGroupOperation} m AssetGroupOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AssetGroup.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AssetGroup.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AssetGroupOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupOperation;
                        })();
    
                        services.MutateAssetGroupsResponse = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAssetGroupResult>|null} [results] MutateAssetGroupsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAssetGroupsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupsResponse.
                             * @implements IMutateAssetGroupsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsResponse=} [p] Properties to set
                             */
                            function MutateAssetGroupsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAssetGroupResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @instance
                             */
                            MutateAssetGroupsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAssetGroupsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @instance
                             */
                            MutateAssetGroupsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAssetGroupsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsResponse} MutateAssetGroupsResponse instance
                             */
                            MutateAssetGroupsResponse.create = function create(properties) {
                                return new MutateAssetGroupsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsResponse} m MutateAssetGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAssetGroupResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupsResponse} message MutateAssetGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsResponse} MutateAssetGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAssetGroupResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsResponse} MutateAssetGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAssetGroupResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupsResponse} MutateAssetGroupsResponse
                             */
                            MutateAssetGroupsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAssetGroupResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupsResponse} m MutateAssetGroupsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAssetGroupResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupsResponse;
                        })();
    
                        services.MutateAssetGroupResult = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupResult
                             * @property {string|null} [resource_name] MutateAssetGroupResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupResult.
                             * @implements IMutateAssetGroupResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupResult=} [p] Properties to set
                             */
                            function MutateAssetGroupResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @instance
                             */
                            MutateAssetGroupResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateAssetGroupResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupResult} MutateAssetGroupResult instance
                             */
                            MutateAssetGroupResult.create = function create(properties) {
                                return new MutateAssetGroupResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupResult} m MutateAssetGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupResult} message MutateAssetGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupResult} MutateAssetGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupResult} MutateAssetGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupResult} MutateAssetGroupResult
                             */
                            MutateAssetGroupResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupResult} m MutateAssetGroupResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupResult;
                        })();
    
                        services.AssetGroupSignalService = (function() {
    
                            /**
                             * Constructs a new AssetGroupSignalService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupSignalService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AssetGroupSignalService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AssetGroupSignalService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AssetGroupSignalService;
    
                            /**
                             * Creates new AssetGroupSignalService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AssetGroupSignalService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AssetGroupSignalService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AssetGroupSignalService|mutateAssetGroupSignals}.
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalService
                             * @typedef MutateAssetGroupSignalsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse} [response] MutateAssetGroupSignalsResponse
                             */
    
                            /**
                             * Calls MutateAssetGroupSignals.
                             * @function mutateAssetGroupSignals
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsRequest} request MutateAssetGroupSignalsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AssetGroupSignalService.MutateAssetGroupSignalsCallback} callback Node-style callback called with the error, if any, and MutateAssetGroupSignalsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AssetGroupSignalService.prototype.mutateAssetGroupSignals = function mutateAssetGroupSignals(request, callback) {
                                return this.rpcCall(mutateAssetGroupSignals, $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest, $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse, request, callback);
                            }, "name", { value: "MutateAssetGroupSignals" });
    
                            /**
                             * Calls MutateAssetGroupSignals.
                             * @function mutateAssetGroupSignals
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsRequest} request MutateAssetGroupSignalsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse>} Promise
                             * @variation 2
                             */
    
                            return AssetGroupSignalService;
                        })();
    
                        services.MutateAssetGroupSignalsRequest = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupSignalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupSignalsRequest
                             * @property {string|null} [customer_id] MutateAssetGroupSignalsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAssetGroupSignalOperation>|null} [operations] MutateAssetGroupSignalsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAssetGroupSignalsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAssetGroupSignalsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAssetGroupSignalsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupSignalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupSignalsRequest.
                             * @implements IMutateAssetGroupSignalsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsRequest=} [p] Properties to set
                             */
                            function MutateAssetGroupSignalsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupSignalsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @instance
                             */
                            MutateAssetGroupSignalsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAssetGroupSignalsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAssetGroupSignalOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @instance
                             */
                            MutateAssetGroupSignalsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAssetGroupSignalsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @instance
                             */
                            MutateAssetGroupSignalsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAssetGroupSignalsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @instance
                             */
                            MutateAssetGroupSignalsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAssetGroupSignalsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @instance
                             */
                            MutateAssetGroupSignalsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAssetGroupSignalsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest} MutateAssetGroupSignalsRequest instance
                             */
                            MutateAssetGroupSignalsRequest.create = function create(properties) {
                                return new MutateAssetGroupSignalsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupSignalsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsRequest} m MutateAssetGroupSignalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupSignalsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupSignalsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsRequest} message MutateAssetGroupSignalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupSignalsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupSignalsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest} MutateAssetGroupSignalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupSignalsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AssetGroupSignalOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupSignalsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest} MutateAssetGroupSignalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupSignalsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupSignalsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupSignalsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupSignalsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest} MutateAssetGroupSignalsRequest
                             */
                            MutateAssetGroupSignalsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupSignalsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest} m MutateAssetGroupSignalsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupSignalsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupSignalsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupSignalsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupSignalsRequest;
                        })();
    
                        services.AssetGroupSignalOperation = (function() {
    
                            /**
                             * Properties of an AssetGroupSignalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAssetGroupSignalOperation
                             * @property {google.ads.googleads.v11.resources.IAssetGroupSignal|null} [create] AssetGroupSignalOperation create
                             * @property {string|null} [remove] AssetGroupSignalOperation remove
                             */
    
                            /**
                             * Constructs a new AssetGroupSignalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetGroupSignalOperation.
                             * @implements IAssetGroupSignalOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAssetGroupSignalOperation=} [p] Properties to set
                             */
                            function AssetGroupSignalOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetGroupSignalOperation create.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupSignal|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @instance
                             */
                            AssetGroupSignalOperation.prototype.create = null;
    
                            /**
                             * AssetGroupSignalOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @instance
                             */
                            AssetGroupSignalOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AssetGroupSignalOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @instance
                             */
                            Object.defineProperty(AssetGroupSignalOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AssetGroupSignalOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupSignalOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AssetGroupSignalOperation} AssetGroupSignalOperation instance
                             */
                            AssetGroupSignalOperation.create = function create(properties) {
                                return new AssetGroupSignalOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AssetGroupSignalOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupSignalOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupSignalOperation} m AssetGroupSignalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupSignalOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupSignal.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetGroupSignalOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AssetGroupSignalOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetGroupSignalOperation} message AssetGroupSignalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetGroupSignalOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetGroupSignalOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AssetGroupSignalOperation} AssetGroupSignalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupSignalOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AssetGroupSignalOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AssetGroupSignal.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetGroupSignalOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AssetGroupSignalOperation} AssetGroupSignalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetGroupSignalOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetGroupSignalOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetGroupSignalOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupSignal.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetGroupSignalOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AssetGroupSignalOperation} AssetGroupSignalOperation
                             */
                            AssetGroupSignalOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AssetGroupSignalOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AssetGroupSignalOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetGroupSignalOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AssetGroupSignal.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetGroupSignalOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AssetGroupSignalOperation} m AssetGroupSignalOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetGroupSignalOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AssetGroupSignal.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetGroupSignalOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AssetGroupSignalOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetGroupSignalOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetGroupSignalOperation;
                        })();
    
                        services.MutateAssetGroupSignalsResponse = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupSignalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupSignalsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAssetGroupSignalResult>|null} [results] MutateAssetGroupSignalsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAssetGroupSignalsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupSignalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupSignalsResponse.
                             * @implements IMutateAssetGroupSignalsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsResponse=} [p] Properties to set
                             */
                            function MutateAssetGroupSignalsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupSignalsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAssetGroupSignalResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @instance
                             */
                            MutateAssetGroupSignalsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAssetGroupSignalsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @instance
                             */
                            MutateAssetGroupSignalsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAssetGroupSignalsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse} MutateAssetGroupSignalsResponse instance
                             */
                            MutateAssetGroupSignalsResponse.create = function create(properties) {
                                return new MutateAssetGroupSignalsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupSignalsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsResponse} m MutateAssetGroupSignalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupSignalsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupSignalsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalsResponse} message MutateAssetGroupSignalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupSignalsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupSignalsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse} MutateAssetGroupSignalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupSignalsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupSignalsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse} MutateAssetGroupSignalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupSignalsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupSignalsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupSignalsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupSignalsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse} MutateAssetGroupSignalsResponse
                             */
                            MutateAssetGroupSignalsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupSignalsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse} m MutateAssetGroupSignalsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupSignalsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupSignalsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupSignalsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupSignalsResponse;
                        })();
    
                        services.MutateAssetGroupSignalResult = (function() {
    
                            /**
                             * Properties of a MutateAssetGroupSignalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetGroupSignalResult
                             * @property {string|null} [resource_name] MutateAssetGroupSignalResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAssetGroupSignal|null} [asset_group_signal] MutateAssetGroupSignalResult asset_group_signal
                             */
    
                            /**
                             * Constructs a new MutateAssetGroupSignalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetGroupSignalResult.
                             * @implements IMutateAssetGroupSignalResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalResult=} [p] Properties to set
                             */
                            function MutateAssetGroupSignalResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetGroupSignalResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @instance
                             */
                            MutateAssetGroupSignalResult.prototype.resource_name = "";
    
                            /**
                             * MutateAssetGroupSignalResult asset_group_signal.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupSignal|null|undefined} asset_group_signal
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @instance
                             */
                            MutateAssetGroupSignalResult.prototype.asset_group_signal = null;
    
                            /**
                             * Creates a new MutateAssetGroupSignalResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalResult} MutateAssetGroupSignalResult instance
                             */
                            MutateAssetGroupSignalResult.create = function create(properties) {
                                return new MutateAssetGroupSignalResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupSignalResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupSignalResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalResult} m MutateAssetGroupSignalResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupSignalResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_group_signal != null && Object.hasOwnProperty.call(m, "asset_group_signal"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupSignal.encode(m.asset_group_signal, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetGroupSignalResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetGroupSignalResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetGroupSignalResult} message MutateAssetGroupSignalResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetGroupSignalResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetGroupSignalResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalResult} MutateAssetGroupSignalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupSignalResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_group_signal = $root.google.ads.googleads.v11.resources.AssetGroupSignal.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetGroupSignalResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalResult} MutateAssetGroupSignalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetGroupSignalResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetGroupSignalResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetGroupSignalResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_group_signal != null && m.hasOwnProperty("asset_group_signal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupSignal.verify(m.asset_group_signal);
                                        if (e)
                                            return "asset_group_signal." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetGroupSignalResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetGroupSignalResult} MutateAssetGroupSignalResult
                             */
                            MutateAssetGroupSignalResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_group_signal != null) {
                                    if (typeof d.asset_group_signal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetGroupSignalResult.asset_group_signal: object expected");
                                    m.asset_group_signal = $root.google.ads.googleads.v11.resources.AssetGroupSignal.fromObject(d.asset_group_signal);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetGroupSignalResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetGroupSignalResult} m MutateAssetGroupSignalResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetGroupSignalResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_group_signal = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_group_signal != null && m.hasOwnProperty("asset_group_signal")) {
                                    d.asset_group_signal = $root.google.ads.googleads.v11.resources.AssetGroupSignal.toObject(m.asset_group_signal, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetGroupSignalResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetGroupSignalResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetGroupSignalResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetGroupSignalResult;
                        })();
    
                        services.AssetService = (function() {
    
                            /**
                             * Constructs a new AssetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AssetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AssetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AssetService;
    
                            /**
                             * Creates new AssetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AssetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AssetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AssetService|mutateAssets}.
                             * @memberof google.ads.googleads.v11.services.AssetService
                             * @typedef MutateAssetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAssetsResponse} [response] MutateAssetsResponse
                             */
    
                            /**
                             * Calls MutateAssets.
                             * @function mutateAssets
                             * @memberof google.ads.googleads.v11.services.AssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetsRequest} request MutateAssetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AssetService.MutateAssetsCallback} callback Node-style callback called with the error, if any, and MutateAssetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AssetService.prototype.mutateAssets = function mutateAssets(request, callback) {
                                return this.rpcCall(mutateAssets, $root.google.ads.googleads.v11.services.MutateAssetsRequest, $root.google.ads.googleads.v11.services.MutateAssetsResponse, request, callback);
                            }, "name", { value: "MutateAssets" });
    
                            /**
                             * Calls MutateAssets.
                             * @function mutateAssets
                             * @memberof google.ads.googleads.v11.services.AssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetsRequest} request MutateAssetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAssetsResponse>} Promise
                             * @variation 2
                             */
    
                            return AssetService;
                        })();
    
                        services.MutateAssetsRequest = (function() {
    
                            /**
                             * Properties of a MutateAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetsRequest
                             * @property {string|null} [customer_id] MutateAssetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAssetOperation>|null} [operations] MutateAssetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAssetsRequest partial_failure
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAssetsRequest response_content_type
                             * @property {boolean|null} [validate_only] MutateAssetsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetsRequest.
                             * @implements IMutateAssetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetsRequest=} [p] Properties to set
                             */
                            function MutateAssetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @instance
                             */
                            MutateAssetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAssetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAssetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @instance
                             */
                            MutateAssetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAssetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @instance
                             */
                            MutateAssetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAssetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @instance
                             */
                            MutateAssetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * MutateAssetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @instance
                             */
                            MutateAssetsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateAssetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetsRequest} MutateAssetsRequest instance
                             */
                            MutateAssetsRequest.create = function create(properties) {
                                return new MutateAssetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetsRequest} m MutateAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AssetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(24).int32(m.response_content_type);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(40).bool(m.partial_failure);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetsRequest} message MutateAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetsRequest} MutateAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AssetOperation.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 3:
                                        m.response_content_type = r.int32();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetsRequest} MutateAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AssetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetsRequest} MutateAssetsRequest
                             */
                            MutateAssetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AssetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetsRequest} m MutateAssetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.validate_only = false;
                                    d.partial_failure = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AssetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetsRequest;
                        })();
    
                        services.AssetOperation = (function() {
    
                            /**
                             * Properties of an AssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAssetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AssetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAsset|null} [create] AssetOperation create
                             * @property {google.ads.googleads.v11.resources.IAsset|null} [update] AssetOperation update
                             */
    
                            /**
                             * Constructs a new AssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetOperation.
                             * @implements IAssetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAssetOperation=} [p] Properties to set
                             */
                            function AssetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @instance
                             */
                            AssetOperation.prototype.update_mask = null;
    
                            /**
                             * AssetOperation create.
                             * @member {google.ads.googleads.v11.resources.IAsset|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @instance
                             */
                            AssetOperation.prototype.create = null;
    
                            /**
                             * AssetOperation update.
                             * @member {google.ads.googleads.v11.resources.IAsset|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @instance
                             */
                            AssetOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AssetOperation operation.
                             * @member {"create"|"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @instance
                             */
                            Object.defineProperty(AssetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AssetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AssetOperation} AssetOperation instance
                             */
                            AssetOperation.create = function create(properties) {
                                return new AssetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AssetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AssetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetOperation} m AssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.Asset.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Asset.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AssetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetOperation} message AssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AssetOperation} AssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AssetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.Asset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.Asset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AssetOperation} AssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Asset.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Asset.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AssetOperation} AssetOperation
                             */
                            AssetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AssetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AssetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.Asset.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Asset.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AssetOperation} m AssetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.Asset.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Asset.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AssetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetOperation;
                        })();
    
                        services.MutateAssetsResponse = (function() {
    
                            /**
                             * Properties of a MutateAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAssetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAssetResult>|null} [results] MutateAssetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetsResponse.
                             * @implements IMutateAssetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetsResponse=} [p] Properties to set
                             */
                            function MutateAssetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @instance
                             */
                            MutateAssetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateAssetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAssetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @instance
                             */
                            MutateAssetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateAssetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetsResponse} MutateAssetsResponse instance
                             */
                            MutateAssetsResponse.create = function create(properties) {
                                return new MutateAssetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetsResponse} m MutateAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAssetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetsResponse} message MutateAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetsResponse} MutateAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAssetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetsResponse} MutateAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAssetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetsResponse} MutateAssetsResponse
                             */
                            MutateAssetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAssetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetsResponse} m MutateAssetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAssetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetsResponse;
                        })();
    
                        services.MutateAssetResult = (function() {
    
                            /**
                             * Properties of a MutateAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetResult
                             * @property {string|null} [resource_name] MutateAssetResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAsset|null} [asset] MutateAssetResult asset
                             */
    
                            /**
                             * Constructs a new MutateAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetResult.
                             * @implements IMutateAssetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetResult=} [p] Properties to set
                             */
                            function MutateAssetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @instance
                             */
                            MutateAssetResult.prototype.resource_name = "";
    
                            /**
                             * MutateAssetResult asset.
                             * @member {google.ads.googleads.v11.resources.IAsset|null|undefined} asset
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @instance
                             */
                            MutateAssetResult.prototype.asset = null;
    
                            /**
                             * Creates a new MutateAssetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetResult} MutateAssetResult instance
                             */
                            MutateAssetResult.create = function create(properties) {
                                return new MutateAssetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetResult} m MutateAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    $root.google.ads.googleads.v11.resources.Asset.encode(m.asset, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetResult} message MutateAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetResult} MutateAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset = $root.google.ads.googleads.v11.resources.Asset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetResult} MutateAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Asset.verify(m.asset);
                                        if (e)
                                            return "asset." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetResult} MutateAssetResult
                             */
                            MutateAssetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset != null) {
                                    if (typeof d.asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetResult.asset: object expected");
                                    m.asset = $root.google.ads.googleads.v11.resources.Asset.fromObject(d.asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetResult} m MutateAssetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = $root.google.ads.googleads.v11.resources.Asset.toObject(m.asset, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetResult;
                        })();
    
                        services.AssetSetAssetService = (function() {
    
                            /**
                             * Constructs a new AssetSetAssetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetSetAssetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AssetSetAssetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AssetSetAssetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AssetSetAssetService;
    
                            /**
                             * Creates new AssetSetAssetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AssetSetAssetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AssetSetAssetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AssetSetAssetService|mutateAssetSetAssets}.
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetService
                             * @typedef MutateAssetSetAssetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAssetSetAssetsResponse} [response] MutateAssetSetAssetsResponse
                             */
    
                            /**
                             * Calls MutateAssetSetAssets.
                             * @function mutateAssetSetAssets
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsRequest} request MutateAssetSetAssetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AssetSetAssetService.MutateAssetSetAssetsCallback} callback Node-style callback called with the error, if any, and MutateAssetSetAssetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AssetSetAssetService.prototype.mutateAssetSetAssets = function mutateAssetSetAssets(request, callback) {
                                return this.rpcCall(mutateAssetSetAssets, $root.google.ads.googleads.v11.services.MutateAssetSetAssetsRequest, $root.google.ads.googleads.v11.services.MutateAssetSetAssetsResponse, request, callback);
                            }, "name", { value: "MutateAssetSetAssets" });
    
                            /**
                             * Calls MutateAssetSetAssets.
                             * @function mutateAssetSetAssets
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsRequest} request MutateAssetSetAssetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAssetSetAssetsResponse>} Promise
                             * @variation 2
                             */
    
                            return AssetSetAssetService;
                        })();
    
                        services.MutateAssetSetAssetsRequest = (function() {
    
                            /**
                             * Properties of a MutateAssetSetAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetSetAssetsRequest
                             * @property {string|null} [customer_id] MutateAssetSetAssetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAssetSetAssetOperation>|null} [operations] MutateAssetSetAssetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAssetSetAssetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAssetSetAssetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAssetSetAssetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAssetSetAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetSetAssetsRequest.
                             * @implements IMutateAssetSetAssetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsRequest=} [p] Properties to set
                             */
                            function MutateAssetSetAssetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetSetAssetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @instance
                             */
                            MutateAssetSetAssetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAssetSetAssetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAssetSetAssetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @instance
                             */
                            MutateAssetSetAssetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAssetSetAssetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @instance
                             */
                            MutateAssetSetAssetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAssetSetAssetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @instance
                             */
                            MutateAssetSetAssetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAssetSetAssetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @instance
                             */
                            MutateAssetSetAssetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAssetSetAssetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsRequest} MutateAssetSetAssetsRequest instance
                             */
                            MutateAssetSetAssetsRequest.create = function create(properties) {
                                return new MutateAssetSetAssetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetAssetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetAssetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsRequest} m MutateAssetSetAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetAssetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AssetSetAssetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetAssetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetAssetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsRequest} message MutateAssetSetAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetAssetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetSetAssetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsRequest} MutateAssetSetAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetAssetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetSetAssetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AssetSetAssetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetSetAssetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsRequest} MutateAssetSetAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetAssetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetSetAssetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetSetAssetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AssetSetAssetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetSetAssetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsRequest} MutateAssetSetAssetsRequest
                             */
                            MutateAssetSetAssetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetSetAssetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetSetAssetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetAssetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetAssetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AssetSetAssetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetSetAssetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetSetAssetsRequest} m MutateAssetSetAssetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetSetAssetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AssetSetAssetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetSetAssetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetSetAssetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetSetAssetsRequest;
                        })();
    
                        services.AssetSetAssetOperation = (function() {
    
                            /**
                             * Properties of an AssetSetAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAssetSetAssetOperation
                             * @property {google.ads.googleads.v11.resources.IAssetSetAsset|null} [create] AssetSetAssetOperation create
                             * @property {string|null} [remove] AssetSetAssetOperation remove
                             */
    
                            /**
                             * Constructs a new AssetSetAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetSetAssetOperation.
                             * @implements IAssetSetAssetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAssetSetAssetOperation=} [p] Properties to set
                             */
                            function AssetSetAssetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetSetAssetOperation create.
                             * @member {google.ads.googleads.v11.resources.IAssetSetAsset|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @instance
                             */
                            AssetSetAssetOperation.prototype.create = null;
    
                            /**
                             * AssetSetAssetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @instance
                             */
                            AssetSetAssetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AssetSetAssetOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @instance
                             */
                            Object.defineProperty(AssetSetAssetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AssetSetAssetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetSetAssetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AssetSetAssetOperation} AssetSetAssetOperation instance
                             */
                            AssetSetAssetOperation.create = function create(properties) {
                                return new AssetSetAssetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetAssetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AssetSetAssetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetSetAssetOperation} m AssetSetAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAssetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AssetSetAsset.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetAssetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AssetSetAssetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetSetAssetOperation} message AssetSetAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetAssetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetAssetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AssetSetAssetOperation} AssetSetAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAssetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AssetSetAssetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AssetSetAsset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetAssetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AssetSetAssetOperation} AssetSetAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetAssetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetAssetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetAssetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSetAsset.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetAssetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AssetSetAssetOperation} AssetSetAssetOperation
                             */
                            AssetSetAssetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AssetSetAssetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AssetSetAssetOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetSetAssetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AssetSetAsset.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetSetAssetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AssetSetAssetOperation} m AssetSetAssetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetAssetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AssetSetAsset.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetSetAssetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AssetSetAssetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetAssetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetSetAssetOperation;
                        })();
    
                        services.MutateAssetSetAssetsResponse = (function() {
    
                            /**
                             * Properties of a MutateAssetSetAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetSetAssetsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAssetSetAssetResult>|null} [results] MutateAssetSetAssetsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAssetSetAssetsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAssetSetAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetSetAssetsResponse.
                             * @implements IMutateAssetSetAssetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsResponse=} [p] Properties to set
                             */
                            function MutateAssetSetAssetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetSetAssetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAssetSetAssetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @instance
                             */
                            MutateAssetSetAssetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAssetSetAssetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @instance
                             */
                            MutateAssetSetAssetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAssetSetAssetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsResponse} MutateAssetSetAssetsResponse instance
                             */
                            MutateAssetSetAssetsResponse.create = function create(properties) {
                                return new MutateAssetSetAssetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetAssetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetAssetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsResponse} m MutateAssetSetAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetAssetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetAssetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetAssetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetsResponse} message MutateAssetSetAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetAssetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetSetAssetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsResponse} MutateAssetSetAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetAssetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetSetAssetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetSetAssetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsResponse} MutateAssetSetAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetAssetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetSetAssetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetSetAssetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetSetAssetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetsResponse} MutateAssetSetAssetsResponse
                             */
                            MutateAssetSetAssetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetSetAssetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetSetAssetsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetAssetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetAssetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetAssetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetSetAssetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetSetAssetsResponse} m MutateAssetSetAssetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetSetAssetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetSetAssetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetSetAssetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetSetAssetsResponse;
                        })();
    
                        services.MutateAssetSetAssetResult = (function() {
    
                            /**
                             * Properties of a MutateAssetSetAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetSetAssetResult
                             * @property {string|null} [resource_name] MutateAssetSetAssetResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAssetSetAsset|null} [asset_set_asset] MutateAssetSetAssetResult asset_set_asset
                             */
    
                            /**
                             * Constructs a new MutateAssetSetAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetSetAssetResult.
                             * @implements IMutateAssetSetAssetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetResult=} [p] Properties to set
                             */
                            function MutateAssetSetAssetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetSetAssetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @instance
                             */
                            MutateAssetSetAssetResult.prototype.resource_name = "";
    
                            /**
                             * MutateAssetSetAssetResult asset_set_asset.
                             * @member {google.ads.googleads.v11.resources.IAssetSetAsset|null|undefined} asset_set_asset
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @instance
                             */
                            MutateAssetSetAssetResult.prototype.asset_set_asset = null;
    
                            /**
                             * Creates a new MutateAssetSetAssetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetResult} MutateAssetSetAssetResult instance
                             */
                            MutateAssetSetAssetResult.create = function create(properties) {
                                return new MutateAssetSetAssetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetAssetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetAssetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetResult} m MutateAssetSetAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetAssetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_set_asset != null && Object.hasOwnProperty.call(m, "asset_set_asset"))
                                    $root.google.ads.googleads.v11.resources.AssetSetAsset.encode(m.asset_set_asset, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetAssetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetAssetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetAssetResult} message MutateAssetSetAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetAssetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetSetAssetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetResult} MutateAssetSetAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetAssetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetSetAssetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetResult} MutateAssetSetAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetAssetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetSetAssetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetSetAssetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_set_asset != null && m.hasOwnProperty("asset_set_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSetAsset.verify(m.asset_set_asset);
                                        if (e)
                                            return "asset_set_asset." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetSetAssetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetAssetResult} MutateAssetSetAssetResult
                             */
                            MutateAssetSetAssetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_set_asset != null) {
                                    if (typeof d.asset_set_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetAssetResult.asset_set_asset: object expected");
                                    m.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.fromObject(d.asset_set_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetSetAssetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetSetAssetResult} m MutateAssetSetAssetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetSetAssetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_set_asset = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_set_asset != null && m.hasOwnProperty("asset_set_asset")) {
                                    d.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.toObject(m.asset_set_asset, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetSetAssetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetAssetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetSetAssetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetSetAssetResult;
                        })();
    
                        services.AssetSetService = (function() {
    
                            /**
                             * Constructs a new AssetSetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetSetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AssetSetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AssetSetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AssetSetService;
    
                            /**
                             * Creates new AssetSetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetSetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AssetSetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AssetSetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AssetSetService|mutateAssetSets}.
                             * @memberof google.ads.googleads.v11.services.AssetSetService
                             * @typedef MutateAssetSetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAssetSetsResponse} [response] MutateAssetSetsResponse
                             */
    
                            /**
                             * Calls MutateAssetSets.
                             * @function mutateAssetSets
                             * @memberof google.ads.googleads.v11.services.AssetSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsRequest} request MutateAssetSetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AssetSetService.MutateAssetSetsCallback} callback Node-style callback called with the error, if any, and MutateAssetSetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AssetSetService.prototype.mutateAssetSets = function mutateAssetSets(request, callback) {
                                return this.rpcCall(mutateAssetSets, $root.google.ads.googleads.v11.services.MutateAssetSetsRequest, $root.google.ads.googleads.v11.services.MutateAssetSetsResponse, request, callback);
                            }, "name", { value: "MutateAssetSets" });
    
                            /**
                             * Calls MutateAssetSets.
                             * @function mutateAssetSets
                             * @memberof google.ads.googleads.v11.services.AssetSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsRequest} request MutateAssetSetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAssetSetsResponse>} Promise
                             * @variation 2
                             */
    
                            return AssetSetService;
                        })();
    
                        services.MutateAssetSetsRequest = (function() {
    
                            /**
                             * Properties of a MutateAssetSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetSetsRequest
                             * @property {string|null} [customer_id] MutateAssetSetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAssetSetOperation>|null} [operations] MutateAssetSetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateAssetSetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAssetSetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAssetSetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAssetSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetSetsRequest.
                             * @implements IMutateAssetSetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsRequest=} [p] Properties to set
                             */
                            function MutateAssetSetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetSetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @instance
                             */
                            MutateAssetSetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAssetSetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAssetSetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @instance
                             */
                            MutateAssetSetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAssetSetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @instance
                             */
                            MutateAssetSetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAssetSetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @instance
                             */
                            MutateAssetSetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAssetSetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @instance
                             */
                            MutateAssetSetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAssetSetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsRequest} MutateAssetSetsRequest instance
                             */
                            MutateAssetSetsRequest.create = function create(properties) {
                                return new MutateAssetSetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsRequest} m MutateAssetSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AssetSetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsRequest} message MutateAssetSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetSetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsRequest} MutateAssetSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetSetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AssetSetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetSetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsRequest} MutateAssetSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetSetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetSetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AssetSetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetSetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsRequest} MutateAssetSetsRequest
                             */
                            MutateAssetSetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetSetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetSetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AssetSetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetSetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetSetsRequest} m MutateAssetSetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetSetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AssetSetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetSetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetSetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetSetsRequest;
                        })();
    
                        services.AssetSetOperation = (function() {
    
                            /**
                             * Properties of an AssetSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAssetSetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AssetSetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAssetSet|null} [create] AssetSetOperation create
                             * @property {google.ads.googleads.v11.resources.IAssetSet|null} [update] AssetSetOperation update
                             * @property {string|null} [remove] AssetSetOperation remove
                             */
    
                            /**
                             * Constructs a new AssetSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AssetSetOperation.
                             * @implements IAssetSetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAssetSetOperation=} [p] Properties to set
                             */
                            function AssetSetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AssetSetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @instance
                             */
                            AssetSetOperation.prototype.update_mask = null;
    
                            /**
                             * AssetSetOperation create.
                             * @member {google.ads.googleads.v11.resources.IAssetSet|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @instance
                             */
                            AssetSetOperation.prototype.create = null;
    
                            /**
                             * AssetSetOperation update.
                             * @member {google.ads.googleads.v11.resources.IAssetSet|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @instance
                             */
                            AssetSetOperation.prototype.update = null;
    
                            /**
                             * AssetSetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @instance
                             */
                            AssetSetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AssetSetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @instance
                             */
                            Object.defineProperty(AssetSetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AssetSetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetSetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AssetSetOperation} AssetSetOperation instance
                             */
                            AssetSetOperation.create = function create(properties) {
                                return new AssetSetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AssetSetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AssetSetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetSetOperation} m AssetSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.AssetSet.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.AssetSet.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AssetSetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AssetSetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAssetSetOperation} message AssetSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AssetSetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AssetSetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AssetSetOperation} AssetSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AssetSetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.AssetSet.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.AssetSet.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AssetSetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AssetSetOperation} AssetSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AssetSetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AssetSetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AssetSetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSet.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSet.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AssetSetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AssetSetOperation} AssetSetOperation
                             */
                            AssetSetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AssetSetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AssetSetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetSetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetSetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.AssetSet.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AssetSetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.AssetSet.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AssetSetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AssetSetOperation} m AssetSetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AssetSetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.AssetSet.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.AssetSet.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AssetSetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AssetSetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AssetSetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AssetSetOperation;
                        })();
    
                        services.MutateAssetSetsResponse = (function() {
    
                            /**
                             * Properties of a MutateAssetSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetSetsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAssetSetResult>|null} [results] MutateAssetSetsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAssetSetsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAssetSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetSetsResponse.
                             * @implements IMutateAssetSetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsResponse=} [p] Properties to set
                             */
                            function MutateAssetSetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetSetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAssetSetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @instance
                             */
                            MutateAssetSetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAssetSetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @instance
                             */
                            MutateAssetSetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAssetSetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsResponse} MutateAssetSetsResponse instance
                             */
                            MutateAssetSetsResponse.create = function create(properties) {
                                return new MutateAssetSetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsResponse} m MutateAssetSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAssetSetResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetsResponse} message MutateAssetSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetSetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsResponse} MutateAssetSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetSetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAssetSetResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetSetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsResponse} MutateAssetSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetSetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetSetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAssetSetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetSetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetsResponse} MutateAssetSetsResponse
                             */
                            MutateAssetSetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetSetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetSetsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAssetSetResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetSetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetSetsResponse} m MutateAssetSetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetSetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAssetSetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetSetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetSetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetSetsResponse;
                        })();
    
                        services.MutateAssetSetResult = (function() {
    
                            /**
                             * Properties of a MutateAssetSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAssetSetResult
                             * @property {string|null} [resource_name] MutateAssetSetResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAssetSet|null} [asset_set] MutateAssetSetResult asset_set
                             */
    
                            /**
                             * Constructs a new MutateAssetSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAssetSetResult.
                             * @implements IMutateAssetSetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetResult=} [p] Properties to set
                             */
                            function MutateAssetSetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAssetSetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @instance
                             */
                            MutateAssetSetResult.prototype.resource_name = "";
    
                            /**
                             * MutateAssetSetResult asset_set.
                             * @member {google.ads.googleads.v11.resources.IAssetSet|null|undefined} asset_set
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @instance
                             */
                            MutateAssetSetResult.prototype.asset_set = null;
    
                            /**
                             * Creates a new MutateAssetSetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetResult} MutateAssetSetResult instance
                             */
                            MutateAssetSetResult.create = function create(properties) {
                                return new MutateAssetSetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetResult} m MutateAssetSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.asset_set != null && Object.hasOwnProperty.call(m, "asset_set"))
                                    $root.google.ads.googleads.v11.resources.AssetSet.encode(m.asset_set, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAssetSetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAssetSetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAssetSetResult} message MutateAssetSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAssetSetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAssetSetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetResult} MutateAssetSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAssetSetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAssetSetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetResult} MutateAssetSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAssetSetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAssetSetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAssetSetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSet.verify(m.asset_set);
                                        if (e)
                                            return "asset_set." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAssetSetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAssetSetResult} MutateAssetSetResult
                             */
                            MutateAssetSetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAssetSetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAssetSetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.asset_set != null) {
                                    if (typeof d.asset_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAssetSetResult.asset_set: object expected");
                                    m.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.fromObject(d.asset_set);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAssetSetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAssetSetResult} m MutateAssetSetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAssetSetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.asset_set = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    d.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.toObject(m.asset_set, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAssetSetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAssetSetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAssetSetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAssetSetResult;
                        })();
    
                        services.AudienceInsightsService = (function() {
    
                            /**
                             * Constructs a new AudienceInsightsService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceInsightsService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AudienceInsightsService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AudienceInsightsService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AudienceInsightsService;
    
                            /**
                             * Creates new AudienceInsightsService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AudienceInsightsService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AudienceInsightsService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AudienceInsightsService|generateInsightsFinderReport}.
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsService
                             * @typedef GenerateInsightsFinderReportCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse} [response] GenerateInsightsFinderReportResponse
                             */
    
                            /**
                             * Calls GenerateInsightsFinderReport.
                             * @function generateInsightsFinderReport
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportRequest} request GenerateInsightsFinderReportRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AudienceInsightsService.GenerateInsightsFinderReportCallback} callback Node-style callback called with the error, if any, and GenerateInsightsFinderReportResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AudienceInsightsService.prototype.generateInsightsFinderReport = function generateInsightsFinderReport(request, callback) {
                                return this.rpcCall(generateInsightsFinderReport, $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest, $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse, request, callback);
                            }, "name", { value: "GenerateInsightsFinderReport" });
    
                            /**
                             * Calls GenerateInsightsFinderReport.
                             * @function generateInsightsFinderReport
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportRequest} request GenerateInsightsFinderReportRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AudienceInsightsService|listAudienceInsightsAttributes}.
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsService
                             * @typedef ListAudienceInsightsAttributesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse} [response] ListAudienceInsightsAttributesResponse
                             */
    
                            /**
                             * Calls ListAudienceInsightsAttributes.
                             * @function listAudienceInsightsAttributes
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesRequest} request ListAudienceInsightsAttributesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AudienceInsightsService.ListAudienceInsightsAttributesCallback} callback Node-style callback called with the error, if any, and ListAudienceInsightsAttributesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AudienceInsightsService.prototype.listAudienceInsightsAttributes = function listAudienceInsightsAttributes(request, callback) {
                                return this.rpcCall(listAudienceInsightsAttributes, $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest, $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse, request, callback);
                            }, "name", { value: "ListAudienceInsightsAttributes" });
    
                            /**
                             * Calls ListAudienceInsightsAttributes.
                             * @function listAudienceInsightsAttributes
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesRequest} request ListAudienceInsightsAttributesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse>} Promise
                             * @variation 2
                             */
    
                            return AudienceInsightsService;
                        })();
    
                        services.GenerateInsightsFinderReportRequest = (function() {
    
                            /**
                             * Properties of a GenerateInsightsFinderReportRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateInsightsFinderReportRequest
                             * @property {string|null} [customer_id] GenerateInsightsFinderReportRequest customer_id
                             * @property {google.ads.googleads.v11.services.IBasicInsightsAudience|null} [baseline_audience] GenerateInsightsFinderReportRequest baseline_audience
                             * @property {google.ads.googleads.v11.services.IBasicInsightsAudience|null} [specific_audience] GenerateInsightsFinderReportRequest specific_audience
                             * @property {string|null} [customer_insights_group] GenerateInsightsFinderReportRequest customer_insights_group
                             */
    
                            /**
                             * Constructs a new GenerateInsightsFinderReportRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateInsightsFinderReportRequest.
                             * @implements IGenerateInsightsFinderReportRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportRequest=} [p] Properties to set
                             */
                            function GenerateInsightsFinderReportRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateInsightsFinderReportRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @instance
                             */
                            GenerateInsightsFinderReportRequest.prototype.customer_id = "";
    
                            /**
                             * GenerateInsightsFinderReportRequest baseline_audience.
                             * @member {google.ads.googleads.v11.services.IBasicInsightsAudience|null|undefined} baseline_audience
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @instance
                             */
                            GenerateInsightsFinderReportRequest.prototype.baseline_audience = null;
    
                            /**
                             * GenerateInsightsFinderReportRequest specific_audience.
                             * @member {google.ads.googleads.v11.services.IBasicInsightsAudience|null|undefined} specific_audience
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @instance
                             */
                            GenerateInsightsFinderReportRequest.prototype.specific_audience = null;
    
                            /**
                             * GenerateInsightsFinderReportRequest customer_insights_group.
                             * @member {string} customer_insights_group
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @instance
                             */
                            GenerateInsightsFinderReportRequest.prototype.customer_insights_group = "";
    
                            /**
                             * Creates a new GenerateInsightsFinderReportRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest} GenerateInsightsFinderReportRequest instance
                             */
                            GenerateInsightsFinderReportRequest.create = function create(properties) {
                                return new GenerateInsightsFinderReportRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateInsightsFinderReportRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportRequest} m GenerateInsightsFinderReportRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateInsightsFinderReportRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.baseline_audience != null && Object.hasOwnProperty.call(m, "baseline_audience"))
                                    $root.google.ads.googleads.v11.services.BasicInsightsAudience.encode(m.baseline_audience, w.uint32(18).fork()).ldelim();
                                if (m.specific_audience != null && Object.hasOwnProperty.call(m, "specific_audience"))
                                    $root.google.ads.googleads.v11.services.BasicInsightsAudience.encode(m.specific_audience, w.uint32(26).fork()).ldelim();
                                if (m.customer_insights_group != null && Object.hasOwnProperty.call(m, "customer_insights_group"))
                                    w.uint32(34).string(m.customer_insights_group);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateInsightsFinderReportRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportRequest} message GenerateInsightsFinderReportRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateInsightsFinderReportRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateInsightsFinderReportRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest} GenerateInsightsFinderReportRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateInsightsFinderReportRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.baseline_audience = $root.google.ads.googleads.v11.services.BasicInsightsAudience.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.specific_audience = $root.google.ads.googleads.v11.services.BasicInsightsAudience.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.customer_insights_group = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateInsightsFinderReportRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest} GenerateInsightsFinderReportRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateInsightsFinderReportRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateInsightsFinderReportRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateInsightsFinderReportRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.baseline_audience != null && m.hasOwnProperty("baseline_audience")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.BasicInsightsAudience.verify(m.baseline_audience);
                                        if (e)
                                            return "baseline_audience." + e;
                                    }
                                }
                                if (m.specific_audience != null && m.hasOwnProperty("specific_audience")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.BasicInsightsAudience.verify(m.specific_audience);
                                        if (e)
                                            return "specific_audience." + e;
                                    }
                                }
                                if (m.customer_insights_group != null && m.hasOwnProperty("customer_insights_group")) {
                                    if (!$util.isString(m.customer_insights_group))
                                        return "customer_insights_group: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateInsightsFinderReportRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest} GenerateInsightsFinderReportRequest
                             */
                            GenerateInsightsFinderReportRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.baseline_audience != null) {
                                    if (typeof d.baseline_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest.baseline_audience: object expected");
                                    m.baseline_audience = $root.google.ads.googleads.v11.services.BasicInsightsAudience.fromObject(d.baseline_audience);
                                }
                                if (d.specific_audience != null) {
                                    if (typeof d.specific_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest.specific_audience: object expected");
                                    m.specific_audience = $root.google.ads.googleads.v11.services.BasicInsightsAudience.fromObject(d.specific_audience);
                                }
                                if (d.customer_insights_group != null) {
                                    m.customer_insights_group = String(d.customer_insights_group);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateInsightsFinderReportRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest} m GenerateInsightsFinderReportRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateInsightsFinderReportRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.baseline_audience = null;
                                    d.specific_audience = null;
                                    d.customer_insights_group = "";
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.baseline_audience != null && m.hasOwnProperty("baseline_audience")) {
                                    d.baseline_audience = $root.google.ads.googleads.v11.services.BasicInsightsAudience.toObject(m.baseline_audience, o);
                                }
                                if (m.specific_audience != null && m.hasOwnProperty("specific_audience")) {
                                    d.specific_audience = $root.google.ads.googleads.v11.services.BasicInsightsAudience.toObject(m.specific_audience, o);
                                }
                                if (m.customer_insights_group != null && m.hasOwnProperty("customer_insights_group")) {
                                    d.customer_insights_group = m.customer_insights_group;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateInsightsFinderReportRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateInsightsFinderReportRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateInsightsFinderReportRequest;
                        })();
    
                        services.GenerateInsightsFinderReportResponse = (function() {
    
                            /**
                             * Properties of a GenerateInsightsFinderReportResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateInsightsFinderReportResponse
                             * @property {string|null} [saved_report_url] GenerateInsightsFinderReportResponse saved_report_url
                             */
    
                            /**
                             * Constructs a new GenerateInsightsFinderReportResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateInsightsFinderReportResponse.
                             * @implements IGenerateInsightsFinderReportResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportResponse=} [p] Properties to set
                             */
                            function GenerateInsightsFinderReportResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateInsightsFinderReportResponse saved_report_url.
                             * @member {string} saved_report_url
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @instance
                             */
                            GenerateInsightsFinderReportResponse.prototype.saved_report_url = "";
    
                            /**
                             * Creates a new GenerateInsightsFinderReportResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse} GenerateInsightsFinderReportResponse instance
                             */
                            GenerateInsightsFinderReportResponse.create = function create(properties) {
                                return new GenerateInsightsFinderReportResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateInsightsFinderReportResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportResponse} m GenerateInsightsFinderReportResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateInsightsFinderReportResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.saved_report_url != null && Object.hasOwnProperty.call(m, "saved_report_url"))
                                    w.uint32(10).string(m.saved_report_url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateInsightsFinderReportResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateInsightsFinderReportResponse} message GenerateInsightsFinderReportResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateInsightsFinderReportResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateInsightsFinderReportResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse} GenerateInsightsFinderReportResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateInsightsFinderReportResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.saved_report_url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateInsightsFinderReportResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse} GenerateInsightsFinderReportResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateInsightsFinderReportResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateInsightsFinderReportResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateInsightsFinderReportResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.saved_report_url != null && m.hasOwnProperty("saved_report_url")) {
                                    if (!$util.isString(m.saved_report_url))
                                        return "saved_report_url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateInsightsFinderReportResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse} GenerateInsightsFinderReportResponse
                             */
                            GenerateInsightsFinderReportResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse();
                                if (d.saved_report_url != null) {
                                    m.saved_report_url = String(d.saved_report_url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateInsightsFinderReportResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse} m GenerateInsightsFinderReportResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateInsightsFinderReportResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.saved_report_url = "";
                                }
                                if (m.saved_report_url != null && m.hasOwnProperty("saved_report_url")) {
                                    d.saved_report_url = m.saved_report_url;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateInsightsFinderReportResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateInsightsFinderReportResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateInsightsFinderReportResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateInsightsFinderReportResponse;
                        })();
    
                        services.ListAudienceInsightsAttributesRequest = (function() {
    
                            /**
                             * Properties of a ListAudienceInsightsAttributesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListAudienceInsightsAttributesRequest
                             * @property {string|null} [customer_id] ListAudienceInsightsAttributesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.AudienceInsightsDimension>|null} [dimensions] ListAudienceInsightsAttributesRequest dimensions
                             * @property {string|null} [query_text] ListAudienceInsightsAttributesRequest query_text
                             * @property {string|null} [customer_insights_group] ListAudienceInsightsAttributesRequest customer_insights_group
                             */
    
                            /**
                             * Constructs a new ListAudienceInsightsAttributesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListAudienceInsightsAttributesRequest.
                             * @implements IListAudienceInsightsAttributesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesRequest=} [p] Properties to set
                             */
                            function ListAudienceInsightsAttributesRequest(p) {
                                this.dimensions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListAudienceInsightsAttributesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @instance
                             */
                            ListAudienceInsightsAttributesRequest.prototype.customer_id = "";
    
                            /**
                             * ListAudienceInsightsAttributesRequest dimensions.
                             * @member {Array.<google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.AudienceInsightsDimension>} dimensions
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @instance
                             */
                            ListAudienceInsightsAttributesRequest.prototype.dimensions = $util.emptyArray;
    
                            /**
                             * ListAudienceInsightsAttributesRequest query_text.
                             * @member {string} query_text
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @instance
                             */
                            ListAudienceInsightsAttributesRequest.prototype.query_text = "";
    
                            /**
                             * ListAudienceInsightsAttributesRequest customer_insights_group.
                             * @member {string} customer_insights_group
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @instance
                             */
                            ListAudienceInsightsAttributesRequest.prototype.customer_insights_group = "";
    
                            /**
                             * Creates a new ListAudienceInsightsAttributesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest} ListAudienceInsightsAttributesRequest instance
                             */
                            ListAudienceInsightsAttributesRequest.create = function create(properties) {
                                return new ListAudienceInsightsAttributesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListAudienceInsightsAttributesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesRequest} m ListAudienceInsightsAttributesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAudienceInsightsAttributesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.dimensions != null && m.dimensions.length) {
                                    w.uint32(18).fork();
                                    for (var i = 0; i < m.dimensions.length; ++i)
                                        w.int32(m.dimensions[i]);
                                    w.ldelim();
                                }
                                if (m.query_text != null && Object.hasOwnProperty.call(m, "query_text"))
                                    w.uint32(26).string(m.query_text);
                                if (m.customer_insights_group != null && Object.hasOwnProperty.call(m, "customer_insights_group"))
                                    w.uint32(34).string(m.customer_insights_group);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListAudienceInsightsAttributesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesRequest} message ListAudienceInsightsAttributesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAudienceInsightsAttributesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListAudienceInsightsAttributesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest} ListAudienceInsightsAttributesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAudienceInsightsAttributesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.dimensions && m.dimensions.length))
                                            m.dimensions = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.dimensions.push(r.int32());
                                        } else
                                            m.dimensions.push(r.int32());
                                        break;
                                    case 3:
                                        m.query_text = r.string();
                                        break;
                                    case 4:
                                        m.customer_insights_group = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListAudienceInsightsAttributesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest} ListAudienceInsightsAttributesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAudienceInsightsAttributesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListAudienceInsightsAttributesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListAudienceInsightsAttributesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.dimensions != null && m.hasOwnProperty("dimensions")) {
                                    if (!Array.isArray(m.dimensions))
                                        return "dimensions: array expected";
                                    for (var i = 0; i < m.dimensions.length; ++i) {
                                        switch (m.dimensions[i]) {
                                        default:
                                            return "dimensions: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                            break;
                                        }
                                    }
                                }
                                if (m.query_text != null && m.hasOwnProperty("query_text")) {
                                    if (!$util.isString(m.query_text))
                                        return "query_text: string expected";
                                }
                                if (m.customer_insights_group != null && m.hasOwnProperty("customer_insights_group")) {
                                    if (!$util.isString(m.customer_insights_group))
                                        return "customer_insights_group: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListAudienceInsightsAttributesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest} ListAudienceInsightsAttributesRequest
                             */
                            ListAudienceInsightsAttributesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.dimensions) {
                                    if (!Array.isArray(d.dimensions))
                                        throw TypeError(".google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest.dimensions: array expected");
                                    m.dimensions = [];
                                    for (var i = 0; i < d.dimensions.length; ++i) {
                                        switch (d.dimensions[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.dimensions[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.dimensions[i] = 1;
                                            break;
                                        case "CATEGORY":
                                        case 2:
                                            m.dimensions[i] = 2;
                                            break;
                                        case "KNOWLEDGE_GRAPH":
                                        case 3:
                                            m.dimensions[i] = 3;
                                            break;
                                        }
                                    }
                                }
                                if (d.query_text != null) {
                                    m.query_text = String(d.query_text);
                                }
                                if (d.customer_insights_group != null) {
                                    m.customer_insights_group = String(d.customer_insights_group);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListAudienceInsightsAttributesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest} m ListAudienceInsightsAttributesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListAudienceInsightsAttributesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.dimensions = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.query_text = "";
                                    d.customer_insights_group = "";
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.dimensions && m.dimensions.length) {
                                    d.dimensions = [];
                                    for (var j = 0; j < m.dimensions.length; ++j) {
                                        d.dimensions[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.AudienceInsightsDimension[m.dimensions[j]] : m.dimensions[j];
                                    }
                                }
                                if (m.query_text != null && m.hasOwnProperty("query_text")) {
                                    d.query_text = m.query_text;
                                }
                                if (m.customer_insights_group != null && m.hasOwnProperty("customer_insights_group")) {
                                    d.customer_insights_group = m.customer_insights_group;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListAudienceInsightsAttributesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListAudienceInsightsAttributesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListAudienceInsightsAttributesRequest;
                        })();
    
                        services.ListAudienceInsightsAttributesResponse = (function() {
    
                            /**
                             * Properties of a ListAudienceInsightsAttributesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListAudienceInsightsAttributesResponse
                             * @property {Array.<google.ads.googleads.v11.services.IAudienceInsightsAttributeMetadata>|null} [attributes] ListAudienceInsightsAttributesResponse attributes
                             */
    
                            /**
                             * Constructs a new ListAudienceInsightsAttributesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListAudienceInsightsAttributesResponse.
                             * @implements IListAudienceInsightsAttributesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesResponse=} [p] Properties to set
                             */
                            function ListAudienceInsightsAttributesResponse(p) {
                                this.attributes = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListAudienceInsightsAttributesResponse attributes.
                             * @member {Array.<google.ads.googleads.v11.services.IAudienceInsightsAttributeMetadata>} attributes
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @instance
                             */
                            ListAudienceInsightsAttributesResponse.prototype.attributes = $util.emptyArray;
    
                            /**
                             * Creates a new ListAudienceInsightsAttributesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse} ListAudienceInsightsAttributesResponse instance
                             */
                            ListAudienceInsightsAttributesResponse.create = function create(properties) {
                                return new ListAudienceInsightsAttributesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListAudienceInsightsAttributesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesResponse} m ListAudienceInsightsAttributesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAudienceInsightsAttributesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.attributes != null && m.attributes.length) {
                                    for (var i = 0; i < m.attributes.length; ++i)
                                        $root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.encode(m.attributes[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListAudienceInsightsAttributesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAudienceInsightsAttributesResponse} message ListAudienceInsightsAttributesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAudienceInsightsAttributesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListAudienceInsightsAttributesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse} ListAudienceInsightsAttributesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAudienceInsightsAttributesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.attributes && m.attributes.length))
                                            m.attributes = [];
                                        m.attributes.push($root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListAudienceInsightsAttributesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse} ListAudienceInsightsAttributesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAudienceInsightsAttributesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListAudienceInsightsAttributesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListAudienceInsightsAttributesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.attributes != null && m.hasOwnProperty("attributes")) {
                                    if (!Array.isArray(m.attributes))
                                        return "attributes: array expected";
                                    for (var i = 0; i < m.attributes.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.verify(m.attributes[i]);
                                            if (e)
                                                return "attributes." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListAudienceInsightsAttributesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse} ListAudienceInsightsAttributesResponse
                             */
                            ListAudienceInsightsAttributesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse();
                                if (d.attributes) {
                                    if (!Array.isArray(d.attributes))
                                        throw TypeError(".google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse.attributes: array expected");
                                    m.attributes = [];
                                    for (var i = 0; i < d.attributes.length; ++i) {
                                        if (typeof d.attributes[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse.attributes: object expected");
                                        m.attributes[i] = $root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.fromObject(d.attributes[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListAudienceInsightsAttributesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse} m ListAudienceInsightsAttributesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListAudienceInsightsAttributesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.attributes = [];
                                }
                                if (m.attributes && m.attributes.length) {
                                    d.attributes = [];
                                    for (var j = 0; j < m.attributes.length; ++j) {
                                        d.attributes[j] = $root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.toObject(m.attributes[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListAudienceInsightsAttributesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListAudienceInsightsAttributesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListAudienceInsightsAttributesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListAudienceInsightsAttributesResponse;
                        })();
    
                        services.AudienceInsightsAttribute = (function() {
    
                            /**
                             * Properties of an AudienceInsightsAttribute.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAudienceInsightsAttribute
                             * @property {google.ads.googleads.v11.common.IAgeRangeInfo|null} [age_range] AudienceInsightsAttribute age_range
                             * @property {google.ads.googleads.v11.common.IGenderInfo|null} [gender] AudienceInsightsAttribute gender
                             * @property {google.ads.googleads.v11.common.ILocationInfo|null} [location] AudienceInsightsAttribute location
                             * @property {google.ads.googleads.v11.common.IUserInterestInfo|null} [user_interest] AudienceInsightsAttribute user_interest
                             * @property {google.ads.googleads.v11.services.IAudienceInsightsEntity|null} [entity] AudienceInsightsAttribute entity
                             * @property {google.ads.googleads.v11.services.IAudienceInsightsCategory|null} [category] AudienceInsightsAttribute category
                             */
    
                            /**
                             * Constructs a new AudienceInsightsAttribute.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceInsightsAttribute.
                             * @implements IAudienceInsightsAttribute
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttribute=} [p] Properties to set
                             */
                            function AudienceInsightsAttribute(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceInsightsAttribute age_range.
                             * @member {google.ads.googleads.v11.common.IAgeRangeInfo|null|undefined} age_range
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             */
                            AudienceInsightsAttribute.prototype.age_range = null;
    
                            /**
                             * AudienceInsightsAttribute gender.
                             * @member {google.ads.googleads.v11.common.IGenderInfo|null|undefined} gender
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             */
                            AudienceInsightsAttribute.prototype.gender = null;
    
                            /**
                             * AudienceInsightsAttribute location.
                             * @member {google.ads.googleads.v11.common.ILocationInfo|null|undefined} location
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             */
                            AudienceInsightsAttribute.prototype.location = null;
    
                            /**
                             * AudienceInsightsAttribute user_interest.
                             * @member {google.ads.googleads.v11.common.IUserInterestInfo|null|undefined} user_interest
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             */
                            AudienceInsightsAttribute.prototype.user_interest = null;
    
                            /**
                             * AudienceInsightsAttribute entity.
                             * @member {google.ads.googleads.v11.services.IAudienceInsightsEntity|null|undefined} entity
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             */
                            AudienceInsightsAttribute.prototype.entity = null;
    
                            /**
                             * AudienceInsightsAttribute category.
                             * @member {google.ads.googleads.v11.services.IAudienceInsightsCategory|null|undefined} category
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             */
                            AudienceInsightsAttribute.prototype.category = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AudienceInsightsAttribute attribute.
                             * @member {"age_range"|"gender"|"location"|"user_interest"|"entity"|"category"|undefined} attribute
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             */
                            Object.defineProperty(AudienceInsightsAttribute.prototype, "attribute", {
                                get: $util.oneOfGetter($oneOfFields = ["age_range", "gender", "location", "user_interest", "entity", "category"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AudienceInsightsAttribute instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttribute=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttribute} AudienceInsightsAttribute instance
                             */
                            AudienceInsightsAttribute.create = function create(properties) {
                                return new AudienceInsightsAttribute(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsAttribute message. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsAttribute.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttribute} m AudienceInsightsAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsAttribute.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.age_range != null && Object.hasOwnProperty.call(m, "age_range"))
                                    $root.google.ads.googleads.v11.common.AgeRangeInfo.encode(m.age_range, w.uint32(10).fork()).ldelim();
                                if (m.gender != null && Object.hasOwnProperty.call(m, "gender"))
                                    $root.google.ads.googleads.v11.common.GenderInfo.encode(m.gender, w.uint32(18).fork()).ldelim();
                                if (m.location != null && Object.hasOwnProperty.call(m, "location"))
                                    $root.google.ads.googleads.v11.common.LocationInfo.encode(m.location, w.uint32(26).fork()).ldelim();
                                if (m.user_interest != null && Object.hasOwnProperty.call(m, "user_interest"))
                                    $root.google.ads.googleads.v11.common.UserInterestInfo.encode(m.user_interest, w.uint32(34).fork()).ldelim();
                                if (m.entity != null && Object.hasOwnProperty.call(m, "entity"))
                                    $root.google.ads.googleads.v11.services.AudienceInsightsEntity.encode(m.entity, w.uint32(42).fork()).ldelim();
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    $root.google.ads.googleads.v11.services.AudienceInsightsCategory.encode(m.category, w.uint32(50).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsAttribute message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsAttribute.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttribute} message AudienceInsightsAttribute message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsAttribute.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceInsightsAttribute message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttribute} AudienceInsightsAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsAttribute.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AudienceInsightsAttribute();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.gender = $root.google.ads.googleads.v11.common.GenderInfo.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.location = $root.google.ads.googleads.v11.common.LocationInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.entity = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.category = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceInsightsAttribute message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttribute} AudienceInsightsAttribute
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsAttribute.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceInsightsAttribute message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceInsightsAttribute.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    p.attribute = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.AgeRangeInfo.verify(m.age_range);
                                        if (e)
                                            return "age_range." + e;
                                    }
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    if (p.attribute === 1)
                                        return "attribute: multiple values";
                                    p.attribute = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GenderInfo.verify(m.gender);
                                        if (e)
                                            return "gender." + e;
                                    }
                                }
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    if (p.attribute === 1)
                                        return "attribute: multiple values";
                                    p.attribute = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.LocationInfo.verify(m.location);
                                        if (e)
                                            return "location." + e;
                                    }
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    if (p.attribute === 1)
                                        return "attribute: multiple values";
                                    p.attribute = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserInterestInfo.verify(m.user_interest);
                                        if (e)
                                            return "user_interest." + e;
                                    }
                                }
                                if (m.entity != null && m.hasOwnProperty("entity")) {
                                    if (p.attribute === 1)
                                        return "attribute: multiple values";
                                    p.attribute = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.verify(m.entity);
                                        if (e)
                                            return "entity." + e;
                                    }
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    if (p.attribute === 1)
                                        return "attribute: multiple values";
                                    p.attribute = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.verify(m.category);
                                        if (e)
                                            return "category." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceInsightsAttribute message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttribute} AudienceInsightsAttribute
                             */
                            AudienceInsightsAttribute.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AudienceInsightsAttribute)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AudienceInsightsAttribute();
                                if (d.age_range != null) {
                                    if (typeof d.age_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsAttribute.age_range: object expected");
                                    m.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.fromObject(d.age_range);
                                }
                                if (d.gender != null) {
                                    if (typeof d.gender !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsAttribute.gender: object expected");
                                    m.gender = $root.google.ads.googleads.v11.common.GenderInfo.fromObject(d.gender);
                                }
                                if (d.location != null) {
                                    if (typeof d.location !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsAttribute.location: object expected");
                                    m.location = $root.google.ads.googleads.v11.common.LocationInfo.fromObject(d.location);
                                }
                                if (d.user_interest != null) {
                                    if (typeof d.user_interest !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsAttribute.user_interest: object expected");
                                    m.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.fromObject(d.user_interest);
                                }
                                if (d.entity != null) {
                                    if (typeof d.entity !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsAttribute.entity: object expected");
                                    m.entity = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.fromObject(d.entity);
                                }
                                if (d.category != null) {
                                    if (typeof d.category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsAttribute.category: object expected");
                                    m.category = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.fromObject(d.category);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceInsightsAttribute message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @static
                             * @param {google.ads.googleads.v11.services.AudienceInsightsAttribute} m AudienceInsightsAttribute
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceInsightsAttribute.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    d.age_range = $root.google.ads.googleads.v11.common.AgeRangeInfo.toObject(m.age_range, o);
                                    if (o.oneofs)
                                        d.attribute = "age_range";
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    d.gender = $root.google.ads.googleads.v11.common.GenderInfo.toObject(m.gender, o);
                                    if (o.oneofs)
                                        d.attribute = "gender";
                                }
                                if (m.location != null && m.hasOwnProperty("location")) {
                                    d.location = $root.google.ads.googleads.v11.common.LocationInfo.toObject(m.location, o);
                                    if (o.oneofs)
                                        d.attribute = "location";
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    d.user_interest = $root.google.ads.googleads.v11.common.UserInterestInfo.toObject(m.user_interest, o);
                                    if (o.oneofs)
                                        d.attribute = "user_interest";
                                }
                                if (m.entity != null && m.hasOwnProperty("entity")) {
                                    d.entity = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.toObject(m.entity, o);
                                    if (o.oneofs)
                                        d.attribute = "entity";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.toObject(m.category, o);
                                    if (o.oneofs)
                                        d.attribute = "category";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceInsightsAttribute to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttribute
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceInsightsAttribute.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceInsightsAttribute;
                        })();
    
                        services.AudienceInsightsTopic = (function() {
    
                            /**
                             * Properties of an AudienceInsightsTopic.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAudienceInsightsTopic
                             * @property {google.ads.googleads.v11.services.IAudienceInsightsEntity|null} [entity] AudienceInsightsTopic entity
                             * @property {google.ads.googleads.v11.services.IAudienceInsightsCategory|null} [category] AudienceInsightsTopic category
                             */
    
                            /**
                             * Constructs a new AudienceInsightsTopic.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceInsightsTopic.
                             * @implements IAudienceInsightsTopic
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsTopic=} [p] Properties to set
                             */
                            function AudienceInsightsTopic(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceInsightsTopic entity.
                             * @member {google.ads.googleads.v11.services.IAudienceInsightsEntity|null|undefined} entity
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @instance
                             */
                            AudienceInsightsTopic.prototype.entity = null;
    
                            /**
                             * AudienceInsightsTopic category.
                             * @member {google.ads.googleads.v11.services.IAudienceInsightsCategory|null|undefined} category
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @instance
                             */
                            AudienceInsightsTopic.prototype.category = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AudienceInsightsTopic topic.
                             * @member {"entity"|"category"|undefined} topic
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @instance
                             */
                            Object.defineProperty(AudienceInsightsTopic.prototype, "topic", {
                                get: $util.oneOfGetter($oneOfFields = ["entity", "category"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AudienceInsightsTopic instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsTopic=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsTopic} AudienceInsightsTopic instance
                             */
                            AudienceInsightsTopic.create = function create(properties) {
                                return new AudienceInsightsTopic(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsTopic message. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsTopic.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsTopic} m AudienceInsightsTopic message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsTopic.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.entity != null && Object.hasOwnProperty.call(m, "entity"))
                                    $root.google.ads.googleads.v11.services.AudienceInsightsEntity.encode(m.entity, w.uint32(10).fork()).ldelim();
                                if (m.category != null && Object.hasOwnProperty.call(m, "category"))
                                    $root.google.ads.googleads.v11.services.AudienceInsightsCategory.encode(m.category, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsTopic message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsTopic.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsTopic} message AudienceInsightsTopic message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsTopic.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceInsightsTopic message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsTopic} AudienceInsightsTopic
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsTopic.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AudienceInsightsTopic();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.entity = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.category = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceInsightsTopic message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsTopic} AudienceInsightsTopic
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsTopic.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceInsightsTopic message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceInsightsTopic.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.entity != null && m.hasOwnProperty("entity")) {
                                    p.topic = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.verify(m.entity);
                                        if (e)
                                            return "entity." + e;
                                    }
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    if (p.topic === 1)
                                        return "topic: multiple values";
                                    p.topic = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.verify(m.category);
                                        if (e)
                                            return "category." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceInsightsTopic message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsTopic} AudienceInsightsTopic
                             */
                            AudienceInsightsTopic.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AudienceInsightsTopic)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AudienceInsightsTopic();
                                if (d.entity != null) {
                                    if (typeof d.entity !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsTopic.entity: object expected");
                                    m.entity = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.fromObject(d.entity);
                                }
                                if (d.category != null) {
                                    if (typeof d.category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsTopic.category: object expected");
                                    m.category = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.fromObject(d.category);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceInsightsTopic message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @static
                             * @param {google.ads.googleads.v11.services.AudienceInsightsTopic} m AudienceInsightsTopic
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceInsightsTopic.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.entity != null && m.hasOwnProperty("entity")) {
                                    d.entity = $root.google.ads.googleads.v11.services.AudienceInsightsEntity.toObject(m.entity, o);
                                    if (o.oneofs)
                                        d.topic = "entity";
                                }
                                if (m.category != null && m.hasOwnProperty("category")) {
                                    d.category = $root.google.ads.googleads.v11.services.AudienceInsightsCategory.toObject(m.category, o);
                                    if (o.oneofs)
                                        d.topic = "category";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceInsightsTopic to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsTopic
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceInsightsTopic.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceInsightsTopic;
                        })();
    
                        services.AudienceInsightsEntity = (function() {
    
                            /**
                             * Properties of an AudienceInsightsEntity.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAudienceInsightsEntity
                             * @property {string|null} [knowledge_graph_machine_id] AudienceInsightsEntity knowledge_graph_machine_id
                             */
    
                            /**
                             * Constructs a new AudienceInsightsEntity.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceInsightsEntity.
                             * @implements IAudienceInsightsEntity
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsEntity=} [p] Properties to set
                             */
                            function AudienceInsightsEntity(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceInsightsEntity knowledge_graph_machine_id.
                             * @member {string} knowledge_graph_machine_id
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @instance
                             */
                            AudienceInsightsEntity.prototype.knowledge_graph_machine_id = "";
    
                            /**
                             * Creates a new AudienceInsightsEntity instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsEntity=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsEntity} AudienceInsightsEntity instance
                             */
                            AudienceInsightsEntity.create = function create(properties) {
                                return new AudienceInsightsEntity(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsEntity message. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsEntity.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsEntity} m AudienceInsightsEntity message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsEntity.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.knowledge_graph_machine_id != null && Object.hasOwnProperty.call(m, "knowledge_graph_machine_id"))
                                    w.uint32(10).string(m.knowledge_graph_machine_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsEntity message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsEntity.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsEntity} message AudienceInsightsEntity message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsEntity.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceInsightsEntity message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsEntity} AudienceInsightsEntity
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsEntity.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AudienceInsightsEntity();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.knowledge_graph_machine_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceInsightsEntity message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsEntity} AudienceInsightsEntity
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsEntity.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceInsightsEntity message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceInsightsEntity.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.knowledge_graph_machine_id != null && m.hasOwnProperty("knowledge_graph_machine_id")) {
                                    if (!$util.isString(m.knowledge_graph_machine_id))
                                        return "knowledge_graph_machine_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceInsightsEntity message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsEntity} AudienceInsightsEntity
                             */
                            AudienceInsightsEntity.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AudienceInsightsEntity)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AudienceInsightsEntity();
                                if (d.knowledge_graph_machine_id != null) {
                                    m.knowledge_graph_machine_id = String(d.knowledge_graph_machine_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceInsightsEntity message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @static
                             * @param {google.ads.googleads.v11.services.AudienceInsightsEntity} m AudienceInsightsEntity
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceInsightsEntity.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.knowledge_graph_machine_id = "";
                                }
                                if (m.knowledge_graph_machine_id != null && m.hasOwnProperty("knowledge_graph_machine_id")) {
                                    d.knowledge_graph_machine_id = m.knowledge_graph_machine_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceInsightsEntity to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsEntity
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceInsightsEntity.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceInsightsEntity;
                        })();
    
                        services.AudienceInsightsCategory = (function() {
    
                            /**
                             * Properties of an AudienceInsightsCategory.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAudienceInsightsCategory
                             * @property {string|null} [category_id] AudienceInsightsCategory category_id
                             */
    
                            /**
                             * Constructs a new AudienceInsightsCategory.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceInsightsCategory.
                             * @implements IAudienceInsightsCategory
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsCategory=} [p] Properties to set
                             */
                            function AudienceInsightsCategory(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceInsightsCategory category_id.
                             * @member {string} category_id
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @instance
                             */
                            AudienceInsightsCategory.prototype.category_id = "";
    
                            /**
                             * Creates a new AudienceInsightsCategory instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsCategory=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsCategory} AudienceInsightsCategory instance
                             */
                            AudienceInsightsCategory.create = function create(properties) {
                                return new AudienceInsightsCategory(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsCategory message. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsCategory.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsCategory} m AudienceInsightsCategory message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsCategory.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.category_id != null && Object.hasOwnProperty.call(m, "category_id"))
                                    w.uint32(10).string(m.category_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsCategory message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsCategory.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsCategory} message AudienceInsightsCategory message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsCategory.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceInsightsCategory message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsCategory} AudienceInsightsCategory
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsCategory.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AudienceInsightsCategory();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.category_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceInsightsCategory message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsCategory} AudienceInsightsCategory
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsCategory.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceInsightsCategory message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceInsightsCategory.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.category_id != null && m.hasOwnProperty("category_id")) {
                                    if (!$util.isString(m.category_id))
                                        return "category_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceInsightsCategory message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsCategory} AudienceInsightsCategory
                             */
                            AudienceInsightsCategory.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AudienceInsightsCategory)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AudienceInsightsCategory();
                                if (d.category_id != null) {
                                    m.category_id = String(d.category_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceInsightsCategory message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @static
                             * @param {google.ads.googleads.v11.services.AudienceInsightsCategory} m AudienceInsightsCategory
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceInsightsCategory.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.category_id = "";
                                }
                                if (m.category_id != null && m.hasOwnProperty("category_id")) {
                                    d.category_id = m.category_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceInsightsCategory to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsCategory
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceInsightsCategory.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceInsightsCategory;
                        })();
    
                        services.BasicInsightsAudience = (function() {
    
                            /**
                             * Properties of a BasicInsightsAudience.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IBasicInsightsAudience
                             * @property {Array.<google.ads.googleads.v11.common.ILocationInfo>|null} [country_location] BasicInsightsAudience country_location
                             * @property {Array.<google.ads.googleads.v11.common.ILocationInfo>|null} [sub_country_locations] BasicInsightsAudience sub_country_locations
                             * @property {google.ads.googleads.v11.common.IGenderInfo|null} [gender] BasicInsightsAudience gender
                             * @property {Array.<google.ads.googleads.v11.common.IAgeRangeInfo>|null} [age_ranges] BasicInsightsAudience age_ranges
                             * @property {Array.<google.ads.googleads.v11.common.IUserInterestInfo>|null} [user_interests] BasicInsightsAudience user_interests
                             * @property {Array.<google.ads.googleads.v11.services.IAudienceInsightsTopic>|null} [topics] BasicInsightsAudience topics
                             */
    
                            /**
                             * Constructs a new BasicInsightsAudience.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BasicInsightsAudience.
                             * @implements IBasicInsightsAudience
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IBasicInsightsAudience=} [p] Properties to set
                             */
                            function BasicInsightsAudience(p) {
                                this.country_location = [];
                                this.sub_country_locations = [];
                                this.age_ranges = [];
                                this.user_interests = [];
                                this.topics = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BasicInsightsAudience country_location.
                             * @member {Array.<google.ads.googleads.v11.common.ILocationInfo>} country_location
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @instance
                             */
                            BasicInsightsAudience.prototype.country_location = $util.emptyArray;
    
                            /**
                             * BasicInsightsAudience sub_country_locations.
                             * @member {Array.<google.ads.googleads.v11.common.ILocationInfo>} sub_country_locations
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @instance
                             */
                            BasicInsightsAudience.prototype.sub_country_locations = $util.emptyArray;
    
                            /**
                             * BasicInsightsAudience gender.
                             * @member {google.ads.googleads.v11.common.IGenderInfo|null|undefined} gender
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @instance
                             */
                            BasicInsightsAudience.prototype.gender = null;
    
                            /**
                             * BasicInsightsAudience age_ranges.
                             * @member {Array.<google.ads.googleads.v11.common.IAgeRangeInfo>} age_ranges
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @instance
                             */
                            BasicInsightsAudience.prototype.age_ranges = $util.emptyArray;
    
                            /**
                             * BasicInsightsAudience user_interests.
                             * @member {Array.<google.ads.googleads.v11.common.IUserInterestInfo>} user_interests
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @instance
                             */
                            BasicInsightsAudience.prototype.user_interests = $util.emptyArray;
    
                            /**
                             * BasicInsightsAudience topics.
                             * @member {Array.<google.ads.googleads.v11.services.IAudienceInsightsTopic>} topics
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @instance
                             */
                            BasicInsightsAudience.prototype.topics = $util.emptyArray;
    
                            /**
                             * Creates a new BasicInsightsAudience instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {google.ads.googleads.v11.services.IBasicInsightsAudience=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.BasicInsightsAudience} BasicInsightsAudience instance
                             */
                            BasicInsightsAudience.create = function create(properties) {
                                return new BasicInsightsAudience(properties);
                            };
    
                            /**
                             * Encodes the specified BasicInsightsAudience message. Does not implicitly {@link google.ads.googleads.v11.services.BasicInsightsAudience.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {google.ads.googleads.v11.services.IBasicInsightsAudience} m BasicInsightsAudience message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BasicInsightsAudience.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.country_location != null && m.country_location.length) {
                                    for (var i = 0; i < m.country_location.length; ++i)
                                        $root.google.ads.googleads.v11.common.LocationInfo.encode(m.country_location[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.sub_country_locations != null && m.sub_country_locations.length) {
                                    for (var i = 0; i < m.sub_country_locations.length; ++i)
                                        $root.google.ads.googleads.v11.common.LocationInfo.encode(m.sub_country_locations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.gender != null && Object.hasOwnProperty.call(m, "gender"))
                                    $root.google.ads.googleads.v11.common.GenderInfo.encode(m.gender, w.uint32(26).fork()).ldelim();
                                if (m.age_ranges != null && m.age_ranges.length) {
                                    for (var i = 0; i < m.age_ranges.length; ++i)
                                        $root.google.ads.googleads.v11.common.AgeRangeInfo.encode(m.age_ranges[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.user_interests != null && m.user_interests.length) {
                                    for (var i = 0; i < m.user_interests.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserInterestInfo.encode(m.user_interests[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.topics != null && m.topics.length) {
                                    for (var i = 0; i < m.topics.length; ++i)
                                        $root.google.ads.googleads.v11.services.AudienceInsightsTopic.encode(m.topics[i], w.uint32(50).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified BasicInsightsAudience message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.BasicInsightsAudience.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {google.ads.googleads.v11.services.IBasicInsightsAudience} message BasicInsightsAudience message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BasicInsightsAudience.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BasicInsightsAudience message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.BasicInsightsAudience} BasicInsightsAudience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BasicInsightsAudience.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.BasicInsightsAudience();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.country_location && m.country_location.length))
                                            m.country_location = [];
                                        m.country_location.push($root.google.ads.googleads.v11.common.LocationInfo.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.sub_country_locations && m.sub_country_locations.length))
                                            m.sub_country_locations = [];
                                        m.sub_country_locations.push($root.google.ads.googleads.v11.common.LocationInfo.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.gender = $root.google.ads.googleads.v11.common.GenderInfo.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        if (!(m.age_ranges && m.age_ranges.length))
                                            m.age_ranges = [];
                                        m.age_ranges.push($root.google.ads.googleads.v11.common.AgeRangeInfo.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        if (!(m.user_interests && m.user_interests.length))
                                            m.user_interests = [];
                                        m.user_interests.push($root.google.ads.googleads.v11.common.UserInterestInfo.decode(r, r.uint32()));
                                        break;
                                    case 6:
                                        if (!(m.topics && m.topics.length))
                                            m.topics = [];
                                        m.topics.push($root.google.ads.googleads.v11.services.AudienceInsightsTopic.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BasicInsightsAudience message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.BasicInsightsAudience} BasicInsightsAudience
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BasicInsightsAudience.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BasicInsightsAudience message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BasicInsightsAudience.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.country_location != null && m.hasOwnProperty("country_location")) {
                                    if (!Array.isArray(m.country_location))
                                        return "country_location: array expected";
                                    for (var i = 0; i < m.country_location.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.LocationInfo.verify(m.country_location[i]);
                                            if (e)
                                                return "country_location." + e;
                                        }
                                    }
                                }
                                if (m.sub_country_locations != null && m.hasOwnProperty("sub_country_locations")) {
                                    if (!Array.isArray(m.sub_country_locations))
                                        return "sub_country_locations: array expected";
                                    for (var i = 0; i < m.sub_country_locations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.LocationInfo.verify(m.sub_country_locations[i]);
                                            if (e)
                                                return "sub_country_locations." + e;
                                        }
                                    }
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.GenderInfo.verify(m.gender);
                                        if (e)
                                            return "gender." + e;
                                    }
                                }
                                if (m.age_ranges != null && m.hasOwnProperty("age_ranges")) {
                                    if (!Array.isArray(m.age_ranges))
                                        return "age_ranges: array expected";
                                    for (var i = 0; i < m.age_ranges.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AgeRangeInfo.verify(m.age_ranges[i]);
                                            if (e)
                                                return "age_ranges." + e;
                                        }
                                    }
                                }
                                if (m.user_interests != null && m.hasOwnProperty("user_interests")) {
                                    if (!Array.isArray(m.user_interests))
                                        return "user_interests: array expected";
                                    for (var i = 0; i < m.user_interests.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserInterestInfo.verify(m.user_interests[i]);
                                            if (e)
                                                return "user_interests." + e;
                                        }
                                    }
                                }
                                if (m.topics != null && m.hasOwnProperty("topics")) {
                                    if (!Array.isArray(m.topics))
                                        return "topics: array expected";
                                    for (var i = 0; i < m.topics.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AudienceInsightsTopic.verify(m.topics[i]);
                                            if (e)
                                                return "topics." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BasicInsightsAudience message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.BasicInsightsAudience} BasicInsightsAudience
                             */
                            BasicInsightsAudience.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.BasicInsightsAudience)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.BasicInsightsAudience();
                                if (d.country_location) {
                                    if (!Array.isArray(d.country_location))
                                        throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.country_location: array expected");
                                    m.country_location = [];
                                    for (var i = 0; i < d.country_location.length; ++i) {
                                        if (typeof d.country_location[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.country_location: object expected");
                                        m.country_location[i] = $root.google.ads.googleads.v11.common.LocationInfo.fromObject(d.country_location[i]);
                                    }
                                }
                                if (d.sub_country_locations) {
                                    if (!Array.isArray(d.sub_country_locations))
                                        throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.sub_country_locations: array expected");
                                    m.sub_country_locations = [];
                                    for (var i = 0; i < d.sub_country_locations.length; ++i) {
                                        if (typeof d.sub_country_locations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.sub_country_locations: object expected");
                                        m.sub_country_locations[i] = $root.google.ads.googleads.v11.common.LocationInfo.fromObject(d.sub_country_locations[i]);
                                    }
                                }
                                if (d.gender != null) {
                                    if (typeof d.gender !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.gender: object expected");
                                    m.gender = $root.google.ads.googleads.v11.common.GenderInfo.fromObject(d.gender);
                                }
                                if (d.age_ranges) {
                                    if (!Array.isArray(d.age_ranges))
                                        throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.age_ranges: array expected");
                                    m.age_ranges = [];
                                    for (var i = 0; i < d.age_ranges.length; ++i) {
                                        if (typeof d.age_ranges[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.age_ranges: object expected");
                                        m.age_ranges[i] = $root.google.ads.googleads.v11.common.AgeRangeInfo.fromObject(d.age_ranges[i]);
                                    }
                                }
                                if (d.user_interests) {
                                    if (!Array.isArray(d.user_interests))
                                        throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.user_interests: array expected");
                                    m.user_interests = [];
                                    for (var i = 0; i < d.user_interests.length; ++i) {
                                        if (typeof d.user_interests[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.user_interests: object expected");
                                        m.user_interests[i] = $root.google.ads.googleads.v11.common.UserInterestInfo.fromObject(d.user_interests[i]);
                                    }
                                }
                                if (d.topics) {
                                    if (!Array.isArray(d.topics))
                                        throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.topics: array expected");
                                    m.topics = [];
                                    for (var i = 0; i < d.topics.length; ++i) {
                                        if (typeof d.topics[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.BasicInsightsAudience.topics: object expected");
                                        m.topics[i] = $root.google.ads.googleads.v11.services.AudienceInsightsTopic.fromObject(d.topics[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BasicInsightsAudience message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @static
                             * @param {google.ads.googleads.v11.services.BasicInsightsAudience} m BasicInsightsAudience
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BasicInsightsAudience.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.country_location = [];
                                    d.sub_country_locations = [];
                                    d.age_ranges = [];
                                    d.user_interests = [];
                                    d.topics = [];
                                }
                                if (o.defaults) {
                                    d.gender = null;
                                }
                                if (m.country_location && m.country_location.length) {
                                    d.country_location = [];
                                    for (var j = 0; j < m.country_location.length; ++j) {
                                        d.country_location[j] = $root.google.ads.googleads.v11.common.LocationInfo.toObject(m.country_location[j], o);
                                    }
                                }
                                if (m.sub_country_locations && m.sub_country_locations.length) {
                                    d.sub_country_locations = [];
                                    for (var j = 0; j < m.sub_country_locations.length; ++j) {
                                        d.sub_country_locations[j] = $root.google.ads.googleads.v11.common.LocationInfo.toObject(m.sub_country_locations[j], o);
                                    }
                                }
                                if (m.gender != null && m.hasOwnProperty("gender")) {
                                    d.gender = $root.google.ads.googleads.v11.common.GenderInfo.toObject(m.gender, o);
                                }
                                if (m.age_ranges && m.age_ranges.length) {
                                    d.age_ranges = [];
                                    for (var j = 0; j < m.age_ranges.length; ++j) {
                                        d.age_ranges[j] = $root.google.ads.googleads.v11.common.AgeRangeInfo.toObject(m.age_ranges[j], o);
                                    }
                                }
                                if (m.user_interests && m.user_interests.length) {
                                    d.user_interests = [];
                                    for (var j = 0; j < m.user_interests.length; ++j) {
                                        d.user_interests[j] = $root.google.ads.googleads.v11.common.UserInterestInfo.toObject(m.user_interests[j], o);
                                    }
                                }
                                if (m.topics && m.topics.length) {
                                    d.topics = [];
                                    for (var j = 0; j < m.topics.length; ++j) {
                                        d.topics[j] = $root.google.ads.googleads.v11.services.AudienceInsightsTopic.toObject(m.topics[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BasicInsightsAudience to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.BasicInsightsAudience
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BasicInsightsAudience.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BasicInsightsAudience;
                        })();
    
                        services.AudienceInsightsAttributeMetadata = (function() {
    
                            /**
                             * Properties of an AudienceInsightsAttributeMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAudienceInsightsAttributeMetadata
                             * @property {google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.AudienceInsightsDimension|null} [dimension] AudienceInsightsAttributeMetadata dimension
                             * @property {google.ads.googleads.v11.services.IAudienceInsightsAttribute|null} [attribute] AudienceInsightsAttributeMetadata attribute
                             * @property {string|null} [display_name] AudienceInsightsAttributeMetadata display_name
                             * @property {number|null} [score] AudienceInsightsAttributeMetadata score
                             * @property {string|null} [display_info] AudienceInsightsAttributeMetadata display_info
                             */
    
                            /**
                             * Constructs a new AudienceInsightsAttributeMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceInsightsAttributeMetadata.
                             * @implements IAudienceInsightsAttributeMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttributeMetadata=} [p] Properties to set
                             */
                            function AudienceInsightsAttributeMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceInsightsAttributeMetadata dimension.
                             * @member {google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.AudienceInsightsDimension} dimension
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @instance
                             */
                            AudienceInsightsAttributeMetadata.prototype.dimension = 0;
    
                            /**
                             * AudienceInsightsAttributeMetadata attribute.
                             * @member {google.ads.googleads.v11.services.IAudienceInsightsAttribute|null|undefined} attribute
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @instance
                             */
                            AudienceInsightsAttributeMetadata.prototype.attribute = null;
    
                            /**
                             * AudienceInsightsAttributeMetadata display_name.
                             * @member {string} display_name
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @instance
                             */
                            AudienceInsightsAttributeMetadata.prototype.display_name = "";
    
                            /**
                             * AudienceInsightsAttributeMetadata score.
                             * @member {number} score
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @instance
                             */
                            AudienceInsightsAttributeMetadata.prototype.score = 0;
    
                            /**
                             * AudienceInsightsAttributeMetadata display_info.
                             * @member {string} display_info
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @instance
                             */
                            AudienceInsightsAttributeMetadata.prototype.display_info = "";
    
                            /**
                             * Creates a new AudienceInsightsAttributeMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttributeMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata} AudienceInsightsAttributeMetadata instance
                             */
                            AudienceInsightsAttributeMetadata.create = function create(properties) {
                                return new AudienceInsightsAttributeMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsAttributeMetadata message. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttributeMetadata} m AudienceInsightsAttributeMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsAttributeMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.dimension != null && Object.hasOwnProperty.call(m, "dimension"))
                                    w.uint32(8).int32(m.dimension);
                                if (m.attribute != null && Object.hasOwnProperty.call(m, "attribute"))
                                    $root.google.ads.googleads.v11.services.AudienceInsightsAttribute.encode(m.attribute, w.uint32(18).fork()).ldelim();
                                if (m.display_name != null && Object.hasOwnProperty.call(m, "display_name"))
                                    w.uint32(26).string(m.display_name);
                                if (m.score != null && Object.hasOwnProperty.call(m, "score"))
                                    w.uint32(33).double(m.score);
                                if (m.display_info != null && Object.hasOwnProperty.call(m, "display_info"))
                                    w.uint32(42).string(m.display_info);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceInsightsAttributeMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceInsightsAttributeMetadata} message AudienceInsightsAttributeMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceInsightsAttributeMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceInsightsAttributeMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata} AudienceInsightsAttributeMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsAttributeMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.dimension = r.int32();
                                        break;
                                    case 2:
                                        m.attribute = $root.google.ads.googleads.v11.services.AudienceInsightsAttribute.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.display_name = r.string();
                                        break;
                                    case 4:
                                        m.score = r.double();
                                        break;
                                    case 5:
                                        m.display_info = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceInsightsAttributeMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata} AudienceInsightsAttributeMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceInsightsAttributeMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceInsightsAttributeMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceInsightsAttributeMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.dimension != null && m.hasOwnProperty("dimension")) {
                                    switch (m.dimension) {
                                    default:
                                        return "dimension: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.attribute != null && m.hasOwnProperty("attribute")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AudienceInsightsAttribute.verify(m.attribute);
                                        if (e)
                                            return "attribute." + e;
                                    }
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    if (!$util.isString(m.display_name))
                                        return "display_name: string expected";
                                }
                                if (m.score != null && m.hasOwnProperty("score")) {
                                    if (typeof m.score !== "number")
                                        return "score: number expected";
                                }
                                if (m.display_info != null && m.hasOwnProperty("display_info")) {
                                    if (!$util.isString(m.display_info))
                                        return "display_info: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceInsightsAttributeMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata} AudienceInsightsAttributeMetadata
                             */
                            AudienceInsightsAttributeMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata();
                                switch (d.dimension) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.dimension = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.dimension = 1;
                                    break;
                                case "CATEGORY":
                                case 2:
                                    m.dimension = 2;
                                    break;
                                case "KNOWLEDGE_GRAPH":
                                case 3:
                                    m.dimension = 3;
                                    break;
                                }
                                if (d.attribute != null) {
                                    if (typeof d.attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata.attribute: object expected");
                                    m.attribute = $root.google.ads.googleads.v11.services.AudienceInsightsAttribute.fromObject(d.attribute);
                                }
                                if (d.display_name != null) {
                                    m.display_name = String(d.display_name);
                                }
                                if (d.score != null) {
                                    m.score = Number(d.score);
                                }
                                if (d.display_info != null) {
                                    m.display_info = String(d.display_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceInsightsAttributeMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata} m AudienceInsightsAttributeMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceInsightsAttributeMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.dimension = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.attribute = null;
                                    d.display_name = "";
                                    d.score = 0;
                                    d.display_info = "";
                                }
                                if (m.dimension != null && m.hasOwnProperty("dimension")) {
                                    d.dimension = o.enums === String ? $root.google.ads.googleads.v11.enums.AudienceInsightsDimensionEnum.AudienceInsightsDimension[m.dimension] : m.dimension;
                                }
                                if (m.attribute != null && m.hasOwnProperty("attribute")) {
                                    d.attribute = $root.google.ads.googleads.v11.services.AudienceInsightsAttribute.toObject(m.attribute, o);
                                }
                                if (m.display_name != null && m.hasOwnProperty("display_name")) {
                                    d.display_name = m.display_name;
                                }
                                if (m.score != null && m.hasOwnProperty("score")) {
                                    d.score = o.json && !isFinite(m.score) ? String(m.score) : m.score;
                                }
                                if (m.display_info != null && m.hasOwnProperty("display_info")) {
                                    d.display_info = m.display_info;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceInsightsAttributeMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AudienceInsightsAttributeMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceInsightsAttributeMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceInsightsAttributeMetadata;
                        })();
    
                        services.AudienceService = (function() {
    
                            /**
                             * Constructs a new AudienceService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function AudienceService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (AudienceService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = AudienceService;
    
                            /**
                             * Creates new AudienceService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {AudienceService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            AudienceService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.AudienceService|mutateAudiences}.
                             * @memberof google.ads.googleads.v11.services.AudienceService
                             * @typedef MutateAudiencesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateAudiencesResponse} [response] MutateAudiencesResponse
                             */
    
                            /**
                             * Calls MutateAudiences.
                             * @function mutateAudiences
                             * @memberof google.ads.googleads.v11.services.AudienceService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesRequest} request MutateAudiencesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.AudienceService.MutateAudiencesCallback} callback Node-style callback called with the error, if any, and MutateAudiencesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(AudienceService.prototype.mutateAudiences = function mutateAudiences(request, callback) {
                                return this.rpcCall(mutateAudiences, $root.google.ads.googleads.v11.services.MutateAudiencesRequest, $root.google.ads.googleads.v11.services.MutateAudiencesResponse, request, callback);
                            }, "name", { value: "MutateAudiences" });
    
                            /**
                             * Calls MutateAudiences.
                             * @function mutateAudiences
                             * @memberof google.ads.googleads.v11.services.AudienceService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesRequest} request MutateAudiencesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateAudiencesResponse>} Promise
                             * @variation 2
                             */
    
                            return AudienceService;
                        })();
    
                        services.MutateAudiencesRequest = (function() {
    
                            /**
                             * Properties of a MutateAudiencesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAudiencesRequest
                             * @property {string|null} [customer_id] MutateAudiencesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IAudienceOperation>|null} [operations] MutateAudiencesRequest operations
                             * @property {boolean|null} [partial_failure] MutateAudiencesRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateAudiencesRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateAudiencesRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateAudiencesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAudiencesRequest.
                             * @implements IMutateAudiencesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesRequest=} [p] Properties to set
                             */
                            function MutateAudiencesRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAudiencesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @instance
                             */
                            MutateAudiencesRequest.prototype.customer_id = "";
    
                            /**
                             * MutateAudiencesRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IAudienceOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @instance
                             */
                            MutateAudiencesRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateAudiencesRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @instance
                             */
                            MutateAudiencesRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateAudiencesRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @instance
                             */
                            MutateAudiencesRequest.prototype.validate_only = false;
    
                            /**
                             * MutateAudiencesRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @instance
                             */
                            MutateAudiencesRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateAudiencesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesRequest} MutateAudiencesRequest instance
                             */
                            MutateAudiencesRequest.create = function create(properties) {
                                return new MutateAudiencesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAudiencesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAudiencesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesRequest} m MutateAudiencesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAudiencesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.AudienceOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAudiencesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAudiencesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesRequest} message MutateAudiencesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAudiencesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAudiencesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesRequest} MutateAudiencesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAudiencesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAudiencesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.AudienceOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAudiencesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesRequest} MutateAudiencesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAudiencesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAudiencesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAudiencesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AudienceOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAudiencesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesRequest} MutateAudiencesRequest
                             */
                            MutateAudiencesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAudiencesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAudiencesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAudiencesRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAudiencesRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.AudienceOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAudiencesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAudiencesRequest} m MutateAudiencesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAudiencesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.AudienceOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAudiencesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAudiencesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAudiencesRequest;
                        })();
    
                        services.MutateAudiencesResponse = (function() {
    
                            /**
                             * Properties of a MutateAudiencesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAudiencesResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateAudienceResult>|null} [results] MutateAudiencesResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateAudiencesResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateAudiencesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAudiencesResponse.
                             * @implements IMutateAudiencesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesResponse=} [p] Properties to set
                             */
                            function MutateAudiencesResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAudiencesResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateAudienceResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @instance
                             */
                            MutateAudiencesResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateAudiencesResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @instance
                             */
                            MutateAudiencesResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateAudiencesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesResponse} MutateAudiencesResponse instance
                             */
                            MutateAudiencesResponse.create = function create(properties) {
                                return new MutateAudiencesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAudiencesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAudiencesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesResponse} m MutateAudiencesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAudiencesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateAudienceResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAudiencesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAudiencesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudiencesResponse} message MutateAudiencesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAudiencesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAudiencesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesResponse} MutateAudiencesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAudiencesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAudiencesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateAudienceResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAudiencesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesResponse} MutateAudiencesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAudiencesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAudiencesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAudiencesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateAudienceResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAudiencesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAudiencesResponse} MutateAudiencesResponse
                             */
                            MutateAudiencesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAudiencesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAudiencesResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAudiencesResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateAudiencesResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateAudienceResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAudiencesResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAudiencesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAudiencesResponse} m MutateAudiencesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAudiencesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateAudienceResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAudiencesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAudiencesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAudiencesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAudiencesResponse;
                        })();
    
                        services.AudienceOperation = (function() {
    
                            /**
                             * Properties of an AudienceOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAudienceOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] AudienceOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IAudience|null} [create] AudienceOperation create
                             * @property {google.ads.googleads.v11.resources.IAudience|null} [update] AudienceOperation update
                             */
    
                            /**
                             * Constructs a new AudienceOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AudienceOperation.
                             * @implements IAudienceOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAudienceOperation=} [p] Properties to set
                             */
                            function AudienceOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AudienceOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @instance
                             */
                            AudienceOperation.prototype.update_mask = null;
    
                            /**
                             * AudienceOperation create.
                             * @member {google.ads.googleads.v11.resources.IAudience|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @instance
                             */
                            AudienceOperation.prototype.create = null;
    
                            /**
                             * AudienceOperation update.
                             * @member {google.ads.googleads.v11.resources.IAudience|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @instance
                             */
                            AudienceOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AudienceOperation operation.
                             * @member {"create"|"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @instance
                             */
                            Object.defineProperty(AudienceOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AudienceOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AudienceOperation} AudienceOperation instance
                             */
                            AudienceOperation.create = function create(properties) {
                                return new AudienceOperation(properties);
                            };
    
                            /**
                             * Encodes the specified AudienceOperation message. Does not implicitly {@link google.ads.googleads.v11.services.AudienceOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceOperation} m AudienceOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.Audience.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Audience.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AudienceOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AudienceOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IAudienceOperation} message AudienceOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AudienceOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AudienceOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AudienceOperation} AudienceOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AudienceOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.Audience.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.Audience.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AudienceOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AudienceOperation} AudienceOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AudienceOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AudienceOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AudienceOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Audience.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Audience.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AudienceOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AudienceOperation} AudienceOperation
                             */
                            AudienceOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AudienceOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AudienceOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.Audience.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AudienceOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Audience.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AudienceOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.AudienceOperation} m AudienceOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AudienceOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.Audience.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Audience.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AudienceOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AudienceOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AudienceOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AudienceOperation;
                        })();
    
                        services.MutateAudienceResult = (function() {
    
                            /**
                             * Properties of a MutateAudienceResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateAudienceResult
                             * @property {string|null} [resource_name] MutateAudienceResult resource_name
                             * @property {google.ads.googleads.v11.resources.IAudience|null} [audience] MutateAudienceResult audience
                             */
    
                            /**
                             * Constructs a new MutateAudienceResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateAudienceResult.
                             * @implements IMutateAudienceResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateAudienceResult=} [p] Properties to set
                             */
                            function MutateAudienceResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateAudienceResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @instance
                             */
                            MutateAudienceResult.prototype.resource_name = "";
    
                            /**
                             * MutateAudienceResult audience.
                             * @member {google.ads.googleads.v11.resources.IAudience|null|undefined} audience
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @instance
                             */
                            MutateAudienceResult.prototype.audience = null;
    
                            /**
                             * Creates a new MutateAudienceResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudienceResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateAudienceResult} MutateAudienceResult instance
                             */
                            MutateAudienceResult.create = function create(properties) {
                                return new MutateAudienceResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateAudienceResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateAudienceResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudienceResult} m MutateAudienceResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAudienceResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.audience != null && Object.hasOwnProperty.call(m, "audience"))
                                    $root.google.ads.googleads.v11.resources.Audience.encode(m.audience, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateAudienceResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateAudienceResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateAudienceResult} message MutateAudienceResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateAudienceResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateAudienceResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateAudienceResult} MutateAudienceResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAudienceResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateAudienceResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.audience = $root.google.ads.googleads.v11.resources.Audience.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateAudienceResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateAudienceResult} MutateAudienceResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateAudienceResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateAudienceResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateAudienceResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Audience.verify(m.audience);
                                        if (e)
                                            return "audience." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateAudienceResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateAudienceResult} MutateAudienceResult
                             */
                            MutateAudienceResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateAudienceResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateAudienceResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.audience != null) {
                                    if (typeof d.audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateAudienceResult.audience: object expected");
                                    m.audience = $root.google.ads.googleads.v11.resources.Audience.fromObject(d.audience);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateAudienceResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateAudienceResult} m MutateAudienceResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateAudienceResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.audience = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    d.audience = $root.google.ads.googleads.v11.resources.Audience.toObject(m.audience, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateAudienceResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateAudienceResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateAudienceResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateAudienceResult;
                        })();
    
                        services.BatchJobService = (function() {
    
                            /**
                             * Constructs a new BatchJobService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BatchJobService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function BatchJobService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (BatchJobService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = BatchJobService;
    
                            /**
                             * Creates new BatchJobService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {BatchJobService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            BatchJobService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BatchJobService|mutateBatchJob}.
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @typedef MutateBatchJobCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateBatchJobResponse} [response] MutateBatchJobResponse
                             */
    
                            /**
                             * Calls MutateBatchJob.
                             * @function mutateBatchJob
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobRequest} request MutateBatchJobRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BatchJobService.MutateBatchJobCallback} callback Node-style callback called with the error, if any, and MutateBatchJobResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BatchJobService.prototype.mutateBatchJob = function mutateBatchJob(request, callback) {
                                return this.rpcCall(mutateBatchJob, $root.google.ads.googleads.v11.services.MutateBatchJobRequest, $root.google.ads.googleads.v11.services.MutateBatchJobResponse, request, callback);
                            }, "name", { value: "MutateBatchJob" });
    
                            /**
                             * Calls MutateBatchJob.
                             * @function mutateBatchJob
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobRequest} request MutateBatchJobRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateBatchJobResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BatchJobService|listBatchJobResults}.
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @typedef ListBatchJobResultsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListBatchJobResultsResponse} [response] ListBatchJobResultsResponse
                             */
    
                            /**
                             * Calls ListBatchJobResults.
                             * @function listBatchJobResults
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsRequest} request ListBatchJobResultsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BatchJobService.ListBatchJobResultsCallback} callback Node-style callback called with the error, if any, and ListBatchJobResultsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BatchJobService.prototype.listBatchJobResults = function listBatchJobResults(request, callback) {
                                return this.rpcCall(listBatchJobResults, $root.google.ads.googleads.v11.services.ListBatchJobResultsRequest, $root.google.ads.googleads.v11.services.ListBatchJobResultsResponse, request, callback);
                            }, "name", { value: "ListBatchJobResults" });
    
                            /**
                             * Calls ListBatchJobResults.
                             * @function listBatchJobResults
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsRequest} request ListBatchJobResultsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListBatchJobResultsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BatchJobService|runBatchJob}.
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @typedef RunBatchJobCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.longrunning.Operation} [response] Operation
                             */
    
                            /**
                             * Calls RunBatchJob.
                             * @function runBatchJob
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IRunBatchJobRequest} request RunBatchJobRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BatchJobService.RunBatchJobCallback} callback Node-style callback called with the error, if any, and Operation
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BatchJobService.prototype.runBatchJob = function runBatchJob(request, callback) {
                                return this.rpcCall(runBatchJob, $root.google.ads.googleads.v11.services.RunBatchJobRequest, $root.google.longrunning.Operation, request, callback);
                            }, "name", { value: "RunBatchJob" });
    
                            /**
                             * Calls RunBatchJob.
                             * @function runBatchJob
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IRunBatchJobRequest} request RunBatchJobRequest message or plain object
                             * @returns {Promise<google.longrunning.Operation>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BatchJobService|addBatchJobOperations}.
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @typedef AddBatchJobOperationsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.AddBatchJobOperationsResponse} [response] AddBatchJobOperationsResponse
                             */
    
                            /**
                             * Calls AddBatchJobOperations.
                             * @function addBatchJobOperations
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsRequest} request AddBatchJobOperationsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BatchJobService.AddBatchJobOperationsCallback} callback Node-style callback called with the error, if any, and AddBatchJobOperationsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BatchJobService.prototype.addBatchJobOperations = function addBatchJobOperations(request, callback) {
                                return this.rpcCall(addBatchJobOperations, $root.google.ads.googleads.v11.services.AddBatchJobOperationsRequest, $root.google.ads.googleads.v11.services.AddBatchJobOperationsResponse, request, callback);
                            }, "name", { value: "AddBatchJobOperations" });
    
                            /**
                             * Calls AddBatchJobOperations.
                             * @function addBatchJobOperations
                             * @memberof google.ads.googleads.v11.services.BatchJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsRequest} request AddBatchJobOperationsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.AddBatchJobOperationsResponse>} Promise
                             * @variation 2
                             */
    
                            return BatchJobService;
                        })();
    
                        services.MutateBatchJobRequest = (function() {
    
                            /**
                             * Properties of a MutateBatchJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBatchJobRequest
                             * @property {string|null} [customer_id] MutateBatchJobRequest customer_id
                             * @property {google.ads.googleads.v11.services.IBatchJobOperation|null} [operation] MutateBatchJobRequest operation
                             */
    
                            /**
                             * Constructs a new MutateBatchJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBatchJobRequest.
                             * @implements IMutateBatchJobRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobRequest=} [p] Properties to set
                             */
                            function MutateBatchJobRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBatchJobRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @instance
                             */
                            MutateBatchJobRequest.prototype.customer_id = "";
    
                            /**
                             * MutateBatchJobRequest operation.
                             * @member {google.ads.googleads.v11.services.IBatchJobOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @instance
                             */
                            MutateBatchJobRequest.prototype.operation = null;
    
                            /**
                             * Creates a new MutateBatchJobRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobRequest} MutateBatchJobRequest instance
                             */
                            MutateBatchJobRequest.create = function create(properties) {
                                return new MutateBatchJobRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBatchJobRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBatchJobRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobRequest} m MutateBatchJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBatchJobRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.BatchJobOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBatchJobRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBatchJobRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobRequest} message MutateBatchJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBatchJobRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBatchJobRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobRequest} MutateBatchJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBatchJobRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBatchJobRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.BatchJobOperation.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBatchJobRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobRequest} MutateBatchJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBatchJobRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBatchJobRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBatchJobRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.BatchJobOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBatchJobRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobRequest} MutateBatchJobRequest
                             */
                            MutateBatchJobRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBatchJobRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBatchJobRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBatchJobRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.BatchJobOperation.fromObject(d.operation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBatchJobRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBatchJobRequest} m MutateBatchJobRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBatchJobRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.BatchJobOperation.toObject(m.operation, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBatchJobRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBatchJobRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBatchJobRequest;
                        })();
    
                        services.BatchJobOperation = (function() {
    
                            /**
                             * Properties of a BatchJobOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IBatchJobOperation
                             * @property {google.ads.googleads.v11.resources.IBatchJob|null} [create] BatchJobOperation create
                             * @property {string|null} [remove] BatchJobOperation remove
                             */
    
                            /**
                             * Constructs a new BatchJobOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BatchJobOperation.
                             * @implements IBatchJobOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IBatchJobOperation=} [p] Properties to set
                             */
                            function BatchJobOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BatchJobOperation create.
                             * @member {google.ads.googleads.v11.resources.IBatchJob|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @instance
                             */
                            BatchJobOperation.prototype.create = null;
    
                            /**
                             * BatchJobOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @instance
                             */
                            BatchJobOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BatchJobOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @instance
                             */
                            Object.defineProperty(BatchJobOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BatchJobOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBatchJobOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.BatchJobOperation} BatchJobOperation instance
                             */
                            BatchJobOperation.create = function create(properties) {
                                return new BatchJobOperation(properties);
                            };
    
                            /**
                             * Encodes the specified BatchJobOperation message. Does not implicitly {@link google.ads.googleads.v11.services.BatchJobOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBatchJobOperation} m BatchJobOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.BatchJob.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(34).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified BatchJobOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.BatchJobOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBatchJobOperation} message BatchJobOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BatchJobOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.BatchJobOperation} BatchJobOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.BatchJobOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.BatchJob.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BatchJobOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.BatchJobOperation} BatchJobOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BatchJobOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BatchJobOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BatchJob.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BatchJobOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.BatchJobOperation} BatchJobOperation
                             */
                            BatchJobOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.BatchJobOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.BatchJobOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BatchJobOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.BatchJob.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BatchJobOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.BatchJobOperation} m BatchJobOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BatchJobOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.BatchJob.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BatchJobOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.BatchJobOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BatchJobOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BatchJobOperation;
                        })();
    
                        services.MutateBatchJobResponse = (function() {
    
                            /**
                             * Properties of a MutateBatchJobResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBatchJobResponse
                             * @property {google.ads.googleads.v11.services.IMutateBatchJobResult|null} [result] MutateBatchJobResponse result
                             */
    
                            /**
                             * Constructs a new MutateBatchJobResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBatchJobResponse.
                             * @implements IMutateBatchJobResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResponse=} [p] Properties to set
                             */
                            function MutateBatchJobResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBatchJobResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateBatchJobResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @instance
                             */
                            MutateBatchJobResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateBatchJobResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResponse} MutateBatchJobResponse instance
                             */
                            MutateBatchJobResponse.create = function create(properties) {
                                return new MutateBatchJobResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBatchJobResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBatchJobResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResponse} m MutateBatchJobResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBatchJobResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateBatchJobResult.encode(m.result, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBatchJobResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBatchJobResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResponse} message MutateBatchJobResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBatchJobResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBatchJobResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResponse} MutateBatchJobResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBatchJobResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBatchJobResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.result = $root.google.ads.googleads.v11.services.MutateBatchJobResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBatchJobResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResponse} MutateBatchJobResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBatchJobResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBatchJobResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBatchJobResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateBatchJobResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBatchJobResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResponse} MutateBatchJobResponse
                             */
                            MutateBatchJobResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBatchJobResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBatchJobResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBatchJobResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateBatchJobResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBatchJobResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBatchJobResponse} m MutateBatchJobResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBatchJobResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateBatchJobResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBatchJobResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBatchJobResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBatchJobResponse;
                        })();
    
                        services.MutateBatchJobResult = (function() {
    
                            /**
                             * Properties of a MutateBatchJobResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBatchJobResult
                             * @property {string|null} [resource_name] MutateBatchJobResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateBatchJobResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBatchJobResult.
                             * @implements IMutateBatchJobResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResult=} [p] Properties to set
                             */
                            function MutateBatchJobResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBatchJobResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @instance
                             */
                            MutateBatchJobResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateBatchJobResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResult} MutateBatchJobResult instance
                             */
                            MutateBatchJobResult.create = function create(properties) {
                                return new MutateBatchJobResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBatchJobResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBatchJobResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResult} m MutateBatchJobResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBatchJobResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBatchJobResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBatchJobResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBatchJobResult} message MutateBatchJobResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBatchJobResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBatchJobResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResult} MutateBatchJobResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBatchJobResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBatchJobResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBatchJobResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResult} MutateBatchJobResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBatchJobResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBatchJobResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBatchJobResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBatchJobResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBatchJobResult} MutateBatchJobResult
                             */
                            MutateBatchJobResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBatchJobResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBatchJobResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBatchJobResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBatchJobResult} m MutateBatchJobResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBatchJobResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBatchJobResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBatchJobResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBatchJobResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBatchJobResult;
                        })();
    
                        services.RunBatchJobRequest = (function() {
    
                            /**
                             * Properties of a RunBatchJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IRunBatchJobRequest
                             * @property {string|null} [resource_name] RunBatchJobRequest resource_name
                             */
    
                            /**
                             * Constructs a new RunBatchJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RunBatchJobRequest.
                             * @implements IRunBatchJobRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IRunBatchJobRequest=} [p] Properties to set
                             */
                            function RunBatchJobRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RunBatchJobRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @instance
                             */
                            RunBatchJobRequest.prototype.resource_name = "";
    
                            /**
                             * Creates a new RunBatchJobRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRunBatchJobRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.RunBatchJobRequest} RunBatchJobRequest instance
                             */
                            RunBatchJobRequest.create = function create(properties) {
                                return new RunBatchJobRequest(properties);
                            };
    
                            /**
                             * Encodes the specified RunBatchJobRequest message. Does not implicitly {@link google.ads.googleads.v11.services.RunBatchJobRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRunBatchJobRequest} m RunBatchJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RunBatchJobRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified RunBatchJobRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.RunBatchJobRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRunBatchJobRequest} message RunBatchJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RunBatchJobRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RunBatchJobRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.RunBatchJobRequest} RunBatchJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RunBatchJobRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.RunBatchJobRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RunBatchJobRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.RunBatchJobRequest} RunBatchJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RunBatchJobRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RunBatchJobRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RunBatchJobRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RunBatchJobRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.RunBatchJobRequest} RunBatchJobRequest
                             */
                            RunBatchJobRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.RunBatchJobRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.RunBatchJobRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RunBatchJobRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.RunBatchJobRequest} m RunBatchJobRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RunBatchJobRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RunBatchJobRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.RunBatchJobRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RunBatchJobRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RunBatchJobRequest;
                        })();
    
                        services.AddBatchJobOperationsRequest = (function() {
    
                            /**
                             * Properties of an AddBatchJobOperationsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAddBatchJobOperationsRequest
                             * @property {string|null} [resource_name] AddBatchJobOperationsRequest resource_name
                             * @property {string|null} [sequence_token] AddBatchJobOperationsRequest sequence_token
                             * @property {Array.<google.ads.googleads.v11.services.IMutateOperation>|null} [mutate_operations] AddBatchJobOperationsRequest mutate_operations
                             */
    
                            /**
                             * Constructs a new AddBatchJobOperationsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AddBatchJobOperationsRequest.
                             * @implements IAddBatchJobOperationsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsRequest=} [p] Properties to set
                             */
                            function AddBatchJobOperationsRequest(p) {
                                this.mutate_operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AddBatchJobOperationsRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @instance
                             */
                            AddBatchJobOperationsRequest.prototype.resource_name = "";
    
                            /**
                             * AddBatchJobOperationsRequest sequence_token.
                             * @member {string} sequence_token
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @instance
                             */
                            AddBatchJobOperationsRequest.prototype.sequence_token = "";
    
                            /**
                             * AddBatchJobOperationsRequest mutate_operations.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateOperation>} mutate_operations
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @instance
                             */
                            AddBatchJobOperationsRequest.prototype.mutate_operations = $util.emptyArray;
    
                            /**
                             * Creates a new AddBatchJobOperationsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsRequest} AddBatchJobOperationsRequest instance
                             */
                            AddBatchJobOperationsRequest.create = function create(properties) {
                                return new AddBatchJobOperationsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified AddBatchJobOperationsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.AddBatchJobOperationsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsRequest} m AddBatchJobOperationsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddBatchJobOperationsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.sequence_token != null && Object.hasOwnProperty.call(m, "sequence_token"))
                                    w.uint32(18).string(m.sequence_token);
                                if (m.mutate_operations != null && m.mutate_operations.length) {
                                    for (var i = 0; i < m.mutate_operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateOperation.encode(m.mutate_operations[i], w.uint32(26).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified AddBatchJobOperationsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AddBatchJobOperationsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsRequest} message AddBatchJobOperationsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddBatchJobOperationsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AddBatchJobOperationsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsRequest} AddBatchJobOperationsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddBatchJobOperationsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AddBatchJobOperationsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.sequence_token = r.string();
                                        break;
                                    case 3:
                                        if (!(m.mutate_operations && m.mutate_operations.length))
                                            m.mutate_operations = [];
                                        m.mutate_operations.push($root.google.ads.googleads.v11.services.MutateOperation.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AddBatchJobOperationsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsRequest} AddBatchJobOperationsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddBatchJobOperationsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AddBatchJobOperationsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AddBatchJobOperationsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.sequence_token != null && m.hasOwnProperty("sequence_token")) {
                                    if (!$util.isString(m.sequence_token))
                                        return "sequence_token: string expected";
                                }
                                if (m.mutate_operations != null && m.hasOwnProperty("mutate_operations")) {
                                    if (!Array.isArray(m.mutate_operations))
                                        return "mutate_operations: array expected";
                                    for (var i = 0; i < m.mutate_operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateOperation.verify(m.mutate_operations[i]);
                                            if (e)
                                                return "mutate_operations." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AddBatchJobOperationsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsRequest} AddBatchJobOperationsRequest
                             */
                            AddBatchJobOperationsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AddBatchJobOperationsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AddBatchJobOperationsRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.sequence_token != null) {
                                    m.sequence_token = String(d.sequence_token);
                                }
                                if (d.mutate_operations) {
                                    if (!Array.isArray(d.mutate_operations))
                                        throw TypeError(".google.ads.googleads.v11.services.AddBatchJobOperationsRequest.mutate_operations: array expected");
                                    m.mutate_operations = [];
                                    for (var i = 0; i < d.mutate_operations.length; ++i) {
                                        if (typeof d.mutate_operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.AddBatchJobOperationsRequest.mutate_operations: object expected");
                                        m.mutate_operations[i] = $root.google.ads.googleads.v11.services.MutateOperation.fromObject(d.mutate_operations[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AddBatchJobOperationsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.AddBatchJobOperationsRequest} m AddBatchJobOperationsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AddBatchJobOperationsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.mutate_operations = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.sequence_token = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.sequence_token != null && m.hasOwnProperty("sequence_token")) {
                                    d.sequence_token = m.sequence_token;
                                }
                                if (m.mutate_operations && m.mutate_operations.length) {
                                    d.mutate_operations = [];
                                    for (var j = 0; j < m.mutate_operations.length; ++j) {
                                        d.mutate_operations[j] = $root.google.ads.googleads.v11.services.MutateOperation.toObject(m.mutate_operations[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AddBatchJobOperationsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AddBatchJobOperationsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AddBatchJobOperationsRequest;
                        })();
    
                        services.AddBatchJobOperationsResponse = (function() {
    
                            /**
                             * Properties of an AddBatchJobOperationsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAddBatchJobOperationsResponse
                             * @property {number|null} [total_operations] AddBatchJobOperationsResponse total_operations
                             * @property {string|null} [next_sequence_token] AddBatchJobOperationsResponse next_sequence_token
                             */
    
                            /**
                             * Constructs a new AddBatchJobOperationsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AddBatchJobOperationsResponse.
                             * @implements IAddBatchJobOperationsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsResponse=} [p] Properties to set
                             */
                            function AddBatchJobOperationsResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AddBatchJobOperationsResponse total_operations.
                             * @member {number} total_operations
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @instance
                             */
                            AddBatchJobOperationsResponse.prototype.total_operations = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * AddBatchJobOperationsResponse next_sequence_token.
                             * @member {string} next_sequence_token
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @instance
                             */
                            AddBatchJobOperationsResponse.prototype.next_sequence_token = "";
    
                            /**
                             * Creates a new AddBatchJobOperationsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsResponse} AddBatchJobOperationsResponse instance
                             */
                            AddBatchJobOperationsResponse.create = function create(properties) {
                                return new AddBatchJobOperationsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified AddBatchJobOperationsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.AddBatchJobOperationsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsResponse} m AddBatchJobOperationsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddBatchJobOperationsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.total_operations != null && Object.hasOwnProperty.call(m, "total_operations"))
                                    w.uint32(8).int64(m.total_operations);
                                if (m.next_sequence_token != null && Object.hasOwnProperty.call(m, "next_sequence_token"))
                                    w.uint32(18).string(m.next_sequence_token);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AddBatchJobOperationsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AddBatchJobOperationsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddBatchJobOperationsResponse} message AddBatchJobOperationsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddBatchJobOperationsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AddBatchJobOperationsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsResponse} AddBatchJobOperationsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddBatchJobOperationsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AddBatchJobOperationsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.total_operations = r.int64();
                                        break;
                                    case 2:
                                        m.next_sequence_token = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AddBatchJobOperationsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsResponse} AddBatchJobOperationsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddBatchJobOperationsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AddBatchJobOperationsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AddBatchJobOperationsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.total_operations != null && m.hasOwnProperty("total_operations")) {
                                    if (!$util.isInteger(m.total_operations) && !(m.total_operations && $util.isInteger(m.total_operations.low) && $util.isInteger(m.total_operations.high)))
                                        return "total_operations: integer|Long expected";
                                }
                                if (m.next_sequence_token != null && m.hasOwnProperty("next_sequence_token")) {
                                    if (!$util.isString(m.next_sequence_token))
                                        return "next_sequence_token: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AddBatchJobOperationsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AddBatchJobOperationsResponse} AddBatchJobOperationsResponse
                             */
                            AddBatchJobOperationsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AddBatchJobOperationsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AddBatchJobOperationsResponse();
                                if (d.total_operations != null) {
                                    if ($util.Long)
                                        (m.total_operations = $util.Long.fromValue(d.total_operations)).unsigned = false;
                                    else if (typeof d.total_operations === "string")
                                        m.total_operations = parseInt(d.total_operations, 10);
                                    else if (typeof d.total_operations === "number")
                                        m.total_operations = d.total_operations;
                                    else if (typeof d.total_operations === "object")
                                        m.total_operations = new $util.LongBits(d.total_operations.low >>> 0, d.total_operations.high >>> 0).toNumber();
                                }
                                if (d.next_sequence_token != null) {
                                    m.next_sequence_token = String(d.next_sequence_token);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AddBatchJobOperationsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.AddBatchJobOperationsResponse} m AddBatchJobOperationsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AddBatchJobOperationsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_operations = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_operations = o.longs === String ? "0" : 0;
                                    d.next_sequence_token = "";
                                }
                                if (m.total_operations != null && m.hasOwnProperty("total_operations")) {
                                    if (typeof m.total_operations === "number")
                                        d.total_operations = o.longs === String ? String(m.total_operations) : m.total_operations;
                                    else
                                        d.total_operations = o.longs === String ? $util.Long.prototype.toString.call(m.total_operations) : o.longs === Number ? new $util.LongBits(m.total_operations.low >>> 0, m.total_operations.high >>> 0).toNumber() : m.total_operations;
                                }
                                if (m.next_sequence_token != null && m.hasOwnProperty("next_sequence_token")) {
                                    d.next_sequence_token = m.next_sequence_token;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AddBatchJobOperationsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AddBatchJobOperationsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AddBatchJobOperationsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AddBatchJobOperationsResponse;
                        })();
    
                        services.ListBatchJobResultsRequest = (function() {
    
                            /**
                             * Properties of a ListBatchJobResultsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListBatchJobResultsRequest
                             * @property {string|null} [resource_name] ListBatchJobResultsRequest resource_name
                             * @property {string|null} [page_token] ListBatchJobResultsRequest page_token
                             * @property {number|null} [page_size] ListBatchJobResultsRequest page_size
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] ListBatchJobResultsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new ListBatchJobResultsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListBatchJobResultsRequest.
                             * @implements IListBatchJobResultsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsRequest=} [p] Properties to set
                             */
                            function ListBatchJobResultsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListBatchJobResultsRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @instance
                             */
                            ListBatchJobResultsRequest.prototype.resource_name = "";
    
                            /**
                             * ListBatchJobResultsRequest page_token.
                             * @member {string} page_token
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @instance
                             */
                            ListBatchJobResultsRequest.prototype.page_token = "";
    
                            /**
                             * ListBatchJobResultsRequest page_size.
                             * @member {number} page_size
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @instance
                             */
                            ListBatchJobResultsRequest.prototype.page_size = 0;
    
                            /**
                             * ListBatchJobResultsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @instance
                             */
                            ListBatchJobResultsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new ListBatchJobResultsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsRequest} ListBatchJobResultsRequest instance
                             */
                            ListBatchJobResultsRequest.create = function create(properties) {
                                return new ListBatchJobResultsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListBatchJobResultsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListBatchJobResultsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsRequest} m ListBatchJobResultsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListBatchJobResultsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                                    w.uint32(18).string(m.page_token);
                                if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                                    w.uint32(24).int32(m.page_size);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(32).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListBatchJobResultsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListBatchJobResultsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsRequest} message ListBatchJobResultsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListBatchJobResultsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListBatchJobResultsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsRequest} ListBatchJobResultsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListBatchJobResultsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListBatchJobResultsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.page_token = r.string();
                                        break;
                                    case 3:
                                        m.page_size = r.int32();
                                        break;
                                    case 4:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListBatchJobResultsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsRequest} ListBatchJobResultsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListBatchJobResultsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListBatchJobResultsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListBatchJobResultsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    if (!$util.isString(m.page_token))
                                        return "page_token: string expected";
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    if (!$util.isInteger(m.page_size))
                                        return "page_size: integer expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListBatchJobResultsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsRequest} ListBatchJobResultsRequest
                             */
                            ListBatchJobResultsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListBatchJobResultsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListBatchJobResultsRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.page_token != null) {
                                    m.page_token = String(d.page_token);
                                }
                                if (d.page_size != null) {
                                    m.page_size = d.page_size | 0;
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListBatchJobResultsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListBatchJobResultsRequest} m ListBatchJobResultsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListBatchJobResultsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.page_token = "";
                                    d.page_size = 0;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    d.page_token = m.page_token;
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    d.page_size = m.page_size;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListBatchJobResultsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListBatchJobResultsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListBatchJobResultsRequest;
                        })();
    
                        services.ListBatchJobResultsResponse = (function() {
    
                            /**
                             * Properties of a ListBatchJobResultsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListBatchJobResultsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IBatchJobResult>|null} [results] ListBatchJobResultsResponse results
                             * @property {string|null} [next_page_token] ListBatchJobResultsResponse next_page_token
                             */
    
                            /**
                             * Constructs a new ListBatchJobResultsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListBatchJobResultsResponse.
                             * @implements IListBatchJobResultsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsResponse=} [p] Properties to set
                             */
                            function ListBatchJobResultsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListBatchJobResultsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IBatchJobResult>} results
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @instance
                             */
                            ListBatchJobResultsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * ListBatchJobResultsResponse next_page_token.
                             * @member {string} next_page_token
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @instance
                             */
                            ListBatchJobResultsResponse.prototype.next_page_token = "";
    
                            /**
                             * Creates a new ListBatchJobResultsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsResponse} ListBatchJobResultsResponse instance
                             */
                            ListBatchJobResultsResponse.create = function create(properties) {
                                return new ListBatchJobResultsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListBatchJobResultsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListBatchJobResultsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsResponse} m ListBatchJobResultsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListBatchJobResultsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.BatchJobResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                                    w.uint32(18).string(m.next_page_token);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListBatchJobResultsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListBatchJobResultsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListBatchJobResultsResponse} message ListBatchJobResultsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListBatchJobResultsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListBatchJobResultsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsResponse} ListBatchJobResultsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListBatchJobResultsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListBatchJobResultsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.BatchJobResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.next_page_token = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListBatchJobResultsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsResponse} ListBatchJobResultsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListBatchJobResultsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListBatchJobResultsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListBatchJobResultsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.BatchJobResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    if (!$util.isString(m.next_page_token))
                                        return "next_page_token: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListBatchJobResultsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListBatchJobResultsResponse} ListBatchJobResultsResponse
                             */
                            ListBatchJobResultsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListBatchJobResultsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListBatchJobResultsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.ListBatchJobResultsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListBatchJobResultsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.BatchJobResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.next_page_token != null) {
                                    m.next_page_token = String(d.next_page_token);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListBatchJobResultsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListBatchJobResultsResponse} m ListBatchJobResultsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListBatchJobResultsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.next_page_token = "";
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.BatchJobResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    d.next_page_token = m.next_page_token;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListBatchJobResultsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListBatchJobResultsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListBatchJobResultsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListBatchJobResultsResponse;
                        })();
    
                        services.BatchJobResult = (function() {
    
                            /**
                             * Properties of a BatchJobResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IBatchJobResult
                             * @property {number|null} [operation_index] BatchJobResult operation_index
                             * @property {google.ads.googleads.v11.services.IMutateOperationResponse|null} [mutate_operation_response] BatchJobResult mutate_operation_response
                             * @property {google.rpc.IStatus|null} [status] BatchJobResult status
                             */
    
                            /**
                             * Constructs a new BatchJobResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BatchJobResult.
                             * @implements IBatchJobResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IBatchJobResult=} [p] Properties to set
                             */
                            function BatchJobResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BatchJobResult operation_index.
                             * @member {number} operation_index
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @instance
                             */
                            BatchJobResult.prototype.operation_index = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * BatchJobResult mutate_operation_response.
                             * @member {google.ads.googleads.v11.services.IMutateOperationResponse|null|undefined} mutate_operation_response
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @instance
                             */
                            BatchJobResult.prototype.mutate_operation_response = null;
    
                            /**
                             * BatchJobResult status.
                             * @member {google.rpc.IStatus|null|undefined} status
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @instance
                             */
                            BatchJobResult.prototype.status = null;
    
                            /**
                             * Creates a new BatchJobResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IBatchJobResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.BatchJobResult} BatchJobResult instance
                             */
                            BatchJobResult.create = function create(properties) {
                                return new BatchJobResult(properties);
                            };
    
                            /**
                             * Encodes the specified BatchJobResult message. Does not implicitly {@link google.ads.googleads.v11.services.BatchJobResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IBatchJobResult} m BatchJobResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.operation_index != null && Object.hasOwnProperty.call(m, "operation_index"))
                                    w.uint32(8).int64(m.operation_index);
                                if (m.mutate_operation_response != null && Object.hasOwnProperty.call(m, "mutate_operation_response"))
                                    $root.google.ads.googleads.v11.services.MutateOperationResponse.encode(m.mutate_operation_response, w.uint32(18).fork()).ldelim();
                                if (m.status != null && Object.hasOwnProperty.call(m, "status"))
                                    $root.google.rpc.Status.encode(m.status, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BatchJobResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.BatchJobResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IBatchJobResult} message BatchJobResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BatchJobResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BatchJobResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.BatchJobResult} BatchJobResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.BatchJobResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.operation_index = r.int64();
                                        break;
                                    case 2:
                                        m.mutate_operation_response = $root.google.ads.googleads.v11.services.MutateOperationResponse.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.status = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BatchJobResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.BatchJobResult} BatchJobResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BatchJobResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BatchJobResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BatchJobResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.operation_index != null && m.hasOwnProperty("operation_index")) {
                                    if (!$util.isInteger(m.operation_index) && !(m.operation_index && $util.isInteger(m.operation_index.low) && $util.isInteger(m.operation_index.high)))
                                        return "operation_index: integer|Long expected";
                                }
                                if (m.mutate_operation_response != null && m.hasOwnProperty("mutate_operation_response")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateOperationResponse.verify(m.mutate_operation_response);
                                        if (e)
                                            return "mutate_operation_response." + e;
                                    }
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.status);
                                        if (e)
                                            return "status." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BatchJobResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.BatchJobResult} BatchJobResult
                             */
                            BatchJobResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.BatchJobResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.BatchJobResult();
                                if (d.operation_index != null) {
                                    if ($util.Long)
                                        (m.operation_index = $util.Long.fromValue(d.operation_index)).unsigned = false;
                                    else if (typeof d.operation_index === "string")
                                        m.operation_index = parseInt(d.operation_index, 10);
                                    else if (typeof d.operation_index === "number")
                                        m.operation_index = d.operation_index;
                                    else if (typeof d.operation_index === "object")
                                        m.operation_index = new $util.LongBits(d.operation_index.low >>> 0, d.operation_index.high >>> 0).toNumber();
                                }
                                if (d.mutate_operation_response != null) {
                                    if (typeof d.mutate_operation_response !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BatchJobResult.mutate_operation_response: object expected");
                                    m.mutate_operation_response = $root.google.ads.googleads.v11.services.MutateOperationResponse.fromObject(d.mutate_operation_response);
                                }
                                if (d.status != null) {
                                    if (typeof d.status !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BatchJobResult.status: object expected");
                                    m.status = $root.google.rpc.Status.fromObject(d.status);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BatchJobResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @static
                             * @param {google.ads.googleads.v11.services.BatchJobResult} m BatchJobResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BatchJobResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.operation_index = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.operation_index = o.longs === String ? "0" : 0;
                                    d.mutate_operation_response = null;
                                    d.status = null;
                                }
                                if (m.operation_index != null && m.hasOwnProperty("operation_index")) {
                                    if (typeof m.operation_index === "number")
                                        d.operation_index = o.longs === String ? String(m.operation_index) : m.operation_index;
                                    else
                                        d.operation_index = o.longs === String ? $util.Long.prototype.toString.call(m.operation_index) : o.longs === Number ? new $util.LongBits(m.operation_index.low >>> 0, m.operation_index.high >>> 0).toNumber() : m.operation_index;
                                }
                                if (m.mutate_operation_response != null && m.hasOwnProperty("mutate_operation_response")) {
                                    d.mutate_operation_response = $root.google.ads.googleads.v11.services.MutateOperationResponse.toObject(m.mutate_operation_response, o);
                                }
                                if (m.status != null && m.hasOwnProperty("status")) {
                                    d.status = $root.google.rpc.Status.toObject(m.status, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BatchJobResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.BatchJobResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BatchJobResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BatchJobResult;
                        })();
    
                        services.GoogleAdsService = (function() {
    
                            /**
                             * Constructs a new GoogleAdsService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GoogleAdsService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function GoogleAdsService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (GoogleAdsService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = GoogleAdsService;
    
                            /**
                             * Creates new GoogleAdsService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {GoogleAdsService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            GoogleAdsService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.GoogleAdsService|search}.
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @typedef SearchCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsResponse} [response] SearchGoogleAdsResponse
                             */
    
                            /**
                             * Calls Search.
                             * @function search
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsRequest} request SearchGoogleAdsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.GoogleAdsService.SearchCallback} callback Node-style callback called with the error, if any, and SearchGoogleAdsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(GoogleAdsService.prototype.search = function search(request, callback) {
                                return this.rpcCall(search, $root.google.ads.googleads.v11.services.SearchGoogleAdsRequest, $root.google.ads.googleads.v11.services.SearchGoogleAdsResponse, request, callback);
                            }, "name", { value: "Search" });
    
                            /**
                             * Calls Search.
                             * @function search
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsRequest} request SearchGoogleAdsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SearchGoogleAdsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.GoogleAdsService|searchStream}.
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @typedef SearchStreamCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse} [response] SearchGoogleAdsStreamResponse
                             */
    
                            /**
                             * Calls SearchStream.
                             * @function searchStream
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamRequest} request SearchGoogleAdsStreamRequest message or plain object
                             * @param {google.ads.googleads.v11.services.GoogleAdsService.SearchStreamCallback} callback Node-style callback called with the error, if any, and SearchGoogleAdsStreamResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(GoogleAdsService.prototype.searchStream = function searchStream(request, callback) {
                                return this.rpcCall(searchStream, $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest, $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse, request, callback);
                            }, "name", { value: "SearchStream" });
    
                            /**
                             * Calls SearchStream.
                             * @function searchStream
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamRequest} request SearchGoogleAdsStreamRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.GoogleAdsService|mutate}.
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @typedef MutateCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateGoogleAdsResponse} [response] MutateGoogleAdsResponse
                             */
    
                            /**
                             * Calls Mutate.
                             * @function mutate
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsRequest} request MutateGoogleAdsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.GoogleAdsService.MutateCallback} callback Node-style callback called with the error, if any, and MutateGoogleAdsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(GoogleAdsService.prototype.mutate = function mutate(request, callback) {
                                return this.rpcCall(mutate, $root.google.ads.googleads.v11.services.MutateGoogleAdsRequest, $root.google.ads.googleads.v11.services.MutateGoogleAdsResponse, request, callback);
                            }, "name", { value: "Mutate" });
    
                            /**
                             * Calls Mutate.
                             * @function mutate
                             * @memberof google.ads.googleads.v11.services.GoogleAdsService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsRequest} request MutateGoogleAdsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateGoogleAdsResponse>} Promise
                             * @variation 2
                             */
    
                            return GoogleAdsService;
                        })();
    
                        services.SearchGoogleAdsRequest = (function() {
    
                            /**
                             * Properties of a SearchGoogleAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISearchGoogleAdsRequest
                             * @property {string|null} [customer_id] SearchGoogleAdsRequest customer_id
                             * @property {string|null} [query] SearchGoogleAdsRequest query
                             * @property {string|null} [page_token] SearchGoogleAdsRequest page_token
                             * @property {number|null} [page_size] SearchGoogleAdsRequest page_size
                             * @property {boolean|null} [validate_only] SearchGoogleAdsRequest validate_only
                             * @property {boolean|null} [return_total_results_count] SearchGoogleAdsRequest return_total_results_count
                             * @property {google.ads.googleads.v11.enums.SummaryRowSettingEnum.SummaryRowSetting|null} [summary_row_setting] SearchGoogleAdsRequest summary_row_setting
                             */
    
                            /**
                             * Constructs a new SearchGoogleAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SearchGoogleAdsRequest.
                             * @implements ISearchGoogleAdsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsRequest=} [p] Properties to set
                             */
                            function SearchGoogleAdsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SearchGoogleAdsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             */
                            SearchGoogleAdsRequest.prototype.customer_id = "";
    
                            /**
                             * SearchGoogleAdsRequest query.
                             * @member {string} query
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             */
                            SearchGoogleAdsRequest.prototype.query = "";
    
                            /**
                             * SearchGoogleAdsRequest page_token.
                             * @member {string} page_token
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             */
                            SearchGoogleAdsRequest.prototype.page_token = "";
    
                            /**
                             * SearchGoogleAdsRequest page_size.
                             * @member {number} page_size
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             */
                            SearchGoogleAdsRequest.prototype.page_size = 0;
    
                            /**
                             * SearchGoogleAdsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             */
                            SearchGoogleAdsRequest.prototype.validate_only = false;
    
                            /**
                             * SearchGoogleAdsRequest return_total_results_count.
                             * @member {boolean} return_total_results_count
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             */
                            SearchGoogleAdsRequest.prototype.return_total_results_count = false;
    
                            /**
                             * SearchGoogleAdsRequest summary_row_setting.
                             * @member {google.ads.googleads.v11.enums.SummaryRowSettingEnum.SummaryRowSetting} summary_row_setting
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             */
                            SearchGoogleAdsRequest.prototype.summary_row_setting = 0;
    
                            /**
                             * Creates a new SearchGoogleAdsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsRequest} SearchGoogleAdsRequest instance
                             */
                            SearchGoogleAdsRequest.create = function create(properties) {
                                return new SearchGoogleAdsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsRequest} m SearchGoogleAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.query != null && Object.hasOwnProperty.call(m, "query"))
                                    w.uint32(18).string(m.query);
                                if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                                    w.uint32(26).string(m.page_token);
                                if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                                    w.uint32(32).int32(m.page_size);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(40).bool(m.validate_only);
                                if (m.return_total_results_count != null && Object.hasOwnProperty.call(m, "return_total_results_count"))
                                    w.uint32(56).bool(m.return_total_results_count);
                                if (m.summary_row_setting != null && Object.hasOwnProperty.call(m, "summary_row_setting"))
                                    w.uint32(64).int32(m.summary_row_setting);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsRequest} message SearchGoogleAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsRequest} SearchGoogleAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.query = r.string();
                                        break;
                                    case 3:
                                        m.page_token = r.string();
                                        break;
                                    case 4:
                                        m.page_size = r.int32();
                                        break;
                                    case 5:
                                        m.validate_only = r.bool();
                                        break;
                                    case 7:
                                        m.return_total_results_count = r.bool();
                                        break;
                                    case 8:
                                        m.summary_row_setting = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsRequest} SearchGoogleAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchGoogleAdsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchGoogleAdsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.query != null && m.hasOwnProperty("query")) {
                                    if (!$util.isString(m.query))
                                        return "query: string expected";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    if (!$util.isString(m.page_token))
                                        return "page_token: string expected";
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    if (!$util.isInteger(m.page_size))
                                        return "page_size: integer expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.return_total_results_count != null && m.hasOwnProperty("return_total_results_count")) {
                                    if (typeof m.return_total_results_count !== "boolean")
                                        return "return_total_results_count: boolean expected";
                                }
                                if (m.summary_row_setting != null && m.hasOwnProperty("summary_row_setting")) {
                                    switch (m.summary_row_setting) {
                                    default:
                                        return "summary_row_setting: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SearchGoogleAdsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsRequest} SearchGoogleAdsRequest
                             */
                            SearchGoogleAdsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SearchGoogleAdsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.query != null) {
                                    m.query = String(d.query);
                                }
                                if (d.page_token != null) {
                                    m.page_token = String(d.page_token);
                                }
                                if (d.page_size != null) {
                                    m.page_size = d.page_size | 0;
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                if (d.return_total_results_count != null) {
                                    m.return_total_results_count = Boolean(d.return_total_results_count);
                                }
                                switch (d.summary_row_setting) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.summary_row_setting = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.summary_row_setting = 1;
                                    break;
                                case "NO_SUMMARY_ROW":
                                case 2:
                                    m.summary_row_setting = 2;
                                    break;
                                case "SUMMARY_ROW_WITH_RESULTS":
                                case 3:
                                    m.summary_row_setting = 3;
                                    break;
                                case "SUMMARY_ROW_ONLY":
                                case 4:
                                    m.summary_row_setting = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SearchGoogleAdsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsRequest} m SearchGoogleAdsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchGoogleAdsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.query = "";
                                    d.page_token = "";
                                    d.page_size = 0;
                                    d.validate_only = false;
                                    d.return_total_results_count = false;
                                    d.summary_row_setting = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.query != null && m.hasOwnProperty("query")) {
                                    d.query = m.query;
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    d.page_token = m.page_token;
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    d.page_size = m.page_size;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.return_total_results_count != null && m.hasOwnProperty("return_total_results_count")) {
                                    d.return_total_results_count = m.return_total_results_count;
                                }
                                if (m.summary_row_setting != null && m.hasOwnProperty("summary_row_setting")) {
                                    d.summary_row_setting = o.enums === String ? $root.google.ads.googleads.v11.enums.SummaryRowSettingEnum.SummaryRowSetting[m.summary_row_setting] : m.summary_row_setting;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SearchGoogleAdsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchGoogleAdsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SearchGoogleAdsRequest;
                        })();
    
                        services.SearchGoogleAdsResponse = (function() {
    
                            /**
                             * Properties of a SearchGoogleAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISearchGoogleAdsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IGoogleAdsRow>|null} [results] SearchGoogleAdsResponse results
                             * @property {string|null} [next_page_token] SearchGoogleAdsResponse next_page_token
                             * @property {number|null} [total_results_count] SearchGoogleAdsResponse total_results_count
                             * @property {google.protobuf.IFieldMask|null} [field_mask] SearchGoogleAdsResponse field_mask
                             * @property {google.ads.googleads.v11.services.IGoogleAdsRow|null} [summary_row] SearchGoogleAdsResponse summary_row
                             */
    
                            /**
                             * Constructs a new SearchGoogleAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SearchGoogleAdsResponse.
                             * @implements ISearchGoogleAdsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsResponse=} [p] Properties to set
                             */
                            function SearchGoogleAdsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SearchGoogleAdsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IGoogleAdsRow>} results
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @instance
                             */
                            SearchGoogleAdsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * SearchGoogleAdsResponse next_page_token.
                             * @member {string} next_page_token
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @instance
                             */
                            SearchGoogleAdsResponse.prototype.next_page_token = "";
    
                            /**
                             * SearchGoogleAdsResponse total_results_count.
                             * @member {number} total_results_count
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @instance
                             */
                            SearchGoogleAdsResponse.prototype.total_results_count = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * SearchGoogleAdsResponse field_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} field_mask
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @instance
                             */
                            SearchGoogleAdsResponse.prototype.field_mask = null;
    
                            /**
                             * SearchGoogleAdsResponse summary_row.
                             * @member {google.ads.googleads.v11.services.IGoogleAdsRow|null|undefined} summary_row
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @instance
                             */
                            SearchGoogleAdsResponse.prototype.summary_row = null;
    
                            /**
                             * Creates a new SearchGoogleAdsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsResponse} SearchGoogleAdsResponse instance
                             */
                            SearchGoogleAdsResponse.create = function create(properties) {
                                return new SearchGoogleAdsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsResponse} m SearchGoogleAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.GoogleAdsRow.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                                    w.uint32(18).string(m.next_page_token);
                                if (m.total_results_count != null && Object.hasOwnProperty.call(m, "total_results_count"))
                                    w.uint32(24).int64(m.total_results_count);
                                if (m.field_mask != null && Object.hasOwnProperty.call(m, "field_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.field_mask, w.uint32(42).fork()).ldelim();
                                if (m.summary_row != null && Object.hasOwnProperty.call(m, "summary_row"))
                                    $root.google.ads.googleads.v11.services.GoogleAdsRow.encode(m.summary_row, w.uint32(50).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsResponse} message SearchGoogleAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsResponse} SearchGoogleAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.GoogleAdsRow.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.next_page_token = r.string();
                                        break;
                                    case 3:
                                        m.total_results_count = r.int64();
                                        break;
                                    case 5:
                                        m.field_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.summary_row = $root.google.ads.googleads.v11.services.GoogleAdsRow.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsResponse} SearchGoogleAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchGoogleAdsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchGoogleAdsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.GoogleAdsRow.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    if (!$util.isString(m.next_page_token))
                                        return "next_page_token: string expected";
                                }
                                if (m.total_results_count != null && m.hasOwnProperty("total_results_count")) {
                                    if (!$util.isInteger(m.total_results_count) && !(m.total_results_count && $util.isInteger(m.total_results_count.low) && $util.isInteger(m.total_results_count.high)))
                                        return "total_results_count: integer|Long expected";
                                }
                                if (m.field_mask != null && m.hasOwnProperty("field_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.field_mask);
                                        if (e)
                                            return "field_mask." + e;
                                    }
                                }
                                if (m.summary_row != null && m.hasOwnProperty("summary_row")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.GoogleAdsRow.verify(m.summary_row);
                                        if (e)
                                            return "summary_row." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SearchGoogleAdsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsResponse} SearchGoogleAdsResponse
                             */
                            SearchGoogleAdsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SearchGoogleAdsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.GoogleAdsRow.fromObject(d.results[i]);
                                    }
                                }
                                if (d.next_page_token != null) {
                                    m.next_page_token = String(d.next_page_token);
                                }
                                if (d.total_results_count != null) {
                                    if ($util.Long)
                                        (m.total_results_count = $util.Long.fromValue(d.total_results_count)).unsigned = false;
                                    else if (typeof d.total_results_count === "string")
                                        m.total_results_count = parseInt(d.total_results_count, 10);
                                    else if (typeof d.total_results_count === "number")
                                        m.total_results_count = d.total_results_count;
                                    else if (typeof d.total_results_count === "object")
                                        m.total_results_count = new $util.LongBits(d.total_results_count.low >>> 0, d.total_results_count.high >>> 0).toNumber();
                                }
                                if (d.field_mask != null) {
                                    if (typeof d.field_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsResponse.field_mask: object expected");
                                    m.field_mask = $root.google.protobuf.FieldMask.fromObject(d.field_mask);
                                }
                                if (d.summary_row != null) {
                                    if (typeof d.summary_row !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsResponse.summary_row: object expected");
                                    m.summary_row = $root.google.ads.googleads.v11.services.GoogleAdsRow.fromObject(d.summary_row);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SearchGoogleAdsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsResponse} m SearchGoogleAdsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchGoogleAdsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.next_page_token = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_results_count = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_results_count = o.longs === String ? "0" : 0;
                                    d.field_mask = null;
                                    d.summary_row = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.GoogleAdsRow.toObject(m.results[j], o);
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    d.next_page_token = m.next_page_token;
                                }
                                if (m.total_results_count != null && m.hasOwnProperty("total_results_count")) {
                                    if (typeof m.total_results_count === "number")
                                        d.total_results_count = o.longs === String ? String(m.total_results_count) : m.total_results_count;
                                    else
                                        d.total_results_count = o.longs === String ? $util.Long.prototype.toString.call(m.total_results_count) : o.longs === Number ? new $util.LongBits(m.total_results_count.low >>> 0, m.total_results_count.high >>> 0).toNumber() : m.total_results_count;
                                }
                                if (m.field_mask != null && m.hasOwnProperty("field_mask")) {
                                    d.field_mask = $root.google.protobuf.FieldMask.toObject(m.field_mask, o);
                                }
                                if (m.summary_row != null && m.hasOwnProperty("summary_row")) {
                                    d.summary_row = $root.google.ads.googleads.v11.services.GoogleAdsRow.toObject(m.summary_row, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SearchGoogleAdsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchGoogleAdsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SearchGoogleAdsResponse;
                        })();
    
                        services.SearchGoogleAdsStreamRequest = (function() {
    
                            /**
                             * Properties of a SearchGoogleAdsStreamRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISearchGoogleAdsStreamRequest
                             * @property {string|null} [customer_id] SearchGoogleAdsStreamRequest customer_id
                             * @property {string|null} [query] SearchGoogleAdsStreamRequest query
                             * @property {google.ads.googleads.v11.enums.SummaryRowSettingEnum.SummaryRowSetting|null} [summary_row_setting] SearchGoogleAdsStreamRequest summary_row_setting
                             */
    
                            /**
                             * Constructs a new SearchGoogleAdsStreamRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SearchGoogleAdsStreamRequest.
                             * @implements ISearchGoogleAdsStreamRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamRequest=} [p] Properties to set
                             */
                            function SearchGoogleAdsStreamRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SearchGoogleAdsStreamRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @instance
                             */
                            SearchGoogleAdsStreamRequest.prototype.customer_id = "";
    
                            /**
                             * SearchGoogleAdsStreamRequest query.
                             * @member {string} query
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @instance
                             */
                            SearchGoogleAdsStreamRequest.prototype.query = "";
    
                            /**
                             * SearchGoogleAdsStreamRequest summary_row_setting.
                             * @member {google.ads.googleads.v11.enums.SummaryRowSettingEnum.SummaryRowSetting} summary_row_setting
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @instance
                             */
                            SearchGoogleAdsStreamRequest.prototype.summary_row_setting = 0;
    
                            /**
                             * Creates a new SearchGoogleAdsStreamRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest} SearchGoogleAdsStreamRequest instance
                             */
                            SearchGoogleAdsStreamRequest.create = function create(properties) {
                                return new SearchGoogleAdsStreamRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsStreamRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamRequest} m SearchGoogleAdsStreamRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsStreamRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.query != null && Object.hasOwnProperty.call(m, "query"))
                                    w.uint32(18).string(m.query);
                                if (m.summary_row_setting != null && Object.hasOwnProperty.call(m, "summary_row_setting"))
                                    w.uint32(24).int32(m.summary_row_setting);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsStreamRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamRequest} message SearchGoogleAdsStreamRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsStreamRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsStreamRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest} SearchGoogleAdsStreamRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsStreamRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.query = r.string();
                                        break;
                                    case 3:
                                        m.summary_row_setting = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsStreamRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest} SearchGoogleAdsStreamRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsStreamRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchGoogleAdsStreamRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchGoogleAdsStreamRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.query != null && m.hasOwnProperty("query")) {
                                    if (!$util.isString(m.query))
                                        return "query: string expected";
                                }
                                if (m.summary_row_setting != null && m.hasOwnProperty("summary_row_setting")) {
                                    switch (m.summary_row_setting) {
                                    default:
                                        return "summary_row_setting: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SearchGoogleAdsStreamRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest} SearchGoogleAdsStreamRequest
                             */
                            SearchGoogleAdsStreamRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.query != null) {
                                    m.query = String(d.query);
                                }
                                switch (d.summary_row_setting) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.summary_row_setting = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.summary_row_setting = 1;
                                    break;
                                case "NO_SUMMARY_ROW":
                                case 2:
                                    m.summary_row_setting = 2;
                                    break;
                                case "SUMMARY_ROW_WITH_RESULTS":
                                case 3:
                                    m.summary_row_setting = 3;
                                    break;
                                case "SUMMARY_ROW_ONLY":
                                case 4:
                                    m.summary_row_setting = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SearchGoogleAdsStreamRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest} m SearchGoogleAdsStreamRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchGoogleAdsStreamRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.query = "";
                                    d.summary_row_setting = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.query != null && m.hasOwnProperty("query")) {
                                    d.query = m.query;
                                }
                                if (m.summary_row_setting != null && m.hasOwnProperty("summary_row_setting")) {
                                    d.summary_row_setting = o.enums === String ? $root.google.ads.googleads.v11.enums.SummaryRowSettingEnum.SummaryRowSetting[m.summary_row_setting] : m.summary_row_setting;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SearchGoogleAdsStreamRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchGoogleAdsStreamRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SearchGoogleAdsStreamRequest;
                        })();
    
                        services.SearchGoogleAdsStreamResponse = (function() {
    
                            /**
                             * Properties of a SearchGoogleAdsStreamResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISearchGoogleAdsStreamResponse
                             * @property {Array.<google.ads.googleads.v11.services.IGoogleAdsRow>|null} [results] SearchGoogleAdsStreamResponse results
                             * @property {google.protobuf.IFieldMask|null} [field_mask] SearchGoogleAdsStreamResponse field_mask
                             * @property {google.ads.googleads.v11.services.IGoogleAdsRow|null} [summary_row] SearchGoogleAdsStreamResponse summary_row
                             * @property {string|null} [request_id] SearchGoogleAdsStreamResponse request_id
                             */
    
                            /**
                             * Constructs a new SearchGoogleAdsStreamResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SearchGoogleAdsStreamResponse.
                             * @implements ISearchGoogleAdsStreamResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamResponse=} [p] Properties to set
                             */
                            function SearchGoogleAdsStreamResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SearchGoogleAdsStreamResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IGoogleAdsRow>} results
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @instance
                             */
                            SearchGoogleAdsStreamResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * SearchGoogleAdsStreamResponse field_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} field_mask
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @instance
                             */
                            SearchGoogleAdsStreamResponse.prototype.field_mask = null;
    
                            /**
                             * SearchGoogleAdsStreamResponse summary_row.
                             * @member {google.ads.googleads.v11.services.IGoogleAdsRow|null|undefined} summary_row
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @instance
                             */
                            SearchGoogleAdsStreamResponse.prototype.summary_row = null;
    
                            /**
                             * SearchGoogleAdsStreamResponse request_id.
                             * @member {string} request_id
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @instance
                             */
                            SearchGoogleAdsStreamResponse.prototype.request_id = "";
    
                            /**
                             * Creates a new SearchGoogleAdsStreamResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse} SearchGoogleAdsStreamResponse instance
                             */
                            SearchGoogleAdsStreamResponse.create = function create(properties) {
                                return new SearchGoogleAdsStreamResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsStreamResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamResponse} m SearchGoogleAdsStreamResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsStreamResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.GoogleAdsRow.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.field_mask != null && Object.hasOwnProperty.call(m, "field_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.field_mask, w.uint32(18).fork()).ldelim();
                                if (m.summary_row != null && Object.hasOwnProperty.call(m, "summary_row"))
                                    $root.google.ads.googleads.v11.services.GoogleAdsRow.encode(m.summary_row, w.uint32(26).fork()).ldelim();
                                if (m.request_id != null && Object.hasOwnProperty.call(m, "request_id"))
                                    w.uint32(34).string(m.request_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsStreamResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsStreamResponse} message SearchGoogleAdsStreamResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsStreamResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsStreamResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse} SearchGoogleAdsStreamResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsStreamResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.GoogleAdsRow.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.field_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.summary_row = $root.google.ads.googleads.v11.services.GoogleAdsRow.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.request_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsStreamResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse} SearchGoogleAdsStreamResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsStreamResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchGoogleAdsStreamResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchGoogleAdsStreamResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.GoogleAdsRow.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.field_mask != null && m.hasOwnProperty("field_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.field_mask);
                                        if (e)
                                            return "field_mask." + e;
                                    }
                                }
                                if (m.summary_row != null && m.hasOwnProperty("summary_row")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.GoogleAdsRow.verify(m.summary_row);
                                        if (e)
                                            return "summary_row." + e;
                                    }
                                }
                                if (m.request_id != null && m.hasOwnProperty("request_id")) {
                                    if (!$util.isString(m.request_id))
                                        return "request_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SearchGoogleAdsStreamResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse} SearchGoogleAdsStreamResponse
                             */
                            SearchGoogleAdsStreamResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.GoogleAdsRow.fromObject(d.results[i]);
                                    }
                                }
                                if (d.field_mask != null) {
                                    if (typeof d.field_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse.field_mask: object expected");
                                    m.field_mask = $root.google.protobuf.FieldMask.fromObject(d.field_mask);
                                }
                                if (d.summary_row != null) {
                                    if (typeof d.summary_row !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse.summary_row: object expected");
                                    m.summary_row = $root.google.ads.googleads.v11.services.GoogleAdsRow.fromObject(d.summary_row);
                                }
                                if (d.request_id != null) {
                                    m.request_id = String(d.request_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SearchGoogleAdsStreamResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse} m SearchGoogleAdsStreamResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchGoogleAdsStreamResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.field_mask = null;
                                    d.summary_row = null;
                                    d.request_id = "";
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.GoogleAdsRow.toObject(m.results[j], o);
                                    }
                                }
                                if (m.field_mask != null && m.hasOwnProperty("field_mask")) {
                                    d.field_mask = $root.google.protobuf.FieldMask.toObject(m.field_mask, o);
                                }
                                if (m.summary_row != null && m.hasOwnProperty("summary_row")) {
                                    d.summary_row = $root.google.ads.googleads.v11.services.GoogleAdsRow.toObject(m.summary_row, o);
                                }
                                if (m.request_id != null && m.hasOwnProperty("request_id")) {
                                    d.request_id = m.request_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SearchGoogleAdsStreamResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsStreamResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchGoogleAdsStreamResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SearchGoogleAdsStreamResponse;
                        })();
    
                        services.GoogleAdsRow = (function() {
    
                            /**
                             * Properties of a GoogleAdsRow.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGoogleAdsRow
                             * @property {google.ads.googleads.v11.resources.IAccountBudget|null} [account_budget] GoogleAdsRow account_budget
                             * @property {google.ads.googleads.v11.resources.IAccountBudgetProposal|null} [account_budget_proposal] GoogleAdsRow account_budget_proposal
                             * @property {google.ads.googleads.v11.resources.IAccountLink|null} [account_link] GoogleAdsRow account_link
                             * @property {google.ads.googleads.v11.resources.IAdGroup|null} [ad_group] GoogleAdsRow ad_group
                             * @property {google.ads.googleads.v11.resources.IAdGroupAd|null} [ad_group_ad] GoogleAdsRow ad_group_ad
                             * @property {google.ads.googleads.v11.resources.IAdGroupAdAssetCombinationView|null} [ad_group_ad_asset_combination_view] GoogleAdsRow ad_group_ad_asset_combination_view
                             * @property {google.ads.googleads.v11.resources.IAdGroupAdAssetView|null} [ad_group_ad_asset_view] GoogleAdsRow ad_group_ad_asset_view
                             * @property {google.ads.googleads.v11.resources.IAdGroupAdLabel|null} [ad_group_ad_label] GoogleAdsRow ad_group_ad_label
                             * @property {google.ads.googleads.v11.resources.IAdGroupAsset|null} [ad_group_asset] GoogleAdsRow ad_group_asset
                             * @property {google.ads.googleads.v11.resources.IAdGroupAudienceView|null} [ad_group_audience_view] GoogleAdsRow ad_group_audience_view
                             * @property {google.ads.googleads.v11.resources.IAdGroupBidModifier|null} [ad_group_bid_modifier] GoogleAdsRow ad_group_bid_modifier
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterion|null} [ad_group_criterion] GoogleAdsRow ad_group_criterion
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer|null} [ad_group_criterion_customizer] GoogleAdsRow ad_group_criterion_customizer
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterionLabel|null} [ad_group_criterion_label] GoogleAdsRow ad_group_criterion_label
                             * @property {google.ads.googleads.v11.resources.IAdGroupCriterionSimulation|null} [ad_group_criterion_simulation] GoogleAdsRow ad_group_criterion_simulation
                             * @property {google.ads.googleads.v11.resources.IAdGroupCustomizer|null} [ad_group_customizer] GoogleAdsRow ad_group_customizer
                             * @property {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null} [ad_group_extension_setting] GoogleAdsRow ad_group_extension_setting
                             * @property {google.ads.googleads.v11.resources.IAdGroupFeed|null} [ad_group_feed] GoogleAdsRow ad_group_feed
                             * @property {google.ads.googleads.v11.resources.IAdGroupLabel|null} [ad_group_label] GoogleAdsRow ad_group_label
                             * @property {google.ads.googleads.v11.resources.IAdGroupSimulation|null} [ad_group_simulation] GoogleAdsRow ad_group_simulation
                             * @property {google.ads.googleads.v11.resources.IAdParameter|null} [ad_parameter] GoogleAdsRow ad_parameter
                             * @property {google.ads.googleads.v11.resources.IAgeRangeView|null} [age_range_view] GoogleAdsRow age_range_view
                             * @property {google.ads.googleads.v11.resources.IAdScheduleView|null} [ad_schedule_view] GoogleAdsRow ad_schedule_view
                             * @property {google.ads.googleads.v11.resources.IDomainCategory|null} [domain_category] GoogleAdsRow domain_category
                             * @property {google.ads.googleads.v11.resources.IAsset|null} [asset] GoogleAdsRow asset
                             * @property {google.ads.googleads.v11.resources.IAssetFieldTypeView|null} [asset_field_type_view] GoogleAdsRow asset_field_type_view
                             * @property {google.ads.googleads.v11.resources.IAssetGroupAsset|null} [asset_group_asset] GoogleAdsRow asset_group_asset
                             * @property {google.ads.googleads.v11.resources.IAssetGroupSignal|null} [asset_group_signal] GoogleAdsRow asset_group_signal
                             * @property {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null} [asset_group_listing_group_filter] GoogleAdsRow asset_group_listing_group_filter
                             * @property {google.ads.googleads.v11.resources.IAssetGroupProductGroupView|null} [asset_group_product_group_view] GoogleAdsRow asset_group_product_group_view
                             * @property {google.ads.googleads.v11.resources.IAssetGroup|null} [asset_group] GoogleAdsRow asset_group
                             * @property {google.ads.googleads.v11.resources.IAssetSetAsset|null} [asset_set_asset] GoogleAdsRow asset_set_asset
                             * @property {google.ads.googleads.v11.resources.IAssetSet|null} [asset_set] GoogleAdsRow asset_set
                             * @property {google.ads.googleads.v11.resources.IBatchJob|null} [batch_job] GoogleAdsRow batch_job
                             * @property {google.ads.googleads.v11.resources.IBiddingDataExclusion|null} [bidding_data_exclusion] GoogleAdsRow bidding_data_exclusion
                             * @property {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null} [bidding_seasonality_adjustment] GoogleAdsRow bidding_seasonality_adjustment
                             * @property {google.ads.googleads.v11.resources.IBiddingStrategy|null} [bidding_strategy] GoogleAdsRow bidding_strategy
                             * @property {google.ads.googleads.v11.resources.IBiddingStrategySimulation|null} [bidding_strategy_simulation] GoogleAdsRow bidding_strategy_simulation
                             * @property {google.ads.googleads.v11.resources.IBillingSetup|null} [billing_setup] GoogleAdsRow billing_setup
                             * @property {google.ads.googleads.v11.resources.ICallView|null} [call_view] GoogleAdsRow call_view
                             * @property {google.ads.googleads.v11.resources.ICampaignBudget|null} [campaign_budget] GoogleAdsRow campaign_budget
                             * @property {google.ads.googleads.v11.resources.ICampaign|null} [campaign] GoogleAdsRow campaign
                             * @property {google.ads.googleads.v11.resources.ICampaignAsset|null} [campaign_asset] GoogleAdsRow campaign_asset
                             * @property {google.ads.googleads.v11.resources.ICampaignAssetSet|null} [campaign_asset_set] GoogleAdsRow campaign_asset_set
                             * @property {google.ads.googleads.v11.resources.ICampaignAudienceView|null} [campaign_audience_view] GoogleAdsRow campaign_audience_view
                             * @property {google.ads.googleads.v11.resources.ICampaignBidModifier|null} [campaign_bid_modifier] GoogleAdsRow campaign_bid_modifier
                             * @property {google.ads.googleads.v11.resources.ICampaignConversionGoal|null} [campaign_conversion_goal] GoogleAdsRow campaign_conversion_goal
                             * @property {google.ads.googleads.v11.resources.ICampaignCriterion|null} [campaign_criterion] GoogleAdsRow campaign_criterion
                             * @property {google.ads.googleads.v11.resources.ICampaignCriterionSimulation|null} [campaign_criterion_simulation] GoogleAdsRow campaign_criterion_simulation
                             * @property {google.ads.googleads.v11.resources.ICampaignCustomizer|null} [campaign_customizer] GoogleAdsRow campaign_customizer
                             * @property {google.ads.googleads.v11.resources.ICampaignDraft|null} [campaign_draft] GoogleAdsRow campaign_draft
                             * @property {google.ads.googleads.v11.resources.ICampaignExperiment|null} [campaign_experiment] GoogleAdsRow campaign_experiment
                             * @property {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null} [campaign_extension_setting] GoogleAdsRow campaign_extension_setting
                             * @property {google.ads.googleads.v11.resources.ICampaignFeed|null} [campaign_feed] GoogleAdsRow campaign_feed
                             * @property {google.ads.googleads.v11.resources.ICampaignGroup|null} [campaign_group] GoogleAdsRow campaign_group
                             * @property {google.ads.googleads.v11.resources.ICampaignLabel|null} [campaign_label] GoogleAdsRow campaign_label
                             * @property {google.ads.googleads.v11.resources.ICampaignSharedSet|null} [campaign_shared_set] GoogleAdsRow campaign_shared_set
                             * @property {google.ads.googleads.v11.resources.ICampaignSimulation|null} [campaign_simulation] GoogleAdsRow campaign_simulation
                             * @property {google.ads.googleads.v11.resources.ICarrierConstant|null} [carrier_constant] GoogleAdsRow carrier_constant
                             * @property {google.ads.googleads.v11.resources.IChangeEvent|null} [change_event] GoogleAdsRow change_event
                             * @property {google.ads.googleads.v11.resources.IChangeStatus|null} [change_status] GoogleAdsRow change_status
                             * @property {google.ads.googleads.v11.resources.ICombinedAudience|null} [combined_audience] GoogleAdsRow combined_audience
                             * @property {google.ads.googleads.v11.resources.IAudience|null} [audience] GoogleAdsRow audience
                             * @property {google.ads.googleads.v11.resources.IConversionAction|null} [conversion_action] GoogleAdsRow conversion_action
                             * @property {google.ads.googleads.v11.resources.IConversionCustomVariable|null} [conversion_custom_variable] GoogleAdsRow conversion_custom_variable
                             * @property {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig|null} [conversion_goal_campaign_config] GoogleAdsRow conversion_goal_campaign_config
                             * @property {google.ads.googleads.v11.resources.IConversionValueRule|null} [conversion_value_rule] GoogleAdsRow conversion_value_rule
                             * @property {google.ads.googleads.v11.resources.IConversionValueRuleSet|null} [conversion_value_rule_set] GoogleAdsRow conversion_value_rule_set
                             * @property {google.ads.googleads.v11.resources.IClickView|null} [click_view] GoogleAdsRow click_view
                             * @property {google.ads.googleads.v11.resources.ICurrencyConstant|null} [currency_constant] GoogleAdsRow currency_constant
                             * @property {google.ads.googleads.v11.resources.ICustomAudience|null} [custom_audience] GoogleAdsRow custom_audience
                             * @property {google.ads.googleads.v11.resources.ICustomConversionGoal|null} [custom_conversion_goal] GoogleAdsRow custom_conversion_goal
                             * @property {google.ads.googleads.v11.resources.ICustomInterest|null} [custom_interest] GoogleAdsRow custom_interest
                             * @property {google.ads.googleads.v11.resources.ICustomer|null} [customer] GoogleAdsRow customer
                             * @property {google.ads.googleads.v11.resources.ICustomerAsset|null} [customer_asset] GoogleAdsRow customer_asset
                             * @property {google.ads.googleads.v11.resources.IAccessibleBiddingStrategy|null} [accessible_bidding_strategy] GoogleAdsRow accessible_bidding_strategy
                             * @property {google.ads.googleads.v11.resources.ICustomerCustomizer|null} [customer_customizer] GoogleAdsRow customer_customizer
                             * @property {google.ads.googleads.v11.resources.ICustomerManagerLink|null} [customer_manager_link] GoogleAdsRow customer_manager_link
                             * @property {google.ads.googleads.v11.resources.ICustomerClientLink|null} [customer_client_link] GoogleAdsRow customer_client_link
                             * @property {google.ads.googleads.v11.resources.ICustomerClient|null} [customer_client] GoogleAdsRow customer_client
                             * @property {google.ads.googleads.v11.resources.ICustomerConversionGoal|null} [customer_conversion_goal] GoogleAdsRow customer_conversion_goal
                             * @property {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null} [customer_extension_setting] GoogleAdsRow customer_extension_setting
                             * @property {google.ads.googleads.v11.resources.ICustomerFeed|null} [customer_feed] GoogleAdsRow customer_feed
                             * @property {google.ads.googleads.v11.resources.ICustomerLabel|null} [customer_label] GoogleAdsRow customer_label
                             * @property {google.ads.googleads.v11.resources.ICustomerNegativeCriterion|null} [customer_negative_criterion] GoogleAdsRow customer_negative_criterion
                             * @property {google.ads.googleads.v11.resources.ICustomerUserAccess|null} [customer_user_access] GoogleAdsRow customer_user_access
                             * @property {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation|null} [customer_user_access_invitation] GoogleAdsRow customer_user_access_invitation
                             * @property {google.ads.googleads.v11.resources.ICustomizerAttribute|null} [customizer_attribute] GoogleAdsRow customizer_attribute
                             * @property {google.ads.googleads.v11.resources.IDetailPlacementView|null} [detail_placement_view] GoogleAdsRow detail_placement_view
                             * @property {google.ads.googleads.v11.resources.IDetailedDemographic|null} [detailed_demographic] GoogleAdsRow detailed_demographic
                             * @property {google.ads.googleads.v11.resources.IDisplayKeywordView|null} [display_keyword_view] GoogleAdsRow display_keyword_view
                             * @property {google.ads.googleads.v11.resources.IDistanceView|null} [distance_view] GoogleAdsRow distance_view
                             * @property {google.ads.googleads.v11.resources.IDynamicSearchAdsSearchTermView|null} [dynamic_search_ads_search_term_view] GoogleAdsRow dynamic_search_ads_search_term_view
                             * @property {google.ads.googleads.v11.resources.IExpandedLandingPageView|null} [expanded_landing_page_view] GoogleAdsRow expanded_landing_page_view
                             * @property {google.ads.googleads.v11.resources.IExtensionFeedItem|null} [extension_feed_item] GoogleAdsRow extension_feed_item
                             * @property {google.ads.googleads.v11.resources.IFeed|null} [feed] GoogleAdsRow feed
                             * @property {google.ads.googleads.v11.resources.IFeedItem|null} [feed_item] GoogleAdsRow feed_item
                             * @property {google.ads.googleads.v11.resources.IFeedItemSet|null} [feed_item_set] GoogleAdsRow feed_item_set
                             * @property {google.ads.googleads.v11.resources.IFeedItemSetLink|null} [feed_item_set_link] GoogleAdsRow feed_item_set_link
                             * @property {google.ads.googleads.v11.resources.IFeedItemTarget|null} [feed_item_target] GoogleAdsRow feed_item_target
                             * @property {google.ads.googleads.v11.resources.IFeedMapping|null} [feed_mapping] GoogleAdsRow feed_mapping
                             * @property {google.ads.googleads.v11.resources.IFeedPlaceholderView|null} [feed_placeholder_view] GoogleAdsRow feed_placeholder_view
                             * @property {google.ads.googleads.v11.resources.IGenderView|null} [gender_view] GoogleAdsRow gender_view
                             * @property {google.ads.googleads.v11.resources.IGeoTargetConstant|null} [geo_target_constant] GoogleAdsRow geo_target_constant
                             * @property {google.ads.googleads.v11.resources.IGeographicView|null} [geographic_view] GoogleAdsRow geographic_view
                             * @property {google.ads.googleads.v11.resources.IGroupPlacementView|null} [group_placement_view] GoogleAdsRow group_placement_view
                             * @property {google.ads.googleads.v11.resources.IHotelGroupView|null} [hotel_group_view] GoogleAdsRow hotel_group_view
                             * @property {google.ads.googleads.v11.resources.IHotelPerformanceView|null} [hotel_performance_view] GoogleAdsRow hotel_performance_view
                             * @property {google.ads.googleads.v11.resources.IHotelReconciliation|null} [hotel_reconciliation] GoogleAdsRow hotel_reconciliation
                             * @property {google.ads.googleads.v11.resources.IIncomeRangeView|null} [income_range_view] GoogleAdsRow income_range_view
                             * @property {google.ads.googleads.v11.resources.IKeywordView|null} [keyword_view] GoogleAdsRow keyword_view
                             * @property {google.ads.googleads.v11.resources.IKeywordPlan|null} [keyword_plan] GoogleAdsRow keyword_plan
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanCampaign|null} [keyword_plan_campaign] GoogleAdsRow keyword_plan_campaign
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword|null} [keyword_plan_campaign_keyword] GoogleAdsRow keyword_plan_campaign_keyword
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanAdGroup|null} [keyword_plan_ad_group] GoogleAdsRow keyword_plan_ad_group
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword|null} [keyword_plan_ad_group_keyword] GoogleAdsRow keyword_plan_ad_group_keyword
                             * @property {google.ads.googleads.v11.resources.IKeywordThemeConstant|null} [keyword_theme_constant] GoogleAdsRow keyword_theme_constant
                             * @property {google.ads.googleads.v11.resources.ILabel|null} [label] GoogleAdsRow label
                             * @property {google.ads.googleads.v11.resources.ILandingPageView|null} [landing_page_view] GoogleAdsRow landing_page_view
                             * @property {google.ads.googleads.v11.resources.ILanguageConstant|null} [language_constant] GoogleAdsRow language_constant
                             * @property {google.ads.googleads.v11.resources.ILocationView|null} [location_view] GoogleAdsRow location_view
                             * @property {google.ads.googleads.v11.resources.IManagedPlacementView|null} [managed_placement_view] GoogleAdsRow managed_placement_view
                             * @property {google.ads.googleads.v11.resources.IMediaFile|null} [media_file] GoogleAdsRow media_file
                             * @property {google.ads.googleads.v11.resources.IMobileAppCategoryConstant|null} [mobile_app_category_constant] GoogleAdsRow mobile_app_category_constant
                             * @property {google.ads.googleads.v11.resources.IMobileDeviceConstant|null} [mobile_device_constant] GoogleAdsRow mobile_device_constant
                             * @property {google.ads.googleads.v11.resources.IOfflineUserDataJob|null} [offline_user_data_job] GoogleAdsRow offline_user_data_job
                             * @property {google.ads.googleads.v11.resources.IOperatingSystemVersionConstant|null} [operating_system_version_constant] GoogleAdsRow operating_system_version_constant
                             * @property {google.ads.googleads.v11.resources.IPaidOrganicSearchTermView|null} [paid_organic_search_term_view] GoogleAdsRow paid_organic_search_term_view
                             * @property {google.ads.googleads.v11.resources.IParentalStatusView|null} [parental_status_view] GoogleAdsRow parental_status_view
                             * @property {google.ads.googleads.v11.resources.IProductBiddingCategoryConstant|null} [product_bidding_category_constant] GoogleAdsRow product_bidding_category_constant
                             * @property {google.ads.googleads.v11.resources.IProductGroupView|null} [product_group_view] GoogleAdsRow product_group_view
                             * @property {google.ads.googleads.v11.resources.IRecommendation|null} [recommendation] GoogleAdsRow recommendation
                             * @property {google.ads.googleads.v11.resources.ISearchTermView|null} [search_term_view] GoogleAdsRow search_term_view
                             * @property {google.ads.googleads.v11.resources.ISharedCriterion|null} [shared_criterion] GoogleAdsRow shared_criterion
                             * @property {google.ads.googleads.v11.resources.ISharedSet|null} [shared_set] GoogleAdsRow shared_set
                             * @property {google.ads.googleads.v11.resources.ISmartCampaignSetting|null} [smart_campaign_setting] GoogleAdsRow smart_campaign_setting
                             * @property {google.ads.googleads.v11.resources.IShoppingPerformanceView|null} [shopping_performance_view] GoogleAdsRow shopping_performance_view
                             * @property {google.ads.googleads.v11.resources.ISmartCampaignSearchTermView|null} [smart_campaign_search_term_view] GoogleAdsRow smart_campaign_search_term_view
                             * @property {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLink|null} [third_party_app_analytics_link] GoogleAdsRow third_party_app_analytics_link
                             * @property {google.ads.googleads.v11.resources.ITopicView|null} [topic_view] GoogleAdsRow topic_view
                             * @property {google.ads.googleads.v11.resources.IExperiment|null} [experiment] GoogleAdsRow experiment
                             * @property {google.ads.googleads.v11.resources.IExperimentArm|null} [experiment_arm] GoogleAdsRow experiment_arm
                             * @property {google.ads.googleads.v11.resources.IUserInterest|null} [user_interest] GoogleAdsRow user_interest
                             * @property {google.ads.googleads.v11.resources.ILifeEvent|null} [life_event] GoogleAdsRow life_event
                             * @property {google.ads.googleads.v11.resources.IUserList|null} [user_list] GoogleAdsRow user_list
                             * @property {google.ads.googleads.v11.resources.IUserLocationView|null} [user_location_view] GoogleAdsRow user_location_view
                             * @property {google.ads.googleads.v11.resources.IRemarketingAction|null} [remarketing_action] GoogleAdsRow remarketing_action
                             * @property {google.ads.googleads.v11.resources.ITopicConstant|null} [topic_constant] GoogleAdsRow topic_constant
                             * @property {google.ads.googleads.v11.resources.IVideo|null} [video] GoogleAdsRow video
                             * @property {google.ads.googleads.v11.resources.IWebpageView|null} [webpage_view] GoogleAdsRow webpage_view
                             * @property {google.ads.googleads.v11.resources.ILeadFormSubmissionData|null} [lead_form_submission_data] GoogleAdsRow lead_form_submission_data
                             * @property {google.ads.googleads.v11.common.IMetrics|null} [metrics] GoogleAdsRow metrics
                             * @property {google.ads.googleads.v11.common.ISegments|null} [segments] GoogleAdsRow segments
                             */
    
                            /**
                             * Constructs a new GoogleAdsRow.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GoogleAdsRow.
                             * @implements IGoogleAdsRow
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGoogleAdsRow=} [p] Properties to set
                             */
                            function GoogleAdsRow(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GoogleAdsRow account_budget.
                             * @member {google.ads.googleads.v11.resources.IAccountBudget|null|undefined} account_budget
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.account_budget = null;
    
                            /**
                             * GoogleAdsRow account_budget_proposal.
                             * @member {google.ads.googleads.v11.resources.IAccountBudgetProposal|null|undefined} account_budget_proposal
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.account_budget_proposal = null;
    
                            /**
                             * GoogleAdsRow account_link.
                             * @member {google.ads.googleads.v11.resources.IAccountLink|null|undefined} account_link
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.account_link = null;
    
                            /**
                             * GoogleAdsRow ad_group.
                             * @member {google.ads.googleads.v11.resources.IAdGroup|null|undefined} ad_group
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group = null;
    
                            /**
                             * GoogleAdsRow ad_group_ad.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAd|null|undefined} ad_group_ad
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_ad = null;
    
                            /**
                             * GoogleAdsRow ad_group_ad_asset_combination_view.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAdAssetCombinationView|null|undefined} ad_group_ad_asset_combination_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_ad_asset_combination_view = null;
    
                            /**
                             * GoogleAdsRow ad_group_ad_asset_view.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAdAssetView|null|undefined} ad_group_ad_asset_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_ad_asset_view = null;
    
                            /**
                             * GoogleAdsRow ad_group_ad_label.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAdLabel|null|undefined} ad_group_ad_label
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_ad_label = null;
    
                            /**
                             * GoogleAdsRow ad_group_asset.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAsset|null|undefined} ad_group_asset
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_asset = null;
    
                            /**
                             * GoogleAdsRow ad_group_audience_view.
                             * @member {google.ads.googleads.v11.resources.IAdGroupAudienceView|null|undefined} ad_group_audience_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_audience_view = null;
    
                            /**
                             * GoogleAdsRow ad_group_bid_modifier.
                             * @member {google.ads.googleads.v11.resources.IAdGroupBidModifier|null|undefined} ad_group_bid_modifier
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_bid_modifier = null;
    
                            /**
                             * GoogleAdsRow ad_group_criterion.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterion|null|undefined} ad_group_criterion
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_criterion = null;
    
                            /**
                             * GoogleAdsRow ad_group_criterion_customizer.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterionCustomizer|null|undefined} ad_group_criterion_customizer
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_criterion_customizer = null;
    
                            /**
                             * GoogleAdsRow ad_group_criterion_label.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterionLabel|null|undefined} ad_group_criterion_label
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_criterion_label = null;
    
                            /**
                             * GoogleAdsRow ad_group_criterion_simulation.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCriterionSimulation|null|undefined} ad_group_criterion_simulation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_criterion_simulation = null;
    
                            /**
                             * GoogleAdsRow ad_group_customizer.
                             * @member {google.ads.googleads.v11.resources.IAdGroupCustomizer|null|undefined} ad_group_customizer
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_customizer = null;
    
                            /**
                             * GoogleAdsRow ad_group_extension_setting.
                             * @member {google.ads.googleads.v11.resources.IAdGroupExtensionSetting|null|undefined} ad_group_extension_setting
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_extension_setting = null;
    
                            /**
                             * GoogleAdsRow ad_group_feed.
                             * @member {google.ads.googleads.v11.resources.IAdGroupFeed|null|undefined} ad_group_feed
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_feed = null;
    
                            /**
                             * GoogleAdsRow ad_group_label.
                             * @member {google.ads.googleads.v11.resources.IAdGroupLabel|null|undefined} ad_group_label
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_label = null;
    
                            /**
                             * GoogleAdsRow ad_group_simulation.
                             * @member {google.ads.googleads.v11.resources.IAdGroupSimulation|null|undefined} ad_group_simulation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_group_simulation = null;
    
                            /**
                             * GoogleAdsRow ad_parameter.
                             * @member {google.ads.googleads.v11.resources.IAdParameter|null|undefined} ad_parameter
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_parameter = null;
    
                            /**
                             * GoogleAdsRow age_range_view.
                             * @member {google.ads.googleads.v11.resources.IAgeRangeView|null|undefined} age_range_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.age_range_view = null;
    
                            /**
                             * GoogleAdsRow ad_schedule_view.
                             * @member {google.ads.googleads.v11.resources.IAdScheduleView|null|undefined} ad_schedule_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.ad_schedule_view = null;
    
                            /**
                             * GoogleAdsRow domain_category.
                             * @member {google.ads.googleads.v11.resources.IDomainCategory|null|undefined} domain_category
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.domain_category = null;
    
                            /**
                             * GoogleAdsRow asset.
                             * @member {google.ads.googleads.v11.resources.IAsset|null|undefined} asset
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset = null;
    
                            /**
                             * GoogleAdsRow asset_field_type_view.
                             * @member {google.ads.googleads.v11.resources.IAssetFieldTypeView|null|undefined} asset_field_type_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_field_type_view = null;
    
                            /**
                             * GoogleAdsRow asset_group_asset.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupAsset|null|undefined} asset_group_asset
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_group_asset = null;
    
                            /**
                             * GoogleAdsRow asset_group_signal.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupSignal|null|undefined} asset_group_signal
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_group_signal = null;
    
                            /**
                             * GoogleAdsRow asset_group_listing_group_filter.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupListingGroupFilter|null|undefined} asset_group_listing_group_filter
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_group_listing_group_filter = null;
    
                            /**
                             * GoogleAdsRow asset_group_product_group_view.
                             * @member {google.ads.googleads.v11.resources.IAssetGroupProductGroupView|null|undefined} asset_group_product_group_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_group_product_group_view = null;
    
                            /**
                             * GoogleAdsRow asset_group.
                             * @member {google.ads.googleads.v11.resources.IAssetGroup|null|undefined} asset_group
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_group = null;
    
                            /**
                             * GoogleAdsRow asset_set_asset.
                             * @member {google.ads.googleads.v11.resources.IAssetSetAsset|null|undefined} asset_set_asset
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_set_asset = null;
    
                            /**
                             * GoogleAdsRow asset_set.
                             * @member {google.ads.googleads.v11.resources.IAssetSet|null|undefined} asset_set
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.asset_set = null;
    
                            /**
                             * GoogleAdsRow batch_job.
                             * @member {google.ads.googleads.v11.resources.IBatchJob|null|undefined} batch_job
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.batch_job = null;
    
                            /**
                             * GoogleAdsRow bidding_data_exclusion.
                             * @member {google.ads.googleads.v11.resources.IBiddingDataExclusion|null|undefined} bidding_data_exclusion
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.bidding_data_exclusion = null;
    
                            /**
                             * GoogleAdsRow bidding_seasonality_adjustment.
                             * @member {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null|undefined} bidding_seasonality_adjustment
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.bidding_seasonality_adjustment = null;
    
                            /**
                             * GoogleAdsRow bidding_strategy.
                             * @member {google.ads.googleads.v11.resources.IBiddingStrategy|null|undefined} bidding_strategy
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.bidding_strategy = null;
    
                            /**
                             * GoogleAdsRow bidding_strategy_simulation.
                             * @member {google.ads.googleads.v11.resources.IBiddingStrategySimulation|null|undefined} bidding_strategy_simulation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.bidding_strategy_simulation = null;
    
                            /**
                             * GoogleAdsRow billing_setup.
                             * @member {google.ads.googleads.v11.resources.IBillingSetup|null|undefined} billing_setup
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.billing_setup = null;
    
                            /**
                             * GoogleAdsRow call_view.
                             * @member {google.ads.googleads.v11.resources.ICallView|null|undefined} call_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.call_view = null;
    
                            /**
                             * GoogleAdsRow campaign_budget.
                             * @member {google.ads.googleads.v11.resources.ICampaignBudget|null|undefined} campaign_budget
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_budget = null;
    
                            /**
                             * GoogleAdsRow campaign.
                             * @member {google.ads.googleads.v11.resources.ICampaign|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign = null;
    
                            /**
                             * GoogleAdsRow campaign_asset.
                             * @member {google.ads.googleads.v11.resources.ICampaignAsset|null|undefined} campaign_asset
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_asset = null;
    
                            /**
                             * GoogleAdsRow campaign_asset_set.
                             * @member {google.ads.googleads.v11.resources.ICampaignAssetSet|null|undefined} campaign_asset_set
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_asset_set = null;
    
                            /**
                             * GoogleAdsRow campaign_audience_view.
                             * @member {google.ads.googleads.v11.resources.ICampaignAudienceView|null|undefined} campaign_audience_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_audience_view = null;
    
                            /**
                             * GoogleAdsRow campaign_bid_modifier.
                             * @member {google.ads.googleads.v11.resources.ICampaignBidModifier|null|undefined} campaign_bid_modifier
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_bid_modifier = null;
    
                            /**
                             * GoogleAdsRow campaign_conversion_goal.
                             * @member {google.ads.googleads.v11.resources.ICampaignConversionGoal|null|undefined} campaign_conversion_goal
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_conversion_goal = null;
    
                            /**
                             * GoogleAdsRow campaign_criterion.
                             * @member {google.ads.googleads.v11.resources.ICampaignCriterion|null|undefined} campaign_criterion
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_criterion = null;
    
                            /**
                             * GoogleAdsRow campaign_criterion_simulation.
                             * @member {google.ads.googleads.v11.resources.ICampaignCriterionSimulation|null|undefined} campaign_criterion_simulation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_criterion_simulation = null;
    
                            /**
                             * GoogleAdsRow campaign_customizer.
                             * @member {google.ads.googleads.v11.resources.ICampaignCustomizer|null|undefined} campaign_customizer
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_customizer = null;
    
                            /**
                             * GoogleAdsRow campaign_draft.
                             * @member {google.ads.googleads.v11.resources.ICampaignDraft|null|undefined} campaign_draft
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_draft = null;
    
                            /**
                             * GoogleAdsRow campaign_experiment.
                             * @member {google.ads.googleads.v11.resources.ICampaignExperiment|null|undefined} campaign_experiment
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_experiment = null;
    
                            /**
                             * GoogleAdsRow campaign_extension_setting.
                             * @member {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null|undefined} campaign_extension_setting
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_extension_setting = null;
    
                            /**
                             * GoogleAdsRow campaign_feed.
                             * @member {google.ads.googleads.v11.resources.ICampaignFeed|null|undefined} campaign_feed
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_feed = null;
    
                            /**
                             * GoogleAdsRow campaign_group.
                             * @member {google.ads.googleads.v11.resources.ICampaignGroup|null|undefined} campaign_group
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_group = null;
    
                            /**
                             * GoogleAdsRow campaign_label.
                             * @member {google.ads.googleads.v11.resources.ICampaignLabel|null|undefined} campaign_label
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_label = null;
    
                            /**
                             * GoogleAdsRow campaign_shared_set.
                             * @member {google.ads.googleads.v11.resources.ICampaignSharedSet|null|undefined} campaign_shared_set
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_shared_set = null;
    
                            /**
                             * GoogleAdsRow campaign_simulation.
                             * @member {google.ads.googleads.v11.resources.ICampaignSimulation|null|undefined} campaign_simulation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.campaign_simulation = null;
    
                            /**
                             * GoogleAdsRow carrier_constant.
                             * @member {google.ads.googleads.v11.resources.ICarrierConstant|null|undefined} carrier_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.carrier_constant = null;
    
                            /**
                             * GoogleAdsRow change_event.
                             * @member {google.ads.googleads.v11.resources.IChangeEvent|null|undefined} change_event
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.change_event = null;
    
                            /**
                             * GoogleAdsRow change_status.
                             * @member {google.ads.googleads.v11.resources.IChangeStatus|null|undefined} change_status
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.change_status = null;
    
                            /**
                             * GoogleAdsRow combined_audience.
                             * @member {google.ads.googleads.v11.resources.ICombinedAudience|null|undefined} combined_audience
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.combined_audience = null;
    
                            /**
                             * GoogleAdsRow audience.
                             * @member {google.ads.googleads.v11.resources.IAudience|null|undefined} audience
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.audience = null;
    
                            /**
                             * GoogleAdsRow conversion_action.
                             * @member {google.ads.googleads.v11.resources.IConversionAction|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.conversion_action = null;
    
                            /**
                             * GoogleAdsRow conversion_custom_variable.
                             * @member {google.ads.googleads.v11.resources.IConversionCustomVariable|null|undefined} conversion_custom_variable
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.conversion_custom_variable = null;
    
                            /**
                             * GoogleAdsRow conversion_goal_campaign_config.
                             * @member {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig|null|undefined} conversion_goal_campaign_config
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.conversion_goal_campaign_config = null;
    
                            /**
                             * GoogleAdsRow conversion_value_rule.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRule|null|undefined} conversion_value_rule
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.conversion_value_rule = null;
    
                            /**
                             * GoogleAdsRow conversion_value_rule_set.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRuleSet|null|undefined} conversion_value_rule_set
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.conversion_value_rule_set = null;
    
                            /**
                             * GoogleAdsRow click_view.
                             * @member {google.ads.googleads.v11.resources.IClickView|null|undefined} click_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.click_view = null;
    
                            /**
                             * GoogleAdsRow currency_constant.
                             * @member {google.ads.googleads.v11.resources.ICurrencyConstant|null|undefined} currency_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.currency_constant = null;
    
                            /**
                             * GoogleAdsRow custom_audience.
                             * @member {google.ads.googleads.v11.resources.ICustomAudience|null|undefined} custom_audience
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.custom_audience = null;
    
                            /**
                             * GoogleAdsRow custom_conversion_goal.
                             * @member {google.ads.googleads.v11.resources.ICustomConversionGoal|null|undefined} custom_conversion_goal
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.custom_conversion_goal = null;
    
                            /**
                             * GoogleAdsRow custom_interest.
                             * @member {google.ads.googleads.v11.resources.ICustomInterest|null|undefined} custom_interest
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.custom_interest = null;
    
                            /**
                             * GoogleAdsRow customer.
                             * @member {google.ads.googleads.v11.resources.ICustomer|null|undefined} customer
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer = null;
    
                            /**
                             * GoogleAdsRow customer_asset.
                             * @member {google.ads.googleads.v11.resources.ICustomerAsset|null|undefined} customer_asset
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_asset = null;
    
                            /**
                             * GoogleAdsRow accessible_bidding_strategy.
                             * @member {google.ads.googleads.v11.resources.IAccessibleBiddingStrategy|null|undefined} accessible_bidding_strategy
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.accessible_bidding_strategy = null;
    
                            /**
                             * GoogleAdsRow customer_customizer.
                             * @member {google.ads.googleads.v11.resources.ICustomerCustomizer|null|undefined} customer_customizer
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_customizer = null;
    
                            /**
                             * GoogleAdsRow customer_manager_link.
                             * @member {google.ads.googleads.v11.resources.ICustomerManagerLink|null|undefined} customer_manager_link
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_manager_link = null;
    
                            /**
                             * GoogleAdsRow customer_client_link.
                             * @member {google.ads.googleads.v11.resources.ICustomerClientLink|null|undefined} customer_client_link
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_client_link = null;
    
                            /**
                             * GoogleAdsRow customer_client.
                             * @member {google.ads.googleads.v11.resources.ICustomerClient|null|undefined} customer_client
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_client = null;
    
                            /**
                             * GoogleAdsRow customer_conversion_goal.
                             * @member {google.ads.googleads.v11.resources.ICustomerConversionGoal|null|undefined} customer_conversion_goal
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_conversion_goal = null;
    
                            /**
                             * GoogleAdsRow customer_extension_setting.
                             * @member {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null|undefined} customer_extension_setting
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_extension_setting = null;
    
                            /**
                             * GoogleAdsRow customer_feed.
                             * @member {google.ads.googleads.v11.resources.ICustomerFeed|null|undefined} customer_feed
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_feed = null;
    
                            /**
                             * GoogleAdsRow customer_label.
                             * @member {google.ads.googleads.v11.resources.ICustomerLabel|null|undefined} customer_label
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_label = null;
    
                            /**
                             * GoogleAdsRow customer_negative_criterion.
                             * @member {google.ads.googleads.v11.resources.ICustomerNegativeCriterion|null|undefined} customer_negative_criterion
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_negative_criterion = null;
    
                            /**
                             * GoogleAdsRow customer_user_access.
                             * @member {google.ads.googleads.v11.resources.ICustomerUserAccess|null|undefined} customer_user_access
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_user_access = null;
    
                            /**
                             * GoogleAdsRow customer_user_access_invitation.
                             * @member {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation|null|undefined} customer_user_access_invitation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customer_user_access_invitation = null;
    
                            /**
                             * GoogleAdsRow customizer_attribute.
                             * @member {google.ads.googleads.v11.resources.ICustomizerAttribute|null|undefined} customizer_attribute
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.customizer_attribute = null;
    
                            /**
                             * GoogleAdsRow detail_placement_view.
                             * @member {google.ads.googleads.v11.resources.IDetailPlacementView|null|undefined} detail_placement_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.detail_placement_view = null;
    
                            /**
                             * GoogleAdsRow detailed_demographic.
                             * @member {google.ads.googleads.v11.resources.IDetailedDemographic|null|undefined} detailed_demographic
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.detailed_demographic = null;
    
                            /**
                             * GoogleAdsRow display_keyword_view.
                             * @member {google.ads.googleads.v11.resources.IDisplayKeywordView|null|undefined} display_keyword_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.display_keyword_view = null;
    
                            /**
                             * GoogleAdsRow distance_view.
                             * @member {google.ads.googleads.v11.resources.IDistanceView|null|undefined} distance_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.distance_view = null;
    
                            /**
                             * GoogleAdsRow dynamic_search_ads_search_term_view.
                             * @member {google.ads.googleads.v11.resources.IDynamicSearchAdsSearchTermView|null|undefined} dynamic_search_ads_search_term_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.dynamic_search_ads_search_term_view = null;
    
                            /**
                             * GoogleAdsRow expanded_landing_page_view.
                             * @member {google.ads.googleads.v11.resources.IExpandedLandingPageView|null|undefined} expanded_landing_page_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.expanded_landing_page_view = null;
    
                            /**
                             * GoogleAdsRow extension_feed_item.
                             * @member {google.ads.googleads.v11.resources.IExtensionFeedItem|null|undefined} extension_feed_item
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.extension_feed_item = null;
    
                            /**
                             * GoogleAdsRow feed.
                             * @member {google.ads.googleads.v11.resources.IFeed|null|undefined} feed
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.feed = null;
    
                            /**
                             * GoogleAdsRow feed_item.
                             * @member {google.ads.googleads.v11.resources.IFeedItem|null|undefined} feed_item
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.feed_item = null;
    
                            /**
                             * GoogleAdsRow feed_item_set.
                             * @member {google.ads.googleads.v11.resources.IFeedItemSet|null|undefined} feed_item_set
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.feed_item_set = null;
    
                            /**
                             * GoogleAdsRow feed_item_set_link.
                             * @member {google.ads.googleads.v11.resources.IFeedItemSetLink|null|undefined} feed_item_set_link
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.feed_item_set_link = null;
    
                            /**
                             * GoogleAdsRow feed_item_target.
                             * @member {google.ads.googleads.v11.resources.IFeedItemTarget|null|undefined} feed_item_target
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.feed_item_target = null;
    
                            /**
                             * GoogleAdsRow feed_mapping.
                             * @member {google.ads.googleads.v11.resources.IFeedMapping|null|undefined} feed_mapping
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.feed_mapping = null;
    
                            /**
                             * GoogleAdsRow feed_placeholder_view.
                             * @member {google.ads.googleads.v11.resources.IFeedPlaceholderView|null|undefined} feed_placeholder_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.feed_placeholder_view = null;
    
                            /**
                             * GoogleAdsRow gender_view.
                             * @member {google.ads.googleads.v11.resources.IGenderView|null|undefined} gender_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.gender_view = null;
    
                            /**
                             * GoogleAdsRow geo_target_constant.
                             * @member {google.ads.googleads.v11.resources.IGeoTargetConstant|null|undefined} geo_target_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.geo_target_constant = null;
    
                            /**
                             * GoogleAdsRow geographic_view.
                             * @member {google.ads.googleads.v11.resources.IGeographicView|null|undefined} geographic_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.geographic_view = null;
    
                            /**
                             * GoogleAdsRow group_placement_view.
                             * @member {google.ads.googleads.v11.resources.IGroupPlacementView|null|undefined} group_placement_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.group_placement_view = null;
    
                            /**
                             * GoogleAdsRow hotel_group_view.
                             * @member {google.ads.googleads.v11.resources.IHotelGroupView|null|undefined} hotel_group_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.hotel_group_view = null;
    
                            /**
                             * GoogleAdsRow hotel_performance_view.
                             * @member {google.ads.googleads.v11.resources.IHotelPerformanceView|null|undefined} hotel_performance_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.hotel_performance_view = null;
    
                            /**
                             * GoogleAdsRow hotel_reconciliation.
                             * @member {google.ads.googleads.v11.resources.IHotelReconciliation|null|undefined} hotel_reconciliation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.hotel_reconciliation = null;
    
                            /**
                             * GoogleAdsRow income_range_view.
                             * @member {google.ads.googleads.v11.resources.IIncomeRangeView|null|undefined} income_range_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.income_range_view = null;
    
                            /**
                             * GoogleAdsRow keyword_view.
                             * @member {google.ads.googleads.v11.resources.IKeywordView|null|undefined} keyword_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.keyword_view = null;
    
                            /**
                             * GoogleAdsRow keyword_plan.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlan|null|undefined} keyword_plan
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.keyword_plan = null;
    
                            /**
                             * GoogleAdsRow keyword_plan_campaign.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanCampaign|null|undefined} keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.keyword_plan_campaign = null;
    
                            /**
                             * GoogleAdsRow keyword_plan_campaign_keyword.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword|null|undefined} keyword_plan_campaign_keyword
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.keyword_plan_campaign_keyword = null;
    
                            /**
                             * GoogleAdsRow keyword_plan_ad_group.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanAdGroup|null|undefined} keyword_plan_ad_group
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.keyword_plan_ad_group = null;
    
                            /**
                             * GoogleAdsRow keyword_plan_ad_group_keyword.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword|null|undefined} keyword_plan_ad_group_keyword
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.keyword_plan_ad_group_keyword = null;
    
                            /**
                             * GoogleAdsRow keyword_theme_constant.
                             * @member {google.ads.googleads.v11.resources.IKeywordThemeConstant|null|undefined} keyword_theme_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.keyword_theme_constant = null;
    
                            /**
                             * GoogleAdsRow label.
                             * @member {google.ads.googleads.v11.resources.ILabel|null|undefined} label
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.label = null;
    
                            /**
                             * GoogleAdsRow landing_page_view.
                             * @member {google.ads.googleads.v11.resources.ILandingPageView|null|undefined} landing_page_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.landing_page_view = null;
    
                            /**
                             * GoogleAdsRow language_constant.
                             * @member {google.ads.googleads.v11.resources.ILanguageConstant|null|undefined} language_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.language_constant = null;
    
                            /**
                             * GoogleAdsRow location_view.
                             * @member {google.ads.googleads.v11.resources.ILocationView|null|undefined} location_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.location_view = null;
    
                            /**
                             * GoogleAdsRow managed_placement_view.
                             * @member {google.ads.googleads.v11.resources.IManagedPlacementView|null|undefined} managed_placement_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.managed_placement_view = null;
    
                            /**
                             * GoogleAdsRow media_file.
                             * @member {google.ads.googleads.v11.resources.IMediaFile|null|undefined} media_file
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.media_file = null;
    
                            /**
                             * GoogleAdsRow mobile_app_category_constant.
                             * @member {google.ads.googleads.v11.resources.IMobileAppCategoryConstant|null|undefined} mobile_app_category_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.mobile_app_category_constant = null;
    
                            /**
                             * GoogleAdsRow mobile_device_constant.
                             * @member {google.ads.googleads.v11.resources.IMobileDeviceConstant|null|undefined} mobile_device_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.mobile_device_constant = null;
    
                            /**
                             * GoogleAdsRow offline_user_data_job.
                             * @member {google.ads.googleads.v11.resources.IOfflineUserDataJob|null|undefined} offline_user_data_job
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.offline_user_data_job = null;
    
                            /**
                             * GoogleAdsRow operating_system_version_constant.
                             * @member {google.ads.googleads.v11.resources.IOperatingSystemVersionConstant|null|undefined} operating_system_version_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.operating_system_version_constant = null;
    
                            /**
                             * GoogleAdsRow paid_organic_search_term_view.
                             * @member {google.ads.googleads.v11.resources.IPaidOrganicSearchTermView|null|undefined} paid_organic_search_term_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.paid_organic_search_term_view = null;
    
                            /**
                             * GoogleAdsRow parental_status_view.
                             * @member {google.ads.googleads.v11.resources.IParentalStatusView|null|undefined} parental_status_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.parental_status_view = null;
    
                            /**
                             * GoogleAdsRow product_bidding_category_constant.
                             * @member {google.ads.googleads.v11.resources.IProductBiddingCategoryConstant|null|undefined} product_bidding_category_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.product_bidding_category_constant = null;
    
                            /**
                             * GoogleAdsRow product_group_view.
                             * @member {google.ads.googleads.v11.resources.IProductGroupView|null|undefined} product_group_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.product_group_view = null;
    
                            /**
                             * GoogleAdsRow recommendation.
                             * @member {google.ads.googleads.v11.resources.IRecommendation|null|undefined} recommendation
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.recommendation = null;
    
                            /**
                             * GoogleAdsRow search_term_view.
                             * @member {google.ads.googleads.v11.resources.ISearchTermView|null|undefined} search_term_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.search_term_view = null;
    
                            /**
                             * GoogleAdsRow shared_criterion.
                             * @member {google.ads.googleads.v11.resources.ISharedCriterion|null|undefined} shared_criterion
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.shared_criterion = null;
    
                            /**
                             * GoogleAdsRow shared_set.
                             * @member {google.ads.googleads.v11.resources.ISharedSet|null|undefined} shared_set
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.shared_set = null;
    
                            /**
                             * GoogleAdsRow smart_campaign_setting.
                             * @member {google.ads.googleads.v11.resources.ISmartCampaignSetting|null|undefined} smart_campaign_setting
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.smart_campaign_setting = null;
    
                            /**
                             * GoogleAdsRow shopping_performance_view.
                             * @member {google.ads.googleads.v11.resources.IShoppingPerformanceView|null|undefined} shopping_performance_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.shopping_performance_view = null;
    
                            /**
                             * GoogleAdsRow smart_campaign_search_term_view.
                             * @member {google.ads.googleads.v11.resources.ISmartCampaignSearchTermView|null|undefined} smart_campaign_search_term_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.smart_campaign_search_term_view = null;
    
                            /**
                             * GoogleAdsRow third_party_app_analytics_link.
                             * @member {google.ads.googleads.v11.resources.IThirdPartyAppAnalyticsLink|null|undefined} third_party_app_analytics_link
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.third_party_app_analytics_link = null;
    
                            /**
                             * GoogleAdsRow topic_view.
                             * @member {google.ads.googleads.v11.resources.ITopicView|null|undefined} topic_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.topic_view = null;
    
                            /**
                             * GoogleAdsRow experiment.
                             * @member {google.ads.googleads.v11.resources.IExperiment|null|undefined} experiment
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.experiment = null;
    
                            /**
                             * GoogleAdsRow experiment_arm.
                             * @member {google.ads.googleads.v11.resources.IExperimentArm|null|undefined} experiment_arm
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.experiment_arm = null;
    
                            /**
                             * GoogleAdsRow user_interest.
                             * @member {google.ads.googleads.v11.resources.IUserInterest|null|undefined} user_interest
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.user_interest = null;
    
                            /**
                             * GoogleAdsRow life_event.
                             * @member {google.ads.googleads.v11.resources.ILifeEvent|null|undefined} life_event
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.life_event = null;
    
                            /**
                             * GoogleAdsRow user_list.
                             * @member {google.ads.googleads.v11.resources.IUserList|null|undefined} user_list
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.user_list = null;
    
                            /**
                             * GoogleAdsRow user_location_view.
                             * @member {google.ads.googleads.v11.resources.IUserLocationView|null|undefined} user_location_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.user_location_view = null;
    
                            /**
                             * GoogleAdsRow remarketing_action.
                             * @member {google.ads.googleads.v11.resources.IRemarketingAction|null|undefined} remarketing_action
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.remarketing_action = null;
    
                            /**
                             * GoogleAdsRow topic_constant.
                             * @member {google.ads.googleads.v11.resources.ITopicConstant|null|undefined} topic_constant
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.topic_constant = null;
    
                            /**
                             * GoogleAdsRow video.
                             * @member {google.ads.googleads.v11.resources.IVideo|null|undefined} video
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.video = null;
    
                            /**
                             * GoogleAdsRow webpage_view.
                             * @member {google.ads.googleads.v11.resources.IWebpageView|null|undefined} webpage_view
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.webpage_view = null;
    
                            /**
                             * GoogleAdsRow lead_form_submission_data.
                             * @member {google.ads.googleads.v11.resources.ILeadFormSubmissionData|null|undefined} lead_form_submission_data
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.lead_form_submission_data = null;
    
                            /**
                             * GoogleAdsRow metrics.
                             * @member {google.ads.googleads.v11.common.IMetrics|null|undefined} metrics
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.metrics = null;
    
                            /**
                             * GoogleAdsRow segments.
                             * @member {google.ads.googleads.v11.common.ISegments|null|undefined} segments
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             */
                            GoogleAdsRow.prototype.segments = null;
    
                            /**
                             * Creates a new GoogleAdsRow instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {google.ads.googleads.v11.services.IGoogleAdsRow=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GoogleAdsRow} GoogleAdsRow instance
                             */
                            GoogleAdsRow.create = function create(properties) {
                                return new GoogleAdsRow(properties);
                            };
    
                            /**
                             * Encodes the specified GoogleAdsRow message. Does not implicitly {@link google.ads.googleads.v11.services.GoogleAdsRow.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {google.ads.googleads.v11.services.IGoogleAdsRow} m GoogleAdsRow message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsRow.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer != null && Object.hasOwnProperty.call(m, "customer"))
                                    $root.google.ads.googleads.v11.resources.Customer.encode(m.customer, w.uint32(10).fork()).ldelim();
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    $root.google.ads.googleads.v11.resources.Campaign.encode(m.campaign, w.uint32(18).fork()).ldelim();
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    $root.google.ads.googleads.v11.resources.AdGroup.encode(m.ad_group, w.uint32(26).fork()).ldelim();
                                if (m.metrics != null && Object.hasOwnProperty.call(m, "metrics"))
                                    $root.google.ads.googleads.v11.common.Metrics.encode(m.metrics, w.uint32(34).fork()).ldelim();
                                if (m.ad_group_ad != null && Object.hasOwnProperty.call(m, "ad_group_ad"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAd.encode(m.ad_group_ad, w.uint32(130).fork()).ldelim();
                                if (m.ad_group_criterion != null && Object.hasOwnProperty.call(m, "ad_group_criterion"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterion.encode(m.ad_group_criterion, w.uint32(138).fork()).ldelim();
                                if (m.bidding_strategy != null && Object.hasOwnProperty.call(m, "bidding_strategy"))
                                    $root.google.ads.googleads.v11.resources.BiddingStrategy.encode(m.bidding_strategy, w.uint32(146).fork()).ldelim();
                                if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                    $root.google.ads.googleads.v11.resources.CampaignBudget.encode(m.campaign_budget, w.uint32(154).fork()).ldelim();
                                if (m.campaign_criterion != null && Object.hasOwnProperty.call(m, "campaign_criterion"))
                                    $root.google.ads.googleads.v11.resources.CampaignCriterion.encode(m.campaign_criterion, w.uint32(162).fork()).ldelim();
                                if (m.keyword_view != null && Object.hasOwnProperty.call(m, "keyword_view"))
                                    $root.google.ads.googleads.v11.resources.KeywordView.encode(m.keyword_view, w.uint32(170).fork()).ldelim();
                                if (m.recommendation != null && Object.hasOwnProperty.call(m, "recommendation"))
                                    $root.google.ads.googleads.v11.resources.Recommendation.encode(m.recommendation, w.uint32(178).fork()).ldelim();
                                if (m.geo_target_constant != null && Object.hasOwnProperty.call(m, "geo_target_constant"))
                                    $root.google.ads.googleads.v11.resources.GeoTargetConstant.encode(m.geo_target_constant, w.uint32(186).fork()).ldelim();
                                if (m.ad_group_bid_modifier != null && Object.hasOwnProperty.call(m, "ad_group_bid_modifier"))
                                    $root.google.ads.googleads.v11.resources.AdGroupBidModifier.encode(m.ad_group_bid_modifier, w.uint32(194).fork()).ldelim();
                                if (m.campaign_group != null && Object.hasOwnProperty.call(m, "campaign_group"))
                                    $root.google.ads.googleads.v11.resources.CampaignGroup.encode(m.campaign_group, w.uint32(202).fork()).ldelim();
                                if (m.campaign_bid_modifier != null && Object.hasOwnProperty.call(m, "campaign_bid_modifier"))
                                    $root.google.ads.googleads.v11.resources.CampaignBidModifier.encode(m.campaign_bid_modifier, w.uint32(210).fork()).ldelim();
                                if (m.shared_set != null && Object.hasOwnProperty.call(m, "shared_set"))
                                    $root.google.ads.googleads.v11.resources.SharedSet.encode(m.shared_set, w.uint32(218).fork()).ldelim();
                                if (m.shared_criterion != null && Object.hasOwnProperty.call(m, "shared_criterion"))
                                    $root.google.ads.googleads.v11.resources.SharedCriterion.encode(m.shared_criterion, w.uint32(234).fork()).ldelim();
                                if (m.campaign_shared_set != null && Object.hasOwnProperty.call(m, "campaign_shared_set"))
                                    $root.google.ads.googleads.v11.resources.CampaignSharedSet.encode(m.campaign_shared_set, w.uint32(242).fork()).ldelim();
                                if (m.topic_constant != null && Object.hasOwnProperty.call(m, "topic_constant"))
                                    $root.google.ads.googleads.v11.resources.TopicConstant.encode(m.topic_constant, w.uint32(250).fork()).ldelim();
                                if (m.keyword_plan != null && Object.hasOwnProperty.call(m, "keyword_plan"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlan.encode(m.keyword_plan, w.uint32(258).fork()).ldelim();
                                if (m.keyword_plan_campaign != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.encode(m.keyword_plan_campaign, w.uint32(266).fork()).ldelim();
                                if (m.keyword_plan_ad_group != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.encode(m.keyword_plan_ad_group, w.uint32(282).fork()).ldelim();
                                if (m.change_status != null && Object.hasOwnProperty.call(m, "change_status"))
                                    $root.google.ads.googleads.v11.resources.ChangeStatus.encode(m.change_status, w.uint32(298).fork()).ldelim();
                                if (m.user_list != null && Object.hasOwnProperty.call(m, "user_list"))
                                    $root.google.ads.googleads.v11.resources.UserList.encode(m.user_list, w.uint32(306).fork()).ldelim();
                                if (m.video != null && Object.hasOwnProperty.call(m, "video"))
                                    $root.google.ads.googleads.v11.resources.Video.encode(m.video, w.uint32(314).fork()).ldelim();
                                if (m.gender_view != null && Object.hasOwnProperty.call(m, "gender_view"))
                                    $root.google.ads.googleads.v11.resources.GenderView.encode(m.gender_view, w.uint32(322).fork()).ldelim();
                                if (m.billing_setup != null && Object.hasOwnProperty.call(m, "billing_setup"))
                                    $root.google.ads.googleads.v11.resources.BillingSetup.encode(m.billing_setup, w.uint32(330).fork()).ldelim();
                                if (m.account_budget != null && Object.hasOwnProperty.call(m, "account_budget"))
                                    $root.google.ads.googleads.v11.resources.AccountBudget.encode(m.account_budget, w.uint32(338).fork()).ldelim();
                                if (m.account_budget_proposal != null && Object.hasOwnProperty.call(m, "account_budget_proposal"))
                                    $root.google.ads.googleads.v11.resources.AccountBudgetProposal.encode(m.account_budget_proposal, w.uint32(346).fork()).ldelim();
                                if (m.topic_view != null && Object.hasOwnProperty.call(m, "topic_view"))
                                    $root.google.ads.googleads.v11.resources.TopicView.encode(m.topic_view, w.uint32(354).fork()).ldelim();
                                if (m.parental_status_view != null && Object.hasOwnProperty.call(m, "parental_status_view"))
                                    $root.google.ads.googleads.v11.resources.ParentalStatusView.encode(m.parental_status_view, w.uint32(362).fork()).ldelim();
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    $root.google.ads.googleads.v11.resources.Feed.encode(m.feed, w.uint32(370).fork()).ldelim();
                                if (m.display_keyword_view != null && Object.hasOwnProperty.call(m, "display_keyword_view"))
                                    $root.google.ads.googleads.v11.resources.DisplayKeywordView.encode(m.display_keyword_view, w.uint32(378).fork()).ldelim();
                                if (m.age_range_view != null && Object.hasOwnProperty.call(m, "age_range_view"))
                                    $root.google.ads.googleads.v11.resources.AgeRangeView.encode(m.age_range_view, w.uint32(386).fork()).ldelim();
                                if (m.campaign_draft != null && Object.hasOwnProperty.call(m, "campaign_draft"))
                                    $root.google.ads.googleads.v11.resources.CampaignDraft.encode(m.campaign_draft, w.uint32(394).fork()).ldelim();
                                if (m.feed_item != null && Object.hasOwnProperty.call(m, "feed_item"))
                                    $root.google.ads.googleads.v11.resources.FeedItem.encode(m.feed_item, w.uint32(402).fork()).ldelim();
                                if (m.hotel_group_view != null && Object.hasOwnProperty.call(m, "hotel_group_view"))
                                    $root.google.ads.googleads.v11.resources.HotelGroupView.encode(m.hotel_group_view, w.uint32(410).fork()).ldelim();
                                if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                                    $root.google.ads.googleads.v11.resources.Label.encode(m.label, w.uint32(418).fork()).ldelim();
                                if (m.managed_placement_view != null && Object.hasOwnProperty.call(m, "managed_placement_view"))
                                    $root.google.ads.googleads.v11.resources.ManagedPlacementView.encode(m.managed_placement_view, w.uint32(426).fork()).ldelim();
                                if (m.product_group_view != null && Object.hasOwnProperty.call(m, "product_group_view"))
                                    $root.google.ads.googleads.v11.resources.ProductGroupView.encode(m.product_group_view, w.uint32(434).fork()).ldelim();
                                if (m.language_constant != null && Object.hasOwnProperty.call(m, "language_constant"))
                                    $root.google.ads.googleads.v11.resources.LanguageConstant.encode(m.language_constant, w.uint32(442).fork()).ldelim();
                                if (m.ad_group_audience_view != null && Object.hasOwnProperty.call(m, "ad_group_audience_view"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAudienceView.encode(m.ad_group_audience_view, w.uint32(458).fork()).ldelim();
                                if (m.feed_mapping != null && Object.hasOwnProperty.call(m, "feed_mapping"))
                                    $root.google.ads.googleads.v11.resources.FeedMapping.encode(m.feed_mapping, w.uint32(466).fork()).ldelim();
                                if (m.user_interest != null && Object.hasOwnProperty.call(m, "user_interest"))
                                    $root.google.ads.googleads.v11.resources.UserInterest.encode(m.user_interest, w.uint32(474).fork()).ldelim();
                                if (m.remarketing_action != null && Object.hasOwnProperty.call(m, "remarketing_action"))
                                    $root.google.ads.googleads.v11.resources.RemarketingAction.encode(m.remarketing_action, w.uint32(482).fork()).ldelim();
                                if (m.customer_manager_link != null && Object.hasOwnProperty.call(m, "customer_manager_link"))
                                    $root.google.ads.googleads.v11.resources.CustomerManagerLink.encode(m.customer_manager_link, w.uint32(490).fork()).ldelim();
                                if (m.customer_client_link != null && Object.hasOwnProperty.call(m, "customer_client_link"))
                                    $root.google.ads.googleads.v11.resources.CustomerClientLink.encode(m.customer_client_link, w.uint32(498).fork()).ldelim();
                                if (m.campaign_feed != null && Object.hasOwnProperty.call(m, "campaign_feed"))
                                    $root.google.ads.googleads.v11.resources.CampaignFeed.encode(m.campaign_feed, w.uint32(506).fork()).ldelim();
                                if (m.customer_feed != null && Object.hasOwnProperty.call(m, "customer_feed"))
                                    $root.google.ads.googleads.v11.resources.CustomerFeed.encode(m.customer_feed, w.uint32(514).fork()).ldelim();
                                if (m.carrier_constant != null && Object.hasOwnProperty.call(m, "carrier_constant"))
                                    $root.google.ads.googleads.v11.resources.CarrierConstant.encode(m.carrier_constant, w.uint32(530).fork()).ldelim();
                                if (m.ad_group_feed != null && Object.hasOwnProperty.call(m, "ad_group_feed"))
                                    $root.google.ads.googleads.v11.resources.AdGroupFeed.encode(m.ad_group_feed, w.uint32(538).fork()).ldelim();
                                if (m.search_term_view != null && Object.hasOwnProperty.call(m, "search_term_view"))
                                    $root.google.ads.googleads.v11.resources.SearchTermView.encode(m.search_term_view, w.uint32(546).fork()).ldelim();
                                if (m.campaign_audience_view != null && Object.hasOwnProperty.call(m, "campaign_audience_view"))
                                    $root.google.ads.googleads.v11.resources.CampaignAudienceView.encode(m.campaign_audience_view, w.uint32(554).fork()).ldelim();
                                if (m.customer_client != null && Object.hasOwnProperty.call(m, "customer_client"))
                                    $root.google.ads.googleads.v11.resources.CustomerClient.encode(m.customer_client, w.uint32(562).fork()).ldelim();
                                if (m.hotel_performance_view != null && Object.hasOwnProperty.call(m, "hotel_performance_view"))
                                    $root.google.ads.googleads.v11.resources.HotelPerformanceView.encode(m.hotel_performance_view, w.uint32(570).fork()).ldelim();
                                if (m.campaign_experiment != null && Object.hasOwnProperty.call(m, "campaign_experiment"))
                                    $root.google.ads.googleads.v11.resources.CampaignExperiment.encode(m.campaign_experiment, w.uint32(674).fork()).ldelim();
                                if (m.extension_feed_item != null && Object.hasOwnProperty.call(m, "extension_feed_item"))
                                    $root.google.ads.googleads.v11.resources.ExtensionFeedItem.encode(m.extension_feed_item, w.uint32(682).fork()).ldelim();
                                if (m.operating_system_version_constant != null && Object.hasOwnProperty.call(m, "operating_system_version_constant"))
                                    $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant.encode(m.operating_system_version_constant, w.uint32(690).fork()).ldelim();
                                if (m.mobile_app_category_constant != null && Object.hasOwnProperty.call(m, "mobile_app_category_constant"))
                                    $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant.encode(m.mobile_app_category_constant, w.uint32(698).fork()).ldelim();
                                if (m.customer_negative_criterion != null && Object.hasOwnProperty.call(m, "customer_negative_criterion"))
                                    $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.encode(m.customer_negative_criterion, w.uint32(706).fork()).ldelim();
                                if (m.ad_schedule_view != null && Object.hasOwnProperty.call(m, "ad_schedule_view"))
                                    $root.google.ads.googleads.v11.resources.AdScheduleView.encode(m.ad_schedule_view, w.uint32(714).fork()).ldelim();
                                if (m.media_file != null && Object.hasOwnProperty.call(m, "media_file"))
                                    $root.google.ads.googleads.v11.resources.MediaFile.encode(m.media_file, w.uint32(722).fork()).ldelim();
                                if (m.domain_category != null && Object.hasOwnProperty.call(m, "domain_category"))
                                    $root.google.ads.googleads.v11.resources.DomainCategory.encode(m.domain_category, w.uint32(730).fork()).ldelim();
                                if (m.feed_placeholder_view != null && Object.hasOwnProperty.call(m, "feed_placeholder_view"))
                                    $root.google.ads.googleads.v11.resources.FeedPlaceholderView.encode(m.feed_placeholder_view, w.uint32(778).fork()).ldelim();
                                if (m.mobile_device_constant != null && Object.hasOwnProperty.call(m, "mobile_device_constant"))
                                    $root.google.ads.googleads.v11.resources.MobileDeviceConstant.encode(m.mobile_device_constant, w.uint32(786).fork()).ldelim();
                                if (m.segments != null && Object.hasOwnProperty.call(m, "segments"))
                                    $root.google.ads.googleads.v11.common.Segments.encode(m.segments, w.uint32(818).fork()).ldelim();
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.encode(m.conversion_action, w.uint32(826).fork()).ldelim();
                                if (m.custom_interest != null && Object.hasOwnProperty.call(m, "custom_interest"))
                                    $root.google.ads.googleads.v11.resources.CustomInterest.encode(m.custom_interest, w.uint32(834).fork()).ldelim();
                                if (m.asset != null && Object.hasOwnProperty.call(m, "asset"))
                                    $root.google.ads.googleads.v11.resources.Asset.encode(m.asset, w.uint32(842).fork()).ldelim();
                                if (m.dynamic_search_ads_search_term_view != null && Object.hasOwnProperty.call(m, "dynamic_search_ads_search_term_view"))
                                    $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView.encode(m.dynamic_search_ads_search_term_view, w.uint32(850).fork()).ldelim();
                                if (m.ad_group_simulation != null && Object.hasOwnProperty.call(m, "ad_group_simulation"))
                                    $root.google.ads.googleads.v11.resources.AdGroupSimulation.encode(m.ad_group_simulation, w.uint32(858).fork()).ldelim();
                                if (m.campaign_label != null && Object.hasOwnProperty.call(m, "campaign_label"))
                                    $root.google.ads.googleads.v11.resources.CampaignLabel.encode(m.campaign_label, w.uint32(866).fork()).ldelim();
                                if (m.product_bidding_category_constant != null && Object.hasOwnProperty.call(m, "product_bidding_category_constant"))
                                    $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant.encode(m.product_bidding_category_constant, w.uint32(874).fork()).ldelim();
                                if (m.ad_group_criterion_simulation != null && Object.hasOwnProperty.call(m, "ad_group_criterion_simulation"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation.encode(m.ad_group_criterion_simulation, w.uint32(882).fork()).ldelim();
                                if (m.campaign_criterion_simulation != null && Object.hasOwnProperty.call(m, "campaign_criterion_simulation"))
                                    $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation.encode(m.campaign_criterion_simulation, w.uint32(890).fork()).ldelim();
                                if (m.ad_group_extension_setting != null && Object.hasOwnProperty.call(m, "ad_group_extension_setting"))
                                    $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.encode(m.ad_group_extension_setting, w.uint32(898).fork()).ldelim();
                                if (m.campaign_extension_setting != null && Object.hasOwnProperty.call(m, "campaign_extension_setting"))
                                    $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.encode(m.campaign_extension_setting, w.uint32(906).fork()).ldelim();
                                if (m.customer_extension_setting != null && Object.hasOwnProperty.call(m, "customer_extension_setting"))
                                    $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.encode(m.customer_extension_setting, w.uint32(914).fork()).ldelim();
                                if (m.ad_group_label != null && Object.hasOwnProperty.call(m, "ad_group_label"))
                                    $root.google.ads.googleads.v11.resources.AdGroupLabel.encode(m.ad_group_label, w.uint32(922).fork()).ldelim();
                                if (m.feed_item_target != null && Object.hasOwnProperty.call(m, "feed_item_target"))
                                    $root.google.ads.googleads.v11.resources.FeedItemTarget.encode(m.feed_item_target, w.uint32(930).fork()).ldelim();
                                if (m.shopping_performance_view != null && Object.hasOwnProperty.call(m, "shopping_performance_view"))
                                    $root.google.ads.googleads.v11.resources.ShoppingPerformanceView.encode(m.shopping_performance_view, w.uint32(938).fork()).ldelim();
                                if (m.detail_placement_view != null && Object.hasOwnProperty.call(m, "detail_placement_view"))
                                    $root.google.ads.googleads.v11.resources.DetailPlacementView.encode(m.detail_placement_view, w.uint32(946).fork()).ldelim();
                                if (m.group_placement_view != null && Object.hasOwnProperty.call(m, "group_placement_view"))
                                    $root.google.ads.googleads.v11.resources.GroupPlacementView.encode(m.group_placement_view, w.uint32(954).fork()).ldelim();
                                if (m.ad_group_ad_label != null && Object.hasOwnProperty.call(m, "ad_group_ad_label"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAdLabel.encode(m.ad_group_ad_label, w.uint32(962).fork()).ldelim();
                                if (m.ad_group_criterion_label != null && Object.hasOwnProperty.call(m, "ad_group_criterion_label"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.encode(m.ad_group_criterion_label, w.uint32(970).fork()).ldelim();
                                if (m.click_view != null && Object.hasOwnProperty.call(m, "click_view"))
                                    $root.google.ads.googleads.v11.resources.ClickView.encode(m.click_view, w.uint32(978).fork()).ldelim();
                                if (m.location_view != null && Object.hasOwnProperty.call(m, "location_view"))
                                    $root.google.ads.googleads.v11.resources.LocationView.encode(m.location_view, w.uint32(986).fork()).ldelim();
                                if (m.customer_label != null && Object.hasOwnProperty.call(m, "customer_label"))
                                    $root.google.ads.googleads.v11.resources.CustomerLabel.encode(m.customer_label, w.uint32(994).fork()).ldelim();
                                if (m.geographic_view != null && Object.hasOwnProperty.call(m, "geographic_view"))
                                    $root.google.ads.googleads.v11.resources.GeographicView.encode(m.geographic_view, w.uint32(1002).fork()).ldelim();
                                if (m.landing_page_view != null && Object.hasOwnProperty.call(m, "landing_page_view"))
                                    $root.google.ads.googleads.v11.resources.LandingPageView.encode(m.landing_page_view, w.uint32(1010).fork()).ldelim();
                                if (m.expanded_landing_page_view != null && Object.hasOwnProperty.call(m, "expanded_landing_page_view"))
                                    $root.google.ads.googleads.v11.resources.ExpandedLandingPageView.encode(m.expanded_landing_page_view, w.uint32(1026).fork()).ldelim();
                                if (m.paid_organic_search_term_view != null && Object.hasOwnProperty.call(m, "paid_organic_search_term_view"))
                                    $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView.encode(m.paid_organic_search_term_view, w.uint32(1034).fork()).ldelim();
                                if (m.ad_parameter != null && Object.hasOwnProperty.call(m, "ad_parameter"))
                                    $root.google.ads.googleads.v11.resources.AdParameter.encode(m.ad_parameter, w.uint32(1042).fork()).ldelim();
                                if (m.ad_group_ad_asset_view != null && Object.hasOwnProperty.call(m, "ad_group_ad_asset_view"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAdAssetView.encode(m.ad_group_ad_asset_view, w.uint32(1050).fork()).ldelim();
                                if (m.distance_view != null && Object.hasOwnProperty.call(m, "distance_view"))
                                    $root.google.ads.googleads.v11.resources.DistanceView.encode(m.distance_view, w.uint32(1058).fork()).ldelim();
                                if (m.experiment != null && Object.hasOwnProperty.call(m, "experiment"))
                                    $root.google.ads.googleads.v11.resources.Experiment.encode(m.experiment, w.uint32(1066).fork()).ldelim();
                                if (m.currency_constant != null && Object.hasOwnProperty.call(m, "currency_constant"))
                                    $root.google.ads.googleads.v11.resources.CurrencyConstant.encode(m.currency_constant, w.uint32(1074).fork()).ldelim();
                                if (m.user_location_view != null && Object.hasOwnProperty.call(m, "user_location_view"))
                                    $root.google.ads.googleads.v11.resources.UserLocationView.encode(m.user_location_view, w.uint32(1082).fork()).ldelim();
                                if (m.offline_user_data_job != null && Object.hasOwnProperty.call(m, "offline_user_data_job"))
                                    $root.google.ads.googleads.v11.resources.OfflineUserDataJob.encode(m.offline_user_data_job, w.uint32(1098).fork()).ldelim();
                                if (m.income_range_view != null && Object.hasOwnProperty.call(m, "income_range_view"))
                                    $root.google.ads.googleads.v11.resources.IncomeRangeView.encode(m.income_range_view, w.uint32(1106).fork()).ldelim();
                                if (m.batch_job != null && Object.hasOwnProperty.call(m, "batch_job"))
                                    $root.google.ads.googleads.v11.resources.BatchJob.encode(m.batch_job, w.uint32(1114).fork()).ldelim();
                                if (m.keyword_plan_campaign_keyword != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign_keyword"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.encode(m.keyword_plan_campaign_keyword, w.uint32(1122).fork()).ldelim();
                                if (m.keyword_plan_ad_group_keyword != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_keyword"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.encode(m.keyword_plan_ad_group_keyword, w.uint32(1130).fork()).ldelim();
                                if (m.campaign_asset != null && Object.hasOwnProperty.call(m, "campaign_asset"))
                                    $root.google.ads.googleads.v11.resources.CampaignAsset.encode(m.campaign_asset, w.uint32(1138).fork()).ldelim();
                                if (m.account_link != null && Object.hasOwnProperty.call(m, "account_link"))
                                    $root.google.ads.googleads.v11.resources.AccountLink.encode(m.account_link, w.uint32(1146).fork()).ldelim();
                                if (m.third_party_app_analytics_link != null && Object.hasOwnProperty.call(m, "third_party_app_analytics_link"))
                                    $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink.encode(m.third_party_app_analytics_link, w.uint32(1154).fork()).ldelim();
                                if (m.change_event != null && Object.hasOwnProperty.call(m, "change_event"))
                                    $root.google.ads.googleads.v11.resources.ChangeEvent.encode(m.change_event, w.uint32(1162).fork()).ldelim();
                                if (m.customer_user_access != null && Object.hasOwnProperty.call(m, "customer_user_access"))
                                    $root.google.ads.googleads.v11.resources.CustomerUserAccess.encode(m.customer_user_access, w.uint32(1170).fork()).ldelim();
                                if (m.custom_audience != null && Object.hasOwnProperty.call(m, "custom_audience"))
                                    $root.google.ads.googleads.v11.resources.CustomAudience.encode(m.custom_audience, w.uint32(1178).fork()).ldelim();
                                if (m.combined_audience != null && Object.hasOwnProperty.call(m, "combined_audience"))
                                    $root.google.ads.googleads.v11.resources.CombinedAudience.encode(m.combined_audience, w.uint32(1186).fork()).ldelim();
                                if (m.feed_item_set != null && Object.hasOwnProperty.call(m, "feed_item_set"))
                                    $root.google.ads.googleads.v11.resources.FeedItemSet.encode(m.feed_item_set, w.uint32(1194).fork()).ldelim();
                                if (m.customer_user_access_invitation != null && Object.hasOwnProperty.call(m, "customer_user_access_invitation"))
                                    $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.encode(m.customer_user_access_invitation, w.uint32(1202).fork()).ldelim();
                                if (m.feed_item_set_link != null && Object.hasOwnProperty.call(m, "feed_item_set_link"))
                                    $root.google.ads.googleads.v11.resources.FeedItemSetLink.encode(m.feed_item_set_link, w.uint32(1210).fork()).ldelim();
                                if (m.call_view != null && Object.hasOwnProperty.call(m, "call_view"))
                                    $root.google.ads.googleads.v11.resources.CallView.encode(m.call_view, w.uint32(1218).fork()).ldelim();
                                if (m.conversion_custom_variable != null && Object.hasOwnProperty.call(m, "conversion_custom_variable"))
                                    $root.google.ads.googleads.v11.resources.ConversionCustomVariable.encode(m.conversion_custom_variable, w.uint32(1226).fork()).ldelim();
                                if (m.ad_group_asset != null && Object.hasOwnProperty.call(m, "ad_group_asset"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAsset.encode(m.ad_group_asset, w.uint32(1234).fork()).ldelim();
                                if (m.customer_asset != null && Object.hasOwnProperty.call(m, "customer_asset"))
                                    $root.google.ads.googleads.v11.resources.CustomerAsset.encode(m.customer_asset, w.uint32(1242).fork()).ldelim();
                                if (m.campaign_simulation != null && Object.hasOwnProperty.call(m, "campaign_simulation"))
                                    $root.google.ads.googleads.v11.resources.CampaignSimulation.encode(m.campaign_simulation, w.uint32(1258).fork()).ldelim();
                                if (m.bidding_strategy_simulation != null && Object.hasOwnProperty.call(m, "bidding_strategy_simulation"))
                                    $root.google.ads.googleads.v11.resources.BiddingStrategySimulation.encode(m.bidding_strategy_simulation, w.uint32(1266).fork()).ldelim();
                                if (m.bidding_data_exclusion != null && Object.hasOwnProperty.call(m, "bidding_data_exclusion"))
                                    $root.google.ads.googleads.v11.resources.BiddingDataExclusion.encode(m.bidding_data_exclusion, w.uint32(1274).fork()).ldelim();
                                if (m.bidding_seasonality_adjustment != null && Object.hasOwnProperty.call(m, "bidding_seasonality_adjustment"))
                                    $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.encode(m.bidding_seasonality_adjustment, w.uint32(1282).fork()).ldelim();
                                if (m.life_event != null && Object.hasOwnProperty.call(m, "life_event"))
                                    $root.google.ads.googleads.v11.resources.LifeEvent.encode(m.life_event, w.uint32(1290).fork()).ldelim();
                                if (m.webpage_view != null && Object.hasOwnProperty.call(m, "webpage_view"))
                                    $root.google.ads.googleads.v11.resources.WebpageView.encode(m.webpage_view, w.uint32(1298).fork()).ldelim();
                                if (m.keyword_theme_constant != null && Object.hasOwnProperty.call(m, "keyword_theme_constant"))
                                    $root.google.ads.googleads.v11.resources.KeywordThemeConstant.encode(m.keyword_theme_constant, w.uint32(1306).fork()).ldelim();
                                if (m.conversion_value_rule != null && Object.hasOwnProperty.call(m, "conversion_value_rule"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.encode(m.conversion_value_rule, w.uint32(1314).fork()).ldelim();
                                if (m.conversion_value_rule_set != null && Object.hasOwnProperty.call(m, "conversion_value_rule_set"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.encode(m.conversion_value_rule_set, w.uint32(1322).fork()).ldelim();
                                if (m.detailed_demographic != null && Object.hasOwnProperty.call(m, "detailed_demographic"))
                                    $root.google.ads.googleads.v11.resources.DetailedDemographic.encode(m.detailed_demographic, w.uint32(1330).fork()).ldelim();
                                if (m.smart_campaign_setting != null && Object.hasOwnProperty.call(m, "smart_campaign_setting"))
                                    $root.google.ads.googleads.v11.resources.SmartCampaignSetting.encode(m.smart_campaign_setting, w.uint32(1338).fork()).ldelim();
                                if (m.asset_field_type_view != null && Object.hasOwnProperty.call(m, "asset_field_type_view"))
                                    $root.google.ads.googleads.v11.resources.AssetFieldTypeView.encode(m.asset_field_type_view, w.uint32(1346).fork()).ldelim();
                                if (m.accessible_bidding_strategy != null && Object.hasOwnProperty.call(m, "accessible_bidding_strategy"))
                                    $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.encode(m.accessible_bidding_strategy, w.uint32(1354).fork()).ldelim();
                                if (m.smart_campaign_search_term_view != null && Object.hasOwnProperty.call(m, "smart_campaign_search_term_view"))
                                    $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView.encode(m.smart_campaign_search_term_view, w.uint32(1362).fork()).ldelim();
                                if (m.asset_group != null && Object.hasOwnProperty.call(m, "asset_group"))
                                    $root.google.ads.googleads.v11.resources.AssetGroup.encode(m.asset_group, w.uint32(1378).fork()).ldelim();
                                if (m.asset_group_asset != null && Object.hasOwnProperty.call(m, "asset_group_asset"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupAsset.encode(m.asset_group_asset, w.uint32(1386).fork()).ldelim();
                                if (m.customer_conversion_goal != null && Object.hasOwnProperty.call(m, "customer_conversion_goal"))
                                    $root.google.ads.googleads.v11.resources.CustomerConversionGoal.encode(m.customer_conversion_goal, w.uint32(1394).fork()).ldelim();
                                if (m.campaign_conversion_goal != null && Object.hasOwnProperty.call(m, "campaign_conversion_goal"))
                                    $root.google.ads.googleads.v11.resources.CampaignConversionGoal.encode(m.campaign_conversion_goal, w.uint32(1402).fork()).ldelim();
                                if (m.custom_conversion_goal != null && Object.hasOwnProperty.call(m, "custom_conversion_goal"))
                                    $root.google.ads.googleads.v11.resources.CustomConversionGoal.encode(m.custom_conversion_goal, w.uint32(1410).fork()).ldelim();
                                if (m.conversion_goal_campaign_config != null && Object.hasOwnProperty.call(m, "conversion_goal_campaign_config"))
                                    $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.encode(m.conversion_goal_campaign_config, w.uint32(1418).fork()).ldelim();
                                if (m.customizer_attribute != null && Object.hasOwnProperty.call(m, "customizer_attribute"))
                                    $root.google.ads.googleads.v11.resources.CustomizerAttribute.encode(m.customizer_attribute, w.uint32(1426).fork()).ldelim();
                                if (m.asset_set != null && Object.hasOwnProperty.call(m, "asset_set"))
                                    $root.google.ads.googleads.v11.resources.AssetSet.encode(m.asset_set, w.uint32(1434).fork()).ldelim();
                                if (m.asset_set_asset != null && Object.hasOwnProperty.call(m, "asset_set_asset"))
                                    $root.google.ads.googleads.v11.resources.AssetSetAsset.encode(m.asset_set_asset, w.uint32(1442).fork()).ldelim();
                                if (m.campaign_asset_set != null && Object.hasOwnProperty.call(m, "campaign_asset_set"))
                                    $root.google.ads.googleads.v11.resources.CampaignAssetSet.encode(m.campaign_asset_set, w.uint32(1450).fork()).ldelim();
                                if (m.asset_group_listing_group_filter != null && Object.hasOwnProperty.call(m, "asset_group_listing_group_filter"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.encode(m.asset_group_listing_group_filter, w.uint32(1458).fork()).ldelim();
                                if (m.experiment_arm != null && Object.hasOwnProperty.call(m, "experiment_arm"))
                                    $root.google.ads.googleads.v11.resources.ExperimentArm.encode(m.experiment_arm, w.uint32(1466).fork()).ldelim();
                                if (m.customer_customizer != null && Object.hasOwnProperty.call(m, "customer_customizer"))
                                    $root.google.ads.googleads.v11.resources.CustomerCustomizer.encode(m.customer_customizer, w.uint32(1474).fork()).ldelim();
                                if (m.ad_group_customizer != null && Object.hasOwnProperty.call(m, "ad_group_customizer"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCustomizer.encode(m.ad_group_customizer, w.uint32(1482).fork()).ldelim();
                                if (m.campaign_customizer != null && Object.hasOwnProperty.call(m, "campaign_customizer"))
                                    $root.google.ads.googleads.v11.resources.CampaignCustomizer.encode(m.campaign_customizer, w.uint32(1490).fork()).ldelim();
                                if (m.ad_group_criterion_customizer != null && Object.hasOwnProperty.call(m, "ad_group_criterion_customizer"))
                                    $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.encode(m.ad_group_criterion_customizer, w.uint32(1498).fork()).ldelim();
                                if (m.hotel_reconciliation != null && Object.hasOwnProperty.call(m, "hotel_reconciliation"))
                                    $root.google.ads.googleads.v11.resources.HotelReconciliation.encode(m.hotel_reconciliation, w.uint32(1506).fork()).ldelim();
                                if (m.asset_group_product_group_view != null && Object.hasOwnProperty.call(m, "asset_group_product_group_view"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView.encode(m.asset_group_product_group_view, w.uint32(1514).fork()).ldelim();
                                if (m.audience != null && Object.hasOwnProperty.call(m, "audience"))
                                    $root.google.ads.googleads.v11.resources.Audience.encode(m.audience, w.uint32(1522).fork()).ldelim();
                                if (m.asset_group_signal != null && Object.hasOwnProperty.call(m, "asset_group_signal"))
                                    $root.google.ads.googleads.v11.resources.AssetGroupSignal.encode(m.asset_group_signal, w.uint32(1530).fork()).ldelim();
                                if (m.lead_form_submission_data != null && Object.hasOwnProperty.call(m, "lead_form_submission_data"))
                                    $root.google.ads.googleads.v11.resources.LeadFormSubmissionData.encode(m.lead_form_submission_data, w.uint32(1538).fork()).ldelim();
                                if (m.ad_group_ad_asset_combination_view != null && Object.hasOwnProperty.call(m, "ad_group_ad_asset_combination_view"))
                                    $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.encode(m.ad_group_ad_asset_combination_view, w.uint32(1546).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GoogleAdsRow message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GoogleAdsRow.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {google.ads.googleads.v11.services.IGoogleAdsRow} message GoogleAdsRow message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GoogleAdsRow.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GoogleAdsRow message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GoogleAdsRow} GoogleAdsRow
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsRow.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GoogleAdsRow();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 42:
                                        m.account_budget = $root.google.ads.googleads.v11.resources.AccountBudget.decode(r, r.uint32());
                                        break;
                                    case 43:
                                        m.account_budget_proposal = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.decode(r, r.uint32());
                                        break;
                                    case 143:
                                        m.account_link = $root.google.ads.googleads.v11.resources.AccountLink.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.decode(r, r.uint32());
                                        break;
                                    case 193:
                                        m.ad_group_ad_asset_combination_view = $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.decode(r, r.uint32());
                                        break;
                                    case 131:
                                        m.ad_group_ad_asset_view = $root.google.ads.googleads.v11.resources.AdGroupAdAssetView.decode(r, r.uint32());
                                        break;
                                    case 120:
                                        m.ad_group_ad_label = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.decode(r, r.uint32());
                                        break;
                                    case 154:
                                        m.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.decode(r, r.uint32());
                                        break;
                                    case 57:
                                        m.ad_group_audience_view = $root.google.ads.googleads.v11.resources.AdGroupAudienceView.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.decode(r, r.uint32());
                                        break;
                                    case 17:
                                        m.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.decode(r, r.uint32());
                                        break;
                                    case 187:
                                        m.ad_group_criterion_customizer = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.decode(r, r.uint32());
                                        break;
                                    case 121:
                                        m.ad_group_criterion_label = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.decode(r, r.uint32());
                                        break;
                                    case 110:
                                        m.ad_group_criterion_simulation = $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation.decode(r, r.uint32());
                                        break;
                                    case 185:
                                        m.ad_group_customizer = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.decode(r, r.uint32());
                                        break;
                                    case 112:
                                        m.ad_group_extension_setting = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 67:
                                        m.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.decode(r, r.uint32());
                                        break;
                                    case 115:
                                        m.ad_group_label = $root.google.ads.googleads.v11.resources.AdGroupLabel.decode(r, r.uint32());
                                        break;
                                    case 107:
                                        m.ad_group_simulation = $root.google.ads.googleads.v11.resources.AdGroupSimulation.decode(r, r.uint32());
                                        break;
                                    case 130:
                                        m.ad_parameter = $root.google.ads.googleads.v11.resources.AdParameter.decode(r, r.uint32());
                                        break;
                                    case 48:
                                        m.age_range_view = $root.google.ads.googleads.v11.resources.AgeRangeView.decode(r, r.uint32());
                                        break;
                                    case 89:
                                        m.ad_schedule_view = $root.google.ads.googleads.v11.resources.AdScheduleView.decode(r, r.uint32());
                                        break;
                                    case 91:
                                        m.domain_category = $root.google.ads.googleads.v11.resources.DomainCategory.decode(r, r.uint32());
                                        break;
                                    case 105:
                                        m.asset = $root.google.ads.googleads.v11.resources.Asset.decode(r, r.uint32());
                                        break;
                                    case 168:
                                        m.asset_field_type_view = $root.google.ads.googleads.v11.resources.AssetFieldTypeView.decode(r, r.uint32());
                                        break;
                                    case 173:
                                        m.asset_group_asset = $root.google.ads.googleads.v11.resources.AssetGroupAsset.decode(r, r.uint32());
                                        break;
                                    case 191:
                                        m.asset_group_signal = $root.google.ads.googleads.v11.resources.AssetGroupSignal.decode(r, r.uint32());
                                        break;
                                    case 182:
                                        m.asset_group_listing_group_filter = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.decode(r, r.uint32());
                                        break;
                                    case 189:
                                        m.asset_group_product_group_view = $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView.decode(r, r.uint32());
                                        break;
                                    case 172:
                                        m.asset_group = $root.google.ads.googleads.v11.resources.AssetGroup.decode(r, r.uint32());
                                        break;
                                    case 180:
                                        m.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.decode(r, r.uint32());
                                        break;
                                    case 179:
                                        m.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.decode(r, r.uint32());
                                        break;
                                    case 139:
                                        m.batch_job = $root.google.ads.googleads.v11.resources.BatchJob.decode(r, r.uint32());
                                        break;
                                    case 159:
                                        m.bidding_data_exclusion = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.decode(r, r.uint32());
                                        break;
                                    case 160:
                                        m.bidding_seasonality_adjustment = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.bidding_strategy = $root.google.ads.googleads.v11.resources.BiddingStrategy.decode(r, r.uint32());
                                        break;
                                    case 158:
                                        m.bidding_strategy_simulation = $root.google.ads.googleads.v11.resources.BiddingStrategySimulation.decode(r, r.uint32());
                                        break;
                                    case 41:
                                        m.billing_setup = $root.google.ads.googleads.v11.resources.BillingSetup.decode(r, r.uint32());
                                        break;
                                    case 152:
                                        m.call_view = $root.google.ads.googleads.v11.resources.CallView.decode(r, r.uint32());
                                        break;
                                    case 19:
                                        m.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.campaign = $root.google.ads.googleads.v11.resources.Campaign.decode(r, r.uint32());
                                        break;
                                    case 142:
                                        m.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.decode(r, r.uint32());
                                        break;
                                    case 181:
                                        m.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.decode(r, r.uint32());
                                        break;
                                    case 69:
                                        m.campaign_audience_view = $root.google.ads.googleads.v11.resources.CampaignAudienceView.decode(r, r.uint32());
                                        break;
                                    case 26:
                                        m.campaign_bid_modifier = $root.google.ads.googleads.v11.resources.CampaignBidModifier.decode(r, r.uint32());
                                        break;
                                    case 175:
                                        m.campaign_conversion_goal = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.decode(r, r.uint32());
                                        break;
                                    case 111:
                                        m.campaign_criterion_simulation = $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation.decode(r, r.uint32());
                                        break;
                                    case 186:
                                        m.campaign_customizer = $root.google.ads.googleads.v11.resources.CampaignCustomizer.decode(r, r.uint32());
                                        break;
                                    case 49:
                                        m.campaign_draft = $root.google.ads.googleads.v11.resources.CampaignDraft.decode(r, r.uint32());
                                        break;
                                    case 84:
                                        m.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.decode(r, r.uint32());
                                        break;
                                    case 113:
                                        m.campaign_extension_setting = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 63:
                                        m.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.decode(r, r.uint32());
                                        break;
                                    case 25:
                                        m.campaign_group = $root.google.ads.googleads.v11.resources.CampaignGroup.decode(r, r.uint32());
                                        break;
                                    case 108:
                                        m.campaign_label = $root.google.ads.googleads.v11.resources.CampaignLabel.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.campaign_shared_set = $root.google.ads.googleads.v11.resources.CampaignSharedSet.decode(r, r.uint32());
                                        break;
                                    case 157:
                                        m.campaign_simulation = $root.google.ads.googleads.v11.resources.CampaignSimulation.decode(r, r.uint32());
                                        break;
                                    case 66:
                                        m.carrier_constant = $root.google.ads.googleads.v11.resources.CarrierConstant.decode(r, r.uint32());
                                        break;
                                    case 145:
                                        m.change_event = $root.google.ads.googleads.v11.resources.ChangeEvent.decode(r, r.uint32());
                                        break;
                                    case 37:
                                        m.change_status = $root.google.ads.googleads.v11.resources.ChangeStatus.decode(r, r.uint32());
                                        break;
                                    case 148:
                                        m.combined_audience = $root.google.ads.googleads.v11.resources.CombinedAudience.decode(r, r.uint32());
                                        break;
                                    case 190:
                                        m.audience = $root.google.ads.googleads.v11.resources.Audience.decode(r, r.uint32());
                                        break;
                                    case 103:
                                        m.conversion_action = $root.google.ads.googleads.v11.resources.ConversionAction.decode(r, r.uint32());
                                        break;
                                    case 153:
                                        m.conversion_custom_variable = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.decode(r, r.uint32());
                                        break;
                                    case 177:
                                        m.conversion_goal_campaign_config = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.decode(r, r.uint32());
                                        break;
                                    case 164:
                                        m.conversion_value_rule = $root.google.ads.googleads.v11.resources.ConversionValueRule.decode(r, r.uint32());
                                        break;
                                    case 165:
                                        m.conversion_value_rule_set = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.decode(r, r.uint32());
                                        break;
                                    case 122:
                                        m.click_view = $root.google.ads.googleads.v11.resources.ClickView.decode(r, r.uint32());
                                        break;
                                    case 134:
                                        m.currency_constant = $root.google.ads.googleads.v11.resources.CurrencyConstant.decode(r, r.uint32());
                                        break;
                                    case 147:
                                        m.custom_audience = $root.google.ads.googleads.v11.resources.CustomAudience.decode(r, r.uint32());
                                        break;
                                    case 176:
                                        m.custom_conversion_goal = $root.google.ads.googleads.v11.resources.CustomConversionGoal.decode(r, r.uint32());
                                        break;
                                    case 104:
                                        m.custom_interest = $root.google.ads.googleads.v11.resources.CustomInterest.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.customer = $root.google.ads.googleads.v11.resources.Customer.decode(r, r.uint32());
                                        break;
                                    case 155:
                                        m.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.decode(r, r.uint32());
                                        break;
                                    case 169:
                                        m.accessible_bidding_strategy = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.decode(r, r.uint32());
                                        break;
                                    case 184:
                                        m.customer_customizer = $root.google.ads.googleads.v11.resources.CustomerCustomizer.decode(r, r.uint32());
                                        break;
                                    case 61:
                                        m.customer_manager_link = $root.google.ads.googleads.v11.resources.CustomerManagerLink.decode(r, r.uint32());
                                        break;
                                    case 62:
                                        m.customer_client_link = $root.google.ads.googleads.v11.resources.CustomerClientLink.decode(r, r.uint32());
                                        break;
                                    case 70:
                                        m.customer_client = $root.google.ads.googleads.v11.resources.CustomerClient.decode(r, r.uint32());
                                        break;
                                    case 174:
                                        m.customer_conversion_goal = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.decode(r, r.uint32());
                                        break;
                                    case 114:
                                        m.customer_extension_setting = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 64:
                                        m.customer_feed = $root.google.ads.googleads.v11.resources.CustomerFeed.decode(r, r.uint32());
                                        break;
                                    case 124:
                                        m.customer_label = $root.google.ads.googleads.v11.resources.CustomerLabel.decode(r, r.uint32());
                                        break;
                                    case 88:
                                        m.customer_negative_criterion = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.decode(r, r.uint32());
                                        break;
                                    case 146:
                                        m.customer_user_access = $root.google.ads.googleads.v11.resources.CustomerUserAccess.decode(r, r.uint32());
                                        break;
                                    case 150:
                                        m.customer_user_access_invitation = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.decode(r, r.uint32());
                                        break;
                                    case 178:
                                        m.customizer_attribute = $root.google.ads.googleads.v11.resources.CustomizerAttribute.decode(r, r.uint32());
                                        break;
                                    case 118:
                                        m.detail_placement_view = $root.google.ads.googleads.v11.resources.DetailPlacementView.decode(r, r.uint32());
                                        break;
                                    case 166:
                                        m.detailed_demographic = $root.google.ads.googleads.v11.resources.DetailedDemographic.decode(r, r.uint32());
                                        break;
                                    case 47:
                                        m.display_keyword_view = $root.google.ads.googleads.v11.resources.DisplayKeywordView.decode(r, r.uint32());
                                        break;
                                    case 132:
                                        m.distance_view = $root.google.ads.googleads.v11.resources.DistanceView.decode(r, r.uint32());
                                        break;
                                    case 106:
                                        m.dynamic_search_ads_search_term_view = $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView.decode(r, r.uint32());
                                        break;
                                    case 128:
                                        m.expanded_landing_page_view = $root.google.ads.googleads.v11.resources.ExpandedLandingPageView.decode(r, r.uint32());
                                        break;
                                    case 85:
                                        m.extension_feed_item = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.decode(r, r.uint32());
                                        break;
                                    case 46:
                                        m.feed = $root.google.ads.googleads.v11.resources.Feed.decode(r, r.uint32());
                                        break;
                                    case 50:
                                        m.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.decode(r, r.uint32());
                                        break;
                                    case 149:
                                        m.feed_item_set = $root.google.ads.googleads.v11.resources.FeedItemSet.decode(r, r.uint32());
                                        break;
                                    case 151:
                                        m.feed_item_set_link = $root.google.ads.googleads.v11.resources.FeedItemSetLink.decode(r, r.uint32());
                                        break;
                                    case 116:
                                        m.feed_item_target = $root.google.ads.googleads.v11.resources.FeedItemTarget.decode(r, r.uint32());
                                        break;
                                    case 58:
                                        m.feed_mapping = $root.google.ads.googleads.v11.resources.FeedMapping.decode(r, r.uint32());
                                        break;
                                    case 97:
                                        m.feed_placeholder_view = $root.google.ads.googleads.v11.resources.FeedPlaceholderView.decode(r, r.uint32());
                                        break;
                                    case 40:
                                        m.gender_view = $root.google.ads.googleads.v11.resources.GenderView.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.geo_target_constant = $root.google.ads.googleads.v11.resources.GeoTargetConstant.decode(r, r.uint32());
                                        break;
                                    case 125:
                                        m.geographic_view = $root.google.ads.googleads.v11.resources.GeographicView.decode(r, r.uint32());
                                        break;
                                    case 119:
                                        m.group_placement_view = $root.google.ads.googleads.v11.resources.GroupPlacementView.decode(r, r.uint32());
                                        break;
                                    case 51:
                                        m.hotel_group_view = $root.google.ads.googleads.v11.resources.HotelGroupView.decode(r, r.uint32());
                                        break;
                                    case 71:
                                        m.hotel_performance_view = $root.google.ads.googleads.v11.resources.HotelPerformanceView.decode(r, r.uint32());
                                        break;
                                    case 188:
                                        m.hotel_reconciliation = $root.google.ads.googleads.v11.resources.HotelReconciliation.decode(r, r.uint32());
                                        break;
                                    case 138:
                                        m.income_range_view = $root.google.ads.googleads.v11.resources.IncomeRangeView.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.keyword_view = $root.google.ads.googleads.v11.resources.KeywordView.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.keyword_plan = $root.google.ads.googleads.v11.resources.KeywordPlan.decode(r, r.uint32());
                                        break;
                                    case 33:
                                        m.keyword_plan_campaign = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.decode(r, r.uint32());
                                        break;
                                    case 140:
                                        m.keyword_plan_campaign_keyword = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.decode(r, r.uint32());
                                        break;
                                    case 35:
                                        m.keyword_plan_ad_group = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.decode(r, r.uint32());
                                        break;
                                    case 141:
                                        m.keyword_plan_ad_group_keyword = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.decode(r, r.uint32());
                                        break;
                                    case 163:
                                        m.keyword_theme_constant = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.decode(r, r.uint32());
                                        break;
                                    case 52:
                                        m.label = $root.google.ads.googleads.v11.resources.Label.decode(r, r.uint32());
                                        break;
                                    case 126:
                                        m.landing_page_view = $root.google.ads.googleads.v11.resources.LandingPageView.decode(r, r.uint32());
                                        break;
                                    case 55:
                                        m.language_constant = $root.google.ads.googleads.v11.resources.LanguageConstant.decode(r, r.uint32());
                                        break;
                                    case 123:
                                        m.location_view = $root.google.ads.googleads.v11.resources.LocationView.decode(r, r.uint32());
                                        break;
                                    case 53:
                                        m.managed_placement_view = $root.google.ads.googleads.v11.resources.ManagedPlacementView.decode(r, r.uint32());
                                        break;
                                    case 90:
                                        m.media_file = $root.google.ads.googleads.v11.resources.MediaFile.decode(r, r.uint32());
                                        break;
                                    case 87:
                                        m.mobile_app_category_constant = $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant.decode(r, r.uint32());
                                        break;
                                    case 98:
                                        m.mobile_device_constant = $root.google.ads.googleads.v11.resources.MobileDeviceConstant.decode(r, r.uint32());
                                        break;
                                    case 137:
                                        m.offline_user_data_job = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.decode(r, r.uint32());
                                        break;
                                    case 86:
                                        m.operating_system_version_constant = $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant.decode(r, r.uint32());
                                        break;
                                    case 129:
                                        m.paid_organic_search_term_view = $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView.decode(r, r.uint32());
                                        break;
                                    case 45:
                                        m.parental_status_view = $root.google.ads.googleads.v11.resources.ParentalStatusView.decode(r, r.uint32());
                                        break;
                                    case 109:
                                        m.product_bidding_category_constant = $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant.decode(r, r.uint32());
                                        break;
                                    case 54:
                                        m.product_group_view = $root.google.ads.googleads.v11.resources.ProductGroupView.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.recommendation = $root.google.ads.googleads.v11.resources.Recommendation.decode(r, r.uint32());
                                        break;
                                    case 68:
                                        m.search_term_view = $root.google.ads.googleads.v11.resources.SearchTermView.decode(r, r.uint32());
                                        break;
                                    case 29:
                                        m.shared_criterion = $root.google.ads.googleads.v11.resources.SharedCriterion.decode(r, r.uint32());
                                        break;
                                    case 27:
                                        m.shared_set = $root.google.ads.googleads.v11.resources.SharedSet.decode(r, r.uint32());
                                        break;
                                    case 167:
                                        m.smart_campaign_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.decode(r, r.uint32());
                                        break;
                                    case 117:
                                        m.shopping_performance_view = $root.google.ads.googleads.v11.resources.ShoppingPerformanceView.decode(r, r.uint32());
                                        break;
                                    case 170:
                                        m.smart_campaign_search_term_view = $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView.decode(r, r.uint32());
                                        break;
                                    case 144:
                                        m.third_party_app_analytics_link = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink.decode(r, r.uint32());
                                        break;
                                    case 44:
                                        m.topic_view = $root.google.ads.googleads.v11.resources.TopicView.decode(r, r.uint32());
                                        break;
                                    case 133:
                                        m.experiment = $root.google.ads.googleads.v11.resources.Experiment.decode(r, r.uint32());
                                        break;
                                    case 183:
                                        m.experiment_arm = $root.google.ads.googleads.v11.resources.ExperimentArm.decode(r, r.uint32());
                                        break;
                                    case 59:
                                        m.user_interest = $root.google.ads.googleads.v11.resources.UserInterest.decode(r, r.uint32());
                                        break;
                                    case 161:
                                        m.life_event = $root.google.ads.googleads.v11.resources.LifeEvent.decode(r, r.uint32());
                                        break;
                                    case 38:
                                        m.user_list = $root.google.ads.googleads.v11.resources.UserList.decode(r, r.uint32());
                                        break;
                                    case 135:
                                        m.user_location_view = $root.google.ads.googleads.v11.resources.UserLocationView.decode(r, r.uint32());
                                        break;
                                    case 60:
                                        m.remarketing_action = $root.google.ads.googleads.v11.resources.RemarketingAction.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.topic_constant = $root.google.ads.googleads.v11.resources.TopicConstant.decode(r, r.uint32());
                                        break;
                                    case 39:
                                        m.video = $root.google.ads.googleads.v11.resources.Video.decode(r, r.uint32());
                                        break;
                                    case 162:
                                        m.webpage_view = $root.google.ads.googleads.v11.resources.WebpageView.decode(r, r.uint32());
                                        break;
                                    case 192:
                                        m.lead_form_submission_data = $root.google.ads.googleads.v11.resources.LeadFormSubmissionData.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.metrics = $root.google.ads.googleads.v11.common.Metrics.decode(r, r.uint32());
                                        break;
                                    case 102:
                                        m.segments = $root.google.ads.googleads.v11.common.Segments.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GoogleAdsRow message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GoogleAdsRow} GoogleAdsRow
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GoogleAdsRow.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GoogleAdsRow message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GoogleAdsRow.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.account_budget != null && m.hasOwnProperty("account_budget")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccountBudget.verify(m.account_budget);
                                        if (e)
                                            return "account_budget." + e;
                                    }
                                }
                                if (m.account_budget_proposal != null && m.hasOwnProperty("account_budget_proposal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.verify(m.account_budget_proposal);
                                        if (e)
                                            return "account_budget_proposal." + e;
                                    }
                                }
                                if (m.account_link != null && m.hasOwnProperty("account_link")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccountLink.verify(m.account_link);
                                        if (e)
                                            return "account_link." + e;
                                    }
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroup.verify(m.ad_group);
                                        if (e)
                                            return "ad_group." + e;
                                    }
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAd.verify(m.ad_group_ad);
                                        if (e)
                                            return "ad_group_ad." + e;
                                    }
                                }
                                if (m.ad_group_ad_asset_combination_view != null && m.hasOwnProperty("ad_group_ad_asset_combination_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.verify(m.ad_group_ad_asset_combination_view);
                                        if (e)
                                            return "ad_group_ad_asset_combination_view." + e;
                                    }
                                }
                                if (m.ad_group_ad_asset_view != null && m.hasOwnProperty("ad_group_ad_asset_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAdAssetView.verify(m.ad_group_ad_asset_view);
                                        if (e)
                                            return "ad_group_ad_asset_view." + e;
                                    }
                                }
                                if (m.ad_group_ad_label != null && m.hasOwnProperty("ad_group_ad_label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.verify(m.ad_group_ad_label);
                                        if (e)
                                            return "ad_group_ad_label." + e;
                                    }
                                }
                                if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAsset.verify(m.ad_group_asset);
                                        if (e)
                                            return "ad_group_asset." + e;
                                    }
                                }
                                if (m.ad_group_audience_view != null && m.hasOwnProperty("ad_group_audience_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupAudienceView.verify(m.ad_group_audience_view);
                                        if (e)
                                            return "ad_group_audience_view." + e;
                                    }
                                }
                                if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.verify(m.ad_group_bid_modifier);
                                        if (e)
                                            return "ad_group_bid_modifier." + e;
                                    }
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterion.verify(m.ad_group_criterion);
                                        if (e)
                                            return "ad_group_criterion." + e;
                                    }
                                }
                                if (m.ad_group_criterion_customizer != null && m.hasOwnProperty("ad_group_criterion_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.verify(m.ad_group_criterion_customizer);
                                        if (e)
                                            return "ad_group_criterion_customizer." + e;
                                    }
                                }
                                if (m.ad_group_criterion_label != null && m.hasOwnProperty("ad_group_criterion_label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.verify(m.ad_group_criterion_label);
                                        if (e)
                                            return "ad_group_criterion_label." + e;
                                    }
                                }
                                if (m.ad_group_criterion_simulation != null && m.hasOwnProperty("ad_group_criterion_simulation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation.verify(m.ad_group_criterion_simulation);
                                        if (e)
                                            return "ad_group_criterion_simulation." + e;
                                    }
                                }
                                if (m.ad_group_customizer != null && m.hasOwnProperty("ad_group_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.verify(m.ad_group_customizer);
                                        if (e)
                                            return "ad_group_customizer." + e;
                                    }
                                }
                                if (m.ad_group_extension_setting != null && m.hasOwnProperty("ad_group_extension_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.verify(m.ad_group_extension_setting);
                                        if (e)
                                            return "ad_group_extension_setting." + e;
                                    }
                                }
                                if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupFeed.verify(m.ad_group_feed);
                                        if (e)
                                            return "ad_group_feed." + e;
                                    }
                                }
                                if (m.ad_group_label != null && m.hasOwnProperty("ad_group_label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupLabel.verify(m.ad_group_label);
                                        if (e)
                                            return "ad_group_label." + e;
                                    }
                                }
                                if (m.ad_group_simulation != null && m.hasOwnProperty("ad_group_simulation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdGroupSimulation.verify(m.ad_group_simulation);
                                        if (e)
                                            return "ad_group_simulation." + e;
                                    }
                                }
                                if (m.ad_parameter != null && m.hasOwnProperty("ad_parameter")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdParameter.verify(m.ad_parameter);
                                        if (e)
                                            return "ad_parameter." + e;
                                    }
                                }
                                if (m.age_range_view != null && m.hasOwnProperty("age_range_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AgeRangeView.verify(m.age_range_view);
                                        if (e)
                                            return "age_range_view." + e;
                                    }
                                }
                                if (m.ad_schedule_view != null && m.hasOwnProperty("ad_schedule_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AdScheduleView.verify(m.ad_schedule_view);
                                        if (e)
                                            return "ad_schedule_view." + e;
                                    }
                                }
                                if (m.domain_category != null && m.hasOwnProperty("domain_category")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.DomainCategory.verify(m.domain_category);
                                        if (e)
                                            return "domain_category." + e;
                                    }
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Asset.verify(m.asset);
                                        if (e)
                                            return "asset." + e;
                                    }
                                }
                                if (m.asset_field_type_view != null && m.hasOwnProperty("asset_field_type_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetFieldTypeView.verify(m.asset_field_type_view);
                                        if (e)
                                            return "asset_field_type_view." + e;
                                    }
                                }
                                if (m.asset_group_asset != null && m.hasOwnProperty("asset_group_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupAsset.verify(m.asset_group_asset);
                                        if (e)
                                            return "asset_group_asset." + e;
                                    }
                                }
                                if (m.asset_group_signal != null && m.hasOwnProperty("asset_group_signal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupSignal.verify(m.asset_group_signal);
                                        if (e)
                                            return "asset_group_signal." + e;
                                    }
                                }
                                if (m.asset_group_listing_group_filter != null && m.hasOwnProperty("asset_group_listing_group_filter")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.verify(m.asset_group_listing_group_filter);
                                        if (e)
                                            return "asset_group_listing_group_filter." + e;
                                    }
                                }
                                if (m.asset_group_product_group_view != null && m.hasOwnProperty("asset_group_product_group_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView.verify(m.asset_group_product_group_view);
                                        if (e)
                                            return "asset_group_product_group_view." + e;
                                    }
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetGroup.verify(m.asset_group);
                                        if (e)
                                            return "asset_group." + e;
                                    }
                                }
                                if (m.asset_set_asset != null && m.hasOwnProperty("asset_set_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSetAsset.verify(m.asset_set_asset);
                                        if (e)
                                            return "asset_set_asset." + e;
                                    }
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AssetSet.verify(m.asset_set);
                                        if (e)
                                            return "asset_set." + e;
                                    }
                                }
                                if (m.batch_job != null && m.hasOwnProperty("batch_job")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BatchJob.verify(m.batch_job);
                                        if (e)
                                            return "batch_job." + e;
                                    }
                                }
                                if (m.bidding_data_exclusion != null && m.hasOwnProperty("bidding_data_exclusion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.verify(m.bidding_data_exclusion);
                                        if (e)
                                            return "bidding_data_exclusion." + e;
                                    }
                                }
                                if (m.bidding_seasonality_adjustment != null && m.hasOwnProperty("bidding_seasonality_adjustment")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.verify(m.bidding_seasonality_adjustment);
                                        if (e)
                                            return "bidding_seasonality_adjustment." + e;
                                    }
                                }
                                if (m.bidding_strategy != null && m.hasOwnProperty("bidding_strategy")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingStrategy.verify(m.bidding_strategy);
                                        if (e)
                                            return "bidding_strategy." + e;
                                    }
                                }
                                if (m.bidding_strategy_simulation != null && m.hasOwnProperty("bidding_strategy_simulation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingStrategySimulation.verify(m.bidding_strategy_simulation);
                                        if (e)
                                            return "bidding_strategy_simulation." + e;
                                    }
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BillingSetup.verify(m.billing_setup);
                                        if (e)
                                            return "billing_setup." + e;
                                    }
                                }
                                if (m.call_view != null && m.hasOwnProperty("call_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CallView.verify(m.call_view);
                                        if (e)
                                            return "call_view." + e;
                                    }
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBudget.verify(m.campaign_budget);
                                        if (e)
                                            return "campaign_budget." + e;
                                    }
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.verify(m.campaign);
                                        if (e)
                                            return "campaign." + e;
                                    }
                                }
                                if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAsset.verify(m.campaign_asset);
                                        if (e)
                                            return "campaign_asset." + e;
                                    }
                                }
                                if (m.campaign_asset_set != null && m.hasOwnProperty("campaign_asset_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAssetSet.verify(m.campaign_asset_set);
                                        if (e)
                                            return "campaign_asset_set." + e;
                                    }
                                }
                                if (m.campaign_audience_view != null && m.hasOwnProperty("campaign_audience_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAudienceView.verify(m.campaign_audience_view);
                                        if (e)
                                            return "campaign_audience_view." + e;
                                    }
                                }
                                if (m.campaign_bid_modifier != null && m.hasOwnProperty("campaign_bid_modifier")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBidModifier.verify(m.campaign_bid_modifier);
                                        if (e)
                                            return "campaign_bid_modifier." + e;
                                    }
                                }
                                if (m.campaign_conversion_goal != null && m.hasOwnProperty("campaign_conversion_goal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.verify(m.campaign_conversion_goal);
                                        if (e)
                                            return "campaign_conversion_goal." + e;
                                    }
                                }
                                if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCriterion.verify(m.campaign_criterion);
                                        if (e)
                                            return "campaign_criterion." + e;
                                    }
                                }
                                if (m.campaign_criterion_simulation != null && m.hasOwnProperty("campaign_criterion_simulation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation.verify(m.campaign_criterion_simulation);
                                        if (e)
                                            return "campaign_criterion_simulation." + e;
                                    }
                                }
                                if (m.campaign_customizer != null && m.hasOwnProperty("campaign_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCustomizer.verify(m.campaign_customizer);
                                        if (e)
                                            return "campaign_customizer." + e;
                                    }
                                }
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignDraft.verify(m.campaign_draft);
                                        if (e)
                                            return "campaign_draft." + e;
                                    }
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExperiment.verify(m.campaign_experiment);
                                        if (e)
                                            return "campaign_experiment." + e;
                                    }
                                }
                                if (m.campaign_extension_setting != null && m.hasOwnProperty("campaign_extension_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.verify(m.campaign_extension_setting);
                                        if (e)
                                            return "campaign_extension_setting." + e;
                                    }
                                }
                                if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignFeed.verify(m.campaign_feed);
                                        if (e)
                                            return "campaign_feed." + e;
                                    }
                                }
                                if (m.campaign_group != null && m.hasOwnProperty("campaign_group")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignGroup.verify(m.campaign_group);
                                        if (e)
                                            return "campaign_group." + e;
                                    }
                                }
                                if (m.campaign_label != null && m.hasOwnProperty("campaign_label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignLabel.verify(m.campaign_label);
                                        if (e)
                                            return "campaign_label." + e;
                                    }
                                }
                                if (m.campaign_shared_set != null && m.hasOwnProperty("campaign_shared_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignSharedSet.verify(m.campaign_shared_set);
                                        if (e)
                                            return "campaign_shared_set." + e;
                                    }
                                }
                                if (m.campaign_simulation != null && m.hasOwnProperty("campaign_simulation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignSimulation.verify(m.campaign_simulation);
                                        if (e)
                                            return "campaign_simulation." + e;
                                    }
                                }
                                if (m.carrier_constant != null && m.hasOwnProperty("carrier_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CarrierConstant.verify(m.carrier_constant);
                                        if (e)
                                            return "carrier_constant." + e;
                                    }
                                }
                                if (m.change_event != null && m.hasOwnProperty("change_event")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ChangeEvent.verify(m.change_event);
                                        if (e)
                                            return "change_event." + e;
                                    }
                                }
                                if (m.change_status != null && m.hasOwnProperty("change_status")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ChangeStatus.verify(m.change_status);
                                        if (e)
                                            return "change_status." + e;
                                    }
                                }
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CombinedAudience.verify(m.combined_audience);
                                        if (e)
                                            return "combined_audience." + e;
                                    }
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Audience.verify(m.audience);
                                        if (e)
                                            return "audience." + e;
                                    }
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.verify(m.conversion_action);
                                        if (e)
                                            return "conversion_action." + e;
                                    }
                                }
                                if (m.conversion_custom_variable != null && m.hasOwnProperty("conversion_custom_variable")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.verify(m.conversion_custom_variable);
                                        if (e)
                                            return "conversion_custom_variable." + e;
                                    }
                                }
                                if (m.conversion_goal_campaign_config != null && m.hasOwnProperty("conversion_goal_campaign_config")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.verify(m.conversion_goal_campaign_config);
                                        if (e)
                                            return "conversion_goal_campaign_config." + e;
                                    }
                                }
                                if (m.conversion_value_rule != null && m.hasOwnProperty("conversion_value_rule")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.verify(m.conversion_value_rule);
                                        if (e)
                                            return "conversion_value_rule." + e;
                                    }
                                }
                                if (m.conversion_value_rule_set != null && m.hasOwnProperty("conversion_value_rule_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.verify(m.conversion_value_rule_set);
                                        if (e)
                                            return "conversion_value_rule_set." + e;
                                    }
                                }
                                if (m.click_view != null && m.hasOwnProperty("click_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ClickView.verify(m.click_view);
                                        if (e)
                                            return "click_view." + e;
                                    }
                                }
                                if (m.currency_constant != null && m.hasOwnProperty("currency_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CurrencyConstant.verify(m.currency_constant);
                                        if (e)
                                            return "currency_constant." + e;
                                    }
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomAudience.verify(m.custom_audience);
                                        if (e)
                                            return "custom_audience." + e;
                                    }
                                }
                                if (m.custom_conversion_goal != null && m.hasOwnProperty("custom_conversion_goal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomConversionGoal.verify(m.custom_conversion_goal);
                                        if (e)
                                            return "custom_conversion_goal." + e;
                                    }
                                }
                                if (m.custom_interest != null && m.hasOwnProperty("custom_interest")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomInterest.verify(m.custom_interest);
                                        if (e)
                                            return "custom_interest." + e;
                                    }
                                }
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Customer.verify(m.customer);
                                        if (e)
                                            return "customer." + e;
                                    }
                                }
                                if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerAsset.verify(m.customer_asset);
                                        if (e)
                                            return "customer_asset." + e;
                                    }
                                }
                                if (m.accessible_bidding_strategy != null && m.hasOwnProperty("accessible_bidding_strategy")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.verify(m.accessible_bidding_strategy);
                                        if (e)
                                            return "accessible_bidding_strategy." + e;
                                    }
                                }
                                if (m.customer_customizer != null && m.hasOwnProperty("customer_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerCustomizer.verify(m.customer_customizer);
                                        if (e)
                                            return "customer_customizer." + e;
                                    }
                                }
                                if (m.customer_manager_link != null && m.hasOwnProperty("customer_manager_link")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerManagerLink.verify(m.customer_manager_link);
                                        if (e)
                                            return "customer_manager_link." + e;
                                    }
                                }
                                if (m.customer_client_link != null && m.hasOwnProperty("customer_client_link")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerClientLink.verify(m.customer_client_link);
                                        if (e)
                                            return "customer_client_link." + e;
                                    }
                                }
                                if (m.customer_client != null && m.hasOwnProperty("customer_client")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerClient.verify(m.customer_client);
                                        if (e)
                                            return "customer_client." + e;
                                    }
                                }
                                if (m.customer_conversion_goal != null && m.hasOwnProperty("customer_conversion_goal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.verify(m.customer_conversion_goal);
                                        if (e)
                                            return "customer_conversion_goal." + e;
                                    }
                                }
                                if (m.customer_extension_setting != null && m.hasOwnProperty("customer_extension_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.verify(m.customer_extension_setting);
                                        if (e)
                                            return "customer_extension_setting." + e;
                                    }
                                }
                                if (m.customer_feed != null && m.hasOwnProperty("customer_feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerFeed.verify(m.customer_feed);
                                        if (e)
                                            return "customer_feed." + e;
                                    }
                                }
                                if (m.customer_label != null && m.hasOwnProperty("customer_label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerLabel.verify(m.customer_label);
                                        if (e)
                                            return "customer_label." + e;
                                    }
                                }
                                if (m.customer_negative_criterion != null && m.hasOwnProperty("customer_negative_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.verify(m.customer_negative_criterion);
                                        if (e)
                                            return "customer_negative_criterion." + e;
                                    }
                                }
                                if (m.customer_user_access != null && m.hasOwnProperty("customer_user_access")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerUserAccess.verify(m.customer_user_access);
                                        if (e)
                                            return "customer_user_access." + e;
                                    }
                                }
                                if (m.customer_user_access_invitation != null && m.hasOwnProperty("customer_user_access_invitation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.verify(m.customer_user_access_invitation);
                                        if (e)
                                            return "customer_user_access_invitation." + e;
                                    }
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomizerAttribute.verify(m.customizer_attribute);
                                        if (e)
                                            return "customizer_attribute." + e;
                                    }
                                }
                                if (m.detail_placement_view != null && m.hasOwnProperty("detail_placement_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.DetailPlacementView.verify(m.detail_placement_view);
                                        if (e)
                                            return "detail_placement_view." + e;
                                    }
                                }
                                if (m.detailed_demographic != null && m.hasOwnProperty("detailed_demographic")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.DetailedDemographic.verify(m.detailed_demographic);
                                        if (e)
                                            return "detailed_demographic." + e;
                                    }
                                }
                                if (m.display_keyword_view != null && m.hasOwnProperty("display_keyword_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.DisplayKeywordView.verify(m.display_keyword_view);
                                        if (e)
                                            return "display_keyword_view." + e;
                                    }
                                }
                                if (m.distance_view != null && m.hasOwnProperty("distance_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.DistanceView.verify(m.distance_view);
                                        if (e)
                                            return "distance_view." + e;
                                    }
                                }
                                if (m.dynamic_search_ads_search_term_view != null && m.hasOwnProperty("dynamic_search_ads_search_term_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView.verify(m.dynamic_search_ads_search_term_view);
                                        if (e)
                                            return "dynamic_search_ads_search_term_view." + e;
                                    }
                                }
                                if (m.expanded_landing_page_view != null && m.hasOwnProperty("expanded_landing_page_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExpandedLandingPageView.verify(m.expanded_landing_page_view);
                                        if (e)
                                            return "expanded_landing_page_view." + e;
                                    }
                                }
                                if (m.extension_feed_item != null && m.hasOwnProperty("extension_feed_item")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.verify(m.extension_feed_item);
                                        if (e)
                                            return "extension_feed_item." + e;
                                    }
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Feed.verify(m.feed);
                                        if (e)
                                            return "feed." + e;
                                    }
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItem.verify(m.feed_item);
                                        if (e)
                                            return "feed_item." + e;
                                    }
                                }
                                if (m.feed_item_set != null && m.hasOwnProperty("feed_item_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemSet.verify(m.feed_item_set);
                                        if (e)
                                            return "feed_item_set." + e;
                                    }
                                }
                                if (m.feed_item_set_link != null && m.hasOwnProperty("feed_item_set_link")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemSetLink.verify(m.feed_item_set_link);
                                        if (e)
                                            return "feed_item_set_link." + e;
                                    }
                                }
                                if (m.feed_item_target != null && m.hasOwnProperty("feed_item_target")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemTarget.verify(m.feed_item_target);
                                        if (e)
                                            return "feed_item_target." + e;
                                    }
                                }
                                if (m.feed_mapping != null && m.hasOwnProperty("feed_mapping")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedMapping.verify(m.feed_mapping);
                                        if (e)
                                            return "feed_mapping." + e;
                                    }
                                }
                                if (m.feed_placeholder_view != null && m.hasOwnProperty("feed_placeholder_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedPlaceholderView.verify(m.feed_placeholder_view);
                                        if (e)
                                            return "feed_placeholder_view." + e;
                                    }
                                }
                                if (m.gender_view != null && m.hasOwnProperty("gender_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.GenderView.verify(m.gender_view);
                                        if (e)
                                            return "gender_view." + e;
                                    }
                                }
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.GeoTargetConstant.verify(m.geo_target_constant);
                                        if (e)
                                            return "geo_target_constant." + e;
                                    }
                                }
                                if (m.geographic_view != null && m.hasOwnProperty("geographic_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.GeographicView.verify(m.geographic_view);
                                        if (e)
                                            return "geographic_view." + e;
                                    }
                                }
                                if (m.group_placement_view != null && m.hasOwnProperty("group_placement_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.GroupPlacementView.verify(m.group_placement_view);
                                        if (e)
                                            return "group_placement_view." + e;
                                    }
                                }
                                if (m.hotel_group_view != null && m.hasOwnProperty("hotel_group_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.HotelGroupView.verify(m.hotel_group_view);
                                        if (e)
                                            return "hotel_group_view." + e;
                                    }
                                }
                                if (m.hotel_performance_view != null && m.hasOwnProperty("hotel_performance_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.HotelPerformanceView.verify(m.hotel_performance_view);
                                        if (e)
                                            return "hotel_performance_view." + e;
                                    }
                                }
                                if (m.hotel_reconciliation != null && m.hasOwnProperty("hotel_reconciliation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.HotelReconciliation.verify(m.hotel_reconciliation);
                                        if (e)
                                            return "hotel_reconciliation." + e;
                                    }
                                }
                                if (m.income_range_view != null && m.hasOwnProperty("income_range_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.IncomeRangeView.verify(m.income_range_view);
                                        if (e)
                                            return "income_range_view." + e;
                                    }
                                }
                                if (m.keyword_view != null && m.hasOwnProperty("keyword_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordView.verify(m.keyword_view);
                                        if (e)
                                            return "keyword_view." + e;
                                    }
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlan.verify(m.keyword_plan);
                                        if (e)
                                            return "keyword_plan." + e;
                                    }
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.verify(m.keyword_plan_campaign);
                                        if (e)
                                            return "keyword_plan_campaign." + e;
                                    }
                                }
                                if (m.keyword_plan_campaign_keyword != null && m.hasOwnProperty("keyword_plan_campaign_keyword")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.verify(m.keyword_plan_campaign_keyword);
                                        if (e)
                                            return "keyword_plan_campaign_keyword." + e;
                                    }
                                }
                                if (m.keyword_plan_ad_group != null && m.hasOwnProperty("keyword_plan_ad_group")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.verify(m.keyword_plan_ad_group);
                                        if (e)
                                            return "keyword_plan_ad_group." + e;
                                    }
                                }
                                if (m.keyword_plan_ad_group_keyword != null && m.hasOwnProperty("keyword_plan_ad_group_keyword")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.verify(m.keyword_plan_ad_group_keyword);
                                        if (e)
                                            return "keyword_plan_ad_group_keyword." + e;
                                    }
                                }
                                if (m.keyword_theme_constant != null && m.hasOwnProperty("keyword_theme_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.verify(m.keyword_theme_constant);
                                        if (e)
                                            return "keyword_theme_constant." + e;
                                    }
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Label.verify(m.label);
                                        if (e)
                                            return "label." + e;
                                    }
                                }
                                if (m.landing_page_view != null && m.hasOwnProperty("landing_page_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.LandingPageView.verify(m.landing_page_view);
                                        if (e)
                                            return "landing_page_view." + e;
                                    }
                                }
                                if (m.language_constant != null && m.hasOwnProperty("language_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.LanguageConstant.verify(m.language_constant);
                                        if (e)
                                            return "language_constant." + e;
                                    }
                                }
                                if (m.location_view != null && m.hasOwnProperty("location_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.LocationView.verify(m.location_view);
                                        if (e)
                                            return "location_view." + e;
                                    }
                                }
                                if (m.managed_placement_view != null && m.hasOwnProperty("managed_placement_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ManagedPlacementView.verify(m.managed_placement_view);
                                        if (e)
                                            return "managed_placement_view." + e;
                                    }
                                }
                                if (m.media_file != null && m.hasOwnProperty("media_file")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MediaFile.verify(m.media_file);
                                        if (e)
                                            return "media_file." + e;
                                    }
                                }
                                if (m.mobile_app_category_constant != null && m.hasOwnProperty("mobile_app_category_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant.verify(m.mobile_app_category_constant);
                                        if (e)
                                            return "mobile_app_category_constant." + e;
                                    }
                                }
                                if (m.mobile_device_constant != null && m.hasOwnProperty("mobile_device_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MobileDeviceConstant.verify(m.mobile_device_constant);
                                        if (e)
                                            return "mobile_device_constant." + e;
                                    }
                                }
                                if (m.offline_user_data_job != null && m.hasOwnProperty("offline_user_data_job")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.verify(m.offline_user_data_job);
                                        if (e)
                                            return "offline_user_data_job." + e;
                                    }
                                }
                                if (m.operating_system_version_constant != null && m.hasOwnProperty("operating_system_version_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant.verify(m.operating_system_version_constant);
                                        if (e)
                                            return "operating_system_version_constant." + e;
                                    }
                                }
                                if (m.paid_organic_search_term_view != null && m.hasOwnProperty("paid_organic_search_term_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView.verify(m.paid_organic_search_term_view);
                                        if (e)
                                            return "paid_organic_search_term_view." + e;
                                    }
                                }
                                if (m.parental_status_view != null && m.hasOwnProperty("parental_status_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ParentalStatusView.verify(m.parental_status_view);
                                        if (e)
                                            return "parental_status_view." + e;
                                    }
                                }
                                if (m.product_bidding_category_constant != null && m.hasOwnProperty("product_bidding_category_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant.verify(m.product_bidding_category_constant);
                                        if (e)
                                            return "product_bidding_category_constant." + e;
                                    }
                                }
                                if (m.product_group_view != null && m.hasOwnProperty("product_group_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ProductGroupView.verify(m.product_group_view);
                                        if (e)
                                            return "product_group_view." + e;
                                    }
                                }
                                if (m.recommendation != null && m.hasOwnProperty("recommendation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Recommendation.verify(m.recommendation);
                                        if (e)
                                            return "recommendation." + e;
                                    }
                                }
                                if (m.search_term_view != null && m.hasOwnProperty("search_term_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SearchTermView.verify(m.search_term_view);
                                        if (e)
                                            return "search_term_view." + e;
                                    }
                                }
                                if (m.shared_criterion != null && m.hasOwnProperty("shared_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SharedCriterion.verify(m.shared_criterion);
                                        if (e)
                                            return "shared_criterion." + e;
                                    }
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SharedSet.verify(m.shared_set);
                                        if (e)
                                            return "shared_set." + e;
                                    }
                                }
                                if (m.smart_campaign_setting != null && m.hasOwnProperty("smart_campaign_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.verify(m.smart_campaign_setting);
                                        if (e)
                                            return "smart_campaign_setting." + e;
                                    }
                                }
                                if (m.shopping_performance_view != null && m.hasOwnProperty("shopping_performance_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ShoppingPerformanceView.verify(m.shopping_performance_view);
                                        if (e)
                                            return "shopping_performance_view." + e;
                                    }
                                }
                                if (m.smart_campaign_search_term_view != null && m.hasOwnProperty("smart_campaign_search_term_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView.verify(m.smart_campaign_search_term_view);
                                        if (e)
                                            return "smart_campaign_search_term_view." + e;
                                    }
                                }
                                if (m.third_party_app_analytics_link != null && m.hasOwnProperty("third_party_app_analytics_link")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink.verify(m.third_party_app_analytics_link);
                                        if (e)
                                            return "third_party_app_analytics_link." + e;
                                    }
                                }
                                if (m.topic_view != null && m.hasOwnProperty("topic_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.TopicView.verify(m.topic_view);
                                        if (e)
                                            return "topic_view." + e;
                                    }
                                }
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Experiment.verify(m.experiment);
                                        if (e)
                                            return "experiment." + e;
                                    }
                                }
                                if (m.experiment_arm != null && m.hasOwnProperty("experiment_arm")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExperimentArm.verify(m.experiment_arm);
                                        if (e)
                                            return "experiment_arm." + e;
                                    }
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.UserInterest.verify(m.user_interest);
                                        if (e)
                                            return "user_interest." + e;
                                    }
                                }
                                if (m.life_event != null && m.hasOwnProperty("life_event")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.LifeEvent.verify(m.life_event);
                                        if (e)
                                            return "life_event." + e;
                                    }
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.UserList.verify(m.user_list);
                                        if (e)
                                            return "user_list." + e;
                                    }
                                }
                                if (m.user_location_view != null && m.hasOwnProperty("user_location_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.UserLocationView.verify(m.user_location_view);
                                        if (e)
                                            return "user_location_view." + e;
                                    }
                                }
                                if (m.remarketing_action != null && m.hasOwnProperty("remarketing_action")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.RemarketingAction.verify(m.remarketing_action);
                                        if (e)
                                            return "remarketing_action." + e;
                                    }
                                }
                                if (m.topic_constant != null && m.hasOwnProperty("topic_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.TopicConstant.verify(m.topic_constant);
                                        if (e)
                                            return "topic_constant." + e;
                                    }
                                }
                                if (m.video != null && m.hasOwnProperty("video")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Video.verify(m.video);
                                        if (e)
                                            return "video." + e;
                                    }
                                }
                                if (m.webpage_view != null && m.hasOwnProperty("webpage_view")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.WebpageView.verify(m.webpage_view);
                                        if (e)
                                            return "webpage_view." + e;
                                    }
                                }
                                if (m.lead_form_submission_data != null && m.hasOwnProperty("lead_form_submission_data")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.LeadFormSubmissionData.verify(m.lead_form_submission_data);
                                        if (e)
                                            return "lead_form_submission_data." + e;
                                    }
                                }
                                if (m.metrics != null && m.hasOwnProperty("metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Metrics.verify(m.metrics);
                                        if (e)
                                            return "metrics." + e;
                                    }
                                }
                                if (m.segments != null && m.hasOwnProperty("segments")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.Segments.verify(m.segments);
                                        if (e)
                                            return "segments." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GoogleAdsRow message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GoogleAdsRow} GoogleAdsRow
                             */
                            GoogleAdsRow.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GoogleAdsRow)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GoogleAdsRow();
                                if (d.account_budget != null) {
                                    if (typeof d.account_budget !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.account_budget: object expected");
                                    m.account_budget = $root.google.ads.googleads.v11.resources.AccountBudget.fromObject(d.account_budget);
                                }
                                if (d.account_budget_proposal != null) {
                                    if (typeof d.account_budget_proposal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.account_budget_proposal: object expected");
                                    m.account_budget_proposal = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.fromObject(d.account_budget_proposal);
                                }
                                if (d.account_link != null) {
                                    if (typeof d.account_link !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.account_link: object expected");
                                    m.account_link = $root.google.ads.googleads.v11.resources.AccountLink.fromObject(d.account_link);
                                }
                                if (d.ad_group != null) {
                                    if (typeof d.ad_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group: object expected");
                                    m.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.fromObject(d.ad_group);
                                }
                                if (d.ad_group_ad != null) {
                                    if (typeof d.ad_group_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_ad: object expected");
                                    m.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.fromObject(d.ad_group_ad);
                                }
                                if (d.ad_group_ad_asset_combination_view != null) {
                                    if (typeof d.ad_group_ad_asset_combination_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_ad_asset_combination_view: object expected");
                                    m.ad_group_ad_asset_combination_view = $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.fromObject(d.ad_group_ad_asset_combination_view);
                                }
                                if (d.ad_group_ad_asset_view != null) {
                                    if (typeof d.ad_group_ad_asset_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_ad_asset_view: object expected");
                                    m.ad_group_ad_asset_view = $root.google.ads.googleads.v11.resources.AdGroupAdAssetView.fromObject(d.ad_group_ad_asset_view);
                                }
                                if (d.ad_group_ad_label != null) {
                                    if (typeof d.ad_group_ad_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_ad_label: object expected");
                                    m.ad_group_ad_label = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.fromObject(d.ad_group_ad_label);
                                }
                                if (d.ad_group_asset != null) {
                                    if (typeof d.ad_group_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_asset: object expected");
                                    m.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.fromObject(d.ad_group_asset);
                                }
                                if (d.ad_group_audience_view != null) {
                                    if (typeof d.ad_group_audience_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_audience_view: object expected");
                                    m.ad_group_audience_view = $root.google.ads.googleads.v11.resources.AdGroupAudienceView.fromObject(d.ad_group_audience_view);
                                }
                                if (d.ad_group_bid_modifier != null) {
                                    if (typeof d.ad_group_bid_modifier !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_bid_modifier: object expected");
                                    m.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.fromObject(d.ad_group_bid_modifier);
                                }
                                if (d.ad_group_criterion != null) {
                                    if (typeof d.ad_group_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_criterion: object expected");
                                    m.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.fromObject(d.ad_group_criterion);
                                }
                                if (d.ad_group_criterion_customizer != null) {
                                    if (typeof d.ad_group_criterion_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_criterion_customizer: object expected");
                                    m.ad_group_criterion_customizer = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.fromObject(d.ad_group_criterion_customizer);
                                }
                                if (d.ad_group_criterion_label != null) {
                                    if (typeof d.ad_group_criterion_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_criterion_label: object expected");
                                    m.ad_group_criterion_label = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.fromObject(d.ad_group_criterion_label);
                                }
                                if (d.ad_group_criterion_simulation != null) {
                                    if (typeof d.ad_group_criterion_simulation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_criterion_simulation: object expected");
                                    m.ad_group_criterion_simulation = $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation.fromObject(d.ad_group_criterion_simulation);
                                }
                                if (d.ad_group_customizer != null) {
                                    if (typeof d.ad_group_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_customizer: object expected");
                                    m.ad_group_customizer = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.fromObject(d.ad_group_customizer);
                                }
                                if (d.ad_group_extension_setting != null) {
                                    if (typeof d.ad_group_extension_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_extension_setting: object expected");
                                    m.ad_group_extension_setting = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.fromObject(d.ad_group_extension_setting);
                                }
                                if (d.ad_group_feed != null) {
                                    if (typeof d.ad_group_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_feed: object expected");
                                    m.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.fromObject(d.ad_group_feed);
                                }
                                if (d.ad_group_label != null) {
                                    if (typeof d.ad_group_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_label: object expected");
                                    m.ad_group_label = $root.google.ads.googleads.v11.resources.AdGroupLabel.fromObject(d.ad_group_label);
                                }
                                if (d.ad_group_simulation != null) {
                                    if (typeof d.ad_group_simulation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_group_simulation: object expected");
                                    m.ad_group_simulation = $root.google.ads.googleads.v11.resources.AdGroupSimulation.fromObject(d.ad_group_simulation);
                                }
                                if (d.ad_parameter != null) {
                                    if (typeof d.ad_parameter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_parameter: object expected");
                                    m.ad_parameter = $root.google.ads.googleads.v11.resources.AdParameter.fromObject(d.ad_parameter);
                                }
                                if (d.age_range_view != null) {
                                    if (typeof d.age_range_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.age_range_view: object expected");
                                    m.age_range_view = $root.google.ads.googleads.v11.resources.AgeRangeView.fromObject(d.age_range_view);
                                }
                                if (d.ad_schedule_view != null) {
                                    if (typeof d.ad_schedule_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.ad_schedule_view: object expected");
                                    m.ad_schedule_view = $root.google.ads.googleads.v11.resources.AdScheduleView.fromObject(d.ad_schedule_view);
                                }
                                if (d.domain_category != null) {
                                    if (typeof d.domain_category !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.domain_category: object expected");
                                    m.domain_category = $root.google.ads.googleads.v11.resources.DomainCategory.fromObject(d.domain_category);
                                }
                                if (d.asset != null) {
                                    if (typeof d.asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset: object expected");
                                    m.asset = $root.google.ads.googleads.v11.resources.Asset.fromObject(d.asset);
                                }
                                if (d.asset_field_type_view != null) {
                                    if (typeof d.asset_field_type_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_field_type_view: object expected");
                                    m.asset_field_type_view = $root.google.ads.googleads.v11.resources.AssetFieldTypeView.fromObject(d.asset_field_type_view);
                                }
                                if (d.asset_group_asset != null) {
                                    if (typeof d.asset_group_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_group_asset: object expected");
                                    m.asset_group_asset = $root.google.ads.googleads.v11.resources.AssetGroupAsset.fromObject(d.asset_group_asset);
                                }
                                if (d.asset_group_signal != null) {
                                    if (typeof d.asset_group_signal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_group_signal: object expected");
                                    m.asset_group_signal = $root.google.ads.googleads.v11.resources.AssetGroupSignal.fromObject(d.asset_group_signal);
                                }
                                if (d.asset_group_listing_group_filter != null) {
                                    if (typeof d.asset_group_listing_group_filter !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_group_listing_group_filter: object expected");
                                    m.asset_group_listing_group_filter = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.fromObject(d.asset_group_listing_group_filter);
                                }
                                if (d.asset_group_product_group_view != null) {
                                    if (typeof d.asset_group_product_group_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_group_product_group_view: object expected");
                                    m.asset_group_product_group_view = $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView.fromObject(d.asset_group_product_group_view);
                                }
                                if (d.asset_group != null) {
                                    if (typeof d.asset_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_group: object expected");
                                    m.asset_group = $root.google.ads.googleads.v11.resources.AssetGroup.fromObject(d.asset_group);
                                }
                                if (d.asset_set_asset != null) {
                                    if (typeof d.asset_set_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_set_asset: object expected");
                                    m.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.fromObject(d.asset_set_asset);
                                }
                                if (d.asset_set != null) {
                                    if (typeof d.asset_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.asset_set: object expected");
                                    m.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.fromObject(d.asset_set);
                                }
                                if (d.batch_job != null) {
                                    if (typeof d.batch_job !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.batch_job: object expected");
                                    m.batch_job = $root.google.ads.googleads.v11.resources.BatchJob.fromObject(d.batch_job);
                                }
                                if (d.bidding_data_exclusion != null) {
                                    if (typeof d.bidding_data_exclusion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.bidding_data_exclusion: object expected");
                                    m.bidding_data_exclusion = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.fromObject(d.bidding_data_exclusion);
                                }
                                if (d.bidding_seasonality_adjustment != null) {
                                    if (typeof d.bidding_seasonality_adjustment !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.bidding_seasonality_adjustment: object expected");
                                    m.bidding_seasonality_adjustment = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.fromObject(d.bidding_seasonality_adjustment);
                                }
                                if (d.bidding_strategy != null) {
                                    if (typeof d.bidding_strategy !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.bidding_strategy: object expected");
                                    m.bidding_strategy = $root.google.ads.googleads.v11.resources.BiddingStrategy.fromObject(d.bidding_strategy);
                                }
                                if (d.bidding_strategy_simulation != null) {
                                    if (typeof d.bidding_strategy_simulation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.bidding_strategy_simulation: object expected");
                                    m.bidding_strategy_simulation = $root.google.ads.googleads.v11.resources.BiddingStrategySimulation.fromObject(d.bidding_strategy_simulation);
                                }
                                if (d.billing_setup != null) {
                                    if (typeof d.billing_setup !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.billing_setup: object expected");
                                    m.billing_setup = $root.google.ads.googleads.v11.resources.BillingSetup.fromObject(d.billing_setup);
                                }
                                if (d.call_view != null) {
                                    if (typeof d.call_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.call_view: object expected");
                                    m.call_view = $root.google.ads.googleads.v11.resources.CallView.fromObject(d.call_view);
                                }
                                if (d.campaign_budget != null) {
                                    if (typeof d.campaign_budget !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_budget: object expected");
                                    m.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.fromObject(d.campaign_budget);
                                }
                                if (d.campaign != null) {
                                    if (typeof d.campaign !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign: object expected");
                                    m.campaign = $root.google.ads.googleads.v11.resources.Campaign.fromObject(d.campaign);
                                }
                                if (d.campaign_asset != null) {
                                    if (typeof d.campaign_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_asset: object expected");
                                    m.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.fromObject(d.campaign_asset);
                                }
                                if (d.campaign_asset_set != null) {
                                    if (typeof d.campaign_asset_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_asset_set: object expected");
                                    m.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.fromObject(d.campaign_asset_set);
                                }
                                if (d.campaign_audience_view != null) {
                                    if (typeof d.campaign_audience_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_audience_view: object expected");
                                    m.campaign_audience_view = $root.google.ads.googleads.v11.resources.CampaignAudienceView.fromObject(d.campaign_audience_view);
                                }
                                if (d.campaign_bid_modifier != null) {
                                    if (typeof d.campaign_bid_modifier !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_bid_modifier: object expected");
                                    m.campaign_bid_modifier = $root.google.ads.googleads.v11.resources.CampaignBidModifier.fromObject(d.campaign_bid_modifier);
                                }
                                if (d.campaign_conversion_goal != null) {
                                    if (typeof d.campaign_conversion_goal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_conversion_goal: object expected");
                                    m.campaign_conversion_goal = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.fromObject(d.campaign_conversion_goal);
                                }
                                if (d.campaign_criterion != null) {
                                    if (typeof d.campaign_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_criterion: object expected");
                                    m.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.fromObject(d.campaign_criterion);
                                }
                                if (d.campaign_criterion_simulation != null) {
                                    if (typeof d.campaign_criterion_simulation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_criterion_simulation: object expected");
                                    m.campaign_criterion_simulation = $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation.fromObject(d.campaign_criterion_simulation);
                                }
                                if (d.campaign_customizer != null) {
                                    if (typeof d.campaign_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_customizer: object expected");
                                    m.campaign_customizer = $root.google.ads.googleads.v11.resources.CampaignCustomizer.fromObject(d.campaign_customizer);
                                }
                                if (d.campaign_draft != null) {
                                    if (typeof d.campaign_draft !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_draft: object expected");
                                    m.campaign_draft = $root.google.ads.googleads.v11.resources.CampaignDraft.fromObject(d.campaign_draft);
                                }
                                if (d.campaign_experiment != null) {
                                    if (typeof d.campaign_experiment !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_experiment: object expected");
                                    m.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.fromObject(d.campaign_experiment);
                                }
                                if (d.campaign_extension_setting != null) {
                                    if (typeof d.campaign_extension_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_extension_setting: object expected");
                                    m.campaign_extension_setting = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.fromObject(d.campaign_extension_setting);
                                }
                                if (d.campaign_feed != null) {
                                    if (typeof d.campaign_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_feed: object expected");
                                    m.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.fromObject(d.campaign_feed);
                                }
                                if (d.campaign_group != null) {
                                    if (typeof d.campaign_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_group: object expected");
                                    m.campaign_group = $root.google.ads.googleads.v11.resources.CampaignGroup.fromObject(d.campaign_group);
                                }
                                if (d.campaign_label != null) {
                                    if (typeof d.campaign_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_label: object expected");
                                    m.campaign_label = $root.google.ads.googleads.v11.resources.CampaignLabel.fromObject(d.campaign_label);
                                }
                                if (d.campaign_shared_set != null) {
                                    if (typeof d.campaign_shared_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_shared_set: object expected");
                                    m.campaign_shared_set = $root.google.ads.googleads.v11.resources.CampaignSharedSet.fromObject(d.campaign_shared_set);
                                }
                                if (d.campaign_simulation != null) {
                                    if (typeof d.campaign_simulation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.campaign_simulation: object expected");
                                    m.campaign_simulation = $root.google.ads.googleads.v11.resources.CampaignSimulation.fromObject(d.campaign_simulation);
                                }
                                if (d.carrier_constant != null) {
                                    if (typeof d.carrier_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.carrier_constant: object expected");
                                    m.carrier_constant = $root.google.ads.googleads.v11.resources.CarrierConstant.fromObject(d.carrier_constant);
                                }
                                if (d.change_event != null) {
                                    if (typeof d.change_event !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.change_event: object expected");
                                    m.change_event = $root.google.ads.googleads.v11.resources.ChangeEvent.fromObject(d.change_event);
                                }
                                if (d.change_status != null) {
                                    if (typeof d.change_status !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.change_status: object expected");
                                    m.change_status = $root.google.ads.googleads.v11.resources.ChangeStatus.fromObject(d.change_status);
                                }
                                if (d.combined_audience != null) {
                                    if (typeof d.combined_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.combined_audience: object expected");
                                    m.combined_audience = $root.google.ads.googleads.v11.resources.CombinedAudience.fromObject(d.combined_audience);
                                }
                                if (d.audience != null) {
                                    if (typeof d.audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.audience: object expected");
                                    m.audience = $root.google.ads.googleads.v11.resources.Audience.fromObject(d.audience);
                                }
                                if (d.conversion_action != null) {
                                    if (typeof d.conversion_action !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.conversion_action: object expected");
                                    m.conversion_action = $root.google.ads.googleads.v11.resources.ConversionAction.fromObject(d.conversion_action);
                                }
                                if (d.conversion_custom_variable != null) {
                                    if (typeof d.conversion_custom_variable !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.conversion_custom_variable: object expected");
                                    m.conversion_custom_variable = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.fromObject(d.conversion_custom_variable);
                                }
                                if (d.conversion_goal_campaign_config != null) {
                                    if (typeof d.conversion_goal_campaign_config !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.conversion_goal_campaign_config: object expected");
                                    m.conversion_goal_campaign_config = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.fromObject(d.conversion_goal_campaign_config);
                                }
                                if (d.conversion_value_rule != null) {
                                    if (typeof d.conversion_value_rule !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.conversion_value_rule: object expected");
                                    m.conversion_value_rule = $root.google.ads.googleads.v11.resources.ConversionValueRule.fromObject(d.conversion_value_rule);
                                }
                                if (d.conversion_value_rule_set != null) {
                                    if (typeof d.conversion_value_rule_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.conversion_value_rule_set: object expected");
                                    m.conversion_value_rule_set = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.fromObject(d.conversion_value_rule_set);
                                }
                                if (d.click_view != null) {
                                    if (typeof d.click_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.click_view: object expected");
                                    m.click_view = $root.google.ads.googleads.v11.resources.ClickView.fromObject(d.click_view);
                                }
                                if (d.currency_constant != null) {
                                    if (typeof d.currency_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.currency_constant: object expected");
                                    m.currency_constant = $root.google.ads.googleads.v11.resources.CurrencyConstant.fromObject(d.currency_constant);
                                }
                                if (d.custom_audience != null) {
                                    if (typeof d.custom_audience !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.custom_audience: object expected");
                                    m.custom_audience = $root.google.ads.googleads.v11.resources.CustomAudience.fromObject(d.custom_audience);
                                }
                                if (d.custom_conversion_goal != null) {
                                    if (typeof d.custom_conversion_goal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.custom_conversion_goal: object expected");
                                    m.custom_conversion_goal = $root.google.ads.googleads.v11.resources.CustomConversionGoal.fromObject(d.custom_conversion_goal);
                                }
                                if (d.custom_interest != null) {
                                    if (typeof d.custom_interest !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.custom_interest: object expected");
                                    m.custom_interest = $root.google.ads.googleads.v11.resources.CustomInterest.fromObject(d.custom_interest);
                                }
                                if (d.customer != null) {
                                    if (typeof d.customer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer: object expected");
                                    m.customer = $root.google.ads.googleads.v11.resources.Customer.fromObject(d.customer);
                                }
                                if (d.customer_asset != null) {
                                    if (typeof d.customer_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_asset: object expected");
                                    m.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.fromObject(d.customer_asset);
                                }
                                if (d.accessible_bidding_strategy != null) {
                                    if (typeof d.accessible_bidding_strategy !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.accessible_bidding_strategy: object expected");
                                    m.accessible_bidding_strategy = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.fromObject(d.accessible_bidding_strategy);
                                }
                                if (d.customer_customizer != null) {
                                    if (typeof d.customer_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_customizer: object expected");
                                    m.customer_customizer = $root.google.ads.googleads.v11.resources.CustomerCustomizer.fromObject(d.customer_customizer);
                                }
                                if (d.customer_manager_link != null) {
                                    if (typeof d.customer_manager_link !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_manager_link: object expected");
                                    m.customer_manager_link = $root.google.ads.googleads.v11.resources.CustomerManagerLink.fromObject(d.customer_manager_link);
                                }
                                if (d.customer_client_link != null) {
                                    if (typeof d.customer_client_link !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_client_link: object expected");
                                    m.customer_client_link = $root.google.ads.googleads.v11.resources.CustomerClientLink.fromObject(d.customer_client_link);
                                }
                                if (d.customer_client != null) {
                                    if (typeof d.customer_client !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_client: object expected");
                                    m.customer_client = $root.google.ads.googleads.v11.resources.CustomerClient.fromObject(d.customer_client);
                                }
                                if (d.customer_conversion_goal != null) {
                                    if (typeof d.customer_conversion_goal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_conversion_goal: object expected");
                                    m.customer_conversion_goal = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.fromObject(d.customer_conversion_goal);
                                }
                                if (d.customer_extension_setting != null) {
                                    if (typeof d.customer_extension_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_extension_setting: object expected");
                                    m.customer_extension_setting = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.fromObject(d.customer_extension_setting);
                                }
                                if (d.customer_feed != null) {
                                    if (typeof d.customer_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_feed: object expected");
                                    m.customer_feed = $root.google.ads.googleads.v11.resources.CustomerFeed.fromObject(d.customer_feed);
                                }
                                if (d.customer_label != null) {
                                    if (typeof d.customer_label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_label: object expected");
                                    m.customer_label = $root.google.ads.googleads.v11.resources.CustomerLabel.fromObject(d.customer_label);
                                }
                                if (d.customer_negative_criterion != null) {
                                    if (typeof d.customer_negative_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_negative_criterion: object expected");
                                    m.customer_negative_criterion = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.fromObject(d.customer_negative_criterion);
                                }
                                if (d.customer_user_access != null) {
                                    if (typeof d.customer_user_access !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_user_access: object expected");
                                    m.customer_user_access = $root.google.ads.googleads.v11.resources.CustomerUserAccess.fromObject(d.customer_user_access);
                                }
                                if (d.customer_user_access_invitation != null) {
                                    if (typeof d.customer_user_access_invitation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customer_user_access_invitation: object expected");
                                    m.customer_user_access_invitation = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.fromObject(d.customer_user_access_invitation);
                                }
                                if (d.customizer_attribute != null) {
                                    if (typeof d.customizer_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.customizer_attribute: object expected");
                                    m.customizer_attribute = $root.google.ads.googleads.v11.resources.CustomizerAttribute.fromObject(d.customizer_attribute);
                                }
                                if (d.detail_placement_view != null) {
                                    if (typeof d.detail_placement_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.detail_placement_view: object expected");
                                    m.detail_placement_view = $root.google.ads.googleads.v11.resources.DetailPlacementView.fromObject(d.detail_placement_view);
                                }
                                if (d.detailed_demographic != null) {
                                    if (typeof d.detailed_demographic !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.detailed_demographic: object expected");
                                    m.detailed_demographic = $root.google.ads.googleads.v11.resources.DetailedDemographic.fromObject(d.detailed_demographic);
                                }
                                if (d.display_keyword_view != null) {
                                    if (typeof d.display_keyword_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.display_keyword_view: object expected");
                                    m.display_keyword_view = $root.google.ads.googleads.v11.resources.DisplayKeywordView.fromObject(d.display_keyword_view);
                                }
                                if (d.distance_view != null) {
                                    if (typeof d.distance_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.distance_view: object expected");
                                    m.distance_view = $root.google.ads.googleads.v11.resources.DistanceView.fromObject(d.distance_view);
                                }
                                if (d.dynamic_search_ads_search_term_view != null) {
                                    if (typeof d.dynamic_search_ads_search_term_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.dynamic_search_ads_search_term_view: object expected");
                                    m.dynamic_search_ads_search_term_view = $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView.fromObject(d.dynamic_search_ads_search_term_view);
                                }
                                if (d.expanded_landing_page_view != null) {
                                    if (typeof d.expanded_landing_page_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.expanded_landing_page_view: object expected");
                                    m.expanded_landing_page_view = $root.google.ads.googleads.v11.resources.ExpandedLandingPageView.fromObject(d.expanded_landing_page_view);
                                }
                                if (d.extension_feed_item != null) {
                                    if (typeof d.extension_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.extension_feed_item: object expected");
                                    m.extension_feed_item = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.fromObject(d.extension_feed_item);
                                }
                                if (d.feed != null) {
                                    if (typeof d.feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.feed: object expected");
                                    m.feed = $root.google.ads.googleads.v11.resources.Feed.fromObject(d.feed);
                                }
                                if (d.feed_item != null) {
                                    if (typeof d.feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.feed_item: object expected");
                                    m.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.fromObject(d.feed_item);
                                }
                                if (d.feed_item_set != null) {
                                    if (typeof d.feed_item_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.feed_item_set: object expected");
                                    m.feed_item_set = $root.google.ads.googleads.v11.resources.FeedItemSet.fromObject(d.feed_item_set);
                                }
                                if (d.feed_item_set_link != null) {
                                    if (typeof d.feed_item_set_link !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.feed_item_set_link: object expected");
                                    m.feed_item_set_link = $root.google.ads.googleads.v11.resources.FeedItemSetLink.fromObject(d.feed_item_set_link);
                                }
                                if (d.feed_item_target != null) {
                                    if (typeof d.feed_item_target !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.feed_item_target: object expected");
                                    m.feed_item_target = $root.google.ads.googleads.v11.resources.FeedItemTarget.fromObject(d.feed_item_target);
                                }
                                if (d.feed_mapping != null) {
                                    if (typeof d.feed_mapping !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.feed_mapping: object expected");
                                    m.feed_mapping = $root.google.ads.googleads.v11.resources.FeedMapping.fromObject(d.feed_mapping);
                                }
                                if (d.feed_placeholder_view != null) {
                                    if (typeof d.feed_placeholder_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.feed_placeholder_view: object expected");
                                    m.feed_placeholder_view = $root.google.ads.googleads.v11.resources.FeedPlaceholderView.fromObject(d.feed_placeholder_view);
                                }
                                if (d.gender_view != null) {
                                    if (typeof d.gender_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.gender_view: object expected");
                                    m.gender_view = $root.google.ads.googleads.v11.resources.GenderView.fromObject(d.gender_view);
                                }
                                if (d.geo_target_constant != null) {
                                    if (typeof d.geo_target_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.geo_target_constant: object expected");
                                    m.geo_target_constant = $root.google.ads.googleads.v11.resources.GeoTargetConstant.fromObject(d.geo_target_constant);
                                }
                                if (d.geographic_view != null) {
                                    if (typeof d.geographic_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.geographic_view: object expected");
                                    m.geographic_view = $root.google.ads.googleads.v11.resources.GeographicView.fromObject(d.geographic_view);
                                }
                                if (d.group_placement_view != null) {
                                    if (typeof d.group_placement_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.group_placement_view: object expected");
                                    m.group_placement_view = $root.google.ads.googleads.v11.resources.GroupPlacementView.fromObject(d.group_placement_view);
                                }
                                if (d.hotel_group_view != null) {
                                    if (typeof d.hotel_group_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.hotel_group_view: object expected");
                                    m.hotel_group_view = $root.google.ads.googleads.v11.resources.HotelGroupView.fromObject(d.hotel_group_view);
                                }
                                if (d.hotel_performance_view != null) {
                                    if (typeof d.hotel_performance_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.hotel_performance_view: object expected");
                                    m.hotel_performance_view = $root.google.ads.googleads.v11.resources.HotelPerformanceView.fromObject(d.hotel_performance_view);
                                }
                                if (d.hotel_reconciliation != null) {
                                    if (typeof d.hotel_reconciliation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.hotel_reconciliation: object expected");
                                    m.hotel_reconciliation = $root.google.ads.googleads.v11.resources.HotelReconciliation.fromObject(d.hotel_reconciliation);
                                }
                                if (d.income_range_view != null) {
                                    if (typeof d.income_range_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.income_range_view: object expected");
                                    m.income_range_view = $root.google.ads.googleads.v11.resources.IncomeRangeView.fromObject(d.income_range_view);
                                }
                                if (d.keyword_view != null) {
                                    if (typeof d.keyword_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.keyword_view: object expected");
                                    m.keyword_view = $root.google.ads.googleads.v11.resources.KeywordView.fromObject(d.keyword_view);
                                }
                                if (d.keyword_plan != null) {
                                    if (typeof d.keyword_plan !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.keyword_plan: object expected");
                                    m.keyword_plan = $root.google.ads.googleads.v11.resources.KeywordPlan.fromObject(d.keyword_plan);
                                }
                                if (d.keyword_plan_campaign != null) {
                                    if (typeof d.keyword_plan_campaign !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.keyword_plan_campaign: object expected");
                                    m.keyword_plan_campaign = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.fromObject(d.keyword_plan_campaign);
                                }
                                if (d.keyword_plan_campaign_keyword != null) {
                                    if (typeof d.keyword_plan_campaign_keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.keyword_plan_campaign_keyword: object expected");
                                    m.keyword_plan_campaign_keyword = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.fromObject(d.keyword_plan_campaign_keyword);
                                }
                                if (d.keyword_plan_ad_group != null) {
                                    if (typeof d.keyword_plan_ad_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.keyword_plan_ad_group: object expected");
                                    m.keyword_plan_ad_group = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.fromObject(d.keyword_plan_ad_group);
                                }
                                if (d.keyword_plan_ad_group_keyword != null) {
                                    if (typeof d.keyword_plan_ad_group_keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.keyword_plan_ad_group_keyword: object expected");
                                    m.keyword_plan_ad_group_keyword = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.fromObject(d.keyword_plan_ad_group_keyword);
                                }
                                if (d.keyword_theme_constant != null) {
                                    if (typeof d.keyword_theme_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.keyword_theme_constant: object expected");
                                    m.keyword_theme_constant = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.fromObject(d.keyword_theme_constant);
                                }
                                if (d.label != null) {
                                    if (typeof d.label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.label: object expected");
                                    m.label = $root.google.ads.googleads.v11.resources.Label.fromObject(d.label);
                                }
                                if (d.landing_page_view != null) {
                                    if (typeof d.landing_page_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.landing_page_view: object expected");
                                    m.landing_page_view = $root.google.ads.googleads.v11.resources.LandingPageView.fromObject(d.landing_page_view);
                                }
                                if (d.language_constant != null) {
                                    if (typeof d.language_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.language_constant: object expected");
                                    m.language_constant = $root.google.ads.googleads.v11.resources.LanguageConstant.fromObject(d.language_constant);
                                }
                                if (d.location_view != null) {
                                    if (typeof d.location_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.location_view: object expected");
                                    m.location_view = $root.google.ads.googleads.v11.resources.LocationView.fromObject(d.location_view);
                                }
                                if (d.managed_placement_view != null) {
                                    if (typeof d.managed_placement_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.managed_placement_view: object expected");
                                    m.managed_placement_view = $root.google.ads.googleads.v11.resources.ManagedPlacementView.fromObject(d.managed_placement_view);
                                }
                                if (d.media_file != null) {
                                    if (typeof d.media_file !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.media_file: object expected");
                                    m.media_file = $root.google.ads.googleads.v11.resources.MediaFile.fromObject(d.media_file);
                                }
                                if (d.mobile_app_category_constant != null) {
                                    if (typeof d.mobile_app_category_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.mobile_app_category_constant: object expected");
                                    m.mobile_app_category_constant = $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant.fromObject(d.mobile_app_category_constant);
                                }
                                if (d.mobile_device_constant != null) {
                                    if (typeof d.mobile_device_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.mobile_device_constant: object expected");
                                    m.mobile_device_constant = $root.google.ads.googleads.v11.resources.MobileDeviceConstant.fromObject(d.mobile_device_constant);
                                }
                                if (d.offline_user_data_job != null) {
                                    if (typeof d.offline_user_data_job !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.offline_user_data_job: object expected");
                                    m.offline_user_data_job = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.fromObject(d.offline_user_data_job);
                                }
                                if (d.operating_system_version_constant != null) {
                                    if (typeof d.operating_system_version_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.operating_system_version_constant: object expected");
                                    m.operating_system_version_constant = $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant.fromObject(d.operating_system_version_constant);
                                }
                                if (d.paid_organic_search_term_view != null) {
                                    if (typeof d.paid_organic_search_term_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.paid_organic_search_term_view: object expected");
                                    m.paid_organic_search_term_view = $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView.fromObject(d.paid_organic_search_term_view);
                                }
                                if (d.parental_status_view != null) {
                                    if (typeof d.parental_status_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.parental_status_view: object expected");
                                    m.parental_status_view = $root.google.ads.googleads.v11.resources.ParentalStatusView.fromObject(d.parental_status_view);
                                }
                                if (d.product_bidding_category_constant != null) {
                                    if (typeof d.product_bidding_category_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.product_bidding_category_constant: object expected");
                                    m.product_bidding_category_constant = $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant.fromObject(d.product_bidding_category_constant);
                                }
                                if (d.product_group_view != null) {
                                    if (typeof d.product_group_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.product_group_view: object expected");
                                    m.product_group_view = $root.google.ads.googleads.v11.resources.ProductGroupView.fromObject(d.product_group_view);
                                }
                                if (d.recommendation != null) {
                                    if (typeof d.recommendation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.recommendation: object expected");
                                    m.recommendation = $root.google.ads.googleads.v11.resources.Recommendation.fromObject(d.recommendation);
                                }
                                if (d.search_term_view != null) {
                                    if (typeof d.search_term_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.search_term_view: object expected");
                                    m.search_term_view = $root.google.ads.googleads.v11.resources.SearchTermView.fromObject(d.search_term_view);
                                }
                                if (d.shared_criterion != null) {
                                    if (typeof d.shared_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.shared_criterion: object expected");
                                    m.shared_criterion = $root.google.ads.googleads.v11.resources.SharedCriterion.fromObject(d.shared_criterion);
                                }
                                if (d.shared_set != null) {
                                    if (typeof d.shared_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.shared_set: object expected");
                                    m.shared_set = $root.google.ads.googleads.v11.resources.SharedSet.fromObject(d.shared_set);
                                }
                                if (d.smart_campaign_setting != null) {
                                    if (typeof d.smart_campaign_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.smart_campaign_setting: object expected");
                                    m.smart_campaign_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.fromObject(d.smart_campaign_setting);
                                }
                                if (d.shopping_performance_view != null) {
                                    if (typeof d.shopping_performance_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.shopping_performance_view: object expected");
                                    m.shopping_performance_view = $root.google.ads.googleads.v11.resources.ShoppingPerformanceView.fromObject(d.shopping_performance_view);
                                }
                                if (d.smart_campaign_search_term_view != null) {
                                    if (typeof d.smart_campaign_search_term_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.smart_campaign_search_term_view: object expected");
                                    m.smart_campaign_search_term_view = $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView.fromObject(d.smart_campaign_search_term_view);
                                }
                                if (d.third_party_app_analytics_link != null) {
                                    if (typeof d.third_party_app_analytics_link !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.third_party_app_analytics_link: object expected");
                                    m.third_party_app_analytics_link = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink.fromObject(d.third_party_app_analytics_link);
                                }
                                if (d.topic_view != null) {
                                    if (typeof d.topic_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.topic_view: object expected");
                                    m.topic_view = $root.google.ads.googleads.v11.resources.TopicView.fromObject(d.topic_view);
                                }
                                if (d.experiment != null) {
                                    if (typeof d.experiment !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.experiment: object expected");
                                    m.experiment = $root.google.ads.googleads.v11.resources.Experiment.fromObject(d.experiment);
                                }
                                if (d.experiment_arm != null) {
                                    if (typeof d.experiment_arm !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.experiment_arm: object expected");
                                    m.experiment_arm = $root.google.ads.googleads.v11.resources.ExperimentArm.fromObject(d.experiment_arm);
                                }
                                if (d.user_interest != null) {
                                    if (typeof d.user_interest !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.user_interest: object expected");
                                    m.user_interest = $root.google.ads.googleads.v11.resources.UserInterest.fromObject(d.user_interest);
                                }
                                if (d.life_event != null) {
                                    if (typeof d.life_event !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.life_event: object expected");
                                    m.life_event = $root.google.ads.googleads.v11.resources.LifeEvent.fromObject(d.life_event);
                                }
                                if (d.user_list != null) {
                                    if (typeof d.user_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.user_list: object expected");
                                    m.user_list = $root.google.ads.googleads.v11.resources.UserList.fromObject(d.user_list);
                                }
                                if (d.user_location_view != null) {
                                    if (typeof d.user_location_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.user_location_view: object expected");
                                    m.user_location_view = $root.google.ads.googleads.v11.resources.UserLocationView.fromObject(d.user_location_view);
                                }
                                if (d.remarketing_action != null) {
                                    if (typeof d.remarketing_action !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.remarketing_action: object expected");
                                    m.remarketing_action = $root.google.ads.googleads.v11.resources.RemarketingAction.fromObject(d.remarketing_action);
                                }
                                if (d.topic_constant != null) {
                                    if (typeof d.topic_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.topic_constant: object expected");
                                    m.topic_constant = $root.google.ads.googleads.v11.resources.TopicConstant.fromObject(d.topic_constant);
                                }
                                if (d.video != null) {
                                    if (typeof d.video !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.video: object expected");
                                    m.video = $root.google.ads.googleads.v11.resources.Video.fromObject(d.video);
                                }
                                if (d.webpage_view != null) {
                                    if (typeof d.webpage_view !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.webpage_view: object expected");
                                    m.webpage_view = $root.google.ads.googleads.v11.resources.WebpageView.fromObject(d.webpage_view);
                                }
                                if (d.lead_form_submission_data != null) {
                                    if (typeof d.lead_form_submission_data !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.lead_form_submission_data: object expected");
                                    m.lead_form_submission_data = $root.google.ads.googleads.v11.resources.LeadFormSubmissionData.fromObject(d.lead_form_submission_data);
                                }
                                if (d.metrics != null) {
                                    if (typeof d.metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.metrics: object expected");
                                    m.metrics = $root.google.ads.googleads.v11.common.Metrics.fromObject(d.metrics);
                                }
                                if (d.segments != null) {
                                    if (typeof d.segments !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GoogleAdsRow.segments: object expected");
                                    m.segments = $root.google.ads.googleads.v11.common.Segments.fromObject(d.segments);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GoogleAdsRow message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @static
                             * @param {google.ads.googleads.v11.services.GoogleAdsRow} m GoogleAdsRow
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GoogleAdsRow.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer = null;
                                    d.campaign = null;
                                    d.ad_group = null;
                                    d.metrics = null;
                                    d.ad_group_ad = null;
                                    d.ad_group_criterion = null;
                                    d.bidding_strategy = null;
                                    d.campaign_budget = null;
                                    d.campaign_criterion = null;
                                    d.keyword_view = null;
                                    d.recommendation = null;
                                    d.geo_target_constant = null;
                                    d.ad_group_bid_modifier = null;
                                    d.campaign_group = null;
                                    d.campaign_bid_modifier = null;
                                    d.shared_set = null;
                                    d.shared_criterion = null;
                                    d.campaign_shared_set = null;
                                    d.topic_constant = null;
                                    d.keyword_plan = null;
                                    d.keyword_plan_campaign = null;
                                    d.keyword_plan_ad_group = null;
                                    d.change_status = null;
                                    d.user_list = null;
                                    d.video = null;
                                    d.gender_view = null;
                                    d.billing_setup = null;
                                    d.account_budget = null;
                                    d.account_budget_proposal = null;
                                    d.topic_view = null;
                                    d.parental_status_view = null;
                                    d.feed = null;
                                    d.display_keyword_view = null;
                                    d.age_range_view = null;
                                    d.campaign_draft = null;
                                    d.feed_item = null;
                                    d.hotel_group_view = null;
                                    d.label = null;
                                    d.managed_placement_view = null;
                                    d.product_group_view = null;
                                    d.language_constant = null;
                                    d.ad_group_audience_view = null;
                                    d.feed_mapping = null;
                                    d.user_interest = null;
                                    d.remarketing_action = null;
                                    d.customer_manager_link = null;
                                    d.customer_client_link = null;
                                    d.campaign_feed = null;
                                    d.customer_feed = null;
                                    d.carrier_constant = null;
                                    d.ad_group_feed = null;
                                    d.search_term_view = null;
                                    d.campaign_audience_view = null;
                                    d.customer_client = null;
                                    d.hotel_performance_view = null;
                                    d.campaign_experiment = null;
                                    d.extension_feed_item = null;
                                    d.operating_system_version_constant = null;
                                    d.mobile_app_category_constant = null;
                                    d.customer_negative_criterion = null;
                                    d.ad_schedule_view = null;
                                    d.media_file = null;
                                    d.domain_category = null;
                                    d.feed_placeholder_view = null;
                                    d.mobile_device_constant = null;
                                    d.segments = null;
                                    d.conversion_action = null;
                                    d.custom_interest = null;
                                    d.asset = null;
                                    d.dynamic_search_ads_search_term_view = null;
                                    d.ad_group_simulation = null;
                                    d.campaign_label = null;
                                    d.product_bidding_category_constant = null;
                                    d.ad_group_criterion_simulation = null;
                                    d.campaign_criterion_simulation = null;
                                    d.ad_group_extension_setting = null;
                                    d.campaign_extension_setting = null;
                                    d.customer_extension_setting = null;
                                    d.ad_group_label = null;
                                    d.feed_item_target = null;
                                    d.shopping_performance_view = null;
                                    d.detail_placement_view = null;
                                    d.group_placement_view = null;
                                    d.ad_group_ad_label = null;
                                    d.ad_group_criterion_label = null;
                                    d.click_view = null;
                                    d.location_view = null;
                                    d.customer_label = null;
                                    d.geographic_view = null;
                                    d.landing_page_view = null;
                                    d.expanded_landing_page_view = null;
                                    d.paid_organic_search_term_view = null;
                                    d.ad_parameter = null;
                                    d.ad_group_ad_asset_view = null;
                                    d.distance_view = null;
                                    d.experiment = null;
                                    d.currency_constant = null;
                                    d.user_location_view = null;
                                    d.offline_user_data_job = null;
                                    d.income_range_view = null;
                                    d.batch_job = null;
                                    d.keyword_plan_campaign_keyword = null;
                                    d.keyword_plan_ad_group_keyword = null;
                                    d.campaign_asset = null;
                                    d.account_link = null;
                                    d.third_party_app_analytics_link = null;
                                    d.change_event = null;
                                    d.customer_user_access = null;
                                    d.custom_audience = null;
                                    d.combined_audience = null;
                                    d.feed_item_set = null;
                                    d.customer_user_access_invitation = null;
                                    d.feed_item_set_link = null;
                                    d.call_view = null;
                                    d.conversion_custom_variable = null;
                                    d.ad_group_asset = null;
                                    d.customer_asset = null;
                                    d.campaign_simulation = null;
                                    d.bidding_strategy_simulation = null;
                                    d.bidding_data_exclusion = null;
                                    d.bidding_seasonality_adjustment = null;
                                    d.life_event = null;
                                    d.webpage_view = null;
                                    d.keyword_theme_constant = null;
                                    d.conversion_value_rule = null;
                                    d.conversion_value_rule_set = null;
                                    d.detailed_demographic = null;
                                    d.smart_campaign_setting = null;
                                    d.asset_field_type_view = null;
                                    d.accessible_bidding_strategy = null;
                                    d.smart_campaign_search_term_view = null;
                                    d.asset_group = null;
                                    d.asset_group_asset = null;
                                    d.customer_conversion_goal = null;
                                    d.campaign_conversion_goal = null;
                                    d.custom_conversion_goal = null;
                                    d.conversion_goal_campaign_config = null;
                                    d.customizer_attribute = null;
                                    d.asset_set = null;
                                    d.asset_set_asset = null;
                                    d.campaign_asset_set = null;
                                    d.asset_group_listing_group_filter = null;
                                    d.experiment_arm = null;
                                    d.customer_customizer = null;
                                    d.ad_group_customizer = null;
                                    d.campaign_customizer = null;
                                    d.ad_group_criterion_customizer = null;
                                    d.hotel_reconciliation = null;
                                    d.asset_group_product_group_view = null;
                                    d.audience = null;
                                    d.asset_group_signal = null;
                                    d.lead_form_submission_data = null;
                                    d.ad_group_ad_asset_combination_view = null;
                                }
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    d.customer = $root.google.ads.googleads.v11.resources.Customer.toObject(m.customer, o);
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = $root.google.ads.googleads.v11.resources.Campaign.toObject(m.campaign, o);
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = $root.google.ads.googleads.v11.resources.AdGroup.toObject(m.ad_group, o);
                                }
                                if (m.metrics != null && m.hasOwnProperty("metrics")) {
                                    d.metrics = $root.google.ads.googleads.v11.common.Metrics.toObject(m.metrics, o);
                                }
                                if (m.ad_group_ad != null && m.hasOwnProperty("ad_group_ad")) {
                                    d.ad_group_ad = $root.google.ads.googleads.v11.resources.AdGroupAd.toObject(m.ad_group_ad, o);
                                }
                                if (m.ad_group_criterion != null && m.hasOwnProperty("ad_group_criterion")) {
                                    d.ad_group_criterion = $root.google.ads.googleads.v11.resources.AdGroupCriterion.toObject(m.ad_group_criterion, o);
                                }
                                if (m.bidding_strategy != null && m.hasOwnProperty("bidding_strategy")) {
                                    d.bidding_strategy = $root.google.ads.googleads.v11.resources.BiddingStrategy.toObject(m.bidding_strategy, o);
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    d.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.toObject(m.campaign_budget, o);
                                }
                                if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                    d.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.toObject(m.campaign_criterion, o);
                                }
                                if (m.keyword_view != null && m.hasOwnProperty("keyword_view")) {
                                    d.keyword_view = $root.google.ads.googleads.v11.resources.KeywordView.toObject(m.keyword_view, o);
                                }
                                if (m.recommendation != null && m.hasOwnProperty("recommendation")) {
                                    d.recommendation = $root.google.ads.googleads.v11.resources.Recommendation.toObject(m.recommendation, o);
                                }
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    d.geo_target_constant = $root.google.ads.googleads.v11.resources.GeoTargetConstant.toObject(m.geo_target_constant, o);
                                }
                                if (m.ad_group_bid_modifier != null && m.hasOwnProperty("ad_group_bid_modifier")) {
                                    d.ad_group_bid_modifier = $root.google.ads.googleads.v11.resources.AdGroupBidModifier.toObject(m.ad_group_bid_modifier, o);
                                }
                                if (m.campaign_group != null && m.hasOwnProperty("campaign_group")) {
                                    d.campaign_group = $root.google.ads.googleads.v11.resources.CampaignGroup.toObject(m.campaign_group, o);
                                }
                                if (m.campaign_bid_modifier != null && m.hasOwnProperty("campaign_bid_modifier")) {
                                    d.campaign_bid_modifier = $root.google.ads.googleads.v11.resources.CampaignBidModifier.toObject(m.campaign_bid_modifier, o);
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    d.shared_set = $root.google.ads.googleads.v11.resources.SharedSet.toObject(m.shared_set, o);
                                }
                                if (m.shared_criterion != null && m.hasOwnProperty("shared_criterion")) {
                                    d.shared_criterion = $root.google.ads.googleads.v11.resources.SharedCriterion.toObject(m.shared_criterion, o);
                                }
                                if (m.campaign_shared_set != null && m.hasOwnProperty("campaign_shared_set")) {
                                    d.campaign_shared_set = $root.google.ads.googleads.v11.resources.CampaignSharedSet.toObject(m.campaign_shared_set, o);
                                }
                                if (m.topic_constant != null && m.hasOwnProperty("topic_constant")) {
                                    d.topic_constant = $root.google.ads.googleads.v11.resources.TopicConstant.toObject(m.topic_constant, o);
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    d.keyword_plan = $root.google.ads.googleads.v11.resources.KeywordPlan.toObject(m.keyword_plan, o);
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    d.keyword_plan_campaign = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.toObject(m.keyword_plan_campaign, o);
                                }
                                if (m.keyword_plan_ad_group != null && m.hasOwnProperty("keyword_plan_ad_group")) {
                                    d.keyword_plan_ad_group = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.toObject(m.keyword_plan_ad_group, o);
                                }
                                if (m.change_status != null && m.hasOwnProperty("change_status")) {
                                    d.change_status = $root.google.ads.googleads.v11.resources.ChangeStatus.toObject(m.change_status, o);
                                }
                                if (m.user_list != null && m.hasOwnProperty("user_list")) {
                                    d.user_list = $root.google.ads.googleads.v11.resources.UserList.toObject(m.user_list, o);
                                }
                                if (m.video != null && m.hasOwnProperty("video")) {
                                    d.video = $root.google.ads.googleads.v11.resources.Video.toObject(m.video, o);
                                }
                                if (m.gender_view != null && m.hasOwnProperty("gender_view")) {
                                    d.gender_view = $root.google.ads.googleads.v11.resources.GenderView.toObject(m.gender_view, o);
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    d.billing_setup = $root.google.ads.googleads.v11.resources.BillingSetup.toObject(m.billing_setup, o);
                                }
                                if (m.account_budget != null && m.hasOwnProperty("account_budget")) {
                                    d.account_budget = $root.google.ads.googleads.v11.resources.AccountBudget.toObject(m.account_budget, o);
                                }
                                if (m.account_budget_proposal != null && m.hasOwnProperty("account_budget_proposal")) {
                                    d.account_budget_proposal = $root.google.ads.googleads.v11.resources.AccountBudgetProposal.toObject(m.account_budget_proposal, o);
                                }
                                if (m.topic_view != null && m.hasOwnProperty("topic_view")) {
                                    d.topic_view = $root.google.ads.googleads.v11.resources.TopicView.toObject(m.topic_view, o);
                                }
                                if (m.parental_status_view != null && m.hasOwnProperty("parental_status_view")) {
                                    d.parental_status_view = $root.google.ads.googleads.v11.resources.ParentalStatusView.toObject(m.parental_status_view, o);
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = $root.google.ads.googleads.v11.resources.Feed.toObject(m.feed, o);
                                }
                                if (m.display_keyword_view != null && m.hasOwnProperty("display_keyword_view")) {
                                    d.display_keyword_view = $root.google.ads.googleads.v11.resources.DisplayKeywordView.toObject(m.display_keyword_view, o);
                                }
                                if (m.age_range_view != null && m.hasOwnProperty("age_range_view")) {
                                    d.age_range_view = $root.google.ads.googleads.v11.resources.AgeRangeView.toObject(m.age_range_view, o);
                                }
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    d.campaign_draft = $root.google.ads.googleads.v11.resources.CampaignDraft.toObject(m.campaign_draft, o);
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    d.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.toObject(m.feed_item, o);
                                }
                                if (m.hotel_group_view != null && m.hasOwnProperty("hotel_group_view")) {
                                    d.hotel_group_view = $root.google.ads.googleads.v11.resources.HotelGroupView.toObject(m.hotel_group_view, o);
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    d.label = $root.google.ads.googleads.v11.resources.Label.toObject(m.label, o);
                                }
                                if (m.managed_placement_view != null && m.hasOwnProperty("managed_placement_view")) {
                                    d.managed_placement_view = $root.google.ads.googleads.v11.resources.ManagedPlacementView.toObject(m.managed_placement_view, o);
                                }
                                if (m.product_group_view != null && m.hasOwnProperty("product_group_view")) {
                                    d.product_group_view = $root.google.ads.googleads.v11.resources.ProductGroupView.toObject(m.product_group_view, o);
                                }
                                if (m.language_constant != null && m.hasOwnProperty("language_constant")) {
                                    d.language_constant = $root.google.ads.googleads.v11.resources.LanguageConstant.toObject(m.language_constant, o);
                                }
                                if (m.ad_group_audience_view != null && m.hasOwnProperty("ad_group_audience_view")) {
                                    d.ad_group_audience_view = $root.google.ads.googleads.v11.resources.AdGroupAudienceView.toObject(m.ad_group_audience_view, o);
                                }
                                if (m.feed_mapping != null && m.hasOwnProperty("feed_mapping")) {
                                    d.feed_mapping = $root.google.ads.googleads.v11.resources.FeedMapping.toObject(m.feed_mapping, o);
                                }
                                if (m.user_interest != null && m.hasOwnProperty("user_interest")) {
                                    d.user_interest = $root.google.ads.googleads.v11.resources.UserInterest.toObject(m.user_interest, o);
                                }
                                if (m.remarketing_action != null && m.hasOwnProperty("remarketing_action")) {
                                    d.remarketing_action = $root.google.ads.googleads.v11.resources.RemarketingAction.toObject(m.remarketing_action, o);
                                }
                                if (m.customer_manager_link != null && m.hasOwnProperty("customer_manager_link")) {
                                    d.customer_manager_link = $root.google.ads.googleads.v11.resources.CustomerManagerLink.toObject(m.customer_manager_link, o);
                                }
                                if (m.customer_client_link != null && m.hasOwnProperty("customer_client_link")) {
                                    d.customer_client_link = $root.google.ads.googleads.v11.resources.CustomerClientLink.toObject(m.customer_client_link, o);
                                }
                                if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                    d.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.toObject(m.campaign_feed, o);
                                }
                                if (m.customer_feed != null && m.hasOwnProperty("customer_feed")) {
                                    d.customer_feed = $root.google.ads.googleads.v11.resources.CustomerFeed.toObject(m.customer_feed, o);
                                }
                                if (m.carrier_constant != null && m.hasOwnProperty("carrier_constant")) {
                                    d.carrier_constant = $root.google.ads.googleads.v11.resources.CarrierConstant.toObject(m.carrier_constant, o);
                                }
                                if (m.ad_group_feed != null && m.hasOwnProperty("ad_group_feed")) {
                                    d.ad_group_feed = $root.google.ads.googleads.v11.resources.AdGroupFeed.toObject(m.ad_group_feed, o);
                                }
                                if (m.search_term_view != null && m.hasOwnProperty("search_term_view")) {
                                    d.search_term_view = $root.google.ads.googleads.v11.resources.SearchTermView.toObject(m.search_term_view, o);
                                }
                                if (m.campaign_audience_view != null && m.hasOwnProperty("campaign_audience_view")) {
                                    d.campaign_audience_view = $root.google.ads.googleads.v11.resources.CampaignAudienceView.toObject(m.campaign_audience_view, o);
                                }
                                if (m.customer_client != null && m.hasOwnProperty("customer_client")) {
                                    d.customer_client = $root.google.ads.googleads.v11.resources.CustomerClient.toObject(m.customer_client, o);
                                }
                                if (m.hotel_performance_view != null && m.hasOwnProperty("hotel_performance_view")) {
                                    d.hotel_performance_view = $root.google.ads.googleads.v11.resources.HotelPerformanceView.toObject(m.hotel_performance_view, o);
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    d.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.toObject(m.campaign_experiment, o);
                                }
                                if (m.extension_feed_item != null && m.hasOwnProperty("extension_feed_item")) {
                                    d.extension_feed_item = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.toObject(m.extension_feed_item, o);
                                }
                                if (m.operating_system_version_constant != null && m.hasOwnProperty("operating_system_version_constant")) {
                                    d.operating_system_version_constant = $root.google.ads.googleads.v11.resources.OperatingSystemVersionConstant.toObject(m.operating_system_version_constant, o);
                                }
                                if (m.mobile_app_category_constant != null && m.hasOwnProperty("mobile_app_category_constant")) {
                                    d.mobile_app_category_constant = $root.google.ads.googleads.v11.resources.MobileAppCategoryConstant.toObject(m.mobile_app_category_constant, o);
                                }
                                if (m.customer_negative_criterion != null && m.hasOwnProperty("customer_negative_criterion")) {
                                    d.customer_negative_criterion = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.toObject(m.customer_negative_criterion, o);
                                }
                                if (m.ad_schedule_view != null && m.hasOwnProperty("ad_schedule_view")) {
                                    d.ad_schedule_view = $root.google.ads.googleads.v11.resources.AdScheduleView.toObject(m.ad_schedule_view, o);
                                }
                                if (m.media_file != null && m.hasOwnProperty("media_file")) {
                                    d.media_file = $root.google.ads.googleads.v11.resources.MediaFile.toObject(m.media_file, o);
                                }
                                if (m.domain_category != null && m.hasOwnProperty("domain_category")) {
                                    d.domain_category = $root.google.ads.googleads.v11.resources.DomainCategory.toObject(m.domain_category, o);
                                }
                                if (m.feed_placeholder_view != null && m.hasOwnProperty("feed_placeholder_view")) {
                                    d.feed_placeholder_view = $root.google.ads.googleads.v11.resources.FeedPlaceholderView.toObject(m.feed_placeholder_view, o);
                                }
                                if (m.mobile_device_constant != null && m.hasOwnProperty("mobile_device_constant")) {
                                    d.mobile_device_constant = $root.google.ads.googleads.v11.resources.MobileDeviceConstant.toObject(m.mobile_device_constant, o);
                                }
                                if (m.segments != null && m.hasOwnProperty("segments")) {
                                    d.segments = $root.google.ads.googleads.v11.common.Segments.toObject(m.segments, o);
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = $root.google.ads.googleads.v11.resources.ConversionAction.toObject(m.conversion_action, o);
                                }
                                if (m.custom_interest != null && m.hasOwnProperty("custom_interest")) {
                                    d.custom_interest = $root.google.ads.googleads.v11.resources.CustomInterest.toObject(m.custom_interest, o);
                                }
                                if (m.asset != null && m.hasOwnProperty("asset")) {
                                    d.asset = $root.google.ads.googleads.v11.resources.Asset.toObject(m.asset, o);
                                }
                                if (m.dynamic_search_ads_search_term_view != null && m.hasOwnProperty("dynamic_search_ads_search_term_view")) {
                                    d.dynamic_search_ads_search_term_view = $root.google.ads.googleads.v11.resources.DynamicSearchAdsSearchTermView.toObject(m.dynamic_search_ads_search_term_view, o);
                                }
                                if (m.ad_group_simulation != null && m.hasOwnProperty("ad_group_simulation")) {
                                    d.ad_group_simulation = $root.google.ads.googleads.v11.resources.AdGroupSimulation.toObject(m.ad_group_simulation, o);
                                }
                                if (m.campaign_label != null && m.hasOwnProperty("campaign_label")) {
                                    d.campaign_label = $root.google.ads.googleads.v11.resources.CampaignLabel.toObject(m.campaign_label, o);
                                }
                                if (m.product_bidding_category_constant != null && m.hasOwnProperty("product_bidding_category_constant")) {
                                    d.product_bidding_category_constant = $root.google.ads.googleads.v11.resources.ProductBiddingCategoryConstant.toObject(m.product_bidding_category_constant, o);
                                }
                                if (m.ad_group_criterion_simulation != null && m.hasOwnProperty("ad_group_criterion_simulation")) {
                                    d.ad_group_criterion_simulation = $root.google.ads.googleads.v11.resources.AdGroupCriterionSimulation.toObject(m.ad_group_criterion_simulation, o);
                                }
                                if (m.campaign_criterion_simulation != null && m.hasOwnProperty("campaign_criterion_simulation")) {
                                    d.campaign_criterion_simulation = $root.google.ads.googleads.v11.resources.CampaignCriterionSimulation.toObject(m.campaign_criterion_simulation, o);
                                }
                                if (m.ad_group_extension_setting != null && m.hasOwnProperty("ad_group_extension_setting")) {
                                    d.ad_group_extension_setting = $root.google.ads.googleads.v11.resources.AdGroupExtensionSetting.toObject(m.ad_group_extension_setting, o);
                                }
                                if (m.campaign_extension_setting != null && m.hasOwnProperty("campaign_extension_setting")) {
                                    d.campaign_extension_setting = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.toObject(m.campaign_extension_setting, o);
                                }
                                if (m.customer_extension_setting != null && m.hasOwnProperty("customer_extension_setting")) {
                                    d.customer_extension_setting = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.toObject(m.customer_extension_setting, o);
                                }
                                if (m.ad_group_label != null && m.hasOwnProperty("ad_group_label")) {
                                    d.ad_group_label = $root.google.ads.googleads.v11.resources.AdGroupLabel.toObject(m.ad_group_label, o);
                                }
                                if (m.feed_item_target != null && m.hasOwnProperty("feed_item_target")) {
                                    d.feed_item_target = $root.google.ads.googleads.v11.resources.FeedItemTarget.toObject(m.feed_item_target, o);
                                }
                                if (m.shopping_performance_view != null && m.hasOwnProperty("shopping_performance_view")) {
                                    d.shopping_performance_view = $root.google.ads.googleads.v11.resources.ShoppingPerformanceView.toObject(m.shopping_performance_view, o);
                                }
                                if (m.detail_placement_view != null && m.hasOwnProperty("detail_placement_view")) {
                                    d.detail_placement_view = $root.google.ads.googleads.v11.resources.DetailPlacementView.toObject(m.detail_placement_view, o);
                                }
                                if (m.group_placement_view != null && m.hasOwnProperty("group_placement_view")) {
                                    d.group_placement_view = $root.google.ads.googleads.v11.resources.GroupPlacementView.toObject(m.group_placement_view, o);
                                }
                                if (m.ad_group_ad_label != null && m.hasOwnProperty("ad_group_ad_label")) {
                                    d.ad_group_ad_label = $root.google.ads.googleads.v11.resources.AdGroupAdLabel.toObject(m.ad_group_ad_label, o);
                                }
                                if (m.ad_group_criterion_label != null && m.hasOwnProperty("ad_group_criterion_label")) {
                                    d.ad_group_criterion_label = $root.google.ads.googleads.v11.resources.AdGroupCriterionLabel.toObject(m.ad_group_criterion_label, o);
                                }
                                if (m.click_view != null && m.hasOwnProperty("click_view")) {
                                    d.click_view = $root.google.ads.googleads.v11.resources.ClickView.toObject(m.click_view, o);
                                }
                                if (m.location_view != null && m.hasOwnProperty("location_view")) {
                                    d.location_view = $root.google.ads.googleads.v11.resources.LocationView.toObject(m.location_view, o);
                                }
                                if (m.customer_label != null && m.hasOwnProperty("customer_label")) {
                                    d.customer_label = $root.google.ads.googleads.v11.resources.CustomerLabel.toObject(m.customer_label, o);
                                }
                                if (m.geographic_view != null && m.hasOwnProperty("geographic_view")) {
                                    d.geographic_view = $root.google.ads.googleads.v11.resources.GeographicView.toObject(m.geographic_view, o);
                                }
                                if (m.landing_page_view != null && m.hasOwnProperty("landing_page_view")) {
                                    d.landing_page_view = $root.google.ads.googleads.v11.resources.LandingPageView.toObject(m.landing_page_view, o);
                                }
                                if (m.expanded_landing_page_view != null && m.hasOwnProperty("expanded_landing_page_view")) {
                                    d.expanded_landing_page_view = $root.google.ads.googleads.v11.resources.ExpandedLandingPageView.toObject(m.expanded_landing_page_view, o);
                                }
                                if (m.paid_organic_search_term_view != null && m.hasOwnProperty("paid_organic_search_term_view")) {
                                    d.paid_organic_search_term_view = $root.google.ads.googleads.v11.resources.PaidOrganicSearchTermView.toObject(m.paid_organic_search_term_view, o);
                                }
                                if (m.ad_parameter != null && m.hasOwnProperty("ad_parameter")) {
                                    d.ad_parameter = $root.google.ads.googleads.v11.resources.AdParameter.toObject(m.ad_parameter, o);
                                }
                                if (m.ad_group_ad_asset_view != null && m.hasOwnProperty("ad_group_ad_asset_view")) {
                                    d.ad_group_ad_asset_view = $root.google.ads.googleads.v11.resources.AdGroupAdAssetView.toObject(m.ad_group_ad_asset_view, o);
                                }
                                if (m.distance_view != null && m.hasOwnProperty("distance_view")) {
                                    d.distance_view = $root.google.ads.googleads.v11.resources.DistanceView.toObject(m.distance_view, o);
                                }
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    d.experiment = $root.google.ads.googleads.v11.resources.Experiment.toObject(m.experiment, o);
                                }
                                if (m.currency_constant != null && m.hasOwnProperty("currency_constant")) {
                                    d.currency_constant = $root.google.ads.googleads.v11.resources.CurrencyConstant.toObject(m.currency_constant, o);
                                }
                                if (m.user_location_view != null && m.hasOwnProperty("user_location_view")) {
                                    d.user_location_view = $root.google.ads.googleads.v11.resources.UserLocationView.toObject(m.user_location_view, o);
                                }
                                if (m.offline_user_data_job != null && m.hasOwnProperty("offline_user_data_job")) {
                                    d.offline_user_data_job = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.toObject(m.offline_user_data_job, o);
                                }
                                if (m.income_range_view != null && m.hasOwnProperty("income_range_view")) {
                                    d.income_range_view = $root.google.ads.googleads.v11.resources.IncomeRangeView.toObject(m.income_range_view, o);
                                }
                                if (m.batch_job != null && m.hasOwnProperty("batch_job")) {
                                    d.batch_job = $root.google.ads.googleads.v11.resources.BatchJob.toObject(m.batch_job, o);
                                }
                                if (m.keyword_plan_campaign_keyword != null && m.hasOwnProperty("keyword_plan_campaign_keyword")) {
                                    d.keyword_plan_campaign_keyword = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.toObject(m.keyword_plan_campaign_keyword, o);
                                }
                                if (m.keyword_plan_ad_group_keyword != null && m.hasOwnProperty("keyword_plan_ad_group_keyword")) {
                                    d.keyword_plan_ad_group_keyword = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.toObject(m.keyword_plan_ad_group_keyword, o);
                                }
                                if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                    d.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.toObject(m.campaign_asset, o);
                                }
                                if (m.account_link != null && m.hasOwnProperty("account_link")) {
                                    d.account_link = $root.google.ads.googleads.v11.resources.AccountLink.toObject(m.account_link, o);
                                }
                                if (m.third_party_app_analytics_link != null && m.hasOwnProperty("third_party_app_analytics_link")) {
                                    d.third_party_app_analytics_link = $root.google.ads.googleads.v11.resources.ThirdPartyAppAnalyticsLink.toObject(m.third_party_app_analytics_link, o);
                                }
                                if (m.change_event != null && m.hasOwnProperty("change_event")) {
                                    d.change_event = $root.google.ads.googleads.v11.resources.ChangeEvent.toObject(m.change_event, o);
                                }
                                if (m.customer_user_access != null && m.hasOwnProperty("customer_user_access")) {
                                    d.customer_user_access = $root.google.ads.googleads.v11.resources.CustomerUserAccess.toObject(m.customer_user_access, o);
                                }
                                if (m.custom_audience != null && m.hasOwnProperty("custom_audience")) {
                                    d.custom_audience = $root.google.ads.googleads.v11.resources.CustomAudience.toObject(m.custom_audience, o);
                                }
                                if (m.combined_audience != null && m.hasOwnProperty("combined_audience")) {
                                    d.combined_audience = $root.google.ads.googleads.v11.resources.CombinedAudience.toObject(m.combined_audience, o);
                                }
                                if (m.feed_item_set != null && m.hasOwnProperty("feed_item_set")) {
                                    d.feed_item_set = $root.google.ads.googleads.v11.resources.FeedItemSet.toObject(m.feed_item_set, o);
                                }
                                if (m.customer_user_access_invitation != null && m.hasOwnProperty("customer_user_access_invitation")) {
                                    d.customer_user_access_invitation = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.toObject(m.customer_user_access_invitation, o);
                                }
                                if (m.feed_item_set_link != null && m.hasOwnProperty("feed_item_set_link")) {
                                    d.feed_item_set_link = $root.google.ads.googleads.v11.resources.FeedItemSetLink.toObject(m.feed_item_set_link, o);
                                }
                                if (m.call_view != null && m.hasOwnProperty("call_view")) {
                                    d.call_view = $root.google.ads.googleads.v11.resources.CallView.toObject(m.call_view, o);
                                }
                                if (m.conversion_custom_variable != null && m.hasOwnProperty("conversion_custom_variable")) {
                                    d.conversion_custom_variable = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.toObject(m.conversion_custom_variable, o);
                                }
                                if (m.ad_group_asset != null && m.hasOwnProperty("ad_group_asset")) {
                                    d.ad_group_asset = $root.google.ads.googleads.v11.resources.AdGroupAsset.toObject(m.ad_group_asset, o);
                                }
                                if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                    d.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.toObject(m.customer_asset, o);
                                }
                                if (m.campaign_simulation != null && m.hasOwnProperty("campaign_simulation")) {
                                    d.campaign_simulation = $root.google.ads.googleads.v11.resources.CampaignSimulation.toObject(m.campaign_simulation, o);
                                }
                                if (m.bidding_strategy_simulation != null && m.hasOwnProperty("bidding_strategy_simulation")) {
                                    d.bidding_strategy_simulation = $root.google.ads.googleads.v11.resources.BiddingStrategySimulation.toObject(m.bidding_strategy_simulation, o);
                                }
                                if (m.bidding_data_exclusion != null && m.hasOwnProperty("bidding_data_exclusion")) {
                                    d.bidding_data_exclusion = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.toObject(m.bidding_data_exclusion, o);
                                }
                                if (m.bidding_seasonality_adjustment != null && m.hasOwnProperty("bidding_seasonality_adjustment")) {
                                    d.bidding_seasonality_adjustment = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.toObject(m.bidding_seasonality_adjustment, o);
                                }
                                if (m.life_event != null && m.hasOwnProperty("life_event")) {
                                    d.life_event = $root.google.ads.googleads.v11.resources.LifeEvent.toObject(m.life_event, o);
                                }
                                if (m.webpage_view != null && m.hasOwnProperty("webpage_view")) {
                                    d.webpage_view = $root.google.ads.googleads.v11.resources.WebpageView.toObject(m.webpage_view, o);
                                }
                                if (m.keyword_theme_constant != null && m.hasOwnProperty("keyword_theme_constant")) {
                                    d.keyword_theme_constant = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.toObject(m.keyword_theme_constant, o);
                                }
                                if (m.conversion_value_rule != null && m.hasOwnProperty("conversion_value_rule")) {
                                    d.conversion_value_rule = $root.google.ads.googleads.v11.resources.ConversionValueRule.toObject(m.conversion_value_rule, o);
                                }
                                if (m.conversion_value_rule_set != null && m.hasOwnProperty("conversion_value_rule_set")) {
                                    d.conversion_value_rule_set = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.toObject(m.conversion_value_rule_set, o);
                                }
                                if (m.detailed_demographic != null && m.hasOwnProperty("detailed_demographic")) {
                                    d.detailed_demographic = $root.google.ads.googleads.v11.resources.DetailedDemographic.toObject(m.detailed_demographic, o);
                                }
                                if (m.smart_campaign_setting != null && m.hasOwnProperty("smart_campaign_setting")) {
                                    d.smart_campaign_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.toObject(m.smart_campaign_setting, o);
                                }
                                if (m.asset_field_type_view != null && m.hasOwnProperty("asset_field_type_view")) {
                                    d.asset_field_type_view = $root.google.ads.googleads.v11.resources.AssetFieldTypeView.toObject(m.asset_field_type_view, o);
                                }
                                if (m.accessible_bidding_strategy != null && m.hasOwnProperty("accessible_bidding_strategy")) {
                                    d.accessible_bidding_strategy = $root.google.ads.googleads.v11.resources.AccessibleBiddingStrategy.toObject(m.accessible_bidding_strategy, o);
                                }
                                if (m.smart_campaign_search_term_view != null && m.hasOwnProperty("smart_campaign_search_term_view")) {
                                    d.smart_campaign_search_term_view = $root.google.ads.googleads.v11.resources.SmartCampaignSearchTermView.toObject(m.smart_campaign_search_term_view, o);
                                }
                                if (m.asset_group != null && m.hasOwnProperty("asset_group")) {
                                    d.asset_group = $root.google.ads.googleads.v11.resources.AssetGroup.toObject(m.asset_group, o);
                                }
                                if (m.asset_group_asset != null && m.hasOwnProperty("asset_group_asset")) {
                                    d.asset_group_asset = $root.google.ads.googleads.v11.resources.AssetGroupAsset.toObject(m.asset_group_asset, o);
                                }
                                if (m.customer_conversion_goal != null && m.hasOwnProperty("customer_conversion_goal")) {
                                    d.customer_conversion_goal = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.toObject(m.customer_conversion_goal, o);
                                }
                                if (m.campaign_conversion_goal != null && m.hasOwnProperty("campaign_conversion_goal")) {
                                    d.campaign_conversion_goal = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.toObject(m.campaign_conversion_goal, o);
                                }
                                if (m.custom_conversion_goal != null && m.hasOwnProperty("custom_conversion_goal")) {
                                    d.custom_conversion_goal = $root.google.ads.googleads.v11.resources.CustomConversionGoal.toObject(m.custom_conversion_goal, o);
                                }
                                if (m.conversion_goal_campaign_config != null && m.hasOwnProperty("conversion_goal_campaign_config")) {
                                    d.conversion_goal_campaign_config = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.toObject(m.conversion_goal_campaign_config, o);
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    d.customizer_attribute = $root.google.ads.googleads.v11.resources.CustomizerAttribute.toObject(m.customizer_attribute, o);
                                }
                                if (m.asset_set != null && m.hasOwnProperty("asset_set")) {
                                    d.asset_set = $root.google.ads.googleads.v11.resources.AssetSet.toObject(m.asset_set, o);
                                }
                                if (m.asset_set_asset != null && m.hasOwnProperty("asset_set_asset")) {
                                    d.asset_set_asset = $root.google.ads.googleads.v11.resources.AssetSetAsset.toObject(m.asset_set_asset, o);
                                }
                                if (m.campaign_asset_set != null && m.hasOwnProperty("campaign_asset_set")) {
                                    d.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.toObject(m.campaign_asset_set, o);
                                }
                                if (m.asset_group_listing_group_filter != null && m.hasOwnProperty("asset_group_listing_group_filter")) {
                                    d.asset_group_listing_group_filter = $root.google.ads.googleads.v11.resources.AssetGroupListingGroupFilter.toObject(m.asset_group_listing_group_filter, o);
                                }
                                if (m.experiment_arm != null && m.hasOwnProperty("experiment_arm")) {
                                    d.experiment_arm = $root.google.ads.googleads.v11.resources.ExperimentArm.toObject(m.experiment_arm, o);
                                }
                                if (m.customer_customizer != null && m.hasOwnProperty("customer_customizer")) {
                                    d.customer_customizer = $root.google.ads.googleads.v11.resources.CustomerCustomizer.toObject(m.customer_customizer, o);
                                }
                                if (m.ad_group_customizer != null && m.hasOwnProperty("ad_group_customizer")) {
                                    d.ad_group_customizer = $root.google.ads.googleads.v11.resources.AdGroupCustomizer.toObject(m.ad_group_customizer, o);
                                }
                                if (m.campaign_customizer != null && m.hasOwnProperty("campaign_customizer")) {
                                    d.campaign_customizer = $root.google.ads.googleads.v11.resources.CampaignCustomizer.toObject(m.campaign_customizer, o);
                                }
                                if (m.ad_group_criterion_customizer != null && m.hasOwnProperty("ad_group_criterion_customizer")) {
                                    d.ad_group_criterion_customizer = $root.google.ads.googleads.v11.resources.AdGroupCriterionCustomizer.toObject(m.ad_group_criterion_customizer, o);
                                }
                                if (m.hotel_reconciliation != null && m.hasOwnProperty("hotel_reconciliation")) {
                                    d.hotel_reconciliation = $root.google.ads.googleads.v11.resources.HotelReconciliation.toObject(m.hotel_reconciliation, o);
                                }
                                if (m.asset_group_product_group_view != null && m.hasOwnProperty("asset_group_product_group_view")) {
                                    d.asset_group_product_group_view = $root.google.ads.googleads.v11.resources.AssetGroupProductGroupView.toObject(m.asset_group_product_group_view, o);
                                }
                                if (m.audience != null && m.hasOwnProperty("audience")) {
                                    d.audience = $root.google.ads.googleads.v11.resources.Audience.toObject(m.audience, o);
                                }
                                if (m.asset_group_signal != null && m.hasOwnProperty("asset_group_signal")) {
                                    d.asset_group_signal = $root.google.ads.googleads.v11.resources.AssetGroupSignal.toObject(m.asset_group_signal, o);
                                }
                                if (m.lead_form_submission_data != null && m.hasOwnProperty("lead_form_submission_data")) {
                                    d.lead_form_submission_data = $root.google.ads.googleads.v11.resources.LeadFormSubmissionData.toObject(m.lead_form_submission_data, o);
                                }
                                if (m.ad_group_ad_asset_combination_view != null && m.hasOwnProperty("ad_group_ad_asset_combination_view")) {
                                    d.ad_group_ad_asset_combination_view = $root.google.ads.googleads.v11.resources.AdGroupAdAssetCombinationView.toObject(m.ad_group_ad_asset_combination_view, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GoogleAdsRow to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GoogleAdsRow
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GoogleAdsRow.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GoogleAdsRow;
                        })();
    
                        services.MutateGoogleAdsRequest = (function() {
    
                            /**
                             * Properties of a MutateGoogleAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateGoogleAdsRequest
                             * @property {string|null} [customer_id] MutateGoogleAdsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IMutateOperation>|null} [mutate_operations] MutateGoogleAdsRequest mutate_operations
                             * @property {boolean|null} [partial_failure] MutateGoogleAdsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateGoogleAdsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateGoogleAdsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateGoogleAdsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateGoogleAdsRequest.
                             * @implements IMutateGoogleAdsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsRequest=} [p] Properties to set
                             */
                            function MutateGoogleAdsRequest(p) {
                                this.mutate_operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateGoogleAdsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @instance
                             */
                            MutateGoogleAdsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateGoogleAdsRequest mutate_operations.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateOperation>} mutate_operations
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @instance
                             */
                            MutateGoogleAdsRequest.prototype.mutate_operations = $util.emptyArray;
    
                            /**
                             * MutateGoogleAdsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @instance
                             */
                            MutateGoogleAdsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateGoogleAdsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @instance
                             */
                            MutateGoogleAdsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateGoogleAdsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @instance
                             */
                            MutateGoogleAdsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateGoogleAdsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsRequest} MutateGoogleAdsRequest instance
                             */
                            MutateGoogleAdsRequest.create = function create(properties) {
                                return new MutateGoogleAdsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateGoogleAdsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateGoogleAdsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsRequest} m MutateGoogleAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateGoogleAdsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.mutate_operations != null && m.mutate_operations.length) {
                                    for (var i = 0; i < m.mutate_operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateOperation.encode(m.mutate_operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateGoogleAdsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateGoogleAdsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsRequest} message MutateGoogleAdsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateGoogleAdsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateGoogleAdsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsRequest} MutateGoogleAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateGoogleAdsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateGoogleAdsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.mutate_operations && m.mutate_operations.length))
                                            m.mutate_operations = [];
                                        m.mutate_operations.push($root.google.ads.googleads.v11.services.MutateOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateGoogleAdsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsRequest} MutateGoogleAdsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateGoogleAdsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateGoogleAdsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateGoogleAdsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.mutate_operations != null && m.hasOwnProperty("mutate_operations")) {
                                    if (!Array.isArray(m.mutate_operations))
                                        return "mutate_operations: array expected";
                                    for (var i = 0; i < m.mutate_operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateOperation.verify(m.mutate_operations[i]);
                                            if (e)
                                                return "mutate_operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateGoogleAdsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsRequest} MutateGoogleAdsRequest
                             */
                            MutateGoogleAdsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateGoogleAdsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateGoogleAdsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.mutate_operations) {
                                    if (!Array.isArray(d.mutate_operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateGoogleAdsRequest.mutate_operations: array expected");
                                    m.mutate_operations = [];
                                    for (var i = 0; i < d.mutate_operations.length; ++i) {
                                        if (typeof d.mutate_operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateGoogleAdsRequest.mutate_operations: object expected");
                                        m.mutate_operations[i] = $root.google.ads.googleads.v11.services.MutateOperation.fromObject(d.mutate_operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateGoogleAdsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateGoogleAdsRequest} m MutateGoogleAdsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateGoogleAdsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.mutate_operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.mutate_operations && m.mutate_operations.length) {
                                    d.mutate_operations = [];
                                    for (var j = 0; j < m.mutate_operations.length; ++j) {
                                        d.mutate_operations[j] = $root.google.ads.googleads.v11.services.MutateOperation.toObject(m.mutate_operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateGoogleAdsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateGoogleAdsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateGoogleAdsRequest;
                        })();
    
                        services.MutateGoogleAdsResponse = (function() {
    
                            /**
                             * Properties of a MutateGoogleAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateGoogleAdsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateGoogleAdsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateOperationResponse>|null} [mutate_operation_responses] MutateGoogleAdsResponse mutate_operation_responses
                             */
    
                            /**
                             * Constructs a new MutateGoogleAdsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateGoogleAdsResponse.
                             * @implements IMutateGoogleAdsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsResponse=} [p] Properties to set
                             */
                            function MutateGoogleAdsResponse(p) {
                                this.mutate_operation_responses = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateGoogleAdsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @instance
                             */
                            MutateGoogleAdsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateGoogleAdsResponse mutate_operation_responses.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateOperationResponse>} mutate_operation_responses
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @instance
                             */
                            MutateGoogleAdsResponse.prototype.mutate_operation_responses = $util.emptyArray;
    
                            /**
                             * Creates a new MutateGoogleAdsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsResponse} MutateGoogleAdsResponse instance
                             */
                            MutateGoogleAdsResponse.create = function create(properties) {
                                return new MutateGoogleAdsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateGoogleAdsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateGoogleAdsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsResponse} m MutateGoogleAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateGoogleAdsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.mutate_operation_responses != null && m.mutate_operation_responses.length) {
                                    for (var i = 0; i < m.mutate_operation_responses.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateOperationResponse.encode(m.mutate_operation_responses[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateGoogleAdsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateGoogleAdsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateGoogleAdsResponse} message MutateGoogleAdsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateGoogleAdsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateGoogleAdsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsResponse} MutateGoogleAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateGoogleAdsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateGoogleAdsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        if (!(m.mutate_operation_responses && m.mutate_operation_responses.length))
                                            m.mutate_operation_responses = [];
                                        m.mutate_operation_responses.push($root.google.ads.googleads.v11.services.MutateOperationResponse.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateGoogleAdsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsResponse} MutateGoogleAdsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateGoogleAdsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateGoogleAdsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateGoogleAdsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.mutate_operation_responses != null && m.hasOwnProperty("mutate_operation_responses")) {
                                    if (!Array.isArray(m.mutate_operation_responses))
                                        return "mutate_operation_responses: array expected";
                                    for (var i = 0; i < m.mutate_operation_responses.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateOperationResponse.verify(m.mutate_operation_responses[i]);
                                            if (e)
                                                return "mutate_operation_responses." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateGoogleAdsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateGoogleAdsResponse} MutateGoogleAdsResponse
                             */
                            MutateGoogleAdsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateGoogleAdsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateGoogleAdsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateGoogleAdsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.mutate_operation_responses) {
                                    if (!Array.isArray(d.mutate_operation_responses))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateGoogleAdsResponse.mutate_operation_responses: array expected");
                                    m.mutate_operation_responses = [];
                                    for (var i = 0; i < d.mutate_operation_responses.length; ++i) {
                                        if (typeof d.mutate_operation_responses[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateGoogleAdsResponse.mutate_operation_responses: object expected");
                                        m.mutate_operation_responses[i] = $root.google.ads.googleads.v11.services.MutateOperationResponse.fromObject(d.mutate_operation_responses[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateGoogleAdsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateGoogleAdsResponse} m MutateGoogleAdsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateGoogleAdsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.mutate_operation_responses = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.mutate_operation_responses && m.mutate_operation_responses.length) {
                                    d.mutate_operation_responses = [];
                                    for (var j = 0; j < m.mutate_operation_responses.length; ++j) {
                                        d.mutate_operation_responses[j] = $root.google.ads.googleads.v11.services.MutateOperationResponse.toObject(m.mutate_operation_responses[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateGoogleAdsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateGoogleAdsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateGoogleAdsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateGoogleAdsResponse;
                        })();
    
                        services.MutateOperation = (function() {
    
                            /**
                             * Properties of a MutateOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateOperation
                             * @property {google.ads.googleads.v11.services.IAdGroupAdLabelOperation|null} [ad_group_ad_label_operation] MutateOperation ad_group_ad_label_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupAdOperation|null} [ad_group_ad_operation] MutateOperation ad_group_ad_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupAssetOperation|null} [ad_group_asset_operation] MutateOperation ad_group_asset_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupBidModifierOperation|null} [ad_group_bid_modifier_operation] MutateOperation ad_group_bid_modifier_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation|null} [ad_group_criterion_customizer_operation] MutateOperation ad_group_criterion_customizer_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation|null} [ad_group_criterion_label_operation] MutateOperation ad_group_criterion_label_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupCriterionOperation|null} [ad_group_criterion_operation] MutateOperation ad_group_criterion_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupCustomizerOperation|null} [ad_group_customizer_operation] MutateOperation ad_group_customizer_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation|null} [ad_group_extension_setting_operation] MutateOperation ad_group_extension_setting_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupFeedOperation|null} [ad_group_feed_operation] MutateOperation ad_group_feed_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupLabelOperation|null} [ad_group_label_operation] MutateOperation ad_group_label_operation
                             * @property {google.ads.googleads.v11.services.IAdGroupOperation|null} [ad_group_operation] MutateOperation ad_group_operation
                             * @property {google.ads.googleads.v11.services.IAdOperation|null} [ad_operation] MutateOperation ad_operation
                             * @property {google.ads.googleads.v11.services.IAdParameterOperation|null} [ad_parameter_operation] MutateOperation ad_parameter_operation
                             * @property {google.ads.googleads.v11.services.IAssetOperation|null} [asset_operation] MutateOperation asset_operation
                             * @property {google.ads.googleads.v11.services.IAssetGroupAssetOperation|null} [asset_group_asset_operation] MutateOperation asset_group_asset_operation
                             * @property {google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation|null} [asset_group_listing_group_filter_operation] MutateOperation asset_group_listing_group_filter_operation
                             * @property {google.ads.googleads.v11.services.IAssetGroupSignalOperation|null} [asset_group_signal_operation] MutateOperation asset_group_signal_operation
                             * @property {google.ads.googleads.v11.services.IAssetGroupOperation|null} [asset_group_operation] MutateOperation asset_group_operation
                             * @property {google.ads.googleads.v11.services.IAssetSetAssetOperation|null} [asset_set_asset_operation] MutateOperation asset_set_asset_operation
                             * @property {google.ads.googleads.v11.services.IAssetSetOperation|null} [asset_set_operation] MutateOperation asset_set_operation
                             * @property {google.ads.googleads.v11.services.IAudienceOperation|null} [audience_operation] MutateOperation audience_operation
                             * @property {google.ads.googleads.v11.services.IBiddingDataExclusionOperation|null} [bidding_data_exclusion_operation] MutateOperation bidding_data_exclusion_operation
                             * @property {google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation|null} [bidding_seasonality_adjustment_operation] MutateOperation bidding_seasonality_adjustment_operation
                             * @property {google.ads.googleads.v11.services.IBiddingStrategyOperation|null} [bidding_strategy_operation] MutateOperation bidding_strategy_operation
                             * @property {google.ads.googleads.v11.services.ICampaignAssetOperation|null} [campaign_asset_operation] MutateOperation campaign_asset_operation
                             * @property {google.ads.googleads.v11.services.ICampaignAssetSetOperation|null} [campaign_asset_set_operation] MutateOperation campaign_asset_set_operation
                             * @property {google.ads.googleads.v11.services.ICampaignBidModifierOperation|null} [campaign_bid_modifier_operation] MutateOperation campaign_bid_modifier_operation
                             * @property {google.ads.googleads.v11.services.ICampaignBudgetOperation|null} [campaign_budget_operation] MutateOperation campaign_budget_operation
                             * @property {google.ads.googleads.v11.services.ICampaignConversionGoalOperation|null} [campaign_conversion_goal_operation] MutateOperation campaign_conversion_goal_operation
                             * @property {google.ads.googleads.v11.services.ICampaignCriterionOperation|null} [campaign_criterion_operation] MutateOperation campaign_criterion_operation
                             * @property {google.ads.googleads.v11.services.ICampaignCustomizerOperation|null} [campaign_customizer_operation] MutateOperation campaign_customizer_operation
                             * @property {google.ads.googleads.v11.services.ICampaignDraftOperation|null} [campaign_draft_operation] MutateOperation campaign_draft_operation
                             * @property {google.ads.googleads.v11.services.ICampaignExperimentOperation|null} [campaign_experiment_operation] MutateOperation campaign_experiment_operation
                             * @property {google.ads.googleads.v11.services.ICampaignExtensionSettingOperation|null} [campaign_extension_setting_operation] MutateOperation campaign_extension_setting_operation
                             * @property {google.ads.googleads.v11.services.ICampaignFeedOperation|null} [campaign_feed_operation] MutateOperation campaign_feed_operation
                             * @property {google.ads.googleads.v11.services.ICampaignGroupOperation|null} [campaign_group_operation] MutateOperation campaign_group_operation
                             * @property {google.ads.googleads.v11.services.ICampaignLabelOperation|null} [campaign_label_operation] MutateOperation campaign_label_operation
                             * @property {google.ads.googleads.v11.services.ICampaignOperation|null} [campaign_operation] MutateOperation campaign_operation
                             * @property {google.ads.googleads.v11.services.ICampaignSharedSetOperation|null} [campaign_shared_set_operation] MutateOperation campaign_shared_set_operation
                             * @property {google.ads.googleads.v11.services.IConversionActionOperation|null} [conversion_action_operation] MutateOperation conversion_action_operation
                             * @property {google.ads.googleads.v11.services.IConversionCustomVariableOperation|null} [conversion_custom_variable_operation] MutateOperation conversion_custom_variable_operation
                             * @property {google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation|null} [conversion_goal_campaign_config_operation] MutateOperation conversion_goal_campaign_config_operation
                             * @property {google.ads.googleads.v11.services.IConversionValueRuleOperation|null} [conversion_value_rule_operation] MutateOperation conversion_value_rule_operation
                             * @property {google.ads.googleads.v11.services.IConversionValueRuleSetOperation|null} [conversion_value_rule_set_operation] MutateOperation conversion_value_rule_set_operation
                             * @property {google.ads.googleads.v11.services.ICustomConversionGoalOperation|null} [custom_conversion_goal_operation] MutateOperation custom_conversion_goal_operation
                             * @property {google.ads.googleads.v11.services.ICustomerAssetOperation|null} [customer_asset_operation] MutateOperation customer_asset_operation
                             * @property {google.ads.googleads.v11.services.ICustomerConversionGoalOperation|null} [customer_conversion_goal_operation] MutateOperation customer_conversion_goal_operation
                             * @property {google.ads.googleads.v11.services.ICustomerCustomizerOperation|null} [customer_customizer_operation] MutateOperation customer_customizer_operation
                             * @property {google.ads.googleads.v11.services.ICustomerExtensionSettingOperation|null} [customer_extension_setting_operation] MutateOperation customer_extension_setting_operation
                             * @property {google.ads.googleads.v11.services.ICustomerFeedOperation|null} [customer_feed_operation] MutateOperation customer_feed_operation
                             * @property {google.ads.googleads.v11.services.ICustomerLabelOperation|null} [customer_label_operation] MutateOperation customer_label_operation
                             * @property {google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation|null} [customer_negative_criterion_operation] MutateOperation customer_negative_criterion_operation
                             * @property {google.ads.googleads.v11.services.ICustomerOperation|null} [customer_operation] MutateOperation customer_operation
                             * @property {google.ads.googleads.v11.services.ICustomizerAttributeOperation|null} [customizer_attribute_operation] MutateOperation customizer_attribute_operation
                             * @property {google.ads.googleads.v11.services.IExperimentOperation|null} [experiment_operation] MutateOperation experiment_operation
                             * @property {google.ads.googleads.v11.services.IExperimentArmOperation|null} [experiment_arm_operation] MutateOperation experiment_arm_operation
                             * @property {google.ads.googleads.v11.services.IExtensionFeedItemOperation|null} [extension_feed_item_operation] MutateOperation extension_feed_item_operation
                             * @property {google.ads.googleads.v11.services.IFeedItemOperation|null} [feed_item_operation] MutateOperation feed_item_operation
                             * @property {google.ads.googleads.v11.services.IFeedItemSetOperation|null} [feed_item_set_operation] MutateOperation feed_item_set_operation
                             * @property {google.ads.googleads.v11.services.IFeedItemSetLinkOperation|null} [feed_item_set_link_operation] MutateOperation feed_item_set_link_operation
                             * @property {google.ads.googleads.v11.services.IFeedItemTargetOperation|null} [feed_item_target_operation] MutateOperation feed_item_target_operation
                             * @property {google.ads.googleads.v11.services.IFeedMappingOperation|null} [feed_mapping_operation] MutateOperation feed_mapping_operation
                             * @property {google.ads.googleads.v11.services.IFeedOperation|null} [feed_operation] MutateOperation feed_operation
                             * @property {google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation|null} [keyword_plan_ad_group_operation] MutateOperation keyword_plan_ad_group_operation
                             * @property {google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation|null} [keyword_plan_ad_group_keyword_operation] MutateOperation keyword_plan_ad_group_keyword_operation
                             * @property {google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation|null} [keyword_plan_campaign_keyword_operation] MutateOperation keyword_plan_campaign_keyword_operation
                             * @property {google.ads.googleads.v11.services.IKeywordPlanCampaignOperation|null} [keyword_plan_campaign_operation] MutateOperation keyword_plan_campaign_operation
                             * @property {google.ads.googleads.v11.services.IKeywordPlanOperation|null} [keyword_plan_operation] MutateOperation keyword_plan_operation
                             * @property {google.ads.googleads.v11.services.ILabelOperation|null} [label_operation] MutateOperation label_operation
                             * @property {google.ads.googleads.v11.services.IMediaFileOperation|null} [media_file_operation] MutateOperation media_file_operation
                             * @property {google.ads.googleads.v11.services.IRemarketingActionOperation|null} [remarketing_action_operation] MutateOperation remarketing_action_operation
                             * @property {google.ads.googleads.v11.services.ISharedCriterionOperation|null} [shared_criterion_operation] MutateOperation shared_criterion_operation
                             * @property {google.ads.googleads.v11.services.ISharedSetOperation|null} [shared_set_operation] MutateOperation shared_set_operation
                             * @property {google.ads.googleads.v11.services.ISmartCampaignSettingOperation|null} [smart_campaign_setting_operation] MutateOperation smart_campaign_setting_operation
                             * @property {google.ads.googleads.v11.services.IUserListOperation|null} [user_list_operation] MutateOperation user_list_operation
                             */
    
                            /**
                             * Constructs a new MutateOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateOperation.
                             * @implements IMutateOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateOperation=} [p] Properties to set
                             */
                            function MutateOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateOperation ad_group_ad_label_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupAdLabelOperation|null|undefined} ad_group_ad_label_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_ad_label_operation = null;
    
                            /**
                             * MutateOperation ad_group_ad_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupAdOperation|null|undefined} ad_group_ad_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_ad_operation = null;
    
                            /**
                             * MutateOperation ad_group_asset_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupAssetOperation|null|undefined} ad_group_asset_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_asset_operation = null;
    
                            /**
                             * MutateOperation ad_group_bid_modifier_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupBidModifierOperation|null|undefined} ad_group_bid_modifier_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_bid_modifier_operation = null;
    
                            /**
                             * MutateOperation ad_group_criterion_customizer_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupCriterionCustomizerOperation|null|undefined} ad_group_criterion_customizer_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_criterion_customizer_operation = null;
    
                            /**
                             * MutateOperation ad_group_criterion_label_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupCriterionLabelOperation|null|undefined} ad_group_criterion_label_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_criterion_label_operation = null;
    
                            /**
                             * MutateOperation ad_group_criterion_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupCriterionOperation|null|undefined} ad_group_criterion_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_criterion_operation = null;
    
                            /**
                             * MutateOperation ad_group_customizer_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupCustomizerOperation|null|undefined} ad_group_customizer_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_customizer_operation = null;
    
                            /**
                             * MutateOperation ad_group_extension_setting_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupExtensionSettingOperation|null|undefined} ad_group_extension_setting_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_extension_setting_operation = null;
    
                            /**
                             * MutateOperation ad_group_feed_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupFeedOperation|null|undefined} ad_group_feed_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_feed_operation = null;
    
                            /**
                             * MutateOperation ad_group_label_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupLabelOperation|null|undefined} ad_group_label_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_label_operation = null;
    
                            /**
                             * MutateOperation ad_group_operation.
                             * @member {google.ads.googleads.v11.services.IAdGroupOperation|null|undefined} ad_group_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_group_operation = null;
    
                            /**
                             * MutateOperation ad_operation.
                             * @member {google.ads.googleads.v11.services.IAdOperation|null|undefined} ad_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_operation = null;
    
                            /**
                             * MutateOperation ad_parameter_operation.
                             * @member {google.ads.googleads.v11.services.IAdParameterOperation|null|undefined} ad_parameter_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.ad_parameter_operation = null;
    
                            /**
                             * MutateOperation asset_operation.
                             * @member {google.ads.googleads.v11.services.IAssetOperation|null|undefined} asset_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.asset_operation = null;
    
                            /**
                             * MutateOperation asset_group_asset_operation.
                             * @member {google.ads.googleads.v11.services.IAssetGroupAssetOperation|null|undefined} asset_group_asset_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.asset_group_asset_operation = null;
    
                            /**
                             * MutateOperation asset_group_listing_group_filter_operation.
                             * @member {google.ads.googleads.v11.services.IAssetGroupListingGroupFilterOperation|null|undefined} asset_group_listing_group_filter_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.asset_group_listing_group_filter_operation = null;
    
                            /**
                             * MutateOperation asset_group_signal_operation.
                             * @member {google.ads.googleads.v11.services.IAssetGroupSignalOperation|null|undefined} asset_group_signal_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.asset_group_signal_operation = null;
    
                            /**
                             * MutateOperation asset_group_operation.
                             * @member {google.ads.googleads.v11.services.IAssetGroupOperation|null|undefined} asset_group_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.asset_group_operation = null;
    
                            /**
                             * MutateOperation asset_set_asset_operation.
                             * @member {google.ads.googleads.v11.services.IAssetSetAssetOperation|null|undefined} asset_set_asset_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.asset_set_asset_operation = null;
    
                            /**
                             * MutateOperation asset_set_operation.
                             * @member {google.ads.googleads.v11.services.IAssetSetOperation|null|undefined} asset_set_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.asset_set_operation = null;
    
                            /**
                             * MutateOperation audience_operation.
                             * @member {google.ads.googleads.v11.services.IAudienceOperation|null|undefined} audience_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.audience_operation = null;
    
                            /**
                             * MutateOperation bidding_data_exclusion_operation.
                             * @member {google.ads.googleads.v11.services.IBiddingDataExclusionOperation|null|undefined} bidding_data_exclusion_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.bidding_data_exclusion_operation = null;
    
                            /**
                             * MutateOperation bidding_seasonality_adjustment_operation.
                             * @member {google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation|null|undefined} bidding_seasonality_adjustment_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.bidding_seasonality_adjustment_operation = null;
    
                            /**
                             * MutateOperation bidding_strategy_operation.
                             * @member {google.ads.googleads.v11.services.IBiddingStrategyOperation|null|undefined} bidding_strategy_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.bidding_strategy_operation = null;
    
                            /**
                             * MutateOperation campaign_asset_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignAssetOperation|null|undefined} campaign_asset_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_asset_operation = null;
    
                            /**
                             * MutateOperation campaign_asset_set_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignAssetSetOperation|null|undefined} campaign_asset_set_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_asset_set_operation = null;
    
                            /**
                             * MutateOperation campaign_bid_modifier_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignBidModifierOperation|null|undefined} campaign_bid_modifier_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_bid_modifier_operation = null;
    
                            /**
                             * MutateOperation campaign_budget_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignBudgetOperation|null|undefined} campaign_budget_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_budget_operation = null;
    
                            /**
                             * MutateOperation campaign_conversion_goal_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignConversionGoalOperation|null|undefined} campaign_conversion_goal_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_conversion_goal_operation = null;
    
                            /**
                             * MutateOperation campaign_criterion_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignCriterionOperation|null|undefined} campaign_criterion_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_criterion_operation = null;
    
                            /**
                             * MutateOperation campaign_customizer_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignCustomizerOperation|null|undefined} campaign_customizer_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_customizer_operation = null;
    
                            /**
                             * MutateOperation campaign_draft_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignDraftOperation|null|undefined} campaign_draft_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_draft_operation = null;
    
                            /**
                             * MutateOperation campaign_experiment_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignExperimentOperation|null|undefined} campaign_experiment_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_experiment_operation = null;
    
                            /**
                             * MutateOperation campaign_extension_setting_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignExtensionSettingOperation|null|undefined} campaign_extension_setting_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_extension_setting_operation = null;
    
                            /**
                             * MutateOperation campaign_feed_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignFeedOperation|null|undefined} campaign_feed_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_feed_operation = null;
    
                            /**
                             * MutateOperation campaign_group_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignGroupOperation|null|undefined} campaign_group_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_group_operation = null;
    
                            /**
                             * MutateOperation campaign_label_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignLabelOperation|null|undefined} campaign_label_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_label_operation = null;
    
                            /**
                             * MutateOperation campaign_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignOperation|null|undefined} campaign_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_operation = null;
    
                            /**
                             * MutateOperation campaign_shared_set_operation.
                             * @member {google.ads.googleads.v11.services.ICampaignSharedSetOperation|null|undefined} campaign_shared_set_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.campaign_shared_set_operation = null;
    
                            /**
                             * MutateOperation conversion_action_operation.
                             * @member {google.ads.googleads.v11.services.IConversionActionOperation|null|undefined} conversion_action_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.conversion_action_operation = null;
    
                            /**
                             * MutateOperation conversion_custom_variable_operation.
                             * @member {google.ads.googleads.v11.services.IConversionCustomVariableOperation|null|undefined} conversion_custom_variable_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.conversion_custom_variable_operation = null;
    
                            /**
                             * MutateOperation conversion_goal_campaign_config_operation.
                             * @member {google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation|null|undefined} conversion_goal_campaign_config_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.conversion_goal_campaign_config_operation = null;
    
                            /**
                             * MutateOperation conversion_value_rule_operation.
                             * @member {google.ads.googleads.v11.services.IConversionValueRuleOperation|null|undefined} conversion_value_rule_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.conversion_value_rule_operation = null;
    
                            /**
                             * MutateOperation conversion_value_rule_set_operation.
                             * @member {google.ads.googleads.v11.services.IConversionValueRuleSetOperation|null|undefined} conversion_value_rule_set_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.conversion_value_rule_set_operation = null;
    
                            /**
                             * MutateOperation custom_conversion_goal_operation.
                             * @member {google.ads.googleads.v11.services.ICustomConversionGoalOperation|null|undefined} custom_conversion_goal_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.custom_conversion_goal_operation = null;
    
                            /**
                             * MutateOperation customer_asset_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerAssetOperation|null|undefined} customer_asset_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_asset_operation = null;
    
                            /**
                             * MutateOperation customer_conversion_goal_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerConversionGoalOperation|null|undefined} customer_conversion_goal_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_conversion_goal_operation = null;
    
                            /**
                             * MutateOperation customer_customizer_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerCustomizerOperation|null|undefined} customer_customizer_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_customizer_operation = null;
    
                            /**
                             * MutateOperation customer_extension_setting_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerExtensionSettingOperation|null|undefined} customer_extension_setting_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_extension_setting_operation = null;
    
                            /**
                             * MutateOperation customer_feed_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerFeedOperation|null|undefined} customer_feed_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_feed_operation = null;
    
                            /**
                             * MutateOperation customer_label_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerLabelOperation|null|undefined} customer_label_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_label_operation = null;
    
                            /**
                             * MutateOperation customer_negative_criterion_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation|null|undefined} customer_negative_criterion_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_negative_criterion_operation = null;
    
                            /**
                             * MutateOperation customer_operation.
                             * @member {google.ads.googleads.v11.services.ICustomerOperation|null|undefined} customer_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customer_operation = null;
    
                            /**
                             * MutateOperation customizer_attribute_operation.
                             * @member {google.ads.googleads.v11.services.ICustomizerAttributeOperation|null|undefined} customizer_attribute_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.customizer_attribute_operation = null;
    
                            /**
                             * MutateOperation experiment_operation.
                             * @member {google.ads.googleads.v11.services.IExperimentOperation|null|undefined} experiment_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.experiment_operation = null;
    
                            /**
                             * MutateOperation experiment_arm_operation.
                             * @member {google.ads.googleads.v11.services.IExperimentArmOperation|null|undefined} experiment_arm_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.experiment_arm_operation = null;
    
                            /**
                             * MutateOperation extension_feed_item_operation.
                             * @member {google.ads.googleads.v11.services.IExtensionFeedItemOperation|null|undefined} extension_feed_item_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.extension_feed_item_operation = null;
    
                            /**
                             * MutateOperation feed_item_operation.
                             * @member {google.ads.googleads.v11.services.IFeedItemOperation|null|undefined} feed_item_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.feed_item_operation = null;
    
                            /**
                             * MutateOperation feed_item_set_operation.
                             * @member {google.ads.googleads.v11.services.IFeedItemSetOperation|null|undefined} feed_item_set_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.feed_item_set_operation = null;
    
                            /**
                             * MutateOperation feed_item_set_link_operation.
                             * @member {google.ads.googleads.v11.services.IFeedItemSetLinkOperation|null|undefined} feed_item_set_link_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.feed_item_set_link_operation = null;
    
                            /**
                             * MutateOperation feed_item_target_operation.
                             * @member {google.ads.googleads.v11.services.IFeedItemTargetOperation|null|undefined} feed_item_target_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.feed_item_target_operation = null;
    
                            /**
                             * MutateOperation feed_mapping_operation.
                             * @member {google.ads.googleads.v11.services.IFeedMappingOperation|null|undefined} feed_mapping_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.feed_mapping_operation = null;
    
                            /**
                             * MutateOperation feed_operation.
                             * @member {google.ads.googleads.v11.services.IFeedOperation|null|undefined} feed_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.feed_operation = null;
    
                            /**
                             * MutateOperation keyword_plan_ad_group_operation.
                             * @member {google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation|null|undefined} keyword_plan_ad_group_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.keyword_plan_ad_group_operation = null;
    
                            /**
                             * MutateOperation keyword_plan_ad_group_keyword_operation.
                             * @member {google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation|null|undefined} keyword_plan_ad_group_keyword_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.keyword_plan_ad_group_keyword_operation = null;
    
                            /**
                             * MutateOperation keyword_plan_campaign_keyword_operation.
                             * @member {google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation|null|undefined} keyword_plan_campaign_keyword_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.keyword_plan_campaign_keyword_operation = null;
    
                            /**
                             * MutateOperation keyword_plan_campaign_operation.
                             * @member {google.ads.googleads.v11.services.IKeywordPlanCampaignOperation|null|undefined} keyword_plan_campaign_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.keyword_plan_campaign_operation = null;
    
                            /**
                             * MutateOperation keyword_plan_operation.
                             * @member {google.ads.googleads.v11.services.IKeywordPlanOperation|null|undefined} keyword_plan_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.keyword_plan_operation = null;
    
                            /**
                             * MutateOperation label_operation.
                             * @member {google.ads.googleads.v11.services.ILabelOperation|null|undefined} label_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.label_operation = null;
    
                            /**
                             * MutateOperation media_file_operation.
                             * @member {google.ads.googleads.v11.services.IMediaFileOperation|null|undefined} media_file_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.media_file_operation = null;
    
                            /**
                             * MutateOperation remarketing_action_operation.
                             * @member {google.ads.googleads.v11.services.IRemarketingActionOperation|null|undefined} remarketing_action_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.remarketing_action_operation = null;
    
                            /**
                             * MutateOperation shared_criterion_operation.
                             * @member {google.ads.googleads.v11.services.ISharedCriterionOperation|null|undefined} shared_criterion_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.shared_criterion_operation = null;
    
                            /**
                             * MutateOperation shared_set_operation.
                             * @member {google.ads.googleads.v11.services.ISharedSetOperation|null|undefined} shared_set_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.shared_set_operation = null;
    
                            /**
                             * MutateOperation smart_campaign_setting_operation.
                             * @member {google.ads.googleads.v11.services.ISmartCampaignSettingOperation|null|undefined} smart_campaign_setting_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.smart_campaign_setting_operation = null;
    
                            /**
                             * MutateOperation user_list_operation.
                             * @member {google.ads.googleads.v11.services.IUserListOperation|null|undefined} user_list_operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            MutateOperation.prototype.user_list_operation = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MutateOperation operation.
                             * @member {"ad_group_ad_label_operation"|"ad_group_ad_operation"|"ad_group_asset_operation"|"ad_group_bid_modifier_operation"|"ad_group_criterion_customizer_operation"|"ad_group_criterion_label_operation"|"ad_group_criterion_operation"|"ad_group_customizer_operation"|"ad_group_extension_setting_operation"|"ad_group_feed_operation"|"ad_group_label_operation"|"ad_group_operation"|"ad_operation"|"ad_parameter_operation"|"asset_operation"|"asset_group_asset_operation"|"asset_group_listing_group_filter_operation"|"asset_group_signal_operation"|"asset_group_operation"|"asset_set_asset_operation"|"asset_set_operation"|"audience_operation"|"bidding_data_exclusion_operation"|"bidding_seasonality_adjustment_operation"|"bidding_strategy_operation"|"campaign_asset_operation"|"campaign_asset_set_operation"|"campaign_bid_modifier_operation"|"campaign_budget_operation"|"campaign_conversion_goal_operation"|"campaign_criterion_operation"|"campaign_customizer_operation"|"campaign_draft_operation"|"campaign_experiment_operation"|"campaign_extension_setting_operation"|"campaign_feed_operation"|"campaign_group_operation"|"campaign_label_operation"|"campaign_operation"|"campaign_shared_set_operation"|"conversion_action_operation"|"conversion_custom_variable_operation"|"conversion_goal_campaign_config_operation"|"conversion_value_rule_operation"|"conversion_value_rule_set_operation"|"custom_conversion_goal_operation"|"customer_asset_operation"|"customer_conversion_goal_operation"|"customer_customizer_operation"|"customer_extension_setting_operation"|"customer_feed_operation"|"customer_label_operation"|"customer_negative_criterion_operation"|"customer_operation"|"customizer_attribute_operation"|"experiment_operation"|"experiment_arm_operation"|"extension_feed_item_operation"|"feed_item_operation"|"feed_item_set_operation"|"feed_item_set_link_operation"|"feed_item_target_operation"|"feed_mapping_operation"|"feed_operation"|"keyword_plan_ad_group_operation"|"keyword_plan_ad_group_keyword_operation"|"keyword_plan_campaign_keyword_operation"|"keyword_plan_campaign_operation"|"keyword_plan_operation"|"label_operation"|"media_file_operation"|"remarketing_action_operation"|"shared_criterion_operation"|"shared_set_operation"|"smart_campaign_setting_operation"|"user_list_operation"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             */
                            Object.defineProperty(MutateOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_ad_label_operation", "ad_group_ad_operation", "ad_group_asset_operation", "ad_group_bid_modifier_operation", "ad_group_criterion_customizer_operation", "ad_group_criterion_label_operation", "ad_group_criterion_operation", "ad_group_customizer_operation", "ad_group_extension_setting_operation", "ad_group_feed_operation", "ad_group_label_operation", "ad_group_operation", "ad_operation", "ad_parameter_operation", "asset_operation", "asset_group_asset_operation", "asset_group_listing_group_filter_operation", "asset_group_signal_operation", "asset_group_operation", "asset_set_asset_operation", "asset_set_operation", "audience_operation", "bidding_data_exclusion_operation", "bidding_seasonality_adjustment_operation", "bidding_strategy_operation", "campaign_asset_operation", "campaign_asset_set_operation", "campaign_bid_modifier_operation", "campaign_budget_operation", "campaign_conversion_goal_operation", "campaign_criterion_operation", "campaign_customizer_operation", "campaign_draft_operation", "campaign_experiment_operation", "campaign_extension_setting_operation", "campaign_feed_operation", "campaign_group_operation", "campaign_label_operation", "campaign_operation", "campaign_shared_set_operation", "conversion_action_operation", "conversion_custom_variable_operation", "conversion_goal_campaign_config_operation", "conversion_value_rule_operation", "conversion_value_rule_set_operation", "custom_conversion_goal_operation", "customer_asset_operation", "customer_conversion_goal_operation", "customer_customizer_operation", "customer_extension_setting_operation", "customer_feed_operation", "customer_label_operation", "customer_negative_criterion_operation", "customer_operation", "customizer_attribute_operation", "experiment_operation", "experiment_arm_operation", "extension_feed_item_operation", "feed_item_operation", "feed_item_set_operation", "feed_item_set_link_operation", "feed_item_target_operation", "feed_mapping_operation", "feed_operation", "keyword_plan_ad_group_operation", "keyword_plan_ad_group_keyword_operation", "keyword_plan_campaign_keyword_operation", "keyword_plan_campaign_operation", "keyword_plan_operation", "label_operation", "media_file_operation", "remarketing_action_operation", "shared_criterion_operation", "shared_set_operation", "smart_campaign_setting_operation", "user_list_operation"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MutateOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateOperation} MutateOperation instance
                             */
                            MutateOperation.create = function create(properties) {
                                return new MutateOperation(properties);
                            };
    
                            /**
                             * Encodes the specified MutateOperation message. Does not implicitly {@link google.ads.googleads.v11.services.MutateOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateOperation} m MutateOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_group_ad_operation != null && Object.hasOwnProperty.call(m, "ad_group_ad_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupAdOperation.encode(m.ad_group_ad_operation, w.uint32(10).fork()).ldelim();
                                if (m.ad_group_bid_modifier_operation != null && Object.hasOwnProperty.call(m, "ad_group_bid_modifier_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.encode(m.ad_group_bid_modifier_operation, w.uint32(18).fork()).ldelim();
                                if (m.ad_group_criterion_operation != null && Object.hasOwnProperty.call(m, "ad_group_criterion_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.encode(m.ad_group_criterion_operation, w.uint32(26).fork()).ldelim();
                                if (m.ad_group_operation != null && Object.hasOwnProperty.call(m, "ad_group_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupOperation.encode(m.ad_group_operation, w.uint32(42).fork()).ldelim();
                                if (m.bidding_strategy_operation != null && Object.hasOwnProperty.call(m, "bidding_strategy_operation"))
                                    $root.google.ads.googleads.v11.services.BiddingStrategyOperation.encode(m.bidding_strategy_operation, w.uint32(50).fork()).ldelim();
                                if (m.campaign_bid_modifier_operation != null && Object.hasOwnProperty.call(m, "campaign_bid_modifier_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.encode(m.campaign_bid_modifier_operation, w.uint32(58).fork()).ldelim();
                                if (m.campaign_budget_operation != null && Object.hasOwnProperty.call(m, "campaign_budget_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignBudgetOperation.encode(m.campaign_budget_operation, w.uint32(66).fork()).ldelim();
                                if (m.campaign_group_operation != null && Object.hasOwnProperty.call(m, "campaign_group_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignGroupOperation.encode(m.campaign_group_operation, w.uint32(74).fork()).ldelim();
                                if (m.campaign_operation != null && Object.hasOwnProperty.call(m, "campaign_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignOperation.encode(m.campaign_operation, w.uint32(82).fork()).ldelim();
                                if (m.campaign_shared_set_operation != null && Object.hasOwnProperty.call(m, "campaign_shared_set_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.encode(m.campaign_shared_set_operation, w.uint32(90).fork()).ldelim();
                                if (m.conversion_action_operation != null && Object.hasOwnProperty.call(m, "conversion_action_operation"))
                                    $root.google.ads.googleads.v11.services.ConversionActionOperation.encode(m.conversion_action_operation, w.uint32(98).fork()).ldelim();
                                if (m.campaign_criterion_operation != null && Object.hasOwnProperty.call(m, "campaign_criterion_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignCriterionOperation.encode(m.campaign_criterion_operation, w.uint32(106).fork()).ldelim();
                                if (m.shared_criterion_operation != null && Object.hasOwnProperty.call(m, "shared_criterion_operation"))
                                    $root.google.ads.googleads.v11.services.SharedCriterionOperation.encode(m.shared_criterion_operation, w.uint32(114).fork()).ldelim();
                                if (m.shared_set_operation != null && Object.hasOwnProperty.call(m, "shared_set_operation"))
                                    $root.google.ads.googleads.v11.services.SharedSetOperation.encode(m.shared_set_operation, w.uint32(122).fork()).ldelim();
                                if (m.user_list_operation != null && Object.hasOwnProperty.call(m, "user_list_operation"))
                                    $root.google.ads.googleads.v11.services.UserListOperation.encode(m.user_list_operation, w.uint32(130).fork()).ldelim();
                                if (m.ad_group_ad_label_operation != null && Object.hasOwnProperty.call(m, "ad_group_ad_label_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.encode(m.ad_group_ad_label_operation, w.uint32(138).fork()).ldelim();
                                if (m.ad_group_criterion_label_operation != null && Object.hasOwnProperty.call(m, "ad_group_criterion_label_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.encode(m.ad_group_criterion_label_operation, w.uint32(146).fork()).ldelim();
                                if (m.ad_group_extension_setting_operation != null && Object.hasOwnProperty.call(m, "ad_group_extension_setting_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.encode(m.ad_group_extension_setting_operation, w.uint32(154).fork()).ldelim();
                                if (m.ad_group_feed_operation != null && Object.hasOwnProperty.call(m, "ad_group_feed_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupFeedOperation.encode(m.ad_group_feed_operation, w.uint32(162).fork()).ldelim();
                                if (m.ad_group_label_operation != null && Object.hasOwnProperty.call(m, "ad_group_label_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupLabelOperation.encode(m.ad_group_label_operation, w.uint32(170).fork()).ldelim();
                                if (m.ad_parameter_operation != null && Object.hasOwnProperty.call(m, "ad_parameter_operation"))
                                    $root.google.ads.googleads.v11.services.AdParameterOperation.encode(m.ad_parameter_operation, w.uint32(178).fork()).ldelim();
                                if (m.asset_operation != null && Object.hasOwnProperty.call(m, "asset_operation"))
                                    $root.google.ads.googleads.v11.services.AssetOperation.encode(m.asset_operation, w.uint32(186).fork()).ldelim();
                                if (m.campaign_draft_operation != null && Object.hasOwnProperty.call(m, "campaign_draft_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignDraftOperation.encode(m.campaign_draft_operation, w.uint32(194).fork()).ldelim();
                                if (m.campaign_experiment_operation != null && Object.hasOwnProperty.call(m, "campaign_experiment_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignExperimentOperation.encode(m.campaign_experiment_operation, w.uint32(202).fork()).ldelim();
                                if (m.campaign_extension_setting_operation != null && Object.hasOwnProperty.call(m, "campaign_extension_setting_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.encode(m.campaign_extension_setting_operation, w.uint32(210).fork()).ldelim();
                                if (m.campaign_feed_operation != null && Object.hasOwnProperty.call(m, "campaign_feed_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignFeedOperation.encode(m.campaign_feed_operation, w.uint32(218).fork()).ldelim();
                                if (m.campaign_label_operation != null && Object.hasOwnProperty.call(m, "campaign_label_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignLabelOperation.encode(m.campaign_label_operation, w.uint32(226).fork()).ldelim();
                                if (m.customer_extension_setting_operation != null && Object.hasOwnProperty.call(m, "customer_extension_setting_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.encode(m.customer_extension_setting_operation, w.uint32(242).fork()).ldelim();
                                if (m.customer_feed_operation != null && Object.hasOwnProperty.call(m, "customer_feed_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerFeedOperation.encode(m.customer_feed_operation, w.uint32(250).fork()).ldelim();
                                if (m.customer_label_operation != null && Object.hasOwnProperty.call(m, "customer_label_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerLabelOperation.encode(m.customer_label_operation, w.uint32(258).fork()).ldelim();
                                if (m.customer_negative_criterion_operation != null && Object.hasOwnProperty.call(m, "customer_negative_criterion_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.encode(m.customer_negative_criterion_operation, w.uint32(274).fork()).ldelim();
                                if (m.customer_operation != null && Object.hasOwnProperty.call(m, "customer_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerOperation.encode(m.customer_operation, w.uint32(282).fork()).ldelim();
                                if (m.extension_feed_item_operation != null && Object.hasOwnProperty.call(m, "extension_feed_item_operation"))
                                    $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.encode(m.extension_feed_item_operation, w.uint32(290).fork()).ldelim();
                                if (m.feed_item_operation != null && Object.hasOwnProperty.call(m, "feed_item_operation"))
                                    $root.google.ads.googleads.v11.services.FeedItemOperation.encode(m.feed_item_operation, w.uint32(298).fork()).ldelim();
                                if (m.feed_item_target_operation != null && Object.hasOwnProperty.call(m, "feed_item_target_operation"))
                                    $root.google.ads.googleads.v11.services.FeedItemTargetOperation.encode(m.feed_item_target_operation, w.uint32(306).fork()).ldelim();
                                if (m.feed_mapping_operation != null && Object.hasOwnProperty.call(m, "feed_mapping_operation"))
                                    $root.google.ads.googleads.v11.services.FeedMappingOperation.encode(m.feed_mapping_operation, w.uint32(314).fork()).ldelim();
                                if (m.feed_operation != null && Object.hasOwnProperty.call(m, "feed_operation"))
                                    $root.google.ads.googleads.v11.services.FeedOperation.encode(m.feed_operation, w.uint32(322).fork()).ldelim();
                                if (m.label_operation != null && Object.hasOwnProperty.call(m, "label_operation"))
                                    $root.google.ads.googleads.v11.services.LabelOperation.encode(m.label_operation, w.uint32(330).fork()).ldelim();
                                if (m.media_file_operation != null && Object.hasOwnProperty.call(m, "media_file_operation"))
                                    $root.google.ads.googleads.v11.services.MediaFileOperation.encode(m.media_file_operation, w.uint32(338).fork()).ldelim();
                                if (m.remarketing_action_operation != null && Object.hasOwnProperty.call(m, "remarketing_action_operation"))
                                    $root.google.ads.googleads.v11.services.RemarketingActionOperation.encode(m.remarketing_action_operation, w.uint32(346).fork()).ldelim();
                                if (m.keyword_plan_ad_group_operation != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_operation"))
                                    $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.encode(m.keyword_plan_ad_group_operation, w.uint32(354).fork()).ldelim();
                                if (m.keyword_plan_campaign_operation != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign_operation"))
                                    $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.encode(m.keyword_plan_campaign_operation, w.uint32(362).fork()).ldelim();
                                if (m.keyword_plan_operation != null && Object.hasOwnProperty.call(m, "keyword_plan_operation"))
                                    $root.google.ads.googleads.v11.services.KeywordPlanOperation.encode(m.keyword_plan_operation, w.uint32(386).fork()).ldelim();
                                if (m.ad_operation != null && Object.hasOwnProperty.call(m, "ad_operation"))
                                    $root.google.ads.googleads.v11.services.AdOperation.encode(m.ad_operation, w.uint32(394).fork()).ldelim();
                                if (m.keyword_plan_ad_group_keyword_operation != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_keyword_operation"))
                                    $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.encode(m.keyword_plan_ad_group_keyword_operation, w.uint32(402).fork()).ldelim();
                                if (m.keyword_plan_campaign_keyword_operation != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign_keyword_operation"))
                                    $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.encode(m.keyword_plan_campaign_keyword_operation, w.uint32(410).fork()).ldelim();
                                if (m.campaign_asset_operation != null && Object.hasOwnProperty.call(m, "campaign_asset_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignAssetOperation.encode(m.campaign_asset_operation, w.uint32(418).fork()).ldelim();
                                if (m.feed_item_set_operation != null && Object.hasOwnProperty.call(m, "feed_item_set_operation"))
                                    $root.google.ads.googleads.v11.services.FeedItemSetOperation.encode(m.feed_item_set_operation, w.uint32(426).fork()).ldelim();
                                if (m.feed_item_set_link_operation != null && Object.hasOwnProperty.call(m, "feed_item_set_link_operation"))
                                    $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.encode(m.feed_item_set_link_operation, w.uint32(434).fork()).ldelim();
                                if (m.conversion_custom_variable_operation != null && Object.hasOwnProperty.call(m, "conversion_custom_variable_operation"))
                                    $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.encode(m.conversion_custom_variable_operation, w.uint32(442).fork()).ldelim();
                                if (m.ad_group_asset_operation != null && Object.hasOwnProperty.call(m, "ad_group_asset_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupAssetOperation.encode(m.ad_group_asset_operation, w.uint32(450).fork()).ldelim();
                                if (m.customer_asset_operation != null && Object.hasOwnProperty.call(m, "customer_asset_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerAssetOperation.encode(m.customer_asset_operation, w.uint32(458).fork()).ldelim();
                                if (m.bidding_data_exclusion_operation != null && Object.hasOwnProperty.call(m, "bidding_data_exclusion_operation"))
                                    $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.encode(m.bidding_data_exclusion_operation, w.uint32(466).fork()).ldelim();
                                if (m.bidding_seasonality_adjustment_operation != null && Object.hasOwnProperty.call(m, "bidding_seasonality_adjustment_operation"))
                                    $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.encode(m.bidding_seasonality_adjustment_operation, w.uint32(474).fork()).ldelim();
                                if (m.smart_campaign_setting_operation != null && Object.hasOwnProperty.call(m, "smart_campaign_setting_operation"))
                                    $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.encode(m.smart_campaign_setting_operation, w.uint32(490).fork()).ldelim();
                                if (m.asset_group_operation != null && Object.hasOwnProperty.call(m, "asset_group_operation"))
                                    $root.google.ads.googleads.v11.services.AssetGroupOperation.encode(m.asset_group_operation, w.uint32(498).fork()).ldelim();
                                if (m.conversion_value_rule_operation != null && Object.hasOwnProperty.call(m, "conversion_value_rule_operation"))
                                    $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.encode(m.conversion_value_rule_operation, w.uint32(506).fork()).ldelim();
                                if (m.conversion_value_rule_set_operation != null && Object.hasOwnProperty.call(m, "conversion_value_rule_set_operation"))
                                    $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.encode(m.conversion_value_rule_set_operation, w.uint32(514).fork()).ldelim();
                                if (m.asset_group_asset_operation != null && Object.hasOwnProperty.call(m, "asset_group_asset_operation"))
                                    $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.encode(m.asset_group_asset_operation, w.uint32(522).fork()).ldelim();
                                if (m.customer_conversion_goal_operation != null && Object.hasOwnProperty.call(m, "customer_conversion_goal_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.encode(m.customer_conversion_goal_operation, w.uint32(530).fork()).ldelim();
                                if (m.campaign_conversion_goal_operation != null && Object.hasOwnProperty.call(m, "campaign_conversion_goal_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.encode(m.campaign_conversion_goal_operation, w.uint32(538).fork()).ldelim();
                                if (m.custom_conversion_goal_operation != null && Object.hasOwnProperty.call(m, "custom_conversion_goal_operation"))
                                    $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.encode(m.custom_conversion_goal_operation, w.uint32(546).fork()).ldelim();
                                if (m.conversion_goal_campaign_config_operation != null && Object.hasOwnProperty.call(m, "conversion_goal_campaign_config_operation"))
                                    $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.encode(m.conversion_goal_campaign_config_operation, w.uint32(554).fork()).ldelim();
                                if (m.customizer_attribute_operation != null && Object.hasOwnProperty.call(m, "customizer_attribute_operation"))
                                    $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.encode(m.customizer_attribute_operation, w.uint32(562).fork()).ldelim();
                                if (m.asset_set_asset_operation != null && Object.hasOwnProperty.call(m, "asset_set_asset_operation"))
                                    $root.google.ads.googleads.v11.services.AssetSetAssetOperation.encode(m.asset_set_asset_operation, w.uint32(570).fork()).ldelim();
                                if (m.asset_set_operation != null && Object.hasOwnProperty.call(m, "asset_set_operation"))
                                    $root.google.ads.googleads.v11.services.AssetSetOperation.encode(m.asset_set_operation, w.uint32(578).fork()).ldelim();
                                if (m.campaign_asset_set_operation != null && Object.hasOwnProperty.call(m, "campaign_asset_set_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.encode(m.campaign_asset_set_operation, w.uint32(586).fork()).ldelim();
                                if (m.ad_group_customizer_operation != null && Object.hasOwnProperty.call(m, "ad_group_customizer_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.encode(m.ad_group_customizer_operation, w.uint32(602).fork()).ldelim();
                                if (m.campaign_customizer_operation != null && Object.hasOwnProperty.call(m, "campaign_customizer_operation"))
                                    $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.encode(m.campaign_customizer_operation, w.uint32(610).fork()).ldelim();
                                if (m.ad_group_criterion_customizer_operation != null && Object.hasOwnProperty.call(m, "ad_group_criterion_customizer_operation"))
                                    $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.encode(m.ad_group_criterion_customizer_operation, w.uint32(618).fork()).ldelim();
                                if (m.asset_group_listing_group_filter_operation != null && Object.hasOwnProperty.call(m, "asset_group_listing_group_filter_operation"))
                                    $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.encode(m.asset_group_listing_group_filter_operation, w.uint32(626).fork()).ldelim();
                                if (m.customer_customizer_operation != null && Object.hasOwnProperty.call(m, "customer_customizer_operation"))
                                    $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.encode(m.customer_customizer_operation, w.uint32(634).fork()).ldelim();
                                if (m.asset_group_signal_operation != null && Object.hasOwnProperty.call(m, "asset_group_signal_operation"))
                                    $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.encode(m.asset_group_signal_operation, w.uint32(642).fork()).ldelim();
                                if (m.audience_operation != null && Object.hasOwnProperty.call(m, "audience_operation"))
                                    $root.google.ads.googleads.v11.services.AudienceOperation.encode(m.audience_operation, w.uint32(650).fork()).ldelim();
                                if (m.experiment_operation != null && Object.hasOwnProperty.call(m, "experiment_operation"))
                                    $root.google.ads.googleads.v11.services.ExperimentOperation.encode(m.experiment_operation, w.uint32(658).fork()).ldelim();
                                if (m.experiment_arm_operation != null && Object.hasOwnProperty.call(m, "experiment_arm_operation"))
                                    $root.google.ads.googleads.v11.services.ExperimentArmOperation.encode(m.experiment_arm_operation, w.uint32(666).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateOperation} message MutateOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateOperation} MutateOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 17:
                                        m.ad_group_ad_label_operation = $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.ad_group_ad_operation = $root.google.ads.googleads.v11.services.AdGroupAdOperation.decode(r, r.uint32());
                                        break;
                                    case 56:
                                        m.ad_group_asset_operation = $root.google.ads.googleads.v11.services.AdGroupAssetOperation.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.ad_group_bid_modifier_operation = $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.decode(r, r.uint32());
                                        break;
                                    case 77:
                                        m.ad_group_criterion_customizer_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.ad_group_criterion_label_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.ad_group_criterion_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.decode(r, r.uint32());
                                        break;
                                    case 75:
                                        m.ad_group_customizer_operation = $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.decode(r, r.uint32());
                                        break;
                                    case 19:
                                        m.ad_group_extension_setting_operation = $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.ad_group_feed_operation = $root.google.ads.googleads.v11.services.AdGroupFeedOperation.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.ad_group_label_operation = $root.google.ads.googleads.v11.services.AdGroupLabelOperation.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.ad_group_operation = $root.google.ads.googleads.v11.services.AdGroupOperation.decode(r, r.uint32());
                                        break;
                                    case 49:
                                        m.ad_operation = $root.google.ads.googleads.v11.services.AdOperation.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.ad_parameter_operation = $root.google.ads.googleads.v11.services.AdParameterOperation.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.asset_operation = $root.google.ads.googleads.v11.services.AssetOperation.decode(r, r.uint32());
                                        break;
                                    case 65:
                                        m.asset_group_asset_operation = $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.decode(r, r.uint32());
                                        break;
                                    case 78:
                                        m.asset_group_listing_group_filter_operation = $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.decode(r, r.uint32());
                                        break;
                                    case 80:
                                        m.asset_group_signal_operation = $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.decode(r, r.uint32());
                                        break;
                                    case 62:
                                        m.asset_group_operation = $root.google.ads.googleads.v11.services.AssetGroupOperation.decode(r, r.uint32());
                                        break;
                                    case 71:
                                        m.asset_set_asset_operation = $root.google.ads.googleads.v11.services.AssetSetAssetOperation.decode(r, r.uint32());
                                        break;
                                    case 72:
                                        m.asset_set_operation = $root.google.ads.googleads.v11.services.AssetSetOperation.decode(r, r.uint32());
                                        break;
                                    case 81:
                                        m.audience_operation = $root.google.ads.googleads.v11.services.AudienceOperation.decode(r, r.uint32());
                                        break;
                                    case 58:
                                        m.bidding_data_exclusion_operation = $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.decode(r, r.uint32());
                                        break;
                                    case 59:
                                        m.bidding_seasonality_adjustment_operation = $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.bidding_strategy_operation = $root.google.ads.googleads.v11.services.BiddingStrategyOperation.decode(r, r.uint32());
                                        break;
                                    case 52:
                                        m.campaign_asset_operation = $root.google.ads.googleads.v11.services.CampaignAssetOperation.decode(r, r.uint32());
                                        break;
                                    case 73:
                                        m.campaign_asset_set_operation = $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.campaign_bid_modifier_operation = $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.campaign_budget_operation = $root.google.ads.googleads.v11.services.CampaignBudgetOperation.decode(r, r.uint32());
                                        break;
                                    case 67:
                                        m.campaign_conversion_goal_operation = $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.campaign_criterion_operation = $root.google.ads.googleads.v11.services.CampaignCriterionOperation.decode(r, r.uint32());
                                        break;
                                    case 76:
                                        m.campaign_customizer_operation = $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.campaign_draft_operation = $root.google.ads.googleads.v11.services.CampaignDraftOperation.decode(r, r.uint32());
                                        break;
                                    case 25:
                                        m.campaign_experiment_operation = $root.google.ads.googleads.v11.services.CampaignExperimentOperation.decode(r, r.uint32());
                                        break;
                                    case 26:
                                        m.campaign_extension_setting_operation = $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.decode(r, r.uint32());
                                        break;
                                    case 27:
                                        m.campaign_feed_operation = $root.google.ads.googleads.v11.services.CampaignFeedOperation.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.campaign_group_operation = $root.google.ads.googleads.v11.services.CampaignGroupOperation.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.campaign_label_operation = $root.google.ads.googleads.v11.services.CampaignLabelOperation.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.campaign_operation = $root.google.ads.googleads.v11.services.CampaignOperation.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.campaign_shared_set_operation = $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.conversion_action_operation = $root.google.ads.googleads.v11.services.ConversionActionOperation.decode(r, r.uint32());
                                        break;
                                    case 55:
                                        m.conversion_custom_variable_operation = $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.decode(r, r.uint32());
                                        break;
                                    case 69:
                                        m.conversion_goal_campaign_config_operation = $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.decode(r, r.uint32());
                                        break;
                                    case 63:
                                        m.conversion_value_rule_operation = $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.decode(r, r.uint32());
                                        break;
                                    case 64:
                                        m.conversion_value_rule_set_operation = $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.decode(r, r.uint32());
                                        break;
                                    case 68:
                                        m.custom_conversion_goal_operation = $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.decode(r, r.uint32());
                                        break;
                                    case 57:
                                        m.customer_asset_operation = $root.google.ads.googleads.v11.services.CustomerAssetOperation.decode(r, r.uint32());
                                        break;
                                    case 66:
                                        m.customer_conversion_goal_operation = $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.decode(r, r.uint32());
                                        break;
                                    case 79:
                                        m.customer_customizer_operation = $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.customer_extension_setting_operation = $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.customer_feed_operation = $root.google.ads.googleads.v11.services.CustomerFeedOperation.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.customer_label_operation = $root.google.ads.googleads.v11.services.CustomerLabelOperation.decode(r, r.uint32());
                                        break;
                                    case 34:
                                        m.customer_negative_criterion_operation = $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.decode(r, r.uint32());
                                        break;
                                    case 35:
                                        m.customer_operation = $root.google.ads.googleads.v11.services.CustomerOperation.decode(r, r.uint32());
                                        break;
                                    case 70:
                                        m.customizer_attribute_operation = $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.decode(r, r.uint32());
                                        break;
                                    case 82:
                                        m.experiment_operation = $root.google.ads.googleads.v11.services.ExperimentOperation.decode(r, r.uint32());
                                        break;
                                    case 83:
                                        m.experiment_arm_operation = $root.google.ads.googleads.v11.services.ExperimentArmOperation.decode(r, r.uint32());
                                        break;
                                    case 36:
                                        m.extension_feed_item_operation = $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.decode(r, r.uint32());
                                        break;
                                    case 37:
                                        m.feed_item_operation = $root.google.ads.googleads.v11.services.FeedItemOperation.decode(r, r.uint32());
                                        break;
                                    case 53:
                                        m.feed_item_set_operation = $root.google.ads.googleads.v11.services.FeedItemSetOperation.decode(r, r.uint32());
                                        break;
                                    case 54:
                                        m.feed_item_set_link_operation = $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.decode(r, r.uint32());
                                        break;
                                    case 38:
                                        m.feed_item_target_operation = $root.google.ads.googleads.v11.services.FeedItemTargetOperation.decode(r, r.uint32());
                                        break;
                                    case 39:
                                        m.feed_mapping_operation = $root.google.ads.googleads.v11.services.FeedMappingOperation.decode(r, r.uint32());
                                        break;
                                    case 40:
                                        m.feed_operation = $root.google.ads.googleads.v11.services.FeedOperation.decode(r, r.uint32());
                                        break;
                                    case 44:
                                        m.keyword_plan_ad_group_operation = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.decode(r, r.uint32());
                                        break;
                                    case 50:
                                        m.keyword_plan_ad_group_keyword_operation = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.decode(r, r.uint32());
                                        break;
                                    case 51:
                                        m.keyword_plan_campaign_keyword_operation = $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.decode(r, r.uint32());
                                        break;
                                    case 45:
                                        m.keyword_plan_campaign_operation = $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.decode(r, r.uint32());
                                        break;
                                    case 48:
                                        m.keyword_plan_operation = $root.google.ads.googleads.v11.services.KeywordPlanOperation.decode(r, r.uint32());
                                        break;
                                    case 41:
                                        m.label_operation = $root.google.ads.googleads.v11.services.LabelOperation.decode(r, r.uint32());
                                        break;
                                    case 42:
                                        m.media_file_operation = $root.google.ads.googleads.v11.services.MediaFileOperation.decode(r, r.uint32());
                                        break;
                                    case 43:
                                        m.remarketing_action_operation = $root.google.ads.googleads.v11.services.RemarketingActionOperation.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.shared_criterion_operation = $root.google.ads.googleads.v11.services.SharedCriterionOperation.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.shared_set_operation = $root.google.ads.googleads.v11.services.SharedSetOperation.decode(r, r.uint32());
                                        break;
                                    case 61:
                                        m.smart_campaign_setting_operation = $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.user_list_operation = $root.google.ads.googleads.v11.services.UserListOperation.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateOperation} MutateOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.ad_group_ad_label_operation != null && m.hasOwnProperty("ad_group_ad_label_operation")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.verify(m.ad_group_ad_label_operation);
                                        if (e)
                                            return "ad_group_ad_label_operation." + e;
                                    }
                                }
                                if (m.ad_group_ad_operation != null && m.hasOwnProperty("ad_group_ad_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupAdOperation.verify(m.ad_group_ad_operation);
                                        if (e)
                                            return "ad_group_ad_operation." + e;
                                    }
                                }
                                if (m.ad_group_asset_operation != null && m.hasOwnProperty("ad_group_asset_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupAssetOperation.verify(m.ad_group_asset_operation);
                                        if (e)
                                            return "ad_group_asset_operation." + e;
                                    }
                                }
                                if (m.ad_group_bid_modifier_operation != null && m.hasOwnProperty("ad_group_bid_modifier_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.verify(m.ad_group_bid_modifier_operation);
                                        if (e)
                                            return "ad_group_bid_modifier_operation." + e;
                                    }
                                }
                                if (m.ad_group_criterion_customizer_operation != null && m.hasOwnProperty("ad_group_criterion_customizer_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.verify(m.ad_group_criterion_customizer_operation);
                                        if (e)
                                            return "ad_group_criterion_customizer_operation." + e;
                                    }
                                }
                                if (m.ad_group_criterion_label_operation != null && m.hasOwnProperty("ad_group_criterion_label_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.verify(m.ad_group_criterion_label_operation);
                                        if (e)
                                            return "ad_group_criterion_label_operation." + e;
                                    }
                                }
                                if (m.ad_group_criterion_operation != null && m.hasOwnProperty("ad_group_criterion_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.verify(m.ad_group_criterion_operation);
                                        if (e)
                                            return "ad_group_criterion_operation." + e;
                                    }
                                }
                                if (m.ad_group_customizer_operation != null && m.hasOwnProperty("ad_group_customizer_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.verify(m.ad_group_customizer_operation);
                                        if (e)
                                            return "ad_group_customizer_operation." + e;
                                    }
                                }
                                if (m.ad_group_extension_setting_operation != null && m.hasOwnProperty("ad_group_extension_setting_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.verify(m.ad_group_extension_setting_operation);
                                        if (e)
                                            return "ad_group_extension_setting_operation." + e;
                                    }
                                }
                                if (m.ad_group_feed_operation != null && m.hasOwnProperty("ad_group_feed_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupFeedOperation.verify(m.ad_group_feed_operation);
                                        if (e)
                                            return "ad_group_feed_operation." + e;
                                    }
                                }
                                if (m.ad_group_label_operation != null && m.hasOwnProperty("ad_group_label_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupLabelOperation.verify(m.ad_group_label_operation);
                                        if (e)
                                            return "ad_group_label_operation." + e;
                                    }
                                }
                                if (m.ad_group_operation != null && m.hasOwnProperty("ad_group_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdGroupOperation.verify(m.ad_group_operation);
                                        if (e)
                                            return "ad_group_operation." + e;
                                    }
                                }
                                if (m.ad_operation != null && m.hasOwnProperty("ad_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdOperation.verify(m.ad_operation);
                                        if (e)
                                            return "ad_operation." + e;
                                    }
                                }
                                if (m.ad_parameter_operation != null && m.hasOwnProperty("ad_parameter_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AdParameterOperation.verify(m.ad_parameter_operation);
                                        if (e)
                                            return "ad_parameter_operation." + e;
                                    }
                                }
                                if (m.asset_operation != null && m.hasOwnProperty("asset_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AssetOperation.verify(m.asset_operation);
                                        if (e)
                                            return "asset_operation." + e;
                                    }
                                }
                                if (m.asset_group_asset_operation != null && m.hasOwnProperty("asset_group_asset_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.verify(m.asset_group_asset_operation);
                                        if (e)
                                            return "asset_group_asset_operation." + e;
                                    }
                                }
                                if (m.asset_group_listing_group_filter_operation != null && m.hasOwnProperty("asset_group_listing_group_filter_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.verify(m.asset_group_listing_group_filter_operation);
                                        if (e)
                                            return "asset_group_listing_group_filter_operation." + e;
                                    }
                                }
                                if (m.asset_group_signal_operation != null && m.hasOwnProperty("asset_group_signal_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.verify(m.asset_group_signal_operation);
                                        if (e)
                                            return "asset_group_signal_operation." + e;
                                    }
                                }
                                if (m.asset_group_operation != null && m.hasOwnProperty("asset_group_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AssetGroupOperation.verify(m.asset_group_operation);
                                        if (e)
                                            return "asset_group_operation." + e;
                                    }
                                }
                                if (m.asset_set_asset_operation != null && m.hasOwnProperty("asset_set_asset_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AssetSetAssetOperation.verify(m.asset_set_asset_operation);
                                        if (e)
                                            return "asset_set_asset_operation." + e;
                                    }
                                }
                                if (m.asset_set_operation != null && m.hasOwnProperty("asset_set_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AssetSetOperation.verify(m.asset_set_operation);
                                        if (e)
                                            return "asset_set_operation." + e;
                                    }
                                }
                                if (m.audience_operation != null && m.hasOwnProperty("audience_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.AudienceOperation.verify(m.audience_operation);
                                        if (e)
                                            return "audience_operation." + e;
                                    }
                                }
                                if (m.bidding_data_exclusion_operation != null && m.hasOwnProperty("bidding_data_exclusion_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.verify(m.bidding_data_exclusion_operation);
                                        if (e)
                                            return "bidding_data_exclusion_operation." + e;
                                    }
                                }
                                if (m.bidding_seasonality_adjustment_operation != null && m.hasOwnProperty("bidding_seasonality_adjustment_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.verify(m.bidding_seasonality_adjustment_operation);
                                        if (e)
                                            return "bidding_seasonality_adjustment_operation." + e;
                                    }
                                }
                                if (m.bidding_strategy_operation != null && m.hasOwnProperty("bidding_strategy_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.BiddingStrategyOperation.verify(m.bidding_strategy_operation);
                                        if (e)
                                            return "bidding_strategy_operation." + e;
                                    }
                                }
                                if (m.campaign_asset_operation != null && m.hasOwnProperty("campaign_asset_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignAssetOperation.verify(m.campaign_asset_operation);
                                        if (e)
                                            return "campaign_asset_operation." + e;
                                    }
                                }
                                if (m.campaign_asset_set_operation != null && m.hasOwnProperty("campaign_asset_set_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.verify(m.campaign_asset_set_operation);
                                        if (e)
                                            return "campaign_asset_set_operation." + e;
                                    }
                                }
                                if (m.campaign_bid_modifier_operation != null && m.hasOwnProperty("campaign_bid_modifier_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.verify(m.campaign_bid_modifier_operation);
                                        if (e)
                                            return "campaign_bid_modifier_operation." + e;
                                    }
                                }
                                if (m.campaign_budget_operation != null && m.hasOwnProperty("campaign_budget_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignBudgetOperation.verify(m.campaign_budget_operation);
                                        if (e)
                                            return "campaign_budget_operation." + e;
                                    }
                                }
                                if (m.campaign_conversion_goal_operation != null && m.hasOwnProperty("campaign_conversion_goal_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.verify(m.campaign_conversion_goal_operation);
                                        if (e)
                                            return "campaign_conversion_goal_operation." + e;
                                    }
                                }
                                if (m.campaign_criterion_operation != null && m.hasOwnProperty("campaign_criterion_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignCriterionOperation.verify(m.campaign_criterion_operation);
                                        if (e)
                                            return "campaign_criterion_operation." + e;
                                    }
                                }
                                if (m.campaign_customizer_operation != null && m.hasOwnProperty("campaign_customizer_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.verify(m.campaign_customizer_operation);
                                        if (e)
                                            return "campaign_customizer_operation." + e;
                                    }
                                }
                                if (m.campaign_draft_operation != null && m.hasOwnProperty("campaign_draft_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignDraftOperation.verify(m.campaign_draft_operation);
                                        if (e)
                                            return "campaign_draft_operation." + e;
                                    }
                                }
                                if (m.campaign_experiment_operation != null && m.hasOwnProperty("campaign_experiment_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignExperimentOperation.verify(m.campaign_experiment_operation);
                                        if (e)
                                            return "campaign_experiment_operation." + e;
                                    }
                                }
                                if (m.campaign_extension_setting_operation != null && m.hasOwnProperty("campaign_extension_setting_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.verify(m.campaign_extension_setting_operation);
                                        if (e)
                                            return "campaign_extension_setting_operation." + e;
                                    }
                                }
                                if (m.campaign_feed_operation != null && m.hasOwnProperty("campaign_feed_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignFeedOperation.verify(m.campaign_feed_operation);
                                        if (e)
                                            return "campaign_feed_operation." + e;
                                    }
                                }
                                if (m.campaign_group_operation != null && m.hasOwnProperty("campaign_group_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignGroupOperation.verify(m.campaign_group_operation);
                                        if (e)
                                            return "campaign_group_operation." + e;
                                    }
                                }
                                if (m.campaign_label_operation != null && m.hasOwnProperty("campaign_label_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignLabelOperation.verify(m.campaign_label_operation);
                                        if (e)
                                            return "campaign_label_operation." + e;
                                    }
                                }
                                if (m.campaign_operation != null && m.hasOwnProperty("campaign_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignOperation.verify(m.campaign_operation);
                                        if (e)
                                            return "campaign_operation." + e;
                                    }
                                }
                                if (m.campaign_shared_set_operation != null && m.hasOwnProperty("campaign_shared_set_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.verify(m.campaign_shared_set_operation);
                                        if (e)
                                            return "campaign_shared_set_operation." + e;
                                    }
                                }
                                if (m.conversion_action_operation != null && m.hasOwnProperty("conversion_action_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ConversionActionOperation.verify(m.conversion_action_operation);
                                        if (e)
                                            return "conversion_action_operation." + e;
                                    }
                                }
                                if (m.conversion_custom_variable_operation != null && m.hasOwnProperty("conversion_custom_variable_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.verify(m.conversion_custom_variable_operation);
                                        if (e)
                                            return "conversion_custom_variable_operation." + e;
                                    }
                                }
                                if (m.conversion_goal_campaign_config_operation != null && m.hasOwnProperty("conversion_goal_campaign_config_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.verify(m.conversion_goal_campaign_config_operation);
                                        if (e)
                                            return "conversion_goal_campaign_config_operation." + e;
                                    }
                                }
                                if (m.conversion_value_rule_operation != null && m.hasOwnProperty("conversion_value_rule_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.verify(m.conversion_value_rule_operation);
                                        if (e)
                                            return "conversion_value_rule_operation." + e;
                                    }
                                }
                                if (m.conversion_value_rule_set_operation != null && m.hasOwnProperty("conversion_value_rule_set_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.verify(m.conversion_value_rule_set_operation);
                                        if (e)
                                            return "conversion_value_rule_set_operation." + e;
                                    }
                                }
                                if (m.custom_conversion_goal_operation != null && m.hasOwnProperty("custom_conversion_goal_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.verify(m.custom_conversion_goal_operation);
                                        if (e)
                                            return "custom_conversion_goal_operation." + e;
                                    }
                                }
                                if (m.customer_asset_operation != null && m.hasOwnProperty("customer_asset_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerAssetOperation.verify(m.customer_asset_operation);
                                        if (e)
                                            return "customer_asset_operation." + e;
                                    }
                                }
                                if (m.customer_conversion_goal_operation != null && m.hasOwnProperty("customer_conversion_goal_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.verify(m.customer_conversion_goal_operation);
                                        if (e)
                                            return "customer_conversion_goal_operation." + e;
                                    }
                                }
                                if (m.customer_customizer_operation != null && m.hasOwnProperty("customer_customizer_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.verify(m.customer_customizer_operation);
                                        if (e)
                                            return "customer_customizer_operation." + e;
                                    }
                                }
                                if (m.customer_extension_setting_operation != null && m.hasOwnProperty("customer_extension_setting_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.verify(m.customer_extension_setting_operation);
                                        if (e)
                                            return "customer_extension_setting_operation." + e;
                                    }
                                }
                                if (m.customer_feed_operation != null && m.hasOwnProperty("customer_feed_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerFeedOperation.verify(m.customer_feed_operation);
                                        if (e)
                                            return "customer_feed_operation." + e;
                                    }
                                }
                                if (m.customer_label_operation != null && m.hasOwnProperty("customer_label_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerLabelOperation.verify(m.customer_label_operation);
                                        if (e)
                                            return "customer_label_operation." + e;
                                    }
                                }
                                if (m.customer_negative_criterion_operation != null && m.hasOwnProperty("customer_negative_criterion_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.verify(m.customer_negative_criterion_operation);
                                        if (e)
                                            return "customer_negative_criterion_operation." + e;
                                    }
                                }
                                if (m.customer_operation != null && m.hasOwnProperty("customer_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerOperation.verify(m.customer_operation);
                                        if (e)
                                            return "customer_operation." + e;
                                    }
                                }
                                if (m.customizer_attribute_operation != null && m.hasOwnProperty("customizer_attribute_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.verify(m.customizer_attribute_operation);
                                        if (e)
                                            return "customizer_attribute_operation." + e;
                                    }
                                }
                                if (m.experiment_operation != null && m.hasOwnProperty("experiment_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ExperimentOperation.verify(m.experiment_operation);
                                        if (e)
                                            return "experiment_operation." + e;
                                    }
                                }
                                if (m.experiment_arm_operation != null && m.hasOwnProperty("experiment_arm_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ExperimentArmOperation.verify(m.experiment_arm_operation);
                                        if (e)
                                            return "experiment_arm_operation." + e;
                                    }
                                }
                                if (m.extension_feed_item_operation != null && m.hasOwnProperty("extension_feed_item_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.verify(m.extension_feed_item_operation);
                                        if (e)
                                            return "extension_feed_item_operation." + e;
                                    }
                                }
                                if (m.feed_item_operation != null && m.hasOwnProperty("feed_item_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.FeedItemOperation.verify(m.feed_item_operation);
                                        if (e)
                                            return "feed_item_operation." + e;
                                    }
                                }
                                if (m.feed_item_set_operation != null && m.hasOwnProperty("feed_item_set_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.FeedItemSetOperation.verify(m.feed_item_set_operation);
                                        if (e)
                                            return "feed_item_set_operation." + e;
                                    }
                                }
                                if (m.feed_item_set_link_operation != null && m.hasOwnProperty("feed_item_set_link_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.verify(m.feed_item_set_link_operation);
                                        if (e)
                                            return "feed_item_set_link_operation." + e;
                                    }
                                }
                                if (m.feed_item_target_operation != null && m.hasOwnProperty("feed_item_target_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.FeedItemTargetOperation.verify(m.feed_item_target_operation);
                                        if (e)
                                            return "feed_item_target_operation." + e;
                                    }
                                }
                                if (m.feed_mapping_operation != null && m.hasOwnProperty("feed_mapping_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.FeedMappingOperation.verify(m.feed_mapping_operation);
                                        if (e)
                                            return "feed_mapping_operation." + e;
                                    }
                                }
                                if (m.feed_operation != null && m.hasOwnProperty("feed_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.FeedOperation.verify(m.feed_operation);
                                        if (e)
                                            return "feed_operation." + e;
                                    }
                                }
                                if (m.keyword_plan_ad_group_operation != null && m.hasOwnProperty("keyword_plan_ad_group_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.verify(m.keyword_plan_ad_group_operation);
                                        if (e)
                                            return "keyword_plan_ad_group_operation." + e;
                                    }
                                }
                                if (m.keyword_plan_ad_group_keyword_operation != null && m.hasOwnProperty("keyword_plan_ad_group_keyword_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.verify(m.keyword_plan_ad_group_keyword_operation);
                                        if (e)
                                            return "keyword_plan_ad_group_keyword_operation." + e;
                                    }
                                }
                                if (m.keyword_plan_campaign_keyword_operation != null && m.hasOwnProperty("keyword_plan_campaign_keyword_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.verify(m.keyword_plan_campaign_keyword_operation);
                                        if (e)
                                            return "keyword_plan_campaign_keyword_operation." + e;
                                    }
                                }
                                if (m.keyword_plan_campaign_operation != null && m.hasOwnProperty("keyword_plan_campaign_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.verify(m.keyword_plan_campaign_operation);
                                        if (e)
                                            return "keyword_plan_campaign_operation." + e;
                                    }
                                }
                                if (m.keyword_plan_operation != null && m.hasOwnProperty("keyword_plan_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordPlanOperation.verify(m.keyword_plan_operation);
                                        if (e)
                                            return "keyword_plan_operation." + e;
                                    }
                                }
                                if (m.label_operation != null && m.hasOwnProperty("label_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.LabelOperation.verify(m.label_operation);
                                        if (e)
                                            return "label_operation." + e;
                                    }
                                }
                                if (m.media_file_operation != null && m.hasOwnProperty("media_file_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MediaFileOperation.verify(m.media_file_operation);
                                        if (e)
                                            return "media_file_operation." + e;
                                    }
                                }
                                if (m.remarketing_action_operation != null && m.hasOwnProperty("remarketing_action_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.RemarketingActionOperation.verify(m.remarketing_action_operation);
                                        if (e)
                                            return "remarketing_action_operation." + e;
                                    }
                                }
                                if (m.shared_criterion_operation != null && m.hasOwnProperty("shared_criterion_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SharedCriterionOperation.verify(m.shared_criterion_operation);
                                        if (e)
                                            return "shared_criterion_operation." + e;
                                    }
                                }
                                if (m.shared_set_operation != null && m.hasOwnProperty("shared_set_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SharedSetOperation.verify(m.shared_set_operation);
                                        if (e)
                                            return "shared_set_operation." + e;
                                    }
                                }
                                if (m.smart_campaign_setting_operation != null && m.hasOwnProperty("smart_campaign_setting_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.verify(m.smart_campaign_setting_operation);
                                        if (e)
                                            return "smart_campaign_setting_operation." + e;
                                    }
                                }
                                if (m.user_list_operation != null && m.hasOwnProperty("user_list_operation")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.UserListOperation.verify(m.user_list_operation);
                                        if (e)
                                            return "user_list_operation." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateOperation} MutateOperation
                             */
                            MutateOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateOperation();
                                if (d.ad_group_ad_label_operation != null) {
                                    if (typeof d.ad_group_ad_label_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_ad_label_operation: object expected");
                                    m.ad_group_ad_label_operation = $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.fromObject(d.ad_group_ad_label_operation);
                                }
                                if (d.ad_group_ad_operation != null) {
                                    if (typeof d.ad_group_ad_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_ad_operation: object expected");
                                    m.ad_group_ad_operation = $root.google.ads.googleads.v11.services.AdGroupAdOperation.fromObject(d.ad_group_ad_operation);
                                }
                                if (d.ad_group_asset_operation != null) {
                                    if (typeof d.ad_group_asset_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_asset_operation: object expected");
                                    m.ad_group_asset_operation = $root.google.ads.googleads.v11.services.AdGroupAssetOperation.fromObject(d.ad_group_asset_operation);
                                }
                                if (d.ad_group_bid_modifier_operation != null) {
                                    if (typeof d.ad_group_bid_modifier_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_bid_modifier_operation: object expected");
                                    m.ad_group_bid_modifier_operation = $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.fromObject(d.ad_group_bid_modifier_operation);
                                }
                                if (d.ad_group_criterion_customizer_operation != null) {
                                    if (typeof d.ad_group_criterion_customizer_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_criterion_customizer_operation: object expected");
                                    m.ad_group_criterion_customizer_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.fromObject(d.ad_group_criterion_customizer_operation);
                                }
                                if (d.ad_group_criterion_label_operation != null) {
                                    if (typeof d.ad_group_criterion_label_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_criterion_label_operation: object expected");
                                    m.ad_group_criterion_label_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.fromObject(d.ad_group_criterion_label_operation);
                                }
                                if (d.ad_group_criterion_operation != null) {
                                    if (typeof d.ad_group_criterion_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_criterion_operation: object expected");
                                    m.ad_group_criterion_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.fromObject(d.ad_group_criterion_operation);
                                }
                                if (d.ad_group_customizer_operation != null) {
                                    if (typeof d.ad_group_customizer_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_customizer_operation: object expected");
                                    m.ad_group_customizer_operation = $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.fromObject(d.ad_group_customizer_operation);
                                }
                                if (d.ad_group_extension_setting_operation != null) {
                                    if (typeof d.ad_group_extension_setting_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_extension_setting_operation: object expected");
                                    m.ad_group_extension_setting_operation = $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.fromObject(d.ad_group_extension_setting_operation);
                                }
                                if (d.ad_group_feed_operation != null) {
                                    if (typeof d.ad_group_feed_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_feed_operation: object expected");
                                    m.ad_group_feed_operation = $root.google.ads.googleads.v11.services.AdGroupFeedOperation.fromObject(d.ad_group_feed_operation);
                                }
                                if (d.ad_group_label_operation != null) {
                                    if (typeof d.ad_group_label_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_label_operation: object expected");
                                    m.ad_group_label_operation = $root.google.ads.googleads.v11.services.AdGroupLabelOperation.fromObject(d.ad_group_label_operation);
                                }
                                if (d.ad_group_operation != null) {
                                    if (typeof d.ad_group_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_group_operation: object expected");
                                    m.ad_group_operation = $root.google.ads.googleads.v11.services.AdGroupOperation.fromObject(d.ad_group_operation);
                                }
                                if (d.ad_operation != null) {
                                    if (typeof d.ad_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_operation: object expected");
                                    m.ad_operation = $root.google.ads.googleads.v11.services.AdOperation.fromObject(d.ad_operation);
                                }
                                if (d.ad_parameter_operation != null) {
                                    if (typeof d.ad_parameter_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.ad_parameter_operation: object expected");
                                    m.ad_parameter_operation = $root.google.ads.googleads.v11.services.AdParameterOperation.fromObject(d.ad_parameter_operation);
                                }
                                if (d.asset_operation != null) {
                                    if (typeof d.asset_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.asset_operation: object expected");
                                    m.asset_operation = $root.google.ads.googleads.v11.services.AssetOperation.fromObject(d.asset_operation);
                                }
                                if (d.asset_group_asset_operation != null) {
                                    if (typeof d.asset_group_asset_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.asset_group_asset_operation: object expected");
                                    m.asset_group_asset_operation = $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.fromObject(d.asset_group_asset_operation);
                                }
                                if (d.asset_group_listing_group_filter_operation != null) {
                                    if (typeof d.asset_group_listing_group_filter_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.asset_group_listing_group_filter_operation: object expected");
                                    m.asset_group_listing_group_filter_operation = $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.fromObject(d.asset_group_listing_group_filter_operation);
                                }
                                if (d.asset_group_signal_operation != null) {
                                    if (typeof d.asset_group_signal_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.asset_group_signal_operation: object expected");
                                    m.asset_group_signal_operation = $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.fromObject(d.asset_group_signal_operation);
                                }
                                if (d.asset_group_operation != null) {
                                    if (typeof d.asset_group_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.asset_group_operation: object expected");
                                    m.asset_group_operation = $root.google.ads.googleads.v11.services.AssetGroupOperation.fromObject(d.asset_group_operation);
                                }
                                if (d.asset_set_asset_operation != null) {
                                    if (typeof d.asset_set_asset_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.asset_set_asset_operation: object expected");
                                    m.asset_set_asset_operation = $root.google.ads.googleads.v11.services.AssetSetAssetOperation.fromObject(d.asset_set_asset_operation);
                                }
                                if (d.asset_set_operation != null) {
                                    if (typeof d.asset_set_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.asset_set_operation: object expected");
                                    m.asset_set_operation = $root.google.ads.googleads.v11.services.AssetSetOperation.fromObject(d.asset_set_operation);
                                }
                                if (d.audience_operation != null) {
                                    if (typeof d.audience_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.audience_operation: object expected");
                                    m.audience_operation = $root.google.ads.googleads.v11.services.AudienceOperation.fromObject(d.audience_operation);
                                }
                                if (d.bidding_data_exclusion_operation != null) {
                                    if (typeof d.bidding_data_exclusion_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.bidding_data_exclusion_operation: object expected");
                                    m.bidding_data_exclusion_operation = $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.fromObject(d.bidding_data_exclusion_operation);
                                }
                                if (d.bidding_seasonality_adjustment_operation != null) {
                                    if (typeof d.bidding_seasonality_adjustment_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.bidding_seasonality_adjustment_operation: object expected");
                                    m.bidding_seasonality_adjustment_operation = $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.fromObject(d.bidding_seasonality_adjustment_operation);
                                }
                                if (d.bidding_strategy_operation != null) {
                                    if (typeof d.bidding_strategy_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.bidding_strategy_operation: object expected");
                                    m.bidding_strategy_operation = $root.google.ads.googleads.v11.services.BiddingStrategyOperation.fromObject(d.bidding_strategy_operation);
                                }
                                if (d.campaign_asset_operation != null) {
                                    if (typeof d.campaign_asset_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_asset_operation: object expected");
                                    m.campaign_asset_operation = $root.google.ads.googleads.v11.services.CampaignAssetOperation.fromObject(d.campaign_asset_operation);
                                }
                                if (d.campaign_asset_set_operation != null) {
                                    if (typeof d.campaign_asset_set_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_asset_set_operation: object expected");
                                    m.campaign_asset_set_operation = $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.fromObject(d.campaign_asset_set_operation);
                                }
                                if (d.campaign_bid_modifier_operation != null) {
                                    if (typeof d.campaign_bid_modifier_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_bid_modifier_operation: object expected");
                                    m.campaign_bid_modifier_operation = $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.fromObject(d.campaign_bid_modifier_operation);
                                }
                                if (d.campaign_budget_operation != null) {
                                    if (typeof d.campaign_budget_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_budget_operation: object expected");
                                    m.campaign_budget_operation = $root.google.ads.googleads.v11.services.CampaignBudgetOperation.fromObject(d.campaign_budget_operation);
                                }
                                if (d.campaign_conversion_goal_operation != null) {
                                    if (typeof d.campaign_conversion_goal_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_conversion_goal_operation: object expected");
                                    m.campaign_conversion_goal_operation = $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.fromObject(d.campaign_conversion_goal_operation);
                                }
                                if (d.campaign_criterion_operation != null) {
                                    if (typeof d.campaign_criterion_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_criterion_operation: object expected");
                                    m.campaign_criterion_operation = $root.google.ads.googleads.v11.services.CampaignCriterionOperation.fromObject(d.campaign_criterion_operation);
                                }
                                if (d.campaign_customizer_operation != null) {
                                    if (typeof d.campaign_customizer_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_customizer_operation: object expected");
                                    m.campaign_customizer_operation = $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.fromObject(d.campaign_customizer_operation);
                                }
                                if (d.campaign_draft_operation != null) {
                                    if (typeof d.campaign_draft_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_draft_operation: object expected");
                                    m.campaign_draft_operation = $root.google.ads.googleads.v11.services.CampaignDraftOperation.fromObject(d.campaign_draft_operation);
                                }
                                if (d.campaign_experiment_operation != null) {
                                    if (typeof d.campaign_experiment_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_experiment_operation: object expected");
                                    m.campaign_experiment_operation = $root.google.ads.googleads.v11.services.CampaignExperimentOperation.fromObject(d.campaign_experiment_operation);
                                }
                                if (d.campaign_extension_setting_operation != null) {
                                    if (typeof d.campaign_extension_setting_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_extension_setting_operation: object expected");
                                    m.campaign_extension_setting_operation = $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.fromObject(d.campaign_extension_setting_operation);
                                }
                                if (d.campaign_feed_operation != null) {
                                    if (typeof d.campaign_feed_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_feed_operation: object expected");
                                    m.campaign_feed_operation = $root.google.ads.googleads.v11.services.CampaignFeedOperation.fromObject(d.campaign_feed_operation);
                                }
                                if (d.campaign_group_operation != null) {
                                    if (typeof d.campaign_group_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_group_operation: object expected");
                                    m.campaign_group_operation = $root.google.ads.googleads.v11.services.CampaignGroupOperation.fromObject(d.campaign_group_operation);
                                }
                                if (d.campaign_label_operation != null) {
                                    if (typeof d.campaign_label_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_label_operation: object expected");
                                    m.campaign_label_operation = $root.google.ads.googleads.v11.services.CampaignLabelOperation.fromObject(d.campaign_label_operation);
                                }
                                if (d.campaign_operation != null) {
                                    if (typeof d.campaign_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_operation: object expected");
                                    m.campaign_operation = $root.google.ads.googleads.v11.services.CampaignOperation.fromObject(d.campaign_operation);
                                }
                                if (d.campaign_shared_set_operation != null) {
                                    if (typeof d.campaign_shared_set_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.campaign_shared_set_operation: object expected");
                                    m.campaign_shared_set_operation = $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.fromObject(d.campaign_shared_set_operation);
                                }
                                if (d.conversion_action_operation != null) {
                                    if (typeof d.conversion_action_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.conversion_action_operation: object expected");
                                    m.conversion_action_operation = $root.google.ads.googleads.v11.services.ConversionActionOperation.fromObject(d.conversion_action_operation);
                                }
                                if (d.conversion_custom_variable_operation != null) {
                                    if (typeof d.conversion_custom_variable_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.conversion_custom_variable_operation: object expected");
                                    m.conversion_custom_variable_operation = $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.fromObject(d.conversion_custom_variable_operation);
                                }
                                if (d.conversion_goal_campaign_config_operation != null) {
                                    if (typeof d.conversion_goal_campaign_config_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.conversion_goal_campaign_config_operation: object expected");
                                    m.conversion_goal_campaign_config_operation = $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.fromObject(d.conversion_goal_campaign_config_operation);
                                }
                                if (d.conversion_value_rule_operation != null) {
                                    if (typeof d.conversion_value_rule_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.conversion_value_rule_operation: object expected");
                                    m.conversion_value_rule_operation = $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.fromObject(d.conversion_value_rule_operation);
                                }
                                if (d.conversion_value_rule_set_operation != null) {
                                    if (typeof d.conversion_value_rule_set_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.conversion_value_rule_set_operation: object expected");
                                    m.conversion_value_rule_set_operation = $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.fromObject(d.conversion_value_rule_set_operation);
                                }
                                if (d.custom_conversion_goal_operation != null) {
                                    if (typeof d.custom_conversion_goal_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.custom_conversion_goal_operation: object expected");
                                    m.custom_conversion_goal_operation = $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.fromObject(d.custom_conversion_goal_operation);
                                }
                                if (d.customer_asset_operation != null) {
                                    if (typeof d.customer_asset_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_asset_operation: object expected");
                                    m.customer_asset_operation = $root.google.ads.googleads.v11.services.CustomerAssetOperation.fromObject(d.customer_asset_operation);
                                }
                                if (d.customer_conversion_goal_operation != null) {
                                    if (typeof d.customer_conversion_goal_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_conversion_goal_operation: object expected");
                                    m.customer_conversion_goal_operation = $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.fromObject(d.customer_conversion_goal_operation);
                                }
                                if (d.customer_customizer_operation != null) {
                                    if (typeof d.customer_customizer_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_customizer_operation: object expected");
                                    m.customer_customizer_operation = $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.fromObject(d.customer_customizer_operation);
                                }
                                if (d.customer_extension_setting_operation != null) {
                                    if (typeof d.customer_extension_setting_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_extension_setting_operation: object expected");
                                    m.customer_extension_setting_operation = $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.fromObject(d.customer_extension_setting_operation);
                                }
                                if (d.customer_feed_operation != null) {
                                    if (typeof d.customer_feed_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_feed_operation: object expected");
                                    m.customer_feed_operation = $root.google.ads.googleads.v11.services.CustomerFeedOperation.fromObject(d.customer_feed_operation);
                                }
                                if (d.customer_label_operation != null) {
                                    if (typeof d.customer_label_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_label_operation: object expected");
                                    m.customer_label_operation = $root.google.ads.googleads.v11.services.CustomerLabelOperation.fromObject(d.customer_label_operation);
                                }
                                if (d.customer_negative_criterion_operation != null) {
                                    if (typeof d.customer_negative_criterion_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_negative_criterion_operation: object expected");
                                    m.customer_negative_criterion_operation = $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.fromObject(d.customer_negative_criterion_operation);
                                }
                                if (d.customer_operation != null) {
                                    if (typeof d.customer_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customer_operation: object expected");
                                    m.customer_operation = $root.google.ads.googleads.v11.services.CustomerOperation.fromObject(d.customer_operation);
                                }
                                if (d.customizer_attribute_operation != null) {
                                    if (typeof d.customizer_attribute_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.customizer_attribute_operation: object expected");
                                    m.customizer_attribute_operation = $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.fromObject(d.customizer_attribute_operation);
                                }
                                if (d.experiment_operation != null) {
                                    if (typeof d.experiment_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.experiment_operation: object expected");
                                    m.experiment_operation = $root.google.ads.googleads.v11.services.ExperimentOperation.fromObject(d.experiment_operation);
                                }
                                if (d.experiment_arm_operation != null) {
                                    if (typeof d.experiment_arm_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.experiment_arm_operation: object expected");
                                    m.experiment_arm_operation = $root.google.ads.googleads.v11.services.ExperimentArmOperation.fromObject(d.experiment_arm_operation);
                                }
                                if (d.extension_feed_item_operation != null) {
                                    if (typeof d.extension_feed_item_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.extension_feed_item_operation: object expected");
                                    m.extension_feed_item_operation = $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.fromObject(d.extension_feed_item_operation);
                                }
                                if (d.feed_item_operation != null) {
                                    if (typeof d.feed_item_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.feed_item_operation: object expected");
                                    m.feed_item_operation = $root.google.ads.googleads.v11.services.FeedItemOperation.fromObject(d.feed_item_operation);
                                }
                                if (d.feed_item_set_operation != null) {
                                    if (typeof d.feed_item_set_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.feed_item_set_operation: object expected");
                                    m.feed_item_set_operation = $root.google.ads.googleads.v11.services.FeedItemSetOperation.fromObject(d.feed_item_set_operation);
                                }
                                if (d.feed_item_set_link_operation != null) {
                                    if (typeof d.feed_item_set_link_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.feed_item_set_link_operation: object expected");
                                    m.feed_item_set_link_operation = $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.fromObject(d.feed_item_set_link_operation);
                                }
                                if (d.feed_item_target_operation != null) {
                                    if (typeof d.feed_item_target_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.feed_item_target_operation: object expected");
                                    m.feed_item_target_operation = $root.google.ads.googleads.v11.services.FeedItemTargetOperation.fromObject(d.feed_item_target_operation);
                                }
                                if (d.feed_mapping_operation != null) {
                                    if (typeof d.feed_mapping_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.feed_mapping_operation: object expected");
                                    m.feed_mapping_operation = $root.google.ads.googleads.v11.services.FeedMappingOperation.fromObject(d.feed_mapping_operation);
                                }
                                if (d.feed_operation != null) {
                                    if (typeof d.feed_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.feed_operation: object expected");
                                    m.feed_operation = $root.google.ads.googleads.v11.services.FeedOperation.fromObject(d.feed_operation);
                                }
                                if (d.keyword_plan_ad_group_operation != null) {
                                    if (typeof d.keyword_plan_ad_group_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.keyword_plan_ad_group_operation: object expected");
                                    m.keyword_plan_ad_group_operation = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.fromObject(d.keyword_plan_ad_group_operation);
                                }
                                if (d.keyword_plan_ad_group_keyword_operation != null) {
                                    if (typeof d.keyword_plan_ad_group_keyword_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.keyword_plan_ad_group_keyword_operation: object expected");
                                    m.keyword_plan_ad_group_keyword_operation = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.fromObject(d.keyword_plan_ad_group_keyword_operation);
                                }
                                if (d.keyword_plan_campaign_keyword_operation != null) {
                                    if (typeof d.keyword_plan_campaign_keyword_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.keyword_plan_campaign_keyword_operation: object expected");
                                    m.keyword_plan_campaign_keyword_operation = $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.fromObject(d.keyword_plan_campaign_keyword_operation);
                                }
                                if (d.keyword_plan_campaign_operation != null) {
                                    if (typeof d.keyword_plan_campaign_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.keyword_plan_campaign_operation: object expected");
                                    m.keyword_plan_campaign_operation = $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.fromObject(d.keyword_plan_campaign_operation);
                                }
                                if (d.keyword_plan_operation != null) {
                                    if (typeof d.keyword_plan_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.keyword_plan_operation: object expected");
                                    m.keyword_plan_operation = $root.google.ads.googleads.v11.services.KeywordPlanOperation.fromObject(d.keyword_plan_operation);
                                }
                                if (d.label_operation != null) {
                                    if (typeof d.label_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.label_operation: object expected");
                                    m.label_operation = $root.google.ads.googleads.v11.services.LabelOperation.fromObject(d.label_operation);
                                }
                                if (d.media_file_operation != null) {
                                    if (typeof d.media_file_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.media_file_operation: object expected");
                                    m.media_file_operation = $root.google.ads.googleads.v11.services.MediaFileOperation.fromObject(d.media_file_operation);
                                }
                                if (d.remarketing_action_operation != null) {
                                    if (typeof d.remarketing_action_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.remarketing_action_operation: object expected");
                                    m.remarketing_action_operation = $root.google.ads.googleads.v11.services.RemarketingActionOperation.fromObject(d.remarketing_action_operation);
                                }
                                if (d.shared_criterion_operation != null) {
                                    if (typeof d.shared_criterion_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.shared_criterion_operation: object expected");
                                    m.shared_criterion_operation = $root.google.ads.googleads.v11.services.SharedCriterionOperation.fromObject(d.shared_criterion_operation);
                                }
                                if (d.shared_set_operation != null) {
                                    if (typeof d.shared_set_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.shared_set_operation: object expected");
                                    m.shared_set_operation = $root.google.ads.googleads.v11.services.SharedSetOperation.fromObject(d.shared_set_operation);
                                }
                                if (d.smart_campaign_setting_operation != null) {
                                    if (typeof d.smart_campaign_setting_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.smart_campaign_setting_operation: object expected");
                                    m.smart_campaign_setting_operation = $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.fromObject(d.smart_campaign_setting_operation);
                                }
                                if (d.user_list_operation != null) {
                                    if (typeof d.user_list_operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperation.user_list_operation: object expected");
                                    m.user_list_operation = $root.google.ads.googleads.v11.services.UserListOperation.fromObject(d.user_list_operation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateOperation} m MutateOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.ad_group_ad_operation != null && m.hasOwnProperty("ad_group_ad_operation")) {
                                    d.ad_group_ad_operation = $root.google.ads.googleads.v11.services.AdGroupAdOperation.toObject(m.ad_group_ad_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_ad_operation";
                                }
                                if (m.ad_group_bid_modifier_operation != null && m.hasOwnProperty("ad_group_bid_modifier_operation")) {
                                    d.ad_group_bid_modifier_operation = $root.google.ads.googleads.v11.services.AdGroupBidModifierOperation.toObject(m.ad_group_bid_modifier_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_bid_modifier_operation";
                                }
                                if (m.ad_group_criterion_operation != null && m.hasOwnProperty("ad_group_criterion_operation")) {
                                    d.ad_group_criterion_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionOperation.toObject(m.ad_group_criterion_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_criterion_operation";
                                }
                                if (m.ad_group_operation != null && m.hasOwnProperty("ad_group_operation")) {
                                    d.ad_group_operation = $root.google.ads.googleads.v11.services.AdGroupOperation.toObject(m.ad_group_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_operation";
                                }
                                if (m.bidding_strategy_operation != null && m.hasOwnProperty("bidding_strategy_operation")) {
                                    d.bidding_strategy_operation = $root.google.ads.googleads.v11.services.BiddingStrategyOperation.toObject(m.bidding_strategy_operation, o);
                                    if (o.oneofs)
                                        d.operation = "bidding_strategy_operation";
                                }
                                if (m.campaign_bid_modifier_operation != null && m.hasOwnProperty("campaign_bid_modifier_operation")) {
                                    d.campaign_bid_modifier_operation = $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.toObject(m.campaign_bid_modifier_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_bid_modifier_operation";
                                }
                                if (m.campaign_budget_operation != null && m.hasOwnProperty("campaign_budget_operation")) {
                                    d.campaign_budget_operation = $root.google.ads.googleads.v11.services.CampaignBudgetOperation.toObject(m.campaign_budget_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_budget_operation";
                                }
                                if (m.campaign_group_operation != null && m.hasOwnProperty("campaign_group_operation")) {
                                    d.campaign_group_operation = $root.google.ads.googleads.v11.services.CampaignGroupOperation.toObject(m.campaign_group_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_group_operation";
                                }
                                if (m.campaign_operation != null && m.hasOwnProperty("campaign_operation")) {
                                    d.campaign_operation = $root.google.ads.googleads.v11.services.CampaignOperation.toObject(m.campaign_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_operation";
                                }
                                if (m.campaign_shared_set_operation != null && m.hasOwnProperty("campaign_shared_set_operation")) {
                                    d.campaign_shared_set_operation = $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.toObject(m.campaign_shared_set_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_shared_set_operation";
                                }
                                if (m.conversion_action_operation != null && m.hasOwnProperty("conversion_action_operation")) {
                                    d.conversion_action_operation = $root.google.ads.googleads.v11.services.ConversionActionOperation.toObject(m.conversion_action_operation, o);
                                    if (o.oneofs)
                                        d.operation = "conversion_action_operation";
                                }
                                if (m.campaign_criterion_operation != null && m.hasOwnProperty("campaign_criterion_operation")) {
                                    d.campaign_criterion_operation = $root.google.ads.googleads.v11.services.CampaignCriterionOperation.toObject(m.campaign_criterion_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_criterion_operation";
                                }
                                if (m.shared_criterion_operation != null && m.hasOwnProperty("shared_criterion_operation")) {
                                    d.shared_criterion_operation = $root.google.ads.googleads.v11.services.SharedCriterionOperation.toObject(m.shared_criterion_operation, o);
                                    if (o.oneofs)
                                        d.operation = "shared_criterion_operation";
                                }
                                if (m.shared_set_operation != null && m.hasOwnProperty("shared_set_operation")) {
                                    d.shared_set_operation = $root.google.ads.googleads.v11.services.SharedSetOperation.toObject(m.shared_set_operation, o);
                                    if (o.oneofs)
                                        d.operation = "shared_set_operation";
                                }
                                if (m.user_list_operation != null && m.hasOwnProperty("user_list_operation")) {
                                    d.user_list_operation = $root.google.ads.googleads.v11.services.UserListOperation.toObject(m.user_list_operation, o);
                                    if (o.oneofs)
                                        d.operation = "user_list_operation";
                                }
                                if (m.ad_group_ad_label_operation != null && m.hasOwnProperty("ad_group_ad_label_operation")) {
                                    d.ad_group_ad_label_operation = $root.google.ads.googleads.v11.services.AdGroupAdLabelOperation.toObject(m.ad_group_ad_label_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_ad_label_operation";
                                }
                                if (m.ad_group_criterion_label_operation != null && m.hasOwnProperty("ad_group_criterion_label_operation")) {
                                    d.ad_group_criterion_label_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionLabelOperation.toObject(m.ad_group_criterion_label_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_criterion_label_operation";
                                }
                                if (m.ad_group_extension_setting_operation != null && m.hasOwnProperty("ad_group_extension_setting_operation")) {
                                    d.ad_group_extension_setting_operation = $root.google.ads.googleads.v11.services.AdGroupExtensionSettingOperation.toObject(m.ad_group_extension_setting_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_extension_setting_operation";
                                }
                                if (m.ad_group_feed_operation != null && m.hasOwnProperty("ad_group_feed_operation")) {
                                    d.ad_group_feed_operation = $root.google.ads.googleads.v11.services.AdGroupFeedOperation.toObject(m.ad_group_feed_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_feed_operation";
                                }
                                if (m.ad_group_label_operation != null && m.hasOwnProperty("ad_group_label_operation")) {
                                    d.ad_group_label_operation = $root.google.ads.googleads.v11.services.AdGroupLabelOperation.toObject(m.ad_group_label_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_label_operation";
                                }
                                if (m.ad_parameter_operation != null && m.hasOwnProperty("ad_parameter_operation")) {
                                    d.ad_parameter_operation = $root.google.ads.googleads.v11.services.AdParameterOperation.toObject(m.ad_parameter_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_parameter_operation";
                                }
                                if (m.asset_operation != null && m.hasOwnProperty("asset_operation")) {
                                    d.asset_operation = $root.google.ads.googleads.v11.services.AssetOperation.toObject(m.asset_operation, o);
                                    if (o.oneofs)
                                        d.operation = "asset_operation";
                                }
                                if (m.campaign_draft_operation != null && m.hasOwnProperty("campaign_draft_operation")) {
                                    d.campaign_draft_operation = $root.google.ads.googleads.v11.services.CampaignDraftOperation.toObject(m.campaign_draft_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_draft_operation";
                                }
                                if (m.campaign_experiment_operation != null && m.hasOwnProperty("campaign_experiment_operation")) {
                                    d.campaign_experiment_operation = $root.google.ads.googleads.v11.services.CampaignExperimentOperation.toObject(m.campaign_experiment_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_experiment_operation";
                                }
                                if (m.campaign_extension_setting_operation != null && m.hasOwnProperty("campaign_extension_setting_operation")) {
                                    d.campaign_extension_setting_operation = $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.toObject(m.campaign_extension_setting_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_extension_setting_operation";
                                }
                                if (m.campaign_feed_operation != null && m.hasOwnProperty("campaign_feed_operation")) {
                                    d.campaign_feed_operation = $root.google.ads.googleads.v11.services.CampaignFeedOperation.toObject(m.campaign_feed_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_feed_operation";
                                }
                                if (m.campaign_label_operation != null && m.hasOwnProperty("campaign_label_operation")) {
                                    d.campaign_label_operation = $root.google.ads.googleads.v11.services.CampaignLabelOperation.toObject(m.campaign_label_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_label_operation";
                                }
                                if (m.customer_extension_setting_operation != null && m.hasOwnProperty("customer_extension_setting_operation")) {
                                    d.customer_extension_setting_operation = $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.toObject(m.customer_extension_setting_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_extension_setting_operation";
                                }
                                if (m.customer_feed_operation != null && m.hasOwnProperty("customer_feed_operation")) {
                                    d.customer_feed_operation = $root.google.ads.googleads.v11.services.CustomerFeedOperation.toObject(m.customer_feed_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_feed_operation";
                                }
                                if (m.customer_label_operation != null && m.hasOwnProperty("customer_label_operation")) {
                                    d.customer_label_operation = $root.google.ads.googleads.v11.services.CustomerLabelOperation.toObject(m.customer_label_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_label_operation";
                                }
                                if (m.customer_negative_criterion_operation != null && m.hasOwnProperty("customer_negative_criterion_operation")) {
                                    d.customer_negative_criterion_operation = $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.toObject(m.customer_negative_criterion_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_negative_criterion_operation";
                                }
                                if (m.customer_operation != null && m.hasOwnProperty("customer_operation")) {
                                    d.customer_operation = $root.google.ads.googleads.v11.services.CustomerOperation.toObject(m.customer_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_operation";
                                }
                                if (m.extension_feed_item_operation != null && m.hasOwnProperty("extension_feed_item_operation")) {
                                    d.extension_feed_item_operation = $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.toObject(m.extension_feed_item_operation, o);
                                    if (o.oneofs)
                                        d.operation = "extension_feed_item_operation";
                                }
                                if (m.feed_item_operation != null && m.hasOwnProperty("feed_item_operation")) {
                                    d.feed_item_operation = $root.google.ads.googleads.v11.services.FeedItemOperation.toObject(m.feed_item_operation, o);
                                    if (o.oneofs)
                                        d.operation = "feed_item_operation";
                                }
                                if (m.feed_item_target_operation != null && m.hasOwnProperty("feed_item_target_operation")) {
                                    d.feed_item_target_operation = $root.google.ads.googleads.v11.services.FeedItemTargetOperation.toObject(m.feed_item_target_operation, o);
                                    if (o.oneofs)
                                        d.operation = "feed_item_target_operation";
                                }
                                if (m.feed_mapping_operation != null && m.hasOwnProperty("feed_mapping_operation")) {
                                    d.feed_mapping_operation = $root.google.ads.googleads.v11.services.FeedMappingOperation.toObject(m.feed_mapping_operation, o);
                                    if (o.oneofs)
                                        d.operation = "feed_mapping_operation";
                                }
                                if (m.feed_operation != null && m.hasOwnProperty("feed_operation")) {
                                    d.feed_operation = $root.google.ads.googleads.v11.services.FeedOperation.toObject(m.feed_operation, o);
                                    if (o.oneofs)
                                        d.operation = "feed_operation";
                                }
                                if (m.label_operation != null && m.hasOwnProperty("label_operation")) {
                                    d.label_operation = $root.google.ads.googleads.v11.services.LabelOperation.toObject(m.label_operation, o);
                                    if (o.oneofs)
                                        d.operation = "label_operation";
                                }
                                if (m.media_file_operation != null && m.hasOwnProperty("media_file_operation")) {
                                    d.media_file_operation = $root.google.ads.googleads.v11.services.MediaFileOperation.toObject(m.media_file_operation, o);
                                    if (o.oneofs)
                                        d.operation = "media_file_operation";
                                }
                                if (m.remarketing_action_operation != null && m.hasOwnProperty("remarketing_action_operation")) {
                                    d.remarketing_action_operation = $root.google.ads.googleads.v11.services.RemarketingActionOperation.toObject(m.remarketing_action_operation, o);
                                    if (o.oneofs)
                                        d.operation = "remarketing_action_operation";
                                }
                                if (m.keyword_plan_ad_group_operation != null && m.hasOwnProperty("keyword_plan_ad_group_operation")) {
                                    d.keyword_plan_ad_group_operation = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.toObject(m.keyword_plan_ad_group_operation, o);
                                    if (o.oneofs)
                                        d.operation = "keyword_plan_ad_group_operation";
                                }
                                if (m.keyword_plan_campaign_operation != null && m.hasOwnProperty("keyword_plan_campaign_operation")) {
                                    d.keyword_plan_campaign_operation = $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.toObject(m.keyword_plan_campaign_operation, o);
                                    if (o.oneofs)
                                        d.operation = "keyword_plan_campaign_operation";
                                }
                                if (m.keyword_plan_operation != null && m.hasOwnProperty("keyword_plan_operation")) {
                                    d.keyword_plan_operation = $root.google.ads.googleads.v11.services.KeywordPlanOperation.toObject(m.keyword_plan_operation, o);
                                    if (o.oneofs)
                                        d.operation = "keyword_plan_operation";
                                }
                                if (m.ad_operation != null && m.hasOwnProperty("ad_operation")) {
                                    d.ad_operation = $root.google.ads.googleads.v11.services.AdOperation.toObject(m.ad_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_operation";
                                }
                                if (m.keyword_plan_ad_group_keyword_operation != null && m.hasOwnProperty("keyword_plan_ad_group_keyword_operation")) {
                                    d.keyword_plan_ad_group_keyword_operation = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.toObject(m.keyword_plan_ad_group_keyword_operation, o);
                                    if (o.oneofs)
                                        d.operation = "keyword_plan_ad_group_keyword_operation";
                                }
                                if (m.keyword_plan_campaign_keyword_operation != null && m.hasOwnProperty("keyword_plan_campaign_keyword_operation")) {
                                    d.keyword_plan_campaign_keyword_operation = $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.toObject(m.keyword_plan_campaign_keyword_operation, o);
                                    if (o.oneofs)
                                        d.operation = "keyword_plan_campaign_keyword_operation";
                                }
                                if (m.campaign_asset_operation != null && m.hasOwnProperty("campaign_asset_operation")) {
                                    d.campaign_asset_operation = $root.google.ads.googleads.v11.services.CampaignAssetOperation.toObject(m.campaign_asset_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_asset_operation";
                                }
                                if (m.feed_item_set_operation != null && m.hasOwnProperty("feed_item_set_operation")) {
                                    d.feed_item_set_operation = $root.google.ads.googleads.v11.services.FeedItemSetOperation.toObject(m.feed_item_set_operation, o);
                                    if (o.oneofs)
                                        d.operation = "feed_item_set_operation";
                                }
                                if (m.feed_item_set_link_operation != null && m.hasOwnProperty("feed_item_set_link_operation")) {
                                    d.feed_item_set_link_operation = $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.toObject(m.feed_item_set_link_operation, o);
                                    if (o.oneofs)
                                        d.operation = "feed_item_set_link_operation";
                                }
                                if (m.conversion_custom_variable_operation != null && m.hasOwnProperty("conversion_custom_variable_operation")) {
                                    d.conversion_custom_variable_operation = $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.toObject(m.conversion_custom_variable_operation, o);
                                    if (o.oneofs)
                                        d.operation = "conversion_custom_variable_operation";
                                }
                                if (m.ad_group_asset_operation != null && m.hasOwnProperty("ad_group_asset_operation")) {
                                    d.ad_group_asset_operation = $root.google.ads.googleads.v11.services.AdGroupAssetOperation.toObject(m.ad_group_asset_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_asset_operation";
                                }
                                if (m.customer_asset_operation != null && m.hasOwnProperty("customer_asset_operation")) {
                                    d.customer_asset_operation = $root.google.ads.googleads.v11.services.CustomerAssetOperation.toObject(m.customer_asset_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_asset_operation";
                                }
                                if (m.bidding_data_exclusion_operation != null && m.hasOwnProperty("bidding_data_exclusion_operation")) {
                                    d.bidding_data_exclusion_operation = $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.toObject(m.bidding_data_exclusion_operation, o);
                                    if (o.oneofs)
                                        d.operation = "bidding_data_exclusion_operation";
                                }
                                if (m.bidding_seasonality_adjustment_operation != null && m.hasOwnProperty("bidding_seasonality_adjustment_operation")) {
                                    d.bidding_seasonality_adjustment_operation = $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.toObject(m.bidding_seasonality_adjustment_operation, o);
                                    if (o.oneofs)
                                        d.operation = "bidding_seasonality_adjustment_operation";
                                }
                                if (m.smart_campaign_setting_operation != null && m.hasOwnProperty("smart_campaign_setting_operation")) {
                                    d.smart_campaign_setting_operation = $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.toObject(m.smart_campaign_setting_operation, o);
                                    if (o.oneofs)
                                        d.operation = "smart_campaign_setting_operation";
                                }
                                if (m.asset_group_operation != null && m.hasOwnProperty("asset_group_operation")) {
                                    d.asset_group_operation = $root.google.ads.googleads.v11.services.AssetGroupOperation.toObject(m.asset_group_operation, o);
                                    if (o.oneofs)
                                        d.operation = "asset_group_operation";
                                }
                                if (m.conversion_value_rule_operation != null && m.hasOwnProperty("conversion_value_rule_operation")) {
                                    d.conversion_value_rule_operation = $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.toObject(m.conversion_value_rule_operation, o);
                                    if (o.oneofs)
                                        d.operation = "conversion_value_rule_operation";
                                }
                                if (m.conversion_value_rule_set_operation != null && m.hasOwnProperty("conversion_value_rule_set_operation")) {
                                    d.conversion_value_rule_set_operation = $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.toObject(m.conversion_value_rule_set_operation, o);
                                    if (o.oneofs)
                                        d.operation = "conversion_value_rule_set_operation";
                                }
                                if (m.asset_group_asset_operation != null && m.hasOwnProperty("asset_group_asset_operation")) {
                                    d.asset_group_asset_operation = $root.google.ads.googleads.v11.services.AssetGroupAssetOperation.toObject(m.asset_group_asset_operation, o);
                                    if (o.oneofs)
                                        d.operation = "asset_group_asset_operation";
                                }
                                if (m.customer_conversion_goal_operation != null && m.hasOwnProperty("customer_conversion_goal_operation")) {
                                    d.customer_conversion_goal_operation = $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.toObject(m.customer_conversion_goal_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_conversion_goal_operation";
                                }
                                if (m.campaign_conversion_goal_operation != null && m.hasOwnProperty("campaign_conversion_goal_operation")) {
                                    d.campaign_conversion_goal_operation = $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.toObject(m.campaign_conversion_goal_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_conversion_goal_operation";
                                }
                                if (m.custom_conversion_goal_operation != null && m.hasOwnProperty("custom_conversion_goal_operation")) {
                                    d.custom_conversion_goal_operation = $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.toObject(m.custom_conversion_goal_operation, o);
                                    if (o.oneofs)
                                        d.operation = "custom_conversion_goal_operation";
                                }
                                if (m.conversion_goal_campaign_config_operation != null && m.hasOwnProperty("conversion_goal_campaign_config_operation")) {
                                    d.conversion_goal_campaign_config_operation = $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.toObject(m.conversion_goal_campaign_config_operation, o);
                                    if (o.oneofs)
                                        d.operation = "conversion_goal_campaign_config_operation";
                                }
                                if (m.customizer_attribute_operation != null && m.hasOwnProperty("customizer_attribute_operation")) {
                                    d.customizer_attribute_operation = $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.toObject(m.customizer_attribute_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customizer_attribute_operation";
                                }
                                if (m.asset_set_asset_operation != null && m.hasOwnProperty("asset_set_asset_operation")) {
                                    d.asset_set_asset_operation = $root.google.ads.googleads.v11.services.AssetSetAssetOperation.toObject(m.asset_set_asset_operation, o);
                                    if (o.oneofs)
                                        d.operation = "asset_set_asset_operation";
                                }
                                if (m.asset_set_operation != null && m.hasOwnProperty("asset_set_operation")) {
                                    d.asset_set_operation = $root.google.ads.googleads.v11.services.AssetSetOperation.toObject(m.asset_set_operation, o);
                                    if (o.oneofs)
                                        d.operation = "asset_set_operation";
                                }
                                if (m.campaign_asset_set_operation != null && m.hasOwnProperty("campaign_asset_set_operation")) {
                                    d.campaign_asset_set_operation = $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.toObject(m.campaign_asset_set_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_asset_set_operation";
                                }
                                if (m.ad_group_customizer_operation != null && m.hasOwnProperty("ad_group_customizer_operation")) {
                                    d.ad_group_customizer_operation = $root.google.ads.googleads.v11.services.AdGroupCustomizerOperation.toObject(m.ad_group_customizer_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_customizer_operation";
                                }
                                if (m.campaign_customizer_operation != null && m.hasOwnProperty("campaign_customizer_operation")) {
                                    d.campaign_customizer_operation = $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.toObject(m.campaign_customizer_operation, o);
                                    if (o.oneofs)
                                        d.operation = "campaign_customizer_operation";
                                }
                                if (m.ad_group_criterion_customizer_operation != null && m.hasOwnProperty("ad_group_criterion_customizer_operation")) {
                                    d.ad_group_criterion_customizer_operation = $root.google.ads.googleads.v11.services.AdGroupCriterionCustomizerOperation.toObject(m.ad_group_criterion_customizer_operation, o);
                                    if (o.oneofs)
                                        d.operation = "ad_group_criterion_customizer_operation";
                                }
                                if (m.asset_group_listing_group_filter_operation != null && m.hasOwnProperty("asset_group_listing_group_filter_operation")) {
                                    d.asset_group_listing_group_filter_operation = $root.google.ads.googleads.v11.services.AssetGroupListingGroupFilterOperation.toObject(m.asset_group_listing_group_filter_operation, o);
                                    if (o.oneofs)
                                        d.operation = "asset_group_listing_group_filter_operation";
                                }
                                if (m.customer_customizer_operation != null && m.hasOwnProperty("customer_customizer_operation")) {
                                    d.customer_customizer_operation = $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.toObject(m.customer_customizer_operation, o);
                                    if (o.oneofs)
                                        d.operation = "customer_customizer_operation";
                                }
                                if (m.asset_group_signal_operation != null && m.hasOwnProperty("asset_group_signal_operation")) {
                                    d.asset_group_signal_operation = $root.google.ads.googleads.v11.services.AssetGroupSignalOperation.toObject(m.asset_group_signal_operation, o);
                                    if (o.oneofs)
                                        d.operation = "asset_group_signal_operation";
                                }
                                if (m.audience_operation != null && m.hasOwnProperty("audience_operation")) {
                                    d.audience_operation = $root.google.ads.googleads.v11.services.AudienceOperation.toObject(m.audience_operation, o);
                                    if (o.oneofs)
                                        d.operation = "audience_operation";
                                }
                                if (m.experiment_operation != null && m.hasOwnProperty("experiment_operation")) {
                                    d.experiment_operation = $root.google.ads.googleads.v11.services.ExperimentOperation.toObject(m.experiment_operation, o);
                                    if (o.oneofs)
                                        d.operation = "experiment_operation";
                                }
                                if (m.experiment_arm_operation != null && m.hasOwnProperty("experiment_arm_operation")) {
                                    d.experiment_arm_operation = $root.google.ads.googleads.v11.services.ExperimentArmOperation.toObject(m.experiment_arm_operation, o);
                                    if (o.oneofs)
                                        d.operation = "experiment_arm_operation";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateOperation;
                        })();
    
                        services.MutateOperationResponse = (function() {
    
                            /**
                             * Properties of a MutateOperationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateOperationResponse
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult|null} [ad_group_ad_label_result] MutateOperationResponse ad_group_ad_label_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupAdResult|null} [ad_group_ad_result] MutateOperationResponse ad_group_ad_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupAssetResult|null} [ad_group_asset_result] MutateOperationResponse ad_group_asset_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult|null} [ad_group_bid_modifier_result] MutateOperationResponse ad_group_bid_modifier_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult|null} [ad_group_criterion_customizer_result] MutateOperationResponse ad_group_criterion_customizer_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult|null} [ad_group_criterion_label_result] MutateOperationResponse ad_group_criterion_label_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupCriterionResult|null} [ad_group_criterion_result] MutateOperationResponse ad_group_criterion_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult|null} [ad_group_customizer_result] MutateOperationResponse ad_group_customizer_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult|null} [ad_group_extension_setting_result] MutateOperationResponse ad_group_extension_setting_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupFeedResult|null} [ad_group_feed_result] MutateOperationResponse ad_group_feed_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupLabelResult|null} [ad_group_label_result] MutateOperationResponse ad_group_label_result
                             * @property {google.ads.googleads.v11.services.IMutateAdGroupResult|null} [ad_group_result] MutateOperationResponse ad_group_result
                             * @property {google.ads.googleads.v11.services.IMutateAdParameterResult|null} [ad_parameter_result] MutateOperationResponse ad_parameter_result
                             * @property {google.ads.googleads.v11.services.IMutateAdResult|null} [ad_result] MutateOperationResponse ad_result
                             * @property {google.ads.googleads.v11.services.IMutateAssetResult|null} [asset_result] MutateOperationResponse asset_result
                             * @property {google.ads.googleads.v11.services.IMutateAssetGroupAssetResult|null} [asset_group_asset_result] MutateOperationResponse asset_group_asset_result
                             * @property {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult|null} [asset_group_listing_group_filter_result] MutateOperationResponse asset_group_listing_group_filter_result
                             * @property {google.ads.googleads.v11.services.IMutateAssetGroupSignalResult|null} [asset_group_signal_result] MutateOperationResponse asset_group_signal_result
                             * @property {google.ads.googleads.v11.services.IMutateAssetGroupResult|null} [asset_group_result] MutateOperationResponse asset_group_result
                             * @property {google.ads.googleads.v11.services.IMutateAssetSetAssetResult|null} [asset_set_asset_result] MutateOperationResponse asset_set_asset_result
                             * @property {google.ads.googleads.v11.services.IMutateAssetSetResult|null} [asset_set_result] MutateOperationResponse asset_set_result
                             * @property {google.ads.googleads.v11.services.IMutateAudienceResult|null} [audience_result] MutateOperationResponse audience_result
                             * @property {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult|null} [bidding_data_exclusion_result] MutateOperationResponse bidding_data_exclusion_result
                             * @property {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult|null} [bidding_seasonality_adjustment_result] MutateOperationResponse bidding_seasonality_adjustment_result
                             * @property {google.ads.googleads.v11.services.IMutateBiddingStrategyResult|null} [bidding_strategy_result] MutateOperationResponse bidding_strategy_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignAssetResult|null} [campaign_asset_result] MutateOperationResponse campaign_asset_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignAssetSetResult|null} [campaign_asset_set_result] MutateOperationResponse campaign_asset_set_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignBidModifierResult|null} [campaign_bid_modifier_result] MutateOperationResponse campaign_bid_modifier_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignBudgetResult|null} [campaign_budget_result] MutateOperationResponse campaign_budget_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult|null} [campaign_conversion_goal_result] MutateOperationResponse campaign_conversion_goal_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignCriterionResult|null} [campaign_criterion_result] MutateOperationResponse campaign_criterion_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignCustomizerResult|null} [campaign_customizer_result] MutateOperationResponse campaign_customizer_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignDraftResult|null} [campaign_draft_result] MutateOperationResponse campaign_draft_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignExperimentResult|null} [campaign_experiment_result] MutateOperationResponse campaign_experiment_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult|null} [campaign_extension_setting_result] MutateOperationResponse campaign_extension_setting_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignFeedResult|null} [campaign_feed_result] MutateOperationResponse campaign_feed_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignGroupResult|null} [campaign_group_result] MutateOperationResponse campaign_group_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignLabelResult|null} [campaign_label_result] MutateOperationResponse campaign_label_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignResult|null} [campaign_result] MutateOperationResponse campaign_result
                             * @property {google.ads.googleads.v11.services.IMutateCampaignSharedSetResult|null} [campaign_shared_set_result] MutateOperationResponse campaign_shared_set_result
                             * @property {google.ads.googleads.v11.services.IMutateConversionActionResult|null} [conversion_action_result] MutateOperationResponse conversion_action_result
                             * @property {google.ads.googleads.v11.services.IMutateConversionCustomVariableResult|null} [conversion_custom_variable_result] MutateOperationResponse conversion_custom_variable_result
                             * @property {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult|null} [conversion_goal_campaign_config_result] MutateOperationResponse conversion_goal_campaign_config_result
                             * @property {google.ads.googleads.v11.services.IMutateConversionValueRuleResult|null} [conversion_value_rule_result] MutateOperationResponse conversion_value_rule_result
                             * @property {google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult|null} [conversion_value_rule_set_result] MutateOperationResponse conversion_value_rule_set_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomConversionGoalResult|null} [custom_conversion_goal_result] MutateOperationResponse custom_conversion_goal_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerAssetResult|null} [customer_asset_result] MutateOperationResponse customer_asset_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult|null} [customer_conversion_goal_result] MutateOperationResponse customer_conversion_goal_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerCustomizerResult|null} [customer_customizer_result] MutateOperationResponse customer_customizer_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult|null} [customer_extension_setting_result] MutateOperationResponse customer_extension_setting_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerFeedResult|null} [customer_feed_result] MutateOperationResponse customer_feed_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerLabelResult|null} [customer_label_result] MutateOperationResponse customer_label_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult|null} [customer_negative_criterion_result] MutateOperationResponse customer_negative_criterion_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomerResult|null} [customer_result] MutateOperationResponse customer_result
                             * @property {google.ads.googleads.v11.services.IMutateCustomizerAttributeResult|null} [customizer_attribute_result] MutateOperationResponse customizer_attribute_result
                             * @property {google.ads.googleads.v11.services.IMutateExtensionFeedItemResult|null} [extension_feed_item_result] MutateOperationResponse extension_feed_item_result
                             * @property {google.ads.googleads.v11.services.IMutateFeedItemResult|null} [feed_item_result] MutateOperationResponse feed_item_result
                             * @property {google.ads.googleads.v11.services.IMutateFeedItemSetResult|null} [feed_item_set_result] MutateOperationResponse feed_item_set_result
                             * @property {google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult|null} [feed_item_set_link_result] MutateOperationResponse feed_item_set_link_result
                             * @property {google.ads.googleads.v11.services.IMutateFeedItemTargetResult|null} [feed_item_target_result] MutateOperationResponse feed_item_target_result
                             * @property {google.ads.googleads.v11.services.IMutateFeedMappingResult|null} [feed_mapping_result] MutateOperationResponse feed_mapping_result
                             * @property {google.ads.googleads.v11.services.IMutateFeedResult|null} [feed_result] MutateOperationResponse feed_result
                             * @property {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult|null} [keyword_plan_ad_group_result] MutateOperationResponse keyword_plan_ad_group_result
                             * @property {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult|null} [keyword_plan_campaign_result] MutateOperationResponse keyword_plan_campaign_result
                             * @property {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult|null} [keyword_plan_ad_group_keyword_result] MutateOperationResponse keyword_plan_ad_group_keyword_result
                             * @property {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult|null} [keyword_plan_campaign_keyword_result] MutateOperationResponse keyword_plan_campaign_keyword_result
                             * @property {google.ads.googleads.v11.services.IMutateKeywordPlansResult|null} [keyword_plan_result] MutateOperationResponse keyword_plan_result
                             * @property {google.ads.googleads.v11.services.IMutateLabelResult|null} [label_result] MutateOperationResponse label_result
                             * @property {google.ads.googleads.v11.services.IMutateMediaFileResult|null} [media_file_result] MutateOperationResponse media_file_result
                             * @property {google.ads.googleads.v11.services.IMutateRemarketingActionResult|null} [remarketing_action_result] MutateOperationResponse remarketing_action_result
                             * @property {google.ads.googleads.v11.services.IMutateSharedCriterionResult|null} [shared_criterion_result] MutateOperationResponse shared_criterion_result
                             * @property {google.ads.googleads.v11.services.IMutateSharedSetResult|null} [shared_set_result] MutateOperationResponse shared_set_result
                             * @property {google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult|null} [smart_campaign_setting_result] MutateOperationResponse smart_campaign_setting_result
                             * @property {google.ads.googleads.v11.services.IMutateUserListResult|null} [user_list_result] MutateOperationResponse user_list_result
                             */
    
                            /**
                             * Constructs a new MutateOperationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateOperationResponse.
                             * @implements IMutateOperationResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateOperationResponse=} [p] Properties to set
                             */
                            function MutateOperationResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateOperationResponse ad_group_ad_label_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupAdLabelResult|null|undefined} ad_group_ad_label_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_ad_label_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_ad_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupAdResult|null|undefined} ad_group_ad_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_ad_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_asset_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupAssetResult|null|undefined} ad_group_asset_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_asset_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_bid_modifier_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupBidModifierResult|null|undefined} ad_group_bid_modifier_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_bid_modifier_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_criterion_customizer_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupCriterionCustomizerResult|null|undefined} ad_group_criterion_customizer_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_criterion_customizer_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_criterion_label_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupCriterionLabelResult|null|undefined} ad_group_criterion_label_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_criterion_label_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_criterion_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupCriterionResult|null|undefined} ad_group_criterion_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_criterion_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_customizer_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupCustomizerResult|null|undefined} ad_group_customizer_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_customizer_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_extension_setting_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupExtensionSettingResult|null|undefined} ad_group_extension_setting_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_extension_setting_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_feed_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupFeedResult|null|undefined} ad_group_feed_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_feed_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_label_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupLabelResult|null|undefined} ad_group_label_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_label_result = null;
    
                            /**
                             * MutateOperationResponse ad_group_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdGroupResult|null|undefined} ad_group_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_group_result = null;
    
                            /**
                             * MutateOperationResponse ad_parameter_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdParameterResult|null|undefined} ad_parameter_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_parameter_result = null;
    
                            /**
                             * MutateOperationResponse ad_result.
                             * @member {google.ads.googleads.v11.services.IMutateAdResult|null|undefined} ad_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.ad_result = null;
    
                            /**
                             * MutateOperationResponse asset_result.
                             * @member {google.ads.googleads.v11.services.IMutateAssetResult|null|undefined} asset_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.asset_result = null;
    
                            /**
                             * MutateOperationResponse asset_group_asset_result.
                             * @member {google.ads.googleads.v11.services.IMutateAssetGroupAssetResult|null|undefined} asset_group_asset_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.asset_group_asset_result = null;
    
                            /**
                             * MutateOperationResponse asset_group_listing_group_filter_result.
                             * @member {google.ads.googleads.v11.services.IMutateAssetGroupListingGroupFilterResult|null|undefined} asset_group_listing_group_filter_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.asset_group_listing_group_filter_result = null;
    
                            /**
                             * MutateOperationResponse asset_group_signal_result.
                             * @member {google.ads.googleads.v11.services.IMutateAssetGroupSignalResult|null|undefined} asset_group_signal_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.asset_group_signal_result = null;
    
                            /**
                             * MutateOperationResponse asset_group_result.
                             * @member {google.ads.googleads.v11.services.IMutateAssetGroupResult|null|undefined} asset_group_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.asset_group_result = null;
    
                            /**
                             * MutateOperationResponse asset_set_asset_result.
                             * @member {google.ads.googleads.v11.services.IMutateAssetSetAssetResult|null|undefined} asset_set_asset_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.asset_set_asset_result = null;
    
                            /**
                             * MutateOperationResponse asset_set_result.
                             * @member {google.ads.googleads.v11.services.IMutateAssetSetResult|null|undefined} asset_set_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.asset_set_result = null;
    
                            /**
                             * MutateOperationResponse audience_result.
                             * @member {google.ads.googleads.v11.services.IMutateAudienceResult|null|undefined} audience_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.audience_result = null;
    
                            /**
                             * MutateOperationResponse bidding_data_exclusion_result.
                             * @member {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult|null|undefined} bidding_data_exclusion_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.bidding_data_exclusion_result = null;
    
                            /**
                             * MutateOperationResponse bidding_seasonality_adjustment_result.
                             * @member {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult|null|undefined} bidding_seasonality_adjustment_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.bidding_seasonality_adjustment_result = null;
    
                            /**
                             * MutateOperationResponse bidding_strategy_result.
                             * @member {google.ads.googleads.v11.services.IMutateBiddingStrategyResult|null|undefined} bidding_strategy_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.bidding_strategy_result = null;
    
                            /**
                             * MutateOperationResponse campaign_asset_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignAssetResult|null|undefined} campaign_asset_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_asset_result = null;
    
                            /**
                             * MutateOperationResponse campaign_asset_set_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignAssetSetResult|null|undefined} campaign_asset_set_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_asset_set_result = null;
    
                            /**
                             * MutateOperationResponse campaign_bid_modifier_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignBidModifierResult|null|undefined} campaign_bid_modifier_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_bid_modifier_result = null;
    
                            /**
                             * MutateOperationResponse campaign_budget_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignBudgetResult|null|undefined} campaign_budget_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_budget_result = null;
    
                            /**
                             * MutateOperationResponse campaign_conversion_goal_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult|null|undefined} campaign_conversion_goal_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_conversion_goal_result = null;
    
                            /**
                             * MutateOperationResponse campaign_criterion_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignCriterionResult|null|undefined} campaign_criterion_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_criterion_result = null;
    
                            /**
                             * MutateOperationResponse campaign_customizer_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignCustomizerResult|null|undefined} campaign_customizer_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_customizer_result = null;
    
                            /**
                             * MutateOperationResponse campaign_draft_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignDraftResult|null|undefined} campaign_draft_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_draft_result = null;
    
                            /**
                             * MutateOperationResponse campaign_experiment_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignExperimentResult|null|undefined} campaign_experiment_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_experiment_result = null;
    
                            /**
                             * MutateOperationResponse campaign_extension_setting_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult|null|undefined} campaign_extension_setting_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_extension_setting_result = null;
    
                            /**
                             * MutateOperationResponse campaign_feed_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignFeedResult|null|undefined} campaign_feed_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_feed_result = null;
    
                            /**
                             * MutateOperationResponse campaign_group_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignGroupResult|null|undefined} campaign_group_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_group_result = null;
    
                            /**
                             * MutateOperationResponse campaign_label_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignLabelResult|null|undefined} campaign_label_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_label_result = null;
    
                            /**
                             * MutateOperationResponse campaign_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignResult|null|undefined} campaign_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_result = null;
    
                            /**
                             * MutateOperationResponse campaign_shared_set_result.
                             * @member {google.ads.googleads.v11.services.IMutateCampaignSharedSetResult|null|undefined} campaign_shared_set_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.campaign_shared_set_result = null;
    
                            /**
                             * MutateOperationResponse conversion_action_result.
                             * @member {google.ads.googleads.v11.services.IMutateConversionActionResult|null|undefined} conversion_action_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.conversion_action_result = null;
    
                            /**
                             * MutateOperationResponse conversion_custom_variable_result.
                             * @member {google.ads.googleads.v11.services.IMutateConversionCustomVariableResult|null|undefined} conversion_custom_variable_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.conversion_custom_variable_result = null;
    
                            /**
                             * MutateOperationResponse conversion_goal_campaign_config_result.
                             * @member {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult|null|undefined} conversion_goal_campaign_config_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.conversion_goal_campaign_config_result = null;
    
                            /**
                             * MutateOperationResponse conversion_value_rule_result.
                             * @member {google.ads.googleads.v11.services.IMutateConversionValueRuleResult|null|undefined} conversion_value_rule_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.conversion_value_rule_result = null;
    
                            /**
                             * MutateOperationResponse conversion_value_rule_set_result.
                             * @member {google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult|null|undefined} conversion_value_rule_set_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.conversion_value_rule_set_result = null;
    
                            /**
                             * MutateOperationResponse custom_conversion_goal_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomConversionGoalResult|null|undefined} custom_conversion_goal_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.custom_conversion_goal_result = null;
    
                            /**
                             * MutateOperationResponse customer_asset_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerAssetResult|null|undefined} customer_asset_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_asset_result = null;
    
                            /**
                             * MutateOperationResponse customer_conversion_goal_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult|null|undefined} customer_conversion_goal_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_conversion_goal_result = null;
    
                            /**
                             * MutateOperationResponse customer_customizer_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerCustomizerResult|null|undefined} customer_customizer_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_customizer_result = null;
    
                            /**
                             * MutateOperationResponse customer_extension_setting_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult|null|undefined} customer_extension_setting_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_extension_setting_result = null;
    
                            /**
                             * MutateOperationResponse customer_feed_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerFeedResult|null|undefined} customer_feed_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_feed_result = null;
    
                            /**
                             * MutateOperationResponse customer_label_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerLabelResult|null|undefined} customer_label_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_label_result = null;
    
                            /**
                             * MutateOperationResponse customer_negative_criterion_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult|null|undefined} customer_negative_criterion_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_negative_criterion_result = null;
    
                            /**
                             * MutateOperationResponse customer_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerResult|null|undefined} customer_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customer_result = null;
    
                            /**
                             * MutateOperationResponse customizer_attribute_result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomizerAttributeResult|null|undefined} customizer_attribute_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.customizer_attribute_result = null;
    
                            /**
                             * MutateOperationResponse extension_feed_item_result.
                             * @member {google.ads.googleads.v11.services.IMutateExtensionFeedItemResult|null|undefined} extension_feed_item_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.extension_feed_item_result = null;
    
                            /**
                             * MutateOperationResponse feed_item_result.
                             * @member {google.ads.googleads.v11.services.IMutateFeedItemResult|null|undefined} feed_item_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.feed_item_result = null;
    
                            /**
                             * MutateOperationResponse feed_item_set_result.
                             * @member {google.ads.googleads.v11.services.IMutateFeedItemSetResult|null|undefined} feed_item_set_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.feed_item_set_result = null;
    
                            /**
                             * MutateOperationResponse feed_item_set_link_result.
                             * @member {google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult|null|undefined} feed_item_set_link_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.feed_item_set_link_result = null;
    
                            /**
                             * MutateOperationResponse feed_item_target_result.
                             * @member {google.ads.googleads.v11.services.IMutateFeedItemTargetResult|null|undefined} feed_item_target_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.feed_item_target_result = null;
    
                            /**
                             * MutateOperationResponse feed_mapping_result.
                             * @member {google.ads.googleads.v11.services.IMutateFeedMappingResult|null|undefined} feed_mapping_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.feed_mapping_result = null;
    
                            /**
                             * MutateOperationResponse feed_result.
                             * @member {google.ads.googleads.v11.services.IMutateFeedResult|null|undefined} feed_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.feed_result = null;
    
                            /**
                             * MutateOperationResponse keyword_plan_ad_group_result.
                             * @member {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult|null|undefined} keyword_plan_ad_group_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.keyword_plan_ad_group_result = null;
    
                            /**
                             * MutateOperationResponse keyword_plan_campaign_result.
                             * @member {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult|null|undefined} keyword_plan_campaign_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.keyword_plan_campaign_result = null;
    
                            /**
                             * MutateOperationResponse keyword_plan_ad_group_keyword_result.
                             * @member {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult|null|undefined} keyword_plan_ad_group_keyword_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.keyword_plan_ad_group_keyword_result = null;
    
                            /**
                             * MutateOperationResponse keyword_plan_campaign_keyword_result.
                             * @member {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult|null|undefined} keyword_plan_campaign_keyword_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.keyword_plan_campaign_keyword_result = null;
    
                            /**
                             * MutateOperationResponse keyword_plan_result.
                             * @member {google.ads.googleads.v11.services.IMutateKeywordPlansResult|null|undefined} keyword_plan_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.keyword_plan_result = null;
    
                            /**
                             * MutateOperationResponse label_result.
                             * @member {google.ads.googleads.v11.services.IMutateLabelResult|null|undefined} label_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.label_result = null;
    
                            /**
                             * MutateOperationResponse media_file_result.
                             * @member {google.ads.googleads.v11.services.IMutateMediaFileResult|null|undefined} media_file_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.media_file_result = null;
    
                            /**
                             * MutateOperationResponse remarketing_action_result.
                             * @member {google.ads.googleads.v11.services.IMutateRemarketingActionResult|null|undefined} remarketing_action_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.remarketing_action_result = null;
    
                            /**
                             * MutateOperationResponse shared_criterion_result.
                             * @member {google.ads.googleads.v11.services.IMutateSharedCriterionResult|null|undefined} shared_criterion_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.shared_criterion_result = null;
    
                            /**
                             * MutateOperationResponse shared_set_result.
                             * @member {google.ads.googleads.v11.services.IMutateSharedSetResult|null|undefined} shared_set_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.shared_set_result = null;
    
                            /**
                             * MutateOperationResponse smart_campaign_setting_result.
                             * @member {google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult|null|undefined} smart_campaign_setting_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.smart_campaign_setting_result = null;
    
                            /**
                             * MutateOperationResponse user_list_result.
                             * @member {google.ads.googleads.v11.services.IMutateUserListResult|null|undefined} user_list_result
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            MutateOperationResponse.prototype.user_list_result = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MutateOperationResponse response.
                             * @member {"ad_group_ad_label_result"|"ad_group_ad_result"|"ad_group_asset_result"|"ad_group_bid_modifier_result"|"ad_group_criterion_customizer_result"|"ad_group_criterion_label_result"|"ad_group_criterion_result"|"ad_group_customizer_result"|"ad_group_extension_setting_result"|"ad_group_feed_result"|"ad_group_label_result"|"ad_group_result"|"ad_parameter_result"|"ad_result"|"asset_result"|"asset_group_asset_result"|"asset_group_listing_group_filter_result"|"asset_group_signal_result"|"asset_group_result"|"asset_set_asset_result"|"asset_set_result"|"audience_result"|"bidding_data_exclusion_result"|"bidding_seasonality_adjustment_result"|"bidding_strategy_result"|"campaign_asset_result"|"campaign_asset_set_result"|"campaign_bid_modifier_result"|"campaign_budget_result"|"campaign_conversion_goal_result"|"campaign_criterion_result"|"campaign_customizer_result"|"campaign_draft_result"|"campaign_experiment_result"|"campaign_extension_setting_result"|"campaign_feed_result"|"campaign_group_result"|"campaign_label_result"|"campaign_result"|"campaign_shared_set_result"|"conversion_action_result"|"conversion_custom_variable_result"|"conversion_goal_campaign_config_result"|"conversion_value_rule_result"|"conversion_value_rule_set_result"|"custom_conversion_goal_result"|"customer_asset_result"|"customer_conversion_goal_result"|"customer_customizer_result"|"customer_extension_setting_result"|"customer_feed_result"|"customer_label_result"|"customer_negative_criterion_result"|"customer_result"|"customizer_attribute_result"|"extension_feed_item_result"|"feed_item_result"|"feed_item_set_result"|"feed_item_set_link_result"|"feed_item_target_result"|"feed_mapping_result"|"feed_result"|"keyword_plan_ad_group_result"|"keyword_plan_campaign_result"|"keyword_plan_ad_group_keyword_result"|"keyword_plan_campaign_keyword_result"|"keyword_plan_result"|"label_result"|"media_file_result"|"remarketing_action_result"|"shared_criterion_result"|"shared_set_result"|"smart_campaign_setting_result"|"user_list_result"|undefined} response
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             */
                            Object.defineProperty(MutateOperationResponse.prototype, "response", {
                                get: $util.oneOfGetter($oneOfFields = ["ad_group_ad_label_result", "ad_group_ad_result", "ad_group_asset_result", "ad_group_bid_modifier_result", "ad_group_criterion_customizer_result", "ad_group_criterion_label_result", "ad_group_criterion_result", "ad_group_customizer_result", "ad_group_extension_setting_result", "ad_group_feed_result", "ad_group_label_result", "ad_group_result", "ad_parameter_result", "ad_result", "asset_result", "asset_group_asset_result", "asset_group_listing_group_filter_result", "asset_group_signal_result", "asset_group_result", "asset_set_asset_result", "asset_set_result", "audience_result", "bidding_data_exclusion_result", "bidding_seasonality_adjustment_result", "bidding_strategy_result", "campaign_asset_result", "campaign_asset_set_result", "campaign_bid_modifier_result", "campaign_budget_result", "campaign_conversion_goal_result", "campaign_criterion_result", "campaign_customizer_result", "campaign_draft_result", "campaign_experiment_result", "campaign_extension_setting_result", "campaign_feed_result", "campaign_group_result", "campaign_label_result", "campaign_result", "campaign_shared_set_result", "conversion_action_result", "conversion_custom_variable_result", "conversion_goal_campaign_config_result", "conversion_value_rule_result", "conversion_value_rule_set_result", "custom_conversion_goal_result", "customer_asset_result", "customer_conversion_goal_result", "customer_customizer_result", "customer_extension_setting_result", "customer_feed_result", "customer_label_result", "customer_negative_criterion_result", "customer_result", "customizer_attribute_result", "extension_feed_item_result", "feed_item_result", "feed_item_set_result", "feed_item_set_link_result", "feed_item_target_result", "feed_mapping_result", "feed_result", "keyword_plan_ad_group_result", "keyword_plan_campaign_result", "keyword_plan_ad_group_keyword_result", "keyword_plan_campaign_keyword_result", "keyword_plan_result", "label_result", "media_file_result", "remarketing_action_result", "shared_criterion_result", "shared_set_result", "smart_campaign_setting_result", "user_list_result"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MutateOperationResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateOperationResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateOperationResponse} MutateOperationResponse instance
                             */
                            MutateOperationResponse.create = function create(properties) {
                                return new MutateOperationResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateOperationResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateOperationResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateOperationResponse} m MutateOperationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateOperationResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_group_ad_result != null && Object.hasOwnProperty.call(m, "ad_group_ad_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.encode(m.ad_group_ad_result, w.uint32(10).fork()).ldelim();
                                if (m.ad_group_bid_modifier_result != null && Object.hasOwnProperty.call(m, "ad_group_bid_modifier_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.encode(m.ad_group_bid_modifier_result, w.uint32(18).fork()).ldelim();
                                if (m.ad_group_criterion_result != null && Object.hasOwnProperty.call(m, "ad_group_criterion_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.encode(m.ad_group_criterion_result, w.uint32(26).fork()).ldelim();
                                if (m.ad_group_result != null && Object.hasOwnProperty.call(m, "ad_group_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupResult.encode(m.ad_group_result, w.uint32(42).fork()).ldelim();
                                if (m.bidding_strategy_result != null && Object.hasOwnProperty.call(m, "bidding_strategy_result"))
                                    $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.encode(m.bidding_strategy_result, w.uint32(50).fork()).ldelim();
                                if (m.campaign_bid_modifier_result != null && Object.hasOwnProperty.call(m, "campaign_bid_modifier_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.encode(m.campaign_bid_modifier_result, w.uint32(58).fork()).ldelim();
                                if (m.campaign_budget_result != null && Object.hasOwnProperty.call(m, "campaign_budget_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.encode(m.campaign_budget_result, w.uint32(66).fork()).ldelim();
                                if (m.campaign_group_result != null && Object.hasOwnProperty.call(m, "campaign_group_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.encode(m.campaign_group_result, w.uint32(74).fork()).ldelim();
                                if (m.campaign_result != null && Object.hasOwnProperty.call(m, "campaign_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignResult.encode(m.campaign_result, w.uint32(82).fork()).ldelim();
                                if (m.campaign_shared_set_result != null && Object.hasOwnProperty.call(m, "campaign_shared_set_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.encode(m.campaign_shared_set_result, w.uint32(90).fork()).ldelim();
                                if (m.conversion_action_result != null && Object.hasOwnProperty.call(m, "conversion_action_result"))
                                    $root.google.ads.googleads.v11.services.MutateConversionActionResult.encode(m.conversion_action_result, w.uint32(98).fork()).ldelim();
                                if (m.campaign_criterion_result != null && Object.hasOwnProperty.call(m, "campaign_criterion_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.encode(m.campaign_criterion_result, w.uint32(106).fork()).ldelim();
                                if (m.shared_criterion_result != null && Object.hasOwnProperty.call(m, "shared_criterion_result"))
                                    $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.encode(m.shared_criterion_result, w.uint32(114).fork()).ldelim();
                                if (m.shared_set_result != null && Object.hasOwnProperty.call(m, "shared_set_result"))
                                    $root.google.ads.googleads.v11.services.MutateSharedSetResult.encode(m.shared_set_result, w.uint32(122).fork()).ldelim();
                                if (m.user_list_result != null && Object.hasOwnProperty.call(m, "user_list_result"))
                                    $root.google.ads.googleads.v11.services.MutateUserListResult.encode(m.user_list_result, w.uint32(130).fork()).ldelim();
                                if (m.ad_group_ad_label_result != null && Object.hasOwnProperty.call(m, "ad_group_ad_label_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.encode(m.ad_group_ad_label_result, w.uint32(138).fork()).ldelim();
                                if (m.ad_group_criterion_label_result != null && Object.hasOwnProperty.call(m, "ad_group_criterion_label_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.encode(m.ad_group_criterion_label_result, w.uint32(146).fork()).ldelim();
                                if (m.ad_group_extension_setting_result != null && Object.hasOwnProperty.call(m, "ad_group_extension_setting_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.encode(m.ad_group_extension_setting_result, w.uint32(154).fork()).ldelim();
                                if (m.ad_group_feed_result != null && Object.hasOwnProperty.call(m, "ad_group_feed_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.encode(m.ad_group_feed_result, w.uint32(162).fork()).ldelim();
                                if (m.ad_group_label_result != null && Object.hasOwnProperty.call(m, "ad_group_label_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.encode(m.ad_group_label_result, w.uint32(170).fork()).ldelim();
                                if (m.ad_parameter_result != null && Object.hasOwnProperty.call(m, "ad_parameter_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdParameterResult.encode(m.ad_parameter_result, w.uint32(178).fork()).ldelim();
                                if (m.asset_result != null && Object.hasOwnProperty.call(m, "asset_result"))
                                    $root.google.ads.googleads.v11.services.MutateAssetResult.encode(m.asset_result, w.uint32(186).fork()).ldelim();
                                if (m.campaign_draft_result != null && Object.hasOwnProperty.call(m, "campaign_draft_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.encode(m.campaign_draft_result, w.uint32(194).fork()).ldelim();
                                if (m.campaign_experiment_result != null && Object.hasOwnProperty.call(m, "campaign_experiment_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.encode(m.campaign_experiment_result, w.uint32(202).fork()).ldelim();
                                if (m.campaign_extension_setting_result != null && Object.hasOwnProperty.call(m, "campaign_extension_setting_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.encode(m.campaign_extension_setting_result, w.uint32(210).fork()).ldelim();
                                if (m.campaign_feed_result != null && Object.hasOwnProperty.call(m, "campaign_feed_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.encode(m.campaign_feed_result, w.uint32(218).fork()).ldelim();
                                if (m.campaign_label_result != null && Object.hasOwnProperty.call(m, "campaign_label_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.encode(m.campaign_label_result, w.uint32(226).fork()).ldelim();
                                if (m.customer_extension_setting_result != null && Object.hasOwnProperty.call(m, "customer_extension_setting_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.encode(m.customer_extension_setting_result, w.uint32(242).fork()).ldelim();
                                if (m.customer_feed_result != null && Object.hasOwnProperty.call(m, "customer_feed_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.encode(m.customer_feed_result, w.uint32(250).fork()).ldelim();
                                if (m.customer_label_result != null && Object.hasOwnProperty.call(m, "customer_label_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.encode(m.customer_label_result, w.uint32(258).fork()).ldelim();
                                if (m.customer_negative_criterion_result != null && Object.hasOwnProperty.call(m, "customer_negative_criterion_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.encode(m.customer_negative_criterion_result, w.uint32(274).fork()).ldelim();
                                if (m.customer_result != null && Object.hasOwnProperty.call(m, "customer_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerResult.encode(m.customer_result, w.uint32(282).fork()).ldelim();
                                if (m.extension_feed_item_result != null && Object.hasOwnProperty.call(m, "extension_feed_item_result"))
                                    $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.encode(m.extension_feed_item_result, w.uint32(290).fork()).ldelim();
                                if (m.feed_item_result != null && Object.hasOwnProperty.call(m, "feed_item_result"))
                                    $root.google.ads.googleads.v11.services.MutateFeedItemResult.encode(m.feed_item_result, w.uint32(298).fork()).ldelim();
                                if (m.feed_item_target_result != null && Object.hasOwnProperty.call(m, "feed_item_target_result"))
                                    $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.encode(m.feed_item_target_result, w.uint32(306).fork()).ldelim();
                                if (m.feed_mapping_result != null && Object.hasOwnProperty.call(m, "feed_mapping_result"))
                                    $root.google.ads.googleads.v11.services.MutateFeedMappingResult.encode(m.feed_mapping_result, w.uint32(314).fork()).ldelim();
                                if (m.feed_result != null && Object.hasOwnProperty.call(m, "feed_result"))
                                    $root.google.ads.googleads.v11.services.MutateFeedResult.encode(m.feed_result, w.uint32(322).fork()).ldelim();
                                if (m.label_result != null && Object.hasOwnProperty.call(m, "label_result"))
                                    $root.google.ads.googleads.v11.services.MutateLabelResult.encode(m.label_result, w.uint32(330).fork()).ldelim();
                                if (m.media_file_result != null && Object.hasOwnProperty.call(m, "media_file_result"))
                                    $root.google.ads.googleads.v11.services.MutateMediaFileResult.encode(m.media_file_result, w.uint32(338).fork()).ldelim();
                                if (m.remarketing_action_result != null && Object.hasOwnProperty.call(m, "remarketing_action_result"))
                                    $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.encode(m.remarketing_action_result, w.uint32(346).fork()).ldelim();
                                if (m.keyword_plan_ad_group_result != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_result"))
                                    $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.encode(m.keyword_plan_ad_group_result, w.uint32(354).fork()).ldelim();
                                if (m.keyword_plan_campaign_result != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign_result"))
                                    $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.encode(m.keyword_plan_campaign_result, w.uint32(362).fork()).ldelim();
                                if (m.keyword_plan_result != null && Object.hasOwnProperty.call(m, "keyword_plan_result"))
                                    $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.encode(m.keyword_plan_result, w.uint32(386).fork()).ldelim();
                                if (m.ad_result != null && Object.hasOwnProperty.call(m, "ad_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdResult.encode(m.ad_result, w.uint32(394).fork()).ldelim();
                                if (m.keyword_plan_ad_group_keyword_result != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_keyword_result"))
                                    $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.encode(m.keyword_plan_ad_group_keyword_result, w.uint32(402).fork()).ldelim();
                                if (m.keyword_plan_campaign_keyword_result != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign_keyword_result"))
                                    $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.encode(m.keyword_plan_campaign_keyword_result, w.uint32(410).fork()).ldelim();
                                if (m.campaign_asset_result != null && Object.hasOwnProperty.call(m, "campaign_asset_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.encode(m.campaign_asset_result, w.uint32(418).fork()).ldelim();
                                if (m.feed_item_set_result != null && Object.hasOwnProperty.call(m, "feed_item_set_result"))
                                    $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.encode(m.feed_item_set_result, w.uint32(426).fork()).ldelim();
                                if (m.feed_item_set_link_result != null && Object.hasOwnProperty.call(m, "feed_item_set_link_result"))
                                    $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.encode(m.feed_item_set_link_result, w.uint32(434).fork()).ldelim();
                                if (m.conversion_custom_variable_result != null && Object.hasOwnProperty.call(m, "conversion_custom_variable_result"))
                                    $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.encode(m.conversion_custom_variable_result, w.uint32(442).fork()).ldelim();
                                if (m.ad_group_asset_result != null && Object.hasOwnProperty.call(m, "ad_group_asset_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.encode(m.ad_group_asset_result, w.uint32(450).fork()).ldelim();
                                if (m.customer_asset_result != null && Object.hasOwnProperty.call(m, "customer_asset_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.encode(m.customer_asset_result, w.uint32(458).fork()).ldelim();
                                if (m.bidding_data_exclusion_result != null && Object.hasOwnProperty.call(m, "bidding_data_exclusion_result"))
                                    $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.encode(m.bidding_data_exclusion_result, w.uint32(466).fork()).ldelim();
                                if (m.bidding_seasonality_adjustment_result != null && Object.hasOwnProperty.call(m, "bidding_seasonality_adjustment_result"))
                                    $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.encode(m.bidding_seasonality_adjustment_result, w.uint32(474).fork()).ldelim();
                                if (m.smart_campaign_setting_result != null && Object.hasOwnProperty.call(m, "smart_campaign_setting_result"))
                                    $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.encode(m.smart_campaign_setting_result, w.uint32(490).fork()).ldelim();
                                if (m.asset_group_result != null && Object.hasOwnProperty.call(m, "asset_group_result"))
                                    $root.google.ads.googleads.v11.services.MutateAssetGroupResult.encode(m.asset_group_result, w.uint32(498).fork()).ldelim();
                                if (m.conversion_value_rule_result != null && Object.hasOwnProperty.call(m, "conversion_value_rule_result"))
                                    $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.encode(m.conversion_value_rule_result, w.uint32(506).fork()).ldelim();
                                if (m.conversion_value_rule_set_result != null && Object.hasOwnProperty.call(m, "conversion_value_rule_set_result"))
                                    $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.encode(m.conversion_value_rule_set_result, w.uint32(514).fork()).ldelim();
                                if (m.asset_group_asset_result != null && Object.hasOwnProperty.call(m, "asset_group_asset_result"))
                                    $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.encode(m.asset_group_asset_result, w.uint32(522).fork()).ldelim();
                                if (m.customer_conversion_goal_result != null && Object.hasOwnProperty.call(m, "customer_conversion_goal_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.encode(m.customer_conversion_goal_result, w.uint32(530).fork()).ldelim();
                                if (m.campaign_conversion_goal_result != null && Object.hasOwnProperty.call(m, "campaign_conversion_goal_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.encode(m.campaign_conversion_goal_result, w.uint32(538).fork()).ldelim();
                                if (m.custom_conversion_goal_result != null && Object.hasOwnProperty.call(m, "custom_conversion_goal_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.encode(m.custom_conversion_goal_result, w.uint32(546).fork()).ldelim();
                                if (m.conversion_goal_campaign_config_result != null && Object.hasOwnProperty.call(m, "conversion_goal_campaign_config_result"))
                                    $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.encode(m.conversion_goal_campaign_config_result, w.uint32(554).fork()).ldelim();
                                if (m.customizer_attribute_result != null && Object.hasOwnProperty.call(m, "customizer_attribute_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.encode(m.customizer_attribute_result, w.uint32(562).fork()).ldelim();
                                if (m.asset_set_asset_result != null && Object.hasOwnProperty.call(m, "asset_set_asset_result"))
                                    $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.encode(m.asset_set_asset_result, w.uint32(570).fork()).ldelim();
                                if (m.asset_set_result != null && Object.hasOwnProperty.call(m, "asset_set_result"))
                                    $root.google.ads.googleads.v11.services.MutateAssetSetResult.encode(m.asset_set_result, w.uint32(578).fork()).ldelim();
                                if (m.campaign_asset_set_result != null && Object.hasOwnProperty.call(m, "campaign_asset_set_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.encode(m.campaign_asset_set_result, w.uint32(586).fork()).ldelim();
                                if (m.customer_customizer_result != null && Object.hasOwnProperty.call(m, "customer_customizer_result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.encode(m.customer_customizer_result, w.uint32(594).fork()).ldelim();
                                if (m.ad_group_customizer_result != null && Object.hasOwnProperty.call(m, "ad_group_customizer_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.encode(m.ad_group_customizer_result, w.uint32(602).fork()).ldelim();
                                if (m.campaign_customizer_result != null && Object.hasOwnProperty.call(m, "campaign_customizer_result"))
                                    $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.encode(m.campaign_customizer_result, w.uint32(610).fork()).ldelim();
                                if (m.ad_group_criterion_customizer_result != null && Object.hasOwnProperty.call(m, "ad_group_criterion_customizer_result"))
                                    $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.encode(m.ad_group_criterion_customizer_result, w.uint32(618).fork()).ldelim();
                                if (m.asset_group_listing_group_filter_result != null && Object.hasOwnProperty.call(m, "asset_group_listing_group_filter_result"))
                                    $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.encode(m.asset_group_listing_group_filter_result, w.uint32(626).fork()).ldelim();
                                if (m.asset_group_signal_result != null && Object.hasOwnProperty.call(m, "asset_group_signal_result"))
                                    $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.encode(m.asset_group_signal_result, w.uint32(634).fork()).ldelim();
                                if (m.audience_result != null && Object.hasOwnProperty.call(m, "audience_result"))
                                    $root.google.ads.googleads.v11.services.MutateAudienceResult.encode(m.audience_result, w.uint32(642).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateOperationResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateOperationResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateOperationResponse} message MutateOperationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateOperationResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateOperationResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateOperationResponse} MutateOperationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateOperationResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateOperationResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 17:
                                        m.ad_group_ad_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.ad_group_ad_result = $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.decode(r, r.uint32());
                                        break;
                                    case 56:
                                        m.ad_group_asset_result = $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.ad_group_bid_modifier_result = $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.decode(r, r.uint32());
                                        break;
                                    case 77:
                                        m.ad_group_criterion_customizer_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.ad_group_criterion_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.ad_group_criterion_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.decode(r, r.uint32());
                                        break;
                                    case 75:
                                        m.ad_group_customizer_result = $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.decode(r, r.uint32());
                                        break;
                                    case 19:
                                        m.ad_group_extension_setting_result = $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.decode(r, r.uint32());
                                        break;
                                    case 20:
                                        m.ad_group_feed_result = $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.decode(r, r.uint32());
                                        break;
                                    case 21:
                                        m.ad_group_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.ad_group_result = $root.google.ads.googleads.v11.services.MutateAdGroupResult.decode(r, r.uint32());
                                        break;
                                    case 22:
                                        m.ad_parameter_result = $root.google.ads.googleads.v11.services.MutateAdParameterResult.decode(r, r.uint32());
                                        break;
                                    case 49:
                                        m.ad_result = $root.google.ads.googleads.v11.services.MutateAdResult.decode(r, r.uint32());
                                        break;
                                    case 23:
                                        m.asset_result = $root.google.ads.googleads.v11.services.MutateAssetResult.decode(r, r.uint32());
                                        break;
                                    case 65:
                                        m.asset_group_asset_result = $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.decode(r, r.uint32());
                                        break;
                                    case 78:
                                        m.asset_group_listing_group_filter_result = $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.decode(r, r.uint32());
                                        break;
                                    case 79:
                                        m.asset_group_signal_result = $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.decode(r, r.uint32());
                                        break;
                                    case 62:
                                        m.asset_group_result = $root.google.ads.googleads.v11.services.MutateAssetGroupResult.decode(r, r.uint32());
                                        break;
                                    case 71:
                                        m.asset_set_asset_result = $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.decode(r, r.uint32());
                                        break;
                                    case 72:
                                        m.asset_set_result = $root.google.ads.googleads.v11.services.MutateAssetSetResult.decode(r, r.uint32());
                                        break;
                                    case 80:
                                        m.audience_result = $root.google.ads.googleads.v11.services.MutateAudienceResult.decode(r, r.uint32());
                                        break;
                                    case 58:
                                        m.bidding_data_exclusion_result = $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.decode(r, r.uint32());
                                        break;
                                    case 59:
                                        m.bidding_seasonality_adjustment_result = $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.bidding_strategy_result = $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.decode(r, r.uint32());
                                        break;
                                    case 52:
                                        m.campaign_asset_result = $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.decode(r, r.uint32());
                                        break;
                                    case 73:
                                        m.campaign_asset_set_result = $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.campaign_bid_modifier_result = $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.campaign_budget_result = $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.decode(r, r.uint32());
                                        break;
                                    case 67:
                                        m.campaign_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.campaign_criterion_result = $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.decode(r, r.uint32());
                                        break;
                                    case 76:
                                        m.campaign_customizer_result = $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.decode(r, r.uint32());
                                        break;
                                    case 24:
                                        m.campaign_draft_result = $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.decode(r, r.uint32());
                                        break;
                                    case 25:
                                        m.campaign_experiment_result = $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.decode(r, r.uint32());
                                        break;
                                    case 26:
                                        m.campaign_extension_setting_result = $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.decode(r, r.uint32());
                                        break;
                                    case 27:
                                        m.campaign_feed_result = $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.campaign_group_result = $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.decode(r, r.uint32());
                                        break;
                                    case 28:
                                        m.campaign_label_result = $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.campaign_result = $root.google.ads.googleads.v11.services.MutateCampaignResult.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.campaign_shared_set_result = $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.conversion_action_result = $root.google.ads.googleads.v11.services.MutateConversionActionResult.decode(r, r.uint32());
                                        break;
                                    case 55:
                                        m.conversion_custom_variable_result = $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.decode(r, r.uint32());
                                        break;
                                    case 69:
                                        m.conversion_goal_campaign_config_result = $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.decode(r, r.uint32());
                                        break;
                                    case 63:
                                        m.conversion_value_rule_result = $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.decode(r, r.uint32());
                                        break;
                                    case 64:
                                        m.conversion_value_rule_set_result = $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.decode(r, r.uint32());
                                        break;
                                    case 68:
                                        m.custom_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.decode(r, r.uint32());
                                        break;
                                    case 57:
                                        m.customer_asset_result = $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.decode(r, r.uint32());
                                        break;
                                    case 66:
                                        m.customer_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.decode(r, r.uint32());
                                        break;
                                    case 74:
                                        m.customer_customizer_result = $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.decode(r, r.uint32());
                                        break;
                                    case 30:
                                        m.customer_extension_setting_result = $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.decode(r, r.uint32());
                                        break;
                                    case 31:
                                        m.customer_feed_result = $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.decode(r, r.uint32());
                                        break;
                                    case 32:
                                        m.customer_label_result = $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.decode(r, r.uint32());
                                        break;
                                    case 34:
                                        m.customer_negative_criterion_result = $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.decode(r, r.uint32());
                                        break;
                                    case 35:
                                        m.customer_result = $root.google.ads.googleads.v11.services.MutateCustomerResult.decode(r, r.uint32());
                                        break;
                                    case 70:
                                        m.customizer_attribute_result = $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.decode(r, r.uint32());
                                        break;
                                    case 36:
                                        m.extension_feed_item_result = $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.decode(r, r.uint32());
                                        break;
                                    case 37:
                                        m.feed_item_result = $root.google.ads.googleads.v11.services.MutateFeedItemResult.decode(r, r.uint32());
                                        break;
                                    case 53:
                                        m.feed_item_set_result = $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.decode(r, r.uint32());
                                        break;
                                    case 54:
                                        m.feed_item_set_link_result = $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.decode(r, r.uint32());
                                        break;
                                    case 38:
                                        m.feed_item_target_result = $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.decode(r, r.uint32());
                                        break;
                                    case 39:
                                        m.feed_mapping_result = $root.google.ads.googleads.v11.services.MutateFeedMappingResult.decode(r, r.uint32());
                                        break;
                                    case 40:
                                        m.feed_result = $root.google.ads.googleads.v11.services.MutateFeedResult.decode(r, r.uint32());
                                        break;
                                    case 44:
                                        m.keyword_plan_ad_group_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.decode(r, r.uint32());
                                        break;
                                    case 45:
                                        m.keyword_plan_campaign_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.decode(r, r.uint32());
                                        break;
                                    case 50:
                                        m.keyword_plan_ad_group_keyword_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.decode(r, r.uint32());
                                        break;
                                    case 51:
                                        m.keyword_plan_campaign_keyword_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.decode(r, r.uint32());
                                        break;
                                    case 48:
                                        m.keyword_plan_result = $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.decode(r, r.uint32());
                                        break;
                                    case 41:
                                        m.label_result = $root.google.ads.googleads.v11.services.MutateLabelResult.decode(r, r.uint32());
                                        break;
                                    case 42:
                                        m.media_file_result = $root.google.ads.googleads.v11.services.MutateMediaFileResult.decode(r, r.uint32());
                                        break;
                                    case 43:
                                        m.remarketing_action_result = $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.shared_criterion_result = $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        m.shared_set_result = $root.google.ads.googleads.v11.services.MutateSharedSetResult.decode(r, r.uint32());
                                        break;
                                    case 61:
                                        m.smart_campaign_setting_result = $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.decode(r, r.uint32());
                                        break;
                                    case 16:
                                        m.user_list_result = $root.google.ads.googleads.v11.services.MutateUserListResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateOperationResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateOperationResponse} MutateOperationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateOperationResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateOperationResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateOperationResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.ad_group_ad_label_result != null && m.hasOwnProperty("ad_group_ad_label_result")) {
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.verify(m.ad_group_ad_label_result);
                                        if (e)
                                            return "ad_group_ad_label_result." + e;
                                    }
                                }
                                if (m.ad_group_ad_result != null && m.hasOwnProperty("ad_group_ad_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.verify(m.ad_group_ad_result);
                                        if (e)
                                            return "ad_group_ad_result." + e;
                                    }
                                }
                                if (m.ad_group_asset_result != null && m.hasOwnProperty("ad_group_asset_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.verify(m.ad_group_asset_result);
                                        if (e)
                                            return "ad_group_asset_result." + e;
                                    }
                                }
                                if (m.ad_group_bid_modifier_result != null && m.hasOwnProperty("ad_group_bid_modifier_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.verify(m.ad_group_bid_modifier_result);
                                        if (e)
                                            return "ad_group_bid_modifier_result." + e;
                                    }
                                }
                                if (m.ad_group_criterion_customizer_result != null && m.hasOwnProperty("ad_group_criterion_customizer_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.verify(m.ad_group_criterion_customizer_result);
                                        if (e)
                                            return "ad_group_criterion_customizer_result." + e;
                                    }
                                }
                                if (m.ad_group_criterion_label_result != null && m.hasOwnProperty("ad_group_criterion_label_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.verify(m.ad_group_criterion_label_result);
                                        if (e)
                                            return "ad_group_criterion_label_result." + e;
                                    }
                                }
                                if (m.ad_group_criterion_result != null && m.hasOwnProperty("ad_group_criterion_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.verify(m.ad_group_criterion_result);
                                        if (e)
                                            return "ad_group_criterion_result." + e;
                                    }
                                }
                                if (m.ad_group_customizer_result != null && m.hasOwnProperty("ad_group_customizer_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.verify(m.ad_group_customizer_result);
                                        if (e)
                                            return "ad_group_customizer_result." + e;
                                    }
                                }
                                if (m.ad_group_extension_setting_result != null && m.hasOwnProperty("ad_group_extension_setting_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.verify(m.ad_group_extension_setting_result);
                                        if (e)
                                            return "ad_group_extension_setting_result." + e;
                                    }
                                }
                                if (m.ad_group_feed_result != null && m.hasOwnProperty("ad_group_feed_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.verify(m.ad_group_feed_result);
                                        if (e)
                                            return "ad_group_feed_result." + e;
                                    }
                                }
                                if (m.ad_group_label_result != null && m.hasOwnProperty("ad_group_label_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.verify(m.ad_group_label_result);
                                        if (e)
                                            return "ad_group_label_result." + e;
                                    }
                                }
                                if (m.ad_group_result != null && m.hasOwnProperty("ad_group_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdGroupResult.verify(m.ad_group_result);
                                        if (e)
                                            return "ad_group_result." + e;
                                    }
                                }
                                if (m.ad_parameter_result != null && m.hasOwnProperty("ad_parameter_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdParameterResult.verify(m.ad_parameter_result);
                                        if (e)
                                            return "ad_parameter_result." + e;
                                    }
                                }
                                if (m.ad_result != null && m.hasOwnProperty("ad_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAdResult.verify(m.ad_result);
                                        if (e)
                                            return "ad_result." + e;
                                    }
                                }
                                if (m.asset_result != null && m.hasOwnProperty("asset_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAssetResult.verify(m.asset_result);
                                        if (e)
                                            return "asset_result." + e;
                                    }
                                }
                                if (m.asset_group_asset_result != null && m.hasOwnProperty("asset_group_asset_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.verify(m.asset_group_asset_result);
                                        if (e)
                                            return "asset_group_asset_result." + e;
                                    }
                                }
                                if (m.asset_group_listing_group_filter_result != null && m.hasOwnProperty("asset_group_listing_group_filter_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.verify(m.asset_group_listing_group_filter_result);
                                        if (e)
                                            return "asset_group_listing_group_filter_result." + e;
                                    }
                                }
                                if (m.asset_group_signal_result != null && m.hasOwnProperty("asset_group_signal_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.verify(m.asset_group_signal_result);
                                        if (e)
                                            return "asset_group_signal_result." + e;
                                    }
                                }
                                if (m.asset_group_result != null && m.hasOwnProperty("asset_group_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAssetGroupResult.verify(m.asset_group_result);
                                        if (e)
                                            return "asset_group_result." + e;
                                    }
                                }
                                if (m.asset_set_asset_result != null && m.hasOwnProperty("asset_set_asset_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.verify(m.asset_set_asset_result);
                                        if (e)
                                            return "asset_set_asset_result." + e;
                                    }
                                }
                                if (m.asset_set_result != null && m.hasOwnProperty("asset_set_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAssetSetResult.verify(m.asset_set_result);
                                        if (e)
                                            return "asset_set_result." + e;
                                    }
                                }
                                if (m.audience_result != null && m.hasOwnProperty("audience_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateAudienceResult.verify(m.audience_result);
                                        if (e)
                                            return "audience_result." + e;
                                    }
                                }
                                if (m.bidding_data_exclusion_result != null && m.hasOwnProperty("bidding_data_exclusion_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.verify(m.bidding_data_exclusion_result);
                                        if (e)
                                            return "bidding_data_exclusion_result." + e;
                                    }
                                }
                                if (m.bidding_seasonality_adjustment_result != null && m.hasOwnProperty("bidding_seasonality_adjustment_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.verify(m.bidding_seasonality_adjustment_result);
                                        if (e)
                                            return "bidding_seasonality_adjustment_result." + e;
                                    }
                                }
                                if (m.bidding_strategy_result != null && m.hasOwnProperty("bidding_strategy_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.verify(m.bidding_strategy_result);
                                        if (e)
                                            return "bidding_strategy_result." + e;
                                    }
                                }
                                if (m.campaign_asset_result != null && m.hasOwnProperty("campaign_asset_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.verify(m.campaign_asset_result);
                                        if (e)
                                            return "campaign_asset_result." + e;
                                    }
                                }
                                if (m.campaign_asset_set_result != null && m.hasOwnProperty("campaign_asset_set_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.verify(m.campaign_asset_set_result);
                                        if (e)
                                            return "campaign_asset_set_result." + e;
                                    }
                                }
                                if (m.campaign_bid_modifier_result != null && m.hasOwnProperty("campaign_bid_modifier_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.verify(m.campaign_bid_modifier_result);
                                        if (e)
                                            return "campaign_bid_modifier_result." + e;
                                    }
                                }
                                if (m.campaign_budget_result != null && m.hasOwnProperty("campaign_budget_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.verify(m.campaign_budget_result);
                                        if (e)
                                            return "campaign_budget_result." + e;
                                    }
                                }
                                if (m.campaign_conversion_goal_result != null && m.hasOwnProperty("campaign_conversion_goal_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.verify(m.campaign_conversion_goal_result);
                                        if (e)
                                            return "campaign_conversion_goal_result." + e;
                                    }
                                }
                                if (m.campaign_criterion_result != null && m.hasOwnProperty("campaign_criterion_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.verify(m.campaign_criterion_result);
                                        if (e)
                                            return "campaign_criterion_result." + e;
                                    }
                                }
                                if (m.campaign_customizer_result != null && m.hasOwnProperty("campaign_customizer_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.verify(m.campaign_customizer_result);
                                        if (e)
                                            return "campaign_customizer_result." + e;
                                    }
                                }
                                if (m.campaign_draft_result != null && m.hasOwnProperty("campaign_draft_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.verify(m.campaign_draft_result);
                                        if (e)
                                            return "campaign_draft_result." + e;
                                    }
                                }
                                if (m.campaign_experiment_result != null && m.hasOwnProperty("campaign_experiment_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.verify(m.campaign_experiment_result);
                                        if (e)
                                            return "campaign_experiment_result." + e;
                                    }
                                }
                                if (m.campaign_extension_setting_result != null && m.hasOwnProperty("campaign_extension_setting_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.verify(m.campaign_extension_setting_result);
                                        if (e)
                                            return "campaign_extension_setting_result." + e;
                                    }
                                }
                                if (m.campaign_feed_result != null && m.hasOwnProperty("campaign_feed_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.verify(m.campaign_feed_result);
                                        if (e)
                                            return "campaign_feed_result." + e;
                                    }
                                }
                                if (m.campaign_group_result != null && m.hasOwnProperty("campaign_group_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.verify(m.campaign_group_result);
                                        if (e)
                                            return "campaign_group_result." + e;
                                    }
                                }
                                if (m.campaign_label_result != null && m.hasOwnProperty("campaign_label_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.verify(m.campaign_label_result);
                                        if (e)
                                            return "campaign_label_result." + e;
                                    }
                                }
                                if (m.campaign_result != null && m.hasOwnProperty("campaign_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignResult.verify(m.campaign_result);
                                        if (e)
                                            return "campaign_result." + e;
                                    }
                                }
                                if (m.campaign_shared_set_result != null && m.hasOwnProperty("campaign_shared_set_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.verify(m.campaign_shared_set_result);
                                        if (e)
                                            return "campaign_shared_set_result." + e;
                                    }
                                }
                                if (m.conversion_action_result != null && m.hasOwnProperty("conversion_action_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateConversionActionResult.verify(m.conversion_action_result);
                                        if (e)
                                            return "conversion_action_result." + e;
                                    }
                                }
                                if (m.conversion_custom_variable_result != null && m.hasOwnProperty("conversion_custom_variable_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.verify(m.conversion_custom_variable_result);
                                        if (e)
                                            return "conversion_custom_variable_result." + e;
                                    }
                                }
                                if (m.conversion_goal_campaign_config_result != null && m.hasOwnProperty("conversion_goal_campaign_config_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.verify(m.conversion_goal_campaign_config_result);
                                        if (e)
                                            return "conversion_goal_campaign_config_result." + e;
                                    }
                                }
                                if (m.conversion_value_rule_result != null && m.hasOwnProperty("conversion_value_rule_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.verify(m.conversion_value_rule_result);
                                        if (e)
                                            return "conversion_value_rule_result." + e;
                                    }
                                }
                                if (m.conversion_value_rule_set_result != null && m.hasOwnProperty("conversion_value_rule_set_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.verify(m.conversion_value_rule_set_result);
                                        if (e)
                                            return "conversion_value_rule_set_result." + e;
                                    }
                                }
                                if (m.custom_conversion_goal_result != null && m.hasOwnProperty("custom_conversion_goal_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.verify(m.custom_conversion_goal_result);
                                        if (e)
                                            return "custom_conversion_goal_result." + e;
                                    }
                                }
                                if (m.customer_asset_result != null && m.hasOwnProperty("customer_asset_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.verify(m.customer_asset_result);
                                        if (e)
                                            return "customer_asset_result." + e;
                                    }
                                }
                                if (m.customer_conversion_goal_result != null && m.hasOwnProperty("customer_conversion_goal_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.verify(m.customer_conversion_goal_result);
                                        if (e)
                                            return "customer_conversion_goal_result." + e;
                                    }
                                }
                                if (m.customer_customizer_result != null && m.hasOwnProperty("customer_customizer_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.verify(m.customer_customizer_result);
                                        if (e)
                                            return "customer_customizer_result." + e;
                                    }
                                }
                                if (m.customer_extension_setting_result != null && m.hasOwnProperty("customer_extension_setting_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.verify(m.customer_extension_setting_result);
                                        if (e)
                                            return "customer_extension_setting_result." + e;
                                    }
                                }
                                if (m.customer_feed_result != null && m.hasOwnProperty("customer_feed_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.verify(m.customer_feed_result);
                                        if (e)
                                            return "customer_feed_result." + e;
                                    }
                                }
                                if (m.customer_label_result != null && m.hasOwnProperty("customer_label_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.verify(m.customer_label_result);
                                        if (e)
                                            return "customer_label_result." + e;
                                    }
                                }
                                if (m.customer_negative_criterion_result != null && m.hasOwnProperty("customer_negative_criterion_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.verify(m.customer_negative_criterion_result);
                                        if (e)
                                            return "customer_negative_criterion_result." + e;
                                    }
                                }
                                if (m.customer_result != null && m.hasOwnProperty("customer_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerResult.verify(m.customer_result);
                                        if (e)
                                            return "customer_result." + e;
                                    }
                                }
                                if (m.customizer_attribute_result != null && m.hasOwnProperty("customizer_attribute_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.verify(m.customizer_attribute_result);
                                        if (e)
                                            return "customizer_attribute_result." + e;
                                    }
                                }
                                if (m.extension_feed_item_result != null && m.hasOwnProperty("extension_feed_item_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.verify(m.extension_feed_item_result);
                                        if (e)
                                            return "extension_feed_item_result." + e;
                                    }
                                }
                                if (m.feed_item_result != null && m.hasOwnProperty("feed_item_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateFeedItemResult.verify(m.feed_item_result);
                                        if (e)
                                            return "feed_item_result." + e;
                                    }
                                }
                                if (m.feed_item_set_result != null && m.hasOwnProperty("feed_item_set_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.verify(m.feed_item_set_result);
                                        if (e)
                                            return "feed_item_set_result." + e;
                                    }
                                }
                                if (m.feed_item_set_link_result != null && m.hasOwnProperty("feed_item_set_link_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.verify(m.feed_item_set_link_result);
                                        if (e)
                                            return "feed_item_set_link_result." + e;
                                    }
                                }
                                if (m.feed_item_target_result != null && m.hasOwnProperty("feed_item_target_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.verify(m.feed_item_target_result);
                                        if (e)
                                            return "feed_item_target_result." + e;
                                    }
                                }
                                if (m.feed_mapping_result != null && m.hasOwnProperty("feed_mapping_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateFeedMappingResult.verify(m.feed_mapping_result);
                                        if (e)
                                            return "feed_mapping_result." + e;
                                    }
                                }
                                if (m.feed_result != null && m.hasOwnProperty("feed_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateFeedResult.verify(m.feed_result);
                                        if (e)
                                            return "feed_result." + e;
                                    }
                                }
                                if (m.keyword_plan_ad_group_result != null && m.hasOwnProperty("keyword_plan_ad_group_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.verify(m.keyword_plan_ad_group_result);
                                        if (e)
                                            return "keyword_plan_ad_group_result." + e;
                                    }
                                }
                                if (m.keyword_plan_campaign_result != null && m.hasOwnProperty("keyword_plan_campaign_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.verify(m.keyword_plan_campaign_result);
                                        if (e)
                                            return "keyword_plan_campaign_result." + e;
                                    }
                                }
                                if (m.keyword_plan_ad_group_keyword_result != null && m.hasOwnProperty("keyword_plan_ad_group_keyword_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.verify(m.keyword_plan_ad_group_keyword_result);
                                        if (e)
                                            return "keyword_plan_ad_group_keyword_result." + e;
                                    }
                                }
                                if (m.keyword_plan_campaign_keyword_result != null && m.hasOwnProperty("keyword_plan_campaign_keyword_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.verify(m.keyword_plan_campaign_keyword_result);
                                        if (e)
                                            return "keyword_plan_campaign_keyword_result." + e;
                                    }
                                }
                                if (m.keyword_plan_result != null && m.hasOwnProperty("keyword_plan_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.verify(m.keyword_plan_result);
                                        if (e)
                                            return "keyword_plan_result." + e;
                                    }
                                }
                                if (m.label_result != null && m.hasOwnProperty("label_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateLabelResult.verify(m.label_result);
                                        if (e)
                                            return "label_result." + e;
                                    }
                                }
                                if (m.media_file_result != null && m.hasOwnProperty("media_file_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateMediaFileResult.verify(m.media_file_result);
                                        if (e)
                                            return "media_file_result." + e;
                                    }
                                }
                                if (m.remarketing_action_result != null && m.hasOwnProperty("remarketing_action_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.verify(m.remarketing_action_result);
                                        if (e)
                                            return "remarketing_action_result." + e;
                                    }
                                }
                                if (m.shared_criterion_result != null && m.hasOwnProperty("shared_criterion_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.verify(m.shared_criterion_result);
                                        if (e)
                                            return "shared_criterion_result." + e;
                                    }
                                }
                                if (m.shared_set_result != null && m.hasOwnProperty("shared_set_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateSharedSetResult.verify(m.shared_set_result);
                                        if (e)
                                            return "shared_set_result." + e;
                                    }
                                }
                                if (m.smart_campaign_setting_result != null && m.hasOwnProperty("smart_campaign_setting_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.verify(m.smart_campaign_setting_result);
                                        if (e)
                                            return "smart_campaign_setting_result." + e;
                                    }
                                }
                                if (m.user_list_result != null && m.hasOwnProperty("user_list_result")) {
                                    if (p.response === 1)
                                        return "response: multiple values";
                                    p.response = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateUserListResult.verify(m.user_list_result);
                                        if (e)
                                            return "user_list_result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateOperationResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateOperationResponse} MutateOperationResponse
                             */
                            MutateOperationResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateOperationResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateOperationResponse();
                                if (d.ad_group_ad_label_result != null) {
                                    if (typeof d.ad_group_ad_label_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_ad_label_result: object expected");
                                    m.ad_group_ad_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.fromObject(d.ad_group_ad_label_result);
                                }
                                if (d.ad_group_ad_result != null) {
                                    if (typeof d.ad_group_ad_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_ad_result: object expected");
                                    m.ad_group_ad_result = $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.fromObject(d.ad_group_ad_result);
                                }
                                if (d.ad_group_asset_result != null) {
                                    if (typeof d.ad_group_asset_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_asset_result: object expected");
                                    m.ad_group_asset_result = $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.fromObject(d.ad_group_asset_result);
                                }
                                if (d.ad_group_bid_modifier_result != null) {
                                    if (typeof d.ad_group_bid_modifier_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_bid_modifier_result: object expected");
                                    m.ad_group_bid_modifier_result = $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.fromObject(d.ad_group_bid_modifier_result);
                                }
                                if (d.ad_group_criterion_customizer_result != null) {
                                    if (typeof d.ad_group_criterion_customizer_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_criterion_customizer_result: object expected");
                                    m.ad_group_criterion_customizer_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.fromObject(d.ad_group_criterion_customizer_result);
                                }
                                if (d.ad_group_criterion_label_result != null) {
                                    if (typeof d.ad_group_criterion_label_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_criterion_label_result: object expected");
                                    m.ad_group_criterion_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.fromObject(d.ad_group_criterion_label_result);
                                }
                                if (d.ad_group_criterion_result != null) {
                                    if (typeof d.ad_group_criterion_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_criterion_result: object expected");
                                    m.ad_group_criterion_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.fromObject(d.ad_group_criterion_result);
                                }
                                if (d.ad_group_customizer_result != null) {
                                    if (typeof d.ad_group_customizer_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_customizer_result: object expected");
                                    m.ad_group_customizer_result = $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.fromObject(d.ad_group_customizer_result);
                                }
                                if (d.ad_group_extension_setting_result != null) {
                                    if (typeof d.ad_group_extension_setting_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_extension_setting_result: object expected");
                                    m.ad_group_extension_setting_result = $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.fromObject(d.ad_group_extension_setting_result);
                                }
                                if (d.ad_group_feed_result != null) {
                                    if (typeof d.ad_group_feed_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_feed_result: object expected");
                                    m.ad_group_feed_result = $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.fromObject(d.ad_group_feed_result);
                                }
                                if (d.ad_group_label_result != null) {
                                    if (typeof d.ad_group_label_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_label_result: object expected");
                                    m.ad_group_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.fromObject(d.ad_group_label_result);
                                }
                                if (d.ad_group_result != null) {
                                    if (typeof d.ad_group_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_group_result: object expected");
                                    m.ad_group_result = $root.google.ads.googleads.v11.services.MutateAdGroupResult.fromObject(d.ad_group_result);
                                }
                                if (d.ad_parameter_result != null) {
                                    if (typeof d.ad_parameter_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_parameter_result: object expected");
                                    m.ad_parameter_result = $root.google.ads.googleads.v11.services.MutateAdParameterResult.fromObject(d.ad_parameter_result);
                                }
                                if (d.ad_result != null) {
                                    if (typeof d.ad_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.ad_result: object expected");
                                    m.ad_result = $root.google.ads.googleads.v11.services.MutateAdResult.fromObject(d.ad_result);
                                }
                                if (d.asset_result != null) {
                                    if (typeof d.asset_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.asset_result: object expected");
                                    m.asset_result = $root.google.ads.googleads.v11.services.MutateAssetResult.fromObject(d.asset_result);
                                }
                                if (d.asset_group_asset_result != null) {
                                    if (typeof d.asset_group_asset_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.asset_group_asset_result: object expected");
                                    m.asset_group_asset_result = $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.fromObject(d.asset_group_asset_result);
                                }
                                if (d.asset_group_listing_group_filter_result != null) {
                                    if (typeof d.asset_group_listing_group_filter_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.asset_group_listing_group_filter_result: object expected");
                                    m.asset_group_listing_group_filter_result = $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.fromObject(d.asset_group_listing_group_filter_result);
                                }
                                if (d.asset_group_signal_result != null) {
                                    if (typeof d.asset_group_signal_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.asset_group_signal_result: object expected");
                                    m.asset_group_signal_result = $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.fromObject(d.asset_group_signal_result);
                                }
                                if (d.asset_group_result != null) {
                                    if (typeof d.asset_group_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.asset_group_result: object expected");
                                    m.asset_group_result = $root.google.ads.googleads.v11.services.MutateAssetGroupResult.fromObject(d.asset_group_result);
                                }
                                if (d.asset_set_asset_result != null) {
                                    if (typeof d.asset_set_asset_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.asset_set_asset_result: object expected");
                                    m.asset_set_asset_result = $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.fromObject(d.asset_set_asset_result);
                                }
                                if (d.asset_set_result != null) {
                                    if (typeof d.asset_set_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.asset_set_result: object expected");
                                    m.asset_set_result = $root.google.ads.googleads.v11.services.MutateAssetSetResult.fromObject(d.asset_set_result);
                                }
                                if (d.audience_result != null) {
                                    if (typeof d.audience_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.audience_result: object expected");
                                    m.audience_result = $root.google.ads.googleads.v11.services.MutateAudienceResult.fromObject(d.audience_result);
                                }
                                if (d.bidding_data_exclusion_result != null) {
                                    if (typeof d.bidding_data_exclusion_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.bidding_data_exclusion_result: object expected");
                                    m.bidding_data_exclusion_result = $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.fromObject(d.bidding_data_exclusion_result);
                                }
                                if (d.bidding_seasonality_adjustment_result != null) {
                                    if (typeof d.bidding_seasonality_adjustment_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.bidding_seasonality_adjustment_result: object expected");
                                    m.bidding_seasonality_adjustment_result = $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.fromObject(d.bidding_seasonality_adjustment_result);
                                }
                                if (d.bidding_strategy_result != null) {
                                    if (typeof d.bidding_strategy_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.bidding_strategy_result: object expected");
                                    m.bidding_strategy_result = $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.fromObject(d.bidding_strategy_result);
                                }
                                if (d.campaign_asset_result != null) {
                                    if (typeof d.campaign_asset_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_asset_result: object expected");
                                    m.campaign_asset_result = $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.fromObject(d.campaign_asset_result);
                                }
                                if (d.campaign_asset_set_result != null) {
                                    if (typeof d.campaign_asset_set_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_asset_set_result: object expected");
                                    m.campaign_asset_set_result = $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.fromObject(d.campaign_asset_set_result);
                                }
                                if (d.campaign_bid_modifier_result != null) {
                                    if (typeof d.campaign_bid_modifier_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_bid_modifier_result: object expected");
                                    m.campaign_bid_modifier_result = $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.fromObject(d.campaign_bid_modifier_result);
                                }
                                if (d.campaign_budget_result != null) {
                                    if (typeof d.campaign_budget_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_budget_result: object expected");
                                    m.campaign_budget_result = $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.fromObject(d.campaign_budget_result);
                                }
                                if (d.campaign_conversion_goal_result != null) {
                                    if (typeof d.campaign_conversion_goal_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_conversion_goal_result: object expected");
                                    m.campaign_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.fromObject(d.campaign_conversion_goal_result);
                                }
                                if (d.campaign_criterion_result != null) {
                                    if (typeof d.campaign_criterion_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_criterion_result: object expected");
                                    m.campaign_criterion_result = $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.fromObject(d.campaign_criterion_result);
                                }
                                if (d.campaign_customizer_result != null) {
                                    if (typeof d.campaign_customizer_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_customizer_result: object expected");
                                    m.campaign_customizer_result = $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.fromObject(d.campaign_customizer_result);
                                }
                                if (d.campaign_draft_result != null) {
                                    if (typeof d.campaign_draft_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_draft_result: object expected");
                                    m.campaign_draft_result = $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.fromObject(d.campaign_draft_result);
                                }
                                if (d.campaign_experiment_result != null) {
                                    if (typeof d.campaign_experiment_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_experiment_result: object expected");
                                    m.campaign_experiment_result = $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.fromObject(d.campaign_experiment_result);
                                }
                                if (d.campaign_extension_setting_result != null) {
                                    if (typeof d.campaign_extension_setting_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_extension_setting_result: object expected");
                                    m.campaign_extension_setting_result = $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.fromObject(d.campaign_extension_setting_result);
                                }
                                if (d.campaign_feed_result != null) {
                                    if (typeof d.campaign_feed_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_feed_result: object expected");
                                    m.campaign_feed_result = $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.fromObject(d.campaign_feed_result);
                                }
                                if (d.campaign_group_result != null) {
                                    if (typeof d.campaign_group_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_group_result: object expected");
                                    m.campaign_group_result = $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.fromObject(d.campaign_group_result);
                                }
                                if (d.campaign_label_result != null) {
                                    if (typeof d.campaign_label_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_label_result: object expected");
                                    m.campaign_label_result = $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.fromObject(d.campaign_label_result);
                                }
                                if (d.campaign_result != null) {
                                    if (typeof d.campaign_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_result: object expected");
                                    m.campaign_result = $root.google.ads.googleads.v11.services.MutateCampaignResult.fromObject(d.campaign_result);
                                }
                                if (d.campaign_shared_set_result != null) {
                                    if (typeof d.campaign_shared_set_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.campaign_shared_set_result: object expected");
                                    m.campaign_shared_set_result = $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.fromObject(d.campaign_shared_set_result);
                                }
                                if (d.conversion_action_result != null) {
                                    if (typeof d.conversion_action_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.conversion_action_result: object expected");
                                    m.conversion_action_result = $root.google.ads.googleads.v11.services.MutateConversionActionResult.fromObject(d.conversion_action_result);
                                }
                                if (d.conversion_custom_variable_result != null) {
                                    if (typeof d.conversion_custom_variable_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.conversion_custom_variable_result: object expected");
                                    m.conversion_custom_variable_result = $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.fromObject(d.conversion_custom_variable_result);
                                }
                                if (d.conversion_goal_campaign_config_result != null) {
                                    if (typeof d.conversion_goal_campaign_config_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.conversion_goal_campaign_config_result: object expected");
                                    m.conversion_goal_campaign_config_result = $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.fromObject(d.conversion_goal_campaign_config_result);
                                }
                                if (d.conversion_value_rule_result != null) {
                                    if (typeof d.conversion_value_rule_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.conversion_value_rule_result: object expected");
                                    m.conversion_value_rule_result = $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.fromObject(d.conversion_value_rule_result);
                                }
                                if (d.conversion_value_rule_set_result != null) {
                                    if (typeof d.conversion_value_rule_set_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.conversion_value_rule_set_result: object expected");
                                    m.conversion_value_rule_set_result = $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.fromObject(d.conversion_value_rule_set_result);
                                }
                                if (d.custom_conversion_goal_result != null) {
                                    if (typeof d.custom_conversion_goal_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.custom_conversion_goal_result: object expected");
                                    m.custom_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.fromObject(d.custom_conversion_goal_result);
                                }
                                if (d.customer_asset_result != null) {
                                    if (typeof d.customer_asset_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_asset_result: object expected");
                                    m.customer_asset_result = $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.fromObject(d.customer_asset_result);
                                }
                                if (d.customer_conversion_goal_result != null) {
                                    if (typeof d.customer_conversion_goal_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_conversion_goal_result: object expected");
                                    m.customer_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.fromObject(d.customer_conversion_goal_result);
                                }
                                if (d.customer_customizer_result != null) {
                                    if (typeof d.customer_customizer_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_customizer_result: object expected");
                                    m.customer_customizer_result = $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.fromObject(d.customer_customizer_result);
                                }
                                if (d.customer_extension_setting_result != null) {
                                    if (typeof d.customer_extension_setting_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_extension_setting_result: object expected");
                                    m.customer_extension_setting_result = $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.fromObject(d.customer_extension_setting_result);
                                }
                                if (d.customer_feed_result != null) {
                                    if (typeof d.customer_feed_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_feed_result: object expected");
                                    m.customer_feed_result = $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.fromObject(d.customer_feed_result);
                                }
                                if (d.customer_label_result != null) {
                                    if (typeof d.customer_label_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_label_result: object expected");
                                    m.customer_label_result = $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.fromObject(d.customer_label_result);
                                }
                                if (d.customer_negative_criterion_result != null) {
                                    if (typeof d.customer_negative_criterion_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_negative_criterion_result: object expected");
                                    m.customer_negative_criterion_result = $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.fromObject(d.customer_negative_criterion_result);
                                }
                                if (d.customer_result != null) {
                                    if (typeof d.customer_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customer_result: object expected");
                                    m.customer_result = $root.google.ads.googleads.v11.services.MutateCustomerResult.fromObject(d.customer_result);
                                }
                                if (d.customizer_attribute_result != null) {
                                    if (typeof d.customizer_attribute_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.customizer_attribute_result: object expected");
                                    m.customizer_attribute_result = $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.fromObject(d.customizer_attribute_result);
                                }
                                if (d.extension_feed_item_result != null) {
                                    if (typeof d.extension_feed_item_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.extension_feed_item_result: object expected");
                                    m.extension_feed_item_result = $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.fromObject(d.extension_feed_item_result);
                                }
                                if (d.feed_item_result != null) {
                                    if (typeof d.feed_item_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.feed_item_result: object expected");
                                    m.feed_item_result = $root.google.ads.googleads.v11.services.MutateFeedItemResult.fromObject(d.feed_item_result);
                                }
                                if (d.feed_item_set_result != null) {
                                    if (typeof d.feed_item_set_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.feed_item_set_result: object expected");
                                    m.feed_item_set_result = $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.fromObject(d.feed_item_set_result);
                                }
                                if (d.feed_item_set_link_result != null) {
                                    if (typeof d.feed_item_set_link_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.feed_item_set_link_result: object expected");
                                    m.feed_item_set_link_result = $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.fromObject(d.feed_item_set_link_result);
                                }
                                if (d.feed_item_target_result != null) {
                                    if (typeof d.feed_item_target_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.feed_item_target_result: object expected");
                                    m.feed_item_target_result = $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.fromObject(d.feed_item_target_result);
                                }
                                if (d.feed_mapping_result != null) {
                                    if (typeof d.feed_mapping_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.feed_mapping_result: object expected");
                                    m.feed_mapping_result = $root.google.ads.googleads.v11.services.MutateFeedMappingResult.fromObject(d.feed_mapping_result);
                                }
                                if (d.feed_result != null) {
                                    if (typeof d.feed_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.feed_result: object expected");
                                    m.feed_result = $root.google.ads.googleads.v11.services.MutateFeedResult.fromObject(d.feed_result);
                                }
                                if (d.keyword_plan_ad_group_result != null) {
                                    if (typeof d.keyword_plan_ad_group_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.keyword_plan_ad_group_result: object expected");
                                    m.keyword_plan_ad_group_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.fromObject(d.keyword_plan_ad_group_result);
                                }
                                if (d.keyword_plan_campaign_result != null) {
                                    if (typeof d.keyword_plan_campaign_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.keyword_plan_campaign_result: object expected");
                                    m.keyword_plan_campaign_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.fromObject(d.keyword_plan_campaign_result);
                                }
                                if (d.keyword_plan_ad_group_keyword_result != null) {
                                    if (typeof d.keyword_plan_ad_group_keyword_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.keyword_plan_ad_group_keyword_result: object expected");
                                    m.keyword_plan_ad_group_keyword_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.fromObject(d.keyword_plan_ad_group_keyword_result);
                                }
                                if (d.keyword_plan_campaign_keyword_result != null) {
                                    if (typeof d.keyword_plan_campaign_keyword_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.keyword_plan_campaign_keyword_result: object expected");
                                    m.keyword_plan_campaign_keyword_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.fromObject(d.keyword_plan_campaign_keyword_result);
                                }
                                if (d.keyword_plan_result != null) {
                                    if (typeof d.keyword_plan_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.keyword_plan_result: object expected");
                                    m.keyword_plan_result = $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.fromObject(d.keyword_plan_result);
                                }
                                if (d.label_result != null) {
                                    if (typeof d.label_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.label_result: object expected");
                                    m.label_result = $root.google.ads.googleads.v11.services.MutateLabelResult.fromObject(d.label_result);
                                }
                                if (d.media_file_result != null) {
                                    if (typeof d.media_file_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.media_file_result: object expected");
                                    m.media_file_result = $root.google.ads.googleads.v11.services.MutateMediaFileResult.fromObject(d.media_file_result);
                                }
                                if (d.remarketing_action_result != null) {
                                    if (typeof d.remarketing_action_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.remarketing_action_result: object expected");
                                    m.remarketing_action_result = $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.fromObject(d.remarketing_action_result);
                                }
                                if (d.shared_criterion_result != null) {
                                    if (typeof d.shared_criterion_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.shared_criterion_result: object expected");
                                    m.shared_criterion_result = $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.fromObject(d.shared_criterion_result);
                                }
                                if (d.shared_set_result != null) {
                                    if (typeof d.shared_set_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.shared_set_result: object expected");
                                    m.shared_set_result = $root.google.ads.googleads.v11.services.MutateSharedSetResult.fromObject(d.shared_set_result);
                                }
                                if (d.smart_campaign_setting_result != null) {
                                    if (typeof d.smart_campaign_setting_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.smart_campaign_setting_result: object expected");
                                    m.smart_campaign_setting_result = $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.fromObject(d.smart_campaign_setting_result);
                                }
                                if (d.user_list_result != null) {
                                    if (typeof d.user_list_result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateOperationResponse.user_list_result: object expected");
                                    m.user_list_result = $root.google.ads.googleads.v11.services.MutateUserListResult.fromObject(d.user_list_result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateOperationResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateOperationResponse} m MutateOperationResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateOperationResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.ad_group_ad_result != null && m.hasOwnProperty("ad_group_ad_result")) {
                                    d.ad_group_ad_result = $root.google.ads.googleads.v11.services.MutateAdGroupAdResult.toObject(m.ad_group_ad_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_ad_result";
                                }
                                if (m.ad_group_bid_modifier_result != null && m.hasOwnProperty("ad_group_bid_modifier_result")) {
                                    d.ad_group_bid_modifier_result = $root.google.ads.googleads.v11.services.MutateAdGroupBidModifierResult.toObject(m.ad_group_bid_modifier_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_bid_modifier_result";
                                }
                                if (m.ad_group_criterion_result != null && m.hasOwnProperty("ad_group_criterion_result")) {
                                    d.ad_group_criterion_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionResult.toObject(m.ad_group_criterion_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_criterion_result";
                                }
                                if (m.ad_group_result != null && m.hasOwnProperty("ad_group_result")) {
                                    d.ad_group_result = $root.google.ads.googleads.v11.services.MutateAdGroupResult.toObject(m.ad_group_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_result";
                                }
                                if (m.bidding_strategy_result != null && m.hasOwnProperty("bidding_strategy_result")) {
                                    d.bidding_strategy_result = $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.toObject(m.bidding_strategy_result, o);
                                    if (o.oneofs)
                                        d.response = "bidding_strategy_result";
                                }
                                if (m.campaign_bid_modifier_result != null && m.hasOwnProperty("campaign_bid_modifier_result")) {
                                    d.campaign_bid_modifier_result = $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.toObject(m.campaign_bid_modifier_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_bid_modifier_result";
                                }
                                if (m.campaign_budget_result != null && m.hasOwnProperty("campaign_budget_result")) {
                                    d.campaign_budget_result = $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.toObject(m.campaign_budget_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_budget_result";
                                }
                                if (m.campaign_group_result != null && m.hasOwnProperty("campaign_group_result")) {
                                    d.campaign_group_result = $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.toObject(m.campaign_group_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_group_result";
                                }
                                if (m.campaign_result != null && m.hasOwnProperty("campaign_result")) {
                                    d.campaign_result = $root.google.ads.googleads.v11.services.MutateCampaignResult.toObject(m.campaign_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_result";
                                }
                                if (m.campaign_shared_set_result != null && m.hasOwnProperty("campaign_shared_set_result")) {
                                    d.campaign_shared_set_result = $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.toObject(m.campaign_shared_set_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_shared_set_result";
                                }
                                if (m.conversion_action_result != null && m.hasOwnProperty("conversion_action_result")) {
                                    d.conversion_action_result = $root.google.ads.googleads.v11.services.MutateConversionActionResult.toObject(m.conversion_action_result, o);
                                    if (o.oneofs)
                                        d.response = "conversion_action_result";
                                }
                                if (m.campaign_criterion_result != null && m.hasOwnProperty("campaign_criterion_result")) {
                                    d.campaign_criterion_result = $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.toObject(m.campaign_criterion_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_criterion_result";
                                }
                                if (m.shared_criterion_result != null && m.hasOwnProperty("shared_criterion_result")) {
                                    d.shared_criterion_result = $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.toObject(m.shared_criterion_result, o);
                                    if (o.oneofs)
                                        d.response = "shared_criterion_result";
                                }
                                if (m.shared_set_result != null && m.hasOwnProperty("shared_set_result")) {
                                    d.shared_set_result = $root.google.ads.googleads.v11.services.MutateSharedSetResult.toObject(m.shared_set_result, o);
                                    if (o.oneofs)
                                        d.response = "shared_set_result";
                                }
                                if (m.user_list_result != null && m.hasOwnProperty("user_list_result")) {
                                    d.user_list_result = $root.google.ads.googleads.v11.services.MutateUserListResult.toObject(m.user_list_result, o);
                                    if (o.oneofs)
                                        d.response = "user_list_result";
                                }
                                if (m.ad_group_ad_label_result != null && m.hasOwnProperty("ad_group_ad_label_result")) {
                                    d.ad_group_ad_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupAdLabelResult.toObject(m.ad_group_ad_label_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_ad_label_result";
                                }
                                if (m.ad_group_criterion_label_result != null && m.hasOwnProperty("ad_group_criterion_label_result")) {
                                    d.ad_group_criterion_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionLabelResult.toObject(m.ad_group_criterion_label_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_criterion_label_result";
                                }
                                if (m.ad_group_extension_setting_result != null && m.hasOwnProperty("ad_group_extension_setting_result")) {
                                    d.ad_group_extension_setting_result = $root.google.ads.googleads.v11.services.MutateAdGroupExtensionSettingResult.toObject(m.ad_group_extension_setting_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_extension_setting_result";
                                }
                                if (m.ad_group_feed_result != null && m.hasOwnProperty("ad_group_feed_result")) {
                                    d.ad_group_feed_result = $root.google.ads.googleads.v11.services.MutateAdGroupFeedResult.toObject(m.ad_group_feed_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_feed_result";
                                }
                                if (m.ad_group_label_result != null && m.hasOwnProperty("ad_group_label_result")) {
                                    d.ad_group_label_result = $root.google.ads.googleads.v11.services.MutateAdGroupLabelResult.toObject(m.ad_group_label_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_label_result";
                                }
                                if (m.ad_parameter_result != null && m.hasOwnProperty("ad_parameter_result")) {
                                    d.ad_parameter_result = $root.google.ads.googleads.v11.services.MutateAdParameterResult.toObject(m.ad_parameter_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_parameter_result";
                                }
                                if (m.asset_result != null && m.hasOwnProperty("asset_result")) {
                                    d.asset_result = $root.google.ads.googleads.v11.services.MutateAssetResult.toObject(m.asset_result, o);
                                    if (o.oneofs)
                                        d.response = "asset_result";
                                }
                                if (m.campaign_draft_result != null && m.hasOwnProperty("campaign_draft_result")) {
                                    d.campaign_draft_result = $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.toObject(m.campaign_draft_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_draft_result";
                                }
                                if (m.campaign_experiment_result != null && m.hasOwnProperty("campaign_experiment_result")) {
                                    d.campaign_experiment_result = $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.toObject(m.campaign_experiment_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_experiment_result";
                                }
                                if (m.campaign_extension_setting_result != null && m.hasOwnProperty("campaign_extension_setting_result")) {
                                    d.campaign_extension_setting_result = $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.toObject(m.campaign_extension_setting_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_extension_setting_result";
                                }
                                if (m.campaign_feed_result != null && m.hasOwnProperty("campaign_feed_result")) {
                                    d.campaign_feed_result = $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.toObject(m.campaign_feed_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_feed_result";
                                }
                                if (m.campaign_label_result != null && m.hasOwnProperty("campaign_label_result")) {
                                    d.campaign_label_result = $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.toObject(m.campaign_label_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_label_result";
                                }
                                if (m.customer_extension_setting_result != null && m.hasOwnProperty("customer_extension_setting_result")) {
                                    d.customer_extension_setting_result = $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.toObject(m.customer_extension_setting_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_extension_setting_result";
                                }
                                if (m.customer_feed_result != null && m.hasOwnProperty("customer_feed_result")) {
                                    d.customer_feed_result = $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.toObject(m.customer_feed_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_feed_result";
                                }
                                if (m.customer_label_result != null && m.hasOwnProperty("customer_label_result")) {
                                    d.customer_label_result = $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.toObject(m.customer_label_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_label_result";
                                }
                                if (m.customer_negative_criterion_result != null && m.hasOwnProperty("customer_negative_criterion_result")) {
                                    d.customer_negative_criterion_result = $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.toObject(m.customer_negative_criterion_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_negative_criterion_result";
                                }
                                if (m.customer_result != null && m.hasOwnProperty("customer_result")) {
                                    d.customer_result = $root.google.ads.googleads.v11.services.MutateCustomerResult.toObject(m.customer_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_result";
                                }
                                if (m.extension_feed_item_result != null && m.hasOwnProperty("extension_feed_item_result")) {
                                    d.extension_feed_item_result = $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.toObject(m.extension_feed_item_result, o);
                                    if (o.oneofs)
                                        d.response = "extension_feed_item_result";
                                }
                                if (m.feed_item_result != null && m.hasOwnProperty("feed_item_result")) {
                                    d.feed_item_result = $root.google.ads.googleads.v11.services.MutateFeedItemResult.toObject(m.feed_item_result, o);
                                    if (o.oneofs)
                                        d.response = "feed_item_result";
                                }
                                if (m.feed_item_target_result != null && m.hasOwnProperty("feed_item_target_result")) {
                                    d.feed_item_target_result = $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.toObject(m.feed_item_target_result, o);
                                    if (o.oneofs)
                                        d.response = "feed_item_target_result";
                                }
                                if (m.feed_mapping_result != null && m.hasOwnProperty("feed_mapping_result")) {
                                    d.feed_mapping_result = $root.google.ads.googleads.v11.services.MutateFeedMappingResult.toObject(m.feed_mapping_result, o);
                                    if (o.oneofs)
                                        d.response = "feed_mapping_result";
                                }
                                if (m.feed_result != null && m.hasOwnProperty("feed_result")) {
                                    d.feed_result = $root.google.ads.googleads.v11.services.MutateFeedResult.toObject(m.feed_result, o);
                                    if (o.oneofs)
                                        d.response = "feed_result";
                                }
                                if (m.label_result != null && m.hasOwnProperty("label_result")) {
                                    d.label_result = $root.google.ads.googleads.v11.services.MutateLabelResult.toObject(m.label_result, o);
                                    if (o.oneofs)
                                        d.response = "label_result";
                                }
                                if (m.media_file_result != null && m.hasOwnProperty("media_file_result")) {
                                    d.media_file_result = $root.google.ads.googleads.v11.services.MutateMediaFileResult.toObject(m.media_file_result, o);
                                    if (o.oneofs)
                                        d.response = "media_file_result";
                                }
                                if (m.remarketing_action_result != null && m.hasOwnProperty("remarketing_action_result")) {
                                    d.remarketing_action_result = $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.toObject(m.remarketing_action_result, o);
                                    if (o.oneofs)
                                        d.response = "remarketing_action_result";
                                }
                                if (m.keyword_plan_ad_group_result != null && m.hasOwnProperty("keyword_plan_ad_group_result")) {
                                    d.keyword_plan_ad_group_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.toObject(m.keyword_plan_ad_group_result, o);
                                    if (o.oneofs)
                                        d.response = "keyword_plan_ad_group_result";
                                }
                                if (m.keyword_plan_campaign_result != null && m.hasOwnProperty("keyword_plan_campaign_result")) {
                                    d.keyword_plan_campaign_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.toObject(m.keyword_plan_campaign_result, o);
                                    if (o.oneofs)
                                        d.response = "keyword_plan_campaign_result";
                                }
                                if (m.keyword_plan_result != null && m.hasOwnProperty("keyword_plan_result")) {
                                    d.keyword_plan_result = $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.toObject(m.keyword_plan_result, o);
                                    if (o.oneofs)
                                        d.response = "keyword_plan_result";
                                }
                                if (m.ad_result != null && m.hasOwnProperty("ad_result")) {
                                    d.ad_result = $root.google.ads.googleads.v11.services.MutateAdResult.toObject(m.ad_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_result";
                                }
                                if (m.keyword_plan_ad_group_keyword_result != null && m.hasOwnProperty("keyword_plan_ad_group_keyword_result")) {
                                    d.keyword_plan_ad_group_keyword_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.toObject(m.keyword_plan_ad_group_keyword_result, o);
                                    if (o.oneofs)
                                        d.response = "keyword_plan_ad_group_keyword_result";
                                }
                                if (m.keyword_plan_campaign_keyword_result != null && m.hasOwnProperty("keyword_plan_campaign_keyword_result")) {
                                    d.keyword_plan_campaign_keyword_result = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.toObject(m.keyword_plan_campaign_keyword_result, o);
                                    if (o.oneofs)
                                        d.response = "keyword_plan_campaign_keyword_result";
                                }
                                if (m.campaign_asset_result != null && m.hasOwnProperty("campaign_asset_result")) {
                                    d.campaign_asset_result = $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.toObject(m.campaign_asset_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_asset_result";
                                }
                                if (m.feed_item_set_result != null && m.hasOwnProperty("feed_item_set_result")) {
                                    d.feed_item_set_result = $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.toObject(m.feed_item_set_result, o);
                                    if (o.oneofs)
                                        d.response = "feed_item_set_result";
                                }
                                if (m.feed_item_set_link_result != null && m.hasOwnProperty("feed_item_set_link_result")) {
                                    d.feed_item_set_link_result = $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.toObject(m.feed_item_set_link_result, o);
                                    if (o.oneofs)
                                        d.response = "feed_item_set_link_result";
                                }
                                if (m.conversion_custom_variable_result != null && m.hasOwnProperty("conversion_custom_variable_result")) {
                                    d.conversion_custom_variable_result = $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.toObject(m.conversion_custom_variable_result, o);
                                    if (o.oneofs)
                                        d.response = "conversion_custom_variable_result";
                                }
                                if (m.ad_group_asset_result != null && m.hasOwnProperty("ad_group_asset_result")) {
                                    d.ad_group_asset_result = $root.google.ads.googleads.v11.services.MutateAdGroupAssetResult.toObject(m.ad_group_asset_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_asset_result";
                                }
                                if (m.customer_asset_result != null && m.hasOwnProperty("customer_asset_result")) {
                                    d.customer_asset_result = $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.toObject(m.customer_asset_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_asset_result";
                                }
                                if (m.bidding_data_exclusion_result != null && m.hasOwnProperty("bidding_data_exclusion_result")) {
                                    d.bidding_data_exclusion_result = $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.toObject(m.bidding_data_exclusion_result, o);
                                    if (o.oneofs)
                                        d.response = "bidding_data_exclusion_result";
                                }
                                if (m.bidding_seasonality_adjustment_result != null && m.hasOwnProperty("bidding_seasonality_adjustment_result")) {
                                    d.bidding_seasonality_adjustment_result = $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.toObject(m.bidding_seasonality_adjustment_result, o);
                                    if (o.oneofs)
                                        d.response = "bidding_seasonality_adjustment_result";
                                }
                                if (m.smart_campaign_setting_result != null && m.hasOwnProperty("smart_campaign_setting_result")) {
                                    d.smart_campaign_setting_result = $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.toObject(m.smart_campaign_setting_result, o);
                                    if (o.oneofs)
                                        d.response = "smart_campaign_setting_result";
                                }
                                if (m.asset_group_result != null && m.hasOwnProperty("asset_group_result")) {
                                    d.asset_group_result = $root.google.ads.googleads.v11.services.MutateAssetGroupResult.toObject(m.asset_group_result, o);
                                    if (o.oneofs)
                                        d.response = "asset_group_result";
                                }
                                if (m.conversion_value_rule_result != null && m.hasOwnProperty("conversion_value_rule_result")) {
                                    d.conversion_value_rule_result = $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.toObject(m.conversion_value_rule_result, o);
                                    if (o.oneofs)
                                        d.response = "conversion_value_rule_result";
                                }
                                if (m.conversion_value_rule_set_result != null && m.hasOwnProperty("conversion_value_rule_set_result")) {
                                    d.conversion_value_rule_set_result = $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.toObject(m.conversion_value_rule_set_result, o);
                                    if (o.oneofs)
                                        d.response = "conversion_value_rule_set_result";
                                }
                                if (m.asset_group_asset_result != null && m.hasOwnProperty("asset_group_asset_result")) {
                                    d.asset_group_asset_result = $root.google.ads.googleads.v11.services.MutateAssetGroupAssetResult.toObject(m.asset_group_asset_result, o);
                                    if (o.oneofs)
                                        d.response = "asset_group_asset_result";
                                }
                                if (m.customer_conversion_goal_result != null && m.hasOwnProperty("customer_conversion_goal_result")) {
                                    d.customer_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.toObject(m.customer_conversion_goal_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_conversion_goal_result";
                                }
                                if (m.campaign_conversion_goal_result != null && m.hasOwnProperty("campaign_conversion_goal_result")) {
                                    d.campaign_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.toObject(m.campaign_conversion_goal_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_conversion_goal_result";
                                }
                                if (m.custom_conversion_goal_result != null && m.hasOwnProperty("custom_conversion_goal_result")) {
                                    d.custom_conversion_goal_result = $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.toObject(m.custom_conversion_goal_result, o);
                                    if (o.oneofs)
                                        d.response = "custom_conversion_goal_result";
                                }
                                if (m.conversion_goal_campaign_config_result != null && m.hasOwnProperty("conversion_goal_campaign_config_result")) {
                                    d.conversion_goal_campaign_config_result = $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.toObject(m.conversion_goal_campaign_config_result, o);
                                    if (o.oneofs)
                                        d.response = "conversion_goal_campaign_config_result";
                                }
                                if (m.customizer_attribute_result != null && m.hasOwnProperty("customizer_attribute_result")) {
                                    d.customizer_attribute_result = $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.toObject(m.customizer_attribute_result, o);
                                    if (o.oneofs)
                                        d.response = "customizer_attribute_result";
                                }
                                if (m.asset_set_asset_result != null && m.hasOwnProperty("asset_set_asset_result")) {
                                    d.asset_set_asset_result = $root.google.ads.googleads.v11.services.MutateAssetSetAssetResult.toObject(m.asset_set_asset_result, o);
                                    if (o.oneofs)
                                        d.response = "asset_set_asset_result";
                                }
                                if (m.asset_set_result != null && m.hasOwnProperty("asset_set_result")) {
                                    d.asset_set_result = $root.google.ads.googleads.v11.services.MutateAssetSetResult.toObject(m.asset_set_result, o);
                                    if (o.oneofs)
                                        d.response = "asset_set_result";
                                }
                                if (m.campaign_asset_set_result != null && m.hasOwnProperty("campaign_asset_set_result")) {
                                    d.campaign_asset_set_result = $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.toObject(m.campaign_asset_set_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_asset_set_result";
                                }
                                if (m.customer_customizer_result != null && m.hasOwnProperty("customer_customizer_result")) {
                                    d.customer_customizer_result = $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.toObject(m.customer_customizer_result, o);
                                    if (o.oneofs)
                                        d.response = "customer_customizer_result";
                                }
                                if (m.ad_group_customizer_result != null && m.hasOwnProperty("ad_group_customizer_result")) {
                                    d.ad_group_customizer_result = $root.google.ads.googleads.v11.services.MutateAdGroupCustomizerResult.toObject(m.ad_group_customizer_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_customizer_result";
                                }
                                if (m.campaign_customizer_result != null && m.hasOwnProperty("campaign_customizer_result")) {
                                    d.campaign_customizer_result = $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.toObject(m.campaign_customizer_result, o);
                                    if (o.oneofs)
                                        d.response = "campaign_customizer_result";
                                }
                                if (m.ad_group_criterion_customizer_result != null && m.hasOwnProperty("ad_group_criterion_customizer_result")) {
                                    d.ad_group_criterion_customizer_result = $root.google.ads.googleads.v11.services.MutateAdGroupCriterionCustomizerResult.toObject(m.ad_group_criterion_customizer_result, o);
                                    if (o.oneofs)
                                        d.response = "ad_group_criterion_customizer_result";
                                }
                                if (m.asset_group_listing_group_filter_result != null && m.hasOwnProperty("asset_group_listing_group_filter_result")) {
                                    d.asset_group_listing_group_filter_result = $root.google.ads.googleads.v11.services.MutateAssetGroupListingGroupFilterResult.toObject(m.asset_group_listing_group_filter_result, o);
                                    if (o.oneofs)
                                        d.response = "asset_group_listing_group_filter_result";
                                }
                                if (m.asset_group_signal_result != null && m.hasOwnProperty("asset_group_signal_result")) {
                                    d.asset_group_signal_result = $root.google.ads.googleads.v11.services.MutateAssetGroupSignalResult.toObject(m.asset_group_signal_result, o);
                                    if (o.oneofs)
                                        d.response = "asset_group_signal_result";
                                }
                                if (m.audience_result != null && m.hasOwnProperty("audience_result")) {
                                    d.audience_result = $root.google.ads.googleads.v11.services.MutateAudienceResult.toObject(m.audience_result, o);
                                    if (o.oneofs)
                                        d.response = "audience_result";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateOperationResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateOperationResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateOperationResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateOperationResponse;
                        })();
    
                        services.BiddingDataExclusionService = (function() {
    
                            /**
                             * Constructs a new BiddingDataExclusionService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BiddingDataExclusionService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function BiddingDataExclusionService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (BiddingDataExclusionService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = BiddingDataExclusionService;
    
                            /**
                             * Creates new BiddingDataExclusionService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {BiddingDataExclusionService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            BiddingDataExclusionService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BiddingDataExclusionService|mutateBiddingDataExclusions}.
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionService
                             * @typedef MutateBiddingDataExclusionsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse} [response] MutateBiddingDataExclusionsResponse
                             */
    
                            /**
                             * Calls MutateBiddingDataExclusions.
                             * @function mutateBiddingDataExclusions
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsRequest} request MutateBiddingDataExclusionsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BiddingDataExclusionService.MutateBiddingDataExclusionsCallback} callback Node-style callback called with the error, if any, and MutateBiddingDataExclusionsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BiddingDataExclusionService.prototype.mutateBiddingDataExclusions = function mutateBiddingDataExclusions(request, callback) {
                                return this.rpcCall(mutateBiddingDataExclusions, $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest, $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse, request, callback);
                            }, "name", { value: "MutateBiddingDataExclusions" });
    
                            /**
                             * Calls MutateBiddingDataExclusions.
                             * @function mutateBiddingDataExclusions
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsRequest} request MutateBiddingDataExclusionsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse>} Promise
                             * @variation 2
                             */
    
                            return BiddingDataExclusionService;
                        })();
    
                        services.MutateBiddingDataExclusionsRequest = (function() {
    
                            /**
                             * Properties of a MutateBiddingDataExclusionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingDataExclusionsRequest
                             * @property {string|null} [customer_id] MutateBiddingDataExclusionsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IBiddingDataExclusionOperation>|null} [operations] MutateBiddingDataExclusionsRequest operations
                             * @property {boolean|null} [partial_failure] MutateBiddingDataExclusionsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateBiddingDataExclusionsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateBiddingDataExclusionsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateBiddingDataExclusionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingDataExclusionsRequest.
                             * @implements IMutateBiddingDataExclusionsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsRequest=} [p] Properties to set
                             */
                            function MutateBiddingDataExclusionsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingDataExclusionsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @instance
                             */
                            MutateBiddingDataExclusionsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateBiddingDataExclusionsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IBiddingDataExclusionOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @instance
                             */
                            MutateBiddingDataExclusionsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateBiddingDataExclusionsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @instance
                             */
                            MutateBiddingDataExclusionsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateBiddingDataExclusionsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @instance
                             */
                            MutateBiddingDataExclusionsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateBiddingDataExclusionsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @instance
                             */
                            MutateBiddingDataExclusionsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateBiddingDataExclusionsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest} MutateBiddingDataExclusionsRequest instance
                             */
                            MutateBiddingDataExclusionsRequest.create = function create(properties) {
                                return new MutateBiddingDataExclusionsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingDataExclusionsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsRequest} m MutateBiddingDataExclusionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingDataExclusionsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingDataExclusionsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsRequest} message MutateBiddingDataExclusionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingDataExclusionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingDataExclusionsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest} MutateBiddingDataExclusionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingDataExclusionsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingDataExclusionsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest} MutateBiddingDataExclusionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingDataExclusionsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingDataExclusionsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingDataExclusionsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingDataExclusionsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest} MutateBiddingDataExclusionsRequest
                             */
                            MutateBiddingDataExclusionsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingDataExclusionsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest} m MutateBiddingDataExclusionsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingDataExclusionsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingDataExclusionsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingDataExclusionsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingDataExclusionsRequest;
                        })();
    
                        services.BiddingDataExclusionOperation = (function() {
    
                            /**
                             * Properties of a BiddingDataExclusionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IBiddingDataExclusionOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] BiddingDataExclusionOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IBiddingDataExclusion|null} [create] BiddingDataExclusionOperation create
                             * @property {google.ads.googleads.v11.resources.IBiddingDataExclusion|null} [update] BiddingDataExclusionOperation update
                             * @property {string|null} [remove] BiddingDataExclusionOperation remove
                             */
    
                            /**
                             * Constructs a new BiddingDataExclusionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BiddingDataExclusionOperation.
                             * @implements IBiddingDataExclusionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IBiddingDataExclusionOperation=} [p] Properties to set
                             */
                            function BiddingDataExclusionOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BiddingDataExclusionOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @instance
                             */
                            BiddingDataExclusionOperation.prototype.update_mask = null;
    
                            /**
                             * BiddingDataExclusionOperation create.
                             * @member {google.ads.googleads.v11.resources.IBiddingDataExclusion|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @instance
                             */
                            BiddingDataExclusionOperation.prototype.create = null;
    
                            /**
                             * BiddingDataExclusionOperation update.
                             * @member {google.ads.googleads.v11.resources.IBiddingDataExclusion|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @instance
                             */
                            BiddingDataExclusionOperation.prototype.update = null;
    
                            /**
                             * BiddingDataExclusionOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @instance
                             */
                            BiddingDataExclusionOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BiddingDataExclusionOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @instance
                             */
                            Object.defineProperty(BiddingDataExclusionOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BiddingDataExclusionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingDataExclusionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.BiddingDataExclusionOperation} BiddingDataExclusionOperation instance
                             */
                            BiddingDataExclusionOperation.create = function create(properties) {
                                return new BiddingDataExclusionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingDataExclusionOperation message. Does not implicitly {@link google.ads.googleads.v11.services.BiddingDataExclusionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingDataExclusionOperation} m BiddingDataExclusionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingDataExclusionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.BiddingDataExclusion.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.BiddingDataExclusion.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingDataExclusionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.BiddingDataExclusionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingDataExclusionOperation} message BiddingDataExclusionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingDataExclusionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingDataExclusionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.BiddingDataExclusionOperation} BiddingDataExclusionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingDataExclusionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingDataExclusionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.BiddingDataExclusionOperation} BiddingDataExclusionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingDataExclusionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingDataExclusionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingDataExclusionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BiddingDataExclusionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.BiddingDataExclusionOperation} BiddingDataExclusionOperation
                             */
                            BiddingDataExclusionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.BiddingDataExclusionOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingDataExclusionOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingDataExclusionOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingDataExclusionOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BiddingDataExclusionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.BiddingDataExclusionOperation} m BiddingDataExclusionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingDataExclusionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BiddingDataExclusionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.BiddingDataExclusionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingDataExclusionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BiddingDataExclusionOperation;
                        })();
    
                        services.MutateBiddingDataExclusionsResponse = (function() {
    
                            /**
                             * Properties of a MutateBiddingDataExclusionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingDataExclusionsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateBiddingDataExclusionsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult>|null} [results] MutateBiddingDataExclusionsResponse results
                             */
    
                            /**
                             * Constructs a new MutateBiddingDataExclusionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingDataExclusionsResponse.
                             * @implements IMutateBiddingDataExclusionsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResponse=} [p] Properties to set
                             */
                            function MutateBiddingDataExclusionsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingDataExclusionsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @instance
                             */
                            MutateBiddingDataExclusionsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateBiddingDataExclusionsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @instance
                             */
                            MutateBiddingDataExclusionsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateBiddingDataExclusionsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse} MutateBiddingDataExclusionsResponse instance
                             */
                            MutateBiddingDataExclusionsResponse.create = function create(properties) {
                                return new MutateBiddingDataExclusionsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingDataExclusionsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResponse} m MutateBiddingDataExclusionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingDataExclusionsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingDataExclusionsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResponse} message MutateBiddingDataExclusionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingDataExclusionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingDataExclusionsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse} MutateBiddingDataExclusionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingDataExclusionsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingDataExclusionsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse} MutateBiddingDataExclusionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingDataExclusionsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingDataExclusionsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingDataExclusionsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingDataExclusionsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse} MutateBiddingDataExclusionsResponse
                             */
                            MutateBiddingDataExclusionsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingDataExclusionsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse} m MutateBiddingDataExclusionsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingDataExclusionsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingDataExclusionsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingDataExclusionsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingDataExclusionsResponse;
                        })();
    
                        services.MutateBiddingDataExclusionsResult = (function() {
    
                            /**
                             * Properties of a MutateBiddingDataExclusionsResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingDataExclusionsResult
                             * @property {string|null} [resource_name] MutateBiddingDataExclusionsResult resource_name
                             * @property {google.ads.googleads.v11.resources.IBiddingDataExclusion|null} [bidding_data_exclusion] MutateBiddingDataExclusionsResult bidding_data_exclusion
                             */
    
                            /**
                             * Constructs a new MutateBiddingDataExclusionsResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingDataExclusionsResult.
                             * @implements IMutateBiddingDataExclusionsResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult=} [p] Properties to set
                             */
                            function MutateBiddingDataExclusionsResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingDataExclusionsResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @instance
                             */
                            MutateBiddingDataExclusionsResult.prototype.resource_name = "";
    
                            /**
                             * MutateBiddingDataExclusionsResult bidding_data_exclusion.
                             * @member {google.ads.googleads.v11.resources.IBiddingDataExclusion|null|undefined} bidding_data_exclusion
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @instance
                             */
                            MutateBiddingDataExclusionsResult.prototype.bidding_data_exclusion = null;
    
                            /**
                             * Creates a new MutateBiddingDataExclusionsResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult} MutateBiddingDataExclusionsResult instance
                             */
                            MutateBiddingDataExclusionsResult.create = function create(properties) {
                                return new MutateBiddingDataExclusionsResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingDataExclusionsResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult} m MutateBiddingDataExclusionsResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingDataExclusionsResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.bidding_data_exclusion != null && Object.hasOwnProperty.call(m, "bidding_data_exclusion"))
                                    $root.google.ads.googleads.v11.resources.BiddingDataExclusion.encode(m.bidding_data_exclusion, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingDataExclusionsResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingDataExclusionsResult} message MutateBiddingDataExclusionsResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingDataExclusionsResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingDataExclusionsResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult} MutateBiddingDataExclusionsResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingDataExclusionsResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.bidding_data_exclusion = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingDataExclusionsResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult} MutateBiddingDataExclusionsResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingDataExclusionsResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingDataExclusionsResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingDataExclusionsResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.bidding_data_exclusion != null && m.hasOwnProperty("bidding_data_exclusion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.verify(m.bidding_data_exclusion);
                                        if (e)
                                            return "bidding_data_exclusion." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingDataExclusionsResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult} MutateBiddingDataExclusionsResult
                             */
                            MutateBiddingDataExclusionsResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.bidding_data_exclusion != null) {
                                    if (typeof d.bidding_data_exclusion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult.bidding_data_exclusion: object expected");
                                    m.bidding_data_exclusion = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.fromObject(d.bidding_data_exclusion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingDataExclusionsResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult} m MutateBiddingDataExclusionsResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingDataExclusionsResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.bidding_data_exclusion = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.bidding_data_exclusion != null && m.hasOwnProperty("bidding_data_exclusion")) {
                                    d.bidding_data_exclusion = $root.google.ads.googleads.v11.resources.BiddingDataExclusion.toObject(m.bidding_data_exclusion, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingDataExclusionsResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingDataExclusionsResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingDataExclusionsResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingDataExclusionsResult;
                        })();
    
                        services.BiddingSeasonalityAdjustmentService = (function() {
    
                            /**
                             * Constructs a new BiddingSeasonalityAdjustmentService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BiddingSeasonalityAdjustmentService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function BiddingSeasonalityAdjustmentService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (BiddingSeasonalityAdjustmentService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = BiddingSeasonalityAdjustmentService;
    
                            /**
                             * Creates new BiddingSeasonalityAdjustmentService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {BiddingSeasonalityAdjustmentService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            BiddingSeasonalityAdjustmentService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentService|mutateBiddingSeasonalityAdjustments}.
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentService
                             * @typedef MutateBiddingSeasonalityAdjustmentsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse} [response] MutateBiddingSeasonalityAdjustmentsResponse
                             */
    
                            /**
                             * Calls MutateBiddingSeasonalityAdjustments.
                             * @function mutateBiddingSeasonalityAdjustments
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsRequest} request MutateBiddingSeasonalityAdjustmentsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentService.MutateBiddingSeasonalityAdjustmentsCallback} callback Node-style callback called with the error, if any, and MutateBiddingSeasonalityAdjustmentsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BiddingSeasonalityAdjustmentService.prototype.mutateBiddingSeasonalityAdjustments = function mutateBiddingSeasonalityAdjustments(request, callback) {
                                return this.rpcCall(mutateBiddingSeasonalityAdjustments, $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest, $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse, request, callback);
                            }, "name", { value: "MutateBiddingSeasonalityAdjustments" });
    
                            /**
                             * Calls MutateBiddingSeasonalityAdjustments.
                             * @function mutateBiddingSeasonalityAdjustments
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsRequest} request MutateBiddingSeasonalityAdjustmentsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse>} Promise
                             * @variation 2
                             */
    
                            return BiddingSeasonalityAdjustmentService;
                        })();
    
                        services.MutateBiddingSeasonalityAdjustmentsRequest = (function() {
    
                            /**
                             * Properties of a MutateBiddingSeasonalityAdjustmentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingSeasonalityAdjustmentsRequest
                             * @property {string|null} [customer_id] MutateBiddingSeasonalityAdjustmentsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation>|null} [operations] MutateBiddingSeasonalityAdjustmentsRequest operations
                             * @property {boolean|null} [partial_failure] MutateBiddingSeasonalityAdjustmentsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateBiddingSeasonalityAdjustmentsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateBiddingSeasonalityAdjustmentsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateBiddingSeasonalityAdjustmentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingSeasonalityAdjustmentsRequest.
                             * @implements IMutateBiddingSeasonalityAdjustmentsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsRequest=} [p] Properties to set
                             */
                            function MutateBiddingSeasonalityAdjustmentsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateBiddingSeasonalityAdjustmentsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest} MutateBiddingSeasonalityAdjustmentsRequest instance
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.create = function create(properties) {
                                return new MutateBiddingSeasonalityAdjustmentsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingSeasonalityAdjustmentsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsRequest} m MutateBiddingSeasonalityAdjustmentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingSeasonalityAdjustmentsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsRequest} message MutateBiddingSeasonalityAdjustmentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingSeasonalityAdjustmentsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest} MutateBiddingSeasonalityAdjustmentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingSeasonalityAdjustmentsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest} MutateBiddingSeasonalityAdjustmentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingSeasonalityAdjustmentsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingSeasonalityAdjustmentsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest} MutateBiddingSeasonalityAdjustmentsRequest
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingSeasonalityAdjustmentsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest} m MutateBiddingSeasonalityAdjustmentsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingSeasonalityAdjustmentsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingSeasonalityAdjustmentsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingSeasonalityAdjustmentsRequest;
                        })();
    
                        services.BiddingSeasonalityAdjustmentOperation = (function() {
    
                            /**
                             * Properties of a BiddingSeasonalityAdjustmentOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IBiddingSeasonalityAdjustmentOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] BiddingSeasonalityAdjustmentOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null} [create] BiddingSeasonalityAdjustmentOperation create
                             * @property {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null} [update] BiddingSeasonalityAdjustmentOperation update
                             * @property {string|null} [remove] BiddingSeasonalityAdjustmentOperation remove
                             */
    
                            /**
                             * Constructs a new BiddingSeasonalityAdjustmentOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BiddingSeasonalityAdjustmentOperation.
                             * @implements IBiddingSeasonalityAdjustmentOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation=} [p] Properties to set
                             */
                            function BiddingSeasonalityAdjustmentOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BiddingSeasonalityAdjustmentOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @instance
                             */
                            BiddingSeasonalityAdjustmentOperation.prototype.update_mask = null;
    
                            /**
                             * BiddingSeasonalityAdjustmentOperation create.
                             * @member {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @instance
                             */
                            BiddingSeasonalityAdjustmentOperation.prototype.create = null;
    
                            /**
                             * BiddingSeasonalityAdjustmentOperation update.
                             * @member {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @instance
                             */
                            BiddingSeasonalityAdjustmentOperation.prototype.update = null;
    
                            /**
                             * BiddingSeasonalityAdjustmentOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @instance
                             */
                            BiddingSeasonalityAdjustmentOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BiddingSeasonalityAdjustmentOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @instance
                             */
                            Object.defineProperty(BiddingSeasonalityAdjustmentOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BiddingSeasonalityAdjustmentOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation} BiddingSeasonalityAdjustmentOperation instance
                             */
                            BiddingSeasonalityAdjustmentOperation.create = function create(properties) {
                                return new BiddingSeasonalityAdjustmentOperation(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingSeasonalityAdjustmentOperation message. Does not implicitly {@link google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation} m BiddingSeasonalityAdjustmentOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingSeasonalityAdjustmentOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingSeasonalityAdjustmentOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingSeasonalityAdjustmentOperation} message BiddingSeasonalityAdjustmentOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingSeasonalityAdjustmentOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingSeasonalityAdjustmentOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation} BiddingSeasonalityAdjustmentOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingSeasonalityAdjustmentOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingSeasonalityAdjustmentOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation} BiddingSeasonalityAdjustmentOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingSeasonalityAdjustmentOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingSeasonalityAdjustmentOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingSeasonalityAdjustmentOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BiddingSeasonalityAdjustmentOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation} BiddingSeasonalityAdjustmentOperation
                             */
                            BiddingSeasonalityAdjustmentOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BiddingSeasonalityAdjustmentOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation} m BiddingSeasonalityAdjustmentOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingSeasonalityAdjustmentOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BiddingSeasonalityAdjustmentOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.BiddingSeasonalityAdjustmentOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingSeasonalityAdjustmentOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BiddingSeasonalityAdjustmentOperation;
                        })();
    
                        services.MutateBiddingSeasonalityAdjustmentsResponse = (function() {
    
                            /**
                             * Properties of a MutateBiddingSeasonalityAdjustmentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingSeasonalityAdjustmentsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateBiddingSeasonalityAdjustmentsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult>|null} [results] MutateBiddingSeasonalityAdjustmentsResponse results
                             */
    
                            /**
                             * Constructs a new MutateBiddingSeasonalityAdjustmentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingSeasonalityAdjustmentsResponse.
                             * @implements IMutateBiddingSeasonalityAdjustmentsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResponse=} [p] Properties to set
                             */
                            function MutateBiddingSeasonalityAdjustmentsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateBiddingSeasonalityAdjustmentsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse} MutateBiddingSeasonalityAdjustmentsResponse instance
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.create = function create(properties) {
                                return new MutateBiddingSeasonalityAdjustmentsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingSeasonalityAdjustmentsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResponse} m MutateBiddingSeasonalityAdjustmentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingSeasonalityAdjustmentsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResponse} message MutateBiddingSeasonalityAdjustmentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingSeasonalityAdjustmentsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse} MutateBiddingSeasonalityAdjustmentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingSeasonalityAdjustmentsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse} MutateBiddingSeasonalityAdjustmentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingSeasonalityAdjustmentsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingSeasonalityAdjustmentsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse} MutateBiddingSeasonalityAdjustmentsResponse
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingSeasonalityAdjustmentsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse} m MutateBiddingSeasonalityAdjustmentsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingSeasonalityAdjustmentsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingSeasonalityAdjustmentsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingSeasonalityAdjustmentsResponse;
                        })();
    
                        services.MutateBiddingSeasonalityAdjustmentsResult = (function() {
    
                            /**
                             * Properties of a MutateBiddingSeasonalityAdjustmentsResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingSeasonalityAdjustmentsResult
                             * @property {string|null} [resource_name] MutateBiddingSeasonalityAdjustmentsResult resource_name
                             * @property {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null} [bidding_seasonality_adjustment] MutateBiddingSeasonalityAdjustmentsResult bidding_seasonality_adjustment
                             */
    
                            /**
                             * Constructs a new MutateBiddingSeasonalityAdjustmentsResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingSeasonalityAdjustmentsResult.
                             * @implements IMutateBiddingSeasonalityAdjustmentsResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult=} [p] Properties to set
                             */
                            function MutateBiddingSeasonalityAdjustmentsResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.prototype.resource_name = "";
    
                            /**
                             * MutateBiddingSeasonalityAdjustmentsResult bidding_seasonality_adjustment.
                             * @member {google.ads.googleads.v11.resources.IBiddingSeasonalityAdjustment|null|undefined} bidding_seasonality_adjustment
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @instance
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.prototype.bidding_seasonality_adjustment = null;
    
                            /**
                             * Creates a new MutateBiddingSeasonalityAdjustmentsResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult} MutateBiddingSeasonalityAdjustmentsResult instance
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.create = function create(properties) {
                                return new MutateBiddingSeasonalityAdjustmentsResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingSeasonalityAdjustmentsResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult} m MutateBiddingSeasonalityAdjustmentsResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.bidding_seasonality_adjustment != null && Object.hasOwnProperty.call(m, "bidding_seasonality_adjustment"))
                                    $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.encode(m.bidding_seasonality_adjustment, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingSeasonalityAdjustmentsResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingSeasonalityAdjustmentsResult} message MutateBiddingSeasonalityAdjustmentsResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingSeasonalityAdjustmentsResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult} MutateBiddingSeasonalityAdjustmentsResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.bidding_seasonality_adjustment = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingSeasonalityAdjustmentsResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult} MutateBiddingSeasonalityAdjustmentsResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingSeasonalityAdjustmentsResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.bidding_seasonality_adjustment != null && m.hasOwnProperty("bidding_seasonality_adjustment")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.verify(m.bidding_seasonality_adjustment);
                                        if (e)
                                            return "bidding_seasonality_adjustment." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingSeasonalityAdjustmentsResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult} MutateBiddingSeasonalityAdjustmentsResult
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.bidding_seasonality_adjustment != null) {
                                    if (typeof d.bidding_seasonality_adjustment !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult.bidding_seasonality_adjustment: object expected");
                                    m.bidding_seasonality_adjustment = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.fromObject(d.bidding_seasonality_adjustment);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingSeasonalityAdjustmentsResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult} m MutateBiddingSeasonalityAdjustmentsResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.bidding_seasonality_adjustment = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.bidding_seasonality_adjustment != null && m.hasOwnProperty("bidding_seasonality_adjustment")) {
                                    d.bidding_seasonality_adjustment = $root.google.ads.googleads.v11.resources.BiddingSeasonalityAdjustment.toObject(m.bidding_seasonality_adjustment, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingSeasonalityAdjustmentsResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingSeasonalityAdjustmentsResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingSeasonalityAdjustmentsResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingSeasonalityAdjustmentsResult;
                        })();
    
                        services.BiddingStrategyService = (function() {
    
                            /**
                             * Constructs a new BiddingStrategyService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BiddingStrategyService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function BiddingStrategyService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (BiddingStrategyService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = BiddingStrategyService;
    
                            /**
                             * Creates new BiddingStrategyService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {BiddingStrategyService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            BiddingStrategyService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BiddingStrategyService|mutateBiddingStrategies}.
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyService
                             * @typedef MutateBiddingStrategiesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateBiddingStrategiesResponse} [response] MutateBiddingStrategiesResponse
                             */
    
                            /**
                             * Calls MutateBiddingStrategies.
                             * @function mutateBiddingStrategies
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesRequest} request MutateBiddingStrategiesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BiddingStrategyService.MutateBiddingStrategiesCallback} callback Node-style callback called with the error, if any, and MutateBiddingStrategiesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BiddingStrategyService.prototype.mutateBiddingStrategies = function mutateBiddingStrategies(request, callback) {
                                return this.rpcCall(mutateBiddingStrategies, $root.google.ads.googleads.v11.services.MutateBiddingStrategiesRequest, $root.google.ads.googleads.v11.services.MutateBiddingStrategiesResponse, request, callback);
                            }, "name", { value: "MutateBiddingStrategies" });
    
                            /**
                             * Calls MutateBiddingStrategies.
                             * @function mutateBiddingStrategies
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesRequest} request MutateBiddingStrategiesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateBiddingStrategiesResponse>} Promise
                             * @variation 2
                             */
    
                            return BiddingStrategyService;
                        })();
    
                        services.MutateBiddingStrategiesRequest = (function() {
    
                            /**
                             * Properties of a MutateBiddingStrategiesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingStrategiesRequest
                             * @property {string|null} [customer_id] MutateBiddingStrategiesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IBiddingStrategyOperation>|null} [operations] MutateBiddingStrategiesRequest operations
                             * @property {boolean|null} [partial_failure] MutateBiddingStrategiesRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateBiddingStrategiesRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateBiddingStrategiesRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateBiddingStrategiesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingStrategiesRequest.
                             * @implements IMutateBiddingStrategiesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesRequest=} [p] Properties to set
                             */
                            function MutateBiddingStrategiesRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingStrategiesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @instance
                             */
                            MutateBiddingStrategiesRequest.prototype.customer_id = "";
    
                            /**
                             * MutateBiddingStrategiesRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IBiddingStrategyOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @instance
                             */
                            MutateBiddingStrategiesRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateBiddingStrategiesRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @instance
                             */
                            MutateBiddingStrategiesRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateBiddingStrategiesRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @instance
                             */
                            MutateBiddingStrategiesRequest.prototype.validate_only = false;
    
                            /**
                             * MutateBiddingStrategiesRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @instance
                             */
                            MutateBiddingStrategiesRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateBiddingStrategiesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesRequest} MutateBiddingStrategiesRequest instance
                             */
                            MutateBiddingStrategiesRequest.create = function create(properties) {
                                return new MutateBiddingStrategiesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingStrategiesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingStrategiesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesRequest} m MutateBiddingStrategiesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingStrategiesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.BiddingStrategyOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingStrategiesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingStrategiesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesRequest} message MutateBiddingStrategiesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingStrategiesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingStrategiesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesRequest} MutateBiddingStrategiesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingStrategiesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingStrategiesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.BiddingStrategyOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingStrategiesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesRequest} MutateBiddingStrategiesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingStrategiesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingStrategiesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingStrategiesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.BiddingStrategyOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingStrategiesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesRequest} MutateBiddingStrategiesRequest
                             */
                            MutateBiddingStrategiesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingStrategiesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingStrategiesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingStrategiesRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateBiddingStrategiesRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.BiddingStrategyOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingStrategiesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingStrategiesRequest} m MutateBiddingStrategiesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingStrategiesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.BiddingStrategyOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingStrategiesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingStrategiesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingStrategiesRequest;
                        })();
    
                        services.BiddingStrategyOperation = (function() {
    
                            /**
                             * Properties of a BiddingStrategyOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IBiddingStrategyOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] BiddingStrategyOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IBiddingStrategy|null} [create] BiddingStrategyOperation create
                             * @property {google.ads.googleads.v11.resources.IBiddingStrategy|null} [update] BiddingStrategyOperation update
                             * @property {string|null} [remove] BiddingStrategyOperation remove
                             */
    
                            /**
                             * Constructs a new BiddingStrategyOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BiddingStrategyOperation.
                             * @implements IBiddingStrategyOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IBiddingStrategyOperation=} [p] Properties to set
                             */
                            function BiddingStrategyOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BiddingStrategyOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @instance
                             */
                            BiddingStrategyOperation.prototype.update_mask = null;
    
                            /**
                             * BiddingStrategyOperation create.
                             * @member {google.ads.googleads.v11.resources.IBiddingStrategy|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @instance
                             */
                            BiddingStrategyOperation.prototype.create = null;
    
                            /**
                             * BiddingStrategyOperation update.
                             * @member {google.ads.googleads.v11.resources.IBiddingStrategy|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @instance
                             */
                            BiddingStrategyOperation.prototype.update = null;
    
                            /**
                             * BiddingStrategyOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @instance
                             */
                            BiddingStrategyOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BiddingStrategyOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @instance
                             */
                            Object.defineProperty(BiddingStrategyOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BiddingStrategyOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingStrategyOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.BiddingStrategyOperation} BiddingStrategyOperation instance
                             */
                            BiddingStrategyOperation.create = function create(properties) {
                                return new BiddingStrategyOperation(properties);
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyOperation message. Does not implicitly {@link google.ads.googleads.v11.services.BiddingStrategyOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingStrategyOperation} m BiddingStrategyOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.BiddingStrategy.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.BiddingStrategy.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BiddingStrategyOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.BiddingStrategyOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBiddingStrategyOperation} message BiddingStrategyOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BiddingStrategyOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BiddingStrategyOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.BiddingStrategyOperation} BiddingStrategyOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.BiddingStrategyOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.BiddingStrategy.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.BiddingStrategy.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BiddingStrategyOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.BiddingStrategyOperation} BiddingStrategyOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BiddingStrategyOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BiddingStrategyOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BiddingStrategyOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingStrategy.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingStrategy.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BiddingStrategyOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.BiddingStrategyOperation} BiddingStrategyOperation
                             */
                            BiddingStrategyOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.BiddingStrategyOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.BiddingStrategyOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingStrategyOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingStrategyOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.BiddingStrategy.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BiddingStrategyOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.BiddingStrategy.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BiddingStrategyOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.BiddingStrategyOperation} m BiddingStrategyOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BiddingStrategyOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.BiddingStrategy.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.BiddingStrategy.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BiddingStrategyOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.BiddingStrategyOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BiddingStrategyOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BiddingStrategyOperation;
                        })();
    
                        services.MutateBiddingStrategiesResponse = (function() {
    
                            /**
                             * Properties of a MutateBiddingStrategiesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingStrategiesResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateBiddingStrategiesResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateBiddingStrategyResult>|null} [results] MutateBiddingStrategiesResponse results
                             */
    
                            /**
                             * Constructs a new MutateBiddingStrategiesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingStrategiesResponse.
                             * @implements IMutateBiddingStrategiesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesResponse=} [p] Properties to set
                             */
                            function MutateBiddingStrategiesResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingStrategiesResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @instance
                             */
                            MutateBiddingStrategiesResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateBiddingStrategiesResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateBiddingStrategyResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @instance
                             */
                            MutateBiddingStrategiesResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateBiddingStrategiesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesResponse} MutateBiddingStrategiesResponse instance
                             */
                            MutateBiddingStrategiesResponse.create = function create(properties) {
                                return new MutateBiddingStrategiesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingStrategiesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingStrategiesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesResponse} m MutateBiddingStrategiesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingStrategiesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingStrategiesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingStrategiesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategiesResponse} message MutateBiddingStrategiesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingStrategiesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingStrategiesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesResponse} MutateBiddingStrategiesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingStrategiesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingStrategiesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingStrategiesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesResponse} MutateBiddingStrategiesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingStrategiesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingStrategiesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingStrategiesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingStrategiesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategiesResponse} MutateBiddingStrategiesResponse
                             */
                            MutateBiddingStrategiesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingStrategiesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingStrategiesResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingStrategiesResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingStrategiesResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateBiddingStrategiesResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingStrategiesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingStrategiesResponse} m MutateBiddingStrategiesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingStrategiesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingStrategiesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategiesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingStrategiesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingStrategiesResponse;
                        })();
    
                        services.MutateBiddingStrategyResult = (function() {
    
                            /**
                             * Properties of a MutateBiddingStrategyResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBiddingStrategyResult
                             * @property {string|null} [resource_name] MutateBiddingStrategyResult resource_name
                             * @property {google.ads.googleads.v11.resources.IBiddingStrategy|null} [bidding_strategy] MutateBiddingStrategyResult bidding_strategy
                             */
    
                            /**
                             * Constructs a new MutateBiddingStrategyResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBiddingStrategyResult.
                             * @implements IMutateBiddingStrategyResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategyResult=} [p] Properties to set
                             */
                            function MutateBiddingStrategyResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBiddingStrategyResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @instance
                             */
                            MutateBiddingStrategyResult.prototype.resource_name = "";
    
                            /**
                             * MutateBiddingStrategyResult bidding_strategy.
                             * @member {google.ads.googleads.v11.resources.IBiddingStrategy|null|undefined} bidding_strategy
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @instance
                             */
                            MutateBiddingStrategyResult.prototype.bidding_strategy = null;
    
                            /**
                             * Creates a new MutateBiddingStrategyResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategyResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategyResult} MutateBiddingStrategyResult instance
                             */
                            MutateBiddingStrategyResult.create = function create(properties) {
                                return new MutateBiddingStrategyResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBiddingStrategyResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingStrategyResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategyResult} m MutateBiddingStrategyResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingStrategyResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.bidding_strategy != null && Object.hasOwnProperty.call(m, "bidding_strategy"))
                                    $root.google.ads.googleads.v11.resources.BiddingStrategy.encode(m.bidding_strategy, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBiddingStrategyResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBiddingStrategyResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBiddingStrategyResult} message MutateBiddingStrategyResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBiddingStrategyResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBiddingStrategyResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategyResult} MutateBiddingStrategyResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingStrategyResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.bidding_strategy = $root.google.ads.googleads.v11.resources.BiddingStrategy.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBiddingStrategyResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategyResult} MutateBiddingStrategyResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBiddingStrategyResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBiddingStrategyResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBiddingStrategyResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.bidding_strategy != null && m.hasOwnProperty("bidding_strategy")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BiddingStrategy.verify(m.bidding_strategy);
                                        if (e)
                                            return "bidding_strategy." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBiddingStrategyResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBiddingStrategyResult} MutateBiddingStrategyResult
                             */
                            MutateBiddingStrategyResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBiddingStrategyResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.bidding_strategy != null) {
                                    if (typeof d.bidding_strategy !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBiddingStrategyResult.bidding_strategy: object expected");
                                    m.bidding_strategy = $root.google.ads.googleads.v11.resources.BiddingStrategy.fromObject(d.bidding_strategy);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBiddingStrategyResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBiddingStrategyResult} m MutateBiddingStrategyResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBiddingStrategyResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.bidding_strategy = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.bidding_strategy != null && m.hasOwnProperty("bidding_strategy")) {
                                    d.bidding_strategy = $root.google.ads.googleads.v11.resources.BiddingStrategy.toObject(m.bidding_strategy, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBiddingStrategyResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBiddingStrategyResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBiddingStrategyResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBiddingStrategyResult;
                        })();
    
                        services.CampaignAssetService = (function() {
    
                            /**
                             * Constructs a new CampaignAssetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignAssetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignAssetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignAssetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignAssetService;
    
                            /**
                             * Creates new CampaignAssetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignAssetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignAssetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignAssetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignAssetService|mutateCampaignAssets}.
                             * @memberof google.ads.googleads.v11.services.CampaignAssetService
                             * @typedef MutateCampaignAssetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetsResponse} [response] MutateCampaignAssetsResponse
                             */
    
                            /**
                             * Calls MutateCampaignAssets.
                             * @function mutateCampaignAssets
                             * @memberof google.ads.googleads.v11.services.CampaignAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsRequest} request MutateCampaignAssetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignAssetService.MutateCampaignAssetsCallback} callback Node-style callback called with the error, if any, and MutateCampaignAssetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignAssetService.prototype.mutateCampaignAssets = function mutateCampaignAssets(request, callback) {
                                return this.rpcCall(mutateCampaignAssets, $root.google.ads.googleads.v11.services.MutateCampaignAssetsRequest, $root.google.ads.googleads.v11.services.MutateCampaignAssetsResponse, request, callback);
                            }, "name", { value: "MutateCampaignAssets" });
    
                            /**
                             * Calls MutateCampaignAssets.
                             * @function mutateCampaignAssets
                             * @memberof google.ads.googleads.v11.services.CampaignAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsRequest} request MutateCampaignAssetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignAssetsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignAssetService;
                        })();
    
                        services.MutateCampaignAssetsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignAssetsRequest
                             * @property {string|null} [customer_id] MutateCampaignAssetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignAssetOperation>|null} [operations] MutateCampaignAssetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignAssetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignAssetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignAssetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignAssetsRequest.
                             * @implements IMutateCampaignAssetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsRequest=} [p] Properties to set
                             */
                            function MutateCampaignAssetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignAssetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @instance
                             */
                            MutateCampaignAssetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignAssetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignAssetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @instance
                             */
                            MutateCampaignAssetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignAssetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @instance
                             */
                            MutateCampaignAssetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignAssetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @instance
                             */
                            MutateCampaignAssetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignAssetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @instance
                             */
                            MutateCampaignAssetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignAssetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsRequest} MutateCampaignAssetsRequest instance
                             */
                            MutateCampaignAssetsRequest.create = function create(properties) {
                                return new MutateCampaignAssetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsRequest} m MutateCampaignAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignAssetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsRequest} message MutateCampaignAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsRequest} MutateCampaignAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignAssetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsRequest} MutateCampaignAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignAssetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignAssetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignAssetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignAssetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsRequest} MutateCampaignAssetsRequest
                             */
                            MutateCampaignAssetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignAssetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignAssetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignAssetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetsRequest} m MutateCampaignAssetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignAssetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignAssetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignAssetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignAssetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignAssetsRequest;
                        })();
    
                        services.CampaignAssetOperation = (function() {
    
                            /**
                             * Properties of a CampaignAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignAssetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignAssetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignAsset|null} [create] CampaignAssetOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignAsset|null} [update] CampaignAssetOperation update
                             * @property {string|null} [remove] CampaignAssetOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignAssetOperation.
                             * @implements ICampaignAssetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignAssetOperation=} [p] Properties to set
                             */
                            function CampaignAssetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignAssetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @instance
                             */
                            CampaignAssetOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignAssetOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignAsset|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @instance
                             */
                            CampaignAssetOperation.prototype.create = null;
    
                            /**
                             * CampaignAssetOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignAsset|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @instance
                             */
                            CampaignAssetOperation.prototype.update = null;
    
                            /**
                             * CampaignAssetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @instance
                             */
                            CampaignAssetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignAssetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignAssetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignAssetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignAssetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignAssetOperation} CampaignAssetOperation instance
                             */
                            CampaignAssetOperation.create = function create(properties) {
                                return new CampaignAssetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignAssetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignAssetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignAssetOperation} m CampaignAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAssetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignAsset.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignAsset.encode(m.update, w.uint32(26).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignAssetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignAssetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignAssetOperation} message CampaignAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAssetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignAssetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignAssetOperation} CampaignAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAssetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignAssetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignAsset.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignAsset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignAssetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignAssetOperation} CampaignAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAssetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignAssetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignAssetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAsset.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAsset.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignAssetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignAssetOperation} CampaignAssetOperation
                             */
                            CampaignAssetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignAssetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignAssetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignAssetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignAssetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignAsset.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignAssetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignAsset.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignAssetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignAssetOperation} m CampaignAssetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignAssetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignAsset.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignAsset.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignAssetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignAssetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignAssetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignAssetOperation;
                        })();
    
                        services.MutateCampaignAssetsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignAssetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignAssetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignAssetResult>|null} [results] MutateCampaignAssetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignAssetsResponse.
                             * @implements IMutateCampaignAssetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsResponse=} [p] Properties to set
                             */
                            function MutateCampaignAssetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignAssetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @instance
                             */
                            MutateCampaignAssetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignAssetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignAssetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @instance
                             */
                            MutateCampaignAssetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignAssetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsResponse} MutateCampaignAssetsResponse instance
                             */
                            MutateCampaignAssetsResponse.create = function create(properties) {
                                return new MutateCampaignAssetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsResponse} m MutateCampaignAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetsResponse} message MutateCampaignAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsResponse} MutateCampaignAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignAssetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsResponse} MutateCampaignAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignAssetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignAssetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignAssetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetsResponse} MutateCampaignAssetsResponse
                             */
                            MutateCampaignAssetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignAssetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignAssetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetsResponse} m MutateCampaignAssetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignAssetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignAssetResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignAssetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignAssetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignAssetsResponse;
                        })();
    
                        services.MutateCampaignAssetResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignAssetResult
                             * @property {string|null} [resource_name] MutateCampaignAssetResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignAsset|null} [campaign_asset] MutateCampaignAssetResult campaign_asset
                             */
    
                            /**
                             * Constructs a new MutateCampaignAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignAssetResult.
                             * @implements IMutateCampaignAssetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetResult=} [p] Properties to set
                             */
                            function MutateCampaignAssetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignAssetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @instance
                             */
                            MutateCampaignAssetResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignAssetResult campaign_asset.
                             * @member {google.ads.googleads.v11.resources.ICampaignAsset|null|undefined} campaign_asset
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @instance
                             */
                            MutateCampaignAssetResult.prototype.campaign_asset = null;
    
                            /**
                             * Creates a new MutateCampaignAssetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetResult} MutateCampaignAssetResult instance
                             */
                            MutateCampaignAssetResult.create = function create(properties) {
                                return new MutateCampaignAssetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetResult} m MutateCampaignAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_asset != null && Object.hasOwnProperty.call(m, "campaign_asset"))
                                    $root.google.ads.googleads.v11.resources.CampaignAsset.encode(m.campaign_asset, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetResult} message MutateCampaignAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetResult} MutateCampaignAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetResult} MutateCampaignAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignAssetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignAssetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAsset.verify(m.campaign_asset);
                                        if (e)
                                            return "campaign_asset." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignAssetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetResult} MutateCampaignAssetResult
                             */
                            MutateCampaignAssetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignAssetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_asset != null) {
                                    if (typeof d.campaign_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetResult.campaign_asset: object expected");
                                    m.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.fromObject(d.campaign_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignAssetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetResult} m MutateCampaignAssetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignAssetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_asset = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_asset != null && m.hasOwnProperty("campaign_asset")) {
                                    d.campaign_asset = $root.google.ads.googleads.v11.resources.CampaignAsset.toObject(m.campaign_asset, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignAssetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignAssetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignAssetResult;
                        })();
    
                        services.CampaignAssetSetService = (function() {
    
                            /**
                             * Constructs a new CampaignAssetSetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignAssetSetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignAssetSetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignAssetSetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignAssetSetService;
    
                            /**
                             * Creates new CampaignAssetSetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignAssetSetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignAssetSetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignAssetSetService|mutateCampaignAssetSets}.
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetService
                             * @typedef MutateCampaignAssetSetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse} [response] MutateCampaignAssetSetsResponse
                             */
    
                            /**
                             * Calls MutateCampaignAssetSets.
                             * @function mutateCampaignAssetSets
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsRequest} request MutateCampaignAssetSetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignAssetSetService.MutateCampaignAssetSetsCallback} callback Node-style callback called with the error, if any, and MutateCampaignAssetSetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignAssetSetService.prototype.mutateCampaignAssetSets = function mutateCampaignAssetSets(request, callback) {
                                return this.rpcCall(mutateCampaignAssetSets, $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest, $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse, request, callback);
                            }, "name", { value: "MutateCampaignAssetSets" });
    
                            /**
                             * Calls MutateCampaignAssetSets.
                             * @function mutateCampaignAssetSets
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsRequest} request MutateCampaignAssetSetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignAssetSetService;
                        })();
    
                        services.MutateCampaignAssetSetsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignAssetSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignAssetSetsRequest
                             * @property {string|null} [customer_id] MutateCampaignAssetSetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignAssetSetOperation>|null} [operations] MutateCampaignAssetSetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignAssetSetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignAssetSetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignAssetSetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignAssetSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignAssetSetsRequest.
                             * @implements IMutateCampaignAssetSetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsRequest=} [p] Properties to set
                             */
                            function MutateCampaignAssetSetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignAssetSetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @instance
                             */
                            MutateCampaignAssetSetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignAssetSetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignAssetSetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @instance
                             */
                            MutateCampaignAssetSetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignAssetSetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @instance
                             */
                            MutateCampaignAssetSetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignAssetSetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @instance
                             */
                            MutateCampaignAssetSetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignAssetSetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @instance
                             */
                            MutateCampaignAssetSetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignAssetSetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest} MutateCampaignAssetSetsRequest instance
                             */
                            MutateCampaignAssetSetsRequest.create = function create(properties) {
                                return new MutateCampaignAssetSetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetSetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsRequest} m MutateCampaignAssetSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetSetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetSetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsRequest} message MutateCampaignAssetSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetSetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetSetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest} MutateCampaignAssetSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetSetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignAssetSetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetSetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest} MutateCampaignAssetSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetSetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignAssetSetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignAssetSetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignAssetSetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest} MutateCampaignAssetSetsRequest
                             */
                            MutateCampaignAssetSetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignAssetSetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest} m MutateCampaignAssetSetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignAssetSetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignAssetSetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignAssetSetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignAssetSetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignAssetSetsRequest;
                        })();
    
                        services.CampaignAssetSetOperation = (function() {
    
                            /**
                             * Properties of a CampaignAssetSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignAssetSetOperation
                             * @property {google.ads.googleads.v11.resources.ICampaignAssetSet|null} [create] CampaignAssetSetOperation create
                             * @property {string|null} [remove] CampaignAssetSetOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignAssetSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignAssetSetOperation.
                             * @implements ICampaignAssetSetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignAssetSetOperation=} [p] Properties to set
                             */
                            function CampaignAssetSetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignAssetSetOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignAssetSet|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @instance
                             */
                            CampaignAssetSetOperation.prototype.create = null;
    
                            /**
                             * CampaignAssetSetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @instance
                             */
                            CampaignAssetSetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignAssetSetOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignAssetSetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignAssetSetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignAssetSetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignAssetSetOperation} CampaignAssetSetOperation instance
                             */
                            CampaignAssetSetOperation.create = function create(properties) {
                                return new CampaignAssetSetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignAssetSetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignAssetSetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignAssetSetOperation} m CampaignAssetSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAssetSetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignAssetSet.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignAssetSetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignAssetSetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignAssetSetOperation} message CampaignAssetSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignAssetSetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignAssetSetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignAssetSetOperation} CampaignAssetSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAssetSetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignAssetSetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignAssetSet.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignAssetSetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignAssetSetOperation} CampaignAssetSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignAssetSetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignAssetSetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignAssetSetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAssetSet.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignAssetSetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignAssetSetOperation} CampaignAssetSetOperation
                             */
                            CampaignAssetSetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignAssetSetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignAssetSetOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignAssetSetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignAssetSet.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignAssetSetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignAssetSetOperation} m CampaignAssetSetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignAssetSetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignAssetSet.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignAssetSetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignAssetSetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignAssetSetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignAssetSetOperation;
                        })();
    
                        services.MutateCampaignAssetSetsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignAssetSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignAssetSetsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignAssetSetResult>|null} [results] MutateCampaignAssetSetsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignAssetSetsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateCampaignAssetSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignAssetSetsResponse.
                             * @implements IMutateCampaignAssetSetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsResponse=} [p] Properties to set
                             */
                            function MutateCampaignAssetSetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignAssetSetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignAssetSetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @instance
                             */
                            MutateCampaignAssetSetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateCampaignAssetSetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @instance
                             */
                            MutateCampaignAssetSetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateCampaignAssetSetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse} MutateCampaignAssetSetsResponse instance
                             */
                            MutateCampaignAssetSetsResponse.create = function create(properties) {
                                return new MutateCampaignAssetSetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetSetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsResponse} m MutateCampaignAssetSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetSetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetSetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetsResponse} message MutateCampaignAssetSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetSetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetSetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse} MutateCampaignAssetSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetSetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetSetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse} MutateCampaignAssetSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetSetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignAssetSetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignAssetSetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignAssetSetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse} MutateCampaignAssetSetsResponse
                             */
                            MutateCampaignAssetSetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignAssetSetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse} m MutateCampaignAssetSetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignAssetSetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignAssetSetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignAssetSetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignAssetSetsResponse;
                        })();
    
                        services.MutateCampaignAssetSetResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignAssetSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignAssetSetResult
                             * @property {string|null} [resource_name] MutateCampaignAssetSetResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignAssetSet|null} [campaign_asset_set] MutateCampaignAssetSetResult campaign_asset_set
                             */
    
                            /**
                             * Constructs a new MutateCampaignAssetSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignAssetSetResult.
                             * @implements IMutateCampaignAssetSetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetResult=} [p] Properties to set
                             */
                            function MutateCampaignAssetSetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignAssetSetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @instance
                             */
                            MutateCampaignAssetSetResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignAssetSetResult campaign_asset_set.
                             * @member {google.ads.googleads.v11.resources.ICampaignAssetSet|null|undefined} campaign_asset_set
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @instance
                             */
                            MutateCampaignAssetSetResult.prototype.campaign_asset_set = null;
    
                            /**
                             * Creates a new MutateCampaignAssetSetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetResult} MutateCampaignAssetSetResult instance
                             */
                            MutateCampaignAssetSetResult.create = function create(properties) {
                                return new MutateCampaignAssetSetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetSetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetSetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetResult} m MutateCampaignAssetSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetSetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_asset_set != null && Object.hasOwnProperty.call(m, "campaign_asset_set"))
                                    $root.google.ads.googleads.v11.resources.CampaignAssetSet.encode(m.campaign_asset_set, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignAssetSetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignAssetSetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignAssetSetResult} message MutateCampaignAssetSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignAssetSetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetSetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetResult} MutateCampaignAssetSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetSetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignAssetSetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetResult} MutateCampaignAssetSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignAssetSetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignAssetSetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignAssetSetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_asset_set != null && m.hasOwnProperty("campaign_asset_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignAssetSet.verify(m.campaign_asset_set);
                                        if (e)
                                            return "campaign_asset_set." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignAssetSetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignAssetSetResult} MutateCampaignAssetSetResult
                             */
                            MutateCampaignAssetSetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignAssetSetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_asset_set != null) {
                                    if (typeof d.campaign_asset_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignAssetSetResult.campaign_asset_set: object expected");
                                    m.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.fromObject(d.campaign_asset_set);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignAssetSetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignAssetSetResult} m MutateCampaignAssetSetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignAssetSetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_asset_set = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_asset_set != null && m.hasOwnProperty("campaign_asset_set")) {
                                    d.campaign_asset_set = $root.google.ads.googleads.v11.resources.CampaignAssetSet.toObject(m.campaign_asset_set, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignAssetSetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignAssetSetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignAssetSetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignAssetSetResult;
                        })();
    
                        services.CampaignBidModifierService = (function() {
    
                            /**
                             * Constructs a new CampaignBidModifierService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignBidModifierService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignBidModifierService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignBidModifierService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignBidModifierService;
    
                            /**
                             * Creates new CampaignBidModifierService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignBidModifierService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignBidModifierService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignBidModifierService|mutateCampaignBidModifiers}.
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierService
                             * @typedef MutateCampaignBidModifiersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse} [response] MutateCampaignBidModifiersResponse
                             */
    
                            /**
                             * Calls MutateCampaignBidModifiers.
                             * @function mutateCampaignBidModifiers
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersRequest} request MutateCampaignBidModifiersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignBidModifierService.MutateCampaignBidModifiersCallback} callback Node-style callback called with the error, if any, and MutateCampaignBidModifiersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignBidModifierService.prototype.mutateCampaignBidModifiers = function mutateCampaignBidModifiers(request, callback) {
                                return this.rpcCall(mutateCampaignBidModifiers, $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest, $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse, request, callback);
                            }, "name", { value: "MutateCampaignBidModifiers" });
    
                            /**
                             * Calls MutateCampaignBidModifiers.
                             * @function mutateCampaignBidModifiers
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersRequest} request MutateCampaignBidModifiersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignBidModifierService;
                        })();
    
                        services.MutateCampaignBidModifiersRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignBidModifiersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignBidModifiersRequest
                             * @property {string|null} [customer_id] MutateCampaignBidModifiersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignBidModifierOperation>|null} [operations] MutateCampaignBidModifiersRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignBidModifiersRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignBidModifiersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignBidModifiersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignBidModifiersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignBidModifiersRequest.
                             * @implements IMutateCampaignBidModifiersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersRequest=} [p] Properties to set
                             */
                            function MutateCampaignBidModifiersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignBidModifiersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @instance
                             */
                            MutateCampaignBidModifiersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignBidModifiersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignBidModifierOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @instance
                             */
                            MutateCampaignBidModifiersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignBidModifiersRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @instance
                             */
                            MutateCampaignBidModifiersRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignBidModifiersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @instance
                             */
                            MutateCampaignBidModifiersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignBidModifiersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @instance
                             */
                            MutateCampaignBidModifiersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignBidModifiersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest} MutateCampaignBidModifiersRequest instance
                             */
                            MutateCampaignBidModifiersRequest.create = function create(properties) {
                                return new MutateCampaignBidModifiersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBidModifiersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersRequest} m MutateCampaignBidModifiersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBidModifiersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBidModifiersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersRequest} message MutateCampaignBidModifiersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBidModifiersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignBidModifiersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest} MutateCampaignBidModifiersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBidModifiersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignBidModifierOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignBidModifiersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest} MutateCampaignBidModifiersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBidModifiersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignBidModifiersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignBidModifiersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignBidModifiersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest} MutateCampaignBidModifiersRequest
                             */
                            MutateCampaignBidModifiersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignBidModifiersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest} m MutateCampaignBidModifiersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignBidModifiersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignBidModifierOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignBidModifiersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignBidModifiersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignBidModifiersRequest;
                        })();
    
                        services.CampaignBidModifierOperation = (function() {
    
                            /**
                             * Properties of a CampaignBidModifierOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignBidModifierOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignBidModifierOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignBidModifier|null} [create] CampaignBidModifierOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignBidModifier|null} [update] CampaignBidModifierOperation update
                             * @property {string|null} [remove] CampaignBidModifierOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignBidModifierOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignBidModifierOperation.
                             * @implements ICampaignBidModifierOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignBidModifierOperation=} [p] Properties to set
                             */
                            function CampaignBidModifierOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignBidModifierOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @instance
                             */
                            CampaignBidModifierOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignBidModifierOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignBidModifier|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @instance
                             */
                            CampaignBidModifierOperation.prototype.create = null;
    
                            /**
                             * CampaignBidModifierOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignBidModifier|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @instance
                             */
                            CampaignBidModifierOperation.prototype.update = null;
    
                            /**
                             * CampaignBidModifierOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @instance
                             */
                            CampaignBidModifierOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignBidModifierOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignBidModifierOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignBidModifierOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBidModifierOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignBidModifierOperation} CampaignBidModifierOperation instance
                             */
                            CampaignBidModifierOperation.create = function create(properties) {
                                return new CampaignBidModifierOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignBidModifierOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignBidModifierOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBidModifierOperation} m CampaignBidModifierOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBidModifierOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignBidModifier.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignBidModifier.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignBidModifierOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignBidModifierOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBidModifierOperation} message CampaignBidModifierOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBidModifierOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignBidModifierOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignBidModifierOperation} CampaignBidModifierOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBidModifierOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignBidModifierOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignBidModifier.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignBidModifier.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignBidModifierOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignBidModifierOperation} CampaignBidModifierOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBidModifierOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignBidModifierOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignBidModifierOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBidModifier.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBidModifier.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignBidModifierOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignBidModifierOperation} CampaignBidModifierOperation
                             */
                            CampaignBidModifierOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignBidModifierOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignBidModifierOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignBidModifierOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignBidModifierOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignBidModifier.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignBidModifierOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignBidModifier.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignBidModifierOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignBidModifierOperation} m CampaignBidModifierOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignBidModifierOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignBidModifier.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignBidModifier.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignBidModifierOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignBidModifierOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignBidModifierOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignBidModifierOperation;
                        })();
    
                        services.MutateCampaignBidModifiersResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignBidModifiersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignBidModifiersResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignBidModifiersResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignBidModifierResult>|null} [results] MutateCampaignBidModifiersResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignBidModifiersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignBidModifiersResponse.
                             * @implements IMutateCampaignBidModifiersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersResponse=} [p] Properties to set
                             */
                            function MutateCampaignBidModifiersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignBidModifiersResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @instance
                             */
                            MutateCampaignBidModifiersResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignBidModifiersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignBidModifierResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @instance
                             */
                            MutateCampaignBidModifiersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignBidModifiersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse} MutateCampaignBidModifiersResponse instance
                             */
                            MutateCampaignBidModifiersResponse.create = function create(properties) {
                                return new MutateCampaignBidModifiersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBidModifiersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersResponse} m MutateCampaignBidModifiersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBidModifiersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBidModifiersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifiersResponse} message MutateCampaignBidModifiersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBidModifiersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignBidModifiersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse} MutateCampaignBidModifiersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBidModifiersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignBidModifiersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse} MutateCampaignBidModifiersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBidModifiersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignBidModifiersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignBidModifiersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignBidModifiersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse} MutateCampaignBidModifiersResponse
                             */
                            MutateCampaignBidModifiersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignBidModifiersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse} m MutateCampaignBidModifiersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignBidModifiersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignBidModifiersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifiersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignBidModifiersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignBidModifiersResponse;
                        })();
    
                        services.MutateCampaignBidModifierResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignBidModifierResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignBidModifierResult
                             * @property {string|null} [resource_name] MutateCampaignBidModifierResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignBidModifier|null} [campaign_bid_modifier] MutateCampaignBidModifierResult campaign_bid_modifier
                             */
    
                            /**
                             * Constructs a new MutateCampaignBidModifierResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignBidModifierResult.
                             * @implements IMutateCampaignBidModifierResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifierResult=} [p] Properties to set
                             */
                            function MutateCampaignBidModifierResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignBidModifierResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @instance
                             */
                            MutateCampaignBidModifierResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignBidModifierResult campaign_bid_modifier.
                             * @member {google.ads.googleads.v11.resources.ICampaignBidModifier|null|undefined} campaign_bid_modifier
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @instance
                             */
                            MutateCampaignBidModifierResult.prototype.campaign_bid_modifier = null;
    
                            /**
                             * Creates a new MutateCampaignBidModifierResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifierResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifierResult} MutateCampaignBidModifierResult instance
                             */
                            MutateCampaignBidModifierResult.create = function create(properties) {
                                return new MutateCampaignBidModifierResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBidModifierResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBidModifierResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifierResult} m MutateCampaignBidModifierResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBidModifierResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_bid_modifier != null && Object.hasOwnProperty.call(m, "campaign_bid_modifier"))
                                    $root.google.ads.googleads.v11.resources.CampaignBidModifier.encode(m.campaign_bid_modifier, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBidModifierResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBidModifierResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBidModifierResult} message MutateCampaignBidModifierResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBidModifierResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignBidModifierResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifierResult} MutateCampaignBidModifierResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBidModifierResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_bid_modifier = $root.google.ads.googleads.v11.resources.CampaignBidModifier.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignBidModifierResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifierResult} MutateCampaignBidModifierResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBidModifierResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignBidModifierResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignBidModifierResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_bid_modifier != null && m.hasOwnProperty("campaign_bid_modifier")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBidModifier.verify(m.campaign_bid_modifier);
                                        if (e)
                                            return "campaign_bid_modifier." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignBidModifierResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBidModifierResult} MutateCampaignBidModifierResult
                             */
                            MutateCampaignBidModifierResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignBidModifierResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_bid_modifier != null) {
                                    if (typeof d.campaign_bid_modifier !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBidModifierResult.campaign_bid_modifier: object expected");
                                    m.campaign_bid_modifier = $root.google.ads.googleads.v11.resources.CampaignBidModifier.fromObject(d.campaign_bid_modifier);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignBidModifierResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignBidModifierResult} m MutateCampaignBidModifierResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignBidModifierResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_bid_modifier = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_bid_modifier != null && m.hasOwnProperty("campaign_bid_modifier")) {
                                    d.campaign_bid_modifier = $root.google.ads.googleads.v11.resources.CampaignBidModifier.toObject(m.campaign_bid_modifier, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignBidModifierResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBidModifierResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignBidModifierResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignBidModifierResult;
                        })();
    
                        services.CampaignBudgetService = (function() {
    
                            /**
                             * Constructs a new CampaignBudgetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignBudgetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignBudgetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignBudgetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignBudgetService;
    
                            /**
                             * Creates new CampaignBudgetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignBudgetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignBudgetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignBudgetService|mutateCampaignBudgets}.
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetService
                             * @typedef MutateCampaignBudgetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignBudgetsResponse} [response] MutateCampaignBudgetsResponse
                             */
    
                            /**
                             * Calls MutateCampaignBudgets.
                             * @function mutateCampaignBudgets
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsRequest} request MutateCampaignBudgetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignBudgetService.MutateCampaignBudgetsCallback} callback Node-style callback called with the error, if any, and MutateCampaignBudgetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignBudgetService.prototype.mutateCampaignBudgets = function mutateCampaignBudgets(request, callback) {
                                return this.rpcCall(mutateCampaignBudgets, $root.google.ads.googleads.v11.services.MutateCampaignBudgetsRequest, $root.google.ads.googleads.v11.services.MutateCampaignBudgetsResponse, request, callback);
                            }, "name", { value: "MutateCampaignBudgets" });
    
                            /**
                             * Calls MutateCampaignBudgets.
                             * @function mutateCampaignBudgets
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsRequest} request MutateCampaignBudgetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignBudgetsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignBudgetService;
                        })();
    
                        services.MutateCampaignBudgetsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignBudgetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignBudgetsRequest
                             * @property {string|null} [customer_id] MutateCampaignBudgetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignBudgetOperation>|null} [operations] MutateCampaignBudgetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignBudgetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignBudgetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignBudgetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignBudgetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignBudgetsRequest.
                             * @implements IMutateCampaignBudgetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsRequest=} [p] Properties to set
                             */
                            function MutateCampaignBudgetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignBudgetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @instance
                             */
                            MutateCampaignBudgetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignBudgetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignBudgetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @instance
                             */
                            MutateCampaignBudgetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignBudgetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @instance
                             */
                            MutateCampaignBudgetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignBudgetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @instance
                             */
                            MutateCampaignBudgetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignBudgetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @instance
                             */
                            MutateCampaignBudgetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignBudgetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsRequest} MutateCampaignBudgetsRequest instance
                             */
                            MutateCampaignBudgetsRequest.create = function create(properties) {
                                return new MutateCampaignBudgetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBudgetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBudgetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsRequest} m MutateCampaignBudgetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBudgetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignBudgetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBudgetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBudgetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsRequest} message MutateCampaignBudgetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBudgetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignBudgetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsRequest} MutateCampaignBudgetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBudgetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignBudgetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignBudgetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignBudgetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsRequest} MutateCampaignBudgetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBudgetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignBudgetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignBudgetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignBudgetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignBudgetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsRequest} MutateCampaignBudgetsRequest
                             */
                            MutateCampaignBudgetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignBudgetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignBudgetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBudgetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBudgetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignBudgetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignBudgetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignBudgetsRequest} m MutateCampaignBudgetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignBudgetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignBudgetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignBudgetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignBudgetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignBudgetsRequest;
                        })();
    
                        services.CampaignBudgetOperation = (function() {
    
                            /**
                             * Properties of a CampaignBudgetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignBudgetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignBudgetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignBudget|null} [create] CampaignBudgetOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignBudget|null} [update] CampaignBudgetOperation update
                             * @property {string|null} [remove] CampaignBudgetOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignBudgetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignBudgetOperation.
                             * @implements ICampaignBudgetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetOperation=} [p] Properties to set
                             */
                            function CampaignBudgetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignBudgetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @instance
                             */
                            CampaignBudgetOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignBudgetOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignBudget|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @instance
                             */
                            CampaignBudgetOperation.prototype.create = null;
    
                            /**
                             * CampaignBudgetOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignBudget|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @instance
                             */
                            CampaignBudgetOperation.prototype.update = null;
    
                            /**
                             * CampaignBudgetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @instance
                             */
                            CampaignBudgetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignBudgetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignBudgetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignBudgetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetOperation} CampaignBudgetOperation instance
                             */
                            CampaignBudgetOperation.create = function create(properties) {
                                return new CampaignBudgetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignBudgetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignBudgetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetOperation} m CampaignBudgetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudgetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignBudget.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignBudget.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignBudgetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignBudgetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetOperation} message CampaignBudgetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudgetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignBudgetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetOperation} CampaignBudgetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudgetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignBudgetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignBudget.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignBudget.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignBudgetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetOperation} CampaignBudgetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudgetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignBudgetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignBudgetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBudget.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBudget.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignBudgetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetOperation} CampaignBudgetOperation
                             */
                            CampaignBudgetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignBudgetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignBudgetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignBudgetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignBudgetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignBudget.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignBudgetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignBudget.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignBudgetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignBudgetOperation} m CampaignBudgetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignBudgetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignBudget.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignBudget.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignBudgetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignBudgetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignBudgetOperation;
                        })();
    
                        services.MutateCampaignBudgetsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignBudgetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignBudgetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignBudgetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignBudgetResult>|null} [results] MutateCampaignBudgetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignBudgetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignBudgetsResponse.
                             * @implements IMutateCampaignBudgetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsResponse=} [p] Properties to set
                             */
                            function MutateCampaignBudgetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignBudgetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @instance
                             */
                            MutateCampaignBudgetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignBudgetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignBudgetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @instance
                             */
                            MutateCampaignBudgetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignBudgetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsResponse} MutateCampaignBudgetsResponse instance
                             */
                            MutateCampaignBudgetsResponse.create = function create(properties) {
                                return new MutateCampaignBudgetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBudgetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBudgetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsResponse} m MutateCampaignBudgetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBudgetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBudgetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBudgetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetsResponse} message MutateCampaignBudgetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBudgetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignBudgetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsResponse} MutateCampaignBudgetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBudgetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignBudgetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignBudgetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsResponse} MutateCampaignBudgetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBudgetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignBudgetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignBudgetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignBudgetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetsResponse} MutateCampaignBudgetsResponse
                             */
                            MutateCampaignBudgetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignBudgetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignBudgetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBudgetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBudgetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBudgetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignBudgetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignBudgetsResponse} m MutateCampaignBudgetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignBudgetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignBudgetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignBudgetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignBudgetsResponse;
                        })();
    
                        services.MutateCampaignBudgetResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignBudgetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignBudgetResult
                             * @property {string|null} [resource_name] MutateCampaignBudgetResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignBudget|null} [campaign_budget] MutateCampaignBudgetResult campaign_budget
                             */
    
                            /**
                             * Constructs a new MutateCampaignBudgetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignBudgetResult.
                             * @implements IMutateCampaignBudgetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetResult=} [p] Properties to set
                             */
                            function MutateCampaignBudgetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignBudgetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @instance
                             */
                            MutateCampaignBudgetResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignBudgetResult campaign_budget.
                             * @member {google.ads.googleads.v11.resources.ICampaignBudget|null|undefined} campaign_budget
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @instance
                             */
                            MutateCampaignBudgetResult.prototype.campaign_budget = null;
    
                            /**
                             * Creates a new MutateCampaignBudgetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetResult} MutateCampaignBudgetResult instance
                             */
                            MutateCampaignBudgetResult.create = function create(properties) {
                                return new MutateCampaignBudgetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBudgetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBudgetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetResult} m MutateCampaignBudgetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBudgetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                    $root.google.ads.googleads.v11.resources.CampaignBudget.encode(m.campaign_budget, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignBudgetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignBudgetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignBudgetResult} message MutateCampaignBudgetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignBudgetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignBudgetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetResult} MutateCampaignBudgetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBudgetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignBudgetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetResult} MutateCampaignBudgetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignBudgetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignBudgetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignBudgetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignBudget.verify(m.campaign_budget);
                                        if (e)
                                            return "campaign_budget." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignBudgetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignBudgetResult} MutateCampaignBudgetResult
                             */
                            MutateCampaignBudgetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignBudgetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_budget != null) {
                                    if (typeof d.campaign_budget !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignBudgetResult.campaign_budget: object expected");
                                    m.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.fromObject(d.campaign_budget);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignBudgetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignBudgetResult} m MutateCampaignBudgetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignBudgetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_budget = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    d.campaign_budget = $root.google.ads.googleads.v11.resources.CampaignBudget.toObject(m.campaign_budget, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignBudgetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignBudgetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignBudgetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignBudgetResult;
                        })();
    
                        services.CampaignConversionGoalService = (function() {
    
                            /**
                             * Constructs a new CampaignConversionGoalService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignConversionGoalService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignConversionGoalService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignConversionGoalService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignConversionGoalService;
    
                            /**
                             * Creates new CampaignConversionGoalService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignConversionGoalService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignConversionGoalService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignConversionGoalService|mutateCampaignConversionGoals}.
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalService
                             * @typedef MutateCampaignConversionGoalsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse} [response] MutateCampaignConversionGoalsResponse
                             */
    
                            /**
                             * Calls MutateCampaignConversionGoals.
                             * @function mutateCampaignConversionGoals
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsRequest} request MutateCampaignConversionGoalsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignConversionGoalService.MutateCampaignConversionGoalsCallback} callback Node-style callback called with the error, if any, and MutateCampaignConversionGoalsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignConversionGoalService.prototype.mutateCampaignConversionGoals = function mutateCampaignConversionGoals(request, callback) {
                                return this.rpcCall(mutateCampaignConversionGoals, $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest, $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse, request, callback);
                            }, "name", { value: "MutateCampaignConversionGoals" });
    
                            /**
                             * Calls MutateCampaignConversionGoals.
                             * @function mutateCampaignConversionGoals
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsRequest} request MutateCampaignConversionGoalsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignConversionGoalService;
                        })();
    
                        services.MutateCampaignConversionGoalsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignConversionGoalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignConversionGoalsRequest
                             * @property {string|null} [customer_id] MutateCampaignConversionGoalsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignConversionGoalOperation>|null} [operations] MutateCampaignConversionGoalsRequest operations
                             * @property {boolean|null} [validate_only] MutateCampaignConversionGoalsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCampaignConversionGoalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignConversionGoalsRequest.
                             * @implements IMutateCampaignConversionGoalsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsRequest=} [p] Properties to set
                             */
                            function MutateCampaignConversionGoalsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignConversionGoalsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @instance
                             */
                            MutateCampaignConversionGoalsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignConversionGoalsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignConversionGoalOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @instance
                             */
                            MutateCampaignConversionGoalsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignConversionGoalsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @instance
                             */
                            MutateCampaignConversionGoalsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCampaignConversionGoalsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest} MutateCampaignConversionGoalsRequest instance
                             */
                            MutateCampaignConversionGoalsRequest.create = function create(properties) {
                                return new MutateCampaignConversionGoalsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignConversionGoalsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsRequest} m MutateCampaignConversionGoalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignConversionGoalsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignConversionGoalsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsRequest} message MutateCampaignConversionGoalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignConversionGoalsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignConversionGoalsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest} MutateCampaignConversionGoalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignConversionGoalsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignConversionGoalsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest} MutateCampaignConversionGoalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignConversionGoalsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignConversionGoalsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignConversionGoalsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignConversionGoalsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest} MutateCampaignConversionGoalsRequest
                             */
                            MutateCampaignConversionGoalsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignConversionGoalsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest} m MutateCampaignConversionGoalsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignConversionGoalsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignConversionGoalsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignConversionGoalsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignConversionGoalsRequest;
                        })();
    
                        services.CampaignConversionGoalOperation = (function() {
    
                            /**
                             * Properties of a CampaignConversionGoalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignConversionGoalOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignConversionGoalOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignConversionGoal|null} [update] CampaignConversionGoalOperation update
                             */
    
                            /**
                             * Constructs a new CampaignConversionGoalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignConversionGoalOperation.
                             * @implements ICampaignConversionGoalOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignConversionGoalOperation=} [p] Properties to set
                             */
                            function CampaignConversionGoalOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignConversionGoalOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @instance
                             */
                            CampaignConversionGoalOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignConversionGoalOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignConversionGoal|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @instance
                             */
                            CampaignConversionGoalOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignConversionGoalOperation operation.
                             * @member {"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignConversionGoalOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignConversionGoalOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignConversionGoalOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignConversionGoalOperation} CampaignConversionGoalOperation instance
                             */
                            CampaignConversionGoalOperation.create = function create(properties) {
                                return new CampaignConversionGoalOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignConversionGoalOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignConversionGoalOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignConversionGoalOperation} m CampaignConversionGoalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignConversionGoalOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignConversionGoal.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignConversionGoalOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignConversionGoalOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignConversionGoalOperation} message CampaignConversionGoalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignConversionGoalOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignConversionGoalOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignConversionGoalOperation} CampaignConversionGoalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignConversionGoalOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignConversionGoalOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignConversionGoalOperation} CampaignConversionGoalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignConversionGoalOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignConversionGoalOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignConversionGoalOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignConversionGoalOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignConversionGoalOperation} CampaignConversionGoalOperation
                             */
                            CampaignConversionGoalOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignConversionGoalOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignConversionGoalOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignConversionGoalOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignConversionGoalOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignConversionGoalOperation} m CampaignConversionGoalOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignConversionGoalOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignConversionGoal.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignConversionGoalOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignConversionGoalOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignConversionGoalOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignConversionGoalOperation;
                        })();
    
                        services.MutateCampaignConversionGoalsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignConversionGoalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignConversionGoalsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult>|null} [results] MutateCampaignConversionGoalsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignConversionGoalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignConversionGoalsResponse.
                             * @implements IMutateCampaignConversionGoalsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsResponse=} [p] Properties to set
                             */
                            function MutateCampaignConversionGoalsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignConversionGoalsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @instance
                             */
                            MutateCampaignConversionGoalsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignConversionGoalsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse} MutateCampaignConversionGoalsResponse instance
                             */
                            MutateCampaignConversionGoalsResponse.create = function create(properties) {
                                return new MutateCampaignConversionGoalsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignConversionGoalsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsResponse} m MutateCampaignConversionGoalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignConversionGoalsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignConversionGoalsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalsResponse} message MutateCampaignConversionGoalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignConversionGoalsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignConversionGoalsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse} MutateCampaignConversionGoalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignConversionGoalsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignConversionGoalsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse} MutateCampaignConversionGoalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignConversionGoalsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignConversionGoalsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignConversionGoalsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignConversionGoalsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse} MutateCampaignConversionGoalsResponse
                             */
                            MutateCampaignConversionGoalsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignConversionGoalsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse} m MutateCampaignConversionGoalsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignConversionGoalsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignConversionGoalsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignConversionGoalsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignConversionGoalsResponse;
                        })();
    
                        services.MutateCampaignConversionGoalResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignConversionGoalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignConversionGoalResult
                             * @property {string|null} [resource_name] MutateCampaignConversionGoalResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCampaignConversionGoalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignConversionGoalResult.
                             * @implements IMutateCampaignConversionGoalResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult=} [p] Properties to set
                             */
                            function MutateCampaignConversionGoalResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignConversionGoalResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @instance
                             */
                            MutateCampaignConversionGoalResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCampaignConversionGoalResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalResult} MutateCampaignConversionGoalResult instance
                             */
                            MutateCampaignConversionGoalResult.create = function create(properties) {
                                return new MutateCampaignConversionGoalResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignConversionGoalResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult} m MutateCampaignConversionGoalResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignConversionGoalResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignConversionGoalResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignConversionGoalResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignConversionGoalResult} message MutateCampaignConversionGoalResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignConversionGoalResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignConversionGoalResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalResult} MutateCampaignConversionGoalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignConversionGoalResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignConversionGoalResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalResult} MutateCampaignConversionGoalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignConversionGoalResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignConversionGoalResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignConversionGoalResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignConversionGoalResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignConversionGoalResult} MutateCampaignConversionGoalResult
                             */
                            MutateCampaignConversionGoalResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignConversionGoalResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignConversionGoalResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignConversionGoalResult} m MutateCampaignConversionGoalResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignConversionGoalResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignConversionGoalResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignConversionGoalResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignConversionGoalResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignConversionGoalResult;
                        })();
    
                        services.CampaignCriterionService = (function() {
    
                            /**
                             * Constructs a new CampaignCriterionService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignCriterionService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignCriterionService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignCriterionService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignCriterionService;
    
                            /**
                             * Creates new CampaignCriterionService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignCriterionService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignCriterionService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignCriterionService|mutateCampaignCriteria}.
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionService
                             * @typedef MutateCampaignCriteriaCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignCriteriaResponse} [response] MutateCampaignCriteriaResponse
                             */
    
                            /**
                             * Calls MutateCampaignCriteria.
                             * @function mutateCampaignCriteria
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaRequest} request MutateCampaignCriteriaRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignCriterionService.MutateCampaignCriteriaCallback} callback Node-style callback called with the error, if any, and MutateCampaignCriteriaResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignCriterionService.prototype.mutateCampaignCriteria = function mutateCampaignCriteria(request, callback) {
                                return this.rpcCall(mutateCampaignCriteria, $root.google.ads.googleads.v11.services.MutateCampaignCriteriaRequest, $root.google.ads.googleads.v11.services.MutateCampaignCriteriaResponse, request, callback);
                            }, "name", { value: "MutateCampaignCriteria" });
    
                            /**
                             * Calls MutateCampaignCriteria.
                             * @function mutateCampaignCriteria
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaRequest} request MutateCampaignCriteriaRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignCriteriaResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignCriterionService;
                        })();
    
                        services.MutateCampaignCriteriaRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignCriteriaRequest
                             * @property {string|null} [customer_id] MutateCampaignCriteriaRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignCriterionOperation>|null} [operations] MutateCampaignCriteriaRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignCriteriaRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignCriteriaRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignCriteriaRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignCriteriaRequest.
                             * @implements IMutateCampaignCriteriaRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaRequest=} [p] Properties to set
                             */
                            function MutateCampaignCriteriaRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignCriteriaRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @instance
                             */
                            MutateCampaignCriteriaRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignCriteriaRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignCriterionOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @instance
                             */
                            MutateCampaignCriteriaRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignCriteriaRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @instance
                             */
                            MutateCampaignCriteriaRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignCriteriaRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @instance
                             */
                            MutateCampaignCriteriaRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignCriteriaRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @instance
                             */
                            MutateCampaignCriteriaRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignCriteriaRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaRequest} MutateCampaignCriteriaRequest instance
                             */
                            MutateCampaignCriteriaRequest.create = function create(properties) {
                                return new MutateCampaignCriteriaRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCriteriaRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCriteriaRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaRequest} m MutateCampaignCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCriteriaRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignCriterionOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCriteriaRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCriteriaRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaRequest} message MutateCampaignCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCriteriaRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignCriteriaRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaRequest} MutateCampaignCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCriteriaRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignCriteriaRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignCriterionOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignCriteriaRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaRequest} MutateCampaignCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCriteriaRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignCriteriaRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignCriteriaRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignCriterionOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignCriteriaRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaRequest} MutateCampaignCriteriaRequest
                             */
                            MutateCampaignCriteriaRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignCriteriaRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignCriteriaRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCriteriaRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCriteriaRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignCriterionOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignCriteriaRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignCriteriaRequest} m MutateCampaignCriteriaRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignCriteriaRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignCriterionOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignCriteriaRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignCriteriaRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignCriteriaRequest;
                        })();
    
                        services.CampaignCriterionOperation = (function() {
    
                            /**
                             * Properties of a CampaignCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignCriterionOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignCriterionOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignCriterion|null} [create] CampaignCriterionOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignCriterion|null} [update] CampaignCriterionOperation update
                             * @property {string|null} [remove] CampaignCriterionOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignCriterionOperation.
                             * @implements ICampaignCriterionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignCriterionOperation=} [p] Properties to set
                             */
                            function CampaignCriterionOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignCriterionOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @instance
                             */
                            CampaignCriterionOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignCriterionOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignCriterion|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @instance
                             */
                            CampaignCriterionOperation.prototype.create = null;
    
                            /**
                             * CampaignCriterionOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignCriterion|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @instance
                             */
                            CampaignCriterionOperation.prototype.update = null;
    
                            /**
                             * CampaignCriterionOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @instance
                             */
                            CampaignCriterionOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignCriterionOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignCriterionOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignCriterionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignCriterionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignCriterionOperation} CampaignCriterionOperation instance
                             */
                            CampaignCriterionOperation.create = function create(properties) {
                                return new CampaignCriterionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignCriterionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignCriterionOperation} m CampaignCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignCriterion.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignCriterion.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCriterionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignCriterionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignCriterionOperation} message CampaignCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCriterionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCriterionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignCriterionOperation} CampaignCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignCriterionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignCriterion.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignCriterion.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCriterionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignCriterionOperation} CampaignCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCriterionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCriterionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCriterionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCriterion.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCriterion.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCriterionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignCriterionOperation} CampaignCriterionOperation
                             */
                            CampaignCriterionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignCriterionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignCriterionOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignCriterionOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignCriterionOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignCriterion.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignCriterionOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignCriterion.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignCriterionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignCriterionOperation} m CampaignCriterionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCriterionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignCriterion.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignCriterion.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignCriterionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignCriterionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCriterionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignCriterionOperation;
                        })();
    
                        services.MutateCampaignCriteriaResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignCriteriaResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignCriteriaResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignCriterionResult>|null} [results] MutateCampaignCriteriaResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignCriteriaResponse.
                             * @implements IMutateCampaignCriteriaResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaResponse=} [p] Properties to set
                             */
                            function MutateCampaignCriteriaResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignCriteriaResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @instance
                             */
                            MutateCampaignCriteriaResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignCriteriaResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignCriterionResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @instance
                             */
                            MutateCampaignCriteriaResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignCriteriaResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaResponse} MutateCampaignCriteriaResponse instance
                             */
                            MutateCampaignCriteriaResponse.create = function create(properties) {
                                return new MutateCampaignCriteriaResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCriteriaResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCriteriaResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaResponse} m MutateCampaignCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCriteriaResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCriteriaResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCriteriaResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriteriaResponse} message MutateCampaignCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCriteriaResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignCriteriaResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaResponse} MutateCampaignCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCriteriaResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignCriteriaResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignCriteriaResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaResponse} MutateCampaignCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCriteriaResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignCriteriaResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignCriteriaResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignCriteriaResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriteriaResponse} MutateCampaignCriteriaResponse
                             */
                            MutateCampaignCriteriaResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignCriteriaResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignCriteriaResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCriteriaResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCriteriaResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCriteriaResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignCriteriaResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignCriteriaResponse} m MutateCampaignCriteriaResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignCriteriaResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignCriteriaResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriteriaResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignCriteriaResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignCriteriaResponse;
                        })();
    
                        services.MutateCampaignCriterionResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignCriterionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignCriterionResult
                             * @property {string|null} [resource_name] MutateCampaignCriterionResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignCriterion|null} [campaign_criterion] MutateCampaignCriterionResult campaign_criterion
                             */
    
                            /**
                             * Constructs a new MutateCampaignCriterionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignCriterionResult.
                             * @implements IMutateCampaignCriterionResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriterionResult=} [p] Properties to set
                             */
                            function MutateCampaignCriterionResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignCriterionResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @instance
                             */
                            MutateCampaignCriterionResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignCriterionResult campaign_criterion.
                             * @member {google.ads.googleads.v11.resources.ICampaignCriterion|null|undefined} campaign_criterion
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @instance
                             */
                            MutateCampaignCriterionResult.prototype.campaign_criterion = null;
    
                            /**
                             * Creates a new MutateCampaignCriterionResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriterionResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriterionResult} MutateCampaignCriterionResult instance
                             */
                            MutateCampaignCriterionResult.create = function create(properties) {
                                return new MutateCampaignCriterionResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCriterionResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCriterionResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriterionResult} m MutateCampaignCriterionResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCriterionResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_criterion != null && Object.hasOwnProperty.call(m, "campaign_criterion"))
                                    $root.google.ads.googleads.v11.resources.CampaignCriterion.encode(m.campaign_criterion, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCriterionResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCriterionResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCriterionResult} message MutateCampaignCriterionResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCriterionResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignCriterionResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriterionResult} MutateCampaignCriterionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCriterionResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignCriterionResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriterionResult} MutateCampaignCriterionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCriterionResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignCriterionResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignCriterionResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCriterion.verify(m.campaign_criterion);
                                        if (e)
                                            return "campaign_criterion." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignCriterionResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCriterionResult} MutateCampaignCriterionResult
                             */
                            MutateCampaignCriterionResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignCriterionResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_criterion != null) {
                                    if (typeof d.campaign_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCriterionResult.campaign_criterion: object expected");
                                    m.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.fromObject(d.campaign_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignCriterionResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignCriterionResult} m MutateCampaignCriterionResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignCriterionResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_criterion = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_criterion != null && m.hasOwnProperty("campaign_criterion")) {
                                    d.campaign_criterion = $root.google.ads.googleads.v11.resources.CampaignCriterion.toObject(m.campaign_criterion, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignCriterionResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCriterionResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignCriterionResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignCriterionResult;
                        })();
    
                        services.CampaignCustomizerService = (function() {
    
                            /**
                             * Constructs a new CampaignCustomizerService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignCustomizerService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignCustomizerService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignCustomizerService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignCustomizerService;
    
                            /**
                             * Creates new CampaignCustomizerService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignCustomizerService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignCustomizerService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignCustomizerService|mutateCampaignCustomizers}.
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerService
                             * @typedef MutateCampaignCustomizersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignCustomizersResponse} [response] MutateCampaignCustomizersResponse
                             */
    
                            /**
                             * Calls MutateCampaignCustomizers.
                             * @function mutateCampaignCustomizers
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersRequest} request MutateCampaignCustomizersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignCustomizerService.MutateCampaignCustomizersCallback} callback Node-style callback called with the error, if any, and MutateCampaignCustomizersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignCustomizerService.prototype.mutateCampaignCustomizers = function mutateCampaignCustomizers(request, callback) {
                                return this.rpcCall(mutateCampaignCustomizers, $root.google.ads.googleads.v11.services.MutateCampaignCustomizersRequest, $root.google.ads.googleads.v11.services.MutateCampaignCustomizersResponse, request, callback);
                            }, "name", { value: "MutateCampaignCustomizers" });
    
                            /**
                             * Calls MutateCampaignCustomizers.
                             * @function mutateCampaignCustomizers
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersRequest} request MutateCampaignCustomizersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignCustomizersResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignCustomizerService;
                        })();
    
                        services.MutateCampaignCustomizersRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignCustomizersRequest
                             * @property {string|null} [customer_id] MutateCampaignCustomizersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignCustomizerOperation>|null} [operations] MutateCampaignCustomizersRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignCustomizersRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignCustomizersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignCustomizersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignCustomizersRequest.
                             * @implements IMutateCampaignCustomizersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersRequest=} [p] Properties to set
                             */
                            function MutateCampaignCustomizersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignCustomizersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @instance
                             */
                            MutateCampaignCustomizersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignCustomizersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignCustomizerOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @instance
                             */
                            MutateCampaignCustomizersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignCustomizersRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @instance
                             */
                            MutateCampaignCustomizersRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignCustomizersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @instance
                             */
                            MutateCampaignCustomizersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignCustomizersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @instance
                             */
                            MutateCampaignCustomizersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignCustomizersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersRequest} MutateCampaignCustomizersRequest instance
                             */
                            MutateCampaignCustomizersRequest.create = function create(properties) {
                                return new MutateCampaignCustomizersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCustomizersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCustomizersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersRequest} m MutateCampaignCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCustomizersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCustomizersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCustomizersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersRequest} message MutateCampaignCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCustomizersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignCustomizersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersRequest} MutateCampaignCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCustomizersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignCustomizersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignCustomizerOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignCustomizersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersRequest} MutateCampaignCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCustomizersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignCustomizersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignCustomizersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignCustomizersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersRequest} MutateCampaignCustomizersRequest
                             */
                            MutateCampaignCustomizersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignCustomizersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignCustomizersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCustomizersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCustomizersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignCustomizersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignCustomizersRequest} m MutateCampaignCustomizersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignCustomizersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignCustomizerOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignCustomizersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignCustomizersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignCustomizersRequest;
                        })();
    
                        services.CampaignCustomizerOperation = (function() {
    
                            /**
                             * Properties of a CampaignCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignCustomizerOperation
                             * @property {google.ads.googleads.v11.resources.ICampaignCustomizer|null} [create] CampaignCustomizerOperation create
                             * @property {string|null} [remove] CampaignCustomizerOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignCustomizerOperation.
                             * @implements ICampaignCustomizerOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignCustomizerOperation=} [p] Properties to set
                             */
                            function CampaignCustomizerOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignCustomizerOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignCustomizer|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @instance
                             */
                            CampaignCustomizerOperation.prototype.create = null;
    
                            /**
                             * CampaignCustomizerOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @instance
                             */
                            CampaignCustomizerOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignCustomizerOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignCustomizerOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignCustomizerOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignCustomizerOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignCustomizerOperation} CampaignCustomizerOperation instance
                             */
                            CampaignCustomizerOperation.create = function create(properties) {
                                return new CampaignCustomizerOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignCustomizerOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignCustomizerOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignCustomizerOperation} m CampaignCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCustomizerOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignCustomizer.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignCustomizerOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignCustomizerOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignCustomizerOperation} message CampaignCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignCustomizerOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignCustomizerOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignCustomizerOperation} CampaignCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCustomizerOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignCustomizerOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignCustomizer.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignCustomizerOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignCustomizerOperation} CampaignCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignCustomizerOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignCustomizerOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignCustomizerOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCustomizer.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignCustomizerOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignCustomizerOperation} CampaignCustomizerOperation
                             */
                            CampaignCustomizerOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignCustomizerOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignCustomizerOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignCustomizerOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignCustomizer.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignCustomizerOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignCustomizerOperation} m CampaignCustomizerOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignCustomizerOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignCustomizer.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignCustomizerOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignCustomizerOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignCustomizerOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignCustomizerOperation;
                        })();
    
                        services.MutateCampaignCustomizersResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignCustomizersResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignCustomizerResult>|null} [results] MutateCampaignCustomizersResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignCustomizersResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateCampaignCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignCustomizersResponse.
                             * @implements IMutateCampaignCustomizersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersResponse=} [p] Properties to set
                             */
                            function MutateCampaignCustomizersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignCustomizersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignCustomizerResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @instance
                             */
                            MutateCampaignCustomizersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateCampaignCustomizersResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @instance
                             */
                            MutateCampaignCustomizersResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateCampaignCustomizersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersResponse} MutateCampaignCustomizersResponse instance
                             */
                            MutateCampaignCustomizersResponse.create = function create(properties) {
                                return new MutateCampaignCustomizersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCustomizersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCustomizersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersResponse} m MutateCampaignCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCustomizersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCustomizersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCustomizersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizersResponse} message MutateCampaignCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCustomizersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignCustomizersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersResponse} MutateCampaignCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCustomizersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignCustomizersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignCustomizersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersResponse} MutateCampaignCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCustomizersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignCustomizersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignCustomizersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignCustomizersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizersResponse} MutateCampaignCustomizersResponse
                             */
                            MutateCampaignCustomizersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignCustomizersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignCustomizersResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCustomizersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCustomizersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCustomizersResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignCustomizersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignCustomizersResponse} m MutateCampaignCustomizersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignCustomizersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignCustomizersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignCustomizersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignCustomizersResponse;
                        })();
    
                        services.MutateCampaignCustomizerResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignCustomizerResult
                             * @property {string|null} [resource_name] MutateCampaignCustomizerResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignCustomizer|null} [campaign_customizer] MutateCampaignCustomizerResult campaign_customizer
                             */
    
                            /**
                             * Constructs a new MutateCampaignCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignCustomizerResult.
                             * @implements IMutateCampaignCustomizerResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizerResult=} [p] Properties to set
                             */
                            function MutateCampaignCustomizerResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignCustomizerResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @instance
                             */
                            MutateCampaignCustomizerResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignCustomizerResult campaign_customizer.
                             * @member {google.ads.googleads.v11.resources.ICampaignCustomizer|null|undefined} campaign_customizer
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @instance
                             */
                            MutateCampaignCustomizerResult.prototype.campaign_customizer = null;
    
                            /**
                             * Creates a new MutateCampaignCustomizerResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizerResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizerResult} MutateCampaignCustomizerResult instance
                             */
                            MutateCampaignCustomizerResult.create = function create(properties) {
                                return new MutateCampaignCustomizerResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCustomizerResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCustomizerResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizerResult} m MutateCampaignCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCustomizerResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_customizer != null && Object.hasOwnProperty.call(m, "campaign_customizer"))
                                    $root.google.ads.googleads.v11.resources.CampaignCustomizer.encode(m.campaign_customizer, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignCustomizerResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignCustomizerResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignCustomizerResult} message MutateCampaignCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignCustomizerResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignCustomizerResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizerResult} MutateCampaignCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCustomizerResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_customizer = $root.google.ads.googleads.v11.resources.CampaignCustomizer.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignCustomizerResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizerResult} MutateCampaignCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignCustomizerResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignCustomizerResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignCustomizerResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_customizer != null && m.hasOwnProperty("campaign_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignCustomizer.verify(m.campaign_customizer);
                                        if (e)
                                            return "campaign_customizer." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignCustomizerResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignCustomizerResult} MutateCampaignCustomizerResult
                             */
                            MutateCampaignCustomizerResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignCustomizerResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_customizer != null) {
                                    if (typeof d.campaign_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignCustomizerResult.campaign_customizer: object expected");
                                    m.campaign_customizer = $root.google.ads.googleads.v11.resources.CampaignCustomizer.fromObject(d.campaign_customizer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignCustomizerResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignCustomizerResult} m MutateCampaignCustomizerResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignCustomizerResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_customizer = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_customizer != null && m.hasOwnProperty("campaign_customizer")) {
                                    d.campaign_customizer = $root.google.ads.googleads.v11.resources.CampaignCustomizer.toObject(m.campaign_customizer, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignCustomizerResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignCustomizerResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignCustomizerResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignCustomizerResult;
                        })();
    
                        services.CampaignDraftService = (function() {
    
                            /**
                             * Constructs a new CampaignDraftService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignDraftService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignDraftService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignDraftService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignDraftService;
    
                            /**
                             * Creates new CampaignDraftService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignDraftService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignDraftService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignDraftService|mutateCampaignDrafts}.
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @typedef MutateCampaignDraftsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignDraftsResponse} [response] MutateCampaignDraftsResponse
                             */
    
                            /**
                             * Calls MutateCampaignDrafts.
                             * @function mutateCampaignDrafts
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsRequest} request MutateCampaignDraftsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignDraftService.MutateCampaignDraftsCallback} callback Node-style callback called with the error, if any, and MutateCampaignDraftsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignDraftService.prototype.mutateCampaignDrafts = function mutateCampaignDrafts(request, callback) {
                                return this.rpcCall(mutateCampaignDrafts, $root.google.ads.googleads.v11.services.MutateCampaignDraftsRequest, $root.google.ads.googleads.v11.services.MutateCampaignDraftsResponse, request, callback);
                            }, "name", { value: "MutateCampaignDrafts" });
    
                            /**
                             * Calls MutateCampaignDrafts.
                             * @function mutateCampaignDrafts
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsRequest} request MutateCampaignDraftsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignDraftsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignDraftService|promoteCampaignDraft}.
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @typedef PromoteCampaignDraftCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.longrunning.Operation} [response] Operation
                             */
    
                            /**
                             * Calls PromoteCampaignDraft.
                             * @function promoteCampaignDraft
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignDraftRequest} request PromoteCampaignDraftRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignDraftService.PromoteCampaignDraftCallback} callback Node-style callback called with the error, if any, and Operation
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignDraftService.prototype.promoteCampaignDraft = function promoteCampaignDraft(request, callback) {
                                return this.rpcCall(promoteCampaignDraft, $root.google.ads.googleads.v11.services.PromoteCampaignDraftRequest, $root.google.longrunning.Operation, request, callback);
                            }, "name", { value: "PromoteCampaignDraft" });
    
                            /**
                             * Calls PromoteCampaignDraft.
                             * @function promoteCampaignDraft
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignDraftRequest} request PromoteCampaignDraftRequest message or plain object
                             * @returns {Promise<google.longrunning.Operation>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignDraftService|listCampaignDraftAsyncErrors}.
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @typedef ListCampaignDraftAsyncErrorsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse} [response] ListCampaignDraftAsyncErrorsResponse
                             */
    
                            /**
                             * Calls ListCampaignDraftAsyncErrors.
                             * @function listCampaignDraftAsyncErrors
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsRequest} request ListCampaignDraftAsyncErrorsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignDraftService.ListCampaignDraftAsyncErrorsCallback} callback Node-style callback called with the error, if any, and ListCampaignDraftAsyncErrorsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignDraftService.prototype.listCampaignDraftAsyncErrors = function listCampaignDraftAsyncErrors(request, callback) {
                                return this.rpcCall(listCampaignDraftAsyncErrors, $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest, $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse, request, callback);
                            }, "name", { value: "ListCampaignDraftAsyncErrors" });
    
                            /**
                             * Calls ListCampaignDraftAsyncErrors.
                             * @function listCampaignDraftAsyncErrors
                             * @memberof google.ads.googleads.v11.services.CampaignDraftService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsRequest} request ListCampaignDraftAsyncErrorsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignDraftService;
                        })();
    
                        services.MutateCampaignDraftsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignDraftsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignDraftsRequest
                             * @property {string|null} [customer_id] MutateCampaignDraftsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignDraftOperation>|null} [operations] MutateCampaignDraftsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignDraftsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignDraftsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignDraftsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignDraftsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignDraftsRequest.
                             * @implements IMutateCampaignDraftsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsRequest=} [p] Properties to set
                             */
                            function MutateCampaignDraftsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignDraftsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @instance
                             */
                            MutateCampaignDraftsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignDraftsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignDraftOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @instance
                             */
                            MutateCampaignDraftsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignDraftsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @instance
                             */
                            MutateCampaignDraftsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignDraftsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @instance
                             */
                            MutateCampaignDraftsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignDraftsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @instance
                             */
                            MutateCampaignDraftsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignDraftsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsRequest} MutateCampaignDraftsRequest instance
                             */
                            MutateCampaignDraftsRequest.create = function create(properties) {
                                return new MutateCampaignDraftsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignDraftsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignDraftsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsRequest} m MutateCampaignDraftsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignDraftsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignDraftOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignDraftsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignDraftsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsRequest} message MutateCampaignDraftsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignDraftsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignDraftsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsRequest} MutateCampaignDraftsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignDraftsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignDraftsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignDraftOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignDraftsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsRequest} MutateCampaignDraftsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignDraftsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignDraftsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignDraftsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignDraftOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignDraftsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsRequest} MutateCampaignDraftsRequest
                             */
                            MutateCampaignDraftsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignDraftsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignDraftsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignDraftsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignDraftsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignDraftOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignDraftsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignDraftsRequest} m MutateCampaignDraftsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignDraftsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignDraftOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignDraftsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignDraftsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignDraftsRequest;
                        })();
    
                        services.PromoteCampaignDraftRequest = (function() {
    
                            /**
                             * Properties of a PromoteCampaignDraftRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPromoteCampaignDraftRequest
                             * @property {string|null} [campaign_draft] PromoteCampaignDraftRequest campaign_draft
                             * @property {boolean|null} [validate_only] PromoteCampaignDraftRequest validate_only
                             */
    
                            /**
                             * Constructs a new PromoteCampaignDraftRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PromoteCampaignDraftRequest.
                             * @implements IPromoteCampaignDraftRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignDraftRequest=} [p] Properties to set
                             */
                            function PromoteCampaignDraftRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PromoteCampaignDraftRequest campaign_draft.
                             * @member {string} campaign_draft
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @instance
                             */
                            PromoteCampaignDraftRequest.prototype.campaign_draft = "";
    
                            /**
                             * PromoteCampaignDraftRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @instance
                             */
                            PromoteCampaignDraftRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new PromoteCampaignDraftRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignDraftRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignDraftRequest} PromoteCampaignDraftRequest instance
                             */
                            PromoteCampaignDraftRequest.create = function create(properties) {
                                return new PromoteCampaignDraftRequest(properties);
                            };
    
                            /**
                             * Encodes the specified PromoteCampaignDraftRequest message. Does not implicitly {@link google.ads.googleads.v11.services.PromoteCampaignDraftRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignDraftRequest} m PromoteCampaignDraftRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteCampaignDraftRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_draft != null && Object.hasOwnProperty.call(m, "campaign_draft"))
                                    w.uint32(10).string(m.campaign_draft);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(16).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromoteCampaignDraftRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PromoteCampaignDraftRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignDraftRequest} message PromoteCampaignDraftRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteCampaignDraftRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromoteCampaignDraftRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignDraftRequest} PromoteCampaignDraftRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteCampaignDraftRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PromoteCampaignDraftRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.campaign_draft = r.string();
                                        break;
                                    case 2:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromoteCampaignDraftRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignDraftRequest} PromoteCampaignDraftRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteCampaignDraftRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromoteCampaignDraftRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromoteCampaignDraftRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    if (!$util.isString(m.campaign_draft))
                                        return "campaign_draft: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PromoteCampaignDraftRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignDraftRequest} PromoteCampaignDraftRequest
                             */
                            PromoteCampaignDraftRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PromoteCampaignDraftRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PromoteCampaignDraftRequest();
                                if (d.campaign_draft != null) {
                                    m.campaign_draft = String(d.campaign_draft);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PromoteCampaignDraftRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.PromoteCampaignDraftRequest} m PromoteCampaignDraftRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromoteCampaignDraftRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.campaign_draft = "";
                                    d.validate_only = false;
                                }
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    d.campaign_draft = m.campaign_draft;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PromoteCampaignDraftRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignDraftRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromoteCampaignDraftRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PromoteCampaignDraftRequest;
                        })();
    
                        services.CampaignDraftOperation = (function() {
    
                            /**
                             * Properties of a CampaignDraftOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignDraftOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignDraftOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignDraft|null} [create] CampaignDraftOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignDraft|null} [update] CampaignDraftOperation update
                             * @property {string|null} [remove] CampaignDraftOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignDraftOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignDraftOperation.
                             * @implements ICampaignDraftOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignDraftOperation=} [p] Properties to set
                             */
                            function CampaignDraftOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignDraftOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @instance
                             */
                            CampaignDraftOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignDraftOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignDraft|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @instance
                             */
                            CampaignDraftOperation.prototype.create = null;
    
                            /**
                             * CampaignDraftOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignDraft|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @instance
                             */
                            CampaignDraftOperation.prototype.update = null;
    
                            /**
                             * CampaignDraftOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @instance
                             */
                            CampaignDraftOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignDraftOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignDraftOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignDraftOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignDraftOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignDraftOperation} CampaignDraftOperation instance
                             */
                            CampaignDraftOperation.create = function create(properties) {
                                return new CampaignDraftOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignDraftOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignDraftOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignDraftOperation} m CampaignDraftOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraftOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignDraft.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignDraft.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignDraftOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignDraftOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignDraftOperation} message CampaignDraftOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDraftOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignDraftOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignDraftOperation} CampaignDraftOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraftOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignDraftOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignDraft.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignDraft.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignDraftOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignDraftOperation} CampaignDraftOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDraftOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignDraftOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignDraftOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignDraft.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignDraft.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignDraftOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignDraftOperation} CampaignDraftOperation
                             */
                            CampaignDraftOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignDraftOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignDraftOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignDraftOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignDraftOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignDraft.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignDraftOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignDraft.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignDraftOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignDraftOperation} m CampaignDraftOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignDraftOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignDraft.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignDraft.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignDraftOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignDraftOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignDraftOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignDraftOperation;
                        })();
    
                        services.MutateCampaignDraftsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignDraftsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignDraftsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignDraftsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignDraftResult>|null} [results] MutateCampaignDraftsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignDraftsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignDraftsResponse.
                             * @implements IMutateCampaignDraftsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsResponse=} [p] Properties to set
                             */
                            function MutateCampaignDraftsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignDraftsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @instance
                             */
                            MutateCampaignDraftsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignDraftsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignDraftResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @instance
                             */
                            MutateCampaignDraftsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignDraftsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsResponse} MutateCampaignDraftsResponse instance
                             */
                            MutateCampaignDraftsResponse.create = function create(properties) {
                                return new MutateCampaignDraftsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignDraftsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignDraftsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsResponse} m MutateCampaignDraftsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignDraftsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignDraftsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignDraftsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftsResponse} message MutateCampaignDraftsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignDraftsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignDraftsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsResponse} MutateCampaignDraftsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignDraftsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignDraftsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignDraftResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignDraftsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsResponse} MutateCampaignDraftsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignDraftsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignDraftsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignDraftsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignDraftsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftsResponse} MutateCampaignDraftsResponse
                             */
                            MutateCampaignDraftsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignDraftsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignDraftsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignDraftsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignDraftsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignDraftsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignDraftsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignDraftsResponse} m MutateCampaignDraftsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignDraftsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignDraftResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignDraftsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignDraftsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignDraftsResponse;
                        })();
    
                        services.MutateCampaignDraftResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignDraftResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignDraftResult
                             * @property {string|null} [resource_name] MutateCampaignDraftResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignDraft|null} [campaign_draft] MutateCampaignDraftResult campaign_draft
                             */
    
                            /**
                             * Constructs a new MutateCampaignDraftResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignDraftResult.
                             * @implements IMutateCampaignDraftResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftResult=} [p] Properties to set
                             */
                            function MutateCampaignDraftResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignDraftResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @instance
                             */
                            MutateCampaignDraftResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignDraftResult campaign_draft.
                             * @member {google.ads.googleads.v11.resources.ICampaignDraft|null|undefined} campaign_draft
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @instance
                             */
                            MutateCampaignDraftResult.prototype.campaign_draft = null;
    
                            /**
                             * Creates a new MutateCampaignDraftResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftResult} MutateCampaignDraftResult instance
                             */
                            MutateCampaignDraftResult.create = function create(properties) {
                                return new MutateCampaignDraftResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignDraftResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignDraftResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftResult} m MutateCampaignDraftResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignDraftResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_draft != null && Object.hasOwnProperty.call(m, "campaign_draft"))
                                    $root.google.ads.googleads.v11.resources.CampaignDraft.encode(m.campaign_draft, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignDraftResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignDraftResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignDraftResult} message MutateCampaignDraftResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignDraftResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignDraftResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftResult} MutateCampaignDraftResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignDraftResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignDraftResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_draft = $root.google.ads.googleads.v11.resources.CampaignDraft.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignDraftResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftResult} MutateCampaignDraftResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignDraftResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignDraftResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignDraftResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignDraft.verify(m.campaign_draft);
                                        if (e)
                                            return "campaign_draft." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignDraftResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignDraftResult} MutateCampaignDraftResult
                             */
                            MutateCampaignDraftResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignDraftResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignDraftResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_draft != null) {
                                    if (typeof d.campaign_draft !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignDraftResult.campaign_draft: object expected");
                                    m.campaign_draft = $root.google.ads.googleads.v11.resources.CampaignDraft.fromObject(d.campaign_draft);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignDraftResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignDraftResult} m MutateCampaignDraftResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignDraftResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_draft = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_draft != null && m.hasOwnProperty("campaign_draft")) {
                                    d.campaign_draft = $root.google.ads.googleads.v11.resources.CampaignDraft.toObject(m.campaign_draft, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignDraftResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignDraftResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignDraftResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignDraftResult;
                        })();
    
                        services.ListCampaignDraftAsyncErrorsRequest = (function() {
    
                            /**
                             * Properties of a ListCampaignDraftAsyncErrorsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListCampaignDraftAsyncErrorsRequest
                             * @property {string|null} [resource_name] ListCampaignDraftAsyncErrorsRequest resource_name
                             * @property {string|null} [page_token] ListCampaignDraftAsyncErrorsRequest page_token
                             * @property {number|null} [page_size] ListCampaignDraftAsyncErrorsRequest page_size
                             */
    
                            /**
                             * Constructs a new ListCampaignDraftAsyncErrorsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListCampaignDraftAsyncErrorsRequest.
                             * @implements IListCampaignDraftAsyncErrorsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsRequest=} [p] Properties to set
                             */
                            function ListCampaignDraftAsyncErrorsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListCampaignDraftAsyncErrorsRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @instance
                             */
                            ListCampaignDraftAsyncErrorsRequest.prototype.resource_name = "";
    
                            /**
                             * ListCampaignDraftAsyncErrorsRequest page_token.
                             * @member {string} page_token
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @instance
                             */
                            ListCampaignDraftAsyncErrorsRequest.prototype.page_token = "";
    
                            /**
                             * ListCampaignDraftAsyncErrorsRequest page_size.
                             * @member {number} page_size
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @instance
                             */
                            ListCampaignDraftAsyncErrorsRequest.prototype.page_size = 0;
    
                            /**
                             * Creates a new ListCampaignDraftAsyncErrorsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest} ListCampaignDraftAsyncErrorsRequest instance
                             */
                            ListCampaignDraftAsyncErrorsRequest.create = function create(properties) {
                                return new ListCampaignDraftAsyncErrorsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListCampaignDraftAsyncErrorsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsRequest} m ListCampaignDraftAsyncErrorsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignDraftAsyncErrorsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                                    w.uint32(18).string(m.page_token);
                                if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                                    w.uint32(24).int32(m.page_size);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListCampaignDraftAsyncErrorsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsRequest} message ListCampaignDraftAsyncErrorsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignDraftAsyncErrorsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListCampaignDraftAsyncErrorsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest} ListCampaignDraftAsyncErrorsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignDraftAsyncErrorsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.page_token = r.string();
                                        break;
                                    case 3:
                                        m.page_size = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListCampaignDraftAsyncErrorsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest} ListCampaignDraftAsyncErrorsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignDraftAsyncErrorsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListCampaignDraftAsyncErrorsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListCampaignDraftAsyncErrorsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    if (!$util.isString(m.page_token))
                                        return "page_token: string expected";
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    if (!$util.isInteger(m.page_size))
                                        return "page_size: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListCampaignDraftAsyncErrorsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest} ListCampaignDraftAsyncErrorsRequest
                             */
                            ListCampaignDraftAsyncErrorsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.page_token != null) {
                                    m.page_token = String(d.page_token);
                                }
                                if (d.page_size != null) {
                                    m.page_size = d.page_size | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListCampaignDraftAsyncErrorsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest} m ListCampaignDraftAsyncErrorsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListCampaignDraftAsyncErrorsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.page_token = "";
                                    d.page_size = 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    d.page_token = m.page_token;
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    d.page_size = m.page_size;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListCampaignDraftAsyncErrorsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListCampaignDraftAsyncErrorsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListCampaignDraftAsyncErrorsRequest;
                        })();
    
                        services.ListCampaignDraftAsyncErrorsResponse = (function() {
    
                            /**
                             * Properties of a ListCampaignDraftAsyncErrorsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListCampaignDraftAsyncErrorsResponse
                             * @property {Array.<google.rpc.IStatus>|null} [errors] ListCampaignDraftAsyncErrorsResponse errors
                             * @property {string|null} [next_page_token] ListCampaignDraftAsyncErrorsResponse next_page_token
                             */
    
                            /**
                             * Constructs a new ListCampaignDraftAsyncErrorsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListCampaignDraftAsyncErrorsResponse.
                             * @implements IListCampaignDraftAsyncErrorsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsResponse=} [p] Properties to set
                             */
                            function ListCampaignDraftAsyncErrorsResponse(p) {
                                this.errors = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListCampaignDraftAsyncErrorsResponse errors.
                             * @member {Array.<google.rpc.IStatus>} errors
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @instance
                             */
                            ListCampaignDraftAsyncErrorsResponse.prototype.errors = $util.emptyArray;
    
                            /**
                             * ListCampaignDraftAsyncErrorsResponse next_page_token.
                             * @member {string} next_page_token
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @instance
                             */
                            ListCampaignDraftAsyncErrorsResponse.prototype.next_page_token = "";
    
                            /**
                             * Creates a new ListCampaignDraftAsyncErrorsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse} ListCampaignDraftAsyncErrorsResponse instance
                             */
                            ListCampaignDraftAsyncErrorsResponse.create = function create(properties) {
                                return new ListCampaignDraftAsyncErrorsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListCampaignDraftAsyncErrorsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsResponse} m ListCampaignDraftAsyncErrorsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignDraftAsyncErrorsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.errors != null && m.errors.length) {
                                    for (var i = 0; i < m.errors.length; ++i)
                                        $root.google.rpc.Status.encode(m.errors[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                                    w.uint32(18).string(m.next_page_token);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListCampaignDraftAsyncErrorsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignDraftAsyncErrorsResponse} message ListCampaignDraftAsyncErrorsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignDraftAsyncErrorsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListCampaignDraftAsyncErrorsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse} ListCampaignDraftAsyncErrorsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignDraftAsyncErrorsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.errors && m.errors.length))
                                            m.errors = [];
                                        m.errors.push($root.google.rpc.Status.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.next_page_token = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListCampaignDraftAsyncErrorsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse} ListCampaignDraftAsyncErrorsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignDraftAsyncErrorsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListCampaignDraftAsyncErrorsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListCampaignDraftAsyncErrorsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.errors != null && m.hasOwnProperty("errors")) {
                                    if (!Array.isArray(m.errors))
                                        return "errors: array expected";
                                    for (var i = 0; i < m.errors.length; ++i) {
                                        {
                                            var e = $root.google.rpc.Status.verify(m.errors[i]);
                                            if (e)
                                                return "errors." + e;
                                        }
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    if (!$util.isString(m.next_page_token))
                                        return "next_page_token: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListCampaignDraftAsyncErrorsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse} ListCampaignDraftAsyncErrorsResponse
                             */
                            ListCampaignDraftAsyncErrorsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse();
                                if (d.errors) {
                                    if (!Array.isArray(d.errors))
                                        throw TypeError(".google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse.errors: array expected");
                                    m.errors = [];
                                    for (var i = 0; i < d.errors.length; ++i) {
                                        if (typeof d.errors[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse.errors: object expected");
                                        m.errors[i] = $root.google.rpc.Status.fromObject(d.errors[i]);
                                    }
                                }
                                if (d.next_page_token != null) {
                                    m.next_page_token = String(d.next_page_token);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListCampaignDraftAsyncErrorsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse} m ListCampaignDraftAsyncErrorsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListCampaignDraftAsyncErrorsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.errors = [];
                                }
                                if (o.defaults) {
                                    d.next_page_token = "";
                                }
                                if (m.errors && m.errors.length) {
                                    d.errors = [];
                                    for (var j = 0; j < m.errors.length; ++j) {
                                        d.errors[j] = $root.google.rpc.Status.toObject(m.errors[j], o);
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    d.next_page_token = m.next_page_token;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListCampaignDraftAsyncErrorsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListCampaignDraftAsyncErrorsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListCampaignDraftAsyncErrorsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListCampaignDraftAsyncErrorsResponse;
                        })();
    
                        services.CampaignExperimentService = (function() {
    
                            /**
                             * Constructs a new CampaignExperimentService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignExperimentService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignExperimentService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignExperimentService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignExperimentService;
    
                            /**
                             * Creates new CampaignExperimentService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignExperimentService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignExperimentService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignExperimentService|createCampaignExperiment}.
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @typedef CreateCampaignExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.longrunning.Operation} [response] Operation
                             */
    
                            /**
                             * Calls CreateCampaignExperiment.
                             * @function createCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentRequest} request CreateCampaignExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignExperimentService.CreateCampaignExperimentCallback} callback Node-style callback called with the error, if any, and Operation
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignExperimentService.prototype.createCampaignExperiment = function createCampaignExperiment(request, callback) {
                                return this.rpcCall(createCampaignExperiment, $root.google.ads.googleads.v11.services.CreateCampaignExperimentRequest, $root.google.longrunning.Operation, request, callback);
                            }, "name", { value: "CreateCampaignExperiment" });
    
                            /**
                             * Calls CreateCampaignExperiment.
                             * @function createCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentRequest} request CreateCampaignExperimentRequest message or plain object
                             * @returns {Promise<google.longrunning.Operation>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignExperimentService|mutateCampaignExperiments}.
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @typedef MutateCampaignExperimentsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignExperimentsResponse} [response] MutateCampaignExperimentsResponse
                             */
    
                            /**
                             * Calls MutateCampaignExperiments.
                             * @function mutateCampaignExperiments
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsRequest} request MutateCampaignExperimentsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignExperimentService.MutateCampaignExperimentsCallback} callback Node-style callback called with the error, if any, and MutateCampaignExperimentsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignExperimentService.prototype.mutateCampaignExperiments = function mutateCampaignExperiments(request, callback) {
                                return this.rpcCall(mutateCampaignExperiments, $root.google.ads.googleads.v11.services.MutateCampaignExperimentsRequest, $root.google.ads.googleads.v11.services.MutateCampaignExperimentsResponse, request, callback);
                            }, "name", { value: "MutateCampaignExperiments" });
    
                            /**
                             * Calls MutateCampaignExperiments.
                             * @function mutateCampaignExperiments
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsRequest} request MutateCampaignExperimentsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignExperimentsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignExperimentService|graduateCampaignExperiment}.
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @typedef GraduateCampaignExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GraduateCampaignExperimentResponse} [response] GraduateCampaignExperimentResponse
                             */
    
                            /**
                             * Calls GraduateCampaignExperiment.
                             * @function graduateCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentRequest} request GraduateCampaignExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignExperimentService.GraduateCampaignExperimentCallback} callback Node-style callback called with the error, if any, and GraduateCampaignExperimentResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignExperimentService.prototype.graduateCampaignExperiment = function graduateCampaignExperiment(request, callback) {
                                return this.rpcCall(graduateCampaignExperiment, $root.google.ads.googleads.v11.services.GraduateCampaignExperimentRequest, $root.google.ads.googleads.v11.services.GraduateCampaignExperimentResponse, request, callback);
                            }, "name", { value: "GraduateCampaignExperiment" });
    
                            /**
                             * Calls GraduateCampaignExperiment.
                             * @function graduateCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentRequest} request GraduateCampaignExperimentRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GraduateCampaignExperimentResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignExperimentService|promoteCampaignExperiment}.
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @typedef PromoteCampaignExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.longrunning.Operation} [response] Operation
                             */
    
                            /**
                             * Calls PromoteCampaignExperiment.
                             * @function promoteCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignExperimentRequest} request PromoteCampaignExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignExperimentService.PromoteCampaignExperimentCallback} callback Node-style callback called with the error, if any, and Operation
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignExperimentService.prototype.promoteCampaignExperiment = function promoteCampaignExperiment(request, callback) {
                                return this.rpcCall(promoteCampaignExperiment, $root.google.ads.googleads.v11.services.PromoteCampaignExperimentRequest, $root.google.longrunning.Operation, request, callback);
                            }, "name", { value: "PromoteCampaignExperiment" });
    
                            /**
                             * Calls PromoteCampaignExperiment.
                             * @function promoteCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignExperimentRequest} request PromoteCampaignExperimentRequest message or plain object
                             * @returns {Promise<google.longrunning.Operation>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignExperimentService|endCampaignExperiment}.
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @typedef EndCampaignExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.protobuf.Empty} [response] Empty
                             */
    
                            /**
                             * Calls EndCampaignExperiment.
                             * @function endCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IEndCampaignExperimentRequest} request EndCampaignExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignExperimentService.EndCampaignExperimentCallback} callback Node-style callback called with the error, if any, and Empty
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignExperimentService.prototype.endCampaignExperiment = function endCampaignExperiment(request, callback) {
                                return this.rpcCall(endCampaignExperiment, $root.google.ads.googleads.v11.services.EndCampaignExperimentRequest, $root.google.protobuf.Empty, request, callback);
                            }, "name", { value: "EndCampaignExperiment" });
    
                            /**
                             * Calls EndCampaignExperiment.
                             * @function endCampaignExperiment
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IEndCampaignExperimentRequest} request EndCampaignExperimentRequest message or plain object
                             * @returns {Promise<google.protobuf.Empty>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignExperimentService|listCampaignExperimentAsyncErrors}.
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @typedef ListCampaignExperimentAsyncErrorsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse} [response] ListCampaignExperimentAsyncErrorsResponse
                             */
    
                            /**
                             * Calls ListCampaignExperimentAsyncErrors.
                             * @function listCampaignExperimentAsyncErrors
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsRequest} request ListCampaignExperimentAsyncErrorsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignExperimentService.ListCampaignExperimentAsyncErrorsCallback} callback Node-style callback called with the error, if any, and ListCampaignExperimentAsyncErrorsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignExperimentService.prototype.listCampaignExperimentAsyncErrors = function listCampaignExperimentAsyncErrors(request, callback) {
                                return this.rpcCall(listCampaignExperimentAsyncErrors, $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest, $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse, request, callback);
                            }, "name", { value: "ListCampaignExperimentAsyncErrors" });
    
                            /**
                             * Calls ListCampaignExperimentAsyncErrors.
                             * @function listCampaignExperimentAsyncErrors
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsRequest} request ListCampaignExperimentAsyncErrorsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignExperimentService;
                        })();
    
                        services.MutateCampaignExperimentsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignExperimentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignExperimentsRequest
                             * @property {string|null} [customer_id] MutateCampaignExperimentsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignExperimentOperation>|null} [operations] MutateCampaignExperimentsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignExperimentsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignExperimentsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignExperimentsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignExperimentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignExperimentsRequest.
                             * @implements IMutateCampaignExperimentsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsRequest=} [p] Properties to set
                             */
                            function MutateCampaignExperimentsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignExperimentsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @instance
                             */
                            MutateCampaignExperimentsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignExperimentsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignExperimentOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @instance
                             */
                            MutateCampaignExperimentsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignExperimentsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @instance
                             */
                            MutateCampaignExperimentsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignExperimentsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @instance
                             */
                            MutateCampaignExperimentsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignExperimentsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @instance
                             */
                            MutateCampaignExperimentsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignExperimentsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsRequest} MutateCampaignExperimentsRequest instance
                             */
                            MutateCampaignExperimentsRequest.create = function create(properties) {
                                return new MutateCampaignExperimentsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExperimentsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExperimentsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsRequest} m MutateCampaignExperimentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExperimentsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignExperimentOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExperimentsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExperimentsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsRequest} message MutateCampaignExperimentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExperimentsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignExperimentsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsRequest} MutateCampaignExperimentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExperimentsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignExperimentsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignExperimentOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignExperimentsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsRequest} MutateCampaignExperimentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExperimentsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignExperimentsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignExperimentsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignExperimentOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignExperimentsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsRequest} MutateCampaignExperimentsRequest
                             */
                            MutateCampaignExperimentsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignExperimentsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignExperimentsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExperimentsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExperimentsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignExperimentOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignExperimentsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignExperimentsRequest} m MutateCampaignExperimentsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignExperimentsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignExperimentOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignExperimentsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignExperimentsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignExperimentsRequest;
                        })();
    
                        services.CampaignExperimentOperation = (function() {
    
                            /**
                             * Properties of a CampaignExperimentOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignExperimentOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignExperimentOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignExperiment|null} [update] CampaignExperimentOperation update
                             * @property {string|null} [remove] CampaignExperimentOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignExperimentOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignExperimentOperation.
                             * @implements ICampaignExperimentOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignExperimentOperation=} [p] Properties to set
                             */
                            function CampaignExperimentOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignExperimentOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @instance
                             */
                            CampaignExperimentOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignExperimentOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignExperiment|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @instance
                             */
                            CampaignExperimentOperation.prototype.update = null;
    
                            /**
                             * CampaignExperimentOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @instance
                             */
                            CampaignExperimentOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignExperimentOperation operation.
                             * @member {"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignExperimentOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignExperimentOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignExperimentOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignExperimentOperation} CampaignExperimentOperation instance
                             */
                            CampaignExperimentOperation.create = function create(properties) {
                                return new CampaignExperimentOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignExperimentOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignExperimentOperation} m CampaignExperimentOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignExperiment.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExperimentOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignExperimentOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignExperimentOperation} message CampaignExperimentOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExperimentOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExperimentOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignExperimentOperation} CampaignExperimentOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignExperimentOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignExperiment.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExperimentOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignExperimentOperation} CampaignExperimentOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExperimentOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExperimentOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExperimentOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExperiment.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExperimentOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignExperimentOperation} CampaignExperimentOperation
                             */
                            CampaignExperimentOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignExperimentOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignExperimentOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignExperimentOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignExperimentOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignExperiment.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignExperimentOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignExperimentOperation} m CampaignExperimentOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExperimentOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignExperiment.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignExperimentOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignExperimentOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExperimentOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignExperimentOperation;
                        })();
    
                        services.MutateCampaignExperimentsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignExperimentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignExperimentsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignExperimentsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignExperimentResult>|null} [results] MutateCampaignExperimentsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignExperimentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignExperimentsResponse.
                             * @implements IMutateCampaignExperimentsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsResponse=} [p] Properties to set
                             */
                            function MutateCampaignExperimentsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignExperimentsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @instance
                             */
                            MutateCampaignExperimentsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignExperimentsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignExperimentResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @instance
                             */
                            MutateCampaignExperimentsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignExperimentsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsResponse} MutateCampaignExperimentsResponse instance
                             */
                            MutateCampaignExperimentsResponse.create = function create(properties) {
                                return new MutateCampaignExperimentsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExperimentsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExperimentsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsResponse} m MutateCampaignExperimentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExperimentsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExperimentsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExperimentsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentsResponse} message MutateCampaignExperimentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExperimentsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignExperimentsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsResponse} MutateCampaignExperimentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExperimentsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignExperimentsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignExperimentsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsResponse} MutateCampaignExperimentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExperimentsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignExperimentsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignExperimentsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignExperimentsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentsResponse} MutateCampaignExperimentsResponse
                             */
                            MutateCampaignExperimentsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignExperimentsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignExperimentsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExperimentsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExperimentsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExperimentsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignExperimentsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignExperimentsResponse} m MutateCampaignExperimentsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignExperimentsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignExperimentsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignExperimentsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignExperimentsResponse;
                        })();
    
                        services.MutateCampaignExperimentResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignExperimentResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignExperimentResult
                             * @property {string|null} [resource_name] MutateCampaignExperimentResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignExperiment|null} [campaign_experiment] MutateCampaignExperimentResult campaign_experiment
                             */
    
                            /**
                             * Constructs a new MutateCampaignExperimentResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignExperimentResult.
                             * @implements IMutateCampaignExperimentResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentResult=} [p] Properties to set
                             */
                            function MutateCampaignExperimentResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignExperimentResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @instance
                             */
                            MutateCampaignExperimentResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignExperimentResult campaign_experiment.
                             * @member {google.ads.googleads.v11.resources.ICampaignExperiment|null|undefined} campaign_experiment
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @instance
                             */
                            MutateCampaignExperimentResult.prototype.campaign_experiment = null;
    
                            /**
                             * Creates a new MutateCampaignExperimentResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentResult} MutateCampaignExperimentResult instance
                             */
                            MutateCampaignExperimentResult.create = function create(properties) {
                                return new MutateCampaignExperimentResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExperimentResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExperimentResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentResult} m MutateCampaignExperimentResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExperimentResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_experiment != null && Object.hasOwnProperty.call(m, "campaign_experiment"))
                                    $root.google.ads.googleads.v11.resources.CampaignExperiment.encode(m.campaign_experiment, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExperimentResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExperimentResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExperimentResult} message MutateCampaignExperimentResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExperimentResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignExperimentResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentResult} MutateCampaignExperimentResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExperimentResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignExperimentResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentResult} MutateCampaignExperimentResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExperimentResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignExperimentResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignExperimentResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExperiment.verify(m.campaign_experiment);
                                        if (e)
                                            return "campaign_experiment." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignExperimentResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExperimentResult} MutateCampaignExperimentResult
                             */
                            MutateCampaignExperimentResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignExperimentResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_experiment != null) {
                                    if (typeof d.campaign_experiment !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExperimentResult.campaign_experiment: object expected");
                                    m.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.fromObject(d.campaign_experiment);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignExperimentResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignExperimentResult} m MutateCampaignExperimentResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignExperimentResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_experiment = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    d.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.toObject(m.campaign_experiment, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignExperimentResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExperimentResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignExperimentResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignExperimentResult;
                        })();
    
                        services.CreateCampaignExperimentRequest = (function() {
    
                            /**
                             * Properties of a CreateCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateCampaignExperimentRequest
                             * @property {string|null} [customer_id] CreateCampaignExperimentRequest customer_id
                             * @property {google.ads.googleads.v11.resources.ICampaignExperiment|null} [campaign_experiment] CreateCampaignExperimentRequest campaign_experiment
                             * @property {boolean|null} [validate_only] CreateCampaignExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new CreateCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateCampaignExperimentRequest.
                             * @implements ICreateCampaignExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentRequest=} [p] Properties to set
                             */
                            function CreateCampaignExperimentRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateCampaignExperimentRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @instance
                             */
                            CreateCampaignExperimentRequest.prototype.customer_id = "";
    
                            /**
                             * CreateCampaignExperimentRequest campaign_experiment.
                             * @member {google.ads.googleads.v11.resources.ICampaignExperiment|null|undefined} campaign_experiment
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @instance
                             */
                            CreateCampaignExperimentRequest.prototype.campaign_experiment = null;
    
                            /**
                             * CreateCampaignExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @instance
                             */
                            CreateCampaignExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new CreateCampaignExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentRequest} CreateCampaignExperimentRequest instance
                             */
                            CreateCampaignExperimentRequest.create = function create(properties) {
                                return new CreateCampaignExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified CreateCampaignExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.CreateCampaignExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentRequest} m CreateCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCampaignExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.campaign_experiment != null && Object.hasOwnProperty.call(m, "campaign_experiment"))
                                    $root.google.ads.googleads.v11.resources.CampaignExperiment.encode(m.campaign_experiment, w.uint32(18).fork()).ldelim();
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateCampaignExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateCampaignExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentRequest} message CreateCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCampaignExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateCampaignExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentRequest} CreateCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCampaignExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateCampaignExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateCampaignExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentRequest} CreateCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCampaignExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateCampaignExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateCampaignExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExperiment.verify(m.campaign_experiment);
                                        if (e)
                                            return "campaign_experiment." + e;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateCampaignExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentRequest} CreateCampaignExperimentRequest
                             */
                            CreateCampaignExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateCampaignExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateCampaignExperimentRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.campaign_experiment != null) {
                                    if (typeof d.campaign_experiment !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CreateCampaignExperimentRequest.campaign_experiment: object expected");
                                    m.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.fromObject(d.campaign_experiment);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateCampaignExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateCampaignExperimentRequest} m CreateCampaignExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateCampaignExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.campaign_experiment = null;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    d.campaign_experiment = $root.google.ads.googleads.v11.resources.CampaignExperiment.toObject(m.campaign_experiment, o);
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateCampaignExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateCampaignExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateCampaignExperimentRequest;
                        })();
    
                        services.CreateCampaignExperimentMetadata = (function() {
    
                            /**
                             * Properties of a CreateCampaignExperimentMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateCampaignExperimentMetadata
                             * @property {string|null} [campaign_experiment] CreateCampaignExperimentMetadata campaign_experiment
                             */
    
                            /**
                             * Constructs a new CreateCampaignExperimentMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateCampaignExperimentMetadata.
                             * @implements ICreateCampaignExperimentMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentMetadata=} [p] Properties to set
                             */
                            function CreateCampaignExperimentMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateCampaignExperimentMetadata campaign_experiment.
                             * @member {string} campaign_experiment
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @instance
                             */
                            CreateCampaignExperimentMetadata.prototype.campaign_experiment = "";
    
                            /**
                             * Creates a new CreateCampaignExperimentMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentMetadata} CreateCampaignExperimentMetadata instance
                             */
                            CreateCampaignExperimentMetadata.create = function create(properties) {
                                return new CreateCampaignExperimentMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified CreateCampaignExperimentMetadata message. Does not implicitly {@link google.ads.googleads.v11.services.CreateCampaignExperimentMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentMetadata} m CreateCampaignExperimentMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCampaignExperimentMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_experiment != null && Object.hasOwnProperty.call(m, "campaign_experiment"))
                                    w.uint32(10).string(m.campaign_experiment);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateCampaignExperimentMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateCampaignExperimentMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCampaignExperimentMetadata} message CreateCampaignExperimentMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCampaignExperimentMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateCampaignExperimentMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentMetadata} CreateCampaignExperimentMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCampaignExperimentMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateCampaignExperimentMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.campaign_experiment = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateCampaignExperimentMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentMetadata} CreateCampaignExperimentMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCampaignExperimentMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateCampaignExperimentMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateCampaignExperimentMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    if (!$util.isString(m.campaign_experiment))
                                        return "campaign_experiment: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateCampaignExperimentMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateCampaignExperimentMetadata} CreateCampaignExperimentMetadata
                             */
                            CreateCampaignExperimentMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateCampaignExperimentMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateCampaignExperimentMetadata();
                                if (d.campaign_experiment != null) {
                                    m.campaign_experiment = String(d.campaign_experiment);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateCampaignExperimentMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateCampaignExperimentMetadata} m CreateCampaignExperimentMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateCampaignExperimentMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.campaign_experiment = "";
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    d.campaign_experiment = m.campaign_experiment;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateCampaignExperimentMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateCampaignExperimentMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateCampaignExperimentMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateCampaignExperimentMetadata;
                        })();
    
                        services.GraduateCampaignExperimentRequest = (function() {
    
                            /**
                             * Properties of a GraduateCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGraduateCampaignExperimentRequest
                             * @property {string|null} [campaign_experiment] GraduateCampaignExperimentRequest campaign_experiment
                             * @property {string|null} [campaign_budget] GraduateCampaignExperimentRequest campaign_budget
                             * @property {boolean|null} [validate_only] GraduateCampaignExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new GraduateCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GraduateCampaignExperimentRequest.
                             * @implements IGraduateCampaignExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentRequest=} [p] Properties to set
                             */
                            function GraduateCampaignExperimentRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GraduateCampaignExperimentRequest campaign_experiment.
                             * @member {string} campaign_experiment
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @instance
                             */
                            GraduateCampaignExperimentRequest.prototype.campaign_experiment = "";
    
                            /**
                             * GraduateCampaignExperimentRequest campaign_budget.
                             * @member {string} campaign_budget
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @instance
                             */
                            GraduateCampaignExperimentRequest.prototype.campaign_budget = "";
    
                            /**
                             * GraduateCampaignExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @instance
                             */
                            GraduateCampaignExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new GraduateCampaignExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentRequest} GraduateCampaignExperimentRequest instance
                             */
                            GraduateCampaignExperimentRequest.create = function create(properties) {
                                return new GraduateCampaignExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GraduateCampaignExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GraduateCampaignExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentRequest} m GraduateCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GraduateCampaignExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_experiment != null && Object.hasOwnProperty.call(m, "campaign_experiment"))
                                    w.uint32(10).string(m.campaign_experiment);
                                if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                    w.uint32(18).string(m.campaign_budget);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GraduateCampaignExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GraduateCampaignExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentRequest} message GraduateCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GraduateCampaignExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GraduateCampaignExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentRequest} GraduateCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GraduateCampaignExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GraduateCampaignExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.campaign_experiment = r.string();
                                        break;
                                    case 2:
                                        m.campaign_budget = r.string();
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GraduateCampaignExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentRequest} GraduateCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GraduateCampaignExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GraduateCampaignExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GraduateCampaignExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    if (!$util.isString(m.campaign_experiment))
                                        return "campaign_experiment: string expected";
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    if (!$util.isString(m.campaign_budget))
                                        return "campaign_budget: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GraduateCampaignExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentRequest} GraduateCampaignExperimentRequest
                             */
                            GraduateCampaignExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GraduateCampaignExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GraduateCampaignExperimentRequest();
                                if (d.campaign_experiment != null) {
                                    m.campaign_experiment = String(d.campaign_experiment);
                                }
                                if (d.campaign_budget != null) {
                                    m.campaign_budget = String(d.campaign_budget);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GraduateCampaignExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GraduateCampaignExperimentRequest} m GraduateCampaignExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GraduateCampaignExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.campaign_experiment = "";
                                    d.campaign_budget = "";
                                    d.validate_only = false;
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    d.campaign_experiment = m.campaign_experiment;
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    d.campaign_budget = m.campaign_budget;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GraduateCampaignExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GraduateCampaignExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GraduateCampaignExperimentRequest;
                        })();
    
                        services.GraduateCampaignExperimentResponse = (function() {
    
                            /**
                             * Properties of a GraduateCampaignExperimentResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGraduateCampaignExperimentResponse
                             * @property {string|null} [graduated_campaign] GraduateCampaignExperimentResponse graduated_campaign
                             */
    
                            /**
                             * Constructs a new GraduateCampaignExperimentResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GraduateCampaignExperimentResponse.
                             * @implements IGraduateCampaignExperimentResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentResponse=} [p] Properties to set
                             */
                            function GraduateCampaignExperimentResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GraduateCampaignExperimentResponse graduated_campaign.
                             * @member {string} graduated_campaign
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @instance
                             */
                            GraduateCampaignExperimentResponse.prototype.graduated_campaign = "";
    
                            /**
                             * Creates a new GraduateCampaignExperimentResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentResponse} GraduateCampaignExperimentResponse instance
                             */
                            GraduateCampaignExperimentResponse.create = function create(properties) {
                                return new GraduateCampaignExperimentResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GraduateCampaignExperimentResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GraduateCampaignExperimentResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentResponse} m GraduateCampaignExperimentResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GraduateCampaignExperimentResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.graduated_campaign != null && Object.hasOwnProperty.call(m, "graduated_campaign"))
                                    w.uint32(10).string(m.graduated_campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GraduateCampaignExperimentResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GraduateCampaignExperimentResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateCampaignExperimentResponse} message GraduateCampaignExperimentResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GraduateCampaignExperimentResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GraduateCampaignExperimentResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentResponse} GraduateCampaignExperimentResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GraduateCampaignExperimentResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GraduateCampaignExperimentResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.graduated_campaign = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GraduateCampaignExperimentResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentResponse} GraduateCampaignExperimentResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GraduateCampaignExperimentResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GraduateCampaignExperimentResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GraduateCampaignExperimentResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.graduated_campaign != null && m.hasOwnProperty("graduated_campaign")) {
                                    if (!$util.isString(m.graduated_campaign))
                                        return "graduated_campaign: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GraduateCampaignExperimentResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GraduateCampaignExperimentResponse} GraduateCampaignExperimentResponse
                             */
                            GraduateCampaignExperimentResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GraduateCampaignExperimentResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GraduateCampaignExperimentResponse();
                                if (d.graduated_campaign != null) {
                                    m.graduated_campaign = String(d.graduated_campaign);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GraduateCampaignExperimentResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GraduateCampaignExperimentResponse} m GraduateCampaignExperimentResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GraduateCampaignExperimentResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.graduated_campaign = "";
                                }
                                if (m.graduated_campaign != null && m.hasOwnProperty("graduated_campaign")) {
                                    d.graduated_campaign = m.graduated_campaign;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GraduateCampaignExperimentResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GraduateCampaignExperimentResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GraduateCampaignExperimentResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GraduateCampaignExperimentResponse;
                        })();
    
                        services.PromoteCampaignExperimentRequest = (function() {
    
                            /**
                             * Properties of a PromoteCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPromoteCampaignExperimentRequest
                             * @property {string|null} [campaign_experiment] PromoteCampaignExperimentRequest campaign_experiment
                             * @property {boolean|null} [validate_only] PromoteCampaignExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new PromoteCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PromoteCampaignExperimentRequest.
                             * @implements IPromoteCampaignExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignExperimentRequest=} [p] Properties to set
                             */
                            function PromoteCampaignExperimentRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PromoteCampaignExperimentRequest campaign_experiment.
                             * @member {string} campaign_experiment
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @instance
                             */
                            PromoteCampaignExperimentRequest.prototype.campaign_experiment = "";
    
                            /**
                             * PromoteCampaignExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @instance
                             */
                            PromoteCampaignExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new PromoteCampaignExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignExperimentRequest} PromoteCampaignExperimentRequest instance
                             */
                            PromoteCampaignExperimentRequest.create = function create(properties) {
                                return new PromoteCampaignExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified PromoteCampaignExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.PromoteCampaignExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignExperimentRequest} m PromoteCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteCampaignExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_experiment != null && Object.hasOwnProperty.call(m, "campaign_experiment"))
                                    w.uint32(10).string(m.campaign_experiment);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(16).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromoteCampaignExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PromoteCampaignExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteCampaignExperimentRequest} message PromoteCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteCampaignExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromoteCampaignExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignExperimentRequest} PromoteCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteCampaignExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PromoteCampaignExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.campaign_experiment = r.string();
                                        break;
                                    case 2:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromoteCampaignExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignExperimentRequest} PromoteCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteCampaignExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromoteCampaignExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromoteCampaignExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    if (!$util.isString(m.campaign_experiment))
                                        return "campaign_experiment: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PromoteCampaignExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PromoteCampaignExperimentRequest} PromoteCampaignExperimentRequest
                             */
                            PromoteCampaignExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PromoteCampaignExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PromoteCampaignExperimentRequest();
                                if (d.campaign_experiment != null) {
                                    m.campaign_experiment = String(d.campaign_experiment);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PromoteCampaignExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.PromoteCampaignExperimentRequest} m PromoteCampaignExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromoteCampaignExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.campaign_experiment = "";
                                    d.validate_only = false;
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    d.campaign_experiment = m.campaign_experiment;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PromoteCampaignExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PromoteCampaignExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromoteCampaignExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PromoteCampaignExperimentRequest;
                        })();
    
                        services.EndCampaignExperimentRequest = (function() {
    
                            /**
                             * Properties of an EndCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IEndCampaignExperimentRequest
                             * @property {string|null} [campaign_experiment] EndCampaignExperimentRequest campaign_experiment
                             * @property {boolean|null} [validate_only] EndCampaignExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new EndCampaignExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an EndCampaignExperimentRequest.
                             * @implements IEndCampaignExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IEndCampaignExperimentRequest=} [p] Properties to set
                             */
                            function EndCampaignExperimentRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * EndCampaignExperimentRequest campaign_experiment.
                             * @member {string} campaign_experiment
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @instance
                             */
                            EndCampaignExperimentRequest.prototype.campaign_experiment = "";
    
                            /**
                             * EndCampaignExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @instance
                             */
                            EndCampaignExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new EndCampaignExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IEndCampaignExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.EndCampaignExperimentRequest} EndCampaignExperimentRequest instance
                             */
                            EndCampaignExperimentRequest.create = function create(properties) {
                                return new EndCampaignExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified EndCampaignExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.EndCampaignExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IEndCampaignExperimentRequest} m EndCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EndCampaignExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_experiment != null && Object.hasOwnProperty.call(m, "campaign_experiment"))
                                    w.uint32(10).string(m.campaign_experiment);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(16).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified EndCampaignExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.EndCampaignExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IEndCampaignExperimentRequest} message EndCampaignExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EndCampaignExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EndCampaignExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.EndCampaignExperimentRequest} EndCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EndCampaignExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.EndCampaignExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.campaign_experiment = r.string();
                                        break;
                                    case 2:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an EndCampaignExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.EndCampaignExperimentRequest} EndCampaignExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EndCampaignExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EndCampaignExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EndCampaignExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    if (!$util.isString(m.campaign_experiment))
                                        return "campaign_experiment: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an EndCampaignExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.EndCampaignExperimentRequest} EndCampaignExperimentRequest
                             */
                            EndCampaignExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.EndCampaignExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.EndCampaignExperimentRequest();
                                if (d.campaign_experiment != null) {
                                    m.campaign_experiment = String(d.campaign_experiment);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an EndCampaignExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.EndCampaignExperimentRequest} m EndCampaignExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EndCampaignExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.campaign_experiment = "";
                                    d.validate_only = false;
                                }
                                if (m.campaign_experiment != null && m.hasOwnProperty("campaign_experiment")) {
                                    d.campaign_experiment = m.campaign_experiment;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this EndCampaignExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.EndCampaignExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EndCampaignExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return EndCampaignExperimentRequest;
                        })();
    
                        services.ListCampaignExperimentAsyncErrorsRequest = (function() {
    
                            /**
                             * Properties of a ListCampaignExperimentAsyncErrorsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListCampaignExperimentAsyncErrorsRequest
                             * @property {string|null} [resource_name] ListCampaignExperimentAsyncErrorsRequest resource_name
                             * @property {string|null} [page_token] ListCampaignExperimentAsyncErrorsRequest page_token
                             * @property {number|null} [page_size] ListCampaignExperimentAsyncErrorsRequest page_size
                             */
    
                            /**
                             * Constructs a new ListCampaignExperimentAsyncErrorsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListCampaignExperimentAsyncErrorsRequest.
                             * @implements IListCampaignExperimentAsyncErrorsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsRequest=} [p] Properties to set
                             */
                            function ListCampaignExperimentAsyncErrorsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListCampaignExperimentAsyncErrorsRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @instance
                             */
                            ListCampaignExperimentAsyncErrorsRequest.prototype.resource_name = "";
    
                            /**
                             * ListCampaignExperimentAsyncErrorsRequest page_token.
                             * @member {string} page_token
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @instance
                             */
                            ListCampaignExperimentAsyncErrorsRequest.prototype.page_token = "";
    
                            /**
                             * ListCampaignExperimentAsyncErrorsRequest page_size.
                             * @member {number} page_size
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @instance
                             */
                            ListCampaignExperimentAsyncErrorsRequest.prototype.page_size = 0;
    
                            /**
                             * Creates a new ListCampaignExperimentAsyncErrorsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest} ListCampaignExperimentAsyncErrorsRequest instance
                             */
                            ListCampaignExperimentAsyncErrorsRequest.create = function create(properties) {
                                return new ListCampaignExperimentAsyncErrorsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListCampaignExperimentAsyncErrorsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsRequest} m ListCampaignExperimentAsyncErrorsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignExperimentAsyncErrorsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                                    w.uint32(18).string(m.page_token);
                                if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                                    w.uint32(24).int32(m.page_size);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListCampaignExperimentAsyncErrorsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsRequest} message ListCampaignExperimentAsyncErrorsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignExperimentAsyncErrorsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListCampaignExperimentAsyncErrorsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest} ListCampaignExperimentAsyncErrorsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignExperimentAsyncErrorsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.page_token = r.string();
                                        break;
                                    case 3:
                                        m.page_size = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListCampaignExperimentAsyncErrorsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest} ListCampaignExperimentAsyncErrorsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignExperimentAsyncErrorsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListCampaignExperimentAsyncErrorsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListCampaignExperimentAsyncErrorsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    if (!$util.isString(m.page_token))
                                        return "page_token: string expected";
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    if (!$util.isInteger(m.page_size))
                                        return "page_size: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListCampaignExperimentAsyncErrorsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest} ListCampaignExperimentAsyncErrorsRequest
                             */
                            ListCampaignExperimentAsyncErrorsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.page_token != null) {
                                    m.page_token = String(d.page_token);
                                }
                                if (d.page_size != null) {
                                    m.page_size = d.page_size | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListCampaignExperimentAsyncErrorsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest} m ListCampaignExperimentAsyncErrorsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListCampaignExperimentAsyncErrorsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.page_token = "";
                                    d.page_size = 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    d.page_token = m.page_token;
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    d.page_size = m.page_size;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListCampaignExperimentAsyncErrorsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListCampaignExperimentAsyncErrorsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListCampaignExperimentAsyncErrorsRequest;
                        })();
    
                        services.ListCampaignExperimentAsyncErrorsResponse = (function() {
    
                            /**
                             * Properties of a ListCampaignExperimentAsyncErrorsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListCampaignExperimentAsyncErrorsResponse
                             * @property {Array.<google.rpc.IStatus>|null} [errors] ListCampaignExperimentAsyncErrorsResponse errors
                             * @property {string|null} [next_page_token] ListCampaignExperimentAsyncErrorsResponse next_page_token
                             */
    
                            /**
                             * Constructs a new ListCampaignExperimentAsyncErrorsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListCampaignExperimentAsyncErrorsResponse.
                             * @implements IListCampaignExperimentAsyncErrorsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsResponse=} [p] Properties to set
                             */
                            function ListCampaignExperimentAsyncErrorsResponse(p) {
                                this.errors = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListCampaignExperimentAsyncErrorsResponse errors.
                             * @member {Array.<google.rpc.IStatus>} errors
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @instance
                             */
                            ListCampaignExperimentAsyncErrorsResponse.prototype.errors = $util.emptyArray;
    
                            /**
                             * ListCampaignExperimentAsyncErrorsResponse next_page_token.
                             * @member {string} next_page_token
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @instance
                             */
                            ListCampaignExperimentAsyncErrorsResponse.prototype.next_page_token = "";
    
                            /**
                             * Creates a new ListCampaignExperimentAsyncErrorsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse} ListCampaignExperimentAsyncErrorsResponse instance
                             */
                            ListCampaignExperimentAsyncErrorsResponse.create = function create(properties) {
                                return new ListCampaignExperimentAsyncErrorsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListCampaignExperimentAsyncErrorsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsResponse} m ListCampaignExperimentAsyncErrorsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignExperimentAsyncErrorsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.errors != null && m.errors.length) {
                                    for (var i = 0; i < m.errors.length; ++i)
                                        $root.google.rpc.Status.encode(m.errors[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                                    w.uint32(18).string(m.next_page_token);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListCampaignExperimentAsyncErrorsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListCampaignExperimentAsyncErrorsResponse} message ListCampaignExperimentAsyncErrorsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListCampaignExperimentAsyncErrorsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListCampaignExperimentAsyncErrorsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse} ListCampaignExperimentAsyncErrorsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignExperimentAsyncErrorsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.errors && m.errors.length))
                                            m.errors = [];
                                        m.errors.push($root.google.rpc.Status.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.next_page_token = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListCampaignExperimentAsyncErrorsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse} ListCampaignExperimentAsyncErrorsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListCampaignExperimentAsyncErrorsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListCampaignExperimentAsyncErrorsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListCampaignExperimentAsyncErrorsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.errors != null && m.hasOwnProperty("errors")) {
                                    if (!Array.isArray(m.errors))
                                        return "errors: array expected";
                                    for (var i = 0; i < m.errors.length; ++i) {
                                        {
                                            var e = $root.google.rpc.Status.verify(m.errors[i]);
                                            if (e)
                                                return "errors." + e;
                                        }
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    if (!$util.isString(m.next_page_token))
                                        return "next_page_token: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListCampaignExperimentAsyncErrorsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse} ListCampaignExperimentAsyncErrorsResponse
                             */
                            ListCampaignExperimentAsyncErrorsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse();
                                if (d.errors) {
                                    if (!Array.isArray(d.errors))
                                        throw TypeError(".google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse.errors: array expected");
                                    m.errors = [];
                                    for (var i = 0; i < d.errors.length; ++i) {
                                        if (typeof d.errors[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse.errors: object expected");
                                        m.errors[i] = $root.google.rpc.Status.fromObject(d.errors[i]);
                                    }
                                }
                                if (d.next_page_token != null) {
                                    m.next_page_token = String(d.next_page_token);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListCampaignExperimentAsyncErrorsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse} m ListCampaignExperimentAsyncErrorsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListCampaignExperimentAsyncErrorsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.errors = [];
                                }
                                if (o.defaults) {
                                    d.next_page_token = "";
                                }
                                if (m.errors && m.errors.length) {
                                    d.errors = [];
                                    for (var j = 0; j < m.errors.length; ++j) {
                                        d.errors[j] = $root.google.rpc.Status.toObject(m.errors[j], o);
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    d.next_page_token = m.next_page_token;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListCampaignExperimentAsyncErrorsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListCampaignExperimentAsyncErrorsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListCampaignExperimentAsyncErrorsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListCampaignExperimentAsyncErrorsResponse;
                        })();
    
                        services.CampaignExtensionSettingService = (function() {
    
                            /**
                             * Constructs a new CampaignExtensionSettingService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignExtensionSettingService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignExtensionSettingService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignExtensionSettingService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignExtensionSettingService;
    
                            /**
                             * Creates new CampaignExtensionSettingService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignExtensionSettingService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignExtensionSettingService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignExtensionSettingService|mutateCampaignExtensionSettings}.
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingService
                             * @typedef MutateCampaignExtensionSettingsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse} [response] MutateCampaignExtensionSettingsResponse
                             */
    
                            /**
                             * Calls MutateCampaignExtensionSettings.
                             * @function mutateCampaignExtensionSettings
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsRequest} request MutateCampaignExtensionSettingsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignExtensionSettingService.MutateCampaignExtensionSettingsCallback} callback Node-style callback called with the error, if any, and MutateCampaignExtensionSettingsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignExtensionSettingService.prototype.mutateCampaignExtensionSettings = function mutateCampaignExtensionSettings(request, callback) {
                                return this.rpcCall(mutateCampaignExtensionSettings, $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest, $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse, request, callback);
                            }, "name", { value: "MutateCampaignExtensionSettings" });
    
                            /**
                             * Calls MutateCampaignExtensionSettings.
                             * @function mutateCampaignExtensionSettings
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsRequest} request MutateCampaignExtensionSettingsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignExtensionSettingService;
                        })();
    
                        services.MutateCampaignExtensionSettingsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignExtensionSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignExtensionSettingsRequest
                             * @property {string|null} [customer_id] MutateCampaignExtensionSettingsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignExtensionSettingOperation>|null} [operations] MutateCampaignExtensionSettingsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignExtensionSettingsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignExtensionSettingsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignExtensionSettingsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignExtensionSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignExtensionSettingsRequest.
                             * @implements IMutateCampaignExtensionSettingsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsRequest=} [p] Properties to set
                             */
                            function MutateCampaignExtensionSettingsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignExtensionSettingsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @instance
                             */
                            MutateCampaignExtensionSettingsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignExtensionSettingsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignExtensionSettingOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @instance
                             */
                            MutateCampaignExtensionSettingsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignExtensionSettingsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @instance
                             */
                            MutateCampaignExtensionSettingsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignExtensionSettingsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @instance
                             */
                            MutateCampaignExtensionSettingsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignExtensionSettingsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @instance
                             */
                            MutateCampaignExtensionSettingsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignExtensionSettingsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest} MutateCampaignExtensionSettingsRequest instance
                             */
                            MutateCampaignExtensionSettingsRequest.create = function create(properties) {
                                return new MutateCampaignExtensionSettingsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExtensionSettingsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsRequest} m MutateCampaignExtensionSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExtensionSettingsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExtensionSettingsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsRequest} message MutateCampaignExtensionSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExtensionSettingsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignExtensionSettingsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest} MutateCampaignExtensionSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExtensionSettingsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignExtensionSettingsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest} MutateCampaignExtensionSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExtensionSettingsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignExtensionSettingsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignExtensionSettingsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignExtensionSettingsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest} MutateCampaignExtensionSettingsRequest
                             */
                            MutateCampaignExtensionSettingsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignExtensionSettingsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest} m MutateCampaignExtensionSettingsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignExtensionSettingsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignExtensionSettingsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignExtensionSettingsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignExtensionSettingsRequest;
                        })();
    
                        services.CampaignExtensionSettingOperation = (function() {
    
                            /**
                             * Properties of a CampaignExtensionSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignExtensionSettingOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignExtensionSettingOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null} [create] CampaignExtensionSettingOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null} [update] CampaignExtensionSettingOperation update
                             * @property {string|null} [remove] CampaignExtensionSettingOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignExtensionSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignExtensionSettingOperation.
                             * @implements ICampaignExtensionSettingOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignExtensionSettingOperation=} [p] Properties to set
                             */
                            function CampaignExtensionSettingOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignExtensionSettingOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @instance
                             */
                            CampaignExtensionSettingOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignExtensionSettingOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @instance
                             */
                            CampaignExtensionSettingOperation.prototype.create = null;
    
                            /**
                             * CampaignExtensionSettingOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @instance
                             */
                            CampaignExtensionSettingOperation.prototype.update = null;
    
                            /**
                             * CampaignExtensionSettingOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @instance
                             */
                            CampaignExtensionSettingOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignExtensionSettingOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignExtensionSettingOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignExtensionSettingOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignExtensionSettingOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignExtensionSettingOperation} CampaignExtensionSettingOperation instance
                             */
                            CampaignExtensionSettingOperation.create = function create(properties) {
                                return new CampaignExtensionSettingOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignExtensionSettingOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignExtensionSettingOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignExtensionSettingOperation} m CampaignExtensionSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExtensionSettingOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignExtensionSettingOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignExtensionSettingOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignExtensionSettingOperation} message CampaignExtensionSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignExtensionSettingOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignExtensionSettingOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignExtensionSettingOperation} CampaignExtensionSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExtensionSettingOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignExtensionSettingOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignExtensionSettingOperation} CampaignExtensionSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignExtensionSettingOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignExtensionSettingOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignExtensionSettingOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignExtensionSettingOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignExtensionSettingOperation} CampaignExtensionSettingOperation
                             */
                            CampaignExtensionSettingOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignExtensionSettingOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignExtensionSettingOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignExtensionSettingOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignExtensionSettingOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignExtensionSettingOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignExtensionSettingOperation} m CampaignExtensionSettingOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignExtensionSettingOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignExtensionSettingOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignExtensionSettingOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignExtensionSettingOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignExtensionSettingOperation;
                        })();
    
                        services.MutateCampaignExtensionSettingsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignExtensionSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignExtensionSettingsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignExtensionSettingsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult>|null} [results] MutateCampaignExtensionSettingsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignExtensionSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignExtensionSettingsResponse.
                             * @implements IMutateCampaignExtensionSettingsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsResponse=} [p] Properties to set
                             */
                            function MutateCampaignExtensionSettingsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignExtensionSettingsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @instance
                             */
                            MutateCampaignExtensionSettingsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignExtensionSettingsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @instance
                             */
                            MutateCampaignExtensionSettingsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignExtensionSettingsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse} MutateCampaignExtensionSettingsResponse instance
                             */
                            MutateCampaignExtensionSettingsResponse.create = function create(properties) {
                                return new MutateCampaignExtensionSettingsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExtensionSettingsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsResponse} m MutateCampaignExtensionSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExtensionSettingsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExtensionSettingsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingsResponse} message MutateCampaignExtensionSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExtensionSettingsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignExtensionSettingsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse} MutateCampaignExtensionSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExtensionSettingsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignExtensionSettingsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse} MutateCampaignExtensionSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExtensionSettingsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignExtensionSettingsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignExtensionSettingsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignExtensionSettingsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse} MutateCampaignExtensionSettingsResponse
                             */
                            MutateCampaignExtensionSettingsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignExtensionSettingsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse} m MutateCampaignExtensionSettingsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignExtensionSettingsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignExtensionSettingsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignExtensionSettingsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignExtensionSettingsResponse;
                        })();
    
                        services.MutateCampaignExtensionSettingResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignExtensionSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignExtensionSettingResult
                             * @property {string|null} [resource_name] MutateCampaignExtensionSettingResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null} [campaign_extension_setting] MutateCampaignExtensionSettingResult campaign_extension_setting
                             */
    
                            /**
                             * Constructs a new MutateCampaignExtensionSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignExtensionSettingResult.
                             * @implements IMutateCampaignExtensionSettingResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult=} [p] Properties to set
                             */
                            function MutateCampaignExtensionSettingResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignExtensionSettingResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @instance
                             */
                            MutateCampaignExtensionSettingResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignExtensionSettingResult campaign_extension_setting.
                             * @member {google.ads.googleads.v11.resources.ICampaignExtensionSetting|null|undefined} campaign_extension_setting
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @instance
                             */
                            MutateCampaignExtensionSettingResult.prototype.campaign_extension_setting = null;
    
                            /**
                             * Creates a new MutateCampaignExtensionSettingResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult} MutateCampaignExtensionSettingResult instance
                             */
                            MutateCampaignExtensionSettingResult.create = function create(properties) {
                                return new MutateCampaignExtensionSettingResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExtensionSettingResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult} m MutateCampaignExtensionSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExtensionSettingResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_extension_setting != null && Object.hasOwnProperty.call(m, "campaign_extension_setting"))
                                    $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.encode(m.campaign_extension_setting, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignExtensionSettingResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignExtensionSettingResult} message MutateCampaignExtensionSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignExtensionSettingResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignExtensionSettingResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult} MutateCampaignExtensionSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExtensionSettingResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_extension_setting = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignExtensionSettingResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult} MutateCampaignExtensionSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignExtensionSettingResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignExtensionSettingResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignExtensionSettingResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_extension_setting != null && m.hasOwnProperty("campaign_extension_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.verify(m.campaign_extension_setting);
                                        if (e)
                                            return "campaign_extension_setting." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignExtensionSettingResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult} MutateCampaignExtensionSettingResult
                             */
                            MutateCampaignExtensionSettingResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_extension_setting != null) {
                                    if (typeof d.campaign_extension_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult.campaign_extension_setting: object expected");
                                    m.campaign_extension_setting = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.fromObject(d.campaign_extension_setting);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignExtensionSettingResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult} m MutateCampaignExtensionSettingResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignExtensionSettingResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_extension_setting = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_extension_setting != null && m.hasOwnProperty("campaign_extension_setting")) {
                                    d.campaign_extension_setting = $root.google.ads.googleads.v11.resources.CampaignExtensionSetting.toObject(m.campaign_extension_setting, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignExtensionSettingResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignExtensionSettingResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignExtensionSettingResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignExtensionSettingResult;
                        })();
    
                        services.CampaignFeedService = (function() {
    
                            /**
                             * Constructs a new CampaignFeedService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignFeedService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignFeedService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignFeedService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignFeedService;
    
                            /**
                             * Creates new CampaignFeedService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignFeedService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignFeedService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignFeedService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignFeedService|mutateCampaignFeeds}.
                             * @memberof google.ads.googleads.v11.services.CampaignFeedService
                             * @typedef MutateCampaignFeedsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignFeedsResponse} [response] MutateCampaignFeedsResponse
                             */
    
                            /**
                             * Calls MutateCampaignFeeds.
                             * @function mutateCampaignFeeds
                             * @memberof google.ads.googleads.v11.services.CampaignFeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsRequest} request MutateCampaignFeedsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignFeedService.MutateCampaignFeedsCallback} callback Node-style callback called with the error, if any, and MutateCampaignFeedsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignFeedService.prototype.mutateCampaignFeeds = function mutateCampaignFeeds(request, callback) {
                                return this.rpcCall(mutateCampaignFeeds, $root.google.ads.googleads.v11.services.MutateCampaignFeedsRequest, $root.google.ads.googleads.v11.services.MutateCampaignFeedsResponse, request, callback);
                            }, "name", { value: "MutateCampaignFeeds" });
    
                            /**
                             * Calls MutateCampaignFeeds.
                             * @function mutateCampaignFeeds
                             * @memberof google.ads.googleads.v11.services.CampaignFeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsRequest} request MutateCampaignFeedsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignFeedsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignFeedService;
                        })();
    
                        services.MutateCampaignFeedsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignFeedsRequest
                             * @property {string|null} [customer_id] MutateCampaignFeedsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignFeedOperation>|null} [operations] MutateCampaignFeedsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignFeedsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignFeedsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignFeedsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignFeedsRequest.
                             * @implements IMutateCampaignFeedsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsRequest=} [p] Properties to set
                             */
                            function MutateCampaignFeedsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignFeedsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @instance
                             */
                            MutateCampaignFeedsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignFeedsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignFeedOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @instance
                             */
                            MutateCampaignFeedsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignFeedsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @instance
                             */
                            MutateCampaignFeedsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignFeedsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @instance
                             */
                            MutateCampaignFeedsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignFeedsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @instance
                             */
                            MutateCampaignFeedsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignFeedsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsRequest} MutateCampaignFeedsRequest instance
                             */
                            MutateCampaignFeedsRequest.create = function create(properties) {
                                return new MutateCampaignFeedsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignFeedsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignFeedsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsRequest} m MutateCampaignFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignFeedsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignFeedOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignFeedsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignFeedsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsRequest} message MutateCampaignFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignFeedsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignFeedsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsRequest} MutateCampaignFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignFeedsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignFeedsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignFeedOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignFeedsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsRequest} MutateCampaignFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignFeedsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignFeedsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignFeedsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignFeedOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignFeedsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsRequest} MutateCampaignFeedsRequest
                             */
                            MutateCampaignFeedsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignFeedsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignFeedsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignFeedsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignFeedsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignFeedOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignFeedsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignFeedsRequest} m MutateCampaignFeedsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignFeedsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignFeedOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignFeedsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignFeedsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignFeedsRequest;
                        })();
    
                        services.CampaignFeedOperation = (function() {
    
                            /**
                             * Properties of a CampaignFeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignFeedOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignFeedOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignFeed|null} [create] CampaignFeedOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignFeed|null} [update] CampaignFeedOperation update
                             * @property {string|null} [remove] CampaignFeedOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignFeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignFeedOperation.
                             * @implements ICampaignFeedOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignFeedOperation=} [p] Properties to set
                             */
                            function CampaignFeedOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignFeedOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @instance
                             */
                            CampaignFeedOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignFeedOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignFeed|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @instance
                             */
                            CampaignFeedOperation.prototype.create = null;
    
                            /**
                             * CampaignFeedOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignFeed|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @instance
                             */
                            CampaignFeedOperation.prototype.update = null;
    
                            /**
                             * CampaignFeedOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @instance
                             */
                            CampaignFeedOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignFeedOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignFeedOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignFeedOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignFeedOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignFeedOperation} CampaignFeedOperation instance
                             */
                            CampaignFeedOperation.create = function create(properties) {
                                return new CampaignFeedOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignFeedOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignFeedOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignFeedOperation} m CampaignFeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignFeedOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignFeed.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignFeed.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignFeedOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignFeedOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignFeedOperation} message CampaignFeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignFeedOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignFeedOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignFeedOperation} CampaignFeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignFeedOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignFeedOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignFeed.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignFeed.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignFeedOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignFeedOperation} CampaignFeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignFeedOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignFeedOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignFeedOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignFeed.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignFeed.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignFeedOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignFeedOperation} CampaignFeedOperation
                             */
                            CampaignFeedOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignFeedOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignFeedOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignFeedOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignFeedOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignFeed.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignFeedOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignFeed.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignFeedOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignFeedOperation} m CampaignFeedOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignFeedOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignFeed.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignFeed.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignFeedOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignFeedOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignFeedOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignFeedOperation;
                        })();
    
                        services.MutateCampaignFeedsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignFeedsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignFeedsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignFeedResult>|null} [results] MutateCampaignFeedsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignFeedsResponse.
                             * @implements IMutateCampaignFeedsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsResponse=} [p] Properties to set
                             */
                            function MutateCampaignFeedsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignFeedsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @instance
                             */
                            MutateCampaignFeedsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignFeedsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignFeedResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @instance
                             */
                            MutateCampaignFeedsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignFeedsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsResponse} MutateCampaignFeedsResponse instance
                             */
                            MutateCampaignFeedsResponse.create = function create(properties) {
                                return new MutateCampaignFeedsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignFeedsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignFeedsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsResponse} m MutateCampaignFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignFeedsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignFeedsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignFeedsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedsResponse} message MutateCampaignFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignFeedsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignFeedsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsResponse} MutateCampaignFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignFeedsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignFeedsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignFeedResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignFeedsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsResponse} MutateCampaignFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignFeedsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignFeedsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignFeedsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignFeedsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedsResponse} MutateCampaignFeedsResponse
                             */
                            MutateCampaignFeedsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignFeedsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignFeedsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignFeedsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignFeedsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignFeedsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignFeedsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignFeedsResponse} m MutateCampaignFeedsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignFeedsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignFeedResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignFeedsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignFeedsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignFeedsResponse;
                        })();
    
                        services.MutateCampaignFeedResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignFeedResult
                             * @property {string|null} [resource_name] MutateCampaignFeedResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignFeed|null} [campaign_feed] MutateCampaignFeedResult campaign_feed
                             */
    
                            /**
                             * Constructs a new MutateCampaignFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignFeedResult.
                             * @implements IMutateCampaignFeedResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedResult=} [p] Properties to set
                             */
                            function MutateCampaignFeedResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignFeedResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @instance
                             */
                            MutateCampaignFeedResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignFeedResult campaign_feed.
                             * @member {google.ads.googleads.v11.resources.ICampaignFeed|null|undefined} campaign_feed
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @instance
                             */
                            MutateCampaignFeedResult.prototype.campaign_feed = null;
    
                            /**
                             * Creates a new MutateCampaignFeedResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedResult} MutateCampaignFeedResult instance
                             */
                            MutateCampaignFeedResult.create = function create(properties) {
                                return new MutateCampaignFeedResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignFeedResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignFeedResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedResult} m MutateCampaignFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignFeedResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_feed != null && Object.hasOwnProperty.call(m, "campaign_feed"))
                                    $root.google.ads.googleads.v11.resources.CampaignFeed.encode(m.campaign_feed, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignFeedResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignFeedResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignFeedResult} message MutateCampaignFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignFeedResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignFeedResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedResult} MutateCampaignFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignFeedResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignFeedResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignFeedResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedResult} MutateCampaignFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignFeedResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignFeedResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignFeedResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignFeed.verify(m.campaign_feed);
                                        if (e)
                                            return "campaign_feed." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignFeedResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignFeedResult} MutateCampaignFeedResult
                             */
                            MutateCampaignFeedResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignFeedResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignFeedResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_feed != null) {
                                    if (typeof d.campaign_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignFeedResult.campaign_feed: object expected");
                                    m.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.fromObject(d.campaign_feed);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignFeedResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignFeedResult} m MutateCampaignFeedResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignFeedResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_feed = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_feed != null && m.hasOwnProperty("campaign_feed")) {
                                    d.campaign_feed = $root.google.ads.googleads.v11.resources.CampaignFeed.toObject(m.campaign_feed, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignFeedResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignFeedResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignFeedResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignFeedResult;
                        })();
    
                        services.CampaignGroupService = (function() {
    
                            /**
                             * Constructs a new CampaignGroupService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignGroupService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignGroupService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignGroupService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignGroupService;
    
                            /**
                             * Creates new CampaignGroupService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignGroupService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignGroupService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignGroupService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignGroupService|mutateCampaignGroups}.
                             * @memberof google.ads.googleads.v11.services.CampaignGroupService
                             * @typedef MutateCampaignGroupsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignGroupsResponse} [response] MutateCampaignGroupsResponse
                             */
    
                            /**
                             * Calls MutateCampaignGroups.
                             * @function mutateCampaignGroups
                             * @memberof google.ads.googleads.v11.services.CampaignGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsRequest} request MutateCampaignGroupsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignGroupService.MutateCampaignGroupsCallback} callback Node-style callback called with the error, if any, and MutateCampaignGroupsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignGroupService.prototype.mutateCampaignGroups = function mutateCampaignGroups(request, callback) {
                                return this.rpcCall(mutateCampaignGroups, $root.google.ads.googleads.v11.services.MutateCampaignGroupsRequest, $root.google.ads.googleads.v11.services.MutateCampaignGroupsResponse, request, callback);
                            }, "name", { value: "MutateCampaignGroups" });
    
                            /**
                             * Calls MutateCampaignGroups.
                             * @function mutateCampaignGroups
                             * @memberof google.ads.googleads.v11.services.CampaignGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsRequest} request MutateCampaignGroupsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignGroupsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignGroupService;
                        })();
    
                        services.MutateCampaignGroupsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignGroupsRequest
                             * @property {string|null} [customer_id] MutateCampaignGroupsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignGroupOperation>|null} [operations] MutateCampaignGroupsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignGroupsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignGroupsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignGroupsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignGroupsRequest.
                             * @implements IMutateCampaignGroupsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsRequest=} [p] Properties to set
                             */
                            function MutateCampaignGroupsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignGroupsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @instance
                             */
                            MutateCampaignGroupsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignGroupsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignGroupOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @instance
                             */
                            MutateCampaignGroupsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignGroupsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @instance
                             */
                            MutateCampaignGroupsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignGroupsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @instance
                             */
                            MutateCampaignGroupsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignGroupsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @instance
                             */
                            MutateCampaignGroupsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignGroupsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsRequest} MutateCampaignGroupsRequest instance
                             */
                            MutateCampaignGroupsRequest.create = function create(properties) {
                                return new MutateCampaignGroupsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignGroupsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignGroupsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsRequest} m MutateCampaignGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignGroupsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignGroupOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignGroupsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignGroupsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsRequest} message MutateCampaignGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignGroupsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignGroupsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsRequest} MutateCampaignGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignGroupsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignGroupsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignGroupOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignGroupsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsRequest} MutateCampaignGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignGroupsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignGroupsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignGroupsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignGroupOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignGroupsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsRequest} MutateCampaignGroupsRequest
                             */
                            MutateCampaignGroupsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignGroupsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignGroupsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignGroupsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignGroupsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignGroupOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignGroupsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignGroupsRequest} m MutateCampaignGroupsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignGroupsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignGroupOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignGroupsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignGroupsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignGroupsRequest;
                        })();
    
                        services.CampaignGroupOperation = (function() {
    
                            /**
                             * Properties of a CampaignGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignGroupOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignGroupOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaignGroup|null} [create] CampaignGroupOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaignGroup|null} [update] CampaignGroupOperation update
                             * @property {string|null} [remove] CampaignGroupOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignGroupOperation.
                             * @implements ICampaignGroupOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignGroupOperation=} [p] Properties to set
                             */
                            function CampaignGroupOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignGroupOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @instance
                             */
                            CampaignGroupOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignGroupOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignGroup|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @instance
                             */
                            CampaignGroupOperation.prototype.create = null;
    
                            /**
                             * CampaignGroupOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaignGroup|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @instance
                             */
                            CampaignGroupOperation.prototype.update = null;
    
                            /**
                             * CampaignGroupOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @instance
                             */
                            CampaignGroupOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignGroupOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignGroupOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignGroupOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignGroupOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignGroupOperation} CampaignGroupOperation instance
                             */
                            CampaignGroupOperation.create = function create(properties) {
                                return new CampaignGroupOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignGroupOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignGroupOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignGroupOperation} m CampaignGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignGroupOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignGroup.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CampaignGroup.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignGroupOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignGroupOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignGroupOperation} message CampaignGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignGroupOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignGroupOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignGroupOperation} CampaignGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignGroupOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignGroupOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignGroup.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CampaignGroup.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignGroupOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignGroupOperation} CampaignGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignGroupOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignGroupOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignGroupOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignGroup.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignGroup.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignGroupOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignGroupOperation} CampaignGroupOperation
                             */
                            CampaignGroupOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignGroupOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignGroupOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignGroupOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignGroupOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignGroup.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignGroupOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CampaignGroup.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignGroupOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignGroupOperation} m CampaignGroupOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignGroupOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignGroup.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CampaignGroup.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignGroupOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignGroupOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignGroupOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignGroupOperation;
                        })();
    
                        services.MutateCampaignGroupsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignGroupsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignGroupResult>|null} [results] MutateCampaignGroupsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignGroupsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateCampaignGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignGroupsResponse.
                             * @implements IMutateCampaignGroupsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsResponse=} [p] Properties to set
                             */
                            function MutateCampaignGroupsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignGroupsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignGroupResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @instance
                             */
                            MutateCampaignGroupsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateCampaignGroupsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @instance
                             */
                            MutateCampaignGroupsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateCampaignGroupsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsResponse} MutateCampaignGroupsResponse instance
                             */
                            MutateCampaignGroupsResponse.create = function create(properties) {
                                return new MutateCampaignGroupsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignGroupsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignGroupsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsResponse} m MutateCampaignGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignGroupsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignGroupsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignGroupsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupsResponse} message MutateCampaignGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignGroupsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignGroupsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsResponse} MutateCampaignGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignGroupsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignGroupsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignGroupResult.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignGroupsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsResponse} MutateCampaignGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignGroupsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignGroupsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignGroupsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignGroupsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupsResponse} MutateCampaignGroupsResponse
                             */
                            MutateCampaignGroupsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignGroupsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignGroupsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignGroupsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignGroupsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignGroupsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignGroupsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignGroupsResponse} m MutateCampaignGroupsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignGroupsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignGroupResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignGroupsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignGroupsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignGroupsResponse;
                        })();
    
                        services.MutateCampaignGroupResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignGroupResult
                             * @property {string|null} [resource_name] MutateCampaignGroupResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignGroup|null} [campaign_group] MutateCampaignGroupResult campaign_group
                             */
    
                            /**
                             * Constructs a new MutateCampaignGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignGroupResult.
                             * @implements IMutateCampaignGroupResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupResult=} [p] Properties to set
                             */
                            function MutateCampaignGroupResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignGroupResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @instance
                             */
                            MutateCampaignGroupResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignGroupResult campaign_group.
                             * @member {google.ads.googleads.v11.resources.ICampaignGroup|null|undefined} campaign_group
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @instance
                             */
                            MutateCampaignGroupResult.prototype.campaign_group = null;
    
                            /**
                             * Creates a new MutateCampaignGroupResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupResult} MutateCampaignGroupResult instance
                             */
                            MutateCampaignGroupResult.create = function create(properties) {
                                return new MutateCampaignGroupResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignGroupResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignGroupResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupResult} m MutateCampaignGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignGroupResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_group != null && Object.hasOwnProperty.call(m, "campaign_group"))
                                    $root.google.ads.googleads.v11.resources.CampaignGroup.encode(m.campaign_group, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignGroupResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignGroupResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignGroupResult} message MutateCampaignGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignGroupResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignGroupResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupResult} MutateCampaignGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignGroupResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignGroupResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_group = $root.google.ads.googleads.v11.resources.CampaignGroup.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignGroupResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupResult} MutateCampaignGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignGroupResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignGroupResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignGroupResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_group != null && m.hasOwnProperty("campaign_group")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignGroup.verify(m.campaign_group);
                                        if (e)
                                            return "campaign_group." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignGroupResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignGroupResult} MutateCampaignGroupResult
                             */
                            MutateCampaignGroupResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignGroupResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignGroupResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_group != null) {
                                    if (typeof d.campaign_group !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignGroupResult.campaign_group: object expected");
                                    m.campaign_group = $root.google.ads.googleads.v11.resources.CampaignGroup.fromObject(d.campaign_group);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignGroupResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignGroupResult} m MutateCampaignGroupResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignGroupResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_group = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_group != null && m.hasOwnProperty("campaign_group")) {
                                    d.campaign_group = $root.google.ads.googleads.v11.resources.CampaignGroup.toObject(m.campaign_group, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignGroupResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignGroupResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignGroupResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignGroupResult;
                        })();
    
                        services.CampaignLabelService = (function() {
    
                            /**
                             * Constructs a new CampaignLabelService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignLabelService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignLabelService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignLabelService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignLabelService;
    
                            /**
                             * Creates new CampaignLabelService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignLabelService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignLabelService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignLabelService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignLabelService|mutateCampaignLabels}.
                             * @memberof google.ads.googleads.v11.services.CampaignLabelService
                             * @typedef MutateCampaignLabelsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignLabelsResponse} [response] MutateCampaignLabelsResponse
                             */
    
                            /**
                             * Calls MutateCampaignLabels.
                             * @function mutateCampaignLabels
                             * @memberof google.ads.googleads.v11.services.CampaignLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsRequest} request MutateCampaignLabelsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignLabelService.MutateCampaignLabelsCallback} callback Node-style callback called with the error, if any, and MutateCampaignLabelsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignLabelService.prototype.mutateCampaignLabels = function mutateCampaignLabels(request, callback) {
                                return this.rpcCall(mutateCampaignLabels, $root.google.ads.googleads.v11.services.MutateCampaignLabelsRequest, $root.google.ads.googleads.v11.services.MutateCampaignLabelsResponse, request, callback);
                            }, "name", { value: "MutateCampaignLabels" });
    
                            /**
                             * Calls MutateCampaignLabels.
                             * @function mutateCampaignLabels
                             * @memberof google.ads.googleads.v11.services.CampaignLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsRequest} request MutateCampaignLabelsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignLabelsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignLabelService;
                        })();
    
                        services.MutateCampaignLabelsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignLabelsRequest
                             * @property {string|null} [customer_id] MutateCampaignLabelsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignLabelOperation>|null} [operations] MutateCampaignLabelsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignLabelsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignLabelsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCampaignLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignLabelsRequest.
                             * @implements IMutateCampaignLabelsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsRequest=} [p] Properties to set
                             */
                            function MutateCampaignLabelsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignLabelsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @instance
                             */
                            MutateCampaignLabelsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignLabelsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignLabelOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @instance
                             */
                            MutateCampaignLabelsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignLabelsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @instance
                             */
                            MutateCampaignLabelsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignLabelsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @instance
                             */
                            MutateCampaignLabelsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCampaignLabelsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsRequest} MutateCampaignLabelsRequest instance
                             */
                            MutateCampaignLabelsRequest.create = function create(properties) {
                                return new MutateCampaignLabelsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignLabelsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignLabelsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsRequest} m MutateCampaignLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignLabelsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignLabelOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignLabelsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignLabelsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsRequest} message MutateCampaignLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignLabelsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignLabelsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsRequest} MutateCampaignLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignLabelsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignLabelsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignLabelOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignLabelsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsRequest} MutateCampaignLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignLabelsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignLabelsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignLabelsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignLabelOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignLabelsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsRequest} MutateCampaignLabelsRequest
                             */
                            MutateCampaignLabelsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignLabelsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignLabelsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignLabelsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignLabelsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignLabelOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignLabelsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignLabelsRequest} m MutateCampaignLabelsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignLabelsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignLabelOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignLabelsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignLabelsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignLabelsRequest;
                        })();
    
                        services.CampaignLabelOperation = (function() {
    
                            /**
                             * Properties of a CampaignLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignLabelOperation
                             * @property {google.ads.googleads.v11.resources.ICampaignLabel|null} [create] CampaignLabelOperation create
                             * @property {string|null} [remove] CampaignLabelOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignLabelOperation.
                             * @implements ICampaignLabelOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignLabelOperation=} [p] Properties to set
                             */
                            function CampaignLabelOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignLabelOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignLabel|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @instance
                             */
                            CampaignLabelOperation.prototype.create = null;
    
                            /**
                             * CampaignLabelOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @instance
                             */
                            CampaignLabelOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignLabelOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignLabelOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignLabelOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignLabelOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignLabelOperation} CampaignLabelOperation instance
                             */
                            CampaignLabelOperation.create = function create(properties) {
                                return new CampaignLabelOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignLabelOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignLabelOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignLabelOperation} m CampaignLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignLabelOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignLabel.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignLabelOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignLabelOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignLabelOperation} message CampaignLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignLabelOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignLabelOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignLabelOperation} CampaignLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignLabelOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignLabelOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignLabel.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignLabelOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignLabelOperation} CampaignLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignLabelOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignLabelOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignLabelOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignLabel.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignLabelOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignLabelOperation} CampaignLabelOperation
                             */
                            CampaignLabelOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignLabelOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignLabelOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignLabelOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignLabel.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignLabelOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignLabelOperation} m CampaignLabelOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignLabelOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignLabel.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignLabelOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignLabelOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignLabelOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignLabelOperation;
                        })();
    
                        services.MutateCampaignLabelsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignLabelsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignLabelsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignLabelResult>|null} [results] MutateCampaignLabelsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignLabelsResponse.
                             * @implements IMutateCampaignLabelsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsResponse=} [p] Properties to set
                             */
                            function MutateCampaignLabelsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignLabelsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @instance
                             */
                            MutateCampaignLabelsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignLabelsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignLabelResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @instance
                             */
                            MutateCampaignLabelsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignLabelsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsResponse} MutateCampaignLabelsResponse instance
                             */
                            MutateCampaignLabelsResponse.create = function create(properties) {
                                return new MutateCampaignLabelsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignLabelsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignLabelsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsResponse} m MutateCampaignLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignLabelsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignLabelsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignLabelsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelsResponse} message MutateCampaignLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignLabelsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignLabelsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsResponse} MutateCampaignLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignLabelsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignLabelsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignLabelResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignLabelsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsResponse} MutateCampaignLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignLabelsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignLabelsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignLabelsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignLabelsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelsResponse} MutateCampaignLabelsResponse
                             */
                            MutateCampaignLabelsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignLabelsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignLabelsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignLabelsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignLabelsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignLabelsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignLabelsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignLabelsResponse} m MutateCampaignLabelsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignLabelsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignLabelResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignLabelsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignLabelsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignLabelsResponse;
                        })();
    
                        services.MutateCampaignLabelResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignLabelResult
                             * @property {string|null} [resource_name] MutateCampaignLabelResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCampaignLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignLabelResult.
                             * @implements IMutateCampaignLabelResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelResult=} [p] Properties to set
                             */
                            function MutateCampaignLabelResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignLabelResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @instance
                             */
                            MutateCampaignLabelResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCampaignLabelResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelResult} MutateCampaignLabelResult instance
                             */
                            MutateCampaignLabelResult.create = function create(properties) {
                                return new MutateCampaignLabelResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignLabelResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignLabelResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelResult} m MutateCampaignLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignLabelResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignLabelResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignLabelResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignLabelResult} message MutateCampaignLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignLabelResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignLabelResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelResult} MutateCampaignLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignLabelResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignLabelResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignLabelResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelResult} MutateCampaignLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignLabelResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignLabelResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignLabelResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignLabelResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignLabelResult} MutateCampaignLabelResult
                             */
                            MutateCampaignLabelResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignLabelResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignLabelResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignLabelResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignLabelResult} m MutateCampaignLabelResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignLabelResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignLabelResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignLabelResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignLabelResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignLabelResult;
                        })();
    
                        services.CampaignService = (function() {
    
                            /**
                             * Constructs a new CampaignService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignService;
    
                            /**
                             * Creates new CampaignService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignService|mutateCampaigns}.
                             * @memberof google.ads.googleads.v11.services.CampaignService
                             * @typedef MutateCampaignsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignsResponse} [response] MutateCampaignsResponse
                             */
    
                            /**
                             * Calls MutateCampaigns.
                             * @function mutateCampaigns
                             * @memberof google.ads.googleads.v11.services.CampaignService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsRequest} request MutateCampaignsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignService.MutateCampaignsCallback} callback Node-style callback called with the error, if any, and MutateCampaignsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignService.prototype.mutateCampaigns = function mutateCampaigns(request, callback) {
                                return this.rpcCall(mutateCampaigns, $root.google.ads.googleads.v11.services.MutateCampaignsRequest, $root.google.ads.googleads.v11.services.MutateCampaignsResponse, request, callback);
                            }, "name", { value: "MutateCampaigns" });
    
                            /**
                             * Calls MutateCampaigns.
                             * @function mutateCampaigns
                             * @memberof google.ads.googleads.v11.services.CampaignService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsRequest} request MutateCampaignsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignService;
                        })();
    
                        services.MutateCampaignsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignsRequest
                             * @property {string|null} [customer_id] MutateCampaignsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignOperation>|null} [operations] MutateCampaignsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignsRequest.
                             * @implements IMutateCampaignsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsRequest=} [p] Properties to set
                             */
                            function MutateCampaignsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @instance
                             */
                            MutateCampaignsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @instance
                             */
                            MutateCampaignsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @instance
                             */
                            MutateCampaignsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @instance
                             */
                            MutateCampaignsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @instance
                             */
                            MutateCampaignsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsRequest} MutateCampaignsRequest instance
                             */
                            MutateCampaignsRequest.create = function create(properties) {
                                return new MutateCampaignsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsRequest} m MutateCampaignsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsRequest} message MutateCampaignsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsRequest} MutateCampaignsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsRequest} MutateCampaignsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsRequest} MutateCampaignsRequest
                             */
                            MutateCampaignsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignsRequest} m MutateCampaignsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignsRequest;
                        })();
    
                        services.CampaignOperation = (function() {
    
                            /**
                             * Properties of a CampaignOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CampaignOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICampaign|null} [create] CampaignOperation create
                             * @property {google.ads.googleads.v11.resources.ICampaign|null} [update] CampaignOperation update
                             * @property {string|null} [remove] CampaignOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignOperation.
                             * @implements ICampaignOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignOperation=} [p] Properties to set
                             */
                            function CampaignOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @instance
                             */
                            CampaignOperation.prototype.update_mask = null;
    
                            /**
                             * CampaignOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaign|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @instance
                             */
                            CampaignOperation.prototype.create = null;
    
                            /**
                             * CampaignOperation update.
                             * @member {google.ads.googleads.v11.resources.ICampaign|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @instance
                             */
                            CampaignOperation.prototype.update = null;
    
                            /**
                             * CampaignOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @instance
                             */
                            CampaignOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignOperation} CampaignOperation instance
                             */
                            CampaignOperation.create = function create(properties) {
                                return new CampaignOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignOperation} m CampaignOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.Campaign.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Campaign.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignOperation} message CampaignOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignOperation} CampaignOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.Campaign.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.Campaign.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignOperation} CampaignOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignOperation} CampaignOperation
                             */
                            CampaignOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.Campaign.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Campaign.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignOperation} m CampaignOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.Campaign.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Campaign.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignOperation;
                        })();
    
                        services.MutateCampaignsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignResult>|null} [results] MutateCampaignsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignsResponse.
                             * @implements IMutateCampaignsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsResponse=} [p] Properties to set
                             */
                            function MutateCampaignsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @instance
                             */
                            MutateCampaignsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @instance
                             */
                            MutateCampaignsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsResponse} MutateCampaignsResponse instance
                             */
                            MutateCampaignsResponse.create = function create(properties) {
                                return new MutateCampaignsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsResponse} m MutateCampaignsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignsResponse} message MutateCampaignsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsResponse} MutateCampaignsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsResponse} MutateCampaignsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignsResponse} MutateCampaignsResponse
                             */
                            MutateCampaignsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignsResponse} m MutateCampaignsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignsResponse;
                        })();
    
                        services.MutateCampaignResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignResult
                             * @property {string|null} [resource_name] MutateCampaignResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaign|null} [campaign] MutateCampaignResult campaign
                             */
    
                            /**
                             * Constructs a new MutateCampaignResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignResult.
                             * @implements IMutateCampaignResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignResult=} [p] Properties to set
                             */
                            function MutateCampaignResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @instance
                             */
                            MutateCampaignResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignResult campaign.
                             * @member {google.ads.googleads.v11.resources.ICampaign|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @instance
                             */
                            MutateCampaignResult.prototype.campaign = null;
    
                            /**
                             * Creates a new MutateCampaignResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignResult} MutateCampaignResult instance
                             */
                            MutateCampaignResult.create = function create(properties) {
                                return new MutateCampaignResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignResult} m MutateCampaignResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    $root.google.ads.googleads.v11.resources.Campaign.encode(m.campaign, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignResult} message MutateCampaignResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignResult} MutateCampaignResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign = $root.google.ads.googleads.v11.resources.Campaign.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignResult} MutateCampaignResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Campaign.verify(m.campaign);
                                        if (e)
                                            return "campaign." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignResult} MutateCampaignResult
                             */
                            MutateCampaignResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign != null) {
                                    if (typeof d.campaign !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignResult.campaign: object expected");
                                    m.campaign = $root.google.ads.googleads.v11.resources.Campaign.fromObject(d.campaign);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignResult} m MutateCampaignResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = $root.google.ads.googleads.v11.resources.Campaign.toObject(m.campaign, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignResult;
                        })();
    
                        services.CampaignSharedSetService = (function() {
    
                            /**
                             * Constructs a new CampaignSharedSetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignSharedSetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CampaignSharedSetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CampaignSharedSetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CampaignSharedSetService;
    
                            /**
                             * Creates new CampaignSharedSetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CampaignSharedSetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CampaignSharedSetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CampaignSharedSetService|mutateCampaignSharedSets}.
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetService
                             * @typedef MutateCampaignSharedSetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse} [response] MutateCampaignSharedSetsResponse
                             */
    
                            /**
                             * Calls MutateCampaignSharedSets.
                             * @function mutateCampaignSharedSets
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsRequest} request MutateCampaignSharedSetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CampaignSharedSetService.MutateCampaignSharedSetsCallback} callback Node-style callback called with the error, if any, and MutateCampaignSharedSetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CampaignSharedSetService.prototype.mutateCampaignSharedSets = function mutateCampaignSharedSets(request, callback) {
                                return this.rpcCall(mutateCampaignSharedSets, $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest, $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse, request, callback);
                            }, "name", { value: "MutateCampaignSharedSets" });
    
                            /**
                             * Calls MutateCampaignSharedSets.
                             * @function mutateCampaignSharedSets
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsRequest} request MutateCampaignSharedSetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse>} Promise
                             * @variation 2
                             */
    
                            return CampaignSharedSetService;
                        })();
    
                        services.MutateCampaignSharedSetsRequest = (function() {
    
                            /**
                             * Properties of a MutateCampaignSharedSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignSharedSetsRequest
                             * @property {string|null} [customer_id] MutateCampaignSharedSetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignSharedSetOperation>|null} [operations] MutateCampaignSharedSetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCampaignSharedSetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCampaignSharedSetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCampaignSharedSetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCampaignSharedSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignSharedSetsRequest.
                             * @implements IMutateCampaignSharedSetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsRequest=} [p] Properties to set
                             */
                            function MutateCampaignSharedSetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignSharedSetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @instance
                             */
                            MutateCampaignSharedSetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCampaignSharedSetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignSharedSetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @instance
                             */
                            MutateCampaignSharedSetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCampaignSharedSetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @instance
                             */
                            MutateCampaignSharedSetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCampaignSharedSetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @instance
                             */
                            MutateCampaignSharedSetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCampaignSharedSetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @instance
                             */
                            MutateCampaignSharedSetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCampaignSharedSetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest} MutateCampaignSharedSetsRequest instance
                             */
                            MutateCampaignSharedSetsRequest.create = function create(properties) {
                                return new MutateCampaignSharedSetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignSharedSetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsRequest} m MutateCampaignSharedSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignSharedSetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignSharedSetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsRequest} message MutateCampaignSharedSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignSharedSetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignSharedSetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest} MutateCampaignSharedSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignSharedSetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CampaignSharedSetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignSharedSetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest} MutateCampaignSharedSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignSharedSetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignSharedSetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignSharedSetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignSharedSetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest} MutateCampaignSharedSetsRequest
                             */
                            MutateCampaignSharedSetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignSharedSetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest} m MutateCampaignSharedSetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignSharedSetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CampaignSharedSetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignSharedSetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignSharedSetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignSharedSetsRequest;
                        })();
    
                        services.CampaignSharedSetOperation = (function() {
    
                            /**
                             * Properties of a CampaignSharedSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignSharedSetOperation
                             * @property {google.ads.googleads.v11.resources.ICampaignSharedSet|null} [create] CampaignSharedSetOperation create
                             * @property {string|null} [remove] CampaignSharedSetOperation remove
                             */
    
                            /**
                             * Constructs a new CampaignSharedSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignSharedSetOperation.
                             * @implements ICampaignSharedSetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignSharedSetOperation=} [p] Properties to set
                             */
                            function CampaignSharedSetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignSharedSetOperation create.
                             * @member {google.ads.googleads.v11.resources.ICampaignSharedSet|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @instance
                             */
                            CampaignSharedSetOperation.prototype.create = null;
    
                            /**
                             * CampaignSharedSetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @instance
                             */
                            CampaignSharedSetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignSharedSetOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @instance
                             */
                            Object.defineProperty(CampaignSharedSetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignSharedSetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignSharedSetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignSharedSetOperation} CampaignSharedSetOperation instance
                             */
                            CampaignSharedSetOperation.create = function create(properties) {
                                return new CampaignSharedSetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignSharedSetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignSharedSetOperation} m CampaignSharedSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CampaignSharedSet.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignSharedSetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignSharedSetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignSharedSetOperation} message CampaignSharedSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignSharedSetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignSharedSetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignSharedSetOperation} CampaignSharedSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignSharedSetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CampaignSharedSet.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignSharedSetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignSharedSetOperation} CampaignSharedSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignSharedSetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignSharedSetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignSharedSetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignSharedSet.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignSharedSetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignSharedSetOperation} CampaignSharedSetOperation
                             */
                            CampaignSharedSetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignSharedSetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignSharedSetOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignSharedSetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CampaignSharedSet.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignSharedSetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignSharedSetOperation} m CampaignSharedSetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignSharedSetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CampaignSharedSet.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignSharedSetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignSharedSetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignSharedSetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignSharedSetOperation;
                        })();
    
                        services.MutateCampaignSharedSetsResponse = (function() {
    
                            /**
                             * Properties of a MutateCampaignSharedSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignSharedSetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCampaignSharedSetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCampaignSharedSetResult>|null} [results] MutateCampaignSharedSetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCampaignSharedSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignSharedSetsResponse.
                             * @implements IMutateCampaignSharedSetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsResponse=} [p] Properties to set
                             */
                            function MutateCampaignSharedSetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignSharedSetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @instance
                             */
                            MutateCampaignSharedSetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCampaignSharedSetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCampaignSharedSetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @instance
                             */
                            MutateCampaignSharedSetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCampaignSharedSetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse} MutateCampaignSharedSetsResponse instance
                             */
                            MutateCampaignSharedSetsResponse.create = function create(properties) {
                                return new MutateCampaignSharedSetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignSharedSetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsResponse} m MutateCampaignSharedSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignSharedSetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignSharedSetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetsResponse} message MutateCampaignSharedSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignSharedSetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignSharedSetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse} MutateCampaignSharedSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignSharedSetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignSharedSetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse} MutateCampaignSharedSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignSharedSetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignSharedSetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignSharedSetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignSharedSetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse} MutateCampaignSharedSetsResponse
                             */
                            MutateCampaignSharedSetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignSharedSetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse} m MutateCampaignSharedSetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignSharedSetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignSharedSetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignSharedSetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignSharedSetsResponse;
                        })();
    
                        services.MutateCampaignSharedSetResult = (function() {
    
                            /**
                             * Properties of a MutateCampaignSharedSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCampaignSharedSetResult
                             * @property {string|null} [resource_name] MutateCampaignSharedSetResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICampaignSharedSet|null} [campaign_shared_set] MutateCampaignSharedSetResult campaign_shared_set
                             */
    
                            /**
                             * Constructs a new MutateCampaignSharedSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCampaignSharedSetResult.
                             * @implements IMutateCampaignSharedSetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetResult=} [p] Properties to set
                             */
                            function MutateCampaignSharedSetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCampaignSharedSetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @instance
                             */
                            MutateCampaignSharedSetResult.prototype.resource_name = "";
    
                            /**
                             * MutateCampaignSharedSetResult campaign_shared_set.
                             * @member {google.ads.googleads.v11.resources.ICampaignSharedSet|null|undefined} campaign_shared_set
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @instance
                             */
                            MutateCampaignSharedSetResult.prototype.campaign_shared_set = null;
    
                            /**
                             * Creates a new MutateCampaignSharedSetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetResult} MutateCampaignSharedSetResult instance
                             */
                            MutateCampaignSharedSetResult.create = function create(properties) {
                                return new MutateCampaignSharedSetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCampaignSharedSetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignSharedSetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetResult} m MutateCampaignSharedSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignSharedSetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_shared_set != null && Object.hasOwnProperty.call(m, "campaign_shared_set"))
                                    $root.google.ads.googleads.v11.resources.CampaignSharedSet.encode(m.campaign_shared_set, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCampaignSharedSetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCampaignSharedSetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCampaignSharedSetResult} message MutateCampaignSharedSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCampaignSharedSetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCampaignSharedSetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetResult} MutateCampaignSharedSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignSharedSetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_shared_set = $root.google.ads.googleads.v11.resources.CampaignSharedSet.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCampaignSharedSetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetResult} MutateCampaignSharedSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCampaignSharedSetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCampaignSharedSetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCampaignSharedSetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_shared_set != null && m.hasOwnProperty("campaign_shared_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CampaignSharedSet.verify(m.campaign_shared_set);
                                        if (e)
                                            return "campaign_shared_set." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCampaignSharedSetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCampaignSharedSetResult} MutateCampaignSharedSetResult
                             */
                            MutateCampaignSharedSetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCampaignSharedSetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_shared_set != null) {
                                    if (typeof d.campaign_shared_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCampaignSharedSetResult.campaign_shared_set: object expected");
                                    m.campaign_shared_set = $root.google.ads.googleads.v11.resources.CampaignSharedSet.fromObject(d.campaign_shared_set);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCampaignSharedSetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCampaignSharedSetResult} m MutateCampaignSharedSetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCampaignSharedSetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.campaign_shared_set = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_shared_set != null && m.hasOwnProperty("campaign_shared_set")) {
                                    d.campaign_shared_set = $root.google.ads.googleads.v11.resources.CampaignSharedSet.toObject(m.campaign_shared_set, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCampaignSharedSetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCampaignSharedSetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCampaignSharedSetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCampaignSharedSetResult;
                        })();
    
                        services.ConversionActionService = (function() {
    
                            /**
                             * Constructs a new ConversionActionService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionActionService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ConversionActionService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ConversionActionService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ConversionActionService;
    
                            /**
                             * Creates new ConversionActionService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionActionService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ConversionActionService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ConversionActionService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionActionService|mutateConversionActions}.
                             * @memberof google.ads.googleads.v11.services.ConversionActionService
                             * @typedef MutateConversionActionsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateConversionActionsResponse} [response] MutateConversionActionsResponse
                             */
    
                            /**
                             * Calls MutateConversionActions.
                             * @function mutateConversionActions
                             * @memberof google.ads.googleads.v11.services.ConversionActionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsRequest} request MutateConversionActionsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionActionService.MutateConversionActionsCallback} callback Node-style callback called with the error, if any, and MutateConversionActionsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionActionService.prototype.mutateConversionActions = function mutateConversionActions(request, callback) {
                                return this.rpcCall(mutateConversionActions, $root.google.ads.googleads.v11.services.MutateConversionActionsRequest, $root.google.ads.googleads.v11.services.MutateConversionActionsResponse, request, callback);
                            }, "name", { value: "MutateConversionActions" });
    
                            /**
                             * Calls MutateConversionActions.
                             * @function mutateConversionActions
                             * @memberof google.ads.googleads.v11.services.ConversionActionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsRequest} request MutateConversionActionsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateConversionActionsResponse>} Promise
                             * @variation 2
                             */
    
                            return ConversionActionService;
                        })();
    
                        services.MutateConversionActionsRequest = (function() {
    
                            /**
                             * Properties of a MutateConversionActionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionActionsRequest
                             * @property {string|null} [customer_id] MutateConversionActionsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IConversionActionOperation>|null} [operations] MutateConversionActionsRequest operations
                             * @property {boolean|null} [partial_failure] MutateConversionActionsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateConversionActionsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateConversionActionsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateConversionActionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionActionsRequest.
                             * @implements IMutateConversionActionsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsRequest=} [p] Properties to set
                             */
                            function MutateConversionActionsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionActionsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @instance
                             */
                            MutateConversionActionsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateConversionActionsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IConversionActionOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @instance
                             */
                            MutateConversionActionsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateConversionActionsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @instance
                             */
                            MutateConversionActionsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateConversionActionsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @instance
                             */
                            MutateConversionActionsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateConversionActionsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @instance
                             */
                            MutateConversionActionsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateConversionActionsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsRequest} MutateConversionActionsRequest instance
                             */
                            MutateConversionActionsRequest.create = function create(properties) {
                                return new MutateConversionActionsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionActionsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionActionsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsRequest} m MutateConversionActionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionActionsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ConversionActionOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionActionsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionActionsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsRequest} message MutateConversionActionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionActionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionActionsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsRequest} MutateConversionActionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionActionsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionActionsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ConversionActionOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionActionsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsRequest} MutateConversionActionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionActionsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionActionsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionActionsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ConversionActionOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionActionsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsRequest} MutateConversionActionsRequest
                             */
                            MutateConversionActionsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionActionsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionActionsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionActionsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionActionsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ConversionActionOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionActionsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionActionsRequest} m MutateConversionActionsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionActionsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ConversionActionOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionActionsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionActionsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionActionsRequest;
                        })();
    
                        services.ConversionActionOperation = (function() {
    
                            /**
                             * Properties of a ConversionActionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IConversionActionOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ConversionActionOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IConversionAction|null} [create] ConversionActionOperation create
                             * @property {google.ads.googleads.v11.resources.IConversionAction|null} [update] ConversionActionOperation update
                             * @property {string|null} [remove] ConversionActionOperation remove
                             */
    
                            /**
                             * Constructs a new ConversionActionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionActionOperation.
                             * @implements IConversionActionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IConversionActionOperation=} [p] Properties to set
                             */
                            function ConversionActionOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionActionOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @instance
                             */
                            ConversionActionOperation.prototype.update_mask = null;
    
                            /**
                             * ConversionActionOperation create.
                             * @member {google.ads.googleads.v11.resources.IConversionAction|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @instance
                             */
                            ConversionActionOperation.prototype.create = null;
    
                            /**
                             * ConversionActionOperation update.
                             * @member {google.ads.googleads.v11.resources.IConversionAction|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @instance
                             */
                            ConversionActionOperation.prototype.update = null;
    
                            /**
                             * ConversionActionOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @instance
                             */
                            ConversionActionOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionActionOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @instance
                             */
                            Object.defineProperty(ConversionActionOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionActionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionActionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ConversionActionOperation} ConversionActionOperation instance
                             */
                            ConversionActionOperation.create = function create(properties) {
                                return new ConversionActionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionActionOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ConversionActionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionActionOperation} m ConversionActionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionActionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ConversionActionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionActionOperation} message ConversionActionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionActionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionActionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ConversionActionOperation} ConversionActionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ConversionActionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.ConversionAction.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.ConversionAction.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionActionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ConversionActionOperation} ConversionActionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionActionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionActionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionActionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionActionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ConversionActionOperation} ConversionActionOperation
                             */
                            ConversionActionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ConversionActionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ConversionActionOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionActionOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionActionOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.ConversionAction.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionActionOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.ConversionAction.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionActionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ConversionActionOperation} m ConversionActionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionActionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.ConversionAction.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.ConversionAction.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionActionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ConversionActionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionActionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionActionOperation;
                        })();
    
                        services.MutateConversionActionsResponse = (function() {
    
                            /**
                             * Properties of a MutateConversionActionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionActionsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateConversionActionsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateConversionActionResult>|null} [results] MutateConversionActionsResponse results
                             */
    
                            /**
                             * Constructs a new MutateConversionActionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionActionsResponse.
                             * @implements IMutateConversionActionsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsResponse=} [p] Properties to set
                             */
                            function MutateConversionActionsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionActionsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @instance
                             */
                            MutateConversionActionsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateConversionActionsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateConversionActionResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @instance
                             */
                            MutateConversionActionsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateConversionActionsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsResponse} MutateConversionActionsResponse instance
                             */
                            MutateConversionActionsResponse.create = function create(properties) {
                                return new MutateConversionActionsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionActionsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionActionsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsResponse} m MutateConversionActionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionActionsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateConversionActionResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionActionsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionActionsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionsResponse} message MutateConversionActionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionActionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionActionsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsResponse} MutateConversionActionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionActionsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionActionsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateConversionActionResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionActionsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsResponse} MutateConversionActionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionActionsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionActionsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionActionsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateConversionActionResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionActionsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionsResponse} MutateConversionActionsResponse
                             */
                            MutateConversionActionsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionActionsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionActionsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionActionsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionActionsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionActionsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateConversionActionResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionActionsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionActionsResponse} m MutateConversionActionsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionActionsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateConversionActionResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionActionsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionActionsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionActionsResponse;
                        })();
    
                        services.MutateConversionActionResult = (function() {
    
                            /**
                             * Properties of a MutateConversionActionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionActionResult
                             * @property {string|null} [resource_name] MutateConversionActionResult resource_name
                             * @property {google.ads.googleads.v11.resources.IConversionAction|null} [conversion_action] MutateConversionActionResult conversion_action
                             */
    
                            /**
                             * Constructs a new MutateConversionActionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionActionResult.
                             * @implements IMutateConversionActionResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionResult=} [p] Properties to set
                             */
                            function MutateConversionActionResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionActionResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @instance
                             */
                            MutateConversionActionResult.prototype.resource_name = "";
    
                            /**
                             * MutateConversionActionResult conversion_action.
                             * @member {google.ads.googleads.v11.resources.IConversionAction|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @instance
                             */
                            MutateConversionActionResult.prototype.conversion_action = null;
    
                            /**
                             * Creates a new MutateConversionActionResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionResult} MutateConversionActionResult instance
                             */
                            MutateConversionActionResult.create = function create(properties) {
                                return new MutateConversionActionResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionActionResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionActionResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionResult} m MutateConversionActionResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionActionResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    $root.google.ads.googleads.v11.resources.ConversionAction.encode(m.conversion_action, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionActionResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionActionResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionActionResult} message MutateConversionActionResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionActionResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionActionResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionResult} MutateConversionActionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionActionResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionActionResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.conversion_action = $root.google.ads.googleads.v11.resources.ConversionAction.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionActionResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionResult} MutateConversionActionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionActionResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionActionResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionActionResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionAction.verify(m.conversion_action);
                                        if (e)
                                            return "conversion_action." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionActionResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionActionResult} MutateConversionActionResult
                             */
                            MutateConversionActionResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionActionResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionActionResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.conversion_action != null) {
                                    if (typeof d.conversion_action !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionActionResult.conversion_action: object expected");
                                    m.conversion_action = $root.google.ads.googleads.v11.resources.ConversionAction.fromObject(d.conversion_action);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionActionResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionActionResult} m MutateConversionActionResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionActionResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.conversion_action = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = $root.google.ads.googleads.v11.resources.ConversionAction.toObject(m.conversion_action, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionActionResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionActionResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionActionResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionActionResult;
                        })();
    
                        services.ConversionCustomVariableService = (function() {
    
                            /**
                             * Constructs a new ConversionCustomVariableService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionCustomVariableService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ConversionCustomVariableService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ConversionCustomVariableService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ConversionCustomVariableService;
    
                            /**
                             * Creates new ConversionCustomVariableService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ConversionCustomVariableService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ConversionCustomVariableService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionCustomVariableService|mutateConversionCustomVariables}.
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableService
                             * @typedef MutateConversionCustomVariablesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse} [response] MutateConversionCustomVariablesResponse
                             */
    
                            /**
                             * Calls MutateConversionCustomVariables.
                             * @function mutateConversionCustomVariables
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesRequest} request MutateConversionCustomVariablesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionCustomVariableService.MutateConversionCustomVariablesCallback} callback Node-style callback called with the error, if any, and MutateConversionCustomVariablesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionCustomVariableService.prototype.mutateConversionCustomVariables = function mutateConversionCustomVariables(request, callback) {
                                return this.rpcCall(mutateConversionCustomVariables, $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest, $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse, request, callback);
                            }, "name", { value: "MutateConversionCustomVariables" });
    
                            /**
                             * Calls MutateConversionCustomVariables.
                             * @function mutateConversionCustomVariables
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesRequest} request MutateConversionCustomVariablesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse>} Promise
                             * @variation 2
                             */
    
                            return ConversionCustomVariableService;
                        })();
    
                        services.MutateConversionCustomVariablesRequest = (function() {
    
                            /**
                             * Properties of a MutateConversionCustomVariablesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionCustomVariablesRequest
                             * @property {string|null} [customer_id] MutateConversionCustomVariablesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IConversionCustomVariableOperation>|null} [operations] MutateConversionCustomVariablesRequest operations
                             * @property {boolean|null} [partial_failure] MutateConversionCustomVariablesRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateConversionCustomVariablesRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateConversionCustomVariablesRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateConversionCustomVariablesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionCustomVariablesRequest.
                             * @implements IMutateConversionCustomVariablesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesRequest=} [p] Properties to set
                             */
                            function MutateConversionCustomVariablesRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionCustomVariablesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @instance
                             */
                            MutateConversionCustomVariablesRequest.prototype.customer_id = "";
    
                            /**
                             * MutateConversionCustomVariablesRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IConversionCustomVariableOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @instance
                             */
                            MutateConversionCustomVariablesRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateConversionCustomVariablesRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @instance
                             */
                            MutateConversionCustomVariablesRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateConversionCustomVariablesRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @instance
                             */
                            MutateConversionCustomVariablesRequest.prototype.validate_only = false;
    
                            /**
                             * MutateConversionCustomVariablesRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @instance
                             */
                            MutateConversionCustomVariablesRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateConversionCustomVariablesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest} MutateConversionCustomVariablesRequest instance
                             */
                            MutateConversionCustomVariablesRequest.create = function create(properties) {
                                return new MutateConversionCustomVariablesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionCustomVariablesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesRequest} m MutateConversionCustomVariablesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionCustomVariablesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionCustomVariablesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesRequest} message MutateConversionCustomVariablesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionCustomVariablesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionCustomVariablesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest} MutateConversionCustomVariablesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionCustomVariablesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionCustomVariablesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest} MutateConversionCustomVariablesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionCustomVariablesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionCustomVariablesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionCustomVariablesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionCustomVariablesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest} MutateConversionCustomVariablesRequest
                             */
                            MutateConversionCustomVariablesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionCustomVariablesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest} m MutateConversionCustomVariablesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionCustomVariablesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionCustomVariablesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionCustomVariablesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionCustomVariablesRequest;
                        })();
    
                        services.ConversionCustomVariableOperation = (function() {
    
                            /**
                             * Properties of a ConversionCustomVariableOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IConversionCustomVariableOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ConversionCustomVariableOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IConversionCustomVariable|null} [create] ConversionCustomVariableOperation create
                             * @property {google.ads.googleads.v11.resources.IConversionCustomVariable|null} [update] ConversionCustomVariableOperation update
                             */
    
                            /**
                             * Constructs a new ConversionCustomVariableOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionCustomVariableOperation.
                             * @implements IConversionCustomVariableOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IConversionCustomVariableOperation=} [p] Properties to set
                             */
                            function ConversionCustomVariableOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionCustomVariableOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @instance
                             */
                            ConversionCustomVariableOperation.prototype.update_mask = null;
    
                            /**
                             * ConversionCustomVariableOperation create.
                             * @member {google.ads.googleads.v11.resources.IConversionCustomVariable|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @instance
                             */
                            ConversionCustomVariableOperation.prototype.create = null;
    
                            /**
                             * ConversionCustomVariableOperation update.
                             * @member {google.ads.googleads.v11.resources.IConversionCustomVariable|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @instance
                             */
                            ConversionCustomVariableOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionCustomVariableOperation operation.
                             * @member {"create"|"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @instance
                             */
                            Object.defineProperty(ConversionCustomVariableOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionCustomVariableOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionCustomVariableOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ConversionCustomVariableOperation} ConversionCustomVariableOperation instance
                             */
                            ConversionCustomVariableOperation.create = function create(properties) {
                                return new ConversionCustomVariableOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariableOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ConversionCustomVariableOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionCustomVariableOperation} m ConversionCustomVariableOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariableOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.ConversionCustomVariable.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.ConversionCustomVariable.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionCustomVariableOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ConversionCustomVariableOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionCustomVariableOperation} message ConversionCustomVariableOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionCustomVariableOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionCustomVariableOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ConversionCustomVariableOperation} ConversionCustomVariableOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariableOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionCustomVariableOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ConversionCustomVariableOperation} ConversionCustomVariableOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionCustomVariableOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionCustomVariableOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionCustomVariableOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionCustomVariableOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ConversionCustomVariableOperation} ConversionCustomVariableOperation
                             */
                            ConversionCustomVariableOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ConversionCustomVariableOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionCustomVariableOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionCustomVariableOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionCustomVariableOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionCustomVariableOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ConversionCustomVariableOperation} m ConversionCustomVariableOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionCustomVariableOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionCustomVariableOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ConversionCustomVariableOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionCustomVariableOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionCustomVariableOperation;
                        })();
    
                        services.MutateConversionCustomVariablesResponse = (function() {
    
                            /**
                             * Properties of a MutateConversionCustomVariablesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionCustomVariablesResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateConversionCustomVariablesResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateConversionCustomVariableResult>|null} [results] MutateConversionCustomVariablesResponse results
                             */
    
                            /**
                             * Constructs a new MutateConversionCustomVariablesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionCustomVariablesResponse.
                             * @implements IMutateConversionCustomVariablesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesResponse=} [p] Properties to set
                             */
                            function MutateConversionCustomVariablesResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionCustomVariablesResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @instance
                             */
                            MutateConversionCustomVariablesResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateConversionCustomVariablesResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateConversionCustomVariableResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @instance
                             */
                            MutateConversionCustomVariablesResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateConversionCustomVariablesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse} MutateConversionCustomVariablesResponse instance
                             */
                            MutateConversionCustomVariablesResponse.create = function create(properties) {
                                return new MutateConversionCustomVariablesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionCustomVariablesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesResponse} m MutateConversionCustomVariablesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionCustomVariablesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionCustomVariablesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariablesResponse} message MutateConversionCustomVariablesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionCustomVariablesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionCustomVariablesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse} MutateConversionCustomVariablesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionCustomVariablesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionCustomVariablesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse} MutateConversionCustomVariablesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionCustomVariablesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionCustomVariablesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionCustomVariablesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionCustomVariablesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse} MutateConversionCustomVariablesResponse
                             */
                            MutateConversionCustomVariablesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionCustomVariablesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse} m MutateConversionCustomVariablesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionCustomVariablesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionCustomVariablesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariablesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionCustomVariablesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionCustomVariablesResponse;
                        })();
    
                        services.MutateConversionCustomVariableResult = (function() {
    
                            /**
                             * Properties of a MutateConversionCustomVariableResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionCustomVariableResult
                             * @property {string|null} [resource_name] MutateConversionCustomVariableResult resource_name
                             * @property {google.ads.googleads.v11.resources.IConversionCustomVariable|null} [conversion_custom_variable] MutateConversionCustomVariableResult conversion_custom_variable
                             */
    
                            /**
                             * Constructs a new MutateConversionCustomVariableResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionCustomVariableResult.
                             * @implements IMutateConversionCustomVariableResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariableResult=} [p] Properties to set
                             */
                            function MutateConversionCustomVariableResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionCustomVariableResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @instance
                             */
                            MutateConversionCustomVariableResult.prototype.resource_name = "";
    
                            /**
                             * MutateConversionCustomVariableResult conversion_custom_variable.
                             * @member {google.ads.googleads.v11.resources.IConversionCustomVariable|null|undefined} conversion_custom_variable
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @instance
                             */
                            MutateConversionCustomVariableResult.prototype.conversion_custom_variable = null;
    
                            /**
                             * Creates a new MutateConversionCustomVariableResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariableResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariableResult} MutateConversionCustomVariableResult instance
                             */
                            MutateConversionCustomVariableResult.create = function create(properties) {
                                return new MutateConversionCustomVariableResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionCustomVariableResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionCustomVariableResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariableResult} m MutateConversionCustomVariableResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionCustomVariableResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.conversion_custom_variable != null && Object.hasOwnProperty.call(m, "conversion_custom_variable"))
                                    $root.google.ads.googleads.v11.resources.ConversionCustomVariable.encode(m.conversion_custom_variable, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionCustomVariableResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionCustomVariableResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionCustomVariableResult} message MutateConversionCustomVariableResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionCustomVariableResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionCustomVariableResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariableResult} MutateConversionCustomVariableResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionCustomVariableResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.conversion_custom_variable = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionCustomVariableResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariableResult} MutateConversionCustomVariableResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionCustomVariableResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionCustomVariableResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionCustomVariableResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.conversion_custom_variable != null && m.hasOwnProperty("conversion_custom_variable")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.verify(m.conversion_custom_variable);
                                        if (e)
                                            return "conversion_custom_variable." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionCustomVariableResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionCustomVariableResult} MutateConversionCustomVariableResult
                             */
                            MutateConversionCustomVariableResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionCustomVariableResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.conversion_custom_variable != null) {
                                    if (typeof d.conversion_custom_variable !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionCustomVariableResult.conversion_custom_variable: object expected");
                                    m.conversion_custom_variable = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.fromObject(d.conversion_custom_variable);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionCustomVariableResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionCustomVariableResult} m MutateConversionCustomVariableResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionCustomVariableResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.conversion_custom_variable = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.conversion_custom_variable != null && m.hasOwnProperty("conversion_custom_variable")) {
                                    d.conversion_custom_variable = $root.google.ads.googleads.v11.resources.ConversionCustomVariable.toObject(m.conversion_custom_variable, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionCustomVariableResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionCustomVariableResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionCustomVariableResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionCustomVariableResult;
                        })();
    
                        services.ConversionGoalCampaignConfigService = (function() {
    
                            /**
                             * Constructs a new ConversionGoalCampaignConfigService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionGoalCampaignConfigService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ConversionGoalCampaignConfigService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ConversionGoalCampaignConfigService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ConversionGoalCampaignConfigService;
    
                            /**
                             * Creates new ConversionGoalCampaignConfigService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ConversionGoalCampaignConfigService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ConversionGoalCampaignConfigService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionGoalCampaignConfigService|mutateConversionGoalCampaignConfigs}.
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigService
                             * @typedef MutateConversionGoalCampaignConfigsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse} [response] MutateConversionGoalCampaignConfigsResponse
                             */
    
                            /**
                             * Calls MutateConversionGoalCampaignConfigs.
                             * @function mutateConversionGoalCampaignConfigs
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsRequest} request MutateConversionGoalCampaignConfigsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionGoalCampaignConfigService.MutateConversionGoalCampaignConfigsCallback} callback Node-style callback called with the error, if any, and MutateConversionGoalCampaignConfigsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionGoalCampaignConfigService.prototype.mutateConversionGoalCampaignConfigs = function mutateConversionGoalCampaignConfigs(request, callback) {
                                return this.rpcCall(mutateConversionGoalCampaignConfigs, $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest, $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse, request, callback);
                            }, "name", { value: "MutateConversionGoalCampaignConfigs" });
    
                            /**
                             * Calls MutateConversionGoalCampaignConfigs.
                             * @function mutateConversionGoalCampaignConfigs
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsRequest} request MutateConversionGoalCampaignConfigsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse>} Promise
                             * @variation 2
                             */
    
                            return ConversionGoalCampaignConfigService;
                        })();
    
                        services.MutateConversionGoalCampaignConfigsRequest = (function() {
    
                            /**
                             * Properties of a MutateConversionGoalCampaignConfigsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionGoalCampaignConfigsRequest
                             * @property {string|null} [customer_id] MutateConversionGoalCampaignConfigsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation>|null} [operations] MutateConversionGoalCampaignConfigsRequest operations
                             * @property {boolean|null} [validate_only] MutateConversionGoalCampaignConfigsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateConversionGoalCampaignConfigsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateConversionGoalCampaignConfigsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionGoalCampaignConfigsRequest.
                             * @implements IMutateConversionGoalCampaignConfigsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsRequest=} [p] Properties to set
                             */
                            function MutateConversionGoalCampaignConfigsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionGoalCampaignConfigsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @instance
                             */
                            MutateConversionGoalCampaignConfigsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateConversionGoalCampaignConfigsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @instance
                             */
                            MutateConversionGoalCampaignConfigsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateConversionGoalCampaignConfigsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @instance
                             */
                            MutateConversionGoalCampaignConfigsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateConversionGoalCampaignConfigsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @instance
                             */
                            MutateConversionGoalCampaignConfigsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateConversionGoalCampaignConfigsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest} MutateConversionGoalCampaignConfigsRequest instance
                             */
                            MutateConversionGoalCampaignConfigsRequest.create = function create(properties) {
                                return new MutateConversionGoalCampaignConfigsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionGoalCampaignConfigsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsRequest} m MutateConversionGoalCampaignConfigsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionGoalCampaignConfigsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(32).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionGoalCampaignConfigsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsRequest} message MutateConversionGoalCampaignConfigsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionGoalCampaignConfigsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionGoalCampaignConfigsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest} MutateConversionGoalCampaignConfigsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionGoalCampaignConfigsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    case 4:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionGoalCampaignConfigsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest} MutateConversionGoalCampaignConfigsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionGoalCampaignConfigsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionGoalCampaignConfigsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionGoalCampaignConfigsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionGoalCampaignConfigsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest} MutateConversionGoalCampaignConfigsRequest
                             */
                            MutateConversionGoalCampaignConfigsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionGoalCampaignConfigsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest} m MutateConversionGoalCampaignConfigsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionGoalCampaignConfigsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionGoalCampaignConfigsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionGoalCampaignConfigsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionGoalCampaignConfigsRequest;
                        })();
    
                        services.ConversionGoalCampaignConfigOperation = (function() {
    
                            /**
                             * Properties of a ConversionGoalCampaignConfigOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IConversionGoalCampaignConfigOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ConversionGoalCampaignConfigOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig|null} [update] ConversionGoalCampaignConfigOperation update
                             */
    
                            /**
                             * Constructs a new ConversionGoalCampaignConfigOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionGoalCampaignConfigOperation.
                             * @implements IConversionGoalCampaignConfigOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation=} [p] Properties to set
                             */
                            function ConversionGoalCampaignConfigOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionGoalCampaignConfigOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @instance
                             */
                            ConversionGoalCampaignConfigOperation.prototype.update_mask = null;
    
                            /**
                             * ConversionGoalCampaignConfigOperation update.
                             * @member {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @instance
                             */
                            ConversionGoalCampaignConfigOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionGoalCampaignConfigOperation operation.
                             * @member {"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @instance
                             */
                            Object.defineProperty(ConversionGoalCampaignConfigOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionGoalCampaignConfigOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation} ConversionGoalCampaignConfigOperation instance
                             */
                            ConversionGoalCampaignConfigOperation.create = function create(properties) {
                                return new ConversionGoalCampaignConfigOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionGoalCampaignConfigOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation} m ConversionGoalCampaignConfigOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionGoalCampaignConfigOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionGoalCampaignConfigOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionGoalCampaignConfigOperation} message ConversionGoalCampaignConfigOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionGoalCampaignConfigOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionGoalCampaignConfigOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation} ConversionGoalCampaignConfigOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionGoalCampaignConfigOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionGoalCampaignConfigOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation} ConversionGoalCampaignConfigOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionGoalCampaignConfigOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionGoalCampaignConfigOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionGoalCampaignConfigOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionGoalCampaignConfigOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation} ConversionGoalCampaignConfigOperation
                             */
                            ConversionGoalCampaignConfigOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionGoalCampaignConfigOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation} m ConversionGoalCampaignConfigOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionGoalCampaignConfigOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionGoalCampaignConfigOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ConversionGoalCampaignConfigOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionGoalCampaignConfigOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionGoalCampaignConfigOperation;
                        })();
    
                        services.MutateConversionGoalCampaignConfigsResponse = (function() {
    
                            /**
                             * Properties of a MutateConversionGoalCampaignConfigsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionGoalCampaignConfigsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult>|null} [results] MutateConversionGoalCampaignConfigsResponse results
                             */
    
                            /**
                             * Constructs a new MutateConversionGoalCampaignConfigsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionGoalCampaignConfigsResponse.
                             * @implements IMutateConversionGoalCampaignConfigsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsResponse=} [p] Properties to set
                             */
                            function MutateConversionGoalCampaignConfigsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionGoalCampaignConfigsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @instance
                             */
                            MutateConversionGoalCampaignConfigsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateConversionGoalCampaignConfigsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse} MutateConversionGoalCampaignConfigsResponse instance
                             */
                            MutateConversionGoalCampaignConfigsResponse.create = function create(properties) {
                                return new MutateConversionGoalCampaignConfigsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionGoalCampaignConfigsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsResponse} m MutateConversionGoalCampaignConfigsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionGoalCampaignConfigsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionGoalCampaignConfigsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigsResponse} message MutateConversionGoalCampaignConfigsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionGoalCampaignConfigsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionGoalCampaignConfigsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse} MutateConversionGoalCampaignConfigsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionGoalCampaignConfigsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionGoalCampaignConfigsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse} MutateConversionGoalCampaignConfigsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionGoalCampaignConfigsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionGoalCampaignConfigsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionGoalCampaignConfigsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionGoalCampaignConfigsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse} MutateConversionGoalCampaignConfigsResponse
                             */
                            MutateConversionGoalCampaignConfigsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionGoalCampaignConfigsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse} m MutateConversionGoalCampaignConfigsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionGoalCampaignConfigsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionGoalCampaignConfigsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionGoalCampaignConfigsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionGoalCampaignConfigsResponse;
                        })();
    
                        services.MutateConversionGoalCampaignConfigResult = (function() {
    
                            /**
                             * Properties of a MutateConversionGoalCampaignConfigResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionGoalCampaignConfigResult
                             * @property {string|null} [resource_name] MutateConversionGoalCampaignConfigResult resource_name
                             * @property {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig|null} [conversion_goal_campaign_config] MutateConversionGoalCampaignConfigResult conversion_goal_campaign_config
                             */
    
                            /**
                             * Constructs a new MutateConversionGoalCampaignConfigResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionGoalCampaignConfigResult.
                             * @implements IMutateConversionGoalCampaignConfigResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult=} [p] Properties to set
                             */
                            function MutateConversionGoalCampaignConfigResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionGoalCampaignConfigResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @instance
                             */
                            MutateConversionGoalCampaignConfigResult.prototype.resource_name = "";
    
                            /**
                             * MutateConversionGoalCampaignConfigResult conversion_goal_campaign_config.
                             * @member {google.ads.googleads.v11.resources.IConversionGoalCampaignConfig|null|undefined} conversion_goal_campaign_config
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @instance
                             */
                            MutateConversionGoalCampaignConfigResult.prototype.conversion_goal_campaign_config = null;
    
                            /**
                             * Creates a new MutateConversionGoalCampaignConfigResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult} MutateConversionGoalCampaignConfigResult instance
                             */
                            MutateConversionGoalCampaignConfigResult.create = function create(properties) {
                                return new MutateConversionGoalCampaignConfigResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionGoalCampaignConfigResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult} m MutateConversionGoalCampaignConfigResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionGoalCampaignConfigResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.conversion_goal_campaign_config != null && Object.hasOwnProperty.call(m, "conversion_goal_campaign_config"))
                                    $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.encode(m.conversion_goal_campaign_config, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionGoalCampaignConfigResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionGoalCampaignConfigResult} message MutateConversionGoalCampaignConfigResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionGoalCampaignConfigResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionGoalCampaignConfigResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult} MutateConversionGoalCampaignConfigResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionGoalCampaignConfigResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.conversion_goal_campaign_config = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionGoalCampaignConfigResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult} MutateConversionGoalCampaignConfigResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionGoalCampaignConfigResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionGoalCampaignConfigResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionGoalCampaignConfigResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.conversion_goal_campaign_config != null && m.hasOwnProperty("conversion_goal_campaign_config")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.verify(m.conversion_goal_campaign_config);
                                        if (e)
                                            return "conversion_goal_campaign_config." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionGoalCampaignConfigResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult} MutateConversionGoalCampaignConfigResult
                             */
                            MutateConversionGoalCampaignConfigResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.conversion_goal_campaign_config != null) {
                                    if (typeof d.conversion_goal_campaign_config !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult.conversion_goal_campaign_config: object expected");
                                    m.conversion_goal_campaign_config = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.fromObject(d.conversion_goal_campaign_config);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionGoalCampaignConfigResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult} m MutateConversionGoalCampaignConfigResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionGoalCampaignConfigResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.conversion_goal_campaign_config = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.conversion_goal_campaign_config != null && m.hasOwnProperty("conversion_goal_campaign_config")) {
                                    d.conversion_goal_campaign_config = $root.google.ads.googleads.v11.resources.ConversionGoalCampaignConfig.toObject(m.conversion_goal_campaign_config, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionGoalCampaignConfigResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionGoalCampaignConfigResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionGoalCampaignConfigResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionGoalCampaignConfigResult;
                        })();
    
                        services.ConversionValueRuleService = (function() {
    
                            /**
                             * Constructs a new ConversionValueRuleService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionValueRuleService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ConversionValueRuleService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ConversionValueRuleService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ConversionValueRuleService;
    
                            /**
                             * Creates new ConversionValueRuleService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ConversionValueRuleService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ConversionValueRuleService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionValueRuleService|mutateConversionValueRules}.
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleService
                             * @typedef MutateConversionValueRulesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRulesResponse} [response] MutateConversionValueRulesResponse
                             */
    
                            /**
                             * Calls MutateConversionValueRules.
                             * @function mutateConversionValueRules
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesRequest} request MutateConversionValueRulesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionValueRuleService.MutateConversionValueRulesCallback} callback Node-style callback called with the error, if any, and MutateConversionValueRulesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionValueRuleService.prototype.mutateConversionValueRules = function mutateConversionValueRules(request, callback) {
                                return this.rpcCall(mutateConversionValueRules, $root.google.ads.googleads.v11.services.MutateConversionValueRulesRequest, $root.google.ads.googleads.v11.services.MutateConversionValueRulesResponse, request, callback);
                            }, "name", { value: "MutateConversionValueRules" });
    
                            /**
                             * Calls MutateConversionValueRules.
                             * @function mutateConversionValueRules
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesRequest} request MutateConversionValueRulesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateConversionValueRulesResponse>} Promise
                             * @variation 2
                             */
    
                            return ConversionValueRuleService;
                        })();
    
                        services.MutateConversionValueRulesRequest = (function() {
    
                            /**
                             * Properties of a MutateConversionValueRulesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionValueRulesRequest
                             * @property {string|null} [customer_id] MutateConversionValueRulesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IConversionValueRuleOperation>|null} [operations] MutateConversionValueRulesRequest operations
                             * @property {boolean|null} [partial_failure] MutateConversionValueRulesRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateConversionValueRulesRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateConversionValueRulesRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateConversionValueRulesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionValueRulesRequest.
                             * @implements IMutateConversionValueRulesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesRequest=} [p] Properties to set
                             */
                            function MutateConversionValueRulesRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionValueRulesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @instance
                             */
                            MutateConversionValueRulesRequest.prototype.customer_id = "";
    
                            /**
                             * MutateConversionValueRulesRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IConversionValueRuleOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @instance
                             */
                            MutateConversionValueRulesRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateConversionValueRulesRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @instance
                             */
                            MutateConversionValueRulesRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateConversionValueRulesRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @instance
                             */
                            MutateConversionValueRulesRequest.prototype.validate_only = false;
    
                            /**
                             * MutateConversionValueRulesRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @instance
                             */
                            MutateConversionValueRulesRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateConversionValueRulesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesRequest} MutateConversionValueRulesRequest instance
                             */
                            MutateConversionValueRulesRequest.create = function create(properties) {
                                return new MutateConversionValueRulesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRulesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRulesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesRequest} m MutateConversionValueRulesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRulesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(32).int32(m.response_content_type);
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(40).bool(m.partial_failure);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRulesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRulesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesRequest} message MutateConversionValueRulesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRulesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionValueRulesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesRequest} MutateConversionValueRulesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRulesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionValueRulesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ConversionValueRuleOperation.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    case 4:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionValueRulesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesRequest} MutateConversionValueRulesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRulesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionValueRulesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionValueRulesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionValueRulesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesRequest} MutateConversionValueRulesRequest
                             */
                            MutateConversionValueRulesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionValueRulesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionValueRulesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRulesRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRulesRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionValueRulesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRulesRequest} m MutateConversionValueRulesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionValueRulesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.partial_failure = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ConversionValueRuleOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionValueRulesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionValueRulesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionValueRulesRequest;
                        })();
    
                        services.ConversionValueRuleOperation = (function() {
    
                            /**
                             * Properties of a ConversionValueRuleOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IConversionValueRuleOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ConversionValueRuleOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IConversionValueRule|null} [create] ConversionValueRuleOperation create
                             * @property {google.ads.googleads.v11.resources.IConversionValueRule|null} [update] ConversionValueRuleOperation update
                             * @property {string|null} [remove] ConversionValueRuleOperation remove
                             */
    
                            /**
                             * Constructs a new ConversionValueRuleOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionValueRuleOperation.
                             * @implements IConversionValueRuleOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleOperation=} [p] Properties to set
                             */
                            function ConversionValueRuleOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionValueRuleOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @instance
                             */
                            ConversionValueRuleOperation.prototype.update_mask = null;
    
                            /**
                             * ConversionValueRuleOperation create.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRule|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @instance
                             */
                            ConversionValueRuleOperation.prototype.create = null;
    
                            /**
                             * ConversionValueRuleOperation update.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRule|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @instance
                             */
                            ConversionValueRuleOperation.prototype.update = null;
    
                            /**
                             * ConversionValueRuleOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @instance
                             */
                            ConversionValueRuleOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionValueRuleOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @instance
                             */
                            Object.defineProperty(ConversionValueRuleOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionValueRuleOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleOperation} ConversionValueRuleOperation instance
                             */
                            ConversionValueRuleOperation.create = function create(properties) {
                                return new ConversionValueRuleOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ConversionValueRuleOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleOperation} m ConversionValueRuleOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ConversionValueRuleOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleOperation} message ConversionValueRuleOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRuleOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleOperation} ConversionValueRuleOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ConversionValueRuleOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.ConversionValueRule.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.ConversionValueRule.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRuleOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleOperation} ConversionValueRuleOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRuleOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRuleOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRuleOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleOperation} ConversionValueRuleOperation
                             */
                            ConversionValueRuleOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ConversionValueRuleOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ConversionValueRuleOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionValueRuleOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionValueRuleOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.ConversionValueRule.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionValueRuleOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.ConversionValueRule.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRuleOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ConversionValueRuleOperation} m ConversionValueRuleOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRuleOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.ConversionValueRule.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.ConversionValueRule.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionValueRuleOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRuleOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionValueRuleOperation;
                        })();
    
                        services.MutateConversionValueRulesResponse = (function() {
    
                            /**
                             * Properties of a MutateConversionValueRulesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionValueRulesResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateConversionValueRuleResult>|null} [results] MutateConversionValueRulesResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateConversionValueRulesResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateConversionValueRulesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionValueRulesResponse.
                             * @implements IMutateConversionValueRulesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesResponse=} [p] Properties to set
                             */
                            function MutateConversionValueRulesResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionValueRulesResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateConversionValueRuleResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @instance
                             */
                            MutateConversionValueRulesResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateConversionValueRulesResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @instance
                             */
                            MutateConversionValueRulesResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateConversionValueRulesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesResponse} MutateConversionValueRulesResponse instance
                             */
                            MutateConversionValueRulesResponse.create = function create(properties) {
                                return new MutateConversionValueRulesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRulesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRulesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesResponse} m MutateConversionValueRulesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRulesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRulesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRulesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRulesResponse} message MutateConversionValueRulesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRulesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionValueRulesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesResponse} MutateConversionValueRulesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRulesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionValueRulesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionValueRulesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesResponse} MutateConversionValueRulesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRulesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionValueRulesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionValueRulesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionValueRulesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRulesResponse} MutateConversionValueRulesResponse
                             */
                            MutateConversionValueRulesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionValueRulesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionValueRulesResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRulesResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRulesResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRulesResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionValueRulesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRulesResponse} m MutateConversionValueRulesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionValueRulesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionValueRulesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRulesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionValueRulesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionValueRulesResponse;
                        })();
    
                        services.MutateConversionValueRuleResult = (function() {
    
                            /**
                             * Properties of a MutateConversionValueRuleResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionValueRuleResult
                             * @property {string|null} [resource_name] MutateConversionValueRuleResult resource_name
                             * @property {google.ads.googleads.v11.resources.IConversionValueRule|null} [conversion_value_rule] MutateConversionValueRuleResult conversion_value_rule
                             */
    
                            /**
                             * Constructs a new MutateConversionValueRuleResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionValueRuleResult.
                             * @implements IMutateConversionValueRuleResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleResult=} [p] Properties to set
                             */
                            function MutateConversionValueRuleResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionValueRuleResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @instance
                             */
                            MutateConversionValueRuleResult.prototype.resource_name = "";
    
                            /**
                             * MutateConversionValueRuleResult conversion_value_rule.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRule|null|undefined} conversion_value_rule
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @instance
                             */
                            MutateConversionValueRuleResult.prototype.conversion_value_rule = null;
    
                            /**
                             * Creates a new MutateConversionValueRuleResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleResult} MutateConversionValueRuleResult instance
                             */
                            MutateConversionValueRuleResult.create = function create(properties) {
                                return new MutateConversionValueRuleResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleResult} m MutateConversionValueRuleResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.conversion_value_rule != null && Object.hasOwnProperty.call(m, "conversion_value_rule"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRule.encode(m.conversion_value_rule, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleResult} message MutateConversionValueRuleResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleResult} MutateConversionValueRuleResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.conversion_value_rule = $root.google.ads.googleads.v11.resources.ConversionValueRule.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleResult} MutateConversionValueRuleResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionValueRuleResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionValueRuleResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.conversion_value_rule != null && m.hasOwnProperty("conversion_value_rule")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRule.verify(m.conversion_value_rule);
                                        if (e)
                                            return "conversion_value_rule." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionValueRuleResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleResult} MutateConversionValueRuleResult
                             */
                            MutateConversionValueRuleResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.conversion_value_rule != null) {
                                    if (typeof d.conversion_value_rule !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRuleResult.conversion_value_rule: object expected");
                                    m.conversion_value_rule = $root.google.ads.googleads.v11.resources.ConversionValueRule.fromObject(d.conversion_value_rule);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionValueRuleResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRuleResult} m MutateConversionValueRuleResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionValueRuleResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.conversion_value_rule = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.conversion_value_rule != null && m.hasOwnProperty("conversion_value_rule")) {
                                    d.conversion_value_rule = $root.google.ads.googleads.v11.resources.ConversionValueRule.toObject(m.conversion_value_rule, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionValueRuleResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionValueRuleResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionValueRuleResult;
                        })();
    
                        services.ConversionValueRuleSetService = (function() {
    
                            /**
                             * Constructs a new ConversionValueRuleSetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionValueRuleSetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ConversionValueRuleSetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ConversionValueRuleSetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ConversionValueRuleSetService;
    
                            /**
                             * Creates new ConversionValueRuleSetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ConversionValueRuleSetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ConversionValueRuleSetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionValueRuleSetService|mutateConversionValueRuleSets}.
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetService
                             * @typedef MutateConversionValueRuleSetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse} [response] MutateConversionValueRuleSetsResponse
                             */
    
                            /**
                             * Calls MutateConversionValueRuleSets.
                             * @function mutateConversionValueRuleSets
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsRequest} request MutateConversionValueRuleSetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionValueRuleSetService.MutateConversionValueRuleSetsCallback} callback Node-style callback called with the error, if any, and MutateConversionValueRuleSetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionValueRuleSetService.prototype.mutateConversionValueRuleSets = function mutateConversionValueRuleSets(request, callback) {
                                return this.rpcCall(mutateConversionValueRuleSets, $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest, $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse, request, callback);
                            }, "name", { value: "MutateConversionValueRuleSets" });
    
                            /**
                             * Calls MutateConversionValueRuleSets.
                             * @function mutateConversionValueRuleSets
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsRequest} request MutateConversionValueRuleSetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse>} Promise
                             * @variation 2
                             */
    
                            return ConversionValueRuleSetService;
                        })();
    
                        services.MutateConversionValueRuleSetsRequest = (function() {
    
                            /**
                             * Properties of a MutateConversionValueRuleSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionValueRuleSetsRequest
                             * @property {string|null} [customer_id] MutateConversionValueRuleSetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IConversionValueRuleSetOperation>|null} [operations] MutateConversionValueRuleSetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateConversionValueRuleSetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateConversionValueRuleSetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateConversionValueRuleSetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateConversionValueRuleSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionValueRuleSetsRequest.
                             * @implements IMutateConversionValueRuleSetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsRequest=} [p] Properties to set
                             */
                            function MutateConversionValueRuleSetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionValueRuleSetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @instance
                             */
                            MutateConversionValueRuleSetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateConversionValueRuleSetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IConversionValueRuleSetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @instance
                             */
                            MutateConversionValueRuleSetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateConversionValueRuleSetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @instance
                             */
                            MutateConversionValueRuleSetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateConversionValueRuleSetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @instance
                             */
                            MutateConversionValueRuleSetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateConversionValueRuleSetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @instance
                             */
                            MutateConversionValueRuleSetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateConversionValueRuleSetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest} MutateConversionValueRuleSetsRequest instance
                             */
                            MutateConversionValueRuleSetsRequest.create = function create(properties) {
                                return new MutateConversionValueRuleSetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleSetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsRequest} m MutateConversionValueRuleSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleSetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(32).int32(m.response_content_type);
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(40).bool(m.partial_failure);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleSetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsRequest} message MutateConversionValueRuleSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleSetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleSetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest} MutateConversionValueRuleSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleSetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    case 4:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleSetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest} MutateConversionValueRuleSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleSetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionValueRuleSetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionValueRuleSetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionValueRuleSetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest} MutateConversionValueRuleSetsRequest
                             */
                            MutateConversionValueRuleSetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionValueRuleSetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest} m MutateConversionValueRuleSetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionValueRuleSetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.partial_failure = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionValueRuleSetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionValueRuleSetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionValueRuleSetsRequest;
                        })();
    
                        services.ConversionValueRuleSetOperation = (function() {
    
                            /**
                             * Properties of a ConversionValueRuleSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IConversionValueRuleSetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ConversionValueRuleSetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IConversionValueRuleSet|null} [create] ConversionValueRuleSetOperation create
                             * @property {google.ads.googleads.v11.resources.IConversionValueRuleSet|null} [update] ConversionValueRuleSetOperation update
                             * @property {string|null} [remove] ConversionValueRuleSetOperation remove
                             */
    
                            /**
                             * Constructs a new ConversionValueRuleSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionValueRuleSetOperation.
                             * @implements IConversionValueRuleSetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleSetOperation=} [p] Properties to set
                             */
                            function ConversionValueRuleSetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionValueRuleSetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @instance
                             */
                            ConversionValueRuleSetOperation.prototype.update_mask = null;
    
                            /**
                             * ConversionValueRuleSetOperation create.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRuleSet|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @instance
                             */
                            ConversionValueRuleSetOperation.prototype.create = null;
    
                            /**
                             * ConversionValueRuleSetOperation update.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRuleSet|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @instance
                             */
                            ConversionValueRuleSetOperation.prototype.update = null;
    
                            /**
                             * ConversionValueRuleSetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @instance
                             */
                            ConversionValueRuleSetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionValueRuleSetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @instance
                             */
                            Object.defineProperty(ConversionValueRuleSetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionValueRuleSetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleSetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleSetOperation} ConversionValueRuleSetOperation instance
                             */
                            ConversionValueRuleSetOperation.create = function create(properties) {
                                return new ConversionValueRuleSetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ConversionValueRuleSetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleSetOperation} m ConversionValueRuleSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionValueRuleSetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ConversionValueRuleSetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionValueRuleSetOperation} message ConversionValueRuleSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionValueRuleSetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleSetOperation} ConversionValueRuleSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionValueRuleSetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleSetOperation} ConversionValueRuleSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionValueRuleSetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionValueRuleSetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionValueRuleSetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionValueRuleSetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ConversionValueRuleSetOperation} ConversionValueRuleSetOperation
                             */
                            ConversionValueRuleSetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ConversionValueRuleSetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionValueRuleSetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionValueRuleSetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionValueRuleSetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionValueRuleSetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ConversionValueRuleSetOperation} m ConversionValueRuleSetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionValueRuleSetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionValueRuleSetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ConversionValueRuleSetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionValueRuleSetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionValueRuleSetOperation;
                        })();
    
                        services.MutateConversionValueRuleSetsResponse = (function() {
    
                            /**
                             * Properties of a MutateConversionValueRuleSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionValueRuleSetsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult>|null} [results] MutateConversionValueRuleSetsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateConversionValueRuleSetsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateConversionValueRuleSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionValueRuleSetsResponse.
                             * @implements IMutateConversionValueRuleSetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsResponse=} [p] Properties to set
                             */
                            function MutateConversionValueRuleSetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionValueRuleSetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @instance
                             */
                            MutateConversionValueRuleSetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateConversionValueRuleSetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @instance
                             */
                            MutateConversionValueRuleSetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateConversionValueRuleSetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse} MutateConversionValueRuleSetsResponse instance
                             */
                            MutateConversionValueRuleSetsResponse.create = function create(properties) {
                                return new MutateConversionValueRuleSetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleSetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsResponse} m MutateConversionValueRuleSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleSetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleSetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetsResponse} message MutateConversionValueRuleSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleSetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleSetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse} MutateConversionValueRuleSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleSetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleSetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse} MutateConversionValueRuleSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleSetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionValueRuleSetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionValueRuleSetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionValueRuleSetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse} MutateConversionValueRuleSetsResponse
                             */
                            MutateConversionValueRuleSetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionValueRuleSetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse} m MutateConversionValueRuleSetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionValueRuleSetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionValueRuleSetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionValueRuleSetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionValueRuleSetsResponse;
                        })();
    
                        services.MutateConversionValueRuleSetResult = (function() {
    
                            /**
                             * Properties of a MutateConversionValueRuleSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateConversionValueRuleSetResult
                             * @property {string|null} [resource_name] MutateConversionValueRuleSetResult resource_name
                             * @property {google.ads.googleads.v11.resources.IConversionValueRuleSet|null} [conversion_value_rule_set] MutateConversionValueRuleSetResult conversion_value_rule_set
                             */
    
                            /**
                             * Constructs a new MutateConversionValueRuleSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateConversionValueRuleSetResult.
                             * @implements IMutateConversionValueRuleSetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult=} [p] Properties to set
                             */
                            function MutateConversionValueRuleSetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateConversionValueRuleSetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @instance
                             */
                            MutateConversionValueRuleSetResult.prototype.resource_name = "";
    
                            /**
                             * MutateConversionValueRuleSetResult conversion_value_rule_set.
                             * @member {google.ads.googleads.v11.resources.IConversionValueRuleSet|null|undefined} conversion_value_rule_set
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @instance
                             */
                            MutateConversionValueRuleSetResult.prototype.conversion_value_rule_set = null;
    
                            /**
                             * Creates a new MutateConversionValueRuleSetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetResult} MutateConversionValueRuleSetResult instance
                             */
                            MutateConversionValueRuleSetResult.create = function create(properties) {
                                return new MutateConversionValueRuleSetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleSetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult} m MutateConversionValueRuleSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleSetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.conversion_value_rule_set != null && Object.hasOwnProperty.call(m, "conversion_value_rule_set"))
                                    $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.encode(m.conversion_value_rule_set, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateConversionValueRuleSetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateConversionValueRuleSetResult} message MutateConversionValueRuleSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateConversionValueRuleSetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleSetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetResult} MutateConversionValueRuleSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleSetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.conversion_value_rule_set = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateConversionValueRuleSetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetResult} MutateConversionValueRuleSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateConversionValueRuleSetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateConversionValueRuleSetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateConversionValueRuleSetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.conversion_value_rule_set != null && m.hasOwnProperty("conversion_value_rule_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.verify(m.conversion_value_rule_set);
                                        if (e)
                                            return "conversion_value_rule_set." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateConversionValueRuleSetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateConversionValueRuleSetResult} MutateConversionValueRuleSetResult
                             */
                            MutateConversionValueRuleSetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateConversionValueRuleSetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.conversion_value_rule_set != null) {
                                    if (typeof d.conversion_value_rule_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateConversionValueRuleSetResult.conversion_value_rule_set: object expected");
                                    m.conversion_value_rule_set = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.fromObject(d.conversion_value_rule_set);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateConversionValueRuleSetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateConversionValueRuleSetResult} m MutateConversionValueRuleSetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateConversionValueRuleSetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.conversion_value_rule_set = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.conversion_value_rule_set != null && m.hasOwnProperty("conversion_value_rule_set")) {
                                    d.conversion_value_rule_set = $root.google.ads.googleads.v11.resources.ConversionValueRuleSet.toObject(m.conversion_value_rule_set, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateConversionValueRuleSetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateConversionValueRuleSetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateConversionValueRuleSetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateConversionValueRuleSetResult;
                        })();
    
                        services.CustomConversionGoalService = (function() {
    
                            /**
                             * Constructs a new CustomConversionGoalService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomConversionGoalService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomConversionGoalService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomConversionGoalService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomConversionGoalService;
    
                            /**
                             * Creates new CustomConversionGoalService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomConversionGoalService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomConversionGoalService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomConversionGoalService|mutateCustomConversionGoals}.
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalService
                             * @typedef MutateCustomConversionGoalsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse} [response] MutateCustomConversionGoalsResponse
                             */
    
                            /**
                             * Calls MutateCustomConversionGoals.
                             * @function mutateCustomConversionGoals
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsRequest} request MutateCustomConversionGoalsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomConversionGoalService.MutateCustomConversionGoalsCallback} callback Node-style callback called with the error, if any, and MutateCustomConversionGoalsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomConversionGoalService.prototype.mutateCustomConversionGoals = function mutateCustomConversionGoals(request, callback) {
                                return this.rpcCall(mutateCustomConversionGoals, $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest, $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse, request, callback);
                            }, "name", { value: "MutateCustomConversionGoals" });
    
                            /**
                             * Calls MutateCustomConversionGoals.
                             * @function mutateCustomConversionGoals
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsRequest} request MutateCustomConversionGoalsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomConversionGoalService;
                        })();
    
                        services.MutateCustomConversionGoalsRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomConversionGoalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomConversionGoalsRequest
                             * @property {string|null} [customer_id] MutateCustomConversionGoalsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomConversionGoalOperation>|null} [operations] MutateCustomConversionGoalsRequest operations
                             * @property {boolean|null} [validate_only] MutateCustomConversionGoalsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomConversionGoalsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomConversionGoalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomConversionGoalsRequest.
                             * @implements IMutateCustomConversionGoalsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsRequest=} [p] Properties to set
                             */
                            function MutateCustomConversionGoalsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomConversionGoalsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @instance
                             */
                            MutateCustomConversionGoalsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomConversionGoalsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomConversionGoalOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @instance
                             */
                            MutateCustomConversionGoalsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomConversionGoalsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @instance
                             */
                            MutateCustomConversionGoalsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomConversionGoalsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @instance
                             */
                            MutateCustomConversionGoalsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomConversionGoalsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest} MutateCustomConversionGoalsRequest instance
                             */
                            MutateCustomConversionGoalsRequest.create = function create(properties) {
                                return new MutateCustomConversionGoalsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomConversionGoalsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsRequest} m MutateCustomConversionGoalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomConversionGoalsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(32).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomConversionGoalsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsRequest} message MutateCustomConversionGoalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomConversionGoalsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomConversionGoalsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest} MutateCustomConversionGoalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomConversionGoalsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomConversionGoalOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    case 4:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomConversionGoalsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest} MutateCustomConversionGoalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomConversionGoalsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomConversionGoalsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomConversionGoalsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomConversionGoalsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest} MutateCustomConversionGoalsRequest
                             */
                            MutateCustomConversionGoalsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomConversionGoalsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest} m MutateCustomConversionGoalsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomConversionGoalsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomConversionGoalOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomConversionGoalsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomConversionGoalsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomConversionGoalsRequest;
                        })();
    
                        services.CustomConversionGoalOperation = (function() {
    
                            /**
                             * Properties of a CustomConversionGoalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomConversionGoalOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomConversionGoalOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomConversionGoal|null} [create] CustomConversionGoalOperation create
                             * @property {google.ads.googleads.v11.resources.ICustomConversionGoal|null} [update] CustomConversionGoalOperation update
                             * @property {string|null} [remove] CustomConversionGoalOperation remove
                             */
    
                            /**
                             * Constructs a new CustomConversionGoalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomConversionGoalOperation.
                             * @implements ICustomConversionGoalOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomConversionGoalOperation=} [p] Properties to set
                             */
                            function CustomConversionGoalOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomConversionGoalOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @instance
                             */
                            CustomConversionGoalOperation.prototype.update_mask = null;
    
                            /**
                             * CustomConversionGoalOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomConversionGoal|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @instance
                             */
                            CustomConversionGoalOperation.prototype.create = null;
    
                            /**
                             * CustomConversionGoalOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomConversionGoal|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @instance
                             */
                            CustomConversionGoalOperation.prototype.update = null;
    
                            /**
                             * CustomConversionGoalOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @instance
                             */
                            CustomConversionGoalOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomConversionGoalOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @instance
                             */
                            Object.defineProperty(CustomConversionGoalOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomConversionGoalOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomConversionGoalOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomConversionGoalOperation} CustomConversionGoalOperation instance
                             */
                            CustomConversionGoalOperation.create = function create(properties) {
                                return new CustomConversionGoalOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoalOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomConversionGoalOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomConversionGoalOperation} m CustomConversionGoalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoalOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomConversionGoal.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomConversionGoal.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomConversionGoalOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomConversionGoalOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomConversionGoalOperation} message CustomConversionGoalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomConversionGoalOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomConversionGoalOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomConversionGoalOperation} CustomConversionGoalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoalOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomConversionGoalOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomConversionGoal.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomConversionGoal.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomConversionGoalOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomConversionGoalOperation} CustomConversionGoalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomConversionGoalOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomConversionGoalOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomConversionGoalOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomConversionGoal.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomConversionGoal.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomConversionGoalOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomConversionGoalOperation} CustomConversionGoalOperation
                             */
                            CustomConversionGoalOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomConversionGoalOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomConversionGoalOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomConversionGoalOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomConversionGoalOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomConversionGoal.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomConversionGoalOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomConversionGoal.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomConversionGoalOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomConversionGoalOperation} m CustomConversionGoalOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomConversionGoalOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomConversionGoal.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomConversionGoal.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomConversionGoalOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomConversionGoalOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomConversionGoalOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomConversionGoalOperation;
                        })();
    
                        services.MutateCustomConversionGoalsResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomConversionGoalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomConversionGoalsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomConversionGoalResult>|null} [results] MutateCustomConversionGoalsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomConversionGoalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomConversionGoalsResponse.
                             * @implements IMutateCustomConversionGoalsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsResponse=} [p] Properties to set
                             */
                            function MutateCustomConversionGoalsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomConversionGoalsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomConversionGoalResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @instance
                             */
                            MutateCustomConversionGoalsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomConversionGoalsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse} MutateCustomConversionGoalsResponse instance
                             */
                            MutateCustomConversionGoalsResponse.create = function create(properties) {
                                return new MutateCustomConversionGoalsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomConversionGoalsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsResponse} m MutateCustomConversionGoalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomConversionGoalsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomConversionGoalsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalsResponse} message MutateCustomConversionGoalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomConversionGoalsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomConversionGoalsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse} MutateCustomConversionGoalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomConversionGoalsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomConversionGoalsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse} MutateCustomConversionGoalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomConversionGoalsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomConversionGoalsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomConversionGoalsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomConversionGoalsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse} MutateCustomConversionGoalsResponse
                             */
                            MutateCustomConversionGoalsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomConversionGoalsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse} m MutateCustomConversionGoalsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomConversionGoalsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomConversionGoalsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomConversionGoalsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomConversionGoalsResponse;
                        })();
    
                        services.MutateCustomConversionGoalResult = (function() {
    
                            /**
                             * Properties of a MutateCustomConversionGoalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomConversionGoalResult
                             * @property {string|null} [resource_name] MutateCustomConversionGoalResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomConversionGoal|null} [custom_conversion_goal] MutateCustomConversionGoalResult custom_conversion_goal
                             */
    
                            /**
                             * Constructs a new MutateCustomConversionGoalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomConversionGoalResult.
                             * @implements IMutateCustomConversionGoalResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalResult=} [p] Properties to set
                             */
                            function MutateCustomConversionGoalResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomConversionGoalResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @instance
                             */
                            MutateCustomConversionGoalResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomConversionGoalResult custom_conversion_goal.
                             * @member {google.ads.googleads.v11.resources.ICustomConversionGoal|null|undefined} custom_conversion_goal
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @instance
                             */
                            MutateCustomConversionGoalResult.prototype.custom_conversion_goal = null;
    
                            /**
                             * Creates a new MutateCustomConversionGoalResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalResult} MutateCustomConversionGoalResult instance
                             */
                            MutateCustomConversionGoalResult.create = function create(properties) {
                                return new MutateCustomConversionGoalResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomConversionGoalResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomConversionGoalResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalResult} m MutateCustomConversionGoalResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomConversionGoalResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.custom_conversion_goal != null && Object.hasOwnProperty.call(m, "custom_conversion_goal"))
                                    $root.google.ads.googleads.v11.resources.CustomConversionGoal.encode(m.custom_conversion_goal, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomConversionGoalResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomConversionGoalResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomConversionGoalResult} message MutateCustomConversionGoalResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomConversionGoalResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomConversionGoalResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalResult} MutateCustomConversionGoalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomConversionGoalResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.custom_conversion_goal = $root.google.ads.googleads.v11.resources.CustomConversionGoal.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomConversionGoalResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalResult} MutateCustomConversionGoalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomConversionGoalResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomConversionGoalResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomConversionGoalResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.custom_conversion_goal != null && m.hasOwnProperty("custom_conversion_goal")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomConversionGoal.verify(m.custom_conversion_goal);
                                        if (e)
                                            return "custom_conversion_goal." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomConversionGoalResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomConversionGoalResult} MutateCustomConversionGoalResult
                             */
                            MutateCustomConversionGoalResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomConversionGoalResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.custom_conversion_goal != null) {
                                    if (typeof d.custom_conversion_goal !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomConversionGoalResult.custom_conversion_goal: object expected");
                                    m.custom_conversion_goal = $root.google.ads.googleads.v11.resources.CustomConversionGoal.fromObject(d.custom_conversion_goal);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomConversionGoalResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomConversionGoalResult} m MutateCustomConversionGoalResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomConversionGoalResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.custom_conversion_goal = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.custom_conversion_goal != null && m.hasOwnProperty("custom_conversion_goal")) {
                                    d.custom_conversion_goal = $root.google.ads.googleads.v11.resources.CustomConversionGoal.toObject(m.custom_conversion_goal, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomConversionGoalResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomConversionGoalResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomConversionGoalResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomConversionGoalResult;
                        })();
    
                        services.CustomerAssetService = (function() {
    
                            /**
                             * Constructs a new CustomerAssetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerAssetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerAssetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerAssetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerAssetService;
    
                            /**
                             * Creates new CustomerAssetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerAssetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerAssetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerAssetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerAssetService|mutateCustomerAssets}.
                             * @memberof google.ads.googleads.v11.services.CustomerAssetService
                             * @typedef MutateCustomerAssetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerAssetsResponse} [response] MutateCustomerAssetsResponse
                             */
    
                            /**
                             * Calls MutateCustomerAssets.
                             * @function mutateCustomerAssets
                             * @memberof google.ads.googleads.v11.services.CustomerAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsRequest} request MutateCustomerAssetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerAssetService.MutateCustomerAssetsCallback} callback Node-style callback called with the error, if any, and MutateCustomerAssetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerAssetService.prototype.mutateCustomerAssets = function mutateCustomerAssets(request, callback) {
                                return this.rpcCall(mutateCustomerAssets, $root.google.ads.googleads.v11.services.MutateCustomerAssetsRequest, $root.google.ads.googleads.v11.services.MutateCustomerAssetsResponse, request, callback);
                            }, "name", { value: "MutateCustomerAssets" });
    
                            /**
                             * Calls MutateCustomerAssets.
                             * @function mutateCustomerAssets
                             * @memberof google.ads.googleads.v11.services.CustomerAssetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsRequest} request MutateCustomerAssetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerAssetsResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerAssetService;
                        })();
    
                        services.MutateCustomerAssetsRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerAssetsRequest
                             * @property {string|null} [customer_id] MutateCustomerAssetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerAssetOperation>|null} [operations] MutateCustomerAssetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCustomerAssetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCustomerAssetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomerAssetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomerAssetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerAssetsRequest.
                             * @implements IMutateCustomerAssetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsRequest=} [p] Properties to set
                             */
                            function MutateCustomerAssetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerAssetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @instance
                             */
                            MutateCustomerAssetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerAssetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerAssetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @instance
                             */
                            MutateCustomerAssetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerAssetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @instance
                             */
                            MutateCustomerAssetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCustomerAssetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @instance
                             */
                            MutateCustomerAssetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomerAssetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @instance
                             */
                            MutateCustomerAssetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomerAssetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsRequest} MutateCustomerAssetsRequest instance
                             */
                            MutateCustomerAssetsRequest.create = function create(properties) {
                                return new MutateCustomerAssetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerAssetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerAssetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsRequest} m MutateCustomerAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerAssetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerAssetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerAssetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerAssetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsRequest} message MutateCustomerAssetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerAssetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerAssetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsRequest} MutateCustomerAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerAssetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerAssetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerAssetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerAssetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsRequest} MutateCustomerAssetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerAssetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerAssetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerAssetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerAssetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerAssetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsRequest} MutateCustomerAssetsRequest
                             */
                            MutateCustomerAssetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerAssetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerAssetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerAssetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerAssetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerAssetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerAssetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerAssetsRequest} m MutateCustomerAssetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerAssetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerAssetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerAssetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerAssetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerAssetsRequest;
                        })();
    
                        services.CustomerAssetOperation = (function() {
    
                            /**
                             * Properties of a CustomerAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerAssetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerAssetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomerAsset|null} [create] CustomerAssetOperation create
                             * @property {google.ads.googleads.v11.resources.ICustomerAsset|null} [update] CustomerAssetOperation update
                             * @property {string|null} [remove] CustomerAssetOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerAssetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerAssetOperation.
                             * @implements ICustomerAssetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerAssetOperation=} [p] Properties to set
                             */
                            function CustomerAssetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerAssetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @instance
                             */
                            CustomerAssetOperation.prototype.update_mask = null;
    
                            /**
                             * CustomerAssetOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerAsset|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @instance
                             */
                            CustomerAssetOperation.prototype.create = null;
    
                            /**
                             * CustomerAssetOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomerAsset|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @instance
                             */
                            CustomerAssetOperation.prototype.update = null;
    
                            /**
                             * CustomerAssetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @instance
                             */
                            CustomerAssetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerAssetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerAssetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerAssetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerAssetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerAssetOperation} CustomerAssetOperation instance
                             */
                            CustomerAssetOperation.create = function create(properties) {
                                return new CustomerAssetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerAssetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerAssetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerAssetOperation} m CustomerAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerAssetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerAsset.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomerAsset.encode(m.update, w.uint32(26).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerAssetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerAssetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerAssetOperation} message CustomerAssetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerAssetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerAssetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerAssetOperation} CustomerAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerAssetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerAssetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerAsset.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomerAsset.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerAssetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerAssetOperation} CustomerAssetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerAssetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerAssetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerAssetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerAsset.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerAsset.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerAssetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerAssetOperation} CustomerAssetOperation
                             */
                            CustomerAssetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerAssetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerAssetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerAssetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerAssetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerAsset.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerAssetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomerAsset.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerAssetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerAssetOperation} m CustomerAssetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerAssetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerAsset.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomerAsset.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerAssetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerAssetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerAssetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerAssetOperation;
                        })();
    
                        services.MutateCustomerAssetsResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerAssetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCustomerAssetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerAssetResult>|null} [results] MutateCustomerAssetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomerAssetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerAssetsResponse.
                             * @implements IMutateCustomerAssetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsResponse=} [p] Properties to set
                             */
                            function MutateCustomerAssetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerAssetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @instance
                             */
                            MutateCustomerAssetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCustomerAssetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerAssetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @instance
                             */
                            MutateCustomerAssetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomerAssetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsResponse} MutateCustomerAssetsResponse instance
                             */
                            MutateCustomerAssetsResponse.create = function create(properties) {
                                return new MutateCustomerAssetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerAssetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerAssetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsResponse} m MutateCustomerAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerAssetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerAssetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerAssetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetsResponse} message MutateCustomerAssetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerAssetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerAssetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsResponse} MutateCustomerAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerAssetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerAssetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerAssetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerAssetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsResponse} MutateCustomerAssetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerAssetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerAssetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerAssetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerAssetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetsResponse} MutateCustomerAssetsResponse
                             */
                            MutateCustomerAssetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerAssetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerAssetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerAssetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerAssetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerAssetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerAssetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerAssetsResponse} m MutateCustomerAssetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerAssetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerAssetResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerAssetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerAssetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerAssetsResponse;
                        })();
    
                        services.MutateCustomerAssetResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerAssetResult
                             * @property {string|null} [resource_name] MutateCustomerAssetResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomerAsset|null} [customer_asset] MutateCustomerAssetResult customer_asset
                             */
    
                            /**
                             * Constructs a new MutateCustomerAssetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerAssetResult.
                             * @implements IMutateCustomerAssetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetResult=} [p] Properties to set
                             */
                            function MutateCustomerAssetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerAssetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @instance
                             */
                            MutateCustomerAssetResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomerAssetResult customer_asset.
                             * @member {google.ads.googleads.v11.resources.ICustomerAsset|null|undefined} customer_asset
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @instance
                             */
                            MutateCustomerAssetResult.prototype.customer_asset = null;
    
                            /**
                             * Creates a new MutateCustomerAssetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetResult} MutateCustomerAssetResult instance
                             */
                            MutateCustomerAssetResult.create = function create(properties) {
                                return new MutateCustomerAssetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerAssetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerAssetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetResult} m MutateCustomerAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerAssetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customer_asset != null && Object.hasOwnProperty.call(m, "customer_asset"))
                                    $root.google.ads.googleads.v11.resources.CustomerAsset.encode(m.customer_asset, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerAssetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerAssetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerAssetResult} message MutateCustomerAssetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerAssetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerAssetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetResult} MutateCustomerAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerAssetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerAssetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerAssetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetResult} MutateCustomerAssetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerAssetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerAssetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerAssetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerAsset.verify(m.customer_asset);
                                        if (e)
                                            return "customer_asset." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerAssetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerAssetResult} MutateCustomerAssetResult
                             */
                            MutateCustomerAssetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerAssetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerAssetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customer_asset != null) {
                                    if (typeof d.customer_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerAssetResult.customer_asset: object expected");
                                    m.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.fromObject(d.customer_asset);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerAssetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerAssetResult} m MutateCustomerAssetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerAssetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customer_asset = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customer_asset != null && m.hasOwnProperty("customer_asset")) {
                                    d.customer_asset = $root.google.ads.googleads.v11.resources.CustomerAsset.toObject(m.customer_asset, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerAssetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerAssetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerAssetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerAssetResult;
                        })();
    
                        services.CustomerConversionGoalService = (function() {
    
                            /**
                             * Constructs a new CustomerConversionGoalService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerConversionGoalService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerConversionGoalService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerConversionGoalService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerConversionGoalService;
    
                            /**
                             * Creates new CustomerConversionGoalService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerConversionGoalService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerConversionGoalService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerConversionGoalService|mutateCustomerConversionGoals}.
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalService
                             * @typedef MutateCustomerConversionGoalsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse} [response] MutateCustomerConversionGoalsResponse
                             */
    
                            /**
                             * Calls MutateCustomerConversionGoals.
                             * @function mutateCustomerConversionGoals
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsRequest} request MutateCustomerConversionGoalsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerConversionGoalService.MutateCustomerConversionGoalsCallback} callback Node-style callback called with the error, if any, and MutateCustomerConversionGoalsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerConversionGoalService.prototype.mutateCustomerConversionGoals = function mutateCustomerConversionGoals(request, callback) {
                                return this.rpcCall(mutateCustomerConversionGoals, $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest, $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse, request, callback);
                            }, "name", { value: "MutateCustomerConversionGoals" });
    
                            /**
                             * Calls MutateCustomerConversionGoals.
                             * @function mutateCustomerConversionGoals
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsRequest} request MutateCustomerConversionGoalsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerConversionGoalService;
                        })();
    
                        services.MutateCustomerConversionGoalsRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerConversionGoalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerConversionGoalsRequest
                             * @property {string|null} [customer_id] MutateCustomerConversionGoalsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerConversionGoalOperation>|null} [operations] MutateCustomerConversionGoalsRequest operations
                             * @property {boolean|null} [validate_only] MutateCustomerConversionGoalsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCustomerConversionGoalsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerConversionGoalsRequest.
                             * @implements IMutateCustomerConversionGoalsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsRequest=} [p] Properties to set
                             */
                            function MutateCustomerConversionGoalsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerConversionGoalsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @instance
                             */
                            MutateCustomerConversionGoalsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerConversionGoalsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerConversionGoalOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @instance
                             */
                            MutateCustomerConversionGoalsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerConversionGoalsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @instance
                             */
                            MutateCustomerConversionGoalsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCustomerConversionGoalsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest} MutateCustomerConversionGoalsRequest instance
                             */
                            MutateCustomerConversionGoalsRequest.create = function create(properties) {
                                return new MutateCustomerConversionGoalsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerConversionGoalsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsRequest} m MutateCustomerConversionGoalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerConversionGoalsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerConversionGoalsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsRequest} message MutateCustomerConversionGoalsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerConversionGoalsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerConversionGoalsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest} MutateCustomerConversionGoalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerConversionGoalsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerConversionGoalsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest} MutateCustomerConversionGoalsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerConversionGoalsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerConversionGoalsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerConversionGoalsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerConversionGoalsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest} MutateCustomerConversionGoalsRequest
                             */
                            MutateCustomerConversionGoalsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerConversionGoalsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest} m MutateCustomerConversionGoalsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerConversionGoalsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerConversionGoalsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerConversionGoalsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerConversionGoalsRequest;
                        })();
    
                        services.CustomerConversionGoalOperation = (function() {
    
                            /**
                             * Properties of a CustomerConversionGoalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerConversionGoalOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerConversionGoalOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomerConversionGoal|null} [update] CustomerConversionGoalOperation update
                             */
    
                            /**
                             * Constructs a new CustomerConversionGoalOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerConversionGoalOperation.
                             * @implements ICustomerConversionGoalOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerConversionGoalOperation=} [p] Properties to set
                             */
                            function CustomerConversionGoalOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerConversionGoalOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @instance
                             */
                            CustomerConversionGoalOperation.prototype.update_mask = null;
    
                            /**
                             * CustomerConversionGoalOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomerConversionGoal|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @instance
                             */
                            CustomerConversionGoalOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerConversionGoalOperation operation.
                             * @member {"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerConversionGoalOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerConversionGoalOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerConversionGoalOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerConversionGoalOperation} CustomerConversionGoalOperation instance
                             */
                            CustomerConversionGoalOperation.create = function create(properties) {
                                return new CustomerConversionGoalOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerConversionGoalOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerConversionGoalOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerConversionGoalOperation} m CustomerConversionGoalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerConversionGoalOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomerConversionGoal.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerConversionGoalOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerConversionGoalOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerConversionGoalOperation} message CustomerConversionGoalOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerConversionGoalOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerConversionGoalOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerConversionGoalOperation} CustomerConversionGoalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerConversionGoalOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerConversionGoalOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerConversionGoalOperation} CustomerConversionGoalOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerConversionGoalOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerConversionGoalOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerConversionGoalOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerConversionGoalOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerConversionGoalOperation} CustomerConversionGoalOperation
                             */
                            CustomerConversionGoalOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerConversionGoalOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerConversionGoalOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerConversionGoalOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerConversionGoalOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerConversionGoalOperation} m CustomerConversionGoalOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerConversionGoalOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomerConversionGoal.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerConversionGoalOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerConversionGoalOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerConversionGoalOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerConversionGoalOperation;
                        })();
    
                        services.MutateCustomerConversionGoalsResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerConversionGoalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerConversionGoalsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult>|null} [results] MutateCustomerConversionGoalsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomerConversionGoalsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerConversionGoalsResponse.
                             * @implements IMutateCustomerConversionGoalsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsResponse=} [p] Properties to set
                             */
                            function MutateCustomerConversionGoalsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerConversionGoalsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @instance
                             */
                            MutateCustomerConversionGoalsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomerConversionGoalsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse} MutateCustomerConversionGoalsResponse instance
                             */
                            MutateCustomerConversionGoalsResponse.create = function create(properties) {
                                return new MutateCustomerConversionGoalsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerConversionGoalsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsResponse} m MutateCustomerConversionGoalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerConversionGoalsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerConversionGoalsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalsResponse} message MutateCustomerConversionGoalsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerConversionGoalsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerConversionGoalsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse} MutateCustomerConversionGoalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerConversionGoalsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerConversionGoalsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse} MutateCustomerConversionGoalsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerConversionGoalsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerConversionGoalsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerConversionGoalsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerConversionGoalsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse} MutateCustomerConversionGoalsResponse
                             */
                            MutateCustomerConversionGoalsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerConversionGoalsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse} m MutateCustomerConversionGoalsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerConversionGoalsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerConversionGoalsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerConversionGoalsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerConversionGoalsResponse;
                        })();
    
                        services.MutateCustomerConversionGoalResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerConversionGoalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerConversionGoalResult
                             * @property {string|null} [resource_name] MutateCustomerConversionGoalResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomerConversionGoalResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerConversionGoalResult.
                             * @implements IMutateCustomerConversionGoalResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult=} [p] Properties to set
                             */
                            function MutateCustomerConversionGoalResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerConversionGoalResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @instance
                             */
                            MutateCustomerConversionGoalResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomerConversionGoalResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalResult} MutateCustomerConversionGoalResult instance
                             */
                            MutateCustomerConversionGoalResult.create = function create(properties) {
                                return new MutateCustomerConversionGoalResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerConversionGoalResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult} m MutateCustomerConversionGoalResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerConversionGoalResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerConversionGoalResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerConversionGoalResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerConversionGoalResult} message MutateCustomerConversionGoalResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerConversionGoalResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerConversionGoalResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalResult} MutateCustomerConversionGoalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerConversionGoalResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerConversionGoalResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalResult} MutateCustomerConversionGoalResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerConversionGoalResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerConversionGoalResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerConversionGoalResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerConversionGoalResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerConversionGoalResult} MutateCustomerConversionGoalResult
                             */
                            MutateCustomerConversionGoalResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerConversionGoalResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerConversionGoalResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerConversionGoalResult} m MutateCustomerConversionGoalResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerConversionGoalResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerConversionGoalResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerConversionGoalResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerConversionGoalResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerConversionGoalResult;
                        })();
    
                        services.CustomerCustomizerService = (function() {
    
                            /**
                             * Constructs a new CustomerCustomizerService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerCustomizerService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerCustomizerService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerCustomizerService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerCustomizerService;
    
                            /**
                             * Creates new CustomerCustomizerService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerCustomizerService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerCustomizerService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerCustomizerService|mutateCustomerCustomizers}.
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerService
                             * @typedef MutateCustomerCustomizersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerCustomizersResponse} [response] MutateCustomerCustomizersResponse
                             */
    
                            /**
                             * Calls MutateCustomerCustomizers.
                             * @function mutateCustomerCustomizers
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersRequest} request MutateCustomerCustomizersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerCustomizerService.MutateCustomerCustomizersCallback} callback Node-style callback called with the error, if any, and MutateCustomerCustomizersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerCustomizerService.prototype.mutateCustomerCustomizers = function mutateCustomerCustomizers(request, callback) {
                                return this.rpcCall(mutateCustomerCustomizers, $root.google.ads.googleads.v11.services.MutateCustomerCustomizersRequest, $root.google.ads.googleads.v11.services.MutateCustomerCustomizersResponse, request, callback);
                            }, "name", { value: "MutateCustomerCustomizers" });
    
                            /**
                             * Calls MutateCustomerCustomizers.
                             * @function mutateCustomerCustomizers
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersRequest} request MutateCustomerCustomizersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerCustomizersResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerCustomizerService;
                        })();
    
                        services.MutateCustomerCustomizersRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerCustomizersRequest
                             * @property {string|null} [customer_id] MutateCustomerCustomizersRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerCustomizerOperation>|null} [operations] MutateCustomerCustomizersRequest operations
                             * @property {boolean|null} [partial_failure] MutateCustomerCustomizersRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCustomerCustomizersRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomerCustomizersRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomerCustomizersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerCustomizersRequest.
                             * @implements IMutateCustomerCustomizersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersRequest=} [p] Properties to set
                             */
                            function MutateCustomerCustomizersRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerCustomizersRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @instance
                             */
                            MutateCustomerCustomizersRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerCustomizersRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerCustomizerOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @instance
                             */
                            MutateCustomerCustomizersRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerCustomizersRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @instance
                             */
                            MutateCustomerCustomizersRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCustomerCustomizersRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @instance
                             */
                            MutateCustomerCustomizersRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomerCustomizersRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @instance
                             */
                            MutateCustomerCustomizersRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomerCustomizersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersRequest} MutateCustomerCustomizersRequest instance
                             */
                            MutateCustomerCustomizersRequest.create = function create(properties) {
                                return new MutateCustomerCustomizersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerCustomizersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerCustomizersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersRequest} m MutateCustomerCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerCustomizersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerCustomizersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerCustomizersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersRequest} message MutateCustomerCustomizersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerCustomizersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerCustomizersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersRequest} MutateCustomerCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerCustomizersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerCustomizersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerCustomizerOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerCustomizersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersRequest} MutateCustomerCustomizersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerCustomizersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerCustomizersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerCustomizersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerCustomizersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersRequest} MutateCustomerCustomizersRequest
                             */
                            MutateCustomerCustomizersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerCustomizersRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerCustomizersRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerCustomizersRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerCustomizersRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerCustomizersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerCustomizersRequest} m MutateCustomerCustomizersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerCustomizersRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerCustomizerOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerCustomizersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerCustomizersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerCustomizersRequest;
                        })();
    
                        services.CustomerCustomizerOperation = (function() {
    
                            /**
                             * Properties of a CustomerCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerCustomizerOperation
                             * @property {google.ads.googleads.v11.resources.ICustomerCustomizer|null} [create] CustomerCustomizerOperation create
                             * @property {string|null} [remove] CustomerCustomizerOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerCustomizerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerCustomizerOperation.
                             * @implements ICustomerCustomizerOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerCustomizerOperation=} [p] Properties to set
                             */
                            function CustomerCustomizerOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerCustomizerOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerCustomizer|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @instance
                             */
                            CustomerCustomizerOperation.prototype.create = null;
    
                            /**
                             * CustomerCustomizerOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @instance
                             */
                            CustomerCustomizerOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerCustomizerOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerCustomizerOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerCustomizerOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerCustomizerOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerCustomizerOperation} CustomerCustomizerOperation instance
                             */
                            CustomerCustomizerOperation.create = function create(properties) {
                                return new CustomerCustomizerOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerCustomizerOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerCustomizerOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerCustomizerOperation} m CustomerCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerCustomizerOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerCustomizer.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerCustomizerOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerCustomizerOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerCustomizerOperation} message CustomerCustomizerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerCustomizerOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerCustomizerOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerCustomizerOperation} CustomerCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerCustomizerOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerCustomizerOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerCustomizer.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerCustomizerOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerCustomizerOperation} CustomerCustomizerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerCustomizerOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerCustomizerOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerCustomizerOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerCustomizer.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerCustomizerOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerCustomizerOperation} CustomerCustomizerOperation
                             */
                            CustomerCustomizerOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerCustomizerOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerCustomizerOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerCustomizerOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerCustomizer.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerCustomizerOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerCustomizerOperation} m CustomerCustomizerOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerCustomizerOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerCustomizer.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerCustomizerOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerCustomizerOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerCustomizerOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerCustomizerOperation;
                        })();
    
                        services.MutateCustomerCustomizersResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerCustomizersResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerCustomizerResult>|null} [results] MutateCustomerCustomizersResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCustomerCustomizersResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateCustomerCustomizersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerCustomizersResponse.
                             * @implements IMutateCustomerCustomizersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersResponse=} [p] Properties to set
                             */
                            function MutateCustomerCustomizersResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerCustomizersResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerCustomizerResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @instance
                             */
                            MutateCustomerCustomizersResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateCustomerCustomizersResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @instance
                             */
                            MutateCustomerCustomizersResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateCustomerCustomizersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersResponse} MutateCustomerCustomizersResponse instance
                             */
                            MutateCustomerCustomizersResponse.create = function create(properties) {
                                return new MutateCustomerCustomizersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerCustomizersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerCustomizersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersResponse} m MutateCustomerCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerCustomizersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerCustomizersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerCustomizersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizersResponse} message MutateCustomerCustomizersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerCustomizersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerCustomizersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersResponse} MutateCustomerCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerCustomizersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerCustomizersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerCustomizersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersResponse} MutateCustomerCustomizersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerCustomizersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerCustomizersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerCustomizersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerCustomizersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizersResponse} MutateCustomerCustomizersResponse
                             */
                            MutateCustomerCustomizersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerCustomizersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerCustomizersResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerCustomizersResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerCustomizersResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerCustomizersResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerCustomizersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerCustomizersResponse} m MutateCustomerCustomizersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerCustomizersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerCustomizersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerCustomizersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerCustomizersResponse;
                        })();
    
                        services.MutateCustomerCustomizerResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerCustomizerResult
                             * @property {string|null} [resource_name] MutateCustomerCustomizerResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomerCustomizer|null} [customer_customizer] MutateCustomerCustomizerResult customer_customizer
                             */
    
                            /**
                             * Constructs a new MutateCustomerCustomizerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerCustomizerResult.
                             * @implements IMutateCustomerCustomizerResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizerResult=} [p] Properties to set
                             */
                            function MutateCustomerCustomizerResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerCustomizerResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @instance
                             */
                            MutateCustomerCustomizerResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomerCustomizerResult customer_customizer.
                             * @member {google.ads.googleads.v11.resources.ICustomerCustomizer|null|undefined} customer_customizer
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @instance
                             */
                            MutateCustomerCustomizerResult.prototype.customer_customizer = null;
    
                            /**
                             * Creates a new MutateCustomerCustomizerResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizerResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizerResult} MutateCustomerCustomizerResult instance
                             */
                            MutateCustomerCustomizerResult.create = function create(properties) {
                                return new MutateCustomerCustomizerResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerCustomizerResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerCustomizerResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizerResult} m MutateCustomerCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerCustomizerResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customer_customizer != null && Object.hasOwnProperty.call(m, "customer_customizer"))
                                    $root.google.ads.googleads.v11.resources.CustomerCustomizer.encode(m.customer_customizer, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerCustomizerResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerCustomizerResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerCustomizerResult} message MutateCustomerCustomizerResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerCustomizerResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerCustomizerResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizerResult} MutateCustomerCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerCustomizerResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customer_customizer = $root.google.ads.googleads.v11.resources.CustomerCustomizer.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerCustomizerResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizerResult} MutateCustomerCustomizerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerCustomizerResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerCustomizerResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerCustomizerResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customer_customizer != null && m.hasOwnProperty("customer_customizer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerCustomizer.verify(m.customer_customizer);
                                        if (e)
                                            return "customer_customizer." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerCustomizerResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerCustomizerResult} MutateCustomerCustomizerResult
                             */
                            MutateCustomerCustomizerResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerCustomizerResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customer_customizer != null) {
                                    if (typeof d.customer_customizer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerCustomizerResult.customer_customizer: object expected");
                                    m.customer_customizer = $root.google.ads.googleads.v11.resources.CustomerCustomizer.fromObject(d.customer_customizer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerCustomizerResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerCustomizerResult} m MutateCustomerCustomizerResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerCustomizerResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customer_customizer = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customer_customizer != null && m.hasOwnProperty("customer_customizer")) {
                                    d.customer_customizer = $root.google.ads.googleads.v11.resources.CustomerCustomizer.toObject(m.customer_customizer, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerCustomizerResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerCustomizerResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerCustomizerResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerCustomizerResult;
                        })();
    
                        services.CustomerExtensionSettingService = (function() {
    
                            /**
                             * Constructs a new CustomerExtensionSettingService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerExtensionSettingService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerExtensionSettingService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerExtensionSettingService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerExtensionSettingService;
    
                            /**
                             * Creates new CustomerExtensionSettingService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerExtensionSettingService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerExtensionSettingService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerExtensionSettingService|mutateCustomerExtensionSettings}.
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingService
                             * @typedef MutateCustomerExtensionSettingsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse} [response] MutateCustomerExtensionSettingsResponse
                             */
    
                            /**
                             * Calls MutateCustomerExtensionSettings.
                             * @function mutateCustomerExtensionSettings
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsRequest} request MutateCustomerExtensionSettingsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerExtensionSettingService.MutateCustomerExtensionSettingsCallback} callback Node-style callback called with the error, if any, and MutateCustomerExtensionSettingsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerExtensionSettingService.prototype.mutateCustomerExtensionSettings = function mutateCustomerExtensionSettings(request, callback) {
                                return this.rpcCall(mutateCustomerExtensionSettings, $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest, $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse, request, callback);
                            }, "name", { value: "MutateCustomerExtensionSettings" });
    
                            /**
                             * Calls MutateCustomerExtensionSettings.
                             * @function mutateCustomerExtensionSettings
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsRequest} request MutateCustomerExtensionSettingsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerExtensionSettingService;
                        })();
    
                        services.MutateCustomerExtensionSettingsRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerExtensionSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerExtensionSettingsRequest
                             * @property {string|null} [customer_id] MutateCustomerExtensionSettingsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerExtensionSettingOperation>|null} [operations] MutateCustomerExtensionSettingsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCustomerExtensionSettingsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCustomerExtensionSettingsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomerExtensionSettingsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomerExtensionSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerExtensionSettingsRequest.
                             * @implements IMutateCustomerExtensionSettingsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsRequest=} [p] Properties to set
                             */
                            function MutateCustomerExtensionSettingsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerExtensionSettingsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @instance
                             */
                            MutateCustomerExtensionSettingsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerExtensionSettingsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerExtensionSettingOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @instance
                             */
                            MutateCustomerExtensionSettingsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerExtensionSettingsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @instance
                             */
                            MutateCustomerExtensionSettingsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCustomerExtensionSettingsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @instance
                             */
                            MutateCustomerExtensionSettingsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomerExtensionSettingsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @instance
                             */
                            MutateCustomerExtensionSettingsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomerExtensionSettingsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest} MutateCustomerExtensionSettingsRequest instance
                             */
                            MutateCustomerExtensionSettingsRequest.create = function create(properties) {
                                return new MutateCustomerExtensionSettingsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerExtensionSettingsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsRequest} m MutateCustomerExtensionSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerExtensionSettingsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerExtensionSettingsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsRequest} message MutateCustomerExtensionSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerExtensionSettingsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerExtensionSettingsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest} MutateCustomerExtensionSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerExtensionSettingsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerExtensionSettingsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest} MutateCustomerExtensionSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerExtensionSettingsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerExtensionSettingsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerExtensionSettingsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerExtensionSettingsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest} MutateCustomerExtensionSettingsRequest
                             */
                            MutateCustomerExtensionSettingsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerExtensionSettingsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest} m MutateCustomerExtensionSettingsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerExtensionSettingsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerExtensionSettingsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerExtensionSettingsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerExtensionSettingsRequest;
                        })();
    
                        services.CustomerExtensionSettingOperation = (function() {
    
                            /**
                             * Properties of a CustomerExtensionSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerExtensionSettingOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerExtensionSettingOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null} [create] CustomerExtensionSettingOperation create
                             * @property {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null} [update] CustomerExtensionSettingOperation update
                             * @property {string|null} [remove] CustomerExtensionSettingOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerExtensionSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerExtensionSettingOperation.
                             * @implements ICustomerExtensionSettingOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerExtensionSettingOperation=} [p] Properties to set
                             */
                            function CustomerExtensionSettingOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerExtensionSettingOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @instance
                             */
                            CustomerExtensionSettingOperation.prototype.update_mask = null;
    
                            /**
                             * CustomerExtensionSettingOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @instance
                             */
                            CustomerExtensionSettingOperation.prototype.create = null;
    
                            /**
                             * CustomerExtensionSettingOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @instance
                             */
                            CustomerExtensionSettingOperation.prototype.update = null;
    
                            /**
                             * CustomerExtensionSettingOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @instance
                             */
                            CustomerExtensionSettingOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerExtensionSettingOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerExtensionSettingOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerExtensionSettingOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerExtensionSettingOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerExtensionSettingOperation} CustomerExtensionSettingOperation instance
                             */
                            CustomerExtensionSettingOperation.create = function create(properties) {
                                return new CustomerExtensionSettingOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerExtensionSettingOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerExtensionSettingOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerExtensionSettingOperation} m CustomerExtensionSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerExtensionSettingOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerExtensionSettingOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerExtensionSettingOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerExtensionSettingOperation} message CustomerExtensionSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerExtensionSettingOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerExtensionSettingOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerExtensionSettingOperation} CustomerExtensionSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerExtensionSettingOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerExtensionSettingOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerExtensionSettingOperation} CustomerExtensionSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerExtensionSettingOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerExtensionSettingOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerExtensionSettingOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerExtensionSettingOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerExtensionSettingOperation} CustomerExtensionSettingOperation
                             */
                            CustomerExtensionSettingOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerExtensionSettingOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerExtensionSettingOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerExtensionSettingOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerExtensionSettingOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerExtensionSettingOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerExtensionSettingOperation} m CustomerExtensionSettingOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerExtensionSettingOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerExtensionSettingOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerExtensionSettingOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerExtensionSettingOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerExtensionSettingOperation;
                        })();
    
                        services.MutateCustomerExtensionSettingsResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerExtensionSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerExtensionSettingsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCustomerExtensionSettingsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult>|null} [results] MutateCustomerExtensionSettingsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomerExtensionSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerExtensionSettingsResponse.
                             * @implements IMutateCustomerExtensionSettingsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsResponse=} [p] Properties to set
                             */
                            function MutateCustomerExtensionSettingsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerExtensionSettingsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @instance
                             */
                            MutateCustomerExtensionSettingsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCustomerExtensionSettingsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @instance
                             */
                            MutateCustomerExtensionSettingsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomerExtensionSettingsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse} MutateCustomerExtensionSettingsResponse instance
                             */
                            MutateCustomerExtensionSettingsResponse.create = function create(properties) {
                                return new MutateCustomerExtensionSettingsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerExtensionSettingsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsResponse} m MutateCustomerExtensionSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerExtensionSettingsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerExtensionSettingsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingsResponse} message MutateCustomerExtensionSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerExtensionSettingsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerExtensionSettingsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse} MutateCustomerExtensionSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerExtensionSettingsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerExtensionSettingsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse} MutateCustomerExtensionSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerExtensionSettingsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerExtensionSettingsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerExtensionSettingsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerExtensionSettingsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse} MutateCustomerExtensionSettingsResponse
                             */
                            MutateCustomerExtensionSettingsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerExtensionSettingsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse} m MutateCustomerExtensionSettingsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerExtensionSettingsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerExtensionSettingsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerExtensionSettingsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerExtensionSettingsResponse;
                        })();
    
                        services.MutateCustomerExtensionSettingResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerExtensionSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerExtensionSettingResult
                             * @property {string|null} [resource_name] MutateCustomerExtensionSettingResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null} [customer_extension_setting] MutateCustomerExtensionSettingResult customer_extension_setting
                             */
    
                            /**
                             * Constructs a new MutateCustomerExtensionSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerExtensionSettingResult.
                             * @implements IMutateCustomerExtensionSettingResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult=} [p] Properties to set
                             */
                            function MutateCustomerExtensionSettingResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerExtensionSettingResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @instance
                             */
                            MutateCustomerExtensionSettingResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomerExtensionSettingResult customer_extension_setting.
                             * @member {google.ads.googleads.v11.resources.ICustomerExtensionSetting|null|undefined} customer_extension_setting
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @instance
                             */
                            MutateCustomerExtensionSettingResult.prototype.customer_extension_setting = null;
    
                            /**
                             * Creates a new MutateCustomerExtensionSettingResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult} MutateCustomerExtensionSettingResult instance
                             */
                            MutateCustomerExtensionSettingResult.create = function create(properties) {
                                return new MutateCustomerExtensionSettingResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerExtensionSettingResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult} m MutateCustomerExtensionSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerExtensionSettingResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customer_extension_setting != null && Object.hasOwnProperty.call(m, "customer_extension_setting"))
                                    $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.encode(m.customer_extension_setting, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerExtensionSettingResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerExtensionSettingResult} message MutateCustomerExtensionSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerExtensionSettingResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerExtensionSettingResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult} MutateCustomerExtensionSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerExtensionSettingResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customer_extension_setting = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerExtensionSettingResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult} MutateCustomerExtensionSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerExtensionSettingResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerExtensionSettingResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerExtensionSettingResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customer_extension_setting != null && m.hasOwnProperty("customer_extension_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.verify(m.customer_extension_setting);
                                        if (e)
                                            return "customer_extension_setting." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerExtensionSettingResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult} MutateCustomerExtensionSettingResult
                             */
                            MutateCustomerExtensionSettingResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customer_extension_setting != null) {
                                    if (typeof d.customer_extension_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult.customer_extension_setting: object expected");
                                    m.customer_extension_setting = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.fromObject(d.customer_extension_setting);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerExtensionSettingResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult} m MutateCustomerExtensionSettingResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerExtensionSettingResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customer_extension_setting = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customer_extension_setting != null && m.hasOwnProperty("customer_extension_setting")) {
                                    d.customer_extension_setting = $root.google.ads.googleads.v11.resources.CustomerExtensionSetting.toObject(m.customer_extension_setting, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerExtensionSettingResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerExtensionSettingResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerExtensionSettingResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerExtensionSettingResult;
                        })();
    
                        services.CustomerFeedService = (function() {
    
                            /**
                             * Constructs a new CustomerFeedService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerFeedService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerFeedService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerFeedService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerFeedService;
    
                            /**
                             * Creates new CustomerFeedService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerFeedService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerFeedService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerFeedService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerFeedService|mutateCustomerFeeds}.
                             * @memberof google.ads.googleads.v11.services.CustomerFeedService
                             * @typedef MutateCustomerFeedsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerFeedsResponse} [response] MutateCustomerFeedsResponse
                             */
    
                            /**
                             * Calls MutateCustomerFeeds.
                             * @function mutateCustomerFeeds
                             * @memberof google.ads.googleads.v11.services.CustomerFeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsRequest} request MutateCustomerFeedsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerFeedService.MutateCustomerFeedsCallback} callback Node-style callback called with the error, if any, and MutateCustomerFeedsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerFeedService.prototype.mutateCustomerFeeds = function mutateCustomerFeeds(request, callback) {
                                return this.rpcCall(mutateCustomerFeeds, $root.google.ads.googleads.v11.services.MutateCustomerFeedsRequest, $root.google.ads.googleads.v11.services.MutateCustomerFeedsResponse, request, callback);
                            }, "name", { value: "MutateCustomerFeeds" });
    
                            /**
                             * Calls MutateCustomerFeeds.
                             * @function mutateCustomerFeeds
                             * @memberof google.ads.googleads.v11.services.CustomerFeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsRequest} request MutateCustomerFeedsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerFeedsResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerFeedService;
                        })();
    
                        services.MutateCustomerFeedsRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerFeedsRequest
                             * @property {string|null} [customer_id] MutateCustomerFeedsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerFeedOperation>|null} [operations] MutateCustomerFeedsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCustomerFeedsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCustomerFeedsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomerFeedsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomerFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerFeedsRequest.
                             * @implements IMutateCustomerFeedsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsRequest=} [p] Properties to set
                             */
                            function MutateCustomerFeedsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerFeedsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @instance
                             */
                            MutateCustomerFeedsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerFeedsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerFeedOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @instance
                             */
                            MutateCustomerFeedsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerFeedsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @instance
                             */
                            MutateCustomerFeedsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCustomerFeedsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @instance
                             */
                            MutateCustomerFeedsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomerFeedsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @instance
                             */
                            MutateCustomerFeedsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomerFeedsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsRequest} MutateCustomerFeedsRequest instance
                             */
                            MutateCustomerFeedsRequest.create = function create(properties) {
                                return new MutateCustomerFeedsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerFeedsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerFeedsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsRequest} m MutateCustomerFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerFeedsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerFeedOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerFeedsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerFeedsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsRequest} message MutateCustomerFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerFeedsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerFeedsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsRequest} MutateCustomerFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerFeedsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerFeedsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerFeedOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerFeedsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsRequest} MutateCustomerFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerFeedsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerFeedsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerFeedsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerFeedOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerFeedsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsRequest} MutateCustomerFeedsRequest
                             */
                            MutateCustomerFeedsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerFeedsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerFeedsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerFeedsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerFeedsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerFeedOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerFeedsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerFeedsRequest} m MutateCustomerFeedsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerFeedsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerFeedOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerFeedsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerFeedsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerFeedsRequest;
                        })();
    
                        services.CustomerFeedOperation = (function() {
    
                            /**
                             * Properties of a CustomerFeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerFeedOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerFeedOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomerFeed|null} [create] CustomerFeedOperation create
                             * @property {google.ads.googleads.v11.resources.ICustomerFeed|null} [update] CustomerFeedOperation update
                             * @property {string|null} [remove] CustomerFeedOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerFeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerFeedOperation.
                             * @implements ICustomerFeedOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerFeedOperation=} [p] Properties to set
                             */
                            function CustomerFeedOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerFeedOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @instance
                             */
                            CustomerFeedOperation.prototype.update_mask = null;
    
                            /**
                             * CustomerFeedOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerFeed|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @instance
                             */
                            CustomerFeedOperation.prototype.create = null;
    
                            /**
                             * CustomerFeedOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomerFeed|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @instance
                             */
                            CustomerFeedOperation.prototype.update = null;
    
                            /**
                             * CustomerFeedOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @instance
                             */
                            CustomerFeedOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerFeedOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerFeedOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerFeedOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerFeedOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerFeedOperation} CustomerFeedOperation instance
                             */
                            CustomerFeedOperation.create = function create(properties) {
                                return new CustomerFeedOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerFeedOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerFeedOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerFeedOperation} m CustomerFeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerFeedOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerFeed.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomerFeed.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerFeedOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerFeedOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerFeedOperation} message CustomerFeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerFeedOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerFeedOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerFeedOperation} CustomerFeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerFeedOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerFeedOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerFeed.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomerFeed.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerFeedOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerFeedOperation} CustomerFeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerFeedOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerFeedOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerFeedOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerFeed.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerFeed.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerFeedOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerFeedOperation} CustomerFeedOperation
                             */
                            CustomerFeedOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerFeedOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerFeedOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerFeedOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerFeedOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerFeed.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerFeedOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomerFeed.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerFeedOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerFeedOperation} m CustomerFeedOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerFeedOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerFeed.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomerFeed.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerFeedOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerFeedOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerFeedOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerFeedOperation;
                        })();
    
                        services.MutateCustomerFeedsResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerFeedsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCustomerFeedsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerFeedResult>|null} [results] MutateCustomerFeedsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomerFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerFeedsResponse.
                             * @implements IMutateCustomerFeedsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsResponse=} [p] Properties to set
                             */
                            function MutateCustomerFeedsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerFeedsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @instance
                             */
                            MutateCustomerFeedsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCustomerFeedsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerFeedResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @instance
                             */
                            MutateCustomerFeedsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomerFeedsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsResponse} MutateCustomerFeedsResponse instance
                             */
                            MutateCustomerFeedsResponse.create = function create(properties) {
                                return new MutateCustomerFeedsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerFeedsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerFeedsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsResponse} m MutateCustomerFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerFeedsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerFeedsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerFeedsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedsResponse} message MutateCustomerFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerFeedsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerFeedsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsResponse} MutateCustomerFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerFeedsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerFeedsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerFeedResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerFeedsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsResponse} MutateCustomerFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerFeedsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerFeedsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerFeedsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerFeedsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedsResponse} MutateCustomerFeedsResponse
                             */
                            MutateCustomerFeedsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerFeedsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerFeedsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerFeedsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerFeedsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerFeedsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerFeedsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerFeedsResponse} m MutateCustomerFeedsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerFeedsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerFeedResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerFeedsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerFeedsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerFeedsResponse;
                        })();
    
                        services.MutateCustomerFeedResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerFeedResult
                             * @property {string|null} [resource_name] MutateCustomerFeedResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomerFeed|null} [customer_feed] MutateCustomerFeedResult customer_feed
                             */
    
                            /**
                             * Constructs a new MutateCustomerFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerFeedResult.
                             * @implements IMutateCustomerFeedResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedResult=} [p] Properties to set
                             */
                            function MutateCustomerFeedResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerFeedResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @instance
                             */
                            MutateCustomerFeedResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomerFeedResult customer_feed.
                             * @member {google.ads.googleads.v11.resources.ICustomerFeed|null|undefined} customer_feed
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @instance
                             */
                            MutateCustomerFeedResult.prototype.customer_feed = null;
    
                            /**
                             * Creates a new MutateCustomerFeedResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedResult} MutateCustomerFeedResult instance
                             */
                            MutateCustomerFeedResult.create = function create(properties) {
                                return new MutateCustomerFeedResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerFeedResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerFeedResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedResult} m MutateCustomerFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerFeedResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customer_feed != null && Object.hasOwnProperty.call(m, "customer_feed"))
                                    $root.google.ads.googleads.v11.resources.CustomerFeed.encode(m.customer_feed, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerFeedResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerFeedResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerFeedResult} message MutateCustomerFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerFeedResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerFeedResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedResult} MutateCustomerFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerFeedResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerFeedResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customer_feed = $root.google.ads.googleads.v11.resources.CustomerFeed.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerFeedResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedResult} MutateCustomerFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerFeedResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerFeedResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerFeedResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customer_feed != null && m.hasOwnProperty("customer_feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerFeed.verify(m.customer_feed);
                                        if (e)
                                            return "customer_feed." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerFeedResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerFeedResult} MutateCustomerFeedResult
                             */
                            MutateCustomerFeedResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerFeedResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerFeedResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customer_feed != null) {
                                    if (typeof d.customer_feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerFeedResult.customer_feed: object expected");
                                    m.customer_feed = $root.google.ads.googleads.v11.resources.CustomerFeed.fromObject(d.customer_feed);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerFeedResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerFeedResult} m MutateCustomerFeedResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerFeedResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customer_feed = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customer_feed != null && m.hasOwnProperty("customer_feed")) {
                                    d.customer_feed = $root.google.ads.googleads.v11.resources.CustomerFeed.toObject(m.customer_feed, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerFeedResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerFeedResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerFeedResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerFeedResult;
                        })();
    
                        services.CustomerLabelService = (function() {
    
                            /**
                             * Constructs a new CustomerLabelService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerLabelService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerLabelService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerLabelService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerLabelService;
    
                            /**
                             * Creates new CustomerLabelService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerLabelService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerLabelService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerLabelService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerLabelService|mutateCustomerLabels}.
                             * @memberof google.ads.googleads.v11.services.CustomerLabelService
                             * @typedef MutateCustomerLabelsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerLabelsResponse} [response] MutateCustomerLabelsResponse
                             */
    
                            /**
                             * Calls MutateCustomerLabels.
                             * @function mutateCustomerLabels
                             * @memberof google.ads.googleads.v11.services.CustomerLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsRequest} request MutateCustomerLabelsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerLabelService.MutateCustomerLabelsCallback} callback Node-style callback called with the error, if any, and MutateCustomerLabelsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerLabelService.prototype.mutateCustomerLabels = function mutateCustomerLabels(request, callback) {
                                return this.rpcCall(mutateCustomerLabels, $root.google.ads.googleads.v11.services.MutateCustomerLabelsRequest, $root.google.ads.googleads.v11.services.MutateCustomerLabelsResponse, request, callback);
                            }, "name", { value: "MutateCustomerLabels" });
    
                            /**
                             * Calls MutateCustomerLabels.
                             * @function mutateCustomerLabels
                             * @memberof google.ads.googleads.v11.services.CustomerLabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsRequest} request MutateCustomerLabelsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerLabelsResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerLabelService;
                        })();
    
                        services.MutateCustomerLabelsRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerLabelsRequest
                             * @property {string|null} [customer_id] MutateCustomerLabelsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerLabelOperation>|null} [operations] MutateCustomerLabelsRequest operations
                             * @property {boolean|null} [partial_failure] MutateCustomerLabelsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCustomerLabelsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCustomerLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerLabelsRequest.
                             * @implements IMutateCustomerLabelsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsRequest=} [p] Properties to set
                             */
                            function MutateCustomerLabelsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerLabelsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @instance
                             */
                            MutateCustomerLabelsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerLabelsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerLabelOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @instance
                             */
                            MutateCustomerLabelsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerLabelsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @instance
                             */
                            MutateCustomerLabelsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCustomerLabelsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @instance
                             */
                            MutateCustomerLabelsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCustomerLabelsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsRequest} MutateCustomerLabelsRequest instance
                             */
                            MutateCustomerLabelsRequest.create = function create(properties) {
                                return new MutateCustomerLabelsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerLabelsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerLabelsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsRequest} m MutateCustomerLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerLabelsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerLabelOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerLabelsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerLabelsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsRequest} message MutateCustomerLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerLabelsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerLabelsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsRequest} MutateCustomerLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerLabelsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerLabelsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerLabelOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerLabelsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsRequest} MutateCustomerLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerLabelsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerLabelsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerLabelsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerLabelOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerLabelsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsRequest} MutateCustomerLabelsRequest
                             */
                            MutateCustomerLabelsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerLabelsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerLabelsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerLabelsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerLabelsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerLabelOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerLabelsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerLabelsRequest} m MutateCustomerLabelsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerLabelsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerLabelOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerLabelsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerLabelsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerLabelsRequest;
                        })();
    
                        services.CustomerLabelOperation = (function() {
    
                            /**
                             * Properties of a CustomerLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerLabelOperation
                             * @property {google.ads.googleads.v11.resources.ICustomerLabel|null} [create] CustomerLabelOperation create
                             * @property {string|null} [remove] CustomerLabelOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerLabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerLabelOperation.
                             * @implements ICustomerLabelOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerLabelOperation=} [p] Properties to set
                             */
                            function CustomerLabelOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerLabelOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerLabel|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @instance
                             */
                            CustomerLabelOperation.prototype.create = null;
    
                            /**
                             * CustomerLabelOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @instance
                             */
                            CustomerLabelOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerLabelOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerLabelOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerLabelOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerLabelOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerLabelOperation} CustomerLabelOperation instance
                             */
                            CustomerLabelOperation.create = function create(properties) {
                                return new CustomerLabelOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerLabelOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerLabelOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerLabelOperation} m CustomerLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerLabelOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerLabel.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerLabelOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerLabelOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerLabelOperation} message CustomerLabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerLabelOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerLabelOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerLabelOperation} CustomerLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerLabelOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerLabelOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerLabel.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerLabelOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerLabelOperation} CustomerLabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerLabelOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerLabelOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerLabelOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerLabel.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerLabelOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerLabelOperation} CustomerLabelOperation
                             */
                            CustomerLabelOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerLabelOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerLabelOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerLabelOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerLabel.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerLabelOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerLabelOperation} m CustomerLabelOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerLabelOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerLabel.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerLabelOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerLabelOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerLabelOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerLabelOperation;
                        })();
    
                        services.MutateCustomerLabelsResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerLabelsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCustomerLabelsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerLabelResult>|null} [results] MutateCustomerLabelsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomerLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerLabelsResponse.
                             * @implements IMutateCustomerLabelsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsResponse=} [p] Properties to set
                             */
                            function MutateCustomerLabelsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerLabelsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @instance
                             */
                            MutateCustomerLabelsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCustomerLabelsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerLabelResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @instance
                             */
                            MutateCustomerLabelsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomerLabelsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsResponse} MutateCustomerLabelsResponse instance
                             */
                            MutateCustomerLabelsResponse.create = function create(properties) {
                                return new MutateCustomerLabelsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerLabelsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerLabelsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsResponse} m MutateCustomerLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerLabelsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerLabelsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerLabelsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelsResponse} message MutateCustomerLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerLabelsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerLabelsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsResponse} MutateCustomerLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerLabelsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerLabelsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerLabelResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerLabelsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsResponse} MutateCustomerLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerLabelsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerLabelsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerLabelsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerLabelsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelsResponse} MutateCustomerLabelsResponse
                             */
                            MutateCustomerLabelsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerLabelsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerLabelsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerLabelsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerLabelsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerLabelsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerLabelsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerLabelsResponse} m MutateCustomerLabelsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerLabelsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerLabelResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerLabelsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerLabelsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerLabelsResponse;
                        })();
    
                        services.MutateCustomerLabelResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerLabelResult
                             * @property {string|null} [resource_name] MutateCustomerLabelResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomerLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerLabelResult.
                             * @implements IMutateCustomerLabelResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelResult=} [p] Properties to set
                             */
                            function MutateCustomerLabelResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerLabelResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @instance
                             */
                            MutateCustomerLabelResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomerLabelResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelResult} MutateCustomerLabelResult instance
                             */
                            MutateCustomerLabelResult.create = function create(properties) {
                                return new MutateCustomerLabelResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerLabelResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerLabelResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelResult} m MutateCustomerLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerLabelResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerLabelResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerLabelResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerLabelResult} message MutateCustomerLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerLabelResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerLabelResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelResult} MutateCustomerLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerLabelResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerLabelResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerLabelResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelResult} MutateCustomerLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerLabelResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerLabelResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerLabelResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerLabelResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerLabelResult} MutateCustomerLabelResult
                             */
                            MutateCustomerLabelResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerLabelResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerLabelResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerLabelResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerLabelResult} m MutateCustomerLabelResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerLabelResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerLabelResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerLabelResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerLabelResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerLabelResult;
                        })();
    
                        services.CustomerNegativeCriterionService = (function() {
    
                            /**
                             * Constructs a new CustomerNegativeCriterionService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerNegativeCriterionService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerNegativeCriterionService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerNegativeCriterionService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerNegativeCriterionService;
    
                            /**
                             * Creates new CustomerNegativeCriterionService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerNegativeCriterionService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerNegativeCriterionService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerNegativeCriterionService|mutateCustomerNegativeCriteria}.
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionService
                             * @typedef MutateCustomerNegativeCriteriaCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse} [response] MutateCustomerNegativeCriteriaResponse
                             */
    
                            /**
                             * Calls MutateCustomerNegativeCriteria.
                             * @function mutateCustomerNegativeCriteria
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaRequest} request MutateCustomerNegativeCriteriaRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerNegativeCriterionService.MutateCustomerNegativeCriteriaCallback} callback Node-style callback called with the error, if any, and MutateCustomerNegativeCriteriaResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerNegativeCriterionService.prototype.mutateCustomerNegativeCriteria = function mutateCustomerNegativeCriteria(request, callback) {
                                return this.rpcCall(mutateCustomerNegativeCriteria, $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest, $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse, request, callback);
                            }, "name", { value: "MutateCustomerNegativeCriteria" });
    
                            /**
                             * Calls MutateCustomerNegativeCriteria.
                             * @function mutateCustomerNegativeCriteria
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaRequest} request MutateCustomerNegativeCriteriaRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerNegativeCriterionService;
                        })();
    
                        services.MutateCustomerNegativeCriteriaRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerNegativeCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerNegativeCriteriaRequest
                             * @property {string|null} [customer_id] MutateCustomerNegativeCriteriaRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation>|null} [operations] MutateCustomerNegativeCriteriaRequest operations
                             * @property {boolean|null} [partial_failure] MutateCustomerNegativeCriteriaRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCustomerNegativeCriteriaRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomerNegativeCriteriaRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomerNegativeCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerNegativeCriteriaRequest.
                             * @implements IMutateCustomerNegativeCriteriaRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaRequest=} [p] Properties to set
                             */
                            function MutateCustomerNegativeCriteriaRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerNegativeCriteriaRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerNegativeCriteriaRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerNegativeCriteriaRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCustomerNegativeCriteriaRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomerNegativeCriteriaRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomerNegativeCriteriaRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest} MutateCustomerNegativeCriteriaRequest instance
                             */
                            MutateCustomerNegativeCriteriaRequest.create = function create(properties) {
                                return new MutateCustomerNegativeCriteriaRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerNegativeCriteriaRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaRequest} m MutateCustomerNegativeCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerNegativeCriteriaRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerNegativeCriteriaRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaRequest} message MutateCustomerNegativeCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerNegativeCriteriaRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerNegativeCriteriaRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest} MutateCustomerNegativeCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerNegativeCriteriaRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerNegativeCriteriaRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest} MutateCustomerNegativeCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerNegativeCriteriaRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerNegativeCriteriaRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerNegativeCriteriaRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerNegativeCriteriaRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest} MutateCustomerNegativeCriteriaRequest
                             */
                            MutateCustomerNegativeCriteriaRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerNegativeCriteriaRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest} m MutateCustomerNegativeCriteriaRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerNegativeCriteriaRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerNegativeCriteriaRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerNegativeCriteriaRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerNegativeCriteriaRequest;
                        })();
    
                        services.CustomerNegativeCriterionOperation = (function() {
    
                            /**
                             * Properties of a CustomerNegativeCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerNegativeCriterionOperation
                             * @property {google.ads.googleads.v11.resources.ICustomerNegativeCriterion|null} [create] CustomerNegativeCriterionOperation create
                             * @property {string|null} [remove] CustomerNegativeCriterionOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerNegativeCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerNegativeCriterionOperation.
                             * @implements ICustomerNegativeCriterionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation=} [p] Properties to set
                             */
                            function CustomerNegativeCriterionOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerNegativeCriterionOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerNegativeCriterion|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @instance
                             */
                            CustomerNegativeCriterionOperation.prototype.create = null;
    
                            /**
                             * CustomerNegativeCriterionOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @instance
                             */
                            CustomerNegativeCriterionOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerNegativeCriterionOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerNegativeCriterionOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerNegativeCriterionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerNegativeCriterionOperation} CustomerNegativeCriterionOperation instance
                             */
                            CustomerNegativeCriterionOperation.create = function create(properties) {
                                return new CustomerNegativeCriterionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerNegativeCriterionOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation} m CustomerNegativeCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerNegativeCriterionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerNegativeCriterionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerNegativeCriterionOperation} message CustomerNegativeCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerNegativeCriterionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerNegativeCriterionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerNegativeCriterionOperation} CustomerNegativeCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerNegativeCriterionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerNegativeCriterionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerNegativeCriterionOperation} CustomerNegativeCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerNegativeCriterionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerNegativeCriterionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerNegativeCriterionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerNegativeCriterionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerNegativeCriterionOperation} CustomerNegativeCriterionOperation
                             */
                            CustomerNegativeCriterionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerNegativeCriterionOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerNegativeCriterionOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerNegativeCriterionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerNegativeCriterionOperation} m CustomerNegativeCriterionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerNegativeCriterionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerNegativeCriterionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerNegativeCriterionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerNegativeCriterionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerNegativeCriterionOperation;
                        })();
    
                        services.MutateCustomerNegativeCriteriaResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerNegativeCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerNegativeCriteriaResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCustomerNegativeCriteriaResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult>|null} [results] MutateCustomerNegativeCriteriaResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomerNegativeCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerNegativeCriteriaResponse.
                             * @implements IMutateCustomerNegativeCriteriaResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResponse=} [p] Properties to set
                             */
                            function MutateCustomerNegativeCriteriaResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerNegativeCriteriaResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateCustomerNegativeCriteriaResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomerNegativeCriteriaResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse} MutateCustomerNegativeCriteriaResponse instance
                             */
                            MutateCustomerNegativeCriteriaResponse.create = function create(properties) {
                                return new MutateCustomerNegativeCriteriaResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerNegativeCriteriaResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResponse} m MutateCustomerNegativeCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerNegativeCriteriaResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerNegativeCriteriaResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResponse} message MutateCustomerNegativeCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerNegativeCriteriaResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerNegativeCriteriaResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse} MutateCustomerNegativeCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerNegativeCriteriaResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerNegativeCriteriaResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse} MutateCustomerNegativeCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerNegativeCriteriaResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerNegativeCriteriaResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerNegativeCriteriaResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerNegativeCriteriaResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse} MutateCustomerNegativeCriteriaResponse
                             */
                            MutateCustomerNegativeCriteriaResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerNegativeCriteriaResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse} m MutateCustomerNegativeCriteriaResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerNegativeCriteriaResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerNegativeCriteriaResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerNegativeCriteriaResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerNegativeCriteriaResponse;
                        })();
    
                        services.MutateCustomerNegativeCriteriaResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerNegativeCriteriaResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerNegativeCriteriaResult
                             * @property {string|null} [resource_name] MutateCustomerNegativeCriteriaResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomerNegativeCriterion|null} [customer_negative_criterion] MutateCustomerNegativeCriteriaResult customer_negative_criterion
                             */
    
                            /**
                             * Constructs a new MutateCustomerNegativeCriteriaResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerNegativeCriteriaResult.
                             * @implements IMutateCustomerNegativeCriteriaResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult=} [p] Properties to set
                             */
                            function MutateCustomerNegativeCriteriaResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerNegativeCriteriaResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomerNegativeCriteriaResult customer_negative_criterion.
                             * @member {google.ads.googleads.v11.resources.ICustomerNegativeCriterion|null|undefined} customer_negative_criterion
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @instance
                             */
                            MutateCustomerNegativeCriteriaResult.prototype.customer_negative_criterion = null;
    
                            /**
                             * Creates a new MutateCustomerNegativeCriteriaResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult} MutateCustomerNegativeCriteriaResult instance
                             */
                            MutateCustomerNegativeCriteriaResult.create = function create(properties) {
                                return new MutateCustomerNegativeCriteriaResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerNegativeCriteriaResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult} m MutateCustomerNegativeCriteriaResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerNegativeCriteriaResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customer_negative_criterion != null && Object.hasOwnProperty.call(m, "customer_negative_criterion"))
                                    $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.encode(m.customer_negative_criterion, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerNegativeCriteriaResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerNegativeCriteriaResult} message MutateCustomerNegativeCriteriaResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerNegativeCriteriaResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerNegativeCriteriaResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult} MutateCustomerNegativeCriteriaResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerNegativeCriteriaResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customer_negative_criterion = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerNegativeCriteriaResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult} MutateCustomerNegativeCriteriaResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerNegativeCriteriaResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerNegativeCriteriaResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerNegativeCriteriaResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customer_negative_criterion != null && m.hasOwnProperty("customer_negative_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.verify(m.customer_negative_criterion);
                                        if (e)
                                            return "customer_negative_criterion." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerNegativeCriteriaResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult} MutateCustomerNegativeCriteriaResult
                             */
                            MutateCustomerNegativeCriteriaResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customer_negative_criterion != null) {
                                    if (typeof d.customer_negative_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult.customer_negative_criterion: object expected");
                                    m.customer_negative_criterion = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.fromObject(d.customer_negative_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerNegativeCriteriaResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult} m MutateCustomerNegativeCriteriaResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerNegativeCriteriaResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customer_negative_criterion = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customer_negative_criterion != null && m.hasOwnProperty("customer_negative_criterion")) {
                                    d.customer_negative_criterion = $root.google.ads.googleads.v11.resources.CustomerNegativeCriterion.toObject(m.customer_negative_criterion, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerNegativeCriteriaResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerNegativeCriteriaResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerNegativeCriteriaResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerNegativeCriteriaResult;
                        })();
    
                        services.CustomerService = (function() {
    
                            /**
                             * Constructs a new CustomerService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerService;
    
                            /**
                             * Creates new CustomerService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerService|mutateCustomer}.
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @typedef MutateCustomerCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerResponse} [response] MutateCustomerResponse
                             */
    
                            /**
                             * Calls MutateCustomer.
                             * @function mutateCustomer
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerRequest} request MutateCustomerRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerService.MutateCustomerCallback} callback Node-style callback called with the error, if any, and MutateCustomerResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerService.prototype.mutateCustomer = function mutateCustomer(request, callback) {
                                return this.rpcCall(mutateCustomer, $root.google.ads.googleads.v11.services.MutateCustomerRequest, $root.google.ads.googleads.v11.services.MutateCustomerResponse, request, callback);
                            }, "name", { value: "MutateCustomer" });
    
                            /**
                             * Calls MutateCustomer.
                             * @function mutateCustomer
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerRequest} request MutateCustomerRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerService|listAccessibleCustomers}.
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @typedef ListAccessibleCustomersCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListAccessibleCustomersResponse} [response] ListAccessibleCustomersResponse
                             */
    
                            /**
                             * Calls ListAccessibleCustomers.
                             * @function listAccessibleCustomers
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersRequest} request ListAccessibleCustomersRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerService.ListAccessibleCustomersCallback} callback Node-style callback called with the error, if any, and ListAccessibleCustomersResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerService.prototype.listAccessibleCustomers = function listAccessibleCustomers(request, callback) {
                                return this.rpcCall(listAccessibleCustomers, $root.google.ads.googleads.v11.services.ListAccessibleCustomersRequest, $root.google.ads.googleads.v11.services.ListAccessibleCustomersResponse, request, callback);
                            }, "name", { value: "ListAccessibleCustomers" });
    
                            /**
                             * Calls ListAccessibleCustomers.
                             * @function listAccessibleCustomers
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersRequest} request ListAccessibleCustomersRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListAccessibleCustomersResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerService|createCustomerClient}.
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @typedef CreateCustomerClientCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.CreateCustomerClientResponse} [response] CreateCustomerClientResponse
                             */
    
                            /**
                             * Calls CreateCustomerClient.
                             * @function createCustomerClient
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientRequest} request CreateCustomerClientRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerService.CreateCustomerClientCallback} callback Node-style callback called with the error, if any, and CreateCustomerClientResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerService.prototype.createCustomerClient = function createCustomerClient(request, callback) {
                                return this.rpcCall(createCustomerClient, $root.google.ads.googleads.v11.services.CreateCustomerClientRequest, $root.google.ads.googleads.v11.services.CreateCustomerClientResponse, request, callback);
                            }, "name", { value: "CreateCustomerClient" });
    
                            /**
                             * Calls CreateCustomerClient.
                             * @function createCustomerClient
                             * @memberof google.ads.googleads.v11.services.CustomerService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientRequest} request CreateCustomerClientRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.CreateCustomerClientResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerService;
                        })();
    
                        services.MutateCustomerRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerRequest
                             * @property {string|null} [customer_id] MutateCustomerRequest customer_id
                             * @property {google.ads.googleads.v11.services.ICustomerOperation|null} [operation] MutateCustomerRequest operation
                             * @property {boolean|null} [validate_only] MutateCustomerRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomerRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomerRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerRequest.
                             * @implements IMutateCustomerRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerRequest=} [p] Properties to set
                             */
                            function MutateCustomerRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @instance
                             */
                            MutateCustomerRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerRequest operation.
                             * @member {google.ads.googleads.v11.services.ICustomerOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @instance
                             */
                            MutateCustomerRequest.prototype.operation = null;
    
                            /**
                             * MutateCustomerRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @instance
                             */
                            MutateCustomerRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomerRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @instance
                             */
                            MutateCustomerRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomerRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerRequest} MutateCustomerRequest instance
                             */
                            MutateCustomerRequest.create = function create(properties) {
                                return new MutateCustomerRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerRequest} m MutateCustomerRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.CustomerOperation.encode(m.operation, w.uint32(34).fork()).ldelim();
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(40).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(48).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerRequest} message MutateCustomerRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerRequest} MutateCustomerRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 4:
                                        m.operation = $root.google.ads.googleads.v11.services.CustomerOperation.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.validate_only = r.bool();
                                        break;
                                    case 6:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerRequest} MutateCustomerRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerRequest} MutateCustomerRequest
                             */
                            MutateCustomerRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.CustomerOperation.fromObject(d.operation);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerRequest} m MutateCustomerRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.CustomerOperation.toObject(m.operation, o);
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerRequest;
                        })();
    
                        services.CreateCustomerClientRequest = (function() {
    
                            /**
                             * Properties of a CreateCustomerClientRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateCustomerClientRequest
                             * @property {string|null} [customer_id] CreateCustomerClientRequest customer_id
                             * @property {google.ads.googleads.v11.resources.ICustomer|null} [customer_client] CreateCustomerClientRequest customer_client
                             * @property {string|null} [email_address] CreateCustomerClientRequest email_address
                             * @property {google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole|null} [access_role] CreateCustomerClientRequest access_role
                             * @property {boolean|null} [validate_only] CreateCustomerClientRequest validate_only
                             */
    
                            /**
                             * Constructs a new CreateCustomerClientRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateCustomerClientRequest.
                             * @implements ICreateCustomerClientRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientRequest=} [p] Properties to set
                             */
                            function CreateCustomerClientRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateCustomerClientRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @instance
                             */
                            CreateCustomerClientRequest.prototype.customer_id = "";
    
                            /**
                             * CreateCustomerClientRequest customer_client.
                             * @member {google.ads.googleads.v11.resources.ICustomer|null|undefined} customer_client
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @instance
                             */
                            CreateCustomerClientRequest.prototype.customer_client = null;
    
                            /**
                             * CreateCustomerClientRequest email_address.
                             * @member {string|null|undefined} email_address
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @instance
                             */
                            CreateCustomerClientRequest.prototype.email_address = null;
    
                            /**
                             * CreateCustomerClientRequest access_role.
                             * @member {google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole} access_role
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @instance
                             */
                            CreateCustomerClientRequest.prototype.access_role = 0;
    
                            /**
                             * CreateCustomerClientRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @instance
                             */
                            CreateCustomerClientRequest.prototype.validate_only = false;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CreateCustomerClientRequest _email_address.
                             * @member {"email_address"|undefined} _email_address
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @instance
                             */
                            Object.defineProperty(CreateCustomerClientRequest.prototype, "_email_address", {
                                get: $util.oneOfGetter($oneOfFields = ["email_address"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CreateCustomerClientRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientRequest} CreateCustomerClientRequest instance
                             */
                            CreateCustomerClientRequest.create = function create(properties) {
                                return new CreateCustomerClientRequest(properties);
                            };
    
                            /**
                             * Encodes the specified CreateCustomerClientRequest message. Does not implicitly {@link google.ads.googleads.v11.services.CreateCustomerClientRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientRequest} m CreateCustomerClientRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCustomerClientRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.customer_client != null && Object.hasOwnProperty.call(m, "customer_client"))
                                    $root.google.ads.googleads.v11.resources.Customer.encode(m.customer_client, w.uint32(18).fork()).ldelim();
                                if (m.access_role != null && Object.hasOwnProperty.call(m, "access_role"))
                                    w.uint32(32).int32(m.access_role);
                                if (m.email_address != null && Object.hasOwnProperty.call(m, "email_address"))
                                    w.uint32(42).string(m.email_address);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(48).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateCustomerClientRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateCustomerClientRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientRequest} message CreateCustomerClientRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCustomerClientRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateCustomerClientRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientRequest} CreateCustomerClientRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCustomerClientRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateCustomerClientRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.customer_client = $root.google.ads.googleads.v11.resources.Customer.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.email_address = r.string();
                                        break;
                                    case 4:
                                        m.access_role = r.int32();
                                        break;
                                    case 6:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateCustomerClientRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientRequest} CreateCustomerClientRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCustomerClientRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateCustomerClientRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateCustomerClientRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.customer_client != null && m.hasOwnProperty("customer_client")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Customer.verify(m.customer_client);
                                        if (e)
                                            return "customer_client." + e;
                                    }
                                }
                                if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                    p._email_address = 1;
                                    if (!$util.isString(m.email_address))
                                        return "email_address: string expected";
                                }
                                if (m.access_role != null && m.hasOwnProperty("access_role")) {
                                    switch (m.access_role) {
                                    default:
                                        return "access_role: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateCustomerClientRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientRequest} CreateCustomerClientRequest
                             */
                            CreateCustomerClientRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateCustomerClientRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateCustomerClientRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.customer_client != null) {
                                    if (typeof d.customer_client !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CreateCustomerClientRequest.customer_client: object expected");
                                    m.customer_client = $root.google.ads.googleads.v11.resources.Customer.fromObject(d.customer_client);
                                }
                                if (d.email_address != null) {
                                    m.email_address = String(d.email_address);
                                }
                                switch (d.access_role) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.access_role = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.access_role = 1;
                                    break;
                                case "ADMIN":
                                case 2:
                                    m.access_role = 2;
                                    break;
                                case "STANDARD":
                                case 3:
                                    m.access_role = 3;
                                    break;
                                case "READ_ONLY":
                                case 4:
                                    m.access_role = 4;
                                    break;
                                case "EMAIL_ONLY":
                                case 5:
                                    m.access_role = 5;
                                    break;
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateCustomerClientRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateCustomerClientRequest} m CreateCustomerClientRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateCustomerClientRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.customer_client = null;
                                    d.access_role = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.customer_client != null && m.hasOwnProperty("customer_client")) {
                                    d.customer_client = $root.google.ads.googleads.v11.resources.Customer.toObject(m.customer_client, o);
                                }
                                if (m.access_role != null && m.hasOwnProperty("access_role")) {
                                    d.access_role = o.enums === String ? $root.google.ads.googleads.v11.enums.AccessRoleEnum.AccessRole[m.access_role] : m.access_role;
                                }
                                if (m.email_address != null && m.hasOwnProperty("email_address")) {
                                    d.email_address = m.email_address;
                                    if (o.oneofs)
                                        d._email_address = "email_address";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateCustomerClientRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateCustomerClientRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateCustomerClientRequest;
                        })();
    
                        services.CustomerOperation = (function() {
    
                            /**
                             * Properties of a CustomerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerOperation
                             * @property {google.ads.googleads.v11.resources.ICustomer|null} [update] CustomerOperation update
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerOperation update_mask
                             */
    
                            /**
                             * Constructs a new CustomerOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerOperation.
                             * @implements ICustomerOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerOperation=} [p] Properties to set
                             */
                            function CustomerOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomer|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @instance
                             */
                            CustomerOperation.prototype.update = null;
    
                            /**
                             * CustomerOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @instance
                             */
                            CustomerOperation.prototype.update_mask = null;
    
                            /**
                             * Creates a new CustomerOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerOperation} CustomerOperation instance
                             */
                            CustomerOperation.create = function create(properties) {
                                return new CustomerOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerOperation} m CustomerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Customer.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerOperation} message CustomerOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerOperation} CustomerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.Customer.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerOperation} CustomerOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Customer.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerOperation} CustomerOperation
                             */
                            CustomerOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerOperation();
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Customer.fromObject(d.update);
                                }
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerOperation} m CustomerOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update = null;
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Customer.toObject(m.update, o);
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerOperation;
                        })();
    
                        services.CreateCustomerClientResponse = (function() {
    
                            /**
                             * Properties of a CreateCustomerClientResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateCustomerClientResponse
                             * @property {string|null} [resource_name] CreateCustomerClientResponse resource_name
                             * @property {string|null} [invitation_link] CreateCustomerClientResponse invitation_link
                             */
    
                            /**
                             * Constructs a new CreateCustomerClientResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateCustomerClientResponse.
                             * @implements ICreateCustomerClientResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientResponse=} [p] Properties to set
                             */
                            function CreateCustomerClientResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateCustomerClientResponse resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @instance
                             */
                            CreateCustomerClientResponse.prototype.resource_name = "";
    
                            /**
                             * CreateCustomerClientResponse invitation_link.
                             * @member {string} invitation_link
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @instance
                             */
                            CreateCustomerClientResponse.prototype.invitation_link = "";
    
                            /**
                             * Creates a new CreateCustomerClientResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientResponse} CreateCustomerClientResponse instance
                             */
                            CreateCustomerClientResponse.create = function create(properties) {
                                return new CreateCustomerClientResponse(properties);
                            };
    
                            /**
                             * Encodes the specified CreateCustomerClientResponse message. Does not implicitly {@link google.ads.googleads.v11.services.CreateCustomerClientResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientResponse} m CreateCustomerClientResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCustomerClientResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(18).string(m.resource_name);
                                if (m.invitation_link != null && Object.hasOwnProperty.call(m, "invitation_link"))
                                    w.uint32(26).string(m.invitation_link);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateCustomerClientResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateCustomerClientResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateCustomerClientResponse} message CreateCustomerClientResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateCustomerClientResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateCustomerClientResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientResponse} CreateCustomerClientResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCustomerClientResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateCustomerClientResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.resource_name = r.string();
                                        break;
                                    case 3:
                                        m.invitation_link = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateCustomerClientResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientResponse} CreateCustomerClientResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateCustomerClientResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateCustomerClientResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateCustomerClientResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.invitation_link != null && m.hasOwnProperty("invitation_link")) {
                                    if (!$util.isString(m.invitation_link))
                                        return "invitation_link: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateCustomerClientResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateCustomerClientResponse} CreateCustomerClientResponse
                             */
                            CreateCustomerClientResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateCustomerClientResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateCustomerClientResponse();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.invitation_link != null) {
                                    m.invitation_link = String(d.invitation_link);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateCustomerClientResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateCustomerClientResponse} m CreateCustomerClientResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateCustomerClientResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.invitation_link = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.invitation_link != null && m.hasOwnProperty("invitation_link")) {
                                    d.invitation_link = m.invitation_link;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateCustomerClientResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateCustomerClientResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateCustomerClientResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateCustomerClientResponse;
                        })();
    
                        services.MutateCustomerResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerResponse
                             * @property {google.ads.googleads.v11.services.IMutateCustomerResult|null} [result] MutateCustomerResponse result
                             */
    
                            /**
                             * Constructs a new MutateCustomerResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerResponse.
                             * @implements IMutateCustomerResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResponse=} [p] Properties to set
                             */
                            function MutateCustomerResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @instance
                             */
                            MutateCustomerResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateCustomerResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResponse} MutateCustomerResponse instance
                             */
                            MutateCustomerResponse.create = function create(properties) {
                                return new MutateCustomerResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResponse} m MutateCustomerResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerResult.encode(m.result, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResponse} message MutateCustomerResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResponse} MutateCustomerResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.result = $root.google.ads.googleads.v11.services.MutateCustomerResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResponse} MutateCustomerResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResponse} MutateCustomerResponse
                             */
                            MutateCustomerResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateCustomerResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerResponse} m MutateCustomerResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateCustomerResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerResponse;
                        })();
    
                        services.MutateCustomerResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerResult
                             * @property {string|null} [resource_name] MutateCustomerResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomer|null} [customer] MutateCustomerResult customer
                             */
    
                            /**
                             * Constructs a new MutateCustomerResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerResult.
                             * @implements IMutateCustomerResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResult=} [p] Properties to set
                             */
                            function MutateCustomerResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @instance
                             */
                            MutateCustomerResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomerResult customer.
                             * @member {google.ads.googleads.v11.resources.ICustomer|null|undefined} customer
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @instance
                             */
                            MutateCustomerResult.prototype.customer = null;
    
                            /**
                             * Creates a new MutateCustomerResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResult} MutateCustomerResult instance
                             */
                            MutateCustomerResult.create = function create(properties) {
                                return new MutateCustomerResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResult} m MutateCustomerResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customer != null && Object.hasOwnProperty.call(m, "customer"))
                                    $root.google.ads.googleads.v11.resources.Customer.encode(m.customer, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerResult} message MutateCustomerResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResult} MutateCustomerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customer = $root.google.ads.googleads.v11.resources.Customer.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResult} MutateCustomerResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Customer.verify(m.customer);
                                        if (e)
                                            return "customer." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerResult} MutateCustomerResult
                             */
                            MutateCustomerResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customer != null) {
                                    if (typeof d.customer !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerResult.customer: object expected");
                                    m.customer = $root.google.ads.googleads.v11.resources.Customer.fromObject(d.customer);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerResult} m MutateCustomerResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customer = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customer != null && m.hasOwnProperty("customer")) {
                                    d.customer = $root.google.ads.googleads.v11.resources.Customer.toObject(m.customer, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerResult;
                        })();
    
                        services.ListAccessibleCustomersRequest = (function() {
    
                            /**
                             * Properties of a ListAccessibleCustomersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListAccessibleCustomersRequest
                             */
    
                            /**
                             * Constructs a new ListAccessibleCustomersRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListAccessibleCustomersRequest.
                             * @implements IListAccessibleCustomersRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersRequest=} [p] Properties to set
                             */
                            function ListAccessibleCustomersRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListAccessibleCustomersRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersRequest} ListAccessibleCustomersRequest instance
                             */
                            ListAccessibleCustomersRequest.create = function create(properties) {
                                return new ListAccessibleCustomersRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListAccessibleCustomersRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListAccessibleCustomersRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersRequest} m ListAccessibleCustomersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAccessibleCustomersRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListAccessibleCustomersRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListAccessibleCustomersRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersRequest} message ListAccessibleCustomersRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAccessibleCustomersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListAccessibleCustomersRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersRequest} ListAccessibleCustomersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAccessibleCustomersRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListAccessibleCustomersRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListAccessibleCustomersRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersRequest} ListAccessibleCustomersRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAccessibleCustomersRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListAccessibleCustomersRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListAccessibleCustomersRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListAccessibleCustomersRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersRequest} ListAccessibleCustomersRequest
                             */
                            ListAccessibleCustomersRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListAccessibleCustomersRequest)
                                    return d;
                                return new $root.google.ads.googleads.v11.services.ListAccessibleCustomersRequest();
                            };
    
                            /**
                             * Creates a plain object from a ListAccessibleCustomersRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListAccessibleCustomersRequest} m ListAccessibleCustomersRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListAccessibleCustomersRequest.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListAccessibleCustomersRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListAccessibleCustomersRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListAccessibleCustomersRequest;
                        })();
    
                        services.ListAccessibleCustomersResponse = (function() {
    
                            /**
                             * Properties of a ListAccessibleCustomersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListAccessibleCustomersResponse
                             * @property {Array.<string>|null} [resource_names] ListAccessibleCustomersResponse resource_names
                             */
    
                            /**
                             * Constructs a new ListAccessibleCustomersResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListAccessibleCustomersResponse.
                             * @implements IListAccessibleCustomersResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersResponse=} [p] Properties to set
                             */
                            function ListAccessibleCustomersResponse(p) {
                                this.resource_names = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListAccessibleCustomersResponse resource_names.
                             * @member {Array.<string>} resource_names
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @instance
                             */
                            ListAccessibleCustomersResponse.prototype.resource_names = $util.emptyArray;
    
                            /**
                             * Creates a new ListAccessibleCustomersResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersResponse} ListAccessibleCustomersResponse instance
                             */
                            ListAccessibleCustomersResponse.create = function create(properties) {
                                return new ListAccessibleCustomersResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListAccessibleCustomersResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListAccessibleCustomersResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersResponse} m ListAccessibleCustomersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAccessibleCustomersResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_names != null && m.resource_names.length) {
                                    for (var i = 0; i < m.resource_names.length; ++i)
                                        w.uint32(10).string(m.resource_names[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListAccessibleCustomersResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListAccessibleCustomersResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListAccessibleCustomersResponse} message ListAccessibleCustomersResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListAccessibleCustomersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListAccessibleCustomersResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersResponse} ListAccessibleCustomersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAccessibleCustomersResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListAccessibleCustomersResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.resource_names && m.resource_names.length))
                                            m.resource_names = [];
                                        m.resource_names.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListAccessibleCustomersResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersResponse} ListAccessibleCustomersResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListAccessibleCustomersResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListAccessibleCustomersResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListAccessibleCustomersResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_names != null && m.hasOwnProperty("resource_names")) {
                                    if (!Array.isArray(m.resource_names))
                                        return "resource_names: array expected";
                                    for (var i = 0; i < m.resource_names.length; ++i) {
                                        if (!$util.isString(m.resource_names[i]))
                                            return "resource_names: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListAccessibleCustomersResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListAccessibleCustomersResponse} ListAccessibleCustomersResponse
                             */
                            ListAccessibleCustomersResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListAccessibleCustomersResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListAccessibleCustomersResponse();
                                if (d.resource_names) {
                                    if (!Array.isArray(d.resource_names))
                                        throw TypeError(".google.ads.googleads.v11.services.ListAccessibleCustomersResponse.resource_names: array expected");
                                    m.resource_names = [];
                                    for (var i = 0; i < d.resource_names.length; ++i) {
                                        m.resource_names[i] = String(d.resource_names[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListAccessibleCustomersResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListAccessibleCustomersResponse} m ListAccessibleCustomersResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListAccessibleCustomersResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.resource_names = [];
                                }
                                if (m.resource_names && m.resource_names.length) {
                                    d.resource_names = [];
                                    for (var j = 0; j < m.resource_names.length; ++j) {
                                        d.resource_names[j] = m.resource_names[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListAccessibleCustomersResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListAccessibleCustomersResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListAccessibleCustomersResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListAccessibleCustomersResponse;
                        })();
    
                        services.CustomizerAttributeService = (function() {
    
                            /**
                             * Constructs a new CustomizerAttributeService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomizerAttributeService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomizerAttributeService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomizerAttributeService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomizerAttributeService;
    
                            /**
                             * Creates new CustomizerAttributeService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomizerAttributeService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomizerAttributeService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomizerAttributeService|mutateCustomizerAttributes}.
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeService
                             * @typedef MutateCustomizerAttributesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomizerAttributesResponse} [response] MutateCustomizerAttributesResponse
                             */
    
                            /**
                             * Calls MutateCustomizerAttributes.
                             * @function mutateCustomizerAttributes
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesRequest} request MutateCustomizerAttributesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomizerAttributeService.MutateCustomizerAttributesCallback} callback Node-style callback called with the error, if any, and MutateCustomizerAttributesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomizerAttributeService.prototype.mutateCustomizerAttributes = function mutateCustomizerAttributes(request, callback) {
                                return this.rpcCall(mutateCustomizerAttributes, $root.google.ads.googleads.v11.services.MutateCustomizerAttributesRequest, $root.google.ads.googleads.v11.services.MutateCustomizerAttributesResponse, request, callback);
                            }, "name", { value: "MutateCustomizerAttributes" });
    
                            /**
                             * Calls MutateCustomizerAttributes.
                             * @function mutateCustomizerAttributes
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesRequest} request MutateCustomizerAttributesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomizerAttributesResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomizerAttributeService;
                        })();
    
                        services.MutateCustomizerAttributesRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomizerAttributesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomizerAttributesRequest
                             * @property {string|null} [customer_id] MutateCustomizerAttributesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomizerAttributeOperation>|null} [operations] MutateCustomizerAttributesRequest operations
                             * @property {boolean|null} [partial_failure] MutateCustomizerAttributesRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateCustomizerAttributesRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateCustomizerAttributesRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateCustomizerAttributesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomizerAttributesRequest.
                             * @implements IMutateCustomizerAttributesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesRequest=} [p] Properties to set
                             */
                            function MutateCustomizerAttributesRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomizerAttributesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @instance
                             */
                            MutateCustomizerAttributesRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomizerAttributesRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomizerAttributeOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @instance
                             */
                            MutateCustomizerAttributesRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomizerAttributesRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @instance
                             */
                            MutateCustomizerAttributesRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateCustomizerAttributesRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @instance
                             */
                            MutateCustomizerAttributesRequest.prototype.validate_only = false;
    
                            /**
                             * MutateCustomizerAttributesRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @instance
                             */
                            MutateCustomizerAttributesRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateCustomizerAttributesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesRequest} MutateCustomizerAttributesRequest instance
                             */
                            MutateCustomizerAttributesRequest.create = function create(properties) {
                                return new MutateCustomizerAttributesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomizerAttributesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomizerAttributesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesRequest} m MutateCustomizerAttributesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomizerAttributesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomizerAttributesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomizerAttributesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesRequest} message MutateCustomizerAttributesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomizerAttributesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomizerAttributesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesRequest} MutateCustomizerAttributesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomizerAttributesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomizerAttributesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomizerAttributeOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomizerAttributesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesRequest} MutateCustomizerAttributesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomizerAttributesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomizerAttributesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomizerAttributesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomizerAttributesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesRequest} MutateCustomizerAttributesRequest
                             */
                            MutateCustomizerAttributesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomizerAttributesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomizerAttributesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomizerAttributesRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomizerAttributesRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomizerAttributesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomizerAttributesRequest} m MutateCustomizerAttributesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomizerAttributesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomizerAttributeOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomizerAttributesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomizerAttributesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomizerAttributesRequest;
                        })();
    
                        services.CustomizerAttributeOperation = (function() {
    
                            /**
                             * Properties of a CustomizerAttributeOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomizerAttributeOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomizerAttributeOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomizerAttribute|null} [create] CustomizerAttributeOperation create
                             * @property {string|null} [remove] CustomizerAttributeOperation remove
                             */
    
                            /**
                             * Constructs a new CustomizerAttributeOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomizerAttributeOperation.
                             * @implements ICustomizerAttributeOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomizerAttributeOperation=} [p] Properties to set
                             */
                            function CustomizerAttributeOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomizerAttributeOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @instance
                             */
                            CustomizerAttributeOperation.prototype.update_mask = null;
    
                            /**
                             * CustomizerAttributeOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomizerAttribute|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @instance
                             */
                            CustomizerAttributeOperation.prototype.create = null;
    
                            /**
                             * CustomizerAttributeOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @instance
                             */
                            CustomizerAttributeOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomizerAttributeOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @instance
                             */
                            Object.defineProperty(CustomizerAttributeOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomizerAttributeOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomizerAttributeOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomizerAttributeOperation} CustomizerAttributeOperation instance
                             */
                            CustomizerAttributeOperation.create = function create(properties) {
                                return new CustomizerAttributeOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomizerAttributeOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomizerAttributeOperation} m CustomizerAttributeOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomizerAttribute.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomizerAttributeOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomizerAttributeOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomizerAttributeOperation} message CustomizerAttributeOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomizerAttributeOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomizerAttributeOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomizerAttributeOperation} CustomizerAttributeOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomizerAttributeOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomizerAttribute.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomizerAttributeOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomizerAttributeOperation} CustomizerAttributeOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomizerAttributeOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomizerAttributeOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomizerAttributeOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomizerAttribute.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomizerAttributeOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomizerAttributeOperation} CustomizerAttributeOperation
                             */
                            CustomizerAttributeOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomizerAttributeOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomizerAttributeOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomizerAttributeOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomizerAttributeOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomizerAttribute.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomizerAttributeOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomizerAttributeOperation} m CustomizerAttributeOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomizerAttributeOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomizerAttribute.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomizerAttributeOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomizerAttributeOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomizerAttributeOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomizerAttributeOperation;
                        })();
    
                        services.MutateCustomizerAttributesResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomizerAttributesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomizerAttributesResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomizerAttributeResult>|null} [results] MutateCustomizerAttributesResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateCustomizerAttributesResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateCustomizerAttributesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomizerAttributesResponse.
                             * @implements IMutateCustomizerAttributesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesResponse=} [p] Properties to set
                             */
                            function MutateCustomizerAttributesResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomizerAttributesResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomizerAttributeResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @instance
                             */
                            MutateCustomizerAttributesResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateCustomizerAttributesResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @instance
                             */
                            MutateCustomizerAttributesResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateCustomizerAttributesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesResponse} MutateCustomizerAttributesResponse instance
                             */
                            MutateCustomizerAttributesResponse.create = function create(properties) {
                                return new MutateCustomizerAttributesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomizerAttributesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomizerAttributesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesResponse} m MutateCustomizerAttributesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomizerAttributesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomizerAttributesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomizerAttributesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributesResponse} message MutateCustomizerAttributesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomizerAttributesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomizerAttributesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesResponse} MutateCustomizerAttributesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomizerAttributesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomizerAttributesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomizerAttributesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesResponse} MutateCustomizerAttributesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomizerAttributesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomizerAttributesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomizerAttributesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomizerAttributesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributesResponse} MutateCustomizerAttributesResponse
                             */
                            MutateCustomizerAttributesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomizerAttributesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomizerAttributesResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomizerAttributesResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomizerAttributesResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomizerAttributesResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomizerAttributesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomizerAttributesResponse} m MutateCustomizerAttributesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomizerAttributesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomizerAttributesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomizerAttributesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomizerAttributesResponse;
                        })();
    
                        services.MutateCustomizerAttributeResult = (function() {
    
                            /**
                             * Properties of a MutateCustomizerAttributeResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomizerAttributeResult
                             * @property {string|null} [resource_name] MutateCustomizerAttributeResult resource_name
                             * @property {google.ads.googleads.v11.resources.ICustomizerAttribute|null} [customizer_attribute] MutateCustomizerAttributeResult customizer_attribute
                             */
    
                            /**
                             * Constructs a new MutateCustomizerAttributeResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomizerAttributeResult.
                             * @implements IMutateCustomizerAttributeResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributeResult=} [p] Properties to set
                             */
                            function MutateCustomizerAttributeResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomizerAttributeResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @instance
                             */
                            MutateCustomizerAttributeResult.prototype.resource_name = "";
    
                            /**
                             * MutateCustomizerAttributeResult customizer_attribute.
                             * @member {google.ads.googleads.v11.resources.ICustomizerAttribute|null|undefined} customizer_attribute
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @instance
                             */
                            MutateCustomizerAttributeResult.prototype.customizer_attribute = null;
    
                            /**
                             * Creates a new MutateCustomizerAttributeResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributeResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributeResult} MutateCustomizerAttributeResult instance
                             */
                            MutateCustomizerAttributeResult.create = function create(properties) {
                                return new MutateCustomizerAttributeResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomizerAttributeResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomizerAttributeResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributeResult} m MutateCustomizerAttributeResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomizerAttributeResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.customizer_attribute != null && Object.hasOwnProperty.call(m, "customizer_attribute"))
                                    $root.google.ads.googleads.v11.resources.CustomizerAttribute.encode(m.customizer_attribute, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomizerAttributeResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomizerAttributeResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomizerAttributeResult} message MutateCustomizerAttributeResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomizerAttributeResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomizerAttributeResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributeResult} MutateCustomizerAttributeResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomizerAttributeResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.customizer_attribute = $root.google.ads.googleads.v11.resources.CustomizerAttribute.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomizerAttributeResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributeResult} MutateCustomizerAttributeResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomizerAttributeResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomizerAttributeResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomizerAttributeResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomizerAttribute.verify(m.customizer_attribute);
                                        if (e)
                                            return "customizer_attribute." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomizerAttributeResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomizerAttributeResult} MutateCustomizerAttributeResult
                             */
                            MutateCustomizerAttributeResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomizerAttributeResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.customizer_attribute != null) {
                                    if (typeof d.customizer_attribute !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomizerAttributeResult.customizer_attribute: object expected");
                                    m.customizer_attribute = $root.google.ads.googleads.v11.resources.CustomizerAttribute.fromObject(d.customizer_attribute);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomizerAttributeResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomizerAttributeResult} m MutateCustomizerAttributeResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomizerAttributeResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.customizer_attribute = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.customizer_attribute != null && m.hasOwnProperty("customizer_attribute")) {
                                    d.customizer_attribute = $root.google.ads.googleads.v11.resources.CustomizerAttribute.toObject(m.customizer_attribute, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomizerAttributeResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomizerAttributeResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomizerAttributeResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomizerAttributeResult;
                        })();
    
                        services.ExperimentArmService = (function() {
    
                            /**
                             * Constructs a new ExperimentArmService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ExperimentArmService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ExperimentArmService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ExperimentArmService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ExperimentArmService;
    
                            /**
                             * Creates new ExperimentArmService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ExperimentArmService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ExperimentArmService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ExperimentArmService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExperimentArmService|mutateExperimentArms}.
                             * @memberof google.ads.googleads.v11.services.ExperimentArmService
                             * @typedef MutateExperimentArmsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateExperimentArmsResponse} [response] MutateExperimentArmsResponse
                             */
    
                            /**
                             * Calls MutateExperimentArms.
                             * @function mutateExperimentArms
                             * @memberof google.ads.googleads.v11.services.ExperimentArmService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsRequest} request MutateExperimentArmsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExperimentArmService.MutateExperimentArmsCallback} callback Node-style callback called with the error, if any, and MutateExperimentArmsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExperimentArmService.prototype.mutateExperimentArms = function mutateExperimentArms(request, callback) {
                                return this.rpcCall(mutateExperimentArms, $root.google.ads.googleads.v11.services.MutateExperimentArmsRequest, $root.google.ads.googleads.v11.services.MutateExperimentArmsResponse, request, callback);
                            }, "name", { value: "MutateExperimentArms" });
    
                            /**
                             * Calls MutateExperimentArms.
                             * @function mutateExperimentArms
                             * @memberof google.ads.googleads.v11.services.ExperimentArmService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsRequest} request MutateExperimentArmsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateExperimentArmsResponse>} Promise
                             * @variation 2
                             */
    
                            return ExperimentArmService;
                        })();
    
                        services.MutateExperimentArmsRequest = (function() {
    
                            /**
                             * Properties of a MutateExperimentArmsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExperimentArmsRequest
                             * @property {string|null} [customer_id] MutateExperimentArmsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IExperimentArmOperation>|null} [operations] MutateExperimentArmsRequest operations
                             * @property {boolean|null} [partial_failure] MutateExperimentArmsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateExperimentArmsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateExperimentArmsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateExperimentArmsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExperimentArmsRequest.
                             * @implements IMutateExperimentArmsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsRequest=} [p] Properties to set
                             */
                            function MutateExperimentArmsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExperimentArmsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @instance
                             */
                            MutateExperimentArmsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateExperimentArmsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IExperimentArmOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @instance
                             */
                            MutateExperimentArmsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateExperimentArmsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @instance
                             */
                            MutateExperimentArmsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateExperimentArmsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @instance
                             */
                            MutateExperimentArmsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateExperimentArmsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @instance
                             */
                            MutateExperimentArmsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateExperimentArmsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsRequest} MutateExperimentArmsRequest instance
                             */
                            MutateExperimentArmsRequest.create = function create(properties) {
                                return new MutateExperimentArmsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExperimentArmsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentArmsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsRequest} m MutateExperimentArmsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentArmsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ExperimentArmOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExperimentArmsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentArmsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsRequest} message MutateExperimentArmsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentArmsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExperimentArmsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsRequest} MutateExperimentArmsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentArmsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExperimentArmsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ExperimentArmOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExperimentArmsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsRequest} MutateExperimentArmsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentArmsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExperimentArmsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExperimentArmsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ExperimentArmOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExperimentArmsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsRequest} MutateExperimentArmsRequest
                             */
                            MutateExperimentArmsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExperimentArmsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExperimentArmsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExperimentArmsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateExperimentArmsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ExperimentArmOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExperimentArmsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExperimentArmsRequest} m MutateExperimentArmsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExperimentArmsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ExperimentArmOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExperimentArmsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExperimentArmsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExperimentArmsRequest;
                        })();
    
                        services.ExperimentArmOperation = (function() {
    
                            /**
                             * Properties of an ExperimentArmOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IExperimentArmOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ExperimentArmOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IExperimentArm|null} [create] ExperimentArmOperation create
                             * @property {google.ads.googleads.v11.resources.IExperimentArm|null} [update] ExperimentArmOperation update
                             * @property {string|null} [remove] ExperimentArmOperation remove
                             */
    
                            /**
                             * Constructs a new ExperimentArmOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ExperimentArmOperation.
                             * @implements IExperimentArmOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IExperimentArmOperation=} [p] Properties to set
                             */
                            function ExperimentArmOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExperimentArmOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @instance
                             */
                            ExperimentArmOperation.prototype.update_mask = null;
    
                            /**
                             * ExperimentArmOperation create.
                             * @member {google.ads.googleads.v11.resources.IExperimentArm|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @instance
                             */
                            ExperimentArmOperation.prototype.create = null;
    
                            /**
                             * ExperimentArmOperation update.
                             * @member {google.ads.googleads.v11.resources.IExperimentArm|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @instance
                             */
                            ExperimentArmOperation.prototype.update = null;
    
                            /**
                             * ExperimentArmOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @instance
                             */
                            ExperimentArmOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExperimentArmOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @instance
                             */
                            Object.defineProperty(ExperimentArmOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExperimentArmOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExperimentArmOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ExperimentArmOperation} ExperimentArmOperation instance
                             */
                            ExperimentArmOperation.create = function create(properties) {
                                return new ExperimentArmOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentArmOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ExperimentArmOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExperimentArmOperation} m ExperimentArmOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentArmOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.ExperimentArm.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.ExperimentArm.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentArmOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ExperimentArmOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExperimentArmOperation} message ExperimentArmOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentArmOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentArmOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ExperimentArmOperation} ExperimentArmOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentArmOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ExperimentArmOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.ExperimentArm.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.ExperimentArm.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentArmOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ExperimentArmOperation} ExperimentArmOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentArmOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentArmOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentArmOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExperimentArm.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExperimentArm.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentArmOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ExperimentArmOperation} ExperimentArmOperation
                             */
                            ExperimentArmOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ExperimentArmOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ExperimentArmOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExperimentArmOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExperimentArmOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.ExperimentArm.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExperimentArmOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.ExperimentArm.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExperimentArmOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ExperimentArmOperation} m ExperimentArmOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentArmOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.ExperimentArm.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.ExperimentArm.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExperimentArmOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ExperimentArmOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentArmOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExperimentArmOperation;
                        })();
    
                        services.MutateExperimentArmsResponse = (function() {
    
                            /**
                             * Properties of a MutateExperimentArmsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExperimentArmsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateExperimentArmsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateExperimentArmResult>|null} [results] MutateExperimentArmsResponse results
                             */
    
                            /**
                             * Constructs a new MutateExperimentArmsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExperimentArmsResponse.
                             * @implements IMutateExperimentArmsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsResponse=} [p] Properties to set
                             */
                            function MutateExperimentArmsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExperimentArmsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @instance
                             */
                            MutateExperimentArmsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateExperimentArmsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateExperimentArmResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @instance
                             */
                            MutateExperimentArmsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateExperimentArmsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsResponse} MutateExperimentArmsResponse instance
                             */
                            MutateExperimentArmsResponse.create = function create(properties) {
                                return new MutateExperimentArmsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExperimentArmsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentArmsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsResponse} m MutateExperimentArmsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentArmsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateExperimentArmResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExperimentArmsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentArmsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmsResponse} message MutateExperimentArmsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentArmsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExperimentArmsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsResponse} MutateExperimentArmsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentArmsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExperimentArmsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateExperimentArmResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExperimentArmsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsResponse} MutateExperimentArmsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentArmsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExperimentArmsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExperimentArmsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateExperimentArmResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExperimentArmsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmsResponse} MutateExperimentArmsResponse
                             */
                            MutateExperimentArmsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExperimentArmsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExperimentArmsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExperimentArmsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExperimentArmsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateExperimentArmsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateExperimentArmResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExperimentArmsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExperimentArmsResponse} m MutateExperimentArmsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExperimentArmsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateExperimentArmResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExperimentArmsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExperimentArmsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExperimentArmsResponse;
                        })();
    
                        services.MutateExperimentArmResult = (function() {
    
                            /**
                             * Properties of a MutateExperimentArmResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExperimentArmResult
                             * @property {string|null} [resource_name] MutateExperimentArmResult resource_name
                             * @property {google.ads.googleads.v11.resources.IExperimentArm|null} [experiment_arm] MutateExperimentArmResult experiment_arm
                             */
    
                            /**
                             * Constructs a new MutateExperimentArmResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExperimentArmResult.
                             * @implements IMutateExperimentArmResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmResult=} [p] Properties to set
                             */
                            function MutateExperimentArmResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExperimentArmResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @instance
                             */
                            MutateExperimentArmResult.prototype.resource_name = "";
    
                            /**
                             * MutateExperimentArmResult experiment_arm.
                             * @member {google.ads.googleads.v11.resources.IExperimentArm|null|undefined} experiment_arm
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @instance
                             */
                            MutateExperimentArmResult.prototype.experiment_arm = null;
    
                            /**
                             * Creates a new MutateExperimentArmResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmResult} MutateExperimentArmResult instance
                             */
                            MutateExperimentArmResult.create = function create(properties) {
                                return new MutateExperimentArmResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExperimentArmResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentArmResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmResult} m MutateExperimentArmResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentArmResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.experiment_arm != null && Object.hasOwnProperty.call(m, "experiment_arm"))
                                    $root.google.ads.googleads.v11.resources.ExperimentArm.encode(m.experiment_arm, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExperimentArmResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentArmResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentArmResult} message MutateExperimentArmResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentArmResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExperimentArmResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmResult} MutateExperimentArmResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentArmResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExperimentArmResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.experiment_arm = $root.google.ads.googleads.v11.resources.ExperimentArm.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExperimentArmResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmResult} MutateExperimentArmResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentArmResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExperimentArmResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExperimentArmResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.experiment_arm != null && m.hasOwnProperty("experiment_arm")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExperimentArm.verify(m.experiment_arm);
                                        if (e)
                                            return "experiment_arm." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExperimentArmResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExperimentArmResult} MutateExperimentArmResult
                             */
                            MutateExperimentArmResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExperimentArmResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExperimentArmResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.experiment_arm != null) {
                                    if (typeof d.experiment_arm !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExperimentArmResult.experiment_arm: object expected");
                                    m.experiment_arm = $root.google.ads.googleads.v11.resources.ExperimentArm.fromObject(d.experiment_arm);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExperimentArmResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExperimentArmResult} m MutateExperimentArmResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExperimentArmResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.experiment_arm = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.experiment_arm != null && m.hasOwnProperty("experiment_arm")) {
                                    d.experiment_arm = $root.google.ads.googleads.v11.resources.ExperimentArm.toObject(m.experiment_arm, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExperimentArmResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExperimentArmResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExperimentArmResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExperimentArmResult;
                        })();
    
                        services.ExperimentService = (function() {
    
                            /**
                             * Constructs a new ExperimentService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ExperimentService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ExperimentService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ExperimentService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ExperimentService;
    
                            /**
                             * Creates new ExperimentService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ExperimentService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ExperimentService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExperimentService|mutateExperiments}.
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @typedef MutateExperimentsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateExperimentsResponse} [response] MutateExperimentsResponse
                             */
    
                            /**
                             * Calls MutateExperiments.
                             * @function mutateExperiments
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsRequest} request MutateExperimentsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExperimentService.MutateExperimentsCallback} callback Node-style callback called with the error, if any, and MutateExperimentsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExperimentService.prototype.mutateExperiments = function mutateExperiments(request, callback) {
                                return this.rpcCall(mutateExperiments, $root.google.ads.googleads.v11.services.MutateExperimentsRequest, $root.google.ads.googleads.v11.services.MutateExperimentsResponse, request, callback);
                            }, "name", { value: "MutateExperiments" });
    
                            /**
                             * Calls MutateExperiments.
                             * @function mutateExperiments
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsRequest} request MutateExperimentsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateExperimentsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExperimentService|endExperiment}.
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @typedef EndExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.protobuf.Empty} [response] Empty
                             */
    
                            /**
                             * Calls EndExperiment.
                             * @function endExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IEndExperimentRequest} request EndExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExperimentService.EndExperimentCallback} callback Node-style callback called with the error, if any, and Empty
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExperimentService.prototype.endExperiment = function endExperiment(request, callback) {
                                return this.rpcCall(endExperiment, $root.google.ads.googleads.v11.services.EndExperimentRequest, $root.google.protobuf.Empty, request, callback);
                            }, "name", { value: "EndExperiment" });
    
                            /**
                             * Calls EndExperiment.
                             * @function endExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IEndExperimentRequest} request EndExperimentRequest message or plain object
                             * @returns {Promise<google.protobuf.Empty>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExperimentService|listExperimentAsyncErrors}.
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @typedef ListExperimentAsyncErrorsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse} [response] ListExperimentAsyncErrorsResponse
                             */
    
                            /**
                             * Calls ListExperimentAsyncErrors.
                             * @function listExperimentAsyncErrors
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsRequest} request ListExperimentAsyncErrorsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExperimentService.ListExperimentAsyncErrorsCallback} callback Node-style callback called with the error, if any, and ListExperimentAsyncErrorsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExperimentService.prototype.listExperimentAsyncErrors = function listExperimentAsyncErrors(request, callback) {
                                return this.rpcCall(listExperimentAsyncErrors, $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest, $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse, request, callback);
                            }, "name", { value: "ListExperimentAsyncErrors" });
    
                            /**
                             * Calls ListExperimentAsyncErrors.
                             * @function listExperimentAsyncErrors
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsRequest} request ListExperimentAsyncErrorsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExperimentService|graduateExperiment}.
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @typedef GraduateExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.protobuf.Empty} [response] Empty
                             */
    
                            /**
                             * Calls GraduateExperiment.
                             * @function graduateExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGraduateExperimentRequest} request GraduateExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExperimentService.GraduateExperimentCallback} callback Node-style callback called with the error, if any, and Empty
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExperimentService.prototype.graduateExperiment = function graduateExperiment(request, callback) {
                                return this.rpcCall(graduateExperiment, $root.google.ads.googleads.v11.services.GraduateExperimentRequest, $root.google.protobuf.Empty, request, callback);
                            }, "name", { value: "GraduateExperiment" });
    
                            /**
                             * Calls GraduateExperiment.
                             * @function graduateExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGraduateExperimentRequest} request GraduateExperimentRequest message or plain object
                             * @returns {Promise<google.protobuf.Empty>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExperimentService|scheduleExperiment}.
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @typedef ScheduleExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.longrunning.Operation} [response] Operation
                             */
    
                            /**
                             * Calls ScheduleExperiment.
                             * @function scheduleExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentRequest} request ScheduleExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExperimentService.ScheduleExperimentCallback} callback Node-style callback called with the error, if any, and Operation
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExperimentService.prototype.scheduleExperiment = function scheduleExperiment(request, callback) {
                                return this.rpcCall(scheduleExperiment, $root.google.ads.googleads.v11.services.ScheduleExperimentRequest, $root.google.longrunning.Operation, request, callback);
                            }, "name", { value: "ScheduleExperiment" });
    
                            /**
                             * Calls ScheduleExperiment.
                             * @function scheduleExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentRequest} request ScheduleExperimentRequest message or plain object
                             * @returns {Promise<google.longrunning.Operation>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExperimentService|promoteExperiment}.
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @typedef PromoteExperimentCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.longrunning.Operation} [response] Operation
                             */
    
                            /**
                             * Calls PromoteExperiment.
                             * @function promoteExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentRequest} request PromoteExperimentRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExperimentService.PromoteExperimentCallback} callback Node-style callback called with the error, if any, and Operation
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExperimentService.prototype.promoteExperiment = function promoteExperiment(request, callback) {
                                return this.rpcCall(promoteExperiment, $root.google.ads.googleads.v11.services.PromoteExperimentRequest, $root.google.longrunning.Operation, request, callback);
                            }, "name", { value: "PromoteExperiment" });
    
                            /**
                             * Calls PromoteExperiment.
                             * @function promoteExperiment
                             * @memberof google.ads.googleads.v11.services.ExperimentService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentRequest} request PromoteExperimentRequest message or plain object
                             * @returns {Promise<google.longrunning.Operation>} Promise
                             * @variation 2
                             */
    
                            return ExperimentService;
                        })();
    
                        services.MutateExperimentsRequest = (function() {
    
                            /**
                             * Properties of a MutateExperimentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExperimentsRequest
                             * @property {string|null} [customer_id] MutateExperimentsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IExperimentOperation>|null} [operations] MutateExperimentsRequest operations
                             * @property {boolean|null} [partial_failure] MutateExperimentsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateExperimentsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateExperimentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExperimentsRequest.
                             * @implements IMutateExperimentsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsRequest=} [p] Properties to set
                             */
                            function MutateExperimentsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExperimentsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @instance
                             */
                            MutateExperimentsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateExperimentsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IExperimentOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @instance
                             */
                            MutateExperimentsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateExperimentsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @instance
                             */
                            MutateExperimentsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateExperimentsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @instance
                             */
                            MutateExperimentsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateExperimentsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsRequest} MutateExperimentsRequest instance
                             */
                            MutateExperimentsRequest.create = function create(properties) {
                                return new MutateExperimentsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExperimentsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsRequest} m MutateExperimentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ExperimentOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExperimentsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsRequest} message MutateExperimentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExperimentsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsRequest} MutateExperimentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExperimentsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ExperimentOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExperimentsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsRequest} MutateExperimentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExperimentsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExperimentsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ExperimentOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExperimentsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsRequest} MutateExperimentsRequest
                             */
                            MutateExperimentsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExperimentsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExperimentsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExperimentsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateExperimentsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ExperimentOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExperimentsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExperimentsRequest} m MutateExperimentsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExperimentsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ExperimentOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExperimentsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExperimentsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExperimentsRequest;
                        })();
    
                        services.ExperimentOperation = (function() {
    
                            /**
                             * Properties of an ExperimentOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IExperimentOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ExperimentOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IExperiment|null} [create] ExperimentOperation create
                             * @property {google.ads.googleads.v11.resources.IExperiment|null} [update] ExperimentOperation update
                             * @property {string|null} [remove] ExperimentOperation remove
                             */
    
                            /**
                             * Constructs a new ExperimentOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ExperimentOperation.
                             * @implements IExperimentOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IExperimentOperation=} [p] Properties to set
                             */
                            function ExperimentOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExperimentOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @instance
                             */
                            ExperimentOperation.prototype.update_mask = null;
    
                            /**
                             * ExperimentOperation create.
                             * @member {google.ads.googleads.v11.resources.IExperiment|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @instance
                             */
                            ExperimentOperation.prototype.create = null;
    
                            /**
                             * ExperimentOperation update.
                             * @member {google.ads.googleads.v11.resources.IExperiment|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @instance
                             */
                            ExperimentOperation.prototype.update = null;
    
                            /**
                             * ExperimentOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @instance
                             */
                            ExperimentOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExperimentOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @instance
                             */
                            Object.defineProperty(ExperimentOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExperimentOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExperimentOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ExperimentOperation} ExperimentOperation instance
                             */
                            ExperimentOperation.create = function create(properties) {
                                return new ExperimentOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ExperimentOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ExperimentOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExperimentOperation} m ExperimentOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.Experiment.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Experiment.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExperimentOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ExperimentOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExperimentOperation} message ExperimentOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExperimentOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExperimentOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ExperimentOperation} ExperimentOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ExperimentOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.Experiment.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.Experiment.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExperimentOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ExperimentOperation} ExperimentOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExperimentOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExperimentOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExperimentOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Experiment.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Experiment.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExperimentOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ExperimentOperation} ExperimentOperation
                             */
                            ExperimentOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ExperimentOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ExperimentOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExperimentOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExperimentOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.Experiment.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExperimentOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Experiment.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExperimentOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ExperimentOperation} m ExperimentOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExperimentOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.Experiment.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Experiment.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExperimentOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ExperimentOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExperimentOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExperimentOperation;
                        })();
    
                        services.MutateExperimentsResponse = (function() {
    
                            /**
                             * Properties of a MutateExperimentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExperimentsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateExperimentsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateExperimentResult>|null} [results] MutateExperimentsResponse results
                             */
    
                            /**
                             * Constructs a new MutateExperimentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExperimentsResponse.
                             * @implements IMutateExperimentsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsResponse=} [p] Properties to set
                             */
                            function MutateExperimentsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExperimentsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @instance
                             */
                            MutateExperimentsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateExperimentsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateExperimentResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @instance
                             */
                            MutateExperimentsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateExperimentsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsResponse} MutateExperimentsResponse instance
                             */
                            MutateExperimentsResponse.create = function create(properties) {
                                return new MutateExperimentsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExperimentsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsResponse} m MutateExperimentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateExperimentResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExperimentsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentsResponse} message MutateExperimentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExperimentsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsResponse} MutateExperimentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExperimentsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateExperimentResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExperimentsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsResponse} MutateExperimentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExperimentsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExperimentsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateExperimentResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExperimentsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExperimentsResponse} MutateExperimentsResponse
                             */
                            MutateExperimentsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExperimentsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExperimentsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExperimentsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExperimentsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateExperimentsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateExperimentResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExperimentsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExperimentsResponse} m MutateExperimentsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExperimentsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateExperimentResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExperimentsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExperimentsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExperimentsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExperimentsResponse;
                        })();
    
                        services.MutateExperimentResult = (function() {
    
                            /**
                             * Properties of a MutateExperimentResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExperimentResult
                             * @property {string|null} [resource_name] MutateExperimentResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateExperimentResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExperimentResult.
                             * @implements IMutateExperimentResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExperimentResult=} [p] Properties to set
                             */
                            function MutateExperimentResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExperimentResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @instance
                             */
                            MutateExperimentResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateExperimentResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExperimentResult} MutateExperimentResult instance
                             */
                            MutateExperimentResult.create = function create(properties) {
                                return new MutateExperimentResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExperimentResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentResult} m MutateExperimentResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExperimentResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExperimentResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExperimentResult} message MutateExperimentResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExperimentResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExperimentResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExperimentResult} MutateExperimentResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExperimentResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExperimentResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExperimentResult} MutateExperimentResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExperimentResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExperimentResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExperimentResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExperimentResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExperimentResult} MutateExperimentResult
                             */
                            MutateExperimentResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExperimentResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExperimentResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExperimentResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExperimentResult} m MutateExperimentResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExperimentResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExperimentResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExperimentResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExperimentResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExperimentResult;
                        })();
    
                        services.EndExperimentRequest = (function() {
    
                            /**
                             * Properties of an EndExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IEndExperimentRequest
                             * @property {string|null} [experiment] EndExperimentRequest experiment
                             * @property {boolean|null} [validate_only] EndExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new EndExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an EndExperimentRequest.
                             * @implements IEndExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IEndExperimentRequest=} [p] Properties to set
                             */
                            function EndExperimentRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * EndExperimentRequest experiment.
                             * @member {string} experiment
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @instance
                             */
                            EndExperimentRequest.prototype.experiment = "";
    
                            /**
                             * EndExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @instance
                             */
                            EndExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new EndExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IEndExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.EndExperimentRequest} EndExperimentRequest instance
                             */
                            EndExperimentRequest.create = function create(properties) {
                                return new EndExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified EndExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.EndExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IEndExperimentRequest} m EndExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EndExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.experiment != null && Object.hasOwnProperty.call(m, "experiment"))
                                    w.uint32(10).string(m.experiment);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(16).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified EndExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.EndExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IEndExperimentRequest} message EndExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EndExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EndExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.EndExperimentRequest} EndExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EndExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.EndExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.experiment = r.string();
                                        break;
                                    case 2:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an EndExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.EndExperimentRequest} EndExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EndExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EndExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EndExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    if (!$util.isString(m.experiment))
                                        return "experiment: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an EndExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.EndExperimentRequest} EndExperimentRequest
                             */
                            EndExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.EndExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.EndExperimentRequest();
                                if (d.experiment != null) {
                                    m.experiment = String(d.experiment);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an EndExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.EndExperimentRequest} m EndExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EndExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.experiment = "";
                                    d.validate_only = false;
                                }
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    d.experiment = m.experiment;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this EndExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.EndExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EndExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return EndExperimentRequest;
                        })();
    
                        services.ListExperimentAsyncErrorsRequest = (function() {
    
                            /**
                             * Properties of a ListExperimentAsyncErrorsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListExperimentAsyncErrorsRequest
                             * @property {string|null} [resource_name] ListExperimentAsyncErrorsRequest resource_name
                             * @property {string|null} [page_token] ListExperimentAsyncErrorsRequest page_token
                             * @property {number|null} [page_size] ListExperimentAsyncErrorsRequest page_size
                             */
    
                            /**
                             * Constructs a new ListExperimentAsyncErrorsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListExperimentAsyncErrorsRequest.
                             * @implements IListExperimentAsyncErrorsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsRequest=} [p] Properties to set
                             */
                            function ListExperimentAsyncErrorsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListExperimentAsyncErrorsRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @instance
                             */
                            ListExperimentAsyncErrorsRequest.prototype.resource_name = "";
    
                            /**
                             * ListExperimentAsyncErrorsRequest page_token.
                             * @member {string} page_token
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @instance
                             */
                            ListExperimentAsyncErrorsRequest.prototype.page_token = "";
    
                            /**
                             * ListExperimentAsyncErrorsRequest page_size.
                             * @member {number} page_size
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @instance
                             */
                            ListExperimentAsyncErrorsRequest.prototype.page_size = 0;
    
                            /**
                             * Creates a new ListExperimentAsyncErrorsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest} ListExperimentAsyncErrorsRequest instance
                             */
                            ListExperimentAsyncErrorsRequest.create = function create(properties) {
                                return new ListExperimentAsyncErrorsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListExperimentAsyncErrorsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsRequest} m ListExperimentAsyncErrorsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListExperimentAsyncErrorsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                                    w.uint32(18).string(m.page_token);
                                if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                                    w.uint32(24).int32(m.page_size);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListExperimentAsyncErrorsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsRequest} message ListExperimentAsyncErrorsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListExperimentAsyncErrorsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListExperimentAsyncErrorsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest} ListExperimentAsyncErrorsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListExperimentAsyncErrorsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.page_token = r.string();
                                        break;
                                    case 3:
                                        m.page_size = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListExperimentAsyncErrorsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest} ListExperimentAsyncErrorsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListExperimentAsyncErrorsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListExperimentAsyncErrorsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListExperimentAsyncErrorsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    if (!$util.isString(m.page_token))
                                        return "page_token: string expected";
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    if (!$util.isInteger(m.page_size))
                                        return "page_size: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListExperimentAsyncErrorsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest} ListExperimentAsyncErrorsRequest
                             */
                            ListExperimentAsyncErrorsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.page_token != null) {
                                    m.page_token = String(d.page_token);
                                }
                                if (d.page_size != null) {
                                    m.page_size = d.page_size | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListExperimentAsyncErrorsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest} m ListExperimentAsyncErrorsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListExperimentAsyncErrorsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.page_token = "";
                                    d.page_size = 0;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    d.page_token = m.page_token;
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    d.page_size = m.page_size;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListExperimentAsyncErrorsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListExperimentAsyncErrorsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListExperimentAsyncErrorsRequest;
                        })();
    
                        services.ListExperimentAsyncErrorsResponse = (function() {
    
                            /**
                             * Properties of a ListExperimentAsyncErrorsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListExperimentAsyncErrorsResponse
                             * @property {Array.<google.rpc.IStatus>|null} [errors] ListExperimentAsyncErrorsResponse errors
                             * @property {string|null} [next_page_token] ListExperimentAsyncErrorsResponse next_page_token
                             */
    
                            /**
                             * Constructs a new ListExperimentAsyncErrorsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListExperimentAsyncErrorsResponse.
                             * @implements IListExperimentAsyncErrorsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsResponse=} [p] Properties to set
                             */
                            function ListExperimentAsyncErrorsResponse(p) {
                                this.errors = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListExperimentAsyncErrorsResponse errors.
                             * @member {Array.<google.rpc.IStatus>} errors
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @instance
                             */
                            ListExperimentAsyncErrorsResponse.prototype.errors = $util.emptyArray;
    
                            /**
                             * ListExperimentAsyncErrorsResponse next_page_token.
                             * @member {string} next_page_token
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @instance
                             */
                            ListExperimentAsyncErrorsResponse.prototype.next_page_token = "";
    
                            /**
                             * Creates a new ListExperimentAsyncErrorsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse} ListExperimentAsyncErrorsResponse instance
                             */
                            ListExperimentAsyncErrorsResponse.create = function create(properties) {
                                return new ListExperimentAsyncErrorsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListExperimentAsyncErrorsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsResponse} m ListExperimentAsyncErrorsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListExperimentAsyncErrorsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.errors != null && m.errors.length) {
                                    for (var i = 0; i < m.errors.length; ++i)
                                        $root.google.rpc.Status.encode(m.errors[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                                    w.uint32(18).string(m.next_page_token);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListExperimentAsyncErrorsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListExperimentAsyncErrorsResponse} message ListExperimentAsyncErrorsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListExperimentAsyncErrorsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListExperimentAsyncErrorsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse} ListExperimentAsyncErrorsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListExperimentAsyncErrorsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.errors && m.errors.length))
                                            m.errors = [];
                                        m.errors.push($root.google.rpc.Status.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.next_page_token = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListExperimentAsyncErrorsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse} ListExperimentAsyncErrorsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListExperimentAsyncErrorsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListExperimentAsyncErrorsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListExperimentAsyncErrorsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.errors != null && m.hasOwnProperty("errors")) {
                                    if (!Array.isArray(m.errors))
                                        return "errors: array expected";
                                    for (var i = 0; i < m.errors.length; ++i) {
                                        {
                                            var e = $root.google.rpc.Status.verify(m.errors[i]);
                                            if (e)
                                                return "errors." + e;
                                        }
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    if (!$util.isString(m.next_page_token))
                                        return "next_page_token: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListExperimentAsyncErrorsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse} ListExperimentAsyncErrorsResponse
                             */
                            ListExperimentAsyncErrorsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse();
                                if (d.errors) {
                                    if (!Array.isArray(d.errors))
                                        throw TypeError(".google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse.errors: array expected");
                                    m.errors = [];
                                    for (var i = 0; i < d.errors.length; ++i) {
                                        if (typeof d.errors[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse.errors: object expected");
                                        m.errors[i] = $root.google.rpc.Status.fromObject(d.errors[i]);
                                    }
                                }
                                if (d.next_page_token != null) {
                                    m.next_page_token = String(d.next_page_token);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListExperimentAsyncErrorsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse} m ListExperimentAsyncErrorsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListExperimentAsyncErrorsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.errors = [];
                                }
                                if (o.defaults) {
                                    d.next_page_token = "";
                                }
                                if (m.errors && m.errors.length) {
                                    d.errors = [];
                                    for (var j = 0; j < m.errors.length; ++j) {
                                        d.errors[j] = $root.google.rpc.Status.toObject(m.errors[j], o);
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    d.next_page_token = m.next_page_token;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListExperimentAsyncErrorsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListExperimentAsyncErrorsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListExperimentAsyncErrorsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListExperimentAsyncErrorsResponse;
                        })();
    
                        services.GraduateExperimentRequest = (function() {
    
                            /**
                             * Properties of a GraduateExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGraduateExperimentRequest
                             * @property {string|null} [experiment] GraduateExperimentRequest experiment
                             * @property {Array.<google.ads.googleads.v11.services.ICampaignBudgetMapping>|null} [campaign_budget_mappings] GraduateExperimentRequest campaign_budget_mappings
                             * @property {boolean|null} [validate_only] GraduateExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new GraduateExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GraduateExperimentRequest.
                             * @implements IGraduateExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGraduateExperimentRequest=} [p] Properties to set
                             */
                            function GraduateExperimentRequest(p) {
                                this.campaign_budget_mappings = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GraduateExperimentRequest experiment.
                             * @member {string} experiment
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @instance
                             */
                            GraduateExperimentRequest.prototype.experiment = "";
    
                            /**
                             * GraduateExperimentRequest campaign_budget_mappings.
                             * @member {Array.<google.ads.googleads.v11.services.ICampaignBudgetMapping>} campaign_budget_mappings
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @instance
                             */
                            GraduateExperimentRequest.prototype.campaign_budget_mappings = $util.emptyArray;
    
                            /**
                             * GraduateExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @instance
                             */
                            GraduateExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new GraduateExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GraduateExperimentRequest} GraduateExperimentRequest instance
                             */
                            GraduateExperimentRequest.create = function create(properties) {
                                return new GraduateExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GraduateExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GraduateExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateExperimentRequest} m GraduateExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GraduateExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.experiment != null && Object.hasOwnProperty.call(m, "experiment"))
                                    w.uint32(10).string(m.experiment);
                                if (m.campaign_budget_mappings != null && m.campaign_budget_mappings.length) {
                                    for (var i = 0; i < m.campaign_budget_mappings.length; ++i)
                                        $root.google.ads.googleads.v11.services.CampaignBudgetMapping.encode(m.campaign_budget_mappings[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GraduateExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GraduateExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGraduateExperimentRequest} message GraduateExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GraduateExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GraduateExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GraduateExperimentRequest} GraduateExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GraduateExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GraduateExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.experiment = r.string();
                                        break;
                                    case 2:
                                        if (!(m.campaign_budget_mappings && m.campaign_budget_mappings.length))
                                            m.campaign_budget_mappings = [];
                                        m.campaign_budget_mappings.push($root.google.ads.googleads.v11.services.CampaignBudgetMapping.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GraduateExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GraduateExperimentRequest} GraduateExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GraduateExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GraduateExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GraduateExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    if (!$util.isString(m.experiment))
                                        return "experiment: string expected";
                                }
                                if (m.campaign_budget_mappings != null && m.hasOwnProperty("campaign_budget_mappings")) {
                                    if (!Array.isArray(m.campaign_budget_mappings))
                                        return "campaign_budget_mappings: array expected";
                                    for (var i = 0; i < m.campaign_budget_mappings.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CampaignBudgetMapping.verify(m.campaign_budget_mappings[i]);
                                            if (e)
                                                return "campaign_budget_mappings." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GraduateExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GraduateExperimentRequest} GraduateExperimentRequest
                             */
                            GraduateExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GraduateExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GraduateExperimentRequest();
                                if (d.experiment != null) {
                                    m.experiment = String(d.experiment);
                                }
                                if (d.campaign_budget_mappings) {
                                    if (!Array.isArray(d.campaign_budget_mappings))
                                        throw TypeError(".google.ads.googleads.v11.services.GraduateExperimentRequest.campaign_budget_mappings: array expected");
                                    m.campaign_budget_mappings = [];
                                    for (var i = 0; i < d.campaign_budget_mappings.length; ++i) {
                                        if (typeof d.campaign_budget_mappings[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GraduateExperimentRequest.campaign_budget_mappings: object expected");
                                        m.campaign_budget_mappings[i] = $root.google.ads.googleads.v11.services.CampaignBudgetMapping.fromObject(d.campaign_budget_mappings[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GraduateExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GraduateExperimentRequest} m GraduateExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GraduateExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.campaign_budget_mappings = [];
                                }
                                if (o.defaults) {
                                    d.experiment = "";
                                    d.validate_only = false;
                                }
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    d.experiment = m.experiment;
                                }
                                if (m.campaign_budget_mappings && m.campaign_budget_mappings.length) {
                                    d.campaign_budget_mappings = [];
                                    for (var j = 0; j < m.campaign_budget_mappings.length; ++j) {
                                        d.campaign_budget_mappings[j] = $root.google.ads.googleads.v11.services.CampaignBudgetMapping.toObject(m.campaign_budget_mappings[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GraduateExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GraduateExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GraduateExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GraduateExperimentRequest;
                        })();
    
                        services.CampaignBudgetMapping = (function() {
    
                            /**
                             * Properties of a CampaignBudgetMapping.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignBudgetMapping
                             * @property {string|null} [experiment_campaign] CampaignBudgetMapping experiment_campaign
                             * @property {string|null} [campaign_budget] CampaignBudgetMapping campaign_budget
                             */
    
                            /**
                             * Constructs a new CampaignBudgetMapping.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignBudgetMapping.
                             * @implements ICampaignBudgetMapping
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetMapping=} [p] Properties to set
                             */
                            function CampaignBudgetMapping(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignBudgetMapping experiment_campaign.
                             * @member {string} experiment_campaign
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @instance
                             */
                            CampaignBudgetMapping.prototype.experiment_campaign = "";
    
                            /**
                             * CampaignBudgetMapping campaign_budget.
                             * @member {string} campaign_budget
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @instance
                             */
                            CampaignBudgetMapping.prototype.campaign_budget = "";
    
                            /**
                             * Creates a new CampaignBudgetMapping instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetMapping=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetMapping} CampaignBudgetMapping instance
                             */
                            CampaignBudgetMapping.create = function create(properties) {
                                return new CampaignBudgetMapping(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignBudgetMapping message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignBudgetMapping.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetMapping} m CampaignBudgetMapping message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudgetMapping.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.experiment_campaign != null && Object.hasOwnProperty.call(m, "experiment_campaign"))
                                    w.uint32(10).string(m.experiment_campaign);
                                if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                    w.uint32(18).string(m.campaign_budget);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignBudgetMapping message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignBudgetMapping.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignBudgetMapping} message CampaignBudgetMapping message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignBudgetMapping.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignBudgetMapping message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetMapping} CampaignBudgetMapping
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudgetMapping.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignBudgetMapping();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.experiment_campaign = r.string();
                                        break;
                                    case 2:
                                        m.campaign_budget = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignBudgetMapping message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetMapping} CampaignBudgetMapping
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignBudgetMapping.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignBudgetMapping message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignBudgetMapping.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.experiment_campaign != null && m.hasOwnProperty("experiment_campaign")) {
                                    if (!$util.isString(m.experiment_campaign))
                                        return "experiment_campaign: string expected";
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    if (!$util.isString(m.campaign_budget))
                                        return "campaign_budget: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignBudgetMapping message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignBudgetMapping} CampaignBudgetMapping
                             */
                            CampaignBudgetMapping.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignBudgetMapping)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignBudgetMapping();
                                if (d.experiment_campaign != null) {
                                    m.experiment_campaign = String(d.experiment_campaign);
                                }
                                if (d.campaign_budget != null) {
                                    m.campaign_budget = String(d.campaign_budget);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignBudgetMapping message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignBudgetMapping} m CampaignBudgetMapping
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignBudgetMapping.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.experiment_campaign = "";
                                    d.campaign_budget = "";
                                }
                                if (m.experiment_campaign != null && m.hasOwnProperty("experiment_campaign")) {
                                    d.experiment_campaign = m.experiment_campaign;
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    d.campaign_budget = m.campaign_budget;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignBudgetMapping to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignBudgetMapping
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignBudgetMapping.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignBudgetMapping;
                        })();
    
                        services.ScheduleExperimentRequest = (function() {
    
                            /**
                             * Properties of a ScheduleExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IScheduleExperimentRequest
                             * @property {string|null} [resource_name] ScheduleExperimentRequest resource_name
                             * @property {boolean|null} [validate_only] ScheduleExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new ScheduleExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ScheduleExperimentRequest.
                             * @implements IScheduleExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentRequest=} [p] Properties to set
                             */
                            function ScheduleExperimentRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ScheduleExperimentRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @instance
                             */
                            ScheduleExperimentRequest.prototype.resource_name = "";
    
                            /**
                             * ScheduleExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @instance
                             */
                            ScheduleExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new ScheduleExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentRequest} ScheduleExperimentRequest instance
                             */
                            ScheduleExperimentRequest.create = function create(properties) {
                                return new ScheduleExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ScheduleExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ScheduleExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentRequest} m ScheduleExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ScheduleExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(16).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ScheduleExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ScheduleExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentRequest} message ScheduleExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ScheduleExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ScheduleExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentRequest} ScheduleExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ScheduleExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ScheduleExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ScheduleExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentRequest} ScheduleExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ScheduleExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ScheduleExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ScheduleExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ScheduleExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentRequest} ScheduleExperimentRequest
                             */
                            ScheduleExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ScheduleExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ScheduleExperimentRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ScheduleExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ScheduleExperimentRequest} m ScheduleExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ScheduleExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.validate_only = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ScheduleExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ScheduleExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ScheduleExperimentRequest;
                        })();
    
                        services.ScheduleExperimentMetadata = (function() {
    
                            /**
                             * Properties of a ScheduleExperimentMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IScheduleExperimentMetadata
                             * @property {string|null} [experiment] ScheduleExperimentMetadata experiment
                             */
    
                            /**
                             * Constructs a new ScheduleExperimentMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ScheduleExperimentMetadata.
                             * @implements IScheduleExperimentMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentMetadata=} [p] Properties to set
                             */
                            function ScheduleExperimentMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ScheduleExperimentMetadata experiment.
                             * @member {string} experiment
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @instance
                             */
                            ScheduleExperimentMetadata.prototype.experiment = "";
    
                            /**
                             * Creates a new ScheduleExperimentMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentMetadata} ScheduleExperimentMetadata instance
                             */
                            ScheduleExperimentMetadata.create = function create(properties) {
                                return new ScheduleExperimentMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified ScheduleExperimentMetadata message. Does not implicitly {@link google.ads.googleads.v11.services.ScheduleExperimentMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentMetadata} m ScheduleExperimentMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ScheduleExperimentMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.experiment != null && Object.hasOwnProperty.call(m, "experiment"))
                                    w.uint32(10).string(m.experiment);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ScheduleExperimentMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ScheduleExperimentMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IScheduleExperimentMetadata} message ScheduleExperimentMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ScheduleExperimentMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ScheduleExperimentMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentMetadata} ScheduleExperimentMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ScheduleExperimentMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ScheduleExperimentMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.experiment = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ScheduleExperimentMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentMetadata} ScheduleExperimentMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ScheduleExperimentMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ScheduleExperimentMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ScheduleExperimentMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    if (!$util.isString(m.experiment))
                                        return "experiment: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ScheduleExperimentMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ScheduleExperimentMetadata} ScheduleExperimentMetadata
                             */
                            ScheduleExperimentMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ScheduleExperimentMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ScheduleExperimentMetadata();
                                if (d.experiment != null) {
                                    m.experiment = String(d.experiment);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ScheduleExperimentMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.ScheduleExperimentMetadata} m ScheduleExperimentMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ScheduleExperimentMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.experiment = "";
                                }
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    d.experiment = m.experiment;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ScheduleExperimentMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ScheduleExperimentMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ScheduleExperimentMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ScheduleExperimentMetadata;
                        })();
    
                        services.PromoteExperimentRequest = (function() {
    
                            /**
                             * Properties of a PromoteExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPromoteExperimentRequest
                             * @property {string|null} [resource_name] PromoteExperimentRequest resource_name
                             * @property {boolean|null} [validate_only] PromoteExperimentRequest validate_only
                             */
    
                            /**
                             * Constructs a new PromoteExperimentRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PromoteExperimentRequest.
                             * @implements IPromoteExperimentRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentRequest=} [p] Properties to set
                             */
                            function PromoteExperimentRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PromoteExperimentRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @instance
                             */
                            PromoteExperimentRequest.prototype.resource_name = "";
    
                            /**
                             * PromoteExperimentRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @instance
                             */
                            PromoteExperimentRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new PromoteExperimentRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentRequest} PromoteExperimentRequest instance
                             */
                            PromoteExperimentRequest.create = function create(properties) {
                                return new PromoteExperimentRequest(properties);
                            };
    
                            /**
                             * Encodes the specified PromoteExperimentRequest message. Does not implicitly {@link google.ads.googleads.v11.services.PromoteExperimentRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentRequest} m PromoteExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteExperimentRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(16).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromoteExperimentRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PromoteExperimentRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentRequest} message PromoteExperimentRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteExperimentRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromoteExperimentRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentRequest} PromoteExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteExperimentRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PromoteExperimentRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromoteExperimentRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentRequest} PromoteExperimentRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteExperimentRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromoteExperimentRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromoteExperimentRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PromoteExperimentRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentRequest} PromoteExperimentRequest
                             */
                            PromoteExperimentRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PromoteExperimentRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PromoteExperimentRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PromoteExperimentRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.PromoteExperimentRequest} m PromoteExperimentRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromoteExperimentRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.validate_only = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PromoteExperimentRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromoteExperimentRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PromoteExperimentRequest;
                        })();
    
                        services.PromoteExperimentMetadata = (function() {
    
                            /**
                             * Properties of a PromoteExperimentMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPromoteExperimentMetadata
                             * @property {string|null} [experiment] PromoteExperimentMetadata experiment
                             */
    
                            /**
                             * Constructs a new PromoteExperimentMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PromoteExperimentMetadata.
                             * @implements IPromoteExperimentMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentMetadata=} [p] Properties to set
                             */
                            function PromoteExperimentMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PromoteExperimentMetadata experiment.
                             * @member {string} experiment
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @instance
                             */
                            PromoteExperimentMetadata.prototype.experiment = "";
    
                            /**
                             * Creates a new PromoteExperimentMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentMetadata} PromoteExperimentMetadata instance
                             */
                            PromoteExperimentMetadata.create = function create(properties) {
                                return new PromoteExperimentMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified PromoteExperimentMetadata message. Does not implicitly {@link google.ads.googleads.v11.services.PromoteExperimentMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentMetadata} m PromoteExperimentMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteExperimentMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.experiment != null && Object.hasOwnProperty.call(m, "experiment"))
                                    w.uint32(10).string(m.experiment);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PromoteExperimentMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PromoteExperimentMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IPromoteExperimentMetadata} message PromoteExperimentMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PromoteExperimentMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PromoteExperimentMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentMetadata} PromoteExperimentMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteExperimentMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PromoteExperimentMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.experiment = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PromoteExperimentMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentMetadata} PromoteExperimentMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PromoteExperimentMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PromoteExperimentMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PromoteExperimentMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    if (!$util.isString(m.experiment))
                                        return "experiment: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PromoteExperimentMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PromoteExperimentMetadata} PromoteExperimentMetadata
                             */
                            PromoteExperimentMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PromoteExperimentMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PromoteExperimentMetadata();
                                if (d.experiment != null) {
                                    m.experiment = String(d.experiment);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PromoteExperimentMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.PromoteExperimentMetadata} m PromoteExperimentMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PromoteExperimentMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.experiment = "";
                                }
                                if (m.experiment != null && m.hasOwnProperty("experiment")) {
                                    d.experiment = m.experiment;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PromoteExperimentMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PromoteExperimentMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PromoteExperimentMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PromoteExperimentMetadata;
                        })();
    
                        services.ExtensionFeedItemService = (function() {
    
                            /**
                             * Constructs a new ExtensionFeedItemService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ExtensionFeedItemService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ExtensionFeedItemService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ExtensionFeedItemService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ExtensionFeedItemService;
    
                            /**
                             * Creates new ExtensionFeedItemService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ExtensionFeedItemService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ExtensionFeedItemService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ExtensionFeedItemService|mutateExtensionFeedItems}.
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemService
                             * @typedef MutateExtensionFeedItemsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse} [response] MutateExtensionFeedItemsResponse
                             */
    
                            /**
                             * Calls MutateExtensionFeedItems.
                             * @function mutateExtensionFeedItems
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsRequest} request MutateExtensionFeedItemsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ExtensionFeedItemService.MutateExtensionFeedItemsCallback} callback Node-style callback called with the error, if any, and MutateExtensionFeedItemsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ExtensionFeedItemService.prototype.mutateExtensionFeedItems = function mutateExtensionFeedItems(request, callback) {
                                return this.rpcCall(mutateExtensionFeedItems, $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest, $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse, request, callback);
                            }, "name", { value: "MutateExtensionFeedItems" });
    
                            /**
                             * Calls MutateExtensionFeedItems.
                             * @function mutateExtensionFeedItems
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsRequest} request MutateExtensionFeedItemsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse>} Promise
                             * @variation 2
                             */
    
                            return ExtensionFeedItemService;
                        })();
    
                        services.MutateExtensionFeedItemsRequest = (function() {
    
                            /**
                             * Properties of a MutateExtensionFeedItemsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExtensionFeedItemsRequest
                             * @property {string|null} [customer_id] MutateExtensionFeedItemsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IExtensionFeedItemOperation>|null} [operations] MutateExtensionFeedItemsRequest operations
                             * @property {boolean|null} [partial_failure] MutateExtensionFeedItemsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateExtensionFeedItemsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateExtensionFeedItemsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateExtensionFeedItemsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExtensionFeedItemsRequest.
                             * @implements IMutateExtensionFeedItemsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsRequest=} [p] Properties to set
                             */
                            function MutateExtensionFeedItemsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExtensionFeedItemsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @instance
                             */
                            MutateExtensionFeedItemsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateExtensionFeedItemsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IExtensionFeedItemOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @instance
                             */
                            MutateExtensionFeedItemsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateExtensionFeedItemsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @instance
                             */
                            MutateExtensionFeedItemsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateExtensionFeedItemsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @instance
                             */
                            MutateExtensionFeedItemsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateExtensionFeedItemsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @instance
                             */
                            MutateExtensionFeedItemsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateExtensionFeedItemsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest} MutateExtensionFeedItemsRequest instance
                             */
                            MutateExtensionFeedItemsRequest.create = function create(properties) {
                                return new MutateExtensionFeedItemsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExtensionFeedItemsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsRequest} m MutateExtensionFeedItemsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExtensionFeedItemsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExtensionFeedItemsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsRequest} message MutateExtensionFeedItemsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExtensionFeedItemsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExtensionFeedItemsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest} MutateExtensionFeedItemsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExtensionFeedItemsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExtensionFeedItemsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest} MutateExtensionFeedItemsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExtensionFeedItemsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExtensionFeedItemsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExtensionFeedItemsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExtensionFeedItemsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest} MutateExtensionFeedItemsRequest
                             */
                            MutateExtensionFeedItemsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExtensionFeedItemsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest} m MutateExtensionFeedItemsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExtensionFeedItemsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExtensionFeedItemsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExtensionFeedItemsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExtensionFeedItemsRequest;
                        })();
    
                        services.ExtensionFeedItemOperation = (function() {
    
                            /**
                             * Properties of an ExtensionFeedItemOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IExtensionFeedItemOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] ExtensionFeedItemOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IExtensionFeedItem|null} [create] ExtensionFeedItemOperation create
                             * @property {google.ads.googleads.v11.resources.IExtensionFeedItem|null} [update] ExtensionFeedItemOperation update
                             * @property {string|null} [remove] ExtensionFeedItemOperation remove
                             */
    
                            /**
                             * Constructs a new ExtensionFeedItemOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ExtensionFeedItemOperation.
                             * @implements IExtensionFeedItemOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IExtensionFeedItemOperation=} [p] Properties to set
                             */
                            function ExtensionFeedItemOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExtensionFeedItemOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @instance
                             */
                            ExtensionFeedItemOperation.prototype.update_mask = null;
    
                            /**
                             * ExtensionFeedItemOperation create.
                             * @member {google.ads.googleads.v11.resources.IExtensionFeedItem|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @instance
                             */
                            ExtensionFeedItemOperation.prototype.create = null;
    
                            /**
                             * ExtensionFeedItemOperation update.
                             * @member {google.ads.googleads.v11.resources.IExtensionFeedItem|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @instance
                             */
                            ExtensionFeedItemOperation.prototype.update = null;
    
                            /**
                             * ExtensionFeedItemOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @instance
                             */
                            ExtensionFeedItemOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExtensionFeedItemOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @instance
                             */
                            Object.defineProperty(ExtensionFeedItemOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExtensionFeedItemOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExtensionFeedItemOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ExtensionFeedItemOperation} ExtensionFeedItemOperation instance
                             */
                            ExtensionFeedItemOperation.create = function create(properties) {
                                return new ExtensionFeedItemOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ExtensionFeedItemOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ExtensionFeedItemOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExtensionFeedItemOperation} m ExtensionFeedItemOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionFeedItemOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.ExtensionFeedItem.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.ExtensionFeedItem.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExtensionFeedItemOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ExtensionFeedItemOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IExtensionFeedItemOperation} message ExtensionFeedItemOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExtensionFeedItemOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExtensionFeedItemOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ExtensionFeedItemOperation} ExtensionFeedItemOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionFeedItemOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExtensionFeedItemOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ExtensionFeedItemOperation} ExtensionFeedItemOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExtensionFeedItemOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExtensionFeedItemOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExtensionFeedItemOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExtensionFeedItemOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ExtensionFeedItemOperation} ExtensionFeedItemOperation
                             */
                            ExtensionFeedItemOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ExtensionFeedItemOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExtensionFeedItemOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExtensionFeedItemOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ExtensionFeedItemOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExtensionFeedItemOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ExtensionFeedItemOperation} m ExtensionFeedItemOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExtensionFeedItemOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExtensionFeedItemOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ExtensionFeedItemOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExtensionFeedItemOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExtensionFeedItemOperation;
                        })();
    
                        services.MutateExtensionFeedItemsResponse = (function() {
    
                            /**
                             * Properties of a MutateExtensionFeedItemsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExtensionFeedItemsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateExtensionFeedItemsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateExtensionFeedItemResult>|null} [results] MutateExtensionFeedItemsResponse results
                             */
    
                            /**
                             * Constructs a new MutateExtensionFeedItemsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExtensionFeedItemsResponse.
                             * @implements IMutateExtensionFeedItemsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsResponse=} [p] Properties to set
                             */
                            function MutateExtensionFeedItemsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExtensionFeedItemsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @instance
                             */
                            MutateExtensionFeedItemsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateExtensionFeedItemsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateExtensionFeedItemResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @instance
                             */
                            MutateExtensionFeedItemsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateExtensionFeedItemsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse} MutateExtensionFeedItemsResponse instance
                             */
                            MutateExtensionFeedItemsResponse.create = function create(properties) {
                                return new MutateExtensionFeedItemsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExtensionFeedItemsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsResponse} m MutateExtensionFeedItemsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExtensionFeedItemsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExtensionFeedItemsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemsResponse} message MutateExtensionFeedItemsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExtensionFeedItemsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExtensionFeedItemsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse} MutateExtensionFeedItemsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExtensionFeedItemsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExtensionFeedItemsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse} MutateExtensionFeedItemsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExtensionFeedItemsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExtensionFeedItemsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExtensionFeedItemsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExtensionFeedItemsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse} MutateExtensionFeedItemsResponse
                             */
                            MutateExtensionFeedItemsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExtensionFeedItemsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse} m MutateExtensionFeedItemsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExtensionFeedItemsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExtensionFeedItemsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExtensionFeedItemsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExtensionFeedItemsResponse;
                        })();
    
                        services.MutateExtensionFeedItemResult = (function() {
    
                            /**
                             * Properties of a MutateExtensionFeedItemResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateExtensionFeedItemResult
                             * @property {string|null} [resource_name] MutateExtensionFeedItemResult resource_name
                             * @property {google.ads.googleads.v11.resources.IExtensionFeedItem|null} [extension_feed_item] MutateExtensionFeedItemResult extension_feed_item
                             */
    
                            /**
                             * Constructs a new MutateExtensionFeedItemResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateExtensionFeedItemResult.
                             * @implements IMutateExtensionFeedItemResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemResult=} [p] Properties to set
                             */
                            function MutateExtensionFeedItemResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateExtensionFeedItemResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @instance
                             */
                            MutateExtensionFeedItemResult.prototype.resource_name = "";
    
                            /**
                             * MutateExtensionFeedItemResult extension_feed_item.
                             * @member {google.ads.googleads.v11.resources.IExtensionFeedItem|null|undefined} extension_feed_item
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @instance
                             */
                            MutateExtensionFeedItemResult.prototype.extension_feed_item = null;
    
                            /**
                             * Creates a new MutateExtensionFeedItemResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemResult} MutateExtensionFeedItemResult instance
                             */
                            MutateExtensionFeedItemResult.create = function create(properties) {
                                return new MutateExtensionFeedItemResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateExtensionFeedItemResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateExtensionFeedItemResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemResult} m MutateExtensionFeedItemResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExtensionFeedItemResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.extension_feed_item != null && Object.hasOwnProperty.call(m, "extension_feed_item"))
                                    $root.google.ads.googleads.v11.resources.ExtensionFeedItem.encode(m.extension_feed_item, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateExtensionFeedItemResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateExtensionFeedItemResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateExtensionFeedItemResult} message MutateExtensionFeedItemResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateExtensionFeedItemResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateExtensionFeedItemResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemResult} MutateExtensionFeedItemResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExtensionFeedItemResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.extension_feed_item = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateExtensionFeedItemResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemResult} MutateExtensionFeedItemResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateExtensionFeedItemResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateExtensionFeedItemResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateExtensionFeedItemResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.extension_feed_item != null && m.hasOwnProperty("extension_feed_item")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.verify(m.extension_feed_item);
                                        if (e)
                                            return "extension_feed_item." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateExtensionFeedItemResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateExtensionFeedItemResult} MutateExtensionFeedItemResult
                             */
                            MutateExtensionFeedItemResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateExtensionFeedItemResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.extension_feed_item != null) {
                                    if (typeof d.extension_feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateExtensionFeedItemResult.extension_feed_item: object expected");
                                    m.extension_feed_item = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.fromObject(d.extension_feed_item);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateExtensionFeedItemResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateExtensionFeedItemResult} m MutateExtensionFeedItemResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateExtensionFeedItemResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.extension_feed_item = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.extension_feed_item != null && m.hasOwnProperty("extension_feed_item")) {
                                    d.extension_feed_item = $root.google.ads.googleads.v11.resources.ExtensionFeedItem.toObject(m.extension_feed_item, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateExtensionFeedItemResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateExtensionFeedItemResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateExtensionFeedItemResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateExtensionFeedItemResult;
                        })();
    
                        services.FeedItemService = (function() {
    
                            /**
                             * Constructs a new FeedItemService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function FeedItemService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (FeedItemService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = FeedItemService;
    
                            /**
                             * Creates new FeedItemService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {FeedItemService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            FeedItemService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.FeedItemService|mutateFeedItems}.
                             * @memberof google.ads.googleads.v11.services.FeedItemService
                             * @typedef MutateFeedItemsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateFeedItemsResponse} [response] MutateFeedItemsResponse
                             */
    
                            /**
                             * Calls MutateFeedItems.
                             * @function mutateFeedItems
                             * @memberof google.ads.googleads.v11.services.FeedItemService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsRequest} request MutateFeedItemsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.FeedItemService.MutateFeedItemsCallback} callback Node-style callback called with the error, if any, and MutateFeedItemsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(FeedItemService.prototype.mutateFeedItems = function mutateFeedItems(request, callback) {
                                return this.rpcCall(mutateFeedItems, $root.google.ads.googleads.v11.services.MutateFeedItemsRequest, $root.google.ads.googleads.v11.services.MutateFeedItemsResponse, request, callback);
                            }, "name", { value: "MutateFeedItems" });
    
                            /**
                             * Calls MutateFeedItems.
                             * @function mutateFeedItems
                             * @memberof google.ads.googleads.v11.services.FeedItemService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsRequest} request MutateFeedItemsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateFeedItemsResponse>} Promise
                             * @variation 2
                             */
    
                            return FeedItemService;
                        })();
    
                        services.MutateFeedItemsRequest = (function() {
    
                            /**
                             * Properties of a MutateFeedItemsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemsRequest
                             * @property {string|null} [customer_id] MutateFeedItemsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IFeedItemOperation>|null} [operations] MutateFeedItemsRequest operations
                             * @property {boolean|null} [partial_failure] MutateFeedItemsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateFeedItemsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateFeedItemsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateFeedItemsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemsRequest.
                             * @implements IMutateFeedItemsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsRequest=} [p] Properties to set
                             */
                            function MutateFeedItemsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @instance
                             */
                            MutateFeedItemsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateFeedItemsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IFeedItemOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @instance
                             */
                            MutateFeedItemsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateFeedItemsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @instance
                             */
                            MutateFeedItemsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateFeedItemsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @instance
                             */
                            MutateFeedItemsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateFeedItemsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @instance
                             */
                            MutateFeedItemsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateFeedItemsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsRequest} MutateFeedItemsRequest instance
                             */
                            MutateFeedItemsRequest.create = function create(properties) {
                                return new MutateFeedItemsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsRequest} m MutateFeedItemsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.FeedItemOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsRequest} message MutateFeedItemsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsRequest} MutateFeedItemsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.FeedItemOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsRequest} MutateFeedItemsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.FeedItemOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsRequest} MutateFeedItemsRequest
                             */
                            MutateFeedItemsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.FeedItemOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemsRequest} m MutateFeedItemsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.FeedItemOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemsRequest;
                        })();
    
                        services.FeedItemOperation = (function() {
    
                            /**
                             * Properties of a FeedItemOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IFeedItemOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] FeedItemOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IFeedItem|null} [create] FeedItemOperation create
                             * @property {google.ads.googleads.v11.resources.IFeedItem|null} [update] FeedItemOperation update
                             * @property {string|null} [remove] FeedItemOperation remove
                             */
    
                            /**
                             * Constructs a new FeedItemOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemOperation.
                             * @implements IFeedItemOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IFeedItemOperation=} [p] Properties to set
                             */
                            function FeedItemOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @instance
                             */
                            FeedItemOperation.prototype.update_mask = null;
    
                            /**
                             * FeedItemOperation create.
                             * @member {google.ads.googleads.v11.resources.IFeedItem|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @instance
                             */
                            FeedItemOperation.prototype.create = null;
    
                            /**
                             * FeedItemOperation update.
                             * @member {google.ads.googleads.v11.resources.IFeedItem|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @instance
                             */
                            FeedItemOperation.prototype.update = null;
    
                            /**
                             * FeedItemOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @instance
                             */
                            FeedItemOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @instance
                             */
                            Object.defineProperty(FeedItemOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.FeedItemOperation} FeedItemOperation instance
                             */
                            FeedItemOperation.create = function create(properties) {
                                return new FeedItemOperation(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemOperation message. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemOperation} m FeedItemOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.FeedItem.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.FeedItem.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemOperation} message FeedItemOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.FeedItemOperation} FeedItemOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.FeedItemOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.FeedItem.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.FeedItem.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.FeedItemOperation} FeedItemOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItem.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItem.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.FeedItemOperation} FeedItemOperation
                             */
                            FeedItemOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.FeedItemOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.FeedItemOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.FeedItem.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.FeedItem.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.FeedItemOperation} m FeedItemOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.FeedItem.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.FeedItem.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.FeedItemOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemOperation;
                        })();
    
                        services.MutateFeedItemsResponse = (function() {
    
                            /**
                             * Properties of a MutateFeedItemsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateFeedItemsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateFeedItemResult>|null} [results] MutateFeedItemsResponse results
                             */
    
                            /**
                             * Constructs a new MutateFeedItemsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemsResponse.
                             * @implements IMutateFeedItemsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsResponse=} [p] Properties to set
                             */
                            function MutateFeedItemsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @instance
                             */
                            MutateFeedItemsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateFeedItemsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateFeedItemResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @instance
                             */
                            MutateFeedItemsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateFeedItemsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsResponse} MutateFeedItemsResponse instance
                             */
                            MutateFeedItemsResponse.create = function create(properties) {
                                return new MutateFeedItemsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsResponse} m MutateFeedItemsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateFeedItemResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemsResponse} message MutateFeedItemsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsResponse} MutateFeedItemsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateFeedItemResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsResponse} MutateFeedItemsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateFeedItemResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemsResponse} MutateFeedItemsResponse
                             */
                            MutateFeedItemsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateFeedItemResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemsResponse} m MutateFeedItemsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateFeedItemResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemsResponse;
                        })();
    
                        services.MutateFeedItemResult = (function() {
    
                            /**
                             * Properties of a MutateFeedItemResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemResult
                             * @property {string|null} [resource_name] MutateFeedItemResult resource_name
                             * @property {google.ads.googleads.v11.resources.IFeedItem|null} [feed_item] MutateFeedItemResult feed_item
                             */
    
                            /**
                             * Constructs a new MutateFeedItemResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemResult.
                             * @implements IMutateFeedItemResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemResult=} [p] Properties to set
                             */
                            function MutateFeedItemResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @instance
                             */
                            MutateFeedItemResult.prototype.resource_name = "";
    
                            /**
                             * MutateFeedItemResult feed_item.
                             * @member {google.ads.googleads.v11.resources.IFeedItem|null|undefined} feed_item
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @instance
                             */
                            MutateFeedItemResult.prototype.feed_item = null;
    
                            /**
                             * Creates a new MutateFeedItemResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemResult} MutateFeedItemResult instance
                             */
                            MutateFeedItemResult.create = function create(properties) {
                                return new MutateFeedItemResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemResult} m MutateFeedItemResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.feed_item != null && Object.hasOwnProperty.call(m, "feed_item"))
                                    $root.google.ads.googleads.v11.resources.FeedItem.encode(m.feed_item, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemResult} message MutateFeedItemResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemResult} MutateFeedItemResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemResult} MutateFeedItemResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItem.verify(m.feed_item);
                                        if (e)
                                            return "feed_item." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemResult} MutateFeedItemResult
                             */
                            MutateFeedItemResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed_item != null) {
                                    if (typeof d.feed_item !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemResult.feed_item: object expected");
                                    m.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.fromObject(d.feed_item);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemResult} m MutateFeedItemResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.feed_item = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.feed_item != null && m.hasOwnProperty("feed_item")) {
                                    d.feed_item = $root.google.ads.googleads.v11.resources.FeedItem.toObject(m.feed_item, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemResult;
                        })();
    
                        services.FeedItemSetLinkService = (function() {
    
                            /**
                             * Constructs a new FeedItemSetLinkService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemSetLinkService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function FeedItemSetLinkService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (FeedItemSetLinkService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = FeedItemSetLinkService;
    
                            /**
                             * Creates new FeedItemSetLinkService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {FeedItemSetLinkService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            FeedItemSetLinkService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.FeedItemSetLinkService|mutateFeedItemSetLinks}.
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkService
                             * @typedef MutateFeedItemSetLinksCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse} [response] MutateFeedItemSetLinksResponse
                             */
    
                            /**
                             * Calls MutateFeedItemSetLinks.
                             * @function mutateFeedItemSetLinks
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksRequest} request MutateFeedItemSetLinksRequest message or plain object
                             * @param {google.ads.googleads.v11.services.FeedItemSetLinkService.MutateFeedItemSetLinksCallback} callback Node-style callback called with the error, if any, and MutateFeedItemSetLinksResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(FeedItemSetLinkService.prototype.mutateFeedItemSetLinks = function mutateFeedItemSetLinks(request, callback) {
                                return this.rpcCall(mutateFeedItemSetLinks, $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest, $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse, request, callback);
                            }, "name", { value: "MutateFeedItemSetLinks" });
    
                            /**
                             * Calls MutateFeedItemSetLinks.
                             * @function mutateFeedItemSetLinks
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksRequest} request MutateFeedItemSetLinksRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse>} Promise
                             * @variation 2
                             */
    
                            return FeedItemSetLinkService;
                        })();
    
                        services.MutateFeedItemSetLinksRequest = (function() {
    
                            /**
                             * Properties of a MutateFeedItemSetLinksRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemSetLinksRequest
                             * @property {string|null} [customer_id] MutateFeedItemSetLinksRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IFeedItemSetLinkOperation>|null} [operations] MutateFeedItemSetLinksRequest operations
                             * @property {boolean|null} [partial_failure] MutateFeedItemSetLinksRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateFeedItemSetLinksRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateFeedItemSetLinksRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemSetLinksRequest.
                             * @implements IMutateFeedItemSetLinksRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksRequest=} [p] Properties to set
                             */
                            function MutateFeedItemSetLinksRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemSetLinksRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @instance
                             */
                            MutateFeedItemSetLinksRequest.prototype.customer_id = "";
    
                            /**
                             * MutateFeedItemSetLinksRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IFeedItemSetLinkOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @instance
                             */
                            MutateFeedItemSetLinksRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateFeedItemSetLinksRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @instance
                             */
                            MutateFeedItemSetLinksRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateFeedItemSetLinksRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @instance
                             */
                            MutateFeedItemSetLinksRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateFeedItemSetLinksRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest} MutateFeedItemSetLinksRequest instance
                             */
                            MutateFeedItemSetLinksRequest.create = function create(properties) {
                                return new MutateFeedItemSetLinksRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetLinksRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksRequest} m MutateFeedItemSetLinksRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetLinksRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetLinksRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksRequest} message MutateFeedItemSetLinksRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetLinksRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetLinksRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest} MutateFeedItemSetLinksRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetLinksRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetLinksRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest} MutateFeedItemSetLinksRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetLinksRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemSetLinksRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemSetLinksRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemSetLinksRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest} MutateFeedItemSetLinksRequest
                             */
                            MutateFeedItemSetLinksRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemSetLinksRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest} m MutateFeedItemSetLinksRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemSetLinksRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemSetLinksRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemSetLinksRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemSetLinksRequest;
                        })();
    
                        services.FeedItemSetLinkOperation = (function() {
    
                            /**
                             * Properties of a FeedItemSetLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IFeedItemSetLinkOperation
                             * @property {google.ads.googleads.v11.resources.IFeedItemSetLink|null} [create] FeedItemSetLinkOperation create
                             * @property {string|null} [remove] FeedItemSetLinkOperation remove
                             */
    
                            /**
                             * Constructs a new FeedItemSetLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemSetLinkOperation.
                             * @implements IFeedItemSetLinkOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IFeedItemSetLinkOperation=} [p] Properties to set
                             */
                            function FeedItemSetLinkOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemSetLinkOperation create.
                             * @member {google.ads.googleads.v11.resources.IFeedItemSetLink|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @instance
                             */
                            FeedItemSetLinkOperation.prototype.create = null;
    
                            /**
                             * FeedItemSetLinkOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @instance
                             */
                            FeedItemSetLinkOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemSetLinkOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @instance
                             */
                            Object.defineProperty(FeedItemSetLinkOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemSetLinkOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemSetLinkOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.FeedItemSetLinkOperation} FeedItemSetLinkOperation instance
                             */
                            FeedItemSetLinkOperation.create = function create(properties) {
                                return new FeedItemSetLinkOperation(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSetLinkOperation message. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemSetLinkOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemSetLinkOperation} m FeedItemSetLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetLinkOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.FeedItemSetLink.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSetLinkOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemSetLinkOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemSetLinkOperation} message FeedItemSetLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetLinkOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSetLinkOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.FeedItemSetLinkOperation} FeedItemSetLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetLinkOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.FeedItemSetLink.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSetLinkOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.FeedItemSetLinkOperation} FeedItemSetLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetLinkOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSetLinkOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSetLinkOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemSetLink.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSetLinkOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.FeedItemSetLinkOperation} FeedItemSetLinkOperation
                             */
                            FeedItemSetLinkOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.FeedItemSetLinkOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemSetLinkOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.FeedItemSetLink.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSetLinkOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.FeedItemSetLinkOperation} m FeedItemSetLinkOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSetLinkOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.FeedItemSetLink.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemSetLinkOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.FeedItemSetLinkOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSetLinkOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemSetLinkOperation;
                        })();
    
                        services.MutateFeedItemSetLinksResponse = (function() {
    
                            /**
                             * Properties of a MutateFeedItemSetLinksResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemSetLinksResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult>|null} [results] MutateFeedItemSetLinksResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateFeedItemSetLinksResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateFeedItemSetLinksResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemSetLinksResponse.
                             * @implements IMutateFeedItemSetLinksResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksResponse=} [p] Properties to set
                             */
                            function MutateFeedItemSetLinksResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemSetLinksResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @instance
                             */
                            MutateFeedItemSetLinksResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateFeedItemSetLinksResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @instance
                             */
                            MutateFeedItemSetLinksResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateFeedItemSetLinksResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse} MutateFeedItemSetLinksResponse instance
                             */
                            MutateFeedItemSetLinksResponse.create = function create(properties) {
                                return new MutateFeedItemSetLinksResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetLinksResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksResponse} m MutateFeedItemSetLinksResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetLinksResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetLinksResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinksResponse} message MutateFeedItemSetLinksResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetLinksResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetLinksResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse} MutateFeedItemSetLinksResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetLinksResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetLinksResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse} MutateFeedItemSetLinksResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetLinksResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemSetLinksResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemSetLinksResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemSetLinksResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse} MutateFeedItemSetLinksResponse
                             */
                            MutateFeedItemSetLinksResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemSetLinksResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse} m MutateFeedItemSetLinksResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemSetLinksResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemSetLinksResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinksResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemSetLinksResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemSetLinksResponse;
                        })();
    
                        services.MutateFeedItemSetLinkResult = (function() {
    
                            /**
                             * Properties of a MutateFeedItemSetLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemSetLinkResult
                             * @property {string|null} [resource_name] MutateFeedItemSetLinkResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateFeedItemSetLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemSetLinkResult.
                             * @implements IMutateFeedItemSetLinkResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult=} [p] Properties to set
                             */
                            function MutateFeedItemSetLinkResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemSetLinkResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @instance
                             */
                            MutateFeedItemSetLinkResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateFeedItemSetLinkResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinkResult} MutateFeedItemSetLinkResult instance
                             */
                            MutateFeedItemSetLinkResult.create = function create(properties) {
                                return new MutateFeedItemSetLinkResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetLinkResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult} m MutateFeedItemSetLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetLinkResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetLinkResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetLinkResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetLinkResult} message MutateFeedItemSetLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetLinkResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetLinkResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinkResult} MutateFeedItemSetLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetLinkResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetLinkResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinkResult} MutateFeedItemSetLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetLinkResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemSetLinkResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemSetLinkResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemSetLinkResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetLinkResult} MutateFeedItemSetLinkResult
                             */
                            MutateFeedItemSetLinkResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetLinkResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemSetLinkResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetLinkResult} m MutateFeedItemSetLinkResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemSetLinkResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemSetLinkResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetLinkResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemSetLinkResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemSetLinkResult;
                        })();
    
                        services.FeedItemSetService = (function() {
    
                            /**
                             * Constructs a new FeedItemSetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemSetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function FeedItemSetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (FeedItemSetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = FeedItemSetService;
    
                            /**
                             * Creates new FeedItemSetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemSetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {FeedItemSetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            FeedItemSetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.FeedItemSetService|mutateFeedItemSets}.
                             * @memberof google.ads.googleads.v11.services.FeedItemSetService
                             * @typedef MutateFeedItemSetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetsResponse} [response] MutateFeedItemSetsResponse
                             */
    
                            /**
                             * Calls MutateFeedItemSets.
                             * @function mutateFeedItemSets
                             * @memberof google.ads.googleads.v11.services.FeedItemSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsRequest} request MutateFeedItemSetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.FeedItemSetService.MutateFeedItemSetsCallback} callback Node-style callback called with the error, if any, and MutateFeedItemSetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(FeedItemSetService.prototype.mutateFeedItemSets = function mutateFeedItemSets(request, callback) {
                                return this.rpcCall(mutateFeedItemSets, $root.google.ads.googleads.v11.services.MutateFeedItemSetsRequest, $root.google.ads.googleads.v11.services.MutateFeedItemSetsResponse, request, callback);
                            }, "name", { value: "MutateFeedItemSets" });
    
                            /**
                             * Calls MutateFeedItemSets.
                             * @function mutateFeedItemSets
                             * @memberof google.ads.googleads.v11.services.FeedItemSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsRequest} request MutateFeedItemSetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateFeedItemSetsResponse>} Promise
                             * @variation 2
                             */
    
                            return FeedItemSetService;
                        })();
    
                        services.MutateFeedItemSetsRequest = (function() {
    
                            /**
                             * Properties of a MutateFeedItemSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemSetsRequest
                             * @property {string|null} [customer_id] MutateFeedItemSetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IFeedItemSetOperation>|null} [operations] MutateFeedItemSetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateFeedItemSetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateFeedItemSetsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateFeedItemSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemSetsRequest.
                             * @implements IMutateFeedItemSetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsRequest=} [p] Properties to set
                             */
                            function MutateFeedItemSetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemSetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @instance
                             */
                            MutateFeedItemSetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateFeedItemSetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IFeedItemSetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @instance
                             */
                            MutateFeedItemSetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateFeedItemSetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @instance
                             */
                            MutateFeedItemSetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateFeedItemSetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @instance
                             */
                            MutateFeedItemSetsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateFeedItemSetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsRequest} MutateFeedItemSetsRequest instance
                             */
                            MutateFeedItemSetsRequest.create = function create(properties) {
                                return new MutateFeedItemSetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsRequest} m MutateFeedItemSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.FeedItemSetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsRequest} message MutateFeedItemSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsRequest} MutateFeedItemSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.FeedItemSetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsRequest} MutateFeedItemSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemSetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemSetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.FeedItemSetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemSetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsRequest} MutateFeedItemSetsRequest
                             */
                            MutateFeedItemSetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemSetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.FeedItemSetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemSetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetsRequest} m MutateFeedItemSetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemSetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.FeedItemSetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemSetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemSetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemSetsRequest;
                        })();
    
                        services.FeedItemSetOperation = (function() {
    
                            /**
                             * Properties of a FeedItemSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IFeedItemSetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] FeedItemSetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IFeedItemSet|null} [create] FeedItemSetOperation create
                             * @property {google.ads.googleads.v11.resources.IFeedItemSet|null} [update] FeedItemSetOperation update
                             * @property {string|null} [remove] FeedItemSetOperation remove
                             */
    
                            /**
                             * Constructs a new FeedItemSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemSetOperation.
                             * @implements IFeedItemSetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IFeedItemSetOperation=} [p] Properties to set
                             */
                            function FeedItemSetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemSetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @instance
                             */
                            FeedItemSetOperation.prototype.update_mask = null;
    
                            /**
                             * FeedItemSetOperation create.
                             * @member {google.ads.googleads.v11.resources.IFeedItemSet|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @instance
                             */
                            FeedItemSetOperation.prototype.create = null;
    
                            /**
                             * FeedItemSetOperation update.
                             * @member {google.ads.googleads.v11.resources.IFeedItemSet|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @instance
                             */
                            FeedItemSetOperation.prototype.update = null;
    
                            /**
                             * FeedItemSetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @instance
                             */
                            FeedItemSetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemSetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @instance
                             */
                            Object.defineProperty(FeedItemSetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemSetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemSetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.FeedItemSetOperation} FeedItemSetOperation instance
                             */
                            FeedItemSetOperation.create = function create(properties) {
                                return new FeedItemSetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemSetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemSetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemSetOperation} m FeedItemSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.FeedItemSet.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.FeedItemSet.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemSetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemSetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemSetOperation} message FeedItemSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemSetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemSetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.FeedItemSetOperation} FeedItemSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.FeedItemSetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.FeedItemSet.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.FeedItemSet.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemSetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.FeedItemSetOperation} FeedItemSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemSetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemSetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemSetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemSet.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemSet.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemSetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.FeedItemSetOperation} FeedItemSetOperation
                             */
                            FeedItemSetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.FeedItemSetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.FeedItemSetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemSetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemSetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.FeedItemSet.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemSetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.FeedItemSet.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemSetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.FeedItemSetOperation} m FeedItemSetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemSetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.FeedItemSet.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.FeedItemSet.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemSetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.FeedItemSetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemSetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemSetOperation;
                        })();
    
                        services.MutateFeedItemSetsResponse = (function() {
    
                            /**
                             * Properties of a MutateFeedItemSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemSetsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateFeedItemSetResult>|null} [results] MutateFeedItemSetsResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateFeedItemSetsResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new MutateFeedItemSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemSetsResponse.
                             * @implements IMutateFeedItemSetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsResponse=} [p] Properties to set
                             */
                            function MutateFeedItemSetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemSetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateFeedItemSetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @instance
                             */
                            MutateFeedItemSetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * MutateFeedItemSetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @instance
                             */
                            MutateFeedItemSetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new MutateFeedItemSetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsResponse} MutateFeedItemSetsResponse instance
                             */
                            MutateFeedItemSetsResponse.create = function create(properties) {
                                return new MutateFeedItemSetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsResponse} m MutateFeedItemSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetsResponse} message MutateFeedItemSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsResponse} MutateFeedItemSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateFeedItemSetResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsResponse} MutateFeedItemSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemSetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemSetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemSetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetsResponse} MutateFeedItemSetsResponse
                             */
                            MutateFeedItemSetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemSetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemSetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemSetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetsResponse} m MutateFeedItemSetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemSetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateFeedItemSetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemSetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemSetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemSetsResponse;
                        })();
    
                        services.MutateFeedItemSetResult = (function() {
    
                            /**
                             * Properties of a MutateFeedItemSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemSetResult
                             * @property {string|null} [resource_name] MutateFeedItemSetResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateFeedItemSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemSetResult.
                             * @implements IMutateFeedItemSetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetResult=} [p] Properties to set
                             */
                            function MutateFeedItemSetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemSetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @instance
                             */
                            MutateFeedItemSetResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateFeedItemSetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetResult} MutateFeedItemSetResult instance
                             */
                            MutateFeedItemSetResult.create = function create(properties) {
                                return new MutateFeedItemSetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetResult} m MutateFeedItemSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemSetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemSetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemSetResult} message MutateFeedItemSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemSetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetResult} MutateFeedItemSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemSetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetResult} MutateFeedItemSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemSetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemSetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemSetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemSetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemSetResult} MutateFeedItemSetResult
                             */
                            MutateFeedItemSetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemSetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemSetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemSetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemSetResult} m MutateFeedItemSetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemSetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemSetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemSetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemSetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemSetResult;
                        })();
    
                        services.FeedItemTargetService = (function() {
    
                            /**
                             * Constructs a new FeedItemTargetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemTargetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function FeedItemTargetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (FeedItemTargetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = FeedItemTargetService;
    
                            /**
                             * Creates new FeedItemTargetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {FeedItemTargetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            FeedItemTargetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.FeedItemTargetService|mutateFeedItemTargets}.
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetService
                             * @typedef MutateFeedItemTargetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateFeedItemTargetsResponse} [response] MutateFeedItemTargetsResponse
                             */
    
                            /**
                             * Calls MutateFeedItemTargets.
                             * @function mutateFeedItemTargets
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsRequest} request MutateFeedItemTargetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.FeedItemTargetService.MutateFeedItemTargetsCallback} callback Node-style callback called with the error, if any, and MutateFeedItemTargetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(FeedItemTargetService.prototype.mutateFeedItemTargets = function mutateFeedItemTargets(request, callback) {
                                return this.rpcCall(mutateFeedItemTargets, $root.google.ads.googleads.v11.services.MutateFeedItemTargetsRequest, $root.google.ads.googleads.v11.services.MutateFeedItemTargetsResponse, request, callback);
                            }, "name", { value: "MutateFeedItemTargets" });
    
                            /**
                             * Calls MutateFeedItemTargets.
                             * @function mutateFeedItemTargets
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsRequest} request MutateFeedItemTargetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateFeedItemTargetsResponse>} Promise
                             * @variation 2
                             */
    
                            return FeedItemTargetService;
                        })();
    
                        services.MutateFeedItemTargetsRequest = (function() {
    
                            /**
                             * Properties of a MutateFeedItemTargetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemTargetsRequest
                             * @property {string|null} [customer_id] MutateFeedItemTargetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IFeedItemTargetOperation>|null} [operations] MutateFeedItemTargetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateFeedItemTargetsRequest partial_failure
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateFeedItemTargetsRequest response_content_type
                             * @property {boolean|null} [validate_only] MutateFeedItemTargetsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateFeedItemTargetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemTargetsRequest.
                             * @implements IMutateFeedItemTargetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsRequest=} [p] Properties to set
                             */
                            function MutateFeedItemTargetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemTargetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @instance
                             */
                            MutateFeedItemTargetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateFeedItemTargetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IFeedItemTargetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @instance
                             */
                            MutateFeedItemTargetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateFeedItemTargetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @instance
                             */
                            MutateFeedItemTargetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateFeedItemTargetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @instance
                             */
                            MutateFeedItemTargetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * MutateFeedItemTargetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @instance
                             */
                            MutateFeedItemTargetsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateFeedItemTargetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsRequest} MutateFeedItemTargetsRequest instance
                             */
                            MutateFeedItemTargetsRequest.create = function create(properties) {
                                return new MutateFeedItemTargetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemTargetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemTargetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsRequest} m MutateFeedItemTargetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemTargetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.FeedItemTargetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(32).bool(m.partial_failure);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemTargetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemTargetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsRequest} message MutateFeedItemTargetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemTargetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemTargetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsRequest} MutateFeedItemTargetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemTargetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemTargetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.FeedItemTargetOperation.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemTargetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsRequest} MutateFeedItemTargetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemTargetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemTargetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemTargetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.FeedItemTargetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemTargetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsRequest} MutateFeedItemTargetsRequest
                             */
                            MutateFeedItemTargetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemTargetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemTargetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemTargetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemTargetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.FeedItemTargetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemTargetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemTargetsRequest} m MutateFeedItemTargetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemTargetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                    d.partial_failure = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.FeedItemTargetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemTargetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemTargetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemTargetsRequest;
                        })();
    
                        services.FeedItemTargetOperation = (function() {
    
                            /**
                             * Properties of a FeedItemTargetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IFeedItemTargetOperation
                             * @property {google.ads.googleads.v11.resources.IFeedItemTarget|null} [create] FeedItemTargetOperation create
                             * @property {string|null} [remove] FeedItemTargetOperation remove
                             */
    
                            /**
                             * Constructs a new FeedItemTargetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedItemTargetOperation.
                             * @implements IFeedItemTargetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IFeedItemTargetOperation=} [p] Properties to set
                             */
                            function FeedItemTargetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedItemTargetOperation create.
                             * @member {google.ads.googleads.v11.resources.IFeedItemTarget|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @instance
                             */
                            FeedItemTargetOperation.prototype.create = null;
    
                            /**
                             * FeedItemTargetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @instance
                             */
                            FeedItemTargetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedItemTargetOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @instance
                             */
                            Object.defineProperty(FeedItemTargetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedItemTargetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemTargetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.FeedItemTargetOperation} FeedItemTargetOperation instance
                             */
                            FeedItemTargetOperation.create = function create(properties) {
                                return new FeedItemTargetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemTargetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemTargetOperation} m FeedItemTargetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.FeedItemTarget.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedItemTargetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.FeedItemTargetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedItemTargetOperation} message FeedItemTargetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedItemTargetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedItemTargetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.FeedItemTargetOperation} FeedItemTargetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.FeedItemTargetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.FeedItemTarget.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedItemTargetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.FeedItemTargetOperation} FeedItemTargetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedItemTargetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedItemTargetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedItemTargetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemTarget.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedItemTargetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.FeedItemTargetOperation} FeedItemTargetOperation
                             */
                            FeedItemTargetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.FeedItemTargetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.FeedItemTargetOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedItemTargetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.FeedItemTarget.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedItemTargetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.FeedItemTargetOperation} m FeedItemTargetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedItemTargetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.FeedItemTarget.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedItemTargetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.FeedItemTargetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedItemTargetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedItemTargetOperation;
                        })();
    
                        services.MutateFeedItemTargetsResponse = (function() {
    
                            /**
                             * Properties of a MutateFeedItemTargetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemTargetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateFeedItemTargetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateFeedItemTargetResult>|null} [results] MutateFeedItemTargetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateFeedItemTargetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemTargetsResponse.
                             * @implements IMutateFeedItemTargetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsResponse=} [p] Properties to set
                             */
                            function MutateFeedItemTargetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemTargetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @instance
                             */
                            MutateFeedItemTargetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateFeedItemTargetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateFeedItemTargetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @instance
                             */
                            MutateFeedItemTargetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateFeedItemTargetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsResponse} MutateFeedItemTargetsResponse instance
                             */
                            MutateFeedItemTargetsResponse.create = function create(properties) {
                                return new MutateFeedItemTargetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemTargetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemTargetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsResponse} m MutateFeedItemTargetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemTargetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemTargetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemTargetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetsResponse} message MutateFeedItemTargetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemTargetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemTargetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsResponse} MutateFeedItemTargetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemTargetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemTargetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemTargetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsResponse} MutateFeedItemTargetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemTargetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemTargetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemTargetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemTargetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetsResponse} MutateFeedItemTargetsResponse
                             */
                            MutateFeedItemTargetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemTargetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemTargetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemTargetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemTargetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemTargetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemTargetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemTargetsResponse} m MutateFeedItemTargetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemTargetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemTargetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemTargetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemTargetsResponse;
                        })();
    
                        services.MutateFeedItemTargetResult = (function() {
    
                            /**
                             * Properties of a MutateFeedItemTargetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedItemTargetResult
                             * @property {string|null} [resource_name] MutateFeedItemTargetResult resource_name
                             * @property {google.ads.googleads.v11.resources.IFeedItemTarget|null} [feed_item_target] MutateFeedItemTargetResult feed_item_target
                             */
    
                            /**
                             * Constructs a new MutateFeedItemTargetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedItemTargetResult.
                             * @implements IMutateFeedItemTargetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetResult=} [p] Properties to set
                             */
                            function MutateFeedItemTargetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedItemTargetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @instance
                             */
                            MutateFeedItemTargetResult.prototype.resource_name = "";
    
                            /**
                             * MutateFeedItemTargetResult feed_item_target.
                             * @member {google.ads.googleads.v11.resources.IFeedItemTarget|null|undefined} feed_item_target
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @instance
                             */
                            MutateFeedItemTargetResult.prototype.feed_item_target = null;
    
                            /**
                             * Creates a new MutateFeedItemTargetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetResult} MutateFeedItemTargetResult instance
                             */
                            MutateFeedItemTargetResult.create = function create(properties) {
                                return new MutateFeedItemTargetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemTargetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemTargetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetResult} m MutateFeedItemTargetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemTargetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.feed_item_target != null && Object.hasOwnProperty.call(m, "feed_item_target"))
                                    $root.google.ads.googleads.v11.resources.FeedItemTarget.encode(m.feed_item_target, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedItemTargetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedItemTargetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedItemTargetResult} message MutateFeedItemTargetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedItemTargetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedItemTargetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetResult} MutateFeedItemTargetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemTargetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.feed_item_target = $root.google.ads.googleads.v11.resources.FeedItemTarget.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedItemTargetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetResult} MutateFeedItemTargetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedItemTargetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedItemTargetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedItemTargetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed_item_target != null && m.hasOwnProperty("feed_item_target")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedItemTarget.verify(m.feed_item_target);
                                        if (e)
                                            return "feed_item_target." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedItemTargetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedItemTargetResult} MutateFeedItemTargetResult
                             */
                            MutateFeedItemTargetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedItemTargetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed_item_target != null) {
                                    if (typeof d.feed_item_target !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedItemTargetResult.feed_item_target: object expected");
                                    m.feed_item_target = $root.google.ads.googleads.v11.resources.FeedItemTarget.fromObject(d.feed_item_target);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedItemTargetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedItemTargetResult} m MutateFeedItemTargetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedItemTargetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.feed_item_target = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.feed_item_target != null && m.hasOwnProperty("feed_item_target")) {
                                    d.feed_item_target = $root.google.ads.googleads.v11.resources.FeedItemTarget.toObject(m.feed_item_target, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedItemTargetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedItemTargetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedItemTargetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedItemTargetResult;
                        })();
    
                        services.FeedMappingService = (function() {
    
                            /**
                             * Constructs a new FeedMappingService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedMappingService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function FeedMappingService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (FeedMappingService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = FeedMappingService;
    
                            /**
                             * Creates new FeedMappingService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedMappingService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {FeedMappingService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            FeedMappingService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.FeedMappingService|mutateFeedMappings}.
                             * @memberof google.ads.googleads.v11.services.FeedMappingService
                             * @typedef MutateFeedMappingsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateFeedMappingsResponse} [response] MutateFeedMappingsResponse
                             */
    
                            /**
                             * Calls MutateFeedMappings.
                             * @function mutateFeedMappings
                             * @memberof google.ads.googleads.v11.services.FeedMappingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsRequest} request MutateFeedMappingsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.FeedMappingService.MutateFeedMappingsCallback} callback Node-style callback called with the error, if any, and MutateFeedMappingsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(FeedMappingService.prototype.mutateFeedMappings = function mutateFeedMappings(request, callback) {
                                return this.rpcCall(mutateFeedMappings, $root.google.ads.googleads.v11.services.MutateFeedMappingsRequest, $root.google.ads.googleads.v11.services.MutateFeedMappingsResponse, request, callback);
                            }, "name", { value: "MutateFeedMappings" });
    
                            /**
                             * Calls MutateFeedMappings.
                             * @function mutateFeedMappings
                             * @memberof google.ads.googleads.v11.services.FeedMappingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsRequest} request MutateFeedMappingsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateFeedMappingsResponse>} Promise
                             * @variation 2
                             */
    
                            return FeedMappingService;
                        })();
    
                        services.MutateFeedMappingsRequest = (function() {
    
                            /**
                             * Properties of a MutateFeedMappingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedMappingsRequest
                             * @property {string|null} [customer_id] MutateFeedMappingsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IFeedMappingOperation>|null} [operations] MutateFeedMappingsRequest operations
                             * @property {boolean|null} [partial_failure] MutateFeedMappingsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateFeedMappingsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateFeedMappingsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateFeedMappingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedMappingsRequest.
                             * @implements IMutateFeedMappingsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsRequest=} [p] Properties to set
                             */
                            function MutateFeedMappingsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedMappingsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @instance
                             */
                            MutateFeedMappingsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateFeedMappingsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IFeedMappingOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @instance
                             */
                            MutateFeedMappingsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateFeedMappingsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @instance
                             */
                            MutateFeedMappingsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateFeedMappingsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @instance
                             */
                            MutateFeedMappingsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateFeedMappingsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @instance
                             */
                            MutateFeedMappingsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateFeedMappingsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsRequest} MutateFeedMappingsRequest instance
                             */
                            MutateFeedMappingsRequest.create = function create(properties) {
                                return new MutateFeedMappingsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedMappingsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedMappingsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsRequest} m MutateFeedMappingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedMappingsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.FeedMappingOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedMappingsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedMappingsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsRequest} message MutateFeedMappingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedMappingsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedMappingsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsRequest} MutateFeedMappingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedMappingsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedMappingsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.FeedMappingOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedMappingsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsRequest} MutateFeedMappingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedMappingsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedMappingsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedMappingsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.FeedMappingOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedMappingsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsRequest} MutateFeedMappingsRequest
                             */
                            MutateFeedMappingsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedMappingsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedMappingsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedMappingsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedMappingsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.FeedMappingOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedMappingsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedMappingsRequest} m MutateFeedMappingsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedMappingsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.FeedMappingOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedMappingsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedMappingsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedMappingsRequest;
                        })();
    
                        services.FeedMappingOperation = (function() {
    
                            /**
                             * Properties of a FeedMappingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IFeedMappingOperation
                             * @property {google.ads.googleads.v11.resources.IFeedMapping|null} [create] FeedMappingOperation create
                             * @property {string|null} [remove] FeedMappingOperation remove
                             */
    
                            /**
                             * Constructs a new FeedMappingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedMappingOperation.
                             * @implements IFeedMappingOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IFeedMappingOperation=} [p] Properties to set
                             */
                            function FeedMappingOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedMappingOperation create.
                             * @member {google.ads.googleads.v11.resources.IFeedMapping|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @instance
                             */
                            FeedMappingOperation.prototype.create = null;
    
                            /**
                             * FeedMappingOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @instance
                             */
                            FeedMappingOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedMappingOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @instance
                             */
                            Object.defineProperty(FeedMappingOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedMappingOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedMappingOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.FeedMappingOperation} FeedMappingOperation instance
                             */
                            FeedMappingOperation.create = function create(properties) {
                                return new FeedMappingOperation(properties);
                            };
    
                            /**
                             * Encodes the specified FeedMappingOperation message. Does not implicitly {@link google.ads.googleads.v11.services.FeedMappingOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedMappingOperation} m FeedMappingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.FeedMapping.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedMappingOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.FeedMappingOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedMappingOperation} message FeedMappingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedMappingOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedMappingOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.FeedMappingOperation} FeedMappingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.FeedMappingOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.FeedMapping.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedMappingOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.FeedMappingOperation} FeedMappingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedMappingOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedMappingOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedMappingOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedMapping.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedMappingOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.FeedMappingOperation} FeedMappingOperation
                             */
                            FeedMappingOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.FeedMappingOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.FeedMappingOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedMappingOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.FeedMapping.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedMappingOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.FeedMappingOperation} m FeedMappingOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedMappingOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.FeedMapping.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedMappingOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.FeedMappingOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedMappingOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedMappingOperation;
                        })();
    
                        services.MutateFeedMappingsResponse = (function() {
    
                            /**
                             * Properties of a MutateFeedMappingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedMappingsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateFeedMappingsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateFeedMappingResult>|null} [results] MutateFeedMappingsResponse results
                             */
    
                            /**
                             * Constructs a new MutateFeedMappingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedMappingsResponse.
                             * @implements IMutateFeedMappingsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsResponse=} [p] Properties to set
                             */
                            function MutateFeedMappingsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedMappingsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @instance
                             */
                            MutateFeedMappingsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateFeedMappingsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateFeedMappingResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @instance
                             */
                            MutateFeedMappingsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateFeedMappingsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsResponse} MutateFeedMappingsResponse instance
                             */
                            MutateFeedMappingsResponse.create = function create(properties) {
                                return new MutateFeedMappingsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedMappingsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedMappingsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsResponse} m MutateFeedMappingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedMappingsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateFeedMappingResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedMappingsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedMappingsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingsResponse} message MutateFeedMappingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedMappingsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedMappingsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsResponse} MutateFeedMappingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedMappingsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedMappingsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateFeedMappingResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedMappingsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsResponse} MutateFeedMappingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedMappingsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedMappingsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedMappingsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateFeedMappingResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedMappingsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingsResponse} MutateFeedMappingsResponse
                             */
                            MutateFeedMappingsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedMappingsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedMappingsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedMappingsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedMappingsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedMappingsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateFeedMappingResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedMappingsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedMappingsResponse} m MutateFeedMappingsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedMappingsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateFeedMappingResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedMappingsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedMappingsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedMappingsResponse;
                        })();
    
                        services.MutateFeedMappingResult = (function() {
    
                            /**
                             * Properties of a MutateFeedMappingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedMappingResult
                             * @property {string|null} [resource_name] MutateFeedMappingResult resource_name
                             * @property {google.ads.googleads.v11.resources.IFeedMapping|null} [feed_mapping] MutateFeedMappingResult feed_mapping
                             */
    
                            /**
                             * Constructs a new MutateFeedMappingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedMappingResult.
                             * @implements IMutateFeedMappingResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingResult=} [p] Properties to set
                             */
                            function MutateFeedMappingResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedMappingResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @instance
                             */
                            MutateFeedMappingResult.prototype.resource_name = "";
    
                            /**
                             * MutateFeedMappingResult feed_mapping.
                             * @member {google.ads.googleads.v11.resources.IFeedMapping|null|undefined} feed_mapping
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @instance
                             */
                            MutateFeedMappingResult.prototype.feed_mapping = null;
    
                            /**
                             * Creates a new MutateFeedMappingResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingResult} MutateFeedMappingResult instance
                             */
                            MutateFeedMappingResult.create = function create(properties) {
                                return new MutateFeedMappingResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedMappingResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedMappingResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingResult} m MutateFeedMappingResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedMappingResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.feed_mapping != null && Object.hasOwnProperty.call(m, "feed_mapping"))
                                    $root.google.ads.googleads.v11.resources.FeedMapping.encode(m.feed_mapping, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedMappingResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedMappingResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedMappingResult} message MutateFeedMappingResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedMappingResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedMappingResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingResult} MutateFeedMappingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedMappingResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedMappingResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.feed_mapping = $root.google.ads.googleads.v11.resources.FeedMapping.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedMappingResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingResult} MutateFeedMappingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedMappingResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedMappingResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedMappingResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed_mapping != null && m.hasOwnProperty("feed_mapping")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.FeedMapping.verify(m.feed_mapping);
                                        if (e)
                                            return "feed_mapping." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedMappingResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedMappingResult} MutateFeedMappingResult
                             */
                            MutateFeedMappingResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedMappingResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedMappingResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed_mapping != null) {
                                    if (typeof d.feed_mapping !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedMappingResult.feed_mapping: object expected");
                                    m.feed_mapping = $root.google.ads.googleads.v11.resources.FeedMapping.fromObject(d.feed_mapping);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedMappingResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedMappingResult} m MutateFeedMappingResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedMappingResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.feed_mapping = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.feed_mapping != null && m.hasOwnProperty("feed_mapping")) {
                                    d.feed_mapping = $root.google.ads.googleads.v11.resources.FeedMapping.toObject(m.feed_mapping, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedMappingResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedMappingResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedMappingResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedMappingResult;
                        })();
    
                        services.FeedService = (function() {
    
                            /**
                             * Constructs a new FeedService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function FeedService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (FeedService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = FeedService;
    
                            /**
                             * Creates new FeedService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {FeedService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            FeedService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.FeedService|mutateFeeds}.
                             * @memberof google.ads.googleads.v11.services.FeedService
                             * @typedef MutateFeedsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateFeedsResponse} [response] MutateFeedsResponse
                             */
    
                            /**
                             * Calls MutateFeeds.
                             * @function mutateFeeds
                             * @memberof google.ads.googleads.v11.services.FeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedsRequest} request MutateFeedsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.FeedService.MutateFeedsCallback} callback Node-style callback called with the error, if any, and MutateFeedsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(FeedService.prototype.mutateFeeds = function mutateFeeds(request, callback) {
                                return this.rpcCall(mutateFeeds, $root.google.ads.googleads.v11.services.MutateFeedsRequest, $root.google.ads.googleads.v11.services.MutateFeedsResponse, request, callback);
                            }, "name", { value: "MutateFeeds" });
    
                            /**
                             * Calls MutateFeeds.
                             * @function mutateFeeds
                             * @memberof google.ads.googleads.v11.services.FeedService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateFeedsRequest} request MutateFeedsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateFeedsResponse>} Promise
                             * @variation 2
                             */
    
                            return FeedService;
                        })();
    
                        services.MutateFeedsRequest = (function() {
    
                            /**
                             * Properties of a MutateFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedsRequest
                             * @property {string|null} [customer_id] MutateFeedsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IFeedOperation>|null} [operations] MutateFeedsRequest operations
                             * @property {boolean|null} [partial_failure] MutateFeedsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateFeedsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateFeedsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateFeedsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedsRequest.
                             * @implements IMutateFeedsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedsRequest=} [p] Properties to set
                             */
                            function MutateFeedsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @instance
                             */
                            MutateFeedsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateFeedsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IFeedOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @instance
                             */
                            MutateFeedsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateFeedsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @instance
                             */
                            MutateFeedsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateFeedsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @instance
                             */
                            MutateFeedsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateFeedsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @instance
                             */
                            MutateFeedsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateFeedsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedsRequest} MutateFeedsRequest instance
                             */
                            MutateFeedsRequest.create = function create(properties) {
                                return new MutateFeedsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedsRequest} m MutateFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.FeedOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedsRequest} message MutateFeedsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedsRequest} MutateFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.FeedOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedsRequest} MutateFeedsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.FeedOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedsRequest} MutateFeedsRequest
                             */
                            MutateFeedsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.FeedOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedsRequest} m MutateFeedsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.FeedOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedsRequest;
                        })();
    
                        services.FeedOperation = (function() {
    
                            /**
                             * Properties of a FeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IFeedOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] FeedOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IFeed|null} [create] FeedOperation create
                             * @property {google.ads.googleads.v11.resources.IFeed|null} [update] FeedOperation update
                             * @property {string|null} [remove] FeedOperation remove
                             */
    
                            /**
                             * Constructs a new FeedOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FeedOperation.
                             * @implements IFeedOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IFeedOperation=} [p] Properties to set
                             */
                            function FeedOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FeedOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @instance
                             */
                            FeedOperation.prototype.update_mask = null;
    
                            /**
                             * FeedOperation create.
                             * @member {google.ads.googleads.v11.resources.IFeed|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @instance
                             */
                            FeedOperation.prototype.create = null;
    
                            /**
                             * FeedOperation update.
                             * @member {google.ads.googleads.v11.resources.IFeed|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @instance
                             */
                            FeedOperation.prototype.update = null;
    
                            /**
                             * FeedOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @instance
                             */
                            FeedOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * FeedOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @instance
                             */
                            Object.defineProperty(FeedOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new FeedOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.FeedOperation} FeedOperation instance
                             */
                            FeedOperation.create = function create(properties) {
                                return new FeedOperation(properties);
                            };
    
                            /**
                             * Encodes the specified FeedOperation message. Does not implicitly {@link google.ads.googleads.v11.services.FeedOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedOperation} m FeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.Feed.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Feed.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified FeedOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.FeedOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IFeedOperation} message FeedOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FeedOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FeedOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.FeedOperation} FeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.FeedOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.Feed.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.Feed.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FeedOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.FeedOperation} FeedOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FeedOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FeedOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FeedOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Feed.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Feed.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FeedOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.FeedOperation} FeedOperation
                             */
                            FeedOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.FeedOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.FeedOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.Feed.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.FeedOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Feed.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FeedOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.FeedOperation} m FeedOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FeedOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.Feed.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Feed.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FeedOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.FeedOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FeedOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FeedOperation;
                        })();
    
                        services.MutateFeedsResponse = (function() {
    
                            /**
                             * Properties of a MutateFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateFeedsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateFeedResult>|null} [results] MutateFeedsResponse results
                             */
    
                            /**
                             * Constructs a new MutateFeedsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedsResponse.
                             * @implements IMutateFeedsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedsResponse=} [p] Properties to set
                             */
                            function MutateFeedsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @instance
                             */
                            MutateFeedsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateFeedsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateFeedResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @instance
                             */
                            MutateFeedsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateFeedsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedsResponse} MutateFeedsResponse instance
                             */
                            MutateFeedsResponse.create = function create(properties) {
                                return new MutateFeedsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedsResponse} m MutateFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateFeedResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedsResponse} message MutateFeedsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedsResponse} MutateFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateFeedResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedsResponse} MutateFeedsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateFeedResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedsResponse} MutateFeedsResponse
                             */
                            MutateFeedsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateFeedsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateFeedResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedsResponse} m MutateFeedsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateFeedResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedsResponse;
                        })();
    
                        services.MutateFeedResult = (function() {
    
                            /**
                             * Properties of a MutateFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateFeedResult
                             * @property {string|null} [resource_name] MutateFeedResult resource_name
                             * @property {google.ads.googleads.v11.resources.IFeed|null} [feed] MutateFeedResult feed
                             */
    
                            /**
                             * Constructs a new MutateFeedResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateFeedResult.
                             * @implements IMutateFeedResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateFeedResult=} [p] Properties to set
                             */
                            function MutateFeedResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateFeedResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @instance
                             */
                            MutateFeedResult.prototype.resource_name = "";
    
                            /**
                             * MutateFeedResult feed.
                             * @member {google.ads.googleads.v11.resources.IFeed|null|undefined} feed
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @instance
                             */
                            MutateFeedResult.prototype.feed = null;
    
                            /**
                             * Creates a new MutateFeedResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateFeedResult} MutateFeedResult instance
                             */
                            MutateFeedResult.create = function create(properties) {
                                return new MutateFeedResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateFeedResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedResult} m MutateFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.feed != null && Object.hasOwnProperty.call(m, "feed"))
                                    $root.google.ads.googleads.v11.resources.Feed.encode(m.feed, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateFeedResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateFeedResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateFeedResult} message MutateFeedResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateFeedResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateFeedResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateFeedResult} MutateFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateFeedResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.feed = $root.google.ads.googleads.v11.resources.Feed.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateFeedResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateFeedResult} MutateFeedResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateFeedResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateFeedResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateFeedResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Feed.verify(m.feed);
                                        if (e)
                                            return "feed." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateFeedResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateFeedResult} MutateFeedResult
                             */
                            MutateFeedResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateFeedResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateFeedResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.feed != null) {
                                    if (typeof d.feed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateFeedResult.feed: object expected");
                                    m.feed = $root.google.ads.googleads.v11.resources.Feed.fromObject(d.feed);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateFeedResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateFeedResult} m MutateFeedResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateFeedResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.feed = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.feed != null && m.hasOwnProperty("feed")) {
                                    d.feed = $root.google.ads.googleads.v11.resources.Feed.toObject(m.feed, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateFeedResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateFeedResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateFeedResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateFeedResult;
                        })();
    
                        services.KeywordPlanAdGroupKeywordService = (function() {
    
                            /**
                             * Constructs a new KeywordPlanAdGroupKeywordService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanAdGroupKeywordService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function KeywordPlanAdGroupKeywordService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (KeywordPlanAdGroupKeywordService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = KeywordPlanAdGroupKeywordService;
    
                            /**
                             * Creates new KeywordPlanAdGroupKeywordService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {KeywordPlanAdGroupKeywordService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            KeywordPlanAdGroupKeywordService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordService|mutateKeywordPlanAdGroupKeywords}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordService
                             * @typedef MutateKeywordPlanAdGroupKeywordsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse} [response] MutateKeywordPlanAdGroupKeywordsResponse
                             */
    
                            /**
                             * Calls MutateKeywordPlanAdGroupKeywords.
                             * @function mutateKeywordPlanAdGroupKeywords
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsRequest} request MutateKeywordPlanAdGroupKeywordsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordService.MutateKeywordPlanAdGroupKeywordsCallback} callback Node-style callback called with the error, if any, and MutateKeywordPlanAdGroupKeywordsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanAdGroupKeywordService.prototype.mutateKeywordPlanAdGroupKeywords = function mutateKeywordPlanAdGroupKeywords(request, callback) {
                                return this.rpcCall(mutateKeywordPlanAdGroupKeywords, $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest, $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse, request, callback);
                            }, "name", { value: "MutateKeywordPlanAdGroupKeywords" });
    
                            /**
                             * Calls MutateKeywordPlanAdGroupKeywords.
                             * @function mutateKeywordPlanAdGroupKeywords
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsRequest} request MutateKeywordPlanAdGroupKeywordsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse>} Promise
                             * @variation 2
                             */
    
                            return KeywordPlanAdGroupKeywordService;
                        })();
    
                        services.MutateKeywordPlanAdGroupKeywordsRequest = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanAdGroupKeywordsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanAdGroupKeywordsRequest
                             * @property {string|null} [customer_id] MutateKeywordPlanAdGroupKeywordsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation>|null} [operations] MutateKeywordPlanAdGroupKeywordsRequest operations
                             * @property {boolean|null} [partial_failure] MutateKeywordPlanAdGroupKeywordsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateKeywordPlanAdGroupKeywordsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanAdGroupKeywordsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanAdGroupKeywordsRequest.
                             * @implements IMutateKeywordPlanAdGroupKeywordsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsRequest=} [p] Properties to set
                             */
                            function MutateKeywordPlanAdGroupKeywordsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanAdGroupKeywordsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateKeywordPlanAdGroupKeywordsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateKeywordPlanAdGroupKeywordsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateKeywordPlanAdGroupKeywordsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateKeywordPlanAdGroupKeywordsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest} MutateKeywordPlanAdGroupKeywordsRequest instance
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.create = function create(properties) {
                                return new MutateKeywordPlanAdGroupKeywordsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupKeywordsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsRequest} m MutateKeywordPlanAdGroupKeywordsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupKeywordsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsRequest} message MutateKeywordPlanAdGroupKeywordsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupKeywordsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest} MutateKeywordPlanAdGroupKeywordsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupKeywordsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest} MutateKeywordPlanAdGroupKeywordsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanAdGroupKeywordsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanAdGroupKeywordsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest} MutateKeywordPlanAdGroupKeywordsRequest
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanAdGroupKeywordsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest} m MutateKeywordPlanAdGroupKeywordsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanAdGroupKeywordsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanAdGroupKeywordsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanAdGroupKeywordsRequest;
                        })();
    
                        services.KeywordPlanAdGroupKeywordOperation = (function() {
    
                            /**
                             * Properties of a KeywordPlanAdGroupKeywordOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanAdGroupKeywordOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] KeywordPlanAdGroupKeywordOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword|null} [create] KeywordPlanAdGroupKeywordOperation create
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword|null} [update] KeywordPlanAdGroupKeywordOperation update
                             * @property {string|null} [remove] KeywordPlanAdGroupKeywordOperation remove
                             */
    
                            /**
                             * Constructs a new KeywordPlanAdGroupKeywordOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanAdGroupKeywordOperation.
                             * @implements IKeywordPlanAdGroupKeywordOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation=} [p] Properties to set
                             */
                            function KeywordPlanAdGroupKeywordOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanAdGroupKeywordOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @instance
                             */
                            KeywordPlanAdGroupKeywordOperation.prototype.update_mask = null;
    
                            /**
                             * KeywordPlanAdGroupKeywordOperation create.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @instance
                             */
                            KeywordPlanAdGroupKeywordOperation.prototype.create = null;
    
                            /**
                             * KeywordPlanAdGroupKeywordOperation update.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanAdGroupKeyword|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @instance
                             */
                            KeywordPlanAdGroupKeywordOperation.prototype.update = null;
    
                            /**
                             * KeywordPlanAdGroupKeywordOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @instance
                             */
                            KeywordPlanAdGroupKeywordOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanAdGroupKeywordOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupKeywordOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanAdGroupKeywordOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation} KeywordPlanAdGroupKeywordOperation instance
                             */
                            KeywordPlanAdGroupKeywordOperation.create = function create(properties) {
                                return new KeywordPlanAdGroupKeywordOperation(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupKeywordOperation message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation} m KeywordPlanAdGroupKeywordOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupKeywordOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupKeywordOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupKeywordOperation} message KeywordPlanAdGroupKeywordOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupKeywordOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupKeywordOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation} KeywordPlanAdGroupKeywordOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupKeywordOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupKeywordOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation} KeywordPlanAdGroupKeywordOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupKeywordOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAdGroupKeywordOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAdGroupKeywordOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAdGroupKeywordOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation} KeywordPlanAdGroupKeywordOperation
                             */
                            KeywordPlanAdGroupKeywordOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAdGroupKeywordOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation} m KeywordPlanAdGroupKeywordOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAdGroupKeywordOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroupKeyword.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanAdGroupKeywordOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupKeywordOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAdGroupKeywordOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanAdGroupKeywordOperation;
                        })();
    
                        services.MutateKeywordPlanAdGroupKeywordsResponse = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanAdGroupKeywordsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanAdGroupKeywordsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateKeywordPlanAdGroupKeywordsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult>|null} [results] MutateKeywordPlanAdGroupKeywordsResponse results
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanAdGroupKeywordsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanAdGroupKeywordsResponse.
                             * @implements IMutateKeywordPlanAdGroupKeywordsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsResponse=} [p] Properties to set
                             */
                            function MutateKeywordPlanAdGroupKeywordsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanAdGroupKeywordsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @instance
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateKeywordPlanAdGroupKeywordsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @instance
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateKeywordPlanAdGroupKeywordsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse} MutateKeywordPlanAdGroupKeywordsResponse instance
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.create = function create(properties) {
                                return new MutateKeywordPlanAdGroupKeywordsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupKeywordsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsResponse} m MutateKeywordPlanAdGroupKeywordsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupKeywordsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordsResponse} message MutateKeywordPlanAdGroupKeywordsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupKeywordsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse} MutateKeywordPlanAdGroupKeywordsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupKeywordsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse} MutateKeywordPlanAdGroupKeywordsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanAdGroupKeywordsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanAdGroupKeywordsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse} MutateKeywordPlanAdGroupKeywordsResponse
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanAdGroupKeywordsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse} m MutateKeywordPlanAdGroupKeywordsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanAdGroupKeywordsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanAdGroupKeywordsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanAdGroupKeywordsResponse;
                        })();
    
                        services.MutateKeywordPlanAdGroupKeywordResult = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanAdGroupKeywordResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanAdGroupKeywordResult
                             * @property {string|null} [resource_name] MutateKeywordPlanAdGroupKeywordResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanAdGroupKeywordResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanAdGroupKeywordResult.
                             * @implements IMutateKeywordPlanAdGroupKeywordResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult=} [p] Properties to set
                             */
                            function MutateKeywordPlanAdGroupKeywordResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanAdGroupKeywordResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @instance
                             */
                            MutateKeywordPlanAdGroupKeywordResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateKeywordPlanAdGroupKeywordResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult} MutateKeywordPlanAdGroupKeywordResult instance
                             */
                            MutateKeywordPlanAdGroupKeywordResult.create = function create(properties) {
                                return new MutateKeywordPlanAdGroupKeywordResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupKeywordResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult} m MutateKeywordPlanAdGroupKeywordResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupKeywordResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupKeywordResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupKeywordResult} message MutateKeywordPlanAdGroupKeywordResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupKeywordResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupKeywordResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult} MutateKeywordPlanAdGroupKeywordResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupKeywordResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupKeywordResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult} MutateKeywordPlanAdGroupKeywordResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupKeywordResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanAdGroupKeywordResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanAdGroupKeywordResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanAdGroupKeywordResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult} MutateKeywordPlanAdGroupKeywordResult
                             */
                            MutateKeywordPlanAdGroupKeywordResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanAdGroupKeywordResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult} m MutateKeywordPlanAdGroupKeywordResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanAdGroupKeywordResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanAdGroupKeywordResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupKeywordResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanAdGroupKeywordResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanAdGroupKeywordResult;
                        })();
    
                        services.KeywordPlanAdGroupService = (function() {
    
                            /**
                             * Constructs a new KeywordPlanAdGroupService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanAdGroupService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function KeywordPlanAdGroupService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (KeywordPlanAdGroupService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = KeywordPlanAdGroupService;
    
                            /**
                             * Creates new KeywordPlanAdGroupService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {KeywordPlanAdGroupService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            KeywordPlanAdGroupService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanAdGroupService|mutateKeywordPlanAdGroups}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupService
                             * @typedef MutateKeywordPlanAdGroupsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse} [response] MutateKeywordPlanAdGroupsResponse
                             */
    
                            /**
                             * Calls MutateKeywordPlanAdGroups.
                             * @function mutateKeywordPlanAdGroups
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsRequest} request MutateKeywordPlanAdGroupsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanAdGroupService.MutateKeywordPlanAdGroupsCallback} callback Node-style callback called with the error, if any, and MutateKeywordPlanAdGroupsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanAdGroupService.prototype.mutateKeywordPlanAdGroups = function mutateKeywordPlanAdGroups(request, callback) {
                                return this.rpcCall(mutateKeywordPlanAdGroups, $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest, $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse, request, callback);
                            }, "name", { value: "MutateKeywordPlanAdGroups" });
    
                            /**
                             * Calls MutateKeywordPlanAdGroups.
                             * @function mutateKeywordPlanAdGroups
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsRequest} request MutateKeywordPlanAdGroupsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse>} Promise
                             * @variation 2
                             */
    
                            return KeywordPlanAdGroupService;
                        })();
    
                        services.MutateKeywordPlanAdGroupsRequest = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanAdGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanAdGroupsRequest
                             * @property {string|null} [customer_id] MutateKeywordPlanAdGroupsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation>|null} [operations] MutateKeywordPlanAdGroupsRequest operations
                             * @property {boolean|null} [partial_failure] MutateKeywordPlanAdGroupsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateKeywordPlanAdGroupsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanAdGroupsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanAdGroupsRequest.
                             * @implements IMutateKeywordPlanAdGroupsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsRequest=} [p] Properties to set
                             */
                            function MutateKeywordPlanAdGroupsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanAdGroupsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateKeywordPlanAdGroupsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateKeywordPlanAdGroupsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateKeywordPlanAdGroupsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @instance
                             */
                            MutateKeywordPlanAdGroupsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateKeywordPlanAdGroupsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest} MutateKeywordPlanAdGroupsRequest instance
                             */
                            MutateKeywordPlanAdGroupsRequest.create = function create(properties) {
                                return new MutateKeywordPlanAdGroupsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsRequest} m MutateKeywordPlanAdGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsRequest} message MutateKeywordPlanAdGroupsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest} MutateKeywordPlanAdGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest} MutateKeywordPlanAdGroupsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanAdGroupsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanAdGroupsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanAdGroupsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest} MutateKeywordPlanAdGroupsRequest
                             */
                            MutateKeywordPlanAdGroupsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanAdGroupsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest} m MutateKeywordPlanAdGroupsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanAdGroupsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanAdGroupsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanAdGroupsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanAdGroupsRequest;
                        })();
    
                        services.KeywordPlanAdGroupOperation = (function() {
    
                            /**
                             * Properties of a KeywordPlanAdGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanAdGroupOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] KeywordPlanAdGroupOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanAdGroup|null} [create] KeywordPlanAdGroupOperation create
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanAdGroup|null} [update] KeywordPlanAdGroupOperation update
                             * @property {string|null} [remove] KeywordPlanAdGroupOperation remove
                             */
    
                            /**
                             * Constructs a new KeywordPlanAdGroupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanAdGroupOperation.
                             * @implements IKeywordPlanAdGroupOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation=} [p] Properties to set
                             */
                            function KeywordPlanAdGroupOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanAdGroupOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @instance
                             */
                            KeywordPlanAdGroupOperation.prototype.update_mask = null;
    
                            /**
                             * KeywordPlanAdGroupOperation create.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanAdGroup|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @instance
                             */
                            KeywordPlanAdGroupOperation.prototype.create = null;
    
                            /**
                             * KeywordPlanAdGroupOperation update.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanAdGroup|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @instance
                             */
                            KeywordPlanAdGroupOperation.prototype.update = null;
    
                            /**
                             * KeywordPlanAdGroupOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @instance
                             */
                            KeywordPlanAdGroupOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanAdGroupOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanAdGroupOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupOperation} KeywordPlanAdGroupOperation instance
                             */
                            KeywordPlanAdGroupOperation.create = function create(properties) {
                                return new KeywordPlanAdGroupOperation(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupOperation message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation} m KeywordPlanAdGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupOperation} message KeywordPlanAdGroupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupOperation} KeywordPlanAdGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupOperation} KeywordPlanAdGroupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAdGroupOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAdGroupOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAdGroupOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupOperation} KeywordPlanAdGroupOperation
                             */
                            KeywordPlanAdGroupOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanAdGroupOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanAdGroupOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAdGroupOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanAdGroupOperation} m KeywordPlanAdGroupOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAdGroupOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.KeywordPlanAdGroup.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanAdGroupOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAdGroupOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanAdGroupOperation;
                        })();
    
                        services.MutateKeywordPlanAdGroupsResponse = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanAdGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanAdGroupsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateKeywordPlanAdGroupsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult>|null} [results] MutateKeywordPlanAdGroupsResponse results
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanAdGroupsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanAdGroupsResponse.
                             * @implements IMutateKeywordPlanAdGroupsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsResponse=} [p] Properties to set
                             */
                            function MutateKeywordPlanAdGroupsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanAdGroupsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @instance
                             */
                            MutateKeywordPlanAdGroupsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateKeywordPlanAdGroupsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @instance
                             */
                            MutateKeywordPlanAdGroupsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateKeywordPlanAdGroupsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse} MutateKeywordPlanAdGroupsResponse instance
                             */
                            MutateKeywordPlanAdGroupsResponse.create = function create(properties) {
                                return new MutateKeywordPlanAdGroupsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsResponse} m MutateKeywordPlanAdGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupsResponse} message MutateKeywordPlanAdGroupsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse} MutateKeywordPlanAdGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse} MutateKeywordPlanAdGroupsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanAdGroupsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanAdGroupsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanAdGroupsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse} MutateKeywordPlanAdGroupsResponse
                             */
                            MutateKeywordPlanAdGroupsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanAdGroupsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse} m MutateKeywordPlanAdGroupsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanAdGroupsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanAdGroupsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanAdGroupsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanAdGroupsResponse;
                        })();
    
                        services.MutateKeywordPlanAdGroupResult = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanAdGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanAdGroupResult
                             * @property {string|null} [resource_name] MutateKeywordPlanAdGroupResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanAdGroupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanAdGroupResult.
                             * @implements IMutateKeywordPlanAdGroupResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult=} [p] Properties to set
                             */
                            function MutateKeywordPlanAdGroupResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanAdGroupResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @instance
                             */
                            MutateKeywordPlanAdGroupResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateKeywordPlanAdGroupResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult} MutateKeywordPlanAdGroupResult instance
                             */
                            MutateKeywordPlanAdGroupResult.create = function create(properties) {
                                return new MutateKeywordPlanAdGroupResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult} m MutateKeywordPlanAdGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanAdGroupResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanAdGroupResult} message MutateKeywordPlanAdGroupResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanAdGroupResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult} MutateKeywordPlanAdGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanAdGroupResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult} MutateKeywordPlanAdGroupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanAdGroupResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanAdGroupResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanAdGroupResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanAdGroupResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult} MutateKeywordPlanAdGroupResult
                             */
                            MutateKeywordPlanAdGroupResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanAdGroupResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult} m MutateKeywordPlanAdGroupResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanAdGroupResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanAdGroupResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanAdGroupResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanAdGroupResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanAdGroupResult;
                        })();
    
                        services.KeywordPlanCampaignKeywordService = (function() {
    
                            /**
                             * Constructs a new KeywordPlanCampaignKeywordService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanCampaignKeywordService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function KeywordPlanCampaignKeywordService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (KeywordPlanCampaignKeywordService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = KeywordPlanCampaignKeywordService;
    
                            /**
                             * Creates new KeywordPlanCampaignKeywordService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {KeywordPlanCampaignKeywordService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            KeywordPlanCampaignKeywordService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanCampaignKeywordService|mutateKeywordPlanCampaignKeywords}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordService
                             * @typedef MutateKeywordPlanCampaignKeywordsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse} [response] MutateKeywordPlanCampaignKeywordsResponse
                             */
    
                            /**
                             * Calls MutateKeywordPlanCampaignKeywords.
                             * @function mutateKeywordPlanCampaignKeywords
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsRequest} request MutateKeywordPlanCampaignKeywordsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanCampaignKeywordService.MutateKeywordPlanCampaignKeywordsCallback} callback Node-style callback called with the error, if any, and MutateKeywordPlanCampaignKeywordsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanCampaignKeywordService.prototype.mutateKeywordPlanCampaignKeywords = function mutateKeywordPlanCampaignKeywords(request, callback) {
                                return this.rpcCall(mutateKeywordPlanCampaignKeywords, $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest, $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse, request, callback);
                            }, "name", { value: "MutateKeywordPlanCampaignKeywords" });
    
                            /**
                             * Calls MutateKeywordPlanCampaignKeywords.
                             * @function mutateKeywordPlanCampaignKeywords
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsRequest} request MutateKeywordPlanCampaignKeywordsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse>} Promise
                             * @variation 2
                             */
    
                            return KeywordPlanCampaignKeywordService;
                        })();
    
                        services.MutateKeywordPlanCampaignKeywordsRequest = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanCampaignKeywordsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanCampaignKeywordsRequest
                             * @property {string|null} [customer_id] MutateKeywordPlanCampaignKeywordsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation>|null} [operations] MutateKeywordPlanCampaignKeywordsRequest operations
                             * @property {boolean|null} [partial_failure] MutateKeywordPlanCampaignKeywordsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateKeywordPlanCampaignKeywordsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanCampaignKeywordsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanCampaignKeywordsRequest.
                             * @implements IMutateKeywordPlanCampaignKeywordsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsRequest=} [p] Properties to set
                             */
                            function MutateKeywordPlanCampaignKeywordsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanCampaignKeywordsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateKeywordPlanCampaignKeywordsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateKeywordPlanCampaignKeywordsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateKeywordPlanCampaignKeywordsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateKeywordPlanCampaignKeywordsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest} MutateKeywordPlanCampaignKeywordsRequest instance
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.create = function create(properties) {
                                return new MutateKeywordPlanCampaignKeywordsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignKeywordsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsRequest} m MutateKeywordPlanCampaignKeywordsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignKeywordsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsRequest} message MutateKeywordPlanCampaignKeywordsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignKeywordsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest} MutateKeywordPlanCampaignKeywordsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignKeywordsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest} MutateKeywordPlanCampaignKeywordsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanCampaignKeywordsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanCampaignKeywordsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest} MutateKeywordPlanCampaignKeywordsRequest
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanCampaignKeywordsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest} m MutateKeywordPlanCampaignKeywordsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanCampaignKeywordsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanCampaignKeywordsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanCampaignKeywordsRequest;
                        })();
    
                        services.KeywordPlanCampaignKeywordOperation = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaignKeywordOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanCampaignKeywordOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] KeywordPlanCampaignKeywordOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword|null} [create] KeywordPlanCampaignKeywordOperation create
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword|null} [update] KeywordPlanCampaignKeywordOperation update
                             * @property {string|null} [remove] KeywordPlanCampaignKeywordOperation remove
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaignKeywordOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanCampaignKeywordOperation.
                             * @implements IKeywordPlanCampaignKeywordOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation=} [p] Properties to set
                             */
                            function KeywordPlanCampaignKeywordOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanCampaignKeywordOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @instance
                             */
                            KeywordPlanCampaignKeywordOperation.prototype.update_mask = null;
    
                            /**
                             * KeywordPlanCampaignKeywordOperation create.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @instance
                             */
                            KeywordPlanCampaignKeywordOperation.prototype.create = null;
    
                            /**
                             * KeywordPlanCampaignKeywordOperation update.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanCampaignKeyword|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @instance
                             */
                            KeywordPlanCampaignKeywordOperation.prototype.update = null;
    
                            /**
                             * KeywordPlanCampaignKeywordOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @instance
                             */
                            KeywordPlanCampaignKeywordOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanCampaignKeywordOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignKeywordOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanCampaignKeywordOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation} KeywordPlanCampaignKeywordOperation instance
                             */
                            KeywordPlanCampaignKeywordOperation.create = function create(properties) {
                                return new KeywordPlanCampaignKeywordOperation(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignKeywordOperation message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation} m KeywordPlanCampaignKeywordOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignKeywordOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignKeywordOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignKeywordOperation} message KeywordPlanCampaignKeywordOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignKeywordOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignKeywordOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation} KeywordPlanCampaignKeywordOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignKeywordOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignKeywordOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation} KeywordPlanCampaignKeywordOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignKeywordOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaignKeywordOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaignKeywordOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaignKeywordOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation} KeywordPlanCampaignKeywordOperation
                             */
                            KeywordPlanCampaignKeywordOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaignKeywordOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation} m KeywordPlanCampaignKeywordOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaignKeywordOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.KeywordPlanCampaignKeyword.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanCampaignKeywordOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignKeywordOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaignKeywordOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanCampaignKeywordOperation;
                        })();
    
                        services.MutateKeywordPlanCampaignKeywordsResponse = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanCampaignKeywordsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanCampaignKeywordsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateKeywordPlanCampaignKeywordsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult>|null} [results] MutateKeywordPlanCampaignKeywordsResponse results
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanCampaignKeywordsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanCampaignKeywordsResponse.
                             * @implements IMutateKeywordPlanCampaignKeywordsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsResponse=} [p] Properties to set
                             */
                            function MutateKeywordPlanCampaignKeywordsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanCampaignKeywordsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @instance
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateKeywordPlanCampaignKeywordsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @instance
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateKeywordPlanCampaignKeywordsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse} MutateKeywordPlanCampaignKeywordsResponse instance
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.create = function create(properties) {
                                return new MutateKeywordPlanCampaignKeywordsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignKeywordsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsResponse} m MutateKeywordPlanCampaignKeywordsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignKeywordsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordsResponse} message MutateKeywordPlanCampaignKeywordsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignKeywordsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse} MutateKeywordPlanCampaignKeywordsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignKeywordsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse} MutateKeywordPlanCampaignKeywordsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanCampaignKeywordsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanCampaignKeywordsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse} MutateKeywordPlanCampaignKeywordsResponse
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanCampaignKeywordsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse} m MutateKeywordPlanCampaignKeywordsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanCampaignKeywordsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanCampaignKeywordsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanCampaignKeywordsResponse;
                        })();
    
                        services.MutateKeywordPlanCampaignKeywordResult = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanCampaignKeywordResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanCampaignKeywordResult
                             * @property {string|null} [resource_name] MutateKeywordPlanCampaignKeywordResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanCampaignKeywordResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanCampaignKeywordResult.
                             * @implements IMutateKeywordPlanCampaignKeywordResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult=} [p] Properties to set
                             */
                            function MutateKeywordPlanCampaignKeywordResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanCampaignKeywordResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @instance
                             */
                            MutateKeywordPlanCampaignKeywordResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateKeywordPlanCampaignKeywordResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult} MutateKeywordPlanCampaignKeywordResult instance
                             */
                            MutateKeywordPlanCampaignKeywordResult.create = function create(properties) {
                                return new MutateKeywordPlanCampaignKeywordResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignKeywordResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult} m MutateKeywordPlanCampaignKeywordResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignKeywordResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignKeywordResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignKeywordResult} message MutateKeywordPlanCampaignKeywordResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignKeywordResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignKeywordResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult} MutateKeywordPlanCampaignKeywordResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignKeywordResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignKeywordResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult} MutateKeywordPlanCampaignKeywordResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignKeywordResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanCampaignKeywordResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanCampaignKeywordResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanCampaignKeywordResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult} MutateKeywordPlanCampaignKeywordResult
                             */
                            MutateKeywordPlanCampaignKeywordResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanCampaignKeywordResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult} m MutateKeywordPlanCampaignKeywordResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanCampaignKeywordResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanCampaignKeywordResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignKeywordResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanCampaignKeywordResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanCampaignKeywordResult;
                        })();
    
                        services.KeywordPlanCampaignService = (function() {
    
                            /**
                             * Constructs a new KeywordPlanCampaignService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanCampaignService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function KeywordPlanCampaignService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (KeywordPlanCampaignService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = KeywordPlanCampaignService;
    
                            /**
                             * Creates new KeywordPlanCampaignService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {KeywordPlanCampaignService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            KeywordPlanCampaignService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanCampaignService|mutateKeywordPlanCampaigns}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignService
                             * @typedef MutateKeywordPlanCampaignsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse} [response] MutateKeywordPlanCampaignsResponse
                             */
    
                            /**
                             * Calls MutateKeywordPlanCampaigns.
                             * @function mutateKeywordPlanCampaigns
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsRequest} request MutateKeywordPlanCampaignsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanCampaignService.MutateKeywordPlanCampaignsCallback} callback Node-style callback called with the error, if any, and MutateKeywordPlanCampaignsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanCampaignService.prototype.mutateKeywordPlanCampaigns = function mutateKeywordPlanCampaigns(request, callback) {
                                return this.rpcCall(mutateKeywordPlanCampaigns, $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest, $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse, request, callback);
                            }, "name", { value: "MutateKeywordPlanCampaigns" });
    
                            /**
                             * Calls MutateKeywordPlanCampaigns.
                             * @function mutateKeywordPlanCampaigns
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsRequest} request MutateKeywordPlanCampaignsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse>} Promise
                             * @variation 2
                             */
    
                            return KeywordPlanCampaignService;
                        })();
    
                        services.MutateKeywordPlanCampaignsRequest = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanCampaignsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanCampaignsRequest
                             * @property {string|null} [customer_id] MutateKeywordPlanCampaignsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignOperation>|null} [operations] MutateKeywordPlanCampaignsRequest operations
                             * @property {boolean|null} [partial_failure] MutateKeywordPlanCampaignsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateKeywordPlanCampaignsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanCampaignsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanCampaignsRequest.
                             * @implements IMutateKeywordPlanCampaignsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsRequest=} [p] Properties to set
                             */
                            function MutateKeywordPlanCampaignsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanCampaignsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateKeywordPlanCampaignsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateKeywordPlanCampaignsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateKeywordPlanCampaignsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @instance
                             */
                            MutateKeywordPlanCampaignsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateKeywordPlanCampaignsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest} MutateKeywordPlanCampaignsRequest instance
                             */
                            MutateKeywordPlanCampaignsRequest.create = function create(properties) {
                                return new MutateKeywordPlanCampaignsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsRequest} m MutateKeywordPlanCampaignsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsRequest} message MutateKeywordPlanCampaignsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest} MutateKeywordPlanCampaignsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest} MutateKeywordPlanCampaignsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanCampaignsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanCampaignsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanCampaignsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest} MutateKeywordPlanCampaignsRequest
                             */
                            MutateKeywordPlanCampaignsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanCampaignsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest} m MutateKeywordPlanCampaignsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanCampaignsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanCampaignsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanCampaignsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanCampaignsRequest;
                        })();
    
                        services.KeywordPlanCampaignOperation = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaignOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanCampaignOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] KeywordPlanCampaignOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanCampaign|null} [create] KeywordPlanCampaignOperation create
                             * @property {google.ads.googleads.v11.resources.IKeywordPlanCampaign|null} [update] KeywordPlanCampaignOperation update
                             * @property {string|null} [remove] KeywordPlanCampaignOperation remove
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaignOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanCampaignOperation.
                             * @implements IKeywordPlanCampaignOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignOperation=} [p] Properties to set
                             */
                            function KeywordPlanCampaignOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanCampaignOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @instance
                             */
                            KeywordPlanCampaignOperation.prototype.update_mask = null;
    
                            /**
                             * KeywordPlanCampaignOperation create.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanCampaign|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @instance
                             */
                            KeywordPlanCampaignOperation.prototype.create = null;
    
                            /**
                             * KeywordPlanCampaignOperation update.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlanCampaign|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @instance
                             */
                            KeywordPlanCampaignOperation.prototype.update = null;
    
                            /**
                             * KeywordPlanCampaignOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @instance
                             */
                            KeywordPlanCampaignOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanCampaignOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanCampaignOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignOperation} KeywordPlanCampaignOperation instance
                             */
                            KeywordPlanCampaignOperation.create = function create(properties) {
                                return new KeywordPlanCampaignOperation(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignOperation message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignOperation} m KeywordPlanCampaignOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignOperation} message KeywordPlanCampaignOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignOperation} KeywordPlanCampaignOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignOperation} KeywordPlanCampaignOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaignOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaignOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaignOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignOperation} KeywordPlanCampaignOperation
                             */
                            KeywordPlanCampaignOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaignOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanCampaignOperation} m KeywordPlanCampaignOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaignOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.KeywordPlanCampaign.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanCampaignOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaignOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanCampaignOperation;
                        })();
    
                        services.MutateKeywordPlanCampaignsResponse = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanCampaignsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanCampaignsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateKeywordPlanCampaignsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult>|null} [results] MutateKeywordPlanCampaignsResponse results
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanCampaignsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanCampaignsResponse.
                             * @implements IMutateKeywordPlanCampaignsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsResponse=} [p] Properties to set
                             */
                            function MutateKeywordPlanCampaignsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanCampaignsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @instance
                             */
                            MutateKeywordPlanCampaignsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateKeywordPlanCampaignsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @instance
                             */
                            MutateKeywordPlanCampaignsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateKeywordPlanCampaignsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse} MutateKeywordPlanCampaignsResponse instance
                             */
                            MutateKeywordPlanCampaignsResponse.create = function create(properties) {
                                return new MutateKeywordPlanCampaignsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsResponse} m MutateKeywordPlanCampaignsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignsResponse} message MutateKeywordPlanCampaignsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse} MutateKeywordPlanCampaignsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse} MutateKeywordPlanCampaignsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanCampaignsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanCampaignsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanCampaignsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse} MutateKeywordPlanCampaignsResponse
                             */
                            MutateKeywordPlanCampaignsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanCampaignsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse} m MutateKeywordPlanCampaignsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanCampaignsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanCampaignsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanCampaignsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanCampaignsResponse;
                        })();
    
                        services.MutateKeywordPlanCampaignResult = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlanCampaignResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlanCampaignResult
                             * @property {string|null} [resource_name] MutateKeywordPlanCampaignResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlanCampaignResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlanCampaignResult.
                             * @implements IMutateKeywordPlanCampaignResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult=} [p] Properties to set
                             */
                            function MutateKeywordPlanCampaignResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlanCampaignResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @instance
                             */
                            MutateKeywordPlanCampaignResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateKeywordPlanCampaignResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult} MutateKeywordPlanCampaignResult instance
                             */
                            MutateKeywordPlanCampaignResult.create = function create(properties) {
                                return new MutateKeywordPlanCampaignResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult} m MutateKeywordPlanCampaignResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlanCampaignResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlanCampaignResult} message MutateKeywordPlanCampaignResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlanCampaignResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult} MutateKeywordPlanCampaignResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlanCampaignResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult} MutateKeywordPlanCampaignResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlanCampaignResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlanCampaignResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlanCampaignResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlanCampaignResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult} MutateKeywordPlanCampaignResult
                             */
                            MutateKeywordPlanCampaignResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlanCampaignResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult} m MutateKeywordPlanCampaignResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlanCampaignResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlanCampaignResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlanCampaignResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlanCampaignResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlanCampaignResult;
                        })();
    
                        services.KeywordPlanService = (function() {
    
                            /**
                             * Constructs a new KeywordPlanService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function KeywordPlanService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (KeywordPlanService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = KeywordPlanService;
    
                            /**
                             * Creates new KeywordPlanService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {KeywordPlanService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            KeywordPlanService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanService|mutateKeywordPlans}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @typedef MutateKeywordPlansCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlansResponse} [response] MutateKeywordPlansResponse
                             */
    
                            /**
                             * Calls MutateKeywordPlans.
                             * @function mutateKeywordPlans
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansRequest} request MutateKeywordPlansRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanService.MutateKeywordPlansCallback} callback Node-style callback called with the error, if any, and MutateKeywordPlansResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanService.prototype.mutateKeywordPlans = function mutateKeywordPlans(request, callback) {
                                return this.rpcCall(mutateKeywordPlans, $root.google.ads.googleads.v11.services.MutateKeywordPlansRequest, $root.google.ads.googleads.v11.services.MutateKeywordPlansResponse, request, callback);
                            }, "name", { value: "MutateKeywordPlans" });
    
                            /**
                             * Calls MutateKeywordPlans.
                             * @function mutateKeywordPlans
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansRequest} request MutateKeywordPlansRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateKeywordPlansResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanService|generateForecastCurve}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @typedef GenerateForecastCurveCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateForecastCurveResponse} [response] GenerateForecastCurveResponse
                             */
    
                            /**
                             * Calls GenerateForecastCurve.
                             * @function generateForecastCurve
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveRequest} request GenerateForecastCurveRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanService.GenerateForecastCurveCallback} callback Node-style callback called with the error, if any, and GenerateForecastCurveResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanService.prototype.generateForecastCurve = function generateForecastCurve(request, callback) {
                                return this.rpcCall(generateForecastCurve, $root.google.ads.googleads.v11.services.GenerateForecastCurveRequest, $root.google.ads.googleads.v11.services.GenerateForecastCurveResponse, request, callback);
                            }, "name", { value: "GenerateForecastCurve" });
    
                            /**
                             * Calls GenerateForecastCurve.
                             * @function generateForecastCurve
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveRequest} request GenerateForecastCurveRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateForecastCurveResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanService|generateForecastTimeSeries}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @typedef GenerateForecastTimeSeriesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse} [response] GenerateForecastTimeSeriesResponse
                             */
    
                            /**
                             * Calls GenerateForecastTimeSeries.
                             * @function generateForecastTimeSeries
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesRequest} request GenerateForecastTimeSeriesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanService.GenerateForecastTimeSeriesCallback} callback Node-style callback called with the error, if any, and GenerateForecastTimeSeriesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanService.prototype.generateForecastTimeSeries = function generateForecastTimeSeries(request, callback) {
                                return this.rpcCall(generateForecastTimeSeries, $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest, $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse, request, callback);
                            }, "name", { value: "GenerateForecastTimeSeries" });
    
                            /**
                             * Calls GenerateForecastTimeSeries.
                             * @function generateForecastTimeSeries
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesRequest} request GenerateForecastTimeSeriesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanService|generateForecastMetrics}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @typedef GenerateForecastMetricsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateForecastMetricsResponse} [response] GenerateForecastMetricsResponse
                             */
    
                            /**
                             * Calls GenerateForecastMetrics.
                             * @function generateForecastMetrics
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsRequest} request GenerateForecastMetricsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanService.GenerateForecastMetricsCallback} callback Node-style callback called with the error, if any, and GenerateForecastMetricsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanService.prototype.generateForecastMetrics = function generateForecastMetrics(request, callback) {
                                return this.rpcCall(generateForecastMetrics, $root.google.ads.googleads.v11.services.GenerateForecastMetricsRequest, $root.google.ads.googleads.v11.services.GenerateForecastMetricsResponse, request, callback);
                            }, "name", { value: "GenerateForecastMetrics" });
    
                            /**
                             * Calls GenerateForecastMetrics.
                             * @function generateForecastMetrics
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsRequest} request GenerateForecastMetricsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateForecastMetricsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanService|generateHistoricalMetrics}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @typedef GenerateHistoricalMetricsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse} [response] GenerateHistoricalMetricsResponse
                             */
    
                            /**
                             * Calls GenerateHistoricalMetrics.
                             * @function generateHistoricalMetrics
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsRequest} request GenerateHistoricalMetricsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanService.GenerateHistoricalMetricsCallback} callback Node-style callback called with the error, if any, and GenerateHistoricalMetricsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanService.prototype.generateHistoricalMetrics = function generateHistoricalMetrics(request, callback) {
                                return this.rpcCall(generateHistoricalMetrics, $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest, $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse, request, callback);
                            }, "name", { value: "GenerateHistoricalMetrics" });
    
                            /**
                             * Calls GenerateHistoricalMetrics.
                             * @function generateHistoricalMetrics
                             * @memberof google.ads.googleads.v11.services.KeywordPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsRequest} request GenerateHistoricalMetricsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse>} Promise
                             * @variation 2
                             */
    
                            return KeywordPlanService;
                        })();
    
                        services.MutateKeywordPlansRequest = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlansRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlansRequest
                             * @property {string|null} [customer_id] MutateKeywordPlansRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanOperation>|null} [operations] MutateKeywordPlansRequest operations
                             * @property {boolean|null} [partial_failure] MutateKeywordPlansRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateKeywordPlansRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlansRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlansRequest.
                             * @implements IMutateKeywordPlansRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansRequest=} [p] Properties to set
                             */
                            function MutateKeywordPlansRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlansRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @instance
                             */
                            MutateKeywordPlansRequest.prototype.customer_id = "";
    
                            /**
                             * MutateKeywordPlansRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @instance
                             */
                            MutateKeywordPlansRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateKeywordPlansRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @instance
                             */
                            MutateKeywordPlansRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateKeywordPlansRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @instance
                             */
                            MutateKeywordPlansRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateKeywordPlansRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansRequest} MutateKeywordPlansRequest instance
                             */
                            MutateKeywordPlansRequest.create = function create(properties) {
                                return new MutateKeywordPlansRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlansRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlansRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansRequest} m MutateKeywordPlansRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlansRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlansRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlansRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansRequest} message MutateKeywordPlansRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlansRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlansRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansRequest} MutateKeywordPlansRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlansRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlansRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.KeywordPlanOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlansRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansRequest} MutateKeywordPlansRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlansRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlansRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlansRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlansRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansRequest} MutateKeywordPlansRequest
                             */
                            MutateKeywordPlansRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlansRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlansRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlansRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlansRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.KeywordPlanOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlansRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlansRequest} m MutateKeywordPlansRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlansRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.KeywordPlanOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlansRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlansRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlansRequest;
                        })();
    
                        services.KeywordPlanOperation = (function() {
    
                            /**
                             * Properties of a KeywordPlanOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] KeywordPlanOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IKeywordPlan|null} [create] KeywordPlanOperation create
                             * @property {google.ads.googleads.v11.resources.IKeywordPlan|null} [update] KeywordPlanOperation update
                             * @property {string|null} [remove] KeywordPlanOperation remove
                             */
    
                            /**
                             * Constructs a new KeywordPlanOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanOperation.
                             * @implements IKeywordPlanOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanOperation=} [p] Properties to set
                             */
                            function KeywordPlanOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @instance
                             */
                            KeywordPlanOperation.prototype.update_mask = null;
    
                            /**
                             * KeywordPlanOperation create.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlan|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @instance
                             */
                            KeywordPlanOperation.prototype.create = null;
    
                            /**
                             * KeywordPlanOperation update.
                             * @member {google.ads.googleads.v11.resources.IKeywordPlan|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @instance
                             */
                            KeywordPlanOperation.prototype.update = null;
    
                            /**
                             * KeywordPlanOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @instance
                             */
                            KeywordPlanOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanOperation} KeywordPlanOperation instance
                             */
                            KeywordPlanOperation.create = function create(properties) {
                                return new KeywordPlanOperation(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanOperation message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanOperation} m KeywordPlanOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlan.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.KeywordPlan.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanOperation} message KeywordPlanOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanOperation} KeywordPlanOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.KeywordPlan.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.KeywordPlan.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanOperation} KeywordPlanOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlan.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.KeywordPlan.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanOperation} KeywordPlanOperation
                             */
                            KeywordPlanOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.KeywordPlan.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.KeywordPlan.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanOperation} m KeywordPlanOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.KeywordPlan.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.KeywordPlan.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanOperation;
                        })();
    
                        services.MutateKeywordPlansResponse = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlansResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlansResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateKeywordPlansResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateKeywordPlansResult>|null} [results] MutateKeywordPlansResponse results
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlansResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlansResponse.
                             * @implements IMutateKeywordPlansResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResponse=} [p] Properties to set
                             */
                            function MutateKeywordPlansResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlansResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @instance
                             */
                            MutateKeywordPlansResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateKeywordPlansResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateKeywordPlansResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @instance
                             */
                            MutateKeywordPlansResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateKeywordPlansResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResponse} MutateKeywordPlansResponse instance
                             */
                            MutateKeywordPlansResponse.create = function create(properties) {
                                return new MutateKeywordPlansResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlansResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlansResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResponse} m MutateKeywordPlansResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlansResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlansResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlansResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResponse} message MutateKeywordPlansResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlansResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlansResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResponse} MutateKeywordPlansResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlansResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlansResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateKeywordPlansResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlansResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResponse} MutateKeywordPlansResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlansResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlansResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlansResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlansResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResponse} MutateKeywordPlansResponse
                             */
                            MutateKeywordPlansResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlansResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlansResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlansResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlansResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateKeywordPlansResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlansResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlansResponse} m MutateKeywordPlansResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlansResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateKeywordPlansResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlansResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlansResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlansResponse;
                        })();
    
                        services.MutateKeywordPlansResult = (function() {
    
                            /**
                             * Properties of a MutateKeywordPlansResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateKeywordPlansResult
                             * @property {string|null} [resource_name] MutateKeywordPlansResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateKeywordPlansResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateKeywordPlansResult.
                             * @implements IMutateKeywordPlansResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResult=} [p] Properties to set
                             */
                            function MutateKeywordPlansResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateKeywordPlansResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @instance
                             */
                            MutateKeywordPlansResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateKeywordPlansResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResult} MutateKeywordPlansResult instance
                             */
                            MutateKeywordPlansResult.create = function create(properties) {
                                return new MutateKeywordPlansResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlansResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlansResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResult} m MutateKeywordPlansResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlansResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateKeywordPlansResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateKeywordPlansResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateKeywordPlansResult} message MutateKeywordPlansResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateKeywordPlansResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateKeywordPlansResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResult} MutateKeywordPlansResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlansResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateKeywordPlansResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateKeywordPlansResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResult} MutateKeywordPlansResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateKeywordPlansResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateKeywordPlansResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateKeywordPlansResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateKeywordPlansResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateKeywordPlansResult} MutateKeywordPlansResult
                             */
                            MutateKeywordPlansResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateKeywordPlansResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateKeywordPlansResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateKeywordPlansResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateKeywordPlansResult} m MutateKeywordPlansResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateKeywordPlansResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateKeywordPlansResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateKeywordPlansResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateKeywordPlansResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateKeywordPlansResult;
                        })();
    
                        services.GenerateForecastCurveRequest = (function() {
    
                            /**
                             * Properties of a GenerateForecastCurveRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateForecastCurveRequest
                             * @property {string|null} [keyword_plan] GenerateForecastCurveRequest keyword_plan
                             */
    
                            /**
                             * Constructs a new GenerateForecastCurveRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateForecastCurveRequest.
                             * @implements IGenerateForecastCurveRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveRequest=} [p] Properties to set
                             */
                            function GenerateForecastCurveRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateForecastCurveRequest keyword_plan.
                             * @member {string} keyword_plan
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @instance
                             */
                            GenerateForecastCurveRequest.prototype.keyword_plan = "";
    
                            /**
                             * Creates a new GenerateForecastCurveRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveRequest} GenerateForecastCurveRequest instance
                             */
                            GenerateForecastCurveRequest.create = function create(properties) {
                                return new GenerateForecastCurveRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateForecastCurveRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastCurveRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveRequest} m GenerateForecastCurveRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastCurveRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_plan != null && Object.hasOwnProperty.call(m, "keyword_plan"))
                                    w.uint32(10).string(m.keyword_plan);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateForecastCurveRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastCurveRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveRequest} message GenerateForecastCurveRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastCurveRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateForecastCurveRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveRequest} GenerateForecastCurveRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastCurveRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateForecastCurveRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.keyword_plan = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateForecastCurveRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveRequest} GenerateForecastCurveRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastCurveRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateForecastCurveRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateForecastCurveRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    if (!$util.isString(m.keyword_plan))
                                        return "keyword_plan: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateForecastCurveRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveRequest} GenerateForecastCurveRequest
                             */
                            GenerateForecastCurveRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateForecastCurveRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateForecastCurveRequest();
                                if (d.keyword_plan != null) {
                                    m.keyword_plan = String(d.keyword_plan);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateForecastCurveRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateForecastCurveRequest} m GenerateForecastCurveRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateForecastCurveRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.keyword_plan = "";
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    d.keyword_plan = m.keyword_plan;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateForecastCurveRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateForecastCurveRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateForecastCurveRequest;
                        })();
    
                        services.GenerateForecastCurveResponse = (function() {
    
                            /**
                             * Properties of a GenerateForecastCurveResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateForecastCurveResponse
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignForecastCurve>|null} [campaign_forecast_curves] GenerateForecastCurveResponse campaign_forecast_curves
                             */
    
                            /**
                             * Constructs a new GenerateForecastCurveResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateForecastCurveResponse.
                             * @implements IGenerateForecastCurveResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveResponse=} [p] Properties to set
                             */
                            function GenerateForecastCurveResponse(p) {
                                this.campaign_forecast_curves = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateForecastCurveResponse campaign_forecast_curves.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignForecastCurve>} campaign_forecast_curves
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @instance
                             */
                            GenerateForecastCurveResponse.prototype.campaign_forecast_curves = $util.emptyArray;
    
                            /**
                             * Creates a new GenerateForecastCurveResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveResponse} GenerateForecastCurveResponse instance
                             */
                            GenerateForecastCurveResponse.create = function create(properties) {
                                return new GenerateForecastCurveResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateForecastCurveResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastCurveResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveResponse} m GenerateForecastCurveResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastCurveResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_forecast_curves != null && m.campaign_forecast_curves.length) {
                                    for (var i = 0; i < m.campaign_forecast_curves.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.encode(m.campaign_forecast_curves[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateForecastCurveResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastCurveResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastCurveResponse} message GenerateForecastCurveResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastCurveResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateForecastCurveResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveResponse} GenerateForecastCurveResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastCurveResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateForecastCurveResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.campaign_forecast_curves && m.campaign_forecast_curves.length))
                                            m.campaign_forecast_curves = [];
                                        m.campaign_forecast_curves.push($root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateForecastCurveResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveResponse} GenerateForecastCurveResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastCurveResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateForecastCurveResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateForecastCurveResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.campaign_forecast_curves != null && m.hasOwnProperty("campaign_forecast_curves")) {
                                    if (!Array.isArray(m.campaign_forecast_curves))
                                        return "campaign_forecast_curves: array expected";
                                    for (var i = 0; i < m.campaign_forecast_curves.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.verify(m.campaign_forecast_curves[i]);
                                            if (e)
                                                return "campaign_forecast_curves." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateForecastCurveResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateForecastCurveResponse} GenerateForecastCurveResponse
                             */
                            GenerateForecastCurveResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateForecastCurveResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateForecastCurveResponse();
                                if (d.campaign_forecast_curves) {
                                    if (!Array.isArray(d.campaign_forecast_curves))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateForecastCurveResponse.campaign_forecast_curves: array expected");
                                    m.campaign_forecast_curves = [];
                                    for (var i = 0; i < d.campaign_forecast_curves.length; ++i) {
                                        if (typeof d.campaign_forecast_curves[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateForecastCurveResponse.campaign_forecast_curves: object expected");
                                        m.campaign_forecast_curves[i] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.fromObject(d.campaign_forecast_curves[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateForecastCurveResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateForecastCurveResponse} m GenerateForecastCurveResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateForecastCurveResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.campaign_forecast_curves = [];
                                }
                                if (m.campaign_forecast_curves && m.campaign_forecast_curves.length) {
                                    d.campaign_forecast_curves = [];
                                    for (var j = 0; j < m.campaign_forecast_curves.length; ++j) {
                                        d.campaign_forecast_curves[j] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.toObject(m.campaign_forecast_curves[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateForecastCurveResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateForecastCurveResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateForecastCurveResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateForecastCurveResponse;
                        })();
    
                        services.GenerateForecastTimeSeriesRequest = (function() {
    
                            /**
                             * Properties of a GenerateForecastTimeSeriesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateForecastTimeSeriesRequest
                             * @property {string|null} [keyword_plan] GenerateForecastTimeSeriesRequest keyword_plan
                             */
    
                            /**
                             * Constructs a new GenerateForecastTimeSeriesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateForecastTimeSeriesRequest.
                             * @implements IGenerateForecastTimeSeriesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesRequest=} [p] Properties to set
                             */
                            function GenerateForecastTimeSeriesRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateForecastTimeSeriesRequest keyword_plan.
                             * @member {string} keyword_plan
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @instance
                             */
                            GenerateForecastTimeSeriesRequest.prototype.keyword_plan = "";
    
                            /**
                             * Creates a new GenerateForecastTimeSeriesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest} GenerateForecastTimeSeriesRequest instance
                             */
                            GenerateForecastTimeSeriesRequest.create = function create(properties) {
                                return new GenerateForecastTimeSeriesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateForecastTimeSeriesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesRequest} m GenerateForecastTimeSeriesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastTimeSeriesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_plan != null && Object.hasOwnProperty.call(m, "keyword_plan"))
                                    w.uint32(10).string(m.keyword_plan);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateForecastTimeSeriesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesRequest} message GenerateForecastTimeSeriesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastTimeSeriesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateForecastTimeSeriesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest} GenerateForecastTimeSeriesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastTimeSeriesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.keyword_plan = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateForecastTimeSeriesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest} GenerateForecastTimeSeriesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastTimeSeriesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateForecastTimeSeriesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateForecastTimeSeriesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    if (!$util.isString(m.keyword_plan))
                                        return "keyword_plan: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateForecastTimeSeriesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest} GenerateForecastTimeSeriesRequest
                             */
                            GenerateForecastTimeSeriesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest();
                                if (d.keyword_plan != null) {
                                    m.keyword_plan = String(d.keyword_plan);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateForecastTimeSeriesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest} m GenerateForecastTimeSeriesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateForecastTimeSeriesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.keyword_plan = "";
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    d.keyword_plan = m.keyword_plan;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateForecastTimeSeriesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateForecastTimeSeriesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateForecastTimeSeriesRequest;
                        })();
    
                        services.GenerateForecastTimeSeriesResponse = (function() {
    
                            /**
                             * Properties of a GenerateForecastTimeSeriesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateForecastTimeSeriesResponse
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanWeeklyTimeSeriesForecast>|null} [weekly_time_series_forecasts] GenerateForecastTimeSeriesResponse weekly_time_series_forecasts
                             */
    
                            /**
                             * Constructs a new GenerateForecastTimeSeriesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateForecastTimeSeriesResponse.
                             * @implements IGenerateForecastTimeSeriesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesResponse=} [p] Properties to set
                             */
                            function GenerateForecastTimeSeriesResponse(p) {
                                this.weekly_time_series_forecasts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateForecastTimeSeriesResponse weekly_time_series_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanWeeklyTimeSeriesForecast>} weekly_time_series_forecasts
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @instance
                             */
                            GenerateForecastTimeSeriesResponse.prototype.weekly_time_series_forecasts = $util.emptyArray;
    
                            /**
                             * Creates a new GenerateForecastTimeSeriesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse} GenerateForecastTimeSeriesResponse instance
                             */
                            GenerateForecastTimeSeriesResponse.create = function create(properties) {
                                return new GenerateForecastTimeSeriesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateForecastTimeSeriesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesResponse} m GenerateForecastTimeSeriesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastTimeSeriesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.weekly_time_series_forecasts != null && m.weekly_time_series_forecasts.length) {
                                    for (var i = 0; i < m.weekly_time_series_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.encode(m.weekly_time_series_forecasts[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateForecastTimeSeriesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastTimeSeriesResponse} message GenerateForecastTimeSeriesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastTimeSeriesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateForecastTimeSeriesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse} GenerateForecastTimeSeriesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastTimeSeriesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.weekly_time_series_forecasts && m.weekly_time_series_forecasts.length))
                                            m.weekly_time_series_forecasts = [];
                                        m.weekly_time_series_forecasts.push($root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateForecastTimeSeriesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse} GenerateForecastTimeSeriesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastTimeSeriesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateForecastTimeSeriesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateForecastTimeSeriesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.weekly_time_series_forecasts != null && m.hasOwnProperty("weekly_time_series_forecasts")) {
                                    if (!Array.isArray(m.weekly_time_series_forecasts))
                                        return "weekly_time_series_forecasts: array expected";
                                    for (var i = 0; i < m.weekly_time_series_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.verify(m.weekly_time_series_forecasts[i]);
                                            if (e)
                                                return "weekly_time_series_forecasts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateForecastTimeSeriesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse} GenerateForecastTimeSeriesResponse
                             */
                            GenerateForecastTimeSeriesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse();
                                if (d.weekly_time_series_forecasts) {
                                    if (!Array.isArray(d.weekly_time_series_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse.weekly_time_series_forecasts: array expected");
                                    m.weekly_time_series_forecasts = [];
                                    for (var i = 0; i < d.weekly_time_series_forecasts.length; ++i) {
                                        if (typeof d.weekly_time_series_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse.weekly_time_series_forecasts: object expected");
                                        m.weekly_time_series_forecasts[i] = $root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.fromObject(d.weekly_time_series_forecasts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateForecastTimeSeriesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse} m GenerateForecastTimeSeriesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateForecastTimeSeriesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.weekly_time_series_forecasts = [];
                                }
                                if (m.weekly_time_series_forecasts && m.weekly_time_series_forecasts.length) {
                                    d.weekly_time_series_forecasts = [];
                                    for (var j = 0; j < m.weekly_time_series_forecasts.length; ++j) {
                                        d.weekly_time_series_forecasts[j] = $root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.toObject(m.weekly_time_series_forecasts[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateForecastTimeSeriesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateForecastTimeSeriesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateForecastTimeSeriesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateForecastTimeSeriesResponse;
                        })();
    
                        services.GenerateForecastMetricsRequest = (function() {
    
                            /**
                             * Properties of a GenerateForecastMetricsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateForecastMetricsRequest
                             * @property {string|null} [keyword_plan] GenerateForecastMetricsRequest keyword_plan
                             */
    
                            /**
                             * Constructs a new GenerateForecastMetricsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateForecastMetricsRequest.
                             * @implements IGenerateForecastMetricsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsRequest=} [p] Properties to set
                             */
                            function GenerateForecastMetricsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateForecastMetricsRequest keyword_plan.
                             * @member {string} keyword_plan
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @instance
                             */
                            GenerateForecastMetricsRequest.prototype.keyword_plan = "";
    
                            /**
                             * Creates a new GenerateForecastMetricsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsRequest} GenerateForecastMetricsRequest instance
                             */
                            GenerateForecastMetricsRequest.create = function create(properties) {
                                return new GenerateForecastMetricsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateForecastMetricsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastMetricsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsRequest} m GenerateForecastMetricsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastMetricsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_plan != null && Object.hasOwnProperty.call(m, "keyword_plan"))
                                    w.uint32(10).string(m.keyword_plan);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateForecastMetricsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastMetricsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsRequest} message GenerateForecastMetricsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastMetricsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateForecastMetricsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsRequest} GenerateForecastMetricsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastMetricsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateForecastMetricsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.keyword_plan = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateForecastMetricsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsRequest} GenerateForecastMetricsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastMetricsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateForecastMetricsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateForecastMetricsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    if (!$util.isString(m.keyword_plan))
                                        return "keyword_plan: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateForecastMetricsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsRequest} GenerateForecastMetricsRequest
                             */
                            GenerateForecastMetricsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateForecastMetricsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateForecastMetricsRequest();
                                if (d.keyword_plan != null) {
                                    m.keyword_plan = String(d.keyword_plan);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateForecastMetricsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateForecastMetricsRequest} m GenerateForecastMetricsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateForecastMetricsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.keyword_plan = "";
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    d.keyword_plan = m.keyword_plan;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateForecastMetricsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateForecastMetricsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateForecastMetricsRequest;
                        })();
    
                        services.GenerateForecastMetricsResponse = (function() {
    
                            /**
                             * Properties of a GenerateForecastMetricsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateForecastMetricsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignForecast>|null} [campaign_forecasts] GenerateForecastMetricsResponse campaign_forecasts
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanAdGroupForecast>|null} [ad_group_forecasts] GenerateForecastMetricsResponse ad_group_forecasts
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanKeywordForecast>|null} [keyword_forecasts] GenerateForecastMetricsResponse keyword_forecasts
                             */
    
                            /**
                             * Constructs a new GenerateForecastMetricsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateForecastMetricsResponse.
                             * @implements IGenerateForecastMetricsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsResponse=} [p] Properties to set
                             */
                            function GenerateForecastMetricsResponse(p) {
                                this.campaign_forecasts = [];
                                this.ad_group_forecasts = [];
                                this.keyword_forecasts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateForecastMetricsResponse campaign_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanCampaignForecast>} campaign_forecasts
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @instance
                             */
                            GenerateForecastMetricsResponse.prototype.campaign_forecasts = $util.emptyArray;
    
                            /**
                             * GenerateForecastMetricsResponse ad_group_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanAdGroupForecast>} ad_group_forecasts
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @instance
                             */
                            GenerateForecastMetricsResponse.prototype.ad_group_forecasts = $util.emptyArray;
    
                            /**
                             * GenerateForecastMetricsResponse keyword_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanKeywordForecast>} keyword_forecasts
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @instance
                             */
                            GenerateForecastMetricsResponse.prototype.keyword_forecasts = $util.emptyArray;
    
                            /**
                             * Creates a new GenerateForecastMetricsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsResponse} GenerateForecastMetricsResponse instance
                             */
                            GenerateForecastMetricsResponse.create = function create(properties) {
                                return new GenerateForecastMetricsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateForecastMetricsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastMetricsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsResponse} m GenerateForecastMetricsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastMetricsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_forecasts != null && m.campaign_forecasts.length) {
                                    for (var i = 0; i < m.campaign_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast.encode(m.campaign_forecasts[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.ad_group_forecasts != null && m.ad_group_forecasts.length) {
                                    for (var i = 0; i < m.ad_group_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.encode(m.ad_group_forecasts[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.keyword_forecasts != null && m.keyword_forecasts.length) {
                                    for (var i = 0; i < m.keyword_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast.encode(m.keyword_forecasts[i], w.uint32(26).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateForecastMetricsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateForecastMetricsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateForecastMetricsResponse} message GenerateForecastMetricsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateForecastMetricsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateForecastMetricsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsResponse} GenerateForecastMetricsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastMetricsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateForecastMetricsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.campaign_forecasts && m.campaign_forecasts.length))
                                            m.campaign_forecasts = [];
                                        m.campaign_forecasts.push($root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.ad_group_forecasts && m.ad_group_forecasts.length))
                                            m.ad_group_forecasts = [];
                                        m.ad_group_forecasts.push($root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.keyword_forecasts && m.keyword_forecasts.length))
                                            m.keyword_forecasts = [];
                                        m.keyword_forecasts.push($root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateForecastMetricsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsResponse} GenerateForecastMetricsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateForecastMetricsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateForecastMetricsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateForecastMetricsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.campaign_forecasts != null && m.hasOwnProperty("campaign_forecasts")) {
                                    if (!Array.isArray(m.campaign_forecasts))
                                        return "campaign_forecasts: array expected";
                                    for (var i = 0; i < m.campaign_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast.verify(m.campaign_forecasts[i]);
                                            if (e)
                                                return "campaign_forecasts." + e;
                                        }
                                    }
                                }
                                if (m.ad_group_forecasts != null && m.hasOwnProperty("ad_group_forecasts")) {
                                    if (!Array.isArray(m.ad_group_forecasts))
                                        return "ad_group_forecasts: array expected";
                                    for (var i = 0; i < m.ad_group_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.verify(m.ad_group_forecasts[i]);
                                            if (e)
                                                return "ad_group_forecasts." + e;
                                        }
                                    }
                                }
                                if (m.keyword_forecasts != null && m.hasOwnProperty("keyword_forecasts")) {
                                    if (!Array.isArray(m.keyword_forecasts))
                                        return "keyword_forecasts: array expected";
                                    for (var i = 0; i < m.keyword_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast.verify(m.keyword_forecasts[i]);
                                            if (e)
                                                return "keyword_forecasts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateForecastMetricsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateForecastMetricsResponse} GenerateForecastMetricsResponse
                             */
                            GenerateForecastMetricsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateForecastMetricsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateForecastMetricsResponse();
                                if (d.campaign_forecasts) {
                                    if (!Array.isArray(d.campaign_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateForecastMetricsResponse.campaign_forecasts: array expected");
                                    m.campaign_forecasts = [];
                                    for (var i = 0; i < d.campaign_forecasts.length; ++i) {
                                        if (typeof d.campaign_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateForecastMetricsResponse.campaign_forecasts: object expected");
                                        m.campaign_forecasts[i] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast.fromObject(d.campaign_forecasts[i]);
                                    }
                                }
                                if (d.ad_group_forecasts) {
                                    if (!Array.isArray(d.ad_group_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateForecastMetricsResponse.ad_group_forecasts: array expected");
                                    m.ad_group_forecasts = [];
                                    for (var i = 0; i < d.ad_group_forecasts.length; ++i) {
                                        if (typeof d.ad_group_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateForecastMetricsResponse.ad_group_forecasts: object expected");
                                        m.ad_group_forecasts[i] = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.fromObject(d.ad_group_forecasts[i]);
                                    }
                                }
                                if (d.keyword_forecasts) {
                                    if (!Array.isArray(d.keyword_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateForecastMetricsResponse.keyword_forecasts: array expected");
                                    m.keyword_forecasts = [];
                                    for (var i = 0; i < d.keyword_forecasts.length; ++i) {
                                        if (typeof d.keyword_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateForecastMetricsResponse.keyword_forecasts: object expected");
                                        m.keyword_forecasts[i] = $root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast.fromObject(d.keyword_forecasts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateForecastMetricsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateForecastMetricsResponse} m GenerateForecastMetricsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateForecastMetricsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.campaign_forecasts = [];
                                    d.ad_group_forecasts = [];
                                    d.keyword_forecasts = [];
                                }
                                if (m.campaign_forecasts && m.campaign_forecasts.length) {
                                    d.campaign_forecasts = [];
                                    for (var j = 0; j < m.campaign_forecasts.length; ++j) {
                                        d.campaign_forecasts[j] = $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast.toObject(m.campaign_forecasts[j], o);
                                    }
                                }
                                if (m.ad_group_forecasts && m.ad_group_forecasts.length) {
                                    d.ad_group_forecasts = [];
                                    for (var j = 0; j < m.ad_group_forecasts.length; ++j) {
                                        d.ad_group_forecasts[j] = $root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.toObject(m.ad_group_forecasts[j], o);
                                    }
                                }
                                if (m.keyword_forecasts && m.keyword_forecasts.length) {
                                    d.keyword_forecasts = [];
                                    for (var j = 0; j < m.keyword_forecasts.length; ++j) {
                                        d.keyword_forecasts[j] = $root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast.toObject(m.keyword_forecasts[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateForecastMetricsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateForecastMetricsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateForecastMetricsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateForecastMetricsResponse;
                        })();
    
                        services.KeywordPlanCampaignForecast = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaignForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanCampaignForecast
                             * @property {string|null} [keyword_plan_campaign] KeywordPlanCampaignForecast keyword_plan_campaign
                             * @property {google.ads.googleads.v11.services.IForecastMetrics|null} [campaign_forecast] KeywordPlanCampaignForecast campaign_forecast
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaignForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanCampaignForecast.
                             * @implements IKeywordPlanCampaignForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecast=} [p] Properties to set
                             */
                            function KeywordPlanCampaignForecast(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanCampaignForecast keyword_plan_campaign.
                             * @member {string|null|undefined} keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @instance
                             */
                            KeywordPlanCampaignForecast.prototype.keyword_plan_campaign = null;
    
                            /**
                             * KeywordPlanCampaignForecast campaign_forecast.
                             * @member {google.ads.googleads.v11.services.IForecastMetrics|null|undefined} campaign_forecast
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @instance
                             */
                            KeywordPlanCampaignForecast.prototype.campaign_forecast = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanCampaignForecast _keyword_plan_campaign.
                             * @member {"keyword_plan_campaign"|undefined} _keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignForecast.prototype, "_keyword_plan_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanCampaignForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecast} KeywordPlanCampaignForecast instance
                             */
                            KeywordPlanCampaignForecast.create = function create(properties) {
                                return new KeywordPlanCampaignForecast(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignForecast message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecast} m KeywordPlanCampaignForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.campaign_forecast != null && Object.hasOwnProperty.call(m, "campaign_forecast"))
                                    $root.google.ads.googleads.v11.services.ForecastMetrics.encode(m.campaign_forecast, w.uint32(18).fork()).ldelim();
                                if (m.keyword_plan_campaign != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign"))
                                    w.uint32(26).string(m.keyword_plan_campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecast} message KeywordPlanCampaignForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecast} KeywordPlanCampaignForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.keyword_plan_campaign = r.string();
                                        break;
                                    case 2:
                                        m.campaign_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecast} KeywordPlanCampaignForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaignForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaignForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    p._keyword_plan_campaign = 1;
                                    if (!$util.isString(m.keyword_plan_campaign))
                                        return "keyword_plan_campaign: string expected";
                                }
                                if (m.campaign_forecast != null && m.hasOwnProperty("campaign_forecast")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ForecastMetrics.verify(m.campaign_forecast);
                                        if (e)
                                            return "campaign_forecast." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaignForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecast} KeywordPlanCampaignForecast
                             */
                            KeywordPlanCampaignForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecast();
                                if (d.keyword_plan_campaign != null) {
                                    m.keyword_plan_campaign = String(d.keyword_plan_campaign);
                                }
                                if (d.campaign_forecast != null) {
                                    if (typeof d.campaign_forecast !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignForecast.campaign_forecast: object expected");
                                    m.campaign_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.fromObject(d.campaign_forecast);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaignForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanCampaignForecast} m KeywordPlanCampaignForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaignForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.campaign_forecast = null;
                                }
                                if (m.campaign_forecast != null && m.hasOwnProperty("campaign_forecast")) {
                                    d.campaign_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.toObject(m.campaign_forecast, o);
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    d.keyword_plan_campaign = m.keyword_plan_campaign;
                                    if (o.oneofs)
                                        d._keyword_plan_campaign = "keyword_plan_campaign";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanCampaignForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaignForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanCampaignForecast;
                        })();
    
                        services.KeywordPlanAdGroupForecast = (function() {
    
                            /**
                             * Properties of a KeywordPlanAdGroupForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanAdGroupForecast
                             * @property {string|null} [keyword_plan_ad_group] KeywordPlanAdGroupForecast keyword_plan_ad_group
                             * @property {google.ads.googleads.v11.services.IForecastMetrics|null} [ad_group_forecast] KeywordPlanAdGroupForecast ad_group_forecast
                             */
    
                            /**
                             * Constructs a new KeywordPlanAdGroupForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanAdGroupForecast.
                             * @implements IKeywordPlanAdGroupForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupForecast=} [p] Properties to set
                             */
                            function KeywordPlanAdGroupForecast(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanAdGroupForecast keyword_plan_ad_group.
                             * @member {string|null|undefined} keyword_plan_ad_group
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @instance
                             */
                            KeywordPlanAdGroupForecast.prototype.keyword_plan_ad_group = null;
    
                            /**
                             * KeywordPlanAdGroupForecast ad_group_forecast.
                             * @member {google.ads.googleads.v11.services.IForecastMetrics|null|undefined} ad_group_forecast
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @instance
                             */
                            KeywordPlanAdGroupForecast.prototype.ad_group_forecast = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanAdGroupForecast _keyword_plan_ad_group.
                             * @member {"keyword_plan_ad_group"|undefined} _keyword_plan_ad_group
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanAdGroupForecast.prototype, "_keyword_plan_ad_group", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_ad_group"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanAdGroupForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupForecast} KeywordPlanAdGroupForecast instance
                             */
                            KeywordPlanAdGroupForecast.create = function create(properties) {
                                return new KeywordPlanAdGroupForecast(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupForecast message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupForecast} m KeywordPlanAdGroupForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_group_forecast != null && Object.hasOwnProperty.call(m, "ad_group_forecast"))
                                    $root.google.ads.googleads.v11.services.ForecastMetrics.encode(m.ad_group_forecast, w.uint32(18).fork()).ldelim();
                                if (m.keyword_plan_ad_group != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group"))
                                    w.uint32(26).string(m.keyword_plan_ad_group);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanAdGroupForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanAdGroupForecast} message KeywordPlanAdGroupForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanAdGroupForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupForecast} KeywordPlanAdGroupForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.keyword_plan_ad_group = r.string();
                                        break;
                                    case 2:
                                        m.ad_group_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanAdGroupForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupForecast} KeywordPlanAdGroupForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanAdGroupForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanAdGroupForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanAdGroupForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.keyword_plan_ad_group != null && m.hasOwnProperty("keyword_plan_ad_group")) {
                                    p._keyword_plan_ad_group = 1;
                                    if (!$util.isString(m.keyword_plan_ad_group))
                                        return "keyword_plan_ad_group: string expected";
                                }
                                if (m.ad_group_forecast != null && m.hasOwnProperty("ad_group_forecast")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ForecastMetrics.verify(m.ad_group_forecast);
                                        if (e)
                                            return "ad_group_forecast." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanAdGroupForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanAdGroupForecast} KeywordPlanAdGroupForecast
                             */
                            KeywordPlanAdGroupForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanAdGroupForecast();
                                if (d.keyword_plan_ad_group != null) {
                                    m.keyword_plan_ad_group = String(d.keyword_plan_ad_group);
                                }
                                if (d.ad_group_forecast != null) {
                                    if (typeof d.ad_group_forecast !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanAdGroupForecast.ad_group_forecast: object expected");
                                    m.ad_group_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.fromObject(d.ad_group_forecast);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanAdGroupForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanAdGroupForecast} m KeywordPlanAdGroupForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanAdGroupForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.ad_group_forecast = null;
                                }
                                if (m.ad_group_forecast != null && m.hasOwnProperty("ad_group_forecast")) {
                                    d.ad_group_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.toObject(m.ad_group_forecast, o);
                                }
                                if (m.keyword_plan_ad_group != null && m.hasOwnProperty("keyword_plan_ad_group")) {
                                    d.keyword_plan_ad_group = m.keyword_plan_ad_group;
                                    if (o.oneofs)
                                        d._keyword_plan_ad_group = "keyword_plan_ad_group";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanAdGroupForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanAdGroupForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanAdGroupForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanAdGroupForecast;
                        })();
    
                        services.KeywordPlanKeywordForecast = (function() {
    
                            /**
                             * Properties of a KeywordPlanKeywordForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanKeywordForecast
                             * @property {string|null} [keyword_plan_ad_group_keyword] KeywordPlanKeywordForecast keyword_plan_ad_group_keyword
                             * @property {google.ads.googleads.v11.services.IForecastMetrics|null} [keyword_forecast] KeywordPlanKeywordForecast keyword_forecast
                             */
    
                            /**
                             * Constructs a new KeywordPlanKeywordForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanKeywordForecast.
                             * @implements IKeywordPlanKeywordForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordForecast=} [p] Properties to set
                             */
                            function KeywordPlanKeywordForecast(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanKeywordForecast keyword_plan_ad_group_keyword.
                             * @member {string|null|undefined} keyword_plan_ad_group_keyword
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @instance
                             */
                            KeywordPlanKeywordForecast.prototype.keyword_plan_ad_group_keyword = null;
    
                            /**
                             * KeywordPlanKeywordForecast keyword_forecast.
                             * @member {google.ads.googleads.v11.services.IForecastMetrics|null|undefined} keyword_forecast
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @instance
                             */
                            KeywordPlanKeywordForecast.prototype.keyword_forecast = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanKeywordForecast _keyword_plan_ad_group_keyword.
                             * @member {"keyword_plan_ad_group_keyword"|undefined} _keyword_plan_ad_group_keyword
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanKeywordForecast.prototype, "_keyword_plan_ad_group_keyword", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_ad_group_keyword"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanKeywordForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordForecast} KeywordPlanKeywordForecast instance
                             */
                            KeywordPlanKeywordForecast.create = function create(properties) {
                                return new KeywordPlanKeywordForecast(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanKeywordForecast message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanKeywordForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordForecast} m KeywordPlanKeywordForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanKeywordForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_forecast != null && Object.hasOwnProperty.call(m, "keyword_forecast"))
                                    $root.google.ads.googleads.v11.services.ForecastMetrics.encode(m.keyword_forecast, w.uint32(18).fork()).ldelim();
                                if (m.keyword_plan_ad_group_keyword != null && Object.hasOwnProperty.call(m, "keyword_plan_ad_group_keyword"))
                                    w.uint32(26).string(m.keyword_plan_ad_group_keyword);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanKeywordForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanKeywordForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordForecast} message KeywordPlanKeywordForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanKeywordForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanKeywordForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordForecast} KeywordPlanKeywordForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanKeywordForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.keyword_plan_ad_group_keyword = r.string();
                                        break;
                                    case 2:
                                        m.keyword_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanKeywordForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordForecast} KeywordPlanKeywordForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanKeywordForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanKeywordForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanKeywordForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.keyword_plan_ad_group_keyword != null && m.hasOwnProperty("keyword_plan_ad_group_keyword")) {
                                    p._keyword_plan_ad_group_keyword = 1;
                                    if (!$util.isString(m.keyword_plan_ad_group_keyword))
                                        return "keyword_plan_ad_group_keyword: string expected";
                                }
                                if (m.keyword_forecast != null && m.hasOwnProperty("keyword_forecast")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ForecastMetrics.verify(m.keyword_forecast);
                                        if (e)
                                            return "keyword_forecast." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanKeywordForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordForecast} KeywordPlanKeywordForecast
                             */
                            KeywordPlanKeywordForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanKeywordForecast();
                                if (d.keyword_plan_ad_group_keyword != null) {
                                    m.keyword_plan_ad_group_keyword = String(d.keyword_plan_ad_group_keyword);
                                }
                                if (d.keyword_forecast != null) {
                                    if (typeof d.keyword_forecast !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanKeywordForecast.keyword_forecast: object expected");
                                    m.keyword_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.fromObject(d.keyword_forecast);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanKeywordForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanKeywordForecast} m KeywordPlanKeywordForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanKeywordForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.keyword_forecast = null;
                                }
                                if (m.keyword_forecast != null && m.hasOwnProperty("keyword_forecast")) {
                                    d.keyword_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.toObject(m.keyword_forecast, o);
                                }
                                if (m.keyword_plan_ad_group_keyword != null && m.hasOwnProperty("keyword_plan_ad_group_keyword")) {
                                    d.keyword_plan_ad_group_keyword = m.keyword_plan_ad_group_keyword;
                                    if (o.oneofs)
                                        d._keyword_plan_ad_group_keyword = "keyword_plan_ad_group_keyword";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanKeywordForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanKeywordForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanKeywordForecast;
                        })();
    
                        services.KeywordPlanCampaignForecastCurve = (function() {
    
                            /**
                             * Properties of a KeywordPlanCampaignForecastCurve.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanCampaignForecastCurve
                             * @property {string|null} [keyword_plan_campaign] KeywordPlanCampaignForecastCurve keyword_plan_campaign
                             * @property {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecastCurve|null} [max_cpc_bid_forecast_curve] KeywordPlanCampaignForecastCurve max_cpc_bid_forecast_curve
                             */
    
                            /**
                             * Constructs a new KeywordPlanCampaignForecastCurve.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanCampaignForecastCurve.
                             * @implements IKeywordPlanCampaignForecastCurve
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecastCurve=} [p] Properties to set
                             */
                            function KeywordPlanCampaignForecastCurve(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanCampaignForecastCurve keyword_plan_campaign.
                             * @member {string|null|undefined} keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @instance
                             */
                            KeywordPlanCampaignForecastCurve.prototype.keyword_plan_campaign = null;
    
                            /**
                             * KeywordPlanCampaignForecastCurve max_cpc_bid_forecast_curve.
                             * @member {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecastCurve|null|undefined} max_cpc_bid_forecast_curve
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @instance
                             */
                            KeywordPlanCampaignForecastCurve.prototype.max_cpc_bid_forecast_curve = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanCampaignForecastCurve _keyword_plan_campaign.
                             * @member {"keyword_plan_campaign"|undefined} _keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanCampaignForecastCurve.prototype, "_keyword_plan_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanCampaignForecastCurve instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecastCurve=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve} KeywordPlanCampaignForecastCurve instance
                             */
                            KeywordPlanCampaignForecastCurve.create = function create(properties) {
                                return new KeywordPlanCampaignForecastCurve(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignForecastCurve message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecastCurve} m KeywordPlanCampaignForecastCurve message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignForecastCurve.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.max_cpc_bid_forecast_curve != null && Object.hasOwnProperty.call(m, "max_cpc_bid_forecast_curve"))
                                    $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.encode(m.max_cpc_bid_forecast_curve, w.uint32(18).fork()).ldelim();
                                if (m.keyword_plan_campaign != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign"))
                                    w.uint32(26).string(m.keyword_plan_campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanCampaignForecastCurve message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanCampaignForecastCurve} message KeywordPlanCampaignForecastCurve message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanCampaignForecastCurve.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignForecastCurve message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve} KeywordPlanCampaignForecastCurve
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignForecastCurve.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.keyword_plan_campaign = r.string();
                                        break;
                                    case 2:
                                        m.max_cpc_bid_forecast_curve = $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanCampaignForecastCurve message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve} KeywordPlanCampaignForecastCurve
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanCampaignForecastCurve.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanCampaignForecastCurve message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanCampaignForecastCurve.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    p._keyword_plan_campaign = 1;
                                    if (!$util.isString(m.keyword_plan_campaign))
                                        return "keyword_plan_campaign: string expected";
                                }
                                if (m.max_cpc_bid_forecast_curve != null && m.hasOwnProperty("max_cpc_bid_forecast_curve")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.verify(m.max_cpc_bid_forecast_curve);
                                        if (e)
                                            return "max_cpc_bid_forecast_curve." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanCampaignForecastCurve message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve} KeywordPlanCampaignForecastCurve
                             */
                            KeywordPlanCampaignForecastCurve.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve();
                                if (d.keyword_plan_campaign != null) {
                                    m.keyword_plan_campaign = String(d.keyword_plan_campaign);
                                }
                                if (d.max_cpc_bid_forecast_curve != null) {
                                    if (typeof d.max_cpc_bid_forecast_curve !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve.max_cpc_bid_forecast_curve: object expected");
                                    m.max_cpc_bid_forecast_curve = $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.fromObject(d.max_cpc_bid_forecast_curve);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanCampaignForecastCurve message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve} m KeywordPlanCampaignForecastCurve
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanCampaignForecastCurve.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.max_cpc_bid_forecast_curve = null;
                                }
                                if (m.max_cpc_bid_forecast_curve != null && m.hasOwnProperty("max_cpc_bid_forecast_curve")) {
                                    d.max_cpc_bid_forecast_curve = $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.toObject(m.max_cpc_bid_forecast_curve, o);
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    d.keyword_plan_campaign = m.keyword_plan_campaign;
                                    if (o.oneofs)
                                        d._keyword_plan_campaign = "keyword_plan_campaign";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanCampaignForecastCurve to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanCampaignForecastCurve
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanCampaignForecastCurve.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanCampaignForecastCurve;
                        })();
    
                        services.KeywordPlanMaxCpcBidForecastCurve = (function() {
    
                            /**
                             * Properties of a KeywordPlanMaxCpcBidForecastCurve.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanMaxCpcBidForecastCurve
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecast>|null} [max_cpc_bid_forecasts] KeywordPlanMaxCpcBidForecastCurve max_cpc_bid_forecasts
                             */
    
                            /**
                             * Constructs a new KeywordPlanMaxCpcBidForecastCurve.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanMaxCpcBidForecastCurve.
                             * @implements IKeywordPlanMaxCpcBidForecastCurve
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecastCurve=} [p] Properties to set
                             */
                            function KeywordPlanMaxCpcBidForecastCurve(p) {
                                this.max_cpc_bid_forecasts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanMaxCpcBidForecastCurve max_cpc_bid_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecast>} max_cpc_bid_forecasts
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @instance
                             */
                            KeywordPlanMaxCpcBidForecastCurve.prototype.max_cpc_bid_forecasts = $util.emptyArray;
    
                            /**
                             * Creates a new KeywordPlanMaxCpcBidForecastCurve instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecastCurve=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve} KeywordPlanMaxCpcBidForecastCurve instance
                             */
                            KeywordPlanMaxCpcBidForecastCurve.create = function create(properties) {
                                return new KeywordPlanMaxCpcBidForecastCurve(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanMaxCpcBidForecastCurve message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecastCurve} m KeywordPlanMaxCpcBidForecastCurve message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanMaxCpcBidForecastCurve.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.max_cpc_bid_forecasts != null && m.max_cpc_bid_forecasts.length) {
                                    for (var i = 0; i < m.max_cpc_bid_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.encode(m.max_cpc_bid_forecasts[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanMaxCpcBidForecastCurve message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecastCurve} message KeywordPlanMaxCpcBidForecastCurve message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanMaxCpcBidForecastCurve.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanMaxCpcBidForecastCurve message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve} KeywordPlanMaxCpcBidForecastCurve
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanMaxCpcBidForecastCurve.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.max_cpc_bid_forecasts && m.max_cpc_bid_forecasts.length))
                                            m.max_cpc_bid_forecasts = [];
                                        m.max_cpc_bid_forecasts.push($root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanMaxCpcBidForecastCurve message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve} KeywordPlanMaxCpcBidForecastCurve
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanMaxCpcBidForecastCurve.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanMaxCpcBidForecastCurve message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanMaxCpcBidForecastCurve.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.max_cpc_bid_forecasts != null && m.hasOwnProperty("max_cpc_bid_forecasts")) {
                                    if (!Array.isArray(m.max_cpc_bid_forecasts))
                                        return "max_cpc_bid_forecasts: array expected";
                                    for (var i = 0; i < m.max_cpc_bid_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.verify(m.max_cpc_bid_forecasts[i]);
                                            if (e)
                                                return "max_cpc_bid_forecasts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanMaxCpcBidForecastCurve message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve} KeywordPlanMaxCpcBidForecastCurve
                             */
                            KeywordPlanMaxCpcBidForecastCurve.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve();
                                if (d.max_cpc_bid_forecasts) {
                                    if (!Array.isArray(d.max_cpc_bid_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.max_cpc_bid_forecasts: array expected");
                                    m.max_cpc_bid_forecasts = [];
                                    for (var i = 0; i < d.max_cpc_bid_forecasts.length; ++i) {
                                        if (typeof d.max_cpc_bid_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve.max_cpc_bid_forecasts: object expected");
                                        m.max_cpc_bid_forecasts[i] = $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.fromObject(d.max_cpc_bid_forecasts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanMaxCpcBidForecastCurve message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve} m KeywordPlanMaxCpcBidForecastCurve
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanMaxCpcBidForecastCurve.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.max_cpc_bid_forecasts = [];
                                }
                                if (m.max_cpc_bid_forecasts && m.max_cpc_bid_forecasts.length) {
                                    d.max_cpc_bid_forecasts = [];
                                    for (var j = 0; j < m.max_cpc_bid_forecasts.length; ++j) {
                                        d.max_cpc_bid_forecasts[j] = $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.toObject(m.max_cpc_bid_forecasts[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanMaxCpcBidForecastCurve to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecastCurve
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanMaxCpcBidForecastCurve.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanMaxCpcBidForecastCurve;
                        })();
    
                        services.KeywordPlanMaxCpcBidForecast = (function() {
    
                            /**
                             * Properties of a KeywordPlanMaxCpcBidForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanMaxCpcBidForecast
                             * @property {number|null} [max_cpc_bid_micros] KeywordPlanMaxCpcBidForecast max_cpc_bid_micros
                             * @property {google.ads.googleads.v11.services.IForecastMetrics|null} [max_cpc_bid_forecast] KeywordPlanMaxCpcBidForecast max_cpc_bid_forecast
                             */
    
                            /**
                             * Constructs a new KeywordPlanMaxCpcBidForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanMaxCpcBidForecast.
                             * @implements IKeywordPlanMaxCpcBidForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecast=} [p] Properties to set
                             */
                            function KeywordPlanMaxCpcBidForecast(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanMaxCpcBidForecast max_cpc_bid_micros.
                             * @member {number|null|undefined} max_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @instance
                             */
                            KeywordPlanMaxCpcBidForecast.prototype.max_cpc_bid_micros = null;
    
                            /**
                             * KeywordPlanMaxCpcBidForecast max_cpc_bid_forecast.
                             * @member {google.ads.googleads.v11.services.IForecastMetrics|null|undefined} max_cpc_bid_forecast
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @instance
                             */
                            KeywordPlanMaxCpcBidForecast.prototype.max_cpc_bid_forecast = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanMaxCpcBidForecast _max_cpc_bid_micros.
                             * @member {"max_cpc_bid_micros"|undefined} _max_cpc_bid_micros
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanMaxCpcBidForecast.prototype, "_max_cpc_bid_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["max_cpc_bid_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanMaxCpcBidForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast} KeywordPlanMaxCpcBidForecast instance
                             */
                            KeywordPlanMaxCpcBidForecast.create = function create(properties) {
                                return new KeywordPlanMaxCpcBidForecast(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanMaxCpcBidForecast message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecast} m KeywordPlanMaxCpcBidForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanMaxCpcBidForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.max_cpc_bid_forecast != null && Object.hasOwnProperty.call(m, "max_cpc_bid_forecast"))
                                    $root.google.ads.googleads.v11.services.ForecastMetrics.encode(m.max_cpc_bid_forecast, w.uint32(18).fork()).ldelim();
                                if (m.max_cpc_bid_micros != null && Object.hasOwnProperty.call(m, "max_cpc_bid_micros"))
                                    w.uint32(24).int64(m.max_cpc_bid_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanMaxCpcBidForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanMaxCpcBidForecast} message KeywordPlanMaxCpcBidForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanMaxCpcBidForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanMaxCpcBidForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast} KeywordPlanMaxCpcBidForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanMaxCpcBidForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.max_cpc_bid_micros = r.int64();
                                        break;
                                    case 2:
                                        m.max_cpc_bid_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanMaxCpcBidForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast} KeywordPlanMaxCpcBidForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanMaxCpcBidForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanMaxCpcBidForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanMaxCpcBidForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.max_cpc_bid_micros != null && m.hasOwnProperty("max_cpc_bid_micros")) {
                                    p._max_cpc_bid_micros = 1;
                                    if (!$util.isInteger(m.max_cpc_bid_micros) && !(m.max_cpc_bid_micros && $util.isInteger(m.max_cpc_bid_micros.low) && $util.isInteger(m.max_cpc_bid_micros.high)))
                                        return "max_cpc_bid_micros: integer|Long expected";
                                }
                                if (m.max_cpc_bid_forecast != null && m.hasOwnProperty("max_cpc_bid_forecast")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ForecastMetrics.verify(m.max_cpc_bid_forecast);
                                        if (e)
                                            return "max_cpc_bid_forecast." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanMaxCpcBidForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast} KeywordPlanMaxCpcBidForecast
                             */
                            KeywordPlanMaxCpcBidForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast();
                                if (d.max_cpc_bid_micros != null) {
                                    if ($util.Long)
                                        (m.max_cpc_bid_micros = $util.Long.fromValue(d.max_cpc_bid_micros)).unsigned = false;
                                    else if (typeof d.max_cpc_bid_micros === "string")
                                        m.max_cpc_bid_micros = parseInt(d.max_cpc_bid_micros, 10);
                                    else if (typeof d.max_cpc_bid_micros === "number")
                                        m.max_cpc_bid_micros = d.max_cpc_bid_micros;
                                    else if (typeof d.max_cpc_bid_micros === "object")
                                        m.max_cpc_bid_micros = new $util.LongBits(d.max_cpc_bid_micros.low >>> 0, d.max_cpc_bid_micros.high >>> 0).toNumber();
                                }
                                if (d.max_cpc_bid_forecast != null) {
                                    if (typeof d.max_cpc_bid_forecast !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast.max_cpc_bid_forecast: object expected");
                                    m.max_cpc_bid_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.fromObject(d.max_cpc_bid_forecast);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanMaxCpcBidForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast} m KeywordPlanMaxCpcBidForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanMaxCpcBidForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.max_cpc_bid_forecast = null;
                                }
                                if (m.max_cpc_bid_forecast != null && m.hasOwnProperty("max_cpc_bid_forecast")) {
                                    d.max_cpc_bid_forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.toObject(m.max_cpc_bid_forecast, o);
                                }
                                if (m.max_cpc_bid_micros != null && m.hasOwnProperty("max_cpc_bid_micros")) {
                                    if (typeof m.max_cpc_bid_micros === "number")
                                        d.max_cpc_bid_micros = o.longs === String ? String(m.max_cpc_bid_micros) : m.max_cpc_bid_micros;
                                    else
                                        d.max_cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.max_cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.max_cpc_bid_micros.low >>> 0, m.max_cpc_bid_micros.high >>> 0).toNumber() : m.max_cpc_bid_micros;
                                    if (o.oneofs)
                                        d._max_cpc_bid_micros = "max_cpc_bid_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanMaxCpcBidForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanMaxCpcBidForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanMaxCpcBidForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanMaxCpcBidForecast;
                        })();
    
                        services.KeywordPlanWeeklyTimeSeriesForecast = (function() {
    
                            /**
                             * Properties of a KeywordPlanWeeklyTimeSeriesForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanWeeklyTimeSeriesForecast
                             * @property {string|null} [keyword_plan_campaign] KeywordPlanWeeklyTimeSeriesForecast keyword_plan_campaign
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanWeeklyForecast>|null} [weekly_forecasts] KeywordPlanWeeklyTimeSeriesForecast weekly_forecasts
                             */
    
                            /**
                             * Constructs a new KeywordPlanWeeklyTimeSeriesForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanWeeklyTimeSeriesForecast.
                             * @implements IKeywordPlanWeeklyTimeSeriesForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyTimeSeriesForecast=} [p] Properties to set
                             */
                            function KeywordPlanWeeklyTimeSeriesForecast(p) {
                                this.weekly_forecasts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanWeeklyTimeSeriesForecast keyword_plan_campaign.
                             * @member {string|null|undefined} keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @instance
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.prototype.keyword_plan_campaign = null;
    
                            /**
                             * KeywordPlanWeeklyTimeSeriesForecast weekly_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanWeeklyForecast>} weekly_forecasts
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @instance
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.prototype.weekly_forecasts = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanWeeklyTimeSeriesForecast _keyword_plan_campaign.
                             * @member {"keyword_plan_campaign"|undefined} _keyword_plan_campaign
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanWeeklyTimeSeriesForecast.prototype, "_keyword_plan_campaign", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_plan_campaign"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanWeeklyTimeSeriesForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyTimeSeriesForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast} KeywordPlanWeeklyTimeSeriesForecast instance
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.create = function create(properties) {
                                return new KeywordPlanWeeklyTimeSeriesForecast(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanWeeklyTimeSeriesForecast message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyTimeSeriesForecast} m KeywordPlanWeeklyTimeSeriesForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_plan_campaign != null && Object.hasOwnProperty.call(m, "keyword_plan_campaign"))
                                    w.uint32(10).string(m.keyword_plan_campaign);
                                if (m.weekly_forecasts != null && m.weekly_forecasts.length) {
                                    for (var i = 0; i < m.weekly_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.encode(m.weekly_forecasts[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanWeeklyTimeSeriesForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyTimeSeriesForecast} message KeywordPlanWeeklyTimeSeriesForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanWeeklyTimeSeriesForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast} KeywordPlanWeeklyTimeSeriesForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.keyword_plan_campaign = r.string();
                                        break;
                                    case 2:
                                        if (!(m.weekly_forecasts && m.weekly_forecasts.length))
                                            m.weekly_forecasts = [];
                                        m.weekly_forecasts.push($root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanWeeklyTimeSeriesForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast} KeywordPlanWeeklyTimeSeriesForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanWeeklyTimeSeriesForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    p._keyword_plan_campaign = 1;
                                    if (!$util.isString(m.keyword_plan_campaign))
                                        return "keyword_plan_campaign: string expected";
                                }
                                if (m.weekly_forecasts != null && m.hasOwnProperty("weekly_forecasts")) {
                                    if (!Array.isArray(m.weekly_forecasts))
                                        return "weekly_forecasts: array expected";
                                    for (var i = 0; i < m.weekly_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.verify(m.weekly_forecasts[i]);
                                            if (e)
                                                return "weekly_forecasts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanWeeklyTimeSeriesForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast} KeywordPlanWeeklyTimeSeriesForecast
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast();
                                if (d.keyword_plan_campaign != null) {
                                    m.keyword_plan_campaign = String(d.keyword_plan_campaign);
                                }
                                if (d.weekly_forecasts) {
                                    if (!Array.isArray(d.weekly_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.weekly_forecasts: array expected");
                                    m.weekly_forecasts = [];
                                    for (var i = 0; i < d.weekly_forecasts.length; ++i) {
                                        if (typeof d.weekly_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast.weekly_forecasts: object expected");
                                        m.weekly_forecasts[i] = $root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.fromObject(d.weekly_forecasts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanWeeklyTimeSeriesForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast} m KeywordPlanWeeklyTimeSeriesForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.weekly_forecasts = [];
                                }
                                if (m.keyword_plan_campaign != null && m.hasOwnProperty("keyword_plan_campaign")) {
                                    d.keyword_plan_campaign = m.keyword_plan_campaign;
                                    if (o.oneofs)
                                        d._keyword_plan_campaign = "keyword_plan_campaign";
                                }
                                if (m.weekly_forecasts && m.weekly_forecasts.length) {
                                    d.weekly_forecasts = [];
                                    for (var j = 0; j < m.weekly_forecasts.length; ++j) {
                                        d.weekly_forecasts[j] = $root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.toObject(m.weekly_forecasts[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanWeeklyTimeSeriesForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyTimeSeriesForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanWeeklyTimeSeriesForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanWeeklyTimeSeriesForecast;
                        })();
    
                        services.KeywordPlanWeeklyForecast = (function() {
    
                            /**
                             * Properties of a KeywordPlanWeeklyForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanWeeklyForecast
                             * @property {string|null} [start_date] KeywordPlanWeeklyForecast start_date
                             * @property {google.ads.googleads.v11.services.IForecastMetrics|null} [forecast] KeywordPlanWeeklyForecast forecast
                             */
    
                            /**
                             * Constructs a new KeywordPlanWeeklyForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanWeeklyForecast.
                             * @implements IKeywordPlanWeeklyForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyForecast=} [p] Properties to set
                             */
                            function KeywordPlanWeeklyForecast(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanWeeklyForecast start_date.
                             * @member {string|null|undefined} start_date
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @instance
                             */
                            KeywordPlanWeeklyForecast.prototype.start_date = null;
    
                            /**
                             * KeywordPlanWeeklyForecast forecast.
                             * @member {google.ads.googleads.v11.services.IForecastMetrics|null|undefined} forecast
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @instance
                             */
                            KeywordPlanWeeklyForecast.prototype.forecast = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanWeeklyForecast _start_date.
                             * @member {"start_date"|undefined} _start_date
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanWeeklyForecast.prototype, "_start_date", {
                                get: $util.oneOfGetter($oneOfFields = ["start_date"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanWeeklyForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyForecast} KeywordPlanWeeklyForecast instance
                             */
                            KeywordPlanWeeklyForecast.create = function create(properties) {
                                return new KeywordPlanWeeklyForecast(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanWeeklyForecast message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyForecast} m KeywordPlanWeeklyForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanWeeklyForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.start_date != null && Object.hasOwnProperty.call(m, "start_date"))
                                    w.uint32(10).string(m.start_date);
                                if (m.forecast != null && Object.hasOwnProperty.call(m, "forecast"))
                                    $root.google.ads.googleads.v11.services.ForecastMetrics.encode(m.forecast, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanWeeklyForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanWeeklyForecast} message KeywordPlanWeeklyForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanWeeklyForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanWeeklyForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyForecast} KeywordPlanWeeklyForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanWeeklyForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.start_date = r.string();
                                        break;
                                    case 2:
                                        m.forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanWeeklyForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyForecast} KeywordPlanWeeklyForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanWeeklyForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanWeeklyForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanWeeklyForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    p._start_date = 1;
                                    if (!$util.isString(m.start_date))
                                        return "start_date: string expected";
                                }
                                if (m.forecast != null && m.hasOwnProperty("forecast")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ForecastMetrics.verify(m.forecast);
                                        if (e)
                                            return "forecast." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanWeeklyForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanWeeklyForecast} KeywordPlanWeeklyForecast
                             */
                            KeywordPlanWeeklyForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanWeeklyForecast();
                                if (d.start_date != null) {
                                    m.start_date = String(d.start_date);
                                }
                                if (d.forecast != null) {
                                    if (typeof d.forecast !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanWeeklyForecast.forecast: object expected");
                                    m.forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.fromObject(d.forecast);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanWeeklyForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanWeeklyForecast} m KeywordPlanWeeklyForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanWeeklyForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.forecast = null;
                                }
                                if (m.start_date != null && m.hasOwnProperty("start_date")) {
                                    d.start_date = m.start_date;
                                    if (o.oneofs)
                                        d._start_date = "start_date";
                                }
                                if (m.forecast != null && m.hasOwnProperty("forecast")) {
                                    d.forecast = $root.google.ads.googleads.v11.services.ForecastMetrics.toObject(m.forecast, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanWeeklyForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanWeeklyForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanWeeklyForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanWeeklyForecast;
                        })();
    
                        services.ForecastMetrics = (function() {
    
                            /**
                             * Properties of a ForecastMetrics.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IForecastMetrics
                             * @property {number|null} [impressions] ForecastMetrics impressions
                             * @property {number|null} [ctr] ForecastMetrics ctr
                             * @property {number|null} [average_cpc] ForecastMetrics average_cpc
                             * @property {number|null} [clicks] ForecastMetrics clicks
                             * @property {number|null} [cost_micros] ForecastMetrics cost_micros
                             */
    
                            /**
                             * Constructs a new ForecastMetrics.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ForecastMetrics.
                             * @implements IForecastMetrics
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IForecastMetrics=} [p] Properties to set
                             */
                            function ForecastMetrics(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ForecastMetrics impressions.
                             * @member {number|null|undefined} impressions
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            ForecastMetrics.prototype.impressions = null;
    
                            /**
                             * ForecastMetrics ctr.
                             * @member {number|null|undefined} ctr
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            ForecastMetrics.prototype.ctr = null;
    
                            /**
                             * ForecastMetrics average_cpc.
                             * @member {number|null|undefined} average_cpc
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            ForecastMetrics.prototype.average_cpc = null;
    
                            /**
                             * ForecastMetrics clicks.
                             * @member {number|null|undefined} clicks
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            ForecastMetrics.prototype.clicks = null;
    
                            /**
                             * ForecastMetrics cost_micros.
                             * @member {number|null|undefined} cost_micros
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            ForecastMetrics.prototype.cost_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ForecastMetrics _impressions.
                             * @member {"impressions"|undefined} _impressions
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            Object.defineProperty(ForecastMetrics.prototype, "_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ForecastMetrics _ctr.
                             * @member {"ctr"|undefined} _ctr
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            Object.defineProperty(ForecastMetrics.prototype, "_ctr", {
                                get: $util.oneOfGetter($oneOfFields = ["ctr"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ForecastMetrics _average_cpc.
                             * @member {"average_cpc"|undefined} _average_cpc
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            Object.defineProperty(ForecastMetrics.prototype, "_average_cpc", {
                                get: $util.oneOfGetter($oneOfFields = ["average_cpc"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ForecastMetrics _clicks.
                             * @member {"clicks"|undefined} _clicks
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            Object.defineProperty(ForecastMetrics.prototype, "_clicks", {
                                get: $util.oneOfGetter($oneOfFields = ["clicks"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ForecastMetrics _cost_micros.
                             * @member {"cost_micros"|undefined} _cost_micros
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             */
                            Object.defineProperty(ForecastMetrics.prototype, "_cost_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["cost_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ForecastMetrics instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecastMetrics=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ForecastMetrics} ForecastMetrics instance
                             */
                            ForecastMetrics.create = function create(properties) {
                                return new ForecastMetrics(properties);
                            };
    
                            /**
                             * Encodes the specified ForecastMetrics message. Does not implicitly {@link google.ads.googleads.v11.services.ForecastMetrics.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecastMetrics} m ForecastMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ForecastMetrics.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(57).double(m.impressions);
                                if (m.ctr != null && Object.hasOwnProperty.call(m, "ctr"))
                                    w.uint32(65).double(m.ctr);
                                if (m.average_cpc != null && Object.hasOwnProperty.call(m, "average_cpc"))
                                    w.uint32(72).int64(m.average_cpc);
                                if (m.clicks != null && Object.hasOwnProperty.call(m, "clicks"))
                                    w.uint32(81).double(m.clicks);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(88).int64(m.cost_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ForecastMetrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ForecastMetrics.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecastMetrics} message ForecastMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ForecastMetrics.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ForecastMetrics message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ForecastMetrics} ForecastMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ForecastMetrics.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ForecastMetrics();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 7:
                                        m.impressions = r.double();
                                        break;
                                    case 8:
                                        m.ctr = r.double();
                                        break;
                                    case 9:
                                        m.average_cpc = r.int64();
                                        break;
                                    case 10:
                                        m.clicks = r.double();
                                        break;
                                    case 11:
                                        m.cost_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ForecastMetrics message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ForecastMetrics} ForecastMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ForecastMetrics.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ForecastMetrics message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ForecastMetrics.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    p._impressions = 1;
                                    if (typeof m.impressions !== "number")
                                        return "impressions: number expected";
                                }
                                if (m.ctr != null && m.hasOwnProperty("ctr")) {
                                    p._ctr = 1;
                                    if (typeof m.ctr !== "number")
                                        return "ctr: number expected";
                                }
                                if (m.average_cpc != null && m.hasOwnProperty("average_cpc")) {
                                    p._average_cpc = 1;
                                    if (!$util.isInteger(m.average_cpc) && !(m.average_cpc && $util.isInteger(m.average_cpc.low) && $util.isInteger(m.average_cpc.high)))
                                        return "average_cpc: integer|Long expected";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    p._clicks = 1;
                                    if (typeof m.clicks !== "number")
                                        return "clicks: number expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    p._cost_micros = 1;
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ForecastMetrics message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ForecastMetrics} ForecastMetrics
                             */
                            ForecastMetrics.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ForecastMetrics)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ForecastMetrics();
                                if (d.impressions != null) {
                                    m.impressions = Number(d.impressions);
                                }
                                if (d.ctr != null) {
                                    m.ctr = Number(d.ctr);
                                }
                                if (d.average_cpc != null) {
                                    if ($util.Long)
                                        (m.average_cpc = $util.Long.fromValue(d.average_cpc)).unsigned = false;
                                    else if (typeof d.average_cpc === "string")
                                        m.average_cpc = parseInt(d.average_cpc, 10);
                                    else if (typeof d.average_cpc === "number")
                                        m.average_cpc = d.average_cpc;
                                    else if (typeof d.average_cpc === "object")
                                        m.average_cpc = new $util.LongBits(d.average_cpc.low >>> 0, d.average_cpc.high >>> 0).toNumber();
                                }
                                if (d.clicks != null) {
                                    m.clicks = Number(d.clicks);
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ForecastMetrics message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.ForecastMetrics} m ForecastMetrics
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ForecastMetrics.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    d.impressions = o.json && !isFinite(m.impressions) ? String(m.impressions) : m.impressions;
                                    if (o.oneofs)
                                        d._impressions = "impressions";
                                }
                                if (m.ctr != null && m.hasOwnProperty("ctr")) {
                                    d.ctr = o.json && !isFinite(m.ctr) ? String(m.ctr) : m.ctr;
                                    if (o.oneofs)
                                        d._ctr = "ctr";
                                }
                                if (m.average_cpc != null && m.hasOwnProperty("average_cpc")) {
                                    if (typeof m.average_cpc === "number")
                                        d.average_cpc = o.longs === String ? String(m.average_cpc) : m.average_cpc;
                                    else
                                        d.average_cpc = o.longs === String ? $util.Long.prototype.toString.call(m.average_cpc) : o.longs === Number ? new $util.LongBits(m.average_cpc.low >>> 0, m.average_cpc.high >>> 0).toNumber() : m.average_cpc;
                                    if (o.oneofs)
                                        d._average_cpc = "average_cpc";
                                }
                                if (m.clicks != null && m.hasOwnProperty("clicks")) {
                                    d.clicks = o.json && !isFinite(m.clicks) ? String(m.clicks) : m.clicks;
                                    if (o.oneofs)
                                        d._clicks = "clicks";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                    if (o.oneofs)
                                        d._cost_micros = "cost_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ForecastMetrics to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ForecastMetrics
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ForecastMetrics.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ForecastMetrics;
                        })();
    
                        services.GenerateHistoricalMetricsRequest = (function() {
    
                            /**
                             * Properties of a GenerateHistoricalMetricsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateHistoricalMetricsRequest
                             * @property {string|null} [keyword_plan] GenerateHistoricalMetricsRequest keyword_plan
                             * @property {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics|null} [aggregate_metrics] GenerateHistoricalMetricsRequest aggregate_metrics
                             * @property {google.ads.googleads.v11.common.IHistoricalMetricsOptions|null} [historical_metrics_options] GenerateHistoricalMetricsRequest historical_metrics_options
                             */
    
                            /**
                             * Constructs a new GenerateHistoricalMetricsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateHistoricalMetricsRequest.
                             * @implements IGenerateHistoricalMetricsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsRequest=} [p] Properties to set
                             */
                            function GenerateHistoricalMetricsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateHistoricalMetricsRequest keyword_plan.
                             * @member {string} keyword_plan
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateHistoricalMetricsRequest.prototype.keyword_plan = "";
    
                            /**
                             * GenerateHistoricalMetricsRequest aggregate_metrics.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics|null|undefined} aggregate_metrics
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateHistoricalMetricsRequest.prototype.aggregate_metrics = null;
    
                            /**
                             * GenerateHistoricalMetricsRequest historical_metrics_options.
                             * @member {google.ads.googleads.v11.common.IHistoricalMetricsOptions|null|undefined} historical_metrics_options
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateHistoricalMetricsRequest.prototype.historical_metrics_options = null;
    
                            /**
                             * Creates a new GenerateHistoricalMetricsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest} GenerateHistoricalMetricsRequest instance
                             */
                            GenerateHistoricalMetricsRequest.create = function create(properties) {
                                return new GenerateHistoricalMetricsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateHistoricalMetricsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsRequest} m GenerateHistoricalMetricsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateHistoricalMetricsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_plan != null && Object.hasOwnProperty.call(m, "keyword_plan"))
                                    w.uint32(10).string(m.keyword_plan);
                                if (m.aggregate_metrics != null && Object.hasOwnProperty.call(m, "aggregate_metrics"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.encode(m.aggregate_metrics, w.uint32(18).fork()).ldelim();
                                if (m.historical_metrics_options != null && Object.hasOwnProperty.call(m, "historical_metrics_options"))
                                    $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.encode(m.historical_metrics_options, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateHistoricalMetricsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsRequest} message GenerateHistoricalMetricsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateHistoricalMetricsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateHistoricalMetricsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest} GenerateHistoricalMetricsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateHistoricalMetricsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.keyword_plan = r.string();
                                        break;
                                    case 2:
                                        m.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateHistoricalMetricsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest} GenerateHistoricalMetricsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateHistoricalMetricsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateHistoricalMetricsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateHistoricalMetricsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    if (!$util.isString(m.keyword_plan))
                                        return "keyword_plan: string expected";
                                }
                                if (m.aggregate_metrics != null && m.hasOwnProperty("aggregate_metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.verify(m.aggregate_metrics);
                                        if (e)
                                            return "aggregate_metrics." + e;
                                    }
                                }
                                if (m.historical_metrics_options != null && m.hasOwnProperty("historical_metrics_options")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.verify(m.historical_metrics_options);
                                        if (e)
                                            return "historical_metrics_options." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateHistoricalMetricsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest} GenerateHistoricalMetricsRequest
                             */
                            GenerateHistoricalMetricsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest();
                                if (d.keyword_plan != null) {
                                    m.keyword_plan = String(d.keyword_plan);
                                }
                                if (d.aggregate_metrics != null) {
                                    if (typeof d.aggregate_metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest.aggregate_metrics: object expected");
                                    m.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.fromObject(d.aggregate_metrics);
                                }
                                if (d.historical_metrics_options != null) {
                                    if (typeof d.historical_metrics_options !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest.historical_metrics_options: object expected");
                                    m.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.fromObject(d.historical_metrics_options);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateHistoricalMetricsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest} m GenerateHistoricalMetricsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateHistoricalMetricsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.keyword_plan = "";
                                    d.aggregate_metrics = null;
                                    d.historical_metrics_options = null;
                                }
                                if (m.keyword_plan != null && m.hasOwnProperty("keyword_plan")) {
                                    d.keyword_plan = m.keyword_plan;
                                }
                                if (m.aggregate_metrics != null && m.hasOwnProperty("aggregate_metrics")) {
                                    d.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.toObject(m.aggregate_metrics, o);
                                }
                                if (m.historical_metrics_options != null && m.hasOwnProperty("historical_metrics_options")) {
                                    d.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.toObject(m.historical_metrics_options, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateHistoricalMetricsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateHistoricalMetricsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateHistoricalMetricsRequest;
                        })();
    
                        services.GenerateHistoricalMetricsResponse = (function() {
    
                            /**
                             * Properties of a GenerateHistoricalMetricsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateHistoricalMetricsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IKeywordPlanKeywordHistoricalMetrics>|null} [metrics] GenerateHistoricalMetricsResponse metrics
                             * @property {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults|null} [aggregate_metric_results] GenerateHistoricalMetricsResponse aggregate_metric_results
                             */
    
                            /**
                             * Constructs a new GenerateHistoricalMetricsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateHistoricalMetricsResponse.
                             * @implements IGenerateHistoricalMetricsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsResponse=} [p] Properties to set
                             */
                            function GenerateHistoricalMetricsResponse(p) {
                                this.metrics = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateHistoricalMetricsResponse metrics.
                             * @member {Array.<google.ads.googleads.v11.services.IKeywordPlanKeywordHistoricalMetrics>} metrics
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @instance
                             */
                            GenerateHistoricalMetricsResponse.prototype.metrics = $util.emptyArray;
    
                            /**
                             * GenerateHistoricalMetricsResponse aggregate_metric_results.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults|null|undefined} aggregate_metric_results
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @instance
                             */
                            GenerateHistoricalMetricsResponse.prototype.aggregate_metric_results = null;
    
                            /**
                             * Creates a new GenerateHistoricalMetricsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse} GenerateHistoricalMetricsResponse instance
                             */
                            GenerateHistoricalMetricsResponse.create = function create(properties) {
                                return new GenerateHistoricalMetricsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateHistoricalMetricsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsResponse} m GenerateHistoricalMetricsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateHistoricalMetricsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.metrics != null && m.metrics.length) {
                                    for (var i = 0; i < m.metrics.length; ++i)
                                        $root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.encode(m.metrics[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.aggregate_metric_results != null && Object.hasOwnProperty.call(m, "aggregate_metric_results"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.encode(m.aggregate_metric_results, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateHistoricalMetricsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateHistoricalMetricsResponse} message GenerateHistoricalMetricsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateHistoricalMetricsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateHistoricalMetricsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse} GenerateHistoricalMetricsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateHistoricalMetricsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.metrics && m.metrics.length))
                                            m.metrics = [];
                                        m.metrics.push($root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateHistoricalMetricsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse} GenerateHistoricalMetricsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateHistoricalMetricsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateHistoricalMetricsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateHistoricalMetricsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.metrics != null && m.hasOwnProperty("metrics")) {
                                    if (!Array.isArray(m.metrics))
                                        return "metrics: array expected";
                                    for (var i = 0; i < m.metrics.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.verify(m.metrics[i]);
                                            if (e)
                                                return "metrics." + e;
                                        }
                                    }
                                }
                                if (m.aggregate_metric_results != null && m.hasOwnProperty("aggregate_metric_results")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.verify(m.aggregate_metric_results);
                                        if (e)
                                            return "aggregate_metric_results." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateHistoricalMetricsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse} GenerateHistoricalMetricsResponse
                             */
                            GenerateHistoricalMetricsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse();
                                if (d.metrics) {
                                    if (!Array.isArray(d.metrics))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse.metrics: array expected");
                                    m.metrics = [];
                                    for (var i = 0; i < d.metrics.length; ++i) {
                                        if (typeof d.metrics[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse.metrics: object expected");
                                        m.metrics[i] = $root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.fromObject(d.metrics[i]);
                                    }
                                }
                                if (d.aggregate_metric_results != null) {
                                    if (typeof d.aggregate_metric_results !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse.aggregate_metric_results: object expected");
                                    m.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.fromObject(d.aggregate_metric_results);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateHistoricalMetricsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse} m GenerateHistoricalMetricsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateHistoricalMetricsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.metrics = [];
                                }
                                if (o.defaults) {
                                    d.aggregate_metric_results = null;
                                }
                                if (m.metrics && m.metrics.length) {
                                    d.metrics = [];
                                    for (var j = 0; j < m.metrics.length; ++j) {
                                        d.metrics[j] = $root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.toObject(m.metrics[j], o);
                                    }
                                }
                                if (m.aggregate_metric_results != null && m.hasOwnProperty("aggregate_metric_results")) {
                                    d.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.toObject(m.aggregate_metric_results, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateHistoricalMetricsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateHistoricalMetricsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateHistoricalMetricsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateHistoricalMetricsResponse;
                        })();
    
                        services.KeywordPlanKeywordHistoricalMetrics = (function() {
    
                            /**
                             * Properties of a KeywordPlanKeywordHistoricalMetrics.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordPlanKeywordHistoricalMetrics
                             * @property {string|null} [search_query] KeywordPlanKeywordHistoricalMetrics search_query
                             * @property {Array.<string>|null} [close_variants] KeywordPlanKeywordHistoricalMetrics close_variants
                             * @property {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics|null} [keyword_metrics] KeywordPlanKeywordHistoricalMetrics keyword_metrics
                             */
    
                            /**
                             * Constructs a new KeywordPlanKeywordHistoricalMetrics.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanKeywordHistoricalMetrics.
                             * @implements IKeywordPlanKeywordHistoricalMetrics
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordHistoricalMetrics=} [p] Properties to set
                             */
                            function KeywordPlanKeywordHistoricalMetrics(p) {
                                this.close_variants = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordPlanKeywordHistoricalMetrics search_query.
                             * @member {string|null|undefined} search_query
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanKeywordHistoricalMetrics.prototype.search_query = null;
    
                            /**
                             * KeywordPlanKeywordHistoricalMetrics close_variants.
                             * @member {Array.<string>} close_variants
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanKeywordHistoricalMetrics.prototype.close_variants = $util.emptyArray;
    
                            /**
                             * KeywordPlanKeywordHistoricalMetrics keyword_metrics.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics|null|undefined} keyword_metrics
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @instance
                             */
                            KeywordPlanKeywordHistoricalMetrics.prototype.keyword_metrics = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordPlanKeywordHistoricalMetrics _search_query.
                             * @member {"search_query"|undefined} _search_query
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @instance
                             */
                            Object.defineProperty(KeywordPlanKeywordHistoricalMetrics.prototype, "_search_query", {
                                get: $util.oneOfGetter($oneOfFields = ["search_query"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordPlanKeywordHistoricalMetrics instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordHistoricalMetrics=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics} KeywordPlanKeywordHistoricalMetrics instance
                             */
                            KeywordPlanKeywordHistoricalMetrics.create = function create(properties) {
                                return new KeywordPlanKeywordHistoricalMetrics(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordPlanKeywordHistoricalMetrics message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordHistoricalMetrics} m KeywordPlanKeywordHistoricalMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanKeywordHistoricalMetrics.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_metrics != null && Object.hasOwnProperty.call(m, "keyword_metrics"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.encode(m.keyword_metrics, w.uint32(18).fork()).ldelim();
                                if (m.close_variants != null && m.close_variants.length) {
                                    for (var i = 0; i < m.close_variants.length; ++i)
                                        w.uint32(26).string(m.close_variants[i]);
                                }
                                if (m.search_query != null && Object.hasOwnProperty.call(m, "search_query"))
                                    w.uint32(34).string(m.search_query);
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordPlanKeywordHistoricalMetrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordPlanKeywordHistoricalMetrics} message KeywordPlanKeywordHistoricalMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordPlanKeywordHistoricalMetrics.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordPlanKeywordHistoricalMetrics message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics} KeywordPlanKeywordHistoricalMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanKeywordHistoricalMetrics.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.search_query = r.string();
                                        break;
                                    case 3:
                                        if (!(m.close_variants && m.close_variants.length))
                                            m.close_variants = [];
                                        m.close_variants.push(r.string());
                                        break;
                                    case 2:
                                        m.keyword_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordPlanKeywordHistoricalMetrics message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics} KeywordPlanKeywordHistoricalMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordPlanKeywordHistoricalMetrics.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordPlanKeywordHistoricalMetrics message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordPlanKeywordHistoricalMetrics.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.search_query != null && m.hasOwnProperty("search_query")) {
                                    p._search_query = 1;
                                    if (!$util.isString(m.search_query))
                                        return "search_query: string expected";
                                }
                                if (m.close_variants != null && m.hasOwnProperty("close_variants")) {
                                    if (!Array.isArray(m.close_variants))
                                        return "close_variants: array expected";
                                    for (var i = 0; i < m.close_variants.length; ++i) {
                                        if (!$util.isString(m.close_variants[i]))
                                            return "close_variants: string[] expected";
                                    }
                                }
                                if (m.keyword_metrics != null && m.hasOwnProperty("keyword_metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.verify(m.keyword_metrics);
                                        if (e)
                                            return "keyword_metrics." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordPlanKeywordHistoricalMetrics message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics} KeywordPlanKeywordHistoricalMetrics
                             */
                            KeywordPlanKeywordHistoricalMetrics.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics();
                                if (d.search_query != null) {
                                    m.search_query = String(d.search_query);
                                }
                                if (d.close_variants) {
                                    if (!Array.isArray(d.close_variants))
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.close_variants: array expected");
                                    m.close_variants = [];
                                    for (var i = 0; i < d.close_variants.length; ++i) {
                                        m.close_variants[i] = String(d.close_variants[i]);
                                    }
                                }
                                if (d.keyword_metrics != null) {
                                    if (typeof d.keyword_metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics.keyword_metrics: object expected");
                                    m.keyword_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.fromObject(d.keyword_metrics);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordPlanKeywordHistoricalMetrics message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics} m KeywordPlanKeywordHistoricalMetrics
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordPlanKeywordHistoricalMetrics.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.close_variants = [];
                                }
                                if (o.defaults) {
                                    d.keyword_metrics = null;
                                }
                                if (m.keyword_metrics != null && m.hasOwnProperty("keyword_metrics")) {
                                    d.keyword_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.toObject(m.keyword_metrics, o);
                                }
                                if (m.close_variants && m.close_variants.length) {
                                    d.close_variants = [];
                                    for (var j = 0; j < m.close_variants.length; ++j) {
                                        d.close_variants[j] = m.close_variants[j];
                                    }
                                }
                                if (m.search_query != null && m.hasOwnProperty("search_query")) {
                                    d.search_query = m.search_query;
                                    if (o.oneofs)
                                        d._search_query = "search_query";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordPlanKeywordHistoricalMetrics to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordPlanKeywordHistoricalMetrics
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordPlanKeywordHistoricalMetrics.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordPlanKeywordHistoricalMetrics;
                        })();
    
                        services.LabelService = (function() {
    
                            /**
                             * Constructs a new LabelService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a LabelService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function LabelService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (LabelService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = LabelService;
    
                            /**
                             * Creates new LabelService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.LabelService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {LabelService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            LabelService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.LabelService|mutateLabels}.
                             * @memberof google.ads.googleads.v11.services.LabelService
                             * @typedef MutateLabelsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateLabelsResponse} [response] MutateLabelsResponse
                             */
    
                            /**
                             * Calls MutateLabels.
                             * @function mutateLabels
                             * @memberof google.ads.googleads.v11.services.LabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateLabelsRequest} request MutateLabelsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.LabelService.MutateLabelsCallback} callback Node-style callback called with the error, if any, and MutateLabelsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(LabelService.prototype.mutateLabels = function mutateLabels(request, callback) {
                                return this.rpcCall(mutateLabels, $root.google.ads.googleads.v11.services.MutateLabelsRequest, $root.google.ads.googleads.v11.services.MutateLabelsResponse, request, callback);
                            }, "name", { value: "MutateLabels" });
    
                            /**
                             * Calls MutateLabels.
                             * @function mutateLabels
                             * @memberof google.ads.googleads.v11.services.LabelService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateLabelsRequest} request MutateLabelsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateLabelsResponse>} Promise
                             * @variation 2
                             */
    
                            return LabelService;
                        })();
    
                        services.MutateLabelsRequest = (function() {
    
                            /**
                             * Properties of a MutateLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateLabelsRequest
                             * @property {string|null} [customer_id] MutateLabelsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ILabelOperation>|null} [operations] MutateLabelsRequest operations
                             * @property {boolean|null} [partial_failure] MutateLabelsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateLabelsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateLabelsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateLabelsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateLabelsRequest.
                             * @implements IMutateLabelsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateLabelsRequest=} [p] Properties to set
                             */
                            function MutateLabelsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateLabelsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @instance
                             */
                            MutateLabelsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateLabelsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ILabelOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @instance
                             */
                            MutateLabelsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateLabelsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @instance
                             */
                            MutateLabelsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateLabelsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @instance
                             */
                            MutateLabelsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateLabelsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @instance
                             */
                            MutateLabelsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateLabelsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateLabelsRequest} MutateLabelsRequest instance
                             */
                            MutateLabelsRequest.create = function create(properties) {
                                return new MutateLabelsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateLabelsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateLabelsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelsRequest} m MutateLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateLabelsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.LabelOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateLabelsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateLabelsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelsRequest} message MutateLabelsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateLabelsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateLabelsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateLabelsRequest} MutateLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateLabelsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateLabelsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.LabelOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateLabelsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateLabelsRequest} MutateLabelsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateLabelsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateLabelsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateLabelsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.LabelOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateLabelsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateLabelsRequest} MutateLabelsRequest
                             */
                            MutateLabelsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateLabelsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateLabelsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateLabelsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateLabelsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.LabelOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateLabelsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateLabelsRequest} m MutateLabelsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateLabelsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.LabelOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateLabelsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateLabelsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateLabelsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateLabelsRequest;
                        })();
    
                        services.LabelOperation = (function() {
    
                            /**
                             * Properties of a LabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ILabelOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] LabelOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ILabel|null} [create] LabelOperation create
                             * @property {google.ads.googleads.v11.resources.ILabel|null} [update] LabelOperation update
                             * @property {string|null} [remove] LabelOperation remove
                             */
    
                            /**
                             * Constructs a new LabelOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a LabelOperation.
                             * @implements ILabelOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ILabelOperation=} [p] Properties to set
                             */
                            function LabelOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * LabelOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @instance
                             */
                            LabelOperation.prototype.update_mask = null;
    
                            /**
                             * LabelOperation create.
                             * @member {google.ads.googleads.v11.resources.ILabel|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @instance
                             */
                            LabelOperation.prototype.create = null;
    
                            /**
                             * LabelOperation update.
                             * @member {google.ads.googleads.v11.resources.ILabel|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @instance
                             */
                            LabelOperation.prototype.update = null;
    
                            /**
                             * LabelOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @instance
                             */
                            LabelOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * LabelOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @instance
                             */
                            Object.defineProperty(LabelOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new LabelOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ILabelOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.LabelOperation} LabelOperation instance
                             */
                            LabelOperation.create = function create(properties) {
                                return new LabelOperation(properties);
                            };
    
                            /**
                             * Encodes the specified LabelOperation message. Does not implicitly {@link google.ads.googleads.v11.services.LabelOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ILabelOperation} m LabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LabelOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.Label.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.Label.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified LabelOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.LabelOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ILabelOperation} message LabelOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            LabelOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a LabelOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.LabelOperation} LabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LabelOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.LabelOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.Label.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.Label.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a LabelOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.LabelOperation} LabelOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            LabelOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a LabelOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            LabelOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Label.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Label.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a LabelOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.LabelOperation} LabelOperation
                             */
                            LabelOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.LabelOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.LabelOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.LabelOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.LabelOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.Label.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.LabelOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.Label.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a LabelOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.LabelOperation} m LabelOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            LabelOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.Label.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.Label.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this LabelOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.LabelOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            LabelOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return LabelOperation;
                        })();
    
                        services.MutateLabelsResponse = (function() {
    
                            /**
                             * Properties of a MutateLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateLabelsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateLabelsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateLabelResult>|null} [results] MutateLabelsResponse results
                             */
    
                            /**
                             * Constructs a new MutateLabelsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateLabelsResponse.
                             * @implements IMutateLabelsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateLabelsResponse=} [p] Properties to set
                             */
                            function MutateLabelsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateLabelsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @instance
                             */
                            MutateLabelsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateLabelsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateLabelResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @instance
                             */
                            MutateLabelsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateLabelsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateLabelsResponse} MutateLabelsResponse instance
                             */
                            MutateLabelsResponse.create = function create(properties) {
                                return new MutateLabelsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateLabelsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateLabelsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelsResponse} m MutateLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateLabelsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateLabelResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateLabelsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateLabelsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelsResponse} message MutateLabelsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateLabelsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateLabelsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateLabelsResponse} MutateLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateLabelsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateLabelsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateLabelResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateLabelsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateLabelsResponse} MutateLabelsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateLabelsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateLabelsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateLabelsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateLabelResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateLabelsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateLabelsResponse} MutateLabelsResponse
                             */
                            MutateLabelsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateLabelsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateLabelsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateLabelsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateLabelsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateLabelsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateLabelResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateLabelsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateLabelsResponse} m MutateLabelsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateLabelsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateLabelResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateLabelsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateLabelsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateLabelsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateLabelsResponse;
                        })();
    
                        services.MutateLabelResult = (function() {
    
                            /**
                             * Properties of a MutateLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateLabelResult
                             * @property {string|null} [resource_name] MutateLabelResult resource_name
                             * @property {google.ads.googleads.v11.resources.ILabel|null} [label] MutateLabelResult label
                             */
    
                            /**
                             * Constructs a new MutateLabelResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateLabelResult.
                             * @implements IMutateLabelResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateLabelResult=} [p] Properties to set
                             */
                            function MutateLabelResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateLabelResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @instance
                             */
                            MutateLabelResult.prototype.resource_name = "";
    
                            /**
                             * MutateLabelResult label.
                             * @member {google.ads.googleads.v11.resources.ILabel|null|undefined} label
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @instance
                             */
                            MutateLabelResult.prototype.label = null;
    
                            /**
                             * Creates a new MutateLabelResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateLabelResult} MutateLabelResult instance
                             */
                            MutateLabelResult.create = function create(properties) {
                                return new MutateLabelResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateLabelResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateLabelResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelResult} m MutateLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateLabelResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                                    $root.google.ads.googleads.v11.resources.Label.encode(m.label, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateLabelResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateLabelResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateLabelResult} message MutateLabelResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateLabelResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateLabelResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateLabelResult} MutateLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateLabelResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateLabelResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.label = $root.google.ads.googleads.v11.resources.Label.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateLabelResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateLabelResult} MutateLabelResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateLabelResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateLabelResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateLabelResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.Label.verify(m.label);
                                        if (e)
                                            return "label." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateLabelResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateLabelResult} MutateLabelResult
                             */
                            MutateLabelResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateLabelResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateLabelResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.label != null) {
                                    if (typeof d.label !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateLabelResult.label: object expected");
                                    m.label = $root.google.ads.googleads.v11.resources.Label.fromObject(d.label);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateLabelResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateLabelResult} m MutateLabelResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateLabelResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.label = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.label != null && m.hasOwnProperty("label")) {
                                    d.label = $root.google.ads.googleads.v11.resources.Label.toObject(m.label, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateLabelResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateLabelResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateLabelResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateLabelResult;
                        })();
    
                        services.MediaFileService = (function() {
    
                            /**
                             * Constructs a new MediaFileService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MediaFileService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function MediaFileService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (MediaFileService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = MediaFileService;
    
                            /**
                             * Creates new MediaFileService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MediaFileService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {MediaFileService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            MediaFileService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.MediaFileService|mutateMediaFiles}.
                             * @memberof google.ads.googleads.v11.services.MediaFileService
                             * @typedef MutateMediaFilesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateMediaFilesResponse} [response] MutateMediaFilesResponse
                             */
    
                            /**
                             * Calls MutateMediaFiles.
                             * @function mutateMediaFiles
                             * @memberof google.ads.googleads.v11.services.MediaFileService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesRequest} request MutateMediaFilesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.MediaFileService.MutateMediaFilesCallback} callback Node-style callback called with the error, if any, and MutateMediaFilesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(MediaFileService.prototype.mutateMediaFiles = function mutateMediaFiles(request, callback) {
                                return this.rpcCall(mutateMediaFiles, $root.google.ads.googleads.v11.services.MutateMediaFilesRequest, $root.google.ads.googleads.v11.services.MutateMediaFilesResponse, request, callback);
                            }, "name", { value: "MutateMediaFiles" });
    
                            /**
                             * Calls MutateMediaFiles.
                             * @function mutateMediaFiles
                             * @memberof google.ads.googleads.v11.services.MediaFileService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesRequest} request MutateMediaFilesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateMediaFilesResponse>} Promise
                             * @variation 2
                             */
    
                            return MediaFileService;
                        })();
    
                        services.MutateMediaFilesRequest = (function() {
    
                            /**
                             * Properties of a MutateMediaFilesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateMediaFilesRequest
                             * @property {string|null} [customer_id] MutateMediaFilesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IMediaFileOperation>|null} [operations] MutateMediaFilesRequest operations
                             * @property {boolean|null} [partial_failure] MutateMediaFilesRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateMediaFilesRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateMediaFilesRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateMediaFilesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateMediaFilesRequest.
                             * @implements IMutateMediaFilesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesRequest=} [p] Properties to set
                             */
                            function MutateMediaFilesRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateMediaFilesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @instance
                             */
                            MutateMediaFilesRequest.prototype.customer_id = "";
    
                            /**
                             * MutateMediaFilesRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IMediaFileOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @instance
                             */
                            MutateMediaFilesRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateMediaFilesRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @instance
                             */
                            MutateMediaFilesRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateMediaFilesRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @instance
                             */
                            MutateMediaFilesRequest.prototype.validate_only = false;
    
                            /**
                             * MutateMediaFilesRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @instance
                             */
                            MutateMediaFilesRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateMediaFilesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesRequest} MutateMediaFilesRequest instance
                             */
                            MutateMediaFilesRequest.create = function create(properties) {
                                return new MutateMediaFilesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateMediaFilesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateMediaFilesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesRequest} m MutateMediaFilesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMediaFilesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.MediaFileOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateMediaFilesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateMediaFilesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesRequest} message MutateMediaFilesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMediaFilesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateMediaFilesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesRequest} MutateMediaFilesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMediaFilesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateMediaFilesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.MediaFileOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateMediaFilesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesRequest} MutateMediaFilesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMediaFilesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateMediaFilesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateMediaFilesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MediaFileOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateMediaFilesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesRequest} MutateMediaFilesRequest
                             */
                            MutateMediaFilesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateMediaFilesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateMediaFilesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateMediaFilesRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateMediaFilesRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.MediaFileOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateMediaFilesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateMediaFilesRequest} m MutateMediaFilesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateMediaFilesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.MediaFileOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateMediaFilesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateMediaFilesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateMediaFilesRequest;
                        })();
    
                        services.MediaFileOperation = (function() {
    
                            /**
                             * Properties of a MediaFileOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMediaFileOperation
                             * @property {google.ads.googleads.v11.resources.IMediaFile|null} [create] MediaFileOperation create
                             */
    
                            /**
                             * Constructs a new MediaFileOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MediaFileOperation.
                             * @implements IMediaFileOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMediaFileOperation=} [p] Properties to set
                             */
                            function MediaFileOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MediaFileOperation create.
                             * @member {google.ads.googleads.v11.resources.IMediaFile|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @instance
                             */
                            MediaFileOperation.prototype.create = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MediaFileOperation operation.
                             * @member {"create"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @instance
                             */
                            Object.defineProperty(MediaFileOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MediaFileOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMediaFileOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MediaFileOperation} MediaFileOperation instance
                             */
                            MediaFileOperation.create = function create(properties) {
                                return new MediaFileOperation(properties);
                            };
    
                            /**
                             * Encodes the specified MediaFileOperation message. Does not implicitly {@link google.ads.googleads.v11.services.MediaFileOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMediaFileOperation} m MediaFileOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaFileOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.MediaFile.encode(m.create, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MediaFileOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MediaFileOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMediaFileOperation} message MediaFileOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MediaFileOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MediaFileOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MediaFileOperation} MediaFileOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaFileOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MediaFileOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.MediaFile.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MediaFileOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MediaFileOperation} MediaFileOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MediaFileOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MediaFileOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MediaFileOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MediaFile.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MediaFileOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MediaFileOperation} MediaFileOperation
                             */
                            MediaFileOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MediaFileOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MediaFileOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MediaFileOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.MediaFile.fromObject(d.create);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MediaFileOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.MediaFileOperation} m MediaFileOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MediaFileOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.MediaFile.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MediaFileOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MediaFileOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MediaFileOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MediaFileOperation;
                        })();
    
                        services.MutateMediaFilesResponse = (function() {
    
                            /**
                             * Properties of a MutateMediaFilesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateMediaFilesResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateMediaFilesResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateMediaFileResult>|null} [results] MutateMediaFilesResponse results
                             */
    
                            /**
                             * Constructs a new MutateMediaFilesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateMediaFilesResponse.
                             * @implements IMutateMediaFilesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesResponse=} [p] Properties to set
                             */
                            function MutateMediaFilesResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateMediaFilesResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @instance
                             */
                            MutateMediaFilesResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateMediaFilesResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateMediaFileResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @instance
                             */
                            MutateMediaFilesResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateMediaFilesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesResponse} MutateMediaFilesResponse instance
                             */
                            MutateMediaFilesResponse.create = function create(properties) {
                                return new MutateMediaFilesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateMediaFilesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateMediaFilesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesResponse} m MutateMediaFilesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMediaFilesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateMediaFileResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateMediaFilesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateMediaFilesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFilesResponse} message MutateMediaFilesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMediaFilesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateMediaFilesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesResponse} MutateMediaFilesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMediaFilesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateMediaFilesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateMediaFileResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateMediaFilesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesResponse} MutateMediaFilesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMediaFilesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateMediaFilesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateMediaFilesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateMediaFileResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateMediaFilesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateMediaFilesResponse} MutateMediaFilesResponse
                             */
                            MutateMediaFilesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateMediaFilesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateMediaFilesResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateMediaFilesResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateMediaFilesResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateMediaFilesResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateMediaFileResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateMediaFilesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateMediaFilesResponse} m MutateMediaFilesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateMediaFilesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateMediaFileResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateMediaFilesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateMediaFilesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateMediaFilesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateMediaFilesResponse;
                        })();
    
                        services.MutateMediaFileResult = (function() {
    
                            /**
                             * Properties of a MutateMediaFileResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateMediaFileResult
                             * @property {string|null} [resource_name] MutateMediaFileResult resource_name
                             * @property {google.ads.googleads.v11.resources.IMediaFile|null} [media_file] MutateMediaFileResult media_file
                             */
    
                            /**
                             * Constructs a new MutateMediaFileResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateMediaFileResult.
                             * @implements IMutateMediaFileResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateMediaFileResult=} [p] Properties to set
                             */
                            function MutateMediaFileResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateMediaFileResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @instance
                             */
                            MutateMediaFileResult.prototype.resource_name = "";
    
                            /**
                             * MutateMediaFileResult media_file.
                             * @member {google.ads.googleads.v11.resources.IMediaFile|null|undefined} media_file
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @instance
                             */
                            MutateMediaFileResult.prototype.media_file = null;
    
                            /**
                             * Creates a new MutateMediaFileResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFileResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateMediaFileResult} MutateMediaFileResult instance
                             */
                            MutateMediaFileResult.create = function create(properties) {
                                return new MutateMediaFileResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateMediaFileResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateMediaFileResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFileResult} m MutateMediaFileResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMediaFileResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.media_file != null && Object.hasOwnProperty.call(m, "media_file"))
                                    $root.google.ads.googleads.v11.resources.MediaFile.encode(m.media_file, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateMediaFileResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateMediaFileResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMediaFileResult} message MutateMediaFileResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMediaFileResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateMediaFileResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateMediaFileResult} MutateMediaFileResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMediaFileResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateMediaFileResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.media_file = $root.google.ads.googleads.v11.resources.MediaFile.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateMediaFileResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateMediaFileResult} MutateMediaFileResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMediaFileResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateMediaFileResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateMediaFileResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.media_file != null && m.hasOwnProperty("media_file")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MediaFile.verify(m.media_file);
                                        if (e)
                                            return "media_file." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateMediaFileResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateMediaFileResult} MutateMediaFileResult
                             */
                            MutateMediaFileResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateMediaFileResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateMediaFileResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.media_file != null) {
                                    if (typeof d.media_file !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateMediaFileResult.media_file: object expected");
                                    m.media_file = $root.google.ads.googleads.v11.resources.MediaFile.fromObject(d.media_file);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateMediaFileResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateMediaFileResult} m MutateMediaFileResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateMediaFileResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.media_file = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.media_file != null && m.hasOwnProperty("media_file")) {
                                    d.media_file = $root.google.ads.googleads.v11.resources.MediaFile.toObject(m.media_file, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateMediaFileResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateMediaFileResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateMediaFileResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateMediaFileResult;
                        })();
    
                        services.RemarketingActionService = (function() {
    
                            /**
                             * Constructs a new RemarketingActionService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RemarketingActionService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function RemarketingActionService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (RemarketingActionService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = RemarketingActionService;
    
                            /**
                             * Creates new RemarketingActionService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RemarketingActionService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {RemarketingActionService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            RemarketingActionService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.RemarketingActionService|mutateRemarketingActions}.
                             * @memberof google.ads.googleads.v11.services.RemarketingActionService
                             * @typedef MutateRemarketingActionsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateRemarketingActionsResponse} [response] MutateRemarketingActionsResponse
                             */
    
                            /**
                             * Calls MutateRemarketingActions.
                             * @function mutateRemarketingActions
                             * @memberof google.ads.googleads.v11.services.RemarketingActionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsRequest} request MutateRemarketingActionsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.RemarketingActionService.MutateRemarketingActionsCallback} callback Node-style callback called with the error, if any, and MutateRemarketingActionsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(RemarketingActionService.prototype.mutateRemarketingActions = function mutateRemarketingActions(request, callback) {
                                return this.rpcCall(mutateRemarketingActions, $root.google.ads.googleads.v11.services.MutateRemarketingActionsRequest, $root.google.ads.googleads.v11.services.MutateRemarketingActionsResponse, request, callback);
                            }, "name", { value: "MutateRemarketingActions" });
    
                            /**
                             * Calls MutateRemarketingActions.
                             * @function mutateRemarketingActions
                             * @memberof google.ads.googleads.v11.services.RemarketingActionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsRequest} request MutateRemarketingActionsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateRemarketingActionsResponse>} Promise
                             * @variation 2
                             */
    
                            return RemarketingActionService;
                        })();
    
                        services.MutateRemarketingActionsRequest = (function() {
    
                            /**
                             * Properties of a MutateRemarketingActionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateRemarketingActionsRequest
                             * @property {string|null} [customer_id] MutateRemarketingActionsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IRemarketingActionOperation>|null} [operations] MutateRemarketingActionsRequest operations
                             * @property {boolean|null} [partial_failure] MutateRemarketingActionsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateRemarketingActionsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateRemarketingActionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateRemarketingActionsRequest.
                             * @implements IMutateRemarketingActionsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsRequest=} [p] Properties to set
                             */
                            function MutateRemarketingActionsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateRemarketingActionsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @instance
                             */
                            MutateRemarketingActionsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateRemarketingActionsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IRemarketingActionOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @instance
                             */
                            MutateRemarketingActionsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateRemarketingActionsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @instance
                             */
                            MutateRemarketingActionsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateRemarketingActionsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @instance
                             */
                            MutateRemarketingActionsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateRemarketingActionsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsRequest} MutateRemarketingActionsRequest instance
                             */
                            MutateRemarketingActionsRequest.create = function create(properties) {
                                return new MutateRemarketingActionsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateRemarketingActionsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateRemarketingActionsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsRequest} m MutateRemarketingActionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateRemarketingActionsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.RemarketingActionOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateRemarketingActionsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateRemarketingActionsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsRequest} message MutateRemarketingActionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateRemarketingActionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateRemarketingActionsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsRequest} MutateRemarketingActionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateRemarketingActionsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateRemarketingActionsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.RemarketingActionOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateRemarketingActionsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsRequest} MutateRemarketingActionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateRemarketingActionsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateRemarketingActionsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateRemarketingActionsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.RemarketingActionOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateRemarketingActionsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsRequest} MutateRemarketingActionsRequest
                             */
                            MutateRemarketingActionsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateRemarketingActionsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateRemarketingActionsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateRemarketingActionsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateRemarketingActionsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.RemarketingActionOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateRemarketingActionsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateRemarketingActionsRequest} m MutateRemarketingActionsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateRemarketingActionsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.RemarketingActionOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateRemarketingActionsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateRemarketingActionsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateRemarketingActionsRequest;
                        })();
    
                        services.RemarketingActionOperation = (function() {
    
                            /**
                             * Properties of a RemarketingActionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IRemarketingActionOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] RemarketingActionOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IRemarketingAction|null} [create] RemarketingActionOperation create
                             * @property {google.ads.googleads.v11.resources.IRemarketingAction|null} [update] RemarketingActionOperation update
                             */
    
                            /**
                             * Constructs a new RemarketingActionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RemarketingActionOperation.
                             * @implements IRemarketingActionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IRemarketingActionOperation=} [p] Properties to set
                             */
                            function RemarketingActionOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RemarketingActionOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @instance
                             */
                            RemarketingActionOperation.prototype.update_mask = null;
    
                            /**
                             * RemarketingActionOperation create.
                             * @member {google.ads.googleads.v11.resources.IRemarketingAction|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @instance
                             */
                            RemarketingActionOperation.prototype.create = null;
    
                            /**
                             * RemarketingActionOperation update.
                             * @member {google.ads.googleads.v11.resources.IRemarketingAction|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @instance
                             */
                            RemarketingActionOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * RemarketingActionOperation operation.
                             * @member {"create"|"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @instance
                             */
                            Object.defineProperty(RemarketingActionOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new RemarketingActionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IRemarketingActionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.RemarketingActionOperation} RemarketingActionOperation instance
                             */
                            RemarketingActionOperation.create = function create(properties) {
                                return new RemarketingActionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified RemarketingActionOperation message. Does not implicitly {@link google.ads.googleads.v11.services.RemarketingActionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IRemarketingActionOperation} m RemarketingActionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RemarketingActionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.RemarketingAction.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.RemarketingAction.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RemarketingActionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.RemarketingActionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IRemarketingActionOperation} message RemarketingActionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RemarketingActionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RemarketingActionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.RemarketingActionOperation} RemarketingActionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RemarketingActionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.RemarketingActionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.RemarketingAction.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.RemarketingAction.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RemarketingActionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.RemarketingActionOperation} RemarketingActionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RemarketingActionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RemarketingActionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RemarketingActionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.RemarketingAction.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.RemarketingAction.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RemarketingActionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.RemarketingActionOperation} RemarketingActionOperation
                             */
                            RemarketingActionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.RemarketingActionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.RemarketingActionOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.RemarketingActionOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.RemarketingActionOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.RemarketingAction.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.RemarketingActionOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.RemarketingAction.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RemarketingActionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.RemarketingActionOperation} m RemarketingActionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RemarketingActionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.RemarketingAction.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.RemarketingAction.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RemarketingActionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.RemarketingActionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RemarketingActionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RemarketingActionOperation;
                        })();
    
                        services.MutateRemarketingActionsResponse = (function() {
    
                            /**
                             * Properties of a MutateRemarketingActionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateRemarketingActionsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateRemarketingActionsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateRemarketingActionResult>|null} [results] MutateRemarketingActionsResponse results
                             */
    
                            /**
                             * Constructs a new MutateRemarketingActionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateRemarketingActionsResponse.
                             * @implements IMutateRemarketingActionsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsResponse=} [p] Properties to set
                             */
                            function MutateRemarketingActionsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateRemarketingActionsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @instance
                             */
                            MutateRemarketingActionsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateRemarketingActionsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateRemarketingActionResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @instance
                             */
                            MutateRemarketingActionsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateRemarketingActionsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsResponse} MutateRemarketingActionsResponse instance
                             */
                            MutateRemarketingActionsResponse.create = function create(properties) {
                                return new MutateRemarketingActionsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateRemarketingActionsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateRemarketingActionsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsResponse} m MutateRemarketingActionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateRemarketingActionsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateRemarketingActionsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateRemarketingActionsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionsResponse} message MutateRemarketingActionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateRemarketingActionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateRemarketingActionsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsResponse} MutateRemarketingActionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateRemarketingActionsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateRemarketingActionsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateRemarketingActionResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateRemarketingActionsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsResponse} MutateRemarketingActionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateRemarketingActionsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateRemarketingActionsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateRemarketingActionsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateRemarketingActionsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionsResponse} MutateRemarketingActionsResponse
                             */
                            MutateRemarketingActionsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateRemarketingActionsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateRemarketingActionsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateRemarketingActionsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateRemarketingActionsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateRemarketingActionsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateRemarketingActionsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateRemarketingActionsResponse} m MutateRemarketingActionsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateRemarketingActionsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateRemarketingActionResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateRemarketingActionsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateRemarketingActionsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateRemarketingActionsResponse;
                        })();
    
                        services.MutateRemarketingActionResult = (function() {
    
                            /**
                             * Properties of a MutateRemarketingActionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateRemarketingActionResult
                             * @property {string|null} [resource_name] MutateRemarketingActionResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateRemarketingActionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateRemarketingActionResult.
                             * @implements IMutateRemarketingActionResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionResult=} [p] Properties to set
                             */
                            function MutateRemarketingActionResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateRemarketingActionResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @instance
                             */
                            MutateRemarketingActionResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateRemarketingActionResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionResult} MutateRemarketingActionResult instance
                             */
                            MutateRemarketingActionResult.create = function create(properties) {
                                return new MutateRemarketingActionResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateRemarketingActionResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateRemarketingActionResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionResult} m MutateRemarketingActionResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateRemarketingActionResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateRemarketingActionResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateRemarketingActionResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateRemarketingActionResult} message MutateRemarketingActionResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateRemarketingActionResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateRemarketingActionResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionResult} MutateRemarketingActionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateRemarketingActionResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateRemarketingActionResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateRemarketingActionResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionResult} MutateRemarketingActionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateRemarketingActionResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateRemarketingActionResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateRemarketingActionResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateRemarketingActionResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateRemarketingActionResult} MutateRemarketingActionResult
                             */
                            MutateRemarketingActionResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateRemarketingActionResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateRemarketingActionResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateRemarketingActionResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateRemarketingActionResult} m MutateRemarketingActionResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateRemarketingActionResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateRemarketingActionResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateRemarketingActionResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateRemarketingActionResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateRemarketingActionResult;
                        })();
    
                        services.SharedCriterionService = (function() {
    
                            /**
                             * Constructs a new SharedCriterionService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SharedCriterionService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function SharedCriterionService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (SharedCriterionService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = SharedCriterionService;
    
                            /**
                             * Creates new SharedCriterionService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SharedCriterionService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {SharedCriterionService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            SharedCriterionService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.SharedCriterionService|mutateSharedCriteria}.
                             * @memberof google.ads.googleads.v11.services.SharedCriterionService
                             * @typedef MutateSharedCriteriaCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateSharedCriteriaResponse} [response] MutateSharedCriteriaResponse
                             */
    
                            /**
                             * Calls MutateSharedCriteria.
                             * @function mutateSharedCriteria
                             * @memberof google.ads.googleads.v11.services.SharedCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaRequest} request MutateSharedCriteriaRequest message or plain object
                             * @param {google.ads.googleads.v11.services.SharedCriterionService.MutateSharedCriteriaCallback} callback Node-style callback called with the error, if any, and MutateSharedCriteriaResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(SharedCriterionService.prototype.mutateSharedCriteria = function mutateSharedCriteria(request, callback) {
                                return this.rpcCall(mutateSharedCriteria, $root.google.ads.googleads.v11.services.MutateSharedCriteriaRequest, $root.google.ads.googleads.v11.services.MutateSharedCriteriaResponse, request, callback);
                            }, "name", { value: "MutateSharedCriteria" });
    
                            /**
                             * Calls MutateSharedCriteria.
                             * @function mutateSharedCriteria
                             * @memberof google.ads.googleads.v11.services.SharedCriterionService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaRequest} request MutateSharedCriteriaRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateSharedCriteriaResponse>} Promise
                             * @variation 2
                             */
    
                            return SharedCriterionService;
                        })();
    
                        services.MutateSharedCriteriaRequest = (function() {
    
                            /**
                             * Properties of a MutateSharedCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSharedCriteriaRequest
                             * @property {string|null} [customer_id] MutateSharedCriteriaRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ISharedCriterionOperation>|null} [operations] MutateSharedCriteriaRequest operations
                             * @property {boolean|null} [partial_failure] MutateSharedCriteriaRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateSharedCriteriaRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateSharedCriteriaRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateSharedCriteriaRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSharedCriteriaRequest.
                             * @implements IMutateSharedCriteriaRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaRequest=} [p] Properties to set
                             */
                            function MutateSharedCriteriaRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSharedCriteriaRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @instance
                             */
                            MutateSharedCriteriaRequest.prototype.customer_id = "";
    
                            /**
                             * MutateSharedCriteriaRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ISharedCriterionOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @instance
                             */
                            MutateSharedCriteriaRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateSharedCriteriaRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @instance
                             */
                            MutateSharedCriteriaRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateSharedCriteriaRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @instance
                             */
                            MutateSharedCriteriaRequest.prototype.validate_only = false;
    
                            /**
                             * MutateSharedCriteriaRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @instance
                             */
                            MutateSharedCriteriaRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateSharedCriteriaRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaRequest} MutateSharedCriteriaRequest instance
                             */
                            MutateSharedCriteriaRequest.create = function create(properties) {
                                return new MutateSharedCriteriaRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSharedCriteriaRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedCriteriaRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaRequest} m MutateSharedCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedCriteriaRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.SharedCriterionOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSharedCriteriaRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedCriteriaRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaRequest} message MutateSharedCriteriaRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedCriteriaRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSharedCriteriaRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaRequest} MutateSharedCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedCriteriaRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSharedCriteriaRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.SharedCriterionOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSharedCriteriaRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaRequest} MutateSharedCriteriaRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedCriteriaRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSharedCriteriaRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSharedCriteriaRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.SharedCriterionOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSharedCriteriaRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaRequest} MutateSharedCriteriaRequest
                             */
                            MutateSharedCriteriaRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSharedCriteriaRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSharedCriteriaRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedCriteriaRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateSharedCriteriaRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.SharedCriterionOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSharedCriteriaRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSharedCriteriaRequest} m MutateSharedCriteriaRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSharedCriteriaRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.SharedCriterionOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSharedCriteriaRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSharedCriteriaRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSharedCriteriaRequest;
                        })();
    
                        services.SharedCriterionOperation = (function() {
    
                            /**
                             * Properties of a SharedCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISharedCriterionOperation
                             * @property {google.ads.googleads.v11.resources.ISharedCriterion|null} [create] SharedCriterionOperation create
                             * @property {string|null} [remove] SharedCriterionOperation remove
                             */
    
                            /**
                             * Constructs a new SharedCriterionOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SharedCriterionOperation.
                             * @implements ISharedCriterionOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISharedCriterionOperation=} [p] Properties to set
                             */
                            function SharedCriterionOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SharedCriterionOperation create.
                             * @member {google.ads.googleads.v11.resources.ISharedCriterion|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @instance
                             */
                            SharedCriterionOperation.prototype.create = null;
    
                            /**
                             * SharedCriterionOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @instance
                             */
                            SharedCriterionOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SharedCriterionOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @instance
                             */
                            Object.defineProperty(SharedCriterionOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SharedCriterionOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISharedCriterionOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SharedCriterionOperation} SharedCriterionOperation instance
                             */
                            SharedCriterionOperation.create = function create(properties) {
                                return new SharedCriterionOperation(properties);
                            };
    
                            /**
                             * Encodes the specified SharedCriterionOperation message. Does not implicitly {@link google.ads.googleads.v11.services.SharedCriterionOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISharedCriterionOperation} m SharedCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedCriterionOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.SharedCriterion.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedCriterionOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SharedCriterionOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISharedCriterionOperation} message SharedCriterionOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedCriterionOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedCriterionOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SharedCriterionOperation} SharedCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedCriterionOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SharedCriterionOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.SharedCriterion.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedCriterionOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SharedCriterionOperation} SharedCriterionOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedCriterionOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedCriterionOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedCriterionOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SharedCriterion.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SharedCriterionOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SharedCriterionOperation} SharedCriterionOperation
                             */
                            SharedCriterionOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SharedCriterionOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SharedCriterionOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SharedCriterionOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.SharedCriterion.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SharedCriterionOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.SharedCriterionOperation} m SharedCriterionOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedCriterionOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.SharedCriterion.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SharedCriterionOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SharedCriterionOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedCriterionOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SharedCriterionOperation;
                        })();
    
                        services.MutateSharedCriteriaResponse = (function() {
    
                            /**
                             * Properties of a MutateSharedCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSharedCriteriaResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateSharedCriteriaResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateSharedCriterionResult>|null} [results] MutateSharedCriteriaResponse results
                             */
    
                            /**
                             * Constructs a new MutateSharedCriteriaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSharedCriteriaResponse.
                             * @implements IMutateSharedCriteriaResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaResponse=} [p] Properties to set
                             */
                            function MutateSharedCriteriaResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSharedCriteriaResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @instance
                             */
                            MutateSharedCriteriaResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateSharedCriteriaResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateSharedCriterionResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @instance
                             */
                            MutateSharedCriteriaResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateSharedCriteriaResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaResponse} MutateSharedCriteriaResponse instance
                             */
                            MutateSharedCriteriaResponse.create = function create(properties) {
                                return new MutateSharedCriteriaResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSharedCriteriaResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedCriteriaResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaResponse} m MutateSharedCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedCriteriaResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSharedCriteriaResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedCriteriaResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriteriaResponse} message MutateSharedCriteriaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedCriteriaResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSharedCriteriaResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaResponse} MutateSharedCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedCriteriaResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSharedCriteriaResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateSharedCriterionResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSharedCriteriaResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaResponse} MutateSharedCriteriaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedCriteriaResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSharedCriteriaResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSharedCriteriaResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSharedCriteriaResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriteriaResponse} MutateSharedCriteriaResponse
                             */
                            MutateSharedCriteriaResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSharedCriteriaResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSharedCriteriaResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedCriteriaResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedCriteriaResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateSharedCriteriaResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSharedCriteriaResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSharedCriteriaResponse} m MutateSharedCriteriaResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSharedCriteriaResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateSharedCriterionResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSharedCriteriaResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriteriaResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSharedCriteriaResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSharedCriteriaResponse;
                        })();
    
                        services.MutateSharedCriterionResult = (function() {
    
                            /**
                             * Properties of a MutateSharedCriterionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSharedCriterionResult
                             * @property {string|null} [resource_name] MutateSharedCriterionResult resource_name
                             * @property {google.ads.googleads.v11.resources.ISharedCriterion|null} [shared_criterion] MutateSharedCriterionResult shared_criterion
                             */
    
                            /**
                             * Constructs a new MutateSharedCriterionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSharedCriterionResult.
                             * @implements IMutateSharedCriterionResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriterionResult=} [p] Properties to set
                             */
                            function MutateSharedCriterionResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSharedCriterionResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @instance
                             */
                            MutateSharedCriterionResult.prototype.resource_name = "";
    
                            /**
                             * MutateSharedCriterionResult shared_criterion.
                             * @member {google.ads.googleads.v11.resources.ISharedCriterion|null|undefined} shared_criterion
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @instance
                             */
                            MutateSharedCriterionResult.prototype.shared_criterion = null;
    
                            /**
                             * Creates a new MutateSharedCriterionResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriterionResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriterionResult} MutateSharedCriterionResult instance
                             */
                            MutateSharedCriterionResult.create = function create(properties) {
                                return new MutateSharedCriterionResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSharedCriterionResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedCriterionResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriterionResult} m MutateSharedCriterionResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedCriterionResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.shared_criterion != null && Object.hasOwnProperty.call(m, "shared_criterion"))
                                    $root.google.ads.googleads.v11.resources.SharedCriterion.encode(m.shared_criterion, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSharedCriterionResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedCriterionResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedCriterionResult} message MutateSharedCriterionResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedCriterionResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSharedCriterionResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriterionResult} MutateSharedCriterionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedCriterionResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSharedCriterionResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.shared_criterion = $root.google.ads.googleads.v11.resources.SharedCriterion.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSharedCriterionResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriterionResult} MutateSharedCriterionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedCriterionResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSharedCriterionResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSharedCriterionResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.shared_criterion != null && m.hasOwnProperty("shared_criterion")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SharedCriterion.verify(m.shared_criterion);
                                        if (e)
                                            return "shared_criterion." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSharedCriterionResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSharedCriterionResult} MutateSharedCriterionResult
                             */
                            MutateSharedCriterionResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSharedCriterionResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSharedCriterionResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.shared_criterion != null) {
                                    if (typeof d.shared_criterion !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedCriterionResult.shared_criterion: object expected");
                                    m.shared_criterion = $root.google.ads.googleads.v11.resources.SharedCriterion.fromObject(d.shared_criterion);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSharedCriterionResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSharedCriterionResult} m MutateSharedCriterionResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSharedCriterionResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.shared_criterion = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.shared_criterion != null && m.hasOwnProperty("shared_criterion")) {
                                    d.shared_criterion = $root.google.ads.googleads.v11.resources.SharedCriterion.toObject(m.shared_criterion, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSharedCriterionResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSharedCriterionResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSharedCriterionResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSharedCriterionResult;
                        })();
    
                        services.SharedSetService = (function() {
    
                            /**
                             * Constructs a new SharedSetService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SharedSetService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function SharedSetService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (SharedSetService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = SharedSetService;
    
                            /**
                             * Creates new SharedSetService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SharedSetService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {SharedSetService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            SharedSetService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.SharedSetService|mutateSharedSets}.
                             * @memberof google.ads.googleads.v11.services.SharedSetService
                             * @typedef MutateSharedSetsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateSharedSetsResponse} [response] MutateSharedSetsResponse
                             */
    
                            /**
                             * Calls MutateSharedSets.
                             * @function mutateSharedSets
                             * @memberof google.ads.googleads.v11.services.SharedSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsRequest} request MutateSharedSetsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.SharedSetService.MutateSharedSetsCallback} callback Node-style callback called with the error, if any, and MutateSharedSetsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(SharedSetService.prototype.mutateSharedSets = function mutateSharedSets(request, callback) {
                                return this.rpcCall(mutateSharedSets, $root.google.ads.googleads.v11.services.MutateSharedSetsRequest, $root.google.ads.googleads.v11.services.MutateSharedSetsResponse, request, callback);
                            }, "name", { value: "MutateSharedSets" });
    
                            /**
                             * Calls MutateSharedSets.
                             * @function mutateSharedSets
                             * @memberof google.ads.googleads.v11.services.SharedSetService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsRequest} request MutateSharedSetsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateSharedSetsResponse>} Promise
                             * @variation 2
                             */
    
                            return SharedSetService;
                        })();
    
                        services.MutateSharedSetsRequest = (function() {
    
                            /**
                             * Properties of a MutateSharedSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSharedSetsRequest
                             * @property {string|null} [customer_id] MutateSharedSetsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ISharedSetOperation>|null} [operations] MutateSharedSetsRequest operations
                             * @property {boolean|null} [partial_failure] MutateSharedSetsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateSharedSetsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateSharedSetsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateSharedSetsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSharedSetsRequest.
                             * @implements IMutateSharedSetsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsRequest=} [p] Properties to set
                             */
                            function MutateSharedSetsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSharedSetsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @instance
                             */
                            MutateSharedSetsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateSharedSetsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ISharedSetOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @instance
                             */
                            MutateSharedSetsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateSharedSetsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @instance
                             */
                            MutateSharedSetsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateSharedSetsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @instance
                             */
                            MutateSharedSetsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateSharedSetsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @instance
                             */
                            MutateSharedSetsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateSharedSetsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsRequest} MutateSharedSetsRequest instance
                             */
                            MutateSharedSetsRequest.create = function create(properties) {
                                return new MutateSharedSetsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSharedSetsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedSetsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsRequest} m MutateSharedSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedSetsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.SharedSetOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSharedSetsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedSetsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsRequest} message MutateSharedSetsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedSetsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSharedSetsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsRequest} MutateSharedSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedSetsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSharedSetsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.SharedSetOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSharedSetsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsRequest} MutateSharedSetsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedSetsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSharedSetsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSharedSetsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.SharedSetOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSharedSetsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsRequest} MutateSharedSetsRequest
                             */
                            MutateSharedSetsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSharedSetsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSharedSetsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedSetsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateSharedSetsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.SharedSetOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSharedSetsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSharedSetsRequest} m MutateSharedSetsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSharedSetsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.SharedSetOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSharedSetsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSharedSetsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSharedSetsRequest;
                        })();
    
                        services.SharedSetOperation = (function() {
    
                            /**
                             * Properties of a SharedSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISharedSetOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] SharedSetOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ISharedSet|null} [create] SharedSetOperation create
                             * @property {google.ads.googleads.v11.resources.ISharedSet|null} [update] SharedSetOperation update
                             * @property {string|null} [remove] SharedSetOperation remove
                             */
    
                            /**
                             * Constructs a new SharedSetOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SharedSetOperation.
                             * @implements ISharedSetOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISharedSetOperation=} [p] Properties to set
                             */
                            function SharedSetOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SharedSetOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @instance
                             */
                            SharedSetOperation.prototype.update_mask = null;
    
                            /**
                             * SharedSetOperation create.
                             * @member {google.ads.googleads.v11.resources.ISharedSet|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @instance
                             */
                            SharedSetOperation.prototype.create = null;
    
                            /**
                             * SharedSetOperation update.
                             * @member {google.ads.googleads.v11.resources.ISharedSet|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @instance
                             */
                            SharedSetOperation.prototype.update = null;
    
                            /**
                             * SharedSetOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @instance
                             */
                            SharedSetOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SharedSetOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @instance
                             */
                            Object.defineProperty(SharedSetOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SharedSetOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISharedSetOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SharedSetOperation} SharedSetOperation instance
                             */
                            SharedSetOperation.create = function create(properties) {
                                return new SharedSetOperation(properties);
                            };
    
                            /**
                             * Encodes the specified SharedSetOperation message. Does not implicitly {@link google.ads.googleads.v11.services.SharedSetOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISharedSetOperation} m SharedSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.SharedSet.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.SharedSet.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SharedSetOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SharedSetOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISharedSetOperation} message SharedSetOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SharedSetOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SharedSetOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SharedSetOperation} SharedSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SharedSetOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.SharedSet.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.SharedSet.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SharedSetOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SharedSetOperation} SharedSetOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SharedSetOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SharedSetOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SharedSetOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SharedSet.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SharedSet.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SharedSetOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SharedSetOperation} SharedSetOperation
                             */
                            SharedSetOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SharedSetOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SharedSetOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SharedSetOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SharedSetOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.SharedSet.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SharedSetOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.SharedSet.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SharedSetOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.SharedSetOperation} m SharedSetOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SharedSetOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.SharedSet.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.SharedSet.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SharedSetOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SharedSetOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SharedSetOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SharedSetOperation;
                        })();
    
                        services.MutateSharedSetsResponse = (function() {
    
                            /**
                             * Properties of a MutateSharedSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSharedSetsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateSharedSetsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateSharedSetResult>|null} [results] MutateSharedSetsResponse results
                             */
    
                            /**
                             * Constructs a new MutateSharedSetsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSharedSetsResponse.
                             * @implements IMutateSharedSetsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsResponse=} [p] Properties to set
                             */
                            function MutateSharedSetsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSharedSetsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @instance
                             */
                            MutateSharedSetsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateSharedSetsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateSharedSetResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @instance
                             */
                            MutateSharedSetsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateSharedSetsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsResponse} MutateSharedSetsResponse instance
                             */
                            MutateSharedSetsResponse.create = function create(properties) {
                                return new MutateSharedSetsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSharedSetsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedSetsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsResponse} m MutateSharedSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedSetsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateSharedSetResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSharedSetsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedSetsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetsResponse} message MutateSharedSetsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedSetsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSharedSetsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsResponse} MutateSharedSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedSetsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSharedSetsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateSharedSetResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSharedSetsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsResponse} MutateSharedSetsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedSetsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSharedSetsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSharedSetsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateSharedSetResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSharedSetsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetsResponse} MutateSharedSetsResponse
                             */
                            MutateSharedSetsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSharedSetsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSharedSetsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedSetsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedSetsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateSharedSetsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateSharedSetResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSharedSetsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSharedSetsResponse} m MutateSharedSetsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSharedSetsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateSharedSetResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSharedSetsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSharedSetsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSharedSetsResponse;
                        })();
    
                        services.MutateSharedSetResult = (function() {
    
                            /**
                             * Properties of a MutateSharedSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSharedSetResult
                             * @property {string|null} [resource_name] MutateSharedSetResult resource_name
                             * @property {google.ads.googleads.v11.resources.ISharedSet|null} [shared_set] MutateSharedSetResult shared_set
                             */
    
                            /**
                             * Constructs a new MutateSharedSetResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSharedSetResult.
                             * @implements IMutateSharedSetResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetResult=} [p] Properties to set
                             */
                            function MutateSharedSetResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSharedSetResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @instance
                             */
                            MutateSharedSetResult.prototype.resource_name = "";
    
                            /**
                             * MutateSharedSetResult shared_set.
                             * @member {google.ads.googleads.v11.resources.ISharedSet|null|undefined} shared_set
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @instance
                             */
                            MutateSharedSetResult.prototype.shared_set = null;
    
                            /**
                             * Creates a new MutateSharedSetResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetResult} MutateSharedSetResult instance
                             */
                            MutateSharedSetResult.create = function create(properties) {
                                return new MutateSharedSetResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSharedSetResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedSetResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetResult} m MutateSharedSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedSetResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.shared_set != null && Object.hasOwnProperty.call(m, "shared_set"))
                                    $root.google.ads.googleads.v11.resources.SharedSet.encode(m.shared_set, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSharedSetResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSharedSetResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSharedSetResult} message MutateSharedSetResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSharedSetResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSharedSetResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetResult} MutateSharedSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedSetResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSharedSetResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.shared_set = $root.google.ads.googleads.v11.resources.SharedSet.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSharedSetResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetResult} MutateSharedSetResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSharedSetResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSharedSetResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSharedSetResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SharedSet.verify(m.shared_set);
                                        if (e)
                                            return "shared_set." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSharedSetResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSharedSetResult} MutateSharedSetResult
                             */
                            MutateSharedSetResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSharedSetResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSharedSetResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.shared_set != null) {
                                    if (typeof d.shared_set !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSharedSetResult.shared_set: object expected");
                                    m.shared_set = $root.google.ads.googleads.v11.resources.SharedSet.fromObject(d.shared_set);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSharedSetResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSharedSetResult} m MutateSharedSetResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSharedSetResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.shared_set = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.shared_set != null && m.hasOwnProperty("shared_set")) {
                                    d.shared_set = $root.google.ads.googleads.v11.resources.SharedSet.toObject(m.shared_set, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSharedSetResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSharedSetResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSharedSetResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSharedSetResult;
                        })();
    
                        services.SmartCampaignSettingService = (function() {
    
                            /**
                             * Constructs a new SmartCampaignSettingService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SmartCampaignSettingService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function SmartCampaignSettingService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (SmartCampaignSettingService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = SmartCampaignSettingService;
    
                            /**
                             * Creates new SmartCampaignSettingService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {SmartCampaignSettingService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            SmartCampaignSettingService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.SmartCampaignSettingService|mutateSmartCampaignSettings}.
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingService
                             * @typedef MutateSmartCampaignSettingsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse} [response] MutateSmartCampaignSettingsResponse
                             */
    
                            /**
                             * Calls MutateSmartCampaignSettings.
                             * @function mutateSmartCampaignSettings
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsRequest} request MutateSmartCampaignSettingsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.SmartCampaignSettingService.MutateSmartCampaignSettingsCallback} callback Node-style callback called with the error, if any, and MutateSmartCampaignSettingsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(SmartCampaignSettingService.prototype.mutateSmartCampaignSettings = function mutateSmartCampaignSettings(request, callback) {
                                return this.rpcCall(mutateSmartCampaignSettings, $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest, $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse, request, callback);
                            }, "name", { value: "MutateSmartCampaignSettings" });
    
                            /**
                             * Calls MutateSmartCampaignSettings.
                             * @function mutateSmartCampaignSettings
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsRequest} request MutateSmartCampaignSettingsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse>} Promise
                             * @variation 2
                             */
    
                            return SmartCampaignSettingService;
                        })();
    
                        services.MutateSmartCampaignSettingsRequest = (function() {
    
                            /**
                             * Properties of a MutateSmartCampaignSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSmartCampaignSettingsRequest
                             * @property {string|null} [customer_id] MutateSmartCampaignSettingsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ISmartCampaignSettingOperation>|null} [operations] MutateSmartCampaignSettingsRequest operations
                             * @property {boolean|null} [partial_failure] MutateSmartCampaignSettingsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateSmartCampaignSettingsRequest validate_only
                             * @property {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType|null} [response_content_type] MutateSmartCampaignSettingsRequest response_content_type
                             */
    
                            /**
                             * Constructs a new MutateSmartCampaignSettingsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSmartCampaignSettingsRequest.
                             * @implements IMutateSmartCampaignSettingsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsRequest=} [p] Properties to set
                             */
                            function MutateSmartCampaignSettingsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSmartCampaignSettingsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @instance
                             */
                            MutateSmartCampaignSettingsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateSmartCampaignSettingsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ISmartCampaignSettingOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @instance
                             */
                            MutateSmartCampaignSettingsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateSmartCampaignSettingsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @instance
                             */
                            MutateSmartCampaignSettingsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateSmartCampaignSettingsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @instance
                             */
                            MutateSmartCampaignSettingsRequest.prototype.validate_only = false;
    
                            /**
                             * MutateSmartCampaignSettingsRequest response_content_type.
                             * @member {google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType} response_content_type
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @instance
                             */
                            MutateSmartCampaignSettingsRequest.prototype.response_content_type = 0;
    
                            /**
                             * Creates a new MutateSmartCampaignSettingsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest} MutateSmartCampaignSettingsRequest instance
                             */
                            MutateSmartCampaignSettingsRequest.create = function create(properties) {
                                return new MutateSmartCampaignSettingsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSmartCampaignSettingsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsRequest} m MutateSmartCampaignSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSmartCampaignSettingsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                if (m.response_content_type != null && Object.hasOwnProperty.call(m, "response_content_type"))
                                    w.uint32(40).int32(m.response_content_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSmartCampaignSettingsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsRequest} message MutateSmartCampaignSettingsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSmartCampaignSettingsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSmartCampaignSettingsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest} MutateSmartCampaignSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSmartCampaignSettingsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.response_content_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSmartCampaignSettingsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest} MutateSmartCampaignSettingsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSmartCampaignSettingsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSmartCampaignSettingsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSmartCampaignSettingsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    switch (m.response_content_type) {
                                    default:
                                        return "response_content_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSmartCampaignSettingsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest} MutateSmartCampaignSettingsRequest
                             */
                            MutateSmartCampaignSettingsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                switch (d.response_content_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.response_content_type = 0;
                                    break;
                                case "RESOURCE_NAME_ONLY":
                                case 1:
                                    m.response_content_type = 1;
                                    break;
                                case "MUTABLE_RESOURCE":
                                case 2:
                                    m.response_content_type = 2;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSmartCampaignSettingsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest} m MutateSmartCampaignSettingsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSmartCampaignSettingsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                    d.response_content_type = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.response_content_type != null && m.hasOwnProperty("response_content_type")) {
                                    d.response_content_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ResponseContentTypeEnum.ResponseContentType[m.response_content_type] : m.response_content_type;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSmartCampaignSettingsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSmartCampaignSettingsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSmartCampaignSettingsRequest;
                        })();
    
                        services.SmartCampaignSettingOperation = (function() {
    
                            /**
                             * Properties of a SmartCampaignSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISmartCampaignSettingOperation
                             * @property {google.ads.googleads.v11.resources.ISmartCampaignSetting|null} [update] SmartCampaignSettingOperation update
                             * @property {google.protobuf.IFieldMask|null} [update_mask] SmartCampaignSettingOperation update_mask
                             */
    
                            /**
                             * Constructs a new SmartCampaignSettingOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SmartCampaignSettingOperation.
                             * @implements ISmartCampaignSettingOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSettingOperation=} [p] Properties to set
                             */
                            function SmartCampaignSettingOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SmartCampaignSettingOperation update.
                             * @member {google.ads.googleads.v11.resources.ISmartCampaignSetting|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @instance
                             */
                            SmartCampaignSettingOperation.prototype.update = null;
    
                            /**
                             * SmartCampaignSettingOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @instance
                             */
                            SmartCampaignSettingOperation.prototype.update_mask = null;
    
                            /**
                             * Creates a new SmartCampaignSettingOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSettingOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSettingOperation} SmartCampaignSettingOperation instance
                             */
                            SmartCampaignSettingOperation.create = function create(properties) {
                                return new SmartCampaignSettingOperation(properties);
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSettingOperation message. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSettingOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSettingOperation} m SmartCampaignSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSettingOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.SmartCampaignSetting.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSettingOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSettingOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSettingOperation} message SmartCampaignSettingOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSettingOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SmartCampaignSettingOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSettingOperation} SmartCampaignSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSettingOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SmartCampaignSettingOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSettingOperation} SmartCampaignSettingOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSettingOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SmartCampaignSettingOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SmartCampaignSettingOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SmartCampaignSettingOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSettingOperation} SmartCampaignSettingOperation
                             */
                            SmartCampaignSettingOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SmartCampaignSettingOperation();
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSettingOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.fromObject(d.update);
                                }
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSettingOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SmartCampaignSettingOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.SmartCampaignSettingOperation} m SmartCampaignSettingOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SmartCampaignSettingOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update = null;
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.toObject(m.update, o);
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SmartCampaignSettingOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSettingOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SmartCampaignSettingOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SmartCampaignSettingOperation;
                        })();
    
                        services.MutateSmartCampaignSettingsResponse = (function() {
    
                            /**
                             * Properties of a MutateSmartCampaignSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSmartCampaignSettingsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateSmartCampaignSettingsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult>|null} [results] MutateSmartCampaignSettingsResponse results
                             */
    
                            /**
                             * Constructs a new MutateSmartCampaignSettingsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSmartCampaignSettingsResponse.
                             * @implements IMutateSmartCampaignSettingsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsResponse=} [p] Properties to set
                             */
                            function MutateSmartCampaignSettingsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSmartCampaignSettingsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @instance
                             */
                            MutateSmartCampaignSettingsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateSmartCampaignSettingsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @instance
                             */
                            MutateSmartCampaignSettingsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateSmartCampaignSettingsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse} MutateSmartCampaignSettingsResponse instance
                             */
                            MutateSmartCampaignSettingsResponse.create = function create(properties) {
                                return new MutateSmartCampaignSettingsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSmartCampaignSettingsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsResponse} m MutateSmartCampaignSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSmartCampaignSettingsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSmartCampaignSettingsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingsResponse} message MutateSmartCampaignSettingsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSmartCampaignSettingsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSmartCampaignSettingsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse} MutateSmartCampaignSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSmartCampaignSettingsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSmartCampaignSettingsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse} MutateSmartCampaignSettingsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSmartCampaignSettingsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSmartCampaignSettingsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSmartCampaignSettingsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSmartCampaignSettingsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse} MutateSmartCampaignSettingsResponse
                             */
                            MutateSmartCampaignSettingsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSmartCampaignSettingsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse} m MutateSmartCampaignSettingsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSmartCampaignSettingsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSmartCampaignSettingsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSmartCampaignSettingsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSmartCampaignSettingsResponse;
                        })();
    
                        services.MutateSmartCampaignSettingResult = (function() {
    
                            /**
                             * Properties of a MutateSmartCampaignSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateSmartCampaignSettingResult
                             * @property {string|null} [resource_name] MutateSmartCampaignSettingResult resource_name
                             * @property {google.ads.googleads.v11.resources.ISmartCampaignSetting|null} [smart_campaign_setting] MutateSmartCampaignSettingResult smart_campaign_setting
                             */
    
                            /**
                             * Constructs a new MutateSmartCampaignSettingResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateSmartCampaignSettingResult.
                             * @implements IMutateSmartCampaignSettingResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult=} [p] Properties to set
                             */
                            function MutateSmartCampaignSettingResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateSmartCampaignSettingResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @instance
                             */
                            MutateSmartCampaignSettingResult.prototype.resource_name = "";
    
                            /**
                             * MutateSmartCampaignSettingResult smart_campaign_setting.
                             * @member {google.ads.googleads.v11.resources.ISmartCampaignSetting|null|undefined} smart_campaign_setting
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @instance
                             */
                            MutateSmartCampaignSettingResult.prototype.smart_campaign_setting = null;
    
                            /**
                             * Creates a new MutateSmartCampaignSettingResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingResult} MutateSmartCampaignSettingResult instance
                             */
                            MutateSmartCampaignSettingResult.create = function create(properties) {
                                return new MutateSmartCampaignSettingResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateSmartCampaignSettingResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult} m MutateSmartCampaignSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSmartCampaignSettingResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.smart_campaign_setting != null && Object.hasOwnProperty.call(m, "smart_campaign_setting"))
                                    $root.google.ads.googleads.v11.resources.SmartCampaignSetting.encode(m.smart_campaign_setting, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateSmartCampaignSettingResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateSmartCampaignSettingResult} message MutateSmartCampaignSettingResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateSmartCampaignSettingResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateSmartCampaignSettingResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingResult} MutateSmartCampaignSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSmartCampaignSettingResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.smart_campaign_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateSmartCampaignSettingResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingResult} MutateSmartCampaignSettingResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateSmartCampaignSettingResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateSmartCampaignSettingResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateSmartCampaignSettingResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.smart_campaign_setting != null && m.hasOwnProperty("smart_campaign_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.verify(m.smart_campaign_setting);
                                        if (e)
                                            return "smart_campaign_setting." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateSmartCampaignSettingResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateSmartCampaignSettingResult} MutateSmartCampaignSettingResult
                             */
                            MutateSmartCampaignSettingResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateSmartCampaignSettingResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.smart_campaign_setting != null) {
                                    if (typeof d.smart_campaign_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateSmartCampaignSettingResult.smart_campaign_setting: object expected");
                                    m.smart_campaign_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.fromObject(d.smart_campaign_setting);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateSmartCampaignSettingResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateSmartCampaignSettingResult} m MutateSmartCampaignSettingResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateSmartCampaignSettingResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.smart_campaign_setting = null;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.smart_campaign_setting != null && m.hasOwnProperty("smart_campaign_setting")) {
                                    d.smart_campaign_setting = $root.google.ads.googleads.v11.resources.SmartCampaignSetting.toObject(m.smart_campaign_setting, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateSmartCampaignSettingResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateSmartCampaignSettingResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateSmartCampaignSettingResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateSmartCampaignSettingResult;
                        })();
    
                        services.UserListService = (function() {
    
                            /**
                             * Constructs a new UserListService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a UserListService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function UserListService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (UserListService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = UserListService;
    
                            /**
                             * Creates new UserListService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UserListService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {UserListService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            UserListService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.UserListService|mutateUserLists}.
                             * @memberof google.ads.googleads.v11.services.UserListService
                             * @typedef MutateUserListsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateUserListsResponse} [response] MutateUserListsResponse
                             */
    
                            /**
                             * Calls MutateUserLists.
                             * @function mutateUserLists
                             * @memberof google.ads.googleads.v11.services.UserListService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateUserListsRequest} request MutateUserListsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.UserListService.MutateUserListsCallback} callback Node-style callback called with the error, if any, and MutateUserListsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(UserListService.prototype.mutateUserLists = function mutateUserLists(request, callback) {
                                return this.rpcCall(mutateUserLists, $root.google.ads.googleads.v11.services.MutateUserListsRequest, $root.google.ads.googleads.v11.services.MutateUserListsResponse, request, callback);
                            }, "name", { value: "MutateUserLists" });
    
                            /**
                             * Calls MutateUserLists.
                             * @function mutateUserLists
                             * @memberof google.ads.googleads.v11.services.UserListService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateUserListsRequest} request MutateUserListsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateUserListsResponse>} Promise
                             * @variation 2
                             */
    
                            return UserListService;
                        })();
    
                        services.MutateUserListsRequest = (function() {
    
                            /**
                             * Properties of a MutateUserListsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateUserListsRequest
                             * @property {string|null} [customer_id] MutateUserListsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IUserListOperation>|null} [operations] MutateUserListsRequest operations
                             * @property {boolean|null} [partial_failure] MutateUserListsRequest partial_failure
                             * @property {boolean|null} [validate_only] MutateUserListsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateUserListsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateUserListsRequest.
                             * @implements IMutateUserListsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateUserListsRequest=} [p] Properties to set
                             */
                            function MutateUserListsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateUserListsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @instance
                             */
                            MutateUserListsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateUserListsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IUserListOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @instance
                             */
                            MutateUserListsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateUserListsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @instance
                             */
                            MutateUserListsRequest.prototype.partial_failure = false;
    
                            /**
                             * MutateUserListsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @instance
                             */
                            MutateUserListsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateUserListsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateUserListsRequest} MutateUserListsRequest instance
                             */
                            MutateUserListsRequest.create = function create(properties) {
                                return new MutateUserListsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateUserListsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateUserListsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListsRequest} m MutateUserListsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateUserListsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.UserListOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateUserListsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateUserListsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListsRequest} message MutateUserListsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateUserListsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateUserListsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateUserListsRequest} MutateUserListsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateUserListsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateUserListsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.UserListOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateUserListsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateUserListsRequest} MutateUserListsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateUserListsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateUserListsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateUserListsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.UserListOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateUserListsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateUserListsRequest} MutateUserListsRequest
                             */
                            MutateUserListsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateUserListsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateUserListsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateUserListsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateUserListsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.UserListOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateUserListsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateUserListsRequest} m MutateUserListsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateUserListsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.UserListOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateUserListsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateUserListsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateUserListsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateUserListsRequest;
                        })();
    
                        services.UserListOperation = (function() {
    
                            /**
                             * Properties of a UserListOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUserListOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] UserListOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IUserList|null} [create] UserListOperation create
                             * @property {google.ads.googleads.v11.resources.IUserList|null} [update] UserListOperation update
                             * @property {string|null} [remove] UserListOperation remove
                             */
    
                            /**
                             * Constructs a new UserListOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a UserListOperation.
                             * @implements IUserListOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUserListOperation=} [p] Properties to set
                             */
                            function UserListOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserListOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @instance
                             */
                            UserListOperation.prototype.update_mask = null;
    
                            /**
                             * UserListOperation create.
                             * @member {google.ads.googleads.v11.resources.IUserList|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @instance
                             */
                            UserListOperation.prototype.create = null;
    
                            /**
                             * UserListOperation update.
                             * @member {google.ads.googleads.v11.resources.IUserList|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @instance
                             */
                            UserListOperation.prototype.update = null;
    
                            /**
                             * UserListOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @instance
                             */
                            UserListOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserListOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @instance
                             */
                            Object.defineProperty(UserListOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserListOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IUserListOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UserListOperation} UserListOperation instance
                             */
                            UserListOperation.create = function create(properties) {
                                return new UserListOperation(properties);
                            };
    
                            /**
                             * Encodes the specified UserListOperation message. Does not implicitly {@link google.ads.googleads.v11.services.UserListOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IUserListOperation} m UserListOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.UserList.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.UserList.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserListOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UserListOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IUserListOperation} message UserListOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserListOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserListOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UserListOperation} UserListOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UserListOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.UserList.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.UserList.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserListOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UserListOperation} UserListOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserListOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserListOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserListOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.UserList.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.UserList.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserListOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UserListOperation} UserListOperation
                             */
                            UserListOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UserListOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UserListOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UserListOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UserListOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.UserList.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UserListOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.UserList.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserListOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.UserListOperation} m UserListOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserListOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.UserList.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.UserList.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserListOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UserListOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserListOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserListOperation;
                        })();
    
                        services.MutateUserListsResponse = (function() {
    
                            /**
                             * Properties of a MutateUserListsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateUserListsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] MutateUserListsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IMutateUserListResult>|null} [results] MutateUserListsResponse results
                             */
    
                            /**
                             * Constructs a new MutateUserListsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateUserListsResponse.
                             * @implements IMutateUserListsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateUserListsResponse=} [p] Properties to set
                             */
                            function MutateUserListsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateUserListsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @instance
                             */
                            MutateUserListsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * MutateUserListsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateUserListResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @instance
                             */
                            MutateUserListsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateUserListsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateUserListsResponse} MutateUserListsResponse instance
                             */
                            MutateUserListsResponse.create = function create(properties) {
                                return new MutateUserListsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateUserListsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateUserListsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListsResponse} m MutateUserListsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateUserListsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateUserListResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateUserListsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateUserListsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListsResponse} message MutateUserListsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateUserListsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateUserListsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateUserListsResponse} MutateUserListsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateUserListsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateUserListsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateUserListResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateUserListsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateUserListsResponse} MutateUserListsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateUserListsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateUserListsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateUserListsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateUserListResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateUserListsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateUserListsResponse} MutateUserListsResponse
                             */
                            MutateUserListsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateUserListsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateUserListsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateUserListsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateUserListsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateUserListsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateUserListResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateUserListsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateUserListsResponse} m MutateUserListsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateUserListsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateUserListResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateUserListsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateUserListsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateUserListsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateUserListsResponse;
                        })();
    
                        services.MutateUserListResult = (function() {
    
                            /**
                             * Properties of a MutateUserListResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateUserListResult
                             * @property {string|null} [resource_name] MutateUserListResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateUserListResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateUserListResult.
                             * @implements IMutateUserListResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateUserListResult=} [p] Properties to set
                             */
                            function MutateUserListResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateUserListResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @instance
                             */
                            MutateUserListResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateUserListResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateUserListResult} MutateUserListResult instance
                             */
                            MutateUserListResult.create = function create(properties) {
                                return new MutateUserListResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateUserListResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateUserListResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListResult} m MutateUserListResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateUserListResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateUserListResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateUserListResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateUserListResult} message MutateUserListResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateUserListResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateUserListResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateUserListResult} MutateUserListResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateUserListResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateUserListResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateUserListResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateUserListResult} MutateUserListResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateUserListResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateUserListResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateUserListResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateUserListResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateUserListResult} MutateUserListResult
                             */
                            MutateUserListResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateUserListResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateUserListResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateUserListResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateUserListResult} m MutateUserListResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateUserListResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateUserListResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateUserListResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateUserListResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateUserListResult;
                        })();
    
                        services.BillingSetupService = (function() {
    
                            /**
                             * Constructs a new BillingSetupService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BillingSetupService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function BillingSetupService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (BillingSetupService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = BillingSetupService;
    
                            /**
                             * Creates new BillingSetupService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BillingSetupService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {BillingSetupService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            BillingSetupService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.BillingSetupService|mutateBillingSetup}.
                             * @memberof google.ads.googleads.v11.services.BillingSetupService
                             * @typedef MutateBillingSetupCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateBillingSetupResponse} [response] MutateBillingSetupResponse
                             */
    
                            /**
                             * Calls MutateBillingSetup.
                             * @function mutateBillingSetup
                             * @memberof google.ads.googleads.v11.services.BillingSetupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupRequest} request MutateBillingSetupRequest message or plain object
                             * @param {google.ads.googleads.v11.services.BillingSetupService.MutateBillingSetupCallback} callback Node-style callback called with the error, if any, and MutateBillingSetupResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(BillingSetupService.prototype.mutateBillingSetup = function mutateBillingSetup(request, callback) {
                                return this.rpcCall(mutateBillingSetup, $root.google.ads.googleads.v11.services.MutateBillingSetupRequest, $root.google.ads.googleads.v11.services.MutateBillingSetupResponse, request, callback);
                            }, "name", { value: "MutateBillingSetup" });
    
                            /**
                             * Calls MutateBillingSetup.
                             * @function mutateBillingSetup
                             * @memberof google.ads.googleads.v11.services.BillingSetupService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupRequest} request MutateBillingSetupRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateBillingSetupResponse>} Promise
                             * @variation 2
                             */
    
                            return BillingSetupService;
                        })();
    
                        services.MutateBillingSetupRequest = (function() {
    
                            /**
                             * Properties of a MutateBillingSetupRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBillingSetupRequest
                             * @property {string|null} [customer_id] MutateBillingSetupRequest customer_id
                             * @property {google.ads.googleads.v11.services.IBillingSetupOperation|null} [operation] MutateBillingSetupRequest operation
                             */
    
                            /**
                             * Constructs a new MutateBillingSetupRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBillingSetupRequest.
                             * @implements IMutateBillingSetupRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupRequest=} [p] Properties to set
                             */
                            function MutateBillingSetupRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBillingSetupRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @instance
                             */
                            MutateBillingSetupRequest.prototype.customer_id = "";
    
                            /**
                             * MutateBillingSetupRequest operation.
                             * @member {google.ads.googleads.v11.services.IBillingSetupOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @instance
                             */
                            MutateBillingSetupRequest.prototype.operation = null;
    
                            /**
                             * Creates a new MutateBillingSetupRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupRequest} MutateBillingSetupRequest instance
                             */
                            MutateBillingSetupRequest.create = function create(properties) {
                                return new MutateBillingSetupRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBillingSetupRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBillingSetupRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupRequest} m MutateBillingSetupRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBillingSetupRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.BillingSetupOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBillingSetupRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBillingSetupRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupRequest} message MutateBillingSetupRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBillingSetupRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBillingSetupRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupRequest} MutateBillingSetupRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBillingSetupRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBillingSetupRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.BillingSetupOperation.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBillingSetupRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupRequest} MutateBillingSetupRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBillingSetupRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBillingSetupRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBillingSetupRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.BillingSetupOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBillingSetupRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupRequest} MutateBillingSetupRequest
                             */
                            MutateBillingSetupRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBillingSetupRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBillingSetupRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBillingSetupRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.BillingSetupOperation.fromObject(d.operation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBillingSetupRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBillingSetupRequest} m MutateBillingSetupRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBillingSetupRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.BillingSetupOperation.toObject(m.operation, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBillingSetupRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBillingSetupRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBillingSetupRequest;
                        })();
    
                        services.BillingSetupOperation = (function() {
    
                            /**
                             * Properties of a BillingSetupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IBillingSetupOperation
                             * @property {google.ads.googleads.v11.resources.IBillingSetup|null} [create] BillingSetupOperation create
                             * @property {string|null} [remove] BillingSetupOperation remove
                             */
    
                            /**
                             * Constructs a new BillingSetupOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a BillingSetupOperation.
                             * @implements IBillingSetupOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IBillingSetupOperation=} [p] Properties to set
                             */
                            function BillingSetupOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * BillingSetupOperation create.
                             * @member {google.ads.googleads.v11.resources.IBillingSetup|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @instance
                             */
                            BillingSetupOperation.prototype.create = null;
    
                            /**
                             * BillingSetupOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @instance
                             */
                            BillingSetupOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * BillingSetupOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @instance
                             */
                            Object.defineProperty(BillingSetupOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new BillingSetupOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBillingSetupOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.BillingSetupOperation} BillingSetupOperation instance
                             */
                            BillingSetupOperation.create = function create(properties) {
                                return new BillingSetupOperation(properties);
                            };
    
                            /**
                             * Encodes the specified BillingSetupOperation message. Does not implicitly {@link google.ads.googleads.v11.services.BillingSetupOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBillingSetupOperation} m BillingSetupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetupOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(10).string(m.remove);
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.BillingSetup.encode(m.create, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified BillingSetupOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.BillingSetupOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IBillingSetupOperation} message BillingSetupOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            BillingSetupOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a BillingSetupOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.BillingSetupOperation} BillingSetupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetupOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.BillingSetupOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.create = $root.google.ads.googleads.v11.resources.BillingSetup.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a BillingSetupOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.BillingSetupOperation} BillingSetupOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            BillingSetupOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a BillingSetupOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            BillingSetupOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.BillingSetup.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a BillingSetupOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.BillingSetupOperation} BillingSetupOperation
                             */
                            BillingSetupOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.BillingSetupOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.BillingSetupOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.BillingSetupOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.BillingSetup.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a BillingSetupOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.BillingSetupOperation} m BillingSetupOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            BillingSetupOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.BillingSetup.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this BillingSetupOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.BillingSetupOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            BillingSetupOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return BillingSetupOperation;
                        })();
    
                        services.MutateBillingSetupResponse = (function() {
    
                            /**
                             * Properties of a MutateBillingSetupResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBillingSetupResponse
                             * @property {google.ads.googleads.v11.services.IMutateBillingSetupResult|null} [result] MutateBillingSetupResponse result
                             */
    
                            /**
                             * Constructs a new MutateBillingSetupResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBillingSetupResponse.
                             * @implements IMutateBillingSetupResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResponse=} [p] Properties to set
                             */
                            function MutateBillingSetupResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBillingSetupResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateBillingSetupResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @instance
                             */
                            MutateBillingSetupResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateBillingSetupResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResponse} MutateBillingSetupResponse instance
                             */
                            MutateBillingSetupResponse.create = function create(properties) {
                                return new MutateBillingSetupResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBillingSetupResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBillingSetupResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResponse} m MutateBillingSetupResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBillingSetupResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateBillingSetupResult.encode(m.result, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBillingSetupResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBillingSetupResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResponse} message MutateBillingSetupResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBillingSetupResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBillingSetupResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResponse} MutateBillingSetupResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBillingSetupResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBillingSetupResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.result = $root.google.ads.googleads.v11.services.MutateBillingSetupResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBillingSetupResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResponse} MutateBillingSetupResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBillingSetupResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBillingSetupResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBillingSetupResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateBillingSetupResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBillingSetupResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResponse} MutateBillingSetupResponse
                             */
                            MutateBillingSetupResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBillingSetupResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBillingSetupResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateBillingSetupResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateBillingSetupResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBillingSetupResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBillingSetupResponse} m MutateBillingSetupResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBillingSetupResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateBillingSetupResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBillingSetupResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBillingSetupResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBillingSetupResponse;
                        })();
    
                        services.MutateBillingSetupResult = (function() {
    
                            /**
                             * Properties of a MutateBillingSetupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateBillingSetupResult
                             * @property {string|null} [resource_name] MutateBillingSetupResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateBillingSetupResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateBillingSetupResult.
                             * @implements IMutateBillingSetupResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResult=} [p] Properties to set
                             */
                            function MutateBillingSetupResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateBillingSetupResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @instance
                             */
                            MutateBillingSetupResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateBillingSetupResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResult} MutateBillingSetupResult instance
                             */
                            MutateBillingSetupResult.create = function create(properties) {
                                return new MutateBillingSetupResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateBillingSetupResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateBillingSetupResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResult} m MutateBillingSetupResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBillingSetupResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateBillingSetupResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateBillingSetupResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateBillingSetupResult} message MutateBillingSetupResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateBillingSetupResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateBillingSetupResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResult} MutateBillingSetupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBillingSetupResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateBillingSetupResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateBillingSetupResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResult} MutateBillingSetupResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateBillingSetupResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateBillingSetupResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateBillingSetupResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateBillingSetupResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateBillingSetupResult} MutateBillingSetupResult
                             */
                            MutateBillingSetupResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateBillingSetupResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateBillingSetupResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateBillingSetupResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateBillingSetupResult} m MutateBillingSetupResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateBillingSetupResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateBillingSetupResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateBillingSetupResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateBillingSetupResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateBillingSetupResult;
                        })();
    
                        services.ConversionAdjustmentUploadService = (function() {
    
                            /**
                             * Constructs a new ConversionAdjustmentUploadService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionAdjustmentUploadService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ConversionAdjustmentUploadService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ConversionAdjustmentUploadService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ConversionAdjustmentUploadService;
    
                            /**
                             * Creates new ConversionAdjustmentUploadService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentUploadService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ConversionAdjustmentUploadService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ConversionAdjustmentUploadService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionAdjustmentUploadService|uploadConversionAdjustments}.
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentUploadService
                             * @typedef UploadConversionAdjustmentsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse} [response] UploadConversionAdjustmentsResponse
                             */
    
                            /**
                             * Calls UploadConversionAdjustments.
                             * @function uploadConversionAdjustments
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentUploadService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsRequest} request UploadConversionAdjustmentsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionAdjustmentUploadService.UploadConversionAdjustmentsCallback} callback Node-style callback called with the error, if any, and UploadConversionAdjustmentsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionAdjustmentUploadService.prototype.uploadConversionAdjustments = function uploadConversionAdjustments(request, callback) {
                                return this.rpcCall(uploadConversionAdjustments, $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest, $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse, request, callback);
                            }, "name", { value: "UploadConversionAdjustments" });
    
                            /**
                             * Calls UploadConversionAdjustments.
                             * @function uploadConversionAdjustments
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentUploadService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsRequest} request UploadConversionAdjustmentsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse>} Promise
                             * @variation 2
                             */
    
                            return ConversionAdjustmentUploadService;
                        })();
    
                        services.UploadConversionAdjustmentsRequest = (function() {
    
                            /**
                             * Properties of an UploadConversionAdjustmentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadConversionAdjustmentsRequest
                             * @property {string|null} [customer_id] UploadConversionAdjustmentsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IConversionAdjustment>|null} [conversion_adjustments] UploadConversionAdjustmentsRequest conversion_adjustments
                             * @property {boolean|null} [partial_failure] UploadConversionAdjustmentsRequest partial_failure
                             * @property {boolean|null} [validate_only] UploadConversionAdjustmentsRequest validate_only
                             */
    
                            /**
                             * Constructs a new UploadConversionAdjustmentsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadConversionAdjustmentsRequest.
                             * @implements IUploadConversionAdjustmentsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsRequest=} [p] Properties to set
                             */
                            function UploadConversionAdjustmentsRequest(p) {
                                this.conversion_adjustments = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadConversionAdjustmentsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @instance
                             */
                            UploadConversionAdjustmentsRequest.prototype.customer_id = "";
    
                            /**
                             * UploadConversionAdjustmentsRequest conversion_adjustments.
                             * @member {Array.<google.ads.googleads.v11.services.IConversionAdjustment>} conversion_adjustments
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @instance
                             */
                            UploadConversionAdjustmentsRequest.prototype.conversion_adjustments = $util.emptyArray;
    
                            /**
                             * UploadConversionAdjustmentsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @instance
                             */
                            UploadConversionAdjustmentsRequest.prototype.partial_failure = false;
    
                            /**
                             * UploadConversionAdjustmentsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @instance
                             */
                            UploadConversionAdjustmentsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new UploadConversionAdjustmentsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest} UploadConversionAdjustmentsRequest instance
                             */
                            UploadConversionAdjustmentsRequest.create = function create(properties) {
                                return new UploadConversionAdjustmentsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified UploadConversionAdjustmentsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsRequest} m UploadConversionAdjustmentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadConversionAdjustmentsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.conversion_adjustments != null && m.conversion_adjustments.length) {
                                    for (var i = 0; i < m.conversion_adjustments.length; ++i)
                                        $root.google.ads.googleads.v11.services.ConversionAdjustment.encode(m.conversion_adjustments[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadConversionAdjustmentsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsRequest} message UploadConversionAdjustmentsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadConversionAdjustmentsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadConversionAdjustmentsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest} UploadConversionAdjustmentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadConversionAdjustmentsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.conversion_adjustments && m.conversion_adjustments.length))
                                            m.conversion_adjustments = [];
                                        m.conversion_adjustments.push($root.google.ads.googleads.v11.services.ConversionAdjustment.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadConversionAdjustmentsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest} UploadConversionAdjustmentsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadConversionAdjustmentsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadConversionAdjustmentsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadConversionAdjustmentsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.conversion_adjustments != null && m.hasOwnProperty("conversion_adjustments")) {
                                    if (!Array.isArray(m.conversion_adjustments))
                                        return "conversion_adjustments: array expected";
                                    for (var i = 0; i < m.conversion_adjustments.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ConversionAdjustment.verify(m.conversion_adjustments[i]);
                                            if (e)
                                                return "conversion_adjustments." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadConversionAdjustmentsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest} UploadConversionAdjustmentsRequest
                             */
                            UploadConversionAdjustmentsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.conversion_adjustments) {
                                    if (!Array.isArray(d.conversion_adjustments))
                                        throw TypeError(".google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest.conversion_adjustments: array expected");
                                    m.conversion_adjustments = [];
                                    for (var i = 0; i < d.conversion_adjustments.length; ++i) {
                                        if (typeof d.conversion_adjustments[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest.conversion_adjustments: object expected");
                                        m.conversion_adjustments[i] = $root.google.ads.googleads.v11.services.ConversionAdjustment.fromObject(d.conversion_adjustments[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadConversionAdjustmentsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest} m UploadConversionAdjustmentsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadConversionAdjustmentsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.conversion_adjustments = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.conversion_adjustments && m.conversion_adjustments.length) {
                                    d.conversion_adjustments = [];
                                    for (var j = 0; j < m.conversion_adjustments.length; ++j) {
                                        d.conversion_adjustments[j] = $root.google.ads.googleads.v11.services.ConversionAdjustment.toObject(m.conversion_adjustments[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadConversionAdjustmentsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadConversionAdjustmentsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadConversionAdjustmentsRequest;
                        })();
    
                        services.UploadConversionAdjustmentsResponse = (function() {
    
                            /**
                             * Properties of an UploadConversionAdjustmentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadConversionAdjustmentsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] UploadConversionAdjustmentsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IConversionAdjustmentResult>|null} [results] UploadConversionAdjustmentsResponse results
                             */
    
                            /**
                             * Constructs a new UploadConversionAdjustmentsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadConversionAdjustmentsResponse.
                             * @implements IUploadConversionAdjustmentsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsResponse=} [p] Properties to set
                             */
                            function UploadConversionAdjustmentsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadConversionAdjustmentsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @instance
                             */
                            UploadConversionAdjustmentsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * UploadConversionAdjustmentsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IConversionAdjustmentResult>} results
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @instance
                             */
                            UploadConversionAdjustmentsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new UploadConversionAdjustmentsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse} UploadConversionAdjustmentsResponse instance
                             */
                            UploadConversionAdjustmentsResponse.create = function create(properties) {
                                return new UploadConversionAdjustmentsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified UploadConversionAdjustmentsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsResponse} m UploadConversionAdjustmentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadConversionAdjustmentsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.ConversionAdjustmentResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadConversionAdjustmentsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadConversionAdjustmentsResponse} message UploadConversionAdjustmentsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadConversionAdjustmentsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadConversionAdjustmentsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse} UploadConversionAdjustmentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadConversionAdjustmentsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.ConversionAdjustmentResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadConversionAdjustmentsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse} UploadConversionAdjustmentsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadConversionAdjustmentsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadConversionAdjustmentsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadConversionAdjustmentsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ConversionAdjustmentResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadConversionAdjustmentsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse} UploadConversionAdjustmentsResponse
                             */
                            UploadConversionAdjustmentsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.ConversionAdjustmentResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadConversionAdjustmentsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse} m UploadConversionAdjustmentsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadConversionAdjustmentsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.ConversionAdjustmentResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadConversionAdjustmentsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadConversionAdjustmentsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadConversionAdjustmentsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadConversionAdjustmentsResponse;
                        })();
    
                        services.ConversionAdjustment = (function() {
    
                            /**
                             * Properties of a ConversionAdjustment.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IConversionAdjustment
                             * @property {google.ads.googleads.v11.services.IGclidDateTimePair|null} [gclid_date_time_pair] ConversionAdjustment gclid_date_time_pair
                             * @property {string|null} [order_id] ConversionAdjustment order_id
                             * @property {string|null} [conversion_action] ConversionAdjustment conversion_action
                             * @property {string|null} [adjustment_date_time] ConversionAdjustment adjustment_date_time
                             * @property {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.ConversionAdjustmentType|null} [adjustment_type] ConversionAdjustment adjustment_type
                             * @property {google.ads.googleads.v11.services.IRestatementValue|null} [restatement_value] ConversionAdjustment restatement_value
                             * @property {Array.<google.ads.googleads.v11.common.IUserIdentifier>|null} [user_identifiers] ConversionAdjustment user_identifiers
                             * @property {string|null} [user_agent] ConversionAdjustment user_agent
                             */
    
                            /**
                             * Constructs a new ConversionAdjustment.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionAdjustment.
                             * @implements IConversionAdjustment
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IConversionAdjustment=} [p] Properties to set
                             */
                            function ConversionAdjustment(p) {
                                this.user_identifiers = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionAdjustment gclid_date_time_pair.
                             * @member {google.ads.googleads.v11.services.IGclidDateTimePair|null|undefined} gclid_date_time_pair
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.gclid_date_time_pair = null;
    
                            /**
                             * ConversionAdjustment order_id.
                             * @member {string|null|undefined} order_id
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.order_id = null;
    
                            /**
                             * ConversionAdjustment conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.conversion_action = null;
    
                            /**
                             * ConversionAdjustment adjustment_date_time.
                             * @member {string|null|undefined} adjustment_date_time
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.adjustment_date_time = null;
    
                            /**
                             * ConversionAdjustment adjustment_type.
                             * @member {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.ConversionAdjustmentType} adjustment_type
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.adjustment_type = 0;
    
                            /**
                             * ConversionAdjustment restatement_value.
                             * @member {google.ads.googleads.v11.services.IRestatementValue|null|undefined} restatement_value
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.restatement_value = null;
    
                            /**
                             * ConversionAdjustment user_identifiers.
                             * @member {Array.<google.ads.googleads.v11.common.IUserIdentifier>} user_identifiers
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.user_identifiers = $util.emptyArray;
    
                            /**
                             * ConversionAdjustment user_agent.
                             * @member {string|null|undefined} user_agent
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            ConversionAdjustment.prototype.user_agent = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionAdjustment _order_id.
                             * @member {"order_id"|undefined} _order_id
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            Object.defineProperty(ConversionAdjustment.prototype, "_order_id", {
                                get: $util.oneOfGetter($oneOfFields = ["order_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAdjustment _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            Object.defineProperty(ConversionAdjustment.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAdjustment _adjustment_date_time.
                             * @member {"adjustment_date_time"|undefined} _adjustment_date_time
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            Object.defineProperty(ConversionAdjustment.prototype, "_adjustment_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["adjustment_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAdjustment _user_agent.
                             * @member {"user_agent"|undefined} _user_agent
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             */
                            Object.defineProperty(ConversionAdjustment.prototype, "_user_agent", {
                                get: $util.oneOfGetter($oneOfFields = ["user_agent"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionAdjustment instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionAdjustment=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustment} ConversionAdjustment instance
                             */
                            ConversionAdjustment.create = function create(properties) {
                                return new ConversionAdjustment(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustment message. Does not implicitly {@link google.ads.googleads.v11.services.ConversionAdjustment.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionAdjustment} m ConversionAdjustment message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustment.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.adjustment_type != null && Object.hasOwnProperty.call(m, "adjustment_type"))
                                    w.uint32(40).int32(m.adjustment_type);
                                if (m.restatement_value != null && Object.hasOwnProperty.call(m, "restatement_value"))
                                    $root.google.ads.googleads.v11.services.RestatementValue.encode(m.restatement_value, w.uint32(50).fork()).ldelim();
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(66).string(m.conversion_action);
                                if (m.adjustment_date_time != null && Object.hasOwnProperty.call(m, "adjustment_date_time"))
                                    w.uint32(74).string(m.adjustment_date_time);
                                if (m.user_identifiers != null && m.user_identifiers.length) {
                                    for (var i = 0; i < m.user_identifiers.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserIdentifier.encode(m.user_identifiers[i], w.uint32(82).fork()).ldelim();
                                }
                                if (m.user_agent != null && Object.hasOwnProperty.call(m, "user_agent"))
                                    w.uint32(90).string(m.user_agent);
                                if (m.gclid_date_time_pair != null && Object.hasOwnProperty.call(m, "gclid_date_time_pair"))
                                    $root.google.ads.googleads.v11.services.GclidDateTimePair.encode(m.gclid_date_time_pair, w.uint32(98).fork()).ldelim();
                                if (m.order_id != null && Object.hasOwnProperty.call(m, "order_id"))
                                    w.uint32(106).string(m.order_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustment message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ConversionAdjustment.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionAdjustment} message ConversionAdjustment message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustment.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionAdjustment message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustment} ConversionAdjustment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustment.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ConversionAdjustment();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 12:
                                        m.gclid_date_time_pair = $root.google.ads.googleads.v11.services.GclidDateTimePair.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.order_id = r.string();
                                        break;
                                    case 8:
                                        m.conversion_action = r.string();
                                        break;
                                    case 9:
                                        m.adjustment_date_time = r.string();
                                        break;
                                    case 5:
                                        m.adjustment_type = r.int32();
                                        break;
                                    case 6:
                                        m.restatement_value = $root.google.ads.googleads.v11.services.RestatementValue.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        if (!(m.user_identifiers && m.user_identifiers.length))
                                            m.user_identifiers = [];
                                        m.user_identifiers.push($root.google.ads.googleads.v11.common.UserIdentifier.decode(r, r.uint32()));
                                        break;
                                    case 11:
                                        m.user_agent = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionAdjustment message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustment} ConversionAdjustment
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustment.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionAdjustment message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionAdjustment.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.gclid_date_time_pair != null && m.hasOwnProperty("gclid_date_time_pair")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.GclidDateTimePair.verify(m.gclid_date_time_pair);
                                        if (e)
                                            return "gclid_date_time_pair." + e;
                                    }
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    p._order_id = 1;
                                    if (!$util.isString(m.order_id))
                                        return "order_id: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.adjustment_date_time != null && m.hasOwnProperty("adjustment_date_time")) {
                                    p._adjustment_date_time = 1;
                                    if (!$util.isString(m.adjustment_date_time))
                                        return "adjustment_date_time: string expected";
                                }
                                if (m.adjustment_type != null && m.hasOwnProperty("adjustment_type")) {
                                    switch (m.adjustment_type) {
                                    default:
                                        return "adjustment_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.restatement_value != null && m.hasOwnProperty("restatement_value")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.RestatementValue.verify(m.restatement_value);
                                        if (e)
                                            return "restatement_value." + e;
                                    }
                                }
                                if (m.user_identifiers != null && m.hasOwnProperty("user_identifiers")) {
                                    if (!Array.isArray(m.user_identifiers))
                                        return "user_identifiers: array expected";
                                    for (var i = 0; i < m.user_identifiers.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserIdentifier.verify(m.user_identifiers[i]);
                                            if (e)
                                                return "user_identifiers." + e;
                                        }
                                    }
                                }
                                if (m.user_agent != null && m.hasOwnProperty("user_agent")) {
                                    p._user_agent = 1;
                                    if (!$util.isString(m.user_agent))
                                        return "user_agent: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionAdjustment message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustment} ConversionAdjustment
                             */
                            ConversionAdjustment.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ConversionAdjustment)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ConversionAdjustment();
                                if (d.gclid_date_time_pair != null) {
                                    if (typeof d.gclid_date_time_pair !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionAdjustment.gclid_date_time_pair: object expected");
                                    m.gclid_date_time_pair = $root.google.ads.googleads.v11.services.GclidDateTimePair.fromObject(d.gclid_date_time_pair);
                                }
                                if (d.order_id != null) {
                                    m.order_id = String(d.order_id);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.adjustment_date_time != null) {
                                    m.adjustment_date_time = String(d.adjustment_date_time);
                                }
                                switch (d.adjustment_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.adjustment_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.adjustment_type = 1;
                                    break;
                                case "RETRACTION":
                                case 2:
                                    m.adjustment_type = 2;
                                    break;
                                case "RESTATEMENT":
                                case 3:
                                    m.adjustment_type = 3;
                                    break;
                                case "ENHANCEMENT":
                                case 4:
                                    m.adjustment_type = 4;
                                    break;
                                }
                                if (d.restatement_value != null) {
                                    if (typeof d.restatement_value !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionAdjustment.restatement_value: object expected");
                                    m.restatement_value = $root.google.ads.googleads.v11.services.RestatementValue.fromObject(d.restatement_value);
                                }
                                if (d.user_identifiers) {
                                    if (!Array.isArray(d.user_identifiers))
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionAdjustment.user_identifiers: array expected");
                                    m.user_identifiers = [];
                                    for (var i = 0; i < d.user_identifiers.length; ++i) {
                                        if (typeof d.user_identifiers[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ConversionAdjustment.user_identifiers: object expected");
                                        m.user_identifiers[i] = $root.google.ads.googleads.v11.common.UserIdentifier.fromObject(d.user_identifiers[i]);
                                    }
                                }
                                if (d.user_agent != null) {
                                    m.user_agent = String(d.user_agent);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionAdjustment message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @static
                             * @param {google.ads.googleads.v11.services.ConversionAdjustment} m ConversionAdjustment
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionAdjustment.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.user_identifiers = [];
                                }
                                if (o.defaults) {
                                    d.adjustment_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.restatement_value = null;
                                    d.gclid_date_time_pair = null;
                                }
                                if (m.adjustment_type != null && m.hasOwnProperty("adjustment_type")) {
                                    d.adjustment_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.ConversionAdjustmentType[m.adjustment_type] : m.adjustment_type;
                                }
                                if (m.restatement_value != null && m.hasOwnProperty("restatement_value")) {
                                    d.restatement_value = $root.google.ads.googleads.v11.services.RestatementValue.toObject(m.restatement_value, o);
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.adjustment_date_time != null && m.hasOwnProperty("adjustment_date_time")) {
                                    d.adjustment_date_time = m.adjustment_date_time;
                                    if (o.oneofs)
                                        d._adjustment_date_time = "adjustment_date_time";
                                }
                                if (m.user_identifiers && m.user_identifiers.length) {
                                    d.user_identifiers = [];
                                    for (var j = 0; j < m.user_identifiers.length; ++j) {
                                        d.user_identifiers[j] = $root.google.ads.googleads.v11.common.UserIdentifier.toObject(m.user_identifiers[j], o);
                                    }
                                }
                                if (m.user_agent != null && m.hasOwnProperty("user_agent")) {
                                    d.user_agent = m.user_agent;
                                    if (o.oneofs)
                                        d._user_agent = "user_agent";
                                }
                                if (m.gclid_date_time_pair != null && m.hasOwnProperty("gclid_date_time_pair")) {
                                    d.gclid_date_time_pair = $root.google.ads.googleads.v11.services.GclidDateTimePair.toObject(m.gclid_date_time_pair, o);
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    d.order_id = m.order_id;
                                    if (o.oneofs)
                                        d._order_id = "order_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionAdjustment to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustment
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionAdjustment.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionAdjustment;
                        })();
    
                        services.RestatementValue = (function() {
    
                            /**
                             * Properties of a RestatementValue.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IRestatementValue
                             * @property {number|null} [adjusted_value] RestatementValue adjusted_value
                             * @property {string|null} [currency_code] RestatementValue currency_code
                             */
    
                            /**
                             * Constructs a new RestatementValue.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RestatementValue.
                             * @implements IRestatementValue
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IRestatementValue=} [p] Properties to set
                             */
                            function RestatementValue(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RestatementValue adjusted_value.
                             * @member {number|null|undefined} adjusted_value
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @instance
                             */
                            RestatementValue.prototype.adjusted_value = null;
    
                            /**
                             * RestatementValue currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @instance
                             */
                            RestatementValue.prototype.currency_code = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * RestatementValue _adjusted_value.
                             * @member {"adjusted_value"|undefined} _adjusted_value
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @instance
                             */
                            Object.defineProperty(RestatementValue.prototype, "_adjusted_value", {
                                get: $util.oneOfGetter($oneOfFields = ["adjusted_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * RestatementValue _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @instance
                             */
                            Object.defineProperty(RestatementValue.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new RestatementValue instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {google.ads.googleads.v11.services.IRestatementValue=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.RestatementValue} RestatementValue instance
                             */
                            RestatementValue.create = function create(properties) {
                                return new RestatementValue(properties);
                            };
    
                            /**
                             * Encodes the specified RestatementValue message. Does not implicitly {@link google.ads.googleads.v11.services.RestatementValue.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {google.ads.googleads.v11.services.IRestatementValue} m RestatementValue message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RestatementValue.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.adjusted_value != null && Object.hasOwnProperty.call(m, "adjusted_value"))
                                    w.uint32(25).double(m.adjusted_value);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(34).string(m.currency_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified RestatementValue message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.RestatementValue.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {google.ads.googleads.v11.services.IRestatementValue} message RestatementValue message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RestatementValue.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RestatementValue message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.RestatementValue} RestatementValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RestatementValue.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.RestatementValue();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.adjusted_value = r.double();
                                        break;
                                    case 4:
                                        m.currency_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RestatementValue message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.RestatementValue} RestatementValue
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RestatementValue.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RestatementValue message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RestatementValue.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.adjusted_value != null && m.hasOwnProperty("adjusted_value")) {
                                    p._adjusted_value = 1;
                                    if (typeof m.adjusted_value !== "number")
                                        return "adjusted_value: number expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RestatementValue message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.RestatementValue} RestatementValue
                             */
                            RestatementValue.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.RestatementValue)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.RestatementValue();
                                if (d.adjusted_value != null) {
                                    m.adjusted_value = Number(d.adjusted_value);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RestatementValue message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @static
                             * @param {google.ads.googleads.v11.services.RestatementValue} m RestatementValue
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RestatementValue.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.adjusted_value != null && m.hasOwnProperty("adjusted_value")) {
                                    d.adjusted_value = o.json && !isFinite(m.adjusted_value) ? String(m.adjusted_value) : m.adjusted_value;
                                    if (o.oneofs)
                                        d._adjusted_value = "adjusted_value";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RestatementValue to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.RestatementValue
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RestatementValue.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RestatementValue;
                        })();
    
                        services.GclidDateTimePair = (function() {
    
                            /**
                             * Properties of a GclidDateTimePair.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGclidDateTimePair
                             * @property {string|null} [gclid] GclidDateTimePair gclid
                             * @property {string|null} [conversion_date_time] GclidDateTimePair conversion_date_time
                             */
    
                            /**
                             * Constructs a new GclidDateTimePair.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GclidDateTimePair.
                             * @implements IGclidDateTimePair
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGclidDateTimePair=} [p] Properties to set
                             */
                            function GclidDateTimePair(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GclidDateTimePair gclid.
                             * @member {string|null|undefined} gclid
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @instance
                             */
                            GclidDateTimePair.prototype.gclid = null;
    
                            /**
                             * GclidDateTimePair conversion_date_time.
                             * @member {string|null|undefined} conversion_date_time
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @instance
                             */
                            GclidDateTimePair.prototype.conversion_date_time = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GclidDateTimePair _gclid.
                             * @member {"gclid"|undefined} _gclid
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @instance
                             */
                            Object.defineProperty(GclidDateTimePair.prototype, "_gclid", {
                                get: $util.oneOfGetter($oneOfFields = ["gclid"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GclidDateTimePair _conversion_date_time.
                             * @member {"conversion_date_time"|undefined} _conversion_date_time
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @instance
                             */
                            Object.defineProperty(GclidDateTimePair.prototype, "_conversion_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GclidDateTimePair instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {google.ads.googleads.v11.services.IGclidDateTimePair=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GclidDateTimePair} GclidDateTimePair instance
                             */
                            GclidDateTimePair.create = function create(properties) {
                                return new GclidDateTimePair(properties);
                            };
    
                            /**
                             * Encodes the specified GclidDateTimePair message. Does not implicitly {@link google.ads.googleads.v11.services.GclidDateTimePair.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {google.ads.googleads.v11.services.IGclidDateTimePair} m GclidDateTimePair message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GclidDateTimePair.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.gclid != null && Object.hasOwnProperty.call(m, "gclid"))
                                    w.uint32(26).string(m.gclid);
                                if (m.conversion_date_time != null && Object.hasOwnProperty.call(m, "conversion_date_time"))
                                    w.uint32(34).string(m.conversion_date_time);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GclidDateTimePair message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GclidDateTimePair.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {google.ads.googleads.v11.services.IGclidDateTimePair} message GclidDateTimePair message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GclidDateTimePair.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GclidDateTimePair message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GclidDateTimePair} GclidDateTimePair
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GclidDateTimePair.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GclidDateTimePair();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.gclid = r.string();
                                        break;
                                    case 4:
                                        m.conversion_date_time = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GclidDateTimePair message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GclidDateTimePair} GclidDateTimePair
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GclidDateTimePair.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GclidDateTimePair message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GclidDateTimePair.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    p._gclid = 1;
                                    if (!$util.isString(m.gclid))
                                        return "gclid: string expected";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    p._conversion_date_time = 1;
                                    if (!$util.isString(m.conversion_date_time))
                                        return "conversion_date_time: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GclidDateTimePair message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GclidDateTimePair} GclidDateTimePair
                             */
                            GclidDateTimePair.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GclidDateTimePair)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GclidDateTimePair();
                                if (d.gclid != null) {
                                    m.gclid = String(d.gclid);
                                }
                                if (d.conversion_date_time != null) {
                                    m.conversion_date_time = String(d.conversion_date_time);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GclidDateTimePair message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @static
                             * @param {google.ads.googleads.v11.services.GclidDateTimePair} m GclidDateTimePair
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GclidDateTimePair.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    d.gclid = m.gclid;
                                    if (o.oneofs)
                                        d._gclid = "gclid";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    d.conversion_date_time = m.conversion_date_time;
                                    if (o.oneofs)
                                        d._conversion_date_time = "conversion_date_time";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GclidDateTimePair to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GclidDateTimePair
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GclidDateTimePair.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GclidDateTimePair;
                        })();
    
                        services.ConversionAdjustmentResult = (function() {
    
                            /**
                             * Properties of a ConversionAdjustmentResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IConversionAdjustmentResult
                             * @property {google.ads.googleads.v11.services.IGclidDateTimePair|null} [gclid_date_time_pair] ConversionAdjustmentResult gclid_date_time_pair
                             * @property {string|null} [order_id] ConversionAdjustmentResult order_id
                             * @property {string|null} [conversion_action] ConversionAdjustmentResult conversion_action
                             * @property {string|null} [adjustment_date_time] ConversionAdjustmentResult adjustment_date_time
                             * @property {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.ConversionAdjustmentType|null} [adjustment_type] ConversionAdjustmentResult adjustment_type
                             */
    
                            /**
                             * Constructs a new ConversionAdjustmentResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionAdjustmentResult.
                             * @implements IConversionAdjustmentResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IConversionAdjustmentResult=} [p] Properties to set
                             */
                            function ConversionAdjustmentResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ConversionAdjustmentResult gclid_date_time_pair.
                             * @member {google.ads.googleads.v11.services.IGclidDateTimePair|null|undefined} gclid_date_time_pair
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             */
                            ConversionAdjustmentResult.prototype.gclid_date_time_pair = null;
    
                            /**
                             * ConversionAdjustmentResult order_id.
                             * @member {string} order_id
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             */
                            ConversionAdjustmentResult.prototype.order_id = "";
    
                            /**
                             * ConversionAdjustmentResult conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             */
                            ConversionAdjustmentResult.prototype.conversion_action = null;
    
                            /**
                             * ConversionAdjustmentResult adjustment_date_time.
                             * @member {string|null|undefined} adjustment_date_time
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             */
                            ConversionAdjustmentResult.prototype.adjustment_date_time = null;
    
                            /**
                             * ConversionAdjustmentResult adjustment_type.
                             * @member {google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.ConversionAdjustmentType} adjustment_type
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             */
                            ConversionAdjustmentResult.prototype.adjustment_type = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ConversionAdjustmentResult _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             */
                            Object.defineProperty(ConversionAdjustmentResult.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ConversionAdjustmentResult _adjustment_date_time.
                             * @member {"adjustment_date_time"|undefined} _adjustment_date_time
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             */
                            Object.defineProperty(ConversionAdjustmentResult.prototype, "_adjustment_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["adjustment_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ConversionAdjustmentResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionAdjustmentResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustmentResult} ConversionAdjustmentResult instance
                             */
                            ConversionAdjustmentResult.create = function create(properties) {
                                return new ConversionAdjustmentResult(properties);
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustmentResult message. Does not implicitly {@link google.ads.googleads.v11.services.ConversionAdjustmentResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionAdjustmentResult} m ConversionAdjustmentResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustmentResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.adjustment_type != null && Object.hasOwnProperty.call(m, "adjustment_type"))
                                    w.uint32(40).int32(m.adjustment_type);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(58).string(m.conversion_action);
                                if (m.adjustment_date_time != null && Object.hasOwnProperty.call(m, "adjustment_date_time"))
                                    w.uint32(66).string(m.adjustment_date_time);
                                if (m.gclid_date_time_pair != null && Object.hasOwnProperty.call(m, "gclid_date_time_pair"))
                                    $root.google.ads.googleads.v11.services.GclidDateTimePair.encode(m.gclid_date_time_pair, w.uint32(74).fork()).ldelim();
                                if (m.order_id != null && Object.hasOwnProperty.call(m, "order_id"))
                                    w.uint32(82).string(m.order_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ConversionAdjustmentResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ConversionAdjustmentResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IConversionAdjustmentResult} message ConversionAdjustmentResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ConversionAdjustmentResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ConversionAdjustmentResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustmentResult} ConversionAdjustmentResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustmentResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ConversionAdjustmentResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 9:
                                        m.gclid_date_time_pair = $root.google.ads.googleads.v11.services.GclidDateTimePair.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.order_id = r.string();
                                        break;
                                    case 7:
                                        m.conversion_action = r.string();
                                        break;
                                    case 8:
                                        m.adjustment_date_time = r.string();
                                        break;
                                    case 5:
                                        m.adjustment_type = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ConversionAdjustmentResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustmentResult} ConversionAdjustmentResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ConversionAdjustmentResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ConversionAdjustmentResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ConversionAdjustmentResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.gclid_date_time_pair != null && m.hasOwnProperty("gclid_date_time_pair")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.GclidDateTimePair.verify(m.gclid_date_time_pair);
                                        if (e)
                                            return "gclid_date_time_pair." + e;
                                    }
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    if (!$util.isString(m.order_id))
                                        return "order_id: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.adjustment_date_time != null && m.hasOwnProperty("adjustment_date_time")) {
                                    p._adjustment_date_time = 1;
                                    if (!$util.isString(m.adjustment_date_time))
                                        return "adjustment_date_time: string expected";
                                }
                                if (m.adjustment_type != null && m.hasOwnProperty("adjustment_type")) {
                                    switch (m.adjustment_type) {
                                    default:
                                        return "adjustment_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ConversionAdjustmentResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ConversionAdjustmentResult} ConversionAdjustmentResult
                             */
                            ConversionAdjustmentResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ConversionAdjustmentResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ConversionAdjustmentResult();
                                if (d.gclid_date_time_pair != null) {
                                    if (typeof d.gclid_date_time_pair !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ConversionAdjustmentResult.gclid_date_time_pair: object expected");
                                    m.gclid_date_time_pair = $root.google.ads.googleads.v11.services.GclidDateTimePair.fromObject(d.gclid_date_time_pair);
                                }
                                if (d.order_id != null) {
                                    m.order_id = String(d.order_id);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.adjustment_date_time != null) {
                                    m.adjustment_date_time = String(d.adjustment_date_time);
                                }
                                switch (d.adjustment_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.adjustment_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.adjustment_type = 1;
                                    break;
                                case "RETRACTION":
                                case 2:
                                    m.adjustment_type = 2;
                                    break;
                                case "RESTATEMENT":
                                case 3:
                                    m.adjustment_type = 3;
                                    break;
                                case "ENHANCEMENT":
                                case 4:
                                    m.adjustment_type = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ConversionAdjustmentResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @static
                             * @param {google.ads.googleads.v11.services.ConversionAdjustmentResult} m ConversionAdjustmentResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ConversionAdjustmentResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.adjustment_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.gclid_date_time_pair = null;
                                    d.order_id = "";
                                }
                                if (m.adjustment_type != null && m.hasOwnProperty("adjustment_type")) {
                                    d.adjustment_type = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionAdjustmentTypeEnum.ConversionAdjustmentType[m.adjustment_type] : m.adjustment_type;
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.adjustment_date_time != null && m.hasOwnProperty("adjustment_date_time")) {
                                    d.adjustment_date_time = m.adjustment_date_time;
                                    if (o.oneofs)
                                        d._adjustment_date_time = "adjustment_date_time";
                                }
                                if (m.gclid_date_time_pair != null && m.hasOwnProperty("gclid_date_time_pair")) {
                                    d.gclid_date_time_pair = $root.google.ads.googleads.v11.services.GclidDateTimePair.toObject(m.gclid_date_time_pair, o);
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    d.order_id = m.order_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ConversionAdjustmentResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ConversionAdjustmentResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ConversionAdjustmentResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ConversionAdjustmentResult;
                        })();
    
                        services.ConversionUploadService = (function() {
    
                            /**
                             * Constructs a new ConversionUploadService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ConversionUploadService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ConversionUploadService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ConversionUploadService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ConversionUploadService;
    
                            /**
                             * Creates new ConversionUploadService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ConversionUploadService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ConversionUploadService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ConversionUploadService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionUploadService|uploadClickConversions}.
                             * @memberof google.ads.googleads.v11.services.ConversionUploadService
                             * @typedef UploadClickConversionsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.UploadClickConversionsResponse} [response] UploadClickConversionsResponse
                             */
    
                            /**
                             * Calls UploadClickConversions.
                             * @function uploadClickConversions
                             * @memberof google.ads.googleads.v11.services.ConversionUploadService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsRequest} request UploadClickConversionsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionUploadService.UploadClickConversionsCallback} callback Node-style callback called with the error, if any, and UploadClickConversionsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionUploadService.prototype.uploadClickConversions = function uploadClickConversions(request, callback) {
                                return this.rpcCall(uploadClickConversions, $root.google.ads.googleads.v11.services.UploadClickConversionsRequest, $root.google.ads.googleads.v11.services.UploadClickConversionsResponse, request, callback);
                            }, "name", { value: "UploadClickConversions" });
    
                            /**
                             * Calls UploadClickConversions.
                             * @function uploadClickConversions
                             * @memberof google.ads.googleads.v11.services.ConversionUploadService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsRequest} request UploadClickConversionsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.UploadClickConversionsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ConversionUploadService|uploadCallConversions}.
                             * @memberof google.ads.googleads.v11.services.ConversionUploadService
                             * @typedef UploadCallConversionsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.UploadCallConversionsResponse} [response] UploadCallConversionsResponse
                             */
    
                            /**
                             * Calls UploadCallConversions.
                             * @function uploadCallConversions
                             * @memberof google.ads.googleads.v11.services.ConversionUploadService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsRequest} request UploadCallConversionsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ConversionUploadService.UploadCallConversionsCallback} callback Node-style callback called with the error, if any, and UploadCallConversionsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ConversionUploadService.prototype.uploadCallConversions = function uploadCallConversions(request, callback) {
                                return this.rpcCall(uploadCallConversions, $root.google.ads.googleads.v11.services.UploadCallConversionsRequest, $root.google.ads.googleads.v11.services.UploadCallConversionsResponse, request, callback);
                            }, "name", { value: "UploadCallConversions" });
    
                            /**
                             * Calls UploadCallConversions.
                             * @function uploadCallConversions
                             * @memberof google.ads.googleads.v11.services.ConversionUploadService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsRequest} request UploadCallConversionsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.UploadCallConversionsResponse>} Promise
                             * @variation 2
                             */
    
                            return ConversionUploadService;
                        })();
    
                        services.UploadClickConversionsRequest = (function() {
    
                            /**
                             * Properties of an UploadClickConversionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadClickConversionsRequest
                             * @property {string|null} [customer_id] UploadClickConversionsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IClickConversion>|null} [conversions] UploadClickConversionsRequest conversions
                             * @property {boolean|null} [partial_failure] UploadClickConversionsRequest partial_failure
                             * @property {boolean|null} [validate_only] UploadClickConversionsRequest validate_only
                             */
    
                            /**
                             * Constructs a new UploadClickConversionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadClickConversionsRequest.
                             * @implements IUploadClickConversionsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsRequest=} [p] Properties to set
                             */
                            function UploadClickConversionsRequest(p) {
                                this.conversions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadClickConversionsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @instance
                             */
                            UploadClickConversionsRequest.prototype.customer_id = "";
    
                            /**
                             * UploadClickConversionsRequest conversions.
                             * @member {Array.<google.ads.googleads.v11.services.IClickConversion>} conversions
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @instance
                             */
                            UploadClickConversionsRequest.prototype.conversions = $util.emptyArray;
    
                            /**
                             * UploadClickConversionsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @instance
                             */
                            UploadClickConversionsRequest.prototype.partial_failure = false;
    
                            /**
                             * UploadClickConversionsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @instance
                             */
                            UploadClickConversionsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new UploadClickConversionsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsRequest} UploadClickConversionsRequest instance
                             */
                            UploadClickConversionsRequest.create = function create(properties) {
                                return new UploadClickConversionsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified UploadClickConversionsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.UploadClickConversionsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsRequest} m UploadClickConversionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadClickConversionsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.conversions != null && m.conversions.length) {
                                    for (var i = 0; i < m.conversions.length; ++i)
                                        $root.google.ads.googleads.v11.services.ClickConversion.encode(m.conversions[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadClickConversionsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadClickConversionsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsRequest} message UploadClickConversionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadClickConversionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadClickConversionsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsRequest} UploadClickConversionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadClickConversionsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadClickConversionsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.conversions && m.conversions.length))
                                            m.conversions = [];
                                        m.conversions.push($root.google.ads.googleads.v11.services.ClickConversion.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadClickConversionsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsRequest} UploadClickConversionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadClickConversionsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadClickConversionsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadClickConversionsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.conversions != null && m.hasOwnProperty("conversions")) {
                                    if (!Array.isArray(m.conversions))
                                        return "conversions: array expected";
                                    for (var i = 0; i < m.conversions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ClickConversion.verify(m.conversions[i]);
                                            if (e)
                                                return "conversions." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadClickConversionsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsRequest} UploadClickConversionsRequest
                             */
                            UploadClickConversionsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadClickConversionsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadClickConversionsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.conversions) {
                                    if (!Array.isArray(d.conversions))
                                        throw TypeError(".google.ads.googleads.v11.services.UploadClickConversionsRequest.conversions: array expected");
                                    m.conversions = [];
                                    for (var i = 0; i < d.conversions.length; ++i) {
                                        if (typeof d.conversions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.UploadClickConversionsRequest.conversions: object expected");
                                        m.conversions[i] = $root.google.ads.googleads.v11.services.ClickConversion.fromObject(d.conversions[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadClickConversionsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadClickConversionsRequest} m UploadClickConversionsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadClickConversionsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.conversions = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.conversions && m.conversions.length) {
                                    d.conversions = [];
                                    for (var j = 0; j < m.conversions.length; ++j) {
                                        d.conversions[j] = $root.google.ads.googleads.v11.services.ClickConversion.toObject(m.conversions[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadClickConversionsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadClickConversionsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadClickConversionsRequest;
                        })();
    
                        services.UploadClickConversionsResponse = (function() {
    
                            /**
                             * Properties of an UploadClickConversionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadClickConversionsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] UploadClickConversionsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.IClickConversionResult>|null} [results] UploadClickConversionsResponse results
                             */
    
                            /**
                             * Constructs a new UploadClickConversionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadClickConversionsResponse.
                             * @implements IUploadClickConversionsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsResponse=} [p] Properties to set
                             */
                            function UploadClickConversionsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadClickConversionsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @instance
                             */
                            UploadClickConversionsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * UploadClickConversionsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IClickConversionResult>} results
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @instance
                             */
                            UploadClickConversionsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new UploadClickConversionsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsResponse} UploadClickConversionsResponse instance
                             */
                            UploadClickConversionsResponse.create = function create(properties) {
                                return new UploadClickConversionsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified UploadClickConversionsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.UploadClickConversionsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsResponse} m UploadClickConversionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadClickConversionsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.ClickConversionResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadClickConversionsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadClickConversionsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadClickConversionsResponse} message UploadClickConversionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadClickConversionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadClickConversionsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsResponse} UploadClickConversionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadClickConversionsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadClickConversionsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.ClickConversionResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadClickConversionsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsResponse} UploadClickConversionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadClickConversionsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadClickConversionsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadClickConversionsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ClickConversionResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadClickConversionsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadClickConversionsResponse} UploadClickConversionsResponse
                             */
                            UploadClickConversionsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadClickConversionsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadClickConversionsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UploadClickConversionsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.UploadClickConversionsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.UploadClickConversionsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.ClickConversionResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadClickConversionsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadClickConversionsResponse} m UploadClickConversionsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadClickConversionsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.ClickConversionResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadClickConversionsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadClickConversionsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadClickConversionsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadClickConversionsResponse;
                        })();
    
                        services.UploadCallConversionsRequest = (function() {
    
                            /**
                             * Properties of an UploadCallConversionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadCallConversionsRequest
                             * @property {string|null} [customer_id] UploadCallConversionsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICallConversion>|null} [conversions] UploadCallConversionsRequest conversions
                             * @property {boolean|null} [partial_failure] UploadCallConversionsRequest partial_failure
                             * @property {boolean|null} [validate_only] UploadCallConversionsRequest validate_only
                             */
    
                            /**
                             * Constructs a new UploadCallConversionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadCallConversionsRequest.
                             * @implements IUploadCallConversionsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsRequest=} [p] Properties to set
                             */
                            function UploadCallConversionsRequest(p) {
                                this.conversions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadCallConversionsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @instance
                             */
                            UploadCallConversionsRequest.prototype.customer_id = "";
    
                            /**
                             * UploadCallConversionsRequest conversions.
                             * @member {Array.<google.ads.googleads.v11.services.ICallConversion>} conversions
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @instance
                             */
                            UploadCallConversionsRequest.prototype.conversions = $util.emptyArray;
    
                            /**
                             * UploadCallConversionsRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @instance
                             */
                            UploadCallConversionsRequest.prototype.partial_failure = false;
    
                            /**
                             * UploadCallConversionsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @instance
                             */
                            UploadCallConversionsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new UploadCallConversionsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsRequest} UploadCallConversionsRequest instance
                             */
                            UploadCallConversionsRequest.create = function create(properties) {
                                return new UploadCallConversionsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified UploadCallConversionsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.UploadCallConversionsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsRequest} m UploadCallConversionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadCallConversionsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.conversions != null && m.conversions.length) {
                                    for (var i = 0; i < m.conversions.length; ++i)
                                        $root.google.ads.googleads.v11.services.CallConversion.encode(m.conversions[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadCallConversionsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadCallConversionsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsRequest} message UploadCallConversionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadCallConversionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadCallConversionsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsRequest} UploadCallConversionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadCallConversionsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadCallConversionsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.conversions && m.conversions.length))
                                            m.conversions = [];
                                        m.conversions.push($root.google.ads.googleads.v11.services.CallConversion.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadCallConversionsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsRequest} UploadCallConversionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadCallConversionsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadCallConversionsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadCallConversionsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.conversions != null && m.hasOwnProperty("conversions")) {
                                    if (!Array.isArray(m.conversions))
                                        return "conversions: array expected";
                                    for (var i = 0; i < m.conversions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CallConversion.verify(m.conversions[i]);
                                            if (e)
                                                return "conversions." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadCallConversionsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsRequest} UploadCallConversionsRequest
                             */
                            UploadCallConversionsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadCallConversionsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadCallConversionsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.conversions) {
                                    if (!Array.isArray(d.conversions))
                                        throw TypeError(".google.ads.googleads.v11.services.UploadCallConversionsRequest.conversions: array expected");
                                    m.conversions = [];
                                    for (var i = 0; i < d.conversions.length; ++i) {
                                        if (typeof d.conversions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.UploadCallConversionsRequest.conversions: object expected");
                                        m.conversions[i] = $root.google.ads.googleads.v11.services.CallConversion.fromObject(d.conversions[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadCallConversionsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadCallConversionsRequest} m UploadCallConversionsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadCallConversionsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.conversions = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.conversions && m.conversions.length) {
                                    d.conversions = [];
                                    for (var j = 0; j < m.conversions.length; ++j) {
                                        d.conversions[j] = $root.google.ads.googleads.v11.services.CallConversion.toObject(m.conversions[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadCallConversionsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadCallConversionsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadCallConversionsRequest;
                        })();
    
                        services.UploadCallConversionsResponse = (function() {
    
                            /**
                             * Properties of an UploadCallConversionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadCallConversionsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] UploadCallConversionsResponse partial_failure_error
                             * @property {Array.<google.ads.googleads.v11.services.ICallConversionResult>|null} [results] UploadCallConversionsResponse results
                             */
    
                            /**
                             * Constructs a new UploadCallConversionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadCallConversionsResponse.
                             * @implements IUploadCallConversionsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsResponse=} [p] Properties to set
                             */
                            function UploadCallConversionsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadCallConversionsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @instance
                             */
                            UploadCallConversionsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * UploadCallConversionsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.ICallConversionResult>} results
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @instance
                             */
                            UploadCallConversionsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new UploadCallConversionsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsResponse} UploadCallConversionsResponse instance
                             */
                            UploadCallConversionsResponse.create = function create(properties) {
                                return new UploadCallConversionsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified UploadCallConversionsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.UploadCallConversionsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsResponse} m UploadCallConversionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadCallConversionsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.CallConversionResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadCallConversionsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadCallConversionsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadCallConversionsResponse} message UploadCallConversionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadCallConversionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadCallConversionsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsResponse} UploadCallConversionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadCallConversionsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadCallConversionsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.CallConversionResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadCallConversionsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsResponse} UploadCallConversionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadCallConversionsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadCallConversionsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadCallConversionsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CallConversionResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadCallConversionsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadCallConversionsResponse} UploadCallConversionsResponse
                             */
                            UploadCallConversionsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadCallConversionsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadCallConversionsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UploadCallConversionsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.UploadCallConversionsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.UploadCallConversionsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.CallConversionResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadCallConversionsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadCallConversionsResponse} m UploadCallConversionsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadCallConversionsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.CallConversionResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadCallConversionsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadCallConversionsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadCallConversionsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadCallConversionsResponse;
                        })();
    
                        services.ClickConversion = (function() {
    
                            /**
                             * Properties of a ClickConversion.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IClickConversion
                             * @property {string|null} [gclid] ClickConversion gclid
                             * @property {string|null} [gbraid] ClickConversion gbraid
                             * @property {string|null} [wbraid] ClickConversion wbraid
                             * @property {string|null} [conversion_action] ClickConversion conversion_action
                             * @property {string|null} [conversion_date_time] ClickConversion conversion_date_time
                             * @property {number|null} [conversion_value] ClickConversion conversion_value
                             * @property {string|null} [currency_code] ClickConversion currency_code
                             * @property {string|null} [order_id] ClickConversion order_id
                             * @property {google.ads.googleads.v11.services.IExternalAttributionData|null} [external_attribution_data] ClickConversion external_attribution_data
                             * @property {Array.<google.ads.googleads.v11.services.ICustomVariable>|null} [custom_variables] ClickConversion custom_variables
                             * @property {google.ads.googleads.v11.services.ICartData|null} [cart_data] ClickConversion cart_data
                             * @property {Array.<google.ads.googleads.v11.common.IUserIdentifier>|null} [user_identifiers] ClickConversion user_identifiers
                             * @property {google.ads.googleads.v11.enums.ConversionEnvironmentEnum.ConversionEnvironment|null} [conversion_environment] ClickConversion conversion_environment
                             */
    
                            /**
                             * Constructs a new ClickConversion.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ClickConversion.
                             * @implements IClickConversion
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IClickConversion=} [p] Properties to set
                             */
                            function ClickConversion(p) {
                                this.custom_variables = [];
                                this.user_identifiers = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ClickConversion gclid.
                             * @member {string|null|undefined} gclid
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.gclid = null;
    
                            /**
                             * ClickConversion gbraid.
                             * @member {string} gbraid
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.gbraid = "";
    
                            /**
                             * ClickConversion wbraid.
                             * @member {string} wbraid
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.wbraid = "";
    
                            /**
                             * ClickConversion conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.conversion_action = null;
    
                            /**
                             * ClickConversion conversion_date_time.
                             * @member {string|null|undefined} conversion_date_time
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.conversion_date_time = null;
    
                            /**
                             * ClickConversion conversion_value.
                             * @member {number|null|undefined} conversion_value
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.conversion_value = null;
    
                            /**
                             * ClickConversion currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.currency_code = null;
    
                            /**
                             * ClickConversion order_id.
                             * @member {string|null|undefined} order_id
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.order_id = null;
    
                            /**
                             * ClickConversion external_attribution_data.
                             * @member {google.ads.googleads.v11.services.IExternalAttributionData|null|undefined} external_attribution_data
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.external_attribution_data = null;
    
                            /**
                             * ClickConversion custom_variables.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomVariable>} custom_variables
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.custom_variables = $util.emptyArray;
    
                            /**
                             * ClickConversion cart_data.
                             * @member {google.ads.googleads.v11.services.ICartData|null|undefined} cart_data
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.cart_data = null;
    
                            /**
                             * ClickConversion user_identifiers.
                             * @member {Array.<google.ads.googleads.v11.common.IUserIdentifier>} user_identifiers
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.user_identifiers = $util.emptyArray;
    
                            /**
                             * ClickConversion conversion_environment.
                             * @member {google.ads.googleads.v11.enums.ConversionEnvironmentEnum.ConversionEnvironment} conversion_environment
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            ClickConversion.prototype.conversion_environment = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ClickConversion _gclid.
                             * @member {"gclid"|undefined} _gclid
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            Object.defineProperty(ClickConversion.prototype, "_gclid", {
                                get: $util.oneOfGetter($oneOfFields = ["gclid"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickConversion _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            Object.defineProperty(ClickConversion.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickConversion _conversion_date_time.
                             * @member {"conversion_date_time"|undefined} _conversion_date_time
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            Object.defineProperty(ClickConversion.prototype, "_conversion_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickConversion _conversion_value.
                             * @member {"conversion_value"|undefined} _conversion_value
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            Object.defineProperty(ClickConversion.prototype, "_conversion_value", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickConversion _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            Object.defineProperty(ClickConversion.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickConversion _order_id.
                             * @member {"order_id"|undefined} _order_id
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             */
                            Object.defineProperty(ClickConversion.prototype, "_order_id", {
                                get: $util.oneOfGetter($oneOfFields = ["order_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ClickConversion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.IClickConversion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ClickConversion} ClickConversion instance
                             */
                            ClickConversion.create = function create(properties) {
                                return new ClickConversion(properties);
                            };
    
                            /**
                             * Encodes the specified ClickConversion message. Does not implicitly {@link google.ads.googleads.v11.services.ClickConversion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.IClickConversion} m ClickConversion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickConversion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.external_attribution_data != null && Object.hasOwnProperty.call(m, "external_attribution_data"))
                                    $root.google.ads.googleads.v11.services.ExternalAttributionData.encode(m.external_attribution_data, w.uint32(58).fork()).ldelim();
                                if (m.gclid != null && Object.hasOwnProperty.call(m, "gclid"))
                                    w.uint32(74).string(m.gclid);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(82).string(m.conversion_action);
                                if (m.conversion_date_time != null && Object.hasOwnProperty.call(m, "conversion_date_time"))
                                    w.uint32(90).string(m.conversion_date_time);
                                if (m.conversion_value != null && Object.hasOwnProperty.call(m, "conversion_value"))
                                    w.uint32(97).double(m.conversion_value);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(106).string(m.currency_code);
                                if (m.order_id != null && Object.hasOwnProperty.call(m, "order_id"))
                                    w.uint32(114).string(m.order_id);
                                if (m.custom_variables != null && m.custom_variables.length) {
                                    for (var i = 0; i < m.custom_variables.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomVariable.encode(m.custom_variables[i], w.uint32(122).fork()).ldelim();
                                }
                                if (m.cart_data != null && Object.hasOwnProperty.call(m, "cart_data"))
                                    $root.google.ads.googleads.v11.services.CartData.encode(m.cart_data, w.uint32(130).fork()).ldelim();
                                if (m.user_identifiers != null && m.user_identifiers.length) {
                                    for (var i = 0; i < m.user_identifiers.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserIdentifier.encode(m.user_identifiers[i], w.uint32(138).fork()).ldelim();
                                }
                                if (m.gbraid != null && Object.hasOwnProperty.call(m, "gbraid"))
                                    w.uint32(146).string(m.gbraid);
                                if (m.wbraid != null && Object.hasOwnProperty.call(m, "wbraid"))
                                    w.uint32(154).string(m.wbraid);
                                if (m.conversion_environment != null && Object.hasOwnProperty.call(m, "conversion_environment"))
                                    w.uint32(160).int32(m.conversion_environment);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ClickConversion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ClickConversion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.IClickConversion} message ClickConversion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickConversion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClickConversion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ClickConversion} ClickConversion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickConversion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ClickConversion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 9:
                                        m.gclid = r.string();
                                        break;
                                    case 18:
                                        m.gbraid = r.string();
                                        break;
                                    case 19:
                                        m.wbraid = r.string();
                                        break;
                                    case 10:
                                        m.conversion_action = r.string();
                                        break;
                                    case 11:
                                        m.conversion_date_time = r.string();
                                        break;
                                    case 12:
                                        m.conversion_value = r.double();
                                        break;
                                    case 13:
                                        m.currency_code = r.string();
                                        break;
                                    case 14:
                                        m.order_id = r.string();
                                        break;
                                    case 7:
                                        m.external_attribution_data = $root.google.ads.googleads.v11.services.ExternalAttributionData.decode(r, r.uint32());
                                        break;
                                    case 15:
                                        if (!(m.custom_variables && m.custom_variables.length))
                                            m.custom_variables = [];
                                        m.custom_variables.push($root.google.ads.googleads.v11.services.CustomVariable.decode(r, r.uint32()));
                                        break;
                                    case 16:
                                        m.cart_data = $root.google.ads.googleads.v11.services.CartData.decode(r, r.uint32());
                                        break;
                                    case 17:
                                        if (!(m.user_identifiers && m.user_identifiers.length))
                                            m.user_identifiers = [];
                                        m.user_identifiers.push($root.google.ads.googleads.v11.common.UserIdentifier.decode(r, r.uint32()));
                                        break;
                                    case 20:
                                        m.conversion_environment = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ClickConversion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ClickConversion} ClickConversion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickConversion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClickConversion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClickConversion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    p._gclid = 1;
                                    if (!$util.isString(m.gclid))
                                        return "gclid: string expected";
                                }
                                if (m.gbraid != null && m.hasOwnProperty("gbraid")) {
                                    if (!$util.isString(m.gbraid))
                                        return "gbraid: string expected";
                                }
                                if (m.wbraid != null && m.hasOwnProperty("wbraid")) {
                                    if (!$util.isString(m.wbraid))
                                        return "wbraid: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    p._conversion_date_time = 1;
                                    if (!$util.isString(m.conversion_date_time))
                                        return "conversion_date_time: string expected";
                                }
                                if (m.conversion_value != null && m.hasOwnProperty("conversion_value")) {
                                    p._conversion_value = 1;
                                    if (typeof m.conversion_value !== "number")
                                        return "conversion_value: number expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    p._order_id = 1;
                                    if (!$util.isString(m.order_id))
                                        return "order_id: string expected";
                                }
                                if (m.external_attribution_data != null && m.hasOwnProperty("external_attribution_data")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ExternalAttributionData.verify(m.external_attribution_data);
                                        if (e)
                                            return "external_attribution_data." + e;
                                    }
                                }
                                if (m.custom_variables != null && m.hasOwnProperty("custom_variables")) {
                                    if (!Array.isArray(m.custom_variables))
                                        return "custom_variables: array expected";
                                    for (var i = 0; i < m.custom_variables.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomVariable.verify(m.custom_variables[i]);
                                            if (e)
                                                return "custom_variables." + e;
                                        }
                                    }
                                }
                                if (m.cart_data != null && m.hasOwnProperty("cart_data")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CartData.verify(m.cart_data);
                                        if (e)
                                            return "cart_data." + e;
                                    }
                                }
                                if (m.user_identifiers != null && m.hasOwnProperty("user_identifiers")) {
                                    if (!Array.isArray(m.user_identifiers))
                                        return "user_identifiers: array expected";
                                    for (var i = 0; i < m.user_identifiers.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserIdentifier.verify(m.user_identifiers[i]);
                                            if (e)
                                                return "user_identifiers." + e;
                                        }
                                    }
                                }
                                if (m.conversion_environment != null && m.hasOwnProperty("conversion_environment")) {
                                    switch (m.conversion_environment) {
                                    default:
                                        return "conversion_environment: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ClickConversion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ClickConversion} ClickConversion
                             */
                            ClickConversion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ClickConversion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ClickConversion();
                                if (d.gclid != null) {
                                    m.gclid = String(d.gclid);
                                }
                                if (d.gbraid != null) {
                                    m.gbraid = String(d.gbraid);
                                }
                                if (d.wbraid != null) {
                                    m.wbraid = String(d.wbraid);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.conversion_date_time != null) {
                                    m.conversion_date_time = String(d.conversion_date_time);
                                }
                                if (d.conversion_value != null) {
                                    m.conversion_value = Number(d.conversion_value);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.order_id != null) {
                                    m.order_id = String(d.order_id);
                                }
                                if (d.external_attribution_data != null) {
                                    if (typeof d.external_attribution_data !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ClickConversion.external_attribution_data: object expected");
                                    m.external_attribution_data = $root.google.ads.googleads.v11.services.ExternalAttributionData.fromObject(d.external_attribution_data);
                                }
                                if (d.custom_variables) {
                                    if (!Array.isArray(d.custom_variables))
                                        throw TypeError(".google.ads.googleads.v11.services.ClickConversion.custom_variables: array expected");
                                    m.custom_variables = [];
                                    for (var i = 0; i < d.custom_variables.length; ++i) {
                                        if (typeof d.custom_variables[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ClickConversion.custom_variables: object expected");
                                        m.custom_variables[i] = $root.google.ads.googleads.v11.services.CustomVariable.fromObject(d.custom_variables[i]);
                                    }
                                }
                                if (d.cart_data != null) {
                                    if (typeof d.cart_data !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ClickConversion.cart_data: object expected");
                                    m.cart_data = $root.google.ads.googleads.v11.services.CartData.fromObject(d.cart_data);
                                }
                                if (d.user_identifiers) {
                                    if (!Array.isArray(d.user_identifiers))
                                        throw TypeError(".google.ads.googleads.v11.services.ClickConversion.user_identifiers: array expected");
                                    m.user_identifiers = [];
                                    for (var i = 0; i < d.user_identifiers.length; ++i) {
                                        if (typeof d.user_identifiers[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ClickConversion.user_identifiers: object expected");
                                        m.user_identifiers[i] = $root.google.ads.googleads.v11.common.UserIdentifier.fromObject(d.user_identifiers[i]);
                                    }
                                }
                                switch (d.conversion_environment) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.conversion_environment = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.conversion_environment = 1;
                                    break;
                                case "APP":
                                case 2:
                                    m.conversion_environment = 2;
                                    break;
                                case "WEB":
                                case 3:
                                    m.conversion_environment = 3;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ClickConversion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.ClickConversion} m ClickConversion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClickConversion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.custom_variables = [];
                                    d.user_identifiers = [];
                                }
                                if (o.defaults) {
                                    d.external_attribution_data = null;
                                    d.cart_data = null;
                                    d.gbraid = "";
                                    d.wbraid = "";
                                    d.conversion_environment = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.external_attribution_data != null && m.hasOwnProperty("external_attribution_data")) {
                                    d.external_attribution_data = $root.google.ads.googleads.v11.services.ExternalAttributionData.toObject(m.external_attribution_data, o);
                                }
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    d.gclid = m.gclid;
                                    if (o.oneofs)
                                        d._gclid = "gclid";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    d.conversion_date_time = m.conversion_date_time;
                                    if (o.oneofs)
                                        d._conversion_date_time = "conversion_date_time";
                                }
                                if (m.conversion_value != null && m.hasOwnProperty("conversion_value")) {
                                    d.conversion_value = o.json && !isFinite(m.conversion_value) ? String(m.conversion_value) : m.conversion_value;
                                    if (o.oneofs)
                                        d._conversion_value = "conversion_value";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.order_id != null && m.hasOwnProperty("order_id")) {
                                    d.order_id = m.order_id;
                                    if (o.oneofs)
                                        d._order_id = "order_id";
                                }
                                if (m.custom_variables && m.custom_variables.length) {
                                    d.custom_variables = [];
                                    for (var j = 0; j < m.custom_variables.length; ++j) {
                                        d.custom_variables[j] = $root.google.ads.googleads.v11.services.CustomVariable.toObject(m.custom_variables[j], o);
                                    }
                                }
                                if (m.cart_data != null && m.hasOwnProperty("cart_data")) {
                                    d.cart_data = $root.google.ads.googleads.v11.services.CartData.toObject(m.cart_data, o);
                                }
                                if (m.user_identifiers && m.user_identifiers.length) {
                                    d.user_identifiers = [];
                                    for (var j = 0; j < m.user_identifiers.length; ++j) {
                                        d.user_identifiers[j] = $root.google.ads.googleads.v11.common.UserIdentifier.toObject(m.user_identifiers[j], o);
                                    }
                                }
                                if (m.gbraid != null && m.hasOwnProperty("gbraid")) {
                                    d.gbraid = m.gbraid;
                                }
                                if (m.wbraid != null && m.hasOwnProperty("wbraid")) {
                                    d.wbraid = m.wbraid;
                                }
                                if (m.conversion_environment != null && m.hasOwnProperty("conversion_environment")) {
                                    d.conversion_environment = o.enums === String ? $root.google.ads.googleads.v11.enums.ConversionEnvironmentEnum.ConversionEnvironment[m.conversion_environment] : m.conversion_environment;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ClickConversion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ClickConversion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClickConversion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ClickConversion;
                        })();
    
                        services.CallConversion = (function() {
    
                            /**
                             * Properties of a CallConversion.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICallConversion
                             * @property {string|null} [caller_id] CallConversion caller_id
                             * @property {string|null} [call_start_date_time] CallConversion call_start_date_time
                             * @property {string|null} [conversion_action] CallConversion conversion_action
                             * @property {string|null} [conversion_date_time] CallConversion conversion_date_time
                             * @property {number|null} [conversion_value] CallConversion conversion_value
                             * @property {string|null} [currency_code] CallConversion currency_code
                             * @property {Array.<google.ads.googleads.v11.services.ICustomVariable>|null} [custom_variables] CallConversion custom_variables
                             */
    
                            /**
                             * Constructs a new CallConversion.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CallConversion.
                             * @implements ICallConversion
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICallConversion=} [p] Properties to set
                             */
                            function CallConversion(p) {
                                this.custom_variables = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallConversion caller_id.
                             * @member {string|null|undefined} caller_id
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            CallConversion.prototype.caller_id = null;
    
                            /**
                             * CallConversion call_start_date_time.
                             * @member {string|null|undefined} call_start_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            CallConversion.prototype.call_start_date_time = null;
    
                            /**
                             * CallConversion conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            CallConversion.prototype.conversion_action = null;
    
                            /**
                             * CallConversion conversion_date_time.
                             * @member {string|null|undefined} conversion_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            CallConversion.prototype.conversion_date_time = null;
    
                            /**
                             * CallConversion conversion_value.
                             * @member {number|null|undefined} conversion_value
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            CallConversion.prototype.conversion_value = null;
    
                            /**
                             * CallConversion currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            CallConversion.prototype.currency_code = null;
    
                            /**
                             * CallConversion custom_variables.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomVariable>} custom_variables
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            CallConversion.prototype.custom_variables = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CallConversion _caller_id.
                             * @member {"caller_id"|undefined} _caller_id
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            Object.defineProperty(CallConversion.prototype, "_caller_id", {
                                get: $util.oneOfGetter($oneOfFields = ["caller_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversion _call_start_date_time.
                             * @member {"call_start_date_time"|undefined} _call_start_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            Object.defineProperty(CallConversion.prototype, "_call_start_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["call_start_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversion _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            Object.defineProperty(CallConversion.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversion _conversion_date_time.
                             * @member {"conversion_date_time"|undefined} _conversion_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            Object.defineProperty(CallConversion.prototype, "_conversion_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversion _conversion_value.
                             * @member {"conversion_value"|undefined} _conversion_value
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            Object.defineProperty(CallConversion.prototype, "_conversion_value", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_value"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversion _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             */
                            Object.defineProperty(CallConversion.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CallConversion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.ICallConversion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CallConversion} CallConversion instance
                             */
                            CallConversion.create = function create(properties) {
                                return new CallConversion(properties);
                            };
    
                            /**
                             * Encodes the specified CallConversion message. Does not implicitly {@link google.ads.googleads.v11.services.CallConversion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.ICallConversion} m CallConversion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallConversion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.caller_id != null && Object.hasOwnProperty.call(m, "caller_id"))
                                    w.uint32(58).string(m.caller_id);
                                if (m.call_start_date_time != null && Object.hasOwnProperty.call(m, "call_start_date_time"))
                                    w.uint32(66).string(m.call_start_date_time);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(74).string(m.conversion_action);
                                if (m.conversion_date_time != null && Object.hasOwnProperty.call(m, "conversion_date_time"))
                                    w.uint32(82).string(m.conversion_date_time);
                                if (m.conversion_value != null && Object.hasOwnProperty.call(m, "conversion_value"))
                                    w.uint32(89).double(m.conversion_value);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(98).string(m.currency_code);
                                if (m.custom_variables != null && m.custom_variables.length) {
                                    for (var i = 0; i < m.custom_variables.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomVariable.encode(m.custom_variables[i], w.uint32(106).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallConversion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CallConversion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.ICallConversion} message CallConversion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallConversion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallConversion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CallConversion} CallConversion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallConversion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CallConversion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 7:
                                        m.caller_id = r.string();
                                        break;
                                    case 8:
                                        m.call_start_date_time = r.string();
                                        break;
                                    case 9:
                                        m.conversion_action = r.string();
                                        break;
                                    case 10:
                                        m.conversion_date_time = r.string();
                                        break;
                                    case 11:
                                        m.conversion_value = r.double();
                                        break;
                                    case 12:
                                        m.currency_code = r.string();
                                        break;
                                    case 13:
                                        if (!(m.custom_variables && m.custom_variables.length))
                                            m.custom_variables = [];
                                        m.custom_variables.push($root.google.ads.googleads.v11.services.CustomVariable.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallConversion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CallConversion} CallConversion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallConversion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallConversion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallConversion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.caller_id != null && m.hasOwnProperty("caller_id")) {
                                    p._caller_id = 1;
                                    if (!$util.isString(m.caller_id))
                                        return "caller_id: string expected";
                                }
                                if (m.call_start_date_time != null && m.hasOwnProperty("call_start_date_time")) {
                                    p._call_start_date_time = 1;
                                    if (!$util.isString(m.call_start_date_time))
                                        return "call_start_date_time: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    p._conversion_date_time = 1;
                                    if (!$util.isString(m.conversion_date_time))
                                        return "conversion_date_time: string expected";
                                }
                                if (m.conversion_value != null && m.hasOwnProperty("conversion_value")) {
                                    p._conversion_value = 1;
                                    if (typeof m.conversion_value !== "number")
                                        return "conversion_value: number expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.custom_variables != null && m.hasOwnProperty("custom_variables")) {
                                    if (!Array.isArray(m.custom_variables))
                                        return "custom_variables: array expected";
                                    for (var i = 0; i < m.custom_variables.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomVariable.verify(m.custom_variables[i]);
                                            if (e)
                                                return "custom_variables." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallConversion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CallConversion} CallConversion
                             */
                            CallConversion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CallConversion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CallConversion();
                                if (d.caller_id != null) {
                                    m.caller_id = String(d.caller_id);
                                }
                                if (d.call_start_date_time != null) {
                                    m.call_start_date_time = String(d.call_start_date_time);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.conversion_date_time != null) {
                                    m.conversion_date_time = String(d.conversion_date_time);
                                }
                                if (d.conversion_value != null) {
                                    m.conversion_value = Number(d.conversion_value);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.custom_variables) {
                                    if (!Array.isArray(d.custom_variables))
                                        throw TypeError(".google.ads.googleads.v11.services.CallConversion.custom_variables: array expected");
                                    m.custom_variables = [];
                                    for (var i = 0; i < d.custom_variables.length; ++i) {
                                        if (typeof d.custom_variables[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.CallConversion.custom_variables: object expected");
                                        m.custom_variables[i] = $root.google.ads.googleads.v11.services.CustomVariable.fromObject(d.custom_variables[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallConversion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @static
                             * @param {google.ads.googleads.v11.services.CallConversion} m CallConversion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallConversion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.custom_variables = [];
                                }
                                if (m.caller_id != null && m.hasOwnProperty("caller_id")) {
                                    d.caller_id = m.caller_id;
                                    if (o.oneofs)
                                        d._caller_id = "caller_id";
                                }
                                if (m.call_start_date_time != null && m.hasOwnProperty("call_start_date_time")) {
                                    d.call_start_date_time = m.call_start_date_time;
                                    if (o.oneofs)
                                        d._call_start_date_time = "call_start_date_time";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    d.conversion_date_time = m.conversion_date_time;
                                    if (o.oneofs)
                                        d._conversion_date_time = "conversion_date_time";
                                }
                                if (m.conversion_value != null && m.hasOwnProperty("conversion_value")) {
                                    d.conversion_value = o.json && !isFinite(m.conversion_value) ? String(m.conversion_value) : m.conversion_value;
                                    if (o.oneofs)
                                        d._conversion_value = "conversion_value";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.custom_variables && m.custom_variables.length) {
                                    d.custom_variables = [];
                                    for (var j = 0; j < m.custom_variables.length; ++j) {
                                        d.custom_variables[j] = $root.google.ads.googleads.v11.services.CustomVariable.toObject(m.custom_variables[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallConversion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CallConversion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallConversion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallConversion;
                        })();
    
                        services.ExternalAttributionData = (function() {
    
                            /**
                             * Properties of an ExternalAttributionData.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IExternalAttributionData
                             * @property {number|null} [external_attribution_credit] ExternalAttributionData external_attribution_credit
                             * @property {string|null} [external_attribution_model] ExternalAttributionData external_attribution_model
                             */
    
                            /**
                             * Constructs a new ExternalAttributionData.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ExternalAttributionData.
                             * @implements IExternalAttributionData
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IExternalAttributionData=} [p] Properties to set
                             */
                            function ExternalAttributionData(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ExternalAttributionData external_attribution_credit.
                             * @member {number|null|undefined} external_attribution_credit
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @instance
                             */
                            ExternalAttributionData.prototype.external_attribution_credit = null;
    
                            /**
                             * ExternalAttributionData external_attribution_model.
                             * @member {string|null|undefined} external_attribution_model
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @instance
                             */
                            ExternalAttributionData.prototype.external_attribution_model = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ExternalAttributionData _external_attribution_credit.
                             * @member {"external_attribution_credit"|undefined} _external_attribution_credit
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @instance
                             */
                            Object.defineProperty(ExternalAttributionData.prototype, "_external_attribution_credit", {
                                get: $util.oneOfGetter($oneOfFields = ["external_attribution_credit"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ExternalAttributionData _external_attribution_model.
                             * @member {"external_attribution_model"|undefined} _external_attribution_model
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @instance
                             */
                            Object.defineProperty(ExternalAttributionData.prototype, "_external_attribution_model", {
                                get: $util.oneOfGetter($oneOfFields = ["external_attribution_model"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ExternalAttributionData instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {google.ads.googleads.v11.services.IExternalAttributionData=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ExternalAttributionData} ExternalAttributionData instance
                             */
                            ExternalAttributionData.create = function create(properties) {
                                return new ExternalAttributionData(properties);
                            };
    
                            /**
                             * Encodes the specified ExternalAttributionData message. Does not implicitly {@link google.ads.googleads.v11.services.ExternalAttributionData.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {google.ads.googleads.v11.services.IExternalAttributionData} m ExternalAttributionData message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExternalAttributionData.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.external_attribution_credit != null && Object.hasOwnProperty.call(m, "external_attribution_credit"))
                                    w.uint32(25).double(m.external_attribution_credit);
                                if (m.external_attribution_model != null && Object.hasOwnProperty.call(m, "external_attribution_model"))
                                    w.uint32(34).string(m.external_attribution_model);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ExternalAttributionData message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ExternalAttributionData.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {google.ads.googleads.v11.services.IExternalAttributionData} message ExternalAttributionData message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ExternalAttributionData.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ExternalAttributionData message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ExternalAttributionData} ExternalAttributionData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExternalAttributionData.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ExternalAttributionData();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.external_attribution_credit = r.double();
                                        break;
                                    case 4:
                                        m.external_attribution_model = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ExternalAttributionData message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ExternalAttributionData} ExternalAttributionData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ExternalAttributionData.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ExternalAttributionData message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ExternalAttributionData.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.external_attribution_credit != null && m.hasOwnProperty("external_attribution_credit")) {
                                    p._external_attribution_credit = 1;
                                    if (typeof m.external_attribution_credit !== "number")
                                        return "external_attribution_credit: number expected";
                                }
                                if (m.external_attribution_model != null && m.hasOwnProperty("external_attribution_model")) {
                                    p._external_attribution_model = 1;
                                    if (!$util.isString(m.external_attribution_model))
                                        return "external_attribution_model: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ExternalAttributionData message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ExternalAttributionData} ExternalAttributionData
                             */
                            ExternalAttributionData.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ExternalAttributionData)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ExternalAttributionData();
                                if (d.external_attribution_credit != null) {
                                    m.external_attribution_credit = Number(d.external_attribution_credit);
                                }
                                if (d.external_attribution_model != null) {
                                    m.external_attribution_model = String(d.external_attribution_model);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ExternalAttributionData message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @static
                             * @param {google.ads.googleads.v11.services.ExternalAttributionData} m ExternalAttributionData
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ExternalAttributionData.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.external_attribution_credit != null && m.hasOwnProperty("external_attribution_credit")) {
                                    d.external_attribution_credit = o.json && !isFinite(m.external_attribution_credit) ? String(m.external_attribution_credit) : m.external_attribution_credit;
                                    if (o.oneofs)
                                        d._external_attribution_credit = "external_attribution_credit";
                                }
                                if (m.external_attribution_model != null && m.hasOwnProperty("external_attribution_model")) {
                                    d.external_attribution_model = m.external_attribution_model;
                                    if (o.oneofs)
                                        d._external_attribution_model = "external_attribution_model";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ExternalAttributionData to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ExternalAttributionData
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ExternalAttributionData.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ExternalAttributionData;
                        })();
    
                        services.ClickConversionResult = (function() {
    
                            /**
                             * Properties of a ClickConversionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IClickConversionResult
                             * @property {string|null} [gclid] ClickConversionResult gclid
                             * @property {string|null} [gbraid] ClickConversionResult gbraid
                             * @property {string|null} [wbraid] ClickConversionResult wbraid
                             * @property {string|null} [conversion_action] ClickConversionResult conversion_action
                             * @property {string|null} [conversion_date_time] ClickConversionResult conversion_date_time
                             * @property {Array.<google.ads.googleads.v11.common.IUserIdentifier>|null} [user_identifiers] ClickConversionResult user_identifiers
                             */
    
                            /**
                             * Constructs a new ClickConversionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ClickConversionResult.
                             * @implements IClickConversionResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IClickConversionResult=} [p] Properties to set
                             */
                            function ClickConversionResult(p) {
                                this.user_identifiers = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ClickConversionResult gclid.
                             * @member {string|null|undefined} gclid
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            ClickConversionResult.prototype.gclid = null;
    
                            /**
                             * ClickConversionResult gbraid.
                             * @member {string} gbraid
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            ClickConversionResult.prototype.gbraid = "";
    
                            /**
                             * ClickConversionResult wbraid.
                             * @member {string} wbraid
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            ClickConversionResult.prototype.wbraid = "";
    
                            /**
                             * ClickConversionResult conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            ClickConversionResult.prototype.conversion_action = null;
    
                            /**
                             * ClickConversionResult conversion_date_time.
                             * @member {string|null|undefined} conversion_date_time
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            ClickConversionResult.prototype.conversion_date_time = null;
    
                            /**
                             * ClickConversionResult user_identifiers.
                             * @member {Array.<google.ads.googleads.v11.common.IUserIdentifier>} user_identifiers
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            ClickConversionResult.prototype.user_identifiers = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ClickConversionResult _gclid.
                             * @member {"gclid"|undefined} _gclid
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            Object.defineProperty(ClickConversionResult.prototype, "_gclid", {
                                get: $util.oneOfGetter($oneOfFields = ["gclid"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickConversionResult _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            Object.defineProperty(ClickConversionResult.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ClickConversionResult _conversion_date_time.
                             * @member {"conversion_date_time"|undefined} _conversion_date_time
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             */
                            Object.defineProperty(ClickConversionResult.prototype, "_conversion_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ClickConversionResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IClickConversionResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ClickConversionResult} ClickConversionResult instance
                             */
                            ClickConversionResult.create = function create(properties) {
                                return new ClickConversionResult(properties);
                            };
    
                            /**
                             * Encodes the specified ClickConversionResult message. Does not implicitly {@link google.ads.googleads.v11.services.ClickConversionResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IClickConversionResult} m ClickConversionResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickConversionResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.gclid != null && Object.hasOwnProperty.call(m, "gclid"))
                                    w.uint32(34).string(m.gclid);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(42).string(m.conversion_action);
                                if (m.conversion_date_time != null && Object.hasOwnProperty.call(m, "conversion_date_time"))
                                    w.uint32(50).string(m.conversion_date_time);
                                if (m.user_identifiers != null && m.user_identifiers.length) {
                                    for (var i = 0; i < m.user_identifiers.length; ++i)
                                        $root.google.ads.googleads.v11.common.UserIdentifier.encode(m.user_identifiers[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.gbraid != null && Object.hasOwnProperty.call(m, "gbraid"))
                                    w.uint32(66).string(m.gbraid);
                                if (m.wbraid != null && Object.hasOwnProperty.call(m, "wbraid"))
                                    w.uint32(74).string(m.wbraid);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ClickConversionResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ClickConversionResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IClickConversionResult} message ClickConversionResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClickConversionResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClickConversionResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ClickConversionResult} ClickConversionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickConversionResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ClickConversionResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.gclid = r.string();
                                        break;
                                    case 8:
                                        m.gbraid = r.string();
                                        break;
                                    case 9:
                                        m.wbraid = r.string();
                                        break;
                                    case 5:
                                        m.conversion_action = r.string();
                                        break;
                                    case 6:
                                        m.conversion_date_time = r.string();
                                        break;
                                    case 7:
                                        if (!(m.user_identifiers && m.user_identifiers.length))
                                            m.user_identifiers = [];
                                        m.user_identifiers.push($root.google.ads.googleads.v11.common.UserIdentifier.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ClickConversionResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ClickConversionResult} ClickConversionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClickConversionResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClickConversionResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClickConversionResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    p._gclid = 1;
                                    if (!$util.isString(m.gclid))
                                        return "gclid: string expected";
                                }
                                if (m.gbraid != null && m.hasOwnProperty("gbraid")) {
                                    if (!$util.isString(m.gbraid))
                                        return "gbraid: string expected";
                                }
                                if (m.wbraid != null && m.hasOwnProperty("wbraid")) {
                                    if (!$util.isString(m.wbraid))
                                        return "wbraid: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    p._conversion_date_time = 1;
                                    if (!$util.isString(m.conversion_date_time))
                                        return "conversion_date_time: string expected";
                                }
                                if (m.user_identifiers != null && m.hasOwnProperty("user_identifiers")) {
                                    if (!Array.isArray(m.user_identifiers))
                                        return "user_identifiers: array expected";
                                    for (var i = 0; i < m.user_identifiers.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.UserIdentifier.verify(m.user_identifiers[i]);
                                            if (e)
                                                return "user_identifiers." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ClickConversionResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ClickConversionResult} ClickConversionResult
                             */
                            ClickConversionResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ClickConversionResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ClickConversionResult();
                                if (d.gclid != null) {
                                    m.gclid = String(d.gclid);
                                }
                                if (d.gbraid != null) {
                                    m.gbraid = String(d.gbraid);
                                }
                                if (d.wbraid != null) {
                                    m.wbraid = String(d.wbraid);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.conversion_date_time != null) {
                                    m.conversion_date_time = String(d.conversion_date_time);
                                }
                                if (d.user_identifiers) {
                                    if (!Array.isArray(d.user_identifiers))
                                        throw TypeError(".google.ads.googleads.v11.services.ClickConversionResult.user_identifiers: array expected");
                                    m.user_identifiers = [];
                                    for (var i = 0; i < d.user_identifiers.length; ++i) {
                                        if (typeof d.user_identifiers[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ClickConversionResult.user_identifiers: object expected");
                                        m.user_identifiers[i] = $root.google.ads.googleads.v11.common.UserIdentifier.fromObject(d.user_identifiers[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ClickConversionResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.ClickConversionResult} m ClickConversionResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClickConversionResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.user_identifiers = [];
                                }
                                if (o.defaults) {
                                    d.gbraid = "";
                                    d.wbraid = "";
                                }
                                if (m.gclid != null && m.hasOwnProperty("gclid")) {
                                    d.gclid = m.gclid;
                                    if (o.oneofs)
                                        d._gclid = "gclid";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    d.conversion_date_time = m.conversion_date_time;
                                    if (o.oneofs)
                                        d._conversion_date_time = "conversion_date_time";
                                }
                                if (m.user_identifiers && m.user_identifiers.length) {
                                    d.user_identifiers = [];
                                    for (var j = 0; j < m.user_identifiers.length; ++j) {
                                        d.user_identifiers[j] = $root.google.ads.googleads.v11.common.UserIdentifier.toObject(m.user_identifiers[j], o);
                                    }
                                }
                                if (m.gbraid != null && m.hasOwnProperty("gbraid")) {
                                    d.gbraid = m.gbraid;
                                }
                                if (m.wbraid != null && m.hasOwnProperty("wbraid")) {
                                    d.wbraid = m.wbraid;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ClickConversionResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ClickConversionResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClickConversionResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ClickConversionResult;
                        })();
    
                        services.CallConversionResult = (function() {
    
                            /**
                             * Properties of a CallConversionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICallConversionResult
                             * @property {string|null} [caller_id] CallConversionResult caller_id
                             * @property {string|null} [call_start_date_time] CallConversionResult call_start_date_time
                             * @property {string|null} [conversion_action] CallConversionResult conversion_action
                             * @property {string|null} [conversion_date_time] CallConversionResult conversion_date_time
                             */
    
                            /**
                             * Constructs a new CallConversionResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CallConversionResult.
                             * @implements ICallConversionResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICallConversionResult=} [p] Properties to set
                             */
                            function CallConversionResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CallConversionResult caller_id.
                             * @member {string|null|undefined} caller_id
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            CallConversionResult.prototype.caller_id = null;
    
                            /**
                             * CallConversionResult call_start_date_time.
                             * @member {string|null|undefined} call_start_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            CallConversionResult.prototype.call_start_date_time = null;
    
                            /**
                             * CallConversionResult conversion_action.
                             * @member {string|null|undefined} conversion_action
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            CallConversionResult.prototype.conversion_action = null;
    
                            /**
                             * CallConversionResult conversion_date_time.
                             * @member {string|null|undefined} conversion_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            CallConversionResult.prototype.conversion_date_time = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CallConversionResult _caller_id.
                             * @member {"caller_id"|undefined} _caller_id
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            Object.defineProperty(CallConversionResult.prototype, "_caller_id", {
                                get: $util.oneOfGetter($oneOfFields = ["caller_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversionResult _call_start_date_time.
                             * @member {"call_start_date_time"|undefined} _call_start_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            Object.defineProperty(CallConversionResult.prototype, "_call_start_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["call_start_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversionResult _conversion_action.
                             * @member {"conversion_action"|undefined} _conversion_action
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            Object.defineProperty(CallConversionResult.prototype, "_conversion_action", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_action"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * CallConversionResult _conversion_date_time.
                             * @member {"conversion_date_time"|undefined} _conversion_date_time
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             */
                            Object.defineProperty(CallConversionResult.prototype, "_conversion_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["conversion_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CallConversionResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.ICallConversionResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CallConversionResult} CallConversionResult instance
                             */
                            CallConversionResult.create = function create(properties) {
                                return new CallConversionResult(properties);
                            };
    
                            /**
                             * Encodes the specified CallConversionResult message. Does not implicitly {@link google.ads.googleads.v11.services.CallConversionResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.ICallConversionResult} m CallConversionResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallConversionResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.caller_id != null && Object.hasOwnProperty.call(m, "caller_id"))
                                    w.uint32(42).string(m.caller_id);
                                if (m.call_start_date_time != null && Object.hasOwnProperty.call(m, "call_start_date_time"))
                                    w.uint32(50).string(m.call_start_date_time);
                                if (m.conversion_action != null && Object.hasOwnProperty.call(m, "conversion_action"))
                                    w.uint32(58).string(m.conversion_action);
                                if (m.conversion_date_time != null && Object.hasOwnProperty.call(m, "conversion_date_time"))
                                    w.uint32(66).string(m.conversion_date_time);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CallConversionResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CallConversionResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.ICallConversionResult} message CallConversionResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CallConversionResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CallConversionResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CallConversionResult} CallConversionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallConversionResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CallConversionResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.caller_id = r.string();
                                        break;
                                    case 6:
                                        m.call_start_date_time = r.string();
                                        break;
                                    case 7:
                                        m.conversion_action = r.string();
                                        break;
                                    case 8:
                                        m.conversion_date_time = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CallConversionResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CallConversionResult} CallConversionResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CallConversionResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CallConversionResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CallConversionResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.caller_id != null && m.hasOwnProperty("caller_id")) {
                                    p._caller_id = 1;
                                    if (!$util.isString(m.caller_id))
                                        return "caller_id: string expected";
                                }
                                if (m.call_start_date_time != null && m.hasOwnProperty("call_start_date_time")) {
                                    p._call_start_date_time = 1;
                                    if (!$util.isString(m.call_start_date_time))
                                        return "call_start_date_time: string expected";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    p._conversion_action = 1;
                                    if (!$util.isString(m.conversion_action))
                                        return "conversion_action: string expected";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    p._conversion_date_time = 1;
                                    if (!$util.isString(m.conversion_date_time))
                                        return "conversion_date_time: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CallConversionResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CallConversionResult} CallConversionResult
                             */
                            CallConversionResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CallConversionResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CallConversionResult();
                                if (d.caller_id != null) {
                                    m.caller_id = String(d.caller_id);
                                }
                                if (d.call_start_date_time != null) {
                                    m.call_start_date_time = String(d.call_start_date_time);
                                }
                                if (d.conversion_action != null) {
                                    m.conversion_action = String(d.conversion_action);
                                }
                                if (d.conversion_date_time != null) {
                                    m.conversion_date_time = String(d.conversion_date_time);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CallConversionResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @static
                             * @param {google.ads.googleads.v11.services.CallConversionResult} m CallConversionResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CallConversionResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.caller_id != null && m.hasOwnProperty("caller_id")) {
                                    d.caller_id = m.caller_id;
                                    if (o.oneofs)
                                        d._caller_id = "caller_id";
                                }
                                if (m.call_start_date_time != null && m.hasOwnProperty("call_start_date_time")) {
                                    d.call_start_date_time = m.call_start_date_time;
                                    if (o.oneofs)
                                        d._call_start_date_time = "call_start_date_time";
                                }
                                if (m.conversion_action != null && m.hasOwnProperty("conversion_action")) {
                                    d.conversion_action = m.conversion_action;
                                    if (o.oneofs)
                                        d._conversion_action = "conversion_action";
                                }
                                if (m.conversion_date_time != null && m.hasOwnProperty("conversion_date_time")) {
                                    d.conversion_date_time = m.conversion_date_time;
                                    if (o.oneofs)
                                        d._conversion_date_time = "conversion_date_time";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CallConversionResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CallConversionResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CallConversionResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CallConversionResult;
                        })();
    
                        services.CustomVariable = (function() {
    
                            /**
                             * Properties of a CustomVariable.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomVariable
                             * @property {string|null} [conversion_custom_variable] CustomVariable conversion_custom_variable
                             * @property {string|null} [value] CustomVariable value
                             */
    
                            /**
                             * Constructs a new CustomVariable.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomVariable.
                             * @implements ICustomVariable
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomVariable=} [p] Properties to set
                             */
                            function CustomVariable(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomVariable conversion_custom_variable.
                             * @member {string} conversion_custom_variable
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @instance
                             */
                            CustomVariable.prototype.conversion_custom_variable = "";
    
                            /**
                             * CustomVariable value.
                             * @member {string} value
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @instance
                             */
                            CustomVariable.prototype.value = "";
    
                            /**
                             * Creates a new CustomVariable instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomVariable=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomVariable} CustomVariable instance
                             */
                            CustomVariable.create = function create(properties) {
                                return new CustomVariable(properties);
                            };
    
                            /**
                             * Encodes the specified CustomVariable message. Does not implicitly {@link google.ads.googleads.v11.services.CustomVariable.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomVariable} m CustomVariable message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomVariable.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.conversion_custom_variable != null && Object.hasOwnProperty.call(m, "conversion_custom_variable"))
                                    w.uint32(10).string(m.conversion_custom_variable);
                                if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                                    w.uint32(18).string(m.value);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomVariable message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomVariable.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomVariable} message CustomVariable message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomVariable.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomVariable message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomVariable} CustomVariable
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomVariable.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomVariable();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.conversion_custom_variable = r.string();
                                        break;
                                    case 2:
                                        m.value = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomVariable message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomVariable} CustomVariable
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomVariable.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomVariable message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomVariable.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.conversion_custom_variable != null && m.hasOwnProperty("conversion_custom_variable")) {
                                    if (!$util.isString(m.conversion_custom_variable))
                                        return "conversion_custom_variable: string expected";
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    if (!$util.isString(m.value))
                                        return "value: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomVariable message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomVariable} CustomVariable
                             */
                            CustomVariable.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomVariable)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomVariable();
                                if (d.conversion_custom_variable != null) {
                                    m.conversion_custom_variable = String(d.conversion_custom_variable);
                                }
                                if (d.value != null) {
                                    m.value = String(d.value);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomVariable message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomVariable} m CustomVariable
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomVariable.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.conversion_custom_variable = "";
                                    d.value = "";
                                }
                                if (m.conversion_custom_variable != null && m.hasOwnProperty("conversion_custom_variable")) {
                                    d.conversion_custom_variable = m.conversion_custom_variable;
                                }
                                if (m.value != null && m.hasOwnProperty("value")) {
                                    d.value = m.value;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomVariable to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomVariable
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomVariable.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomVariable;
                        })();
    
                        services.CartData = (function() {
    
                            /**
                             * Properties of a CartData.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICartData
                             * @property {number|null} [merchant_id] CartData merchant_id
                             * @property {string|null} [feed_country_code] CartData feed_country_code
                             * @property {string|null} [feed_language_code] CartData feed_language_code
                             * @property {number|null} [local_transaction_cost] CartData local_transaction_cost
                             * @property {Array.<google.ads.googleads.v11.services.CartData.IItem>|null} [items] CartData items
                             */
    
                            /**
                             * Constructs a new CartData.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CartData.
                             * @implements ICartData
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICartData=} [p] Properties to set
                             */
                            function CartData(p) {
                                this.items = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CartData merchant_id.
                             * @member {number} merchant_id
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @instance
                             */
                            CartData.prototype.merchant_id = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * CartData feed_country_code.
                             * @member {string} feed_country_code
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @instance
                             */
                            CartData.prototype.feed_country_code = "";
    
                            /**
                             * CartData feed_language_code.
                             * @member {string} feed_language_code
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @instance
                             */
                            CartData.prototype.feed_language_code = "";
    
                            /**
                             * CartData local_transaction_cost.
                             * @member {number} local_transaction_cost
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @instance
                             */
                            CartData.prototype.local_transaction_cost = 0;
    
                            /**
                             * CartData items.
                             * @member {Array.<google.ads.googleads.v11.services.CartData.IItem>} items
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @instance
                             */
                            CartData.prototype.items = $util.emptyArray;
    
                            /**
                             * Creates a new CartData instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {google.ads.googleads.v11.services.ICartData=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CartData} CartData instance
                             */
                            CartData.create = function create(properties) {
                                return new CartData(properties);
                            };
    
                            /**
                             * Encodes the specified CartData message. Does not implicitly {@link google.ads.googleads.v11.services.CartData.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {google.ads.googleads.v11.services.ICartData} m CartData message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CartData.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.feed_country_code != null && Object.hasOwnProperty.call(m, "feed_country_code"))
                                    w.uint32(18).string(m.feed_country_code);
                                if (m.feed_language_code != null && Object.hasOwnProperty.call(m, "feed_language_code"))
                                    w.uint32(26).string(m.feed_language_code);
                                if (m.local_transaction_cost != null && Object.hasOwnProperty.call(m, "local_transaction_cost"))
                                    w.uint32(33).double(m.local_transaction_cost);
                                if (m.items != null && m.items.length) {
                                    for (var i = 0; i < m.items.length; ++i)
                                        $root.google.ads.googleads.v11.services.CartData.Item.encode(m.items[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.merchant_id != null && Object.hasOwnProperty.call(m, "merchant_id"))
                                    w.uint32(48).int64(m.merchant_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CartData message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CartData.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {google.ads.googleads.v11.services.ICartData} message CartData message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CartData.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CartData message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CartData} CartData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CartData.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CartData();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.merchant_id = r.int64();
                                        break;
                                    case 2:
                                        m.feed_country_code = r.string();
                                        break;
                                    case 3:
                                        m.feed_language_code = r.string();
                                        break;
                                    case 4:
                                        m.local_transaction_cost = r.double();
                                        break;
                                    case 5:
                                        if (!(m.items && m.items.length))
                                            m.items = [];
                                        m.items.push($root.google.ads.googleads.v11.services.CartData.Item.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CartData message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CartData} CartData
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CartData.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CartData message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CartData.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                    if (!$util.isInteger(m.merchant_id) && !(m.merchant_id && $util.isInteger(m.merchant_id.low) && $util.isInteger(m.merchant_id.high)))
                                        return "merchant_id: integer|Long expected";
                                }
                                if (m.feed_country_code != null && m.hasOwnProperty("feed_country_code")) {
                                    if (!$util.isString(m.feed_country_code))
                                        return "feed_country_code: string expected";
                                }
                                if (m.feed_language_code != null && m.hasOwnProperty("feed_language_code")) {
                                    if (!$util.isString(m.feed_language_code))
                                        return "feed_language_code: string expected";
                                }
                                if (m.local_transaction_cost != null && m.hasOwnProperty("local_transaction_cost")) {
                                    if (typeof m.local_transaction_cost !== "number")
                                        return "local_transaction_cost: number expected";
                                }
                                if (m.items != null && m.hasOwnProperty("items")) {
                                    if (!Array.isArray(m.items))
                                        return "items: array expected";
                                    for (var i = 0; i < m.items.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CartData.Item.verify(m.items[i]);
                                            if (e)
                                                return "items." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CartData message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CartData} CartData
                             */
                            CartData.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CartData)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CartData();
                                if (d.merchant_id != null) {
                                    if ($util.Long)
                                        (m.merchant_id = $util.Long.fromValue(d.merchant_id)).unsigned = false;
                                    else if (typeof d.merchant_id === "string")
                                        m.merchant_id = parseInt(d.merchant_id, 10);
                                    else if (typeof d.merchant_id === "number")
                                        m.merchant_id = d.merchant_id;
                                    else if (typeof d.merchant_id === "object")
                                        m.merchant_id = new $util.LongBits(d.merchant_id.low >>> 0, d.merchant_id.high >>> 0).toNumber();
                                }
                                if (d.feed_country_code != null) {
                                    m.feed_country_code = String(d.feed_country_code);
                                }
                                if (d.feed_language_code != null) {
                                    m.feed_language_code = String(d.feed_language_code);
                                }
                                if (d.local_transaction_cost != null) {
                                    m.local_transaction_cost = Number(d.local_transaction_cost);
                                }
                                if (d.items) {
                                    if (!Array.isArray(d.items))
                                        throw TypeError(".google.ads.googleads.v11.services.CartData.items: array expected");
                                    m.items = [];
                                    for (var i = 0; i < d.items.length; ++i) {
                                        if (typeof d.items[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.CartData.items: object expected");
                                        m.items[i] = $root.google.ads.googleads.v11.services.CartData.Item.fromObject(d.items[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CartData message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @static
                             * @param {google.ads.googleads.v11.services.CartData} m CartData
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CartData.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.items = [];
                                }
                                if (o.defaults) {
                                    d.feed_country_code = "";
                                    d.feed_language_code = "";
                                    d.local_transaction_cost = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.merchant_id = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.merchant_id = o.longs === String ? "0" : 0;
                                }
                                if (m.feed_country_code != null && m.hasOwnProperty("feed_country_code")) {
                                    d.feed_country_code = m.feed_country_code;
                                }
                                if (m.feed_language_code != null && m.hasOwnProperty("feed_language_code")) {
                                    d.feed_language_code = m.feed_language_code;
                                }
                                if (m.local_transaction_cost != null && m.hasOwnProperty("local_transaction_cost")) {
                                    d.local_transaction_cost = o.json && !isFinite(m.local_transaction_cost) ? String(m.local_transaction_cost) : m.local_transaction_cost;
                                }
                                if (m.items && m.items.length) {
                                    d.items = [];
                                    for (var j = 0; j < m.items.length; ++j) {
                                        d.items[j] = $root.google.ads.googleads.v11.services.CartData.Item.toObject(m.items[j], o);
                                    }
                                }
                                if (m.merchant_id != null && m.hasOwnProperty("merchant_id")) {
                                    if (typeof m.merchant_id === "number")
                                        d.merchant_id = o.longs === String ? String(m.merchant_id) : m.merchant_id;
                                    else
                                        d.merchant_id = o.longs === String ? $util.Long.prototype.toString.call(m.merchant_id) : o.longs === Number ? new $util.LongBits(m.merchant_id.low >>> 0, m.merchant_id.high >>> 0).toNumber() : m.merchant_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CartData to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CartData
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CartData.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            CartData.Item = (function() {
    
                                /**
                                 * Properties of an Item.
                                 * @memberof google.ads.googleads.v11.services.CartData
                                 * @interface IItem
                                 * @property {string|null} [product_id] Item product_id
                                 * @property {number|null} [quantity] Item quantity
                                 * @property {number|null} [unit_price] Item unit_price
                                 */
    
                                /**
                                 * Constructs a new Item.
                                 * @memberof google.ads.googleads.v11.services.CartData
                                 * @classdesc Represents an Item.
                                 * @implements IItem
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.CartData.IItem=} [p] Properties to set
                                 */
                                function Item(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * Item product_id.
                                 * @member {string} product_id
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @instance
                                 */
                                Item.prototype.product_id = "";
    
                                /**
                                 * Item quantity.
                                 * @member {number} quantity
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @instance
                                 */
                                Item.prototype.quantity = 0;
    
                                /**
                                 * Item unit_price.
                                 * @member {number} unit_price
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @instance
                                 */
                                Item.prototype.unit_price = 0;
    
                                /**
                                 * Creates a new Item instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {google.ads.googleads.v11.services.CartData.IItem=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.CartData.Item} Item instance
                                 */
                                Item.create = function create(properties) {
                                    return new Item(properties);
                                };
    
                                /**
                                 * Encodes the specified Item message. Does not implicitly {@link google.ads.googleads.v11.services.CartData.Item.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {google.ads.googleads.v11.services.CartData.IItem} m Item message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Item.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.product_id != null && Object.hasOwnProperty.call(m, "product_id"))
                                        w.uint32(10).string(m.product_id);
                                    if (m.quantity != null && Object.hasOwnProperty.call(m, "quantity"))
                                        w.uint32(16).int32(m.quantity);
                                    if (m.unit_price != null && Object.hasOwnProperty.call(m, "unit_price"))
                                        w.uint32(25).double(m.unit_price);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified Item message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CartData.Item.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {google.ads.googleads.v11.services.CartData.IItem} message Item message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Item.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an Item message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.CartData.Item} Item
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Item.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CartData.Item();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.product_id = r.string();
                                            break;
                                        case 2:
                                            m.quantity = r.int32();
                                            break;
                                        case 3:
                                            m.unit_price = r.double();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes an Item message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.CartData.Item} Item
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Item.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an Item message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                Item.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.product_id != null && m.hasOwnProperty("product_id")) {
                                        if (!$util.isString(m.product_id))
                                            return "product_id: string expected";
                                    }
                                    if (m.quantity != null && m.hasOwnProperty("quantity")) {
                                        if (!$util.isInteger(m.quantity))
                                            return "quantity: integer expected";
                                    }
                                    if (m.unit_price != null && m.hasOwnProperty("unit_price")) {
                                        if (typeof m.unit_price !== "number")
                                            return "unit_price: number expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an Item message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.CartData.Item} Item
                                 */
                                Item.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.CartData.Item)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.CartData.Item();
                                    if (d.product_id != null) {
                                        m.product_id = String(d.product_id);
                                    }
                                    if (d.quantity != null) {
                                        m.quantity = d.quantity | 0;
                                    }
                                    if (d.unit_price != null) {
                                        m.unit_price = Number(d.unit_price);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from an Item message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @static
                                 * @param {google.ads.googleads.v11.services.CartData.Item} m Item
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                Item.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.product_id = "";
                                        d.quantity = 0;
                                        d.unit_price = 0;
                                    }
                                    if (m.product_id != null && m.hasOwnProperty("product_id")) {
                                        d.product_id = m.product_id;
                                    }
                                    if (m.quantity != null && m.hasOwnProperty("quantity")) {
                                        d.quantity = m.quantity;
                                    }
                                    if (m.unit_price != null && m.hasOwnProperty("unit_price")) {
                                        d.unit_price = o.json && !isFinite(m.unit_price) ? String(m.unit_price) : m.unit_price;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this Item to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.CartData.Item
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                Item.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return Item;
                            })();
    
                            return CartData;
                        })();
    
                        services.CustomAudienceService = (function() {
    
                            /**
                             * Constructs a new CustomAudienceService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomAudienceService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomAudienceService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomAudienceService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomAudienceService;
    
                            /**
                             * Creates new CustomAudienceService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomAudienceService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomAudienceService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomAudienceService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomAudienceService|mutateCustomAudiences}.
                             * @memberof google.ads.googleads.v11.services.CustomAudienceService
                             * @typedef MutateCustomAudiencesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomAudiencesResponse} [response] MutateCustomAudiencesResponse
                             */
    
                            /**
                             * Calls MutateCustomAudiences.
                             * @function mutateCustomAudiences
                             * @memberof google.ads.googleads.v11.services.CustomAudienceService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesRequest} request MutateCustomAudiencesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomAudienceService.MutateCustomAudiencesCallback} callback Node-style callback called with the error, if any, and MutateCustomAudiencesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomAudienceService.prototype.mutateCustomAudiences = function mutateCustomAudiences(request, callback) {
                                return this.rpcCall(mutateCustomAudiences, $root.google.ads.googleads.v11.services.MutateCustomAudiencesRequest, $root.google.ads.googleads.v11.services.MutateCustomAudiencesResponse, request, callback);
                            }, "name", { value: "MutateCustomAudiences" });
    
                            /**
                             * Calls MutateCustomAudiences.
                             * @function mutateCustomAudiences
                             * @memberof google.ads.googleads.v11.services.CustomAudienceService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesRequest} request MutateCustomAudiencesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomAudiencesResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomAudienceService;
                        })();
    
                        services.MutateCustomAudiencesRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomAudiencesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomAudiencesRequest
                             * @property {string|null} [customer_id] MutateCustomAudiencesRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomAudienceOperation>|null} [operations] MutateCustomAudiencesRequest operations
                             * @property {boolean|null} [validate_only] MutateCustomAudiencesRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCustomAudiencesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomAudiencesRequest.
                             * @implements IMutateCustomAudiencesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesRequest=} [p] Properties to set
                             */
                            function MutateCustomAudiencesRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomAudiencesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @instance
                             */
                            MutateCustomAudiencesRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomAudiencesRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomAudienceOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @instance
                             */
                            MutateCustomAudiencesRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomAudiencesRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @instance
                             */
                            MutateCustomAudiencesRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCustomAudiencesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesRequest} MutateCustomAudiencesRequest instance
                             */
                            MutateCustomAudiencesRequest.create = function create(properties) {
                                return new MutateCustomAudiencesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomAudiencesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomAudiencesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesRequest} m MutateCustomAudiencesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomAudiencesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomAudienceOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomAudiencesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomAudiencesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesRequest} message MutateCustomAudiencesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomAudiencesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomAudiencesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesRequest} MutateCustomAudiencesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomAudiencesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomAudiencesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomAudienceOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomAudiencesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesRequest} MutateCustomAudiencesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomAudiencesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomAudiencesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomAudiencesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomAudienceOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomAudiencesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesRequest} MutateCustomAudiencesRequest
                             */
                            MutateCustomAudiencesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomAudiencesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomAudiencesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomAudiencesRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomAudiencesRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomAudienceOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomAudiencesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomAudiencesRequest} m MutateCustomAudiencesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomAudiencesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomAudienceOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomAudiencesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomAudiencesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomAudiencesRequest;
                        })();
    
                        services.CustomAudienceOperation = (function() {
    
                            /**
                             * Properties of a CustomAudienceOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomAudienceOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomAudienceOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomAudience|null} [create] CustomAudienceOperation create
                             * @property {google.ads.googleads.v11.resources.ICustomAudience|null} [update] CustomAudienceOperation update
                             * @property {string|null} [remove] CustomAudienceOperation remove
                             */
    
                            /**
                             * Constructs a new CustomAudienceOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomAudienceOperation.
                             * @implements ICustomAudienceOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomAudienceOperation=} [p] Properties to set
                             */
                            function CustomAudienceOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomAudienceOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @instance
                             */
                            CustomAudienceOperation.prototype.update_mask = null;
    
                            /**
                             * CustomAudienceOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomAudience|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @instance
                             */
                            CustomAudienceOperation.prototype.create = null;
    
                            /**
                             * CustomAudienceOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomAudience|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @instance
                             */
                            CustomAudienceOperation.prototype.update = null;
    
                            /**
                             * CustomAudienceOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @instance
                             */
                            CustomAudienceOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomAudienceOperation operation.
                             * @member {"create"|"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @instance
                             */
                            Object.defineProperty(CustomAudienceOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomAudienceOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomAudienceOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomAudienceOperation} CustomAudienceOperation instance
                             */
                            CustomAudienceOperation.create = function create(properties) {
                                return new CustomAudienceOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomAudienceOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomAudienceOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomAudienceOperation} m CustomAudienceOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomAudience.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomAudience.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(26).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomAudienceOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomAudienceOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomAudienceOperation} message CustomAudienceOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomAudienceOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomAudienceOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomAudienceOperation} CustomAudienceOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomAudienceOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomAudience.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomAudience.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomAudienceOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomAudienceOperation} CustomAudienceOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomAudienceOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomAudienceOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomAudienceOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomAudience.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomAudience.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomAudienceOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomAudienceOperation} CustomAudienceOperation
                             */
                            CustomAudienceOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomAudienceOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomAudienceOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomAudienceOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomAudienceOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomAudience.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomAudienceOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomAudience.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomAudienceOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomAudienceOperation} m CustomAudienceOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomAudienceOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomAudience.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomAudience.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomAudienceOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomAudienceOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomAudienceOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomAudienceOperation;
                        })();
    
                        services.MutateCustomAudiencesResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomAudiencesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomAudiencesResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomAudienceResult>|null} [results] MutateCustomAudiencesResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomAudiencesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomAudiencesResponse.
                             * @implements IMutateCustomAudiencesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesResponse=} [p] Properties to set
                             */
                            function MutateCustomAudiencesResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomAudiencesResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomAudienceResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @instance
                             */
                            MutateCustomAudiencesResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomAudiencesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesResponse} MutateCustomAudiencesResponse instance
                             */
                            MutateCustomAudiencesResponse.create = function create(properties) {
                                return new MutateCustomAudiencesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomAudiencesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomAudiencesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesResponse} m MutateCustomAudiencesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomAudiencesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomAudienceResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomAudiencesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomAudiencesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudiencesResponse} message MutateCustomAudiencesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomAudiencesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomAudiencesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesResponse} MutateCustomAudiencesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomAudiencesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomAudiencesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomAudienceResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomAudiencesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesResponse} MutateCustomAudiencesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomAudiencesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomAudiencesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomAudiencesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomAudienceResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomAudiencesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudiencesResponse} MutateCustomAudiencesResponse
                             */
                            MutateCustomAudiencesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomAudiencesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomAudiencesResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomAudiencesResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomAudiencesResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomAudienceResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomAudiencesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomAudiencesResponse} m MutateCustomAudiencesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomAudiencesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomAudienceResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomAudiencesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudiencesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomAudiencesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomAudiencesResponse;
                        })();
    
                        services.MutateCustomAudienceResult = (function() {
    
                            /**
                             * Properties of a MutateCustomAudienceResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomAudienceResult
                             * @property {string|null} [resource_name] MutateCustomAudienceResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomAudienceResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomAudienceResult.
                             * @implements IMutateCustomAudienceResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudienceResult=} [p] Properties to set
                             */
                            function MutateCustomAudienceResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomAudienceResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @instance
                             */
                            MutateCustomAudienceResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomAudienceResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudienceResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudienceResult} MutateCustomAudienceResult instance
                             */
                            MutateCustomAudienceResult.create = function create(properties) {
                                return new MutateCustomAudienceResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomAudienceResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomAudienceResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudienceResult} m MutateCustomAudienceResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomAudienceResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomAudienceResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomAudienceResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomAudienceResult} message MutateCustomAudienceResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomAudienceResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomAudienceResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudienceResult} MutateCustomAudienceResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomAudienceResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomAudienceResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomAudienceResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudienceResult} MutateCustomAudienceResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomAudienceResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomAudienceResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomAudienceResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomAudienceResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomAudienceResult} MutateCustomAudienceResult
                             */
                            MutateCustomAudienceResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomAudienceResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomAudienceResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomAudienceResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomAudienceResult} m MutateCustomAudienceResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomAudienceResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomAudienceResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomAudienceResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomAudienceResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomAudienceResult;
                        })();
    
                        services.CustomInterestService = (function() {
    
                            /**
                             * Constructs a new CustomInterestService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomInterestService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomInterestService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomInterestService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomInterestService;
    
                            /**
                             * Creates new CustomInterestService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomInterestService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomInterestService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomInterestService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomInterestService|mutateCustomInterests}.
                             * @memberof google.ads.googleads.v11.services.CustomInterestService
                             * @typedef MutateCustomInterestsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomInterestsResponse} [response] MutateCustomInterestsResponse
                             */
    
                            /**
                             * Calls MutateCustomInterests.
                             * @function mutateCustomInterests
                             * @memberof google.ads.googleads.v11.services.CustomInterestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsRequest} request MutateCustomInterestsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomInterestService.MutateCustomInterestsCallback} callback Node-style callback called with the error, if any, and MutateCustomInterestsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomInterestService.prototype.mutateCustomInterests = function mutateCustomInterests(request, callback) {
                                return this.rpcCall(mutateCustomInterests, $root.google.ads.googleads.v11.services.MutateCustomInterestsRequest, $root.google.ads.googleads.v11.services.MutateCustomInterestsResponse, request, callback);
                            }, "name", { value: "MutateCustomInterests" });
    
                            /**
                             * Calls MutateCustomInterests.
                             * @function mutateCustomInterests
                             * @memberof google.ads.googleads.v11.services.CustomInterestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsRequest} request MutateCustomInterestsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomInterestsResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomInterestService;
                        })();
    
                        services.MutateCustomInterestsRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomInterestsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomInterestsRequest
                             * @property {string|null} [customer_id] MutateCustomInterestsRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomInterestOperation>|null} [operations] MutateCustomInterestsRequest operations
                             * @property {boolean|null} [validate_only] MutateCustomInterestsRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCustomInterestsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomInterestsRequest.
                             * @implements IMutateCustomInterestsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsRequest=} [p] Properties to set
                             */
                            function MutateCustomInterestsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomInterestsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @instance
                             */
                            MutateCustomInterestsRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomInterestsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomInterestOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @instance
                             */
                            MutateCustomInterestsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomInterestsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @instance
                             */
                            MutateCustomInterestsRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCustomInterestsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsRequest} MutateCustomInterestsRequest instance
                             */
                            MutateCustomInterestsRequest.create = function create(properties) {
                                return new MutateCustomInterestsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomInterestsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomInterestsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsRequest} m MutateCustomInterestsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomInterestsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomInterestOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomInterestsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomInterestsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsRequest} message MutateCustomInterestsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomInterestsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomInterestsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsRequest} MutateCustomInterestsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomInterestsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomInterestsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomInterestOperation.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomInterestsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsRequest} MutateCustomInterestsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomInterestsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomInterestsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomInterestsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomInterestOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomInterestsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsRequest} MutateCustomInterestsRequest
                             */
                            MutateCustomInterestsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomInterestsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomInterestsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomInterestsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomInterestsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomInterestOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomInterestsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomInterestsRequest} m MutateCustomInterestsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomInterestsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomInterestOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomInterestsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomInterestsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomInterestsRequest;
                        })();
    
                        services.CustomInterestOperation = (function() {
    
                            /**
                             * Properties of a CustomInterestOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomInterestOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomInterestOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomInterest|null} [create] CustomInterestOperation create
                             * @property {google.ads.googleads.v11.resources.ICustomInterest|null} [update] CustomInterestOperation update
                             */
    
                            /**
                             * Constructs a new CustomInterestOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomInterestOperation.
                             * @implements ICustomInterestOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomInterestOperation=} [p] Properties to set
                             */
                            function CustomInterestOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomInterestOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @instance
                             */
                            CustomInterestOperation.prototype.update_mask = null;
    
                            /**
                             * CustomInterestOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomInterest|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @instance
                             */
                            CustomInterestOperation.prototype.create = null;
    
                            /**
                             * CustomInterestOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomInterest|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @instance
                             */
                            CustomInterestOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomInterestOperation operation.
                             * @member {"create"|"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @instance
                             */
                            Object.defineProperty(CustomInterestOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomInterestOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomInterestOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomInterestOperation} CustomInterestOperation instance
                             */
                            CustomInterestOperation.create = function create(properties) {
                                return new CustomInterestOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomInterestOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomInterestOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomInterestOperation} m CustomInterestOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomInterest.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomInterest.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomInterestOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomInterestOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomInterestOperation} message CustomInterestOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomInterestOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomInterestOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomInterestOperation} CustomInterestOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomInterestOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomInterest.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomInterest.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomInterestOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomInterestOperation} CustomInterestOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomInterestOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomInterestOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomInterestOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomInterest.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomInterest.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomInterestOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomInterestOperation} CustomInterestOperation
                             */
                            CustomInterestOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomInterestOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomInterestOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomInterestOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomInterestOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomInterest.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomInterestOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomInterest.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomInterestOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomInterestOperation} m CustomInterestOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomInterestOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomInterest.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomInterest.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomInterestOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomInterestOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomInterestOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomInterestOperation;
                        })();
    
                        services.MutateCustomInterestsResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomInterestsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomInterestsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomInterestResult>|null} [results] MutateCustomInterestsResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomInterestsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomInterestsResponse.
                             * @implements IMutateCustomInterestsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsResponse=} [p] Properties to set
                             */
                            function MutateCustomInterestsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomInterestsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomInterestResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @instance
                             */
                            MutateCustomInterestsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomInterestsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsResponse} MutateCustomInterestsResponse instance
                             */
                            MutateCustomInterestsResponse.create = function create(properties) {
                                return new MutateCustomInterestsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomInterestsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomInterestsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsResponse} m MutateCustomInterestsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomInterestsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomInterestResult.encode(m.results[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomInterestsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomInterestsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestsResponse} message MutateCustomInterestsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomInterestsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomInterestsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsResponse} MutateCustomInterestsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomInterestsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomInterestsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomInterestResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomInterestsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsResponse} MutateCustomInterestsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomInterestsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomInterestsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomInterestsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomInterestResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomInterestsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestsResponse} MutateCustomInterestsResponse
                             */
                            MutateCustomInterestsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomInterestsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomInterestsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomInterestsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomInterestsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomInterestResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomInterestsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomInterestsResponse} m MutateCustomInterestsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomInterestsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomInterestResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomInterestsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomInterestsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomInterestsResponse;
                        })();
    
                        services.MutateCustomInterestResult = (function() {
    
                            /**
                             * Properties of a MutateCustomInterestResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomInterestResult
                             * @property {string|null} [resource_name] MutateCustomInterestResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomInterestResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomInterestResult.
                             * @implements IMutateCustomInterestResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestResult=} [p] Properties to set
                             */
                            function MutateCustomInterestResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomInterestResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @instance
                             */
                            MutateCustomInterestResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomInterestResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestResult} MutateCustomInterestResult instance
                             */
                            MutateCustomInterestResult.create = function create(properties) {
                                return new MutateCustomInterestResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomInterestResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomInterestResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestResult} m MutateCustomInterestResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomInterestResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomInterestResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomInterestResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomInterestResult} message MutateCustomInterestResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomInterestResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomInterestResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestResult} MutateCustomInterestResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomInterestResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomInterestResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomInterestResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestResult} MutateCustomInterestResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomInterestResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomInterestResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomInterestResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomInterestResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomInterestResult} MutateCustomInterestResult
                             */
                            MutateCustomInterestResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomInterestResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomInterestResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomInterestResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomInterestResult} m MutateCustomInterestResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomInterestResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomInterestResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomInterestResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomInterestResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomInterestResult;
                        })();
    
                        services.CustomerClientLinkService = (function() {
    
                            /**
                             * Constructs a new CustomerClientLinkService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerClientLinkService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerClientLinkService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerClientLinkService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerClientLinkService;
    
                            /**
                             * Creates new CustomerClientLinkService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerClientLinkService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerClientLinkService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerClientLinkService|mutateCustomerClientLink}.
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkService
                             * @typedef MutateCustomerClientLinkCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerClientLinkResponse} [response] MutateCustomerClientLinkResponse
                             */
    
                            /**
                             * Calls MutateCustomerClientLink.
                             * @function mutateCustomerClientLink
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkRequest} request MutateCustomerClientLinkRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerClientLinkService.MutateCustomerClientLinkCallback} callback Node-style callback called with the error, if any, and MutateCustomerClientLinkResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerClientLinkService.prototype.mutateCustomerClientLink = function mutateCustomerClientLink(request, callback) {
                                return this.rpcCall(mutateCustomerClientLink, $root.google.ads.googleads.v11.services.MutateCustomerClientLinkRequest, $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResponse, request, callback);
                            }, "name", { value: "MutateCustomerClientLink" });
    
                            /**
                             * Calls MutateCustomerClientLink.
                             * @function mutateCustomerClientLink
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkRequest} request MutateCustomerClientLinkRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerClientLinkResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerClientLinkService;
                        })();
    
                        services.MutateCustomerClientLinkRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerClientLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerClientLinkRequest
                             * @property {string|null} [customer_id] MutateCustomerClientLinkRequest customer_id
                             * @property {google.ads.googleads.v11.services.ICustomerClientLinkOperation|null} [operation] MutateCustomerClientLinkRequest operation
                             * @property {boolean|null} [validate_only] MutateCustomerClientLinkRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCustomerClientLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerClientLinkRequest.
                             * @implements IMutateCustomerClientLinkRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkRequest=} [p] Properties to set
                             */
                            function MutateCustomerClientLinkRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerClientLinkRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @instance
                             */
                            MutateCustomerClientLinkRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerClientLinkRequest operation.
                             * @member {google.ads.googleads.v11.services.ICustomerClientLinkOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @instance
                             */
                            MutateCustomerClientLinkRequest.prototype.operation = null;
    
                            /**
                             * MutateCustomerClientLinkRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @instance
                             */
                            MutateCustomerClientLinkRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCustomerClientLinkRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkRequest} MutateCustomerClientLinkRequest instance
                             */
                            MutateCustomerClientLinkRequest.create = function create(properties) {
                                return new MutateCustomerClientLinkRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerClientLinkRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerClientLinkRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkRequest} m MutateCustomerClientLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerClientLinkRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.CustomerClientLinkOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerClientLinkRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerClientLinkRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkRequest} message MutateCustomerClientLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerClientLinkRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerClientLinkRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkRequest} MutateCustomerClientLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerClientLinkRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerClientLinkRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.CustomerClientLinkOperation.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerClientLinkRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkRequest} MutateCustomerClientLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerClientLinkRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerClientLinkRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerClientLinkRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerClientLinkOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerClientLinkRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkRequest} MutateCustomerClientLinkRequest
                             */
                            MutateCustomerClientLinkRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerClientLinkRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerClientLinkRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerClientLinkRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.CustomerClientLinkOperation.fromObject(d.operation);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerClientLinkRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerClientLinkRequest} m MutateCustomerClientLinkRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerClientLinkRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.CustomerClientLinkOperation.toObject(m.operation, o);
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerClientLinkRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerClientLinkRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerClientLinkRequest;
                        })();
    
                        services.CustomerClientLinkOperation = (function() {
    
                            /**
                             * Properties of a CustomerClientLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerClientLinkOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerClientLinkOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomerClientLink|null} [create] CustomerClientLinkOperation create
                             * @property {google.ads.googleads.v11.resources.ICustomerClientLink|null} [update] CustomerClientLinkOperation update
                             */
    
                            /**
                             * Constructs a new CustomerClientLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerClientLinkOperation.
                             * @implements ICustomerClientLinkOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerClientLinkOperation=} [p] Properties to set
                             */
                            function CustomerClientLinkOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerClientLinkOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @instance
                             */
                            CustomerClientLinkOperation.prototype.update_mask = null;
    
                            /**
                             * CustomerClientLinkOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerClientLink|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @instance
                             */
                            CustomerClientLinkOperation.prototype.create = null;
    
                            /**
                             * CustomerClientLinkOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomerClientLink|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @instance
                             */
                            CustomerClientLinkOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerClientLinkOperation operation.
                             * @member {"create"|"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerClientLinkOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerClientLinkOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerClientLinkOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerClientLinkOperation} CustomerClientLinkOperation instance
                             */
                            CustomerClientLinkOperation.create = function create(properties) {
                                return new CustomerClientLinkOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerClientLinkOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerClientLinkOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerClientLinkOperation} m CustomerClientLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClientLinkOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerClientLink.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomerClientLink.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerClientLinkOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerClientLinkOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerClientLinkOperation} message CustomerClientLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerClientLinkOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerClientLinkOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerClientLinkOperation} CustomerClientLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClientLinkOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerClientLinkOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerClientLink.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomerClientLink.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerClientLinkOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerClientLinkOperation} CustomerClientLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerClientLinkOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerClientLinkOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerClientLinkOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerClientLink.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerClientLink.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerClientLinkOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerClientLinkOperation} CustomerClientLinkOperation
                             */
                            CustomerClientLinkOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerClientLinkOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerClientLinkOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerClientLinkOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerClientLinkOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerClientLink.fromObject(d.create);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerClientLinkOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomerClientLink.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerClientLinkOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerClientLinkOperation} m CustomerClientLinkOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerClientLinkOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerClientLink.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomerClientLink.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerClientLinkOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerClientLinkOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerClientLinkOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerClientLinkOperation;
                        })();
    
                        services.MutateCustomerClientLinkResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerClientLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerClientLinkResponse
                             * @property {google.ads.googleads.v11.services.IMutateCustomerClientLinkResult|null} [result] MutateCustomerClientLinkResponse result
                             */
    
                            /**
                             * Constructs a new MutateCustomerClientLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerClientLinkResponse.
                             * @implements IMutateCustomerClientLinkResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResponse=} [p] Properties to set
                             */
                            function MutateCustomerClientLinkResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerClientLinkResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerClientLinkResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @instance
                             */
                            MutateCustomerClientLinkResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateCustomerClientLinkResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResponse} MutateCustomerClientLinkResponse instance
                             */
                            MutateCustomerClientLinkResponse.create = function create(properties) {
                                return new MutateCustomerClientLinkResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerClientLinkResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerClientLinkResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResponse} m MutateCustomerClientLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerClientLinkResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult.encode(m.result, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerClientLinkResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerClientLinkResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResponse} message MutateCustomerClientLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerClientLinkResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerClientLinkResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResponse} MutateCustomerClientLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerClientLinkResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.result = $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerClientLinkResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResponse} MutateCustomerClientLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerClientLinkResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerClientLinkResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerClientLinkResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerClientLinkResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResponse} MutateCustomerClientLinkResponse
                             */
                            MutateCustomerClientLinkResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerClientLinkResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerClientLinkResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerClientLinkResponse} m MutateCustomerClientLinkResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerClientLinkResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerClientLinkResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerClientLinkResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerClientLinkResponse;
                        })();
    
                        services.MutateCustomerClientLinkResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerClientLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerClientLinkResult
                             * @property {string|null} [resource_name] MutateCustomerClientLinkResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomerClientLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerClientLinkResult.
                             * @implements IMutateCustomerClientLinkResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResult=} [p] Properties to set
                             */
                            function MutateCustomerClientLinkResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerClientLinkResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @instance
                             */
                            MutateCustomerClientLinkResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomerClientLinkResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResult} MutateCustomerClientLinkResult instance
                             */
                            MutateCustomerClientLinkResult.create = function create(properties) {
                                return new MutateCustomerClientLinkResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerClientLinkResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerClientLinkResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResult} m MutateCustomerClientLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerClientLinkResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerClientLinkResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerClientLinkResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerClientLinkResult} message MutateCustomerClientLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerClientLinkResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerClientLinkResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResult} MutateCustomerClientLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerClientLinkResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerClientLinkResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResult} MutateCustomerClientLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerClientLinkResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerClientLinkResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerClientLinkResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerClientLinkResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerClientLinkResult} MutateCustomerClientLinkResult
                             */
                            MutateCustomerClientLinkResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerClientLinkResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerClientLinkResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerClientLinkResult} m MutateCustomerClientLinkResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerClientLinkResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerClientLinkResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerClientLinkResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerClientLinkResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerClientLinkResult;
                        })();
    
                        services.CustomerManagerLinkService = (function() {
    
                            /**
                             * Constructs a new CustomerManagerLinkService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerManagerLinkService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerManagerLinkService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerManagerLinkService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerManagerLinkService;
    
                            /**
                             * Creates new CustomerManagerLinkService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerManagerLinkService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerManagerLinkService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerManagerLinkService|mutateCustomerManagerLink}.
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkService
                             * @typedef MutateCustomerManagerLinkCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse} [response] MutateCustomerManagerLinkResponse
                             */
    
                            /**
                             * Calls MutateCustomerManagerLink.
                             * @function mutateCustomerManagerLink
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkRequest} request MutateCustomerManagerLinkRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerManagerLinkService.MutateCustomerManagerLinkCallback} callback Node-style callback called with the error, if any, and MutateCustomerManagerLinkResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerManagerLinkService.prototype.mutateCustomerManagerLink = function mutateCustomerManagerLink(request, callback) {
                                return this.rpcCall(mutateCustomerManagerLink, $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest, $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse, request, callback);
                            }, "name", { value: "MutateCustomerManagerLink" });
    
                            /**
                             * Calls MutateCustomerManagerLink.
                             * @function mutateCustomerManagerLink
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkRequest} request MutateCustomerManagerLinkRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerManagerLinkService|moveManagerLink}.
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkService
                             * @typedef MoveManagerLinkCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MoveManagerLinkResponse} [response] MoveManagerLinkResponse
                             */
    
                            /**
                             * Calls MoveManagerLink.
                             * @function moveManagerLink
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkRequest} request MoveManagerLinkRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerManagerLinkService.MoveManagerLinkCallback} callback Node-style callback called with the error, if any, and MoveManagerLinkResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerManagerLinkService.prototype.moveManagerLink = function moveManagerLink(request, callback) {
                                return this.rpcCall(moveManagerLink, $root.google.ads.googleads.v11.services.MoveManagerLinkRequest, $root.google.ads.googleads.v11.services.MoveManagerLinkResponse, request, callback);
                            }, "name", { value: "MoveManagerLink" });
    
                            /**
                             * Calls MoveManagerLink.
                             * @function moveManagerLink
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkRequest} request MoveManagerLinkRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MoveManagerLinkResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerManagerLinkService;
                        })();
    
                        services.MutateCustomerManagerLinkRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerManagerLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerManagerLinkRequest
                             * @property {string|null} [customer_id] MutateCustomerManagerLinkRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.ICustomerManagerLinkOperation>|null} [operations] MutateCustomerManagerLinkRequest operations
                             * @property {boolean|null} [validate_only] MutateCustomerManagerLinkRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateCustomerManagerLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerManagerLinkRequest.
                             * @implements IMutateCustomerManagerLinkRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkRequest=} [p] Properties to set
                             */
                            function MutateCustomerManagerLinkRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerManagerLinkRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @instance
                             */
                            MutateCustomerManagerLinkRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerManagerLinkRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.ICustomerManagerLinkOperation>} operations
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @instance
                             */
                            MutateCustomerManagerLinkRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * MutateCustomerManagerLinkRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @instance
                             */
                            MutateCustomerManagerLinkRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateCustomerManagerLinkRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest} MutateCustomerManagerLinkRequest instance
                             */
                            MutateCustomerManagerLinkRequest.create = function create(properties) {
                                return new MutateCustomerManagerLinkRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerManagerLinkRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkRequest} m MutateCustomerManagerLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerManagerLinkRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.CustomerManagerLinkOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerManagerLinkRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkRequest} message MutateCustomerManagerLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerManagerLinkRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerManagerLinkRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest} MutateCustomerManagerLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerManagerLinkRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.CustomerManagerLinkOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerManagerLinkRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest} MutateCustomerManagerLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerManagerLinkRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerManagerLinkRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerManagerLinkRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.CustomerManagerLinkOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerManagerLinkRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest} MutateCustomerManagerLinkRequest
                             */
                            MutateCustomerManagerLinkRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.CustomerManagerLinkOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerManagerLinkRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest} m MutateCustomerManagerLinkRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerManagerLinkRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.CustomerManagerLinkOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerManagerLinkRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerManagerLinkRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerManagerLinkRequest;
                        })();
    
                        services.MoveManagerLinkRequest = (function() {
    
                            /**
                             * Properties of a MoveManagerLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMoveManagerLinkRequest
                             * @property {string|null} [customer_id] MoveManagerLinkRequest customer_id
                             * @property {string|null} [previous_customer_manager_link] MoveManagerLinkRequest previous_customer_manager_link
                             * @property {string|null} [new_manager] MoveManagerLinkRequest new_manager
                             * @property {boolean|null} [validate_only] MoveManagerLinkRequest validate_only
                             */
    
                            /**
                             * Constructs a new MoveManagerLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MoveManagerLinkRequest.
                             * @implements IMoveManagerLinkRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkRequest=} [p] Properties to set
                             */
                            function MoveManagerLinkRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MoveManagerLinkRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @instance
                             */
                            MoveManagerLinkRequest.prototype.customer_id = "";
    
                            /**
                             * MoveManagerLinkRequest previous_customer_manager_link.
                             * @member {string} previous_customer_manager_link
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @instance
                             */
                            MoveManagerLinkRequest.prototype.previous_customer_manager_link = "";
    
                            /**
                             * MoveManagerLinkRequest new_manager.
                             * @member {string} new_manager
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @instance
                             */
                            MoveManagerLinkRequest.prototype.new_manager = "";
    
                            /**
                             * MoveManagerLinkRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @instance
                             */
                            MoveManagerLinkRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MoveManagerLinkRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkRequest} MoveManagerLinkRequest instance
                             */
                            MoveManagerLinkRequest.create = function create(properties) {
                                return new MoveManagerLinkRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MoveManagerLinkRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MoveManagerLinkRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkRequest} m MoveManagerLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MoveManagerLinkRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.previous_customer_manager_link != null && Object.hasOwnProperty.call(m, "previous_customer_manager_link"))
                                    w.uint32(18).string(m.previous_customer_manager_link);
                                if (m.new_manager != null && Object.hasOwnProperty.call(m, "new_manager"))
                                    w.uint32(26).string(m.new_manager);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(32).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MoveManagerLinkRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MoveManagerLinkRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkRequest} message MoveManagerLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MoveManagerLinkRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MoveManagerLinkRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkRequest} MoveManagerLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MoveManagerLinkRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MoveManagerLinkRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.previous_customer_manager_link = r.string();
                                        break;
                                    case 3:
                                        m.new_manager = r.string();
                                        break;
                                    case 4:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MoveManagerLinkRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkRequest} MoveManagerLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MoveManagerLinkRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MoveManagerLinkRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MoveManagerLinkRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.previous_customer_manager_link != null && m.hasOwnProperty("previous_customer_manager_link")) {
                                    if (!$util.isString(m.previous_customer_manager_link))
                                        return "previous_customer_manager_link: string expected";
                                }
                                if (m.new_manager != null && m.hasOwnProperty("new_manager")) {
                                    if (!$util.isString(m.new_manager))
                                        return "new_manager: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MoveManagerLinkRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkRequest} MoveManagerLinkRequest
                             */
                            MoveManagerLinkRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MoveManagerLinkRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MoveManagerLinkRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.previous_customer_manager_link != null) {
                                    m.previous_customer_manager_link = String(d.previous_customer_manager_link);
                                }
                                if (d.new_manager != null) {
                                    m.new_manager = String(d.new_manager);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MoveManagerLinkRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MoveManagerLinkRequest} m MoveManagerLinkRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MoveManagerLinkRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.previous_customer_manager_link = "";
                                    d.new_manager = "";
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.previous_customer_manager_link != null && m.hasOwnProperty("previous_customer_manager_link")) {
                                    d.previous_customer_manager_link = m.previous_customer_manager_link;
                                }
                                if (m.new_manager != null && m.hasOwnProperty("new_manager")) {
                                    d.new_manager = m.new_manager;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MoveManagerLinkRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MoveManagerLinkRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MoveManagerLinkRequest;
                        })();
    
                        services.CustomerManagerLinkOperation = (function() {
    
                            /**
                             * Properties of a CustomerManagerLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerManagerLinkOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerManagerLinkOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomerManagerLink|null} [update] CustomerManagerLinkOperation update
                             */
    
                            /**
                             * Constructs a new CustomerManagerLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerManagerLinkOperation.
                             * @implements ICustomerManagerLinkOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerManagerLinkOperation=} [p] Properties to set
                             */
                            function CustomerManagerLinkOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerManagerLinkOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @instance
                             */
                            CustomerManagerLinkOperation.prototype.update_mask = null;
    
                            /**
                             * CustomerManagerLinkOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomerManagerLink|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @instance
                             */
                            CustomerManagerLinkOperation.prototype.update = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerManagerLinkOperation operation.
                             * @member {"update"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerManagerLinkOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerManagerLinkOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerManagerLinkOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerManagerLinkOperation} CustomerManagerLinkOperation instance
                             */
                            CustomerManagerLinkOperation.create = function create(properties) {
                                return new CustomerManagerLinkOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerManagerLinkOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerManagerLinkOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerManagerLinkOperation} m CustomerManagerLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerManagerLinkOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomerManagerLink.encode(m.update, w.uint32(18).fork()).ldelim();
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerManagerLinkOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerManagerLinkOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerManagerLinkOperation} message CustomerManagerLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerManagerLinkOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerManagerLinkOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerManagerLinkOperation} CustomerManagerLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerManagerLinkOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerManagerLinkOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomerManagerLink.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerManagerLinkOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerManagerLinkOperation} CustomerManagerLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerManagerLinkOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerManagerLinkOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerManagerLinkOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerManagerLink.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerManagerLinkOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerManagerLinkOperation} CustomerManagerLinkOperation
                             */
                            CustomerManagerLinkOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerManagerLinkOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerManagerLinkOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerManagerLinkOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerManagerLinkOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomerManagerLink.fromObject(d.update);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerManagerLinkOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerManagerLinkOperation} m CustomerManagerLinkOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerManagerLinkOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomerManagerLink.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerManagerLinkOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerManagerLinkOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerManagerLinkOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerManagerLinkOperation;
                        })();
    
                        services.MutateCustomerManagerLinkResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerManagerLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerManagerLinkResponse
                             * @property {Array.<google.ads.googleads.v11.services.IMutateCustomerManagerLinkResult>|null} [results] MutateCustomerManagerLinkResponse results
                             */
    
                            /**
                             * Constructs a new MutateCustomerManagerLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerManagerLinkResponse.
                             * @implements IMutateCustomerManagerLinkResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResponse=} [p] Properties to set
                             */
                            function MutateCustomerManagerLinkResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerManagerLinkResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IMutateCustomerManagerLinkResult>} results
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @instance
                             */
                            MutateCustomerManagerLinkResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * Creates a new MutateCustomerManagerLinkResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse} MutateCustomerManagerLinkResponse instance
                             */
                            MutateCustomerManagerLinkResponse.create = function create(properties) {
                                return new MutateCustomerManagerLinkResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerManagerLinkResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResponse} m MutateCustomerManagerLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerManagerLinkResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerManagerLinkResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResponse} message MutateCustomerManagerLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerManagerLinkResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerManagerLinkResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse} MutateCustomerManagerLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerManagerLinkResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerManagerLinkResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse} MutateCustomerManagerLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerManagerLinkResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerManagerLinkResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerManagerLinkResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerManagerLinkResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse} MutateCustomerManagerLinkResponse
                             */
                            MutateCustomerManagerLinkResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult.fromObject(d.results[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerManagerLinkResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse} m MutateCustomerManagerLinkResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerManagerLinkResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult.toObject(m.results[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerManagerLinkResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerManagerLinkResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerManagerLinkResponse;
                        })();
    
                        services.MoveManagerLinkResponse = (function() {
    
                            /**
                             * Properties of a MoveManagerLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMoveManagerLinkResponse
                             * @property {string|null} [resource_name] MoveManagerLinkResponse resource_name
                             */
    
                            /**
                             * Constructs a new MoveManagerLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MoveManagerLinkResponse.
                             * @implements IMoveManagerLinkResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkResponse=} [p] Properties to set
                             */
                            function MoveManagerLinkResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MoveManagerLinkResponse resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @instance
                             */
                            MoveManagerLinkResponse.prototype.resource_name = "";
    
                            /**
                             * Creates a new MoveManagerLinkResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkResponse} MoveManagerLinkResponse instance
                             */
                            MoveManagerLinkResponse.create = function create(properties) {
                                return new MoveManagerLinkResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MoveManagerLinkResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MoveManagerLinkResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkResponse} m MoveManagerLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MoveManagerLinkResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MoveManagerLinkResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MoveManagerLinkResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMoveManagerLinkResponse} message MoveManagerLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MoveManagerLinkResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MoveManagerLinkResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkResponse} MoveManagerLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MoveManagerLinkResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MoveManagerLinkResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MoveManagerLinkResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkResponse} MoveManagerLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MoveManagerLinkResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MoveManagerLinkResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MoveManagerLinkResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MoveManagerLinkResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MoveManagerLinkResponse} MoveManagerLinkResponse
                             */
                            MoveManagerLinkResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MoveManagerLinkResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MoveManagerLinkResponse();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MoveManagerLinkResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MoveManagerLinkResponse} m MoveManagerLinkResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MoveManagerLinkResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MoveManagerLinkResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MoveManagerLinkResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MoveManagerLinkResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MoveManagerLinkResponse;
                        })();
    
                        services.MutateCustomerManagerLinkResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerManagerLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerManagerLinkResult
                             * @property {string|null} [resource_name] MutateCustomerManagerLinkResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomerManagerLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerManagerLinkResult.
                             * @implements IMutateCustomerManagerLinkResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResult=} [p] Properties to set
                             */
                            function MutateCustomerManagerLinkResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerManagerLinkResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @instance
                             */
                            MutateCustomerManagerLinkResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomerManagerLinkResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResult} MutateCustomerManagerLinkResult instance
                             */
                            MutateCustomerManagerLinkResult.create = function create(properties) {
                                return new MutateCustomerManagerLinkResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerManagerLinkResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerManagerLinkResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResult} m MutateCustomerManagerLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerManagerLinkResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerManagerLinkResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerManagerLinkResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerManagerLinkResult} message MutateCustomerManagerLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerManagerLinkResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerManagerLinkResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResult} MutateCustomerManagerLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerManagerLinkResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerManagerLinkResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResult} MutateCustomerManagerLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerManagerLinkResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerManagerLinkResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerManagerLinkResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerManagerLinkResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerManagerLinkResult} MutateCustomerManagerLinkResult
                             */
                            MutateCustomerManagerLinkResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerManagerLinkResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerManagerLinkResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerManagerLinkResult} m MutateCustomerManagerLinkResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerManagerLinkResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerManagerLinkResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerManagerLinkResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerManagerLinkResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerManagerLinkResult;
                        })();
    
                        services.CustomerUserAccessInvitationService = (function() {
    
                            /**
                             * Constructs a new CustomerUserAccessInvitationService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerUserAccessInvitationService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerUserAccessInvitationService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerUserAccessInvitationService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerUserAccessInvitationService;
    
                            /**
                             * Creates new CustomerUserAccessInvitationService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerUserAccessInvitationService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerUserAccessInvitationService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerUserAccessInvitationService|mutateCustomerUserAccessInvitation}.
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationService
                             * @typedef MutateCustomerUserAccessInvitationCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse} [response] MutateCustomerUserAccessInvitationResponse
                             */
    
                            /**
                             * Calls MutateCustomerUserAccessInvitation.
                             * @function mutateCustomerUserAccessInvitation
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationRequest} request MutateCustomerUserAccessInvitationRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerUserAccessInvitationService.MutateCustomerUserAccessInvitationCallback} callback Node-style callback called with the error, if any, and MutateCustomerUserAccessInvitationResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerUserAccessInvitationService.prototype.mutateCustomerUserAccessInvitation = function mutateCustomerUserAccessInvitation(request, callback) {
                                return this.rpcCall(mutateCustomerUserAccessInvitation, $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest, $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse, request, callback);
                            }, "name", { value: "MutateCustomerUserAccessInvitation" });
    
                            /**
                             * Calls MutateCustomerUserAccessInvitation.
                             * @function mutateCustomerUserAccessInvitation
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationRequest} request MutateCustomerUserAccessInvitationRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerUserAccessInvitationService;
                        })();
    
                        services.MutateCustomerUserAccessInvitationRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerUserAccessInvitationRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerUserAccessInvitationRequest
                             * @property {string|null} [customer_id] MutateCustomerUserAccessInvitationRequest customer_id
                             * @property {google.ads.googleads.v11.services.ICustomerUserAccessInvitationOperation|null} [operation] MutateCustomerUserAccessInvitationRequest operation
                             */
    
                            /**
                             * Constructs a new MutateCustomerUserAccessInvitationRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerUserAccessInvitationRequest.
                             * @implements IMutateCustomerUserAccessInvitationRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationRequest=} [p] Properties to set
                             */
                            function MutateCustomerUserAccessInvitationRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerUserAccessInvitationRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @instance
                             */
                            MutateCustomerUserAccessInvitationRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerUserAccessInvitationRequest operation.
                             * @member {google.ads.googleads.v11.services.ICustomerUserAccessInvitationOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @instance
                             */
                            MutateCustomerUserAccessInvitationRequest.prototype.operation = null;
    
                            /**
                             * Creates a new MutateCustomerUserAccessInvitationRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest} MutateCustomerUserAccessInvitationRequest instance
                             */
                            MutateCustomerUserAccessInvitationRequest.create = function create(properties) {
                                return new MutateCustomerUserAccessInvitationRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessInvitationRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationRequest} m MutateCustomerUserAccessInvitationRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessInvitationRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessInvitationRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationRequest} message MutateCustomerUserAccessInvitationRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessInvitationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessInvitationRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest} MutateCustomerUserAccessInvitationRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessInvitationRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessInvitationRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest} MutateCustomerUserAccessInvitationRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessInvitationRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerUserAccessInvitationRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerUserAccessInvitationRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerUserAccessInvitationRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest} MutateCustomerUserAccessInvitationRequest
                             */
                            MutateCustomerUserAccessInvitationRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.fromObject(d.operation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerUserAccessInvitationRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest} m MutateCustomerUserAccessInvitationRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerUserAccessInvitationRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.toObject(m.operation, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerUserAccessInvitationRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerUserAccessInvitationRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerUserAccessInvitationRequest;
                        })();
    
                        services.CustomerUserAccessInvitationOperation = (function() {
    
                            /**
                             * Properties of a CustomerUserAccessInvitationOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerUserAccessInvitationOperation
                             * @property {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation|null} [create] CustomerUserAccessInvitationOperation create
                             * @property {string|null} [remove] CustomerUserAccessInvitationOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerUserAccessInvitationOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerUserAccessInvitationOperation.
                             * @implements ICustomerUserAccessInvitationOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessInvitationOperation=} [p] Properties to set
                             */
                            function CustomerUserAccessInvitationOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerUserAccessInvitationOperation create.
                             * @member {google.ads.googleads.v11.resources.ICustomerUserAccessInvitation|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @instance
                             */
                            CustomerUserAccessInvitationOperation.prototype.create = null;
    
                            /**
                             * CustomerUserAccessInvitationOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @instance
                             */
                            CustomerUserAccessInvitationOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerUserAccessInvitationOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerUserAccessInvitationOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerUserAccessInvitationOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessInvitationOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation} CustomerUserAccessInvitationOperation instance
                             */
                            CustomerUserAccessInvitationOperation.create = function create(properties) {
                                return new CustomerUserAccessInvitationOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessInvitationOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessInvitationOperation} m CustomerUserAccessInvitationOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessInvitationOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessInvitationOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessInvitationOperation} message CustomerUserAccessInvitationOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessInvitationOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerUserAccessInvitationOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation} CustomerUserAccessInvitationOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessInvitationOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerUserAccessInvitationOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation} CustomerUserAccessInvitationOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessInvitationOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerUserAccessInvitationOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerUserAccessInvitationOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerUserAccessInvitationOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation} CustomerUserAccessInvitationOperation
                             */
                            CustomerUserAccessInvitationOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerUserAccessInvitationOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation} m CustomerUserAccessInvitationOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerUserAccessInvitationOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.resources.CustomerUserAccessInvitation.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerUserAccessInvitationOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessInvitationOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerUserAccessInvitationOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerUserAccessInvitationOperation;
                        })();
    
                        services.MutateCustomerUserAccessInvitationResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerUserAccessInvitationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerUserAccessInvitationResponse
                             * @property {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResult|null} [result] MutateCustomerUserAccessInvitationResponse result
                             */
    
                            /**
                             * Constructs a new MutateCustomerUserAccessInvitationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerUserAccessInvitationResponse.
                             * @implements IMutateCustomerUserAccessInvitationResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResponse=} [p] Properties to set
                             */
                            function MutateCustomerUserAccessInvitationResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerUserAccessInvitationResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @instance
                             */
                            MutateCustomerUserAccessInvitationResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateCustomerUserAccessInvitationResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse} MutateCustomerUserAccessInvitationResponse instance
                             */
                            MutateCustomerUserAccessInvitationResponse.create = function create(properties) {
                                return new MutateCustomerUserAccessInvitationResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessInvitationResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResponse} m MutateCustomerUserAccessInvitationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessInvitationResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult.encode(m.result, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessInvitationResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResponse} message MutateCustomerUserAccessInvitationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessInvitationResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessInvitationResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse} MutateCustomerUserAccessInvitationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessInvitationResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.result = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessInvitationResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse} MutateCustomerUserAccessInvitationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessInvitationResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerUserAccessInvitationResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerUserAccessInvitationResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerUserAccessInvitationResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse} MutateCustomerUserAccessInvitationResponse
                             */
                            MutateCustomerUserAccessInvitationResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerUserAccessInvitationResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse} m MutateCustomerUserAccessInvitationResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerUserAccessInvitationResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerUserAccessInvitationResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerUserAccessInvitationResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerUserAccessInvitationResponse;
                        })();
    
                        services.MutateCustomerUserAccessInvitationResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerUserAccessInvitationResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerUserAccessInvitationResult
                             * @property {string|null} [resource_name] MutateCustomerUserAccessInvitationResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomerUserAccessInvitationResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerUserAccessInvitationResult.
                             * @implements IMutateCustomerUserAccessInvitationResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResult=} [p] Properties to set
                             */
                            function MutateCustomerUserAccessInvitationResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerUserAccessInvitationResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @instance
                             */
                            MutateCustomerUserAccessInvitationResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomerUserAccessInvitationResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult} MutateCustomerUserAccessInvitationResult instance
                             */
                            MutateCustomerUserAccessInvitationResult.create = function create(properties) {
                                return new MutateCustomerUserAccessInvitationResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessInvitationResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResult} m MutateCustomerUserAccessInvitationResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessInvitationResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessInvitationResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessInvitationResult} message MutateCustomerUserAccessInvitationResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessInvitationResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessInvitationResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult} MutateCustomerUserAccessInvitationResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessInvitationResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessInvitationResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult} MutateCustomerUserAccessInvitationResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessInvitationResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerUserAccessInvitationResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerUserAccessInvitationResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerUserAccessInvitationResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult} MutateCustomerUserAccessInvitationResult
                             */
                            MutateCustomerUserAccessInvitationResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerUserAccessInvitationResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult} m MutateCustomerUserAccessInvitationResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerUserAccessInvitationResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerUserAccessInvitationResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessInvitationResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerUserAccessInvitationResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerUserAccessInvitationResult;
                        })();
    
                        services.CustomerUserAccessService = (function() {
    
                            /**
                             * Constructs a new CustomerUserAccessService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerUserAccessService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function CustomerUserAccessService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (CustomerUserAccessService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = CustomerUserAccessService;
    
                            /**
                             * Creates new CustomerUserAccessService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {CustomerUserAccessService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            CustomerUserAccessService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.CustomerUserAccessService|mutateCustomerUserAccess}.
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessService
                             * @typedef MutateCustomerUserAccessCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessResponse} [response] MutateCustomerUserAccessResponse
                             */
    
                            /**
                             * Calls MutateCustomerUserAccess.
                             * @function mutateCustomerUserAccess
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessRequest} request MutateCustomerUserAccessRequest message or plain object
                             * @param {google.ads.googleads.v11.services.CustomerUserAccessService.MutateCustomerUserAccessCallback} callback Node-style callback called with the error, if any, and MutateCustomerUserAccessResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(CustomerUserAccessService.prototype.mutateCustomerUserAccess = function mutateCustomerUserAccess(request, callback) {
                                return this.rpcCall(mutateCustomerUserAccess, $root.google.ads.googleads.v11.services.MutateCustomerUserAccessRequest, $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResponse, request, callback);
                            }, "name", { value: "MutateCustomerUserAccess" });
    
                            /**
                             * Calls MutateCustomerUserAccess.
                             * @function mutateCustomerUserAccess
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessRequest} request MutateCustomerUserAccessRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateCustomerUserAccessResponse>} Promise
                             * @variation 2
                             */
    
                            return CustomerUserAccessService;
                        })();
    
                        services.MutateCustomerUserAccessRequest = (function() {
    
                            /**
                             * Properties of a MutateCustomerUserAccessRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerUserAccessRequest
                             * @property {string|null} [customer_id] MutateCustomerUserAccessRequest customer_id
                             * @property {google.ads.googleads.v11.services.ICustomerUserAccessOperation|null} [operation] MutateCustomerUserAccessRequest operation
                             */
    
                            /**
                             * Constructs a new MutateCustomerUserAccessRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerUserAccessRequest.
                             * @implements IMutateCustomerUserAccessRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessRequest=} [p] Properties to set
                             */
                            function MutateCustomerUserAccessRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerUserAccessRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @instance
                             */
                            MutateCustomerUserAccessRequest.prototype.customer_id = "";
    
                            /**
                             * MutateCustomerUserAccessRequest operation.
                             * @member {google.ads.googleads.v11.services.ICustomerUserAccessOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @instance
                             */
                            MutateCustomerUserAccessRequest.prototype.operation = null;
    
                            /**
                             * Creates a new MutateCustomerUserAccessRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessRequest} MutateCustomerUserAccessRequest instance
                             */
                            MutateCustomerUserAccessRequest.create = function create(properties) {
                                return new MutateCustomerUserAccessRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessRequest} m MutateCustomerUserAccessRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.CustomerUserAccessOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessRequest} message MutateCustomerUserAccessRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessRequest} MutateCustomerUserAccessRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.CustomerUserAccessOperation.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessRequest} MutateCustomerUserAccessRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerUserAccessRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerUserAccessRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CustomerUserAccessOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerUserAccessRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessRequest} MutateCustomerUserAccessRequest
                             */
                            MutateCustomerUserAccessRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerUserAccessRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerUserAccessRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.CustomerUserAccessOperation.fromObject(d.operation);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerUserAccessRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessRequest} m MutateCustomerUserAccessRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerUserAccessRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.CustomerUserAccessOperation.toObject(m.operation, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerUserAccessRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerUserAccessRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerUserAccessRequest;
                        })();
    
                        services.CustomerUserAccessOperation = (function() {
    
                            /**
                             * Properties of a CustomerUserAccessOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICustomerUserAccessOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] CustomerUserAccessOperation update_mask
                             * @property {google.ads.googleads.v11.resources.ICustomerUserAccess|null} [update] CustomerUserAccessOperation update
                             * @property {string|null} [remove] CustomerUserAccessOperation remove
                             */
    
                            /**
                             * Constructs a new CustomerUserAccessOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CustomerUserAccessOperation.
                             * @implements ICustomerUserAccessOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessOperation=} [p] Properties to set
                             */
                            function CustomerUserAccessOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CustomerUserAccessOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @instance
                             */
                            CustomerUserAccessOperation.prototype.update_mask = null;
    
                            /**
                             * CustomerUserAccessOperation update.
                             * @member {google.ads.googleads.v11.resources.ICustomerUserAccess|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @instance
                             */
                            CustomerUserAccessOperation.prototype.update = null;
    
                            /**
                             * CustomerUserAccessOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @instance
                             */
                            CustomerUserAccessOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CustomerUserAccessOperation operation.
                             * @member {"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @instance
                             */
                            Object.defineProperty(CustomerUserAccessOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CustomerUserAccessOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessOperation} CustomerUserAccessOperation instance
                             */
                            CustomerUserAccessOperation.create = function create(properties) {
                                return new CustomerUserAccessOperation(properties);
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessOperation message. Does not implicitly {@link google.ads.googleads.v11.services.CustomerUserAccessOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessOperation} m CustomerUserAccessOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.CustomerUserAccess.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CustomerUserAccessOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CustomerUserAccessOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ICustomerUserAccessOperation} message CustomerUserAccessOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CustomerUserAccessOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CustomerUserAccessOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessOperation} CustomerUserAccessOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CustomerUserAccessOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.CustomerUserAccess.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CustomerUserAccessOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessOperation} CustomerUserAccessOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CustomerUserAccessOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CustomerUserAccessOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CustomerUserAccessOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.CustomerUserAccess.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CustomerUserAccessOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CustomerUserAccessOperation} CustomerUserAccessOperation
                             */
                            CustomerUserAccessOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CustomerUserAccessOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CustomerUserAccessOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerUserAccessOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CustomerUserAccessOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.CustomerUserAccess.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CustomerUserAccessOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.CustomerUserAccessOperation} m CustomerUserAccessOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CustomerUserAccessOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.CustomerUserAccess.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CustomerUserAccessOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CustomerUserAccessOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CustomerUserAccessOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CustomerUserAccessOperation;
                        })();
    
                        services.MutateCustomerUserAccessResponse = (function() {
    
                            /**
                             * Properties of a MutateCustomerUserAccessResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerUserAccessResponse
                             * @property {google.ads.googleads.v11.services.IMutateCustomerUserAccessResult|null} [result] MutateCustomerUserAccessResponse result
                             */
    
                            /**
                             * Constructs a new MutateCustomerUserAccessResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerUserAccessResponse.
                             * @implements IMutateCustomerUserAccessResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResponse=} [p] Properties to set
                             */
                            function MutateCustomerUserAccessResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerUserAccessResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateCustomerUserAccessResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @instance
                             */
                            MutateCustomerUserAccessResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateCustomerUserAccessResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResponse} MutateCustomerUserAccessResponse instance
                             */
                            MutateCustomerUserAccessResponse.create = function create(properties) {
                                return new MutateCustomerUserAccessResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResponse} m MutateCustomerUserAccessResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult.encode(m.result, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResponse} message MutateCustomerUserAccessResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResponse} MutateCustomerUserAccessResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.result = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResponse} MutateCustomerUserAccessResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerUserAccessResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerUserAccessResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerUserAccessResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResponse} MutateCustomerUserAccessResponse
                             */
                            MutateCustomerUserAccessResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateCustomerUserAccessResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerUserAccessResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessResponse} m MutateCustomerUserAccessResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerUserAccessResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerUserAccessResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerUserAccessResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerUserAccessResponse;
                        })();
    
                        services.MutateCustomerUserAccessResult = (function() {
    
                            /**
                             * Properties of a MutateCustomerUserAccessResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateCustomerUserAccessResult
                             * @property {string|null} [resource_name] MutateCustomerUserAccessResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateCustomerUserAccessResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateCustomerUserAccessResult.
                             * @implements IMutateCustomerUserAccessResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResult=} [p] Properties to set
                             */
                            function MutateCustomerUserAccessResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateCustomerUserAccessResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @instance
                             */
                            MutateCustomerUserAccessResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateCustomerUserAccessResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResult} MutateCustomerUserAccessResult instance
                             */
                            MutateCustomerUserAccessResult.create = function create(properties) {
                                return new MutateCustomerUserAccessResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResult} m MutateCustomerUserAccessResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateCustomerUserAccessResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateCustomerUserAccessResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateCustomerUserAccessResult} message MutateCustomerUserAccessResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateCustomerUserAccessResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResult} MutateCustomerUserAccessResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateCustomerUserAccessResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResult} MutateCustomerUserAccessResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateCustomerUserAccessResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateCustomerUserAccessResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateCustomerUserAccessResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateCustomerUserAccessResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateCustomerUserAccessResult} MutateCustomerUserAccessResult
                             */
                            MutateCustomerUserAccessResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateCustomerUserAccessResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateCustomerUserAccessResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateCustomerUserAccessResult} m MutateCustomerUserAccessResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateCustomerUserAccessResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateCustomerUserAccessResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateCustomerUserAccessResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateCustomerUserAccessResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateCustomerUserAccessResult;
                        })();
    
                        services.GeoTargetConstantService = (function() {
    
                            /**
                             * Constructs a new GeoTargetConstantService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GeoTargetConstantService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function GeoTargetConstantService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (GeoTargetConstantService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = GeoTargetConstantService;
    
                            /**
                             * Creates new GeoTargetConstantService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {GeoTargetConstantService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            GeoTargetConstantService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.GeoTargetConstantService|suggestGeoTargetConstants}.
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantService
                             * @typedef SuggestGeoTargetConstantsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse} [response] SuggestGeoTargetConstantsResponse
                             */
    
                            /**
                             * Calls SuggestGeoTargetConstants.
                             * @function suggestGeoTargetConstants
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsRequest} request SuggestGeoTargetConstantsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.GeoTargetConstantService.SuggestGeoTargetConstantsCallback} callback Node-style callback called with the error, if any, and SuggestGeoTargetConstantsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(GeoTargetConstantService.prototype.suggestGeoTargetConstants = function suggestGeoTargetConstants(request, callback) {
                                return this.rpcCall(suggestGeoTargetConstants, $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest, $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse, request, callback);
                            }, "name", { value: "SuggestGeoTargetConstants" });
    
                            /**
                             * Calls SuggestGeoTargetConstants.
                             * @function suggestGeoTargetConstants
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsRequest} request SuggestGeoTargetConstantsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse>} Promise
                             * @variation 2
                             */
    
                            return GeoTargetConstantService;
                        })();
    
                        services.SuggestGeoTargetConstantsRequest = (function() {
    
                            /**
                             * Properties of a SuggestGeoTargetConstantsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestGeoTargetConstantsRequest
                             * @property {string|null} [locale] SuggestGeoTargetConstantsRequest locale
                             * @property {string|null} [country_code] SuggestGeoTargetConstantsRequest country_code
                             * @property {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.ILocationNames|null} [location_names] SuggestGeoTargetConstantsRequest location_names
                             * @property {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.IGeoTargets|null} [geo_targets] SuggestGeoTargetConstantsRequest geo_targets
                             */
    
                            /**
                             * Constructs a new SuggestGeoTargetConstantsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestGeoTargetConstantsRequest.
                             * @implements ISuggestGeoTargetConstantsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsRequest=} [p] Properties to set
                             */
                            function SuggestGeoTargetConstantsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestGeoTargetConstantsRequest locale.
                             * @member {string|null|undefined} locale
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             */
                            SuggestGeoTargetConstantsRequest.prototype.locale = null;
    
                            /**
                             * SuggestGeoTargetConstantsRequest country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             */
                            SuggestGeoTargetConstantsRequest.prototype.country_code = null;
    
                            /**
                             * SuggestGeoTargetConstantsRequest location_names.
                             * @member {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.ILocationNames|null|undefined} location_names
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             */
                            SuggestGeoTargetConstantsRequest.prototype.location_names = null;
    
                            /**
                             * SuggestGeoTargetConstantsRequest geo_targets.
                             * @member {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.IGeoTargets|null|undefined} geo_targets
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             */
                            SuggestGeoTargetConstantsRequest.prototype.geo_targets = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SuggestGeoTargetConstantsRequest _locale.
                             * @member {"locale"|undefined} _locale
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             */
                            Object.defineProperty(SuggestGeoTargetConstantsRequest.prototype, "_locale", {
                                get: $util.oneOfGetter($oneOfFields = ["locale"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SuggestGeoTargetConstantsRequest _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             */
                            Object.defineProperty(SuggestGeoTargetConstantsRequest.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SuggestGeoTargetConstantsRequest query.
                             * @member {"location_names"|"geo_targets"|undefined} query
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             */
                            Object.defineProperty(SuggestGeoTargetConstantsRequest.prototype, "query", {
                                get: $util.oneOfGetter($oneOfFields = ["location_names", "geo_targets"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SuggestGeoTargetConstantsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest} SuggestGeoTargetConstantsRequest instance
                             */
                            SuggestGeoTargetConstantsRequest.create = function create(properties) {
                                return new SuggestGeoTargetConstantsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestGeoTargetConstantsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsRequest} m SuggestGeoTargetConstantsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestGeoTargetConstantsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.location_names != null && Object.hasOwnProperty.call(m, "location_names"))
                                    $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.encode(m.location_names, w.uint32(10).fork()).ldelim();
                                if (m.geo_targets != null && Object.hasOwnProperty.call(m, "geo_targets"))
                                    $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.encode(m.geo_targets, w.uint32(18).fork()).ldelim();
                                if (m.locale != null && Object.hasOwnProperty.call(m, "locale"))
                                    w.uint32(50).string(m.locale);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(58).string(m.country_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestGeoTargetConstantsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsRequest} message SuggestGeoTargetConstantsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestGeoTargetConstantsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestGeoTargetConstantsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest} SuggestGeoTargetConstantsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestGeoTargetConstantsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.locale = r.string();
                                        break;
                                    case 7:
                                        m.country_code = r.string();
                                        break;
                                    case 1:
                                        m.location_names = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.geo_targets = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestGeoTargetConstantsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest} SuggestGeoTargetConstantsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestGeoTargetConstantsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestGeoTargetConstantsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestGeoTargetConstantsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.locale != null && m.hasOwnProperty("locale")) {
                                    p._locale = 1;
                                    if (!$util.isString(m.locale))
                                        return "locale: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.location_names != null && m.hasOwnProperty("location_names")) {
                                    p.query = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.verify(m.location_names);
                                        if (e)
                                            return "location_names." + e;
                                    }
                                }
                                if (m.geo_targets != null && m.hasOwnProperty("geo_targets")) {
                                    if (p.query === 1)
                                        return "query: multiple values";
                                    p.query = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.verify(m.geo_targets);
                                        if (e)
                                            return "geo_targets." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestGeoTargetConstantsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest} SuggestGeoTargetConstantsRequest
                             */
                            SuggestGeoTargetConstantsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest();
                                if (d.locale != null) {
                                    m.locale = String(d.locale);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.location_names != null) {
                                    if (typeof d.location_names !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.location_names: object expected");
                                    m.location_names = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.fromObject(d.location_names);
                                }
                                if (d.geo_targets != null) {
                                    if (typeof d.geo_targets !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.geo_targets: object expected");
                                    m.geo_targets = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.fromObject(d.geo_targets);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestGeoTargetConstantsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest} m SuggestGeoTargetConstantsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestGeoTargetConstantsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.location_names != null && m.hasOwnProperty("location_names")) {
                                    d.location_names = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.toObject(m.location_names, o);
                                    if (o.oneofs)
                                        d.query = "location_names";
                                }
                                if (m.geo_targets != null && m.hasOwnProperty("geo_targets")) {
                                    d.geo_targets = $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.toObject(m.geo_targets, o);
                                    if (o.oneofs)
                                        d.query = "geo_targets";
                                }
                                if (m.locale != null && m.hasOwnProperty("locale")) {
                                    d.locale = m.locale;
                                    if (o.oneofs)
                                        d._locale = "locale";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestGeoTargetConstantsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestGeoTargetConstantsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            SuggestGeoTargetConstantsRequest.LocationNames = (function() {
    
                                /**
                                 * Properties of a LocationNames.
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                                 * @interface ILocationNames
                                 * @property {Array.<string>|null} [names] LocationNames names
                                 */
    
                                /**
                                 * Constructs a new LocationNames.
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                                 * @classdesc Represents a LocationNames.
                                 * @implements ILocationNames
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.ILocationNames=} [p] Properties to set
                                 */
                                function LocationNames(p) {
                                    this.names = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * LocationNames names.
                                 * @member {Array.<string>} names
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @instance
                                 */
                                LocationNames.prototype.names = $util.emptyArray;
    
                                /**
                                 * Creates a new LocationNames instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.ILocationNames=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames} LocationNames instance
                                 */
                                LocationNames.create = function create(properties) {
                                    return new LocationNames(properties);
                                };
    
                                /**
                                 * Encodes the specified LocationNames message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.ILocationNames} m LocationNames message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocationNames.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.names != null && m.names.length) {
                                        for (var i = 0; i < m.names.length; ++i)
                                            w.uint32(18).string(m.names[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified LocationNames message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.ILocationNames} message LocationNames message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocationNames.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a LocationNames message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames} LocationNames
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocationNames.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            if (!(m.names && m.names.length))
                                                m.names = [];
                                            m.names.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a LocationNames message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames} LocationNames
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocationNames.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a LocationNames message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                LocationNames.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.names != null && m.hasOwnProperty("names")) {
                                        if (!Array.isArray(m.names))
                                            return "names: array expected";
                                        for (var i = 0; i < m.names.length; ++i) {
                                            if (!$util.isString(m.names[i]))
                                                return "names: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a LocationNames message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames} LocationNames
                                 */
                                LocationNames.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames();
                                    if (d.names) {
                                        if (!Array.isArray(d.names))
                                            throw TypeError(".google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames.names: array expected");
                                        m.names = [];
                                        for (var i = 0; i < d.names.length; ++i) {
                                            m.names[i] = String(d.names[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a LocationNames message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames} m LocationNames
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                LocationNames.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.names = [];
                                    }
                                    if (m.names && m.names.length) {
                                        d.names = [];
                                        for (var j = 0; j < m.names.length; ++j) {
                                            d.names[j] = m.names[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this LocationNames to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.LocationNames
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                LocationNames.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return LocationNames;
                            })();
    
                            SuggestGeoTargetConstantsRequest.GeoTargets = (function() {
    
                                /**
                                 * Properties of a GeoTargets.
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                                 * @interface IGeoTargets
                                 * @property {Array.<string>|null} [geo_target_constants] GeoTargets geo_target_constants
                                 */
    
                                /**
                                 * Constructs a new GeoTargets.
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest
                                 * @classdesc Represents a GeoTargets.
                                 * @implements IGeoTargets
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.IGeoTargets=} [p] Properties to set
                                 */
                                function GeoTargets(p) {
                                    this.geo_target_constants = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * GeoTargets geo_target_constants.
                                 * @member {Array.<string>} geo_target_constants
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @instance
                                 */
                                GeoTargets.prototype.geo_target_constants = $util.emptyArray;
    
                                /**
                                 * Creates a new GeoTargets instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.IGeoTargets=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets} GeoTargets instance
                                 */
                                GeoTargets.create = function create(properties) {
                                    return new GeoTargets(properties);
                                };
    
                                /**
                                 * Encodes the specified GeoTargets message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.IGeoTargets} m GeoTargets message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                GeoTargets.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.geo_target_constants != null && m.geo_target_constants.length) {
                                        for (var i = 0; i < m.geo_target_constants.length; ++i)
                                            w.uint32(18).string(m.geo_target_constants[i]);
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified GeoTargets message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.IGeoTargets} message GeoTargets message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                GeoTargets.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a GeoTargets message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets} GeoTargets
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                GeoTargets.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            if (!(m.geo_target_constants && m.geo_target_constants.length))
                                                m.geo_target_constants = [];
                                            m.geo_target_constants.push(r.string());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a GeoTargets message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets} GeoTargets
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                GeoTargets.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a GeoTargets message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                GeoTargets.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.geo_target_constants != null && m.hasOwnProperty("geo_target_constants")) {
                                        if (!Array.isArray(m.geo_target_constants))
                                            return "geo_target_constants: array expected";
                                        for (var i = 0; i < m.geo_target_constants.length; ++i) {
                                            if (!$util.isString(m.geo_target_constants[i]))
                                                return "geo_target_constants: string[] expected";
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a GeoTargets message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets} GeoTargets
                                 */
                                GeoTargets.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets();
                                    if (d.geo_target_constants) {
                                        if (!Array.isArray(d.geo_target_constants))
                                            throw TypeError(".google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets.geo_target_constants: array expected");
                                        m.geo_target_constants = [];
                                        for (var i = 0; i < d.geo_target_constants.length; ++i) {
                                            m.geo_target_constants[i] = String(d.geo_target_constants[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a GeoTargets message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets} m GeoTargets
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                GeoTargets.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.geo_target_constants = [];
                                    }
                                    if (m.geo_target_constants && m.geo_target_constants.length) {
                                        d.geo_target_constants = [];
                                        for (var j = 0; j < m.geo_target_constants.length; ++j) {
                                            d.geo_target_constants[j] = m.geo_target_constants[j];
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this GeoTargets to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsRequest.GeoTargets
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                GeoTargets.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return GeoTargets;
                            })();
    
                            return SuggestGeoTargetConstantsRequest;
                        })();
    
                        services.SuggestGeoTargetConstantsResponse = (function() {
    
                            /**
                             * Properties of a SuggestGeoTargetConstantsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestGeoTargetConstantsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IGeoTargetConstantSuggestion>|null} [geo_target_constant_suggestions] SuggestGeoTargetConstantsResponse geo_target_constant_suggestions
                             */
    
                            /**
                             * Constructs a new SuggestGeoTargetConstantsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestGeoTargetConstantsResponse.
                             * @implements ISuggestGeoTargetConstantsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsResponse=} [p] Properties to set
                             */
                            function SuggestGeoTargetConstantsResponse(p) {
                                this.geo_target_constant_suggestions = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestGeoTargetConstantsResponse geo_target_constant_suggestions.
                             * @member {Array.<google.ads.googleads.v11.services.IGeoTargetConstantSuggestion>} geo_target_constant_suggestions
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @instance
                             */
                            SuggestGeoTargetConstantsResponse.prototype.geo_target_constant_suggestions = $util.emptyArray;
    
                            /**
                             * Creates a new SuggestGeoTargetConstantsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse} SuggestGeoTargetConstantsResponse instance
                             */
                            SuggestGeoTargetConstantsResponse.create = function create(properties) {
                                return new SuggestGeoTargetConstantsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestGeoTargetConstantsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsResponse} m SuggestGeoTargetConstantsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestGeoTargetConstantsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.geo_target_constant_suggestions != null && m.geo_target_constant_suggestions.length) {
                                    for (var i = 0; i < m.geo_target_constant_suggestions.length; ++i)
                                        $root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion.encode(m.geo_target_constant_suggestions[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestGeoTargetConstantsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestGeoTargetConstantsResponse} message SuggestGeoTargetConstantsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestGeoTargetConstantsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestGeoTargetConstantsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse} SuggestGeoTargetConstantsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestGeoTargetConstantsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.geo_target_constant_suggestions && m.geo_target_constant_suggestions.length))
                                            m.geo_target_constant_suggestions = [];
                                        m.geo_target_constant_suggestions.push($root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestGeoTargetConstantsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse} SuggestGeoTargetConstantsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestGeoTargetConstantsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestGeoTargetConstantsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestGeoTargetConstantsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.geo_target_constant_suggestions != null && m.hasOwnProperty("geo_target_constant_suggestions")) {
                                    if (!Array.isArray(m.geo_target_constant_suggestions))
                                        return "geo_target_constant_suggestions: array expected";
                                    for (var i = 0; i < m.geo_target_constant_suggestions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion.verify(m.geo_target_constant_suggestions[i]);
                                            if (e)
                                                return "geo_target_constant_suggestions." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestGeoTargetConstantsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse} SuggestGeoTargetConstantsResponse
                             */
                            SuggestGeoTargetConstantsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse();
                                if (d.geo_target_constant_suggestions) {
                                    if (!Array.isArray(d.geo_target_constant_suggestions))
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse.geo_target_constant_suggestions: array expected");
                                    m.geo_target_constant_suggestions = [];
                                    for (var i = 0; i < d.geo_target_constant_suggestions.length; ++i) {
                                        if (typeof d.geo_target_constant_suggestions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse.geo_target_constant_suggestions: object expected");
                                        m.geo_target_constant_suggestions[i] = $root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion.fromObject(d.geo_target_constant_suggestions[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestGeoTargetConstantsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse} m SuggestGeoTargetConstantsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestGeoTargetConstantsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.geo_target_constant_suggestions = [];
                                }
                                if (m.geo_target_constant_suggestions && m.geo_target_constant_suggestions.length) {
                                    d.geo_target_constant_suggestions = [];
                                    for (var j = 0; j < m.geo_target_constant_suggestions.length; ++j) {
                                        d.geo_target_constant_suggestions[j] = $root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion.toObject(m.geo_target_constant_suggestions[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestGeoTargetConstantsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestGeoTargetConstantsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestGeoTargetConstantsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestGeoTargetConstantsResponse;
                        })();
    
                        services.GeoTargetConstantSuggestion = (function() {
    
                            /**
                             * Properties of a GeoTargetConstantSuggestion.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGeoTargetConstantSuggestion
                             * @property {string|null} [locale] GeoTargetConstantSuggestion locale
                             * @property {number|null} [reach] GeoTargetConstantSuggestion reach
                             * @property {string|null} [search_term] GeoTargetConstantSuggestion search_term
                             * @property {google.ads.googleads.v11.resources.IGeoTargetConstant|null} [geo_target_constant] GeoTargetConstantSuggestion geo_target_constant
                             * @property {Array.<google.ads.googleads.v11.resources.IGeoTargetConstant>|null} [geo_target_constant_parents] GeoTargetConstantSuggestion geo_target_constant_parents
                             */
    
                            /**
                             * Constructs a new GeoTargetConstantSuggestion.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GeoTargetConstantSuggestion.
                             * @implements IGeoTargetConstantSuggestion
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGeoTargetConstantSuggestion=} [p] Properties to set
                             */
                            function GeoTargetConstantSuggestion(p) {
                                this.geo_target_constant_parents = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GeoTargetConstantSuggestion locale.
                             * @member {string|null|undefined} locale
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            GeoTargetConstantSuggestion.prototype.locale = null;
    
                            /**
                             * GeoTargetConstantSuggestion reach.
                             * @member {number|null|undefined} reach
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            GeoTargetConstantSuggestion.prototype.reach = null;
    
                            /**
                             * GeoTargetConstantSuggestion search_term.
                             * @member {string|null|undefined} search_term
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            GeoTargetConstantSuggestion.prototype.search_term = null;
    
                            /**
                             * GeoTargetConstantSuggestion geo_target_constant.
                             * @member {google.ads.googleads.v11.resources.IGeoTargetConstant|null|undefined} geo_target_constant
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            GeoTargetConstantSuggestion.prototype.geo_target_constant = null;
    
                            /**
                             * GeoTargetConstantSuggestion geo_target_constant_parents.
                             * @member {Array.<google.ads.googleads.v11.resources.IGeoTargetConstant>} geo_target_constant_parents
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            GeoTargetConstantSuggestion.prototype.geo_target_constant_parents = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GeoTargetConstantSuggestion _locale.
                             * @member {"locale"|undefined} _locale
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstantSuggestion.prototype, "_locale", {
                                get: $util.oneOfGetter($oneOfFields = ["locale"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoTargetConstantSuggestion _reach.
                             * @member {"reach"|undefined} _reach
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstantSuggestion.prototype, "_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GeoTargetConstantSuggestion _search_term.
                             * @member {"search_term"|undefined} _search_term
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             */
                            Object.defineProperty(GeoTargetConstantSuggestion.prototype, "_search_term", {
                                get: $util.oneOfGetter($oneOfFields = ["search_term"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GeoTargetConstantSuggestion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.IGeoTargetConstantSuggestion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GeoTargetConstantSuggestion} GeoTargetConstantSuggestion instance
                             */
                            GeoTargetConstantSuggestion.create = function create(properties) {
                                return new GeoTargetConstantSuggestion(properties);
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstantSuggestion message. Does not implicitly {@link google.ads.googleads.v11.services.GeoTargetConstantSuggestion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.IGeoTargetConstantSuggestion} m GeoTargetConstantSuggestion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstantSuggestion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.geo_target_constant != null && Object.hasOwnProperty.call(m, "geo_target_constant"))
                                    $root.google.ads.googleads.v11.resources.GeoTargetConstant.encode(m.geo_target_constant, w.uint32(34).fork()).ldelim();
                                if (m.geo_target_constant_parents != null && m.geo_target_constant_parents.length) {
                                    for (var i = 0; i < m.geo_target_constant_parents.length; ++i)
                                        $root.google.ads.googleads.v11.resources.GeoTargetConstant.encode(m.geo_target_constant_parents[i], w.uint32(42).fork()).ldelim();
                                }
                                if (m.locale != null && Object.hasOwnProperty.call(m, "locale"))
                                    w.uint32(50).string(m.locale);
                                if (m.reach != null && Object.hasOwnProperty.call(m, "reach"))
                                    w.uint32(56).int64(m.reach);
                                if (m.search_term != null && Object.hasOwnProperty.call(m, "search_term"))
                                    w.uint32(66).string(m.search_term);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GeoTargetConstantSuggestion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GeoTargetConstantSuggestion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.IGeoTargetConstantSuggestion} message GeoTargetConstantSuggestion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GeoTargetConstantSuggestion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GeoTargetConstantSuggestion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GeoTargetConstantSuggestion} GeoTargetConstantSuggestion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstantSuggestion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.locale = r.string();
                                        break;
                                    case 7:
                                        m.reach = r.int64();
                                        break;
                                    case 8:
                                        m.search_term = r.string();
                                        break;
                                    case 4:
                                        m.geo_target_constant = $root.google.ads.googleads.v11.resources.GeoTargetConstant.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        if (!(m.geo_target_constant_parents && m.geo_target_constant_parents.length))
                                            m.geo_target_constant_parents = [];
                                        m.geo_target_constant_parents.push($root.google.ads.googleads.v11.resources.GeoTargetConstant.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GeoTargetConstantSuggestion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GeoTargetConstantSuggestion} GeoTargetConstantSuggestion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GeoTargetConstantSuggestion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GeoTargetConstantSuggestion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GeoTargetConstantSuggestion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.locale != null && m.hasOwnProperty("locale")) {
                                    p._locale = 1;
                                    if (!$util.isString(m.locale))
                                        return "locale: string expected";
                                }
                                if (m.reach != null && m.hasOwnProperty("reach")) {
                                    p._reach = 1;
                                    if (!$util.isInteger(m.reach) && !(m.reach && $util.isInteger(m.reach.low) && $util.isInteger(m.reach.high)))
                                        return "reach: integer|Long expected";
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    p._search_term = 1;
                                    if (!$util.isString(m.search_term))
                                        return "search_term: string expected";
                                }
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.GeoTargetConstant.verify(m.geo_target_constant);
                                        if (e)
                                            return "geo_target_constant." + e;
                                    }
                                }
                                if (m.geo_target_constant_parents != null && m.hasOwnProperty("geo_target_constant_parents")) {
                                    if (!Array.isArray(m.geo_target_constant_parents))
                                        return "geo_target_constant_parents: array expected";
                                    for (var i = 0; i < m.geo_target_constant_parents.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.GeoTargetConstant.verify(m.geo_target_constant_parents[i]);
                                            if (e)
                                                return "geo_target_constant_parents." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GeoTargetConstantSuggestion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GeoTargetConstantSuggestion} GeoTargetConstantSuggestion
                             */
                            GeoTargetConstantSuggestion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GeoTargetConstantSuggestion();
                                if (d.locale != null) {
                                    m.locale = String(d.locale);
                                }
                                if (d.reach != null) {
                                    if ($util.Long)
                                        (m.reach = $util.Long.fromValue(d.reach)).unsigned = false;
                                    else if (typeof d.reach === "string")
                                        m.reach = parseInt(d.reach, 10);
                                    else if (typeof d.reach === "number")
                                        m.reach = d.reach;
                                    else if (typeof d.reach === "object")
                                        m.reach = new $util.LongBits(d.reach.low >>> 0, d.reach.high >>> 0).toNumber();
                                }
                                if (d.search_term != null) {
                                    m.search_term = String(d.search_term);
                                }
                                if (d.geo_target_constant != null) {
                                    if (typeof d.geo_target_constant !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GeoTargetConstantSuggestion.geo_target_constant: object expected");
                                    m.geo_target_constant = $root.google.ads.googleads.v11.resources.GeoTargetConstant.fromObject(d.geo_target_constant);
                                }
                                if (d.geo_target_constant_parents) {
                                    if (!Array.isArray(d.geo_target_constant_parents))
                                        throw TypeError(".google.ads.googleads.v11.services.GeoTargetConstantSuggestion.geo_target_constant_parents: array expected");
                                    m.geo_target_constant_parents = [];
                                    for (var i = 0; i < d.geo_target_constant_parents.length; ++i) {
                                        if (typeof d.geo_target_constant_parents[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GeoTargetConstantSuggestion.geo_target_constant_parents: object expected");
                                        m.geo_target_constant_parents[i] = $root.google.ads.googleads.v11.resources.GeoTargetConstant.fromObject(d.geo_target_constant_parents[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GeoTargetConstantSuggestion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.GeoTargetConstantSuggestion} m GeoTargetConstantSuggestion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GeoTargetConstantSuggestion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.geo_target_constant_parents = [];
                                }
                                if (o.defaults) {
                                    d.geo_target_constant = null;
                                }
                                if (m.geo_target_constant != null && m.hasOwnProperty("geo_target_constant")) {
                                    d.geo_target_constant = $root.google.ads.googleads.v11.resources.GeoTargetConstant.toObject(m.geo_target_constant, o);
                                }
                                if (m.geo_target_constant_parents && m.geo_target_constant_parents.length) {
                                    d.geo_target_constant_parents = [];
                                    for (var j = 0; j < m.geo_target_constant_parents.length; ++j) {
                                        d.geo_target_constant_parents[j] = $root.google.ads.googleads.v11.resources.GeoTargetConstant.toObject(m.geo_target_constant_parents[j], o);
                                    }
                                }
                                if (m.locale != null && m.hasOwnProperty("locale")) {
                                    d.locale = m.locale;
                                    if (o.oneofs)
                                        d._locale = "locale";
                                }
                                if (m.reach != null && m.hasOwnProperty("reach")) {
                                    if (typeof m.reach === "number")
                                        d.reach = o.longs === String ? String(m.reach) : m.reach;
                                    else
                                        d.reach = o.longs === String ? $util.Long.prototype.toString.call(m.reach) : o.longs === Number ? new $util.LongBits(m.reach.low >>> 0, m.reach.high >>> 0).toNumber() : m.reach;
                                    if (o.oneofs)
                                        d._reach = "reach";
                                }
                                if (m.search_term != null && m.hasOwnProperty("search_term")) {
                                    d.search_term = m.search_term;
                                    if (o.oneofs)
                                        d._search_term = "search_term";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GeoTargetConstantSuggestion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GeoTargetConstantSuggestion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GeoTargetConstantSuggestion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GeoTargetConstantSuggestion;
                        })();
    
                        services.GoogleAdsFieldService = (function() {
    
                            /**
                             * Constructs a new GoogleAdsFieldService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GoogleAdsFieldService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function GoogleAdsFieldService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (GoogleAdsFieldService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = GoogleAdsFieldService;
    
                            /**
                             * Creates new GoogleAdsFieldService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GoogleAdsFieldService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {GoogleAdsFieldService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            GoogleAdsFieldService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.GoogleAdsFieldService|getGoogleAdsField}.
                             * @memberof google.ads.googleads.v11.services.GoogleAdsFieldService
                             * @typedef GetGoogleAdsFieldCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.resources.GoogleAdsField} [response] GoogleAdsField
                             */
    
                            /**
                             * Calls GetGoogleAdsField.
                             * @function getGoogleAdsField
                             * @memberof google.ads.googleads.v11.services.GoogleAdsFieldService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGetGoogleAdsFieldRequest} request GetGoogleAdsFieldRequest message or plain object
                             * @param {google.ads.googleads.v11.services.GoogleAdsFieldService.GetGoogleAdsFieldCallback} callback Node-style callback called with the error, if any, and GoogleAdsField
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(GoogleAdsFieldService.prototype.getGoogleAdsField = function getGoogleAdsField(request, callback) {
                                return this.rpcCall(getGoogleAdsField, $root.google.ads.googleads.v11.services.GetGoogleAdsFieldRequest, $root.google.ads.googleads.v11.resources.GoogleAdsField, request, callback);
                            }, "name", { value: "GetGoogleAdsField" });
    
                            /**
                             * Calls GetGoogleAdsField.
                             * @function getGoogleAdsField
                             * @memberof google.ads.googleads.v11.services.GoogleAdsFieldService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGetGoogleAdsFieldRequest} request GetGoogleAdsFieldRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.resources.GoogleAdsField>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.GoogleAdsFieldService|searchGoogleAdsFields}.
                             * @memberof google.ads.googleads.v11.services.GoogleAdsFieldService
                             * @typedef SearchGoogleAdsFieldsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse} [response] SearchGoogleAdsFieldsResponse
                             */
    
                            /**
                             * Calls SearchGoogleAdsFields.
                             * @function searchGoogleAdsFields
                             * @memberof google.ads.googleads.v11.services.GoogleAdsFieldService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsRequest} request SearchGoogleAdsFieldsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.GoogleAdsFieldService.SearchGoogleAdsFieldsCallback} callback Node-style callback called with the error, if any, and SearchGoogleAdsFieldsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(GoogleAdsFieldService.prototype.searchGoogleAdsFields = function searchGoogleAdsFields(request, callback) {
                                return this.rpcCall(searchGoogleAdsFields, $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest, $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse, request, callback);
                            }, "name", { value: "SearchGoogleAdsFields" });
    
                            /**
                             * Calls SearchGoogleAdsFields.
                             * @function searchGoogleAdsFields
                             * @memberof google.ads.googleads.v11.services.GoogleAdsFieldService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsRequest} request SearchGoogleAdsFieldsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse>} Promise
                             * @variation 2
                             */
    
                            return GoogleAdsFieldService;
                        })();
    
                        services.GetGoogleAdsFieldRequest = (function() {
    
                            /**
                             * Properties of a GetGoogleAdsFieldRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGetGoogleAdsFieldRequest
                             * @property {string|null} [resource_name] GetGoogleAdsFieldRequest resource_name
                             */
    
                            /**
                             * Constructs a new GetGoogleAdsFieldRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GetGoogleAdsFieldRequest.
                             * @implements IGetGoogleAdsFieldRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGetGoogleAdsFieldRequest=} [p] Properties to set
                             */
                            function GetGoogleAdsFieldRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GetGoogleAdsFieldRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @instance
                             */
                            GetGoogleAdsFieldRequest.prototype.resource_name = "";
    
                            /**
                             * Creates a new GetGoogleAdsFieldRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetGoogleAdsFieldRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GetGoogleAdsFieldRequest} GetGoogleAdsFieldRequest instance
                             */
                            GetGoogleAdsFieldRequest.create = function create(properties) {
                                return new GetGoogleAdsFieldRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GetGoogleAdsFieldRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GetGoogleAdsFieldRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetGoogleAdsFieldRequest} m GetGoogleAdsFieldRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GetGoogleAdsFieldRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GetGoogleAdsFieldRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GetGoogleAdsFieldRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetGoogleAdsFieldRequest} message GetGoogleAdsFieldRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GetGoogleAdsFieldRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GetGoogleAdsFieldRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GetGoogleAdsFieldRequest} GetGoogleAdsFieldRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GetGoogleAdsFieldRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GetGoogleAdsFieldRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GetGoogleAdsFieldRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GetGoogleAdsFieldRequest} GetGoogleAdsFieldRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GetGoogleAdsFieldRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GetGoogleAdsFieldRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GetGoogleAdsFieldRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GetGoogleAdsFieldRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GetGoogleAdsFieldRequest} GetGoogleAdsFieldRequest
                             */
                            GetGoogleAdsFieldRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GetGoogleAdsFieldRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GetGoogleAdsFieldRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GetGoogleAdsFieldRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GetGoogleAdsFieldRequest} m GetGoogleAdsFieldRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GetGoogleAdsFieldRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GetGoogleAdsFieldRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GetGoogleAdsFieldRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GetGoogleAdsFieldRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GetGoogleAdsFieldRequest;
                        })();
    
                        services.SearchGoogleAdsFieldsRequest = (function() {
    
                            /**
                             * Properties of a SearchGoogleAdsFieldsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISearchGoogleAdsFieldsRequest
                             * @property {string|null} [query] SearchGoogleAdsFieldsRequest query
                             * @property {string|null} [page_token] SearchGoogleAdsFieldsRequest page_token
                             * @property {number|null} [page_size] SearchGoogleAdsFieldsRequest page_size
                             */
    
                            /**
                             * Constructs a new SearchGoogleAdsFieldsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SearchGoogleAdsFieldsRequest.
                             * @implements ISearchGoogleAdsFieldsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsRequest=} [p] Properties to set
                             */
                            function SearchGoogleAdsFieldsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SearchGoogleAdsFieldsRequest query.
                             * @member {string} query
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @instance
                             */
                            SearchGoogleAdsFieldsRequest.prototype.query = "";
    
                            /**
                             * SearchGoogleAdsFieldsRequest page_token.
                             * @member {string} page_token
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @instance
                             */
                            SearchGoogleAdsFieldsRequest.prototype.page_token = "";
    
                            /**
                             * SearchGoogleAdsFieldsRequest page_size.
                             * @member {number} page_size
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @instance
                             */
                            SearchGoogleAdsFieldsRequest.prototype.page_size = 0;
    
                            /**
                             * Creates a new SearchGoogleAdsFieldsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest} SearchGoogleAdsFieldsRequest instance
                             */
                            SearchGoogleAdsFieldsRequest.create = function create(properties) {
                                return new SearchGoogleAdsFieldsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsFieldsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsRequest} m SearchGoogleAdsFieldsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsFieldsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.query != null && Object.hasOwnProperty.call(m, "query"))
                                    w.uint32(10).string(m.query);
                                if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                                    w.uint32(18).string(m.page_token);
                                if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                                    w.uint32(24).int32(m.page_size);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsFieldsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsRequest} message SearchGoogleAdsFieldsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsFieldsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsFieldsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest} SearchGoogleAdsFieldsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsFieldsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.query = r.string();
                                        break;
                                    case 2:
                                        m.page_token = r.string();
                                        break;
                                    case 3:
                                        m.page_size = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsFieldsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest} SearchGoogleAdsFieldsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsFieldsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchGoogleAdsFieldsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchGoogleAdsFieldsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.query != null && m.hasOwnProperty("query")) {
                                    if (!$util.isString(m.query))
                                        return "query: string expected";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    if (!$util.isString(m.page_token))
                                        return "page_token: string expected";
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    if (!$util.isInteger(m.page_size))
                                        return "page_size: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SearchGoogleAdsFieldsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest} SearchGoogleAdsFieldsRequest
                             */
                            SearchGoogleAdsFieldsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest();
                                if (d.query != null) {
                                    m.query = String(d.query);
                                }
                                if (d.page_token != null) {
                                    m.page_token = String(d.page_token);
                                }
                                if (d.page_size != null) {
                                    m.page_size = d.page_size | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SearchGoogleAdsFieldsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest} m SearchGoogleAdsFieldsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchGoogleAdsFieldsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.query = "";
                                    d.page_token = "";
                                    d.page_size = 0;
                                }
                                if (m.query != null && m.hasOwnProperty("query")) {
                                    d.query = m.query;
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    d.page_token = m.page_token;
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    d.page_size = m.page_size;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SearchGoogleAdsFieldsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchGoogleAdsFieldsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SearchGoogleAdsFieldsRequest;
                        })();
    
                        services.SearchGoogleAdsFieldsResponse = (function() {
    
                            /**
                             * Properties of a SearchGoogleAdsFieldsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISearchGoogleAdsFieldsResponse
                             * @property {Array.<google.ads.googleads.v11.resources.IGoogleAdsField>|null} [results] SearchGoogleAdsFieldsResponse results
                             * @property {string|null} [next_page_token] SearchGoogleAdsFieldsResponse next_page_token
                             * @property {number|null} [total_results_count] SearchGoogleAdsFieldsResponse total_results_count
                             */
    
                            /**
                             * Constructs a new SearchGoogleAdsFieldsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SearchGoogleAdsFieldsResponse.
                             * @implements ISearchGoogleAdsFieldsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsResponse=} [p] Properties to set
                             */
                            function SearchGoogleAdsFieldsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SearchGoogleAdsFieldsResponse results.
                             * @member {Array.<google.ads.googleads.v11.resources.IGoogleAdsField>} results
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @instance
                             */
                            SearchGoogleAdsFieldsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * SearchGoogleAdsFieldsResponse next_page_token.
                             * @member {string} next_page_token
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @instance
                             */
                            SearchGoogleAdsFieldsResponse.prototype.next_page_token = "";
    
                            /**
                             * SearchGoogleAdsFieldsResponse total_results_count.
                             * @member {number} total_results_count
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @instance
                             */
                            SearchGoogleAdsFieldsResponse.prototype.total_results_count = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new SearchGoogleAdsFieldsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse} SearchGoogleAdsFieldsResponse instance
                             */
                            SearchGoogleAdsFieldsResponse.create = function create(properties) {
                                return new SearchGoogleAdsFieldsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsFieldsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsResponse} m SearchGoogleAdsFieldsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsFieldsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.resources.GoogleAdsField.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                                    w.uint32(18).string(m.next_page_token);
                                if (m.total_results_count != null && Object.hasOwnProperty.call(m, "total_results_count"))
                                    w.uint32(24).int64(m.total_results_count);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SearchGoogleAdsFieldsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISearchGoogleAdsFieldsResponse} message SearchGoogleAdsFieldsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SearchGoogleAdsFieldsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsFieldsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse} SearchGoogleAdsFieldsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsFieldsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.resources.GoogleAdsField.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.next_page_token = r.string();
                                        break;
                                    case 3:
                                        m.total_results_count = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SearchGoogleAdsFieldsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse} SearchGoogleAdsFieldsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SearchGoogleAdsFieldsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SearchGoogleAdsFieldsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SearchGoogleAdsFieldsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.GoogleAdsField.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    if (!$util.isString(m.next_page_token))
                                        return "next_page_token: string expected";
                                }
                                if (m.total_results_count != null && m.hasOwnProperty("total_results_count")) {
                                    if (!$util.isInteger(m.total_results_count) && !(m.total_results_count && $util.isInteger(m.total_results_count.low) && $util.isInteger(m.total_results_count.high)))
                                        return "total_results_count: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SearchGoogleAdsFieldsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse} SearchGoogleAdsFieldsResponse
                             */
                            SearchGoogleAdsFieldsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.resources.GoogleAdsField.fromObject(d.results[i]);
                                    }
                                }
                                if (d.next_page_token != null) {
                                    m.next_page_token = String(d.next_page_token);
                                }
                                if (d.total_results_count != null) {
                                    if ($util.Long)
                                        (m.total_results_count = $util.Long.fromValue(d.total_results_count)).unsigned = false;
                                    else if (typeof d.total_results_count === "string")
                                        m.total_results_count = parseInt(d.total_results_count, 10);
                                    else if (typeof d.total_results_count === "number")
                                        m.total_results_count = d.total_results_count;
                                    else if (typeof d.total_results_count === "object")
                                        m.total_results_count = new $util.LongBits(d.total_results_count.low >>> 0, d.total_results_count.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SearchGoogleAdsFieldsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse} m SearchGoogleAdsFieldsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SearchGoogleAdsFieldsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.next_page_token = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_results_count = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_results_count = o.longs === String ? "0" : 0;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.resources.GoogleAdsField.toObject(m.results[j], o);
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    d.next_page_token = m.next_page_token;
                                }
                                if (m.total_results_count != null && m.hasOwnProperty("total_results_count")) {
                                    if (typeof m.total_results_count === "number")
                                        d.total_results_count = o.longs === String ? String(m.total_results_count) : m.total_results_count;
                                    else
                                        d.total_results_count = o.longs === String ? $util.Long.prototype.toString.call(m.total_results_count) : o.longs === Number ? new $util.LongBits(m.total_results_count.low >>> 0, m.total_results_count.high >>> 0).toNumber() : m.total_results_count;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SearchGoogleAdsFieldsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SearchGoogleAdsFieldsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SearchGoogleAdsFieldsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SearchGoogleAdsFieldsResponse;
                        })();
    
                        services.InvoiceService = (function() {
    
                            /**
                             * Constructs a new InvoiceService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an InvoiceService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function InvoiceService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (InvoiceService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = InvoiceService;
    
                            /**
                             * Creates new InvoiceService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.InvoiceService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {InvoiceService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            InvoiceService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.InvoiceService|listInvoices}.
                             * @memberof google.ads.googleads.v11.services.InvoiceService
                             * @typedef ListInvoicesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListInvoicesResponse} [response] ListInvoicesResponse
                             */
    
                            /**
                             * Calls ListInvoices.
                             * @function listInvoices
                             * @memberof google.ads.googleads.v11.services.InvoiceService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListInvoicesRequest} request ListInvoicesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.InvoiceService.ListInvoicesCallback} callback Node-style callback called with the error, if any, and ListInvoicesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(InvoiceService.prototype.listInvoices = function listInvoices(request, callback) {
                                return this.rpcCall(listInvoices, $root.google.ads.googleads.v11.services.ListInvoicesRequest, $root.google.ads.googleads.v11.services.ListInvoicesResponse, request, callback);
                            }, "name", { value: "ListInvoices" });
    
                            /**
                             * Calls ListInvoices.
                             * @function listInvoices
                             * @memberof google.ads.googleads.v11.services.InvoiceService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListInvoicesRequest} request ListInvoicesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListInvoicesResponse>} Promise
                             * @variation 2
                             */
    
                            return InvoiceService;
                        })();
    
                        services.ListInvoicesRequest = (function() {
    
                            /**
                             * Properties of a ListInvoicesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListInvoicesRequest
                             * @property {string|null} [customer_id] ListInvoicesRequest customer_id
                             * @property {string|null} [billing_setup] ListInvoicesRequest billing_setup
                             * @property {string|null} [issue_year] ListInvoicesRequest issue_year
                             * @property {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear|null} [issue_month] ListInvoicesRequest issue_month
                             */
    
                            /**
                             * Constructs a new ListInvoicesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListInvoicesRequest.
                             * @implements IListInvoicesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListInvoicesRequest=} [p] Properties to set
                             */
                            function ListInvoicesRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListInvoicesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @instance
                             */
                            ListInvoicesRequest.prototype.customer_id = "";
    
                            /**
                             * ListInvoicesRequest billing_setup.
                             * @member {string} billing_setup
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @instance
                             */
                            ListInvoicesRequest.prototype.billing_setup = "";
    
                            /**
                             * ListInvoicesRequest issue_year.
                             * @member {string} issue_year
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @instance
                             */
                            ListInvoicesRequest.prototype.issue_year = "";
    
                            /**
                             * ListInvoicesRequest issue_month.
                             * @member {google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear} issue_month
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @instance
                             */
                            ListInvoicesRequest.prototype.issue_month = 0;
    
                            /**
                             * Creates a new ListInvoicesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListInvoicesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListInvoicesRequest} ListInvoicesRequest instance
                             */
                            ListInvoicesRequest.create = function create(properties) {
                                return new ListInvoicesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListInvoicesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListInvoicesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListInvoicesRequest} m ListInvoicesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListInvoicesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.billing_setup != null && Object.hasOwnProperty.call(m, "billing_setup"))
                                    w.uint32(18).string(m.billing_setup);
                                if (m.issue_year != null && Object.hasOwnProperty.call(m, "issue_year"))
                                    w.uint32(26).string(m.issue_year);
                                if (m.issue_month != null && Object.hasOwnProperty.call(m, "issue_month"))
                                    w.uint32(32).int32(m.issue_month);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListInvoicesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListInvoicesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListInvoicesRequest} message ListInvoicesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListInvoicesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListInvoicesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListInvoicesRequest} ListInvoicesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListInvoicesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListInvoicesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.billing_setup = r.string();
                                        break;
                                    case 3:
                                        m.issue_year = r.string();
                                        break;
                                    case 4:
                                        m.issue_month = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListInvoicesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListInvoicesRequest} ListInvoicesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListInvoicesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListInvoicesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListInvoicesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    if (!$util.isString(m.billing_setup))
                                        return "billing_setup: string expected";
                                }
                                if (m.issue_year != null && m.hasOwnProperty("issue_year")) {
                                    if (!$util.isString(m.issue_year))
                                        return "issue_year: string expected";
                                }
                                if (m.issue_month != null && m.hasOwnProperty("issue_month")) {
                                    switch (m.issue_month) {
                                    default:
                                        return "issue_month: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListInvoicesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListInvoicesRequest} ListInvoicesRequest
                             */
                            ListInvoicesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListInvoicesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListInvoicesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.billing_setup != null) {
                                    m.billing_setup = String(d.billing_setup);
                                }
                                if (d.issue_year != null) {
                                    m.issue_year = String(d.issue_year);
                                }
                                switch (d.issue_month) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.issue_month = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.issue_month = 1;
                                    break;
                                case "JANUARY":
                                case 2:
                                    m.issue_month = 2;
                                    break;
                                case "FEBRUARY":
                                case 3:
                                    m.issue_month = 3;
                                    break;
                                case "MARCH":
                                case 4:
                                    m.issue_month = 4;
                                    break;
                                case "APRIL":
                                case 5:
                                    m.issue_month = 5;
                                    break;
                                case "MAY":
                                case 6:
                                    m.issue_month = 6;
                                    break;
                                case "JUNE":
                                case 7:
                                    m.issue_month = 7;
                                    break;
                                case "JULY":
                                case 8:
                                    m.issue_month = 8;
                                    break;
                                case "AUGUST":
                                case 9:
                                    m.issue_month = 9;
                                    break;
                                case "SEPTEMBER":
                                case 10:
                                    m.issue_month = 10;
                                    break;
                                case "OCTOBER":
                                case 11:
                                    m.issue_month = 11;
                                    break;
                                case "NOVEMBER":
                                case 12:
                                    m.issue_month = 12;
                                    break;
                                case "DECEMBER":
                                case 13:
                                    m.issue_month = 13;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListInvoicesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListInvoicesRequest} m ListInvoicesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListInvoicesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.billing_setup = "";
                                    d.issue_year = "";
                                    d.issue_month = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.billing_setup != null && m.hasOwnProperty("billing_setup")) {
                                    d.billing_setup = m.billing_setup;
                                }
                                if (m.issue_year != null && m.hasOwnProperty("issue_year")) {
                                    d.issue_year = m.issue_year;
                                }
                                if (m.issue_month != null && m.hasOwnProperty("issue_month")) {
                                    d.issue_month = o.enums === String ? $root.google.ads.googleads.v11.enums.MonthOfYearEnum.MonthOfYear[m.issue_month] : m.issue_month;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListInvoicesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListInvoicesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListInvoicesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListInvoicesRequest;
                        })();
    
                        services.ListInvoicesResponse = (function() {
    
                            /**
                             * Properties of a ListInvoicesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListInvoicesResponse
                             * @property {Array.<google.ads.googleads.v11.resources.IInvoice>|null} [invoices] ListInvoicesResponse invoices
                             */
    
                            /**
                             * Constructs a new ListInvoicesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListInvoicesResponse.
                             * @implements IListInvoicesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListInvoicesResponse=} [p] Properties to set
                             */
                            function ListInvoicesResponse(p) {
                                this.invoices = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListInvoicesResponse invoices.
                             * @member {Array.<google.ads.googleads.v11.resources.IInvoice>} invoices
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @instance
                             */
                            ListInvoicesResponse.prototype.invoices = $util.emptyArray;
    
                            /**
                             * Creates a new ListInvoicesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListInvoicesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListInvoicesResponse} ListInvoicesResponse instance
                             */
                            ListInvoicesResponse.create = function create(properties) {
                                return new ListInvoicesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListInvoicesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListInvoicesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListInvoicesResponse} m ListInvoicesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListInvoicesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.invoices != null && m.invoices.length) {
                                    for (var i = 0; i < m.invoices.length; ++i)
                                        $root.google.ads.googleads.v11.resources.Invoice.encode(m.invoices[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListInvoicesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListInvoicesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListInvoicesResponse} message ListInvoicesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListInvoicesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListInvoicesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListInvoicesResponse} ListInvoicesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListInvoicesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListInvoicesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.invoices && m.invoices.length))
                                            m.invoices = [];
                                        m.invoices.push($root.google.ads.googleads.v11.resources.Invoice.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListInvoicesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListInvoicesResponse} ListInvoicesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListInvoicesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListInvoicesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListInvoicesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.invoices != null && m.hasOwnProperty("invoices")) {
                                    if (!Array.isArray(m.invoices))
                                        return "invoices: array expected";
                                    for (var i = 0; i < m.invoices.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Invoice.verify(m.invoices[i]);
                                            if (e)
                                                return "invoices." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListInvoicesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListInvoicesResponse} ListInvoicesResponse
                             */
                            ListInvoicesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListInvoicesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListInvoicesResponse();
                                if (d.invoices) {
                                    if (!Array.isArray(d.invoices))
                                        throw TypeError(".google.ads.googleads.v11.services.ListInvoicesResponse.invoices: array expected");
                                    m.invoices = [];
                                    for (var i = 0; i < d.invoices.length; ++i) {
                                        if (typeof d.invoices[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListInvoicesResponse.invoices: object expected");
                                        m.invoices[i] = $root.google.ads.googleads.v11.resources.Invoice.fromObject(d.invoices[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListInvoicesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListInvoicesResponse} m ListInvoicesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListInvoicesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.invoices = [];
                                }
                                if (m.invoices && m.invoices.length) {
                                    d.invoices = [];
                                    for (var j = 0; j < m.invoices.length; ++j) {
                                        d.invoices[j] = $root.google.ads.googleads.v11.resources.Invoice.toObject(m.invoices[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListInvoicesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListInvoicesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListInvoicesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListInvoicesResponse;
                        })();
    
                        services.KeywordPlanIdeaService = (function() {
    
                            /**
                             * Constructs a new KeywordPlanIdeaService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordPlanIdeaService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function KeywordPlanIdeaService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (KeywordPlanIdeaService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = KeywordPlanIdeaService;
    
                            /**
                             * Creates new KeywordPlanIdeaService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {KeywordPlanIdeaService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            KeywordPlanIdeaService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanIdeaService|generateKeywordIdeas}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @typedef GenerateKeywordIdeasCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateKeywordIdeaResponse} [response] GenerateKeywordIdeaResponse
                             */
    
                            /**
                             * Calls GenerateKeywordIdeas.
                             * @function generateKeywordIdeas
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeasRequest} request GenerateKeywordIdeasRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanIdeaService.GenerateKeywordIdeasCallback} callback Node-style callback called with the error, if any, and GenerateKeywordIdeaResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanIdeaService.prototype.generateKeywordIdeas = function generateKeywordIdeas(request, callback) {
                                return this.rpcCall(generateKeywordIdeas, $root.google.ads.googleads.v11.services.GenerateKeywordIdeasRequest, $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResponse, request, callback);
                            }, "name", { value: "GenerateKeywordIdeas" });
    
                            /**
                             * Calls GenerateKeywordIdeas.
                             * @function generateKeywordIdeas
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeasRequest} request GenerateKeywordIdeasRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateKeywordIdeaResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanIdeaService|generateKeywordHistoricalMetrics}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @typedef GenerateKeywordHistoricalMetricsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse} [response] GenerateKeywordHistoricalMetricsResponse
                             */
    
                            /**
                             * Calls GenerateKeywordHistoricalMetrics.
                             * @function generateKeywordHistoricalMetrics
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsRequest} request GenerateKeywordHistoricalMetricsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanIdeaService.GenerateKeywordHistoricalMetricsCallback} callback Node-style callback called with the error, if any, and GenerateKeywordHistoricalMetricsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanIdeaService.prototype.generateKeywordHistoricalMetrics = function generateKeywordHistoricalMetrics(request, callback) {
                                return this.rpcCall(generateKeywordHistoricalMetrics, $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest, $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse, request, callback);
                            }, "name", { value: "GenerateKeywordHistoricalMetrics" });
    
                            /**
                             * Calls GenerateKeywordHistoricalMetrics.
                             * @function generateKeywordHistoricalMetrics
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsRequest} request GenerateKeywordHistoricalMetricsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordPlanIdeaService|generateAdGroupThemes}.
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @typedef GenerateAdGroupThemesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateAdGroupThemesResponse} [response] GenerateAdGroupThemesResponse
                             */
    
                            /**
                             * Calls GenerateAdGroupThemes.
                             * @function generateAdGroupThemes
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesRequest} request GenerateAdGroupThemesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordPlanIdeaService.GenerateAdGroupThemesCallback} callback Node-style callback called with the error, if any, and GenerateAdGroupThemesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordPlanIdeaService.prototype.generateAdGroupThemes = function generateAdGroupThemes(request, callback) {
                                return this.rpcCall(generateAdGroupThemes, $root.google.ads.googleads.v11.services.GenerateAdGroupThemesRequest, $root.google.ads.googleads.v11.services.GenerateAdGroupThemesResponse, request, callback);
                            }, "name", { value: "GenerateAdGroupThemes" });
    
                            /**
                             * Calls GenerateAdGroupThemes.
                             * @function generateAdGroupThemes
                             * @memberof google.ads.googleads.v11.services.KeywordPlanIdeaService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesRequest} request GenerateAdGroupThemesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateAdGroupThemesResponse>} Promise
                             * @variation 2
                             */
    
                            return KeywordPlanIdeaService;
                        })();
    
                        services.GenerateKeywordIdeasRequest = (function() {
    
                            /**
                             * Properties of a GenerateKeywordIdeasRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateKeywordIdeasRequest
                             * @property {string|null} [customer_id] GenerateKeywordIdeasRequest customer_id
                             * @property {string|null} [language] GenerateKeywordIdeasRequest language
                             * @property {Array.<string>|null} [geo_target_constants] GenerateKeywordIdeasRequest geo_target_constants
                             * @property {boolean|null} [include_adult_keywords] GenerateKeywordIdeasRequest include_adult_keywords
                             * @property {string|null} [page_token] GenerateKeywordIdeasRequest page_token
                             * @property {number|null} [page_size] GenerateKeywordIdeasRequest page_size
                             * @property {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork|null} [keyword_plan_network] GenerateKeywordIdeasRequest keyword_plan_network
                             * @property {Array.<google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation>|null} [keyword_annotation] GenerateKeywordIdeasRequest keyword_annotation
                             * @property {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics|null} [aggregate_metrics] GenerateKeywordIdeasRequest aggregate_metrics
                             * @property {google.ads.googleads.v11.common.IHistoricalMetricsOptions|null} [historical_metrics_options] GenerateKeywordIdeasRequest historical_metrics_options
                             * @property {google.ads.googleads.v11.services.IKeywordAndUrlSeed|null} [keyword_and_url_seed] GenerateKeywordIdeasRequest keyword_and_url_seed
                             * @property {google.ads.googleads.v11.services.IKeywordSeed|null} [keyword_seed] GenerateKeywordIdeasRequest keyword_seed
                             * @property {google.ads.googleads.v11.services.IUrlSeed|null} [url_seed] GenerateKeywordIdeasRequest url_seed
                             * @property {google.ads.googleads.v11.services.ISiteSeed|null} [site_seed] GenerateKeywordIdeasRequest site_seed
                             */
    
                            /**
                             * Constructs a new GenerateKeywordIdeasRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateKeywordIdeasRequest.
                             * @implements IGenerateKeywordIdeasRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeasRequest=} [p] Properties to set
                             */
                            function GenerateKeywordIdeasRequest(p) {
                                this.geo_target_constants = [];
                                this.keyword_annotation = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateKeywordIdeasRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.customer_id = "";
    
                            /**
                             * GenerateKeywordIdeasRequest language.
                             * @member {string|null|undefined} language
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.language = null;
    
                            /**
                             * GenerateKeywordIdeasRequest geo_target_constants.
                             * @member {Array.<string>} geo_target_constants
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.geo_target_constants = $util.emptyArray;
    
                            /**
                             * GenerateKeywordIdeasRequest include_adult_keywords.
                             * @member {boolean} include_adult_keywords
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.include_adult_keywords = false;
    
                            /**
                             * GenerateKeywordIdeasRequest page_token.
                             * @member {string} page_token
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.page_token = "";
    
                            /**
                             * GenerateKeywordIdeasRequest page_size.
                             * @member {number} page_size
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.page_size = 0;
    
                            /**
                             * GenerateKeywordIdeasRequest keyword_plan_network.
                             * @member {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork} keyword_plan_network
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.keyword_plan_network = 0;
    
                            /**
                             * GenerateKeywordIdeasRequest keyword_annotation.
                             * @member {Array.<google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation>} keyword_annotation
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.keyword_annotation = $util.emptyArray;
    
                            /**
                             * GenerateKeywordIdeasRequest aggregate_metrics.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics|null|undefined} aggregate_metrics
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.aggregate_metrics = null;
    
                            /**
                             * GenerateKeywordIdeasRequest historical_metrics_options.
                             * @member {google.ads.googleads.v11.common.IHistoricalMetricsOptions|null|undefined} historical_metrics_options
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.historical_metrics_options = null;
    
                            /**
                             * GenerateKeywordIdeasRequest keyword_and_url_seed.
                             * @member {google.ads.googleads.v11.services.IKeywordAndUrlSeed|null|undefined} keyword_and_url_seed
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.keyword_and_url_seed = null;
    
                            /**
                             * GenerateKeywordIdeasRequest keyword_seed.
                             * @member {google.ads.googleads.v11.services.IKeywordSeed|null|undefined} keyword_seed
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.keyword_seed = null;
    
                            /**
                             * GenerateKeywordIdeasRequest url_seed.
                             * @member {google.ads.googleads.v11.services.IUrlSeed|null|undefined} url_seed
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.url_seed = null;
    
                            /**
                             * GenerateKeywordIdeasRequest site_seed.
                             * @member {google.ads.googleads.v11.services.ISiteSeed|null|undefined} site_seed
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            GenerateKeywordIdeasRequest.prototype.site_seed = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GenerateKeywordIdeasRequest _language.
                             * @member {"language"|undefined} _language
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            Object.defineProperty(GenerateKeywordIdeasRequest.prototype, "_language", {
                                get: $util.oneOfGetter($oneOfFields = ["language"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GenerateKeywordIdeasRequest seed.
                             * @member {"keyword_and_url_seed"|"keyword_seed"|"url_seed"|"site_seed"|undefined} seed
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             */
                            Object.defineProperty(GenerateKeywordIdeasRequest.prototype, "seed", {
                                get: $util.oneOfGetter($oneOfFields = ["keyword_and_url_seed", "keyword_seed", "url_seed", "site_seed"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GenerateKeywordIdeasRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeasRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeasRequest} GenerateKeywordIdeasRequest instance
                             */
                            GenerateKeywordIdeasRequest.create = function create(properties) {
                                return new GenerateKeywordIdeasRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordIdeasRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeasRequest} m GenerateKeywordIdeasRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordIdeasRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.keyword_and_url_seed != null && Object.hasOwnProperty.call(m, "keyword_and_url_seed"))
                                    $root.google.ads.googleads.v11.services.KeywordAndUrlSeed.encode(m.keyword_and_url_seed, w.uint32(18).fork()).ldelim();
                                if (m.keyword_seed != null && Object.hasOwnProperty.call(m, "keyword_seed"))
                                    $root.google.ads.googleads.v11.services.KeywordSeed.encode(m.keyword_seed, w.uint32(26).fork()).ldelim();
                                if (m.url_seed != null && Object.hasOwnProperty.call(m, "url_seed"))
                                    $root.google.ads.googleads.v11.services.UrlSeed.encode(m.url_seed, w.uint32(42).fork()).ldelim();
                                if (m.keyword_plan_network != null && Object.hasOwnProperty.call(m, "keyword_plan_network"))
                                    w.uint32(72).int32(m.keyword_plan_network);
                                if (m.include_adult_keywords != null && Object.hasOwnProperty.call(m, "include_adult_keywords"))
                                    w.uint32(80).bool(m.include_adult_keywords);
                                if (m.site_seed != null && Object.hasOwnProperty.call(m, "site_seed"))
                                    $root.google.ads.googleads.v11.services.SiteSeed.encode(m.site_seed, w.uint32(90).fork()).ldelim();
                                if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                                    w.uint32(98).string(m.page_token);
                                if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                                    w.uint32(104).int32(m.page_size);
                                if (m.language != null && Object.hasOwnProperty.call(m, "language"))
                                    w.uint32(114).string(m.language);
                                if (m.geo_target_constants != null && m.geo_target_constants.length) {
                                    for (var i = 0; i < m.geo_target_constants.length; ++i)
                                        w.uint32(122).string(m.geo_target_constants[i]);
                                }
                                if (m.aggregate_metrics != null && Object.hasOwnProperty.call(m, "aggregate_metrics"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.encode(m.aggregate_metrics, w.uint32(130).fork()).ldelim();
                                if (m.keyword_annotation != null && m.keyword_annotation.length) {
                                    w.uint32(138).fork();
                                    for (var i = 0; i < m.keyword_annotation.length; ++i)
                                        w.int32(m.keyword_annotation[i]);
                                    w.ldelim();
                                }
                                if (m.historical_metrics_options != null && Object.hasOwnProperty.call(m, "historical_metrics_options"))
                                    $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.encode(m.historical_metrics_options, w.uint32(146).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordIdeasRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeasRequest} message GenerateKeywordIdeasRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordIdeasRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateKeywordIdeasRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeasRequest} GenerateKeywordIdeasRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordIdeasRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateKeywordIdeasRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 14:
                                        m.language = r.string();
                                        break;
                                    case 15:
                                        if (!(m.geo_target_constants && m.geo_target_constants.length))
                                            m.geo_target_constants = [];
                                        m.geo_target_constants.push(r.string());
                                        break;
                                    case 10:
                                        m.include_adult_keywords = r.bool();
                                        break;
                                    case 12:
                                        m.page_token = r.string();
                                        break;
                                    case 13:
                                        m.page_size = r.int32();
                                        break;
                                    case 9:
                                        m.keyword_plan_network = r.int32();
                                        break;
                                    case 17:
                                        if (!(m.keyword_annotation && m.keyword_annotation.length))
                                            m.keyword_annotation = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.keyword_annotation.push(r.int32());
                                        } else
                                            m.keyword_annotation.push(r.int32());
                                        break;
                                    case 16:
                                        m.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.decode(r, r.uint32());
                                        break;
                                    case 18:
                                        m.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.keyword_and_url_seed = $root.google.ads.googleads.v11.services.KeywordAndUrlSeed.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.keyword_seed = $root.google.ads.googleads.v11.services.KeywordSeed.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.url_seed = $root.google.ads.googleads.v11.services.UrlSeed.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.site_seed = $root.google.ads.googleads.v11.services.SiteSeed.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateKeywordIdeasRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeasRequest} GenerateKeywordIdeasRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordIdeasRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateKeywordIdeasRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateKeywordIdeasRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.language != null && m.hasOwnProperty("language")) {
                                    p._language = 1;
                                    if (!$util.isString(m.language))
                                        return "language: string expected";
                                }
                                if (m.geo_target_constants != null && m.hasOwnProperty("geo_target_constants")) {
                                    if (!Array.isArray(m.geo_target_constants))
                                        return "geo_target_constants: array expected";
                                    for (var i = 0; i < m.geo_target_constants.length; ++i) {
                                        if (!$util.isString(m.geo_target_constants[i]))
                                            return "geo_target_constants: string[] expected";
                                    }
                                }
                                if (m.include_adult_keywords != null && m.hasOwnProperty("include_adult_keywords")) {
                                    if (typeof m.include_adult_keywords !== "boolean")
                                        return "include_adult_keywords: boolean expected";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    if (!$util.isString(m.page_token))
                                        return "page_token: string expected";
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    if (!$util.isInteger(m.page_size))
                                        return "page_size: integer expected";
                                }
                                if (m.keyword_plan_network != null && m.hasOwnProperty("keyword_plan_network")) {
                                    switch (m.keyword_plan_network) {
                                    default:
                                        return "keyword_plan_network: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.keyword_annotation != null && m.hasOwnProperty("keyword_annotation")) {
                                    if (!Array.isArray(m.keyword_annotation))
                                        return "keyword_annotation: array expected";
                                    for (var i = 0; i < m.keyword_annotation.length; ++i) {
                                        switch (m.keyword_annotation[i]) {
                                        default:
                                            return "keyword_annotation: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                            break;
                                        }
                                    }
                                }
                                if (m.aggregate_metrics != null && m.hasOwnProperty("aggregate_metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.verify(m.aggregate_metrics);
                                        if (e)
                                            return "aggregate_metrics." + e;
                                    }
                                }
                                if (m.historical_metrics_options != null && m.hasOwnProperty("historical_metrics_options")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.verify(m.historical_metrics_options);
                                        if (e)
                                            return "historical_metrics_options." + e;
                                    }
                                }
                                if (m.keyword_and_url_seed != null && m.hasOwnProperty("keyword_and_url_seed")) {
                                    p.seed = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordAndUrlSeed.verify(m.keyword_and_url_seed);
                                        if (e)
                                            return "keyword_and_url_seed." + e;
                                    }
                                }
                                if (m.keyword_seed != null && m.hasOwnProperty("keyword_seed")) {
                                    if (p.seed === 1)
                                        return "seed: multiple values";
                                    p.seed = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.KeywordSeed.verify(m.keyword_seed);
                                        if (e)
                                            return "keyword_seed." + e;
                                    }
                                }
                                if (m.url_seed != null && m.hasOwnProperty("url_seed")) {
                                    if (p.seed === 1)
                                        return "seed: multiple values";
                                    p.seed = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.UrlSeed.verify(m.url_seed);
                                        if (e)
                                            return "url_seed." + e;
                                    }
                                }
                                if (m.site_seed != null && m.hasOwnProperty("site_seed")) {
                                    if (p.seed === 1)
                                        return "seed: multiple values";
                                    p.seed = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SiteSeed.verify(m.site_seed);
                                        if (e)
                                            return "site_seed." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateKeywordIdeasRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeasRequest} GenerateKeywordIdeasRequest
                             */
                            GenerateKeywordIdeasRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateKeywordIdeasRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateKeywordIdeasRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.language != null) {
                                    m.language = String(d.language);
                                }
                                if (d.geo_target_constants) {
                                    if (!Array.isArray(d.geo_target_constants))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.geo_target_constants: array expected");
                                    m.geo_target_constants = [];
                                    for (var i = 0; i < d.geo_target_constants.length; ++i) {
                                        m.geo_target_constants[i] = String(d.geo_target_constants[i]);
                                    }
                                }
                                if (d.include_adult_keywords != null) {
                                    m.include_adult_keywords = Boolean(d.include_adult_keywords);
                                }
                                if (d.page_token != null) {
                                    m.page_token = String(d.page_token);
                                }
                                if (d.page_size != null) {
                                    m.page_size = d.page_size | 0;
                                }
                                switch (d.keyword_plan_network) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_network = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_network = 1;
                                    break;
                                case "GOOGLE_SEARCH":
                                case 2:
                                    m.keyword_plan_network = 2;
                                    break;
                                case "GOOGLE_SEARCH_AND_PARTNERS":
                                case 3:
                                    m.keyword_plan_network = 3;
                                    break;
                                }
                                if (d.keyword_annotation) {
                                    if (!Array.isArray(d.keyword_annotation))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.keyword_annotation: array expected");
                                    m.keyword_annotation = [];
                                    for (var i = 0; i < d.keyword_annotation.length; ++i) {
                                        switch (d.keyword_annotation[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.keyword_annotation[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.keyword_annotation[i] = 1;
                                            break;
                                        case "KEYWORD_CONCEPT":
                                        case 2:
                                            m.keyword_annotation[i] = 2;
                                            break;
                                        }
                                    }
                                }
                                if (d.aggregate_metrics != null) {
                                    if (typeof d.aggregate_metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.aggregate_metrics: object expected");
                                    m.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.fromObject(d.aggregate_metrics);
                                }
                                if (d.historical_metrics_options != null) {
                                    if (typeof d.historical_metrics_options !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.historical_metrics_options: object expected");
                                    m.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.fromObject(d.historical_metrics_options);
                                }
                                if (d.keyword_and_url_seed != null) {
                                    if (typeof d.keyword_and_url_seed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.keyword_and_url_seed: object expected");
                                    m.keyword_and_url_seed = $root.google.ads.googleads.v11.services.KeywordAndUrlSeed.fromObject(d.keyword_and_url_seed);
                                }
                                if (d.keyword_seed != null) {
                                    if (typeof d.keyword_seed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.keyword_seed: object expected");
                                    m.keyword_seed = $root.google.ads.googleads.v11.services.KeywordSeed.fromObject(d.keyword_seed);
                                }
                                if (d.url_seed != null) {
                                    if (typeof d.url_seed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.url_seed: object expected");
                                    m.url_seed = $root.google.ads.googleads.v11.services.UrlSeed.fromObject(d.url_seed);
                                }
                                if (d.site_seed != null) {
                                    if (typeof d.site_seed !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeasRequest.site_seed: object expected");
                                    m.site_seed = $root.google.ads.googleads.v11.services.SiteSeed.fromObject(d.site_seed);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateKeywordIdeasRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateKeywordIdeasRequest} m GenerateKeywordIdeasRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateKeywordIdeasRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.geo_target_constants = [];
                                    d.keyword_annotation = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.keyword_plan_network = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.include_adult_keywords = false;
                                    d.page_token = "";
                                    d.page_size = 0;
                                    d.aggregate_metrics = null;
                                    d.historical_metrics_options = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.keyword_and_url_seed != null && m.hasOwnProperty("keyword_and_url_seed")) {
                                    d.keyword_and_url_seed = $root.google.ads.googleads.v11.services.KeywordAndUrlSeed.toObject(m.keyword_and_url_seed, o);
                                    if (o.oneofs)
                                        d.seed = "keyword_and_url_seed";
                                }
                                if (m.keyword_seed != null && m.hasOwnProperty("keyword_seed")) {
                                    d.keyword_seed = $root.google.ads.googleads.v11.services.KeywordSeed.toObject(m.keyword_seed, o);
                                    if (o.oneofs)
                                        d.seed = "keyword_seed";
                                }
                                if (m.url_seed != null && m.hasOwnProperty("url_seed")) {
                                    d.url_seed = $root.google.ads.googleads.v11.services.UrlSeed.toObject(m.url_seed, o);
                                    if (o.oneofs)
                                        d.seed = "url_seed";
                                }
                                if (m.keyword_plan_network != null && m.hasOwnProperty("keyword_plan_network")) {
                                    d.keyword_plan_network = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork[m.keyword_plan_network] : m.keyword_plan_network;
                                }
                                if (m.include_adult_keywords != null && m.hasOwnProperty("include_adult_keywords")) {
                                    d.include_adult_keywords = m.include_adult_keywords;
                                }
                                if (m.site_seed != null && m.hasOwnProperty("site_seed")) {
                                    d.site_seed = $root.google.ads.googleads.v11.services.SiteSeed.toObject(m.site_seed, o);
                                    if (o.oneofs)
                                        d.seed = "site_seed";
                                }
                                if (m.page_token != null && m.hasOwnProperty("page_token")) {
                                    d.page_token = m.page_token;
                                }
                                if (m.page_size != null && m.hasOwnProperty("page_size")) {
                                    d.page_size = m.page_size;
                                }
                                if (m.language != null && m.hasOwnProperty("language")) {
                                    d.language = m.language;
                                    if (o.oneofs)
                                        d._language = "language";
                                }
                                if (m.geo_target_constants && m.geo_target_constants.length) {
                                    d.geo_target_constants = [];
                                    for (var j = 0; j < m.geo_target_constants.length; ++j) {
                                        d.geo_target_constants[j] = m.geo_target_constants[j];
                                    }
                                }
                                if (m.aggregate_metrics != null && m.hasOwnProperty("aggregate_metrics")) {
                                    d.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.toObject(m.aggregate_metrics, o);
                                }
                                if (m.keyword_annotation && m.keyword_annotation.length) {
                                    d.keyword_annotation = [];
                                    for (var j = 0; j < m.keyword_annotation.length; ++j) {
                                        d.keyword_annotation[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation[m.keyword_annotation[j]] : m.keyword_annotation[j];
                                    }
                                }
                                if (m.historical_metrics_options != null && m.hasOwnProperty("historical_metrics_options")) {
                                    d.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.toObject(m.historical_metrics_options, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateKeywordIdeasRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeasRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateKeywordIdeasRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateKeywordIdeasRequest;
                        })();
    
                        services.KeywordAndUrlSeed = (function() {
    
                            /**
                             * Properties of a KeywordAndUrlSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordAndUrlSeed
                             * @property {string|null} [url] KeywordAndUrlSeed url
                             * @property {Array.<string>|null} [keywords] KeywordAndUrlSeed keywords
                             */
    
                            /**
                             * Constructs a new KeywordAndUrlSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordAndUrlSeed.
                             * @implements IKeywordAndUrlSeed
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordAndUrlSeed=} [p] Properties to set
                             */
                            function KeywordAndUrlSeed(p) {
                                this.keywords = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordAndUrlSeed url.
                             * @member {string|null|undefined} url
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @instance
                             */
                            KeywordAndUrlSeed.prototype.url = null;
    
                            /**
                             * KeywordAndUrlSeed keywords.
                             * @member {Array.<string>} keywords
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @instance
                             */
                            KeywordAndUrlSeed.prototype.keywords = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * KeywordAndUrlSeed _url.
                             * @member {"url"|undefined} _url
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @instance
                             */
                            Object.defineProperty(KeywordAndUrlSeed.prototype, "_url", {
                                get: $util.oneOfGetter($oneOfFields = ["url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new KeywordAndUrlSeed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordAndUrlSeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordAndUrlSeed} KeywordAndUrlSeed instance
                             */
                            KeywordAndUrlSeed.create = function create(properties) {
                                return new KeywordAndUrlSeed(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordAndUrlSeed message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordAndUrlSeed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordAndUrlSeed} m KeywordAndUrlSeed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordAndUrlSeed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.url != null && Object.hasOwnProperty.call(m, "url"))
                                    w.uint32(26).string(m.url);
                                if (m.keywords != null && m.keywords.length) {
                                    for (var i = 0; i < m.keywords.length; ++i)
                                        w.uint32(34).string(m.keywords[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordAndUrlSeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordAndUrlSeed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordAndUrlSeed} message KeywordAndUrlSeed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordAndUrlSeed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordAndUrlSeed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordAndUrlSeed} KeywordAndUrlSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordAndUrlSeed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordAndUrlSeed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.url = r.string();
                                        break;
                                    case 4:
                                        if (!(m.keywords && m.keywords.length))
                                            m.keywords = [];
                                        m.keywords.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordAndUrlSeed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordAndUrlSeed} KeywordAndUrlSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordAndUrlSeed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordAndUrlSeed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordAndUrlSeed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    p._url = 1;
                                    if (!$util.isString(m.url))
                                        return "url: string expected";
                                }
                                if (m.keywords != null && m.hasOwnProperty("keywords")) {
                                    if (!Array.isArray(m.keywords))
                                        return "keywords: array expected";
                                    for (var i = 0; i < m.keywords.length; ++i) {
                                        if (!$util.isString(m.keywords[i]))
                                            return "keywords: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordAndUrlSeed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordAndUrlSeed} KeywordAndUrlSeed
                             */
                            KeywordAndUrlSeed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordAndUrlSeed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordAndUrlSeed();
                                if (d.url != null) {
                                    m.url = String(d.url);
                                }
                                if (d.keywords) {
                                    if (!Array.isArray(d.keywords))
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordAndUrlSeed.keywords: array expected");
                                    m.keywords = [];
                                    for (var i = 0; i < d.keywords.length; ++i) {
                                        m.keywords[i] = String(d.keywords[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordAndUrlSeed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordAndUrlSeed} m KeywordAndUrlSeed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordAndUrlSeed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.keywords = [];
                                }
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    d.url = m.url;
                                    if (o.oneofs)
                                        d._url = "url";
                                }
                                if (m.keywords && m.keywords.length) {
                                    d.keywords = [];
                                    for (var j = 0; j < m.keywords.length; ++j) {
                                        d.keywords[j] = m.keywords[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordAndUrlSeed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordAndUrlSeed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordAndUrlSeed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordAndUrlSeed;
                        })();
    
                        services.KeywordSeed = (function() {
    
                            /**
                             * Properties of a KeywordSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IKeywordSeed
                             * @property {Array.<string>|null} [keywords] KeywordSeed keywords
                             */
    
                            /**
                             * Constructs a new KeywordSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordSeed.
                             * @implements IKeywordSeed
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IKeywordSeed=} [p] Properties to set
                             */
                            function KeywordSeed(p) {
                                this.keywords = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * KeywordSeed keywords.
                             * @member {Array.<string>} keywords
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @instance
                             */
                            KeywordSeed.prototype.keywords = $util.emptyArray;
    
                            /**
                             * Creates a new KeywordSeed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordSeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.KeywordSeed} KeywordSeed instance
                             */
                            KeywordSeed.create = function create(properties) {
                                return new KeywordSeed(properties);
                            };
    
                            /**
                             * Encodes the specified KeywordSeed message. Does not implicitly {@link google.ads.googleads.v11.services.KeywordSeed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordSeed} m KeywordSeed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordSeed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keywords != null && m.keywords.length) {
                                    for (var i = 0; i < m.keywords.length; ++i)
                                        w.uint32(18).string(m.keywords[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified KeywordSeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.KeywordSeed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IKeywordSeed} message KeywordSeed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            KeywordSeed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a KeywordSeed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.KeywordSeed} KeywordSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordSeed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.KeywordSeed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        if (!(m.keywords && m.keywords.length))
                                            m.keywords = [];
                                        m.keywords.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a KeywordSeed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.KeywordSeed} KeywordSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            KeywordSeed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a KeywordSeed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            KeywordSeed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keywords != null && m.hasOwnProperty("keywords")) {
                                    if (!Array.isArray(m.keywords))
                                        return "keywords: array expected";
                                    for (var i = 0; i < m.keywords.length; ++i) {
                                        if (!$util.isString(m.keywords[i]))
                                            return "keywords: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a KeywordSeed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.KeywordSeed} KeywordSeed
                             */
                            KeywordSeed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.KeywordSeed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.KeywordSeed();
                                if (d.keywords) {
                                    if (!Array.isArray(d.keywords))
                                        throw TypeError(".google.ads.googleads.v11.services.KeywordSeed.keywords: array expected");
                                    m.keywords = [];
                                    for (var i = 0; i < d.keywords.length; ++i) {
                                        m.keywords[i] = String(d.keywords[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a KeywordSeed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.KeywordSeed} m KeywordSeed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            KeywordSeed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.keywords = [];
                                }
                                if (m.keywords && m.keywords.length) {
                                    d.keywords = [];
                                    for (var j = 0; j < m.keywords.length; ++j) {
                                        d.keywords[j] = m.keywords[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this KeywordSeed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.KeywordSeed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            KeywordSeed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return KeywordSeed;
                        })();
    
                        services.SiteSeed = (function() {
    
                            /**
                             * Properties of a SiteSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISiteSeed
                             * @property {string|null} [site] SiteSeed site
                             */
    
                            /**
                             * Constructs a new SiteSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SiteSeed.
                             * @implements ISiteSeed
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISiteSeed=} [p] Properties to set
                             */
                            function SiteSeed(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SiteSeed site.
                             * @member {string|null|undefined} site
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @instance
                             */
                            SiteSeed.prototype.site = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SiteSeed _site.
                             * @member {"site"|undefined} _site
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @instance
                             */
                            Object.defineProperty(SiteSeed.prototype, "_site", {
                                get: $util.oneOfGetter($oneOfFields = ["site"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SiteSeed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.ISiteSeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SiteSeed} SiteSeed instance
                             */
                            SiteSeed.create = function create(properties) {
                                return new SiteSeed(properties);
                            };
    
                            /**
                             * Encodes the specified SiteSeed message. Does not implicitly {@link google.ads.googleads.v11.services.SiteSeed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.ISiteSeed} m SiteSeed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SiteSeed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.site != null && Object.hasOwnProperty.call(m, "site"))
                                    w.uint32(18).string(m.site);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SiteSeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SiteSeed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.ISiteSeed} message SiteSeed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SiteSeed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SiteSeed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SiteSeed} SiteSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SiteSeed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SiteSeed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.site = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SiteSeed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SiteSeed} SiteSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SiteSeed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SiteSeed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SiteSeed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.site != null && m.hasOwnProperty("site")) {
                                    p._site = 1;
                                    if (!$util.isString(m.site))
                                        return "site: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SiteSeed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SiteSeed} SiteSeed
                             */
                            SiteSeed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SiteSeed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SiteSeed();
                                if (d.site != null) {
                                    m.site = String(d.site);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SiteSeed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.SiteSeed} m SiteSeed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SiteSeed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.site != null && m.hasOwnProperty("site")) {
                                    d.site = m.site;
                                    if (o.oneofs)
                                        d._site = "site";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SiteSeed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SiteSeed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SiteSeed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SiteSeed;
                        })();
    
                        services.UrlSeed = (function() {
    
                            /**
                             * Properties of an UrlSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUrlSeed
                             * @property {string|null} [url] UrlSeed url
                             */
    
                            /**
                             * Constructs a new UrlSeed.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UrlSeed.
                             * @implements IUrlSeed
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUrlSeed=} [p] Properties to set
                             */
                            function UrlSeed(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UrlSeed url.
                             * @member {string|null|undefined} url
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @instance
                             */
                            UrlSeed.prototype.url = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UrlSeed _url.
                             * @member {"url"|undefined} _url
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @instance
                             */
                            Object.defineProperty(UrlSeed.prototype, "_url", {
                                get: $util.oneOfGetter($oneOfFields = ["url"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UrlSeed instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IUrlSeed=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UrlSeed} UrlSeed instance
                             */
                            UrlSeed.create = function create(properties) {
                                return new UrlSeed(properties);
                            };
    
                            /**
                             * Encodes the specified UrlSeed message. Does not implicitly {@link google.ads.googleads.v11.services.UrlSeed.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IUrlSeed} m UrlSeed message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UrlSeed.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.url != null && Object.hasOwnProperty.call(m, "url"))
                                    w.uint32(18).string(m.url);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UrlSeed message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UrlSeed.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.IUrlSeed} message UrlSeed message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UrlSeed.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UrlSeed message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UrlSeed} UrlSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UrlSeed.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UrlSeed();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.url = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UrlSeed message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UrlSeed} UrlSeed
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UrlSeed.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UrlSeed message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UrlSeed.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    p._url = 1;
                                    if (!$util.isString(m.url))
                                        return "url: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UrlSeed message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UrlSeed} UrlSeed
                             */
                            UrlSeed.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UrlSeed)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UrlSeed();
                                if (d.url != null) {
                                    m.url = String(d.url);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UrlSeed message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @static
                             * @param {google.ads.googleads.v11.services.UrlSeed} m UrlSeed
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UrlSeed.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.url != null && m.hasOwnProperty("url")) {
                                    d.url = m.url;
                                    if (o.oneofs)
                                        d._url = "url";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UrlSeed to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UrlSeed
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UrlSeed.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UrlSeed;
                        })();
    
                        services.GenerateKeywordIdeaResponse = (function() {
    
                            /**
                             * Properties of a GenerateKeywordIdeaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateKeywordIdeaResponse
                             * @property {Array.<google.ads.googleads.v11.services.IGenerateKeywordIdeaResult>|null} [results] GenerateKeywordIdeaResponse results
                             * @property {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults|null} [aggregate_metric_results] GenerateKeywordIdeaResponse aggregate_metric_results
                             * @property {string|null} [next_page_token] GenerateKeywordIdeaResponse next_page_token
                             * @property {number|null} [total_size] GenerateKeywordIdeaResponse total_size
                             */
    
                            /**
                             * Constructs a new GenerateKeywordIdeaResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateKeywordIdeaResponse.
                             * @implements IGenerateKeywordIdeaResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResponse=} [p] Properties to set
                             */
                            function GenerateKeywordIdeaResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateKeywordIdeaResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IGenerateKeywordIdeaResult>} results
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @instance
                             */
                            GenerateKeywordIdeaResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * GenerateKeywordIdeaResponse aggregate_metric_results.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults|null|undefined} aggregate_metric_results
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @instance
                             */
                            GenerateKeywordIdeaResponse.prototype.aggregate_metric_results = null;
    
                            /**
                             * GenerateKeywordIdeaResponse next_page_token.
                             * @member {string} next_page_token
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @instance
                             */
                            GenerateKeywordIdeaResponse.prototype.next_page_token = "";
    
                            /**
                             * GenerateKeywordIdeaResponse total_size.
                             * @member {number} total_size
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @instance
                             */
                            GenerateKeywordIdeaResponse.prototype.total_size = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * Creates a new GenerateKeywordIdeaResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResponse} GenerateKeywordIdeaResponse instance
                             */
                            GenerateKeywordIdeaResponse.create = function create(properties) {
                                return new GenerateKeywordIdeaResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordIdeaResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordIdeaResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResponse} m GenerateKeywordIdeaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordIdeaResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                                    w.uint32(18).string(m.next_page_token);
                                if (m.total_size != null && Object.hasOwnProperty.call(m, "total_size"))
                                    w.uint32(24).int64(m.total_size);
                                if (m.aggregate_metric_results != null && Object.hasOwnProperty.call(m, "aggregate_metric_results"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.encode(m.aggregate_metric_results, w.uint32(34).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordIdeaResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordIdeaResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResponse} message GenerateKeywordIdeaResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordIdeaResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateKeywordIdeaResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResponse} GenerateKeywordIdeaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordIdeaResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        m.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.next_page_token = r.string();
                                        break;
                                    case 3:
                                        m.total_size = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateKeywordIdeaResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResponse} GenerateKeywordIdeaResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordIdeaResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateKeywordIdeaResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateKeywordIdeaResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.aggregate_metric_results != null && m.hasOwnProperty("aggregate_metric_results")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.verify(m.aggregate_metric_results);
                                        if (e)
                                            return "aggregate_metric_results." + e;
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    if (!$util.isString(m.next_page_token))
                                        return "next_page_token: string expected";
                                }
                                if (m.total_size != null && m.hasOwnProperty("total_size")) {
                                    if (!$util.isInteger(m.total_size) && !(m.total_size && $util.isInteger(m.total_size.low) && $util.isInteger(m.total_size.high)))
                                        return "total_size: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateKeywordIdeaResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResponse} GenerateKeywordIdeaResponse
                             */
                            GenerateKeywordIdeaResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeaResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeaResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.aggregate_metric_results != null) {
                                    if (typeof d.aggregate_metric_results !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeaResponse.aggregate_metric_results: object expected");
                                    m.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.fromObject(d.aggregate_metric_results);
                                }
                                if (d.next_page_token != null) {
                                    m.next_page_token = String(d.next_page_token);
                                }
                                if (d.total_size != null) {
                                    if ($util.Long)
                                        (m.total_size = $util.Long.fromValue(d.total_size)).unsigned = false;
                                    else if (typeof d.total_size === "string")
                                        m.total_size = parseInt(d.total_size, 10);
                                    else if (typeof d.total_size === "number")
                                        m.total_size = d.total_size;
                                    else if (typeof d.total_size === "object")
                                        m.total_size = new $util.LongBits(d.total_size.low >>> 0, d.total_size.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateKeywordIdeaResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateKeywordIdeaResponse} m GenerateKeywordIdeaResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateKeywordIdeaResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.next_page_token = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_size = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_size = o.longs === String ? "0" : 0;
                                    d.aggregate_metric_results = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                                    d.next_page_token = m.next_page_token;
                                }
                                if (m.total_size != null && m.hasOwnProperty("total_size")) {
                                    if (typeof m.total_size === "number")
                                        d.total_size = o.longs === String ? String(m.total_size) : m.total_size;
                                    else
                                        d.total_size = o.longs === String ? $util.Long.prototype.toString.call(m.total_size) : o.longs === Number ? new $util.LongBits(m.total_size.low >>> 0, m.total_size.high >>> 0).toNumber() : m.total_size;
                                }
                                if (m.aggregate_metric_results != null && m.hasOwnProperty("aggregate_metric_results")) {
                                    d.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.toObject(m.aggregate_metric_results, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateKeywordIdeaResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateKeywordIdeaResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateKeywordIdeaResponse;
                        })();
    
                        services.GenerateKeywordIdeaResult = (function() {
    
                            /**
                             * Properties of a GenerateKeywordIdeaResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateKeywordIdeaResult
                             * @property {string|null} [text] GenerateKeywordIdeaResult text
                             * @property {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics|null} [keyword_idea_metrics] GenerateKeywordIdeaResult keyword_idea_metrics
                             * @property {google.ads.googleads.v11.common.IKeywordAnnotations|null} [keyword_annotations] GenerateKeywordIdeaResult keyword_annotations
                             * @property {Array.<string>|null} [close_variants] GenerateKeywordIdeaResult close_variants
                             */
    
                            /**
                             * Constructs a new GenerateKeywordIdeaResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateKeywordIdeaResult.
                             * @implements IGenerateKeywordIdeaResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResult=} [p] Properties to set
                             */
                            function GenerateKeywordIdeaResult(p) {
                                this.close_variants = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateKeywordIdeaResult text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @instance
                             */
                            GenerateKeywordIdeaResult.prototype.text = null;
    
                            /**
                             * GenerateKeywordIdeaResult keyword_idea_metrics.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics|null|undefined} keyword_idea_metrics
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @instance
                             */
                            GenerateKeywordIdeaResult.prototype.keyword_idea_metrics = null;
    
                            /**
                             * GenerateKeywordIdeaResult keyword_annotations.
                             * @member {google.ads.googleads.v11.common.IKeywordAnnotations|null|undefined} keyword_annotations
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @instance
                             */
                            GenerateKeywordIdeaResult.prototype.keyword_annotations = null;
    
                            /**
                             * GenerateKeywordIdeaResult close_variants.
                             * @member {Array.<string>} close_variants
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @instance
                             */
                            GenerateKeywordIdeaResult.prototype.close_variants = $util.emptyArray;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GenerateKeywordIdeaResult _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @instance
                             */
                            Object.defineProperty(GenerateKeywordIdeaResult.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GenerateKeywordIdeaResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResult} GenerateKeywordIdeaResult instance
                             */
                            GenerateKeywordIdeaResult.create = function create(properties) {
                                return new GenerateKeywordIdeaResult(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordIdeaResult message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordIdeaResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResult} m GenerateKeywordIdeaResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordIdeaResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_idea_metrics != null && Object.hasOwnProperty.call(m, "keyword_idea_metrics"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.encode(m.keyword_idea_metrics, w.uint32(26).fork()).ldelim();
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(42).string(m.text);
                                if (m.keyword_annotations != null && Object.hasOwnProperty.call(m, "keyword_annotations"))
                                    $root.google.ads.googleads.v11.common.KeywordAnnotations.encode(m.keyword_annotations, w.uint32(50).fork()).ldelim();
                                if (m.close_variants != null && m.close_variants.length) {
                                    for (var i = 0; i < m.close_variants.length; ++i)
                                        w.uint32(58).string(m.close_variants[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordIdeaResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordIdeaResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordIdeaResult} message GenerateKeywordIdeaResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordIdeaResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateKeywordIdeaResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResult} GenerateKeywordIdeaResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordIdeaResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.text = r.string();
                                        break;
                                    case 3:
                                        m.keyword_idea_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.keyword_annotations = $root.google.ads.googleads.v11.common.KeywordAnnotations.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        if (!(m.close_variants && m.close_variants.length))
                                            m.close_variants = [];
                                        m.close_variants.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateKeywordIdeaResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResult} GenerateKeywordIdeaResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordIdeaResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateKeywordIdeaResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateKeywordIdeaResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.keyword_idea_metrics != null && m.hasOwnProperty("keyword_idea_metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.verify(m.keyword_idea_metrics);
                                        if (e)
                                            return "keyword_idea_metrics." + e;
                                    }
                                }
                                if (m.keyword_annotations != null && m.hasOwnProperty("keyword_annotations")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordAnnotations.verify(m.keyword_annotations);
                                        if (e)
                                            return "keyword_annotations." + e;
                                    }
                                }
                                if (m.close_variants != null && m.hasOwnProperty("close_variants")) {
                                    if (!Array.isArray(m.close_variants))
                                        return "close_variants: array expected";
                                    for (var i = 0; i < m.close_variants.length; ++i) {
                                        if (!$util.isString(m.close_variants[i]))
                                            return "close_variants: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateKeywordIdeaResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordIdeaResult} GenerateKeywordIdeaResult
                             */
                            GenerateKeywordIdeaResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateKeywordIdeaResult();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                if (d.keyword_idea_metrics != null) {
                                    if (typeof d.keyword_idea_metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeaResult.keyword_idea_metrics: object expected");
                                    m.keyword_idea_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.fromObject(d.keyword_idea_metrics);
                                }
                                if (d.keyword_annotations != null) {
                                    if (typeof d.keyword_annotations !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeaResult.keyword_annotations: object expected");
                                    m.keyword_annotations = $root.google.ads.googleads.v11.common.KeywordAnnotations.fromObject(d.keyword_annotations);
                                }
                                if (d.close_variants) {
                                    if (!Array.isArray(d.close_variants))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordIdeaResult.close_variants: array expected");
                                    m.close_variants = [];
                                    for (var i = 0; i < d.close_variants.length; ++i) {
                                        m.close_variants[i] = String(d.close_variants[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateKeywordIdeaResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateKeywordIdeaResult} m GenerateKeywordIdeaResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateKeywordIdeaResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.close_variants = [];
                                }
                                if (o.defaults) {
                                    d.keyword_idea_metrics = null;
                                    d.keyword_annotations = null;
                                }
                                if (m.keyword_idea_metrics != null && m.hasOwnProperty("keyword_idea_metrics")) {
                                    d.keyword_idea_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.toObject(m.keyword_idea_metrics, o);
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.keyword_annotations != null && m.hasOwnProperty("keyword_annotations")) {
                                    d.keyword_annotations = $root.google.ads.googleads.v11.common.KeywordAnnotations.toObject(m.keyword_annotations, o);
                                }
                                if (m.close_variants && m.close_variants.length) {
                                    d.close_variants = [];
                                    for (var j = 0; j < m.close_variants.length; ++j) {
                                        d.close_variants[j] = m.close_variants[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateKeywordIdeaResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordIdeaResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateKeywordIdeaResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateKeywordIdeaResult;
                        })();
    
                        services.GenerateKeywordHistoricalMetricsRequest = (function() {
    
                            /**
                             * Properties of a GenerateKeywordHistoricalMetricsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateKeywordHistoricalMetricsRequest
                             * @property {string|null} [customer_id] GenerateKeywordHistoricalMetricsRequest customer_id
                             * @property {Array.<string>|null} [keywords] GenerateKeywordHistoricalMetricsRequest keywords
                             * @property {string|null} [language] GenerateKeywordHistoricalMetricsRequest language
                             * @property {boolean|null} [include_adult_keywords] GenerateKeywordHistoricalMetricsRequest include_adult_keywords
                             * @property {Array.<string>|null} [geo_target_constants] GenerateKeywordHistoricalMetricsRequest geo_target_constants
                             * @property {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork|null} [keyword_plan_network] GenerateKeywordHistoricalMetricsRequest keyword_plan_network
                             * @property {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics|null} [aggregate_metrics] GenerateKeywordHistoricalMetricsRequest aggregate_metrics
                             * @property {google.ads.googleads.v11.common.IHistoricalMetricsOptions|null} [historical_metrics_options] GenerateKeywordHistoricalMetricsRequest historical_metrics_options
                             */
    
                            /**
                             * Constructs a new GenerateKeywordHistoricalMetricsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateKeywordHistoricalMetricsRequest.
                             * @implements IGenerateKeywordHistoricalMetricsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsRequest=} [p] Properties to set
                             */
                            function GenerateKeywordHistoricalMetricsRequest(p) {
                                this.keywords = [];
                                this.geo_target_constants = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.customer_id = "";
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest keywords.
                             * @member {Array.<string>} keywords
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.keywords = $util.emptyArray;
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest language.
                             * @member {string|null|undefined} language
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.language = null;
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest include_adult_keywords.
                             * @member {boolean} include_adult_keywords
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.include_adult_keywords = false;
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest geo_target_constants.
                             * @member {Array.<string>} geo_target_constants
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.geo_target_constants = $util.emptyArray;
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest keyword_plan_network.
                             * @member {google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork} keyword_plan_network
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.keyword_plan_network = 0;
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest aggregate_metrics.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanAggregateMetrics|null|undefined} aggregate_metrics
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.aggregate_metrics = null;
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest historical_metrics_options.
                             * @member {google.ads.googleads.v11.common.IHistoricalMetricsOptions|null|undefined} historical_metrics_options
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.historical_metrics_options = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GenerateKeywordHistoricalMetricsRequest _language.
                             * @member {"language"|undefined} _language
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             */
                            Object.defineProperty(GenerateKeywordHistoricalMetricsRequest.prototype, "_language", {
                                get: $util.oneOfGetter($oneOfFields = ["language"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GenerateKeywordHistoricalMetricsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest} GenerateKeywordHistoricalMetricsRequest instance
                             */
                            GenerateKeywordHistoricalMetricsRequest.create = function create(properties) {
                                return new GenerateKeywordHistoricalMetricsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordHistoricalMetricsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsRequest} m GenerateKeywordHistoricalMetricsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordHistoricalMetricsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.keywords != null && m.keywords.length) {
                                    for (var i = 0; i < m.keywords.length; ++i)
                                        w.uint32(18).string(m.keywords[i]);
                                }
                                if (m.historical_metrics_options != null && Object.hasOwnProperty.call(m, "historical_metrics_options"))
                                    $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.encode(m.historical_metrics_options, w.uint32(26).fork()).ldelim();
                                if (m.language != null && Object.hasOwnProperty.call(m, "language"))
                                    w.uint32(34).string(m.language);
                                if (m.include_adult_keywords != null && Object.hasOwnProperty.call(m, "include_adult_keywords"))
                                    w.uint32(40).bool(m.include_adult_keywords);
                                if (m.geo_target_constants != null && m.geo_target_constants.length) {
                                    for (var i = 0; i < m.geo_target_constants.length; ++i)
                                        w.uint32(50).string(m.geo_target_constants[i]);
                                }
                                if (m.keyword_plan_network != null && Object.hasOwnProperty.call(m, "keyword_plan_network"))
                                    w.uint32(56).int32(m.keyword_plan_network);
                                if (m.aggregate_metrics != null && Object.hasOwnProperty.call(m, "aggregate_metrics"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.encode(m.aggregate_metrics, w.uint32(66).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordHistoricalMetricsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsRequest} message GenerateKeywordHistoricalMetricsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordHistoricalMetricsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateKeywordHistoricalMetricsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest} GenerateKeywordHistoricalMetricsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordHistoricalMetricsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.keywords && m.keywords.length))
                                            m.keywords = [];
                                        m.keywords.push(r.string());
                                        break;
                                    case 4:
                                        m.language = r.string();
                                        break;
                                    case 5:
                                        m.include_adult_keywords = r.bool();
                                        break;
                                    case 6:
                                        if (!(m.geo_target_constants && m.geo_target_constants.length))
                                            m.geo_target_constants = [];
                                        m.geo_target_constants.push(r.string());
                                        break;
                                    case 7:
                                        m.keyword_plan_network = r.int32();
                                        break;
                                    case 8:
                                        m.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateKeywordHistoricalMetricsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest} GenerateKeywordHistoricalMetricsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordHistoricalMetricsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateKeywordHistoricalMetricsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateKeywordHistoricalMetricsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.keywords != null && m.hasOwnProperty("keywords")) {
                                    if (!Array.isArray(m.keywords))
                                        return "keywords: array expected";
                                    for (var i = 0; i < m.keywords.length; ++i) {
                                        if (!$util.isString(m.keywords[i]))
                                            return "keywords: string[] expected";
                                    }
                                }
                                if (m.language != null && m.hasOwnProperty("language")) {
                                    p._language = 1;
                                    if (!$util.isString(m.language))
                                        return "language: string expected";
                                }
                                if (m.include_adult_keywords != null && m.hasOwnProperty("include_adult_keywords")) {
                                    if (typeof m.include_adult_keywords !== "boolean")
                                        return "include_adult_keywords: boolean expected";
                                }
                                if (m.geo_target_constants != null && m.hasOwnProperty("geo_target_constants")) {
                                    if (!Array.isArray(m.geo_target_constants))
                                        return "geo_target_constants: array expected";
                                    for (var i = 0; i < m.geo_target_constants.length; ++i) {
                                        if (!$util.isString(m.geo_target_constants[i]))
                                            return "geo_target_constants: string[] expected";
                                    }
                                }
                                if (m.keyword_plan_network != null && m.hasOwnProperty("keyword_plan_network")) {
                                    switch (m.keyword_plan_network) {
                                    default:
                                        return "keyword_plan_network: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                }
                                if (m.aggregate_metrics != null && m.hasOwnProperty("aggregate_metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.verify(m.aggregate_metrics);
                                        if (e)
                                            return "aggregate_metrics." + e;
                                    }
                                }
                                if (m.historical_metrics_options != null && m.hasOwnProperty("historical_metrics_options")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.verify(m.historical_metrics_options);
                                        if (e)
                                            return "historical_metrics_options." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateKeywordHistoricalMetricsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest} GenerateKeywordHistoricalMetricsRequest
                             */
                            GenerateKeywordHistoricalMetricsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.keywords) {
                                    if (!Array.isArray(d.keywords))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest.keywords: array expected");
                                    m.keywords = [];
                                    for (var i = 0; i < d.keywords.length; ++i) {
                                        m.keywords[i] = String(d.keywords[i]);
                                    }
                                }
                                if (d.language != null) {
                                    m.language = String(d.language);
                                }
                                if (d.include_adult_keywords != null) {
                                    m.include_adult_keywords = Boolean(d.include_adult_keywords);
                                }
                                if (d.geo_target_constants) {
                                    if (!Array.isArray(d.geo_target_constants))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest.geo_target_constants: array expected");
                                    m.geo_target_constants = [];
                                    for (var i = 0; i < d.geo_target_constants.length; ++i) {
                                        m.geo_target_constants[i] = String(d.geo_target_constants[i]);
                                    }
                                }
                                switch (d.keyword_plan_network) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.keyword_plan_network = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.keyword_plan_network = 1;
                                    break;
                                case "GOOGLE_SEARCH":
                                case 2:
                                    m.keyword_plan_network = 2;
                                    break;
                                case "GOOGLE_SEARCH_AND_PARTNERS":
                                case 3:
                                    m.keyword_plan_network = 3;
                                    break;
                                }
                                if (d.aggregate_metrics != null) {
                                    if (typeof d.aggregate_metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest.aggregate_metrics: object expected");
                                    m.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.fromObject(d.aggregate_metrics);
                                }
                                if (d.historical_metrics_options != null) {
                                    if (typeof d.historical_metrics_options !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest.historical_metrics_options: object expected");
                                    m.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.fromObject(d.historical_metrics_options);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateKeywordHistoricalMetricsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest} m GenerateKeywordHistoricalMetricsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateKeywordHistoricalMetricsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.keywords = [];
                                    d.geo_target_constants = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.historical_metrics_options = null;
                                    d.include_adult_keywords = false;
                                    d.keyword_plan_network = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.aggregate_metrics = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.keywords && m.keywords.length) {
                                    d.keywords = [];
                                    for (var j = 0; j < m.keywords.length; ++j) {
                                        d.keywords[j] = m.keywords[j];
                                    }
                                }
                                if (m.historical_metrics_options != null && m.hasOwnProperty("historical_metrics_options")) {
                                    d.historical_metrics_options = $root.google.ads.googleads.v11.common.HistoricalMetricsOptions.toObject(m.historical_metrics_options, o);
                                }
                                if (m.language != null && m.hasOwnProperty("language")) {
                                    d.language = m.language;
                                    if (o.oneofs)
                                        d._language = "language";
                                }
                                if (m.include_adult_keywords != null && m.hasOwnProperty("include_adult_keywords")) {
                                    d.include_adult_keywords = m.include_adult_keywords;
                                }
                                if (m.geo_target_constants && m.geo_target_constants.length) {
                                    d.geo_target_constants = [];
                                    for (var j = 0; j < m.geo_target_constants.length; ++j) {
                                        d.geo_target_constants[j] = m.geo_target_constants[j];
                                    }
                                }
                                if (m.keyword_plan_network != null && m.hasOwnProperty("keyword_plan_network")) {
                                    d.keyword_plan_network = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordPlanNetworkEnum.KeywordPlanNetwork[m.keyword_plan_network] : m.keyword_plan_network;
                                }
                                if (m.aggregate_metrics != null && m.hasOwnProperty("aggregate_metrics")) {
                                    d.aggregate_metrics = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetrics.toObject(m.aggregate_metrics, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateKeywordHistoricalMetricsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateKeywordHistoricalMetricsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateKeywordHistoricalMetricsRequest;
                        })();
    
                        services.GenerateKeywordHistoricalMetricsResponse = (function() {
    
                            /**
                             * Properties of a GenerateKeywordHistoricalMetricsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateKeywordHistoricalMetricsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResult>|null} [results] GenerateKeywordHistoricalMetricsResponse results
                             * @property {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults|null} [aggregate_metric_results] GenerateKeywordHistoricalMetricsResponse aggregate_metric_results
                             */
    
                            /**
                             * Constructs a new GenerateKeywordHistoricalMetricsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateKeywordHistoricalMetricsResponse.
                             * @implements IGenerateKeywordHistoricalMetricsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResponse=} [p] Properties to set
                             */
                            function GenerateKeywordHistoricalMetricsResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateKeywordHistoricalMetricsResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResult>} results
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * GenerateKeywordHistoricalMetricsResponse aggregate_metric_results.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanAggregateMetricResults|null|undefined} aggregate_metric_results
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsResponse.prototype.aggregate_metric_results = null;
    
                            /**
                             * Creates a new GenerateKeywordHistoricalMetricsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse} GenerateKeywordHistoricalMetricsResponse instance
                             */
                            GenerateKeywordHistoricalMetricsResponse.create = function create(properties) {
                                return new GenerateKeywordHistoricalMetricsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordHistoricalMetricsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResponse} m GenerateKeywordHistoricalMetricsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordHistoricalMetricsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.aggregate_metric_results != null && Object.hasOwnProperty.call(m, "aggregate_metric_results"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.encode(m.aggregate_metric_results, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordHistoricalMetricsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResponse} message GenerateKeywordHistoricalMetricsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordHistoricalMetricsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateKeywordHistoricalMetricsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse} GenerateKeywordHistoricalMetricsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordHistoricalMetricsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateKeywordHistoricalMetricsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse} GenerateKeywordHistoricalMetricsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordHistoricalMetricsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateKeywordHistoricalMetricsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateKeywordHistoricalMetricsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.aggregate_metric_results != null && m.hasOwnProperty("aggregate_metric_results")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.verify(m.aggregate_metric_results);
                                        if (e)
                                            return "aggregate_metric_results." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateKeywordHistoricalMetricsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse} GenerateKeywordHistoricalMetricsResponse
                             */
                            GenerateKeywordHistoricalMetricsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.aggregate_metric_results != null) {
                                    if (typeof d.aggregate_metric_results !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse.aggregate_metric_results: object expected");
                                    m.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.fromObject(d.aggregate_metric_results);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateKeywordHistoricalMetricsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse} m GenerateKeywordHistoricalMetricsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateKeywordHistoricalMetricsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.aggregate_metric_results = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.aggregate_metric_results != null && m.hasOwnProperty("aggregate_metric_results")) {
                                    d.aggregate_metric_results = $root.google.ads.googleads.v11.common.KeywordPlanAggregateMetricResults.toObject(m.aggregate_metric_results, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateKeywordHistoricalMetricsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateKeywordHistoricalMetricsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateKeywordHistoricalMetricsResponse;
                        })();
    
                        services.GenerateKeywordHistoricalMetricsResult = (function() {
    
                            /**
                             * Properties of a GenerateKeywordHistoricalMetricsResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateKeywordHistoricalMetricsResult
                             * @property {string|null} [text] GenerateKeywordHistoricalMetricsResult text
                             * @property {Array.<string>|null} [close_variants] GenerateKeywordHistoricalMetricsResult close_variants
                             * @property {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics|null} [keyword_metrics] GenerateKeywordHistoricalMetricsResult keyword_metrics
                             */
    
                            /**
                             * Constructs a new GenerateKeywordHistoricalMetricsResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateKeywordHistoricalMetricsResult.
                             * @implements IGenerateKeywordHistoricalMetricsResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResult=} [p] Properties to set
                             */
                            function GenerateKeywordHistoricalMetricsResult(p) {
                                this.close_variants = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateKeywordHistoricalMetricsResult text.
                             * @member {string|null|undefined} text
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsResult.prototype.text = null;
    
                            /**
                             * GenerateKeywordHistoricalMetricsResult close_variants.
                             * @member {Array.<string>} close_variants
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsResult.prototype.close_variants = $util.emptyArray;
    
                            /**
                             * GenerateKeywordHistoricalMetricsResult keyword_metrics.
                             * @member {google.ads.googleads.v11.common.IKeywordPlanHistoricalMetrics|null|undefined} keyword_metrics
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @instance
                             */
                            GenerateKeywordHistoricalMetricsResult.prototype.keyword_metrics = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GenerateKeywordHistoricalMetricsResult _text.
                             * @member {"text"|undefined} _text
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @instance
                             */
                            Object.defineProperty(GenerateKeywordHistoricalMetricsResult.prototype, "_text", {
                                get: $util.oneOfGetter($oneOfFields = ["text"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GenerateKeywordHistoricalMetricsResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult} GenerateKeywordHistoricalMetricsResult instance
                             */
                            GenerateKeywordHistoricalMetricsResult.create = function create(properties) {
                                return new GenerateKeywordHistoricalMetricsResult(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordHistoricalMetricsResult message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResult} m GenerateKeywordHistoricalMetricsResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordHistoricalMetricsResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.text != null && Object.hasOwnProperty.call(m, "text"))
                                    w.uint32(10).string(m.text);
                                if (m.keyword_metrics != null && Object.hasOwnProperty.call(m, "keyword_metrics"))
                                    $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.encode(m.keyword_metrics, w.uint32(18).fork()).ldelim();
                                if (m.close_variants != null && m.close_variants.length) {
                                    for (var i = 0; i < m.close_variants.length; ++i)
                                        w.uint32(26).string(m.close_variants[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateKeywordHistoricalMetricsResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateKeywordHistoricalMetricsResult} message GenerateKeywordHistoricalMetricsResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateKeywordHistoricalMetricsResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateKeywordHistoricalMetricsResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult} GenerateKeywordHistoricalMetricsResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordHistoricalMetricsResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.text = r.string();
                                        break;
                                    case 3:
                                        if (!(m.close_variants && m.close_variants.length))
                                            m.close_variants = [];
                                        m.close_variants.push(r.string());
                                        break;
                                    case 2:
                                        m.keyword_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateKeywordHistoricalMetricsResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult} GenerateKeywordHistoricalMetricsResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateKeywordHistoricalMetricsResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateKeywordHistoricalMetricsResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateKeywordHistoricalMetricsResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    p._text = 1;
                                    if (!$util.isString(m.text))
                                        return "text: string expected";
                                }
                                if (m.close_variants != null && m.hasOwnProperty("close_variants")) {
                                    if (!Array.isArray(m.close_variants))
                                        return "close_variants: array expected";
                                    for (var i = 0; i < m.close_variants.length; ++i) {
                                        if (!$util.isString(m.close_variants[i]))
                                            return "close_variants: string[] expected";
                                    }
                                }
                                if (m.keyword_metrics != null && m.hasOwnProperty("keyword_metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.verify(m.keyword_metrics);
                                        if (e)
                                            return "keyword_metrics." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateKeywordHistoricalMetricsResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult} GenerateKeywordHistoricalMetricsResult
                             */
                            GenerateKeywordHistoricalMetricsResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult();
                                if (d.text != null) {
                                    m.text = String(d.text);
                                }
                                if (d.close_variants) {
                                    if (!Array.isArray(d.close_variants))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.close_variants: array expected");
                                    m.close_variants = [];
                                    for (var i = 0; i < d.close_variants.length; ++i) {
                                        m.close_variants[i] = String(d.close_variants[i]);
                                    }
                                }
                                if (d.keyword_metrics != null) {
                                    if (typeof d.keyword_metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult.keyword_metrics: object expected");
                                    m.keyword_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.fromObject(d.keyword_metrics);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateKeywordHistoricalMetricsResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult} m GenerateKeywordHistoricalMetricsResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateKeywordHistoricalMetricsResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.close_variants = [];
                                }
                                if (o.defaults) {
                                    d.keyword_metrics = null;
                                }
                                if (m.text != null && m.hasOwnProperty("text")) {
                                    d.text = m.text;
                                    if (o.oneofs)
                                        d._text = "text";
                                }
                                if (m.keyword_metrics != null && m.hasOwnProperty("keyword_metrics")) {
                                    d.keyword_metrics = $root.google.ads.googleads.v11.common.KeywordPlanHistoricalMetrics.toObject(m.keyword_metrics, o);
                                }
                                if (m.close_variants && m.close_variants.length) {
                                    d.close_variants = [];
                                    for (var j = 0; j < m.close_variants.length; ++j) {
                                        d.close_variants[j] = m.close_variants[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateKeywordHistoricalMetricsResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateKeywordHistoricalMetricsResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateKeywordHistoricalMetricsResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateKeywordHistoricalMetricsResult;
                        })();
    
                        services.GenerateAdGroupThemesRequest = (function() {
    
                            /**
                             * Properties of a GenerateAdGroupThemesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateAdGroupThemesRequest
                             * @property {string|null} [customer_id] GenerateAdGroupThemesRequest customer_id
                             * @property {Array.<string>|null} [keywords] GenerateAdGroupThemesRequest keywords
                             * @property {Array.<string>|null} [ad_groups] GenerateAdGroupThemesRequest ad_groups
                             */
    
                            /**
                             * Constructs a new GenerateAdGroupThemesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateAdGroupThemesRequest.
                             * @implements IGenerateAdGroupThemesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesRequest=} [p] Properties to set
                             */
                            function GenerateAdGroupThemesRequest(p) {
                                this.keywords = [];
                                this.ad_groups = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateAdGroupThemesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @instance
                             */
                            GenerateAdGroupThemesRequest.prototype.customer_id = "";
    
                            /**
                             * GenerateAdGroupThemesRequest keywords.
                             * @member {Array.<string>} keywords
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @instance
                             */
                            GenerateAdGroupThemesRequest.prototype.keywords = $util.emptyArray;
    
                            /**
                             * GenerateAdGroupThemesRequest ad_groups.
                             * @member {Array.<string>} ad_groups
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @instance
                             */
                            GenerateAdGroupThemesRequest.prototype.ad_groups = $util.emptyArray;
    
                            /**
                             * Creates a new GenerateAdGroupThemesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesRequest} GenerateAdGroupThemesRequest instance
                             */
                            GenerateAdGroupThemesRequest.create = function create(properties) {
                                return new GenerateAdGroupThemesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateAdGroupThemesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateAdGroupThemesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesRequest} m GenerateAdGroupThemesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateAdGroupThemesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.keywords != null && m.keywords.length) {
                                    for (var i = 0; i < m.keywords.length; ++i)
                                        w.uint32(18).string(m.keywords[i]);
                                }
                                if (m.ad_groups != null && m.ad_groups.length) {
                                    for (var i = 0; i < m.ad_groups.length; ++i)
                                        w.uint32(26).string(m.ad_groups[i]);
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateAdGroupThemesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateAdGroupThemesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesRequest} message GenerateAdGroupThemesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateAdGroupThemesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateAdGroupThemesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesRequest} GenerateAdGroupThemesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateAdGroupThemesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateAdGroupThemesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.keywords && m.keywords.length))
                                            m.keywords = [];
                                        m.keywords.push(r.string());
                                        break;
                                    case 3:
                                        if (!(m.ad_groups && m.ad_groups.length))
                                            m.ad_groups = [];
                                        m.ad_groups.push(r.string());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateAdGroupThemesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesRequest} GenerateAdGroupThemesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateAdGroupThemesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateAdGroupThemesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateAdGroupThemesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.keywords != null && m.hasOwnProperty("keywords")) {
                                    if (!Array.isArray(m.keywords))
                                        return "keywords: array expected";
                                    for (var i = 0; i < m.keywords.length; ++i) {
                                        if (!$util.isString(m.keywords[i]))
                                            return "keywords: string[] expected";
                                    }
                                }
                                if (m.ad_groups != null && m.hasOwnProperty("ad_groups")) {
                                    if (!Array.isArray(m.ad_groups))
                                        return "ad_groups: array expected";
                                    for (var i = 0; i < m.ad_groups.length; ++i) {
                                        if (!$util.isString(m.ad_groups[i]))
                                            return "ad_groups: string[] expected";
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateAdGroupThemesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesRequest} GenerateAdGroupThemesRequest
                             */
                            GenerateAdGroupThemesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateAdGroupThemesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateAdGroupThemesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.keywords) {
                                    if (!Array.isArray(d.keywords))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateAdGroupThemesRequest.keywords: array expected");
                                    m.keywords = [];
                                    for (var i = 0; i < d.keywords.length; ++i) {
                                        m.keywords[i] = String(d.keywords[i]);
                                    }
                                }
                                if (d.ad_groups) {
                                    if (!Array.isArray(d.ad_groups))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateAdGroupThemesRequest.ad_groups: array expected");
                                    m.ad_groups = [];
                                    for (var i = 0; i < d.ad_groups.length; ++i) {
                                        m.ad_groups[i] = String(d.ad_groups[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateAdGroupThemesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateAdGroupThemesRequest} m GenerateAdGroupThemesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateAdGroupThemesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.keywords = [];
                                    d.ad_groups = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.keywords && m.keywords.length) {
                                    d.keywords = [];
                                    for (var j = 0; j < m.keywords.length; ++j) {
                                        d.keywords[j] = m.keywords[j];
                                    }
                                }
                                if (m.ad_groups && m.ad_groups.length) {
                                    d.ad_groups = [];
                                    for (var j = 0; j < m.ad_groups.length; ++j) {
                                        d.ad_groups[j] = m.ad_groups[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateAdGroupThemesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateAdGroupThemesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateAdGroupThemesRequest;
                        })();
    
                        services.GenerateAdGroupThemesResponse = (function() {
    
                            /**
                             * Properties of a GenerateAdGroupThemesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateAdGroupThemesResponse
                             * @property {Array.<google.ads.googleads.v11.services.IAdGroupKeywordSuggestion>|null} [ad_group_keyword_suggestions] GenerateAdGroupThemesResponse ad_group_keyword_suggestions
                             * @property {Array.<google.ads.googleads.v11.services.IUnusableAdGroup>|null} [unusable_ad_groups] GenerateAdGroupThemesResponse unusable_ad_groups
                             */
    
                            /**
                             * Constructs a new GenerateAdGroupThemesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateAdGroupThemesResponse.
                             * @implements IGenerateAdGroupThemesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesResponse=} [p] Properties to set
                             */
                            function GenerateAdGroupThemesResponse(p) {
                                this.ad_group_keyword_suggestions = [];
                                this.unusable_ad_groups = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateAdGroupThemesResponse ad_group_keyword_suggestions.
                             * @member {Array.<google.ads.googleads.v11.services.IAdGroupKeywordSuggestion>} ad_group_keyword_suggestions
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @instance
                             */
                            GenerateAdGroupThemesResponse.prototype.ad_group_keyword_suggestions = $util.emptyArray;
    
                            /**
                             * GenerateAdGroupThemesResponse unusable_ad_groups.
                             * @member {Array.<google.ads.googleads.v11.services.IUnusableAdGroup>} unusable_ad_groups
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @instance
                             */
                            GenerateAdGroupThemesResponse.prototype.unusable_ad_groups = $util.emptyArray;
    
                            /**
                             * Creates a new GenerateAdGroupThemesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesResponse} GenerateAdGroupThemesResponse instance
                             */
                            GenerateAdGroupThemesResponse.create = function create(properties) {
                                return new GenerateAdGroupThemesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateAdGroupThemesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateAdGroupThemesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesResponse} m GenerateAdGroupThemesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateAdGroupThemesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_group_keyword_suggestions != null && m.ad_group_keyword_suggestions.length) {
                                    for (var i = 0; i < m.ad_group_keyword_suggestions.length; ++i)
                                        $root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion.encode(m.ad_group_keyword_suggestions[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.unusable_ad_groups != null && m.unusable_ad_groups.length) {
                                    for (var i = 0; i < m.unusable_ad_groups.length; ++i)
                                        $root.google.ads.googleads.v11.services.UnusableAdGroup.encode(m.unusable_ad_groups[i], w.uint32(18).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateAdGroupThemesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateAdGroupThemesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateAdGroupThemesResponse} message GenerateAdGroupThemesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateAdGroupThemesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateAdGroupThemesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesResponse} GenerateAdGroupThemesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateAdGroupThemesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateAdGroupThemesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.ad_group_keyword_suggestions && m.ad_group_keyword_suggestions.length))
                                            m.ad_group_keyword_suggestions = [];
                                        m.ad_group_keyword_suggestions.push($root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        if (!(m.unusable_ad_groups && m.unusable_ad_groups.length))
                                            m.unusable_ad_groups = [];
                                        m.unusable_ad_groups.push($root.google.ads.googleads.v11.services.UnusableAdGroup.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateAdGroupThemesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesResponse} GenerateAdGroupThemesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateAdGroupThemesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateAdGroupThemesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateAdGroupThemesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.ad_group_keyword_suggestions != null && m.hasOwnProperty("ad_group_keyword_suggestions")) {
                                    if (!Array.isArray(m.ad_group_keyword_suggestions))
                                        return "ad_group_keyword_suggestions: array expected";
                                    for (var i = 0; i < m.ad_group_keyword_suggestions.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion.verify(m.ad_group_keyword_suggestions[i]);
                                            if (e)
                                                return "ad_group_keyword_suggestions." + e;
                                        }
                                    }
                                }
                                if (m.unusable_ad_groups != null && m.hasOwnProperty("unusable_ad_groups")) {
                                    if (!Array.isArray(m.unusable_ad_groups))
                                        return "unusable_ad_groups: array expected";
                                    for (var i = 0; i < m.unusable_ad_groups.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.UnusableAdGroup.verify(m.unusable_ad_groups[i]);
                                            if (e)
                                                return "unusable_ad_groups." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateAdGroupThemesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateAdGroupThemesResponse} GenerateAdGroupThemesResponse
                             */
                            GenerateAdGroupThemesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateAdGroupThemesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateAdGroupThemesResponse();
                                if (d.ad_group_keyword_suggestions) {
                                    if (!Array.isArray(d.ad_group_keyword_suggestions))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateAdGroupThemesResponse.ad_group_keyword_suggestions: array expected");
                                    m.ad_group_keyword_suggestions = [];
                                    for (var i = 0; i < d.ad_group_keyword_suggestions.length; ++i) {
                                        if (typeof d.ad_group_keyword_suggestions[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateAdGroupThemesResponse.ad_group_keyword_suggestions: object expected");
                                        m.ad_group_keyword_suggestions[i] = $root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion.fromObject(d.ad_group_keyword_suggestions[i]);
                                    }
                                }
                                if (d.unusable_ad_groups) {
                                    if (!Array.isArray(d.unusable_ad_groups))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateAdGroupThemesResponse.unusable_ad_groups: array expected");
                                    m.unusable_ad_groups = [];
                                    for (var i = 0; i < d.unusable_ad_groups.length; ++i) {
                                        if (typeof d.unusable_ad_groups[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateAdGroupThemesResponse.unusable_ad_groups: object expected");
                                        m.unusable_ad_groups[i] = $root.google.ads.googleads.v11.services.UnusableAdGroup.fromObject(d.unusable_ad_groups[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateAdGroupThemesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateAdGroupThemesResponse} m GenerateAdGroupThemesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateAdGroupThemesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.ad_group_keyword_suggestions = [];
                                    d.unusable_ad_groups = [];
                                }
                                if (m.ad_group_keyword_suggestions && m.ad_group_keyword_suggestions.length) {
                                    d.ad_group_keyword_suggestions = [];
                                    for (var j = 0; j < m.ad_group_keyword_suggestions.length; ++j) {
                                        d.ad_group_keyword_suggestions[j] = $root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion.toObject(m.ad_group_keyword_suggestions[j], o);
                                    }
                                }
                                if (m.unusable_ad_groups && m.unusable_ad_groups.length) {
                                    d.unusable_ad_groups = [];
                                    for (var j = 0; j < m.unusable_ad_groups.length; ++j) {
                                        d.unusable_ad_groups[j] = $root.google.ads.googleads.v11.services.UnusableAdGroup.toObject(m.unusable_ad_groups[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateAdGroupThemesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateAdGroupThemesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateAdGroupThemesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateAdGroupThemesResponse;
                        })();
    
                        services.AdGroupKeywordSuggestion = (function() {
    
                            /**
                             * Properties of an AdGroupKeywordSuggestion.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAdGroupKeywordSuggestion
                             * @property {string|null} [keyword_text] AdGroupKeywordSuggestion keyword_text
                             * @property {string|null} [suggested_keyword_text] AdGroupKeywordSuggestion suggested_keyword_text
                             * @property {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType|null} [suggested_match_type] AdGroupKeywordSuggestion suggested_match_type
                             * @property {string|null} [suggested_ad_group] AdGroupKeywordSuggestion suggested_ad_group
                             * @property {string|null} [suggested_campaign] AdGroupKeywordSuggestion suggested_campaign
                             */
    
                            /**
                             * Constructs a new AdGroupKeywordSuggestion.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AdGroupKeywordSuggestion.
                             * @implements IAdGroupKeywordSuggestion
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAdGroupKeywordSuggestion=} [p] Properties to set
                             */
                            function AdGroupKeywordSuggestion(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AdGroupKeywordSuggestion keyword_text.
                             * @member {string} keyword_text
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @instance
                             */
                            AdGroupKeywordSuggestion.prototype.keyword_text = "";
    
                            /**
                             * AdGroupKeywordSuggestion suggested_keyword_text.
                             * @member {string} suggested_keyword_text
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @instance
                             */
                            AdGroupKeywordSuggestion.prototype.suggested_keyword_text = "";
    
                            /**
                             * AdGroupKeywordSuggestion suggested_match_type.
                             * @member {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType} suggested_match_type
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @instance
                             */
                            AdGroupKeywordSuggestion.prototype.suggested_match_type = 0;
    
                            /**
                             * AdGroupKeywordSuggestion suggested_ad_group.
                             * @member {string} suggested_ad_group
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @instance
                             */
                            AdGroupKeywordSuggestion.prototype.suggested_ad_group = "";
    
                            /**
                             * AdGroupKeywordSuggestion suggested_campaign.
                             * @member {string} suggested_campaign
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @instance
                             */
                            AdGroupKeywordSuggestion.prototype.suggested_campaign = "";
    
                            /**
                             * Creates a new AdGroupKeywordSuggestion instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupKeywordSuggestion=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AdGroupKeywordSuggestion} AdGroupKeywordSuggestion instance
                             */
                            AdGroupKeywordSuggestion.create = function create(properties) {
                                return new AdGroupKeywordSuggestion(properties);
                            };
    
                            /**
                             * Encodes the specified AdGroupKeywordSuggestion message. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupKeywordSuggestion.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupKeywordSuggestion} m AdGroupKeywordSuggestion message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupKeywordSuggestion.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_text != null && Object.hasOwnProperty.call(m, "keyword_text"))
                                    w.uint32(10).string(m.keyword_text);
                                if (m.suggested_keyword_text != null && Object.hasOwnProperty.call(m, "suggested_keyword_text"))
                                    w.uint32(18).string(m.suggested_keyword_text);
                                if (m.suggested_match_type != null && Object.hasOwnProperty.call(m, "suggested_match_type"))
                                    w.uint32(24).int32(m.suggested_match_type);
                                if (m.suggested_ad_group != null && Object.hasOwnProperty.call(m, "suggested_ad_group"))
                                    w.uint32(34).string(m.suggested_ad_group);
                                if (m.suggested_campaign != null && Object.hasOwnProperty.call(m, "suggested_campaign"))
                                    w.uint32(42).string(m.suggested_campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AdGroupKeywordSuggestion message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AdGroupKeywordSuggestion.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.IAdGroupKeywordSuggestion} message AdGroupKeywordSuggestion message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AdGroupKeywordSuggestion.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AdGroupKeywordSuggestion message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AdGroupKeywordSuggestion} AdGroupKeywordSuggestion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupKeywordSuggestion.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.keyword_text = r.string();
                                        break;
                                    case 2:
                                        m.suggested_keyword_text = r.string();
                                        break;
                                    case 3:
                                        m.suggested_match_type = r.int32();
                                        break;
                                    case 4:
                                        m.suggested_ad_group = r.string();
                                        break;
                                    case 5:
                                        m.suggested_campaign = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AdGroupKeywordSuggestion message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AdGroupKeywordSuggestion} AdGroupKeywordSuggestion
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AdGroupKeywordSuggestion.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AdGroupKeywordSuggestion message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AdGroupKeywordSuggestion.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keyword_text != null && m.hasOwnProperty("keyword_text")) {
                                    if (!$util.isString(m.keyword_text))
                                        return "keyword_text: string expected";
                                }
                                if (m.suggested_keyword_text != null && m.hasOwnProperty("suggested_keyword_text")) {
                                    if (!$util.isString(m.suggested_keyword_text))
                                        return "suggested_keyword_text: string expected";
                                }
                                if (m.suggested_match_type != null && m.hasOwnProperty("suggested_match_type")) {
                                    switch (m.suggested_match_type) {
                                    default:
                                        return "suggested_match_type: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.suggested_ad_group != null && m.hasOwnProperty("suggested_ad_group")) {
                                    if (!$util.isString(m.suggested_ad_group))
                                        return "suggested_ad_group: string expected";
                                }
                                if (m.suggested_campaign != null && m.hasOwnProperty("suggested_campaign")) {
                                    if (!$util.isString(m.suggested_campaign))
                                        return "suggested_campaign: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AdGroupKeywordSuggestion message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AdGroupKeywordSuggestion} AdGroupKeywordSuggestion
                             */
                            AdGroupKeywordSuggestion.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AdGroupKeywordSuggestion();
                                if (d.keyword_text != null) {
                                    m.keyword_text = String(d.keyword_text);
                                }
                                if (d.suggested_keyword_text != null) {
                                    m.suggested_keyword_text = String(d.suggested_keyword_text);
                                }
                                switch (d.suggested_match_type) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.suggested_match_type = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.suggested_match_type = 1;
                                    break;
                                case "EXACT":
                                case 2:
                                    m.suggested_match_type = 2;
                                    break;
                                case "PHRASE":
                                case 3:
                                    m.suggested_match_type = 3;
                                    break;
                                case "BROAD":
                                case 4:
                                    m.suggested_match_type = 4;
                                    break;
                                }
                                if (d.suggested_ad_group != null) {
                                    m.suggested_ad_group = String(d.suggested_ad_group);
                                }
                                if (d.suggested_campaign != null) {
                                    m.suggested_campaign = String(d.suggested_campaign);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AdGroupKeywordSuggestion message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @static
                             * @param {google.ads.googleads.v11.services.AdGroupKeywordSuggestion} m AdGroupKeywordSuggestion
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AdGroupKeywordSuggestion.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.keyword_text = "";
                                    d.suggested_keyword_text = "";
                                    d.suggested_match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.suggested_ad_group = "";
                                    d.suggested_campaign = "";
                                }
                                if (m.keyword_text != null && m.hasOwnProperty("keyword_text")) {
                                    d.keyword_text = m.keyword_text;
                                }
                                if (m.suggested_keyword_text != null && m.hasOwnProperty("suggested_keyword_text")) {
                                    d.suggested_keyword_text = m.suggested_keyword_text;
                                }
                                if (m.suggested_match_type != null && m.hasOwnProperty("suggested_match_type")) {
                                    d.suggested_match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType[m.suggested_match_type] : m.suggested_match_type;
                                }
                                if (m.suggested_ad_group != null && m.hasOwnProperty("suggested_ad_group")) {
                                    d.suggested_ad_group = m.suggested_ad_group;
                                }
                                if (m.suggested_campaign != null && m.hasOwnProperty("suggested_campaign")) {
                                    d.suggested_campaign = m.suggested_campaign;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AdGroupKeywordSuggestion to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AdGroupKeywordSuggestion
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AdGroupKeywordSuggestion.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AdGroupKeywordSuggestion;
                        })();
    
                        services.UnusableAdGroup = (function() {
    
                            /**
                             * Properties of an UnusableAdGroup.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUnusableAdGroup
                             * @property {string|null} [ad_group] UnusableAdGroup ad_group
                             * @property {string|null} [campaign] UnusableAdGroup campaign
                             */
    
                            /**
                             * Constructs a new UnusableAdGroup.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UnusableAdGroup.
                             * @implements IUnusableAdGroup
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUnusableAdGroup=} [p] Properties to set
                             */
                            function UnusableAdGroup(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UnusableAdGroup ad_group.
                             * @member {string} ad_group
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @instance
                             */
                            UnusableAdGroup.prototype.ad_group = "";
    
                            /**
                             * UnusableAdGroup campaign.
                             * @member {string} campaign
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @instance
                             */
                            UnusableAdGroup.prototype.campaign = "";
    
                            /**
                             * Creates a new UnusableAdGroup instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.services.IUnusableAdGroup=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UnusableAdGroup} UnusableAdGroup instance
                             */
                            UnusableAdGroup.create = function create(properties) {
                                return new UnusableAdGroup(properties);
                            };
    
                            /**
                             * Encodes the specified UnusableAdGroup message. Does not implicitly {@link google.ads.googleads.v11.services.UnusableAdGroup.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.services.IUnusableAdGroup} m UnusableAdGroup message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UnusableAdGroup.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                    w.uint32(10).string(m.ad_group);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UnusableAdGroup message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UnusableAdGroup.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.services.IUnusableAdGroup} message UnusableAdGroup message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UnusableAdGroup.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UnusableAdGroup message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UnusableAdGroup} UnusableAdGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UnusableAdGroup.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UnusableAdGroup();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.ad_group = r.string();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UnusableAdGroup message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UnusableAdGroup} UnusableAdGroup
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UnusableAdGroup.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UnusableAdGroup message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UnusableAdGroup.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    if (!$util.isString(m.ad_group))
                                        return "ad_group: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UnusableAdGroup message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UnusableAdGroup} UnusableAdGroup
                             */
                            UnusableAdGroup.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UnusableAdGroup)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UnusableAdGroup();
                                if (d.ad_group != null) {
                                    m.ad_group = String(d.ad_group);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UnusableAdGroup message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @static
                             * @param {google.ads.googleads.v11.services.UnusableAdGroup} m UnusableAdGroup
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UnusableAdGroup.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.ad_group = "";
                                    d.campaign = "";
                                }
                                if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                    d.ad_group = m.ad_group;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UnusableAdGroup to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UnusableAdGroup
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UnusableAdGroup.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UnusableAdGroup;
                        })();
    
                        services.KeywordThemeConstantService = (function() {
    
                            /**
                             * Constructs a new KeywordThemeConstantService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a KeywordThemeConstantService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function KeywordThemeConstantService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (KeywordThemeConstantService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = KeywordThemeConstantService;
    
                            /**
                             * Creates new KeywordThemeConstantService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.KeywordThemeConstantService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {KeywordThemeConstantService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            KeywordThemeConstantService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.KeywordThemeConstantService|suggestKeywordThemeConstants}.
                             * @memberof google.ads.googleads.v11.services.KeywordThemeConstantService
                             * @typedef SuggestKeywordThemeConstantsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse} [response] SuggestKeywordThemeConstantsResponse
                             */
    
                            /**
                             * Calls SuggestKeywordThemeConstants.
                             * @function suggestKeywordThemeConstants
                             * @memberof google.ads.googleads.v11.services.KeywordThemeConstantService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsRequest} request SuggestKeywordThemeConstantsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.KeywordThemeConstantService.SuggestKeywordThemeConstantsCallback} callback Node-style callback called with the error, if any, and SuggestKeywordThemeConstantsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(KeywordThemeConstantService.prototype.suggestKeywordThemeConstants = function suggestKeywordThemeConstants(request, callback) {
                                return this.rpcCall(suggestKeywordThemeConstants, $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest, $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse, request, callback);
                            }, "name", { value: "SuggestKeywordThemeConstants" });
    
                            /**
                             * Calls SuggestKeywordThemeConstants.
                             * @function suggestKeywordThemeConstants
                             * @memberof google.ads.googleads.v11.services.KeywordThemeConstantService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsRequest} request SuggestKeywordThemeConstantsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse>} Promise
                             * @variation 2
                             */
    
                            return KeywordThemeConstantService;
                        })();
    
                        services.SuggestKeywordThemeConstantsRequest = (function() {
    
                            /**
                             * Properties of a SuggestKeywordThemeConstantsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestKeywordThemeConstantsRequest
                             * @property {string|null} [query_text] SuggestKeywordThemeConstantsRequest query_text
                             * @property {string|null} [country_code] SuggestKeywordThemeConstantsRequest country_code
                             * @property {string|null} [language_code] SuggestKeywordThemeConstantsRequest language_code
                             */
    
                            /**
                             * Constructs a new SuggestKeywordThemeConstantsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestKeywordThemeConstantsRequest.
                             * @implements ISuggestKeywordThemeConstantsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsRequest=} [p] Properties to set
                             */
                            function SuggestKeywordThemeConstantsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestKeywordThemeConstantsRequest query_text.
                             * @member {string} query_text
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @instance
                             */
                            SuggestKeywordThemeConstantsRequest.prototype.query_text = "";
    
                            /**
                             * SuggestKeywordThemeConstantsRequest country_code.
                             * @member {string} country_code
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @instance
                             */
                            SuggestKeywordThemeConstantsRequest.prototype.country_code = "";
    
                            /**
                             * SuggestKeywordThemeConstantsRequest language_code.
                             * @member {string} language_code
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @instance
                             */
                            SuggestKeywordThemeConstantsRequest.prototype.language_code = "";
    
                            /**
                             * Creates a new SuggestKeywordThemeConstantsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest} SuggestKeywordThemeConstantsRequest instance
                             */
                            SuggestKeywordThemeConstantsRequest.create = function create(properties) {
                                return new SuggestKeywordThemeConstantsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemeConstantsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsRequest} m SuggestKeywordThemeConstantsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemeConstantsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.query_text != null && Object.hasOwnProperty.call(m, "query_text"))
                                    w.uint32(10).string(m.query_text);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(18).string(m.country_code);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(26).string(m.language_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemeConstantsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsRequest} message SuggestKeywordThemeConstantsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemeConstantsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemeConstantsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest} SuggestKeywordThemeConstantsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemeConstantsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.query_text = r.string();
                                        break;
                                    case 2:
                                        m.country_code = r.string();
                                        break;
                                    case 3:
                                        m.language_code = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemeConstantsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest} SuggestKeywordThemeConstantsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemeConstantsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestKeywordThemeConstantsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestKeywordThemeConstantsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.query_text != null && m.hasOwnProperty("query_text")) {
                                    if (!$util.isString(m.query_text))
                                        return "query_text: string expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestKeywordThemeConstantsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest} SuggestKeywordThemeConstantsRequest
                             */
                            SuggestKeywordThemeConstantsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest();
                                if (d.query_text != null) {
                                    m.query_text = String(d.query_text);
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestKeywordThemeConstantsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest} m SuggestKeywordThemeConstantsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestKeywordThemeConstantsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.query_text = "";
                                    d.country_code = "";
                                    d.language_code = "";
                                }
                                if (m.query_text != null && m.hasOwnProperty("query_text")) {
                                    d.query_text = m.query_text;
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestKeywordThemeConstantsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestKeywordThemeConstantsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestKeywordThemeConstantsRequest;
                        })();
    
                        services.SuggestKeywordThemeConstantsResponse = (function() {
    
                            /**
                             * Properties of a SuggestKeywordThemeConstantsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestKeywordThemeConstantsResponse
                             * @property {Array.<google.ads.googleads.v11.resources.IKeywordThemeConstant>|null} [keyword_theme_constants] SuggestKeywordThemeConstantsResponse keyword_theme_constants
                             */
    
                            /**
                             * Constructs a new SuggestKeywordThemeConstantsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestKeywordThemeConstantsResponse.
                             * @implements ISuggestKeywordThemeConstantsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsResponse=} [p] Properties to set
                             */
                            function SuggestKeywordThemeConstantsResponse(p) {
                                this.keyword_theme_constants = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestKeywordThemeConstantsResponse keyword_theme_constants.
                             * @member {Array.<google.ads.googleads.v11.resources.IKeywordThemeConstant>} keyword_theme_constants
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @instance
                             */
                            SuggestKeywordThemeConstantsResponse.prototype.keyword_theme_constants = $util.emptyArray;
    
                            /**
                             * Creates a new SuggestKeywordThemeConstantsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse} SuggestKeywordThemeConstantsResponse instance
                             */
                            SuggestKeywordThemeConstantsResponse.create = function create(properties) {
                                return new SuggestKeywordThemeConstantsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemeConstantsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsResponse} m SuggestKeywordThemeConstantsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemeConstantsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_theme_constants != null && m.keyword_theme_constants.length) {
                                    for (var i = 0; i < m.keyword_theme_constants.length; ++i)
                                        $root.google.ads.googleads.v11.resources.KeywordThemeConstant.encode(m.keyword_theme_constants[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemeConstantsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemeConstantsResponse} message SuggestKeywordThemeConstantsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemeConstantsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemeConstantsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse} SuggestKeywordThemeConstantsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemeConstantsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.keyword_theme_constants && m.keyword_theme_constants.length))
                                            m.keyword_theme_constants = [];
                                        m.keyword_theme_constants.push($root.google.ads.googleads.v11.resources.KeywordThemeConstant.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemeConstantsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse} SuggestKeywordThemeConstantsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemeConstantsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestKeywordThemeConstantsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestKeywordThemeConstantsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keyword_theme_constants != null && m.hasOwnProperty("keyword_theme_constants")) {
                                    if (!Array.isArray(m.keyword_theme_constants))
                                        return "keyword_theme_constants: array expected";
                                    for (var i = 0; i < m.keyword_theme_constants.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.verify(m.keyword_theme_constants[i]);
                                            if (e)
                                                return "keyword_theme_constants." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestKeywordThemeConstantsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse} SuggestKeywordThemeConstantsResponse
                             */
                            SuggestKeywordThemeConstantsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse();
                                if (d.keyword_theme_constants) {
                                    if (!Array.isArray(d.keyword_theme_constants))
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse.keyword_theme_constants: array expected");
                                    m.keyword_theme_constants = [];
                                    for (var i = 0; i < d.keyword_theme_constants.length; ++i) {
                                        if (typeof d.keyword_theme_constants[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse.keyword_theme_constants: object expected");
                                        m.keyword_theme_constants[i] = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.fromObject(d.keyword_theme_constants[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestKeywordThemeConstantsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse} m SuggestKeywordThemeConstantsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestKeywordThemeConstantsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.keyword_theme_constants = [];
                                }
                                if (m.keyword_theme_constants && m.keyword_theme_constants.length) {
                                    d.keyword_theme_constants = [];
                                    for (var j = 0; j < m.keyword_theme_constants.length; ++j) {
                                        d.keyword_theme_constants[j] = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.toObject(m.keyword_theme_constants[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestKeywordThemeConstantsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemeConstantsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestKeywordThemeConstantsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestKeywordThemeConstantsResponse;
                        })();
    
                        services.MerchantCenterLinkService = (function() {
    
                            /**
                             * Constructs a new MerchantCenterLinkService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MerchantCenterLinkService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function MerchantCenterLinkService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (MerchantCenterLinkService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = MerchantCenterLinkService;
    
                            /**
                             * Creates new MerchantCenterLinkService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {MerchantCenterLinkService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            MerchantCenterLinkService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.MerchantCenterLinkService|listMerchantCenterLinks}.
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @typedef ListMerchantCenterLinksCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListMerchantCenterLinksResponse} [response] ListMerchantCenterLinksResponse
                             */
    
                            /**
                             * Calls ListMerchantCenterLinks.
                             * @function listMerchantCenterLinks
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksRequest} request ListMerchantCenterLinksRequest message or plain object
                             * @param {google.ads.googleads.v11.services.MerchantCenterLinkService.ListMerchantCenterLinksCallback} callback Node-style callback called with the error, if any, and ListMerchantCenterLinksResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(MerchantCenterLinkService.prototype.listMerchantCenterLinks = function listMerchantCenterLinks(request, callback) {
                                return this.rpcCall(listMerchantCenterLinks, $root.google.ads.googleads.v11.services.ListMerchantCenterLinksRequest, $root.google.ads.googleads.v11.services.ListMerchantCenterLinksResponse, request, callback);
                            }, "name", { value: "ListMerchantCenterLinks" });
    
                            /**
                             * Calls ListMerchantCenterLinks.
                             * @function listMerchantCenterLinks
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksRequest} request ListMerchantCenterLinksRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListMerchantCenterLinksResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.MerchantCenterLinkService|getMerchantCenterLink}.
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @typedef GetMerchantCenterLinkCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.resources.MerchantCenterLink} [response] MerchantCenterLink
                             */
    
                            /**
                             * Calls GetMerchantCenterLink.
                             * @function getMerchantCenterLink
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGetMerchantCenterLinkRequest} request GetMerchantCenterLinkRequest message or plain object
                             * @param {google.ads.googleads.v11.services.MerchantCenterLinkService.GetMerchantCenterLinkCallback} callback Node-style callback called with the error, if any, and MerchantCenterLink
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(MerchantCenterLinkService.prototype.getMerchantCenterLink = function getMerchantCenterLink(request, callback) {
                                return this.rpcCall(getMerchantCenterLink, $root.google.ads.googleads.v11.services.GetMerchantCenterLinkRequest, $root.google.ads.googleads.v11.resources.MerchantCenterLink, request, callback);
                            }, "name", { value: "GetMerchantCenterLink" });
    
                            /**
                             * Calls GetMerchantCenterLink.
                             * @function getMerchantCenterLink
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGetMerchantCenterLinkRequest} request GetMerchantCenterLinkRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.resources.MerchantCenterLink>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.MerchantCenterLinkService|mutateMerchantCenterLink}.
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @typedef MutateMerchantCenterLinkCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse} [response] MutateMerchantCenterLinkResponse
                             */
    
                            /**
                             * Calls MutateMerchantCenterLink.
                             * @function mutateMerchantCenterLink
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkRequest} request MutateMerchantCenterLinkRequest message or plain object
                             * @param {google.ads.googleads.v11.services.MerchantCenterLinkService.MutateMerchantCenterLinkCallback} callback Node-style callback called with the error, if any, and MutateMerchantCenterLinkResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(MerchantCenterLinkService.prototype.mutateMerchantCenterLink = function mutateMerchantCenterLink(request, callback) {
                                return this.rpcCall(mutateMerchantCenterLink, $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest, $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse, request, callback);
                            }, "name", { value: "MutateMerchantCenterLink" });
    
                            /**
                             * Calls MutateMerchantCenterLink.
                             * @function mutateMerchantCenterLink
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkRequest} request MutateMerchantCenterLinkRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse>} Promise
                             * @variation 2
                             */
    
                            return MerchantCenterLinkService;
                        })();
    
                        services.ListMerchantCenterLinksRequest = (function() {
    
                            /**
                             * Properties of a ListMerchantCenterLinksRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListMerchantCenterLinksRequest
                             * @property {string|null} [customer_id] ListMerchantCenterLinksRequest customer_id
                             */
    
                            /**
                             * Constructs a new ListMerchantCenterLinksRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListMerchantCenterLinksRequest.
                             * @implements IListMerchantCenterLinksRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksRequest=} [p] Properties to set
                             */
                            function ListMerchantCenterLinksRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListMerchantCenterLinksRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @instance
                             */
                            ListMerchantCenterLinksRequest.prototype.customer_id = "";
    
                            /**
                             * Creates a new ListMerchantCenterLinksRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksRequest} ListMerchantCenterLinksRequest instance
                             */
                            ListMerchantCenterLinksRequest.create = function create(properties) {
                                return new ListMerchantCenterLinksRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListMerchantCenterLinksRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListMerchantCenterLinksRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksRequest} m ListMerchantCenterLinksRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListMerchantCenterLinksRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListMerchantCenterLinksRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListMerchantCenterLinksRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksRequest} message ListMerchantCenterLinksRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListMerchantCenterLinksRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListMerchantCenterLinksRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksRequest} ListMerchantCenterLinksRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListMerchantCenterLinksRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListMerchantCenterLinksRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListMerchantCenterLinksRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksRequest} ListMerchantCenterLinksRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListMerchantCenterLinksRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListMerchantCenterLinksRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListMerchantCenterLinksRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListMerchantCenterLinksRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksRequest} ListMerchantCenterLinksRequest
                             */
                            ListMerchantCenterLinksRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListMerchantCenterLinksRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListMerchantCenterLinksRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListMerchantCenterLinksRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListMerchantCenterLinksRequest} m ListMerchantCenterLinksRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListMerchantCenterLinksRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListMerchantCenterLinksRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListMerchantCenterLinksRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListMerchantCenterLinksRequest;
                        })();
    
                        services.ListMerchantCenterLinksResponse = (function() {
    
                            /**
                             * Properties of a ListMerchantCenterLinksResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListMerchantCenterLinksResponse
                             * @property {Array.<google.ads.googleads.v11.resources.IMerchantCenterLink>|null} [merchant_center_links] ListMerchantCenterLinksResponse merchant_center_links
                             */
    
                            /**
                             * Constructs a new ListMerchantCenterLinksResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListMerchantCenterLinksResponse.
                             * @implements IListMerchantCenterLinksResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksResponse=} [p] Properties to set
                             */
                            function ListMerchantCenterLinksResponse(p) {
                                this.merchant_center_links = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListMerchantCenterLinksResponse merchant_center_links.
                             * @member {Array.<google.ads.googleads.v11.resources.IMerchantCenterLink>} merchant_center_links
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @instance
                             */
                            ListMerchantCenterLinksResponse.prototype.merchant_center_links = $util.emptyArray;
    
                            /**
                             * Creates a new ListMerchantCenterLinksResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksResponse} ListMerchantCenterLinksResponse instance
                             */
                            ListMerchantCenterLinksResponse.create = function create(properties) {
                                return new ListMerchantCenterLinksResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListMerchantCenterLinksResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListMerchantCenterLinksResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksResponse} m ListMerchantCenterLinksResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListMerchantCenterLinksResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.merchant_center_links != null && m.merchant_center_links.length) {
                                    for (var i = 0; i < m.merchant_center_links.length; ++i)
                                        $root.google.ads.googleads.v11.resources.MerchantCenterLink.encode(m.merchant_center_links[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListMerchantCenterLinksResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListMerchantCenterLinksResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListMerchantCenterLinksResponse} message ListMerchantCenterLinksResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListMerchantCenterLinksResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListMerchantCenterLinksResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksResponse} ListMerchantCenterLinksResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListMerchantCenterLinksResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListMerchantCenterLinksResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.merchant_center_links && m.merchant_center_links.length))
                                            m.merchant_center_links = [];
                                        m.merchant_center_links.push($root.google.ads.googleads.v11.resources.MerchantCenterLink.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListMerchantCenterLinksResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksResponse} ListMerchantCenterLinksResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListMerchantCenterLinksResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListMerchantCenterLinksResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListMerchantCenterLinksResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.merchant_center_links != null && m.hasOwnProperty("merchant_center_links")) {
                                    if (!Array.isArray(m.merchant_center_links))
                                        return "merchant_center_links: array expected";
                                    for (var i = 0; i < m.merchant_center_links.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.MerchantCenterLink.verify(m.merchant_center_links[i]);
                                            if (e)
                                                return "merchant_center_links." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListMerchantCenterLinksResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListMerchantCenterLinksResponse} ListMerchantCenterLinksResponse
                             */
                            ListMerchantCenterLinksResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListMerchantCenterLinksResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListMerchantCenterLinksResponse();
                                if (d.merchant_center_links) {
                                    if (!Array.isArray(d.merchant_center_links))
                                        throw TypeError(".google.ads.googleads.v11.services.ListMerchantCenterLinksResponse.merchant_center_links: array expected");
                                    m.merchant_center_links = [];
                                    for (var i = 0; i < d.merchant_center_links.length; ++i) {
                                        if (typeof d.merchant_center_links[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListMerchantCenterLinksResponse.merchant_center_links: object expected");
                                        m.merchant_center_links[i] = $root.google.ads.googleads.v11.resources.MerchantCenterLink.fromObject(d.merchant_center_links[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListMerchantCenterLinksResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListMerchantCenterLinksResponse} m ListMerchantCenterLinksResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListMerchantCenterLinksResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.merchant_center_links = [];
                                }
                                if (m.merchant_center_links && m.merchant_center_links.length) {
                                    d.merchant_center_links = [];
                                    for (var j = 0; j < m.merchant_center_links.length; ++j) {
                                        d.merchant_center_links[j] = $root.google.ads.googleads.v11.resources.MerchantCenterLink.toObject(m.merchant_center_links[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListMerchantCenterLinksResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListMerchantCenterLinksResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListMerchantCenterLinksResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListMerchantCenterLinksResponse;
                        })();
    
                        services.GetMerchantCenterLinkRequest = (function() {
    
                            /**
                             * Properties of a GetMerchantCenterLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGetMerchantCenterLinkRequest
                             * @property {string|null} [resource_name] GetMerchantCenterLinkRequest resource_name
                             */
    
                            /**
                             * Constructs a new GetMerchantCenterLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GetMerchantCenterLinkRequest.
                             * @implements IGetMerchantCenterLinkRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGetMerchantCenterLinkRequest=} [p] Properties to set
                             */
                            function GetMerchantCenterLinkRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GetMerchantCenterLinkRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @instance
                             */
                            GetMerchantCenterLinkRequest.prototype.resource_name = "";
    
                            /**
                             * Creates a new GetMerchantCenterLinkRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetMerchantCenterLinkRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GetMerchantCenterLinkRequest} GetMerchantCenterLinkRequest instance
                             */
                            GetMerchantCenterLinkRequest.create = function create(properties) {
                                return new GetMerchantCenterLinkRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GetMerchantCenterLinkRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GetMerchantCenterLinkRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetMerchantCenterLinkRequest} m GetMerchantCenterLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GetMerchantCenterLinkRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GetMerchantCenterLinkRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GetMerchantCenterLinkRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGetMerchantCenterLinkRequest} message GetMerchantCenterLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GetMerchantCenterLinkRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GetMerchantCenterLinkRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GetMerchantCenterLinkRequest} GetMerchantCenterLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GetMerchantCenterLinkRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GetMerchantCenterLinkRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GetMerchantCenterLinkRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GetMerchantCenterLinkRequest} GetMerchantCenterLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GetMerchantCenterLinkRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GetMerchantCenterLinkRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GetMerchantCenterLinkRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GetMerchantCenterLinkRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GetMerchantCenterLinkRequest} GetMerchantCenterLinkRequest
                             */
                            GetMerchantCenterLinkRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GetMerchantCenterLinkRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GetMerchantCenterLinkRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GetMerchantCenterLinkRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GetMerchantCenterLinkRequest} m GetMerchantCenterLinkRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GetMerchantCenterLinkRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GetMerchantCenterLinkRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GetMerchantCenterLinkRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GetMerchantCenterLinkRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GetMerchantCenterLinkRequest;
                        })();
    
                        services.MutateMerchantCenterLinkRequest = (function() {
    
                            /**
                             * Properties of a MutateMerchantCenterLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateMerchantCenterLinkRequest
                             * @property {string|null} [customer_id] MutateMerchantCenterLinkRequest customer_id
                             * @property {google.ads.googleads.v11.services.IMerchantCenterLinkOperation|null} [operation] MutateMerchantCenterLinkRequest operation
                             * @property {boolean|null} [validate_only] MutateMerchantCenterLinkRequest validate_only
                             */
    
                            /**
                             * Constructs a new MutateMerchantCenterLinkRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateMerchantCenterLinkRequest.
                             * @implements IMutateMerchantCenterLinkRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkRequest=} [p] Properties to set
                             */
                            function MutateMerchantCenterLinkRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateMerchantCenterLinkRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @instance
                             */
                            MutateMerchantCenterLinkRequest.prototype.customer_id = "";
    
                            /**
                             * MutateMerchantCenterLinkRequest operation.
                             * @member {google.ads.googleads.v11.services.IMerchantCenterLinkOperation|null|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @instance
                             */
                            MutateMerchantCenterLinkRequest.prototype.operation = null;
    
                            /**
                             * MutateMerchantCenterLinkRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @instance
                             */
                            MutateMerchantCenterLinkRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new MutateMerchantCenterLinkRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest} MutateMerchantCenterLinkRequest instance
                             */
                            MutateMerchantCenterLinkRequest.create = function create(properties) {
                                return new MutateMerchantCenterLinkRequest(properties);
                            };
    
                            /**
                             * Encodes the specified MutateMerchantCenterLinkRequest message. Does not implicitly {@link google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkRequest} m MutateMerchantCenterLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMerchantCenterLinkRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operation != null && Object.hasOwnProperty.call(m, "operation"))
                                    $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation.encode(m.operation, w.uint32(18).fork()).ldelim();
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateMerchantCenterLinkRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkRequest} message MutateMerchantCenterLinkRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMerchantCenterLinkRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateMerchantCenterLinkRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest} MutateMerchantCenterLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMerchantCenterLinkRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.operation = $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateMerchantCenterLinkRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest} MutateMerchantCenterLinkRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMerchantCenterLinkRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateMerchantCenterLinkRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateMerchantCenterLinkRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation.verify(m.operation);
                                        if (e)
                                            return "operation." + e;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateMerchantCenterLinkRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest} MutateMerchantCenterLinkRequest
                             */
                            MutateMerchantCenterLinkRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operation != null) {
                                    if (typeof d.operation !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest.operation: object expected");
                                    m.operation = $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation.fromObject(d.operation);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateMerchantCenterLinkRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest} m MutateMerchantCenterLinkRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateMerchantCenterLinkRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.operation = null;
                                    d.validate_only = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operation != null && m.hasOwnProperty("operation")) {
                                    d.operation = $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation.toObject(m.operation, o);
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateMerchantCenterLinkRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateMerchantCenterLinkRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateMerchantCenterLinkRequest;
                        })();
    
                        services.MerchantCenterLinkOperation = (function() {
    
                            /**
                             * Properties of a MerchantCenterLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMerchantCenterLinkOperation
                             * @property {google.protobuf.IFieldMask|null} [update_mask] MerchantCenterLinkOperation update_mask
                             * @property {google.ads.googleads.v11.resources.IMerchantCenterLink|null} [update] MerchantCenterLinkOperation update
                             * @property {string|null} [remove] MerchantCenterLinkOperation remove
                             */
    
                            /**
                             * Constructs a new MerchantCenterLinkOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MerchantCenterLinkOperation.
                             * @implements IMerchantCenterLinkOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMerchantCenterLinkOperation=} [p] Properties to set
                             */
                            function MerchantCenterLinkOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MerchantCenterLinkOperation update_mask.
                             * @member {google.protobuf.IFieldMask|null|undefined} update_mask
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @instance
                             */
                            MerchantCenterLinkOperation.prototype.update_mask = null;
    
                            /**
                             * MerchantCenterLinkOperation update.
                             * @member {google.ads.googleads.v11.resources.IMerchantCenterLink|null|undefined} update
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @instance
                             */
                            MerchantCenterLinkOperation.prototype.update = null;
    
                            /**
                             * MerchantCenterLinkOperation remove.
                             * @member {string|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @instance
                             */
                            MerchantCenterLinkOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * MerchantCenterLinkOperation operation.
                             * @member {"update"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @instance
                             */
                            Object.defineProperty(MerchantCenterLinkOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["update", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new MerchantCenterLinkOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMerchantCenterLinkOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MerchantCenterLinkOperation} MerchantCenterLinkOperation instance
                             */
                            MerchantCenterLinkOperation.create = function create(properties) {
                                return new MerchantCenterLinkOperation(properties);
                            };
    
                            /**
                             * Encodes the specified MerchantCenterLinkOperation message. Does not implicitly {@link google.ads.googleads.v11.services.MerchantCenterLinkOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMerchantCenterLinkOperation} m MerchantCenterLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterLinkOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.update != null && Object.hasOwnProperty.call(m, "update"))
                                    $root.google.ads.googleads.v11.resources.MerchantCenterLink.encode(m.update, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    w.uint32(18).string(m.remove);
                                if (m.update_mask != null && Object.hasOwnProperty.call(m, "update_mask"))
                                    $root.google.protobuf.FieldMask.encode(m.update_mask, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MerchantCenterLinkOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MerchantCenterLinkOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IMerchantCenterLinkOperation} message MerchantCenterLinkOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MerchantCenterLinkOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MerchantCenterLinkOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MerchantCenterLinkOperation} MerchantCenterLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterLinkOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.update_mask = $root.google.protobuf.FieldMask.decode(r, r.uint32());
                                        break;
                                    case 1:
                                        m.update = $root.google.ads.googleads.v11.resources.MerchantCenterLink.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MerchantCenterLinkOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MerchantCenterLinkOperation} MerchantCenterLinkOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MerchantCenterLinkOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MerchantCenterLinkOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MerchantCenterLinkOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    {
                                        var e = $root.google.protobuf.FieldMask.verify(m.update_mask);
                                        if (e)
                                            return "update_mask." + e;
                                    }
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.MerchantCenterLink.verify(m.update);
                                        if (e)
                                            return "update." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (!$util.isString(m.remove))
                                        return "remove: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MerchantCenterLinkOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MerchantCenterLinkOperation} MerchantCenterLinkOperation
                             */
                            MerchantCenterLinkOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MerchantCenterLinkOperation();
                                if (d.update_mask != null) {
                                    if (typeof d.update_mask !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MerchantCenterLinkOperation.update_mask: object expected");
                                    m.update_mask = $root.google.protobuf.FieldMask.fromObject(d.update_mask);
                                }
                                if (d.update != null) {
                                    if (typeof d.update !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MerchantCenterLinkOperation.update: object expected");
                                    m.update = $root.google.ads.googleads.v11.resources.MerchantCenterLink.fromObject(d.update);
                                }
                                if (d.remove != null) {
                                    m.remove = String(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MerchantCenterLinkOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.MerchantCenterLinkOperation} m MerchantCenterLinkOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MerchantCenterLinkOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.update_mask = null;
                                }
                                if (m.update != null && m.hasOwnProperty("update")) {
                                    d.update = $root.google.ads.googleads.v11.resources.MerchantCenterLink.toObject(m.update, o);
                                    if (o.oneofs)
                                        d.operation = "update";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = m.remove;
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.update_mask != null && m.hasOwnProperty("update_mask")) {
                                    d.update_mask = $root.google.protobuf.FieldMask.toObject(m.update_mask, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MerchantCenterLinkOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MerchantCenterLinkOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MerchantCenterLinkOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MerchantCenterLinkOperation;
                        })();
    
                        services.MutateMerchantCenterLinkResponse = (function() {
    
                            /**
                             * Properties of a MutateMerchantCenterLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateMerchantCenterLinkResponse
                             * @property {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResult|null} [result] MutateMerchantCenterLinkResponse result
                             */
    
                            /**
                             * Constructs a new MutateMerchantCenterLinkResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateMerchantCenterLinkResponse.
                             * @implements IMutateMerchantCenterLinkResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResponse=} [p] Properties to set
                             */
                            function MutateMerchantCenterLinkResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateMerchantCenterLinkResponse result.
                             * @member {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResult|null|undefined} result
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @instance
                             */
                            MutateMerchantCenterLinkResponse.prototype.result = null;
    
                            /**
                             * Creates a new MutateMerchantCenterLinkResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse} MutateMerchantCenterLinkResponse instance
                             */
                            MutateMerchantCenterLinkResponse.create = function create(properties) {
                                return new MutateMerchantCenterLinkResponse(properties);
                            };
    
                            /**
                             * Encodes the specified MutateMerchantCenterLinkResponse message. Does not implicitly {@link google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResponse} m MutateMerchantCenterLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMerchantCenterLinkResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.result != null && Object.hasOwnProperty.call(m, "result"))
                                    $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult.encode(m.result, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateMerchantCenterLinkResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResponse} message MutateMerchantCenterLinkResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMerchantCenterLinkResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateMerchantCenterLinkResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse} MutateMerchantCenterLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMerchantCenterLinkResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.result = $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateMerchantCenterLinkResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse} MutateMerchantCenterLinkResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMerchantCenterLinkResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateMerchantCenterLinkResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateMerchantCenterLinkResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult.verify(m.result);
                                        if (e)
                                            return "result." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateMerchantCenterLinkResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse} MutateMerchantCenterLinkResponse
                             */
                            MutateMerchantCenterLinkResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse();
                                if (d.result != null) {
                                    if (typeof d.result !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse.result: object expected");
                                    m.result = $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult.fromObject(d.result);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateMerchantCenterLinkResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse} m MutateMerchantCenterLinkResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateMerchantCenterLinkResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.result = null;
                                }
                                if (m.result != null && m.hasOwnProperty("result")) {
                                    d.result = $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult.toObject(m.result, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateMerchantCenterLinkResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateMerchantCenterLinkResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateMerchantCenterLinkResponse;
                        })();
    
                        services.MutateMerchantCenterLinkResult = (function() {
    
                            /**
                             * Properties of a MutateMerchantCenterLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IMutateMerchantCenterLinkResult
                             * @property {string|null} [resource_name] MutateMerchantCenterLinkResult resource_name
                             */
    
                            /**
                             * Constructs a new MutateMerchantCenterLinkResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a MutateMerchantCenterLinkResult.
                             * @implements IMutateMerchantCenterLinkResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResult=} [p] Properties to set
                             */
                            function MutateMerchantCenterLinkResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * MutateMerchantCenterLinkResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @instance
                             */
                            MutateMerchantCenterLinkResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new MutateMerchantCenterLinkResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResult} MutateMerchantCenterLinkResult instance
                             */
                            MutateMerchantCenterLinkResult.create = function create(properties) {
                                return new MutateMerchantCenterLinkResult(properties);
                            };
    
                            /**
                             * Encodes the specified MutateMerchantCenterLinkResult message. Does not implicitly {@link google.ads.googleads.v11.services.MutateMerchantCenterLinkResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResult} m MutateMerchantCenterLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMerchantCenterLinkResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified MutateMerchantCenterLinkResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.MutateMerchantCenterLinkResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IMutateMerchantCenterLinkResult} message MutateMerchantCenterLinkResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MutateMerchantCenterLinkResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MutateMerchantCenterLinkResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResult} MutateMerchantCenterLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMerchantCenterLinkResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a MutateMerchantCenterLinkResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResult} MutateMerchantCenterLinkResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MutateMerchantCenterLinkResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MutateMerchantCenterLinkResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MutateMerchantCenterLinkResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MutateMerchantCenterLinkResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.MutateMerchantCenterLinkResult} MutateMerchantCenterLinkResult
                             */
                            MutateMerchantCenterLinkResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.MutateMerchantCenterLinkResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a MutateMerchantCenterLinkResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @static
                             * @param {google.ads.googleads.v11.services.MutateMerchantCenterLinkResult} m MutateMerchantCenterLinkResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MutateMerchantCenterLinkResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this MutateMerchantCenterLinkResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.MutateMerchantCenterLinkResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MutateMerchantCenterLinkResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return MutateMerchantCenterLinkResult;
                        })();
    
                        services.OfflineUserDataJobService = (function() {
    
                            /**
                             * Constructs a new OfflineUserDataJobService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an OfflineUserDataJobService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function OfflineUserDataJobService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (OfflineUserDataJobService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = OfflineUserDataJobService;
    
                            /**
                             * Creates new OfflineUserDataJobService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {OfflineUserDataJobService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            OfflineUserDataJobService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.OfflineUserDataJobService|createOfflineUserDataJob}.
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @typedef CreateOfflineUserDataJobCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse} [response] CreateOfflineUserDataJobResponse
                             */
    
                            /**
                             * Calls CreateOfflineUserDataJob.
                             * @function createOfflineUserDataJob
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobRequest} request CreateOfflineUserDataJobRequest message or plain object
                             * @param {google.ads.googleads.v11.services.OfflineUserDataJobService.CreateOfflineUserDataJobCallback} callback Node-style callback called with the error, if any, and CreateOfflineUserDataJobResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(OfflineUserDataJobService.prototype.createOfflineUserDataJob = function createOfflineUserDataJob(request, callback) {
                                return this.rpcCall(createOfflineUserDataJob, $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest, $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse, request, callback);
                            }, "name", { value: "CreateOfflineUserDataJob" });
    
                            /**
                             * Calls CreateOfflineUserDataJob.
                             * @function createOfflineUserDataJob
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobRequest} request CreateOfflineUserDataJobRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.OfflineUserDataJobService|addOfflineUserDataJobOperations}.
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @typedef AddOfflineUserDataJobOperationsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse} [response] AddOfflineUserDataJobOperationsResponse
                             */
    
                            /**
                             * Calls AddOfflineUserDataJobOperations.
                             * @function addOfflineUserDataJobOperations
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsRequest} request AddOfflineUserDataJobOperationsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.OfflineUserDataJobService.AddOfflineUserDataJobOperationsCallback} callback Node-style callback called with the error, if any, and AddOfflineUserDataJobOperationsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(OfflineUserDataJobService.prototype.addOfflineUserDataJobOperations = function addOfflineUserDataJobOperations(request, callback) {
                                return this.rpcCall(addOfflineUserDataJobOperations, $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest, $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse, request, callback);
                            }, "name", { value: "AddOfflineUserDataJobOperations" });
    
                            /**
                             * Calls AddOfflineUserDataJobOperations.
                             * @function addOfflineUserDataJobOperations
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsRequest} request AddOfflineUserDataJobOperationsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.OfflineUserDataJobService|runOfflineUserDataJob}.
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @typedef RunOfflineUserDataJobCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.longrunning.Operation} [response] Operation
                             */
    
                            /**
                             * Calls RunOfflineUserDataJob.
                             * @function runOfflineUserDataJob
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IRunOfflineUserDataJobRequest} request RunOfflineUserDataJobRequest message or plain object
                             * @param {google.ads.googleads.v11.services.OfflineUserDataJobService.RunOfflineUserDataJobCallback} callback Node-style callback called with the error, if any, and Operation
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(OfflineUserDataJobService.prototype.runOfflineUserDataJob = function runOfflineUserDataJob(request, callback) {
                                return this.rpcCall(runOfflineUserDataJob, $root.google.ads.googleads.v11.services.RunOfflineUserDataJobRequest, $root.google.longrunning.Operation, request, callback);
                            }, "name", { value: "RunOfflineUserDataJob" });
    
                            /**
                             * Calls RunOfflineUserDataJob.
                             * @function runOfflineUserDataJob
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IRunOfflineUserDataJobRequest} request RunOfflineUserDataJobRequest message or plain object
                             * @returns {Promise<google.longrunning.Operation>} Promise
                             * @variation 2
                             */
    
                            return OfflineUserDataJobService;
                        })();
    
                        services.CreateOfflineUserDataJobRequest = (function() {
    
                            /**
                             * Properties of a CreateOfflineUserDataJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateOfflineUserDataJobRequest
                             * @property {string|null} [customer_id] CreateOfflineUserDataJobRequest customer_id
                             * @property {google.ads.googleads.v11.resources.IOfflineUserDataJob|null} [job] CreateOfflineUserDataJobRequest job
                             * @property {boolean|null} [validate_only] CreateOfflineUserDataJobRequest validate_only
                             * @property {boolean|null} [enable_match_rate_range_preview] CreateOfflineUserDataJobRequest enable_match_rate_range_preview
                             */
    
                            /**
                             * Constructs a new CreateOfflineUserDataJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateOfflineUserDataJobRequest.
                             * @implements ICreateOfflineUserDataJobRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobRequest=} [p] Properties to set
                             */
                            function CreateOfflineUserDataJobRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateOfflineUserDataJobRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @instance
                             */
                            CreateOfflineUserDataJobRequest.prototype.customer_id = "";
    
                            /**
                             * CreateOfflineUserDataJobRequest job.
                             * @member {google.ads.googleads.v11.resources.IOfflineUserDataJob|null|undefined} job
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @instance
                             */
                            CreateOfflineUserDataJobRequest.prototype.job = null;
    
                            /**
                             * CreateOfflineUserDataJobRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @instance
                             */
                            CreateOfflineUserDataJobRequest.prototype.validate_only = false;
    
                            /**
                             * CreateOfflineUserDataJobRequest enable_match_rate_range_preview.
                             * @member {boolean} enable_match_rate_range_preview
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @instance
                             */
                            CreateOfflineUserDataJobRequest.prototype.enable_match_rate_range_preview = false;
    
                            /**
                             * Creates a new CreateOfflineUserDataJobRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest} CreateOfflineUserDataJobRequest instance
                             */
                            CreateOfflineUserDataJobRequest.create = function create(properties) {
                                return new CreateOfflineUserDataJobRequest(properties);
                            };
    
                            /**
                             * Encodes the specified CreateOfflineUserDataJobRequest message. Does not implicitly {@link google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobRequest} m CreateOfflineUserDataJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateOfflineUserDataJobRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.job != null && Object.hasOwnProperty.call(m, "job"))
                                    $root.google.ads.googleads.v11.resources.OfflineUserDataJob.encode(m.job, w.uint32(18).fork()).ldelim();
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(24).bool(m.validate_only);
                                if (m.enable_match_rate_range_preview != null && Object.hasOwnProperty.call(m, "enable_match_rate_range_preview"))
                                    w.uint32(40).bool(m.enable_match_rate_range_preview);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateOfflineUserDataJobRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobRequest} message CreateOfflineUserDataJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateOfflineUserDataJobRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateOfflineUserDataJobRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest} CreateOfflineUserDataJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateOfflineUserDataJobRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.job = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.validate_only = r.bool();
                                        break;
                                    case 5:
                                        m.enable_match_rate_range_preview = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateOfflineUserDataJobRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest} CreateOfflineUserDataJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateOfflineUserDataJobRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateOfflineUserDataJobRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateOfflineUserDataJobRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.job != null && m.hasOwnProperty("job")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.verify(m.job);
                                        if (e)
                                            return "job." + e;
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                if (m.enable_match_rate_range_preview != null && m.hasOwnProperty("enable_match_rate_range_preview")) {
                                    if (typeof m.enable_match_rate_range_preview !== "boolean")
                                        return "enable_match_rate_range_preview: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateOfflineUserDataJobRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest} CreateOfflineUserDataJobRequest
                             */
                            CreateOfflineUserDataJobRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.job != null) {
                                    if (typeof d.job !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest.job: object expected");
                                    m.job = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.fromObject(d.job);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                if (d.enable_match_rate_range_preview != null) {
                                    m.enable_match_rate_range_preview = Boolean(d.enable_match_rate_range_preview);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateOfflineUserDataJobRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest} m CreateOfflineUserDataJobRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateOfflineUserDataJobRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.job = null;
                                    d.validate_only = false;
                                    d.enable_match_rate_range_preview = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.job != null && m.hasOwnProperty("job")) {
                                    d.job = $root.google.ads.googleads.v11.resources.OfflineUserDataJob.toObject(m.job, o);
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.enable_match_rate_range_preview != null && m.hasOwnProperty("enable_match_rate_range_preview")) {
                                    d.enable_match_rate_range_preview = m.enable_match_rate_range_preview;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateOfflineUserDataJobRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateOfflineUserDataJobRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateOfflineUserDataJobRequest;
                        })();
    
                        services.CreateOfflineUserDataJobResponse = (function() {
    
                            /**
                             * Properties of a CreateOfflineUserDataJobResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICreateOfflineUserDataJobResponse
                             * @property {string|null} [resource_name] CreateOfflineUserDataJobResponse resource_name
                             */
    
                            /**
                             * Constructs a new CreateOfflineUserDataJobResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CreateOfflineUserDataJobResponse.
                             * @implements ICreateOfflineUserDataJobResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobResponse=} [p] Properties to set
                             */
                            function CreateOfflineUserDataJobResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CreateOfflineUserDataJobResponse resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @instance
                             */
                            CreateOfflineUserDataJobResponse.prototype.resource_name = "";
    
                            /**
                             * Creates a new CreateOfflineUserDataJobResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse} CreateOfflineUserDataJobResponse instance
                             */
                            CreateOfflineUserDataJobResponse.create = function create(properties) {
                                return new CreateOfflineUserDataJobResponse(properties);
                            };
    
                            /**
                             * Encodes the specified CreateOfflineUserDataJobResponse message. Does not implicitly {@link google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobResponse} m CreateOfflineUserDataJobResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateOfflineUserDataJobResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified CreateOfflineUserDataJobResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ICreateOfflineUserDataJobResponse} message CreateOfflineUserDataJobResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CreateOfflineUserDataJobResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CreateOfflineUserDataJobResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse} CreateOfflineUserDataJobResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateOfflineUserDataJobResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CreateOfflineUserDataJobResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse} CreateOfflineUserDataJobResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CreateOfflineUserDataJobResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CreateOfflineUserDataJobResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CreateOfflineUserDataJobResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CreateOfflineUserDataJobResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse} CreateOfflineUserDataJobResponse
                             */
                            CreateOfflineUserDataJobResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CreateOfflineUserDataJobResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse} m CreateOfflineUserDataJobResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CreateOfflineUserDataJobResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CreateOfflineUserDataJobResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CreateOfflineUserDataJobResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CreateOfflineUserDataJobResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CreateOfflineUserDataJobResponse;
                        })();
    
                        services.RunOfflineUserDataJobRequest = (function() {
    
                            /**
                             * Properties of a RunOfflineUserDataJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IRunOfflineUserDataJobRequest
                             * @property {string|null} [resource_name] RunOfflineUserDataJobRequest resource_name
                             * @property {boolean|null} [validate_only] RunOfflineUserDataJobRequest validate_only
                             */
    
                            /**
                             * Constructs a new RunOfflineUserDataJobRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RunOfflineUserDataJobRequest.
                             * @implements IRunOfflineUserDataJobRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IRunOfflineUserDataJobRequest=} [p] Properties to set
                             */
                            function RunOfflineUserDataJobRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RunOfflineUserDataJobRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @instance
                             */
                            RunOfflineUserDataJobRequest.prototype.resource_name = "";
    
                            /**
                             * RunOfflineUserDataJobRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @instance
                             */
                            RunOfflineUserDataJobRequest.prototype.validate_only = false;
    
                            /**
                             * Creates a new RunOfflineUserDataJobRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRunOfflineUserDataJobRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.RunOfflineUserDataJobRequest} RunOfflineUserDataJobRequest instance
                             */
                            RunOfflineUserDataJobRequest.create = function create(properties) {
                                return new RunOfflineUserDataJobRequest(properties);
                            };
    
                            /**
                             * Encodes the specified RunOfflineUserDataJobRequest message. Does not implicitly {@link google.ads.googleads.v11.services.RunOfflineUserDataJobRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRunOfflineUserDataJobRequest} m RunOfflineUserDataJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RunOfflineUserDataJobRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(16).bool(m.validate_only);
                                return w;
                            };
    
                            /**
                             * Encodes the specified RunOfflineUserDataJobRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.RunOfflineUserDataJobRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRunOfflineUserDataJobRequest} message RunOfflineUserDataJobRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RunOfflineUserDataJobRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RunOfflineUserDataJobRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.RunOfflineUserDataJobRequest} RunOfflineUserDataJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RunOfflineUserDataJobRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.RunOfflineUserDataJobRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RunOfflineUserDataJobRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.RunOfflineUserDataJobRequest} RunOfflineUserDataJobRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RunOfflineUserDataJobRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RunOfflineUserDataJobRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RunOfflineUserDataJobRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RunOfflineUserDataJobRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.RunOfflineUserDataJobRequest} RunOfflineUserDataJobRequest
                             */
                            RunOfflineUserDataJobRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.RunOfflineUserDataJobRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.RunOfflineUserDataJobRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RunOfflineUserDataJobRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.RunOfflineUserDataJobRequest} m RunOfflineUserDataJobRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RunOfflineUserDataJobRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.validate_only = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RunOfflineUserDataJobRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.RunOfflineUserDataJobRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RunOfflineUserDataJobRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RunOfflineUserDataJobRequest;
                        })();
    
                        services.AddOfflineUserDataJobOperationsRequest = (function() {
    
                            /**
                             * Properties of an AddOfflineUserDataJobOperationsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAddOfflineUserDataJobOperationsRequest
                             * @property {string|null} [resource_name] AddOfflineUserDataJobOperationsRequest resource_name
                             * @property {boolean|null} [enable_partial_failure] AddOfflineUserDataJobOperationsRequest enable_partial_failure
                             * @property {boolean|null} [enable_warnings] AddOfflineUserDataJobOperationsRequest enable_warnings
                             * @property {Array.<google.ads.googleads.v11.services.IOfflineUserDataJobOperation>|null} [operations] AddOfflineUserDataJobOperationsRequest operations
                             * @property {boolean|null} [validate_only] AddOfflineUserDataJobOperationsRequest validate_only
                             */
    
                            /**
                             * Constructs a new AddOfflineUserDataJobOperationsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AddOfflineUserDataJobOperationsRequest.
                             * @implements IAddOfflineUserDataJobOperationsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsRequest=} [p] Properties to set
                             */
                            function AddOfflineUserDataJobOperationsRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AddOfflineUserDataJobOperationsRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             */
                            AddOfflineUserDataJobOperationsRequest.prototype.resource_name = "";
    
                            /**
                             * AddOfflineUserDataJobOperationsRequest enable_partial_failure.
                             * @member {boolean|null|undefined} enable_partial_failure
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             */
                            AddOfflineUserDataJobOperationsRequest.prototype.enable_partial_failure = null;
    
                            /**
                             * AddOfflineUserDataJobOperationsRequest enable_warnings.
                             * @member {boolean|null|undefined} enable_warnings
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             */
                            AddOfflineUserDataJobOperationsRequest.prototype.enable_warnings = null;
    
                            /**
                             * AddOfflineUserDataJobOperationsRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IOfflineUserDataJobOperation>} operations
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             */
                            AddOfflineUserDataJobOperationsRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * AddOfflineUserDataJobOperationsRequest validate_only.
                             * @member {boolean} validate_only
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             */
                            AddOfflineUserDataJobOperationsRequest.prototype.validate_only = false;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * AddOfflineUserDataJobOperationsRequest _enable_partial_failure.
                             * @member {"enable_partial_failure"|undefined} _enable_partial_failure
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             */
                            Object.defineProperty(AddOfflineUserDataJobOperationsRequest.prototype, "_enable_partial_failure", {
                                get: $util.oneOfGetter($oneOfFields = ["enable_partial_failure"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * AddOfflineUserDataJobOperationsRequest _enable_warnings.
                             * @member {"enable_warnings"|undefined} _enable_warnings
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             */
                            Object.defineProperty(AddOfflineUserDataJobOperationsRequest.prototype, "_enable_warnings", {
                                get: $util.oneOfGetter($oneOfFields = ["enable_warnings"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new AddOfflineUserDataJobOperationsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest} AddOfflineUserDataJobOperationsRequest instance
                             */
                            AddOfflineUserDataJobOperationsRequest.create = function create(properties) {
                                return new AddOfflineUserDataJobOperationsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified AddOfflineUserDataJobOperationsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsRequest} m AddOfflineUserDataJobOperationsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddOfflineUserDataJobOperationsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.OfflineUserDataJobOperation.encode(m.operations[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.enable_partial_failure != null && Object.hasOwnProperty.call(m, "enable_partial_failure"))
                                    w.uint32(32).bool(m.enable_partial_failure);
                                if (m.validate_only != null && Object.hasOwnProperty.call(m, "validate_only"))
                                    w.uint32(40).bool(m.validate_only);
                                if (m.enable_warnings != null && Object.hasOwnProperty.call(m, "enable_warnings"))
                                    w.uint32(48).bool(m.enable_warnings);
                                return w;
                            };
    
                            /**
                             * Encodes the specified AddOfflineUserDataJobOperationsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsRequest} message AddOfflineUserDataJobOperationsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddOfflineUserDataJobOperationsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AddOfflineUserDataJobOperationsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest} AddOfflineUserDataJobOperationsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddOfflineUserDataJobOperationsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 4:
                                        m.enable_partial_failure = r.bool();
                                        break;
                                    case 6:
                                        m.enable_warnings = r.bool();
                                        break;
                                    case 3:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.OfflineUserDataJobOperation.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        m.validate_only = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AddOfflineUserDataJobOperationsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest} AddOfflineUserDataJobOperationsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddOfflineUserDataJobOperationsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AddOfflineUserDataJobOperationsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AddOfflineUserDataJobOperationsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.enable_partial_failure != null && m.hasOwnProperty("enable_partial_failure")) {
                                    p._enable_partial_failure = 1;
                                    if (typeof m.enable_partial_failure !== "boolean")
                                        return "enable_partial_failure: boolean expected";
                                }
                                if (m.enable_warnings != null && m.hasOwnProperty("enable_warnings")) {
                                    p._enable_warnings = 1;
                                    if (typeof m.enable_warnings !== "boolean")
                                        return "enable_warnings: boolean expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.OfflineUserDataJobOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    if (typeof m.validate_only !== "boolean")
                                        return "validate_only: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AddOfflineUserDataJobOperationsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest} AddOfflineUserDataJobOperationsRequest
                             */
                            AddOfflineUserDataJobOperationsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.enable_partial_failure != null) {
                                    m.enable_partial_failure = Boolean(d.enable_partial_failure);
                                }
                                if (d.enable_warnings != null) {
                                    m.enable_warnings = Boolean(d.enable_warnings);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.OfflineUserDataJobOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.validate_only != null) {
                                    m.validate_only = Boolean(d.validate_only);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AddOfflineUserDataJobOperationsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest} m AddOfflineUserDataJobOperationsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AddOfflineUserDataJobOperationsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.resource_name = "";
                                    d.validate_only = false;
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.OfflineUserDataJobOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.enable_partial_failure != null && m.hasOwnProperty("enable_partial_failure")) {
                                    d.enable_partial_failure = m.enable_partial_failure;
                                    if (o.oneofs)
                                        d._enable_partial_failure = "enable_partial_failure";
                                }
                                if (m.validate_only != null && m.hasOwnProperty("validate_only")) {
                                    d.validate_only = m.validate_only;
                                }
                                if (m.enable_warnings != null && m.hasOwnProperty("enable_warnings")) {
                                    d.enable_warnings = m.enable_warnings;
                                    if (o.oneofs)
                                        d._enable_warnings = "enable_warnings";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AddOfflineUserDataJobOperationsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AddOfflineUserDataJobOperationsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AddOfflineUserDataJobOperationsRequest;
                        })();
    
                        services.OfflineUserDataJobOperation = (function() {
    
                            /**
                             * Properties of an OfflineUserDataJobOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IOfflineUserDataJobOperation
                             * @property {google.ads.googleads.v11.common.IUserData|null} [create] OfflineUserDataJobOperation create
                             * @property {google.ads.googleads.v11.common.IUserData|null} [remove] OfflineUserDataJobOperation remove
                             * @property {boolean|null} [remove_all] OfflineUserDataJobOperation remove_all
                             */
    
                            /**
                             * Constructs a new OfflineUserDataJobOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an OfflineUserDataJobOperation.
                             * @implements IOfflineUserDataJobOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IOfflineUserDataJobOperation=} [p] Properties to set
                             */
                            function OfflineUserDataJobOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * OfflineUserDataJobOperation create.
                             * @member {google.ads.googleads.v11.common.IUserData|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @instance
                             */
                            OfflineUserDataJobOperation.prototype.create = null;
    
                            /**
                             * OfflineUserDataJobOperation remove.
                             * @member {google.ads.googleads.v11.common.IUserData|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @instance
                             */
                            OfflineUserDataJobOperation.prototype.remove = null;
    
                            /**
                             * OfflineUserDataJobOperation remove_all.
                             * @member {boolean|null|undefined} remove_all
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @instance
                             */
                            OfflineUserDataJobOperation.prototype.remove_all = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * OfflineUserDataJobOperation operation.
                             * @member {"create"|"remove"|"remove_all"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @instance
                             */
                            Object.defineProperty(OfflineUserDataJobOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove", "remove_all"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new OfflineUserDataJobOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IOfflineUserDataJobOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.OfflineUserDataJobOperation} OfflineUserDataJobOperation instance
                             */
                            OfflineUserDataJobOperation.create = function create(properties) {
                                return new OfflineUserDataJobOperation(properties);
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobOperation message. Does not implicitly {@link google.ads.googleads.v11.services.OfflineUserDataJobOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IOfflineUserDataJobOperation} m OfflineUserDataJobOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.common.UserData.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    $root.google.ads.googleads.v11.common.UserData.encode(m.remove, w.uint32(18).fork()).ldelim();
                                if (m.remove_all != null && Object.hasOwnProperty.call(m, "remove_all"))
                                    w.uint32(24).bool(m.remove_all);
                                return w;
                            };
    
                            /**
                             * Encodes the specified OfflineUserDataJobOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.OfflineUserDataJobOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IOfflineUserDataJobOperation} message OfflineUserDataJobOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OfflineUserDataJobOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.OfflineUserDataJobOperation} OfflineUserDataJobOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.OfflineUserDataJobOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.common.UserData.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = $root.google.ads.googleads.v11.common.UserData.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.remove_all = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OfflineUserDataJobOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.OfflineUserDataJobOperation} OfflineUserDataJobOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OfflineUserDataJobOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OfflineUserDataJobOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OfflineUserDataJobOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserData.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserData.verify(m.remove);
                                        if (e)
                                            return "remove." + e;
                                    }
                                }
                                if (m.remove_all != null && m.hasOwnProperty("remove_all")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    if (typeof m.remove_all !== "boolean")
                                        return "remove_all: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an OfflineUserDataJobOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.OfflineUserDataJobOperation} OfflineUserDataJobOperation
                             */
                            OfflineUserDataJobOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.OfflineUserDataJobOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.OfflineUserDataJobOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.OfflineUserDataJobOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.common.UserData.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    if (typeof d.remove !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.OfflineUserDataJobOperation.remove: object expected");
                                    m.remove = $root.google.ads.googleads.v11.common.UserData.fromObject(d.remove);
                                }
                                if (d.remove_all != null) {
                                    m.remove_all = Boolean(d.remove_all);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an OfflineUserDataJobOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.OfflineUserDataJobOperation} m OfflineUserDataJobOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OfflineUserDataJobOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.common.UserData.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = $root.google.ads.googleads.v11.common.UserData.toObject(m.remove, o);
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                if (m.remove_all != null && m.hasOwnProperty("remove_all")) {
                                    d.remove_all = m.remove_all;
                                    if (o.oneofs)
                                        d.operation = "remove_all";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this OfflineUserDataJobOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.OfflineUserDataJobOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OfflineUserDataJobOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return OfflineUserDataJobOperation;
                        })();
    
                        services.AddOfflineUserDataJobOperationsResponse = (function() {
    
                            /**
                             * Properties of an AddOfflineUserDataJobOperationsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IAddOfflineUserDataJobOperationsResponse
                             * @property {google.rpc.IStatus|null} [partial_failure_error] AddOfflineUserDataJobOperationsResponse partial_failure_error
                             * @property {google.rpc.IStatus|null} [warning] AddOfflineUserDataJobOperationsResponse warning
                             */
    
                            /**
                             * Constructs a new AddOfflineUserDataJobOperationsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an AddOfflineUserDataJobOperationsResponse.
                             * @implements IAddOfflineUserDataJobOperationsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsResponse=} [p] Properties to set
                             */
                            function AddOfflineUserDataJobOperationsResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * AddOfflineUserDataJobOperationsResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @instance
                             */
                            AddOfflineUserDataJobOperationsResponse.prototype.partial_failure_error = null;
    
                            /**
                             * AddOfflineUserDataJobOperationsResponse warning.
                             * @member {google.rpc.IStatus|null|undefined} warning
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @instance
                             */
                            AddOfflineUserDataJobOperationsResponse.prototype.warning = null;
    
                            /**
                             * Creates a new AddOfflineUserDataJobOperationsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse} AddOfflineUserDataJobOperationsResponse instance
                             */
                            AddOfflineUserDataJobOperationsResponse.create = function create(properties) {
                                return new AddOfflineUserDataJobOperationsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified AddOfflineUserDataJobOperationsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsResponse} m AddOfflineUserDataJobOperationsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddOfflineUserDataJobOperationsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(10).fork()).ldelim();
                                if (m.warning != null && Object.hasOwnProperty.call(m, "warning"))
                                    $root.google.rpc.Status.encode(m.warning, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified AddOfflineUserDataJobOperationsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IAddOfflineUserDataJobOperationsResponse} message AddOfflineUserDataJobOperationsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AddOfflineUserDataJobOperationsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AddOfflineUserDataJobOperationsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse} AddOfflineUserDataJobOperationsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddOfflineUserDataJobOperationsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.warning = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an AddOfflineUserDataJobOperationsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse} AddOfflineUserDataJobOperationsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AddOfflineUserDataJobOperationsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AddOfflineUserDataJobOperationsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AddOfflineUserDataJobOperationsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                if (m.warning != null && m.hasOwnProperty("warning")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.warning);
                                        if (e)
                                            return "warning." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AddOfflineUserDataJobOperationsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse} AddOfflineUserDataJobOperationsResponse
                             */
                            AddOfflineUserDataJobOperationsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse();
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                if (d.warning != null) {
                                    if (typeof d.warning !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse.warning: object expected");
                                    m.warning = $root.google.rpc.Status.fromObject(d.warning);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an AddOfflineUserDataJobOperationsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse} m AddOfflineUserDataJobOperationsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AddOfflineUserDataJobOperationsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                    d.warning = null;
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                if (m.warning != null && m.hasOwnProperty("warning")) {
                                    d.warning = $root.google.rpc.Status.toObject(m.warning, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this AddOfflineUserDataJobOperationsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.AddOfflineUserDataJobOperationsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AddOfflineUserDataJobOperationsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return AddOfflineUserDataJobOperationsResponse;
                        })();
    
                        services.PaymentsAccountService = (function() {
    
                            /**
                             * Constructs a new PaymentsAccountService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PaymentsAccountService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function PaymentsAccountService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (PaymentsAccountService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = PaymentsAccountService;
    
                            /**
                             * Creates new PaymentsAccountService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PaymentsAccountService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {PaymentsAccountService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            PaymentsAccountService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.PaymentsAccountService|listPaymentsAccounts}.
                             * @memberof google.ads.googleads.v11.services.PaymentsAccountService
                             * @typedef ListPaymentsAccountsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListPaymentsAccountsResponse} [response] ListPaymentsAccountsResponse
                             */
    
                            /**
                             * Calls ListPaymentsAccounts.
                             * @function listPaymentsAccounts
                             * @memberof google.ads.googleads.v11.services.PaymentsAccountService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsRequest} request ListPaymentsAccountsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.PaymentsAccountService.ListPaymentsAccountsCallback} callback Node-style callback called with the error, if any, and ListPaymentsAccountsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(PaymentsAccountService.prototype.listPaymentsAccounts = function listPaymentsAccounts(request, callback) {
                                return this.rpcCall(listPaymentsAccounts, $root.google.ads.googleads.v11.services.ListPaymentsAccountsRequest, $root.google.ads.googleads.v11.services.ListPaymentsAccountsResponse, request, callback);
                            }, "name", { value: "ListPaymentsAccounts" });
    
                            /**
                             * Calls ListPaymentsAccounts.
                             * @function listPaymentsAccounts
                             * @memberof google.ads.googleads.v11.services.PaymentsAccountService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsRequest} request ListPaymentsAccountsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListPaymentsAccountsResponse>} Promise
                             * @variation 2
                             */
    
                            return PaymentsAccountService;
                        })();
    
                        services.ListPaymentsAccountsRequest = (function() {
    
                            /**
                             * Properties of a ListPaymentsAccountsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListPaymentsAccountsRequest
                             * @property {string|null} [customer_id] ListPaymentsAccountsRequest customer_id
                             */
    
                            /**
                             * Constructs a new ListPaymentsAccountsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListPaymentsAccountsRequest.
                             * @implements IListPaymentsAccountsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsRequest=} [p] Properties to set
                             */
                            function ListPaymentsAccountsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListPaymentsAccountsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @instance
                             */
                            ListPaymentsAccountsRequest.prototype.customer_id = "";
    
                            /**
                             * Creates a new ListPaymentsAccountsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsRequest} ListPaymentsAccountsRequest instance
                             */
                            ListPaymentsAccountsRequest.create = function create(properties) {
                                return new ListPaymentsAccountsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListPaymentsAccountsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListPaymentsAccountsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsRequest} m ListPaymentsAccountsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPaymentsAccountsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListPaymentsAccountsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListPaymentsAccountsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsRequest} message ListPaymentsAccountsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPaymentsAccountsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListPaymentsAccountsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsRequest} ListPaymentsAccountsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPaymentsAccountsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListPaymentsAccountsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListPaymentsAccountsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsRequest} ListPaymentsAccountsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPaymentsAccountsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListPaymentsAccountsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListPaymentsAccountsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListPaymentsAccountsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsRequest} ListPaymentsAccountsRequest
                             */
                            ListPaymentsAccountsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListPaymentsAccountsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListPaymentsAccountsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListPaymentsAccountsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListPaymentsAccountsRequest} m ListPaymentsAccountsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListPaymentsAccountsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListPaymentsAccountsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListPaymentsAccountsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListPaymentsAccountsRequest;
                        })();
    
                        services.ListPaymentsAccountsResponse = (function() {
    
                            /**
                             * Properties of a ListPaymentsAccountsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListPaymentsAccountsResponse
                             * @property {Array.<google.ads.googleads.v11.resources.IPaymentsAccount>|null} [payments_accounts] ListPaymentsAccountsResponse payments_accounts
                             */
    
                            /**
                             * Constructs a new ListPaymentsAccountsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListPaymentsAccountsResponse.
                             * @implements IListPaymentsAccountsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsResponse=} [p] Properties to set
                             */
                            function ListPaymentsAccountsResponse(p) {
                                this.payments_accounts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListPaymentsAccountsResponse payments_accounts.
                             * @member {Array.<google.ads.googleads.v11.resources.IPaymentsAccount>} payments_accounts
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @instance
                             */
                            ListPaymentsAccountsResponse.prototype.payments_accounts = $util.emptyArray;
    
                            /**
                             * Creates a new ListPaymentsAccountsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsResponse} ListPaymentsAccountsResponse instance
                             */
                            ListPaymentsAccountsResponse.create = function create(properties) {
                                return new ListPaymentsAccountsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListPaymentsAccountsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListPaymentsAccountsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsResponse} m ListPaymentsAccountsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPaymentsAccountsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.payments_accounts != null && m.payments_accounts.length) {
                                    for (var i = 0; i < m.payments_accounts.length; ++i)
                                        $root.google.ads.googleads.v11.resources.PaymentsAccount.encode(m.payments_accounts[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListPaymentsAccountsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListPaymentsAccountsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPaymentsAccountsResponse} message ListPaymentsAccountsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPaymentsAccountsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListPaymentsAccountsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsResponse} ListPaymentsAccountsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPaymentsAccountsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListPaymentsAccountsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.payments_accounts && m.payments_accounts.length))
                                            m.payments_accounts = [];
                                        m.payments_accounts.push($root.google.ads.googleads.v11.resources.PaymentsAccount.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListPaymentsAccountsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsResponse} ListPaymentsAccountsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPaymentsAccountsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListPaymentsAccountsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListPaymentsAccountsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.payments_accounts != null && m.hasOwnProperty("payments_accounts")) {
                                    if (!Array.isArray(m.payments_accounts))
                                        return "payments_accounts: array expected";
                                    for (var i = 0; i < m.payments_accounts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.PaymentsAccount.verify(m.payments_accounts[i]);
                                            if (e)
                                                return "payments_accounts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListPaymentsAccountsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListPaymentsAccountsResponse} ListPaymentsAccountsResponse
                             */
                            ListPaymentsAccountsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListPaymentsAccountsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListPaymentsAccountsResponse();
                                if (d.payments_accounts) {
                                    if (!Array.isArray(d.payments_accounts))
                                        throw TypeError(".google.ads.googleads.v11.services.ListPaymentsAccountsResponse.payments_accounts: array expected");
                                    m.payments_accounts = [];
                                    for (var i = 0; i < d.payments_accounts.length; ++i) {
                                        if (typeof d.payments_accounts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListPaymentsAccountsResponse.payments_accounts: object expected");
                                        m.payments_accounts[i] = $root.google.ads.googleads.v11.resources.PaymentsAccount.fromObject(d.payments_accounts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListPaymentsAccountsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListPaymentsAccountsResponse} m ListPaymentsAccountsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListPaymentsAccountsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.payments_accounts = [];
                                }
                                if (m.payments_accounts && m.payments_accounts.length) {
                                    d.payments_accounts = [];
                                    for (var j = 0; j < m.payments_accounts.length; ++j) {
                                        d.payments_accounts[j] = $root.google.ads.googleads.v11.resources.PaymentsAccount.toObject(m.payments_accounts[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListPaymentsAccountsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListPaymentsAccountsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListPaymentsAccountsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListPaymentsAccountsResponse;
                        })();
    
                        services.ReachPlanService = (function() {
    
                            /**
                             * Constructs a new ReachPlanService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ReachPlanService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ReachPlanService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ReachPlanService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ReachPlanService;
    
                            /**
                             * Creates new ReachPlanService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ReachPlanService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ReachPlanService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ReachPlanService|listPlannableLocations}.
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @typedef ListPlannableLocationsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListPlannableLocationsResponse} [response] ListPlannableLocationsResponse
                             */
    
                            /**
                             * Calls ListPlannableLocations.
                             * @function listPlannableLocations
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsRequest} request ListPlannableLocationsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ReachPlanService.ListPlannableLocationsCallback} callback Node-style callback called with the error, if any, and ListPlannableLocationsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ReachPlanService.prototype.listPlannableLocations = function listPlannableLocations(request, callback) {
                                return this.rpcCall(listPlannableLocations, $root.google.ads.googleads.v11.services.ListPlannableLocationsRequest, $root.google.ads.googleads.v11.services.ListPlannableLocationsResponse, request, callback);
                            }, "name", { value: "ListPlannableLocations" });
    
                            /**
                             * Calls ListPlannableLocations.
                             * @function listPlannableLocations
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsRequest} request ListPlannableLocationsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListPlannableLocationsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ReachPlanService|listPlannableProducts}.
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @typedef ListPlannableProductsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ListPlannableProductsResponse} [response] ListPlannableProductsResponse
                             */
    
                            /**
                             * Calls ListPlannableProducts.
                             * @function listPlannableProducts
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsRequest} request ListPlannableProductsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ReachPlanService.ListPlannableProductsCallback} callback Node-style callback called with the error, if any, and ListPlannableProductsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ReachPlanService.prototype.listPlannableProducts = function listPlannableProducts(request, callback) {
                                return this.rpcCall(listPlannableProducts, $root.google.ads.googleads.v11.services.ListPlannableProductsRequest, $root.google.ads.googleads.v11.services.ListPlannableProductsResponse, request, callback);
                            }, "name", { value: "ListPlannableProducts" });
    
                            /**
                             * Calls ListPlannableProducts.
                             * @function listPlannableProducts
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsRequest} request ListPlannableProductsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ListPlannableProductsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ReachPlanService|generateProductMixIdeas}.
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @typedef GenerateProductMixIdeasCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateProductMixIdeasResponse} [response] GenerateProductMixIdeasResponse
                             */
    
                            /**
                             * Calls GenerateProductMixIdeas.
                             * @function generateProductMixIdeas
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasRequest} request GenerateProductMixIdeasRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ReachPlanService.GenerateProductMixIdeasCallback} callback Node-style callback called with the error, if any, and GenerateProductMixIdeasResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ReachPlanService.prototype.generateProductMixIdeas = function generateProductMixIdeas(request, callback) {
                                return this.rpcCall(generateProductMixIdeas, $root.google.ads.googleads.v11.services.GenerateProductMixIdeasRequest, $root.google.ads.googleads.v11.services.GenerateProductMixIdeasResponse, request, callback);
                            }, "name", { value: "GenerateProductMixIdeas" });
    
                            /**
                             * Calls GenerateProductMixIdeas.
                             * @function generateProductMixIdeas
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasRequest} request GenerateProductMixIdeasRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateProductMixIdeasResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ReachPlanService|generateReachForecast}.
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @typedef GenerateReachForecastCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.GenerateReachForecastResponse} [response] GenerateReachForecastResponse
                             */
    
                            /**
                             * Calls GenerateReachForecast.
                             * @function generateReachForecast
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastRequest} request GenerateReachForecastRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ReachPlanService.GenerateReachForecastCallback} callback Node-style callback called with the error, if any, and GenerateReachForecastResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ReachPlanService.prototype.generateReachForecast = function generateReachForecast(request, callback) {
                                return this.rpcCall(generateReachForecast, $root.google.ads.googleads.v11.services.GenerateReachForecastRequest, $root.google.ads.googleads.v11.services.GenerateReachForecastResponse, request, callback);
                            }, "name", { value: "GenerateReachForecast" });
    
                            /**
                             * Calls GenerateReachForecast.
                             * @function generateReachForecast
                             * @memberof google.ads.googleads.v11.services.ReachPlanService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastRequest} request GenerateReachForecastRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.GenerateReachForecastResponse>} Promise
                             * @variation 2
                             */
    
                            return ReachPlanService;
                        })();
    
                        services.ListPlannableLocationsRequest = (function() {
    
                            /**
                             * Properties of a ListPlannableLocationsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListPlannableLocationsRequest
                             */
    
                            /**
                             * Constructs a new ListPlannableLocationsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListPlannableLocationsRequest.
                             * @implements IListPlannableLocationsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsRequest=} [p] Properties to set
                             */
                            function ListPlannableLocationsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new ListPlannableLocationsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsRequest} ListPlannableLocationsRequest instance
                             */
                            ListPlannableLocationsRequest.create = function create(properties) {
                                return new ListPlannableLocationsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListPlannableLocationsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableLocationsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsRequest} m ListPlannableLocationsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableLocationsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListPlannableLocationsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableLocationsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsRequest} message ListPlannableLocationsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableLocationsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListPlannableLocationsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsRequest} ListPlannableLocationsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableLocationsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListPlannableLocationsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListPlannableLocationsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsRequest} ListPlannableLocationsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableLocationsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListPlannableLocationsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListPlannableLocationsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a ListPlannableLocationsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsRequest} ListPlannableLocationsRequest
                             */
                            ListPlannableLocationsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListPlannableLocationsRequest)
                                    return d;
                                return new $root.google.ads.googleads.v11.services.ListPlannableLocationsRequest();
                            };
    
                            /**
                             * Creates a plain object from a ListPlannableLocationsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListPlannableLocationsRequest} m ListPlannableLocationsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListPlannableLocationsRequest.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this ListPlannableLocationsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListPlannableLocationsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListPlannableLocationsRequest;
                        })();
    
                        services.ListPlannableLocationsResponse = (function() {
    
                            /**
                             * Properties of a ListPlannableLocationsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListPlannableLocationsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IPlannableLocation>|null} [plannable_locations] ListPlannableLocationsResponse plannable_locations
                             */
    
                            /**
                             * Constructs a new ListPlannableLocationsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListPlannableLocationsResponse.
                             * @implements IListPlannableLocationsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsResponse=} [p] Properties to set
                             */
                            function ListPlannableLocationsResponse(p) {
                                this.plannable_locations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListPlannableLocationsResponse plannable_locations.
                             * @member {Array.<google.ads.googleads.v11.services.IPlannableLocation>} plannable_locations
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @instance
                             */
                            ListPlannableLocationsResponse.prototype.plannable_locations = $util.emptyArray;
    
                            /**
                             * Creates a new ListPlannableLocationsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsResponse} ListPlannableLocationsResponse instance
                             */
                            ListPlannableLocationsResponse.create = function create(properties) {
                                return new ListPlannableLocationsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListPlannableLocationsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableLocationsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsResponse} m ListPlannableLocationsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableLocationsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.plannable_locations != null && m.plannable_locations.length) {
                                    for (var i = 0; i < m.plannable_locations.length; ++i)
                                        $root.google.ads.googleads.v11.services.PlannableLocation.encode(m.plannable_locations[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListPlannableLocationsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableLocationsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableLocationsResponse} message ListPlannableLocationsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableLocationsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListPlannableLocationsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsResponse} ListPlannableLocationsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableLocationsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListPlannableLocationsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.plannable_locations && m.plannable_locations.length))
                                            m.plannable_locations = [];
                                        m.plannable_locations.push($root.google.ads.googleads.v11.services.PlannableLocation.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListPlannableLocationsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsResponse} ListPlannableLocationsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableLocationsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListPlannableLocationsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListPlannableLocationsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.plannable_locations != null && m.hasOwnProperty("plannable_locations")) {
                                    if (!Array.isArray(m.plannable_locations))
                                        return "plannable_locations: array expected";
                                    for (var i = 0; i < m.plannable_locations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.PlannableLocation.verify(m.plannable_locations[i]);
                                            if (e)
                                                return "plannable_locations." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListPlannableLocationsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListPlannableLocationsResponse} ListPlannableLocationsResponse
                             */
                            ListPlannableLocationsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListPlannableLocationsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListPlannableLocationsResponse();
                                if (d.plannable_locations) {
                                    if (!Array.isArray(d.plannable_locations))
                                        throw TypeError(".google.ads.googleads.v11.services.ListPlannableLocationsResponse.plannable_locations: array expected");
                                    m.plannable_locations = [];
                                    for (var i = 0; i < d.plannable_locations.length; ++i) {
                                        if (typeof d.plannable_locations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListPlannableLocationsResponse.plannable_locations: object expected");
                                        m.plannable_locations[i] = $root.google.ads.googleads.v11.services.PlannableLocation.fromObject(d.plannable_locations[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListPlannableLocationsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListPlannableLocationsResponse} m ListPlannableLocationsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListPlannableLocationsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.plannable_locations = [];
                                }
                                if (m.plannable_locations && m.plannable_locations.length) {
                                    d.plannable_locations = [];
                                    for (var j = 0; j < m.plannable_locations.length; ++j) {
                                        d.plannable_locations[j] = $root.google.ads.googleads.v11.services.PlannableLocation.toObject(m.plannable_locations[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListPlannableLocationsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListPlannableLocationsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListPlannableLocationsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListPlannableLocationsResponse;
                        })();
    
                        services.PlannableLocation = (function() {
    
                            /**
                             * Properties of a PlannableLocation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPlannableLocation
                             * @property {string|null} [id] PlannableLocation id
                             * @property {string|null} [name] PlannableLocation name
                             * @property {number|null} [parent_country_id] PlannableLocation parent_country_id
                             * @property {string|null} [country_code] PlannableLocation country_code
                             * @property {string|null} [location_type] PlannableLocation location_type
                             */
    
                            /**
                             * Constructs a new PlannableLocation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PlannableLocation.
                             * @implements IPlannableLocation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPlannableLocation=} [p] Properties to set
                             */
                            function PlannableLocation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PlannableLocation id.
                             * @member {string|null|undefined} id
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            PlannableLocation.prototype.id = null;
    
                            /**
                             * PlannableLocation name.
                             * @member {string|null|undefined} name
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            PlannableLocation.prototype.name = null;
    
                            /**
                             * PlannableLocation parent_country_id.
                             * @member {number|null|undefined} parent_country_id
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            PlannableLocation.prototype.parent_country_id = null;
    
                            /**
                             * PlannableLocation country_code.
                             * @member {string|null|undefined} country_code
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            PlannableLocation.prototype.country_code = null;
    
                            /**
                             * PlannableLocation location_type.
                             * @member {string|null|undefined} location_type
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            PlannableLocation.prototype.location_type = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PlannableLocation _id.
                             * @member {"id"|undefined} _id
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            Object.defineProperty(PlannableLocation.prototype, "_id", {
                                get: $util.oneOfGetter($oneOfFields = ["id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannableLocation _name.
                             * @member {"name"|undefined} _name
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            Object.defineProperty(PlannableLocation.prototype, "_name", {
                                get: $util.oneOfGetter($oneOfFields = ["name"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannableLocation _parent_country_id.
                             * @member {"parent_country_id"|undefined} _parent_country_id
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            Object.defineProperty(PlannableLocation.prototype, "_parent_country_id", {
                                get: $util.oneOfGetter($oneOfFields = ["parent_country_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannableLocation _country_code.
                             * @member {"country_code"|undefined} _country_code
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            Object.defineProperty(PlannableLocation.prototype, "_country_code", {
                                get: $util.oneOfGetter($oneOfFields = ["country_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannableLocation _location_type.
                             * @member {"location_type"|undefined} _location_type
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             */
                            Object.defineProperty(PlannableLocation.prototype, "_location_type", {
                                get: $util.oneOfGetter($oneOfFields = ["location_type"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PlannableLocation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannableLocation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PlannableLocation} PlannableLocation instance
                             */
                            PlannableLocation.create = function create(properties) {
                                return new PlannableLocation(properties);
                            };
    
                            /**
                             * Encodes the specified PlannableLocation message. Does not implicitly {@link google.ads.googleads.v11.services.PlannableLocation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannableLocation} m PlannableLocation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannableLocation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.id != null && Object.hasOwnProperty.call(m, "id"))
                                    w.uint32(34).string(m.id);
                                if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                                    w.uint32(42).string(m.name);
                                if (m.parent_country_id != null && Object.hasOwnProperty.call(m, "parent_country_id"))
                                    w.uint32(48).int64(m.parent_country_id);
                                if (m.country_code != null && Object.hasOwnProperty.call(m, "country_code"))
                                    w.uint32(58).string(m.country_code);
                                if (m.location_type != null && Object.hasOwnProperty.call(m, "location_type"))
                                    w.uint32(66).string(m.location_type);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlannableLocation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PlannableLocation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannableLocation} message PlannableLocation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannableLocation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlannableLocation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PlannableLocation} PlannableLocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannableLocation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PlannableLocation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.id = r.string();
                                        break;
                                    case 5:
                                        m.name = r.string();
                                        break;
                                    case 6:
                                        m.parent_country_id = r.int64();
                                        break;
                                    case 7:
                                        m.country_code = r.string();
                                        break;
                                    case 8:
                                        m.location_type = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlannableLocation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PlannableLocation} PlannableLocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannableLocation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlannableLocation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlannableLocation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    p._id = 1;
                                    if (!$util.isString(m.id))
                                        return "id: string expected";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    p._name = 1;
                                    if (!$util.isString(m.name))
                                        return "name: string expected";
                                }
                                if (m.parent_country_id != null && m.hasOwnProperty("parent_country_id")) {
                                    p._parent_country_id = 1;
                                    if (!$util.isInteger(m.parent_country_id) && !(m.parent_country_id && $util.isInteger(m.parent_country_id.low) && $util.isInteger(m.parent_country_id.high)))
                                        return "parent_country_id: integer|Long expected";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    p._country_code = 1;
                                    if (!$util.isString(m.country_code))
                                        return "country_code: string expected";
                                }
                                if (m.location_type != null && m.hasOwnProperty("location_type")) {
                                    p._location_type = 1;
                                    if (!$util.isString(m.location_type))
                                        return "location_type: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PlannableLocation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PlannableLocation} PlannableLocation
                             */
                            PlannableLocation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PlannableLocation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PlannableLocation();
                                if (d.id != null) {
                                    m.id = String(d.id);
                                }
                                if (d.name != null) {
                                    m.name = String(d.name);
                                }
                                if (d.parent_country_id != null) {
                                    if ($util.Long)
                                        (m.parent_country_id = $util.Long.fromValue(d.parent_country_id)).unsigned = false;
                                    else if (typeof d.parent_country_id === "string")
                                        m.parent_country_id = parseInt(d.parent_country_id, 10);
                                    else if (typeof d.parent_country_id === "number")
                                        m.parent_country_id = d.parent_country_id;
                                    else if (typeof d.parent_country_id === "object")
                                        m.parent_country_id = new $util.LongBits(d.parent_country_id.low >>> 0, d.parent_country_id.high >>> 0).toNumber();
                                }
                                if (d.country_code != null) {
                                    m.country_code = String(d.country_code);
                                }
                                if (d.location_type != null) {
                                    m.location_type = String(d.location_type);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PlannableLocation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @static
                             * @param {google.ads.googleads.v11.services.PlannableLocation} m PlannableLocation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlannableLocation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.id != null && m.hasOwnProperty("id")) {
                                    d.id = m.id;
                                    if (o.oneofs)
                                        d._id = "id";
                                }
                                if (m.name != null && m.hasOwnProperty("name")) {
                                    d.name = m.name;
                                    if (o.oneofs)
                                        d._name = "name";
                                }
                                if (m.parent_country_id != null && m.hasOwnProperty("parent_country_id")) {
                                    if (typeof m.parent_country_id === "number")
                                        d.parent_country_id = o.longs === String ? String(m.parent_country_id) : m.parent_country_id;
                                    else
                                        d.parent_country_id = o.longs === String ? $util.Long.prototype.toString.call(m.parent_country_id) : o.longs === Number ? new $util.LongBits(m.parent_country_id.low >>> 0, m.parent_country_id.high >>> 0).toNumber() : m.parent_country_id;
                                    if (o.oneofs)
                                        d._parent_country_id = "parent_country_id";
                                }
                                if (m.country_code != null && m.hasOwnProperty("country_code")) {
                                    d.country_code = m.country_code;
                                    if (o.oneofs)
                                        d._country_code = "country_code";
                                }
                                if (m.location_type != null && m.hasOwnProperty("location_type")) {
                                    d.location_type = m.location_type;
                                    if (o.oneofs)
                                        d._location_type = "location_type";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PlannableLocation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PlannableLocation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlannableLocation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PlannableLocation;
                        })();
    
                        services.ListPlannableProductsRequest = (function() {
    
                            /**
                             * Properties of a ListPlannableProductsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListPlannableProductsRequest
                             * @property {string|null} [plannable_location_id] ListPlannableProductsRequest plannable_location_id
                             */
    
                            /**
                             * Constructs a new ListPlannableProductsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListPlannableProductsRequest.
                             * @implements IListPlannableProductsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsRequest=} [p] Properties to set
                             */
                            function ListPlannableProductsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListPlannableProductsRequest plannable_location_id.
                             * @member {string} plannable_location_id
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @instance
                             */
                            ListPlannableProductsRequest.prototype.plannable_location_id = "";
    
                            /**
                             * Creates a new ListPlannableProductsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsRequest} ListPlannableProductsRequest instance
                             */
                            ListPlannableProductsRequest.create = function create(properties) {
                                return new ListPlannableProductsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ListPlannableProductsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableProductsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsRequest} m ListPlannableProductsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableProductsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.plannable_location_id != null && Object.hasOwnProperty.call(m, "plannable_location_id"))
                                    w.uint32(18).string(m.plannable_location_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListPlannableProductsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableProductsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsRequest} message ListPlannableProductsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableProductsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListPlannableProductsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsRequest} ListPlannableProductsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableProductsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListPlannableProductsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.plannable_location_id = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListPlannableProductsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsRequest} ListPlannableProductsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableProductsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListPlannableProductsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListPlannableProductsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.plannable_location_id != null && m.hasOwnProperty("plannable_location_id")) {
                                    if (!$util.isString(m.plannable_location_id))
                                        return "plannable_location_id: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListPlannableProductsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsRequest} ListPlannableProductsRequest
                             */
                            ListPlannableProductsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListPlannableProductsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListPlannableProductsRequest();
                                if (d.plannable_location_id != null) {
                                    m.plannable_location_id = String(d.plannable_location_id);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListPlannableProductsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ListPlannableProductsRequest} m ListPlannableProductsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListPlannableProductsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.plannable_location_id = "";
                                }
                                if (m.plannable_location_id != null && m.hasOwnProperty("plannable_location_id")) {
                                    d.plannable_location_id = m.plannable_location_id;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListPlannableProductsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListPlannableProductsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListPlannableProductsRequest;
                        })();
    
                        services.ListPlannableProductsResponse = (function() {
    
                            /**
                             * Properties of a ListPlannableProductsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IListPlannableProductsResponse
                             * @property {Array.<google.ads.googleads.v11.services.IProductMetadata>|null} [product_metadata] ListPlannableProductsResponse product_metadata
                             */
    
                            /**
                             * Constructs a new ListPlannableProductsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ListPlannableProductsResponse.
                             * @implements IListPlannableProductsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsResponse=} [p] Properties to set
                             */
                            function ListPlannableProductsResponse(p) {
                                this.product_metadata = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ListPlannableProductsResponse product_metadata.
                             * @member {Array.<google.ads.googleads.v11.services.IProductMetadata>} product_metadata
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @instance
                             */
                            ListPlannableProductsResponse.prototype.product_metadata = $util.emptyArray;
    
                            /**
                             * Creates a new ListPlannableProductsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsResponse} ListPlannableProductsResponse instance
                             */
                            ListPlannableProductsResponse.create = function create(properties) {
                                return new ListPlannableProductsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ListPlannableProductsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableProductsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsResponse} m ListPlannableProductsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableProductsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.product_metadata != null && m.product_metadata.length) {
                                    for (var i = 0; i < m.product_metadata.length; ++i)
                                        $root.google.ads.googleads.v11.services.ProductMetadata.encode(m.product_metadata[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ListPlannableProductsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ListPlannableProductsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IListPlannableProductsResponse} message ListPlannableProductsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ListPlannableProductsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ListPlannableProductsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsResponse} ListPlannableProductsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableProductsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ListPlannableProductsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.product_metadata && m.product_metadata.length))
                                            m.product_metadata = [];
                                        m.product_metadata.push($root.google.ads.googleads.v11.services.ProductMetadata.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ListPlannableProductsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsResponse} ListPlannableProductsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ListPlannableProductsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ListPlannableProductsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ListPlannableProductsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.product_metadata != null && m.hasOwnProperty("product_metadata")) {
                                    if (!Array.isArray(m.product_metadata))
                                        return "product_metadata: array expected";
                                    for (var i = 0; i < m.product_metadata.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ProductMetadata.verify(m.product_metadata[i]);
                                            if (e)
                                                return "product_metadata." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ListPlannableProductsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ListPlannableProductsResponse} ListPlannableProductsResponse
                             */
                            ListPlannableProductsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ListPlannableProductsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ListPlannableProductsResponse();
                                if (d.product_metadata) {
                                    if (!Array.isArray(d.product_metadata))
                                        throw TypeError(".google.ads.googleads.v11.services.ListPlannableProductsResponse.product_metadata: array expected");
                                    m.product_metadata = [];
                                    for (var i = 0; i < d.product_metadata.length; ++i) {
                                        if (typeof d.product_metadata[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ListPlannableProductsResponse.product_metadata: object expected");
                                        m.product_metadata[i] = $root.google.ads.googleads.v11.services.ProductMetadata.fromObject(d.product_metadata[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ListPlannableProductsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ListPlannableProductsResponse} m ListPlannableProductsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ListPlannableProductsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.product_metadata = [];
                                }
                                if (m.product_metadata && m.product_metadata.length) {
                                    d.product_metadata = [];
                                    for (var j = 0; j < m.product_metadata.length; ++j) {
                                        d.product_metadata[j] = $root.google.ads.googleads.v11.services.ProductMetadata.toObject(m.product_metadata[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ListPlannableProductsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ListPlannableProductsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ListPlannableProductsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ListPlannableProductsResponse;
                        })();
    
                        services.ProductMetadata = (function() {
    
                            /**
                             * Properties of a ProductMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IProductMetadata
                             * @property {string|null} [plannable_product_code] ProductMetadata plannable_product_code
                             * @property {string|null} [plannable_product_name] ProductMetadata plannable_product_name
                             * @property {google.ads.googleads.v11.services.IPlannableTargeting|null} [plannable_targeting] ProductMetadata plannable_targeting
                             */
    
                            /**
                             * Constructs a new ProductMetadata.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ProductMetadata.
                             * @implements IProductMetadata
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IProductMetadata=} [p] Properties to set
                             */
                            function ProductMetadata(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductMetadata plannable_product_code.
                             * @member {string|null|undefined} plannable_product_code
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @instance
                             */
                            ProductMetadata.prototype.plannable_product_code = null;
    
                            /**
                             * ProductMetadata plannable_product_name.
                             * @member {string} plannable_product_name
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @instance
                             */
                            ProductMetadata.prototype.plannable_product_name = "";
    
                            /**
                             * ProductMetadata plannable_targeting.
                             * @member {google.ads.googleads.v11.services.IPlannableTargeting|null|undefined} plannable_targeting
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @instance
                             */
                            ProductMetadata.prototype.plannable_targeting = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductMetadata _plannable_product_code.
                             * @member {"plannable_product_code"|undefined} _plannable_product_code
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @instance
                             */
                            Object.defineProperty(ProductMetadata.prototype, "_plannable_product_code", {
                                get: $util.oneOfGetter($oneOfFields = ["plannable_product_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductMetadata instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IProductMetadata=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ProductMetadata} ProductMetadata instance
                             */
                            ProductMetadata.create = function create(properties) {
                                return new ProductMetadata(properties);
                            };
    
                            /**
                             * Encodes the specified ProductMetadata message. Does not implicitly {@link google.ads.googleads.v11.services.ProductMetadata.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IProductMetadata} m ProductMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductMetadata.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.plannable_targeting != null && Object.hasOwnProperty.call(m, "plannable_targeting"))
                                    $root.google.ads.googleads.v11.services.PlannableTargeting.encode(m.plannable_targeting, w.uint32(18).fork()).ldelim();
                                if (m.plannable_product_name != null && Object.hasOwnProperty.call(m, "plannable_product_name"))
                                    w.uint32(26).string(m.plannable_product_name);
                                if (m.plannable_product_code != null && Object.hasOwnProperty.call(m, "plannable_product_code"))
                                    w.uint32(34).string(m.plannable_product_code);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductMetadata message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ProductMetadata.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.IProductMetadata} message ProductMetadata message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductMetadata message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ProductMetadata} ProductMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductMetadata.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ProductMetadata();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 4:
                                        m.plannable_product_code = r.string();
                                        break;
                                    case 3:
                                        m.plannable_product_name = r.string();
                                        break;
                                    case 2:
                                        m.plannable_targeting = $root.google.ads.googleads.v11.services.PlannableTargeting.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductMetadata message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ProductMetadata} ProductMetadata
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductMetadata.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductMetadata message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductMetadata.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    p._plannable_product_code = 1;
                                    if (!$util.isString(m.plannable_product_code))
                                        return "plannable_product_code: string expected";
                                }
                                if (m.plannable_product_name != null && m.hasOwnProperty("plannable_product_name")) {
                                    if (!$util.isString(m.plannable_product_name))
                                        return "plannable_product_name: string expected";
                                }
                                if (m.plannable_targeting != null && m.hasOwnProperty("plannable_targeting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.PlannableTargeting.verify(m.plannable_targeting);
                                        if (e)
                                            return "plannable_targeting." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductMetadata message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ProductMetadata} ProductMetadata
                             */
                            ProductMetadata.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ProductMetadata)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ProductMetadata();
                                if (d.plannable_product_code != null) {
                                    m.plannable_product_code = String(d.plannable_product_code);
                                }
                                if (d.plannable_product_name != null) {
                                    m.plannable_product_name = String(d.plannable_product_name);
                                }
                                if (d.plannable_targeting != null) {
                                    if (typeof d.plannable_targeting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ProductMetadata.plannable_targeting: object expected");
                                    m.plannable_targeting = $root.google.ads.googleads.v11.services.PlannableTargeting.fromObject(d.plannable_targeting);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductMetadata message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @static
                             * @param {google.ads.googleads.v11.services.ProductMetadata} m ProductMetadata
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductMetadata.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.plannable_targeting = null;
                                    d.plannable_product_name = "";
                                }
                                if (m.plannable_targeting != null && m.hasOwnProperty("plannable_targeting")) {
                                    d.plannable_targeting = $root.google.ads.googleads.v11.services.PlannableTargeting.toObject(m.plannable_targeting, o);
                                }
                                if (m.plannable_product_name != null && m.hasOwnProperty("plannable_product_name")) {
                                    d.plannable_product_name = m.plannable_product_name;
                                }
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    d.plannable_product_code = m.plannable_product_code;
                                    if (o.oneofs)
                                        d._plannable_product_code = "plannable_product_code";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductMetadata to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ProductMetadata
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductMetadata.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductMetadata;
                        })();
    
                        services.PlannableTargeting = (function() {
    
                            /**
                             * Properties of a PlannableTargeting.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPlannableTargeting
                             * @property {Array.<google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.ReachPlanAgeRange>|null} [age_ranges] PlannableTargeting age_ranges
                             * @property {Array.<google.ads.googleads.v11.common.IGenderInfo>|null} [genders] PlannableTargeting genders
                             * @property {Array.<google.ads.googleads.v11.common.IDeviceInfo>|null} [devices] PlannableTargeting devices
                             * @property {Array.<google.ads.googleads.v11.enums.ReachPlanNetworkEnum.ReachPlanNetwork>|null} [networks] PlannableTargeting networks
                             */
    
                            /**
                             * Constructs a new PlannableTargeting.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PlannableTargeting.
                             * @implements IPlannableTargeting
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPlannableTargeting=} [p] Properties to set
                             */
                            function PlannableTargeting(p) {
                                this.age_ranges = [];
                                this.genders = [];
                                this.devices = [];
                                this.networks = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PlannableTargeting age_ranges.
                             * @member {Array.<google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.ReachPlanAgeRange>} age_ranges
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @instance
                             */
                            PlannableTargeting.prototype.age_ranges = $util.emptyArray;
    
                            /**
                             * PlannableTargeting genders.
                             * @member {Array.<google.ads.googleads.v11.common.IGenderInfo>} genders
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @instance
                             */
                            PlannableTargeting.prototype.genders = $util.emptyArray;
    
                            /**
                             * PlannableTargeting devices.
                             * @member {Array.<google.ads.googleads.v11.common.IDeviceInfo>} devices
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @instance
                             */
                            PlannableTargeting.prototype.devices = $util.emptyArray;
    
                            /**
                             * PlannableTargeting networks.
                             * @member {Array.<google.ads.googleads.v11.enums.ReachPlanNetworkEnum.ReachPlanNetwork>} networks
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @instance
                             */
                            PlannableTargeting.prototype.networks = $util.emptyArray;
    
                            /**
                             * Creates a new PlannableTargeting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannableTargeting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PlannableTargeting} PlannableTargeting instance
                             */
                            PlannableTargeting.create = function create(properties) {
                                return new PlannableTargeting(properties);
                            };
    
                            /**
                             * Encodes the specified PlannableTargeting message. Does not implicitly {@link google.ads.googleads.v11.services.PlannableTargeting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannableTargeting} m PlannableTargeting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannableTargeting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.age_ranges != null && m.age_ranges.length) {
                                    w.uint32(10).fork();
                                    for (var i = 0; i < m.age_ranges.length; ++i)
                                        w.int32(m.age_ranges[i]);
                                    w.ldelim();
                                }
                                if (m.genders != null && m.genders.length) {
                                    for (var i = 0; i < m.genders.length; ++i)
                                        $root.google.ads.googleads.v11.common.GenderInfo.encode(m.genders[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.devices != null && m.devices.length) {
                                    for (var i = 0; i < m.devices.length; ++i)
                                        $root.google.ads.googleads.v11.common.DeviceInfo.encode(m.devices[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.networks != null && m.networks.length) {
                                    w.uint32(34).fork();
                                    for (var i = 0; i < m.networks.length; ++i)
                                        w.int32(m.networks[i]);
                                    w.ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlannableTargeting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PlannableTargeting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannableTargeting} message PlannableTargeting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannableTargeting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlannableTargeting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PlannableTargeting} PlannableTargeting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannableTargeting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PlannableTargeting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.age_ranges && m.age_ranges.length))
                                            m.age_ranges = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.age_ranges.push(r.int32());
                                        } else
                                            m.age_ranges.push(r.int32());
                                        break;
                                    case 2:
                                        if (!(m.genders && m.genders.length))
                                            m.genders = [];
                                        m.genders.push($root.google.ads.googleads.v11.common.GenderInfo.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        if (!(m.devices && m.devices.length))
                                            m.devices = [];
                                        m.devices.push($root.google.ads.googleads.v11.common.DeviceInfo.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.networks && m.networks.length))
                                            m.networks = [];
                                        if ((t & 7) === 2) {
                                            var c2 = r.uint32() + r.pos;
                                            while (r.pos < c2)
                                                m.networks.push(r.int32());
                                        } else
                                            m.networks.push(r.int32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlannableTargeting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PlannableTargeting} PlannableTargeting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannableTargeting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlannableTargeting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlannableTargeting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.age_ranges != null && m.hasOwnProperty("age_ranges")) {
                                    if (!Array.isArray(m.age_ranges))
                                        return "age_ranges: array expected";
                                    for (var i = 0; i < m.age_ranges.length; ++i) {
                                        switch (m.age_ranges[i]) {
                                        default:
                                            return "age_ranges: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 503001:
                                        case 2:
                                        case 3:
                                        case 4:
                                        case 5:
                                        case 6:
                                        case 7:
                                        case 8:
                                        case 503002:
                                        case 9:
                                        case 10:
                                        case 11:
                                        case 12:
                                        case 13:
                                        case 503003:
                                        case 14:
                                        case 15:
                                        case 16:
                                        case 17:
                                        case 503004:
                                        case 18:
                                        case 19:
                                        case 20:
                                        case 503005:
                                        case 21:
                                        case 503006:
                                            break;
                                        }
                                    }
                                }
                                if (m.genders != null && m.hasOwnProperty("genders")) {
                                    if (!Array.isArray(m.genders))
                                        return "genders: array expected";
                                    for (var i = 0; i < m.genders.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.GenderInfo.verify(m.genders[i]);
                                            if (e)
                                                return "genders." + e;
                                        }
                                    }
                                }
                                if (m.devices != null && m.hasOwnProperty("devices")) {
                                    if (!Array.isArray(m.devices))
                                        return "devices: array expected";
                                    for (var i = 0; i < m.devices.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.DeviceInfo.verify(m.devices[i]);
                                            if (e)
                                                return "devices." + e;
                                        }
                                    }
                                }
                                if (m.networks != null && m.hasOwnProperty("networks")) {
                                    if (!Array.isArray(m.networks))
                                        return "networks: array expected";
                                    for (var i = 0; i < m.networks.length; ++i) {
                                        switch (m.networks[i]) {
                                        default:
                                            return "networks: enum value[] expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PlannableTargeting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PlannableTargeting} PlannableTargeting
                             */
                            PlannableTargeting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PlannableTargeting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PlannableTargeting();
                                if (d.age_ranges) {
                                    if (!Array.isArray(d.age_ranges))
                                        throw TypeError(".google.ads.googleads.v11.services.PlannableTargeting.age_ranges: array expected");
                                    m.age_ranges = [];
                                    for (var i = 0; i < d.age_ranges.length; ++i) {
                                        switch (d.age_ranges[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.age_ranges[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.age_ranges[i] = 1;
                                            break;
                                        case "AGE_RANGE_18_24":
                                        case 503001:
                                            m.age_ranges[i] = 503001;
                                            break;
                                        case "AGE_RANGE_18_34":
                                        case 2:
                                            m.age_ranges[i] = 2;
                                            break;
                                        case "AGE_RANGE_18_44":
                                        case 3:
                                            m.age_ranges[i] = 3;
                                            break;
                                        case "AGE_RANGE_18_49":
                                        case 4:
                                            m.age_ranges[i] = 4;
                                            break;
                                        case "AGE_RANGE_18_54":
                                        case 5:
                                            m.age_ranges[i] = 5;
                                            break;
                                        case "AGE_RANGE_18_64":
                                        case 6:
                                            m.age_ranges[i] = 6;
                                            break;
                                        case "AGE_RANGE_18_65_UP":
                                        case 7:
                                            m.age_ranges[i] = 7;
                                            break;
                                        case "AGE_RANGE_21_34":
                                        case 8:
                                            m.age_ranges[i] = 8;
                                            break;
                                        case "AGE_RANGE_25_34":
                                        case 503002:
                                            m.age_ranges[i] = 503002;
                                            break;
                                        case "AGE_RANGE_25_44":
                                        case 9:
                                            m.age_ranges[i] = 9;
                                            break;
                                        case "AGE_RANGE_25_49":
                                        case 10:
                                            m.age_ranges[i] = 10;
                                            break;
                                        case "AGE_RANGE_25_54":
                                        case 11:
                                            m.age_ranges[i] = 11;
                                            break;
                                        case "AGE_RANGE_25_64":
                                        case 12:
                                            m.age_ranges[i] = 12;
                                            break;
                                        case "AGE_RANGE_25_65_UP":
                                        case 13:
                                            m.age_ranges[i] = 13;
                                            break;
                                        case "AGE_RANGE_35_44":
                                        case 503003:
                                            m.age_ranges[i] = 503003;
                                            break;
                                        case "AGE_RANGE_35_49":
                                        case 14:
                                            m.age_ranges[i] = 14;
                                            break;
                                        case "AGE_RANGE_35_54":
                                        case 15:
                                            m.age_ranges[i] = 15;
                                            break;
                                        case "AGE_RANGE_35_64":
                                        case 16:
                                            m.age_ranges[i] = 16;
                                            break;
                                        case "AGE_RANGE_35_65_UP":
                                        case 17:
                                            m.age_ranges[i] = 17;
                                            break;
                                        case "AGE_RANGE_45_54":
                                        case 503004:
                                            m.age_ranges[i] = 503004;
                                            break;
                                        case "AGE_RANGE_45_64":
                                        case 18:
                                            m.age_ranges[i] = 18;
                                            break;
                                        case "AGE_RANGE_45_65_UP":
                                        case 19:
                                            m.age_ranges[i] = 19;
                                            break;
                                        case "AGE_RANGE_50_65_UP":
                                        case 20:
                                            m.age_ranges[i] = 20;
                                            break;
                                        case "AGE_RANGE_55_64":
                                        case 503005:
                                            m.age_ranges[i] = 503005;
                                            break;
                                        case "AGE_RANGE_55_65_UP":
                                        case 21:
                                            m.age_ranges[i] = 21;
                                            break;
                                        case "AGE_RANGE_65_UP":
                                        case 503006:
                                            m.age_ranges[i] = 503006;
                                            break;
                                        }
                                    }
                                }
                                if (d.genders) {
                                    if (!Array.isArray(d.genders))
                                        throw TypeError(".google.ads.googleads.v11.services.PlannableTargeting.genders: array expected");
                                    m.genders = [];
                                    for (var i = 0; i < d.genders.length; ++i) {
                                        if (typeof d.genders[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.PlannableTargeting.genders: object expected");
                                        m.genders[i] = $root.google.ads.googleads.v11.common.GenderInfo.fromObject(d.genders[i]);
                                    }
                                }
                                if (d.devices) {
                                    if (!Array.isArray(d.devices))
                                        throw TypeError(".google.ads.googleads.v11.services.PlannableTargeting.devices: array expected");
                                    m.devices = [];
                                    for (var i = 0; i < d.devices.length; ++i) {
                                        if (typeof d.devices[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.PlannableTargeting.devices: object expected");
                                        m.devices[i] = $root.google.ads.googleads.v11.common.DeviceInfo.fromObject(d.devices[i]);
                                    }
                                }
                                if (d.networks) {
                                    if (!Array.isArray(d.networks))
                                        throw TypeError(".google.ads.googleads.v11.services.PlannableTargeting.networks: array expected");
                                    m.networks = [];
                                    for (var i = 0; i < d.networks.length; ++i) {
                                        switch (d.networks[i]) {
                                        default:
                                        case "UNSPECIFIED":
                                        case 0:
                                            m.networks[i] = 0;
                                            break;
                                        case "UNKNOWN":
                                        case 1:
                                            m.networks[i] = 1;
                                            break;
                                        case "YOUTUBE":
                                        case 2:
                                            m.networks[i] = 2;
                                            break;
                                        case "GOOGLE_VIDEO_PARTNERS":
                                        case 3:
                                            m.networks[i] = 3;
                                            break;
                                        case "YOUTUBE_AND_GOOGLE_VIDEO_PARTNERS":
                                        case 4:
                                            m.networks[i] = 4;
                                            break;
                                        }
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PlannableTargeting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @static
                             * @param {google.ads.googleads.v11.services.PlannableTargeting} m PlannableTargeting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlannableTargeting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.age_ranges = [];
                                    d.genders = [];
                                    d.devices = [];
                                    d.networks = [];
                                }
                                if (m.age_ranges && m.age_ranges.length) {
                                    d.age_ranges = [];
                                    for (var j = 0; j < m.age_ranges.length; ++j) {
                                        d.age_ranges[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.ReachPlanAgeRange[m.age_ranges[j]] : m.age_ranges[j];
                                    }
                                }
                                if (m.genders && m.genders.length) {
                                    d.genders = [];
                                    for (var j = 0; j < m.genders.length; ++j) {
                                        d.genders[j] = $root.google.ads.googleads.v11.common.GenderInfo.toObject(m.genders[j], o);
                                    }
                                }
                                if (m.devices && m.devices.length) {
                                    d.devices = [];
                                    for (var j = 0; j < m.devices.length; ++j) {
                                        d.devices[j] = $root.google.ads.googleads.v11.common.DeviceInfo.toObject(m.devices[j], o);
                                    }
                                }
                                if (m.networks && m.networks.length) {
                                    d.networks = [];
                                    for (var j = 0; j < m.networks.length; ++j) {
                                        d.networks[j] = o.enums === String ? $root.google.ads.googleads.v11.enums.ReachPlanNetworkEnum.ReachPlanNetwork[m.networks[j]] : m.networks[j];
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PlannableTargeting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PlannableTargeting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlannableTargeting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PlannableTargeting;
                        })();
    
                        services.GenerateProductMixIdeasRequest = (function() {
    
                            /**
                             * Properties of a GenerateProductMixIdeasRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateProductMixIdeasRequest
                             * @property {string|null} [customer_id] GenerateProductMixIdeasRequest customer_id
                             * @property {string|null} [plannable_location_id] GenerateProductMixIdeasRequest plannable_location_id
                             * @property {string|null} [currency_code] GenerateProductMixIdeasRequest currency_code
                             * @property {number|null} [budget_micros] GenerateProductMixIdeasRequest budget_micros
                             * @property {google.ads.googleads.v11.services.IPreferences|null} [preferences] GenerateProductMixIdeasRequest preferences
                             */
    
                            /**
                             * Constructs a new GenerateProductMixIdeasRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateProductMixIdeasRequest.
                             * @implements IGenerateProductMixIdeasRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasRequest=} [p] Properties to set
                             */
                            function GenerateProductMixIdeasRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateProductMixIdeasRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @instance
                             */
                            GenerateProductMixIdeasRequest.prototype.customer_id = "";
    
                            /**
                             * GenerateProductMixIdeasRequest plannable_location_id.
                             * @member {string} plannable_location_id
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @instance
                             */
                            GenerateProductMixIdeasRequest.prototype.plannable_location_id = "";
    
                            /**
                             * GenerateProductMixIdeasRequest currency_code.
                             * @member {string} currency_code
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @instance
                             */
                            GenerateProductMixIdeasRequest.prototype.currency_code = "";
    
                            /**
                             * GenerateProductMixIdeasRequest budget_micros.
                             * @member {number} budget_micros
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @instance
                             */
                            GenerateProductMixIdeasRequest.prototype.budget_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * GenerateProductMixIdeasRequest preferences.
                             * @member {google.ads.googleads.v11.services.IPreferences|null|undefined} preferences
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @instance
                             */
                            GenerateProductMixIdeasRequest.prototype.preferences = null;
    
                            /**
                             * Creates a new GenerateProductMixIdeasRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasRequest} GenerateProductMixIdeasRequest instance
                             */
                            GenerateProductMixIdeasRequest.create = function create(properties) {
                                return new GenerateProductMixIdeasRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateProductMixIdeasRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateProductMixIdeasRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasRequest} m GenerateProductMixIdeasRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateProductMixIdeasRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.preferences != null && Object.hasOwnProperty.call(m, "preferences"))
                                    $root.google.ads.googleads.v11.services.Preferences.encode(m.preferences, w.uint32(42).fork()).ldelim();
                                if (m.plannable_location_id != null && Object.hasOwnProperty.call(m, "plannable_location_id"))
                                    w.uint32(50).string(m.plannable_location_id);
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(58).string(m.currency_code);
                                if (m.budget_micros != null && Object.hasOwnProperty.call(m, "budget_micros"))
                                    w.uint32(64).int64(m.budget_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateProductMixIdeasRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateProductMixIdeasRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasRequest} message GenerateProductMixIdeasRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateProductMixIdeasRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateProductMixIdeasRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasRequest} GenerateProductMixIdeasRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateProductMixIdeasRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateProductMixIdeasRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 6:
                                        m.plannable_location_id = r.string();
                                        break;
                                    case 7:
                                        m.currency_code = r.string();
                                        break;
                                    case 8:
                                        m.budget_micros = r.int64();
                                        break;
                                    case 5:
                                        m.preferences = $root.google.ads.googleads.v11.services.Preferences.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateProductMixIdeasRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasRequest} GenerateProductMixIdeasRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateProductMixIdeasRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateProductMixIdeasRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateProductMixIdeasRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.plannable_location_id != null && m.hasOwnProperty("plannable_location_id")) {
                                    if (!$util.isString(m.plannable_location_id))
                                        return "plannable_location_id: string expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.budget_micros != null && m.hasOwnProperty("budget_micros")) {
                                    if (!$util.isInteger(m.budget_micros) && !(m.budget_micros && $util.isInteger(m.budget_micros.low) && $util.isInteger(m.budget_micros.high)))
                                        return "budget_micros: integer|Long expected";
                                }
                                if (m.preferences != null && m.hasOwnProperty("preferences")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.Preferences.verify(m.preferences);
                                        if (e)
                                            return "preferences." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateProductMixIdeasRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasRequest} GenerateProductMixIdeasRequest
                             */
                            GenerateProductMixIdeasRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateProductMixIdeasRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateProductMixIdeasRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.plannable_location_id != null) {
                                    m.plannable_location_id = String(d.plannable_location_id);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.budget_micros != null) {
                                    if ($util.Long)
                                        (m.budget_micros = $util.Long.fromValue(d.budget_micros)).unsigned = false;
                                    else if (typeof d.budget_micros === "string")
                                        m.budget_micros = parseInt(d.budget_micros, 10);
                                    else if (typeof d.budget_micros === "number")
                                        m.budget_micros = d.budget_micros;
                                    else if (typeof d.budget_micros === "object")
                                        m.budget_micros = new $util.LongBits(d.budget_micros.low >>> 0, d.budget_micros.high >>> 0).toNumber();
                                }
                                if (d.preferences != null) {
                                    if (typeof d.preferences !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateProductMixIdeasRequest.preferences: object expected");
                                    m.preferences = $root.google.ads.googleads.v11.services.Preferences.fromObject(d.preferences);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateProductMixIdeasRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateProductMixIdeasRequest} m GenerateProductMixIdeasRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateProductMixIdeasRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.preferences = null;
                                    d.plannable_location_id = "";
                                    d.currency_code = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.budget_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.budget_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.preferences != null && m.hasOwnProperty("preferences")) {
                                    d.preferences = $root.google.ads.googleads.v11.services.Preferences.toObject(m.preferences, o);
                                }
                                if (m.plannable_location_id != null && m.hasOwnProperty("plannable_location_id")) {
                                    d.plannable_location_id = m.plannable_location_id;
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                }
                                if (m.budget_micros != null && m.hasOwnProperty("budget_micros")) {
                                    if (typeof m.budget_micros === "number")
                                        d.budget_micros = o.longs === String ? String(m.budget_micros) : m.budget_micros;
                                    else
                                        d.budget_micros = o.longs === String ? $util.Long.prototype.toString.call(m.budget_micros) : o.longs === Number ? new $util.LongBits(m.budget_micros.low >>> 0, m.budget_micros.high >>> 0).toNumber() : m.budget_micros;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateProductMixIdeasRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateProductMixIdeasRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateProductMixIdeasRequest;
                        })();
    
                        services.Preferences = (function() {
    
                            /**
                             * Properties of a Preferences.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPreferences
                             * @property {boolean|null} [is_skippable] Preferences is_skippable
                             * @property {boolean|null} [starts_with_sound] Preferences starts_with_sound
                             * @property {google.ads.googleads.v11.enums.ReachPlanAdLengthEnum.ReachPlanAdLength|null} [ad_length] Preferences ad_length
                             * @property {boolean|null} [top_content_only] Preferences top_content_only
                             * @property {boolean|null} [has_guaranteed_price] Preferences has_guaranteed_price
                             */
    
                            /**
                             * Constructs a new Preferences.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a Preferences.
                             * @implements IPreferences
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPreferences=} [p] Properties to set
                             */
                            function Preferences(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Preferences is_skippable.
                             * @member {boolean|null|undefined} is_skippable
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Preferences.prototype.is_skippable = null;
    
                            /**
                             * Preferences starts_with_sound.
                             * @member {boolean|null|undefined} starts_with_sound
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Preferences.prototype.starts_with_sound = null;
    
                            /**
                             * Preferences ad_length.
                             * @member {google.ads.googleads.v11.enums.ReachPlanAdLengthEnum.ReachPlanAdLength} ad_length
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Preferences.prototype.ad_length = 0;
    
                            /**
                             * Preferences top_content_only.
                             * @member {boolean|null|undefined} top_content_only
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Preferences.prototype.top_content_only = null;
    
                            /**
                             * Preferences has_guaranteed_price.
                             * @member {boolean|null|undefined} has_guaranteed_price
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Preferences.prototype.has_guaranteed_price = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Preferences _is_skippable.
                             * @member {"is_skippable"|undefined} _is_skippable
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Object.defineProperty(Preferences.prototype, "_is_skippable", {
                                get: $util.oneOfGetter($oneOfFields = ["is_skippable"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Preferences _starts_with_sound.
                             * @member {"starts_with_sound"|undefined} _starts_with_sound
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Object.defineProperty(Preferences.prototype, "_starts_with_sound", {
                                get: $util.oneOfGetter($oneOfFields = ["starts_with_sound"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Preferences _top_content_only.
                             * @member {"top_content_only"|undefined} _top_content_only
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Object.defineProperty(Preferences.prototype, "_top_content_only", {
                                get: $util.oneOfGetter($oneOfFields = ["top_content_only"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Preferences _has_guaranteed_price.
                             * @member {"has_guaranteed_price"|undefined} _has_guaranteed_price
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             */
                            Object.defineProperty(Preferences.prototype, "_has_guaranteed_price", {
                                get: $util.oneOfGetter($oneOfFields = ["has_guaranteed_price"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Preferences instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {google.ads.googleads.v11.services.IPreferences=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.Preferences} Preferences instance
                             */
                            Preferences.create = function create(properties) {
                                return new Preferences(properties);
                            };
    
                            /**
                             * Encodes the specified Preferences message. Does not implicitly {@link google.ads.googleads.v11.services.Preferences.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {google.ads.googleads.v11.services.IPreferences} m Preferences message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Preferences.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_length != null && Object.hasOwnProperty.call(m, "ad_length"))
                                    w.uint32(24).int32(m.ad_length);
                                if (m.is_skippable != null && Object.hasOwnProperty.call(m, "is_skippable"))
                                    w.uint32(48).bool(m.is_skippable);
                                if (m.starts_with_sound != null && Object.hasOwnProperty.call(m, "starts_with_sound"))
                                    w.uint32(56).bool(m.starts_with_sound);
                                if (m.top_content_only != null && Object.hasOwnProperty.call(m, "top_content_only"))
                                    w.uint32(64).bool(m.top_content_only);
                                if (m.has_guaranteed_price != null && Object.hasOwnProperty.call(m, "has_guaranteed_price"))
                                    w.uint32(72).bool(m.has_guaranteed_price);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Preferences message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.Preferences.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {google.ads.googleads.v11.services.IPreferences} message Preferences message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Preferences.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Preferences message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.Preferences} Preferences
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Preferences.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.Preferences();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.is_skippable = r.bool();
                                        break;
                                    case 7:
                                        m.starts_with_sound = r.bool();
                                        break;
                                    case 3:
                                        m.ad_length = r.int32();
                                        break;
                                    case 8:
                                        m.top_content_only = r.bool();
                                        break;
                                    case 9:
                                        m.has_guaranteed_price = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Preferences message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.Preferences} Preferences
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Preferences.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Preferences message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Preferences.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.is_skippable != null && m.hasOwnProperty("is_skippable")) {
                                    p._is_skippable = 1;
                                    if (typeof m.is_skippable !== "boolean")
                                        return "is_skippable: boolean expected";
                                }
                                if (m.starts_with_sound != null && m.hasOwnProperty("starts_with_sound")) {
                                    p._starts_with_sound = 1;
                                    if (typeof m.starts_with_sound !== "boolean")
                                        return "starts_with_sound: boolean expected";
                                }
                                if (m.ad_length != null && m.hasOwnProperty("ad_length")) {
                                    switch (m.ad_length) {
                                    default:
                                        return "ad_length: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                if (m.top_content_only != null && m.hasOwnProperty("top_content_only")) {
                                    p._top_content_only = 1;
                                    if (typeof m.top_content_only !== "boolean")
                                        return "top_content_only: boolean expected";
                                }
                                if (m.has_guaranteed_price != null && m.hasOwnProperty("has_guaranteed_price")) {
                                    p._has_guaranteed_price = 1;
                                    if (typeof m.has_guaranteed_price !== "boolean")
                                        return "has_guaranteed_price: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Preferences message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.Preferences} Preferences
                             */
                            Preferences.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.Preferences)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.Preferences();
                                if (d.is_skippable != null) {
                                    m.is_skippable = Boolean(d.is_skippable);
                                }
                                if (d.starts_with_sound != null) {
                                    m.starts_with_sound = Boolean(d.starts_with_sound);
                                }
                                switch (d.ad_length) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.ad_length = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.ad_length = 1;
                                    break;
                                case "SIX_SECONDS":
                                case 2:
                                    m.ad_length = 2;
                                    break;
                                case "FIFTEEN_OR_TWENTY_SECONDS":
                                case 3:
                                    m.ad_length = 3;
                                    break;
                                case "TWENTY_SECONDS_OR_MORE":
                                case 4:
                                    m.ad_length = 4;
                                    break;
                                }
                                if (d.top_content_only != null) {
                                    m.top_content_only = Boolean(d.top_content_only);
                                }
                                if (d.has_guaranteed_price != null) {
                                    m.has_guaranteed_price = Boolean(d.has_guaranteed_price);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Preferences message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @static
                             * @param {google.ads.googleads.v11.services.Preferences} m Preferences
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Preferences.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.ad_length = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.ad_length != null && m.hasOwnProperty("ad_length")) {
                                    d.ad_length = o.enums === String ? $root.google.ads.googleads.v11.enums.ReachPlanAdLengthEnum.ReachPlanAdLength[m.ad_length] : m.ad_length;
                                }
                                if (m.is_skippable != null && m.hasOwnProperty("is_skippable")) {
                                    d.is_skippable = m.is_skippable;
                                    if (o.oneofs)
                                        d._is_skippable = "is_skippable";
                                }
                                if (m.starts_with_sound != null && m.hasOwnProperty("starts_with_sound")) {
                                    d.starts_with_sound = m.starts_with_sound;
                                    if (o.oneofs)
                                        d._starts_with_sound = "starts_with_sound";
                                }
                                if (m.top_content_only != null && m.hasOwnProperty("top_content_only")) {
                                    d.top_content_only = m.top_content_only;
                                    if (o.oneofs)
                                        d._top_content_only = "top_content_only";
                                }
                                if (m.has_guaranteed_price != null && m.hasOwnProperty("has_guaranteed_price")) {
                                    d.has_guaranteed_price = m.has_guaranteed_price;
                                    if (o.oneofs)
                                        d._has_guaranteed_price = "has_guaranteed_price";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Preferences to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.Preferences
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Preferences.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Preferences;
                        })();
    
                        services.GenerateProductMixIdeasResponse = (function() {
    
                            /**
                             * Properties of a GenerateProductMixIdeasResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateProductMixIdeasResponse
                             * @property {Array.<google.ads.googleads.v11.services.IProductAllocation>|null} [product_allocation] GenerateProductMixIdeasResponse product_allocation
                             */
    
                            /**
                             * Constructs a new GenerateProductMixIdeasResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateProductMixIdeasResponse.
                             * @implements IGenerateProductMixIdeasResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasResponse=} [p] Properties to set
                             */
                            function GenerateProductMixIdeasResponse(p) {
                                this.product_allocation = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateProductMixIdeasResponse product_allocation.
                             * @member {Array.<google.ads.googleads.v11.services.IProductAllocation>} product_allocation
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @instance
                             */
                            GenerateProductMixIdeasResponse.prototype.product_allocation = $util.emptyArray;
    
                            /**
                             * Creates a new GenerateProductMixIdeasResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasResponse} GenerateProductMixIdeasResponse instance
                             */
                            GenerateProductMixIdeasResponse.create = function create(properties) {
                                return new GenerateProductMixIdeasResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateProductMixIdeasResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateProductMixIdeasResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasResponse} m GenerateProductMixIdeasResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateProductMixIdeasResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.product_allocation != null && m.product_allocation.length) {
                                    for (var i = 0; i < m.product_allocation.length; ++i)
                                        $root.google.ads.googleads.v11.services.ProductAllocation.encode(m.product_allocation[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateProductMixIdeasResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateProductMixIdeasResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateProductMixIdeasResponse} message GenerateProductMixIdeasResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateProductMixIdeasResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateProductMixIdeasResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasResponse} GenerateProductMixIdeasResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateProductMixIdeasResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateProductMixIdeasResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.product_allocation && m.product_allocation.length))
                                            m.product_allocation = [];
                                        m.product_allocation.push($root.google.ads.googleads.v11.services.ProductAllocation.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateProductMixIdeasResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasResponse} GenerateProductMixIdeasResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateProductMixIdeasResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateProductMixIdeasResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateProductMixIdeasResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.product_allocation != null && m.hasOwnProperty("product_allocation")) {
                                    if (!Array.isArray(m.product_allocation))
                                        return "product_allocation: array expected";
                                    for (var i = 0; i < m.product_allocation.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ProductAllocation.verify(m.product_allocation[i]);
                                            if (e)
                                                return "product_allocation." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateProductMixIdeasResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateProductMixIdeasResponse} GenerateProductMixIdeasResponse
                             */
                            GenerateProductMixIdeasResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateProductMixIdeasResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateProductMixIdeasResponse();
                                if (d.product_allocation) {
                                    if (!Array.isArray(d.product_allocation))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateProductMixIdeasResponse.product_allocation: array expected");
                                    m.product_allocation = [];
                                    for (var i = 0; i < d.product_allocation.length; ++i) {
                                        if (typeof d.product_allocation[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateProductMixIdeasResponse.product_allocation: object expected");
                                        m.product_allocation[i] = $root.google.ads.googleads.v11.services.ProductAllocation.fromObject(d.product_allocation[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateProductMixIdeasResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateProductMixIdeasResponse} m GenerateProductMixIdeasResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateProductMixIdeasResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.product_allocation = [];
                                }
                                if (m.product_allocation && m.product_allocation.length) {
                                    d.product_allocation = [];
                                    for (var j = 0; j < m.product_allocation.length; ++j) {
                                        d.product_allocation[j] = $root.google.ads.googleads.v11.services.ProductAllocation.toObject(m.product_allocation[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateProductMixIdeasResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateProductMixIdeasResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateProductMixIdeasResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateProductMixIdeasResponse;
                        })();
    
                        services.ProductAllocation = (function() {
    
                            /**
                             * Properties of a ProductAllocation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IProductAllocation
                             * @property {string|null} [plannable_product_code] ProductAllocation plannable_product_code
                             * @property {number|null} [budget_micros] ProductAllocation budget_micros
                             */
    
                            /**
                             * Constructs a new ProductAllocation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ProductAllocation.
                             * @implements IProductAllocation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IProductAllocation=} [p] Properties to set
                             */
                            function ProductAllocation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ProductAllocation plannable_product_code.
                             * @member {string|null|undefined} plannable_product_code
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @instance
                             */
                            ProductAllocation.prototype.plannable_product_code = null;
    
                            /**
                             * ProductAllocation budget_micros.
                             * @member {number|null|undefined} budget_micros
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @instance
                             */
                            ProductAllocation.prototype.budget_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ProductAllocation _plannable_product_code.
                             * @member {"plannable_product_code"|undefined} _plannable_product_code
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @instance
                             */
                            Object.defineProperty(ProductAllocation.prototype, "_plannable_product_code", {
                                get: $util.oneOfGetter($oneOfFields = ["plannable_product_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * ProductAllocation _budget_micros.
                             * @member {"budget_micros"|undefined} _budget_micros
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @instance
                             */
                            Object.defineProperty(ProductAllocation.prototype, "_budget_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["budget_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ProductAllocation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {google.ads.googleads.v11.services.IProductAllocation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ProductAllocation} ProductAllocation instance
                             */
                            ProductAllocation.create = function create(properties) {
                                return new ProductAllocation(properties);
                            };
    
                            /**
                             * Encodes the specified ProductAllocation message. Does not implicitly {@link google.ads.googleads.v11.services.ProductAllocation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {google.ads.googleads.v11.services.IProductAllocation} m ProductAllocation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductAllocation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.plannable_product_code != null && Object.hasOwnProperty.call(m, "plannable_product_code"))
                                    w.uint32(26).string(m.plannable_product_code);
                                if (m.budget_micros != null && Object.hasOwnProperty.call(m, "budget_micros"))
                                    w.uint32(32).int64(m.budget_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ProductAllocation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ProductAllocation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {google.ads.googleads.v11.services.IProductAllocation} message ProductAllocation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ProductAllocation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ProductAllocation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ProductAllocation} ProductAllocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductAllocation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ProductAllocation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.plannable_product_code = r.string();
                                        break;
                                    case 4:
                                        m.budget_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ProductAllocation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ProductAllocation} ProductAllocation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ProductAllocation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ProductAllocation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ProductAllocation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    p._plannable_product_code = 1;
                                    if (!$util.isString(m.plannable_product_code))
                                        return "plannable_product_code: string expected";
                                }
                                if (m.budget_micros != null && m.hasOwnProperty("budget_micros")) {
                                    p._budget_micros = 1;
                                    if (!$util.isInteger(m.budget_micros) && !(m.budget_micros && $util.isInteger(m.budget_micros.low) && $util.isInteger(m.budget_micros.high)))
                                        return "budget_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ProductAllocation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ProductAllocation} ProductAllocation
                             */
                            ProductAllocation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ProductAllocation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ProductAllocation();
                                if (d.plannable_product_code != null) {
                                    m.plannable_product_code = String(d.plannable_product_code);
                                }
                                if (d.budget_micros != null) {
                                    if ($util.Long)
                                        (m.budget_micros = $util.Long.fromValue(d.budget_micros)).unsigned = false;
                                    else if (typeof d.budget_micros === "string")
                                        m.budget_micros = parseInt(d.budget_micros, 10);
                                    else if (typeof d.budget_micros === "number")
                                        m.budget_micros = d.budget_micros;
                                    else if (typeof d.budget_micros === "object")
                                        m.budget_micros = new $util.LongBits(d.budget_micros.low >>> 0, d.budget_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ProductAllocation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @static
                             * @param {google.ads.googleads.v11.services.ProductAllocation} m ProductAllocation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ProductAllocation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    d.plannable_product_code = m.plannable_product_code;
                                    if (o.oneofs)
                                        d._plannable_product_code = "plannable_product_code";
                                }
                                if (m.budget_micros != null && m.hasOwnProperty("budget_micros")) {
                                    if (typeof m.budget_micros === "number")
                                        d.budget_micros = o.longs === String ? String(m.budget_micros) : m.budget_micros;
                                    else
                                        d.budget_micros = o.longs === String ? $util.Long.prototype.toString.call(m.budget_micros) : o.longs === Number ? new $util.LongBits(m.budget_micros.low >>> 0, m.budget_micros.high >>> 0).toNumber() : m.budget_micros;
                                    if (o.oneofs)
                                        d._budget_micros = "budget_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ProductAllocation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ProductAllocation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ProductAllocation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ProductAllocation;
                        })();
    
                        services.GenerateReachForecastRequest = (function() {
    
                            /**
                             * Properties of a GenerateReachForecastRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateReachForecastRequest
                             * @property {string|null} [customer_id] GenerateReachForecastRequest customer_id
                             * @property {string|null} [currency_code] GenerateReachForecastRequest currency_code
                             * @property {google.ads.googleads.v11.services.ICampaignDuration|null} [campaign_duration] GenerateReachForecastRequest campaign_duration
                             * @property {number|null} [cookie_frequency_cap] GenerateReachForecastRequest cookie_frequency_cap
                             * @property {google.ads.googleads.v11.services.IFrequencyCap|null} [cookie_frequency_cap_setting] GenerateReachForecastRequest cookie_frequency_cap_setting
                             * @property {number|null} [min_effective_frequency] GenerateReachForecastRequest min_effective_frequency
                             * @property {google.ads.googleads.v11.services.IEffectiveFrequencyLimit|null} [effective_frequency_limit] GenerateReachForecastRequest effective_frequency_limit
                             * @property {google.ads.googleads.v11.services.ITargeting|null} [targeting] GenerateReachForecastRequest targeting
                             * @property {Array.<google.ads.googleads.v11.services.IPlannedProduct>|null} [planned_products] GenerateReachForecastRequest planned_products
                             * @property {google.ads.googleads.v11.services.IForecastMetricOptions|null} [forecast_metric_options] GenerateReachForecastRequest forecast_metric_options
                             */
    
                            /**
                             * Constructs a new GenerateReachForecastRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateReachForecastRequest.
                             * @implements IGenerateReachForecastRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastRequest=} [p] Properties to set
                             */
                            function GenerateReachForecastRequest(p) {
                                this.planned_products = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateReachForecastRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.customer_id = "";
    
                            /**
                             * GenerateReachForecastRequest currency_code.
                             * @member {string|null|undefined} currency_code
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.currency_code = null;
    
                            /**
                             * GenerateReachForecastRequest campaign_duration.
                             * @member {google.ads.googleads.v11.services.ICampaignDuration|null|undefined} campaign_duration
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.campaign_duration = null;
    
                            /**
                             * GenerateReachForecastRequest cookie_frequency_cap.
                             * @member {number|null|undefined} cookie_frequency_cap
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.cookie_frequency_cap = null;
    
                            /**
                             * GenerateReachForecastRequest cookie_frequency_cap_setting.
                             * @member {google.ads.googleads.v11.services.IFrequencyCap|null|undefined} cookie_frequency_cap_setting
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.cookie_frequency_cap_setting = null;
    
                            /**
                             * GenerateReachForecastRequest min_effective_frequency.
                             * @member {number|null|undefined} min_effective_frequency
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.min_effective_frequency = null;
    
                            /**
                             * GenerateReachForecastRequest effective_frequency_limit.
                             * @member {google.ads.googleads.v11.services.IEffectiveFrequencyLimit|null|undefined} effective_frequency_limit
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.effective_frequency_limit = null;
    
                            /**
                             * GenerateReachForecastRequest targeting.
                             * @member {google.ads.googleads.v11.services.ITargeting|null|undefined} targeting
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.targeting = null;
    
                            /**
                             * GenerateReachForecastRequest planned_products.
                             * @member {Array.<google.ads.googleads.v11.services.IPlannedProduct>} planned_products
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.planned_products = $util.emptyArray;
    
                            /**
                             * GenerateReachForecastRequest forecast_metric_options.
                             * @member {google.ads.googleads.v11.services.IForecastMetricOptions|null|undefined} forecast_metric_options
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            GenerateReachForecastRequest.prototype.forecast_metric_options = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * GenerateReachForecastRequest _currency_code.
                             * @member {"currency_code"|undefined} _currency_code
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            Object.defineProperty(GenerateReachForecastRequest.prototype, "_currency_code", {
                                get: $util.oneOfGetter($oneOfFields = ["currency_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GenerateReachForecastRequest _cookie_frequency_cap.
                             * @member {"cookie_frequency_cap"|undefined} _cookie_frequency_cap
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            Object.defineProperty(GenerateReachForecastRequest.prototype, "_cookie_frequency_cap", {
                                get: $util.oneOfGetter($oneOfFields = ["cookie_frequency_cap"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GenerateReachForecastRequest _min_effective_frequency.
                             * @member {"min_effective_frequency"|undefined} _min_effective_frequency
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            Object.defineProperty(GenerateReachForecastRequest.prototype, "_min_effective_frequency", {
                                get: $util.oneOfGetter($oneOfFields = ["min_effective_frequency"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * GenerateReachForecastRequest _effective_frequency_limit.
                             * @member {"effective_frequency_limit"|undefined} _effective_frequency_limit
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             */
                            Object.defineProperty(GenerateReachForecastRequest.prototype, "_effective_frequency_limit", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_frequency_limit"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new GenerateReachForecastRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastRequest} GenerateReachForecastRequest instance
                             */
                            GenerateReachForecastRequest.create = function create(properties) {
                                return new GenerateReachForecastRequest(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateReachForecastRequest message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateReachForecastRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastRequest} m GenerateReachForecastRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateReachForecastRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.campaign_duration != null && Object.hasOwnProperty.call(m, "campaign_duration"))
                                    $root.google.ads.googleads.v11.services.CampaignDuration.encode(m.campaign_duration, w.uint32(26).fork()).ldelim();
                                if (m.targeting != null && Object.hasOwnProperty.call(m, "targeting"))
                                    $root.google.ads.googleads.v11.services.Targeting.encode(m.targeting, w.uint32(50).fork()).ldelim();
                                if (m.planned_products != null && m.planned_products.length) {
                                    for (var i = 0; i < m.planned_products.length; ++i)
                                        $root.google.ads.googleads.v11.services.PlannedProduct.encode(m.planned_products[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.cookie_frequency_cap_setting != null && Object.hasOwnProperty.call(m, "cookie_frequency_cap_setting"))
                                    $root.google.ads.googleads.v11.services.FrequencyCap.encode(m.cookie_frequency_cap_setting, w.uint32(66).fork()).ldelim();
                                if (m.currency_code != null && Object.hasOwnProperty.call(m, "currency_code"))
                                    w.uint32(74).string(m.currency_code);
                                if (m.cookie_frequency_cap != null && Object.hasOwnProperty.call(m, "cookie_frequency_cap"))
                                    w.uint32(80).int32(m.cookie_frequency_cap);
                                if (m.min_effective_frequency != null && Object.hasOwnProperty.call(m, "min_effective_frequency"))
                                    w.uint32(88).int32(m.min_effective_frequency);
                                if (m.effective_frequency_limit != null && Object.hasOwnProperty.call(m, "effective_frequency_limit"))
                                    $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit.encode(m.effective_frequency_limit, w.uint32(98).fork()).ldelim();
                                if (m.forecast_metric_options != null && Object.hasOwnProperty.call(m, "forecast_metric_options"))
                                    $root.google.ads.googleads.v11.services.ForecastMetricOptions.encode(m.forecast_metric_options, w.uint32(106).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateReachForecastRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateReachForecastRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastRequest} message GenerateReachForecastRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateReachForecastRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateReachForecastRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastRequest} GenerateReachForecastRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateReachForecastRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateReachForecastRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 9:
                                        m.currency_code = r.string();
                                        break;
                                    case 3:
                                        m.campaign_duration = $root.google.ads.googleads.v11.services.CampaignDuration.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.cookie_frequency_cap = r.int32();
                                        break;
                                    case 8:
                                        m.cookie_frequency_cap_setting = $root.google.ads.googleads.v11.services.FrequencyCap.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.min_effective_frequency = r.int32();
                                        break;
                                    case 12:
                                        m.effective_frequency_limit = $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.targeting = $root.google.ads.googleads.v11.services.Targeting.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        if (!(m.planned_products && m.planned_products.length))
                                            m.planned_products = [];
                                        m.planned_products.push($root.google.ads.googleads.v11.services.PlannedProduct.decode(r, r.uint32()));
                                        break;
                                    case 13:
                                        m.forecast_metric_options = $root.google.ads.googleads.v11.services.ForecastMetricOptions.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateReachForecastRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastRequest} GenerateReachForecastRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateReachForecastRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateReachForecastRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateReachForecastRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    p._currency_code = 1;
                                    if (!$util.isString(m.currency_code))
                                        return "currency_code: string expected";
                                }
                                if (m.campaign_duration != null && m.hasOwnProperty("campaign_duration")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.CampaignDuration.verify(m.campaign_duration);
                                        if (e)
                                            return "campaign_duration." + e;
                                    }
                                }
                                if (m.cookie_frequency_cap != null && m.hasOwnProperty("cookie_frequency_cap")) {
                                    p._cookie_frequency_cap = 1;
                                    if (!$util.isInteger(m.cookie_frequency_cap))
                                        return "cookie_frequency_cap: integer expected";
                                }
                                if (m.cookie_frequency_cap_setting != null && m.hasOwnProperty("cookie_frequency_cap_setting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.FrequencyCap.verify(m.cookie_frequency_cap_setting);
                                        if (e)
                                            return "cookie_frequency_cap_setting." + e;
                                    }
                                }
                                if (m.min_effective_frequency != null && m.hasOwnProperty("min_effective_frequency")) {
                                    p._min_effective_frequency = 1;
                                    if (!$util.isInteger(m.min_effective_frequency))
                                        return "min_effective_frequency: integer expected";
                                }
                                if (m.effective_frequency_limit != null && m.hasOwnProperty("effective_frequency_limit")) {
                                    p._effective_frequency_limit = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit.verify(m.effective_frequency_limit);
                                        if (e)
                                            return "effective_frequency_limit." + e;
                                    }
                                }
                                if (m.targeting != null && m.hasOwnProperty("targeting")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.Targeting.verify(m.targeting);
                                        if (e)
                                            return "targeting." + e;
                                    }
                                }
                                if (m.planned_products != null && m.hasOwnProperty("planned_products")) {
                                    if (!Array.isArray(m.planned_products))
                                        return "planned_products: array expected";
                                    for (var i = 0; i < m.planned_products.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.PlannedProduct.verify(m.planned_products[i]);
                                            if (e)
                                                return "planned_products." + e;
                                        }
                                    }
                                }
                                if (m.forecast_metric_options != null && m.hasOwnProperty("forecast_metric_options")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ForecastMetricOptions.verify(m.forecast_metric_options);
                                        if (e)
                                            return "forecast_metric_options." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateReachForecastRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastRequest} GenerateReachForecastRequest
                             */
                            GenerateReachForecastRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateReachForecastRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateReachForecastRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.currency_code != null) {
                                    m.currency_code = String(d.currency_code);
                                }
                                if (d.campaign_duration != null) {
                                    if (typeof d.campaign_duration !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastRequest.campaign_duration: object expected");
                                    m.campaign_duration = $root.google.ads.googleads.v11.services.CampaignDuration.fromObject(d.campaign_duration);
                                }
                                if (d.cookie_frequency_cap != null) {
                                    m.cookie_frequency_cap = d.cookie_frequency_cap | 0;
                                }
                                if (d.cookie_frequency_cap_setting != null) {
                                    if (typeof d.cookie_frequency_cap_setting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastRequest.cookie_frequency_cap_setting: object expected");
                                    m.cookie_frequency_cap_setting = $root.google.ads.googleads.v11.services.FrequencyCap.fromObject(d.cookie_frequency_cap_setting);
                                }
                                if (d.min_effective_frequency != null) {
                                    m.min_effective_frequency = d.min_effective_frequency | 0;
                                }
                                if (d.effective_frequency_limit != null) {
                                    if (typeof d.effective_frequency_limit !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastRequest.effective_frequency_limit: object expected");
                                    m.effective_frequency_limit = $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit.fromObject(d.effective_frequency_limit);
                                }
                                if (d.targeting != null) {
                                    if (typeof d.targeting !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastRequest.targeting: object expected");
                                    m.targeting = $root.google.ads.googleads.v11.services.Targeting.fromObject(d.targeting);
                                }
                                if (d.planned_products) {
                                    if (!Array.isArray(d.planned_products))
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastRequest.planned_products: array expected");
                                    m.planned_products = [];
                                    for (var i = 0; i < d.planned_products.length; ++i) {
                                        if (typeof d.planned_products[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastRequest.planned_products: object expected");
                                        m.planned_products[i] = $root.google.ads.googleads.v11.services.PlannedProduct.fromObject(d.planned_products[i]);
                                    }
                                }
                                if (d.forecast_metric_options != null) {
                                    if (typeof d.forecast_metric_options !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastRequest.forecast_metric_options: object expected");
                                    m.forecast_metric_options = $root.google.ads.googleads.v11.services.ForecastMetricOptions.fromObject(d.forecast_metric_options);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateReachForecastRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateReachForecastRequest} m GenerateReachForecastRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateReachForecastRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.planned_products = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.campaign_duration = null;
                                    d.targeting = null;
                                    d.cookie_frequency_cap_setting = null;
                                    d.forecast_metric_options = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.campaign_duration != null && m.hasOwnProperty("campaign_duration")) {
                                    d.campaign_duration = $root.google.ads.googleads.v11.services.CampaignDuration.toObject(m.campaign_duration, o);
                                }
                                if (m.targeting != null && m.hasOwnProperty("targeting")) {
                                    d.targeting = $root.google.ads.googleads.v11.services.Targeting.toObject(m.targeting, o);
                                }
                                if (m.planned_products && m.planned_products.length) {
                                    d.planned_products = [];
                                    for (var j = 0; j < m.planned_products.length; ++j) {
                                        d.planned_products[j] = $root.google.ads.googleads.v11.services.PlannedProduct.toObject(m.planned_products[j], o);
                                    }
                                }
                                if (m.cookie_frequency_cap_setting != null && m.hasOwnProperty("cookie_frequency_cap_setting")) {
                                    d.cookie_frequency_cap_setting = $root.google.ads.googleads.v11.services.FrequencyCap.toObject(m.cookie_frequency_cap_setting, o);
                                }
                                if (m.currency_code != null && m.hasOwnProperty("currency_code")) {
                                    d.currency_code = m.currency_code;
                                    if (o.oneofs)
                                        d._currency_code = "currency_code";
                                }
                                if (m.cookie_frequency_cap != null && m.hasOwnProperty("cookie_frequency_cap")) {
                                    d.cookie_frequency_cap = m.cookie_frequency_cap;
                                    if (o.oneofs)
                                        d._cookie_frequency_cap = "cookie_frequency_cap";
                                }
                                if (m.min_effective_frequency != null && m.hasOwnProperty("min_effective_frequency")) {
                                    d.min_effective_frequency = m.min_effective_frequency;
                                    if (o.oneofs)
                                        d._min_effective_frequency = "min_effective_frequency";
                                }
                                if (m.effective_frequency_limit != null && m.hasOwnProperty("effective_frequency_limit")) {
                                    d.effective_frequency_limit = $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit.toObject(m.effective_frequency_limit, o);
                                    if (o.oneofs)
                                        d._effective_frequency_limit = "effective_frequency_limit";
                                }
                                if (m.forecast_metric_options != null && m.hasOwnProperty("forecast_metric_options")) {
                                    d.forecast_metric_options = $root.google.ads.googleads.v11.services.ForecastMetricOptions.toObject(m.forecast_metric_options, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateReachForecastRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateReachForecastRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateReachForecastRequest;
                        })();
    
                        services.EffectiveFrequencyLimit = (function() {
    
                            /**
                             * Properties of an EffectiveFrequencyLimit.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IEffectiveFrequencyLimit
                             * @property {number|null} [effective_frequency_breakdown_limit] EffectiveFrequencyLimit effective_frequency_breakdown_limit
                             */
    
                            /**
                             * Constructs a new EffectiveFrequencyLimit.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an EffectiveFrequencyLimit.
                             * @implements IEffectiveFrequencyLimit
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyLimit=} [p] Properties to set
                             */
                            function EffectiveFrequencyLimit(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * EffectiveFrequencyLimit effective_frequency_breakdown_limit.
                             * @member {number} effective_frequency_breakdown_limit
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @instance
                             */
                            EffectiveFrequencyLimit.prototype.effective_frequency_breakdown_limit = 0;
    
                            /**
                             * Creates a new EffectiveFrequencyLimit instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyLimit=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyLimit} EffectiveFrequencyLimit instance
                             */
                            EffectiveFrequencyLimit.create = function create(properties) {
                                return new EffectiveFrequencyLimit(properties);
                            };
    
                            /**
                             * Encodes the specified EffectiveFrequencyLimit message. Does not implicitly {@link google.ads.googleads.v11.services.EffectiveFrequencyLimit.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyLimit} m EffectiveFrequencyLimit message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EffectiveFrequencyLimit.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.effective_frequency_breakdown_limit != null && Object.hasOwnProperty.call(m, "effective_frequency_breakdown_limit"))
                                    w.uint32(8).int32(m.effective_frequency_breakdown_limit);
                                return w;
                            };
    
                            /**
                             * Encodes the specified EffectiveFrequencyLimit message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.EffectiveFrequencyLimit.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyLimit} message EffectiveFrequencyLimit message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EffectiveFrequencyLimit.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EffectiveFrequencyLimit message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyLimit} EffectiveFrequencyLimit
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EffectiveFrequencyLimit.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.effective_frequency_breakdown_limit = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an EffectiveFrequencyLimit message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyLimit} EffectiveFrequencyLimit
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EffectiveFrequencyLimit.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EffectiveFrequencyLimit message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EffectiveFrequencyLimit.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.effective_frequency_breakdown_limit != null && m.hasOwnProperty("effective_frequency_breakdown_limit")) {
                                    if (!$util.isInteger(m.effective_frequency_breakdown_limit))
                                        return "effective_frequency_breakdown_limit: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an EffectiveFrequencyLimit message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyLimit} EffectiveFrequencyLimit
                             */
                            EffectiveFrequencyLimit.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.EffectiveFrequencyLimit();
                                if (d.effective_frequency_breakdown_limit != null) {
                                    m.effective_frequency_breakdown_limit = d.effective_frequency_breakdown_limit | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an EffectiveFrequencyLimit message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @static
                             * @param {google.ads.googleads.v11.services.EffectiveFrequencyLimit} m EffectiveFrequencyLimit
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EffectiveFrequencyLimit.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.effective_frequency_breakdown_limit = 0;
                                }
                                if (m.effective_frequency_breakdown_limit != null && m.hasOwnProperty("effective_frequency_breakdown_limit")) {
                                    d.effective_frequency_breakdown_limit = m.effective_frequency_breakdown_limit;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this EffectiveFrequencyLimit to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyLimit
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EffectiveFrequencyLimit.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return EffectiveFrequencyLimit;
                        })();
    
                        services.FrequencyCap = (function() {
    
                            /**
                             * Properties of a FrequencyCap.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IFrequencyCap
                             * @property {number|null} [impressions] FrequencyCap impressions
                             * @property {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit|null} [time_unit] FrequencyCap time_unit
                             */
    
                            /**
                             * Constructs a new FrequencyCap.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a FrequencyCap.
                             * @implements IFrequencyCap
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IFrequencyCap=} [p] Properties to set
                             */
                            function FrequencyCap(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * FrequencyCap impressions.
                             * @member {number} impressions
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @instance
                             */
                            FrequencyCap.prototype.impressions = 0;
    
                            /**
                             * FrequencyCap time_unit.
                             * @member {google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit} time_unit
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @instance
                             */
                            FrequencyCap.prototype.time_unit = 0;
    
                            /**
                             * Creates a new FrequencyCap instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {google.ads.googleads.v11.services.IFrequencyCap=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.FrequencyCap} FrequencyCap instance
                             */
                            FrequencyCap.create = function create(properties) {
                                return new FrequencyCap(properties);
                            };
    
                            /**
                             * Encodes the specified FrequencyCap message. Does not implicitly {@link google.ads.googleads.v11.services.FrequencyCap.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {google.ads.googleads.v11.services.IFrequencyCap} m FrequencyCap message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCap.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.time_unit != null && Object.hasOwnProperty.call(m, "time_unit"))
                                    w.uint32(16).int32(m.time_unit);
                                if (m.impressions != null && Object.hasOwnProperty.call(m, "impressions"))
                                    w.uint32(24).int32(m.impressions);
                                return w;
                            };
    
                            /**
                             * Encodes the specified FrequencyCap message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.FrequencyCap.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {google.ads.googleads.v11.services.IFrequencyCap} message FrequencyCap message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FrequencyCap.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FrequencyCap message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.FrequencyCap} FrequencyCap
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCap.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.FrequencyCap();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.impressions = r.int32();
                                        break;
                                    case 2:
                                        m.time_unit = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a FrequencyCap message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.FrequencyCap} FrequencyCap
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FrequencyCap.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FrequencyCap message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FrequencyCap.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    if (!$util.isInteger(m.impressions))
                                        return "impressions: integer expected";
                                }
                                if (m.time_unit != null && m.hasOwnProperty("time_unit")) {
                                    switch (m.time_unit) {
                                    default:
                                        return "time_unit: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FrequencyCap message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.FrequencyCap} FrequencyCap
                             */
                            FrequencyCap.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.FrequencyCap)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.FrequencyCap();
                                if (d.impressions != null) {
                                    m.impressions = d.impressions | 0;
                                }
                                switch (d.time_unit) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.time_unit = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.time_unit = 1;
                                    break;
                                case "DAY":
                                case 2:
                                    m.time_unit = 2;
                                    break;
                                case "WEEK":
                                case 3:
                                    m.time_unit = 3;
                                    break;
                                case "MONTH":
                                case 4:
                                    m.time_unit = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a FrequencyCap message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @static
                             * @param {google.ads.googleads.v11.services.FrequencyCap} m FrequencyCap
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FrequencyCap.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.time_unit = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.impressions = 0;
                                }
                                if (m.time_unit != null && m.hasOwnProperty("time_unit")) {
                                    d.time_unit = o.enums === String ? $root.google.ads.googleads.v11.enums.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit[m.time_unit] : m.time_unit;
                                }
                                if (m.impressions != null && m.hasOwnProperty("impressions")) {
                                    d.impressions = m.impressions;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this FrequencyCap to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.FrequencyCap
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FrequencyCap.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return FrequencyCap;
                        })();
    
                        services.Targeting = (function() {
    
                            /**
                             * Properties of a Targeting.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ITargeting
                             * @property {string|null} [plannable_location_id] Targeting plannable_location_id
                             * @property {google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.ReachPlanAgeRange|null} [age_range] Targeting age_range
                             * @property {Array.<google.ads.googleads.v11.common.IGenderInfo>|null} [genders] Targeting genders
                             * @property {Array.<google.ads.googleads.v11.common.IDeviceInfo>|null} [devices] Targeting devices
                             * @property {google.ads.googleads.v11.enums.ReachPlanNetworkEnum.ReachPlanNetwork|null} [network] Targeting network
                             */
    
                            /**
                             * Constructs a new Targeting.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a Targeting.
                             * @implements ITargeting
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ITargeting=} [p] Properties to set
                             */
                            function Targeting(p) {
                                this.genders = [];
                                this.devices = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Targeting plannable_location_id.
                             * @member {string|null|undefined} plannable_location_id
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @instance
                             */
                            Targeting.prototype.plannable_location_id = null;
    
                            /**
                             * Targeting age_range.
                             * @member {google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.ReachPlanAgeRange} age_range
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @instance
                             */
                            Targeting.prototype.age_range = 0;
    
                            /**
                             * Targeting genders.
                             * @member {Array.<google.ads.googleads.v11.common.IGenderInfo>} genders
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @instance
                             */
                            Targeting.prototype.genders = $util.emptyArray;
    
                            /**
                             * Targeting devices.
                             * @member {Array.<google.ads.googleads.v11.common.IDeviceInfo>} devices
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @instance
                             */
                            Targeting.prototype.devices = $util.emptyArray;
    
                            /**
                             * Targeting network.
                             * @member {google.ads.googleads.v11.enums.ReachPlanNetworkEnum.ReachPlanNetwork} network
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @instance
                             */
                            Targeting.prototype.network = 0;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Targeting _plannable_location_id.
                             * @member {"plannable_location_id"|undefined} _plannable_location_id
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @instance
                             */
                            Object.defineProperty(Targeting.prototype, "_plannable_location_id", {
                                get: $util.oneOfGetter($oneOfFields = ["plannable_location_id"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Targeting instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {google.ads.googleads.v11.services.ITargeting=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.Targeting} Targeting instance
                             */
                            Targeting.create = function create(properties) {
                                return new Targeting(properties);
                            };
    
                            /**
                             * Encodes the specified Targeting message. Does not implicitly {@link google.ads.googleads.v11.services.Targeting.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {google.ads.googleads.v11.services.ITargeting} m Targeting message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Targeting.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.age_range != null && Object.hasOwnProperty.call(m, "age_range"))
                                    w.uint32(16).int32(m.age_range);
                                if (m.genders != null && m.genders.length) {
                                    for (var i = 0; i < m.genders.length; ++i)
                                        $root.google.ads.googleads.v11.common.GenderInfo.encode(m.genders[i], w.uint32(26).fork()).ldelim();
                                }
                                if (m.devices != null && m.devices.length) {
                                    for (var i = 0; i < m.devices.length; ++i)
                                        $root.google.ads.googleads.v11.common.DeviceInfo.encode(m.devices[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.network != null && Object.hasOwnProperty.call(m, "network"))
                                    w.uint32(40).int32(m.network);
                                if (m.plannable_location_id != null && Object.hasOwnProperty.call(m, "plannable_location_id"))
                                    w.uint32(50).string(m.plannable_location_id);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Targeting message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.Targeting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {google.ads.googleads.v11.services.ITargeting} message Targeting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Targeting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Targeting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.Targeting} Targeting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Targeting.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.Targeting();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 6:
                                        m.plannable_location_id = r.string();
                                        break;
                                    case 2:
                                        m.age_range = r.int32();
                                        break;
                                    case 3:
                                        if (!(m.genders && m.genders.length))
                                            m.genders = [];
                                        m.genders.push($root.google.ads.googleads.v11.common.GenderInfo.decode(r, r.uint32()));
                                        break;
                                    case 4:
                                        if (!(m.devices && m.devices.length))
                                            m.devices = [];
                                        m.devices.push($root.google.ads.googleads.v11.common.DeviceInfo.decode(r, r.uint32()));
                                        break;
                                    case 5:
                                        m.network = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Targeting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.Targeting} Targeting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Targeting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Targeting message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Targeting.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.plannable_location_id != null && m.hasOwnProperty("plannable_location_id")) {
                                    p._plannable_location_id = 1;
                                    if (!$util.isString(m.plannable_location_id))
                                        return "plannable_location_id: string expected";
                                }
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    switch (m.age_range) {
                                    default:
                                        return "age_range: enum value expected";
                                    case 0:
                                    case 1:
                                    case 503001:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                    case 6:
                                    case 7:
                                    case 8:
                                    case 503002:
                                    case 9:
                                    case 10:
                                    case 11:
                                    case 12:
                                    case 13:
                                    case 503003:
                                    case 14:
                                    case 15:
                                    case 16:
                                    case 17:
                                    case 503004:
                                    case 18:
                                    case 19:
                                    case 20:
                                    case 503005:
                                    case 21:
                                    case 503006:
                                        break;
                                    }
                                }
                                if (m.genders != null && m.hasOwnProperty("genders")) {
                                    if (!Array.isArray(m.genders))
                                        return "genders: array expected";
                                    for (var i = 0; i < m.genders.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.GenderInfo.verify(m.genders[i]);
                                            if (e)
                                                return "genders." + e;
                                        }
                                    }
                                }
                                if (m.devices != null && m.hasOwnProperty("devices")) {
                                    if (!Array.isArray(m.devices))
                                        return "devices: array expected";
                                    for (var i = 0; i < m.devices.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.DeviceInfo.verify(m.devices[i]);
                                            if (e)
                                                return "devices." + e;
                                        }
                                    }
                                }
                                if (m.network != null && m.hasOwnProperty("network")) {
                                    switch (m.network) {
                                    default:
                                        return "network: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Targeting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.Targeting} Targeting
                             */
                            Targeting.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.Targeting)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.Targeting();
                                if (d.plannable_location_id != null) {
                                    m.plannable_location_id = String(d.plannable_location_id);
                                }
                                switch (d.age_range) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.age_range = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.age_range = 1;
                                    break;
                                case "AGE_RANGE_18_24":
                                case 503001:
                                    m.age_range = 503001;
                                    break;
                                case "AGE_RANGE_18_34":
                                case 2:
                                    m.age_range = 2;
                                    break;
                                case "AGE_RANGE_18_44":
                                case 3:
                                    m.age_range = 3;
                                    break;
                                case "AGE_RANGE_18_49":
                                case 4:
                                    m.age_range = 4;
                                    break;
                                case "AGE_RANGE_18_54":
                                case 5:
                                    m.age_range = 5;
                                    break;
                                case "AGE_RANGE_18_64":
                                case 6:
                                    m.age_range = 6;
                                    break;
                                case "AGE_RANGE_18_65_UP":
                                case 7:
                                    m.age_range = 7;
                                    break;
                                case "AGE_RANGE_21_34":
                                case 8:
                                    m.age_range = 8;
                                    break;
                                case "AGE_RANGE_25_34":
                                case 503002:
                                    m.age_range = 503002;
                                    break;
                                case "AGE_RANGE_25_44":
                                case 9:
                                    m.age_range = 9;
                                    break;
                                case "AGE_RANGE_25_49":
                                case 10:
                                    m.age_range = 10;
                                    break;
                                case "AGE_RANGE_25_54":
                                case 11:
                                    m.age_range = 11;
                                    break;
                                case "AGE_RANGE_25_64":
                                case 12:
                                    m.age_range = 12;
                                    break;
                                case "AGE_RANGE_25_65_UP":
                                case 13:
                                    m.age_range = 13;
                                    break;
                                case "AGE_RANGE_35_44":
                                case 503003:
                                    m.age_range = 503003;
                                    break;
                                case "AGE_RANGE_35_49":
                                case 14:
                                    m.age_range = 14;
                                    break;
                                case "AGE_RANGE_35_54":
                                case 15:
                                    m.age_range = 15;
                                    break;
                                case "AGE_RANGE_35_64":
                                case 16:
                                    m.age_range = 16;
                                    break;
                                case "AGE_RANGE_35_65_UP":
                                case 17:
                                    m.age_range = 17;
                                    break;
                                case "AGE_RANGE_45_54":
                                case 503004:
                                    m.age_range = 503004;
                                    break;
                                case "AGE_RANGE_45_64":
                                case 18:
                                    m.age_range = 18;
                                    break;
                                case "AGE_RANGE_45_65_UP":
                                case 19:
                                    m.age_range = 19;
                                    break;
                                case "AGE_RANGE_50_65_UP":
                                case 20:
                                    m.age_range = 20;
                                    break;
                                case "AGE_RANGE_55_64":
                                case 503005:
                                    m.age_range = 503005;
                                    break;
                                case "AGE_RANGE_55_65_UP":
                                case 21:
                                    m.age_range = 21;
                                    break;
                                case "AGE_RANGE_65_UP":
                                case 503006:
                                    m.age_range = 503006;
                                    break;
                                }
                                if (d.genders) {
                                    if (!Array.isArray(d.genders))
                                        throw TypeError(".google.ads.googleads.v11.services.Targeting.genders: array expected");
                                    m.genders = [];
                                    for (var i = 0; i < d.genders.length; ++i) {
                                        if (typeof d.genders[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.Targeting.genders: object expected");
                                        m.genders[i] = $root.google.ads.googleads.v11.common.GenderInfo.fromObject(d.genders[i]);
                                    }
                                }
                                if (d.devices) {
                                    if (!Array.isArray(d.devices))
                                        throw TypeError(".google.ads.googleads.v11.services.Targeting.devices: array expected");
                                    m.devices = [];
                                    for (var i = 0; i < d.devices.length; ++i) {
                                        if (typeof d.devices[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.Targeting.devices: object expected");
                                        m.devices[i] = $root.google.ads.googleads.v11.common.DeviceInfo.fromObject(d.devices[i]);
                                    }
                                }
                                switch (d.network) {
                                case "UNSPECIFIED":
                                case 0:
                                    m.network = 0;
                                    break;
                                case "UNKNOWN":
                                case 1:
                                    m.network = 1;
                                    break;
                                case "YOUTUBE":
                                case 2:
                                    m.network = 2;
                                    break;
                                case "GOOGLE_VIDEO_PARTNERS":
                                case 3:
                                    m.network = 3;
                                    break;
                                case "YOUTUBE_AND_GOOGLE_VIDEO_PARTNERS":
                                case 4:
                                    m.network = 4;
                                    break;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Targeting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @static
                             * @param {google.ads.googleads.v11.services.Targeting} m Targeting
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Targeting.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.genders = [];
                                    d.devices = [];
                                }
                                if (o.defaults) {
                                    d.age_range = o.enums === String ? "UNSPECIFIED" : 0;
                                    d.network = o.enums === String ? "UNSPECIFIED" : 0;
                                }
                                if (m.age_range != null && m.hasOwnProperty("age_range")) {
                                    d.age_range = o.enums === String ? $root.google.ads.googleads.v11.enums.ReachPlanAgeRangeEnum.ReachPlanAgeRange[m.age_range] : m.age_range;
                                }
                                if (m.genders && m.genders.length) {
                                    d.genders = [];
                                    for (var j = 0; j < m.genders.length; ++j) {
                                        d.genders[j] = $root.google.ads.googleads.v11.common.GenderInfo.toObject(m.genders[j], o);
                                    }
                                }
                                if (m.devices && m.devices.length) {
                                    d.devices = [];
                                    for (var j = 0; j < m.devices.length; ++j) {
                                        d.devices[j] = $root.google.ads.googleads.v11.common.DeviceInfo.toObject(m.devices[j], o);
                                    }
                                }
                                if (m.network != null && m.hasOwnProperty("network")) {
                                    d.network = o.enums === String ? $root.google.ads.googleads.v11.enums.ReachPlanNetworkEnum.ReachPlanNetwork[m.network] : m.network;
                                }
                                if (m.plannable_location_id != null && m.hasOwnProperty("plannable_location_id")) {
                                    d.plannable_location_id = m.plannable_location_id;
                                    if (o.oneofs)
                                        d._plannable_location_id = "plannable_location_id";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Targeting to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.Targeting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Targeting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Targeting;
                        })();
    
                        services.CampaignDuration = (function() {
    
                            /**
                             * Properties of a CampaignDuration.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ICampaignDuration
                             * @property {number|null} [duration_in_days] CampaignDuration duration_in_days
                             * @property {google.ads.googleads.v11.common.IDateRange|null} [date_range] CampaignDuration date_range
                             */
    
                            /**
                             * Constructs a new CampaignDuration.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a CampaignDuration.
                             * @implements ICampaignDuration
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ICampaignDuration=} [p] Properties to set
                             */
                            function CampaignDuration(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * CampaignDuration duration_in_days.
                             * @member {number|null|undefined} duration_in_days
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @instance
                             */
                            CampaignDuration.prototype.duration_in_days = null;
    
                            /**
                             * CampaignDuration date_range.
                             * @member {google.ads.googleads.v11.common.IDateRange|null|undefined} date_range
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @instance
                             */
                            CampaignDuration.prototype.date_range = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * CampaignDuration _duration_in_days.
                             * @member {"duration_in_days"|undefined} _duration_in_days
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @instance
                             */
                            Object.defineProperty(CampaignDuration.prototype, "_duration_in_days", {
                                get: $util.oneOfGetter($oneOfFields = ["duration_in_days"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new CampaignDuration instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignDuration=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.CampaignDuration} CampaignDuration instance
                             */
                            CampaignDuration.create = function create(properties) {
                                return new CampaignDuration(properties);
                            };
    
                            /**
                             * Encodes the specified CampaignDuration message. Does not implicitly {@link google.ads.googleads.v11.services.CampaignDuration.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignDuration} m CampaignDuration message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDuration.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.duration_in_days != null && Object.hasOwnProperty.call(m, "duration_in_days"))
                                    w.uint32(16).int32(m.duration_in_days);
                                if (m.date_range != null && Object.hasOwnProperty.call(m, "date_range"))
                                    $root.google.ads.googleads.v11.common.DateRange.encode(m.date_range, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified CampaignDuration message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.CampaignDuration.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {google.ads.googleads.v11.services.ICampaignDuration} message CampaignDuration message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            CampaignDuration.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a CampaignDuration message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.CampaignDuration} CampaignDuration
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDuration.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.CampaignDuration();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 2:
                                        m.duration_in_days = r.int32();
                                        break;
                                    case 3:
                                        m.date_range = $root.google.ads.googleads.v11.common.DateRange.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a CampaignDuration message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.CampaignDuration} CampaignDuration
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            CampaignDuration.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a CampaignDuration message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            CampaignDuration.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.duration_in_days != null && m.hasOwnProperty("duration_in_days")) {
                                    p._duration_in_days = 1;
                                    if (!$util.isInteger(m.duration_in_days))
                                        return "duration_in_days: integer expected";
                                }
                                if (m.date_range != null && m.hasOwnProperty("date_range")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.DateRange.verify(m.date_range);
                                        if (e)
                                            return "date_range." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a CampaignDuration message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.CampaignDuration} CampaignDuration
                             */
                            CampaignDuration.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.CampaignDuration)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.CampaignDuration();
                                if (d.duration_in_days != null) {
                                    m.duration_in_days = d.duration_in_days | 0;
                                }
                                if (d.date_range != null) {
                                    if (typeof d.date_range !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.CampaignDuration.date_range: object expected");
                                    m.date_range = $root.google.ads.googleads.v11.common.DateRange.fromObject(d.date_range);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a CampaignDuration message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @static
                             * @param {google.ads.googleads.v11.services.CampaignDuration} m CampaignDuration
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            CampaignDuration.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.date_range = null;
                                }
                                if (m.duration_in_days != null && m.hasOwnProperty("duration_in_days")) {
                                    d.duration_in_days = m.duration_in_days;
                                    if (o.oneofs)
                                        d._duration_in_days = "duration_in_days";
                                }
                                if (m.date_range != null && m.hasOwnProperty("date_range")) {
                                    d.date_range = $root.google.ads.googleads.v11.common.DateRange.toObject(m.date_range, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this CampaignDuration to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.CampaignDuration
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            CampaignDuration.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return CampaignDuration;
                        })();
    
                        services.PlannedProduct = (function() {
    
                            /**
                             * Properties of a PlannedProduct.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPlannedProduct
                             * @property {string|null} [plannable_product_code] PlannedProduct plannable_product_code
                             * @property {number|null} [budget_micros] PlannedProduct budget_micros
                             */
    
                            /**
                             * Constructs a new PlannedProduct.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PlannedProduct.
                             * @implements IPlannedProduct
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPlannedProduct=} [p] Properties to set
                             */
                            function PlannedProduct(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PlannedProduct plannable_product_code.
                             * @member {string|null|undefined} plannable_product_code
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @instance
                             */
                            PlannedProduct.prototype.plannable_product_code = null;
    
                            /**
                             * PlannedProduct budget_micros.
                             * @member {number|null|undefined} budget_micros
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @instance
                             */
                            PlannedProduct.prototype.budget_micros = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PlannedProduct _plannable_product_code.
                             * @member {"plannable_product_code"|undefined} _plannable_product_code
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @instance
                             */
                            Object.defineProperty(PlannedProduct.prototype, "_plannable_product_code", {
                                get: $util.oneOfGetter($oneOfFields = ["plannable_product_code"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannedProduct _budget_micros.
                             * @member {"budget_micros"|undefined} _budget_micros
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @instance
                             */
                            Object.defineProperty(PlannedProduct.prototype, "_budget_micros", {
                                get: $util.oneOfGetter($oneOfFields = ["budget_micros"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PlannedProduct instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProduct=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PlannedProduct} PlannedProduct instance
                             */
                            PlannedProduct.create = function create(properties) {
                                return new PlannedProduct(properties);
                            };
    
                            /**
                             * Encodes the specified PlannedProduct message. Does not implicitly {@link google.ads.googleads.v11.services.PlannedProduct.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProduct} m PlannedProduct message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannedProduct.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.plannable_product_code != null && Object.hasOwnProperty.call(m, "plannable_product_code"))
                                    w.uint32(26).string(m.plannable_product_code);
                                if (m.budget_micros != null && Object.hasOwnProperty.call(m, "budget_micros"))
                                    w.uint32(32).int64(m.budget_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlannedProduct message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PlannedProduct.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProduct} message PlannedProduct message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannedProduct.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlannedProduct message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PlannedProduct} PlannedProduct
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannedProduct.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PlannedProduct();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.plannable_product_code = r.string();
                                        break;
                                    case 4:
                                        m.budget_micros = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlannedProduct message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PlannedProduct} PlannedProduct
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannedProduct.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlannedProduct message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlannedProduct.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    p._plannable_product_code = 1;
                                    if (!$util.isString(m.plannable_product_code))
                                        return "plannable_product_code: string expected";
                                }
                                if (m.budget_micros != null && m.hasOwnProperty("budget_micros")) {
                                    p._budget_micros = 1;
                                    if (!$util.isInteger(m.budget_micros) && !(m.budget_micros && $util.isInteger(m.budget_micros.low) && $util.isInteger(m.budget_micros.high)))
                                        return "budget_micros: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PlannedProduct message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PlannedProduct} PlannedProduct
                             */
                            PlannedProduct.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PlannedProduct)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PlannedProduct();
                                if (d.plannable_product_code != null) {
                                    m.plannable_product_code = String(d.plannable_product_code);
                                }
                                if (d.budget_micros != null) {
                                    if ($util.Long)
                                        (m.budget_micros = $util.Long.fromValue(d.budget_micros)).unsigned = false;
                                    else if (typeof d.budget_micros === "string")
                                        m.budget_micros = parseInt(d.budget_micros, 10);
                                    else if (typeof d.budget_micros === "number")
                                        m.budget_micros = d.budget_micros;
                                    else if (typeof d.budget_micros === "object")
                                        m.budget_micros = new $util.LongBits(d.budget_micros.low >>> 0, d.budget_micros.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PlannedProduct message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @static
                             * @param {google.ads.googleads.v11.services.PlannedProduct} m PlannedProduct
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlannedProduct.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    d.plannable_product_code = m.plannable_product_code;
                                    if (o.oneofs)
                                        d._plannable_product_code = "plannable_product_code";
                                }
                                if (m.budget_micros != null && m.hasOwnProperty("budget_micros")) {
                                    if (typeof m.budget_micros === "number")
                                        d.budget_micros = o.longs === String ? String(m.budget_micros) : m.budget_micros;
                                    else
                                        d.budget_micros = o.longs === String ? $util.Long.prototype.toString.call(m.budget_micros) : o.longs === Number ? new $util.LongBits(m.budget_micros.low >>> 0, m.budget_micros.high >>> 0).toNumber() : m.budget_micros;
                                    if (o.oneofs)
                                        d._budget_micros = "budget_micros";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PlannedProduct to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PlannedProduct
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlannedProduct.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PlannedProduct;
                        })();
    
                        services.GenerateReachForecastResponse = (function() {
    
                            /**
                             * Properties of a GenerateReachForecastResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IGenerateReachForecastResponse
                             * @property {google.ads.googleads.v11.services.IOnTargetAudienceMetrics|null} [on_target_audience_metrics] GenerateReachForecastResponse on_target_audience_metrics
                             * @property {google.ads.googleads.v11.services.IReachCurve|null} [reach_curve] GenerateReachForecastResponse reach_curve
                             */
    
                            /**
                             * Constructs a new GenerateReachForecastResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a GenerateReachForecastResponse.
                             * @implements IGenerateReachForecastResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastResponse=} [p] Properties to set
                             */
                            function GenerateReachForecastResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * GenerateReachForecastResponse on_target_audience_metrics.
                             * @member {google.ads.googleads.v11.services.IOnTargetAudienceMetrics|null|undefined} on_target_audience_metrics
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @instance
                             */
                            GenerateReachForecastResponse.prototype.on_target_audience_metrics = null;
    
                            /**
                             * GenerateReachForecastResponse reach_curve.
                             * @member {google.ads.googleads.v11.services.IReachCurve|null|undefined} reach_curve
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @instance
                             */
                            GenerateReachForecastResponse.prototype.reach_curve = null;
    
                            /**
                             * Creates a new GenerateReachForecastResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastResponse} GenerateReachForecastResponse instance
                             */
                            GenerateReachForecastResponse.create = function create(properties) {
                                return new GenerateReachForecastResponse(properties);
                            };
    
                            /**
                             * Encodes the specified GenerateReachForecastResponse message. Does not implicitly {@link google.ads.googleads.v11.services.GenerateReachForecastResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastResponse} m GenerateReachForecastResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateReachForecastResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.on_target_audience_metrics != null && Object.hasOwnProperty.call(m, "on_target_audience_metrics"))
                                    $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics.encode(m.on_target_audience_metrics, w.uint32(10).fork()).ldelim();
                                if (m.reach_curve != null && Object.hasOwnProperty.call(m, "reach_curve"))
                                    $root.google.ads.googleads.v11.services.ReachCurve.encode(m.reach_curve, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified GenerateReachForecastResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.GenerateReachForecastResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IGenerateReachForecastResponse} message GenerateReachForecastResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            GenerateReachForecastResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a GenerateReachForecastResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastResponse} GenerateReachForecastResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateReachForecastResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.GenerateReachForecastResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.on_target_audience_metrics = $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.reach_curve = $root.google.ads.googleads.v11.services.ReachCurve.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a GenerateReachForecastResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastResponse} GenerateReachForecastResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            GenerateReachForecastResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a GenerateReachForecastResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            GenerateReachForecastResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.on_target_audience_metrics != null && m.hasOwnProperty("on_target_audience_metrics")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics.verify(m.on_target_audience_metrics);
                                        if (e)
                                            return "on_target_audience_metrics." + e;
                                    }
                                }
                                if (m.reach_curve != null && m.hasOwnProperty("reach_curve")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ReachCurve.verify(m.reach_curve);
                                        if (e)
                                            return "reach_curve." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a GenerateReachForecastResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.GenerateReachForecastResponse} GenerateReachForecastResponse
                             */
                            GenerateReachForecastResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.GenerateReachForecastResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.GenerateReachForecastResponse();
                                if (d.on_target_audience_metrics != null) {
                                    if (typeof d.on_target_audience_metrics !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastResponse.on_target_audience_metrics: object expected");
                                    m.on_target_audience_metrics = $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics.fromObject(d.on_target_audience_metrics);
                                }
                                if (d.reach_curve != null) {
                                    if (typeof d.reach_curve !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.GenerateReachForecastResponse.reach_curve: object expected");
                                    m.reach_curve = $root.google.ads.googleads.v11.services.ReachCurve.fromObject(d.reach_curve);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a GenerateReachForecastResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.GenerateReachForecastResponse} m GenerateReachForecastResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            GenerateReachForecastResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.on_target_audience_metrics = null;
                                    d.reach_curve = null;
                                }
                                if (m.on_target_audience_metrics != null && m.hasOwnProperty("on_target_audience_metrics")) {
                                    d.on_target_audience_metrics = $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics.toObject(m.on_target_audience_metrics, o);
                                }
                                if (m.reach_curve != null && m.hasOwnProperty("reach_curve")) {
                                    d.reach_curve = $root.google.ads.googleads.v11.services.ReachCurve.toObject(m.reach_curve, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this GenerateReachForecastResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.GenerateReachForecastResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            GenerateReachForecastResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return GenerateReachForecastResponse;
                        })();
    
                        services.ReachCurve = (function() {
    
                            /**
                             * Properties of a ReachCurve.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IReachCurve
                             * @property {Array.<google.ads.googleads.v11.services.IReachForecast>|null} [reach_forecasts] ReachCurve reach_forecasts
                             */
    
                            /**
                             * Constructs a new ReachCurve.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ReachCurve.
                             * @implements IReachCurve
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IReachCurve=} [p] Properties to set
                             */
                            function ReachCurve(p) {
                                this.reach_forecasts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ReachCurve reach_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IReachForecast>} reach_forecasts
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @instance
                             */
                            ReachCurve.prototype.reach_forecasts = $util.emptyArray;
    
                            /**
                             * Creates a new ReachCurve instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IReachCurve=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ReachCurve} ReachCurve instance
                             */
                            ReachCurve.create = function create(properties) {
                                return new ReachCurve(properties);
                            };
    
                            /**
                             * Encodes the specified ReachCurve message. Does not implicitly {@link google.ads.googleads.v11.services.ReachCurve.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IReachCurve} m ReachCurve message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachCurve.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.reach_forecasts != null && m.reach_forecasts.length) {
                                    for (var i = 0; i < m.reach_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.ReachForecast.encode(m.reach_forecasts[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified ReachCurve message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ReachCurve.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.IReachCurve} message ReachCurve message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachCurve.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ReachCurve message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ReachCurve} ReachCurve
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachCurve.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ReachCurve();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.reach_forecasts && m.reach_forecasts.length))
                                            m.reach_forecasts = [];
                                        m.reach_forecasts.push($root.google.ads.googleads.v11.services.ReachForecast.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ReachCurve message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ReachCurve} ReachCurve
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachCurve.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ReachCurve message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ReachCurve.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.reach_forecasts != null && m.hasOwnProperty("reach_forecasts")) {
                                    if (!Array.isArray(m.reach_forecasts))
                                        return "reach_forecasts: array expected";
                                    for (var i = 0; i < m.reach_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ReachForecast.verify(m.reach_forecasts[i]);
                                            if (e)
                                                return "reach_forecasts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ReachCurve message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ReachCurve} ReachCurve
                             */
                            ReachCurve.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ReachCurve)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ReachCurve();
                                if (d.reach_forecasts) {
                                    if (!Array.isArray(d.reach_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.ReachCurve.reach_forecasts: array expected");
                                    m.reach_forecasts = [];
                                    for (var i = 0; i < d.reach_forecasts.length; ++i) {
                                        if (typeof d.reach_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ReachCurve.reach_forecasts: object expected");
                                        m.reach_forecasts[i] = $root.google.ads.googleads.v11.services.ReachForecast.fromObject(d.reach_forecasts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ReachCurve message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @static
                             * @param {google.ads.googleads.v11.services.ReachCurve} m ReachCurve
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ReachCurve.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.reach_forecasts = [];
                                }
                                if (m.reach_forecasts && m.reach_forecasts.length) {
                                    d.reach_forecasts = [];
                                    for (var j = 0; j < m.reach_forecasts.length; ++j) {
                                        d.reach_forecasts[j] = $root.google.ads.googleads.v11.services.ReachForecast.toObject(m.reach_forecasts[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ReachCurve to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ReachCurve
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ReachCurve.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ReachCurve;
                        })();
    
                        services.ReachForecast = (function() {
    
                            /**
                             * Properties of a ReachForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IReachForecast
                             * @property {number|null} [cost_micros] ReachForecast cost_micros
                             * @property {google.ads.googleads.v11.services.IForecast|null} [forecast] ReachForecast forecast
                             * @property {Array.<google.ads.googleads.v11.services.IPlannedProductReachForecast>|null} [planned_product_reach_forecasts] ReachForecast planned_product_reach_forecasts
                             */
    
                            /**
                             * Constructs a new ReachForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ReachForecast.
                             * @implements IReachForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IReachForecast=} [p] Properties to set
                             */
                            function ReachForecast(p) {
                                this.planned_product_reach_forecasts = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ReachForecast cost_micros.
                             * @member {number} cost_micros
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @instance
                             */
                            ReachForecast.prototype.cost_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * ReachForecast forecast.
                             * @member {google.ads.googleads.v11.services.IForecast|null|undefined} forecast
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @instance
                             */
                            ReachForecast.prototype.forecast = null;
    
                            /**
                             * ReachForecast planned_product_reach_forecasts.
                             * @member {Array.<google.ads.googleads.v11.services.IPlannedProductReachForecast>} planned_product_reach_forecasts
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @instance
                             */
                            ReachForecast.prototype.planned_product_reach_forecasts = $util.emptyArray;
    
                            /**
                             * Creates a new ReachForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IReachForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ReachForecast} ReachForecast instance
                             */
                            ReachForecast.create = function create(properties) {
                                return new ReachForecast(properties);
                            };
    
                            /**
                             * Encodes the specified ReachForecast message. Does not implicitly {@link google.ads.googleads.v11.services.ReachForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IReachForecast} m ReachForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.forecast != null && Object.hasOwnProperty.call(m, "forecast"))
                                    $root.google.ads.googleads.v11.services.Forecast.encode(m.forecast, w.uint32(18).fork()).ldelim();
                                if (m.planned_product_reach_forecasts != null && m.planned_product_reach_forecasts.length) {
                                    for (var i = 0; i < m.planned_product_reach_forecasts.length; ++i)
                                        $root.google.ads.googleads.v11.services.PlannedProductReachForecast.encode(m.planned_product_reach_forecasts[i], w.uint32(34).fork()).ldelim();
                                }
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(40).int64(m.cost_micros);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ReachForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ReachForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IReachForecast} message ReachForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ReachForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ReachForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ReachForecast} ReachForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ReachForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 2:
                                        m.forecast = $root.google.ads.googleads.v11.services.Forecast.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        if (!(m.planned_product_reach_forecasts && m.planned_product_reach_forecasts.length))
                                            m.planned_product_reach_forecasts = [];
                                        m.planned_product_reach_forecasts.push($root.google.ads.googleads.v11.services.PlannedProductReachForecast.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ReachForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ReachForecast} ReachForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ReachForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ReachForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ReachForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.forecast != null && m.hasOwnProperty("forecast")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.Forecast.verify(m.forecast);
                                        if (e)
                                            return "forecast." + e;
                                    }
                                }
                                if (m.planned_product_reach_forecasts != null && m.hasOwnProperty("planned_product_reach_forecasts")) {
                                    if (!Array.isArray(m.planned_product_reach_forecasts))
                                        return "planned_product_reach_forecasts: array expected";
                                    for (var i = 0; i < m.planned_product_reach_forecasts.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.PlannedProductReachForecast.verify(m.planned_product_reach_forecasts[i]);
                                            if (e)
                                                return "planned_product_reach_forecasts." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ReachForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ReachForecast} ReachForecast
                             */
                            ReachForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ReachForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ReachForecast();
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.forecast != null) {
                                    if (typeof d.forecast !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ReachForecast.forecast: object expected");
                                    m.forecast = $root.google.ads.googleads.v11.services.Forecast.fromObject(d.forecast);
                                }
                                if (d.planned_product_reach_forecasts) {
                                    if (!Array.isArray(d.planned_product_reach_forecasts))
                                        throw TypeError(".google.ads.googleads.v11.services.ReachForecast.planned_product_reach_forecasts: array expected");
                                    m.planned_product_reach_forecasts = [];
                                    for (var i = 0; i < d.planned_product_reach_forecasts.length; ++i) {
                                        if (typeof d.planned_product_reach_forecasts[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ReachForecast.planned_product_reach_forecasts: object expected");
                                        m.planned_product_reach_forecasts[i] = $root.google.ads.googleads.v11.services.PlannedProductReachForecast.fromObject(d.planned_product_reach_forecasts[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ReachForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.ReachForecast} m ReachForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ReachForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.planned_product_reach_forecasts = [];
                                }
                                if (o.defaults) {
                                    d.forecast = null;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cost_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cost_micros = o.longs === String ? "0" : 0;
                                }
                                if (m.forecast != null && m.hasOwnProperty("forecast")) {
                                    d.forecast = $root.google.ads.googleads.v11.services.Forecast.toObject(m.forecast, o);
                                }
                                if (m.planned_product_reach_forecasts && m.planned_product_reach_forecasts.length) {
                                    d.planned_product_reach_forecasts = [];
                                    for (var j = 0; j < m.planned_product_reach_forecasts.length; ++j) {
                                        d.planned_product_reach_forecasts[j] = $root.google.ads.googleads.v11.services.PlannedProductReachForecast.toObject(m.planned_product_reach_forecasts[j], o);
                                    }
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ReachForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ReachForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ReachForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ReachForecast;
                        })();
    
                        services.Forecast = (function() {
    
                            /**
                             * Properties of a Forecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IForecast
                             * @property {number|null} [on_target_reach] Forecast on_target_reach
                             * @property {number|null} [total_reach] Forecast total_reach
                             * @property {number|null} [on_target_impressions] Forecast on_target_impressions
                             * @property {number|null} [total_impressions] Forecast total_impressions
                             * @property {number|null} [viewable_impressions] Forecast viewable_impressions
                             * @property {Array.<google.ads.googleads.v11.services.IEffectiveFrequencyBreakdown>|null} [effective_frequency_breakdowns] Forecast effective_frequency_breakdowns
                             * @property {number|null} [on_target_coview_reach] Forecast on_target_coview_reach
                             * @property {number|null} [total_coview_reach] Forecast total_coview_reach
                             * @property {number|null} [on_target_coview_impressions] Forecast on_target_coview_impressions
                             * @property {number|null} [total_coview_impressions] Forecast total_coview_impressions
                             */
    
                            /**
                             * Constructs a new Forecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a Forecast.
                             * @implements IForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IForecast=} [p] Properties to set
                             */
                            function Forecast(p) {
                                this.effective_frequency_breakdowns = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Forecast on_target_reach.
                             * @member {number|null|undefined} on_target_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.on_target_reach = null;
    
                            /**
                             * Forecast total_reach.
                             * @member {number|null|undefined} total_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.total_reach = null;
    
                            /**
                             * Forecast on_target_impressions.
                             * @member {number|null|undefined} on_target_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.on_target_impressions = null;
    
                            /**
                             * Forecast total_impressions.
                             * @member {number|null|undefined} total_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.total_impressions = null;
    
                            /**
                             * Forecast viewable_impressions.
                             * @member {number|null|undefined} viewable_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.viewable_impressions = null;
    
                            /**
                             * Forecast effective_frequency_breakdowns.
                             * @member {Array.<google.ads.googleads.v11.services.IEffectiveFrequencyBreakdown>} effective_frequency_breakdowns
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.effective_frequency_breakdowns = $util.emptyArray;
    
                            /**
                             * Forecast on_target_coview_reach.
                             * @member {number|null|undefined} on_target_coview_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.on_target_coview_reach = null;
    
                            /**
                             * Forecast total_coview_reach.
                             * @member {number|null|undefined} total_coview_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.total_coview_reach = null;
    
                            /**
                             * Forecast on_target_coview_impressions.
                             * @member {number|null|undefined} on_target_coview_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.on_target_coview_impressions = null;
    
                            /**
                             * Forecast total_coview_impressions.
                             * @member {number|null|undefined} total_coview_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Forecast.prototype.total_coview_impressions = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Forecast _on_target_reach.
                             * @member {"on_target_reach"|undefined} _on_target_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_on_target_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["on_target_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _total_reach.
                             * @member {"total_reach"|undefined} _total_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_total_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["total_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _on_target_impressions.
                             * @member {"on_target_impressions"|undefined} _on_target_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_on_target_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["on_target_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _total_impressions.
                             * @member {"total_impressions"|undefined} _total_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_total_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["total_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _viewable_impressions.
                             * @member {"viewable_impressions"|undefined} _viewable_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_viewable_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["viewable_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _on_target_coview_reach.
                             * @member {"on_target_coview_reach"|undefined} _on_target_coview_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_on_target_coview_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["on_target_coview_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _total_coview_reach.
                             * @member {"total_coview_reach"|undefined} _total_coview_reach
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_total_coview_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["total_coview_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _on_target_coview_impressions.
                             * @member {"on_target_coview_impressions"|undefined} _on_target_coview_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_on_target_coview_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["on_target_coview_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Forecast _total_coview_impressions.
                             * @member {"total_coview_impressions"|undefined} _total_coview_impressions
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             */
                            Object.defineProperty(Forecast.prototype, "_total_coview_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["total_coview_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Forecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.Forecast} Forecast instance
                             */
                            Forecast.create = function create(properties) {
                                return new Forecast(properties);
                            };
    
                            /**
                             * Encodes the specified Forecast message. Does not implicitly {@link google.ads.googleads.v11.services.Forecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecast} m Forecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Forecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.on_target_reach != null && Object.hasOwnProperty.call(m, "on_target_reach"))
                                    w.uint32(40).int64(m.on_target_reach);
                                if (m.total_reach != null && Object.hasOwnProperty.call(m, "total_reach"))
                                    w.uint32(48).int64(m.total_reach);
                                if (m.on_target_impressions != null && Object.hasOwnProperty.call(m, "on_target_impressions"))
                                    w.uint32(56).int64(m.on_target_impressions);
                                if (m.total_impressions != null && Object.hasOwnProperty.call(m, "total_impressions"))
                                    w.uint32(64).int64(m.total_impressions);
                                if (m.viewable_impressions != null && Object.hasOwnProperty.call(m, "viewable_impressions"))
                                    w.uint32(72).int64(m.viewable_impressions);
                                if (m.effective_frequency_breakdowns != null && m.effective_frequency_breakdowns.length) {
                                    for (var i = 0; i < m.effective_frequency_breakdowns.length; ++i)
                                        $root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown.encode(m.effective_frequency_breakdowns[i], w.uint32(82).fork()).ldelim();
                                }
                                if (m.on_target_coview_reach != null && Object.hasOwnProperty.call(m, "on_target_coview_reach"))
                                    w.uint32(88).int64(m.on_target_coview_reach);
                                if (m.total_coview_reach != null && Object.hasOwnProperty.call(m, "total_coview_reach"))
                                    w.uint32(96).int64(m.total_coview_reach);
                                if (m.on_target_coview_impressions != null && Object.hasOwnProperty.call(m, "on_target_coview_impressions"))
                                    w.uint32(104).int64(m.on_target_coview_impressions);
                                if (m.total_coview_impressions != null && Object.hasOwnProperty.call(m, "total_coview_impressions"))
                                    w.uint32(112).int64(m.total_coview_impressions);
                                return w;
                            };
    
                            /**
                             * Encodes the specified Forecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.Forecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecast} message Forecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Forecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Forecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.Forecast} Forecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Forecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.Forecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 5:
                                        m.on_target_reach = r.int64();
                                        break;
                                    case 6:
                                        m.total_reach = r.int64();
                                        break;
                                    case 7:
                                        m.on_target_impressions = r.int64();
                                        break;
                                    case 8:
                                        m.total_impressions = r.int64();
                                        break;
                                    case 9:
                                        m.viewable_impressions = r.int64();
                                        break;
                                    case 10:
                                        if (!(m.effective_frequency_breakdowns && m.effective_frequency_breakdowns.length))
                                            m.effective_frequency_breakdowns = [];
                                        m.effective_frequency_breakdowns.push($root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown.decode(r, r.uint32()));
                                        break;
                                    case 11:
                                        m.on_target_coview_reach = r.int64();
                                        break;
                                    case 12:
                                        m.total_coview_reach = r.int64();
                                        break;
                                    case 13:
                                        m.on_target_coview_impressions = r.int64();
                                        break;
                                    case 14:
                                        m.total_coview_impressions = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a Forecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.Forecast} Forecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Forecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Forecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Forecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.on_target_reach != null && m.hasOwnProperty("on_target_reach")) {
                                    p._on_target_reach = 1;
                                    if (!$util.isInteger(m.on_target_reach) && !(m.on_target_reach && $util.isInteger(m.on_target_reach.low) && $util.isInteger(m.on_target_reach.high)))
                                        return "on_target_reach: integer|Long expected";
                                }
                                if (m.total_reach != null && m.hasOwnProperty("total_reach")) {
                                    p._total_reach = 1;
                                    if (!$util.isInteger(m.total_reach) && !(m.total_reach && $util.isInteger(m.total_reach.low) && $util.isInteger(m.total_reach.high)))
                                        return "total_reach: integer|Long expected";
                                }
                                if (m.on_target_impressions != null && m.hasOwnProperty("on_target_impressions")) {
                                    p._on_target_impressions = 1;
                                    if (!$util.isInteger(m.on_target_impressions) && !(m.on_target_impressions && $util.isInteger(m.on_target_impressions.low) && $util.isInteger(m.on_target_impressions.high)))
                                        return "on_target_impressions: integer|Long expected";
                                }
                                if (m.total_impressions != null && m.hasOwnProperty("total_impressions")) {
                                    p._total_impressions = 1;
                                    if (!$util.isInteger(m.total_impressions) && !(m.total_impressions && $util.isInteger(m.total_impressions.low) && $util.isInteger(m.total_impressions.high)))
                                        return "total_impressions: integer|Long expected";
                                }
                                if (m.viewable_impressions != null && m.hasOwnProperty("viewable_impressions")) {
                                    p._viewable_impressions = 1;
                                    if (!$util.isInteger(m.viewable_impressions) && !(m.viewable_impressions && $util.isInteger(m.viewable_impressions.low) && $util.isInteger(m.viewable_impressions.high)))
                                        return "viewable_impressions: integer|Long expected";
                                }
                                if (m.effective_frequency_breakdowns != null && m.hasOwnProperty("effective_frequency_breakdowns")) {
                                    if (!Array.isArray(m.effective_frequency_breakdowns))
                                        return "effective_frequency_breakdowns: array expected";
                                    for (var i = 0; i < m.effective_frequency_breakdowns.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown.verify(m.effective_frequency_breakdowns[i]);
                                            if (e)
                                                return "effective_frequency_breakdowns." + e;
                                        }
                                    }
                                }
                                if (m.on_target_coview_reach != null && m.hasOwnProperty("on_target_coview_reach")) {
                                    p._on_target_coview_reach = 1;
                                    if (!$util.isInteger(m.on_target_coview_reach) && !(m.on_target_coview_reach && $util.isInteger(m.on_target_coview_reach.low) && $util.isInteger(m.on_target_coview_reach.high)))
                                        return "on_target_coview_reach: integer|Long expected";
                                }
                                if (m.total_coview_reach != null && m.hasOwnProperty("total_coview_reach")) {
                                    p._total_coview_reach = 1;
                                    if (!$util.isInteger(m.total_coview_reach) && !(m.total_coview_reach && $util.isInteger(m.total_coview_reach.low) && $util.isInteger(m.total_coview_reach.high)))
                                        return "total_coview_reach: integer|Long expected";
                                }
                                if (m.on_target_coview_impressions != null && m.hasOwnProperty("on_target_coview_impressions")) {
                                    p._on_target_coview_impressions = 1;
                                    if (!$util.isInteger(m.on_target_coview_impressions) && !(m.on_target_coview_impressions && $util.isInteger(m.on_target_coview_impressions.low) && $util.isInteger(m.on_target_coview_impressions.high)))
                                        return "on_target_coview_impressions: integer|Long expected";
                                }
                                if (m.total_coview_impressions != null && m.hasOwnProperty("total_coview_impressions")) {
                                    p._total_coview_impressions = 1;
                                    if (!$util.isInteger(m.total_coview_impressions) && !(m.total_coview_impressions && $util.isInteger(m.total_coview_impressions.low) && $util.isInteger(m.total_coview_impressions.high)))
                                        return "total_coview_impressions: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Forecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.Forecast} Forecast
                             */
                            Forecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.Forecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.Forecast();
                                if (d.on_target_reach != null) {
                                    if ($util.Long)
                                        (m.on_target_reach = $util.Long.fromValue(d.on_target_reach)).unsigned = false;
                                    else if (typeof d.on_target_reach === "string")
                                        m.on_target_reach = parseInt(d.on_target_reach, 10);
                                    else if (typeof d.on_target_reach === "number")
                                        m.on_target_reach = d.on_target_reach;
                                    else if (typeof d.on_target_reach === "object")
                                        m.on_target_reach = new $util.LongBits(d.on_target_reach.low >>> 0, d.on_target_reach.high >>> 0).toNumber();
                                }
                                if (d.total_reach != null) {
                                    if ($util.Long)
                                        (m.total_reach = $util.Long.fromValue(d.total_reach)).unsigned = false;
                                    else if (typeof d.total_reach === "string")
                                        m.total_reach = parseInt(d.total_reach, 10);
                                    else if (typeof d.total_reach === "number")
                                        m.total_reach = d.total_reach;
                                    else if (typeof d.total_reach === "object")
                                        m.total_reach = new $util.LongBits(d.total_reach.low >>> 0, d.total_reach.high >>> 0).toNumber();
                                }
                                if (d.on_target_impressions != null) {
                                    if ($util.Long)
                                        (m.on_target_impressions = $util.Long.fromValue(d.on_target_impressions)).unsigned = false;
                                    else if (typeof d.on_target_impressions === "string")
                                        m.on_target_impressions = parseInt(d.on_target_impressions, 10);
                                    else if (typeof d.on_target_impressions === "number")
                                        m.on_target_impressions = d.on_target_impressions;
                                    else if (typeof d.on_target_impressions === "object")
                                        m.on_target_impressions = new $util.LongBits(d.on_target_impressions.low >>> 0, d.on_target_impressions.high >>> 0).toNumber();
                                }
                                if (d.total_impressions != null) {
                                    if ($util.Long)
                                        (m.total_impressions = $util.Long.fromValue(d.total_impressions)).unsigned = false;
                                    else if (typeof d.total_impressions === "string")
                                        m.total_impressions = parseInt(d.total_impressions, 10);
                                    else if (typeof d.total_impressions === "number")
                                        m.total_impressions = d.total_impressions;
                                    else if (typeof d.total_impressions === "object")
                                        m.total_impressions = new $util.LongBits(d.total_impressions.low >>> 0, d.total_impressions.high >>> 0).toNumber();
                                }
                                if (d.viewable_impressions != null) {
                                    if ($util.Long)
                                        (m.viewable_impressions = $util.Long.fromValue(d.viewable_impressions)).unsigned = false;
                                    else if (typeof d.viewable_impressions === "string")
                                        m.viewable_impressions = parseInt(d.viewable_impressions, 10);
                                    else if (typeof d.viewable_impressions === "number")
                                        m.viewable_impressions = d.viewable_impressions;
                                    else if (typeof d.viewable_impressions === "object")
                                        m.viewable_impressions = new $util.LongBits(d.viewable_impressions.low >>> 0, d.viewable_impressions.high >>> 0).toNumber();
                                }
                                if (d.effective_frequency_breakdowns) {
                                    if (!Array.isArray(d.effective_frequency_breakdowns))
                                        throw TypeError(".google.ads.googleads.v11.services.Forecast.effective_frequency_breakdowns: array expected");
                                    m.effective_frequency_breakdowns = [];
                                    for (var i = 0; i < d.effective_frequency_breakdowns.length; ++i) {
                                        if (typeof d.effective_frequency_breakdowns[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.Forecast.effective_frequency_breakdowns: object expected");
                                        m.effective_frequency_breakdowns[i] = $root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown.fromObject(d.effective_frequency_breakdowns[i]);
                                    }
                                }
                                if (d.on_target_coview_reach != null) {
                                    if ($util.Long)
                                        (m.on_target_coview_reach = $util.Long.fromValue(d.on_target_coview_reach)).unsigned = false;
                                    else if (typeof d.on_target_coview_reach === "string")
                                        m.on_target_coview_reach = parseInt(d.on_target_coview_reach, 10);
                                    else if (typeof d.on_target_coview_reach === "number")
                                        m.on_target_coview_reach = d.on_target_coview_reach;
                                    else if (typeof d.on_target_coview_reach === "object")
                                        m.on_target_coview_reach = new $util.LongBits(d.on_target_coview_reach.low >>> 0, d.on_target_coview_reach.high >>> 0).toNumber();
                                }
                                if (d.total_coview_reach != null) {
                                    if ($util.Long)
                                        (m.total_coview_reach = $util.Long.fromValue(d.total_coview_reach)).unsigned = false;
                                    else if (typeof d.total_coview_reach === "string")
                                        m.total_coview_reach = parseInt(d.total_coview_reach, 10);
                                    else if (typeof d.total_coview_reach === "number")
                                        m.total_coview_reach = d.total_coview_reach;
                                    else if (typeof d.total_coview_reach === "object")
                                        m.total_coview_reach = new $util.LongBits(d.total_coview_reach.low >>> 0, d.total_coview_reach.high >>> 0).toNumber();
                                }
                                if (d.on_target_coview_impressions != null) {
                                    if ($util.Long)
                                        (m.on_target_coview_impressions = $util.Long.fromValue(d.on_target_coview_impressions)).unsigned = false;
                                    else if (typeof d.on_target_coview_impressions === "string")
                                        m.on_target_coview_impressions = parseInt(d.on_target_coview_impressions, 10);
                                    else if (typeof d.on_target_coview_impressions === "number")
                                        m.on_target_coview_impressions = d.on_target_coview_impressions;
                                    else if (typeof d.on_target_coview_impressions === "object")
                                        m.on_target_coview_impressions = new $util.LongBits(d.on_target_coview_impressions.low >>> 0, d.on_target_coview_impressions.high >>> 0).toNumber();
                                }
                                if (d.total_coview_impressions != null) {
                                    if ($util.Long)
                                        (m.total_coview_impressions = $util.Long.fromValue(d.total_coview_impressions)).unsigned = false;
                                    else if (typeof d.total_coview_impressions === "string")
                                        m.total_coview_impressions = parseInt(d.total_coview_impressions, 10);
                                    else if (typeof d.total_coview_impressions === "number")
                                        m.total_coview_impressions = d.total_coview_impressions;
                                    else if (typeof d.total_coview_impressions === "object")
                                        m.total_coview_impressions = new $util.LongBits(d.total_coview_impressions.low >>> 0, d.total_coview_impressions.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a Forecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @static
                             * @param {google.ads.googleads.v11.services.Forecast} m Forecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Forecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.effective_frequency_breakdowns = [];
                                }
                                if (m.on_target_reach != null && m.hasOwnProperty("on_target_reach")) {
                                    if (typeof m.on_target_reach === "number")
                                        d.on_target_reach = o.longs === String ? String(m.on_target_reach) : m.on_target_reach;
                                    else
                                        d.on_target_reach = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_reach) : o.longs === Number ? new $util.LongBits(m.on_target_reach.low >>> 0, m.on_target_reach.high >>> 0).toNumber() : m.on_target_reach;
                                    if (o.oneofs)
                                        d._on_target_reach = "on_target_reach";
                                }
                                if (m.total_reach != null && m.hasOwnProperty("total_reach")) {
                                    if (typeof m.total_reach === "number")
                                        d.total_reach = o.longs === String ? String(m.total_reach) : m.total_reach;
                                    else
                                        d.total_reach = o.longs === String ? $util.Long.prototype.toString.call(m.total_reach) : o.longs === Number ? new $util.LongBits(m.total_reach.low >>> 0, m.total_reach.high >>> 0).toNumber() : m.total_reach;
                                    if (o.oneofs)
                                        d._total_reach = "total_reach";
                                }
                                if (m.on_target_impressions != null && m.hasOwnProperty("on_target_impressions")) {
                                    if (typeof m.on_target_impressions === "number")
                                        d.on_target_impressions = o.longs === String ? String(m.on_target_impressions) : m.on_target_impressions;
                                    else
                                        d.on_target_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_impressions) : o.longs === Number ? new $util.LongBits(m.on_target_impressions.low >>> 0, m.on_target_impressions.high >>> 0).toNumber() : m.on_target_impressions;
                                    if (o.oneofs)
                                        d._on_target_impressions = "on_target_impressions";
                                }
                                if (m.total_impressions != null && m.hasOwnProperty("total_impressions")) {
                                    if (typeof m.total_impressions === "number")
                                        d.total_impressions = o.longs === String ? String(m.total_impressions) : m.total_impressions;
                                    else
                                        d.total_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.total_impressions) : o.longs === Number ? new $util.LongBits(m.total_impressions.low >>> 0, m.total_impressions.high >>> 0).toNumber() : m.total_impressions;
                                    if (o.oneofs)
                                        d._total_impressions = "total_impressions";
                                }
                                if (m.viewable_impressions != null && m.hasOwnProperty("viewable_impressions")) {
                                    if (typeof m.viewable_impressions === "number")
                                        d.viewable_impressions = o.longs === String ? String(m.viewable_impressions) : m.viewable_impressions;
                                    else
                                        d.viewable_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.viewable_impressions) : o.longs === Number ? new $util.LongBits(m.viewable_impressions.low >>> 0, m.viewable_impressions.high >>> 0).toNumber() : m.viewable_impressions;
                                    if (o.oneofs)
                                        d._viewable_impressions = "viewable_impressions";
                                }
                                if (m.effective_frequency_breakdowns && m.effective_frequency_breakdowns.length) {
                                    d.effective_frequency_breakdowns = [];
                                    for (var j = 0; j < m.effective_frequency_breakdowns.length; ++j) {
                                        d.effective_frequency_breakdowns[j] = $root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown.toObject(m.effective_frequency_breakdowns[j], o);
                                    }
                                }
                                if (m.on_target_coview_reach != null && m.hasOwnProperty("on_target_coview_reach")) {
                                    if (typeof m.on_target_coview_reach === "number")
                                        d.on_target_coview_reach = o.longs === String ? String(m.on_target_coview_reach) : m.on_target_coview_reach;
                                    else
                                        d.on_target_coview_reach = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_coview_reach) : o.longs === Number ? new $util.LongBits(m.on_target_coview_reach.low >>> 0, m.on_target_coview_reach.high >>> 0).toNumber() : m.on_target_coview_reach;
                                    if (o.oneofs)
                                        d._on_target_coview_reach = "on_target_coview_reach";
                                }
                                if (m.total_coview_reach != null && m.hasOwnProperty("total_coview_reach")) {
                                    if (typeof m.total_coview_reach === "number")
                                        d.total_coview_reach = o.longs === String ? String(m.total_coview_reach) : m.total_coview_reach;
                                    else
                                        d.total_coview_reach = o.longs === String ? $util.Long.prototype.toString.call(m.total_coview_reach) : o.longs === Number ? new $util.LongBits(m.total_coview_reach.low >>> 0, m.total_coview_reach.high >>> 0).toNumber() : m.total_coview_reach;
                                    if (o.oneofs)
                                        d._total_coview_reach = "total_coview_reach";
                                }
                                if (m.on_target_coview_impressions != null && m.hasOwnProperty("on_target_coview_impressions")) {
                                    if (typeof m.on_target_coview_impressions === "number")
                                        d.on_target_coview_impressions = o.longs === String ? String(m.on_target_coview_impressions) : m.on_target_coview_impressions;
                                    else
                                        d.on_target_coview_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_coview_impressions) : o.longs === Number ? new $util.LongBits(m.on_target_coview_impressions.low >>> 0, m.on_target_coview_impressions.high >>> 0).toNumber() : m.on_target_coview_impressions;
                                    if (o.oneofs)
                                        d._on_target_coview_impressions = "on_target_coview_impressions";
                                }
                                if (m.total_coview_impressions != null && m.hasOwnProperty("total_coview_impressions")) {
                                    if (typeof m.total_coview_impressions === "number")
                                        d.total_coview_impressions = o.longs === String ? String(m.total_coview_impressions) : m.total_coview_impressions;
                                    else
                                        d.total_coview_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.total_coview_impressions) : o.longs === Number ? new $util.LongBits(m.total_coview_impressions.low >>> 0, m.total_coview_impressions.high >>> 0).toNumber() : m.total_coview_impressions;
                                    if (o.oneofs)
                                        d._total_coview_impressions = "total_coview_impressions";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this Forecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.Forecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Forecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return Forecast;
                        })();
    
                        services.PlannedProductReachForecast = (function() {
    
                            /**
                             * Properties of a PlannedProductReachForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPlannedProductReachForecast
                             * @property {string|null} [plannable_product_code] PlannedProductReachForecast plannable_product_code
                             * @property {number|null} [cost_micros] PlannedProductReachForecast cost_micros
                             * @property {google.ads.googleads.v11.services.IPlannedProductForecast|null} [planned_product_forecast] PlannedProductReachForecast planned_product_forecast
                             */
    
                            /**
                             * Constructs a new PlannedProductReachForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PlannedProductReachForecast.
                             * @implements IPlannedProductReachForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPlannedProductReachForecast=} [p] Properties to set
                             */
                            function PlannedProductReachForecast(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PlannedProductReachForecast plannable_product_code.
                             * @member {string} plannable_product_code
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @instance
                             */
                            PlannedProductReachForecast.prototype.plannable_product_code = "";
    
                            /**
                             * PlannedProductReachForecast cost_micros.
                             * @member {number} cost_micros
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @instance
                             */
                            PlannedProductReachForecast.prototype.cost_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * PlannedProductReachForecast planned_product_forecast.
                             * @member {google.ads.googleads.v11.services.IPlannedProductForecast|null|undefined} planned_product_forecast
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @instance
                             */
                            PlannedProductReachForecast.prototype.planned_product_forecast = null;
    
                            /**
                             * Creates a new PlannedProductReachForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProductReachForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PlannedProductReachForecast} PlannedProductReachForecast instance
                             */
                            PlannedProductReachForecast.create = function create(properties) {
                                return new PlannedProductReachForecast(properties);
                            };
    
                            /**
                             * Encodes the specified PlannedProductReachForecast message. Does not implicitly {@link google.ads.googleads.v11.services.PlannedProductReachForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProductReachForecast} m PlannedProductReachForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannedProductReachForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.plannable_product_code != null && Object.hasOwnProperty.call(m, "plannable_product_code"))
                                    w.uint32(10).string(m.plannable_product_code);
                                if (m.cost_micros != null && Object.hasOwnProperty.call(m, "cost_micros"))
                                    w.uint32(16).int64(m.cost_micros);
                                if (m.planned_product_forecast != null && Object.hasOwnProperty.call(m, "planned_product_forecast"))
                                    $root.google.ads.googleads.v11.services.PlannedProductForecast.encode(m.planned_product_forecast, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlannedProductReachForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PlannedProductReachForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProductReachForecast} message PlannedProductReachForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannedProductReachForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlannedProductReachForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PlannedProductReachForecast} PlannedProductReachForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannedProductReachForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PlannedProductReachForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.plannable_product_code = r.string();
                                        break;
                                    case 2:
                                        m.cost_micros = r.int64();
                                        break;
                                    case 3:
                                        m.planned_product_forecast = $root.google.ads.googleads.v11.services.PlannedProductForecast.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlannedProductReachForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PlannedProductReachForecast} PlannedProductReachForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannedProductReachForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlannedProductReachForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlannedProductReachForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    if (!$util.isString(m.plannable_product_code))
                                        return "plannable_product_code: string expected";
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (!$util.isInteger(m.cost_micros) && !(m.cost_micros && $util.isInteger(m.cost_micros.low) && $util.isInteger(m.cost_micros.high)))
                                        return "cost_micros: integer|Long expected";
                                }
                                if (m.planned_product_forecast != null && m.hasOwnProperty("planned_product_forecast")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.PlannedProductForecast.verify(m.planned_product_forecast);
                                        if (e)
                                            return "planned_product_forecast." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PlannedProductReachForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PlannedProductReachForecast} PlannedProductReachForecast
                             */
                            PlannedProductReachForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PlannedProductReachForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PlannedProductReachForecast();
                                if (d.plannable_product_code != null) {
                                    m.plannable_product_code = String(d.plannable_product_code);
                                }
                                if (d.cost_micros != null) {
                                    if ($util.Long)
                                        (m.cost_micros = $util.Long.fromValue(d.cost_micros)).unsigned = false;
                                    else if (typeof d.cost_micros === "string")
                                        m.cost_micros = parseInt(d.cost_micros, 10);
                                    else if (typeof d.cost_micros === "number")
                                        m.cost_micros = d.cost_micros;
                                    else if (typeof d.cost_micros === "object")
                                        m.cost_micros = new $util.LongBits(d.cost_micros.low >>> 0, d.cost_micros.high >>> 0).toNumber();
                                }
                                if (d.planned_product_forecast != null) {
                                    if (typeof d.planned_product_forecast !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.PlannedProductReachForecast.planned_product_forecast: object expected");
                                    m.planned_product_forecast = $root.google.ads.googleads.v11.services.PlannedProductForecast.fromObject(d.planned_product_forecast);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PlannedProductReachForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.PlannedProductReachForecast} m PlannedProductReachForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlannedProductReachForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.plannable_product_code = "";
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.cost_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.cost_micros = o.longs === String ? "0" : 0;
                                    d.planned_product_forecast = null;
                                }
                                if (m.plannable_product_code != null && m.hasOwnProperty("plannable_product_code")) {
                                    d.plannable_product_code = m.plannable_product_code;
                                }
                                if (m.cost_micros != null && m.hasOwnProperty("cost_micros")) {
                                    if (typeof m.cost_micros === "number")
                                        d.cost_micros = o.longs === String ? String(m.cost_micros) : m.cost_micros;
                                    else
                                        d.cost_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cost_micros) : o.longs === Number ? new $util.LongBits(m.cost_micros.low >>> 0, m.cost_micros.high >>> 0).toNumber() : m.cost_micros;
                                }
                                if (m.planned_product_forecast != null && m.hasOwnProperty("planned_product_forecast")) {
                                    d.planned_product_forecast = $root.google.ads.googleads.v11.services.PlannedProductForecast.toObject(m.planned_product_forecast, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PlannedProductReachForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PlannedProductReachForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlannedProductReachForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PlannedProductReachForecast;
                        })();
    
                        services.PlannedProductForecast = (function() {
    
                            /**
                             * Properties of a PlannedProductForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IPlannedProductForecast
                             * @property {number|null} [on_target_reach] PlannedProductForecast on_target_reach
                             * @property {number|null} [total_reach] PlannedProductForecast total_reach
                             * @property {number|null} [on_target_impressions] PlannedProductForecast on_target_impressions
                             * @property {number|null} [total_impressions] PlannedProductForecast total_impressions
                             * @property {number|null} [viewable_impressions] PlannedProductForecast viewable_impressions
                             * @property {number|null} [on_target_coview_reach] PlannedProductForecast on_target_coview_reach
                             * @property {number|null} [total_coview_reach] PlannedProductForecast total_coview_reach
                             * @property {number|null} [on_target_coview_impressions] PlannedProductForecast on_target_coview_impressions
                             * @property {number|null} [total_coview_impressions] PlannedProductForecast total_coview_impressions
                             */
    
                            /**
                             * Constructs a new PlannedProductForecast.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a PlannedProductForecast.
                             * @implements IPlannedProductForecast
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IPlannedProductForecast=} [p] Properties to set
                             */
                            function PlannedProductForecast(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * PlannedProductForecast on_target_reach.
                             * @member {number} on_target_reach
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.on_target_reach = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * PlannedProductForecast total_reach.
                             * @member {number} total_reach
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.total_reach = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * PlannedProductForecast on_target_impressions.
                             * @member {number} on_target_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.on_target_impressions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * PlannedProductForecast total_impressions.
                             * @member {number} total_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.total_impressions = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * PlannedProductForecast viewable_impressions.
                             * @member {number|null|undefined} viewable_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.viewable_impressions = null;
    
                            /**
                             * PlannedProductForecast on_target_coview_reach.
                             * @member {number|null|undefined} on_target_coview_reach
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.on_target_coview_reach = null;
    
                            /**
                             * PlannedProductForecast total_coview_reach.
                             * @member {number|null|undefined} total_coview_reach
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.total_coview_reach = null;
    
                            /**
                             * PlannedProductForecast on_target_coview_impressions.
                             * @member {number|null|undefined} on_target_coview_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.on_target_coview_impressions = null;
    
                            /**
                             * PlannedProductForecast total_coview_impressions.
                             * @member {number|null|undefined} total_coview_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            PlannedProductForecast.prototype.total_coview_impressions = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * PlannedProductForecast _viewable_impressions.
                             * @member {"viewable_impressions"|undefined} _viewable_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            Object.defineProperty(PlannedProductForecast.prototype, "_viewable_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["viewable_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannedProductForecast _on_target_coview_reach.
                             * @member {"on_target_coview_reach"|undefined} _on_target_coview_reach
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            Object.defineProperty(PlannedProductForecast.prototype, "_on_target_coview_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["on_target_coview_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannedProductForecast _total_coview_reach.
                             * @member {"total_coview_reach"|undefined} _total_coview_reach
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            Object.defineProperty(PlannedProductForecast.prototype, "_total_coview_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["total_coview_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannedProductForecast _on_target_coview_impressions.
                             * @member {"on_target_coview_impressions"|undefined} _on_target_coview_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            Object.defineProperty(PlannedProductForecast.prototype, "_on_target_coview_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["on_target_coview_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * PlannedProductForecast _total_coview_impressions.
                             * @member {"total_coview_impressions"|undefined} _total_coview_impressions
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             */
                            Object.defineProperty(PlannedProductForecast.prototype, "_total_coview_impressions", {
                                get: $util.oneOfGetter($oneOfFields = ["total_coview_impressions"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new PlannedProductForecast instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProductForecast=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.PlannedProductForecast} PlannedProductForecast instance
                             */
                            PlannedProductForecast.create = function create(properties) {
                                return new PlannedProductForecast(properties);
                            };
    
                            /**
                             * Encodes the specified PlannedProductForecast message. Does not implicitly {@link google.ads.googleads.v11.services.PlannedProductForecast.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProductForecast} m PlannedProductForecast message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannedProductForecast.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.on_target_reach != null && Object.hasOwnProperty.call(m, "on_target_reach"))
                                    w.uint32(8).int64(m.on_target_reach);
                                if (m.total_reach != null && Object.hasOwnProperty.call(m, "total_reach"))
                                    w.uint32(16).int64(m.total_reach);
                                if (m.on_target_impressions != null && Object.hasOwnProperty.call(m, "on_target_impressions"))
                                    w.uint32(24).int64(m.on_target_impressions);
                                if (m.total_impressions != null && Object.hasOwnProperty.call(m, "total_impressions"))
                                    w.uint32(32).int64(m.total_impressions);
                                if (m.viewable_impressions != null && Object.hasOwnProperty.call(m, "viewable_impressions"))
                                    w.uint32(40).int64(m.viewable_impressions);
                                if (m.on_target_coview_reach != null && Object.hasOwnProperty.call(m, "on_target_coview_reach"))
                                    w.uint32(48).int64(m.on_target_coview_reach);
                                if (m.total_coview_reach != null && Object.hasOwnProperty.call(m, "total_coview_reach"))
                                    w.uint32(56).int64(m.total_coview_reach);
                                if (m.on_target_coview_impressions != null && Object.hasOwnProperty.call(m, "on_target_coview_impressions"))
                                    w.uint32(64).int64(m.on_target_coview_impressions);
                                if (m.total_coview_impressions != null && Object.hasOwnProperty.call(m, "total_coview_impressions"))
                                    w.uint32(72).int64(m.total_coview_impressions);
                                return w;
                            };
    
                            /**
                             * Encodes the specified PlannedProductForecast message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.PlannedProductForecast.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.IPlannedProductForecast} message PlannedProductForecast message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            PlannedProductForecast.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a PlannedProductForecast message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.PlannedProductForecast} PlannedProductForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannedProductForecast.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.PlannedProductForecast();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.on_target_reach = r.int64();
                                        break;
                                    case 2:
                                        m.total_reach = r.int64();
                                        break;
                                    case 3:
                                        m.on_target_impressions = r.int64();
                                        break;
                                    case 4:
                                        m.total_impressions = r.int64();
                                        break;
                                    case 5:
                                        m.viewable_impressions = r.int64();
                                        break;
                                    case 6:
                                        m.on_target_coview_reach = r.int64();
                                        break;
                                    case 7:
                                        m.total_coview_reach = r.int64();
                                        break;
                                    case 8:
                                        m.on_target_coview_impressions = r.int64();
                                        break;
                                    case 9:
                                        m.total_coview_impressions = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a PlannedProductForecast message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.PlannedProductForecast} PlannedProductForecast
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            PlannedProductForecast.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a PlannedProductForecast message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            PlannedProductForecast.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.on_target_reach != null && m.hasOwnProperty("on_target_reach")) {
                                    if (!$util.isInteger(m.on_target_reach) && !(m.on_target_reach && $util.isInteger(m.on_target_reach.low) && $util.isInteger(m.on_target_reach.high)))
                                        return "on_target_reach: integer|Long expected";
                                }
                                if (m.total_reach != null && m.hasOwnProperty("total_reach")) {
                                    if (!$util.isInteger(m.total_reach) && !(m.total_reach && $util.isInteger(m.total_reach.low) && $util.isInteger(m.total_reach.high)))
                                        return "total_reach: integer|Long expected";
                                }
                                if (m.on_target_impressions != null && m.hasOwnProperty("on_target_impressions")) {
                                    if (!$util.isInteger(m.on_target_impressions) && !(m.on_target_impressions && $util.isInteger(m.on_target_impressions.low) && $util.isInteger(m.on_target_impressions.high)))
                                        return "on_target_impressions: integer|Long expected";
                                }
                                if (m.total_impressions != null && m.hasOwnProperty("total_impressions")) {
                                    if (!$util.isInteger(m.total_impressions) && !(m.total_impressions && $util.isInteger(m.total_impressions.low) && $util.isInteger(m.total_impressions.high)))
                                        return "total_impressions: integer|Long expected";
                                }
                                if (m.viewable_impressions != null && m.hasOwnProperty("viewable_impressions")) {
                                    p._viewable_impressions = 1;
                                    if (!$util.isInteger(m.viewable_impressions) && !(m.viewable_impressions && $util.isInteger(m.viewable_impressions.low) && $util.isInteger(m.viewable_impressions.high)))
                                        return "viewable_impressions: integer|Long expected";
                                }
                                if (m.on_target_coview_reach != null && m.hasOwnProperty("on_target_coview_reach")) {
                                    p._on_target_coview_reach = 1;
                                    if (!$util.isInteger(m.on_target_coview_reach) && !(m.on_target_coview_reach && $util.isInteger(m.on_target_coview_reach.low) && $util.isInteger(m.on_target_coview_reach.high)))
                                        return "on_target_coview_reach: integer|Long expected";
                                }
                                if (m.total_coview_reach != null && m.hasOwnProperty("total_coview_reach")) {
                                    p._total_coview_reach = 1;
                                    if (!$util.isInteger(m.total_coview_reach) && !(m.total_coview_reach && $util.isInteger(m.total_coview_reach.low) && $util.isInteger(m.total_coview_reach.high)))
                                        return "total_coview_reach: integer|Long expected";
                                }
                                if (m.on_target_coview_impressions != null && m.hasOwnProperty("on_target_coview_impressions")) {
                                    p._on_target_coview_impressions = 1;
                                    if (!$util.isInteger(m.on_target_coview_impressions) && !(m.on_target_coview_impressions && $util.isInteger(m.on_target_coview_impressions.low) && $util.isInteger(m.on_target_coview_impressions.high)))
                                        return "on_target_coview_impressions: integer|Long expected";
                                }
                                if (m.total_coview_impressions != null && m.hasOwnProperty("total_coview_impressions")) {
                                    p._total_coview_impressions = 1;
                                    if (!$util.isInteger(m.total_coview_impressions) && !(m.total_coview_impressions && $util.isInteger(m.total_coview_impressions.low) && $util.isInteger(m.total_coview_impressions.high)))
                                        return "total_coview_impressions: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a PlannedProductForecast message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.PlannedProductForecast} PlannedProductForecast
                             */
                            PlannedProductForecast.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.PlannedProductForecast)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.PlannedProductForecast();
                                if (d.on_target_reach != null) {
                                    if ($util.Long)
                                        (m.on_target_reach = $util.Long.fromValue(d.on_target_reach)).unsigned = false;
                                    else if (typeof d.on_target_reach === "string")
                                        m.on_target_reach = parseInt(d.on_target_reach, 10);
                                    else if (typeof d.on_target_reach === "number")
                                        m.on_target_reach = d.on_target_reach;
                                    else if (typeof d.on_target_reach === "object")
                                        m.on_target_reach = new $util.LongBits(d.on_target_reach.low >>> 0, d.on_target_reach.high >>> 0).toNumber();
                                }
                                if (d.total_reach != null) {
                                    if ($util.Long)
                                        (m.total_reach = $util.Long.fromValue(d.total_reach)).unsigned = false;
                                    else if (typeof d.total_reach === "string")
                                        m.total_reach = parseInt(d.total_reach, 10);
                                    else if (typeof d.total_reach === "number")
                                        m.total_reach = d.total_reach;
                                    else if (typeof d.total_reach === "object")
                                        m.total_reach = new $util.LongBits(d.total_reach.low >>> 0, d.total_reach.high >>> 0).toNumber();
                                }
                                if (d.on_target_impressions != null) {
                                    if ($util.Long)
                                        (m.on_target_impressions = $util.Long.fromValue(d.on_target_impressions)).unsigned = false;
                                    else if (typeof d.on_target_impressions === "string")
                                        m.on_target_impressions = parseInt(d.on_target_impressions, 10);
                                    else if (typeof d.on_target_impressions === "number")
                                        m.on_target_impressions = d.on_target_impressions;
                                    else if (typeof d.on_target_impressions === "object")
                                        m.on_target_impressions = new $util.LongBits(d.on_target_impressions.low >>> 0, d.on_target_impressions.high >>> 0).toNumber();
                                }
                                if (d.total_impressions != null) {
                                    if ($util.Long)
                                        (m.total_impressions = $util.Long.fromValue(d.total_impressions)).unsigned = false;
                                    else if (typeof d.total_impressions === "string")
                                        m.total_impressions = parseInt(d.total_impressions, 10);
                                    else if (typeof d.total_impressions === "number")
                                        m.total_impressions = d.total_impressions;
                                    else if (typeof d.total_impressions === "object")
                                        m.total_impressions = new $util.LongBits(d.total_impressions.low >>> 0, d.total_impressions.high >>> 0).toNumber();
                                }
                                if (d.viewable_impressions != null) {
                                    if ($util.Long)
                                        (m.viewable_impressions = $util.Long.fromValue(d.viewable_impressions)).unsigned = false;
                                    else if (typeof d.viewable_impressions === "string")
                                        m.viewable_impressions = parseInt(d.viewable_impressions, 10);
                                    else if (typeof d.viewable_impressions === "number")
                                        m.viewable_impressions = d.viewable_impressions;
                                    else if (typeof d.viewable_impressions === "object")
                                        m.viewable_impressions = new $util.LongBits(d.viewable_impressions.low >>> 0, d.viewable_impressions.high >>> 0).toNumber();
                                }
                                if (d.on_target_coview_reach != null) {
                                    if ($util.Long)
                                        (m.on_target_coview_reach = $util.Long.fromValue(d.on_target_coview_reach)).unsigned = false;
                                    else if (typeof d.on_target_coview_reach === "string")
                                        m.on_target_coview_reach = parseInt(d.on_target_coview_reach, 10);
                                    else if (typeof d.on_target_coview_reach === "number")
                                        m.on_target_coview_reach = d.on_target_coview_reach;
                                    else if (typeof d.on_target_coview_reach === "object")
                                        m.on_target_coview_reach = new $util.LongBits(d.on_target_coview_reach.low >>> 0, d.on_target_coview_reach.high >>> 0).toNumber();
                                }
                                if (d.total_coview_reach != null) {
                                    if ($util.Long)
                                        (m.total_coview_reach = $util.Long.fromValue(d.total_coview_reach)).unsigned = false;
                                    else if (typeof d.total_coview_reach === "string")
                                        m.total_coview_reach = parseInt(d.total_coview_reach, 10);
                                    else if (typeof d.total_coview_reach === "number")
                                        m.total_coview_reach = d.total_coview_reach;
                                    else if (typeof d.total_coview_reach === "object")
                                        m.total_coview_reach = new $util.LongBits(d.total_coview_reach.low >>> 0, d.total_coview_reach.high >>> 0).toNumber();
                                }
                                if (d.on_target_coview_impressions != null) {
                                    if ($util.Long)
                                        (m.on_target_coview_impressions = $util.Long.fromValue(d.on_target_coview_impressions)).unsigned = false;
                                    else if (typeof d.on_target_coview_impressions === "string")
                                        m.on_target_coview_impressions = parseInt(d.on_target_coview_impressions, 10);
                                    else if (typeof d.on_target_coview_impressions === "number")
                                        m.on_target_coview_impressions = d.on_target_coview_impressions;
                                    else if (typeof d.on_target_coview_impressions === "object")
                                        m.on_target_coview_impressions = new $util.LongBits(d.on_target_coview_impressions.low >>> 0, d.on_target_coview_impressions.high >>> 0).toNumber();
                                }
                                if (d.total_coview_impressions != null) {
                                    if ($util.Long)
                                        (m.total_coview_impressions = $util.Long.fromValue(d.total_coview_impressions)).unsigned = false;
                                    else if (typeof d.total_coview_impressions === "string")
                                        m.total_coview_impressions = parseInt(d.total_coview_impressions, 10);
                                    else if (typeof d.total_coview_impressions === "number")
                                        m.total_coview_impressions = d.total_coview_impressions;
                                    else if (typeof d.total_coview_impressions === "object")
                                        m.total_coview_impressions = new $util.LongBits(d.total_coview_impressions.low >>> 0, d.total_coview_impressions.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a PlannedProductForecast message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @static
                             * @param {google.ads.googleads.v11.services.PlannedProductForecast} m PlannedProductForecast
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            PlannedProductForecast.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.on_target_reach = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.on_target_reach = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_reach = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_reach = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.on_target_impressions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.on_target_impressions = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_impressions = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_impressions = o.longs === String ? "0" : 0;
                                }
                                if (m.on_target_reach != null && m.hasOwnProperty("on_target_reach")) {
                                    if (typeof m.on_target_reach === "number")
                                        d.on_target_reach = o.longs === String ? String(m.on_target_reach) : m.on_target_reach;
                                    else
                                        d.on_target_reach = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_reach) : o.longs === Number ? new $util.LongBits(m.on_target_reach.low >>> 0, m.on_target_reach.high >>> 0).toNumber() : m.on_target_reach;
                                }
                                if (m.total_reach != null && m.hasOwnProperty("total_reach")) {
                                    if (typeof m.total_reach === "number")
                                        d.total_reach = o.longs === String ? String(m.total_reach) : m.total_reach;
                                    else
                                        d.total_reach = o.longs === String ? $util.Long.prototype.toString.call(m.total_reach) : o.longs === Number ? new $util.LongBits(m.total_reach.low >>> 0, m.total_reach.high >>> 0).toNumber() : m.total_reach;
                                }
                                if (m.on_target_impressions != null && m.hasOwnProperty("on_target_impressions")) {
                                    if (typeof m.on_target_impressions === "number")
                                        d.on_target_impressions = o.longs === String ? String(m.on_target_impressions) : m.on_target_impressions;
                                    else
                                        d.on_target_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_impressions) : o.longs === Number ? new $util.LongBits(m.on_target_impressions.low >>> 0, m.on_target_impressions.high >>> 0).toNumber() : m.on_target_impressions;
                                }
                                if (m.total_impressions != null && m.hasOwnProperty("total_impressions")) {
                                    if (typeof m.total_impressions === "number")
                                        d.total_impressions = o.longs === String ? String(m.total_impressions) : m.total_impressions;
                                    else
                                        d.total_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.total_impressions) : o.longs === Number ? new $util.LongBits(m.total_impressions.low >>> 0, m.total_impressions.high >>> 0).toNumber() : m.total_impressions;
                                }
                                if (m.viewable_impressions != null && m.hasOwnProperty("viewable_impressions")) {
                                    if (typeof m.viewable_impressions === "number")
                                        d.viewable_impressions = o.longs === String ? String(m.viewable_impressions) : m.viewable_impressions;
                                    else
                                        d.viewable_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.viewable_impressions) : o.longs === Number ? new $util.LongBits(m.viewable_impressions.low >>> 0, m.viewable_impressions.high >>> 0).toNumber() : m.viewable_impressions;
                                    if (o.oneofs)
                                        d._viewable_impressions = "viewable_impressions";
                                }
                                if (m.on_target_coview_reach != null && m.hasOwnProperty("on_target_coview_reach")) {
                                    if (typeof m.on_target_coview_reach === "number")
                                        d.on_target_coview_reach = o.longs === String ? String(m.on_target_coview_reach) : m.on_target_coview_reach;
                                    else
                                        d.on_target_coview_reach = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_coview_reach) : o.longs === Number ? new $util.LongBits(m.on_target_coview_reach.low >>> 0, m.on_target_coview_reach.high >>> 0).toNumber() : m.on_target_coview_reach;
                                    if (o.oneofs)
                                        d._on_target_coview_reach = "on_target_coview_reach";
                                }
                                if (m.total_coview_reach != null && m.hasOwnProperty("total_coview_reach")) {
                                    if (typeof m.total_coview_reach === "number")
                                        d.total_coview_reach = o.longs === String ? String(m.total_coview_reach) : m.total_coview_reach;
                                    else
                                        d.total_coview_reach = o.longs === String ? $util.Long.prototype.toString.call(m.total_coview_reach) : o.longs === Number ? new $util.LongBits(m.total_coview_reach.low >>> 0, m.total_coview_reach.high >>> 0).toNumber() : m.total_coview_reach;
                                    if (o.oneofs)
                                        d._total_coview_reach = "total_coview_reach";
                                }
                                if (m.on_target_coview_impressions != null && m.hasOwnProperty("on_target_coview_impressions")) {
                                    if (typeof m.on_target_coview_impressions === "number")
                                        d.on_target_coview_impressions = o.longs === String ? String(m.on_target_coview_impressions) : m.on_target_coview_impressions;
                                    else
                                        d.on_target_coview_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_coview_impressions) : o.longs === Number ? new $util.LongBits(m.on_target_coview_impressions.low >>> 0, m.on_target_coview_impressions.high >>> 0).toNumber() : m.on_target_coview_impressions;
                                    if (o.oneofs)
                                        d._on_target_coview_impressions = "on_target_coview_impressions";
                                }
                                if (m.total_coview_impressions != null && m.hasOwnProperty("total_coview_impressions")) {
                                    if (typeof m.total_coview_impressions === "number")
                                        d.total_coview_impressions = o.longs === String ? String(m.total_coview_impressions) : m.total_coview_impressions;
                                    else
                                        d.total_coview_impressions = o.longs === String ? $util.Long.prototype.toString.call(m.total_coview_impressions) : o.longs === Number ? new $util.LongBits(m.total_coview_impressions.low >>> 0, m.total_coview_impressions.high >>> 0).toNumber() : m.total_coview_impressions;
                                    if (o.oneofs)
                                        d._total_coview_impressions = "total_coview_impressions";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this PlannedProductForecast to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.PlannedProductForecast
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            PlannedProductForecast.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return PlannedProductForecast;
                        })();
    
                        services.OnTargetAudienceMetrics = (function() {
    
                            /**
                             * Properties of an OnTargetAudienceMetrics.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IOnTargetAudienceMetrics
                             * @property {number|null} [youtube_audience_size] OnTargetAudienceMetrics youtube_audience_size
                             * @property {number|null} [census_audience_size] OnTargetAudienceMetrics census_audience_size
                             */
    
                            /**
                             * Constructs a new OnTargetAudienceMetrics.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an OnTargetAudienceMetrics.
                             * @implements IOnTargetAudienceMetrics
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IOnTargetAudienceMetrics=} [p] Properties to set
                             */
                            function OnTargetAudienceMetrics(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * OnTargetAudienceMetrics youtube_audience_size.
                             * @member {number|null|undefined} youtube_audience_size
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @instance
                             */
                            OnTargetAudienceMetrics.prototype.youtube_audience_size = null;
    
                            /**
                             * OnTargetAudienceMetrics census_audience_size.
                             * @member {number|null|undefined} census_audience_size
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @instance
                             */
                            OnTargetAudienceMetrics.prototype.census_audience_size = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * OnTargetAudienceMetrics _youtube_audience_size.
                             * @member {"youtube_audience_size"|undefined} _youtube_audience_size
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @instance
                             */
                            Object.defineProperty(OnTargetAudienceMetrics.prototype, "_youtube_audience_size", {
                                get: $util.oneOfGetter($oneOfFields = ["youtube_audience_size"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * OnTargetAudienceMetrics _census_audience_size.
                             * @member {"census_audience_size"|undefined} _census_audience_size
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @instance
                             */
                            Object.defineProperty(OnTargetAudienceMetrics.prototype, "_census_audience_size", {
                                get: $util.oneOfGetter($oneOfFields = ["census_audience_size"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new OnTargetAudienceMetrics instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IOnTargetAudienceMetrics=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.OnTargetAudienceMetrics} OnTargetAudienceMetrics instance
                             */
                            OnTargetAudienceMetrics.create = function create(properties) {
                                return new OnTargetAudienceMetrics(properties);
                            };
    
                            /**
                             * Encodes the specified OnTargetAudienceMetrics message. Does not implicitly {@link google.ads.googleads.v11.services.OnTargetAudienceMetrics.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IOnTargetAudienceMetrics} m OnTargetAudienceMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OnTargetAudienceMetrics.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.youtube_audience_size != null && Object.hasOwnProperty.call(m, "youtube_audience_size"))
                                    w.uint32(24).int64(m.youtube_audience_size);
                                if (m.census_audience_size != null && Object.hasOwnProperty.call(m, "census_audience_size"))
                                    w.uint32(32).int64(m.census_audience_size);
                                return w;
                            };
    
                            /**
                             * Encodes the specified OnTargetAudienceMetrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.OnTargetAudienceMetrics.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.IOnTargetAudienceMetrics} message OnTargetAudienceMetrics message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            OnTargetAudienceMetrics.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an OnTargetAudienceMetrics message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.OnTargetAudienceMetrics} OnTargetAudienceMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OnTargetAudienceMetrics.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.youtube_audience_size = r.int64();
                                        break;
                                    case 4:
                                        m.census_audience_size = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an OnTargetAudienceMetrics message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.OnTargetAudienceMetrics} OnTargetAudienceMetrics
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            OnTargetAudienceMetrics.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an OnTargetAudienceMetrics message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            OnTargetAudienceMetrics.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.youtube_audience_size != null && m.hasOwnProperty("youtube_audience_size")) {
                                    p._youtube_audience_size = 1;
                                    if (!$util.isInteger(m.youtube_audience_size) && !(m.youtube_audience_size && $util.isInteger(m.youtube_audience_size.low) && $util.isInteger(m.youtube_audience_size.high)))
                                        return "youtube_audience_size: integer|Long expected";
                                }
                                if (m.census_audience_size != null && m.hasOwnProperty("census_audience_size")) {
                                    p._census_audience_size = 1;
                                    if (!$util.isInteger(m.census_audience_size) && !(m.census_audience_size && $util.isInteger(m.census_audience_size.low) && $util.isInteger(m.census_audience_size.high)))
                                        return "census_audience_size: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an OnTargetAudienceMetrics message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.OnTargetAudienceMetrics} OnTargetAudienceMetrics
                             */
                            OnTargetAudienceMetrics.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.OnTargetAudienceMetrics();
                                if (d.youtube_audience_size != null) {
                                    if ($util.Long)
                                        (m.youtube_audience_size = $util.Long.fromValue(d.youtube_audience_size)).unsigned = false;
                                    else if (typeof d.youtube_audience_size === "string")
                                        m.youtube_audience_size = parseInt(d.youtube_audience_size, 10);
                                    else if (typeof d.youtube_audience_size === "number")
                                        m.youtube_audience_size = d.youtube_audience_size;
                                    else if (typeof d.youtube_audience_size === "object")
                                        m.youtube_audience_size = new $util.LongBits(d.youtube_audience_size.low >>> 0, d.youtube_audience_size.high >>> 0).toNumber();
                                }
                                if (d.census_audience_size != null) {
                                    if ($util.Long)
                                        (m.census_audience_size = $util.Long.fromValue(d.census_audience_size)).unsigned = false;
                                    else if (typeof d.census_audience_size === "string")
                                        m.census_audience_size = parseInt(d.census_audience_size, 10);
                                    else if (typeof d.census_audience_size === "number")
                                        m.census_audience_size = d.census_audience_size;
                                    else if (typeof d.census_audience_size === "object")
                                        m.census_audience_size = new $util.LongBits(d.census_audience_size.low >>> 0, d.census_audience_size.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an OnTargetAudienceMetrics message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @static
                             * @param {google.ads.googleads.v11.services.OnTargetAudienceMetrics} m OnTargetAudienceMetrics
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            OnTargetAudienceMetrics.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.youtube_audience_size != null && m.hasOwnProperty("youtube_audience_size")) {
                                    if (typeof m.youtube_audience_size === "number")
                                        d.youtube_audience_size = o.longs === String ? String(m.youtube_audience_size) : m.youtube_audience_size;
                                    else
                                        d.youtube_audience_size = o.longs === String ? $util.Long.prototype.toString.call(m.youtube_audience_size) : o.longs === Number ? new $util.LongBits(m.youtube_audience_size.low >>> 0, m.youtube_audience_size.high >>> 0).toNumber() : m.youtube_audience_size;
                                    if (o.oneofs)
                                        d._youtube_audience_size = "youtube_audience_size";
                                }
                                if (m.census_audience_size != null && m.hasOwnProperty("census_audience_size")) {
                                    if (typeof m.census_audience_size === "number")
                                        d.census_audience_size = o.longs === String ? String(m.census_audience_size) : m.census_audience_size;
                                    else
                                        d.census_audience_size = o.longs === String ? $util.Long.prototype.toString.call(m.census_audience_size) : o.longs === Number ? new $util.LongBits(m.census_audience_size.low >>> 0, m.census_audience_size.high >>> 0).toNumber() : m.census_audience_size;
                                    if (o.oneofs)
                                        d._census_audience_size = "census_audience_size";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this OnTargetAudienceMetrics to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.OnTargetAudienceMetrics
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            OnTargetAudienceMetrics.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return OnTargetAudienceMetrics;
                        })();
    
                        services.EffectiveFrequencyBreakdown = (function() {
    
                            /**
                             * Properties of an EffectiveFrequencyBreakdown.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IEffectiveFrequencyBreakdown
                             * @property {number|null} [effective_frequency] EffectiveFrequencyBreakdown effective_frequency
                             * @property {number|null} [on_target_reach] EffectiveFrequencyBreakdown on_target_reach
                             * @property {number|null} [total_reach] EffectiveFrequencyBreakdown total_reach
                             * @property {number|null} [effective_coview_reach] EffectiveFrequencyBreakdown effective_coview_reach
                             * @property {number|null} [on_target_effective_coview_reach] EffectiveFrequencyBreakdown on_target_effective_coview_reach
                             */
    
                            /**
                             * Constructs a new EffectiveFrequencyBreakdown.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an EffectiveFrequencyBreakdown.
                             * @implements IEffectiveFrequencyBreakdown
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyBreakdown=} [p] Properties to set
                             */
                            function EffectiveFrequencyBreakdown(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * EffectiveFrequencyBreakdown effective_frequency.
                             * @member {number} effective_frequency
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             */
                            EffectiveFrequencyBreakdown.prototype.effective_frequency = 0;
    
                            /**
                             * EffectiveFrequencyBreakdown on_target_reach.
                             * @member {number} on_target_reach
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             */
                            EffectiveFrequencyBreakdown.prototype.on_target_reach = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * EffectiveFrequencyBreakdown total_reach.
                             * @member {number} total_reach
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             */
                            EffectiveFrequencyBreakdown.prototype.total_reach = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * EffectiveFrequencyBreakdown effective_coview_reach.
                             * @member {number|null|undefined} effective_coview_reach
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             */
                            EffectiveFrequencyBreakdown.prototype.effective_coview_reach = null;
    
                            /**
                             * EffectiveFrequencyBreakdown on_target_effective_coview_reach.
                             * @member {number|null|undefined} on_target_effective_coview_reach
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             */
                            EffectiveFrequencyBreakdown.prototype.on_target_effective_coview_reach = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * EffectiveFrequencyBreakdown _effective_coview_reach.
                             * @member {"effective_coview_reach"|undefined} _effective_coview_reach
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             */
                            Object.defineProperty(EffectiveFrequencyBreakdown.prototype, "_effective_coview_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["effective_coview_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * EffectiveFrequencyBreakdown _on_target_effective_coview_reach.
                             * @member {"on_target_effective_coview_reach"|undefined} _on_target_effective_coview_reach
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             */
                            Object.defineProperty(EffectiveFrequencyBreakdown.prototype, "_on_target_effective_coview_reach", {
                                get: $util.oneOfGetter($oneOfFields = ["on_target_effective_coview_reach"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new EffectiveFrequencyBreakdown instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyBreakdown=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyBreakdown} EffectiveFrequencyBreakdown instance
                             */
                            EffectiveFrequencyBreakdown.create = function create(properties) {
                                return new EffectiveFrequencyBreakdown(properties);
                            };
    
                            /**
                             * Encodes the specified EffectiveFrequencyBreakdown message. Does not implicitly {@link google.ads.googleads.v11.services.EffectiveFrequencyBreakdown.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyBreakdown} m EffectiveFrequencyBreakdown message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EffectiveFrequencyBreakdown.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.effective_frequency != null && Object.hasOwnProperty.call(m, "effective_frequency"))
                                    w.uint32(8).int32(m.effective_frequency);
                                if (m.on_target_reach != null && Object.hasOwnProperty.call(m, "on_target_reach"))
                                    w.uint32(16).int64(m.on_target_reach);
                                if (m.total_reach != null && Object.hasOwnProperty.call(m, "total_reach"))
                                    w.uint32(24).int64(m.total_reach);
                                if (m.effective_coview_reach != null && Object.hasOwnProperty.call(m, "effective_coview_reach"))
                                    w.uint32(32).int64(m.effective_coview_reach);
                                if (m.on_target_effective_coview_reach != null && Object.hasOwnProperty.call(m, "on_target_effective_coview_reach"))
                                    w.uint32(40).int64(m.on_target_effective_coview_reach);
                                return w;
                            };
    
                            /**
                             * Encodes the specified EffectiveFrequencyBreakdown message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.EffectiveFrequencyBreakdown.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {google.ads.googleads.v11.services.IEffectiveFrequencyBreakdown} message EffectiveFrequencyBreakdown message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EffectiveFrequencyBreakdown.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EffectiveFrequencyBreakdown message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyBreakdown} EffectiveFrequencyBreakdown
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EffectiveFrequencyBreakdown.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.effective_frequency = r.int32();
                                        break;
                                    case 2:
                                        m.on_target_reach = r.int64();
                                        break;
                                    case 3:
                                        m.total_reach = r.int64();
                                        break;
                                    case 4:
                                        m.effective_coview_reach = r.int64();
                                        break;
                                    case 5:
                                        m.on_target_effective_coview_reach = r.int64();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an EffectiveFrequencyBreakdown message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyBreakdown} EffectiveFrequencyBreakdown
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EffectiveFrequencyBreakdown.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EffectiveFrequencyBreakdown message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EffectiveFrequencyBreakdown.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.effective_frequency != null && m.hasOwnProperty("effective_frequency")) {
                                    if (!$util.isInteger(m.effective_frequency))
                                        return "effective_frequency: integer expected";
                                }
                                if (m.on_target_reach != null && m.hasOwnProperty("on_target_reach")) {
                                    if (!$util.isInteger(m.on_target_reach) && !(m.on_target_reach && $util.isInteger(m.on_target_reach.low) && $util.isInteger(m.on_target_reach.high)))
                                        return "on_target_reach: integer|Long expected";
                                }
                                if (m.total_reach != null && m.hasOwnProperty("total_reach")) {
                                    if (!$util.isInteger(m.total_reach) && !(m.total_reach && $util.isInteger(m.total_reach.low) && $util.isInteger(m.total_reach.high)))
                                        return "total_reach: integer|Long expected";
                                }
                                if (m.effective_coview_reach != null && m.hasOwnProperty("effective_coview_reach")) {
                                    p._effective_coview_reach = 1;
                                    if (!$util.isInteger(m.effective_coview_reach) && !(m.effective_coview_reach && $util.isInteger(m.effective_coview_reach.low) && $util.isInteger(m.effective_coview_reach.high)))
                                        return "effective_coview_reach: integer|Long expected";
                                }
                                if (m.on_target_effective_coview_reach != null && m.hasOwnProperty("on_target_effective_coview_reach")) {
                                    p._on_target_effective_coview_reach = 1;
                                    if (!$util.isInteger(m.on_target_effective_coview_reach) && !(m.on_target_effective_coview_reach && $util.isInteger(m.on_target_effective_coview_reach.low) && $util.isInteger(m.on_target_effective_coview_reach.high)))
                                        return "on_target_effective_coview_reach: integer|Long expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an EffectiveFrequencyBreakdown message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.EffectiveFrequencyBreakdown} EffectiveFrequencyBreakdown
                             */
                            EffectiveFrequencyBreakdown.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.EffectiveFrequencyBreakdown();
                                if (d.effective_frequency != null) {
                                    m.effective_frequency = d.effective_frequency | 0;
                                }
                                if (d.on_target_reach != null) {
                                    if ($util.Long)
                                        (m.on_target_reach = $util.Long.fromValue(d.on_target_reach)).unsigned = false;
                                    else if (typeof d.on_target_reach === "string")
                                        m.on_target_reach = parseInt(d.on_target_reach, 10);
                                    else if (typeof d.on_target_reach === "number")
                                        m.on_target_reach = d.on_target_reach;
                                    else if (typeof d.on_target_reach === "object")
                                        m.on_target_reach = new $util.LongBits(d.on_target_reach.low >>> 0, d.on_target_reach.high >>> 0).toNumber();
                                }
                                if (d.total_reach != null) {
                                    if ($util.Long)
                                        (m.total_reach = $util.Long.fromValue(d.total_reach)).unsigned = false;
                                    else if (typeof d.total_reach === "string")
                                        m.total_reach = parseInt(d.total_reach, 10);
                                    else if (typeof d.total_reach === "number")
                                        m.total_reach = d.total_reach;
                                    else if (typeof d.total_reach === "object")
                                        m.total_reach = new $util.LongBits(d.total_reach.low >>> 0, d.total_reach.high >>> 0).toNumber();
                                }
                                if (d.effective_coview_reach != null) {
                                    if ($util.Long)
                                        (m.effective_coview_reach = $util.Long.fromValue(d.effective_coview_reach)).unsigned = false;
                                    else if (typeof d.effective_coview_reach === "string")
                                        m.effective_coview_reach = parseInt(d.effective_coview_reach, 10);
                                    else if (typeof d.effective_coview_reach === "number")
                                        m.effective_coview_reach = d.effective_coview_reach;
                                    else if (typeof d.effective_coview_reach === "object")
                                        m.effective_coview_reach = new $util.LongBits(d.effective_coview_reach.low >>> 0, d.effective_coview_reach.high >>> 0).toNumber();
                                }
                                if (d.on_target_effective_coview_reach != null) {
                                    if ($util.Long)
                                        (m.on_target_effective_coview_reach = $util.Long.fromValue(d.on_target_effective_coview_reach)).unsigned = false;
                                    else if (typeof d.on_target_effective_coview_reach === "string")
                                        m.on_target_effective_coview_reach = parseInt(d.on_target_effective_coview_reach, 10);
                                    else if (typeof d.on_target_effective_coview_reach === "number")
                                        m.on_target_effective_coview_reach = d.on_target_effective_coview_reach;
                                    else if (typeof d.on_target_effective_coview_reach === "object")
                                        m.on_target_effective_coview_reach = new $util.LongBits(d.on_target_effective_coview_reach.low >>> 0, d.on_target_effective_coview_reach.high >>> 0).toNumber();
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an EffectiveFrequencyBreakdown message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @static
                             * @param {google.ads.googleads.v11.services.EffectiveFrequencyBreakdown} m EffectiveFrequencyBreakdown
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EffectiveFrequencyBreakdown.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.effective_frequency = 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.on_target_reach = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.on_target_reach = o.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var n = new $util.Long(0, 0, false);
                                        d.total_reach = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                    } else
                                        d.total_reach = o.longs === String ? "0" : 0;
                                }
                                if (m.effective_frequency != null && m.hasOwnProperty("effective_frequency")) {
                                    d.effective_frequency = m.effective_frequency;
                                }
                                if (m.on_target_reach != null && m.hasOwnProperty("on_target_reach")) {
                                    if (typeof m.on_target_reach === "number")
                                        d.on_target_reach = o.longs === String ? String(m.on_target_reach) : m.on_target_reach;
                                    else
                                        d.on_target_reach = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_reach) : o.longs === Number ? new $util.LongBits(m.on_target_reach.low >>> 0, m.on_target_reach.high >>> 0).toNumber() : m.on_target_reach;
                                }
                                if (m.total_reach != null && m.hasOwnProperty("total_reach")) {
                                    if (typeof m.total_reach === "number")
                                        d.total_reach = o.longs === String ? String(m.total_reach) : m.total_reach;
                                    else
                                        d.total_reach = o.longs === String ? $util.Long.prototype.toString.call(m.total_reach) : o.longs === Number ? new $util.LongBits(m.total_reach.low >>> 0, m.total_reach.high >>> 0).toNumber() : m.total_reach;
                                }
                                if (m.effective_coview_reach != null && m.hasOwnProperty("effective_coview_reach")) {
                                    if (typeof m.effective_coview_reach === "number")
                                        d.effective_coview_reach = o.longs === String ? String(m.effective_coview_reach) : m.effective_coview_reach;
                                    else
                                        d.effective_coview_reach = o.longs === String ? $util.Long.prototype.toString.call(m.effective_coview_reach) : o.longs === Number ? new $util.LongBits(m.effective_coview_reach.low >>> 0, m.effective_coview_reach.high >>> 0).toNumber() : m.effective_coview_reach;
                                    if (o.oneofs)
                                        d._effective_coview_reach = "effective_coview_reach";
                                }
                                if (m.on_target_effective_coview_reach != null && m.hasOwnProperty("on_target_effective_coview_reach")) {
                                    if (typeof m.on_target_effective_coview_reach === "number")
                                        d.on_target_effective_coview_reach = o.longs === String ? String(m.on_target_effective_coview_reach) : m.on_target_effective_coview_reach;
                                    else
                                        d.on_target_effective_coview_reach = o.longs === String ? $util.Long.prototype.toString.call(m.on_target_effective_coview_reach) : o.longs === Number ? new $util.LongBits(m.on_target_effective_coview_reach.low >>> 0, m.on_target_effective_coview_reach.high >>> 0).toNumber() : m.on_target_effective_coview_reach;
                                    if (o.oneofs)
                                        d._on_target_effective_coview_reach = "on_target_effective_coview_reach";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this EffectiveFrequencyBreakdown to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.EffectiveFrequencyBreakdown
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EffectiveFrequencyBreakdown.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return EffectiveFrequencyBreakdown;
                        })();
    
                        services.ForecastMetricOptions = (function() {
    
                            /**
                             * Properties of a ForecastMetricOptions.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IForecastMetricOptions
                             * @property {boolean|null} [include_coview] ForecastMetricOptions include_coview
                             */
    
                            /**
                             * Constructs a new ForecastMetricOptions.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ForecastMetricOptions.
                             * @implements IForecastMetricOptions
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IForecastMetricOptions=} [p] Properties to set
                             */
                            function ForecastMetricOptions(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ForecastMetricOptions include_coview.
                             * @member {boolean} include_coview
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @instance
                             */
                            ForecastMetricOptions.prototype.include_coview = false;
    
                            /**
                             * Creates a new ForecastMetricOptions instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecastMetricOptions=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ForecastMetricOptions} ForecastMetricOptions instance
                             */
                            ForecastMetricOptions.create = function create(properties) {
                                return new ForecastMetricOptions(properties);
                            };
    
                            /**
                             * Encodes the specified ForecastMetricOptions message. Does not implicitly {@link google.ads.googleads.v11.services.ForecastMetricOptions.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecastMetricOptions} m ForecastMetricOptions message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ForecastMetricOptions.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.include_coview != null && Object.hasOwnProperty.call(m, "include_coview"))
                                    w.uint32(8).bool(m.include_coview);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ForecastMetricOptions message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ForecastMetricOptions.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {google.ads.googleads.v11.services.IForecastMetricOptions} message ForecastMetricOptions message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ForecastMetricOptions.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ForecastMetricOptions message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ForecastMetricOptions} ForecastMetricOptions
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ForecastMetricOptions.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ForecastMetricOptions();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.include_coview = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a ForecastMetricOptions message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ForecastMetricOptions} ForecastMetricOptions
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ForecastMetricOptions.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ForecastMetricOptions message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ForecastMetricOptions.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.include_coview != null && m.hasOwnProperty("include_coview")) {
                                    if (typeof m.include_coview !== "boolean")
                                        return "include_coview: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ForecastMetricOptions message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ForecastMetricOptions} ForecastMetricOptions
                             */
                            ForecastMetricOptions.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ForecastMetricOptions)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ForecastMetricOptions();
                                if (d.include_coview != null) {
                                    m.include_coview = Boolean(d.include_coview);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a ForecastMetricOptions message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @static
                             * @param {google.ads.googleads.v11.services.ForecastMetricOptions} m ForecastMetricOptions
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ForecastMetricOptions.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.include_coview = false;
                                }
                                if (m.include_coview != null && m.hasOwnProperty("include_coview")) {
                                    d.include_coview = m.include_coview;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ForecastMetricOptions to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ForecastMetricOptions
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ForecastMetricOptions.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ForecastMetricOptions;
                        })();
    
                        services.RecommendationService = (function() {
    
                            /**
                             * Constructs a new RecommendationService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RecommendationService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function RecommendationService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (RecommendationService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = RecommendationService;
    
                            /**
                             * Creates new RecommendationService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RecommendationService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {RecommendationService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            RecommendationService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.RecommendationService|applyRecommendation}.
                             * @memberof google.ads.googleads.v11.services.RecommendationService
                             * @typedef ApplyRecommendationCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.ApplyRecommendationResponse} [response] ApplyRecommendationResponse
                             */
    
                            /**
                             * Calls ApplyRecommendation.
                             * @function applyRecommendation
                             * @memberof google.ads.googleads.v11.services.RecommendationService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationRequest} request ApplyRecommendationRequest message or plain object
                             * @param {google.ads.googleads.v11.services.RecommendationService.ApplyRecommendationCallback} callback Node-style callback called with the error, if any, and ApplyRecommendationResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(RecommendationService.prototype.applyRecommendation = function applyRecommendation(request, callback) {
                                return this.rpcCall(applyRecommendation, $root.google.ads.googleads.v11.services.ApplyRecommendationRequest, $root.google.ads.googleads.v11.services.ApplyRecommendationResponse, request, callback);
                            }, "name", { value: "ApplyRecommendation" });
    
                            /**
                             * Calls ApplyRecommendation.
                             * @function applyRecommendation
                             * @memberof google.ads.googleads.v11.services.RecommendationService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationRequest} request ApplyRecommendationRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.ApplyRecommendationResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.RecommendationService|dismissRecommendation}.
                             * @memberof google.ads.googleads.v11.services.RecommendationService
                             * @typedef DismissRecommendationCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.DismissRecommendationResponse} [response] DismissRecommendationResponse
                             */
    
                            /**
                             * Calls DismissRecommendation.
                             * @function dismissRecommendation
                             * @memberof google.ads.googleads.v11.services.RecommendationService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationRequest} request DismissRecommendationRequest message or plain object
                             * @param {google.ads.googleads.v11.services.RecommendationService.DismissRecommendationCallback} callback Node-style callback called with the error, if any, and DismissRecommendationResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(RecommendationService.prototype.dismissRecommendation = function dismissRecommendation(request, callback) {
                                return this.rpcCall(dismissRecommendation, $root.google.ads.googleads.v11.services.DismissRecommendationRequest, $root.google.ads.googleads.v11.services.DismissRecommendationResponse, request, callback);
                            }, "name", { value: "DismissRecommendation" });
    
                            /**
                             * Calls DismissRecommendation.
                             * @function dismissRecommendation
                             * @memberof google.ads.googleads.v11.services.RecommendationService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationRequest} request DismissRecommendationRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.DismissRecommendationResponse>} Promise
                             * @variation 2
                             */
    
                            return RecommendationService;
                        })();
    
                        services.ApplyRecommendationRequest = (function() {
    
                            /**
                             * Properties of an ApplyRecommendationRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IApplyRecommendationRequest
                             * @property {string|null} [customer_id] ApplyRecommendationRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IApplyRecommendationOperation>|null} [operations] ApplyRecommendationRequest operations
                             * @property {boolean|null} [partial_failure] ApplyRecommendationRequest partial_failure
                             */
    
                            /**
                             * Constructs a new ApplyRecommendationRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ApplyRecommendationRequest.
                             * @implements IApplyRecommendationRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationRequest=} [p] Properties to set
                             */
                            function ApplyRecommendationRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ApplyRecommendationRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @instance
                             */
                            ApplyRecommendationRequest.prototype.customer_id = "";
    
                            /**
                             * ApplyRecommendationRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IApplyRecommendationOperation>} operations
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @instance
                             */
                            ApplyRecommendationRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * ApplyRecommendationRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @instance
                             */
                            ApplyRecommendationRequest.prototype.partial_failure = false;
    
                            /**
                             * Creates a new ApplyRecommendationRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationRequest} ApplyRecommendationRequest instance
                             */
                            ApplyRecommendationRequest.create = function create(properties) {
                                return new ApplyRecommendationRequest(properties);
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationRequest message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationRequest} m ApplyRecommendationRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.encode(m.operations[i], w.uint32(18).fork()).ldelim();
                                }
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(24).bool(m.partial_failure);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationRequest} message ApplyRecommendationRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ApplyRecommendationRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationRequest} ApplyRecommendationRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.ApplyRecommendationOperation.decode(r, r.uint32()));
                                        break;
                                    case 3:
                                        m.partial_failure = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ApplyRecommendationRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationRequest} ApplyRecommendationRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ApplyRecommendationRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ApplyRecommendationRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ApplyRecommendationRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationRequest} ApplyRecommendationRequest
                             */
                            ApplyRecommendationRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ApplyRecommendationRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ApplyRecommendationRequest} m ApplyRecommendationRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ApplyRecommendationRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.toObject(m.operations[j], o);
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ApplyRecommendationRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ApplyRecommendationRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ApplyRecommendationRequest;
                        })();
    
                        services.ApplyRecommendationOperation = (function() {
    
                            /**
                             * Properties of an ApplyRecommendationOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IApplyRecommendationOperation
                             * @property {string|null} [resource_name] ApplyRecommendationOperation resource_name
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICampaignBudgetParameters|null} [campaign_budget] ApplyRecommendationOperation campaign_budget
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITextAdParameters|null} [text_ad] ApplyRecommendationOperation text_ad
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.IKeywordParameters|null} [keyword] ApplyRecommendationOperation keyword
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetCpaOptInParameters|null} [target_cpa_opt_in] ApplyRecommendationOperation target_cpa_opt_in
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetRoasOptInParameters|null} [target_roas_opt_in] ApplyRecommendationOperation target_roas_opt_in
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICalloutExtensionParameters|null} [callout_extension] ApplyRecommendationOperation callout_extension
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICallExtensionParameters|null} [call_extension] ApplyRecommendationOperation call_extension
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.ISitelinkExtensionParameters|null} [sitelink_extension] ApplyRecommendationOperation sitelink_extension
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.IMoveUnusedBudgetParameters|null} [move_unused_budget] ApplyRecommendationOperation move_unused_budget
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdParameters|null} [responsive_search_ad] ApplyRecommendationOperation responsive_search_ad
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.IUseBroadMatchKeywordParameters|null} [use_broad_match_keyword] ApplyRecommendationOperation use_broad_match_keyword
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdAssetParameters|null} [responsive_search_ad_asset] ApplyRecommendationOperation responsive_search_ad_asset
                             * @property {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdImproveAdStrengthParameters|null} [responsive_search_ad_improve_ad_strength] ApplyRecommendationOperation responsive_search_ad_improve_ad_strength
                             */
    
                            /**
                             * Constructs a new ApplyRecommendationOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ApplyRecommendationOperation.
                             * @implements IApplyRecommendationOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationOperation=} [p] Properties to set
                             */
                            function ApplyRecommendationOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ApplyRecommendationOperation resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.resource_name = "";
    
                            /**
                             * ApplyRecommendationOperation campaign_budget.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICampaignBudgetParameters|null|undefined} campaign_budget
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.campaign_budget = null;
    
                            /**
                             * ApplyRecommendationOperation text_ad.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITextAdParameters|null|undefined} text_ad
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.text_ad = null;
    
                            /**
                             * ApplyRecommendationOperation keyword.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.IKeywordParameters|null|undefined} keyword
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.keyword = null;
    
                            /**
                             * ApplyRecommendationOperation target_cpa_opt_in.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetCpaOptInParameters|null|undefined} target_cpa_opt_in
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.target_cpa_opt_in = null;
    
                            /**
                             * ApplyRecommendationOperation target_roas_opt_in.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetRoasOptInParameters|null|undefined} target_roas_opt_in
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.target_roas_opt_in = null;
    
                            /**
                             * ApplyRecommendationOperation callout_extension.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICalloutExtensionParameters|null|undefined} callout_extension
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.callout_extension = null;
    
                            /**
                             * ApplyRecommendationOperation call_extension.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICallExtensionParameters|null|undefined} call_extension
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.call_extension = null;
    
                            /**
                             * ApplyRecommendationOperation sitelink_extension.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.ISitelinkExtensionParameters|null|undefined} sitelink_extension
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.sitelink_extension = null;
    
                            /**
                             * ApplyRecommendationOperation move_unused_budget.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.IMoveUnusedBudgetParameters|null|undefined} move_unused_budget
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.move_unused_budget = null;
    
                            /**
                             * ApplyRecommendationOperation responsive_search_ad.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdParameters|null|undefined} responsive_search_ad
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.responsive_search_ad = null;
    
                            /**
                             * ApplyRecommendationOperation use_broad_match_keyword.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.IUseBroadMatchKeywordParameters|null|undefined} use_broad_match_keyword
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.use_broad_match_keyword = null;
    
                            /**
                             * ApplyRecommendationOperation responsive_search_ad_asset.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdAssetParameters|null|undefined} responsive_search_ad_asset
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.responsive_search_ad_asset = null;
    
                            /**
                             * ApplyRecommendationOperation responsive_search_ad_improve_ad_strength.
                             * @member {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdImproveAdStrengthParameters|null|undefined} responsive_search_ad_improve_ad_strength
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            ApplyRecommendationOperation.prototype.responsive_search_ad_improve_ad_strength = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * ApplyRecommendationOperation apply_parameters.
                             * @member {"campaign_budget"|"text_ad"|"keyword"|"target_cpa_opt_in"|"target_roas_opt_in"|"callout_extension"|"call_extension"|"sitelink_extension"|"move_unused_budget"|"responsive_search_ad"|"use_broad_match_keyword"|"responsive_search_ad_asset"|"responsive_search_ad_improve_ad_strength"|undefined} apply_parameters
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             */
                            Object.defineProperty(ApplyRecommendationOperation.prototype, "apply_parameters", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign_budget", "text_ad", "keyword", "target_cpa_opt_in", "target_roas_opt_in", "callout_extension", "call_extension", "sitelink_extension", "move_unused_budget", "responsive_search_ad", "use_broad_match_keyword", "responsive_search_ad_asset", "responsive_search_ad_improve_ad_strength"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new ApplyRecommendationOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation} ApplyRecommendationOperation instance
                             */
                            ApplyRecommendationOperation.create = function create(properties) {
                                return new ApplyRecommendationOperation(properties);
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationOperation message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationOperation} m ApplyRecommendationOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                if (m.campaign_budget != null && Object.hasOwnProperty.call(m, "campaign_budget"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters.encode(m.campaign_budget, w.uint32(18).fork()).ldelim();
                                if (m.text_ad != null && Object.hasOwnProperty.call(m, "text_ad"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.encode(m.text_ad, w.uint32(26).fork()).ldelim();
                                if (m.keyword != null && Object.hasOwnProperty.call(m, "keyword"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters.encode(m.keyword, w.uint32(34).fork()).ldelim();
                                if (m.target_cpa_opt_in != null && Object.hasOwnProperty.call(m, "target_cpa_opt_in"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters.encode(m.target_cpa_opt_in, w.uint32(42).fork()).ldelim();
                                if (m.callout_extension != null && Object.hasOwnProperty.call(m, "callout_extension"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.encode(m.callout_extension, w.uint32(50).fork()).ldelim();
                                if (m.call_extension != null && Object.hasOwnProperty.call(m, "call_extension"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.encode(m.call_extension, w.uint32(58).fork()).ldelim();
                                if (m.sitelink_extension != null && Object.hasOwnProperty.call(m, "sitelink_extension"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.encode(m.sitelink_extension, w.uint32(66).fork()).ldelim();
                                if (m.move_unused_budget != null && Object.hasOwnProperty.call(m, "move_unused_budget"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters.encode(m.move_unused_budget, w.uint32(74).fork()).ldelim();
                                if (m.target_roas_opt_in != null && Object.hasOwnProperty.call(m, "target_roas_opt_in"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters.encode(m.target_roas_opt_in, w.uint32(82).fork()).ldelim();
                                if (m.responsive_search_ad != null && Object.hasOwnProperty.call(m, "responsive_search_ad"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.encode(m.responsive_search_ad, w.uint32(90).fork()).ldelim();
                                if (m.use_broad_match_keyword != null && Object.hasOwnProperty.call(m, "use_broad_match_keyword"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters.encode(m.use_broad_match_keyword, w.uint32(98).fork()).ldelim();
                                if (m.responsive_search_ad_asset != null && Object.hasOwnProperty.call(m, "responsive_search_ad_asset"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.encode(m.responsive_search_ad_asset, w.uint32(106).fork()).ldelim();
                                if (m.responsive_search_ad_improve_ad_strength != null && Object.hasOwnProperty.call(m, "responsive_search_ad_improve_ad_strength"))
                                    $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.encode(m.responsive_search_ad_improve_ad_strength, w.uint32(114).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationOperation} message ApplyRecommendationOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ApplyRecommendationOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation} ApplyRecommendationOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    case 2:
                                        m.campaign_budget = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.text_ad = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.decode(r, r.uint32());
                                        break;
                                    case 4:
                                        m.keyword = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.target_cpa_opt_in = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters.decode(r, r.uint32());
                                        break;
                                    case 10:
                                        m.target_roas_opt_in = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters.decode(r, r.uint32());
                                        break;
                                    case 6:
                                        m.callout_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.decode(r, r.uint32());
                                        break;
                                    case 7:
                                        m.call_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.decode(r, r.uint32());
                                        break;
                                    case 8:
                                        m.sitelink_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.move_unused_budget = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters.decode(r, r.uint32());
                                        break;
                                    case 11:
                                        m.responsive_search_ad = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.decode(r, r.uint32());
                                        break;
                                    case 12:
                                        m.use_broad_match_keyword = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters.decode(r, r.uint32());
                                        break;
                                    case 13:
                                        m.responsive_search_ad_asset = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.decode(r, r.uint32());
                                        break;
                                    case 14:
                                        m.responsive_search_ad_improve_ad_strength = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ApplyRecommendationOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation} ApplyRecommendationOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ApplyRecommendationOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ApplyRecommendationOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters.verify(m.campaign_budget);
                                        if (e)
                                            return "campaign_budget." + e;
                                    }
                                }
                                if (m.text_ad != null && m.hasOwnProperty("text_ad")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.verify(m.text_ad);
                                        if (e)
                                            return "text_ad." + e;
                                    }
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters.verify(m.keyword);
                                        if (e)
                                            return "keyword." + e;
                                    }
                                }
                                if (m.target_cpa_opt_in != null && m.hasOwnProperty("target_cpa_opt_in")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters.verify(m.target_cpa_opt_in);
                                        if (e)
                                            return "target_cpa_opt_in." + e;
                                    }
                                }
                                if (m.target_roas_opt_in != null && m.hasOwnProperty("target_roas_opt_in")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters.verify(m.target_roas_opt_in);
                                        if (e)
                                            return "target_roas_opt_in." + e;
                                    }
                                }
                                if (m.callout_extension != null && m.hasOwnProperty("callout_extension")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.verify(m.callout_extension);
                                        if (e)
                                            return "callout_extension." + e;
                                    }
                                }
                                if (m.call_extension != null && m.hasOwnProperty("call_extension")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.verify(m.call_extension);
                                        if (e)
                                            return "call_extension." + e;
                                    }
                                }
                                if (m.sitelink_extension != null && m.hasOwnProperty("sitelink_extension")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.verify(m.sitelink_extension);
                                        if (e)
                                            return "sitelink_extension." + e;
                                    }
                                }
                                if (m.move_unused_budget != null && m.hasOwnProperty("move_unused_budget")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters.verify(m.move_unused_budget);
                                        if (e)
                                            return "move_unused_budget." + e;
                                    }
                                }
                                if (m.responsive_search_ad != null && m.hasOwnProperty("responsive_search_ad")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.verify(m.responsive_search_ad);
                                        if (e)
                                            return "responsive_search_ad." + e;
                                    }
                                }
                                if (m.use_broad_match_keyword != null && m.hasOwnProperty("use_broad_match_keyword")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters.verify(m.use_broad_match_keyword);
                                        if (e)
                                            return "use_broad_match_keyword." + e;
                                    }
                                }
                                if (m.responsive_search_ad_asset != null && m.hasOwnProperty("responsive_search_ad_asset")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.verify(m.responsive_search_ad_asset);
                                        if (e)
                                            return "responsive_search_ad_asset." + e;
                                    }
                                }
                                if (m.responsive_search_ad_improve_ad_strength != null && m.hasOwnProperty("responsive_search_ad_improve_ad_strength")) {
                                    if (p.apply_parameters === 1)
                                        return "apply_parameters: multiple values";
                                    p.apply_parameters = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.verify(m.responsive_search_ad_improve_ad_strength);
                                        if (e)
                                            return "responsive_search_ad_improve_ad_strength." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ApplyRecommendationOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation} ApplyRecommendationOperation
                             */
                            ApplyRecommendationOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                if (d.campaign_budget != null) {
                                    if (typeof d.campaign_budget !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.campaign_budget: object expected");
                                    m.campaign_budget = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters.fromObject(d.campaign_budget);
                                }
                                if (d.text_ad != null) {
                                    if (typeof d.text_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.text_ad: object expected");
                                    m.text_ad = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.fromObject(d.text_ad);
                                }
                                if (d.keyword != null) {
                                    if (typeof d.keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.keyword: object expected");
                                    m.keyword = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters.fromObject(d.keyword);
                                }
                                if (d.target_cpa_opt_in != null) {
                                    if (typeof d.target_cpa_opt_in !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.target_cpa_opt_in: object expected");
                                    m.target_cpa_opt_in = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters.fromObject(d.target_cpa_opt_in);
                                }
                                if (d.target_roas_opt_in != null) {
                                    if (typeof d.target_roas_opt_in !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.target_roas_opt_in: object expected");
                                    m.target_roas_opt_in = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters.fromObject(d.target_roas_opt_in);
                                }
                                if (d.callout_extension != null) {
                                    if (typeof d.callout_extension !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.callout_extension: object expected");
                                    m.callout_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.fromObject(d.callout_extension);
                                }
                                if (d.call_extension != null) {
                                    if (typeof d.call_extension !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.call_extension: object expected");
                                    m.call_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.fromObject(d.call_extension);
                                }
                                if (d.sitelink_extension != null) {
                                    if (typeof d.sitelink_extension !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.sitelink_extension: object expected");
                                    m.sitelink_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.fromObject(d.sitelink_extension);
                                }
                                if (d.move_unused_budget != null) {
                                    if (typeof d.move_unused_budget !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.move_unused_budget: object expected");
                                    m.move_unused_budget = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters.fromObject(d.move_unused_budget);
                                }
                                if (d.responsive_search_ad != null) {
                                    if (typeof d.responsive_search_ad !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.responsive_search_ad: object expected");
                                    m.responsive_search_ad = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.fromObject(d.responsive_search_ad);
                                }
                                if (d.use_broad_match_keyword != null) {
                                    if (typeof d.use_broad_match_keyword !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.use_broad_match_keyword: object expected");
                                    m.use_broad_match_keyword = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters.fromObject(d.use_broad_match_keyword);
                                }
                                if (d.responsive_search_ad_asset != null) {
                                    if (typeof d.responsive_search_ad_asset !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.responsive_search_ad_asset: object expected");
                                    m.responsive_search_ad_asset = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.fromObject(d.responsive_search_ad_asset);
                                }
                                if (d.responsive_search_ad_improve_ad_strength != null) {
                                    if (typeof d.responsive_search_ad_improve_ad_strength !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.responsive_search_ad_improve_ad_strength: object expected");
                                    m.responsive_search_ad_improve_ad_strength = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.fromObject(d.responsive_search_ad_improve_ad_strength);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ApplyRecommendationOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation} m ApplyRecommendationOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ApplyRecommendationOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                if (m.campaign_budget != null && m.hasOwnProperty("campaign_budget")) {
                                    d.campaign_budget = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters.toObject(m.campaign_budget, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "campaign_budget";
                                }
                                if (m.text_ad != null && m.hasOwnProperty("text_ad")) {
                                    d.text_ad = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.toObject(m.text_ad, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "text_ad";
                                }
                                if (m.keyword != null && m.hasOwnProperty("keyword")) {
                                    d.keyword = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters.toObject(m.keyword, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "keyword";
                                }
                                if (m.target_cpa_opt_in != null && m.hasOwnProperty("target_cpa_opt_in")) {
                                    d.target_cpa_opt_in = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters.toObject(m.target_cpa_opt_in, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "target_cpa_opt_in";
                                }
                                if (m.callout_extension != null && m.hasOwnProperty("callout_extension")) {
                                    d.callout_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.toObject(m.callout_extension, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "callout_extension";
                                }
                                if (m.call_extension != null && m.hasOwnProperty("call_extension")) {
                                    d.call_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.toObject(m.call_extension, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "call_extension";
                                }
                                if (m.sitelink_extension != null && m.hasOwnProperty("sitelink_extension")) {
                                    d.sitelink_extension = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.toObject(m.sitelink_extension, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "sitelink_extension";
                                }
                                if (m.move_unused_budget != null && m.hasOwnProperty("move_unused_budget")) {
                                    d.move_unused_budget = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters.toObject(m.move_unused_budget, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "move_unused_budget";
                                }
                                if (m.target_roas_opt_in != null && m.hasOwnProperty("target_roas_opt_in")) {
                                    d.target_roas_opt_in = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters.toObject(m.target_roas_opt_in, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "target_roas_opt_in";
                                }
                                if (m.responsive_search_ad != null && m.hasOwnProperty("responsive_search_ad")) {
                                    d.responsive_search_ad = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.toObject(m.responsive_search_ad, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "responsive_search_ad";
                                }
                                if (m.use_broad_match_keyword != null && m.hasOwnProperty("use_broad_match_keyword")) {
                                    d.use_broad_match_keyword = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters.toObject(m.use_broad_match_keyword, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "use_broad_match_keyword";
                                }
                                if (m.responsive_search_ad_asset != null && m.hasOwnProperty("responsive_search_ad_asset")) {
                                    d.responsive_search_ad_asset = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.toObject(m.responsive_search_ad_asset, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "responsive_search_ad_asset";
                                }
                                if (m.responsive_search_ad_improve_ad_strength != null && m.hasOwnProperty("responsive_search_ad_improve_ad_strength")) {
                                    d.responsive_search_ad_improve_ad_strength = $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.toObject(m.responsive_search_ad_improve_ad_strength, o);
                                    if (o.oneofs)
                                        d.apply_parameters = "responsive_search_ad_improve_ad_strength";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ApplyRecommendationOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ApplyRecommendationOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            ApplyRecommendationOperation.CampaignBudgetParameters = (function() {
    
                                /**
                                 * Properties of a CampaignBudgetParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface ICampaignBudgetParameters
                                 * @property {number|null} [new_budget_amount_micros] CampaignBudgetParameters new_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new CampaignBudgetParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a CampaignBudgetParameters.
                                 * @implements ICampaignBudgetParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICampaignBudgetParameters=} [p] Properties to set
                                 */
                                function CampaignBudgetParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CampaignBudgetParameters new_budget_amount_micros.
                                 * @member {number|null|undefined} new_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @instance
                                 */
                                CampaignBudgetParameters.prototype.new_budget_amount_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * CampaignBudgetParameters _new_budget_amount_micros.
                                 * @member {"new_budget_amount_micros"|undefined} _new_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @instance
                                 */
                                Object.defineProperty(CampaignBudgetParameters.prototype, "_new_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["new_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new CampaignBudgetParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICampaignBudgetParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters} CampaignBudgetParameters instance
                                 */
                                CampaignBudgetParameters.create = function create(properties) {
                                    return new CampaignBudgetParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified CampaignBudgetParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICampaignBudgetParameters} m CampaignBudgetParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CampaignBudgetParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.new_budget_amount_micros != null && Object.hasOwnProperty.call(m, "new_budget_amount_micros"))
                                        w.uint32(16).int64(m.new_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CampaignBudgetParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICampaignBudgetParameters} message CampaignBudgetParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CampaignBudgetParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CampaignBudgetParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters} CampaignBudgetParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CampaignBudgetParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.new_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CampaignBudgetParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters} CampaignBudgetParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CampaignBudgetParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CampaignBudgetParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CampaignBudgetParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.new_budget_amount_micros != null && m.hasOwnProperty("new_budget_amount_micros")) {
                                        p._new_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.new_budget_amount_micros) && !(m.new_budget_amount_micros && $util.isInteger(m.new_budget_amount_micros.low) && $util.isInteger(m.new_budget_amount_micros.high)))
                                            return "new_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CampaignBudgetParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters} CampaignBudgetParameters
                                 */
                                CampaignBudgetParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters();
                                    if (d.new_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.new_budget_amount_micros = $util.Long.fromValue(d.new_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.new_budget_amount_micros === "string")
                                            m.new_budget_amount_micros = parseInt(d.new_budget_amount_micros, 10);
                                        else if (typeof d.new_budget_amount_micros === "number")
                                            m.new_budget_amount_micros = d.new_budget_amount_micros;
                                        else if (typeof d.new_budget_amount_micros === "object")
                                            m.new_budget_amount_micros = new $util.LongBits(d.new_budget_amount_micros.low >>> 0, d.new_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CampaignBudgetParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters} m CampaignBudgetParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CampaignBudgetParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.new_budget_amount_micros != null && m.hasOwnProperty("new_budget_amount_micros")) {
                                        if (typeof m.new_budget_amount_micros === "number")
                                            d.new_budget_amount_micros = o.longs === String ? String(m.new_budget_amount_micros) : m.new_budget_amount_micros;
                                        else
                                            d.new_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.new_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.new_budget_amount_micros.low >>> 0, m.new_budget_amount_micros.high >>> 0).toNumber() : m.new_budget_amount_micros;
                                        if (o.oneofs)
                                            d._new_budget_amount_micros = "new_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CampaignBudgetParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CampaignBudgetParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CampaignBudgetParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CampaignBudgetParameters;
                            })();
    
                            ApplyRecommendationOperation.TextAdParameters = (function() {
    
                                /**
                                 * Properties of a TextAdParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface ITextAdParameters
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [ad] TextAdParameters ad
                                 */
    
                                /**
                                 * Constructs a new TextAdParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a TextAdParameters.
                                 * @implements ITextAdParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITextAdParameters=} [p] Properties to set
                                 */
                                function TextAdParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TextAdParameters ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} ad
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @instance
                                 */
                                TextAdParameters.prototype.ad = null;
    
                                /**
                                 * Creates a new TextAdParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITextAdParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters} TextAdParameters instance
                                 */
                                TextAdParameters.create = function create(properties) {
                                    return new TextAdParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified TextAdParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITextAdParameters} m TextAdParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TextAdParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.ad != null && Object.hasOwnProperty.call(m, "ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.ad, w.uint32(10).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TextAdParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITextAdParameters} message TextAdParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TextAdParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TextAdParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters} TextAdParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TextAdParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TextAdParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters} TextAdParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TextAdParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TextAdParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TextAdParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.ad);
                                            if (e)
                                                return "ad." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TextAdParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters} TextAdParameters
                                 */
                                TextAdParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters();
                                    if (d.ad != null) {
                                        if (typeof d.ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters.ad: object expected");
                                        m.ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.ad);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TextAdParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters} m TextAdParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TextAdParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.ad = null;
                                    }
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        d.ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.ad, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TextAdParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TextAdParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TextAdParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TextAdParameters;
                            })();
    
                            ApplyRecommendationOperation.KeywordParameters = (function() {
    
                                /**
                                 * Properties of a KeywordParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface IKeywordParameters
                                 * @property {string|null} [ad_group] KeywordParameters ad_group
                                 * @property {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType|null} [match_type] KeywordParameters match_type
                                 * @property {number|null} [cpc_bid_micros] KeywordParameters cpc_bid_micros
                                 */
    
                                /**
                                 * Constructs a new KeywordParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a KeywordParameters.
                                 * @implements IKeywordParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IKeywordParameters=} [p] Properties to set
                                 */
                                function KeywordParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * KeywordParameters ad_group.
                                 * @member {string|null|undefined} ad_group
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @instance
                                 */
                                KeywordParameters.prototype.ad_group = null;
    
                                /**
                                 * KeywordParameters match_type.
                                 * @member {google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType} match_type
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @instance
                                 */
                                KeywordParameters.prototype.match_type = 0;
    
                                /**
                                 * KeywordParameters cpc_bid_micros.
                                 * @member {number|null|undefined} cpc_bid_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @instance
                                 */
                                KeywordParameters.prototype.cpc_bid_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * KeywordParameters _ad_group.
                                 * @member {"ad_group"|undefined} _ad_group
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @instance
                                 */
                                Object.defineProperty(KeywordParameters.prototype, "_ad_group", {
                                    get: $util.oneOfGetter($oneOfFields = ["ad_group"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * KeywordParameters _cpc_bid_micros.
                                 * @member {"cpc_bid_micros"|undefined} _cpc_bid_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @instance
                                 */
                                Object.defineProperty(KeywordParameters.prototype, "_cpc_bid_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["cpc_bid_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new KeywordParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IKeywordParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters} KeywordParameters instance
                                 */
                                KeywordParameters.create = function create(properties) {
                                    return new KeywordParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified KeywordParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IKeywordParameters} m KeywordParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                KeywordParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.match_type != null && Object.hasOwnProperty.call(m, "match_type"))
                                        w.uint32(16).int32(m.match_type);
                                    if (m.ad_group != null && Object.hasOwnProperty.call(m, "ad_group"))
                                        w.uint32(34).string(m.ad_group);
                                    if (m.cpc_bid_micros != null && Object.hasOwnProperty.call(m, "cpc_bid_micros"))
                                        w.uint32(40).int64(m.cpc_bid_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified KeywordParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IKeywordParameters} message KeywordParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                KeywordParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a KeywordParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters} KeywordParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                KeywordParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 4:
                                            m.ad_group = r.string();
                                            break;
                                        case 2:
                                            m.match_type = r.int32();
                                            break;
                                        case 5:
                                            m.cpc_bid_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a KeywordParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters} KeywordParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                KeywordParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a KeywordParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                KeywordParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                        p._ad_group = 1;
                                        if (!$util.isString(m.ad_group))
                                            return "ad_group: string expected";
                                    }
                                    if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                        switch (m.match_type) {
                                        default:
                                            return "match_type: enum value expected";
                                        case 0:
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4:
                                            break;
                                        }
                                    }
                                    if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                        p._cpc_bid_micros = 1;
                                        if (!$util.isInteger(m.cpc_bid_micros) && !(m.cpc_bid_micros && $util.isInteger(m.cpc_bid_micros.low) && $util.isInteger(m.cpc_bid_micros.high)))
                                            return "cpc_bid_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a KeywordParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters} KeywordParameters
                                 */
                                KeywordParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters();
                                    if (d.ad_group != null) {
                                        m.ad_group = String(d.ad_group);
                                    }
                                    switch (d.match_type) {
                                    case "UNSPECIFIED":
                                    case 0:
                                        m.match_type = 0;
                                        break;
                                    case "UNKNOWN":
                                    case 1:
                                        m.match_type = 1;
                                        break;
                                    case "EXACT":
                                    case 2:
                                        m.match_type = 2;
                                        break;
                                    case "PHRASE":
                                    case 3:
                                        m.match_type = 3;
                                        break;
                                    case "BROAD":
                                    case 4:
                                        m.match_type = 4;
                                        break;
                                    }
                                    if (d.cpc_bid_micros != null) {
                                        if ($util.Long)
                                            (m.cpc_bid_micros = $util.Long.fromValue(d.cpc_bid_micros)).unsigned = false;
                                        else if (typeof d.cpc_bid_micros === "string")
                                            m.cpc_bid_micros = parseInt(d.cpc_bid_micros, 10);
                                        else if (typeof d.cpc_bid_micros === "number")
                                            m.cpc_bid_micros = d.cpc_bid_micros;
                                        else if (typeof d.cpc_bid_micros === "object")
                                            m.cpc_bid_micros = new $util.LongBits(d.cpc_bid_micros.low >>> 0, d.cpc_bid_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a KeywordParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters} m KeywordParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                KeywordParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.match_type = o.enums === String ? "UNSPECIFIED" : 0;
                                    }
                                    if (m.match_type != null && m.hasOwnProperty("match_type")) {
                                        d.match_type = o.enums === String ? $root.google.ads.googleads.v11.enums.KeywordMatchTypeEnum.KeywordMatchType[m.match_type] : m.match_type;
                                    }
                                    if (m.ad_group != null && m.hasOwnProperty("ad_group")) {
                                        d.ad_group = m.ad_group;
                                        if (o.oneofs)
                                            d._ad_group = "ad_group";
                                    }
                                    if (m.cpc_bid_micros != null && m.hasOwnProperty("cpc_bid_micros")) {
                                        if (typeof m.cpc_bid_micros === "number")
                                            d.cpc_bid_micros = o.longs === String ? String(m.cpc_bid_micros) : m.cpc_bid_micros;
                                        else
                                            d.cpc_bid_micros = o.longs === String ? $util.Long.prototype.toString.call(m.cpc_bid_micros) : o.longs === Number ? new $util.LongBits(m.cpc_bid_micros.low >>> 0, m.cpc_bid_micros.high >>> 0).toNumber() : m.cpc_bid_micros;
                                        if (o.oneofs)
                                            d._cpc_bid_micros = "cpc_bid_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this KeywordParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.KeywordParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                KeywordParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return KeywordParameters;
                            })();
    
                            ApplyRecommendationOperation.TargetCpaOptInParameters = (function() {
    
                                /**
                                 * Properties of a TargetCpaOptInParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface ITargetCpaOptInParameters
                                 * @property {number|null} [target_cpa_micros] TargetCpaOptInParameters target_cpa_micros
                                 * @property {number|null} [new_campaign_budget_amount_micros] TargetCpaOptInParameters new_campaign_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new TargetCpaOptInParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a TargetCpaOptInParameters.
                                 * @implements ITargetCpaOptInParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetCpaOptInParameters=} [p] Properties to set
                                 */
                                function TargetCpaOptInParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetCpaOptInParameters target_cpa_micros.
                                 * @member {number|null|undefined} target_cpa_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @instance
                                 */
                                TargetCpaOptInParameters.prototype.target_cpa_micros = null;
    
                                /**
                                 * TargetCpaOptInParameters new_campaign_budget_amount_micros.
                                 * @member {number|null|undefined} new_campaign_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @instance
                                 */
                                TargetCpaOptInParameters.prototype.new_campaign_budget_amount_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetCpaOptInParameters _target_cpa_micros.
                                 * @member {"target_cpa_micros"|undefined} _target_cpa_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @instance
                                 */
                                Object.defineProperty(TargetCpaOptInParameters.prototype, "_target_cpa_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_cpa_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * TargetCpaOptInParameters _new_campaign_budget_amount_micros.
                                 * @member {"new_campaign_budget_amount_micros"|undefined} _new_campaign_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @instance
                                 */
                                Object.defineProperty(TargetCpaOptInParameters.prototype, "_new_campaign_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["new_campaign_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetCpaOptInParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetCpaOptInParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters} TargetCpaOptInParameters instance
                                 */
                                TargetCpaOptInParameters.create = function create(properties) {
                                    return new TargetCpaOptInParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetCpaOptInParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetCpaOptInParameters} m TargetCpaOptInParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetCpaOptInParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_cpa_micros != null && Object.hasOwnProperty.call(m, "target_cpa_micros"))
                                        w.uint32(24).int64(m.target_cpa_micros);
                                    if (m.new_campaign_budget_amount_micros != null && Object.hasOwnProperty.call(m, "new_campaign_budget_amount_micros"))
                                        w.uint32(32).int64(m.new_campaign_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetCpaOptInParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetCpaOptInParameters} message TargetCpaOptInParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetCpaOptInParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetCpaOptInParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters} TargetCpaOptInParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetCpaOptInParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 3:
                                            m.target_cpa_micros = r.int64();
                                            break;
                                        case 4:
                                            m.new_campaign_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetCpaOptInParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters} TargetCpaOptInParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetCpaOptInParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetCpaOptInParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetCpaOptInParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                        p._target_cpa_micros = 1;
                                        if (!$util.isInteger(m.target_cpa_micros) && !(m.target_cpa_micros && $util.isInteger(m.target_cpa_micros.low) && $util.isInteger(m.target_cpa_micros.high)))
                                            return "target_cpa_micros: integer|Long expected";
                                    }
                                    if (m.new_campaign_budget_amount_micros != null && m.hasOwnProperty("new_campaign_budget_amount_micros")) {
                                        p._new_campaign_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.new_campaign_budget_amount_micros) && !(m.new_campaign_budget_amount_micros && $util.isInteger(m.new_campaign_budget_amount_micros.low) && $util.isInteger(m.new_campaign_budget_amount_micros.high)))
                                            return "new_campaign_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetCpaOptInParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters} TargetCpaOptInParameters
                                 */
                                TargetCpaOptInParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters();
                                    if (d.target_cpa_micros != null) {
                                        if ($util.Long)
                                            (m.target_cpa_micros = $util.Long.fromValue(d.target_cpa_micros)).unsigned = false;
                                        else if (typeof d.target_cpa_micros === "string")
                                            m.target_cpa_micros = parseInt(d.target_cpa_micros, 10);
                                        else if (typeof d.target_cpa_micros === "number")
                                            m.target_cpa_micros = d.target_cpa_micros;
                                        else if (typeof d.target_cpa_micros === "object")
                                            m.target_cpa_micros = new $util.LongBits(d.target_cpa_micros.low >>> 0, d.target_cpa_micros.high >>> 0).toNumber();
                                    }
                                    if (d.new_campaign_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.new_campaign_budget_amount_micros = $util.Long.fromValue(d.new_campaign_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.new_campaign_budget_amount_micros === "string")
                                            m.new_campaign_budget_amount_micros = parseInt(d.new_campaign_budget_amount_micros, 10);
                                        else if (typeof d.new_campaign_budget_amount_micros === "number")
                                            m.new_campaign_budget_amount_micros = d.new_campaign_budget_amount_micros;
                                        else if (typeof d.new_campaign_budget_amount_micros === "object")
                                            m.new_campaign_budget_amount_micros = new $util.LongBits(d.new_campaign_budget_amount_micros.low >>> 0, d.new_campaign_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetCpaOptInParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters} m TargetCpaOptInParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetCpaOptInParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.target_cpa_micros != null && m.hasOwnProperty("target_cpa_micros")) {
                                        if (typeof m.target_cpa_micros === "number")
                                            d.target_cpa_micros = o.longs === String ? String(m.target_cpa_micros) : m.target_cpa_micros;
                                        else
                                            d.target_cpa_micros = o.longs === String ? $util.Long.prototype.toString.call(m.target_cpa_micros) : o.longs === Number ? new $util.LongBits(m.target_cpa_micros.low >>> 0, m.target_cpa_micros.high >>> 0).toNumber() : m.target_cpa_micros;
                                        if (o.oneofs)
                                            d._target_cpa_micros = "target_cpa_micros";
                                    }
                                    if (m.new_campaign_budget_amount_micros != null && m.hasOwnProperty("new_campaign_budget_amount_micros")) {
                                        if (typeof m.new_campaign_budget_amount_micros === "number")
                                            d.new_campaign_budget_amount_micros = o.longs === String ? String(m.new_campaign_budget_amount_micros) : m.new_campaign_budget_amount_micros;
                                        else
                                            d.new_campaign_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.new_campaign_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.new_campaign_budget_amount_micros.low >>> 0, m.new_campaign_budget_amount_micros.high >>> 0).toNumber() : m.new_campaign_budget_amount_micros;
                                        if (o.oneofs)
                                            d._new_campaign_budget_amount_micros = "new_campaign_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetCpaOptInParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetCpaOptInParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetCpaOptInParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TargetCpaOptInParameters;
                            })();
    
                            ApplyRecommendationOperation.TargetRoasOptInParameters = (function() {
    
                                /**
                                 * Properties of a TargetRoasOptInParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface ITargetRoasOptInParameters
                                 * @property {number|null} [target_roas] TargetRoasOptInParameters target_roas
                                 * @property {number|null} [new_campaign_budget_amount_micros] TargetRoasOptInParameters new_campaign_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new TargetRoasOptInParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a TargetRoasOptInParameters.
                                 * @implements ITargetRoasOptInParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetRoasOptInParameters=} [p] Properties to set
                                 */
                                function TargetRoasOptInParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * TargetRoasOptInParameters target_roas.
                                 * @member {number|null|undefined} target_roas
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @instance
                                 */
                                TargetRoasOptInParameters.prototype.target_roas = null;
    
                                /**
                                 * TargetRoasOptInParameters new_campaign_budget_amount_micros.
                                 * @member {number|null|undefined} new_campaign_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @instance
                                 */
                                TargetRoasOptInParameters.prototype.new_campaign_budget_amount_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * TargetRoasOptInParameters _target_roas.
                                 * @member {"target_roas"|undefined} _target_roas
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @instance
                                 */
                                Object.defineProperty(TargetRoasOptInParameters.prototype, "_target_roas", {
                                    get: $util.oneOfGetter($oneOfFields = ["target_roas"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * TargetRoasOptInParameters _new_campaign_budget_amount_micros.
                                 * @member {"new_campaign_budget_amount_micros"|undefined} _new_campaign_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @instance
                                 */
                                Object.defineProperty(TargetRoasOptInParameters.prototype, "_new_campaign_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["new_campaign_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new TargetRoasOptInParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetRoasOptInParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters} TargetRoasOptInParameters instance
                                 */
                                TargetRoasOptInParameters.create = function create(properties) {
                                    return new TargetRoasOptInParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified TargetRoasOptInParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetRoasOptInParameters} m TargetRoasOptInParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetRoasOptInParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.target_roas != null && Object.hasOwnProperty.call(m, "target_roas"))
                                        w.uint32(9).double(m.target_roas);
                                    if (m.new_campaign_budget_amount_micros != null && Object.hasOwnProperty.call(m, "new_campaign_budget_amount_micros"))
                                        w.uint32(16).int64(m.new_campaign_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified TargetRoasOptInParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ITargetRoasOptInParameters} message TargetRoasOptInParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                TargetRoasOptInParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a TargetRoasOptInParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters} TargetRoasOptInParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetRoasOptInParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.target_roas = r.double();
                                            break;
                                        case 2:
                                            m.new_campaign_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a TargetRoasOptInParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters} TargetRoasOptInParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                TargetRoasOptInParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a TargetRoasOptInParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                TargetRoasOptInParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                        p._target_roas = 1;
                                        if (typeof m.target_roas !== "number")
                                            return "target_roas: number expected";
                                    }
                                    if (m.new_campaign_budget_amount_micros != null && m.hasOwnProperty("new_campaign_budget_amount_micros")) {
                                        p._new_campaign_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.new_campaign_budget_amount_micros) && !(m.new_campaign_budget_amount_micros && $util.isInteger(m.new_campaign_budget_amount_micros.low) && $util.isInteger(m.new_campaign_budget_amount_micros.high)))
                                            return "new_campaign_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a TargetRoasOptInParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters} TargetRoasOptInParameters
                                 */
                                TargetRoasOptInParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters();
                                    if (d.target_roas != null) {
                                        m.target_roas = Number(d.target_roas);
                                    }
                                    if (d.new_campaign_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.new_campaign_budget_amount_micros = $util.Long.fromValue(d.new_campaign_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.new_campaign_budget_amount_micros === "string")
                                            m.new_campaign_budget_amount_micros = parseInt(d.new_campaign_budget_amount_micros, 10);
                                        else if (typeof d.new_campaign_budget_amount_micros === "number")
                                            m.new_campaign_budget_amount_micros = d.new_campaign_budget_amount_micros;
                                        else if (typeof d.new_campaign_budget_amount_micros === "object")
                                            m.new_campaign_budget_amount_micros = new $util.LongBits(d.new_campaign_budget_amount_micros.low >>> 0, d.new_campaign_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a TargetRoasOptInParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters} m TargetRoasOptInParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                TargetRoasOptInParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.target_roas != null && m.hasOwnProperty("target_roas")) {
                                        d.target_roas = o.json && !isFinite(m.target_roas) ? String(m.target_roas) : m.target_roas;
                                        if (o.oneofs)
                                            d._target_roas = "target_roas";
                                    }
                                    if (m.new_campaign_budget_amount_micros != null && m.hasOwnProperty("new_campaign_budget_amount_micros")) {
                                        if (typeof m.new_campaign_budget_amount_micros === "number")
                                            d.new_campaign_budget_amount_micros = o.longs === String ? String(m.new_campaign_budget_amount_micros) : m.new_campaign_budget_amount_micros;
                                        else
                                            d.new_campaign_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.new_campaign_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.new_campaign_budget_amount_micros.low >>> 0, m.new_campaign_budget_amount_micros.high >>> 0).toNumber() : m.new_campaign_budget_amount_micros;
                                        if (o.oneofs)
                                            d._new_campaign_budget_amount_micros = "new_campaign_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this TargetRoasOptInParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.TargetRoasOptInParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                TargetRoasOptInParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return TargetRoasOptInParameters;
                            })();
    
                            ApplyRecommendationOperation.CalloutExtensionParameters = (function() {
    
                                /**
                                 * Properties of a CalloutExtensionParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface ICalloutExtensionParameters
                                 * @property {Array.<google.ads.googleads.v11.common.ICalloutFeedItem>|null} [callout_extensions] CalloutExtensionParameters callout_extensions
                                 */
    
                                /**
                                 * Constructs a new CalloutExtensionParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a CalloutExtensionParameters.
                                 * @implements ICalloutExtensionParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICalloutExtensionParameters=} [p] Properties to set
                                 */
                                function CalloutExtensionParameters(p) {
                                    this.callout_extensions = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CalloutExtensionParameters callout_extensions.
                                 * @member {Array.<google.ads.googleads.v11.common.ICalloutFeedItem>} callout_extensions
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @instance
                                 */
                                CalloutExtensionParameters.prototype.callout_extensions = $util.emptyArray;
    
                                /**
                                 * Creates a new CalloutExtensionParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICalloutExtensionParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters} CalloutExtensionParameters instance
                                 */
                                CalloutExtensionParameters.create = function create(properties) {
                                    return new CalloutExtensionParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified CalloutExtensionParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICalloutExtensionParameters} m CalloutExtensionParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CalloutExtensionParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.callout_extensions != null && m.callout_extensions.length) {
                                        for (var i = 0; i < m.callout_extensions.length; ++i)
                                            $root.google.ads.googleads.v11.common.CalloutFeedItem.encode(m.callout_extensions[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CalloutExtensionParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICalloutExtensionParameters} message CalloutExtensionParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CalloutExtensionParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CalloutExtensionParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters} CalloutExtensionParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CalloutExtensionParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.callout_extensions && m.callout_extensions.length))
                                                m.callout_extensions = [];
                                            m.callout_extensions.push($root.google.ads.googleads.v11.common.CalloutFeedItem.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CalloutExtensionParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters} CalloutExtensionParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CalloutExtensionParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CalloutExtensionParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CalloutExtensionParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.callout_extensions != null && m.hasOwnProperty("callout_extensions")) {
                                        if (!Array.isArray(m.callout_extensions))
                                            return "callout_extensions: array expected";
                                        for (var i = 0; i < m.callout_extensions.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.CalloutFeedItem.verify(m.callout_extensions[i]);
                                                if (e)
                                                    return "callout_extensions." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CalloutExtensionParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters} CalloutExtensionParameters
                                 */
                                CalloutExtensionParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters();
                                    if (d.callout_extensions) {
                                        if (!Array.isArray(d.callout_extensions))
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.callout_extensions: array expected");
                                        m.callout_extensions = [];
                                        for (var i = 0; i < d.callout_extensions.length; ++i) {
                                            if (typeof d.callout_extensions[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters.callout_extensions: object expected");
                                            m.callout_extensions[i] = $root.google.ads.googleads.v11.common.CalloutFeedItem.fromObject(d.callout_extensions[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CalloutExtensionParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters} m CalloutExtensionParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CalloutExtensionParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.callout_extensions = [];
                                    }
                                    if (m.callout_extensions && m.callout_extensions.length) {
                                        d.callout_extensions = [];
                                        for (var j = 0; j < m.callout_extensions.length; ++j) {
                                            d.callout_extensions[j] = $root.google.ads.googleads.v11.common.CalloutFeedItem.toObject(m.callout_extensions[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CalloutExtensionParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CalloutExtensionParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CalloutExtensionParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CalloutExtensionParameters;
                            })();
    
                            ApplyRecommendationOperation.CallExtensionParameters = (function() {
    
                                /**
                                 * Properties of a CallExtensionParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface ICallExtensionParameters
                                 * @property {Array.<google.ads.googleads.v11.common.ICallFeedItem>|null} [call_extensions] CallExtensionParameters call_extensions
                                 */
    
                                /**
                                 * Constructs a new CallExtensionParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a CallExtensionParameters.
                                 * @implements ICallExtensionParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICallExtensionParameters=} [p] Properties to set
                                 */
                                function CallExtensionParameters(p) {
                                    this.call_extensions = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * CallExtensionParameters call_extensions.
                                 * @member {Array.<google.ads.googleads.v11.common.ICallFeedItem>} call_extensions
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @instance
                                 */
                                CallExtensionParameters.prototype.call_extensions = $util.emptyArray;
    
                                /**
                                 * Creates a new CallExtensionParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICallExtensionParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters} CallExtensionParameters instance
                                 */
                                CallExtensionParameters.create = function create(properties) {
                                    return new CallExtensionParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified CallExtensionParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICallExtensionParameters} m CallExtensionParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CallExtensionParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.call_extensions != null && m.call_extensions.length) {
                                        for (var i = 0; i < m.call_extensions.length; ++i)
                                            $root.google.ads.googleads.v11.common.CallFeedItem.encode(m.call_extensions[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified CallExtensionParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ICallExtensionParameters} message CallExtensionParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                CallExtensionParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a CallExtensionParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters} CallExtensionParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CallExtensionParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.call_extensions && m.call_extensions.length))
                                                m.call_extensions = [];
                                            m.call_extensions.push($root.google.ads.googleads.v11.common.CallFeedItem.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a CallExtensionParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters} CallExtensionParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                CallExtensionParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a CallExtensionParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                CallExtensionParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.call_extensions != null && m.hasOwnProperty("call_extensions")) {
                                        if (!Array.isArray(m.call_extensions))
                                            return "call_extensions: array expected";
                                        for (var i = 0; i < m.call_extensions.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.CallFeedItem.verify(m.call_extensions[i]);
                                                if (e)
                                                    return "call_extensions." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a CallExtensionParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters} CallExtensionParameters
                                 */
                                CallExtensionParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters();
                                    if (d.call_extensions) {
                                        if (!Array.isArray(d.call_extensions))
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.call_extensions: array expected");
                                        m.call_extensions = [];
                                        for (var i = 0; i < d.call_extensions.length; ++i) {
                                            if (typeof d.call_extensions[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters.call_extensions: object expected");
                                            m.call_extensions[i] = $root.google.ads.googleads.v11.common.CallFeedItem.fromObject(d.call_extensions[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a CallExtensionParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters} m CallExtensionParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                CallExtensionParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.call_extensions = [];
                                    }
                                    if (m.call_extensions && m.call_extensions.length) {
                                        d.call_extensions = [];
                                        for (var j = 0; j < m.call_extensions.length; ++j) {
                                            d.call_extensions[j] = $root.google.ads.googleads.v11.common.CallFeedItem.toObject(m.call_extensions[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this CallExtensionParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.CallExtensionParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                CallExtensionParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return CallExtensionParameters;
                            })();
    
                            ApplyRecommendationOperation.SitelinkExtensionParameters = (function() {
    
                                /**
                                 * Properties of a SitelinkExtensionParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface ISitelinkExtensionParameters
                                 * @property {Array.<google.ads.googleads.v11.common.ISitelinkFeedItem>|null} [sitelink_extensions] SitelinkExtensionParameters sitelink_extensions
                                 */
    
                                /**
                                 * Constructs a new SitelinkExtensionParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a SitelinkExtensionParameters.
                                 * @implements ISitelinkExtensionParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ISitelinkExtensionParameters=} [p] Properties to set
                                 */
                                function SitelinkExtensionParameters(p) {
                                    this.sitelink_extensions = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * SitelinkExtensionParameters sitelink_extensions.
                                 * @member {Array.<google.ads.googleads.v11.common.ISitelinkFeedItem>} sitelink_extensions
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @instance
                                 */
                                SitelinkExtensionParameters.prototype.sitelink_extensions = $util.emptyArray;
    
                                /**
                                 * Creates a new SitelinkExtensionParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ISitelinkExtensionParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters} SitelinkExtensionParameters instance
                                 */
                                SitelinkExtensionParameters.create = function create(properties) {
                                    return new SitelinkExtensionParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified SitelinkExtensionParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ISitelinkExtensionParameters} m SitelinkExtensionParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SitelinkExtensionParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.sitelink_extensions != null && m.sitelink_extensions.length) {
                                        for (var i = 0; i < m.sitelink_extensions.length; ++i)
                                            $root.google.ads.googleads.v11.common.SitelinkFeedItem.encode(m.sitelink_extensions[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified SitelinkExtensionParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ISitelinkExtensionParameters} message SitelinkExtensionParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                SitelinkExtensionParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a SitelinkExtensionParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters} SitelinkExtensionParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SitelinkExtensionParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.sitelink_extensions && m.sitelink_extensions.length))
                                                m.sitelink_extensions = [];
                                            m.sitelink_extensions.push($root.google.ads.googleads.v11.common.SitelinkFeedItem.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a SitelinkExtensionParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters} SitelinkExtensionParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                SitelinkExtensionParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a SitelinkExtensionParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                SitelinkExtensionParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.sitelink_extensions != null && m.hasOwnProperty("sitelink_extensions")) {
                                        if (!Array.isArray(m.sitelink_extensions))
                                            return "sitelink_extensions: array expected";
                                        for (var i = 0; i < m.sitelink_extensions.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.SitelinkFeedItem.verify(m.sitelink_extensions[i]);
                                                if (e)
                                                    return "sitelink_extensions." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a SitelinkExtensionParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters} SitelinkExtensionParameters
                                 */
                                SitelinkExtensionParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters();
                                    if (d.sitelink_extensions) {
                                        if (!Array.isArray(d.sitelink_extensions))
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.sitelink_extensions: array expected");
                                        m.sitelink_extensions = [];
                                        for (var i = 0; i < d.sitelink_extensions.length; ++i) {
                                            if (typeof d.sitelink_extensions[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters.sitelink_extensions: object expected");
                                            m.sitelink_extensions[i] = $root.google.ads.googleads.v11.common.SitelinkFeedItem.fromObject(d.sitelink_extensions[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a SitelinkExtensionParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters} m SitelinkExtensionParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                SitelinkExtensionParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.sitelink_extensions = [];
                                    }
                                    if (m.sitelink_extensions && m.sitelink_extensions.length) {
                                        d.sitelink_extensions = [];
                                        for (var j = 0; j < m.sitelink_extensions.length; ++j) {
                                            d.sitelink_extensions[j] = $root.google.ads.googleads.v11.common.SitelinkFeedItem.toObject(m.sitelink_extensions[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this SitelinkExtensionParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.SitelinkExtensionParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                SitelinkExtensionParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return SitelinkExtensionParameters;
                            })();
    
                            ApplyRecommendationOperation.MoveUnusedBudgetParameters = (function() {
    
                                /**
                                 * Properties of a MoveUnusedBudgetParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface IMoveUnusedBudgetParameters
                                 * @property {number|null} [budget_micros_to_move] MoveUnusedBudgetParameters budget_micros_to_move
                                 */
    
                                /**
                                 * Constructs a new MoveUnusedBudgetParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a MoveUnusedBudgetParameters.
                                 * @implements IMoveUnusedBudgetParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IMoveUnusedBudgetParameters=} [p] Properties to set
                                 */
                                function MoveUnusedBudgetParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * MoveUnusedBudgetParameters budget_micros_to_move.
                                 * @member {number|null|undefined} budget_micros_to_move
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @instance
                                 */
                                MoveUnusedBudgetParameters.prototype.budget_micros_to_move = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * MoveUnusedBudgetParameters _budget_micros_to_move.
                                 * @member {"budget_micros_to_move"|undefined} _budget_micros_to_move
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @instance
                                 */
                                Object.defineProperty(MoveUnusedBudgetParameters.prototype, "_budget_micros_to_move", {
                                    get: $util.oneOfGetter($oneOfFields = ["budget_micros_to_move"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new MoveUnusedBudgetParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IMoveUnusedBudgetParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters} MoveUnusedBudgetParameters instance
                                 */
                                MoveUnusedBudgetParameters.create = function create(properties) {
                                    return new MoveUnusedBudgetParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified MoveUnusedBudgetParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IMoveUnusedBudgetParameters} m MoveUnusedBudgetParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MoveUnusedBudgetParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.budget_micros_to_move != null && Object.hasOwnProperty.call(m, "budget_micros_to_move"))
                                        w.uint32(16).int64(m.budget_micros_to_move);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified MoveUnusedBudgetParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IMoveUnusedBudgetParameters} message MoveUnusedBudgetParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                MoveUnusedBudgetParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a MoveUnusedBudgetParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters} MoveUnusedBudgetParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MoveUnusedBudgetParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 2:
                                            m.budget_micros_to_move = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a MoveUnusedBudgetParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters} MoveUnusedBudgetParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                MoveUnusedBudgetParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a MoveUnusedBudgetParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                MoveUnusedBudgetParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.budget_micros_to_move != null && m.hasOwnProperty("budget_micros_to_move")) {
                                        p._budget_micros_to_move = 1;
                                        if (!$util.isInteger(m.budget_micros_to_move) && !(m.budget_micros_to_move && $util.isInteger(m.budget_micros_to_move.low) && $util.isInteger(m.budget_micros_to_move.high)))
                                            return "budget_micros_to_move: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a MoveUnusedBudgetParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters} MoveUnusedBudgetParameters
                                 */
                                MoveUnusedBudgetParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters();
                                    if (d.budget_micros_to_move != null) {
                                        if ($util.Long)
                                            (m.budget_micros_to_move = $util.Long.fromValue(d.budget_micros_to_move)).unsigned = false;
                                        else if (typeof d.budget_micros_to_move === "string")
                                            m.budget_micros_to_move = parseInt(d.budget_micros_to_move, 10);
                                        else if (typeof d.budget_micros_to_move === "number")
                                            m.budget_micros_to_move = d.budget_micros_to_move;
                                        else if (typeof d.budget_micros_to_move === "object")
                                            m.budget_micros_to_move = new $util.LongBits(d.budget_micros_to_move.low >>> 0, d.budget_micros_to_move.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a MoveUnusedBudgetParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters} m MoveUnusedBudgetParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                MoveUnusedBudgetParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.budget_micros_to_move != null && m.hasOwnProperty("budget_micros_to_move")) {
                                        if (typeof m.budget_micros_to_move === "number")
                                            d.budget_micros_to_move = o.longs === String ? String(m.budget_micros_to_move) : m.budget_micros_to_move;
                                        else
                                            d.budget_micros_to_move = o.longs === String ? $util.Long.prototype.toString.call(m.budget_micros_to_move) : o.longs === Number ? new $util.LongBits(m.budget_micros_to_move.low >>> 0, m.budget_micros_to_move.high >>> 0).toNumber() : m.budget_micros_to_move;
                                        if (o.oneofs)
                                            d._budget_micros_to_move = "budget_micros_to_move";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this MoveUnusedBudgetParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.MoveUnusedBudgetParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                MoveUnusedBudgetParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return MoveUnusedBudgetParameters;
                            })();
    
                            ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters = (function() {
    
                                /**
                                 * Properties of a ResponsiveSearchAdAssetParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface IResponsiveSearchAdAssetParameters
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [updated_ad] ResponsiveSearchAdAssetParameters updated_ad
                                 */
    
                                /**
                                 * Constructs a new ResponsiveSearchAdAssetParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a ResponsiveSearchAdAssetParameters.
                                 * @implements IResponsiveSearchAdAssetParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdAssetParameters=} [p] Properties to set
                                 */
                                function ResponsiveSearchAdAssetParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ResponsiveSearchAdAssetParameters updated_ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} updated_ad
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @instance
                                 */
                                ResponsiveSearchAdAssetParameters.prototype.updated_ad = null;
    
                                /**
                                 * Creates a new ResponsiveSearchAdAssetParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdAssetParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters} ResponsiveSearchAdAssetParameters instance
                                 */
                                ResponsiveSearchAdAssetParameters.create = function create(properties) {
                                    return new ResponsiveSearchAdAssetParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdAssetParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdAssetParameters} m ResponsiveSearchAdAssetParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdAssetParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.updated_ad != null && Object.hasOwnProperty.call(m, "updated_ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.updated_ad, w.uint32(10).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdAssetParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdAssetParameters} message ResponsiveSearchAdAssetParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdAssetParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdAssetParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters} ResponsiveSearchAdAssetParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdAssetParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.updated_ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdAssetParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters} ResponsiveSearchAdAssetParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdAssetParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ResponsiveSearchAdAssetParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ResponsiveSearchAdAssetParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.updated_ad != null && m.hasOwnProperty("updated_ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.updated_ad);
                                            if (e)
                                                return "updated_ad." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ResponsiveSearchAdAssetParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters} ResponsiveSearchAdAssetParameters
                                 */
                                ResponsiveSearchAdAssetParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters();
                                    if (d.updated_ad != null) {
                                        if (typeof d.updated_ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters.updated_ad: object expected");
                                        m.updated_ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.updated_ad);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ResponsiveSearchAdAssetParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters} m ResponsiveSearchAdAssetParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ResponsiveSearchAdAssetParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.updated_ad = null;
                                    }
                                    if (m.updated_ad != null && m.hasOwnProperty("updated_ad")) {
                                        d.updated_ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.updated_ad, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ResponsiveSearchAdAssetParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ResponsiveSearchAdAssetParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ResponsiveSearchAdAssetParameters;
                            })();
    
                            ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters = (function() {
    
                                /**
                                 * Properties of a ResponsiveSearchAdImproveAdStrengthParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface IResponsiveSearchAdImproveAdStrengthParameters
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [updated_ad] ResponsiveSearchAdImproveAdStrengthParameters updated_ad
                                 */
    
                                /**
                                 * Constructs a new ResponsiveSearchAdImproveAdStrengthParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a ResponsiveSearchAdImproveAdStrengthParameters.
                                 * @implements IResponsiveSearchAdImproveAdStrengthParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdImproveAdStrengthParameters=} [p] Properties to set
                                 */
                                function ResponsiveSearchAdImproveAdStrengthParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ResponsiveSearchAdImproveAdStrengthParameters updated_ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} updated_ad
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @instance
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.prototype.updated_ad = null;
    
                                /**
                                 * Creates a new ResponsiveSearchAdImproveAdStrengthParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdImproveAdStrengthParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters} ResponsiveSearchAdImproveAdStrengthParameters instance
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.create = function create(properties) {
                                    return new ResponsiveSearchAdImproveAdStrengthParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdImproveAdStrengthParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdImproveAdStrengthParameters} m ResponsiveSearchAdImproveAdStrengthParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.updated_ad != null && Object.hasOwnProperty.call(m, "updated_ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.updated_ad, w.uint32(10).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdImproveAdStrengthParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdImproveAdStrengthParameters} message ResponsiveSearchAdImproveAdStrengthParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdImproveAdStrengthParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters} ResponsiveSearchAdImproveAdStrengthParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.updated_ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdImproveAdStrengthParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters} ResponsiveSearchAdImproveAdStrengthParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ResponsiveSearchAdImproveAdStrengthParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.updated_ad != null && m.hasOwnProperty("updated_ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.updated_ad);
                                            if (e)
                                                return "updated_ad." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ResponsiveSearchAdImproveAdStrengthParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters} ResponsiveSearchAdImproveAdStrengthParameters
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters();
                                    if (d.updated_ad != null) {
                                        if (typeof d.updated_ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters.updated_ad: object expected");
                                        m.updated_ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.updated_ad);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ResponsiveSearchAdImproveAdStrengthParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters} m ResponsiveSearchAdImproveAdStrengthParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.updated_ad = null;
                                    }
                                    if (m.updated_ad != null && m.hasOwnProperty("updated_ad")) {
                                        d.updated_ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.updated_ad, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ResponsiveSearchAdImproveAdStrengthParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ResponsiveSearchAdImproveAdStrengthParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ResponsiveSearchAdImproveAdStrengthParameters;
                            })();
    
                            ApplyRecommendationOperation.ResponsiveSearchAdParameters = (function() {
    
                                /**
                                 * Properties of a ResponsiveSearchAdParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface IResponsiveSearchAdParameters
                                 * @property {google.ads.googleads.v11.resources.IAd|null} [ad] ResponsiveSearchAdParameters ad
                                 */
    
                                /**
                                 * Constructs a new ResponsiveSearchAdParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a ResponsiveSearchAdParameters.
                                 * @implements IResponsiveSearchAdParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdParameters=} [p] Properties to set
                                 */
                                function ResponsiveSearchAdParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * ResponsiveSearchAdParameters ad.
                                 * @member {google.ads.googleads.v11.resources.IAd|null|undefined} ad
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @instance
                                 */
                                ResponsiveSearchAdParameters.prototype.ad = null;
    
                                /**
                                 * Creates a new ResponsiveSearchAdParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters} ResponsiveSearchAdParameters instance
                                 */
                                ResponsiveSearchAdParameters.create = function create(properties) {
                                    return new ResponsiveSearchAdParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdParameters} m ResponsiveSearchAdParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.ad != null && Object.hasOwnProperty.call(m, "ad"))
                                        $root.google.ads.googleads.v11.resources.Ad.encode(m.ad, w.uint32(10).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified ResponsiveSearchAdParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IResponsiveSearchAdParameters} message ResponsiveSearchAdParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                ResponsiveSearchAdParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters} ResponsiveSearchAdParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.ad = $root.google.ads.googleads.v11.resources.Ad.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a ResponsiveSearchAdParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters} ResponsiveSearchAdParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                ResponsiveSearchAdParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a ResponsiveSearchAdParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                ResponsiveSearchAdParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.Ad.verify(m.ad);
                                            if (e)
                                                return "ad." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a ResponsiveSearchAdParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters} ResponsiveSearchAdParameters
                                 */
                                ResponsiveSearchAdParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters();
                                    if (d.ad != null) {
                                        if (typeof d.ad !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters.ad: object expected");
                                        m.ad = $root.google.ads.googleads.v11.resources.Ad.fromObject(d.ad);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a ResponsiveSearchAdParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters} m ResponsiveSearchAdParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                ResponsiveSearchAdParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.ad = null;
                                    }
                                    if (m.ad != null && m.hasOwnProperty("ad")) {
                                        d.ad = $root.google.ads.googleads.v11.resources.Ad.toObject(m.ad, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this ResponsiveSearchAdParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.ResponsiveSearchAdParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                ResponsiveSearchAdParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return ResponsiveSearchAdParameters;
                            })();
    
                            ApplyRecommendationOperation.UseBroadMatchKeywordParameters = (function() {
    
                                /**
                                 * Properties of a UseBroadMatchKeywordParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @interface IUseBroadMatchKeywordParameters
                                 * @property {number|null} [new_budget_amount_micros] UseBroadMatchKeywordParameters new_budget_amount_micros
                                 */
    
                                /**
                                 * Constructs a new UseBroadMatchKeywordParameters.
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation
                                 * @classdesc Represents a UseBroadMatchKeywordParameters.
                                 * @implements IUseBroadMatchKeywordParameters
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IUseBroadMatchKeywordParameters=} [p] Properties to set
                                 */
                                function UseBroadMatchKeywordParameters(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * UseBroadMatchKeywordParameters new_budget_amount_micros.
                                 * @member {number|null|undefined} new_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @instance
                                 */
                                UseBroadMatchKeywordParameters.prototype.new_budget_amount_micros = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * UseBroadMatchKeywordParameters _new_budget_amount_micros.
                                 * @member {"new_budget_amount_micros"|undefined} _new_budget_amount_micros
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @instance
                                 */
                                Object.defineProperty(UseBroadMatchKeywordParameters.prototype, "_new_budget_amount_micros", {
                                    get: $util.oneOfGetter($oneOfFields = ["new_budget_amount_micros"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new UseBroadMatchKeywordParameters instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IUseBroadMatchKeywordParameters=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters} UseBroadMatchKeywordParameters instance
                                 */
                                UseBroadMatchKeywordParameters.create = function create(properties) {
                                    return new UseBroadMatchKeywordParameters(properties);
                                };
    
                                /**
                                 * Encodes the specified UseBroadMatchKeywordParameters message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IUseBroadMatchKeywordParameters} m UseBroadMatchKeywordParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                UseBroadMatchKeywordParameters.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.new_budget_amount_micros != null && Object.hasOwnProperty.call(m, "new_budget_amount_micros"))
                                        w.uint32(8).int64(m.new_budget_amount_micros);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified UseBroadMatchKeywordParameters message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.IUseBroadMatchKeywordParameters} message UseBroadMatchKeywordParameters message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                UseBroadMatchKeywordParameters.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a UseBroadMatchKeywordParameters message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters} UseBroadMatchKeywordParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                UseBroadMatchKeywordParameters.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.new_budget_amount_micros = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a UseBroadMatchKeywordParameters message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters} UseBroadMatchKeywordParameters
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                UseBroadMatchKeywordParameters.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a UseBroadMatchKeywordParameters message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                UseBroadMatchKeywordParameters.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    var p = {};
                                    if (m.new_budget_amount_micros != null && m.hasOwnProperty("new_budget_amount_micros")) {
                                        p._new_budget_amount_micros = 1;
                                        if (!$util.isInteger(m.new_budget_amount_micros) && !(m.new_budget_amount_micros && $util.isInteger(m.new_budget_amount_micros.low) && $util.isInteger(m.new_budget_amount_micros.high)))
                                            return "new_budget_amount_micros: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a UseBroadMatchKeywordParameters message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters} UseBroadMatchKeywordParameters
                                 */
                                UseBroadMatchKeywordParameters.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters();
                                    if (d.new_budget_amount_micros != null) {
                                        if ($util.Long)
                                            (m.new_budget_amount_micros = $util.Long.fromValue(d.new_budget_amount_micros)).unsigned = false;
                                        else if (typeof d.new_budget_amount_micros === "string")
                                            m.new_budget_amount_micros = parseInt(d.new_budget_amount_micros, 10);
                                        else if (typeof d.new_budget_amount_micros === "number")
                                            m.new_budget_amount_micros = d.new_budget_amount_micros;
                                        else if (typeof d.new_budget_amount_micros === "object")
                                            m.new_budget_amount_micros = new $util.LongBits(d.new_budget_amount_micros.low >>> 0, d.new_budget_amount_micros.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a UseBroadMatchKeywordParameters message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @static
                                 * @param {google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters} m UseBroadMatchKeywordParameters
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                UseBroadMatchKeywordParameters.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (m.new_budget_amount_micros != null && m.hasOwnProperty("new_budget_amount_micros")) {
                                        if (typeof m.new_budget_amount_micros === "number")
                                            d.new_budget_amount_micros = o.longs === String ? String(m.new_budget_amount_micros) : m.new_budget_amount_micros;
                                        else
                                            d.new_budget_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.new_budget_amount_micros) : o.longs === Number ? new $util.LongBits(m.new_budget_amount_micros.low >>> 0, m.new_budget_amount_micros.high >>> 0).toNumber() : m.new_budget_amount_micros;
                                        if (o.oneofs)
                                            d._new_budget_amount_micros = "new_budget_amount_micros";
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this UseBroadMatchKeywordParameters to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.ApplyRecommendationOperation.UseBroadMatchKeywordParameters
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                UseBroadMatchKeywordParameters.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return UseBroadMatchKeywordParameters;
                            })();
    
                            return ApplyRecommendationOperation;
                        })();
    
                        services.ApplyRecommendationResponse = (function() {
    
                            /**
                             * Properties of an ApplyRecommendationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IApplyRecommendationResponse
                             * @property {Array.<google.ads.googleads.v11.services.IApplyRecommendationResult>|null} [results] ApplyRecommendationResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] ApplyRecommendationResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new ApplyRecommendationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ApplyRecommendationResponse.
                             * @implements IApplyRecommendationResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResponse=} [p] Properties to set
                             */
                            function ApplyRecommendationResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ApplyRecommendationResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.IApplyRecommendationResult>} results
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @instance
                             */
                            ApplyRecommendationResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * ApplyRecommendationResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @instance
                             */
                            ApplyRecommendationResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new ApplyRecommendationResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResponse} ApplyRecommendationResponse instance
                             */
                            ApplyRecommendationResponse.create = function create(properties) {
                                return new ApplyRecommendationResponse(properties);
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationResponse message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResponse} m ApplyRecommendationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.ApplyRecommendationResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResponse} message ApplyRecommendationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ApplyRecommendationResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResponse} ApplyRecommendationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.ApplyRecommendationResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ApplyRecommendationResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResponse} ApplyRecommendationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ApplyRecommendationResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ApplyRecommendationResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.ApplyRecommendationResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ApplyRecommendationResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResponse} ApplyRecommendationResponse
                             */
                            ApplyRecommendationResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.ApplyRecommendationResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.ApplyRecommendationResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ApplyRecommendationResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ApplyRecommendationResponse} m ApplyRecommendationResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ApplyRecommendationResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.ApplyRecommendationResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ApplyRecommendationResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ApplyRecommendationResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ApplyRecommendationResponse;
                        })();
    
                        services.ApplyRecommendationResult = (function() {
    
                            /**
                             * Properties of an ApplyRecommendationResult.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IApplyRecommendationResult
                             * @property {string|null} [resource_name] ApplyRecommendationResult resource_name
                             */
    
                            /**
                             * Constructs a new ApplyRecommendationResult.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an ApplyRecommendationResult.
                             * @implements IApplyRecommendationResult
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResult=} [p] Properties to set
                             */
                            function ApplyRecommendationResult(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * ApplyRecommendationResult resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @instance
                             */
                            ApplyRecommendationResult.prototype.resource_name = "";
    
                            /**
                             * Creates a new ApplyRecommendationResult instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResult=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResult} ApplyRecommendationResult instance
                             */
                            ApplyRecommendationResult.create = function create(properties) {
                                return new ApplyRecommendationResult(properties);
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationResult message. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationResult.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResult} m ApplyRecommendationResult message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationResult.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified ApplyRecommendationResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.ApplyRecommendationResult.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.IApplyRecommendationResult} message ApplyRecommendationResult message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ApplyRecommendationResult.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an ApplyRecommendationResult message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResult} ApplyRecommendationResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationResult.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.ApplyRecommendationResult();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an ApplyRecommendationResult message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResult} ApplyRecommendationResult
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ApplyRecommendationResult.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an ApplyRecommendationResult message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ApplyRecommendationResult.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an ApplyRecommendationResult message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.ApplyRecommendationResult} ApplyRecommendationResult
                             */
                            ApplyRecommendationResult.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.ApplyRecommendationResult)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.ApplyRecommendationResult();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an ApplyRecommendationResult message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @static
                             * @param {google.ads.googleads.v11.services.ApplyRecommendationResult} m ApplyRecommendationResult
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ApplyRecommendationResult.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this ApplyRecommendationResult to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.ApplyRecommendationResult
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ApplyRecommendationResult.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return ApplyRecommendationResult;
                        })();
    
                        services.DismissRecommendationRequest = (function() {
    
                            /**
                             * Properties of a DismissRecommendationRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IDismissRecommendationRequest
                             * @property {string|null} [customer_id] DismissRecommendationRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.DismissRecommendationRequest.IDismissRecommendationOperation>|null} [operations] DismissRecommendationRequest operations
                             * @property {boolean|null} [partial_failure] DismissRecommendationRequest partial_failure
                             */
    
                            /**
                             * Constructs a new DismissRecommendationRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a DismissRecommendationRequest.
                             * @implements IDismissRecommendationRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationRequest=} [p] Properties to set
                             */
                            function DismissRecommendationRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DismissRecommendationRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @instance
                             */
                            DismissRecommendationRequest.prototype.customer_id = "";
    
                            /**
                             * DismissRecommendationRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.DismissRecommendationRequest.IDismissRecommendationOperation>} operations
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @instance
                             */
                            DismissRecommendationRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * DismissRecommendationRequest partial_failure.
                             * @member {boolean} partial_failure
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @instance
                             */
                            DismissRecommendationRequest.prototype.partial_failure = false;
    
                            /**
                             * Creates a new DismissRecommendationRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest} DismissRecommendationRequest instance
                             */
                            DismissRecommendationRequest.create = function create(properties) {
                                return new DismissRecommendationRequest(properties);
                            };
    
                            /**
                             * Encodes the specified DismissRecommendationRequest message. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationRequest} m DismissRecommendationRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DismissRecommendationRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.partial_failure != null && Object.hasOwnProperty.call(m, "partial_failure"))
                                    w.uint32(16).bool(m.partial_failure);
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation.encode(m.operations[i], w.uint32(26).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified DismissRecommendationRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationRequest} message DismissRecommendationRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DismissRecommendationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DismissRecommendationRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest} DismissRecommendationRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DismissRecommendationRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.DismissRecommendationRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 3:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure = r.bool();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DismissRecommendationRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest} DismissRecommendationRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DismissRecommendationRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DismissRecommendationRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DismissRecommendationRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    if (typeof m.partial_failure !== "boolean")
                                        return "partial_failure: boolean expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DismissRecommendationRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest} DismissRecommendationRequest
                             */
                            DismissRecommendationRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.DismissRecommendationRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.DismissRecommendationRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.DismissRecommendationRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.DismissRecommendationRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.partial_failure != null) {
                                    m.partial_failure = Boolean(d.partial_failure);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DismissRecommendationRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.DismissRecommendationRequest} m DismissRecommendationRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DismissRecommendationRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.partial_failure = false;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.partial_failure != null && m.hasOwnProperty("partial_failure")) {
                                    d.partial_failure = m.partial_failure;
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation.toObject(m.operations[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DismissRecommendationRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DismissRecommendationRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            DismissRecommendationRequest.DismissRecommendationOperation = (function() {
    
                                /**
                                 * Properties of a DismissRecommendationOperation.
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                                 * @interface IDismissRecommendationOperation
                                 * @property {string|null} [resource_name] DismissRecommendationOperation resource_name
                                 */
    
                                /**
                                 * Constructs a new DismissRecommendationOperation.
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest
                                 * @classdesc Represents a DismissRecommendationOperation.
                                 * @implements IDismissRecommendationOperation
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationRequest.IDismissRecommendationOperation=} [p] Properties to set
                                 */
                                function DismissRecommendationOperation(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * DismissRecommendationOperation resource_name.
                                 * @member {string} resource_name
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @instance
                                 */
                                DismissRecommendationOperation.prototype.resource_name = "";
    
                                /**
                                 * Creates a new DismissRecommendationOperation instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationRequest.IDismissRecommendationOperation=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation} DismissRecommendationOperation instance
                                 */
                                DismissRecommendationOperation.create = function create(properties) {
                                    return new DismissRecommendationOperation(properties);
                                };
    
                                /**
                                 * Encodes the specified DismissRecommendationOperation message. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationRequest.IDismissRecommendationOperation} m DismissRecommendationOperation message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DismissRecommendationOperation.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                        w.uint32(10).string(m.resource_name);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified DismissRecommendationOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationRequest.IDismissRecommendationOperation} message DismissRecommendationOperation message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DismissRecommendationOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a DismissRecommendationOperation message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation} DismissRecommendationOperation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DismissRecommendationOperation.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.resource_name = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a DismissRecommendationOperation message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation} DismissRecommendationOperation
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DismissRecommendationOperation.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a DismissRecommendationOperation message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                DismissRecommendationOperation.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                        if (!$util.isString(m.resource_name))
                                            return "resource_name: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a DismissRecommendationOperation message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation} DismissRecommendationOperation
                                 */
                                DismissRecommendationOperation.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation();
                                    if (d.resource_name != null) {
                                        m.resource_name = String(d.resource_name);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a DismissRecommendationOperation message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation} m DismissRecommendationOperation
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                DismissRecommendationOperation.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.resource_name = "";
                                    }
                                    if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                        d.resource_name = m.resource_name;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this DismissRecommendationOperation to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationRequest.DismissRecommendationOperation
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                DismissRecommendationOperation.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return DismissRecommendationOperation;
                            })();
    
                            return DismissRecommendationRequest;
                        })();
    
                        services.DismissRecommendationResponse = (function() {
    
                            /**
                             * Properties of a DismissRecommendationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IDismissRecommendationResponse
                             * @property {Array.<google.ads.googleads.v11.services.DismissRecommendationResponse.IDismissRecommendationResult>|null} [results] DismissRecommendationResponse results
                             * @property {google.rpc.IStatus|null} [partial_failure_error] DismissRecommendationResponse partial_failure_error
                             */
    
                            /**
                             * Constructs a new DismissRecommendationResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a DismissRecommendationResponse.
                             * @implements IDismissRecommendationResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationResponse=} [p] Properties to set
                             */
                            function DismissRecommendationResponse(p) {
                                this.results = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * DismissRecommendationResponse results.
                             * @member {Array.<google.ads.googleads.v11.services.DismissRecommendationResponse.IDismissRecommendationResult>} results
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @instance
                             */
                            DismissRecommendationResponse.prototype.results = $util.emptyArray;
    
                            /**
                             * DismissRecommendationResponse partial_failure_error.
                             * @member {google.rpc.IStatus|null|undefined} partial_failure_error
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @instance
                             */
                            DismissRecommendationResponse.prototype.partial_failure_error = null;
    
                            /**
                             * Creates a new DismissRecommendationResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse} DismissRecommendationResponse instance
                             */
                            DismissRecommendationResponse.create = function create(properties) {
                                return new DismissRecommendationResponse(properties);
                            };
    
                            /**
                             * Encodes the specified DismissRecommendationResponse message. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationResponse} m DismissRecommendationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DismissRecommendationResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.results != null && m.results.length) {
                                    for (var i = 0; i < m.results.length; ++i)
                                        $root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult.encode(m.results[i], w.uint32(10).fork()).ldelim();
                                }
                                if (m.partial_failure_error != null && Object.hasOwnProperty.call(m, "partial_failure_error"))
                                    $root.google.rpc.Status.encode(m.partial_failure_error, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified DismissRecommendationResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IDismissRecommendationResponse} message DismissRecommendationResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DismissRecommendationResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DismissRecommendationResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse} DismissRecommendationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DismissRecommendationResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.DismissRecommendationResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.results && m.results.length))
                                            m.results = [];
                                        m.results.push($root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.partial_failure_error = $root.google.rpc.Status.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a DismissRecommendationResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse} DismissRecommendationResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DismissRecommendationResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DismissRecommendationResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DismissRecommendationResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.results != null && m.hasOwnProperty("results")) {
                                    if (!Array.isArray(m.results))
                                        return "results: array expected";
                                    for (var i = 0; i < m.results.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult.verify(m.results[i]);
                                            if (e)
                                                return "results." + e;
                                        }
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    {
                                        var e = $root.google.rpc.Status.verify(m.partial_failure_error);
                                        if (e)
                                            return "partial_failure_error." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DismissRecommendationResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse} DismissRecommendationResponse
                             */
                            DismissRecommendationResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.DismissRecommendationResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.DismissRecommendationResponse();
                                if (d.results) {
                                    if (!Array.isArray(d.results))
                                        throw TypeError(".google.ads.googleads.v11.services.DismissRecommendationResponse.results: array expected");
                                    m.results = [];
                                    for (var i = 0; i < d.results.length; ++i) {
                                        if (typeof d.results[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.DismissRecommendationResponse.results: object expected");
                                        m.results[i] = $root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult.fromObject(d.results[i]);
                                    }
                                }
                                if (d.partial_failure_error != null) {
                                    if (typeof d.partial_failure_error !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.DismissRecommendationResponse.partial_failure_error: object expected");
                                    m.partial_failure_error = $root.google.rpc.Status.fromObject(d.partial_failure_error);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a DismissRecommendationResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.DismissRecommendationResponse} m DismissRecommendationResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DismissRecommendationResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.results = [];
                                }
                                if (o.defaults) {
                                    d.partial_failure_error = null;
                                }
                                if (m.results && m.results.length) {
                                    d.results = [];
                                    for (var j = 0; j < m.results.length; ++j) {
                                        d.results[j] = $root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult.toObject(m.results[j], o);
                                    }
                                }
                                if (m.partial_failure_error != null && m.hasOwnProperty("partial_failure_error")) {
                                    d.partial_failure_error = $root.google.rpc.Status.toObject(m.partial_failure_error, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this DismissRecommendationResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DismissRecommendationResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            DismissRecommendationResponse.DismissRecommendationResult = (function() {
    
                                /**
                                 * Properties of a DismissRecommendationResult.
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                                 * @interface IDismissRecommendationResult
                                 * @property {string|null} [resource_name] DismissRecommendationResult resource_name
                                 */
    
                                /**
                                 * Constructs a new DismissRecommendationResult.
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse
                                 * @classdesc Represents a DismissRecommendationResult.
                                 * @implements IDismissRecommendationResult
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationResponse.IDismissRecommendationResult=} [p] Properties to set
                                 */
                                function DismissRecommendationResult(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * DismissRecommendationResult resource_name.
                                 * @member {string} resource_name
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @instance
                                 */
                                DismissRecommendationResult.prototype.resource_name = "";
    
                                /**
                                 * Creates a new DismissRecommendationResult instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationResponse.IDismissRecommendationResult=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult} DismissRecommendationResult instance
                                 */
                                DismissRecommendationResult.create = function create(properties) {
                                    return new DismissRecommendationResult(properties);
                                };
    
                                /**
                                 * Encodes the specified DismissRecommendationResult message. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationResponse.IDismissRecommendationResult} m DismissRecommendationResult message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DismissRecommendationResult.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                        w.uint32(10).string(m.resource_name);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified DismissRecommendationResult message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationResponse.IDismissRecommendationResult} message DismissRecommendationResult message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                DismissRecommendationResult.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a DismissRecommendationResult message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult} DismissRecommendationResult
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DismissRecommendationResult.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.resource_name = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a DismissRecommendationResult message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult} DismissRecommendationResult
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                DismissRecommendationResult.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a DismissRecommendationResult message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                DismissRecommendationResult.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                        if (!$util.isString(m.resource_name))
                                            return "resource_name: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a DismissRecommendationResult message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult} DismissRecommendationResult
                                 */
                                DismissRecommendationResult.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult();
                                    if (d.resource_name != null) {
                                        m.resource_name = String(d.resource_name);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a DismissRecommendationResult message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @static
                                 * @param {google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult} m DismissRecommendationResult
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                DismissRecommendationResult.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.resource_name = "";
                                    }
                                    if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                        d.resource_name = m.resource_name;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this DismissRecommendationResult to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.DismissRecommendationResponse.DismissRecommendationResult
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                DismissRecommendationResult.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return DismissRecommendationResult;
                            })();
    
                            return DismissRecommendationResponse;
                        })();
    
                        services.SmartCampaignSuggestService = (function() {
    
                            /**
                             * Constructs a new SmartCampaignSuggestService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SmartCampaignSuggestService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function SmartCampaignSuggestService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (SmartCampaignSuggestService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = SmartCampaignSuggestService;
    
                            /**
                             * Creates new SmartCampaignSuggestService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {SmartCampaignSuggestService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            SmartCampaignSuggestService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.SmartCampaignSuggestService|suggestSmartCampaignBudgetOptions}.
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @typedef SuggestSmartCampaignBudgetOptionsCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse} [response] SuggestSmartCampaignBudgetOptionsResponse
                             */
    
                            /**
                             * Calls SuggestSmartCampaignBudgetOptions.
                             * @function suggestSmartCampaignBudgetOptions
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsRequest} request SuggestSmartCampaignBudgetOptionsRequest message or plain object
                             * @param {google.ads.googleads.v11.services.SmartCampaignSuggestService.SuggestSmartCampaignBudgetOptionsCallback} callback Node-style callback called with the error, if any, and SuggestSmartCampaignBudgetOptionsResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(SmartCampaignSuggestService.prototype.suggestSmartCampaignBudgetOptions = function suggestSmartCampaignBudgetOptions(request, callback) {
                                return this.rpcCall(suggestSmartCampaignBudgetOptions, $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest, $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse, request, callback);
                            }, "name", { value: "SuggestSmartCampaignBudgetOptions" });
    
                            /**
                             * Calls SuggestSmartCampaignBudgetOptions.
                             * @function suggestSmartCampaignBudgetOptions
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsRequest} request SuggestSmartCampaignBudgetOptionsRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.SmartCampaignSuggestService|suggestSmartCampaignAd}.
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @typedef SuggestSmartCampaignAdCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse} [response] SuggestSmartCampaignAdResponse
                             */
    
                            /**
                             * Calls SuggestSmartCampaignAd.
                             * @function suggestSmartCampaignAd
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdRequest} request SuggestSmartCampaignAdRequest message or plain object
                             * @param {google.ads.googleads.v11.services.SmartCampaignSuggestService.SuggestSmartCampaignAdCallback} callback Node-style callback called with the error, if any, and SuggestSmartCampaignAdResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(SmartCampaignSuggestService.prototype.suggestSmartCampaignAd = function suggestSmartCampaignAd(request, callback) {
                                return this.rpcCall(suggestSmartCampaignAd, $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest, $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse, request, callback);
                            }, "name", { value: "SuggestSmartCampaignAd" });
    
                            /**
                             * Calls SuggestSmartCampaignAd.
                             * @function suggestSmartCampaignAd
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdRequest} request SuggestSmartCampaignAdRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse>} Promise
                             * @variation 2
                             */
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.SmartCampaignSuggestService|suggestKeywordThemes}.
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @typedef SuggestKeywordThemesCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.SuggestKeywordThemesResponse} [response] SuggestKeywordThemesResponse
                             */
    
                            /**
                             * Calls SuggestKeywordThemes.
                             * @function suggestKeywordThemes
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesRequest} request SuggestKeywordThemesRequest message or plain object
                             * @param {google.ads.googleads.v11.services.SmartCampaignSuggestService.SuggestKeywordThemesCallback} callback Node-style callback called with the error, if any, and SuggestKeywordThemesResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(SmartCampaignSuggestService.prototype.suggestKeywordThemes = function suggestKeywordThemes(request, callback) {
                                return this.rpcCall(suggestKeywordThemes, $root.google.ads.googleads.v11.services.SuggestKeywordThemesRequest, $root.google.ads.googleads.v11.services.SuggestKeywordThemesResponse, request, callback);
                            }, "name", { value: "SuggestKeywordThemes" });
    
                            /**
                             * Calls SuggestKeywordThemes.
                             * @function suggestKeywordThemes
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestService
                             * @instance
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesRequest} request SuggestKeywordThemesRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.SuggestKeywordThemesResponse>} Promise
                             * @variation 2
                             */
    
                            return SmartCampaignSuggestService;
                        })();
    
                        services.SuggestSmartCampaignBudgetOptionsRequest = (function() {
    
                            /**
                             * Properties of a SuggestSmartCampaignBudgetOptionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestSmartCampaignBudgetOptionsRequest
                             * @property {string|null} [customer_id] SuggestSmartCampaignBudgetOptionsRequest customer_id
                             * @property {string|null} [campaign] SuggestSmartCampaignBudgetOptionsRequest campaign
                             * @property {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo|null} [suggestion_info] SuggestSmartCampaignBudgetOptionsRequest suggestion_info
                             */
    
                            /**
                             * Constructs a new SuggestSmartCampaignBudgetOptionsRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestSmartCampaignBudgetOptionsRequest.
                             * @implements ISuggestSmartCampaignBudgetOptionsRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsRequest=} [p] Properties to set
                             */
                            function SuggestSmartCampaignBudgetOptionsRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @instance
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.prototype.customer_id = "";
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsRequest campaign.
                             * @member {string|null|undefined} campaign
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @instance
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.prototype.campaign = null;
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsRequest suggestion_info.
                             * @member {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo|null|undefined} suggestion_info
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @instance
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.prototype.suggestion_info = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsRequest suggestion_data.
                             * @member {"campaign"|"suggestion_info"|undefined} suggestion_data
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @instance
                             */
                            Object.defineProperty(SuggestSmartCampaignBudgetOptionsRequest.prototype, "suggestion_data", {
                                get: $util.oneOfGetter($oneOfFields = ["campaign", "suggestion_info"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SuggestSmartCampaignBudgetOptionsRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest} SuggestSmartCampaignBudgetOptionsRequest instance
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.create = function create(properties) {
                                return new SuggestSmartCampaignBudgetOptionsRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignBudgetOptionsRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsRequest} m SuggestSmartCampaignBudgetOptionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.campaign != null && Object.hasOwnProperty.call(m, "campaign"))
                                    w.uint32(18).string(m.campaign);
                                if (m.suggestion_info != null && Object.hasOwnProperty.call(m, "suggestion_info"))
                                    $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.encode(m.suggestion_info, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignBudgetOptionsRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsRequest} message SuggestSmartCampaignBudgetOptionsRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignBudgetOptionsRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest} SuggestSmartCampaignBudgetOptionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.campaign = r.string();
                                        break;
                                    case 3:
                                        m.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignBudgetOptionsRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest} SuggestSmartCampaignBudgetOptionsRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestSmartCampaignBudgetOptionsRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    p.suggestion_data = 1;
                                    if (!$util.isString(m.campaign))
                                        return "campaign: string expected";
                                }
                                if (m.suggestion_info != null && m.hasOwnProperty("suggestion_info")) {
                                    if (p.suggestion_data === 1)
                                        return "suggestion_data: multiple values";
                                    p.suggestion_data = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.verify(m.suggestion_info);
                                        if (e)
                                            return "suggestion_info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestSmartCampaignBudgetOptionsRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest} SuggestSmartCampaignBudgetOptionsRequest
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.campaign != null) {
                                    m.campaign = String(d.campaign);
                                }
                                if (d.suggestion_info != null) {
                                    if (typeof d.suggestion_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest.suggestion_info: object expected");
                                    m.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.fromObject(d.suggestion_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestSmartCampaignBudgetOptionsRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest} m SuggestSmartCampaignBudgetOptionsRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.campaign != null && m.hasOwnProperty("campaign")) {
                                    d.campaign = m.campaign;
                                    if (o.oneofs)
                                        d.suggestion_data = "campaign";
                                }
                                if (m.suggestion_info != null && m.hasOwnProperty("suggestion_info")) {
                                    d.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.toObject(m.suggestion_info, o);
                                    if (o.oneofs)
                                        d.suggestion_data = "suggestion_info";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestSmartCampaignBudgetOptionsRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestSmartCampaignBudgetOptionsRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestSmartCampaignBudgetOptionsRequest;
                        })();
    
                        services.SmartCampaignSuggestionInfo = (function() {
    
                            /**
                             * Properties of a SmartCampaignSuggestionInfo.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISmartCampaignSuggestionInfo
                             * @property {string|null} [final_url] SmartCampaignSuggestionInfo final_url
                             * @property {string|null} [language_code] SmartCampaignSuggestionInfo language_code
                             * @property {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>|null} [ad_schedules] SmartCampaignSuggestionInfo ad_schedules
                             * @property {Array.<google.ads.googleads.v11.common.IKeywordThemeInfo>|null} [keyword_themes] SmartCampaignSuggestionInfo keyword_themes
                             * @property {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.IBusinessContext|null} [business_context] SmartCampaignSuggestionInfo business_context
                             * @property {string|null} [business_profile_location] SmartCampaignSuggestionInfo business_profile_location
                             * @property {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ILocationList|null} [location_list] SmartCampaignSuggestionInfo location_list
                             * @property {google.ads.googleads.v11.common.IProximityInfo|null} [proximity] SmartCampaignSuggestionInfo proximity
                             */
    
                            /**
                             * Constructs a new SmartCampaignSuggestionInfo.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SmartCampaignSuggestionInfo.
                             * @implements ISmartCampaignSuggestionInfo
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo=} [p] Properties to set
                             */
                            function SmartCampaignSuggestionInfo(p) {
                                this.ad_schedules = [];
                                this.keyword_themes = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SmartCampaignSuggestionInfo final_url.
                             * @member {string} final_url
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.final_url = "";
    
                            /**
                             * SmartCampaignSuggestionInfo language_code.
                             * @member {string} language_code
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.language_code = "";
    
                            /**
                             * SmartCampaignSuggestionInfo ad_schedules.
                             * @member {Array.<google.ads.googleads.v11.common.IAdScheduleInfo>} ad_schedules
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.ad_schedules = $util.emptyArray;
    
                            /**
                             * SmartCampaignSuggestionInfo keyword_themes.
                             * @member {Array.<google.ads.googleads.v11.common.IKeywordThemeInfo>} keyword_themes
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.keyword_themes = $util.emptyArray;
    
                            /**
                             * SmartCampaignSuggestionInfo business_context.
                             * @member {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.IBusinessContext|null|undefined} business_context
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.business_context = null;
    
                            /**
                             * SmartCampaignSuggestionInfo business_profile_location.
                             * @member {string|null|undefined} business_profile_location
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.business_profile_location = null;
    
                            /**
                             * SmartCampaignSuggestionInfo location_list.
                             * @member {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ILocationList|null|undefined} location_list
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.location_list = null;
    
                            /**
                             * SmartCampaignSuggestionInfo proximity.
                             * @member {google.ads.googleads.v11.common.IProximityInfo|null|undefined} proximity
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            SmartCampaignSuggestionInfo.prototype.proximity = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SmartCampaignSuggestionInfo business_setting.
                             * @member {"business_context"|"business_profile_location"|undefined} business_setting
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            Object.defineProperty(SmartCampaignSuggestionInfo.prototype, "business_setting", {
                                get: $util.oneOfGetter($oneOfFields = ["business_context", "business_profile_location"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SmartCampaignSuggestionInfo geo_target.
                             * @member {"location_list"|"proximity"|undefined} geo_target
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             */
                            Object.defineProperty(SmartCampaignSuggestionInfo.prototype, "geo_target", {
                                get: $util.oneOfGetter($oneOfFields = ["location_list", "proximity"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SmartCampaignSuggestionInfo instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo} SmartCampaignSuggestionInfo instance
                             */
                            SmartCampaignSuggestionInfo.create = function create(properties) {
                                return new SmartCampaignSuggestionInfo(properties);
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSuggestionInfo message. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo} m SmartCampaignSuggestionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSuggestionInfo.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.final_url != null && Object.hasOwnProperty.call(m, "final_url"))
                                    w.uint32(10).string(m.final_url);
                                if (m.language_code != null && Object.hasOwnProperty.call(m, "language_code"))
                                    w.uint32(26).string(m.language_code);
                                if (m.location_list != null && Object.hasOwnProperty.call(m, "location_list"))
                                    $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.encode(m.location_list, w.uint32(34).fork()).ldelim();
                                if (m.proximity != null && Object.hasOwnProperty.call(m, "proximity"))
                                    $root.google.ads.googleads.v11.common.ProximityInfo.encode(m.proximity, w.uint32(42).fork()).ldelim();
                                if (m.ad_schedules != null && m.ad_schedules.length) {
                                    for (var i = 0; i < m.ad_schedules.length; ++i)
                                        $root.google.ads.googleads.v11.common.AdScheduleInfo.encode(m.ad_schedules[i], w.uint32(50).fork()).ldelim();
                                }
                                if (m.keyword_themes != null && m.keyword_themes.length) {
                                    for (var i = 0; i < m.keyword_themes.length; ++i)
                                        $root.google.ads.googleads.v11.common.KeywordThemeInfo.encode(m.keyword_themes[i], w.uint32(58).fork()).ldelim();
                                }
                                if (m.business_context != null && Object.hasOwnProperty.call(m, "business_context"))
                                    $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext.encode(m.business_context, w.uint32(66).fork()).ldelim();
                                if (m.business_profile_location != null && Object.hasOwnProperty.call(m, "business_profile_location"))
                                    w.uint32(74).string(m.business_profile_location);
                                return w;
                            };
    
                            /**
                             * Encodes the specified SmartCampaignSuggestionInfo message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo} message SmartCampaignSuggestionInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SmartCampaignSuggestionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SmartCampaignSuggestionInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo} SmartCampaignSuggestionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSuggestionInfo.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.final_url = r.string();
                                        break;
                                    case 3:
                                        m.language_code = r.string();
                                        break;
                                    case 6:
                                        if (!(m.ad_schedules && m.ad_schedules.length))
                                            m.ad_schedules = [];
                                        m.ad_schedules.push($root.google.ads.googleads.v11.common.AdScheduleInfo.decode(r, r.uint32()));
                                        break;
                                    case 7:
                                        if (!(m.keyword_themes && m.keyword_themes.length))
                                            m.keyword_themes = [];
                                        m.keyword_themes.push($root.google.ads.googleads.v11.common.KeywordThemeInfo.decode(r, r.uint32()));
                                        break;
                                    case 8:
                                        m.business_context = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext.decode(r, r.uint32());
                                        break;
                                    case 9:
                                        m.business_profile_location = r.string();
                                        break;
                                    case 4:
                                        m.location_list = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.decode(r, r.uint32());
                                        break;
                                    case 5:
                                        m.proximity = $root.google.ads.googleads.v11.common.ProximityInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SmartCampaignSuggestionInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo} SmartCampaignSuggestionInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SmartCampaignSuggestionInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SmartCampaignSuggestionInfo message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SmartCampaignSuggestionInfo.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.final_url != null && m.hasOwnProperty("final_url")) {
                                    if (!$util.isString(m.final_url))
                                        return "final_url: string expected";
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    if (!$util.isString(m.language_code))
                                        return "language_code: string expected";
                                }
                                if (m.ad_schedules != null && m.hasOwnProperty("ad_schedules")) {
                                    if (!Array.isArray(m.ad_schedules))
                                        return "ad_schedules: array expected";
                                    for (var i = 0; i < m.ad_schedules.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.AdScheduleInfo.verify(m.ad_schedules[i]);
                                            if (e)
                                                return "ad_schedules." + e;
                                        }
                                    }
                                }
                                if (m.keyword_themes != null && m.hasOwnProperty("keyword_themes")) {
                                    if (!Array.isArray(m.keyword_themes))
                                        return "keyword_themes: array expected";
                                    for (var i = 0; i < m.keyword_themes.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.common.KeywordThemeInfo.verify(m.keyword_themes[i]);
                                            if (e)
                                                return "keyword_themes." + e;
                                        }
                                    }
                                }
                                if (m.business_context != null && m.hasOwnProperty("business_context")) {
                                    p.business_setting = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext.verify(m.business_context);
                                        if (e)
                                            return "business_context." + e;
                                    }
                                }
                                if (m.business_profile_location != null && m.hasOwnProperty("business_profile_location")) {
                                    if (p.business_setting === 1)
                                        return "business_setting: multiple values";
                                    p.business_setting = 1;
                                    if (!$util.isString(m.business_profile_location))
                                        return "business_profile_location: string expected";
                                }
                                if (m.location_list != null && m.hasOwnProperty("location_list")) {
                                    p.geo_target = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.verify(m.location_list);
                                        if (e)
                                            return "location_list." + e;
                                    }
                                }
                                if (m.proximity != null && m.hasOwnProperty("proximity")) {
                                    if (p.geo_target === 1)
                                        return "geo_target: multiple values";
                                    p.geo_target = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.ProximityInfo.verify(m.proximity);
                                        if (e)
                                            return "proximity." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SmartCampaignSuggestionInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo} SmartCampaignSuggestionInfo
                             */
                            SmartCampaignSuggestionInfo.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo();
                                if (d.final_url != null) {
                                    m.final_url = String(d.final_url);
                                }
                                if (d.language_code != null) {
                                    m.language_code = String(d.language_code);
                                }
                                if (d.ad_schedules) {
                                    if (!Array.isArray(d.ad_schedules))
                                        throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ad_schedules: array expected");
                                    m.ad_schedules = [];
                                    for (var i = 0; i < d.ad_schedules.length; ++i) {
                                        if (typeof d.ad_schedules[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ad_schedules: object expected");
                                        m.ad_schedules[i] = $root.google.ads.googleads.v11.common.AdScheduleInfo.fromObject(d.ad_schedules[i]);
                                    }
                                }
                                if (d.keyword_themes) {
                                    if (!Array.isArray(d.keyword_themes))
                                        throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.keyword_themes: array expected");
                                    m.keyword_themes = [];
                                    for (var i = 0; i < d.keyword_themes.length; ++i) {
                                        if (typeof d.keyword_themes[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.keyword_themes: object expected");
                                        m.keyword_themes[i] = $root.google.ads.googleads.v11.common.KeywordThemeInfo.fromObject(d.keyword_themes[i]);
                                    }
                                }
                                if (d.business_context != null) {
                                    if (typeof d.business_context !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.business_context: object expected");
                                    m.business_context = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext.fromObject(d.business_context);
                                }
                                if (d.business_profile_location != null) {
                                    m.business_profile_location = String(d.business_profile_location);
                                }
                                if (d.location_list != null) {
                                    if (typeof d.location_list !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.location_list: object expected");
                                    m.location_list = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.fromObject(d.location_list);
                                }
                                if (d.proximity != null) {
                                    if (typeof d.proximity !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.proximity: object expected");
                                    m.proximity = $root.google.ads.googleads.v11.common.ProximityInfo.fromObject(d.proximity);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SmartCampaignSuggestionInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @static
                             * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo} m SmartCampaignSuggestionInfo
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SmartCampaignSuggestionInfo.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.ad_schedules = [];
                                    d.keyword_themes = [];
                                }
                                if (o.defaults) {
                                    d.final_url = "";
                                    d.language_code = "";
                                }
                                if (m.final_url != null && m.hasOwnProperty("final_url")) {
                                    d.final_url = m.final_url;
                                }
                                if (m.language_code != null && m.hasOwnProperty("language_code")) {
                                    d.language_code = m.language_code;
                                }
                                if (m.location_list != null && m.hasOwnProperty("location_list")) {
                                    d.location_list = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.toObject(m.location_list, o);
                                    if (o.oneofs)
                                        d.geo_target = "location_list";
                                }
                                if (m.proximity != null && m.hasOwnProperty("proximity")) {
                                    d.proximity = $root.google.ads.googleads.v11.common.ProximityInfo.toObject(m.proximity, o);
                                    if (o.oneofs)
                                        d.geo_target = "proximity";
                                }
                                if (m.ad_schedules && m.ad_schedules.length) {
                                    d.ad_schedules = [];
                                    for (var j = 0; j < m.ad_schedules.length; ++j) {
                                        d.ad_schedules[j] = $root.google.ads.googleads.v11.common.AdScheduleInfo.toObject(m.ad_schedules[j], o);
                                    }
                                }
                                if (m.keyword_themes && m.keyword_themes.length) {
                                    d.keyword_themes = [];
                                    for (var j = 0; j < m.keyword_themes.length; ++j) {
                                        d.keyword_themes[j] = $root.google.ads.googleads.v11.common.KeywordThemeInfo.toObject(m.keyword_themes[j], o);
                                    }
                                }
                                if (m.business_context != null && m.hasOwnProperty("business_context")) {
                                    d.business_context = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext.toObject(m.business_context, o);
                                    if (o.oneofs)
                                        d.business_setting = "business_context";
                                }
                                if (m.business_profile_location != null && m.hasOwnProperty("business_profile_location")) {
                                    d.business_profile_location = m.business_profile_location;
                                    if (o.oneofs)
                                        d.business_setting = "business_profile_location";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SmartCampaignSuggestionInfo to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SmartCampaignSuggestionInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            SmartCampaignSuggestionInfo.LocationList = (function() {
    
                                /**
                                 * Properties of a LocationList.
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                                 * @interface ILocationList
                                 * @property {Array.<google.ads.googleads.v11.common.ILocationInfo>|null} [locations] LocationList locations
                                 */
    
                                /**
                                 * Constructs a new LocationList.
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                                 * @classdesc Represents a LocationList.
                                 * @implements ILocationList
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ILocationList=} [p] Properties to set
                                 */
                                function LocationList(p) {
                                    this.locations = [];
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * LocationList locations.
                                 * @member {Array.<google.ads.googleads.v11.common.ILocationInfo>} locations
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @instance
                                 */
                                LocationList.prototype.locations = $util.emptyArray;
    
                                /**
                                 * Creates a new LocationList instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ILocationList=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList} LocationList instance
                                 */
                                LocationList.create = function create(properties) {
                                    return new LocationList(properties);
                                };
    
                                /**
                                 * Encodes the specified LocationList message. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ILocationList} m LocationList message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocationList.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.locations != null && m.locations.length) {
                                        for (var i = 0; i < m.locations.length; ++i)
                                            $root.google.ads.googleads.v11.common.LocationInfo.encode(m.locations[i], w.uint32(10).fork()).ldelim();
                                    }
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified LocationList message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.ILocationList} message LocationList message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                LocationList.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a LocationList message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList} LocationList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocationList.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            if (!(m.locations && m.locations.length))
                                                m.locations = [];
                                            m.locations.push($root.google.ads.googleads.v11.common.LocationInfo.decode(r, r.uint32()));
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a LocationList message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList} LocationList
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                LocationList.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a LocationList message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                LocationList.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.locations != null && m.hasOwnProperty("locations")) {
                                        if (!Array.isArray(m.locations))
                                            return "locations: array expected";
                                        for (var i = 0; i < m.locations.length; ++i) {
                                            {
                                                var e = $root.google.ads.googleads.v11.common.LocationInfo.verify(m.locations[i]);
                                                if (e)
                                                    return "locations." + e;
                                            }
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a LocationList message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList} LocationList
                                 */
                                LocationList.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList();
                                    if (d.locations) {
                                        if (!Array.isArray(d.locations))
                                            throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.locations: array expected");
                                        m.locations = [];
                                        for (var i = 0; i < d.locations.length; ++i) {
                                            if (typeof d.locations[i] !== "object")
                                                throw TypeError(".google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList.locations: object expected");
                                            m.locations[i] = $root.google.ads.googleads.v11.common.LocationInfo.fromObject(d.locations[i]);
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a LocationList message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList} m LocationList
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                LocationList.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.arrays || o.defaults) {
                                        d.locations = [];
                                    }
                                    if (m.locations && m.locations.length) {
                                        d.locations = [];
                                        for (var j = 0; j < m.locations.length; ++j) {
                                            d.locations[j] = $root.google.ads.googleads.v11.common.LocationInfo.toObject(m.locations[j], o);
                                        }
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this LocationList to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.LocationList
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                LocationList.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return LocationList;
                            })();
    
                            SmartCampaignSuggestionInfo.BusinessContext = (function() {
    
                                /**
                                 * Properties of a BusinessContext.
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                                 * @interface IBusinessContext
                                 * @property {string|null} [business_name] BusinessContext business_name
                                 */
    
                                /**
                                 * Constructs a new BusinessContext.
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo
                                 * @classdesc Represents a BusinessContext.
                                 * @implements IBusinessContext
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.IBusinessContext=} [p] Properties to set
                                 */
                                function BusinessContext(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * BusinessContext business_name.
                                 * @member {string} business_name
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @instance
                                 */
                                BusinessContext.prototype.business_name = "";
    
                                /**
                                 * Creates a new BusinessContext instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.IBusinessContext=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext} BusinessContext instance
                                 */
                                BusinessContext.create = function create(properties) {
                                    return new BusinessContext(properties);
                                };
    
                                /**
                                 * Encodes the specified BusinessContext message. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.IBusinessContext} m BusinessContext message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                BusinessContext.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.business_name != null && Object.hasOwnProperty.call(m, "business_name"))
                                        w.uint32(10).string(m.business_name);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified BusinessContext message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.IBusinessContext} message BusinessContext message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                BusinessContext.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a BusinessContext message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext} BusinessContext
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                BusinessContext.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.business_name = r.string();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a BusinessContext message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext} BusinessContext
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                BusinessContext.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a BusinessContext message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                BusinessContext.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                        if (!$util.isString(m.business_name))
                                            return "business_name: string expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a BusinessContext message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext} BusinessContext
                                 */
                                BusinessContext.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext();
                                    if (d.business_name != null) {
                                        m.business_name = String(d.business_name);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a BusinessContext message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext} m BusinessContext
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                BusinessContext.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        d.business_name = "";
                                    }
                                    if (m.business_name != null && m.hasOwnProperty("business_name")) {
                                        d.business_name = m.business_name;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this BusinessContext to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.BusinessContext
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                BusinessContext.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return BusinessContext;
                            })();
    
                            return SmartCampaignSuggestionInfo;
                        })();
    
                        services.SuggestSmartCampaignBudgetOptionsResponse = (function() {
    
                            /**
                             * Properties of a SuggestSmartCampaignBudgetOptionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestSmartCampaignBudgetOptionsResponse
                             * @property {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption|null} [low] SuggestSmartCampaignBudgetOptionsResponse low
                             * @property {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption|null} [recommended] SuggestSmartCampaignBudgetOptionsResponse recommended
                             * @property {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption|null} [high] SuggestSmartCampaignBudgetOptionsResponse high
                             */
    
                            /**
                             * Constructs a new SuggestSmartCampaignBudgetOptionsResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestSmartCampaignBudgetOptionsResponse.
                             * @implements ISuggestSmartCampaignBudgetOptionsResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsResponse=} [p] Properties to set
                             */
                            function SuggestSmartCampaignBudgetOptionsResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsResponse low.
                             * @member {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption|null|undefined} low
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @instance
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.prototype.low = null;
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsResponse recommended.
                             * @member {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption|null|undefined} recommended
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @instance
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.prototype.recommended = null;
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsResponse high.
                             * @member {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption|null|undefined} high
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @instance
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.prototype.high = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsResponse _low.
                             * @member {"low"|undefined} _low
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @instance
                             */
                            Object.defineProperty(SuggestSmartCampaignBudgetOptionsResponse.prototype, "_low", {
                                get: $util.oneOfGetter($oneOfFields = ["low"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsResponse _recommended.
                             * @member {"recommended"|undefined} _recommended
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @instance
                             */
                            Object.defineProperty(SuggestSmartCampaignBudgetOptionsResponse.prototype, "_recommended", {
                                get: $util.oneOfGetter($oneOfFields = ["recommended"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * SuggestSmartCampaignBudgetOptionsResponse _high.
                             * @member {"high"|undefined} _high
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @instance
                             */
                            Object.defineProperty(SuggestSmartCampaignBudgetOptionsResponse.prototype, "_high", {
                                get: $util.oneOfGetter($oneOfFields = ["high"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new SuggestSmartCampaignBudgetOptionsResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse} SuggestSmartCampaignBudgetOptionsResponse instance
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.create = function create(properties) {
                                return new SuggestSmartCampaignBudgetOptionsResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignBudgetOptionsResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsResponse} m SuggestSmartCampaignBudgetOptionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.low != null && Object.hasOwnProperty.call(m, "low"))
                                    $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.encode(m.low, w.uint32(10).fork()).ldelim();
                                if (m.recommended != null && Object.hasOwnProperty.call(m, "recommended"))
                                    $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.encode(m.recommended, w.uint32(18).fork()).ldelim();
                                if (m.high != null && Object.hasOwnProperty.call(m, "high"))
                                    $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.encode(m.high, w.uint32(26).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignBudgetOptionsResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignBudgetOptionsResponse} message SuggestSmartCampaignBudgetOptionsResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignBudgetOptionsResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse} SuggestSmartCampaignBudgetOptionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.low = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.recommended = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.decode(r, r.uint32());
                                        break;
                                    case 3:
                                        m.high = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignBudgetOptionsResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse} SuggestSmartCampaignBudgetOptionsResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestSmartCampaignBudgetOptionsResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.low != null && m.hasOwnProperty("low")) {
                                    p._low = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.verify(m.low);
                                        if (e)
                                            return "low." + e;
                                    }
                                }
                                if (m.recommended != null && m.hasOwnProperty("recommended")) {
                                    p._recommended = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.verify(m.recommended);
                                        if (e)
                                            return "recommended." + e;
                                    }
                                }
                                if (m.high != null && m.hasOwnProperty("high")) {
                                    p._high = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.verify(m.high);
                                        if (e)
                                            return "high." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestSmartCampaignBudgetOptionsResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse} SuggestSmartCampaignBudgetOptionsResponse
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse();
                                if (d.low != null) {
                                    if (typeof d.low !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.low: object expected");
                                    m.low = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.fromObject(d.low);
                                }
                                if (d.recommended != null) {
                                    if (typeof d.recommended !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.recommended: object expected");
                                    m.recommended = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.fromObject(d.recommended);
                                }
                                if (d.high != null) {
                                    if (typeof d.high !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.high: object expected");
                                    m.high = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.fromObject(d.high);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestSmartCampaignBudgetOptionsResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse} m SuggestSmartCampaignBudgetOptionsResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.low != null && m.hasOwnProperty("low")) {
                                    d.low = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.toObject(m.low, o);
                                    if (o.oneofs)
                                        d._low = "low";
                                }
                                if (m.recommended != null && m.hasOwnProperty("recommended")) {
                                    d.recommended = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.toObject(m.recommended, o);
                                    if (o.oneofs)
                                        d._recommended = "recommended";
                                }
                                if (m.high != null && m.hasOwnProperty("high")) {
                                    d.high = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.toObject(m.high, o);
                                    if (o.oneofs)
                                        d._high = "high";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestSmartCampaignBudgetOptionsResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestSmartCampaignBudgetOptionsResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            SuggestSmartCampaignBudgetOptionsResponse.Metrics = (function() {
    
                                /**
                                 * Properties of a Metrics.
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                                 * @interface IMetrics
                                 * @property {number|null} [min_daily_clicks] Metrics min_daily_clicks
                                 * @property {number|null} [max_daily_clicks] Metrics max_daily_clicks
                                 */
    
                                /**
                                 * Constructs a new Metrics.
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                                 * @classdesc Represents a Metrics.
                                 * @implements IMetrics
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IMetrics=} [p] Properties to set
                                 */
                                function Metrics(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * Metrics min_daily_clicks.
                                 * @member {number} min_daily_clicks
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @instance
                                 */
                                Metrics.prototype.min_daily_clicks = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * Metrics max_daily_clicks.
                                 * @member {number} max_daily_clicks
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @instance
                                 */
                                Metrics.prototype.max_daily_clicks = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * Creates a new Metrics instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IMetrics=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics} Metrics instance
                                 */
                                Metrics.create = function create(properties) {
                                    return new Metrics(properties);
                                };
    
                                /**
                                 * Encodes the specified Metrics message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IMetrics} m Metrics message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Metrics.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.min_daily_clicks != null && Object.hasOwnProperty.call(m, "min_daily_clicks"))
                                        w.uint32(8).int64(m.min_daily_clicks);
                                    if (m.max_daily_clicks != null && Object.hasOwnProperty.call(m, "max_daily_clicks"))
                                        w.uint32(16).int64(m.max_daily_clicks);
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified Metrics message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IMetrics} message Metrics message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Metrics.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a Metrics message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics} Metrics
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Metrics.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.min_daily_clicks = r.int64();
                                            break;
                                        case 2:
                                            m.max_daily_clicks = r.int64();
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a Metrics message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics} Metrics
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Metrics.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a Metrics message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                Metrics.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.min_daily_clicks != null && m.hasOwnProperty("min_daily_clicks")) {
                                        if (!$util.isInteger(m.min_daily_clicks) && !(m.min_daily_clicks && $util.isInteger(m.min_daily_clicks.low) && $util.isInteger(m.min_daily_clicks.high)))
                                            return "min_daily_clicks: integer|Long expected";
                                    }
                                    if (m.max_daily_clicks != null && m.hasOwnProperty("max_daily_clicks")) {
                                        if (!$util.isInteger(m.max_daily_clicks) && !(m.max_daily_clicks && $util.isInteger(m.max_daily_clicks.low) && $util.isInteger(m.max_daily_clicks.high)))
                                            return "max_daily_clicks: integer|Long expected";
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a Metrics message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics} Metrics
                                 */
                                Metrics.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics();
                                    if (d.min_daily_clicks != null) {
                                        if ($util.Long)
                                            (m.min_daily_clicks = $util.Long.fromValue(d.min_daily_clicks)).unsigned = false;
                                        else if (typeof d.min_daily_clicks === "string")
                                            m.min_daily_clicks = parseInt(d.min_daily_clicks, 10);
                                        else if (typeof d.min_daily_clicks === "number")
                                            m.min_daily_clicks = d.min_daily_clicks;
                                        else if (typeof d.min_daily_clicks === "object")
                                            m.min_daily_clicks = new $util.LongBits(d.min_daily_clicks.low >>> 0, d.min_daily_clicks.high >>> 0).toNumber();
                                    }
                                    if (d.max_daily_clicks != null) {
                                        if ($util.Long)
                                            (m.max_daily_clicks = $util.Long.fromValue(d.max_daily_clicks)).unsigned = false;
                                        else if (typeof d.max_daily_clicks === "string")
                                            m.max_daily_clicks = parseInt(d.max_daily_clicks, 10);
                                        else if (typeof d.max_daily_clicks === "number")
                                            m.max_daily_clicks = d.max_daily_clicks;
                                        else if (typeof d.max_daily_clicks === "object")
                                            m.max_daily_clicks = new $util.LongBits(d.max_daily_clicks.low >>> 0, d.max_daily_clicks.high >>> 0).toNumber();
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a Metrics message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics} m Metrics
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                Metrics.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.min_daily_clicks = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.min_daily_clicks = o.longs === String ? "0" : 0;
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.max_daily_clicks = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.max_daily_clicks = o.longs === String ? "0" : 0;
                                    }
                                    if (m.min_daily_clicks != null && m.hasOwnProperty("min_daily_clicks")) {
                                        if (typeof m.min_daily_clicks === "number")
                                            d.min_daily_clicks = o.longs === String ? String(m.min_daily_clicks) : m.min_daily_clicks;
                                        else
                                            d.min_daily_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.min_daily_clicks) : o.longs === Number ? new $util.LongBits(m.min_daily_clicks.low >>> 0, m.min_daily_clicks.high >>> 0).toNumber() : m.min_daily_clicks;
                                    }
                                    if (m.max_daily_clicks != null && m.hasOwnProperty("max_daily_clicks")) {
                                        if (typeof m.max_daily_clicks === "number")
                                            d.max_daily_clicks = o.longs === String ? String(m.max_daily_clicks) : m.max_daily_clicks;
                                        else
                                            d.max_daily_clicks = o.longs === String ? $util.Long.prototype.toString.call(m.max_daily_clicks) : o.longs === Number ? new $util.LongBits(m.max_daily_clicks.low >>> 0, m.max_daily_clicks.high >>> 0).toNumber() : m.max_daily_clicks;
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this Metrics to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                Metrics.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return Metrics;
                            })();
    
                            SuggestSmartCampaignBudgetOptionsResponse.BudgetOption = (function() {
    
                                /**
                                 * Properties of a BudgetOption.
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                                 * @interface IBudgetOption
                                 * @property {number|null} [daily_amount_micros] BudgetOption daily_amount_micros
                                 * @property {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IMetrics|null} [metrics] BudgetOption metrics
                                 */
    
                                /**
                                 * Constructs a new BudgetOption.
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse
                                 * @classdesc Represents a BudgetOption.
                                 * @implements IBudgetOption
                                 * @constructor
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption=} [p] Properties to set
                                 */
                                function BudgetOption(p) {
                                    if (p)
                                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                            if (p[ks[i]] != null)
                                                this[ks[i]] = p[ks[i]];
                                }
    
                                /**
                                 * BudgetOption daily_amount_micros.
                                 * @member {number} daily_amount_micros
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @instance
                                 */
                                BudgetOption.prototype.daily_amount_micros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                                /**
                                 * BudgetOption metrics.
                                 * @member {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IMetrics|null|undefined} metrics
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @instance
                                 */
                                BudgetOption.prototype.metrics = null;
    
                                /**
                                 * Creates a new BudgetOption instance using the specified properties.
                                 * @function create
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption=} [properties] Properties to set
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption} BudgetOption instance
                                 */
                                BudgetOption.create = function create(properties) {
                                    return new BudgetOption(properties);
                                };
    
                                /**
                                 * Encodes the specified BudgetOption message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption} m BudgetOption message or plain object to encode
                                 * @param {$protobuf.Writer} [w] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                BudgetOption.encode = function encode(m, w) {
                                    if (!w)
                                        w = $Writer.create();
                                    if (m.daily_amount_micros != null && Object.hasOwnProperty.call(m, "daily_amount_micros"))
                                        w.uint32(8).int64(m.daily_amount_micros);
                                    if (m.metrics != null && Object.hasOwnProperty.call(m, "metrics"))
                                        $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics.encode(m.metrics, w.uint32(18).fork()).ldelim();
                                    return w;
                                };
    
                                /**
                                 * Encodes the specified BudgetOption message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.IBudgetOption} message BudgetOption message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                BudgetOption.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a BudgetOption message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                                 * @param {number} [l] Message length if known beforehand
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption} BudgetOption
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                BudgetOption.decode = function decode(r, l) {
                                    if (!(r instanceof $Reader))
                                        r = $Reader.create(r);
                                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption();
                                    while (r.pos < c) {
                                        var t = r.uint32();
                                        switch (t >>> 3) {
                                        case 1:
                                            m.daily_amount_micros = r.int64();
                                            break;
                                        case 2:
                                            m.metrics = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics.decode(r, r.uint32());
                                            break;
                                        default:
                                            r.skipType(t & 7);
                                            break;
                                        }
                                    }
                                    return m;
                                };
    
                                /**
                                 * Decodes a BudgetOption message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption} BudgetOption
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                BudgetOption.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a BudgetOption message.
                                 * @function verify
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {Object.<string,*>} m Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                BudgetOption.verify = function verify(m) {
                                    if (typeof m !== "object" || m === null)
                                        return "object expected";
                                    if (m.daily_amount_micros != null && m.hasOwnProperty("daily_amount_micros")) {
                                        if (!$util.isInteger(m.daily_amount_micros) && !(m.daily_amount_micros && $util.isInteger(m.daily_amount_micros.low) && $util.isInteger(m.daily_amount_micros.high)))
                                            return "daily_amount_micros: integer|Long expected";
                                    }
                                    if (m.metrics != null && m.hasOwnProperty("metrics")) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics.verify(m.metrics);
                                            if (e)
                                                return "metrics." + e;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates a BudgetOption message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {Object.<string,*>} d Plain object
                                 * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption} BudgetOption
                                 */
                                BudgetOption.fromObject = function fromObject(d) {
                                    if (d instanceof $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption)
                                        return d;
                                    var m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption();
                                    if (d.daily_amount_micros != null) {
                                        if ($util.Long)
                                            (m.daily_amount_micros = $util.Long.fromValue(d.daily_amount_micros)).unsigned = false;
                                        else if (typeof d.daily_amount_micros === "string")
                                            m.daily_amount_micros = parseInt(d.daily_amount_micros, 10);
                                        else if (typeof d.daily_amount_micros === "number")
                                            m.daily_amount_micros = d.daily_amount_micros;
                                        else if (typeof d.daily_amount_micros === "object")
                                            m.daily_amount_micros = new $util.LongBits(d.daily_amount_micros.low >>> 0, d.daily_amount_micros.high >>> 0).toNumber();
                                    }
                                    if (d.metrics != null) {
                                        if (typeof d.metrics !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption.metrics: object expected");
                                        m.metrics = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics.fromObject(d.metrics);
                                    }
                                    return m;
                                };
    
                                /**
                                 * Creates a plain object from a BudgetOption message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @static
                                 * @param {google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption} m BudgetOption
                                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                BudgetOption.toObject = function toObject(m, o) {
                                    if (!o)
                                        o = {};
                                    var d = {};
                                    if (o.defaults) {
                                        if ($util.Long) {
                                            var n = new $util.Long(0, 0, false);
                                            d.daily_amount_micros = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                                        } else
                                            d.daily_amount_micros = o.longs === String ? "0" : 0;
                                        d.metrics = null;
                                    }
                                    if (m.daily_amount_micros != null && m.hasOwnProperty("daily_amount_micros")) {
                                        if (typeof m.daily_amount_micros === "number")
                                            d.daily_amount_micros = o.longs === String ? String(m.daily_amount_micros) : m.daily_amount_micros;
                                        else
                                            d.daily_amount_micros = o.longs === String ? $util.Long.prototype.toString.call(m.daily_amount_micros) : o.longs === Number ? new $util.LongBits(m.daily_amount_micros.low >>> 0, m.daily_amount_micros.high >>> 0).toNumber() : m.daily_amount_micros;
                                    }
                                    if (m.metrics != null && m.hasOwnProperty("metrics")) {
                                        d.metrics = $root.google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.Metrics.toObject(m.metrics, o);
                                    }
                                    return d;
                                };
    
                                /**
                                 * Converts this BudgetOption to JSON.
                                 * @function toJSON
                                 * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                BudgetOption.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                return BudgetOption;
                            })();
    
                            return SuggestSmartCampaignBudgetOptionsResponse;
                        })();
    
                        services.SuggestSmartCampaignAdRequest = (function() {
    
                            /**
                             * Properties of a SuggestSmartCampaignAdRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestSmartCampaignAdRequest
                             * @property {string|null} [customer_id] SuggestSmartCampaignAdRequest customer_id
                             * @property {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo|null} [suggestion_info] SuggestSmartCampaignAdRequest suggestion_info
                             */
    
                            /**
                             * Constructs a new SuggestSmartCampaignAdRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestSmartCampaignAdRequest.
                             * @implements ISuggestSmartCampaignAdRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdRequest=} [p] Properties to set
                             */
                            function SuggestSmartCampaignAdRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestSmartCampaignAdRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @instance
                             */
                            SuggestSmartCampaignAdRequest.prototype.customer_id = "";
    
                            /**
                             * SuggestSmartCampaignAdRequest suggestion_info.
                             * @member {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo|null|undefined} suggestion_info
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @instance
                             */
                            SuggestSmartCampaignAdRequest.prototype.suggestion_info = null;
    
                            /**
                             * Creates a new SuggestSmartCampaignAdRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest} SuggestSmartCampaignAdRequest instance
                             */
                            SuggestSmartCampaignAdRequest.create = function create(properties) {
                                return new SuggestSmartCampaignAdRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignAdRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdRequest} m SuggestSmartCampaignAdRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignAdRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.suggestion_info != null && Object.hasOwnProperty.call(m, "suggestion_info"))
                                    $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.encode(m.suggestion_info, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignAdRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdRequest} message SuggestSmartCampaignAdRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignAdRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignAdRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest} SuggestSmartCampaignAdRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignAdRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignAdRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest} SuggestSmartCampaignAdRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignAdRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestSmartCampaignAdRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestSmartCampaignAdRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.suggestion_info != null && m.hasOwnProperty("suggestion_info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.verify(m.suggestion_info);
                                        if (e)
                                            return "suggestion_info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestSmartCampaignAdRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest} SuggestSmartCampaignAdRequest
                             */
                            SuggestSmartCampaignAdRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.suggestion_info != null) {
                                    if (typeof d.suggestion_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest.suggestion_info: object expected");
                                    m.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.fromObject(d.suggestion_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestSmartCampaignAdRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest} m SuggestSmartCampaignAdRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestSmartCampaignAdRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.suggestion_info = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.suggestion_info != null && m.hasOwnProperty("suggestion_info")) {
                                    d.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.toObject(m.suggestion_info, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestSmartCampaignAdRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestSmartCampaignAdRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestSmartCampaignAdRequest;
                        })();
    
                        services.SuggestSmartCampaignAdResponse = (function() {
    
                            /**
                             * Properties of a SuggestSmartCampaignAdResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestSmartCampaignAdResponse
                             * @property {google.ads.googleads.v11.common.ISmartCampaignAdInfo|null} [ad_info] SuggestSmartCampaignAdResponse ad_info
                             */
    
                            /**
                             * Constructs a new SuggestSmartCampaignAdResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestSmartCampaignAdResponse.
                             * @implements ISuggestSmartCampaignAdResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdResponse=} [p] Properties to set
                             */
                            function SuggestSmartCampaignAdResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestSmartCampaignAdResponse ad_info.
                             * @member {google.ads.googleads.v11.common.ISmartCampaignAdInfo|null|undefined} ad_info
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @instance
                             */
                            SuggestSmartCampaignAdResponse.prototype.ad_info = null;
    
                            /**
                             * Creates a new SuggestSmartCampaignAdResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse} SuggestSmartCampaignAdResponse instance
                             */
                            SuggestSmartCampaignAdResponse.create = function create(properties) {
                                return new SuggestSmartCampaignAdResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignAdResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdResponse} m SuggestSmartCampaignAdResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignAdResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.ad_info != null && Object.hasOwnProperty.call(m, "ad_info"))
                                    $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.encode(m.ad_info, w.uint32(10).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestSmartCampaignAdResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestSmartCampaignAdResponse} message SuggestSmartCampaignAdResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestSmartCampaignAdResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignAdResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse} SuggestSmartCampaignAdResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignAdResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.ad_info = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestSmartCampaignAdResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse} SuggestSmartCampaignAdResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestSmartCampaignAdResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestSmartCampaignAdResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestSmartCampaignAdResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.ad_info != null && m.hasOwnProperty("ad_info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.verify(m.ad_info);
                                        if (e)
                                            return "ad_info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestSmartCampaignAdResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse} SuggestSmartCampaignAdResponse
                             */
                            SuggestSmartCampaignAdResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse();
                                if (d.ad_info != null) {
                                    if (typeof d.ad_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse.ad_info: object expected");
                                    m.ad_info = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.fromObject(d.ad_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestSmartCampaignAdResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse} m SuggestSmartCampaignAdResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestSmartCampaignAdResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.ad_info = null;
                                }
                                if (m.ad_info != null && m.hasOwnProperty("ad_info")) {
                                    d.ad_info = $root.google.ads.googleads.v11.common.SmartCampaignAdInfo.toObject(m.ad_info, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestSmartCampaignAdResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestSmartCampaignAdResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestSmartCampaignAdResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestSmartCampaignAdResponse;
                        })();
    
                        services.SuggestKeywordThemesRequest = (function() {
    
                            /**
                             * Properties of a SuggestKeywordThemesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestKeywordThemesRequest
                             * @property {string|null} [customer_id] SuggestKeywordThemesRequest customer_id
                             * @property {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo|null} [suggestion_info] SuggestKeywordThemesRequest suggestion_info
                             */
    
                            /**
                             * Constructs a new SuggestKeywordThemesRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestKeywordThemesRequest.
                             * @implements ISuggestKeywordThemesRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesRequest=} [p] Properties to set
                             */
                            function SuggestKeywordThemesRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestKeywordThemesRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @instance
                             */
                            SuggestKeywordThemesRequest.prototype.customer_id = "";
    
                            /**
                             * SuggestKeywordThemesRequest suggestion_info.
                             * @member {google.ads.googleads.v11.services.ISmartCampaignSuggestionInfo|null|undefined} suggestion_info
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @instance
                             */
                            SuggestKeywordThemesRequest.prototype.suggestion_info = null;
    
                            /**
                             * Creates a new SuggestKeywordThemesRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesRequest} SuggestKeywordThemesRequest instance
                             */
                            SuggestKeywordThemesRequest.create = function create(properties) {
                                return new SuggestKeywordThemesRequest(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemesRequest message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemesRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesRequest} m SuggestKeywordThemesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemesRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.suggestion_info != null && Object.hasOwnProperty.call(m, "suggestion_info"))
                                    $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.encode(m.suggestion_info, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemesRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemesRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesRequest} message SuggestKeywordThemesRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemesRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesRequest} SuggestKeywordThemesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemesRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemesRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 2:
                                        m.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemesRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesRequest} SuggestKeywordThemesRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemesRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestKeywordThemesRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestKeywordThemesRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.suggestion_info != null && m.hasOwnProperty("suggestion_info")) {
                                    {
                                        var e = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.verify(m.suggestion_info);
                                        if (e)
                                            return "suggestion_info." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestKeywordThemesRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesRequest} SuggestKeywordThemesRequest
                             */
                            SuggestKeywordThemesRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestKeywordThemesRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemesRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.suggestion_info != null) {
                                    if (typeof d.suggestion_info !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestKeywordThemesRequest.suggestion_info: object expected");
                                    m.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.fromObject(d.suggestion_info);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestKeywordThemesRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestKeywordThemesRequest} m SuggestKeywordThemesRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestKeywordThemesRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.customer_id = "";
                                    d.suggestion_info = null;
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.suggestion_info != null && m.hasOwnProperty("suggestion_info")) {
                                    d.suggestion_info = $root.google.ads.googleads.v11.services.SmartCampaignSuggestionInfo.toObject(m.suggestion_info, o);
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestKeywordThemesRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestKeywordThemesRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestKeywordThemesRequest;
                        })();
    
                        services.SuggestKeywordThemesResponse = (function() {
    
                            /**
                             * Properties of a SuggestKeywordThemesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface ISuggestKeywordThemesResponse
                             * @property {Array.<google.ads.googleads.v11.resources.IKeywordThemeConstant>|null} [keyword_themes] SuggestKeywordThemesResponse keyword_themes
                             */
    
                            /**
                             * Constructs a new SuggestKeywordThemesResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a SuggestKeywordThemesResponse.
                             * @implements ISuggestKeywordThemesResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesResponse=} [p] Properties to set
                             */
                            function SuggestKeywordThemesResponse(p) {
                                this.keyword_themes = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * SuggestKeywordThemesResponse keyword_themes.
                             * @member {Array.<google.ads.googleads.v11.resources.IKeywordThemeConstant>} keyword_themes
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @instance
                             */
                            SuggestKeywordThemesResponse.prototype.keyword_themes = $util.emptyArray;
    
                            /**
                             * Creates a new SuggestKeywordThemesResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesResponse} SuggestKeywordThemesResponse instance
                             */
                            SuggestKeywordThemesResponse.create = function create(properties) {
                                return new SuggestKeywordThemesResponse(properties);
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemesResponse message. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemesResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesResponse} m SuggestKeywordThemesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemesResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.keyword_themes != null && m.keyword_themes.length) {
                                    for (var i = 0; i < m.keyword_themes.length; ++i)
                                        $root.google.ads.googleads.v11.resources.KeywordThemeConstant.encode(m.keyword_themes[i], w.uint32(10).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified SuggestKeywordThemesResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.SuggestKeywordThemesResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.ISuggestKeywordThemesResponse} message SuggestKeywordThemesResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SuggestKeywordThemesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemesResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesResponse} SuggestKeywordThemesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemesResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemesResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        if (!(m.keyword_themes && m.keyword_themes.length))
                                            m.keyword_themes = [];
                                        m.keyword_themes.push($root.google.ads.googleads.v11.resources.KeywordThemeConstant.decode(r, r.uint32()));
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a SuggestKeywordThemesResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesResponse} SuggestKeywordThemesResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SuggestKeywordThemesResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SuggestKeywordThemesResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SuggestKeywordThemesResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.keyword_themes != null && m.hasOwnProperty("keyword_themes")) {
                                    if (!Array.isArray(m.keyword_themes))
                                        return "keyword_themes: array expected";
                                    for (var i = 0; i < m.keyword_themes.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.verify(m.keyword_themes[i]);
                                            if (e)
                                                return "keyword_themes." + e;
                                        }
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SuggestKeywordThemesResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.SuggestKeywordThemesResponse} SuggestKeywordThemesResponse
                             */
                            SuggestKeywordThemesResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.SuggestKeywordThemesResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.SuggestKeywordThemesResponse();
                                if (d.keyword_themes) {
                                    if (!Array.isArray(d.keyword_themes))
                                        throw TypeError(".google.ads.googleads.v11.services.SuggestKeywordThemesResponse.keyword_themes: array expected");
                                    m.keyword_themes = [];
                                    for (var i = 0; i < d.keyword_themes.length; ++i) {
                                        if (typeof d.keyword_themes[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.SuggestKeywordThemesResponse.keyword_themes: object expected");
                                        m.keyword_themes[i] = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.fromObject(d.keyword_themes[i]);
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a SuggestKeywordThemesResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.SuggestKeywordThemesResponse} m SuggestKeywordThemesResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SuggestKeywordThemesResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.keyword_themes = [];
                                }
                                if (m.keyword_themes && m.keyword_themes.length) {
                                    d.keyword_themes = [];
                                    for (var j = 0; j < m.keyword_themes.length; ++j) {
                                        d.keyword_themes[j] = $root.google.ads.googleads.v11.resources.KeywordThemeConstant.toObject(m.keyword_themes[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this SuggestKeywordThemesResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.SuggestKeywordThemesResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SuggestKeywordThemesResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return SuggestKeywordThemesResponse;
                        })();
    
                        services.ThirdPartyAppAnalyticsLinkService = (function() {
    
                            /**
                             * Constructs a new ThirdPartyAppAnalyticsLinkService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a ThirdPartyAppAnalyticsLinkService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function ThirdPartyAppAnalyticsLinkService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (ThirdPartyAppAnalyticsLinkService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = ThirdPartyAppAnalyticsLinkService;
    
                            /**
                             * Creates new ThirdPartyAppAnalyticsLinkService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.ThirdPartyAppAnalyticsLinkService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {ThirdPartyAppAnalyticsLinkService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            ThirdPartyAppAnalyticsLinkService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.ThirdPartyAppAnalyticsLinkService|regenerateShareableLinkId}.
                             * @memberof google.ads.googleads.v11.services.ThirdPartyAppAnalyticsLinkService
                             * @typedef RegenerateShareableLinkIdCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse} [response] RegenerateShareableLinkIdResponse
                             */
    
                            /**
                             * Calls RegenerateShareableLinkId.
                             * @function regenerateShareableLinkId
                             * @memberof google.ads.googleads.v11.services.ThirdPartyAppAnalyticsLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdRequest} request RegenerateShareableLinkIdRequest message or plain object
                             * @param {google.ads.googleads.v11.services.ThirdPartyAppAnalyticsLinkService.RegenerateShareableLinkIdCallback} callback Node-style callback called with the error, if any, and RegenerateShareableLinkIdResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(ThirdPartyAppAnalyticsLinkService.prototype.regenerateShareableLinkId = function regenerateShareableLinkId(request, callback) {
                                return this.rpcCall(regenerateShareableLinkId, $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest, $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse, request, callback);
                            }, "name", { value: "RegenerateShareableLinkId" });
    
                            /**
                             * Calls RegenerateShareableLinkId.
                             * @function regenerateShareableLinkId
                             * @memberof google.ads.googleads.v11.services.ThirdPartyAppAnalyticsLinkService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdRequest} request RegenerateShareableLinkIdRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse>} Promise
                             * @variation 2
                             */
    
                            return ThirdPartyAppAnalyticsLinkService;
                        })();
    
                        services.RegenerateShareableLinkIdRequest = (function() {
    
                            /**
                             * Properties of a RegenerateShareableLinkIdRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IRegenerateShareableLinkIdRequest
                             * @property {string|null} [resource_name] RegenerateShareableLinkIdRequest resource_name
                             */
    
                            /**
                             * Constructs a new RegenerateShareableLinkIdRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RegenerateShareableLinkIdRequest.
                             * @implements IRegenerateShareableLinkIdRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdRequest=} [p] Properties to set
                             */
                            function RegenerateShareableLinkIdRequest(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * RegenerateShareableLinkIdRequest resource_name.
                             * @member {string} resource_name
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @instance
                             */
                            RegenerateShareableLinkIdRequest.prototype.resource_name = "";
    
                            /**
                             * Creates a new RegenerateShareableLinkIdRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest} RegenerateShareableLinkIdRequest instance
                             */
                            RegenerateShareableLinkIdRequest.create = function create(properties) {
                                return new RegenerateShareableLinkIdRequest(properties);
                            };
    
                            /**
                             * Encodes the specified RegenerateShareableLinkIdRequest message. Does not implicitly {@link google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdRequest} m RegenerateShareableLinkIdRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RegenerateShareableLinkIdRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.resource_name != null && Object.hasOwnProperty.call(m, "resource_name"))
                                    w.uint32(10).string(m.resource_name);
                                return w;
                            };
    
                            /**
                             * Encodes the specified RegenerateShareableLinkIdRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdRequest} message RegenerateShareableLinkIdRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RegenerateShareableLinkIdRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RegenerateShareableLinkIdRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest} RegenerateShareableLinkIdRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RegenerateShareableLinkIdRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.resource_name = r.string();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RegenerateShareableLinkIdRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest} RegenerateShareableLinkIdRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RegenerateShareableLinkIdRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RegenerateShareableLinkIdRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RegenerateShareableLinkIdRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    if (!$util.isString(m.resource_name))
                                        return "resource_name: string expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a RegenerateShareableLinkIdRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest} RegenerateShareableLinkIdRequest
                             */
                            RegenerateShareableLinkIdRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest();
                                if (d.resource_name != null) {
                                    m.resource_name = String(d.resource_name);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a RegenerateShareableLinkIdRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest} m RegenerateShareableLinkIdRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RegenerateShareableLinkIdRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.defaults) {
                                    d.resource_name = "";
                                }
                                if (m.resource_name != null && m.hasOwnProperty("resource_name")) {
                                    d.resource_name = m.resource_name;
                                }
                                return d;
                            };
    
                            /**
                             * Converts this RegenerateShareableLinkIdRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RegenerateShareableLinkIdRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RegenerateShareableLinkIdRequest;
                        })();
    
                        services.RegenerateShareableLinkIdResponse = (function() {
    
                            /**
                             * Properties of a RegenerateShareableLinkIdResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IRegenerateShareableLinkIdResponse
                             */
    
                            /**
                             * Constructs a new RegenerateShareableLinkIdResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a RegenerateShareableLinkIdResponse.
                             * @implements IRegenerateShareableLinkIdResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdResponse=} [p] Properties to set
                             */
                            function RegenerateShareableLinkIdResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * Creates a new RegenerateShareableLinkIdResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse} RegenerateShareableLinkIdResponse instance
                             */
                            RegenerateShareableLinkIdResponse.create = function create(properties) {
                                return new RegenerateShareableLinkIdResponse(properties);
                            };
    
                            /**
                             * Encodes the specified RegenerateShareableLinkIdResponse message. Does not implicitly {@link google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdResponse} m RegenerateShareableLinkIdResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RegenerateShareableLinkIdResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                return w;
                            };
    
                            /**
                             * Encodes the specified RegenerateShareableLinkIdResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IRegenerateShareableLinkIdResponse} message RegenerateShareableLinkIdResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            RegenerateShareableLinkIdResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a RegenerateShareableLinkIdResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse} RegenerateShareableLinkIdResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RegenerateShareableLinkIdResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a RegenerateShareableLinkIdResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse} RegenerateShareableLinkIdResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            RegenerateShareableLinkIdResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a RegenerateShareableLinkIdResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            RegenerateShareableLinkIdResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                return null;
                            };
    
                            /**
                             * Creates a RegenerateShareableLinkIdResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse} RegenerateShareableLinkIdResponse
                             */
                            RegenerateShareableLinkIdResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse)
                                    return d;
                                return new $root.google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse();
                            };
    
                            /**
                             * Creates a plain object from a RegenerateShareableLinkIdResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse} m RegenerateShareableLinkIdResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            RegenerateShareableLinkIdResponse.toObject = function toObject() {
                                return {};
                            };
    
                            /**
                             * Converts this RegenerateShareableLinkIdResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.RegenerateShareableLinkIdResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            RegenerateShareableLinkIdResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return RegenerateShareableLinkIdResponse;
                        })();
    
                        services.UserDataService = (function() {
    
                            /**
                             * Constructs a new UserDataService service.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a UserDataService
                             * @extends $protobuf.rpc.Service
                             * @constructor
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             */
                            function UserDataService(rpcImpl, requestDelimited, responseDelimited) {
                                $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                            }
    
                            (UserDataService.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = UserDataService;
    
                            /**
                             * Creates new UserDataService service using the specified rpc implementation.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UserDataService
                             * @static
                             * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                             * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                             * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                             * @returns {UserDataService} RPC service. Useful where requests and/or responses are streamed.
                             */
                            UserDataService.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                                return new this(rpcImpl, requestDelimited, responseDelimited);
                            };
    
                            /**
                             * Callback as used by {@link google.ads.googleads.v11.services.UserDataService|uploadUserData}.
                             * @memberof google.ads.googleads.v11.services.UserDataService
                             * @typedef UploadUserDataCallback
                             * @type {function}
                             * @param {Error|null} error Error, if any
                             * @param {google.ads.googleads.v11.services.UploadUserDataResponse} [response] UploadUserDataResponse
                             */
    
                            /**
                             * Calls UploadUserData.
                             * @function uploadUserData
                             * @memberof google.ads.googleads.v11.services.UserDataService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadUserDataRequest} request UploadUserDataRequest message or plain object
                             * @param {google.ads.googleads.v11.services.UserDataService.UploadUserDataCallback} callback Node-style callback called with the error, if any, and UploadUserDataResponse
                             * @returns {undefined}
                             * @variation 1
                             */
                            Object.defineProperty(UserDataService.prototype.uploadUserData = function uploadUserData(request, callback) {
                                return this.rpcCall(uploadUserData, $root.google.ads.googleads.v11.services.UploadUserDataRequest, $root.google.ads.googleads.v11.services.UploadUserDataResponse, request, callback);
                            }, "name", { value: "UploadUserData" });
    
                            /**
                             * Calls UploadUserData.
                             * @function uploadUserData
                             * @memberof google.ads.googleads.v11.services.UserDataService
                             * @instance
                             * @param {google.ads.googleads.v11.services.IUploadUserDataRequest} request UploadUserDataRequest message or plain object
                             * @returns {Promise<google.ads.googleads.v11.services.UploadUserDataResponse>} Promise
                             * @variation 2
                             */
    
                            return UserDataService;
                        })();
    
                        services.UploadUserDataRequest = (function() {
    
                            /**
                             * Properties of an UploadUserDataRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadUserDataRequest
                             * @property {string|null} [customer_id] UploadUserDataRequest customer_id
                             * @property {Array.<google.ads.googleads.v11.services.IUserDataOperation>|null} [operations] UploadUserDataRequest operations
                             * @property {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata|null} [customer_match_user_list_metadata] UploadUserDataRequest customer_match_user_list_metadata
                             */
    
                            /**
                             * Constructs a new UploadUserDataRequest.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadUserDataRequest.
                             * @implements IUploadUserDataRequest
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadUserDataRequest=} [p] Properties to set
                             */
                            function UploadUserDataRequest(p) {
                                this.operations = [];
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadUserDataRequest customer_id.
                             * @member {string} customer_id
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @instance
                             */
                            UploadUserDataRequest.prototype.customer_id = "";
    
                            /**
                             * UploadUserDataRequest operations.
                             * @member {Array.<google.ads.googleads.v11.services.IUserDataOperation>} operations
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @instance
                             */
                            UploadUserDataRequest.prototype.operations = $util.emptyArray;
    
                            /**
                             * UploadUserDataRequest customer_match_user_list_metadata.
                             * @member {google.ads.googleads.v11.common.ICustomerMatchUserListMetadata|null|undefined} customer_match_user_list_metadata
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @instance
                             */
                            UploadUserDataRequest.prototype.customer_match_user_list_metadata = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UploadUserDataRequest metadata.
                             * @member {"customer_match_user_list_metadata"|undefined} metadata
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @instance
                             */
                            Object.defineProperty(UploadUserDataRequest.prototype, "metadata", {
                                get: $util.oneOfGetter($oneOfFields = ["customer_match_user_list_metadata"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UploadUserDataRequest instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadUserDataRequest=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadUserDataRequest} UploadUserDataRequest instance
                             */
                            UploadUserDataRequest.create = function create(properties) {
                                return new UploadUserDataRequest(properties);
                            };
    
                            /**
                             * Encodes the specified UploadUserDataRequest message. Does not implicitly {@link google.ads.googleads.v11.services.UploadUserDataRequest.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadUserDataRequest} m UploadUserDataRequest message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadUserDataRequest.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.customer_id != null && Object.hasOwnProperty.call(m, "customer_id"))
                                    w.uint32(10).string(m.customer_id);
                                if (m.customer_match_user_list_metadata != null && Object.hasOwnProperty.call(m, "customer_match_user_list_metadata"))
                                    $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.encode(m.customer_match_user_list_metadata, w.uint32(18).fork()).ldelim();
                                if (m.operations != null && m.operations.length) {
                                    for (var i = 0; i < m.operations.length; ++i)
                                        $root.google.ads.googleads.v11.services.UserDataOperation.encode(m.operations[i], w.uint32(26).fork()).ldelim();
                                }
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadUserDataRequest message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadUserDataRequest.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadUserDataRequest} message UploadUserDataRequest message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadUserDataRequest.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadUserDataRequest message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadUserDataRequest} UploadUserDataRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadUserDataRequest.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadUserDataRequest();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.customer_id = r.string();
                                        break;
                                    case 3:
                                        if (!(m.operations && m.operations.length))
                                            m.operations = [];
                                        m.operations.push($root.google.ads.googleads.v11.services.UserDataOperation.decode(r, r.uint32()));
                                        break;
                                    case 2:
                                        m.customer_match_user_list_metadata = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadUserDataRequest message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadUserDataRequest} UploadUserDataRequest
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadUserDataRequest.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadUserDataRequest message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadUserDataRequest.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    if (!$util.isString(m.customer_id))
                                        return "customer_id: string expected";
                                }
                                if (m.operations != null && m.hasOwnProperty("operations")) {
                                    if (!Array.isArray(m.operations))
                                        return "operations: array expected";
                                    for (var i = 0; i < m.operations.length; ++i) {
                                        {
                                            var e = $root.google.ads.googleads.v11.services.UserDataOperation.verify(m.operations[i]);
                                            if (e)
                                                return "operations." + e;
                                        }
                                    }
                                }
                                if (m.customer_match_user_list_metadata != null && m.hasOwnProperty("customer_match_user_list_metadata")) {
                                    p.metadata = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.verify(m.customer_match_user_list_metadata);
                                        if (e)
                                            return "customer_match_user_list_metadata." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadUserDataRequest message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadUserDataRequest} UploadUserDataRequest
                             */
                            UploadUserDataRequest.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadUserDataRequest)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadUserDataRequest();
                                if (d.customer_id != null) {
                                    m.customer_id = String(d.customer_id);
                                }
                                if (d.operations) {
                                    if (!Array.isArray(d.operations))
                                        throw TypeError(".google.ads.googleads.v11.services.UploadUserDataRequest.operations: array expected");
                                    m.operations = [];
                                    for (var i = 0; i < d.operations.length; ++i) {
                                        if (typeof d.operations[i] !== "object")
                                            throw TypeError(".google.ads.googleads.v11.services.UploadUserDataRequest.operations: object expected");
                                        m.operations[i] = $root.google.ads.googleads.v11.services.UserDataOperation.fromObject(d.operations[i]);
                                    }
                                }
                                if (d.customer_match_user_list_metadata != null) {
                                    if (typeof d.customer_match_user_list_metadata !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UploadUserDataRequest.customer_match_user_list_metadata: object expected");
                                    m.customer_match_user_list_metadata = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.fromObject(d.customer_match_user_list_metadata);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadUserDataRequest message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadUserDataRequest} m UploadUserDataRequest
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadUserDataRequest.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (o.arrays || o.defaults) {
                                    d.operations = [];
                                }
                                if (o.defaults) {
                                    d.customer_id = "";
                                }
                                if (m.customer_id != null && m.hasOwnProperty("customer_id")) {
                                    d.customer_id = m.customer_id;
                                }
                                if (m.customer_match_user_list_metadata != null && m.hasOwnProperty("customer_match_user_list_metadata")) {
                                    d.customer_match_user_list_metadata = $root.google.ads.googleads.v11.common.CustomerMatchUserListMetadata.toObject(m.customer_match_user_list_metadata, o);
                                    if (o.oneofs)
                                        d.metadata = "customer_match_user_list_metadata";
                                }
                                if (m.operations && m.operations.length) {
                                    d.operations = [];
                                    for (var j = 0; j < m.operations.length; ++j) {
                                        d.operations[j] = $root.google.ads.googleads.v11.services.UserDataOperation.toObject(m.operations[j], o);
                                    }
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadUserDataRequest to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadUserDataRequest
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadUserDataRequest.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadUserDataRequest;
                        })();
    
                        services.UserDataOperation = (function() {
    
                            /**
                             * Properties of a UserDataOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUserDataOperation
                             * @property {google.ads.googleads.v11.common.IUserData|null} [create] UserDataOperation create
                             * @property {google.ads.googleads.v11.common.IUserData|null} [remove] UserDataOperation remove
                             */
    
                            /**
                             * Constructs a new UserDataOperation.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents a UserDataOperation.
                             * @implements IUserDataOperation
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUserDataOperation=} [p] Properties to set
                             */
                            function UserDataOperation(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UserDataOperation create.
                             * @member {google.ads.googleads.v11.common.IUserData|null|undefined} create
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @instance
                             */
                            UserDataOperation.prototype.create = null;
    
                            /**
                             * UserDataOperation remove.
                             * @member {google.ads.googleads.v11.common.IUserData|null|undefined} remove
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @instance
                             */
                            UserDataOperation.prototype.remove = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UserDataOperation operation.
                             * @member {"create"|"remove"|undefined} operation
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @instance
                             */
                            Object.defineProperty(UserDataOperation.prototype, "operation", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "remove"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UserDataOperation instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IUserDataOperation=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UserDataOperation} UserDataOperation instance
                             */
                            UserDataOperation.create = function create(properties) {
                                return new UserDataOperation(properties);
                            };
    
                            /**
                             * Encodes the specified UserDataOperation message. Does not implicitly {@link google.ads.googleads.v11.services.UserDataOperation.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IUserDataOperation} m UserDataOperation message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserDataOperation.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.create != null && Object.hasOwnProperty.call(m, "create"))
                                    $root.google.ads.googleads.v11.common.UserData.encode(m.create, w.uint32(10).fork()).ldelim();
                                if (m.remove != null && Object.hasOwnProperty.call(m, "remove"))
                                    $root.google.ads.googleads.v11.common.UserData.encode(m.remove, w.uint32(18).fork()).ldelim();
                                return w;
                            };
    
                            /**
                             * Encodes the specified UserDataOperation message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UserDataOperation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.IUserDataOperation} message UserDataOperation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UserDataOperation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a UserDataOperation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UserDataOperation} UserDataOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserDataOperation.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UserDataOperation();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 1:
                                        m.create = $root.google.ads.googleads.v11.common.UserData.decode(r, r.uint32());
                                        break;
                                    case 2:
                                        m.remove = $root.google.ads.googleads.v11.common.UserData.decode(r, r.uint32());
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes a UserDataOperation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UserDataOperation} UserDataOperation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UserDataOperation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a UserDataOperation message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UserDataOperation.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserData.verify(m.create);
                                        if (e)
                                            return "create." + e;
                                    }
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    if (p.operation === 1)
                                        return "operation: multiple values";
                                    p.operation = 1;
                                    {
                                        var e = $root.google.ads.googleads.v11.common.UserData.verify(m.remove);
                                        if (e)
                                            return "remove." + e;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a UserDataOperation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UserDataOperation} UserDataOperation
                             */
                            UserDataOperation.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UserDataOperation)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UserDataOperation();
                                if (d.create != null) {
                                    if (typeof d.create !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UserDataOperation.create: object expected");
                                    m.create = $root.google.ads.googleads.v11.common.UserData.fromObject(d.create);
                                }
                                if (d.remove != null) {
                                    if (typeof d.remove !== "object")
                                        throw TypeError(".google.ads.googleads.v11.services.UserDataOperation.remove: object expected");
                                    m.remove = $root.google.ads.googleads.v11.common.UserData.fromObject(d.remove);
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from a UserDataOperation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @static
                             * @param {google.ads.googleads.v11.services.UserDataOperation} m UserDataOperation
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UserDataOperation.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.create != null && m.hasOwnProperty("create")) {
                                    d.create = $root.google.ads.googleads.v11.common.UserData.toObject(m.create, o);
                                    if (o.oneofs)
                                        d.operation = "create";
                                }
                                if (m.remove != null && m.hasOwnProperty("remove")) {
                                    d.remove = $root.google.ads.googleads.v11.common.UserData.toObject(m.remove, o);
                                    if (o.oneofs)
                                        d.operation = "remove";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UserDataOperation to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UserDataOperation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UserDataOperation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UserDataOperation;
                        })();
    
                        services.UploadUserDataResponse = (function() {
    
                            /**
                             * Properties of an UploadUserDataResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @interface IUploadUserDataResponse
                             * @property {string|null} [upload_date_time] UploadUserDataResponse upload_date_time
                             * @property {number|null} [received_operations_count] UploadUserDataResponse received_operations_count
                             */
    
                            /**
                             * Constructs a new UploadUserDataResponse.
                             * @memberof google.ads.googleads.v11.services
                             * @classdesc Represents an UploadUserDataResponse.
                             * @implements IUploadUserDataResponse
                             * @constructor
                             * @param {google.ads.googleads.v11.services.IUploadUserDataResponse=} [p] Properties to set
                             */
                            function UploadUserDataResponse(p) {
                                if (p)
                                    for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                        if (p[ks[i]] != null)
                                            this[ks[i]] = p[ks[i]];
                            }
    
                            /**
                             * UploadUserDataResponse upload_date_time.
                             * @member {string|null|undefined} upload_date_time
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @instance
                             */
                            UploadUserDataResponse.prototype.upload_date_time = null;
    
                            /**
                             * UploadUserDataResponse received_operations_count.
                             * @member {number|null|undefined} received_operations_count
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @instance
                             */
                            UploadUserDataResponse.prototype.received_operations_count = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * UploadUserDataResponse _upload_date_time.
                             * @member {"upload_date_time"|undefined} _upload_date_time
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @instance
                             */
                            Object.defineProperty(UploadUserDataResponse.prototype, "_upload_date_time", {
                                get: $util.oneOfGetter($oneOfFields = ["upload_date_time"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * UploadUserDataResponse _received_operations_count.
                             * @member {"received_operations_count"|undefined} _received_operations_count
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @instance
                             */
                            Object.defineProperty(UploadUserDataResponse.prototype, "_received_operations_count", {
                                get: $util.oneOfGetter($oneOfFields = ["received_operations_count"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new UploadUserDataResponse instance using the specified properties.
                             * @function create
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadUserDataResponse=} [properties] Properties to set
                             * @returns {google.ads.googleads.v11.services.UploadUserDataResponse} UploadUserDataResponse instance
                             */
                            UploadUserDataResponse.create = function create(properties) {
                                return new UploadUserDataResponse(properties);
                            };
    
                            /**
                             * Encodes the specified UploadUserDataResponse message. Does not implicitly {@link google.ads.googleads.v11.services.UploadUserDataResponse.verify|verify} messages.
                             * @function encode
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadUserDataResponse} m UploadUserDataResponse message or plain object to encode
                             * @param {$protobuf.Writer} [w] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadUserDataResponse.encode = function encode(m, w) {
                                if (!w)
                                    w = $Writer.create();
                                if (m.upload_date_time != null && Object.hasOwnProperty.call(m, "upload_date_time"))
                                    w.uint32(26).string(m.upload_date_time);
                                if (m.received_operations_count != null && Object.hasOwnProperty.call(m, "received_operations_count"))
                                    w.uint32(32).int32(m.received_operations_count);
                                return w;
                            };
    
                            /**
                             * Encodes the specified UploadUserDataResponse message, length delimited. Does not implicitly {@link google.ads.googleads.v11.services.UploadUserDataResponse.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.IUploadUserDataResponse} message UploadUserDataResponse message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            UploadUserDataResponse.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an UploadUserDataResponse message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                             * @param {number} [l] Message length if known beforehand
                             * @returns {google.ads.googleads.v11.services.UploadUserDataResponse} UploadUserDataResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadUserDataResponse.decode = function decode(r, l) {
                                if (!(r instanceof $Reader))
                                    r = $Reader.create(r);
                                var c = l === undefined ? r.len : r.pos + l, m = new $root.google.ads.googleads.v11.services.UploadUserDataResponse();
                                while (r.pos < c) {
                                    var t = r.uint32();
                                    switch (t >>> 3) {
                                    case 3:
                                        m.upload_date_time = r.string();
                                        break;
                                    case 4:
                                        m.received_operations_count = r.int32();
                                        break;
                                    default:
                                        r.skipType(t & 7);
                                        break;
                                    }
                                }
                                return m;
                            };
    
                            /**
                             * Decodes an UploadUserDataResponse message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.ads.googleads.v11.services.UploadUserDataResponse} UploadUserDataResponse
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            UploadUserDataResponse.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an UploadUserDataResponse message.
                             * @function verify
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {Object.<string,*>} m Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            UploadUserDataResponse.verify = function verify(m) {
                                if (typeof m !== "object" || m === null)
                                    return "object expected";
                                var p = {};
                                if (m.upload_date_time != null && m.hasOwnProperty("upload_date_time")) {
                                    p._upload_date_time = 1;
                                    if (!$util.isString(m.upload_date_time))
                                        return "upload_date_time: string expected";
                                }
                                if (m.received_operations_count != null && m.hasOwnProperty("received_operations_count")) {
                                    p._received_operations_count = 1;
                                    if (!$util.isInteger(m.received_operations_count))
                                        return "received_operations_count: integer expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates an UploadUserDataResponse message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {Object.<string,*>} d Plain object
                             * @returns {google.ads.googleads.v11.services.UploadUserDataResponse} UploadUserDataResponse
                             */
                            UploadUserDataResponse.fromObject = function fromObject(d) {
                                if (d instanceof $root.google.ads.googleads.v11.services.UploadUserDataResponse)
                                    return d;
                                var m = new $root.google.ads.googleads.v11.services.UploadUserDataResponse();
                                if (d.upload_date_time != null) {
                                    m.upload_date_time = String(d.upload_date_time);
                                }
                                if (d.received_operations_count != null) {
                                    m.received_operations_count = d.received_operations_count | 0;
                                }
                                return m;
                            };
    
                            /**
                             * Creates a plain object from an UploadUserDataResponse message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @static
                             * @param {google.ads.googleads.v11.services.UploadUserDataResponse} m UploadUserDataResponse
                             * @param {$protobuf.IConversionOptions} [o] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            UploadUserDataResponse.toObject = function toObject(m, o) {
                                if (!o)
                                    o = {};
                                var d = {};
                                if (m.upload_date_time != null && m.hasOwnProperty("upload_date_time")) {
                                    d.upload_date_time = m.upload_date_time;
                                    if (o.oneofs)
                                        d._upload_date_time = "upload_date_time";
                                }
                                if (m.received_operations_count != null && m.hasOwnProperty("received_operations_count")) {
                                    d.received_operations_count = m.received_operations_count;
                                    if (o.oneofs)
                                        d._received_operations_count = "received_operations_count";
                                }
                                return d;
                            };
    
                            /**
                             * Converts this UploadUserDataResponse to JSON.
                             * @function toJSON
                             * @memberof google.ads.googleads.v11.services.UploadUserDataResponse
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            UploadUserDataResponse.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            return UploadUserDataResponse;
                        })();
    
                        return services;
                    })();
    
                    return v11;
                })();
    
                return googleads;
            })();
    
            return ads;
        })();
    
        google.api = (function() {
    
            /**
             * Namespace api.
             * @memberof google
             * @namespace
             */
            var api = {};
    
            /**
             * FieldBehavior enum.
             * @name google.api.FieldBehavior
             * @enum {number}
             * @property {number} FIELD_BEHAVIOR_UNSPECIFIED=0 FIELD_BEHAVIOR_UNSPECIFIED value
             * @property {number} OPTIONAL=1 OPTIONAL value
             * @property {number} REQUIRED=2 REQUIRED value
             * @property {number} OUTPUT_ONLY=3 OUTPUT_ONLY value
             * @property {number} INPUT_ONLY=4 INPUT_ONLY value
             * @property {number} IMMUTABLE=5 IMMUTABLE value
             * @property {number} UNORDERED_LIST=6 UNORDERED_LIST value
             * @property {number} NON_EMPTY_DEFAULT=7 NON_EMPTY_DEFAULT value
             */
            api.FieldBehavior = (function() {
                var valuesById = {}, values = Object.create(valuesById);
                values[valuesById[0] = "FIELD_BEHAVIOR_UNSPECIFIED"] = 0;
                values[valuesById[1] = "OPTIONAL"] = 1;
                values[valuesById[2] = "REQUIRED"] = 2;
                values[valuesById[3] = "OUTPUT_ONLY"] = 3;
                values[valuesById[4] = "INPUT_ONLY"] = 4;
                values[valuesById[5] = "IMMUTABLE"] = 5;
                values[valuesById[6] = "UNORDERED_LIST"] = 6;
                values[valuesById[7] = "NON_EMPTY_DEFAULT"] = 7;
                return values;
            })();
    
            api.ResourceDescriptor = (function() {
    
                /**
                 * Properties of a ResourceDescriptor.
                 * @memberof google.api
                 * @interface IResourceDescriptor
                 * @property {string|null} [type] ResourceDescriptor type
                 * @property {Array.<string>|null} [pattern] ResourceDescriptor pattern
                 * @property {string|null} [name_field] ResourceDescriptor name_field
                 * @property {google.api.ResourceDescriptor.History|null} [history] ResourceDescriptor history
                 * @property {string|null} [plural] ResourceDescriptor plural
                 * @property {string|null} [singular] ResourceDescriptor singular
                 * @property {Array.<google.api.ResourceDescriptor.Style>|null} [style] ResourceDescriptor style
                 */
    
                /**
                 * Constructs a new ResourceDescriptor.
                 * @memberof google.api
                 * @classdesc Represents a ResourceDescriptor.
                 * @implements IResourceDescriptor
                 * @constructor
                 * @param {google.api.IResourceDescriptor=} [p] Properties to set
                 */
                function ResourceDescriptor(p) {
                    this.pattern = [];
                    this.style = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * ResourceDescriptor type.
                 * @member {string} type
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.type = "";
    
                /**
                 * ResourceDescriptor pattern.
                 * @member {Array.<string>} pattern
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.pattern = $util.emptyArray;
    
                /**
                 * ResourceDescriptor name_field.
                 * @member {string} name_field
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.name_field = "";
    
                /**
                 * ResourceDescriptor history.
                 * @member {google.api.ResourceDescriptor.History} history
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.history = 0;
    
                /**
                 * ResourceDescriptor plural.
                 * @member {string} plural
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.plural = "";
    
                /**
                 * ResourceDescriptor singular.
                 * @member {string} singular
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.singular = "";
    
                /**
                 * ResourceDescriptor style.
                 * @member {Array.<google.api.ResourceDescriptor.Style>} style
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.style = $util.emptyArray;
    
                /**
                 * Creates a new ResourceDescriptor instance using the specified properties.
                 * @function create
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.IResourceDescriptor=} [properties] Properties to set
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor instance
                 */
                ResourceDescriptor.create = function create(properties) {
                    return new ResourceDescriptor(properties);
                };
    
                /**
                 * Encodes the specified ResourceDescriptor message. Does not implicitly {@link google.api.ResourceDescriptor.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.IResourceDescriptor} m ResourceDescriptor message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceDescriptor.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                        w.uint32(10).string(m.type);
                    if (m.pattern != null && m.pattern.length) {
                        for (var i = 0; i < m.pattern.length; ++i)
                            w.uint32(18).string(m.pattern[i]);
                    }
                    if (m.name_field != null && Object.hasOwnProperty.call(m, "name_field"))
                        w.uint32(26).string(m.name_field);
                    if (m.history != null && Object.hasOwnProperty.call(m, "history"))
                        w.uint32(32).int32(m.history);
                    if (m.plural != null && Object.hasOwnProperty.call(m, "plural"))
                        w.uint32(42).string(m.plural);
                    if (m.singular != null && Object.hasOwnProperty.call(m, "singular"))
                        w.uint32(50).string(m.singular);
                    if (m.style != null && m.style.length) {
                        w.uint32(82).fork();
                        for (var i = 0; i < m.style.length; ++i)
                            w.int32(m.style[i]);
                        w.ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified ResourceDescriptor message, length delimited. Does not implicitly {@link google.api.ResourceDescriptor.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.IResourceDescriptor} message ResourceDescriptor message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceDescriptor.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ResourceDescriptor message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceDescriptor.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.api.ResourceDescriptor();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.type = r.string();
                            break;
                        case 2:
                            if (!(m.pattern && m.pattern.length))
                                m.pattern = [];
                            m.pattern.push(r.string());
                            break;
                        case 3:
                            m.name_field = r.string();
                            break;
                        case 4:
                            m.history = r.int32();
                            break;
                        case 5:
                            m.plural = r.string();
                            break;
                        case 6:
                            m.singular = r.string();
                            break;
                        case 10:
                            if (!(m.style && m.style.length))
                                m.style = [];
                            if ((t & 7) === 2) {
                                var c2 = r.uint32() + r.pos;
                                while (r.pos < c2)
                                    m.style.push(r.int32());
                            } else
                                m.style.push(r.int32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a ResourceDescriptor message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceDescriptor.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ResourceDescriptor message.
                 * @function verify
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ResourceDescriptor.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.type != null && m.hasOwnProperty("type")) {
                        if (!$util.isString(m.type))
                            return "type: string expected";
                    }
                    if (m.pattern != null && m.hasOwnProperty("pattern")) {
                        if (!Array.isArray(m.pattern))
                            return "pattern: array expected";
                        for (var i = 0; i < m.pattern.length; ++i) {
                            if (!$util.isString(m.pattern[i]))
                                return "pattern: string[] expected";
                        }
                    }
                    if (m.name_field != null && m.hasOwnProperty("name_field")) {
                        if (!$util.isString(m.name_field))
                            return "name_field: string expected";
                    }
                    if (m.history != null && m.hasOwnProperty("history")) {
                        switch (m.history) {
                        default:
                            return "history: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    }
                    if (m.plural != null && m.hasOwnProperty("plural")) {
                        if (!$util.isString(m.plural))
                            return "plural: string expected";
                    }
                    if (m.singular != null && m.hasOwnProperty("singular")) {
                        if (!$util.isString(m.singular))
                            return "singular: string expected";
                    }
                    if (m.style != null && m.hasOwnProperty("style")) {
                        if (!Array.isArray(m.style))
                            return "style: array expected";
                        for (var i = 0; i < m.style.length; ++i) {
                            switch (m.style[i]) {
                            default:
                                return "style: enum value[] expected";
                            case 0:
                            case 1:
                                break;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a ResourceDescriptor message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor
                 */
                ResourceDescriptor.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.api.ResourceDescriptor)
                        return d;
                    var m = new $root.google.api.ResourceDescriptor();
                    if (d.type != null) {
                        m.type = String(d.type);
                    }
                    if (d.pattern) {
                        if (!Array.isArray(d.pattern))
                            throw TypeError(".google.api.ResourceDescriptor.pattern: array expected");
                        m.pattern = [];
                        for (var i = 0; i < d.pattern.length; ++i) {
                            m.pattern[i] = String(d.pattern[i]);
                        }
                    }
                    if (d.name_field != null) {
                        m.name_field = String(d.name_field);
                    }
                    switch (d.history) {
                    case "HISTORY_UNSPECIFIED":
                    case 0:
                        m.history = 0;
                        break;
                    case "ORIGINALLY_SINGLE_PATTERN":
                    case 1:
                        m.history = 1;
                        break;
                    case "FUTURE_MULTI_PATTERN":
                    case 2:
                        m.history = 2;
                        break;
                    }
                    if (d.plural != null) {
                        m.plural = String(d.plural);
                    }
                    if (d.singular != null) {
                        m.singular = String(d.singular);
                    }
                    if (d.style) {
                        if (!Array.isArray(d.style))
                            throw TypeError(".google.api.ResourceDescriptor.style: array expected");
                        m.style = [];
                        for (var i = 0; i < d.style.length; ++i) {
                            switch (d.style[i]) {
                            default:
                            case "STYLE_UNSPECIFIED":
                            case 0:
                                m.style[i] = 0;
                                break;
                            case "DECLARATIVE_FRIENDLY":
                            case 1:
                                m.style[i] = 1;
                                break;
                            }
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a ResourceDescriptor message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.ResourceDescriptor} m ResourceDescriptor
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ResourceDescriptor.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.pattern = [];
                        d.style = [];
                    }
                    if (o.defaults) {
                        d.type = "";
                        d.name_field = "";
                        d.history = o.enums === String ? "HISTORY_UNSPECIFIED" : 0;
                        d.plural = "";
                        d.singular = "";
                    }
                    if (m.type != null && m.hasOwnProperty("type")) {
                        d.type = m.type;
                    }
                    if (m.pattern && m.pattern.length) {
                        d.pattern = [];
                        for (var j = 0; j < m.pattern.length; ++j) {
                            d.pattern[j] = m.pattern[j];
                        }
                    }
                    if (m.name_field != null && m.hasOwnProperty("name_field")) {
                        d.name_field = m.name_field;
                    }
                    if (m.history != null && m.hasOwnProperty("history")) {
                        d.history = o.enums === String ? $root.google.api.ResourceDescriptor.History[m.history] : m.history;
                    }
                    if (m.plural != null && m.hasOwnProperty("plural")) {
                        d.plural = m.plural;
                    }
                    if (m.singular != null && m.hasOwnProperty("singular")) {
                        d.singular = m.singular;
                    }
                    if (m.style && m.style.length) {
                        d.style = [];
                        for (var j = 0; j < m.style.length; ++j) {
                            d.style[j] = o.enums === String ? $root.google.api.ResourceDescriptor.Style[m.style[j]] : m.style[j];
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this ResourceDescriptor to JSON.
                 * @function toJSON
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ResourceDescriptor.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * History enum.
                 * @name google.api.ResourceDescriptor.History
                 * @enum {number}
                 * @property {number} HISTORY_UNSPECIFIED=0 HISTORY_UNSPECIFIED value
                 * @property {number} ORIGINALLY_SINGLE_PATTERN=1 ORIGINALLY_SINGLE_PATTERN value
                 * @property {number} FUTURE_MULTI_PATTERN=2 FUTURE_MULTI_PATTERN value
                 */
                ResourceDescriptor.History = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "HISTORY_UNSPECIFIED"] = 0;
                    values[valuesById[1] = "ORIGINALLY_SINGLE_PATTERN"] = 1;
                    values[valuesById[2] = "FUTURE_MULTI_PATTERN"] = 2;
                    return values;
                })();
    
                /**
                 * Style enum.
                 * @name google.api.ResourceDescriptor.Style
                 * @enum {number}
                 * @property {number} STYLE_UNSPECIFIED=0 STYLE_UNSPECIFIED value
                 * @property {number} DECLARATIVE_FRIENDLY=1 DECLARATIVE_FRIENDLY value
                 */
                ResourceDescriptor.Style = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "STYLE_UNSPECIFIED"] = 0;
                    values[valuesById[1] = "DECLARATIVE_FRIENDLY"] = 1;
                    return values;
                })();
    
                return ResourceDescriptor;
            })();
    
            api.ResourceReference = (function() {
    
                /**
                 * Properties of a ResourceReference.
                 * @memberof google.api
                 * @interface IResourceReference
                 * @property {string|null} [type] ResourceReference type
                 * @property {string|null} [child_type] ResourceReference child_type
                 */
    
                /**
                 * Constructs a new ResourceReference.
                 * @memberof google.api
                 * @classdesc Represents a ResourceReference.
                 * @implements IResourceReference
                 * @constructor
                 * @param {google.api.IResourceReference=} [p] Properties to set
                 */
                function ResourceReference(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * ResourceReference type.
                 * @member {string} type
                 * @memberof google.api.ResourceReference
                 * @instance
                 */
                ResourceReference.prototype.type = "";
    
                /**
                 * ResourceReference child_type.
                 * @member {string} child_type
                 * @memberof google.api.ResourceReference
                 * @instance
                 */
                ResourceReference.prototype.child_type = "";
    
                /**
                 * Creates a new ResourceReference instance using the specified properties.
                 * @function create
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.IResourceReference=} [properties] Properties to set
                 * @returns {google.api.ResourceReference} ResourceReference instance
                 */
                ResourceReference.create = function create(properties) {
                    return new ResourceReference(properties);
                };
    
                /**
                 * Encodes the specified ResourceReference message. Does not implicitly {@link google.api.ResourceReference.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.IResourceReference} m ResourceReference message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceReference.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                        w.uint32(10).string(m.type);
                    if (m.child_type != null && Object.hasOwnProperty.call(m, "child_type"))
                        w.uint32(18).string(m.child_type);
                    return w;
                };
    
                /**
                 * Encodes the specified ResourceReference message, length delimited. Does not implicitly {@link google.api.ResourceReference.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.IResourceReference} message ResourceReference message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceReference.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ResourceReference message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.api.ResourceReference} ResourceReference
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceReference.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.api.ResourceReference();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.type = r.string();
                            break;
                        case 2:
                            m.child_type = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a ResourceReference message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.ResourceReference} ResourceReference
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceReference.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ResourceReference message.
                 * @function verify
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ResourceReference.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.type != null && m.hasOwnProperty("type")) {
                        if (!$util.isString(m.type))
                            return "type: string expected";
                    }
                    if (m.child_type != null && m.hasOwnProperty("child_type")) {
                        if (!$util.isString(m.child_type))
                            return "child_type: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a ResourceReference message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.api.ResourceReference} ResourceReference
                 */
                ResourceReference.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.api.ResourceReference)
                        return d;
                    var m = new $root.google.api.ResourceReference();
                    if (d.type != null) {
                        m.type = String(d.type);
                    }
                    if (d.child_type != null) {
                        m.child_type = String(d.child_type);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a ResourceReference message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.ResourceReference} m ResourceReference
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ResourceReference.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.type = "";
                        d.child_type = "";
                    }
                    if (m.type != null && m.hasOwnProperty("type")) {
                        d.type = m.type;
                    }
                    if (m.child_type != null && m.hasOwnProperty("child_type")) {
                        d.child_type = m.child_type;
                    }
                    return d;
                };
    
                /**
                 * Converts this ResourceReference to JSON.
                 * @function toJSON
                 * @memberof google.api.ResourceReference
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ResourceReference.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return ResourceReference;
            })();
    
            api.Http = (function() {
    
                /**
                 * Properties of a Http.
                 * @memberof google.api
                 * @interface IHttp
                 * @property {Array.<google.api.IHttpRule>|null} [rules] Http rules
                 * @property {boolean|null} [fully_decode_reserved_expansion] Http fully_decode_reserved_expansion
                 */
    
                /**
                 * Constructs a new Http.
                 * @memberof google.api
                 * @classdesc Represents a Http.
                 * @implements IHttp
                 * @constructor
                 * @param {google.api.IHttp=} [p] Properties to set
                 */
                function Http(p) {
                    this.rules = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * Http rules.
                 * @member {Array.<google.api.IHttpRule>} rules
                 * @memberof google.api.Http
                 * @instance
                 */
                Http.prototype.rules = $util.emptyArray;
    
                /**
                 * Http fully_decode_reserved_expansion.
                 * @member {boolean} fully_decode_reserved_expansion
                 * @memberof google.api.Http
                 * @instance
                 */
                Http.prototype.fully_decode_reserved_expansion = false;
    
                /**
                 * Creates a new Http instance using the specified properties.
                 * @function create
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.IHttp=} [properties] Properties to set
                 * @returns {google.api.Http} Http instance
                 */
                Http.create = function create(properties) {
                    return new Http(properties);
                };
    
                /**
                 * Encodes the specified Http message. Does not implicitly {@link google.api.Http.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.IHttp} m Http message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Http.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.rules != null && m.rules.length) {
                        for (var i = 0; i < m.rules.length; ++i)
                            $root.google.api.HttpRule.encode(m.rules[i], w.uint32(10).fork()).ldelim();
                    }
                    if (m.fully_decode_reserved_expansion != null && Object.hasOwnProperty.call(m, "fully_decode_reserved_expansion"))
                        w.uint32(16).bool(m.fully_decode_reserved_expansion);
                    return w;
                };
    
                /**
                 * Encodes the specified Http message, length delimited. Does not implicitly {@link google.api.Http.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.IHttp} message Http message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Http.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Http message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.Http
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.api.Http} Http
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Http.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.api.Http();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            if (!(m.rules && m.rules.length))
                                m.rules = [];
                            m.rules.push($root.google.api.HttpRule.decode(r, r.uint32()));
                            break;
                        case 2:
                            m.fully_decode_reserved_expansion = r.bool();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a Http message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.Http
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.Http} Http
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Http.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Http message.
                 * @function verify
                 * @memberof google.api.Http
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Http.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.rules != null && m.hasOwnProperty("rules")) {
                        if (!Array.isArray(m.rules))
                            return "rules: array expected";
                        for (var i = 0; i < m.rules.length; ++i) {
                            {
                                var e = $root.google.api.HttpRule.verify(m.rules[i]);
                                if (e)
                                    return "rules." + e;
                            }
                        }
                    }
                    if (m.fully_decode_reserved_expansion != null && m.hasOwnProperty("fully_decode_reserved_expansion")) {
                        if (typeof m.fully_decode_reserved_expansion !== "boolean")
                            return "fully_decode_reserved_expansion: boolean expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a Http message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.Http
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.api.Http} Http
                 */
                Http.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.api.Http)
                        return d;
                    var m = new $root.google.api.Http();
                    if (d.rules) {
                        if (!Array.isArray(d.rules))
                            throw TypeError(".google.api.Http.rules: array expected");
                        m.rules = [];
                        for (var i = 0; i < d.rules.length; ++i) {
                            if (typeof d.rules[i] !== "object")
                                throw TypeError(".google.api.Http.rules: object expected");
                            m.rules[i] = $root.google.api.HttpRule.fromObject(d.rules[i]);
                        }
                    }
                    if (d.fully_decode_reserved_expansion != null) {
                        m.fully_decode_reserved_expansion = Boolean(d.fully_decode_reserved_expansion);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a Http message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.Http} m Http
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Http.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.rules = [];
                    }
                    if (o.defaults) {
                        d.fully_decode_reserved_expansion = false;
                    }
                    if (m.rules && m.rules.length) {
                        d.rules = [];
                        for (var j = 0; j < m.rules.length; ++j) {
                            d.rules[j] = $root.google.api.HttpRule.toObject(m.rules[j], o);
                        }
                    }
                    if (m.fully_decode_reserved_expansion != null && m.hasOwnProperty("fully_decode_reserved_expansion")) {
                        d.fully_decode_reserved_expansion = m.fully_decode_reserved_expansion;
                    }
                    return d;
                };
    
                /**
                 * Converts this Http to JSON.
                 * @function toJSON
                 * @memberof google.api.Http
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Http.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return Http;
            })();
    
            api.HttpRule = (function() {
    
                /**
                 * Properties of a HttpRule.
                 * @memberof google.api
                 * @interface IHttpRule
                 * @property {string|null} [selector] HttpRule selector
                 * @property {string|null} [get] HttpRule get
                 * @property {string|null} [put] HttpRule put
                 * @property {string|null} [post] HttpRule post
                 * @property {string|null} ["delete"] HttpRule delete
                 * @property {string|null} [patch] HttpRule patch
                 * @property {google.api.ICustomHttpPattern|null} [custom] HttpRule custom
                 * @property {string|null} [body] HttpRule body
                 * @property {string|null} [response_body] HttpRule response_body
                 * @property {Array.<google.api.IHttpRule>|null} [additional_bindings] HttpRule additional_bindings
                 */
    
                /**
                 * Constructs a new HttpRule.
                 * @memberof google.api
                 * @classdesc Represents a HttpRule.
                 * @implements IHttpRule
                 * @constructor
                 * @param {google.api.IHttpRule=} [p] Properties to set
                 */
                function HttpRule(p) {
                    this.additional_bindings = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * HttpRule selector.
                 * @member {string} selector
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.selector = "";
    
                /**
                 * HttpRule get.
                 * @member {string|null|undefined} get
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.get = null;
    
                /**
                 * HttpRule put.
                 * @member {string|null|undefined} put
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.put = null;
    
                /**
                 * HttpRule post.
                 * @member {string|null|undefined} post
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.post = null;
    
                /**
                 * HttpRule delete.
                 * @member {string|null|undefined} delete
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype["delete"] = null;
    
                /**
                 * HttpRule patch.
                 * @member {string|null|undefined} patch
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.patch = null;
    
                /**
                 * HttpRule custom.
                 * @member {google.api.ICustomHttpPattern|null|undefined} custom
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.custom = null;
    
                /**
                 * HttpRule body.
                 * @member {string} body
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.body = "";
    
                /**
                 * HttpRule response_body.
                 * @member {string} response_body
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.response_body = "";
    
                /**
                 * HttpRule additional_bindings.
                 * @member {Array.<google.api.IHttpRule>} additional_bindings
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.additional_bindings = $util.emptyArray;
    
                // OneOf field names bound to virtual getters and setters
                var $oneOfFields;
    
                /**
                 * HttpRule pattern.
                 * @member {"get"|"put"|"post"|"delete"|"patch"|"custom"|undefined} pattern
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                Object.defineProperty(HttpRule.prototype, "pattern", {
                    get: $util.oneOfGetter($oneOfFields = ["get", "put", "post", "delete", "patch", "custom"]),
                    set: $util.oneOfSetter($oneOfFields)
                });
    
                /**
                 * Creates a new HttpRule instance using the specified properties.
                 * @function create
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.IHttpRule=} [properties] Properties to set
                 * @returns {google.api.HttpRule} HttpRule instance
                 */
                HttpRule.create = function create(properties) {
                    return new HttpRule(properties);
                };
    
                /**
                 * Encodes the specified HttpRule message. Does not implicitly {@link google.api.HttpRule.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.IHttpRule} m HttpRule message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                HttpRule.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.selector != null && Object.hasOwnProperty.call(m, "selector"))
                        w.uint32(10).string(m.selector);
                    if (m.get != null && Object.hasOwnProperty.call(m, "get"))
                        w.uint32(18).string(m.get);
                    if (m.put != null && Object.hasOwnProperty.call(m, "put"))
                        w.uint32(26).string(m.put);
                    if (m.post != null && Object.hasOwnProperty.call(m, "post"))
                        w.uint32(34).string(m.post);
                    if (m["delete"] != null && Object.hasOwnProperty.call(m, "delete"))
                        w.uint32(42).string(m["delete"]);
                    if (m.patch != null && Object.hasOwnProperty.call(m, "patch"))
                        w.uint32(50).string(m.patch);
                    if (m.body != null && Object.hasOwnProperty.call(m, "body"))
                        w.uint32(58).string(m.body);
                    if (m.custom != null && Object.hasOwnProperty.call(m, "custom"))
                        $root.google.api.CustomHttpPattern.encode(m.custom, w.uint32(66).fork()).ldelim();
                    if (m.additional_bindings != null && m.additional_bindings.length) {
                        for (var i = 0; i < m.additional_bindings.length; ++i)
                            $root.google.api.HttpRule.encode(m.additional_bindings[i], w.uint32(90).fork()).ldelim();
                    }
                    if (m.response_body != null && Object.hasOwnProperty.call(m, "response_body"))
                        w.uint32(98).string(m.response_body);
                    return w;
                };
    
                /**
                 * Encodes the specified HttpRule message, length delimited. Does not implicitly {@link google.api.HttpRule.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.IHttpRule} message HttpRule message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                HttpRule.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a HttpRule message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.api.HttpRule} HttpRule
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                HttpRule.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.api.HttpRule();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.selector = r.string();
                            break;
                        case 2:
                            m.get = r.string();
                            break;
                        case 3:
                            m.put = r.string();
                            break;
                        case 4:
                            m.post = r.string();
                            break;
                        case 5:
                            m["delete"] = r.string();
                            break;
                        case 6:
                            m.patch = r.string();
                            break;
                        case 8:
                            m.custom = $root.google.api.CustomHttpPattern.decode(r, r.uint32());
                            break;
                        case 7:
                            m.body = r.string();
                            break;
                        case 12:
                            m.response_body = r.string();
                            break;
                        case 11:
                            if (!(m.additional_bindings && m.additional_bindings.length))
                                m.additional_bindings = [];
                            m.additional_bindings.push($root.google.api.HttpRule.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a HttpRule message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.HttpRule} HttpRule
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                HttpRule.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a HttpRule message.
                 * @function verify
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                HttpRule.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    var p = {};
                    if (m.selector != null && m.hasOwnProperty("selector")) {
                        if (!$util.isString(m.selector))
                            return "selector: string expected";
                    }
                    if (m.get != null && m.hasOwnProperty("get")) {
                        p.pattern = 1;
                        if (!$util.isString(m.get))
                            return "get: string expected";
                    }
                    if (m.put != null && m.hasOwnProperty("put")) {
                        if (p.pattern === 1)
                            return "pattern: multiple values";
                        p.pattern = 1;
                        if (!$util.isString(m.put))
                            return "put: string expected";
                    }
                    if (m.post != null && m.hasOwnProperty("post")) {
                        if (p.pattern === 1)
                            return "pattern: multiple values";
                        p.pattern = 1;
                        if (!$util.isString(m.post))
                            return "post: string expected";
                    }
                    if (m["delete"] != null && m.hasOwnProperty("delete")) {
                        if (p.pattern === 1)
                            return "pattern: multiple values";
                        p.pattern = 1;
                        if (!$util.isString(m["delete"]))
                            return "delete: string expected";
                    }
                    if (m.patch != null && m.hasOwnProperty("patch")) {
                        if (p.pattern === 1)
                            return "pattern: multiple values";
                        p.pattern = 1;
                        if (!$util.isString(m.patch))
                            return "patch: string expected";
                    }
                    if (m.custom != null && m.hasOwnProperty("custom")) {
                        if (p.pattern === 1)
                            return "pattern: multiple values";
                        p.pattern = 1;
                        {
                            var e = $root.google.api.CustomHttpPattern.verify(m.custom);
                            if (e)
                                return "custom." + e;
                        }
                    }
                    if (m.body != null && m.hasOwnProperty("body")) {
                        if (!$util.isString(m.body))
                            return "body: string expected";
                    }
                    if (m.response_body != null && m.hasOwnProperty("response_body")) {
                        if (!$util.isString(m.response_body))
                            return "response_body: string expected";
                    }
                    if (m.additional_bindings != null && m.hasOwnProperty("additional_bindings")) {
                        if (!Array.isArray(m.additional_bindings))
                            return "additional_bindings: array expected";
                        for (var i = 0; i < m.additional_bindings.length; ++i) {
                            {
                                var e = $root.google.api.HttpRule.verify(m.additional_bindings[i]);
                                if (e)
                                    return "additional_bindings." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a HttpRule message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.api.HttpRule} HttpRule
                 */
                HttpRule.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.api.HttpRule)
                        return d;
                    var m = new $root.google.api.HttpRule();
                    if (d.selector != null) {
                        m.selector = String(d.selector);
                    }
                    if (d.get != null) {
                        m.get = String(d.get);
                    }
                    if (d.put != null) {
                        m.put = String(d.put);
                    }
                    if (d.post != null) {
                        m.post = String(d.post);
                    }
                    if (d["delete"] != null) {
                        m["delete"] = String(d["delete"]);
                    }
                    if (d.patch != null) {
                        m.patch = String(d.patch);
                    }
                    if (d.custom != null) {
                        if (typeof d.custom !== "object")
                            throw TypeError(".google.api.HttpRule.custom: object expected");
                        m.custom = $root.google.api.CustomHttpPattern.fromObject(d.custom);
                    }
                    if (d.body != null) {
                        m.body = String(d.body);
                    }
                    if (d.response_body != null) {
                        m.response_body = String(d.response_body);
                    }
                    if (d.additional_bindings) {
                        if (!Array.isArray(d.additional_bindings))
                            throw TypeError(".google.api.HttpRule.additional_bindings: array expected");
                        m.additional_bindings = [];
                        for (var i = 0; i < d.additional_bindings.length; ++i) {
                            if (typeof d.additional_bindings[i] !== "object")
                                throw TypeError(".google.api.HttpRule.additional_bindings: object expected");
                            m.additional_bindings[i] = $root.google.api.HttpRule.fromObject(d.additional_bindings[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a HttpRule message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.HttpRule} m HttpRule
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                HttpRule.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.additional_bindings = [];
                    }
                    if (o.defaults) {
                        d.selector = "";
                        d.body = "";
                        d.response_body = "";
                    }
                    if (m.selector != null && m.hasOwnProperty("selector")) {
                        d.selector = m.selector;
                    }
                    if (m.get != null && m.hasOwnProperty("get")) {
                        d.get = m.get;
                        if (o.oneofs)
                            d.pattern = "get";
                    }
                    if (m.put != null && m.hasOwnProperty("put")) {
                        d.put = m.put;
                        if (o.oneofs)
                            d.pattern = "put";
                    }
                    if (m.post != null && m.hasOwnProperty("post")) {
                        d.post = m.post;
                        if (o.oneofs)
                            d.pattern = "post";
                    }
                    if (m["delete"] != null && m.hasOwnProperty("delete")) {
                        d["delete"] = m["delete"];
                        if (o.oneofs)
                            d.pattern = "delete";
                    }
                    if (m.patch != null && m.hasOwnProperty("patch")) {
                        d.patch = m.patch;
                        if (o.oneofs)
                            d.pattern = "patch";
                    }
                    if (m.body != null && m.hasOwnProperty("body")) {
                        d.body = m.body;
                    }
                    if (m.custom != null && m.hasOwnProperty("custom")) {
                        d.custom = $root.google.api.CustomHttpPattern.toObject(m.custom, o);
                        if (o.oneofs)
                            d.pattern = "custom";
                    }
                    if (m.additional_bindings && m.additional_bindings.length) {
                        d.additional_bindings = [];
                        for (var j = 0; j < m.additional_bindings.length; ++j) {
                            d.additional_bindings[j] = $root.google.api.HttpRule.toObject(m.additional_bindings[j], o);
                        }
                    }
                    if (m.response_body != null && m.hasOwnProperty("response_body")) {
                        d.response_body = m.response_body;
                    }
                    return d;
                };
    
                /**
                 * Converts this HttpRule to JSON.
                 * @function toJSON
                 * @memberof google.api.HttpRule
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                HttpRule.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return HttpRule;
            })();
    
            api.CustomHttpPattern = (function() {
    
                /**
                 * Properties of a CustomHttpPattern.
                 * @memberof google.api
                 * @interface ICustomHttpPattern
                 * @property {string|null} [kind] CustomHttpPattern kind
                 * @property {string|null} [path] CustomHttpPattern path
                 */
    
                /**
                 * Constructs a new CustomHttpPattern.
                 * @memberof google.api
                 * @classdesc Represents a CustomHttpPattern.
                 * @implements ICustomHttpPattern
                 * @constructor
                 * @param {google.api.ICustomHttpPattern=} [p] Properties to set
                 */
                function CustomHttpPattern(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * CustomHttpPattern kind.
                 * @member {string} kind
                 * @memberof google.api.CustomHttpPattern
                 * @instance
                 */
                CustomHttpPattern.prototype.kind = "";
    
                /**
                 * CustomHttpPattern path.
                 * @member {string} path
                 * @memberof google.api.CustomHttpPattern
                 * @instance
                 */
                CustomHttpPattern.prototype.path = "";
    
                /**
                 * Creates a new CustomHttpPattern instance using the specified properties.
                 * @function create
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.ICustomHttpPattern=} [properties] Properties to set
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern instance
                 */
                CustomHttpPattern.create = function create(properties) {
                    return new CustomHttpPattern(properties);
                };
    
                /**
                 * Encodes the specified CustomHttpPattern message. Does not implicitly {@link google.api.CustomHttpPattern.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.ICustomHttpPattern} m CustomHttpPattern message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CustomHttpPattern.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.kind != null && Object.hasOwnProperty.call(m, "kind"))
                        w.uint32(10).string(m.kind);
                    if (m.path != null && Object.hasOwnProperty.call(m, "path"))
                        w.uint32(18).string(m.path);
                    return w;
                };
    
                /**
                 * Encodes the specified CustomHttpPattern message, length delimited. Does not implicitly {@link google.api.CustomHttpPattern.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.ICustomHttpPattern} message CustomHttpPattern message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CustomHttpPattern.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a CustomHttpPattern message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CustomHttpPattern.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.api.CustomHttpPattern();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.kind = r.string();
                            break;
                        case 2:
                            m.path = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a CustomHttpPattern message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CustomHttpPattern.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a CustomHttpPattern message.
                 * @function verify
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                CustomHttpPattern.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.kind != null && m.hasOwnProperty("kind")) {
                        if (!$util.isString(m.kind))
                            return "kind: string expected";
                    }
                    if (m.path != null && m.hasOwnProperty("path")) {
                        if (!$util.isString(m.path))
                            return "path: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a CustomHttpPattern message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern
                 */
                CustomHttpPattern.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.api.CustomHttpPattern)
                        return d;
                    var m = new $root.google.api.CustomHttpPattern();
                    if (d.kind != null) {
                        m.kind = String(d.kind);
                    }
                    if (d.path != null) {
                        m.path = String(d.path);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a CustomHttpPattern message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.CustomHttpPattern} m CustomHttpPattern
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                CustomHttpPattern.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.kind = "";
                        d.path = "";
                    }
                    if (m.kind != null && m.hasOwnProperty("kind")) {
                        d.kind = m.kind;
                    }
                    if (m.path != null && m.hasOwnProperty("path")) {
                        d.path = m.path;
                    }
                    return d;
                };
    
                /**
                 * Converts this CustomHttpPattern to JSON.
                 * @function toJSON
                 * @memberof google.api.CustomHttpPattern
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                CustomHttpPattern.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return CustomHttpPattern;
            })();
    
            return api;
        })();
    
        google.protobuf = (function() {
    
            /**
             * Namespace protobuf.
             * @memberof google
             * @namespace
             */
            var protobuf = {};
    
            protobuf.FileDescriptorSet = (function() {
    
                /**
                 * Properties of a FileDescriptorSet.
                 * @memberof google.protobuf
                 * @interface IFileDescriptorSet
                 * @property {Array.<google.protobuf.IFileDescriptorProto>|null} [file] FileDescriptorSet file
                 */
    
                /**
                 * Constructs a new FileDescriptorSet.
                 * @memberof google.protobuf
                 * @classdesc Represents a FileDescriptorSet.
                 * @implements IFileDescriptorSet
                 * @constructor
                 * @param {google.protobuf.IFileDescriptorSet=} [p] Properties to set
                 */
                function FileDescriptorSet(p) {
                    this.file = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * FileDescriptorSet file.
                 * @member {Array.<google.protobuf.IFileDescriptorProto>} file
                 * @memberof google.protobuf.FileDescriptorSet
                 * @instance
                 */
                FileDescriptorSet.prototype.file = $util.emptyArray;
    
                /**
                 * Creates a new FileDescriptorSet instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.IFileDescriptorSet=} [properties] Properties to set
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet instance
                 */
                FileDescriptorSet.create = function create(properties) {
                    return new FileDescriptorSet(properties);
                };
    
                /**
                 * Encodes the specified FileDescriptorSet message. Does not implicitly {@link google.protobuf.FileDescriptorSet.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.IFileDescriptorSet} m FileDescriptorSet message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorSet.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.file != null && m.file.length) {
                        for (var i = 0; i < m.file.length; ++i)
                            $root.google.protobuf.FileDescriptorProto.encode(m.file[i], w.uint32(10).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified FileDescriptorSet message, length delimited. Does not implicitly {@link google.protobuf.FileDescriptorSet.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.IFileDescriptorSet} message FileDescriptorSet message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorSet.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FileDescriptorSet message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorSet.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.FileDescriptorSet();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            if (!(m.file && m.file.length))
                                m.file = [];
                            m.file.push($root.google.protobuf.FileDescriptorProto.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a FileDescriptorSet message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorSet.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FileDescriptorSet message.
                 * @function verify
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FileDescriptorSet.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.file != null && m.hasOwnProperty("file")) {
                        if (!Array.isArray(m.file))
                            return "file: array expected";
                        for (var i = 0; i < m.file.length; ++i) {
                            {
                                var e = $root.google.protobuf.FileDescriptorProto.verify(m.file[i]);
                                if (e)
                                    return "file." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a FileDescriptorSet message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet
                 */
                FileDescriptorSet.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.FileDescriptorSet)
                        return d;
                    var m = new $root.google.protobuf.FileDescriptorSet();
                    if (d.file) {
                        if (!Array.isArray(d.file))
                            throw TypeError(".google.protobuf.FileDescriptorSet.file: array expected");
                        m.file = [];
                        for (var i = 0; i < d.file.length; ++i) {
                            if (typeof d.file[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorSet.file: object expected");
                            m.file[i] = $root.google.protobuf.FileDescriptorProto.fromObject(d.file[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a FileDescriptorSet message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.FileDescriptorSet} m FileDescriptorSet
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FileDescriptorSet.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.file = [];
                    }
                    if (m.file && m.file.length) {
                        d.file = [];
                        for (var j = 0; j < m.file.length; ++j) {
                            d.file[j] = $root.google.protobuf.FileDescriptorProto.toObject(m.file[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this FileDescriptorSet to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FileDescriptorSet
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FileDescriptorSet.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return FileDescriptorSet;
            })();
    
            protobuf.FileDescriptorProto = (function() {
    
                /**
                 * Properties of a FileDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IFileDescriptorProto
                 * @property {string|null} [name] FileDescriptorProto name
                 * @property {string|null} ["package"] FileDescriptorProto package
                 * @property {Array.<string>|null} [dependency] FileDescriptorProto dependency
                 * @property {Array.<number>|null} [public_dependency] FileDescriptorProto public_dependency
                 * @property {Array.<number>|null} [weak_dependency] FileDescriptorProto weak_dependency
                 * @property {Array.<google.protobuf.IDescriptorProto>|null} [message_type] FileDescriptorProto message_type
                 * @property {Array.<google.protobuf.IEnumDescriptorProto>|null} [enum_type] FileDescriptorProto enum_type
                 * @property {Array.<google.protobuf.IServiceDescriptorProto>|null} [service] FileDescriptorProto service
                 * @property {Array.<google.protobuf.IFieldDescriptorProto>|null} [extension] FileDescriptorProto extension
                 * @property {google.protobuf.IFileOptions|null} [options] FileDescriptorProto options
                 * @property {google.protobuf.ISourceCodeInfo|null} [source_code_info] FileDescriptorProto source_code_info
                 * @property {string|null} [syntax] FileDescriptorProto syntax
                 */
    
                /**
                 * Constructs a new FileDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a FileDescriptorProto.
                 * @implements IFileDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IFileDescriptorProto=} [p] Properties to set
                 */
                function FileDescriptorProto(p) {
                    this.dependency = [];
                    this.public_dependency = [];
                    this.weak_dependency = [];
                    this.message_type = [];
                    this.enum_type = [];
                    this.service = [];
                    this.extension = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * FileDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.name = "";
    
                /**
                 * FileDescriptorProto package.
                 * @member {string} package
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype["package"] = "";
    
                /**
                 * FileDescriptorProto dependency.
                 * @member {Array.<string>} dependency
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.dependency = $util.emptyArray;
    
                /**
                 * FileDescriptorProto public_dependency.
                 * @member {Array.<number>} public_dependency
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.public_dependency = $util.emptyArray;
    
                /**
                 * FileDescriptorProto weak_dependency.
                 * @member {Array.<number>} weak_dependency
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.weak_dependency = $util.emptyArray;
    
                /**
                 * FileDescriptorProto message_type.
                 * @member {Array.<google.protobuf.IDescriptorProto>} message_type
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.message_type = $util.emptyArray;
    
                /**
                 * FileDescriptorProto enum_type.
                 * @member {Array.<google.protobuf.IEnumDescriptorProto>} enum_type
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.enum_type = $util.emptyArray;
    
                /**
                 * FileDescriptorProto service.
                 * @member {Array.<google.protobuf.IServiceDescriptorProto>} service
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.service = $util.emptyArray;
    
                /**
                 * FileDescriptorProto extension.
                 * @member {Array.<google.protobuf.IFieldDescriptorProto>} extension
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.extension = $util.emptyArray;
    
                /**
                 * FileDescriptorProto options.
                 * @member {google.protobuf.IFileOptions|null|undefined} options
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.options = null;
    
                /**
                 * FileDescriptorProto source_code_info.
                 * @member {google.protobuf.ISourceCodeInfo|null|undefined} source_code_info
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.source_code_info = null;
    
                /**
                 * FileDescriptorProto syntax.
                 * @member {string} syntax
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.syntax = "";
    
                /**
                 * Creates a new FileDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.IFileDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto instance
                 */
                FileDescriptorProto.create = function create(properties) {
                    return new FileDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified FileDescriptorProto message. Does not implicitly {@link google.protobuf.FileDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.IFileDescriptorProto} m FileDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m["package"] != null && Object.hasOwnProperty.call(m, "package"))
                        w.uint32(18).string(m["package"]);
                    if (m.dependency != null && m.dependency.length) {
                        for (var i = 0; i < m.dependency.length; ++i)
                            w.uint32(26).string(m.dependency[i]);
                    }
                    if (m.message_type != null && m.message_type.length) {
                        for (var i = 0; i < m.message_type.length; ++i)
                            $root.google.protobuf.DescriptorProto.encode(m.message_type[i], w.uint32(34).fork()).ldelim();
                    }
                    if (m.enum_type != null && m.enum_type.length) {
                        for (var i = 0; i < m.enum_type.length; ++i)
                            $root.google.protobuf.EnumDescriptorProto.encode(m.enum_type[i], w.uint32(42).fork()).ldelim();
                    }
                    if (m.service != null && m.service.length) {
                        for (var i = 0; i < m.service.length; ++i)
                            $root.google.protobuf.ServiceDescriptorProto.encode(m.service[i], w.uint32(50).fork()).ldelim();
                    }
                    if (m.extension != null && m.extension.length) {
                        for (var i = 0; i < m.extension.length; ++i)
                            $root.google.protobuf.FieldDescriptorProto.encode(m.extension[i], w.uint32(58).fork()).ldelim();
                    }
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.FileOptions.encode(m.options, w.uint32(66).fork()).ldelim();
                    if (m.source_code_info != null && Object.hasOwnProperty.call(m, "source_code_info"))
                        $root.google.protobuf.SourceCodeInfo.encode(m.source_code_info, w.uint32(74).fork()).ldelim();
                    if (m.public_dependency != null && m.public_dependency.length) {
                        for (var i = 0; i < m.public_dependency.length; ++i)
                            w.uint32(80).int32(m.public_dependency[i]);
                    }
                    if (m.weak_dependency != null && m.weak_dependency.length) {
                        for (var i = 0; i < m.weak_dependency.length; ++i)
                            w.uint32(88).int32(m.weak_dependency[i]);
                    }
                    if (m.syntax != null && Object.hasOwnProperty.call(m, "syntax"))
                        w.uint32(98).string(m.syntax);
                    return w;
                };
    
                /**
                 * Encodes the specified FileDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.FileDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.IFileDescriptorProto} message FileDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FileDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.FileDescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            m["package"] = r.string();
                            break;
                        case 3:
                            if (!(m.dependency && m.dependency.length))
                                m.dependency = [];
                            m.dependency.push(r.string());
                            break;
                        case 10:
                            if (!(m.public_dependency && m.public_dependency.length))
                                m.public_dependency = [];
                            if ((t & 7) === 2) {
                                var c2 = r.uint32() + r.pos;
                                while (r.pos < c2)
                                    m.public_dependency.push(r.int32());
                            } else
                                m.public_dependency.push(r.int32());
                            break;
                        case 11:
                            if (!(m.weak_dependency && m.weak_dependency.length))
                                m.weak_dependency = [];
                            if ((t & 7) === 2) {
                                var c2 = r.uint32() + r.pos;
                                while (r.pos < c2)
                                    m.weak_dependency.push(r.int32());
                            } else
                                m.weak_dependency.push(r.int32());
                            break;
                        case 4:
                            if (!(m.message_type && m.message_type.length))
                                m.message_type = [];
                            m.message_type.push($root.google.protobuf.DescriptorProto.decode(r, r.uint32()));
                            break;
                        case 5:
                            if (!(m.enum_type && m.enum_type.length))
                                m.enum_type = [];
                            m.enum_type.push($root.google.protobuf.EnumDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 6:
                            if (!(m.service && m.service.length))
                                m.service = [];
                            m.service.push($root.google.protobuf.ServiceDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 7:
                            if (!(m.extension && m.extension.length))
                                m.extension = [];
                            m.extension.push($root.google.protobuf.FieldDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 8:
                            m.options = $root.google.protobuf.FileOptions.decode(r, r.uint32());
                            break;
                        case 9:
                            m.source_code_info = $root.google.protobuf.SourceCodeInfo.decode(r, r.uint32());
                            break;
                        case 12:
                            m.syntax = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a FileDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FileDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FileDescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m["package"] != null && m.hasOwnProperty("package")) {
                        if (!$util.isString(m["package"]))
                            return "package: string expected";
                    }
                    if (m.dependency != null && m.hasOwnProperty("dependency")) {
                        if (!Array.isArray(m.dependency))
                            return "dependency: array expected";
                        for (var i = 0; i < m.dependency.length; ++i) {
                            if (!$util.isString(m.dependency[i]))
                                return "dependency: string[] expected";
                        }
                    }
                    if (m.public_dependency != null && m.hasOwnProperty("public_dependency")) {
                        if (!Array.isArray(m.public_dependency))
                            return "public_dependency: array expected";
                        for (var i = 0; i < m.public_dependency.length; ++i) {
                            if (!$util.isInteger(m.public_dependency[i]))
                                return "public_dependency: integer[] expected";
                        }
                    }
                    if (m.weak_dependency != null && m.hasOwnProperty("weak_dependency")) {
                        if (!Array.isArray(m.weak_dependency))
                            return "weak_dependency: array expected";
                        for (var i = 0; i < m.weak_dependency.length; ++i) {
                            if (!$util.isInteger(m.weak_dependency[i]))
                                return "weak_dependency: integer[] expected";
                        }
                    }
                    if (m.message_type != null && m.hasOwnProperty("message_type")) {
                        if (!Array.isArray(m.message_type))
                            return "message_type: array expected";
                        for (var i = 0; i < m.message_type.length; ++i) {
                            {
                                var e = $root.google.protobuf.DescriptorProto.verify(m.message_type[i]);
                                if (e)
                                    return "message_type." + e;
                            }
                        }
                    }
                    if (m.enum_type != null && m.hasOwnProperty("enum_type")) {
                        if (!Array.isArray(m.enum_type))
                            return "enum_type: array expected";
                        for (var i = 0; i < m.enum_type.length; ++i) {
                            {
                                var e = $root.google.protobuf.EnumDescriptorProto.verify(m.enum_type[i]);
                                if (e)
                                    return "enum_type." + e;
                            }
                        }
                    }
                    if (m.service != null && m.hasOwnProperty("service")) {
                        if (!Array.isArray(m.service))
                            return "service: array expected";
                        for (var i = 0; i < m.service.length; ++i) {
                            {
                                var e = $root.google.protobuf.ServiceDescriptorProto.verify(m.service[i]);
                                if (e)
                                    return "service." + e;
                            }
                        }
                    }
                    if (m.extension != null && m.hasOwnProperty("extension")) {
                        if (!Array.isArray(m.extension))
                            return "extension: array expected";
                        for (var i = 0; i < m.extension.length; ++i) {
                            {
                                var e = $root.google.protobuf.FieldDescriptorProto.verify(m.extension[i]);
                                if (e)
                                    return "extension." + e;
                            }
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.FileOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    if (m.source_code_info != null && m.hasOwnProperty("source_code_info")) {
                        {
                            var e = $root.google.protobuf.SourceCodeInfo.verify(m.source_code_info);
                            if (e)
                                return "source_code_info." + e;
                        }
                    }
                    if (m.syntax != null && m.hasOwnProperty("syntax")) {
                        if (!$util.isString(m.syntax))
                            return "syntax: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a FileDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto
                 */
                FileDescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.FileDescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.FileDescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d["package"] != null) {
                        m["package"] = String(d["package"]);
                    }
                    if (d.dependency) {
                        if (!Array.isArray(d.dependency))
                            throw TypeError(".google.protobuf.FileDescriptorProto.dependency: array expected");
                        m.dependency = [];
                        for (var i = 0; i < d.dependency.length; ++i) {
                            m.dependency[i] = String(d.dependency[i]);
                        }
                    }
                    if (d.public_dependency) {
                        if (!Array.isArray(d.public_dependency))
                            throw TypeError(".google.protobuf.FileDescriptorProto.public_dependency: array expected");
                        m.public_dependency = [];
                        for (var i = 0; i < d.public_dependency.length; ++i) {
                            m.public_dependency[i] = d.public_dependency[i] | 0;
                        }
                    }
                    if (d.weak_dependency) {
                        if (!Array.isArray(d.weak_dependency))
                            throw TypeError(".google.protobuf.FileDescriptorProto.weak_dependency: array expected");
                        m.weak_dependency = [];
                        for (var i = 0; i < d.weak_dependency.length; ++i) {
                            m.weak_dependency[i] = d.weak_dependency[i] | 0;
                        }
                    }
                    if (d.message_type) {
                        if (!Array.isArray(d.message_type))
                            throw TypeError(".google.protobuf.FileDescriptorProto.message_type: array expected");
                        m.message_type = [];
                        for (var i = 0; i < d.message_type.length; ++i) {
                            if (typeof d.message_type[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.message_type: object expected");
                            m.message_type[i] = $root.google.protobuf.DescriptorProto.fromObject(d.message_type[i]);
                        }
                    }
                    if (d.enum_type) {
                        if (!Array.isArray(d.enum_type))
                            throw TypeError(".google.protobuf.FileDescriptorProto.enum_type: array expected");
                        m.enum_type = [];
                        for (var i = 0; i < d.enum_type.length; ++i) {
                            if (typeof d.enum_type[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.enum_type: object expected");
                            m.enum_type[i] = $root.google.protobuf.EnumDescriptorProto.fromObject(d.enum_type[i]);
                        }
                    }
                    if (d.service) {
                        if (!Array.isArray(d.service))
                            throw TypeError(".google.protobuf.FileDescriptorProto.service: array expected");
                        m.service = [];
                        for (var i = 0; i < d.service.length; ++i) {
                            if (typeof d.service[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.service: object expected");
                            m.service[i] = $root.google.protobuf.ServiceDescriptorProto.fromObject(d.service[i]);
                        }
                    }
                    if (d.extension) {
                        if (!Array.isArray(d.extension))
                            throw TypeError(".google.protobuf.FileDescriptorProto.extension: array expected");
                        m.extension = [];
                        for (var i = 0; i < d.extension.length; ++i) {
                            if (typeof d.extension[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.extension: object expected");
                            m.extension[i] = $root.google.protobuf.FieldDescriptorProto.fromObject(d.extension[i]);
                        }
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.FileDescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.FileOptions.fromObject(d.options);
                    }
                    if (d.source_code_info != null) {
                        if (typeof d.source_code_info !== "object")
                            throw TypeError(".google.protobuf.FileDescriptorProto.source_code_info: object expected");
                        m.source_code_info = $root.google.protobuf.SourceCodeInfo.fromObject(d.source_code_info);
                    }
                    if (d.syntax != null) {
                        m.syntax = String(d.syntax);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a FileDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.FileDescriptorProto} m FileDescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FileDescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.dependency = [];
                        d.message_type = [];
                        d.enum_type = [];
                        d.service = [];
                        d.extension = [];
                        d.public_dependency = [];
                        d.weak_dependency = [];
                    }
                    if (o.defaults) {
                        d.name = "";
                        d["package"] = "";
                        d.options = null;
                        d.source_code_info = null;
                        d.syntax = "";
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m["package"] != null && m.hasOwnProperty("package")) {
                        d["package"] = m["package"];
                    }
                    if (m.dependency && m.dependency.length) {
                        d.dependency = [];
                        for (var j = 0; j < m.dependency.length; ++j) {
                            d.dependency[j] = m.dependency[j];
                        }
                    }
                    if (m.message_type && m.message_type.length) {
                        d.message_type = [];
                        for (var j = 0; j < m.message_type.length; ++j) {
                            d.message_type[j] = $root.google.protobuf.DescriptorProto.toObject(m.message_type[j], o);
                        }
                    }
                    if (m.enum_type && m.enum_type.length) {
                        d.enum_type = [];
                        for (var j = 0; j < m.enum_type.length; ++j) {
                            d.enum_type[j] = $root.google.protobuf.EnumDescriptorProto.toObject(m.enum_type[j], o);
                        }
                    }
                    if (m.service && m.service.length) {
                        d.service = [];
                        for (var j = 0; j < m.service.length; ++j) {
                            d.service[j] = $root.google.protobuf.ServiceDescriptorProto.toObject(m.service[j], o);
                        }
                    }
                    if (m.extension && m.extension.length) {
                        d.extension = [];
                        for (var j = 0; j < m.extension.length; ++j) {
                            d.extension[j] = $root.google.protobuf.FieldDescriptorProto.toObject(m.extension[j], o);
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.FileOptions.toObject(m.options, o);
                    }
                    if (m.source_code_info != null && m.hasOwnProperty("source_code_info")) {
                        d.source_code_info = $root.google.protobuf.SourceCodeInfo.toObject(m.source_code_info, o);
                    }
                    if (m.public_dependency && m.public_dependency.length) {
                        d.public_dependency = [];
                        for (var j = 0; j < m.public_dependency.length; ++j) {
                            d.public_dependency[j] = m.public_dependency[j];
                        }
                    }
                    if (m.weak_dependency && m.weak_dependency.length) {
                        d.weak_dependency = [];
                        for (var j = 0; j < m.weak_dependency.length; ++j) {
                            d.weak_dependency[j] = m.weak_dependency[j];
                        }
                    }
                    if (m.syntax != null && m.hasOwnProperty("syntax")) {
                        d.syntax = m.syntax;
                    }
                    return d;
                };
    
                /**
                 * Converts this FileDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FileDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return FileDescriptorProto;
            })();
    
            protobuf.DescriptorProto = (function() {
    
                /**
                 * Properties of a DescriptorProto.
                 * @memberof google.protobuf
                 * @interface IDescriptorProto
                 * @property {string|null} [name] DescriptorProto name
                 * @property {Array.<google.protobuf.IFieldDescriptorProto>|null} [field] DescriptorProto field
                 * @property {Array.<google.protobuf.IFieldDescriptorProto>|null} [extension] DescriptorProto extension
                 * @property {Array.<google.protobuf.IDescriptorProto>|null} [nested_type] DescriptorProto nested_type
                 * @property {Array.<google.protobuf.IEnumDescriptorProto>|null} [enum_type] DescriptorProto enum_type
                 * @property {Array.<google.protobuf.DescriptorProto.IExtensionRange>|null} [extension_range] DescriptorProto extension_range
                 * @property {Array.<google.protobuf.IOneofDescriptorProto>|null} [oneof_decl] DescriptorProto oneof_decl
                 * @property {google.protobuf.IMessageOptions|null} [options] DescriptorProto options
                 * @property {Array.<google.protobuf.DescriptorProto.IReservedRange>|null} [reserved_range] DescriptorProto reserved_range
                 * @property {Array.<string>|null} [reserved_name] DescriptorProto reserved_name
                 */
    
                /**
                 * Constructs a new DescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a DescriptorProto.
                 * @implements IDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IDescriptorProto=} [p] Properties to set
                 */
                function DescriptorProto(p) {
                    this.field = [];
                    this.extension = [];
                    this.nested_type = [];
                    this.enum_type = [];
                    this.extension_range = [];
                    this.oneof_decl = [];
                    this.reserved_range = [];
                    this.reserved_name = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * DescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.name = "";
    
                /**
                 * DescriptorProto field.
                 * @member {Array.<google.protobuf.IFieldDescriptorProto>} field
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.field = $util.emptyArray;
    
                /**
                 * DescriptorProto extension.
                 * @member {Array.<google.protobuf.IFieldDescriptorProto>} extension
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.extension = $util.emptyArray;
    
                /**
                 * DescriptorProto nested_type.
                 * @member {Array.<google.protobuf.IDescriptorProto>} nested_type
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.nested_type = $util.emptyArray;
    
                /**
                 * DescriptorProto enum_type.
                 * @member {Array.<google.protobuf.IEnumDescriptorProto>} enum_type
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.enum_type = $util.emptyArray;
    
                /**
                 * DescriptorProto extension_range.
                 * @member {Array.<google.protobuf.DescriptorProto.IExtensionRange>} extension_range
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.extension_range = $util.emptyArray;
    
                /**
                 * DescriptorProto oneof_decl.
                 * @member {Array.<google.protobuf.IOneofDescriptorProto>} oneof_decl
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.oneof_decl = $util.emptyArray;
    
                /**
                 * DescriptorProto options.
                 * @member {google.protobuf.IMessageOptions|null|undefined} options
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.options = null;
    
                /**
                 * DescriptorProto reserved_range.
                 * @member {Array.<google.protobuf.DescriptorProto.IReservedRange>} reserved_range
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.reserved_range = $util.emptyArray;
    
                /**
                 * DescriptorProto reserved_name.
                 * @member {Array.<string>} reserved_name
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.reserved_name = $util.emptyArray;
    
                /**
                 * Creates a new DescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.IDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto instance
                 */
                DescriptorProto.create = function create(properties) {
                    return new DescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified DescriptorProto message. Does not implicitly {@link google.protobuf.DescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.IDescriptorProto} m DescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.field != null && m.field.length) {
                        for (var i = 0; i < m.field.length; ++i)
                            $root.google.protobuf.FieldDescriptorProto.encode(m.field[i], w.uint32(18).fork()).ldelim();
                    }
                    if (m.nested_type != null && m.nested_type.length) {
                        for (var i = 0; i < m.nested_type.length; ++i)
                            $root.google.protobuf.DescriptorProto.encode(m.nested_type[i], w.uint32(26).fork()).ldelim();
                    }
                    if (m.enum_type != null && m.enum_type.length) {
                        for (var i = 0; i < m.enum_type.length; ++i)
                            $root.google.protobuf.EnumDescriptorProto.encode(m.enum_type[i], w.uint32(34).fork()).ldelim();
                    }
                    if (m.extension_range != null && m.extension_range.length) {
                        for (var i = 0; i < m.extension_range.length; ++i)
                            $root.google.protobuf.DescriptorProto.ExtensionRange.encode(m.extension_range[i], w.uint32(42).fork()).ldelim();
                    }
                    if (m.extension != null && m.extension.length) {
                        for (var i = 0; i < m.extension.length; ++i)
                            $root.google.protobuf.FieldDescriptorProto.encode(m.extension[i], w.uint32(50).fork()).ldelim();
                    }
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.MessageOptions.encode(m.options, w.uint32(58).fork()).ldelim();
                    if (m.oneof_decl != null && m.oneof_decl.length) {
                        for (var i = 0; i < m.oneof_decl.length; ++i)
                            $root.google.protobuf.OneofDescriptorProto.encode(m.oneof_decl[i], w.uint32(66).fork()).ldelim();
                    }
                    if (m.reserved_range != null && m.reserved_range.length) {
                        for (var i = 0; i < m.reserved_range.length; ++i)
                            $root.google.protobuf.DescriptorProto.ReservedRange.encode(m.reserved_range[i], w.uint32(74).fork()).ldelim();
                    }
                    if (m.reserved_name != null && m.reserved_name.length) {
                        for (var i = 0; i < m.reserved_name.length; ++i)
                            w.uint32(82).string(m.reserved_name[i]);
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified DescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.DescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.IDescriptorProto} message DescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a DescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.DescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            if (!(m.field && m.field.length))
                                m.field = [];
                            m.field.push($root.google.protobuf.FieldDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 6:
                            if (!(m.extension && m.extension.length))
                                m.extension = [];
                            m.extension.push($root.google.protobuf.FieldDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 3:
                            if (!(m.nested_type && m.nested_type.length))
                                m.nested_type = [];
                            m.nested_type.push($root.google.protobuf.DescriptorProto.decode(r, r.uint32()));
                            break;
                        case 4:
                            if (!(m.enum_type && m.enum_type.length))
                                m.enum_type = [];
                            m.enum_type.push($root.google.protobuf.EnumDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 5:
                            if (!(m.extension_range && m.extension_range.length))
                                m.extension_range = [];
                            m.extension_range.push($root.google.protobuf.DescriptorProto.ExtensionRange.decode(r, r.uint32()));
                            break;
                        case 8:
                            if (!(m.oneof_decl && m.oneof_decl.length))
                                m.oneof_decl = [];
                            m.oneof_decl.push($root.google.protobuf.OneofDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 7:
                            m.options = $root.google.protobuf.MessageOptions.decode(r, r.uint32());
                            break;
                        case 9:
                            if (!(m.reserved_range && m.reserved_range.length))
                                m.reserved_range = [];
                            m.reserved_range.push($root.google.protobuf.DescriptorProto.ReservedRange.decode(r, r.uint32()));
                            break;
                        case 10:
                            if (!(m.reserved_name && m.reserved_name.length))
                                m.reserved_name = [];
                            m.reserved_name.push(r.string());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a DescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a DescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                DescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.field != null && m.hasOwnProperty("field")) {
                        if (!Array.isArray(m.field))
                            return "field: array expected";
                        for (var i = 0; i < m.field.length; ++i) {
                            {
                                var e = $root.google.protobuf.FieldDescriptorProto.verify(m.field[i]);
                                if (e)
                                    return "field." + e;
                            }
                        }
                    }
                    if (m.extension != null && m.hasOwnProperty("extension")) {
                        if (!Array.isArray(m.extension))
                            return "extension: array expected";
                        for (var i = 0; i < m.extension.length; ++i) {
                            {
                                var e = $root.google.protobuf.FieldDescriptorProto.verify(m.extension[i]);
                                if (e)
                                    return "extension." + e;
                            }
                        }
                    }
                    if (m.nested_type != null && m.hasOwnProperty("nested_type")) {
                        if (!Array.isArray(m.nested_type))
                            return "nested_type: array expected";
                        for (var i = 0; i < m.nested_type.length; ++i) {
                            {
                                var e = $root.google.protobuf.DescriptorProto.verify(m.nested_type[i]);
                                if (e)
                                    return "nested_type." + e;
                            }
                        }
                    }
                    if (m.enum_type != null && m.hasOwnProperty("enum_type")) {
                        if (!Array.isArray(m.enum_type))
                            return "enum_type: array expected";
                        for (var i = 0; i < m.enum_type.length; ++i) {
                            {
                                var e = $root.google.protobuf.EnumDescriptorProto.verify(m.enum_type[i]);
                                if (e)
                                    return "enum_type." + e;
                            }
                        }
                    }
                    if (m.extension_range != null && m.hasOwnProperty("extension_range")) {
                        if (!Array.isArray(m.extension_range))
                            return "extension_range: array expected";
                        for (var i = 0; i < m.extension_range.length; ++i) {
                            {
                                var e = $root.google.protobuf.DescriptorProto.ExtensionRange.verify(m.extension_range[i]);
                                if (e)
                                    return "extension_range." + e;
                            }
                        }
                    }
                    if (m.oneof_decl != null && m.hasOwnProperty("oneof_decl")) {
                        if (!Array.isArray(m.oneof_decl))
                            return "oneof_decl: array expected";
                        for (var i = 0; i < m.oneof_decl.length; ++i) {
                            {
                                var e = $root.google.protobuf.OneofDescriptorProto.verify(m.oneof_decl[i]);
                                if (e)
                                    return "oneof_decl." + e;
                            }
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.MessageOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    if (m.reserved_range != null && m.hasOwnProperty("reserved_range")) {
                        if (!Array.isArray(m.reserved_range))
                            return "reserved_range: array expected";
                        for (var i = 0; i < m.reserved_range.length; ++i) {
                            {
                                var e = $root.google.protobuf.DescriptorProto.ReservedRange.verify(m.reserved_range[i]);
                                if (e)
                                    return "reserved_range." + e;
                            }
                        }
                    }
                    if (m.reserved_name != null && m.hasOwnProperty("reserved_name")) {
                        if (!Array.isArray(m.reserved_name))
                            return "reserved_name: array expected";
                        for (var i = 0; i < m.reserved_name.length; ++i) {
                            if (!$util.isString(m.reserved_name[i]))
                                return "reserved_name: string[] expected";
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a DescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto
                 */
                DescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.DescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.DescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.field) {
                        if (!Array.isArray(d.field))
                            throw TypeError(".google.protobuf.DescriptorProto.field: array expected");
                        m.field = [];
                        for (var i = 0; i < d.field.length; ++i) {
                            if (typeof d.field[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.field: object expected");
                            m.field[i] = $root.google.protobuf.FieldDescriptorProto.fromObject(d.field[i]);
                        }
                    }
                    if (d.extension) {
                        if (!Array.isArray(d.extension))
                            throw TypeError(".google.protobuf.DescriptorProto.extension: array expected");
                        m.extension = [];
                        for (var i = 0; i < d.extension.length; ++i) {
                            if (typeof d.extension[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.extension: object expected");
                            m.extension[i] = $root.google.protobuf.FieldDescriptorProto.fromObject(d.extension[i]);
                        }
                    }
                    if (d.nested_type) {
                        if (!Array.isArray(d.nested_type))
                            throw TypeError(".google.protobuf.DescriptorProto.nested_type: array expected");
                        m.nested_type = [];
                        for (var i = 0; i < d.nested_type.length; ++i) {
                            if (typeof d.nested_type[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.nested_type: object expected");
                            m.nested_type[i] = $root.google.protobuf.DescriptorProto.fromObject(d.nested_type[i]);
                        }
                    }
                    if (d.enum_type) {
                        if (!Array.isArray(d.enum_type))
                            throw TypeError(".google.protobuf.DescriptorProto.enum_type: array expected");
                        m.enum_type = [];
                        for (var i = 0; i < d.enum_type.length; ++i) {
                            if (typeof d.enum_type[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.enum_type: object expected");
                            m.enum_type[i] = $root.google.protobuf.EnumDescriptorProto.fromObject(d.enum_type[i]);
                        }
                    }
                    if (d.extension_range) {
                        if (!Array.isArray(d.extension_range))
                            throw TypeError(".google.protobuf.DescriptorProto.extension_range: array expected");
                        m.extension_range = [];
                        for (var i = 0; i < d.extension_range.length; ++i) {
                            if (typeof d.extension_range[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.extension_range: object expected");
                            m.extension_range[i] = $root.google.protobuf.DescriptorProto.ExtensionRange.fromObject(d.extension_range[i]);
                        }
                    }
                    if (d.oneof_decl) {
                        if (!Array.isArray(d.oneof_decl))
                            throw TypeError(".google.protobuf.DescriptorProto.oneof_decl: array expected");
                        m.oneof_decl = [];
                        for (var i = 0; i < d.oneof_decl.length; ++i) {
                            if (typeof d.oneof_decl[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.oneof_decl: object expected");
                            m.oneof_decl[i] = $root.google.protobuf.OneofDescriptorProto.fromObject(d.oneof_decl[i]);
                        }
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.DescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.MessageOptions.fromObject(d.options);
                    }
                    if (d.reserved_range) {
                        if (!Array.isArray(d.reserved_range))
                            throw TypeError(".google.protobuf.DescriptorProto.reserved_range: array expected");
                        m.reserved_range = [];
                        for (var i = 0; i < d.reserved_range.length; ++i) {
                            if (typeof d.reserved_range[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.reserved_range: object expected");
                            m.reserved_range[i] = $root.google.protobuf.DescriptorProto.ReservedRange.fromObject(d.reserved_range[i]);
                        }
                    }
                    if (d.reserved_name) {
                        if (!Array.isArray(d.reserved_name))
                            throw TypeError(".google.protobuf.DescriptorProto.reserved_name: array expected");
                        m.reserved_name = [];
                        for (var i = 0; i < d.reserved_name.length; ++i) {
                            m.reserved_name[i] = String(d.reserved_name[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a DescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.DescriptorProto} m DescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                DescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.field = [];
                        d.nested_type = [];
                        d.enum_type = [];
                        d.extension_range = [];
                        d.extension = [];
                        d.oneof_decl = [];
                        d.reserved_range = [];
                        d.reserved_name = [];
                    }
                    if (o.defaults) {
                        d.name = "";
                        d.options = null;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.field && m.field.length) {
                        d.field = [];
                        for (var j = 0; j < m.field.length; ++j) {
                            d.field[j] = $root.google.protobuf.FieldDescriptorProto.toObject(m.field[j], o);
                        }
                    }
                    if (m.nested_type && m.nested_type.length) {
                        d.nested_type = [];
                        for (var j = 0; j < m.nested_type.length; ++j) {
                            d.nested_type[j] = $root.google.protobuf.DescriptorProto.toObject(m.nested_type[j], o);
                        }
                    }
                    if (m.enum_type && m.enum_type.length) {
                        d.enum_type = [];
                        for (var j = 0; j < m.enum_type.length; ++j) {
                            d.enum_type[j] = $root.google.protobuf.EnumDescriptorProto.toObject(m.enum_type[j], o);
                        }
                    }
                    if (m.extension_range && m.extension_range.length) {
                        d.extension_range = [];
                        for (var j = 0; j < m.extension_range.length; ++j) {
                            d.extension_range[j] = $root.google.protobuf.DescriptorProto.ExtensionRange.toObject(m.extension_range[j], o);
                        }
                    }
                    if (m.extension && m.extension.length) {
                        d.extension = [];
                        for (var j = 0; j < m.extension.length; ++j) {
                            d.extension[j] = $root.google.protobuf.FieldDescriptorProto.toObject(m.extension[j], o);
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.MessageOptions.toObject(m.options, o);
                    }
                    if (m.oneof_decl && m.oneof_decl.length) {
                        d.oneof_decl = [];
                        for (var j = 0; j < m.oneof_decl.length; ++j) {
                            d.oneof_decl[j] = $root.google.protobuf.OneofDescriptorProto.toObject(m.oneof_decl[j], o);
                        }
                    }
                    if (m.reserved_range && m.reserved_range.length) {
                        d.reserved_range = [];
                        for (var j = 0; j < m.reserved_range.length; ++j) {
                            d.reserved_range[j] = $root.google.protobuf.DescriptorProto.ReservedRange.toObject(m.reserved_range[j], o);
                        }
                    }
                    if (m.reserved_name && m.reserved_name.length) {
                        d.reserved_name = [];
                        for (var j = 0; j < m.reserved_name.length; ++j) {
                            d.reserved_name[j] = m.reserved_name[j];
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this DescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                DescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                DescriptorProto.ExtensionRange = (function() {
    
                    /**
                     * Properties of an ExtensionRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @interface IExtensionRange
                     * @property {number|null} [start] ExtensionRange start
                     * @property {number|null} [end] ExtensionRange end
                     * @property {google.protobuf.IExtensionRangeOptions|null} [options] ExtensionRange options
                     */
    
                    /**
                     * Constructs a new ExtensionRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @classdesc Represents an ExtensionRange.
                     * @implements IExtensionRange
                     * @constructor
                     * @param {google.protobuf.DescriptorProto.IExtensionRange=} [p] Properties to set
                     */
                    function ExtensionRange(p) {
                        if (p)
                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                if (p[ks[i]] != null)
                                    this[ks[i]] = p[ks[i]];
                    }
    
                    /**
                     * ExtensionRange start.
                     * @member {number} start
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     */
                    ExtensionRange.prototype.start = 0;
    
                    /**
                     * ExtensionRange end.
                     * @member {number} end
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     */
                    ExtensionRange.prototype.end = 0;
    
                    /**
                     * ExtensionRange options.
                     * @member {google.protobuf.IExtensionRangeOptions|null|undefined} options
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     */
                    ExtensionRange.prototype.options = null;
    
                    /**
                     * Creates a new ExtensionRange instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IExtensionRange=} [properties] Properties to set
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange instance
                     */
                    ExtensionRange.create = function create(properties) {
                        return new ExtensionRange(properties);
                    };
    
                    /**
                     * Encodes the specified ExtensionRange message. Does not implicitly {@link google.protobuf.DescriptorProto.ExtensionRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IExtensionRange} m ExtensionRange message or plain object to encode
                     * @param {$protobuf.Writer} [w] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ExtensionRange.encode = function encode(m, w) {
                        if (!w)
                            w = $Writer.create();
                        if (m.start != null && Object.hasOwnProperty.call(m, "start"))
                            w.uint32(8).int32(m.start);
                        if (m.end != null && Object.hasOwnProperty.call(m, "end"))
                            w.uint32(16).int32(m.end);
                        if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                            $root.google.protobuf.ExtensionRangeOptions.encode(m.options, w.uint32(26).fork()).ldelim();
                        return w;
                    };
    
                    /**
                     * Encodes the specified ExtensionRange message, length delimited. Does not implicitly {@link google.protobuf.DescriptorProto.ExtensionRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IExtensionRange} message ExtensionRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ExtensionRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an ExtensionRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                     * @param {number} [l] Message length if known beforehand
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ExtensionRange.decode = function decode(r, l) {
                        if (!(r instanceof $Reader))
                            r = $Reader.create(r);
                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.DescriptorProto.ExtensionRange();
                        while (r.pos < c) {
                            var t = r.uint32();
                            switch (t >>> 3) {
                            case 1:
                                m.start = r.int32();
                                break;
                            case 2:
                                m.end = r.int32();
                                break;
                            case 3:
                                m.options = $root.google.protobuf.ExtensionRangeOptions.decode(r, r.uint32());
                                break;
                            default:
                                r.skipType(t & 7);
                                break;
                            }
                        }
                        return m;
                    };
    
                    /**
                     * Decodes an ExtensionRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ExtensionRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an ExtensionRange message.
                     * @function verify
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {Object.<string,*>} m Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ExtensionRange.verify = function verify(m) {
                        if (typeof m !== "object" || m === null)
                            return "object expected";
                        if (m.start != null && m.hasOwnProperty("start")) {
                            if (!$util.isInteger(m.start))
                                return "start: integer expected";
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            if (!$util.isInteger(m.end))
                                return "end: integer expected";
                        }
                        if (m.options != null && m.hasOwnProperty("options")) {
                            {
                                var e = $root.google.protobuf.ExtensionRangeOptions.verify(m.options);
                                if (e)
                                    return "options." + e;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates an ExtensionRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {Object.<string,*>} d Plain object
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange
                     */
                    ExtensionRange.fromObject = function fromObject(d) {
                        if (d instanceof $root.google.protobuf.DescriptorProto.ExtensionRange)
                            return d;
                        var m = new $root.google.protobuf.DescriptorProto.ExtensionRange();
                        if (d.start != null) {
                            m.start = d.start | 0;
                        }
                        if (d.end != null) {
                            m.end = d.end | 0;
                        }
                        if (d.options != null) {
                            if (typeof d.options !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.ExtensionRange.options: object expected");
                            m.options = $root.google.protobuf.ExtensionRangeOptions.fromObject(d.options);
                        }
                        return m;
                    };
    
                    /**
                     * Creates a plain object from an ExtensionRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.ExtensionRange} m ExtensionRange
                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ExtensionRange.toObject = function toObject(m, o) {
                        if (!o)
                            o = {};
                        var d = {};
                        if (o.defaults) {
                            d.start = 0;
                            d.end = 0;
                            d.options = null;
                        }
                        if (m.start != null && m.hasOwnProperty("start")) {
                            d.start = m.start;
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            d.end = m.end;
                        }
                        if (m.options != null && m.hasOwnProperty("options")) {
                            d.options = $root.google.protobuf.ExtensionRangeOptions.toObject(m.options, o);
                        }
                        return d;
                    };
    
                    /**
                     * Converts this ExtensionRange to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ExtensionRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    return ExtensionRange;
                })();
    
                DescriptorProto.ReservedRange = (function() {
    
                    /**
                     * Properties of a ReservedRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @interface IReservedRange
                     * @property {number|null} [start] ReservedRange start
                     * @property {number|null} [end] ReservedRange end
                     */
    
                    /**
                     * Constructs a new ReservedRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @classdesc Represents a ReservedRange.
                     * @implements IReservedRange
                     * @constructor
                     * @param {google.protobuf.DescriptorProto.IReservedRange=} [p] Properties to set
                     */
                    function ReservedRange(p) {
                        if (p)
                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                if (p[ks[i]] != null)
                                    this[ks[i]] = p[ks[i]];
                    }
    
                    /**
                     * ReservedRange start.
                     * @member {number} start
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @instance
                     */
                    ReservedRange.prototype.start = 0;
    
                    /**
                     * ReservedRange end.
                     * @member {number} end
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @instance
                     */
                    ReservedRange.prototype.end = 0;
    
                    /**
                     * Creates a new ReservedRange instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IReservedRange=} [properties] Properties to set
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange instance
                     */
                    ReservedRange.create = function create(properties) {
                        return new ReservedRange(properties);
                    };
    
                    /**
                     * Encodes the specified ReservedRange message. Does not implicitly {@link google.protobuf.DescriptorProto.ReservedRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IReservedRange} m ReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [w] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReservedRange.encode = function encode(m, w) {
                        if (!w)
                            w = $Writer.create();
                        if (m.start != null && Object.hasOwnProperty.call(m, "start"))
                            w.uint32(8).int32(m.start);
                        if (m.end != null && Object.hasOwnProperty.call(m, "end"))
                            w.uint32(16).int32(m.end);
                        return w;
                    };
    
                    /**
                     * Encodes the specified ReservedRange message, length delimited. Does not implicitly {@link google.protobuf.DescriptorProto.ReservedRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IReservedRange} message ReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReservedRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReservedRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                     * @param {number} [l] Message length if known beforehand
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReservedRange.decode = function decode(r, l) {
                        if (!(r instanceof $Reader))
                            r = $Reader.create(r);
                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.DescriptorProto.ReservedRange();
                        while (r.pos < c) {
                            var t = r.uint32();
                            switch (t >>> 3) {
                            case 1:
                                m.start = r.int32();
                                break;
                            case 2:
                                m.end = r.int32();
                                break;
                            default:
                                r.skipType(t & 7);
                                break;
                            }
                        }
                        return m;
                    };
    
                    /**
                     * Decodes a ReservedRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReservedRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReservedRange message.
                     * @function verify
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {Object.<string,*>} m Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReservedRange.verify = function verify(m) {
                        if (typeof m !== "object" || m === null)
                            return "object expected";
                        if (m.start != null && m.hasOwnProperty("start")) {
                            if (!$util.isInteger(m.start))
                                return "start: integer expected";
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            if (!$util.isInteger(m.end))
                                return "end: integer expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ReservedRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {Object.<string,*>} d Plain object
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange
                     */
                    ReservedRange.fromObject = function fromObject(d) {
                        if (d instanceof $root.google.protobuf.DescriptorProto.ReservedRange)
                            return d;
                        var m = new $root.google.protobuf.DescriptorProto.ReservedRange();
                        if (d.start != null) {
                            m.start = d.start | 0;
                        }
                        if (d.end != null) {
                            m.end = d.end | 0;
                        }
                        return m;
                    };
    
                    /**
                     * Creates a plain object from a ReservedRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.ReservedRange} m ReservedRange
                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReservedRange.toObject = function toObject(m, o) {
                        if (!o)
                            o = {};
                        var d = {};
                        if (o.defaults) {
                            d.start = 0;
                            d.end = 0;
                        }
                        if (m.start != null && m.hasOwnProperty("start")) {
                            d.start = m.start;
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            d.end = m.end;
                        }
                        return d;
                    };
    
                    /**
                     * Converts this ReservedRange to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReservedRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    return ReservedRange;
                })();
    
                return DescriptorProto;
            })();
    
            protobuf.ExtensionRangeOptions = (function() {
    
                /**
                 * Properties of an ExtensionRangeOptions.
                 * @memberof google.protobuf
                 * @interface IExtensionRangeOptions
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] ExtensionRangeOptions uninterpreted_option
                 */
    
                /**
                 * Constructs a new ExtensionRangeOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an ExtensionRangeOptions.
                 * @implements IExtensionRangeOptions
                 * @constructor
                 * @param {google.protobuf.IExtensionRangeOptions=} [p] Properties to set
                 */
                function ExtensionRangeOptions(p) {
                    this.uninterpreted_option = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * ExtensionRangeOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @instance
                 */
                ExtensionRangeOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * Creates a new ExtensionRangeOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.IExtensionRangeOptions=} [properties] Properties to set
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions instance
                 */
                ExtensionRangeOptions.create = function create(properties) {
                    return new ExtensionRangeOptions(properties);
                };
    
                /**
                 * Encodes the specified ExtensionRangeOptions message. Does not implicitly {@link google.protobuf.ExtensionRangeOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.IExtensionRangeOptions} m ExtensionRangeOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ExtensionRangeOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified ExtensionRangeOptions message, length delimited. Does not implicitly {@link google.protobuf.ExtensionRangeOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.IExtensionRangeOptions} message ExtensionRangeOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ExtensionRangeOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an ExtensionRangeOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ExtensionRangeOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.ExtensionRangeOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an ExtensionRangeOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ExtensionRangeOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an ExtensionRangeOptions message.
                 * @function verify
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ExtensionRangeOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an ExtensionRangeOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions
                 */
                ExtensionRangeOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.ExtensionRangeOptions)
                        return d;
                    var m = new $root.google.protobuf.ExtensionRangeOptions();
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.ExtensionRangeOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.ExtensionRangeOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an ExtensionRangeOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.ExtensionRangeOptions} m ExtensionRangeOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ExtensionRangeOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this ExtensionRangeOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ExtensionRangeOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return ExtensionRangeOptions;
            })();
    
            protobuf.FieldDescriptorProto = (function() {
    
                /**
                 * Properties of a FieldDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IFieldDescriptorProto
                 * @property {string|null} [name] FieldDescriptorProto name
                 * @property {number|null} [number] FieldDescriptorProto number
                 * @property {google.protobuf.FieldDescriptorProto.Label|null} [label] FieldDescriptorProto label
                 * @property {google.protobuf.FieldDescriptorProto.Type|null} [type] FieldDescriptorProto type
                 * @property {string|null} [type_name] FieldDescriptorProto type_name
                 * @property {string|null} [extendee] FieldDescriptorProto extendee
                 * @property {string|null} [default_value] FieldDescriptorProto default_value
                 * @property {number|null} [oneof_index] FieldDescriptorProto oneof_index
                 * @property {string|null} [json_name] FieldDescriptorProto json_name
                 * @property {google.protobuf.IFieldOptions|null} [options] FieldDescriptorProto options
                 * @property {boolean|null} [proto3_optional] FieldDescriptorProto proto3_optional
                 */
    
                /**
                 * Constructs a new FieldDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a FieldDescriptorProto.
                 * @implements IFieldDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IFieldDescriptorProto=} [p] Properties to set
                 */
                function FieldDescriptorProto(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * FieldDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.name = "";
    
                /**
                 * FieldDescriptorProto number.
                 * @member {number} number
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.number = 0;
    
                /**
                 * FieldDescriptorProto label.
                 * @member {google.protobuf.FieldDescriptorProto.Label} label
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.label = 1;
    
                /**
                 * FieldDescriptorProto type.
                 * @member {google.protobuf.FieldDescriptorProto.Type} type
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.type = 1;
    
                /**
                 * FieldDescriptorProto type_name.
                 * @member {string} type_name
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.type_name = "";
    
                /**
                 * FieldDescriptorProto extendee.
                 * @member {string} extendee
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.extendee = "";
    
                /**
                 * FieldDescriptorProto default_value.
                 * @member {string} default_value
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.default_value = "";
    
                /**
                 * FieldDescriptorProto oneof_index.
                 * @member {number} oneof_index
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.oneof_index = 0;
    
                /**
                 * FieldDescriptorProto json_name.
                 * @member {string} json_name
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.json_name = "";
    
                /**
                 * FieldDescriptorProto options.
                 * @member {google.protobuf.IFieldOptions|null|undefined} options
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.options = null;
    
                /**
                 * FieldDescriptorProto proto3_optional.
                 * @member {boolean} proto3_optional
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.proto3_optional = false;
    
                /**
                 * Creates a new FieldDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.IFieldDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto instance
                 */
                FieldDescriptorProto.create = function create(properties) {
                    return new FieldDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified FieldDescriptorProto message. Does not implicitly {@link google.protobuf.FieldDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.IFieldDescriptorProto} m FieldDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldDescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.extendee != null && Object.hasOwnProperty.call(m, "extendee"))
                        w.uint32(18).string(m.extendee);
                    if (m.number != null && Object.hasOwnProperty.call(m, "number"))
                        w.uint32(24).int32(m.number);
                    if (m.label != null && Object.hasOwnProperty.call(m, "label"))
                        w.uint32(32).int32(m.label);
                    if (m.type != null && Object.hasOwnProperty.call(m, "type"))
                        w.uint32(40).int32(m.type);
                    if (m.type_name != null && Object.hasOwnProperty.call(m, "type_name"))
                        w.uint32(50).string(m.type_name);
                    if (m.default_value != null && Object.hasOwnProperty.call(m, "default_value"))
                        w.uint32(58).string(m.default_value);
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.FieldOptions.encode(m.options, w.uint32(66).fork()).ldelim();
                    if (m.oneof_index != null && Object.hasOwnProperty.call(m, "oneof_index"))
                        w.uint32(72).int32(m.oneof_index);
                    if (m.json_name != null && Object.hasOwnProperty.call(m, "json_name"))
                        w.uint32(82).string(m.json_name);
                    if (m.proto3_optional != null && Object.hasOwnProperty.call(m, "proto3_optional"))
                        w.uint32(136).bool(m.proto3_optional);
                    return w;
                };
    
                /**
                 * Encodes the specified FieldDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.FieldDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.IFieldDescriptorProto} message FieldDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FieldDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldDescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.FieldDescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 3:
                            m.number = r.int32();
                            break;
                        case 4:
                            m.label = r.int32();
                            break;
                        case 5:
                            m.type = r.int32();
                            break;
                        case 6:
                            m.type_name = r.string();
                            break;
                        case 2:
                            m.extendee = r.string();
                            break;
                        case 7:
                            m.default_value = r.string();
                            break;
                        case 9:
                            m.oneof_index = r.int32();
                            break;
                        case 10:
                            m.json_name = r.string();
                            break;
                        case 8:
                            m.options = $root.google.protobuf.FieldOptions.decode(r, r.uint32());
                            break;
                        case 17:
                            m.proto3_optional = r.bool();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a FieldDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FieldDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FieldDescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.number != null && m.hasOwnProperty("number")) {
                        if (!$util.isInteger(m.number))
                            return "number: integer expected";
                    }
                    if (m.label != null && m.hasOwnProperty("label")) {
                        switch (m.label) {
                        default:
                            return "label: enum value expected";
                        case 1:
                        case 2:
                        case 3:
                            break;
                        }
                    }
                    if (m.type != null && m.hasOwnProperty("type")) {
                        switch (m.type) {
                        default:
                            return "type: enum value expected";
                        case 1:
                        case 2:
                        case 3:
                        case 4:
                        case 5:
                        case 6:
                        case 7:
                        case 8:
                        case 9:
                        case 10:
                        case 11:
                        case 12:
                        case 13:
                        case 14:
                        case 15:
                        case 16:
                        case 17:
                        case 18:
                            break;
                        }
                    }
                    if (m.type_name != null && m.hasOwnProperty("type_name")) {
                        if (!$util.isString(m.type_name))
                            return "type_name: string expected";
                    }
                    if (m.extendee != null && m.hasOwnProperty("extendee")) {
                        if (!$util.isString(m.extendee))
                            return "extendee: string expected";
                    }
                    if (m.default_value != null && m.hasOwnProperty("default_value")) {
                        if (!$util.isString(m.default_value))
                            return "default_value: string expected";
                    }
                    if (m.oneof_index != null && m.hasOwnProperty("oneof_index")) {
                        if (!$util.isInteger(m.oneof_index))
                            return "oneof_index: integer expected";
                    }
                    if (m.json_name != null && m.hasOwnProperty("json_name")) {
                        if (!$util.isString(m.json_name))
                            return "json_name: string expected";
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.FieldOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    if (m.proto3_optional != null && m.hasOwnProperty("proto3_optional")) {
                        if (typeof m.proto3_optional !== "boolean")
                            return "proto3_optional: boolean expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a FieldDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto
                 */
                FieldDescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.FieldDescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.FieldDescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.number != null) {
                        m.number = d.number | 0;
                    }
                    switch (d.label) {
                    case "LABEL_OPTIONAL":
                    case 1:
                        m.label = 1;
                        break;
                    case "LABEL_REQUIRED":
                    case 2:
                        m.label = 2;
                        break;
                    case "LABEL_REPEATED":
                    case 3:
                        m.label = 3;
                        break;
                    }
                    switch (d.type) {
                    case "TYPE_DOUBLE":
                    case 1:
                        m.type = 1;
                        break;
                    case "TYPE_FLOAT":
                    case 2:
                        m.type = 2;
                        break;
                    case "TYPE_INT64":
                    case 3:
                        m.type = 3;
                        break;
                    case "TYPE_UINT64":
                    case 4:
                        m.type = 4;
                        break;
                    case "TYPE_INT32":
                    case 5:
                        m.type = 5;
                        break;
                    case "TYPE_FIXED64":
                    case 6:
                        m.type = 6;
                        break;
                    case "TYPE_FIXED32":
                    case 7:
                        m.type = 7;
                        break;
                    case "TYPE_BOOL":
                    case 8:
                        m.type = 8;
                        break;
                    case "TYPE_STRING":
                    case 9:
                        m.type = 9;
                        break;
                    case "TYPE_GROUP":
                    case 10:
                        m.type = 10;
                        break;
                    case "TYPE_MESSAGE":
                    case 11:
                        m.type = 11;
                        break;
                    case "TYPE_BYTES":
                    case 12:
                        m.type = 12;
                        break;
                    case "TYPE_UINT32":
                    case 13:
                        m.type = 13;
                        break;
                    case "TYPE_ENUM":
                    case 14:
                        m.type = 14;
                        break;
                    case "TYPE_SFIXED32":
                    case 15:
                        m.type = 15;
                        break;
                    case "TYPE_SFIXED64":
                    case 16:
                        m.type = 16;
                        break;
                    case "TYPE_SINT32":
                    case 17:
                        m.type = 17;
                        break;
                    case "TYPE_SINT64":
                    case 18:
                        m.type = 18;
                        break;
                    }
                    if (d.type_name != null) {
                        m.type_name = String(d.type_name);
                    }
                    if (d.extendee != null) {
                        m.extendee = String(d.extendee);
                    }
                    if (d.default_value != null) {
                        m.default_value = String(d.default_value);
                    }
                    if (d.oneof_index != null) {
                        m.oneof_index = d.oneof_index | 0;
                    }
                    if (d.json_name != null) {
                        m.json_name = String(d.json_name);
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.FieldDescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.FieldOptions.fromObject(d.options);
                    }
                    if (d.proto3_optional != null) {
                        m.proto3_optional = Boolean(d.proto3_optional);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a FieldDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.FieldDescriptorProto} m FieldDescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FieldDescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                        d.extendee = "";
                        d.number = 0;
                        d.label = o.enums === String ? "LABEL_OPTIONAL" : 1;
                        d.type = o.enums === String ? "TYPE_DOUBLE" : 1;
                        d.type_name = "";
                        d.default_value = "";
                        d.options = null;
                        d.oneof_index = 0;
                        d.json_name = "";
                        d.proto3_optional = false;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.extendee != null && m.hasOwnProperty("extendee")) {
                        d.extendee = m.extendee;
                    }
                    if (m.number != null && m.hasOwnProperty("number")) {
                        d.number = m.number;
                    }
                    if (m.label != null && m.hasOwnProperty("label")) {
                        d.label = o.enums === String ? $root.google.protobuf.FieldDescriptorProto.Label[m.label] : m.label;
                    }
                    if (m.type != null && m.hasOwnProperty("type")) {
                        d.type = o.enums === String ? $root.google.protobuf.FieldDescriptorProto.Type[m.type] : m.type;
                    }
                    if (m.type_name != null && m.hasOwnProperty("type_name")) {
                        d.type_name = m.type_name;
                    }
                    if (m.default_value != null && m.hasOwnProperty("default_value")) {
                        d.default_value = m.default_value;
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.FieldOptions.toObject(m.options, o);
                    }
                    if (m.oneof_index != null && m.hasOwnProperty("oneof_index")) {
                        d.oneof_index = m.oneof_index;
                    }
                    if (m.json_name != null && m.hasOwnProperty("json_name")) {
                        d.json_name = m.json_name;
                    }
                    if (m.proto3_optional != null && m.hasOwnProperty("proto3_optional")) {
                        d.proto3_optional = m.proto3_optional;
                    }
                    return d;
                };
    
                /**
                 * Converts this FieldDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FieldDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Type enum.
                 * @name google.protobuf.FieldDescriptorProto.Type
                 * @enum {number}
                 * @property {number} TYPE_DOUBLE=1 TYPE_DOUBLE value
                 * @property {number} TYPE_FLOAT=2 TYPE_FLOAT value
                 * @property {number} TYPE_INT64=3 TYPE_INT64 value
                 * @property {number} TYPE_UINT64=4 TYPE_UINT64 value
                 * @property {number} TYPE_INT32=5 TYPE_INT32 value
                 * @property {number} TYPE_FIXED64=6 TYPE_FIXED64 value
                 * @property {number} TYPE_FIXED32=7 TYPE_FIXED32 value
                 * @property {number} TYPE_BOOL=8 TYPE_BOOL value
                 * @property {number} TYPE_STRING=9 TYPE_STRING value
                 * @property {number} TYPE_GROUP=10 TYPE_GROUP value
                 * @property {number} TYPE_MESSAGE=11 TYPE_MESSAGE value
                 * @property {number} TYPE_BYTES=12 TYPE_BYTES value
                 * @property {number} TYPE_UINT32=13 TYPE_UINT32 value
                 * @property {number} TYPE_ENUM=14 TYPE_ENUM value
                 * @property {number} TYPE_SFIXED32=15 TYPE_SFIXED32 value
                 * @property {number} TYPE_SFIXED64=16 TYPE_SFIXED64 value
                 * @property {number} TYPE_SINT32=17 TYPE_SINT32 value
                 * @property {number} TYPE_SINT64=18 TYPE_SINT64 value
                 */
                FieldDescriptorProto.Type = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[1] = "TYPE_DOUBLE"] = 1;
                    values[valuesById[2] = "TYPE_FLOAT"] = 2;
                    values[valuesById[3] = "TYPE_INT64"] = 3;
                    values[valuesById[4] = "TYPE_UINT64"] = 4;
                    values[valuesById[5] = "TYPE_INT32"] = 5;
                    values[valuesById[6] = "TYPE_FIXED64"] = 6;
                    values[valuesById[7] = "TYPE_FIXED32"] = 7;
                    values[valuesById[8] = "TYPE_BOOL"] = 8;
                    values[valuesById[9] = "TYPE_STRING"] = 9;
                    values[valuesById[10] = "TYPE_GROUP"] = 10;
                    values[valuesById[11] = "TYPE_MESSAGE"] = 11;
                    values[valuesById[12] = "TYPE_BYTES"] = 12;
                    values[valuesById[13] = "TYPE_UINT32"] = 13;
                    values[valuesById[14] = "TYPE_ENUM"] = 14;
                    values[valuesById[15] = "TYPE_SFIXED32"] = 15;
                    values[valuesById[16] = "TYPE_SFIXED64"] = 16;
                    values[valuesById[17] = "TYPE_SINT32"] = 17;
                    values[valuesById[18] = "TYPE_SINT64"] = 18;
                    return values;
                })();
    
                /**
                 * Label enum.
                 * @name google.protobuf.FieldDescriptorProto.Label
                 * @enum {number}
                 * @property {number} LABEL_OPTIONAL=1 LABEL_OPTIONAL value
                 * @property {number} LABEL_REQUIRED=2 LABEL_REQUIRED value
                 * @property {number} LABEL_REPEATED=3 LABEL_REPEATED value
                 */
                FieldDescriptorProto.Label = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[1] = "LABEL_OPTIONAL"] = 1;
                    values[valuesById[2] = "LABEL_REQUIRED"] = 2;
                    values[valuesById[3] = "LABEL_REPEATED"] = 3;
                    return values;
                })();
    
                return FieldDescriptorProto;
            })();
    
            protobuf.OneofDescriptorProto = (function() {
    
                /**
                 * Properties of an OneofDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IOneofDescriptorProto
                 * @property {string|null} [name] OneofDescriptorProto name
                 * @property {google.protobuf.IOneofOptions|null} [options] OneofDescriptorProto options
                 */
    
                /**
                 * Constructs a new OneofDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents an OneofDescriptorProto.
                 * @implements IOneofDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IOneofDescriptorProto=} [p] Properties to set
                 */
                function OneofDescriptorProto(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * OneofDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @instance
                 */
                OneofDescriptorProto.prototype.name = "";
    
                /**
                 * OneofDescriptorProto options.
                 * @member {google.protobuf.IOneofOptions|null|undefined} options
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @instance
                 */
                OneofDescriptorProto.prototype.options = null;
    
                /**
                 * Creates a new OneofDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.IOneofDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto instance
                 */
                OneofDescriptorProto.create = function create(properties) {
                    return new OneofDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified OneofDescriptorProto message. Does not implicitly {@link google.protobuf.OneofDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.IOneofDescriptorProto} m OneofDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofDescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.OneofOptions.encode(m.options, w.uint32(18).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified OneofDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.OneofDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.IOneofDescriptorProto} message OneofDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an OneofDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofDescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.OneofDescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            m.options = $root.google.protobuf.OneofOptions.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an OneofDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an OneofDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                OneofDescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.OneofOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an OneofDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto
                 */
                OneofDescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.OneofDescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.OneofDescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.OneofDescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.OneofOptions.fromObject(d.options);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an OneofDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.OneofDescriptorProto} m OneofDescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                OneofDescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                        d.options = null;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.OneofOptions.toObject(m.options, o);
                    }
                    return d;
                };
    
                /**
                 * Converts this OneofDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                OneofDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return OneofDescriptorProto;
            })();
    
            protobuf.EnumDescriptorProto = (function() {
    
                /**
                 * Properties of an EnumDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IEnumDescriptorProto
                 * @property {string|null} [name] EnumDescriptorProto name
                 * @property {Array.<google.protobuf.IEnumValueDescriptorProto>|null} [value] EnumDescriptorProto value
                 * @property {google.protobuf.IEnumOptions|null} [options] EnumDescriptorProto options
                 * @property {Array.<google.protobuf.EnumDescriptorProto.IEnumReservedRange>|null} [reserved_range] EnumDescriptorProto reserved_range
                 * @property {Array.<string>|null} [reserved_name] EnumDescriptorProto reserved_name
                 */
    
                /**
                 * Constructs a new EnumDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumDescriptorProto.
                 * @implements IEnumDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IEnumDescriptorProto=} [p] Properties to set
                 */
                function EnumDescriptorProto(p) {
                    this.value = [];
                    this.reserved_range = [];
                    this.reserved_name = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * EnumDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.name = "";
    
                /**
                 * EnumDescriptorProto value.
                 * @member {Array.<google.protobuf.IEnumValueDescriptorProto>} value
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.value = $util.emptyArray;
    
                /**
                 * EnumDescriptorProto options.
                 * @member {google.protobuf.IEnumOptions|null|undefined} options
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.options = null;
    
                /**
                 * EnumDescriptorProto reserved_range.
                 * @member {Array.<google.protobuf.EnumDescriptorProto.IEnumReservedRange>} reserved_range
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.reserved_range = $util.emptyArray;
    
                /**
                 * EnumDescriptorProto reserved_name.
                 * @member {Array.<string>} reserved_name
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.reserved_name = $util.emptyArray;
    
                /**
                 * Creates a new EnumDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto instance
                 */
                EnumDescriptorProto.create = function create(properties) {
                    return new EnumDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified EnumDescriptorProto message. Does not implicitly {@link google.protobuf.EnumDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumDescriptorProto} m EnumDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumDescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.value != null && m.value.length) {
                        for (var i = 0; i < m.value.length; ++i)
                            $root.google.protobuf.EnumValueDescriptorProto.encode(m.value[i], w.uint32(18).fork()).ldelim();
                    }
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.EnumOptions.encode(m.options, w.uint32(26).fork()).ldelim();
                    if (m.reserved_range != null && m.reserved_range.length) {
                        for (var i = 0; i < m.reserved_range.length; ++i)
                            $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.encode(m.reserved_range[i], w.uint32(34).fork()).ldelim();
                    }
                    if (m.reserved_name != null && m.reserved_name.length) {
                        for (var i = 0; i < m.reserved_name.length; ++i)
                            w.uint32(42).string(m.reserved_name[i]);
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified EnumDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.EnumDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumDescriptorProto} message EnumDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumDescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.EnumDescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            if (!(m.value && m.value.length))
                                m.value = [];
                            m.value.push($root.google.protobuf.EnumValueDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 3:
                            m.options = $root.google.protobuf.EnumOptions.decode(r, r.uint32());
                            break;
                        case 4:
                            if (!(m.reserved_range && m.reserved_range.length))
                                m.reserved_range = [];
                            m.reserved_range.push($root.google.protobuf.EnumDescriptorProto.EnumReservedRange.decode(r, r.uint32()));
                            break;
                        case 5:
                            if (!(m.reserved_name && m.reserved_name.length))
                                m.reserved_name = [];
                            m.reserved_name.push(r.string());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an EnumDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumDescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.value != null && m.hasOwnProperty("value")) {
                        if (!Array.isArray(m.value))
                            return "value: array expected";
                        for (var i = 0; i < m.value.length; ++i) {
                            {
                                var e = $root.google.protobuf.EnumValueDescriptorProto.verify(m.value[i]);
                                if (e)
                                    return "value." + e;
                            }
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.EnumOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    if (m.reserved_range != null && m.hasOwnProperty("reserved_range")) {
                        if (!Array.isArray(m.reserved_range))
                            return "reserved_range: array expected";
                        for (var i = 0; i < m.reserved_range.length; ++i) {
                            {
                                var e = $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.verify(m.reserved_range[i]);
                                if (e)
                                    return "reserved_range." + e;
                            }
                        }
                    }
                    if (m.reserved_name != null && m.hasOwnProperty("reserved_name")) {
                        if (!Array.isArray(m.reserved_name))
                            return "reserved_name: array expected";
                        for (var i = 0; i < m.reserved_name.length; ++i) {
                            if (!$util.isString(m.reserved_name[i]))
                                return "reserved_name: string[] expected";
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto
                 */
                EnumDescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.EnumDescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.EnumDescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.value) {
                        if (!Array.isArray(d.value))
                            throw TypeError(".google.protobuf.EnumDescriptorProto.value: array expected");
                        m.value = [];
                        for (var i = 0; i < d.value.length; ++i) {
                            if (typeof d.value[i] !== "object")
                                throw TypeError(".google.protobuf.EnumDescriptorProto.value: object expected");
                            m.value[i] = $root.google.protobuf.EnumValueDescriptorProto.fromObject(d.value[i]);
                        }
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.EnumDescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.EnumOptions.fromObject(d.options);
                    }
                    if (d.reserved_range) {
                        if (!Array.isArray(d.reserved_range))
                            throw TypeError(".google.protobuf.EnumDescriptorProto.reserved_range: array expected");
                        m.reserved_range = [];
                        for (var i = 0; i < d.reserved_range.length; ++i) {
                            if (typeof d.reserved_range[i] !== "object")
                                throw TypeError(".google.protobuf.EnumDescriptorProto.reserved_range: object expected");
                            m.reserved_range[i] = $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.fromObject(d.reserved_range[i]);
                        }
                    }
                    if (d.reserved_name) {
                        if (!Array.isArray(d.reserved_name))
                            throw TypeError(".google.protobuf.EnumDescriptorProto.reserved_name: array expected");
                        m.reserved_name = [];
                        for (var i = 0; i < d.reserved_name.length; ++i) {
                            m.reserved_name[i] = String(d.reserved_name[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an EnumDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.EnumDescriptorProto} m EnumDescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumDescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.value = [];
                        d.reserved_range = [];
                        d.reserved_name = [];
                    }
                    if (o.defaults) {
                        d.name = "";
                        d.options = null;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.value && m.value.length) {
                        d.value = [];
                        for (var j = 0; j < m.value.length; ++j) {
                            d.value[j] = $root.google.protobuf.EnumValueDescriptorProto.toObject(m.value[j], o);
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.EnumOptions.toObject(m.options, o);
                    }
                    if (m.reserved_range && m.reserved_range.length) {
                        d.reserved_range = [];
                        for (var j = 0; j < m.reserved_range.length; ++j) {
                            d.reserved_range[j] = $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.toObject(m.reserved_range[j], o);
                        }
                    }
                    if (m.reserved_name && m.reserved_name.length) {
                        d.reserved_name = [];
                        for (var j = 0; j < m.reserved_name.length; ++j) {
                            d.reserved_name[j] = m.reserved_name[j];
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this EnumDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                EnumDescriptorProto.EnumReservedRange = (function() {
    
                    /**
                     * Properties of an EnumReservedRange.
                     * @memberof google.protobuf.EnumDescriptorProto
                     * @interface IEnumReservedRange
                     * @property {number|null} [start] EnumReservedRange start
                     * @property {number|null} [end] EnumReservedRange end
                     */
    
                    /**
                     * Constructs a new EnumReservedRange.
                     * @memberof google.protobuf.EnumDescriptorProto
                     * @classdesc Represents an EnumReservedRange.
                     * @implements IEnumReservedRange
                     * @constructor
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange=} [p] Properties to set
                     */
                    function EnumReservedRange(p) {
                        if (p)
                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                if (p[ks[i]] != null)
                                    this[ks[i]] = p[ks[i]];
                    }
    
                    /**
                     * EnumReservedRange start.
                     * @member {number} start
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @instance
                     */
                    EnumReservedRange.prototype.start = 0;
    
                    /**
                     * EnumReservedRange end.
                     * @member {number} end
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @instance
                     */
                    EnumReservedRange.prototype.end = 0;
    
                    /**
                     * Creates a new EnumReservedRange instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange=} [properties] Properties to set
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange instance
                     */
                    EnumReservedRange.create = function create(properties) {
                        return new EnumReservedRange(properties);
                    };
    
                    /**
                     * Encodes the specified EnumReservedRange message. Does not implicitly {@link google.protobuf.EnumDescriptorProto.EnumReservedRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange} m EnumReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [w] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    EnumReservedRange.encode = function encode(m, w) {
                        if (!w)
                            w = $Writer.create();
                        if (m.start != null && Object.hasOwnProperty.call(m, "start"))
                            w.uint32(8).int32(m.start);
                        if (m.end != null && Object.hasOwnProperty.call(m, "end"))
                            w.uint32(16).int32(m.end);
                        return w;
                    };
    
                    /**
                     * Encodes the specified EnumReservedRange message, length delimited. Does not implicitly {@link google.protobuf.EnumDescriptorProto.EnumReservedRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange} message EnumReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    EnumReservedRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an EnumReservedRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                     * @param {number} [l] Message length if known beforehand
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    EnumReservedRange.decode = function decode(r, l) {
                        if (!(r instanceof $Reader))
                            r = $Reader.create(r);
                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.EnumDescriptorProto.EnumReservedRange();
                        while (r.pos < c) {
                            var t = r.uint32();
                            switch (t >>> 3) {
                            case 1:
                                m.start = r.int32();
                                break;
                            case 2:
                                m.end = r.int32();
                                break;
                            default:
                                r.skipType(t & 7);
                                break;
                            }
                        }
                        return m;
                    };
    
                    /**
                     * Decodes an EnumReservedRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    EnumReservedRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an EnumReservedRange message.
                     * @function verify
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {Object.<string,*>} m Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    EnumReservedRange.verify = function verify(m) {
                        if (typeof m !== "object" || m === null)
                            return "object expected";
                        if (m.start != null && m.hasOwnProperty("start")) {
                            if (!$util.isInteger(m.start))
                                return "start: integer expected";
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            if (!$util.isInteger(m.end))
                                return "end: integer expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates an EnumReservedRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {Object.<string,*>} d Plain object
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange
                     */
                    EnumReservedRange.fromObject = function fromObject(d) {
                        if (d instanceof $root.google.protobuf.EnumDescriptorProto.EnumReservedRange)
                            return d;
                        var m = new $root.google.protobuf.EnumDescriptorProto.EnumReservedRange();
                        if (d.start != null) {
                            m.start = d.start | 0;
                        }
                        if (d.end != null) {
                            m.end = d.end | 0;
                        }
                        return m;
                    };
    
                    /**
                     * Creates a plain object from an EnumReservedRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.EnumReservedRange} m EnumReservedRange
                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    EnumReservedRange.toObject = function toObject(m, o) {
                        if (!o)
                            o = {};
                        var d = {};
                        if (o.defaults) {
                            d.start = 0;
                            d.end = 0;
                        }
                        if (m.start != null && m.hasOwnProperty("start")) {
                            d.start = m.start;
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            d.end = m.end;
                        }
                        return d;
                    };
    
                    /**
                     * Converts this EnumReservedRange to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    EnumReservedRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    return EnumReservedRange;
                })();
    
                return EnumDescriptorProto;
            })();
    
            protobuf.EnumValueDescriptorProto = (function() {
    
                /**
                 * Properties of an EnumValueDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IEnumValueDescriptorProto
                 * @property {string|null} [name] EnumValueDescriptorProto name
                 * @property {number|null} [number] EnumValueDescriptorProto number
                 * @property {google.protobuf.IEnumValueOptions|null} [options] EnumValueDescriptorProto options
                 */
    
                /**
                 * Constructs a new EnumValueDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumValueDescriptorProto.
                 * @implements IEnumValueDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IEnumValueDescriptorProto=} [p] Properties to set
                 */
                function EnumValueDescriptorProto(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * EnumValueDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 */
                EnumValueDescriptorProto.prototype.name = "";
    
                /**
                 * EnumValueDescriptorProto number.
                 * @member {number} number
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 */
                EnumValueDescriptorProto.prototype.number = 0;
    
                /**
                 * EnumValueDescriptorProto options.
                 * @member {google.protobuf.IEnumValueOptions|null|undefined} options
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 */
                EnumValueDescriptorProto.prototype.options = null;
    
                /**
                 * Creates a new EnumValueDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumValueDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto instance
                 */
                EnumValueDescriptorProto.create = function create(properties) {
                    return new EnumValueDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified EnumValueDescriptorProto message. Does not implicitly {@link google.protobuf.EnumValueDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumValueDescriptorProto} m EnumValueDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueDescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.number != null && Object.hasOwnProperty.call(m, "number"))
                        w.uint32(16).int32(m.number);
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.EnumValueOptions.encode(m.options, w.uint32(26).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified EnumValueDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.EnumValueDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumValueDescriptorProto} message EnumValueDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumValueDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueDescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.EnumValueDescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            m.number = r.int32();
                            break;
                        case 3:
                            m.options = $root.google.protobuf.EnumValueOptions.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an EnumValueDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumValueDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumValueDescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.number != null && m.hasOwnProperty("number")) {
                        if (!$util.isInteger(m.number))
                            return "number: integer expected";
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.EnumValueOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumValueDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto
                 */
                EnumValueDescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.EnumValueDescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.EnumValueDescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.number != null) {
                        m.number = d.number | 0;
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.EnumValueDescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.EnumValueOptions.fromObject(d.options);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an EnumValueDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.EnumValueDescriptorProto} m EnumValueDescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumValueDescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                        d.number = 0;
                        d.options = null;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.number != null && m.hasOwnProperty("number")) {
                        d.number = m.number;
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.EnumValueOptions.toObject(m.options, o);
                    }
                    return d;
                };
    
                /**
                 * Converts this EnumValueDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumValueDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return EnumValueDescriptorProto;
            })();
    
            protobuf.ServiceDescriptorProto = (function() {
    
                /**
                 * Properties of a ServiceDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IServiceDescriptorProto
                 * @property {string|null} [name] ServiceDescriptorProto name
                 * @property {Array.<google.protobuf.IMethodDescriptorProto>|null} [method] ServiceDescriptorProto method
                 * @property {google.protobuf.IServiceOptions|null} [options] ServiceDescriptorProto options
                 */
    
                /**
                 * Constructs a new ServiceDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a ServiceDescriptorProto.
                 * @implements IServiceDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IServiceDescriptorProto=} [p] Properties to set
                 */
                function ServiceDescriptorProto(p) {
                    this.method = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * ServiceDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 */
                ServiceDescriptorProto.prototype.name = "";
    
                /**
                 * ServiceDescriptorProto method.
                 * @member {Array.<google.protobuf.IMethodDescriptorProto>} method
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 */
                ServiceDescriptorProto.prototype.method = $util.emptyArray;
    
                /**
                 * ServiceDescriptorProto options.
                 * @member {google.protobuf.IServiceOptions|null|undefined} options
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 */
                ServiceDescriptorProto.prototype.options = null;
    
                /**
                 * Creates a new ServiceDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.IServiceDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto instance
                 */
                ServiceDescriptorProto.create = function create(properties) {
                    return new ServiceDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified ServiceDescriptorProto message. Does not implicitly {@link google.protobuf.ServiceDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.IServiceDescriptorProto} m ServiceDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceDescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.method != null && m.method.length) {
                        for (var i = 0; i < m.method.length; ++i)
                            $root.google.protobuf.MethodDescriptorProto.encode(m.method[i], w.uint32(18).fork()).ldelim();
                    }
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.ServiceOptions.encode(m.options, w.uint32(26).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified ServiceDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.ServiceDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.IServiceDescriptorProto} message ServiceDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ServiceDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceDescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.ServiceDescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            if (!(m.method && m.method.length))
                                m.method = [];
                            m.method.push($root.google.protobuf.MethodDescriptorProto.decode(r, r.uint32()));
                            break;
                        case 3:
                            m.options = $root.google.protobuf.ServiceOptions.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a ServiceDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ServiceDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ServiceDescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.method != null && m.hasOwnProperty("method")) {
                        if (!Array.isArray(m.method))
                            return "method: array expected";
                        for (var i = 0; i < m.method.length; ++i) {
                            {
                                var e = $root.google.protobuf.MethodDescriptorProto.verify(m.method[i]);
                                if (e)
                                    return "method." + e;
                            }
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.ServiceOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a ServiceDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto
                 */
                ServiceDescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.ServiceDescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.ServiceDescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.method) {
                        if (!Array.isArray(d.method))
                            throw TypeError(".google.protobuf.ServiceDescriptorProto.method: array expected");
                        m.method = [];
                        for (var i = 0; i < d.method.length; ++i) {
                            if (typeof d.method[i] !== "object")
                                throw TypeError(".google.protobuf.ServiceDescriptorProto.method: object expected");
                            m.method[i] = $root.google.protobuf.MethodDescriptorProto.fromObject(d.method[i]);
                        }
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.ServiceDescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.ServiceOptions.fromObject(d.options);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a ServiceDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.ServiceDescriptorProto} m ServiceDescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ServiceDescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.method = [];
                    }
                    if (o.defaults) {
                        d.name = "";
                        d.options = null;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.method && m.method.length) {
                        d.method = [];
                        for (var j = 0; j < m.method.length; ++j) {
                            d.method[j] = $root.google.protobuf.MethodDescriptorProto.toObject(m.method[j], o);
                        }
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.ServiceOptions.toObject(m.options, o);
                    }
                    return d;
                };
    
                /**
                 * Converts this ServiceDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ServiceDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return ServiceDescriptorProto;
            })();
    
            protobuf.MethodDescriptorProto = (function() {
    
                /**
                 * Properties of a MethodDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IMethodDescriptorProto
                 * @property {string|null} [name] MethodDescriptorProto name
                 * @property {string|null} [input_type] MethodDescriptorProto input_type
                 * @property {string|null} [output_type] MethodDescriptorProto output_type
                 * @property {google.protobuf.IMethodOptions|null} [options] MethodDescriptorProto options
                 * @property {boolean|null} [client_streaming] MethodDescriptorProto client_streaming
                 * @property {boolean|null} [server_streaming] MethodDescriptorProto server_streaming
                 */
    
                /**
                 * Constructs a new MethodDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a MethodDescriptorProto.
                 * @implements IMethodDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IMethodDescriptorProto=} [p] Properties to set
                 */
                function MethodDescriptorProto(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * MethodDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.name = "";
    
                /**
                 * MethodDescriptorProto input_type.
                 * @member {string} input_type
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.input_type = "";
    
                /**
                 * MethodDescriptorProto output_type.
                 * @member {string} output_type
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.output_type = "";
    
                /**
                 * MethodDescriptorProto options.
                 * @member {google.protobuf.IMethodOptions|null|undefined} options
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.options = null;
    
                /**
                 * MethodDescriptorProto client_streaming.
                 * @member {boolean} client_streaming
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.client_streaming = false;
    
                /**
                 * MethodDescriptorProto server_streaming.
                 * @member {boolean} server_streaming
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.server_streaming = false;
    
                /**
                 * Creates a new MethodDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.IMethodDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto instance
                 */
                MethodDescriptorProto.create = function create(properties) {
                    return new MethodDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified MethodDescriptorProto message. Does not implicitly {@link google.protobuf.MethodDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.IMethodDescriptorProto} m MethodDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodDescriptorProto.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.input_type != null && Object.hasOwnProperty.call(m, "input_type"))
                        w.uint32(18).string(m.input_type);
                    if (m.output_type != null && Object.hasOwnProperty.call(m, "output_type"))
                        w.uint32(26).string(m.output_type);
                    if (m.options != null && Object.hasOwnProperty.call(m, "options"))
                        $root.google.protobuf.MethodOptions.encode(m.options, w.uint32(34).fork()).ldelim();
                    if (m.client_streaming != null && Object.hasOwnProperty.call(m, "client_streaming"))
                        w.uint32(40).bool(m.client_streaming);
                    if (m.server_streaming != null && Object.hasOwnProperty.call(m, "server_streaming"))
                        w.uint32(48).bool(m.server_streaming);
                    return w;
                };
    
                /**
                 * Encodes the specified MethodDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.MethodDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.IMethodDescriptorProto} message MethodDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a MethodDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodDescriptorProto.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.MethodDescriptorProto();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            m.input_type = r.string();
                            break;
                        case 3:
                            m.output_type = r.string();
                            break;
                        case 4:
                            m.options = $root.google.protobuf.MethodOptions.decode(r, r.uint32());
                            break;
                        case 5:
                            m.client_streaming = r.bool();
                            break;
                        case 6:
                            m.server_streaming = r.bool();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a MethodDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a MethodDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                MethodDescriptorProto.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.input_type != null && m.hasOwnProperty("input_type")) {
                        if (!$util.isString(m.input_type))
                            return "input_type: string expected";
                    }
                    if (m.output_type != null && m.hasOwnProperty("output_type")) {
                        if (!$util.isString(m.output_type))
                            return "output_type: string expected";
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        {
                            var e = $root.google.protobuf.MethodOptions.verify(m.options);
                            if (e)
                                return "options." + e;
                        }
                    }
                    if (m.client_streaming != null && m.hasOwnProperty("client_streaming")) {
                        if (typeof m.client_streaming !== "boolean")
                            return "client_streaming: boolean expected";
                    }
                    if (m.server_streaming != null && m.hasOwnProperty("server_streaming")) {
                        if (typeof m.server_streaming !== "boolean")
                            return "server_streaming: boolean expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a MethodDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto
                 */
                MethodDescriptorProto.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.MethodDescriptorProto)
                        return d;
                    var m = new $root.google.protobuf.MethodDescriptorProto();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.input_type != null) {
                        m.input_type = String(d.input_type);
                    }
                    if (d.output_type != null) {
                        m.output_type = String(d.output_type);
                    }
                    if (d.options != null) {
                        if (typeof d.options !== "object")
                            throw TypeError(".google.protobuf.MethodDescriptorProto.options: object expected");
                        m.options = $root.google.protobuf.MethodOptions.fromObject(d.options);
                    }
                    if (d.client_streaming != null) {
                        m.client_streaming = Boolean(d.client_streaming);
                    }
                    if (d.server_streaming != null) {
                        m.server_streaming = Boolean(d.server_streaming);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a MethodDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.MethodDescriptorProto} m MethodDescriptorProto
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                MethodDescriptorProto.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                        d.input_type = "";
                        d.output_type = "";
                        d.options = null;
                        d.client_streaming = false;
                        d.server_streaming = false;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.input_type != null && m.hasOwnProperty("input_type")) {
                        d.input_type = m.input_type;
                    }
                    if (m.output_type != null && m.hasOwnProperty("output_type")) {
                        d.output_type = m.output_type;
                    }
                    if (m.options != null && m.hasOwnProperty("options")) {
                        d.options = $root.google.protobuf.MethodOptions.toObject(m.options, o);
                    }
                    if (m.client_streaming != null && m.hasOwnProperty("client_streaming")) {
                        d.client_streaming = m.client_streaming;
                    }
                    if (m.server_streaming != null && m.hasOwnProperty("server_streaming")) {
                        d.server_streaming = m.server_streaming;
                    }
                    return d;
                };
    
                /**
                 * Converts this MethodDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                MethodDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return MethodDescriptorProto;
            })();
    
            protobuf.FileOptions = (function() {
    
                /**
                 * Properties of a FileOptions.
                 * @memberof google.protobuf
                 * @interface IFileOptions
                 * @property {string|null} [java_package] FileOptions java_package
                 * @property {string|null} [java_outer_classname] FileOptions java_outer_classname
                 * @property {boolean|null} [java_multiple_files] FileOptions java_multiple_files
                 * @property {boolean|null} [java_generate_equals_and_hash] FileOptions java_generate_equals_and_hash
                 * @property {boolean|null} [java_string_check_utf8] FileOptions java_string_check_utf8
                 * @property {google.protobuf.FileOptions.OptimizeMode|null} [optimize_for] FileOptions optimize_for
                 * @property {string|null} [go_package] FileOptions go_package
                 * @property {boolean|null} [cc_generic_services] FileOptions cc_generic_services
                 * @property {boolean|null} [java_generic_services] FileOptions java_generic_services
                 * @property {boolean|null} [py_generic_services] FileOptions py_generic_services
                 * @property {boolean|null} [php_generic_services] FileOptions php_generic_services
                 * @property {boolean|null} [deprecated] FileOptions deprecated
                 * @property {boolean|null} [cc_enable_arenas] FileOptions cc_enable_arenas
                 * @property {string|null} [objc_class_prefix] FileOptions objc_class_prefix
                 * @property {string|null} [csharp_namespace] FileOptions csharp_namespace
                 * @property {string|null} [swift_prefix] FileOptions swift_prefix
                 * @property {string|null} [php_class_prefix] FileOptions php_class_prefix
                 * @property {string|null} [php_namespace] FileOptions php_namespace
                 * @property {string|null} [php_metadata_namespace] FileOptions php_metadata_namespace
                 * @property {string|null} [ruby_package] FileOptions ruby_package
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] FileOptions uninterpreted_option
                 * @property {Array.<google.api.IResourceDescriptor>|null} [".google.api.resource_definition"] FileOptions .google.api.resource_definition
                 */
    
                /**
                 * Constructs a new FileOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a FileOptions.
                 * @implements IFileOptions
                 * @constructor
                 * @param {google.protobuf.IFileOptions=} [p] Properties to set
                 */
                function FileOptions(p) {
                    this.uninterpreted_option = [];
                    this[".google.api.resource_definition"] = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * FileOptions java_package.
                 * @member {string} java_package
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.java_package = "";
    
                /**
                 * FileOptions java_outer_classname.
                 * @member {string} java_outer_classname
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.java_outer_classname = "";
    
                /**
                 * FileOptions java_multiple_files.
                 * @member {boolean} java_multiple_files
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.java_multiple_files = false;
    
                /**
                 * FileOptions java_generate_equals_and_hash.
                 * @member {boolean} java_generate_equals_and_hash
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.java_generate_equals_and_hash = false;
    
                /**
                 * FileOptions java_string_check_utf8.
                 * @member {boolean} java_string_check_utf8
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.java_string_check_utf8 = false;
    
                /**
                 * FileOptions optimize_for.
                 * @member {google.protobuf.FileOptions.OptimizeMode} optimize_for
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.optimize_for = 1;
    
                /**
                 * FileOptions go_package.
                 * @member {string} go_package
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.go_package = "";
    
                /**
                 * FileOptions cc_generic_services.
                 * @member {boolean} cc_generic_services
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.cc_generic_services = false;
    
                /**
                 * FileOptions java_generic_services.
                 * @member {boolean} java_generic_services
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.java_generic_services = false;
    
                /**
                 * FileOptions py_generic_services.
                 * @member {boolean} py_generic_services
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.py_generic_services = false;
    
                /**
                 * FileOptions php_generic_services.
                 * @member {boolean} php_generic_services
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.php_generic_services = false;
    
                /**
                 * FileOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.deprecated = false;
    
                /**
                 * FileOptions cc_enable_arenas.
                 * @member {boolean} cc_enable_arenas
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.cc_enable_arenas = true;
    
                /**
                 * FileOptions objc_class_prefix.
                 * @member {string} objc_class_prefix
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.objc_class_prefix = "";
    
                /**
                 * FileOptions csharp_namespace.
                 * @member {string} csharp_namespace
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.csharp_namespace = "";
    
                /**
                 * FileOptions swift_prefix.
                 * @member {string} swift_prefix
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.swift_prefix = "";
    
                /**
                 * FileOptions php_class_prefix.
                 * @member {string} php_class_prefix
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.php_class_prefix = "";
    
                /**
                 * FileOptions php_namespace.
                 * @member {string} php_namespace
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.php_namespace = "";
    
                /**
                 * FileOptions php_metadata_namespace.
                 * @member {string} php_metadata_namespace
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.php_metadata_namespace = "";
    
                /**
                 * FileOptions ruby_package.
                 * @member {string} ruby_package
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.ruby_package = "";
    
                /**
                 * FileOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * FileOptions .google.api.resource_definition.
                 * @member {Array.<google.api.IResourceDescriptor>} .google.api.resource_definition
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype[".google.api.resource_definition"] = $util.emptyArray;
    
                /**
                 * Creates a new FileOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.IFileOptions=} [properties] Properties to set
                 * @returns {google.protobuf.FileOptions} FileOptions instance
                 */
                FileOptions.create = function create(properties) {
                    return new FileOptions(properties);
                };
    
                /**
                 * Encodes the specified FileOptions message. Does not implicitly {@link google.protobuf.FileOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.IFileOptions} m FileOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.java_package != null && Object.hasOwnProperty.call(m, "java_package"))
                        w.uint32(10).string(m.java_package);
                    if (m.java_outer_classname != null && Object.hasOwnProperty.call(m, "java_outer_classname"))
                        w.uint32(66).string(m.java_outer_classname);
                    if (m.optimize_for != null && Object.hasOwnProperty.call(m, "optimize_for"))
                        w.uint32(72).int32(m.optimize_for);
                    if (m.java_multiple_files != null && Object.hasOwnProperty.call(m, "java_multiple_files"))
                        w.uint32(80).bool(m.java_multiple_files);
                    if (m.go_package != null && Object.hasOwnProperty.call(m, "go_package"))
                        w.uint32(90).string(m.go_package);
                    if (m.cc_generic_services != null && Object.hasOwnProperty.call(m, "cc_generic_services"))
                        w.uint32(128).bool(m.cc_generic_services);
                    if (m.java_generic_services != null && Object.hasOwnProperty.call(m, "java_generic_services"))
                        w.uint32(136).bool(m.java_generic_services);
                    if (m.py_generic_services != null && Object.hasOwnProperty.call(m, "py_generic_services"))
                        w.uint32(144).bool(m.py_generic_services);
                    if (m.java_generate_equals_and_hash != null && Object.hasOwnProperty.call(m, "java_generate_equals_and_hash"))
                        w.uint32(160).bool(m.java_generate_equals_and_hash);
                    if (m.deprecated != null && Object.hasOwnProperty.call(m, "deprecated"))
                        w.uint32(184).bool(m.deprecated);
                    if (m.java_string_check_utf8 != null && Object.hasOwnProperty.call(m, "java_string_check_utf8"))
                        w.uint32(216).bool(m.java_string_check_utf8);
                    if (m.cc_enable_arenas != null && Object.hasOwnProperty.call(m, "cc_enable_arenas"))
                        w.uint32(248).bool(m.cc_enable_arenas);
                    if (m.objc_class_prefix != null && Object.hasOwnProperty.call(m, "objc_class_prefix"))
                        w.uint32(290).string(m.objc_class_prefix);
                    if (m.csharp_namespace != null && Object.hasOwnProperty.call(m, "csharp_namespace"))
                        w.uint32(298).string(m.csharp_namespace);
                    if (m.swift_prefix != null && Object.hasOwnProperty.call(m, "swift_prefix"))
                        w.uint32(314).string(m.swift_prefix);
                    if (m.php_class_prefix != null && Object.hasOwnProperty.call(m, "php_class_prefix"))
                        w.uint32(322).string(m.php_class_prefix);
                    if (m.php_namespace != null && Object.hasOwnProperty.call(m, "php_namespace"))
                        w.uint32(330).string(m.php_namespace);
                    if (m.php_generic_services != null && Object.hasOwnProperty.call(m, "php_generic_services"))
                        w.uint32(336).bool(m.php_generic_services);
                    if (m.php_metadata_namespace != null && Object.hasOwnProperty.call(m, "php_metadata_namespace"))
                        w.uint32(354).string(m.php_metadata_namespace);
                    if (m.ruby_package != null && Object.hasOwnProperty.call(m, "ruby_package"))
                        w.uint32(362).string(m.ruby_package);
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    if (m[".google.api.resource_definition"] != null && m[".google.api.resource_definition"].length) {
                        for (var i = 0; i < m[".google.api.resource_definition"].length; ++i)
                            $root.google.api.ResourceDescriptor.encode(m[".google.api.resource_definition"][i], w.uint32(8426).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified FileOptions message, length delimited. Does not implicitly {@link google.protobuf.FileOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.IFileOptions} message FileOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FileOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.FileOptions} FileOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.FileOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.java_package = r.string();
                            break;
                        case 8:
                            m.java_outer_classname = r.string();
                            break;
                        case 10:
                            m.java_multiple_files = r.bool();
                            break;
                        case 20:
                            m.java_generate_equals_and_hash = r.bool();
                            break;
                        case 27:
                            m.java_string_check_utf8 = r.bool();
                            break;
                        case 9:
                            m.optimize_for = r.int32();
                            break;
                        case 11:
                            m.go_package = r.string();
                            break;
                        case 16:
                            m.cc_generic_services = r.bool();
                            break;
                        case 17:
                            m.java_generic_services = r.bool();
                            break;
                        case 18:
                            m.py_generic_services = r.bool();
                            break;
                        case 42:
                            m.php_generic_services = r.bool();
                            break;
                        case 23:
                            m.deprecated = r.bool();
                            break;
                        case 31:
                            m.cc_enable_arenas = r.bool();
                            break;
                        case 36:
                            m.objc_class_prefix = r.string();
                            break;
                        case 37:
                            m.csharp_namespace = r.string();
                            break;
                        case 39:
                            m.swift_prefix = r.string();
                            break;
                        case 40:
                            m.php_class_prefix = r.string();
                            break;
                        case 41:
                            m.php_namespace = r.string();
                            break;
                        case 44:
                            m.php_metadata_namespace = r.string();
                            break;
                        case 45:
                            m.ruby_package = r.string();
                            break;
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        case 1053:
                            if (!(m[".google.api.resource_definition"] && m[".google.api.resource_definition"].length))
                                m[".google.api.resource_definition"] = [];
                            m[".google.api.resource_definition"].push($root.google.api.ResourceDescriptor.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a FileOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FileOptions} FileOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FileOptions message.
                 * @function verify
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FileOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.java_package != null && m.hasOwnProperty("java_package")) {
                        if (!$util.isString(m.java_package))
                            return "java_package: string expected";
                    }
                    if (m.java_outer_classname != null && m.hasOwnProperty("java_outer_classname")) {
                        if (!$util.isString(m.java_outer_classname))
                            return "java_outer_classname: string expected";
                    }
                    if (m.java_multiple_files != null && m.hasOwnProperty("java_multiple_files")) {
                        if (typeof m.java_multiple_files !== "boolean")
                            return "java_multiple_files: boolean expected";
                    }
                    if (m.java_generate_equals_and_hash != null && m.hasOwnProperty("java_generate_equals_and_hash")) {
                        if (typeof m.java_generate_equals_and_hash !== "boolean")
                            return "java_generate_equals_and_hash: boolean expected";
                    }
                    if (m.java_string_check_utf8 != null && m.hasOwnProperty("java_string_check_utf8")) {
                        if (typeof m.java_string_check_utf8 !== "boolean")
                            return "java_string_check_utf8: boolean expected";
                    }
                    if (m.optimize_for != null && m.hasOwnProperty("optimize_for")) {
                        switch (m.optimize_for) {
                        default:
                            return "optimize_for: enum value expected";
                        case 1:
                        case 2:
                        case 3:
                            break;
                        }
                    }
                    if (m.go_package != null && m.hasOwnProperty("go_package")) {
                        if (!$util.isString(m.go_package))
                            return "go_package: string expected";
                    }
                    if (m.cc_generic_services != null && m.hasOwnProperty("cc_generic_services")) {
                        if (typeof m.cc_generic_services !== "boolean")
                            return "cc_generic_services: boolean expected";
                    }
                    if (m.java_generic_services != null && m.hasOwnProperty("java_generic_services")) {
                        if (typeof m.java_generic_services !== "boolean")
                            return "java_generic_services: boolean expected";
                    }
                    if (m.py_generic_services != null && m.hasOwnProperty("py_generic_services")) {
                        if (typeof m.py_generic_services !== "boolean")
                            return "py_generic_services: boolean expected";
                    }
                    if (m.php_generic_services != null && m.hasOwnProperty("php_generic_services")) {
                        if (typeof m.php_generic_services !== "boolean")
                            return "php_generic_services: boolean expected";
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        if (typeof m.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    }
                    if (m.cc_enable_arenas != null && m.hasOwnProperty("cc_enable_arenas")) {
                        if (typeof m.cc_enable_arenas !== "boolean")
                            return "cc_enable_arenas: boolean expected";
                    }
                    if (m.objc_class_prefix != null && m.hasOwnProperty("objc_class_prefix")) {
                        if (!$util.isString(m.objc_class_prefix))
                            return "objc_class_prefix: string expected";
                    }
                    if (m.csharp_namespace != null && m.hasOwnProperty("csharp_namespace")) {
                        if (!$util.isString(m.csharp_namespace))
                            return "csharp_namespace: string expected";
                    }
                    if (m.swift_prefix != null && m.hasOwnProperty("swift_prefix")) {
                        if (!$util.isString(m.swift_prefix))
                            return "swift_prefix: string expected";
                    }
                    if (m.php_class_prefix != null && m.hasOwnProperty("php_class_prefix")) {
                        if (!$util.isString(m.php_class_prefix))
                            return "php_class_prefix: string expected";
                    }
                    if (m.php_namespace != null && m.hasOwnProperty("php_namespace")) {
                        if (!$util.isString(m.php_namespace))
                            return "php_namespace: string expected";
                    }
                    if (m.php_metadata_namespace != null && m.hasOwnProperty("php_metadata_namespace")) {
                        if (!$util.isString(m.php_metadata_namespace))
                            return "php_metadata_namespace: string expected";
                    }
                    if (m.ruby_package != null && m.hasOwnProperty("ruby_package")) {
                        if (!$util.isString(m.ruby_package))
                            return "ruby_package: string expected";
                    }
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    if (m[".google.api.resource_definition"] != null && m.hasOwnProperty(".google.api.resource_definition")) {
                        if (!Array.isArray(m[".google.api.resource_definition"]))
                            return ".google.api.resource_definition: array expected";
                        for (var i = 0; i < m[".google.api.resource_definition"].length; ++i) {
                            {
                                var e = $root.google.api.ResourceDescriptor.verify(m[".google.api.resource_definition"][i]);
                                if (e)
                                    return ".google.api.resource_definition." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a FileOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.FileOptions} FileOptions
                 */
                FileOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.FileOptions)
                        return d;
                    var m = new $root.google.protobuf.FileOptions();
                    if (d.java_package != null) {
                        m.java_package = String(d.java_package);
                    }
                    if (d.java_outer_classname != null) {
                        m.java_outer_classname = String(d.java_outer_classname);
                    }
                    if (d.java_multiple_files != null) {
                        m.java_multiple_files = Boolean(d.java_multiple_files);
                    }
                    if (d.java_generate_equals_and_hash != null) {
                        m.java_generate_equals_and_hash = Boolean(d.java_generate_equals_and_hash);
                    }
                    if (d.java_string_check_utf8 != null) {
                        m.java_string_check_utf8 = Boolean(d.java_string_check_utf8);
                    }
                    switch (d.optimize_for) {
                    case "SPEED":
                    case 1:
                        m.optimize_for = 1;
                        break;
                    case "CODE_SIZE":
                    case 2:
                        m.optimize_for = 2;
                        break;
                    case "LITE_RUNTIME":
                    case 3:
                        m.optimize_for = 3;
                        break;
                    }
                    if (d.go_package != null) {
                        m.go_package = String(d.go_package);
                    }
                    if (d.cc_generic_services != null) {
                        m.cc_generic_services = Boolean(d.cc_generic_services);
                    }
                    if (d.java_generic_services != null) {
                        m.java_generic_services = Boolean(d.java_generic_services);
                    }
                    if (d.py_generic_services != null) {
                        m.py_generic_services = Boolean(d.py_generic_services);
                    }
                    if (d.php_generic_services != null) {
                        m.php_generic_services = Boolean(d.php_generic_services);
                    }
                    if (d.deprecated != null) {
                        m.deprecated = Boolean(d.deprecated);
                    }
                    if (d.cc_enable_arenas != null) {
                        m.cc_enable_arenas = Boolean(d.cc_enable_arenas);
                    }
                    if (d.objc_class_prefix != null) {
                        m.objc_class_prefix = String(d.objc_class_prefix);
                    }
                    if (d.csharp_namespace != null) {
                        m.csharp_namespace = String(d.csharp_namespace);
                    }
                    if (d.swift_prefix != null) {
                        m.swift_prefix = String(d.swift_prefix);
                    }
                    if (d.php_class_prefix != null) {
                        m.php_class_prefix = String(d.php_class_prefix);
                    }
                    if (d.php_namespace != null) {
                        m.php_namespace = String(d.php_namespace);
                    }
                    if (d.php_metadata_namespace != null) {
                        m.php_metadata_namespace = String(d.php_metadata_namespace);
                    }
                    if (d.ruby_package != null) {
                        m.ruby_package = String(d.ruby_package);
                    }
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.FileOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.FileOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    if (d[".google.api.resource_definition"]) {
                        if (!Array.isArray(d[".google.api.resource_definition"]))
                            throw TypeError(".google.protobuf.FileOptions..google.api.resource_definition: array expected");
                        m[".google.api.resource_definition"] = [];
                        for (var i = 0; i < d[".google.api.resource_definition"].length; ++i) {
                            if (typeof d[".google.api.resource_definition"][i] !== "object")
                                throw TypeError(".google.protobuf.FileOptions..google.api.resource_definition: object expected");
                            m[".google.api.resource_definition"][i] = $root.google.api.ResourceDescriptor.fromObject(d[".google.api.resource_definition"][i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a FileOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.FileOptions} m FileOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FileOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                        d[".google.api.resource_definition"] = [];
                    }
                    if (o.defaults) {
                        d.java_package = "";
                        d.java_outer_classname = "";
                        d.optimize_for = o.enums === String ? "SPEED" : 1;
                        d.java_multiple_files = false;
                        d.go_package = "";
                        d.cc_generic_services = false;
                        d.java_generic_services = false;
                        d.py_generic_services = false;
                        d.java_generate_equals_and_hash = false;
                        d.deprecated = false;
                        d.java_string_check_utf8 = false;
                        d.cc_enable_arenas = true;
                        d.objc_class_prefix = "";
                        d.csharp_namespace = "";
                        d.swift_prefix = "";
                        d.php_class_prefix = "";
                        d.php_namespace = "";
                        d.php_generic_services = false;
                        d.php_metadata_namespace = "";
                        d.ruby_package = "";
                    }
                    if (m.java_package != null && m.hasOwnProperty("java_package")) {
                        d.java_package = m.java_package;
                    }
                    if (m.java_outer_classname != null && m.hasOwnProperty("java_outer_classname")) {
                        d.java_outer_classname = m.java_outer_classname;
                    }
                    if (m.optimize_for != null && m.hasOwnProperty("optimize_for")) {
                        d.optimize_for = o.enums === String ? $root.google.protobuf.FileOptions.OptimizeMode[m.optimize_for] : m.optimize_for;
                    }
                    if (m.java_multiple_files != null && m.hasOwnProperty("java_multiple_files")) {
                        d.java_multiple_files = m.java_multiple_files;
                    }
                    if (m.go_package != null && m.hasOwnProperty("go_package")) {
                        d.go_package = m.go_package;
                    }
                    if (m.cc_generic_services != null && m.hasOwnProperty("cc_generic_services")) {
                        d.cc_generic_services = m.cc_generic_services;
                    }
                    if (m.java_generic_services != null && m.hasOwnProperty("java_generic_services")) {
                        d.java_generic_services = m.java_generic_services;
                    }
                    if (m.py_generic_services != null && m.hasOwnProperty("py_generic_services")) {
                        d.py_generic_services = m.py_generic_services;
                    }
                    if (m.java_generate_equals_and_hash != null && m.hasOwnProperty("java_generate_equals_and_hash")) {
                        d.java_generate_equals_and_hash = m.java_generate_equals_and_hash;
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        d.deprecated = m.deprecated;
                    }
                    if (m.java_string_check_utf8 != null && m.hasOwnProperty("java_string_check_utf8")) {
                        d.java_string_check_utf8 = m.java_string_check_utf8;
                    }
                    if (m.cc_enable_arenas != null && m.hasOwnProperty("cc_enable_arenas")) {
                        d.cc_enable_arenas = m.cc_enable_arenas;
                    }
                    if (m.objc_class_prefix != null && m.hasOwnProperty("objc_class_prefix")) {
                        d.objc_class_prefix = m.objc_class_prefix;
                    }
                    if (m.csharp_namespace != null && m.hasOwnProperty("csharp_namespace")) {
                        d.csharp_namespace = m.csharp_namespace;
                    }
                    if (m.swift_prefix != null && m.hasOwnProperty("swift_prefix")) {
                        d.swift_prefix = m.swift_prefix;
                    }
                    if (m.php_class_prefix != null && m.hasOwnProperty("php_class_prefix")) {
                        d.php_class_prefix = m.php_class_prefix;
                    }
                    if (m.php_namespace != null && m.hasOwnProperty("php_namespace")) {
                        d.php_namespace = m.php_namespace;
                    }
                    if (m.php_generic_services != null && m.hasOwnProperty("php_generic_services")) {
                        d.php_generic_services = m.php_generic_services;
                    }
                    if (m.php_metadata_namespace != null && m.hasOwnProperty("php_metadata_namespace")) {
                        d.php_metadata_namespace = m.php_metadata_namespace;
                    }
                    if (m.ruby_package != null && m.hasOwnProperty("ruby_package")) {
                        d.ruby_package = m.ruby_package;
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    if (m[".google.api.resource_definition"] && m[".google.api.resource_definition"].length) {
                        d[".google.api.resource_definition"] = [];
                        for (var j = 0; j < m[".google.api.resource_definition"].length; ++j) {
                            d[".google.api.resource_definition"][j] = $root.google.api.ResourceDescriptor.toObject(m[".google.api.resource_definition"][j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this FileOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FileOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * OptimizeMode enum.
                 * @name google.protobuf.FileOptions.OptimizeMode
                 * @enum {number}
                 * @property {number} SPEED=1 SPEED value
                 * @property {number} CODE_SIZE=2 CODE_SIZE value
                 * @property {number} LITE_RUNTIME=3 LITE_RUNTIME value
                 */
                FileOptions.OptimizeMode = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[1] = "SPEED"] = 1;
                    values[valuesById[2] = "CODE_SIZE"] = 2;
                    values[valuesById[3] = "LITE_RUNTIME"] = 3;
                    return values;
                })();
    
                return FileOptions;
            })();
    
            protobuf.MessageOptions = (function() {
    
                /**
                 * Properties of a MessageOptions.
                 * @memberof google.protobuf
                 * @interface IMessageOptions
                 * @property {boolean|null} [message_set_wire_format] MessageOptions message_set_wire_format
                 * @property {boolean|null} [no_standard_descriptor_accessor] MessageOptions no_standard_descriptor_accessor
                 * @property {boolean|null} [deprecated] MessageOptions deprecated
                 * @property {boolean|null} [map_entry] MessageOptions map_entry
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] MessageOptions uninterpreted_option
                 * @property {google.api.IResourceDescriptor|null} [".google.api.resource"] MessageOptions .google.api.resource
                 */
    
                /**
                 * Constructs a new MessageOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a MessageOptions.
                 * @implements IMessageOptions
                 * @constructor
                 * @param {google.protobuf.IMessageOptions=} [p] Properties to set
                 */
                function MessageOptions(p) {
                    this.uninterpreted_option = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * MessageOptions message_set_wire_format.
                 * @member {boolean} message_set_wire_format
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.message_set_wire_format = false;
    
                /**
                 * MessageOptions no_standard_descriptor_accessor.
                 * @member {boolean} no_standard_descriptor_accessor
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.no_standard_descriptor_accessor = false;
    
                /**
                 * MessageOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.deprecated = false;
    
                /**
                 * MessageOptions map_entry.
                 * @member {boolean} map_entry
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.map_entry = false;
    
                /**
                 * MessageOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * MessageOptions .google.api.resource.
                 * @member {google.api.IResourceDescriptor|null|undefined} .google.api.resource
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype[".google.api.resource"] = null;
    
                /**
                 * Creates a new MessageOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.IMessageOptions=} [properties] Properties to set
                 * @returns {google.protobuf.MessageOptions} MessageOptions instance
                 */
                MessageOptions.create = function create(properties) {
                    return new MessageOptions(properties);
                };
    
                /**
                 * Encodes the specified MessageOptions message. Does not implicitly {@link google.protobuf.MessageOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.IMessageOptions} m MessageOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MessageOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.message_set_wire_format != null && Object.hasOwnProperty.call(m, "message_set_wire_format"))
                        w.uint32(8).bool(m.message_set_wire_format);
                    if (m.no_standard_descriptor_accessor != null && Object.hasOwnProperty.call(m, "no_standard_descriptor_accessor"))
                        w.uint32(16).bool(m.no_standard_descriptor_accessor);
                    if (m.deprecated != null && Object.hasOwnProperty.call(m, "deprecated"))
                        w.uint32(24).bool(m.deprecated);
                    if (m.map_entry != null && Object.hasOwnProperty.call(m, "map_entry"))
                        w.uint32(56).bool(m.map_entry);
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    if (m[".google.api.resource"] != null && Object.hasOwnProperty.call(m, ".google.api.resource"))
                        $root.google.api.ResourceDescriptor.encode(m[".google.api.resource"], w.uint32(8426).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified MessageOptions message, length delimited. Does not implicitly {@link google.protobuf.MessageOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.IMessageOptions} message MessageOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MessageOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a MessageOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.MessageOptions} MessageOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MessageOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.MessageOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.message_set_wire_format = r.bool();
                            break;
                        case 2:
                            m.no_standard_descriptor_accessor = r.bool();
                            break;
                        case 3:
                            m.deprecated = r.bool();
                            break;
                        case 7:
                            m.map_entry = r.bool();
                            break;
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        case 1053:
                            m[".google.api.resource"] = $root.google.api.ResourceDescriptor.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a MessageOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.MessageOptions} MessageOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MessageOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a MessageOptions message.
                 * @function verify
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                MessageOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.message_set_wire_format != null && m.hasOwnProperty("message_set_wire_format")) {
                        if (typeof m.message_set_wire_format !== "boolean")
                            return "message_set_wire_format: boolean expected";
                    }
                    if (m.no_standard_descriptor_accessor != null && m.hasOwnProperty("no_standard_descriptor_accessor")) {
                        if (typeof m.no_standard_descriptor_accessor !== "boolean")
                            return "no_standard_descriptor_accessor: boolean expected";
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        if (typeof m.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    }
                    if (m.map_entry != null && m.hasOwnProperty("map_entry")) {
                        if (typeof m.map_entry !== "boolean")
                            return "map_entry: boolean expected";
                    }
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    if (m[".google.api.resource"] != null && m.hasOwnProperty(".google.api.resource")) {
                        {
                            var e = $root.google.api.ResourceDescriptor.verify(m[".google.api.resource"]);
                            if (e)
                                return ".google.api.resource." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a MessageOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.MessageOptions} MessageOptions
                 */
                MessageOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.MessageOptions)
                        return d;
                    var m = new $root.google.protobuf.MessageOptions();
                    if (d.message_set_wire_format != null) {
                        m.message_set_wire_format = Boolean(d.message_set_wire_format);
                    }
                    if (d.no_standard_descriptor_accessor != null) {
                        m.no_standard_descriptor_accessor = Boolean(d.no_standard_descriptor_accessor);
                    }
                    if (d.deprecated != null) {
                        m.deprecated = Boolean(d.deprecated);
                    }
                    if (d.map_entry != null) {
                        m.map_entry = Boolean(d.map_entry);
                    }
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.MessageOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.MessageOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    if (d[".google.api.resource"] != null) {
                        if (typeof d[".google.api.resource"] !== "object")
                            throw TypeError(".google.protobuf.MessageOptions..google.api.resource: object expected");
                        m[".google.api.resource"] = $root.google.api.ResourceDescriptor.fromObject(d[".google.api.resource"]);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a MessageOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.MessageOptions} m MessageOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                MessageOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                    }
                    if (o.defaults) {
                        d.message_set_wire_format = false;
                        d.no_standard_descriptor_accessor = false;
                        d.deprecated = false;
                        d.map_entry = false;
                        d[".google.api.resource"] = null;
                    }
                    if (m.message_set_wire_format != null && m.hasOwnProperty("message_set_wire_format")) {
                        d.message_set_wire_format = m.message_set_wire_format;
                    }
                    if (m.no_standard_descriptor_accessor != null && m.hasOwnProperty("no_standard_descriptor_accessor")) {
                        d.no_standard_descriptor_accessor = m.no_standard_descriptor_accessor;
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        d.deprecated = m.deprecated;
                    }
                    if (m.map_entry != null && m.hasOwnProperty("map_entry")) {
                        d.map_entry = m.map_entry;
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    if (m[".google.api.resource"] != null && m.hasOwnProperty(".google.api.resource")) {
                        d[".google.api.resource"] = $root.google.api.ResourceDescriptor.toObject(m[".google.api.resource"], o);
                    }
                    return d;
                };
    
                /**
                 * Converts this MessageOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                MessageOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return MessageOptions;
            })();
    
            protobuf.FieldOptions = (function() {
    
                /**
                 * Properties of a FieldOptions.
                 * @memberof google.protobuf
                 * @interface IFieldOptions
                 * @property {google.protobuf.FieldOptions.CType|null} [ctype] FieldOptions ctype
                 * @property {boolean|null} [packed] FieldOptions packed
                 * @property {google.protobuf.FieldOptions.JSType|null} [jstype] FieldOptions jstype
                 * @property {boolean|null} [lazy] FieldOptions lazy
                 * @property {boolean|null} [unverified_lazy] FieldOptions unverified_lazy
                 * @property {boolean|null} [deprecated] FieldOptions deprecated
                 * @property {boolean|null} [weak] FieldOptions weak
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] FieldOptions uninterpreted_option
                 * @property {Array.<google.api.FieldBehavior>|null} [".google.api.field_behavior"] FieldOptions .google.api.field_behavior
                 * @property {google.api.IResourceReference|null} [".google.api.resource_reference"] FieldOptions .google.api.resource_reference
                 */
    
                /**
                 * Constructs a new FieldOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a FieldOptions.
                 * @implements IFieldOptions
                 * @constructor
                 * @param {google.protobuf.IFieldOptions=} [p] Properties to set
                 */
                function FieldOptions(p) {
                    this.uninterpreted_option = [];
                    this[".google.api.field_behavior"] = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * FieldOptions ctype.
                 * @member {google.protobuf.FieldOptions.CType} ctype
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.ctype = 0;
    
                /**
                 * FieldOptions packed.
                 * @member {boolean} packed
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.packed = false;
    
                /**
                 * FieldOptions jstype.
                 * @member {google.protobuf.FieldOptions.JSType} jstype
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.jstype = 0;
    
                /**
                 * FieldOptions lazy.
                 * @member {boolean} lazy
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.lazy = false;
    
                /**
                 * FieldOptions unverified_lazy.
                 * @member {boolean} unverified_lazy
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.unverified_lazy = false;
    
                /**
                 * FieldOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.deprecated = false;
    
                /**
                 * FieldOptions weak.
                 * @member {boolean} weak
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.weak = false;
    
                /**
                 * FieldOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * FieldOptions .google.api.field_behavior.
                 * @member {Array.<google.api.FieldBehavior>} .google.api.field_behavior
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype[".google.api.field_behavior"] = $util.emptyArray;
    
                /**
                 * FieldOptions .google.api.resource_reference.
                 * @member {google.api.IResourceReference|null|undefined} .google.api.resource_reference
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype[".google.api.resource_reference"] = null;
    
                /**
                 * Creates a new FieldOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.IFieldOptions=} [properties] Properties to set
                 * @returns {google.protobuf.FieldOptions} FieldOptions instance
                 */
                FieldOptions.create = function create(properties) {
                    return new FieldOptions(properties);
                };
    
                /**
                 * Encodes the specified FieldOptions message. Does not implicitly {@link google.protobuf.FieldOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.IFieldOptions} m FieldOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.ctype != null && Object.hasOwnProperty.call(m, "ctype"))
                        w.uint32(8).int32(m.ctype);
                    if (m.packed != null && Object.hasOwnProperty.call(m, "packed"))
                        w.uint32(16).bool(m.packed);
                    if (m.deprecated != null && Object.hasOwnProperty.call(m, "deprecated"))
                        w.uint32(24).bool(m.deprecated);
                    if (m.lazy != null && Object.hasOwnProperty.call(m, "lazy"))
                        w.uint32(40).bool(m.lazy);
                    if (m.jstype != null && Object.hasOwnProperty.call(m, "jstype"))
                        w.uint32(48).int32(m.jstype);
                    if (m.weak != null && Object.hasOwnProperty.call(m, "weak"))
                        w.uint32(80).bool(m.weak);
                    if (m.unverified_lazy != null && Object.hasOwnProperty.call(m, "unverified_lazy"))
                        w.uint32(120).bool(m.unverified_lazy);
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    if (m[".google.api.field_behavior"] != null && m[".google.api.field_behavior"].length) {
                        w.uint32(8418).fork();
                        for (var i = 0; i < m[".google.api.field_behavior"].length; ++i)
                            w.int32(m[".google.api.field_behavior"][i]);
                        w.ldelim();
                    }
                    if (m[".google.api.resource_reference"] != null && Object.hasOwnProperty.call(m, ".google.api.resource_reference"))
                        $root.google.api.ResourceReference.encode(m[".google.api.resource_reference"], w.uint32(8442).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified FieldOptions message, length delimited. Does not implicitly {@link google.protobuf.FieldOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.IFieldOptions} message FieldOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FieldOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.FieldOptions} FieldOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.FieldOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.ctype = r.int32();
                            break;
                        case 2:
                            m.packed = r.bool();
                            break;
                        case 6:
                            m.jstype = r.int32();
                            break;
                        case 5:
                            m.lazy = r.bool();
                            break;
                        case 15:
                            m.unverified_lazy = r.bool();
                            break;
                        case 3:
                            m.deprecated = r.bool();
                            break;
                        case 10:
                            m.weak = r.bool();
                            break;
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        case 1052:
                            if (!(m[".google.api.field_behavior"] && m[".google.api.field_behavior"].length))
                                m[".google.api.field_behavior"] = [];
                            if ((t & 7) === 2) {
                                var c2 = r.uint32() + r.pos;
                                while (r.pos < c2)
                                    m[".google.api.field_behavior"].push(r.int32());
                            } else
                                m[".google.api.field_behavior"].push(r.int32());
                            break;
                        case 1055:
                            m[".google.api.resource_reference"] = $root.google.api.ResourceReference.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a FieldOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FieldOptions} FieldOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FieldOptions message.
                 * @function verify
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FieldOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.ctype != null && m.hasOwnProperty("ctype")) {
                        switch (m.ctype) {
                        default:
                            return "ctype: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    }
                    if (m.packed != null && m.hasOwnProperty("packed")) {
                        if (typeof m.packed !== "boolean")
                            return "packed: boolean expected";
                    }
                    if (m.jstype != null && m.hasOwnProperty("jstype")) {
                        switch (m.jstype) {
                        default:
                            return "jstype: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    }
                    if (m.lazy != null && m.hasOwnProperty("lazy")) {
                        if (typeof m.lazy !== "boolean")
                            return "lazy: boolean expected";
                    }
                    if (m.unverified_lazy != null && m.hasOwnProperty("unverified_lazy")) {
                        if (typeof m.unverified_lazy !== "boolean")
                            return "unverified_lazy: boolean expected";
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        if (typeof m.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    }
                    if (m.weak != null && m.hasOwnProperty("weak")) {
                        if (typeof m.weak !== "boolean")
                            return "weak: boolean expected";
                    }
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    if (m[".google.api.field_behavior"] != null && m.hasOwnProperty(".google.api.field_behavior")) {
                        if (!Array.isArray(m[".google.api.field_behavior"]))
                            return ".google.api.field_behavior: array expected";
                        for (var i = 0; i < m[".google.api.field_behavior"].length; ++i) {
                            switch (m[".google.api.field_behavior"][i]) {
                            default:
                                return ".google.api.field_behavior: enum value[] expected";
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                                break;
                            }
                        }
                    }
                    if (m[".google.api.resource_reference"] != null && m.hasOwnProperty(".google.api.resource_reference")) {
                        {
                            var e = $root.google.api.ResourceReference.verify(m[".google.api.resource_reference"]);
                            if (e)
                                return ".google.api.resource_reference." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a FieldOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.FieldOptions} FieldOptions
                 */
                FieldOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.FieldOptions)
                        return d;
                    var m = new $root.google.protobuf.FieldOptions();
                    switch (d.ctype) {
                    case "STRING":
                    case 0:
                        m.ctype = 0;
                        break;
                    case "CORD":
                    case 1:
                        m.ctype = 1;
                        break;
                    case "STRING_PIECE":
                    case 2:
                        m.ctype = 2;
                        break;
                    }
                    if (d.packed != null) {
                        m.packed = Boolean(d.packed);
                    }
                    switch (d.jstype) {
                    case "JS_NORMAL":
                    case 0:
                        m.jstype = 0;
                        break;
                    case "JS_STRING":
                    case 1:
                        m.jstype = 1;
                        break;
                    case "JS_NUMBER":
                    case 2:
                        m.jstype = 2;
                        break;
                    }
                    if (d.lazy != null) {
                        m.lazy = Boolean(d.lazy);
                    }
                    if (d.unverified_lazy != null) {
                        m.unverified_lazy = Boolean(d.unverified_lazy);
                    }
                    if (d.deprecated != null) {
                        m.deprecated = Boolean(d.deprecated);
                    }
                    if (d.weak != null) {
                        m.weak = Boolean(d.weak);
                    }
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.FieldOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.FieldOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    if (d[".google.api.field_behavior"]) {
                        if (!Array.isArray(d[".google.api.field_behavior"]))
                            throw TypeError(".google.protobuf.FieldOptions..google.api.field_behavior: array expected");
                        m[".google.api.field_behavior"] = [];
                        for (var i = 0; i < d[".google.api.field_behavior"].length; ++i) {
                            switch (d[".google.api.field_behavior"][i]) {
                            default:
                            case "FIELD_BEHAVIOR_UNSPECIFIED":
                            case 0:
                                m[".google.api.field_behavior"][i] = 0;
                                break;
                            case "OPTIONAL":
                            case 1:
                                m[".google.api.field_behavior"][i] = 1;
                                break;
                            case "REQUIRED":
                            case 2:
                                m[".google.api.field_behavior"][i] = 2;
                                break;
                            case "OUTPUT_ONLY":
                            case 3:
                                m[".google.api.field_behavior"][i] = 3;
                                break;
                            case "INPUT_ONLY":
                            case 4:
                                m[".google.api.field_behavior"][i] = 4;
                                break;
                            case "IMMUTABLE":
                            case 5:
                                m[".google.api.field_behavior"][i] = 5;
                                break;
                            case "UNORDERED_LIST":
                            case 6:
                                m[".google.api.field_behavior"][i] = 6;
                                break;
                            case "NON_EMPTY_DEFAULT":
                            case 7:
                                m[".google.api.field_behavior"][i] = 7;
                                break;
                            }
                        }
                    }
                    if (d[".google.api.resource_reference"] != null) {
                        if (typeof d[".google.api.resource_reference"] !== "object")
                            throw TypeError(".google.protobuf.FieldOptions..google.api.resource_reference: object expected");
                        m[".google.api.resource_reference"] = $root.google.api.ResourceReference.fromObject(d[".google.api.resource_reference"]);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a FieldOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.FieldOptions} m FieldOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FieldOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                        d[".google.api.field_behavior"] = [];
                    }
                    if (o.defaults) {
                        d.ctype = o.enums === String ? "STRING" : 0;
                        d.packed = false;
                        d.deprecated = false;
                        d.lazy = false;
                        d.jstype = o.enums === String ? "JS_NORMAL" : 0;
                        d.weak = false;
                        d.unverified_lazy = false;
                        d[".google.api.resource_reference"] = null;
                    }
                    if (m.ctype != null && m.hasOwnProperty("ctype")) {
                        d.ctype = o.enums === String ? $root.google.protobuf.FieldOptions.CType[m.ctype] : m.ctype;
                    }
                    if (m.packed != null && m.hasOwnProperty("packed")) {
                        d.packed = m.packed;
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        d.deprecated = m.deprecated;
                    }
                    if (m.lazy != null && m.hasOwnProperty("lazy")) {
                        d.lazy = m.lazy;
                    }
                    if (m.jstype != null && m.hasOwnProperty("jstype")) {
                        d.jstype = o.enums === String ? $root.google.protobuf.FieldOptions.JSType[m.jstype] : m.jstype;
                    }
                    if (m.weak != null && m.hasOwnProperty("weak")) {
                        d.weak = m.weak;
                    }
                    if (m.unverified_lazy != null && m.hasOwnProperty("unverified_lazy")) {
                        d.unverified_lazy = m.unverified_lazy;
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    if (m[".google.api.field_behavior"] && m[".google.api.field_behavior"].length) {
                        d[".google.api.field_behavior"] = [];
                        for (var j = 0; j < m[".google.api.field_behavior"].length; ++j) {
                            d[".google.api.field_behavior"][j] = o.enums === String ? $root.google.api.FieldBehavior[m[".google.api.field_behavior"][j]] : m[".google.api.field_behavior"][j];
                        }
                    }
                    if (m[".google.api.resource_reference"] != null && m.hasOwnProperty(".google.api.resource_reference")) {
                        d[".google.api.resource_reference"] = $root.google.api.ResourceReference.toObject(m[".google.api.resource_reference"], o);
                    }
                    return d;
                };
    
                /**
                 * Converts this FieldOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FieldOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * CType enum.
                 * @name google.protobuf.FieldOptions.CType
                 * @enum {number}
                 * @property {number} STRING=0 STRING value
                 * @property {number} CORD=1 CORD value
                 * @property {number} STRING_PIECE=2 STRING_PIECE value
                 */
                FieldOptions.CType = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "STRING"] = 0;
                    values[valuesById[1] = "CORD"] = 1;
                    values[valuesById[2] = "STRING_PIECE"] = 2;
                    return values;
                })();
    
                /**
                 * JSType enum.
                 * @name google.protobuf.FieldOptions.JSType
                 * @enum {number}
                 * @property {number} JS_NORMAL=0 JS_NORMAL value
                 * @property {number} JS_STRING=1 JS_STRING value
                 * @property {number} JS_NUMBER=2 JS_NUMBER value
                 */
                FieldOptions.JSType = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "JS_NORMAL"] = 0;
                    values[valuesById[1] = "JS_STRING"] = 1;
                    values[valuesById[2] = "JS_NUMBER"] = 2;
                    return values;
                })();
    
                return FieldOptions;
            })();
    
            protobuf.OneofOptions = (function() {
    
                /**
                 * Properties of an OneofOptions.
                 * @memberof google.protobuf
                 * @interface IOneofOptions
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] OneofOptions uninterpreted_option
                 */
    
                /**
                 * Constructs a new OneofOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an OneofOptions.
                 * @implements IOneofOptions
                 * @constructor
                 * @param {google.protobuf.IOneofOptions=} [p] Properties to set
                 */
                function OneofOptions(p) {
                    this.uninterpreted_option = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * OneofOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.OneofOptions
                 * @instance
                 */
                OneofOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * Creates a new OneofOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.IOneofOptions=} [properties] Properties to set
                 * @returns {google.protobuf.OneofOptions} OneofOptions instance
                 */
                OneofOptions.create = function create(properties) {
                    return new OneofOptions(properties);
                };
    
                /**
                 * Encodes the specified OneofOptions message. Does not implicitly {@link google.protobuf.OneofOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.IOneofOptions} m OneofOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified OneofOptions message, length delimited. Does not implicitly {@link google.protobuf.OneofOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.IOneofOptions} message OneofOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an OneofOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.OneofOptions} OneofOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.OneofOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an OneofOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.OneofOptions} OneofOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an OneofOptions message.
                 * @function verify
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                OneofOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an OneofOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.OneofOptions} OneofOptions
                 */
                OneofOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.OneofOptions)
                        return d;
                    var m = new $root.google.protobuf.OneofOptions();
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.OneofOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.OneofOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an OneofOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.OneofOptions} m OneofOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                OneofOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this OneofOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.OneofOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                OneofOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return OneofOptions;
            })();
    
            protobuf.EnumOptions = (function() {
    
                /**
                 * Properties of an EnumOptions.
                 * @memberof google.protobuf
                 * @interface IEnumOptions
                 * @property {boolean|null} [allow_alias] EnumOptions allow_alias
                 * @property {boolean|null} [deprecated] EnumOptions deprecated
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] EnumOptions uninterpreted_option
                 */
    
                /**
                 * Constructs a new EnumOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumOptions.
                 * @implements IEnumOptions
                 * @constructor
                 * @param {google.protobuf.IEnumOptions=} [p] Properties to set
                 */
                function EnumOptions(p) {
                    this.uninterpreted_option = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * EnumOptions allow_alias.
                 * @member {boolean} allow_alias
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.allow_alias = false;
    
                /**
                 * EnumOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.deprecated = false;
    
                /**
                 * EnumOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * Creates a new EnumOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.IEnumOptions=} [properties] Properties to set
                 * @returns {google.protobuf.EnumOptions} EnumOptions instance
                 */
                EnumOptions.create = function create(properties) {
                    return new EnumOptions(properties);
                };
    
                /**
                 * Encodes the specified EnumOptions message. Does not implicitly {@link google.protobuf.EnumOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.IEnumOptions} m EnumOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.allow_alias != null && Object.hasOwnProperty.call(m, "allow_alias"))
                        w.uint32(16).bool(m.allow_alias);
                    if (m.deprecated != null && Object.hasOwnProperty.call(m, "deprecated"))
                        w.uint32(24).bool(m.deprecated);
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified EnumOptions message, length delimited. Does not implicitly {@link google.protobuf.EnumOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.IEnumOptions} message EnumOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.EnumOptions} EnumOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.EnumOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 2:
                            m.allow_alias = r.bool();
                            break;
                        case 3:
                            m.deprecated = r.bool();
                            break;
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an EnumOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumOptions} EnumOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumOptions message.
                 * @function verify
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.allow_alias != null && m.hasOwnProperty("allow_alias")) {
                        if (typeof m.allow_alias !== "boolean")
                            return "allow_alias: boolean expected";
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        if (typeof m.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    }
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.EnumOptions} EnumOptions
                 */
                EnumOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.EnumOptions)
                        return d;
                    var m = new $root.google.protobuf.EnumOptions();
                    if (d.allow_alias != null) {
                        m.allow_alias = Boolean(d.allow_alias);
                    }
                    if (d.deprecated != null) {
                        m.deprecated = Boolean(d.deprecated);
                    }
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.EnumOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.EnumOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an EnumOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.EnumOptions} m EnumOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                    }
                    if (o.defaults) {
                        d.allow_alias = false;
                        d.deprecated = false;
                    }
                    if (m.allow_alias != null && m.hasOwnProperty("allow_alias")) {
                        d.allow_alias = m.allow_alias;
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        d.deprecated = m.deprecated;
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this EnumOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return EnumOptions;
            })();
    
            protobuf.EnumValueOptions = (function() {
    
                /**
                 * Properties of an EnumValueOptions.
                 * @memberof google.protobuf
                 * @interface IEnumValueOptions
                 * @property {boolean|null} [deprecated] EnumValueOptions deprecated
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] EnumValueOptions uninterpreted_option
                 */
    
                /**
                 * Constructs a new EnumValueOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumValueOptions.
                 * @implements IEnumValueOptions
                 * @constructor
                 * @param {google.protobuf.IEnumValueOptions=} [p] Properties to set
                 */
                function EnumValueOptions(p) {
                    this.uninterpreted_option = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * EnumValueOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 */
                EnumValueOptions.prototype.deprecated = false;
    
                /**
                 * EnumValueOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 */
                EnumValueOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * Creates a new EnumValueOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.IEnumValueOptions=} [properties] Properties to set
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions instance
                 */
                EnumValueOptions.create = function create(properties) {
                    return new EnumValueOptions(properties);
                };
    
                /**
                 * Encodes the specified EnumValueOptions message. Does not implicitly {@link google.protobuf.EnumValueOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.IEnumValueOptions} m EnumValueOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.deprecated != null && Object.hasOwnProperty.call(m, "deprecated"))
                        w.uint32(8).bool(m.deprecated);
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified EnumValueOptions message, length delimited. Does not implicitly {@link google.protobuf.EnumValueOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.IEnumValueOptions} message EnumValueOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumValueOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.EnumValueOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.deprecated = r.bool();
                            break;
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an EnumValueOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumValueOptions message.
                 * @function verify
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumValueOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        if (typeof m.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    }
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumValueOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions
                 */
                EnumValueOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.EnumValueOptions)
                        return d;
                    var m = new $root.google.protobuf.EnumValueOptions();
                    if (d.deprecated != null) {
                        m.deprecated = Boolean(d.deprecated);
                    }
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.EnumValueOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.EnumValueOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an EnumValueOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.EnumValueOptions} m EnumValueOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumValueOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                    }
                    if (o.defaults) {
                        d.deprecated = false;
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        d.deprecated = m.deprecated;
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this EnumValueOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumValueOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return EnumValueOptions;
            })();
    
            protobuf.ServiceOptions = (function() {
    
                /**
                 * Properties of a ServiceOptions.
                 * @memberof google.protobuf
                 * @interface IServiceOptions
                 * @property {boolean|null} [deprecated] ServiceOptions deprecated
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] ServiceOptions uninterpreted_option
                 * @property {string|null} [".google.api.default_host"] ServiceOptions .google.api.default_host
                 * @property {string|null} [".google.api.oauth_scopes"] ServiceOptions .google.api.oauth_scopes
                 */
    
                /**
                 * Constructs a new ServiceOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a ServiceOptions.
                 * @implements IServiceOptions
                 * @constructor
                 * @param {google.protobuf.IServiceOptions=} [p] Properties to set
                 */
                function ServiceOptions(p) {
                    this.uninterpreted_option = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * ServiceOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype.deprecated = false;
    
                /**
                 * ServiceOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * ServiceOptions .google.api.default_host.
                 * @member {string} .google.api.default_host
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype[".google.api.default_host"] = "";
    
                /**
                 * ServiceOptions .google.api.oauth_scopes.
                 * @member {string} .google.api.oauth_scopes
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype[".google.api.oauth_scopes"] = "";
    
                /**
                 * Creates a new ServiceOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.IServiceOptions=} [properties] Properties to set
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions instance
                 */
                ServiceOptions.create = function create(properties) {
                    return new ServiceOptions(properties);
                };
    
                /**
                 * Encodes the specified ServiceOptions message. Does not implicitly {@link google.protobuf.ServiceOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.IServiceOptions} m ServiceOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.deprecated != null && Object.hasOwnProperty.call(m, "deprecated"))
                        w.uint32(264).bool(m.deprecated);
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    if (m[".google.api.default_host"] != null && Object.hasOwnProperty.call(m, ".google.api.default_host"))
                        w.uint32(8394).string(m[".google.api.default_host"]);
                    if (m[".google.api.oauth_scopes"] != null && Object.hasOwnProperty.call(m, ".google.api.oauth_scopes"))
                        w.uint32(8402).string(m[".google.api.oauth_scopes"]);
                    return w;
                };
    
                /**
                 * Encodes the specified ServiceOptions message, length delimited. Does not implicitly {@link google.protobuf.ServiceOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.IServiceOptions} message ServiceOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ServiceOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.ServiceOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 33:
                            m.deprecated = r.bool();
                            break;
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        case 1049:
                            m[".google.api.default_host"] = r.string();
                            break;
                        case 1050:
                            m[".google.api.oauth_scopes"] = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a ServiceOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ServiceOptions message.
                 * @function verify
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ServiceOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        if (typeof m.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    }
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    if (m[".google.api.default_host"] != null && m.hasOwnProperty(".google.api.default_host")) {
                        if (!$util.isString(m[".google.api.default_host"]))
                            return ".google.api.default_host: string expected";
                    }
                    if (m[".google.api.oauth_scopes"] != null && m.hasOwnProperty(".google.api.oauth_scopes")) {
                        if (!$util.isString(m[".google.api.oauth_scopes"]))
                            return ".google.api.oauth_scopes: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a ServiceOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions
                 */
                ServiceOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.ServiceOptions)
                        return d;
                    var m = new $root.google.protobuf.ServiceOptions();
                    if (d.deprecated != null) {
                        m.deprecated = Boolean(d.deprecated);
                    }
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.ServiceOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.ServiceOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    if (d[".google.api.default_host"] != null) {
                        m[".google.api.default_host"] = String(d[".google.api.default_host"]);
                    }
                    if (d[".google.api.oauth_scopes"] != null) {
                        m[".google.api.oauth_scopes"] = String(d[".google.api.oauth_scopes"]);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a ServiceOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.ServiceOptions} m ServiceOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ServiceOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                    }
                    if (o.defaults) {
                        d.deprecated = false;
                        d[".google.api.default_host"] = "";
                        d[".google.api.oauth_scopes"] = "";
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        d.deprecated = m.deprecated;
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    if (m[".google.api.default_host"] != null && m.hasOwnProperty(".google.api.default_host")) {
                        d[".google.api.default_host"] = m[".google.api.default_host"];
                    }
                    if (m[".google.api.oauth_scopes"] != null && m.hasOwnProperty(".google.api.oauth_scopes")) {
                        d[".google.api.oauth_scopes"] = m[".google.api.oauth_scopes"];
                    }
                    return d;
                };
    
                /**
                 * Converts this ServiceOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ServiceOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return ServiceOptions;
            })();
    
            protobuf.MethodOptions = (function() {
    
                /**
                 * Properties of a MethodOptions.
                 * @memberof google.protobuf
                 * @interface IMethodOptions
                 * @property {boolean|null} [deprecated] MethodOptions deprecated
                 * @property {google.protobuf.MethodOptions.IdempotencyLevel|null} [idempotency_level] MethodOptions idempotency_level
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpreted_option] MethodOptions uninterpreted_option
                 * @property {google.api.IHttpRule|null} [".google.api.http"] MethodOptions .google.api.http
                 * @property {Array.<string>|null} [".google.api.method_signature"] MethodOptions .google.api.method_signature
                 * @property {google.longrunning.IOperationInfo|null} [".google.longrunning.operation_info"] MethodOptions .google.longrunning.operation_info
                 */
    
                /**
                 * Constructs a new MethodOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a MethodOptions.
                 * @implements IMethodOptions
                 * @constructor
                 * @param {google.protobuf.IMethodOptions=} [p] Properties to set
                 */
                function MethodOptions(p) {
                    this.uninterpreted_option = [];
                    this[".google.api.method_signature"] = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * MethodOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype.deprecated = false;
    
                /**
                 * MethodOptions idempotency_level.
                 * @member {google.protobuf.MethodOptions.IdempotencyLevel} idempotency_level
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype.idempotency_level = 0;
    
                /**
                 * MethodOptions uninterpreted_option.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpreted_option
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype.uninterpreted_option = $util.emptyArray;
    
                /**
                 * MethodOptions .google.api.http.
                 * @member {google.api.IHttpRule|null|undefined} .google.api.http
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype[".google.api.http"] = null;
    
                /**
                 * MethodOptions .google.api.method_signature.
                 * @member {Array.<string>} .google.api.method_signature
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype[".google.api.method_signature"] = $util.emptyArray;
    
                /**
                 * MethodOptions .google.longrunning.operation_info.
                 * @member {google.longrunning.IOperationInfo|null|undefined} .google.longrunning.operation_info
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype[".google.longrunning.operation_info"] = null;
    
                /**
                 * Creates a new MethodOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.IMethodOptions=} [properties] Properties to set
                 * @returns {google.protobuf.MethodOptions} MethodOptions instance
                 */
                MethodOptions.create = function create(properties) {
                    return new MethodOptions(properties);
                };
    
                /**
                 * Encodes the specified MethodOptions message. Does not implicitly {@link google.protobuf.MethodOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.IMethodOptions} m MethodOptions message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodOptions.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.deprecated != null && Object.hasOwnProperty.call(m, "deprecated"))
                        w.uint32(264).bool(m.deprecated);
                    if (m.idempotency_level != null && Object.hasOwnProperty.call(m, "idempotency_level"))
                        w.uint32(272).int32(m.idempotency_level);
                    if (m.uninterpreted_option != null && m.uninterpreted_option.length) {
                        for (var i = 0; i < m.uninterpreted_option.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(m.uninterpreted_option[i], w.uint32(7994).fork()).ldelim();
                    }
                    if (m[".google.longrunning.operation_info"] != null && Object.hasOwnProperty.call(m, ".google.longrunning.operation_info"))
                        $root.google.longrunning.OperationInfo.encode(m[".google.longrunning.operation_info"], w.uint32(8394).fork()).ldelim();
                    if (m[".google.api.method_signature"] != null && m[".google.api.method_signature"].length) {
                        for (var i = 0; i < m[".google.api.method_signature"].length; ++i)
                            w.uint32(8410).string(m[".google.api.method_signature"][i]);
                    }
                    if (m[".google.api.http"] != null && Object.hasOwnProperty.call(m, ".google.api.http"))
                        $root.google.api.HttpRule.encode(m[".google.api.http"], w.uint32(578365826).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified MethodOptions message, length delimited. Does not implicitly {@link google.protobuf.MethodOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.IMethodOptions} message MethodOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a MethodOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.MethodOptions} MethodOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodOptions.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.MethodOptions();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 33:
                            m.deprecated = r.bool();
                            break;
                        case 34:
                            m.idempotency_level = r.int32();
                            break;
                        case 999:
                            if (!(m.uninterpreted_option && m.uninterpreted_option.length))
                                m.uninterpreted_option = [];
                            m.uninterpreted_option.push($root.google.protobuf.UninterpretedOption.decode(r, r.uint32()));
                            break;
                        case 72295728:
                            m[".google.api.http"] = $root.google.api.HttpRule.decode(r, r.uint32());
                            break;
                        case 1051:
                            if (!(m[".google.api.method_signature"] && m[".google.api.method_signature"].length))
                                m[".google.api.method_signature"] = [];
                            m[".google.api.method_signature"].push(r.string());
                            break;
                        case 1049:
                            m[".google.longrunning.operation_info"] = $root.google.longrunning.OperationInfo.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a MethodOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.MethodOptions} MethodOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a MethodOptions message.
                 * @function verify
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                MethodOptions.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        if (typeof m.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    }
                    if (m.idempotency_level != null && m.hasOwnProperty("idempotency_level")) {
                        switch (m.idempotency_level) {
                        default:
                            return "idempotency_level: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    }
                    if (m.uninterpreted_option != null && m.hasOwnProperty("uninterpreted_option")) {
                        if (!Array.isArray(m.uninterpreted_option))
                            return "uninterpreted_option: array expected";
                        for (var i = 0; i < m.uninterpreted_option.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.verify(m.uninterpreted_option[i]);
                                if (e)
                                    return "uninterpreted_option." + e;
                            }
                        }
                    }
                    if (m[".google.api.http"] != null && m.hasOwnProperty(".google.api.http")) {
                        {
                            var e = $root.google.api.HttpRule.verify(m[".google.api.http"]);
                            if (e)
                                return ".google.api.http." + e;
                        }
                    }
                    if (m[".google.api.method_signature"] != null && m.hasOwnProperty(".google.api.method_signature")) {
                        if (!Array.isArray(m[".google.api.method_signature"]))
                            return ".google.api.method_signature: array expected";
                        for (var i = 0; i < m[".google.api.method_signature"].length; ++i) {
                            if (!$util.isString(m[".google.api.method_signature"][i]))
                                return ".google.api.method_signature: string[] expected";
                        }
                    }
                    if (m[".google.longrunning.operation_info"] != null && m.hasOwnProperty(".google.longrunning.operation_info")) {
                        {
                            var e = $root.google.longrunning.OperationInfo.verify(m[".google.longrunning.operation_info"]);
                            if (e)
                                return ".google.longrunning.operation_info." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a MethodOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.MethodOptions} MethodOptions
                 */
                MethodOptions.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.MethodOptions)
                        return d;
                    var m = new $root.google.protobuf.MethodOptions();
                    if (d.deprecated != null) {
                        m.deprecated = Boolean(d.deprecated);
                    }
                    switch (d.idempotency_level) {
                    case "IDEMPOTENCY_UNKNOWN":
                    case 0:
                        m.idempotency_level = 0;
                        break;
                    case "NO_SIDE_EFFECTS":
                    case 1:
                        m.idempotency_level = 1;
                        break;
                    case "IDEMPOTENT":
                    case 2:
                        m.idempotency_level = 2;
                        break;
                    }
                    if (d.uninterpreted_option) {
                        if (!Array.isArray(d.uninterpreted_option))
                            throw TypeError(".google.protobuf.MethodOptions.uninterpreted_option: array expected");
                        m.uninterpreted_option = [];
                        for (var i = 0; i < d.uninterpreted_option.length; ++i) {
                            if (typeof d.uninterpreted_option[i] !== "object")
                                throw TypeError(".google.protobuf.MethodOptions.uninterpreted_option: object expected");
                            m.uninterpreted_option[i] = $root.google.protobuf.UninterpretedOption.fromObject(d.uninterpreted_option[i]);
                        }
                    }
                    if (d[".google.api.http"] != null) {
                        if (typeof d[".google.api.http"] !== "object")
                            throw TypeError(".google.protobuf.MethodOptions..google.api.http: object expected");
                        m[".google.api.http"] = $root.google.api.HttpRule.fromObject(d[".google.api.http"]);
                    }
                    if (d[".google.api.method_signature"]) {
                        if (!Array.isArray(d[".google.api.method_signature"]))
                            throw TypeError(".google.protobuf.MethodOptions..google.api.method_signature: array expected");
                        m[".google.api.method_signature"] = [];
                        for (var i = 0; i < d[".google.api.method_signature"].length; ++i) {
                            m[".google.api.method_signature"][i] = String(d[".google.api.method_signature"][i]);
                        }
                    }
                    if (d[".google.longrunning.operation_info"] != null) {
                        if (typeof d[".google.longrunning.operation_info"] !== "object")
                            throw TypeError(".google.protobuf.MethodOptions..google.longrunning.operation_info: object expected");
                        m[".google.longrunning.operation_info"] = $root.google.longrunning.OperationInfo.fromObject(d[".google.longrunning.operation_info"]);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a MethodOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.MethodOptions} m MethodOptions
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                MethodOptions.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.uninterpreted_option = [];
                        d[".google.api.method_signature"] = [];
                    }
                    if (o.defaults) {
                        d.deprecated = false;
                        d.idempotency_level = o.enums === String ? "IDEMPOTENCY_UNKNOWN" : 0;
                        d[".google.longrunning.operation_info"] = null;
                        d[".google.api.http"] = null;
                    }
                    if (m.deprecated != null && m.hasOwnProperty("deprecated")) {
                        d.deprecated = m.deprecated;
                    }
                    if (m.idempotency_level != null && m.hasOwnProperty("idempotency_level")) {
                        d.idempotency_level = o.enums === String ? $root.google.protobuf.MethodOptions.IdempotencyLevel[m.idempotency_level] : m.idempotency_level;
                    }
                    if (m.uninterpreted_option && m.uninterpreted_option.length) {
                        d.uninterpreted_option = [];
                        for (var j = 0; j < m.uninterpreted_option.length; ++j) {
                            d.uninterpreted_option[j] = $root.google.protobuf.UninterpretedOption.toObject(m.uninterpreted_option[j], o);
                        }
                    }
                    if (m[".google.longrunning.operation_info"] != null && m.hasOwnProperty(".google.longrunning.operation_info")) {
                        d[".google.longrunning.operation_info"] = $root.google.longrunning.OperationInfo.toObject(m[".google.longrunning.operation_info"], o);
                    }
                    if (m[".google.api.method_signature"] && m[".google.api.method_signature"].length) {
                        d[".google.api.method_signature"] = [];
                        for (var j = 0; j < m[".google.api.method_signature"].length; ++j) {
                            d[".google.api.method_signature"][j] = m[".google.api.method_signature"][j];
                        }
                    }
                    if (m[".google.api.http"] != null && m.hasOwnProperty(".google.api.http")) {
                        d[".google.api.http"] = $root.google.api.HttpRule.toObject(m[".google.api.http"], o);
                    }
                    return d;
                };
    
                /**
                 * Converts this MethodOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                MethodOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * IdempotencyLevel enum.
                 * @name google.protobuf.MethodOptions.IdempotencyLevel
                 * @enum {number}
                 * @property {number} IDEMPOTENCY_UNKNOWN=0 IDEMPOTENCY_UNKNOWN value
                 * @property {number} NO_SIDE_EFFECTS=1 NO_SIDE_EFFECTS value
                 * @property {number} IDEMPOTENT=2 IDEMPOTENT value
                 */
                MethodOptions.IdempotencyLevel = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "IDEMPOTENCY_UNKNOWN"] = 0;
                    values[valuesById[1] = "NO_SIDE_EFFECTS"] = 1;
                    values[valuesById[2] = "IDEMPOTENT"] = 2;
                    return values;
                })();
    
                return MethodOptions;
            })();
    
            protobuf.UninterpretedOption = (function() {
    
                /**
                 * Properties of an UninterpretedOption.
                 * @memberof google.protobuf
                 * @interface IUninterpretedOption
                 * @property {Array.<google.protobuf.UninterpretedOption.INamePart>|null} [name] UninterpretedOption name
                 * @property {string|null} [identifier_value] UninterpretedOption identifier_value
                 * @property {number|null} [positive_int_value] UninterpretedOption positive_int_value
                 * @property {number|null} [negative_int_value] UninterpretedOption negative_int_value
                 * @property {number|null} [double_value] UninterpretedOption double_value
                 * @property {Uint8Array|null} [string_value] UninterpretedOption string_value
                 * @property {string|null} [aggregate_value] UninterpretedOption aggregate_value
                 */
    
                /**
                 * Constructs a new UninterpretedOption.
                 * @memberof google.protobuf
                 * @classdesc Represents an UninterpretedOption.
                 * @implements IUninterpretedOption
                 * @constructor
                 * @param {google.protobuf.IUninterpretedOption=} [p] Properties to set
                 */
                function UninterpretedOption(p) {
                    this.name = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * UninterpretedOption name.
                 * @member {Array.<google.protobuf.UninterpretedOption.INamePart>} name
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.name = $util.emptyArray;
    
                /**
                 * UninterpretedOption identifier_value.
                 * @member {string} identifier_value
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.identifier_value = "";
    
                /**
                 * UninterpretedOption positive_int_value.
                 * @member {number} positive_int_value
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.positive_int_value = $util.Long ? $util.Long.fromBits(0,0,true) : 0;
    
                /**
                 * UninterpretedOption negative_int_value.
                 * @member {number} negative_int_value
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.negative_int_value = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                /**
                 * UninterpretedOption double_value.
                 * @member {number} double_value
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.double_value = 0;
    
                /**
                 * UninterpretedOption string_value.
                 * @member {Uint8Array} string_value
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.string_value = $util.newBuffer([]);
    
                /**
                 * UninterpretedOption aggregate_value.
                 * @member {string} aggregate_value
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.aggregate_value = "";
    
                /**
                 * Creates a new UninterpretedOption instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.IUninterpretedOption=} [properties] Properties to set
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption instance
                 */
                UninterpretedOption.create = function create(properties) {
                    return new UninterpretedOption(properties);
                };
    
                /**
                 * Encodes the specified UninterpretedOption message. Does not implicitly {@link google.protobuf.UninterpretedOption.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.IUninterpretedOption} m UninterpretedOption message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UninterpretedOption.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && m.name.length) {
                        for (var i = 0; i < m.name.length; ++i)
                            $root.google.protobuf.UninterpretedOption.NamePart.encode(m.name[i], w.uint32(18).fork()).ldelim();
                    }
                    if (m.identifier_value != null && Object.hasOwnProperty.call(m, "identifier_value"))
                        w.uint32(26).string(m.identifier_value);
                    if (m.positive_int_value != null && Object.hasOwnProperty.call(m, "positive_int_value"))
                        w.uint32(32).uint64(m.positive_int_value);
                    if (m.negative_int_value != null && Object.hasOwnProperty.call(m, "negative_int_value"))
                        w.uint32(40).int64(m.negative_int_value);
                    if (m.double_value != null && Object.hasOwnProperty.call(m, "double_value"))
                        w.uint32(49).double(m.double_value);
                    if (m.string_value != null && Object.hasOwnProperty.call(m, "string_value"))
                        w.uint32(58).bytes(m.string_value);
                    if (m.aggregate_value != null && Object.hasOwnProperty.call(m, "aggregate_value"))
                        w.uint32(66).string(m.aggregate_value);
                    return w;
                };
    
                /**
                 * Encodes the specified UninterpretedOption message, length delimited. Does not implicitly {@link google.protobuf.UninterpretedOption.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.IUninterpretedOption} message UninterpretedOption message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UninterpretedOption.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an UninterpretedOption message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UninterpretedOption.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.UninterpretedOption();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 2:
                            if (!(m.name && m.name.length))
                                m.name = [];
                            m.name.push($root.google.protobuf.UninterpretedOption.NamePart.decode(r, r.uint32()));
                            break;
                        case 3:
                            m.identifier_value = r.string();
                            break;
                        case 4:
                            m.positive_int_value = r.uint64();
                            break;
                        case 5:
                            m.negative_int_value = r.int64();
                            break;
                        case 6:
                            m.double_value = r.double();
                            break;
                        case 7:
                            m.string_value = r.bytes();
                            break;
                        case 8:
                            m.aggregate_value = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an UninterpretedOption message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UninterpretedOption.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an UninterpretedOption message.
                 * @function verify
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                UninterpretedOption.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!Array.isArray(m.name))
                            return "name: array expected";
                        for (var i = 0; i < m.name.length; ++i) {
                            {
                                var e = $root.google.protobuf.UninterpretedOption.NamePart.verify(m.name[i]);
                                if (e)
                                    return "name." + e;
                            }
                        }
                    }
                    if (m.identifier_value != null && m.hasOwnProperty("identifier_value")) {
                        if (!$util.isString(m.identifier_value))
                            return "identifier_value: string expected";
                    }
                    if (m.positive_int_value != null && m.hasOwnProperty("positive_int_value")) {
                        if (!$util.isInteger(m.positive_int_value) && !(m.positive_int_value && $util.isInteger(m.positive_int_value.low) && $util.isInteger(m.positive_int_value.high)))
                            return "positive_int_value: integer|Long expected";
                    }
                    if (m.negative_int_value != null && m.hasOwnProperty("negative_int_value")) {
                        if (!$util.isInteger(m.negative_int_value) && !(m.negative_int_value && $util.isInteger(m.negative_int_value.low) && $util.isInteger(m.negative_int_value.high)))
                            return "negative_int_value: integer|Long expected";
                    }
                    if (m.double_value != null && m.hasOwnProperty("double_value")) {
                        if (typeof m.double_value !== "number")
                            return "double_value: number expected";
                    }
                    if (m.string_value != null && m.hasOwnProperty("string_value")) {
                        if (!(m.string_value && typeof m.string_value.length === "number" || $util.isString(m.string_value)))
                            return "string_value: buffer expected";
                    }
                    if (m.aggregate_value != null && m.hasOwnProperty("aggregate_value")) {
                        if (!$util.isString(m.aggregate_value))
                            return "aggregate_value: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates an UninterpretedOption message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption
                 */
                UninterpretedOption.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.UninterpretedOption)
                        return d;
                    var m = new $root.google.protobuf.UninterpretedOption();
                    if (d.name) {
                        if (!Array.isArray(d.name))
                            throw TypeError(".google.protobuf.UninterpretedOption.name: array expected");
                        m.name = [];
                        for (var i = 0; i < d.name.length; ++i) {
                            if (typeof d.name[i] !== "object")
                                throw TypeError(".google.protobuf.UninterpretedOption.name: object expected");
                            m.name[i] = $root.google.protobuf.UninterpretedOption.NamePart.fromObject(d.name[i]);
                        }
                    }
                    if (d.identifier_value != null) {
                        m.identifier_value = String(d.identifier_value);
                    }
                    if (d.positive_int_value != null) {
                        if ($util.Long)
                            (m.positive_int_value = $util.Long.fromValue(d.positive_int_value)).unsigned = true;
                        else if (typeof d.positive_int_value === "string")
                            m.positive_int_value = parseInt(d.positive_int_value, 10);
                        else if (typeof d.positive_int_value === "number")
                            m.positive_int_value = d.positive_int_value;
                        else if (typeof d.positive_int_value === "object")
                            m.positive_int_value = new $util.LongBits(d.positive_int_value.low >>> 0, d.positive_int_value.high >>> 0).toNumber(true);
                    }
                    if (d.negative_int_value != null) {
                        if ($util.Long)
                            (m.negative_int_value = $util.Long.fromValue(d.negative_int_value)).unsigned = false;
                        else if (typeof d.negative_int_value === "string")
                            m.negative_int_value = parseInt(d.negative_int_value, 10);
                        else if (typeof d.negative_int_value === "number")
                            m.negative_int_value = d.negative_int_value;
                        else if (typeof d.negative_int_value === "object")
                            m.negative_int_value = new $util.LongBits(d.negative_int_value.low >>> 0, d.negative_int_value.high >>> 0).toNumber();
                    }
                    if (d.double_value != null) {
                        m.double_value = Number(d.double_value);
                    }
                    if (d.string_value != null) {
                        if (typeof d.string_value === "string")
                            $util.base64.decode(d.string_value, m.string_value = $util.newBuffer($util.base64.length(d.string_value)), 0);
                        else if (d.string_value.length)
                            m.string_value = d.string_value;
                    }
                    if (d.aggregate_value != null) {
                        m.aggregate_value = String(d.aggregate_value);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an UninterpretedOption message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.UninterpretedOption} m UninterpretedOption
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                UninterpretedOption.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.name = [];
                    }
                    if (o.defaults) {
                        d.identifier_value = "";
                        if ($util.Long) {
                            var n = new $util.Long(0, 0, true);
                            d.positive_int_value = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                        } else
                            d.positive_int_value = o.longs === String ? "0" : 0;
                        if ($util.Long) {
                            var n = new $util.Long(0, 0, false);
                            d.negative_int_value = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                        } else
                            d.negative_int_value = o.longs === String ? "0" : 0;
                        d.double_value = 0;
                        if (o.bytes === String)
                            d.string_value = "";
                        else {
                            d.string_value = [];
                            if (o.bytes !== Array)
                                d.string_value = $util.newBuffer(d.string_value);
                        }
                        d.aggregate_value = "";
                    }
                    if (m.name && m.name.length) {
                        d.name = [];
                        for (var j = 0; j < m.name.length; ++j) {
                            d.name[j] = $root.google.protobuf.UninterpretedOption.NamePart.toObject(m.name[j], o);
                        }
                    }
                    if (m.identifier_value != null && m.hasOwnProperty("identifier_value")) {
                        d.identifier_value = m.identifier_value;
                    }
                    if (m.positive_int_value != null && m.hasOwnProperty("positive_int_value")) {
                        if (typeof m.positive_int_value === "number")
                            d.positive_int_value = o.longs === String ? String(m.positive_int_value) : m.positive_int_value;
                        else
                            d.positive_int_value = o.longs === String ? $util.Long.prototype.toString.call(m.positive_int_value) : o.longs === Number ? new $util.LongBits(m.positive_int_value.low >>> 0, m.positive_int_value.high >>> 0).toNumber(true) : m.positive_int_value;
                    }
                    if (m.negative_int_value != null && m.hasOwnProperty("negative_int_value")) {
                        if (typeof m.negative_int_value === "number")
                            d.negative_int_value = o.longs === String ? String(m.negative_int_value) : m.negative_int_value;
                        else
                            d.negative_int_value = o.longs === String ? $util.Long.prototype.toString.call(m.negative_int_value) : o.longs === Number ? new $util.LongBits(m.negative_int_value.low >>> 0, m.negative_int_value.high >>> 0).toNumber() : m.negative_int_value;
                    }
                    if (m.double_value != null && m.hasOwnProperty("double_value")) {
                        d.double_value = o.json && !isFinite(m.double_value) ? String(m.double_value) : m.double_value;
                    }
                    if (m.string_value != null && m.hasOwnProperty("string_value")) {
                        d.string_value = o.bytes === String ? $util.base64.encode(m.string_value, 0, m.string_value.length) : o.bytes === Array ? Array.prototype.slice.call(m.string_value) : m.string_value;
                    }
                    if (m.aggregate_value != null && m.hasOwnProperty("aggregate_value")) {
                        d.aggregate_value = m.aggregate_value;
                    }
                    return d;
                };
    
                /**
                 * Converts this UninterpretedOption to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                UninterpretedOption.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                UninterpretedOption.NamePart = (function() {
    
                    /**
                     * Properties of a NamePart.
                     * @memberof google.protobuf.UninterpretedOption
                     * @interface INamePart
                     * @property {string} name_part NamePart name_part
                     * @property {boolean} is_extension NamePart is_extension
                     */
    
                    /**
                     * Constructs a new NamePart.
                     * @memberof google.protobuf.UninterpretedOption
                     * @classdesc Represents a NamePart.
                     * @implements INamePart
                     * @constructor
                     * @param {google.protobuf.UninterpretedOption.INamePart=} [p] Properties to set
                     */
                    function NamePart(p) {
                        if (p)
                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                if (p[ks[i]] != null)
                                    this[ks[i]] = p[ks[i]];
                    }
    
                    /**
                     * NamePart name_part.
                     * @member {string} name_part
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @instance
                     */
                    NamePart.prototype.name_part = "";
    
                    /**
                     * NamePart is_extension.
                     * @member {boolean} is_extension
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @instance
                     */
                    NamePart.prototype.is_extension = false;
    
                    /**
                     * Creates a new NamePart instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.INamePart=} [properties] Properties to set
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart instance
                     */
                    NamePart.create = function create(properties) {
                        return new NamePart(properties);
                    };
    
                    /**
                     * Encodes the specified NamePart message. Does not implicitly {@link google.protobuf.UninterpretedOption.NamePart.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.INamePart} m NamePart message or plain object to encode
                     * @param {$protobuf.Writer} [w] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    NamePart.encode = function encode(m, w) {
                        if (!w)
                            w = $Writer.create();
                        w.uint32(10).string(m.name_part);
                        w.uint32(16).bool(m.is_extension);
                        return w;
                    };
    
                    /**
                     * Encodes the specified NamePart message, length delimited. Does not implicitly {@link google.protobuf.UninterpretedOption.NamePart.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.INamePart} message NamePart message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    NamePart.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a NamePart message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                     * @param {number} [l] Message length if known beforehand
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    NamePart.decode = function decode(r, l) {
                        if (!(r instanceof $Reader))
                            r = $Reader.create(r);
                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.UninterpretedOption.NamePart();
                        while (r.pos < c) {
                            var t = r.uint32();
                            switch (t >>> 3) {
                            case 1:
                                m.name_part = r.string();
                                break;
                            case 2:
                                m.is_extension = r.bool();
                                break;
                            default:
                                r.skipType(t & 7);
                                break;
                            }
                        }
                        if (!m.hasOwnProperty("name_part"))
                            throw $util.ProtocolError("missing required 'name_part'", { instance: m });
                        if (!m.hasOwnProperty("is_extension"))
                            throw $util.ProtocolError("missing required 'is_extension'", { instance: m });
                        return m;
                    };
    
                    /**
                     * Decodes a NamePart message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    NamePart.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a NamePart message.
                     * @function verify
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {Object.<string,*>} m Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    NamePart.verify = function verify(m) {
                        if (typeof m !== "object" || m === null)
                            return "object expected";
                        if (!$util.isString(m.name_part))
                            return "name_part: string expected";
                        if (typeof m.is_extension !== "boolean")
                            return "is_extension: boolean expected";
                        return null;
                    };
    
                    /**
                     * Creates a NamePart message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {Object.<string,*>} d Plain object
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart
                     */
                    NamePart.fromObject = function fromObject(d) {
                        if (d instanceof $root.google.protobuf.UninterpretedOption.NamePart)
                            return d;
                        var m = new $root.google.protobuf.UninterpretedOption.NamePart();
                        if (d.name_part != null) {
                            m.name_part = String(d.name_part);
                        }
                        if (d.is_extension != null) {
                            m.is_extension = Boolean(d.is_extension);
                        }
                        return m;
                    };
    
                    /**
                     * Creates a plain object from a NamePart message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.NamePart} m NamePart
                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    NamePart.toObject = function toObject(m, o) {
                        if (!o)
                            o = {};
                        var d = {};
                        if (o.defaults) {
                            d.name_part = "";
                            d.is_extension = false;
                        }
                        if (m.name_part != null && m.hasOwnProperty("name_part")) {
                            d.name_part = m.name_part;
                        }
                        if (m.is_extension != null && m.hasOwnProperty("is_extension")) {
                            d.is_extension = m.is_extension;
                        }
                        return d;
                    };
    
                    /**
                     * Converts this NamePart to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    NamePart.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    return NamePart;
                })();
    
                return UninterpretedOption;
            })();
    
            protobuf.SourceCodeInfo = (function() {
    
                /**
                 * Properties of a SourceCodeInfo.
                 * @memberof google.protobuf
                 * @interface ISourceCodeInfo
                 * @property {Array.<google.protobuf.SourceCodeInfo.ILocation>|null} [location] SourceCodeInfo location
                 */
    
                /**
                 * Constructs a new SourceCodeInfo.
                 * @memberof google.protobuf
                 * @classdesc Represents a SourceCodeInfo.
                 * @implements ISourceCodeInfo
                 * @constructor
                 * @param {google.protobuf.ISourceCodeInfo=} [p] Properties to set
                 */
                function SourceCodeInfo(p) {
                    this.location = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * SourceCodeInfo location.
                 * @member {Array.<google.protobuf.SourceCodeInfo.ILocation>} location
                 * @memberof google.protobuf.SourceCodeInfo
                 * @instance
                 */
                SourceCodeInfo.prototype.location = $util.emptyArray;
    
                /**
                 * Creates a new SourceCodeInfo instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.ISourceCodeInfo=} [properties] Properties to set
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo instance
                 */
                SourceCodeInfo.create = function create(properties) {
                    return new SourceCodeInfo(properties);
                };
    
                /**
                 * Encodes the specified SourceCodeInfo message. Does not implicitly {@link google.protobuf.SourceCodeInfo.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.ISourceCodeInfo} m SourceCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                SourceCodeInfo.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.location != null && m.location.length) {
                        for (var i = 0; i < m.location.length; ++i)
                            $root.google.protobuf.SourceCodeInfo.Location.encode(m.location[i], w.uint32(10).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified SourceCodeInfo message, length delimited. Does not implicitly {@link google.protobuf.SourceCodeInfo.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.ISourceCodeInfo} message SourceCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                SourceCodeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a SourceCodeInfo message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                SourceCodeInfo.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.SourceCodeInfo();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            if (!(m.location && m.location.length))
                                m.location = [];
                            m.location.push($root.google.protobuf.SourceCodeInfo.Location.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a SourceCodeInfo message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                SourceCodeInfo.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a SourceCodeInfo message.
                 * @function verify
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                SourceCodeInfo.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.location != null && m.hasOwnProperty("location")) {
                        if (!Array.isArray(m.location))
                            return "location: array expected";
                        for (var i = 0; i < m.location.length; ++i) {
                            {
                                var e = $root.google.protobuf.SourceCodeInfo.Location.verify(m.location[i]);
                                if (e)
                                    return "location." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a SourceCodeInfo message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo
                 */
                SourceCodeInfo.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.SourceCodeInfo)
                        return d;
                    var m = new $root.google.protobuf.SourceCodeInfo();
                    if (d.location) {
                        if (!Array.isArray(d.location))
                            throw TypeError(".google.protobuf.SourceCodeInfo.location: array expected");
                        m.location = [];
                        for (var i = 0; i < d.location.length; ++i) {
                            if (typeof d.location[i] !== "object")
                                throw TypeError(".google.protobuf.SourceCodeInfo.location: object expected");
                            m.location[i] = $root.google.protobuf.SourceCodeInfo.Location.fromObject(d.location[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a SourceCodeInfo message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.SourceCodeInfo} m SourceCodeInfo
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                SourceCodeInfo.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.location = [];
                    }
                    if (m.location && m.location.length) {
                        d.location = [];
                        for (var j = 0; j < m.location.length; ++j) {
                            d.location[j] = $root.google.protobuf.SourceCodeInfo.Location.toObject(m.location[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this SourceCodeInfo to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.SourceCodeInfo
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                SourceCodeInfo.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                SourceCodeInfo.Location = (function() {
    
                    /**
                     * Properties of a Location.
                     * @memberof google.protobuf.SourceCodeInfo
                     * @interface ILocation
                     * @property {Array.<number>|null} [path] Location path
                     * @property {Array.<number>|null} [span] Location span
                     * @property {string|null} [leading_comments] Location leading_comments
                     * @property {string|null} [trailing_comments] Location trailing_comments
                     * @property {Array.<string>|null} [leading_detached_comments] Location leading_detached_comments
                     */
    
                    /**
                     * Constructs a new Location.
                     * @memberof google.protobuf.SourceCodeInfo
                     * @classdesc Represents a Location.
                     * @implements ILocation
                     * @constructor
                     * @param {google.protobuf.SourceCodeInfo.ILocation=} [p] Properties to set
                     */
                    function Location(p) {
                        this.path = [];
                        this.span = [];
                        this.leading_detached_comments = [];
                        if (p)
                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                if (p[ks[i]] != null)
                                    this[ks[i]] = p[ks[i]];
                    }
    
                    /**
                     * Location path.
                     * @member {Array.<number>} path
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.path = $util.emptyArray;
    
                    /**
                     * Location span.
                     * @member {Array.<number>} span
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.span = $util.emptyArray;
    
                    /**
                     * Location leading_comments.
                     * @member {string} leading_comments
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.leading_comments = "";
    
                    /**
                     * Location trailing_comments.
                     * @member {string} trailing_comments
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.trailing_comments = "";
    
                    /**
                     * Location leading_detached_comments.
                     * @member {Array.<string>} leading_detached_comments
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.leading_detached_comments = $util.emptyArray;
    
                    /**
                     * Creates a new Location instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.ILocation=} [properties] Properties to set
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location instance
                     */
                    Location.create = function create(properties) {
                        return new Location(properties);
                    };
    
                    /**
                     * Encodes the specified Location message. Does not implicitly {@link google.protobuf.SourceCodeInfo.Location.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.ILocation} m Location message or plain object to encode
                     * @param {$protobuf.Writer} [w] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Location.encode = function encode(m, w) {
                        if (!w)
                            w = $Writer.create();
                        if (m.path != null && m.path.length) {
                            w.uint32(10).fork();
                            for (var i = 0; i < m.path.length; ++i)
                                w.int32(m.path[i]);
                            w.ldelim();
                        }
                        if (m.span != null && m.span.length) {
                            w.uint32(18).fork();
                            for (var i = 0; i < m.span.length; ++i)
                                w.int32(m.span[i]);
                            w.ldelim();
                        }
                        if (m.leading_comments != null && Object.hasOwnProperty.call(m, "leading_comments"))
                            w.uint32(26).string(m.leading_comments);
                        if (m.trailing_comments != null && Object.hasOwnProperty.call(m, "trailing_comments"))
                            w.uint32(34).string(m.trailing_comments);
                        if (m.leading_detached_comments != null && m.leading_detached_comments.length) {
                            for (var i = 0; i < m.leading_detached_comments.length; ++i)
                                w.uint32(50).string(m.leading_detached_comments[i]);
                        }
                        return w;
                    };
    
                    /**
                     * Encodes the specified Location message, length delimited. Does not implicitly {@link google.protobuf.SourceCodeInfo.Location.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.ILocation} message Location message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Location.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Location message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                     * @param {number} [l] Message length if known beforehand
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Location.decode = function decode(r, l) {
                        if (!(r instanceof $Reader))
                            r = $Reader.create(r);
                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.SourceCodeInfo.Location();
                        while (r.pos < c) {
                            var t = r.uint32();
                            switch (t >>> 3) {
                            case 1:
                                if (!(m.path && m.path.length))
                                    m.path = [];
                                if ((t & 7) === 2) {
                                    var c2 = r.uint32() + r.pos;
                                    while (r.pos < c2)
                                        m.path.push(r.int32());
                                } else
                                    m.path.push(r.int32());
                                break;
                            case 2:
                                if (!(m.span && m.span.length))
                                    m.span = [];
                                if ((t & 7) === 2) {
                                    var c2 = r.uint32() + r.pos;
                                    while (r.pos < c2)
                                        m.span.push(r.int32());
                                } else
                                    m.span.push(r.int32());
                                break;
                            case 3:
                                m.leading_comments = r.string();
                                break;
                            case 4:
                                m.trailing_comments = r.string();
                                break;
                            case 6:
                                if (!(m.leading_detached_comments && m.leading_detached_comments.length))
                                    m.leading_detached_comments = [];
                                m.leading_detached_comments.push(r.string());
                                break;
                            default:
                                r.skipType(t & 7);
                                break;
                            }
                        }
                        return m;
                    };
    
                    /**
                     * Decodes a Location message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Location.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Location message.
                     * @function verify
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {Object.<string,*>} m Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Location.verify = function verify(m) {
                        if (typeof m !== "object" || m === null)
                            return "object expected";
                        if (m.path != null && m.hasOwnProperty("path")) {
                            if (!Array.isArray(m.path))
                                return "path: array expected";
                            for (var i = 0; i < m.path.length; ++i) {
                                if (!$util.isInteger(m.path[i]))
                                    return "path: integer[] expected";
                            }
                        }
                        if (m.span != null && m.hasOwnProperty("span")) {
                            if (!Array.isArray(m.span))
                                return "span: array expected";
                            for (var i = 0; i < m.span.length; ++i) {
                                if (!$util.isInteger(m.span[i]))
                                    return "span: integer[] expected";
                            }
                        }
                        if (m.leading_comments != null && m.hasOwnProperty("leading_comments")) {
                            if (!$util.isString(m.leading_comments))
                                return "leading_comments: string expected";
                        }
                        if (m.trailing_comments != null && m.hasOwnProperty("trailing_comments")) {
                            if (!$util.isString(m.trailing_comments))
                                return "trailing_comments: string expected";
                        }
                        if (m.leading_detached_comments != null && m.hasOwnProperty("leading_detached_comments")) {
                            if (!Array.isArray(m.leading_detached_comments))
                                return "leading_detached_comments: array expected";
                            for (var i = 0; i < m.leading_detached_comments.length; ++i) {
                                if (!$util.isString(m.leading_detached_comments[i]))
                                    return "leading_detached_comments: string[] expected";
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Location message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {Object.<string,*>} d Plain object
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location
                     */
                    Location.fromObject = function fromObject(d) {
                        if (d instanceof $root.google.protobuf.SourceCodeInfo.Location)
                            return d;
                        var m = new $root.google.protobuf.SourceCodeInfo.Location();
                        if (d.path) {
                            if (!Array.isArray(d.path))
                                throw TypeError(".google.protobuf.SourceCodeInfo.Location.path: array expected");
                            m.path = [];
                            for (var i = 0; i < d.path.length; ++i) {
                                m.path[i] = d.path[i] | 0;
                            }
                        }
                        if (d.span) {
                            if (!Array.isArray(d.span))
                                throw TypeError(".google.protobuf.SourceCodeInfo.Location.span: array expected");
                            m.span = [];
                            for (var i = 0; i < d.span.length; ++i) {
                                m.span[i] = d.span[i] | 0;
                            }
                        }
                        if (d.leading_comments != null) {
                            m.leading_comments = String(d.leading_comments);
                        }
                        if (d.trailing_comments != null) {
                            m.trailing_comments = String(d.trailing_comments);
                        }
                        if (d.leading_detached_comments) {
                            if (!Array.isArray(d.leading_detached_comments))
                                throw TypeError(".google.protobuf.SourceCodeInfo.Location.leading_detached_comments: array expected");
                            m.leading_detached_comments = [];
                            for (var i = 0; i < d.leading_detached_comments.length; ++i) {
                                m.leading_detached_comments[i] = String(d.leading_detached_comments[i]);
                            }
                        }
                        return m;
                    };
    
                    /**
                     * Creates a plain object from a Location message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.Location} m Location
                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Location.toObject = function toObject(m, o) {
                        if (!o)
                            o = {};
                        var d = {};
                        if (o.arrays || o.defaults) {
                            d.path = [];
                            d.span = [];
                            d.leading_detached_comments = [];
                        }
                        if (o.defaults) {
                            d.leading_comments = "";
                            d.trailing_comments = "";
                        }
                        if (m.path && m.path.length) {
                            d.path = [];
                            for (var j = 0; j < m.path.length; ++j) {
                                d.path[j] = m.path[j];
                            }
                        }
                        if (m.span && m.span.length) {
                            d.span = [];
                            for (var j = 0; j < m.span.length; ++j) {
                                d.span[j] = m.span[j];
                            }
                        }
                        if (m.leading_comments != null && m.hasOwnProperty("leading_comments")) {
                            d.leading_comments = m.leading_comments;
                        }
                        if (m.trailing_comments != null && m.hasOwnProperty("trailing_comments")) {
                            d.trailing_comments = m.trailing_comments;
                        }
                        if (m.leading_detached_comments && m.leading_detached_comments.length) {
                            d.leading_detached_comments = [];
                            for (var j = 0; j < m.leading_detached_comments.length; ++j) {
                                d.leading_detached_comments[j] = m.leading_detached_comments[j];
                            }
                        }
                        return d;
                    };
    
                    /**
                     * Converts this Location to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Location.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    return Location;
                })();
    
                return SourceCodeInfo;
            })();
    
            protobuf.GeneratedCodeInfo = (function() {
    
                /**
                 * Properties of a GeneratedCodeInfo.
                 * @memberof google.protobuf
                 * @interface IGeneratedCodeInfo
                 * @property {Array.<google.protobuf.GeneratedCodeInfo.IAnnotation>|null} [annotation] GeneratedCodeInfo annotation
                 */
    
                /**
                 * Constructs a new GeneratedCodeInfo.
                 * @memberof google.protobuf
                 * @classdesc Represents a GeneratedCodeInfo.
                 * @implements IGeneratedCodeInfo
                 * @constructor
                 * @param {google.protobuf.IGeneratedCodeInfo=} [p] Properties to set
                 */
                function GeneratedCodeInfo(p) {
                    this.annotation = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * GeneratedCodeInfo annotation.
                 * @member {Array.<google.protobuf.GeneratedCodeInfo.IAnnotation>} annotation
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @instance
                 */
                GeneratedCodeInfo.prototype.annotation = $util.emptyArray;
    
                /**
                 * Creates a new GeneratedCodeInfo instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.IGeneratedCodeInfo=} [properties] Properties to set
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo instance
                 */
                GeneratedCodeInfo.create = function create(properties) {
                    return new GeneratedCodeInfo(properties);
                };
    
                /**
                 * Encodes the specified GeneratedCodeInfo message. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.IGeneratedCodeInfo} m GeneratedCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GeneratedCodeInfo.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.annotation != null && m.annotation.length) {
                        for (var i = 0; i < m.annotation.length; ++i)
                            $root.google.protobuf.GeneratedCodeInfo.Annotation.encode(m.annotation[i], w.uint32(10).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified GeneratedCodeInfo message, length delimited. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.IGeneratedCodeInfo} message GeneratedCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GeneratedCodeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a GeneratedCodeInfo message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GeneratedCodeInfo.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.GeneratedCodeInfo();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            if (!(m.annotation && m.annotation.length))
                                m.annotation = [];
                            m.annotation.push($root.google.protobuf.GeneratedCodeInfo.Annotation.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a GeneratedCodeInfo message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GeneratedCodeInfo.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a GeneratedCodeInfo message.
                 * @function verify
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                GeneratedCodeInfo.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.annotation != null && m.hasOwnProperty("annotation")) {
                        if (!Array.isArray(m.annotation))
                            return "annotation: array expected";
                        for (var i = 0; i < m.annotation.length; ++i) {
                            {
                                var e = $root.google.protobuf.GeneratedCodeInfo.Annotation.verify(m.annotation[i]);
                                if (e)
                                    return "annotation." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a GeneratedCodeInfo message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo
                 */
                GeneratedCodeInfo.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.GeneratedCodeInfo)
                        return d;
                    var m = new $root.google.protobuf.GeneratedCodeInfo();
                    if (d.annotation) {
                        if (!Array.isArray(d.annotation))
                            throw TypeError(".google.protobuf.GeneratedCodeInfo.annotation: array expected");
                        m.annotation = [];
                        for (var i = 0; i < d.annotation.length; ++i) {
                            if (typeof d.annotation[i] !== "object")
                                throw TypeError(".google.protobuf.GeneratedCodeInfo.annotation: object expected");
                            m.annotation[i] = $root.google.protobuf.GeneratedCodeInfo.Annotation.fromObject(d.annotation[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a GeneratedCodeInfo message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.GeneratedCodeInfo} m GeneratedCodeInfo
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                GeneratedCodeInfo.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.annotation = [];
                    }
                    if (m.annotation && m.annotation.length) {
                        d.annotation = [];
                        for (var j = 0; j < m.annotation.length; ++j) {
                            d.annotation[j] = $root.google.protobuf.GeneratedCodeInfo.Annotation.toObject(m.annotation[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this GeneratedCodeInfo to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                GeneratedCodeInfo.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                GeneratedCodeInfo.Annotation = (function() {
    
                    /**
                     * Properties of an Annotation.
                     * @memberof google.protobuf.GeneratedCodeInfo
                     * @interface IAnnotation
                     * @property {Array.<number>|null} [path] Annotation path
                     * @property {string|null} [source_file] Annotation source_file
                     * @property {number|null} [begin] Annotation begin
                     * @property {number|null} [end] Annotation end
                     */
    
                    /**
                     * Constructs a new Annotation.
                     * @memberof google.protobuf.GeneratedCodeInfo
                     * @classdesc Represents an Annotation.
                     * @implements IAnnotation
                     * @constructor
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation=} [p] Properties to set
                     */
                    function Annotation(p) {
                        this.path = [];
                        if (p)
                            for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                                if (p[ks[i]] != null)
                                    this[ks[i]] = p[ks[i]];
                    }
    
                    /**
                     * Annotation path.
                     * @member {Array.<number>} path
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.path = $util.emptyArray;
    
                    /**
                     * Annotation source_file.
                     * @member {string} source_file
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.source_file = "";
    
                    /**
                     * Annotation begin.
                     * @member {number} begin
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.begin = 0;
    
                    /**
                     * Annotation end.
                     * @member {number} end
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.end = 0;
    
                    /**
                     * Creates a new Annotation instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation=} [properties] Properties to set
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation instance
                     */
                    Annotation.create = function create(properties) {
                        return new Annotation(properties);
                    };
    
                    /**
                     * Encodes the specified Annotation message. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.Annotation.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation} m Annotation message or plain object to encode
                     * @param {$protobuf.Writer} [w] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Annotation.encode = function encode(m, w) {
                        if (!w)
                            w = $Writer.create();
                        if (m.path != null && m.path.length) {
                            w.uint32(10).fork();
                            for (var i = 0; i < m.path.length; ++i)
                                w.int32(m.path[i]);
                            w.ldelim();
                        }
                        if (m.source_file != null && Object.hasOwnProperty.call(m, "source_file"))
                            w.uint32(18).string(m.source_file);
                        if (m.begin != null && Object.hasOwnProperty.call(m, "begin"))
                            w.uint32(24).int32(m.begin);
                        if (m.end != null && Object.hasOwnProperty.call(m, "end"))
                            w.uint32(32).int32(m.end);
                        return w;
                    };
    
                    /**
                     * Encodes the specified Annotation message, length delimited. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.Annotation.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation} message Annotation message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Annotation.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an Annotation message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                     * @param {number} [l] Message length if known beforehand
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Annotation.decode = function decode(r, l) {
                        if (!(r instanceof $Reader))
                            r = $Reader.create(r);
                        var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.GeneratedCodeInfo.Annotation();
                        while (r.pos < c) {
                            var t = r.uint32();
                            switch (t >>> 3) {
                            case 1:
                                if (!(m.path && m.path.length))
                                    m.path = [];
                                if ((t & 7) === 2) {
                                    var c2 = r.uint32() + r.pos;
                                    while (r.pos < c2)
                                        m.path.push(r.int32());
                                } else
                                    m.path.push(r.int32());
                                break;
                            case 2:
                                m.source_file = r.string();
                                break;
                            case 3:
                                m.begin = r.int32();
                                break;
                            case 4:
                                m.end = r.int32();
                                break;
                            default:
                                r.skipType(t & 7);
                                break;
                            }
                        }
                        return m;
                    };
    
                    /**
                     * Decodes an Annotation message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Annotation.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an Annotation message.
                     * @function verify
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {Object.<string,*>} m Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Annotation.verify = function verify(m) {
                        if (typeof m !== "object" || m === null)
                            return "object expected";
                        if (m.path != null && m.hasOwnProperty("path")) {
                            if (!Array.isArray(m.path))
                                return "path: array expected";
                            for (var i = 0; i < m.path.length; ++i) {
                                if (!$util.isInteger(m.path[i]))
                                    return "path: integer[] expected";
                            }
                        }
                        if (m.source_file != null && m.hasOwnProperty("source_file")) {
                            if (!$util.isString(m.source_file))
                                return "source_file: string expected";
                        }
                        if (m.begin != null && m.hasOwnProperty("begin")) {
                            if (!$util.isInteger(m.begin))
                                return "begin: integer expected";
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            if (!$util.isInteger(m.end))
                                return "end: integer expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates an Annotation message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {Object.<string,*>} d Plain object
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation
                     */
                    Annotation.fromObject = function fromObject(d) {
                        if (d instanceof $root.google.protobuf.GeneratedCodeInfo.Annotation)
                            return d;
                        var m = new $root.google.protobuf.GeneratedCodeInfo.Annotation();
                        if (d.path) {
                            if (!Array.isArray(d.path))
                                throw TypeError(".google.protobuf.GeneratedCodeInfo.Annotation.path: array expected");
                            m.path = [];
                            for (var i = 0; i < d.path.length; ++i) {
                                m.path[i] = d.path[i] | 0;
                            }
                        }
                        if (d.source_file != null) {
                            m.source_file = String(d.source_file);
                        }
                        if (d.begin != null) {
                            m.begin = d.begin | 0;
                        }
                        if (d.end != null) {
                            m.end = d.end | 0;
                        }
                        return m;
                    };
    
                    /**
                     * Creates a plain object from an Annotation message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.Annotation} m Annotation
                     * @param {$protobuf.IConversionOptions} [o] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Annotation.toObject = function toObject(m, o) {
                        if (!o)
                            o = {};
                        var d = {};
                        if (o.arrays || o.defaults) {
                            d.path = [];
                        }
                        if (o.defaults) {
                            d.source_file = "";
                            d.begin = 0;
                            d.end = 0;
                        }
                        if (m.path && m.path.length) {
                            d.path = [];
                            for (var j = 0; j < m.path.length; ++j) {
                                d.path[j] = m.path[j];
                            }
                        }
                        if (m.source_file != null && m.hasOwnProperty("source_file")) {
                            d.source_file = m.source_file;
                        }
                        if (m.begin != null && m.hasOwnProperty("begin")) {
                            d.begin = m.begin;
                        }
                        if (m.end != null && m.hasOwnProperty("end")) {
                            d.end = m.end;
                        }
                        return d;
                    };
    
                    /**
                     * Converts this Annotation to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Annotation.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    return Annotation;
                })();
    
                return GeneratedCodeInfo;
            })();
    
            protobuf.Duration = (function() {
    
                /**
                 * Properties of a Duration.
                 * @memberof google.protobuf
                 * @interface IDuration
                 * @property {number|null} [seconds] Duration seconds
                 * @property {number|null} [nanos] Duration nanos
                 */
    
                /**
                 * Constructs a new Duration.
                 * @memberof google.protobuf
                 * @classdesc Represents a Duration.
                 * @implements IDuration
                 * @constructor
                 * @param {google.protobuf.IDuration=} [p] Properties to set
                 */
                function Duration(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * Duration seconds.
                 * @member {number} seconds
                 * @memberof google.protobuf.Duration
                 * @instance
                 */
                Duration.prototype.seconds = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                /**
                 * Duration nanos.
                 * @member {number} nanos
                 * @memberof google.protobuf.Duration
                 * @instance
                 */
                Duration.prototype.nanos = 0;
    
                /**
                 * Creates a new Duration instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.IDuration=} [properties] Properties to set
                 * @returns {google.protobuf.Duration} Duration instance
                 */
                Duration.create = function create(properties) {
                    return new Duration(properties);
                };
    
                /**
                 * Encodes the specified Duration message. Does not implicitly {@link google.protobuf.Duration.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.IDuration} m Duration message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Duration.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.seconds != null && Object.hasOwnProperty.call(m, "seconds"))
                        w.uint32(8).int64(m.seconds);
                    if (m.nanos != null && Object.hasOwnProperty.call(m, "nanos"))
                        w.uint32(16).int32(m.nanos);
                    return w;
                };
    
                /**
                 * Encodes the specified Duration message, length delimited. Does not implicitly {@link google.protobuf.Duration.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.IDuration} message Duration message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Duration.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Duration message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.Duration} Duration
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Duration.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.Duration();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.seconds = r.int64();
                            break;
                        case 2:
                            m.nanos = r.int32();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a Duration message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Duration} Duration
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Duration.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Duration message.
                 * @function verify
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Duration.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.seconds != null && m.hasOwnProperty("seconds")) {
                        if (!$util.isInteger(m.seconds) && !(m.seconds && $util.isInteger(m.seconds.low) && $util.isInteger(m.seconds.high)))
                            return "seconds: integer|Long expected";
                    }
                    if (m.nanos != null && m.hasOwnProperty("nanos")) {
                        if (!$util.isInteger(m.nanos))
                            return "nanos: integer expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a Duration message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.Duration} Duration
                 */
                Duration.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.Duration)
                        return d;
                    var m = new $root.google.protobuf.Duration();
                    if (d.seconds != null) {
                        if ($util.Long)
                            (m.seconds = $util.Long.fromValue(d.seconds)).unsigned = false;
                        else if (typeof d.seconds === "string")
                            m.seconds = parseInt(d.seconds, 10);
                        else if (typeof d.seconds === "number")
                            m.seconds = d.seconds;
                        else if (typeof d.seconds === "object")
                            m.seconds = new $util.LongBits(d.seconds.low >>> 0, d.seconds.high >>> 0).toNumber();
                    }
                    if (d.nanos != null) {
                        m.nanos = d.nanos | 0;
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a Duration message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.Duration} m Duration
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Duration.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        if ($util.Long) {
                            var n = new $util.Long(0, 0, false);
                            d.seconds = o.longs === String ? n.toString() : o.longs === Number ? n.toNumber() : n;
                        } else
                            d.seconds = o.longs === String ? "0" : 0;
                        d.nanos = 0;
                    }
                    if (m.seconds != null && m.hasOwnProperty("seconds")) {
                        if (typeof m.seconds === "number")
                            d.seconds = o.longs === String ? String(m.seconds) : m.seconds;
                        else
                            d.seconds = o.longs === String ? $util.Long.prototype.toString.call(m.seconds) : o.longs === Number ? new $util.LongBits(m.seconds.low >>> 0, m.seconds.high >>> 0).toNumber() : m.seconds;
                    }
                    if (m.nanos != null && m.hasOwnProperty("nanos")) {
                        d.nanos = m.nanos;
                    }
                    return d;
                };
    
                /**
                 * Converts this Duration to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Duration
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Duration.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return Duration;
            })();
    
            protobuf.FieldMask = (function() {
    
                /**
                 * Properties of a FieldMask.
                 * @memberof google.protobuf
                 * @interface IFieldMask
                 * @property {Array.<string>|null} [paths] FieldMask paths
                 */
    
                /**
                 * Constructs a new FieldMask.
                 * @memberof google.protobuf
                 * @classdesc Represents a FieldMask.
                 * @implements IFieldMask
                 * @constructor
                 * @param {google.protobuf.IFieldMask=} [p] Properties to set
                 */
                function FieldMask(p) {
                    this.paths = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * FieldMask paths.
                 * @member {Array.<string>} paths
                 * @memberof google.protobuf.FieldMask
                 * @instance
                 */
                FieldMask.prototype.paths = $util.emptyArray;
    
                /**
                 * Creates a new FieldMask instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.IFieldMask=} [properties] Properties to set
                 * @returns {google.protobuf.FieldMask} FieldMask instance
                 */
                FieldMask.create = function create(properties) {
                    return new FieldMask(properties);
                };
    
                /**
                 * Encodes the specified FieldMask message. Does not implicitly {@link google.protobuf.FieldMask.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.IFieldMask} m FieldMask message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldMask.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.paths != null && m.paths.length) {
                        for (var i = 0; i < m.paths.length; ++i)
                            w.uint32(10).string(m.paths[i]);
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified FieldMask message, length delimited. Does not implicitly {@link google.protobuf.FieldMask.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.IFieldMask} message FieldMask message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldMask.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FieldMask message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.FieldMask} FieldMask
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldMask.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.FieldMask();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            if (!(m.paths && m.paths.length))
                                m.paths = [];
                            m.paths.push(r.string());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a FieldMask message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FieldMask} FieldMask
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldMask.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FieldMask message.
                 * @function verify
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FieldMask.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.paths != null && m.hasOwnProperty("paths")) {
                        if (!Array.isArray(m.paths))
                            return "paths: array expected";
                        for (var i = 0; i < m.paths.length; ++i) {
                            if (!$util.isString(m.paths[i]))
                                return "paths: string[] expected";
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a FieldMask message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.FieldMask} FieldMask
                 */
                FieldMask.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.FieldMask)
                        return d;
                    var m = new $root.google.protobuf.FieldMask();
                    if (d.paths) {
                        if (!Array.isArray(d.paths))
                            throw TypeError(".google.protobuf.FieldMask.paths: array expected");
                        m.paths = [];
                        for (var i = 0; i < d.paths.length; ++i) {
                            m.paths[i] = String(d.paths[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a FieldMask message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.FieldMask} m FieldMask
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FieldMask.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.paths = [];
                    }
                    if (m.paths && m.paths.length) {
                        d.paths = [];
                        for (var j = 0; j < m.paths.length; ++j) {
                            d.paths[j] = m.paths[j];
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this FieldMask to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FieldMask
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FieldMask.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return FieldMask;
            })();
    
            protobuf.Any = (function() {
    
                /**
                 * Properties of an Any.
                 * @memberof google.protobuf
                 * @interface IAny
                 * @property {string|null} [type_url] Any type_url
                 * @property {Uint8Array|null} [value] Any value
                 */
    
                /**
                 * Constructs a new Any.
                 * @memberof google.protobuf
                 * @classdesc Represents an Any.
                 * @implements IAny
                 * @constructor
                 * @param {google.protobuf.IAny=} [p] Properties to set
                 */
                function Any(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * Any type_url.
                 * @member {string} type_url
                 * @memberof google.protobuf.Any
                 * @instance
                 */
                Any.prototype.type_url = "";
    
                /**
                 * Any value.
                 * @member {Uint8Array} value
                 * @memberof google.protobuf.Any
                 * @instance
                 */
                Any.prototype.value = $util.newBuffer([]);
    
                /**
                 * Creates a new Any instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.IAny=} [properties] Properties to set
                 * @returns {google.protobuf.Any} Any instance
                 */
                Any.create = function create(properties) {
                    return new Any(properties);
                };
    
                /**
                 * Encodes the specified Any message. Does not implicitly {@link google.protobuf.Any.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.IAny} m Any message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Any.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.type_url != null && Object.hasOwnProperty.call(m, "type_url"))
                        w.uint32(10).string(m.type_url);
                    if (m.value != null && Object.hasOwnProperty.call(m, "value"))
                        w.uint32(18).bytes(m.value);
                    return w;
                };
    
                /**
                 * Encodes the specified Any message, length delimited. Does not implicitly {@link google.protobuf.Any.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.IAny} message Any message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Any.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Any message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.Any} Any
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Any.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.Any();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.type_url = r.string();
                            break;
                        case 2:
                            m.value = r.bytes();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an Any message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Any} Any
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Any.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Any message.
                 * @function verify
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Any.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.type_url != null && m.hasOwnProperty("type_url")) {
                        if (!$util.isString(m.type_url))
                            return "type_url: string expected";
                    }
                    if (m.value != null && m.hasOwnProperty("value")) {
                        if (!(m.value && typeof m.value.length === "number" || $util.isString(m.value)))
                            return "value: buffer expected";
                    }
                    return null;
                };
    
                /**
                 * Creates an Any message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.Any} Any
                 */
                Any.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.Any)
                        return d;
                    var m = new $root.google.protobuf.Any();
                    if (d.type_url != null) {
                        m.type_url = String(d.type_url);
                    }
                    if (d.value != null) {
                        if (typeof d.value === "string")
                            $util.base64.decode(d.value, m.value = $util.newBuffer($util.base64.length(d.value)), 0);
                        else if (d.value.length)
                            m.value = d.value;
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an Any message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.Any} m Any
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Any.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.type_url = "";
                        if (o.bytes === String)
                            d.value = "";
                        else {
                            d.value = [];
                            if (o.bytes !== Array)
                                d.value = $util.newBuffer(d.value);
                        }
                    }
                    if (m.type_url != null && m.hasOwnProperty("type_url")) {
                        d.type_url = m.type_url;
                    }
                    if (m.value != null && m.hasOwnProperty("value")) {
                        d.value = o.bytes === String ? $util.base64.encode(m.value, 0, m.value.length) : o.bytes === Array ? Array.prototype.slice.call(m.value) : m.value;
                    }
                    return d;
                };
    
                /**
                 * Converts this Any to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Any
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Any.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return Any;
            })();
    
            protobuf.Empty = (function() {
    
                /**
                 * Properties of an Empty.
                 * @memberof google.protobuf
                 * @interface IEmpty
                 */
    
                /**
                 * Constructs a new Empty.
                 * @memberof google.protobuf
                 * @classdesc Represents an Empty.
                 * @implements IEmpty
                 * @constructor
                 * @param {google.protobuf.IEmpty=} [p] Properties to set
                 */
                function Empty(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * Creates a new Empty instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.IEmpty=} [properties] Properties to set
                 * @returns {google.protobuf.Empty} Empty instance
                 */
                Empty.create = function create(properties) {
                    return new Empty(properties);
                };
    
                /**
                 * Encodes the specified Empty message. Does not implicitly {@link google.protobuf.Empty.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.IEmpty} m Empty message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Empty.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    return w;
                };
    
                /**
                 * Encodes the specified Empty message, length delimited. Does not implicitly {@link google.protobuf.Empty.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.IEmpty} message Empty message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Empty.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Empty message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.protobuf.Empty} Empty
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Empty.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.protobuf.Empty();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an Empty message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Empty} Empty
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Empty.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Empty message.
                 * @function verify
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Empty.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    return null;
                };
    
                /**
                 * Creates an Empty message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.protobuf.Empty} Empty
                 */
                Empty.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.protobuf.Empty)
                        return d;
                    return new $root.google.protobuf.Empty();
                };
    
                /**
                 * Creates a plain object from an Empty message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.Empty} m Empty
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Empty.toObject = function toObject() {
                    return {};
                };
    
                /**
                 * Converts this Empty to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Empty
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Empty.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return Empty;
            })();
    
            return protobuf;
        })();
    
        google.rpc = (function() {
    
            /**
             * Namespace rpc.
             * @memberof google
             * @namespace
             */
            var rpc = {};
    
            rpc.Status = (function() {
    
                /**
                 * Properties of a Status.
                 * @memberof google.rpc
                 * @interface IStatus
                 * @property {number|null} [code] Status code
                 * @property {string|null} [message] Status message
                 * @property {Array.<google.protobuf.IAny>|null} [details] Status details
                 */
    
                /**
                 * Constructs a new Status.
                 * @memberof google.rpc
                 * @classdesc Represents a Status.
                 * @implements IStatus
                 * @constructor
                 * @param {google.rpc.IStatus=} [p] Properties to set
                 */
                function Status(p) {
                    this.details = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * Status code.
                 * @member {number} code
                 * @memberof google.rpc.Status
                 * @instance
                 */
                Status.prototype.code = 0;
    
                /**
                 * Status message.
                 * @member {string} message
                 * @memberof google.rpc.Status
                 * @instance
                 */
                Status.prototype.message = "";
    
                /**
                 * Status details.
                 * @member {Array.<google.protobuf.IAny>} details
                 * @memberof google.rpc.Status
                 * @instance
                 */
                Status.prototype.details = $util.emptyArray;
    
                /**
                 * Creates a new Status instance using the specified properties.
                 * @function create
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.IStatus=} [properties] Properties to set
                 * @returns {google.rpc.Status} Status instance
                 */
                Status.create = function create(properties) {
                    return new Status(properties);
                };
    
                /**
                 * Encodes the specified Status message. Does not implicitly {@link google.rpc.Status.verify|verify} messages.
                 * @function encode
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.IStatus} m Status message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Status.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.code != null && Object.hasOwnProperty.call(m, "code"))
                        w.uint32(8).int32(m.code);
                    if (m.message != null && Object.hasOwnProperty.call(m, "message"))
                        w.uint32(18).string(m.message);
                    if (m.details != null && m.details.length) {
                        for (var i = 0; i < m.details.length; ++i)
                            $root.google.protobuf.Any.encode(m.details[i], w.uint32(26).fork()).ldelim();
                    }
                    return w;
                };
    
                /**
                 * Encodes the specified Status message, length delimited. Does not implicitly {@link google.rpc.Status.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.IStatus} message Status message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Status.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Status message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.rpc.Status
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.rpc.Status} Status
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Status.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.rpc.Status();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.code = r.int32();
                            break;
                        case 2:
                            m.message = r.string();
                            break;
                        case 3:
                            if (!(m.details && m.details.length))
                                m.details = [];
                            m.details.push($root.google.protobuf.Any.decode(r, r.uint32()));
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a Status message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.rpc.Status
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.rpc.Status} Status
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Status.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Status message.
                 * @function verify
                 * @memberof google.rpc.Status
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Status.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.code != null && m.hasOwnProperty("code")) {
                        if (!$util.isInteger(m.code))
                            return "code: integer expected";
                    }
                    if (m.message != null && m.hasOwnProperty("message")) {
                        if (!$util.isString(m.message))
                            return "message: string expected";
                    }
                    if (m.details != null && m.hasOwnProperty("details")) {
                        if (!Array.isArray(m.details))
                            return "details: array expected";
                        for (var i = 0; i < m.details.length; ++i) {
                            {
                                var e = $root.google.protobuf.Any.verify(m.details[i]);
                                if (e)
                                    return "details." + e;
                            }
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a Status message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.rpc.Status
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.rpc.Status} Status
                 */
                Status.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.rpc.Status)
                        return d;
                    var m = new $root.google.rpc.Status();
                    if (d.code != null) {
                        m.code = d.code | 0;
                    }
                    if (d.message != null) {
                        m.message = String(d.message);
                    }
                    if (d.details) {
                        if (!Array.isArray(d.details))
                            throw TypeError(".google.rpc.Status.details: array expected");
                        m.details = [];
                        for (var i = 0; i < d.details.length; ++i) {
                            if (typeof d.details[i] !== "object")
                                throw TypeError(".google.rpc.Status.details: object expected");
                            m.details[i] = $root.google.protobuf.Any.fromObject(d.details[i]);
                        }
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a Status message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.Status} m Status
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Status.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.details = [];
                    }
                    if (o.defaults) {
                        d.code = 0;
                        d.message = "";
                    }
                    if (m.code != null && m.hasOwnProperty("code")) {
                        d.code = m.code;
                    }
                    if (m.message != null && m.hasOwnProperty("message")) {
                        d.message = m.message;
                    }
                    if (m.details && m.details.length) {
                        d.details = [];
                        for (var j = 0; j < m.details.length; ++j) {
                            d.details[j] = $root.google.protobuf.Any.toObject(m.details[j], o);
                        }
                    }
                    return d;
                };
    
                /**
                 * Converts this Status to JSON.
                 * @function toJSON
                 * @memberof google.rpc.Status
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Status.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return Status;
            })();
    
            return rpc;
        })();
    
        google.longrunning = (function() {
    
            /**
             * Namespace longrunning.
             * @memberof google
             * @namespace
             */
            var longrunning = {};
    
            longrunning.Operations = (function() {
    
                /**
                 * Constructs a new Operations service.
                 * @memberof google.longrunning
                 * @classdesc Represents an Operations
                 * @extends $protobuf.rpc.Service
                 * @constructor
                 * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                 * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                 * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                 */
                function Operations(rpcImpl, requestDelimited, responseDelimited) {
                    $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                }
    
                (Operations.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = Operations;
    
                /**
                 * Creates new Operations service using the specified rpc implementation.
                 * @function create
                 * @memberof google.longrunning.Operations
                 * @static
                 * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                 * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                 * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                 * @returns {Operations} RPC service. Useful where requests and/or responses are streamed.
                 */
                Operations.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                    return new this(rpcImpl, requestDelimited, responseDelimited);
                };
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|listOperations}.
                 * @memberof google.longrunning.Operations
                 * @typedef ListOperationsCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.longrunning.ListOperationsResponse} [response] ListOperationsResponse
                 */
    
                /**
                 * Calls ListOperations.
                 * @function listOperations
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IListOperationsRequest} request ListOperationsRequest message or plain object
                 * @param {google.longrunning.Operations.ListOperationsCallback} callback Node-style callback called with the error, if any, and ListOperationsResponse
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.listOperations = function listOperations(request, callback) {
                    return this.rpcCall(listOperations, $root.google.longrunning.ListOperationsRequest, $root.google.longrunning.ListOperationsResponse, request, callback);
                }, "name", { value: "ListOperations" });
    
                /**
                 * Calls ListOperations.
                 * @function listOperations
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IListOperationsRequest} request ListOperationsRequest message or plain object
                 * @returns {Promise<google.longrunning.ListOperationsResponse>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|getOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef GetOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.longrunning.Operation} [response] Operation
                 */
    
                /**
                 * Calls GetOperation.
                 * @function getOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IGetOperationRequest} request GetOperationRequest message or plain object
                 * @param {google.longrunning.Operations.GetOperationCallback} callback Node-style callback called with the error, if any, and Operation
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.getOperation = function getOperation(request, callback) {
                    return this.rpcCall(getOperation, $root.google.longrunning.GetOperationRequest, $root.google.longrunning.Operation, request, callback);
                }, "name", { value: "GetOperation" });
    
                /**
                 * Calls GetOperation.
                 * @function getOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IGetOperationRequest} request GetOperationRequest message or plain object
                 * @returns {Promise<google.longrunning.Operation>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|deleteOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef DeleteOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.protobuf.Empty} [response] Empty
                 */
    
                /**
                 * Calls DeleteOperation.
                 * @function deleteOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IDeleteOperationRequest} request DeleteOperationRequest message or plain object
                 * @param {google.longrunning.Operations.DeleteOperationCallback} callback Node-style callback called with the error, if any, and Empty
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.deleteOperation = function deleteOperation(request, callback) {
                    return this.rpcCall(deleteOperation, $root.google.longrunning.DeleteOperationRequest, $root.google.protobuf.Empty, request, callback);
                }, "name", { value: "DeleteOperation" });
    
                /**
                 * Calls DeleteOperation.
                 * @function deleteOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IDeleteOperationRequest} request DeleteOperationRequest message or plain object
                 * @returns {Promise<google.protobuf.Empty>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|cancelOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef CancelOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.protobuf.Empty} [response] Empty
                 */
    
                /**
                 * Calls CancelOperation.
                 * @function cancelOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.ICancelOperationRequest} request CancelOperationRequest message or plain object
                 * @param {google.longrunning.Operations.CancelOperationCallback} callback Node-style callback called with the error, if any, and Empty
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.cancelOperation = function cancelOperation(request, callback) {
                    return this.rpcCall(cancelOperation, $root.google.longrunning.CancelOperationRequest, $root.google.protobuf.Empty, request, callback);
                }, "name", { value: "CancelOperation" });
    
                /**
                 * Calls CancelOperation.
                 * @function cancelOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.ICancelOperationRequest} request CancelOperationRequest message or plain object
                 * @returns {Promise<google.protobuf.Empty>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|waitOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef WaitOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.longrunning.Operation} [response] Operation
                 */
    
                /**
                 * Calls WaitOperation.
                 * @function waitOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IWaitOperationRequest} request WaitOperationRequest message or plain object
                 * @param {google.longrunning.Operations.WaitOperationCallback} callback Node-style callback called with the error, if any, and Operation
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.waitOperation = function waitOperation(request, callback) {
                    return this.rpcCall(waitOperation, $root.google.longrunning.WaitOperationRequest, $root.google.longrunning.Operation, request, callback);
                }, "name", { value: "WaitOperation" });
    
                /**
                 * Calls WaitOperation.
                 * @function waitOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IWaitOperationRequest} request WaitOperationRequest message or plain object
                 * @returns {Promise<google.longrunning.Operation>} Promise
                 * @variation 2
                 */
    
                return Operations;
            })();
    
            longrunning.Operation = (function() {
    
                /**
                 * Properties of an Operation.
                 * @memberof google.longrunning
                 * @interface IOperation
                 * @property {string|null} [name] Operation name
                 * @property {google.protobuf.IAny|null} [metadata] Operation metadata
                 * @property {boolean|null} [done] Operation done
                 * @property {google.rpc.IStatus|null} [error] Operation error
                 * @property {google.protobuf.IAny|null} [response] Operation response
                 */
    
                /**
                 * Constructs a new Operation.
                 * @memberof google.longrunning
                 * @classdesc Represents an Operation.
                 * @implements IOperation
                 * @constructor
                 * @param {google.longrunning.IOperation=} [p] Properties to set
                 */
                function Operation(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * Operation name.
                 * @member {string} name
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.name = "";
    
                /**
                 * Operation metadata.
                 * @member {google.protobuf.IAny|null|undefined} metadata
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.metadata = null;
    
                /**
                 * Operation done.
                 * @member {boolean} done
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.done = false;
    
                /**
                 * Operation error.
                 * @member {google.rpc.IStatus|null|undefined} error
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.error = null;
    
                /**
                 * Operation response.
                 * @member {google.protobuf.IAny|null|undefined} response
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.response = null;
    
                // OneOf field names bound to virtual getters and setters
                var $oneOfFields;
    
                /**
                 * Operation result.
                 * @member {"error"|"response"|undefined} result
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Object.defineProperty(Operation.prototype, "result", {
                    get: $util.oneOfGetter($oneOfFields = ["error", "response"]),
                    set: $util.oneOfSetter($oneOfFields)
                });
    
                /**
                 * Creates a new Operation instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.IOperation=} [properties] Properties to set
                 * @returns {google.longrunning.Operation} Operation instance
                 */
                Operation.create = function create(properties) {
                    return new Operation(properties);
                };
    
                /**
                 * Encodes the specified Operation message. Does not implicitly {@link google.longrunning.Operation.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.IOperation} m Operation message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Operation.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.metadata != null && Object.hasOwnProperty.call(m, "metadata"))
                        $root.google.protobuf.Any.encode(m.metadata, w.uint32(18).fork()).ldelim();
                    if (m.done != null && Object.hasOwnProperty.call(m, "done"))
                        w.uint32(24).bool(m.done);
                    if (m.error != null && Object.hasOwnProperty.call(m, "error"))
                        $root.google.rpc.Status.encode(m.error, w.uint32(34).fork()).ldelim();
                    if (m.response != null && Object.hasOwnProperty.call(m, "response"))
                        $root.google.protobuf.Any.encode(m.response, w.uint32(42).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified Operation message, length delimited. Does not implicitly {@link google.longrunning.Operation.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.IOperation} message Operation message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Operation.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Operation message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.Operation} Operation
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Operation.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.Operation();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            m.metadata = $root.google.protobuf.Any.decode(r, r.uint32());
                            break;
                        case 3:
                            m.done = r.bool();
                            break;
                        case 4:
                            m.error = $root.google.rpc.Status.decode(r, r.uint32());
                            break;
                        case 5:
                            m.response = $root.google.protobuf.Any.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an Operation message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.Operation} Operation
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Operation.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Operation message.
                 * @function verify
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Operation.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    var p = {};
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.metadata != null && m.hasOwnProperty("metadata")) {
                        {
                            var e = $root.google.protobuf.Any.verify(m.metadata);
                            if (e)
                                return "metadata." + e;
                        }
                    }
                    if (m.done != null && m.hasOwnProperty("done")) {
                        if (typeof m.done !== "boolean")
                            return "done: boolean expected";
                    }
                    if (m.error != null && m.hasOwnProperty("error")) {
                        p.result = 1;
                        {
                            var e = $root.google.rpc.Status.verify(m.error);
                            if (e)
                                return "error." + e;
                        }
                    }
                    if (m.response != null && m.hasOwnProperty("response")) {
                        if (p.result === 1)
                            return "result: multiple values";
                        p.result = 1;
                        {
                            var e = $root.google.protobuf.Any.verify(m.response);
                            if (e)
                                return "response." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an Operation message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.Operation} Operation
                 */
                Operation.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.Operation)
                        return d;
                    var m = new $root.google.longrunning.Operation();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.metadata != null) {
                        if (typeof d.metadata !== "object")
                            throw TypeError(".google.longrunning.Operation.metadata: object expected");
                        m.metadata = $root.google.protobuf.Any.fromObject(d.metadata);
                    }
                    if (d.done != null) {
                        m.done = Boolean(d.done);
                    }
                    if (d.error != null) {
                        if (typeof d.error !== "object")
                            throw TypeError(".google.longrunning.Operation.error: object expected");
                        m.error = $root.google.rpc.Status.fromObject(d.error);
                    }
                    if (d.response != null) {
                        if (typeof d.response !== "object")
                            throw TypeError(".google.longrunning.Operation.response: object expected");
                        m.response = $root.google.protobuf.Any.fromObject(d.response);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an Operation message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.Operation} m Operation
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Operation.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                        d.metadata = null;
                        d.done = false;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.metadata != null && m.hasOwnProperty("metadata")) {
                        d.metadata = $root.google.protobuf.Any.toObject(m.metadata, o);
                    }
                    if (m.done != null && m.hasOwnProperty("done")) {
                        d.done = m.done;
                    }
                    if (m.error != null && m.hasOwnProperty("error")) {
                        d.error = $root.google.rpc.Status.toObject(m.error, o);
                        if (o.oneofs)
                            d.result = "error";
                    }
                    if (m.response != null && m.hasOwnProperty("response")) {
                        d.response = $root.google.protobuf.Any.toObject(m.response, o);
                        if (o.oneofs)
                            d.result = "response";
                    }
                    return d;
                };
    
                /**
                 * Converts this Operation to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.Operation
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Operation.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return Operation;
            })();
    
            longrunning.GetOperationRequest = (function() {
    
                /**
                 * Properties of a GetOperationRequest.
                 * @memberof google.longrunning
                 * @interface IGetOperationRequest
                 * @property {string|null} [name] GetOperationRequest name
                 */
    
                /**
                 * Constructs a new GetOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a GetOperationRequest.
                 * @implements IGetOperationRequest
                 * @constructor
                 * @param {google.longrunning.IGetOperationRequest=} [p] Properties to set
                 */
                function GetOperationRequest(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * GetOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.GetOperationRequest
                 * @instance
                 */
                GetOperationRequest.prototype.name = "";
    
                /**
                 * Creates a new GetOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.IGetOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest instance
                 */
                GetOperationRequest.create = function create(properties) {
                    return new GetOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified GetOperationRequest message. Does not implicitly {@link google.longrunning.GetOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.IGetOperationRequest} m GetOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GetOperationRequest.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    return w;
                };
    
                /**
                 * Encodes the specified GetOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.GetOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.IGetOperationRequest} message GetOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GetOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a GetOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GetOperationRequest.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.GetOperationRequest();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a GetOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GetOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a GetOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                GetOperationRequest.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a GetOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest
                 */
                GetOperationRequest.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.GetOperationRequest)
                        return d;
                    var m = new $root.google.longrunning.GetOperationRequest();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a GetOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.GetOperationRequest} m GetOperationRequest
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                GetOperationRequest.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    return d;
                };
    
                /**
                 * Converts this GetOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.GetOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                GetOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return GetOperationRequest;
            })();
    
            longrunning.ListOperationsRequest = (function() {
    
                /**
                 * Properties of a ListOperationsRequest.
                 * @memberof google.longrunning
                 * @interface IListOperationsRequest
                 * @property {string|null} [name] ListOperationsRequest name
                 * @property {string|null} [filter] ListOperationsRequest filter
                 * @property {number|null} [page_size] ListOperationsRequest page_size
                 * @property {string|null} [page_token] ListOperationsRequest page_token
                 */
    
                /**
                 * Constructs a new ListOperationsRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a ListOperationsRequest.
                 * @implements IListOperationsRequest
                 * @constructor
                 * @param {google.longrunning.IListOperationsRequest=} [p] Properties to set
                 */
                function ListOperationsRequest(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * ListOperationsRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.name = "";
    
                /**
                 * ListOperationsRequest filter.
                 * @member {string} filter
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.filter = "";
    
                /**
                 * ListOperationsRequest page_size.
                 * @member {number} page_size
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.page_size = 0;
    
                /**
                 * ListOperationsRequest page_token.
                 * @member {string} page_token
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.page_token = "";
    
                /**
                 * Creates a new ListOperationsRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.IListOperationsRequest=} [properties] Properties to set
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest instance
                 */
                ListOperationsRequest.create = function create(properties) {
                    return new ListOperationsRequest(properties);
                };
    
                /**
                 * Encodes the specified ListOperationsRequest message. Does not implicitly {@link google.longrunning.ListOperationsRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.IListOperationsRequest} m ListOperationsRequest message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsRequest.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.filter != null && Object.hasOwnProperty.call(m, "filter"))
                        w.uint32(10).string(m.filter);
                    if (m.page_size != null && Object.hasOwnProperty.call(m, "page_size"))
                        w.uint32(16).int32(m.page_size);
                    if (m.page_token != null && Object.hasOwnProperty.call(m, "page_token"))
                        w.uint32(26).string(m.page_token);
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(34).string(m.name);
                    return w;
                };
    
                /**
                 * Encodes the specified ListOperationsRequest message, length delimited. Does not implicitly {@link google.longrunning.ListOperationsRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.IListOperationsRequest} message ListOperationsRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ListOperationsRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsRequest.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.ListOperationsRequest();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 4:
                            m.name = r.string();
                            break;
                        case 1:
                            m.filter = r.string();
                            break;
                        case 2:
                            m.page_size = r.int32();
                            break;
                        case 3:
                            m.page_token = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a ListOperationsRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ListOperationsRequest message.
                 * @function verify
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ListOperationsRequest.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.filter != null && m.hasOwnProperty("filter")) {
                        if (!$util.isString(m.filter))
                            return "filter: string expected";
                    }
                    if (m.page_size != null && m.hasOwnProperty("page_size")) {
                        if (!$util.isInteger(m.page_size))
                            return "page_size: integer expected";
                    }
                    if (m.page_token != null && m.hasOwnProperty("page_token")) {
                        if (!$util.isString(m.page_token))
                            return "page_token: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a ListOperationsRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest
                 */
                ListOperationsRequest.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.ListOperationsRequest)
                        return d;
                    var m = new $root.google.longrunning.ListOperationsRequest();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.filter != null) {
                        m.filter = String(d.filter);
                    }
                    if (d.page_size != null) {
                        m.page_size = d.page_size | 0;
                    }
                    if (d.page_token != null) {
                        m.page_token = String(d.page_token);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a ListOperationsRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.ListOperationsRequest} m ListOperationsRequest
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ListOperationsRequest.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.filter = "";
                        d.page_size = 0;
                        d.page_token = "";
                        d.name = "";
                    }
                    if (m.filter != null && m.hasOwnProperty("filter")) {
                        d.filter = m.filter;
                    }
                    if (m.page_size != null && m.hasOwnProperty("page_size")) {
                        d.page_size = m.page_size;
                    }
                    if (m.page_token != null && m.hasOwnProperty("page_token")) {
                        d.page_token = m.page_token;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    return d;
                };
    
                /**
                 * Converts this ListOperationsRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ListOperationsRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return ListOperationsRequest;
            })();
    
            longrunning.ListOperationsResponse = (function() {
    
                /**
                 * Properties of a ListOperationsResponse.
                 * @memberof google.longrunning
                 * @interface IListOperationsResponse
                 * @property {Array.<google.longrunning.IOperation>|null} [operations] ListOperationsResponse operations
                 * @property {string|null} [next_page_token] ListOperationsResponse next_page_token
                 */
    
                /**
                 * Constructs a new ListOperationsResponse.
                 * @memberof google.longrunning
                 * @classdesc Represents a ListOperationsResponse.
                 * @implements IListOperationsResponse
                 * @constructor
                 * @param {google.longrunning.IListOperationsResponse=} [p] Properties to set
                 */
                function ListOperationsResponse(p) {
                    this.operations = [];
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * ListOperationsResponse operations.
                 * @member {Array.<google.longrunning.IOperation>} operations
                 * @memberof google.longrunning.ListOperationsResponse
                 * @instance
                 */
                ListOperationsResponse.prototype.operations = $util.emptyArray;
    
                /**
                 * ListOperationsResponse next_page_token.
                 * @member {string} next_page_token
                 * @memberof google.longrunning.ListOperationsResponse
                 * @instance
                 */
                ListOperationsResponse.prototype.next_page_token = "";
    
                /**
                 * Creates a new ListOperationsResponse instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.IListOperationsResponse=} [properties] Properties to set
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse instance
                 */
                ListOperationsResponse.create = function create(properties) {
                    return new ListOperationsResponse(properties);
                };
    
                /**
                 * Encodes the specified ListOperationsResponse message. Does not implicitly {@link google.longrunning.ListOperationsResponse.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.IListOperationsResponse} m ListOperationsResponse message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsResponse.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.operations != null && m.operations.length) {
                        for (var i = 0; i < m.operations.length; ++i)
                            $root.google.longrunning.Operation.encode(m.operations[i], w.uint32(10).fork()).ldelim();
                    }
                    if (m.next_page_token != null && Object.hasOwnProperty.call(m, "next_page_token"))
                        w.uint32(18).string(m.next_page_token);
                    return w;
                };
    
                /**
                 * Encodes the specified ListOperationsResponse message, length delimited. Does not implicitly {@link google.longrunning.ListOperationsResponse.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.IListOperationsResponse} message ListOperationsResponse message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ListOperationsResponse message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsResponse.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.ListOperationsResponse();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            if (!(m.operations && m.operations.length))
                                m.operations = [];
                            m.operations.push($root.google.longrunning.Operation.decode(r, r.uint32()));
                            break;
                        case 2:
                            m.next_page_token = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a ListOperationsResponse message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsResponse.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ListOperationsResponse message.
                 * @function verify
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ListOperationsResponse.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.operations != null && m.hasOwnProperty("operations")) {
                        if (!Array.isArray(m.operations))
                            return "operations: array expected";
                        for (var i = 0; i < m.operations.length; ++i) {
                            {
                                var e = $root.google.longrunning.Operation.verify(m.operations[i]);
                                if (e)
                                    return "operations." + e;
                            }
                        }
                    }
                    if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                        if (!$util.isString(m.next_page_token))
                            return "next_page_token: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a ListOperationsResponse message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse
                 */
                ListOperationsResponse.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.ListOperationsResponse)
                        return d;
                    var m = new $root.google.longrunning.ListOperationsResponse();
                    if (d.operations) {
                        if (!Array.isArray(d.operations))
                            throw TypeError(".google.longrunning.ListOperationsResponse.operations: array expected");
                        m.operations = [];
                        for (var i = 0; i < d.operations.length; ++i) {
                            if (typeof d.operations[i] !== "object")
                                throw TypeError(".google.longrunning.ListOperationsResponse.operations: object expected");
                            m.operations[i] = $root.google.longrunning.Operation.fromObject(d.operations[i]);
                        }
                    }
                    if (d.next_page_token != null) {
                        m.next_page_token = String(d.next_page_token);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a ListOperationsResponse message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.ListOperationsResponse} m ListOperationsResponse
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ListOperationsResponse.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.arrays || o.defaults) {
                        d.operations = [];
                    }
                    if (o.defaults) {
                        d.next_page_token = "";
                    }
                    if (m.operations && m.operations.length) {
                        d.operations = [];
                        for (var j = 0; j < m.operations.length; ++j) {
                            d.operations[j] = $root.google.longrunning.Operation.toObject(m.operations[j], o);
                        }
                    }
                    if (m.next_page_token != null && m.hasOwnProperty("next_page_token")) {
                        d.next_page_token = m.next_page_token;
                    }
                    return d;
                };
    
                /**
                 * Converts this ListOperationsResponse to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.ListOperationsResponse
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ListOperationsResponse.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return ListOperationsResponse;
            })();
    
            longrunning.CancelOperationRequest = (function() {
    
                /**
                 * Properties of a CancelOperationRequest.
                 * @memberof google.longrunning
                 * @interface ICancelOperationRequest
                 * @property {string|null} [name] CancelOperationRequest name
                 */
    
                /**
                 * Constructs a new CancelOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a CancelOperationRequest.
                 * @implements ICancelOperationRequest
                 * @constructor
                 * @param {google.longrunning.ICancelOperationRequest=} [p] Properties to set
                 */
                function CancelOperationRequest(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * CancelOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.CancelOperationRequest
                 * @instance
                 */
                CancelOperationRequest.prototype.name = "";
    
                /**
                 * Creates a new CancelOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.ICancelOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest instance
                 */
                CancelOperationRequest.create = function create(properties) {
                    return new CancelOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified CancelOperationRequest message. Does not implicitly {@link google.longrunning.CancelOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.ICancelOperationRequest} m CancelOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CancelOperationRequest.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    return w;
                };
    
                /**
                 * Encodes the specified CancelOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.CancelOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.ICancelOperationRequest} message CancelOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CancelOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a CancelOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CancelOperationRequest.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.CancelOperationRequest();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a CancelOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CancelOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a CancelOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                CancelOperationRequest.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a CancelOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest
                 */
                CancelOperationRequest.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.CancelOperationRequest)
                        return d;
                    var m = new $root.google.longrunning.CancelOperationRequest();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a CancelOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.CancelOperationRequest} m CancelOperationRequest
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                CancelOperationRequest.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    return d;
                };
    
                /**
                 * Converts this CancelOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.CancelOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                CancelOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return CancelOperationRequest;
            })();
    
            longrunning.DeleteOperationRequest = (function() {
    
                /**
                 * Properties of a DeleteOperationRequest.
                 * @memberof google.longrunning
                 * @interface IDeleteOperationRequest
                 * @property {string|null} [name] DeleteOperationRequest name
                 */
    
                /**
                 * Constructs a new DeleteOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a DeleteOperationRequest.
                 * @implements IDeleteOperationRequest
                 * @constructor
                 * @param {google.longrunning.IDeleteOperationRequest=} [p] Properties to set
                 */
                function DeleteOperationRequest(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * DeleteOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @instance
                 */
                DeleteOperationRequest.prototype.name = "";
    
                /**
                 * Creates a new DeleteOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.IDeleteOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest instance
                 */
                DeleteOperationRequest.create = function create(properties) {
                    return new DeleteOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified DeleteOperationRequest message. Does not implicitly {@link google.longrunning.DeleteOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.IDeleteOperationRequest} m DeleteOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DeleteOperationRequest.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    return w;
                };
    
                /**
                 * Encodes the specified DeleteOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.DeleteOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.IDeleteOperationRequest} message DeleteOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DeleteOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a DeleteOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DeleteOperationRequest.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.DeleteOperationRequest();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a DeleteOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DeleteOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a DeleteOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                DeleteOperationRequest.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a DeleteOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest
                 */
                DeleteOperationRequest.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.DeleteOperationRequest)
                        return d;
                    var m = new $root.google.longrunning.DeleteOperationRequest();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a DeleteOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.DeleteOperationRequest} m DeleteOperationRequest
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                DeleteOperationRequest.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    return d;
                };
    
                /**
                 * Converts this DeleteOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                DeleteOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return DeleteOperationRequest;
            })();
    
            longrunning.WaitOperationRequest = (function() {
    
                /**
                 * Properties of a WaitOperationRequest.
                 * @memberof google.longrunning
                 * @interface IWaitOperationRequest
                 * @property {string|null} [name] WaitOperationRequest name
                 * @property {google.protobuf.IDuration|null} [timeout] WaitOperationRequest timeout
                 */
    
                /**
                 * Constructs a new WaitOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a WaitOperationRequest.
                 * @implements IWaitOperationRequest
                 * @constructor
                 * @param {google.longrunning.IWaitOperationRequest=} [p] Properties to set
                 */
                function WaitOperationRequest(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * WaitOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.WaitOperationRequest
                 * @instance
                 */
                WaitOperationRequest.prototype.name = "";
    
                /**
                 * WaitOperationRequest timeout.
                 * @member {google.protobuf.IDuration|null|undefined} timeout
                 * @memberof google.longrunning.WaitOperationRequest
                 * @instance
                 */
                WaitOperationRequest.prototype.timeout = null;
    
                /**
                 * Creates a new WaitOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.IWaitOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest instance
                 */
                WaitOperationRequest.create = function create(properties) {
                    return new WaitOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified WaitOperationRequest message. Does not implicitly {@link google.longrunning.WaitOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.IWaitOperationRequest} m WaitOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                WaitOperationRequest.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.name != null && Object.hasOwnProperty.call(m, "name"))
                        w.uint32(10).string(m.name);
                    if (m.timeout != null && Object.hasOwnProperty.call(m, "timeout"))
                        $root.google.protobuf.Duration.encode(m.timeout, w.uint32(18).fork()).ldelim();
                    return w;
                };
    
                /**
                 * Encodes the specified WaitOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.WaitOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.IWaitOperationRequest} message WaitOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                WaitOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a WaitOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                WaitOperationRequest.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.WaitOperationRequest();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.name = r.string();
                            break;
                        case 2:
                            m.timeout = $root.google.protobuf.Duration.decode(r, r.uint32());
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes a WaitOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                WaitOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a WaitOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                WaitOperationRequest.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.name != null && m.hasOwnProperty("name")) {
                        if (!$util.isString(m.name))
                            return "name: string expected";
                    }
                    if (m.timeout != null && m.hasOwnProperty("timeout")) {
                        {
                            var e = $root.google.protobuf.Duration.verify(m.timeout);
                            if (e)
                                return "timeout." + e;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a WaitOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest
                 */
                WaitOperationRequest.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.WaitOperationRequest)
                        return d;
                    var m = new $root.google.longrunning.WaitOperationRequest();
                    if (d.name != null) {
                        m.name = String(d.name);
                    }
                    if (d.timeout != null) {
                        if (typeof d.timeout !== "object")
                            throw TypeError(".google.longrunning.WaitOperationRequest.timeout: object expected");
                        m.timeout = $root.google.protobuf.Duration.fromObject(d.timeout);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from a WaitOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.WaitOperationRequest} m WaitOperationRequest
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                WaitOperationRequest.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.name = "";
                        d.timeout = null;
                    }
                    if (m.name != null && m.hasOwnProperty("name")) {
                        d.name = m.name;
                    }
                    if (m.timeout != null && m.hasOwnProperty("timeout")) {
                        d.timeout = $root.google.protobuf.Duration.toObject(m.timeout, o);
                    }
                    return d;
                };
    
                /**
                 * Converts this WaitOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.WaitOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                WaitOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return WaitOperationRequest;
            })();
    
            longrunning.OperationInfo = (function() {
    
                /**
                 * Properties of an OperationInfo.
                 * @memberof google.longrunning
                 * @interface IOperationInfo
                 * @property {string|null} [response_type] OperationInfo response_type
                 * @property {string|null} [metadata_type] OperationInfo metadata_type
                 */
    
                /**
                 * Constructs a new OperationInfo.
                 * @memberof google.longrunning
                 * @classdesc Represents an OperationInfo.
                 * @implements IOperationInfo
                 * @constructor
                 * @param {google.longrunning.IOperationInfo=} [p] Properties to set
                 */
                function OperationInfo(p) {
                    if (p)
                        for (var ks = Object.keys(p), i = 0; i < ks.length; ++i)
                            if (p[ks[i]] != null)
                                this[ks[i]] = p[ks[i]];
                }
    
                /**
                 * OperationInfo response_type.
                 * @member {string} response_type
                 * @memberof google.longrunning.OperationInfo
                 * @instance
                 */
                OperationInfo.prototype.response_type = "";
    
                /**
                 * OperationInfo metadata_type.
                 * @member {string} metadata_type
                 * @memberof google.longrunning.OperationInfo
                 * @instance
                 */
                OperationInfo.prototype.metadata_type = "";
    
                /**
                 * Creates a new OperationInfo instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.IOperationInfo=} [properties] Properties to set
                 * @returns {google.longrunning.OperationInfo} OperationInfo instance
                 */
                OperationInfo.create = function create(properties) {
                    return new OperationInfo(properties);
                };
    
                /**
                 * Encodes the specified OperationInfo message. Does not implicitly {@link google.longrunning.OperationInfo.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.IOperationInfo} m OperationInfo message or plain object to encode
                 * @param {$protobuf.Writer} [w] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OperationInfo.encode = function encode(m, w) {
                    if (!w)
                        w = $Writer.create();
                    if (m.response_type != null && Object.hasOwnProperty.call(m, "response_type"))
                        w.uint32(10).string(m.response_type);
                    if (m.metadata_type != null && Object.hasOwnProperty.call(m, "metadata_type"))
                        w.uint32(18).string(m.metadata_type);
                    return w;
                };
    
                /**
                 * Encodes the specified OperationInfo message, length delimited. Does not implicitly {@link google.longrunning.OperationInfo.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.IOperationInfo} message OperationInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OperationInfo.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an OperationInfo message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} r Reader or buffer to decode from
                 * @param {number} [l] Message length if known beforehand
                 * @returns {google.longrunning.OperationInfo} OperationInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OperationInfo.decode = function decode(r, l) {
                    if (!(r instanceof $Reader))
                        r = $Reader.create(r);
                    var c = l === undefined ? r.len : r.pos + l, m = new $root.google.longrunning.OperationInfo();
                    while (r.pos < c) {
                        var t = r.uint32();
                        switch (t >>> 3) {
                        case 1:
                            m.response_type = r.string();
                            break;
                        case 2:
                            m.metadata_type = r.string();
                            break;
                        default:
                            r.skipType(t & 7);
                            break;
                        }
                    }
                    return m;
                };
    
                /**
                 * Decodes an OperationInfo message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.OperationInfo} OperationInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OperationInfo.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an OperationInfo message.
                 * @function verify
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {Object.<string,*>} m Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                OperationInfo.verify = function verify(m) {
                    if (typeof m !== "object" || m === null)
                        return "object expected";
                    if (m.response_type != null && m.hasOwnProperty("response_type")) {
                        if (!$util.isString(m.response_type))
                            return "response_type: string expected";
                    }
                    if (m.metadata_type != null && m.hasOwnProperty("metadata_type")) {
                        if (!$util.isString(m.metadata_type))
                            return "metadata_type: string expected";
                    }
                    return null;
                };
    
                /**
                 * Creates an OperationInfo message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {Object.<string,*>} d Plain object
                 * @returns {google.longrunning.OperationInfo} OperationInfo
                 */
                OperationInfo.fromObject = function fromObject(d) {
                    if (d instanceof $root.google.longrunning.OperationInfo)
                        return d;
                    var m = new $root.google.longrunning.OperationInfo();
                    if (d.response_type != null) {
                        m.response_type = String(d.response_type);
                    }
                    if (d.metadata_type != null) {
                        m.metadata_type = String(d.metadata_type);
                    }
                    return m;
                };
    
                /**
                 * Creates a plain object from an OperationInfo message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.OperationInfo} m OperationInfo
                 * @param {$protobuf.IConversionOptions} [o] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                OperationInfo.toObject = function toObject(m, o) {
                    if (!o)
                        o = {};
                    var d = {};
                    if (o.defaults) {
                        d.response_type = "";
                        d.metadata_type = "";
                    }
                    if (m.response_type != null && m.hasOwnProperty("response_type")) {
                        d.response_type = m.response_type;
                    }
                    if (m.metadata_type != null && m.hasOwnProperty("metadata_type")) {
                        d.metadata_type = m.metadata_type;
                    }
                    return d;
                };
    
                /**
                 * Converts this OperationInfo to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.OperationInfo
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                OperationInfo.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                return OperationInfo;
            })();
    
            return longrunning;
        })();
    
        return google;
    })();

    return $root;
});
